!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 09 Jan 2019
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Program MPI_Scheduler
  Use Aerosols, Only: AerExC_DB, AerExt_DBG
  Use OutInf,   Only: ErrLog, OutMode
  use mpi
  use St_Input, Only: CI_Pars, S_N_D, SND_WL, SND_NAME, HOME_DIR
  use St_Vars,  only: MPI_FLG, CI_FLG
  Use Types,    only: DYN_CHAR_L, DYN_CHAR

  integer ierr, my_id, NumProcs, root_process, error, NProc
  integer n_list, core_id, info, nn, status(MPI_STATUS_SIZE) 
  integer, allocatable :: request(:)
  Logical FlgRoot, Flg
  character (len=1) NextProc
  Double Precision  tts, tt2
  Integer, Allocatable :: ProcIdx(:)
  Logical, allocatable :: CoreIdle(:), ProcDone(:)
  Character (LEN=:), allocatable :: Str, ProcName
  Type (DYN_CHAR_L), allocatable :: list(:)
  
!==================================================================
  Interface
     Subroutine Command_Line_Arguments (list)
       Use Types,   only: DYN_CHAR_L
       Type (DYN_CHAR_L), allocatable :: list(:)
     End Subroutine Command_Line_Arguments
  End Interface

  MPI_FLG = .TRUE.
  root_process = 0
  call MPI_INIT (ierr)
  call MPI_COMM_SIZE (MPI_COMM_WORLD, NN, ierr) !number of processes
  NumProcs = NN  - 1
  call MPI_COMM_RANK (MPI_COMM_WORLD, my_id, ierr) !current process id

! Global initialization
  if (my_id .eq. root_process) then
     Allocate(request(NumProcs))
     FlgRoot = .TRUE.
     Call Command_Line_Arguments (list)
     n_list=size(list)
     Allocate(ProcDone(n_list),ProcIdx(NumProcs), CoreIdle(NumProcs))
     ProcDone(:) = .False.
     CoreIdle(:) = .False.
     Call Global_Settings 
     If(Associated(ErrLog)) Nullify(ErrLog%next_msg)
  Else
     FlgRoot = .False.
  End If

  ! Broadcasting common data sets
  tts = Time_in_Seconds()
  If(FlgRoot) write(6,*) 'Start broadcasting:' 
  Call Bcast_DynChar (HOME_DIR, FlgRoot)
  Call Bcast_DynChar (S_N_D, FlgRoot)
  Call Bcast_DynChar (SND_NAME, FlgRoot)
  call MPI_BARRIER(MPI_COMM_WORLD, ierr)
  call MPI_BCAST(DBLE(SND_WL), 1, MPI_DOUBLE_PRECISION, 0, MPI_COMM_WORLD, ierr)
  Call Bcast_DynChar (OutMode, FlgRoot)
  Call Bcast_DynChar_Arr (CI_Pars, FlgRoot)

  tt2 = Time_in_Seconds()
  If(FlgRoot) write(6,*) 'End of broadcasting: ', tt2 - tts

  if (FlgRoot) then !first process
     ! Each worker core gets an initial job
     write(6,*) 'Sending names: '
     NProc = NumProcs
     do core_id = 1, NumProcs
        ProcIdx(core_id) = core_id
        If(core_id > n_list) Then
           Str = 'No Input'
        Else
           Str = list(core_id)%s
        End If
        Call Send_DynChar (Str, core_id)
     end do
     tt2 = Time_in_Seconds()
     If(FlgRoot) write(6,*) 'End of sending names: ', tt2 - tts

  ! Checking for completed processes and sending new jobs
     do core_id = 1, NumProcs
        call MPI_IRECV( info, 1, MPI_INT, core_id, &
                9999, MPI_COMM_WORLD, request(core_id), ierr)
     end do

     Flg = .false.
    ! Sending jobs until the input job list is completed
     do
        call sleep(1)
        ! Check all worker processes and send additional jobs if necessary
        do core_id = 1, NumProcs
           If(CoreIdle(core_id)) Cycle ! Core is idle, no need to test
           !to see it the job is completed
           call MPI_TEST(request(core_id), Flg, status, ierr)
           
           If(.not.Flg) cycle ! If the process is not yet completed no further action is needed
           tt2 = Time_in_Seconds()
           write(6,'(A, 3(1X,I0))')'Processing is completed (core ID, list entry, time in sec):', core_id, &
                ProcIdx(core_id), int(tt2 - tts)
!           write(6,*)"Processing for "//list(ProcIdx(core_id))%s//' is completed: ', core_id, ProcIdx(core_id), tt2 - tts
           ProcDone(ProcIdx(core_id)) = .True.
           NProc = NProc + 1
           If(NProc > n_list) Then
              NextProc = 'N'
              ! Sending the information that the next process will not be submitted
              call MPI_SEND(NextProc, 1, MPI_CHAR, core_id, 9998, MPI_COMM_WORLD, ierr)
              ! Change to "tt2=time(), tt2 must be integer then"?
              tt2 = Time_in_Seconds()
              write(6,'(A,2(1X,I0))')"No more jobs for core:", core_id, Int(tt2 - tts)
              CoreIdle(core_id) = .True.
              cycle
           Else
              NextProc = 'Y'
              ! Sending the information that the next process will be submitted
              call MPI_SEND(NextProc, 1, MPI_CHAR, core_id, 9998, MPI_COMM_WORLD, ierr)
           End If
           ProcIdx(core_id) = NProc
!           tt2 = Time_in_Seconds()
!           write(6,*)"Process for "//list(ProcIdx(core_id))%s//' is sent to core_id: ', core_id, ProcIdx(core_id), tt2 - tts
           ! Sending the process name to work on 
           Call Send_DynChar (list(ProcIdx(core_id))%s, core_id)
           Flg = .False.
           ! Sending request to return the exit status
           call MPI_IRECV( info, 1, MPI_INT, core_id, &
                9999, MPI_COMM_WORLD, request(core_id), ierr)
        End Do

        If(count(ProcDone) == n_list) Exit
     end do

  Else
     ! Recieving and processing initial jobs
     Call Recv_DynChar (ProcName)
        write(6,*) "Recieved:"//ProcName
     call MainProc (ProcName, info)
     call MPI_SEND( info, 1, MPI_INT, root_process, 9999, &
          MPI_COMM_WORLD, ierr)

    ! Recieving jobs until the input job list is completed
     Do 
        ! Recieving the information if the next process will be submitted
        call MPI_RECV(NextProc, 1, MPI_CHAR, root_process, 9998, MPI_COMM_WORLD, status, ierr)
        ! Exit if there are no more jobs to process 
        If(NextProc == 'N') Exit
        ! Recieving the process name to work on 
        Call Recv_DynChar (ProcName)
        write(6,*) "Recieved:"//ProcName
        ! Calling the processing routine
        call MainProc (ProcName, info)
        ! Returning the exit status
        call MPI_SEND( info, 1, MPI_INT, root_process, 9999, &
             MPI_COMM_WORLD, ierr) 
     End Do
  End If

  call MPI_FINALIZE(ierr)

  stop
Contains
!***************************************************************************
Subroutine Send_DynChar_Arr (Arr, core_id)
  Integer core_id
  Type (DYN_CHAR), Allocatable :: Arr(:)
  ! Local Parameters
  Integer Arr_S, II, err
  !==================================================================
  Arr_S = Size(Arr) 
  call MPI_SEND(Arr_S, 1, MPI_INT, core_id, 9998, MPI_COMM_WORLD, ierr)
  Do II = 1, Arr_S
     Call Send_DynChar (Arr(II)%S, core_id)
  End Do
  Return
End Subroutine Send_DynChar_Arr
!***************************************************************************
Subroutine Bcast_DynChar_Arr (Arr, Root)
  Logical Root
  Type (DYN_CHAR), Allocatable :: Arr(:)
  ! Local Parameters
  Integer NN, LS, II, ierr
  !==================================================================
  If(Root) NN = Size(Arr) 
  call MPI_BARRIER(MPI_COMM_WORLD, ierr)
  call MPI_BCAST(NN, 1, MPI_INT, 0, MPI_COMM_WORLD, ierr)
  If(.Not.Root) Allocate(Arr(NN)) 
  Do II = 1, NN
     Call Bcast_DynChar (Arr(II)%S, Root)
  End Do
  Return
End Subroutine Bcast_DynChar_Arr
!***************************************************************************
Subroutine Bcast_DynChar (Str, Root)
  Logical Root
  Character (Len=:), Allocatable :: Str
  ! Local Parameters
  Integer LS, ierr
  !==================================================================
  If(Root) LS = Len(Str)
  call MPI_BARRIER(MPI_COMM_WORLD, ierr)
  call MPI_BCAST(LS, 1, MPI_INT, 0, MPI_COMM_WORLD, ierr)
  If(.Not.Root) Allocate(Character(Len=LS) :: Str)
  call MPI_BARRIER(MPI_COMM_WORLD, ierr)
  call MPI_BCAST(Str, LS, MPI_CHAR, 0, MPI_COMM_WORLD, ierr)
  Return
End Subroutine Bcast_DynChar
!***************************************************************************
Subroutine Send_DynChar (Str, core_id)
  Integer core_id
  Character (Len=:), Allocatable :: Str
  ! Local Parameters
  Integer St_L, KK, ierr
  !==================================================================
  St_L = Len(Str) 
  call MPI_SEND(St_L, 1, MPI_INT, core_id, 9998, MPI_COMM_WORLD, ierr)
  call MPI_SEND(Str, St_L, MPI_CHAR, core_id, 9998, MPI_COMM_WORLD, ierr)
  Return
End Subroutine Send_DynChar
!***************************************************************************
Subroutine Send_AerExt_DBG (AerExC_DB, core_id)
  Integer core_id
  Type (AerExt_DBG) :: AerExC_DB
  ! Local Parameters
  Integer Arr_S, II, err, Nlats, Nlons
  Double Precision, Allocatable :: Lats(:), Lons(:) 
  !==================================================================
  call MPI_SEND(AerExC_DB%InitF, 1, MPI_LOGICAL, core_id, 9998, MPI_COMM_WORLD, ierr)
  call MPI_SEND(AerExC_DB%MS_2000, 1, MPI_INT, core_id, 9998, MPI_COMM_WORLD, ierr)
  NLats = Size(AerExC_DB%Lats) ; NLons = Size(AerExC_DB%Lons)
  call MPI_SEND(NLats, 1, MPI_INT, core_id, 9998, MPI_COMM_WORLD, ierr)
  call MPI_SEND(NLons, 1, MPI_INT, core_id, 9998, MPI_COMM_WORLD, ierr)
  Allocate(Lats, Source=(DBLE(AerExC_DB%Lats)))
  Allocate(Lons, Source=(DBLE(AerExC_DB%Lons)))
  call MPI_SEND(Lats, NLats, MPI_DOUBLE_PRECISION, core_id, 9998, MPI_COMM_WORLD, ierr)
  call MPI_SEND(Lons, NLons, MPI_DOUBLE_PRECISION, core_id, 9998, MPI_COMM_WORLD, ierr)
  Call Send_DynChar_Arr (AerExC_DB%Data, core_id)
  Return
End Subroutine Send_AerExt_DBG
!***************************************************************************
Subroutine RECV_DynChar_Arr (Arr)
  Type (DYN_CHAR), Allocatable :: Arr(:)
  ! Local Parameters
  Character (Len=1) Char
  Integer Arr_S, St_L, II, KK, ierr
  !==================================================================
  call MPI_RECV (Arr_S, 1 , MPI_INT, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)
  Allocate(Arr(Arr_S))
  Do II = 1, Arr_S
     Call RECV_DynChar (Arr(ii)%S)
  End Do
  Return
End Subroutine RECV_DynChar_Arr
!***************************************************************************
Subroutine RECV_DynChar (Str)
  Character (Len=:), Allocatable :: Str
  ! Local Parameters
  Integer Arr_S, St_L, ierr
  !==================================================================
  call MPI_RECV (St_L, 1 , MPI_INT, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)
  If(Allocated(Str)) DeAllocate(Str)
  Allocate(Character(Len=St_L) :: Str)
  call MPI_RECV (Str, St_L, MPI_CHAR, root_process,  &
       MPI_ANY_TAG, MPI_COMM_WORLD, status, ierr)
  Return
End Subroutine RECV_DynChar
!***************************************************************************
Subroutine RECV_AerExt_DBG (AerExC_DB)
  Type (AerExt_DBG) :: AerExC_DB
  ! Local Parameters
  Integer Arr_S, II, err, NLats, NLons
  Double Precision, Allocatable :: Lats(:), Lons(:) 
  !==================================================================
  call MPI_RECV (AerExC_DB%InitF, 1 , MPI_LOGICAL, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  call MPI_RECV (AerExC_DB%MS_2000, 1 , MPI_INT, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  call MPI_RECV (NLats, 1 , MPI_INT, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  call MPI_RECV (NLons, 1 , MPI_INT, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  Allocate(Lats(NLats)) ; Allocate(Lons(Nlons))
  call MPI_RECV (Lats, NLats, MPI_DOUBLE_PRECISION, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  call MPI_RECV (Lons, NLons, MPI_DOUBLE_PRECISION, root_process, MPI_ANY_TAG, &
       MPI_COMM_WORLD, status, ierr)    
  Allocate(AerExC_DB%Lats(NLats), AerExC_DB%Lons(NLons))
  AerExC_DB%Lats = Lats ;  AerExC_DB%Lons = Lons
  Call RECV_DynChar_Arr (AerExC_DB%Data)
  Return
End Subroutine RECV_AerExt_DBG
!***************************************************************************
Function Time_in_Seconds
  Double Precision  Time_in_Seconds
  ! Local Parameters
  character (len=10) time
  Double Precision  Hrs, Mins, Secs, tt
  !==================================================================
  call DATE_AND_TIME(time=TIME)
  read(TIME(1:2),*) hrs
  read(TIME(3:4),*) mins
  read(TIME(5:),*) secs
  Time_in_Seconds = Hrs*3600.+ Mins*60. + Secs
  Return
End Function Time_in_Seconds
!***************************************************************************
Subroutine MainProc (Name, info)
  Character (Len=*) Name
  Integer Info
  ! Local Parameters
  Character (Len=:), Allocatable:: AddChar 
  Integer NU, IOS
  Double Precision tt2

  !==================================================================
  
  if(Name == 'No Input') return
  call gt_iface (Name, info)
 
!  call sleep((my_id + 5)*2)
     
  tt2 = Time_in_Seconds()
  If(Len_Trim(Name) > 0) Then
     AddChar = "/"
  Else
     AddChar = ""
  End If
  if(info == 0) then
     open(NewUnit=NU, iostat=IOS, file = Name//AddChar//'Status_proc_done')
     write(NU,'(A,I0,1x,ES10.2)') 'Normal termination (process id, time in sec): ', my_id, tt2 - tts
     close(NU)
     write(6,*) "Status  = "//Name//AddChar//'Status_proc_done' 
  else
     open(NewUnit=NU, file = Name//AddChar//'Status_proc_err')
     write(NU,'(A,I0,1x,ES10.2)') 'Abnormal termination (process id, time in sec): ', my_id, tt2 - tts
     close(NU)
     write(6,*) "Status  = "//Name//AddChar//'Status_proc_err' 
  end if
  
  open(NewUnit=NU, iostat=IOS, file = Name//AddChar//'Status_proc_run', status='old')
  if (IOS == 0) close(NU, status='delete')
  Return
End Subroutine MainProc
!***************************************************************************
end Program MPI_Scheduler
!***************************************************************************
