!****************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 21 Dec 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
SUBROUTINE ABS_COEFF_READ (NSPEC, WL, FIRST_RUN, *)
  ! Called by "rtm_init.f" and "ring_interface.f"

  USE ABSORBERS,   ONLY: GAS_INDEX, ST_GAS_IDX, N_XSEC_MOL
  USE BRDF,        ONLY: WL0, Alf0
  USE CIA_VARS,    ONLY: CIA_INFO
  USE GRIDS,       ONLY: NLAYERS
  USE INFO,        ONLY: TG_INFO
  USE IRRAD,       ONLY: IRRAD_FLAG, FILENAME_SOLSPEC, SOL_FLUX, WL_IRRAD_DIM, &
                         SCALE_SS
  USE LOG_PARS,    ONLY: LOG_LEVEL, LUNIT, EU, C_UNIT, N_SSO
  USE PROFILES,    ONLY: TEMP, PRESS
  USE ST_VARS,     ONLY: PIE, RRS_TEST
  USE ST_INPUT,    ONLY: PWAC_NAME, PSI_NAME, CHL_NAME, SOL_CDF !DAY_OF_YEAR
  USE THERMAL_VAR, ONLY: SET_SOL_USR, SOL_FLUX_USR
  USE TOOLS,       ONLY: CHK_STAT, FIND_IDX, READ_FILE  
  USE TYPE_VARS,   ONLY: KND
  USE WATER,       ONLY: UWT, DO_ABSUW_AKIMA, N_ABS_UW, ABS_COEFF_UW, PW_ABS,&
                         PSIT, PSIS, TS_CORR
  USE WATER_ICE,   ONLY: IWS, LoIW, Na_ice, LA_N_IW
  USE X_SECTS,     ONLY: PATH => X_SECTION_PATH, N_TEMP, ABS_COEFF_NTEMP, &
                         DO_O3TEMP_GOMEFM, BP_FLAG, DO_XSECTION_AKIMA, &
                         ABS_COEFF_O3, ABS_COEFF, O3_BP_COEFF12, O3_BP_COEFF13,&
                         O3_UV_START, X_SECTION_FILE, O3_UV_END, O3_UV_CS, &
                         SR_H_FLAG, ABS_COEFF_O2, O3_UV_CS_FILE
  !-------------------------------------------------------------------
  ! Calculates trace gas absorption cross sections
  ! and calculates products of Bass&Paur coefficients
  ! for the temperature weighting functions
  INTEGER,    INTENT(IN) :: NSPEC
  LOGICAL,    INTENT(IN) :: FIRST_RUN
  REAL (KND), INTENT(IN) :: WL(NSPEC)
  ! Local variables
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ABS_COEFF_READ'
  CHARACTER (LEN=:), Allocatable :: FILE_NAME, WLD_INP
  INTEGER   I, J, K, M, MAX_PTS, II, KK, IOERROR, N_O3_UV, IDX, IiW, N_READ, &
       NN1, N_TEMP_RANGE, N_TEMP_O2, N_O2_UV, N_GAS_XS
  LOGICAL  WL_IN_NM, SR_FLAG(NSPEC), H_FLAG(NSPEC)
  REAL (KND) WAVE(8000), O3COEFF(3,8000), XXX(NSPEC), Nu, Nu0, DWL
  REAL (KND), ALLOCATABLE :: WL_O3_UV(:), G1(:,:), WL_READ(:), DATA_READ(:,:), &
       WL_O2_UV(:), O2_COEFF(:,:,:)
  REAL (KND), ALLOCATABLE :: WL_R(:), DATA_R(:,:), DD(:)
  REAL (KND), PARAMETER :: DAYS_IN_YEAR = 365.0D0
  REAL (KND)  WL_S, WL_E, DELTA, DD1         !, SCALE, DAY_ANGLE
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  MAX_PTS = 1000
  WL_IN_NM = .TRUE.
  WLD_INP = 'nm'
  N_GAS_XS = COUNT(TG_INFO(1:N_XSEC_MOL)%FM_FLG)
  WL_S = MINVAL(WL,1) ;  WL_E   = MAXVAL(WL,1) 
  
  ALLOCATE(ABS_COEFF(N_GAS_XS,NSPEC,N_TEMP), BP_FLAG(NSPEC), WL_READ(MAX_PTS),&!A
       SR_H_FLAG(NSPEC), O3_UV_CS(NSPEC), DATA_READ(MAX_PTS,1)) ! A
  
  O3_UV_CS = .FALSE.
  IF (CHK_STAT('o3', TG_INFO%NAME, TG_INFO%FM_FLG)) THEN
     BP_FLAG = .FALSE.
     WHERE(WL >= O3_UV_START .AND. WL <= O3_UV_END) BP_FLAG = .TRUE.
     N_O3_UV = COUNT(BP_FLAG)
     WHERE(WL < O3_UV_START) O3_UV_CS = .TRUE.
  ELSE
     N_O3_UV = 0
  END IF
  
  IF (CHK_STAT('o2', TG_INFO%NAME, TG_INFO%FM_FLG)) THEN
     SR_FLAG = .FALSE. ;  H_FLAG = .FALSE.
     ! Check for Schumann-Runge wavelengths
     WHERE(WL >= 1.0D+7/5.7D+4 .AND. WL <= 1.0D+7/4.9D+4) SR_FLAG = .TRUE.
     ! Check for Herzberg wavelengths.
     WHERE(WL >= 190.0D0 .AND. WL <= 240.0D0 ) H_FLAG = .TRUE.
     SR_H_FLAG = SR_FLAG.OR.H_FLAG
     N_O2_UV = COUNT(SR_H_FLAG)
  ELSE
     N_O2_UV = 0
  END IF
  
  ABS_COEFF = 0.0D0 

  ! Irradiance
  ! **********
  IF( IRRAD_FLAG ) THEN
     ! DAY_ANGLE = PIE*2.0D0*(DAY_OF_YEAR - 1)/DAYS_IN_YEAR 
     ! SCALE = (1.000110D0 + 0.034221D0*DCOS(DAY_ANGLE) + &
     !     0.001280D0*DSIN(DAY_ANGLE) + 0.000719D0*DCOS(2.D0* &
     !     DAY_ANGLE) + 0.000077D0*DSIN(2.D0*DAY_ANGLE))
     ALLOCATE(SOL_FLUX(NSPEC)) ! A 

     ! Wavelength dimension in the file of extra-terrestrial spectra
     WLD_INP = WL_IRRAD_DIM(1)
     IF( TRIM(WL_IRRAD_DIM(5)) == 'air' ) THEN
        DD1 = 1.D0/WL_S/WL_S
        ! Wavelength of the solar spectrum in vacuum
        WL_S = WL_S*(6.4328d-05 + 2.94981d-08/(1.46d-04 - DD1) + &
             2.554d-10/(4.1d-05 - DD1) + 1.d0)
        DD1 = 1.D0/WL_E/WL_E
        WL_E = WL_E*(6.4328d-05 + 2.94981d-08/(1.46d-04 - DD1) + &
             2.554d-10/(4.1d-05 - DD1) + 1.d0)
     ENDIF
     ! WL_S and WL_E are given in [nm] independent of 
     ! 'Spectral segment info' input value in "control.inp"
     ! Start and end wavelengths should be given in the same dimension
     ! as input wavelength of solar spectrum
     SELECT CASE( TRIM(WL_IRRAD_DIM(1)) )
     CASE('nm')
        DWL = 0.3d0
     CASE('micron')
        WL_S = WL_S/1.d+3 ; WL_E = WL_E/1.d+3 ; DWL = 0.3d0/1.D+3
     CASE('angstrom')
        WL_S = WL_S*1.d+1 ; WL_E = WL_E*1.d+1 ; DWL = 0.3d0*1.d+1
     END SELECT
     CALL READ_FILE (1, TRIM(FILENAME_SOLSPEC), DATA_R, *880, &
          XS=WL_S, XE=WL_E, XAX=WL_R, NPT=3, CDF=SOL_CDF)
     !CALL READ_FILE (1, TRIM(FILENAME_SOLSPEC), DATA_R, *880, &
     !     XS=WL_S, XE=WL_E, XAX=WL_R, NPT=1, CDF=SOL_CDF)
     N_READ = SIZE(WL_R)
     IF(N_READ == 0) GOTO 201
     
     ! Convert wavelength of solar spectrum in [nm]
     SELECT CASE( TRIM(WL_IRRAD_DIM(1)) )
     CASE('micron')    ;  WL_R(:) = WL_R(:)*1.d+3 
     CASE('angstrom')  ;  WL_R(:) = WL_R(:)/1.d+1
     END SELECT
     
     IF( TRIM(WL_IRRAD_DIM(5)) == 'air' ) THEN
        ALLOCATE( DD(N_READ) )
        DD(:) = 1.D0/WL_R(:)/WL_R(:)
        DD(:) = 6.4328d-05 + 2.94981d-08/(1.46d-04 - DD(:)) + &
             2.554d-10/(4.1d-05 - DD(:)) + 1.d0
        ! Wavelength of the solar spectrum in vacuum
        WL_R(:) = WL_R(:)/DD(:)
     ENDIF
     
     IF(MINVAL(WL_R) > WL(1).OR.MAXVAL(WL_R) < WL(NSPEC)) GOTO 201

     IF( N_READ<3 ) THEN
        CALL AL_INTERP (N_READ, WL_R, DATA_R(:,1), NSPEC, WL(1:NSPEC), &
             SOL_FLUX(1:NSPEC), .FALSE., *880)
     ELSE
        CALL AL_INTERP (N_READ, WL_R, DATA_R(:,1), NSPEC, WL(1:NSPEC), &
             SOL_FLUX(1:NSPEC), .TRUE., *880)
     ENDIF
     ! WRITE(*,*) '    After AL_INTERP sol_flux '
     ! WRITE(*,*) '     WL(1) = ',WL(1:NSPEC), SOL_FLUX(1:NSPEC)
     ! WRITE(*,*)  '   SCALE_SS  ', SCALE_SS
     ! PAUSE
     SOL_FLUX(:) = SCALE_SS*SOL_FLUX(:)

     ! WRITE(*,*) '   SOL_FLUX in "abs_coef_read"  '
     ! DO I = 1, NSPEC
     !    WRITE(*,'(2x,F8.3,2x,E12.4)') WL(I), SOL_FLUX(I)
     ! END DO

     ! WRITE(*,*) '    NSPEC      = ', NSPEC
     ! WRITE(*,*) '     WL(1)     = ', WL(1), SOL_FLUX(1)
     ! WRITE(*,*) '     WL(NSPEC) = ', WL(NSPEC), SOL_FLUX(NSPEC)
     ! PAUSE
     ! SOL_FLUX(1:NSPEC) = SOL_FLUX(1)
     ! SOL_FLUX(1:NSPEC) = SOL_FLUX(1:NSPEC)*SCALE

     ! Test for comparison with DISORT
     IF( RRS_TEST ) THEN
        Nu0 = 25414.d0      ! WL0 = 393.48 nm
        SOL_FLUX(1:NSPEC) = 1.d0
        DO I = 1,NSPEC
           Nu = 1.d07/WL(I)
           DELTA = DABS( Nu - Nu0 )               
           IF( DELTA < 70.d0 ) SOL_FLUX(I) = 1.d0 - 0.82d0*(1.d0-DELTA/70.d0)
           ! Example of a much deeper solar line
           ! SOL_FLUX(I) = 1.d0 - 0.98d0*(1.d0-DELTA/70.d0)
        ENDDO
     ENDIF
     ! WRITE(*,*) '   Number of SOL_FLUX :', NSPEC
     IF( SET_SOL_USR ) SOL_FLUX(1:NSPEC) = SOL_FLUX_USR
     
  END IF  !  IRRAD_FLAG

  ! IF(ALLOCATED(SOL_FLUX)) THEN
  !    WRITE(*,*) '  SOL_FLUX - allocated  '
  ! ELSE
  !    WRITE(*,*) '  SOL_FLUX - not allocated  '
  ! ENDIF
  ! WRITE(*,*) '  After   IRRAD_FLAG', SOL_FLUX(:)
  ! PAUSE

  ! Default wavelength dimension in the files containing absorption coeff.
  WLD_INP = 'nm'

  ! *************************************************************
  ! ... Open ozone file and read in absorption cross-sections
  !  write(*,*) '   before N_GAS_XS'
  
  DO K = 1, N_GAS_XS
     M = ST_GAS_IDX(K)
     DO II = 1, ABS_COEFF_NTEMP(M)
        FILE_NAME = TRIM(PATH)//TRIM(X_SECTION_FILE(M,II))
        CALL READ_CS(FILE_NAME, 1, *880)  

        IF(N_READ < 1) CYCLE
        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
             NSPEC, WL(1:NSPEC), ABS_COEFF(K,1:NSPEC,II), DO_XSECTION_AKIMA,&
             *880)
     END DO
  END DO

  ! Use  Molina & Molina cross sections if ozone temperature 
  ! parametterization not available 
  ! WL < 235 nm for GOME FM and WL < 245 nm for BP
  IF(ANY(O3_UV_CS(1:NSPEC))) THEN 
     IDX = FIND_IDX('o3', TG_INFO%NAME)
     TG_INFO(IDX)%TYPE = 'uv_o3'
     
     K = GAS_INDEX(IDX)
     
     DO II = 1, ABS_COEFF_NTEMP(N_XSEC_MOL + 1)
        
        FILE_NAME = TRIM(PATH)//TRIM(X_SECTION_FILE(N_XSEC_MOL + 1,II))
        CALL READ_CS(FILE_NAME, 1, *880)  
        
        IF(N_READ < 1) CYCLE
        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
             NSPEC, WL(1:NSPEC), XXX(1:NSPEC), DO_XSECTION_AKIMA, *880)
        
        WHERE(O3_UV_CS(1:NSPEC)) ABS_COEFF(K,1:NSPEC,II) = XXX(1:NSPEC)
     END DO
  END IF


  IF(N_O3_UV /= 0) THEN
     ! ******************************************************************
     ! Open file with polynomial coefficients for temperature
     ! dependent ozone absorption cross sections and read in.
     ! Interpolate polynomial coefficients onto appropriate
     ! wavelength grid between about 245 and 365 nm. In this
     ! spectral range the first polynomial coefficient replaces
     ! the previously calculated array. 

     ALLOCATE(ABS_COEFF_O3(N_O3_UV,NLAYERS), O3_BP_COEFF12(N_O3_UV), & ! A
          O3_BP_COEFF13(N_O3_UV), WL_O3_UV(N_O3_UV), G1(3,N_O3_UV)) ! A
         
     II = FIND_IDX('o3', TG_INFO%NAME)
     TG_INFO(II)%TYPE = 'uv_o3'
     
     FILE_NAME = TRIM(PATH)//TRIM(O3_UV_CS_FILE)
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//FILE_NAME
     OPEN (C_UNIT, FILE = FILE_NAME, STATUS = 'OLD')

     IF (.NOT.DO_O3TEMP_GOMEFM) THEN
        READ(C_UNIT,*) 
        DO I = 1,538
           READ(C_UNIT,'(5(F10.5,1X))') WAVE(5*I - 4),WAVE(5*I - 3), &
                WAVE(5*I - 2), WAVE(5*I - 1), WAVE(5*I)
        END DO
        DO K = 1, 3
           READ(C_UNIT,*) 
           DO I = 1,538
              READ(C_UNIT,'(5(d12.5,1X))') O3COEFF(K,5*I - 4), &
                   O3COEFF(K,5*I - 3), O3COEFF(K,5*I - 2), &
                   O3COEFF(K,5*I - 1), O3COEFF(K,5*I)
           END DO
        END DO
        CLOSE(C_UNIT)
        
        NN1 = 2687
        
     ELSE
               
        ! Skip header:
        !        DO I = 1,7
        !           READ(C_UNIT,*)
        !        END DO
        ! Read data (wl, coeff1, coeff2, coeff3):
        I = 0 ;  IOERROR = 0
        DO WHILE (IOERROR.ge.0)
           I = I + 1
           READ(C_UNIT,*, IOSTAT=IOERROR) WAVE(I), (O3COEFF(K,I), K = 1,3)
           if(IOERROR.gt.0) I=I-1
        END DO
        CLOSE(C_UNIT)
        
        NN1 = I - 1
     END IF

     WL_O3_UV = PACK(WL,BP_FLAG)
     
     DO K = 1, 3
        CALL AL_INTERP(NN1, WAVE(1:NN1), O3COEFF(K,1:NN1), N_O3_UV, &
             WL_O3_UV, G1(K,:), DO_XSECTION_AKIMA, *880)
     END DO
     
     FORALL(I = 1:NLAYERS) ABS_COEFF_O3(:,I) = 1.0D-20*G1(1,:)* &
          (1.0D0 + G1(2,:)*(TEMP(I) - 273.15d0) + &
          G1(3,:)*(TEMP(I) - 273.15d0)**2)
     
     O3_BP_COEFF12(:) = 1.0D-20*G1(1,:)*G1(2,:)
     O3_BP_COEFF13(:) = 1.0D-20*G1(1,:)*G1(3,:)
     
  END IF
  
  IF(N_O2_UV == 0) THEN
     
     DEALLOCATE(SR_H_FLAG)
     
  ELSE
     
     II = FIND_IDX('o2', TG_INFO%NAME)
     IF(TG_INFO(II)%TYPE.NE.'off') GOTO 200
     TG_INFO(II)%TYPE = 'uv_o2'
     
     DEALLOCATE(WL_READ, DATA_READ)
     IF(ANY(SR_FLAG)) THEN
        MAX_PTS = 16000
        N_TEMP_O2 = 3
     ELSE
        MAX_PTS = 100
        N_TEMP_O2 = 1
     END IF
     
     ALLOCATE(WL_O2_UV(N_O2_UV), O2_COEFF(3,N_O2_UV,N_TEMP_O2), & ! A
          ABS_COEFF_O2(N_O2_UV,NLAYERS), WL_READ(MAX_PTS), & ! A
          DATA_READ(MAX_PTS,3)) ! A
     
     WL_O2_UV = PACK(WL,SR_H_FLAG)
     ABS_COEFF_O2 = 0.0D0
     
     IF(ANY(SR_FLAG)) THEN
        ! WL_IN_NM = .FALSE.
        WLD_INP = 'cm-1'
        DO II = 1, N_TEMP_O2
           FILE_NAME = TRIM(PATH)//TRIM(X_SECTION_FILE(N_XSEC_MOL + 2, II))
           CALL READ_CS(FILE_NAME, 3, *880)  
           
           DO K = 1, 3
              CALL AL_INTERP (N_READ, WL_READ(1:N_READ), &
                   DATA_READ(1:N_READ,K), N_O2_UV, WL_O2_UV, &
                   O2_COEFF(K,1:N_O2_UV,II), DO_XSECTION_AKIMA,*880)
           END DO
        END DO
        
        DO I = 1, NLAYERS
           DELTA = ((TEMP(I) - 100.)/10.) ** 2
           SELECT CASE (FLOOR(TEMP(I)))
           CASE (:189) ;    N_TEMP_RANGE = 1
           CASE (190:280) ; N_TEMP_RANGE = 2
           CASE (281:) ;    N_TEMP_RANGE = 3
           END SELECT
           ABS_COEFF_O2(1:N_O2_UV,I) = &
                (O2_COEFF(1,1:N_O2_UV,N_TEMP_RANGE)*DELTA**2 &
                + O2_COEFF(2,1:N_O2_UV,N_TEMP_RANGE)*DELTA &
                + O2_COEFF(3,1:N_O2_UV,N_TEMP_RANGE))*1.D-20
           !                                          ======
           ! Factor given by Minschwanger --------------^
        END DO
     END IF

     IF(ANY(H_FLAG)) THEN
        !            WL_IN_NM = .TRUE.
        WLD_INP = 'nm'
        FILE_NAME = TRIM(PATH)//TRIM(X_SECTION_FILE(N_XSEC_MOL + 3,1))
        CALL READ_CS(FILE_NAME, 2, *880)  

        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,2), &
             N_O2_UV, WL_O2_UV, O2_COEFF(1,1:N_O2_UV,1), DO_XSECTION_AKIMA,*880)

        ! Calculation from Herzberg Continuum from
        ! formula (3), Yoshino, 1988
        FORALL(J = 1:NLAYERS, I = 1:N_O2_UV, H_FLAG(I)) &
             ABS_COEFF_O2(I,J) = ABS_COEFF_O2(I,J) + &
             EXP(-69.7374D0*(DLOG(204.87D0/WL_O2_UV(I)))**2)* &
             6.884D0*(204.87D0/WL_O2_UV(I))*1.0D-24 &
             + O2_COEFF(1,I,1)*1.D-26/1.33D0*PRESS(J)*0.2095D0
        !                      |___________| |_______________|
        !                       Torr -> hPa  O2 partial pressure
     END IF
  END IF
  
  IF(ANY(CIA_INFO%Flg)) CALL READ_HITRAN_CIA (NSPEC, WL, *880)

  IF( .NOT.UWT ) THEN
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  ENDIF
  
  ! Reading of absorption coefficients for ice/water constituents
  !
  IF( ALLOCATED(WL_READ) ) DEALLOCATE(WL_READ, DATA_READ)
  MAX_PTS = 1000

  ALLOCATE( WL_READ(MAX_PTS), DATA_READ(MAX_PTS,N_ABS_UW) )
  IF( LoIW(1) ) THEN
     ALLOCATE(IWS(1)%ABS_COEFF_UW(N_ABS_UW + Na_ice,NSPEC) )
  ENDIF
  IF( LoIW(2) ) THEN
     ALLOCATE(IWS(2)%ABS_COEFF_UW(N_ABS_UW + IWS(2)%Na_W(1),NSPEC) )
  ENDIF
  
  ! Salinity and temperature correction of water absorption
  IF( TS_CORR ) ALLOCATE( PSIT(NSPEC), PSIS(NSPEC) )

  ! Wavelength in input files is expressed in [nm]
  WLD_INP = 'nm'
  
  IF( IWS(2)%PW_ABS_FILE.AND.TS_CORR ) THEN
     !********************** Korrekturen ****************************
     !  Reading of pure water absorption coefficient [m-1].
     !  Second input parameter defines the number of columns 
     !  in the input files, which contains cross-section
     CALL READ_CS(PSI_NAME, 2, *880)  
     CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), NSPEC,&
          WL(1:NSPEC), PSIS(1:NSPEC), DO_ABSUW_AKIMA, *880)
     CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,2), NSPEC, &
          WL(1:NSPEC), PSIT(1:NSPEC), DO_ABSUW_AKIMA, *880)
     !******************** Korrekturen Ende *************************   
  ENDIF

!  WRITE(*,*) '  LoIW(1)   ', LoIW(1)
!  WRITE(*,*) '  N_abs_UW  ', N_abs_UW
  
! ... Pure ice absorption coefficient [m-1].
  IF( LoIW(1) ) THEN 
     IF( IWS(1)%PW_ABS_FILE ) THEN
        ! Second input parameter defines the number of columns 
        ! in the input files, which contains cross-section
        CALL READ_CS(IWS(1)%PWAC_NAME, 1, *880)
        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
             NSPEC, WL(1:NSPEC), IWS(1)%ABS_COEFF_UW(1,1:NSPEC), &
             DO_ABSUW_AKIMA, *880)
!        WRITE(*,*) '   DO_ABSUW_AKIMA ', DO_ABSUW_AKIMA
!        OPEN( 47, file='./DATA_OUT/abs_ice.tes' )
!        DO I = 1, NSPEC
!           WRITE(47,*) WL(I), IWS(1)%ABS_COEFF_UW(1,I)
!        ENDDO
!        CLOSE( 47 )
!        STOP
     ELSE
        ! Pure ice absorption coefficient set to constant input value
        IWS(1)%ABS_COEFF_UW(1,1:NSPEC) = IWS(1)%PW_ABS
     ENDIF
  ENDIF  ! Pure ice absorption 

! ... Pure water absorption coefficient [m-1].
  IF( LoIW(2) ) THEN 
     IF( IWS(2)%PW_ABS_FILE ) THEN
        ! Second input parameter defines the number of columns 
        ! in the input files, which contains cross-section
        CALL READ_CS(IWS(2)%PWAC_NAME, 1, *880)  
        ! write(*,*) '    After pure water absorption '
        ! write(*,*) WL_READ(1), WL_READ(N_READ)
        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
             NSPEC, WL(1:NSPEC), IWS(2)%ABS_COEFF_UW(1,1:NSPEC), &
             DO_ABSUW_AKIMA, *880)
        ! Interpolation of water abs. coefficient on the reference 
        ! wavelength. Extrapolation as ZERO!
        IF( WL0 >= WL_READ(1) .AND. WL0 <= WL_READ(N_READ) ) THEN
           CALL LIN_INT(N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
                1, WL0, Alf0, 'Z')
        ELSE
           ! Set the absorption coefficient of pure water at the wavelength 440 nm
           Alf0 = 0.00635d0
        ENDIF
     ELSE
        ! Pure water absorption coefficient set to constant input value
        IWS(2)%ABS_COEFF_UW(1,1:NSPEC) = IWS(2)%PW_ABS
        ! Alf0 = PW_ABS
        ! Set the absorption coefficient of pure water at the wavelength 440 nm
        Alf0 = 0.00635d0
     ENDIF
  ENDIF  ! Pure water absorption

  DO I = 1, 2
     IF( .NOT.IWS(I)%Absw_chl ) CYCLE
     ! Reading of chlorophyll specific absorption coefficient
     CALL READ_CS(IWS(I)%CHL_NAME, 1, *880)  
     ! write(*,*) '    Before  chlorophyll specific '
     ! write(*,*) '  N_READ = ', N_READ
     CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), NSPEC, &
          WL(1:NSPEC), IWS(I)%ABS_COEFF_UW(2,1:NSPEC), DO_ABSUW_AKIMA, *880)
     ! write(*,*) '    After  chlorophyll specific '
  ENDDO
      
! *****  Reading absorption coefficient of additional substances in ice and water 
! ... Additional absorber in ice layers
  KK = 0
  DO I = 1, LA_N_IW
     DO II = 1, IWS(1)%Na_W(I)
        KK = KK + 1
        CALL READ_CS(IWS(1)%AFN_coef(II,I)%S, 1, *880)  
        IF( N_READ==0 ) THEN
           IWS(1)%ABS_COEFF_UW(2+KK,1:NSPEC) = 0.d0
        ELSE
           CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
                NSPEC, WL(1:NSPEC), IWS(1)%ABS_COEFF_UW(2+KK,1:NSPEC), &
                DO_ABSUW_AKIMA, *880)
        ENDIF
     ENDDO  ! over additional absorber in ice and water (II)
  ENDDO  ! over ice layers (I)
  
! ... Additional absorber in  water
  DO II = 1, IWS(2)%Na_W(1)
     CALL READ_CS(IWS(2)%AFN_coef(II,1)%S, 1, *880)  
     IF( N_READ==0 ) THEN
        IWS(2)%ABS_COEFF_UW(2+II,1:NSPEC) = 0.d0
     ELSE
        CALL AL_INTERP (N_READ, WL_READ(1:N_READ), DATA_READ(1:N_READ,1), &
             NSPEC, WL(1:NSPEC), IWS(2)%ABS_COEFF_UW(2+II,1:NSPEC), &
             DO_ABSUW_AKIMA, *880)
     ENDIF
  ENDDO  ! over additional absorber in ice and water (II)

  ! WRITE(*,*) ' --->     "abs_coeff_read.f"    <--- '
  ! WRITE(*,*) ' ---> Test absorption coefficients in water'
  ! DO I = 1, NSPEC
  !    WRITE(*,'(2x,F8.3,10(2x,e12.4))') WL(I),&
  !        (IWS(1)%ABS_COEFF_UW(K,I), K=1,N_ABS_UW+IWS(1)%Na_W(1))
  ! ENDDO

  ! Use normalization factor to calculate cross-section instead
  !   of absorption coefficient for additional absorber in water.
  ! It will be done for additional absorber in ice in "st_uwt_coeff.f90"
  DO II = 1, IWS(2)%Na_W(1)
     IWS(2)%ABS_COEFF_UW(2+II,1:NSPEC) = &
          IWS(2)%ABS_COEFF_UW(2+II,1:NSPEC)/IWS(2)%C_NF(II,1)
  ENDDO  ! over additional absorber in water (II)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  !******************************************************************

200 CALL WRITE_MESSAGE (R_NAME, 'ERROR', '')
  WRITE(EU, *)'   Wrong gas type for O2, please check if '
  WRITE(EU, *)'   line-by-line/ESFT mode is switched off'
  RETURN 1
  
201 CALL WRITE_MESSAGE (R_NAME, 'ERROR', '')
  WRITE(EU, *)&
       '   Solar irradiance spectrum does not cover selected spectral range !'
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
!************************************************************
!************************************************************
  SUBROUTINE READ_CS(FILENAME, N_COL, *)

    INTEGER, INTENT(IN) :: N_COL
    CHARACTER (LEN = :), Allocatable :: FILENAME
    ! Local parameters
    CHARACTER (LEN = 7), PARAMETER :: R_NAME = 'READ_CS'
    INTEGER   II, K, KK
    CHARACTER (LEN = 1) :: HEADERLINE
    REAL (KND) TMP(N_COL), WL_START, WL_END, WL_TMP, WL_LOC, WL_NEXT, &
         WL_OTHER, SN, SO, SN_1, SO_1
    REAL (KND), ALLOCATABLE :: TMP_1(:), TMP_2(:,:)
    !------------------------------------------------------------------

    IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//FILENAME 
    OPEN(C_UNIT, FILE = FILENAME, STATUS ='OLD')
    
    HEADERLINE = '#'
    DO WHILE (HEADERLINE == ';' .OR. HEADERLINE == '#' &
         .OR. HEADERLINE == '>'.OR. HEADERLINE == '*')
       READ(C_UNIT,*) HEADERLINE
    END DO
    BACKSPACE(C_UNIT) 
    
    WL_START = MINVAL(WL,1)
    WL_END   = MAXVAL(WL,1) 
    
    WL_LOC = 0.0D0
    N_READ  = 0
    
    READ(C_UNIT,*,ERR=112,END=111) WL_TMP
    WL_LOC = WL_CONVERT(WL_TMP, WLD_INP)
    WL_NEXT = WL(MINLOC(DABS(WL_LOC - WL),1))
    WL_OTHER = WL(MAXLOC(DABS(WL_LOC - WL),1))
    SN = SIGN(1.0D0,WL_LOC - WL_NEXT)
    SN_1 = SN
    SO = SIGN(1.0D0,WL_LOC - WL_OTHER)
    SO_1 = SN
    KK = 1
    IF(WL_LOC >= WL_START .AND. WL_LOC <= WL_END) GOTO 101
    
    DO 
       READ(C_UNIT,*,ERR=112,END=111) WL_TMP
       WL_LOC = WL_CONVERT(WL_TMP, WLD_INP)
       SN_1 = SIGN(1.0D0,WL_LOC - WL_NEXT)
       KK = KK + 1
       IF(SN_1 .NE. SN) EXIT
    END DO
101 CONTINUE
    DO II  = 1, MIN(KK,4) ! Three additional points for akima
       BACKSPACE(C_UNIT)
    END DO
    
    KK = 0
    DO 
       READ(C_UNIT,*,ERR=112,END=111) WL_TMP, (TMP(K), K = 1, N_COL)
       WL_LOC = WL_CONVERT(WL_TMP, WLD_INP)
       ! WRITE(*,*) '--->', WL_LOC, WL_START, WL_END
       ! WRITE(*,*) '--->', WL_LOC, TMP(1)
       N_READ = N_READ + 1
       IF(N_READ.GT.MAX_PTS) THEN
          ALLOCATE(TMP_1(MAX_PTS), TMP_2(MAX_PTS,N_COL))
          TMP_1(1:MAX_PTS)  = WL_READ(1:MAX_PTS)
          TMP_2(1:MAX_PTS,:) = DATA_READ(1:MAX_PTS,1:N_COL)
          DEALLOCATE(WL_READ, DATA_READ)
          MAX_PTS = MAX_PTS*2
          ALLOCATE(WL_READ(MAX_PTS), DATA_READ(MAX_PTS,N_COL))
          WL_READ(1:N_READ - 1) = TMP_1(1:N_READ - 1)
          DATA_READ(1:N_READ - 1,:) = TMP_2(1:N_READ - 1,:)
          DEALLOCATE(TMP_1, TMP_2)
       END IF
       WL_READ(N_READ) =  WL_LOC
       DATA_READ(N_READ,1:N_COL) = TMP(:)
       SO_1 = SIGN(1.0D0,WL_LOC - WL_OTHER)
       IF(SO_1 .NE. SO) THEN 
          KK = KK + 1
          IF(KK == 3) EXIT ! Three additional points for akima
       END IF
    END DO
    ! N_READ = N_READ - 1
111 CONTINUE
    CLOSE(C_UNIT)

    ! pause
    
    IF(MAXVAL(WL_READ(1:N_READ)) < MINVAL(WL).OR. &
         MINVAL(WL_READ(1:N_READ)) > MAXVAL(WL)) THEN
       N_READ = 0
       WL_READ = 0.0D0
    END IF
    
    RETURN 

112 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
    RETURN 1
    
  END SUBROUTINE READ_CS
  !********************************************************************
  DOUBLE PRECISION FUNCTION WL_CONVERT(WL_TMP, WLD_INP)
    CHARACTER (LEN=*) WLD_INP
    REAL (KND)  WL_TMP
    SELECT CASE( TRIM(WLD_INP) )
    CASE('nm')       ;  WL_CONVERT = WL_TMP
    CASE('micron')   ;  WL_CONVERT = WL_TMP*1.D+3
    CASE('angstrom') ;  WL_CONVERT = WL_TMP*1.d-1
    CASE('cm-1')     ;  WL_CONVERT = 1.D+7/WL_TMP
    END SELECT
    RETURN 
  END FUNCTION WL_CONVERT
  
  !**************************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    IF(ALLOCATED(PSIS)) DEALLOCATE( PSIS,PSIT )
    IF(ALLOCATED(ABS_COEFF)) DEALLOCATE(ABS_COEFF, BP_FLAG, O3_UV_CS)
    IF(ALLOCATED(SR_H_FLAG)) DEALLOCATE(SR_H_FLAG)
    IF(ALLOCATED(SOL_FLUX)) DEALLOCATE(SOL_FLUX) 
    IF(ALLOCATED(O3_BP_COEFF12)) DEALLOCATE(ABS_COEFF_O3, &
         O3_BP_COEFF12, O3_BP_COEFF13) 
    IF(ALLOCATED(ABS_COEFF_O2)) DEALLOCATE(ABS_COEFF_O2)
    IF(ALLOCATED(ABS_COEFF_UW)) DEALLOCATE(ABS_COEFF_UW)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!**************************************************************************
END SUBROUTINE ABS_COEFF_READ
!**************************************************************************
