!***************************************************************************
!	SCIATRAN - Version 4.5 - Last modification: 23 Jul 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE AKIMA_ALL_FINE (N_X, X_LOC, INP_ARR, LOG_FLAG, X_COEFF, AKIMA_FLG)

!  USE ST_VARS, ONLY: AKIMA_FLG
  USE TYPES,   ONLY: KND
  
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'AKIMA_ALL_FINE'
  LOGICAL     LOG_FLAG, AKIMA_FLG
  INTEGER     N_X, I
  REAL (KND)  INP_ARR(N_X), FUNC_X(N_X + 4)
  REAL (KND)  ST(N_X + 4), T(N_X + 4)
  REAL (KND)  X_COEFF(N_X,4), X_LOC(N_X + 4)
  
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ! Extrapolation of function:
  ! Auxilliary points are inverted and moved 2 indices up
  ! *****************************************************
  IF(LOG_FLAG) THEN
     DO I = 1, N_X
        FUNC_X(I + 2) = DLOG(INP_ARR(N_X - I + 1))
     END DO
  ELSE
     DO I = 1, N_X
        FUNC_X(I + 2) = INP_ARR(N_X - I + 1)
     END DO
  END IF

  ! Caculation of slopes of curves
  ! ******************************
  DO I = 3, N_X + 1
     ST(I) = (FUNC_X(I + 1) - FUNC_X(I))/(X_LOC(I + 1) - X_LOC(I))
  END DO
  
  IF(AKIMA_FLG) THEN
     ST(2) = 2.0D0*ST(3) - ST(4)
     ST(1) = 2.0D0*ST(2) - ST(3)
     ST(N_X + 2) = 2.0D0*ST(N_X + 1) - ST(N_X)
     ST(N_X + 3) = 2.0D0*ST(N_X + 2) - ST(N_X + 1)
     ! Calculation of all polynomial slopes
     ! ************************************
     DO I = 3, N_X + 2
        IF(ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1)) THEN
           T(I) = 0.5D0*(ST(I + 1) + ST(I))
        ELSE
           T(I) = (ABS(ST(I + 1) - ST(I))*ST(I - 1) + &
                ABS(ST(I - 1) - ST(I - 2))*ST(I))/&
                (ABS(ST(I + 1) - ST(I)) + ABS(ST(I - 1) - ST(I - 2)))
        END IF
     END DO
     ! Calculation of polynomial coefficients
     ! **************************************
     DO I = 1, N_X - 1
        X_COEFF(I,1) = FUNC_X(I + 2)
        X_COEFF(I,2) = T(I + 2)
        X_COEFF(I,3) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3))/&
             (X_LOC(I + 3) - X_LOC(I + 2))
        X_COEFF(I,4) = (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))&
             /(X_LOC(I + 3) - X_LOC(I + 2))**2
     END DO
     X_COEFF(N_X,1) = FUNC_X(N_X + 2)
     X_COEFF(N_X,2:4) = 0.0D0
  ELSE !NOT.AKIMA_FLG
     ST(N_X + 2) = 0.d0 
     FORALL (I = 1:N_X)
        X_COEFF(I,1) = FUNC_X(I + 2)
        X_COEFF(I,2) = ST(I + 2)
     END FORALL
     X_COEFF(:,3:4) = 0.0D0
  END IF 
  
  CALL ST_TRACK (R_NAME, 1, 'O')  
  RETURN

END SUBROUTINE AKIMA_ALL_FINE
      
!*************************************************************************
SUBROUTINE AKIMA_END_FINE (N_X, X_AXIS, NX_NEW, X_NEW, IDX_START, X_AL, &
     X_SCR, X_SCA, X_ASC, X_SCC, X_Plf, AL0_OUT, SCR_OUT, SCA_OUT, ASC_OUT, &
     SCC_OUT, Plf_OUT)
  
  USE INTERN_DIMS, ONLY: Nm_ice
  USE TYPES,       ONLY: KND
  USE ST_VARS,     ONLY: F_LGI_SCR, F_LGI_SCA, F_LGI_EXT, AKIMA_FLG
  
  INTEGER             IDX_X, NX_NEW, N_X, II
  INTEGER             IDX_START(NX_NEW)
  
  REAL (KND)  X_AXIS(N_X), X_NEW(NX_NEW)
  REAL (KND), DIMENSION (N_X,4):: X_ASC, X_SCA, X_SCR, X_AL, X_Plf
  REAL (KND), DIMENSION (NX_NEW):: AL0_OUT, SCR_OUT, SCA_OUT, ASC_OUT, Plf_OUT
  REAL (KND)  X_SCC(N_X,4,Nm_ice), SCC_OUT(NX_NEW,Nm_ice)
  REAL (KND)  DELTA

  DO IDX_X = 1, NX_NEW   
     DELTA = X_NEW(IDX_X) - X_AXIS(IDX_START(IDX_X)) 
     II = IDX_START(IDX_X)
     
     AL0_OUT(IDX_X) = X_AL(II,1) + X_AL(II,2)*DELTA
     IF(AKIMA_FLG) AL0_OUT(IDX_X) = AL0_OUT(IDX_X) + &
          X_AL(II,3)*DELTA**2 + X_AL(II,4)*DELTA**3
     IF(F_LGI_EXT) AL0_OUT(IDX_X) = DEXP(AL0_OUT(IDX_X))
     
     SCR_OUT(IDX_X) = X_SCR(II,1) + X_SCR(II,2)*DELTA
     IF(AKIMA_FLG) SCR_OUT(IDX_X) = SCR_OUT(IDX_X) + &
          X_SCR(II,3)*DELTA**2 + X_SCR(II,4)*DELTA**3
     IF(F_LGI_SCR) SCR_OUT(IDX_X) = DEXP(SCR_OUT(IDX_X))
     
     SCA_OUT(IDX_X) = X_SCA(II,1) + X_SCA(II,2)*DELTA
     IF(AKIMA_FLG) SCA_OUT(IDX_X) = SCA_OUT(IDX_X) + &
          X_SCA(II,3)*DELTA**2 + X_SCA(II,4)*DELTA**3
     IF(F_LGI_SCA) SCA_OUT(IDX_X) = DEXP(SCA_OUT(IDX_X))

     Plf_OUT(IDX_X) = X_Plf(II,1) + X_Plf(II,2)*DELTA
     IF(AKIMA_FLG) Plf_OUT(IDX_X) = Plf_OUT(IDX_X) + &
          X_Plf(II,3)*DELTA**2 + X_Plf(II,4)*DELTA**3
     IF(F_LGI_SCA) Plf_OUT(IDX_X) = DEXP(Plf_OUT(IDX_X))
     
     ASC_OUT(IDX_X) = X_ASC(II,1) + X_ASC(II,2)*DELTA
     IF(F_LGI_SCA) ASC_OUT(IDX_X) = ASC_OUT(IDX_X) + &
          X_ASC(II,3)*DELTA**2 + X_ASC(II,4)*DELTA**3

     SCC_OUT(IDX_X,:) = X_SCC(II,1,:) + X_SCC(II,2,:)*DELTA
     IF(F_LGI_SCA) SCC_OUT(IDX_X,:) = SCC_OUT(IDX_X,:) + &
          X_SCC(II,3,:)*DELTA**2 + X_SCC(II,4,:)*DELTA**3
  END DO   ! IDX_X
  
  RETURN 
END SUBROUTINE AKIMA_END_FINE
! *************************************************************************
SUBROUTINE LIN_END_FINE (N_X, X_AXIS, NX_NEW, X_NEW, IDX_START, X_AL, &
     X_SCR, X_SCA, X_ASC, AL0_OUT, SCR_OUT, SCA_OUT, ASC_OUT, LOG_FLAG)

  LOGICAL             LOG_FLAG
  INTEGER             IDX_X, NX_NEW, N_X, Mk, I
  INTEGER             IDX_START(NX_NEW)
  DOUBLE PRECISION, DIMENSION(NX_NEW) &
       :: X_NEW, AL0_OUT, SCR_OUT, SCA_OUT,ASC_OUT 
  DOUBLE PRECISION, DIMENSION(N_X) &
       :: X_AXIS, X_ASC, X_SCA, X_SCR, X_AL

  DOUBLE PRECISION    Coef(N_X,4)
  DOUBLE PRECISION    DELTA 
  
  coef(N_X,:) = 0.0d0
  IF( LOG_FLAG ) THEN
     DO I = 1, N_X - 1
        Delta = X_AXIS(I) - X_AXIS(I+1)
        Coef(I,1) = DLOG(X_AL(I)/X_AL(I+1))/Delta
        Coef(I,2) = DLOG(X_SCR(I)/X_SCR(I+1))/Delta
        Coef(I,3) = DLOG(X_SCA(I)/X_SCA(I+1))/Delta
        Coef(I,4) = ( X_ASC(I)-X_ASC(I+1) )/Delta
     ENDDO
     DO IDX_X = 1,NX_NEW - 1
        Mk = N_X - IDX_START(IDX_X) + 1
        DELTA = X_NEW(IDX_X) - X_AXIS(Mk)
        AL0_OUT(IDX_X) = DEXP( X_AL (Mk) + Coef(Mk-1,1)*Delta )
        SCR_OUT(IDX_X) = DEXP( X_SCR(Mk) + Coef(Mk-1,2)*Delta )
        SCA_OUT(IDX_X) = DEXP( X_SCA(Mk) + Coef(Mk-1,3)*Delta )
        ASC_OUT(IDX_X) =  X_ASC(Mk) + Coef(Mk-1,4)*Delta
     END DO                 ! IDX_X
     Mk = N_X - IDX_START(NX_NEW) + 1
     AL0_OUT(NX_NEW) =  X_AL (Mk) 
     SCR_OUT(NX_NEW) =  X_SCR(Mk) 
     SCA_OUT(NX_NEW) =  X_SCA(Mk) 
     ASC_OUT(NX_NEW) =  X_ASC(Mk)         
  ELSE
     DO I = 1,N_X - 1
        Delta = X_AXIS(I) - X_AXIS(I+1)
        Coef(I,1) = ( X_AL(I)  - X_AL(I+1) )/Delta
        Coef(I,2) = ( X_SCR(I) - X_SCR(I+1) )/Delta
        Coef(I,3) = ( X_SCA(I) - X_SCA(I+1) )/Delta
        Coef(I,4) = ( X_ASC(I) - X_ASC(I+1) )/Delta
     ENDDO
     DO IDX_X = 1,NX_NEW - 1
        Mk = N_X - IDX_START(IDX_X) + 1
        DELTA = X_NEW(IDX_X) - X_AXIS(Mk)
        AL0_OUT(IDX_X) =  X_AL (Mk) + Coef(Mk-1,1)*Delta
        SCR_OUT(IDX_X) =  X_SCR(Mk) + Coef(Mk-1,2)*Delta
        SCA_OUT(IDX_X) =  X_SCA(Mk) + Coef(Mk-1,3)*Delta
        ASC_OUT(IDX_X) =  X_ASC(Mk) + Coef(Mk-1,4)*Delta
     END DO                 ! IDX_X
     Mk = N_X - IDX_START(NX_NEW) + 1
     AL0_OUT(NX_NEW) =  X_AL (Mk) 
     SCR_OUT(NX_NEW) =  X_SCR(Mk) 
     SCA_OUT(NX_NEW) =  X_SCA(Mk) 
     ASC_OUT(NX_NEW) =  X_ASC(Mk)         
  END IF
  
  RETURN 
END SUBROUTINE LIN_END_FINE
!*************************************************************************
SUBROUTINE LIN_FINE_1 (N_X, X_AXIS, NX_NEW, X_NEW, IDX_START, INP, OUT, &
     LOG_FLAG)
  LOGICAL             LOG_FLAG
  INTEGER             IDX_X, NX_NEW, N_X, Mk, I
  INTEGER             IDX_START(NX_NEW)
  DOUBLE PRECISION, DIMENSION(NX_NEW) :: X_NEW,  OUT
  DOUBLE PRECISION, DIMENSION(N_X)    :: X_AXIS, INP
  
  DOUBLE PRECISION    Coef(N_X)
  DOUBLE PRECISION    DELTA 

  coef(N_X) = 0.0d0
  IF( LOG_FLAG ) THEN
     DO I = 1, N_X - 1
        Coef(I) = DLOG(INP(I)/INP(I+1))/(X_AXIS(I)-X_AXIS(I+1))
     ENDDO
     DO IDX_X = 1,NX_NEW - 1
        Mk = N_X - IDX_START(IDX_X) + 1
        DELTA = X_NEW(IDX_X) - X_AXIS(Mk)
        OUT(IDX_X) = DEXP( INP(Mk) + Coef(Mk-1)*Delta )
     END DO  ! IDX_X
     OUT(NX_NEW) = INP(N_X-IDX_START(NX_NEW)+1) 
  ELSE
     DO I = 1,N_X - 1
        Coef(I) = (INP(I) - INP(I+1))/(X_AXIS(I) - X_AXIS(I+1))
     ENDDO
     DO IDX_X = 1,NX_NEW - 1
        Mk = N_X - IDX_START(IDX_X) + 1
        OUT(IDX_X) = INP(Mk) + Coef(Mk-1)*(X_NEW(IDX_X)-X_AXIS(Mk))
     END DO  ! IDX_X
     OUT(NX_NEW) = INP(N_X-IDX_START(NX_NEW)+1) 
  END IF
  
  RETURN 
END SUBROUTINE LIN_FINE_1

!*************************************************************************
SUBROUTINE AKIMA_TH_FINE (X_AX, X, X_AL, X_SCR, X_SCA, X_ASC, X_SCC, X_Plf, &
     AL0_TH, SCR_TH, SCA_TH, ASC_TH, SCC_TH, Plf_TH)

  USE TYPES,   ONLY: KND
  USE ST_VARS, ONLY: F_LGI_SCR, F_LGI_SCA, F_LGI_EXT, AKIMA_FLG

  REAL (KND), DIMENSION(4):: X_AL, X_SCR, X_ASC, X_SCA, X_SCC, X_Plf
  REAL (KND) DD, X, AL0_TH, SCR_TH, SCA_TH, ASC_TH, X_AX, SCC_TH, Plf_TH

  DD = X - X_AX 

  SCR_TH = X_SCR(1) + X_SCR(2)*DD
  IF(AKIMA_FLG) SCR_TH = SCR_TH + X_SCR(3)*DD**2 + X_SCR(4)*DD**3
  IF(F_LGI_SCR) SCR_TH = DEXP(SCR_TH)
      
  SCA_TH = X_SCA(1) + X_SCA(2)*DD
  IF(AKIMA_FLG) SCA_TH = SCA_TH + X_SCA(3)*DD**2 + X_SCA(4)*DD**3
  IF(F_LGI_SCA) SCA_TH = DEXP(SCA_TH)
      
  ASC_TH = X_ASC(1) + X_ASC(2)*DD
  IF(AKIMA_FLG) ASC_TH = ASC_TH + X_ASC(3)*DD**2 + X_ASC(4)*DD**3

  AL0_TH = X_AL(1) + X_AL(2)*DD
  IF(AKIMA_FLG) AL0_TH =  AL0_TH + X_AL(3)*DD**2 + X_AL(4)*DD**3
  IF(F_LGI_EXT) AL0_TH = DEXP(AL0_TH)

  SCC_TH = X_SCC(1) + X_SCC(2)*DD
  IF(AKIMA_FLG) SCC_TH = SCC_TH + X_SCC(3)*DD**2 + X_SCC(4)*DD**3
  IF(F_LGI_EXT) SCC_TH = DEXP(SCC_TH)

  Plf_TH = X_Plf(1) + X_Plf(2)*DD
  IF(AKIMA_FLG) Plf_TH = Plf_TH + X_Plf(3)*DD**2 + X_Plf(4)*DD**3
  IF(F_LGI_EXT) Plf_TH = DEXP(Plf_TH)
     
  RETURN
END SUBROUTINE AKIMA_TH_FINE
!*************************************************************************

