C********************************************************************C
C
C	SCIATRAN - Version 1.2.15 - Last modification: 23 Aug 2002
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
c *************************************************************************
      Subroutine AKIMA_INTERP (n, x, y, m, x_in, y_in, error )

c  Akima interpolation:

c  Given n-dimensional arrays of abcissae x and tabulated function y,
c  this routine returns the m-dimensional array of Akima-interpolated
c  values y_in on the abcissae x_in

c  Note: Extrapolation is done with zeros. x and x_in
c        may BOTH increase or decrease. 

      character*12      routine_name
      parameter        (routine_name = 'akima_interp')

      LOGICAL             ERROR
      INTEGER             N, M, K

      DOUBLE PRECISION    X_IN(M), Y_IN(M)
      DOUBLE PRECISION, DIMENSION (N) :: A, B, C, D, X, Y
      DOUBLE PRECISION, DIMENSION (N + 4) :: X1, Y1

      error = .false.

      if (x(2).gt.x(1)) then
         x1(3:N + 2) = x(1:N) ;  y1(3:N + 2) = y(1:N)
      else
         x1(3:N + 2) = x(N:1:-1) ;  y1(3:N + 2) = y(N:1:-1)
      endif

      a(:) = 0.0d0 ; b(:) = 0.0d0 ; c(:) = 0.0d0 ; d(:) = 0.0d0

c  Calculate of polynomial coefficients

      Call akima_sub (X1, Y1, n, a, b, c, d)

c  Calculation of interpolated values

      Do k = 1, m
         Call akima_polyn (n, X1(3:N - 2), a, b, c, d, 
     &                     x_in(k), y_in(k), error)
         if (error) then
            call write_message(routine_name, 'ERROR', 
     &                         'Error in akima_polyn')
            return
         endif
      End Do

      Return
      End

c *************************************************************************
      Subroutine AKIMA_EXTRAPOL (n, x_arr, y_arr, st)

      Integer            n, i
      Double Precision   st(n + 4), x_arr(n+4), y_arr(n+4)
      
c     Calculation of extrapolated x-values left and right

      x_arr(2) = x_arr(3) + x_arr(4) - x_arr(5)
      x_arr(1) = x_arr(2) + x_arr(3) - x_arr(4)

      x_arr(n+3) = x_arr(n+2) + x_arr(n+1) - x_arr(n)
      x_arr(n+4) = x_arr(n+3) + x_arr(n+2) - x_arr(n+1)

c     Caculation of slopes of curves

      FORALL(I = 3:N + 1) 
     $     st(i) = (y_arr(i + 1) - y_arr(i))/(x_arr(i + 1) - x_arr(i))

c     Calculation of extrapolated y-values left and right

      y_arr(2) = (x_arr(3)-x_arr(2)) * (st(4)-2.0d0*st(3)) + y_arr(3)
      st(2)    = (y_arr(3)-y_arr(2)) / (x_arr(3)-x_arr(2))

      y_arr(1) = (x_arr(2)-x_arr(1)) * (st(3)-2.0d0*st(2)) + y_arr(2)
      st(1)    = (y_arr(2) - y_arr(1)) / (x_arr(2) - x_arr(1))

      y_arr(n+3) = (2.0d0*st(n+1) - st(n)) * (x_arr(n+3)-x_arr(n+2)) 
     &             + y_arr(n+2)
      st(n+2)    = (y_arr(n+3) - y_arr(n+2))/(x_arr(n+3) - x_arr(n+2))

      y_arr(n+4) = (2.0d0*st(n+2) - st(n+1)) * (x_arr(n+4)-x_arr(n+3))
     &             + y_arr(n+3)
      st(n+3)    = (y_arr(n+4) - y_arr(n+3))/(x_arr(n+4) - x_arr(n+3))

c     End of extrapolation

      Return
      End

c *********************************************************************
      Subroutine AKIMA_POLYN (n, x_arr, a, b, c, d, x, y, error )

c  given an n-dim. array of abscissa x_arr and arrays a, b, c, d of
c  polynomial coefficients as returned by the akima interpolation,
c  this subroutine returns the interpolated value y at abscissa x

      character*11      routine_name
      parameter        (routine_name = 'akima_polyn')

      Logical           error
      Integer           n, j
      DOUBLE PRECISION, DIMENSION (N) :: A, B, C, D, X_ARR
      DOUBLE PRECISION    x, y, delta, xl


c     Initialize some variables
c     ( mainly to keep the compilers quiet)
c     -------------------------------------
      j = 0
      error = .false.

c ----------------------------------------------------
c Extrapolate with zeros,
c changed by M. Buchwitz (21 March 1996):

      if ( x .lt. x_arr(1) ) then
         y = 0.0d0
         return
      endif

      if ( x .gt. x_arr(n) ) then
         y = 0.0d0
         return
      endif
c ----------------------------------------------------

      Call LIN_SEARCH (x, xl, j, n, x_arr, error)
      if (error) then
         call write_message(routine_name, 'ERROR', 
     &        'When calling lin_search')
         return
      endif

      delta = x - x_arr(j)

      y = a(j) + b(j) * delta + c(j) * delta**2 + d(j) * delta**3


      Return
      End

c *************************************************************************
      Subroutine AKIMA_SUB (x_arr, y_arr, n, a_arr, b_arr, c_arr, d_arr)

      INTEGER           N, I
      DOUBLE PRECISION, DIMENSION (N) :: A_ARR, B_ARR, C_ARR, D_ARR
      DOUBLE PRECISION, DIMENSION (N + 4) :: X_ARR, Y_ARR, ST, T

c     Start of actual interpolation

      Call akima_extrapol (n, x_arr, y_arr, st)

c     Calculation of all polynomial slopes

      Do i = 3, n+2
         
         If ( st(i-2) .eq. st(i-1) .and. st(i) .eq. st(i+1) ) Then
            t(i) = 0.5d0 * (st(i+1) + st(i))
         Else
            t(i) = ( dabs( st(i+1) - st(i)   ) * st(i-1) +
     &               dabs( st(i-1) - st(i-2) ) * st(i)    ) /
     &             ( dabs(st(i+1)-st(i)) + dabs(st(i-1)-st(i-2)) )
         End If

      End Do


c     Calculation of polynomial coefficients

      FORALL(I = 1:N - 1)
         a_arr(i) = y_arr(i + 2)
         b_arr(i) = t(i + 2)
         c_arr(i) = ( 3.0d0*st(i+2) - 2.0d0*t(i + 2) - t(i + 3) ) /
     &              ( x_arr(i + 3) - x_arr(i + 2) )
         d_arr(i) = ( t(i + 2) + t(i + 3) - 2.0d0*st(i+2) ) /
     &              ( x_arr(i + 3) - x_arr(i + 2) )**2

      END FORALL

      Return
      End
c *************************************************************************
