C********************************************************************C
C
C       SCIATRAN - Version 3.2 - Last modification: 10 Apr 2013
C
C	Copyright (C) 1995-1998, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ANGLES (ANG, EARTH_RADIUS, Z_IN, Z_OUT)

C     Z_IN and Z_OUT must be on the same side
C     with respect to the tahgent height.

      DOUBLE PRECISION     ANG(3)
      DOUBLE PRECISION     COS_PSI_IN, MJU_IN, PHI_IN, COS_PSI_OUT 
      DOUBLE PRECISION     MJU_OUT, PHI_OUT, Z_IN, EARTH_RADIUS, SD
      DOUBLE PRECISION     Z_OUT, R, H_0, DELTA, SIN_1, SIN_PSI, S1 
      DOUBLE PRECISION     COS_PHI_0, SIN_PHI_0, ETA_0, ZETA_0, KSI_0
      DOUBLE PRECISION     SIN_PSI_OUT, SIN_OUT

      COS_PSI_IN = ANG(1); MJU_IN = ANG(2); PHI_IN = ANG(3)

      R = EARTH_RADIUS + Z_IN
      H_0 = R*DSQRT(1.0D0 - (MJU_IN)**2) - EARTH_RADIUS

      SIN_1 = DSQRT(1.0D0 - MJU_IN**2)
      COS_PHI_0 = DCOS(PHI_IN)
      SIN_PHI_0 = DSIN(PHI_IN)

      IF(H_0.GT.Z_OUT) STOP 'Tangent height is above TOA'

      DELTA =  DABS( ! Added in version 3.2 
     $     DSQRT((2.0D0*EARTH_RADIUS + Z_IN + H_0)*(Z_IN - H_0))
     $     - DSQRT((2.0D0*EARTH_RADIUS + Z_OUT + H_0)*(Z_OUT - H_0)))
      MJU_OUT = (MJU_IN*R - DELTA)/
     $     DSQRT((MJU_IN*R - DELTA)**2 + (R*SIN_1)**2)
      SIN_OUT = R*SIN_1/(EARTH_RADIUS + Z_OUT)
            
      SIN_PSI = DSQRT(1.0D0 - COS_PSI_IN**2)
      ZETA_0 = MJU_IN*COS_PSI_IN - SIN_1*SIN_PSI*COS_PHI_0
      KSI_0 = MJU_IN*SIN_PSI + SIN_1*COS_PSI_IN*COS_PHI_0

      COS_PSI_OUT = (COS_PSI_IN*R - DELTA*ZETA_0)/DSQRT((COS_PSI_IN*R - 
     $     DELTA*ZETA_0)**2 + (SIN_PSI*R - DELTA*KSI_0)**2
     $     + (DELTA*SIN_1*SIN_PHI_0)**2)
      SIN_PSI_OUT = DSQRT(1.0D0 - DMAX1(1.0D0,COS_PSI_OUT**2))
      
      ETA_0 = ((R - DELTA*MJU_IN)*SIN_PSI*COS_PHI_0 - 
     $     DELTA*SIN_1*COS_PSI_IN)/(EARTH_RADIUS + Z_OUT)
      
      ETA_0 = ETA_0 + 1.0D-39   ! numerical stabilization
      
      S1 = ETA_0/(DSQRT(ETA_0**2 + (SIN_PSI*SIN_PHI_0)**2))
      S1 = S1 - SIGN(1.0D-13,S1) ! numerical stabilization
      
      SD = R*SIN_PSI*SIN_1*SIN_PHI_0/(EARTH_RADIUS + Z_OUT)/
     $     (SIN_PSI_OUT*SIN_OUT + 1.0D-78)
      
      PHI_OUT = SIGN(DACOS(S1),SD)

      ANG(1) = COS_PSI_OUT; ANG(2) = MJU_OUT; ANG(3) = PHI_OUT

      RETURN 
      END

C     ********************************************************

