C***********************************************************************
C                                                                      *
C       SCIATRAN - Version 4.4 - Last modification: 27 Mar 2021        *
C       Copyright (C) Institute of Remote Sensing,                     *
C                      University of Bremen, Germany                   * 
C                                                                      *
C***********************************************************************
      SUBROUTINE APPR_PLANCK(JPOS, POS, N_P, TE, AL, SCA_AL, *)
c     ... Called by "dom_spherical.f"

      USE GRIDS,       ONLY: NL => NLAYERS, ZZZ
      USE INTERN_DIMS, ONLY: NL_pla, NN => Nf_pla, NL_FINE_pla
      USE IRRAD,       ONLY: SOLSPEC => SOL_FLUX, WL_IRRAD_DIM
      USE LOG_PARS,    ONLY: EU, C_UNIT
      USE PROFILES,    ONLY: TTT => TEMP
      USE RTM_OUTPUT,  ONLY: MES_OUT_SOL
      USE ST_VARS,     ONLY: RTM_TYPE, PIE, M_PLA, INT_MS_FLAG, WF_FLAG
      USE THERMAL_VAR, ONLY: OUT_PLANCK, Path_for_PLANCK, WNUMLO, HC_K, 
     $                       WNUMHI, INTEGR_PLANCK, Pla_acc
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: TE_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) ::  JPOS, N_P
      REAL (KND), INTENT(IN) ::  POS, AL(NL), SCA_AL(NL)
c     --------------------
      TYPE (TE_type)  TE
c     --------------------
      INTEGER, POINTER :: N_pla(:)
      REAL (KND), POINTER :: Mup(:), PL_COEF(:,:)
c     Local variables
c     ---------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'APPR_PLANCK'
      INTEGER  I, J, K, NL_FINE
      REAL (KND), DIMENSION (N_P) :: AL_N, Z_N, T_PP, PL_FINE, T_FINE
      REAL (KND)  WLM
      REAL (KND), PARAMETER :: hP = 6.626071d-34, C = 299792458.0d0
c     ==================================================================
      CALL ST_TRACK(R_NAME, 1, 'I')

      N_pla => TE%N_pla   ;   Mup   => TE%Mup
      PL_COEF  => TE%PL_COEF

      NL_FINE = 0
      DO J = 1,NL-1
         DO K = 1, NN
            NL_FINE = NL_FINE + 1
            Z_N(NL_FINE) =  ZZZ(J) - 
     $           (ZZZ(J) - ZZZ(J+1))/DBLE(NN)*DBLE(K-1)
         END DO
      END DO
      NL_FINE = NL_FINE + 1
      Z_N(NL_FINE) =  ZZZ(NL)

c      Linear interpolation of the temperature on the fine altitude grid
      CALL LIN_INT(NL, ZZZ, TTT, N_P, Z_N, T_FINE, 'Z')
     
c      Linear interpolation of the ext. coeff. on the fine altitude grid
      CALL LIN_INT(NL, ZZZ, AL, N_P, Z_N, AL_N, 'Z')

c      ... Planck function on the fine altitude grid
      IF( INTEGR_PLANCK ) THEN
         DO I = 1,N_P
            CALL PLKAVG( WNUMLO, WNUMHI, T_FINE(I), PL_FINE(I), *880 )
         ENDDO
         PL_FINE(:) = PL_FINE(:)/SOLSPEC(JPOS)*Pie
      ELSE
         WLM = POS*1.0D-9
C        ... The Planck function dimension [Phot/s/nm/cm2/sr]:
         PL_FINE(:) = 2.d0*C/WLM**4/(DEXP(HC_K/WLM/T_FINE(:)) - 1.D0)
c        ... W/m^2/m/sr
         IF( TRIM(WL_IRRAD_DIM(2))=='W' ) PL_FINE(:)=PL_FINE(:)*hP*C/WLM
         PL_FINE(:) = M_PLA*PL_FINE(:)/SOLSPEC(JPOS)*Pie
      ENDIF

      IF( TRIM(RTM_TYPE) == 'spher_scat' ) THEN
c        ... Scat. coeff. lLinear interpolation  on fine altitude grid
         CALL LIN_INT(NL, ZZZ, SCA_AL, N_P, Z_N, T_PP, 'Z')
c        ... Thermal emission source function on the fine-grid
         TE%PLA_FINE(1:N_P) = (AL_N(N_P:1:-1) - T_PP(N_P:1:-1))
     $        *PL_FINE(N_P:1:-1)
         IF( WF_FLAG ) TE%B_FINE(1:N_P) = PL_FINE(N_P:1:-1)
c        ... The approximation of Planck functiondoes not need
         IF( TE%NO_SUN.AND.(.NOT.INT_MS_FLAG(JPOS)) ) THEN
            CALL ST_TRACK(R_NAME, 1, 'O')
            RETURN
         ENDIF
      ENDIF

      CALL SIN_FIT

      CALL ST_TRACK(R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c *************************************************************************
      SUBROUTINE SIN_FIT
      INTEGER           I, J, K, Up, Dn, L, Nt, Nf
      INTEGER, PARAMETER :: UNIT = 44
      LOGICAL           OUT, STATUS
      CHARACTER (LEN=4) INT_TO_CHAR
      REAL (KND)        Crit, Ck, Cks, Dtp, Reld
      REAL (KND)   ALW(N_P-1), DZR(N_P), Mat1(NN,NN),
     $             Dif(NN), Tp(NN), P_ec(NN), Res(NN), X(NN), 
     $             S_UM(NN), Prel(NN), A_VEC( (NN + 1)*(NN + 2)/2 ), 
     $             Tra(NN,NN), Tinv(NN,NN), RHS(NN) 
      REAL (KND), ALLOCATABLE :: P_APR(:)
      COMPLEX (kind(0.0d0)) :: Mu_co
      EXTERNAL     INT_TO_CHAR

      DZR (N_P) = 0.D0
      DZR(1:N_P - 1) = Z_N(1:N_P - 1) - Z_N(2:N_P)
      ALW(1:N_P-1) = (AL_N(1:N_P-1) + AL_N(2:N_P))*0.5D0

c     Vertical optical depth for the fine altitude grid
c      WRITE(*,*) '       ======>> Before :'
      T_PP(1) = 0.0D0 
      DO  J = 1, N_P - 1
         T_PP(J+1) = T_PP(J) + ALW(J)*DZR(J)
      ENDDO

      Crit = Pla_acc/100.d0  ;  PL_COEF = 0.d0 

      DO I = 1, NL-1            ! Over all sub-layers
         Up =  (I-1)*NN + 1; Dn = Up + NN 
         Nf = Dn - Up - 1       ! Number of points for fit
         Dtp = T_PP(Dn) - T_PP(Up)
         IF( PL_FINE(Dn) == 0.d0 ) THEN
            Mup(I) = 0.d0 ; Nt = 1 ; CYCLE
         ENDIF
         Mup(I) = DLOG( PL_FINE(Dn)/PL_FINE(Up) )/Dtp
         Tp(1:Nf) = PL_FINE(Up)*DEXP( Mup(I)*(T_PP(Up+1:Dn-1)-T_PP(Up)))
         P_ec(1:Nf) = DABS( 1.0D0 - Tp(1:Nf)/PL_FINE(Up+1:Dn-1) )
         Reld = MAXVAL( P_ec(1:Nf),1 )
cc          WRITE(*,*) ' Reld start ', Reld*100., Mup(I)
c           WRITE(*,*) ' Sub-layers    ', I, Mup(I)
c           WRITE(*,*) ' Planck Dn, Up ', PL_FINE(Dn), PL_FINE(Up)
c           WRITE(*,*)  'T_PP(Dn), T_PP(Up)',T_PP(Dn), T_PP(Up)
c           WRITE(*,*) '       Dtp     ', Dtp
cc        .. To use only exponential-in-tau approximation
cc          N_pla(I) = 1  ;  CYCLE
         IF( Reld <= Crit ) Then
            N_pla(I) = 1
            CYCLE
         ENDIF

         X(1:Nf)   = (T_PP(Up+1:Dn-1) - T_PP(Up))/Dtp
         Dif(1:Nf) = (PL_FINE(Up+1:Dn-1) - Tp(1:Nf))/Tp(1:Nf)

         Nt = 1 ; OUT = .FALSE. ; STATUS = .FALSE.
            
 21      CONTINUE
         Mat1(Nt,1:Nf) = DSIN( DBLE(Nt)*X(1:Nf)*Pie )
         TRA(1:Nt,1:Nt) = 
     $        MATMUL( MAT1(1:Nt,1:Nf),TRANSPOSE(MAT1(1:Nt,1:Nf)) )
         FORALL (K = 1:Nt) 
     $        A_VEC(K*(K - 1)/2 + 1:K*(K + 1)/2) = TRA(K,1:K)
         CALL CHINV_LOC (Nt*(Nt+1)/2, Nt, A_VEC, STATUS)
         IF( STATUS ) THEN
c           No invert-matrix is found. Further iteration impossible.
c           =======================================================
            OUT = .TRUE.  ;  Nt = Nt-1  ;  GOTO 123
         ENDIF

         FORALL (K = 1:Nt) 
     $        Tinv(K,1:K) = A_VEC(K*(K - 1)/2 +1:K*(K + 1)/2)
         FORALL (K = 1:Nt, L = 1:Nt, L > K) Tinv(K,L) = Tinv(L,K)
         RHS(1:Nt) = MATMUL( MAT1(1:Nt,1:Nf),Dif(1:Nf) )
         ReS(1:Nt) = MATMUL( Tinv(1:Nt,1:Nt),RHS(1:Nt) )
         
 123     FORALL( K = 1:Nf )
     $        S_UM(K) = SUM( RES(1:Nt)*MAT1(1:Nt,K) )
         Prel(1:Nf) = Tp(1:Nf)*(1.0D0 + S_UM(1:Nf))
         P_ec(1:Nf) = DABS(1.0D0 - Prel(1:Nf)/PL_FINE(Up+1:Dn-1))
         Reld = MAXVAL( P_ec(1:Nf),1 )

c         WRITE(*,*) '       Reld 125 ', Reld*100.
         IF( OUT ) Then
            PL_COEF(I,1:Nt) = RES(1:Nt)  ;  N_pla(I) = Nt+1  ;  CYCLE
         ENDIF
         IF( Reld <= Crit ) Then
            PL_COEF(I,1:Nt) = RES(1:Nt)  ;  N_pla(I) = Nt+1  ;  CYCLE
         ENDIF
         IF( Nt >= Nf-1 ) Then
            PL_COEF(I,1:Nt) = RES(1:Nt)  ; N_pla(I) = Nt+1
c               write(*,*) '... Nt >= N_max ...'
c               WRITE(*,*) 'UP - ',Z_N(Up),' Dn - ',Z_N(Dn)
            IF( MES_OUT_SOL ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $              'Selected accuracy for Planck function '
     $              //'approximation is not reached: '
     $              //'requested accuracy is 0.01 %')           
               MES_OUT_SOL = .FALSE.
            ENDIF
            CYCLE
         ELSE
            Nt = Nt + 1  ;  GOTO 21
         ENDIF
      ENDDO  ! over sublayers (I)
      
      IF(.NOT. OUT_PLANCK ) RETURN
      ALLOCATE( P_APR(N_P) )
      DO I = 1, NL-1 
         Up =  (I-1)*NN + 1; Dn = Up + NN 
         Nf = Dn - Up - 1       ! Number of points for fit
         Nt = N_pla(I) - 1
         P_APR(Up:Dn) = 
     $        PL_FINE(Up)*DEXP( Mup(I)*(T_PP(Up:Dn)-T_PP(Up)) )
         X(1:Nf) = T_PP(Up+1:Dn-1) - T_PP(Up)

         S_UM = 0.d0 ; Dtp = T_PP(Dn) - T_PP(Up)
         DO K = 1,Nt
c            Ck = DBLE(K)*Pie/Dtp
c            Mu_co = DCMP( Mup(I),Ck )
c            S_UM(1:Nf) = S_UM(1:Nf) + PL_COEF(I,K)*
c     $           AIMAG( EXP( CMPLX(X(1:Nf),0.d0,KND)*Mu_co) )
            Ck = Dtp/DBLE(K)/Pie  ;  Cks = Ck*Mup(I)
            Mu_co = -Ck/DCMP( Cks,1.d0 )
            S_UM(1:Nf) = S_UM(1:Nf) + PL_COEF(I,K)*
     $           AIMAG( EXP(-CMPLX(X(1:Nf),0.d0,KND)/Mu_co) )
         ENDDO
         P_APR(Up+1:Dn-1) = P_APR(Up+1:Dn-1) + PL_FINE(Up)*S_UM(1:Nf)
      ENDDO  ! over sub-layers (I)

      OPEN( C_UNIT,file = TRIM(Path_for_PLANCK) )
      WRITE( C_UNIT,'(A)') '# Number of coefficient used '//
     $     'in approximation for different layers'
      WRITE( C_UNIT,'(A)') '#  No        Layer '//
     $     '        N_k       Bo'
      DO I = 1, NL-1 
         WRITE(C_UNIT,'(1x,I3,2x,f7.3,A,F7.3,2x,I3,2x,E12.4)') 
     $        I, ZZZ(I),'  - ',ZZZ(I+1), N_PLA(I) - 1, Mup(I)
      ENDDO
      WRITE( C_UNIT,'(A)') 
     $     '# '
      WRITE( C_UNIT,'(A)') 
     $     '#     Exact and approximate Planck function '
      WRITE( C_UNIT,'(A)') 
     $     '# Altitude    T(z)      '//
     $     'Exact      Approximate  Relative error '
      WRITE( C_UNIT,'(A)') 
     $     '#  [km]       [K]         '//
     $     'depending on input         [%]        '
      DO J = 1, N_P
         WRITE(C_UNIT,'(2x,F7.3,2x,F7.2,3(2x,e12.4))') 
     $           Z_N(J), T_fine(J), PL_FINE(J), P_APR(J),
     $        (PL_FINE(J)-P_APR(J))/PL_FINE(J)*100.
      ENDDO
      CLOSE(C_UNIT)
      Return
      END SUBROUTINE SIN_FIT      

C***********************************************************************
C*                                                                     *
C*               SUBROUTINE FOR MATRIX INVERSION                       *
C*                    CRAY optimized version                           *
C*                                                                     *
C***********************************************************************
      SUBROUTINE CHINV_LOC(NVEC, N, A, STATUS)
      INTEGER       NVEC, N, K, I, J
      LOGICAL       STATUS
      REAL (KND)    A(NVEC), X

      A(1) = 1.0D0/DSQRT(A(1))
      FORALL (I = 2:N) A(I*(I - 1)/2 + 1) = A(I*(I - 1)/2 + 1)*A(1)

      DO I = 2, N
         DO J = 2, I - 1
            X = A(I*(I - 1)/2 + J)
            DO K = 1, J - 1
               X = X - A(I*(I - 1)/2 + K)*A(J*(J - 1)/2 + K)
            END DO
            A(I*(I - 1)/2 + J)=X*A(J*(J - 1)/2 + J)
         END DO
         X = A(I*(I + 1)/2)
         DO K = I*(I - 1)/2 + 1, I*(I + 1)/2  - 1
            X = X - A(K)*A(K)
         END DO
         IF( X <= 0.d0 ) THEN
c            WRITE(*,*) 'X <= 0.0d0', X
            STATUS = .TRUE.
            RETURN
         ENDIF
         A(I*(I + 1)/2) = 1.0D0/DSQRT(X)
      END DO

C      INVERS L MATRIX
      DO I= 2, N
         DO J = 2, I
            X = 0
            DO  K = 1, I - J + 1
               X = X - A(I*(I + 1)/2 - K)*
     $              A(J - 1 + (I - K - 1)*(I - K)/2)
            END DO
            A(I*(I - 1)/2 + J - 1)=A(I*(I + 1)/2)*X
         END DO
      END DO

C      INVERS A MATRIX
      DO I= 1, N
         DO J = 1, I
            X = 0
            DO  K = I, N
               X = X + A(K*(K - 1)/2 + I)*A(K*(K - 1)/2 + J)
            END DO
            A(I*(I - 1)/2 + J) = X
         END DO
      END DO

      RETURN
      END SUBROUTINE CHINV_LOC

C  *******************************************
      END
