C***********************************************************************
C                                                                      
C	SCIATRAN - Version 4.5 - Last modification: 13 Jan 2021
C                                                                      
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C                                                                      
C***********************************************************************
      SUBROUTINE APPR_SPHER_LS_MAIN(NLOS, AL, SLS, SWF, IMS_FLG, *)
c     ... Called from "dom_spherical.f"

      USE GRIDS,       ONLY: NLAYERS, ZZZ, N_P => NL_FINE, 
     $                       INTERP_IDX, Z_FINE
      USE LOG_PARS,    ONLY: LOG_LEVEL, N_DSO, EU, DIR_OUT, LUNIT
      USE LOS_ANGLES,  ONLY: DirSun, N_BELOW, N_BE, IND_LIM_A, IND_LIM_B
      USE RAMAN,       ONLY: RRS_ADJ
      USE RTM_OUTPUT,  ONLY: MES_OUT_LS
      USE ST_VARS,     ONLY: R2D => RAD_TO_DEG,Er => EARTH_RADIUS,N_Eff,
     $                       M_LIM, Add_lev, DO_REFRAC, PIE, WF_FLAG,  
     $                       SHI_LIM_C, GeoD => GeoD_OUT
      USE SPHER_PARS,  ONLY: NN, N_OBS_ALT, N_LOS_ANG, NFA => N_FOV_ANG, 
     $                       ALT_OUT, LW_FOV, Crit_STA
      USE TOOLS,       ONLY: E_CSZA
      USE TYPES,       ONLY: KND, DCMP, PCC_TYPE, SWF_type, SLS_TYPE,
     $                       Eff_Sol_Idx
      USE WF_NUM,      ONLY: NUM_WF, NO_SOL, NO_LS, SOL_AND_LS, 
     $                       AL_BASE, AL_1_BASE, AL_1_PERT
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) ::  NLOS
      LOGICAL  IMS_FLG
      REAL (KND), INTENT(IN) ::  AL(NLAYERS)
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'APPR_SPHER_LS_MAIN'
c     ----------------------------------------------------------------
c     ... Spherical line-of-sight |
c     -----------------------------
      TYPE (SLS_type) :: SLS
c     ----------------------------------------------------------------
      TYPE (PCC_type) :: PCC    ! Photochemistry
      TYPE (SWF_type), TARGET :: SWF    ! "Spherical" weighting function

c     Local variables
c     ---------------
      INTEGER  I, J, Js, Ja, Jt, Jv, Jf, NLS, K1, K2, NTH, JL, Ji, K, 
     $     Up, Dn, Ups, Dns, L, Nt, Nf, IA, N_end, K_Jv, K_MIN, K_O, 
     $     NVA
      INTEGER  NN_LS(0:NLAYERS), K_pos(NFA*NLOS,N_Eff+1), 
     $     IOB_Z(NLAYERS)
      LOGICAL  TEST_APR, OUT, STATUS, REFR_FLAG(NLAYERS)
      CHARACTER (LEN=5)  NAM_OUT
      CHARACTER (LEN=4)  INT_TO_CHAR
      INTEGER, PARAMETER :: UNIT = 44
      REAL (KND), PARAMETER :: ACC_INV = 1.d-04
      REAL (KND) Crit, Ck, Dtp, Reld, E_max, SZA, DUM, 
     $     Dif(NN), Tp(NN), P_ec(NN), Res(NN), X(NN), 
     $     Mat1(NN,NN), A_VEC( (NN + 1)*(NN + 2)/2 ), Edi(NN,NN),
     $     S_UM(NN), Prel(NN), RHS(NN), Tra(NN,NN), Tinv(NN,NN)
      COMPLEX (kind(0.0d0)) :: Mu_co
      INTEGER, POINTER :: IOB(:)
      REAL (KND), TARGET :: T_SP(GeoD%MaxLS,GeoD%NVA)
      REAL (KND)  ALW(N_P-1), T_PP(N_P), Z_N(N_P),  
     $     AL_N(N_P), al_ln(nlayers), AL_Th(NLOS*NFA)
      REAL (KND), DIMENSION (NLAYERS) :: TH, COS_SOL, XM_1, PHI_LOS
      REAL (KND), DIMENSION (GeoD%MaxLS) :: H_EXP, AL_1, al_1_LN
c     ... Allocatable
      REAL (KND), ALLOCATABLE :: Apr_Exp(:,:), Path(:,:), DS_LOC(:,:)
c     ... Pointers
      INTEGER,    POINTER :: N_sph(:), IND_LOC(:)
      REAL (KND), POINTER :: Mue(:), SIN_COEF(:,:), 
     $     Eff_mu(:,:), Po_SP(:), ZLS(:), D_DS(:,:) 
      TYPE(Eff_Sol_Idx), POINTER :: IEffS(:,:)
c     ... Externals
      EXTERNAL  INT_TO_CHAR
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1
      Eff_mu => SLS%Eff_mu ;  IEffS => SWF%IEffS
c                  WRITE(*,*)  '  SHAPE(Eff_mu)  ',  SHAPE(Eff_mu)
c     The approximation of spherical transmission alog LoS can be 
c       tested if the flag "TEST_APR" is set to .TRUE.
c     -------------------------------------------------------------- 
      TEST_APR = .FALSE.
c      TEST_APR = .TRUE.

      Z_N(:) = Z_FINE(N_P:1:-1) ! Z_N is arranged from the top to the bottom
      CALL LIN_INT(NLAYERS, ZZZ, AL, N_P, Z_N, AL_N, 'Z')

      ALW(1:N_P-1)   = (AL_N(1:N_P-1) + AL_N(2:N_P))*0.5D0
c     ... The optical depth for the fine grid along vertical direction
c                               --------------
      T_PP(1) = 0.d0 
      DO  J = 1, N_P - 1
         T_PP(J+1) = T_PP(J) + ALW(J)*( Z_N(J)-Z_N(J+1) )
      ENDDO
c             write(*,*)  '  *** N_OBS_ALT(1) ', N_OBS_ALT(1)
c             write(*,*)  '  *** Add_lev      ', Add_lev
      IA = 1
      DO  Ji = 1, N_LOS_ANG ! Number of LoS angles 
         DO Jf = 1,NFA ! Number of directions within FOV
            Jv = (Ji - 1)*NFA + Jf
c           ... NTH is the number of point where Th is located 
c               or the number of last point if Th is not presented
            NTH   = GeoD%LVG(IA,Jv)%NTH
c           ... Number of points along l-of-s
            NLS   = GeoD%LVG(IA,Jv)%NLS
c           ... Altitude grid along selected LoS, starting from instrument
            ZLS  => GeoD%LVG(IA,Jv)%ZLS
c                   WRITE(*,'(A,*(1x,F8.3))') '   ===> ZLS = ', ZLS
            D_DS => GeoD%LVG(IA,Jv)%DS(:,:)
c           ... Interpolation on the grid along line-of-sight
c           -------------------------------------------------
            IF( NUM_WF ) THEN
               CALL LIN_INT(NLAYERS,ZZZ,AL_BASE,NLS,ZLS,AL_1_LN,'Z')
               AL_1_BASE(1:NLS) = AL_1_LN(1:NLS) ! it is no perturbed extinction
               CALL LIN_INT(NLAYERS, ZZZ, AL, NLS, ZLS, AL_1_LN,'Z')
               AL_1_PERT(1:NLS) = AL_1_LN(1:NLS) ! it is perturbed extinction
c               ... for calculation without contribution from solar exponent
               IF( NO_SOL) AL_1(1:NLS) = AL_1_LN(1:NLS)
c              ... for calculation without LS exponent
               IF( NO_LS ) AL_1(1:NLS) = AL_1_BASE(1:NLS)
               IF( SOL_AND_LS ) AL_1(1:NLS) = AL_1_PERT(1:NLS)
            ELSE
c              al_ln(:) = DLOG( AL(:) )
               al_ln(:) = AL(:)
               CALL LIN_INT(NLAYERS, ZZZ,AL_LN,NLS,ZLS,AL_1_LN,'Z')
c              AL_1(1:NLS) = DEXP( AL_1_LN(1:NLS) )
               AL_1(:) =  AL_1_LN(:)
c              ... Save extinction coefficient at the tangent height  
c                  or at the end of line-of-sight
               AL_Th(Jv) = AL_1_LN(NTH) 
            ENDIF
c           ... The optical depth along all LoS, starting from instrument
            T_SP(1,Jv) = 0.0D0
            DO I = 2, NLS
               T_SP(I,Jv) = T_SP(I-1,Jv) + AL_1(I-1)*D_DS(2,I-1)
     $              + AL_1(I)*D_DS(1,I) 
            END DO
c           ... Delate or make as a local array ??
            SLS%LSE(IA,Jv)%EX(:) = DEXP( -T_SP(1:NLS,Jv) )
c           ... Alternative version of integration along LoS
            SLS%LSE(IA,Jv)%EX1(0) = 1.0d0 ;  DUM = 0.0d0
            DO I = 1, NLS
               DUM = DUM + AL_1(I)*(D_DS(1,I) + D_DS(2,I)) 
               SLS%LSE(IA,Jv)%EX1(I) = DEXP(-DUM)
            END DO
         ENDDO   ! over all lS within FOV (Jf,Jv))
      ENDDO  ! over line-of-sight (Ji)

      IF( (.NOT.WF_FLAG).AND.(.NOT.RRS_ADJ) ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      DO Jv = 1, NFA*NLOS
         NLS = GeoD%LVG(IA,Jv)%NLS ; ZLS => GeoD%LVG(IA,Jv)%ZLS
         CALL LIN_INT(NLAYERS,ZZZ,AL, NLS,ZLS,AL_1,'Z')         
         CALL FIND_EFF_ANGLES
      ENDDO
c         DO I = 1, NFA*NLOS
c            WRITE(*,'(2x,I5,*(2x,e12.4))')
c     $           I, ( DACOS(Eff_mu(I,K))/Pie*180., K = 1,M_LIM(I))
c         ENDDO      
c         PAUSE
      IF( NN == 1 ) TEST_APR = .FALSE.      
      IF( TEST_APR ) ALLOCATE( Apr_Exp(GeoD%MaxLS,NLOS*NFA),
     $     Path(GeoD%MaxLS,NLOS*NFA) )

c     ... Accuracy of the spherical transmission function approximation
      Crit = Crit_STA/100.0d0
      
c     ... Approximation of Transmission along LoS is done for 
c         each direction within FoV
      NVA = GeoD%NVA
      DO Js = 1, NVA
         NLS = GeoD%LVG(IA,Js)%NLS  ;  NTH = GeoD%LVG(IA,Js)%NTH 
         ZLS  => GeoD%LVG(IA,Js)%ZLS
         IOB => GeoD%LVG(IA,Js)%IOB(:)
         IF( GeoD%LVG(IA,Js)%C_VA(1) > 0.d0 ) THEN
c        ... For downward direction of selected LoS
c        -----------------------------------------
c               write(*,'(10x,A)') ' *****  SIN_FIT_BEFOR_TH ********'
            CALL SET_PAR(1)  ;  CALL SIN_FIT_BEFOR_TH
            IF( NLS <= NTH ) CYCLE  ! surface before Th
c               write(*,'(10x,A)') ' *****  SIN_FIT_AFTER_TH ********'
            CALL SET_PAR(2)  ;  CALL SIN_FIT_AFTER_TH
         ELSE
c        ... For upnward direction of selected LoS 
c        -----------------------------------------
            IF( NLS==1 ) CYCLE  ! Instrument on the surface
c                write(*,'(10x,A)')
c     $          ' *****  SIN_FIT_AFTER_TH **** upnward direction of LoS'
            CALL SET_PAR(1)  ;  CALL SIN_FIT_AFTER_TH
c               write(*,*)
c     $              ' Number of expansion coefficients for layers:'
c               DO I = 1, NLAYERS-1
c                 WRITE(*,'(1x,F8.3,2x,I3)') ZZZ(I), SLS%N_sph_ls(Js,I,1)
c               ENDDO
         ENDIF
c              write(*,*)  '    '
      ENDDO  ! over all LoS and directions within FoV (Js)
      
c      IF( IMS_FLG.AND.(WF_FLAG.OR.RRS_ADJ)) THEN      
      IF( WF_FLAG.OR.RRS_ADJ ) THEN      
c        Effective solar zenith angle is defined 
c        for the central line-of-sight within the field-of-view only
         K1 = 0 
         DO Js = 1, GeoD%NSol
            DO Jv = 1, NLOS
c              ... Select single direction within FOV
               K_O = NFA*Jv - LW_FOV(Jv)
               DO Ja = 1, GeoD%NPhi
                  K1 = K1 + 1
                  SWF%Ief_Sol(K1) = M_LIM(K_O)
                  SWF%Ief_Sum(K1) = SHI_LIM_C(Jv)
                  DO I = 1,M_LIM(K_O)
                     IEffS(K1,I)%IA = IA ;  IEffS(K1,I)%IV = K_O
                     IEffS(K1,I)%IS = Js ;  IEffS(K1,I)%IP = Ja
                     IEffS(K1,I)%ILS = K_pos(K_O,I) ! ? Jv
                  ENDDO  ! over effective angles (I)
               ENDDO  ! over azimuth angles (Ja)
            ENDDO  ! over LoS angles (Jv)
         ENDDO  ! over solar angles (Js)
                 
c        Calculation of DS_SUN for effective solar zenith angles
c        -------------------------------------------------------
c         REFR_FLAG = .FALSE.    ! First without refraction      
         IOB_Z(:) = [(I, I = 1,NLAYERS)]
         CALL LIN_INT( NLAYERS, ZZZ, AL, N_P, Z_N, AL_N, 'Z' )
         XM_1   (:) = 1.0d0     ! Nadir-zenith line-of-sight
         PHI_LOS(:) = 0.0d0

c         WRITE(*,*) '  Eff sol angle used for calculations '      
c         WRITE(*,*) '  Number of LoS incl FOV : ', GeoD%NVA

         ALLOCATE(DS_LOC(NLAYERS,NLAYERS))
c         write(*,*) '      Sol_ang - '
         K1 = 0
         DO Js = 1, GeoD%NSol
            DO Jt = 1, NLOS
               Jv = Jt*NFA - LW_FOV(Jt)
c               WRITE(*,*) '   Select within FOV ', Jv
               DO Ja = 1, GeoD%NPhi
                  K1 = K1 + 1
                  DO JL = 1,M_LIM(Jv)
                     COS_SOL(:) = E_CSZA(IEffS(K1,JL),'G')
c                     write(*,*) '  -----> Js, K1, Jl ', Js, K1, Jl
c                     WRITE(*,*) DACOS(COS_SOL(1))/Pie*180.
                     TH(:) = (Er + ZZZ(:))*
     $                    SQRT(1.0D0 - COS_SOL(:)**2) - Er
                     IF(DO_REFRAC) THEN
                        CALL REFR_INTERFACE(NLAYERS, ZZZ, COS_SOL, TH, 
     $                       REFR_FLAG, *880)
                     ELSE
                        REFR_FLAG = .FALSE.
                     END IF
                     CALL DirSun (NLAYERS, COS_SOL, TH, ZZZ, IOB_Z, AL,
     $                    AL_N(N_P:1:-1), PCC, SWF%SOL_JL(:,K1,JL),
     $                    REFR_FLAG, .FALSE., .TRUE., DS_LOC)
                     SWF%DS_SUN_JL(:,:,K1,JL) = DS_LOC
!                     write(6,*) SWF%DS_SUN_JL(:,:,K1,JL)
                  ENDDO  ! over effective angles (JL)
               ENDDO  ! over azimuth angles (Ja)
            ENDDO
         ENDDO  ! over actual solar angles (Js)

         DEALLOCATE(DS_LOC)

         IF (LOG_LEVEL(1) >= N_DSO) THEN
            WRITE(LUNIT,*)
     $           ' *** Values of solar and LoS angles for layers:'
            CALL WRITE_SEL_ALL
         ENDIF
c         WRITE(*,*) ' ============================================= '      

      ENDIF  ! WF_FLAG.OR.RRS_ADJ

c      stop ' ***** In appr_spher_ls_main ******'
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
      CONTAINS
c *************************************************************************
      SUBROUTINE SET_PAR(II)
      INTEGER, INTENT(IN):: II
        N_sph    => SLS%N_sph_ls(Js,:,II)
        Mue      => SLS%Mue_ls(Js,:,II)
        SIN_COEF => SLS%SIN_COEF_LS(Js,:,:,II)
        IND_LOC  => SLS%IND_LS(:,Js,II)
      END SUBROUTINE SET_PAR

c *************************************************************************
      SUBROUTINE SIN_FIT_BEFOR_TH
      INTEGER       N_Start
      REAL (KND)    T_dn, T_up
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SIN_FIT_BEFOR_TH'

      CALL ST_TRACK (R_NAME, 1, 'I')

      N_Start = N_OBS_ALT(1)
      IF( Add_lev ) N_Start = N_Start - 1
      SIN_COEF = 0.0d0 ; N_sph = 0

c     ... N_below(Js) is level number of the standard altitude grid  
c          which is below or equal to the tangent height
      N_End = N_below(Js)
      DO I = N_Start, NLAYERS -1
         NN_LS(I) = COUNT(IOB(1:NTH) <= I + 1)
      ENDDO
      NN_LS(N_Start-1) = 1

c     ... Copy transmission along line-of-sight to the help-array
      H_EXP(1:NTH) = SLS%LSE(IA,Js)%EX(1:NTH)

c      write(*,'(A,2(2x,I3))') ' ******* start and end ', N_Start , N_End
c      write(*,*) '  Interval Zls  ', ZLS(NTH), ZLS(1)
c      write(*,*) '  Interval Tra  ', H_EXP(NTH), H_EXP(1)

c         ... Approximation for the layer containing the observational point
c         ------------------------------------------------------------------
      I = N_Start
      Up  = (I-1)*NN + 1 ; Dn = Up + NN 
      Ups = NN_LS(I-1)   ; Dns = NN_LS(I) ; IND_LOC(I) = Ups 

      T_dn = T_PP(Dn)    ; T_up = T_PP(Up)

      Up = Up + NN - (Dns - Ups)
      T_up = T_PP(Up) 
     $     + (AL_N(Up)+AL_1(1))*(Z_N(Up)-GeoD%Alt(IA))*0.50d0
      Nf = Dns - Ups - 1    
      Mue(I) = (T_dn - T_up)/(T_SP(Dns,Js) - T_SP(Ups,Js))
      IF( Nf == 0 ) THEN ; N_sph(I) = 1 ; GOTO 223 ; ENDIF 
      Tp(1:Nf) = H_EXP(Ups)*DEXP(-(T_PP(Up+1:Dn-1)-T_up)/Mue(I))
      P_ec(1:Nf) = DABS( 1.0D0 - Tp(1:Nf)/H_EXP(Ups+1:Dns-1) )
      Reld = MAXVAL( P_ec(1:Nf),1 )
      IF( Reld <= Crit ) Then ; N_sph(I) = 1 ; GOTO 223 ; ENDIF
      X(1:Nf) = (T_PP(Up+1:Dn-1)-T_up)/(T_dn - T_up)
      Dif(1:Nf) = (H_EXP(Ups+1:Dns-1) - Tp(1:Nf))/Tp(1:Nf)
      CALL MAIN_FIT(*223)
 223  CONTINUE

      DO I = N_Start+1, N_End-1 ! Over all other sub-layers
         Up  = (I-1)*NN + 1 ; Dn = Up + NN 
         Ups = NN_LS(I-1) ; Dns = NN_LS(I) 
         IND_LOC(I) = Ups 
         T_dn = T_PP(Dn)  ; T_up = T_PP(Up)
         IF( I == N_End-1 ) THEN
            Dn = Up + Dns - Ups  
            T_dn = T_PP(Dn-1) + (AL_N(Dn-1) + AL_Th(Js))
     $           *(Z_N(Dn-1) - GeoD%LVG(IA,Js)%Th_V)*0.50d0
c               WRITE(*,*) '  Recalculated Tau at T_H point '
c               WRITE(*,'(3(10x,f7.3))') Z_N(Dn-1), Th_V(Js), Z_N(Dn)
c               WRITE(*,'(3(2x,e15.7))') T_PP(Dn-1), T_dn, T_pp(Dn)
c               WRITE(*,'(3(10x,f7.3))') ZLS(Dns-1), ZLS(Dns)
         ENDIF
         Nf = Dns - Ups - 1     ! Number of points for fit
         IF( H_EXP(Dns) <= 1.0D-30 ) THEN
            N_sph(I) = 1  ;  Mue(I) = 1.0d0  ;  CYCLE
         ENDIF
         Mue(I) = (T_dn - T_up)/(T_SP(Dns,Js) - T_SP(Ups,Js))
         IF( Nf == 0 ) THEN ; N_sph(I) = 1 ; CYCLE ; ENDIF

         Tp(1:Nf) = H_EXP(Ups)*
     $           DEXP(-(T_PP(Up+1:Dn-1)-T_up)/Mue(I))

         P_ec(1:Nf) = DABS( 1.0D0 - Tp(1:Nf)/H_EXP(Ups+1:Dns-1) )
         Reld = MAXVAL( P_ec(1:Nf),1 )

         IF( Reld <= Crit ) Then ; N_sph(I) = 1 ; CYCLE ; ENDIF
         X(1:Nf) = (T_PP(Up+1:Dn-1)-T_up)/(T_dn - T_up)
         Dif(1:Nf) = (H_EXP(Ups+1:Dns-1) - Tp(1:Nf))/Tp(1:Nf)

         CALL MAIN_FIT(*222)
 222     CONTINUE
      ENDDO  ! over sublayers (I)
      IND_LOC(N_End) = Dns      ! ??

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      END SUBROUTINE SIN_FIT_BEFOR_TH  

c *************************************************************************
      SUBROUTINE SIN_FIT_AFTER_TH
      INTEGER       S1, S2, NN_f, N_DIM, N_Start
      REAL (KND)    T_dn
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SIN_FIT_AFTER_TH'

      CALL ST_TRACK (R_NAME, 1, 'I')
 
      SIN_COEF = 0.0d0 ; N_sph = 0 
c     ... The spherical transmission function will be approximated for all
c         l-of-s within field-of-view of instrument. The approximation 
c         is performed layer-by-layer starting from the top of atmosphere.
      N_Start = 1
c      WRITE(*,*) '   NLS and NTH ', NLS, NTH
      IF( NLS > NTH ) THEN      ! Tangent height on the line-of-sight
c        ... N_below(Js) is level number of the standard altitude grid  
c            which is below or equal to the tangent height
         S1 = NLS ; S2 = NTH ; N_end = N_below(Js) 
         N_DIM = S1 - S2 + 1
      ELSE
c       ... if LoS directed downwards: N_below(Js) = NLAYERS ;
c           if LoS directed upwards:   N_below(Js) =  
c               level number of the standard altitude grid which 
c               is below or equal to the altitude of observer position
         S1 = NTH ; S2 = 1  ; N_end = N_below(Js) 
         N_DIM = NTH
      ENDIF
c     ... Copy 
      H_EXP(1:N_DIM) = SLS%LSE(IA,Js)%EX(S1:S2:-1)

c      write(*,'(A,2(2x,I3))') ' ******* Start and End levels ',
c     $     N_Start , N_End - 1
c      write(*,*) '  TH_FLG & N_DIM ', GeoD%LVG(IA,Js)%TH_FLG, N_DIM
c      write(*,*) '  Interval Zls   ', ZLS(S1), ZLS(S2)
c      write(*,*) '  Interval Tra   ', H_EXP(1), H_EXP(N_DIM)

      Po_SP => T_SP(S1:S2:-1,Js)
      DO I = 1, NLAYERS
         NN_LS(I) = COUNT( IOB(S2:S1) == I )
      ENDDO
c      write(*,*) '              IOB(S2:S1)  ', IOB(S2:S1) 
c      write(*,*) ' -------------------------------------------------'
c      write(*,*) '              NN_LS(1:NLAYERS)  ', NN_LS(1:NLAYERS)
      DO I = 1, N_end - 1         
         NN_f = SUM( NN_LS(1:I) )
         Up =  (I-1)*NN + 1 ; Dn = Up + NN 
         Ups = NN_f         ; Dns = Ups + NN_LS(I+1)
         IND_LOC(I) = NLS - Ups + 1
         Nf = Dns - Ups - 1     ! Number of points for fit in the layer
c         write(*,'(A,I3,2x,F8.3,4x,F8.3,A,F8.3)') '  I, ZZZ layer top ', 
c     $        I, ZZZ(I), ZLS(NLS-Dns+1),' - ' ,ZLS(NLS-Ups+1)
         IF( H_EXP(Dns) <= 1.0D-30 ) THEN
            N_sph(I)  = 1
            Mue(I)    = 1.0d0
            CYCLE
         ENDIF
         T_dn = T_PP(Dn)
cc         IF( I == N_end-2 ) THEN
c         IF( I > 88 ) THEN
c            Mue(I) = 1.74598d-04
c            N_sph(I) = 1
c            CYCLE
c         ENDIF
         IF( I == N_end-1 ) THEN
!            write(*,*) '   T_dn 1: ', T_dn 
!            write(*,*) ' I == N_end-1 ', GeoD%LVG(IA,Js)%Th_V
            Dn = Up + NN_LS(I+1)
            T_dn = T_PP(Dn-1) + (Z_N(Dn-1) - GeoD%LVG(IA,Js)%Th_V)
     $           *(T_PP(Dn-1)-T_PP(Dn))/(Z_N(Dn)-Z_N(Dn-1))
!            write(*,*) '   T_dn 2: ', T_dn 
!            write(*,*) '        T_dn - T_PP(Up) ', T_dn - T_PP(Up)
!            write(*,*) '  Po_SP(Dns)-Po_SP(Ups) ', Po_SP(Dns)-Po_SP(Ups)
!            write(*,*) '    Mue(I)',
!     $           -(T_dn - T_PP(Up))/(Po_SP(Dns) - Po_SP(Ups))
c            Mue(I) = -(T_dn - T_PP(Up))/(Po_SP(Dns)-Po_SP(Ups))
c            Mue(I) = 1.74598d-04
c            N_sph(I) = 1
c            CYCLE
         ENDIF
         
         Mue(I) = -(T_dn - T_PP(Up))/(Po_SP(Dns)-Po_SP(Ups))
         IF( Nf == 0 ) THEN 
            N_sph(I) = 1 ; CYCLE
         ENDIF
         Tp(1:Nf) = H_EXP(Dns)*
     $        DEXP(-(T_dn - T_PP(Up+1:Dn-1))/Mue(I))
         P_ec(1:Nf) = DABS( 1.0D0 - Tp(1:Nf)/H_EXP(Ups+1:Dns-1) )
         Reld = MAXVAL( P_ec(1:Nf),1 )
         IF( Reld <= Crit ) Then ; N_sph(I) = 1 ; CYCLE ; ENDIF
         X(1:Nf) = (T_dn - T_PP(Up+1:Dn-1))/(T_dn - T_PP(Up))
         Dif(1:Nf) = (H_EXP(Ups+1:Dns-1) - Tp(1:Nf))/Tp(1:Nf)
         CALL MAIN_FIT(*222)
 222     CONTINUE
      ENDDO  ! over sublayers (I)
      IND_LOC(N_end) = NLS - Dns + 1 ! Proverit' dly Th
         
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
         
      END SUBROUTINE SIN_FIT_AFTER_TH      

c ******************************************************************
      SUBROUTINE MAIN_FIT(*)
        Nt = 1 ; OUT = .FALSE. ; STATUS = .FALSE.
c        write(*,*) ' ****** Sublayer ',I, ZZZ(I), Mue(I)
 21     Continue
        Mat1(Nt,1:Nf) = DSIN( DBLE(Nt)*X(1:Nf)*Pie )
        TRA(1:Nt,1:Nt) = 
     $       MATMUL( MAT1(1:Nt,1:Nf),TRANSPOSE(MAT1(1:Nt,1:Nf)) )
        FORALL (K = 1:Nt) 
     $       A_VEC(K*(K - 1)/2 + 1:K*(K + 1)/2) = TRA(K,1:K)
        CALL CHINV_LOC (Nt*(Nt+1)/2, Nt, A_VEC, STATUS)
        IF( STATUS ) THEN
           OUT = .TRUE.
           Nt = Nt-1
           GOTO 123
        ENDIF
        FORALL (K = 1:Nt) 
     $       Tinv(K,1:K) = A_VEC(K*(K - 1)/2 +1:K*(K + 1)/2)
        FORALL (K = 1:Nt, L = 1:Nt, L > K) 
     $       Tinv(K,L) = Tinv(L,K)

c       ... Precision of inverse matrix:
        Edi = 0.d0
        Edi(1:Nt,1:Nt) = 
     $       MATMUL( Tinv(1:Nt,1:Nt),TRA(1:Nt,1:Nt) )
        FORALL( K = 1:Nt ) Edi(K,K) = Edi(K,K) - 1.d0
        E_max = MAXVAL( DABS(EDI) )
c       ... If required accuracy of inverse matrix calculation
c           does not achieved
        IF( E_max > ACC_INV ) THEN
           OUT = .TRUE. ; Nt = Nt-1 ; GOTO 123
        ENDIF

        RHS(1:Nt) = MATMUL( MAT1(1:Nt,1:Nf),Dif(1:Nf) )
        ReS(1:Nt) = MATMUL( Tinv(1:Nt,1:Nt),RHS(1:Nt) )
        
 123    FORALL( K = 1:Nf )
     $       S_UM(K) = SUM( RES(1:Nt)*MAT1(1:Nt,K) )

        Prel(1:Nf) = Tp(1:Nf)*(1.0D0 + S_UM(1:Nf))
        P_ec(1:Nf) = DABS(1.0D0 - Prel(1:Nf)/H_EXP(Ups+1:Dns-1))
        
        Reld = MAXVAL( P_ec(1:Nf),1 )
c            write(*,*) '     Nt, Reld - 1', Nt, Reld, Crit
c            write(*,*) ' Dif = ', Dif(1:Nf)
        IF( OUT ) Then
           SIN_COEF(I,1:Nt) = RES(1:Nt)
           N_sph(I) = Nt+1
           RETURN 1
        ENDIF
        IF( Reld <= Crit ) Then
           SIN_COEF(I,1:Nt) = RES(1:Nt)
           N_sph(I) = Nt+1
           RETURN 1
        ENDIF
        IF( Nt >= Nf-1 ) Then
           SIN_COEF(I,1:Nt) = RES(1:Nt)
           N_sph(I) = Nt+1
c           write(*,*) '*******... Nt >= N_max ...********'
c           WRITE(*,*) ' - ',ZZZ(I),' - ',ZZZ(I+1)
           IF( MES_OUT_LS ) THEN
              CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $             'Selected accuracy for transmission approximation '
     $             //'along line-of-sight is not reached: ')
              WRITE(EU,*) ' requested accuracy ', Crit
              WRITE(EU,*) '  obtained accuracy ', Reld
              MES_OUT_LS = .FALSE.
           ENDIF
           RETURN 1
        ELSE
           Nt = Nt + 1
           GOTO 21
        ENDIF
      END SUBROUTINE MAIN_FIT

c *************************************************************************
c     The effective LoS zenith angle is defined for all line-of-sight 
c                     within FOV 
c***************************************************************************
      SUBROUTINE FIND_EFF_ANGLES
      INTEGER NLS

c      WRITE(*,*) '     Call "FIND_EFF_ANGLES" Jv = ', Jv
      NLS = GeoD%LVG(IA,Jv)%NLS
      H_EXP(1:NLS) = SLS%LSE(IA,Jv)%EX(:)*AL_1(1:NLS)
      
      IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN
c         write(*,*) '   ------> LoS downwards ', GeoD%LVG(IA,Jv)%C_VA(1) 
         DO I = 1,N_BE(Jv)
            K1 = IND_LIM_B(Jv,I) ; K2 = IND_LIM_B(Jv,I+1)
            K_min = MAXLOC( H_EXP(K1:K2),1 ) + K1 - 1
            K_pos(Jv,I) = K_min
            Eff_Mu(Jv,I) = GeoD%LVG(IA,Jv)%C_VA(K_MIN)
c            WRITE(*,'(3(2x,f7.3),4x,es15.7 )') 
c     $           ZLS(K1), ZLS(K_min), ZLS(K2), Eff_Mu(Jv,I)
         ENDDO
      ELSE
c         write(*,*) '    ------> LoS upwards ', GeoD%LVG(IA,Jv)%C_VA(1) 
         DO I = 1,N_BE(Jv)
            K2 = IND_LIM_B(Jv,I) ; K1 = IND_LIM_B(Jv,I+1)
            K_min = MAXLOC( H_EXP(K1:K2),1 ) + K1 - 1
c              WRITE(*,'(3(5x,f7.3))') ZLS(K1),ZLS(K_min),ZLS(K2)
            K_pos(Jv,I) = K_min
            Eff_Mu(Jv,I) = GeoD%LVG(IA,Jv)%C_VA(K_MIN)
c              WRITE(*,*) I, DACOS(Eff_Mu(Jv,I))*R2D
         ENDDO
      ENDIF
      K = 1
c      IF(  N_Be(Jv)+1 < M_LIM(Jv) ) THEN
c         write(*,*) '     Layers after Th '
c      ENDIF
      DO I = N_Be(Jv)+1, M_LIM(Jv)
         K2 = IND_LIM_A(Jv,K) ; K1 = IND_LIM_A(Jv,K+1)               
         K_min = MAXLOC( H_EXP(K1:K2),1 ) + K1 - 1
         K_pos(Jv,I) = K_min
         Eff_Mu(Jv,I) = GeoD%LVG(IA,Jv)%C_VA(K_MIN)
         K = K + 1
c         WRITE(*,'(3(2x,f7.3),4x,es15.7)') 
c     $        ZLS(K1),ZLS(K_min),ZLS(K2),Eff_Mu(Jv,I)
      ENDDO
      END SUBROUTINE FIND_EFF_ANGLES

C***********************************************************************
C*                                                                     *
C*               SUBROUTINE FOR MATRIX INVERSION                       *
C*                    CRAY optimized version                           *
C*                                                                     *
C***********************************************************************
      SUBROUTINE CHINV_LOC(NVEC, N, A, STATUS)

      INTEGER       NVEC, N, K, I, J
      LOGICAL       STATUS
      REAL (KND)    A(NVEC), X

      A(1) = 1.0D0/DSQRT(A(1))
      FORALL (I = 2:N) A(I*(I - 1)/2 + 1) = A(I*(I - 1)/2 + 1)*A(1)

      DO I = 2, N
         DO J = 2, I - 1
            X = A(I*(I - 1)/2 + J)
            DO K = 1, J - 1
               X = X - A(I*(I - 1)/2 + K)*A(J*(J - 1)/2 + K)
            END DO
            A(I*(I - 1)/2 + J)=X*A(J*(J - 1)/2 + J)
         END DO
         X = A(I*(I + 1)/2)
         DO K = I*(I - 1)/2 + 1, I*(I + 1)/2  - 1
            X = X - A(K)*A(K)
         END DO
         IF( X <= 0.0d0 ) THEN
            WRITE(*,*) 'X <= 0.0d0', X
            STATUS = .TRUE.
            RETURN
         ENDIF
         A(I*(I + 1)/2) = 1.0D0/DSQRT(X)
      END DO

C      INVERS L MATRIX
      DO I= 2, N
         DO J = 2, I
            X = 0
            DO  K = 1, I - J + 1
               X = X - A(I*(I + 1)/2 - K)*
     $              A(J - 1 + (I - K - 1)*(I - K)/2)
            END DO
            A(I*(I - 1)/2 + J - 1)=A(I*(I + 1)/2)*X
         END DO
      END DO

C      INVERS A MATRIX
      DO I= 1, N
         DO J = 1, I
            X = 0
            DO  K = I, N
               X = X + A(K*(K - 1)/2 + I)*A(K*(K - 1)/2 + J)
            END DO
            A(I*(I - 1)/2 + J) = X
         END DO
      END DO

      RETURN
      END SUBROUTINE CHINV_LOC
C  *******************************************
      SUBROUTINE WRITE_SEL_ALL
      INTEGER          KK, K_Jv
       WRITE(LUNIT,*)
     $      '  --------                             ------------ '
       WRITE(LUNIT,*)
     $      '  --- Eff LoS angles for selected dir. in FOV   -------- '
       WRITE(LUNIT,*)
     $      '  --------                             ------------ '
       DO K_Jv = 1,NLOS
          Jv = K_Jv*NFA - LW_FOV(K_Jv)
          WRITE(LUNIT,*) ' Line-of-sight  : ',K_Jv
          DO I = 1,N_Be(Jv)
             WRITE(LUNIT,500) 
     $            I, GeoD%LVG(IA,Jv)%ZLS(IND_LIM_B(Jv,I)),' -',
     $            GeoD%LVG(IA,Jv)%ZLS(IND_LIM_B(Jv,I+1)),' at',
     $            GeoD%LVG(IA,Jv)%ZLS(K_pos(Jv,I)),
     $            DACOS(Eff_Mu(Jv,I))*R2D
          ENDDO
          K = 1
          WRITE(LUNIT,*) 
     $               '    ======================================== '
          DO I = N_Be(Jv)+1,M_LIM(Jv)
             WRITE(LUNIT,500) I, 
     $            GeoD%LVG(IA,Jv)%ZLS(IND_LIM_A(Jv,K+1)),' -',
     $            GeoD%LVG(IA,Jv)%ZLS(IND_LIM_A(Jv,K)),' at',
     $            GeoD%LVG(IA,Jv)%ZLS(K_pos(Jv,I)),
     $            DACOS(Eff_Mu(Jv,I))*R2D
             K = K + 1
          ENDDO               
       ENDDO
c      Eff solar zenith angle for the line-of-sight having min. Th
       WRITE(LUNIT,*)
     $      '  --------                             ------------ '
       WRITE(LUNIT,*)
     $      '  --------       Eff solar angles      ------------ '
       WRITE(LUNIT,*)
     $      '  --------                             ------------ '
       KK = 0
       DO Jv = 1,NLOS
          K_O = Jv*NFA - LW_FOV(Jv) 
          DO Ja = 1,GeoD%NPhi ; KK = KK + 1
             DO Js = 1,GeoD%NSol
                WRITE(LUNIT,*) '         Solar  : ',Js
                WRITE(LUNIT,*) ' Line-of-sight  : ',Jv
                WRITE(LUNIT,*) '       Azimuth  : ',Ja
                K1 = GeoD%NPhi*NLOS*(Js-1) + KK
                DO I = 1,N_Be(K_O)
                   SZA = ACOS(E_CSZA(IEffS(K1,I),'G'))*R2D
                   WRITE(*,500) 
     $                  I, GeoD%LVG(IA,K_O)%ZLS(IND_LIM_B(K_O,I)),' -',
     $                  GeoD%LVG(IA,K_O)%ZLS(IND_LIM_B(K_O,I+1)),' at',
     $                  GeoD%LVG(IA,K_O)%ZLS(K_pos(K_O,I)), SZA
                ENDDO
                K = 1
                WRITE(LUNIT,*) 
     $               '    ======================================== '
                DO I = N_Be(K_O)+1,M_LIM(K_O)
                   SZA = ACOS(E_CSZA(IEffS(K1,I),'G'))*R2D
                   WRITE(LUNIT,500) I, 
     $                  GeoD%LVG(IA,K_O)%ZLS(IND_LIM_A(K_O,K+1)),' -',
     $                  GeoD%LVG(IA,K_O)%ZLS(IND_LIM_A(K_O,K)),' at',
     $                  GeoD%LVG(IA,K_O)%ZLS(K_pos(K_O,I)), SZA
                   K = K + 1
                ENDDO               
             ENDDO
          ENDDO
       ENDDO
 500   FORMAT( 2x,I3,2x,f7.3,A2,f7.3,A3,f7.3,3(2x,f7.3) )
      END SUBROUTINE WRITE_SEL_ALL
      END
