C***********************************************************************
C                                                                      *
C       SCIATRAN - Version 3.8 - Last modification: 20 Feb 2017        *
C                                                                      *
C       Copyright (C) - Institute of Remote Sensing,                   *
C                       University of Bremen, Germany                  *
C                                                                      *
C       Approximation of the spherical transmittance with a sum        *
C       of exponents (see Sec. Spher-appr in STB)                      *
C***********************************************************************
      SUBROUTINE APPR_SPHER_TRANS(NSZA, LPOS, JJESFT, AL, POS, SSB, SWF, 
     $     *)
c     ... Called from "dom_spherical.f"
      USE CLOUDS,      ONLY: DO_CLOUD, NCL => N_CLD_LAYERS, J_TOP, 
     $                       CLD_LAYER_NUM
      USE GRIDS,       ONLY: NL => NLAYERS, ZZZ, I_IDX => INTERP_IDX,
     $                       N_P => NL_FINE, SOL_GRD, Z_FINE
      USE LOG_PARS,    ONLY: EU, DIR_OUT
      USE LOS_Angles,  ONLY: DirSun
      USE PHOTOCHEM,   ONLY: N_SOL_PC
      USE RTM_OUTPUT,  ONLY: MES_OUT_SOL
      USE ST_VARS,     ONLY: PreCalcAngs, PIE, RTM_TYPE, N_Eff, WF_FLAG,
     $                       GeoD => GeoD_PS, FLG_MS
      USE SPHER_PARS,  ONLY: NN, Check_STA, Crit_STA
      USE TYPES,       ONLY: KND, DCMP, SSB_TYPE, PCC_TYPE, SWF_type
c        For test with numerical perturbations
      USE WF_NUM,      ONLY: NUM_WF
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) ::  NSZA, LPOS, JJESFT
      REAL (KND), INTENT(IN) ::  POS, AL(NL)
      TYPE (SSB_type) :: SSB    ! Spherical solar beam 
      TYPE (SWF_type) :: SWF    ! "Spherical" weighting function 
      TYPE (PCC_type) :: PCC    ! Photochemistry
c     ---------------------------------------------------------------- 
c     Local variables
c     ---------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'APPR_SPHER_TRANS'
      INTEGER     I, J, K, Js, Up, Dn, Nsh_loc(NSZA)
      LOGICAL     DO_DS
      REAL (KND), ALLOCATABLE ::  DS_SUN(:,:)
      REAL (KND), DIMENSION (N_P) :: 
     $     COS_SOL, TH, XM_1, PHI_LOS, AL_N, AL_1_LN, Z_N
      REAL (KND) AL_LN(NL), COS_MIN
      INTEGER, POINTER ::   N_sha(:), N_sph(:,:)
      REAL (KND), POINTER :: Mue(:,:), SIN_COEF(:,:,:), Tr_out(:,:), 
     $     T_PP(:), SOL_EXP(:,:)
c     ==================================================================
      CALL ST_TRACK (R_NAME, 1, 'I') 

      COS_MIN = DCOS( 89.999d0/180.d0*Pie )

      N_sha => SSB%N_sha  ;  N_sph => SSB%N_sph
      Mue => SSB%Mue      ;  SIN_COEF => SSB%SIN_COEF
      Tr_out => SSB%EXS

      T_PP => SWF%T_fine  ;  SOL_EXP => SWF%SOL_EXP 
      DO_DS = .FALSE.
      IF (RTM_TYPE /= 'spher_scat' .AND. WF_FLAG ) DO_DS = .TRUE.

c     Refraction
c     ***********
      Z_N(:) = Z_FINE(N_P:1:-1) ! from the top to the bottom

c        For test with numerical perturbations
c          no contribution from solar exponent
      IF( NUM_WF ) THEN
c         CALL LIN_INT(NL, ZZZ, AL_BASE, N_P, Z_N, AL_N , 'Z')
c          .... Variation for sol. exp only
c          AL_LN(:) = AL_1_PERT(1:NL) !AL_1_PERT is changed in _ls_main !!
          AL_LN(:) = AL(1:NL)
          CALL LIN_INT(NL, ZZZ, AL_LN, N_P, Z_N, AL_1_LN, 'Z')
          AL_N(:) = AL_1_LN(:)
      ELSE
c         AL_LN(:) = DLOG( AL(:) )
          AL_LN(:) = AL(:)
          CALL LIN_INT(NL, ZZZ, AL_LN, N_P, Z_N, AL_1_LN, 'Z')
c          AL_N(:) = DEXP( AL_1_LN(:) )
          AL_N(:) = AL_1_LN(:)
      ENDIF

      AL_1_LN(1:N_P) = AL_N(N_P:1:-1)

      XM_1   (:) = 1.d0        ! Nadir-zenith line-of-sight
      PHI_LOS(:) = 0.d0

      IF( DO_DS ) ALLOCATE(DS_SUN(N_P,NL))
      DO I = 1,NSZA
         ASSOCIATE (LVG => GeoD%LVG(1,1), LSG => GeoD%LSG(1,1,I,1))
         CALL DirSun (N_P, LSG%C_SZA, LSG%TH, LVG%ZLS, LVG%IOB, AL_N,
     $        AL_1_LN, PCC, SOL_EXP(:,I), LSG%REFR_DSB, PreCalcAngs, 
     $        DO_DS, DS_SUN, LSG%DSB)

           IF( DO_DS ) SWF%DS_SUN_JL(:,:,I,1) = DS_SUN(:,:)
            
            Nsh_loc(I) = N_P
            DO J = 1, N_P
               IF(LSG%C_SZA(J) < 0.d0 .AND. LSG%TH(J) < 0.d0) THEN
                  Nsh_loc(I) = J-1  ;  EXIT
               ENDIF
            END DO
         END ASSOCIATE
      ENDDO                     ! over solar angles (I)
      IF( DO_DS ) DEALLOCATE(DS_SUN)

      CALL SIN_FIT

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c *************************************************************************
      SUBROUTINE SIN_FIT
      INTEGER  I, J, K, L, Nt, Nf, II, KK, IDX_SE!, Zn
      INTEGER, PARAMETER :: UNIT = 44
      LOGICAL  OUT, STATUS
      REAL (KND), PARAMETER :: ACC_INV = 1.d-04
      REAL (KND)  Crit, Ck, Dtp, Reld, DD, E_max, Pre_AC
      REAL (KND)  T_SP(N_P), ALW(N_P-1), DZR(N_P), 
     $     Dif(NN), Tp(NN), P_ec(NN), Res(NN), X(NN), Mat1(NN,NN),
     $     S_UM(NN), Prel(NN), A_VEC( (NN + 1)*(NN + 2)/2 ), 
     $     Tra(NN,NN), Tinv(NN,NN), RHS(NN), EDI(NN,NN)
      REAL (KND), ALLOCATABLE :: E_COM(:,:)
      COMPLEX (kind(0.0d0)) :: Mu_co
c       Arrays for integral test
c      REAL (KND) Muv, Muef, Ex1, Ex2, Sint(NL-1), Sint_ap(NL-1),
c     $     Cint(NL-1), Cint_ap(NL-1)      

      DZR (N_P) = 0.D0
      DZR(1:N_P - 1) = Z_N(1:N_P - 1) - Z_N(2:N_P)
      ALW(1:N_P-1) = (AL_N(1:N_P-1) + AL_N(2:N_P))*0.5D0
      T_PP(1) = 0.d0 
      DO  J = 1, N_P - 1
         T_PP(J+1) = T_PP(J) + ALW(J)*DZR(J)
      ENDDO
c      WRITE(*,*) ' ===> TAU_spher :', T_PP(N_P)

c     ... Accuracy of the spherical transmission function approximation
      Crit = Crit_STA/100.d0
c     ... Number of terms used in approximation of spherical transmission
c         function (see Eq.(Ta-sph) in STB)
      N_sph = 0
      SIN_COEF = 0.d0 

      DO Js = 1,NSZA
c                      WRITE(*,*) ' Sol angle: ', Js
c                      WRITE(*,*) ' Z_shadow = ', Z_N(Nsh_loc(Js))
         Tr_out(NL,Js) = SOL_EXP(N_P,Js) ; N_sha(Js) = NL
         DO I = 1, NL-1  ! Over all layers of Standard Altitude Grid (SAG)
c                       WRITE(*,'(A,e12.4,A,e12.4)') '   Layer -- ', 
c     $                      ZZZ(I),' - ',ZZZ(I+1)
            Up =  (I-1)*NN + 1; Dn = Up + NN 
c           ... Check if the lower level of I-th SAG layer in shadow
            IF( Dn > Nsh_loc(Js) ) Then
               N_sha(Js) = I
c                      Write(*,*) 'Z_shadow of SAG ', Js, ZZZ(N_sha(Js))
               EXIT
            ENDIF
c           ... Nf+1 is the number of fine grid layers within SAG layer
c               Nf is the number of points that can be used for fitting
            Nf = Dn - Up - 1            
c           ... Spherical transmission at the top of SAG layer is too small  
c               to perform approximation

            IF( SOL_EXP(Up,Js) <= 1.d-20 ) THEN
c                 WRITE(*,*) '  Sol_exp at "up" < 1.d-20 ---> cycle', I-1
               N_sph(Js,I) = 1 ; Tr_out(I,Js) = 0.d0 
               IF( I-1 > 0 ) THEN  ! Compare 1 
                  Mue(Js,I) = Mue(Js,I-1)
               ELSE
                  Mue(Js,I) = DABS(SOL_GRD(Js))
               ENDIF
               CYCLE
            ENDIF

c           ... Save solar exponent at the top of SAG layer
            Tr_out(I,Js) = SOL_EXP(Up,Js)
        
c           ... The layer is optically very thick. No approximation
c               can be employed. Geometrical SZA is used.
            IF( SOL_EXP(Up,Js)==0.d0.OR.SOL_EXP(Dn,Js) < 1.d-60 ) THEN
               Mue(Js,I) = DMAX1( DABS(SOL_GRD(Js)),COS_MIN )
c                WRITE(*,*) '  ---> No approximation can be employed: '//
c     $                     'Solar cosine is used instead  ' 
               N_sph(Js,I) = 1 ; CYCLE
            ENDIF

c           ... Average secant approximation Eq.(Eff-cos) in STB. 
            T_SP(Up:Dn) = -DLOG( SOL_EXP(Up:Dn,Js) )
            Mue(Js,I) = (T_PP(Dn) - T_PP(Up))/(T_SP(Dn) - T_SP(Up))

c           ... There are no sub-layers to perform approximation.
c               Average secant approximation will be used.
            IF( Nf == 0 ) THEN 
               N_sph(Js,I) = 1  ;  CYCLE
            ENDIF

c           ... Spherical transmission within SAG layer
c               using effective cosine term only
            Tp(1:Nf) = SOL_EXP(Up,Js)*
     $           DEXP( -(T_PP(Up+1:Dn-1)-T_PP(Up))/Mue(Js,I) )

c           ... Relative approximation error
            P_ec(1:Nf) = DABS( 1.d0 - Tp(1:Nf)/SOL_EXP(Up+1:Dn-1,Js) )

            Reld = MAXVAL( P_ec(1:Nf),1 ) 
            Pre_AC = Reld
            IF( Reld <= Crit ) Then
               N_sph(Js,I) = 1 
c              ... The average secant approximation provides required
c                   accuracy within I-th SAG layer
c                   WRITE(*,'(A,1x,E12.4,1x,A,I3,2x,E12.4)') 
c     $              ' --> Average secant', Reld*100.,' % ', 
c     $              N_sph(Js,I), SOL_EXP(Dn,Js)
c                   WRITE(*,*)  Mue(Js,I), SOL_GRD(Js)
               CYCLE
            ENDIF

            KK = Up+1
            DO K = 1,Nf 
               IF( Tp(K) > 0.d0 ) THEN
                 Dif(K) = SOL_EXP(KK,Js)/Tp(K) - 1.d0
              ELSE
                 Dif(K) = 0.d0
              ENDIF
              KK = KK + 1
            ENDDO
c                   IF( I==46.and.Js==16 ) THEN
c                   IF( ZZZ(I) < 6.d0 ) THEN
c                      WRITE(*,*) '  For Dif ', ZZZ(I), Js
c                      WRITE(*,*) '  SOL_EXP at top ', SOL_EXP(Up,Js)
c                      KK = Up+1
c                      DO K = 1, Nf
c                         WRITE(*,'(1x,i3,3(1x,e12.4))')  
c     $                        K, SOL_EXP(KK,Js), Tp(K), Dif(K)
c                         KK = KK + 1
c                      ENDDO
c                      WRITE(*,*) '  SOL_EXP at bot ', SOL_EXP(Dn,Js)
c                   ENDIF
            Nt = 1 ; OUT = .FALSE. ; STATUS = .FALSE.            

c           ... Argument x (without Pi) in Eq.(Ta-sin) (see STB)
            X(1:Nf) = (T_PP(Up+1:Dn-1)-T_PP(Up))/(T_PP(Dn) - T_PP(Up))
c           ... Sequential calculation of coefficients ak 
c               (see Eq.(Ta-sin) in STB)
 21         Continue
c           ... The term sin( kx ) in Eq.(Ta-sin) 
            Mat1(Nt,1:Nf) = DSIN( DBLE(Nt)*X(1:Nf)*Pie )

            TRA(1:Nt,1:Nt) = 
     $           MATMUL( MAT1(1:Nt,1:Nf),TRANSPOSE(MAT1(1:Nt,1:Nf)) )
            FORALL (K = 1:Nt) 
     $           A_VEC(K*(K - 1)/2 + 1:K*(K + 1)/2) = TRA(K,1:K)
            CALL CHINV_LOC (Nt*(Nt+1)/2, Nt, A_VEC, STATUS)
            IF( STATUS ) THEN
c              ... The inverse matrix cannot be calculated
c               WRITE(*,*) '-------->> No inversion ', SOL_EXP(Dn,Js)
               OUT = .TRUE. ;  Nt = Nt-1  ;  GOTO 123
            ENDIF
            FORALL (K = 1:Nt) 
     $           Tinv(K,1:K) = A_VEC(K*(K - 1)/2 +1:K*(K + 1)/2)
            FORALL (K = 1:Nt, L = 1:Nt, L > K) Tinv(K,L) = Tinv(L,K)

c           ... Check if inverse matrix is properly calculated
            Edi = 0.d0
            Edi(1:Nt,1:Nt) = 
     $           MATMUL( Tinv(1:Nt,1:Nt),TRA(1:Nt,1:Nt) )
            FORALL( K = 1:Nt ) Edi(K,K) = Edi(K,K) - 1.d0
            E_max = MAXVAL( DABS(EDI) )
            IF( E_max > ACC_INV ) THEN
c              ... The accuracy of inverse matrix calculation
c                  does not satisfy the accuracy-criteria
c                    WRITE(*,*) '-------->> Inversion is inaccurate', 
c     $                      SOL_EXP(Dn,Js)
               OUT = .TRUE. ; Nt = Nt-1 ; GOTO 123
            ENDIF
c           ....................................................
            RHS(1:Nt) = MATMUL( MAT1(1:Nt,1:Nf),Dif(1:Nf) )
c           ... Coefficients ak (k=1,Nt) in Eq.(Ta-sin) 
            ReS(1:Nt) = MATMUL( Tinv(1:Nt,1:Nt),RHS(1:Nt) )
cc                   IF( I==46.and.Js==16 ) THEN
c                   IF( I > 95 ) THEN
c                      WRITE(*,*) '   ReS Nt= ',Nt
c                      WRITE(*,*) I, Js, ReS(1:Nt)
c                   ENDIF
 123        FORALL( K = 1:Nf )
     $           S_UM(K) = SUM( ReS(1:Nt)*MAT1(1:Nt,K) )
c           ... Spherical transmission approximation using additional terms
            Prel(1:Nf) = Tp(1:Nf)*(1.d0 + S_UM(1:Nf))            

c           ... Relative approximation error
            P_ec(1:Nf) = DABS(1.d0 - Prel(1:Nf)/SOL_EXP(Up+1:Dn-1,Js))
            Reld = MAXVAL( P_ec(1:Nf),1 )
c                   IF( ZZZ(I) < 6.d0 ) THEN
c                      WRITE(*,*) '  Top of SAG level ', ZZZ(I), Js
c                      WRITE(*,*) '   Number of termd ', Nt
c                      KK = Up+1
c                      DO K = 1, Nf
c                         WRITE(*,'(1x,i3,3(1x,e12.4))')  
c     $                        K, SOL_EXP(KK,Js), Prel(K), P_ec(K)*100.
c                         KK = KK + 1
c                      ENDDO
c                      WRITE(*,*) '  SOL_EXP at bot ', SOL_EXP(Dn,Js)
c                   ENDIF
            IF( OUT ) Then
               IF( Reld < Pre_AC ) THEN
                  SIN_COEF(Js,I,1:Nt) = ReS(1:Nt) ; N_sph(Js,I) = Nt+1
               ELSE
                  SIN_COEF(Js,I,1:Nt) = 0.d0 ; N_sph(Js,I) = 1
               ENDIF
c                    WRITE(*,'(A,I3,A,I3)')
c     $                   ' --> OUT ', Nt, '  --> ',N_sph(Js,I)
               CYCLE
            ENDIF

c           ... Required approximation accuracy is achieved
            IF( Reld <= Crit ) Then
c                      WRITE(*,'(A,1x,E12.4,1x,A,I3,2x,E12.4)') 
c     $                      ' --> Accuracy is achieved ',Reld*100.,' %', 
c     $                       Nt, SOL_EXP(Dn,Js)
               SIN_COEF(Js,I,1:Nt) = ReS(1:Nt) ; N_sph(Js,I) = Nt+1
               CYCLE
            ENDIF

            IF( Nt >= Nf-1 ) Then
c                     WRITE(*,'(A,$)') ' --> ... Nt >= N_max ...'
               IF( Reld < Pre_AC ) THEN
                  SIN_COEF(Js,I,1:Nt) = ReS(1:Nt) ; N_sph(Js,I) = Nt+1
c                     WRITE(*,'(A,1x,ES8.1,1x,A,2x,E12.4)') 
c     $                   '  obtained accuracy is  ', Reld*100.,' %', 
c     $                    SOL_EXP(Dn,Js)
               ELSE
                  SIN_COEF(Js,I,1:Nt) = 0.d0 ; N_sph(Js,I) = 1
c                     WRITE(*,'(A,1x,ES8.1,1x,A)') 
c     $                   '  used AC with accuracy', Pre_AC*100.,' %'
               ENDIF

               IF( MES_OUT_SOL ) THEN
                  CALL WRITE_MESSAGE(R_NAME, 'WARNING', ' ')
                  WRITE(EU,'(A)') 
     $                 '  Selected accuracy for transmission '//
     $                 'approximation'
                  WRITE(EU,'(A)') '  along solar beam is not reached:'
                  WRITE(EU,'(A,1x,ES8.1,1x,A)') 
     $                 '  requested accuracy is ',Crit*100.,' %'
                  WRITE(EU,'(A,1x,ES8.1,1x,A)') 
     $                 '  obtained accuracy is  ',Reld*100.,' %'
                  WRITE(EU,'(A,1x,I5)') 
     $                 '  Number of parameters  ',Nt
                  WRITE(EU,'(A,1x,F7.2)') 
     $                 '  Solar angle number  ',
     $                 DACOS(SOL_GRD(Js))/Pie*180.d0
                  WRITE(EU,'(A,1x,ES8.1,A,ES8.1,A)') 
     $                 '  Level :',Z_N(Up),' - ',Z_N(Dn),' km'
c                  WRITE(EU,*) '  Check file "control.inp" '//
                  MES_OUT_SOL = .FALSE.
               ENDIF
               CYCLE
            ELSE
               Nt = Nt + 1 ;  GOTO 21
            ENDIF
         ENDDO  ! over layers standard altitude grid (I)
      ENDDO  ! over solar angles (Js)

c      WRITE(*,*) '    SIN_COEF for first SZA:   '
c      DO I = 1, NL-1
cc         WRITE(*,'(2x,I3,*(1x,e12.4))') I, SIN_COEF(1:NSZA,I,1)
c         WRITE(*,'(2x,I3,*(1x,e12.4))') I, SIN_COEF(1,I,:)
c      ENDDO
c      WRITE(*,*) '    Mue amd Tr_out:   '
c      DO I = 1, NL-1
c         WRITE(*,'(2x,I3,20(1x,e12.4))') I, Mue(1,I), Tr_out(I,1)
c      ENDDO
c      OPEN( UNIT, FILE='./DATA_OUT/sol_exp.out',RECL=150000 )
c        DO J = 1,N_P
c           WRITE(UNIT,'(1x,I5,*(1x,e15.7))')
c     $          J, (SOL_EXP(J,Js), Js = 1,NSZA)
c        ENDDO
c      CLOSE(UNIT)
c      Pause

      IF( .NOT.Check_STA ) RETURN

c     **** Test of integrals ******
c      Js = NSZA
c      Muv = 0.01d0
c      DO I = 1, N_sha(Js)-1
c         Up =  (I-1)*NN + 1; Dn = Up + NN 
c         Sint(I) = 0.0d0 ; Cint(I) = 0.0d0
c         DO K = Up,Dn-1
c            Dtp = T_PP(K+1) - T_PP(K)
c            Muef = Dtp/(T_SP(K+1) - T_SP(K))
c            Sint(I) = Sint(I) + Muef*Muv/(Muef + Muv)*
c     $           (1.0d0 - DEXP(-Dtp/Muef)*DEXP(-Dtp/Muv))*SOL_EXP(K,Js)*
c     $           DEXP(-(T_PP(K)-T_PP(Up))/Muv)
c            Cint(I) = Cint(I) + Muef*Muv/(Muef - Muv)*
c     $           (DEXP(-Dtp/Muef) - DEXP(-Dtp/Muv))*SOL_EXP(K,Js)*
c     $           DEXP(-(T_PP(Dn)-T_PP(K+1))/Muv)
c         ENDDO
c
c         Dtp = T_PP(Dn) - T_PP(Up)
c         Ex1 = DEXP(-Dtp/Mue(Js,I)) ; Ex2 = DEXP(-Dtp/Muv)
c         Sint_ap(I) = Mue(Js,I)*Muv/(Mue(Js,I) + Muv)*(1.0d0 - Ex1*Ex2)
c         Cint_ap(I) = Mue(Js,I)*Muv/(Mue(Js,I) - Muv)*(Ex1 - Ex2)
c         Nt = N_sph(Js,I)-1 ; Zn = -1
c         DO K = 1,Nt
c            Ck = Dtp/DBLE(K)/Pie
c            Mu_co = Mue(Js,I)*Ck/DCMP( Ck,-Mue(Js,I) )
c            Sint_ap(I) = Sint_ap(I) + SIN_COEF(Js,I,K)*
c     $           AIMAG( Mu_co*Muv/(Mu_co + Muv) )*(1.0d0 - Zn*Ex1*Ex2)
c            Cint_ap(I) = Cint_ap(I) + SIN_COEF(Js,I,K)*
c     $          AIMAG( Mu_co*Muv/(Mu_co - Muv) ) *(Zn*Ex1 - Ex2)
c            Zn = -Zn
c         ENDDO
c         Sint_ap(I) = Sint_ap(I)*SOL_EXP(Up,Js)
c         Cint_ap(I) = Cint_ap(I)*SOL_EXP(Up,Js)
c      ENDDO
c      Js = NSZA
c      DO K = 1, N_sha(Js)-1
c         Up =  (K-1)*NN + 1; Dn = Up + NN 
c         WRITE(*,'(1x,I3,2(2x,f6.2),2x,I3,1x,I3,4(1x,e12.4))') 
c     $        K, Z_N(Up), Z_N(Dn), NN-1, N_sph(Js,K),
c     $        (Sint(K)-Sint_ap(K))/Sint(K)*100., 
c     $        (Cint(K)-Cint_ap(K))/Cint(K)*100.
c      ENDDO
      ALLOCATE( E_COM(NL,NSZA) )
      E_COM = 0.d0
      Do Js = 1,NSZA
         L = 1
c         WRITE(*,*) ' Js,  N_sha(Js)-1 ',Js, N_sha(Js)-1
         DO I = 1, N_sha(Js)-1
            Up =  (I-1)*NN + 1; Dn = Up + NN  ; Nf = Dn - Up - 1 
            IF( Nf == 0 ) CYCLE
            X(1:Nf) = T_PP(Up+1:Dn-1) - T_PP(Up) ; Nt = N_sph(Js,I)-1
            IF( Nt == 0 ) THEN
               Tp(1:Nf) = SOL_EXP(Up,Js)*DEXP( -X(1:Nf)/Mue(Js,I) )
            ELSE
               S_UM = 0.d0 ; Dtp = T_PP(Dn) - T_PP(Up)          
               DO K = 1,Nt
                  Ck = Dtp/DBLE(K)/Pie
                  Mu_co = Mue(Js,I)*Ck/DCMP( Ck,-Mue(Js,I) )
                  S_UM(1:Nf) = S_UM(1:Nf) + SIN_COEF(Js,I,K)*
     $                 AIMAG( EXP(-CMPLX(X(1:Nf),0.d0,KND)/Mu_co) )
               ENDDO
               Tp(1:Nf) = 
     $              SOL_EXP(Up,Js)*(DEXP(-X(1:Nf)/Mue(Js,I))+S_UM(1:Nf))
            ENDIF
            P_ec(1:Nf) = 1.d0 - Tp(1:Nf)/SOL_EXP(Up+1:Dn-1,Js)
            J = MAXLOC( DABS(P_ec(1:Nf)),1 )
            E_COM(L,Js) = P_ec(J) 
            L = L + 1
         ENDDO     ! over standard layers (I)
      ENDDO     ! over solar angles (Js)

      WRITE(*,*)
     $     ' ******** Write file ./DATA_OUT/appr_spher.out ********** '
      OPEN( UNIT, FILE='./DATA_OUT/appr_spher.out' )
      WRITE(UNIT,'(A)') 
     $  '#   --------- Approximation of spherical transmission --------'
      WRITE(UNIT,'(A,2x,I3)') 
     $     '#   Number of solar zenith angles:     ', NSZA
      WRITE(UNIT,'(A,2x,F7.2)') 
     $     '#   Altitude of shadow [km] (if any):  ', ZZZ(N_sha(1))
      WRITE(UNIT,'(A,2x,I3)') 
     $     '#   Number of sub-layers:              ', NN
      WRITE(UNIT,'(A,2x,e12.4)') 
     $     '#   Required accuracy [%]:             ', Crit_STA
      WRITE(UNIT,'(A)') '#   Column     Content'
      WRITE(UNIT,'(A)') 
     $     '#     1   - height of the middle layer [km] standard grid'
      WRITE(UNIT,'(A)') 
     $     '#     2   - geometrical thick of standard layer [km]'
      WRITE(UNIT,'(A)') 
     $     '#     3   - maximal approximation error [%] within layer'
      WRITE(UNIT,'(A)') 
     $  '#     4   - spherical transmission at bottom of standard layer'
      WRITE(UNIT,'(A)') 
     $     '#     5   - number of terms used in approximation'
      WRITE(UNIT,'(A)') 
     $     '#     6   - solar zenith angle'
      WRITE(UNIT,'(A)')
     $     '#  * Altitude with T(z) < 1.e-30 are removed *'
      WRITE(UNIT,'(A)')
     $     '#  *      T(z) is spherical transmission function'
      WRITE(UNIT,'(A)') '#    1         2           3'//
     $     '            4        5    6'
      DO I = 1, NL-1
         IF( Tr_out(I,1)==0.d0 ) CYCLE
         WRITE(UNIT,
     $        '(2(1x,f9.5),100(1x,e12.4,1x,e12.4,1x,I3,1x,f7.3))') 
     $        (ZZZ(I)+ZZZ(I+1))/2.0, ZZZ(I)-ZZZ(I+1),
     $        (E_COM(I,Js)*100., Tr_out(I,Js), N_sph(Js,I), 
     $        DACOS(SOL_GRD(Js))/Pie*180.d0, Js=1,NSZA)
      ENDDO
      CLOSE(UNIT)
      WRITE(*,*) 
     $     ' ********          End of test option          ********** ' 
      Stop ' STOP in routine "sin_fit.f" '

      END SUBROUTINE SIN_FIT      
C***********************************************************************
C*                                                                     *
C*               SUBROUTINE FOR MATRIX INVERSION                       *
C*                    CRAY optimized version                           *
C*                                                                     *
C***********************************************************************
      SUBROUTINE CHINV_LOC(NVEC, N, A, STATUS)

      INTEGER           NVEC, N, K, I, J
      LOGICAL           STATUS
      DOUBLE PRECISION  A(NVEC), X

      A(1) = 1.0D0/DSQRT(A(1))
      FORALL (I = 2:N) A(I*(I - 1)/2 + 1) = A(I*(I - 1)/2 + 1)*A(1)

      DO I = 2, N
         DO J = 2, I - 1
            X = A(I*(I - 1)/2 + J)
            DO K = 1, J - 1
               X = X - A(I*(I - 1)/2 + K)*A(J*(J - 1)/2 + K)
            END DO
            A(I*(I - 1)/2 + J)=X*A(J*(J - 1)/2 + J)
         END DO
         X = A(I*(I + 1)/2)
         DO K = I*(I - 1)/2 + 1, I*(I + 1)/2  - 1
            X = X - A(K)*A(K)
         END DO
         IF( X <= 0.0d0 ) THEN
c            WRITE(*,*) 'X negative :', X
c            WRITE(*,*) 'Message from "appr_spher_trans.f"'
            STATUS = .TRUE.
            RETURN
         ENDIF
         A(I*(I + 1)/2) = 1.0D0/DSQRT(X)
      END DO

C      INVERS L MATRIX
      DO I= 2, N
         DO J = 2, I
            X = 0
            DO  K = 1, I - J + 1
               X = X - A(I*(I + 1)/2 - K)*
     $              A(J - 1 + (I - K - 1)*(I - K)/2)
            END DO
            A(I*(I - 1)/2 + J - 1)=A(I*(I + 1)/2)*X
         END DO
      END DO

C      INVERS A MATRIX
      DO I= 1, N
         DO J = 1, I
            X = 0
            DO  K = I, N
               X = X + A(K*(K - 1)/2 + I)*A(K*(K - 1)/2 + J)
            END DO
            A(I*(I - 1)/2 + J) = X
         END DO
      END DO

      RETURN
      END SUBROUTINE CHINV_LOC

C  *******************************************
      END
