      SUBROUTINE BRDF_OCEAN_VEC( N_ref, X_ref, N_Phi, X1, N_INC, 
     $     COSI, Nwl_Inp, WL_Inp, R1_OUT )

      USE DEPOLARIZATION_ALBEDO_AEROSOL, 
     $                    ONLY : RefW_r, RefW_i, Nwl_BRF, WL_BRF
      USE LOG_CONTROL,    ONLY : LUNIT, LOG_LEVEL, N_DSO, N_ESO
      USE ST_VARS,        ONLY : Pie
      USE TYPES,          ONLY : DCMP

      INTEGER              N_ref, N_Phi, N_INC, Nwl_Inp     
      INTEGER              I, J, K, L, Ore, Nin, Nvs
      PARAMETER            (Nvs=4)
      DOUBLE PRECISION     R1_out(Nvs*N_ref,Nvs*N_INC,N_Phi,Nwl_Inp)
      DOUBLE PRECISION     X_ref(N_ref), X1(N_Phi), COSI(N_INC), 
     $                     WL_Inp(Nwl_Inp)
      DOUBLE PRECISION     DMUR, DUMI, PHII, G
      DOUBLE PRECISION     R(4,4)
      DOUBLE PRECISION, TARGET ::  Rezr(1), Rezi(1) 
      DOUBLE PRECISION, POINTER :: P_refr(:), P_refi(:)
      COMPLEX (kind(0.0d0)) :: CN1, CN2

c      Set BRDF parameters at the needed wavelength grid
c      =================================================
      IF( Nwl_Inp == 1 .AND. Nwl_BRF > 1) THEN
c         Interpolation on the current wl using SS_STOKES entry
         CALL LIN_SEARCH_NEW(WL_INP(1), G, I, K, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.D0-G)*RefW_r(I) + G*RefW_r(K)
         P_refr => Rezr(1:1)
         Rezi(1) = (1.D0-G)*RefW_i(I) + G*RefW_i(K)
         P_refi => Rezi(1:1)
c         IF (LOG_LEVEL(1) >= N_ESO)  THEN
c            WRITE(LUNIT,*) 
c     $           'Interpolated index of refraction for Nwl_BRF = ', 
c     $           Nwl_BRF
c            WRITE(LUNIT,*) '  and Nwl_Inp = ', Nwl_Inp
c            WRITE(LUNIT,'(2x,f9.6,2x,e15.7)')  P_refr(1), P_refi(1)
c         ENDIF
      ELSE
c        Using previously interpolated or constant values
         P_refr => RefW_r(:)  ;  P_refi => RefW_i(:)
c         IF (LOG_LEVEL(1) >= N_ESO)  THEN
c            WRITE(LUNIT,*) 
c     $           '  Index of refraction as it Nwl_BRF = ', Nwl_BRF
c            WRITE(LUNIT,*) '  and Nwl_Inp = ', Nwl_Inp
c            DO I = 1,Nwl_BRF
c               WRITE(LUNIT,'(2x,I3,2x,F7.2,2x,f9.6,2x,e15.7)')  
c     $              I, WL_BRF(I), P_refr(I), P_refi(I)
c            ENDDO
c         ENDIF
      ENDIF

      WRITE(*,*) '      brdf_ocean_vec'

      R1_OUT = 0.d0
c     ... Refractive indexes of the air   ... auxiliary variables
      CN1 = DCMP( 1.d0 )     ;      PHII  = 0.d0

c      Calculation of the reflection matrix
      DO L = 1,Nwl_Inp
         CN2 = DCMP( P_refr(L), P_refi(L) ) 
         DO J = 1,N_ref         ! Over all reflection zenith angle
            Ore = Nvs*(J-1)  ;  DMUR = X_ref(J)
            DO I = 1,N_INC      ! Over all incidence zenith angle
               Nin = Nvs*(I-1)  ;  DUMI = COSI(I)
c               R1_out(Ore+1:Ore+3,Nvs*I,:,L) =  0.d0
c               R1_out(Nvs*J,Nin+1:Nin+3,:,L) =  0.d0
               DO K = 1,N_Phi
                  CALL RMATR(CN1, CN2, DUMI, PHII, DMUR, X1(K), R)
c                  R(1,2:3) = -R(1,2:3)
c                  R(2,1)   = -R(2,1)
c                  R(2,3)   = -R(2,3)
c                  R(3,1:2) = -R(3,1:2)
                  R1_out(Ore+1:Ore+3,Nin+1:Nin+3,K,L) = R(1:3,1:3) 
                  R1_out(Nvs*J,      Nvs*I,      K,L) = R(4,4)

c                  IF( K==10 .and. I==1 ) THEN
c                     WRITE(*,'(1x,F8.3,1x, F8.3,3(2x,E12.4))') 
c     $                    DACOS(DMur)/Pie*180., X1(K)/Pie*180., R(4,4)
c                  ENDIF

               ENDDO
            ENDDO
         ENDDO
      ENDDO

      RETURN
      END SUBROUTINE BRDF_OCEAN_VEC
C**************************************************************

C   Calculation of the Stokes reflection matrix for illumination
C   from above for a statistically rough surface separating two 
C   half-spaces with refractive indices of the upper and lower 
C   half-spaces equal to CN1 and CN2, respectively.
C   The effect of shadowing is not included in this subroutine but
C   is added in the main program.

C   DMUI = ABS(COSINE OF THE INCIDENT ZENITH ANGLE)
C   PHII = INCIDENT AZIMUTH ANGLE.
C   DMUR = ABS(COSINE OF THE REFLECTION ZENITH ANGLE).
C   PHIR = REFLECTION AZIMUTH ANGLE
C   R - (4X4) REFLECTION MATRIX

      SUBROUTINE RMATR(CN1, CN2, DMUI, PHII, DMUR, PHIR, R)

      DOUBLE PRECISION       R(4,4), PHIR
      DOUBLE PRECISION       AF, AF11, AF12, AF21, AF22
      DOUBLE PRECISION       DCOEFF, DMUI, DMOD, DMUR, DCOSI,
     $                       DSINI, DCOSR, DSINR, PART,
     $                       DSI, DSR, E1, E2, E3, E4, FACT1, FACTOR, 
     $                       PHII, PI1, PI2, PI3, PR1, PR2, PR3, 
     $                       PIKR, PRKI, RDZ2, RDZ4, 
     $                       TIKR, TI1, TI2, TI3, TRKI, TR1, TR2, TR3,
     $                       XI1, UNIT1, UNIT2, UNIT3, 
     $                       VI1, VI2, VI3, VP1, VP2, VP3, VR1, VR2, VR3
      COMPLEX (kind(0.0d0)) :: CN1, CN2, CXI2, C1, C2, CRPER, CRPAR, 
     $                         CF11, CF12, CF21, CF22, CI, C21, C22, 
     $                         CTTTP, CTTPT, CTTPP, CTPPT, CTPPP, CPTPP
      
      IF(DABS(DMUI-1D0).LT.1d-9) DMUI=0.999999999999d0
      IF(DABS(DMUR-1D0).LT.1d-9) DMUR=0.999999999999d0

      DCOSI = DCOS(PHII)  ;  DSINI = DSIN(PHII)
      DCOSR = DCOS(PHIR)  ;  DSINR = DSIN(PHIR)      
      DSI   = DSQRT(1.d0-DMUI*DMUI)  ;  DSR   = DSQRT(1.d0-DMUR*DMUR)
      
C     ... Cartezian components of the unit vectors of the incident:
c     -------------------------------------------------------------
      VI1 = DSI*DCOSI  ;  VI2 = DSI*DSINI  ;  VI3 = -DMUI
C     ... and  scattered beams:
c     -------------------------
      VR1 = DSR*DCOSR  ;  VR2 = DSR*DSINR  ;  VR3 = DMUR

C     ... Local surface normal for specular reflection (n0-n)
c     -------------------------------------------------------
      UNIT1 = VI1 - VR1  ;  UNIT2 = VI2 - VR2  ;  UNIT3 = VI3 - VR3
      
c     ... Norm of the vector directed along the surface normal
c     --------------------------------------------------------
      FACT1 = UNIT1*UNIT1 + UNIT2*UNIT2 + UNIT3*UNIT3
      FACTOR = DSQRT(1.d0/FACT1)

c     ... Cosine of the reflection angle 
c     ----------------------------------
      XI1  = FACTOR*(UNIT1*VI1 + UNIT2*VI2 + UNIT3*VI3)

C     ... Fresnel reflection coefficients
c     -----------------------------------
c      write(*,*) ' Ugol padeniya :', dacos(XI1)/3.14159*180.
      CXI2 = 1.d0 - (1.d0 - XI1*XI1)*CN1*CN1/(CN2*CN2)
      CXI2 = CDSQRT(CXI2)

c     ... Perpendicular component with respect to the local surface
c     -------------------------------------------------------------
      C1 = CN1*XI1  ;  C2 = CN2*CXI2
      CRPER = (C1 - C2)/(C1 + C2)

c     ... Parallel component with respect to the local surface
c     --------------------------------------------------------
      C1   = CN2*XI1  ;  C2   = CN1*CXI2
      CRPAR = (C1 - C2)/(C1 + C2)

c      WRITE(*,*) '           C_PAR and C_per ::'
c      WRITE(*,*) DBLE( CRPAR**2 )
c      WRITE(*,*) DBLE( CRPER**2 )

C     Calculation of the amplitude scattering matrix
c     ----------------------------------------------
c     Cartezian components of the Theta vectors of the incident:
c     ---------------------------------------------------------
      TI1 = -DMUI*DCOSI ; TI2 = -DMUI*DSINI ; TI3 = -DSI
c     and  scattered beams:
c     ---------------------
      TR1 = DMUR*DCOSR  ; TR2 = DMUR*DSINR ; TR3 = -DSR

c     Cartezian components of the Phi vectors of the incident:
c     ---------------------------------------------------------
      PI1 = -DSINI  ;  PI2 = DCOSI  ;  PI3 = 0D0
c     and  scattered beams:
c     ---------------------
      PR1 = -DSINR  ;  PR2 = DCOSR  ;  PR3 = 0D0

      PIKR = PI1*VR1 + PI2*VR2 + PI3*VR3
      PRKI = PR1*VI1 + PR2*VI2 + PR3*VI3
      TIKR = TI1*VR1 + TI2*VR2 + TI3*VR3
      TRKI = TR1*VI1 + TR2*VI2 + TR3*VI3

      E1 = PIKR*PRKI
      E2 = TIKR*TRKI
      E3 = TIKR*PRKI
      E4 = PIKR*TRKI

      CF11 =  E1*CRPER + E2*CRPAR
      CF12 = -E3*CRPER + E4*CRPAR
      CF21 = -E4*CRPER + E3*CRPAR
      CF22 =  E2*CRPER + E1*CRPAR

C     Calculation of the stokes reflection matrix
c     -------------------------------------------
c      Cartezian components of the vector corresponding to the vector product
c       [n0xn] :
      VP1= VI2*VR3-VI3*VR2 ; VP2= VI3*VR1-VI1*VR3 ; VP3= VI1*VR2-VI2*VR1
      
      DMOD = VP1*VP1 + VP2*VP2 + VP3*VP3
      DMOD = DMOD*DMOD
    
      RDZ2 = UNIT3*UNIT3
      RDZ4 = RDZ2*RDZ2

      DCOEFF = 1.d0/DMOD

c     ... This is a part of the wave slope PDF and 
c         reflection function without dividing 
c         by Cosr (cosine of reflection angle)
      PART = FACT1*FACT1/(4.d0*DMUI*RDZ4)
c     ... To return the reflection matrix only for 
c         test purpose set PART = 1.d0
c      PART = 1.d0

      DCOEFF = DCOEFF*PART

      AF = 0.5D0*DCOEFF
      AF11 = CDABS(CF11)
      AF12 = CDABS(CF12)
      AF21 = CDABS(CF21)
      AF22 = CDABS(CF22)
      AF11 = AF11*AF11
      AF12 = AF12*AF12
      AF21 = AF21*AF21
      AF22 = AF22*AF22

      R(1,1) = (AF11 + AF12 + AF21 + AF22)*AF
      R(1,2) = (AF11 - AF12 + AF21 - AF22)*AF
      R(2,1) = (AF11 - AF22 + AF12 - AF21)*AF
      R(2,2) = (AF11 - AF12 - AF21 + AF22)*AF

      CI = (0D0, -1D0)

      C21 = DCONJG(CF21)
      C22 = DCONJG(CF22)
      CTTTP = CF11*DCONJG(CF12)
      CTTPT = CF11*C21
      CTTPP = CF11*C22
      CTPPT = CF12*C21
      CTPPP = CF12*C22
      CPTPP = CF21*C22

      R(1,3) =  DBLE(    (-CTTTP-CPTPP))*DCOEFF
      R(1,4) = -DBLE( CI*( CTTTP+CPTPP))*DCOEFF
      R(2,3) =  DBLE(    (-CTTTP+CPTPP))*DCOEFF
      R(2,4) = -DBLE( CI*( CTTTP-CPTPP))*DCOEFF
      R(3,1) =  DBLE(    (-CTTPT-CTPPP))*DCOEFF
      R(3,2) =  DBLE(    (-CTTPT+CTPPP))*DCOEFF
      R(3,3) =  DBLE(    ( CTTPP+CTPPT))*DCOEFF
      R(3,4) =  DBLE( CI*( CTTPP-CTPPT))*DCOEFF
      R(4,1) =  DBLE( CI*( CTTPT+CTPPP))*DCOEFF
      R(4,2) =  DBLE( CI*( CTTPT-CTPPP))*DCOEFF
      R(4,3) = -DBLE( CI*( CTTPP+CTPPT))*DCOEFF
      R(4,4) =  DBLE(    ( CTTPP-CTPPT))*DCOEFF

      RETURN
      END SUBROUTINE RMATR 
