!*********************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 07 Feb 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!*********************************************************************
SUBROUTINE CHECK_DOM_WF(*)
  ! Called by "gt_settings_check.f"

  USE INFO,       ONLY: RET_INFO, TG_INFO
  USE LOG_PARS,   ONLY: EU
  USE ST_INPUT,   ONLY: BAND_START
  USE ST_VARS,    ONLY: SCATT_MODE, RTM_TYPE, Vec_Sca, Ns_wf, BRDF_FLAG, & 
                        SURF_TYPE, Nc_der, Ray_to_WF
  USE TOOLS,      ONLY: INQ_WF
  USE TYPE_VARS,  ONLY: KND
! ----------------------------------------------------------------------

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'CHECK_DOM_WF'
  CHARACTER (LEN=100) Msg(10)
  INTEGER    I, II, JJ, KK, N_Msg, N_tg
! -----------------------------------------------------------------------
  CALL ST_TRACK (R_NAME, 1, 'I')

! ... Restrictions with respect to WF in general case
! =========================================================
  IF( INQ_WF('anps', 'RIN') ) THEN
     Msg(1)='   Average number of photon scattering calculation '
     Msg(2)='   is not temporary available.'
     CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 2, *880)
  ENDIF
! ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
  IF(BAND_START(1) <= 240.d0) THEN
     Msg(1)='   Pressure and teperature weighting functions '
     Msg(2)='   can not be calculated for Schumann-Runge and '
     Msg(3)='   Herzberg bands'
     CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 3, *880)
  END IF
! ..............................................................
  SELECT CASE (TRIM(SCATT_MODE))
  CASE('wf_ss_ms','sm_wf_ss')
     Msg(1)='Settings "sm_wf_ss" and "wf_ss_ms" have only an effect'
     Msg(2)='  if "CDI" mode is selected. '
     Msg(3)='Check "Scattering mode" parameter in "control_ac.inp" file'
     Msg(4)='  and "RTM-CORE" parameter in "control.inp" file.'
     CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
  CASE('wf_ss')
     IF( TRIM(RTM_TYPE)/='spher_scat' ) THEN
        Msg(1)='Settings "wf_ss" have only an effect if:'
        Msg(2)='a) "CDI" mode is active;'
        Msg(3)='b) "DOM" mode with "spher_scat" is selected.'
        Msg(4)='Check "Scattering mode" parameter in "control_ac.inp" file.'
        Msg(5)=' and "RTM-CORE" parameter in "control.inp" file.'
        CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
     END IF
  END SELECT
! ..............................................................

! ... Restrictions with respect to WF in spherical geometry
! =========================================================
  IF( RTM_TYPE == 'spher_scat' ) THEN
     IF( INQ_WF('cld','RIT') ) THEN
        Msg(1)='    Weighting functions for cloud parameters '
        Msg(2)='    are not supported for calculation in the '
        Msg(3)='    "spher_scat" mode. Check file "control_wf.inp".'
        CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 3, *880)
     ENDIF
     IF(INQ_WF('aer','RIT').AND.INQ_WF('press','RIN')) THEN
        Msg(1)='Weighting functions for aerosol and pressure'
        Msg(2)='  can not be calculated simultaneously in the'
        Msg(3)='  "spher_scat" mode.'
        Msg(4)='Check file "control_wf.inp".'
        CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
     ENDIF
     IF(INQ_WF('aer','RIT').AND.INQ_WF('temp','RIN')) THEN
        Msg(1)='Weighting functions for aerosol and'
        Msg(2)='  temperature can not be calculated'
        Msg(3)='  simultaneously in the "spher_scat" mode.'
        Msg(4)='Check file "control_wf.inp".'
        CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
     ENDIF
     IF(INQ_WF('aer','RIT').AND.Ray_to_WF) THEN
        Msg(1)='Weighting functions for aerosol and Rayleigh'
        Msg(2)='  can not be calculated simultaneously in the'
        Msg(3)='  "spher_scat" mode.'
        Msg(4)='Check file "control_wf.inp".'
        CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
     ENDIF
!    ... WFs: Vector case in spherical geometry
     IF( Vec_sca=='vect_rte' ) THEN
        
        Msg(1)='  Restrictions for WFs in spherical geometry including polarization.'
!       ... There are WFs except of absorbing gases
        !IF( ANY(RET_INFO%TYPE/='tg') ) THEN
        !   Msg(2)='Calculation WF for gaseous absorber only is allowed.'
        !   Msg(3)='Check "Weighting functions: trace gases" control'
        !   Msg(4)=' parameter in "control_wf.inp" file.'
        !   CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
        !ENDIF        
!        IF( Ns_wf > 1 ) THEN
!           Msg(2)='Calculation WF for only single component of '
!           Msg(3)='  Stokes vector is implemented.'
!           Msg(4)='Check "Weighting functions for Stokes component"'
!           Msg(5)=' control parameter in "control_wf.inp" file.'
!           CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5,*880)
!        ENDIF
        N_tg = COUNT( RET_INFO%TYPE=='tg' )
        IF( N_tg > 1 ) THEN
           Msg(2)='Calculation WF for single gaseous absorber only is allowed.'
           Msg(3)='Check "Weighting functions: trace gases" control'
           Msg(4)=' parameter in "control_wf.inp" file.'
           CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
        ENDIF
        SELECT CASE( TRIM(SURF_TYPE) )
        CASE('ocean','mRPV_facet')
           IF( ANY(Nc_der(:) > 1) ) THEN
              Msg(2)='Calculation derivative of Q, U, V components is not allowed'
              Msg(3)='for the options "ocean" and "mRPV_facet".'
              Msg(4)='Check "Weighting functions for Stokes component" control'
              Msg(5)='  parameter in "control_wf.inp" file and "Surface type"'
              Msg(6)='  control parameter in "control_brdf.inp" file.'
              CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 6, *880)
           ENDIF
        END SELECT
!        IF( BRDF_FLAG ) THEN
!           Msg(2)='  Lambertian reflection is implemented only.'
!           Msg(3)='Check "Surface reflection type" control parameter'
!           Msg(4)='  in "control.inp" file.' 
!           CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
!        ENDIF
     ENDIF  ! vector case in spherical geometry
  ENDIF
!  <<<<<<<<<<<<<  WF in spherical geometry >>>>>>>>>>>>>>>>>>>

! ... Restrictions with respect to WF for mode "dir_ref"
  IF( TRIM(SCATT_MODE) == 'dir_ref' ) THEN
     DO KK = 1, UBOUND(RET_INFO,1)
        IF(RET_INFO(KK)%TYPE == 'tg') CYCLE
        SELECT CASE (RET_INFO(KK)%NAME)
        CASE ('temp', 'albedo', 'flu_sfl')
        CASE DEFAULT
           Msg(1)='   Mode "dir_ref" can be used for calculation '
           Msg(2)='   of gaseous absorber weighting function, '
           Msg(3)='   temperature weighting function, albedo, and'
           Msg(4)='   land fluorescence scaling parameter only.'
           Msg(5)='   Check control fields in "control_wf.inp" file and '
           Msg(6)='   "Scattering mode" parameter in "control_ac.inp".'
           CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 6, *880)
        END SELECT
     ENDDO
  ENDIF  ! "dir_ref" mode

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

END SUBROUTINE CHECK_DOM_WF
