!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 26 Oct 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE CIA_DENSITIES

  USE CIA_VARS,  ONLY: CIA_INFO, CIA_VMR_MODE, CIA_VMR_VAL, N2_Air_Corr
  USE GRIDS,     ONLY: NLAYERS
  USE INFO,      ONLY: TG_INFO
  USE PROFILES,  ONLY: AIR_DENSITY, TEMP
  USE ST_VARS,   ONLY: SMKM
  USE TOOLS,     ONLY: FIND_IDX
  USE TYPE_VARS, ONLY: KND

! Local variables:
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'CIA_DENSITIES'
  INTEGER    I_P, II
  REAL (KND) Rel_Eff(NLAYERS), DENS(NLAYERS)

  CALL ST_TRACK (R_NAME, 1, 'I')

  DO I_P = 1, Size(CIA_INFO)
     If(.Not.CIA_INFO(I_P)%Flg) Cycle
     IF(ALLOCATED(CIA_INFO(I_P)%DENS)) DEALLOCATE(CIA_INFO(I_P)%DENS)
     ALLOCATE(CIA_INFO(I_P)%DENS(NLAYERS)) !A
     SELECT CASE (TRIM(CIA_INFO(I_P)%Name))
     CASE('N2-N2')
! The formula includes induced absobrtion by N2 due to collisions with
! both N2 and O2 in accordance with Lafferty et al., Appl. Opt., 35, 1996.
        DENS(:) = AIR_DENSITY(:)*CIA_VMR_VAL(1)
        IF(N2_Air_Corr) THEN
           Rel_Eff(:) = 1.294D0 - 0.4545D0*Temp(:)/296.D0
           IF(CIA_VMR_MODE(2) == 'F') THEN
              II = FIND_IDX('o2', TG_INFO%NAME)
              CIA_INFO(I_P)%DENS = DENS(:)*(DENS(:) + &
                   TG_INFO(II)%ND(:)*Rel_Eff(:))
           ELSE
              CIA_INFO(I_P)%DENS = DENS(:)*AIR_DENSITY(:)* &
                   (CIA_VMR_VAL(1) + CIA_VMR_VAL(2)*Rel_Eff(:))
           END IF
        ELSE
           CIA_INFO(I_P)%DENS = DENS(:)**2
        END IF
     CASE('O2-O2')
        IF(CIA_VMR_MODE(2) == 'F') THEN
           II = FIND_IDX('o2', TG_INFO%NAME)
           CIA_INFO(I_P)%DENS = TG_INFO(II)%ND(:)**2
        ELSE
           CIA_INFO(I_P)%DENS = (AIR_DENSITY(:)*CIA_VMR_VAL(2))**2
        END IF
     CASE('O2-N2')
        DENS(:) = AIR_DENSITY(:)*CIA_VMR_VAL(1)
        IF(CIA_VMR_MODE(2) == 'F') THEN
           II = FIND_IDX('o2', TG_INFO%NAME)
           CIA_INFO(I_P)%DENS = DENS(:)*TG_INFO(II)%ND(:)
        ELSE
           CIA_INFO(I_P)%DENS = DENS(:)*CIA_VMR_VAL(2)*AIR_DENSITY(:)
        END IF
     CASE('O2-CO2')
        IF(CIA_VMR_MODE(2) == 'F') THEN
           II = FIND_IDX('o2', TG_INFO%NAME)
           DENS(:) = TG_INFO(II)%ND(:)
        ELSE
           DENS(:) = AIR_DENSITY(:)*CIA_VMR_VAL(2)
        END IF
        IF(CIA_VMR_MODE(3) == 'F') THEN
           II = FIND_IDX('co2', TG_INFO%NAME)
           CIA_INFO(I_P)%DENS = DENS(:)*TG_INFO(II)%ND(:)
        ELSE
           CIA_INFO(I_P)%DENS = DENS(:)*CIA_VMR_VAL(3)*AIR_DENSITY(:)
        END IF
     END SELECT
     CIA_INFO(I_P)%DENS = CIA_INFO(I_P)%DENS/SMKM
  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE CIA_DENSITIES
