!***********************************************************************
!
!       SCIATRAN - Version 4.5 - Last modification: 17 Aug 2021
!
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!************************************************************************
SUBROUTINE CLIM_IFACE (TEMP, CONC, *)

  USE LOG_PARS, ONLY: DO_LOGFILE, C_UNIT, LUNIT
  USE GRIDS,        ONLY: NLAYERS, ZZZ
  Use Profiles,       Only: O3_TOTAL_COLUMN
  USE ST_VARS,   ONLY: LATITUDE
  USE ST_INPUT, ONLY : DO_PROFILES_AKIMA, CLIM_PATH, CLIM_LABEL, DAY_OF_YEAR, YEAR
  Use  Types,         Only: KND, DYN_CHAR
      
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'CLIM_IFACE'
 
  INTEGER             NL_CLIM, NL_1, I
  INTEGER             NPROF(9), TPROF(3)
  INTEGER, PARAMETER :: NIUP = 61, NTOMS = 34, NTOMSV8 = 61, NKNMI = 61, NTOMSM = 61
      
  CHARACTER (LEN=:), Allocatable ::  TOMSV8_FILE, KNMI_FILE, TOMS_MON_FILE, Path
  TYPE (DYN_CHAR) IUP_FILE(9), TOMS_FILE(3)
  CHARACTER (LEN=4)  EXT_NAME(9)
  REAL (KND)    TEMP(NLAYERS), CONC(NLAYERS)
  REAL (KND), ALLOCATABLE :: TMP(:), Z_CLIM(:), TEMP_CLIM(:),  WO3_CLIM(:)
  REAL (KND)    SLOPE, T_MIN
  REAL (KND), PARAMETER :: DDZ = 1.0D-4
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
 
  SELECT CASE (CLIM_LABEL)
  CASE('IUP_BREMEN', 'IUP_2018')
     NL_CLIM = NIUP ;  NL_1 = NL_CLIM + 2
     ALLOCATE(Z_CLIM(NL_1), WO3_CLIM(NL_1),TEMP_CLIM(NL_1))
     EXT_NAME = (/'trop', 'nmws', 'nmsf', 'smws', 'smsf', 'npws', 'npsf', 'spws', 'spsf'/)
     If(CLIM_LABEL == 'IUP_BREMEN') Then
        NPROF = (/5, 11, 7, 10, 5, 11, 7, 8, 6/)
        Path = CLIM_PATH//'/IUP/'
     Else
        NPROF = (/4, 8, 5, 6, 4, 7, 6, 7, 4/)
        Path = CLIM_PATH//'/IUP_2018/'
     End If
     DO I = 1, 9 
        IUP_FILE(i)%S = Path//'IUP_temp_o3_'//ext_name(i)
     END DO
     CALL IUP_CLIMATOLOGY (Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM))
         
  CASE('TOMS_V7')
     NL_CLIM = NTOMS ;  NL_1 = NL_CLIM + 2
     ALLOCATE(Z_CLIM(NL_1), WO3_CLIM(NL_1),TEMP_CLIM(NL_1))
     TOMS_FILE(1)%S = clim_path//'/TOMSV7/TOMS_temp_o3_low'
     TOMS_FILE(2)%S = clim_path// '/TOMSV7/TOMS_temp_o3_middle'
     TOMS_FILE(3)%S = clim_path// '/TOMSV7/TOMS_temp_o3_high'
     TPROF = (/6,10,10/) 
     CALL TOMS_CLIMATOLOGY (Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM))
         
  CASE('TOMS_V8')
      NL_CLIM = NTOMSV8 ;  NL_1 = NL_CLIM + 2
     ALLOCATE(Z_CLIM(NL_1), WO3_CLIM(NL_1),TEMP_CLIM(NL_1))
     TOMSV8_FILE = TRIM(clim_path)//'/TOMSV8/TOMSV8_climatology.dat'
     Call TOMSV8_climatology (Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM))
     
  CASE('KNMI')
     NL_CLIM = NKNMI ;  NL_1 = NL_CLIM + 2
     ALLOCATE(Z_CLIM(NL_1), WO3_CLIM(NL_1),TEMP_CLIM(NL_1))
     KNMI_FILE = TRIM(clim_path)//'/KNMI/KNMI_climatology.dat'
     CALL KNMI_CLIMATOLOGY (Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM))
     
  CASE('TOMS_M')
     NL_CLIM = NTOMSM ;  NL_1 = NL_CLIM + 2
     ALLOCATE(Z_CLIM(NL_1), WO3_CLIM(NL_1),TEMP_CLIM(NL_1))
     TOMS_MON_FILE = TRIM(clim_path)// '/TOMSV8_MON/TOMS_MONTHLY_clim.dat'
     Call TOMS_MONTHLY_clim (Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM))
         
  END SELECT
  
  IF(Z_CLIM(1) > Z_CLIM(NL_CLIM)) THEN
     ALLOCATE(TMP(NL_CLIM))
     TMP(1:NL_CLIM) = Z_CLIM(NL_CLIM:1:-1)
     Z_CLIM(1:NL_CLIM) = TMP(1:NL_CLIM)
     TMP(1:NL_CLIM) = TEMP_CLIM(NL_CLIM:1:-1)
     TEMP_CLIM(1:NL_CLIM) = TMP(1:NL_CLIM)
     TMP(1:NL_CLIM) = WO3_CLIM(NL_CLIM:1:-1)
     WO3_CLIM(1:NL_CLIM) = TMP(1:NL_CLIM)
  END IF
  
  IF (Z_CLIM(1) < 0.0D0)  CALL WRITE_MESSAGE(R_NAME,  &
       'WARNING', 'Lowest profile altitude < 0.0 km')
  
  CALL AL_INTERP (NL_CLIM, Z_CLIM(1:NL_CLIM), WO3_CLIM(1:NL_CLIM), NLAYERS, &
       ZZZ(1:NLAYERS), CONC(1:NLAYERS), DO_PROFILES_AKIMA, *125)
  WHERE(ZZZ(:) < Z_CLIM(1)) CONC(:) = WO3_CLIM(1)
  WHERE(ZZZ(:) > Z_CLIM(NL_CLIM)) CONC(:) = WO3_CLIM(NL_CLIM)
            
  ! Check if BOA extrapolation is required 
  !  --------------------------------------
  IF (Z_CLIM(1).GT.ZZZ(NLAYERS)) THEN
     NL_CLIM = NL_CLIM + 1
     Z_CLIM(1:NL_CLIM) = CSHIFT(Z_CLIM(1:NL_CLIM),-1)
     TEMP_CLIM(1:NL_CLIM) = CSHIFT(TEMP_CLIM(1:NL_CLIM),-1)
     
     Z_CLIM(1) = ZZZ(NLAYERS) - DDZ
     SLOPE = (Z_CLIM(1) - Z_CLIM(2))/(Z_CLIM(2) - Z_CLIM(3))
     TEMP_CLIM(1) = TEMP_CLIM(2) + (TEMP_CLIM(2) - TEMP_CLIM(3))*SLOPE
  ENDIF
      
  ! Check if TOA extrapolation is required 
  ! --------------------------------------
  IF (Z_CLIM(NL_CLIM).LT.ZZZ(1)) THEN
     NL_CLIM = NL_CLIM + 1
     Z_CLIM(NL_CLIM) = ZZZ(1) + DDZ
     SLOPE = (Z_CLIM(NL_CLIM) - Z_CLIM(NL_CLIM - 1))/&
          (Z_CLIM(NL_CLIM - 1) - Z_CLIM(NL_CLIM - 2))
     T_MIN = DMIN1(TEMP_CLIM(NL_CLIM - 1),190.0D0)
     TEMP_CLIM(NL_CLIM) = DMAX1(T_MIN, TEMP_CLIM(NL_CLIM - 1) + &
          (TEMP_CLIM(NL_CLIM - 1) - TEMP_CLIM(NL_CLIM - 2))*SLOPE)
  ENDIF
      
  CALL AL_INTERP(NL_CLIM, Z_CLIM(1:NL_CLIM), TEMP_CLIM(1:NL_CLIM), NLAYERS, &
       ZZZ(1:NLAYERS), TEMP(1:NLAYERS), DO_PROFILES_AKIMA, *123)
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
123 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error (temperature)')
  RETURN 1
125 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error (ozone)')
  RETURN 1
  
CONTAINS
 !**************************************************************************
 !**************************************************************************
  SUBROUTINE IUP_CLIMATOLOGY (ZZ, TT, WO3_OUT)
    
    INTEGER             KK_1, KK_2, I
    REAL (KND)  TEMP_MIX(NL_CLIM,9), WO3_MIX(NL_CLIM,9)
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ
    REAL (KND), DIMENSION (NL_CLIM) :: scale_ht    !Nora
    REAL (KND)  PP(NL_CLIM), a_den(nl_clim)  !Nora
    REAL (KND)  totoz, dz,cmix,tmix   !Nora
    
    ! IUP climatology:tropics,nhmidlatitude winter/spring,nhmidlatitude
    ! summer/fall,shmidlatitude winter/spring,shmidlatitude summer/fall,
    ! nhpolar winter/spring,nhpolar summer/fall,shpolar winter/spring,
    ! and shpolar summer/fall.(NINE CASES)
    DO I = 1, 9
       CALL READ_CLIM (IUP_FILE(I)%S, NPROF(I), O3_TOTAL_COLUMN, ZZ, &
            WO3_MIX(:,I), TEMP_MIX(:,I))
    END DO
    
    ! ------------------------------------------------------------------
    ! Mix profiles depending upon the latitude and month
    ! ------------------------------------------------------------------
    IF(LATITUDE < 0.0D0) THEN
       KK_1 = 3 ;  KK_2 = 1
    ELSE
       KK_1 = 0 ;  KK_2 = 0
    END IF
    
    SELECT CASE (FLOOR(ABS(LATITUDE)))
    CASE(:24)
       wo3_out(:) = wo3_mix(:,1)
       tt(:) = temp_mix(:,1)
       
    CASE(25:34)
       cmix=(abs(latitude)-25.)/10.d0
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:)=(1.d0-cmix)*wo3_mix(:,1) + cmix*wo3_mix(:,2 + KK_1)
          tt(:) =  (1.d0-cmix)*temp_mix(:,1) + cmix*temp_mix(:,2 + KK_1)
       CASE(121:181)
          tmix=(day_of_year-120.)/62.d0
          wo3_out(:) = (1.d0-cmix)*wo3_mix(:,1) + cmix*( &
               (1.d0-tmix)*wo3_mix(:,2 + KK_1) + tmix*wo3_mix(:,3 + KK_2))
          tt(:) = (1.d0-cmix)*temp_mix(:,1) + cmix*( &
               (1.d0-tmix)*temp_mix(:,2 + KK_1) + tmix*temp_mix(:,3 + KK_2))
       CASE(182:304)
          wo3_out(:)=(1.d0-cmix)*wo3_mix(:,1) + cmix*wo3_mix(:,3 + KK_2)
          tt(:) = (1.d0-cmix)*temp_mix(:,1) + cmix*temp_mix(:,3 + KK_2)
       CASE (305:)
          tmix=(day_of_year-304.)/62.d0
          wo3_out(:) = (1.d0-cmix)*wo3_mix(:,1) + cmix*( &
               tmix*wo3_mix(:,2 + KK_1) + (1.d0-tmix)*wo3_mix(:,3 + KK_2))
          tt(:) = (1.d0-cmix)*temp_mix(:,1) + cmix*( &
               tmix*temp_mix(:,2 + KK_1) + (1.d0-tmix)*temp_mix(:,3 + KK_2))
       END SELECT
       
    CASE(35:54)
       cmix=(dabs(latitude)-35.)/10.d0
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = wo3_mix(:,2 + KK_1)
          tt(:) = temp_mix(:,2 + KK_1)
       CASE(121:181)
          tmix=(day_of_year-120.)/62.d0
          wo3_out(:) = (1.d0-tmix)*wo3_mix(:,2 + KK_1) + &
               tmix*wo3_mix(:,3 + KK_2)
          tt(:) = (1.d0-tmix)*temp_mix(:,2 + KK_1) + tmix*temp_mix(:,3 + KK_2)
       CASE(182:304)
          wo3_out(:) = wo3_mix(:,3 + KK_2)
          tt(:) = temp_mix(:,3 + KK_2)
       CASE (305:)
          tmix=(day_of_year-304.)/62.d0
          wo3_out(:) = tmix*wo3_mix(:,2 + KK_1) + &
               (1.d0-tmix)*wo3_mix(:,3 + KK_2)
          tt(:) = tmix*temp_mix(:,2 + KK_1) + (1.d0-tmix)*temp_mix(:,3 + KK_2)
       END SELECT
       
    CASE(55:64)
       cmix=(dabs(latitude)-55.)/10.d0
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = (1.d0-cmix)*wo3_mix(:,2 + KK_1) +&
               cmix*wo3_mix(:,6 + KK_1)
          tt(:) = (1.d0-cmix)*temp_mix(:,2 + KK_1) +cmix*temp_mix(:,6 + KK_1)
       CASE(121:181)
          tmix=(day_of_year-120.)/62.d0
          wo3_out(:) = (1.d0-tmix)*((1.d0-cmix)*wo3_mix(:,2 + KK_1) + cmix* &
               wo3_mix(:,3 + KK_2)) + tmix*((1.d0-cmix)*wo3_mix(:,6 + KK_1) +&
               cmix*wo3_mix(:,7 + KK_2))
          tt(:) =(1.d0-tmix)*((1.d0-cmix)*temp_mix(:,2 + KK_1) + cmix*&
               temp_mix(:,3 + KK_2)) +tmix*((1.d0-cmix)*temp_mix(:,6 + KK_1)+&
               cmix*temp_mix(:,7 + KK_2))
       CASE(182:304)
          wo3_out(:) = (1.d0-cmix)*wo3_mix(:,3 + KK_2) + &
               cmix*wo3_mix(:,7 + KK_2)
          tt(:) = (1.d0-cmix)*temp_mix(:,3 + KK_2) +cmix*temp_mix(:,7 + KK_2)
       CASE (305:)
          tmix=(day_of_year-304.)/62.d0
          wo3_out(:) = tmix*((1.d0-cmix)*wo3_mix(:,2 + KK_1) + &
               cmix*wo3_mix(:,3 + KK_2)) + &
               (1.d0-tmix)*((1.d0-cmix)*wo3_mix(:,6 + KK_1) +&
               cmix*wo3_mix(:,7 + KK_2))
          tt(:) =tmix*((1.d0-cmix)*temp_mix(:,2 + KK_1) +&
               cmix*temp_mix(:,3 + KK_2))+&
               (1.d0-tmix)*((1.d0-cmix)*temp_mix(:,6 + KK_1) + &
               cmix*temp_mix(:,7 + KK_2))
       END SELECT
       
    CASE(65:)
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = wo3_mix(:,6 + KK_1)
          tt(:) = temp_mix(:,6 + KK_1)
       CASE(121:181)
          tmix=(day_of_year-120.)/62.d0
          wo3_out(:) = (1.d0-tmix)*wo3_mix(:,6 + KK_1) + &
               tmix*wo3_mix(:,7 + KK_2)
          tt(:) = (1.d0-tmix)*temp_mix(:,6 + KK_1) + tmix*temp_mix(:,7 + KK_2)
       CASE(182:304)
          wo3_out(:) = wo3_mix(:,7 + KK_2)
          tt(:) = temp_mix(:,7 + KK_2)
       CASE (305:)
          tmix=(day_of_year-304.)/62.d0
          wo3_out(:) = tmix*wo3_mix(:,6 + KK_1) + &
               (1.d0-tmix)*wo3_mix(:,7 + KK_2)
       END SELECT
       
    END SELECT
    
    RETURN
  END SUBROUTINE IUP_CLIMATOLOGY
  !**************************************************************************
  
  SUBROUTINE IUP_CLIMATOLOGY_NoInterp (ZZ, TT, WO3_OUT)
    
    INTEGER             KK_1, KK_2, I
    REAL (KND)  TEMP_MIX(NL_CLIM,9), WO3_MIX(NL_CLIM,9)
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ
    
    ! IUP climatology:tropics,nhmidlatitude winter/spring,nhmidlatitude
    ! summer/fall,shmidlatitude winter/spring,shmidlatitude summer/fall,
    ! nhpolar winter/spring,nhpolar summer/fall,shpolar winter/spring,
    ! and shpolar summer/fall.(NINE CASES)
    DO I = 1, 9
       CALL READ_CLIM (IUP_FILE(I)%S, NPROF(I), O3_TOTAL_COLUMN, ZZ, &
            WO3_MIX(:,I), TEMP_MIX(:,I))
    END DO

    ! ------------------------------------------------------------------
    ! Mix profiles depending upon the latitude and month
    ! ------------------------------------------------------------------
    IF(LATITUDE < 0.0D0) THEN
       KK_1 = 3 ;  KK_2 = 1
    ELSE
       KK_1 = 0 ;  KK_2 = 0
    END IF
    
    SELECT CASE (FLOOR(DABS(LATITUDE)))
    CASE(:24)
       wo3_out(:) = wo3_mix(:,1) 
       tt(:) = temp_mix(:,1)   
       
    CASE(25:34)
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120) 
          wo3_out(:) = 0.5*(wo3_mix(:,1) + wo3_mix(:,2 + KK_1))
          tt(:) =  0.5*(temp_mix(:,1) + temp_mix(:,2 + KK_1))
       CASE(182:304)
          wo3_out(:)=0.5*(wo3_mix(:,1) + wo3_mix(:,3 + KK_2))
          tt(:) = 0.5*(temp_mix(:,1) + temp_mix(:,3 + KK_2))  
       CASE DEFAULT
          wo3_out(:) = (wo3_mix(:,1) + wo3_mix(:,2 + KK_1) + &
               wo3_mix(:,3 + KK_2))/3.0D0
          tt(:) = (temp_mix(:,1) + temp_mix(:,2 + KK_1) + &
               temp_mix(:,3 + KK_2))/3.0D0
       END SELECT
       
    CASE(35:54)
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = wo3_mix(:,2 + KK_1) 
          tt(:) = temp_mix(:,2 + KK_1)  
       CASE(182:304)
          wo3_out(:) = wo3_mix(:,3 + KK_2) 
          tt(:) = temp_mix(:,3 + KK_2)
       CASE DEFAULT
          wo3_out(:) = 0.5*(wo3_mix(:,2 + KK_1) + wo3_mix(:,3 + KK_2))
          tt(:) = 0.5*(temp_mix(:,2 + KK_1) + temp_mix(:,3 + KK_2)) 
       END SELECT
       
    CASE(55:64)
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = 0.5*(wo3_mix(:,2 + KK_1) + wo3_mix(:,6 + KK_1))
          tt(:) = 0.5*(temp_mix(:,2 + KK_1) + temp_mix(:,6 + KK_1))
       CASE(182:304)
          wo3_out(:) = 0.5*(wo3_mix(:,3 + KK_2) + wo3_mix(:,7 + KK_2))
          tt(:) = 0.5*(temp_mix(:,3 + KK_2) + temp_mix(:,7 + KK_2))  
       CASE DEFAULT
          wo3_out(:) = 0.25*(wo3_mix(:,2 + KK_1) + wo3_mix(:,3 + KK_2) &
               + wo3_mix(:,6 + KK_1) + wo3_mix(:,7 + KK_2))
          tt(:) = 0.25*(temp_mix(:,2 + KK_1) + temp_mix(:,3 + KK_2) + &
               temp_mix(:,6 + KK_1) + temp_mix(:,7 + KK_2)) 
       END SELECT

    CASE(65:)
       SELECT CASE (DAY_OF_YEAR)
       CASE(:120)
          wo3_out(:) = wo3_mix(:,6 + KK_1) 
          tt(:) = temp_mix(:,6 + KK_1)
       CASE(182:304)
          wo3_out(:) = wo3_mix(:,7 + KK_2) 
          tt(:) = temp_mix(:,7 + KK_2)  
       CASE DEFAULT
          wo3_out(:) = 0.5*(wo3_mix(:,6 + KK_1) + wo3_mix(:,7 + KK_2))
          tt(:) = 0.5*(temp_mix(:,6 + KK_1) + temp_mix(:,7 + KK_2)) 
       END SELECT
       
    END SELECT
      
    RETURN
  END SUBROUTINE IUP_CLIMATOLOGY_NOINTERP
  
  !**************************************************************************
  SUBROUTINE READ_CLIM (FL_N, N_P, TOZ, HT, OZ_MX, T_MX) 
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_CLIM'
    
    INTEGER             K, I, N_P, IDX, KDUM
    CHARACTER (LEN=80)  HEADERLINE 
    CHARACTER (LEN=*)   FL_N  
    REAL (KND)   HT(NL_CLIM), OZ_MX(NL_CLIM), T_MX(NL_CLIM)
    REAL (KND)    O3_COLUMN(N_P)
    REAL (KND), DIMENSION (NL_CLIM,N_P) ::TEMP, WO3
    REAL (KND)    TOZ
    
    IF (DO_LOGFILE) WRITE(LUNIT,*) ' *** Reading: '//TRIM(fl_n) 
    OPEN (C_UNIT, FILE=FL_N, STATUS='OLD')              
    ! -----------------------------------------
    ! Skip headerlines
    ! -----------------------------------------
    DO
       READ(C_UNIT,'(A)') HEADERLINE
       IF (HEADERLINE(1:1).NE.';') EXIT
    END DO
    BACKSPACE C_UNIT
    ! -----------------------------------------
    ! Read total ozone column of ozone profile
    ! -----------------------------------------
    DO I = 1, N_P
       READ(C_UNIT,*) O3_COLUMN(I)
       DO K = NL_CLIM, 1, -1
          READ(C_UNIT,*) KDUM, HT(K), TEMP(K,I), WO3(K,I)
       ENDDO
    END DO
    
    CLOSE(C_UNIT)
    
    Interpolation: Block 
      If (TOZ <= O3_COLUMN(1)) Then
         Forall(K = 1:NL_CLIM) 
            OZ_MX(K) = WO3(K,1)
            T_MX(K) = TEMP(K,1)
         End Forall
         Exit Interpolation
      End If

      If(TOZ  >= O3_COLUMN(N_P)) Then
         FORALL(K = 1:NL_CLIM) 
            OZ_MX(K) = WO3(K,N_P)
            T_MX(K) = TEMP(K,N_P)
         End Forall
         Exit Interpolation
      End If

      DO I = 1, N_P - 1
         IF(TOZ >= O3_COLUMN(I).AND.TOZ < O3_COLUMN(I+1)) IDX = I
      END DO
      FORALL(K = 1:NL_CLIM)
         OZ_MX(K) = WO3(K,IDX) + (TOZ - O3_COLUMN(IDX))*&
              (WO3(K,IDX + 1) - WO3(K,IDX))/(O3_COLUMN(IDX + 1) - O3_COLUMN(IDX))
         
         T_MX(K) = TEMP(K,IDX) + (TOZ - O3_COLUMN(IDX))*&
              (TEMP(K,IDX + 1) - TEMP(K,IDX))/&
              (O3_COLUMN(IDX + 1) - O3_COLUMN(IDX))
      END FORALL
    End Block Interpolation

     RETURN
  END SUBROUTINE READ_CLIM
  
  !**************************************************************************
  SUBROUTINE TOMS_CLIMATOLOGY (ZZ, TT, WO3_OUT)
    
    REAL (KND)  TEMP_MIX(NL_CLIM,9), WO3_MIX(NL_CLIM,9)
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ

    DO I = 1, 3
       CALL READ_CLIM (TOMS_FILE(I)%S, TPROF(I), O3_TOTAL_COLUMN, ZZ, &
            WO3_MIX(:,I), TEMP_MIX(:,I))
    END DO

    ! ------------------------------------------------------------------
    ! Mix profiles depending upon the latitude
    ! ------------------------------------------------------------------
    SELECT CASE (FLOOR(DABS(LATITUDE)))
    CASE(:24)
       wo3_out(:) = wo3_mix(:,1) 
       tt(:) = temp_mix(:,1)   
       
    CASE(25:34)
       wo3_out(:)=0.5*(wo3_mix(:,1) + wo3_mix(:,2)) 
       tt(:) =  0.5 * temp_mix(:,1) + 0.5 * temp_mix(:,2)  
       
    CASE(35:54)
       wo3_out(:) = wo3_mix(:,2) 
       tt(:) =  temp_mix(:,2)
       
    CASE(55:64)
       wo3_out(:)=0.5*wo3_mix(:,2) + 0.5*wo3_mix(:,3) 
       tt(:) =  0.5*(temp_mix(:,2) + temp_mix(:,3))
       
    CASE(65:)
       wo3_out(:) = wo3_mix(:,3) 
       tt(:) =  temp_mix(:,3) 
    END SELECT
    
    RETURN
  END SUBROUTINE TOMS_CLIMATOLOGY
  
  
  !**************************************************************************
  SUBROUTINE TOMSV8_CLIMATOLOGY (ZZ, TT, WO3_OUT)

    INTEGER  IMON, ILAT, ICL, ERST_POS, N_PROF, LAST_POS
    INTEGER  IDX, K, I, S_LAT,S_MN, S_DY, ILEV, MY_VAL(10)
    INTEGER, PARAMETER :: N_MON = 12, N_LAT = 18
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ

    REAL (KND)  O3(12,18,NL_CLIM,10), TEMP(12,18,NL_CLIM)
    REAL (KND)  O3_COLUMN(10)
      
    ! Read TOMS V8 ozone and CIRA temperature climatologies.
    IF (DO_LOGFILE) WRITE(LUNIT,*) ' *** Reading: '//TRIM(TOMSV8_FILE)
    OPEN (C_UNIT, FILE = TOMSV8_FILE, STATUS = 'OLD')
    do imon = 1,n_mon
       do ilat = 1,n_lat
          read(C_UNIT,*)
          read(C_UNIT,*)
          do ilev = 1, NL_CLIM
             read(C_UNIT,*) ZZ(ilev), temp(imon,ilat,ilev), (o3(imon,ilat,ilev,icl),icl = 1,10)
          end do
       end do
    end do
    CLOSE(C_UNIT)

    ! ozone cases of TOMS column classified climatology
    FORALL(ICL = 1:10) O3_COLUMN(ICL) = 125. + (icl - 1) * 50.
    
    CALL CALDAY(DAY_OF_YEAR, YEAR, S_MN, S_DY)
    CALL LATBAND(LATITUDE, S_LAT)
    
    TT(:) = TEMP(S_MN,S_LAT,:)
    
    !  check if the values are 999.000
    where(o3(s_mn,s_lat,1,1:10) .ne. 999.000)
       my_val = 1
    elsewhere 
       my_val = 0
    endwhere
    n_prof = count(my_val == 1)
    erst_pos = maxloc(my_val,1)
    last_pos = erst_pos + n_prof - 1
    
    Interpolation: Block 
      If (O3_TOTAL_COLUMN <= O3_COLUMN(ERST_POS)) Then
         Forall(K = 1:NL_CLIM) WO3_OUT(K) = O3(S_MN,S_LAT,K,ERST_POS)
         Exit Interpolation
      End If
      
      If(O3_TOTAL_COLUMN  >= O3_COLUMN(LAST_POS)) Then
         FORALL(K = 1:NL_CLIM) WO3_OUT(K) = O3(S_MN,S_LAT,K,LAST_POS)
         Exit Interpolation
      End If
      
      DO I = 1, N_PROF - 1
         IF(O3_TOTAL_COLUMN.GE.O3_COLUMN(ERST_POS - 1 + I) &
              .AND.O3_TOTAL_COLUMN.LT.O3_COLUMN(ERST_POS + I)) &
              IDX = ERST_POS - 1 + I
      END DO
    
      FORALL(K = 1:NL_CLIM) WO3_OUT(K) = O3(S_MN,S_LAT,K,IDX) + &
              (O3_TOTAL_COLUMN - O3_COLUMN(IDX))*&
              (O3(S_MN,S_LAT,K,IDX + 1) - O3(S_MN,S_LAT,K,IDX))/&
              (O3_COLUMN(IDX + 1) - O3_COLUMN(IDX))
    End Block Interpolation
    
    RETURN
  END SUBROUTINE TOMSV8_CLIMATOLOGY

  !**************************************************************************
  SUBROUTINE KNMI_CLIMATOLOGY (ZZ, TT, WO3_OUT) 
    
    INTEGER             IMON, ILEV, I_LAT, MN, DY, ILAT
    INTEGER, PARAMETER :: N_MON  = 12
    
    REAL (KND)    O3(12,NL_CLIM,17), TEMP(12,NL_CLIM,17)
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ
    REAL (KND)    COLUMN, SC_FACTOR
    REAL (KND), PARAMETER :: KBOLTZ = 1.38E-23

    ! read KNMI ozone and CIRA temperature climatologies.
    IF (DO_LOGFILE) WRITE(LUNIT,*) ' *** Reading: '//TRIM(KNMI_FILE)
    OPEN (C_UNIT,FILE = KNMI_FILE,STATUS = 'OLD')
    read(C_UNIT,*)
    do imon = 1, n_mon
       read(C_UNIT,*) 
       read(C_UNIT,*)
       do ilev = 1, NL_CLIM
          read(C_UNIT,*) ZZ(ilev),(o3(imon,ilev,ilat),ilat=1,17)
       end do
       read(C_UNIT,*)
       do ilev = 1, NL_CLIM
          read(C_UNIT,*) ZZ(ilev),(temp(imon,ilev,ilat),ilat=1,17)
       end do
    end do
    close(C_UNIT)
       
    call calday(DAY_OF_YEAR, YEAR, mn, dy)
    call klatband(LATITUDE, i_lat)
    
    TT(:) = TEMP(MN,:,I_LAT)
    WO3_OUT(:) = O3(MN,:,I_LAT)
       
    RETURN
  END SUBROUTINE KNMI_CLIMATOLOGY

  !**************************************************************************
  SUBROUTINE TOMS_MONTHLY_CLIM (ZZ, TT, WO3_OUT) 
    
    INTEGER             IMON, ILEV, I_LAT, MN, DY, ILAT
    INTEGER, PARAMETER :: N_MON  = 12
    
    REAL (KND)   O3(12,NL_CLIM,18), TEMP(12,NL_CLIM,18)
    REAL (KND), DIMENSION (NL_CLIM) :: WO3_OUT, TT, ZZ
    REAL (KND)    COLUMN, SC_FACTOR
    REAL (KND), PARAMETER :: KBOLTZ = 1.38D-23

    !     read KNMI ozone and CIRA temperature climatologies.
    IF (DO_LOGFILE) WRITE(LUNIT,*) ' *** Reading: '//TRIM(TOMS_MON_FILE)
    OPEN (C_UNIT,FILE = TOMS_MON_FILE,STATUS = 'OLD')
    read(C_UNIT,*)
    do imon = 1, n_mon
       read(C_UNIT,*) 
       read(C_UNIT,*)
       do ilev = 1,NL_CLIM
          read(C_UNIT,*) zz(ilev),(o3(imon,ilev,ilat),ilat=1,18)
       end do
       read(C_UNIT,*)
       do ilev = 1,NL_CLIM
          read(C_UNIT,*) zz(ilev),(temp(imon,ilev,ilat),ilat=1,18)
       end do
    end do
    close(C_UNIT)
     
    call calday(DAY_OF_YEAR, YEAR,mn,dy)
    call latband(LATITUDE,i_lat)
    
    TT(:) = TEMP(MN,:,I_LAT)
    WO3_OUT(:) = O3(MN,:,I_LAT)
       
    RETURN
  END SUBROUTINE TOMS_MONTHLY_CLIM
  
!**************************************************************************
END SUBROUTINE CLIM_IFACE
!**************************************************************************
! find the proper profile based on the latitude -80 to 80.
subroutine klatband(latitude, iband) 
  Use Types, Only: KND
  
  REAL (KND)  latitude, width, min_lat
  integer          iband
  
  min_lat = -80.0
  width = 10.0
  
  if (latitude .lt. -85.0) iband = 1
  
  if (latitude .ge. -85.0 .and. latitude .lt. 85.0) &
       iband = NINT((latitude + width - min_lat)/width) 
      
  if (latitude .ge. 85.0) iband = 17
 
  return
end subroutine klatband

!**************************************************************************
! find the proper profile based on the latitude -90 to 90.
subroutine latband(g_lat,iband) 

  Use Types, Only: KND

  REAL (KND)  g_lat, bands(19)    
  integer                 iband,j
  bands = (/-90.,-80.,-70.,-60.,-50.,-40.,-30.,-20.,-10.,0., 10.,20.,30.,40.,50.,60.,70.,80.,90./)

  do j = 1, 18 
     if (g_lat .ge. bands(j) .and. g_lat .le. bands(j+1)) iband = j
  end do
  if (g_lat .gt. 89.9) then
     iband = 18
  end if
  return
end subroutine latband

!**************************************************************************
! find the month by using the day of the year.
subroutine calday( jday, year, month, day ) 
  
  integer  jday, year, month, day, i, lp
  integer  mndys(12)    
  mndys = (/0,31,59,90,120,151,181,212,243,273,304,334/)
  
  if ( MOD(year,400) == 0 .or.(MOD(year,4) == 0 .and. MOD(year,100) /= 0) ) then
     lp = 1
  else
     lp = 0
  end if
  
  do i = 1, 2
     if ( jday > mndys(i) ) month = i
  end do
  do i = 3, 12
     if ( jday > mndys(i)+lp ) month = i
  end do
  
  day = jday - mndys(month)
  if ( month > 2 ) day = day - lp
  
  return
end subroutine calday
!**************************************************************************

