!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 23 Aug 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine Command_Line_Arguments (list)
  Use OutInf,  Only: OutMode
  Use St_Vars, only: CI_FLG, CI_DIR
  Use Tools,   only: Split_String, Upper_Lower_Case, CUT_STR, Read_Text
  Use Types,   only: DYN_CHAR, DYN_CHAR_L

  Type (DYN_CHAR_L), allocatable :: list(:)
  
  ! Local parameters
  integer  n_args, info, I_Arg, Idx, n_list
  Logical  ListFound
  character (len = 300) str_read
  Type (DYN_CHAR), allocatable :: Str_Arr(:)
  
  ! Allowed command line arguments:
  ! Without flags: name of the file containing the working directory list or
  !                name of the working directory
  ! With flags:
  !                LIST='name of the file containing the working directory list'
  !                CID='common input directory name'
  !                Mode='HPC' - high performance computing mode, reduces the 
  !                             number of output files 
  !==================================================================
  
  n_args = COMMAND_ARGUMENT_COUNT() 

  CI_FLG = .False. ; ListFound = .False. ; OutMode = ' '

  if(n_args > 0) then
     Do I_Arg = 1, n_args
        CALL get_command_argument(I_Arg, str_read)
        Idx = Index(str_read, '=')
        If (Idx == 0) Then
           ! Old-style command line arguments, the argument might be eigher
           ! a directory name or a file name (only one argument is allowed)
           Call Read_Text (trim(Str_Read), List, info)
           if(info /= 0) then ! The argument is not a file name, assume it is a name of the directory
              n_list = 1
              allocate(list(n_list))
              list(1)%S = Trim(Str_Read)
              list(1)%L = Len_Trim(Str_Read)
           end if
           ListFound = .True.
           exit ! Therminate loop as only one argument is allowed
        End If
        Call Split_String(Trim(Str_Read), Str_Arr, '=')
        Call Upper_Lower_Case (STR_ARR(1)%S  ,'U2L')  
        Select Case (Str_Arr(1)%S)
        Case ('dir')
           ListFound = .True.
           allocate(list(1))
           list(1)%S = Trim(STR_ARR(2)%S)
           list(1)%L = Len_Trim(STR_ARR(2)%S)
        Case ('list')
           ListFound = .True.
           Call Read_Text (trim(Str_Arr(2)%S), List, info, SkipEmpty=.True.)
        Case ('cid')
           CI_FLG = .TRUE.
           CI_DIR = CUT_STR(Str_Arr(2)%S)
        Case ('mode')
           Call Upper_Lower_Case (STR_ARR(2)%S  ,'L2U')
           If(STR_ARR(2)%S == 'HPC') OutMode = STR_ARR(2)%S
        End Select
     End Do
  end if

  If(.Not.ListFound) Then  ! No list of directories is provided
     n_list = 1
     allocate(list(n_list))
     list(1)%S = './'
     list(1)%L = Len_Trim(list(1)%S)
  End If

  return
!**************************************************************************
End Subroutine Command_Line_Arguments
!**************************************************************************
