!********************************************************************C
!
!	SCIATRAN - Version 4.0 - Last modification: 28 Aug 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!********************************************************************C
SUBROUTINE CREATE_INFO_ARR (N_ADD)
! ... called by "gt_iface.f" and "ret_control.f"
  USE AER_MIE,    ONLY: M_AC
  USE ALB_PARS,   ONLY: Nc_ALB
  USE CLOUDS,     ONLY: N_CLD_LAYERS
  USE INFO,       ONLY: RET_INFO, TG_INFO, AER_INFO, PAR_INFO, CLD_INFO
  USE ST_VARS,    ONLY: SURF_TYPE
  USE TOOLS,      ONLY: SPLIT_STRING, FIND_IDX
  USE TYPES,      ONLY: DYN_CHAR

  INTEGER  N_ADD
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'CREATE_INFO_ARR'
  !                    See "mie_aer_setup.f"
  INTEGER  NN, II, KK
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:) 
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')
  CALL DEALLOCATE_ARRAYS
  
  NN = COUNT(TG_INFO(:)%WF_FLG) + COUNT(AER_INFO(:)%WF_FLG) + &
       COUNT(PAR_INFO(:)%WF_FLG) + COUNT(CLD_INFO(:)%WF_FLG)

  ALLOCATE(RET_INFO(NN + N_ADD))
  KK = 0
  DO II = 1, UBOUND(TG_INFO,1)
     IF(.NOT.TG_INFO(II)%WF_FLG) CYCLE
     KK = KK + 1
     RET_INFO(KK)%NAME = TG_INFO(II)%NAME
     RET_INFO(KK)%LONG_NAME = TG_INFO(II)%LONG_NAME
     RET_INFO(KK)%WF_DIM = TG_INFO(II)%WF_DIM
     IF( TG_INFO(II)%M_TYPE == 'atom' ) THEN
        RET_INFO(KK)%TYPE = 'atom'
     ELSE
        RET_INFO(KK)%TYPE = 'tg'
     ENDIF
     RET_INFO(KK)%VS = 'V'
  END DO
  DO II = 1, UBOUND(AER_INFO,1)
     IF(.NOT.AER_INFO(II)%WF_FLG) CYCLE
     KK = KK + 1
     RET_INFO(KK)%NAME = AER_INFO(II)%NAME
     RET_INFO(KK)%LONG_NAME = AER_INFO(II)%LONG_NAME
     RET_INFO(KK)%WF_DIM = AER_INFO(II)%WF_DIM
     RET_INFO(KK)%TYPE = 'aer'
     RET_INFO(KK)%VS = AER_INFO(II)%VSI
  END DO
  
!  write(*,*) '  UBOUND(PAR_INFO,1) ', UBOUND(PAR_INFO,1)
  DO II = 1, UBOUND(PAR_INFO,1)
     IF(.NOT.PAR_INFO(II)%WF_FLG) CYCLE
     KK = KK + 1
!     write(*,*) '    PAR_INFO(II)%NAME  ', ii, kk, PAR_INFO(II)%NAME
!     write(*,*) '  PAR_INFO(II)%WF_FLG) ', ii, kk, PAR_INFO(II)%WF_FLG
     RET_INFO(KK)%NAME      = PAR_INFO(II)%NAME
     RET_INFO(KK)%LONG_NAME = PAR_INFO(II)%LONG_NAME
     RET_INFO(KK)%WF_DIM    = PAR_INFO(II)%WF_DIM
     CALL SPLIT_STRING(PAR_INFO(II)%NAME, STR_ARR, '_' )
     SELECT CASE( STR_ARR(1)%S )
     CASE('prosail')
        RET_INFO(KK)%TYPE = STR_ARR(1)%S
     CASE DEFAULT
        RET_INFO(KK)%TYPE = 'atm_p'
     END SELECT
     RET_INFO(KK)%VS = PAR_INFO(II)%VSI
  END DO
  !  pause '   in create_info_arr'
  DO II = 1, UBOUND(CLD_INFO,1)
     IF(.NOT.CLD_INFO(II)%WF_FLG) CYCLE
     KK = KK + 1
     RET_INFO(KK)%NAME = CLD_INFO(II)%NAME
     RET_INFO(KK)%LONG_NAME = CLD_INFO(II)%LONG_NAME
     RET_INFO(KK)%WF_DIM = CLD_INFO(II)%WF_DIM
     RET_INFO(KK)%VS = CLD_INFO(II)%VSI
     IF(CLD_INFO(II)%VSI == 'S') THEN
        RET_INFO(KK)%TYPE = 'cld-s'
     ELSE
        RET_INFO(KK)%TYPE = 'cld-v'
     END IF
  END DO

  DO II = NN + 1, UBOUND(RET_INFO,1)
     KK = KK + 1
     RET_INFO(KK)%NAME = 'undefined'
     RET_INFO(KK)%LONG_NAME = 'undefined'
     RET_INFO(KK)%TYPE = ''
     RET_INFO(KK)%VS = ''
  END DO 

  RET_INFO(:)%STAT = .TRUE. ! Parameter switched on  
  RET_INFO(:)%WF_ON = .TRUE. ! Weighting function needs to be calculated
! The name of the weighting function is the same as the parameter name
  RET_INFO(:)%WF_NAME = RET_INFO(:)%NAME 
! No limits for the parameters is set (used in shi_squ routine)
  RET_INFO(:)%PAR_MAX = +1.0d+20
  RET_INFO(:)%PAR_MIN = -1.0d+20
! Number of sub-parameters is set to 1
  RET_INFO(:)%N_SBP = 1

!  II = FIND_IDX('albedo', RET_INFO%NAME)
!  WRITE(*,*) '    SURF_TYPE   ', SURF_TYPE
!  SELECT CASE( TRIM(SURF_TYPE) )
!  CASE('Ross-Li')      ; RET_INFO(II)%N_SBP = 3
!  CASE('Ross-Li-snow') ; RET_INFO(II)%N_SBP = 4
!  END SELECT
!  WRITE(*,*) '   II = ', II
!  WRITE(*,*) '     RET_INFO(II)%N_SBP = ', RET_INFO(II)%N_SBP
  
  DO II = 1, UBOUND(RET_INFO,1)
     RET_INFO(II)%RP = 'undefined'
     IF(RET_INFO(II)%TYPE == 'cld-s') THEN
        ! For scalar cloud parameters each cloud layer is a sub-parameter
        RET_INFO(II)%N_SBP = N_CLD_LAYERS 
     END IF
!     IF( RET_INFO(II)%NAME == 'albedo' ) THEN
!        ! Each albedo fit function is a sub-parameter        
!        RET_INFO(II)%N_SBP = Nc_ALB 
!     ENDIF
     ! RET_INFO%I_S and RET_INFO%I_E are defined in rtm_output.f90
     ALLOCATE(RET_INFO(II)%VSI(RET_INFO(II)%N_SBP))
     IF(RET_INFO(II)%N_SBP > 1.AND.RET_INFO(II)%VS /= 'S') THEN
        ! Here selection of several altitude ranges from a parameter
        ! can be done
        STOP 'Not yet implemented !'
     ELSE
        RET_INFO(II)%VSI(:) = RET_INFO(II)%VS
     END IF

     SELECT CASE( RET_INFO(II)%NAME )
     CASE('aer-modr_smod','aer-modr_f','aer-modr_c', &
          'aer-var_smod','aer-var_f','aer-var_c')
       RET_INFO(II)%N_PARS = M_AC
!       RET_INFO(II)%N_PARS = 5
    CASE DEFAULT
       RET_INFO(II)%N_PARS = 1
    END SELECT

     RET_INFO(II)%Z_L = -9999. ; RET_INFO(II)%Z_U = 9999. 
     RET_INFO(II)%WL_SEL = .FALSE.
  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

CONTAINS
!***************************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    Integer JJ
    
    IF(ALLOCATED(RET_INFO)) THEN
       DO II = 1, SIZE(RET_INFO)
          IF(ALLOCATED(RET_INFO(II)%PRE)) DEALLOCATE(RET_INFO(II)%PRE, &
               RET_INFO(II)%APR, RET_INFO(II)%FACTOR) 
          IF(ALLOCATED(RET_INFO(II)%SEG_S)) DEALLOCATE( & 
               RET_INFO(II)%SEG_S, RET_INFO(II)%SEG_E, RET_INFO(II)%WL_MIN, &
               RET_INFO(II)%WL_MAX, RET_INFO(II)%DS_FLG, RET_INFO(II)%VSI)
          If(Allocated(RET_INFO(II)%REJ_LOS)) Then
             Do JJ =1, Size(RET_INFO(II)%REJ_LOS)
                DeAllocate(RET_INFO(II)%REJ_LOS(JJ)%Flg)
             End Do
             DeAllocate(RET_INFO(II)%REJ_LOS)
          End If
       END DO
       DEALLOCATE(RET_INFO)
    END IF
       
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***************************************************************************
 END SUBROUTINE CREATE_INFO_ARR
!***************************************************************************
