!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 25 Nov 2023
!
!	Institute of Remote Sensing, University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE DOM_SPHERICAL( LPOS, POS, SRR, SVR, SRR_JACT_INP )
  ! Called by "rtm_main.f"
  
  USE ABSORBERS, ONLY: N_GAS, ST_GAS_IDX, N_ESFT_MOL, AMF_IDX, GAS_INDEX
  USE AEROSOLS,  ONLY: LA_N, NG_aer, Plus_mi, Dwr_mi, N_dwr, Mie_layer, &
                         N_AOT, AOT_MAL, LA_S, LA_E, AER_COEFF
  USE ALT_INPUT, ONLY: ALIN_ACT, ALIN_RAY
  USE ANISOTROPY,ONLY: DEP_NWS, DEP_WS, DEP_VS
  USE BRDF,      ONLY: Mh_BRF, Ref_type, Nwl_BRF, WL_BRF, RefW_r, RefW_i, Nder
  USE CLOUDS,    ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT, NG_cld, CLD_F_DM,&
                       CLD_DM, PROF_VARS, NCW => N_CLD_NETWAVS, PR_SOOT, &
                       Prof_Cpar, CLW => CLD_NETWAVS
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE CLD_INP,   ONLY: Plus_ice, ADD_ABS, Nphr, Phm_const, Deriv_phc, &
                       Use_Ydb_SCM, Ydb_ang, MLCS
  USE DATA_RET,  ONLY: RET_FLAG
  USE EMISSION,  ONLY: N_AT, Emi_MOMS
  USE ESFT,      ONLY: N_ESFT_COEFF, DO_ESFT_SPEC_MOL, DO_ESFT
  USE FLU_LAND,  ONLY: FLU_L, FLU_SCAL, NFL, WLF, SPF, USE_DGF, DGF_PAR, &
                       GAU, CONVERT_AMP
  USE GRIDS,     ONLY: NLAYERS, DZR, DZ, SOL_GRD, NL_FINE, GRID_MARK, ZZZ, &
                       NSZA => NSOL, Z_FINE, GRID_MARK_UW, INTERP_IDX, &
                       ALT_EXT, Z_INV
  USE GT_POLYN,  ONLY: NLEGEN, XM, AXM, AG, XM_W, AXM_W, AG_W
  USE HYDROSOLS, ONLY: LA_N_HS!, N_HPC
  USE INFO,      ONLY: TG_INFO, WF_DB
  USE I_DIMS,    ONLY: NL_PC, NL_ice, Nm_ice, Nlg_Ray, Ns_SFN, NL_MS, NSOL_MS,&
                       N_SFN, NL_AER, NL_CLD, NLG_AER, NLG_CLD, NLG_ALL_A,&
                       NLG_ALL_C, NL_fine_pla, Nm_vs, NL_pla, Nf_pla, Nl_Cr,&
                       Ng_Cr, Nls_Cr, Nso_Cr, N_C2, Nl_Ar, Ng_Ar, Nls_Ar,&
                       Nso_Ar, N_A2, Ng1_brf, Ng2_brf, Ns_brf, Nh_brf, Nvs, &
                       NL_soot, Nlg_ray_W, Nm_vs_W, NN_pla
  USE IRRAD,     ONLY: Fbe, SOL_FLUX, WL_IRRAD_DIM, NO_SUN
  USE LBL,       ONLY: DO_LBL, Q_RATIO
  USE LOG_PARS,  ONLY: DIR_OUT, EU, LOG_LEVEL, N_RSO, N_ESO, N_XSO, Work_Dir, &
                       LUNIT, C_UNIT
  USE LOS_ANGLES,ONLY: N_below
  USE NODES,     ONLY: N_NODE
  USE PROFILES,  ONLY: PRESS, TEMP, AIR_DENSITY
  USE RAMAN,     ONLY: N_RG, R_GAM, Sigma_RR, WL_FACT, N_WL_C, DEL_WL,&
                       LINE_GAS, RRS_ADJ, RRS_DO
  USE RETR_ITER, ONLY: ITERATION
  USE RTM_OUTPUT,ONLY: TG_VOD, WF_TO_AMF, Wup, N_OP, VERT_COL
  USE SLAB_2L,   ONLY: Ome0, TES_2L_SLAB
  USE ST_VARS,   ONLY: DO_AEROSOL, INT_MS_FLAG, RTM_MODE, Pie, RTM_TYPE, &
                       RAD_TO_DEG, TSM, F_LGI_EXT, ADAPTIVE_GRID, CRIT_AG, &
                       RTM_CORE, DO_RAMAN, N_Eff, Enhance_acc, Nh_s, &
                       BRDF_FLAG, N_RHS, ARB_OL, RAD_FIELD, WF_FLAG, DIFB, &
                       SPECULAR, SOLVER, NS_wf, Emi_FLG, N_HARM, OWF_FLAG, &
                       SCATT_MODE, WL_RTM, INEL_WHERE,SURF_TYPE, ERROR, &
                       GeoD => GeoD_Out, AG_aer, Mh_AG, CRIT_AG_Mh, F_LGI_SCR, &
                       F_LGI_SCA, AKIMA_FLG, DO_PLANCK, NSTRMS, SOL_ANG, &
                       Sec_order, Finc, Kvs
  USE SPHER_PARS,ONLY: NOA => N_OBS_ALT, Naz => N_AZ_ANG, SET_OUT_ALT, &
                       ALTITUDE, NLSA => N_LOS_ANG, ANG_OUT, &
                       NLO => N_LOS_OUT, LW_FOV, FOV_WEIGHTS, &
                       NFA => N_FOV_ANG, ANG_SEL_FLAG, Nf => NN, JV_S, JV_E
  USE TOOLS,     ONLY: INQ_WF, E_CSZA
  USE TYPES,     ONLY: KND, PWF_TYPE, LOA_type, LOW_TYPE, TE_TYPE, UDB_TYPE, &
                       SSB_TYPE, GSL_TYPE, RRS_TYPE, PCC_TYPE, OSM_TYPE, &
                       AGR_type, LWF_TYPE, VRS_TYPE, PHA_MATR, SWF_type, &
                       SLS_type, AuxDataInterp
  USE WATER,     ONLY: UWT, NL_W => NLAYERS_UW, NN_W => NSTR_W, NLEG_W, &
                       NLSA_W, NSZA_W, ZZZ_UW, A_RTmat, NSTR_T, Ref_ind, &
                       Obs_POSITION, N_ABS_UW, Na_W, INCL_INEL, Sol_off, &
                       INEL_ADJ, Wei_dif, ERDR 
  USE WATER_ICE, ONLY: IWS, N_IW, LA_N_IW, IWT, LoIW
  USE WF_NUM,    ONLY: NUM_WF, NUM_RAY, NUM_AER, AL_BASE, NUM_PRE, NUM_CLD, &
                       PERTURB, AL_1_PERT, NUM_TEM, SCA_or_ABS, Wat_or_Ice, &
                       AL_1_BASE
  !------------------------------------------------------------------
  
  INTEGER, INTENT(IN) :: LPOS
  INTEGER, OPTIONAL :: SRR_JACT_INP
  !-----------------------------------------      
  ! Local parameters
  !-----------------
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'DOM_SPHERICAL'
  CHARACTER (LEN=72) Msg(10)
  INTEGER    MHARM, I, J, K, L, Js, NESFT, NESFT_IDX, IOB, I_GAS, Jmc, Ks, &
       K1, NWv, NAv, SRR_Jact
  INTEGER    JESFT, JJESFT, E_IDX(2*N_ESFT_COEFF,N_ESFT_MOL)
  LOGICAL    DO_OUT, DO_CONV1, ESFT_FLAG_LOC, COMPARE_TO_DISORT
  LOGICAL    VC_FLAG, SAV_WEI
  REAL (KND) E_W, TG_CS(NLAYERS,N_GAS), TAC(NLAYERS,N_GAS), FLS(1)
  REAL (KND) TG_CROSS_SEC(NLAYERS,N_GAS), DEPOL, DEP_W, POS, Int_dir, &
       Dir_atm, Dir_oce, Int_oce
  REAL (KND) Abs_soot(NL_soot) !, ALBEDO
  REAL (KND), DIMENSION (NLAYERS) :: TGA, SCA_AL, FFF 
  ! From the structure IWS
  INTEGER    N_HPC
  !---------------------------------------
  TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
  TYPE (LOW_type) :: LOW    ! Local optical parameters of water
  TYPE (LWF_type) :: LWF    ! Local weighting functions
  TYPE (OSM_type) :: OSM    ! Original scattering matrices
  TYPE (PWF_type) :: PWF    ! General weighting function 
  TYPE (TE_type)  :: TE     ! Thermal emission
  TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
  TYPE (SSB_type) :: SSB_A, SSB_W ! Spherical solar beam
  TYPE (GSL_type) :: GSL_A, GSL_W ! Angles 
  TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
  TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
  TYPE (PCC_type) :: PCC    ! Photochemistry
  TYPE (AGR_type) :: AGR_A, AGR_W
  TYPE (SWF_type) :: SWF    ! "Spherical" weighting function 
  !-----------------------------
  ! Spherical line-of-sight |
  !-----------------------------
  TYPE (SLS_type) :: SLS
  TYPE (AuxDataInterp), Target :: ADI
  !---------------------------------------------------------------- 
  TYPE (PHA_MATR) PH_M
  INTEGER  Jr, J_Ine, N_PL, NRL, N_OBS_AG, M_max, N_Beam, Iot, KK, M, Ka, IA,&
       IV, LAY_GR(NLAYERS-1,2), Nvi, Ivs, Jv, Ja
  INTEGER, POINTER :: Lst(:), Lfi(:)
  REAL (KND) S1, WL_ST, Wdo, DUM, Fd_ref_Cox, Fd_tra_Cox, Norm
  REAL (KND) EXC_0(NLAYERS), SCC_0(NLAYERS)
  REAL (KND) RADOUT(N_OP,NLO,Nvs), MuS(NSZA), MuV(NLSA), W_LEAV(NLO,Nvs), &
       SCAD(NLAYERS), AL_NDM(NLAYERS), ALW(NLAYERS-1), T_AU(NLAYERS), &
       Fre_RS(NSZA), Tra_co(NSZA), Fre_TS(NSZA,Nvs,Nvs)
  REAL (KND), POINTER :: SCR(:), SCA(:), EXA(:), SCC(:), EXC(:), EXI(:), &
       AL(:), P_SCC(:), P_EXC(:), SCI(:), MuSW(:)
  REAL (KND), ALLOCATABLE :: ALW_W(:), T_AU_W(:), Cor_NT(:), Plh(:), &
       Tra_AW(:,:,:)
  ! Preliminary: radiation field for limb
  REAL (KND), ALLOCATABLE ::  RAD_PS(:,:,:), RADs_PS(:,:,:,:)
  !==================================================================
  INTERFACE
     SUBROUTINE ST_SS_STOKES(JPOS, JJESFT, POS, LOA, E_W, ALBEDO, FLU_int, &
          OSM, TAC, PWF, SRR, SRR_Jact, LSE, IND_LS, TE, ADI, RADOUT)
       USE ABSORBERS,   ONLY: N_GAS
       USE GRIDS,       ONLY: NLAYERS
       USE INTERN_DIMS, ONLY: Nvs
       USE RTM_OUTPUT,  ONLY: N_OP
       USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT, NFA => N_FOV_ANG
       USE ST_VARS,      ONLY: GeoD => GeoD_Out
       USE TYPES,        ONLY: KND, LOA_TYPE, PWF_TYPE, OSM_TYPE, LWF_TYPE, &
                               RRS_TYPE, AuxDataInterp, LSE_type, TE_TYPE
       INTEGER JPOS, JJESFT, SRR_Jact, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
       REAL (KND) E_W, ALBEDO, FLU_int, POS, TAC(NLAYERS,N_GAS), &
            RADOUT(N_OP,NLO,Nvs)
       ! Input structures:
       TYPE (AuxDataInterp), Target :: ADI
       TYPE (LOA_type) :: LOA ! Local optical parameters of atmosphere 
       TYPE (LWF_type) :: LWF ! Local weighting functions
       TYPE (OSM_type) :: OSM ! Original scattering matrices
       TYPE (PWF_type) :: PWF ! General weighting function 
       TYPE (RRS_type) :: SRR ! RRS related parameters
       TYPE(LSE_type)  :: LSE(GeoD%NAlts,GeoD%NVA)
       TYPE (TE_type)  :: TE  ! Thermal emission
         
       INTENT(IN) :: JPOS, JJESFT, IND_LS, ALBEDO, FLU_int, E_W, POS, &
            LOA, OSM, PWF, ADI
       INTENT(INOUT) :: RADOUT, SRR
     END SUBROUTINE ST_SS_STOKES
      
     SUBROUTINE ST_DOM_TAU(JPOS, JJESFT, Iot, Mh, POS, E_W, AGR_A, AGR_W, &
          GSL_A, GSL_W, UDB_A, UDB_W, LOA, LOW, LWF, TAC, OSM, SLS, SSB_A, &
          SSB_W, TE, SWF, PWF, Abs_soot, RAD_PS, RADs_PS, DEPOL, DO_OUT, &
          DO_CONV1, SRR, SRR_Jact, SVR, RADOUT, W_LEAV, PH_M, ADI, *)
       USE ABSORBERS, ONLY: N_GAS
       USE I_DIMS,    ONLY: Nvs, NL_soot
       USE RTM_OUTPUT,ONLY: N_OP
       USE SPHER_PARS,ONLY: N_SL => N_LOS_OUT
       USE ST_VARS,   ONLY: GeoD => GeoD_Out
       USE TYPES,     ONLY: KND, LOA_TYPE, PWF_TYPE, LOW_TYPE, TE_TYPE, &
                            UDB_TYPE, SSB_TYPE, GSL_TYPE, RRS_TYPE, OSM_TYPE, &
                            AGR_type, LWF_TYPE, VRS_TYPE, PHA_MATR, SWF_type, &
                            SLS_type, AuxDataInterp
       INTEGER,    INTENT(IN) :: JPOS, JJESFT, SRR_Jact, Iot, Mh
       TYPE (AGR_type) AGR_A, AGR_W
       TYPE (AuxDataInterp), Target :: ADI
       TYPE (GSL_type) GSL_A, GSL_W 
       TYPE (LOA_type) LOA  
       TYPE (LOW_type) LOW 
       TYPE (LWF_type) LWF 
       TYPE (OSM_type) OSM 
       TYPE (PHA_MATR) PH_M
       TYPE (PWF_type) PWF 
       TYPE (SSB_type) SSB_A, SSB_W 
       TYPE (SLS_type) SLS
       TYPE (RRS_type) SRR 
       TYPE (VRS_type) SVR 
       TYPE (SWF_type) SWF    
       TYPE (TE_type)  TE
       TYPE (UDB_TYPE) UDB_A, UDB_W
       REAL (KND), INTENT(IN) :: E_W, POS, TAC(AGR_A%NL,N_GAS), &
            Abs_soot(NL_soot), DEPOL
       REAL (KND) RADOUT(N_OP,N_SL,Nvs), W_LEAV(N_SL,Nvs)
       REAL (KND), ALLOCATABLE :: RAD_PS(:,:,:), RADs_PS(:,:,:,:)
       LOGICAL DO_OUT, DO_CONV1
     END SUBROUTINE ST_DOM_TAU
  END INTERFACE
  !==================================================================
  
  IF(ANY(ERROR)) RETURN 
      
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ! Changed by A. Rozanov 27.01.2021. Needed for a parallel execution
  If(Present(SRR_JACT_INP)) Then
     SRR_Jact = SRR_JACT_INP
  Else
     SRR_Jact = SRR%Jact
  End If
  
! -----------    Forward-adjoint technique is used --------------
!              RRS_ADJ = .TRUE.     RRS_DO  = .FALSE.    
! ... APPR_RRS='exact matching': 
!     --------------------------
!       SRR_Jact runs from 1 to N_ACT, i.e., over all actual 
!           wavelength. For j-th value of SRR_Jact integer
!       LPOS runs in this routine from 
!           (j-1)*(N_WL_C+1)+1 to j*(N_WL_C+1), 
!           i.e., over j-th actual wavelength and after that
!           over all corresponding excitation wavelengths;
!
! ... APPR_RRS='simple binning':
!     -------------------------
!       This routine will be called at first for all actual 
!       wavelengths. Integer variable 
!          SRR_Jact runs from 1 to N_ACT. 
!                   For j-th value of SRR_Jact integer variable
!          LPOS gives the position of j-th actual wavelength in the
!               extended wavelength grid
! ******************************************************************
!
! -----------    Discrete-ordinates technique is used --------------
!              RRS_ADJ = .FALSE.     RRS_DO  = .TRUE.
! ... APPR_RRS='exact matching':
!     -------------------------
!       SRR%Act_WL(LPOS) = .FALSE. (SRR%Exi_WL(LPOS) = .TRUE.) than
!       SRR%Jexi runs from 1 to N_WL_C i.e., over all excitation 
!           wavelengths for a given actual wavelength. SRR%Jexi
!           is used in routine "SAVE_EXC_RRS" to save all 
!           needed data at excitation wavelengths.
!       LPOS counts corresponding wavelengths in extended WL-grid
!       SRR_Jact does not use.
!
!       SRR%Act_WL(LPOS) = .TRUE. then
!       SRR_Jact counts the actual wavelengths. SRR_Jact
!           is used in routine "gpp_converge" to save end-results 
!           for intensity.
!       LPOS defines position of this actual wavelength in 
!           extended WL-grid
!       SRR%Jexi does not use.
!
! ... APPR_RRS='simple binning':
!     -------------------------
!
! RADOUT = 0.d0
!  write(*,*) '    Check RADOUT 1: '
!  write(*,*) '   SUM(RADOUT(1,:,:)) ',  SUM(RADOUT(1,:,:))
!  WRITE(*,*) '    "dom_spherical"  1 '

  Nav = Nvs*NSTRMS  ;   Nwv = Nvs*NN_W
      
  ALLOCATE( LOA%TG_CS(N_GAS), UDB_A%Ftra(NSZA,Kvs,Nvs) )
  FORALL( Ivs=1:Kvs, J=1:Nvs ) UDB_A%Ftra(:,Ivs,J) = Finc(Ivs,J)
  
  IF( Emi_FLG ) ALLOCATE( LOA%EMC(N_AT,NLAYERS), LOA%EPF(N_AT,Nlg_Ray,Nm_vs),&
          PH_M%MET(N_AT) )
  
  KK = 0 ; Ka = 0
  DO I = 1, SIZE(TG_INFO)
     IF(.NOT.TG_INFO(I)%FM_FLG) CYCLE
     KK = KK + 1
     LOA%TG_CS(KK)%NAME = TG_INFO(I)%NAME
     LOA%TG_CS(KK)%TYPE = TG_INFO(I)%M_TYPE
     
     ALLOCATE( LOA%TG_CS(KK)%CS(NLAYERS) )
     
     IF( LOA%TG_CS(KK)%NAME=='na') THEN
        Ka = Ka + 1
        ALLOCATE( LOA%TG_CS(KK)%E_P(3) )
        ! Position of the current wavelength as compare
        ! to separation wavenumber of D2 and D1 Na lines
        IF( 1.d7/POS > TG_INFO(I)%S_NU ) THEN
           ! Phase function weights for D2 sodium line
           LOA%TG_CS(KK)%E_P(:) = TG_INFO(I)%ALP_EP(2,:)
        ELSE
           ! Phase function weights for D1 sodium line
           LOA%TG_CS(KK)%E_P(:) = TG_INFO(I)%ALP_EP(1,:)
        ENDIF
        PH_M%MET(Ka)%NAME = LOA%TG_CS(KK)%NAME
     ENDIF
     IF( LOA%TG_CS(KK)%NAME=='mgi') THEN
        Ka = Ka + 1
        ALLOCATE( LOA%TG_CS(KK)%E_P(3) )
        IF( 1.d7/POS > TG_INFO(I)%S_NU ) THEN
           LOA%TG_CS(KK)%E_P(:) = TG_INFO(I)%ALP_EP(2,:)
        ELSE
           LOA%TG_CS(KK)%E_P(:) = TG_INFO(I)%ALP_EP(1,:)
        ENDIF
        PH_M%MET(Ka)%NAME = LOA%TG_CS(KK)%NAME
     ENDIF
     IF( LOA%TG_CS(KK)%NAME=='mga') THEN
        Ka = Ka + 1
        ! E1 and E3 the same for emission lines
        ALLOCATE( LOA%TG_CS(KK)%E_P(3) )
        LOA%TG_CS(KK)%E_P(:) = TG_INFO(I)%ALP_EP(1,:)
        PH_M%MET(Ka)%NAME = LOA%TG_CS(KK)%NAME
     ENDIF
  ENDDO   ! over all gaseous absorber (I)
  
  SAV_WEI = DO_SSRCA.OR.SCATT_MODE=='ss'.OR.RTM_TYPE=='spher_scat'
  
  SELECT CASE( TRIM(INEL_WHERE) )
  CASE('In water')     ; J_Ine = SVR%Jact
  CASE('In air+water') ; J_Ine = SVR%Jact
  CASE('In air')       ; J_Ine = SRR_Jact
  CASE('Nowhere')      ; J_Ine = LPOS    
  END SELECT
  ! It will be used for actual wavelengths only
  ! IF( DO_RAMAN ) J_Ine = SRR_Jact
  
  Wdo = 1.d0 - Wup
  COMPARE_TO_DISORT = .FALSE.
  AGR_A%ADAPTIVE_GRID = ADAPTIVE_GRID
  
  IF(INT_MS_FLAG(LPOS)) THEN
     ALLOCATE( RAD_PS (NLAYERS, NSZA, N_SFN) )
     ALLOCATE( RADs_PS(NLAYERS, NSZA, N_NODE, Nvs) )
     RAD_PS = 0.d0 ; RADs_PS = 0.d0
  END IF
  
  ! Boundary conditions :
  ALLOCATE(UDB_A%BRF_GG(Ng1_brf,Ng2_brf,Nh_brf), & ! A
       UDB_A%BRF_GS(Ng1_brf,Ns_brf,Nh_brf), UDB_A%Fre_R(NSTRMS), & ! A
       UDB_A%FR_sol(NSZA,Nvs,Nvs), UDB_A%Fre_Rv(NSTRMS,Nvs,Nvs), &! A
       UDB_A%Fre_Tv(NSTRMS,Nvs,Nvs)  )  ! A

  IF( TRIM(SURF_TYPE)=='prosail'.AND.Nder > 0) THEN
     ALLOCATE( UDB_A%Der_GG(Ng1_brf,Ng2_brf,Nh_brf,Nder), & ! A
          UDB_A%Der_GS(Ng1_brf,Ns_brf,Nh_brf,Nder) )      ! A
  ENDIF

!   Define wavelength dependent refractive index. The structure 
!   element Ref_wd will be initialized here if 
!   1) UWT = .TRUE.
!   2) BRDF 'ocean' option is active
!   In all other cases Ref_type = 'non'
!WRITE(*,*) '    TRIM(Ref_type) = ', TRIM(Ref_type)
  SELECT CASE( TRIM(Ref_type) )
  CASE('non')
  CASE('const')
     ALLOCATE(UDB_A%Ref_wd(2))
     UDB_A%Ref_wd(1) = RefW_r(1) 
     UDB_A%Ref_wd(2) = RefW_i(1) 
  CASE('wl_dep') 
     ALLOCATE(UDB_A%Ref_wd(2),UDB_W%Ref_wd(2))
     CALL LIN_INT (Nwl_BRF, WL_BRF, RefW_r, 1, POS, UDB_A%Ref_wd(1), 'C')
     CALL LIN_INT (Nwl_BRF, WL_BRF, RefW_i, 1, POS, UDB_A%Ref_wd(2), 'C')
     UDB_W%Ref_wd(:) = UDB_A%Ref_wd(:)
  END SELECT
  
  ! WRITE(*,*) '    "dom_spherical"  3 '

  ! Preliminary to avoid multiple calculations of 
  ! the Fresnel reflection coefficient.
  ! This routine is located in "trans_refl_matr.f"
  IF(TRIM(RTM_CORE) /= 'ASYMP'.AND.Ref_type /= 'non')  THEN
     CALL Fre_REF  (NSTRMS, UDB_A%Fre_R,  XM, UDB_A%Ref_wd, 'aa')
     CALL Fre_Ref_v(NSTRMS, UDB_A%Fre_Rv, XM, UDB_A%Ref_wd)
     CALL Fre_Taw_v(NSTRMS, UDB_A%Fre_Tv, XM, UDB_A%Ref_wd)
  ENDIF
  
  UDB_A%BRDF_FLAG = BRDF_FLAG
  UDB_A%SPECULAR  = SPECULAR
  
  UDB_A%LBC = 'Lambert'
  IF( UDB_A%BRDF_FLAG ) THEN
     UDB_A%LBC = 'Bidirect'
     ! UDB_A%SPECULAR is set to .true. (read_brdf.f90)
     ! in the case of the 'melt_pond' BRDF model
     IF( UDB_A%SPECULAR ) UDB_A%LBC = 'Flat water'
  ENDIF
  
  ! Preliminary only Lambert ocean surface 
  UDB_W%BRDF_FLAG = .FALSE.  ;  UDB_W%SPECULAR  = .FALSE.
  UDB_W%LBC = 'Lambert'
  IF( UDB_W%BRDF_FLAG ) UDB_W%LBC = 'Bidirect'
  
  IF( LOG_LEVEL(1) >= N_ESO ) THEN
     WRITE(LUNIT,*) '    Initialization of BRDF_FLAG ' 
     WRITE(LUNIT,*) '    and "specular reflection" mode ' 
     WRITE(LUNIT,*) '    ',UDB_A%BRDF_FLAG, ' and ', UDB_W%BRDF_FLAG
     WRITE(LUNIT,*) '    ' ,UDB_A%SPECULAR, ' and ',  UDB_W%SPECULAR
     WRITE(LUNIT,*) '      Lower boundary condition flag ' 
     WRITE(LUNIT,*) '   Atmos      Ocean ' 
     WRITE(LUNIT,'(3x,A,2x,A)') UDB_A%LBC, UDB_W%LBC
  ENDIF
  
  ! Thermal emission :
  ALLOCATE(TE%N_pla(NL_pla-1), TE%Mup(NL_pla-1), TE%PL_COEF(NL_pla-1,Nf_pla),&
       TE%PLA_FINE(NL_fine_pla), TE%NPLANCK(NL_pla), TE%OPLANCK(NL_pla), &
       TE%Aj_pla(NN_pla,NL_pla-1) )
  
  IF( WF_FLAG ) ALLOCATE( TE%B_FINE(NL_fine_pla) )
      
  ! Spherical solar beam :
  ALLOCATE( SSB_A%N_sha(NSZA), SSB_A%N_sph(NSZA,NLAYERS-1),&
       SSB_A%Mue( NSZA,NLAYERS-1), SSB_A%EXS( NLAYERS,NSZA ),&
       SSB_A%SIN_COEF( NSZA,NLAYERS-1,Nf ) )

  ! Spherical line-of-sight :
  ALLOCATE(SLS%IND_LS(NLAYERS,NLSA*NFA,2), SLS%N_sph_ls(NLSA*NFA,NLAYERS-1,2),&
       SLS%Mue_ls(NLSA*NFA,NLAYERS-1,2), &
       SLS%SIN_COEF_LS(NLSA*NFA,NLAYERS-1,Nf,2), &
       SLS%Eff_mu(NLSA*NFA,N_Eff), SLS%LSE(GeoD%NAlts,GeoD%NVA) )

  DO IA = 1, GeoD%NAlts
     DO IV = 1, GeoD%NVA
        ALLOCATE(SLS%LSE(IA,IV)%EX(GeoD%LVG(IA,IV)%NLS), &
             SLS%LSE(IA,IV)%EX1(0:GeoD%LVG(IA,IV)%NLS) )
     END DO
  END DO
  
  ! Arrays for "Spherical" weighting function |
  ALLOCATE(SWF%SOL_EXP(NL_FINE,NSZA ), SWF%T_fine(NL_FINE), &
       SWF%IEffS(NLO,N_Eff), SWF%Ief_Sol(NLO), SWF%Ief_Sum(NLO), &
       SWF%SOL_JL(NLAYERS,NLO,N_Eff) )

  IF( WF_FLAG.OR.RRS_ADJ ) THEN
     IF (RTM_TYPE == 'spher_scat')  THEN
        ALLOCATE (SWF%DS_SUN_JL (NLAYERS,NLAYERS,NLO,N_Eff))
     ELSE
        ! ALLOCATE (SWF%DS_SUN_JL (NL_FINE,NLAYERS,NLSA,N_Eff))          
        ALLOCATE (SWF%DS_SUN_JL (NL_FINE,NLAYERS,NSZA,N_Eff))
     END IF
  END IF
  
  IF( RRS_ADJ.AND.(TRIM(RTM_TYPE)=='spher_scat') ) &
       ALLOCATE( SWF%Egs_sph(Nav, Nvs*NLO, N_Eff, 2, 3) )

  ! Arrays for different kind of weighting functions
  ALLOCATE(PWF%WF_esp(NLAYERS,NLO,NS_wf), & 
       PWF%WF_ssp(NLAYERS,NLO,NS_wf,1+N_dwr), &
       PWF%P_WF_ABSTERM(NLAYERS), PWF%T_WF_ABSTERM(NLAYERS) )

!  NLG_ALL_A = 1
!  NLG_ALL_C = 1
  
  !Internal dimensions for the aerosol phase matrix moments
  !========================================================
  IF( DO_AEROSOL ) THEN
     IF( LA_N == 1.AND.ITERATION == 1) THEN
        IF( ZZZ(LA_S(1)) < ZZZ(1) ) THEN
           DO I = 1, LA_S(1) - 1
              IF( AER_COEFF(2,I,1) /= 0.d0 ) GOTO 102
           ENDDO
        ENDIF
     ENDIF
     
     IF( Plus_mi ) THEN
        NL_Aer = NLAYERS
     ELSE
        NL_Aer = LA_N 
     ENDIF
     Nlg_Aer = NLEGEN 
     IF( DO_SSRCA.OR.TRIM(RTM_TYPE)=='spher_scat'.OR.SCATT_MODE =='ss') THEN
        ! write(*,*) '   dom_spher: NLG_ALL_A  ', NG_aer
        NLG_ALL_A = NG_aer
        ALLOCATE( OSM%AER_ALL(NLG_ALL_A,NL_Aer,Nm_vs) ) ! A
        IF( Dwr_mi ) ALLOCATE( OSM%DER_ALL(NLG_ALL_A,NL_Aer,Nm_vs,N_dwr) ) ! A
     END IF
  ELSE
     NL_Aer = 1 ; Nlg_Aer = 1
  ENDIF
  
  ALLOCATE( AGR_A%LAY_GR(NLAYERS-1,2) )
  
  ! Internal dimensions for the cloud phase matrix moments:
  !    NG_cld - maximal number of expansion coefficients 
  !             in the cloud phase matrixes 
  !           (setting in "general_cloud_init.f")
  !======================================================
  IF( DO_CLOUD ) THEN
     L = 1  
     IF( ANY(Plus_ice) ) L = L + 1
     ALLOCATE( LOA%DM_CLD(N_CLD_LAYERS,Nphr,L) )
     
     NL_Cld = N_CLD_LAYERS ; NLG_Cld = NLEGEN 
     IF( DO_SSRCA .OR. TRIM(RTM_TYPE)=='spher_scat' &
          .OR.SCATT_MODE =='ss' ) THEN
        NLG_ALL_C = NG_cld
!        WRITE(*,*) '     -------> NLG_ALL_C ', NLG_ALL_C
        ALLOCATE( OSM%CLD_ALL(NLG_ALL_C,NL_Cld,Nphr,Nm_ice,Nm_vs) )
        IF( Use_Ydb_SCM )  ALLOCATE( OSM%CLD_SCM &
             (Ydb_ang, NL_Cld, Nphr, Nm_ice, Nm_vs) ) ! A
        IF( Deriv_phc ) THEN
           ALLOCATE(OSM%Der_CLD_ALL(NLG_ALL_C, NL_Cld, Nphr, Nm_ice, Nm_vs)) ! A
           IF( Use_Ydb_SCM ) &
           ALLOCATE(OSM%Der_CLD_SCM(Ydb_ang, NL_Cld, Nphr, Nm_ice, Nm_vs)) ! A
        ENDIF  ! Deriv_phc
     ENDIF
  ELSE
     NL_Cld = 1 ; NLG_Cld = 1
  ENDIF

  IF(TRIM(RTM_CORE) /= 'ASYMP') THEN
     ALLOCATE( GSL_A%XM(NSTRMS), GSL_A%AXM(NSTRMS), GSL_A%AG(NSTRMS) )
     GSL_A%XM(:) = XM(:) ; GSL_A%AXM(:) = AXM(:) 
     GSL_A%AG(:) = AG(:)
  ENDIF
  
  ALLOCATE( GSL_A%MuS(NSZA), GSL_A%MuV(NLSA), &
       GSL_A%EXL(NLAYERS,NLSA), GSL_A%AG_ZERO(NLAYERS-1) ) ! A
  
  GSL_A%NN = NSTRMS  ;  GSL_A%NSZA = NSZA  ;  GSL_A%NLSA = NLSA
  GSL_A%AG_ZERO(:) = .FALSE.
  ! PAUSE
  ALLOCATE( UDB_A%FoINC(Nav,N_RHS) )
  ! Set first guess for the water-leaving intensity
  UDB_A%FoINC(:,:) = 0.d0
  UDB_A%Flu_int = 0.d0
  
  ! If land fluorescence is included. This option is not used
  ! in combination with inelastic processes

  IF( FLU_L ) THEN
     IF( DIFB ) THEN
        UDB_A%Flu_int = 1.d0
     ELSE
        IF( USE_DGF ) THEN
           FLS(1) = GAU(1,POS,DGF_PAR(1)%Amp) + GAU(2,POS,DGF_PAR(2)%Amp)
           IF( TRIM(WL_IRRAD_DIM(2))=='phot/s' ) FLS(1) =  FLS(1)*POS
        ELSE
           CALL AKIMA_INT(NFL, WLF, SPF, 1, POS, FLS)
        ENDIF
        UDB_A%Flu_int = FLU_SCAL*FLS(1)/SOL_FLUX(LPOS)*Pie
        ! UDB_A%Flu_int = 1.d0
        ! WRITE(*,*) '  UDB_A%Flu_int = ', UDB_A%Flu_int
     ENDIF
  ENDIF  ! fluorescence

!... Allocate elements of the structure LOW:
!    local optical properties of the water
! WRITE(*,*) '   dom_spherical: UWT =  ', UWT
! PAUSE
  IF( UWT ) THEN
     ! write(*,*) '     -------- UWT-0  -------------- '
     IA = 1
     IF(RTM_MODE=='flux'.OR.RTM_MODE=='spher_alb') THEN
        ! ERDR = .FALSE.
     ELSE
        ! Exact reflection of direct solar light must be calculated 
        ! only in the case of down-looking observer in atmosphere
        SELECT CASE( GeoD%View(IA)%S )
        CASE('up')
           IF( TRIM(Obs_POSITION(1))=='Ocean' ) ERDR = .FALSE.
        END SELECT
     ENDIF
         
     NSZA_W = NSZA
     IF( UDB_A%SPECULAR ) THEN 
        UDB_A%LBC = 'Flat water'  ; UDB_W%UBC = 'Flat water'
     ELSE
        UDB_A%LBC = 'Rough water' ; UDB_W%UBC = 'Rough water'
     ENDIF
     
     AGR_W%ADAPTIVE_GRID = .FALSE.
     
     ALLOCATE(UDB_W%Fre_R(NN_W), UDB_W%Ewa_G(NN_W), UDB_W%Fre_Rv(NN_W,Nvs,Nvs),&
          UDB_W%Fre_Tv(NN_W,Nvs,Nvs), UDB_A%Eaw_G(NSTRMS) ) ! A

     ! Set transmission matrix water --> air:
     UDB_W%Fre_Tv(       1:Nstr_T,:,:) = 0.d0
     UDB_W%Fre_Tv(Nstr_T+1:NN_W,  :,:) = UDB_A%Fre_Tv(:,:,:)

! Preliminary 
! Reflection coefficient in the total-reflection range
     IF( NSTR_T > 1 ) UDB_W%Fre_R(1:NSTR_T) = 1.d0
! Reflection coefficient in the refraction region,
! assuming that the number of Gauss nodes in the 
! refractive region in the ocean is the same as
! in the atmosphere (according Jin and Stamnes)
     UDB_W%Fre_R(NSTR_T+1:NN_W) = UDB_A%Fre_R(:)

     ALLOCATE( LOW%SCR_W(NL_W), LOW%AL_W(NL_W), LOW%BET(NL_W),&
          LOW%ABS_UW(NL_W,N_ABS_UW+Na_W), &     ! A
          LOW%RAY_MOMS_W(Nlg_Ray_W,Nm_vs_W) ) ! A

     N_HPC = MAX( IWS(1)%N_HPC,IWS(2)%N_HPC )
     IF( N_HPC > 0 ) THEN
        K1 = MAX( LA_N_HS, LA_N_IW )
        ALLOCATE( LOW%HSCA(NL_W,N_HPC,N_IW), LOW%HEXT(NL_W,N_HPC,N_IW), & ! A
             LOW%HSOL_MOMS(NLEG_W, K1, Nm_vs, N_HPC, N_IW) ) ! A Preliminary
     ENDIF

     ALLOCATE( AGR_W%LAY_GR(NL_W-1,2) )

     ALLOCATE( GSL_W%XM(NN_W), GSL_W%AXM(NN_W), GSL_W%AG(NN_W), &
          GSL_W%MuS(NSZA_W), GSL_W%MuV(NLSA_W), GSL_W%EXL(NL_W,NLSA),&
          UDB_A%Ref_AA_LOS(Nvs*NLSA,Nvs*NSZA,Naz) )
         
     ! Allocate structure elements if dependence of water refractive
     ! index is accounted for
     IF( Nwl_BRF > 1 ) THEN
        ! Preliminary M_max and N_Beam are set as in "trans_refl_matr.f"
!        M_max = 50  ;  N_Beam = NSZA
        M_max = N_HARM  ;  N_Beam = NSZA
        IF( WF_FLAG ) N_Beam = N_Beam + NLSA
        ALLOCATE( UDB_A%Ra_GG (Nav,Nav,M_Max), &  ! A
             UDB_A%Ra_GS (Nav,Nvs*N_Beam,M_Max), & ! A
             UDB_A%Taw_GG(Nwv,Nav,M_Max), UDB_A%Taw_GS(Nwv,Nvs*NSZA,M_Max),& ! A
             UDB_W%Rw_GG (Nwv,Nwv,M_Max), UDB_W%Twa_GG(Nav,Nwv,M_Max) ) ! A
     ENDIF  ! Nwl_BRF > 1

     GSL_W%EXL = 0.d0

     GSL_W%NN = NN_W
     GSL_W%XM (:) =  XM_W(:)
     GSL_W%AXM(:) = AXM_W(:)
     GSL_W%AG (:) =  AG_W(:)
     ! Preliminary the same as in the atmosphere
     GSL_W%NSZA = NSZA    !NSZA_W
     GSL_W%NLSA = NLSA    !NLSA_W
     ! Spherical solar beam for the case of plane-parallel only:
     ALLOCATE( SSB_W%N_sha(GSL_W%NSZA), SSB_W%N_sph(GSL_W%NSZA,NL_W-1),& ! A
          SSB_W%Mue(GSL_W%NSZA,NL_W-1), &
          SSB_W%SIN_COEF(GSL_W%NSZA,NL_W-1,Nf), & ! A Preliminary
          SSB_W%EXS(NL_W,GSL_W%NSZA) )   ! A

     ALLOCATE( ALW_W(NL_W-1), T_AU_W(NL_W) )
     
     ! Reflection matrix in the total refraction region
     CALL Fre_tot(NN_W, UDB_W%Fre_Rv, GSL_W%XM, UDB_A%Ref_wd)
     
     ! Incident radiation from the atmosphere and from the ocean
     SELECT CASE( TRIM(UDB_A%LBC) )
     CASE('Flat water')
        ! The interaction ocean-atmosphere takes place within the
        ! Fresnel cone only. The number of Gauss angles within 
        ! the atmosphere and the Fresnel cone has to be equal.
        ALLOCATE( UDB_W%FoINC(GSL_A%NN*Nvs,N_RHS) )
     CASE('Rough water')
        ALLOCATE( UDB_W%FoINC(GSL_W%NN*Nvs,N_RHS) )
     END SELECT
     ! write(*,*) '     -------- UWT-1  -------------- '
         
  ENDIF  ! UWT

  ! Allocate elements of the structure LOA:
  ! local optical properties of the atmosphere
  ALLOCATE(LOA%SCR(NLAYERS),  LOA%RAY_MOMS(Nlg_Ray,Nm_vs), LOA%SCA(NLAYERS),&
       LOA%EXA(NLAYERS), LOA%AS_A(NLAYERS), LOA%DM_A(NLAYERS), &
       LOA%AER_MOMS(Nlg_Aer,NL_Aer,Nm_vs), LOA%SCC(NLAYERS), &
       LOA%EXC(NLAYERS), LOA%CSM(Nm_ice,NLAYERS), LOA%AL(NLAYERS), &
       LOA%SCC_NOSCALE(NLAYERS,Nm_ice), LOA%SCI(NL_ice), LOA%EXI(NLAYERS), &  
       LOA%CLD_MOMS(NLG_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs),   LOA%BET(NLAYERS) )

  IF( DO_CLOUD ) THEN
     IF( .NOT.ANY(MLCS(:)%Phm_co) )&
          ALLOCATE( LOA%DR_MOMS(NLG_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs) ) ! A
  ENDIF
  IF( Dwr_mi ) THEN
     ALLOCATE( LOA%ADR_MOMS(NLEGEN,NL_Aer,Nm_vs,N_Dwr) ) ! A
     LOA%ADR_MOMS = 0.d0
  ENDIF
  !------------------------------------------------------------
  
  SCR => LOA%SCR 
  SCA => LOA%SCA  ;   EXA => LOA%EXA
  SCC => LOA%SCC  ;   EXC => LOA%EXC
  SCI => LOA%SCI  ;   EXI => LOA%EXI
  AL  => LOA%AL
  
  SCR = 0.D0 ; SCA = 0.D0 ; AL = 0.D0  
  EXC = 0.D0 ; SCC = 0.D0 ; LOA%SCC_NOSCALE = 0.D0 
  SCI = 0.D0 ; EXI = 0.d0 ; EXA = 0.D0 
  LOA%DM_A(:) = 1.d0
  ! Internal dimensions for the cloud scattering phase functions
  ! ============================================================
  IF( INQ_WF('cld', 'WFT') ) THEN
     Nl_Cr = N_CLD_LAYERS ; N_C2 = 2
     Ng_Cr = Nav ; Nls_Cr = Nvs*NLSA ; Nso_Cr = Nvs*NSZA 
  ELSE
     Nl_Cr = 1 ; Ng_Cr = 1 ; Nls_Cr = 1 ; Nso_Cr = 1 ; N_C2 = 1
  ENDIF
  
! Internal dimensions for the aerosol scattering phase functions
! ==============================================================
  IF( INQ_WF('aer', 'WFT') ) THEN
     Nl_Ar = LA_N ; N_A2 = 2
     Ng_Ar = Nav ; Nls_Ar = Nvs*NLSA ; Nso_Ar = Nvs*NSZA 
  ELSE
     Nl_Ar = 1 ; Ng_Ar = 1 ; Nls_Ar = 1 ; Nso_Ar = 1 ; N_A2 = 1
  ENDIF

  LOA%CLD_MOMS = 0.d0
  
  ! WRITE(*,*)  '  NUM_WF     -  ', NUM_WF
  ! WRITE(*,*)  '  NUM_CLD    -  ', NUM_CLD
  ! WRITE(*,*)  '  SCA_or_ABS -  ', SCA_or_ABS
  ! PAUSE
      
  IF( NUM_WF ) CALL TEM_PRE_NUMERIC(*880)
  IF( NUM_CLD ) THEN 
     SELECT CASE( SCA_or_ABS )
     CASE('lw_content','eff_radius','soot__conc','par_number')
        CALL GT_CLOUD_PARAMETER_SETUP(*880)
     END SELECT
  ENDIF
  
  ! IF( DO_CLOUD.AND.RET_FLAG ) CALL GT_CLOUD_PARAMETER_SETUP(*880)
  
  ! King factor actual wavelength.
  ! This routine is located in "st_refr_king.f" collection.
  CALL KING_ACTUAL(POS, DEPOL, *894)

  IF( DO_RAMAN.AND.SRR%Exi_WL(LPOS) ) THEN
     ! Interpolate anisotropy of polarizability at an 
     ! RRS excitation wavelength (POS)
     DO I = 1,N_RG
        ! CALL AKIMA_INTERP( DEP_NWS, DEP_WS, R_GAM(I,:), 1, POS, &
        !    SRR%GAM(I), ERROR )
        CALL LIN_INT(DEP_NWS,DEP_WS,R_GAM(I,:),1,POS,SRR%GAM(I),'C')
     END DO

     ! Test with approximation of Chance and Spur
     ! SRR%GAM(1) = (0.07149d0 + 45.9364d0/&
     ! ( 48.2716d0 - 1.d0/(POS*1.d-3)**2 ))*1.d-24
     ! SRR%GAM(1) = SRR%GAM(1)**2*1.d-12
     ! SRR%GAM(2) = (-6.01466d0 + 2385.57d0/&
     ! ( 186.099d0 - 1.d0/(POS*1.d-3)**2 ))*1.d-25
     ! SRR%GAM(2) = SRR%GAM(2)**2*1.d-12
     ! End test 

     ! If DO technique is selected, save anisotropy of 
     ! polarizability at excitation wavelengths 
     ! (It will be used in routine "part_sol_rrs.f")
     IF( RRS_DO ) THEN
        ! WRITE(*,*) '  Save Gam at ', SRR%Jexi
        SRR%GAM_WL(:,SRR%Jexi) = SRR%GAM(:)
     ENDIF
  ENDIF
  
  ! The output is depolarization ratio for water: DEP_W
  IF( UWT ) CALL DEPOL_FAC_WATER(*880)
  
  IF( DO_CLOUD ) THEN
     ! This version allows extrapolation as a constant
     CALL LIN_SEARCH_NEW(POS, LOA%DP_CLD, LOA%Cil, LOA%Cir, NCW,CLW)
  ENDIF
  
  IF(TRIM(RTM_CORE) /= 'ASYMP')  CALL GT_HI_WD(POS, DEPOL, DEP_W, &
       LOW, LOA, OSM, UDB_A, UDB_W, *880)

  IF( ALIN_ACT(2) ) THEN
     ! Alternative input of Rayleigh scattering coefficient
     ! WRITE(*,*) '   shape(SCR)  ', shape(SCR)
     ! WRITE(*,*) '   shape(ALIN_RAY)  ', shape(ALIN_RAY)
     SCR(:) = ALIN_RAY(LPOS,:)
  ELSE
     ! Calculation of Rayleigh scattering coefficient
     CALL RAYLEIGH_SCATT(LPOS, POS, DEPOL, SCR, *880)
  ENDIF

  ! Total Raman scattering coefficient at actual wavelength
  IF( DO_RAMAN.AND.SRR%Act_WL(LPOS) ) THEN
!           --Test C-R--
! IF( DO_RAMAN ) THEN
!  ... Interpolate anisotropy of polarizability to the 
!      actual wavelength (POS)
     DO I = 1,N_RG
        ! CALL AKIMA_INTERP( DEP_NWS, DEP_WS, R_GAM(I,:), 1, POS, &
        !    SRR%GAM(I), ERROR )
        CALL LIN_INT(DEP_NWS,DEP_WS,R_GAM(I,:),1,POS,SRR%GAM(I),'C')
     END DO
! Test with approximation
! SRR%GAM(1) = (0.07149d0 + 45.9364d0/ &
!     ( 48.2716d0 - 1.d0/(POS*1.d-3)**2 ))*1.d-24
! SRR%GAM(1) = SRR%GAM(1)**2*1.d-12
! SRR%GAM(2) = (-6.01466d0 + 2385.57d0/&
!     ( 186.099d0 - 1.d0/(POS*1.d-3)**2 ))*1.d-25
! SRR%GAM(2) = SRR%GAM(2)**2*1.d-12
! End test

     IF( TRIM(TSM)=='speed' )  SRR%GAM_WL(:,SRR_Jact) = SRR%GAM(:)
     ! Calculate the total Raman scattering coefficient summing up
     ! the energy loss in all Stokes and Anti-Stokes Raman lines.
     ! (see Eq.(Ram-tot-sum) in STB)
     S1 = 0.d0
     ! OPEN(44,file=DIR_OUT//'/Raman.dat')
     DO Jr = 1,N_WL_C
        ! WL_ST correspond to the position of Raman lines in the
        ! sequence Stokes --> anti-Stokes
        WL_ST = 1.D+9/( 1.D+9/POS + DEL_WL(Jr) )
        S1 = S1 + WL_FACT*SUM(Sigma_RR(Jr,:))/WL_ST**4*SRR%GAM(LINE_GAS(Jr))

        ! WRITE(44,'(2x,I3,2x,F8.3,2x,E12.4)') Jr, WL_ST,&  
        !    WL_FACT*SUM(Sigma_RR(Jr,:))/WL_ST**4*SRR%GAM(LINE_GAS(Jr))
     ENDDO
! CLOSE(44)
! S1 is divided by SUM(SCR(:)) because SUM(Sigma_RR(Jr,:)) was used,
! i.e., the sum of rotational Raman scattering coefficients over
! all layers
     SRR%R_TOT = S1/SUM(SCR(:))
! Test of oscillations with analytical total Raman scattering
! coefficient
! SRR%R_TOT = 2.5d0*DEPOL/(2.d0 + DEPOL)
! WRITE(*,*) '   Pract R_tot: ', SRR%R_TOT
!  PAUSE
  ENDIF

  ! Used below for test of atoms WF
  IF( NUM_RAY ) SCR(:) = SCR(:)*PERTURB(:)
  
  CALL GT_SCATT(LPOS, POS, LOA, *880)
  
  IF( DO_PLANCK ) CALL ST_PLANCK_INIT (LPOS, POS, TE%OPLANCK, &
       TE%OPLANCK_SURF, *880)

  ! Rayleigh scattering coefficient of water
  IF( UWT ) CALL ST_UWT_COEFF(LPOS, POS, LOW, UDB_A, UDB_W, GSL_A, &
       GSL_W, DEP_W, SVR, *880)
  ! WRITE(*,*) '    :', UDB_W%Ewa_G(NN_W)

  ! Additional absorber like "soot" will be added to the 
  ! cloud extinction coefficient
  IF( ANY(ADD_ABS) ) THEN
     CALL ADD_SOOT(POS, Abs_soot)
     EXC(:) = EXC(:) + Abs_soot(:)
  ENDIF

  IF( NUM_CLD ) THEN 
     ! EXC does not contain additional absorber
     IF( ANY(ADD_ABS) ) EXC(:) = EXC(:) - Abs_soot(:)
     SELECT CASE( TRIM(Wat_or_Ice) )
     CASE('water','nomix')
        P_EXC => EXC(1:NLAYERS) ; P_SCC => SCC(1:NLAYERS)
        EXC_0(:) = EXC(1:NLAYERS)
        SCC_0(:) = SCC(1:NLAYERS)
     CASE('ice')
        P_EXC => EXI(1:NLAYERS) ; P_SCC => SCI(1:NLAYERS)
     END SELECT
     
     SELECT CASE( SCA_or_ABS )
     CASE('scattering')
        ! Perturbation of the cloud scattering coefficient
        P_EXC(:) = (P_EXC(:)-P_SCC(:)) + P_SCC(:)*PERTURB(:)
        P_SCC(:) = P_SCC(:)*PERTURB(:)
        LOA%SCC_NOSCALE(:,1) = LOA%SCC_NOSCALE(:,1)*PERTURB(:)
     CASE('absorption')
        ! Perturbation of the cloud absorption coefficient
        SELECT CASE( TRIM(Wat_or_Ice) )
        CASE('water','nomix')
           EXC(:) = (EXC(:)-SCC(:))*PERTURB(:) + SCC(:)
        CASE('ice')
           EXI(:) = (EXI(:)-SCI(:))*PERTURB(:) + SCI(:)
        END SELECT
     END SELECT
     IF( ANY(ADD_ABS) ) EXC(:) = EXC(:) + Abs_soot(:)
  ENDIF
  
  IF( NUM_AER ) THEN 
     SELECT CASE( SCA_or_ABS )
     CASE('scattering')
        ! Perturbation of the aerosol scattering coefficient
        EXA(:) = (EXA(:)-SCA(:)) + SCA(:)*PERTURB(:)  
        SCA(:) = SCA(:)*PERTURB(:) 
        ! WRITE(*,*) ' 2 : ', SUM(EXA), SUM(SCA)
        
     CASE('absorption')
        ! Perturbation of the aerosol absorption coefficient
        EXA(:) = (EXA(:)-SCA(:))*PERTURB(:) + SCA(:)  
     CASE('aer_dens')
        ! Perturbation of the aerosol number density
        ! Mie_layer is allocated in "mie_aer" mode only !!
        IF( ALLOCATED(Mie_layer) ) THEN
           WHERE( Mie_layer(:) )
              SCA(:) = SCA(:)*PERTURB(:) 
              EXA(:) = EXA(:)*PERTURB(:) 
           END WHERE
        ELSE
           SCA(:) = SCA(:)*PERTURB(:) 
           EXA(:) = EXA(:)*PERTURB(:) 
        ENDIF
     END SELECT
  ENDIF
  
  IF(DO_ESFT) THEN
     ESFT_FLAG_LOC = ANY(DO_ESFT_SPEC_MOL(LPOS,:))
  ELSE
     ESFT_FLAG_LOC = .FALSE.
  END IF
  
  IF(ESFT_FLAG_LOC) THEN
     ! Subroutines IDX_ESFT and WEIGHT_ESFT are in "st_esft.f"
     CALL IDX_ESFT(LPOS, E_IDX, NESFT, NESFT_IDX, *880)
  ELSE
     NESFT = 1
  ENDIF
  TG_CS(:,:) = 0.0D0
  
  ! Save number of ESFT coefficients at actual wavelength
  IF( DO_RAMAN.AND.SRR%Act_WL(LPOS) )  SRR%NESFT(SRR_Jact) = NESFT
  
  IF( LOG_LEVEL(1) >= N_RSO  ) THEN
     IF( SRR%Act_WL(LPOS).AND.SVR%ACT_WL(LPOS) ) THEN
        WRITE(LUNIT, &
             '(2X,"*** Radiance calculation point ",I6, " at", 1PG14.6, " nm")') &
             LPOS, POS
     ELSE
        WRITE(LUNIT, &
             '(2X,"*** Radiance calculation excitation wavelength ",I6," at", 1PG14.6, " nm")') LPOS, POS
     ENDIF
  ENDIF
  
  SCA_AL(:) = SCR(:) + SCA(:) + SCC(:)
  IF( ANY(Plus_ice) ) SCA_AL(:) = SCA_AL(:) + SCI(:)
  
  ! DO I = 1,NLAYERS
  !    IF( EXC(I) == 0.d0 ) CYCLE
  !    WRITE(*,*) ZZZ(I), SCC(I)/EXC(I)
  ! ENDDO

  CALL SET_ANGLES
      
  ! Start direct reflection and transmission matrices --------
  IF( UWT ) THEN
     IF( TRIM(A_RTmat)=='NT_ICF' ) THEN
        ALLOCATE( Cor_NT(NSZA) )
        CALL NT_CORRECTION  ! ===> Cor_NT(:)
     ENDIF  ! 'NT_ICF'
     ! WRITE(*,*) '      =======>  ERDR  ', ERDR
     IF( ERDR ) THEN
        ALLOCATE( Tra_AW(Nvs*NSZA,Nvs*NSZA,1) )
        CALL TRM_LOS(NSZA, NLSA, Naz, MuS, MuV, ANG_OUT(3,1:Naz),&
             UDB_A%Ref_wd, UDB_A%Ref_AA_LOS, Tra_AW)
        IF( TRIM(A_RTmat)=='NT_ICF' ) THEN
           DO K = 1, NSZA  ;  K1 = Nvs*(K-1)
              UDB_A%Ref_AA_LOS(:,K1+1:K1+Nvs,:) = &
                   UDB_A%Ref_AA_LOS(:,K1+1:K1+Nvs,:)/Cor_NT(K)
              Tra_AW(:,K1+1:K1+Nvs,1) = &
                   Tra_AW(:,K1+1:K1+Nvs,1)/Cor_NT(K)
           ENDDO  ! over solar angles (K)
        ENDIF  ! 'NT_ICF'
     ENDIF   ! ERDR
     
     ! write(*,*) '    Mh = 1'
     ! DO I = Nvs*(NN_W - 1) + 1, Nwv
     !    write(*,'(*(2x,e12.4))') (UDB_A%Taw_GS(I,K,1), K = 1,Nvs)
     ! ENDDO
     ! write(*,*) '    Mh = 2'
     ! DO I = Nvs*(NN_W - 1) + 1, Nwv
     !    write(*,'(*(2x,e12.4))') (UDB_A%Taw_GS(I,K,2), K = 1,Nvs)
     ! ENDDO
     ! write(*,*) '    Mh = 3'
     ! DO I = Nvs*(NN_W - 1) + 1, Nwv
     !    write(*,'(*(2x,e12.4))') (UDB_A%Taw_GS(I,K,3), K = 1,Nvs)
     ! ENDDO
     ! write(*,*) '  ========================= '
     ! WRITE(*,*) '   SHAPE(Tra_AW) ', SHAPE(Tra_AW)
     ! DO I =  1, Nvs
     !    write(*,'(*(2x,e12.4))') (Tra_AW(I,K,1), K = 1,Nvs)
     ! ENDDO
         
  ENDIF  ! UWT
  !----------------------------------------------------------------
      
  DO Iot = 1, N_AOT+1
     IF( Iot > 1 ) THEN
        SCA_AL(:) = SCA_AL(:) + SCA(:)*(AOT_MAL(Iot-1) - 1.d0)
        SCA(:) = SCA(:)*AOT_MAL(Iot-1)
        EXA(:) = EXA(:)*AOT_MAL(Iot-1)
     ENDIF
     ! Initialization of the logical variable DO_OUT 
     ! in the case of the single scattering mode (23.08.13)
     DO_OUT = .TRUE.

     DO JJESFT = 1,NESFT
        AGR_A%ADAPTIVE_GRID = ADAPTIVE_GRID
        IF(ESFT_FLAG_LOC) THEN
           ! This routine is placed in "st_esft.f"
           CALL WEIGHT_ESFT(LPOS,JJESFT,JESFT,NESFT,NESFT_IDX,E_W)
        ELSE
           JESFT = JJESFT  ;  E_W = 1.d0
        ENDIF
        
        ! Save ESFT coefficients at actual wavelength if 
        ! single-scattering correction algorithm is used
        IF( DO_RAMAN.AND.SRR%Act_WL(LPOS).AND.SAV_WEI ) &
             SRR%WEI(SRR_Jact,JJESFT) = E_W

       CALL ST_ABS_COEFF(LPOS, POS, JJESFT, E_IDX, TG_CROSS_SEC,  &
             LOA, PWF, PCC, TGA, *880)
        ! WRITE(*,*) '  st_abs_coeff Al ',  AL(NLAYERS)
        ! WRITE(*,*) '    LOA%Al        ',LOA%AL(NLAYERS)
        ! WRITE(*,*) '    LOA%SCA       ',LOA%SCA(NLAYERS)

        ! Copy preliminary cross-sections in new structure 
        DO KK = 1, N_GAS
           LOA%TG_CS(KK)%CS(:) = TG_CROSS_SEC(:,KK)
        ENDDO
        
        IF( Emi_FLG ) THEN
           Ka = 0
           DO KK = 1, N_GAS 
              M = ST_GAS_IDX(KK)
              IF( TG_INFO(M)%M_TYPE /= 'atom' ) CYCLE
              Ka = Ka + 1                  
              ! WRITE(*,*) '   M gas number in all   ', M
              ! WRITE(*,*) '  KK selected gas number ', KK
              ! WRITE(*,*) '  Ka atom number         ', Ka
              ! Absorption coefficient of atoms and ions
              LOA%EMC(Ka,:) = LOA%TG_CS(KK)%CS(:)*TG_INFO(M)%ND(:)
              ! Phase function moments of atoms and ions
              FORALL( K=1:Nlg_Ray ) LOA%EPF(Ka,K,:) = &
                   LOA%TG_CS(KK)%E_P(K)*Emi_MOMS(K,:)
           ENDDO
        ENDIF   !  atoms emission
        
        IF( TES_2L_SLAB ) THEN 
           LOA%AL(:)   = LOA%SCR(:)/Ome0(1)
           LOW%AL_W(:) = LOW%SCR_W(:)/Ome0(2)
        ENDIF
        
        IF( LOG_LEVEL(1) <= N_XSO ) THEN
           WRITE(LUNIT,*) '       SCR:', SUM(SCR)
           WRITE(LUNIT,*) '    SCA_AL:', SUM(SCA_AL)
           WRITE(LUNIT,*) '    LOA%AL:', SUM(LOA%AL)
        ENDIF
        
        TG_CS(:,:) = TG_CS(:,:) + E_W*TG_CROSS_SEC(:,:)
        
        FORALL(J = 1:NLAYERS, K = 1:N_GAS) TAC(J,K) = TG_CROSS_SEC(J,K)
           
        ! For solar exp only !
        IF(NUM_WF) AL_1_PERT(1:NLAYERS) = AL(:)
           
        ! Set up the Lambertian surface properties 
        Call GT_SURFACE(1, LPOS, POS, UDB_A, UDB_W, *880)
        
        IF( COMPARE_TO_DISORT ) CALL OUT_INFOR(*880)
        
        IF( TRIM(RTM_CORE) == 'ASYMP' ) THEN
           CALL GPP_REFL_ASYMPTOTIC( LPOS, JJESFT, NSTRMS, POS, &
                LOA, DEPOL, E_W, UDB_A%ALBEDO, TGA, *880)
           CYCLE            
        ENDIF
        
        ! Wavelength dependent arrays for st_dom_tau
        ALW(:) = Wup*AL(1:NLAYERS-1) + Wdo*AL(2:NLAYERS)
        T_AU(1) = 0.d0 
        ! Optical depth in the atmosphere
        DO J = 1,NLAYERS - 1
           T_AU(J+1) = T_AU(J) + ALW(J)*DZR(J)
        ENDDO
        
        IF (LOG_LEVEL(1) >= N_ESO)  &
             WRITE(LUNIT,*) ' ===> TAU_dom :', T_AU(NLAYERS)
        
        IF( DO_PLANCK ) THEN
           TE%NO_SUN = NO_SUN
           ! Multiplied by emissivity in "ST_PLANCK_INIT.f"
           TE%PLANCK_SURF = TE%OPLANCK_SURF 
           ! TE%PLANCK_SURF = (1.d0 - UDB_A%ALBEDO)*TE%OPLANCK_SURF 
           TE%ADD_PLANCK = .TRUE.
           N_PL = (NLAYERS - 1)*Nf_pla + 1
           CALL APPR_PLANCK(LPOS, POS, N_PL, TE, AL, SCA_AL, *880)
           ! Planck function contains now multiplier 
           ! (1 - single.scat.alb)
           TE%NPLANCK(1:NLAYERS-1) = TE%OPLANCK(1:NLAYERS-1) &
                *( 1.d0 - ( Wup*SCA_AL(1:NLAYERS-1) &
                + Wdo*SCA_AL(2:NLAYERS) )/ALW(:) )
           TE%NPLANCK(NLAYERS) = TE%OPLANCK(NLAYERS)
           ! To exclude thermal emission of atmosphere
           ! TE%NPLANCK(:) = 0.d0
        ELSE
           TE%ADD_PLANCK = .FALSE.  ;  TE%PLANCK_SURF = 0.d0
           TE%NO_SUN = .FALSE.
        ENDIF
        
        ! Transmission of the solar beams in atmosphere and ocean
        CALL SOL_TRANS (*880)

        ! Transmission along line-of-sight in atmosphere and ocean
        CALL LOS_TRANS (*880)

        IF( RRS_ADJ.AND.(TRIM(RTM_TYPE)=='spher_scat') ) THEN
           CALL RAM_EGS_SPH (*880)
        ENDIF
        
        IF( TRIM(SCATT_MODE)=='dir_ref') THEN
           CALL ST_DIR_REFL(LPOS, POS, UDB_A%ALBEDO, E_W, LOA, &
                SSB_A, SRR, SRR_Jact, PWF, TAC, UDB_A%Flu_int, *880) 
           CYCLE
        ENDIF
        
        ! Interpolation of all optical parameters on the fine 
        !           altitude grid (use spher_grid)
        !----------------------------------------------------------
        IF (RTM_TYPE == 'spher_scat') THEN
           IF( ALLOCATED(ADI%AL0_OUT)) DEALLOCATE(ADI%AL0_OUT, ADI%SCR_OUT,&
                ADI%SCA_OUT, ADI%ASC_OUT, ADI%Plf_OUT, ADI%X_SCR,  ADI%X_AL, &
                ADI%X_SCA, ADI%X_ASC, ADI%SCC_NSO, ADI%X_SCC, ADI%X_Plf)

           ALLOCATE(ADI%X_SCR(NLAYERS,4), ADI%X_AL(NLAYERS,4), &
                ADI%X_SCA(NLAYERS,4), ADI%X_ASC(NLAYERS,4),&
                ADI%X_SCC(NLAYERS,4,Nm_ice), ADI%X_Plf(NLAYERS,4),&
                ADI%AL0_OUT(NL_FINE), ADI%SCR_OUT(NL_FINE), &
                ADI%SCA_OUT(NL_FINE), ADI%ASC_OUT(NL_FINE), &
                ADI%SCC_NSO(NL_FINE,Nm_ice), ADI%Plf_OUT(NL_FINE) )
           
           AL_NDM(:) = AL(:)
           
           IF( DO_AEROSOL ) THEN
              WHERE( LOA%DM_A(:) /= 1.d0 )
                 SCAD(:) = LOA%SCA(:)/(1.d0 - LOA%DM_A(:))
              ELSEWHERE
                 SCAD(:) = LOA%SCA(:)
              END WHERE
!             ... Use non-scaled aerosol scattering coefficient
              IF( RAD_FIELD ) AL_NDM(:) = AL(:) - LOA%SCA(:) + SCAD(:)
           ENDIF
           
           IF( DO_CLOUD.AND.RAD_FIELD ) THEN
              DO Jmc = 1, Nm_ice
!                ... Use non-scaled cloud scattering coefficient
                 AL_NDM(:) = AL_NDM(:) - LOA%CSM(Jmc,:) + LOA%SCC_NOSCALE(:,Jmc)
              ENDDO  ! over water/ice phase (Jmc)              
           ENDIF

           ! Extinction coefficient:
           CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, AL_NDM, F_LGI_EXT,&
                ADI%X_AL, AKIMA_FLG)
           ! Rayleigh scattering coefficient:
           CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, LOA%SCR, F_LGI_SCR,&
                ADI%X_SCR, AKIMA_FLG)
           ! Aerosol
           ADI%X_SCA = 0.0D0  ;  ADI%X_ASC = 0.0D0
           IF(DO_AEROSOL) THEN
              CALL AKIMA_ALL_FINE (NLAYERS, ALT_EXT, SCAD,&
                   F_LGI_SCA, ADI%X_SCA, AKIMA_FLG)
              CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, LOA%AS_A, &
                   .FALSE., ADI%X_ASC, AKIMA_FLG)
           END IF
           ! Planck function
           ADI%X_Plf = 0.0D0 
           IF( DO_PLANCK ) THEN
              ALLOCATE( Plh(NLAYERS) )
              Plh(:) = (AL(:) - SCA_AL(:))*TE%OPLANCK(:)
              CALL AKIMA_ALL_FINE (NLAYERS, ALT_EXT, Plh,&
                   F_LGI_SCA, ADI%X_Plf, AKIMA_FLG)
              DEALLOCATE( Plh )
           END IF
           ! Clouds
           ADI%X_SCC = 0.d0
           IF( DO_CLOUD ) THEN
              DO Jmc = 1, Nm_ice
                 CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, LOA%SCC_NOSCALE(:,Jmc),&
                      .FALSE., ADI%X_SCC(:,:,Jmc), .FALSE.)
              ENDDO  ! over water/ice phase (Jmc)
              ! Interpolate profile of effective radius water 
              ! and/or ice on the fine altitude grid  
!             IF( .NOT.Phm_const ) THEN
              IF( .NOT.ANY(MLCS(:)%Phm_co) ) THEN
                !--- Scat. matr. depends on effective radius --------
                 IF(ALLOCATED(ADI%EfWI)) DEALLOCATE(ADI%EfWI)
                 ALLOCATE(ADI%EfWI(NL_FINE,Nm_ice))
                 KK = 2
                 DO J = 1, Nm_ice
                    CALL LIN_FINE_1(NLAYERS, ZZZ, NL_FINE, Z_FINE, INTERP_IDX,&
                         Prof_Cpar(:,KK), ADI%EfWI(:,J), .FALSE.)
                    KK = KK + 2
                 ENDDO  ! over water/ice phase (J,KK)
                 !----------------------------------------------------
              END IF  ! .NOT.ANY(MLCS(:)%Phm_co
           ENDIF  ! DO_CLOUD
           
           CALL AKIMA_END_FINE (NLAYERS, Z_INV, NL_FINE, Z_FINE, INTERP_IDX, &
                ADI%X_AL, ADI%X_SCR, ADI%X_SCA, ADI%X_ASC, ADI%X_SCC, &
                ADI%X_Plf, ADI%AL0_OUT, ADI%SCR_OUT, ADI%SCA_OUT, ADI%ASC_OUT,&
                ADI%SCC_NSO, ADI%Plf_OUT )               
           ! Interpolation of Delta-M scaling factor 
           !  on the fine grid
           IF( INT_MS_FLAG(LPOS) ) THEN
              IF(ALLOCATED(ADI%DM_AO)) DEALLOCATE(ADI%DM_AO)
              ALLOCATE(ADI%DM_AO(NL_FINE))
              CALL LIN_FINE_1( NLAYERS, ZZZ, NL_FINE, Z_FINE, & 
                   INTERP_IDX, LOA%DM_A, ADI%DM_AO, .FALSE.)
              ! write(*,*) '   shape( LOA%DM_A ) ', shape( LOA%DM_A )
              ! write(*,*) '  LOA%DM_A(NLAYERS-1:NLAYERS) = ',&
              !    LOA%DM_A(NLAYERS-1:NLAYERS )
              ! write(*,*) ' ADI%DM_AO(1:2) = ', ADI%DM_AO(1:2)
              IF( DO_CLOUD ) THEN
                 IF(ALLOCATED(ADI%SCCO)) DEALLOCATE(ADI%SCCO)
                 ALLOCATE(ADI%SCCO(NL_FINE,Nm_ice))
                 DO J = 1, Nm_ice
                    CALL LIN_FINE_1( NLAYERS, ZZZ, NL_FINE, Z_FINE, INTERP_IDX,&
                         LOA%CSM(J,:), ADI%SCCO(:,J), .FALSE.)
                 ENDDO
              END IF  ! DO_CLOUD
           END IF  ! INT_MS_FLAG
        END IF ! RTM_TYPE == 'spher_scat'
        !----------------------------------------------------------
        
        ! Start single scattering approximation for plane-parallel
        ! or spherical geometry and than return 
        !    WRITE(*,*) '    INT_MS_FLAG(LPOS) = ', INT_MS_FLAG(LPOS)
        IF( .NOT.INT_MS_FLAG(LPOS) ) THEN 
           CALL SINGLE_SCAT(*880) ;  CYCLE
        ENDIF
        
        ! New altitude grid in the atmosphere will be calculated 
        CALL MAKE_NEW_GRID(1, .FALSE.)
        ! WRITE(*,*) '   AGR_A%ADAPTIVE_GRID = ', AGR_A%ADAPTIVE_GRID
        ! PAUSE

        AGR_A%NrL = NrL ; AGR_A%NL = NLAYERS 
        AGR_A%N_OBS  = N_OBS_AG
        AGR_A%LAY_GR(:,:) = LAY_GR(:,:)         
        ! WRITE(*,*) '   NRL atm: ', AGR_A%NrL
        ! WRITE(*,*) AGR_A%LAY_GR(:,1) 
        IF( UWT.AND.(.NOT.ARB_OL) ) THEN
           AGR_W%NrL = NL_W ; AGR_W%NL = NL_W
           AGR_W%N_OBS  = AGR_A%N_OBS
           DO I = 1,NL_W-1
              AGR_W%LAY_GR(I,1) = I   ;  AGR_W%LAY_GR(I,2) = I
           ENDDO
           ! WRITE(*,*) '   NRL ocean: ',AGR_W%NrL
           ! WRITE(*,*) AGR_W%LAY_GR(:,1) 
        ENDIF  ! UWT.AND.(.NOT.ARB_OL)
        
        ! It will be used if a solution not only forward but also
        ! adjoint RTE is needed
        CALL SET_AGR_Nxl
            
        DO_OUT = .FALSE.  
        !***************** Do-loop over harmonics *******************
        DO MHARM = 1,N_HARM ! Over Fourier harmonics loop
! DO MHARM = 1, 4 ! Over Fourier harmonics loop
!    WRITE(*,*) '   ------ Harmonic :    ', Mharm, NRL-1
           IF( MHARM == 4.OR.MHARM == Mh_AG ) THEN
              CALL MAKE_NEW_GRID(MHARM, .TRUE.)
              CALL SET_AGR_Nxl
              ! WRITE(*,*) '   AGR_A%ADAPTIVE_GRID ', MHARM, AGR_A%ADAPTIVE_GRID
              ! PAUSE
              AGR_A%NrL = NrL ; AGR_A%NL = NLAYERS 
              AGR_A%N_OBS  = N_OBS_AG
              AGR_A%LAY_GR(:,:) = LAY_GR(:,:)
           ENDIF
           
           ! IF( MHARM > 1 )  UDB_W%Ftra = 0.d0
           ! IF( MHARM > 1 )  SSB_W%EXS = 0.d0

               
           IF( MHARM > Nh_s ) GOTO 101
           IF( MHARM > 1 ) TE%ADD_PLANCK = .FALSE.
           IF( BRDF_FLAG .AND. MHARM > Mh_BRF ) GOTO 103
           UDB_A%FoINC = 0.d0
           IF( UWT ) UDB_W%FoINC = 0.d0
           
           CALL GT_PHASE_LAYERS(LOA, LOW)

           IF(MHARM > 1) CALL GT_SURFACE(MHARM, LPOS, POS, UDB_A, UDB_W,*880)

           CALL ST_DOM_TAU(LPOS, JJESFT, Iot,Mharm, POS, E_W, AGR_A, AGR_W,&
                GSL_A, GSL_W, UDB_A, UDB_W, LOA, LOW, LWF, TAC, OSM, SLS, &
                SSB_A, SSB_W, TE, SWF, PWF, Abs_soot, RAD_PS, RADs_PS, DEPOL,&
                DO_OUT, DO_CONV1, SRR, SRR_Jact, SVR, RADOUT, W_LEAV, PH_M, &
                ADI, *880)

           IF( .NOT.RAD_FIELD ) THEN
              ! Routine "gpp_converge.f" will be called if the 
              ! wavelength is marked as "actual" (see "rtm_main.f")
              IF( SVR%USE_CONV(LPOS) ) CALL GPP_CONVERGE(NSTRMS, MHARM, J_Ine,&
                   Iot, RADOUT, W_LEAV, DO_OUT, DO_CONV1)
           ENDIF
           ! Convergence of all radiation field is checked 
           ! in the subroutine "SPHERE_FIELD_S" called by 
           ! "st_dom_tau.f"

           IF( DO_OUT ) THEN 
              ! WRITE(*,*) '   Convergence at Mha = ',Mharm
              EXIT 
           ENDIF   ! of convergence check   
        END DO  ! over Fourier harmonics (MHARM)
        !***************** End do-loop over harmonics *******************
     END DO  ! over ESFT (JJESFT)
  END DO  ! over aerosol OT (Iot)
  
  IF( SVR%USE_CONV(LPOS).AND.(.NOT.DO_OUT) ) THEN
     CALL WRITE_MESSAGE(R_NAME,'WARNING','')
     WRITE(EU,'(a)') '     Convergence is not reached. The number of Legendre'
     WRITE(EU,'(a)') '     moments is not large enough to satisfy'// &
          ' convergence criteria.'
     WRITE(EU,'(a,2x,I3)') '     Number of Legendre moments ',N_HARM 
     WRITE(EU,'(a,2x,I3)') '     Spectral point ', LPOS
  ENDIF

  IF (LOG_LEVEL(1) >= N_ESO) CALL OPTICAL_DEPTH(LPOS, POS, LOA, LOW, TG_CS)   
      
  SELECT CASE (TRIM(RTM_MODE))
  CASE ('block_amf')
     FORALL(J = 1:NLAYERS) WF_TO_AMF(LPOS,J) = &
          0.5d0*DZ(J)*TG_CS(J,GAS_INDEX(AMF_IDX))
  CASE ('amf', 'slant_col')
     VC_FLAG = .FALSE.
     IF(LPOS == 1) VC_FLAG = .TRUE.
     I_GAS = GAS_INDEX(AMF_IDX)
     FFF(:) = TG_CS(1:NLAYERS,I_GAS)*TG_INFO(AMF_IDX)%ND(:)
     CALL VERT_INTEGR(FFF, TG_VOD(LPOS,:,1))         
     IF(VC_FLAG) THEN
        CALL VERT_INTEGR(TG_INFO(AMF_IDX)%ND, VERT_COL)
        IF( TG_INFO(AMF_IDX)%NAME == 'o4' ) VERT_COL(:) = VERT_COL(:)*1.D+40
     END IF
  END SELECT
  
  DO IA = 1, GeoD%NAlts
     DO IV = 1, GeoD%NVA
        DEALLOCATE( SLS%LSE(IA,IV)%EX, SLS%LSE(IA,IV)%EX1 )
     END DO
  END DO
  DEALLOCATE( SSB_A%N_sha, SSB_A%N_sph, SSB_A%Mue, SSB_A%EXS, & ! D
       SSB_A%SIN_COEF, SLS%IND_LS, SLS%N_sph_ls, SLS%Mue_ls,& ! D
       SLS%SIN_COEF_LS, SLS%Eff_mu, GSL_A%EXL, GSL_A%AG_ZERO, & ! D
       SWF%SOL_EXP, SWF%T_fine, SWF%IEffS, SWF%Ief_Sol, SWF%Ief_Sum, & ! D
       SWF%SOL_JL, PWF%WF_esp, PWF%WF_ssp, PWF%P_WF_ABSTERM, & ! D
       PWF%T_WF_ABSTERM, LOA%SCR, UDB_A%Ftra, LOA%EXA, LOA%AS_A, LOA%SCA,& ! D
       LOA%SCC, LOA%EXC, LOA%CSM, LOA%SCI, LOA%EXI, LOA%AL, & ! D
       LOA%SCC_NOSCALE, SLS%LSE)
  IF( WF_FLAG.OR.RRS_ADJ ) DEALLOCATE (SWF%DS_SUN_JL)

  IF( RRS_ADJ.AND.(TRIM(RTM_TYPE)=='spher_scat') ) DEALLOCATE( SWF%Egs_sph )

  IF( Dwr_mi ) DEALLOCATE( LOA%ADR_MOMS ) ! D
  
  IF( DO_SSRCA.OR.TRIM(RTM_TYPE)=='spher_scat' ) THEN
     IF( DO_AEROSOL )  THEN 
        DEALLOCATE( OSM%AER_ALL )
        IF( Dwr_mi ) DEALLOCATE( OSM%DER_ALL ) ! D
     ENDIF
     IF( DO_CLOUD )  THEN
        DEALLOCATE( OSM%CLD_ALL )                       ! D
        IF( Deriv_phc )   DEALLOCATE( OSM%Der_CLD_ALL ) ! D
        IF( Use_Ydb_SCM ) DEALLOCATE( OSM%CLD_SCM )     ! D
     ENDIF
  END IF
  
  ! IF( TRIM(Ref_type) /= 'non' ) DEALLOCATE( UDB_A%Ref_wd, UDB_W%Ref_wd )
      
  IF( DO_CLOUD ) THEN
     IF( .NOT.ANY(MLCS(:)%Phm_co) )  DEALLOCATE( LOA%DR_MOMS )
  ENDIF
  
  DEALLOCATE( UDB_A%BRF_GG, UDB_A%BRF_GS, UDB_A%Fre_R, UDB_A%Fre_Rv, & ! D
       UDB_A%FoINC ) ! D

  IF( TRIM(SURF_TYPE)=='prosail'.AND.Nder > 0) &
       DEALLOCATE( UDB_A%Der_GG, UDB_A%Der_GS )         ! D

  IF( Emi_FLG ) DEALLOCATE( LOA%EMC, LOA%EPF, PH_M%MET)  ! D
  
  IF( UWT ) THEN
     DEALLOCATE(UDB_W%Fre_R, UDB_W%Fre_Rv, UDB_W%Ewa_G, UDB_A%Eaw_G,& ! D
          LOW%SCR_W, LOW%AL_W, LOW%BET, LOW%ABS_UW, AGR_W%LAY_GR, &   ! D
          LOW%RAY_MOMS_W, GSL_W%EXL, SSB_W%EXS, GSL_W%XM, GSL_W%AXM, & ! D
          GSL_W%AG, GSL_W%MuS, GSL_W%MuV, SSB_W%N_sha, SSB_W%Mue,  &  ! D
          SSB_W%N_sph, SSB_W%SIN_COEF )
     IF( UDB_A%SPECULAR ) THEN
        DEALLOCATE( UDB_A%Fref, UDB_W%Ftra )
     ENDIF
     IF( N_HPC > 0 ) DEALLOCATE( LOW%HEXT, LOW%HSCA, LOW%HSOL_MOMS )
     IF( ASSOCIATED(UDB_A%Ra_GG) ) THEN
        DEALLOCATE( UDB_A%Ref_AA_LOS )
        IF( Nwl_BRF > 1 ) THEN
           ! Arrays were allocated in this routine
           DEALLOCATE( UDB_A%Ra_GG, UDB_A%Ra_GS, UDB_A%Taw_GG, & ! D
                UDB_A%Taw_GS, UDB_W%Rw_GG,UDB_W%Twa_GG  )      ! D
        ELSE
           ! Arrays were associated with target in "gt_hi_wd.f"
           NULLIFY( UDB_A%Ra_GG, UDB_A%Ra_GS, UDB_A%Taw_GG,  &  ! D
                UDB_A%Taw_GS, UDB_W%Rw_GG,UDB_W%Twa_GG  )      ! D
        END IF
     ENDIF
  ENDIF  ! UWT

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Nh_max too small increase it in gt_hi_wi.f ' )
  GOTO 881

  ! Aerosol extinction coefficient is non zero at altitudes
  ! above the top of highest aerosol layers.
102 CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Aerosol is initialized incorrect.')
  WRITE(EU,'(A,2x,2(2x,f10.5))') '     Aerosol layers between :', &
       ZZZ(LA_E(LA_N)),ZZZ(LA_S(1))
  DO K = LA_S(1)+1, NLAYERS
     IF( AER_COEFF(2,K,1) == 0.d0 ) EXIT
  ENDDO
  IF( K > NLAYERS ) K = NLAYERS
  WRITE(EU,'(A,2x,2(2x,f10.5))') &
       '     Aerosol number density :', ZZZ(K), ZZZ(I)
  GOTO 881
  !-----------------------------------------------------------------   

103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
  WRITE(EU,'(a)') '     Pre-calculated number of Fourier harmonics for BRDF'
  WRITE(EU,'(a)') '     is too small to satisfy convergence criteria.'
  WRITE(EU,'(a,2x,I3)') '     Number of pre-calculated Fourier harmonics', &
       Mh_BRF
  WRITE(EU,'(a,2x,I3,a)') '              Calculation performed already for ',&
       MHARM, ' harmonics.'
  WRITE(EU,'(a)') '     Recommendation:'
  WRITE(EU,'(a)') '     increase parameters N_Phi and M_max in '// &
       '"general_brdf_setup.f"'
  WRITE(EU,'(a)') '     or set "Single scattering correction" control '
  WRITE(EU,'(a)') '     parameter in "control_ac.inp" file to .TRUE.'
  GOTO 881

894 CALL WRITE_MESSAGE(R_NAME,'ERROR', 'ERROR IN LINTP2')
  GOTO 881

880 WRITE(EU,*) ' ---> Called by '//TRIM(R_NAME)
881 WRITE(EU,*) ' ---> Calling sequence: gt_iface - rtm_main'
  WRITE(EU,*) ' ---> Error inside the wavelenght loop'
  
  WRITE(LUNIT,*)
  WRITE(LUNIT,*) ' >>>>>>>> !!! Abnormal termination !!! <<<<<<<<<<<'
  WRITE(LUNIT,*) ' SCIATRAN stopped in dom_spherical: see errors.log'
  WRITE(LUNIT,*) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
  WRITE(LUNIT,*)

  CLOSE(EU)
  STOP

CONTAINS
  !************************************************************
  !    ... Perform Nakagima and Tanaka corrections
  !*************************************************************
  SUBROUTINE NT_CORRECTION
    INTEGER  Nvk
    REAL (KND)  Rd, Td, Def
    !==========================================================
    NWv = Nvs*NN_W  ;   NAv = Nvs*NSTRMS
    ! Nakagjima & Tanaka correction Ocean -> Atmosphere:
    DO K = 1,NN_W  ;  Nvk = Nvs*(K-1)
       ! Flux reflected back to the ocean
       Rd = SUM( UDB_W%Rw_GG(1:NWv:Nvs,Nvk+1,1)*AXM_W(:) )
       ! Flux transmitted into atmosphere
       Td = SUM( UDB_W%Twa_GG(1:NAv:Nvs,Nvk+1,1)*AXM(:) )
       ! Energy balance
       Def = 1.d0 - 2.d0*Pie*(Rd + Td)/XM_W(K)
       ! IF(  K == NN_W ) Def = 0.d0
       UDB_W%Rw_GG(1:NWv,Nvk+1:Nvk+Nvs,:) = &
            UDB_W%Rw_GG(1:NWv,Nvk+1:Nvk+Nvs,:)/(1.d0 - Def)
       UDB_W%Twa_GG(1:NAv,Nvk+1:Nvk+Nvs,:) = &
            UDB_W%Twa_GG(1:NAv,Nvk+1:Nvk+Nvs,:)/(1.d0 - Def)
    ENDDO

    ! Nakagjima & Tanaka correction Atmosphere -> Ocean:
    DO K = 1,NSTRMS  ;  Nvk = Nvs*(K-1)
       ! Flux reflected back to the atmosphere
       Rd = SUM( UDB_A%Ra_GG (1:NAv:Nvs,Nvk+1,1)*AXM )
       ! Flux transmitted into ocean
       Td = SUM( UDB_A%Taw_GG(1:NWv:Nvs,Nvk+1,1)*AXM_W )
       ! Energy balance
       Def = 1.d0 - 2.d0*Pie*(Rd + Td)/XM(K)
       ! IF(  K == NSTRMS ) THEN
       !    write(*,*) '  Ra_gg  ', 1.d0 - 2.d0*Pie*(Rd)/XM(K)
       !    write(*,*) '  Taw_gg ', 1.d0 - 2.d0*Pie*(Td)/XM(K)
       ! ENDIF
       UDB_A%Ra_GG(1:NAv,Nvk+1:Nvk+Nvs,:) = &
            UDB_A%Ra_GG(1:NAv,Nvk+1:Nvk+Nvs,:)/(1.d0-Def)
       UDB_A%Taw_GG(1:NWv,Nvk+1:Nvk+Nvs,:) = &
            UDB_A%Taw_GG(1:NWv,Nvk+1:Nvk+Nvs,:)/(1.d0-Def)
    ENDDO
      
    ! The same correction but for solar directions
    DO K = 1, NSZA  ;  Nvk = Nvs*(K-1)
       Rd = SUM( UDB_A%Ra_GS (1:NAv:Nvs,Nvk+1,1)*AXM(:) )
       Td = SUM( UDB_A%Taw_GS(1:NWv:Nvs,Nvk+1,1)*AXM_W(:) )
       Def = 1.d0 - 2.d0*Pie*(Rd + Td)/DABS( MuS(K) )
       Cor_NT(K) = 1.d0 - Def
       UDB_A%Ra_GS(1:NAv,Nvk+1:Nvk+Nvs,:) = &
            UDB_A%Ra_GS(1:NAv,Nvk+1:Nvk+Nvs,:)/(1.d0 - Def)
       UDB_A%Taw_GS(1:NWv,Nvk+1:Nvk+Nvs,:) = &
            UDB_A%Taw_GS(1:NWv,Nvk+1:Nvk+Nvs,:)/(1.d0 - Def)
    ENDDO
    ! Nakagima and Tanaka correction approach
    ! ---------------------------------------------------
  END SUBROUTINE NT_CORRECTION

  !********************************************************************    
  SUBROUTINE SEPARATE_GAU_DELTA
    INTEGER IM1
!******  Explanation of separation the transmitted light  ********
!           into diffuse  and Dirac-delta component        
! Direct solar intensity just above surface         
!    Int_dir    = Pie*SSB_A%EXS(NLAYERS,1)
! Flux of direct solar just above surface
!    Dir_atm    = Int_dir* Mus(1)
! Reflected flux atmosphere-->atmosphere
!    Fd_ref_Cox = Int_dir* 2.*Pie*SUM &
!        ( UDB_A%Ra_GS(1:Nav:Nvs,1,1)*GSL_A%AXM(1:NSTRMS) )
! Transmitted flux atmosphere-->ocean
!    Fd_tra_Cox = Int_dir* 2.*Pie *SUM &
!        ( UDB_A%Taw_GS(1:Nwv:Nvs,1,1)*GSL_W%AXM(1:NN_W) )
! Transmitted flux related to the Dirac-delta function 
!    Dir_oce = Dir_atm - Fd_ref_Cox - Wei_dif*Fd_tra_Cox
! Intensity delta-component of the direct solar light just
!    below surface
! Int_oce = Dir_oce/MusW(1)
!    Norm = Int_oce/Int_dir/Mus(1)*MuSW(1)
!**************************************************************
      
!     ... Array Tra_AW is the transmission matrix of the direct
!         solar light (atmosphere-->ocean) accounting for 
!         wind-roughened interface (output of "trm_los.f")
    Ivs = 1
    DO K = 1, NSZA   ;   K1 = Nvs*(K-1)  ;  KK = K1 + 1
       ! Norm = 1.d0 - 2.d0*Pie/Mus(K)* &
       !    ( SUM( UDB_A%Ra_GS(1:Nav:Nvs,KK,1)*GSL_A%AXM(1:NSTRMS)) &
       !     + Wei_dif*SUM( UDB_A%Taw_GS(1:Nwv:Nvs,KK,1)*GSL_W%AXM(1:NN_W)) )

       Norm = 2.d0*Pie*(1.d0 - Wei_dif)/Mus(K)*&
            SUM( UDB_A%Taw_GS(1:Nwv:Nvs,KK,1)*GSL_W%AXM(1:NN_W))
       
       ! IM1 = MAXLOC( UDB_A%Taw_GS(1:Nwv:Nvs,KK,1),1 )
       ! WRITE(*,*) '    Im1 = ', Im1
       ! DO J = Im1-2,Im1+2
       !   WRITE(*,'(2x,I3, 2x,F8.3,*(2x,e12.4) )') J,
       !   DACOS(GSL_W%XM(J))/Pie*180.,
       !   UDB_A%Taw_GS(J,KK,1)
       ! ENDDO
       ! Norm = 2.d0*Pie*(1.d0 - Wei_dif)/Mus(K)*&
       !   UDB_A%Taw_GS(Im1,KK,1)*GSL_W%AXM(Im1)        
       ! WRITE(*,*) '   SHAPE(Tra_AW) ', SHAPE(Tra_AW)
       ! WRITE(*,*) '   SHAPE(Finc)   ', SHAPE(Finc) 
       DO J = 1, Nvs
          UDB_W%Ftra(K,Ivs,J) = Norm/Tra_AW(K1+1,K1+1,1) &
               *SUM( Tra_AW(K1+J,K1+1:K1+Nvs,1) *Finc(Ivs,:) )
          ! WRITE(*,*) '   J = 1', J
       ENDDO
       ! UDB_A%Taw_GS(Im1,KK,:) = Wei_dif*UDB_A%Taw_GS(Im1,KK,:)
    ENDDO  ! over solar angles (K)

    UDB_A%Taw_GS = Wei_dif*UDB_A%Taw_GS
    ! UDB_A%Taw_GS(:,:,1) = Wei_dif*UDB_A%Taw_GS(:,:,1)

    ! WRITE(*,*) '   UDB_W%Ftra "separate_gau_delta" 2: '
    ! WRITE(*,'(2x,*(2x,e12.4))') UDB_W%Ftra(:,1,1)

  END SUBROUTINE SEPARATE_GAU_DELTA

  !********************************************************************
  SUBROUTINE SET_AGR_Nxl
      
    IF( WF_FLAG.OR.RRS_ADJ.OR.INEL_ADJ.OR.OWF_FLAG &
         .OR.TRIM(SOLVER)=='Solv_ASI') THEN
       ! Start and end of the integration along line-of-sight
       ! for the pseudo-forward RTE
       IA = 1
       IF( UWT ) THEN
          SELECT CASE( GeoD%View(IA)%S )
          CASE('up')
             ! Upward radiation (coupled)
             SELECT CASE( TRIM(Obs_POSITION(1)) )
             CASE('Atmos')
                AGR_A%Nxl_1 = N_OBS_AG  ; AGR_A%Nxl_2 = NrL      -1 
                AGR_W%Nxl_1 = 1         ; AGR_W%Nxl_2 = AGR_W%NrL-1 
             CASE('Ocean') 
                AGR_A%Nxl_1 = 1         ; AGR_A%Nxl_2 = -1 
                AGR_W%Nxl_1 = N_OBS_AG  ; AGR_W%Nxl_2 = AGR_W%NrL-1 
             END SELECT
          CASE('down')
             ! Downward radiation (coupled)
             SELECT CASE( TRIM(Obs_POSITION(1)) )
             CASE('Atmos')
                AGR_A%Nxl_1 = 1   ; AGR_A%Nxl_2 = N_OBS_AG - 1
                AGR_W%Nxl_1 = 1   ; AGR_W%Nxl_2 = -1 
             CASE('Ocean')    ! Instrument in the ocean
                ! AGR_A%Nxl_1 = 1 ; AGR_A%Nxl_2 = NL_W - 1
                AGR_A%Nxl_1 = 1 ; AGR_A%Nxl_2 = NrL - 1
                AGR_W%Nxl_1 = 1 ; AGR_W%Nxl_2 = N_OBS_AG - 1
             END SELECT
          END SELECT
       ELSE  ! UWT = .FALSE.
          SELECT CASE( GeoD%View(IA)%S )
          CASE('up')
             AGR_A%Nxl_1 = N_OBS_AG ; AGR_A%Nxl_2 = NRL-1
          CASE('down')
             AGR_A%Nxl_1 = 1        ; AGR_A%Nxl_2 = N_OBS_AG-1
          END SELECT
       ENDIF  ! Option UWT 
    ENDIF  ! WF_FLAG.or.RRS_ADJ.or.INEL_ADJ.or.OWF_FLAG 
    
  END SUBROUTINE SET_AGR_Nxl
  
  !******************************************************************
  SUBROUTINE SET_ANGLES
    
    ! Set line-of-sight angles in atmosphere
    SELECT CASE( ANG_SEL_FLAG )
    CASE( 'one' )
       MuV(:) = DABS( ANG_OUT(2,1:NLSA) )
    CASE( 'all' )
       MuV(:) = DABS( ANG_OUT(2,1:NLSA*Naz:Naz) )
    END SELECT
    GSL_A%MuV(:) = MuV(:)
    
    ! Set solar angles in atmosphere
    MuS(:) = SOL_GRD(1:NSZA)
    GSL_A%MuS(:) = MuS(:)
    
    IF( UWT ) THEN 
       SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
       CASE('Atmos')
          GSL_W%MuV(:) = SQRT( MuV(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind
       CASE('Ocean')
          GSL_W%MuV(:) = MuV(:) ! Preliminary
       END SELECT
       ! Ignoring wind in the case of plane-parallel atmosphere
       ! In the case of spherical atmosphere will be recalculated
       ! in the routine "sol_trans.f"
       GSL_W%MuS(:) = DSQRT(MuS(:)**2 + Ref_ind**2 - 1.d0)/Ref_ind
       ! WRITE(*,*) '   '^
       ! WRITE(*,*) '  Solar zenith angles air/water '
       ! DO I = 1, NSZA
       !    WRITE(*,'(3x,I2,2(2x,F6.2))') &
       !          I, ACOS(MuS(I))/Pie*180.,ACOS(GSL_W%MuS(I))/Pie*180.
       ! ENDDO
       ! WRITE(*,*) '   '
    END IF
    
  END SUBROUTINE SET_ANGLES
  !************************************************************************

  SUBROUTINE SOL_TRANS (*)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SOL_TRANS'
    !==================================================================
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    !Solar transmission in the atmosphere
    !------------------------------------
    SELECT CASE (TRIM(RTM_TYPE))
    CASE('ps_scat', 'spher_scat')
       ! The initialization of MuS within this routine again !!
       IF( TE%NO_SUN ) THEN
          SSB_A%N_sph = 1  ;  SSB_A%N_sha(:) = NLAYERS
       ELSE
          
          ! WRITE(*,*) '   +++++ NSZA before APPR_SPHER_TRANS ', NSZA
            
          CALL APPR_SPHER_TRANS (NSZA, LPOS, JJESFT, AL, POS, SSB_A, SWF, *880)
          DO Js = 1,NSZA  ;  L = SSB_A%N_sha(Js)
             IF( L < NLAYERS ) Then
                SSB_A%N_sph(Js,L:NLAYERS-1) = 1
                SSB_A%Mue(Js,L:NLAYERS-1)   = 1.d0
                SSB_A%EXS(L:NLAYERS,Js)     = 0.d0
             ENDIF
          ENDDO
       ENDIF
    CASE DEFAULT
       SSB_A%N_sph = 1  ;  SSB_A%N_sha(:) = NLAYERS
       FORALL( I = 1:NLAYERS-1 ) SSB_A%Mue(:,I) = MuS(:)
       FORALL( I = 1:NSZA ) SSB_A%EXS(:,I) = DEXP(-T_AU(:)/MuS(I))
       ! WRITE(*,*) '  SSB_A%EXS(:,1) '
       ! WRITE(*,*) SSB_A%EXS(:,1)
    END SELECT
    
    ! It is used to eliminate contribution of direct solar beam
    SSB_A%EXS = Fbe*SSB_A%EXS
    IF( TE%NO_SUN ) SSB_A%EXS = 0.d0
    
    ! The contribution of the direct solar light in atmosphere
    ! is set to zero at an actual wavelength
    IF( Sol_off.AND.SVR%ACT_WL(LPOS) ) SSB_A%EXS(:,:) = 0.d0
    
    IF( UDB_A%SPECULAR ) THEN
       ALLOCATE( UDB_A%Fref(NSZA,Kvs,Nvs) )
       CALL Fre_Ref_v(NSZA, UDB_A%FR_sol, GSL_A%MuS, UDB_A%Ref_wd)
       DO J = 1, Nvs
          DO Ivs = 1,Kvs
             DO K = 1, NSZA
                UDB_A%Fref(K,Ivs,J) = SUM( UDB_A%FR_sol(K,J,:)*Finc(Ivs,:) )
             ENDDO
          ENDDO
       ENDDO
       !--------------------------------------------------------------------- 
       !. Reflection matrix
       ! write(*,*) ' SHAPE(UDB_A%Fref) ', SHAPE(UDB_A%Fref)  
       ! write(*,*) '    Reflection "dom_spherical.f": direct incidence '
       ! DO J = 1, Nvs
       !    write(*,'(2x,*(1x,E12.4))') (UDB_A%FR_sol(1,J,K), K = 1, Nvs)
       ! ENDDO
       ! write(*,*) '   --------- '
       ! DO Ivs = 1,Kvs
       !    write(*,'(2x,*(1x,E12.4))') (UDB_A%Fref(1,Ivs,J), J =1,Nvs ) 
       ! ENDDO
       !--------------------------------------------------------------------- 
       ! WRITE(*,'(A,2x,e12.4)') &
       !    '   Fresnel reflectance for the diffuse incidence',&
       !           2.*SUM( UDB_A%Fre_Rv(:,1,1)*Xm(:)*AG(:) )
       ! STOP
    ENDIF  ! UDB_A%SPECULAR
    
    !===> Solar transmission in the ocean
    IF( UWT ) THEN
       SSB_W%N_sph = 1  ;  SSB_W%N_sha(:) = NL_W
       ! Recalculate solar angle just below ocean surface
       ! in the case of the spherical atmosphere
       GSL_W%MuS(:) = &
            SQRT(SSB_A%Mue(:,NLAYERS-1)**2 + Ref_ind**2 - 1.d0)/Ref_ind 
       ! The same SZA at all depths in the ocean
       FORALL( I = 1:NL_W-1 ) SSB_W%Mue(:,I) = GSL_W%MuS(:)
       ALW_W(:) = Wup*LOW%AL_W(1:NL_W-1) + Wdo*LOW%AL_W(2:NL_W)
       T_AU_W(1) = 0.d0 
       ! Optical depth in the ocean
       DO  J = 1, NL_W - 1
          T_AU_W(J+1) = T_AU_W(J) + ALW_W(J)*(ZZZ_UW(J+1) - ZZZ_UW(J))
       ENDDO
       MuSW => GSL_W%MuS(:)
       
       ! Fresnel reflection coefficient for solar beam.
       ! This routine is located in "trans_refl_matr.f"
       CALL Fre_Ref_v(NSZA, UDB_A%FR_sol, GSL_A%MuS, UDB_A%Ref_wd)
       
       CALL Fre_REF  (NSZA, Fre_RS, GSL_A%MuS, UDB_A%Ref_wd, 'aa')
       ! Fresnel transmission matrix including polarization 
       CALL Fre_Taw_v(NSZA, Fre_TS, GSL_A%MuS, UDB_A%Ref_wd)
       
       ALLOCATE( UDB_W%Ftra(NSZA,Kvs,Nvs) )
       DO J = 1, Nvs
          DO Ivs = 1,Kvs
             DO K = 1, NSZA
                ! Transmitted direct solar light
                UDB_W%Ftra(K,Ivs,J) = SUM( Fre_TS(K,J,:)*Finc(Ivs,:) )
             ENDDO
          ENDDO
       ENDDO

       ! WRITE(*,*) '   UDB_W%Ftra 1: ', SHAPE( UDB_W%Ftra )
       ! WRITE(*,'(2x,*(2x,e12.4))') UDB_W%Ftra(:,1,1)
         
       IF( UDB_A%SPECULAR ) THEN
          !===> Specular reflection 
          DO K = 1, NSZA  ;  S1 = UDB_W%Ftra(K,1,1)
             UDB_W%Ftra(K,1,:) = UDB_W%Ftra(K,1,:)/S1
             Tra_co(K) = Mus(K)*S1/MuSW(K)
          ENDDO
          !<=== Specular reflection (option "UWT")
       ELSE
          !===> Wind roughened interface reflection 
          SELECT CASE( TRIM(A_RTmat) )
          CASE('FT')
             ! Transmitted in the water solar light is considered
             ! as delta-function ignoring roughness (the same as 
             ! in the case of specular reflection)
             DO K = 1, NSZA  ; S1 = UDB_W%Ftra(K,1,1)
                UDB_W%Ftra(K,1,:) = UDB_W%Ftra(K,1,:)/S1
                Tra_co(K) = Mus(K)*S1/MuSW(K)
             ENDDO
          CASE('NT_ICF','NT_ECF')
             IF( Wei_dif < 1.d0 ) THEN
                ! Separation of the delta function in transmitted direct light
                CALL SEPARATE_GAU_DELTA ! --> UDB_W%Ftra
                DO K = 1, NSZA  ;  S1 = UDB_W%Ftra(K,1,1)
                   Tra_co(K) = S1
                   IF( S1 == 0.d0 ) CYCLE
                   UDB_W%Ftra(K,1,:) = UDB_W%Ftra(K,1,:)/S1
                ENDDO
             ELSE
                UDB_W%Ftra = 0.d0 ;  Tra_co(:)  = 0.d0
             ENDIF
          END SELECT
          !<=== Rough interface reflection 
       ENDIF
       
       FORALL( I = 1:GSL_W%NSZA ) SSB_W%EXS(:,I) = EXP( -T_AU_W(:)/MuSW(I) ) &
            *SSB_A%EXS(NLAYERS,I) *Tra_co(I)
       
       ! The contribution of the direct solar light in water 
       ! is set to zero at an actual wavelength
       IF( Sol_off.AND.SVR%ACT_WL(LPOS) ) SSB_W%EXS(:,:) = 0.d0
       
    ENDIF  ! UWT
    !<=== Solar transmission in the ocean
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  END SUBROUTINE SOL_TRANS
  
  !**********************************************************************
  SUBROUTINE RAM_EGS_SPH (*)
    
    USE GT_POLYN,  ONLY: XM, Plm_G, Dp2_G, Dm2_G, SGN
    USE ST_VARS,   ONLY: Vec_Sca
    !-------------------------------------------------------------------
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RAM_EGS_SPH'
    INTEGER  I, Leg, L1, JL, K1, K2, K11, Ks, M
    INTEGER  SIGN_M
    REAL (KND)  Phh(Nlg_Ray,NSTRMS), XSOL(N_Eff), Plm_S(3,Nlg_Ray,NLO,N_Eff)
    REAL (KND), ALLOCATABLE :: Dp2_S(:,:,:,:), Dm2_S(:,:,:,:)
    REAL (KND), POINTER :: Pop(:,:), Pom(:,:)
    TARGET :: Dp2_S, Dm2_S
    !==================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    Leg = Nlg_Ray
    ! WRITE(*,*) ' NLO =  ', NLO
    SELECT CASE(Vec_Sca)
       ! Calculation of the phase function 
       ! "local solar angles" ==> "Gauss angles" along L-of-S
    CASE('scal_rte')
       DO K1 = 1, NLO
          ! Number of effective solar angles along line-of-sight
          K2 = SWF%Ief_SOL(K1)
          DO KK = 1, K2
             XSOL(KK) = - E_CSZA(SWF%IEffS(K1,KK),'R')
          END DO
          DO M = 1,3
             CALL WIGNER_PLG( XSOL(1:K2), K2, M-1, Leg, Plm_S(M,:,K1,1:K2) )
          ENDDO   ! over harmonics (M)
       ENDDO  ! over line-of-sight (K1)
       DO M = 1,3
          L1 = Leg - M + 1
          FORALL( K = 1:NSTRMS ) Phh(1:L1,K) = &
               SRR%Ram_MOMS(M:Leg,1)*Plm_G(M,M:Leg,K)
          DO K1 = 1,NSTRMS 
             DO K2 = 1,NLO
                DO JL = 1,SWF%Ief_Sol(K2)
                   SWF%Egs_sph(K1,K2,JL,1,M) = &
                        SUM(Plm_S(M,M:Leg,K2,JL)*Phh(1:L1,K1)*SGN(1:L1) )
                   SWF%Egs_sph(K1,K2,JL,2,M) = &
                        SUM( Plm_S(M,M:Leg,K2,JL)*Phh(1:L1,K1) )
                ENDDO
             ENDDO
             SWF%Egs_sph(K1,:,:,:,M) = SWF%Egs_sph(K1,:,:,:,M)/XM(K1)
          ENDDO  ! over Gauss angled (K1)
       ENDDO  ! over harmonics (M)
       ! Calculation of the phase matrix 
       ! "local solar angles" ==> "Gauss angles" along L-of-S
    CASE('vect_rte')
       ALLOCATE(Dp2_S(3,Nlg_Ray,NLO,N_Eff), Dm2_S(3,Nlg_Ray,NLO,N_Eff)) ! A
       DO K1 = 1, NLO
          K2 = SWF%Ief_SOL(K1)
          DO KK = 1, K2
             XSOL(KK) = E_CSZA(SWF%IEffS(K1,KK),'R')
          END DO
          DO M = 1, 3
             CALL WIGNER_PLG( XSOL(1:K2), K2, M-1, Leg, Plm_S(M,:,K1,1:K2) )
             Pop => Dp2_S(M,:,K1,1:K2) ; Pom => Dm2_S(M,:,K1,1:K2)
             SIGN_M = 1 - 2*MODULO(M-1,2)
             CALL WIGNER_FUNCTION(XSOL(1:K2), K2, M-1, Leg, Pop, Pom)
             Pop(:,:) = 0.5d0*( Pop(:,:) + Pom(:,:) )
             Pom(:,:) = Pop(:,:) - Pom(:,:)
             Pop(:,:) = SIGN_M*Pop(:,:) ! Rlm(mu)
             Pom(:,:) = SIGN_M*Pom(:,:) ! Tlm(mu)
          ENDDO  ! over harmonics (M)
       ENDDO  ! over line-of-sight (K1)
       DO M = 1, 3
          CALL PHASE_MATR_SPH( M, NSTRMS, NLO, Leg, SRR%Ram_MOMS, SWF%Ief_SOL,&
               Plm_S(M,:,:,:), Dp2_S(M,:,:,:), Dm2_S(M,:,:,:), &
               SWF%Egs_sph(:,:,:,:,M) )
       ENDDO
       ! Output for Jl = 1, and JL = 2
       ! OPEN(41,file = './DATA_OUT/test-0.dat')
       ! DO K2 = 1,NLO
       !    WRITE(41,*) ' LoS ',K2
       !    Ks = (K2-1)*Nvs
       !    
       !    DO K1 = 1,NSTRMS 
       !       WRITE(41,*) ' Gauss ',K1
       !       K11 = 1
       !       DO Ka = (K1-1)*Nvs+1,K1*Nvs
       !          WRITE(41,'(2(1x,I3),4(e13.5),2x,4(e13.5))') &
       !              K1, K11, SWF%Egs_sph(Ka,Ks+1:K2*Nvs, 1, 1, 1),&
       !              SWF%Egs_sph(Ka,Ks+1:K2*Nvs, 2, 1, 1)
       !          K11 = K11 + 1
       !       ENDDO               ! Stokes comp. for Gauss angles (Ka)
       !    ENDDO                  ! Gauss angles (K1)         
       ! ENDDO                     ! Line-of=sight (K2)
       ! CLOSE( 41 )     
       ! STOP
    END SELECT                ! Vec_Sca

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END SUBROUTINE RAM_EGS_SPH
  
  !**********************************************************************
  SUBROUTINE LOS_TRANS (*)
    USE ST_VARS,     ONLY: RAD_FIELD
    INTEGER Jv
    REAL (KND)  Fre_LS(NLSA), LS_co(NLSA)
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'LOS_TRANS'
    
    IF( Sec_order ) RETURN
    SELECT CASE( TRIM(RTM_MODE) )
    CASE('flux', 'spher_alb')  ;  RETURN
    END SELECT
    If( RAD_FIELD.AND.(.NOT.WF_FLAG).AND.SCATT_MODE /='ss' ) RETURN

    CALL ST_TRACK (R_NAME, 1, 'I')

    ! Transmission along line-of-sight in the atmosphere
    SELECT CASE (TRIM(RTM_TYPE))
    CASE( 'spher_scat' )
       ! Approximation of transmission along LS by a sum of exponents
       CALL APPR_SPHER_LS_MAIN ( NLSA, AL, SLS, SWF, INT_MS_FLAG(LPOS), *880 )
       GSL_A%EXL = 0.d0 ;  IA = 1
       DO Jv = 1, NLSA
          K = Jv*NFA - LW_FOV(Jv)
          IF( GeoD%LVG(IA,K)%C_VA(1) > 0.d0) THEN
             IF(GeoD%LVG(IA,K)%NTH < GeoD%LVG(IA,K)%NLS) CYCLE
             J = GeoD%LVG(IA,K)%NTH
             GSL_A%EXL(NLAYERS,Jv) = SLS%LSE(IA,K)%EX(J)
          ENDIF  ! Contribution from surface
          ! WRITE(*,*) '   K and SLS%LSE(IA,K)%EX(J) ', K
          ! WRITE(*,*) SLS%LSE(IA,K)%EX(:)
       ENDDO                  ! over LoS angles (Jv)
       ! WRITE(*,*)
       ! WRITE(*,*) '   See surface Exp: ', NLSA, GSL_A%EXL(NLAYERS,:) 
       ! WRITE(*,*)

    CASE ('pp_scat','ps_scat')
       GSL_A%EXL = 0.d0 ; IOB = NOA(1) ; IA = 1
       !===============================================================
       IF( UWT ) THEN
          SELECT CASE( GeoD%VIEW(IA)%S )
          CASE('up') 
             ! Measured upward radiation in coupled OA system
             SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
                ! Observer in the atmosphere
             CASE('Atmos')
                FORALL( I = 1:NLSA ) GSL_A%EXL(IOB:NLAYERS,I) = &
                     DEXP(-( T_AU(IOB:NLAYERS)-T_AU(IOB) )/MuV(I))
                ! Preliminary under assumption that "instrument" in the
                ! atmosphere and directed downward. Ignoring roughness
                ! Fresnel reflection coefficient for line-of-sight "beam"
                ! considered as pseudo-forward source.
                ! This routine is located in "trans_refl_matr.f"
                CALL Fre_REF (NLSA, Fre_LS, GSL_A%MuV, UDB_A%Ref_wd, 'aa')
                LS_co(:) = GSL_A%MuV(:)*(1.d0 - Fre_LS(:))
                FORALL( I = 1:NLSA ) GSL_W%EXL(:,I) = EXP(-T_AU_W(:)/&
                     GSL_W%MuV(I))*GSL_A%EXL(NLAYERS,I)*LS_co(I)/GSL_W%MuV(I)
                ! Observer in the ocean
             CASE('Ocean')          
                FORALL( I = 1:NLSA ) GSL_W%EXL(IOB:NL_W,I) = &
                     DEXP(-( T_AU_W(IOB:NL_W)-T_AU_W(IOB) )/MuV(I))
             END SELECT  !  observer positions
          CASE('down') 
             ! Measured downward radiation in coupled OA system
             SELECT CASE (TRIM(RTM_MODE))
             CASE('wf','ret')
                STOP ' Option "down" is not implemented yet '
             END SELECT
          CASE('both')
             STOP ' Option "both" is not implemented yet '
          END SELECT  ! 'up', 'down' and 'both' in coupled model
          !===============================================================
       ELSE  ! UWT = .FALSE.
          Lst => GeoD%LVG(IA,:)%IALS ; Lfi => GeoD%LVG(IA,:)%IALE
          DO Jv = 1,NLSA  !;  Js = Jv*NFA - LW_FOV(Jv) 
             Js = Jv  !  No field-of-view in pp_scat option
             IF( GeoD%LVG(IA,Js)%C_VA(1) > 0.d0) THEN ! upward radiation
                FORALL( KK = Lst(Js):Lfi(Js)+1 ) GSL_A%EXL(KK,Jv) = &
                     DEXP( -(T_AU(KK)-T_AU(Lst(Js)) )/MuV(Jv))
             ELSE  ! downward radiation
                FORALL( KK = Lst(Js):Lfi(Js)+1 )  GSL_A%EXL(KK,Jv) = &
                     DEXP( -( T_AU(Lfi(Js)+1) - T_AU(KK))/MuV(Jv) )
             ENDIF  ! upward or downward radiation
          ENDDO  ! over LoS (Jv)
          !===============================================================
          ! WRITE(*,*) '     GSL_A%EXL(KK,Jv) in "dom_spherical" ', &
          !    Lst(1), Lfi(1)  
          ! DO I = 1, NLAYERS
          !    WRITE(*,'(2x,F8.3, 2x,e12.4)')  ZZZ(I), GSL_A%EXL(I,1)
          ! ENDDO               
       ENDIF
!===============================================================
    END SELECT

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
  END SUBROUTINE LOS_TRANS

  !**************************************************************************
  SUBROUTINE SINGLE_SCAT(*)
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SINGLE_SCAT'

    SELECT CASE (TRIM(RTM_TYPE))
    CASE( 'spher_scat' )
       ! Single scattering approximation for the calculation of 
       ! radiation (Stokes vector) along line-of-sights in the 
       ! spherical atmosphere as well as WFs using the routine
       ! "spher_wf.f"
       CALL ST_SS_STOKES(LPOS, JJESFT, POS, LOA, E_W, UDB_A%ALBEDO, &
            UDB_A%FLU_int, OSM, TAC, PWF, SRR, SRR_Jact, SLS%LSE,&
            SLS%IND_LS, TE, ADI, RADOUT)
    CASE DEFAULT
         
       ! Single scattering approximation for the calculation of 
       ! reflected and transmitted radiation (Stokes vector) 
       ! in the plane-parallel or pseudo-spherical approximation
       ! WRITE(*,*) '  befor gpp_ss Al ', LOA%AL(NLAYERS-1:NLAYERS)
       CALL GPP_SS_STOKES(LPOS, JJESFT, Iot, POS, UDB_A%ALBEDO, E_W, OSM, &
            GSL_A, LOA, LWF, PWF, SSB_A, TE, SRR, SRR_Jact, RADOUT, TAC, &
            Abs_soot, UDB_A%Flu_int, *880)
    END SELECT

    RETURN
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  END SUBROUTINE SINGLE_SCAT
  
  !*************************************************************************
  SUBROUTINE DEPOL_FAC_WATER(*)
    USE WATER,     ONLY: DEPOL_WATER, DEP_W_INPUT
    !--------------------------------------------------------------------
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'DEPOL_FAC_WATER'
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    SELECT CASE( TRIM( IWS(2)%DEPOL_WATER) )
    CASE('constant')
       IF( IWT ) THEN
          ! Depolarization will be used from the "control_uwt.inp" file
          DEP_W = IWS(2)%DEP_W_INPUT
       ELSE
          IF( LoIW(1) ) DEP_W = IWS(1)%DEP_W_INPUT ! from "control_uit.inp"
          IF( LoIW(2) ) DEP_W = IWS(2)%DEP_W_INPUT ! from "control_uwt.inp"
       ENDIF
    CASE('from file')
       CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
       WRITE(EU,*) '   The option "Depolarization ratio of water "from file"" '
       WRITE(EU,*) '   is not implemented yet.'
       WRITE(EU,*) ' ---> Called by '//TRIM(R_NAME)
       RETURN 1
    END SELECT
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
      
  END SUBROUTINE DEPOL_FAC_WATER
  !***********************************************************************

  SUBROUTINE TEM_PRE_NUMERIC(*)
    USE ESFT,  ONLY: ESFT_OVERLAP_MODE, ESFT_ALPHA, ESFT_COEFF, &
                     ESFT_MOL_OVERLAP_IDX1, ESFT_MOL_OVERLAP_IDX2,&
                     ESFT_NCOEFF, ESFT_WEIGHTS, D_ESFT_COEFF_DT, &
                     D_ESFT_COEFF_DP
    USE ST_VARS,ONLY: WL_GRID
    !------------------------------------------------------------------

    CHARACTER (LEN=15), PARAMETER :: R_NAME = 'TEM_PRE_NUMERIC'
    INTEGER  NSPEC
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')
    
    IF( NUM_TEM ) THEN 
       TEMP(:) = TEMP(:) + PERTURB(:)
       ! Variation of air density (linear approximation)
       ! caused by absolute variation of temperature.
       AIR_DENSITY(:) = AIR_DENSITY(:) - AIR_DENSITY(:)/Temp(:)*PERTURB(:)
       IF( DO_ESFT ) THEN
          ! Numerical perturbation is performed for a single
          ! wavelength therefore            
          NSPEC = 1
          CALL GT_ESFT_PARAMETER_SETUP(WL_GRID, NSPEC, *880)
       END IF
       IF( DO_LBL ) THEN
          CALL ST_PART_SUMS (*880)
       END IF
    ENDIF
    IF( NUM_PRE ) THEN 
       PRESS(:) = PRESS(:)*PERTURB(:)
       AIR_DENSITY(:) = AIR_DENSITY(:)*PERTURB(:)
    ENDIF
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
  END SUBROUTINE TEM_PRE_NUMERIC
  
  !********************************************************************
  SUBROUTINE OUT_INFOR(*)
    USE AEROSOLS,    ONLY: LA_S, IDX_AER
    USE CLOUDS,      ONLY: NCW => N_CLD_NETWAVS, CLW => CLD_NETWAVS
    USE DELTA_M,     ONLY: D_MF_AER, AER_TRUNC_IDX
    USE IRRAD,       ONLY: SOLSPEC => SOL_FLUX, EXTR_FLUX
    USE PROFILES,    ONLY: TTT => TEMP
    USE ST_VARS,     ONLY: PIE
    USE THERMAL_VAR, ONLY: BTEM, WNUMLO, WNUMHI, INTEGR_PLANCK
    !------------------------------------------------------------------

    CHARACTER (LEN=*), PARAMETER ::  R_NAME = 'OUT_INFOR'
    INTEGER    I, K, J, L, LOSA, LOZ, Ws, Ws1
    REAL (KND) S1, HDZ, F_DM, DC, ASC(NLAYERS), Mlos(NLSA), SCA_S(NLAYERS),&
         S_MOM(LA_N,NLEGEN), ABS_S(NLAYERS), SHE(NLEGEN), &
         CLD_HEL(NLAYERS,NLEGEN)
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')

    IF( 2*NSTRMS .NE. NLEGEN-1 ) THEN
       CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
            'Comparison to the DISORT incorrect' )
       WRITE(EU,*) '    Number of stream : ', NSTRMS 
       WRITE(EU,*) '    Number of moments: ', NLEGEN 
       WRITE(EU,*) '    Comparison to DISORT requires '// &
            '2*(Number of stream) = Number of moments - 1' 
       RETURN 1
    ENDIF
    
    IF( AER_TRUNC_IDX .NE. NLEGEN-1 ) THEN
       CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Comparison to DISORT incorrect')
       WRITE(EU,*) '    Number of moments: ',  NLEGEN
       WRITE(EU,*) '    Truncation index : ',  AER_TRUNC_IDX
       WRITE(EU,*) '    Comparison to DISORT requires '//&
            'Number of moments - 1 = Truncation index' 
       
       RETURN 1
    ENDIF
    
    SELECT CASE( ANG_SEL_FLAG )
    CASE( 'one' )
       Mlos(:) = DABS( ANG_OUT(2,1:NLSA) )
    CASE( 'all' )
       Mlos(:) = DABS( ANG_OUT(2,1:NLSA*Naz:Naz) )
    END SELECT
    
    IF( RTM_MODE == 'flux' ) THEN
       LOSA = 1 ; Mlos(1) = 1.0
       LOZ = 1
       OPEN(C_UNIT,file = DIR_OUT//'/to_disort-geom.dat')
       WRITE(C_UNIT,*) '   '
       
       ! Cosine of solar zenith angles
       WRITE(C_UNIT,*) NSZA
       DO I = 1,NSZA
          WRITE(C_UNIT,*) DCOS( SOL_ANG(I)/RAD_TO_DEG )
       ENDDO

       ! Cosine of line-of-sight angles
       WRITE(C_UNIT,*) LOSA
       DO I = 1,LOSA
          WRITE(C_UNIT,*) Mlos(I)
       ENDDO
       
       ! Azimuthal angles
       WRITE(C_UNIT,*) LOZ
       DO I = 1,LOZ
          WRITE(C_UNIT,*) ANG_OUT(3,I)*RAD_TO_DEG
       ENDDO
       CLOSE(C_UNIT)
    ELSE
       IF( NLSA > 1 ) THEN
          DO I = 2,NLSA
             IF( Mlos(I) <= Mlos(I-1) ) THEN
                CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
                     'Comparison to DISORT incorrect')
                WRITE(EU,*) '    Comparison to DISORT requires that '
                WRITE(EU,*) '    cosines line-of-sight angles decrease '
                RETURN 1
             ENDIF
          ENDDO
       ENDIF

       OPEN(C_UNIT,file = DIR_OUT//'/to_disort-geom.dat')
       WRITE(C_UNIT,*) '   '
         
       ! Cosine of solar zenith angles
       WRITE(C_UNIT,*) NSZA
       DO I = 1,NSZA
          WRITE(C_UNIT,*) DCOS( SOL_ANG(I)/RAD_TO_DEG )
       ENDDO

       ! Cosine of line-of-sight angles
       WRITE(C_UNIT,*) NLSA
       DO I = 1,NLSA
          WRITE(C_UNIT,*) Mlos(I)
       ENDDO
       
       ! Azimuthal angles
       WRITE(C_UNIT,*) Naz
       DO I = 1,Naz
          WRITE(C_UNIT,*) ANG_OUT(3,I)*RAD_TO_DEG
       ENDDO
       CLOSE(C_UNIT)
    ENDIF
    
    SELECT CASE( TRIM(EXTR_FLUX) )
    CASE('Pi')    ;  S1 = PIE
    CASE('Unity') ;  S1 = 1.d0
    CASE('File')  ;  S1 = Fbe*SOLSPEC(LPOS)
    END SELECT
    
    OPEN(C_UNIT,file = DIR_OUT//'/to_disort.dat')
    WRITE(C_UNIT,*) NLAYERS-1, 2*NSTRMS, UDB_A%ALBEDO
    WRITE(C_UNIT,*) S1, DO_PLANCK
    IF( DO_PLANCK ) THEN
       WRITE(C_UNIT,*) INTEGR_PLANCK
       IF( INTEGR_PLANCK ) THEN              
          WRITE(C_UNIT,*) BTEM, WNUMLO, WNUMHI
       ELSE
          ! Wavenumber [cm-1]
          WRITE(C_UNIT,*) BTEM, 1.d0/(POS*1.D-07)
       ENDIF
       DO I = 1,NLAYERS
          WRITE(C_UNIT,*) TTT(I)
       ENDDO
    ENDIF
    CLOSE(C_UNIT)
    
    SCA_S(:) = SCR(:)
    
    WHERE( (1.d0-LOA%DM_A(:)) == 0.d0 )
       ASC(:) = SCA(:)
    ELSEWHERE
       ASC(:) = SCA(:)/(1.d0 - LOA%DM_A(:))
    END WHERE
    
    ABS_S(:) = EXA(:) - SCA(:)
    SCA_S(:) = SCA_S(:) + ASC(:)
    CLD_HEL = 0.d0
    IF( DO_CLOUD ) THEN
       ABS_S(:) = ABS_S(:) + (EXC(:) - SCC(:))
       SCA_S(:) = SCA_S(:) + LOA%SCC_NOSCALE(:,1)
       ! SCA_S(:) = SCA_S(:) + SCC(:)
       CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, NCW, CLW)
       ! WRITE(*,*) ' dom_spherical:'
       IF( Nphr > 1 ) THEN
          WRITE(*,*) '  Inhomogeneous phase function is not implemented'
          WRITE(*,*) '         for the comparison with DISORT! '
          WRITE(*,*) '  Check "out_infor.f" in "dom_spherical.f"'
          STOP
       ENDIF
       DO J = 1,N_CLD_LAYERS
          F_DM = (1.d0 - DC)*CLD_F_DM(Ws,J,1,1) + DC*CLD_F_DM(Ws1,J,1,1)
          DO I = J_TOP(J)+1, J_BOT(J)-1
             CLD_HEL(I,NLEGEN) = F_DM*DBLE(2*CLD_DM(J) + 1)
             DO L = 1,NLEGEN-1
                CLD_HEL(I,L) = LOA%CLD_MOMS(L,J,1,1,1)*(1.d0 - F_DM) + &
                     F_DM*DBLE(2*L-1)
             ENDDO
          ENDDO
       ENDDO
    ENDIF
    
    ! Total optical thickness and single scattering albedo of each layer
    !-------------------------------------------------------------------
    OPEN(C_UNIT,file=DIR_OUT//'/to_disort-2.dat')
    DO J = 1,NLAYERS - 1
       HDZ = 0.5D0*DZR(J)
       S1 = (TGA(J) + TGA(J+1)) ! gaseous absorption
       S1 = S1 + (ABS_S(J) + ABS_S(J+1)) ! aerosol+cloud absorption 
       S1 = S1 + (SCA_S(J) + SCA_S(J+1)) ! all scattering
       WRITE(C_UNIT,*) S1*HDZ, (SCA_S(J) + SCA_S(J+1))/S1
    ENDDO
    CLOSE(C_UNIT)
    
    ! Moments of the scattering functions
    !------------------------------------      
    OPEN(C_UNIT,file=DIR_OUT//'to_disort-moms.dat')
    DO I = 1,LA_N
       F_DM = LOA%DM_A(LA_S(I))
       S_MOM(I,NLEGEN) = F_DM*DBLE(2*AER_TRUNC_IDX + 1)
       DO L = 1,NLEGEN-1
          S_MOM(I,L) = LOA%AER_MOMS(L,I,1)*(1.d0 - F_DM) + F_DM*DBLE(2*L-1)
       ENDDO
    ENDDO
    ! RAY_MOMS(Nlg_Ray,Nm_vs)
    DO K = 1,NLAYERS-1
       S1 = (SCA_S(K) + SCA_S(K+1))/2.0
       SHE(:) = (S_MOM(IDX_AER(K),:)*ASC(K) &
            + S_MOM(IDX_AER(K+1),:)*ASC(K+1))/S1/2.d0
       SHE(:) = SHE(:) +  (CLD_HEL(K,:)*  LOA%SCC_NOSCALE(K,1) &
            + CLD_HEL(K+1,:)*LOA%SCC_NOSCALE(K+1,1))/S1/2.d0
       SHE(1:3) = SHE(1:3) + LOA%Ray_moms(1:3,1)*(SCR(K) + SCR(K+1))/S1/2.d0
       WRITE(C_UNIT,'(100(1xE15.7))') (SHE(J)/DBLE(2*J-1), J=1,NLEGEN)
    ENDDO
    
    CLOSE(C_UNIT)
    
    CALL ST_TRACK (R_NAME, 1, 'O')
  END SUBROUTINE OUT_INFOR
  
  !********************************************************************
  SUBROUTINE MAKE_NEW_GRID(Mha, COMBINE)
    
    LOGICAL, INTENT(IN) :: COMBINE
    INTEGER, INTENT(IN) :: Mha
    !------------------------------------------------------------------
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'MAKE_NEW_GRID'
    LOGICAL     MASK_NL(NLAYERS-1), MODIFY
    INTEGER     NL, I1, Jv, K1, E_MIN(NLAYERS-1), Id_a(NLAYERS-1), &
         Ja_TOP(1), Ja_BOT(1)
    REAL (KND)  Wdo, Crit_loc, S2, Ore, Oae, Ocl, CV, SCA_Lay,&
         Ome_sr(NLAYERS,3), D_ome(NLAYERS-1,3)        
    REAL (KND), PARAMETER :: Lim=1.D-15, Lim_crit=1.0d-5
    INTEGER, ALLOCATABLE :: E_uw(:), Id_uw(:)
    LOGICAL, ALLOCATABLE :: MASK_uw(:)
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')
    
    ! IF( JJESFT > 1 ) RETURN
    IF( ARB_OL .AND. .NOT.AGR_A%ADAPTIVE_GRID) THEN
       ! WRITE(*,*) '    ?????  '
       ! PAUSE
       MASK_NL(:) = .FALSE. ; MASK_NL(1) = .TRUE. 
       DO I = 1,NLAYERS - 1
          IF( .NOT.GRID_MARK(I) ) MASK_NL(I)=.TRUE.
       ENDDO
       I = COUNT(MASK_NL) ; NRL = I + 1
       E_MIN(:) = (/(I, I = 1,NLAYERS-1)/)
       Id_a(1:I) = PACK(E_MIN,MASK_NL)
       Lay_gr(1:I,1) = Id_a(1:I)
       DO K = 1,NRL-2
          Lay_gr(K,2) = Lay_gr(K+1,1) - 1
       ENDDO
       Lay_gr(NRL-1,2) = NLAYERS-1
       !OPEN( 11,file = Work_Dir//'/lay_gr_a.dat')
       !WRITE(11,'(2x,i3)') NRL-1
       !DO K = 1,NRL-1
       !   WRITE(11,'(2(2x,e15.7,2x,i3))') ZZZ(Lay_gr(K,1)), Lay_gr(K,1),&
       !        ZZZ(Lay_gr(K,2) + 1), Lay_gr(K,2) + 1
       !ENDDO
       !CLOSE(11)
       AGR_A%ADAPTIVE_GRID = .TRUE.
       N_OBS_AG = NOA(1)
       ! write(*,*) '    n_obs_ag = ',N_OBS_AG
       ! stop

       ! Additional user-defined output levels in ocean
       IF( .NOT.UWT ) RETURN
       AGR_W%NL = NL_W
       ALLOCATE( Id_uw(NL_W), E_uw(NL_W), MASK_UW(NL_W) )
       MASK_uw(:) = .FALSE. ; MASK_UW(1) = .TRUE. 
       DO I = 1,NL_W - 1
          ! WRITE(*,*) I, ZZZ_UW(I), GRID_MARK_UW(I)
          IF( .NOT.GRID_MARK_UW(I) ) MASK_UW(I) = .TRUE.
       ENDDO
       I = COUNT( MASK_uw ) ; AGR_W%NRL = I + 1
       E_uw(:) = (/(I, I = 1,NL_W-1)/)
       Id_uw(1:I) = PACK(E_uw,MASK_uw)
       AGR_W%Lay_gr(1:I,1) = Id_uw(1:I)
       DO K = 1,AGR_W%NRL-2
          AGR_W%Lay_gr(K,2) = AGR_W%Lay_gr(K+1,1) - 1
       ENDDO
       AGR_W%Lay_gr(AGR_W%NRL-1,2) = NL_W-1
       !OPEN( 11,file = Work_Dir//'/lay_gr_o.dat')
       !WRITE(11,'(2x,i3)') AGR_W%NRL-1
       !DO K = 1,AGR_W%NRL-1
       !   WRITE(11,'(2(2x,e15.7,2x,i3))') ZZZ_UW(AGR_W%Lay_gr(K,1)),&
       !        AGR_W%Lay_gr(K,1), ZZZ_UW(AGR_W%Lay_gr(K,2) + 1), &
       !        AGR_W%Lay_gr(K,2)+1
       !ENDDO
       !CLOSE(11)
       AGR_W%ADAPTIVE_GRID = .TRUE.
       ! WRITE(*,*) '   NRL ocean with ORB_OL: ',AGR_W%NrL
       CALL ST_TRACK (R_NAME, 1, 'O')
       RETURN
    ENDIF  ! ARB_OL = .true.

    !------------------------------------------------------------------
    MODIFY = .TRUE. 
    IF( .NOT.AGR_A%ADAPTIVE_GRID ) THEN
       ! Altitude grid without modification for all harmonics
       ! WRITE(*,*) '  AG_aer  ', AG_aer
       IF( .NOT.AG_aer )  MODIFY = .FALSE.
       ! Altitude grid without modification 
       ! WRITE(*,*) '  COMBINE  ', COMBINE
       IF( .NOT.COMBINE ) MODIFY = .FALSE.
    ELSE
       IF( Mha <  Mh_AG.AND.Crit_AG    < Lim_crit) MODIFY = .FALSE.
       IF( Mha >= Mh_AG.AND.Crit_AG_Mh < Lim_crit) MODIFY = .FALSE.
    ENDIF
    
    IF( .NOT.MODIFY ) THEN
       NRL = NLAYERS  ;  N_OBS_AG = NOA(1)
       DO I = 1,NLAYERS-1
          LAY_GR(I,1) = I   ;  LAY_GR(I,2) = I
       ENDDO
       AGR_A%ADAPTIVE_GRID = .FALSE.
       ! WRITE(*,*) ' -------> ADAPTIVE_GRID = ', ADAPTIVE_GRID 
       ! WRITE(*,*) ' -- AGR_A%ADAPTIVE_GRID = ', AGR_A%ADAPTIVE_GRID
       ! WRITE(*,*) ' ------->        MODIFY = ', MODIFY, Mha
       ! WRITE(*,*) ' *****  No modification of altitude grid ******'
       CALL ST_TRACK (R_NAME, 1, 'O')
       RETURN
    ELSE
       AGR_A%ADAPTIVE_GRID = .TRUE.
       ! WRITE(*,*) ' -------> ADAPTIVE_GRID = ', ADAPTIVE_GRID 
       ! WRITE(*,*) ' -- AGR_A%ADAPTIVE_GRID = ', AGR_A%ADAPTIVE_GRID
       ! WRITE(*,*) ' ------->        MODIFY = ', MODIFY, Mha
    ENDIF
    !------------------------------------------------------------------

    NL = NLAYERS ; Wdo = 1.d0 - Wup ; Ome_sr = 0.d0
    N_OBS_AG = NOA(1)
    MASK_NL(:) = .FALSE. ;  MASK_NL(1) = .TRUE.
    
    ! WRITE(*,*) '   SHAPE( MASK_NL ) ', SHAPE( MASK_NL )
    ! Select levels corresponding to the tangent height of the 
    !                   solar beam
    !----------------------------------------------------------------
    FORALL( I = 1:NSZA, SSB_A%N_sha(I) <= NL-1) &
         MASK_NL(SSB_A%N_sha(I)) = .TRUE.

    ! WRITE(*,*) '   - 1 '

    !Select levels corresponding to the tangent height of the 
    !           line-of-sight ( if "spher_scat" mode only )
    !----------------------------------------------------------------
    IF( Enhance_acc ) THEN
       DO Jv = 1,NLSA  ;  I1 = Jv*NFA - LW_FOV(Jv)
          IF( N_below(I1) <= NL-1 ) MASK_NL( N_below(I1) ) = .TRUE.
       ENDDO
       ! I1 = NFA - LW_FOV(1)
       ! FORALL( I = I1:NLSA*NFA:NFA, N_below(I) <= NL-1) &
       !     MASK_NL(N_below(I)) = .TRUE.
    ENDIF

    ! WRITE(*,*) '   - 2 '

    ! Select level corresponding to the position of observer
    IF( NOA(1) <= NL-1 ) MASK_NL(NOA(1)) = .TRUE.
    E_MIN(:) = (/(I, I = 1,NL-1)/)

    ! WRITE(*,*) '   - 3 '
         
    ! Single scattering albedo in all standard layers
    D_Ome(1:NL-1,1) = (Wup*SCR(1:NL-1) + Wdo*SCR(2:NL))/ALW
    D_Ome(1:NL-1,2) = (Wup*SCA(1:NL-1) + Wdo*SCA(2:NL))/ALW
    D_Ome(1:NL-1,3) = (Wup*SCC(1:NL-1) + Wdo*SCC(2:NL))/ALW
    
    !----------------------------------------------------------------
    ! Select levels corresponding to positions of the cloud top  
    !                 and cloud bottom heights
    IF( DO_CLOUD ) THEN 
       DO I = N_CLD_LAYERS,1,-1
          MASK_NL(J_TOP(I)+1)=.TRUE. ; MASK_NL(J_BOT(I)-1)=.TRUE.
       ENDDO
    ENDIF

    ! WRITE(*,*) '   - 4 '

    !----------------------------------------------------------------
    IF( DO_AEROSOL ) THEN 
       ! Select top of aerosol layer
       DO I = 1, NL-1
          IF( REAL((SCA(I) + SCA(I+1))/2.d0) > 0.d0 ) THEN
             MASK_NL(I)=.TRUE. ; Ja_TOP(1) = I ; EXIT
          ENDIF
       ENDDO
       ! Select bottom of aerosol layer
       DO I = NL,2,-1
          IF( REAL((SCA(I) + SCA(I-1))/2.d0) > 0.d0 ) THEN
             Ja_BOT(1) = I
             IF( I==NL ) EXIT
             MASK_NL(I)=.TRUE. ; EXIT
          ENDIF
       ENDDO
       ! WRITE(*,*) '  Top and bottom of aerosol layer: '
       ! WRITE(*,*)  ZZZ(Ja_TOP(1)), ZZZ(Ja_BOT(1))
    ENDIF  ! DO_AEROSOL
    !----------------------------------------------------------------

    ! WRITE(*,*) '  AG and COMBINE ', AGR_A%ADAPTIVE_GRID, COMBINE, Mha

    ! Use different criterion for harmonic smaller and larger Mh_AG
    IF( Mha >= Mh_AG ) THEN
       Crit_loc = Crit_AG_Mh
    ELSE
       Crit_loc = Crit_AG
    ENDIF
    
    I1 = 1 ; K1 = 3 
    DO I = 1,NL-1
       Oae = 0.d0 ; Ocl = 0.d0
       S2 = SUM( ALW(I1:K1-1) )
       IF( SCR(NL) > 0.0d0 ) Then
          Ore = (SUM(SCR(I1+1:K1-1)) + Wup*SCR(I1) + Wdo*SCR(K1))/S2
          Ome_sr(I1:K1-1,1) = D_Ome(I1:K1-1,1)/Ore - 1.0d0
       ENDIF
       IF( SCA(K1) > Lim ) Then
          Oae = (SUM(SCA(I1+1:K1-1)) + Wup*SCA(I1) + Wdo*SCA(K1))/S2
          Ome_sr(I1:K1-1,2) = D_Ome(I1:K1-1,2)/Oae - 1.0d0
       ENDIF
       IF( SCC(K1) > 0.d0 ) Then
          Ocl = (SUM(SCC(I1+1:K1-1)) + Wup*SCC(I1) + Wdo*SCC(K1))/S2
          Ome_sr(I1:K1-1,3) = D_Ome(I1:K1-1,3)/Ocl - 1.0d0
       ENDIF
       CV = MAXVAL( DABS(Ome_sr(I1:K1-1,1:3)) )
       
       ! WRITE(*,'(2(2x,I3),A,I3,2(2x,e12.4),2x,F8.3)') I, I1, ' - ',K1,&
       !    REAL( (SCA(I) + SCA(I+1))/2.d0 ), REAL(Oae), ZZZ(I)

       IF( AG_aer.AND.COMBINE.AND.Mha>3 ) THEN
          IF( (REAL(Oae)==0.d0).AND.(REAL(Ocl)==0.d0) ) THEN 
             CV = -1.d0 
          ENDIF
       ENDIF
       IF( CV >= Crit_loc ) THEN 
          MASK_NL(K1-1) = .TRUE.
          I1 = K1-1 ; K1 = I1 + 2
          IF( K1 > NL ) EXIT
       ELSE
          K1 = K1 + 1
          IF( K1 > NL ) EXIT
       ENDIF
    ENDDO  ! over layers of main grid (I)
    
    IF( AG_aer.AND.COMBINE.AND.Mha>3 ) THEN
       ! No grid modification within aerosol layer
       WHERE( REAL(D_OME(:,2)) > 0.d0 ) MASK_NL(:) = .TRUE. 
       ! No grid modification within cloud layers
       WHERE( REAL(D_OME(:,3)) > 0.d0 ) MASK_NL(:) = .TRUE. 
       ! All layers marked AG_ZERO(:)=.TRUE. will be skipped 
       ! calculating Gaussian field in "st_up_down_sfi.f"
       WHERE( REAL(D_OME(:,2))==0.d0.AND.REAL(D_OME(:,3))==0.d0 ) 
          GSL_A%AG_ZERO(:) = .TRUE.
       END WHERE
    ENDIF

    I = COUNT(MASK_NL)
    NRL = I + 1

    Id_a(1:I) = PACK(E_MIN,MASK_NL)
    IF(NOA(1) == NL) THEN
       N_OBS_AG = NRL
    ELSE
       DO K = 1,I
          IF( Id_a(K) == NOA(1) ) THEN
             N_OBS_AG = K
             EXIT
          ENDIF
       END DO
    ENDIF
    
    Lay_gr(1:I,1) = Id_a(1:I)
    DO K = 1,NRL-2
       Lay_gr(K,2) = Lay_gr(K+1,1) - 1
    ENDDO
    Lay_gr(NRL-1,2) = NL-1
    
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       !OPEN( 11,file = Work_Dir//'/lay_gr.dat')
       !WRITE(11,'(2x,i3)') NRL-1
       !! WRITE(*,'(2x,i3)') NRL-1
       !DO K = 1,NRL-1
       !   WRITE(11,'(2(2x,e15.7,2x,i3))')  &
       !   ! WRITE(*,'(2(2x,e15.7,2x,i3))')  
       !        ZZZ(Lay_gr(K,1)), Lay_gr(K,1), &
       !        ZZZ(Lay_gr(K,2) + 1), Lay_gr(K,2) + 1
       !ENDDO
       !CLOSE(11)
       WRITE(LUNIT,*)  '----- ADAPTIVE_GRID ----- '
       WRITE(LUNIT,'(a22,2x,I3)') '    Number of layers :', NRL-1
    ENDIF
    ! WRITE(LUNIT,*)  '----- ADAPTIVE_GRID -- layers  ', NRL-1
    
    CALL ST_TRACK (R_NAME, 1, 'O')
  END SUBROUTINE MAKE_NEW_GRID
  
END SUBROUTINE DOM_SPHERICAL
!***********************************************************************

SUBROUTINE ADD_SOOT(X, Abs_soot)
  USE CLOUDS,      ONLY: NCL => N_CLD_LAYERS, J_TOP, J_BOT, Pr_soot
  USE CLD_INP,     ONLY: ADD_ABS, Inp_form, MLCS, TDS
  USE INTERN_DIMS, ONLY: NL_soot
  USE ST_VARS,     ONLY: Pie
  USE TYPE_VARS,   ONLY: KND, DCMP
  !------------------------------------------------------------------

  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ADD_SOOT'
  REAL (KND) X, Abs_soot(NL_soot)
  INTEGER    I, II, J, JT, JB, IDI
  REAL (KND) ReM, ImM, Res, C_km
  COMPLEX (kind(0.0d0)) :: Cmu
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  
  IDI = 1 ; Abs_soot(:) = 0.d0
  DO I = 1, NCL
     IF( .NOT.ADD_ABS(I).OR. TDS(I)%S=='ice-sm' ) CYCLE
     SELECT CASE( Inp_form(I)%S )
     CASE('analytic')
        C_km = 1.0d3
        IF( X <= 500.d0 ) THEN
           Res = DEXP( -1.5d-2*(X - 390.d0) ) 
        ELSE
           Res = DEXP( -1.5d-2*110.d0 - 1.1d-2*(X - 500.d0) ) 
        ENDIF
     CASE('numeric')
        C_km = 1.0
        II = MLCS(I)%N_WL_SOOT
        ! Interpolation of the real and imaginary part of refr. index
        !------------------------------------------------------------
        CALL LIN_INT(II, MLCS(I)%WL_SOOT, MLCS(I)%REFR_SOOT(:,1), IDI,X,ReM,'C')
        CALL LIN_INT(II, MLCS(I)%WL_SOOT, MLCS(I)%REFR_SOOT(:,2), IDI,X,ImM,'C')
        ! Complex refractive index m = n - i*k
        Cmu = DCMP( ReM,-ImM )
        ! absorption coef. in [km-1] AIMAG - imaginary part of complex number
        Res = -6.d0*Pie/X*AIMAG((Cmu**2 - 1.d0)/(Cmu**2 + 2.d0))*1.d12 
        ! WRITE(*,'(A,2x,e12.4)') '   Res = ', RES
     END SELECT
     !  Top and bottom without infinite sublayerts
     !---------------------------------------------
     JT = J_TOP(I) + 1 ; JB = J_BOT(I) - 1
     Abs_soot(JT:JB) = Pr_soot(JT:JB)*Res*C_km
!     IF( Vhom(I) ) THEN
!        Abs_soot(JT:JB) = Par_mod(I,1)*Res *C_km
!     ELSE
!        Abs_soot(JT:JB) = Pr_soot(JT:JB)*Res*C_km
!     ENDIF
  ENDDO  ! over cloud layers (I)
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE ADD_SOOT
!*************************************************************************
