C********************************************************************C
C
C       SCIATRAN - Version 3.8 - Last modification: 11 Feb 2018
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE EXT_WL_ALL(N_ACT, WL_ACT, SRR, SVR, *)
c     ... Call in "rtm_main.f"

      USE LOG_PARS,  ONLY: EU, LUNIT, DIR_OUT
      USE RAMAN,     ONLY: N_WL_C, DEL_WL, DEL_RAM
      USE ST_VARS,   ONLY: Pie, APPR_RRS, INEL_PROC
      USE TYPE_VARS, ONLY: KND
      USE TYPES,     ONLY: RRS_TYPE, VRS_TYPE
      USE VRS,       ONLY: N_FRF, K_nu, A_FRF, Dnu_FRF, Sig_FRF, A_FChl,
     $                     STEP_VRS, Bet_0R, Nsig, Integr_FRF, Qe_Chl,
     $                     Tvs_Use, Tvs_Exc, Tvs_Act, Lrf => Lam_ref,
     $                     Lmi => Lam_int, Contour, WLG_Chl, Sig_Chl, 
     $                     N_phm
      USE WATER,     ONLY: INCL_INEL, Step_Flu, Wl_Flu, Nwl_QE, 
     $                     WL_QE, A_QE, A_Fa, B_Fa, A_Ha, B_Ha,
     $                     FH_REF, Fu_par, Hu_par, Fua_c, Hua_c, Abs_flu
c     ------------------------------------------------------------------

c     ... Input parameters
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
      INTEGER    N_ACT
      REAL (KND) WL_ACT(N_ACT)
c     ... Local variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'EXT_WL_ALL'
      CHARACTER (LEN=3)  CN1, CN2, CN3
      INTEGER, PARAMETER :: MAX_EXR = 100
      INTEGER   I, J, K, Jr, K1, KK, Ksr, N_EXR, N_AUX, Ne_flu, Lna2,
     $     NG_WL, N1, N2, Nr_Exi, Kma, Ia, Nsv, Nsr, Nwg, Np, Nvr,
     $     N_P(MAX_EXR), Nr_ext, Nr_aux, Na_exi
      REAL (KND), PARAMETER :: D9=1.D+9
      REAL (KND) Ab1, Ab2, An1, An2, Brc, Dn1, Dn2, Dsig, FRF_INT,
     $     Nuj, Nu_ex, Nu_act, Nu_step, Nut, Rho, Wm_CDOM, WLT, DELTA,
     $     S1, S2, S3, Shift, N_mol, G_MIN, G_MAX, G_Step, DEL_CDOM, 
     $     D1L, D1R, Step_VF
      REAL (KND) Nuc(N_FRF), WLv_MIN(MAX_EXR), WLv_MAX(MAX_EXR), 
     $      WLr_MIN(MAX_EXR), WLr_MAX(MAX_EXR)
      LOGICAL, ALLOCATABLE :: L_vrs(:), L_Flu(:), L_Rrs(:), L_exi(:), 
     $     Lo_FLu(:), Lo_VRS(:), Lo_RRS(:), ActR_WL(:), Fi_rrs(:), 
     $     Fi_vrs(:), Fi_Flu(:), L_hel(:), Fi_hel(:)
      REAL (KND), ALLOCATABLE :: WL_Aux(:), WLr_Ext(:), Wle_flu(:),
     $     A_Exi(:,:), WL_INT(:), G_WL(:), WLr_AUX(:), WLr_Exi(:),
     $     WL_wg(:), D1(:), D2(:)
      TARGET :: WL_Aux
      REAL (KND), POINTER :: WL_Ext(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     ... Nr_aux is the number of excitation RRS wavelengths and actual 
c         wavelengths
      Nr_aux = N_ACT*(N_WL_C + 1)
c         Nr_exi is the full number of RRS excitation wavelengths
      Nr_exi = N_Act*N_WL_C 

c     ... Auxiliary grid is given in the array WLr_Aux.
      ALLOCATE( WLr_Aux(Nr_aux), WLr_Exi(Nr_Exi) )

c         It is assumed that for each actual wavelength there is  
c         a sequence of N_WL_C + 1 wavelengths:
c         - first wavelength in this sequence is an actual wavelength, 
c           ---------------------------------------------------------
c         - following N_WL_C wavelengths are excitation RRS wavelengths 
c           which produced Raman lines having the position equal to the
c           actual wavelength. 
      KK = 0
      DO I = 1,N_ACT
         Jr = (I-1)*(N_WL_C+1) + 1  ;   WLr_Aux(Jr) = WL_ACT(I)
         DO J = 1,N_WL_C
c           ... Array DEL_WL is used to define positions of all
c               excitation wavelengths which produced Raman lines 
c               having the position at actual wavelength
            KK = KK + 1 ; WLT = D9/( D9/WL_ACT(I) - DEL_WL(J) )
            WLr_Aux(Jr+J) = WLT
            WLr_Exi(KK)   = WLT
         ENDDO
      ENDDO

c     ... The routine "RANGE_RRS" returns:
c           Nr_ext 
c             - number of spectral points in rotational Raman band;
c           WLr_EXT(1:Nr_ext) 
c             - wavelength grid for rotational Raman band (this grid
c               contains exact positions of all actual wavelengths);
c           Nsr 
c             - number of segments if Raman band is discontinuous;
c           WLr_MIN(1:Nsr) WLr_MAX(1:Nsr) 
c             - minimal and maximal wavelengths for each segment.
      CALL RANGE_RRS

c        Wavelength grid in the spectral range of excitation
c        ===================================================
      SELECT CASE( TRIM(INEL_PROC) )

c     ... VRS, Chlorophyll, CDOM & rotational Raman excitation range
c     --------------------------------------------------------------
      CASE('VRS_Fb','VRS_Fr','VRS_Fr_Fb')
c        ... The number of phase function which will be used
c            to describe all inelastic processes in ocean
c            - Rayleigh and isotropic phase functions -
         N_phm = 2

c        ... Maximal actual wavelength 
         G_MAX = MAXVAL( WL_ACT ) ; DEL_CDOM = 20.d0
c        ... Maximal wavelength where contribution from 
c            CDOM excitation region should be accounted for
         G_MAX = G_MAX + DEL_CDOM
         Wm_CDOM = DMIN1(G_MAX,WL_Flu(2))

c        ... Wavelength grid for fluorescence
         Ne_flu = CEILING( (Wm_CDOM - Wl_Flu(1))/Step_Flu(1) )  + 1
         ALLOCATE( Wle_flu(Ne_flu) )
c        ... Fluorescence excitation range
         Wle_flu(1) = WL_Flu(1)
         DO I = 1, Ne_flu - 1
            Wle_flu(I+1) = Wle_flu(I) + Step_Flu(1)
         ENDDO
         Wle_flu(Ne_flu) = Wm_CDOM

c        ... Number of segments and bounds of VRS excitation range
         CALL RANGE_VRS
                WRITE(*,*) '      '
                WRITE(*,*) '    Number of excitation VRS segments: ',Nsv
                WRITE(*,*) '    Minimal and maximal wavelength:'
                DO I = 1,Nsv 
                   WRITE(*,'(2x,I3,2(2x,F7.2))') I,WLv_MIN(I),WLv_MAX(I)
                ENDDO
                WRITE(*,*) '    Fluorescence range:'
                WRITE(*,'(5x,2(2x,F7.2))') Wle_flu(1), Wle_flu(Ne_flu)

c        ... Min and max wavelengths for VRS excitation range
         Dn1 = MINVAL( WLv_MIN(1:Nsv) ) ; Dn2 = MAXVAL( WLv_MAX(1:Nsv) ) 

c        ... Min and max wavelengths for all spectral range including
c            RRS, VRS and fluorescence excitation ranges
         G_MIN = DMIN1(Dn1,Wle_flu(1),     Wlr_Ext(1)) 
         G_MAX = DMAX1(Dn2,Wle_flu(Ne_flu),Wlr_Ext(Nr_ext))
               WRITE(*,*) 
     $        '    Wavelength range including all inelastic processes:'
               WRITE(*,'(5x,2(2x,F7.2))') G_MIN , G_MAX 

c        ... The step to define WL-grid in full spectral range containing 
c            excitation ranges of RRS, VRS and Flu. inelastic processes 
c            is equal to the step within the RRS excitation range. 
c            This full spectral range will be called in following as 
c            combined excitation range (CER)
         G_Step = DEL_RAM

c        ... Number of wavelengths within CER
         Ng_WL   = CEILING( (G_MAX - G_MIN)/G_Step )  + 1
         ALLOCATE(G_WL(Ng_WL), L_vrs(Ng_WL), L_flu(Ng_WL), L_rrs(Ng_WL),
     $        L_exi(NG_WL) )

c        ... Wavelength grid in CER
         G_WL(1) = G_MIN
         DO I = 1, Ng_WL - 1
            G_WL(I+1) = G_WL(I) + G_Step
         ENDDO
c        ... Set exact value for right bound 
         G_WL(Ng_WL) = G_MAX
                WRITE(*,*) '   Bounds of combined excitation range :'
                WRITE(*,'(5x,2(2x,F7.3))') G_WL(1), G_WL(Ng_WL)

         L_Vrs(:) = .FALSE.
c        ... True elements of array L_vrs(:) mark wavelengths 
c            pertaining to VRS excitation range
         DO I = 1,Nsv 
            WHERE( G_WL(:)>=WLv_MIN(I).AND.G_WL(:)<=WLv_MAX(I) )
               L_Vrs(:) = .TRUE.
            END WHERE
         ENDDO  ! over segments of VRS excitation range

         L_Flu(:) = .FALSE.
c        ... True elements of array L_Flu(:) mark wavelengths 
c            pertaining to fluorescence excitation range
         WHERE( G_WL(:)>=Wle_flu(1).AND.G_WL(:)<=Wle_flu(Ne_flu) )
            L_Flu(:) = .TRUE.
         END WHERE

         L_Rrs(:) = .FALSE.
         DO I = 1,Nsr
c           ... True elements of array L_Rrs(:) mark wavelengths 
c               pertaining to RRS excitation ranges
            WHERE( G_WL(:)>=Wlr_MIN(I).AND.G_WL(:)<=Wlr_MAX(I) )
               L_Rrs(:) = .TRUE.
            END WHERE
         ENDDO  ! over segments of RRS excitation range (I)

c        ... True elements of array L_exi(:) mark wavelengths pertaining 
c            to RRS, VRS and fluorescence excitation ranges. False 
c            elements of array L_exi(:) define "empty" wavelength intervals
c            between excitation regions
         L_exi(:) = L_Vrs(:).OR.L_Flu(:).OR.L_Rrs(:)

c        ... Number of wavelengths in combined excitation range without
c            gap between segments.
         Nwg = COUNT( L_exi(:) )
         ALLOCATE( WL_wg(Nwg), Lo_VRS(Nwg), Lo_RRS(Nwg), Lo_Flu(Nwg) )
c        ... Extract wavelength grid excluding gap
         WL_wg(:)  = PACK(G_WL, L_exi)
         Lo_VRS(:) = PACK(L_Vrs,L_exi)
         Lo_RRS(:) = PACK(L_Rrs,L_exi)
         Lo_Flu(:) = PACK(L_Flu,L_exi)
c        ... .TRUE. elements of arrays Lo_VRS(:), Lo_RRS(:) and Lo_Flu(:)
c             mark wavelengths pertaining to VRS, RRS and fluorescence
c             excitation ranges, respectively.
               OPEN(47,file=DIR_OUT//'WL_wg.DAT-V_R_F')
                 DO I = 1,Nwg
                    S1=0. ; S2=0. ; S3=0.
                    IF( Lo_VRS(I) ) S1 = 1.
                    IF( Lo_RRS(I) ) S2 = 2.
                    IF( Lo_Flu(I) ) S3 = 3.
                    WRITE(47,'(2x,f7.3,3(1x,F4.1))') 
     $                   WL_wg(I), S1, S2, S3
                 ENDDO
                 CLOSE(47)

c        ... At the next step we need to redefine the wavelength grid within
c            VRS and fluorescence excitation ranges which do not coincide with 
c            RRS excitation ranges. It is assumed a priori that integration 
c            step in VRS and fluorescence ranges is always greater than 
c            RRS excitation binning
         ALLOCATE(L_hel(Nwg)) ; L_hel(:)=.FALSE.
         L_hel(1) = .TRUE. ; S1 = WL_wg(1)
c        ... .TRUE. elements of arrays L_hel(:) mark after this do-loop
c            wavelengths of the grid WL_wg(:) with the step corresponding 
c            to the desired integration step within VRS excitation ranges
c            and all wavelengths within RRS excitation ranges
         Step_VF = DMIN1(STEP_VRS,Step_Flu(1))
         DO I = 2,Nwg            
            IF( SNGL(WL_wg(I)) <= SNGL(S1 + Step_VF) ) CYCLE
            L_hel(I-1) = .TRUE. ; S1 = WL_wg(I-1)
         ENDDO
c        ... .TRUE. elements of arrays L_hel(:) mark now wavelengths of 
c             the grid WL_wg(:) which should be selected in the resulting
c             excitation wavelength grid
         L_hel = L_hel.OR.Lo_Rrs

c        ... Number of wavelengths in general excitation range
         N_AUX = COUNT( L_hel(:) )

c        ... Allocate auxiliary wavelength grid containing excitation
c            wavelengths for rotational, vibrational Raman and fluorescence
         ALLOCATE( WL_AUX(N_AUX), SVR%Lo_VRS(N_AUX), Fi_Flu(N_AUX), 
     $        Fi_rrs(N_AUX), Fi_vrs(N_AUX), Fi_hel(N_AUX) )

c        ... Set wavelength grid excluding "empty" wavelength intervals
         WL_AUX(:) = PACK(  WL_wg,L_hel )
         Fi_rrs(:) = PACK( Lo_Rrs,L_hel )
         Fi_vrs(:) = PACK( Lo_vrs,L_hel )
         Fi_Flu(:) = PACK( Lo_Flu,L_hel )

         Fi_hel = Fi_vrs.OR.Fi_Flu
c              ... Output for checking
               OPEN(47,file=DIR_OUT//'WL_aux.DAT-V_R_F')
               DO I = 1,N_AUX
                  CN1 = ' - ' ; CN2 = ' - ' ; CN3 = ' - '
                  IF( Fi_vrs(I) ) CN1 = 'vrs'
                  IF( Fi_rrs(I) ) CN2 = 'rrs'
                  IF( Fi_Flu(I) ) CN3 = 'flu'
                  WRITE(47,'(2x,I5,2x,f7.2,3(2x,A))') 
     $                 I, WL_AUX(I), CN1, CN2, CN3
               ENDDO
               CLOSE(47)

         SVR%Lo_VRS(:) = Fi_vrs(:)
         Fi_vrs(:) = Fi_hel(:)

c------>> End of  VRS, Chlorophyll, CDOM & rotational Raman excitation range

c     ... Fluorescence excitation & rotational Raman excitation range
c     ---------------------------------------------------------------
      CASE('F_Chl','F_CDOM','Fr_Fb')
c        ... The number of phase function which will be used
c            to describe all inelastic processes in ocean
c            - Isotropic phase function only -
         N_phm = 1

c        ... Maximal actual wavelength 
         G_MAX = MAXVAL( WL_ACT )

c        ... There is no fluorescence contribution in selected
c            actual wavelength range
         IF( G_MAX <= WL_Flu(1) ) GOTO 201

c        ... If CDOM fluorescence is involved
         IF( TRIM(INEL_PROC)=='F_CDOM' ) THEN
            DEL_CDOM = 20.d0
c           ... Maximal wavelength where contribution from 
c               excitation region should be calculated
            G_MAX = G_MAX + DEL_CDOM
            Wm_CDOM = DMIN1(G_MAX,WL_Flu(2))
         ELSE
            Wm_CDOM = WL_Flu(2)
         ENDIF

c        ... Wavelength grid for fluorescence
         Ne_flu = CEILING( (Wm_CDOM - Wl_Flu(1))/Step_Flu(1) )  + 1
         ALLOCATE( Wle_flu(Ne_flu) )
c        ... Fluorescence excitation range
         Wle_flu(1) = WL_Flu(1)
         DO I = 1, Ne_flu - 1
            Wle_flu(I+1) = Wle_flu(I) + Step_Flu(1)
         ENDDO
         Wle_flu(Ne_flu) = Wm_CDOM
c        ------------------------------------------------
c            There is no contribution of VRS 
         DN1 = 1.d+20 ;  DN2 = -1.d+20
c        ------------------------------------------------

c        ... Min and max wavelengths for all spectral range including
c            RRS and fluorescence excitation ranges
         G_MIN = DMIN1(Dn1,Wle_flu(1),     Wlr_Ext(1)) 
         G_MAX = DMAX1(Dn2,Wle_flu(Ne_flu),Wlr_Ext(Nr_ext))
               WRITE(*,*) 
     $        '    Wavelength range including all inelastic processes:'
               WRITE(*,'(5x,2(2x,F7.2))') G_MIN , G_MAX 

c        ... The step to define WL-grid in full spectral range containing 
c            excitation ranges of RRS, VRS and Flu. inelastic processes 
c            is equal to the step within the RRS excitation range. 
c            This full spectral range will be called in following as 
c            combined excitation range (CER)
         G_Step = DEL_RAM

c        ... Number of wavelengths within CER
         Ng_WL   = CEILING( (G_MAX - G_MIN)/G_Step )  + 1
         ALLOCATE(G_WL(Ng_WL), L_vrs(Ng_WL), L_flu(Ng_WL), L_rrs(Ng_WL),
     $        L_exi(NG_WL) )

c        ... Wavelength grid in CER
         G_WL(1) = G_MIN
         DO I = 1, Ng_WL - 1
            G_WL(I+1) = G_WL(I) + G_Step
         ENDDO
c        ... Set exact value for right bound 
         G_WL(Ng_WL) = G_MAX

         L_Vrs(:) = .FALSE. ; L_Flu(:) = .FALSE.
c        ... True elements of array L_Flu(:) mark wavelengths 
c            pertaining to fluorescence excitation range
         WHERE( G_WL(:)>=Wle_flu(1).AND.G_WL(:)<=Wle_flu(Ne_flu) )
            L_Flu(:) = .TRUE.
         END WHERE
         L_Rrs(:) = .FALSE.
         DO I = 1,Nsr
c           ... True elements of array L_Rrs(:) mark wavelengths 
c               pertaining to RRS excitation ranges
            WHERE( G_WL(:)>=Wlr_MIN(I).AND.G_WL(:)<=Wlr_MAX(I) )
               L_Rrs(:) = .TRUE.
            END WHERE
         ENDDO   ! over segments of RRS excitation range (I)

c        ... True elements of array L_exi(:) mark wavelengths pertaining 
c            to RRS, VRS and fluorescence excitation ranges. False 
c            elements of array L_exi(:) define "empty" wavelength intervals
c            between excitation regions
         L_exi(:) = L_Vrs(:).OR.L_Flu(:).OR.L_Rrs(:)

c        ... Number of wavelengths in combined excitation range without
c            gap between segments.
         Nwg = COUNT( L_exi(:) )
         ALLOCATE( WL_wg(Nwg), Lo_VRS(Nwg), Lo_RRS(Nwg), Lo_Flu(Nwg) )
c        ... Extract wavelength grid excluding gap
         WL_wg(:)  = PACK(G_WL, L_exi)
         Lo_VRS(:) = PACK(L_Vrs,L_exi)
         Lo_RRS(:) = PACK(L_Rrs,L_exi)
         Lo_Flu(:) = PACK(L_Flu,L_exi)
c        ... .TRUE. elements of arrays Lo_VRS(:), Lo_RRS(:) and Lo_Flu(:)
c             mark wavelengths pertaining to VRS, RRS and fluorescence
c             excitation ranges, respectively.
               OPEN(47,file=DIR_OUT//'WL_wg.DAT-V_R_F')
                 DO I = 1,Nwg
                    S1=0. ; S2=0. ; S3=0.
                    IF( Lo_VRS(I) ) S1 = 1.
                    IF( Lo_RRS(I) ) S2 = 2.
                    IF( Lo_Flu(I) ) S3 = 3.
                    WRITE(47,'(2x,f7.3,3(1x,F4.1))') 
     $                   WL_wg(I), S1, S2, S3
                 ENDDO
                 CLOSE(47)

c        ... At the next step we need to redefine the wavelength grid within
c            VRS and fluorescence excitation ranges which do not coincide with 
c            RRS excitation ranges. It is assumed a priori that integration 
c            step in VRS and fluorescence ranges is always greater than 
c            RRS excitation binning
         ALLOCATE(L_hel(Nwg)) ; L_hel(:)=.FALSE.
         L_hel(1) = .TRUE. ; S1 = WL_wg(1)
c        ... .TRUE. elements of arrays L_hel(:) mark after this do-loop
c            wavelengths of the grid WL_wg(:) with the step corresponding 
c            to the desired integration step within VRS excitation ranges
c            and all wavelengths within RRS excitation ranges
         Step_VF = Step_Flu(1)
         DO I = 2,Nwg            
            IF( SNGL(WL_wg(I)) <= SNGL(S1 + Step_VF) ) CYCLE
            L_hel(I-1) = .TRUE. ; S1 = WL_wg(I-1)
         ENDDO
c        ... .TRUE. elements of arrays L_hel(:) mark now wavelengths of 
c             the grid WL_wg(:) which should be selected in the resulting
c             excitation wavelength grid
         L_hel = L_hel.OR.Lo_Rrs

c        ... Number of wavelengths in general excitation range
         N_aux = COUNT( L_hel(:) )

c        ... Allocate auxiliary wavelength grid containing excitation
c            wavelengths for rotational, vibrational Raman and fluorescence
         ALLOCATE( WL_AUX(N_aux), SVR%Lo_VRS(N_aux), Fi_Flu(N_aux), 
     $        Fi_rrs(N_aux), Fi_vrs(N_aux) )

c        ... Set wavelength grid excluding "empty" wavelength intervals
         WL_aux(:) = PACK(  WL_wg,L_hel )
         Fi_rrs(:) = PACK( Lo_Rrs,L_hel )
         Fi_vrs(:) = PACK( Lo_vrs,L_hel )
         Fi_Flu(:) = PACK( Lo_Flu,L_hel )

              WRITE(*,*) '    WL     Flu VRS RRS '
               DO I = 1,N_aux
                  WRITE(*,'(2x,I5,2x,f7.2,3(2x,L))') 
     $                 I, WL_aux(I), Fi_Flu(I), Fi_vrs(I), Fi_rrs(I)
               ENDDO
c              ... Output for checking
               OPEN(47,file=DIR_OUT//'WL_aux.DAT-V_R_F')
               DO I = 1,N_AUX
                  CN1 = ' - ' ; CN2 = ' - ' ; CN3 = ' - '
                  IF( Fi_vrs(I) ) CN1 = 'vrs'
                  IF( Fi_rrs(I) ) CN2 = 'rrs'
                  IF( Fi_Flu(I) ) CN3 = 'flu'
                  WRITE(47,'(2x,I5,2x,f7.2,3(2x,A))') 
     $                 I, WL_aux(I), CN1, CN2, CN3
               ENDDO
               CLOSE(47)

c        ... Will be used to mark excitation wavelengths belonging
c            to fluorescence processes
         Fi_vrs(:) = Fi_Flu(:)
c        ... Because there is no VRS by this mode
         SVR%Lo_VRS(:) = .FALSE.
c------->>  End of Fluorescence & rotational Raman excitation range

c     ... Vibrational & rotational Raman excitation range
c     ---------------------------------------------------
      CASE('VRS')
c        ... The number of phase function which will be used
c            to describe all inelastic processes in ocean
c            - Rayleigh phase function only -
         N_phm = 1

c        ... Shift between excitation and actual wavelengths is 
c            defined as a weighted center positions of four 
c            Gaussian shapes
         Shift = SUM( Dnu_FRF(:)*A_FRF(:) )

c  ----------------------------------------------------------------------------
c  ... Integration over VRS excitation range should be performed if RRS 
c                          is accounted for
c  ----------------------------------------------------------------------------
c     ... The routine "RANGE_VRS" returns:
c           Nsv
c             - number of segments if vibrational Raman band is discontinuous;
c           WLv_MIN(1:Nsv) WLr_MAX(1:Nsv) 
c             - minimal and maximal wavelengths for each segment.
         CALL RANGE_VRS
            WRITE(*,*) '    Number of excitation VRS segments: ',Nsv
            WRITE(*,*) '    Minimal and maximal wavelength of segments:'
            DO I = 1,Nsv 
               WRITE(*,'(2x,I3,2(2x,F7.3))') I,WLv_MIN(I),WLv_MAX(I)
            ENDDO
            write(*,*) '           End VRS'
            write(*,*) ' '

c        ... Min and max wavelength for full VRS excitation range
         Dn1 = MINVAL( WLv_MIN(1:Nsv) ) ; Dn2 = MAXVAL( WLv_MAX(1:Nsv) ) 

c        ... Min and max wavelength for full RRS excitation range
         S1 = MINVAL( WLr_MIN(1:Nsr) ) ; S2 = MAXVAL( WLr_MAX(1:Nsr) ) 

c        ... Min and max wavelengths for spectral range including
c            RRS and VRS excitation ranges
         G_MIN = DMIN1( Dn1,S1 ) ; G_MAX = DMAX1( Dn2,S2 )

c        ... The step to define WL-grid in full spectral range containing 
c            excitation ranges of RRS and VRS inelastic processes is equal
c            to the step within the RRS excitation range. This full spectral  
c            range will be called as combined excitation range (CER)
         G_Step = DEL_RAM

c        ... Number of wavelengths within CER
         Ng_WL = CEILING( (G_MAX - G_MIN)/G_Step )  + 1
         ALLOCATE(G_WL(Ng_WL), L_vrs(Ng_WL), L_Rrs(Ng_WL), L_exi(Ng_WL))

c        ... Wavelength grid in CER
         G_WL(1) = G_MIN
         DO I = 1, Ng_WL - 1
            G_WL(I+1) = G_WL(I) + G_Step
         ENDDO
c        ... Set exact value for right bound 
         G_WL(Ng_WL) = G_MAX
                WRITE(*,*) '   Bounds of combined excitation range :'
                WRITE(*,'(5x,2(2x,F7.3))') G_WL(1), G_WL(Ng_WL)

         L_Vrs(:) = .FALSE.
         DO I = 1,Nsv 
c           ... True elements of array L_vrs(:) mark wavelengths 
c               pertaining to VRS excitation range within CER
            WHERE( G_WL(:)>=WLv_MIN(I).AND.G_WL(:)<=WLv_MAX(I) )
               L_Vrs(:) = .TRUE.
            END WHERE
         ENDDO   ! over segments of VRS excitation range (I)

         L_Rrs(:) = .FALSE.
         DO I = 1,Nsr 
c           ... True elements of array L_Rrs(:) mark wavelengths pertaining 
c               to RRS excitation range within CER
            WHERE( G_WL(:)>=Wlr_MIN(I).AND.G_WL(:)<=Wlr_MAX(I) )
               L_Rrs(:) = .TRUE.
            END WHERE
         ENDDO                  ! over segments of RRS excitation range (I)

c        ... True elements of array L_exi(:) mark wavelengths pertaining 
c            to RRS and/or VRS excitation ranges. .False. elements of  
c            array L_exi(:) define "empty" wavelength intervals (gap)
c            between excitation segments within CER
         L_exi(:) = L_Vrs(:).OR.L_Rrs(:)

c        ... Number of wavelengths in combined excitation range without
c            gap between segments.
         Nwg = COUNT( L_exi(:) )

         ALLOCATE( WL_wg(Nwg), Lo_VRS(Nwg), Lo_RRS(Nwg) )

c        ... Extract wavelength grid excluding gap
         WL_wg(:)  = PACK(G_WL, L_exi)
         Lo_VRS(:) = PACK(L_Vrs,L_exi)
         Lo_RRS(:) = PACK(L_Rrs,L_exi)
c        ... .TRUE. elements of arrays Lo_VRS(:) and Lo_RRS(:)
c             mark wavelengths pertaining to VRS and RRS 
c             excitation ranges, respectively.

c        ... As a last step we need to redefine the wavelength grid within
c            VRS excitation ranges which do not coincide with RRS 
c            excitation ranges. It is assumed a priori that integration 
c            step in VRS range is always greater than RRS excitation binning
         ALLOCATE(L_hel(Nwg)) ; L_hel(:)=.FALSE.
         L_hel(1) = .TRUE. ; S1 = WL_wg(1)
c        ... .TRUE. elements of arrays L_hel(:) mark after this do-loop
c            wavelengths of the grid WL_wg(:) with the step corresponding 
c            to the desired integration step within VRS excitation ranges
c            and all wavelengths within RRS excitation ranges
         DO I = 2,Nwg            
            IF( SNGL(WL_wg(I)) <= SNGL(S1 + STEP_VRS) ) CYCLE
            L_hel(I-1) = .TRUE. ; S1 = WL_wg(I-1)
         ENDDO
c        ... .TRUE. elements of arrays L_hel(:) mark now wavelengths of 
c             the grid WL_wg(:) which should be selected in the resulting
c             excitation wavelength grid
         L_hel = L_hel.OR.Lo_Rrs

         N_AUX = COUNT( L_hel(:) )

c        ... Allocate auxiliary wavelength grid containing excitation
c            wavelengths for both rotational and vibrational Raman
         ALLOCATE( WL_aux(N_AUX), SVR%Lo_VRS(N_AUX), 
     $        Fi_vrs(N_AUX), Fi_rrs(N_AUX) )

         WL_aux(:) = PACK(  WL_wg,L_hel )
         Fi_rrs(:) = PACK( Lo_rrs,L_hel )
         Fi_vrs(:) = PACK( Lo_vrs,L_hel )

         SVR%Lo_VRS(:) = Fi_vrs(:)

               OPEN(47,file=DIR_OUT//'TEST_WL.DAT-2')
                 DO I = 1,N_AUX
                    CN1 = ' - ' ; CN2 = ' - '
                    IF( Fi_VRS(I) ) CN1 = 'vrs'
                    IF( Fi_RRS(I) ) CN2 = 'rrs'
                    WRITE(47,'(2x,I5,2x,f7.3,2x,A,2x,A)') 
     $                   I, WL_aux(I), CN1, CN2
                 ENDDO
               CLOSE(47)

c     ... End of vibrational & rotational Raman excitation range
      END SELECT

c <--------------------------------------------------------------------->
c     ... Full number of excitation wavelengths (for all inelastic processes)
c         in the extended WL-grid
      Na_exi = N_Aux

      ALLOCATE( SRR%NUM_IN(Na_exi), D1(Na_exi), D2(Na_exi), 
     $     SRR%Npo_sav(Na_exi) ) ! A

c     ... Pointer WL_Ext is used below for simplification reason only
      WL_Ext => WL_aux(:)

c     ... Do-loop over all excitation wavelengths of the extended grid      
      DO I = 1,Na_exi
c        ... It will be defined here spectral intervals around each
c            excitation wave1length of extended WL-grid which will be 
c            called in following as bins
         IF( I==1 ) THEN
            D1L = 0.d0 ; D1R = (WL_Ext(2) - WL_Ext(1))/2.d0
         ENDIF
         IF( I==Na_exi ) THEN
            D1L = (WL_Ext(Na_exi) - WL_Ext(Na_exi-1))/2.d0 
            D1R = 0.d0
         ENDIF
         IF( I>1.AND.I<Na_exi ) THEN
            D1L = (WL_Ext(I)   - WL_Ext(I-1))/2.d0
            D1R = (WL_Ext(I+1) - WL_Ext(I)  )/2.d0                 
         ENDIF
         D1(I) = WL_Ext(I) - D1L ; D2(I) = WL_Ext(I) + D1R
      ENDDO
c
c     ... In follows that the I-th bin is defined as:
c          WL_Ext(I-1)  D1(I)     WL_Ext(I)    D2(I)      WL_Ext(I+1)
c             {..........[...........|...........]...........}
c                         <------ I-th bin ----->

c     ... The amount of excitation wavelengths within the I-th bin
c         will be stored in array SRR%NUM_IN(I)
      SRR%NUM_IN(:) = 0
      DO I = 1,Na_exi
         KK = 0 ; D1L = D1(I) ; D1R = D2(I) 
         DO K = 1,Nr_Exi
c           ... Count all RRS excitation wavelengths which are 
c               within I-th bin
            IF( WLr_Exi(K)>=D1L.AND.WLr_Exi(K)<=D1R )  THEN
               KK = KK+1  ;  SRR%NUM_IN(I) = KK
            ENDIF
         ENDDO                  ! over all RRS excitation wavelengths
      ENDDO                     ! over wavelengths of extended WL-grid

c     ... Test that all RRS excitation wavelengths are accounted for
      WRITE(*,*)  '   -->', SUM(SRR%NUM_IN(:)), N_Act*N_WL_C

c     ... Estimate maximal number of RRS excitation wavelengths within bin
c         to define dimension of arrays SRR%ACT_IN and SRR%RIN_IN.
c         See below explanations concerning the meaning of these arrays.
      Kma = MAXVAL( SRR%NUM_IN(:) )
      ALLOCATE( SRR%ACT_IN(Na_exi,Kma), SRR%RIN_IN(Na_exi,Kma),
     $     SRR%N_EtoA(Na_exi,N_ACT) )

      SRR%ACT_IN(:,:) = 0 ; SRR%RIN_IN(:,:) = 0 ; SRR%N_EtoA(:,:) = 0

c     ... Do-loop over all excitation wavelengths within extended 
c         wl-grid. Arrays SRR%ACT_IN and SRR%RIN_IN will be defined 
c         after following do-loop 
      DO I = 1,Na_exi
         KK = 0 ; D1L = D1(I) ; D1R = D2(I)
c        ... Do-loop over all actual wavelengths
         DO Ia = 1,N_Act
            K1 = 0
            Jr = (Ia - 1)*(N_WL_C + 1) + 1
c           ... Do-loop over all RRS excitation wavelengths belonging
c               to Ia-th actual wavelength
            DO K = 1,N_WL_C
c              ... Excitation wavelength within I-th bin
               IF( WLr_Aux(Jr+K)>=D1L .AND. WLr_Aux(Jr+K)<=D1R ) THEN
                  KK = KK + 1 ; K1 = K1 + 1
c                 ... Element of array SRR%ACT_IN(I,KK) gives the 
c                     actual wavelength number to which contributes 
c                     KK-th excitation wavelength from I-th bin
                  SRR%ACT_IN(I,KK) = Ia
c                 ... Element of array SRR%RIN_IN(I,KK) gives the 
c                     Raman line number corresponding to KK-th 
c                     excitation wavelength
                  SRR%RIN_IN(I,KK) = K
               ENDIF
            ENDDO
c           ... Amount of excitation wavelengths within the I-th bin 
c               contributing to Ia-th actual wavelength 
            SRR%N_EtoA(I,Ia) = K1
c           * In contrast to SRR%NUM_IN(I) which defines the full 
c             number of excitation wavelength within I-th bin, 
c             N_EtoA(I,Ia) defines the amount of excitation 
c             wavelengths or each actual wavelength, i.e., 
c             SRR%NUM_IN(I) = SUM( N_EtoA(I,:) )

         ENDDO                  ! over all actual wavelengths
         IF( SRR%NUM_IN(I) /= SUM(SRR%N_EtoA(I,:)) ) GOTO 200
      ENDDO                     ! over wavelengths of extended WL-grid

c     ... Number of wavelengths of the extended WL-grid which will 
c         be calculated and should be saved     
      SRR%N_Aux = COUNT( SRR%NUM_IN(:) /= 0 )

c      OPEN(72,file=DIR_OUT//'Test-2.dat',RECL=150000)
c        DO I = 1,Na_exi
cc          ... Number of excitation wavelengths in I-th bin
c           KK = SRR%NUM_IN(I)
c           IF( Fi_rrs(I) ) THEN
c              WRITE(72,'(2x,I5,2x,f7.3,2x,100(A,I2,1x,I2,A))') 
c     $             I, WL_Ext(I), 
c     $             ('[',SRR%RIN_IN(I,J), SRR%ACT_IN(I,J),']', J=1,KK)
c           ELSE
c              WRITE(72,'(2x,I5,2x,f7.3,2x,500(1x,I5))') 
c     $             I, WL_Ext(I), KK, 
c     $             (SRR%RIN_IN(I,J), SRR%ACT_IN(I,J),J=1,KK)
c           ENDIF
c        ENDDO
c      CLOSE(72)
c <--------------------------------------------------------------------->

c        ... Save number of excitation wavelengths in SVR%N_AUX 
c            and total number of wavelengths in SVR%N_EXT
      SVR%N_aux = N_aux  ;  SVR%N_ext = N_aux + N_act
      SRR%N_ext = SVR%N_ext

      ALLOCATE( SVR%WL_EXT(SVR%N_ext),                   ! A
     $     SRR%Act_WL(SVR%N_ext), SRR%Exi_WL(SVR%N_ext), ! A
     $     SVR%Act_WL(SVR%N_ext), SVR%Exi_WL(SVR%N_ext), ! A
     $     SRR%WL_ext(SVR%N_ext) )                       ! A

c     ... Extended wavelength grid
      SVR%WL_ext(1:N_aux)           = WL_aux(1:N_aux)
      SVR%WL_ext(N_aux+1:SVR%N_EXT) = WL_act(1:N_act)

c     ... The same wavelength-grid is stored in structure SRR% and SVR%
      SRR%WL_ext(:) = SVR%WL_ext(:)

c                K = 0
c                WRITE(*,*) '  RRS excitation WLs: '
c                DO I = 1, N_AUX
c                   IF( Fi_rrs(I) ) THEN
c                      K = K + 1
c                      WRITE(*,'(2x,I5,2x,F8.3,2(2x,I3))') 
c     $                     K, WL_Aux(I), SRR%NUM_IN(I)
c                   ENDIF
c                ENDDO
c                WRITE(*,*) '  VRS excitation WLs: '
c                K = 0
c                DO I = 1, N_AUX
c                   IF( Fi_Vrs(I) ) THEN
c                      K = K + 1
c                      WRITE(*,'(2(2x,I5),2x,F8.3)') I, K, WL_Aux(I)
c                   ENDIF
c                ENDDO

c     ... Actual wavelengths in the extended wavelength-array WL_ext
c         will be marked by .TRUE. in the logical array SVR%Act_WL
      SVR%Act_WL(1:N_aux)           = .FALSE.
      SVR%Act_WL(N_aux+1:SVR%N_EXT) = .TRUE.

c     ... Logical array SVR%Exi_WL(:) contains .TRUE. for wavelengths 
c         corresponding to the VRS and fluorescence excitation ranges only. 
c         For all other wavelengths including actual ones this array 
c         contains .FALSE.
      SVR%Exi_WL(:)       = .FALSE.
      SVR%Exi_WL(1:N_aux) = Fi_vrs(:)

      write(*,*)  '   SVR%N_AUX = ', SVR%N_AUX, N_aux
      write(*,*) '   SHAPE(Fi_rrs)',  SHAPE(Fi_rrs )
      write(*,*) '   SHAPE(SRR%Exi_WL)',  SHAPE(SRR%Exi_WL )


      WRITE(*,*) '             N_act = ', N_act
      WRITE(*,*) '            Na_exi = ', Na_exi
      WRITE(*,*) 'COUNT( Fi_vrs(:) ) = ', COUNT( Fi_vrs(:) )
      WRITE(*,*) 'COUNT( Fi_rrs(:) ) = ', COUNT( Fi_rrs(:) )

c     ... Logical variables for RRS
      SRR%Act_WL(:) = SVR%Act_WL(:)

c     ... Logical array SRR%Exi_WL(:) contains .TRUE. for wavelengths 
c                       ---
c         corresponding to the RRS excitation ranges only. 
c         For all other wavelengths including actual this array 
c         contains .FALSE.
      SRR%Exi_WL(:)       = .FALSE.
      SRR%Exi_WL(1:N_aux) = Fi_rrs(:)


c      stop
c     ... Calculate wavelength redistribution function
c     ------------------------------------------------
      SELECT CASE( TRIM(INEL_PROC) )
      CASE('VRS')
         Nvr = 1
c        ... Defined in all excitation range ! (Preliminary)
         ALLOCATE( SVR%V_FRF(Nvr,Na_Exi,N_ACT) )
         CALL FRF_VRS_INT(Nvr)           ! VRS
      CASE('F_Chl')
         CALL FRF_RED           ! Chlorophyll fluorescence 
      CASE('F_CDOM')
         ALLOCATE( SVR%V_FRF(2,N_AUX,N_ACT) )
         CALL FRF_BLUE          ! CDOM fluorescence 
      CASE('Fr_Fb')
         ALLOCATE( SVR%V_FRF(2,N_AUX,N_ACT) )
         CALL FRF_RED  ;  CALL FRF_BLUE
      CASE('VRS_Fb')
         Nvr = 3
         ALLOCATE( SVR%V_FRF(Nvr,N_AUX,N_ACT) )
         CALL FRF_VRS_INT(Nvr) 
         CALL FRF_BLUE
      CASE('VRS_Fr')
         Nvr = 1
         ALLOCATE( SVR%V_FRF(Nvr,N_AUX,N_ACT) )
         CALL FRF_VRS_INT(Nvr) 
         CALL FRF_RED
       CASE('VRS_Fr_Fb')
         Nvr = 3
         ALLOCATE( SVR%V_FRF(Nvr,N_AUX,N_ACT) )
         CALL FRF_VRS_INT(Nvr) 
         CALL FRF_RED
         CALL FRF_BLUE
      END SELECT

cc     ... To remove contribution of VRS
c      SVR%Tot_Vrs(:) = 0.d0
c      STOP

      write(*,*) '  COUNT( SVR%Exi_WL(:)) ', COUNT( SVR%Exi_WL(:))
      write(*,*) '  COUNT( SRR%Exi_WL(:)) ', COUNT( SRR%Exi_WL(:))
      write(*,*) '              SRR%N_Aux ', SRR%N_Aux
c      stop

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Fatal error !')
      WRITE(EU,'(a)') '   It is detected the loss of RRS excitation '//
     $     'wavelength. '
      WRITE(EU,'(a)') '   Please contact the program developer !'
      GOTO 881

 201  CALL WRITE_MESSAGE(R_NAME,'ERROR','')
      WRITE(EU,'(A,2x,F7.3,A)')
     $     '     Maximal value of emission wavelength',G_MAX,' nm'
      WRITE(EU,'(A,2x,F7.3,A)') 
     $     '     is smaller than minimal excitation wavelength',
     $     WL_Flu(1),' nm'
      WRITE(EU,'(A)')
     $     '     Fluorescence can not be calculated.'
      WRITE(EU,'(A)')
     $     '     Switch off fluorescence in "control_uwt.inp" file.'
      GOTO 881

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

 881  CONTINUE
      RETURN 1
      CONTAINS
c **********************************************************************

      SUBROUTINE RANGE_RRS

      INTEGER     Ia, Ke, N_TMP, N1, Nws
      LOGICAL     From_right, From_left
      REAL (KND)  D1L, D1R, D2L, DEL_ACT, MIN_WL, MAX_WL, WL_L, WL_R
      LOGICAL,    ALLOCATABLE :: ACT_TMP(:)
      REAL (KND), ALLOCATABLE :: WL_TMP(:)
c     ==================================================================

c     ... Min and max wavelengths in the auxiliary RRS wavelength grid
      MIN_WL = MINVAL(WLr_Aux)
      MAX_WL = MAXVAL(WLr_Aux)
                 WRITE(*,*) '      Auxiliary RRS WL-grid    '
                 WRITE(*,*) '  exact min[WLr_Aux] and max[WLr_Aux] :'
                 WRITE(*,*)  MIN_WL, MAX_WL

      IF( N_ACT > 1 ) THEN
         DEL_ACT = WL_ACT(2) - WL_ACT(1)
c        ... Select step for new grading
         DELTA = DMIN1( DEL_ACT,DEL_RAM )
      ELSE
         DELTA = DEL_RAM
      ENDIF

c     ... Estimate number of spectral points in the extended
c         wavelength grid which will be constructed below
      N_TMP = 2*INT((MAX_WL - MIN_WL)/DELTA) 
      ALLOCATE( WL_TMP(N_TMP), ACT_TMP(N_TMP) )
      ACT_TMP(:) = .FALSE. 

c     ... Number of segments if rotation Raman excitation 
c         range is discontinuous
      Nsr = 0

c     ... Number of wavelengths in segment
      Nws = 0

      Ke = 0  ;  From_right = .FALSE.  ;  From_left  = .TRUE.
c     ... Do-loop oveer all actual wavelength
      DO Ia = 1,N_ACT
         Jr = (Ia - 1)*(N_WL_C + 1)
c        ... D1L and D1R are minimal and maximal wavelengths
c            of the spectral interval around Ia-th actual
c            wavelength, where all corresponding excitation RRS
c            wavelengths are located. This interval will be 
c            called below as the Raman band of Ia-th actual 
c            wavelength
         D1L = MINVAL( WLr_Aux(Jr+1:Jr+N_WL_C+1) )
         D1R = MAXVAL( WLr_Aux(Jr+1:Jr+N_WL_C+1) )
c        ... Extended wavelength grid left from actual wavelength
c            i.e., left Raman margin
         IF( From_left ) THEN
c           ... Smallest integer greater than argument of function
c               CEILING will be used as a number of wavelength 
c               in the extended grid within left Raman margin 
            N1 = CEILING( (WL_ACT(Ia)-D1L)/DEL_RAM )
            DO J = N1,1,-1
c              ... Ke counts wavelengths in the extebded grid
               Ke = Ke + 1 ; Nws = Nws + 1
c              ... WL_TMP contains wavelength grid in the left 
c                  Raman margin 
               WL_TMP(Ke) = WL_ACT(Ia) - J*DEL_RAM
            ENDDO
         ENDIF
         Ke = Ke + 1 ; Nws = Nws + 1
c        ... Last wavelength in this sequence is set to Ia-th 
c            actual wavelength
         WL_TMP(Ke) = WL_ACT(Ia) ; ACT_TMP(Ke) = .TRUE.

         IF( Ia==N_ACT ) THEN
c           ... Right Raman margin should be constracted for 
c               last actual wavelength
            From_right = .TRUE.
         ELSE
c           ... Position of the next actual wavelength in the
c               auxiliary wl-grid
            Jr = Jr + (N_WL_C + 1)
c           ... D2L is a minimal wavelength of the Raman band
c               corresponding to the next actual wavelength
            D2L = MINVAL( WLr_Aux(Jr+1:Jr+N_WL_C+1) )
            IF( D1R < D2L ) From_right = .TRUE.
c           ... There is no overlap between right Raman margin of
c               Ia-th and left Raman margin of Ia+1 actual 
c               wavelengths. Right Raman margin should be constracted 
c               for Ia-th actual wavelength.
         ENDIF

c        ... Thus, the logical From_right = .TRUE. if:
c            - Raman margins of two subsequent actual wavelengths do 
c              not overlap;
c            - WL(Ia) is the last actual wavelength
         IF( From_right ) THEN
c           ... Extended wavelength grid should be constructed  
c               from the right of actual wavelength WL(Ia):
c               | left Raman | right Raman  |
c               |   margin   |    margin    |
c               D1L         WL(Ia)         D1R  D2L   WL(Ia+1)
c               [............|..............]   [.......|......]
c                            <extended grid>
            N1 = CEILING( (D1R-WL_ACT(Ia))/DEL_RAM )
            DO J = 1,N1
               Ke = Ke + 1 ; Nws = Nws + 1
               WL_TMP(Ke) = WL_ACT(Ia) + J*DEL_RAM
            ENDDO
c           ... Current segment is already constructed
            WLr_MIN(Nsr+1) = WL_TMP(Ke - Nws + 1)
            WLr_MAX(Nsr+1) = WL_TMP(Ke)
            Nsr = Nsr + 1
                WRITE(*,*) '   RRS segments :'
                WRITE(*,'(2x,4(1x,I5))') Nsr, Ke - Nws + 1, Ke, Nws
                WRITE(*,*) Nsr, WL_TMP(Ke-Nws+1), WL_TMP(Ke)
            Nws = 0 

c           ... Set to .TRUE. because for the next actual wavelength,
c               WL(Ia+1), the left margin is not yet constructed
            From_left  = .TRUE.
c           ... Set to .FALSE. because this is the start value for
c               the next actual wavelength
            From_right = .FALSE.
         ELSE
c        ... There is the overlap of the right and left Raman margins
            WL_L = WL_ACT(Ia) ; WL_R = WL_ACT(Ia+1)
c           ... Extended wavelength grid should be constructed  
c               between actual wavelengths WL(Ia) and WL(Ia+1)
c               D1L      WL(Ia)        D1R    
c               [..........|.......[....].......|........]
c                                 D2L         WL(Ia+1)
c                           <-- extended grid -->
            N1 = ANINT((WL_R - WL_L)/DEL_RAM) 
            DELTA = (WL_R - WL_L)/N1
            DO J = 1,N1 - 1
               Ke = Ke + 1  ;  Nws = Nws + 1 
               WL_TMP(Ke) = WL_L + J*DELTA
            ENDDO
c           ... Set to .FALSE. because for the next actual wavelength,
c               i.e., WL(Ia+1) the left margin is already constructed
            From_left = .FALSE.
         ENDIF
      ENDDO                     ! End do-loop over actual wavelengths

      write(*,*) ' '
      WRITE(*,*) '  Number of auxiliary RRS WL grid : ',Ke

      WRITE(LUNIT,*) '   Number of actual wavelengths   :', N_ACT
      WRITE(LUNIT,*) '   Number of extended wavelengths : ', Ke
      WRITE(LUNIT,*) '   min[WLr_Aux] and max[WLr_Aux]  :'
      WRITE(LUNIT,*)  MIN_WL, MAX_WL
      WRITE(LUNIT,*) '  First and last WL in auxiliary RRS grid :'
      WRITE(LUNIT,*)  WL_TMP(1),  WL_TMP(Ke)
      write(*,*) '           End RRS'
      write(*,*) ' '
      
      ALLOCATE( WLr_Ext(Ke), ActR_WL(Ke) )
      Nr_ext = Ke

c     ... Save extended wavelength grid in WLr_Ext array
      WLr_EXT(:) = WL_TMP(1:Ke) 

c     ... Save logical array which .true. elements mark actual
c         wavelengths within extended WL-grid
      ActR_WL(:) = ACT_TMP(1:Ke)

      DEALLOCATE(WL_TMP, ACT_TMP)
      
      END SUBROUTINE RANGE_RRS
c ************************************************************************

      SUBROUTINE RANGE_VRS
      REAL (KND) T_MAX, T_MIN, DUM(N_FRF)

c     ... Integration range in [cm-1] for four Gaussian shapes
      DUM(:) = Nsig*DSQRT(2.d0)*Sig_FRF(:)
c     ... Position (in cm-1) of the "center" of VRS excitation band 
c         corresponding to the first actual wavelength
      Nuc(:) = K_nu/WL_ACT(1) + Dnu_FRF(:)
c     ... Minimal and maximal wavelengths for this excitation band
      WLv_MAX(1) = MAXVAL( K_nu/(Nuc(:) - DUM(:)) )
      WLv_MIN(1) = MINVAL( K_nu/(Nuc(:) + DUM(:)) )
c
c     | excitation band |
c     [.................]                     |.......
c  WLv_MIN(1)        WLv_MAX(1)              WL_ACT(1)
c
c     ... Segmentation of full excitation range of VRS.
c         It will be investigated below whether there is no
c         overlap between two following excitations bands 
      Nsv = 1
      DO I = 2,N_ACT
c        ... Position (in cm-1) of the "center" of I-th excitation band 
c            corresponding to the first actual wavelength
         Nuc(:) = K_nu/WL_ACT(I) + Dnu_FRF(:)
c        ... Minimal and maximal wavelengths for I-th excitation band
         T_MAX = MAXVAL( K_nu/(Nuc(:) - DUM(:)) )
         T_MIN = MINVAL( K_nu/(Nuc(:) + DUM(:)) )
c        ... Segmentation of the VRS excitation range
         IF( T_MIN > WLv_MAX(Nsv) ) THEN
c           There is no overlap between two following excitation bands
c                               T_MIN          T_MAX         WL_ACT(I)
c           [..............]      [..............]      |       |   
c          WLv_MIN(Nsv) WLv_MAX(Nsv)                  WL_ACT(Nsv)
c
            Nsv = Nsv + 1
            WLv_MAX(Nsv) = T_MAX ; WLv_MIN(Nsv) = T_MIN
         ELSE
c           There is overlap between two following excitation bands
c                     T_MIN          T_MAX      WL_ACT(I)
c            [..........[...]..........]      |    |   
c           WLv_MIN(Nsv) WLv_MAX(Nsv)       WL_ACT(Nsv)

            WLv_MAX(Nsv) = T_MAX
         ENDIF
      ENDDO

      END SUBROUTINE RANGE_VRS
c **********************************************************************

      SUBROUTINE FRF_RED

      USE VRS, ONLY: Nh, WLH, Apeh
c     ------------------------------------------------------------------

      INTEGER, PARAMETER:: N_sig = 4
      REAL (KND)  TMP(N_AUX), Sth, WL_L, WL_R
      REAL (KND), ALLOCATABLE:: Em_f(:)
c     ==================================================================

c     ... Integration step in [nm]
      Sth = 1.d0

c     ... Sig_Chl is root mean square of Gaussian emission band
      WL_L = WLG_Chl - N_sig*Sig_Chl
      WL_R = WLG_Chl + N_sig*Sig_Chl
      Nh = CEILING( (Wl_R - Wl_L)/Sth )  + 1
      ALLOCATE( WLH(Nh), Em_f(Nh), Apeh(Nh) )

c     ... WL-grid to integrate 
      WLH(1) = WL_L
      DO I = 1, Nh - 1
         WLH(I+1) = WLH(I) + Sth
      ENDDO
      WLH(Nh) = Wl_R

c     ... Calculate fluorescence emission function 
      Em_f(:) = 1.d0/DSQRT(2.d0*Pie)/Sig_Chl/WLH(:)
     $     *DEXP(-(WLH(:) - WLG_Chl)**2/2.d0/Sig_Chl**2 )
      DO I = 1, Nh-2
         Apeh(I) = 0.5d0*Em_f(I) + SUM( Em_f(I+1:Nh-1) )
      ENDDO
      Apeh(1:Nh-2) = Apeh(1:Nh-2) + 0.5d0*Em_f(Nh)
      Apeh(Nh-1) = 0.5d0*(Em_f(Nh-1) + Em_f(Nh))
      Apeh(:) = Apeh(:)*Sth

cc     ... Output for test purpose
c      OPEN( 44, file=DIR_OUT//'Apeh.dat' )
c        DO I = 1,Nh
c           WRITE(44,*) WLH(I), Apeh(I)
c        ENDDO
c      CLOSE(44)

c     ... Integrate over full excitation wavelength range to 
c         obtain fluorescence absorption part of chlorophyll-a
      TMP(:) = 1.d0/DSQRT(2.d0*Pie)/Sig_Chl/WL_AUX(:)
     $     *DEXP(-(WL_AUX(:) - WLG_Chl)**2/2.d0/Sig_Chl**2 )

      TMP(1)     = TMP(1)*(WL_aux(2) - WL_aux(1))
      TMP(N_AUX) = TMP(N_aux)*(WL_aux(N_aux) - WL_aux(N_aux-1))
      DO I = 2,N_AUX-1
         TMP(I) = TMP(I)*(WL_aux(I+1) - WL_aux(I-1))
      ENDDO
      A_FChl = SUM( TMP(:) )*0.5d0 

      WRITE(*,*) ' ------ In routine "ext_wl_all" -------'
      WRITE(*,*) '  Old integration range', WL_aux(1), WL_aux(N_aux)
      WRITE(*,*) '  Compare old and new with relative difference:'
      WRITE(*,*) A_FChl, Apeh(1), (A_FChl - Apeh(1))/A_FChl*100.

      END SUBROUTINE FRF_RED
c **********************************************************************

      SUBROUTINE FRF_BLUE
      INTEGER    N_WL
      REAL (KND) WL1, WL2, DWL, Iex, Iem
      REAL (KND), ALLOCATABLE :: Re_Fu(:,:,:)

      ALLOCATE( Re_Fu(2,N_AUX,N_ACT), A_Exi(2,N_AUX), Abs_flu(2,N_AUX) )

      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(1,:), N_AUX,WL_AUX,A_Exi(1,:), 'Z')
      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(2,:), N_AUX,WL_AUX,A_Exi(2,:), 'Z')

c     ... Wavelength grid to integrate over emission band
      WL1 = WL_QE(1) ;  WL2 = 700.d0 ; DWL = 10.d0
      N_WL = CEILING( (WL2 - WL1)/DWL ) + 1
      ALLOCATE( WL_INT(N_WL) ) ; WL_INT(1) = WL1
      DO I = 1, N_WL - 1
         WL_INT(I+1) = WL_INT(I) + DWL
      ENDDO
      WL_INT(N_WL) = WL2

c     ... Fluorescence wavelength redistribution functions
c         containing specific absorption coefficients
      DO I = 1, N_AUX
         Iex = 1.d0/WL_AUX(I)
c        ... Fulvic acid
         Ab1 = A_Exi(1,I)
     $        *Fu_par(1)*DEXP( Fu_par(2)*(FH_REF-WL_AUX(I)) )
         An1 =        A_Fa(1)*Iex + B_Fa(1)
         Dn1 = 0.6d0*(A_Fa(2)*Iex + B_Fa(2))
c        ... Humic acid
         Ab2 = A_Exi(2,I)
     $        *Hu_par(1)*DEXP( Hu_par(2)*(FH_REF-WL_AUX(I)) )
         An2 =        A_Ha(1)*Iex + B_Ha(1)
         Dn2 = 0.6d0*(A_Ha(2)*Iex + B_Ha(2))
         DO K = 1,N_ACT
            Iem = 1.d0/WL_ACT(K)
            Re_Fu(1,I,K) = Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
            Re_Fu(2,I,K) = Ab2*DEXP(-((Iem - An2)/Dn2)**2)*Iem/Iex
         ENDDO

c        ... Integration over emission range
         S1 = Ab1/Iex/2.d0
     $        *( DEXP(-((1.d0/WL_INT(1)   -An1)/Dn1)**2)/WL_INT(1)
     $        + DEXP(-((1.d0/WL_INT(N_WL)-An1)/Dn1)**2)/WL_INT(N_WL) )
         S2 = Ab2/Iex/2.d0
     $        *( DEXP(-((1.d0/WL_INT(1)   -An2)/Dn2)**2)/WL_INT(1)
     $        + DEXP(-((1.d0/WL_INT(N_WL)-An2)/Dn2)**2)/WL_INT(N_WL) )
         DO K = 2,N_WL - 1
            Iem = 1.d0/WL_INT(K)
            S1 = S1 + Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
            S2 = S2 + Ab2*DEXP(-((Iem - An2)/Dn2)**2)*Iem/Iex
         ENDDO
         Abs_flu(1,I) = S1*Dwl  ;  Abs_flu(2,I) = S2*Dwl
      ENDDO
      SVR%V_FRF(1:2,:,:) = Re_Fu(1:2,:,:)

c     ... Write test results
      WRITE(*,*) '   Number of actual WL: ',N_ACT
      open(72,file=DIR_OUT//'abs_flu.dat')
      DO I = 1, N_AUX
         write(72,*) WL_AUX(I), Abs_flu(1,I), Abs_flu(2,I)
      ENDDO
      close(72)
      open(72,file=DIR_OUT//'act_wl.dat')
      DO I = 1, N_ACT
         write(72,*) WL_ACT(I)
      ENDDO
      close(72)
      open(72,file=DIR_OUT//'FRF_Fa.dat')
      DO I = 1, N_AUX
         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
     $        WL_AUX(I), (Re_Fu(1,I,K),K=1,N_ACT)
      ENDDO
      close(72)
      open(72,file=DIR_OUT//'FRF_Ha.dat')
      DO I = 1, N_AUX
         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
     $        WL_AUX(I), (Re_Fu(2,I,K),K=1,N_ACT)
      ENDDO
      close(72)

      END SUBROUTINE FRF_BLUE
c **********************************************************************

      SUBROUTINE FRF_VRS_INT(Nn)
      INTEGER, INTENT(IN) :: Nn
      INTEGER  N_c, Nint, K2
      REAL (KND)  Norm, T_MAX, T_MIN, T_T, S1, DUM(N_FRF)
      REAL (KND), ALLOCATABLE :: Re_Fu(:,:,:), DEL_EXT(:)

      ALLOCATE( SVR%Tot_Vrs(SVR%N_EXT), DEL_EXT(Na_Exi) ) ! A

c     ... It is allowed for the test purpose to use constant
c         total Raman scattering coefficient performing the 
c         integration over excitation wave range
      IF( Tvs_use ) THEN
c        ... Use constant user-defined total Raman scattering coefficients
         SVR%Tot_Vrs(1:Na_Exi) = Tvs_exc
         SVR%Tot_Vrs(Na_Exi+1:SVR%N_EXT) = Tvs_act
      ELSE
c        ... Calculate total Raman scattering coefficients as
c            a function of excitation wavelength
         N_c = SVR%N_EXT
         SELECT CASE( TRIM(Contour) )
         CASE('Inverse_power_law')
c           ... In previous versions Lmi==4
c               Bet_0R is calculated in "read_uwt.f"
            SVR%Tot_Vrs(1:Na_Exi) = Bet_0R*(Lrf/WL_AUX(:))**Lmi
            SVR%Tot_Vrs(Na_Exi+1:N_c)= Bet_0R*(Lrf/WL_ACT(:))**Lmi
         CASE('Resonance')
            Shift = SUM( Dnu_FRF(:)*A_FRF(:) )
            SVR%Tot_Vrs(1:Na_Exi) = Bet_0R*(K_nu/WL_AUX(:) - Shift)**4
     $           /( (K_nu/Lmi)**2 - (K_nu/WL_AUX(:))**2)**2
            SVR%Tot_Vrs(Na_Exi+1:N_c) = Bet_0R*(K_nu/WL_ACT(:) 
     $           - Shift)**4/( (K_nu/Lmi)**2 - (K_nu/WL_ACT(:))**2)**2
         END SELECT
      ENDIF

      WHERE( .NOT.SVR%Lo_VRS(:) )
         SVR%Tot_Vrs(1:Na_Exi) = 0.d0
      END WHERE

      ALLOCATE( Re_Fu(1,Na_Exi,N_Act) )
c     ... Norm-factor in the VRS frequency redistribution function (FRF)
      Norm = DSQRT(2.d0*Pie)*SUM( A_FRF(:)*Sig_FRF(:) )
c     ... Frequency redistribution function (FRF)
      DO K = 1,N_Act
         Nuc(:) = K_nu/WL_ACT(K) + Dnu_FRF(:)
         DO I = 1, Na_Exi
            Nut = K_nu/WL_AUX(I)
            Re_Fu(1,I,K) = SUM( A_FRF(:)
     $           *DEXP( -( (Nut-Nuc(:))/Sig_FRF(:))**2/2.d0 ) )
         ENDDO
         Re_Fu(1,:,K) = Re_Fu(1,:,K)/Norm
      ENDDO
c        ... Quadrature coefficients
         DEL_EXT(1)     = SVR%WL_EXT(2)     - SVR%WL_EXT(1)
         DEL_EXT(Na_Exi) = SVR%WL_EXT(Na_Exi) - SVR%WL_EXT(Na_Exi-1)
         DEL_EXT(2:Na_Exi-1) 
     $        = SVR%WL_EXT(3:Na_Exi)-SVR%WL_EXT(1:Na_Exi-2)

c        ... Correct quadrature coefficients at left and right 
c            bounds of a segment
         DO I = 1,Nsv
            K2 = MINLOC( DABS(WL_Aux(:)-WLv_MAX(I)),1 )
            DEL_EXT(K2) = WL_Aux(K2) - WL_Aux(K2-1)
            IF( I>1 ) THEN
               K2 = MINLOC( DABS(WL_Aux(:)-WLv_MIN(I)),1 )
               DEL_EXT(K2) = WL_Aux(K2+1) - WL_Aux(K2)
            ENDIF
         ENDDO
         DEL_EXT(:) = DEL_EXT(:)/2.d0
c         DEL_EXT(:) = 1.d0

c        ... Vibrational Raman scattering coefficient
         DO K = 1,N_Act
            SVR%V_FRF(Nn,:,K) = SVR%Tot_Vrs(1:Na_Exi)*Re_Fu(1,:,K)
     $           *K_nu/WL_AUX(:)**2*DEL_EXT(:)
         ENDDO

c         WRITE(*,*) '    Total RSC within excitation range'
c         DO I = 1, Na_Exi
c            WRITE(*,'(2x,f7.2,2x,f8.5)')  WL_AUX(I), SVR%Tot_Vrs(I)
c         ENDDO
c         WRITE(*,*) '    Total RSC at emission wavelengths'
c         DO I = 1, N_Act
c            WRITE(*,'(2x,f7.2,2x,f8.5)') 
c     $           WL_ACT(I), SVR%Tot_Vrs(Na_Exi+I)
c         ENDDO
c         STOP '  In "ext_wl_all.f" '

         write(*,*) '  N_Act = ', N_Act
         open(72,file=DIR_OUT//'FRF_VRS_all.dat',RECL=10000)
         DO I = 1, Na_Exi
            write(72,'(2x,F7.2,100(1x,e12.4E3))') 
     $           WL_AUX(I), (SVR%V_FRF(Nn,I,K),K=1,N_Act)
         ENDDO
         close(72)
c         STOP

      END SUBROUTINE FRF_VRS_INT
c **********************************************************************

      END
