C********************************************************************C
C
C       SCIATRAN - Version 3.7 - Last modification: 19 Feb 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE EXT_WL_RRS(N_ACT, WL_ACT, SRR, *)
c     ... Call in "rtm_main.f"

      USE ST_VARS,   ONLY: Pie, APPR_RRS
      USE LOG_PARS,  ONLY: EU, LUNIT, DIR_OUT
      USE RAMAN,     ONLY: N_WL_C, DEL_WL, DEL_RAM
      USE TYPE_VARS, ONLY: KND
      USE TYPES,     ONLY: RRS_TYPE
c     ------------------------------------------------------------------

      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
      INTEGER    N_ACT
      REAL (KND) WL_ACT(N_ACT)
c     ... Local variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'EXT_WL_RRS'
      INTEGER   I, J, K, Ia, K1, K2, KK, Kma, Jr, N_exi, N_ext, N0_ext
      REAL (KND)  D1L, D1R, D2L, WL_L, WL_R, WLT, DUM
      REAL (KND), PARAMETER :: D9 = 1.D+9
      REAL (KND), ALLOCATABLE :: D1(:), D2(:)
      REAL (KND), ALLOCATABLE, TARGET :: W0_ext(:), WL_Exi(:)
      REAL (KND), POINTER :: WL_Ext(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     ... N_ACT is the number of actual wavelengths 
c         N_WL_C is the number of excitation wavelengths 
c           which produce Raman lines having the same position as an
c           actual wavelength
c         N_exi is the full number of excitation wavelengths
      N_exi = N_ACT*N_WL_C 

c     ... N0_Ext is the number of wavelengths in an extended WL-grid
c         which contains all excitation and actual wavelengths
      N0_Ext = N_ACT*(N_WL_C + 1)

c     ... Auxiliary wavelength grid is given in the array W0_Ext.
      ALLOCATE( W0_Ext(N0_Ext), WL_Exi(N_Exi) )

c         There is a sequence of N_WL_C+1 wavelengths 
c         for each actual wavelength:
c         - first wavelength in this sequence is an actual wavelength, 
c           ---------------------------------------------------------
c         - following N_WL_C wavelengths are excitation wavelengths 
c           which produced Raman lines having the position at actual 
c           wavelength. 
      KK = 0
      DO I = 1,N_ACT
         Jr = (I-1)*(N_WL_C+1) + 1
         W0_Ext(Jr) = WL_ACT(I)
         DO J = 1,N_WL_C
c           ... Array DEL_WL is used to define positions of all
c               excitation wavelengths which produced Raman lines 
c               having the position at actual wavelength
            KK = KK + 1 ; WLT = D9/( D9/WL_ACT(I) - DEL_WL(J) )

cc            ... This is the case of inverse position of the Stokes
cc                and anti-stokes Raman bands
c            KK = KK + 1 ; WLT = D9/( D9/WL_ACT(I) + DEL_WL(J) )

            W0_Ext(Jr+J) = WLT
c           ... All excitation wavelengths are in array WL_Exi(:)
            WL_Exi(KK)   = WLT
         ENDDO  ! over Raman lines (J)
      ENDDO  ! over actual wavelengths (I)

cc ***************** Test output 1 ****************************
c      WRITE(*,*) '  All excitation wavelengths KK =  ',KK
c      DO I = 1, KK
c         WRITE(*,'(2x,I3,2x,F9.4)')  I, WL_Exi(I) 
c      ENDDO
c      STOP
cc ***************** End test output 1 ****************************

c     ... Positions of all excitation wavelengths are exact if option
c         'exact matching' is selected
      IF( TRIM(APPR_RRS) == 'exact matching' ) THEN
         SRR%N_EXT = N0_Ext
         ALLOCATE( 
     $        SRR%WL_EXT(N0_Ext), SRR%Act_WL(N0_Ext),   ! A
     $        SRR%Exi_WL(N0_Ext), SRR%A2E(N_ACT),       ! A
     $        SRR%NUM_IN(N0_Ext), SRR%ACT_IN(N0_Ext,1), ! A
     $        SRR%RIN_IN(N0_Ext,1) )                    ! A

c        ... In this option extended WL-grid is the same as auxiliary one
         SRR%WL_EXT(:) = W0_Ext(:) 
         SRR%Act_WL(:) = .FALSE. ; SRR%NUM_IN(:) = 1

c        ... Do-loop over actual wavelengths
         DO I = 1,N_ACT
            Jr = (I-1)*(N_WL_C+1) 

c           ... All actual wavelengths are marked by .true.
c               It is used here that in the sequence of N_WL_C+1 
c               wavelengths the first wavelength is an actual one
            SRR%Act_WL(Jr+1) = .TRUE.

c           ... Location of I-th actual wavelength in the extended 
c               WL-grid is saved in array element SRR%A2E(I) 
c               (Actual-To-Extended)
            SRR%A2E(I) = Jr+1

            DO J = 1,N_WL_C+1
               K = Jr + J
c              ... Element of array SRR%ACT_IN(K,1) gives the 
c                  actual wavelength number to which contributes 
c                  K-th excitation wavelength.
               SRR%ACT_IN(K,1) = I
c              ... Element of array SRR%RIN_IN(K,1) gives the 
c                  Raman line number corresponding to K-th 
c                  excitation wavelength
               SRR%RIN_IN(K,1) = J-1
c                  * For simplification reasons arrays SRR%ACT_IN(K,1) 
c                  and SRR%RIN_IN(K,1) contain actual wavelengths also
c                  which will be not used, however, by the program 
c                  because e.g., for I-th actual wavelength 
c                  SRR%RIN_IN( (I-1)*(N_WL_C+1)+1,1 ) = 0
            ENDDO  ! over 

cc ***************** Test output 2 ****************************
c            IF( I==1 ) THEN
c               write(*,'(2x,2x,f8.3)') W0_Ext(Jr+1)
c               DO J = 1,N_WL_C+1
c                  K = Jr + J
c                  write(*,'(2x,I3, 2x,f8.3, *(2x,I3))') 
c     $                 J, W0_Ext(K), SRR%RIN_IN(K,1), SRR%ACT_IN(K,1)
c               ENDDO
c            ENDIF
c            STOP
cc ***************** End test output 2 ****************************

         ENDDO  ! over actual wavelengths (I)

c        ... Number of excitation wavelengths
         SRR%N_Aux = N_ACT*N_WL_C
         SRR%Exi_WL(:) = .NOT.SRR%Act_WL(:)

         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF  ! exact matching

c **************************************************************
c ***************** Option "simple binning" ********************
c **************************************************************

c     ... Structure elements SRR%WL_EXT(:) and SRR%Act_WL(:) 
c         will be allocated and calculated in this routine
      CALL EXTENDED_GRID

c     ... Full number of wavelengths N_Ext in the extended grid 
c         is defined in "EXTENDED_GRID"
      SRR%N_Ext = N_Ext
      WL_Ext =>  SRR%WL_EXT(:) 

      WRITE(Lunit,*)
     $     ' Full number of wavelengths in the extended grid', N_Ext
c      STOP

      ALLOCATE( SRR%NUM_IN(N_Ext), D1(N_Ext), D2(N_Ext), 
     $     SRR%Npo_sav(N_Ext) )

c     ... Do-loop over all wavelengths of the extended grid      
      DO I = 1,N_Ext
c        ... It will be defined here spectral intervals around each
c            wavelength of extended WL-grid which will be called
c            in following as bins
         IF( I==1 ) THEN
            D1L = 0.d0 ; D1R = (WL_Ext(2) - WL_Ext(1))/2.d0
         ENDIF
         IF( I==N_Ext ) THEN
            D1L = (WL_Ext(N_Ext) - WL_Ext(N_Ext-1))/2.d0 ; D1R = 0.d0
         ENDIF
         IF( I>1.AND.I<N_Ext ) THEN
            D1L = (WL_Ext(I)   - WL_Ext(I-1))/2.d0
            D1R = (WL_Ext(I+1) - WL_Ext(I)  )/2.d0                 
         ENDIF
         D1(I) = WL_Ext(I) - D1L ; D2(I) = WL_Ext(I) + D1R
      ENDDO  ! over extended wavelength grid (I)

c     ... In following the I-th bin is defined as :
c          WL_Ext(I-1)  D1(I)     WL_Ext(I)    D2(I)      WL_Ext(I+1)
c             {..........[...........|...........]...........}
c                         <------ I-th bin ----->

c     ... The amount of excitation wavelengths within the I-th bin
c         will be stored in array SRR%NUM_IN(I)
      SRR%NUM_IN(:) = 0
c     ... Do-loop over all wavelengths of extended WL-grid
      DO I = 1,N_Ext
         KK = 0 ; D1L = D1(I) ; D1R = D2(I) 
c        ... Do-loop over all excitation wavelengths
         DO K = 1,N_Exi
c           ... Count all excitation wavelengths which are within I-th bin
            IF( WL_Exi(K)>=D1L.AND.WL_Exi(K)<=D1R )  THEN
               KK = KK+1  ;  SRR%NUM_IN(I) = KK
            ENDIF
         ENDDO  ! over excitation wavelengths (K)
      ENDDO  ! over extended wavelength grid (I)

      WRITE(Lunit,'(A)') 
     $     ' Check that all excitation wavelengths are accounted for'
      WRITE(Lunit,*)  '   -->', SUM(SRR%NUM_IN(:)), N_ACT*N_WL_C

c     ... Estimate maximal number of excitation wavelength in a bin
c         to define dimension of arrays SRR%ACT_IN and SRR%RIN_IN.
c         See below explanations concerning the meaning of these arrays
      Kma = MAXVAL( SRR%NUM_IN(:) )
      WRITE(Lunit,'(A)') 
     $     '  Maximal number of excitation wavelength in a bin'
      WRITE(Lunit,'(A,2x,I3)') ' Kma =  ', Kma

      ALLOCATE( SRR%ACT_IN(N_Ext,Kma), SRR%RIN_IN(N_Ext,Kma),
     $     SRR%N_EtoA(N_Ext,N_ACT) )

      SRR%ACT_IN(:,:) = 0 ; SRR%RIN_IN(:,:) = 0 ; SRR%N_EtoA(:,:) = 0

c     ... Arrays SRR%ACT_IN and SRR%RIN_IN will be defined within 
c         this do-loop over bins of extended wl-grid. 
      DO I = 1,N_Ext
         KK = 0
c        ... Select spectral interval corresponding to the i-th bin
         D1L = D1(I) ; D1R = D2(I)
c        ... Do-loop over all actual wavelengths
         DO Ia = 1,N_ACT
            K1 = 0
c           ... Select position of Ia-th actual wavelength
            Jr = (Ia - 1)*(N_WL_C + 1) + 1
c           ... Do-loop over all excitation wavelengths belonging
c               to Ia-th actual wavelength
            DO K = 1,N_WL_C
c              ... Excitation wavelength within I-th bin
               IF(W0_Ext(Jr+K)>=D1L .AND. W0_Ext(Jr+K)<=D1R) THEN
                  KK = KK + 1 ; K1 = K1 + 1
c                 ... Element of array SRR%ACT_IN(I,KK) gives the 
c                     actual wavelength number to which contributes 
c                     KK-th excitation wavelength from I-th bin
                  SRR%ACT_IN(I,KK) = Ia
c                 ... Element of array SRR%RIN_IN(I,KK) gives the 
c                     Raman line number corresponding to KK-th 
c                     excitation wavelength
                  SRR%RIN_IN(I,KK) = K
               ENDIF
            ENDDO  ! over excitation wavelength (K)
c           ... Amount of excitation wavelengths within the I-th bin 
c               contributing to Ia-th actual wavelength 
            SRR%N_EtoA(I,Ia) = K1
c           * In contrast to SRR%NUM_IN(I) which defines the full 
c             number of excitation wavelength within I-th bin, 
c             NAB(I,Ia) defines the amount of excitation wavelengths
c             or each actual wavelength, i.e., 
c             SRR%NUM_IN(I) = SUM( NAB(I,:) )
         ENDDO  ! over actual wavelengths (Ia)
         IF( SRR%NUM_IN(I) /= SUM(SRR%N_EtoA(I,:)) ) GOTO 200
      ENDDO  ! over bins of extended WL grid (I)
c
c      DO Ia = 1,N_ACT
cc        ... Check that a bin around actual wavelength does not
cc            contain excitation wavelengths
c         IF( SRR%N_EtoA(SRR%A2E(Ia),Ia) /= 0 ) GOTO 201
c      ENDDO

c     ... Number of wavelengths of the extended WL-grid which will 
c         be calculated and should be saved     
      SRR%N_Aux = COUNT( SRR%NUM_IN(:) /= 0 )

c      OPEN(72,file=DIR_OUT//'/Test-2.dat',RECL=150000)
c        K = 1
c        DO I = 1,N_Ext
cc          ... Number of excitation wavelengths in I-th bin
c           KK = SRR%NUM_IN(I)
c           IF( SRR%Act_WL(I) ) THEN
c              WRITE(72,'(2x,I5,2(2x,f7.3),2x,500(1x,I5))') 
c     $             I, WL_Ext(I), WL_ACT(K), KK, 
c     $             (SRR%RIN_IN(I,J), SRR%ACT_IN(I,J), J=1,KK)
c              K = K + 1
c           ELSE
c              WRITE(72,'(2x,I5,2x,f7.3,2x,500(1x,I5))') 
c     $             I, WL_Ext(I), KK, 
c     $             (SRR%RIN_IN(I,J), SRR%ACT_IN(I,J),J=1,KK)
c           ENDIF
c        ENDDO
c      CLOSE(72)

      WRITE(Lunit,*) '   Number of wavelengths in extended grid: ',N_Ext
c      PAUSE

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Fatal error !')
      WRITE(EU,'(a)') '   It is detected the loss of excitation '//
     $     'wavelength. '
      WRITE(EU,'(a)') '   Please contact the program developer !'
      GOTO 881

 201  CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
      WRITE(EU,'(a)') '   The bin width is too larger to '//
     $     'properly calculate Raman scattering'
      WRITE(EU,'(a)')  '   employing the mode "simple binning".'
      WRITE(EU,'(a)')  
     $     '   To perform calculations, the "Raman binning step"'
      WRITE(EU,'(a)')
     $     '   control parameter in "control_rrs.inp" file '
      WRITE(EU,'(a)')
     $     '   should be decreased.'
      GOTO 881

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

 881  CONTINUE
      RETURN 1

      CONTAINS

c **********************************************************************
      SUBROUTINE EXTENDED_GRID

      INTEGER     N_TMP, N1
      LOGICAL     From_right, From_left
      REAL (KND)  MIN_WL, MAX_WL, DEL_ACT, DELTA
      LOGICAL,    ALLOCATABLE :: ACT_TMP(:)
      REAL (KND), ALLOCATABLE :: WL_TMP(:)
c     ==================================================================

c     ... Min and max wavelengths in the auxiliary wavelength grid
      MIN_WL = MINVAL(W0_Ext)
      MAX_WL = MAXVAL(W0_Ext)
      WRITE(LUNIT,*) '  min[W0_Ext] and max[W0_Ext] :'
      WRITE(LUNIT,*)  MIN_WL, MAX_WL

      IF( N_Act > 1 ) THEN
         DEL_ACT = WL_ACT(2) - WL_ACT(1)
c        ... Select step for new grading
c            DEL_RAM is the Raman binning step
         DELTA = DMIN1( DEL_ACT,DEL_RAM )
      ELSE
         DELTA = DEL_RAM
      ENDIF

c     ... Estimate number of spectral points in the extended
c         wavelength grid which will be constructed below
      N_TMP = 2*INT((MAX_WL - MIN_WL)/DELTA) 
      ALLOCATE( WL_TMP(N_TMP), ACT_TMP(N_TMP), SRR%A2E(SRR%N_ACT) )
      ACT_TMP(:) = .FALSE. 

      N_Ext = 0  ;  From_right = .FALSE.  ;  From_left  = .TRUE.

c     ... Do-loop oveer all actual wavelength
      DO Ia = 1,N_ACT
         Jr = (Ia - 1)*(N_WL_C + 1)
c        ... D1L and D1R are minimal and maximal wavelengths
c            of the spectral interval around Ia-th actual
c            wavelength, where all corresponding excitation 
c            wavelength are located. This interval will be 
c            called below as the Raman band of Ia-th actual 
c            wavelength
         D1L = MINVAL( W0_Ext(Jr+1:Jr+N_WL_C+1) )
         D1R = MAXVAL( W0_Ext(Jr+1:Jr+N_WL_C+1) )

c        ... Extended wavelength grid left from actual wavelength
c            i.e., left Raman margin
         IF( From_left ) THEN
c           ... Smallest integer greater than argument of function
c               CEILING will be used as a number of wavelength 
c               in the extended grid within left Raman margin 
            N1 = CEILING( (WL_ACT(Ia)-D1L)/DEL_RAM )
            DO J = N1,1,-1
c              ... N_Ext counts wavelengths in the extended grid
               N_Ext = N_Ext + 1
c              ... WL_TMP contains wavelength grid in descending order  
c                  for the left Raman margin 
               WL_TMP(N_Ext) = WL_ACT(Ia) - J*DEL_RAM
            ENDDO  ! over wavelengths within left margin (J)
         ENDIF  ! From_left

         N_Ext = N_Ext + 1
c        ... Last wavelength in this sequence is set to Ia-th 
c            actual wavelength
         WL_TMP(N_Ext) = WL_ACT(Ia) ; ACT_TMP(N_Ext) = .TRUE.

c        ... Ia-th element in array SRR%A2E(Ia) defines position 
c            of Ia-th actual wavelength in the extended grid
         SRR%A2E(Ia) = N_Ext

         IF( Ia==N_ACT ) THEN
c           ... Right Raman margin should be constracted for 
c               last actual wavelength
            From_right = .TRUE.
         ELSE
c           ... Position of the next actual wavelength in the
c               auxiliary wl-grid
            Jr = Jr + (N_WL_C + 1)
c           ... D2L is a minimal wavelength of the Raman band
c               corresponding to the next actual wavelength
            D2L = MINVAL( W0_Ext(Jr+1:Jr+N_WL_C+1) )
            IF( D1R < D2L ) From_right = .TRUE.
c           ... There is no overlap between right Raman margin of
c               Ia-th and left Raman margin of Ia+1 actual 
c               wavelengths. Right Raman margin should be constracted 
c               for Ia-th actual wavelength.
         ENDIF

c        ... Thus, the logical From_right = .TRUE. if:
c            - Raman margins of two subsequent actual wavelengths 
c              do not overlap;
c            - WL(Ia) is the last actual wavelength
         IF( From_right ) THEN
c           ... Extended wavelength grid should be constructed  
c               from the right of actual wavelength WL(Ia):
c               | left Raman | right Raman  |
c               |   margin   |    margin    |
c               D1L         WL(Ia)         D1R  D2L   WL(Ia+1)
c               [............|..............]   [.......|......]
c                            <extended grid>
            N1 = CEILING( (D1R-WL_ACT(Ia))/DEL_RAM )
            DO J = 1,N1
               N_Ext = N_Ext + 1
               WL_TMP(N_Ext) = WL_ACT(Ia) + J*DEL_RAM
            ENDDO

c           ... Set From_left to .TRUE. because for the next actual
c               wavelength, WL(Ia+1), the left margin is not yet constructed
            From_left  = .TRUE.

c           ... Set From_right to .FALSE. because this is the start 
c               value for the next actual wavelength
            From_right = .FALSE.
         ELSE
c        ... There is an overlap of the right and left Raman margins
c        ===========================================================
            WL_L = WL_ACT(Ia) ; WL_R = WL_ACT(Ia+1)
c           ... Extended wavelength grid should be constructed  
c               between actual wavelengths WL(Ia) and WL(Ia+1)
c               D1L      WL(Ia)        D1R    
c               [..........|.......[....].......|........]
c                                 D2L         WL(Ia+1)
c                           <-- extended grid -->

            N1 = ANINT((WL_R - WL_L)/DEL_RAM) 
            IF( N1 > 0 ) DELTA = (WL_R - WL_L)/N1            
c              WRITE(*,'(A,I3)')         '  Ia =  ', Ia
c              WRITE(*,'(A,2(2x,F8.4))') '  WL_L and  WL_R ', WL_L, WL_R
c              WRITE(*,'(A,E12.4)')      '  DEL_RAM =  ', DEL_RAM
c              WRITE(*,'(A,I3)')         '  N1 =  ', N1
c              WRITE(*,'(A,E12.4)')      '  Delta =  ', DELTA
            DO J = 1,N1 - 1
               N_Ext = N_Ext + 1  ;  WL_TMP(N_Ext) = WL_L + J*DELTA
            ENDDO
c           ... Set to .FALSE. because for the next actual wavelength,
c               i.e., WL(Ia+1) the left margin is already constructed
            From_left = .FALSE.
         ENDIF
      ENDDO  ! over actual wavelengths (Ia)

      WRITE(LUNIT,*) '   Number of actual wavelengths   :', N_ACT
      WRITE(LUNIT,*) '   Number of extended wavelengths : ',N_Ext
      WRITE(LUNIT,*) '   min[W0_Ext] and max[W0_Ext] :'
      WRITE(LUNIT,*)  MIN_WL, MAX_WL
      WRITE(LUNIT,*) '  First and last WL in the extended grid :'
      WRITE(LUNIT,*)  WL_TMP(1),  WL_TMP(N_Ext)

c      STOP

      ALLOCATE( SRR%WL_Ext(N_Ext),SRR%Act_WL(N_Ext),SRR%Exi_WL(N_Ext) )

c     ... Save extended wavelength grid in the structure element
c         SRR%WL_EXT
      SRR%WL_EXT(:) = WL_TMP(1:N_Ext) 

c      PAUSE
c      DO I = 1, N_Ext
c         WRITE(*,'(2x,I3,2x,F8.3)') I, SRR%WL_EXT(I)
c      ENDDO

c     ... Save logical array which .true. elements mark actual
c         wavelengths within extended WL-grid
      SRR%Act_WL(:) = ACT_TMP(1:N_Ext)

c      DO I = 1, N_Ext
c         IF( SRR%Act_WL(I) ) THEN
c            WRITE(*,'(2x,I5,2x,F8.3)') I, SRR%WL_EXT(I)
c         ENDIF
c      ENDDO

c     ... The array SRR%Exi_WL(:) is not equal to SRR%Act_WL(:)
c         in the case if of RRS and inelastic processes in ocean
c         should be calculated simultaneously
      SRR%Exi_WL(:) = .NOT.SRR%Act_WL(:)

      DEALLOCATE(WL_TMP, ACT_TMP)
      
      END SUBROUTINE EXTENDED_GRID

      END

