C********************************************************************C
C
C       SCIATRAN - Version 3.6 - Last modification: 01 Apr 2016
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE EXT_WL_VRS(N_ACT, WL_ACT, SVR, *)
c     ... Call in "rtm_main.f"

      USE LOG_PARS,  ONLY: EU, LUNIT, DIR_OUT
      USE ST_VARS,   ONLY: Pie, APPR_RRS, INEL_PROC
      USE TYPE_VARS, ONLY: KND
      USE TYPES,     ONLY: VRS_TYPE
      USE VRS,       ONLY: N_FRF, K_nu, A_FRF, Dnu_FRF, Sig_FRF, A_FChl,
     $                     STEP_VRS, Bet_0R, Nsig, Integr_FRF, Qe_Chl,
     $                     Tvs_Use, Tvs_Exc, Tvs_Act, Lrf => Lam_ref,
     $                     Lmi => Lam_int, Contour, WLG_Chl, Sig_Chl, 
     $                     N_phm
      USE WATER,     ONLY: INCL_INEL, Step_Flu, Wl_Flu, Nwl_QE, 
     $                     WL_QE, A_QE, A_Fa, B_Fa, A_Ha, B_Ha, 
     $                     FH_REF, Fu_par, Hu_par, Fua_c, Hua_c, 
     $                     Flu_CDOM, Abs_flu, N_FC
c     ------------------------------------------------------------------

c     ... Input parameters
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
      INTEGER    N_ACT
      REAL (KND) WL_ACT(N_ACT)
c     ... Local variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'EXT_WL_VRS'
      CHARACTER (LEN=72) Msg(10)
      INTEGER, PARAMETER :: MAX_EXR = 100
      INTEGER   I, J, K, N_Exi, Ne_flu, Lna2, Nse, NG_WL,
     $     Nvr, N_P(MAX_EXR)
      REAL (KND), PARAMETER :: D30 = 1.D-30
      REAL (KND) Ab1, Ab2, An1, An2, Brc, Dn1, Dn2, Dsig, FRF_INT,
     $     Nuj, Nu_ex, Nu_act, Nu_step, Nut, Rho, WL_INC, 
     $     S1, S2, Shift, N_mol, G_MIN, G_MAX, Min_Ste, DEL_CDOM
      REAL (KND) Nuc(N_FRF), WL_MIN(MAX_EXR), WL_MAX(MAX_EXR)
      LOGICAL, ALLOCATABLE :: L_vrs(:), L_flu(:), L_exi(:), Lo_FLu(:),
     $     Lo_VRS(:)
      REAL (KND), ALLOCATABLE :: WL_Exi(:), WL_TMP(:), WLe_flu(:),
     $     G_WL(:)

c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
c
c        Wavelength grid in the spectral range of excitation
c        ===================================================
      SELECT CASE( TRIM(INEL_PROC) )

c     ... VRS and Chlorophyll and/or CDOM fluorescence
c     ------------------------------------------------
      CASE('VRS_Fb','VRS_Fr','VRS_Fr_Fb')
c        ... The number of phase functions which will be used
c            to describe all inelastic processes involved
         N_phm = 2
c        ... Wavelength grid for fluorescence
         Ne_flu = CEILING( (Wl_Flu(2) - Wl_Flu(1))/Step_Flu(1) )  + 1
         ALLOCATE( WLe_flu(Ne_flu) )
                  WRITE(*,*) '  ---> Selected step :', Step_Flu(1)
c        ... Fluorescence excitation WL-grid
         WLe_flu(1) = WL_Flu(1)
         DO I = 1, Ne_flu - 1
            WLe_flu(I+1) = WLe_flu(I) + Step_Flu(1)
         ENDDO
         WLe_flu(Ne_flu) = Wl_Flu(2)

c        ... Number of segments and bounds of VRS excitation range
         CALL RANGE_VRS
                WRITE(*,*) '    Number of excitation VRS segments: ',Nse
                WRITE(*,*) '    Minimal and maximal wavelength:'
                DO I = 1,Nse 
                   WRITE(*,'(2x,I3,2(2x,F7.2))') I, WL_MIN(I), WL_MAX(I)
                ENDDO
                WRITE(*,*) '    Fluorescence range:'
                WRITE(*,'(5x,2(2x,F7.2))') WLe_flu(1), WLe_flu(Ne_flu)
c        ... Min and max wavelengths for VRS excitation range
         Dn1 = MINVAL( WL_MIN(1:Nse) ) ; Dn2 = MAXVAL( WL_MAX(1:Nse) ) 

c        ... Min and max wavelengths for all spectral range including
c            VRS and fluorescence excitation range
         G_MIN = DMIN1(Dn1,WLe_flu(1)) 
         G_MAX = DMAX1(Dn2,WLe_flu(Ne_flu))
                WRITE(*,*) G_MIN , G_MAX 
         Min_Ste = DMIN1(STEP_VRS,Step_Flu(1))

         Ng_WL   = CEILING( (G_MAX - G_MIN)/Min_Ste )  + 1
         ALLOCATE( G_WL(Ng_WL), L_vrs(Ng_WL), L_flu(Ng_WL) )
c        ... Wavelength grid in all excitation range
         G_WL(1) = G_MIN
         DO I = 1, Ng_WL - 1
            G_WL(I+1) = G_WL(I) + Min_Ste
         ENDDO
         G_WL(Ng_WL) = G_MAX
                WRITE(*,*) '    All range:'
                WRITE(*,'(5x,2(2x,F7.2))') G_WL(1), G_WL(Ng_WL)
         L_vrs(:) = .FALSE.
c        ... Do-loop over all segments of VRS excitation range
         DO I = 1,Nse 
            WHERE( G_WL(:)>=WL_MIN(I).AND.G_WL(:)<=WL_MAX(I) )
c              ... True elements of array L_vrs(:) mark wavelengths 
c                  pertaining to VRS excitation range
               L_vrs(:) = .TRUE.
            END WHERE
         ENDDO

         L_flu(:) = .FALSE.
c        ... True elements of array L_flu(:) mark wavelengths pertaining 
c            to fluorescence excitation ranges
         WHERE( G_WL(:)>=WLe_flu(1).AND.G_WL(:)<=WLe_flu(Ne_flu) )
            L_flu(:) = .TRUE.
         END WHERE

c        ... Number of wavelengths in all excitation ranges
         N_Exi = COUNT(  L_vrs(:).OR.L_flu(:) )
                WRITE(*,*) '  Ng_WL = ', Ng_WL
                WRITE(*,*) '  N_Exi = ', N_Exi
         ALLOCATE( L_exi(Ng_WL) )
c        ... True elements of array L_exi(:) mark wavelengths pertaining 
c            to VRS and fluorescence excitation ranges
         L_exi(:) = L_vrs(:).OR.L_flu(:)
         ALLOCATE( WL_Exi(N_Exi), SVR%Lo_VRS(N_Exi), Lo_Flu(N_Exi) )
         WL_Exi(:)     = PACK(G_WL, L_exi)
         SVR%Lo_VRS(:) = PACK(L_vrs,L_exi)
         Lo_Flu(:)     = PACK(L_flu,L_exi)

               K = 1
               DO I = 1,N_Exi
                  IF( SVR%Lo_VRS(I) ) THEN 
c                     WL_Exi(I) = WL_Exi(I) + 0.016
                     WRITE(*,'(2x,I3,2x,f8.3,2(2x,L))') 
     $                    K, WL_Exi(I), Lo_Flu(I), SVR%Lo_VRS(I)
                     K = K + 1
                  ENDIF
               ENDDO
c        STOP  " 'VRS_Fb','VRS_Fr','VRS_Fr_Fb' "
c     ... Fluorescence excitation range
c     ---------------------------------
      CASE('F_Chl','F_CDOM','Fr_Fb')
c        ... The number of phase function which will be used
c            to describe all inelastic processes involved
         N_phm = 1
c        ... Maximal wavelength among all actual
         G_MAX = MAXVAL( WL_ACT )
             WRITE(*,*) G_MAX, WL_Flu(1)
c        ... There is no fluorescence contribution in selected
c            actual wavelength range 
         IF( G_MAX <= WL_Flu(1) ) GOTO 201

         IF( TRIM(INEL_PROC)=='F_CDOM' ) THEN
c        ------------- For CDOM fluorescence --------------
            DEL_CDOM = 20.d0
c           ... Maximal wavelength where contribution from 
c               excitation region should be calculated
            G_MAX = G_MAX + DEL_CDOM
            WL_INC = DMIN1(G_MAX,WL_Flu(2))
c        --------------------------------------------------
         ELSE
            WL_INC = WL_Flu(2)
         ENDIF
c        ... Number of wavelengths within excitation range
         N_Exi = CEILING( (WL_INC - Wl_Flu(1))/Step_Flu(1) )  + 1
         ALLOCATE( WL_Exi(N_Exi), SVR%Lo_VRS(N_Exi))  
         SVR%Lo_VRS(:) = .FALSE.

c        ... Wavelength grid for fluorescence excitation range
         WL_Exi(1) = WL_Flu(1)
         DO I = 1, N_Exi - 1
            WL_Exi(I+1) = WL_Exi(I) + Step_Flu(1)
         ENDDO
         WL_Exi(N_Exi) = WL_INC

         WRITE(*,*) '   Fluorescence excitation WLs:'
         DO I = 1, N_Exi
            WRITE(*,'(2x,I3,2x,F8.3)') I, WL_Exi(I)
         ENDDO

c     ... VRS excitation range
c     ------------------------
      CASE('VRS')
c        ... The number of phase functions which will be used
c            to describe all inelastic processes involved
         N_phm = 1
c        ... Shift between excitation and actual wavelengths is 
c            defined as a weighted center positions of four 
c            Gaussian shapes
         Shift = SUM( Dnu_FRF(:)*A_FRF(:) )

c        ... No integration over VRS excitation range is required
         IF( .NOT.Integr_FRF ) THEN
c           ... Number of excitation wavelength is equal to actual
            N_Exi = N_ACT  
            ALLOCATE( WL_Exi(N_Exi), SVR%Lo_VRS(N_Exi) )
            SVR%Lo_VRS(:) = .TRUE.
            WL_Exi(:) = 1.d0/(K_nu/WL_ACT(:) + Shift)*K_nu
c        ... Integration over VRS excitation range should be performed
         ELSE
c            ... Number of segments and bounds of VRS excitation bands
            CALL RANGE_VRS
                WRITE(*,*) '    Number of excitation VRS segments: ',Nse
                WRITE(*,*) '    Minimal and maximal wavelength:'
                DO I = 1,Nse 
                   WRITE(*,'(2x,I3,2(2x,F8.3))') I, WL_MIN(I), WL_MAX(I)
                ENDDO
c            CALL GRID_VRS
c <<--------------------------------------------------------------------->>
c           ... As in combined RRS-VRS case
            G_MIN = WL_MIN(1) ;  G_MAX = WL_MAX(Nse)
c           ... Number of wavelengths within all excitation ranges
            Ng_WL = CEILING( (G_MAX - G_MIN)/Step_VRS )  + 1
            ALLOCATE( G_WL(Ng_WL), L_vrs(Ng_WL) )
c           ... Wavelength grid in CER
            G_WL(1) = G_MIN
            DO I = 1, Ng_WL - 1
               G_WL(I+1) = G_WL(I) + Step_VRS
            ENDDO
            L_vrs(:) = .FALSE.
c           ... Do-loop over segments of VRS excitation range
            DO I = 1,Nse
               WHERE( G_WL(:)>=WL_MIN(I).AND.G_WL(:)<=WL_MAX(I) )
                  L_vrs(:) = .TRUE.
               END WHERE
            ENDDO
            N_Exi = COUNT( L_vrs(:) )
c           ... Number of wavelengths in excitation range without
c               gap between segments.
            ALLOCATE( WL_Exi(N_exi), SVR%Lo_VRS(N_exi) )
            WL_exi(:)     = PACK(G_WL, L_vrs)
            SVR%Lo_VRS(:) = .TRUE.
c <<---------------------------------------------------------------------->>

         ENDIF

         WRITE(*,*) '   VRS excitation WLs:'
         DO I = 1, N_Exi
            WRITE(*,'(2x,I3,2x,F8.3)') I, WL_Exi(I)
         ENDDO
c         STOP  " 'VRS' "

      END SELECT

c     ... Save number of excitation wavelengths and 
c         total number of wavelengths
      SVR%N_AUX = N_Exi  ;  SVR%N_EXT = N_Exi + N_ACT
      ALLOCATE( SVR%WL_EXT(SVR%N_EXT),
     $     SVR%Act_WL(SVR%N_EXT), SVR%Exi_WL(SVR%N_EXT) ) ! A

c     ... Extended wavelength grid
      SVR%WL_EXT(1:N_Exi)           = WL_Exi(1:N_Exi)
      SVR%WL_EXT(N_Exi+1:SVR%N_EXT) = WL_ACT(1:N_ACT)

c     ... Actual wavelengths in the extended wavelength-array WL_EXT
c         will be marked by .TRUE.
      SVR%Act_WL(1:N_Exi)           = .FALSE.
      SVR%Act_WL(N_Exi+1:SVR%N_EXT) = .TRUE.
      SVR%Exi_WL(:)                 = .NOT.SVR%Act_WL(:)
 
c     ... Calculate wavelength redistribution function
c     ------------------------------------------------
      SELECT CASE( TRIM(INEL_PROC) )
      CASE('VRS')
         Nvr = 1
         ALLOCATE( SVR%V_FRF(Nvr,N_Exi,N_Act) )
         CALL FRF_VRS(Nvr)           ! VRS
      CASE('F_Chl')
c        ... Chlorophyll fluorescence 
         CALL FRF_RED 
      CASE('F_CDOM')
         ALLOCATE( SVR%V_FRF(2,N_Exi,N_Act) )
c         CALL FRF_BLUE          ! CDOM fluorescence 
         SELECT CASE( Flu_CDOM ) 
           CASE('FHa') ;  CALL FRF_BLUE_FHa   
           CASE('ACR') ;  CALL FRF_BLUE_ACR
         END SELECT
      CASE('Fr_Fb')
         ALLOCATE( SVR%V_FRF(2,N_Exi,N_Act) )
c        ... Chlorophyll fluorescence 
         CALL FRF_RED  
c        ... CDOM fluorescence 
         SELECT CASE( Flu_CDOM ) 
           CASE('FHa') ;  CALL FRF_BLUE_FHa   
           CASE('ACR') ;  CALL FRF_BLUE_ACR
         END SELECT
      CASE('VRS_Fb')
         Nvr = 3
         ALLOCATE( SVR%V_FRF(Nvr,N_Exi,N_Act) )
         CALL FRF_VRS(Nvr) 
c        ... CDOM fluorescence 
         SELECT CASE( Flu_CDOM ) 
           CASE('FHa') ;  CALL FRF_BLUE_FHa   
           CASE('ACR') ;  CALL FRF_BLUE_ACR
         END SELECT
      CASE('VRS_Fr')
         Nvr = 1
         ALLOCATE( SVR%V_FRF(Nvr,N_Exi,N_Act) )
         CALL FRF_VRS(Nvr) 
c        ... Chlorophyll fluorescence 
         CALL FRF_RED
       CASE('VRS_Fr_Fb')
         Nvr = 3
         ALLOCATE( SVR%V_FRF(Nvr,N_Exi,N_Act) )
         CALL FRF_VRS(Nvr) 
c        ... Chlorophyll fluorescence 
         CALL FRF_RED
c        ... CDOM fluorescence 
         SELECT CASE( Flu_CDOM ) 
           CASE('FHa') ;  CALL FRF_BLUE_FHa   
           CASE('ACR') ;  CALL FRF_BLUE_ACR
         END SELECT

      END SELECT

c      STOP '   End of ext_wl_rrs'

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 201  CALL WRITE_MESSAGE(R_NAME,'ERROR','')
      WRITE(EU,'(A,2x,F7.3,A)')
     $     '     Maximal value of emission wavelength',G_MAX,' nm'
      WRITE(EU,'(A,2x,F7.3,A)') 
     $     '     is smaller than minimal excitation wavelength',
     $     WL_Flu(1),' nm'
      WRITE(EU,'(A)')
     $     '     Fluorescence can not be calculated.'
      WRITE(EU,'(A)')
     $     '     Switch off fluorescence in "control_uwt.inp" file.'
      GOTO 881

c         Msg(1)='Medium where inelastic process should be accounted'
c         Msg(2)='  for is wrong defined.'
c         Msg(3)='Fatal error. Inform program developer.'
c         CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

 881  CONTINUE
      RETURN 1
      CONTAINS
c **********************************************************************

      SUBROUTINE RANGE_VRS
      REAL (KND) T_MAX, T_MIN, DUM(N_FRF)

c     ... Integration range in [cm-1] for four Gaussian shapes
      DUM(:) = Nsig*DSQRT(2.d0)*Sig_FRF(:)
c     ... Position (in cm-1) of the "center" of excitation band 
c         corresponding to the first actual wavelength
      Nuc(:) = K_nu/WL_Act(1) + Dnu_FRF(:)
c     ... Minimal and maximal wavelengths for this excitation band
      WL_MAX(1) = MAXVAL( K_nu/(Nuc(:) - DUM(:)) )
      WL_MIN(1) = MINVAL( K_nu/(Nuc(:) + DUM(:)) )
c
c     | excitation band |
c     [.................]                     |.......
c  WL_MIN(1)        WL_MAX(1)              WL_ACT(1)
c
c     ... Segmentation of full excitation range of VRS.
c         It will be investigated below whether there is no
c         overlap between two following excitations bands 
      Nse = 1
      DO I = 2,N_Act
c        ... Position (in cm-1) of the "center" of I-th excitation band 
c            corresponding to the first actual wavelength
         Nuc(:) = K_nu/WL_Act(I) + Dnu_FRF(:)
c        ... Minimal and maximal wavelengths for I-th excitation band
         T_MAX = MAXVAL( K_nu/(Nuc(:) - DUM(:)) )
         T_MIN = MINVAL( K_nu/(Nuc(:) + DUM(:)) )
c        ... Segmentation of the VRS excitation range
         IF( T_MIN > WL_MAX(Nse) ) THEN
c           There is no overlap between two following excitation bands
c                               T_MIN          T_MAX         WL_ACT(I)
c           [..............]      [..............]      |       |   
c          WL_MIN(Nse) WL_MAX(Nse)                  WL_ACT(Nse)
c
            Nse = Nse + 1
            WL_MAX(Nse) = T_MAX ; WL_MIN(Nse) = T_MIN
            WRITE(*,*) '   For actual wavelength ', I, WL_ACT(I)
            WRITE(*,*) '   new excitation segment is introduced'
         ELSE
c           There is overlap between two following excitation bands
c                     T_MIN          T_MAX      WL_ACT(I)
c            [..........[...]..........]      |    |   
c           WL_MIN(Nse) WL_MAX(Nse)       WL_ACT(Nse)

            WL_MAX(Nse) = T_MAX
         ENDIF
      ENDDO

      END SUBROUTINE RANGE_VRS
c **********************************************************************

      SUBROUTINE GRID_VRS

c       ... Minimal and maximal wavelength for all VRS excitation
c           segments
        G_MIN = WL_MIN(1) ;  G_MAX = WL_MAX(Nse)
c       ... Number of wavelengths within all excitation ranges
        Ng_WL = CEILING( (G_MAX - G_MIN)/Step_VRS )  + 1
        ALLOCATE( G_WL(Ng_WL), L_vrs(Ng_WL) )
c       ... Wavelength grid in CER
        G_WL(1) = G_MIN
        DO I = 1, Ng_WL - 1
           G_WL(I+1) = G_WL(I) + Step_VRS
        ENDDO
        L_vrs(:) = .FALSE.
c       ... Do-loop over segments of VRS excitation range
        DO I = 1,Nse
           WHERE( G_WL(:)>=WL_MIN(I).AND.G_WL(:)<=WL_MAX(I) )
              L_vrs(:) = .TRUE.
           END WHERE
        ENDDO

        N_Exi = COUNT( L_vrs(:) )
c           ... Number of wavelengths in excitation range without
c               gap between segments.
        ALLOCATE( WL_Exi(N_exi), SVR%Lo_VRS(N_exi) )
        WL_exi(:)     = PACK(G_WL, L_vrs)
        SVR%Lo_VRS(:) = .TRUE.

      END SUBROUTINE GRID_VRS
c **********************************************************************

      SUBROUTINE FRF_RED

      USE VRS, ONLY: Nh, WLH, Apeh
c     ------------------------------------------------------------------

      INTEGER, PARAMETER:: N_sig = 4
      REAL (KND)  TMP(N_Exi), Sth, WL_L, WL_R
      REAL (KND), ALLOCATABLE:: Em_f(:)
c     ==================================================================

c     ... Integration step in [nm]
      Sth = 1.d0

c     ... Sig_Chl is root mean square of Gaussian emission band
c         WLG_Chl - wavelength corresponding to the maximum of
c         Gaussian emission band 
      WL_L = WLG_Chl - N_sig*Sig_Chl
      WL_R = WLG_Chl + N_sig*Sig_Chl
      Nh = CEILING( (Wl_R - Wl_L)/Sth )  + 1
      ALLOCATE( WLH(Nh), Em_f(Nh), Apeh(Nh) )

c     ... WL-grid to integrate over CHl-a emission band 
      WLH(1) = WL_L
      DO I = 1, Nh - 1
         WLH(I+1) = WLH(I) + Sth
      ENDDO
      WLH(Nh) = Wl_R

c     ... Calculate fluorescence emission function 
      Em_f(:) = 1.d0/DSQRT(2.d0*Pie)/Sig_Chl/WLH(:)
     $     *DEXP(-(WLH(:) - WLG_Chl)**2/2.d0/Sig_Chl**2 )
      DO I = 1, Nh-2
         Apeh(I) = 0.5d0*Em_f(I) + SUM( Em_f(I+1:Nh-1) )
      ENDDO
      Apeh(1:Nh-2) = Apeh(1:Nh-2) + 0.5d0*Em_f(Nh)
      Apeh(Nh-1) = 0.5d0*(Em_f(Nh-1) + Em_f(Nh))
      Apeh(:) = Apeh(:)*Sth

c     ... Output for test purpose
      OPEN( 44, file=DIR_OUT//'/Apeh.dat' )
        DO I = 1,Nh
           WRITE(44,*) WLH(I), Apeh(I)
        ENDDO
      CLOSE(44)

c     ... Integrate over full excitation wavelength range to 
c         obtain fluorescence absorption part of chlorophyll-a
      TMP(:) = 1.d0/DSQRT(2.d0*Pie)/Sig_Chl/WL_Exi(:)
     $     *DEXP(-(WL_Exi(:) - WLG_Chl)**2/2.d0/Sig_Chl**2 )
      
      TMP(1)     = TMP(1)*(WL_Exi(2) - WL_Exi(1))
      TMP(N_Exi) = TMP(N_Exi)*(WL_Exi(N_Exi) - WL_Exi(N_Exi-1))
      DO I = 2,N_Exi-1
         TMP(I) = TMP(I)*(WL_Exi(I+1) - WL_Exi(I-1))
      ENDDO
      A_FChl = SUM( TMP(:) )*0.5d0 

      WRITE(*,*) ' ------ In routine "ext_wl_vrs" -------'
      WRITE(*,*) '  Old integration range', WL_Exi(1), WL_Exi(N_Exi)
      WRITE(*,*) '  Compare old and new with relative difference:'
      WRITE(*,*) A_FChl, Apeh(1), (A_FChl - Apeh(1))/A_FChl*100.
c      STOP

      END SUBROUTINE FRF_RED
c **********************************************************************

c      SUBROUTINE FRF_BLUE
c      INTEGER    N_WL, Nk, Kk
c      REAL (KND) WL1, WL2, DWL, Iex, Iem
c      REAL (KND), ALLOCATABLE :: Re_Fu(:,:,:), WL_INT(:), A_Exi(:,:)
c      REAL (KND)  abs_tes(N_exi)
c      ALLOCATE( Re_Fu(2,N_Exi,N_ACT), A_Exi(2,N_Exi), Abs_flu(2,N_Exi) )
c      
cc     ... Interpolation of FRF parameters on the given excitation 
cc         wavelength grid
c      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(1,:), N_Exi,WL_Exi,A_Exi(1,:), 'Z')
c      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(2,:), N_Exi,WL_Exi,A_Exi(2,:), 'Z')
c
cc     ... Spectral range of excitation is in the wavelength
cc         WL_QE(1:Nwl_QE)
c
c
cc     ... Wavelength grid to integrate over emission band
cc         WL1 is minimal wavelength of the excitation range
c      WL1 = WL_QE(1) ;  WL2 = 700.d0 ; DWL = 10.d0
c
c      N_WL = CEILING((WL2 - WL1)/DWL) + 1
c      ALLOCATE( WL_INT(N_WL) ) ; WL_INT(1) = WL1
c      DO I = 1, N_WL - 1
c         WL_INT(I+1) = WL_INT(I) + DWL
c      ENDDO
c      WL_INT(N_WL) = WL2
c
cc     ... Fluorescence wavelength redistribution functions
cc         multiplied by specific absorption coefficients
c      DO I = 1, N_Exi
c         Iex = 1.d0/WL_Exi(I)
cc        ... Fulvic acid
c         Ab1 = A_Exi(1,I)
cc              ... Specific absorption coefficient (see Eq.(Spec-ac) 
cc                  in SCIATRAN Theoretical Background (STB))
c     $        *Fu_par(1)*DEXP( Fu_par(2)*(FH_REF-WL_Exi(I)) )
c         An1 =        A_Fa(1)*Iex + B_Fa(1)
c         Dn1 = 0.6d0*(A_Fa(2)*Iex + B_Fa(2))
c
cc        ... Humic acid
c         Ab2 = A_Exi(2,I)
cc              ... Specific absorption coefficient
cc                  (see Eq.(Spec-ac) in STB)
c     $        *Hu_par(1)*DEXP( Hu_par(2)*(FH_REF-WL_Exi(I)) )
c         An2 =        A_Ha(1)*Iex + B_Ha(1)
c         Dn2 = 0.6d0*(A_Ha(2)*Iex + B_Ha(2))
c
cc        ... Redistribution function (see Eq.(Flu-ef) and (Flu-wrf)
cc            in STB)
c         DO K = 1,N_ACT
c            Iem = 1.d0/WL_ACT(K)
c            Re_Fu(1,I,K) = Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
c            Re_Fu(2,I,K) = Ab2*DEXP(-((Iem - An2)/Dn2)**2)*Iem/Iex
c         ENDDO
c
cc        ... Integration over emission range to calculate late 
cc            the fluorescence part of CDOM absorption coefficient
cc            (see Eq.(Fpart-ac) in STB)
c         Nk = MINLOC( DABS(WL_INT(:)-WL_Exi(I)),1 )
c         IF( WL_Exi(I) > WL_INT(Nk) ) THEN
c            Iem = 1.d0/WL_Exi(I) ; Kk = Nk+1
c         ELSE
c            Iem = 1.d0/WL_Exi(I) ; Kk = Nk
c         END IF
c         WRITE(*,*) I, WL_Exi(I), WL_INT(Kk)
c
c         S1 = Ab1/Iex/2.d0
c     $        *( DEXP(-((1.d0/WL_INT(1)  -An1)/Dn1)**2)*Iem
c     $        + DEXP(-((1.d0/WL_INT(N_WL)-An1)/Dn1)**2)/WL_INT(N_WL) )
c         DO K = Nk,N_WL - 1
c            Iem = 1.d0/WL_INT(K)
c            S1 = S1 + Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex            
c         ENDDO
c         Abs_tes(I) = S1*Dwl 
c
c
c         S1 = Ab1/Iex/2.d0
c     $        *( DEXP(-((1.d0/WL_INT(1)  -An1)/Dn1)**2)/WL_INT(1)
c     $        + DEXP(-((1.d0/WL_INT(N_WL)-An1)/Dn1)**2)/WL_INT(N_WL) )
c         S2 = Ab2/Iex/2.d0
c     $        *( DEXP(-((1.d0/WL_INT(1)  -An2)/Dn2)**2)/WL_INT(1)
c     $        + DEXP(-((1.d0/WL_INT(N_WL)-An2)/Dn2)**2)/WL_INT(N_WL) )
c         DO K = 2,N_WL - 1
c            Iem = 1.d0/WL_INT(K)
c            S1 = S1 + Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
c            S2 = S2 + Ab2*DEXP(-((Iem - An2)/Dn2)**2)*Iem/Iex
c         ENDDO
c         Abs_flu(1,I) = S1*Dwl  ;  Abs_flu(2,I) = S2*Dwl
c      ENDDO
c      SVR%V_FRF(1:2,:,:) = Re_Fu(1:2,:,:)
c
cc     ... Write test results
c      WRITE(*,*) '   Number of actual WL: ',N_ACT
c      open(72,file=DIR_OUT//'/abs_flu.dat')
c      DO I = 1, N_Exi
c         write(72,'(2x,4(1x,E15.7))') 
c     $        WL_Exi(I), Abs_flu(1,I), Abs_flu(2,I), abs_tes(I)
c      ENDDO
c      close(72)
c
c      STOP
c
c      open(72,file=DIR_OUT//'/act_wl.dat')
c      DO I = 1, N_ACT
c         write(72,*) WL_ACT(I)
c      ENDDO
c      close(72)
c      open(72,file=DIR_OUT//'/FRF_Fa.dat')
c      DO I = 1, N_Exi
c         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
c     $        WL_Exi(I), (Re_Fu(1,I,K),K=1,N_ACT)
c      ENDDO
c      close(72)
c      open(72,file=DIR_OUT//'/FRF_Ha.dat')
c      DO I = 1, N_Exi
c         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
c     $        WL_Exi(I), (Re_Fu(2,I,K),K=1,N_ACT)
c      ENDDO
c      close(72)
c
c      END SUBROUTINE FRF_BLUE
c **********************************************************************

      SUBROUTINE FRF_BLUE_ACR
      USE WATER,   ONLY: Nwl_QEx, WL_QEx, F_QEx, Nwl_QEm, WL_QEm, F_QEm
c     ------------------------------------------------------------------

      INTEGER    Nwi, Kp
      REAL (KND) WL1, WL2, DWL
      REAL (KND), ALLOCATABLE :: WLi(:), Em_loc(:), Hel(:)
c     ==================================================================

      ALLOCATE( Abs_flu(N_FC,Nwl_QEx) )

c     ... Start of excitation spectral range
      WL1 = WL_QEx(1) 
c     ... End of emission spectral range
      WL2 = WL_QEm(Nwl_QEm)

c     ... Wavelength grid to integrate over all emission band
c         to calculate fluorescent absorption part
      DWL = 1.d0  ;  NWi = CEILING((WL2 - WL1)/DWL) + 1
      ALLOCATE( WLi(NWi) ) 
      WLi(1) = WL1
      DO I = 1, NWi - 1
         WLi(I+1) = WLi(I) + DWL
      ENDDO
      WLi(NWi) = WL2
      ALLOCATE( Em_Loc(NWi), Hel(NWi) )

c     ... Do-loop over fluorescent components
      DO J = 1,N_FC
c        ... Emission function will be interpolated at this wavelength 
c            grid (extrapolation is zero)
         CALL LIN_INT(Nwl_QEm, WL_QEm, F_QEm(J,:), NWi, WLi, Em_Loc,'Z')
c        ... Do-loop over excitation wavelengths
         DO I = 1, Nwl_QEx
            Hel(:) = F_QEx(J,I)*WL_QEx(I)*Em_Loc(:)/WLi(:)
c           ... Integration over emission band should start at
c               current excitation wavelength, which has the 
c               following position within array WLi:
            Kp = MINLOC( DABS(WL_QEx(I)-WLi(:)),1 )
            S1 =  0.d0
            DO K = Kp, NWi-1
               S1 = S1 + (Hel(K)+Hel(K+1))*(WLi(K+1)-Wli(K))
            ENDDO
            Abs_flu(J,I) = S1/2.d0         
         ENDDO
      ENDDO

      open(72,file=DIR_OUT//'/abs_flu_ACR.dat')
      DO I = 1, Nwl_QEx
         write(72,'(2x,10(1x,E15.7))') 
     $        WL_QEx(I), (abs_flu(K,I),K=1,N_FC)
      ENDDO
      close(72)

c      STOP 'storp in "ext_wl_vrs.f"'

      END SUBROUTINE FRF_BLUE_ACR
c **********************************************************************

      SUBROUTINE FRF_BLUE_FHa

      INTEGER    Nwi, Kp
      REAL (KND) WL1, WL2, DWL, Iex, Iem
      REAL (KND), ALLOCATABLE :: Re_Fu(:,:,:), A_Exi(:,:), H_FRF(:),
     $     WLi(:)
      REAL (KND)  A_Xa(2), B_Xa(2), Xu_par(2)
c     ==================================================================

      ALLOCATE( Re_Fu(2,N_Exi,N_ACT), A_Exi(2,N_Exi), 
     $     Abs_flu(2,Nwl_QE) )  ! A
      
c     ... Interpolation of FRF parameters on the given excitation 
c         wavelength grid
      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(1,:), N_Exi,WL_Exi,A_Exi(1,:), 'Z')
      CALL LIN_INT(Nwl_QE,WL_QE,A_QE(2,:), N_Exi,WL_Exi,A_Exi(2,:), 'Z')

c     ... Spectral range of excitation is in the wavelength WL_QE(1:Nwl_QE). 
c         Extract start and end of emission range
      WL1 = WL_QE(1) ;   WL2 = 700.d0 

c     ... Wavelength grid to integrate over all emission band. Here
c         it is assumed that the emission band started at beginning
c         of the excitation range.
c     ... Step to integrate over emission range 
      DWL = 1.d0  ;  NWi = CEILING((WL2 - WL1)/DWL) + 1
      ALLOCATE( WLi(NWi), H_FRF(Nwi) ) 
      WLi(1) = WL1
      DO I = 1, NWi - 1
         WLi(I+1) = WLi(I) + DWL
      ENDDO
      WLi(NWi) = WL2

      DO J = 1,2
         IF( J == 1 ) THEN
            Xu_par = Fu_par(:) ; A_Xa = A_Fa(:) ; B_Xa = B_Fa(:)
         ELSE
            Xu_par = Hu_par(:) ; A_Xa = A_Ha(:) ; B_Xa = B_Ha(:)            
         ENDIF
c        ... Do-loop over excitation wavelengths
         DO I = 1, Nwl_QE
            Iex = 1.d0/WL_QE(I)
            Ab1 = A_QE(J,I)
     $           *Xu_par(1)*DEXP( Xu_par(2)*(FH_REF-WL_QE(I)) )
            An1 =        A_Xa(1)*Iex + B_Xa(1)
            Dn1 = 0.6d0*(A_Xa(2)*Iex + B_Xa(2))
c           ... For the current excitation wavelength will be
c               calculated FRF in all needed spectral range
            DO K = 1, NWi
               Iem = 1.d0/WLi(K)
               H_FRF(K) = Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
            ENDDO
c           ... Integration over emission band should start at
c               current excitation wavelength, which has the 
c               following position within array WLi:
            Kp = MINLOC( DABS(WL_QE(I)-WLi(:)),1 )
            S1 =  0.d0
            DO K = Kp, NWi-1
               S1 = S1 + (H_FRF(K)+H_FRF(K+1))*(WLi(K+1)-Wli(K))
            ENDDO
            Abs_flu(J,I) = S1/2.d0         
         ENDDO
      ENDDO

c     ... Fluorescence wavelength redistribution functions
c         multiplied by specific absorption coefficients
      DO I = 1, N_Exi
         Iex = 1.d0/WL_Exi(I)
c        ... Fulvic acid
         Ab1 = A_Exi(1,I)
c              ... Specific absorption coefficient (see Eq.(Spec-ac) 
c                  in SCIATRAN Theoretical Background (STB))
     $        *Fu_par(1)*DEXP( Fu_par(2)*(FH_REF-WL_Exi(I)) )
         An1 =        A_Fa(1)*Iex + B_Fa(1)
         Dn1 = 0.6d0*(A_Fa(2)*Iex + B_Fa(2))

c        ... Humic acid
         Ab2 = A_Exi(2,I)
c              ... Specific absorption coefficient
c                  (see Eq.(Spec-ac) in STB)
     $        *Hu_par(1)*DEXP( Hu_par(2)*(FH_REF-WL_Exi(I)) )
         An2 =        A_Ha(1)*Iex + B_Ha(1)
         Dn2 = 0.6d0*(A_Ha(2)*Iex + B_Ha(2))

c        ... Redistribution function (see Eq.(Flu-ef) and (Flu-wrf)
c            in STB)
         DO K = 1,N_ACT
            Iem = 1.d0/WL_ACT(K)
            Re_Fu(1,I,K) = Ab1*DEXP(-((Iem - An1)/Dn1)**2)*Iem/Iex
            Re_Fu(2,I,K) = Ab2*DEXP(-((Iem - An2)/Dn2)**2)*Iem/Iex
         ENDDO

      ENDDO
      SVR%V_FRF(1:2,:,:) = Re_Fu(1:2,:,:)

c     ... Write test results
      WRITE(*,*) '   Number of actual WL: ',N_ACT

      open(72,file=DIR_OUT//'/abs_flu_tes.dat')
      DO I = 1, Nwl_QE
         write(72,'(2x,4(1x,E15.7))') 
     $        WL_QE(I), abs_flu(1,I), abs_flu(2,I)
      ENDDO
      close(72)

      open(72,file=DIR_OUT//'/act_wl.dat')
      DO I = 1, N_ACT
         write(72,*) WL_ACT(I)
      ENDDO
      close(72)
      open(72,file=DIR_OUT//'/FRF_Fa.dat')
      DO I = 1, N_Exi
         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
     $        WL_Exi(I), (Re_Fu(1,I,K),K=1,N_ACT)
      ENDDO
      close(72)
      open(72,file=DIR_OUT//'/FRF_Ha.dat')
      DO I = 1, N_Exi
         write(72,'(2x,F7.2,100(1x,e12.4E3))') 
     $        WL_Exi(I), (Re_Fu(2,I,K),K=1,N_ACT)
      ENDDO
      close(72)

      END SUBROUTINE FRF_BLUE_FHa
c **********************************************************************

      SUBROUTINE FRF_VRS(Nn)
      INTEGER, INTENT(IN) :: Nn
      INTEGER  N_c, Nint, K2
      REAL (KND)  Norm, T_MAX, T_MIN, T_T, S1, DUM(N_FRF)
      REAL (KND), ALLOCATABLE :: Re_Fu(:,:,:), DEL_EXT(:), R_int(:)
c     ------------------------------------------------------------------

      ALLOCATE( SVR%Tot_Vrs(SVR%N_EXT), DEL_EXT(N_Exi), R_int(N_Exi) ) ! A

c     ... It is allowed for the test purpose to use constant
c         total Raman scattering coefficient performing the 
c         integration over excitation wave range
      IF( Tvs_use ) THEN
c        ... Use constant user-defined total Raman scattering coefficients
         SVR%Tot_Vrs(1:N_Exi) = Tvs_exc
         SVR%Tot_Vrs(N_Exi+1:SVR%N_EXT) = Tvs_act
      ELSE
c        ... Calculate total Raman scattering coefficients as
c            a function of excitation wavelength
         N_c = SVR%N_EXT
         SELECT CASE( TRIM(Contour) )
         CASE('Inverse_power_law')
c           ... For this option Lmi is an exponent of inverse power 
c               law (input parameter). In previous versions Lmi==4.
c                    Bet_0R is calculated in "read_uwt.f"
            SVR%Tot_Vrs(1:N_Exi)     = Bet_0R*( Lrf/WL_Exi(:) )**Lmi
            SVR%Tot_Vrs(N_Exi+1:N_c) = Bet_0R*( Lrf/WL_ACT(:) )**Lmi

         CASE('Resonance')
c           ... For this option Lmi is a position of the intermediate 
c               energy level for water [nm] (input parameter). 
            Shift = SUM( Dnu_FRF(:)*A_FRF(:) )
            SVR%Tot_Vrs(1:N_Exi) = Bet_0R*(K_nu/WL_Exi(:) - Shift)**4
     $           /( (K_nu/Lmi)**2 - (K_nu/WL_Exi(:))**2)**2
            SVR%Tot_Vrs(N_Exi+1:N_c) = Bet_0R*(K_nu/WL_ACT(:) 
     $           - Shift)**4/( (K_nu/Lmi)**2 - (K_nu/WL_ACT(:))**2)**2

         END SELECT
      ENDIF

      WHERE( .NOT.SVR%Lo_VRS(:) )
         SVR%Tot_Vrs(1:N_Exi) = 0.d0
      END WHERE

      IF( .NOT.Integr_FRF ) THEN
         SVR%V_FRF = 0.d0
         Norm = DSQRT(2.d0*Pie)*SUM( A_FRF(:)*Sig_FRF(:) )
c        ... Integration range [cm^-1]
         DUM(:) = Nsig*DSQRT(2.d0)*Sig_FRF(:)
         DO K = 1,N_Act
c           ... Position of the excitation wavelengths in [cm^-1] 
            Nuc(:) = K_nu/WL_ACT(K) + Dnu_FRF(:)
            T_MAX = MAXVAL( K_nu/(Nuc(:) - DUM(:)) )
            T_MIN = MINVAL( K_nu/(Nuc(:) + DUM(:)) )
c            WRITE(*,'(A,2(2x,E12.4))') '  Integration range: ',
c     $           T_MIN, T_MAX
            Nint  = CEILING( (T_MAX - T_MIN)/STEP_VRS ) + 1
            S1 = 0.d0
            DO I = 1, Nint
               T_T = T_MIN + (I-1)*STEP_VRS ; Nut = K_nu/T_T
               S1 = S1 + SUM( A_FRF(:)
     $              *DEXP( -( (Nut-Nuc(:))/Sig_FRF(:))**2/2.d0 ) )
     $              *K_nu/T_T**2
            ENDDO
            WRITE(*,'(A,2x,f4.1)') 
     $           '                                  Nsig: ', Nsig
            WRITE(*,'(A,2(2x,E12.4))') 
     $           ' Integral of FRF over excitation range: ', 
     $           S1/Norm*STEP_VRS
            WRITE(*,*) 
            SVR%V_FRF(Nn,K,K) = S1/Norm*STEP_VRS*SVR%Tot_Vrs(K)
            WRITE(*,*) ' Raman scattering at excitation wl :'
            WRITE(*,'(2x,E12.4)') SVR%V_FRF(Nn,K,K)
         ENDDO    ! over excitation wavelengths (K)
      ELSE
         ALLOCATE( Re_Fu(1,N_Exi,N_ACT) ) ! A

c        ... Norm-factor in the VRS frequency redistribution function (FRF)
         Norm = DSQRT(2.d0*Pie)*SUM( A_FRF(:)*Sig_FRF(:) )

c        ... Frequency redistribution function (FRF)
         DO K = 1,N_ACT
c           ... Position of the excitation wavelengths in [cm^-1] 
            Nuc(:) = K_nu/WL_ACT(K) + Dnu_FRF(:)
            DO I = 1, N_Exi
               Nut = K_nu/WL_Exi(I)
               Re_Fu(1,I,K) = SUM( A_FRF(:)
     $              *DEXP( -( (Nut-Nuc(:))/Sig_FRF(:))**2/2.d0 ) )
            ENDDO
            Re_Fu(1,:,K) = Re_Fu(1,:,K)/Norm
         ENDDO
c        ... Quadrature coefficients
         DEL_EXT(1)         = SVR%WL_EXT(2)      - SVR%WL_EXT(1)
         DEL_EXT(2:N_Exi-1) = SVR%WL_EXT(3:N_Exi)- SVR%WL_EXT(1:N_Exi-2)
         DEL_EXT(N_Exi)     = SVR%WL_EXT(N_Exi)  - SVR%WL_EXT(N_Exi-1)

c        ... Correct quadrature coefficients at left and right 
c            bounds of a segment
         DO I = 1,Nse
            K2 = MINLOC( DABS(WL_Exi(:)-WL_MAX(I)),1 )
            DEL_EXT(K2) = WL_Exi(K2) - WL_Exi(K2-1)
            IF( I > 1 ) THEN
               K2 = MINLOC( DABS(WL_Exi(:)-WL_MIN(I)),1 )
               DEL_EXT(K2) = WL_Exi(K2+1) - WL_Exi(K2)
            ENDIF
         ENDDO
         DEL_EXT(:) = DEL_EXT(:)/2.d0
c         DEL_EXT(:) = 1.d0

c     ******************************************************************
c            Renormalization of FRF using employed numerical 
c            integration quadrature and spectral range
c     ******************************************************************
         WRITE(*,'(A)') 
     $        '  Integral of FRF over excitation range :'
         Dum(:) = Nsig*DSQRT(2.d0)*Sig_FRF(:)
         DO K = 1,N_ACT
            Nuc(:) = K_nu/WL_ACT(K) + Dnu_FRF(:)
c           ... Integration range for four Gaussian bands
            T_MAX = MAXVAL( K_nu/(Nuc(:) - Dum(:)) )
            T_MIN = MINVAL( K_nu/(Nuc(:) + Dum(:)) )
            WHERE( WL_Exi(:)>=T_MIN.AND.WL_Exi(:)<=T_MAX )
               R_int(:) = Re_Fu(1,:,K)*DEL_EXT(:)/WL_exi(:)**2*K_nu
            ELSE WHERE
               R_int(:) = 0.d0
            END WHERE
            S1 = SUM( R_int(:) )
            Re_Fu(1,:,K) = Re_Fu(1,:,K)/S1
            WRITE(*,'(2x,I2,2x,F8.3,2x,e12.4)') K, WL_ACT(K), S1
         ENDDO
c     ******************************************************************
c
c        ... Vibrational Raman scattering coefficient
c            [K_nu] = nm/cm ; [Re_Fu]   = cm
c            [WL_Exi]  = nm ; [DEL_EXT] = nm
         DO K = 1,N_ACT
            SVR%V_FRF(Nn,:,K) = SVR%Tot_Vrs(1:N_Exi)*Re_Fu(1,:,K)
     $           *K_nu/WL_Exi(:)**2*DEL_EXT(:)
         ENDDO

c         WRITE(*,*) '    Total RSC within excitation range'
c         DO I = 1, N_Exi
c            WRITE(*,'(2x,f7.2,2x,f8.5)')  WL_Exi(I), SVR%Tot_Vrs(I)
c         ENDDO
c         WRITE(*,*) '    Total RSC at emission wavelengths'
c         DO I = 1, N_ACT
c            WRITE(*,'(2x,f7.2,2x,f8.5)') WL_ACT(I), SVR%Tot_Vrs(N_Exi+I)
c         ENDDO

c         STOP '  In "ext_wl_vrs.f" '
c         write(*,*) '  N_ACT = ', N_ACT
c         open(72,file=DIR_OUT//'/FRF_VRS_vrs.dat',RECL=10000)
c         DO I = 1, N_Exi
c            write(72,'(2x,F7.2,100(1x,e12.4E3))') 
c     $           WL_Exi(I), (SVR%V_FRF(Nn,I,K),K=1,N_ACT)
c         ENDDO
c         close(72)
c         STOP
      ENDIF

      END SUBROUTINE FRF_VRS
c **********************************************************************

      END
