!********************************************************************C
!
!	SCIATRAN - Version 3.5 - Last modification: 16 Apr 2015
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!********************************************************************C
SUBROUTINE EXTEND_INFO_ARR (*)
! Called by "gt_iface.f" and "ret_control.f"

  USE AEROSOLS,   ONLY: EXA_OUT_FLAG, AP_Mie, Rf_max, Rf_min, Rc_max, MODIS_AP
  USE DATA_RET,   ONLY: PRESET_APR_NAME, TPS_NAME
  USE INFO,       ONLY: RET_INFO, AER_INFO
  USE ST_VARS,    ONLY: RET_MODE, DO_AEROSOL
  USE TOOLS,      ONLY: CHK_STAT, FIND_IDX, INQ_WF
  USE TYPE_VARS,  ONLY: KND

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'EXTEND_INFO_ARR'
  CHARACTER (LEN=200) STR_READ, STR_READ_ARR(100)
  INTEGER    KK
  REAL (KND) Rc_min 
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')

  SELECT CASE (TRIM(RET_MODE))
  CASE('aer-bim','aer-hom','aer-ratio')
     ! Aerosol parameters which will be retrieved  are defined in 
     ! "control_wf.inp" file. These are all parameters listed  
     ! in "Weighting functions: aerosol parameters" input line.

     ! Scaling factor of aerosol number density: 
     IF( CHK_STAT('aer-dens', AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-dens',RET_INFO%NAME)
        RET_INFO(KK)%N_PARS = 1
        RET_INFO(KK)%WF_DIM = 'rel'
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        IF( MODIS_AP ) THEN
           ! Min and max. AOD
           RET_INFO(KK)%PAR_MIN = 0.1
           RET_INFO(KK)%PAR_MAX = 3.
        ELSE
           ! Min and max. number of aerosol particles
           ! Continental clean:
           RET_INFO(KK)%PAR_MIN = 1.d+2
           RET_INFO(KK)%PAR_MAX = 1.d+8
           ! RET_INFO(KK)%PAR_MIN = 2.d+3 
           ! RET_INFO(KK)%PAR_MAX = 5.d+3
           ! RET_INFO(KK)%PAR_MIN = 1.d+4 
           ! RET_INFO(KK)%PAR_MAX = 3.d+4
        ENDIF
     ENDIF

     ! Modal radius of the fine mode: 
     IF( CHK_STAT('aer-modr_f', AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-modr_f',RET_INFO%NAME)
        RET_INFO(KK)%WF_DIM    = 'rel'
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        ! minimal value of the fine mode modal radius [micron]:
        ! Rf_min = 0.01d0  ;  Rf_max = 0.3d0
        Rf_min = 0.01d0  ;  Rf_max = 0.25d0
        RET_INFO(KK)%PAR_MIN = Rf_min 
        RET_INFO(KK)%PAR_MAX = Rf_max 
     ENDIF

     ! Modal radius of the course mode: 
     IF( CHK_STAT('aer-modr_c', AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-modr_c',RET_INFO%NAME)
        ! Wf should be calculated for absolute variation
        RET_INFO(KK)%WF_DIM    = 'rel' ! this does not work
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        ! Min and max value of course mode modal radius [micron]:
        ! Rc_min = Rf_max ; Rc_max = 2.5d0
        Rc_min = 0.3 ; Rc_max = 6.0d0
        RET_INFO(KK)%PAR_MIN = Rc_min
        RET_INFO(KK)%PAR_MAX = Rc_max
     ENDIF
     ! Variance (fine mode):
     IF( CHK_STAT('aer-var_f', AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-var_f',RET_INFO%NAME)
        RET_INFO(KK)%WF_DIM    = 'rel'
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        ! Min and max value of fine mode variance:
        Rf_min = 2.0d0  ;  Rf_max = 3.d0
        RET_INFO(KK)%PAR_MIN = (DLOG(Rf_min))**2
        RET_INFO(KK)%PAR_MAX = (DLOG(Rf_max))**2
     ENDIF

     ! Variance (course mode):
     IF( CHK_STAT('aer-var_c', AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-var_c',RET_INFO%NAME)
        RET_INFO(KK)%WF_DIM    = 'rel' ! this does not work
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        ! Min and max value of course mode variance:
        Rc_min = 2.d0 ; Rc_max = 3.d0
        RET_INFO(KK)%PAR_MIN = (DLOG(Rc_min))**2
        RET_INFO(KK)%PAR_MAX = (DLOG(Rc_max))**2
     ENDIF

     ! Fraction of course mode: 
     IF( CHK_STAT('aer-cm_frac',AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
        KK = FIND_IDX('aer-cm_frac',RET_INFO%NAME)
        ! Use WF in the retrieval process for absolute variations
        RET_INFO(KK)%WF_DIM    = 'rel' 
        RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
        ! minimal and maximal values of coarse fraction
        ! Continental clean:
        RET_INFO(KK)%PAR_MIN = 0.1d-4 ;  RET_INFO(KK)%PAR_MAX = 0.1d-3 
        ! RET_INFO(KK)%PAR_MIN = 0.3d-6 ;  RET_INFO(KK)%PAR_MAX = 0.2d-3 
     ENDIF

     ! Surface albedo
     ! IF(TRIM(RET_MODE)=='aer-hom' .AND. .NOT.INQ_WF('albedo','RIN')) GOTO 605

  CASE('lwc_ref')
     KK = FIND_IDX('cld-tau_nomix',RET_INFO%NAME)
     RET_INFO(KK)%WF_DIM    = 'rel'
     RET_INFO(KK)%PAR_MIN =   0.0d0 ;  RET_INFO(KK)%PAR_MAX = 300.0d0
         
     KK = FIND_IDX('cld-ref_nomix',RET_INFO%NAME)
     ! Name for auxiliary weighting function
     RET_INFO(KK)%NAME    = 'cld-ref_top'
     RET_INFO(KK)%WF_DIM    = 'abs'
     ! Name for corresponding basis weighting function
     RET_INFO(KK)%WF_NAME = 'cld_ref'
     ! The parameter should be considered as integrated
     RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
     ! Lower and upper bound of constrain
!     RET_INFO(KK)%PAR_MIN =  2.0d0 ;  RET_INFO(KK)%PAR_MAX = 20.0d0
     RET_INFO(KK)%PAR_MIN =  5.0d0 ;  RET_INFO(KK)%PAR_MAX = 60.0d0
           
     KK = FIND_IDX('undefined',RET_INFO%NAME)
     RET_INFO(KK)%NAME    = 'cld-ref_bot'
     RET_INFO(KK)%WF_DIM    = 'abs'
     RET_INFO(KK)%WF_NAME = 'cld_ref'
     RET_INFO(KK)%WF_ON   = .FALSE.
     RET_INFO(KK)%TYPE    = 'cld-s'
     RET_INFO(KK)%VS     = 'V' ;  RET_INFO(KK)%VSI     = 'I'
     RET_INFO(KK)%PAR_MIN =  2.0d0 ;  RET_INFO(KK)%PAR_MAX = 20.0d0
  END SELECT

  SELECT CASE (TRIM(PRESET_APR_NAME))
  CASE('H2O')
     TPS_NAME = 'h2o'
     IF(DO_AEROSOL) THEN
        KK = FIND_IDX('undefined',RET_INFO%NAME)
        RET_INFO(KK)%NAME    = 'exa_s'
        RET_INFO(KK)%WF_NAME = RET_INFO(KK)%NAME
        RET_INFO(KK)%TYPE    = 'num'
        RET_INFO(KK)%VS     = 'S' ;  RET_INFO(KK)%VSI = 'S'
        RET_INFO(KK)%WF_DIM    = 'rel'
     END IF
     KK = FIND_IDX('elev',RET_INFO%NAME)
     RET_INFO(KK)%NAME    = 'xxx'
     RET_INFO(KK)%STAT    = .FALSE.
     KK = FIND_IDX('undefined',RET_INFO%NAME)
     RET_INFO(KK)%NAME    = 'elev'
     RET_INFO(KK)%WF_NAME = RET_INFO(KK)%NAME
     RET_INFO(KK)%TYPE    = 'num'
     RET_INFO(KK)%VS     = 'S' ;  RET_INFO(KK)%VSI = 'S'
     RET_INFO(KK)%WF_DIM  = 'abs'
     KK = FIND_IDX('undefined',RET_INFO%NAME)
     RET_INFO(KK)%NAME    = 'tps_col'
     RET_INFO(KK)%WF_NAME = RET_INFO(KK)%NAME
     RET_INFO(KK)%STAT    = .FALSE.
     RET_INFO(KK)%TYPE    = 'num'
     RET_INFO(KK)%VS     = 'S' ;  RET_INFO(KK)%VSI = 'S'
     RET_INFO(KK)%WF_DIM  = 'rel'
  END SELECT

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
 
!605 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The "aer-hom" aerosol retrieval ' //&
!         'mode requires retrieval of surface albedo as well.')
  RETURN 1

END SUBROUTINE EXTEND_INFO_ARR
!***************************************************************************
