c *************************************************************************
c
C       SCIATRAN - Version 3.9 - Last modification: 29 Dec 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c    Upward and downward radiance at the bottom of each homogeneous layers
c            for all Gauss angels and viewing zenith angles
c            derived using the solution of the linear system 
c                         (Pseudo-forward RTE)
c
c **************************************************************************
      SUBROUTINE Field_Adj_Gau(NL, NN, LVEC, SDO, PART_TU, PART_TD, AFI,
     $     SOL_TOP)
c     ... Call in "st_adjoint_a.f"
c                 "st_adjoint_o.f"

      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: DIR_OUT
      USE ST_VARS,     ONLY: N_RHS, N_Adj, SDVI
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) :: NL, NN
      REAL (KND), INTENT(IN) :: LVEC(2*Nvs*NN*(NL-1),N_Adj), 
     $     Part_TU(NL,2*Nvs*NN,N_Adj), Part_TD(NL,2*Nvs*NN,N_Adj),
     $     SOL_TOP(2*Nvs*NN,N_RHS)
c     -------------------------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
c     ----------------------
      REAL (KND), INTENT(OUT):: Afi(NL,2*Nvs*NN,N_Adj)
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'FIELD_ADJ_GAU'
      INTEGER     I, Jv, Jco, Jro, Lk, K1, LL1, LR1, NNv
      REAL (KND)  RHS(2*Nvs*NN)
      REAL (KND), POINTER :: Pp(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = Nvs*NN
c     ... Pseudo-forward field at the top of first layer
      FORALL( Jv=1:N_Adj ) AFI(1,:,Jv) = SOL_TOP(:,Jv+SDVI)
      DO I = 1, NL - 1
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1-1
         Pp => SDO%EXP_NU_NL(:,I)
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
         DO Jv = 1,N_Adj
            RHS(:)     = LVEC( LL1:LR1+NNv,Jv )
            RHS(1:Jro) = LVEC( LL1:Lk+Jro,Jv )*Pp(1:Jro)
            DO K1 = Jro+1, Jro+2*Jco, 2
               RHS(K1) = 
     $              LVEC(Lk+K1,Jv)  *Pp(K1) + LVEC(Lk+K1+1,Jv)*Pp(K1+1)
               RHS(K1+1) = 
     $              LVEC(Lk+K1+1,Jv)*Pp(K1) - LVEC(Lk+K1,Jv)  *Pp(K1+1)
            ENDDO
c           ... Field of the pseudo-forward Stokes vector at the bottom 
c               of the layer "I"
            AFI(I+1,:,Jv)  = MATMUL(SDO%G_NL(I,:,:),RHS(:)) 
     $           + PART_TD(I,:,Jv)
         ENDDO
      ENDDO

cc       WRITE(*,*) '  N_Adj = ', N_Adj
c       WRITE(*,*) '****** afi-1.std is written ***** '
c       Open( 12, file = DIR_OUT//'/afi-1.std' )
c       DO I = 1, NL
c          WRITE( 12,'(*(1x,e15.7))')
c     $         ZZZ(I), (AFI(I,K1,1), K1 = 1,2*NNv)
c       ENDDO
c       Close(12)
c       Open( 12, file = DIR_OUT//'/afi-2.std' )
c       DO I = 1, NL
c          WRITE( 12,'(*(1x,e15.7))')
c     $         ZZZ(I), (AFI(I,K1,2), K1 = 1,2*NNv)
c       ENDDO
c       Close(12)
c
c        Open( 12, file = DIR_OUT//'/afi-3.std' )
c       DO I = 1, NL
c          WRITE( 12,'(*(1x,e15.7))')
c     $         ZZZ(I), (AFI(I,K1,3), K1 = 1,2*NNv)
c       ENDDO
c       Close(12)
c       Open( 12, file = DIR_OUT//'/afi-4.std' )
c       DO I = 1, NL
c          WRITE( 12,'(*(1x,e15.7))')
c     $         ZZZ(I), (AFI(I,K1,4), K1 = 1,2*NNv)
c       ENDDO
c       Close(12)
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
