***************************************************************************
C
C        SCIATRAN - Version 3.5 - Last modification: 20 Oct 2015
C
c    Upward and downward radiance at the bottom of each homogeneous layers
c            for all Gauss angels and solar zenith angles
c       derived using the solution of the linear system (Forward RTE)
c
c **************************************************************************
      SUBROUTINE Field_Dir_Gau( NN, NSZA, NL, XM, DTR, PART_D, MVEC,
     $     SOL_TOP, SDO, DFI )
c     ... Called by "field_iface.f"
c                   "st_dom_tau.f"
      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs
      USE ST_VARS,     ONLY: N_RHS, Kvs
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NN, NSZA, NL
      REAL (KND), INTENT(IN) :: XM(NN), DTR(NL), 
     $     SOL_TOP(2*Nvs*NN,N_RHS), MVEC(2*Nvs*NN*(NL-1),N_RHS), 
     $     PART_D(NL,2*Nvs*NN,Kvs*NSZA)
      REAL (KND), INTENT(OUT):: DFI(NL,Kvs*NSZA,2*Nvs*NN)
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
c     ----------------------
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'FIELD_DIR_GAU'
      INTEGER   I, J, Js, K, K1, LL1, LR1, Lk, Jco, Jro, NNv
      REAL (KND) S1, XM_TWO(2*NN), RHS(2*Nvs*NN)
      REAL (KND), POINTER :: Pp(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = Nvs*NN

      XM_TWO(1   :NN  ) =  XM(1:NN)
      XM_TWO(NN+1:2*NN) = -XM(1:NN)

!         WRITE(*,*) '          N_RHS - ', N_RHS
!         WRITE(*,*) '           NSZA - ', NSZA
!         WRITE(*,*) '           MVEC - ', SUM( MVEC(:,1:NSZA) )
!         WRITE(*,*) '  SDO%EXP_NU_NL - ', SUM( SDO%EXP_NU_NL )
!         WRITE(*,*) '       SDO%G_NL - ', SUM( SDO%G_NL )

c     ... Field at the top of first layer
      FORALL( Js=1:Kvs*NSZA ) DFI(1,Js,:) = SOL_TOP(:,Js)

c     ... Intensity field is obtained at the bottom of all layer
      DO I = 1, NL - 1
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1-1
         Pp => SDO%EXP_NU_NL(:,I)
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
c        ... Replace matrix elements in a layer with conservative scattering
         IF( SDO%MASK_NL(I) ) THEN
            S1 = 3.d0/SDO%H1(I) ; K1 = SDO%E_MIN(I)
c           ... The value is given at the bottom of corresponding layer
            SDO%G_NL(I,1:2*NNv:Nvs,    K1) =  S1*XM_TWO(:)
            SDO%G_NL(I,1:2*NNv:Nvs,NNv+K1) = -S1*XM_TWO(:)+DTR(I+1)
         ENDIF

         DO Js = 1,Kvs*NSZA
            RHS(:)     = MVEC( LL1:LR1+NNv,Js )
            RHS(1:Jro) = MVEC( LL1:Lk+Jro,Js )*Pp(1:Jro)
            DO K1 = Jro+1, Jro+2*Jco, 2
               RHS(K1) = 
     $              MVEC(Lk+K1,Js)  *Pp(K1) + MVEC(Lk+K1+1,Js)*Pp(K1+1)
               RHS(K1+1) = 
     $              MVEC(Lk+K1+1,Js)*Pp(K1) - MVEC(Lk+K1,Js)  *Pp(K1+1)
            ENDDO
c           ... Up- and downward forward Stokes vectors at the bottom 
c               of layer "I"
            DFI(I+1,Js,:) = MATMUL(SDO%G_NL(I,:,:),RHS(:)) 
     $           + PART_D(I,:,Js)             
         ENDDO
      ENDDO

c      Js = 1
c      WRITE(*,*) ' ******* DFI(NL,1,ref) and  '
c      WRITE(*,*) '         DFI(NL-1,1,ref) in "field_dir_gau" : '
c      WRITE(*,*) '         for SZA    : ',Js
c      DO K = NNv+1,2*NNv
c         WRITE(*,'(5x,I5,*(2x,E12.4))') K, DFI(NL,Js,K), DFI(NL-1,Js,K)
c      ENDDO

c         WRITE(*,*) '   SHAPE(DFI) - ', SHAPE(DFI)
c         WRITE(*,*) '     0 - ', SUM( DFI )
c            Js = 1
c            IF( Kvs > 1 ) Js = 2
c
c            write(*,*) '   Upward at top in "field_dir_gau":'
c            write(*,*) '   Stokes I    Q     U'
c            K = 1
c            do K1 = NNv, 2*NNv-Nvs, Nvs
c               WRITE(*,'(2x,I5,3(2x,E15.7))') K, 
c     $              DFI(1,Js,K1+1), DFI(1,Js,K1+2), DFI(1,Js,K1+3)
c               K = K + 1
c            enddo
c            write(*,*) '   Downward at bottom in "field_dir_gau":'
c            write(*,*) '   Stokes I    Q     U'
c            K = 1
c            do K1 = 0, NNv-Nvs, Nvs
c               WRITE(*,'(2x,I5,3(2x,E15.7))') K, 
c     $              DFI(NL,Js,K1+1), DFI(NL,Js,K1+2), DFI(NL,Js,K1+3)
c               K = K + 1
c            enddo
c      Js = 1
c      WRITE(*,*) ' ******* DFI output  : ', SHAPE(DFI)
c      WRITE(*,*) '   Number of SZA     : ', NSZA
c      WRITE(*,*) '   Number of NL      : ', NL
c      WRITE(*,*) '   Number of columns : ', 2*NNv
cc
c      Open(12,file='./DATA_OUT/DFI.dat',RECL=15000)
c      DO I = 1, NL
cc        WRITE( 12,'(1x,I3,130(1x,e15.7))')
cc     $        I,(DFI(I,Js,J), J = 1,2*NNv)
c         WRITE(12,'(130(1x,e15.7))')
c     $        ZZZ(I),(DFI(I,Js,J), J = 1,2*NNv)
c      ENDDO
c      Close(12)
c      PAUSE
c      STOP '   STOP in "Field_Dir_Gau"'

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END 
c **********************************************************************
