C********************************************************************C
C
C	SCIATRAN - Version 3.9 - Last modification: 28 Dec 2017
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE FIELD_IFACE( Mh, AGR_A, AGR_W, GSL_A, GSL_W,
     $     IOP_A, IOP_W, LAS_A, LAS_W, LOA, LOW, Ph_zA, Ph_zW,
     $     SDO_A, SDO_W, SSB_A, SSB_W, TE, UDB_A, UDB_W, Obs_POSITION )
c     ... Called by "st_dom_tau.f"

      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT, ONLY: N_OP
      USE TYPE_VARS,  ONLY: KND
      USE TYPES,      ONLY: LOA_TYPE, LOW_TYPE, SDO_TYPE, TE_TYPE,
     $                      UDB_TYPE, SSB_TYPE, GSL_TYPE, LAS_TYPE,
     $                      IOP_TYPE, AGR_type
      USE WATER,      ONLY: UWT
c     ==================================================================

      TYPE (AGR_type) :: AGR_A, AGR_W ! Altitude grids
      TYPE (GSL_type) :: GSL_A, GSL_W ! Angles
      TYPE (IOP_TYPE), TARGET :: IOP_A, IOP_W ! Integral optical parameters
      TYPE (LAS_TYPE) :: LAS_A, LAS_W ! Linear algebraic system
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of water
      TYPE (SDO_type) :: SDO_A, SDO_W ! DOM auxiliary solution variables
      TYPE (SSB_type) :: SSB_A, SSB_W ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission 
      TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
c     ----------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA, Ph_zW
c     ---------------------------------------
      INTEGER, INTENT(IN)    :: Mh
      CHARACTER (LEN=5)  Obs_POSITION(N_OP)
c     ... local variables
      CHARACTER (LEN=11), PARAMETER :: R_NAME = 'FIELD_IFACE'
      INTEGER   I, J, Js, NN, NL, NSZA
      REAL (KND), POINTER :: XM(:), DTR(:), MVEC(:,:), DFI(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     ... New version (output in both medium for "flux" and "spher_alb")
      CALL CASE_A
      IF( UWT ) CALL CASE_B
         
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE CASE_A
c         WRITE(*,*) '    Integral optical parameters for Atmosphere'
         NN = GSL_A%NN  ;  NL = AGR_A%NL  ;  NSZA = GSL_A%NSZA         
         XM   => GSL_A%XM
         DTR  => IOP_A%DTR 
         MVEC => IOP_A%MVEC
         DFI  => IOP_A%DFI
         IF( AGR_A%ADAPTIVE_GRID ) THEN
c            write(*,*) '     Adaptive Mh = ', Mh
            CALL ST_UP_DOWN_SFI(Mh, NN, NSZA, NL, IOP_A, SSB_A, TE,
     $           SDO_A, LAS_A%Ref_surf, UDB_A, LOA%Bet, 
     $           Ph_zA, GSL_A, 'Atmos')
         ELSE
c            write(*,*) '     Standard Mh = ', Mh
            CALL Field_Dir_Gau( NN, NSZA, NL, XM, DTR,
     $           LAS_A%PART_D, MVEC, LAS_A%SOL_TOP, SDO_A, DFI )
         ENDIF
cc        ... Gauss field of forward RTE in atmosphere
c         IF( Mh==1 ) THEN
c            Js = 1
c            write(*,*) '  DFI: Mh, and Js ', Mh, Js
c            DO I = 1, NL
c               WRITE(*,'(1x,I3,*(2x,e12.4))') I,
c     $              (DFI(I,Js,Nvs*NN+J), J=1,Nvs)
c            ENDDO
c         ENDIF
      END SUBROUTINE CASE_A
c **********************************************************************

      SUBROUTINE CASE_B
c         WRITE(*,*) '    Integral optical parameters for Ocean'
         NN = GSL_W%NN  ;  NL = AGR_W%NL  ;  NSZA = GSL_W%NSZA         
         XM   => GSL_W%XM
         DTR  => IOP_W%DTR 
         MVEC => IOP_W%MVEC
         DFI  => IOP_W%DFI
         IF( AGR_W%ADAPTIVE_GRID ) THEN
            CALL ST_UP_DOWN_SFI(Mh, NN, NSZA, NL, IOP_W, SSB_W, TE,
     $           SDO_W, LAS_W%Ref_surf, UDB_W, LOW%Bet, 
     $           Ph_zW, GSL_W, 'Ocean')
c            WRITE(*,*) '        ---->    AG'
         ELSE
            CALL Field_Dir_Gau( NN, NSZA, NL, XM, DTR,
     $           LAS_W%PART_D, MVEC, LAS_W%SOL_TOP, SDO_W, DFI )
c            WRITE(*,*) '        ----> No AG'
         ENDIF

c           write(*,*) '    DFI output for top of ocean ', NL
c           DO I = 1, NL
c              WRITE( *,'(2x,I2,130(1x,e12.4))')
c     $             I,SUM( DFI(I,1,1:NN) ), SUM( DFI(I,1,NN+1:2*NN) )
c           ENDDO

cc     ... Gauss field of forward RTE in ocean
c         OPEN(12,file=DIR_OUT'DFI_O.dat',RECL=15000)
c         Js = 1
c         DO I = 1, NL
c            WRITE( 12,'(130(1x,e15.7))') (DFI(I,Js,J), J = 1,2*NN)
c         ENDDO
c         CLOSE(12)

      END SUBROUTINE CASE_B

      END 
