***************************************************************************
C
C        SCIATRAN - Version 3.9 - Last modification: 18 Dec 2017
C
c    Upward and downward pseudo-forward radiance at the bottom of each 
c     homogeneous layers for all Gauss angels and solar zenith angles
c   derived using the solution of the linear system (pseudo-forward RTE)
c
c      It is used to test solution of the forward RTE obtained 
c               employing the forward-adjoint technique
c **************************************************************************
      SUBROUTINE Field_Pf_Gau(NN, NSZA, NL, Mh, XM, AG, Ph_zA, ALW, 
     $     DTR,  PART_TD, PART_TU, MVEC, SDO, DFI, EXS, EXL, MuV, NLSA,
     $     E_W, RADOUT, ALBEDO, ADO, MuS)
c     ... Call in  "st_dom_tau.f" if SOLVER is set to "Solv_ASI"

      USE GRIDS,       ONLY: DZR
      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: N_OP, Wup
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_OBS_ALT, N_AZ_VAR, ANG_OUT,
     $                       N_LOS_OUT
      USE ST_VARS,     ONLY: F14, N_RHS, N_Adj, Kvs, SDVI, Pie, Finc, 
     $                       Vec_Sca, Nc_der, GeoD => GeoD_OUT
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, ADO_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NN, NSZA, NLSA, NL, Mh
      REAL (KND), INTENT(IN) :: ALBEDO, XM(NN), DTR(NL), AG(NN), E_W, 
     $     MVEC(2*Nvs*NN*(NL-1),N_RHS), PART_TD(NL,2*Nvs*NN,N_Adj), 
     $     EXL(NL,NLSA), PART_TU(NL,2*Nvs*NN,N_Adj), ALW(NL-1), 
     $     EXS(NL,NSZA), MuV(NLSA), Mus(NSZA)
      REAL (KND), INTENT(OUT):: DFI(NL,Kvs*NSZA,2*Nvs*NN)
      REAL (KND) RADOUT(N_OP,N_LOS_OUT,Nvs)
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
c     ----------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA
c     ----------------------
c     ... Local parameters
c     ----------------------
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'FIELD_Pf_GAU'
      INTEGER   I, J, Jg, Jv, JJ, Js, K, Kg, Kg1, Kg2, K1, K2, Ks, 
     $     K_lim, L, LL1, LR1, Lk, Jco, Jro, NNv, Kv, IA
      REAL (KND), DIMENSION(NL,Nvs*NN,NSZA) :: Qm_Mu, Qp_Mu
      REAL (KND) S1, CV, Xg, XM_TWO(2*NN), RHS(2*Nvs*NN), XML(NN), Cazm,
     $     SOL_TOP(2*Nvs*NN,N_RHS), AGL(Nvs*NN), Qp(Nvs*NN), Qm(Nvs*NN),
     $     S4(Nvs), Scal_1(NL,NSZA), Re1(NSZA), Re2(NSZA,NLSA),
     $     MSL(Nvs,Nvs), S_Ls(NL,NLSA,NSZA), Term_alb(NSZA,NLSA)
      REAL (KND), POINTER :: Pp(:), Ex1(:), P01(:,:,:), P02(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = Nvs*NN
c     ... Cosine of the viewing angle
      CV = DABS( ANG_OUT(2,1) )

c     ... Phase matrices are calculated in "phase_matr.f" 
c       +Gauss --> Sol_ang             Gauss --> -Sol_ang
      P01 => Ph_zA%Egs(:,:,:,1)  ;  P02 => Ph_zA%Egs(:,:,:,2)

      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         K_lim = 0
         XML(:) = XM(:) ; AGL(:) = AG(:)
      CASE( 'vect_rte' )
         K_lim = 3
         IF( Mh==1 ) K_lim=1
         XML(:) = 1.d0  
         DO K = 1,NNv 
            K2 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K2) 
         ENDDO
      END SELECT

c     ... Single scattering source function (right-hand side of
c         the forward RTE) without multiplier e^{-\tau/mu0}
      DO Js = 1,NSZA
         Ks = (Js-1)*Nvs
         DO I = 1,NL
c           ... Because P01 was designed to describe Gau --> Sol, 
c           (see "phase_matr.f"), we multiply P01^T*Finc(1,:) to obtain 
c           single scattering Sol --> Gau.
c               The multiplication of P01^T by D4 from right is omitted
c           because it is assumed that Finc=diag{1,0,0,0}.
            DO Jg = 1,NN
               Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs ; Xg = XML(Jg)
               DO K2 = 1,Nvs
                  Qp(Kg1+K2-1) = 
     $                 SUM( P01(I,Kg1:Kg2,Ks+K2)*Xg*Finc(1,:) )
               ENDDO
            ENDDO  
c           ... The multiplication of P01^T by D4 from left is down
c               See Eq.(Qp-sol) in STB  
            FORALL( K = 4:NNv:Nvs ) Qp(K) = -Qp(K)

c           ... Because P02 was designed to describe Gau --> -Sol, 
c           (see "phase_matr.f") we multiply P02^T*Finc(1,:) to obtain 
c           single scattering Sol --> -Gau.
c               The multiplication of P02^T by D4 from right is omitted
c           because it is assumed that Finc=diag{1,0,0,0}.
            DO Jg = 1,NN
               Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs ; Xg = XML(Jg)
               DO K2 = 1,Nvs
                  Qm(Kg1+K2-1) = 
     $                 SUM( P02(I,Kg1:Kg2,Ks+K2)*Xg*Finc(1,:) )
               ENDDO 
            ENDDO  
c           ... The multiplication of P02^T by D3 from left is down
c               See Eq.(Qm-sol) in STB  
            FORALL( K = 3:NNv:Nvs ) Qm(K) = -Qm(K)

c           ... Multiplier e^{-\tau/mu0} is accounted for
            Qp_Mu(I,:,Js) = F14*Qp(:)*EXS(I,Js)
            Qm_Mu(I,:,Js) = F14*Qm(:)*EXS(I,Js) 
         ENDDO                  ! ovre layers (I)
      ENDDO                     ! over solar angle (Js)

      XM_TWO(1   :NN  ) =  XM(1:NN)
      XM_TWO(NN+1:2*NN) = -XM(1:NN)

c     ... Pseudo-forward solution at the top and bottom of a medium
c
      Ex1 => SDO%Ex_AG(:,1) ; Jco = SDO%NcC(1)  ;  Jro = NNv - 2*Jco
      DO JJ = 1,N_Adj
         Js = JJ + SDVI
c           ... At the top of the medium
         RHS(1:NNv)         = MVEC(    1:NNv    ,Js)
         RHS(1+NNv:NNv+Jro) = MVEC(1+NNv:NNv+Jro,Js)*Ex1(1:Jro)
         DO K = Jro+1, Jro+2*Jco, 2
            RHS(NNv+K)  = 
     $           MVEC(NNv+K,Js)  *Ex1(K) + MVEC(NNv+K+1,Js)*Ex1(K+1)
            RHS(NNv+K+1)= 
     $           MVEC(NNv+K+1,Js)*Ex1(K) - MVEC(NNv+K,Js)  *Ex1(K+1)
         ENDDO
         SOL_TOP(:,Js) = MATMUL(SDO%G_NL(1,:,:),RHS(1:2*NNv) ) 
     $        + PART_TU(1,:,JJ)
      ENDDO

c     ... Field at the top of first layer
      FORALL( Js=1:Kvs*NSZA ) DFI(1,Js,:) = SOL_TOP(:,Js)

c     ... Intensity field of the pseudo-forward radiation is obtained 
c         at the bottom of all layers
      DO I = 1, NL - 1
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1-1
         Pp => SDO%EXP_NU_NL(:,I)
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco

c        ... Replace matrix elements in a layer with conservative scattering
         IF( SDO%MASK_NL(I) ) THEN
            S1 = 3.d0/SDO%H1(I) ; K1 = SDO%E_MIN(I)
c           ... The value is given at the bottom of corresponding layer
            SDO%G_NL(I,1:2*NNv:Nvs,    K1) =  S1*XM_TWO(:)
            SDO%G_NL(I,1:2*NNv:Nvs,NNv+K1) = -S1*XM_TWO(:)+DTR(I+1)
         ENDIF

         DO Js = 1,Kvs*NSZA
            RHS(:)     = MVEC( LL1:LR1+NNv,Js )
            RHS(1:Jro) = MVEC( LL1:Lk+Jro,Js )*Pp(1:Jro)
            DO K1 = Jro+1, Jro+2*Jco, 2
               RHS(K1) = 
     $              MVEC(Lk+K1,Js)  *Pp(K1) + MVEC(Lk+K1+1,Js)*Pp(K1+1)
               RHS(K1+1) = 
     $              MVEC(Lk+K1+1,Js)*Pp(K1) - MVEC(Lk+K1,Js)  *Pp(K1+1)
            ENDDO
c           ... Field of the forward Stokes vector at the bottom of layer "I"
            DFI(I+1,Js,:)  = MATMUL(SDO%G_NL(I,:,:),RHS(:)) 
     $           + PART_TD(I,:,Js)             
         ENDDO
      ENDDO

c     ... The upward pseudo-forward Stokes vector is obtained if 
c         we change the sign of the 3 an 4th components 
      DO I = 1, NL
         DO Js = 1,NSZA
            FORALL( K = NNv+3:2*NNv:Nvs ) DFI(I,Js,K)= -DFI(I,Js,K)
            FORALL( K = NNv+4:2*NNv:Nvs ) DFI(I,Js,K)= -DFI(I,Js,K)
         ENDDO
      ENDDO

      DO I = 1, NL
         DO Js = 1,NSZA
            RHS(:) = DFI(I,Js,:)
c           ... Scalar product over Gaussian angles
            DO K = 1,4
               S4(K) = SUM( 
     $              ( RHS(NNv+K:2*NNv:Nvs)*Qp_Mu(I,K:NNv:Nvs,Js)
     $              +RHS(    K:NNv  :Nvs)*Qm_Mu(I,K:NNv:Nvs,Js) )
     $              *AG(1:NN) )
            ENDDO
c           ... Change sign of 3th component in the scalar product
            S4(3) = -S4(3)

            Scal_1(I,Js) = SUM( S4(:) )
         ENDDO
      ENDDO

c      ... Integrate over layers
      DO Js = 1, NSZA
         S1 = 0.d0
         DO I = 1,NL-1
            S1 = S1 + (Scal_1(I,Js) + Scal_1(I+1,Js))*0.5d0*DZR(I)
         ENDDO
         Re1(Js) = S1
      ENDDO
       
      WRITE(*,*) '   '
      WRITE(*,*) '  -------- field_pf_gau -------------  '
c       WRITE(*,*) '  Re1 =  ', Re1(1)
c       WRITE(*,*) '  -----------------------------------  '

c      ... Single scattering source function of the forward RTE
c          in the direction of observation
      DO I = 1,NL
         DO Js = 1,NSZA
            Ks = (Js-1)*Nvs
            DO Jv = 1,NLSA
               Kv = (Jv-1)*Nvs
               MSL(:,:) =  Ph_zA%EsL(I,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1)
               DO K2 = 1,Nvs
                  S4(K2) = SUM( MSL(:,K2)*Finc(1,:) )
               ENDDO 
               S_Ls(I,Jv,Js) = 
     $              F14*S4(Nc_der(1))*EXS(I,Js)*EXL(I,Jv)/MuV(Jv)
            ENDDO
         ENDDO
      ENDDO

c      ... Integrate over layers
      DO Js = 1, NSZA
         DO Jv = 1, NLSA
            S1 = 0.d0
            DO I = 1,NL-1
               S1 = S1 + (S_Ls(I,Jv,Js) + S_Ls(I+1,Jv,Js))*0.5d0*DZR(I)
            ENDDO
            Re2(Js,Jv) = S1
         ENDDO
      ENDDO
c       WRITE(*,*) '  -----------------------------------  '
c       WRITE(*,*) '  Re2 =  ', Re2(1,1)
c       WRITE(*,*) '  -----------------------------------  '

      Re2(1,1) = Re1(1) + Re2(1,1)

      IA = 1  ;  Term_alb = 0.d0
      IF( Mh==1.AND.Nc_der(1)==1 ) THEN
         DO Jv = 1,NLSA
            IF( GeoD%LVG(IA,Jv)%C_VA(1) <= 0.d0 ) CYCLE
            Term_alb(:,Jv) = 
     $           Albedo*EXS(NL,:)*( ADO%Flux_d(Jv) + EXL(NL,Jv) )
         ENDDO
      ENDIF

      DO Js = 1,NSZA
         DO Jv = Jv_S(Js),Jv_E(Js)
            IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN ! reflected
               Re2(Js,Jv) = Re2(Js,Jv) + Term_alb(Js,Jv) *2.*MuS(Js)
            ELSE ! transmitted
               Re2(Js,Jv) = Re2(Js,Jv)
     $              + Albedo*EXS(NL,JS)*ADO%Flux_d(Jv)*MuS(Js)
            ENDIF
         ENDDO
      ENDDO
      RADOUT = 0.d0
      
      J = 0
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         DO Js = 1,NSZA
            DO K = JV_S(Js),JV_E(Js)
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  RADOUT(1,J,1) = Re2(Js,K)*E_W
     $                 *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
               ENDDO
            ENDDO
         ENDDO
      CASE( 'vect_rte' )
         IF( Nc_der(1) < 3 ) THEN
            DO Js = 1,NSZA
               DO K = JV_S(Js),JV_E(Js)
                  DO L = 1,N_AZ_VAR  
                     J = J + 1 
                     Cazm = 0.5d0*DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                     RADOUT(1,J,Nc_der(1)) = Re2(Js,K)*E_W*Cazm
                  ENDDO
               ENDDO
            ENDDO
         ELSE
            DO Js = 1,NSZA
               DO K = JV_S(Js),JV_E(Js)
                  DO L = 1,N_AZ_VAR  
                     J = J + 1 
                     Cazm = 0.5d0*DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )
                     RADOUT(1,J,Nc_der(1)) = Re2(Js,K)*E_W*Cazm
                  ENDDO
               ENDDO
            ENDDO
         ENDIF

         WRITE(*,*) '  -----------------------------------  '
         WRITE(*,'(A,I2,2x,E15.7)') 
     $        ' Mh = ', Mh,  RADOUT(1,:,Nc_der(1))
      END SELECT

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END 
c **********************************************************************
c Expl*)
c
c      
c     The same hold for matrix P02 to obtain single scattering 
c     Sol_ang --> -Gauss.
c     Multiplication P01^T and P02^T from left and right by
c     D4=diag{1,1,1,-1} is omitted because single scattered
c     solar radiation does not contain V Stokes component.
c     !!! It should be corrected in the case of !!!
c                 incoming polarized light
c
c **************************************************************************
