********************************************************************C
C
C       SCIATRAN - Version 3.1 - Last modification: 09 Jun 2013
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C *********************************************************************

c                  !!! For the reflected intensity only !!!
c         !!! Only real separation constants in the vector case !!!
c                   --------------------------------
c     ... Scalar product <Ia,Q> proverka vosstanovleniya intensivnosty
c            dlya Mu(N_DEL) po rechenijy sopr. uravneniya
c     ... Radiance from adjoint equation
c --------------------------------------------------------------------
      SUBROUTINE FLS_ASI(NL, NN, NSZA, NLSA, Mh, MuS, MuV, ALBEDO,
     $     E_W, ALW, DTR, XM, AG, Ph_zA, PVEC, SDO, ADO, EXL, EXS, 
     $     Ex_D, RADOUT)
c     ... Call in "st_dom_tau.f"

      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: N_OP, Wup
      USE ST_VARS,     ONLY: F14, DO_SATELLITE, N_RHS, Finc, Nc_der,
     $                       Vec_Sca, Ainc
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_OBS_ALT, N_AZ_VAR, ANG_OUT,
     $                       N_LOS_OUT
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, ADO_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NL, NN, NSZA, NLSA, Mh
      REAL (KND), INTENT(IN) :: ALBEDO, E_W, ALW(NL-1), DTR(NL),
     $     XM(NN), AG(NN), Mus(NSZA), MuV(NLSA), EXS(NL,NSZA),
     $     EXL(NL,NLSA), Ex_D(Nvs*NN,NL-1), PVEC(2*Nvs*NN*(NL-1),N_RHS)
      REAL (KND) RADOUT(N_OP,N_LOS_OUT,Nvs)
      TARGET :: Ex_D
c     -----------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
c     ----------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA
c     -------------------------------------------------------------
c     ... Local parameters
c     --------------------      
      CHARACTER (LEN=7), PARAMETER :: R_NAME = 'FLS_ASI'
      INTEGER   I, J, K, L, Js, Jv, Jg, K2, Kg1, Kg2, Ks, Kv, NNv,
     $     LL1, LR1, N_OBS, K_lim
      REAL (KND)  Wdo, CS, Mud, S1, S2, Mu_efs, SA, CN, Xg, Cazm,
     $     S4(Nvs), Ad_ap(Nvs*NN), Ad_bp(Nvs*NN), Sjm(Nvs*NN), 
     $     S_Ls(NL-1,NLSA,NSZA), TE1(2*Nvs*NN), RHS(2*Nvs*NN),
     $     Int_dir(NL-1), Scal_prod(NSZA,NLSA), AGL(Nvs*NN),
     $     Term_alb(NSZA,NLSA), XML(NN), Qp(Nvs*NN), Qm(Nvs*NN),
     $     Cjm(Nvs*NN,NL-1,NSZA), MSL(Nvs,Nvs),
     $     Re1(NSZA,NLSA)
      REAL (KND), DIMENSION(NL-1,Nvs*NN,NSZA) :: Qm_Mu, Qp_Mu
      REAL (KND), POINTER :: Nu_j(:), EXP_NU(:), Enu(:), 
     $     a_j(:), b_j(:), Exp_D(:), P01(:,:,:), P02(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      N_OBS = N_OBS_ALT(1)      ! Preliminary ??
      NNv = Nvs*NN
      P01 => Ph_zA%Egs(:,:,:,1)  ;  P02 => Ph_zA%Egs(:,:,:,2)

c      WRITE(*,*) ' SHAPE(PVEC) ', SHAPE(PVEC)
c      WRITE(*,*) '     N_RHS = ', N_RHS
      RADOUT = 0.d0

      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         K_lim = 0
         XML(:) = XM(:) ; AGL(:) = AG(:)
      CASE( 'vect_rte' )
         K_lim = 3
         IF( Mh==1 ) K_lim=1
         XML(:) = 1.d0  
         DO K = 1,NNv 
            K2 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K2) 
         ENDDO
      END SELECT

      DO I = 1,NL-1 
c        ... Input arrays 
         Nu_j   => SDO%Nuj_lay(:,I)  ; EXP_NU => SDO%EXP_NU_NL(:,I)
         Exp_D  => Ex_D(:,I)
         S2 = DTR(I+1)
         DO Js = 1, NSZA  
            CS = MuS(Js) ; CN = DEXP( -S2/CS )
c          ... Auxiliary function after integration over optical 
c              depth within each atmospheric layer
            WHERE( REAL(Nu_j(:)) /= REAL(CS) )
               Cjm(:,I,Js) = Nu_j(:)*CS/(Nu_j(:) - CS)
     $               *(Exp_NU(:) - CN)*Exp_D(:)
            ELSEWHERE
               Cjm(:,I,Js) = Exp_NU(:)*S2*Exp_D(:)
            END WHERE
         ENDDO                  ! over solar zenith angle (Js)
      ENDDO                     ! over layers (I)

      Wdo = 1.d0 - Wup
      DO Js = 1,NSZA
c        ... In the scalar case Ks = Js-1
         Ks = (Js-1)*Nvs
         DO I = 1,NL-1
c           ... Single scattering source function in the forward RTE
c               without multiplier e^{\tau/mu0}. 
c               See Eq.(Qf-term) in STB and explanation below Expl*) 
c               w.r.t. matrices P01 and P02.
            DO Jg = 1,NN
               Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs ; Xg = XML(Jg)
               DO K2 = 1,Nvs
                  Qp(Kg1+K2-1) = SUM( (Wup*P01(I,Kg1:Kg2,Ks+K2)
     $                 + Wdo*P01(I+1,Kg1:Kg2,Ks+K2))*Xg*Finc(1,:) )

                  Qm(Kg1+K2-1) = SUM( (Wup*P02(I,Kg1:Kg2,Ks+K2)
     $                 + Wdo*P02(I+1,Kg1:Kg2,Ks+K2))*Xg*Finc(1,:) )
               ENDDO            ! over Stokes components
            ENDDO               ! over Gauss angles (Jg)
            Qp_Mu( I,:,Js ) = F14*Qp(:)/ALW(I)
            Qm_Mu( I,:,Js ) = F14*Qm(:)/ALW(I)

c           ... Single scattering source function of the forward RTE
c               in the direction of pseudo source
            DO Jv = JV_S(Js),JV_E(Js)
               Kv = (Jv-1)*Nvs
               MSL(:,:) = 
     $              ( Wup*Ph_zA%EsL(I,  Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) 
     $              + Wdo*Ph_zA%EsL(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) )
               DO K2 = 1,Nvs
                  S4(K2) = SUM( Finc(1,:)*MSL(:,K2) )
               ENDDO            ! over all Stokes components (K2)

               S_Ls(I,Jv,Js) = F14/ALW(I)*S4(Nc_der(1))

c               S_Ls(I,Jv,Js) = F14/ALW(I) 
c     $              *SUM( Finc(1,:)
c     $              *( Wup*Ph_zA%EsL(I,  Kv+1:Kv+Nvs,Ks+Nc_der(1),1) 
c     $                +Wdo*Ph_zA%EsL(I+1,Kv+1:Kv+Nvs,Ks+Nc_der(1),1) ) )

            ENDDO
cc            ... Old scalar version ...
c            Qp_Mu( I,:,Js ) = F14*(Wup*P01(I,:,Js)+Wdo*P01(I+1,:,Js))
c     $           /ALW(I)*XM(1:NN)
c            Qm_Mu( I,:,Js ) = F14*(Wup*P02(I,:,Js)+Wdo*P02(I+1,:,Js))
c     $           /ALW(I)*XM(1:NN)
c            DO Jv = JV_S(Js),JV_E(Js)
c               S_Ls(I,Jv,Js) = F14/ALW(I)
c     $              *( Wup*Ph_zA%EsL(I,  Jv,Js,1) 
c     $               + Wdo*Ph_zA%EsL(I+1,Jv,Js,1) )
c            ENDDO
c            ...........................
         ENDDO
      ENDDO

      Scal_prod = 0.d0 ; Re1 = 0.d0
      DO Js = 1,NSZA
         CS = MuS(Js)
         DO Jv = JV_S(Js),JV_E(Js)
            Mud = MuV(Jv)
            Mu_efs = CS*Mud/(CS + Mud )         
            DO I = 1, NL - 1
               Nu_j => SDO%Nuj_lay(:,I)
               Enu  => SDO%Exp_NU_NL(:,I)

               S1 = DEXP( -DTR(I+1)/CS )
               SA = DEXP( -DTR(I+1)/Mud )
               Sjm(:) = Nu_j(:)*CS*(1.d0 - Enu(:)*S1)/(Nu_j(:) + CS)
               a_j => ADO%Aj_ls(:,Jv,I)
               b_j => ADO%Bj_ls(:,Jv,I)
            
               WHERE( REAL(Nu_j(:)) /= REAL(Mud) )
                  Ad_ap(:) = Nu_j(:)*Mud*a_j(:)/(Nu_j(:)-Mud)*EXL(I,Jv)*
     $                 (Sjm(:) - Mu_efs*(1.d0 - S1*SA))
               ELSEWHERE
                  Ad_ap(:) = a_j(:)*Mu_efs*Mud*EXL(I,Jv)*
     $                 (Sjm(:) - Mu_efs*(1.d0 - S1*SA) )
               END WHERE
               Ad_bp(:) = Mud*Nu_j(:)*b_j(:)/(Nu_j(:) + Mud)*EXL(I,Jv)*
     $              (Mu_efs*(1.d0 - S1*SA) - SA*Cjm(:,I,Js))
            
               LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv

               TE1( 1:NNv ) = 
     $              PVEC(    LL1:LR1,Jv   )*Sjm(:) + Ad_ap(:)
               TE1( NNv+1:2*NNv ) = 
     $              PVEC( LR1+1:LR1+NNv,Jv )*Cjm(:,I,Js) + Ad_bp(:)

c              ... This is "solution" of pseudo-forward RTE for 
c                  the Fourier component Mh
               RHS(:) = MATMUL( SDO%G_NL(I,:,:),TE1(1:2*NNv) )

c              ... Change sign of U and V components to obtain solution
c                  of pseudo-forward RTE for upwilling radiation 
c                      (see Eq.(Int-up) in STB)
               IF( Nvs > 1 ) THEN
c                  FORALL( K = 3:NNv:Nvs ) RHS(K) = -RHS(K)

c                  FORALL( K = NNv+3:2*NNv:Nvs ) RHS(K) = -RHS(K)
                  FORALL( K = NNv+4:2*NNv:Nvs ) RHS(K) = -RHS(K)
               ENDIF
               S4(:) = 0.d0
               DO Ks = 0,K_lim
                  S1 = 0.d0
                  DO K = 1,NNv,Nvs
                     S1 = S1 + ( RHS(K+Ks+NNv)*Qp_Mu(I,K+Ks,Js)
     $                    + RHS(K+Ks)*Qm_Mu(I,K+Ks,Js) )*AGL(K+Ks)
                  ENDDO         ! over Gaussian nodes
                  S4(Ks+1) = S1
               ENDDO            ! over Stokes components
c               IF( Nvs > 1 ) S4(3) = -S4(3) 

               Re1(Js,Jv)  =  Re1(Js,Jv) + SUM(S4(:))*EXS(I,Js)

               Scal_prod(Js,Jv) = Scal_prod(Js,Jv) 
     $              + SUM(S4(:))*EXS(I,Js) 

c               Scal_prod(Js,Jv) = Scal_prod(Js,Jv)
c     $              + SUM( (RHS(1+NNv:2*NNv)*Qp_Mu(I,:,Js) 
c     $                    + RHS(1:NNv)*Qm_Mu(I,:,Js))*AGL(:) )*EXS(I,Js)

            ENDDO               ! over layers (I)
         ENDDO                  ! over line-of-sight angles (Jv)
      ENDDO                     ! over solar angles (Js)

       WRITE(*,*) '  -------- fls_asi.f ----------------  '
c       WRITE(*,*) '  Re1 =  ', Re1(1,1)
c       WRITE(*,*) '  -----------------------------------  '

      IF( DO_SATELLITE ) THEN   ! Reflected radiation
         IF( Mh == 1 ) THEN
            FORALL( Jv = 1:NLSA ) Term_alb(:,Jv) = 
     $            Albedo*EXS(NL,:)*( ADO%FLUX_D(Jv) + EXL(NL,Jv) )
         Else
            Term_alb = 0.d0
         ENDIF
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)
               Int_dir(N_OBS:NL-1) = MuS(Js)/( Mus(Js) + MuV(Jv) )
     $              *( EXS(N_OBS:NL-1,Js)*EXL(N_OBS:NL-1,Jv) 
     $              - EXS(N_OBS+1:NL,Js)*EXL(N_OBS+1:NL,Jv) )

               Scal_prod(Js,Jv) = Scal_prod(Js,Jv) 
     $              + MuS(Js)*Term_alb(Js,Jv) 
c                   ... Contribution to the scalar product of the 
c                       single scattering forward source function
c                       and direct adjoint intensity
     $              + SUM( S_Ls(N_OBS:NL-1,Jv,Js)*Int_dir(N_OBS:NL-1) )

c               WRITE(*,*) '  Re2 =  ', 
c     $              SUM( S_Ls(N_OBS:NL-1,Jv,Js)*Int_dir(N_OBS:NL-1) )
c               WRITE(*,*) '  -----------------------------------  '

            ENDDO               ! over line-of-sight (Jv)
         ENDDO                  ! over solar angles (Js)
      ELSE       ! Transmitted radiation
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)
               Int_dir(1:N_OBS-1) = MuS(JS)/( MuS(JS) - MuV(Jv) )
     $              *( EXS(2:N_OBS,JS)*EXL(2:N_OBS,Jv) 
     $              - EXS(1:N_OBS-1,JS)*EXL(1:N_OBS-1,Jv) )            
               Scal_prod(Js,Jv) = Scal_prod(Js,Jv) +
     $              SUM( S_Ls(1:N_OBS-1,Jv,JS)*Int_dir(1:N_OBS-1) )
     $              + Albedo*EXS(NL,JS)*ADO%Flux_d(Jv)*MuS(JS)
            ENDDO
         ENDDO
      ENDIF

      J = 0
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         DO Js = 1,NSZA
            DO K = JV_S(Js),JV_E(Js)
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  RADOUT(1,J,1) = Scal_prod(Js,K)*E_W
     $                 *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
               ENDDO
            ENDDO
         ENDDO
      CASE( 'vect_rte' )
         IF( Nc_der(1) < 3 ) THEN
            DO Js = 1,NSZA
               DO K = JV_S(Js),JV_E(Js)
                  DO L = 1,N_AZ_VAR  
                     J = J + 1 
                     Cazm = 0.5d0*DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                     RADOUT(1,J,Nc_der(1)) = Scal_prod(Js,K)*E_W*Cazm
                  ENDDO
               ENDDO
            ENDDO
         ELSE
            DO Js = 1,NSZA
               DO K = JV_S(Js),JV_E(Js)
                  DO L = 1,N_AZ_VAR  
                     J = J + 1 
                     Cazm = 0.5d0*DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )
                     RADOUT(1,J,Nc_der(1)) = Scal_prod(Js,K)*E_W*Cazm
                  ENDDO
               ENDDO
            ENDDO
         ENDIF

         WRITE(*,'(A,I2,2x,E15.7)') 
     $        ' Mh = ', Mh, RADOUT(1,:,Nc_der(1))
      END SELECT

      NULLIFY(Nu_j, EXP_NU, Enu, a_j, b_j, Exp_D, P01, P02)
      
      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN
      END 
c **************************************************************************
c Expl*)
c
c      Because P01 was designed to describe +Gauss --> Sol_ang, 
c     we multiply P01^T*Finc(1,:) to obtain single scattering 
c     Sol_ang --> +Gauss.
c     The same hold for matrix P02 to obtain single scattering 
c     Sol_ang --> -Gauss.
c     Multiplication P01^T and P02^T from left and right by
c     D4=diag{1,1,1,-1} is omitted because single scattered
c     solar radiation does not contain V Stokes component.
c     !!! It should be corrected in the case of !!!
c                 incoming polarized light
c
