C********************************************************************C
C
C      SCIATRAN - Version 4.3 - Last modification: 10 Dec 2020
C                 Matrix of the linear algebraic system 
C
C********************************************************************C
      SUBROUTINE FORM_MATRIX( FLAG, NRL, NL, Mh, NN, NNv, SDO, LAY_GR,
     $     Ex_AG, DTR_AG, UDB, GSL, MATTEB, INDX )
c     ... Called by "solv_dom.f"

      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_DSO, N_ESO 
      USE ST_VARS,     ONLY: Pie, AA_BRF, SURF_TYPE
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, UDB_TYPE, GSL_TYPE
      USE SLAB_2L,     ONLY: TES_2L_SLAB, Fr_01, Fr_20
      USE WATER,       ONLY: UWT
c     ==================================================================

      CHARACTER (LEN=1), INTENT(IN)  :: FLAG   
      INTEGER,    INTENT(IN)  :: NRL, NL, Mh, NN, NNv, LAY_GR(NL-1,2)
      REAL (KND), INTENT(IN)  :: DTR_AG(NrL)
      REAL (KND), INTENT(IN), TARGET :: Ex_AG(NNv,NRL-1)
      INTEGER,    INTENT(OUT) :: INDX(2*NNv*(NrL-1))
      REAL (KND), INTENT(OUT) :: MATTEB(9*NNv-2,2*NNv*(NrL-1))
c     -------------------------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables 
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
      TYPE (GSL_TYPE) :: GSL    ! Angles
c     ----------------------
c     ... Local variables
      CHARACTER (LEN=11), PARAMETER :: R_NAME = 'FORM_MATRIX'
      INTEGER  I, Ig, K, Ks, K1, J, Jco, Jro, NMIN, NMAX, Yn, 
     $     LL1, LL2, LR1, LR2, KL, KU, NR2, INFO, Yi, Yip
      REAL (KND) Plus_DT(NN), MTT(NRL-2,2*NNv,4*NNv), DUM(2*NNv,4*NNv),
     $     MAT(2*NNv*(NRL-1),2*NNv*(NRL-1)), RHS(2*NNv),
     $     RHS_BRF(2*NNv,NNv), HDU(NNv)
      INTEGER, POINTER ::  E_MIN(:)
      LOGICAL, POINTER ::  MASK(:)
      REAL (KND), POINTER :: G_AG(:,:,:), EXP_NU(:), Gp_p(:,:), 
     $     Gm_p(:,:)
      REAL (KND), TARGET ::  PHI(NNv,2*NNv)     
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      E_MIN => SDO%E_MIN  ;  G_AG => SDO%G_NL
      MASK  => SDO%MASK_NL

      Yn = LAY_GR(NrL-1,1)
      DO I = 1,NrL - 2
         Yi = LAY_GR(I,1) ; Yip = LAY_GR(I+1,1)
         Jco = SDO%NcC(Yi)  ;  Jro = NNv - 2*Jco
         EXP_NU => Ex_AG(:,I)
         Gp_p   => G_AG(Yi,:,1:NNv )
         DO K = 1,2*NNv
            MTT( I,K,1:Jro ) = Gp_p( K,1:Jro )*EXP_NU(1:Jro)
            DO J = Jro+1, Jro+2*Jco, 2
               MTT( I,K,J ) = 
     $              Gp_p(K,J)*Exp_Nu(J) - Gp_p(K,J+1)*Exp_Nu(J+1) 
               MTT( I,K,J+1 ) = 
     $              Gp_p(K,J)*Exp_Nu(J+1) + Gp_p(K,J+1)*Exp_Nu(J) 
            ENDDO
         ENDDO

         MTT(I,1:2*NNv,  NNv+1:2*NNv )=  G_AG(Yi, 1:2*NNv,NNv+1:2*NNv)
         MTT(I,1:2*NNv,2*NNv+1:3*NNv )= -G_AG(Yip,1:2*NNv,    1:NNv  )

c  !!!! Dlya formirovki pod-matrizu nizhnego sloya chislo Jro and Jco !!!
c          sleduet obnovit'  VR correction after JPL comparisons
         Jco = SDO%NcC(Yip)  ;  Jro = NNv - 2*Jco
c        -----------------------------------------
         EXP_NU => Ex_AG(:,I+1)
         Gm_p   => G_AG(Yip,:,NNv+1:2*NNv )
         DO K1 = 1,2*NNv
            MTT(I,K1,3*NNv+1:3*NNv+Jro )=-Gm_p(K1,1:Jro)*EXP_NU(1:Jro)
            DO J = Jro+1, Jro+2*Jco, 2
               MTT(I,K1,3*NNv+J) = 
     $              -(Gm_p(K1,J)*Exp_Nu(J) - Gm_p(K1,J+1)*Exp_Nu(J+1))
               MTT(I,K1,3*NNv+J+1) =                  
     $              -(Gm_p(K1,J)*Exp_Nu(J+1) + Gm_p(K1,J+1)*Exp_Nu(J))
            ENDDO
         ENDDO
      END DO
c             WRITE(*,*) '   -1  '
      DO I = 1, NrL - 2  ! Conservative 
         Yi = LAY_GR(I,1) ; Yip = LAY_GR(I+1,1)
         IF( MASK(Yi) ) THEN
            MTT(I,1:2*NNv:Nvs,NNv + E_MIN(Yi)) = 
     $           MTT(I,1:2*NNv:Nvs,NNv + E_MIN(Yi)) + DTR_AG(I+1)
         ENDIF
         IF( MASK(Yip) ) THEN
            MTT(I,1:2*NNv:Nvs,2*NNv + E_MIN(Yip)) = 
     $           MTT(I,1:2*NNv:Nvs,2*NNv + E_MIN(Yip)) - DTR_AG(I+2)
         ENDIF
      ENDDO
c           WRITE(*,*) '   -2  '        
      MAT(:,:) = 0.d0
      DO I = 1, NrL-2
         LL1 = (2*I-1)*NNv+1 ; LL2 = 2*(I-1)*NNv+1
         LR1 = (2*I+1)*NNv   ; LR2 = 2*(I+1)*NNv
         MAT( LL1:LR1, LL2:LR2 ) = MTT(I,1:2*NNv,1:4*NNv)
      ENDDO
        
      NR2 = 2*NNv*(NrL-1)
c      WRITE(*,*) '   -3  '
c     ==================================================
c                   Matrix elements for the 
c                  Upper boundary condition 
c     ==================================================
      EXP_NU => Ex_AG(:,1)  
      Jco = SDO%NcC(1)  ;  Jro = NNv - 2*Jco
      SELECT CASE( FLAG )
      CASE('A','a') 
         Plus_DT(:) = DTR_AG(2)
         Gp_p  => G_AG(1,1:NNv,:)
c        ... Overwrite Plus_DT and Gp_p in the case two-layers slab
         IF( TES_2L_SLAB ) CALL TWO_SLAB_1

      CASE('O','o') 
         SELECT CASE( TRIM(UDB%UBC) )
         CASE('Flat water')
c           ... Vector case
            Plus_DT(:) = UDB%Fre_Tv(:,1,1)*DTR_AG(2)
            DO I = 1,2*NNv
               DO Ig = 1, NN  ;  Ks = (Ig-1)*Nvs
                  DO K = 1, Nvs
                     PHI(Ks+K,I) = G_AG(1,Ks+K,I) -
     $                    SUM( UDB%Fre_Rv(Ig,K,1:Nvs)
     $                    *G_AG(1,NNv+Ks+1:NNv+Ks+Nvs,I) )
                  ENDDO  ! over Stokes components (K)
               ENDDO  ! over Gauss angles (Ig)
            ENDDO
            Gp_p => PHI(1:NNv,1:2*NNv)
            
         CASE('Rough water')
            DO K = 1,NN  ;  K1 = Nvs*(K-1) + 1
               Plus_DT(K) = DTR_AG(2)*(1.d0 - 2.d0*Pie
     $              *SUM( UDB%Rw_GG(K1,1:NNv:Nvs,Mh)*GSL%AG(:)) )
            ENDDO
            DO I = 1, NNv 
               DO K = 1,Nvs 
                  HDU(K:NNv:Nvs) = UDB%Rw_GG(I,K:NNv:Nvs,Mh)*GSL%AG(:)
               ENDDO  ! over Stokes components
c              ... Sign for U and V components of the Stokes vector
c                  (corrected by VR on 10.12.2020)
               IF( Nvs > 1 ) THEN
                  HDU(3:NNv:Nvs) = -HDU(3:NNv:Nvs) 
                  HDU(4:NNv:Nvs) = -HDU(4:NNv:Nvs) 
               ENDIF
               DO K = 1,2*NNv
                  PHI(I,K) = G_AG(1,I,K) - SUM( HDU(1:NNv)
     $                 *G_AG(1,NNv+1:2*NNv,K) )*2.d0*Pie
               ENDDO
            ENDDO   
            Gp_p => PHI(1:NNv,1:2*NNv)

         END SELECT
      END SELECT
c      WRITE(*,*) '   -4  '
c     ... Matrix elements corresponding to the integration constants Aj
c         -------------------------------------------------------------
      MAT( 1:NNv,1:NNv ) = Gp_p(1:NNv,1:NNv) 
      DO J = 1, Jro
         DUM( 1:NNv,J ) = Gp_p(1:NNv,NNv+J)*EXP_NU(J) 
      END DO
c     ... Complex separation constants
      DO J = Jro+1, Jro+2*Jco, 2
         DUM( 1:NNv,J ) = 
     $        Gp_p(:,NNv+J)*EXP_NU(J)   - Gp_p(:,NNv+J+1)*EXP_NU(J+1) 
         DUM( 1:NNv,J+1 ) = 
     $        Gp_p(:,NNv+J)*EXP_NU(J+1) + Gp_p(:,NNv+J+1)*EXP_NU(J) 
      ENDDO        
c     ... Matrix elements corresponding to the integration constants Bj
c         -------------------------------------------------------------
      MAT( 1:NNv,NNv+1:2*NNv ) = DUM( 1:NNv,1:NNv )

c      WRITE(*,*) '   -5  '
c      WRITE(*,*) '   form_matrix: MAT after LBC ', SUM(MAT(:,:))

      IF( MASK(1) ) THEN  
         MAT( 1:NNv:Nvs,E_MIN(1) ) = 
     $        MAT( 1:NNv:Nvs,E_MIN(1) ) + Plus_DT(:) 
      ENDIF  ! conservative case

c     ==================================================
c                 Matrix elements for the  
c                Lower boundary condition
c     ==================================================
      IF( MASK(Yn) ) THEN       ! Conservative
         G_AG(Yn,1:2*NNv:Nvs, NNv + E_MIN(Yn)) = 
     $        G_AG(Yn,1:2*NNv:Nvs,NNv + E_MIN(Yn)) + DTR_AG(NrL)
      ENDIF
        
      Exp_Nu => Ex_AG(:,NrL-1)
c     ... Corresponds to the upward radiation and contains all separation const
      Gp_p   => G_AG(Yn,NNv+1:2*NNv,: )

      Jco = SDO%NcC(Yn)  ;  Jro = NNv - 2*Jco
c     ... Calculate matrix elements corresponding to the integration constants
c         Aj at the bottom of the medium
      FORALL (J = 1:Jro ) DUM(1:NNv,J) = Gp_p(:,J)*Exp_Nu(J)
c         ... Complex separation constants
      DO J = Jro+1, Jro+2*Jco, 2
         DUM(1:NNv,J)   = Gp_p(:,J)*Exp_Nu(J)  - Gp_p(:,J+1)*Exp_Nu(J+1) 
         DUM(1:NNv,J+1) = Gp_p(:,J)*Exp_Nu(J+1) + Gp_p(:,J+1)*Exp_Nu(J)
      ENDDO        

      LL1 = (2*NrL-3)*NNv + 1 ; LR1 = 2*(NrL-1)*NNv
c     ...Matrix elements corresponding to the integration constants Aj
      MAT( LL1:LR1,2*(NrL-2)*NNv+1:(2*NrL-3)*NNv ) = DUM(1:NNv,1:NNv)
c     ...Matrix elements corresponding to the integration constants Bj
      MAT( LL1:LR1,LL1:LR1  ) = Gp_p(:,NNv+1:2*NNv) 

c     ... Change sign of U and V component in the matrix of LAS
      IF( Nvs>1 ) THEN
         LL2 = 2*(NrL-2)*NNv+1
         DO K = 1,NN
            DO K1 = 4*K-2, 4*K-1
               MAT( K1+LL1,LL2:LR1 ) = -MAT( K1+LL1,LL2:LR1 )
            ENDDO
         ENDDO
      ENDIF

c      WRITE(*,*) '   -6  '
c      WRITE(*,*) '   form_matrix: MAT after LBC ', SUM(MAT(:,:))

c     ==========================================================        
c     Lower boundary condition in the case of Lambert reflection
c                        or BRDF
c     ==========================================================
c      WRITE(*,*) '  Before BRDF_FLAG  '

      IF( Mh==1 .OR. UDB%BRDF_FLAG .OR. UWT ) THEN
c        ... Corresponds to the downward radiance and contains all 
c            separation const
         Gp_p   => G_AG(Yn,1:NNv,: )
         FORALL (J = 1:Jro ) DUM(1:NNv,J ) = Gp_p(:,J)*Exp_Nu(J)
c        ... If there are complex separation constants
         DO J = Jro+1, Jro+2*Jco, 2
            DUM(1:NNv,J) = 
     $           Gp_p(:,J)*Exp_Nu(J) - Gp_p(:,J+1)*Exp_Nu(J+1) 
            DUM(1:NNv,J+1) = 
     $           Gp_p(:,J)*Exp_Nu(J+1) + Gp_p(:,J+1)*Exp_Nu(J) 
         ENDDO        
         DUM( 1:NNv,NNv+1:2*NNv ) = G_AG(Yn,1:NNv,NNv+1:2*NNv )
      ENDIF

c      WRITE(*,*) '   -7  '
c     ... Array DUM(1:NNv,1:2*NNv) represents the matrix-vector form of
c         the downward radiance for NN Gaussian angles:
c                           |A_1|
c                           | . |
c                           | . |
c               Idown = DUM*|Ann|  ;
c                           |B_1|      
c                           | . |
c                           | . |
c                           |Bnn|
c      WRITE(*,*) '   TRIM(UDB%LBC) ====>  ', TRIM(UDB%LBC) 
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
         IF( Mh==1 ) THEN
            IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,*) 
     $           ' ====> Lambert reflection "form_natrix"',UDB%ALBEDO
            DO J = 1,2*NNv
               RHS(J) = SUM( DUM(1:NNv:Nvs,J)*GSL%AXM(1:NN) )
            ENDDO
            RHS(:) = 2.d0*UDB%ALBEDO*RHS(:)
            IF( AA_BRF ) RHS(:) = 0.d0
            DO J = LL1,LR1,Nvs 
               MAT( J,2*(NrL-2)*NNv+1:LR1 ) = 
     $              MAT( J,2*(NrL-2)*NNv+1:LR1 ) - RHS(:)
            ENDDO
         ENDIF
      CASE('Bidirect')
         DO J = 1,2*NNv  
            DO K = 1,Nvs        
               HDU(K:NNv:Nvs) = DUM(K:NNv:Nvs,J)*GSL%AXM(1:NN)
            ENDDO   ! over Stokes components (K)
            DO K = 1,NNv 
               RHS_BRF(J,K) = SUM( UDB%BRF_GG(K,:,Mh)*HDU(1:NNv) )
            ENDDO
         ENDDO  
c        ... Surface reflectance set to zero
         IF( AA_BRF ) RHS_BRF = 0.d0
c        ... Vector case with vector BRDF
         K = 1
         DO J = LL1,LR1 
            MAT( J,2*(NrL-2)*NNv+1:LR1 ) = 
     $           MAT( J,2*(NrL-2)*NNv+1:LR1 ) - 2.d0*RHS_BRF(:,K)
            K = K + 1
         ENDDO

      CASE('Flat water')
         IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,*) 
     $        ' ====> Specular reflection mode "form_natrix"'
c        ... Fresnel reflectance of diffuse radiation
         DO J = 1,2*NNv 
            DO Ig = 1, NN  ;  Ks = (Ig-1)*Nvs
               DO K = 1, Nvs
                  HDU(Ks+K)=SUM( UDB%Fre_Rv(Ig,K,:)*DUM(Ks+1:Ks+Nvs,J) )
               ENDDO
            ENDDO
            RHS_BRF(J,:) = HDU(:) !*2.
         ENDDO  ! over streams (J)
         
         IF( TRIM(SURF_TYPE)=='melt_pond') THEN
c            RHS_BRF = 0.d0
            DO J = 1,2*NNv  
               DO K = 1,Nvs        
                  HDU(K:NNv:Nvs) = DUM(K:NNv:Nvs,J)*GSL%AXM(1:NN)
               ENDDO  ! over Stokes components (K)
               DO K = 1,NNv 
                  RHS_BRF(J,K) = RHS_BRF(J,K) + 2.d0*
     $                 SUM( UDB%BRF_GG(K,:,Mh)*HDU(1:NNv) )
               ENDDO
            ENDDO
         ENDIF
c         WRITE(*,*) '  Mh = ', Mh, SUM(UDB%BRF_GG(NNv,:,Mh)*HDU(1:NNv))
c         PAUSE 
         K = 1
         DO J = LL1,LR1 
            MAT( J,2*(NrL-2)*NNv+1:LR1 ) = 
     $           MAT( J,2*(NrL-2)*NNv+1:LR1 ) - RHS_BRF(:,K) !2.d0*
            K = K + 1
         ENDDO

      CASE('Rough water')
c         write(*,*) '   form_matrix Mh eto ', Mh
c         write(*,*) '  SIZE( UDB%Ra_GG ) ', SIZE( UDB%Ra_GG ) 
         DO J = 1,2*NNv
            DO K = 1,Nvs 
               HDU(K:NNv:Nvs) = DUM(K:NNv:Nvs,J)*GSL%AG(1:NN)
            ENDDO  ! over Stokes components
            DO K = 1,NNv 
               RHS_BRF(J,K) = SUM( UDB%Ra_GG(K,:,Mh)*HDU(1:NNv) )
            ENDDO
         ENDDO  ;  K = 1
c        ... Vector case with vector BRDF
         DO J = LL1,LR1 
            MAT( J,2*(NrL-2)*NNv+1:LR1 ) = 
     $           MAT( J,2*(NrL-2)*NNv+1:LR1 ) - 2.d0*RHS_BRF(:,K)*Pie
            K = K + 1
         ENDDO
      END SELECT
c      WRITE(*,*) '   -8  '
      IF( TES_2L_SLAB ) CALL TWO_SLAB

      MATTEB = 0.d0 ; KU = 3*NNv-1 ; KL = 3*NNv-1
      DO I = 1,NR2
         DO J = 1,NR2
            NMIN = MAX(J-KU,1)
            NMAX = MIN(J+KL,NR2)
            IF ((I.GE.NMIN).AND.(I.LE.NMAX)) THEN
               MATTEB(KL+KU+I-J+1,J) = MAT(I,J)
            END IF
         END DO
      END DO

c      WRITE(*,*) '   -9  '
c      WRITE(*,*) '   SHAPE(MATTEB): ', SHAPE( MATTEB )
c      WRITE(*,*) '   form_matrix: before DGBTRF ', SUM(MATTEB(:,:))
c      WRITE(*,*) '   form_matrix: before DGBTRF ', SUM(INDX)

c     ... LU-decomposition of the linear system matrix
c     ================================================
      INFO = 0  
      IF(LOG_LEVEL(1) >= N_DSO) WRITE(LUNIT,*) '   Strat DGBTRF'
      CALL DGBTRF(NR2, NR2, 3*NNv-1, 3*NNv-1, MATTEB, 9*NNv-2, INDX, 
     $     INFO)
      IF(INFO /= 0) 
     $     WRITE(LUNIT,*) ' DGBTRF error message: INFO =', INFO

c      WRITE(*,*) '   -10  '
c      WRITE(*,*) '   form_matrix: after DGBTRF ', SUM(MATTEB(:,:))
c      WRITE(*,*) '   form_matrix: after DGBTRF ', SUM(INDX)

      NULLIFY(G_AG, EXP_NU, Gp_p, Gm_p)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c ********************************************************************

      SUBROUTINE TWO_SLAB_1
      WRITE(*,*) '   TEST_2L_SLAB - top of atmosphere'
      Plus_DT(:) = (1.d0 - Fr_01(:))*DTR_AG(2)
      K = 1
      DO I = 1, NN              ! Scalar case only
         PHI(I,:) = G_AG(1,I,:) - Fr_01(K)*G_AG(1,NN+I,:)
         K = K + 1
      ENDDO
      Gp_p => PHI(1:NNv,1:2*NNv)
      END SUBROUTINE TWO_SLAB_1
c **********************************************************************

      SUBROUTINE TWO_SLAB
      SELECT CASE( FLAG )
      CASE('A','a') 
      CASE('O','o') 
         WRITE(*,*) 
     $       '  ----> Fresnel reflection from the surface of second med'
         DO J = 1,2*NNv
            DO K = 1,Nvs        ! for all Stokes components
               HDU(K:NNv:Nvs) = DUM(K:NNv:Nvs,J)*Fr_20(1:NN)
            ENDDO
            DO K = 1,NNv 
               RHS_BRF(J,K) = HDU(K)
            ENDDO
         ENDDO
         K = 1
         DO J = LL1,LR1 
            MAT( J,2*(NrL-2)*NNv+1:LR1 ) = 
     $           MAT( J,2*(NrL-2)*NNv+1:LR1 ) - RHS_BRF(:,K) !2.d0*
            K = K + 1
         ENDDO
      END SELECT
      END SUBROUTINE TWO_SLAB

      END SUBROUTINE FORM_MATRIX
