C *************************************************************************
C
C       SCIATRAN - Version 3.9 - Last modification: 31 Dec 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c                  Recalculation of the array AFI (ADO_)
c    Calculation of the reflected and transmitted pseudo-forward intensity 
c                       ----------------------------------------
c    at the Gauss angles for all levels and all viewing angles
c       employing Source Function Integration technique
c                (Used in the plane-parallel mode)
c *************************************************************************
      SUBROUTINE GAU_Adj_UP_DOWN(NL, NN, NLSA, Mh, ALBEDO, XM, AG1, MuV,
     $     EXL, IOP, Egg, SDO, S_GL, ADO, UDB, Ref_surf)
c     ... Called from "st_adjoint_a.f"
c                     "st_adjoint_o.f"

      USE BRDF,        ONLY: Npg
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: Work_Dir
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: N_Adj, Vec_Sca, GeoD => GeoD_OUT, NS_wf, 
     $                       Ainc, Nc_der, VST, SDVI, N_RHS
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: SDO_TYPE, ADO_TYPE, IOP_TYPE, UDB_TYPE
c     ------------------------------------------------------------------

      INTEGER,           INTENT(IN) :: NL, NN, NLSA, Mh
      REAL (KND),        INTENT(IN) :: ALBEDO, XM(NN), MuV(NLSA), 
     $     EXL(NL,NLSA), Egg(NL,Nvs*NN,Nvs*NN,2), AG1(NN),
     $     S_GL(NL,Nvs*NN,Nvs*NLSA,2), Ref_surf(N_RHS,Nvs*(NN+Npg))
c     -----------------------------------------------
      TYPE (IOP_TYPE), TARGET :: IOP    ! Integral optical parameters
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     -------------------------------------------------------------
c     ... Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'GAU_Adj_UP_DOWN'
      INTEGER  I, IA, Iexl, INI, K, K1, K2, Kc, Kg, Kg1, Kg2, 
     $     Kv, Kv1, Kv2, Jv,  
     $     Jg, Jco, Jro, Jw, Jw1, LL1, LR1, NNv, Max_Jco
      INTEGER, POINTER :: Id_a(:), Id_b(:)
      REAL (KND), DIMENSION(NN*Nvs) :: Tt1, Tt2, Tr1, Tr2, Nup, Num,
     $     Apr, Bpr, Apt, Bpt
      REAL (KND), DIMENSION(N_Adj,NN*Nvs) :: Tru, Trg
      REAL (KND)  Wdo, DTA, Cg, Ecg, Cv, E_xl, DUM, C_dbl, C_aim, 
     $     S_dbl, S_aim, SOU(NLSA), Qp(Nvs), Qm(Nvs), V_XM(NN), 
     $     Tra_gau(NL,N_Adj,Nvs*NN), Ref_gau(NL,N_Adj,Nvs*NN), 
     $     Itl(NL-1,N_Adj,Nvs*NN), Irl(NL-1,N_Adj,Nvs*NN), 
     $     Exp_XM(NL-1,NN)
      REAL (KND), DIMENSION(Nvs*NN,Nvs*NN) ::  Faj, Fbj
      REAL (KND), TARGET :: C_gau_ls(NLSA), S_gau_ls(NLSA), Sjm(NN*Nvs),
     $     Cjm(NN*Nvs)
      REAL (KND), ALLOCATABLE :: HDU(:)
      REAL (KND), POINTER :: Pcg(:), Psg(:), Pcj(:), Psj(:), Nu_j(:), 
     $     ALW(:), DTR(:), Exp_Nu(:), EXP_D(:), EXP_U(:), Gp_p(:,:), 
     $     PAj(:), PBj(:),  Gm_p(:,:), A_j(:), B_j(:),
     $     Ajs(:,:,:), Bjs(:,:,:)
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2, Cnu
      COMPLEX (kind(0.0d0)), ALLOCATABLE :: Cj_d(:), Cj_u(:), 
     $     Sj_d(:), Sj_u(:)      
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IA = 1 
      NNv = Nvs*NN ; Wdo = 1.d0 - Wup
      ALW  => IOP%ALW  ;  DTR  => IOP%DTR
      Id_a => IOP%Id_a ;  Id_b => IOP%Id_b
      Ajs  => ADO%Ajs  ;  Bjs  => ADO%Bjs

      Max_Jco = MAXVAL( SDO%NcC(:) )
      IF( Max_Jco > 0 ) ALLOCATE( Cj_d(Max_Jco), Cj_u(Max_Jco), 
     $     Sj_d(Max_Jco), Sj_u(Max_Jco) )

      V_XM(:) = 1.d0
      IF( Vec_Sca=='vect_rte' ) V_XM(:) = XM(:)
      
      DO I = 1,NL-1             ! for all layers 
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco  
c         WRITE(*,*) '  I and Jco ', I, SDO%NcC(I) 
c        ... Routine "aux_in_layer" performs calculation of: 
c              Faj, Fbj, LL1, LR1, DTA, Nu_j, Exp_Nu, EXP_D, EXP_U
         CALL AUX_IN_LAYER
         SOU(:) = DEXP( -DTR(I+1)/MuV(:) )
         DO Jg = 1,NN           ! for all Gauss angles
            Kg = (Jg-1)*Nvs
            Kg1 = (Jg-1)*Nvs+1 ;  Kg2 = Jg*Nvs

            Cg = XM(Jg) ; Ecg = DEXP( -DTA/Cg ) ! DTA = DTR(I+1)
            Nup(1:Jro) = Nu_j(1:Jro)/(Nu_j(1:Jro) + Cg) 

            Sjm(1:Jro) = (1.d0 - Exp_Nu(1:Jro)*Ecg)*Cg*Nup(1:Jro)
            WHERE( REAL(Nu_j(1:Jro)) /= REAL(Cg) )
               Num(1:Jro) = Nu_j(1:Jro)/(Nu_j(1:Jro) - Cg)
               Cjm(1:Jro) = Nu_j(1:Jro)*Cg/(Nu_j(1:Jro)-Cg)
     $              *(Exp_Nu(1:Jro) - Ecg)
            ELSEWHERE
               Cjm(1:Jro) = Exp_Nu(1:Jro)*DTA
            END WHERE            
            Kc = 1
c           ... Use complex separation constants:
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nu_j(K1),Nu_j(K1+1) )
               Cnu = DCMP( Exp_Nu(K1),Exp_Nu(K1+1) )
               C2 = C1*Cg*(Cnu - Ecg)/(C1 - Cg) 
c                   Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)
               Cj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Cj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))
c              ----------------------------------------------
               C2 = C1*Cg*(1.d0 - Cnu*Ecg)/(C1 + Cg)
c                   Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
               Sj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Sj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))
               Kc = Kc + 1
            ENDDO

            S_gau_ls(:) = MuV(:)*Cg/(MuV(:) + Cg)*(1.d0 - SOU(:)*Ecg)
            WHERE( Real(MuV(:)) /= Real(Cg) )
               C_gau_ls(:) = MuV(:)*Cg/(MuV(:) - Cg)*(SOU(:) - Ecg)
            ELSEWHERE
               C_gau_ls(:) = Ecg*DTA
            END WHERE

            DO Jv = 1,NLSA      ! for all line-of-sight
               IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN 
                  Pcg => C_gau_ls(:) ; Psg => S_gau_ls(:)
                  Pcj => Cjm(:)      ; Psj => Sjm(:)  ; Iexl = 0
               ELSE
                  Psg => C_gau_ls(:) ; Pcg => S_gau_ls(:)
                  Psj => Cjm(:)      ; Pcj => Sjm(:)  ; Iexl = 1
               ENDIF
               E_xl = EXL(I+Iexl,Jv)
               Cv = MuV(Jv)

               Tt1 = 0.d0 ; Tr2 = 0.d0
               DO Jw = 1, Ns_wf
                  Jw1 = (Jv - 1)*NS_wf + Jw
c              ... Dimension A_j and B_j is NNv
                  B_j => ADO%Bj_ls(:,Jw1,I) ; A_j => ADO%Aj_ls(:,Jw1,I) 
                  PAj => ADO%LVEC(  LL1:LR1,Jw1 )
                  PBj => ADO%LVEC(LR1+1:LR1+NNv,Jw1)
c              ... Auxiliary functions for reflection and transmission:
                  Apr(:) =  ( -Ajs(I,:,Jw1)*Ecg + Ajs(Id_a(I)-1,:,Jw1)
     $                 + A_j(:)*Psg(Jv)*E_xl )*Cg
                  Bpr(:) = ( Bjs(I,:,Jw1) - Bjs(Id_b(I)+1,:,Jw1)*Ecg
     $                 - B_j(:)*Psg(Jv)*E_xl )*Cg

                  Apt(:) = ( Ajs(I,:,Jw1) - Ajs(Id_a(I)-1,:,Jw1)*Ecg
     $                 - A_j(:)*Pcg(Jv)*E_xl )*Cg
                  Bpt(:) = ( -Bjs(I,:,Jw1)*Ecg + Bjs(Id_b(I)+1,:,Jw1)
     $                 + B_j(:)*Pcg(Jv)*E_xl )*Cg
                  
                  WHERE( REAL(Nu_j(1:Jro)) /= REAL(Cg) )
                     Tt1(1:Jro) = Apt(1:Jro)*Num(1:Jro)
                     Tr2(1:Jro) = Bpr(1:Jro)*Num(1:Jro)
                  ELSEWHERE
                     WHERE( REAL(Nu_j(1:Jro)) == REAL(Cv) )
                        Tt1(1:Jro) = A_j(1:Jro)*E_xl*DTA*DTA*0.5d0*Ecg
                     ELSEWHERE
                        Tt1(1:Jro) = A_j(1:Jro)*Cg*Cv/(Cv - Cg)
     $                       *(Pcg(Jv) - DTA*Ecg)*E_xl
                     END WHERE
                     Tr2(1:Jro) = B_j(1:Jro)*Cg*Cv/(Cv + Cg)*E_xl
     $                    *( Psg(Jv) - DTA*Ecg*SOU(Jv) )
                  END WHERE            

c                 reflected component Aj 
                  Tr1(1:Jro) =  Apr(1:Jro)*Nup(1:Jro)
     $                 + PAj(1:Jro)*Psj(1:Jro)*EXP_U(1:Jro)
c                 reflected component Bj
                  Tr2(1:Jro) = Tr2(1:Jro)
     $                 + PBj(1:Jro)*Pcj(1:Jro)*EXP_D(1:Jro) 
c     c                   transmitted component Aj
                  Tt1(1:Jro) = Tt1(1:Jro)
     $                 + PAj(1:Jro)*Pcj(1:Jro)*EXP_U(1:Jro) 
c     c                   transmitted component Bj
                  Tt2(1:Jro) = Bpt(1:Jro)*Nup(1:Jro)
     $                 + PBj(1:Jro)*Psj(1:Jro)*Exp_D(1:Jro) 
c                 ... Calculate Tr1, Tr2, Tt1, and Tt2 for 
c                     Jro+1:Jro+2*Jco
                  IF( Jco > 0 ) CALL COMPLEX_Tr_Tt

                  Kv = (Jv-1)*Nvs ; Kv1 = Kv+1 ;  Kv2 = Jv*Nvs
                  DO K2 = 1,Nvs
                     Qp(K2) = SUM( (Wup*S_GL(I,Kg1:Kg2,Kv+K2,1)
     $                    + Wdo*S_GL(I+1,Kg1:Kg2,Kv+K2,1))*Ainc(Jw,:) )
                     Qm(K2) = SUM( (Wup*S_GL(I,Kg1:Kg2,Kv+K2,2)
     $                    + Wdo*S_GL(I+1,Kg1:Kg2,Kv+K2,2))*Ainc(Jw,:) )
                  ENDDO
                  Qp(:) = Qp(:)*E_xl/V_XM(Jg)/ALW(I)  /2.d0/Cv  
                  Qm(:) = Qm(:)*E_xl/V_XM(Jg)/ALW(I)  /2.d0/Cv  
c                 ... Multiply the phase matrices by D4 from left
                  IF( Nvs > 1 ) THEN
                     Qp(4) = -Qp(4) ; Qm(4) = -Qm(4)
                  ENDIF

                  K2 = 1
                  DO K1 = (Jg-1)*Nvs+1, Jg*Nvs
                     Itl(I,Jw1,K1) = Pcg(Jv)*Qp(K2)
     $                    + SUM( Tt1(:)*Faj(K1,:) + Tt2(:)*Fbj(K1,:) )
                     Irl(I,Jw1,K1) = Psg(Jv)*Qm(K2)
     $                    + SUM( Tr1(:)*Fbj(K1,:) + Tr2(:)*Faj(K1,:) )
                     K2 = K2 + 1
                  ENDDO  ! over Stokes components (K1)
               ENDDO  ! over derivatives of Stokes comp. (Jw, Jw1)
            ENDDO  ! over Line-of-sight angles (Jv)
         ENDDO  ! over Gauss angles (Jg)
      ENDDO  ! over layers (I)

      FORALL( I=1:NL-1 ) Exp_XM(I,:) = DEXP(-DTR(I+1)/XM(:))

      Trg(:,:) = 0.0d0
      Tra_gau(1,:,:) = Trg(:,:)
      DO I = 1,NL-1
         DO Jg = 1,NN ; DUM = Exp_XM(I,Jg) 
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Trg(:,K1) = Trg(:,K1)*DUM + Itl(I,:,K1)
            ENDDO
         ENDDO
         Tra_gau(I+1,:,:) = Trg(:,:) 
      ENDDO

      Tru = 0.d0  
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
         DO Jw = 1,NS_wf 
            IF( Nc_der(Jw) /= 1 ) CYCLE
            DO Jv = 1, NLSA
               Jw1 = (Jv - 1)*Ns_wf + Jw
               Tru(Jw1,1:NNv:Nvs) = 2.d0*Albedo*ADO%FLUX_D(Jw1) !*VST
               IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0.AND.(.NOT.DO_SSRCA) )
     $              Tru(Jw1,1:NNv:Nvs) = Tru(Jw1,1:NNv:Nvs) 
     $              + 2.d0*Albedo*EXL(NL,Jv) !*VST
            ENDDO  ! over LoS (Jv)
         ENDDO  ! over  Stokes component derivatives (Jw)

      CASE('Bidirect')         
c     ... Ref_surf is calculated in "st_top_bot.f" and contains 
c         forward and pseudo-forward Stokes vector components
c         reflected from a surface at all Gauss and viewing angles
         BIDIRECT_INIT: Block
           DO Jv = 1,N_Adj
              Tru(Jv,:) = 2.d0*Ref_surf(Jv+SDVI,1:NNv) 
           ENDDO                   
c           write(*,'(a,*(2x,e12.4))') '  Tru 1 ', Tru(2,2:4)
c           write(*,*) ' GeoD%View(IA)%S ', GeoD%View(IA)%S

         IF( GeoD%View(IA)%S=='down'.OR.DO_SSRCA ) EXIT BIDIRECT_INIT
           ALLOCATE( HDU(NNv) )
           DO Jv = 1,NLSA
              IF( GeoD%LVG(IA,Jv)%C_VA(1) <= 0.d0) CYCLE !downward radiation
              INI = Nvs*(SDVI+Jv-1) 
              DO Jw = 1,NS_wf ; Jw1 = NS_wf*(Jv-1) + Jw
                 K2 = INI + Nc_der(Jw)
                 HDU(:) = UDB%BRF_GS(1:NNv,K2,Mh)
                 Tru(Jw1,:) = Tru(Jw1,:) 
     $              + 2.d0*EXL(NL,Jv)*HDU(:) !*VST !???
              ENDDO  ! over number of Stokes WF (Jw,Jw1)
           ENDDO  ! over line-of-sight (Jv)
         
         END Block BIDIRECT_INIT

      END SELECT ! 'Lambert' or 'Bidirect'

c     ... U and V components of the Stokes vector
c         sleduet menyat' znak tol'ko u Tru 
      IF( Vec_Sca == 'vect_rte' ) THEN
         DO Jg = 1,NN 
            DO K1 = (Jg-1)*Nvs+3, Jg*Nvs
               Tru(:,K1)   = -Tru(:,K1)
            ENDDO 
         ENDDO
      ENDIF

      Ref_gau(NL,:,:) = Tru(:,:)
      DO I = NL - 1,1,-1
         DO Jg = 1,NN ; DUM = Exp_XM(I,Jg) 
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Tru(:,K1) = Tru(:,K1)*DUM + Irl(I,:,K1)
            ENDDO
         ENDDO
         Ref_gau(I,:,:) = Tru(:,:) 
      ENDDO

      FORALL( K = 1:N_Adj, I = 1:NL ) 
        ADO%Afi(I,NNv+1:2*NNv,K) = Ref_gau(I,K,1:NNv)
        ADO%Afi(I,1:NNv,K)       = Tra_gau(I,K,1:NNv)
      END FORALL

c       Open( 12, file = './DATA_OUT/afi-1_s.tes' )
c       DO I = 1, NL
c          WRITE( 12,'(*(1x,e15.7))')
c     $         ZZZ(I), (REAL(ADO%AFI(I,K,1)), K = 1,NNv)
c       ENDDO
c       Close(12)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
      SUBROUTINE COMPLEX_Tr_Tt
      Kc = 1
      DO K1 = Jro+1, Jro+2*Jco, 2
         Cnu = DCMP( Nu_j(K1),Nu_j(K1+1) )
         C1 = DCMP(Apr(K1),Apr(K1+1))*Cnu/(Cnu+Cg)
         S_dbl = DBLE(Sj_u(Kc)) ; S_aim = AIMAG(Sj_u(Kc))
         Tr1(K1) = PAj(K1)*S_dbl + PAj(K1+1)*S_aim   + 2.d0*DBLE(C1)
         Tr1(K1+1)= PAj(K1+1)*S_dbl - PAj(K1)*S_aim - 2.d0*AIMAG(C1)
         
         C1 = DCMP(Apt(K1),Apt(K1+1))*Cnu/(Cnu-Cg)
         C_dbl = DBLE(Cj_u(Kc)) ; C_aim = AIMAG(Cj_u(Kc))
         Tt1(K1)  = PAj(K1) *C_dbl + PAj(K1+1)*C_aim + 2.d0*DBLE(C1)
         Tt1(K1+1)= PAj(K1+1)*C_dbl - PAj(K1)*C_aim - 2.d0*AIMAG(C1)
         
         C2 = DCMP(Bpr(K1),Bpr(K1+1))*Cnu/(Cnu-Cg)
         C_dbl = DBLE(Cj_d(Kc)) ; C_aim = AIMAG(Cj_d(Kc))
         Tr2(K1)  = PBj(K1) *C_dbl + PBj(K1+1)*C_aim + 2.d0*DBLE(C2)
         Tr2(K1+1)= PBj(K1+1)*C_dbl - PBj(K1)*C_aim - 2.d0*AIMAG(C2)

         C2 = DCMP(Bpt(K1),Bpt(K1+1))*Cnu/(Cnu+Cg)
         S_dbl = DBLE(Sj_d(Kc)) ; S_aim = AIMAG(Sj_d(Kc))
         Tt2(K1)  = PBj(K1)*S_dbl + PBj(K1+1)*S_aim  + 2.d0*DBLE(C2)
         Tt2(K1+1)= PBj(K1+1)*S_dbl - PBj(K1)*S_aim - 2.d0*AIMAG(C2)
         Kc = Kc + 1
      ENDDO  ! over complex separation constants (Kc)
      END SUBROUTINE COMPLEX_Tr_Tt


c **********************************************************************
      SUBROUTINE AUX_IN_LAYER
      REAL (KND), 
     $   DIMENSION (NNv,NNv) :: Ph_g1, Ph_g2, TTT

      SELECT CASE( Vec_Sca )
      CASE( 'vect_rte' )
c     ... It should be reproduced at first in the vector case 
c         phase matrix corresponding to the scattering +Gauss <-- +Gauss
         TTT(:,:)= Wup*Egg(I,:,:,1) + Wdo*Egg(I+1,:,:,1)
         TTT(:,:) = TTT(:,:) + (Wup*Egg(I,:,:,2) + Wdo*Egg(I+1,:,:,2))
         
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 
         DO Jv = 1,NN
            Ph_g1(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
         FORALL( K = 1:NN ) Ph_g1(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g1(:,Nvs*(K-1)+1:Nvs*K)*AG1(K)

c     ... It should be reproduced at first in the vector case 
c         phase matrix corresponding to the scattering +Gauss <-- -Gauss
         TTT(:,:) =  Wup*Egg(I,:,:,2) + Wdo*Egg(I+1,:,:,2)
         TTT(:,:) = (Wup*Egg(I,:,:,1) + Wdo*Egg(I+1,:,:,1)) - TTT(:,:)
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 
         DO Jv = 1,NN
            Ph_g2(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
         FORALL( K = 1:NN ) Ph_g2(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g2(:,Nvs*(K-1)+1:Nvs*K)*AG1(K)
      CASE( 'scal_rte' )
         FORALL( K = 1:NN )
c                     Mu <=> Mu
           Ph_g1(K,:) = AG1(:)*(Wup*Egg(I,K,:,1) + Wdo*Egg(I+1,K,:,1))
c                     Mu <=> -Mu
           Ph_g2(K,:) = AG1(:)*(Wup*Egg(I,K,:,2) + Wdo*Egg(I+1,K,:,2))
         END FORALL 
      END SELECT

      Ph_g1(:,:) = Ph_g1(:,:)/ALW(I) ; Ph_g2(:,:) = Ph_g2(:,:)/ALW(I)
      Gp_p => SDO%G_NL(I,    1:NNv,  1:NNv) 
      Gm_p => SDO%G_NL(I,1+NNv:2*NNv,1:NNv)

      DO K = 1,NNv
         Faj(:,K) = MATMUL(Ph_g2(:,:),Gm_p(:,K)) +
     $        MATMUL(Ph_g1(:,:),Gp_p(:,K))
         Fbj(:,K) = MATMUL(Ph_g2(:,:),Gp_p(:,K)) + 
     $        MATMUL(Ph_g1(:,:),Gm_p(:,K))
      ENDDO
      LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv
      DTA = DTR(I+1)
      Nu_j   =>  SDO%Nuj_lay(:,I) 
      Exp_Nu =>  SDO%EXP_NU_NL  (:,I)
      EXP_D  =>  IOP%Ex_D(:,I)
      EXP_U  =>  IOP%Ex_U(:,I)
c        SOU(:) = DEXP( -DTR_U(I)/MuV(:) )
      END SUBROUTINE AUX_IN_LAYER

      END 

