***************************************************************************
C
C       SCIATRAN - Version 3.9 - Last modification: 15 Jan 2018
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c                  Recalculation of the array AFI (ADO_)
c    Calculation of the  reflected and transmitted pseudo-forward intensity 
c    at the Gauss angles for all levels and all viewing angles
c       employing Source Function Integration technique 
c                   (Used in the spherical mode)
c *********************************************************************
      SUBROUTINE GAU_Adj_UP_DOWN_SPH( NL, NN, NLSA, Mh, ALBEDO, GSL, 
     $     IOP, Egg, SDO, SLS, ADS, ADO, UDB, Ref_surf)
c     ... Called by "st_adjoint_a.f" if ADAPTIVE_GRID = .TRUE.

      USE BRDF,        ONLY: Npg
      USE GRIDS,       ONLY: ZZZ 
      USE INTERN_DIMS, ONLY: Nvs, NNv
      USE LOG_PARS,    ONLY: DIR_OUT
      USE LOS_ANGLES,  ONLY: N_BE, N_BELOW
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: Pie, N_Adj, Vec_Sca, Add_lev, M_LIM, N_Eff,
     $                       SHI_LIM_C, ID_LIM, GeoD => GeoD_OUT, N_RHS,
     $                       Nc_der, SDVI, NS_wf
      USE SPHER_PARS,  ONLY: NFA => N_FOV_ANG, N_OBS_ALT, LW_FOV, 
     $                       FOV_WEIGHTS
      USE TYPES,       ONLY: KND, DCMP, SDO_TYPE, ADO_TYPE, GSL_TYPE,
     $                       IOP_TYPE, SLS_type, UDB_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) ::  NL, NN, NLSA, Mh
      REAL (KND), INTENT(IN) ::  ALBEDO, Egg(NL,NNv,NNv,2), 
     $     Ref_surf(N_RHS,Nvs*(NN+Npg))
c     ----------------------------------------------------------------
c     ... Spherical line-of-sight |
c     -----------------------------
      TYPE (SLS_type) :: SLS
c     -----------------------------------------------
      TYPE (IOP_TYPE), TARGET :: IOP    ! Integral optical parameters
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (GSL_type) :: GSL
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     -------------------------------------------------------------
c     ... Adjoint DOM auxiliary spherical geometry |
c     ----------------------------------------------
      TYPE ADS_type
        SEQUENCE
        LOGICAL,    POINTER :: Act_ag(:,:)
        REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
      END TYPE ADS_type
      TYPE (ADS_type) :: ADS
c     -------------------------------------------------------------
c     ... Local parameters
      CHARACTER (LEN=19), PARAMETER :: R_NAME = 'GAU_Adj_UP_DOWN_SPH'
      INTEGER  I, K, K1, K2, Jv, Jg, LL1, LR1, Ih, JL, P_M, Kt, N_1, IA, 
     $     Jf, Jvs, Jco, Jro, KK, Jw, Jw1, MLI, Isp, Max_Jco, Kc
      INTEGER, ALLOCATABLE :: I12(:,:), Iz(:,:)
      INTEGER, POINTER :: Id_a(:), Id_b(:)
      REAL (KND) Wdo, DTA, Mue, E_Mv, Cg, Ecg, Tbp, Tbm, DLR, WFOV, DUM,
     $     Exm(NN), Nup(NNv), Num(NNv), AG1(NN), S1, Qm(Nvs), Qp(Nvs),
     $     Tr1(N_adj,NNv), Tt1(N_adj,NNv), Itl(NL-1,N_Adj,NNv), 
     $     Tr2(N_adj,NNv), Tt2(N_adj,NNv), Irl(NL-1,N_Adj,NNv), 
     $     Tru(NNv,N_Adj), Trg(NNv,N_Adj), 
     $     Faj(NNv,NNv), Fbj(NNv,NNv), Exp_XM(NL-1,NN)
      REAL (KND), DIMENSION(NNv) :: Apr, Bpr, Apt, Bpt
      REAL (KND), TARGET :: Sjm(NNv), Cjm(NNv), Jbr(NL-1,NLSA,NN,2), 
     $     C_gls(NN), S_gls(NN), Jbt(NL-1,NLSA,NN,2)
      REAL (KND), ALLOCATABLE :: HDU(:)
      REAL (KND), POINTER :: Pcg(:), Psg(:), Pcj(:), Psj(:), Nu_j(:), 
     $     Exp_Nu(:), EXP_D(:), EXP_U(:), Gp_p(:,:), XM(:), ALW(:), 
     $     Gm_p(:,:), A_j(:), B_j(:), rs_k(:), AFI(:,:,:), DTR(:), 
     $     Pit(:), Pir(:), Ajs(:,:,:), Bjs(:,:,:),
     $     PAj(:), PBj(:)
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2, Cnu
      COMPLEX (kind(0.0d0)), ALLOCATABLE :: Cj_d(:), Cj_u(:), 
     $     Sj_d(:), Sj_u(:)      
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1 
      ALW  => IOP%ALW  ;  DTR  => IOP%DTR
      Id_a => IOP%Id_a ;  Id_b => IOP%Id_b
      Ajs  => ADO%Ajs  ;  Bjs  => ADO%Bjs 
      AFI  => ADO%AFI  ;  XM   => GSL%XM

      Afi = 0.d0
      AG1(1:NN) = 0.5d0*GSL%AG(1:NN)

      Max_Jco = MAXVAL( SDO%NcC(:) )
      IF( Max_Jco > 0 ) ALLOCATE( Cj_d(Max_Jco), Cj_u(Max_Jco), 
     $     Sj_d(Max_Jco), Sj_u(Max_Jco) )

      Wdo = 1.d0 - Wup
      K1 = MAXVAL( M_LIM(:) )  
      ALLOCATE( I12(NLSA,K1+1), Iz(NLSA,K1+1) )

c      WRITE(*,*) '  **** gau_adj_up_down_sph.f ***** '
c      WRITE(*,*) '    SHAPE(SDO%Nuj_lay) ', SHAPE(SDO%Nuj_lay)
c      WRITE(*,*) '    SHAPE(SDO%EXP_NU_NL) ', SHAPE(SDO%EXP_NU_NL)

      DO Jv = 1, NLSA  
         K2 = Jv*NFA - LW_FOV(Jv)  ! single selected direction
         IF( GeoD%LVG(IA,K2)%C_VA(1) > 0.d0 ) THEN
            I12( Jv,1:N_BE(K2) )           = 1
            I12( Jv,N_BE(K2)+1:M_LIM(K2) ) = 2
            Iz ( Jv,1:N_BE(K2) )           = -1
            Iz ( Jv,N_BE(K2)+1:M_LIM(K2) ) = 1
         ELSE
            I12( Jv,1:N_BE(K2) ) = 1
            Iz ( Jv,1:N_BE(K2) ) = 1
         ENDIF
      ENDDO

      Jbr = 0.d0 ; Jbt = 0.d0
      DO Jv = 1,NLSA
         DO Kt = (Jv-1)*NFA+1,Jv*NFA ! all directions within FoV
c         DO Kt = 2, 2
            IF( GeoD%LVG(IA,Kt)%C_VA(1) > 0.d0 ) THEN
               Pcg => C_gls(:) ; Psg => S_gls(:) ; P_M = -1  
               N_1 = N_OBS_ALT(1)  
               IF(Add_lev) N_1 = N_OBS_ALT(1) - 1  ! TEST ?  
c               WRITE(*,*) '   Downward pseudo-source P_M = ', P_M
            ELSE
               Pcg => S_gls(:) ; Psg => C_gls(:) ; P_M = 1
               N_1 = 1
c               WRITE(*,*) '   Upward pseudo-source P_M = ', P_M
            ENDIF

            Jf = Kt - (CEILING( FLOAT(Kt)/FLOAT(NFA)) - 1)*NFA
            WFOV = FOV_WEIGHTS(Jf)
c            WFOV = 1.d0

            Ih = 1 
c           ...  For all layers using transmission function before Th
c            WRITE(*,*) '    Use layers before Th  ----> '
            DO I = N_1, N_below(Kt)-1
c             ... Skip calculation within standard layer (see dom_spherical.f)
               IF( GSL%AG_ZERO(I) ) CYCLE               
               DLR = DTR(I+1)
               IF( I == N_below(Kt)-1 ) DLR = DLR/(ZZZ(I) - ZZZ(I+1))*
     $              (ZZZ(I) - GeoD%LVG(IA,Kt)%Th_V)
c                  WRITE(*,'(5(2x,I3))') 
c         $              Jv, Kt, Ih, I, SLS%IND_LS(  I,Kt,Ih)
c              ... Parameters of the approximation transmission function
c                  within layers before Th (see Eq.(Appr_b_ls) in STB) 
               Mue  = SLS%Mue_ls      (Kt,I,Ih) 
               rs_k => SLS%SIN_COEF_LS(Kt,I,:,Ih)
               K1   = SLS%N_sph_ls    (Kt,I,Ih)
               Exm(:) = DEXP( -DLR/XM(:) ) ; E_Mv = DEXP( -DLR/Mue )
c              ... Calculation auxiliary functions C_b and S_b 
c                  (see Eqs. (C_b-fin) and (S_b-fin) in STB) using
c                  Gaussian nodes instead of separation constants
               Call SC_FUNC_SPH
     $              (K1, XM, Exm, Mue, E_Mv, DLR, C_gls, S_gls)
c              ... Spherical transmission function at the top and bottom
c                  of j-th layer
               Tbm = SLS%LSE(IA,Kt)%EX( SLS%IND_LS(  I,Kt,Ih) ) ! top  T_(l-1)
               Tbp = SLS%LSE(IA,Kt)%EX( SLS%IND_LS(I+1,Kt,Ih) ) ! bot. T_l
c              ... Function Jbr (Eq.(Int_I_br) in STB)
               Jbr(I,Jv,:,Ih) = Jbr(I,Jv,:,Ih) + 
     $              ( Tbp*Exm(:) - Tbm + Tbm*Psg(:)/XM(:) ) *WFOV
c              ... Function Jbt (Eq.(Int_I_bt) in STB)
               Jbt(I,Jv,:,Ih) = Jbt(I,Jv,:,Ih) + 
     $              ( Tbp - Tbm*Exm(:) + P_M*Tbm*Pcg(:)/XM(:) ) *WFOV
c                   WRITE(*,'(1x,I3,*(2x,e12.4))') 
c     $                  I, Jbt(I,Jv,1,Ih),  Jbr(I,Jv,1,Ih)
            ENDDO  ! over layers before Th (I)

            IF(GeoD%LVG(IA,Kt)%NTH == GeoD%LVG(IA,Kt)%NLS) CYCLE
c            WRITE(*,*) '    Use layers after Th  ----> '
c           ***********************************************************
            Ih = 2
c           ... For all layers using the transmission function after Th
c           ***********************************************************
            DO I = 1,N_below(Kt)-1
c              ... Skip calculation within standard layer (see dom_spherical.f)
               IF( GSL%AG_ZERO(I) ) CYCLE
               DLR = DTR(I+1)
               IF( I == N_below(Kt)-1 ) 
     $              DLR = DLR/(ZZZ(I) - ZZZ(I+1))*
     $              (ZZZ(I) - GeoD%LVG(IA,Kt)%Th_V)
               Mue = SLS%Mue_ls(Kt,I,Ih) ; E_Mv = DEXP( -DLR/Mue )
               rs_k => SLS%SIN_COEF_LS(Kt,I,:,Ih)
               K1 = SLS%N_sph_ls(Kt,I,Ih)
               Exm(:) = DEXP( -DLR/XM(:) )
               Call SC_FUNC_SPH
     $              (K1, XM, Exm, Mue, E_Mv, DLR, C_gls, S_gls)
c              ... Spherical transmission at the top and bottom of j-th layer
               Tbm = SLS%LSE(IA,Kt)%EX(SLS%IND_LS(  I,Kt,Ih)) ! top  T_(l-1)
               Tbp = SLS%LSE(IA,Kt)%EX(SLS%IND_LS(I+1,Kt,Ih)) ! bot. T_l
               Jbr(I,Jv,:,Ih) = Jbr(I,Jv,:,Ih) + 
c                   ... Add function Jar (Eq.(Int_I_ar) in STB)
     $              ( Tbp*Exm(:) - Tbm + Tbp*C_gls(:)/XM(:) )*WFOV
               Jbt(I,Jv,:,Ih) = Jbt(I,Jv,:,Ih) + 
c                   ... Add function Jat (Eq.(Int_I_at) in STB)
     $              ( Tbp - Tbm*Exm(:) + Tbp*S_gls(:)/XM(:) )*WFOV
            ENDDO  ! over layers after Th (I)
         ENDDO  ! over all directions witin FOV (Kt)
      ENDDO  ! over line-of-sight (Jv)

      Itl = 0.d0  ;  Irl = 0.d0
      DO I = 1,NL-1 ! for all layers of original altitude grid
c        ... Skip calculation within standard layer (see dom_spherical.f)
         IF( GSL%AG_ZERO(I) ) CYCLE
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco  
c        ... "AUX_IN_LAYER" ==> Faj, Fbj, LL1, LR1, DTA, 
c                  Nu_j, Exp_Nu, EXP_D, EXP_U
         CALL AUX_IN_LAYER
         DO Jg = 1,NN   ! for all Gauss angles
            Cg = XM(Jg) ; Ecg = DEXP( -DTA/Cg ) ! DTA = DTR(I+1)
            Nup(:) = Cg *Nu_j/(Nu_j + Cg) 

            Sjm(1:Jro) = (1.0d0 - Exp_Nu(1:Jro)*Ecg)*Nup(1:Jro)
            WHERE( REAL(Nu_j(:)) /= REAL(Cg) )
               Cjm(1:Jro) = Nu_j(1:Jro)*Cg/(Nu_j(1:Jro) - Cg)
     $              *(Exp_Nu(1:Jro) - Ecg)
            ELSEWHERE
               Cjm(1:Jro) = Exp_Nu(1:Jro)*DTA
            END WHERE            
            Kc = 1
c           ... Use complex separation constants:
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nu_j(K1),Nu_j(K1+1) )
               Cnu = DCMP( Exp_Nu(K1),Exp_Nu(K1+1) )
               C2 = C1*Cg*(Cnu - Ecg)/(C1 - Cg) 
c                   Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)
               Cj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Cj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))
c              ----------------------------------------------
               C2 = C1*Cg*(1.d0 - Cnu*Ecg)/(C1 + Cg)
c                   Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
               Sj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Sj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))
               Kc = Kc + 1
            ENDDO

            DO KK = 1,Jro
               IF( Cg == Nu_j(KK) ) Cg = Cg*(1.d0 + 1.D-12)
            ENDDO
            Num(:) = Cg *Nu_j/(Nu_j - Cg)

            DO Jv = 1,NLSA      ! for all line-of-sight
               Jvs = Jv*NFA - LW_FOV(Jv) 
               IF( GeoD%LVG(IA,Jvs)%C_VA(1) > 0.d0 ) THEN
                  Pcj => Cjm(:)   ; Psj => Sjm(:)  
               ELSE
                  Psj => Cjm(:)   ; Pcj => Sjm(:)
               ENDIF

               Pir => Jbr(I,Jv,Jg,:) ; Pit => Jbt(I,Jv,Jg,:)
               DO JL = 1, M_LIM(Jvs) ! for all sub-layers along LoS
                  K1 = SHI_LIM_C(Jv) + JL
                  K2 = I12(Jv,JL)  ! select "before" Th (1) or "after" Th (2) 
c              ... Contribution of solution of homogeneous RTE
c                  into the source function
c                 ... Extract integration constants Aj and Bj, respectively.
                  PAj => ADO%LVEC(  LL1:LR1,    K1)
                  PBj => ADO%LVEC(LR1+1:LR1+NNv,K1)
                  Tr1(K1,1:Jro) = PAj(1:Jro) *Psj(1:Jro)*EXP_U(1:Jro) ! Aj*S*Ex
                  Tr2(K1,1:Jro) = PBj(1:Jro) *Pcj(1:Jro)*EXP_D(1:Jro) ! Bj*C*Ex
                  Tt1(K1,1:Jro) = PAj(1:Jro) *Pcj(1:Jro)*EXP_U(1:Jro) ! Aj*C*Ex 
                  Tt2(K1,1:Jro) = PBj(1:Jro) *Psj(1:Jro)*Exp_D(1:Jro) ! Bj*S*Ex
                  
              Apr(:) = ( -Ajs(I,:,K1)*Ecg + Ajs(Id_a(I)-1,:,K1) )*Nup(:)
              Bpr(:) = (  Bjs(I,:,K1) - Bjs(Id_b(I)+1,:,K1)*Ecg )*Num(:)
              Apt(:) = (  Ajs(I,:,K1) - Ajs(Id_a(I)-1,:,K1)*Ecg )*Num(:)
              Bpt(:) = (  Bjs(Id_b(I)+1,:,K1) - Bjs(I,:,K1)*Ecg )*Nup(:) 

                  Tr1(K1,:) =  Tr1(K1,:) + Apr(:) 
                  Tr2(K1,:) =  Tr2(K1,:) + Bpr(:) 
                  Tt1(K1,:) =  Tt1(K1,:) + Apt(:) 
                  Tt2(K1,:) =  Tt2(K1,:) + Bpt(:) 
                                    
c                 ... Dimension A_j and B_j is NNv
c                     A_j and B_j are used here without integration over FoV
c                     at the selected direction within FoV
                  A_j => ADO%Aj_sph(I,:,Jvs,1,K2) 
                  B_j => ADO%Bj_sph(I,:,Jvs,1,K2)
                  IF( ID_LIM(Jvs,I,K2) == JL ) THEN   
                     Tr1(K1,:) =  Tr1(K1,:) + A_j(:)*Pir(K2)*Nup(:) 
                     Tr2(K1,:) =  Tr2(K1,:) - B_j(:)*Pir(K2)*Num(:) 
                     Tt1(K1,:) =  Tt1(K1,:) - A_j(:)*Pit(K2)*Num(:) 
                     Tt2(K1,:) =  Tt2(K1,:) + B_j(:)*Pit(K2)*Nup(:) 

                     Qp(1) = ( Wup*ADS%PLS(I,Jg,Jv,JL) 
     $                       + Wdo*ADS%PLS(I+1,Jg,Jv,JL) )
                     Qm(1) = ( Wup*ADS%MLS(I,Jg,Jv,JL) 
     $                       + Wdo*ADS%MLS(I+1,Jg,Jv,JL) )
                     Qp(:) = Qp(:)*Iz(Jv,JL)/ALW(I)/2.d0
                     Qm(:) = Qm(:)*Iz(Jv,JL)/ALW(I)/2.d0
                     
                     Itl(I,K1,Jg) = Pit(K2)*Qp(1)  
     $                  + SUM(Tt1(K1,:)*Faj(Jg,:) + Tt2(K1,:)*Fbj(Jg,:))

                     Irl(I,K1,Jg) = Pir(K2)*Qm(1)
     $                  + SUM(Tr1(K1,:)*Fbj(Jg,:) + Tr2(K1,:)*Faj(Jg,:))
                  ELSE
c                     The contributions in the reflected and transmitted 
c                     source function from solution of homogeneous RTE only
                     Itl(I,K1,Jg) = 
     $                    SUM(Tt1(K1,:)*Faj(Jg,:) + Tt2(K1,:)*Fbj(Jg,:))
                     Irl(I,K1,Jg) = 
     $                    SUM(Tr1(K1,:)*Fbj(Jg,:) + Tr2(K1,:)*Faj(Jg,:))
                  ENDIF
               ENDDO  ! over sub-intervals (JL)
            ENDDO  ! over LoS (Jv)
         ENDDO  ! over Gauss angles (Jg)
      ENDDO  ! over main altitude layers (I)
      
      FORALL( I=1:NL-1 ) Exp_XM(I,:) = DEXP(-DTR(I+1)/XM(:))
      Trg = 0.d0 ; Afi(1,1:NNv,:) = Trg(:,:)
      DO I = 1,NL-1
         DO Jg = 1,NN
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Trg(K1,:) = Trg(K1,:)*Exp_XM(I,Jg) + Itl(I,:,K1)
            ENDDO  ! over Stokes components (K1)
         ENDDO  ! over Gaussian angles (Jg)
         Afi(I+1,1:NNv,:) = Trg(:,:) 
      ENDDO
c
c     Reflected from surface pseudo-forward radiation
c
      Tru = 0.d0
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
         LBC: Block
c        ---------------------------------------------------------------
         IF( Mh > 1 ) EXIT LBC  ! ====>
c        ...............................................................
         DO Jv = 1, NLSA ; Jvs = Jv*NFA - LW_FOV(Jv)
            MLI = M_LIM(Jvs)  
            Isp = Ns_wf*SHI_LIM_C(Jv) ! shift for all previous Jv
            DO JL = 1,MLI
               DO Jw = 1,NS_wf 
                  IF( Nc_der(Jw) /= 1 ) CYCLE
                  Jw1 = Isp + (Jw-1)*MLI + JL
                  Tru(1:NNv:Nvs,Jw1) = 2.d0*Albedo*ADO%FLUX_D(Jw1) 
c                 ... Downward pseudo source
                  IF( JL==MLI.AND.GeoD%LVG(IA,Jv)%C_VA(1)>0.d0 )
     $                 Tru(1:NNv:Nvs,Jw1) = Tru(1:NNv:Nvs,Jw1)
     $                    + 2.d0*Albedo*GSL%EXL(NL,Jv) 
               ENDDO  ! over  Stokes component derivatives (Jw)
            ENDDO  ! over sub-intervals along LoS (JL)
         ENDDO  ! over LoS (Jv)
c        ---------------------------------------------------------------
         END Block LBC
      CASE('Bidirect')         
         LBC_BRF: Block
c        ---------------------------------------------------------------
         DO Jv = 1,N_Adj
            Tru(:,Jv) = 2.d0*Ref_surf(Jv+SDVI,1:NNv) 
         ENDDO                   
         IF( GeoD%View(IA)%S=='down' ) EXIT LBC_BRF ! ====>
c        ...............................................................
         ALLOCATE( HDU(NNv) )
c         WRITE(*,*) '   SHAPE(UDB%BRF_GS)', SHAPE(UDB%BRF_GS)
         DO Jv = 1,NLSA ;  Jvs = Jv*NFA-LW_FOV(Jv) ; MLI = M_LIM(Jvs) 
            Isp = Ns_wf*SHI_LIM_C(Jv) ! shift for all previous Jv
            IF( GeoD%LVG(IA,Jvs)%C_VA(1) <= 0.d0) CYCLE ! downward radiation 
            DO JL = MLI,MLI
               DO Jw = 1,NS_wf 
                  Jw1 = Isp + (Jw-1)*MLI + JL
                  K2 = Nvs*(SDVI + Jv - 1) + Nc_der(Jw)
                  HDU(:) = UDB%BRF_GS(1:NNv,K2,Mh)
                  Tru(:,Jw1) = Tru(:,Jw1) 
     $                 + 2.d0*GSL%EXL(NL,Jv)*HDU(:) 
               ENDDO  ! over number of Stokes WF (Jw,Jw1)
            ENDDO  ! over sub-intervals along LoS (JL)
         ENDDO  ! over line-of-sight (Jv)
         END Block LBC_BRF
c        ---------------------------------------------------------------         
      END SELECT ! 'Lambert' or 'Bidirect'

      Afi(NL,NNv+1:2*NNv,:) = Tru(:,:)
      DO I = NL-1,1,-1
         DO Jg = 1,NN
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Tru(K1,:) = Tru(K1,:)*Exp_XM(I,Jg) + Irl(I,:,K1)
            ENDDO  ! over Stokes components (K1)
         ENDDO  ! over Gaussian angles (Jg)
         Afi(I,NNv+1:2*NNv,:) = Tru(1:NNv,:)
      ENDDO

c      WRITE(*,*) '****** afi-1.sfi is written ***** '
c      Open( 12, file = DIR_OUT//'/afi-1.sfi' )
c      DO I = 1, NL
c         WRITE( 12,'(*(1x,e15.7))')
c     $        ZZZ(I),(AFI(I,K1,1), K1 = 1,2*NNv)
c      ENDDO
c      Close(12)
c      Open( 12, file = DIR_OUT//'/afi-2.sfi' )
c      DO I = 1, NL
c         WRITE( 12,'(*(1x,e15.7))')
c     $        ZZZ(I),(AFI(I,K1,2), K1 = 1,2*NNv)
c      ENDDO
c      Close(12)
c      Open( 12, file = DIR_OUT//'/afi-3.sfi' )
c      DO I = 1, NL
c         WRITE( 12,'(*(1x,e15.7))')
c     $        ZZZ(I),(AFI(I,K1,3), K1 = 1,2*NNv)
c      ENDDO
c      Close(12)
c      Open( 12, file = DIR_OUT//'/afi-4.sfi' )
c      DO I = 1, NL
c         WRITE( 12,'(*(1x,e15.7))')
c     $        ZZZ(I),(AFI(I,K1,4), K1 = 1,2*NNv)
c      ENDDO
c      Close(12)

      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
      SUBROUTINE SC_FUNC_SPH( Nco, Ar, Ear, Sc, Esc, Dtl, Cres, Sres )
      INTEGER      Nco, Jc
      REAL (KND)   Zn, Ck, Ar(NN), Ear(NN), Sc, Esc, Dtl,
     $             Cres(NN), Sres(NN)
      COMPLEX (kind(0.0d0)) :: Muc

      Sres(:) = Ar(:)*Sc*( 1.d0 - Ear(:)*Esc )/( Ar(:) + Sc )
      WHERE( REAL(Ar(:)) /= REAL(Sc) )
         Cres(:) = Ar(:)*Sc*(Ear(:) - Esc)/(Ar(:) - Sc)
      ELSEWHERE
         Cres(:) = Ear(:)*Dtl
      END WHERE
      Zn = -1
      DO Jc = 1,Nco-1
         Ck = Dtl/DBLE(Jc)/Pie ; Muc = Sc*Ck/DCMP( Ck,-Sc )
         Cres(:) = Cres(:) + AIMAG( Muc*Ar(:)/(Muc-Ar(:)) )
     $        *rs_k(Jc)*(Zn*Esc - Ear(:))
         Sres(:) = Sres(:) + AIMAG( Muc*Ar(:)/(Muc+Ar(:)) )
     $        *rs_k(Jc)*(1.0d0 - Zn*Esc*Ear(:))
         Zn = -Zn                  
      ENDDO
      END SUBROUTINE SC_FUNC_SPH

c **********************************************************************
      SUBROUTINE AUX_IN_LAYER
      REAL (KND), 
     $   DIMENSION (NNv,NNv) :: Ph_g1, Ph_g2, TTT

      SELECT CASE( Vec_Sca )
      CASE( 'vect_rte' )
c     ... It should be reproduced at first in the vector case 
c         phase matrix corresponding to the scattering +Gauss <-- +Gauss
         TTT(:,:)= Wup*Egg(I,:,:,1) + Wdo*Egg(I+1,:,:,1)
         TTT(:,:) = TTT(:,:) 
     $        + (Wup*Egg(I,:,:,2) + Wdo*Egg(I+1,:,:,2))
         
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 
         DO Jv = 1,NN
            Ph_g1(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
         FORALL( K = 1:NN ) Ph_g1(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g1(:,Nvs*(K-1)+1:Nvs*K)*AG1(K)

c        ... It should be reproduced at first in the vector case 
c            phase matrix corresponding to the scattering 
c            +Gauss <-- -Gauss
         TTT(:,:)= Wup*Egg(I,:,:,2) + Wdo*Egg(I+1,:,:,2)
         TTT(:,:) = 
     $        (Wup*Egg(I,:,:,1) + Wdo*Egg(I+1,:,:,1)) 
     $        - TTT(:,:)
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 
         DO Jv = 1,NN
            Ph_g2(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
         FORALL( K = 1:NN ) Ph_g2(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g2(:,Nvs*(K-1)+1:Nvs*K)*AG1(K)
      CASE( 'scal_rte' )
         FORALL( K = 1:NN )
c                     Mu <=> Mu
            Ph_g1(K,:) = AG1(:)*(Wup*Egg(I,K,:,1) + Wdo*Egg(I+1,K,:,1))
c                     Mu <=> -Mu
            Ph_g2(K,:) = AG1(:)*(Wup*Egg(I,K,:,2) + Wdo*Egg(I+1,K,:,2))
         END FORALL 
      END SELECT

      Ph_g1(:,:) = Ph_g1(:,:)/ALW(I) ; Ph_g2(:,:) = Ph_g2(:,:)/ALW(I)
      Gp_p => SDO%G_NL(I,1:NNv,1:NNv) 
      Gm_p => SDO%G_NL(I,1+NNv:2*NNv,1:NNv)

      DO K = 1,NNv
         Faj(:,K) = MATMUL(Ph_g2(:,:),Gm_p(:,K)) +
     $        MATMUL(Ph_g1(:,:),Gp_p(:,K))
         Fbj(:,K) = MATMUL(Ph_g2(:,:),Gp_p(:,K)) + 
     $        MATMUL(Ph_g1(:,:),Gm_p(:,K))
      ENDDO

      LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv
      DTA = DTR(I+1)
      Nu_j   =>  SDO%Nuj_lay(:,I) 
      Exp_Nu =>  SDO%EXP_NU_NL  (:,I)
c     ... Dimension Ex_U(NNv,NL-1)
      EXP_D  =>  IOP%Ex_D(:,I)
      EXP_U  =>  IOP%Ex_U(:,I)
      END SUBROUTINE AUX_IN_LAYER

      END 
