c**********************************************************************
C
C       SCIATRAN - Version 3.9 - Last modification: 18 Dec 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c     Source-function integration of the reflected pseudo-forward
c                 intensity in the solar direction
c
c *********************************************************************
      SUBROUTINE GAU_Adj_UP_SOL(NL, NN, NLSA, NSZA, XM, AG1, MuV, MuS, 
     $     EXL, SDO, IOP, Ph_zA, ADO, ASOL_ADJ)
c     ... Call in "st_adjoint_a.f"
c                 "st_adjoint_o.f"

      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: DIR_OUT
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: N_RHS, N_Adj, SDVI, NS_wf, GeoD =>GeoD_OUT,
     $                       Pie, NSTRMS, Ainc, Vec_Sca
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: SDO_TYPE, ADO_TYPE, IOP_TYPE
      USE WATER,       ONLY: Obs_POSITION
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NL, NN, NLSA, NSZA
      REAL (KND), INTENT(IN) :: XM(NN), AG1(NN), MuV(NLSA), 
     $     MuS(NSZA), EXL(NL,NLSA)
      REAL (KND) ASOL_ADJ(N_Adj,Nvs*NSZA)
c     --------------
      TYPE phase_tdz
       SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA
c     -------------------------------
      TYPE (IOP_TYPE), TARGET :: IOP    ! Integral optical parameters
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
c     -------------------------------------------------------------
c     ... Local variables
c     -------------------
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'GAU_ADJ_UP_SOL'
      INTEGER, POINTER :: Id_a(:), Id_b(:)
      INTEGER  I, IA, Iexl, K, Kg, Ks, K1, Kv, Jg, Js, Jv, Jro, Jco,
     $     Jw, Jw1, LL1, LR1, Lk, NNv
      REAL (KND)  Wdo, DTA, Cv, ECV, Sc
      REAL (KND)  Sou(NLSA), Isl(NL-1,N_Adj,Nvs*NSZA), L_S(Nvs,Nvs),
     $     XML(NN)
      REAL (KND), DIMENSION (Nvs*NSZA,Nvs*NN) :: Ugp, Ugm, Sg1, Sg2
      REAL (KND), DIMENSION (Nvs*NN) :: TN1, TN2, Sjm, Cjm, Apa, Bpa
      REAL (KND),  TARGET :: Csl(NLSA), Ssl(NLSA)
      REAL (KND), POINTER :: Nuj(:), Enu(:), EXP_D(:), EXP_U(:),
     $     P_cs(:), Phip(:,:), Phim(:,:), ALW(:), DTR(:)
      COMPLEX (kind(0.0d0)) :: C1, C2, Cnu, Cmu
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = Nvs*NN     ;  IA = 1
      ALW  => IOP%ALW  ;  DTR  => IOP%DTR
      Id_a => IOP%Id_a ;  Id_b => IOP%Id_b

      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:)
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
      END SELECT
      Wdo = 1.d0 - Wup
      
c     ... Here we need to calculate scattering from the Gaussian nodes
c         to the direction opposite to the solar direction (-mu0), i.e.
c             -mu0 <-- Gau and -mu0 <-- -Gau
      DO I = 1,NL-1
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco 
         DO Js = 1,NSZA 
            Ks = Nvs*(Js-1)
            DO Jg = 1,NN 
               Kg = Nvs*(Jg-1)
c              ... The matrix ^M = Egs(:,:,2) is designed so
c                  that D*^M describes the phase mtrix -Sol <-- Gau
c                  It is used at this step without multiplication by D
c                  from left.
               Ugp(Ks+1:Ks+Nvs,Kg+1:Kg+Nvs) = 
     $              (Wup*Ph_zA%Egs(I,Kg+1:Kg+Nvs,Ks+1:Ks+Nvs,2) 
     $              + Wdo*Ph_zA%Egs(I+1,Kg+1:Kg+Nvs,Ks+1:Ks+Nvs,2))
     $              *AG1(Jg)/MuS(Js)*XML(Jg)

c              ... The matrix M = Egs(:,:,1) is designed so 
c                  that M describes the phase mtrix Sol <-- Gau,
c                  The matrix D*M*D should be used to obtain 
c                  phase mtrix -Sol <-- -Gau.
c                  It is used at this step without multiplication by D
c                  from left and from right. 
c                  The multiplication by D from right is omitted 
c                  because below this matrix will be multiplied by the 
c                  Stokes vector of the upward pseudo-forward radiation 
c                  which should be multiplied by D from left, i.e. 
c                  ^M*D * D*S(-Gau)
               Ugm(Ks+1:Ks+Nvs,Kg+1:Kg+Nvs) = 
     $              (Wup*Ph_zA%Egs(I,Kg+1:Kg+Nvs,Ks+1:Ks+Nvs,1) 
     $              + Wdo*Ph_zA%Egs(I+1,Kg+1:Kg+Nvs,Ks+1:Ks+Nvs,1))
     $              *AG1(Jg)/MuS(Js)*XML(Jg)
            ENDDO
         ENDDO

         Ugp(:,:) = Ugp(:,:)/ALW(I) ; Ugm(:,:) = Ugm(:,:)/ALW(I) 

c        ... The matrix G_NL has the following structure 
c            (see homog_partic.f)
c                        1:NNv  1+NNv:2*NNv
c             1:NNv   |  Phi+,    Phi- |  
c         1+NNv:2*NNv |  Phi-,    Phi+ |
c
         Phip => SDO%G_NL(I,    1:NNv,  1:NNv) 
         Phim => SDO%G_NL(I,1+NNv:2*NNv,1:NNv)

c        ... local arrays Sg1 and Sg2
c            These matrices should be multiplied by D from left
c            see Eq.(SF-p-forw) in STB.
         DO K = 1,NNv
            Sg1(:,K) = 
     $           MATMUL(Ugp(:,:),Phip(:,K)) + MATMUL(Ugm(:,:),Phim(:,K))
            Sg2(:,K) = 
     $           MATMUL(Ugp(:,:),Phim(:,K)) + MATMUL(Ugm(:,:),Phip(:,K))
         ENDDO

         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1 - 1
         DTA = DTR(I+1)
c           ... Input arrays
         Nuj    =>  SDO%Nuj_lay(:,I) 
         Enu    =>  SDO%EXP_NU_NL(:,I)
         EXP_D  =>  IOP%Ex_D(:,I)
         EXP_U  =>  IOP%Ex_U(:,I)

         Sou(:) = DEXP( -DTA/MuV(:) )             
         DO Js = 1,NSZA
            Ks = (Js-1)*Nvs
            CV = MuS(Js) ; ECV = DEXP( -DTA/CV )            

c           --> a) real separation constants
            Sjm(1:Jro) = Nuj(1:Jro)*CV/(Nuj(1:Jro) + CV)
     $           *(1.d0 - Enu(1:Jro)*ECV)*EXP_U(1:Jro) !*Exu(1:Jro) ???
            WHERE( REAL(Nuj(1:Jro)) /= REAL(Cv) )
               Cjm(1:Jro) = Nuj(1:Jro)*Cv/(Nuj(1:Jro) - Cv)
     $              *(Enu(1:Jro) - ECV)*Exp_D(1:Jro)
            ELSEWHERE
               Cjm(1:Jro) = Enu(1:Jro)*DTA*Exp_D(1:Jro)
            END WHERE
c           --> b) imaginary separation constants
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nuj(K1),Nuj(K1+1) )
               Cnu = DCMP( Enu(K1),Enu(K1+1) )
               C2 = C1*Cv*(Cnu - ECV)/(C1 - Cv)
     $              *DCMP(EXP_D(K1),EXP_D(K1+1))
               Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)

               C2 = C1*Cv*(1.d0 - Cnu*ECV)/(C1 + Cv)
     $              *DCMP(EXP_U(K1),EXP_U(K1+1))
c     $              *DCMP(Exu(K1),Exu(K1+1)) ???
               Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
            ENDDO

            Ssl(:) = CV*Muv(:)/(CV + MuV(:))*(1.d0 - Sou(:)*ECV)
            WHERE ( Real(MuV(:)) /= Real(CV) ) 
               Csl(:) = MuV(:)*CV*(Sou(:)-ECV)/(MuV(:)-CV)
            ELSEWHERE
               Csl(:) = ECV*DTA
            END WHERE
            
            DO Jv = 1,NLSA 
               IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN
                  Iexl = 0  ;  P_cs => Ssl(:) 
               ELSE
                  Iexl = 1  ;  P_cs => Csl(:)
               ENDIF
               Kv = (Jv-1)*Nvs
               DO Jw = 1,NS_wf
                  Jw1 = NS_wf*(Jv-1) + Jw
                  Apa(:) = 
     $                 (-ADO%Ajs(I,:,Jw1)*ECV + ADO%Ajs(Id_a(I)-1,:,Jw1)
     $                 + ADO%Aj_ls(:,Jw1,I)*P_cs(Jv)*EXL(I+Iexl,Jv) )*Cv
                  Bpa(:) = 
     $                 ( ADO%Bjs(I,:,Jw1) - ADO%Bjs(Id_b(I)+1,:,Jw1)*ECV
     $                 - ADO%Bj_ls(:,Jw1,I)*P_cs(Jv)*EXL(I+Iexl,Jv) )*Cv

c                  WRITE(*,*) I, Jw, SUM( Bpa(:) )
c                  IF( Jw==2 ) WRITE(*,*) I, Jw, SUM( Bpa(:) )

c                ... Real separation constants
                  TN1(1:Jro) = ADO%LVEC(LL1:Lk+Jro,Jw1)*Sjm(1:Jro)
     $                 + Apa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) + Cv)
                  TN2(1:Jro) = ADO%LVEC(LR1+1:LR1+Jro,Jw1)*Cjm(1:Jro)
     $                 + Bpa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) - Cv)

c                 ... Complex separation constants
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     Cnu = DCMP( Nuj(K1),Nuj(K1+1) )
                     C1 = DCMP(Apa(K1),Apa(K1+1))*Cnu/(Cnu + Cv)
                     Tn1(K1) = ADO%LVEC(Lk+K1,Jw1)*Sjm(K1) + 
     $                    ADO%LVEC(Lk+K1+1,Jw1)*Sjm(K1+1)+2.d0*DBLE(C1)
                     Tn1(K1+1) = ADO%LVEC(Lk+K1+1,Jw1)*Sjm(K1) - 
     $                    ADO%LVEC(Lk+K1,Jw1)*Sjm(K1+1) - 2.d0*AIMAG(C1)

                     C2 = DCMP(Bpa(K1),Bpa(K1+1))*Cnu/(Cnu - Cv)
                     Tn2(K1) = ADO%LVEC(LR1+K1,Jw1)*Cjm(K1) +
     $                    ADO%LVEC(LR1+K1+1,Jw1)*Cjm(K1+1)+2.d0*DBLE(C2)
                     Tn2(K1+1) = ADO%LVEC(LR1+K1+1,Jw1)*Cjm(K1) - 
     $                    ADO%LVEC(LR1+K1,Jw1)*Cjm(K1+1)-2.d0*AIMAG(C2)
                  ENDDO  ! over complex separation constants (K1)

                  Sc = P_cs(Jv)*EXL(I+Iexl,Jv)/2.d0/Cv/MuV(Jv)/ALW(I)

c             ... Dimension of Ph_zA%EsL(NL,Nvs*NLSA,Nvs*NSZA,2)
c                   It is used here to calculate single scattering of 
c                 the pseudo-forward radiation -Sol <-- -Los.
c                 The matrix M = EsL(:,:,:,1) is designed so that 
c                 M describes the phase mtrix Sol <-- Los.
c                   The matrix D*M*D4 should be used to obtain 
c                 single scattering phase mtrix -Sol <-- -Los 
c                 (see Eq.(Qpr-los-sol) in STB).
c                    The multiplication by D from left is not used
c                 at this step. 
c                    The multiplication by D4 from right is not used
c                 preliminary also ???
                  L_S(:,:) = Sc*
     $                 ( Wup*Ph_zA%EsL(I,  Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) 
     $                 + Wdo*Ph_zA%EsL(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) )
                  DO K1 = Ks+1,Ks+Nvs
                     Isl(I,Jw1,K1) = SUM( L_S(K1-Ks,:)*Ainc(Jw,:) )
     $                    + SUM( Tn1(:)*Sg1(K1,:) + Tn2(:)*Sg2(K1,:) )
                  ENDDO  ! over all Stokes components (K1)

               ENDDO  ! over Stokes component derivatives (Jw,Jw1)
            ENDDO   ! over viewing angles (Jv)
         ENDDO  ! over solar angles (Js)
      ENDDO  ! over layers (I)

c     ... The multiplication by D from left is not performed
c         because in case of the unpolarized solar light the 
c         only first component of this Stokes vector will be
c         used (see Eq.(T1-gen) in STB)
      IOP%Ref_sol(NL,:,:) = ASOL_ADJ(:,:)
      DO I = NL - 1,1,-1
         DO Js = 1,NSZA
            IF( Mus(Js) <= 0.d0 ) CYCLE
            Sc = DEXP(-DTR(I+1)/MuS(Js))
            DO K1 = (Js-1)*Nvs+1,Js*Nvs
               ASOL_ADJ(:,K1) = Sc*ASOL_ADJ(:,K1) + Isl(I,:,K1)
            ENDDO
         ENDDO  ! over solar angles (Js)
         IOP%Ref_sol(I,:,:) = ASOL_ADJ(:,:) 
      ENDDO  ! over main altitude layers (I)

      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN

      END 
