C********************************************************************C
C
C       SCIATRAN - Version 4.2 - Last modification: 24 Jan 2020
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c *********************************************************************
      SUBROUTINE GENERAL_BRDF_WF(NN, NSZA, NLSA, Mh, JPOS, POS, E_W,  
     $     MuS, MuV, EXL, EXS, AFI, UDB_A, SOL_BOT)
c     ... Call in "st_adjoint_a.f"

      USE BRDF,        ONLY: Npg, Exact_brf, Nder
      USE GT_POLYN,    ONLY: XM, AXM
      USE INFO,        ONLY: WF_DB, RET_INFO
      USE INTERN_DIMS, ONLY: Nvs
      USE PRO_SAIL,    ONLY: LAI
      USE SPHER_PARS,  ONLY: JV_S, JV_E, NLO => N_LOS_OUT,  
     $                       Phi => ANG_OUT, NAZ => N_AZ_VAR
      USE ST_VARS,     ONLY: N_Adj, N_Eff, N_RHS, GeoD => GeoD_OUT,
     $                       SDVI, SURF_TYPE
      USE TOOLS,       ONLY: FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: UDB_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NN, NSZA, NLSA, Mh, JPOS
      REAL (KND), INTENT(IN) :: E_W, POS, MuS(NSZA),  
     $     EXS(NSZA), EXL(NLSA),
     $     AFI(2*NN*Nvs,N_Adj), MuV(NLSA), SOL_BOT(2*Nvs*NN,N_RHS)
c     -------------------------------
      TYPE (UDB_TYPE) :: UDB_A  ! Boundary conditions
c     ... Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'GENERAL_BRDF_WF'
      LOGICAL  FLG
      INTEGER  I, IA, J, K, Id, II, Ko, Ja, Js, Jv, JL, Iks, K1, Ws,
     $     Ws1, NNv
      REAL (KND)  DC
      REAL (KND)  ALB_WF(NLO,Nder), ASM(NLSA),
     $     dAdj(NLSA,Nvs*NSZA,Nder), A_int(Nvs*NSZA), 
     $     RHS(2*Nvs*NN), dRef(N_RHS,Nvs*(NN+Npg),Nder),
     $     Fint_1(Nvs*NLSA), Fint_2(Nvs*NLSA), 
     $     Ref_for(NSZA,Nvs*NLSA,Nder), X_ref(NN+1)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( Exact_brf ) 
     $     STOP '   Exact_brf in "general_brdf_wf.f" '

      NNv = Nvs*NN
      X_ref(1:NN) = XM(:) ; X_ref(NN+1) = 1.d0
      IA = 1
      DO Js = 1,N_RHS
         DO J = 1,Nvs
            RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AXM(1:NN)
         ENDDO  ! over Stokes components (J)
c        ... Reflected from the surface forward and pseudo-forward 
c            radiation using derivative of BRDF for Gaussian and viewing angles
         DO K = 1,Nvs*(NN+Npg) 
            DO Id = 1, Nder
               dRef(Js,K,Id) = SUM( UDB_A%Der_GG(K,:,Mh,Id)*RHS(1:NNv) )
            ENDDO
         ENDDO              
      ENDDO  ! over solar angle (Js)

      DO Id = 1,Nder
         DO Jv = 1,NLSA  ;  Ko = Jv+SDVI
            DO K1 = 1,Nvs 
c            ... Interpolate reflected from the surface  
c                pseudo-forward intensity onto solar direction 
               CALL AKIMA_INT(NN+1, X_ref, 
     $              dRef(Ko,K1:Nvs*(NN+1):Nvs,Id),  
     $              NSZA, MuS, A_int(K1:Nvs*NSZA:Nvs) )
            ENDDO  ! over Stokes components (K1)
c           ... Second term in Eq.(WF_BRDF) (see STB)
            dAdj(Jv,:,Id) = A_int(:)*EXS(:)*Mus(:) ! Scalar only !
         ENDDO  ! over line-of-sight (Jv)
      ENDDO  ! over derivatives (Id)

c    ... First term in ALB_WF(J,Id) (see  Eq.(WF_BRDF) in STB): 
c        scalar product of reflected from the surface forward intensity
c        and pseudo-forward intensity 
      J = 0
      DO Js = 1,NSZA
         DO Jv = JV_S(Js),JV_E(Js)  
            DO Ja = 1,NAZ  ;  J = J + 1
               Dc = DCOS( Phi(3,J)*DBLE(Mh-1) )
               DO Id = 1,Nder
                  ALB_WF(J,Id) = (2.d0
     $                 *SUM( dRef(Js,1:NN,Id)*AFI(1:NN,Jv)*AXM(:) )
     $                 + dAdj(Jv,Js,Id) )*Dc
               ENDDO  ! over derivatives (Id)
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight (Jv)
      ENDDO  ! over solar angle (Js)

c     !!! Below for scalar case only !!! Reflected radiation
      IF( GeoD%VIEW(IA)%S=='up' ) THEN
         DO Id = 1,Nder
            DO Js = 1,NSZA
               DO K1 = 1,Nvs 
c                 ... Interpolate reflected from the surface  
c                     forward intensity onto observation direction 
                  CALL AKIMA_INT(NN+1, X_ref, 
     $                 dRef(Js,K1:Nvs*(NN+1):Nvs,Id),  
     $                 NLSA, Muv, Fint_1(K1:Nvs*NLSA:Nvs) )
c                 ... Interpolation of directional reflection onto
c                     observation direction
                  CALL AKIMA_INT(NN+1, X_ref, 
     $               UDB_A%Der_GS(K1:Nvs*(NN+1):Nvs,(Js-1)*Nvs+1,Mh,Id),  
     $               NLSA, Muv, Fint_2(K1:Nvs*NLSA:Nvs) )
               ENDDO  ! over Stokes components (K1)
c              ... Thrid plus fourth terms in Eq.(WF_BRDF) (see STB)
               Ref_for(Js,:,Id) = 2.d0*Fint_1(:)*EXL(:) 
     $              + EXL(:)*EXS(Js)*MuS(Js)*Fint_2(:)
            ENDDO  ! over solar angle (Js)
         ENDDO  ! over derivatives (Id)
         J = 0
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  
               DO Ja = 1,NAZ  ;  J = J + 1
                  Dc = DCOS( Phi(3,J)*DBLE(Mh-1) )
                  DO Id = 1,Nder
                     ALB_WF(J,Id) = ALB_WF(J,Id) + Ref_for(Js,Jv,Id)*Dc
                  ENDDO  ! over derivatives (Id)
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over line-of-sight (Jv)
         ENDDO  ! over solar angle (Js)
      ENDIF  ! reflected radiation
      
      Id = 1
      DO II = 1,SIZE( RET_INFO )
         IF( .NOT.RET_INFO(II)%STAT ) CYCLE
         IF( .NOT.(RET_INFO(II)%TYPE=='prosail') ) CYCLE
         WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1) 
     $        + ALB_WF(:,Id)*E_W
         Id = Id + 1
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
