C***********************************************************************
C
C       SCIATRAN - Version 4.7 - Last modification: 30 Oct 2023
C
C       Institute of Remote Sensing, University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE GENERAL_CLOUD_INIT(START_WAV, END_WAV, *)
c     ... Called by "gt_iface.f"

      USE ALT_INPUT,   ONLY: ALIN_ACT, ALIN_NWL, ALIN_WL, NC_CLD, 
     $                       ALIN_CLD_A, ALIN_CLD_S, ALIN_CLD_EC,
     $                       IPRT_ALIN
      USE CLD_INP,     ONLY: USER_EK_NAME, DBC_NAME, PHASE_NAME, LC_CLD,
     $                       D_MF, TDS, Plus_ice, CON_VAL_CLD, UC_CLD,
     $                       CDB_PATH => CLOUD_DATA_BASE_PATH, N_ref_db,
     $                       MAN_AUTO_CLD, OUT_PH_CLD, Path_PH_CLD, 
     $                       S_name, CRST, N_ref_Hex, N_ref_Fra, MLCS,
     $                       N_ref_Bdb, S_name_Bdb, Use_Bdb, N_Rdb,
     $                       N_ref_Ydb, Use_Ydb, S_name_Ydb, TotA_Bdb,
     $                       S_name_Fra, S_name_Hex, Refr_water_inp, 
     $                       Refr_ice_inp, N_ref_sm, 
     $                       HOW_CLD_COEF, Nr_db, Ydb_ang, Use_Ydb_SCM
      USE CLOUDS,      ONLY: CLD_LAYER_COEFF, CLD_SAVE_COEFF, 
     $                       SCC_LAYER_NOSCALE, SCAT_MAT, SCAT_ANG,
     $                       SCI_LAYER_NOSCALE,  N_Red_WL, Red_WL_Ydb,
     $                       NCW => N_CLD_NETWAVS, NG_cld, Greek_cld,
     $                       CLD_NETWAVS, C_P_M => CLD_PHAS_MOMS,
     $                       N_CLD_LAYERS, CLD_DM, REF_IMAG_W, F_Dfit, 
     $                       REF_IMAG_I, CLD_LAYER_NUM, CLD_F_DM,
     $                       CLD_LAYER_DERIV, Red_WL
      USE CLOUDS_SS,   ONLY: DO_SSRCA, MAXMOM, R_INF_WL, G_CLD_WL
      USE DATA_RET,    ONLY: RET_FLAG
      USE GT_POLYN,    ONLY: NLEGEN
      USE INTERN_DIMS, ONLY: Nm_vs, NLG_ALL_C
      USE LOG_PARS,    ONLY: LOG_LEVEL, C_UNIT, DIR_OUT, LUNIT, N_DSO, 
     $                       N_ESO, EU
      USE ST_VARS,     ONLY: RTD => RAD_TO_DEG, PIE, RTM_CORE, RTM_TYPE,
     $                       WF_FLAG
      USE SPHER_PARS,  ONLY: ANG_OUT
      USE TOOLS,       ONLY: READ_FILE
      USE TYPE_VARS,   ONLY: KND
      USE WF_NUM,      ONLY: NUM_CLD
c     ------------------------------------------------------------------

      REAL (KND), INTENT(IN) ::  START_WAV, END_WAV
c     ... Local variables
      CHARACTER (LEN=24), PARAMETER :: R_NAME='GENERAL_CLOUD_INIT'
      CHARACTER (LEN=200) NAM_DB(5), ICE_WL_DB
      CHARACTER (LEN=1)   DUMCHAR, START
      CHARACTER (LEN=80)  NDB_Water, NDB_Ice
      Character (Len=107), Allocatable:: RecRead(:)
      Character (Len=103), Allocatable:: RecScm(:)
      PARAMETER (START  = '>')
      INTEGER  MAXMOM_LOC
      INTEGER  CDELTAM, NCL, RLN, N1, Max_DB, Max_ice, NCWI, N_1, P_1, 
     $     I, II, K, WVLS, WVLE, NM_WVL, Max_USE, IREAD, N_water, N_Ice, 
     $     Max_SL, INX, N_PHA, Ws, Ws1, N_MOM_USER, Max_ang, R_start, 
     $     R_end, Max_Bdb, Kma, NN, CDM, Max_sm
      LOGICAL  USE_WATER_WL
      INTEGER, ALLOCATABLE :: N_DB_MOM(:,:,:), N_US_MOM(:,:), N_DM(:)
      LOGICAL, ALLOCATABLE :: MASK(:,:,:)
      REAL (KND) DUM, G_SYM, TMP1, TMP2(Nm_vs), Coe(NLEGEN), F2, Fpar, 
     $     P_TETA, F_DM
      REAL (KND), ALLOCATABLE ::  M_WVL(:), R_COF(:),
     $     REF_WVL(:), REF_IMAG_LOC(:), TTT(:,:)              
      REAL (KND), ALLOCATABLE, TARGET :: MOM_DB(:,:,:,:,:,:)
      REAL (KND), POINTER :: MOM_USER(:), Phm_p(:,:)
      REAL (KND), EXTERNAL :: PLGNDR, Time_in_Seconds
c     =============================================================

      CALL ST_TRACK (R_NAME, 1, 'I') 
      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here
      IF( Use_Ydb.AND.(N_CLD_LAYERS==1).AND.TDS(1)%S=='ice' ) THEN
         MAXMOM_LOC = 1000
      ELSE
         MAXMOM_LOC = 20000
      ENDIF

c     ... N_PHA is set to 1 if there is not mixed cloud
c     -------------------------------------------------
      NDB_Ice = 'Crystals/'
      N_PHA = 1
      IF( ANY(Plus_ice) ) N_PHA = N_PHA + 1
c     ... Further path to DB of water and ice
      IF( IPRT_ALIN.AND.ALIN_ACT(4) ) THEN
         NAM_DB(1) = './B4/Water_wvl.dbs'
         NAM_DB(4) = './B4/Water_refr_idx.dat'
         NAM_DB(5) = './B4/Waren-Brandt_2008.dat'
      ELSE
         NDB_Water = 'Water/'
c         ... The following files will be used:
c             reference wavelength grid for water DB [micron]
         NAM_DB(1) = TRIM(CDB_PATH)//TRIM(NDB_Water)//'Water_wvl.dbs'
c       ... refractive index of water 
         NAM_DB(4) =
     $        TRIM(CDB_PATH)//TRIM(NDB_Water)//TRIM(Refr_water_inp)
c        ... refractive index of ice
         NAM_DB(5) = TRIM(CDB_PATH)//TRIM(NDB_Ice)//TRIM(Refr_ice_inp)
      ENDIF

c      WRITE(*,*) '  TRIM(CDB_PATH)  ', TRIM(CDB_PATH)
c      WRITE(*,*) '  TRIM(NDB_Water) ', TRIM(NDB_Water)
c      WRITE(*,*)  ' NAM_DB(1) ', NAM_DB(1) 
c      PAUSE
c     WRITE(*,*) '   NAM_DB(1) ', TRIM(CDB_PATH)//'Water_wvl.dbs'

      ICE_WL_DB   = '_wvl.dbs'

c     ... extinction, single scattering albedo and number  
c         of expansion coefficients
      NAM_DB(2) = '_ssa.dbs'
c     ... expansion coefficients
      NAM_DB(3) = '_moms.dbs'

c     ... Set reference wavelength grid for cloud optical parameters
c     --------------------------------------------------------------
      IF( TRIM(TDS(1)%S)=='alay' ) THEN
c        ...  for aerosol layer optical parameters
         OPEN(C_UNIT,FILE=USER_EK_NAME(1,1)%S,STATUS='OLD')
           CALL SKIP_HEADER( C_UNIT )
           READ( C_UNIT,'(i4)' ) NM_WVL
           ALLOCATE ( M_WVL(NM_WVL) ) ! A
           DO K = 1, NM_WVL
              READ( C_UNIT,* ) M_WVL(K)
           ENDDO
         CLOSE( C_UNIT )
         WVLS = 1  ;  WVLE = NM_WVL
         NCW = WVLE - WVLS + 1
      ELSE
         USE_WATER_WL = .TRUE.
c        ... for water clouds database
         IF( Red_WL_Ydb ) THEN
            NM_WVL = N_Red_WL ; ALLOCATE ( M_WVL(NM_WVL) )
            M_WVL(:) = Red_WL(:)
         ELSE
            OPEN(C_UNIT,FILE=TRIM(NAM_DB(1)),STATUS='OLD')
            READ( C_UNIT,'(i4)' ) NM_WVL
            ALLOCATE ( M_WVL(NM_WVL) ) ! A
            DO K = 1, NM_WVL
               READ( C_UNIT,* ) M_WVL(K)
            ENDDO
            M_WVL(:) = M_WVL(:)*1.D+03 ! Convert to [nm]
            CLOSE( C_UNIT )
         ENDIF
c        ... Find position of start- and end-wavelengths of current 
c            net in the reference wavelength array.
c        ----------------------------------------------------------
         CALL LIN_SEARCH_NEW(START_WAV, DUM, WVLS, Ws1, NM_WVL, M_WVL)
         CALL LIN_SEARCH_NEW(END_WAV,   DUM, Ws,  WVLE, NM_WVL, M_WVL)
c        ... Total number of actual reference wavelengths 
c        ------------------------------------------------
         NCW = WVLE - WVLS + 1
c         WRITE(*,*) '   NCW of CLD WL grid = ', NCW
      ENDIF
c        WRITE(*,*) '    Water :'
c        WRITE(*,*) '  Full number of WL in reference grid: ',NM_WVL 
c        WRITE(*,*) '           First WL in reference grid: ',M_WVL(1)
c     ... End of reading the reference wavelength grid
c     ------------------------------------------------

c     ... N_DB_MOM is defined for all wavelength grid in water DB
      ALLOCATE( N_DB_MOM(NM_WVL,N_CLD_LAYERS,N_ref_db), 
     $     MASK(NM_WVL,N_CLD_LAYERS,N_ref_db),
     $     CLD_NETWAVS(NCW), N_DM(N_CLD_LAYERS) ) ! A

c     ... The following setting is for the case if the DB phase matrixes
c         for water, ice, and sm-model are not presented:
c     ------------------------------------------------------------------
      N_DB_MOM = 1  ;  Max_ice = 1  ; Max_Bdb = 1 ;  Max_sm = 1

c     ... Maximal number of sub-layers in clouds
c     ------------------------------------------
      Max_SL = INT( MAXVAL( CLD_LAYER_NUM,1 ) ) + 3

      SET_Nre: Block
        IF( Use_Bdb ) THEN ; N1 = MAX( Max_SL,N_Ref_db,N_Ref_Bdb )
           N_Rdb = N_Ref_Bdb ; EXIT SET_Nre
        ENDIF
        IF( Use_Ydb ) THEN ; N1 = MAX( Max_SL,N_Ref_db,N_Ref_Ydb )
           N_Rdb = N_Ref_Ydb ; EXIT SET_Nre
        ENDIF
        N1 = MAX( Max_SL,N_Ref_db,N_Ref_HEX )
        N_Rdb = N_Ref_db
      END Block SET_Nre

      ALLOCATE( 
     $     CLD_LAYER_COEFF(N_CLD_LAYERS,NCW,2*N_PHA,N1), ! A
     $     SCC_LAYER_NOSCALE(N_CLD_LAYERS,NCW,N1) ) ! A   VR 16.02.11
      CLD_LAYER_COEFF = 0.d0

      IF( N_PHA == 2 )
     $     ALLOCATE( SCI_LAYER_NOSCALE(N_CLD_LAYERS,NCW,N1) )
      
      IF( WF_FLAG ) THEN
         ALLOCATE( CLD_LAYER_DERIV(N_CLD_LAYERS,NCW,2*N_PHA,N1) )
         CLD_LAYER_DERIV = 0.d0
      ENDIF
c     Extract actual wavelengths relevant to the required spectral range
c     ------------------------------------------------------------------
      CLD_NETWAVS(1:NCW) = M_WVL(WVLS:WVLE) 

!      WRITE(*,*) '   Number of CLD_NETWAVS  ', NCW
c      WRITE(*,'(*(2x,F8.2))') CLD_NETWAVS(1:NCW) 
c
c     ... Kma is maximal number of effective radii of water and/or
c         ice crystals used
      Kma = 1
      IF( .NOT.(IPRT_ALIN.AND.ALIN_ACT(4)) ) THEN
         DO I = 1, N_CLD_LAYERS
            K = MAXVAL( MLCS(I)%Npf(:)%Ndb_ref )
            IF( Kma <= K ) Kma = K
         ENDDO
      ENDIF
!      write(*,*) '    Selected maximal dimension Kma ', Kma
!      write(*,*) '    PAUSE in general_cloud_init.f '
!      PAUSE
      
      ALLOCATE( N_US_MOM(N_CLD_LAYERS,N_PHA), ! A
     $     C_P_M (NCW,NLEGEN,    N_CLD_LAYERS, Kma, N_PHA, Nm_vs), ! A
     $     MOM_DB(NCW,MAXMOM_LOC,N_CLD_LAYERS, Kma, N_PHA, Nm_vs), ! A
     $     CLD_F_DM(NCW, N_CLD_LAYERS, Kma, N_PHA) ) ! A
      MOM_DB = 0.d0  ;  C_P_M = 0.d0

      IF( Use_Ydb_SCM ) THEN
         ALLOCATE( SCAT_ANG(Ydb_ang),  
     $        SCAT_MAT(NCW,Ydb_ang,N_CLD_LAYERS,Kma,N_PHA,Nm_vs) )
         SCAT_MAT = 0.d0
      ENDIF

c     ... The following setting is for the case if the user phase matrixes 
c         are not present:
      N_US_MOM = 1

      IF( TRIM(HOW_CLD_COEF)=='analytically' ) CALL READ_REFR_IND

      IF( D_MF == 'D-F'.OR. D_MF == 'D-S' ) 
     $     ALLOCATE ( F_Dfit(N_CLD_LAYERS,2) )

c     ... Do-loop over all cloud layers in the atmosphere
c      WRITE(*,*) '         Calculation of MOM_DB for all layers '
      DO Ncl = 1, N_CLD_LAYERS
         CDELTAM  =  CLD_DM(Ncl)
c         ALLOCATE( MLCS(Ncl)%Npf(N_PHA) )
c         WRITE(*,*) '    =====> Cloud layer ', Ncl
         SELECT CASE( PHASE_NAME(Ncl)%S )
         CASE ( 'db','sm','hg' )
           SELECT CASE ( TDS(Ncl)%S )
              CASE('water')
                 N_1 = 1 ; P_1 = 1
c                 WRITE(*,*) '   Water Ncl, P_1 = ', Ncl, P_1
c                 CALL READ_DB_WATER( Nr_db(1) )
                 Nn = MLCS(Ncl)%Npf(P_1)%Ndb_ref
                 CALL READ_DB_WATER( Nn )
              CASE('ice-sm')
                 N_1 = 1 ; P_1 = 1
                 IF( ANY(Plus_ice) )  THEN 
                    N_1 = 3 ; P_1 = 2
                 ENDIF
c                ... MOM_DB is calculated accoding to stochastic model
                 IF( Nm_vs==1 ) THEN ;  CALL SM_Model(*880)
                 ELSE         ;  CALL SM_Model_V(Ncl, Nam_DB, N_1, *880)
                 ENDIF
              CASE('mixed-sm')
                 N_1 = 1 ; P_1 = 1
                 Nn = MLCS(Ncl)%Npf(P_1)%Ndb_ref
                 CALL READ_DB_WATER( Nn ) 
                 N_1 = 3 ; P_1 = 2
c                ... MOM_DB is calculated accoding to stochastic model
                 IF( Nm_vs==1 ) THEN ;  CALL SM_Model(*880)
                 ELSE         ;  CALL SM_Model_V(Ncl, Nam_DB, N_1, *880)
                 ENDIF
              CASE('ice')
                 N_1 = 1 ; P_1 = 1 
                 IF( ANY(Plus_ice) )  THEN 
                    N_1 = 3 ; P_1 = 2
                 ENDIF
c                 WRITE(*,*) '   Ice Ncl, P_1 = ', Ncl, P_1
                 Nn = MLCS(Ncl)%Npf(P_1)%Ndb_ref
                 Re1_DB: Block
                   IF( Use_Bdb ) THEN
c                      CALL READ_DB_BDB(Nr_db(2),*880) ; EXIT Re1_DB
                      CALL READ_DB_BDB(Nn,*880) ; EXIT Re1_DB
                   ENDIF
                   IF( Use_Ydb ) THEN
c                      CALL READ_DB_YDB(Nr_db(2),*880) ; EXIT Re1_DB
                      CALL READ_DB_YDB(Nn,*880) ; EXIT Re1_DB
                   ENDIF
                   CALL READ_DB_ICE(*880)
                 END Block Re1_DB
              CASE('mixed')
                 N_1 = 1 ; P_1 = 1
                 Nn = MLCS(Ncl)%Npf(P_1)%Ndb_ref
                 CALL READ_DB_WATER( Nn ) 
                 N_1 = 3 ; P_1 = 2
                 Nn = MLCS(Ncl)%Npf(P_1)%Ndb_ref
c                 write(*,*) '      Nn = ', MLCS(Ncl)%Npf(P_1)%Ndb_ref
                 Re2_DB: Block
                   IF( Use_Bdb ) THEN
c                      CALL READ_DB_BDB(Nr_db(2),*880) ; EXIT Re2_DB
                      CALL READ_DB_BDB(Nn,*880) ; EXIT Re2_DB
                   ENDIF
                   IF( Use_Ydb ) THEN
c                      CALL READ_DB_YDB(Nr_db(2),*880) ; EXIT Re2_DB
                      CALL READ_DB_YDB(Nn,*880) ; EXIT Re2_DB
                   ENDIF
                   CALL READ_DB_ICE(*880)
                 END Block Re2_DB
              END SELECT
         END SELECT  ! for 'db', 'sm', 'hg' cases
            
         IF(CDELTAM .GT. 0) THEN
            N_DM(NCL) = CDELTAM + 1
         ELSE
            N_DM(NCL) = NLEGEN
         END IF
         RLN = N_DM(NCL)
         
c        Read/compute phase function, depending on selected option
c        ---------------------------------------------------------
c         WRITE(*,*) '     Setting matrix C_P_M for Ncl = ', Ncl
         SELECT CASE( PHASE_NAME(Ncl)%S )
         CASE ('db','sm')
c            WRITE(*,*) MOM_DB(1,1,Ncl,1,1,1), MOM_DB(1,1,Ncl,1,2,1) 
            DO I = 1, NCW   
               C_P_M(I,1:RLN,Ncl,:,1:N_PHA,1:Nm_vs) = 
     $              MOM_DB(I,1:RLN,Ncl,:,1:N_PHA,1:Nm_vs) 
            END DO
         CASE ('hg')            ! scalar case only
            DO I = 1, NCW
               DO II = 1,N_PHA
                  G_SYM = MOM_DB(I,2,Ncl,1,II,1) 
                  FORALL ( K=1:RLN )
     $                 C_P_M(I,K,Ncl,1,II,1) = G_SYM**DBLE(K-1) 
               ENDDO
            END DO
         CASE ('iso')           ! scalar case only
            DO I = 1, NCW
               C_P_M(I,1,    Ncl,1,1:N_PHA,1) = 1.D0 
               C_P_M(I,2:RLN,Ncl,1,1:N_PHA,1) = 0.0D0 
            END DO
         CASE ('user')
c          ... Read user cloud phase-function ( expansion coefficients )
c              for all cloud layers
c          -------------------------------------------------------------
           SELECT CASE ( TRIM(TDS(NCL)%S) )
           CASE('alay')
              INX = 1 ; N_1 = 1 ; P_1 = 1 ; IREAD = 1
              CALL READ_PHASE_AEROSOL(*880)              
           CASE('water')
              INX = 1 ; N_1 = 1 ; P_1 = 1 ; IREAD = 1
              CALL READ_PHASE_LEGEN_USER(*880)
           CASE('ice')
              IREAD = 2
              IF( ANY(Plus_ice) ) THEN
                 INX = 2 ; N_1 = 3 ; P_1 = 2
              ELSE
                 INX = 1 ; N_1 = 1 ; P_1 = 1
              ENDIF
              CALL READ_PHASE_LEGEN_USER(*880)
           CASE('mixed','mixed-sm')
c             ... Reading for water
              INX = 1 ; N_1 = 1 ; P_1 = 1 ; IREAD = 1
              CALL READ_PHASE_LEGEN_USER(*880)
c             ... Reading for ice
              INX = 2 ; N_1 = 3 ; P_1 = 2 ; IREAD = 2
              CALL READ_PHASE_LEGEN_USER(*880)
           END SELECT
           N_MOM_USER = N_US_MOM(NCL,1)
c          ------ case "user"------------------------
         END SELECT           
c        ... Reflection for infinity cloud without abssorbtion
c            =================================================
         IF( TRIM(RTM_CORE) == "ASYMP" ) CALL ASYMPTOTIC_INF

      ENDDO  ! over cloud layers (NCL)

c     ... Save DB extinction and scattering coefficients if numerical
c         perturbation for any cloud parameter will be performed
c         or mode 'ret' (retrieval) will be used
      IF( NUM_CLD .OR. RET_FLAG) THEN
         ALLOCATE( CLD_SAVE_COEFF(N_CLD_LAYERS,NCW,2*N_PHA,N_Rdb) )
         CLD_SAVE_COEFF(:,:,:,:) = CLD_LAYER_COEFF(:,:,:,1:N_Rdb)
      ENDIF

      MASK = .FALSE.
      MASK(WVLS:WVLE,1:N_CLD_LAYERS,1:N_ref_db) = .TRUE.

c     ... Maximal number of moments from water data base
      Max_DB  = MAXVAL( N_DB_MOM,MASK )
c     ... Maximal number of moments for user defined matrices
      Max_Use = MAXVAL( N_US_MOM )

c      WRITE(*,*) Max_DB, Max_Use, Max_ice, Max_sm, Max_Bdb
      NG_cld  = MAX( Max_DB,Max_Use,Max_ice,Max_sm,Max_Bdb )
      
!     ... NLG_ALL_C will be set to NG_cld in "dom_spherical.f90", if
!         DO_SSRCA.or.RTM_TYPE=='spher_scat'.or.SCATT_MODE =='ss'
      NLG_ALL_C = 1

      IF( LOG_LEVEL(1) >= N_DSO ) THEN
         WRITE(LUNIT,*) ' **** Max number of moments from DB for water'
         WRITE(LUNIT,*) '      for the cloud phase functions: ', Max_DB

         WRITE(LUNIT,*) ' **** Max number of moments from Baum DB'
         WRITE(LUNIT,*) '      for the cloud phase functions: ', Max_Bdb

         WRITE(LUNIT,*) ' **** Max number of user moments '
         WRITE(LUNIT,*) '      for the cloud phase functions: ', Max_USE

         WRITE(LUNIT,*) ' **** Max number of moments from DB for ice'
         WRITE(LUNIT,*) '      for the cloud phase functions: ', Max_ice
      
         WRITE(LUNIT,*) '     Ng_cld = ', NG_cld
      ENDIF

c     ... Save all moments of the scattering matrixes
c     -----------------------------------------------
      IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat') THEN
c         write(*,*) NCW, NG_cld, N_CLD_LAYERS, Kma, N_PHA, Nm_vs
         ALLOCATE( Greek_cld(NCW,NG_cld,N_CLD_LAYERS,Kma,N_PHA,Nm_vs) )
         Greek_cld(:,1:Ng_cld,:,:,:,:) = MOM_DB(:,1:Ng_cld,:,:,:,:)
      ENDIF
     
c     ... Truncation of the user-defined phase function 
      DO NCL = 1, N_CLD_LAYERS
         IF( PHASE_NAME(NCL)%S /= 'user' ) CYCLE
         RLN = N_DM(NCL)
         SELECT CASE ( TRIM(D_MF) )
         CASE('D-F')            ! for the scalar case only
            N_MOM_USER = NG_cld
            IF(NCL > 1) DEALLOCATE( R_COF )
            ALLOCATE ( R_COF(RLN-1) ) 
            DO K = 1,N_PHA
               MOM_USER => MOM_DB(1,1:NG_cld,NCL,1,K,1)
               CALL Delta_Sfit( Ng_cld, 1800 )
               WRITE(*,*) '  F_Dfit(NCL,K) = ',NCL, K
               F_Dfit(NCL,K) = 1.d0 - R_COF(1)
               WRITE(*,*) '  *** save in CLD_PHAS_MOMS '
               DO I = 1, NCW 
                  C_P_M(I,1:RLN-1,Ncl,1,K,1) = R_COF(:) 
                  C_P_M(I,    RLN,Ncl,1,K,1) = 0.0d0
               ENDDO
               WRITE(*,*) '    End save '
            ENDDO
         CASE('D-S')            ! for the scalar case only
            N_MOM_USER = NG_cld
            IF(NCL > 1) DEALLOCATE( R_COF )
            ALLOCATE ( R_COF(RLN) ) 
            DO K = 1,N_PHA
               MOM_USER => MOM_DB(1,1:NG_cld,NCL,1,K,1)
c                 Max_ang = MAX( 2000,Ng_cld )
               Max_ang = MAX( 8000,Ng_cld )
c               WRITE(*,*) '  Ng_cld = ', Ng_cld

               CALL Delta_S( Ng_cld, Max_ang )
               
               WRITE(*,*) '  S_Dfit(NCL,K) = ',NCL, K
               F_Dfit(NCL,K) = F2
               F_Dfit(NCL,2) = Fpar ! second dimenstion is used for test
               WRITE(*,*) '  *** save in CLD_PHAS_MOMS RLN = ', RLN
               DO I = 1, NCW 
                  C_P_M(I,1:RLN,Ncl,1,K,1) = Coe(:) 
               ENDDO
               WRITE(*,*) '    End save in Delta-S'
            ENDDO
         CASE('D-M') 
            IF( IPRT_ALIN ) THEN
               CDM  = CLD_DM(1)
               Phm_p => MOM_DB(1,:,1,1,1,1:Nm_vs)
               F_DM = Phm_p(CDM+1,1)/DBLE(2*CDM + 1)
c              ... Redefine moments of truncated phase function
c                  for all basis phase functions
               FORALL( K = 1:CDM ) Phm_p(K,1) = 
     $              ( Phm_p(K,1) - F_DM*DBLE(2*K-1) )/(1.d0 - F_DM)
               FORALL( K = 1:CDM ) 
               Phm_p(K,2:4) = (Phm_p(K,2:4)
     $              - F_DM*DBLE(2*K-1))/(1.d0 - F_DM)
               Phm_p(K,5:6) = Phm_p(K,5:6)/(1.d0 - F_DM)
               END FORALL
               CLD_F_DM(:,1,1,1) = F_DM
               Phm_p( CDM+1:NLEGEN,: ) = 0.d0
            ENDIF  ! IPRT test including cloud
            DO I = 1, NCW
               C_P_M(I,1:RLN,Ncl,:,1:N_PHA,1:Nm_vs) = 
     $              MOM_DB(1,1:RLN,Ncl,:,1:N_PHA,1:Nm_vs) ! vector
            ENDDO
         END SELECT
      ENDDO  ! over cloud layers (NCL)

c      STOP '   Stop in GENERAL_CLOUD_INIT before RETURN'
      CALL ST_TRACK (R_NAME, 1, 'O') 
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS

C     ******************************************************************
c       Reading imaginary part  of water and ice refractive index 
c                          from database
C     ******************************************************************
      SUBROUTINE READ_REFR_IND
c     Open file for water refractive index
c     ====================================
      OPEN(C_UNIT,FILE = TRIM(NAM_DB(4)), STATUS='old')
         CALL SKIP_HEADER( C_UNIT )
         Read( C_UNIT,* ) N_water
         ALLOCATE ( REF_WVL(N_water),REF_IMAG_LOC(N_water) )
         DO K = 1, N_water
            READ( C_UNIT,* ) REF_WVL(K), DUM, REF_IMAG_LOC(K)
         ENDDO
      CLOSE( C_UNIT )
      ALLOCATE ( REF_IMAG_W(NCW) )
      REF_WVL(:) = REF_WVL(:)*1000.0d0 ! Transform from [mkm] => [nm]
c      WRITE(*,*) '   End reading ', TRIM(NAM_DB(4))

c     Interpolation imaginary part on the actual wavelength grid
c     ==========================================================
      Call LIN_INT (N_water, REF_WVL, REF_IMAG_LOC,
     $     NCW, CLD_NETWAVS, REF_IMAG_W, 'Z')

c     Open file for ice refractive index
c     ==================================
      OPEN(C_UNIT,FILE = TRIM(NAM_DB(5)), STATUS='old')
         Read( C_UNIT,* ) N_Ice
         IF( N_Ice > N_water ) THEN
            DEALLOCATE( REF_WVL, REF_IMAG_LOC )
            ALLOCATE ( REF_WVL(N_Ice), REF_IMAG_LOC(N_Ice) )
         ENDIF
         DO K = 1, N_Ice
            READ( C_UNIT,* ) REF_WVL(K), DUM, REF_IMAG_LOC(K)
         ENDDO
      CLOSE( C_UNIT )
c                WRITE(*,*) '   End reading "ice....dat"'
      ALLOCATE ( REF_IMAG_I(NCW) )
      REF_WVL(:) = REF_WVL(:)*1000.0d0 ! Transform from [mkm] => [nm]

c     Interpolation imaginary part on the actual wavelength grid
c     ==========================================================
      Call LIN_INT (N_ice, REF_WVL, REF_IMAG_LOC,
     $     NCW, CLD_NETWAVS, REF_IMAG_I, 'Z')

      END SUBROUTINE READ_REFR_IND
      
c     ******************************************************************
c           Calculate phase function of ice layer using
c             stochastic mixture model (scalar version)
c     ******************************************************************
      SUBROUTINE SM_MODEL(*)
      USE INTERN_DIMS, ONLY: Nvs
      USE CLD_INP,     ONLY:  ADD_ABS, MLCS
      USE CLOUDS,      ONLY: CLDPROFILE, Ref_ip, RefI_r, RefI_i
c     -----------------------------------------------------------------
      INTEGER I, J, K, L, N_ANG, Ltc, Iph
      REAL (KND) STEP, R_ri(2),  Rout_int, One_out,
     $     Rr, U1, D1, T2, T3, Td, x, omeg
      REAL (KND), ALLOCATABLE::  ANG_i(:), As_tout(:), Tout(:,:,:),
     $     Rout(:,:,:), Rin(:,:,:), Ft(:), Cas_tout(:), 
     $     Ca_i(:), Tet_t(:), Ctet_t(:), Ft0(:), 
     $     Rout_ang(:)
      INTEGER, PARAMETER :: N_gau = 500, Mum = 500
      INTEGER  Na_st, Nc_fin(NCW)
      LOGICAL  Error
      REAL (KND)  Poly(MUM,N_gau), Omega(NCW,1), Alph(NCW,1),
     $     Yel_conc(1)
      REAL (KND)  Abs_yel, Alpha, Norm_fin, XS, XE,
     $     WL_loc, Yel_390
      REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, Ft_gau,
     $     Fin_gau, Rout_gau, Mult, Mult2, Pht, Del,
     $     Ph_t2, Ph_fin
      REAL (KND), ALLOCATABLE :: Sa_inp(:), Sa_rad(:),
     $     Fin_inp(:), t2_Cof(:), to_cof(:), ro_cof(:), f_cof(:),
     $     DATA_R(:,:), REFI_WVL(:)
c     =================================================================
      
c     ... Refractive index of ice (WL input [micron])
c -----------------------------------------------------------------------
      XS = CLD_NETWAVS(1)  /1.d03     !  [nm] ==> [microm] 
      XE = CLD_NETWAVS(NCW) /1.d03
      CALL READ_FILE(3, NAM_DB(5), DATA_R, *880, XS, XE,
     $     NPT = 1, Skip_line = 1)
      N_ice = Size(DATA_R,1)
      ALLOCATE ( REFI_WVL(N_Ice),RefI_r(NCW),RefI_i(NCW)  )
      REFI_WVL(:) = DATA_R(:,1)         ! wavelength in [micron] at this step
      REFI_WVL(:) = REFI_WVL(:)*1000.0d0 ! Transform from [mkm] => [nm]      
      Call LIN_INT (N_ice, REFI_WVL, DATA_R(:,2),
     $     NCW, CLD_NETWAVS, RefI_r, 'Z')
      Call LIN_INT (N_ice, REFI_WVL, DATA_R(:,3),
     $     NCW, CLD_NETWAVS, RefI_i, 'Z')
      DEALLOCATE( DATA_R )
c      DO I = 1, NCW
c         WRITE(*,'(2x,F8.2,*(2x,e12.4))') CLD_NETWAVS(I),
c     $        RefI_r(I), RefI_i(I)
c      ENDDO
c      Pause '   Pause after reading of refractive index of ice'
c -----------------------------------------------------------------------
      N_ANG = 181
c     ... Gauss grid and weights for integration (-1,1)
      CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
c     ... Legendre polynomials for all Gauss angles
      CALL WIGNER_PLG( X_gau, N_gau, 0, MUM, Poly )
      X_rad(:) = DACOS( X_gau(N_gau:1:-1) )
      Na_st = 181
      ALLOCATE( Sa_inp(Na_st), Sa_rad(Na_st), Fin_inp(Na_st),
     $     t2_Cof(Mum), to_Cof(Mum), ro_Cof(Mum), f_Cof(Mum), ! A
     $     Tet_t(Na_st), Ctet_t(Na_st), Rout_ang(Na_st) )     ! A

      to_Cof = 0.d0 ;  ro_Cof = 0.d0
      
c     ... Equidistant grid of scattering angles 
      STEP = 180.d0/DBLE( Na_st-1 ) ; Sa_inp(1) = 0.d0
      DO I = 2,Na_st
         Sa_inp(I) = Sa_inp(I-1) + STEP
      ENDDO
c     ... Convert [deg] to [rad] for interpolation of the ph 
      Sa_rad(:) = Sa_inp(:)/RTD
c     ... Auxiliary functions for Legendre series
      DO I = 1, MuM
         Mult2(I) = (2.d0*DBLE(I) - 1.d0)/2.d0
         Mult(I)  = (2.d0*DBLE(I) - 1.d0)
      ENDDO
      
c     ... Vertically homogeneous monodisperse snow layer
c      WRITE(*,*)  '  D at the top ', CLDPROFILE(Ncl,1,7), ' [micron]'
c      WRITE(*,*) '  SHAPE(CLDPROFILE) ', SHAPE(CLDPROFILE)
c      WRITE(*,'(*(1x,e12.4))') CLDPROFILE(Ncl,:,7)
c      WRITE(*,*) SIZE( CLDPROFILE(Ncl,:,7) )

c     ... Incident angles in the case of internal reflection corresponding
c         to the scattering angles [0,180]
      Tet_t(:)  = (180.d0 - Sa_inp(:))/2.d0
      Ctet_t(:) = DCOS( Tet_t(:)/180.d0*Pie )

      ALLOCATE( ANG_i(N_ANG), Tout(N_ANG,Nvs,Nvs),
     $     As_tout(N_ANG), Cas_tout(N_ANG), Ca_i(N_ANG), 
     $     Rout(N_ANG,Nvs,Nvs), Ft(Na_st), Ft0(N_ANG),
     $     Rin(Na_st,1,1) )
      
c     ... Equidistant grid of incident angles air-ice interface [0,90]
      STEP = 90.d0/DBLE( N_ANG-1 ) ; ANG_i(1) = 0.d0         
      DO I = 2,N_ANG
         ANG_i(I) = ANG_i(I-1) + STEP
      ENDDO

c     ... Assuming constant over snow layer yellow matter absorption coefficient
      Yel_390 = 0.d0
      IF( ADD_ABS(Ncl) ) 
     $     Yel_390 = MLCS(Ncl)%C_SOOT(1) *1.d-9 ! m^-1 ==> nm^-1      
      Yel_conc(1) = Yel_390
      CALL OMEGA_SM( NCW, CLD_NETWAVS, RefI_r, RefI_i, NCL, 1,
     $     CLDPROFILE(Ncl,1,7), Omega, Alph, Yel_conc )
      
      Ca_i(:) = DCOS( ANG_I(:)/180.*Pie )
      DO II = 1, NCW
c       ... Setting of refractive index
         R_ri(1) = RefI_r(II)  ;  R_ri(2) = RefI_i(II)         
c        ... Scattering angles corresponding to the Fresnel refraction
c            air-ice interface at incident angles [0,Pi/2]
         DO I = 1,N_ANG
            As_tout(I) = ANG_i(I)
     $           - DASIN( DSIN(ANG_i(I)/180.d0*Pie)/R_ri(1) )/Pie*180.d0
         ENDDO
!         Ca_i(:) = DCOS( ANG_I(:)/180.*Pie )
         
!        ... Fresnel transmission air-ice at incident angles [0,Pi/2]
!            Tout[N_ANG,Nvs,Nvs]
         CALL Fre_Taw_v(N_ANG, Tout, Ca_I, R_ri)
!        ... Fresnel reflection air-ice at incident angles [0,Pi/2]
!            Rout[N_ANG,Nvs,Nvs]
         CALL Fre_Ref_v(N_ANG, Rout, Ca_I, R_ri)
!        ... Fresnel internal reflection at incident angles [0,Pi/2]
!            with values equal to 1 for total internal reflection
         CALL Fre_tot(Na_st, Rin, Ctet_t, R_ri)
      
         Cas_tout(:) = DCOS( As_tout(:)/180.d0*Pie )
         Ft0(:) = 0.d0

         DO I = 1, N_ANG
            Ft0(I) = Tout(I,1,1)
     $           *R_ri(1)**2*( R_ri(1)*Cas_tout(I) - 1.d0 )
     $           *( R_ri(1)- Cas_tout(I) )/Pie
     $           /( R_ri(1)**2 - 2.d0*R_ri(1)*Cas_tout(I) + 1.d0 )**2
         ENDDO
c        ... Interpolate function Ft0 for scattering angles between
c            [0, 180] with zero values for scattering angles > crit.
         CALL LIN_INT(N_ang, As_tout, Ft0, Na_st, Sa_inp, Ft, 'Z')         
!         DO I = 1, Na_st
!            write(*,*) I, Ft(I)
!         ENDDO
         Fin_inp(:) = Rin(:,1,1) /4.d0/Pie
c        ... Interpolate scattering function in the case of internal 
c            reflection on the Gauss grid 
         CALL AKIMA_INTERP( Na_st, Sa_rad, Fin_inp,
     $        N_gau, X_rad,  Fin_gau, Error)  
         IF( Error ) GOTO 201
c        ... Invert to start from negative value
         Fin_gau(:) = Fin_gau(N_gau:1:-1)
      
c        ... Expansion coefficients of the intrpolated phase function
         Pht(:) = 0.d0
         DO L = 1,MUM
            ro_cof(L)  = SUM( Fin_gau(:)*Poly(L,:)*AG_gau(:) )*2.d0*Pie
c            WRITE(*,*), L, ro_cof(L)
            DO J = 1,N_gau
               Pht(J) = Pht(J)
     $              + Poly(L,J)*ro_cof(L)*Mult2(L)/2.d0/Pie
            ENDDO
            Del(:) = DABS( Fin_gau(:) - Pht(:) )/Fin_gau(:)*100.
            IF( MAXVAL( Del(:) ) <= 0.1d0 ) EXIT
         ENDDO  ! over expansion coefficients (L)
      
c     ... Interpolate scattering function Ft on the Gauss grid
         CALL AKIMA_INTERP( Na_st, Sa_rad, Ft,
     $                      N_gau,  X_rad, Ft_gau, Error)  
         IF( Error ) GOTO 201
c     ... Invert to start from negative values
         Ft_gau(:) = Ft_gau(N_gau:1:-1)

c     ... Expansion coefficients of the intrpolated phase function Ft
         Pht(:) = 0.d0
         DO L = 1,MUM
            to_cof(L) = SUM( Ft_gau(:)*Poly(L,:)*AG_gau(:) )*2.d0*Pie
            DO J = 1,N_gau
               Pht(J) = Pht(J)
     $              + Poly(L,J)*to_cof(L)*Mult2(L)/2.d0/Pie
            ENDDO
            Del(:) = DABS(Ft_gau(:) - Pht(:))
            IF( MAXVAL( Del(:) ) <= 1.d-8 ) EXIT
         ENDDO
         Ltc = L
         
         WL_loc = CLD_NETWAVS(II) ;  Abs_yel = 0.d0 
         IF( ADD_ABS(Ncl) ) THEN
            IF( WL_loc <= 500.d0 ) THEN
               Abs_yel = Yel_390*DEXP( -1.5d-2*(WL_loc - 390.d0) ) 
            ELSE
               Abs_yel = Yel_390
     $              *DEXP(-1.5d-2*110.d0 - 1.1d-2*(WL_loc- 500.d0))
            ENDIF
         ENDIF
         
cc        ... Set to zero in the case of non-absorbing media         
cc         Alpha = 0.d0
c         Alpha = (4.d0*Pie*R_ri(2)/WL_loc + Abs_yel) *Ref_ip
         
         Rr = R_ri(1)   ! real part of refraction index
         U1 = 2.0*(5.0*Rr**6 + 8.0*Rr**5 + 6.0*Rr**4 - 5.0*Rr**3 -Rr-1)
         D1 = 3.0*( Rr**3 +  Rr**2 + Rr + 1.0)*(Rr**4 - 1.0)
         T2 = Rr**2*(Rr**2 - 1.0)**2/( Rr**2 + 1.0)**3
     $        *DLOG( (Rr+1.0)/(Rr-1.0) )
         T3 = 8.0*Rr**4*(Rr**4 + 1.0)/(Rr**4-1.0)**2/(Rr**2 + 1.0)
     $        *DLOG( Rr )
         Td = U1/D1 + T2 - T3

!        ... in do loop over wavelengths
c         WRITE(*,*) '   In "sm_model" layer number & P_1 ', Ncl, P_1
c         WRITE(*,*) '   Number of effective radii ',
c     $        MLCS(Ncl)%Npf(P_1)%Ndb_ref
c         WRITE(*,'(*(1x,e12.4))')  MLCS(Ncl)%Npf(P_1)%Pdb_ref(:)
         DO Iph = 1, MLCS(Ncl)%Npf(P_1)%Ndb_ref
c            Ref_ip = CLDPROFILE(Ncl,1,7) *1.d3 ! [micron] in [nm]
            Ref_ip = MLCS(Ncl)%Npf(P_1)%Pdb_ref(Iph) *1.d3 ! [micron] in [nm]
c            WRITE(*,*) '  Iph & Mean chord ', Iph, Ref_ip
c           ... Set to zero in the case of non-absorbing media         
c            Alpha = 0.d0
            Alpha = (4.d0*Pie*R_ri(2)/WL_loc + Abs_yel) *Ref_ip
c            x = Rr**2* Alpha
c            Omeg = 1.0 - x*Td/(x + Td)         
cc        ... Save extinction and scattering coefficients corresponding
cc            to the unit volume concentration
c         WRITE(*,*) '  Shape(CLD_LAYER_COEFF)  ',Shape(CLD_LAYER_COEFF)
c         CLD_LAYER_COEFF(NCL,II,N_1,1) = 1.d0/CLDPROFILE(Ncl,1,7)*1.d9 ! [km-1] 
c         CLD_LAYER_COEFF(NCL,II,N_1+1,1) =
c     $        1.d0/CLDPROFILE(Ncl,1,7)*Omeg*1.d9 
            DO J = 1, Ltc
               t2_cof(J) = to_cof(J)**2
     $              /(1.d0 + Alpha - ro_cof(J)) /R_ri(1)**2 *Mult(J)
            ENDDO
            DO J = 1,N_gau
               Ph_t2(J) = SUM( Poly(1:Ltc,J)*t2_cof(1:Ltc) )
            ENDDO
c          ... Start from 1 to -1 
            Rout_ang(:) =  Rout(Na_st:1:-1,1,1)
c          ... Interpolate scattering function Ft on the Gauss grid
            CALL AKIMA_INTERP( Na_st, Sa_rad, Rout_ang,
     $           N_gau, X_rad,  Rout_gau, Error)  
            IF( Error ) GOTO 201
c          ... Invert to start from negative values
            Rout_gau(:) =  Rout_gau(N_gau:1:-1)

            Ph_fin(:) = Rout_gau(:) + Ph_t2(:)
            Norm_fin = SUM( Ph_fin(:)*AG_gau(:) )*0.5d0
            Ph_fin(:) = Ph_fin(:)/Norm_fin
      
c         write(*,*) '  Norm_fin = ', Norm_fin
c         write(*,*) '  Asymmetry parameter (numerical)'
c         write(*,*) SUM( Ph_fin(:)*AG_gau(:)*X_gau(:) )*0.5d0

c           ... Expansion coefficients of the final phase function
            Pht(:) = 0.d0
            DO L = 1,MUM
               f_cof(L) = Mult2(L)*SUM( Ph_fin(:)*Poly(L,:)*AG_gau(:) )
               DO J = 1,N_gau
                  Pht(J) = Pht(J) + Poly(L,J)*f_cof(L)
               ENDDO
               Del(:) = DABS((Ph_fin(:) - Pht(:)))/Ph_fin(:)*100.d0
               IF( MAXVAL( Del(:) ) <= 0.1d0 ) THEN
                  MOM_DB(II,1:L,Ncl,Iph,P_1,1) = f_cof(1:L)
                  EXIT
               ENDIF
            ENDDO  ! over expansion coefficients (L)
            IF( L == MuM ) 
     $           MOM_DB(II,1:L,Ncl,Iph,P_1,1) = f_cof(1:L)
c            WRITE(*,*) '  SUM(MOM_DB(II,1:L,Ncl,Iph,P_1,1))',
c     $           SUM(MOM_DB(II,1:L,Ncl,Iph,P_1,1))
            Nc_fin(II) = L
            IF( II==1.AND.Iph==1 ) THEN         
               CALL WRITE_PHM(N_gau, X_gau, Ph_fin, Pht )
            ENDIF
         ENDDO  ! over effective radius (mean chord) (Iph)
      ENDDO  ! over cloud wavelength grid (II)

c     WRITE(*,*) '  SIZE(MOM_DB(1,1,1,:,1,1)', SIZE(MOM_DB(1,1,1,:,1,1))
      Max_sm = MAXVAL( Nc_fin )
c      WRITE(*,*) '   Max number of moments: ', Max_sm
c      stop '   Stop in "general_cloud_init" '

      RETURN
      
 201  Call WRITE_MESSAGE(R_NAME,'ERROR', 'Interpolation error')
      WRITE(EU,'(A,1x,2(2x,I3))') '   Subroutine "akima_interp" '
      RETURN 1     
      
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
      
      END SUBROUTINE SM_MODEL
c **********************************************************************
      
      SUBROUTINE WRITE_PHM(N_gau, X_gau, Ph0, Ph1 )
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------      
      INTEGER J, N_gau
      REAL (KND), DIMENSION(N_gau):: X_gau, Ph0, Ph1
c     ==================================================================
      OPEN( 44,FILE='./DATA_OUT/Fin_out.dat' )
         WRITE(44,'(a)') '#  Interpolated original and '//
     $     'recalculated phase functions'
        DO J = 1,N_gau
           WRITE(44,'(1x,3(e15.7))') 
     $          DACOS(X_gau(J))*RTD, Ph0(J), Ph1(J)
        ENDDO
        CLOSE(44)
        RETURN
      END SUBROUTINE WRITE_PHM
      
C     ******************************************************************
c           Reading optical characteristics of water clouds
c         from database (within mixed or water cloud layers)
c     ******************************************************************
      SUBROUTINE READ_DB_WATER( N_phr )
      INTEGER     N_phr, J, JJ, I_tmp
      REAL (KND)  MIE_EXT(NM_WVL), MIE_SSA(NM_WVL), DUM1, DUM2
      CHARACTER (LEN=:), ALLOCATABLE::  FULL_PATH, CDB_W
      CHARACTER (LEN=6)   F_name
      CHARACTER (LEN=80)  Ref_val(N_phr)

c        Reading of ext. coef. and  ssa. for clouds mie-data
c        Files "Greek-<Reff>_ssa.dbs", Reff-effective radius in [micron]
c               contain 3 columns
c               1 - extinction coefficient
c               2 - single scattering albedo
c               3 - number of expansion coefficients for the scattering 
c                   matrix
c        ---------------------------------------------------------------
      F_name = 'Greek-'
      CDB_W = TRIM(CDB_PATH)//'Water/'
c     ... Reading scattering and extinction coefficients from water DB
c     ----------------------------------------------------------------
      DO J = 1, N_ref_db
         FULL_PATH = TRIM(CDB_W)//TRIM(F_name)//TRIM(S_name(J))
     $        //TRIM(NAM_DB(2))
c                   WRITE(*,*) '--> Reading :', J,
c     $                 TRIM(F_name)//TRIM(S_name(J))//TRIM(NAM_DB(2))
c                    WRITE(*,*) '            :', TRIM(FULL_PATH)
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
           DO K = 1,NM_WVL
              READ(C_UNIT,'(2(D14.7),I5)') 
     $             MIE_EXT(K), MIE_SSA(K), N_DB_MOM(K,NCL,J)
           ENDDO
         CLOSE( C_UNIT ) 
c        ... Remove wrong multiplier in extended water database
c            for Ref=2 and Ref>20 mkm in order to represent 
c            extinction cross-section in [mkm^2]
         IF( J==1.OR.J>10 ) MIE_EXT(:) = MIE_EXT(:)*1.d+03

c        ... Save extinction coefficient from water database
c        ---------------------------------------------------
         CLD_LAYER_COEFF(NCL,1:NCW,1,J) = MIE_EXT(WVLS:WVLE) 
c        ... Save scattering coefficient from water database
c        ---------------------------------------------------
         CLD_LAYER_COEFF(NCL,1:NCW,2,J) = MIE_EXT(WVLS:WVLE)
     $        *MIE_SSA(WVLS:WVLE) 
      ENDDO  ! over effective radius in DB (J)

      IF( MLCS(Ncl)%Phm_co ) THEN
c        ... Reading the number of moments in the phase function
c            expansion for a given input effective radius water droplets
         FULL_PATH = TRIM(CDB_W)//DBC_NAME(NCL,1)%S
     $        //TRIM(NAM_DB(2))
c                WRITE(*,*) '  N_MOMS water:', TRIM(FULL_PATH)
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         DO K = 1,NM_WVL
            READ(C_UNIT,'(2(D14.7),I5)') 
     $           DUM1, DUM2, N_DB_MOM(K,NCL,1)
         ENDDO
         CLOSE( C_UNIT ) 
c        ... After this reading will be used N_DB_MOM(K,NCL,1) only
c        ... Copy file name in the variable Ref_val
         Ref_val(1) = DBC_NAME(NCL,1)%S
      ELSE
c        ... The moments filenames for all effective radius in DB
         DO J = 1, N_phr
            Ref_val(J) = TRIM(F_name)//S_name(J)
         ENDDO
      ENDIF  ! MLCS(Ncl)%Phm_co

c        Reading moments of the phase matrixes in DB for water particles
c        ---------------------------------------------------------------
      DO J = 1,N_phr 
         FULL_PATH = TRIM(CDB_W)//TRIM(Ref_val(J))//TRIM(NAM_DB(3))
c                WRITE(*,*) 
c     $          '   Reading: ',TRIM(TRIM(Ref_val(J))//TRIM(NAM_DB(3)))
         OPEN(C_UNIT,FILE = TRIM(FULL_PATH),STATUS='OLD',RECL=500000)
            R_start = SUM( N_DB_MOM(1:WVLS-1,NCL,J) )
            R_end   = SUM( N_DB_MOM(1:WVLE,NCL,J) )
c           Skip moments for all wavelengths before actual
c           ----------------------------------------------
            DO K = 1,R_start ; READ( C_UNIT,* ) ; ENDDO 
c           ----------------------------------------------
            II = 0
            DO K = WVLS, WVLE  ;  II = II + 1
c              ... Number of moments should be read ?
c               WRITE(*,*) ' .... Numbe of moments = ', J, K, NCL,
c     $              N_DB_MOM(K,NCL,J)
               DO I = 1,N_DB_MOM(K,NCL,J)
                  READ(C_UNIT,'(D11.4,I5,6(D14.7))') 
     $                 Tmp1, I_tmp, (TMP2(JJ), JJ=1,Nm_vs)
                  MOM_DB(II,I,Ncl,J,1,1:Nm_vs) = Tmp2(1:Nm_vs)
                  IF( I == 1 ) THEN
                     IF( I_tmp /= 1 ) THEN
                        STOP 'Error reading of water database'
                     ENDIF
                  ENDIF
               ENDDO
c            WRITE(*,*) '      WL, Ncl, Ref :', II, NCL, J
            ENDDO  ! over actual WL range (K,II)
            IF( Nm_vs > 1 ) THEN
               MOM_DB(:,:,Ncl,J,1,5) = -MOM_DB(:,:,Ncl,J,1,5)
               MOM_DB(:,:,Ncl,J,1,6) = -MOM_DB(:,:,Ncl,J,1,6)
            ENDIF
         CLOSE( C_UNIT ) 
      ENDDO  ! over effective radius of water droplets (J) 

      MASK = .FALSE. 
      MASK(WVLS:WVLE,1:N_CLD_LAYERS,1:N_ref_db)=.TRUE.
c     ... Maximal number of expansion coefficients within selected 
c         wavelength range of water DB
      N_MOM_USER = MAXVAL( N_DB_MOM,MASK )

c      WRITE(*,*) "     From 'general_cloud_init.f' "
c      WRITE(*,*) '   Max number of moments (water) :', N_MOM_USER
c      WRITE(*,*) '     '
c      JJ = N_DB_MOM(WVLS,1,1) 
c      CALL COEF_TO_SCATMATR(JJ, Nm_vs, MOM_DB(1,1:JJ,1,1,1,1:Nm_vs), 
c     $     180, 'E', './DATA_OUT/Ph_Water_wl=550_ref=40.dat')
c      STOP '  Stop after output of ice cloud scattering matrix'
      
      END SUBROUTINE READ_DB_WATER
      
c **********************************************************************
c           Reading optical characteristics of ice clouds
c                   from Yang database
c     ******************************************************************
      SUBROUTINE READ_DB_YDB(N_phr,*)

      USE CLOUDS,  ONLY: CLDPROFILE
      USE CLD_INP, ONLY:  N_Ydb, MLCS, Nf_ice, USE_PSD, 
     $                    Fd, CRST, Dim_Ydb, D_FU 

      INTEGER     N_phr, Nfm, Nph_lay
      INTEGER     I, J, K, KK, JJ, I_tmp, Nwl_x, Wls, Wle, Max_loc, Ifo,
     $     Nd_s, Nd_e, Na, Iw, J1s, J3
      CHARACTER (LEN=400)  FULL_PATH
      CHARACTER (LEN=100)  F_name, Dpi, Ex_nam, SSA_nam, NEC_nam, 
     $     EC_nam, SCM_nam
      CHARACTER (LEN=80), ALLOCATABLE :: Ref_x(:,:)
      INTEGER, ALLOCATABLE :: Nx_MOM(:,:,:), Jha(:)
      LOGICAL, ALLOCATABLE :: M_x(:,:,:)
      REAL (KND)  DUMM, S_WL, E_WL, DUM_READ(N_Ydb), D1, D2, tts1, tts2
      REAL (KND), ALLOCATABLE :: Lo_EXT(:,:,:), Lo_SSA(:,:,:), WL_x(:), 
     $     H_ext(:,:,:), H_ssa(:), DB_LOC(:,:,:,:,:), H_sca(:,:,:), 
     $     SCAT_LOC(:,:,:,:,:), DBI(:)
      TARGET :: DB_LOC, SCAT_LOC
      REAL (KND), POINTER :: ExPo(:), ScPo(:), MgP(:,:), SmP(:,:)
c    -------------------------------------------------------------------
c     ... Selected number of dimensions for cloud layer used
      Nfm = Nf_ice(Ncl)
      
      Ex_nam  = '_extinction_cross_section.dat'
      SSA_nam = '_single_scattering_albedo.dat'
      NEC_nam = '_N-moms.dbs'
      EC_nam  = '_moms.dbs'
      SCM_nam = '_scat-mat.dbs'

      IF( Use_PSD ) THEN
         Nd_s = 1  ; Nd_e = N_Ydb
         Na  = N_Ydb
      ELSE
         Nd_s = MLCS(Ncl)%Nd_s
         Nd_e = MLCS(Ncl)%Nd_e
         Na   = MLCS(Ncl)%Na_Ydb 
      ENDIF

      IF( N_phr==1 ) THEN
         Nph_lay = 1
      ELSE
        Nph_lay = Na 
      END IF

c      WRITE(*,'(A,1x,I2)') ' ++++++ Cloud layer ++++++  ', Ncl
c      WRITE(*,*) '     Kma, N_phr, Na ', Kma, N_phr, Na
c      WRITE(*,*) '   SHAPE(Dim_Ydb)  ',  SHAPE(Dim_Ydb)
c      WRITE(*,'(2x,A,2(2x,I3))')
c     $     ' Range of ice crystal dimensions which will be read',
c     $     Nd_s, Nd_e
c      WRITE(*,*) '              NCW = ', NCW
c      WRITE(*,*) '      SHAPE(MOM_DB) ', SHAPE(MOM_DB) 
      
      ALLOCATE( Ref_x(Na,Nfm) )
      
c    ... Reading wavelength grid [nm] from DB
c     ------------------------------------------------------------------
      FULL_PATH = CDB_PATH//CRST(NCL)%SHP(1)%ICE_DIR//'_wavelength.dat'
c                 WRITE(*,*) '   Reading WL grid from selected DB: '
c                 WRITE(*,*) TRIM(Dpi)//TRIM(F_name)//'_wavelength.dat'
      OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         READ(C_UNIT,*) Nwl_x  ;  ALLOCATE( WL_x(Nwl_x) )
         DO J = 1, Nwl_x 
            READ(C_UNIT,*) WL_x(J)
         ENDDO
      CLOSE(C_UNIT)
c     ------------------------------------------------------------------

      ALLOCATE( Lo_EXT(Nwl_x,Na,Nfm), Lo_SSA(Nwl_x,Na,Nfm) )      
      S_WL = M_WVL(WVLS) ; E_WL = M_WVL(WVLE)
c                 WRITE(*,*) '  Wavelengths for RT :  ', WVLE-WVLS+1
c                 WRITE(*,*) S_WL, E_WL
c                 WRITE(*,*) 
      CALL LIN_SEARCH_NEW(S_WL, DUM, Wls, Ws1, Nwl_x, WL_x)
      CALL LIN_SEARCH_NEW(E_WL,  DUM, Ws, Wle, Nwl_x, WL_x)
c     ... Total number of actual wavelengths within DB
c     -----------------------------------------------------
      NCWI = Wle - Wls + 1
c                WRITE(*,*) '  Actual wavelengths in DB:  ', NCWI
c                WRITE(*,*) WL_x(Wls), WL_x(Wle)
c                WRITE(*,*) ' WLS and WLe: ', Wls, Wle
      ALLOCATE( DB_LOC(NCWI,MAXMOM_LOC,N_phr,Nm_vs,Nfm), DBI(Na) )

c      write(*,*) '    N_phr, Na ', N_phr, Na
c      write(*,*) '    SHAPE( DBI ) ',  SHAPE( DBI )
c      PAUSE
      
      DB_LOC = 0.d0
!     ... Scattering matrices will be used
      IF( Use_Ydb_SCM ) THEN
         ALLOCATE( SCAT_LOC(NCWI,Ydb_ang,N_phr,Nm_vs,Nfm) )
         SCAT_LOC = 0.d0
      ENDIF

c    ===> Reading all extinction coefficients in local array
c    -------------------------------------------------------------------
      Lo_EXT = 0.d0
        WRITE(*,*) '*****   Reading extinction coefficients: ',
     $     TRIM(CDB_PATH)
c        WRITE(*,*) '  Range from Nd_s to Nd_e ', Nd_s, Nd_e
      DO Ifo = 1, Nfm
         FULL_PATH = CDB_PATH//CRST(NCL)%SHP(Ifo)%ICE_DIR//TRIM(Ex_nam)
cc        -----------------------------------------------------------------
c         WRITE(*,'(1x,I2,1x,A)') Ifo, TRIM(FULL_PATH)
cc        -----------------------------------------------------------------
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         DO I = 1, Nwl_x 
            READ(C_UNIT,*) (DUM_READ(J), J = 1,N_Ydb) ! all dimensions
c           ... Select ext. coeff. in required dimension range:
            Lo_EXT(I,1:Na,Ifo) = DUM_READ(Nd_s:Nd_e)
         ENDDO
         CLOSE(C_UNIT)
      ENDDO  ! over selected ice habit forms (Ifo)
c    -------------------------------------------------------------------
c    ===> Reading single scattering albedo from DB in local array
c    -------------------------------------------------------------------
      WRITE(*,*) '*****   Reading SSA:  '
      Lo_SSA = 0.d0
      DO Ifo = 1, Nfm
        FULL_PATH = CDB_PATH//CRST(NCL)%SHP(Ifo)%ICE_DIR//TRIM(SSA_nam)
cc        -----------------------------------------------------------------
c         WRITE(*,'(1x,I2,1x,A)') Ifo, TRIM(FULL_PATH)
cc        -----------------------------------------------------------------
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
           CALL SKIP_HEADER( C_UNIT )
           DO I = 1, Nwl_x 
              READ(C_UNIT,*) (DUM_READ(J), J = 1,N_Ydb) ! all dimensions
c             ... Select SSA in required dimension range:
              Lo_SSA(I,1:Na,Ifo) = DUM_READ(Nd_s:Nd_e)
           ENDDO
         CLOSE(C_UNIT)
      ENDDO ! over selected ice habit forms (Ifo)
c     -------------------------------------------------------------------
c
c     ... Interpolate on the WL-grid of water cloud DB
c    -------------------------------------------------------------------
c           WRITE(*,*) 'WL water at the main grid ', CLD_NETWAVS(:)
      ALLOCATE( H_ext(NCW,Na,Nfm), H_sca(NCW,Na,Nfm), H_ssa(NCW) )
      DO J = 1, Na
         DO Ifo = 1, Nfm
            Call LIN_INT(Nwl_x, WL_x, Lo_EXT(:,J,Ifo), NCW, CLD_NETWAVS, 
     $           H_ext(:,J,Ifo), 'C')
            Call LIN_INT(Nwl_x, WL_x, Lo_SSA(:,J,Ifo), NCW, CLD_NETWAVS, 
     $           H_ssa, 'C')
            H_sca(:,J,Ifo) = H_ext(:,J,Ifo)*H_ssa(:)
         ENDDO  ! over ice habit forms (Ifo)
      ENDDO  ! over particle dimensions (J) (depends on cloud layer)
      DO J = 1, MLCS(Ncl)%Na_Ydb 
         ExPo =>  CLD_LAYER_COEFF(NCL,1:NCW,N_1,J)
         ScPo =>  CLD_LAYER_COEFF(NCL,1:NCW,N_1+1,J)

c        ... Save extinction and scattering coefficients from selected 
c            database interpolated on the water DB WL grid
         IF( Use_PSD ) THEN
            DO Iw = 1, NCW
               DO Ifo = 1, Nfm 
                  ExPo(Iw) = ExPo(Iw) +  SUM( H_ext(Iw,:,Ifo)
     $                 *CRST(Ncl)%SHP(Ifo)%Fr_hab*Fd(J,:) )
                  ScPo(Iw) = ScPo(Iw) 
     $                 +  SUM( H_sca(Iw,:,Ifo)
     $                 *CRST(Ncl)%SHP(Ifo)%Fr_hab*Fd(J,:) )
               ENDDO  ! over ice habit forms (Ifo)
            ENDDO  ! over actual wavelengths (Iw)
         ELSE
            DO Ifo = 1, Nfm
               DUMM = CRST(Ncl)%SHP(Ifo)%Fr_hab(J-1+Nd_s)
c               WRITE(*,'(2x,I2,2x,f8.2,3x,I2,2x,f6.3)')
c     $              J, D_FU(J-1+Nd_s), Ifo, DUMM
               ExPo(:) = ExPo(:) +  H_ext(:,J,Ifo)*DUMM
               ScPo(:) = ScPo(:) +  H_sca(:,J,Ifo)*DUMM
            ENDDO  ! over ice habit forms (Ifo)
         ENDIF
      ENDDO  ! over selected particle dimensions (J)
c    -------------------------------------------------------------------

      DEALLOCATE( H_ext, Lo_EXT, Lo_SSA )
      ALLOCATE( Nx_MOM(Nwl_x,Nfm,Na), M_x(Nwl_x,Nfm,Na) )

c     ===> Reading number of expansion coefficients in DB
c     ------------------------------------------------------------------
      WRITE(*,'(A,I3)')
     $  '*****   Reading number of moments for number of dimensions = ',
     $     Na
      DO J = 1, Na
         DO Ifo = 1, Nfm
            FULL_PATH = CDB_PATH//CRST(NCL)%SHP(Ifo)%ICE_DIR//
     $           '_'//TRIM(S_name_Ydb(NCL,J))//TRIM(NEC_nam)
c           ------------------------------------------------------
c            IF( J==1.OR.J==Na ) THEN
c               WRITE(*,'(1x,I3,1x,I2,1x,A)')  J, Ifo, TRIM(FULL_PATH)
c            ENDIF
c           ------------------------------------------------------
            OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
              DO K = 1,Nwl_x
                 READ(C_UNIT,*) DUMM, Nx_MOM(K,Ifo,J)
              ENDDO  ! over WL in DB (K)
            CLOSE( C_UNIT )
         ENDDO  ! over selected ice habit forms (Ifo)
      ENDDO  ! over particle dimensions (J) (depends on cloud layer)
c     ------------------------------------------------------------------
      
      M_x = .FALSE. ;  M_x(WLS:WLE,1:Nfm,1:Na) = .TRUE.
c     ... Maximal number of expansion coefficients within selected 
c         wavelength range of selected DB
      Max_loc = MAXVAL( Nx_MOM,M_x )
c      WRITE(*,*) '   Max_loc in Ydb: ', Max_loc

c      WRITE(*,*)
c     $     '*****   Select filenames for reading according to option'
c      WRITE(*,*) '      MLCS(Ncl)%Phm_co & Use_PSD   ', 
c     $     MLCS(Ncl)%Phm_co, Use_PSD
c      WRITE(*,*) '       Number of ice shapes ', Nfm
c      WRITE(*,*) '              Number of Na ', Na

      IF( MLCS(Ncl)%Phm_co.AND.(.NOT.Use_PSD) ) THEN
c     ... Vertically homogeneous monodisperse layer
c         WRITE(*,*)  '  D at the top ', CLDPROFILE(Ncl,1,7)
         DUMM = CLDPROFILE(Ncl,1,7)
         JJ = MINLOC( DABS(DUMM - Dim_Ydb(Ncl,:)),1 )
         J1s = JJ 
         J = 1
         DO Ifo = 1, Nfm
            Nx_MOM(:,Ifo,1) = Nx_MOM(:,Ifo,JJ)
            Ref_x(:,Ifo) = CRST(NCL)%SHP(Ifo)%ICE_DIR//
     $           '_'//S_name_Ydb(NCL,JJ)
!            WRITE(*,'(1x,I3,1x,I2,1x,A)') J, Ifo, TRIM(Ref_x(J,Ifo))
         ENDDO  ! over habit shapes (Ifo)
      ELSE
c     ... Vertically inhomogeneous or polydisperse layer
c        ... The moment filenames for all effective radius in DB
         J1s = 1 
         DO J = 1, Na
            DO Ifo = 1, Nfm
               Ref_x(J,Ifo) =
     $              CRST(NCL)%SHP(Ifo)%ICE_DIR//'_'//S_name_Ydb(NCL,J)
               !WRITE(*,'(1x,I3,1x,I2,1x,A)') J, Ifo, TRIM(Ref_x(J,Ifo))
            ENDDO  ! over selected ice habit forms (Ifo)
         ENDDO  ! over effective radius in DB (J)
      ENDIF  ! MLCS(Ncl)%Phm_co
c      pause

      WRITE(*,*) '  --------------------------------------------------'
      WRITE(*,*)
     $     '*****   Reading moments and scattering matrices Nph_lay = ',
     $     Nph_lay
c     --------------------------------------------------------------
c     ===> Reading moments of the scattering matrix for selected ice 
c          crystal forms and dimensions
c      tts1 = Time_in_Seconds()
c      WRITE(*,*) '  SHAPE(Nx_MOM) ',  SHAPE(Nx_MOM)
c      WRITE(*,'(A,3(2x,I3))')  '   Kma, Na, Nph_lay', Kma, Na, Nph_lay
c      WRITE(*,*) '  SHAPE(MOM_DB) ', SHAPE(MOM_DB) 
c      WRITE(*,*) '       Nfm  = ', Nfm
      DO J = 1, Nph_lay
         DO Ifo = 1, Nfm
            FULL_PATH = TRIM(CDB_PATH)//TRIM(Ref_x(J,Ifo))//TRIM(EC_nam)
c           -----------------------------------------------------------
c            IF( J == 1.OR.J == Nph_lay ) THEN
c               WRITE(*,'(1x,I3,1x,I2,1x,A)') J, Ifo, TRIM(FULL_PATH)
c           ENDIF
c           -----------------------------------------------------------
            R_start = SUM( Nx_MOM(1:WLS-1,Ifo,J) )
            NN = SUM( Nx_MOM(WLS:WLE,Ifo,J) )
            Allocate(RecRead( R_start+NN))
            OPEN(C_UNIT,FILE=TRIM(FULL_PATH), STATUS='OLD',RECL=50000)
            Read(C_UNIT,'(A)') RecRead
            Close(C_UNIT)
            II = 0 ; KK = 0
            DO K = WLS, WLE               ;  II = II + 1
               DO I = 1, Nx_MOM(K,Ifo,J)  ;  KK = KK + 1
                  READ(RecRead(R_start+KK)(17:),*)
     $                 (DB_LOC(II,I,J,JJ,Ifo),JJ=1,Nm_vs)
               END DO
            END DO
            Deallocate(RecRead)
            IF( Nm_vs > 1 ) THEN
               DB_LOC(:,:,J,5,Ifo) = -DB_LOC(:,:,J,5,Ifo)
               DB_LOC(:,:,J,6,Ifo) = -DB_LOC(:,:,J,6,Ifo)
            ENDIF
         ENDDO  ! over selected ice habit forms (Ifo)
         IF( .NOT.Use_Ydb_SCM ) CYCLE
c         WRITE(*,'(1x,I3,1x,A)') J, 
c     $        '  ------------------------------------------------------'
c        ===============================================================
c        ... Reading scattering matrixes in selected DB
         DO Ifo = 1, Nfm
            FULL_PATH= TRIM(CDB_PATH)//TRIM(Ref_x(J,Ifo))//TRIM(SCM_nam)
c           -----------------------------------------------------------
c            IF( J == 1.OR. J == Nph_lay ) THEN
c               WRITE(*,'(1x,I3,1x,I2,1x,A)') J, Ifo, TRIM(FULL_PATH)
c            ENDIF
c          -----------------------------------------------------------
c           ... Number of lines which should be skipped
            R_start = (WLS-1)*Ydb_ang
            NN = (WLE - WLS + 1)*Ydb_ang
            Allocate(RecScm(R_start+NN))
            OPEN(C_UNIT,FILE=TRIM(FULL_PATH), STATUS='OLD',RECL=50000)
            Read(C_UNIT,'(A)') RecScm
            Close(C_UNIT)
            II = 0 ;  KK = 0
            DO K = WLS, WLE       ;  II = II + 1
               DO I = 1, Ydb_ang  ;  KK = KK + 1
                  READ(RecScm(R_start+KK)(1:6),*) SCAT_ANG(I)
                  READ(RecScm(R_start+KK)(7:),*) (TMP2(JJ),JJ=1,Nm_vs)
c                 ... Remove normalization by F11
                  DO JJ = 2, Nm_vs
                     TMP2(JJ) = TMP2(JJ)*TMP2(1) 
                  ENDDO
                  SCAT_Loc(II,I,J,:,Ifo) = TMP2(1:Nm_vs)
               ENDDO  ! over scattering angles (I)
c              ... Check fist and last scattering angle
               IF( SCAT_ANG(1)       /= 0.d0   ) GOTO 133
               IF( SCAT_ANG(Ydb_ang) /= 180.d0 ) GOTO 134
            ENDDO  ! over actual WL range (K,II)
c     ===============================================================
            Deallocate(RecScm)
         ENDDO  ! over selected ice habit forms (Ifo)
      ENDDO  ! over selected particle maximal dimension (J) 
c     ------------------------------------------------------------------

c     tts2 = Time_in_Seconds()
c      WRITE(*,*) '   Time: ', tts2 - tts1
c      Pause
      
      IF( Use_Ydb_SCM ) THEN
         SCAT_ANG(:) = DCOS( SCAT_ANG(:)/RTD )
      ENDIF

      NULLIFY( ScPo )
      
c     ... Interpolate on the "main" wavelength grid         
c     -----------------------------------------------
      ALLOCATE( Jha(N_phr) )
      Jha(1) = J1s
      IF( N_phr > 1 ) FORALL( I=1:Nph_lay ) Jha(I) = I

c      WRITE(*,*) '  Use_PSD   =  ', Use_PSD 
c      WRITE(*,*) '       Na   =  ', Na
c      WRITE(*,*) '   SHAPE(CLD_LAYER_COEFF) ', SHAPE(CLD_LAYER_COEFF)
      
      IF( Use_PSD ) THEN
c     --------- Polydisperse  medium --------------------------
         DO I = 1,NCW
            CALL LIN_SEARCH_NEW(CLD_NETWAVS(I), D1, Ws, Ws1, 
     $           NCWI, WL_x(WLS:WLE))
            D2 = 1.d0 - D1
            ScPo => CLD_LAYER_COEFF(NCL,I,N_1+1,1:Na)
            DO J = 1, Na 
               MgP => MOM_DB(I,:,NCL,J,P_1,:)
               DO II = 1, Max_loc
                  DO KK = 1, Nm_vs
                     DO Ifo = 1, Nfm
                        DBI(:) = D2*DB_LOC(Ws,II,:,KK,Ifo)
     $                       + D1*DB_LOC(Ws1,II,:,KK,Ifo)
                        MgP(II,KK) = MgP(II,KK) + SUM( DBI(:)
     $                       *H_SCA(I,:,Ifo)*CRST(Ncl)%SHP(Ifo)%Fr_hab
     $                       *Fd(J,:) )/ScPo(J)
                     ENDDO  ! over selected ice habit forms (Ifo)
                  ENDDO  ! over Stokes components (KK)
               ENDDO  ! over expansion coefficients (II)
               IF( .NOT.Use_Ydb_SCM ) CYCLE
               SmP => SCAT_MAT(I,:,NCL,J,P_1,:)
               DO II = 1, Ydb_ang
                  DO KK = 1, Nm_vs
                     DO Ifo = 1, Nfm
                        ASSOCIATE( A0 => SCAT_Loc(:,II,:,KK,Ifo) )
                        DBI(:) = D2*A0(Ws,:) + D1*A0(Ws1,:)
                        SmP(II,KK) = SmP(II,KK) + SUM( DBI(:)
     $                       *H_SCA(I,:,Ifo)*CRST(Ncl)%SHP(Ifo)%Fr_hab
     $                       *Fd(J,:) )
                        END ASSOCIATE
                     ENDDO  ! over selected ice habit forms (Ifo)
                  ENDDO  ! over Stokes components (KK)
               ENDDO            ! over scattering angles (II)
               SmP(:,:) = SmP(:,:)/ScPo(J)
            ENDDO  ! over selected particle maximal dimension (J) 
         ENDDO  ! over selected wavelengths (I)
      ELSE
c     --------- Monodisperse  medium --------------------------
         DO I = 1,NCW
            CALL LIN_SEARCH_NEW(CLD_NETWAVS(I), D1, Ws, Ws1, 
     $           NCWI, WL_x(WLS:WLE))
            D2 = 1.d0 - D1
            ScPo => CLD_LAYER_COEFF(NCL,I,N_1+1,1:Na)
            JJ = Nd_s
            DO J = 1, Nph_lay 
               J3 = Jha(J)
               MgP => MOM_DB(I,:,NCL,J,P_1,:)
               DO Ifo = 1, Nfm
                  DUMM = CRST(Ncl)%SHP(Ifo)%Fr_hab(JJ)
                  MgP(:,:) = MgP(:,:) + (D2*DB_LOC(Ws ,:,J,:,Ifo)
     $                 + D1*DB_LOC(Ws1,:,J,:,Ifo))
     $                 * H_SCA(I,J3,Ifo)*DUMM/ScPo(J3)
               ENDDO  ! over selected ice habit forms (Ifo)
               IF( .NOT.Use_Ydb_SCM ) THEN
                  JJ = JJ + 1  ;   CYCLE
               ENDIF
c              ================================================
               SmP => SCAT_MAT(I,:,NCL,J,P_1,:)
               DO Ifo = 1, Nfm
                  DUMM = CRST(Ncl)%SHP(Ifo)%Fr_hab(JJ)
                  SmP(:,:) = SmP(:,:) + (D2*SCAT_Loc(Ws,:,J,:,Ifo)
     $                 + D1*SCAT_Loc(Ws1,:,J,:,Ifo))
     $                 * H_SCA(I,J3,Ifo)*DUMM/ScPo(J3)       
               ENDDO ! over selected ice habit forms (Ifo)
               JJ = JJ + 1
            ENDDO  ! over selected particle maximal dimension (J) 
         ENDDO  ! over selected wavelengths (I)
      ENDIF
      
c            WRITE(*,*) '   Max_Bdb in Bdb: ', Max_Bdb      
      Max_Bdb = MAX( Max_loc,Max_Bdb ) 
      N_MOM_USER = Max_Bdb

c      WRITE(*,*) "     From 'general_cloud_init.f' "
c      WRITE(*,*) '   Max number of moments (selected DB) :', N_MOM_USER
      
      NULLIFY (ExPo, ScPo, MgP, SmP)

c      OPEN( 77, file='./DATA_OUT/Exp-cof.dat' )
c      DO JJ = 1, Nx_MOM(WLS,1,1)
c         WRITE(77,*) JJ, MOM_DB(1,JJ,1,1,1,1:Nm_vs)
c      ENDDO
c     CLOSE( 77 )
      
c     JJ = Nx_MOM(WLS,1,1) 
c      CALL COEF_TO_SCATMATR(JJ, Nm_vs, MOM_DB(1,1:JJ,1,1,1,1:Nm_vs), 
c     $     180, 'E', './DATA_OUT/Ph_Young.dat')      
c      STOP '  Stop after output of ice cloud scattering matrix'

      RETURN

 133  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $     '    First scattering angle is not equal to 0.d0')
      WRITE(EU,'(A,1x,2(2x,I3))') 
     $     '   Eff. radius and wavelength ', J,K
      RETURN 1

 134  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $     '    Last scattering angle is not equal to 180.d0')
      WRITE(EU,'(A,1x,2(2x,I3))') 
     $     '   Eff. radius and wavelength ', J,K
      RETURN 1

      END SUBROUTINE READ_DB_YDB
c **********************************************************************

C     ******************************************************************
c           Reading optical characteristics of ice clouds
c                        from Baum database
c     ******************************************************************
      SUBROUTINE READ_DB_BDB(N_phr,*)
      INTEGER     N_phr
      INTEGER     I, J, K, JJ, I_tmp, Nwl_Bdb, WLS, WLE, Max_loc
      CHARACTER (LEN=200) FULL_PATH      
      CHARACTER (LEN=80)   F_name, Dpi
      CHARACTER (LEN=80)  Ref_val(N_phr)
      INTEGER, ALLOCATABLE :: N_BDB_MOM(:,:,:)
      LOGICAL, ALLOCATABLE :: M_Bdb(:,:,:)
      REAL (KND)  DUMM, START_WL, END_WL 
      REAL (KND), ALLOCATABLE :: Lo_EXT(:,:), Lo_SSA(:,:), WL_Bdb(:), 
     $     H_ext(:), H_ssa(:), DB_LOC(:,:,:,:)
c    -------------------------------------------------------------------

      Dpi = 'Crystals/Baum_DB/'
      F_name = 'AggregateSolidColumns_'

c      ALLOCATE( TotA_Bdb(N_ref_Bdb) )
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//TRIM(F_name)//
     $     'effective_radius.dat'
      OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         DO J = 1, N_ref_Bdb
            READ( C_UNIT,* ) DUMM, TotA_Bdb(J)
         ENDDO
      CLOSE(C_UNIT)
      
c    ... Reading wavelength grid [nm]
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//TRIM(F_name)//
     $     'wavelength.dat'
c                 WRITE(*,*) '   Reading WL grid  '
c                 WRITE(*,*) TRIM( FULL_PATH )
      OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         READ( C_UNIT,* ) Nwl_Bdb
         ALLOCATE( WL_Bdb(Nwl_Bdb),  Lo_EXT(Nwl_Bdb,N_ref_Bdb), 
     $        Lo_SSA(Nwl_Bdb,N_ref_Bdb) )
         DO J = 1, Nwl_Bdb 
            READ( C_UNIT,* ) WL_Bdb(J)
         ENDDO
      CLOSE(C_UNIT)

      START_WL = M_WVL(WVLS) ; END_WL = M_WVL(WVLE)
      CALL LIN_SEARCH_NEW(START_WL, DUM, WLS, Ws1, Nwl_Bdb, WL_Bdb)
      CALL LIN_SEARCH_NEW(END_WL,   DUM, Ws,  WLE, Nwl_Bdb, WL_Bdb)
c     ... Total number of actual wavelengths within Baum DB
c     -----------------------------------------------------
c                 WRITE(*,*) '  Actual wavelength for Baum DB  '
c                 WRITE(*,*) WL_Bdb(WLS), WL_Bdb(WLE)
      NCWI = WLE - WLS + 1
      
c      WRITE(*,*) '    Ncl =  ', Ncl
c      WRITE(*,*) '   N_phr =  ', N_phr
      ALLOCATE( DB_LOC(NCWI,MAXMOM_LOC,N_phr,Nm_vs), 
     $     M_Bdb(Nwl_Bdb,N_CLD_LAYERS,N_ref_Bdb) )
      DB_LOC = 0.d0

c    ... Reading extinction coefficients
c    ===================================
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//TRIM(F_name)//
     $     'extinction_coefficient_over_iwc.dat'
c                 WRITE(*,*) '   Reading extinction coefficients  '
c                 WRITE(*,*) TRIM( FULL_PATH )
      OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         DO I = 1, Nwl_Bdb 
            READ(C_UNIT,*) (Lo_EXT(I,J), J = 1,N_ref_Bdb)
         ENDDO
      CLOSE(C_UNIT)
c    ... Reading single scattering albedo
c    ====================================
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//TRIM(F_name)//
     $     'single_scattering_albedo.dat'
c                 WRITE(*,*) '   Reading SSA  '
c                 WRITE(*,*) TRIM( FULL_PATH )
      OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         CALL SKIP_HEADER( C_UNIT )
         DO I = 1, Nwl_Bdb 
            READ(C_UNIT,*) (Lo_SSA(I,J), J = 1,N_ref_Bdb)
         ENDDO
      CLOSE(C_UNIT)
c
c     ... Interpolate on the WL-grid of water cloud DB
c      WRITE(*,*) 'WL water at the main grid ',
c     $        CLD_NETWAVS(:)
      ALLOCATE( H_ext(NCW), H_ssa(NCW)  )
      DO J = 1,N_ref_Bdb
         Call LIN_INT(Nwl_Bdb, WL_Bdb, Lo_EXT(:,J), NCW, CLD_NETWAVS, 
     $        H_ext, 'C')
         Call LIN_INT(Nwl_Bdb, WL_Bdb, Lo_SSA(:,J), NCW, CLD_NETWAVS, 
     $        H_ssa, 'C')
c        ... Save extinction and scattering coefficients from
c             Baum database interpolated on the water DB
c        ---------------------------------------------------
         CLD_LAYER_COEFF(NCL,1:NCW,N_1,J) = H_ext(:) 
         CLD_LAYER_COEFF(NCL,1:NCW,N_1+1,J) = H_ext(:)*H_ssa(:)
      ENDDO  ! over effective radius (J)

      DEALLOCATE( H_ext, H_ssa,  Lo_EXT, Lo_SSA )

      ALLOCATE( N_BDB_MOM(Nwl_Bdb,N_CLD_LAYERS,N_ref_Bdb) )

c     ... Reading number of expansion coefficients in Baum DB
c     -------------------------------------------------------
      DO J = 1, N_ref_Bdb
         FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//TRIM(F_name)//'EK_'//
     $        TRIM(S_name_Bdb(J))//'.dat_N-moms.dbs'
c                 WRITE(*,*) '   Reading N-moms  ', TRIM( FULL_PATH )
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
           DO K = 1,Nwl_Bdb
              READ(C_UNIT,*) DUMM, N_BDB_MOM(K,NCL,J)
           ENDDO
         CLOSE( C_UNIT ) 
      ENDDO  ! over effective radius in Baum DB (J)

      IF( MLCS(Ncl)%Phm_co ) THEN
c        ... The number of moments for selected input effective radius
         FULL_PATH = TRIM(CDB_PATH)//TRIM(Dpi)//DBC_NAME(NCL,2)%S
     $        //'.dat_N-moms.dbs'
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
         DO K = 1,Nwl_Bdb
            READ(C_UNIT,*) DUMM, N_BDB_MOM(K,NCL,1)
         ENDDO
         CLOSE( C_UNIT ) 
c        ... After this reading will be used N_BDB_MOM(K,NCL,1) only
c        ... Copy file name in the variable Ref_val
         Ref_val(1) = TRIM(Dpi)//DBC_NAME(NCL,2)%S
      ELSE
c        ... The moments filenames for all effective radius in DB
         DO J = 1,N_phr  
            Ref_val(J) = TRIM(Dpi)//TRIM(F_name)//'EK_'//S_name_Bdb(J)
         ENDDO
      ENDIF  ! MLCS(Ncl)%Phm_co

c        Reading moments of the phase matrixes in Baum DB
c        ------------------------------------------------
      DO J = 1,N_phr 
         FULL_PATH = TRIM(CDB_PATH)//TRIM(Ref_val(J))//'.dat_moms.dbs'
         OPEN(C_UNIT,FILE = TRIM(FULL_PATH),STATUS='OLD',RECL=500000)
            R_start = SUM( N_BDB_MOM(1:WLS-1,NCL,J) )
c           ... Skip moments for all wavelengths before actual
            DO K = 1,R_start ; READ( C_UNIT,* ) ; ENDDO 
            II = 0
            DO K = WLS, WLE  ;  II = II + 1
c               WRITE(*,*)  II, K
               DO I = 1, N_BDB_MOM(K,NCL,J)
                  READ(C_UNIT,*) Tmp1, I_tmp, (TMP2(JJ), JJ=1,Nm_vs)
                  DB_LOC(II,I,J,:) = Tmp2(:)
                  IF( I == 1 ) THEN
                     IF( I_tmp /= 1 ) THEN
                        STOP 'Error reading of ice Baum database'
                     ENDIF
                  ENDIF
               ENDDO  ! over number of moments (I)
            ENDDO  ! over actual WL range (K,II)
            IF( Nm_vs > 1 ) THEN
               DB_LOC(:,:,J,5) = -DB_LOC(:,:,J,5)
               DB_LOC(:,:,J,6) = -DB_LOC(:,:,J,6)
            ENDIF
         CLOSE( C_UNIT ) 
      ENDDO  ! over particle effective radius (J) 

c     ... Interpolate on the "main" wavelength grid         
c     -----------------------------------------------
      DO I = 1,NCW
         CALL LIN_SEARCH_NEW(CLD_NETWAVS(I), DUM, Ws, Ws1, 
     $        NCWI, WL_Bdb(WLS:WLE))
         DO J = 1, N_phr 
            MOM_DB(I,:,NCL,J,P_1,:) = 
     $           (1.D0 - DUM)*DB_LOC(Ws,:,J,:) + DUM*DB_LOC(Ws1,:,J,:) 
c            MOM_DB(I,:,NCL,J,P_1,:) = DB_LOC(2,:,J,:)
         ENDDO
      ENDDO  ! over DB wavelengths (I)

      M_Bdb = .FALSE.
c      M_Bdb(WLS:WLE,1:N_CLD_LAYERS,1:N_phr) = .TRUE.
      M_Bdb(WLS:WLE,Ncl,1:N_phr) = .TRUE.
c     ... Maximal number of expansion coefficients within selected 
c         wavelength range of Baum DB
      Max_loc = MAXVAL( N_Bdb_MOM,M_Bdb )
      
c            WRITE(*,*) 'WLS, WLE',  WLS, WLE
c            DO I = 1, N_phr
c               WRITE(*,'(1x,*(2x,I5))') I, (N_Bdb_MOM(K,1,I), K=WLS,WLE) 
c            ENDDO
c      WRITE(*,*) '   Max_loc in BDB: ', Max_loc
c      WRITE(*,*) '   Max_Bdb in BDB: ', Max_Bdb
      
      Max_Bdb = MAX( Max_loc,Max_Bdb ) 
      N_MOM_USER = Max_Bdb

c      WRITE(*,*) "     From 'general_cloud_init.f' "
c      WRITE(*,*) '   Max number of moments (Baum DB) :', N_MOM_USER
c      WRITE(*,*) '     ' 
c      JJ = N_Bdb_MOM(WLS,1,1) 
c      CALL COEF_TO_SCATMATR(JJ, Nm_vs, MOM_DB(1,1:JJ,1,1,1,1:Nm_vs), 
c     $     180, 'E', './DATA_OUT/Ph_Baum.dat')      
c      STOP '  Stop after output of ice cloud scattering matrix'
      
      END SUBROUTINE READ_DB_BDB
c **********************************************************************
 
c     *************************************************************
c           Reading optical characteristics of ice clouds
c                            from database
c           In this routine the wavelength grid can not be the
c           same as for water database (reference wavelengths). 
c           Therefore,  
c           within this routine the local wavelength grid is used.
c           The optical characteristics for ice particles will be
c           interpolated on the actual wavelength grid which 
c           is set to the grid in database for water particles, i.e.,
c           in array CLD_NETWAVS(1:NCW)
c     ******************************************************************
      SUBROUTINE READ_DB_ICE(*)
      INTEGER  J, K, I_tmp, N_WL, WLS, WLE, Max_loc, N_READ
      REAL (KND)  START_WL, END_WL, DB_EXT(NCW), DB_SSA(NCW)
      CHARACTER (LEN=80)  Name_inp, Ndb_p
      CHARACTER (LEN=200) FULL_PATH 
      CHARACTER (LEN=4), POINTER:: Name_READ(:)
      INTEGER, ALLOCATABLE:: N_momi(:)
      LOGICAL, ALLOCATABLE:: L_momi(:)
      REAL (KND), ALLOCATABLE:: WL(:), Exti(:), Ssai(:), MOM_LOC(:,:,:)
c     ------------------------------------------------------------------

      SELECT CASE( CRST(NCL)%SHP(1)%ICE_FORM)
      CASE('hex')
         Ndb_p = TRIM(NDB_Ice)//'Hexagons/'
         N_READ = N_ref_HEX  ;  Name_READ => S_name_HEX(:)
         Name_inp = 'Greek_Hexagon_L100_r'
      CASE('fra')
         N_READ = N_ref_Fra  ;  Name_READ => S_name_Fra(:)
         Ndb_p = TRIM(NDB_Ice)//'Fractal/'
         Name_inp = 'Greek_Fractal_'
      CASE DEFAULT
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $        'Form of ice particles is not recognized ')
         WRITE(EU,*) '  Cloud layer  : ', NCL
         WRITE(EU,*) '  Form : ', CRST(NCL)%SHP(1)%ICE_FORM
         RETURN 1
      END SELECT

c     ... Read local wavelength grid for ice particles databases
c         File: "Greek_..._wvl.dbs". It is assumed that the 
c         wavelength grid is in [micron] in this file
c     ------------------------------------------------------------
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Ndb_p)//
     $     DBC_NAME(NCL,2)%S//TRIM(ICE_WL_DB)
      OPEN(C_UNIT,FILE=TRIM(FULL_PATH),STATUS='OLD')
        READ(C_UNIT,*) N_WL ;  ALLOCATE( WL(N_WL))
        DO K = 1, N_WL
           READ(C_UNIT,*) WL(K)
        ENDDO
        WL(:) = WL(:)*1.D+03    ! Convert to [nm]
      CLOSE( C_UNIT )

      ALLOCATE( Exti(N_WL), Ssai(N_WL), N_momi(N_WL), L_momi(N_WL) )

c     ... Find position of start- and end-wavelengths of "main"  
c         wavelength grid in wavelength array of a local grid for ice
c     ----------------------------------------------------------------
c      WRITE(*,*) '    Ice :'
c      WRITE(*,*) '  Full number of WL in reference ice grid: ',N_WL 
c      WRITE(*,*) '           First WL in reference ice grid: ',WL(1)

      START_WL = M_WVL(WVLS) ; END_WL = M_WVL(WVLE)
      CALL LIN_SEARCH_NEW(START_WL, DUM, WLS, Ws1, N_WL, WL)
      CALL LIN_SEARCH_NEW(END_WL,   DUM, Ws,  WLE, N_WL, WL)
c     ... Total number of "ice" actual wavelengths 
c     --------------------------------------------
      NCWI = WLE - WLS + 1

c     ... Reading all scattering and extinction coefficients from DB
c     --------------------------------------------------------------
      DO J = 1, N_READ
         FULL_PATH = TRIM(CDB_PATH)//TRIM(Ndb_p)//
     $        TRIM(Name_inp)//TRIM(Name_READ(J))//TRIM(NAM_DB(2))
c         WRITE(*,*) '--> Reading ice DB    :', J, TRIM(FULL_PATH)
c                      WRITE(*,*) '    SS albedo from DB :'
         OPEN(C_UNIT, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
          DO K = 1,N_WL
            READ(C_UNIT,'(2(D14.7),I5)') Exti(K), Ssai(K), N_momi(K)
         ENDDO
         CLOSE( C_UNIT ) 
         Call LIN_INT(N_WL, WL, Exti, NCW, CLD_NETWAVS, DB_EXT, 'C')
         Call LIN_INT(N_WL, WL, Ssai, NCW, CLD_NETWAVS, DB_SSA, 'C')

c        Save extinction coefficient from ice database
c        ----------------------------------------------
         CLD_LAYER_COEFF(NCL,1:NCW,N_1, J) = DB_EXT(1:NCW)
c        Save scattering coefficient from ice database
c        ----------------------------------------------
         CLD_LAYER_COEFF(NCL,1:NCW,N_1+1,J) = 
     $        DB_EXT(1:NCW)*DB_SSA(1:NCW)
c              WRITE(*,*) '   SS-albedo from DB :', DB_SSA(1:NCW)
c              WRITE(*,*) '   =================='
      ENDDO  ! over available sizes fractal and hexagonal prisms (J)

c      '../../DATA_BASES/CLOUDS/' 
c      +  'Greek_Fractal_100'   --> 'AggregateSolidColumns_EK_5.00'
c      +    '_ssa.dbs'          --> '.dat'
c     ... Reading selected scattering and extinction coefficients from DB
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Ndb_p)//DBC_NAME(NCL,2)%S
     $     //TRIM(NAM_DB(2))         

      WRITE(*,*) '--> Reading selected coefficients  :',
     $     TRIM(DBC_NAME(NCL,2)%S)
c      WRITE(*,*) '--> Reading selected coefficients  :', TRIM(FULL_PATH)

      OPEN( C_UNIT,FILE = TRIM(FULL_PATH),STATUS='OLD' )
      DO K = 1,N_WL
         READ(C_UNIT,'(2(D14.7),I5)') Exti(K), Ssai(K), N_momi(K)
      ENDDO
      CLOSE( C_UNIT ) 

      ALLOCATE( MOM_LOC(NCWI,MAXMOM_LOC,Nm_vs) )
      MOM_LOC = 0.d0

c     ... Reading moments of the phase matrixes for ice particles
c     -----------------------------------------------------------
      FULL_PATH = TRIM(CDB_PATH)//TRIM(Ndb_p)//
     $     DBC_NAME(NCL,2)%S//TRIM(NAM_DB(3))

c      WRITE(*,*) '--> Reading selected moments  :', TRIM(FULL_PATH)

      OPEN(C_UNIT,FILE = TRIM(FULL_PATH),STATUS='OLD',RECL=500000)         
      R_start = SUM( N_momi(1:WLS-1) )
      R_end   = SUM( N_momi(1:WLE) )
c     ... Skip moments for all wavelengths before actual
c     --------------------------------------------------
      DO K = 1,R_start ; READ( C_UNIT,* ) ; ENDDO            
      II = 0
      DO K = WLS, WLE  ;  II = II + 1
         IF( N_momi(K) > MAXMOM_LOC ) THEN
            WRITE(*,*) '  ******* MAXMOM_LOC is too small ******* '
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Actual number of phase function moments '//
     $           'read from database is too large')
            WRITE(EU,*) 
     $           '  Actual number of moments  : ', N_momi(K)
            WRITE(EU,*) 
     $           '  Maximal number of moments : ', MAXMOM_LOC
            WRITE(EU,*) 
     $           '  Increase parameter MAXMOM_LOC in ', R_NAME
            RETURN 1
         ENDIF
         DO I = 1,N_momi(K)
            READ(C_UNIT,'(D11.4,I5,6(D14.7))') 
     $           Tmp1, I_tmp, (TMP2(J), J=1,Nm_vs)
            MOM_LOC(II,I,1:Nm_vs) = Tmp2(1:Nm_vs)
            IF( I == 1 ) THEN
c              WRITE(*,*) '  For wavelength: ', Tmp1
               IF( I_tmp /= 1 ) THEN
                  STOP 'Error reading of ice database'
               ENDIF
            ENDIF
         ENDDO
      ENDDO
      IF( Nm_vs > 1 ) THEN
         MOM_LOC(:,:,5) = -MOM_LOC(:,:,5)
         MOM_LOC(:,:,6) = -MOM_LOC(:,:,6)
      ENDIF
      CLOSE( C_UNIT ) 

c     ... Interpolate on the "main" wavelength grid         
c     -----------------------------------------------
      DO I = 1,NCW
c            WRITE(*,*) '  Moment interpolation on WL = ', CLD_NETWAVS(I)
         CALL LIN_SEARCH_NEW(CLD_NETWAVS(I), DUM, Ws, Ws1, 
     $        NCWI, WL(WLS:WLE))
         MOM_DB(I,:,NCL,1,P_1,:) = 
     $        (1.D0 - DUM)*MOM_LOC(Ws,:,:) + DUM*MOM_LOC(Ws1,:,:) 
c            MOM_DB(I,:,NCL,1,P_1,:) = MOM_LOC(2,:,:)
c         WRITE(*,*) I, MOM_DB(I,1:10, NCL, 1, P_1, 1)
      ENDDO

      L_momi = .FALSE.  ;  L_momi(WLS:WLE) = .TRUE.
      Max_loc = MAXVAL( N_momi,L_momi )
      Max_ice = MAX( Max_loc,Max_ice ) 
c     ... Maximal number of expansion coefficients within selected 
c         wavelength range of ice DB
      N_MOM_USER = Max_ice

c      I = N_momi(WLS)
c      CALL COEF_TO_SCATMATR(I, Nm_vs, MOM_DB(1,1:I,1,1,1,1:Nm_vs), 
c     $     180, 'E', './DATA_OUT/Ph_Frac.dat')
c      STOP '  Stop after output of ice cloud scattering matrix'
      
      END SUBROUTINE READ_DB_ICE
C     *************************************************************

      SUBROUTINE READ_PHASE_AEROSOL(*)

      USE CLD_INP, ONLY:  Aer_ref

      CHARACTER (LEN=*), PARAMETER :: R_NAME='READ_PHASE_AEROSOL'
      INTEGER   Iw, Iwp, Ksu
      CHARACTER (LEN=200) AER_NAMES(NM_WVL+1)
      CHARACTER (LEN=2)  ETYPE
      REAL (KND) Mom_tes
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I') 

c     ... Reading of filenames containing extinction, scattering cross
c         section as well as scattering matrix
      OPEN( C_UNIT, file=USER_EK_NAME(1,2)%S, STATUS='OLD' )
 200     Read (C_UNIT,'(a1)' ) DUMCHAR                    
         IF ( DUMCHAR .NE. START ) Go To 200
c       ... +1 to read filename containing extinction cross section
c            at reference wavelength
        DO I = 1, NM_WVL + 1
           READ(C_UNIT,*)  AER_NAMES(I)
           WRITE(*,*) I, TRIM( AER_NAMES(I) )
        ENDDO
      CLOSE(C_UNIT)

c     ... Reading extinction cross-sections at reference wavelength.
      OPEN(C_UNIT, file=TRIM(AER_NAMES(1)), STATUS='OLD')
        CALL SKIP_HEADER( C_UNIT )
        READ( C_UNIT,* ) Aer_ref        
      CLOSE(C_UNIT)

      Iwp = 2
      DO Iw = 1, NM_WVL 
         OPEN(C_UNIT, file=TRIM(AER_NAMES(Iwp)), STATUS='OLD')
         CALL SKIP_HEADER( C_UNIT )
c        ... Reading of extinction and scattering cross-sections.
         READ( C_UNIT,* ) TMP1, TMP2(1)
c                   write(*,*) TMP1, TMP2(1)
c        ... Scattering and extinction coefficients can be 
c            recalculated in the gt_cloud_parameter_setup.f
c        ---------------------------------------------------
         CLD_LAYER_COEFF( NCL,Iw,1,: ) = TMP1
         CLD_LAYER_COEFF( NCL,Iw,2,: ) = TMP2(1)
         II = 0
         DO I = 1, MAXMOM_LOC
            II = II + 1
            READ(C_UNIT,*,END=20) TMP1, (TMP2(K), K=1,Nm_vs)
            Ksu = I - IDINT(TMP1)
            IF( Ksu/=1 .AND. Ksu/=0 ) GOTO 132
            MOM_DB(Iw,II,NCL,1,P_1,1:Nm_vs) = TMP2(1:Nm_vs)
         END DO
 20      CONTINUE
         IF( Nm_vs > 1 ) THEN
            MOM_DB(Iw,:,Ncl,1,P_1,5) = -MOM_DB(Iw,:,Ncl,1,P_1,5)
            MOM_DB(Iw,:,Ncl,1,P_1,6) = -MOM_DB(Iw,:,Ncl,1,P_1,6)
         ENDIF
c          ... Number of expansion coefficients for the user defined phase 
c              matrix. NCL - cloud layer number; 
c                      INX = 1 for water, ice, aerosol layer cloud;
c              Save maximal value which is usually takes place for the
c              shortest wavelength
         IF( Iw==1 ) N_US_MOM(NCL,INX) = II - 1
         CLOSE( C_UNIT )

c        ... Check of first expansion coefficients
         Mom_tes = MOM_DB(Iw,1,NCL,1,P_1,1)
         ETYPE = 'E1'
         IF( DABS(Mom_tes - 1.d0) > 1.d-8 ) GOTO 132
c        ... Test of second expansion coefficients
         Mom_tes = MOM_DB(Iw,2,NCL,1,P_1,1)/3.
         ETYPE = 'E2'
         IF( Mom_tes > 1.d0 ) GOTO 132
         ETYPE = 'E3'
         IF( Mom_tes < 0.d0 ) GOTO 132
         Iwp = Iwp + 1
      ENDDO  ! over aerosol layer wavelength grid (Iw)

      CALL ST_TRACK (R_NAME, 1, 'O') 
      RETURN
  
 132  CONTINUE
      SELECT CASE(ETYPE)
      CASE('E0')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        ' Probably you tray to use phase function instead '//
     $        'of phase matrix.')
         WRITE(EU,'(A,2x,A)') 
     $     '    In the vector case the program needs phase matrix.'
      CASE('E1')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    First expansion coefficient of the '//
     $        'phase function is not equal to 1.')
         WRITE(EU,'(A)') 
     $        '    Check normalization of the phase function used.'
      CASE('E2')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    Second expansion coefficient (asymmetry factor) ')
         WRITE(EU,'(A)') 
     $        '    of phase function greater than 1.'
      CASE('E3')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    Second expansion coefficient (asymmetry factor) ')
         WRITE(EU,'(A)') 
     $        '    of phase function is negative.'
      END SELECT         

      WRITE(EU,'(A,2x,A)') '    Wrong structure of input file: ',
     $     USER_EK_NAME(NCL,IREAD)%S
      WRITE(EU,'(A,2x,A)') '    or an error during the reading.'
      WRITE(EU,'(A)') '    Check input file "cloud.inp" and the file'
      WRITE(EU,'(A,2x,A)') '    containing expansion coefficients: ',
     $     USER_EK_NAME(NCL,IREAD)%S

      RETURN 1
      
      END SUBROUTINE READ_PHASE_AEROSOL
c ************************************************************************

      SUBROUTINE READ_PHASE_LEGEN_USER(*)
      INTEGER  KF, Ksu
      CHARACTER (LEN=2)  ETYPE
      REAL (KND) Mom_tes
c         WRITE( *,* ) '... read user moments ...'
c         write( *,* ) '..... ',TRIM(USER_EK_NAME(NCL,IREAD))

      IF( ALIN_ACT(4) ) THEN
c         WRITE(*,*) '  SHAPE(MOM_DB) ', SHAPE(MOM_DB)         
         CLD_LAYER_COEFF( NCL,1:NCW,N_1  ,1 ) = 1.d0
         CLD_LAYER_COEFF( NCL,1:NCW,N_1+1,1 ) = 1.d0
         N_US_MOM(NCL,INX) = NC_CLD
         DO I = 1, NC_CLD
            FORALL( KF = 1:NCW )
     $           MOM_DB(KF,I,NCL,1,P_1,1:Nm_vs) = ALIN_CLD_EC(I,1:Nm_vs)
         END DO
      ELSE
c        ... USER_EK_NAME(:,2) is defined always for two phase functions:
c            first belongs to the water
c            second to the ice
         OPEN(C_UNIT, file=USER_EK_NAME(NCL,IREAD)%S, STATUS='OLD')
 200     Read (C_UNIT,'(a1)' ) DUMCHAR                    
         IF ( DUMCHAR .NE. START ) Go To 200
c          ... Reading of extinction and scattering cross-sections.
         READ( C_UNIT,* ) TMP1, TMP2(1)
c           write(*,*) TMP1, TMP2(1)
c           ... Scattering and extinction coefficients can be 
c               recalculated in the gt_cloud_parameter_setup.f
c           ---------------------------------------------------
         CLD_LAYER_COEFF( NCL,1:NCW,N_1  ,1 ) = TMP1
         CLD_LAYER_COEFF( NCL,1:NCW,N_1+1,1 ) = TMP2(1)
         II = 0
         DO I = 1, MAXMOM_LOC
            II = II + 1
            READ(C_UNIT,*,END=20) TMP1, (TMP2(K), K=1,Nm_vs)
            Ksu = I - IDINT(TMP1)
            IF( Ksu/=1 .AND. Ksu/=0 ) GOTO 132
            FORALL( KF = 1:NCW )
     $           MOM_DB(KF,II,NCL,1,P_1,1:Nm_vs) = TMP2(1:Nm_vs)
         END DO
 20      CONTINUE
         IF( Nm_vs > 1 ) THEN
            MOM_DB(:,:,Ncl,1,P_1,5) = -MOM_DB(:,:,Ncl,1,P_1,5)
            MOM_DB(:,:,Ncl,1,P_1,6) = -MOM_DB(:,:,Ncl,1,P_1,6)
         ENDIF
c          ... Number of expansion coefficients for the user defined phase 
c              matrix. NCL - cloud layer number; 
c                      INX = 1 for water or ice cloud;
c                      INX = 1,2 for the mixed cloud
         N_US_MOM(NCL,INX) = II - 1
         CLOSE( C_UNIT )

c        ... Check of first expansion coefficients
         Mom_tes = MOM_DB(1,1,NCL,1,P_1,1)
         ETYPE = 'E1'
         IF( DABS(Mom_tes - 1.d0) > 1.d-8 ) GOTO 132
c        ... Test of second expansion coefficients
         Mom_tes = MOM_DB(1,2,NCL,1,P_1,1)/3.
         ETYPE = 'E2'
         IF( Mom_tes > 1.d0 ) GOTO 132
         ETYPE = 'E3'
         IF( Mom_tes < 0.d0 ) GOTO 132
      ENDIF

      RETURN
  
 132  CONTINUE
      SELECT CASE(ETYPE)
      CASE('E0')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        ' Probably you tray to use phase function instead '//
     $        'of phase matrix.')
      WRITE(EU,'(A,2x,A)') 
     $     '    In the vector case the program needs phase matrix.'
      CASE('E1')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    First expansion coefficient of the '//
     $        'phase function is not equal to 1.')
         WRITE(EU,'(A)') 
     $        '    Check normalization of the phase function used.'
      CASE('E2')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    Second expansion coefficient (asymmetry factor) ')
         WRITE(EU,'(A)') 
     $        '    of phase function greater than 1.'
      CASE('E3')
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 
     $        '    Second expansion coefficient (asymmetry factor) ')
         WRITE(EU,'(A)') 
     $        '    of phase function is negative.'
      END SELECT         

      WRITE(EU,'(A,2x,A)') '    Wrong structure of input file: ',
     $     USER_EK_NAME(NCL,IREAD)%S
      WRITE(EU,'(A,2x,A)') '    or an error during the reading.'
      WRITE(EU,'(A)') '    Check input file "cloud.inp" and the file'
      WRITE(EU,'(A,2x,A)') '    containing expansion coefficients: ',
     $     USER_EK_NAME(NCL,IREAD)%S

      RETURN 1
      
      END SUBROUTINE READ_PHASE_LEGEN_USER

c ********************************************************************
c                  Delta-S
c*********************************************************************
      SUBROUTINE Delta_S( NMOM, Nang )
c       Input parameters:
c                       NMOM - number of expansion coefficients
c                       Nang - number of discrete angles used for the 
c                              representation of the phase function 
c                              in [0-180] deg scattering angles.
      INTEGER           NMOM, Nang
      INTEGER           I, J, K, K1, Ks, Ke, Nne, N_int,
     $                  K1_max, K1_min, N_ds, MRS_min, N_start, N_end, 
     $                  Mrs_back, Otr_dm, Npol(9), Nopt(9)
      INTEGER, ALLOCATABLE:: N_gr(:), N_pos(:), Otr(:)
      LOGICAL           Fix_ang, DS_Test_run
      REAL (KND)  Kat_ang, Af, Bf, Alf, S1, S2, Con_min, Con_max
      REAL (KND), DIMENSION (Nang) :: X_ang, X_gau, AG_gau, 
     $     AG_inv, Phm1, Ph_prime, Diff, Ph_leg
      REAL (KND), TARGET::  PL_RES(NMOM,Nang)
      REAL (KND), DIMENSION (NLEGEN):: X_tes, AG_tes, Int_gr
      REAL (KND), ALLOCATABLE:: SKO(:), Lead(:), Capr(:,:), 
     $     F_par(:), SKO_b(:), RMS(:)
      REAL (KND), POINTER:: Pln(:)
c         Test for delta-M
      REAL (KND)  Coe_dm(NLEGEN), Ph_dm(Nang)

      WRITE(*,*) '   *********** Delta-S **************'
c      WRITE(*,*) '  NMOM and Nang '
c      WRITE(*,*) NMOM, Nang
      DS_Test_run = .FALSE.

      Con_min = 0.1d0 ; Con_min = DCOS( Con_min/RTD ) 
      Con_max = 6.0d0 ; Con_max = DCOS( Con_max/RTD ) 

      SELECT CASE ( MAN_AUTO_CLD(NCL) )
      CASE( 'manu' )
         Fix_ang = .TRUE.  ;  Kat_ang  = LC_CLD(NCL)         
      CASE( 'auto' )
         Fix_ang = .FALSE. 
         Con_min = LC_CLD(NCL)  ; Con_min = DCOS( Con_min/RTD ) 
         Con_max = UC_CLD(NCL)  ; Con_max = DCOS( Con_max/RTD ) 
      END SELECT

c       Npol is introduced for the run of the programm with different 
c             numbers of Legendre polynomials in the "Fix_ang" mode
c        Water 10 aef   
c      !!!!! Only for the test calculations !!!!!!
      SELECT CASE ( USER_EK_NAME(1,1)%S )
      CASE('spher.print_aef_10_mkm_645nm')
         WRITE(*,*) '   Selected Ph :'
         WRITE(*,*) 'spher.print_aef_10_mkm_645nm'
         Npol(1) = 16    ; Nopt(1) = 385
         Npol(2) = 24    ; Nopt(2) = 288
         Npol(3) = 32    ; Nopt(3) = 229
         Npol(4) = 48    ; Nopt(4) = 158
         Npol(5) = 64    ; Nopt(5) = 234
         Npol(6) = 96    ; Nopt(6) = 236
         Npol(7) = 128   ; Nopt(7) = 178
         Npol(8) = 192   ; Nopt(8) = 159
         Npol(9) = 256   ; Nopt(9) = 244
      CASE('aer_g=0.73.dat')
         WRITE(*,*) '   Selected Ph :'
         WRITE(*,*) 'aer_g=0.73.dat'
         Npol(1) = 16    ; Nopt(1) = 464
         Npol(2) = 24    ; Nopt(2) = 267
         Npol(3) = 32    ; Nopt(3) = 158
         Npol(4) = 48    ; Nopt(4) = 35
         Npol(5) = 64    ; Nopt(5) = 4
      END SELECT
c     ... To compare D-S results to the D-M method number of polynomials 
c       for the expansion of the truncated phase function will be used as:
      N_ds = NLEGEN - 1

c     ... Create Gauss nodes and weighs in the interval [-1,1]
      CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_gau, Nang )
c     ... Invert Gauss grid to start from zero scattering angle 
      X_ang(:) = X_gau(Nang:1:-1)  ;  Ag_inv(:) = AG_gau(Nang:1:-1)
c     ... Legendre polynomials for all Gauss angles
      CALL WIGNER_PLG( X_ang, Nang, 0, NMOM, PL_RES )
c     ... Calculate phase function for all Gauss nodes using moments 
c        of the phase function
      DO I = 1, Nang
         Phm1(I) = SUM( PL_RES(:,I)*MOM_USER(:) )
      ENDDO
c                  Test output 
c     ... Create Gauss nodes and weighs in the interval [-1,1]
c         for the polynomial of the order Nlegen
      CALL GAULEG ( -1.D0, 1.D0, X_tes, AG_tes, Nlegen )
      X_tes(:) = X_tes(Nlegen:1:-1) 
      S1 = ( X_tes(1) + X_tes(2) )/2.0d0
      IF( Con_max > S1 ) THEN
         WRITE(*,*) '----> Shift of the maximal cone range to:'
         Con_max = S1
         WRITE(*,'(10x,F7.3)') DACOS(Con_max)*RTD
      ENDIF
      DO I = 1, Nlegen-1
         Int_gr(I) = ( X_tes(I) + X_tes(I+1) )/2.0d0
         IF( Int_gr(I) <= Con_max ) THEN
            N_int = I
            EXIT
         ENDIF
      ENDDO
      DO I = 1,Nang
         IF( X_ang(I) < Con_min ) Then
            K1_min = I-1  ;  EXIT
         ENDIF
      ENDDO
      DO I = 1,Nang
         IF( X_ang(I) < Con_max ) Then
            K1_max = I  ;  EXIT
         ENDIF
      ENDDO

      ALLOCATE( Lead(K1_max) )
      Nne = N_ds+1 ; J = K1_min - 1
      S1 = SUM( Phm1(1:J)*PL_res(Nne,1:J)*Ag_inv(1:J) )
      DO K1 = K1_min, K1_max
         Pln => PL_res(:,K1)  ! All polynomial for a given angle
         Alf = X_ang(K1)  ;  Af = 1.d0 - Alf         
         DO I = 2, NMOM - 1
            Af = Af + MOM_USER(I)/DBLE(2*I-1)*(Pln(I-1) - Pln(I+1))
         ENDDO
         Bf   = (1.d0 - Alf)*Phm1(K1)
         Fpar = ( 1.d0 - 0.5d0*(Af - Bf) )
         S1 = S1 + Phm1(K1)*Pln(Nne)*Ag_inv(K1)
         S2 = S1*DBLE(2*Nne-1) - Phm1(K1)*(Pln(Nne-1) - Pln(Nne+1))
         Lead(K1) = ( MOM_USER(Nne) - S2/2.0d0 )/Fpar
      ENDDO
      Lead(1:K1_min-1) = Lead(K1_min)      

      ALLOCATE( N_gr(N_int+1), N_pos(N_int) )
      N_gr(1) = K1_min
      DO I = 1,N_int
         DO K = 1,Nang
            IF( X_ang(K) < Int_gr(I) ) Then
               N_gr(I+1) = K  ;  EXIT
            ENDIF
         ENDDO
      ENDDO
      IF( N_gr(N_int+1) > K1_max ) N_gr(N_int+1) = K1_max
      WRITE(*,*) '   Position in the ionterval '
      DO I = 1,N_int
         Ks = N_gr(I) ;  Ke = N_gr(I+1)
         WRITE(*,'(5x,I3,2(2x,f7.3))') I, DACOS(X_ang(Ks))*RTD, 
     $        DACOS(X_ang(Ke))*RTD
         N_pos(I) = (Ks-1) + MINLOC( DABS(Lead(Ks:Ke)),1 )
      ENDDO

      WRITE(*,*) '  Namber of intervals containing min values '
      WRITE(*,*) '    and positions of the minimal values '
      DO I = 1,N_int
         WRITE(*,'(5x,I3,2(2x,f7.3))') I, DACOS(Int_gr(I))*RTD, 
     $        DACOS(X_ang(N_pos(I)))*RTD
      ENDDO

      WRITE(*,*) '  Minimal and Maximal value of the cone :'
      WRITE(*,'(10x,f7.3)') DACOS(X_ang(K1_min))*RTD, 
     $     DACOS(X_ang(K1_max))*RTD
      
      WRITE(*,*) ' ------------------- ___ -----------------------'
      WRITE(*,*) ' ------------------- ___ -----------------------'
      WRITE(*,*) ' ------------------- ___ -----------------------'

      ALLOCATE( SKO(K1_max), Otr(K1_max), Capr(Nlegen,N_int), 
     $     F_par(N_int), SKO_b(K1_max) )       ! A
      Otr(:) = 1

      IF( Fix_ang ) GOTO 400
c
c    ************ ********************************  *************
c            For the test purpose the approximation error 
c            of the original phase function is found 
c            as a function of the cone angle within a given
c                        angle range 
c    ************************************************************
      IF( DS_Test_run ) THEN
         ALLOCATE( RMS(K1_max) )
         Ks = 1 ; Ke = Nang
         DO K1 = K1_min,K1_max
            Pln => PL_res(:,K1) ! All polynomial for a given angle
            Alf = X_ang(K1)  ;  Af = 1.d0 - Alf
            DO I = 2, NMOM - 1
               Af = Af - MOM_USER(I)/DBLE(2*I-1)*(Pln(I+1) - Pln(I-1))
            ENDDO
            Bf = (1.d0 - Alf)*Phm1(K1) 
            Fpar = (1.d0 - 0.5d0*(Af - Bf))
c           ... The phase function without strong forward pick
            Ph_prime(1:K1)      = Phm1(K1)/Fpar
            Ph_prime(K1+1:Nang) = Phm1(K1+1:Nang)/Fpar
            DO K = 1,N_ds
               Coe(K) = (K-0.5d0)*SUM(Ph_prime(:)*PL_RES(K,:)*AG_inv(:))
            ENDDO
c           ... Using obtained expansion coefficients the 
c               trancated phase function is recalculated:
            DO I = 1, Nang
               Ph_leg(I) = SUM( PL_RES(1:N_ds,I)*Coe(1:N_ds) )
               IF(Ph_leg(I) < 0.0d0) Otr(K1) = -1
            ENDDO
c           ... Aproximation error for all scattering angles
c               Test
            Diff(:) = 1.d0 - Ph_leg(:)/Ph_prime(:)
c            Diff(:) = Ph_prime(:) - Ph_leg(:) 
            SKO(K1) = SUM( Diff(Ks:Ke)**2/DBLE(Ke-Ks+1) )
            RMS(K1) = 0.5d0*SUM( Diff(:)**2*AG_inv(:) )
            RMS(K1) = DSQRT( RMS(K1) )
         ENDDO

c         Test for phase function error in the delta-M method
         S1 = MOM_USER(NLEGEN-1+1)/DBLE(2*(NLEGEN-1) + 1)
         DO K = 1, NLEGEN-1
            Coe_dm(K) = ( MOM_USER(K) - S1*DBLE(2*K-1) )/(1.D0 - S1)
         ENDDO
         Coe_dm(NLEGEN) = 0.0d0
         write(*,*) '*** Check negative values by Delta-M approx :'
         Otr_dm = 1
         DO I = 1, Nang
            Ph_dm(I)   = SUM( PL_RES(1:NLEGEN,I)*Coe_dm(1:NLEGEN) )
            IF(Ph_dm(I) < 0.0d0 ) THEN
               Otr_dm = -1
               EXIT
            ENDIF
         ENDDO
         write(*,*) '*** End Check *****'

         OPEN( 44, file = DIR_OUT//'/SKO-DS.test' )
         DO I = K1_min,K1_max
c                  Test
            WRITE(44,'(2x,f7.3,3(2x,E12.4),2(2x,I2))')   
     $           DACOS( X_ang(I) )*RTD, DSQRT(SKO(I)), RMS(I), 
     $           Lead(I), Otr(I), Otr_dm
c            WRITE(44,'(2x,f7.3,4(2x,E12.4))')   
c     $           DACOS( X_ang(I) )*RTD, SKO(I), Lead(I)
         ENDDO
         CLOSE(44)

         STOP 'Test_run is down'
      ENDIF  ! End if for Test_run

c     ********************************************************************
c                 Calculations for the slected cat-angle
c     ********************************************************************
 400  CONTINUE
      N_start = 1 
      IF( Fix_ang ) THEN
         Kat_ang = DCOS( Kat_ang/RTD )
         DO I = 1,Nang
            IF( X_ang(I) < Kat_ang ) Then
               K1 = I-1  ;  EXIT
            ENDIF
         ENDDO
c         DO I = 1,9
c            IF( Npol(I) == N_ds ) THEN
c               K1 = Nopt(I)
c               EXIT
c            ENDIF
c         ENDDO         
         N_pos(1) = K1  ;  N_end = 1
      ELSE
         N_end = N_int
      ENDIF
c     ****** End calculations for a given cone angle ***********
      WRITE(*,*) ' ***** Cat-angle for forward cone to be used **** '
      WRITE(*,*) '        snd obtained approximation error  '
      DO J = N_start, N_end
         K1 = N_pos(J)  ;  Alf = X_ang(K1)         
         Af = 1.d0 - Alf
         DO I = 2, NMOM-1
            Af = Af - MOM_USER(I)/DBLE(2*I-1)
     $           *(PL_res(I+1,K1) - PL_res(I-1,K1))
         ENDDO
         Bf = Phm1(K1)*(1.d0 - Alf)
         F_par(J) = (1.d0 - 0.5d0*(Af - Bf))
         Ph_prime(1:K1)      = Phm1(K1)/F_par(J)
         Ph_prime(K1+1:Nang) = Phm1(K1+1:Nang)/F_par(J)
         DO K = 1,N_ds
            Capr(K,J) = (K-0.5d0)*SUM(Ph_prime(:)*PL_RES(K,:)*AG_inv(:))
         ENDDO
         Capr(NLEGEN,J) = 0.0d0
c     ... Aproximation error for all scattering angles
         DO I = 1, Nang
            Ph_leg(I) = SUM( PL_RES(1:N_ds,I)*Capr(1:N_ds,J) )
            IF(Ph_leg(I) < 0.0d0) Otr(J) = -1
         ENDDO
         Ks = 1 ; Ke = Nang
         Diff(:) = 1.d0 - Ph_leg(:)/Ph_prime(:)
         SKO(J) = SUM( Diff(Ks:Ke)**2/DBLE(Ke-Ks+1) )

         Ks = Nang/2 ; Ke = Nang
         SKO_b(J) = SUM( Diff(Ks:Ke)**2/DBLE(Ke-Ks+1) )

         WRITE(*,'(10x,2(2x,I5),f7.3,2(2x,E12.4),2x,e16.8,2x,I2)') 
     $        J, K1, DACOS( X_ang(K1) )*RTD, SKO(J), SKO_b(J), 
     $        SUM( Ph_leg(:)*AG_inv(:) )/2.0d0, Otr(J)
      ENDDO                     !End do-loop over selected intervals

      IF( .NOT.Fix_ang ) THEN               
         MRS_min = MINLOC( SKO(1:N_int),1 )
         MRS_Back= MINLOC( SKO_b(1:N_int),1 )
         WRITE(*,*) ' Position of MIN MRS     : ',MRS_min
         WRITE(*,*) ' Position of MIN MRS back: ',MRS_back
         IF( Otr(MRS_min) == -1 ) THEN
            WRITE(*,*) '**************************************'
            WRITE(*,*) ' Approx. Ph negative at optimal angle '
            WRITE(*,*) '        for all directions            '
            WRITE(*,*) '**************************************'
         ENDIF
         IF( Otr(MRS_back) == -1 ) THEN
            WRITE(*,*) '**************************************'
            WRITE(*,*) ' Approx. Ph negative at optimal angle '
            WRITE(*,*) '        for backward directions       '
            WRITE(*,*) '**************************************'
         ENDIF
         IF( MRS_min > MRS_back ) MRS_min = MRS_back
         Fpar = F_par(MRS_min)  ;  F2 = 1.d0 - F_par(MRS_min) 
         Coe(:) = Capr(:,MRS_min)

         CON_VAL_CLD(NCL) = DACOS(X_ANG(N_pos(MRS_min)))*RTD

         OPEN( 44, file = DIR_OUT//'/Opt_ang.dat' )
         WRITE( 44,'(3(2x,e15.7),2x,I2)') 
     $        DACOS(X_ANG(N_pos(MRS_min)))*RTD, SKO(MRS_min), 
     $        SKO_b(MRS_back), Otr(MRS_min)
         DO I = 1, N_int
            WRITE( 44,'(3(2x,e15.7),2x,I2)') 
     $           DACOS(X_ANG(N_pos(I)))*RTD, SKO(I), SKO_b(I), 
     $           Otr(I)
         ENDDO
         CLOSE(44)

      ELSE
         Fpar = F_par(1)  ;  F2 = 1.d0 - F_par(1) ;  Coe(:) = Capr(:,1)
         CON_VAL_CLD(NCL) = DACOS(X_ANG(N_pos(1)))*RTD
      ENDIF

c     ... Reproduce the phase function using obtained Leg. coeff.
      IF( OUT_PH_CLD ) THEN
         DO I = 1, Nang
            Ph_leg(I) = SUM( PL_RES(1:N_ds,I)*Coe(1:N_ds) )
         ENDDO
         OPEN( 44, file = Path_PH_CLD(NCL)%S )
         WRITE(44,'(2A)') 
     $        '#  Additional output file for phase functions ',
     $        'employing Delta-S approach'
         WRITE(44,'(A,2x,I5)')
     $        '#   Number of coeff. in the original Ph          :', NMOM
         WRITE(44,'(A,2x,I5)')
     $        '#   Number of coeff. used for the approximate Ph :', N_ds
         WRITE(44,'(A,2x,f6.3,1x,A3)')
     $        '#   Value of the forward cone                     :',
     $        CON_VAL_CLD(NCL)
         WRITE(44,'(A)') '#   Columns:'
         WRITE(44,'(A)') '#   1 - Cosine of the scattering angle'
         WRITE(44,'(A)') '#   2 - scattering angle, deg'
         WRITE(44,'(A)') '#   3 - original phase function'
         WRITE(44,'(A)') '#   4 - approximate phase function'
         DO I = 1, Nang
            WRITE( 44,'(7(2x,e15.7))') 
     $           X_ANG(I), DACOS(X_ANG(I))*RTD, Phm1(I), Ph_leg(I)*Fpar
         ENDDO
         CLOSE(44)
      ENDIF
      END SUBROUTINE Delta_S

c ********************************************************************
c                  Delta-Sfit
c*********************************************************************
      SUBROUTINE Delta_Sfit( NMOM, Nang )
c       Input parameters:
c                       NMOM - number of expansion coefficients
c                       Nang - number of discrete angles used for the 
c                              representation of the phase function 
c                              in [0-180] deg scattering angles.
      LOGICAL   Fix_ang, SVD, DF_Test_Run
      INTEGER   NMOM, Nang, I, J, K, K1, Ks, Ke, N_ds, 
     $     N_ret, K1_max, K1_min, K_end
      INTEGER, ALLOCATABLE:: Otr(:)
      REAL (KND)  Kat_ang, Alf, Con_min, Con_max
      REAL (KND), ALLOCATABLE:: SKO(:)
      REAL (KND), DIMENSION (Nang) :: X_ang, X_gau, AG_gau, 
     $     Phm1, Diff, Ph_leg
      REAL (KND)  PL_RES( NMOM,Nang )
      REAL (KND), ALLOCATABLE::  A_M(:,:), AINV(:,:), A_VEC(:), 
     $     PL_Nor(:,:), PL_NOR1(:,:), Ylef(:), Retr(:), 
     $     H1(:), Dpr(:)

      WRITE(*,*) '   *********** Delta-fit **************'
      WRITE(*,*) '  NMOM and Nang '
      WRITE(*,*) NMOM, Nang

      SVD         = .FALSE.
      DF_Test_Run = .FALSE.

      Con_min = 1.d0  ; Con_min = DCOS( Con_min/RTD ) 
      Con_max = 10.0d0 ; Con_max = DCOS( Con_max/RTD ) 

      SELECT CASE ( MAN_AUTO_CLD(NCL) )
      CASE( 'manu' )
         Fix_ang = .TRUE.  ;  Kat_ang  = LC_CLD(NCL)         
      CASE( 'auto' )
         Fix_ang = .FALSE. 
         Con_min = LC_CLD(NCL)  ; Con_min = DCOS( Con_min/RTD ) 
         Con_max = UC_CLD(NCL)  ; Con_max = DCOS( Con_max/RTD ) 
      END SELECT

c     ... To compare D-S results to the D-M method number of polynomials 
c        for the expansion of the truncated phase function will be used as:
      N_ds = NLEGEN - 1

c     N_ret = N_ds - 1
      N_ret = N_ds
      ALLOCATE( A_M(N_ret,N_ret), AINV(N_ret,N_ret),  
     $     Pl_Nor(N_ret,Nang), Pl_Nor1(N_ret,Nang),
     $     A_VEC( (N_ret + 1)*(N_ret + 2)/2 ),
     $     Ylef(Nang), Retr(N_ret), H1(N_ret) )

c     ... Create Gauss nodes and weighs in the interval [-1,1]
      CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_gau, Nang )
c     ... Invert Gauss grid to start from zero scattering angle 
      X_ang(:) = X_gau(Nang:1:-1) 
      
c     ... Legendre polynomials for all Gauss angles
      CALL WIGNER_PLG( X_ang, Nang, 0, NMOM, PL_RES )

c     ... Calculate phase function for all Gauss nodes using moments 
c        of the phase function
      DO I = 1, Nang
         Phm1(I) = SUM( PL_RES(:,I)*MOM_USER(:) )
      ENDDO
c     ... The positions of the cone boarders
      DO I = 1,Nang
         IF( X_ang(I) < Con_min ) Then ; K1_min = I-1 ; EXIT ; ENDIF
      ENDDO
      DO I = 1,Nang
         IF( X_ang(I) < Con_max ) Then ; K1_max = I   ;  EXIT ; ENDIF
      ENDDO

      WRITE(*,*) '  Minimal and Maximal value of the cone :'
      WRITE(*,'(10x,f7.3)') DACOS(X_ang(K1_min))*RTD, 
     $     DACOS(X_ang(K1_max))*RTD

      ALLOCATE( SKO(K1_max), Dpr(K1_max), Otr(K1_max)  ) 
      SKO = 0.0d0
      IF( Fix_ang ) GOTO 400
c
c    ************  Optimal value of the cone angle  *************
c          The iterative procedure is used to obtain optimal 
c                 value of the cat-angle
      Otr(:) = 1 ; Dpr(K1_min) = 1.d05 ; K_end = 0
      DO K1 = K1_min,K1_max 
         Alf = X_ang(K1) 
         PL_Nor = 0.0d0 ; PL_NOR1 = 0.0d0
         DO K = 1,Nang
            PL_Nor(1:N_ret,K) =  PL_RES(1:N_ret,K)/Phm1(K)
         ENDDO
         Ylef(:) = 1.d0 
         PL_NOR1( 1:N_ret,K1:Nang) = PL_NOR( 1:N_ret,K1:Nang)      
         A_M(:,:) = MATMUL( PL_NOR(1:N_ret,:),
     $        TRANSPOSE(PL_NOR1(1:N_ret,:)) )

         FORALL (I = 1:N_ret) 
     $        A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = A_M(I,1:I)

         IF( SVD ) THEN
            CALL MATRIX_INVERSE_SVD ( A_M, N_ret, N_ret, AINV )
            H1(:)  = MATMUL( PL_NOR1(1:N_ret,:),Ylef(:) )
            Retr(:) = MATMUL( AINV,H1 )
         ELSE
            CALL CHINV3 (N_ret*(N_ret+1)/2, N_ret, A_VEC)
            FORALL (I = 1:N_ret) 
     $           A_M(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
            FORALL (I = 1:N_ret, J = 1:N_ret, J > I) A_M(I,J) = A_M(J,I)
            H1(:)  = MATMUL( PL_NOR1(1:N_ret,:),Ylef(:) )
            Retr(:) = MATMUL( A_M,H1 )
         ENDIF
         Coe(1:N_ds) = Retr(:)

c        ... Using obtained expansion coefficients, the 
c          trancated phase function is recalculated:
         DO I = 1, Nang
            Ph_leg(I) = SUM( PL_RES(1:N_ds,I)*Coe(1:N_ds) )
            IF(Ph_leg(I) < 0.0d0) Otr(K1) = -1            
         ENDDO

c        ... Aproximation error for the scattering angles outside the cone
c            The approximation error increases for the increasing alpha 
c            for all range of the scattering angles 
         Ks = K1 ; Ke = Nang
         Diff(:) = (Phm1(:) - Ph_leg(:))/Phm1(:)*100.0d0
         SKO(K1) = SUM( Diff(Ks:Ke)**2/DBLE(Ke-Ks+1) )
         WRITE(*,*) K1, SKO(K1)

         IF( K1 > K1_min+1 ) THEN 
            Dpr(K1) = (1.d0 - SKO(K1)/SKO(K1-1))*100.0d0
         ENDIF
         IF( Otr(K1) == -1 ) THEN
            WRITE(*,*) '  *******  Ph is negative **************** '
            K_end = K1 - 1
            EXIT
         ENDIF
         IF( Dpr(K1) < 0.1d0 ) THEN
            K_end = K_end + 1
            IF( K_end > 4 ) THEN
               WRITE(*,*) '  -----> For the cone : ', K1, SKO(K1)
               WRITE(*,*) '  **************************************** '
               WRITE(*,*) '  End of do loop Dpr < 0.1%           '
               WRITE(*,*) K1
               WRITE(*,*) '  **************************************** '
               K_end = K1 - 4
               EXIT
            ENDIF
         ELSE
            K_end = 0
         ENDIF
      ENDDO
      IF( K_end == 0 ) K_end = K1 - 1
      WRITE(*,*) ' End value of K_end ', K_end
      K1 = K_end

      OPEN( 44, file = DIR_OUT//'/SKO-DF.test' )
      DO J = K1_min,K1_max
         WRITE(44,'(2x,f7.3, 2x,E12.4, 2x,I2)') 
     $        DACOS( X_ang(J) )*RTD, DSQRT(SKO(J)), Otr(J)
         IF( Otr(J) == -1 ) EXIT
      ENDDO
      CLOSE(44)
      IF( DF_Test_Run ) THEN 
         STOP ' ----> Stop in "general_cloud_init.f". Test run is done.'
      ENDIF
c     ********************************************************************
c                 Calculations for the slected cat-angle
c     ********************************************************************
c     ... Optimal value of the cat-angle:

 400  CONTINUE
      IF( Fix_ang ) THEN
         Kat_ang = DCOS( Kat_ang/RTD )
         DO I = 1,Nang
            IF( X_ang(I) < Kat_ang ) Then
               K1 = I  ;  EXIT
            ENDIF
         ENDDO
      ENDIF
      Alf = X_ang(K1)
      CON_VAL_CLD(NCL) = DACOS(Alf)*RTD

      WRITE(*,*) ' ***** Cat-angle for forward cone**** '
      WRITE(*,'(2x,i3,2(2x,e15.7))')  K1, CON_VAL_CLD(NCL) 

      PL_Nor = 0.0d0 ; PL_NOR1 = 0.0d0
      DO K = 1,Nang
         PL_Nor(1:N_ret,K) =  PL_RES(1:N_ret,K)/Phm1(K)
c         Ylef(K) = 1.d0 - 1.d0/Ph_prime(K) 
         Ylef(K) = 1.d0 
      ENDDO
      PL_NOR1( 1:N_ret,K1:Nang) = PL_NOR( 1:N_ret,K1:Nang)

      A_M(:,:)=MATMUL( PL_NOR(1:N_ret,:),TRANSPOSE(PL_NOR1(1:N_ret,:)) )

      IF( SVD ) THEN
         CALL MATRIX_INVERSE_SVD ( A_M, N_ret, N_ret, AINV )
         H1(:)  = MATMUL( PL_NOR1(1:N_ret,:),Ylef(:) )
         Retr(:) = MATMUL( AINV,H1 )
      ELSE
         FORALL (I = 1:N_ret) 
     $        A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = A_M(I,1:I)
         CALL CHINV3 (N_ret*(N_ret+1)/2, N_ret, A_VEC)
         FORALL (I = 1:N_ret) 
     $        A_M(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
         FORALL (I = 1:N_ret, J = 1:N_ret, J > I) A_M(I,J) = A_M(J,I)
         H1(:)  = MATMUL( PL_NOR1(1:N_ret,:),Ylef(:) )
         Retr(:) = MATMUL( A_M,H1 )
      ENDIF

      R_cof(:) = Retr(:)
      WRITE(*,*) '   Fpar delta-sfit = ', Retr(1)

c     ... Reproduce the phase function using obtained Leg. coeff.
      IF( OUT_PH_CLD ) THEN
         DO I = 1, Nang
            Ph_leg(I)  = SUM( PL_RES(1:NLEGEN,I)*Retr(1:NLEGEN) )
         ENDDO
         OPEN( 44, file = Path_PH_CLD(NCL)%S )
         WRITE(44,'(2A)') 
     $        '#  Additional output file for phase functions ',
     $        'employing Delta-fit approach'
         WRITE(44,'(A,2x,I5)')
     $        '#   Number of coeff. in the original Ph          :', NMOM
         WRITE(44,'(A,2x,I5)')
     $        '#   Number of coeff. used for the approximate Ph :', N_ds
         WRITE(44,'(A,2x,f6.3,1x,A3)')
     $        '#   Value of the forward cone                     :',
     $        CON_VAL_CLD(NCL)
         WRITE(44,'(A)') '#   Columns:'
         WRITE(44,'(A)') '#   1 - Cosine of the scattering angle'
         WRITE(44,'(A)') '#   2 - scattering angle, deg'
         WRITE(44,'(A)') '#   3 - original phase function'
         WRITE(44,'(A)') '#   4 - approximate phase function'
         DO I = 1, Nang
            WRITE( 44,'(7(2x,e15.7))') 
     $           X_ANG(I), DACOS(X_ANG(I))*RTD, Phm1(I), Ph_leg(I)
         ENDDO
         CLOSE(44)
      ENDIF
c      STOP
      END SUBROUTINE Delta_Sfit

c*********************************************************************
      SUBROUTINE ASYMPTOTIC_INF
      Implicit None
      REAL (KND)  SUNZEN, LOSZEN, RELAZM,  PRD, S1, 
     $     Scat, Muc, Muc0, P_LEG(N_MOM_USER,1)
      REAL (KND), ALLOCATABLE :: P_SCA(:,:)
c     ------------------------------------------------------------------

      ALLOCATE( R_INF_WL(NCW),G_CLD_WL(NCW) )

      G_CLD_WL(:) = MOM_DB(:,2,Ncl,1,1,1)/3.d0
      SUNZEN = DACOS(ANG_OUT(1,1))*RTD
      LOSZEN = DACOS(DABS(ANG_OUT(2,1)))*RTD
      RELAZM = ANG_OUT(3,1)
      
c          WRITE( *,* ) '6... asymptotic ...'
      Muc = DCOS( LOSZEN/RTD )
      Muc0 = ANG_OUT(1,1)
      PRD = Muc*Muc0

c      CALL GT_R_INFINITY ( sunzen, loszen, relazm, REFL, ERROR )
c      write(*,'(2x,3(2x,e12.4))') 
c     $    REFL, r_inf, (REFL - r_inf)/REFL*100.

c      WRITE(*,*) '   Number of NCW = ',NCW

      SELECT CASE( TDS(NCL)%S )
      CASE('water')
         IF( LOSZEN > 0.5 ) THEN
            ALLOCATE( P_SCA(N_MOM_USER,1) )
            DO I = 1, N_MOM_USER
               P_LEG(I,1) = PLGNDR(I-1,0,-Muc)*PLGNDR(I-1,0,Muc0)
            ENDDO
            SCAT = -PRD + DSQRT((1.d0-Muc*Muc)*(1.d0-Muc0*Muc0))
     $           *DCOS( RELAZM )
            CALL WIGNER_PLG([SCAT], 1, 0, N_MOM_USER, P_SCA)
            DO I = 1, NCW
               MOM_USER => MOM_DB(I,1:N_MOM_USER,NCL,1,1,1)
               S1 = SUM( (P_SCA(:,1)-P_LEG(:,1))*MOM_USER(:) )
               WRITE(*,*) I, S1
               R_INF_WL(I) = 
     $              (4.d0 - 2.5d0*(Muc + Muc0) + 10.3d0*PRD + S1)
     $              /(Muc + Muc0)/4.D0
            ENDDO
         ELSE
            SCAT = DCOS(PIE - SUNZEN/RTD)
            CALL WIGNER_PLG([SCAT], 1, 0, N_MOM_USER, P_LEG)
            DO I = 1, NCW
               MOM_USER => MOM_DB(I,1:N_MOM_USER,NCL,1,1,1)
               P_TETA = SUM( P_LEG(:,1)*MOM_USER(:) )
               WRITE(*,*) I, P_TETA
               R_INF_WL(I) = 
     $              (1.48D0 + 7.76D0*PRD + P_TETA)/(Muc + Muc0)/4.D0
            ENDDO
         ENDIF
      CASE('ice')
         SCAT = -PRD + DSQRT((1.d0-Muc*Muc)*(1.d0-Muc0*Muc0))
     $        *DCOS( RELAZM )
         CALL WIGNER_PLG([SCAT], 1, 0, N_MOM_USER, P_LEG)
         DO I = 1, NCW
            MOM_USER => MOM_DB(I,1:N_MOM_USER,NCL,1,1,1)
            P_TETA = SUM( P_LEG(:,1)*MOM_USER(:) )
            R_INF_WL(I) = (1.247D0 + 1.186D0*(Muc + Muc0) + 5.157D0*PRD
     $           + P_TETA)/(Muc + Muc0)/4.0D0
         ENDDO
      END SELECT
      
      RETURN
      END SUBROUTINE ASYMPTOTIC_INF

c *****************************************************************

      SUBROUTINE DEALLOCATE_ARRAYS
      
      IF(ALLOCATED(CLD_NETWAVS)) DEALLOCATE(CLD_NETWAVS, C_P_M,
     $     CLD_LAYER_COEFF, CLD_LAYER_DERIV, SCC_LAYER_NOSCALE,
     $     CLD_F_DM, REF_IMAG_W, REF_IMAG_I)
      IF(ALLOCATED(SCI_LAYER_NOSCALE)) DEALLOCATE(SCI_LAYER_NOSCALE)
      IF(ALLOCATED(F_Dfit)) DEALLOCATE(F_Dfit)
      IF(ALLOCATED(CLD_SAVE_COEFF)) DEALLOCATE(CLD_SAVE_COEFF)
      IF(ALLOCATED(Greek_cld)) DEALLOCATE(Greek_cld)
      IF(ALLOCATED(SCAT_MAT)) DEALLOCATE(SCAT_MAT, SCAT_ANG)
      IF(ALLOCATED(R_INF_WL)) DEALLOCATE(R_INF_WL, G_CLD_WL)
      
      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS
c *****************************************************************
      END  ! of the main subroutine "general_cloud_init.f"
C********************************************************************C

      subroutine matrix_inverse_svd ( a, adim, maxadim, ainv )

c  inversion of a square ( adim x adim )-matrix "a" 
c  ( formal dimension: maxdim x maxdim ) 
c  using singular value decomposition SVD

      implicit none

      integer           i, j, k, adim, maxadim, maxdim
      parameter         ( maxdim = 1024 )
      double precision  a ( maxadim, maxadim ), 
     $                  ainv ( maxadim, maxadim ),
     $                  u ( maxdim, maxdim ), v ( maxdim, maxdim ),
     $                  w ( maxdim ), tmp ( maxdim, maxdim )
      logical           error

c  perform SVD:
      do i = 1, maxdim
         do j = 1, maxdim
            u ( i, j ) = 0.0d0
         enddo
      enddo
      do i = 1, adim
         do j = 1, adim
            u ( i, j ) = a ( i, j )
         enddo
      enddo
      call svdcmp ( u, adim, adim, maxdim, maxdim, w, v, error )

c  calculate inverse matrix A^-1  = V * diag(1/w) * U^T :
      do i = 1, maxdim
         do j = 1, maxdim
            tmp ( i, j ) = 0.d0
         enddo
      enddo
      do i = 1, maxadim
         do j = 1, maxadim
            ainv ( i, j ) = 0.d0
         enddo
      enddo
      do i = 1, adim
         do j = 1, adim
            if ( w (i) .ne. 0.d0 ) tmp ( i, j ) = u ( j, i ) / w (i)
         enddo
      enddo
      do i = 1, adim
         do j = 1, adim
            ainv ( i, j ) = 0.0d0
            do k = 1, adim
               ainv ( i, j ) = ainv ( i, j ) 
     $              + v ( i, k ) * tmp ( k, j )
            enddo
         enddo
      enddo

      end

c  ********************************************************************
      SUBROUTINE svdcmp(a,m,n,mp,np,w,v,error)
      implicit none !rh
      INTEGER m,mp,n,np,NMAX
      DOUBLE PRECISION a(mp,np),v(np,np),w(np)
      PARAMETER (NMAX = 1024)
CU    USES pythag
      INTEGER i,its,j,jj,k,l,nm
      DOUBLE PRECISION anorm,c,f,g,h,s,scale,x,y,z,rv1(NMAX),pythag
      logical error
      error = .false.
      g=0.0d0
      scale=0.0d0
      anorm=0.0d0
      do 25 i=1,n
        l=i+1
        rv1(i)=scale*g
        g=0.0d0
        s=0.0d0
        scale=0.0d0
        if(i.le.m)then
          do 11 k=i,m
            scale=scale+abs(a(k,i))
11        continue
          if(scale.ne.0.0d0)then
            do 12 k=i,m
              a(k,i)=a(k,i)/scale
              s=s+a(k,i)*a(k,i)
12          continue
            f=a(i,i)
            g=-sign(sqrt(s),f)
            h=f*g-s
            a(i,i)=f-g
            do 15 j=l,n
              s=0.0d0
              do 13 k=i,m
                s=s+a(k,i)*a(k,j)
13            continue
              f=s/h
              do 14 k=i,m
                a(k,j)=a(k,j)+f*a(k,i)
14            continue
15          continue
            do 16 k=i,m
              a(k,i)=scale*a(k,i)
16          continue
          endif
        endif
        w(i)=scale *g
        g=0.0d0
        s=0.0d0
        scale=0.0d0
        if((i.le.m).and.(i.ne.n))then
          do 17 k=l,n
            scale=scale+abs(a(i,k))
17        continue
          if(scale.ne.0.0d0)then
            do 18 k=l,n
              a(i,k)=a(i,k)/scale
              s=s+a(i,k)*a(i,k)
18          continue
            f=a(i,l)
            g=-sign(sqrt(s),f)
            h=f*g-s
            a(i,l)=f-g
            do 19 k=l,n
              rv1(k)=a(i,k)/h
19          continue
            do 23 j=l,m
              s=0.0d0
              do 21 k=l,n
                s=s+a(j,k)*a(i,k)
21            continue
              do 22 k=l,n
                a(j,k)=a(j,k)+s*rv1(k)
22            continue
23          continue
            do 24 k=l,n
              a(i,k)=scale*a(i,k)
24          continue
          endif
        endif
        anorm=max(anorm,(abs(w(i))+abs(rv1(i))))
25    continue
      do 32 i=n,1,-1
        if(i.lt.n)then
          if(g.ne.0.0d0)then
            do 26 j=l,n
              v(j,i)=(a(i,j)/a(i,l))/g
26          continue
            do 29 j=l,n
              s=0.0d0
              do 27 k=l,n
                s=s+a(i,k)*v(k,j)
27            continue
              do 28 k=l,n
                v(k,j)=v(k,j)+s*v(k,i)
28            continue
29          continue
          endif
          do 31 j=l,n
            v(i,j)=0.0d0
            v(j,i)=0.0d0
31        continue
        endif
        v(i,i)=1.d0
        g=rv1(i)
        l=i
32    continue
      do 39 i=min(m,n),1,-1
        l=i+1
        g=w(i)
        do 33 j=l,n
          a(i,j)=0.0d0
33      continue
        if(g.ne.0.0d0)then
          g=1.d0/g
          do 36 j=l,n
            s=0.0d0
            do 34 k=l,m
              s=s+a(k,i)*a(k,j)
34          continue
            f=(s/a(i,i))*g
            do 35 k=i,m
              a(k,j)=a(k,j)+f*a(k,i)
35          continue
36        continue
          do 37 j=i,m
            a(j,i)=a(j,i)*g
37        continue
        else
          do 38 j= i,m
            a(j,i)=0.0d0
38        continue
        endif
        a(i,i)=a(i,i)+1.d0
39    continue
      do 49 k=n,1,-1
        do 48 its=1,30
          do 41 l=k,1,-1
            nm=l-1
            if((abs(rv1(l))+anorm).eq.anorm)  goto 2
            if((abs(w(nm))+anorm).eq.anorm)  goto 1
41        continue
1         c=0.0d0
          s=1.d0
          do 43 i=l,k
            f=s*rv1(i)
            rv1(i)=c*rv1(i)
            if((abs(f)+anorm).eq.anorm) goto 2
            g=w(i)
            h=pythag(f,g)
            w(i)=h
            h=1.d0/h
            c= (g*h)
            s=-(f*h)
            do 42 j=1,m
              y=a(j,nm)
              z=a(j,i)
              a(j,nm)=(y*c)+(z*s)
              a(j,i)=-(y*s)+(z*c)
42          continue
43        continue
2         z=w(k)
          if(l.eq.k)then
            if(z.lt.0.0d0)then
              w(k)=-z
              do 44 j=1,n
                v(j,k)=-v(j,k)
44            continue
            endif
            goto 3
          endif
          if(its.eq.30) then
c            stop'ERROR in SVDCMP: no convergence (its=30)'
             write(*,*) ' *** ERROR in SVDCMP: no convergence (its=30)'
             error = .true.
             return
          endif
          x=w(l)
          nm=k-1
          y=w(nm)
          g=rv1(nm)
          h=rv1(k)
          f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0d0*h*y)
          g=pythag(f,1.d0)
          f=((x-z)*(x+z)+h*((y/(f+sign(g,f)))-h))/x
          c=1.d0
          s=1.d0
          do 47 j=l,nm
            i=j+1
            g=rv1(i)
            y=w(i)
            h=s*g
            g=c*g
            z=pythag(f,h)
            rv1(j)=z
            c=f/z
            s=h/z
            f= (x*c)+(g*s)
            g=-(x*s)+(g*c)
            h=y*s
            y=y*c
            do 45 jj=1,n
              x=v(jj,j)
              z=v(jj,i)
              v(jj,j)= (x*c)+(z*s)
              v(jj,i)=-(x*s)+(z*c)
45          continue
            z=pythag(f,h)
            w(j)=z
            if(z.ne.0.0d0)then
              z=1.d0/z
              c=f*z
              s=h*z
            endif
            f= (c*g)+(s*y)
            x=-(s*g)+(c*y)
            do 46 jj=1,m
              y=a(jj,j)
              z=a(jj,i)
              a(jj,j)= (y*c)+(z*s)
              a(jj,i)=-(y*s)+(z*c)
46          continue
47        continue
          rv1(l)=0.0d0
          rv1(k)=f
          w(k)=x
48      continue
3       continue
49    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software )&%.

      FUNCTION pythag(a,b)
      DOUBLE PRECISION a,b,pythag
      DOUBLE PRECISION absa,absb
      absa=abs(a)
      absb=abs(b)
      if(absa.gt.absb)then
        pythag=absa*sqrt(1.d0+(absb/absa)**2)
      else
        if(absb.eq.0.d0)then
          pythag=0.d0
        else
          pythag=absb*sqrt(1.d0+(absa/absb)**2)
        endif
      endif
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software )&%.

c  ********************************************************************
      Function Time_in_Seconds ()
      Double Precision  Time_in_Seconds
! Local Parameters
      character (len=10) time
      Double Precision  Hrs, Mins, Secs, tt
!==================================================================
      call DATE_AND_TIME(time=TIME)
      read(TIME(1:2),*) hrs
      read(TIME(3:4),*) mins
      read(TIME(5:),*) secs
      Time_in_Seconds = Hrs*3600.+ Mins*60. + Secs
      Return
      End Function Time_in_Seconds
