!***************************************************************************
!
!	SCIATRAN - Version 3.9 - Last modification: 21 Dec 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE GEOM_ALONG_LOS (*)
  ! Called by "rtm_init.f" if RTM_TYPE is set to 'spher_scat'

  USE GRIDS,        ONLY: NLAYERS, ZZZ
  USE LOS_ANGLES,   ONLY: N_BELOW, N_BE, IND_LIM_A, IND_LIM_B, LIM_GR, N_AF
  USE LOG_PARS,     ONLY: LOG_LEVEL, N_DSO, LUNIT, EU
  USE RAMAN,        ONLY: RRS_ADJ
  USE SPHER_PARS,   ONLY: NN, NLOS => N_LOS_ANG, NFA => N_FOV_ANG, N_OBS_ALT, & 
                          NBE_INP, NAF_INP, DEL_INP, ONE_MS_SZ, LW_FOV
  USE ST_VARS,      ONLY: N_Adj, WF_FLAG, ID_LIM, M_LIM, SHI_LIM, &
                          SHI_LIM_C, NS_wf, Add_lev, GeoD => GeoD_OUT
  USE TYPES,        ONLY: KND
! ------------------------------------------------------------------------  
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GEOM_ALONG_LOS'
  INTEGER   I, K, NTH, IA, K1, K2, Jv, Jvs, Jf, NLS, NN1, NN2, Jt, Ifov, &
       IM1, IM2
  INTEGER, ALLOCATABLE :: IH(:)
  REAL (KND) SD, Z_end, Mu_los
  REAL (KND) Del_1(NLAYERS), Z_1(NLOS*NFA,NLAYERS), &
       Z_2(NLOS*NFA,NLAYERS)
  REAL (KND), POINTER :: ZLS(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  IA = 1
  IF(ALLOCATED(ID_LIM)) DEALLOCATE(ID_LIM, N_BELOW, N_BE, N_AF)
  ! For the central line-of-sights only
  ! N_Adj defines full number of right-hand sides which will be used
  !   for the solution of the adjoint RTE in the spherical case 
  !   if the calculation of the intensity only is needed.
  N_Adj = NLOS
  
  !... In the case of 'pp_scat' or 'ps_scat'
  !    N_Adj = NS_wf*NLSA is set in subroutine "rtm_init.f" 
  IF( .NOT.ALLOCATED(LW_FOV) ) ALLOCATE( LW_FOV(NLOS) )
  LW_FOV(:) = (NFA-1)/2 ! middle 

  ! ************** If the weighting functions are required *************
  !                  or Raman scattering employing 
  !                   forward-adjoint technique
  IF( WF_FLAG.OR.RRS_ADJ ) THEN
!     write(*,*) '   ==> geom_along_los: '
     ALLOCATE( ID_LIM(NLOS*NFA,NLAYERS,2), N_Below(NLOS*NFA), &!A 
          N_BE (NLOS*NFA), N_AF (NLOS*NFA)) !A
     Z_2 = -1.0d0 ; Z_1 = -1.0d0 ; ID_LIM = 0         
     ! Each line-of-sight is divided on the sub-intervals.
     ! N_BE - the number of sub-intervals before tangent hight
     ! N_AF - the number of sub-intervals after tangent hight

     ! ... In the case of the integration over an instrument 
     !     field-of-view the numbers of N_BE and N_AF have to be 
     !     the same for all line-of-sights within field-of-view !
!        write(*,*) '    ONE_MS_SZ  ', ONE_MS_SZ
     Del_1(:) = DEL_INP
     IF(ONE_MS_SZ) THEN
        N_BE(:) = 1  ;  N_AF(:) = 1
     ELSE
        N_BE(:) = NBE_INP   ! Number of layers before Th (see "gt_fileread.f90")
     END IF

     DO Jf = 1, NLOS*NFA  ! over all LoS within FoV
        NTH = GeoD%LVG(IA,Jf)%NTH  
!       ... Number of points along LoS
        NLS = GeoD%LVG(IA,Jf)%NLS 
        Mu_los = GeoD%LVG(IA,Jf)%C_VA(1) 
!       ... Fine-grid altitudes along LoS
        ZLS => GeoD%LVG(IA,Jf)%ZLS           
        ! Z_1(Jf,1) altitudes of TH
        Z_1(Jf,1) = GeoD%LVG(IA,Jf)%Th_V
        
        IF( Mu_los > 0.d0 ) THEN ! LoS directed downward
!           WRITE(*,*) '  LoS  directed downward '
           Z_end = GeoD%Alt(IA)
           IF( NLS <= NTH ) THEN
              N_Below(Jf) = NLAYERS ; NN1 = NBE_INP
           ELSE
!             ... Save level number of the standard altitude grid  
!                 which is below or equal to the tangent height
              N_Below(Jf) =  GeoD%LVG(IA,Jf)%IOB(NTH)
              NN1 = NBE_INP  ;  NN2 = NAF_INP
           ENDIF
        ELSE   ! ***** LoS directed upward *****
!           WRITE(*,*) '  LoS  directed upward '
!          ... Z_end - altitudes of observation point and TOA
           Z_end = ZZZ(1)
!          ... Save level number of the standard altitude grid 
!              which is below or equal to the altitude of observer position
           N_Below(Jf) = N_OBS_ALT(1)
           NN1 = NAF_INP
        ENDIF  ! Mu_los > OR < 0.
!       ----------------------------------------------------------------
        IF (LOG_LEVEL(1) >= N_DSO)  THEN
           IF( Jf==1 ) WRITE(*,*) '    Jf,  Th_V,  Z below Th '
           WRITE(*,'(2x,I3,2(2x,F8.3))') &
                Jf, GeoD%LVG(IA,Jf)%Th_V, ZZZ(N_Below(Jf))
        ENDIF
!       ----------------------------------------------------------------
!       ... Calculate altitudes of sub-intervals along LoS starting from
!           Z_1(Jf,1) till Z_end
!        WRITE(*,*) '  Altitude of Th ', Z_1(Jf,1), ZZZ(N_Below(Jf))
        K = 1
!        DO I = 2,NLAYERS
        DO I = 2,NN1+1
           IF( REAL(Z_1(Jf,I-1)) == REAL(Z_end) ) THEN
              IF( K == 1 ) STOP '  K=1 !!! geom_along_los.f90 !!! '
              NN1 = K - 1 ; EXIT
           ENDIF
           SD = Z_1(Jf,I-1) + Del_1(K)
           IF( REAL(SD) >= REAL(Z_end) ) THEN
              NN1 = K ; EXIT
           ELSE
              K1 = MINLOC( DABS(ZZZ(1:NLAYERS) - SD),1 )
              Z_1(Jf,I) = ZZZ(K1) ; K = K + 1
!             ... Next selected altitude can not be below or equal previous one
              IF( Z_1(Jf,I) <= Z_1(Jf,I-1) ) Z_1(Jf,I) = ZZZ(K1-1)
           ENDIF
        ENDDO

!        IF( DO_SATELLITE.AND.Add_lev ) THEN
!           Z_1(Jf,N_BE(Jf)+1) = ZZZ(N_OBS_ALT(1)-1)
!        ELSE
!           Z_1(Jf,N_BE(Jf)+1) = Z_end
!        ENDIF
        Z_1(Jf,NN1+1) = Z_end

        ! Will be counted from top to down
        Z_1(Jf,1:NN1+1) = Z_1(Jf,NN1+1:1:-1) 
        
!       ... Calculate altitudes of sub-intervals alog LoS starting from
!           Th till TOA (if Mu_los > 0 and NLS > NTH)
!        WRITE(*,*) ' ---- Mu_los =  ', Mu_los
        IF( Mu_los > 0 ) THEN
           IF( NLS > NTH ) THEN
              Z_2(Jf,1) = GeoD%LVG(IA,Jf)%Th_V ; K = 1
              DO I = 2,NN2
!             ... Previous selected altitude can not be equal to TOA
                 IF( REAL(Z_2(Jf,I-1)) == REAL(ZZZ(1)) ) THEN
                    NN2 = K - 1 ; EXIT
                 ENDIF
                 SD = Z_2(Jf,I-1) + Del_1(K)
                 K1 = MINLOC( DABS(ZZZ(1:NLAYERS) - SD),1 )
                 Z_2(Jf,I) = ZZZ(K1) ; K = K + 1
!             ... Next selected altitude can not be below or equal previous one
                 IF( Z_2(Jf,I) <= Z_2(Jf,I-1) )  Z_2(Jf,I) = ZZZ(K1-1)
              ENDDO
              Z_2(Jf,NN2+1)   = ZZZ(1)
              Z_2(Jf,1:NN2+1) = Z_2(Jf,NN2+1:1:-1) 
              N_AF(Jf) = NN2 ; N_BE(Jf) = NN1
           ELSE
              N_AF(Jf) = 0   ; N_BE(Jf) = NN1
           ENDIF
        ELSE
           N_AF(Jf) = 0    ; N_BE(Jf) = NN1
        ENDIF
!       ================================================================
        IF (LOG_LEVEL(1) >= N_DSO)  THEN
                write(*,'(A,I3)') '   N_Be(Jf) = ', N_Be(Jf)
                write(*,'(A,I3)') '   N_AF(Jf) = ', N_AF(Jf)
           WRITE(*,*) '**** selected altitudes of subintervals ******'
           WRITE(*,*) ' N Int. Be Th  Af Th (-1 if not present)'
           DO I = 1, MAX(N_BE(Jf), N_AF(Jf)) + 1
              WRITE(*,'(2x,I3,2(2x,f7.3))') I, Z_1(Jf,I), Z_2(Jf,I)
           ENDDO
           WRITE(*,*) '****------------------------------------******'
        ENDIF
!       ================================================================
        ! Full number of sub-intervals along line-of-sight:
        M_LIM(Jf) = N_BE(Jf) + N_AF(Jf)
        DO I = 1, NLAYERS-1 
           DO K = 1,NN1
              IF( ZZZ(I) <= Z_1(Jf,K).AND.ZZZ(I) > Z_1(Jf,K+1)) THEN
                 ID_LIM(Jf,I,1) = K
              ENDIF
           ENDDO  ! over sub-intervals before tangent height (K)
           IF( Mu_los > 0 .AND. NLS > NTH ) THEN
              DO K = 1,N_AF(Jf)
                 IF( ZZZ(I) <= Z_2(Jf,K).AND.ZZZ(I) > Z_2(Jf,K+1)) THEN
                    ID_LIM(Jf,I,2) = K + N_BE(Jf)
                 ENDIF
              ENDDO  ! over sub-intervals after tangent height (K)
           ENDIF
        ENDDO  ! over standard layers (I)
     ENDDO  ! over line-of-sight within field-of-view (Jf)

     IF( NFA > 1 ) THEN
        ALLOCATE( IH(NFA) )
        DO Jv = 1,NLOS
           K1 = (Jv-1)*NFA+1  ;  K2 = Jv*NFA
           IM1 = MAXLOC(M_LIM(K1:K2),1)

!           IM2 = MAXLOC(M_LIM(K1:K2),1,BACK=.TRUE.)
!           IF( IM1==IM2 ) THEN
!              LW_FOV(Jv) = NFA - IM1
!           ELSE
!              LW_FOV(Jv) = (NFA-1)/2 ! middle
!           ENDIF

           IH(:) = M_LIM(K1:K2) ;  IH(1:NFA) = IH(NFA:1:-1)
           IM2 = MAXLOC(IH(:),1)
           IF( IM1==IM2 ) THEN
              LW_FOV(Jv) = (NFA-1)/2 ! middle
           ELSE
              LW_FOV(Jv) = NFA - IM1
           ENDIF

!           do i = k1, k2
!              write(*,*) Jv, I,  M_LIM(I)
!           enddo
!           WRITE(*,*) ' IM1 and IM2 ', IM1, IM2
!           write(*,*) '  Selected direction  ', LW_FOV(Jv)
        ENDDO
     ENDIF  ! NFA > 1
!! ***************************************************************************
     IF (LOG_LEVEL(1) >= N_DSO)  THEN
        WRITE(*,*) ' _________ ID_LIM for all LOS ___________ '
        DO Jv = 1,NLOS
           K1 = NFA*(Jv - 1)
           WRITE(*,*) ' LOS ___________ = ', Jv
           DO I = 1,NLAYERS
              WRITE(*,'(2x,I3,2x,f8.4,2x,*(1x,i2))') I, ZZZ(I),& 
                   ( ID_LIM(K + K1,I,1),ID_LIM(K + K1,I,2), K = 1,NFA)
           ENDDO  ! over altitude levels (I)
        ENDDO  ! over line-of-sight (Jv)
!      WRITE(*,*) '     ====================================  '
!      Jv = 0
!      DO Jt = 1, NLOS*NFA, NFA  
!         Jv = Jv + 1 ; Ifov = Jv*NFA-LW_FOV(Jv)
!         Nth = N_below(Ifov)
!         WRITE(*,*) '   N_below(Ifov) ', N_below(Ifov)
!         DO I = Nth-3, Nth-1 
!            WRITE(*,*) I, ID_LIM(Ifov,I,1)
!         ENDDO
!      ENDDO
!
!      WRITE(*,*) '     '
!      WRITE(*,*) '   M_LIM(:)   ', M_LIM(:) 
!      WRITE(*,*) '    N_Be(:)   ', N_Be(:) 
!      WRITE(*,*) ' _________  ___________ '
!     ! PAUSE '  Pause in "geom_along_los.f"  '
     ENDIF  ! debug output
!! ***************************************************************************

!      STOP '  Pause in "geom_along_los.f"  '
! ***************************************************************************
!      WRITE(*,*) '  All M_lim ', M_LIM(:)
      
     K1 = MAXVAL( M_LIM(:) )
     IF(.NOT.ALLOCATED(IND_LIM_A) ) THEN
        ALLOCATE(IND_LIM_A(NLOS*NFA,K1+1), IND_LIM_B(NLOS*NFA,K1+1),  & !A
             SHI_LIM(NLOS*NFA), SHI_LIM_C(NLOS), Lim_gr(NLOS,K1,2) )    !A
     ENDIF

     ! Lim_gr(:,:,1) gives index of the initial altitude grids 
     !               corresponding to the top of limb layering
     ! Lim_gr(:,:,2) the same but for the bottom of limb layering
     ! !!! This array is used in the subroutine "part_sol_arte_spher.f" !!!
     ! !!!  if option ADAPTIVE_GRID is switched on.                     !!!
     DO Jv = 1,NLOS
        K2 = Jv*NFA - LW_FOV(Jv)
        DO K1 = 1,N_BE(K2)
           lim_gr(Jv,K1,1) = MINLOC( DABS(ZZZ(1:NLAYERS) - Z_1(K2,K1)),1)
           lim_gr(Jv,K1,2) = MINLOC( DABS(ZZZ(1:NLAYERS) - Z_1(K2,K1+1)),1 )
        ENDDO
        lim_gr(Jv,N_BE(K2),2) = N_Below(K2)
        ! Tangent height is not present:
        IF(GeoD%LVG(IA,K2)%NTH == GeoD%LVG(IA,K2)%NLS) CYCLE
        K = 1
        DO K1 = N_BE(K2)+1, M_LIM(K2)
           lim_gr(Jv,K1,1) = MINLOC( DABS(ZZZ(1:NLAYERS) - Z_2(K2,K)),1)
           lim_gr(Jv,K1,2) = MINLOC( DABS(ZZZ(1:NLAYERS) - Z_2(K2,K+1)),1 )
           K = K + 1
        ENDDO
        lim_gr(Jv,M_LIM(K2),2) = N_Below(K2) ! Check dimension
     ENDDO

     ! Output for visual control
! ***************************************************************************
     IF (LOG_LEVEL(1) >= N_DSO)  THEN         
        WRITE(LUNIT,*) '  Top positions of Levels before and after Th'
        WRITE(LUNIT,'(A)') '   ------- a selected direction within FOV ----- ' 
        WRITE(LUNIT,'(A)') '      Jv   K        Left  Right' 
        DO Jv = 1,NLOS
           Jt = Jv*NFA - LW_FOV(Jv)
           DO K = 1, N_Be(Jt)
             WRITE(LUNIT,'(2x,2(2x,I3),5x,2(2x,I4))') &
                  Jv, K, Lim_gr(Jv,K,1), Lim_gr(Jv,K,2)
           ENDDO
           WRITE(LUNIT,'(A)') '   --------------------------------------------- ' 
           K1 = 1
           DO K = N_BE(Jt)+1, M_LIM(Jt)
             WRITE(LUNIT,'(2x,2(2x,I3),5x,2(2x,I4))') &
                  Jv, K1, Lim_gr(Jv,K,1), Lim_gr(Jv,K,2)
             K1 = K1 + 1
           ENDDO
           WRITE(LUNIT,'(A)') '   --------------------------------------------- ' 
        ENDDO

        WRITE(LUNIT,*) '**** selected altitudes of subintervals ******'
        WRITE(LUNIT,*) '   NFA = ', NFA 
        DO Jv = 1,NLOS
           K1 = (Jv-1)*NFA  ;  K2 = Jv*NFA
           WRITE(LUNIT,*) '   '
           WRITE(LUNIT,*) '   Los angle :', Jv
           WRITE(LUNIT,*) '       ** Before Th **'
           WRITE(LUNIT,*) ' No   LoS within FOV --->'
           DO I = 1,MAXVAL( N_BE(K1+1:K2) ) + 1
              WRITE(LUNIT,'(1x,I2,20(1x,f7.2))') I, (Z_1((Jv-1)*NFA+K,I), K=1,NFA)
           ENDDO
!          ===================================================================
           WRITE(LUNIT,*) '       ** After Th **'
           WRITE(LUNIT,*) ' No   LoS within FOV --->'
           DO I = 1,MAXVAL( N_AF(K1+1:K2) ) + 1
              WRITE(LUNIT,'(1x,I2,20(1x,f7.2))') I, (Z_2((Jv-1)*NFA+K,I), K=1,NFA)
           ENDDO
        ENDDO
     ENDIF  ! output for visual control
! ***************************************************************************

     ! Full number of sub-intervals for all line-of-sight 
     ! 2 - NLOS*NFA
     SHI_LIM(1) = 0
     DO Jf = 2,NLOS*NFA
        SHI_LIM(Jf) = SUM( M_LIM(1:Jf-1) )
     ENDDO

     ! SHI_LIM_C contains information for each line-of-sight direction
     !   about full number of sub-intervals used for all
     !   previous directions.
     SHI_LIM_C(1) = 0  
     DO Jv = 2, NLOS 
        SHI_LIM_C(Jv) =  SHI_LIM_C(Jv-1) + M_LIM((Jv-1)*NFA-LW_FOV(Jv-1)) 
     ENDDO

     ! Arrays IND_LIM_B and IND_LIM_A will be used in the subroutine
     ! "appr_spher_ls_main.f" to calculate effective LoS angle
     DO Jf = 1, NLOS*NFA 
        NTH = GeoD%LVG(IA,Jf)%NTH
        NLS = GeoD%LVG(IA,Jf)%NLS
        ZLS => GeoD%LVG(IA,Jf)%ZLS           
        DO K = 1,N_BE(Jf)+1
           SD = Z_1(Jf,K)
           IND_LIM_B(Jf,K) = MINLOC( DABS(ZLS(1:NTH) - SD),1 )
        ENDDO
        IF( NLS > NTH ) THEN 
!          ... Tangent height is presented
           DO K = 1,N_AF(Jf)+1
              SD = Z_2(Jf,K)
              IND_LIM_A(Jf,K) = NTH - 1 + &
                   MINLOC( DABS(ZLS(NTH:NLS)-SD),1 )
           ENDDO
        ENDIF
     ENDDO  ! over line-of-sight including field-of-view (Jf)
     
     ! N_adj defines full number of right-hand sides 
     !    of the adjoint RTE in the case of spherical geometry 
     !    if the calculation of weighting functions is needed.
     N_adj = 0
     ! The summation below is performed for the last, middle or first 
     !    LoS direction within field-of-view depending on LW_FOV
     DO Jv = 1,NLOS  ;  Jvs = Jv*NFA - LW_FOV(Jv)
        N_adj = N_adj + M_LIM(Jvs)
     ENDDO
     ! If WFs should be calculated in spherical geometry for 
     ! NS_wf components of Stokes vector then
     N_adj = N_adj*NS_wf

!     write(*,*) '   <== geom_along_los: '
  ENDIF  ! WF_FLAG.OR.RRS_ADJ
!         WRITE(*,*) '   "geom_along_los" '
!         WRITE(*,'(A,*(2x,I3))') '    Number of Nlos = ', NLOS
!         WRITE(*,'(A,*(2x,I3))') '   Number of N_adj = ', N_adj
!         WRITE(*,'(A,*(2x,I3))') '            LW_FOV = ', LW_FOV(:)
!         STOP '     In "geom_along_los"'
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE GEOM_ALONG_LOS
!***************************************************************************
