C********************************************************************C
C
C       SCIATRAN - Version 2.0 - Last modification: 06 Nov 2003
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      FUNCTION GET_ABS_COEFF (GAS_NUM, T_NUM, JPOS, ACT_TEMP, IDX) 
c
c     Calculates absorption coefficient at temperature
c     act_temp from given cross sections abs_coeff(1..nabs_temp)
c     at temperatures abs_temp(1..nabs_temp) using linear interpolation
c     and (constant) extrapolation with boundary value. 
c     Note: assumes temperatures in increasing order

      USE CROSS_SECTIONS, ONLY: ABS_COEFF, ABS_COEFF_NTEMP, 
     $     ABS_COEFF_TEMP

      CHARACTER (LEN=16), PARAMETER :: ROUTINE_NAME = 'get_abs_coeff'

      INTEGER              GAS_NUM, NABS_TEMP, I, JPOS, IDX, T_NUM
      DOUBLE PRECISION     ACT_TEMP, GET_ABS_COEFF


      NABS_TEMP = ABS_COEFF_NTEMP(T_NUM)

      IF (NABS_TEMP.EQ.1.OR.ACT_TEMP.LE.ABS_COEFF_TEMP(T_NUM,1)) THEN
         GET_ABS_COEFF = ABS_COEFF(GAS_NUM,JPOS,1)
         IDX = 0
         RETURN
      ENDIF

      IF (ACT_TEMP.GE.ABS_COEFF_TEMP(T_NUM,NABS_TEMP)) THEN
         GET_ABS_COEFF = ABS_COEFF(GAS_NUM,JPOS,NABS_TEMP)
         IDX = 0
         RETURN
      ENDIF

      DO I = 2, NABS_TEMP
         IF (ACT_TEMP.LE.ABS_COEFF_TEMP(T_NUM,I)) THEN
            GET_ABS_COEFF = ABS_COEFF(GAS_NUM,JPOS,I - 1) +
     $           (ACT_TEMP - ABS_COEFF_TEMP(T_NUM,I - 1))/
     $           (ABS_COEFF_TEMP(T_NUM,I) - ABS_COEFF_TEMP(T_NUM,I - 1))
     $           *(ABS_COEFF(GAS_NUM,JPOS,I) - 
     $           ABS_COEFF(GAS_NUM,JPOS,I - 1))
            IDX = I
c            IF( GET_ABS_COEFF  > 1.0d-05 ) THEN
c               WRITE(*,*) JPOS, I, ABS_COEFF(GAS_NUM,JPOS,I),
c     $              ABS_COEFF(GAS_NUM,JPOS,I - 1)
c            ENDIF
            RETURN
         ENDIF
      ENDDO

      END
C********************************************************************C
