C********************************************************************C
C
C	SCIATRAN - Version 4.6 - Last modification: 06 Mar 2023
C
C	Copyright (C) 1995-2013, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GPP_CONVERGE(NSTRMS, MHARM, JPOS, Iot, RADOUT, W_LEAV,  
     $     DO_OUT, DO_CONV1)
c     ... Call in "dom_spherical.f"

      USE CLOUDS,      ONLY: DO_CLOUD
      USE GRIDS,       ONLY: NLAYERS
      USE INTERN_DIMS, ONLY: Nvs, Nlg_ray
      USE LOG_PARS,    ONLY: LOG_LEVEL, N_ESO, N_DSO, LUNIT
      USE RTM_OUTPUT,  ONLY: N_OP, STOK_OUT
      USE ST_VARS,     ONLY: RTM_MODE, DO_AEROSOL, Vec_Sca, BRDF_FLAG,
     $                       DO_AZIMUTH, ACCUR, I_LAY
      USE SPHER_PARS,  ONLY: N_LOS_OUT 
      USE TYPE_VARS,   ONLY: KND
      USE WATER,       ONLY: UWT, OUT_WLR, NLAYERS_UW
c     ------------------------------------------------------------------

C     Adds all contributions from azimuthal harmonics for radiance.
C     Test of convergence based on radiance.

      INTEGER, INTENT(IN) ::  NSTRMS, MHARM, JPOS, Iot
      REAL (KND) RADOUT(N_OP,N_LOS_OUT,Nvs), W_LEAV(N_LOS_OUT,Nvs)
      LOGICAL	 DO_OUT, DO_CONV1
c     ... Local parameters
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'GPP_CONVERGE'
      INTEGER	 I1, J1, Ks, N_OUT
      LOGICAL	 DO_CONV
      REAL (KND)  DIFER
      REAL (KND), ALLOCATABLE :: DELTA(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
c      write(*,*) '   OUT_WLR in gpp_converge.f ', OUT_WLR
      N_OUT = N_OP
c      write(*,*) '   N_OUT eto ', N_OUT
c      PAUSE
      IF( OUT_WLR ) N_OUT = N_OUT + 1
      ALLOCATE( DELTA(N_OUT,N_LOS_OUT,Nvs) )
c      WRITE(*,*) '   "gpp_converge": RADOUT :', JPOS, RADOUT(:,:,:)
c      WRITE(*,*) '    "gpp_converge": W_leav :', JPOS, W_LEAV(:,:)
c      DO I1 = 1,7
c         WRITE(*,'(1x,I3,4(2x,e15.7))') 
c     $        I1, RADOUT(1,I1,1), RADOUT(1,I1,2),
c     $        RADOUT(1,I1,3), RADOUT(1,I1,4)
c      ENDDO

C     If first harmonic, just reset
C     If no azimuth terms required, set output flags
c      write(*,*) ' In gpp_converge'
      IF (MHARM .EQ. 1) THEN
C        ..Output flags
         DO_CONV1 = .FALSE.
c            WRITE(*,*) '  DO_AZIMUTH ',DO_AZIMUTH
         IF ( .NOT. DO_AZIMUTH ) THEN
            DO_CONV1 = .TRUE.  ;  DO_OUT   = .TRUE.
            IF(TRIM(RTM_MODE) == 'flux' .OR. 
     $              TRIM(RTM_MODE) == 'spher_alb') THEN
               CALL ST_TRACK (R_NAME, 1, 'O')
               RETURN
            ENDIF
         ENDIF
c         WRITE(*,*) '             N_OP = ', N_OP
c         WRITE(*,*) '             JPOS = ', JPOS
c         WRITE(*,*) '    SHAPE(STOK_OUT) ', SHAPE(STOK_OUT)
c         WRITE(*,*) '    SHAPE(RADOUT)   ', SHAPE(RADOUT)
c         WRITE(*,*) '     SHAPE(W_LEAV)  ', SHAPE(W_LEAV)
         STOK_OUT(JPOS,Iot,1:N_OP,:,:)  =
     $        STOK_OUT(JPOS,Iot,1:N_OP,:,:) + RADOUT(1:N_OP,:,:)
c         WRITE(*,*) '  -------> "gpp_converge.f" :'
c         WRITE(*,'(6X,4(1X,E12.4))') (STOK_OUT(JPOS,Iot,1,1,J1),J1=1,4)
c         WRITE(*,'(A,I3)') ' Mharm = ',Mharm
c         WRITE(*,'(A,2(2x,e12.4))') '  Resulting radiance  ', 
c     $        STOK_OUT(JPOS,Iot,1,1,1), STOK_OUT(JPOS,Iot,1,N_LOS_OUT,1)
c         WRITE(*,*) '==========================================='
         IF( OUT_WLR ) STOK_OUT(JPOS,Iot,N_OP+1,:,:)  
     $        = STOK_OUT(JPOS,Iot,N_OP+1,:,:) + W_LEAV(:,:)
      ELSE
C     ... Other harmonics
C     ==================================================================
         DO_CONV = .TRUE.
C       ..For all spectral points (multiple scattering only)
C         Add Fourier component to the intensity sum
c            WRITE(*,*) '   ***** RADOUT times 2 in "gpp_converge.f"!! '
         DELTA(1:N_OP,:,:) = 2.d0*RADOUT(1:N_OP,:,:) 

         IF( OUT_WLR ) DELTA(N_OP+1,:,:) = 2.d0*W_LEAV(:,:) 
         STOK_OUT(JPOS,Iot,:,:,:) = STOK_OUT(JPOS,Iot,:,:,:) 
     $        + DELTA(:,:,:)

c         WRITE(*,'(A,I3)') ' Mharm = ',Mharm
c         WRITE(*,'(A,2(2x,e12.4))') '  Resulting radiance  ', 
c     $        STOK_OUT(JPOS,Iot,1,1,1), STOK_OUT(JPOS,Iot,1,N_LOS_OUT,1)

c        ... Convergence is analyzed for the I and Q components of the 
c            Stokes vector
         SELECT CASE ( Vec_Sca )
         CASE( 'scal_rte' )
            DIFER = MAXVAL(DABS(DELTA(:,:,1)/STOK_OUT(JPOS,Iot,:,:,1)))
            IF(LOG_LEVEL(1) >= N_ESO) THEN
               WRITE(Lunit,*) '             SCALAR CASE  '
               WRITE(Lunit,*)
     $              '  ------>> Difference and accuracy creat. '
               WRITE(Lunit,'(5x,2(2x,e12.4))') DIFER, ACCUR
            ENDIF
         CASE( 'vect_rte' )
c            WRITE(*,*) '  -------> "gpp_converge.f" Mh :', Mharm
c            WRITE(*,'(2X,*(1X,E12.4))') STOK_OUT(JPOS,Iot,1,:,2) 
	    DELTA(:,:,1) = DELTA(:,:,1)/STOK_OUT(JPOS,Iot,:,:,1)
            DO J1 = 1,N_OUT
               DO I1 = 1,N_LOS_OUT
c                 ... Use for the convergence criterion all Stokes 
c                     components
                  DO Ks = 2, Nvs
                     IF( DABS(STOK_OUT(JPOS,Iot,J1,I1,Ks))>0.d0 ) THEN 
                        DELTA(J1,I1,Ks) = DELTA(J1,I1,Ks)
     $                       /STOK_OUT(JPOS,Iot,J1,I1,Ks)
                     ELSE
                        DELTA(J1,I1,Ks) = 0.d0
                     ENDIF
                  ENDDO
               ENDDO
            ENDDO
            DIFER = MAXVAL( DABS(DELTA(:,:,1:4)) )
            IF(LOG_LEVEL(1) >= N_ESO) THEN
               WRITE(LUNIT,*) '             VECTOR CASE  '
               WRITE(LUNIT,*)
     $              '  ------>> Difference and accuracy creat. '
               WRITE(LUNIT,'(5x,2(2x,e12.4),1x,I3)') DIFER, ACCUR, Mharm
            ENDIF
         END SELECT

         IF ( DO_CONV .AND. (DIFER > ACCUR) ) DO_CONV = .FALSE.  

c        ... set output flag
c        -------------------
         IF ( DO_CONV1 .AND. DO_CONV) DO_OUT = .TRUE.
         DO_CONV1 = DO_CONV

         IF((.NOT.DO_AEROSOL).AND.(.NOT.DO_CLOUD).AND.(.NOT.UWT)) THEN 
           IF((Mharm == Nlg_Ray).AND.(.NOT.BRDF_FLAG)) DO_OUT = .TRUE.
         ENDIF

c         IF(DO_OUT) 
c     $        WRITE(*,*) '       -----> Harmonics used:', MHARM

      ENDIF
c     =====
c         WRITE(*,*) '   ------>  end of gpp_converge'
c         WRITE(*,*) '   STOK_OUT :', JPOS,  STOK_OUT(JPOS,:,:,:,:)
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
C********************************************************************C
