C********************************************************************
C
C	SCIATRAN - Version 4.2 - Last modification: 21 Jun 2020
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************
      SUBROUTINE GPP_REFL_ASYMPTOTIC( JPOS, JJESFT, NN, POS, 
     $     LOA, DEPOL, E_W, ALBEDO, TGA, * )

      USE CLOUDS,         ONLY: J_TOP, J_BOT, PROF_VARS
      USE CLOUDS_RETR,    ONLY: WRT_GT
      USE CLOUDS_SS,      ONLY: R_INF_WL, G_CLD_WL
      USE GRIDS,          ONLY: NL => NLAYERS, ZZ => ZZZ, DZR
      USE LOG_CONTROL,    ONLY: ERROR_TXT
      USE RTM_OUTPUT,     ONLY: SPEC_OUT, DR_FIN_TOP, DR_FIN_GT, DR_TAU,
     $                          R_SS
      USE ST_VARS,        ONLY: DO_AEROSOL, RAD_TO_DEG
      USE SPHER_CONTROL,  ONLY: ANG_OUT
      USE TYPES,          ONLY: KND, LOA_TYPE

      CHARACTER (LEN = 19), PARAMETER :: R_NAME = 'GPP_REFL_ASYMPTOTIC'
      INTEGER, INTENT(IN)  :: JPOS, JJESFT, NN
      REAL (KND), INTENT(IN) :: POS, DEPOL, E_W, ALBEDO
      REAL (KND), DIMENSION (NL) :: TGA
c     ----------------------------------------------------------------
c     ... Local optical parameters atmosphere |
c     -----------------------------------------
      TYPE (LOA_type) :: LOA
c     ----------------------
C     ... Local arrays
c     ----------------
      LOGICAL              ERROR, L_MULT
      REAL (KND), DIMENSION (NL) :: SOC, SCAT_ANG, PHF_AER, 
     $                     PHF_RAY, AL_LOC, EX_MU, T_AU, EXA_LOC
c     ... Local variables
c     -------------------
      INTEGER        I, J, JT, JB, NLC
      REAL (KND)     S, MU, MU0, SS_ABOVE_CLD, SS_ABOVE_SUR, MU_ef,
     $     ARAY, BRAY,  SSA_TOP, T_FUL, RS, R_DIF,
     $     T_AER, RSA, A_EFF, TT, K_MU, T_ABS, T_GAS, T_ABS_SUR,
     $     R_LOC, MULT, ABS_TOP, T_SCA, ABS_BOT, TTC,
     $     DT_ABS_TOP, T_NS, DT_ABS_BOT, DT_AER,
     $     Dss_Dtop, Dtrans_Dtop, Dr_Dgt, Dr_DtDb, Dr_Dopt, 
     $     Doef_Dh, Doef_DL, SSA_NEW, Drd_Dgt, Drd_DtDb,
     $     Drs_Dgt, Drs_DtDb, DUMM, Doef_Dt, R_INF, 
     $     G_CLD_SYM, D_MULT, SCA_EF, ABS_EF
      REAL (KND), POINTER :: SCC_NOSCALE(:), AL(:)

C     Start of code
c     -------------
      CALL ST_TRACK (R_NAME, 1, 'I')
      
      SCC_NOSCALE => LOA%SCC_NOSCALE(:,1)
      AL => LOA%AL

c     Initialize some variables
c     ( mainly to keep the compilers quiet)

      error   = .false. 
      ERROR_TXT = 'Error in '//R_NAME

c     ... Interpolation of r_inf_wl on a given wavelength
      R_INF = (1.d0 - LOA%DP_CLD)*R_INF_WL(LOA%Cil) 
     $     + LOA%DP_CLD*R_INF_WL(LOA%Cir)

      G_CLD_SYM = (1.d0 - LOA%DP_CLD)*G_CLD_WL(LOA%Cil) 
     $     + LOA%DP_CLD*G_CLD_WL(LOA%Cir)

      AL_LOC(:) = AL(1:NL) 

c     Full extinction coefficient without cloud extinction
      AL_LOC(:) = AL_LOC(:) - LOA%EXC(:)

C     Cosine of LOS zenith angle
      MU = ANG_OUT(2,1)

C     Rayleigh phase function coefficients
      ARAY = 1.5D0 * (1.0D0 + DEPOL) / (2.0D0 + DEPOL) 
      BRAY = 1.5D0 * (1.0D0 - DEPOL) / (2.0D0 + DEPOL) 

c    Calc. of cosine of scattering angle between incoming and outgoing ray
      MU0 = ANG_OUT(1,1)
      Mu_ef = 1.0d0/MU + 1.0d0/MU0

      SCAT_ANG(:) = -MU*MU0
     &     + DSQRT(1.0D0 - MU**2)*DSQRT(1.0D0 - MU0**2)
     &     *DCOS( ANG_OUT(3,1) )

c     Rayleigh scattering phase fct. (normalised to 4*pi)
c     =================================================== 
      PHF_RAY(:) = ARAY + BRAY * SCAT_ANG(:)**2
      SOC(:) = PHF_RAY(:) * LOA%SCR(:) 

      IF ( DO_AEROSOL ) THEN
c        ... Henyey-Greenstein phase fct. (normalised to 4*pi)
c        =====================================================
         PHF_AER(:) = (1.0D0 - LOA%AS_A(:)**2) 
     $        /(1.0D0 - 2.0D0*LOA%AS_A(:)*SCAT_ANG(:) 
     $        + LOA%AS_A(:)**2)**1.5D0
         SOC(:) = SOC(:) + PHF_AER(:)*LOA%SCA(:) 
      ENDIF

c      WRITE( *,* ) '-3'
c     Cloud base and cloud top for first cloud layer only
c     ---------------------------------------------------
      JT = J_TOP(1) ; JB = J_BOT(1) ; NLC = JB - JT + 1

c     Correction factor for inhomogenious absorbtion in the cloud,
c     effective ss-albedo at the cloud top (SSA_TOP), and optical 
c     thickness of the cloud including gas and aerosol absorbtion (T_FUL)
c     ------------------------------------------------------------
      CALL CALC_MULT
c      write( *,* ) ' CALC_MULT '

c     Reflection from cloud with finit optical thickness and absorbtion
c     as output R_FIN, spherical albedo RS, diffused transmittance TT
c     -----------------------------------------------------------------
      CALL REFL_FIN_ABS
c      write( *,* ) ' REFL_FIN_ABS  '

c     Optical depth between TOA and clodu top, output => SS_ABOVE_CLD
c     ===============================================================
      CALL SS_ABOVE_CLOUD

c      write( *,* ) ' SS_ABOVE_CLOUD '
c     Aerosol + Rayleigh optical thickness between surface and cloud bottom
c     =====================================================================
      EXA_LOC(JB:NL) = LOA%EXA(JB:NL) + LOA%SCR(JB:NL) 

      EXA_LOC(JB:NL-1) = 
     $     ( EXA_LOC(JB:NL-1) + EXA_LOC(JB+1:NL) )*DZR(JB:NL-1)
      T_AER = SUM ( EXA_LOC(JB:NL-1) )*0.5D0

c     Gas optical thickness between surface and cloud bottom
c     ==========================================================
      EXA_LOC(JB:NL-1) = ( TGA(JB:NL-1) + TGA(JB+1:NL) )*DZR(JB:NL-1)
      T_GAS = SUM ( EXA_LOC(JB:NL-1) )*0.5D0

c     Spherical albedo approximation of aerosol layer under cloud
c     ===========================================================
      RSA    = 0.265*T_AER - 0.865*T_AER**2 + 1.164*T_AER**3
      DT_AER = (0.265    - 2.*0.865*T_AER + 3.*1.164*T_AER**2)
     $     * (LOA%EXA(JB) + LOA%SCR(JB))

c     Effective surface albedo
c     ========================
      A_EFF = RSA + ALBEDO * (1.0 - RSA)**2/( 1.0 - ALBEDO*RSA )

      R_DIF = A_EFF*TTC**2*K_MU/(1.0 - A_EFF*RS)* DEXP(-1.7*2.*T_GAS)
      R_LOC = R_LOC + R_DIF

      DT_AER = R_DIF/A_EFF/(1.0 - A_EFF*RS)*DT_AER
      DT_AER = DT_AER*( (1.0 - ALBEDO)/(1.0 - ALBEDO*RSA) )**2

c     Calculate optical thickness of each layer for gaseous absorption only
c     =====================================================================
      AL_LOC(1:NL-1) = (TGA(1:NL-1) + TGA(2:NL))*DZR(1:NL-1)

c     Optical thickness for absorbtion above cloud
c     ============================================
      T_ABS = SUM( AL_LOC(1:JT-1) )*0.5D0

c     Optical thickness for absorbtion above surface
c     ==============================================
      T_ABS_SUR = T_ABS  + SUM( AL_LOC(JT:NL-1) )*0.5D0

c     Transmission and derivative of transmision w.r.t cloud top hight
c     ================================================================
      T_NS = DEXP( -T_ABS*Mu_ef ) ; Dtrans_Dtop = T_NS*TGA(JT)*Mu_ef

c     Final calculation of the reflection function
c     ============================================
      IF( jjesft .EQ. 1 ) THEN 
         SPEC_OUT (JPOS,1,1) = 0.0D0 ; DR_FIN_TOP(JPOS,1) = 0.0D0
         DR_FIN_GT(JPOS,1) = 0.0D0 ; DR_TAU(JPOS,1)     = 0.0D0
         R_SS (JPOS,1)     = 0.0d0
      ENDIF

      WRT_GT = .TRUE.
      IF( WRT_GT ) THEN
         DR_FIN_TOP(JPOS,1) = DR_FIN_TOP(JPOS,1) + 
     $        ( R_LOC*Dtrans_Dtop + Dss_Dtop + 
     $        ( Dr_DtDb + DT_AER 
     $        + R_DIF*Drd_DtDb 
     $        + R_DIF*A_EFF/(1.0-A_EFF*RS)*Drs_DtDb
     $        -  R_DIF*3.4*ABS_BOT )*T_NS ) * E_W

         DR_FIN_GT(JPOS,1) = DR_FIN_GT(JPOS,1) + 
     $        ( 
     $        Dr_Dgt 
     $        - DT_AER 
     $        + R_DIF*Drd_Dgt 
     $        + R_DIF*A_EFF/(1.0-A_EFF*RS)*Drs_Dgt
     $        + R_DIF*3.4*ABS_BOT 
     $        )* T_NS * E_W

      ELSE
         DUMM = Dr_Dgt - DT_AER 
     $        + R_DIF*Drd_Dgt 
     $        + R_DIF*A_EFF/(1.0-A_EFF*RS)*Drs_Dgt
     $        + R_DIF*3.4*ABS_BOT 

         DR_FIN_GT(JPOS,1) = DR_FIN_GT(JPOS,1) - DUMM*T_NS*E_W

         DR_FIN_TOP(JPOS,1) = DR_FIN_TOP(JPOS,1) + 
     $        ( R_LOC*Dtrans_Dtop + Dss_Dtop + 
     $        ( Dr_DtDb + DT_AER 
     $        + R_DIF*Drd_DtDb 
     $        + R_DIF*A_EFF/(1.0-A_EFF*RS)*Drs_DtDb
     $        -  R_DIF*3.4*ABS_BOT + DUMM )*T_NS ) * E_W

      ENDIF

      R_LOC            = R_LOC * T_NS + SS_ABOVE_CLD
      SPEC_OUT(JPOS,1,1) = SPEC_OUT(JPOS,1,1) + R_LOC * E_W
      DR_TAU(JPOS,1)   = DR_TAU(JPOS,1) + Dr_Dopt*T_NS*E_W

c     ... Modified single-scattering approximation for cloud free part
      R_SS(JPOS,1) = R_SS(JPOS,1) + ( ALBEDO *DEXP( -T_ABS_SUR*Mu_ef )
     $     + SS_ABOVE_SUR ) * E_W

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
C********************************************************************C

      CONTAINS
C********************************************************************C
      SUBROUTINE CALC_MULT
      IMPLICIT NONE
      INTEGER           ITER, JP
      DOUBLE PRECISION  T_OPT, OME_DT, YY
      DOUBLE PRECISION  RUT1, D_D, PSI, TGS, XX
      DOUBLE PRECISION, DIMENSION (NLC) :: ABS, SSA, EXT, HELP, DABS_DZ,
     $     SSA_DZB, SSA_DZT, ABS_CLD, EXT_CLD
      LOGICAL           MASK(NLC)

c     Gas absorption coefficient as a function of altitude in the cloud
c            Cloud absorption should not be included !!
c     -----------------------------------------------------------------
         ABS( 1:NLC ) = AL(JT:JB) - LOA%SCR(JT:JB) 
     $     - LOA%EXA(JT:JB) - LOA%EXC(JT:JB)

C        OPTICAL DEPTH of THE CLOUD WITHOUT GAS ABSORBTION
c        --------------------------------------------------------
         T_SCA = SUM( ( ZZ(JT:JB-1) - ZZ(JT+1:JB) )*
     $        ( SCC_NOSCALE(JT:JB-1) + SCC_NOSCALE(JT+1:JB) ) )/2.0d0

c        First infinitesimal cloud layer is not used for calculation
c        -----------------------------------------------------------
         IF( ABS(2) .LE. 0.0d0  ) THEN
            DABS_DZ = 0.0d0
         ELSE
            DABS_DZ(2) = ( ABS(3)   - ABS(2) )/(ZZ(JT+2) - ZZ(JT+1))
c           DT_ABS_TOP = DABS_DZ(2)
c             Test for derivative
c            write(*,*) ABS(2), ABS(3)
            XX = ( DLOG(ABS(3)/ABS(2)) )/(ZZ(JT+2) - ZZ(JT+1))
            XX = DEXP( DLOG( ABS(2) ) + 0.001*XX )
            DT_ABS_TOP = ( XX - ABS(2) )/0.001

            DABS_DZ(1) = DABS_DZ(2)
            J = 3
            DO I = JT+2,JB-2
               DABS_DZ(J) = ( ABS(J+1) - ABS(J-1) )/(ZZ(I+1) - ZZ(I-1))
               J = J + 1
            ENDDO
            DABS_DZ(NLC-1)=(ABS(NLC-1)-ABS(NLC-2))/(ZZ(JB-1) - ZZ(JB-2))
            DT_ABS_BOT    = DABS_DZ(NLC-1)
            DABS_DZ(NLC)  = DABS_DZ(NLC-1)
         ENDIF

c         WRITE( *,* ) '-4'

c        Here is used cloud abs. coef. to avoid delta-M scaling
c        ----------------------------------------------------------
         ABS_CLD(1:NLC) =  LOA%EXC( JT:JB ) - LOA%SCC( JT:JB )
         EXT_CLD(1:NLC) =  ABS_CLD(1:NLC) + SCC_NOSCALE( JT:JB )

c        Gas absorption and cloud extinction profiles in the cloud 
c        ==========================================================
         EXT( 1:NLC ) = ABS( 1:NLC ) +  EXT_CLD(:) 

C       OPTICAL DEPTH of THE CLOUD INCLUDING GAS ABSORBTION
C       --------------------------------------------------------
         I = JT ; T_AU(1) = 0.0D0
         DO  J = 1, NLC - 1
            T_AU(J+1) = T_AU(J) + (EXT(J) + EXT(J+1))*0.5D0*DZR(I)
            I = I + 1
         ENDDO
         T_FUL = T_AU( NLC )

c         WRITE( *,* ) '-5'
c        Initial values for ss.albedo and mean absorption
c        ------------------------------------------------
         SCA_EF  = T_SCA/( ZZ(JT) - ZZ(JB) )
c         ABS_EF  = ABS(2) 
         ABS_EF  = ABS(2) + ABS_CLD(2) ! To include the absorption of water

         SSA( 1:NLC ) = SCC_NOSCALE(JT:JB)/( EXT_CLD(:) + ABS(:) )

c        Derivative ssa wrt z_top for constant geometrical thickness
c        -----------------------------------------------------------
         SSA_DZT( 2:NLC-1 ) = -SSA(2:NLC-1 )**2
     $        /SCC_NOSCALE(JT+1:JB-1)*DABS_DZ(2:NLC-1)

c        Derivative ssa wrt z_bot for constant cloud top height
c        -----------------------------------------------------------
         SSA_DZB( 2:NLC-1 ) = SSA(2:NLC-1)**2*ABS(2:NLC-1)/
     $        EXT_CLD(2:NLC-1)/SCC_NOSCALE(JT+1:JB-1)*prof_vars(2:NLC-1)
     $        +SSA_DZT( 2:NLC-1 )*(ZZ(JT)-ZZ(JT+1:JB-1))/(ZZ(JT)-ZZ(JB))

c         WRITE( *,* ) '-6'
         SSA_TOP = SSA(2)
c         ABS_TOP = ABS(1)
c         ABS_BOT = ABS(NLC)
c        ... To avoid infinity if there is no gaseous absorption at all
         ABS_TOP = ABS(1)   + ABS_CLD(2)
         ABS_BOT = ABS(NLC) + ABS_CLD(NLC-1)

         ITER = 0 ; L_MULT = .false.

         IF( (1.0d0 - SSA_TOP) < 0.1D-07 ) THEN
            L_MULT = .TRUE.
c            WRITE( *,* ) ' (1.0 - SSA_TOP) < 0.1D-07 ', 1.d0 - SSA_TOP
            GOTO 2
         ENDIF

         TGS   = DSQRT( 3.0d0*(1.0d0 - G_CLD_SYM) )
c         T_OPT = 1.0d0/DSQRT( (1.0d0 - SSA(2)) )/TGS
         T_OPT = 1.0d0/DSQRT( (1.0d0 - SSA(NLC-1)) )/TGS

c         WRITE( *,* ) '-7 t_opt1 = ', T_OPT
         IF( T_OPT >= T_FUL ) THEN 
            L_MULT = .TRUE.
            GOTO 2
         ENDIF


         CALL LINTP2(NLC, T_AU, SSA, 1, T_OPT, SSA_NEW, ERROR)
         T_OPT = 1.0d0/DSQRT((1.0d0 - SSA_NEW))/TGS

c         WRITE( *,* ) '-7 t_opt2 = ', T_OPT

 44      CONTINUE
c         WRITE( *,* ) ' -- ', T_AU(3), T_OPT

         CALL LINTP2(NLC, T_AU, SSA, 1, T_OPT, SSA_NEW, ERROR)
         IF( error ) stop ' ERORR in LINTP2 '
         MASK(:) = .false. ; HELP(:) = T_AU(1:NLC) - T_OPT
         WHERE( HELP(1:NLC) < 0.0 ) MASK(1:NLC) = .true. 

         JP = COUNT(MASK)
c         WRITE( *,* ) ' * ', SSA_NEW
c         WRITE( *,* ) ' - ', ITER, JP, NLC
         IF( JP >= NLC ) THEN
            L_MULT = .TRUE. ; GOTO 2
         ENDIF
         D_D = DSQRT((1.0d0 - SSA_NEW))
         PSI = 1.0d0/T_OPT - D_D*TGS       
         OME_DT = ( SSA(JP+1) - SSA(JP) )/( T_AU(JP+1) - T_AU(JP) )
         YY = -1.0d0/T_OPT**2 + TGS/D_D*OME_DT/2.0d0

c         WRITE( *,* ) ' --- ',  T_OPT, - PSI/YY
         RUT1 = T_OPT - PSI/YY
         IF( RUT1 <= 0.0 ) THEN 
            WRITE( *,* ) '******** Optimal T_OPT is not found  *******'
c            write( *,* ) ' ...... TEST T_OPT = 7.5 ....'
c            T_OPT = 7.5
c            CALL LINTP2(NLC, T_AU, SSA, 1, T_OPT, SSA_NEW, ERROR)
            GOTO 3
         ENDIF
         IF( DABS(RUT1 - T_OPT) > 0.001 ) THEN
            T_OPT = RUT1
            ITER = ITER + 1
            IF( ITER > 20 ) THEN
               WRITE( *,* ) '******** NUMBER OF ITERATION > 20 *******'
               GOTO 3
            ENDIF
            GOTO 44
         ENDIF

 3       CONTINUE

         CALL LINTP2(NLC-2, T_AU(2:NLC-1), SSA_DZT(2:NLC-1), 
     $        1, T_OPT, XX, ERROR)
         RUT1 = -TGS/D_D/2.0d0/YY*XX

         SSA_DZT(2:NLC-1) = SSA(2:NLC-1) + SSA_DZT(2:NLC-1)*0.01
         CALL LINTP2(NLC-2, T_AU(2:NLC-1), SSA_DZT(2:NLC-1), 
     $        1, T_OPT + RUT1*0.01, XX, ERROR)
         Doef_Dh = ( XX - SSA_NEW )/0.01

         CALL LINTP2(NLC-2, T_AU(2:NLC-1), SSA_DZB(2:NLC-1), 
     $        1, T_OPT, XX, ERROR)
         RUT1 = -TGS/D_D/2.0d0/YY*XX

         SSA_DZB(2:NLC-1) = SSA(2:NLC-1) + SSA_DZB(2:NLC-1)*0.01
         CALL LINTP2(NLC-2, T_AU(2:NLC-1), SSA_DZB(2:NLC-1), 
     $        1, T_OPT + RUT1*0.01, YY, ERROR)

c        Sign "-" than d/dl = -d/dz_bot
c        ------------------------------
         Doef_DL = -( YY - SSA_NEW )/0.01
         Doef_Dt = SSA_NEW*( 1.0d0 - SSA_NEW )/T_FUL

 2       CONTINUE
c         WRITE( *,* ) ' End iteration'
      RETURN
      END SUBROUTINE
C **********************************************************************
      SUBROUTINE REFL_FIN_ABS
      DOUBLE PRECISION ALFA, K_MU0, PRD, ZT, YT, BB, GAM, VES
      DOUBLE PRECISION R1, XT, P37, TRM, VES_T
      DOUBLE PRECISION h1, h2, ZT_V, DEL, T1, T2, YTM, S_EXP
      DOUBLE PRECISION R2a, R2b, R2, Ay, Ax, TE1, TE2, SSA_TES
      PARAMETER        ( ALFA = 1.07D0, P37 = 3.0d0/7.0d0  )
         K_MU = P37*(1.0d0 + 2.0d0*MU) ; K_MU0 = P37*(1.0d0 + 2.0d0*MU0)
         K_MU  = K_MU * K_MU0
         PRD = MU * MU0
         ZT  = 0.75D0*(1.0D0 - G_CLD_SYM)*T_FUL
         TRM = (4.86D0 - 13.08D0*PRD + 12.76D0*PRD**2 )/T_FUL**3

c         WRITE( *,* ) ' (1.0 - SSA_TOP) before ', 1.d0 - SSA_TOP
c         WRITE( *,* ) ' L_MULT = ',L_MULT
         IF( (1.0D0 - SSA_TOP) < 1.0D-06 ) THEN
            TT = 1.0D0 / ( ALFA + ZT )
            R_LOC = R_INF - ( TT - TRM )*K_MU
            Dr_Dgt  = 0.0d0
            Dr_DtDb = 0.0d0
c            write( *,* ) ' no abs -->> R_LOC ', jjesft, R_LOC
         ELSE
            IF( L_MULT ) THEN
c              if L_MULT = .true. opt. value for ssa_new is not found
               SSA_TOP = SCA_EF/( SCA_EF + ABS_EF )
c               WRITE( *,* ) ' (1.0 - SSA_TOP) _EF ', 1.d0 - SSA_TOP
               YT = 4.0D0*DSQRT((1.0 - SSA_TOP)/3.0/(1.0 - G_CLD_SYM))
c               WRITE( *,* ) '  YT _EF ', YT
               VES = DSINH(YT) / DSINH( YT*(ALFA + ZT) )
               MULT = ( (1.0 - ves)*abs_top + ves*abs_bot )

c               WRITE( *,* ) '... ves     = ', ves
c               WRITE( *,* ) '... abs_top = ', abs_top
c               WRITE( *,* ) '... abs_bot = ', abs_bot
c               WRITE( *,* ) '... MULT = ', MULT

               SSA_NEW = SCA_EF/( SCA_EF + MULT )

c               WRITE( *,* ) '... SCA_EF = ', SCA_EF
c               WRITE( *,* ) '... ABS_EF = ', ABS_EF
c               WRITE( *,* ) '... 1.- SSA_NEW = ',1.d0 - SSA_NEW
               
               GAM = DSQRT( 3.*(1.0 - SSA_TOP)*(1.0 - G_CLD_SYM) )
               YTM = YT*(ALFA + ZT)
               T2  = 1.0 / DTANH( YTM ) ; T1 = 1.0 / DTANH( YT )
               BB  = 2.*SSA_TOP*(1.-SSA_TOP)/T_SCA
               DEL = ZZ(JT) - ZZ(JB)
               D_MULT = VES*( (ALFA*T2-T1)*BB/GAM
     $                       - T2*( GAM - T_FUL*BB/YT ) )
     $                 *(abs_bot-abs_top)
c               D_MULT = 0.
               Doef_Dt = SSA_NEW*( 1./SCA_EF/DEL 
     $                  - (1./DEL + D_MULT)/( SCA_EF + MULT ) )

               SCA_EF  = T_SCA/( ZZ(JT) - ZZ(JB) - 0.01)
               SSA_TOP = SCA_EF/( SCA_EF + ABS_EF )
               YT = 4.0D0*DSQRT((1.0 - SSA_TOP)/3.0/(1.0 - G_CLD_SYM))
               VES_T = DSINH(YT) / DSINH( YT*(ALFA + ZT) )
               h1 = abs_bot + dt_abs_bot*0.01
               MULT = ( (1.0 - ves_T)*abs_top + ves_T*h1 )
               SSA_TES = SCA_EF/( SCA_EF + MULT )
               SSA_TES = (SSA_TES - SSA_NEW)/0.01
               Doef_DL = - SSA_TES

               DEL     = ZZ(JT+1) - ZZ(JT+2)
               SCA_EF  = T_SCA/( ZZ(JT) - ZZ(JB) )
               SSA_TOP = SCA_EF/( SCA_EF + ABS_EF + dt_abs_top*DEL)
               YT = 4.0D0*DSQRT((1.0 - SSA_TOP)/3.0/(1.0 - G_CLD_SYM))

               ZT_V = 0.75D0*(1.0D0-G_CLD_SYM)*(abs_top - abs_bot)*DEL
               VES_T = DSINH(YT) / DSINH( YT*(ALFA + ZT + ZT_V) )
               h1 = abs_bot + dt_abs_bot*DEL
               h2 = abs_top + dt_abs_top*DEL
               MULT = ( (1.0 - ves_T)*h2 + ves_T*h1 )
               SSA_TES = SCA_EF/( SCA_EF + MULT )

               SSA_TES = (SSA_TES - SSA_NEW)/DEL
c               SSA_TES = 0.0
               Doef_Dh = SSA_TES 
            ENDIF

c            S_EXP = DEXP( -YT*(1.0 - 0.05*YT) ) ! in old version
            YT = 4.0D0*DSQRT((1.0 - SSA_NEW)/3.0/(1.0 - G_CLD_SYM))

c            write( *,* ) ' abs--->> Yt ', YT
            
            S_EXP = DEXP( -YT*(1.0 - 0.05*YT) )  ! New version

            R1 = R_INF*DEXP( -YT*(1.0 - 0.05*YT)*K_MU/R_INF )

            YTM = YT*(ALFA + ZT)
            XT = YT * ZT
            TT = DSINH(YT) / DSINH( YTM )
            BB = DEXP(-XT)
            IF( TT*BB < TRM ) THEN
               TTC = TT ; TRM = 0.0
            ELSE
               TTC = TT - TRM/BB
            ENDIF
            R2a = TT*DEXP(-XT - YT)*K_MU
            R2b = TRM*DEXP(-YT)*K_MU
            R2  = -R2a + R2b
            R_LOC = R1 + R2

c            write( *,* ) ' abs--->> R1 ', R1
c            write( *,* ) ' abs--->> R2 ', R2
c            write( *,* ) ' abs--->> R_LOC ', jjesft, R_LOC

            RS = S_EXP - (R2a - R2b)/K_MU 

C           DERIVATIVE R_FIN W.R.T GEOMWTRICLA THICKNESS
c           ============================================
            GAM  = DSQRT( 3.*(1.0 - SSA_NEW)*(1.0 - G_CLD_SYM) )
            T2 = 1.0 / DTANH( YTM )

            Dr_Dopt = R2a*GAM*( T2 + 1.0 ) - 3.*R2b/T_FUL

            T1 = 1.0 / DTANH( YT )
            Ay = R2a*( 1.0 - T1 + ALFA*T2 ) - R2b
            Ax = R2a*( 1.0 + T2 )

c           WRITE( *,* ) '... GAM     = ',GAM
c           WRITE( *,* ) '  G_CLD_SYM = ',G_CLD_SYM
c           WRITE( *,* ) '... R_INF   = ',R_INF

            TE1 = ( R1*K_MU/R_INF*(1.0 - 0.1*YT) - Ay - XT/YT*Ax )
     $           * 2.0/GAM
            TE2 = ( Ax - 3.0/XT*R2b )*GAM

            Dr_Dopt = Dr_Dopt + TE1*Doef_Dt
c                                 - ? proverit'
            Dr_Dgt  = TE1*Doef_DL + TE2*ABS_BOT
            Dr_DtDb = TE1*Doef_Dh + TE2*( ABS_TOP - ABS_BOT )

            TE1 = -( R2a*(YTM/YT*T2 - T1) + XT/YT*R2b )*4./GAM/R2
            TE2 =  ( R2a*T2 + (XT-3.0)/XT*R2b )*2.0*GAM/R2
            Drd_Dgt  = TE1*Doef_DL + TE2*ABS_BOT
            Drd_DtDb = TE1*Doef_Dh + TE2*( ABS_TOP - ABS_BOT )

            TE1 = ( S_EXP*K_MU*(1.0 - 0.1*YT)
     $           - Ay - XT/YT*Ax )*2./GAM/K_MU
            TE2 = ( Ax - 3.0/XT*R2b )*GAM/K_MU
            Drs_Dgt  = TE1*Doef_DL + TE2*ABS_BOT
            Drs_DtDb = TE1*Doef_Dh + TE2*( ABS_TOP - ABS_BOT )
         endif
         
c         write( *,* ) ' ------>> R_LOC ', jjesft
c         write( *,'(3(2x,e15.7))') R_LOC,  Dr_DtDb, Dr_Dgt

      RETURN
      END SUBROUTINE
C********************************************************************C
      SUBROUTINE SS_ABOVE_CLOUD

         T_AU(1) = 0.0D0
c         DO  J = 1, JT - 1
         DO  J = 1, NL - 1
            T_AU(J+1) = T_AU(J) + (AL_LOC(J)+AL_LOC(J+1))*0.5D0*DZR(J)
         ENDDO
c         EX_MU (1:JT) = DEXP( -T_AU(1:JT)*Mu_ef )
         EX_MU (:) = DEXP( -T_AU(:)*Mu_ef )

c        SS in the atmosphere between TOA and cloud top
c        ----------------------------------------------
c         SOC(1:JT) = SOC(1:JT)/AL_LOC(1:JT)
         SOC(:) = SOC(:)/AL_LOC(:)
         S = 0.0D0
         DO I = 1,JT-1
            S = S + (SOC(I) + SOC(I+1))*( EX_MU(I) - EX_MU(I+1) )
         ENDDO
c        It is not multiplyed by MU0 to calculate reflection function
         SS_ABOVE_CLD = S/8.0D0/( MU + MU0 )

c        Derivative of Single scattering w.r.t. cloud top altitude
         Dss_Dtop = -SOC(JT)*EX_MU(JT)*AL_LOC(JT)/4.0D0/MU/MU0

         DO I = JT,NL-1
            S = S + (SOC(I) + SOC(I+1))*( EX_MU(I) - EX_MU(I+1) )
         ENDDO
         SS_ABOVE_SUR = S/8.0D0/( MU + MU0 )

      RETURN
      END SUBROUTINE
C********************************************************************C
      END
C********************************************************************C
