!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 10 Feb 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine GT_ALBEDO_SETUP(*)
  ! Called by "gt_iface.f"

  USE AEROSOLS,  ONLY: MODIS_AP
  USE ALB_PARS,  ONLY: N_AWD => N_ALBEDO_WAVS_DB, ALBEDO_WAVS_DB, &
                       ALBEDO_DB, Nc_ALB, ALB_WEI, INF_ALB
  USE LOG_PARS,  ONLY: EU, LUNIT, DO_LOGFILE
  USE ST_INPUT,  ONLY: ALBEDO_FILENAME, ALB_CDF
  USE ST_VARS,   ONLY: RTM_MODE
  Use Tools,     Only: READ_FILE
  USE TYPE_VARS, ONLY: KND
  !------------------------------------------------------------------
  ! OUTPUT: N_ALBEDO_WAVS_DB, ALBEDO_WAVS_DB, ALBEDO_DB
  !------------------------------------------------------------------
  CHARACTER (LEN=24), PARAMETER :: R_NAME='GT_ALBEDO_SETUP'
  INTEGER    I, K, Nr, N1, N2
  REAL (KND) PREV_WAVS, ACT_WAVS, DUM
  REAL (KND), ALLOCATABLE :: DATA_R(:,:)
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 3, 'I')

  ! Read spectral albedo
  ! --------------------
  IF (DO_LOGFILE) WRITE(LUNIT, *) ' *** Reading: ', TRIM(ALBEDO_FILENAME)
  Call READ_FILE (Nc_ALB + 1, ALBEDO_FILENAME, DATA_R, *880, I_XC=1, &
       CDF=ALB_CDF, Skip_line=1)
  N_AWD = Size(DATA_R,1)
  IF(ALLOCATED(ALBEDO_WAVS_DB)) DEALLOCATE( ALBEDO_WAVS_DB, ALBEDO_DB)
  ALLOCATE( ALBEDO_WAVS_DB(N_AWD), ALBEDO_DB(N_AWD,Nc_ALB) )  ! A
  ALBEDO_WAVS_DB = DATA_R(:,1)
  ALBEDO_DB = DATA_R(:,2:)

  DO I = 2, N_AWD
     IF (ALBEDO_WAVS_DB(I).LE.ALBEDO_WAVS_DB(I - 1)) GOTO 120
  ENDDO
  
  IF( Nc_ALB==1 ) THEN
     DO I = 1, N_AWD
        IF (ALBEDO_DB(I,1)<0.d0.OR.ALBEDO_DB(I,1)>1.d0) GOTO 121
     ENDDO
  ENDIF
  CALL ST_TRACK (R_NAME, 3, 'O')
  RETURN
  
120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong order spectral albedo wavelength')
  WRITE(EU,*) '    Act. wavelength:     ', ALBEDO_WAVS_DB(I)
  WRITE(EU,*) '    Previous wavelength: ', ALBEDO_WAVS_DB(I-1)
  RETURN 1
  
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Spectral albedo out of bounds')
  WRITE(EU,*) '    Value read: ', I, ALBEDO_DB(I,1)
  RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

!**************************************************************************
End Subroutine GT_ALBEDO_SETUP
!**************************************************************************
