C********************************************************************C
C
C	SCIATRAN - Version 4.4 - Last modification: 22 Mar 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_CHECK(WL_G, NSPEC, *)
c     ... Call in "rtm_main.f"

      USE ABSORBERS, ONLY: N_ESFT_MOL
      USE AEROSOLS,  ONLY: N_AER_WAVS, AER_WAVS, LA_N, LA_S, LA_E, 
     $                     L_TOP, COMPAT_MODE, Plus_mi, New_lay, 
     $                     USE_AP, RT_AG
      USE ALB_PARS,  ONLY: DO_SPECTRAL_ALBEDO, N_ALBEDO_WAVS_DB, 
     $                     ALBEDO_WAVS_DB
      USE CLOUDS,    ONLY: NCW => N_CLD_NETWAVS, CLW => CLD_NETWAVS,
     $                     DO_CLOUD
      USE GRIDS,     ONLY: NLAYERS, ZZZ
      USE ESFT,      ONLY: DO_ESFT, ESFT_NCOEFF, ESFT_OVERLAP_MODE, 
     $                     ESFT_MOL_OVERLAP_IDX1, ESFT_MOL_OVERLAP_IDX2
      USE LOG_PARS,  ONLY: EU
      USE ST_VARS,   ONLY: DO_AEROSOL, WF_FLAG, DO_PC
      USE ST_INPUT,  ONLY: NET_NAME
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) :: NSPEC
      REAL (KND) WL_G(NSPEC)
c     ... Local variables
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'GT_CHECK'
      INTEGER   I, J, NOVERLAP
      REAL (KND)  START_LAMDA, END_LAMDA
      REAL (KND), PARAMETER :: DZ_LIM = 0.1d0
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( WF_FLAG ) THEN
         IF(DO_PC) THEN
            CALL WRITE_MESSAGE ( R_NAME, 'ERROR',
     $           'Weighting functions and block air mass factors are '
     $           //'incompatible with photochemistry')
            RETURN 1
         END IF
      END IF         

C     Checks on wavelength inputs
C     ---------------------------
      START_LAMDA = MINVAL(WL_G(:))
      END_LAMDA   = MAXVAL(WL_G(:))

C     Check if aerosol input covers wavelengths
c     --------------------------------------
      IF ( DO_AEROSOL ) THEN
c      Check of aerosol layering
c     --------------------------
         IF( .NOT.Plus_mi ) THEN
c            DO I = 1, LA_N
c               WRITE(*,*) I, LA_E(I), LA_S(I)
c            ENDDO
c            DO I = 1, LA_N
c               WRITE(*,*) I, ZZZ(LA_E(I)), ZZZ(LA_S(I))
c            ENDDO
            IF( RT_AG=='user' ) THEN
               DO I = 1,LA_N
                  IF( .NOT.New_lay .AND. LA_S(I)==LA_E(I) .AND.
     $                 .NOT.COMPAT_MODE) THEN
                     CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
                     WRITE(EU,*) 
     $               '    The main altitude grid (see "control.inp"): ',
     $                    TRIM(NET_NAME)
                     WRITE(EU,'(a,2x,I2)') 
     $               '     does not allow to resolve aerosol layer:', I
                     WRITE(EU,*) 
     $                    '    The top and the bottom of this layer are'
                     WRITE(EU,'(a,f7.2,a,f7.2,a)')
     $                    '    ',ZZZ(LA_E(I)),'   - ',ZZZ(LA_S(I)),' km'
                     WRITE(EU,*) 
     $                 '    The required top of this layer is'
                     WRITE(EU,'(a,f7.2,a)')'    ',L_TOP(I),' km'
                     WRITE(EU,*) 
     $                 '    Change the main altitude grid or the top'//
     $                    ' altitude of the aerosol layer.'
                     RETURN 1               
                  ENDIF
               ENDDO
               DO I = 1,LA_N
                  IF( DABS(L_TOP(I) - ZZZ(LA_S(I))) > DZ_LIM ) THEN
                     CALL WRITE_MESSAGE(R_NAME, 'WARNING', '')
                     WRITE(EU,'(a,2x,I2)') 
     $               '    The top altitude of aerosol layer ', I
                     WRITE(EU,'(a)') 
     $               '    does not match exactly the required value'
                     WRITE(EU,'(a,f7.2,a)') 
     $               '    required :', L_TOP(I),' km'
                     WRITE(EU,'(a,f7.2,a)') 
     $               '    used     :', ZZZ(LA_S(I)),' km'
                  ENDIF
               ENDDO
            ENDIF  ! RT_AG=='user'
         ENDIF
         IF ( END_LAMDA>AER_WAVS(N_AER_WAVS).AND.(.NOT.USE_AP) ) THEN
            CALL WRITE_MESSAGE ( R_NAME, 'ERROR',
     &           'Last wavelength not covered by aerosol data')
            RETURN 1
         ENDIF
      ENDIF

C     Check if albedo input covers wavelengths
c     --------------------------------------
      IF(DO_SPECTRAL_ALBEDO) THEN
         IF (ALBEDO_WAVS_DB(1) > START_LAMDA .OR. 
     $        ALBEDO_WAVS_DB(N_ALBEDO_WAVS_DB) < END_LAMDA ) THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Spectral range'//
     $            'not covered by spectral albedo data base')
            WRITE(EU,'(A,2(2x,F9.3))') 
     $           '    Min. and max. required wavelength:  ',
     $           START_LAMDA, END_LAMDA
            WRITE(EU,'(A,2(2x,F9.3))') 
     $           '    Min. and max. data base wavelength: ',
     $           ALBEDO_WAVS_DB(1), ALBEDO_WAVS_DB(N_ALBEDO_WAVS_DB)
            RETURN 1
        ENDIF
      ENDIF

C     Check cloud input covers wavelengths
c     ------------------------------------
      IF(DO_CLOUD) THEN
c         IF( START_LAMDA < CLW(1) ) THEN
c            CALL WRITE_MESSAGE ( R_NAME, 'ERROR',
c     &           'First wavelength not covered by cloud data')
c            RETURN 1
c         ENDIF
         IF ( END_LAMDA > CLW(NCW) ) THEN
            CALL WRITE_MESSAGE ( R_NAME, 'ERROR',
     &           'Last wavelength not covered by cloud data')
            RETURN 1
         ENDIF
      ENDIF

c     Check ESFT overlap mode 2 requirements
c     --------------------------------------
      if (do_esft) then
         do i = 1, nspec
            if (esft_overlap_mode(i).eq.2) then
               noverlap = 0
               do j = 1, N_ESFT_MOL
                  if (esft_ncoeff(i,j).gt.1) then
                     noverlap=noverlap+1
                     if (j.ne.esft_mol_overlap_idx1(i).and.
     &                   j.ne.esft_mol_overlap_idx2(i)) then
                         call write_message(r_name, 'ERROR',
     &                   'Wrong molecule idx in ESFT overlap mode 2')
                         write(EU,*) 
     &                   '    ESFT molecule idx is: ', j
                         write(EU,*) 
     &                   '    ESFT molecule idx should be: ', 
     &                        esft_mol_overlap_idx1(i)
                         write(EU,*) 
     &                   '    or: ', 
     &                        esft_mol_overlap_idx2(i)
                        return 1
                     endif
                  endif
               enddo
               if (noverlap.ne.2) then
                  call write_message(r_name, 'ERROR',
     &            'ESFT overlap mode 2: more than 2 absorber with M>1')
                  write(EU,*) '    Act. wavelength: ', WL_G(i)
                  return 1
               endif
            endif
         enddo
      endif

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
C********************************************************************C
