C********************************************************************C
C
C       SCIATRAN - Version 3.4 - Last modification: 15 Jan 2014
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_CLIMATOLOGY_INPUT ( PATH_TO_CLIMATOLOGY_DB, 
     $     DO_PROFILE_LAT_INTERP, MPI_PROFILE_FILENAME, CLIMAUNIT, *)

      USE ST_INPUT, ONLY: MAX_LAY, ALBEDO_FILENAME, DAY_OF_YEAR 
      USE ST_VARS,  ONLY: GROUND_HEIGHT, LATITUDE, LONGITUDE
      USE LOG_PARS, ONLY: LUNIT, EU, LOG_LEVEL, C_UNIT, DIR_OUT, N_SSO,
     $                     N_DSO
      USE DEPOLARIZATION_ALBEDO_AEROSOL, ONLY: DO_SPECTRAL_ALBEDO, 
     $                                         ALBEDO_CONST
      
      CHARACTER (LEN=*), PARAMETER :: R_NAME='GT_CLIMATOLOGY_INPUT'
      
      INTEGER              TYPE_ID, CLIMAUNIT, LL, ID_NUM, ID_2
      
      LOGICAL              DO_PROFILE_LAT_INTERP
      
      CHARACTER (LEN=*)    PATH_TO_CLIMATOLOGY_DB, MPI_PROFILE_FILENAME
      CHARACTER (LEN=2)    MON_ID
      CHARACTER (LEN=3)    LAT_ID(18)
      CHARACTER (LEN=15)   OUTFILENAME, ISCCP_CLOUD_TYPE(9)
      CHARACTER (LEN=150)  MPI_FILENAME, PATH, FILE1, FILE2, DIR_NAME
      CHARACTER (LEN=*), PARAMETER :: CHAR = "''", FMT_LOG = '(L1)'
      DATA ISCCP_CLOUD_TYPE /'Cirrus',  'Altocumulus', ' Cumulus',
     $      'Cirrostratus', 'Altostratus', 'Stratocumulus',
     $      'Deep convection', 'Nimbostratus', 'Stratus'/


      DOUBLE PRECISION, PARAMETER :: DELZ_CLOUD_FREE=0.2D0

c      Check some input parameters
c      ---------------------------
       if (latitude.le.-90.0d0.or.latitude.ge.90.0d0) then
          call write_message(r_name, 'ERROR', 
     $    'Latitude out of bounds (-90 < Lat < +90)')
          write(eu, *) '    Value is: ', latitude
          return 1
       endif
       if (longitude.lt.0.0d0.or.longitude.gt.360.0d0) then
          call write_message(r_name, 'ERROR', 
     $    'Longitude out of bounds (0 - 360)')
          write(eu, *) '    Value is: ', longitude
          return 1
       endif
       if (day_of_year.lt.1.or.day_of_year.gt.360) then
          call write_message(r_name, 'ERROR', 
     $    'Day of year climatology out of bounds (1 - 360)')
          write(eu, *) '    Value is: ', day_of_year
          return 1
       endif

       IF( LOG_LEVEL(1) >= N_SSO ) write(lunit,*) 
     $      ' *** writing: '//TRIM(DIR_OUT)//'/CLIMATOLOGY.OUT'

       open(unit=climaunit, file=TRIM(DIR_OUT)//'/CLIMATOLOGY.OUT')
       write(climaunit,'(a)') 
     &'***************************************************************'
       write(climaunit,'(a)') 
       write(climaunit,'(a)') 
     $      ' Climatology output file. Contains settings to be changed '
       write(climaunit,'(a)') 
     $      ' according to the specified date and geolocation'
       write(climaunit,'(a)') 
       write(climaunit,'(a)') 
     &'***************************************************************'
       write(climaunit,*)

c      ----------------- MPI profile filename -----------------
       LL = INDEX(MPI_PROFILE_FILENAME,'/mon',.TRUE.)
       PATH = MPI_PROFILE_FILENAME(1:LL)
       CALL GET_MPI_FILENAME(*880)
       
       MPI_PROFILE_FILENAME = MPI_FILENAME
       WRITE(CLIMAUNIT, '(2A)') 'Standard profile scenario file name: ',
     $      CHAR//TRIM(MPI_FILENAME)//CHAR
       WRITE(CLIMAUNIT,*)

c      ----------------- surface elevation setup --------------
       CALL SURFACE_ELEVATION(*880)
       WRITE(CLIMAUNIT, '(A,F6.2)') 'Height above sea level, [km]: ',
     $      GROUND_HEIGHT
       WRITE(CLIMAUNIT,*)


c      ----------------- surface type setup --------------------
       CALL SURFACE_TYPE(*880)

       IF( .NOT. DO_SPECTRAL_ALBEDO ) THEN
          CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $         'Constant albedo for 760 nm is used')
       ENDIF
          
       CALL TRIM_PATH(ALBEDO_FILENAME, DIR_NAME)
          
       SELECT CASE (TYPE_ID)
          CASE(1) 
             ALBEDO_FILENAME = TRIM(DIR_NAME)//'albedo_rg_sand.dat'
             ALBEDO_CONST = 0.294d0
          CASE(2) 
             ALBEDO_FILENAME = TRIM(DIR_NAME)//'albedo_rg_soil.dat'
             ALBEDO_CONST = 0.195d0
          CASE(3) 
             ALBEDO_FILENAME = TRIM(DIR_NAME)//'albedo_rg_snow.dat'
             ALBEDO_CONST = 0.908d0
          CASE(4) 
             ALBEDO_FILENAME =TRIM(DIR_NAME)//'albedo_rg_vegetation.dat'
             ALBEDO_CONST = 0.410d0
          CASE(5) 
             ALBEDO_FILENAME = TRIM(DIR_NAME)//'albedo_rg_water.dat'
             ALBEDO_CONST = 0.012d0
          CASE DEFAULT ;  GOTO 209
       END SELECT

       WRITE(CLIMAUNIT, '(A,L1)') 'Do spectral albedo from data base: ',
     $      DO_SPECTRAL_ALBEDO
       WRITE(CLIMAUNIT,*)
       WRITE(CLIMAUNIT, '(2A)') 'Spectral albedo filename: ',
     $       CHAR//TRIM(ALBEDO_FILENAME)//CHAR
       WRITE(CLIMAUNIT,*)

       CLOSE(CLIMAUNIT)
       RETURN

 209   CONTINUE
       CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $      'Wrong surface type index (1,2,3,4,5)')
       WRITE(EU,*) '    index is: ', TYPE_ID
       RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

       CONTAINS
c************************************************************************
      SUBROUTINE GET_MPI_FILENAME(*)

      CHARACTER (LEN=*), PARAMETER :: R_NAME='GET_MPI_FILENAME'

      DOUBLE PRECISION     LAT_VAL(18)

      SELECT CASE (DAY_OF_YEAR)
         CASE(1:30)    ;  MON_ID = '01'
         CASE(31:60)   ;  MON_ID = '02'
         CASE(61:90)   ;  MON_ID = '03'
         CASE(91:120)  ;  MON_ID = '04'
         CASE(121:150) ;  MON_ID = '05'
         CASE(151:180) ;  MON_ID = '06'
         CASE(181:210) ;  MON_ID = '07'
         CASE(211:240) ;  MON_ID = '08'
         CASE(241:270) ;  MON_ID = '09'
         CASE(271:300) ;  MON_ID = '10'
         CASE(301:330) ;  MON_ID = '11'
         CASE(331:360) ;  MON_ID = '12'
         CASE DEFAULT
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Day out of bounds (1-360)!')
            WRITE(EU,*) '    Value is: ', DAY_OF_YEAR
            RETURN 1
      END SELECT

      LAT_ID = (/'85s','75s','65s','55s','45s','35s','25s','15s','05s',
     $     '05n','15n','25n','35n','45n','55n','65n','75n','85n'/)
      LAT_VAL = (/-85.0D0, -75.0D0, -65.0D0, -55.0D0, -45.0D0, -35.0D0, 
     $     -25.0D0, -15.0D0, -5.0D0, 5.0D0, 15.0D0, 25.0D0, 35.0D0, 
     $     45.0D0, 55.0D0, 65.0D0, 75.0D0, 85.0D0/)

      ID_NUM = MINLOC(ABS(LAT_VAL(:) - LATITUDE),1)
      IF(LATITUDE.GE.LAT_VAL(ID_NUM)) THEN
         ID_2 = ID_NUM + 1
      ELSE
         ID_2 = ID_NUM - 1
      END IF
      IF(ID_2.LT.1.OR.ID_2.GT.18) DO_PROFILE_LAT_INTERP = .FALSE.

      MPI_FILENAME = TRIM(PATH)//
     $     'mon'//MON_ID//'lat'//LAT_ID(ID_NUM)//'.mpi'

c     if latitude interpolation determine second filename
c     ---------------------------------------------------
      IF (DO_PROFILE_LAT_INTERP) THEN
c     -------------------------------

         FILE1 = MPI_FILENAME
         FILE2 = TRIM(PATH)//'mon'//MON_ID//'lat'//LAT_ID(ID_2)//'.mpi'
         OUTFILENAME = 'profile_int.mpi'
      
         CALL GT_LAT_INTERP_PROFILE(LAT_VAL(ID_NUM),LAT_VAL(ID_2),*880)
 
c        latitude interpolated MPI profiles are stored locally !
         MPI_FILENAME = OUTFILENAME

      ENDIF
c     -----
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE

c************************************************************************
      SUBROUTINE GT_LAT_INTERP_PROFILE (LAT_1, LAT_2, *)

      CHARACTER (LEN=*), PARAMETER :: R_NAME='GT_LAT_INTERP_PROFILE'

      CHARACTER (LEN=80)   HEADER

      INTEGER              NL_1, NL_2, NL_OUT, I, J
      INTEGER              K_1(MAX_LAY), K_2(MAX_LAY), K_OUT(MAX_LAY)

      LOGICAL              DO_IDENTICAL_GRIDS, ERROR

      DOUBLE PRECISION     Z_1(MAX_LAY), Z_2(MAX_LAY), Z_OUT(MAX_LAY)
      DOUBLE PRECISION     P_1(10,MAX_LAY), P_2(10,MAX_LAY)
      DOUBLE PRECISION     P_OUT(10,MAX_LAY), PTMP2(MAX_LAY)
      DOUBLE PRECISION     LAT_1, LAT_2, DEL_LAT


c     Read lowest latitude file
c     -------------------------
      IF( LOG_LEVEL(1) >= N_SSO ) 
     $     WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILE1)
      OPEN(UNIT=C_UNIT, FILE=FILE1, STATUS='OLD')
      DO
         READ(C_UNIT,'(A)') HEADER
         IF (HEADER(1:1).NE.';') EXIT
      END DO
      BACKSPACE C_UNIT
      READ(C_UNIT,*) NL_1
      BACKSPACE C_UNIT
      DO I = NL_1, 1, -1
         READ(C_UNIT,*) K_1(I), Z_1(I), (P_1(J,I), J=1,10)
      ENDDO
      CLOSE(C_UNIT)
      

c     If both latitudes essentially identical just copy to results arrays
c     -------------------------------------------------------------------
       IF (ABS(LAT_1 - LAT_2).LT.-0.01D0) THEN
         NL_OUT = NL_1
         K_OUT(1:NL_OUT) = K_1(1:NL_1)
         Z_OUT(1:NL_OUT) = Z_1(1:NL_1)
         P_OUT(:,1:NL_OUT) = P_1(:,1:NL_1)
         GOTO 900
      ENDIF

c     Read highest latitude file
c     --------------------------
      IF( LOG_LEVEL(1) >= N_SSO ) 
     $     WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILE2)
      OPEN(UNIT=C_UNIT, FILE=FILE2, STATUS='OLD')
      DO
         READ(C_UNIT,'(A)') HEADER
         IF (HEADER(1:1).NE.';') EXIT
      END DO
      BACKSPACE C_UNIT
      READ(C_UNIT,*) NL_2
      BACKSPACE C_UNIT
      DO I = NL_2, 1, -1
         READ(C_UNIT,*) K_2(I), Z_2(I), (P_2(J,I), J=1,10)
      ENDDO
      CLOSE(C_UNIT)
      
c     Identical altitude grids ?
c     --------------------------
      DO_IDENTICAL_GRIDS = .FALSE.
      IF (NL_1 == NL_2) THEN
         DO_IDENTICAL_GRIDS = .TRUE.
         DO I = 1, NL_1
            IF(ABS(Z_1(I) - Z_2(I)).GT.0.001D0) 
     $           DO_IDENTICAL_GRIDS = .FALSE.
         ENDDO
      ENDIF

c     Normalised latitude difference
c     ------------------------------
      DEL_LAT = (LATITUDE - LAT_1)/(LAT_2 - LAT_1)

      IF (.NOT.DO_IDENTICAL_GRIDS) THEN
c     Different altitude grids: interpolate high latitude file on
c     LOWEST latitude altitude grid
c     -> LOWEST latitude altitude grid is reference altitude grid !
c
c     Extrapolation:
c     If high-lat grid completely covers low-lat/ref grid no 
c     extrapolation is required; however, at surface or at TOA
c     high-grid might be too small: in this case use low-lat
c     data/profiles for extrapolation
c     -------------------------------------------------------------
         DO J = 1, 10
            CALL LINTP2(NL_2, Z_2(1:NL_2), P_2(J,1:NL_2), 
     $           NL_1, Z_1, PTMP2, ERROR)
            IF (ERROR) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Error in lintp2')
               RETURN 1
            END IF
            P_2(J,1:NL_1) = PTMP2(1:NL_1)
         ENDDO
c     high-grid might not cover low-grid completety: 
c     copy approp. values from low-grid to new high-grid!
c     (lintp2 uses constant extrapolation!)
         DO I = 1, NL_1
            IF (Z_1(I).LT.Z_2(1)) P_2(:,I) = P_2(:,I)
            IF (Z_1(I).GT.Z_2(NL_2)) P_2(:,I) = P_1(:,I)
         END DO
         K_2(1:NL_1) = K_1(1:NL_1)
         Z_2(1:NL_1) = Z_1(1:NL_1)
      ENDIF

c     Now grids are identical: make latitude interpolation
c     ----------------------------------------------------
      NL_OUT = NL_1
      K_OUT(1:NL_OUT) = K_1(1:NL_1)
      Z_OUT(1:NL_OUT) = Z_1(1:NL_1)
      FORALL(I = 1:NL_OUT, J = 1:10) 
     $     P_OUT(J,I) = P_1(J,I) + (P_2(J,I) - P_1(J,I))*DEL_LAT

c     Store results file (locally !)
c     ------------------------------
 900  CONTINUE
      IF( LOG_LEVEL(1) >= N_SSO )
     $     WRITE(LUNIT,*) ' *** Writing: ', TRIM(OUTFILENAME)
      OPEN(UNIT=C_UNIT, FILE=OUTFILENAME)
      WRITE(C_UNIT,'(A)')
     $     '; Atmospheric profiles (interpolated by latitude)'
      WRITE(C_UNIT,'(A)')'; Derived from: '//MPI_FILENAME//
     $     ' and mon'//MON_ID//'lat'//LAT_ID(ID_2)//'.mpi'
      WRITE(C_UNIT,'(A, F6.2)')'; Latitude: ', LATITUDE
      WRITE(C_UNIT,'(A)') 
     &'; k      z        p       t           o3          no2          '
     &//'so2          clo          bro          no3'
     &//'         hcho         oclo'
      DO I = NL_OUT, 1, -1
         write(C_UNIT,'(i3,1x,f6.2,1x,f8.2,1x,f7.2,8(2x,1pe11.4))')
     $   K_OUT(I), Z_OUT(I), (P_OUT(J,I), J=1,10)
      ENDDO
      CLOSE(C_UNIT)

      END SUBROUTINE
C********************************************************************C
      SUBROUTINE SURFACE_ELEVATION(*)

c       Input: Latitude (from -90 to 90), Longitude (between 0 and 360)
c       Output: Surface Height [meter]
c
c       Data have been compiled from unpublished sources (Thomas Kurosu,
c       R. Guzzi, private communications).

      CHARACTER (LEN=17), PARAMETER :: R_NAME='SURFACE_ELEVATION'

      INTEGER, PARAMETER :: NUM_LAT = 181, NUM_LON = 362

      INTEGER              N_LAT, N_LON, N_1, N_2

      CHARACTER (LEN=200)  FILE_GH

      DOUBLE PRECISION     LAT(NUM_LAT), LON(NUM_LON)
      DOUBLE PRECISION     HEIGHT(NUM_LON,NUM_LAT)
      DOUBLE PRECISION     LAT_SLOPE, LON_SLOPE

      N_LAT = FLOOR(LATITUDE) + 91
      N_LON = FLOOR(LONGITUDE - 0.5) + 2

      FILE_GH = TRIM(PATH_TO_CLIMATOLOGY_DB)//'guzzi_hgt1x1.dat'
      IF( LOG_LEVEL(1) >= N_SSO ) 
     $     WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILE_GH)
      OPEN(C_UNIT,FILE=FILE_GH, ERR=999, STATUS='OLD')

      READ(C_UNIT,*) N_1, N_2
      IF(N_1.NE.NUM_LON.AND.N_2.NE.NUM_LAT) GOTO 200
      READ(C_UNIT,*) 
       
      READ(C_UNIT,*) LON
      READ(C_UNIT,*) 
      READ(C_UNIT,*) LAT
      READ(C_UNIT,*) 
      READ(C_UNIT,*) HEIGHT

      CLOSE(C_UNIT)

c     Twodimensional Interpolation of the Tabulated Values
c     ****************************************************
      LAT_SLOPE = (LATITUDE - LAT(N_LAT))/(LAT(N_LAT + 1) - LAT(N_LAT))
      LON_SLOPE = (LONGITUDE - LON(N_LON))/(LON(N_LON + 1) - LON(N_LON))

      GROUND_HEIGHT = 
     $     (1.0D0 - LAT_SLOPE)*(1.0D0 - LON_SLOPE)*HEIGHT(N_LON, N_LAT)+ 
     $     LAT_SLOPE*(1.0D0 - LON_SLOPE)*HEIGHT(N_LON, N_LAT + 1) + 
     $     LAT_SLOPE*LON_SLOPE*HEIGHT(N_LON + 1, N_LAT + 1) + 
     $     (1.0D0 - LAT_SLOPE)*LON_SLOPE*HEIGHT(N_LON + 1, N_LAT)

      GROUND_HEIGHT = GROUND_HEIGHT/1.0D3 ! M -> KM

      RETURN

 200  CONTINUE
      CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong data bese input')
      WRITE(EU,*) '    File: ', TRIM(FILE_GH)
      RETURN 1

 999  CONTINUE
      CALL WRITE_MESSAGE(R_NAME,
     $     'ERROR', 'Can not open input file!')
      WRITE(EU,*) '    ?: ', TRIM(FILE_GH)
      RETURN 1
      
      END SUBROUTINE

C********************************************************************C
      SUBROUTINE SURFACE_TYPE(*)

c       Input: Latitude (between -90 and 90), Longitude (from 0 to 360)
c       Output: Vegetation type 
c
c       Data are taken from the GUZZI Vegetation Data Set (1x1-Grid).
c 
c       The output is an integer number between 1 and 5 corresponding to the 
c       following classification scheme:
c       1 - Sand; 2 - Soil; 3 - Ice/Snow ; 4 - Vegetation ; 5 - Water

      CHARACTER (LEN=200)  FILE_ST
      CHARACTER (LEN=12), PARAMETER :: R_NAME='SURFACE_TYPE'

      INTEGER, PARAMETER :: NUM_LAT = 182, NUM_LON = 362
      INTEGER              I, N_LAT, N_LON, N_1, N_2
      INTEGER              S_TYPE(NUM_LON,NUM_LAT)
      DOUBLE PRECISION     LAT(NUM_LAT), LON(NUM_LON)
      
      FILE_ST = TRIM(PATH_TO_CLIMATOLOGY_DB)//'guzzi_veg1x1.dat'
      IF( LOG_LEVEL(1) >= N_DSO ) 
     $     WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILE_ST)
      OPEN(C_UNIT,FILE=FILE_ST, ERR=999, STATUS='OLD')

      READ(C_UNIT,*) N_1, N_2
      IF(N_1.NE.NUM_LON.AND.N_2.NE.NUM_LAT) GOTO 200
      READ(C_UNIT,*) 
       
      READ(C_UNIT,*) LON
      READ(C_UNIT,*) 
      READ(C_UNIT,*) LAT

      DO I = 1, NUM_LAT 
         READ(C_UNIT,*) 
         READ(C_UNIT,*) S_TYPE(:,I)
      END DO
      CLOSE(C_UNIT)

      N_LAT = MINLOC(DABS(LAT(:) - LATITUDE),1)
      N_LON = MINLOC(DABS(LON(:) - LONGITUDE),1)
      TYPE_ID=S_TYPE(N_LON, N_LAT)
      
      RETURN

 200  CONTINUE
      CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong data bese input')
      WRITE(EU,*) '    File: ', TRIM(FILE_ST)
      RETURN 1

 999  CONTINUE
      CALL WRITE_MESSAGE(R_NAME,
     $     'ERROR', 'Can not open input file!')
      WRITE(EU,*) '    ?: ', TRIM(FILE_ST)
      RETURN 1
      
      END SUBROUTINE

C********************************************************************C
      END
C********************************************************************C
      SUBROUTINE TRIM_PATH(NAME_INP, PATH)
      INTEGER             I, OUT_LEN, N_CHAR, N_PATH
      CHARACTER (LEN=*)   NAME_INP, PATH

      OUT_LEN = 0
      N_CHAR = LEN_TRIM(NAME_INP)
      DO I = N_CHAR, 1, -1
         IF(NAME_INP(I:I) == '/') THEN
            OUT_LEN = I
            EXIT
         END IF
      END DO

      N_PATH = LEN(PATH)
      PATH(1:OUT_LEN) = NAME_INP(1:OUT_LEN)
      FORALL(I = OUT_LEN + 1:N_PATH) PATH(I:I) = ' '

      RETURN
      END
c************************************************************************
