C********************************************************************C
C
C	SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_CLOUD_CHECK (*)
c     Call in "gt_settings_check.f"

      USE CLOUDS,     ONLY: DO_CLOUD, N_CLDPROF, CLDPROFILE, Var_type,
     $                      N_CLD_LAYERS, CLD_DM, CLD_LAYER_HEIGHTS
      USE CLD_INP,    ONLY: D_MF, PHASE_NAME, TDS, CLD_INHOM, Use_Bdb,
     $                      HOW_CLD_COEF, Plus_ice, Phm_const, MLCS, 
     $                      Use_Ydb, Use_Ydb_SCM, CRST
      USE CLOUDS_SS,  ONLY: DO_SSRCA
      USE DELTA_M,    ONLY: AER_DM_FLG
      USE GRIDS,      ONLY: NLAYERS, ZZZ
      USE GT_POLYN,   ONLY: NLEGEN
      USE INFO,       ONLY: CLD_INFO
      USE LOG_PARS,   ONLY: EU
      USE ST_VARS,    ONLY: GROUND_HEIGHT, RTM_CORE, RTM_TYPE,
     $                      RAD_FIELD, USE_PSM
      USE SPHER_PARS, ONLY: N_SOL_ANG, N_LOS_ANG, N_AZ_ANG, Z_SOL_F
      USE TYPE_VARS,  ONLY: KND
      USE WF_NUM,     ONLY: NUM_CLD, Wat_or_Ice
c     ------------------------------------------------------------------

c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'GT_CLOUD_CHECK'
      CHARACTER (LEN=80) TXT
      CHARACTER (LEN=100) Msg(10)
      INTEGER     I, J, K, INX, II
      REAL (KND)  BASE, TOP, BBASE, TTOP
      LOGICAL     MWP_TYPE
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     WRITE(*,*) '  -1 '
      IF( TRIM(RTM_TYPE)=='spher_scat'.AND.RAD_FIELD ) THEN
         IF( Use_Ydb.AND.(.NOT.Use_Ydb_SCM) ) THEN
            Msg(1)='Usage Yang DB of ice crystals in spherical '
            Msg(2)='  geometry requires the second entry of parameter'
            Msg(3)='  "Form of ice crystals" has to be set to "yes".'
            Msg(4)='Check files "cf_cloud.inp" '
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880) 
         ENDIF
         IF( Use_Ydb.AND.(.NOT.Use_PSM) ) THEN
            Msg(1)='The option "Use precalculated scattering matrix"'
            Msg(2)='  has to be switched on for correct usage of  '
            Msg(3)='  ice crystals Yang DB in spherical geometry. '
            Msg(4)='Check files "control_ac.inp" '
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)             
         ENDIF
      ENDIF

c      WRITE(*,*) '   Use_Ydb_SCM  DO_SSRCA ', Use_Ydb_SCM, DO_SSRCA
      IF(Use_Ydb_SCM.AND.(.NOT.DO_SSRCA)  ) THEN
         IF( TRIM(RTM_TYPE) /= 'spher_scat') THEN
            Msg(1)='Correct scattering matrix for Yang DB '
            Msg(2)='  can be used only if the parameter   '
            Msg(3)='  "Single scattering correction" is set to "t".'
            Msg(4)='Check files "cf_cloud.inp" and "control_ac.inp"'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
      ENDIF

      DO II = 1, UBOUND(CLD_INFO,1)
         IF(.NOT.CLD_INFO(II)%WF_FLG) CYCLE

         IF (TRIM(RTM_TYPE) == 'spher_scat') THEN
            Call WRITE_MESSAGE (R_NAME, 'ERROR',
     $           'Weighting functions for cloud parameters are not'
     $           //' implemented for "spher_scat" RTM type')
            RETURN 1 
         ENDIF

         SELECT CASE ( CLD_INFO(II)%NAME )
         CASE('cld-sca_nomix','cld-abs_nomix','cld-soot')
         CASE('cld-sca_water','cld-abs_water')
         CASE('cld-sca_ice','cld-abs_ice')
         CASE('cld-pnc_nomix','cld-pnc_water','cld-pnc_ice')
         CASE('cld-lwc_nomix','cld-lwc_water','cld-lwc_ice')
         CASE('cld-ref_nomix','cld-ref_water','cld-ref_ice')
c           ... Numerical calculation of derivatives
            IF(TRIM(HOW_CLD_COEF) == 'from data base') THEN
               DO J = 1,N_CLD_LAYERS
                  IF( PHASE_NAME(J)%S=='user' ) GOTO 202
                  IF( TDS(J)%S=='ice'.OR.TDS(J)%S=='mixed') THEN
                     SELECT CASE( CRST(J)%SHP(1)%ICE_FORM)
                     CASE('hex','fra')
              Msg(1)='Weighting function w.r.t. effective radius for'
              Msg(2)='  "Hexagon" and "Fractal" particles requires that'
              Msg(3)='  parameter '
              Msg(4)='  "Scattering and absorption coefficients"'
              Msg(5)='  should be set to "analytically".' 
              Msg(6)='Check control file "cf_cloud.inp".' 
              CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)            
                     CASE DEFAULT
                     END SELECT
                  ENDIF
               ENDDO
            ENDIF

         CASE('cld-cth','cld-cbh','cld-tau_nomix','cld-tau_water',
     $           'cld-tau_ice')
             Msg(1)='Weighting function for scalar cloud parameters'
             Msg(2)='  "CTH", "CBH", and "TAU " can be calculated'
            IF (TRIM(RTM_TYPE) /= 'pp_scat') THEN
               Msg(3)='   in the plane-parallel approximation only!'
               Msg(4)='Check "RTM_TYPE" in the file "control.inp".' 
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)            
            ENDIF
            IF( N_SOL_ANG > 1 ) THEN
               Msg(3)='   for a single solar angle only!'
               Msg(4)='Set the number of solar angles to 1'
               Msg(5)='   in the file "control_geom.inp".' 
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
            ENDIF
            IF( (N_LOS_ANG > 1) .OR. (N_AZ_ANG > 1) ) THEN
               Msg(3)='   for a single viewing angle only!'
               Msg(4)='Set the number of viewing and/or azimuth '//
     $              'angles to 1'
               Msg(5)='   in the file "control_geom.inp".' 
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
            ENDIF
            IF( DO_SSRCA ) THEN
               Msg(1)='Weighting function for scalar cloud parameters'
               Msg(2)='  "CTH", "CBH", and "TAU " cannot be calculated'
               Msg(3)='   if "Single scattering correction" '//
     $              'parameter is set to "t".'
               Msg(4)='Check file "control_ac.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)                
            ENDIF

         CASE DEFAULT
            Msg(1)='Weighting function type of cloud is not recognized.'
            Msg(2)='  Check WF names in control file "control_wf.inp"'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)                
         END SELECT               
      ENDDO
c      WRITE(*,*) '  -2 '

c     Numerical perturbation of cloud parameters
c     ------------------------------------------
      IF( NUM_CLD ) THEN
         DO I = 1, N_CLD_LAYERS
            SELECT CASE ( TDS(I)%S )
            CASE ('water') 
c           ------------------------------------------------------------
               IF( TRIM(Wat_or_Ice) == 'ice' ) THEN
                  Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                  Msg(1)='Numerical perturbation for "ice"'
                  Msg(2)='  can be performed only if '
                  Msg(3)='  "Thermodynamic state" is set to "mixed". '
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
               ENDIF
            CASE ('ice') 
c           ------------------------------------------------------------
               IF( TRIM(Wat_or_Ice) == 'water' ) THEN
                  Msg(1)='Numerical perturbation for "water"'
                  Msg(2)='  can be performed only if '
                  Msg(3)='  "Thermodynamic state" is set to "mixed". '
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
               ENDIF
            CASE('mixed')
c           ------------------------------------------------------------
               IF( TRIM(Wat_or_Ice) == 'nomix'  ) THEN
                  Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                  WRITE(EU,*)
     $            '   Thermodynamic state of cloud: ', TDS(I)%S
                  WRITE(EU,*)
     $            '   Numerical perturbation for: ', TRIM(Wat_or_Ice)
                  WRITE(EU,*) 
     $            '   Numerical perturbation for option "nomix"'
                  WRITE(EU,*) 
     $            '   can be performed only if "Thermodynamic state"'
                  WRITE(EU,*) 
     $            '   is set to "water" or "ice" for all layers.'
                  RETURN  1               
               ENDIF
c           ------------------------------------------------------------ 
           END SELECT
         ENDDO  ! over cloud layers (I)
      ENDIF  ! NUM_CLD

c     Phase function depends on the altitude within cloud layer      
c     ---------------------------------------------------------
      IF( .NOT.Phm_const ) THEN
c         IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
c            Call WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
c            WRITE (EU,'(A)')
c     $           '   Sorry, "spher_scat" mode is not allowed if '
c            WRITE (EU,'(A)')
c     $           '    the phase function depends on the altitude.'
c            WRITE (EU,'(A)')
c     $           '   Check control parameter '//
c     $           '"Use constant phase function within cloud layer"'
c            WRITE (EU,'(A)')
c     $           '   in the input file "cf_cloud.inp".'
c            RETURN  1
c         ENDIF
c              What this means?
         IF( D_MF=='D-F'.OR.  D_MF=='D-S') THEN
            Call WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
            WRITE (EU,'(A)')
     $           '   Delta-M truncation technique is allowed only '
            WRITE (EU,'(A)')
     $           '   if the phase function depends on the altitude.'
            WRITE (EU,'(A)')
     $           '   Check control parameter "Delta-M or Delta-Fit"'
            WRITE (EU,'(A)')
     $           '   in the input file "cf_cloud.inp".'
            RETURN  1
         ENDIF      
      ENDIF

      DO I = 1, N_CLD_LAYERS
         IF( MLCS(I)%Phm_co ) CYCLE
         SELECT CASE( TDS(I)%S )
         CASE('ice','water')
         CASE('mixed')
            IF(.NOT.Use_Ydb) THEN
               Call WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
               WRITE (EU,'(A)')
     $              '   Phase function has to be constant within '
               WRITE (EU,'(A)')
     $              '   a mixed cloud layer.'
               WRITE (EU,'(A)')
     $            '   Check control parameter'//
     $            '   "Use constant phase function within cloud layer"'
               WRITE (EU,'(A)')
     $              '   in the input file "cf_cloud.inp".'
               RETURN  1
            ENDIF
         END SELECT

         SELECT CASE( PHASE_NAME(I)%S )
         CASE ('hg','iso','user')
            Call WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
            WRITE (EU,'(A)')
     $           '   Phase function can depend on altitude within '
            WRITE (EU,'(A)')
     $           '   an cloud layer if '//
     $           '"Phase function of cloud layers"'
            WRITE (EU,'(A)')
     $           '   is set to "db" only.'
            WRITE (EU,'(A)')
     $           '   Check control parameter'//
     $           '   "Phase function of cloud layers"'
            WRITE (EU,'(A)')
     $           '   in the input file "cf_cloud.inp".'
            RETURN  1
         END SELECT
      ENDDO  ! over cloud layers (I)
c      ENDIF

c      WRITE(*,*) '  -3 '

c     Clouds as Layers
c     ----------------
      IF( DO_CLOUD ) Then
         IF( TRIM(RTM_TYPE) == 'spher_scat') THEN
            TOP = CLD_LAYER_HEIGHTS(1,2)
c            WRITE(*,*) TOP
            IF( Z_SOL_F < TOP ) THEN
               CALL WRITE_MESSAGE (R_NAME, 'ERROR',           
     $              'Cloud in spherical atmosphere.')
               WRITE (EU,'(A)') 
     $              '   Accuracy of spherical transmission '//
     $              'calculation need to be enhanced.'
               WRITE (EU,'(A)')
     $              '   Set parameter "Solar fine grid height" in'
               WRITE (EU,'(A,f7.2,A)')
     $              '   control file "control_ac.inp" greater than ~',
     $              TOP + 2,' km' 
               WRITE (EU,'(A,1x,f7.2,A)') 
     $              '   cloud top height is :', TOP,' km'
               RETURN 1 
            ENDIF
         ENDIF
         IF( TRIM(RTM_CORE) == 'ASYMP' ) THEN
            AER_DM_FLG = .FALSE.
            CLD_DM(1) = 0
            Call WRITE_MESSAGE (R_NAME, 'WARNING',        
     $           ' Aerosol D-M switch off; Cloud D-M switch off') 
         ENDIF

c        Consistence with the Delta-fit option
c        =====================================
         DO I = 1, N_CLD_LAYERS
            IF( PHASE_NAME(I)%S=='user' ) THEN
               IF(TRIM(CLD_INHOM)=='yes'.AND.TDS(I)%S/='alay') THEN
                  IF(TRIM(HOW_CLD_COEF) == 'from data base') GOTO 200
               ENDIF
            ENDIF
            IF( PHASE_NAME(I)%S /= 'user' ) THEN
               IF( D_MF == 'D-F'.OR.  D_MF == 'D-S') THEN
                  Call WRITE_MESSAGE (R_NAME, 'ERROR',
     $             'Delta-fit(or S) approximation for cloud is allowed'
     $                 //' for user-defined phase function only')
                  RETURN  1
               ENDIF
            ENDIF
         ENDDO

c        Consistence with single scattering correction algorithm
c        =======================================================
c         WRITE(*,*) '  DO_SSRCA :', DO_SSRCA
         IF( DO_SSRCA ) THEN
            IF( Use_Ydb.AND.(.NOT.Use_Ydb_SCM) ) THEN
               Msg(1)='If "Single scattering correction" is set to "t",'
               Msg(2)='  correct scattering matrix for Yang DB ice '
               Msg(3)='  habits can be used only if the second entry '
               Msg(4)='  in the "Form of ice crystals" input line '
               Msg(5)='  is set to "yes".'
               Msg(6)='Check files "cf_cloud.inp" and "control_ac.inp"'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
            ENDIF
            TXT = 'Single scattering correction is not allowed'
            IF (TRIM(RTM_TYPE) == 'spher_scat') THEN
               Call WRITE_MESSAGE(R_NAME,'ERROR',' ')
               WRITE(EU,'(8x,A)') TRIM(TXT)
               WRITE(EU,'(8x,A)') 'for "spher_scat" RTM type.'
               RETURN  1               
            ENDIF
c            DO I = 1, N_CLD_LAYERS            
c               IF( TDS(I)%S == 'mixed' ) THEN
c                  Call WRITE_MESSAGE(R_NAME,'ERROR',' ')
c                  WRITE(EU,'(8x,A)') TRIM(TXT)
c                  WRITE(EU,'(8x,A)') 'for mixed cloud.'
c                  RETURN  1
c               ENDIF
c            ENDDO
         ENDIF

c        Quantities are cloud layer dependent
c        ----------------------------------------------
         MWP_TYPE = .FALSE.
         DO I = 1, N_CLD_LAYERS
            IF( TDS(I)%S == 'mixed' ) THEN
               MWP_TYPE = .TRUE.               
               IF( Use_Bdb ) THEN
c                  Call WRITE_MESSAGE(R_NAME,'ERROR',' ')
c                  WRITE(EU,'(8x,A)')
c     $                 ' Option "mixed" should be checked if ' 
c                  WRITE(EU,'(8x,A)')
c     $                 '"AggregateSolidColumns" ice crystal used.'
c                  RETURN  1
                  Call WRITE_MESSAGE(R_NAME,'WARNING',' ')
                  WRITE(EU,'(8x,A)')
     $                 ' Option "mixed" should be checked if ' 
                  WRITE(EU,'(8x,A)')
     $                 '"AggregateSolidColumns" ice crystal used.'
               ENDIF
            ENDIF
         ENDDO

         INX = 1
         IF( ANY(Plus_ice) ) INX = 2
         DO I = 1, N_CLD_LAYERS
c           Check cloud thermodynamic state
c           ===============================
!            WRITE(*,*)  I, INX, TDS(I)%S, Var_type(I,INX)%S
            SELECT CASE ( TDS(I)%S )
               CASE('alay')
                  SELECT CASE ( Var_type(I,1)%S )
                  CASE ('TAU')
                  CASE DEFAULT
                     Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                     WRITE (EU,*)
     $               '  Type of input integral parameter is not allowed'
                     WRITE (EU,*) 'Selected parameter: ',Var_type(I,1)%S
                     WRITE (EU,111) I
                     RETURN 1               
                  END SELECT
                  
               CASE('water')
                  SELECT CASE ( Var_type(I,1)%S )
                  CASE ('TAU','LWP','LWC','PNC','IWP','IWC','CNC')
                  CASE DEFAULT
                     Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                     WRITE (EU,*)
     $               '   Type of input integral parameter not found'
                     WRITE (EU,*) 'Selected parameter: ',Var_type(I,1)%S
                     WRITE (EU,111) I
                     RETURN 1               
                  END SELECT
               CASE('ice-sm')
                  SELECT CASE ( Var_type(I,INX)%S )
                  CASE ('TAU','LWP','LWC','PNC','IWP','IWC','CNC')
                  CASE DEFAULT
                     Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                     WRITE (EU,*)
     $               '   Type of input integral parameter not found'
                     WRITE (EU,*) 'Selected parameter: ', 
     $                    Var_type(I,INX)%S
                     WRITE (EU,111) I
                     RETURN 1               
                  END SELECT
               CASE('ice')
                  SELECT CASE ( Var_type(I,INX)%S )
                  CASE ('TAU','LWP','LWC','PNC','IWP','IWC','CNC')
                  CASE DEFAULT
                     Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                     WRITE (EU,*)
     $               '   Type of input integral parameter not found'
                     WRITE (EU,*) 'Selected parameter: ', 
     $                    Var_type(I,INX)%S
                     WRITE (EU,111) I
                     RETURN 1               
                  END SELECT
               CASE('mixed','mixed-sm')
                  DO K = 1,2
                     SELECT CASE ( Var_type(I,K)%S )
                     CASE ('TAU','LWP','LWC','PNC','IWP','IWC','CNC')
                     CASE DEFAULT
                        Call WRITE_MESSAGE (R_NAME, 'ERROR',' ')
                        WRITE (EU,*)
     $                  '   Type of input integral parameter not found'
                        WRITE (EU,*) 'Selected parameter: ', 
     $                       Var_type(I,K)%S
                        WRITE (EU,111) I
                        RETURN 1               
                     END SELECT
                  ENDDO
c                  IF (TRIM(RTM_TYPE) == 'spher_scat') THEN
c                     Call WRITE_MESSAGE (R_NAME, 'ERROR',
c     $                    'Mixed cloud are not allowed'
c     $                    //' for "spher_scat" RTM type')
c                     RETURN 1               
c                  ENDIF
               CASE DEFAULT
                  Call WRITE_MESSAGE (R_NAME, 'ERROR',
     $                 'Thermodynamic cloud state not found')
                  Write (EU,111) i
                  RETURN 1               
            END SELECT

c           ... Delta-M truncation
c           -----------------------
            IF ( CLD_DM(I) > NLEGEN )  Then
               Call WRITE_MESSAGE (R_NAME, 'ERROR',           
     $              'Cloud Delta-M truncation out of range')        
               RETURN 1                                             
            End If

c     (v) Are the cloud boundaries out of range?
c     ------------------------------------------
            BASE =  CLD_LAYER_HEIGHTS(I,1)
            TOP  =  CLD_LAYER_HEIGHTS(I,2)
            IF( BASE .GE. TOP ) THEN
               CALL WRITE_MESSAGE (R_NAME, 'ERROR',           
     $              'Conflicting cloud boundaries')                
               WRITE (EU,111) I
               WRITE (EU,'(A,1x,f8.5)') '   cloud base :', BASE
               WRITE (EU,'(A,1x,f8.5)') '   cloud top  :', TOP
               RETURN 1         
            END IF
            IF( BASE < GROUND_HEIGHT ) THEN 
               CALL WRITE_MESSAGE (R_NAME, 'ERROR',          
     $              'Cloud bottom below ground surface')             
               WRITE (EU,111) I
               RETURN  1                                             
            END IF

c     (vi) Overlapping cloud layers?
c     ------------------------------
            Do J = I+1, N_CLD_LAYERS
               BBASE =  CLD_LAYER_HEIGHTS(J,1)
               TTOP  =  CLD_LAYER_HEIGHTS(J,2)

               If ( ( (TOP  .LT. TTOP) .AND. (TOP  .GT. BBASE) ) .or.
     $              ( (BASE .LT. TTOP) .AND. (BASE .GT. BBASE) ) ) Then
                  Call WRITE_MESSAGE (R_NAME, 'ERROR',       
     $                 'Overlapping cloud layers')                 
                  Write (EU,111) j
                  RETURN  1                                       
               End If
            End Do

c     (vii) Cloud particle profiles                                  
c     ------------------------------                                  
c     - nothing should be negative                                    
c     - we should have at least two values, otherwise we can't        
c     > determine profile                                             
c     - altitude values for particle profiles should be within cloud  
c     > range                                                         
c     - if profile altitudes do not cover the complete cloud range,   
c     > extent to cloud regime                                        
            If(Allocated(CLDPROFILE)) Then            
               DO J = 1, N_CLDPROF(I)                                  
                  IF ( CLDPROFILE(I,J,1) .LT. 0.0D0 .OR.               
     $                 CLDPROFILE(I,J,2) .LT. 0.0D0      ) THEN        
                     Call WRITE_MESSAGE (R_NAME, 'ERROR',        
     $                    'Cloud particle profiles out of range')          
                     WRITE (*,111) I                                   
                     RETURN 1                                         
                  END IF                                               
               END DO                                                  
               IF ( N_CLDPROF(I) .EQ. 1 ) THEN  
                  N_CLDPROF(I) = 2                       
                  CLDPROFILE(I,1,1)            = TOP
                  CLDPROFILE(I,N_CLDPROF(I),1) = BASE
                  CLDPROFILE(I,N_CLDPROF(I),2) = CLDPROFILE(I,1,2)
                  Call WRITE_MESSAGE (R_NAME, 'WARNING',
     $                 'Cloud particle profile should contain at least') 
                  Call WRITE_MESSAGE (R_NAME, 'WARNING',
     $                 'two entries. Adjusting appropriately.')  
                  WRITE (*,112) I
               END IF
            Endif
            
         END DO
         
      END IF

c      WRITE(*,*) '  -4 '
      
 111  Format ('    Error occured for cloud layer # ', i2 )
 112  Format ('    Warning occured for cloud layer # ', i2 )          

      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

 200  Call WRITE_MESSAGE (R_NAME, 'ERROR',
     $     'For user defined phase function the vertically')
      WRITE(EU,*) 
     $     '   inhomogeneous cloud can be calculated setting '
      WRITE(EU,*) 
     $     '   "Scattering and absorption coefficients" to '//
     $     '"analytically" only.'
      WRITE(EU,*) 
     $     '   Check control file "cloud.ibp". '
      RETURN 1               

c 201  Call WRITE_MESSAGE (R_NAME, 'ERROR',
c     $     'Weighting functions for cloud parameters'
c     $     //' in the case of multilayered mixed clouds')
c      WRITE(EU,*) 
c     $     '   can be calculated only if "Thermodynamic state"'
c      WRITE(EU,*) 
c     $     '   is set to "mixed" for all cloud layers'
c      RETURN 1 

 202  Call WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
      Write (EU,*) 
     $     '    Weighting function w.r.t. effective radius'
     $     //' for user defined phase function '
      Write (EU,*) 
     $     '    requires that parameter '  
      Write (EU,*) 
     $     '    "Scattering and absorption coefficients"'//
     $     ' should be set to "analytically".'
      Write (EU,*) 
     $     '    Check control file "cf_cloud.inp".'
      RETURN 1

      END
C********************************************************************C
