!***************************************************************************
!
!       SCIATRAN - Version 4.7 - Last modification: 08 Dec 2023
!
!       Institute of Remote Sensing, University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE GT_CLOUD_INPUT(DO_GEOLOCATION, *)
  ! Called by "gt_fileread.f90"

  USE ALT_INPUT, ONLY: ALIN_FLAG, ALIN_ACT, ALIN_Z, ALIN_NZ
  USE CLOUDS,    ONLY: TAU_CLD_INP, CLD_DM, CLD_LAYER_HEIGHTS, J_BOT, J_TOP, &
       CLDPROFILE, N_CLDPROF, LWP, IWP, CLD_LAYER_NUM, &
       TRANS_LAY, GT_TL, Red_WL, N_Red_WL, Red_WL_Ydb,  &
       DO_CLOUD, Var_type, NCL => N_CLD_LAYERS, SCALE_PROF
  USE CLD_INP,   ONLY: CLOUD_DATA_BASE_PATH, D_MF, LC_CLD, &
                       Nf_ice, N_ref_ice, Ref_ice, &
                       USER_EK_NAME, HOW_CLD_COEF, CLD_INHOM, TDS, Plus_ice, &
                       PHASE_NAME, DBC_NAME, SUB_LAYERING, USE_PSD, Inp_form, &
                       MAN_AUTO_CLD, UC_CLD, Phm_const, CON_VAL_CLD, OUT_PH_CLD,&
                       Path_PH_CLD, Refr_water_inp, Refr_ice_inp, ADD_ABS,&
                       N_ref_db, Nphr, Deriv_Phc, Use_Bdb, N_ref_Bdb, Nr_db, &
                       Use_Ydb, Use_Ydb_SCM, N_ref_Ydb, S_name_Ydb, N_Ydb, &
                       MLCS, Fd, CRST, Ref_Bdb, Ref_db, Ref_Ydb, Use_Dim, &
                       Dim_Ydb, Add_Abs_mod, N_Par_mod, N_ref_sm
  USE CLD_RET,   ONLY: Ref_wi
  USE INFO,      ONLY: CLD_INFO
  USE LOG_PARS,  ONLY: EU, Work_Dir
  USE GT_POLYN,  ONLY: NLEGEN
  USE SLAB_2L,   ONLY: TES_2L_SLAB
  USE ST_VARS,   ONLY: GROUND_HEIGHT, RTM_CORE, VHL
  USE TOOLS,     ONLY: Read_Input, SET_PATH, UPPER_LOWER_CASE, READ_FILE,  &
                       SPLIT_STRING, CUT_STR, RemoveChars, CutChars
  USE TYPES,     ONLY: KND, DYN_CHAR

  LOGICAL, INTENT(IN) :: DO_GEOLOCATION
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'GT_CLOUD_INPUT'
  INTEGER    KK, MAX_CLDPROF, I_WF, I, K, J, INX, K_TR, II, Num_FORM, Nl_ice, &
       Ifo, Kint, Kcom, JJ, Ig, NMC, Nrb, I1, I2, KJ, Nwl, Nalt
  INTEGER, PARAMETER :: CLDUNIT = 99, MAX_LAY = 100, N_shape = 13, Ns_Y = 9
  CHARACTER (LEN=1)    DUMCHAR  
  CHARACTER (LEN=*), PARAMETER :: START  = '>' , FMT_LOG1 = '(L1)'
  Character (Len=:), Allocatable :: All_Young, MD_Baum,  MD_Frac, MD_Hexa, FNAME,&
       STR_INP, S_CHAR, SCM_CH, D_range, Init_prof
  CHARACTER (LEN = :), ALLOCATABLE :: Str
  Type(Dyn_Char), Allocatable :: MD_Crys(:), Long_name(:), Short_name(:)
  Logical  SkipFlg, F_water
  LOGICAL, ALLOCATABLE ::  Sel_Range(:)
  REAL (KND)  Re1, Re2, Re3, Re4, DEL, DUM, Kp
  REAL (KND), ALLOCATABLE :: CLD_PRO_LOC(:,:,:), CLD_PRO_TMP(:,:,:), &
       Dmin(:), Dmax(:), DATA_R(:,:)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:), STR_ARR_1(:),&
       HEL(:,:)
  Type Integer_Arr
     Integer, Allocatable:: NN(:)
  End Type Integer_Arr
  Type(Integer_Arr), Allocatable :: Nsel_form(:)
  Type Linked_List_Entry
     Integer NN
     Character (Len = :), Allocatable :: Line
     Type (Linked_List_Entry), Pointer :: NextElem
  End Type Linked_List_Entry
  Type (Linked_List_Entry), Pointer :: FirstElem, Element => Null()
  CHARACTER (LEN=4) INT_TO_CHAR
  EXTERNAL     INT_TO_CHAR
  !==================================================================
  ! Read general control flag and return if no clouds are present
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
  ! conditionally deallocated here

  ALLOCATE( Ref_wi(2) ) ; Ref_wi(:) = .FALSE.

  N_Par_mod = 1  ! Preliminary setting for DOM_BK model only !

  IF( ALIN_ACT(4) ) THEN
     DO_CLOUD   = .TRUE.
     Red_WL_Ydb = .FALSE.
  ELSE
     IF( VHL ) THEN
        DO_CLOUD = .FALSE.
     ELSE
        STR_INP = 'Clouds present?'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S, *, ERR = 101) DO_CLOUD
     END IF
  ENDIF

  IF( TES_2L_SLAB ) DO_CLOUD = .FALSE.

  IF ( .NOT. DO_CLOUD ) THEN
     ALLOCATE( Plus_ice(1), ADD_ABS(1) )
     Plus_ice(:) = .FALSE. ;  ADD_ABS(:) = .FALSE.
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  ELSE
     IF(TRIM(RTM_CORE) == 'CDI')  GOTO 210           
  END IF

  ALLOCATE( Long_name(N_shape),Short_name(N_shape),MD_Crys(N_shape) ) 
  All_Young = 'Crystals/Yang_DB/'
  MD_Baum   = 'Crystals/Baum_DB/'
  MD_Frac   = 'Crystals/Fractal/'
  MD_Hexa   = 'Crystals/Hexagons/'
  Long_name(1)%S = 'column_8elements'       ;  Short_name(1)%S = 'col'
  Long_name(2)%S = 'droxtal'                ;  Short_name(2)%S = 'dro'
  Long_name(3)%S = 'hollow_column'          ;  Short_name(3)%S = 'hoc'
  Long_name(4)%S = 'hollow_bullet_rosette'  ;  Short_name(4)%S = 'hbr'
  Long_name(5)%S = 'plate'                  ;  Short_name(5)%S = 'pla'
  Long_name(6)%S = 'plate_10elements'       ;  Short_name(6)%S = 'p10'
  Long_name(7)%S = 'plate_5elements'        ;  Short_name(7)%S = 'p5e'
  Long_name(8)%S = 'solid_bullet_rosette'   ;  Short_name(8)%S = 'sbr'
  Long_name(9)%S = 'solid_column'           ;  Short_name(9)%S = 'soc'
  Long_name(10)%S = 'Hexagon'               ;  Short_name(10)%S = 'hex'
  Long_name(11)%S = 'Fractal'               ;  Short_name(11)%S = 'fra'
  Long_name(12)%S = 'AggregateSolidColumns' ;  Short_name(12)%S = 'agg'
  Long_name(13)%S = 'Stochastic_model'      ;  Short_name(13)%S = 'smo'

  DO I = 1, Ns_Y
     MD_Crys(I)%S = All_Young//Long_name(I)%S
  ENDDO
  MD_Crys(10)%S = MD_Hexa 
  MD_Crys(11)%S = MD_Frac 
  MD_Crys(12)%S = MD_Baum
  MD_Crys(13)%S = 'Not used'


  ! Read cloud parameters
  ! =====================
  STR_INP = 'Path to cloud data bases'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  !  WRITE(*,*) ' READ_ARR(1)%S  ', TRIM(READ_ARR(1)%S)
  !  WRITE(*,*) '         ==============================   '

!  CLOUD_DATA_BASE_PATH = SET_PATH(READ_ARR(1)%S)

    CLOUD_DATA_BASE_PATH = READ_ARR(1)%S
!    WRITE(*,*) '    CLOUD_DATA_BASE_PATH  ', TRIM(CLOUD_DATA_BASE_PATH)

  STR_INP = 'Refractive index of water and ice'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  Refr_water_inp = CUT_STR(STR_ARR(1)%S)
  Refr_ice_inp = CUT_STR(STR_ARR(2)%S)

  If(DO_GEOLOCATION .OR. ALIN_ACT(4)) Then
     NCL = 1
  Else
     STR_INP = 'Cloud layer information'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     NCL = Size(READ_ARR)
  End If

  ALLOCATE( MLCS(NCL), Dmin(NCL), Dmax(NCL), Sel_Range(NCL), Inp_form(NCL), &
       N_CLDPROF(NCL), J_TOP(NCL),  CLD_DM(NCL), J_BOT(NCL), &! A
       CLD_LAYER_HEIGHTS(NCL,2), TAU_CLD_INP(NCL,2), &! A
       Nf_ice(NCL), N_ref_ice(NCL), Add_Abs_mod(NCL),  &
       DBC_NAME(NCL,2), Plus_ice(NCL), SUB_LAYERING(NCL), Var_type(NCL,2), &! A
       CLD_LAYER_NUM(NCL), USER_EK_NAME(NCL,2), IWP(NCL), LWP(NCL), &! A
       CLD_PRO_LOC(NCL,MAX_LAY,7), ADD_ABS(NCL), TDS(NCL), HEL(NCL,2), &! A
       CLD_PRO_TMP(NCL,MAX_LAY,7), PHASE_NAME(NCL) ) ! A
  Sel_Range(:) = .FALSE.

  FORALL(II = 1:NCL,KK = 1:2) DBC_NAME(II,KK)%S     = 'not used   '
  FORALL(II = 1:NCL,KK = 1:2) USER_EK_NAME(II,KK)%S = 'not used   '

  MLCS(:)%Phm_co = .TRUE.

  ! Altitudes of cloud top and base for all cloud layers
  ! =========================================================
  ! Cloud layers have to be non-overlapping. 
  IF( ALIN_ACT(4) ) THEN
     !         CLD_LAYER_HEIGHTS(1,1) = ALIN_Z(ALIN_NZ)
     !         CLD_LAYER_HEIGHTS(1,2) = ALIN_Z(1)
     CLD_LAYER_HEIGHTS(1,1) = 2.d0
     CLD_LAYER_HEIGHTS(1,2) = 3.d0
     !         CLD_LAYER_NUM(1) = 60
     CLD_LAYER_NUM(1) = 1  ; SUB_LAYERING(1)%S  = 'equidistant'
     TDS(1)%S    = 'water' ; CLD_INHOM   = 'no'
     HEL(1,1)%S  = 'TAU'   ; HEL(1,2)%S  = 'TAU'
     PHM_const   = .TRUE.  ; Plus_ice(:) = .FALSE.
     Nphr = 1 
  ELSE
!    ... After reading  the 'Cloud layer information' control field
     Do I = 1, NCL
        Call Split_String(READ_ARR(I)%S, STR_ARR)
        TDS(I)%S = CUT_STR(STR_ARR(1)%S)
        SELECT CASE ( TDS(I)%S )
        CASE ('water','ice','mixed','mixed-sm','ice-sm','alay')
        CASE DEFAULT
           GOTO 238
        END SELECT
        Nf_ice(:) = 0
        READ(STR_ARR(2)%S,*,ERR=101) CLD_LAYER_HEIGHTS(I,1)
        READ(STR_ARR(3)%S,*,ERR=101) CLD_LAYER_HEIGHTS(I,2)
        READ(STR_ARR(4)%S,*,ERR=101) CLD_LAYER_NUM(I)
        SUB_LAYERING(I)%S = CUT_STR(STR_ARR(5)%S)
        SELECT CASE ( SUB_LAYERING(I)%S )
        CASE('equidistant','logarithmic')
        CASE DEFAULT
           GOTO 213
        END SELECT
     End Do  ! over cloud layers (I)

     !    ... Geometrical thickness of transition layer between cloud
     !        and cloud-free medium
     ! =========================================================
     STR_INP = 'Thickness of transition layer'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL UPPER_LOWER_CASE (READ_ARR(1)%S  ,'U2L')
     SELECT CASE( READ_ARR(1)%S )
     CASE ('dynamic')
        TRANS_LAY = 'Dynamic'
     CASE ('default')
        TRANS_LAY = 'Default'
     CASE DEFAULT
        TRANS_LAY = 'User_def'
        READ(READ_ARR(1)%S,*,ERR=101) GT_TL 
     END SELECT

     Plus_ice(:) = .FALSE.
     IF( TDS(1)%S == 'alay' .AND. NCL > 1 ) GOTO 223

!    ... Plus_ice(:) is .true. for single mixed (ice+water) cloud layer
!    ------------------------------------------------------------------
     F_water = .FALSE.
     DO I = 1, Ncl
        IF( TDS(I)%S=='mixed'.OR.TDS(I)%S=='mixed-sm' ) THEN
           Plus_ice(:) = .TRUE. ; EXIT
        ENDIF
        IF( TDS(I)%S=='water' ) F_water = .TRUE.
     ENDDO

!     write(*,*) 'COUNT(Plus_ice) & F_water ', COUNT(Plus_ice), F_water 
     IF( F_water ) THEN
        DO I = 1, Ncl
           SELECT CASE( TDS(I)%S )
           CASE('ice','ice-sm')
              Plus_ice(:) = .TRUE. ;  EXIT
           END SELECT
        ENDDO
     ENDIF
     
     SCM_Ch = 'no' ! Will be used in the case of Yang database
     Allocate(CRST(NCL), Nsel_form(NCL))

     Do II = 1, NCL
        If(TDS(II)%S == "ice-sm".OR.TDS(II)%S == "mixed-sm") THEN
           Num_FORM = 13
           Allocate(CRST(II)%SHP(1), Nsel_form(II)%NN(1))
           CRST(II)%SHP(1)%ICE_FORM = Short_name(Num_FORM)%S
           Nsel_form(II)%NN(1) = Num_FORM
           CRST(II)%SHP(1)%ICE_DIR = MD_Crys(Num_FORM)%S//'/'&
                //Long_name(Num_FORM)%S
        ENDIF
     ENDDO  ! over cloud layers (II)
     
     STR_INP = 'Form of ice crystals'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL UPPER_LOWER_CASE (READ_ARR(1)%S  ,'U2L')
     READ_ARR(1)%S = CUT_STR(READ_ARR(1)%S)
     CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
          
     Select Case (STR_ARR(1)%S)
     Case('yang')
        !        WRITE(*,*)  '   Case - Yang database '
        If (RTM_CORE /= "TRN") Then
           If(Size(STR_ARR) < 2) GoTo 228 ! Checking that SCM_Ch is presented
           SCM_Ch = CUT_STR(STR_ARR(2)%S)
        Else
           SCM_Ch = "no" ! No need for phase function in the transmission mode
        End If
        !                write(*,*) '   SCM_Ch  ', SCM_Ch
        ! Must be an extra loop as the variable with index KK + 1 is used
        ! within the main loop
        Do KK = 2, Size(READ_ARR)
           Call Upper_Lower_Case (READ_ARR(KK)%S  ,'U2L')
        End Do
        II = 0 
        Do KK = 2, Size(READ_ARR)
           READ_ARR(KK)%S = CUT_STR(READ_ARR(KK)%S)
!          ... Check if the substring "--- layer" is present
!           write(*,*) '    ---- Index   ', Index(READ_ARR(KK)%S, "--- layer")
           If( Index(READ_ARR(KK)%S, "--- layer") /= 0 ) Then
              
!             ... Remove all characters from second input line:
!              READ_ARR(KK)%S = RemoveChars(READ_ARR(KK)%S)
!              If( Len_Trim(READ_ARR(KK)%S) == 0 ) GoTo 2374
!!             ... Reading the number of the layer
!              CALL SPLIT_STRING(READ_ARR(KK)%S, STR_ARR)
!              READ(STR_ARR(1)%S,*,Err=101) II

!            ... Check if the substring "--->" is present
              KJ = Index(READ_ARR(KK)%S, "--->")
              If(KJ == 0) GoTo  2374
!             ... Reading the number of the layer
              Str = CutChars( READ_ARR(KK)%S(1:KJ - 1) )
!            ... Check if the number of layer is present
              If( LEN_TRIM(Str) == 0) GoTo  2373
              READ(Str,*,Err=101) II 
              If(II > Size(TDS)) Then ! Input for a non-existing level found
                 SkipFlg = .True. ! Skip reading parameters
                 Cycle
              End If
              READ_ARR(KK)%S = RemoveChars(READ_ARR(KK)%S)
              CALL SPLIT_STRING(READ_ARR(KK)%S, STR_ARR)
!              write(*,*) '     Layer number after spliting ', II
!              write(*,*) '     TDS(II)%S  ', TDS(II)%S
              Select Case (TDS(II)%S)
              Case('ice', 'mixed') ! Current cloud layer is 'ice' or 'mixed'
                 SkipFlg = .False.
!                ... Reading grid of maximal dimension
                 If( Size(STR_ARR) == 1 ) Then
                    Kint = 2
                    Allocate(CRST(II)%DGR(Kint))
                    CRST(II)%DGR(1) = 2.0D0 ;  CRST(II)%DGR(2) = 1.0D+4
                 Else
                    Kint = Size(STR_ARR) - 1
                     Allocate( CRST(II)%DGR(Kint) )
                    Do JJ = 1, Kint
                       READ(STR_ARR(JJ + 1)%S,*,Err=101) CRST(II)%DGR(JJ)
                    End Do
                 End If
              Case Default  ! Current cloud layer is not 'ice' or 'mixed'
                 SkipFlg = .True. ! Skip reading parameters
              End Select
              Cycle
           End If
           If(II == 0) GoTo 2371
!          ... Skip possible input if it is not an ice or mixed cloud
           If(SkipFlg) Cycle 
!          ... Reading habit mixture data
           Call AddLine (READ_ARR(KK)%S)
           If(KK /= Size(READ_ARR)) Then
              If(Index(READ_ARR(KK + 1)%S, "--- layer") == 0) Cycle
           End If
           Call ReadData (*202, *101)
        End Do  ! over number of inpt lines except of first (KK)
!        do II = 1, NCL
!           if(allocated( CRST(II)%DGR)) then
!              write(6,*) II, CRST(II)%DGR
!              do jj = 1, size(CRST(II)%SHP)
!                 write(6,*) JJ
!                 write(6,*) CRST(II)%SHP(JJ)%ICE_FORM
!                 write(6,*) CRST(II)%SHP(JJ)%ICE_DIR
!                 write(6,*) CRST(II)%SHP(JJ)%P_ROU
!                 Do Ig = 1, Kint
!                    write(6,*) CRST(II)%SHP(JJ)%F_HAB(Ig)
!                 End Do  ! over grid points (Ig)
!              end do
!           end if
!        end do
     Case ('baum','hexagon','fractal')
        !  WRITE(*,*) '   2: STR_ARR(1)%S ', STR_ARR(1)%S
        IF( STR_ARR(1)%S=='hexagon' ) Num_FORM = 10
        IF( STR_ARR(1)%S=='fractal' ) Num_FORM = 11
        IF( STR_ARR(1)%S=='baum' )    Num_FORM = 12
        Do II = 1, NCL
           If(TDS(II)%S == "ice-sm".OR.TDS(II)%S == "mixed-sm") Cycle
           Allocate(CRST(II)%SHP(1), Nsel_form(II)%NN(1))
           CRST(II)%SHP(1)%ICE_FORM = Short_name(Num_FORM)%S
           CRST(II)%SHP(1)%ICE_DIR  = MD_Crys(Num_FORM)%S//'/'//&
                Long_name(Num_FORM)%S
           Nsel_form(II)%NN(1) = Num_FORM
        ENDDO  ! over cloud layers (II)
        !  WRITE(*,*) '    End of CRST definition'
     Case Default
        CALL NOT_FOUND (R_NAME, STR_INP, STR_ARR(1)%S, *880)     
     End Select
     !===================================================================    
!     PAUSE

     !   -----------------------------------------------------------------     
     !    STOP '     Stop after 102'
     !     WRITE(*,*) '    Use scattering matrix from DB = ', SCM_Ch
     Use_Bdb = .FALSE.    ; Use_Ydb = .FALSE. ; Use_Ydb_SCM = .FALSE.
     Red_WL_Ydb = .FALSE.
!    ... Check if the "Form of ice crystals" input parameter 
!        is properly set for "ice" or "mixed" cloud
     DO I = 1,NCL
        IF( TDS(I)%S=='water'.OR.TDS(I)%S=='ice-sm'.OR.TDS(I)%S=='mixed-sm' ) CYCLE
        SELECT CASE( CRST(I)%SHP(1)%ICE_FORM)
        CASE('hex','fra')
        CASE('agg')
           Use_Bdb = .TRUE.
        CASE('col','dro','hoc','hbr','pla','p10','p5e','sbr','soc')
           Use_Ydb = .TRUE.
           SELECT CASE( SCM_Ch )
           CASE('yes')  ;  Use_Ydb_SCM = .TRUE.
           CASE('no')   ;  Use_Ydb_SCM = .FALSE.
           CASE DEFAULT
              GOTO 224
           END SELECT
        CASE DEFAULT
           GOTO 220
        END SELECT
     ENDDO  ! over cloud layers (I)

     !    Set vertical profiles of particle parameters within clouds
     !    ==========================================================
     Init_prof = 'file'

     STR_INP = 'Cloud layers are vertically inhomogeneous'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
     CLD_INHOM  = CUT_STR(STR_ARR(1)%S)
     IF( SIZE(STR_ARR) > 1 ) THEN
        Init_prof = CUT_STR(STR_ARR(2)%S)
        SELECT CASE ( Init_prof)
        CASE ('file','const')
        CASE DEFAULT
           CALL NOT_FOUND (R_NAME, STR_INP, CLD_INHOM, *880)
        END SELECT
     ENDIF

     SELECT CASE( CLD_INHOM )
     CASE('yes')
        !  The following input option will be used if second entry of
        !    'Cloud layers are vertically inhomogeneous' control parameter
        !     is set to 'const'
        CLD_PRO_TMP = 1.d0 ;  CLD_PRO_TMP(:,1,1) = 0.d0
        STR_INP = 'Size of cloud particles'
        CALL Read_Input (STR_INP, READ_ARR, *880)
!       ... New version of input
       DO K = 1, NCL
          CLD_PRO_TMP(K,1:2,6) = 100.d0
          CALL SPLIT_STRING(READ_ARR(K)%S, STR_ARR)
          READ(STR_ARR(1)%S, *, ERR = 101) Re1
          IF( SIZE(STR_ARR) == 1 ) THEN
             DO KK = 3,7,2 ; CLD_PRO_TMP(K,1:2,KK) = Re1 ; ENDDO
          ELSE
             READ(STR_ARR(2)%S, *, ERR = 101) Re2
             CLD_PRO_TMP(K,1:2,3) = Re1
             DO KK = 5,7,2 ; CLD_PRO_TMP(K,1:2,KK) = Re2 ; ENDDO
          ENDIF
!          write(*,'(1x,I2,*(2x,F6.1))') K, (CLD_PRO_TMP(K,1,KK), KK=3,7,2)
       ENDDO ! over cloud layers (K)        
!!       ... Old version of input
!        DO K = 1, NCL                                 
!           READ(READ_ARR(K)%S, *, ERR = 101)  Re1, Re2, Re3, Re4
!           CLD_PRO_TMP(K,1:2,3) = Re1
!           CLD_PRO_TMP(K,1:2,5) = Re2
!           CLD_PRO_TMP(K,1:2,6) = Re3
!           CLD_PRO_TMP(K,1:2,7) = Re4
!           write(*,'(1x,I2,*(2x,F6.1))') K, (CLD_PRO_TMP(K,1,KK), KK=3,7,2)
!        ENDDO
!        stop
        ! -------------------------------------------------------------------
        ! Cloud parameters vertical profile ( Provided by user )
        STR_INP = 'Cloud particle profile input file'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        DO K = 1, NCL
           FNAME = SET_PATH(READ_ARR(K)%S)
           ! Read LOCAL cloud particle shape profile:
           ! ----------------------------------------
           !   WRITE(*,*) ' Ncl = ', K, FNAME
           OPEN ( CLDUNIT, FILE = FNAME, STATUS = 'OLD' )  
           DO   ! Read until "start of data" character is encountered
              READ (CLDUNIT,'(A1)') DUMCHAR                    
              IF ( DUMCHAR .EQ. START ) EXIT
           END DO
           KK = 0                                 
           DO   ! Read data                       
              KK = KK + 1               
              READ( CLDUNIT, *, ERR = 211, END = 222 )  &    
                   ( CLD_PRO_LOC(K,KK,I), I=1,7 )
              !              WRITE(*,'(1x,I3,*(1x,e12.4))') KK, CLD_PRO_LOC(K,KK,:)
              IF(KK == MAX_LAY) GOTO 232
           END DO
222        KK = KK - 1  
!           WRITE(*,*) '  KK = ', KK
           CLOSE( CLDUNIT )
           IF(KK.LE.1) GOTO 212                
           DEL = CLD_PRO_LOC(K,2,1) - CLD_PRO_LOC(K,1,1)
           IF( DEL <= 0.d0 ) GOTO 214
           N_CLDPROF(K) = KK       
        ENDDO  ! over cloud layers (K=1, NCL)
        ! ... Replace with constant dimension from input option
        !     'Size of cloud particles'    
        IF( Init_prof=='const' ) THEN
           DO K = 1, NCL
              KK = N_CLDPROF(K)
              CLD_PRO_LOC(K,1:KK,3) = CLD_PRO_TMP(K,1,3)
              CLD_PRO_LOC(K,1:KK,5) = CLD_PRO_TMP(K,1,5)
              CLD_PRO_LOC(K,1:KK,7) = CLD_PRO_TMP(K,1,7)
           ENDDO
        ENDIF
        MAX_CLDPROF = MAXVAL(N_CLDPROF(:))
     CASE('no')
        MAX_CLDPROF = 2    ; N_CLDPROF(:) = MAX_CLDPROF
        CLD_PRO_LOC = 1.d0 ; CLD_PRO_LOC(:,1,1) = 0.d0
        STR_INP = 'Size of cloud particles'
        CALL Read_Input (STR_INP, READ_ARR, *880)
!       ... New version of input
       DO K = 1, NCL
          CLD_PRO_LOC(K,1:2,6) = 100.d0
          CALL SPLIT_STRING(READ_ARR(K)%S, STR_ARR)
          READ(STR_ARR(1)%S, *, ERR = 101) Re1
          IF( SIZE(STR_ARR) == 1 ) THEN
             DO KK = 3,7,2 ; CLD_PRO_LOC(K,1:2,KK) = Re1 ; ENDDO
          ELSE
             READ(STR_ARR(2)%S, *, ERR = 101) Re2
             CLD_PRO_LOC(K,1:2,3) = Re1
             DO KK = 5,7,2 ; CLD_PRO_LOC(K,1:2,KK) = Re2 ; ENDDO
          ENDIF
!          write(*,*) '     Check of sizing '
!          write(*,'(1x,I2,*(2x,F6.1))') K, (CLD_PRO_LOC(K,1,KK), KK=3,7,2)
       ENDDO ! over cloud layers (K) 
!        DO K = 1, NCL                                 
!           READ(READ_ARR(K)%S, *, ERR = 101)  Re1, Re2, Re3, Re4
!           CLD_PRO_LOC(K,1:2,3) = Re1
!           CLD_PRO_LOC(K,1:2,5) = Re2
!           CLD_PRO_LOC(K,1:2,6) = Re3
!           CLD_PRO_LOC(K,1:2,7) = Re4
!        ENDDO        
     CASE DEFAULT
        GOTO 208
     END SELECT
     !    ... Save vertical profiles of cloud parameters 
     ALLOCATE( CLDPROFILE(NCL,MAX_CLDPROF,7) )
     CLDPROFILE(:,:,:) = CLD_PRO_LOC(:,1:MAX_CLDPROF,:)

     !    ... This setting in the case if the scattering matrix is 
     !        independent of effective radius
     Phm_const = .TRUE. ;  Nphr = 1 ; Nr_db(:) = 1

     !    ... Does scattering matrix depend on the effective radius? 
     STR_INP = 'Use constant phase function within cloud layer'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
     IF( Size(STR_ARR) /= Ncl ) GOTO 240
     DO K = 1, NCL
        READ(STR_ARR(K)%S, *, ERR = 101) MLCS(K)%Phm_co
        IF( TDS(K)%S=='water' ) CYCLE
        IF( .NOT.MLCS(K)%Phm_co ) THEN
           SELECT CASE( CRST(K)%SHP(1)%ICE_FORM )
           CASE('hex','fra')
              GOTO 241
           END SELECT
        ENDIF
     ENDDO
     !     DO K = 1, NCL
     !        IF( TDS(K)%S=='ice-sm'.AND.(.NOT.MLCS(K)%Phm_co) ) GOTO 236
     !     ENDDO
     IF( ANY( .NOT.MLCS(:)%Phm_co ).AND.CLD_INHOM=='no' ) GOTO 239
     ! =========================================================================

     USE_PSD = .FALSE.
     !    ... Read number of effective radii, wavelength, and
     !        cross-sections for referent wavelength (Yang DB)
     !     WRITE(*,*) '    Use_Ydb =  ', Use_Ydb

     IF( Use_Ydb ) THEN        
        STR_INP = 'Reduced wavelength grid'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
        READ(STR_ARR(1)%S, *, ERR = 101) Red_WL_Ydb
        IF( Red_WL_Ydb ) THEN
           N_Red_WL = SIZE( STR_ARR ) - 1
           !           WRITE(*,*) '  N_Red_WL  ', N_Red_WL
           ALLOCATE( Red_WL(N_Red_WL) )
           DO I = 1, N_Red_WL
              READ(STR_ARR(I+1)%S,*,ERR=101) Red_WL(I)
              !              write(*,*) I, Red_WL(I)
           ENDDO
        ENDIF
        !        pause
        STR_INP = 'Use PSD for ice crystals'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        READ(STR_ARR(1)%S, *, ERR = 101) USE_PSD
        READ(STR_ARR(2)%S, *, ERR = 101) Kp ! Shape parameters of G-distribution

        !        WRITE(*,*) '  Phm_const, CLD_INHOM, USE_PSD = ', &
        !             Phm_const, CLD_INHOM, USE_PSD
        ! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        !       ... New version with MLCS(:)%Phm_co
        STR_INP = 'Ice crystal dimension range'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        IF( SIZE(READ_ARR) < NCL ) GOTO 226
        DO I = 1, NCL
          IF( MLCS(I)%Phm_co.AND.USE_PSD )  GOTO 242
!          ... Scattering matrix is dependent on effective radius
          CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
          D_range = CUT_STR(STR_ARR(1)%S)
!          WRITE(*,*) '    D_range  ', D_range 
          SELECT CASE( D_range )
          CASE('auto')
             IF( USE_PSD ) GOTO 235 
             KK = N_CLDPROF(I)
!               WRITE(*,*) '   Layer is ', I, KK
!               WRITE(*,*) CLDPROFILE(I,1:KK,7)
             Dmax(I) = MAXVAL( CLDPROFILE(I,1:KK,7) )
             Dmin(I) = MINVAL( CLDPROFILE(I,1:KK,7) )
             Sel_Range(I) = .TRUE.
          CASE('on')
             IF( USE_PSD ) GOTO 235 
             READ(STR_ARR(2)%S,*,ERR=101)  Dmin(I)
             READ(STR_ARR(3)%S,*,ERR=101)  Dmax(I)
             Sel_Range(I) = .TRUE.
          CASE('off')
             Sel_Range(I) = .FALSE.
          CASE DEFAULT
             CALL NOT_FOUND (R_NAME, STR_INP, D_range, *880)
          END SELECT
!             IF( Sel_Range(I) ) THEN
!                WRITE(*,*)  ' Selected dimension range in cloud layer ', I 
!                WRITE(*,*) '   Inp  Dmin, Dmax: ' , Dmin(I), Dmax(I)
!             ENDIF
       ENDDO  ! over cloud layers (I)
        ! +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        CALL READ_YDB_DIM(*880)
     ENDIF  ! Use_Ydb

     ! The filename(s) in cloud database
     !  ================================
     STR_INP = 'Filenames in database'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     DO I = 1, NCL
        IF( MLCS(I)%Phm_co ) THEN
!          ... Phase function independent of effective radius
           CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR )
           SELECT CASE ( TDS(I)%S )
           CASE ('water')
              DBC_NAME(I,1)%S = CUT_STR(STR_ARR(1)%S)
           CASE ('ice') 
!              DBC_NAME(I,2)%S = CUT_STR(STR_ARR(2)%S)
              DBC_NAME(I,2)%S = CUT_STR(STR_ARR(1)%S)
           CASE ('mixed','mixed-sm') 
              DBC_NAME(I,1)%S = CUT_STR(STR_ARR(1)%S)
              DBC_NAME(I,2)%S = CUT_STR(STR_ARR(2)%S)
           END SELECT
        ELSE
           DBC_NAME(I,1)%S = 'all'
           DBC_NAME(I,2)%S = 'all'
        ENDIF
        !write(*,*) '    Check filenames in DB layer - ', I
        !write(*,*) '    Water - ', DBC_NAME(I,1)%S 
        !write(*,*) '    Ice   - ', DBC_NAME(I,2)%S 
     ENDDO

     !stop
     
     ! Type of input integral parameter
     ! ================================
     TAU_CLD_INP = 0.d0 ;  LWP = 0.d0  ;  IWP = 0.d0
     DO I = 1, NCL 
        HEL(I,1)%S = 'WATER_PATH' ; HEL(I,2)%S = 'WATER_PATH'
     ENDDO
     STR_INP = 'Type of input integral parameter'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     DO I = 1, NCL 
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR )
!        HEL(I,1)%S = CUT_STR(STR_ARR(1)%S)
!        HEL(I,2)%S = CUT_STR(STR_ARR(2)%S)
        SELECT CASE( TDS(I)%S )
        CASE('mixed','mixed-sm')
           HEL(I,1)%S = CUT_STR(STR_ARR(1)%S)
           II = 3
           SELECT CASE( HEL(I,1)%S )
           CASE('TAU')
              READ(STR_ARR(2)%S,*,ERR=101) TAU_CLD_INP(I,1)
           CASE('WATER_PATH')
              READ(STR_ARR(2)%S,*,ERR=101) LWP(I) 
           CASE('LWC_PROFILE','DENS_PROFILE') ; II = 2
           END SELECT
!          ... Reading ice fraction
           HEL(I,2)%S = CUT_STR(STR_ARR(II)%S)
           SELECT CASE( HEL(I,2)%S )
           CASE('TAU')
              READ(STR_ARR(II+1)%S,*,ERR=101) TAU_CLD_INP(I,2)
           CASE('WATER_PATH')
              READ(STR_ARR(II+1)%S,*,ERR=101) IWP(I) 
           CASE('IWC_PROFILE') ; II = 2 
           END SELECT
!           WRITE(*,*) I, CUT_STR(STR_ARR(1)%S)
!           WRITE(*,*) I, CUT_STR(STR_ARR(II)%S)
        CASE('water')
           HEL(I,1)%S = CUT_STR(STR_ARR(1)%S)
           Re1 = 0.d0
           IF( SIZE(STR_ARR) > 1 ) READ(STR_ARR(2)%S,*,ERR=101) Re1
           SELECT CASE( HEL(I,1)%S )
           CASE('TAU') ; TAU_CLD_INP(I,1) = Re1
           CASE('WATER_PATH','LWC_PROFILE') ; LWP(I) = Re1
           END SELECT
!               WRITE(*,*) I, CUT_STR(STR_ARR(1)%S), Re1
        CASE('ice','ice-sm')
           Inx = 1
           IF( Plus_ice(I) ) Inx = 2
           HEL(I,Inx)%S = CUT_STR(STR_ARR(1)%S)
           Re1 = 0.d0
           IF( SIZE(STR_ARR) > 1 ) READ(STR_ARR(2)%S,*,ERR=101) Re1
           SELECT CASE( HEL(I,Inx)%S )
           CASE('TAU') ; TAU_CLD_INP(I,Inx) = Re1
           CASE('WATER_PATH','IWC_PROFILE') ; IWP(I) = Re1
           END SELECT
!               WRITE(*,*) '   "gt_cloud_input": ', HEL(I,Inx)%S
!               WRITE(*,*) I, CUT_STR(STR_ARR(1)%S), TAU_CLD_INP(I,Inx)
        END SELECT
        ! Check the case of aerosol layer
        IF( TDS(I)%S == 'alay' ) THEN
           IF(HEL(I,1)%S /= 'TAU' ) GOTO 221
           READ(STR_ARR(2)%S,*,ERR=101) TAU_CLD_INP(I,1)
        ENDIF
     END DO  ! over cloud layer (I)

     ! Which of vertical profile will be scaled if input integral
     ! parameter is set to 'TAU'
     ! ==========================================================
     STR_INP = 'Will be scaled vertical profile of'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     SCALE_PROF = CUT_STR(READ_ARR(1)%S)
     SELECT CASE (SCALE_PROF)
     CASE('Scale-WC','Scale-NC')
     CASE DEFAULT
        GOTO 215
     END SELECT
  ENDIF

  Do I = 1, NCL                                
     IF( Plus_ice(I) ) THEN
        CALL SET_VAR_TYPE_W( I,1,1 ) ; CALL SET_VAR_TYPE_I( I,2,2 )
        CYCLE
     ENDIF
     SELECT CASE ( TDS(I)%S )
     CASE ('alay')         ;  Var_type(I,1)%S = 'TAU'
     CASE ('water')        ;  CALL SET_VAR_TYPE_W( I,1,1 )
     CASE ('ice','ice-sm') ;  CALL SET_VAR_TYPE_I( I,1,1 )
     END SELECT
  End Do
! ... Check character Var_type
  Do I = 1, NCL
!     WRITE(*,*) '   Plus_ice(I) ', Plus_ice(I)
     IF( Plus_ice(I) ) THEN 
        SELECT CASE( TDS(I)%S )
        CASE('mixed','mixed-sm')
           WRITE(*,'(2x,I2,2(2x,A))') I, Var_type(I,1)%S, Var_type(I,2)%S
        CASE('water')
           WRITE(*,'(2x,I2,*(2x,A))')    I, Var_type(I,1)%S
        CASE('ice','ice-sm')
           WRITE(*,'(2x,I2,*(2x,A))')    I, Var_type(I,2)%S
        END SELECT
     ELSE
        SELECT CASE( TDS(I)%S )
        CASE('mixed','mixed-sm')
           WRITE(*,'(2x,I2,2(2x,A))') I, Var_type(I,1)%S, Var_type(I,2)%S
        CASE('water')
           WRITE(*,'(2x,I2,*(2x,A))')    I, Var_type(I,1)%S
        CASE('ice','ice-sm')
           WRITE(*,'(2x,I2,*(2x,A))')    I, Var_type(I,1)%S
        END SELECT
     ENDIF
  ENDDO
! ... End check character Var_type
!  PAUSE
  
  INX = 1
  IF( ANY(Plus_ice) ) INX = 2
  DO I = 1, NCL
     IF( TDS(I)%S == 'ice-sm'.OR.TDS(I)%S == 'mixed-sm' ) THEN
        IF( Var_type(I,INX)%S == 'CNC' ) GOTO 243
     ENDIF
  ENDDO
  
  ! Cloud layer input integral parameter
  ! ====================================
!  TAU_CLD_INP = 0.d0
!  STR_INP = 'Input integral parameter'
!  CALL Read_Input (STR_INP, READ_ARR, *880)
!  DO I = 1, NCL                                
!     SELECT CASE ( TDS(I)%S )
!     CASE('alay')
!        ! aerosol layer OT
!        READ(READ_ARR(I)%S, *, ERR = 101) TAU_CLD_INP(I,1)
!        LWP(I) = 0.d0  ;   IWP(I) = 0.d0
!     CASE ('water')
!        READ(READ_ARR(I)%S, *, ERR = 101) &
!        !    <-------- water ------->  <-- ice -->
!             TAU_CLD_INP(I,1), LWP(I), Re1, IWP(I) 
!     CASE ('ice','ice-sm')
!        INX = 1
!        IF( ANY(Plus_ice) ) INX = 2
!        READ(READ_ARR(I)%S, *, ERR = 101) &
!       !    <- water ->  <--------- ice --------->
!             Re1, LWP(I), TAU_CLD_INP(I,INX), IWP(I) 
!     CASE ('mixed','mixed-sm')            
!        READ(READ_ARR(I)%S, *, ERR = 101) TAU_CLD_INP(I,1), LWP(I), &
!             TAU_CLD_INP(I,2), IWP(I) 
!     END SELECT
!  END DO  ! over cloud layers (I)

  IF( ALIN_ACT(4) ) THEN
     PHASE_NAME(1)%S = 'user'
  ELSE
     STR_INP = 'Phase function of cloud layers'
     CALL Read_Input (STR_INP, READ_ARR, *880)

     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
     IF( Size(STR_ARR) /= Ncl ) GOTO 240
     DO I = 1, NCL
        PHASE_NAME(I)%S  = CUT_STR( STR_ARR(I)%S )
!        PHASE_NAME(I)%S  = CUT_STR(READ_ARR(I)%S)
     END DO
     
     ! Check Phase function types
     DO I = 1,NCL
        SELECT CASE( PHASE_NAME(I)%S )
        CASE('db','sm','hg','iso','user')
        CASE DEFAULT
           GOTO 209
        END SELECT
     ENDDO
     ! User-defined expansions coefficients and phase-function
     ! =======================================================
     STR_INP = 'Expansion coefficients input file'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     DO I = 1, NCL
        IF( PHASE_NAME(I)%S /= 'user' ) CYCLE
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR )
        SELECT CASE ( TDS(I)%S )
        CASE('alay')
           ! reading files containing wavelength grid and
           ! filenames of expansion coefficients
           USER_EK_NAME(I,1)%S = CUT_STR(STR_ARR(1)%S)
           USER_EK_NAME(I,2)%S = CUT_STR(STR_ARR(2)%S)
        CASE ('water') 
           USER_EK_NAME(I,1)%S = CUT_STR(STR_ARR(1)%S)
        CASE ('ice') 
!           USER_EK_NAME(I,2)%S = CUT_STR(STR_ARR(2)%S)
           USER_EK_NAME(I,2)%S = CUT_STR(STR_ARR(1)%S)
        CASE ('mixed','mixed-sm') 
           USER_EK_NAME(I,1)%S = CUT_STR(STR_ARR(1)%S)
           USER_EK_NAME(I,2)%S = CUT_STR(STR_ARR(2)%S)
        END SELECT
     END DO  ! over cloud layers (I)
  ENDIF

  STR_INP = 'Delta-M or Delta-Fit'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  D_MF = CUT_STR(READ_ARR(1)%S)

  ! Delta-M truncation of cloud phase function
  ! ===========================================
  STR_INP = 'Delta-m truncation for cloud layers'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  IF( Size(STR_ARR) /= Ncl ) GOTO 240
  DO I = 1, NCL
!     READ(READ_ARR(I)%S, *, ERR = 101) CLD_DM(I)
     READ(STR_ARR(I)%S, *, ERR = 101) CLD_DM(I)
  ENDDO
  !  write(*,*) '    -01'
  IF(RTM_CORE /= 'ASYMP') THEN
     DO I = 1,NCL
        IF( CLD_DM(I) >= NLEGEN ) THEN
           K_TR = CLD_DM(I)
           GOTO 219
        ENDIF
     ENDDO
  ENDIF

  IF( CLD_INHOM == 'no' ) THEN
     !     WRITE(*,*) '  CLD_INHOM = ', CLD_INHOM
     DO  I = 1, NCL
        DO K = 1,2
           SELECT CASE( HEL(I,K)%S )
           CASE ('LWC','PNC','IWC','CNC')
              GOTO 206
           CASE DEFAULT
           END SELECT
        ENDDO
     ENDDO  ! over cloud layers (I)
  ENDIF  ! CLD_INHOM == 'no'
  !  write(*,*) '    -02'

  IF( Use_Ydb ) THEN
     DO K = 1, NCL
!        write(*,*) '   Cloud layer and Sel_range(K) ', K, Sel_range(K)
        IF( .NOT.Sel_range(K) ) CYCLE
        KK = N_CLDPROF(K)
        DUM = MAXVAL( CLDPROFILE(K,1:KK,7) )
        IF( DUM > Dmax(K) ) GOTO 204
        DUM = MINVAL( CLDPROFILE(K,1:KK,7) )
        IF( DUM < Dmin(K) ) GOTO 203
     ENDDO  ! over cloud layers (K)
  ENDIF
  !
  ! ======>  Set components of the structure MLCS <======
  !  
  IF( ANY(Plus_ice) ) THEN
     MLCS(:)%N_pha = 2
  ELSE
     MLCS(:)%N_pha = 1
  ENDIF
  DO K = 1, NCL
     ALLOCATE( MLCS(K)%Npf(MLCS(1)%N_pha) )
  ENDDO

  DO K = 1, NCL
     MLCS(K)%Npf(:)%Y_Scm = .FALSE.
  ENDDO
  NMC = COUNT(Plus_ice)
  !  WRITE(*,*) '  =======>   NMC = ', NMC
  !  WRITE(*,*) '     Use_Ydb_SCM = ', Use_Ydb_SCM

  ! ... Scattering matrix will be used for Yang DB.
  !  WRITE(*,*) '   Use_Ydb_SCM ', Use_Ydb_SCM
  IF( Use_Ydb_SCM ) THEN
     !     ... There are clouds having water and ice phase (or single mixed cloud)
!     IF( ANY(Plus_ice) ) KK = 2
!     !     ... There are only ice clouds
!     IF( NMC==0.AND.TDS(1)%S=='ice' ) KK = 1
     KK = 1
     IF( ANY(Plus_ice) ) KK = 2
     DO K = 1, NCL
        MLCS(K)%Npf(KK)%Y_Scm = .TRUE.
     ENDDO
  ENDIF

  ! IF( .NOT.Phm_const ) THEN
  ! ... Phase function depends on the effective radius of ice crystals
  DO K = 1, NCL
!     IF( MLCS(K)%Phm_co )   MLCS(K)%Npf(:)%Ndb_ref = 1
     MLCS(K)%Npf(:)%Ndb_ref = 1
  ENDDO

!  WRITE(*,*) '   layer 1 ', MLCS(1)%Npf(:)%Ndb_ref
!  WRITE(*,*) '   layer 2 ', MLCS(2)%Npf(:)%Ndb_ref
!      write(*,*) '  NMC eto  ', NMC  
  IF( NMC==0 ) THEN
  !  ... This option for water clouds or ice clouds (Yang or Baum DB), i.e.,
  !      single cloud or multi-layer clouds with the same phase condition
     DO K = 1, NCL
        IF( MLCS(K)%Phm_co ) CYCLE ! If phase function does not depend on Ref
        SELECT CASE ( TDS(K)%S )
        CASE('water')
           ALLOCATE( MLCS(K)%Npf(1)%Pdb_ref(N_Ref_db) )
           MLCS(K)%Npf(1)%Ndb_ref = N_Ref_db
           MLCS(K)%Npf(1)%Pdb_ref(:) = Ref_db(:)
        CASE('ice')
           IF( Use_Bdb ) THEN  ;  Nrb = N_Ref_Bdb
              ALLOCATE( MLCS(K)%Npf(1)%Pdb_ref(Nrb) )
              MLCS(K)%Npf(1)%Ndb_ref    = Nrb
              MLCS(K)%Npf(1)%Pdb_ref(:) = Ref_Bdb(1:Nrb) 
           ENDIF  ! Use_Bdb
           IF( Use_Ydb ) THEN  ;  Nrb = MLCS(K)%Na_Ydb
              ALLOCATE( MLCS(K)%Npf(1)%Pdb_ref(Nrb) )
              MLCS(K)%Npf(1)%Ndb_ref    = Nrb
              IF( Use_Dim ) THEN
                 MLCS(K)%Npf(1)%Pdb_ref(:) = Dim_Ydb(K,1:Nrb)
              ELSE
                 MLCS(K)%Npf(1)%Pdb_ref(:) = Ref_Ydb(K,1:Nrb)  
              ENDIF  ! Use_Ydb
           ENDIF
        CASE('ice-sm')
!           WRITE(*,*) '    Start set_pdb_ref '
           CALL SET_PDB_REF( 1 )
           MLCS(K)%Npf(1)%Ndb_ref  = Nrb
!           WRITE(*,*) 'Nrb = ', Nrb
!           WRITE(*,'(*(1x,E12.4))') MLCS(K)%Npf(1)%Pdb_ref(:)
        END SELECT
     ENDDO  ! over cloud layers (K)
  ELSE
!    ---- Single mixed cloud or multi-layer clouds with different
!            phase conditions, i.e., ANY(Plus_ice) = .TRUE.
     DO K = 1, NCL
        IF( MLCS(K)%Phm_co )  CYCLE
!       ... Water
        IF(TDS(K)%S=='water'.OR.TDS(K)%S=='mixed'.OR.TDS(K)%S=='mixed-sm') THEN
           ALLOCATE( MLCS(K)%Npf(1)%Pdb_ref(N_Ref_db) )
           MLCS(K)%Npf(1)%Ndb_ref    = N_Ref_db
           MLCS(K)%Npf(1)%Pdb_ref(:) = Ref_db(:)
        ELSE
           MLCS(K)%Npf(1)%Ndb_ref = 1
           ALLOCATE( MLCS(K)%Npf(1)%Pdb_ref(1) )
           MLCS(K)%Npf(1)%Pdb_ref(1) = Ref_db(1)
        ENDIF
!       ... Ice
!       ---------------------------------------------------------------
        SET_Nrb: Block
        ! ... Yang DB 
          IF( Use_Ydb )  THEN  ;  Nrb = MLCS(K)%Na_Ydb
             ALLOCATE( MLCS(K)%Npf(2)%Pdb_ref(Nrb) )
             IF( Use_Dim ) THEN
                MLCS(K)%Npf(2)%Pdb_ref(:)  = Dim_Ydb(K,1:Nrb)
             ELSE
                MLCS(K)%Npf(2)%Pdb_ref(:)  = Ref_Ydb(K,1:Nrb)
             ENDIF  ;  EXIT SET_Nrb
          END IF  ! Use_Ydb
          !  ... Baum DB
          IF( Use_Bdb )  THEN  ;  Nrb = N_Ref_Bdb
             ALLOCATE( MLCS(K)%Npf(2)%Pdb_ref(Nrb) )
             MLCS(K)%Npf(2)%Pdb_ref(:) = Ref_Bdb(:)
             EXIT SET_Nrb
          END IF  ! Baum DB
          IF( TDS(K)%S == 'ice-sm'.OR.TDS(K)%S == 'mixed-sm' ) THEN
             CALL SET_PDB_REF( 2 )
!             WRITE(*,*) 'Nrb = ', Nrb
!             WRITE(*,'(*(1x,E12.4))') MLCS(K)%Npf(2)%Pdb_ref(:)
!             write(*,*) '   In gt_cloud_input PAUSE '
!             PAUSE 
             EXIT SET_Nrb
          ENDIF  ! 'ice-sm'
          Nrb = 1
        END Block SET_NRB
        MLCS(K)%Npf(2)%Ndb_ref = Nrb
        IF( .NOT.ALLOCATED(MLCS(K)%Npf(2)%Pdb_ref) ) THEN
           ALLOCATE( MLCS(K)%Npf(2)%Pdb_ref(1) )
           MLCS(K)%Npf(2)%Pdb_ref(1) = CLDPROFILE(K,1,7)
        ENDIF
     ENDDO  ! over cloud layers (K)
  END IF  ! NMC==0

  DO K = 1, NCL
     IF( MLCS(K)%Phm_co.AND.(TDS(K)%S=='ice-sm') ) THEN
        DO KK = 1, MLCS(K)%N_pha
!           write(*,*) '       Layer & Nrb ', K, KK, MLCS(K)%Npf(KK)%Ndb_ref
           IF( .NOT.ALLOCATED(MLCS(K)%Npf(KK)%Pdb_ref) )  THEN
              ALLOCATE( MLCS(K)%Npf(KK)%Pdb_ref(1) )
              MLCS(K)%Npf(KK)%Pdb_ref(1) = CLDPROFILE(K,1,7) 
           ENDIF
        ENDDO
     ENDIF  ! on layers with homogeneous phase function
!     WRITE(*,*) '   "gt_cloud_input.f90" Layer number ', K
!     WRITE(*,*) Plus_ice(K), TDS(K)%S, MLCS(K)%Npf(:)%Ndb_ref
  ENDDO
  
  N_ref_sm = 1 
  IF( ANY(.NOT.MLCS(:)%Phm_co) ) THEN
!    ... Scattering matrix depends on a particle effective radius
     Nl_ice = 0
     KK = 1  ;  IF( NMC > 0 ) KK = 2
!     WRITE(*,*) '    Check  ', MLCS(1)%Npf(KK)%Ndb_ref
     DO I = 1, NCL
!        WRITE(*,*) '    Layer ', I, TDS(I)%S, MLCS(I)%Npf(:)%Ndb_ref
        IF( TDS(I)%S=='ice' ) Nl_ice = Nl_ice + 1
        IF( TDS(I)%S=='ice-sm' )  THEN
           IF( MLCS(I)%Npf(KK)%Ndb_ref > N_ref_sm ) &
                N_ref_sm = MLCS(I)%Npf(KK)%Ndb_ref
        ENDIF
  
     ENDDO
     SET_NPH: Block
       IF( Nl_ice /= NCL ) THEN  ! There are water or mixed clouds
          Nr_db(1) = N_ref_db
       ENDIF
       IF( Use_Bdb ) THEN
          Nr_db(2) = N_ref_Bdb
          ALLOCATE( Ref_ice(NCL,N_ref_Bdb) )
          EXIT SET_NPH
       ENDIF
       IF( Use_Ydb ) THEN
          !         ... N_ref_Ydb defines the maximal number of actual 
          !             ice habit dimensions (in all cloud layers) 
          !             selected in the subroutine "read_ydb_dim" (line 348)
          Nr_db(2) = N_ref_Ydb
          ALLOCATE( Ref_ice(NCL,N_ref_Ydb) )
          EXIT SET_NPH
       ENDIF
     END Block SET_NPH
!     Nphr = MAX( Nr_db(1),Nr_db(2) )
  ENDIF  ! ANY( .NOT.MLCS(:)%Phm_co )

  !  IF( Use_Ydb.AND.Phm_const ) THEN
  IF( Use_Ydb.AND.ANY(MLCS(:)%Phm_co) ) THEN
     Nr_db(2) = N_ref_Ydb
!     Nphr = MAX( Nr_db(1),Nr_db(2) )
  ENDIF

  DO I = 1, NCL
     K = MAXVAL( MLCS(I)%Npf(:)%Ndb_ref )
     IF( Nphr <= K ) Nphr = K
  ENDDO

!  WRITE(*,*) '   Imeem: Nphr = ',Nphr
!  PAUSE
  !
  ! ======>  Set components of the structure MLCS <======
  !   
  !  write(*,*) '   Scattering and absorption coefficients - 1'
  STR_INP = 'Scattering and absorption coefficients'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  HOW_CLD_COEF = CUT_STR(READ_ARR(1)%S)
  SELECT CASE( HOW_CLD_COEF )
  CASE('from data base','analytically')
  CASE DEFAULT
     GOTO 207
  END SELECT

  ! The setting of the forward cone range
  ! ======================================
  SELECT CASE ( D_MF )
  CASE ( 'D-F','D-S' )
     ALLOCATE( LC_CLD(NCL),UC_CLD(NCL),MAN_AUTO_CLD(NCL),CON_VAL_CLD(NCL) ) !A
     STR_INP = 'Lower and upper value of forward cone'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Do I = 1, NCL
        READ(READ_ARR(I)%S, *, ERR = 101) LC_CLD(I), UC_CLD(I)
        IF( DABS( UC_CLD(I) - LC_CLD(I) ) < 0.1d0 ) THEN
           MAN_AUTO_CLD(I) = 'manu'
        ELSE
           MAN_AUTO_CLD(I) = 'auto'
        ENDIF
     End Do
  CASE DEFAULT
  END SELECT

  ! Output of phase functions
  ! ==========================
  STR_INP = 'Phase functions output'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR = 101) OUT_PH_CLD
  IF( OUT_PH_CLD ) THEN
     ALLOCATE( Path_PH_CLD(NCL) )
     DO I = 1,NCL
        Path_PH_CLD(I)%S = CUT_STR(READ_ARR(I)%S)
     ENDDO
  ENDIF

  ! Additional absorption within cloud layers
  ! -----------------------------------------
  IF( ALIN_ACT(4) ) THEN
     ADD_ABS(1) = .FALSE.
  ELSE
     STR_INP = 'Additional absorber within cloud'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     DO I = 1,NCL
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        READ(STR_ARR(1)%S,*,ERR = 101) ADD_ABS(I)
        IF( .NOT.ADD_ABS(I) ) CYCLE
        IF( SIZE(STR_ARR) < 2 ) GOTO 229
        Inp_form(I)%S = CUT_STR(STR_ARR(2)%S)
        SELECT CASE( Inp_form(I)%S )
        CASE('numeric')
        CASE('analytic')
           IF( SIZE(STR_ARR) /= 3 ) GOTO 2291
           Add_Abs_mod(I)%S = CUT_STR(STR_ARR(3)%S)
           SELECT CASE( Add_Abs_mod(I)%S )
           CASE('DOM_BK')
           CASE DEFAULT ; GOTO 234
           END SELECT
        CASE DEFAULT
           GOTO 233
        END SELECT
     ENDDO  ! over cloud layers (I)

     IF( ANY(ADD_ABS) ) THEN
!  *********  Start: Wavelength grid and refractive index of absorbers *****
        STR_INP = 'Wavelength grid and refractive index of absorbers'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        I1 = 1
        DO I = 1,NCL
           IF( Inp_form(I)%S /= 'numeric' ) CYCLE
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR, '>')
           IF( SIZE(STR_ARR) == 2 )  FNAME = SET_PATH(STR_ARR(2)%S)
           II = 0
           Call Upper_Lower_Case (READ_ARR(I1)%S  ,'U2L')
           READ_ARR(I1)%S = CUT_STR( READ_ARR(I1)%S )
           If( Index(READ_ARR(I1)%S, "--- layer") /= 0 ) Then
              KJ = Index(READ_ARR(I1)%S, "--->")
              If(KJ == 0) GoTo  2374
              Str = CutChars( READ_ARR(I1)%S(1:KJ - 1) )
              If( LEN_TRIM(Str) == 0) GoTo  2375
              READ(Str,*,Err=101) II
!              WRITE(*,*) '  II and I  ', II, I
              IF( II /= I ) GoTo 2376 
           End If
           If(II == 0) GoTo 2371
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR, '>')
           IF( SIZE(STR_ARR) == 2 ) THEN
              CALL READ_FILE(3, FNAME, DATA_R, *880)
              Nwl = Size(DATA_R,1) ; MLCS(I)%N_WL_SOOT = Nwl
              ALLOCATE( MLCS(I)%WL_SOOT(Nwl), MLCS(I)%REFR_SOOT(Nwl,2) )
              MLCS(I)%WL_SOOT(:) = DATA_R(1:Nwl,1)
              MLCS(I)%REFR_SOOT(:,1) = DATA_R(1:Nwl,2)
              MLCS(I)%REFR_SOOT(:,2) = DATA_R(1:Nwl,3)
              DEALLOCATE( DATA_R )              
              I1 = I1 + 1
              CYCLE
           ENDIF
           I1 = I1 + 1  ! Read from next
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR_1) ! Wavelength grid           
           MLCS(I)%N_WL_SOOT = SIZE(STR_ARR_1)
           II = MLCS(I)%N_WL_SOOT
!           WRITE(*,*) '     SIZE(STR_ARR_1) eto ', SIZE(STR_ARR_1)
           ALLOCATE( MLCS(I)%WL_SOOT(II), MLCS(I)%REFR_SOOT(II,2) )
           DO K = 1,MLCS(I)%N_WL_SOOT 
              READ(STR_ARR_1(K)%S,*,ERR = 101) MLCS(I)%WL_SOOT(K)
           END DO  ! over reference wavelength grid (I)
           I1 = I1 + 1
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR) ! Real part of refr. index
           I1 = I1 + 1
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR_1) ! Imaginary part of refr. index
           II = 1
           DO K = 1, MLCS(I)%N_WL_SOOT
              READ(STR_ARR(K)%S,  *, ERR = 101) MLCS(I)%REFR_SOOT(II,1)
              READ(STR_ARR_1(K)%S,*, ERR = 101) MLCS(I)%REFR_SOOT(II,2)
              II = II + 1
           ENDDO
           I1 = I1 + 1
        ENDDO
!  *********  End: Wavelength grid and refractive index of absorbers *****
!     ENDIF
        
!     IF( ANY(ADD_ABS) ) THEN
!       write(*,*) 'Check the refractive index of absorber:'
!       DO I = 1, NCL
!          IF( ADD_ABS(I).AND.Inp_form(I)%S == 'numeric' ) THEN
!             write(*,'(A,2x,I2)') '     Layer ', I
!             DO K = 1, MLCS(I)%N_WL_SOOT 
!                WRITE(*,'(2x,F7.2,2x,F4.2,2x,F4.2)') MLCS(I)%WL_SOOT(K), &
!                     MLCS(I)%REFR_SOOT(K,1), MLCS(I)%REFR_SOOT(K,2)
!             ENDDO
!          END IF
!       ENDDO  ! over cloud layers

!  *****************  Start: Relative concentration of absorber *******************
        STR_INP = 'Relative concentration of absorber'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        I1 = 1
        DO I = 1, NCL
           IF( .NOT.ADD_ABS(I) ) CYCLE  ! cloud layer without additional absorber

           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR, '>')
           IF( SIZE(STR_ARR) == 2 )  FNAME = SET_PATH(STR_ARR(2)%S) ! there is filename

           II = 0
           Call Upper_Lower_Case (READ_ARR(I1)%S  ,'U2L')
           READ_ARR(I1)%S = CUT_STR( READ_ARR(I1)%S )
           If( Index(READ_ARR(I1)%S, "--- layer") /= 0 ) Then
!            ... Check if the substring "--->" is present
              KJ = Index(READ_ARR(I1)%S, "--->")
              If(KJ == 0) GoTo  2374
!             ... Reading the number of the layer
              Str = CutChars( READ_ARR(I1)%S(1:KJ - 1) )
!            ... Check if the number of layer is present
              If( LEN_TRIM(Str) == 0) GoTo  2375
              READ(Str,*,Err=101) II
              IF( II /= I ) GoTo 2376 
!              write(*,*) '     Layer number after spliting ', II
           ENDIF
           If(II == 0) GoTo 2371

           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR, '>')
           IF( SIZE(STR_ARR) == 2 ) THEN
              CALL READ_FILE(2, FNAME, DATA_R, *880)
              Nalt = Size(DATA_R,1) ; MLCS(I)%NZ_SOOT = Nalt
              ALLOCATE( MLCS(I)%Z_SOOT(Nalt), MLCS(I)%C_SOOT(Nalt) )
              MLCS(I)%Z_SOOT(:) = DATA_R(1:Nalt,1)
              MLCS(I)%C_SOOT(:) = DATA_R(1:Nalt,2)
              DEALLOCATE( DATA_R )              
              I1 = I1 + 1
              CYCLE
           ENDIF
           
           I1 = I1 + 1
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR)
           I1 = I1 + 1
           CALL SPLIT_STRING(READ_ARR(I1)%S, STR_ARR_1)
           MLCS(I)%NZ_SOOT = SIZE( STR_ARR )           
           ALLOCATE( MLCS(I)%Z_SOOT(MLCS(I)%NZ_SOOT), &
                MLCS(I)%C_SOOT(MLCS(I)%NZ_SOOT) )
           II = 1
           DO K = 1, MLCS(I)%NZ_SOOT
              READ(STR_ARR(K)%S, *, ERR = 101) MLCS(I)%Z_SOOT(II)
              READ(STR_ARR_1(K)%S,*, ERR = 101) MLCS(I)%C_SOOT(II)
              II = II + 1
           ENDDO
           I1 = I1 + 1
        ENDDO  ! over cloud layers (I)
!!  *****************  End: Relative concentration of absorber *******************
!       write(*,*) 'Check the additional absorber concentration profile:'
!       DO I = 1, NCL
!          IF( .NOT.ADD_ABS(I) ) CYCLE
!          write(*,'(A,2x,I2)') '     Layer ', I
!          DO K = 1, MLCS(I)%NZ_SOOT
!             WRITE(*,*) MLCS(I)%Z_SOOT(K), MLCS(I)%C_SOOT(K)
!          ENDDO
!       ENDDO
!!        WRITE(*,*) '    Stop in "gt_cloud_input..f90" '
!!        STOP
     ENDIF  ! ANY(ADD_ABS) 
  ENDIF

  ! Does selected weighting function compatible with the type
  ! of input parameter?
  DO I = 1, UBOUND(CLD_INFO,1)
     IF(.NOT.CLD_INFO(I)%WF_FLG) CYCLE
     SELECT CASE( CLD_INFO(I)%NAME(5:7) ) 
     CASE('sca','abs')
        DO K = 1,NCL
           IF( Var_type(K,1)%S == 'TAU' ) GOTO 216
        ENDDO
     CASE('pnc')
        DO K = 1,NCL
           SELECT CASE( Var_type(K,1)%S )
           CASE('TAU','LWP','LWC','IWP','IWC')
              GOTO 217
           END SELECT
        ENDDO
     END SELECT
  ENDDO

  Deriv_Phc = .FALSE.

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  ! <<<----------------------- Error messages ------------------------->>>
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP
  RETURN 1

202 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
  WRITE (EU,'(A,2x,I2)') '  Form of ice crystal in input line ', Ifo
  WRITE (EU,*) '    is not recognized.'
  WRITE (EU,*) '    Check input field of "Form of ice crystals" parameter'
  WRITE (EU,*) '    in the file "cf_cloud.inp". '
  RETURN 1                                      

203 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
  WRITE (EU,*) '  Required dimension of ice crystal is outside  '
  WRITE (EU,*) '    of selected range in layer ',K
  WRITE (EU,*) '  required:  ', DUM
  WRITE (EU,*) '  selected range: ', Dmin(K),' -- ',Dmax(K)
  WRITE (EU,*) '  Check the "Ice crystal dimension range"'// &
       ' control parameter.'
  RETURN 1

204 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
  WRITE (EU,*) '  Required dimension of ice crystal is outside  '
  WRITE (EU,*) '    of selected range in layer ', K
  WRITE (EU,*) '  required:  ', DUM
  WRITE (EU,*) '  selected range: ', Dmin(K),' -- ',Dmax(K)
  WRITE (EU,*) '  Check the "Ice crystal dimension range"'// &
       ' control parameter.'
  RETURN 1

205 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
  WRITE (EU,*) '  Control parameter "Form of ice crystals" requires'
  WRITE (EU,*) ' three input entries if first entry is ice particle shape'
  WRITE (EU,*) ' from Yang database.'
  WRITE (EU,*) '  Set "yes" or "no" as second entry'
  WRITE (EU,*) ' and roughness as third entry.'
  WRITE (EU,*) '    Cloud layer :', I
  WRITE (EU,*) '    Check input file "cf_cloud.inp".'
  RETURN 1                                      

206 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE (EU,*) '    Liquid/Ice water content and particles/crystals density'
  WRITE (EU,*) '    input parameters require vertically inhomogeneity.'
  WRITE (EU,*) '    Set "Cloud layers are vertically inhomogeneous" to "yes"'
  WRITE (EU,*) '    and use file "cld_prof_inp.*".'
  RETURN 1

207 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '"Scattering '// &
       'and absorption coefficients" is not recognized')
  RETURN 1

208 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Input value of "Cloud layers are '// &
       'inhomogeneous" is not recognized')
  RETURN 1

209 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Phase function of '//&
       'cloud layers is not recognized')
  RETURN 1

210 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Clouds are not implemented'// &
       'in the "CDI" mode (check "RTM-CORE" input parameter)')
  RETURN 1

211 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading cloud vertical profile')
  WRITE (EU,*) '    Current cloud layer: ', K
  RETURN 1                                         

212 CALL WRITE_MESSAGE (R_NAME, 'ERROR',    &
       'CLOUD PROFILE: no data points found') 
  RETURN 1                                      

213 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '"Altitude '//&
       'sub-grid within layers" is not recognized')
  WRITE (EU,*) '    Check file "cf_cloud.inp". '
  RETURN 1

214 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
  WRITE (EU,*) '    CLOUD PROFILE: dimensionless altitudes within cloud'
  WRITE (EU,*) '    should be given in ascending order (0 -> 1)'
  WRITE (EU,*) '    Cloud layer :',K
  WRITE (EU,*) '    Check input file :'//FNAME
  RETURN 1                                      

215 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,*) '    Control parameter '
  WRITE (EU,*) '    "Will be scaled vertical profile of" is not recognized'
  WRITE (EU,*) '    Check input file "cf_cloud.inp".'
  RETURN 1

216 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,*) '    Weighting function w.r.t. scattering and '//&
       'absorption coefficient '
  WRITE (EU,*) '    is not allowed if "Type of input integral parameter" '//&
       'is set to "TAU"'
  WRITE (EU,*) '    Check input file "cf_cloud.inp".'
  RETURN 1

217 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,*) '    Weighting function w.r.t. particle number concentration '&
       ' is allowed only if "Type of input integral parameter" is set '//&
       'to "DENS_PROFILE".'
  WRITE (EU,*) '    Check input file "cf_cloud.inp".'
  RETURN 1

218 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE(EU,*) '    Single scattering correction algorithm is not allowed '
  WRITE(EU,*) '    for calculation of cloud top height, bottom height,'
  WRITE(EU,*) '    and cloud optical thickness weighting functions.'
  WRITE(EU,*) '    Check "Weighting function for parameters" '
  WRITE(EU,*) '    control line in the file "cf_cloud.inp".'
  RETURN 1

219 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,*) '    Delta-m truncation index for cloud has to be'
  WRITE (EU,*) '    less than the number of Legendre moments '
  WRITE (EU,'(A,1x,I4)') '              NLEGEN = ', NLEGEN
  WRITE (EU,'(A,1x,I4)') '    Truncation index = ', K_TR
  WRITE (EU,*) '    Check input file "control_ac.inp" and "cf_cloud.inp".'
  RETURN 1

220 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '    Control parameter '
  WRITE (EU,'(A)') '    "Form of ice crystals" is not properly defined '      
  WRITE (EU,'(A,1x,I2)') '    for cloud layer I = ',I 
  WRITE (EU,'(A)') '    Check input file "cf_cloud.inp".'
  RETURN 1

221 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,*) '    In the case of aerosol layer control parameter'
  WRITE(EU,*) '      "Type of input integral parameter" should be set to '
  WRITE(EU,*) '      "TAU" only.'
  WRITE(EU,*) '    Check "Type of input integral parameter" control line'
  WRITE(EU,*) '      in the file "cf_cloud.inp".'
  RETURN 1

223 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,*) '    In the case of aerosol layer control parameter'
  WRITE(EU,*) '    "Number of cloud layers" should be set to 1.'
  WRITE(EU,*) '   Check "Number of cloud layers" and "Thermodynamic state"'
  WRITE(EU,*) '   control lines in the file "cf_cloud.inp".'
  RETURN 1

224 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Second entry of control parameter '
  WRITE (EU,'(A)') ' "Form of ice crystals" has an invalid value '
  WRITE (EU,'(A,1x,I2)') ' for cloud layer I = ',I 
  WRITE (EU,'(A)') '    Check input file "cf_cloud.inp".'
  RETURN 1

225 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Sum of relative concentration of ice habit'
  WRITE (EU,'(A,1x,I2)') '    in cloud layer ', I
  WRITE (EU,'(A)')       '    is not equal to unit  '
  WRITE (EU,'(A)') '  Check input parameters "Thermodynamic state" and '
  WRITE (EU,'(A)') '    "Form of ice crystals" in the file "cf_cloud.inp".'
  RETURN 1

226 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Number of input lines in the '
  WRITE (EU,'(A)') '    "Ice crystal dimension range" input field '
  WRITE (EU,'(A)') '    less than the number of cloud layers.'
  WRITE (EU,'(A)') '  Check the file "cf_cloud.inp".'
  RETURN 1

227 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Error input of parameter "Form of ice crystals"'
  RETURN 1

228 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Wrong input in the "Form of ice crystals" field:'
  WRITE (EU,'(A)') '    for Yang data base second entry in the first input line'
  WRITE (EU,'(A)') '    should be set to "no" or "yes".'
  WRITE (EU,'(A)') '  Check "Form of ice crystals" in the file "cf_cloud.inp".'
  RETURN 1

230 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Number of input lines in the "Form of ice crystals" '
  WRITE (EU,'(A)') '    input field is incorrect.'
  WRITE (EU,'(A)') '  Check "Form of ice crystals" in the file "cf_cloud.inp".'
  RETURN 1

231 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  "Mixed" cloud option has to be checked!'
  RETURN 1

232 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Number of cloud sub-layers larger than MAX_LAY'
  WRITE (EU,*) '    Current KK and MAX_LAAY : ', KK, MAX_LAY
  RETURN 1

229 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Second entry of "Additional absorber within cloud"'
  WRITE (EU,*) '    input field should be set to "numeric" or "analytic".'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2291 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Third entry of "Additional absorber within cloud"'
  WRITE (EU,*) '    input field should be given if the "analytic" option is selected.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

233 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Second entry of "Additional absorber within cloud"'
  WRITE (EU,*) '    input field is not recognized.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

234 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Third entry of "Additional absorber within cloud" input field'
  WRITE (EU,*) '    in the case of "analytic" option can be only set to "DOM_BK".'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

235 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  First input entry of  the "Ice crystal dimension range" '
  WRITE (EU,'(A)') '    input field has to be set to "off", '
  WRITE (EU,'(A)') '    if first entry of "Use PSD for ice crystals" parameter'
  WRITE (EU,'(A)') '    was set to "t". '
  WRITE (EU,'(A)') '  Check the input file "cf_cloud.inp".'  
  RETURN 1

236 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  In the case of stochastic model the parameter'
  WRITE (EU,*)     '    "Use constant phase function within cloud layer"'
  WRITE (EU,*)     '     should be set to "t" for this layer.'
  WRITE (EU,*)     '  Preliminary restriction.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2371 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A,1x,A,1x,A)') '  Wrong input in the "',STR_INP,'" field:'
  WRITE (EU,'(A)') '    the substring "--- layer" is not found.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

!2372 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
!  WRITE (EU,'(A)') '  Wrong input in the "Form of ice crystals" field:'
!  WRITE (EU,'(A)') '    the substring "--->" is not found.'
!  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
!  RETURN 1

2373 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  Wrong input in the "Form of ice crystals" field:'
  WRITE (EU,'(A)') '    in the input line containing "--- layer" substring'
  WRITE (EU,'(A)') '    no data about the number of layer is found.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2370 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A,1x,A,1x,A)') '  Wrong input in the "',STR_INP,'" field:'
  WRITE (EU,'(A)') '    the Relative concentration of absorber is not defined'
  WRITE (EU,'(A)') '    for all layers with additional absorber as given in'
  WRITE (EU,'(A)') '    the "Additional absorber within cloud" input field.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2374 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A,1x,A,1x,A)') '  Wrong input in the "',STR_INP,'" field:'
  WRITE (EU,'(A)') '    the substring "--->" is not found.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2375 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A,1x,A,1x,A)') '  Wrong input in the "',STR_INP,'" field:'
  WRITE (EU,'(A)') '    in the input line containing "--- layer" substring'
  WRITE (EU,'(A)') '    no data about the number of layer is found.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

2376 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A,1x,A,1x,A)') '  Wrong input in the "',STR_INP,'" field:'
  WRITE (EU,'(A)') '    in the input line containing "--- layer" substring'
  WRITE (EU,'(A)') '    the number of layer is inconsistent with the input'
  WRITE (EU,'(A)') '    in the "Additional absorber within cloud" input field.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

238 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,*) '    Thermodynamic state of the cloud in '
  WRITE (EU,*) '      "Cloud layer information" input line is not recognized'
  WRITE (EU,*) '    Check input file "cf_cloud.inp".'
  RETURN 1

239 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  In the case of vertically homogeneous clouds the parameter'
  WRITE (EU,*)     '    "Use constant phase function within cloud layer"'
  WRITE (EU,*)     '     should be set to "t" for all layers.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

240 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE (EU,'(A)') '  Number of entries in the input line '
  WRITE (EU,'(3x,A)') TRIM(STR_INP)
  WRITE (EU,'(A)') '    less than the number of cloud layers.'
  WRITE (EU,'(A)') '  Check the file "cf_cloud.inp".'
  RETURN 1
  
241 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  In the case of fractal or hexagon crystal form'
  WRITE (EU,*)     '    the parameter "Use constant phase function within cloud layer"'
  WRITE (EU,*)     '     should be set to "t" for this layer.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

242 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  In the case of polydisperse ice crystals, '
  WRITE (EU,*)     '    "Use PSD for ice crystals" is set to "t", the parameter'
  WRITE (EU,*)     '    "Use constant phase function within cloud layer"'
  WRITE (EU,*)     '    should be set to "f" for all cloud layers.'
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1
  
243 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE (EU,'(A)') '  In the case of stochastic model of snow layer, '
  WRITE (EU,*)     '    the number density of ice crystals as'
  WRITE (EU,*)     '    "Type of input integral parameter" is not allowed.'
  WRITE (EU,*)     '  Use "TAU", "WATER_PATH" or "IWC_PROFILE". '
  WRITE (EU,'(A)') '  Check the control file "cf_cloud.inp".'
  RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

CONTAINS
  !**************************************************************************
  SUBROUTINE SET_VAR_TYPE_W( II,K1,KK )
    INTEGER  II, K1, KK
    SELECT CASE ( HEL(II,K1)%S )
    CASE ('TAU')          ; Var_type(II,KK)%S = 'TAU'
    CASE ('WATER_PATH')   ; Var_type(II,KK)%S = 'LWP'
    CASE ('LWC_PROFILE')  ; Var_type(II,KK)%S = 'LWC'
    CASE ('DENS_PROFILE') ; Var_type(II,KK)%S = 'PNC'
    END SELECT
  END SUBROUTINE SET_VAR_TYPE_W
  !**************************************************************************
  SUBROUTINE SET_VAR_TYPE_I( II,K1,KK )
    INTEGER  II, K1, KK
    SELECT CASE ( HEL(II,K1)%S )
    CASE ('TAU')          ; Var_type(II,KK)%S = 'TAU'
    CASE ('WATER_PATH')   ; Var_type(II,KK)%S = 'IWP'
    CASE ('IWC_PROFILE')  ; Var_type(II,KK)%S = 'IWC'
    CASE ('DENS_PROFILE') ; Var_type(II,KK)%S = 'CNC'
    END SELECT
  END SUBROUTINE SET_VAR_TYPE_I
     
  !**************************************************************************
  SUBROUTINE SET_PDB_REF( KK )
    INTEGER  INP_LYR, I, KK
    REAL (KND)  R_max, R_min, Del
    !   ========================================================================
    INP_LYR = N_CLDPROF(K) 
    R_max = MAXVAL( CLDPROFILE(K,1:INP_LYR,7) )
    R_min = MINVAL( CLDPROFILE(K,1:INP_LYR,7) )
    IF( R_max == R_min ) THEN
       Nrb = 2
       ALLOCATE( MLCS(K)%Npf(KK)%Pdb_ref(Nrb) )
       MLCS(K)%Npf(KK)%Pdb_ref(1) = 0.99d0*R_min
       MLCS(K)%Npf(KK)%Pdb_ref(2) = 1.01d0*R_min
    ELSE
       Nrb = 11
       ALLOCATE( MLCS(K)%Npf(KK)%Pdb_ref(Nrb) )
       Del = (R_max - R_min)/ (Nrb - 1)
       MLCS(K)%Npf(KK)%Pdb_ref(1) = R_min
       DO I = 2, Nrb
          MLCS(K)%Npf(KK)%Pdb_ref(I) =  MLCS(K)%Npf(KK)%Pdb_ref(I-1) + Del
       ENDDO
    ENDIF
  END SUBROUTINE SET_PDB_REF

  !**************************************************************************
  SUBROUTINE READ_YDB_DIM(*)
    USE CLD_INP,   ONLY: Ext_Ydb,  Vol_Ydb, Dim_Ydb, & 
         CDB_PATH => CLOUD_DATA_BASE_PATH, D_FU
    !   ------------------------------------------------------------------------
    CHARACTER (LEN=120)  Dpi
    CHARACTER (LEN=8)  R_CHR
    Character (Len=:), Allocatable :: FULL_PATH, F_name
    INTEGER  C_U, J, JJ, J1, Nwl_Ydb, Ws, Ws1
    INTEGER  N_act_Ydb
    REAL (KND)  DUM, DD1, DD2, SS
    REAL (KND), PARAMETER:: WL_REF = 500.0d0
    REAL (KND), ALLOCATABLE:: WL_Ydb(:), DUM_READ(:), Apr_Ydb(:,:), &
         Rd(:), Qc(:), DBL(:), DSL(:)
    Type TwoD_Real_Arr
       Real (Knd), Allocatable :: Arr (:,:)
    End Type TwoD_Real_Arr
    Type (TwoD_Real_Arr), Allocatable :: R_FU(:), A_FU(:), V_FU(:)  
    !   ========================================================================
!    Num_FORM = Nsel_form(1)%NN(1) 
    !   ====================================================================      
    ! Reading effective radius, projected area, and volume of 
    ! ice habit which shape will be used in calculations
    ALLOCATE(R_FU(Ncl), A_FU(Ncl), V_FU(Ncl))
    DO I = 1, NCL
       DO Ifo = 1, Nf_ice(I)
          Num_FORM = Nsel_form(I)%NN(Ifo)
          FULL_PATH = CDB_PATH//MD_Crys(Num_FORM)%S//'/'&
               //CRST(I)%SHP(Ifo)%P_ROU//'/'//Long_name(Num_FORM)%S//  &
               '_size_volume.dat'
          !          WRITE(*,*) '  ---> ', TRIM(Dpi)//TRIM(F_name)//'_size_volume.dat'
          OPEN(NewUnit = C_U, FILE = FULL_PATH, STATUS = 'OLD')
          CALL SKIP_HEADER(C_U)
          ! Read full number of maximal dimensions in database
          READ( C_U,* ) N_Ydb
          If (.Not.Allocated(D_FU)) Allocate(D_FU(N_Ydb))
          If (.Not.Allocated(R_FU(I)%Arr)) &
               Allocate(R_FU(I)%Arr(Nf_ice(I),N_Ydb), &
               A_FU(I)%Arr(Nf_ice(I),N_Ydb), V_FU(I)%Arr(Nf_ice(I),N_Ydb) )
          DO J = 1, N_Ydb
             READ(C_U,*) D_FU(J), R_FU(I)%Arr(Ifo,J), A_FU(I)%Arr(Ifo,J),&
                  V_FU(I)%Arr(Ifo,J) 
          ENDDO
          CLOSE(C_U)          
       ENDDO  ! over habits (Ifo)
    ENDDO  ! over cloud layers (I)
    !   ------------------ End of reading -------------------------    

    ALLOCATE( DBL(N_Ydb) )
    DBL(:) = LOG( D_FU(:) )
    DO II = 1, NCL
       Select Case (TDS(II)%S)
       Case ("water", "ice-sm")
          Cycle
       End Select
       Kint = SIZE(CRST(II)%DGR) ;  ALLOCATE( DSL(Kint) )
       DSL(:) = LOG( CRST(II)%DGR(:) )
       DO Ifo = 1, Nf_ice(II)
          Allocate(CRST(II)%SHP(Ifo)%Fr_hab(N_Ydb))
          !         ... Interpolate habit fraction on the full grid of maximal 
          !             dimension (linear w.r.t. log D) 
          CALL LIN_INT(Kint, DSL, CRST(II)%SHP(Ifo)%F_hab(:), &
               N_Ydb, DBL, CRST(II)%SHP(Ifo)%Fr_hab(:),'C')
       ENDDO  ! over habits (Ifo)
       DEALLOCATE( DSL )
    ENDDO  ! over cloud layers (II)

    !   ...  Select interval of dimensions between Dmin and Dmax
    DO I = 1, NCL
       J1 = 1
       !       WRITE(*,*) '   I, Sel_Range(I) ', I, Sel_Range(I) 
       IF( Sel_Range(I) ) THEN
          DUM = Dmin(I)
          IF( DUM==D_FU(1) ) J1 = 2
          DO J = J1, N_Ydb
             DD1 = D_FU(J)
             IF( DD1 >= DUM ) THEN
                MLCS(I)%Nd_s = J-1 ; DUM = 1.0d+20
             ENDIF
             IF( DD1 >= Dmax(I) ) THEN
                MLCS(I)%Nd_e = J
                EXIT
             ENDIF
          ENDDO  ! over ice habit maximal dimensions (J)
          !          WRITE(*,*) '  MLCS(I)%Nd_s ', MLCS(I)%Nd_s
          !          WRITE(*,*) '  MLCS(I)%Nd_e ', MLCS(I)%Nd_e
       ELSE
          MLCS(I)%Nd_e = N_Ydb ; MLCS(I)%Nd_s = 1
       ENDIF
       MLCS(I)%Na_Ydb = MLCS(I)%Nd_e - MLCS(I)%Nd_s + 1
       !       WRITE(*,*) '  ***** MLCS(I)%Na_Ydb ', MLCS(I)%Na_Ydb
    ENDDO  ! over cloud layers (I)
    !       stop

    !!   Output for checking of ice habit fraction in the selected range   
    !! ---------------------------------------------------------------------
    !    OPEN( 47, file='./DATA_OUT/Habit-fraction.dat')
    !    DO II = 1, NCL
    !!       WRITE(*,*) '   Cloud layer ', II
    !       DO K = MLCS(II)%Nd_s, MLCS(II)%Nd_e
    !          DUM = 0.d0
    !          DO Ifo = 1, Nf_ice(II)
    !             DUM = DUM + CRST(II)%SHP(Ifo)%Fr_hab(K)
    !          ENDDO
    !          WRITE(47,'(1x,f9.2,*(2x,f6.3))') D_FU(K), &
    !               (CRST(II)%SHP(Ifo)%Fr_hab(K), Ifo = 1,Nf_ice(II)), DUM
    !       ENDDO  ! over selected dimension range (K)
    !    ENDDO  ! over cloud layers (II)
    !    CLOSE(47)
    !    Stop  '  Stop in "gt_cloud_input.f90" '
    !! ---------------------------------------------------------------------

    !   ... Maximal number  of actual dimensions which will be used
    !    WRITE(*,*) ' ----->  MLCS(:)%Na_Ydb  ', MLCS(:)%Na_Ydb
    N_act_Ydb = MAXVAL( MLCS(:)%Na_Ydb )
    !    PAUSE

    ALLOCATE(Ref_Ydb(NCL,N_act_Ydb), Ext_Ydb(NCL,N_act_Ydb), &
         Apr_Ydb(NCL,N_act_Ydb), Vol_Ydb(NCL,N_act_Ydb),  &
         Dim_Ydb(NCL,N_act_Ydb) )

    !    WRITE(*,*) '        N_act_Ydb  ', N_act_Ydb
    !    WRITE(*,*) '    SHAPE( D_Fu )  ', SHAPE( D_FU ) 
    !    WRITE(*,*) ' SHAPE( Dim_Ydb )  ', SHAPE( Dim_Ydb ) 

    !   ... Save actual values of maximal dimensions    
    DO I = 1, NCL
       !       write(6,*) I, D_FU
       ASSOCIATE ( N1 => MLCS(I)%Nd_s, N2 => MLCS(I)%Nd_e )
         !         WRITE(*,*) '   N1 and N2  ', N1, N2
         Dim_Ydb(I,1:N2-N1+1) = D_FU(N1:N2)
       END ASSOCIATE
    ENDDO  ! over cloud layers (I)
    IF( Use_PSD ) THEN
       ALLOCATE( S_name_Ydb(NCL,N_Ydb))
       !       WRITE(*,*) '   gt_cloud_init: S_name_Ydb '
       DO I = 1, NCL
          DO J = 1, N_Ydb
             WRITE( R_CHR,'(F8.1)' ) D_FU(J)
             S_name_Ydb(I,J) = CUT_STR (R_CHR)
             !             WRITE(*,*) J, D_FU(J), S_name_Ydb(I,J)
          ENDDO  ! over max dimension (J)
       ENDDO  ! over cloud layers (I)
    ELSE
       ALLOCATE( S_name_Ydb(NCL,N_act_Ydb))
       DO I = 1, NCL
          DO J = 1, N_act_Ydb
             WRITE( R_CHR,'(F8.1)' ) Dim_Ydb(I,J)
             S_name_Ydb(I,J) = CUT_STR (R_CHR)
!             WRITE(*,*) I, J, S_name_Ydb(I,J)
          ENDDO  ! over max dimension (J)
       ENDDO  ! over cloud layers (I)
    ENDIF

    IF( Use_PSD ) THEN
       ALLOCATE( Rd(N_Ydb), Qc(N_Ydb), Fd(N_act_Ydb,N_Ydb))
       ! Calculation of quadrature coefficients
       Qc(1) = (D_FU(2) - D_FU(1))*0.5d0
       Qc(N_Ydb) = (D_FU(N_Ydb) - D_FU(N_Ydb-1))*0.5d0
       DO K = 2, N_Ydb-1
          Qc(K) = (D_FU(K+1) - D_FU(K-1))*0.5d0
       ENDDO
       J = 0
       !       WRITE(*,*) '   gt_cloud_init: PSD containing quadrature coefficients'
       DO JJ = MLCS(1)%Nd_s, MLCS(1)%Nd_e  ;  J = J + 1
          Rd(:) =  D_FU(:)/ D_FU(JJ)
          Rd(:) = (Rd(:))**(Kp-1)*EXP( -(Kp-1)*Rd(:) ) ! Gamma PSD for D_0 = D_FU(JJ)
          SS = SUM( Rd(:)*Qc(:) )
          Fd(J,:) = Rd(:)/SS *Qc(:)
          !                  WRITE(*,*) J, JJ, SUM( Fd(J,:) ) 
       ENDDO  ! over actual dimensions (JJ,J)
    ENDIF  ! Use_PSD
    !
    !    WRITE(*,*) '  Set Fd = 1 for D = '
    !    Fd = 0.d0
    !    J = 0
    !    DO JJ = MLCS(1)%Nd_s, MLCS(1)%Nd_e  ;  J = J + 1
    !       WRITE(*,*) '    D_FU(JJ)  ', D_FU(JJ)
    !       Fd(J,JJ) = 1.d0
    !    ENDDO  ! over actual dimensions (JJ,J)
    !
    !    Calculation of effective radius of habit mixture model
    Apr_Ydb = 0.d0 ; Vol_Ydb = 0.d0
    DO I = 1, NCL
       !             WRITE(*,*) '   Cloud layer ', I
       IF( Use_PSD ) THEN
          !      ... Polydisperse ice crystals:
          !      =================================================================
          DO K = 1, N_act_Ydb
             DO Ifo = 1, Nf_ice(I)
                Apr_Ydb(I,K) = Apr_Ydb(I,K) + SUM( A_Fu(I)%Arr(Ifo,:) &
                     *Fd(K,:)*CRST(I)%SHP(Ifo)%Fr_hab(:) )
                Vol_Ydb(I,K) = Vol_Ydb(I,K) + SUM( V_Fu(I)%Arr(Ifo,:) &
                     *Fd(K,:)*CRST(I)%SHP(Ifo)%Fr_hab(:) )
             ENDDO  ! over ice crystal forms (Ifo)
          ENDDO  ! over crystal modal dimensions (K)
          Ref_Ydb(I,:) = 3.d0/4.d0*Vol_Ydb(I,:)/Apr_Ydb(I,:)
          !          DO K = 1, N_act_Ydb
          !             WRITE(*,'(2x,I3,*(2x,E12.4))') K, Ref_Ydb(I,K), &
          !                  Vol_Ydb(I,K), Apr_Ydb(I,K)
          !          ENDDO
          !          OPEN(C_UNIT, FILE = 'Test.dat')
          !          Ifo = 1 ;  K = 2
          !          DO KK = 1, N_Ydb
          !             IF( Fd(K,KK) < 1.d-40 ) THEN
          !                DUM = 0.d0
          !             ELSE
          !                DUM = Fd(K,KK)
          !             ENDIF
          !             WRITE(C_UNIT,'(*(2x,E15.7))') D_FU(KK), R_FU(I,Ifo,KK), &
          !                  A_FU(I,Ifo,KK), V_FU(I,Ifo,KK), DUM, &
          !                  F_hD(I,Ifo,KK) , F_hD(I,2,KK) 
          !          ENDDO
          !          CLOSE(c_UNIT)
          !          stop
       ELSE
          !      ... Monodisperse ice crystals:
          !      =======================================================================
          ASSOCIATE ( N1 => MLCS(I)%Nd_s, N2 => MLCS(I)%Nd_e )
            !          WRITE(*,*) '  Selected Dim Range: ', N1, N2
            KK = 1
            DO K = N1, N2
               DO Ifo = 1, Nf_ice(I)
                  Apr_Ydb(I,KK) = Apr_Ydb(I,KK) + &
                       CRST(I)%SHP(Ifo)%Fr_hab(K)*A_Fu(I)%Arr(Ifo,K)
                  Vol_Ydb(I,KK) = Vol_Ydb(I,KK) + &
                       CRST(I)%SHP(Ifo)%Fr_hab(K)*V_Fu(I)%Arr(Ifo,K)
               ENDDO  ! over ice crystal forms (Ifo)
               KK = KK + 1
            ENDDO  ! over crystals dimension (K, KK)
            !             OPEN(C_UNIT, FILE = 'Test-mix.dat')
            !             DO K = N1, N2
            !                WRITE(C_UNIT,'(*(2x,E15.7))') D_FU(K), &
            !                     CRST(I)%SHP(1)%Fr_hab(K), CRST(I)%SHP(2)%Fr_hab(K)
            !             ENDDO
            !             CLOSE(c_UNIT)
          END ASSOCIATE
          Ref_Ydb(I,:) = 3.d0/4.d0*Vol_Ydb(I,:)/Apr_Ydb(I,:)
       ENDIF
    ENDDO  !  over cloud layers (I)

!        WRITE(*,*) '   -----> D_FU and Ref_Ydb:'
!        II = 1  ;  KK = 1
!        DO K = MLCS(II)%Nd_s, MLCS(II)%Nd_e
!           WRITE(*,'(*(2x,f8.2))') D_FU(K), Ref_Ydb(II,KK)
!           KK = KK + 1
!        ENDDO
!        stop

    !   -----------------------------------------------------------------      
    !    ... Reading extinction cross-sections at reference wavelength
    !   -----------------------------------------------------------------      
    ALLOCATE( DUM_READ(N_Ydb) )
    Ext_Ydb = 0.d0
    DO I = 1, NCL
       ASSOCIATE (Nd_s => MLCS(I)%Nd_s, Nd_e => MLCS(I)%Nd_e, &
            Na => MLCS(I)%Na_Ydb)
         DO Ifo = 1, Nf_ice(I)
            Num_FORM = Nsel_form(I)%NN(Ifo) 
            !-----------------------------------------------------------      
            ! Reading wavelength grid [nm]
            !-----------------------------------------------------------      
            F_name = CDB_PATH//MD_Crys(Num_FORM)%S//'/'//&
                 CRST(I)%SHP(Ifo)%P_ROU//'/'//Long_name(Num_FORM)%S
            FULL_PATH = F_name//'_wavelength.dat'
            OPEN(NewUnit = C_U, FILE = FULL_PATH, STATUS = 'OLD')
            CALL SKIP_HEADER( C_U )
            READ( C_U,* )
            Ws = 0
            Do  
               Read(C_U, *, END = 500) DUM
               If(DUM > WL_REF) Exit
               Ws = Ws + 1
            End Do
500         Continue

            FULL_PATH = F_name//'_extinction_cross_section.dat'
            OPEN(NewUnit = C_U, FILE = TRIM(FULL_PATH), STATUS = 'OLD')
            CALL SKIP_HEADER( C_U )
            DO J = 1, Ws-1
               READ(C_U,*)
            ENDDO
            ! Reading extinction coefficient for all dimensions
            READ(C_U,*) (DUM_READ(J), J = 1,N_Ydb)
            CLOSE(C_U)
            IF( Use_PSD ) THEN
               DO K = 1, Na
                  Ext_Ydb(I,K) = Ext_Ydb(I,K) + SUM( DUM_READ(:)&
                       *Fd(K,:)*CRST(I)%SHP(Ifo)%Fr_hab(:) )
               ENDDO  ! over crystals dimension (K)
            ELSE
               KK = 1
               DO K = Nd_s, Nd_e
                  Ext_Ydb(I,KK) = Ext_Ydb(I,KK) &
                       + CRST(I)%SHP(Ifo)%Fr_hab(K)*DUM_READ(K)
                  KK = KK + 1
               ENDDO  ! over crystals dimension (K, KK)
            ENDIF  ! Use_PSD --> Polydisperse crystals
         ENDDO  ! over ice crystal forms (Ifo)
       END ASSOCIATE
    ENDDO  ! over cloud layers (I)
    !   -----------------------------------------------------------------      

    !    WRITE(*,*) '  N_act_Ydb   ', N_act_Ydb 
    !    PAUSE

    N_ref_Ydb  =  N_act_Ydb 

  END SUBROUTINE READ_YDB_DIM

  !**************************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    IF( Use_Bdb.OR.Use_Ydb ) DEALLOCATE(Ref_ice)
    IF( ALLOCATED(Ref_wi) ) DEALLOCATE(Ref_wi)
    IF(ALLOCATED(Plus_ice)) DEALLOCATE(Plus_ice, ADD_ABS)
    IF(ALLOCATED(N_CLDPROF)) DEALLOCATE(N_CLDPROF, CLD_DM, J_BOT,  &
         CLD_LAYER_HEIGHTS, TAU_CLD_INP, DBC_NAME, SUB_LAYERING, Inp_form, &
         Add_Abs_mod, CLD_LAYER_NUM, USER_EK_NAME, CLD_PRO_LOC, Var_type,  &
         IWP, LWP, TDS, HEL, PHASE_NAME, CLDPROFILE, Nf_ice, N_ref_ice)
    IF(ALLOCATED(LC_CLD)) DEALLOCATE(LC_CLD,UC_CLD,MAN_AUTO_CLD, CON_VAL_CLD)
    IF(ALLOCATED(Path_PH_CLD)) DEALLOCATE(Path_PH_CLD)
    !    IF(ALLOCATED(WL_SOOT)) DEALLOCATE(WL_SOOT, REFR_SOOT, Z_SOOT, C_SOOT)
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
  !*****************************************************************
  Subroutine AddLine (Line)
    Character (Len = :), Allocatable:: Line
    If (.Not.Associated(Element)) Then
       AllocatE(FirstElem)
       Element => FirstElem
       Element%NN = 1
    Else
       Allocate(Element%NextElem)
       Element%NextElem%NN = Element%NN + 1
       Element => Element%NextElem
    End If
    Element%Line = Line
  End Subroutine AddLine
  !*****************************************************************
  Subroutine FinalizeInput (StrArr)
    Integer NN
    Type (Dyn_Char), Allocatable :: StrArr(:)
    Type (Linked_List_Entry), Pointer :: Next

    Nullify(Element%NextElem)
    NN = Element%NN
    If(Allocated(StrArr)) DeAllocate(StrArr)
    ALLOCATE (StrArr(NN))

    Element => FirstElem
    Do While (Associated(Element))
       StrArr(Element%NN)%S = Element%Line
       Next => Element%NextElem
       DEALLOCATE (Element%Line) ; DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       Element => Next
    END DO
  End Subroutine FinalizeInput
  !*****************************************************************
  Subroutine ReadData (*,*)
    Integer JJ
    Character (Len = :), Allocatable :: TXT
    Call FinalizeInput (Str_Arr)
    Nf_ice(II) = Size(Str_Arr)
    Allocate( CRST(II)%SHP(Nf_ice(II)), Nsel_form(II)%NN(Nf_ice(II)) )
    Do JJ = 1, Nf_ice(II)
       Call Split_String(Str_Arr(JJ)%S, Str_Arr_1)

       ! Fraction of habit at grid points             
       Allocate(CRST(II)%SHP(JJ)%F_HAB, Mold = CRST(II)%DGR)
       TXT = CUT_STR( Str_Arr_1(1)%S )
       DO J = 1, Ns_Y
          IF( TRIM(TXT) == Long_name(J)%S ) THEN
             CRST(II)%SHP(JJ)%ICE_FORM = Short_name(J)%S
             Num_FORM = J ; Nsel_form(II)%NN(JJ) = J
             EXIT
          ENDIF
       ENDDO  ! over all shapes in Yang DB (J)
       IF( Nsel_form(II)%NN(JJ) == 0 ) Return 1
       CRST(II)%SHP(JJ)%P_ROU = CUT_STR( Str_Arr_1(2)%S )
       CRST(II)%SHP(JJ)%ICE_DIR = MD_Crys(Num_FORM)%S//'/'//&
            CRST(II)%SHP(JJ)%P_ROU//'/'//Long_name(Num_FORM)%S
       Do Ig = 1, Kint
          READ(Str_Arr_1(Ig + 2)%S,*,ERR=700) CRST(II)%SHP(JJ)%F_HAB(Ig)
       End Do  ! over grid points (Ig)
    End Do  ! over habit shapes (JJ)
    Return
700 Return 2
  End Subroutine ReadData
  !*****************************************************************

END SUBROUTINE GT_CLOUD_INPUT
!********************************************************************C
