C********************************************************************C
C
C	SCIATRAN - Version 2.0 - Last modification: 16 Sep 2003
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
c     Read one line from specified input unit, and look up the resulting
c     character string in a table; returns its index if table, or -1 if
c     not found. (Comparison is not case-sensitive)

      Subroutine NAME_LOOKUP ( iunit, table, ntable, value, * )

      Implicit None

      CHARACTER (LEN = 11), PARAMETER :: ROUTINE_NAME = 'NAME_LOOKUP'

c     Input arguments
c     ---------------
      Integer         iunit     ! Unit number to read from
      Integer         ntable    ! Number of entries in table

      Character*(*)   table(ntable) ! Lookup table

C     The table need not have any particular ordering of the entries,
c     and mixed case may be used.  (Of course, all this would work
c     faster if both particulars were more constrained; but we are
c     assuming tables with a dozen entries or less, and a program which
c     spends immensely more time doing calculations than input.)

      Logical         error

c     Output arguments
c     ----------------
      Integer         value

c     Local variables
c     ---------------
      Character*132  line

c     Start of code
c     -------------
      error = .false.

      Read ( unit = iunit, fmt ='(A)', End=200 ) line

      IF( TRIM(line) .EQ. 'user' ) THEN
         value = 6
         return
      ENDIF

      Call TABLE_LOOKUP ( error, line, table, ntable, value )

      If (error) Then
         call write_message(routine_name, 'ERROR', 
     $   'Error in table_lookup')
         return 1
      Endif

      RETURN

c     Branch here on input error.      
 200  CONTINUE
      VALUE = -1
      CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $     'End of file encountered in NAME_LOOKUP')
      RETURN 1 

      END
c     *******************************************************************
c     *******************************************************************
c     *******************************************************************
C     Look up the specified character string in a lookup table;
C     return its index in table, or -1 if not found.
C     Comparison is not case-sensitive, and leading blanks are removed.

      Subroutine TABLE_LOOKUP (ERROR, STRING, TABLE, NTABLE, VALUE)

      USE LOG_CONTROL, ONLY : ERRORUNIT

      Implicit None

      CHARACTER (LEN = 12), PARAMETER :: ROUTINE_NAME = 'TABLE_LOOKUP'

C     Input arguments.
C     String to look up:
      Character * (*) STRING

C     Number of entries in table:
      Integer         ntable

C     Lookup table:
      Character* (*) table (ntable)

C     The table need not have any particular ordering of the entries,
c     and
C     mixed case may be used.  (Of course, all this would work faster if
C     both particulars were more constrained; but we are assuming tables
C     with a dozen entries or less, and a program which spends immensely
C     more time doing calculations than input.)

C     Modified argument.  ERROR is set to .TRUE. if string cannot be
C     located in table.  Otherwise it's left as is.

      Logical         ERROR

C     Output arguments.
      INTEGER         VALUE

C     Local variables.
      INTEGER         III, START
      Character * 132 COMPARE

C     No local variables need to be SAVEd.
C***********************************************************************

      VALUE = -1

C     If the string is null, we know at once that it won't match.
C     Otherwise, find its first nonblank character.

      If (STRING  .EQ.  ' ') GO TO 200
      CALL UCSTRING ( STRING )
      START = 1
      Do 50 III = 1, LEN (STRING)
         If (STRING (III:III) .NE. ' ') Then
            START = III
            GO TO 75
         End If
 50   Continue

 75   Continue

      Do 100 III = 1, NTABLE
         COMPARE = TABLE ( III )
         CALL UCSTRING ( COMPARE )
         If ( STRING (START:) .EQ. COMPARE ) Then
            VALUE = III
            GO TO 200
         End If
 100  Continue

 200  Continue

      If (VALUE  .EQ.  -1) Then

         call write_message(routine_name, 'ERROR', 
     $        'Can''t find input string in table')
         write(errorunit,*) '    String: ', string
         error = .true.
         return

      End If

      Return
      End
c     *******************************************************************
c     *******************************************************************
c     *******************************************************************
C  Convert a string to upper-case.  (ASCII character set assumed.)
      SUBROUTINE UCSTRING ( STRING )

      CHARACTER (LEN = *)  STRING
      INTEGER         CCC, I

      DO I = 1, LEN_TRIM(STRING)
         CCC = IACHAR(STRING (I:I)) 
         IF(CCC >= 97 .AND. CCC <= 122) STRING (I:I) = ACHAR(CCC - 32)
      END DO

      RETURN
      END

c     *******************************************************************






