C********************************************************************C
C
C	SCIATRAN - Version 4.5 - Last modification: 20 Oct 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
c ***********************************************************************
      SUBROUTINE GT_ESFT_PARAMETER_SETUP(WL, NSPEC, *)
c     ... Called by "rtm_init.f", "dom_spherical.f"

      USE ABSORBERS, ONLY: N_ESFT_MOL,  
     $                     N_ST_MOL, MIX_ABS_START
      USE GRIDS,     ONLY: NLAYERS
      USE INFO,      ONLY: TG_INFO
      USE LOG_PARS,  ONLY: LUNIT, EU, LOG_LEVEL, N_SSO,
     $                     Work_Dir
      USE PROFILES,  ONLY: PRESS, TEMP
      USE ESFT,      ONLY: ESFT_DB_PATH, DO_ESFT_SPEC_MOL, 
     $                     ESFT_OVERLAP_MODE, ESFT_COEFF, ESFT_ALPHA,
     $                     ESFT_MOL_OVERLAP_IDX1, ESFT_MOL_OVERLAP_IDX2, 
     $                     ESFT_NCOEFF, N_ESFT_COEFF, ESFT_WEIGHTS,  
     $                     D_DP => D_ESFT_COEFF_DP, 
     $                     D_DT => D_ESFT_COEFF_DT
      USE ST_VARS,   ONLY: WF_FLAG, LA_FLAG, N_HR_EXT, DO_RAMAN
      USE TOOLS,     ONLY: Read_Input
      USE TYPES,     ONLY: KND, DYN_CHAR 
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) ::  NSPEC
      REAL (KND)  WL(NSPEC)
c     ... Local variables
      CHARACTER (LEN = *), PARAMETER :: R_NAME='GT_ESFT_PARAMETER_SETUP'
      LOGICAL  ALPHA_FOUND, READ_DB
      INTEGER  I, J, K, L, JJ, KK, LL, NPRESS_DB, SPEC_IDX, NTEMP_DB, 
     $     NALPHA, NUM, N_DEFAULT, MM, N_DB, N_INTERVALS, CU
      INTEGER, PARAMETER :: MAX_ESFT_DB = 25,
     $     MAX_NTEMP_DB = 10, MAX_NPRESS_DB = 20, MAX_NALPHA = 6000, 
     $     MAX_ESFT_SPECPTS = 10000
      INTEGER  M_GREATER_ONE(NSPEC)
      CHARACTER (LEN=30)  DB_NAME(MAX_ESFT_DB)
      CHARACTER (LEN=200) FILENAME
      CHARACTER (LEN=80)  STRING
      CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
      CHARACTER (LEN=1)  CHAR
      CHARACTER (LEN=27), ALLOCATABLE :: ALPHA_FILENAME(:)
      REAL (KND)  TEMP_DB(MAX_NTEMP_DB), PRESS_DB(MAX_NPRESS_DB),
     $     DB_WEIGHTS(N_ESFT_COEFF)
      INTEGER, DIMENSION(MAX_ESFT_DB) :: DB_NP, DB_NPRESS, DB_NTEMP, 
     $     DB_NCOEFF
      REAL (KND), DIMENSION( MAX_NALPHA  ) :: CENTER_WL, ALPHA
      REAL (KND), DIMENSION( MAX_ESFT_DB ) :: DB_S, DB_E, DB_SAMP      
      REAL (KND), TARGET :: DB_COEFF(MAX_ESFT_SPECPTS,  
     $     MAX_NPRESS_DB, MAX_NTEMP_DB, N_ESFT_COEFF)
      REAL (KND), POINTER :: ESFT_DB(:,:)
      REAL (KND), ALLOCATABLE :: ALPHA_DEFAULT_START_WL(:), 
     $     ALPHA_DEFAULT_END_WL(:), ALPHA_FILE_START_WL(:), 
     $     ALPHA_FILE_END_WL(:)
      REAL (KND), PARAMETER ::  ALPHA_DEFAULT = 0.5D0
      REAL (KND)  ACT_PP, ACT_TT
      TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here

      ALLOCATE (ESFT_WEIGHTS(NSPEC,N_ESFT_COEFF,N_ESFT_MOL), ! A
     $     ESFT_MOL_OVERLAP_IDX2(NSPEC), ESFT_MOL_OVERLAP_IDX1(NSPEC), ! A
     $     ESFT_NCOEFF(NSPEC,N_ESFT_MOL), ESFT_OVERLAP_MODE(NSPEC),  ! A
     $     DO_ESFT_SPEC_MOL(NSPEC,N_ESFT_MOL), ESFT_ALPHA(NSPEC),! A
     $     ESFT_COEFF(NSPEC,N_ESFT_COEFF,NLAYERS,N_ESFT_MOL)) ! A


      ESFT_MOL_OVERLAP_IDX1(:) = 0
      ESFT_MOL_OVERLAP_IDX2(:) = 0
      ESFT_ALPHA           (:) = 0.0D0
      DO_ESFT_SPEC_MOL   (:,:) = .FALSE.
      ESFT_NCOEFF        (:,:) = 0
      ESFT_WEIGHTS     (:,:,:) = 0.0D0
      ESFT_COEFF     (:,:,:,:) = 0.0D0

      IF( WF_FLAG ) THEN
         ALLOCATE(D_DP( NSPEC,N_ESFT_COEFF,NLAYERS,N_ESFT_MOL ), ! A
     $        D_DT( NSPEC,N_ESFT_COEFF,NLAYERS,N_ESFT_MOL ) ) ! A
         D_DP(:,:,:,:) = 0.0D0
         D_DT(:,:,:,:) = 0.0D0
      END IF

      IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** READING: esft.inp'

      DO JJ = 1, N_ESFT_MOL 
         K = JJ + MIX_ABS_START - 1
         IF (.NOT.TG_INFO(K)%FM_FLG ) CYCLE
C        READ ESFT USER DATA BASE INFO
C        -----------------------------
         CALL READ_ESFT_INP (TG_INFO(K)%NameUC, *880)

         KK = 1
         DO LL = 1, N_DB
            READ_DB = .TRUE.
c            DO I = KK, N_HR_EXT  ! new st_wl_setup
            DO I = KK, NSPEC
               IF(WL(I) > DB_E(LL)) EXIT
               IF(WL(I) < DB_S(LL)) CYCLE
               IF(.NOT.DO_RAMAN) THEN
                  IF(.NOT.LA_FLAG(I)) CYCLE
               ENDIF
               DO_ESFT_SPEC_MOL(I,JJ) = .TRUE.
c              ---------------------------------------------------------------
c              Read ESFT data bases, Interpolate ESFT coefficients on altitude 
c              grid. Determine number of coefficients and weights for each 
c              spectral point. 
c              Note: the weights and the number of coefficients for different 
c              absorbers have to be IDENTICAL for a given spectral point.
c             ---------------------------------------------------------------
               IF(READ_DB) CALL READ_DB_FORMAT (*880)
               READ_DB = .FALSE.
            
               SPEC_IDX = CEILING((WL(I) - DB_S(LL))/DB_SAMP(LL))
               
               ESFT_NCOEFF(I,JJ) = DB_NCOEFF(LL)
               
               ESFT_WEIGHTS(I,1:DB_NCOEFF(LL),JJ) = 
     $              DB_WEIGHTS(1:DB_NCOEFF(LL))
               
               NPRESS_DB = DB_NPRESS(LL)
               NTEMP_DB =  DB_NTEMP(LL)
               
c               Altitude interpolation
               DO J = 1, NLAYERS
                  ACT_PP = PRESS(J)
                  ACT_TT = TEMP(J)
                  DO K = 1, ESFT_NCOEFF(I,JJ)
                     ESFT_DB => 
     $                    DB_COEFF(SPEC_IDX,1:NPRESS_DB,1:NTEMP_DB,K)
                     CALL INTERP_EXTRAPOL_AND_WF
                  ENDDO          
               ENDDO            

c     -------------------------------------------------
            ENDDO               ! I = KK, N_HR_EXT
            KK = I
         ENDDO                  ! LL = 1, N_DB
      ENDDO                     ! JJ = 1, N_ESFT_MOL 

c     Overlapping line absorbers:
c     ---------------------------

c     ESFT overlap modes (controlled by: esft_overlap_mode(lambda))
c     ------------------------------------------------------------
c     esft_overlap_mode = 1
c     ---------------------
c     Default mode
c     In case of overlapping esft-absorber essentially assumes
c     perfect correlation, i.e. 
c             k_1 first absorber with k_1 second absorber, ..., weight w_1
c             k_2 first absorber with k_2 second absorber, ..., weight w_2
c             ...
c             k_M first absorber with k_M second absorber, ..., weight w_M
c     Requires the same number of coefficients (M) and the same weights
c     for all esft-absorber except for M=1 absorber.

c     esft_overlap_mode = 2
c     ---------------------
c     Alpha mode      
c     Two strong overlapping esft-absorbers. There might be other
c     esft-absorbers as well but they have to have only one
c     coefficient, i.e. they have to be weak.
c     Uses mixing concept with mixing parameter alpha.
c     alpha might be read in from file or a default value will be used
c     (alpha_default).
c     Mode only active for spectral ranges specified in rtm_esft.inp.

c     Set default overlap mode for all wavelength
c     -------------------------------------------
c     esft_overlap_mode = 1 means:
c          - Essentially perfect correlation assumption. 
c          - Mode requires that
c            the number of coefficients (M) is equal for all
c            overlapping esft-absorber (except for absorber with M=1)!

      esft_overlap_mode( 1:NSPEC ) = 1

c     Read ESFT user data base info
c     -----------------------------
      IF(LOG_LEVEL(1) >= N_SSO)  write(lunit,*) 
     $     ' *** Reading: rtm_esft.inp -->> mixing mode'

      STR_INP  = 'Mixing parameter alpha filename'
      CALL Read_Input (STR_INP, READ_ARR, *880)
      READ(READ_ARR(1)%S, *, ERR = 101) N_INTERVALS

      ALLOCATE(ALPHA_FILE_START_WL(N_INTERVALS), 
     $     ALPHA_FILE_END_WL(N_INTERVALS), ALPHA_FILENAME(N_INTERVALS))
      do i = 1, n_intervals
         read( READ_ARR(I + 1)%S,* ) num, alpha_file_start_wl(i),
     &        alpha_file_end_wl(i), alpha_filename(i)
      enddo

      STR_INP  = 'Mixing parameter alpha default'
      CALL Read_Input (STR_INP, READ_ARR, *880)
      READ(READ_ARR(1)%S, *, ERR = 101) N_DEFAULT

      IF(LOG_LEVEL(1) >= N_SSO)  write(lunit,*) 

      ALLOCATE(ALPHA_DEFAULT_START_WL(N_INTERVALS), 
     $     ALPHA_DEFAULT_END_WL(N_INTERVALS))
      do i = 1, N_DEFAULT
         read(READ_ARR(I + 1)%S,*) num, alpha_default_start_wl(i),
     &        alpha_default_end_wl(i)
      enddo

c     Determine for each wavelength 
c     - the number of overlapping esft-absorber (nsimultan)
c     - the number of molecules with M>1 (m_greater_one)
c     - also store the esft mol indices for first two molecules
c       with M>1
c     -----------------------------------------------------
      M_GREATER_ONE(:) = 0
      DO I = 1, NSPEC
         IF( .NOT.LA_FLAG(I) ) CYCLE

         DO J = 1, N_ESFT_MOL
            IF(.NOT.DO_ESFT_SPEC_MOL(I,J)) CYCLE

            IF(ESFT_NCOEFF(I,J) > 1) 
     $           M_GREATER_ONE(I) = M_GREATER_ONE(I) + 1

            IF(M_GREATER_ONE(I) == 1.AND.ESFT_MOL_OVERLAP_IDX1(I) == 0) 
     $           ESFT_MOL_OVERLAP_IDX1( I ) = J

            IF(M_GREATER_ONE(I) == 2.AND.ESFT_MOL_OVERLAP_IDX2(I) == 0)
     $           ESFT_MOL_OVERLAP_IDX2( I ) = J
            
         ENDDO
      ENDDO

c     Check if all M's (for M>1) are identical
c     ---------------------------------------
      DO I = 1, NSPEC
         IF( .NOT.LA_FLAG(I) ) CYCLE
         MM = 0
         IF(M_GREATER_ONE(I) <= 1) CYCLE
         DO J = 1, N_ESFT_MOL
            IF(.NOT.DO_ESFT_SPEC_MOL(I,J)) CYCLE
            IF(ESFT_NCOEFF(I,J) <= 1) CYCLE

            IF (MM.EQ.0) THEN
               MM = ESFT_NCOEFF(I,J)
            ELSE
               IF(ESFT_NCOEFF(I,J).NE.MM) THEN
                  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $                 'Number of coefficients M differ (M>1only)')
                  RETURN 1
               ENDIF
            ENDIF
               
         ENDDO
      ENDDO

c      WRITE( *,* ) ' ------->> TEST OVERLAP <<---------- '
c      do i = 1, nspec
c         write(*,'(1x,i4,1x,f8.3,1x,i4,1x,d11.3)') 
c     &      i, spec(1,i), esft_overlap_mode(i), esft_alpha(i)
c         write(*,*) esft_mol_overlap_idx1(i),esft_mol_overlap_idx2(i)
c      enddo

c     Set overlap mode 2 (use mixing parameter alpha)
c     -----------------------------------------------
c     First set DEFAULT alpha values only
c     Skip this section if current wavelength range outside user specified
c     default alpha spectral range :
c          1, 1230.0, 1310.0, 'O2 (strong) H2O (weak)    ' 
c          2, 1670.0, 1720.0, 'CH4 H2O                   '

      DO I = 1, NSPEC
         IF( .NOT.LA_FLAG(I) ) CYCLE
         IF(M_GREATER_ONE(I) /= 2) CYCLE
         DO J = 1, N_DEFAULT
            IF(WL(I) >= ALPHA_DEFAULT_START_WL(J).AND.
     &           WL(I) <= ALPHA_DEFAULT_END_WL(J)) THEN
               ESFT_OVERLAP_MODE(I) = 2
               ESFT_ALPHA(I) = ALPHA_DEFAULT
               EXIT
            ENDIF 
         ENDDO
      ENDDO

c     Now set data base alpha values (might overwrite default values)
c     Skip this section if current wavelength range outside user specified
c     file alpha spectral range :
c        1, 1430.0, 1453.0, 'alpha_h2o_co2_1430_1453.dat' 
c        2, 1920.0, 2060.0, 'alpha_h2o_co2_1920_2060.dat' 
c        3, 2300.0, 2400.0, 'alpha_h2o_ch4_2300_2400.dat' 

      DO J = 1, N_INTERVALS
         IF (WL(1).GT.ALPHA_FILE_END_WL(J) 
     &         .OR.WL(NSPEC).LT.ALPHA_FILE_START_WL(J)) CYCLE

         FILENAME=TRIM(ESFT_DB_PATH)//ALPHA_FILENAME(J)
         IF(LOG_LEVEL(1) >= N_SSO) 
     $        WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILENAME)
         OPEN(NEWUNIT=CU, FILE = FILENAME, STATUS='OLD')

         DO
            READ(CU,'(A)') CHAR
            IF (CHAR /= ';') EXIT
         END DO
         BACKSPACE CU

         READ(CU,*) NALPHA
         IF (NALPHA.GT.MAX_NALPHA) GOTO 293

         DO K = 1, NALPHA
            READ(CU, *) CENTER_WL(K), ALPHA(K)
         ENDDO
         CLOSE(CU)

         DO I = 1, NSPEC
         IF( .NOT.LA_FLAG(I) ) CYCLE
            IF (WL(I) > ALPHA_FILE_END_WL(J)) EXIT
            IF (WL(I) < ALPHA_FILE_START_WL(J)) CYCLE
            IF(M_GREATER_ONE(I) /= 2) CYCLE

            ALPHA_FOUND = .FALSE.
            DO K = 1, NALPHA
               IF (WL(I) > CENTER_WL(K) - 0.00001D0.AND.
     $              WL(I) < CENTER_WL(K) + 0.00001D0) THEN
c     alpha value exists for current wavelength
                  ESFT_OVERLAP_MODE(I) = 2
                  ESFT_ALPHA(I) = ALPHA(K)
                  ALPHA_FOUND = .TRUE.
               ENDIF
            ENDDO
            IF(.NOT.ALPHA_FOUND) GOTO 294
         ENDDO
      ENDDO


      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 101  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
      WRITE(EU,*) '    Parameter to read: ', STR_INP
      RETURN 1

 293  call write_message(r_name, 'ERROR', 
     $     'Too many alpha values in data base file')
      write(EU,*) '    Max. (max_nalpha): ', max_nalpha
      write(EU,*) '    Reqir. (nalpha):   ', nalpha
      RETURN 1
      
 294  call write_message(r_name, 'ERROR', 
     $     'Alpha not found in data base')
      write(EU,*) '    Act. wavelength:   ', WL(I)
      RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
      
      CONTAINS
C**************************************************************************
      SUBROUTINE READ_ESFT_INP ( GN, * )
      CHARACTER (LEN = 13), PARAMETER :: R_NAME='READ_ESFT_INP'
      CHARACTER *(*) GN

      STR_INP  = 'ESFT '//TG_INFO(K)%NameUC//' data bases'
      CALL Read_Input (STR_INP, READ_ARR, *880)
      READ(READ_ARR(1)%S, *, ERR = 101) N_DB
      IF (N_DB <= 0 ) THEN
         call write_message(r_name, 'WARNING', 
     $        'No ESFT data bases --> '//GN)
         N_DB = 0
         RETURN
      ENDIF
      IF (N_DB > MAX_ESFT_DB) THEN
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $        'Too many ESFT data bases --> '//GN )
         WRITE(EU,*)
     $        '    Current max_esft_db value is: ', max_esft_db
         WRITE(EU,*)
     $        '    Required number is:           ', N_DB
         WRITE(EU,*)
     $        '    Increase max_esft_db in gt_esft_parameter_setup'
         RETURN 1
      ENDIF

      DO I = 1, N_DB
         READ(READ_ARR(I + 1)%S,*) NUM, DB_NAME(I), DB_S(I), DB_E(I),  
     $        DB_SAMP(I), DB_NP(I), DB_NPRESS(I), DB_NTEMP(I),
     $        DB_NCOEFF(I)


         IF (DB_NPRESS(I) > MAX_NPRESS_DB) THEN
            call write_message(r_name, 'ERROR', 
     $           'Too many pressures in '//GN//' ESFT data base')
            write(EU,*) '    Max. value: ', max_npress_db
            write(EU,*) 
     &           '    Increase max. value in gt_esft_parameter_setup'
            RETURN 1
         ENDIF
         IF (DB_NTEMP(I) > MAX_NTEMP_DB) THEN
            call write_message(r_name, 'ERROR', 
     $           'Too many temperatures in '//GN//' ESFT data base')
            write(EU,*) '    Max. value: ', max_ntemp_db
            write(EU,*) 
     &           '    Increase max. value in gt_esft_parameter_setup'
            RETURN 1
         ENDIF
         IF (DB_NCOEFF(I) > N_ESFT_COEFF) THEN
            call write_message(r_name, 'ERROR', 
     $           'Too many ESFT coefficients for '//GN//' data base')
            write(EU,*) '    Max. value: ', n_esft_coeff
            write(EU,*) 
     &           '    Increase max. value in GOMETRAN.PARS'
            RETURN 1            
         ENDIF
         if (DB_NP(I) > max_esft_specpts) then 
            call write_message(r_name, 'ERROR', 
     $           'Too many ESFT spec. points for '//GN//' data base')
            write(EU,*) '    Max. value: ', max_esft_specpts
            write(EU,*) 
     &           '    Increase max. value in gt_esft_parameter_setup'
            return 1            
         endif
         if (i > 1) then
            if ( DB_E(I - 1)*0.99999d0 > DB_S(I) ) then
               call write_message(r_name, 'ERROR', 
     $             'ESFT '//GN//' data bases not ordered by wavelength')
               write(EU,*) '    ',
     $              DB_E(i - 1), ' > ',DB_S(i)
               return 1
            endif
         endif
      enddo

      RETURN

 101  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
      WRITE(EU,*) '    Parameter to read: ', STR_INP
      RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE READ_ESFT_INP
c *********************************************************************

      SUBROUTINE INTERP_EXTRAPOL_AND_WF 
c     (Constant) extrapolation of ESFT coefficients in case p and/or T
c     outside ESFT data base range

      INTEGER    IDX_P, IDX_T, IDX_P_1, IDX_T_1, IDX_1
      REAL (KND) HV, T, U, YP1

      IDX_P =  MINLOC(DABS(PRESS_DB(1:NPRESS_DB) - ACT_PP),1)
      IDX_T = MINLOC(DABS(TEMP_DB(1:NTEMP_DB) - ACT_TT),1)

      IF( ACT_PP >= PRESS_DB(NPRESS_DB).OR.ACT_PP <= PRESS_DB(1)) THEN
         IF(ACT_TT >= TEMP_DB(NTEMP_DB).OR.ACT_TT <= TEMP_DB(1)) THEN
            ESFT_COEFF(I,K,J,JJ) = ESFT_DB(IDX_P,IDX_T)
            RETURN
         END IF

         IDX_1 = IDX_T + INT(SIGN(1.0D0,(ACT_TT - TEMP_DB(IDX_T))))

         HV = (ESFT_DB(IDX_P,IDX_1) - ESFT_DB(IDX_P,IDX_T))/
     $        (TEMP_DB(IDX_1) - TEMP_DB(IDX_T))
         ESFT_COEFF(I,K,J,JJ) = ESFT_DB(IDX_P,IDX_T) + 
     $        HV*(ACT_TT - TEMP_DB(IDX_T))

         IF( WF_FLAG ) D_DT(I,K,J,JJ) = HV
         RETURN
      ENDIF

      IF(ACT_TT >= TEMP_DB(NTEMP_DB).OR.ACT_TT <= TEMP_DB(1)) THEN
         IDX_P = MINLOC(DABS(PRESS_DB(1:NPRESS_DB) - ACT_PP),1)
         IDX_1 = IDX_P + INT(SIGN(1.0D0,(ACT_PP - PRESS_DB(IDX_P))))

         HV = (ESFT_DB(IDX_1,IDX_T) - ESFT_DB(IDX_P,IDX_T))/
     $        (PRESS_DB(IDX_1) - PRESS_DB(IDX_P))

         ESFT_COEFF(I,K,J,JJ) = ESFT_DB(IDX_P,IDX_T) + 
     $        HV*(ACT_PP - PRESS_DB(IDX_P))

         IF( WF_FLAG ) D_DP(I,K,J,JJ) = HV
         RETURN
      ENDIF

      IDX_T_1 = IDX_T + INT(SIGN(1.0D0,(ACT_TT - TEMP_DB(IDX_T))))
      IDX_P_1 = IDX_P + INT(SIGN(1.0D0,(ACT_PP - PRESS_DB(IDX_P))))

c     Interpolation
c     -------------
      T = (ACT_PP - PRESS_DB(IDX_P_1))/
     $     (PRESS_DB(IDX_P) - PRESS_DB(IDX_P_1)) 
      U = (ACT_TT - TEMP_DB(IDX_T_1))/
     $     (TEMP_DB(IDX_T) - TEMP_DB(IDX_T_1))
      
      ESFT_COEFF(I,K,J,JJ) = T*U*ESFT_DB(IDX_P,IDX_T) +
     $     (1.0D0 - T)*(1.0D0 - U)*ESFT_DB(IDX_P_1,IDX_T_1 ) 
     $     + T*(1.0 - U)*ESFT_DB(IDX_P,IDX_T_1) 
     $     + (1.0 - T)*U*ESFT_DB(IDX_P_1,IDX_T)

c     Calculation for Weighting function
c     ==================================
      IF( WF_FLAG  ) THEN
         YP1 = T*ESFT_DB(IDX_P,IDX_T)  - T*ESFT_DB(IDX_P,IDX_T_1 )+ 
     $        (1.0 - T)*ESFT_DB(IDX_P_1,IDX_T) - 
     $        (1.0 - T)*ESFT_DB(IDX_P_1,IDX_T_1)
         D_DT(I,K,J,JJ) = YP1/(TEMP_DB(IDX_T) - TEMP_DB(IDX_T_1))

         YP1 = U*ESFT_DB(IDX_P,IDX_T ) - U*ESFT_DB(IDX_P_1,IDX_T)
     $        + (1.0 - U)*ESFT_DB(IDX_P,IDX_T_1 ) - 
     $        (1.0 - U)*ESFT_DB(IDX_P_1,IDX_T_1 )        
         D_DP(I,K,J,JJ) = YP1/(PRESS_DB(IDX_P) - PRESS_DB(IDX_P_1))
      ENDIF

      RETURN
      END SUBROUTINE INTERP_EXTRAPOL_AND_WF 
c ***********************************************************************

      SUBROUTINE READ_DB_FORMAT (*)
      CHARACTER (LEN=33)  EXPECTED_STRING
      INTEGER   I_1, I_2, II, NWL, IPRE, ITEM, IWL, IWEI, NPTS, CU
      REAL (KND) WL_1, WL_2, LAM1, LAM2
      
      FILENAME = TRIM(ESFT_DB_PATH)//'esft_'//
     $     TRIM(DB_NAME(LL))//'.db'
      IF(LOG_LEVEL(1) >= N_SSO) 
     $     WRITE(LUNIT,*) ' *** Reading: '//TRIM(FILENAME)
         
      OPEN(NEWUNIT = CU, FILE = FILENAME, STATUS='OLD')
c     skip header
c     ===========
      do ii = 1, 11 
         read(cu,'(a)') string 
      enddo
c     wavelength
c     ==========
      READ( CU,'(A)' ) STRING
      READ( CU,* ) NWL
      IF( NWL .NE. DB_NP(LL) ) GO TO 193
      READ( CU,* ) IWL, LAM1, LAM2
      WL_1 = LAM1 ;  WL_2 = DB_S(LL)
      IF( DABS(WL_1 - WL_2) > 0.000001D0 ) GO TO 194
      DO II = 2, DB_NP(LL) - 1 
         READ(CU,*) 
      ENDDO
      READ( CU,* ) IWL, LAM1, LAM2
      WL_1 = LAM2 ;  WL_2 = DB_E(LL)
      IF( DABS(WL_1 - WL_2) > 0.000001D0 ) GO TO 194
c     pressure
c     ========
      READ( CU,'(A)' ) STRING
      EXPECTED_STRING = 'Pressures [hPa]:'
      IF( TRIM(STRING).NE.TRIM(EXPECTED_STRING) ) GOTO 196
      READ( CU,* ) IPRE
      I_1 =  IPRE ;  I_2 = DB_NPRESS(LL)
      IF(I_1.NE.I_2) GO TO 197
      DO II = 1, DB_NPRESS(LL)
         READ( CU,* ) ITEM, PRESS_DB(II)
      ENDDO
      DO II = 2, DB_NPRESS(LL)
         IF (PRESS_DB(II - 1) > PRESS_DB(II)) GOTO 198
      ENDDO            
c     temperature
c     ===========
      READ( CU,'(A)' ) STRING
      EXPECTED_STRING = 'Temperatures [K]:'
      IF( TRIM(STRING).NE.TRIM(EXPECTED_STRING) ) GOTO 196
      READ( CU,* ) ITEM
      I_1 =  ITEM ;  I_2 = DB_NTEMP(LL)
      IF(I_1.NE.I_2) GO TO 197
      DO II = 1, DB_NTEMP(LL)
         READ( cu,* ) ITEM, TEMP_DB(II)
      ENDDO
      DO II = 2, DB_NTEMP(LL)
         IF(TEMP_DB(II - 1) > TEMP_DB(II)) GOTO 198 
      ENDDO
c     weights
c     =======
      READ( CU,'(A)' ) STRING
      EXPECTED_STRING = 'Weights [-]:'
      IF( TRIM(STRING).NE.TRIM(EXPECTED_STRING) ) GOTO 196
      READ( CU,* ) IWEI
      I_1 =  IWEI ;  I_2 = DB_NCOEFF(LL)
      IF(I_1.NE.I_2) GO TO 197
      DO II = 1, DB_NCOEFF(LL)
         READ( CU,* ) IWEI, DB_WEIGHTS(II)
      ENDDO               
c     coefficients
c     ============
      READ( CU,'(A)' ) STRING
      EXPECTED_STRING = 'ESFT coefficients [cm2/molecule]:'
      IF( TRIM(STRING).NE.TRIM(EXPECTED_STRING) ) GOTO 196
      READ( CU,'(A)' ) STRING
      READ(CU, *) NPTS
      IF( DB_NP(LL) > MAX_ESFT_SPECPTS) GO TO 205
      DO II = 1, DB_NP(LL)
         DO J = 1, DB_NPRESS(LL)
            DO K = 1, DB_NTEMP(LL)
               READ(CU,*) NWL, IPRE, ITEM,
     &              (DB_COEFF(II,J,K,L), L = 1,DB_NCOEFF(LL))
            ENDDO
         ENDDO
      ENDDO               
c     End of file
c     ===========
      read( cu,'(a)' ) string
      EXPECTED_STRING = 'EOF'
      IF( TRIM(STRING).NE.TRIM(EXPECTED_STRING) ) GOTO 196
      CLOSE( cu )
      
      RETURN
      
 193  call write_message(r_name, 'ERROR', 
     $     'Unexpected # of wavelength in ESFT data base')
      write(EU,*) '    Read:     ', nwl
      write(EU,*) '    Expected: ', DB_NP(LL)
      RETURN 1
      
 194  call write_message(r_name, 'ERROR', 
     $     'Unexpected wavelength in ESFT data base')
      write(EU,*) '    Read:     ', WL_1
      write(EU,*) '    Expected: ', WL_2
      return 1
      
 196  call write_message(r_name, 'ERROR', 
     $     'String expected: '//TRIM(EXPECTED_STRING))
      write(EU,*) '    Read:     ', TRIM(STRING)
      return 1
      
 197  call write_message(r_name, 'ERROR',
     $     'Wrong number of '//TRIM(EXPECTED_STRING))
      write(EU,*) '    Read:     ', I_1
      write(EU,*) '    Expected: ', I_2
      return 1
      
 198  call write_message(r_name, 'ERROR', TRIM(EXPECTED_STRING)
     $     //' must increase in ESFT data base')
      return 1
      
 205  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $     'Too many spec. points in ESFT data base')
      WRITE(LUNIT,*) '    ESFT data base name:',TRIM(DB_NAME(LL))
      WRITE(LUNIT,*) '    Max. value (max_esft_specpts): ',
     &     MAX_ESFT_SPECPTS
      RETURN 1
      
      END SUBROUTINE READ_DB_FORMAT
      
C***********************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS
       
      IF(ALLOCATED(ESFT_WEIGHTS)) DEALLOCATE(ESFT_WEIGHTS, 
     $     ESFT_MOL_OVERLAP_IDX2, ESFT_MOL_OVERLAP_IDX1, ESFT_NCOEFF,
     $     ESFT_OVERLAP_MODE, DO_ESFT_SPEC_MOL, ESFT_ALPHA, ESFT_COEFF)
      IF(ALLOCATED(D_DP)) DEALLOCATE(D_DP, D_DT)

      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS

c ***********************************************************************
      END 
c ***********************************************************************


