C********************************************************************C
C
C	SCIATRAN - Version 4.6 - Last modification: 08 Apr 2022
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_HI_WD(POS, DEPOL, DEP_W, LOW, LOA, OSM,
     $     UDB_A, UDB_W, *)
c     ... Call in "dom_spherical.f"

      USE AEROSOLS,    ONLY: N_AER_WAVS, AER_WAVS, APH_VS,
     $                       Plus_mi, SCI => GT_SCAT_IDX, 
     $                       Greek_aer, NAW_MI, AW_MI, APH_VS_mi, 
     $                       Greek_Aer_mi, AC_MI, Mie_layer, AP_Mie,
     $                       Imi=>Nga_mi, Idi=>Nga_db, LA_N, LA_S,
     $                       Dwr_mi, N_dwr, APH_DER, D_Greek, LA_E,
     $                       IDX_AER_db, IDX_AER_mi, IDX_AER
      USE ALT_INPUT,   ONLY: ALIN_ACT, ALIN_RAY_EC
      USE BRDF,        ONLY: Nwl_BRF, WL_BRF, BRF_GG_WL, BRF_GS_WL,
     $                       Nwl_veg, WL_veg, Alb_svk, N_svk, Wgr_svk,
     $                       Ker_cof => IVG_cof, Nder, Der_GG_WL, 
     $                       Der_GS_WL, NIce, WL_refi_idx, Refi_idx,
     $                       Ref_ip, Mh_BRF
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, Greek_cld,
     $                       NCW => N_CLD_NETWAVS, CLW => CLD_NETWAVS, 
     $                       C_P_M => CLD_PHAS_MOMS, CLD_F_DM, SCAT_MAT
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE CLD_INP,     ONLY: Deriv_Phc, Ref_DB, Ref_ice, MLCS,
     $                       N_ref_db, N_ref_ice, Use_Ydb_SCM, Ydb_ang
      USE GRIDS,       ONLY: NLAYERS, ZZZ, NSOL, XSOL => SOL_GRD
      USE GT_POLYN,    ONLY: XM
      USE HYDROSOLS,   ONLY: SEL_VSC
      USE INTERN_DIMS, ONLY: Nm_ice, Nm_vs, Nlg_Ray, NL_AER, Nlg_Aer, 
     $                       Nlg_ALL_A, NL_CLD, NLG_CLD, Nlg_ALL_C, 
     $                       Ng1_brf, Ng2_brf, Ns_brf, Nh_brf, 
     $                       Nlg_Ray_W, Nm_vs_W, Nvs
      USE LOG_PARS,    ONLY: ERROR_TXT
      USE ST_VARS,     ONLY: DO_AEROSOL, Vec_Sca, RTM_TYPE, SURF_TYPE,
     $                       Pie, NN => NSTRMS, SCATT_MODE
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE, LOW_TYPE, UDB_TYPE, OSM_TYPE
      USE SLAB_2L,     ONLY: TES_2L_SLAB
      USE WATER,       ONLY: UWT, Ra_GG_WL, Taw_GG_WL, 
     $                       Rw_GG_WL, Twa_GG_WL, Ra_GS_WL, Taw_GS_WL
      USE WATER_ICE,   ONLY: IWS, N_IW, IWT
      USE WF_NUM,      ONLY: NUM_AER, SCA_or_ABS, PERTURB
c     ------------------------------------------------------------------

      REAL (KND), INTENT(IN) ::   POS, DEPOL, DEP_W
c     ----------------------
      TYPE (LOA_type) :: LOA    ! Optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Optical parameters of water
      TYPE (OSM_type) :: OSM    ! Original scattering matrices      
      TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
c     -------------------------------
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'GT_HI_WD'
      LOGICAL    ERROR
      INTEGER    I, Idr, J, K, L, Ws, Ws1, Wm, Wm1, I_per, I_der, N_db,
     $           Js, Jh, Jg, II, Jmc, IiW
      REAL (KND)  Aveg, DC, Uc, Dm
      REAL (KND), ALLOCATABLE :: Du(:), HH(:)
      REAL (KND), POINTER :: Drm(:), Rdb(:), Php(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
c
c ************************* Rayleigh ****************************
      IF( ALIN_ACT(2) ) THEN
         LOA%RAY_MOMS(1:3,1:Nm_vs) = ALIN_RAY_EC(1:3,1:Nm_vs)
      ELSE
         CALL SET_MOM_RAY( LOA%RAY_MOMS, DEPOL )
      ENDIF      
c      WRITE(*,*) ' Rayleigh depol:  ', DEPOL
c      DO I = 1,3
c         WRITE(*,'(6(1x,e15.7))') (LOA%RAY_MOMS(I,J), J=1,Nm_vs)
c      ENDDO
c      CALL COEF_TO_SCATMATR(Nlg_Ray,Nm_vs,LOA%RAY_MOMS,19,'E',
c     $     DIR_OUT//'Test_Ray.dat')
c      STOP
c
c ************************* BRDF surface *************************
c      WRITE(*,*) '  ---- TRIM(UDB_A%LBC) ', TRIM(UDB_A%LBC)
c      WRITE(*,*) '  ---- SURF_TYPE ', SURF_TYPE
c      PAUSE

      IF( TRIM(UDB_A%LBC)=='Bidirect'.OR.SURF_TYPE=='melt_pond') THEN
         IF( Nwl_BRF > 1 ) THEN
            SELECT CASE( TRIM(SURF_TYPE) )
            CASE('snow_artm','white_si')
               CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
               UDB_A%BRF_GG(:,:,:) = (1.d0 - DC)*BRF_GG_WL(:,:,:,Ws) 
     $              + DC*BRF_GG_WL(:,:,:,Ws1) 
               UDB_A%BRF_GS(:,:,:) = (1.d0 - DC)*BRF_GS_WL(:,:,:,Ws) 
     $              + DC*BRF_GS_WL(:,:,:,Ws1)

            CASE('Ross-Li','Ross-Li-snow','snow_fasmar')
c              ... For this option  Nwl_BRF+1 denotes the number 
c                  of "kernels" but not the number of wavelengths
               ALLOCATE( Du(Nwl_BRF+1) )
               CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
               Du(:) = (1.d0 - Dc)*Ker_cof(:,Ws) + Dc*Ker_cof(:,Ws1)
c               WRITE(*,'(2x,*(3x,es12.4))') Du(:)
c               PAUSE
c              ... Contribution of K_vol and K_geo for all harmonics
               UDB_A%BRF_GG(:,:,:) = Du(2)*BRF_GG_WL(:,:,:,1)
     $              + Du(3)*BRF_GG_WL(:,:,:,2)
c              ... Contribution of f_iso in the first harmonic only
               UDB_A%BRF_GG(1:Ng1_brf:Nvs,1:Ng2_brf:Nvs,1) = 
     $              UDB_A%BRF_GG(1:Ng1_brf:Nvs,1:Ng2_brf:Nvs,1) + Du(1)

               UDB_A%BRF_GS(:,:,:) = Du(2)*BRF_GS_WL(:,:,:,1)
     $              + Du(3)*BRF_GS_WL(:,:,:,2)
c              ... Contribution of f_iso in the first harmonic only
               UDB_A%BRF_GS(1:Ng1_brf:Nvs,1:Ns_brf:Nvs,1) = 
     $              UDB_A%BRF_GS(1:Ng1_brf:Nvs,1:Ns_brf:Nvs,1) + Du(1)
c              ... Contribution of K_snw if present             
               IF( Nwl_BRF==3 ) THEN
                  UDB_A%BRF_GG(:,:,:) = UDB_A%BRF_GG(:,:,:)
     $                 + Du(4)*BRF_GG_WL(:,:,:,3)
                  UDB_A%BRF_GS(:,:,:) = UDB_A%BRF_GS(:,:,:)
     $                 + Du(4)*BRF_GS_WL(:,:,:,3)
               ENDIF

            CASE('svk')
               CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
               Aveg = (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
               Ws = 1
               IF( POS >= Wgr_svk ) Ws = Ws + 1
               UDB_A%BRF_GG(:,:,:) = Aveg*N_svk(Ws)*BRF_GG_WL(:,:,:,Ws)
               UDB_A%BRF_GS(:,:,:) = Aveg*N_svk(Ws)*BRF_GS_WL(:,:,:,Ws)

            CASE('rff')
               CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
               UDB_A%BRF_GG(:,:,:) = (1.d0 - DC)*BRF_GG_WL(:,:,:,Ws) 
     $              + DC*BRF_GG_WL(:,:,:,Ws1)
               UDB_A%BRF_GS(:,:,:) = (1.d0 - DC)*BRF_GS_WL(:,:,:,Ws) 
     $              + DC*BRF_GS_WL(:,:,:,Ws1)
c              ,,, Interpolation on solar angles for each harmonic
               DO Js = 1, NSOL
                  IF( XSOL(Js) <= 0.d0 ) CYCLE
                  CALL LIN_SEARCH_NEW(XSOL(Js), Dc, Ws, Ws1, NN, XM)
                  DO Jh = 1, Mh_BRF
                     UDB_A%BRF_GS(:,Js,Jh) = (1.d0 - Dc)
     $                    *UDB_A%BRF_GG(:,Ws,Jh)
     $                    + Dc*UDB_A%BRF_GG(:,Ws1,Jh)
                  ENDDO  ! over Fourier harmonics (Jh)
               ENDDO  ! over solar angles (Js)

            CASE DEFAULT
               CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
               UDB_A%BRF_GG(:,:,:) = (1.d0 - DC)*BRF_GG_WL(:,:,:,Ws) 
     $              + DC*BRF_GG_WL(:,:,:,Ws1)
               UDB_A%BRF_GS(:,:,:) = (1.d0 - DC)*BRF_GS_WL(:,:,:,Ws) 
     $              + DC*BRF_GS_WL(:,:,:,Ws1)
c               WRITE(*,*) UDB_A%BRF_GS(:,1,1)
c               WRITE(*,*) '   SUM( BRF_GG_WL ) = ', SUM( BRF_GG_WL )
c               PAUSE '  Pause in "gt_hi_wd" '
c              ... Nder > 0 if derivatives in the case "prosail" 
c                  should be calculated
               DO I = 1, Nder 
                  UDB_A%Der_GG(:,:,:,I) = 
     $                 (1.d0 - DC)*Der_GG_WL(:,:,:,Ws,I) 
     $                 + DC*Der_GG_WL(:,:,:,Ws1,I)
                  UDB_A%Der_GS(:,:,:,I) = 
     $                 (1.d0 - DC)*Der_GS_WL(:,:,:,Ws,I) 
     $                 + DC*Der_GS_WL(:,:,:,Ws1,I)
               ENDDO

            END SELECT
         ELSE
c           ... BRDF is given at single wavelength
            UDB_A%BRF_GG(:,:,:) = BRF_GG_WL(:,:,:,1)
            UDB_A%BRF_GS(:,:,:) = BRF_GS_WL(:,:,:,1)
         ENDIF
      ENDIF  ! UDB_A%LBC) == 'Bidirect'
c
c ************************* Aerosol *********************************
c
      IF(DO_AEROSOL) THEN
         Call LIN_SEARCH_NEW(POS, DC, Ws, Ws1, N_AER_WAVS, AER_WAVS)

         IF( Plus_mi ) THEN
c        ... Mie-layers and standard aerosol layers
c        ==========================================
c           ... Second dimension of array LOA%AER_MOMS is the number
c               of altitude levels
            Call LIN_SEARCH_NEW(POS, Dm, Wm, Wm1, NAW_MI, AW_MI)
            DO I = 1,NLAYERS
               IF( .NOT.Mie_layer(I) ) THEN
                  LOA%AER_MOMS(:,I,1:Nm_vs) = 
     $                 (1.d0 - DC)*APH_VS(:,IDX_AER_db(I),1:Nm_vs,Ws) 
     $                 + DC*APH_VS(:,IDX_AER_db(I),1:Nm_vs,Ws1)
               ELSE
                  LOA%AER_MOMS(:,I,1:Nm_vs) = 
     $                 (1.d0 - Dm)*APH_VS_mi(:,IDX_AER_mi(I),1:Nm_vs,Wm) 
     $                 + Dm*APH_VS_mi(:,IDX_AER_mi(I),1:Nm_vs,Wm1)
               ENDIF
            ENDDO
c           ... Derivatives of the expansion coefficients in the case
c               of database and mie-mode.
            IF( Dwr_mi ) THEN
               DO Idr = 1, N_dwr
                  DO I = 1,NLAYERS
                     LOA%ADR_MOMS(:,I,:,Idr) =
     $                    (1.d0-Dm)*APH_DER(:,IDX_AER_mi(I),:,Wm ,Idr) 
     $                         + Dm*APH_DER(:,IDX_AER_mi(I),:,Wm1,Idr)
                  ENDDO
               ENDDO
            ENDIF
c           ... Single scattering with original phase matrix will be used
            IF( DO_SSRCA.OR.TRIM(RTM_TYPE)=='spher_scat' ) THEN
c               WRITE(*,*) '    IDX_AER_db and  IDX_AER_mi'
c               DO I = 1,NLAYERS
c                 WRITE(*,'(2x,e15.7,2x,L,2(2x,I3))') ZZZ(I),
c     $                  Mie_layer(I), IDX_AER_db(I), IDX_AER_mi(I)
c               ENDDO
c               WRITE(*,*) '       ---------------------  '
               OSM%AER_ALL = 0.d0
               DO I = 1,NLAYERS
                  IF( .NOT.Mie_layer(I) ) THEN
                     OSM%AER_ALL(1:Idi,I,1:Nm_vs) = (1.d0 - DC)
     $                    *Greek_aer(1:Idi,IDX_AER_db(I),1:Nm_vs,Ws) 
     $                   + DC*Greek_aer(1:Idi,IDX_AER_db(I),1:Nm_vs,Ws1)
                  ELSE
                     OSM%AER_ALL(1:Imi,I,1:Nm_vs) = (1.d0 - Dm)
     $                 *Greek_aer_mi(1:Imi,IDX_AER_mi(I),1:Nm_vs,Wm) 
     $                + Dm*Greek_aer_mi(1:Imi,IDX_AER_mi(I),1:Nm_vs,Wm1)
                  ENDIF
c                  WRITE(*,'(A,2(2x,I3),2(2x,e15.7))') 
c     $                 '  Mie lay for AER_ALL ', I, IDX_AER_mi(I),
c     $                 OSM%AER_ALL(2:3,I,1)
               ENDDO   ! over main altitude levels (I)
c               WRITE(*,*) 
               IF( Dwr_mi ) THEN
                  OSM%DER_ALL = 0.d0
                  DO Idr = 1, N_dwr
                     DO I = 1,NLAYERS
                        IF( .NOT.Mie_layer(I) ) CYCLE
c                        WRITE(*,'(A,2(2x,I3))')
c     $                       '  Mie lay for DER_ALL ', I, IDX_AER_mi(I)
                        OSM%DER_ALL(:,I,:,Idr) =
     $                       (1.d0-Dm)*D_greek(:,IDX_AER_mi(I),:,Wm,Idr) 
     $                       + Dm*D_Greek(:,IDX_AER_mi(I),:,Wm1,Idr)
                     ENDDO
                  ENDDO
               ENDIF
c               WRITE(*,*) ' OSM%DER_ALL in gt_hi_wd:  ', NLG_ALL_A,
c     $              SUM( OSM%DER_ALL(1:NLG_ALL_A,47,1,1) )
            ENDIF

c           ... Numerical perturbation technique is used with respect
c               to phase matrix expansion coefficients
            IF( NUM_AER.AND.TRIM(SCA_or_ABS)=='aer_modr' ) THEN 
c           =======================================================
c              ... In fact NLAYERS is the number of altitude levels
               I_per = 0
               DO I = 1, NLAYERS
                  IF( .NOT.Mie_layer(I) ) CYCLE
                  IF( REAL((PERTURB(I) - 1.d0)) /= 0. ) THEN
                     I_per = I
                     EXIT
                  ENDIF
               ENDDO
               WRITE( *,* ) '  I_per =   ', I_per

               I_der = 1

               IF( I_per > 0 ) THEN
                  Aveg = (PERTURB(I_per) - 1.d0)*AP_Mie(I_per,1)
                  IF( I_per == NLAYERS )   Aveg = Aveg/2.d0
                  IF( I_per == NLAYERS-1 ) Aveg = Aveg*0.65
                  IF( I_per == NLAYERS ) THEN                        
                     LOA%AER_MOMS(:,I_per-1,:) = LOA%AER_MOMS(:,I_per,:)
     $                    + LOA%ADR_MOMS(:,I_per,:,I_der)*Aveg
                  ELSE
                     LOA%AER_MOMS(:,I_per,:) = LOA%AER_MOMS(:,I_per,:)
     $                    + LOA%ADR_MOMS(:,I_per,:,I_der)*Aveg
                  ENDIF
               ENDIF

               IF( DO_SSRCA.OR.TRIM(RTM_TYPE)=='spher_scat' ) THEN
                  IF( I_per > 0 ) THEN
                     Aveg = (PERTURB(I_per)-1.d0)*AP_Mie(I_per,1)

                     IF( I_per == NLAYERS )    Aveg = Aveg/2.d0
                     IF( I_per == NLAYERS-1 )  Aveg = Aveg*0.65

                     IF( I_per == NLAYERS ) THEN                        
                        OSM%AER_ALL(:,I_per-1,:) =OSM%AER_ALL(:,I_per,:)
     $                       + OSM%DER_ALL(:,I_per,:,I_der)*Aveg
                     ELSE
                        OSM%AER_ALL(:,I_per,:) = OSM%AER_ALL(:,I_per,:)
     $                       + OSM%DER_ALL(:,I_per,:,I_der)*Aveg
                     ENDIF
                  ENDIF

               ENDIF
c           =======================================================
            ENDIF  ! NUM_AER (numerical perturbation)
         ELSE
c        ... Mie-layers or standard layers only (no mix)
c        ===============================================
c               WRITE(*,*) '  Shape(APH_VS): ', Shape(APH_VS)
c           ... Second dimension of array LOA%AER_MOMS is the number
c               of aerosol mie-layers
            LOA%AER_MOMS(:,:,1:Nm_vs) = (1.d0-DC)*APH_VS(:,:,1:Nm_vs,Ws) 
     $           + DC*APH_VS(:,:,1:Nm_vs,Ws1)
c           ... Derivatives of the expansion coefficients with respect 
c               to PSD function parameters in the case of Mie-mode only.
            IF( Dwr_mi ) THEN
c               WRITE(*,*) '  Shape(APH_DER): ', Shape(APH_DER)
               DO Idr = 1, N_dwr
                  LOA%ADR_MOMS(:,:,:,Idr) =
     $                 (1.d0-DC)*APH_DER(:,:,:,Ws ,Idr) 
     $                      + DC*APH_DER(:,:,:,Ws1,Idr)
               ENDDO
            ENDIF
c            WRITE(*,*) '   "gt_hi_wd " RTM_TYPE = ', TRIM(RTM_TYPE)
c            WRITE(*,*) '    DO_SSRCA               ', DO_SSRCA            
c           ... Single scattering with original phase matrix will be used
            IF( DO_SSRCA .OR. TRIM(RTM_TYPE)=='spher_scat'
     $           .OR.SCATT_MODE=='ss' ) THEN
               OSM%AER_ALL(:,:,1:Nm_vs) = 
     $              (1.d0 - DC)*Greek_aer(:,:,1:Nm_vs,Ws) 
     $                     + DC*Greek_aer(:,:,1:Nm_vs,Ws1)
c               WRITE(*,*) '   Mie-layers only  LA_N = ', LA_N
c               DO I = 1, LA_N
c                  WRITE(*,'(A,2x,I3,*(2x,e15.7))') 
c     $                 '  Mie lay for "mi" only ', I, 
c     $                 SUM( OSM%AER_ALL(:,I,1) )
c               ENDDO
               IF( Dwr_mi ) THEN
c                  WRITE(*,*) '  Shape(D_Greek): ', Shape(D_Greek)
                  DO Idr = 1, N_dwr
                     OSM%DER_ALL(:,:,:,Idr) =
     $                    (1.d0-DC)*D_Greek(:,:,:,Ws ,Idr) 
     $                         + DC*D_Greek(:,:,:,Ws1,Idr)
                  ENDDO
               ENDIF
            ENDIF  ! Single scattering correction or 'spher_scat'
         ENDIF  ! Mie-layers only
      ENDIF  ! DO_AEROSOL
c
c ************************* Clouds *********************************
c
      IF(DO_CLOUD) THEN
         Uc = 1.d0 - LOA%DP_CLD
         DO J = 1, N_CLD_LAYERS   
c           ... Interpolation of truncated phase matrix expansion 
c                coefficients on an actual wavelength 
            LOA%CLD_MOMS(:,J,:,:,1:Nm_vs) 
     $           = Uc*C_P_M(LOA%Cil,:,J,:,:,1:Nm_vs) 
     $           + LOA%DP_CLD*C_P_M(LOA%Cir,:,J,:,:,1:Nm_vs)
c           ... Interpolation of D-M factor  on an actual wavelength
            LOA%DM_CLD(J,:,:) = Uc*CLD_F_DM(LOA%Cil,J,:,:) 
     $           + LOA%DP_CLD*CLD_F_DM(LOA%Cir,J,:,:)
         ENDDO

         IF( Deriv_Phc ) THEN
            DO J = 1, N_CLD_LAYERS   
c              ... Derivatives of D-M factor with respect to
c                  effective radius 
c                  LOA%DM_CLD will be used in "set_output_wf.f"
               DO Jmc = 1, Nm_ice
                  N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
                  Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
                  Drm => LOA%DM_CLD(J,:,Jmc)
                  ALLOCATE( HH(N_db) )
                  HH(1) = (Drm(2) - Drm(1))/(Rdb(2) - Rdb(1))
                  DO K = 2,N_db-1
                     HH(K) = (Drm(K+1) - Drm(K))/(Rdb(K+1) - Rdb(K))
                  ENDDO
                  HH(N_db) = HH(N_db-1)
                  LOA%DM_CLD(J,:,Jmc) = HH(:)
                  DEALLOCATE( HH )
               ENDDO  ! over water/ice phase (Jmc)
            ENDDO  ! over cloud layers (J)
         ENDIF  ! Deriv_Phc

         IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN 
c           ... Interpolation of Greek symbols to an actual wavelength
c            WRITE(*,*) '  SHAPE(OSM%CLD_ALL) ',SHAPE(OSM%CLD_ALL) 
c            WRITE(*,*) '  SHAPE(Greek_cld)   ',SHAPE(Greek_cld)
c            WRITE(*,*) '        ======================     '
c            PAUSE
            DO J = 1, N_CLD_LAYERS  
               OSM%CLD_ALL(:,J,:,:,1:Nm_vs) = 
     $              Uc*Greek_cld(LOA%Cil,:,J,:,:,1:Nm_vs)
     $              + LOA%DP_CLD*Greek_cld(LOA%Cir,:,J,:,:,1:Nm_vs)
            ENDDO
            IF( Use_Ydb_SCM ) THEN
c           ... Interpolate scattering matrix for Yang DB:               
               DO J = 1, N_CLD_LAYERS  
                  OSM%CLD_SCM(:,J,:,:,1:Nm_vs) = 
     $                 Uc*SCAT_MAT(LOA%Cil,:,J,:,:,1:Nm_vs)
     $                 + LOA%DP_CLD*SCAT_MAT(LOA%Cir,:,J,:,:,1:Nm_vs)
               ENDDO
            ENDIF
            
            IF( Deriv_phc ) THEN
c              ... Derivatives of expansion coefficients with respect to
c                  effective radius  
               IF( Use_Ydb_SCM ) THEN ; II = Ydb_ang
               ELSE                   ; II = NLG_ALL_C
               ENDIF
               DO J = 1, N_CLD_LAYERS   
                  DO Jmc = 1, Nm_ice
                     N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
                     Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
                     DO I = 1, II
                        DO L = 1,Nm_vs
                           IF( Use_Ydb_SCM ) THEN 
                              Php =>     OSM%CLD_SCM(I,J,:,Jmc,L) 
                              Drm => OSM%Der_CLD_SCM(I,J,:,Jmc,L)
                           ELSE
                              Php =>     OSM%CLD_ALL(I,J,:,Jmc,L) 
                              Drm => OSM%Der_CLD_ALL(I,J,:,Jmc,L)
                           ENDIF
                           Drm(1) = (Php(2) - Php(1))/(Rdb(2) - Rdb(1))
                           DO K = 2,N_db-1
                              Drm(K) = (Php(K+1) - Php(K))
     $                             /(Rdb(K+1) - Rdb(K))
                           ENDDO ! over R_eff in database (K)
                           Drm(N_db) = (Php(N_db) - Php(N_db-1))
     $                          /(Rdb(N_db) - Rdb(N_db-1))
                        ENDDO   ! over elements of scattering matrix (L)
                     ENDDO  ! over expansion coefficients (I)
                  ENDDO  ! over water/ice phase (Jmc)
               ENDDO  ! over cloud layers (J)               
            ENDIF  ! Deriv_Phc
         ENDIF  ! DO_SSRCA or 'spher_scat'
      ENDIF  ! DO_CLOUD

      IF( .NOT.UWT) THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN    !  There is no radiative transfer in water
      ENDIF
      
c ************************* Water *************************************
      CALL SET_MOM_RAY( LOW%RAY_MOMS_W, DEP_W )
c      WRITE(*,*) ' Water  ', DEP_W
c      WRITE(*,*) LOW%RAY_MOMS_W(:,:)
c     WRITE(*,*) '       Ray_moms_w is set to ray_moms'
c      LOW%RAY_MOMS_W = LOA%RAY_MOMS

c     ... Interpolation of the hydrosol expansion coefficients 
      IiW = 1

c      WRITE(*,*) '  Shape( LOW%HSOL_MOMS )  ' , Shape( LOW%HSOL_MOMS )
c      WRITE(*,*) '   IWS(1)%N_HPC = ', IWS(1)%N_HPC
      
      DO I = 1, IWS(1)%N_HPC
         Call LIN_SEARCH(POS, DC, Ws, IWS(1)%N_HS_WL, IWS(1)%HS_WL,
     $        ERROR)
         IF(ERROR) GOTO 202
         LOW%HSOL_MOMS(:,:,1:Nm_vs,I,IiW) = 
     $        (1.d0 - DC)*IWS(1)%HSOL_PHAS_VS(:,:,1:Nm_vs,Ws,I) 
     $        + DC*IWS(1)%HSOL_PHAS_VS(:,:,1:Nm_vs,Ws+1,I)
      ENDDO  ! over inclusions in ice (I)         
      IF( IWT ) IiW = IiW + 1
      
      DO I = 1, IWS(2)%N_HPC
         SELECT CASE( SEL_VSC(I) )
         CASE('Kos') ;  LOW%HSOL_MOMS(:,:,1:Nm_vs,I,IiW) =
     $        IWS(2)%HSOL_PHAS_VS(:,:,1:Nm_vs,1,I)
         CASE('Kol') ;  LOW%HSOL_MOMS(:,:,1:Nm_vs,I,IiW) = 
     $           IWS(2)%HSOL_PHAS_VS(:,:,1:Nm_vs,1,I)
         CASE('Use')
            Call LIN_SEARCH
     $           (POS, DC, Ws, IWS(2)%N_HS_WL, IWS(2)%HS_WL, ERROR)
            IF(ERROR) GOTO 202
            LOW%HSOL_MOMS(:,:,1:Nm_vs,I,IiW) = 
     $           (1.D0 - DC)*IWS(2)%HSOL_PHAS_VS(:,:,1:Nm_vs,Ws,I) 
     $           + DC*IWS(2)%HSOL_PHAS_VS(:,:,1:Nm_vs,Ws+1,I)
         END SELECT
      ENDDO  ! over hydrosol components (I)
c      ENDIF

      IF( TRIM(UDB_A%LBC) == 'Flat water' ) THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN 
      ENDIF
c     ... We don't need the reflection and transmission matrixes 
c         for the case "Flat water" 
      IF( Nwl_BRF > 1 ) THEN
         CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_BRF, WL_BRF)

         UDB_A%Ra_GG = (1.d0 - DC)*Ra_GG_WL(:,:,:,Ws)
     $        + DC*Ra_GG_WL(:,:,:,Ws1)
         UDB_A%Ra_GS = (1.d0 - DC)*Ra_GS_WL(:,:,:,Ws)
     $        + DC*Ra_GS_WL(:,:,:,Ws1)
         UDB_A%Taw_GG = (1.d0 - DC)*Taw_GG_WL(:,:,:,Ws)
     $        + DC*Taw_GG_WL(:,:,:,Ws1)
         UDB_A%Taw_GS = (1.d0 - DC)*Taw_GS_WL(:,:,:,Ws)
     $        + DC*Taw_GS_WL(:,:,:,Ws1)
         UDB_W%Rw_GG = (1.d0 - DC)*Rw_GG_WL(:,:,:,Ws)
     $        + DC*Rw_GG_WL(:,:,:,Ws1)
         UDB_W%Twa_GG = (1.d0 - DC)*Twa_GG_WL(:,:,:,Ws)
     $        + DC*Twa_GG_WL(:,:,:,Ws1)
      ELSE
         UDB_A%Ra_GG  => Ra_GG_WL(:,:,:,1)
         UDB_A%Ra_GS  => Ra_GS_WL(:,:,:,1)

         UDB_A%Taw_GG => Taw_GG_WL(:,:,:,1)
         UDB_A%Taw_GS => Taw_GS_WL(:,:,:,1)

         UDB_W%Rw_GG  => Rw_GG_WL(:,:,:,1)
         UDB_W%Twa_GG => Twa_GG_WL(:,:,:,1)
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 201  CALL WRITE_MESSAGE(R_NAME,'ERROR', 
     $     'Lin_search error: Aerosol phase_moms')
      ERROR_TXT = 'Error in '//R_NAME
      GOTO 881

 202  CALL WRITE_MESSAGE(R_NAME,'ERROR', 
     $     'Lin_search error: Hydrosol phase_moms')
      ERROR_TXT = 'Error in '//R_NAME
      GOTO 881

 881  CALL WRITE_MESSAGE(R_NAME, 'ERROR',TRIM(ERROR_TXT) )
      ERROR_TXT = 'Error in '//R_NAME
      RETURN 1

      CONTAINS
c ********************************************************************

      SUBROUTINE SET_MOM_RAY( X_MOMS, DEP_V )
      REAL (KND)   DEP_V, X_MOMS(Nlg_Ray,Nm_vs)

      X_MOMS = 0.d0  ;  Dc = (1.d0 - DEP_V)/(2.d0 + DEP_V)
C     ... Rayleigh phase function moments:
      SELECT CASE (Vec_Sca)
      CASE('scal_rte')
         X_MOMS(1,1) = 1.d0 ; X_MOMS(2,1) = 0.d0 ; X_MOMS(3,1) = Dc
      CASE('vect_rte')
c           first harmonic:         
c        ---==============-----------------------------------------
         X_MOMS(1,1) = 1.d0     ! alpha1
c           second harmonic:
c        ---==============------------------------------------------
         X_MOMS(2,4) = 3.d0*(1.D0 - 2.d0*DEP_V)/(2.D0 + DEP_V) ! alpha4
c           third harmonic:
c        ---==============------------------------------------------
         X_MOMS(3,1) = Dc       ! alpha1
         X_MOMS(3,2) = 6.d0*Dc  ! alpha2
c         X_MOMS(3,5) = DSQRT(6.d0)*Dc ! beta1
c        ... The expansion coefficient beta1  will be used  
c            within the SCIATRAN model with the sign '-' what
c            coincides with the convention of Hovenier, Domke,
c             and van der Mee
         X_MOMS(3,5) = -DSQRT(6.d0)*Dc ! beta1

      END SELECT

c     ... Isotropic scattering for this test calculation
      IF( TES_2L_SLAB ) X_MOMS(3,1) = 0.d0

c     CALL COEF_TO_SCATMATR(3, Nm_vs, X_MOMS(1:3,1:Nm_vs),
c     $     19, 'E', DIR_OUT//'test-1.dat')

      END SUBROUTINE SET_MOM_RAY
      END
C *********************************************************************
