c********************************************************************C
c
c       SCIATRAN - Version 4.5 - Last modification: 12 Nov 2021
c       Copyright (C) - Institute of Remote Sensing,
c                       University of Bremen, Germany
c
c********************************************************************C
      SUBROUTINE GT_HI_WI(XSOL, *)
c     ... Called  by "rtm_init.f"

      USE GRIDS,       ONLY: NSOL
      USE GT_POLYN,    ONLY: NLEGEN, XM, AG, AG1, AXM, PLM, PLMM, SGN, 
     $                       Plm_G, Plm_L, Plm_S, Dp2_G, Dm2_G, Dp2_L, 
     $                       Dm2_L, XM_W, AG_W, AXM_W, Plm_GW, Plm_LW, 
     $                       Dp2_S, Dm2_S, Plm_Sm, Plm_SW,
     $                       Dp2_GW, Dm2_GW, Dp2_SW, Dm2_SW,
     $                       Dp2_LW, Dm2_LW
      USE INTERN_DIMS, ONLY: Nvs, NNv
      USE IRRAD,       ONLY: Fbe
      USE LOG_PARS,    ONLY: EU, LOG_LEVEL, N_ESO, LUNIT, DIR_OUT
      USE NODES,       ONLY: N_NODE, MJU_NODE, PHI_NODE
      USE RTM_OUTPUT,  ONLY: N_OP
      USE SLAB_2L,     ONLY: Ref_idx, TES_2L_SLAB, NSTR_T1, Fr_01, 
     $                       Fr_12, Fr_20, F_Ref_inc
      USE SPHER_PARS,  ONLY: N_LOS_OUT, ANG_OUT,
     $                       N_LOS_ANG, ANG_SEL_FLAG, JV_S, 
     $                       JV_E, N_AZ_ANG, N_AZ_VAR
      USE ST_VARS,     ONLY: RTM_CORE, Pie, Nh_s, Vec_Sca, RAD_TO_DEG,
     $                       INT_OUT, RAD_FIELD, Interp_RF,
     $                       NSTR => NSTRMS
      USE TYPE_VARS,   ONLY: KND
      USE WATER,       ONLY: UWT, NSTR_W, NLEG_W, NLSA_W, NSTR_T, 
     $                       Ref_ind, Obs_POSITION
c     ------------------------------------------------------------------
c    ... Input parameters      
      REAL (KND)  XSOL(NSOL) 
c     ... Local parameters
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'GT_HI_WI'
      INTEGER  NLSA
      INTEGER  I, J, K, L, M, SIGN_M, NN0, K_MAX, N_GAU
      LOGICAL  VIEW_NZ
      REAL (KND)  Azm, Mu0, Xmu, T1, T2, Zn, Rri_01, Rri_10, C1_tru, 
     $     C1_trd, C2_tru, C2_trd, Rind
      REAL (KND), ALLOCATABLE :: XM_ref(:), XSOL_W(:), XM_T(:), AG_T(:), 
     $     XM_V(:), AG_V(:), CR_10(:), CR_01(:), Hel(:), XM_1(:), 
     $     COLSA(:)
      TARGET ::   XM_V
      REAL (KND), POINTER :: CR(:), CI(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL DEALLOCATE_ARRAYS     

c      VIEW_NZ = .TRUE.
      VIEW_NZ = .FALSE.

      NLSA = N_LOS_ANG
      IF( .NOT.Interp_RF ) NLSA = N_NODE
      ALLOCATE( COLSA(NLSA) )

      IF( TES_2L_SLAB ) THEN
c        ... Exclude direct solar illumination at the top of the medium
c         ----------------------------------------------------------
         Fbe = 0.d0
c        It is assumed that the top of a two-layer slab is illuminated
c        by the diffuse isotropic light with intensity equal to 1 and
c        the incident flux equal to Pie.
c
c        ... Number of streams in the total-reflection region:
c        -----------------------------------------------------
         IF( Ref_idx(1) == 1.d0 .AND. Ref_idx(2) == 1.d0 ) NSTR_T1 = 0
c        ...Relative refractive index of the lower layer to the upper:
c        -------------------------------------------------------------
         Ref_ind = Ref_idx(2)/Ref_idx(1)
         CALL TWO_SLAB_1
      ELSE
c        ... Gauss angles and weights in the atmosphere
         N_GAU = NSTR
         ALLOCATE( XM(NSTR), AG(NSTR) )
c       ... Additional angle corresponding to the zenith direction, with 
c           the ZERO weight is introduced to avoid extrapolation 
c            1) at the air-water interface for line-of-sight angles
c               near and at nadir or zenith directions;
c            2) in the case if the angular interpolation will be 
c               performed instead of SFI technique to obtain intensity 
c               at desired using-defined zenith angles;
c            3) if radiation field should be used for the 
c               initialization of full spherical mode 
c         IF( UWT.OR.INT_OUT ) THEN
         IF( VIEW_NZ.OR.UWT.OR.INT_OUT.OR.
     $        (RAD_FIELD.AND.Interp_RF) ) THEN
            N_GAU = NSTR - 1 ; XM(NSTR) = 1.d0 ; AG(NSTR) = 0.d0
         ENDIF

         CALL GAULEG(0.d0, 1.d0, XM(1:N_GAU), AG(1:N_GAU), N_GAU)

         IF( LOG_LEVEL(1) >= N_ESO ) THEN
            WRITE(*,*) '  Gaussian angles  '
            DO I = 1, NSTR
               WRITE(LUNIT,'(2x,I3,2x,f9.4)') I, DACOS(XM(I))/Pie*180.
            ENDDO         
         ENDIF
c         OPEN( 44, file='./DATA_OUT/XM_AG-8.dat' )
c           DO I = 1, NSTR
c              WRITE(44,'(*(2x,e15.7))') XM(I), AG(I)
c           ENDDO
c         CLOSE( 44 )
      ENDIF

      ALLOCATE( AG1(NSTR), AXM(NSTR), PLM(NLEGEN,NSTR),       ! A
     $     PLMM(NLEGEN,NSTR), Plm_G(Nh_s,NLEGEN,NSTR),        ! A
     $     Plm_L(Nh_s,NLEGEN,NLSA), Plm_S(Nh_s,NLEGEN,NSOL) ) ! A

      K_MAX = Nlegen
      IF( UWT )  K_MAX = MAX(Nleg_W,Nlegen) 
      ALLOCATE( SGN(K_MAX) )
      SGN(:) = (/((-1.0)**(I + 1), I = 1,K_MAX)/)
      ALLOCATE( Plm_Sm(Nh_s,NLEGEN,NSOL) )
c     ... Auxiliary weights and products
      AXM(:) = AG(:)*XM(:)  ;  AG1(:) = AG(:)*0.5d0

      IF( .NOT.Interp_RF ) THEN
         COLSA(:) = MJU_NODE(:)
      ELSE
         SELECT CASE( ANG_SEL_FLAG )
         CASE( 'one' )
            COLSA(:) = ANG_OUT(2,1:NLSA)
         CASE( 'all' )
            COLSA(:) = ANG_OUT(2,1:NLSA*N_AZ_ANG:N_AZ_ANG)
         END SELECT
      ENDIF

c     ... Preliminary for the vector case :
c         --------------------------------
c         the cosines of line-of-sight angles 
c         will be used as negative for the reflected and 
c         as positive for the transmitted radiation, i.e. opposite 
c         to the scalar case.
c          --------------------------------------------------------
c         | This means that in the vector case we define the polar |
c         | angle and solar zenith angle relative to downward      |
c         | vertical tau-axis.                                     |
c          --------------------------------------------------------

c      WRITE(*,*) '  SZA as inp:', DACOS(XSOL(:))*RAD_TO_DEG
c      WRITE(*,*) '  LOS as iput :', 
c     $     DACOS(ANG_OUT(2,1:NLSA*N_AZ_ANG:N_AZ_ANG))*RAD_TO_DEG

      IF( Vec_Sca == 'vect_rte' ) THEN 
         COLSA(:) = -COLSA(:) ; XSOL(:) = -XSOL(:) ! Sign Mu Mu0 Vec 1
      ENDIF

      IF (LOG_LEVEL(1) >= N_ESO) CALL WRITE_ANGLES
c      CALL WRITE_ANGLES
c      STOP
      
c     Functions Plm_(G,L,S) are defined according
c          to Eq.~(P-lm) in STB
      DO M = 1, Nh_s
c        ... Calculation of the Legendre polynomials for: 
c                         the Gauss angles,
         CALL WIGNER_PLG( XM,  NSTR, M-1, Nlegen, Plm_G(M,:,:) )
c        ...              the line-of-sight angles,
         CALL WIGNER_PLG( COLSA, NLSA, M-1, Nlegen, Plm_L(M,:,:) )
c        ...              the solar zenith angles
         CALL WIGNER_PLG( XSOL, NSOL, M-1, Nlegen, Plm_S(M,:,:) )
c        ...              if specular reflection takes place
         CALL WIGNER_PLG( -XSOL, NSOL, M-1, Nlegen, Plm_Sm(M,:,:) )
      ENDDO
c      DO I = 1, Nlegen
c         WRITE(*,'(1x,I3,*(2x,e15.7))')
c     $        I, Plm_L(1,I,1), Plm_L(2,I,1), Plm_L(3,I,1)
c      ENDDO      
      IF( UWT ) THEN
c        ... Preliminary it is used constant refractive index of water 
         K_MAX = MAX(NLEG_W,Nh_s)   ! Preliminary

         IF( TES_2L_SLAB ) CALL TWO_SLAB_2

c        ... In the case of specular reflection full number of streams
c            consist from NSTR (used for the atmosphere) in the refraction
c            region, plus NSTR_T - angles in the total-reflection region.
         IF( Ref_ind < 1.01d0 ) NSTR_T = 0
            
c        ... In the current version NSTR_W (in the Fresnel cone) is set 
c            to equal NSTR (see gt_settings_check.f)
         NSTR_W = NSTR_W + NSTR_T
         If(Allocated(XM_w)) DeAllocate(XM_W,AG_W, AXM_W,Plm_GW,Plm_SW)
         ALLOCATE( XM_W(NSTR_W), AG_W(NSTR_W), AXM_W(NSTR_W), 
     $        Plm_GW(Nh_s,K_MAX,NSTR_W), Plm_SW(Nh_s,K_MAX,NSOL) ) ! A

c        ... If specular reflection is used (calm water surface)
         ALLOCATE( XSOL_W(NSOL) )
         XSOL_W(:) = DSQRT( XSOL(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind
         XSOL_W = SIGN( XSOL_W,XSOL )

         IF (LOG_LEVEL(1) >= N_ESO) THEN 
            WRITE(*,*) '  Solar angles in water:'
            DO I = 1,NSOL
               WRITE(LUNIT,'(1x,I3,2x,F8.3,2x,E12.4)') 
     $              I, DACOS( XSOL_W(I) )/Pie*180., XSOL_W(I)
            ENDDO
         ENDIF
         
c        ... Gauss nodes and weights in the total-reflection region
         IF( Ref_ind > 1.01d0 ) THEN
            ALLOCATE( XM_T(NSTR_T), AG_T(NSTR_T) )            
            C2_tru = DSQRT( Ref_ind**2 - 1.d0 )/Ref_ind
            IF (LOG_LEVEL(1) >= N_ESO) THEN 
               WRITE(*,*) '   Total-reflection angle:'
               WRITE(*,'(A,2x,f8.3)') '   ', DACOS(C2_tru)/Pie*180.
            ENDIF
c           .....................
            CALL GAULEG(0.d0, 1.d0, XM_T, AG_T, NSTR_T)
c        ... Transform to the interval [0,C2_tru]
            XM_W(1:NSTR_T) = XM_T(:)*C2_tru
            AG_W(1:NSTR_T) = AG_T(:)*C2_tru
         ENDIF

c        ... Gauss nodes and weights in the refraction region according
c            to assumption used by Jin and Stamnes
         XM_W(NSTR_T+1:NSTR_W) = 
     $        DSQRT( XM(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind

         IF( XM_W(NSTR_W) > 1.0d0 ) XM_W(NSTR_W) = 1.0d0

         AG_W(NSTR_T+1:NSTR_W) = AG(:)
     $        *XM(:)/XM_W(NSTR_T+1:NSTR_W)/Ref_ind**2
         AXM_W(:) = AG_W(:)*XM_W(:)
c         WRITE(*,*) '=================================================='
c         WRITE(*,*) '   Sum of Gauss weight '
c         WRITE(*,*) '   Atm   = ', SUM( AG )
c         WRITE(*,*) '   Oce_1 = ', SUM( AG_W(NSTR_T+1:NSTR_W))
c         WRITE(*,*) '   Oce_2 = ', SUM( AG_W(1:NSTR_W))
c         WRITE(*,*) '=================================================='
c         WRITE(*,*) '      UWT, TRIM(Obs_POSITION(1)) ',
c     $        TRIM(Obs_POSITION(1))
         if(Allocated(Plm_LW)) DeAllocate (Plm_LW)
         if(Allocated(XM_Ref)) DeAllocate (XM_Ref)
         SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
         CASE('Atmos')  ! Observer in the atmosphere
            NLSA_W = NLSA
            ALLOCATE( XM_Ref(NLSA_W), Plm_LW(Nh_s,K_MAX,NLSA_W) )
c           ... Corresponding line-of-sight angles in the water:
            XM_Ref(:) = DSQRT( COLSA(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind
            XM_Ref = SIGN( XM_Ref,COLSA )
         CASE('Ocean')          ! Observer in the ocean
c           ... The line-of-sight angles define the observer directions
c               in the ocean
            NLSA_W = NLSA
            ALLOCATE( XM_Ref(NLSA_W), Plm_LW(Nh_s,K_MAX,NLSA_W) )
            XM_Ref(:) = COLSA(:)
c            ALLOCATE( Hel(NLSA) ) ; T2 = 1.d0/Ref_ind ; K = 0
c            DO I = 1,NLSA
c              ... Line-of-sight angles in the atmosphere corresponding
c                  to the l-of-s angles in water which are within 
c                  Fresnel cone
c               IF( DABS(COLSA(I)) >= C2_tru ) THEN
c                  K = K + 1
c                  Hel(K) = DSQRT( COLSA(I)**2 + T2**2 - 1.d0 )/T2
c                  Hel(K) = SIGN( Hel(K),COLSA(I) )
c               ENDIF
c            ENDDO
c            NLSA_W = K
c            NLSA_W = NLSA       ! Preliminary to avoide new dimension
c            ALLOCATE( XM_Ref(NLSA_W), Plm_LW(Nh_s,K_MAX,NLSA_W) )
c            XM_Ref(1:K) = Hel(1:K)
c            XM_Ref(K+1:NLSA) = COLSA(K+1:NLSA)
c            DEALLOCATE( Hel )
c            WRITE(*,*) '   Number of L-of-S in atmosphere ', NLSA_W 
c            K = 0
c            DO I = 1, NLSA
c               IF( DABS(COLSA(I)) >= C2_tru ) THEN
c                  K = K + 1
c                  WRITE(*,'(2x,I3,2(2x,F5.2))') 
c     $                 I, DACOS(DABS(COLSA(I)))/Pie*180.,
c     $                 DACOS(DABS(XM_Ref(K)))/Pie*180.  
c               ELSE
c                  WRITE(*,'(2x,I3,2x,F5.2,A,2x,F5.2)') 
c     $                 I, DACOS(DABS(COLSA(I)))/Pie*180.,
c     $                 ' > Crit ', DACOS(C2_tru)/Pie*180.
c               ENDIF
c            ENDDO
         CASE('')  ! Flux calculations
            NLSA_W = NLSA
            ALLOCATE( XM_Ref(NLSA_W), Plm_LW(Nh_s,K_MAX,NLSA_W) )
            XM_Ref(:) = COLSA(:)
         END SELECT

c        ... Legendre polynomials for the Gauss angles: 
         DO M = 1, Nh_s
            CALL WIGNER_PLG(XM_W,   NSTR_W, M-1, K_MAX, Plm_GW(M,:,:))
            CALL WIGNER_PLG(XM_Ref, NLSA_W, M-1, K_MAX, Plm_LW(M,:,:))
            CALL WIGNER_PLG(XSOL_W, NSOL,   M-1, K_MAX, Plm_SW(M,:,:))
         ENDDO
        IF(LOG_LEVEL(1) >= N_ESO) THEN 
           WRITE(LUNIT,*) '      Water refractive index ', Ref_ind
           WRITE(LUNIT,*)
     $        ' ---> "gt_hi_wi.f": Gauss nodes and weights in ocean:'
            WRITE(LUNIT,*) '      Total-reflection region'
            DO I = 1, NSTR_T
               WRITE(LUNIT,'(2x,I3,1x,f8.3,1x,e12.4)') I, 
     $              DACOS(XM_W(I))/Pie*180., AG_W(I)
            ENDDO
            WRITE(LUNIT,*) '      Refraction region'
            DO I = NSTR_T+1, NSTR_W
               WRITE(LUNIT,'(2x,I3,1x,f8.3,1x,e12.4)') I, 
     $              DACOS(XM_W(I))/Pie*180., AG_W(I)
            ENDDO
          ENDIF
      ENDIF ! UWT

c      WRITE(*,*) '   -3'
C     Legendre polynomials for these Gauss nodes
      IF(TRIM(RTM_CORE) /= 'ASYMP') THEN
         CALL WIGNER_PLG( XM, NSTR, 0, NLEGEN, PLM )
         FORALL( J = 1:NLEGEN:2 ) PLMM(J,:) =  PLM(J,:)
         FORALL( J = 2:NLEGEN:2 ) PLMM(J,:) = -PLM(J,:)
      ENDIF

      IF( Vec_Sca == 'scal_rte' ) THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

c *********************************************************************
c     ... Allocate and calculate functions related to vector case
c                ------ Atmosphere  ------
c *********************************************************************
      ALLOCATE( Dp2_G(Nh_s,NLEGEN,NSTR), Dm2_G(Nh_s,NLEGEN,NSTR), ! A
     $     Dp2_L(Nh_s,NLEGEN,NLSA), Dm2_L(Nh_s,NLEGEN,NLSA),      ! A
     $     Dp2_S(Nh_s,NLEGEN,NSOL), Dm2_S(Nh_s,NLEGEN,NSOL) )     ! A

c     Functions Dp2_(G,L) and Dm2_(G,L) are defined according
c          to Eqs.~(R-lm), (T-lm) in STB
c      WRITE(*,*) '   Vector case - atmosphere '
      DO M = 1, Nh_s
         SIGN_M = 1 - 2*MODULO(M-1,2)
         CALL WIGNER_FUNCTION
     $        (XM, NSTR, M-1, Nlegen, Dp2_G(M,:,:), Dm2_G(M,:,:) )
         CALL Dp_Dm( Nlegen, NSTR, Dp2_G(M,:,:), Dm2_G(M,:,:) )
         CALL WIGNER_FUNCTION
     $        (COLSA, NLSA, M-1, Nlegen, Dp2_L(M,:,:), Dm2_L(M,:,:) )
         CALL Dp_Dm( Nlegen, NLSA, Dp2_L(M,:,:), Dm2_L(M,:,:) )
         CALL WIGNER_FUNCTION
     $        (XSOL, NSOL, M-1, Nlegen, Dp2_S(M,:,:), Dm2_S(M,:,:) )
         CALL Dp_Dm( Nlegen, NSOL, Dp2_S(M,:,:), Dm2_S(M,:,:) )
      ENDDO

      IF( .NOT.UWT ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

c *********************************************************************
c     ... Allocate and calculate functions related to vector case
c                ------ Ocean  ------
c *********************************************************************
      ALLOCATE( Dp2_GW(Nh_s,K_MAX,NSTR_W), Dm2_GW(Nh_s,K_MAX,NSTR_W), ! A
     $     Dp2_LW(Nh_s,K_MAX,NLSA_W), Dm2_LW(Nh_s,K_MAX,NLSA_W), ! A
     $     Dp2_SW(Nh_s,K_MAX,NSOL), Dm2_SW(Nh_s,K_MAX,NSOL) ) ! A

c      WRITE(*,*) '   Vector case - ocean K_MAX = ', K_MAX
      DO M = 1, Nh_s
         SIGN_M = 1 - 2*MODULO(M-1,2)
         CALL WIGNER_FUNCTION
     $        (XM_W, NSTR_W, M-1, K_MAX, Dp2_GW(M,:,:), Dm2_GW(M,:,:) )
         CALL Dp_Dm( K_MAX, NSTR_W, Dp2_GW(M,:,:), Dm2_GW(M,:,:) )
         CALL WIGNER_FUNCTION
     $       (XM_Ref, NLSA_W, M-1, K_MAX, Dp2_LW(M,:,:), Dm2_LW(M,:,:) )
         CALL Dp_Dm( K_MAX, NLSA_W, Dp2_LW(M,:,:), Dm2_LW(M,:,:) )
         CALL WIGNER_FUNCTION
     $        (XSOL_W, NSOL, M-1, K_MAX, Dp2_SW(M,:,:), Dm2_SW(M,:,:) )
         CALL Dp_Dm( K_MAX, NSOL, Dp2_SW(M,:,:), Dm2_SW(M,:,:) )
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c **********************************************************************
      SUBROUTINE Dp_Dm(N1,N2,Dp2,Dm2)
      INTEGER    N1, N2
      REAL (KND) Dp2(N1,N2), Dm2(N1,N2)
      Dp2(:,:) = 0.5d0*( Dp2(:,:) + Dm2(:,:) )
      Dm2(:,:) = Dp2(:,:) - Dm2(:,:)
      Dp2(:,:) = SIGN_M*Dp2(:,:) ! Rlm(mu)
      Dm2(:,:) = SIGN_M*Dm2(:,:) ! Tlm(mu)
     
      END SUBROUTINE
      
c  **********************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS

         IF(ALLOCATED(XM)) DEALLOCATE(XM)
         IF(ALLOCATED(AG)) DEALLOCATE(AG)
         IF(ALLOCATED(AG1)) DEALLOCATE(AG1)
         IF(ALLOCATED(AXM)) DEALLOCATE(AXM)
         IF(ALLOCATED(PLM)) DEALLOCATE(PLM)
         IF(ALLOCATED(PLMM)) DEALLOCATE(PLMM)
         IF(ALLOCATED(PLM_G)) DEALLOCATE(PLM_G)
         IF(ALLOCATED(PLM_L)) DEALLOCATE(PLM_L)
         IF(ALLOCATED(PLM_S)) DEALLOCATE(PLM_S)
         IF(ALLOCATED(PLM_SM)) DEALLOCATE(PLM_SM)
         IF(ALLOCATED(SGN)) DEALLOCATE(SGN)
         If(Allocated(Dp2_G)) DeAllocate (Dp2_G, Dm2_G, Dp2_L, Dm2_L, 
     $     Dp2_S, Dm2_S)

      END SUBROUTINE DEALLOCATE_ARRAYS
c **********************************************************************

      SUBROUTINE WRITE_ANGLES
      WRITE(LUNIT,*) '   ------ Mu and Mu0 in "gt_hi_wi" ------- '
      IF(.NOT.INTERP_RF) THEN
         WRITE(LUNIT,*)
     $        ' Line-of-sight zenith (wrt z) and azimuthal angles:'
         DO I = 1,NLSA
c            WRITE(LUNIT,'(1x,I3,2x,F8.3,*(2x,E12.4))') 
c     $           I, 180.-DACOS( COLSA(I) )/Pie*180, PHI_NODE(I)/Pie*180.
            WRITE(LUNIT,'(1x,I3,*(2x,E12.4))') 
     $           I, COLSA(I), MJU_NODE(I), PHI_NODE(I)/Pie*180.
         ENDDO
      ELSE
         WRITE(LUNIT,*)'   Line-of-sight angles, their cosines:'
         DO I = 1,NLSA
            WRITE(LUNIT,'(1x,I3,2x,F8.3,*(2x,E12.4))') 
     $           I, DACOS( COLSA(I) )/Pie*180, COLSA(I)
         ENDDO
      ENDIF
      WRITE(LUNIT,*) '   Solar zenith angles and their cosines:'
      DO I = 1,NSOL
         WRITE(LUNIT,'(1x,I3,2x,F8.3,2x,E12.4)') 
     $        I, DACOS( XSOL(I) )/Pie*180., XSOL(I)
      ENDDO
      WRITE(LUNIT,*) ' '
      
      END SUBROUTINE WRITE_ANGLES

c **********************************************************************
      SUBROUTINE TWO_SLAB_1
c        ... Gauss nodes and weights in the total-reflection region
         NN0 = NSTR
         ALLOCATE( XM_V(NN0), AG_V(NN0) )
         CALL GAULEG(0.d0, 1.d0, XM_V, AG_V, NN0)
c        ... Add additional angles in the total-reflection range
         NSTR = NSTR + NSTR_T1
         NNv = Nvs*NSTR
         ALLOCATE( XM(NSTR), AG(NSTR) )
         
         IF( Ref_idx(1) > 1.01d0 ) THEN            
            Rind = Ref_idx(2)/Ref_idx(1)
            IF( Rind >= 1.d0 ) THEN
               WRITE(*,*) '   Layer 2(lower) is optically thicker'//
     $              ' than layer 1(upper)'
               WRITE(*,*) '    --------------------------------   '
               WRITE(*,*) '    Total reflection in the layer 1:  ' 
               WRITE(*,*) '       at top        at bottom'
               WRITE(*,*) '        yes               no  '
               WRITE(*,*) '    --------------------------------   '
               WRITE(*,*) '        Total-reflection angle :       '
               C1_tru = DSQRT( Ref_idx(1)**2 - 1.d0 )/Ref_idx(1)
               WRITE(*,'(10x,f7.2,10x,A)') DACOS(C1_tru)/Pie*180.,' - '

c              ... Additional Gauss grid to describe transfer in the
c                  total reflection range
               ALLOCATE( XM_T(NSTR_T1), AG_T(NSTR_T1) )
               CALL GAULEG(0.d0, 1.d0, XM_T, AG_T, NSTR_T1)
c              ... Transform to the range [0,C1_tru]
               XM(1:NSTR_T1) = XM_T(:)*C1_tru
               AG(1:NSTR_T1) = AG_T(:)*C1_tru
               DEALLOCATE( XM_T, AG_T )
            ENDIF
            IF( Rind < 1.d0 ) THEN
               WRITE(*,*) '   Layer 2(lower) is optically thinner'//
     $              ' as layer 1(upper)'
               WRITE(*,*) '   There is total reflection from'//
     $              ' the lower layer'
               STOP '   This is not implemented jet'
            ENDIF

         ENDIF
c        ... 
         XM(NSTR_T1+1:NSTR) = 
     $        DSQRT( XM_V(:)**2 + Ref_idx(1)**2 - 1.d0 )/Ref_idx(1)
         AG(NSTR_T1+1:NSTR) = AG_V(:)
     $        *XM_V(:)/XM(NSTR_T1+1:NSTR)/Ref_idx(1)**2
c
c ------->  Test for the reflection coefficients <---------------------
c        Gauss angles in the vacuum (medium 0) 
c                                    - XM_V(1:NN0)       
c        Gauss angle in the refractive region of medium 1 
c                                    - XM_1(1:NN0)        
         ALLOCATE( CR_01(NN0), CR_10(NN0), Hel(NN0), XM_1(NN0) )
c        ... Incident from vacuum on the medium with refractive
c            index Ref_idx(1)
         Rri_01 = 1.d0/Ref_idx(1)

         XM_1(:) = DSQRT( 1.d0 - Rri_01**2*(1.d0 - XM_V(:)**2) ) 
         CR_01(:) = 0.5d0*(
     $        ((XM_1(:)-Rri_01*XM_V(:))/(XM_1(:)+Rri_01*XM_V(:)))**2 +
     $        ((Rri_01*XM_1(:)-XM_V(:))/(Rri_01*XM_1(:)+XM_V(:)))**2 )

         Rri_10 = Ref_idx(1)/1.d0
         Hel(:)  = DSQRT( 1.d0 - Rri_10**2*(1.d0 - XM_1(:)**2) )
         CR_10(:) = 0.5d0*(
     $        ((Hel(:)-Rri_10*XM_1(:))/(Hel(:)+Rri_10*XM_1(:)))**2 +
     $        ((Rri_10*Hel(:)-XM_1(:))/(Rri_10*Hel(:)+XM_1(:)))**2 )
         WRITE(*,*) '   Comparison of the reflection coefficients'
         WRITE(*,'(A,5x,A)') 
     $        '     0   ---->   1','  1   ---->   0'
         DO I = 1,NN0
            WRITE(*,'(2(2x,f7.3,2x,e12.4),3x,e15.7)')
     $           DACOS(XM_V(I))/Pie*180., CR_01(I),
     $           DACOS(XM_1(I))/Pie*180., CR_10(I), CR_01(I)-CR_10(I)
         END DO
         
c ------->  Test for the reflection coefficients <---------------------
         WRITE(*,*) '  NSTR   = ', NSTR
         ALLOCATE(Fr_01(NSTR), Fr_12(NSTR))
         CI => XM_V             !Incident
         CR => XM(NSTR_T1+1:NSTR) !Refracted
         Rri_10 = Ref_idx(1)/1.d0

         Fr_01(1:NSTR_T1) = 1.d0
         Fr_01(NSTR_T1+1:NSTR) = 0.5d0*
     $        (  ((CI - Rri_10*CR)/(CI + Rri_10*CR))**2
     $         + ((CR - Rri_10*CI)/(CR + Rri_10*CI))**2 )

c        ... Reflected flux from the upper interface of a two-layer slab
         F_Ref_inc = 
     $        2.d0*Pie*SUM( Fr_01(NSTR_T1+1:NSTR)*AG_V(:)*XM_V(:) )

         k = NSTR_T1+1
         DO I = 1,NN0
            WRITE(*,'(3(2x,f7.3,2x,e12.4))')
     $           DACOS(XM_V(I))/Pie*180., CR_01(I),
     $           DACOS(XM_1(I))/Pie*180., CR_10(I), 
     $           DACOS(XM(k))/Pie*180., Fr_01(k)
            k = k + 1
         END DO
         DEALLOCATE( Hel )
      END SUBROUTINE TWO_SLAB_1
c **********************************************************************

      SUBROUTINE TWO_SLAB_2
c           ... Second layer with respect to the "vacuum"
         IF( Ref_idx(2) > 1.01d0 ) THEN            
            Rind = Ref_idx(2)/Ref_idx(1)
            IF( Rind >= 1.d0 ) THEN
               WRITE(*,*) '   Layer 2(lower) is optically thicker'//
     $              ' than layer 1(upper)'
               WRITE(*,*) '    --------------------------------   '
               WRITE(*,*) '    Total reflection in the layer 2:  ' 
               WRITE(*,*) '       at top        at bottom'
               WRITE(*,*) '        yes              yes  '
               WRITE(*,*) '    --------------------------------   '
               WRITE(*,*) '        Total-reflection angle :       '
               C2_tru = DSQRT( Rind**2 - 1.d0 )/Rind
               C2_trd = DSQRT( Ref_idx(2)**2 - 1.d0 )/Ref_idx(2)
               WRITE(*,'(10x,f7.2,10x,f7.2)') 
     $              DACOS(C2_tru)/Pie*180.,DACOS(C2_trd)/Pie*180.
c              ... Gauss grid to describe transfer in the
c                  total reflection range {0,C2_tru}
               ALLOCATE( XM_T(NSTR_T1), AG_T(NSTR_T1) )
               CALL GAULEG(0.d0, 1.d0, XM_T, AG_T, NSTR_T1)
            ENDIF
         ENDIF

         STOP '   stop in "IF"'
         ALLOCATE( Fr_20(NSTR_W) )
c        Preliminary
         Fr_20(:) = Fr_01(:)
c        Fr_20(:) = 1.0d0       ! 100% reflection from surface
      END SUBROUTINE TWO_SLAB_2

      END
c ****************************************************************
