C**********************************************************************
C
C       SCIATRAN - Version 4.0 - Last modification: 17 Nov 2018
C       Copyright (C) Institute of Remote Sensing,
C                     University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE GT_HI_WI_SS(XSOL, *)
c     ... Call in "rtm_init.f"

      USE AEROSOLS,    ONLY: NG_aer
      USE CLOUDS,      ONLY: DO_CLOUD, NG_cld
      USE GRIDS,       ONLY: NSOL
      USE GT_POLYN,    ONLY: Plm_ss, Dp2_ss, Dm2_ss
      USE INTERN_DIMS, ONLY: Nlg_ray
      USE LOG_PARS,    ONLY: LOG_LEVEL, N_ESO
      USE NODES,       ONLY: N_NODE, MJU_NODE, PHI_NODE, N_PLUS
      USE OutInf,      ONLY: DIR_OUT, EU, C_UNIT
      USE PS_INTERN,   ONLY: COS_SCAT
      USE SPHER_PARS,  ONLY: N_SOL_ANG, N_LOS_OUT, ANG_OUT,
     $                       N_LOS_ANG, ANG_SEL_FLAG, JV_S, 
     $                       JV_E, N_AZ_ANG, N_AZ_VAR
      USE ST_VARS,     ONLY: DO_AEROSOL, Pie, Vec_Sca, SOL_ANG, 
     $                       RAD_TO_DEG, SSC_RF
      USE TYPE_VARS,   ONLY: KND
      USE VECT_VARS,   ONLY: INC_POL, Rot_r, Rot_l
      USE WATER,       ONLY: INS_VIEW
c     ------------------------------------------------------------------

      REAL (KND), INTENT(IN)::  XSOL(NSOL) 
c     ... Local parameters
      CHARACTER (LEN=11), PARAMETER :: R_NAME = 'GT_HI_WI_SS'
      INTEGER     I, J, Js, K, L, M, N_PLM, NLSA, NLO, N_Phi, NSZA
      INTEGER, ALLOCATABLE :: Jh1(:), Jh2(:)
      REAL (KND)  Azm, Mu0, Xmu, T1, T2, Zn, LSOL(NSOL)
      REAL (KND), ALLOCATABLE ::  COLSA(:), PHI_LOC(:)
c     ==================================================================

c     ... Single scattering correction algorithm is employed in the case
c         'pp_scat' or 'ps_scat':
c        XSOL contains the input solar zenith angles (NSOL = NSZA).
c          "spher_scat":
c        XSOL is calculated in "sol_grid.f90" and contains the  solar 
c        zenith angles according to the "Solar grid" control parameter.
      
      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( SSC_RF ) THEN
         NLSA = N_NODE ;  NLO = N_NODE*NSOL ; N_Phi = 1
         ALLOCATE( Jh1, SOURCE=[(1,I=1,NSOL)])
         ALLOCATE( Jh2, SOURCE=[(N_NODE,I=1,NSOL)] )
         ALLOCATE( PHI_LOC(N_NODE*NSOL) )
         J = 0
         DO Js = 1,NSOL
            DO K = 1, N_NODE ; J = J + 1
               PHI_LOC(J) = PHI_NODE(K)
            ENDDO
         ENDDO
c        ... Solar zenith angles are given  with respect to z-axis
         NSZA = NSOL ;  LSOL(:) = -ABS(XSOL(:))
      ELSE
         NLSA = N_LOS_ANG ; NLO = N_LOS_OUT ; N_Phi = N_AZ_VAR
         ALLOCATE( Jh1, SOURCE=JV_S )
         ALLOCATE( Jh2, SOURCE=JV_E )
         ALLOCATE( PHI_LOC(N_LOS_OUT) ) ; PHI_LOC(:) = ANG_OUT(3,:)
c        ... Solar zenith angles with respect to z-axis
         NSZA = N_SOL_ANG  ;  LSOL(:) = -ABS(XSOL(:))
      ENDIF
      ALLOCATE( COLSA(NLSA) )

      IF( SSC_RF ) THEN
         COLSA(:) = MJU_NODE(:)
c         WRITE(*,*) '  Spherical case, viewing angles ====>'
c         WRITE(*,*) '   COLSA(1)        up   ', COLSA(1)
c         WRITE(*,*) '   COLSA(1+N_PLUS) down ', COLSA(1+N_PLUS)
c         WRITE(*,*) '  ==================='
      ELSE
c        ... Directions of radiation propagation w.r.t. z-axis
         SELECT CASE( ANG_SEL_FLAG )
         CASE( 'one' ) ;  COLSA(:) = ANG_OUT(2,1:NLSA)
         CASE( 'all' ) ;  COLSA(:) = ANG_OUT(2,1:NLSA*N_AZ_ANG:N_AZ_ANG)
c     $           SNGL( 1.0*ANG_OUT(2,1:NLSA*N_AZ_ANG:N_AZ_ANG) )
         END SELECT
!         WRITE(*,*) '  Plane-parallel case, cos. viewing angles ====> '
!         WRITE(*,*) '   COLSA(:)  ', COLSA(:)
!         WRITE(*,*) '  ==================='
      ENDIF

c     ... Preliminary for the vector case :
c         --------------------------------
c         the cosines of line-of-sight angles will be used as negative 
c         for the reflected and as positive for the transmitted radiation,
c         i.e. opposite to the scalar case.
c          --------------------------------------------------------
c         | This means that in the vector case we define the polar |
c         | angle and solar zenith angle with respect to tau-axis. |
c          --------------------------------------------------------
      IF( Vec_Sca == 'vect_rte' ) THEN 
         COLSA(:) = -COLSA(:)  ;  LSOL(:) = -LSOL(:) ! Sign Mu Mu0 Vec 1
      ENDIF

      N_PLM = Nlg_Ray 
      IF( DO_AEROSOL ) N_PLM = MAX( N_PLM,NG_Aer )
      IF( DO_CLOUD )   N_PLM = MAX( N_PLM,NG_cld )
     
      If(Allocated(Plm_ss)) DeAllocate(Plm_ss, COS_SCAT)
      ALLOCATE( Plm_ss(N_PLM,NLO), COS_SCAT(NLO) ) ! A
c     ... Scattering angle solar ---> line-of-sight
      J = 0
      DO I = 1,NSZA           ; T1 = 1.0d0 - LSOL(I)*LSOL(I)
         DO K = Jh1(I),Jh2(I) ; T2 = 1.0d0 - COLSA(K)*COLSA(K)
            DO L = 1, N_Phi   ; J = J + 1
               COS_SCAT(J) = COLSA(K)*LSOL(I)
     $              + DSQRT(T1*T2)*DCOS( PHI_LOC(J) )
            ENDDO
         ENDDO
      ENDDO

      IF( ANG_SEL_FLAG=='one'.AND.LOG_LEVEL(1)>=N_ESO ) THEN
         OPEN(C_UNIT, file=DIR_OUT//'/SCAT_ANG.dat')
         DO I = 1, NSZA
            WRITE(C_UNIT,*) I, DACOS(COS_SCAT(I))/Pie*180.
         ENDDO
         CLOSE(C_UNIT)
      ENDIF

      CALL WIGNER_PLG(COS_SCAT, NLO, 0, N_PLM, Plm_ss )

      IF( Vec_Sca == 'scal_rte' ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      END IF
c     ... Select sign of the viewing angle cosine for calculation 
c         of rotation matrix. 
c         Convention: 
c         1. positive rotation is clockwise when looking in the direction
c            of propagation 
c         2. polar angles are measured from positive z-xis
      IF( SSC_RF ) THEN
         COLSA(:) = -COLSA(:) 
      ELSE
         SELECT CASE( TRIM(INS_VIEW) )
         CASE('up') ; COLSA(:) = -DABS( COLSA(:) ) ! downward radiation
         CASE('dn') ; COLSA(:) =  DABS( COLSA(:) ) ! upward radiation
         END SELECT
      ENDIF
c     ... This is for the case of the unpolarized incident solar light:
c         only certain elements of the rotation matrix are needed.
c         Right rotational matrix can be omitted because incident light
c         is not polarized.
      IF( .NOT.INC_POL ) THEN
         If (Allocated(Rot_l)) DeAllocate(Rot_l)
         ALLOCATE( Rot_l(NLO,4,4) ) ! A
         Rot_l = 0.d0  
c        ... We have accounted for here that the cosine of 
c            the solar angle is always negative when measured
c            w.r.t positive z-axis
         J = 0
         DO I = 1,NSZA            ;  Mu0 = -DABS(LSOL(I))
            DO K = Jh1(I),Jh2(I)  ;  Xmu = COLSA(K)
               DO L = 1,N_Phi     ; J = J + 1  ;  Azm = PHI_LOC(J)
c                 ... Meridional plane coincides with the scattering
c                     plane. No rotation is needed.
                  IF( REAL(Azm)==REAL(Pie).OR.REAL(Azm)==0.0 ) THEN 
                     Rot_l(J,2,2) = 1.d0 ; Rot_l(J,3,2) = 0.d0
                     CYCLE
                  ENDIF
c                 ... Convention about positive rotation angle:
c                     | 1     0        0     |
c                     | 0  cos(2a)  -sin(2a) |
c                     | 0  sin(2a)   cos(2a) |
c                     | 0     0        0     |
c                 ... 0 < phi_sca - phi_inc < pi
                  Zn = -1.d0
c                 ... pi < phi_sca - phi_inc < 2*pi
                  IF( Azm > Pie )  Zn = 1.d0
c                 ... Eq.(Cos2-rot) in STB. 
                  T2 = (Mu0 - Xmu*COS_SCAT(J))
     $                 /DSQRT((1.d0 - Xmu*Xmu)*(1.d0-COS_SCAT(J)**2))
                  IF( REAL(Xmu)==-1.0 ) T2 =  DCOS(Azm)
                  IF( REAL(Xmu)== 1.0 ) T2 = -DCOS(Azm)
c                 ... Eq.(Cos_el) in STB
                  Rot_l(J,2,2) = 2.d0*T2*T2 - 1.d0 
c                 ... Eq.(Sin_el) in STB
                  Rot_l(J,3,2) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
               ENDDO
            ENDDO
         ENDDO

c         WRITE(*,*) '   ======> The case INC_POL = .FALSE. <========='
c         WRITE(*,*) '  Rot. matix (left) Los 1'
c         DO I = 1,4
c            WRITE(*,'(4(2x,e12.4))') (Rot_l(1,I,J), J=1,4)
c         ENDDO
c         WRITE(*,*) '  Rot. matix (left) Los 2'
c         DO I = 1,4
c            WRITE(*,'(4(2x,e12.4))') (Rot_l(2,I,J), J=1,4)
c         ENDDO

         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

c     ... If the incident light is polarized then:
      ALLOCATE( Dp2_ss(N_PLM,NLO), Dm2_ss(N_PLM,NLO),
     $     Rot_r(NLO,4,4), Rot_l(NLO,4,4) ) ! A
      
      CALL WIGNER_FUNCTION
     $     (COS_SCAT, NLO, 2, N_PLM, Dp2_ss(:,:), Dm2_ss(:,:) )

c     ... Rotation matrixes
      Rot_r = 0.d0        ; Rot_l = 0.d0 
      Rot_r(:,1,1) = 1.d0 ; Rot_r(:,4,4) = 1.d0
      Rot_l(:,1,1) = 1.d0 ; Rot_l(:,4,4) = 1.d0
      J = 0
      DO I = 1,NSZA
         Mu0 = LSOL(I)
         DO K = Jh1(I),Jh2(I)
            Xmu = COLSA(K)
            DO L = 1,N_Phi  ;  J = J + 1 
c                   Eto nuzhno pereproverit' !!!
               IF( DABS(Xmu) .EQ. 1.d0 .OR. Mu0 .EQ. 1.d0 .OR. 
     $              PHI_LOC(J) .EQ. 0.d0 ) THEN
                  Rot_r(J,2,2) = 1.d0 ; Rot_l(J,2,2) = 1.d0 
               ELSE
                  T1 = ( Xmu - Mu0*COS_SCAT(J) )
     $                 /DSQRT((1.D0 - Mu0*Mu0)*(1.D0 - COS_SCAT(J)**2))
                  Rot_r(J,2,2) = 2.D0*T1*T1 - 1.D0 
                  Rot_r(J,3,2) = 2.D0*DSQRT( DABS(1.D0 - T1*T1) )*T1
                  
                  T2 = ( Mu0 - Xmu*COS_SCAT(J) )
     $                 /DSQRT((1.D0 - Xmu*Xmu)*(1.D0 - COS_SCAT(J)**2))
                  Rot_l(J,2,2) = 2.d0*T2*T2 - 1.d0 
                  Rot_l(J,3,2) = 2.d0*dsqrt( DABS(1.d0 - T2*T2) )*T2 
               ENDIF
            ENDDO
         ENDDO
      ENDDO
      Rot_r(:,3,3) =  Rot_r(:,2,2)
      Rot_r(:,2,3) = -Rot_r(:,3,2)

      Rot_l(:,3,3) =  Rot_l(:,2,2)
      Rot_l(:,2,3) = -Rot_l(:,3,2)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END
c ****************************************************************
