c************************************************************************
c
c       SCIATRAN - Version 4.6 - Last modification: 28 Jun 2022
c       Copyright (C) - Institute of Remote Sensing,
c                       University of Bremen, Germany
c
c***********************************************************************
c***********************************************************************
c
c     Interface program for SCIATRAN
c
c     This program and all related subroutines have been
c     written by V. Rozanov, A. Rozanov, M. Buchwitz, K. Bramstedt,
c     L. N. Lamsal, J. Meyer-Arnek, M. Vountas
c     University of Bremen, Germany,
c     Institute of Remote Sensing.
c
c     Use and distribution is only permitted when agreed upon
c     by the authors.
c
c     Point of contact:
c
c     A. Rozanov
c     University of Bremen, FB 1
c     Institute of Remote Sensing
c     P.O. Box: 33 04 40
c     Otto_hahn-Alle
c     28334 Bremen
c     Germany
c
c     Tel.:   +49 (0) 421 218 62084
c     Fax.:   +49 (0) 421 218 62070 
c     E-mail: Alexei.Rozanov@iup.physik.uni-bremen.de
c
c********************************************************************C
      SUBROUTINE GT_IFACE (Work_Dir_inp, Status)

      USE ABSORBERS,  ONLY: N_ST_MOL, ST_GAS_IDX, MIX_ABS_START
      USE AEROSOLS,   ONLY: EXA_OUT_FLAG, FACTOR_EXA_S, RT_AG
      USE ALB_PARS,   ONLY: DO_SPECTRAL_ALBEDO
      USE ALT_INPUT,  ONLY: ALIN_FLAG, ALIN_NWL, ALIN_WL, SOUT_FLAG,
     $                      DIR_SOUT
      USE ANISOTROPY, ONLY: DEP_NWS, DEP_WS, DEP_VS,
     $                      DO_DEPOL_WAVELENGTH, DEPOL_INPUT,
     $                      DEPOL_FILENAME
      USE BRDF,       ONLY: Ref_type, Nwl_BRF, WL_BRF, RefW_r, RefW_i
      USE CLOUDS,     ONLY: DO_CLOUD
      USE CLD_RET,    ONLY: NEW_CLD, RET_QUALITY
      USE DATA_RET,   ONLY: RET_FLAG, MOD_READ_FLAG, FACTOR_TPS_COL, 
     $                      MDS, WL_MDS, PRESET_APR_NAME
      USE EMISSION,   ONLY: N_AT
      USE ESFT,       ONLY: DO_ESFT
      USE FLU_LAND,   ONLY: FLU_L, FLU_NAME, FLU_SCAL, NFL, WLF, SPF,
     $                      USE_DGF
      USE GRIDS,      ONLY: NLAYERS_INP, ZZZ_INP, NDEP_INP, DEP_INP,
     $                      Z_TOA, Z_BOA, Z_LEV, Zl_step
      USE INFO,       ONLY: TG_INFO, WF_DB, PAR_INFO
      USE LBL,        ONLY: DO_LBL, SEG_WL, NSEGS_LBL
      USE NODES,      ONLY: NEW_NODES
      Use OutInf,     Only: EU, LUNIT, C_UNIT, N_ESO, FUNIT, WARN_FLAG, 
     $                      LOG_LEVEL, N_RSO, N_SSO, N_DSO, DelayOut,
     $                      OneOutFile, Dir_Out, OutD, OutMode, 
     $     WrtModRes, WrtRetPrf, WrtRetFit, TRC_FLG
      Use Pol_Corr,   Only: PolCorrFlg => Flg, PolCorr
      USE RAMAN,      ONLY: R_GAM, N_RG, INCR_RANGE
      USE RETR_ITER,  ONLY: ITERATION, TG_SCALE_FLAG
      USE RTM_PARS,   ONLY: MAX_ITER, DATA_ITER
      USE ST_VARS,    ONLY: RAD_TO_DEG, GROUND_HEIGHT, CONVOL_FLAG,
     $                      LA_FLAG, NEW_SPEC_GRID, Emi_FLG, DIFB,
     $                      RTM_CORE, HR_NSEG, RET_MODE,
     $                      SEG_NUM_OUT, WF_FLAG, NSEGS, WL_SEGS,
     $                      MSK_RTM, RAD_FIELD, DO_AZIMUTH, DO_RAMAN,
     $                      DO_INTERNAL_CHECK, VHL
      USE SPHER_PARS, ONLY: ANG_OUT, N_LOS_OUT
      USE ST_INPUT,   ONLY: PROF_FILES, BAND_START, BAND_NUM, BAND_STEP, 
     $                      NET_NAME, DEP_NAME, WL_UNITS, WL_CDF
      USE TOOLS,      ONLY: UPPER_LOWER_CASE, INIT_PARS, Read_Input, 
     $                      CHK_STAT, SET_PTR, FIND_IDX, SET_PATH,
     $                      ResetOut, WriteOut, CloseOut, Read_File
      USE TYPES,      ONLY: KND, DYN_CHAR,  TE_TYPE
      Use VersNum,    Only: VERSION
      USE VRS,        ONLY: N_FRF, K_nu, Dnu_FRF, Sig_FRF, Nsig
      USE WATER,      ONLY: UWT, UIT, OC_LAYERING, NL_UW => NLAYERS_UW,
     $                      OC_DEP, INCL_INEL, WL_Flu, OB_TYPE
      USE WF_NUM,     ONLY: NUM_WF

      use cs_table
      use cs_table_map
c     ==================================================================

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Main program GT_IFACE'
      CHARACTER (LEN=2)   LOG_SF
      CHARACTER (LEN=80)  TXT
      CHARACTER (LEN=80)  WL_CHOSE_FLAG
      CHARACTER (LEN=:), ALLOCATABLE :: HITRAN_DIR, WL_DATA_FILE, 
     $     STR_INP
      CHARACTER (LEN=*)   Work_Dir_inp
      INTEGER  NSPEC_INP, I, J, NSPEC, II, K, JSEG, J_S, J_E, MAX_PTS,
     $     N_COL_WL, N_READ, LL, N_pix, Status, CU
      INTEGER, PARAMETER ::   MAX_LEVEL = 100000
      INTEGER, ALLOCATABLE :: SPECSEGS(:,:)
      LOGICAL  NEXT_ITERATION, FIRST_RUN, NEW_ALT_GRID, READ_FLAG,
     $     WL_GRID_FLAG, DO_NEAR_NADIR, NEXT_PIX, NEW_SEGS
      Logical, Allocatable :: FLG(:)
      REAL (KND), ALLOCATABLE :: Z_READ(:), DATA_R(:,:), DUM(:)
      REAL (KND), ALLOCATABLE, TARGET :: WL_INP(:)
      REAL (KND), POINTER :: WL_OUT(:)
      REAL (KND)  NEAR_NADIR_DEFINITION, F_KING, WL_START, WL_END, ZN,
     $     LOSZEN_MAX, SUNZEN_MAX, RO2, RN2, Z_STEP, WL_S, WL_E,X_1, X_2
      TYPE (DATA_ITER), POINTER :: RTM_PTR

      type(table_t), pointer  :: table_buff
      TYPE (TE_type)  :: TE     ! Thermal emission
      integer                 :: err
c     ==================================================================

      INTERFACE
       SUBROUTINE GT_FILEREAD (DO_NEAR_NADIR, NEAR_NADIR_DEFINITION,
     $     HITRAN_DIR, N_COL_WL, WL_DATA_FILE, WL_GRID_FLAG, *)
       USE TYPE_VARS,  ONLY: KND
       INTEGER   N_COL_WL
       LOGICAL   DO_NEAR_NADIR, WL_GRID_FLAG
       CHARACTER (LEN=:), ALLOCATABLE :: HITRAN_DIR, WL_DATA_FILE
       REAL (KND)  NEAR_NADIR_DEFINITION
       END        
      END INTERFACE

      Call Init_IO (Work_Dir_Inp, LOG_SF, *880)

      CALL ST_TRACK (R_NAME, 3, 'I')

      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here
      
      HITRAN_DIR = ''

      Status = 1 
      RAD_TO_DEG = 180.0D0/DACOS(-1.0D0)

      NEW_CLD = .FALSE.
      NEW_SPEC_GRID = .FALSE.
      TG_SCALE_FLAG = .FALSE.
      WARN_FLAG = .FALSE.
      EXA_OUT_FLAG = .FALSE.
      ITERATION = 1
      RET_MODE = ''
      FACTOR_EXA_S = 1.0D0
      PRESET_APR_NAME = 'None'
      RET_QUALITY = 'Undefined'
 
c     ============================================
c     Read all scenario setup parameters from file
c     ============================================
c        write(6,*) "before"
      Call GT_FILEREAD (DO_NEAR_NADIR, NEAR_NADIR_DEFINITION,
     $     HITRAN_DIR, N_COL_WL, WL_DATA_FILE, WL_GRID_FLAG, *880)
c        write(6,*) "after"

! Define if we write out intensities and weighting functions from the
! forward model
      If(OutMode == 'HPC') Then
         If(RET_FLAG) Then
            WrtModRes = .False.
         Else
            WrtModRes = .True.
         End If
         WrtRetPrf = .False.
         WrtRetPrf = .False.
         WrtRetFit = .False.
      Else
         IF(LOG_LEVEL(1) >= N_SSO .OR. LOG_LEVEL(2) >= N_ESO 
     $        .OR. TRC_FLG(1) .OR. .NOT.RET_FLAG ) Then
            WrtModRes = .True.
         Else
            WrtModRes = .False.
         End If
         WrtRetPrf = .True.
         WrtRetFit = .True.
      End If

      IF( SOUT_FLAG ) THEN
         OPEN(78,file=TRIM(DIR_SOUT)//'gas_abs.dat')
         OPEN(79,file=TRIM(DIR_SOUT)//'ray_sca.dat')
         OPEN(80,file=TRIM(DIR_SOUT)//'aer_sca.dat')
         OPEN(81,file=TRIM(DIR_SOUT)//'aer_abs.dat')
         OPEN(82,file=TRIM(DIR_SOUT)//'cld_sca.dat')
         OPEN(83,file=TRIM(DIR_SOUT)//'cld_abs.dat')
      ENDIF

C     IF NO LINE-ABSORBER PRESENT, SWITCH OFF LINE-BY-LINE/ESFT SCHEME
      IF(.NOT.ANY(TG_INFO(MIX_ABS_START:N_ST_MOL)%FM_FLG)) THEN
         IF (DO_LBL) THEN
            CALL WRITE_MESSAGE(R_NAME, 'WARNING',
     $           'Apparently no line-by-line calculations needed')
            WRITE(EU,*) '    Line-by-line option switched off'
            DO_LBL = .FALSE.
         END IF
         IF (DO_ESFT) THEN
             CALL WRITE_MESSAGE(R_NAME, 'WARNING',
     $       'Apparently no esft calculations needed')
             WRITE(EU,*) '    Esft option switched off'
            DO_ESFT = .FALSE.
         ENDIF
      ENDIF

C     If no line-by-line or esft calculations required switch off
C     all line absorber (except O2 in UV spectral region)
      IF (DO_LBL.OR.DO_ESFT) THEN
         IF(BAND_START(1) < 240.0D0) GOTO 200
      ELSE
         IF(BAND_START(1) < 240.0D0) THEN
            FORALL(I = 1:SIZE(TG_INFO), TG_INFO(I)%M_TYPE == 'line'.AND.
     $           TG_INFO(I)%NAME /= 'o2') TG_INFO(I)%FM_FLG = .FALSE.
         ELSE
            FORALL(I = 1:SIZE(TG_INFO), TG_INFO(I)%M_TYPE == 'line') 
     $           TG_INFO(I)%FM_FLG = .FALSE.
         END IF
      END IF

c     ====================================================
c     Initialization of index array ST_GAS_IDX,
c     Attention, the array is reinitialized in rtm_main.f
c     after all unnecessary absorbers are turned off!
c     ====================================================
      ALLOCATE(ST_GAS_IDX(COUNT(TG_INFO%FM_FLG))) ! A
      ST_GAS_IDX(:) = PACK((/(I, I =1,N_ST_MOL)/),TG_INFO%FM_FLG)

c     ====================================================
c             Reading forward model altitude grid
c     ====================================================
      ALLOCATE( Z_READ(MAX_LEVEL) ) ! A
      IF(.Not.Allocated(Z_LEV)) THEN
         IF( VHL ) THEN
            Z_READ(1:2) = [0.,1.]
            NLAYERS_INP = 2
         ELSE
            IF(LOG_LEVEL(1) >= N_SSO)
     $           WRITE(LUNIT,*) ' *** Reading: ', TRIM(NET_NAME)
            OPEN(C_UNIT,FILE = TRIM(NET_NAME),STATUS = 'OLD')
            CALL SKIP_HEADER (C_UNIT)
            DO I = 1, MAX_LEVEL
               READ(C_UNIT, *, END=22, ERR=203) Z_READ(I)
            END DO
 22         CONTINUE
            NLAYERS_INP = I - 1
            CLOSE (C_UNIT)
         END IF
      ELSE
         If(Allocated(Zl_STEP)) Then 
            K = 1
            DO I = 1, Size(Z_LEV) - 1
               Z_READ(K) = Z_LEV(I)
               J = 0 
               DO
                  J = J + 1 ; K = K + 1
                  Z_READ(K) = Z_LEV(I) + Zl_STEP(I)*J
                  IF( Z_READ(K) >= Z_LEV(I + 1) ) EXIT
               ENDDO
            ENDDO
            Z_READ(K) = Z_LEV(I)
            NLAYERS_INP = K
            DeAllocate(Zl_STEP)
         Else
            NLAYERS_INP = Size(Z_LEV)
            Z_READ(1:NLAYERS_INP) = Z_LEV(:)
         End If
         DeAllocate(Z_LEV)
      ENDIF

      IF( RT_AG=='merra'.OR.RT_AG=='xb_merra') THEN
         CALL MERRA_AG(*880)
      ELSE
         ALLOCATE( ZZZ_INP(NLAYERS_INP) ) ! A
         ZZZ_INP(:) = Z_READ(1:NLAYERS_INP)
         DEALLOCATE( Z_READ )
         IF( ZZZ_INP(1) > ZZZ_INP(NLAYERS_INP) )
     $        ZZZ_INP(1:NLAYERS_INP) = ZZZ_INP(NLAYERS_INP:1:-1)
         DO I = 1,NLAYERS_INP-1
            IF( ZZZ_INP(I+1) < ZZZ_INP(I) ) GOTO 202
         ENDDO
      ENDIF
      
c     ==========================================================
c     Reading forward model depth grid for underwater transport
c     ==========================================================
      IF( UWT ) THEN
         IF( TRIM(OC_LAYERING) == 'from file' ) THEN
            ALLOCATE( Z_READ(MAX_LEVEL) ) ! A
            IF(LOG_LEVEL(1) >= N_SSO)
     $           WRITE(LUNIT,*) ' *** Reading: ', DEP_NAME
            OPEN(C_UNIT,FILE = DEP_NAME,STATUS = 'OLD')
               DO I = 1, MAX_LEVEL
                  READ(C_UNIT,*,END=33) Z_READ(I)
               END DO
 33            CONTINUE
               NDEP_INP = I - 1
               ALLOCATE( DEP_INP(NDEP_INP) ) ! A
               DEP_INP(:) = Z_READ(1:NDEP_INP)
               DEALLOCATE( Z_READ )
               DO J = 1, NDEP_INP-1
                  IF( DEP_INP(J) < 0.d0 ) GOTO 213
                  IF( (DEP_INP(J+1)-DEP_INP(J)) <= 0.d0 ) GOTO 214
               ENDDO
            CLOSE (C_UNIT)
         ELSE
            NDEP_INP = NL_UW ; ALLOCATE( DEP_INP(NL_UW) )
            DEP_INP(1) = 0.d0 ; DEP_INP(NL_UW) = OC_DEP
            IF( NL_UW > 2 ) THEN
               Z_STEP = (DEP_INP(NL_UW) - DEP_INP(1))/DBLE(NL_UW-1)
               DO J = 2, NL_UW
                  DEP_INP(J) = DEP_INP(J-1) + Z_STEP
               ENDDO
            ENDIF
         ENDIF
      ENDIF
!      WRITE(*,*) '    NDEP_INP  ', NDEP_INP
!      DO I = 1, NDEP_INP
!         WRITE(*,*) I, DEP_INP(I)
!      ENDDO
!     STOP '  In gt_iface.f'

c     ===========================
c     Reading geometry parameters
c     ===========================
      Z_BOA = GROUND_HEIGHT ;  Z_TOA = MAXVAL(ZZZ_INP(:))
      CALL READ_GEOM_PARS(Z_BOA, Z_TOA, *880)

c     ===========================================================
c     Read wavelength dependent depolarisation ratios if required
c
c     With RAMAN:
c     Read wavelength dependent depolarisation ratio, king factor
c     and anisotropies of the polarizability (gamma) of oxygen
c     and nitrogen (!mv) if required (gamma is squared and
c     converted to [m^6]):
c     ===========================================================
      IF (DO_DEPOL_WAVELENGTH.OR.DO_RAMAN) THEN

         ALLOCATE( DEP_WS(DEP_NWS), DEP_VS(DEP_NWS) ) ! A

         IF(LOG_LEVEL(1) >= N_SSO)
     $        WRITE(LUNIT,*) ' *** Reading: ', TRIM(DEPOL_FILENAME)

         OPEN(UNIT=C_UNIT,FILE=TRIM(DEPOL_FILENAME),STATUS='OLD')
 2155    READ(C_UNIT,'(A80)') TXT
         IF ( TXT(1:1) .EQ. ';' ) GOTO 2155

         BACKSPACE C_UNIT
         IF(DO_RAMAN) THEN
            ALLOCATE( R_GAM(N_RG,DEP_NWS) ) ! A
            DO I = 1, DEP_NWS
               READ(C_UNIT,*) DEP_WS(I), F_KING, DEP_VS(I), RO2, RN2
               R_GAM(1,I)= RO2*RO2*1.d-12
               R_GAM(2,I)= RN2*RN2*1.d-12
            ENDDO

cc           ... Constant values are used
c            R_GAM(1,:) =  1.2094d-24*1.2094d-24*1.d-12
c            R_GAM(2,:) =  7.3908d-25*7.3908d-25*1.d-12

cc           ... Approximation of Chance & Spur
c            R_GAM(1,:) = (0.07149d0
c     $           + 45.9364d0/( 48.2716d0 - 1.d0/(DEP_WS(:)*1.d-3)**2 ))
c     $           *1.d-24
c            R_GAM(1,:) = R_GAM(1,:)**2*1.d-12
c            R_GAM(2,:) = (-6.01466d0
c     $           + 2385.57d0/( 186.099d0 - 1.d0/(DEP_WS(:)*1.d-3)**2 ))
c     $           *1.d-25
c            R_GAM(2,:) = R_GAM(2,:)**2*1.d-12

         ELSE
            DO I = 1, DEP_NWS
               READ(C_UNIT,*) DEP_WS(I), F_KING, DEP_VS(I), RO2, RN2
            ENDDO
         END IF
         CLOSE(C_UNIT)
         DEPOL_INPUT = 0.0D0
      END IF

      IF(LOG_LEVEL(1) >= N_DSO)  THEN
         WRITE (LUNIT,*)
         WRITE (LUNIT,*) ' *** User selected wavelength segments:'
         WRITE (LUNIT,*) ' *** Number of segments: ', NSEGS
         WRITE (LUNIT,*) ' *** Start_wl no step:'
         DO I = 1, NSEGS
            WRITE (LUNIT,'(1X,F9.3,2X,I5,2X,F9.3)')
     $           BAND_START(I), BAND_NUM(I), BAND_STEP(I)
         END DO
      END IF

c     =======================
c     Lambertian albedo setup
c     =======================
       IF (DO_SPECTRAL_ALBEDO) CALL GT_ALBEDO_SETUP(*880)

c     ======================
c     Create wavelength grid
c     ======================
      IF( .NOT.WL_GRID_FLAG ) WL_CHOSE_FLAG = 'wl_master'
      IF( WL_GRID_FLAG ) WL_CHOSE_FLAG = 'from_file'
      IF( ALIN_FLAG )    WL_CHOSE_FLAG = 'from_file_alin'

      ALLOCATE(WL_SEGS(2,NSEGS), SPECSEGS(2,NSEGS))
      DO JSEG = 1, NSEGS
         WL_SEGS(1,JSEG) = BAND_START(JSEG) - 1.0D-10
         WL_SEGS(2,JSEG) = BAND_START(JSEG) +
     $        BAND_STEP(JSEG)*(BAND_NUM(JSEG) - 1) + 1.0D-10
      END DO

      SELECT CASE( TRIM(WL_CHOSE_FLAG) )
      CASE('wl_master')
         NSPEC_INP = SUM(BAND_NUM(1:NSEGS))
         ALLOCATE(WL_INP(NSPEC_INP)) ! A

         SPECSEGS(1,1) = 0 ;  SPECSEGS(2,1) = BAND_NUM(1)
         DO  I = 2, NSEGS
            SPECSEGS(2,I) = BAND_NUM(I)
            SPECSEGS(1,I) = SPECSEGS(1,I - 1) + BAND_NUM(I - 1)
         END DO

         K = 0
         DO  I = 1, NSEGS
            DO J = 1, BAND_NUM(I)
               K = K + 1
               WL_INP(K) = BAND_START(I) + BAND_STEP(I)*(J - 1)
            END DO
         END DO
      CASE('from_file')
         Call READ_FILE (N_COL_WL, WL_DATA_FILE, DATA_R, *880,
     $        I_XC=N_COL_WL, XS=WL_SEGS(1,1), XE=WL_SEGS(2,NSEGS), 
     $        CDF=WL_CDF)
         ALLOCATE(FLG(SIZE(DATA_R(:,N_COL_WL))))

         SPECSEGS(1,1) = 0 ; FLG = .FALSE.
         DO JSEG = 1, NSEGS
            WHERE(DATA_R(:,N_COL_WL) >= WL_SEGS(1,JSEG).AND.
     $           DATA_R(:,N_COL_WL) <=  WL_SEGS(2,JSEG))
     $           FLG = .TRUE.
            IF(JSEG > 1) Then
               SPECSEGS(1,JSEG) = 
     $           SPECSEGS(1,JSEG - 1) + SPECSEGS(2,JSEG - 1)
               SPECSEGS(2,JSEG) = 
     $              COUNT(FLG) - SUM(SPECSEGS(2,1:JSEG - 1))
            Else
               SPECSEGS(2,JSEG) = COUNT(FLG)
            End If
         End Do            

         NSPEC_INP = COUNT(FLG)
         ALLOCATE(WL_INP(NSPEC_INP)) ! A
         WL_INP(:) = Pack(DATA_R(:,N_COL_WL),FLG(:))
         DEALLOCATE(DATA_R)
      CASE('from_file_alin')
         NSEGS = 1
         NSPEC_INP = ALIN_NWL
         ALLOCATE(WL_INP(NSPEC_INP)) ! A
         SPECSEGS(1,1) = 0
         SPECSEGS(2,1) = NSPEC_INP
         WL_INP(1:NSPEC_INP) = ALIN_WL(1:NSPEC_INP)
      END SELECT

!     Reset WL_SEGS to get exact values at the boundaries
      DO JSEG = 1, NSEGS
         J_S = SPECSEGS(1, JSEG) + 1
         J_E = J_S + SPECSEGS(2, JSEG) - 1
         WL_SEGS(1,JSEG) = WL_INP(J_S)
         WL_SEGS(2,JSEG) = WL_INP(J_E)
      END DO

c     =========================================
c     Replace wavelength grid in retrieval mode
c     =========================================
      IF( RET_FLAG )  THEN
C     Retrieval supports only wavelength grid in nm
         CALL READ_RET_MAIN (*880)
         NSPEC = SIZE(WL_MDS)
         WL_OUT => WL_MDS
         NEW_SEGS = .TRUE.
      ELSE
         IF(WL_UNITS == 'cm-1') THEN
            WL_INP(1:NSPEC_INP) = 1.0D+7/WL_INP(NSPEC_INP:1:-1)
            DO JSEG = 1, NSEGS
               X_1 = 1.0D+7/WL_SEGS(1,JSEG)
               X_2 = 1.0D+7/WL_SEGS(2,JSEG)
               WL_SEGS(1,JSEG) = MIN(X_1, X_2)
               WL_SEGS(2,JSEG) = MAX(X_1, X_2)
            END DO
            NEW_SEGS = .TRUE.
         ELSE
            NEW_SEGS = .FALSE.
         END IF
         MOD_READ_FLAG = .TRUE.
         WL_OUT => WL_INP
         NSPEC = NSPEC_INP
         ALLOCATE(MSK_RTM(NSPEC,N_LOS_OUT)) 
         MSK_RTM = .TRUE.
!        Create RET_INFO array is needed to initialize weighting functions
         IF(WF_FLAG.OR.NUM_WF) CALL CREATE_INFO_ARR (0)
         MAX_ITER = 1
      END IF

!     Initialize RTM parameters
      CALL INIT_PARS
  
      IF(NEW_SEGS) THEN
!     Re-calculate the boundaries of the spectral segments
         DO  I = 1, NSEGS
            SPECSEGS(1,I) = COUNT(WL_OUT < WL_SEGS(1,I))
            SPECSEGS(2,I) = 
     $           COUNT(WL_OUT <= WL_SEGS(2,I)) - SPECSEGS(1,I)
         END DO
      END IF

      ALLOCATE(SEG_NUM_OUT(NSPEC))
      IF( ALIN_FLAG ) THEN
         SEG_NUM_OUT(:) = 1
      ELSE
         SEG_NUM_OUT = 0
         DO JSEG = 1, NSEGS
            J_S = SPECSEGS(1, JSEG) + 1
            J_E   = J_S + SPECSEGS(2, JSEG) - 1
            WHERE(WL_OUT(:) >= WL_OUT(J_S).AND.
     $           WL_OUT(:) <= WL_OUT(J_E)) SEG_NUM_OUT(:) = JSEG
         END DO
         IF(ANY(SEG_NUM_OUT(:) == 0)) GOTO 201
      ENDIF
c     ===================================
c     Check all scenario setup parameters
c     ===================================
      CALL GT_SETTINGS_CHECK (*880)

c     ===============================================
c     New wavelength grid for ESFT and/or convolution
c     ===============================================
      IF( CONVOL_FLAG.OR.DO_ESFT ) THEN
         CALL ST_WL_SETUP( WL_OUT, NSPEC, SPECSEGS, NSEGS, SEG_NUM_OUT,
     $        WL_START, WL_END, *880 )

c         STOP '   in "gt_iface.f" '

      ELSE
         ALLOCATE(LA_FLAG(NSPEC)) !A
         LA_FLAG = .TRUE.
         WL_START = WL_OUT(1) ;  WL_END = WL_OUT(NSPEC)

         IF(DO_LBL) THEN
            ALLOCATE(SEG_WL(2,NSEGS)) !A
            NSEGS_LBL = NSEGS
            DO  I = 1, NSEGS
               SEG_WL(1,I) = WL_OUT(SPECSEGS(1,I) + 1)
               SEG_WL(2,I) = WL_OUT(SPECSEGS(1,I) + SPECSEGS(2,I))
            END DO
         END IF

      END IF

C      WRITE(*,*) '  NSPEC = ', NSPEC
C      WRITE(*,*) '  START   ', WL_START
C      WRITE(*,*) '    END   ', WL_END
c      PAUSE

c     ... Modify "start" and "end" wavelengths for rotation Raman
c         calculations
      IF( DO_RAMAN ) THEN
c        ... To take into account increasing of Raman margin with
c            the increasing of wavelength. It is assumed that
c            Raman margin is INCR_RANGE for wavelengths smaller 500 nm
         IF( WL_START >= 500.d0 ) THEN
            WL_START = WL_START - INCR_RANGE*(WL_START/500.d0)**2
            WL_END   = WL_END   + INCR_RANGE*(WL_END/500.d0)**2
         ELSE
            WL_START = WL_START - INCR_RANGE
            WL_END   = WL_END   + INCR_RANGE
         ENDIF
      ENDIF
c      IF( DO_RAMAN ) THEN
c         WL_START = WL_START - INCR_RANGE
c         WL_END   = WL_END   + INCR_RANGE
c      ENDIF

c     ... Modify "start" wavelength for vibrational Raman calculations
      IF( INCL_INEL(1) ) THEN
         ALLOCATE( DUM(N_FRF) )
         WRITE(*,*) '    WL_START orig :', WL_START
         DUM(:) = K_nu/WL_START + Dnu_FRF(:)
     $        + Nsig*DSQRT(2.d0)*Sig_FRF(:)
         WL_START = MINVAL( K_nu/DUM(:) )
         DEALLOCATE(DUM)
         WRITE(*,*) '    WL_START modif :', WL_START
      ENDIF

c     ... Extent spectral range if fluorescence should be accounted for
      IF( ANY(INCL_INEL(2:3)) ) THEN
         WL_START = DMIN1(WL_START,WL_FLU(1))
         WL_END   = DMAX1(WL_END,  WL_Flu(2))
      ENDIF

c     ... Temporary solution (for 1 segment only) to perform
c         monochromatic LBL calculations including inelastic
c         scattering processes trough ocean
      IF( ANY(INCL_INEL(:)).OR.DO_RAMAN ) THEN
         IF(DO_LBL) THEN
            SEG_WL(1,1) = WL_START
            SEG_WL(2,1) = WL_END
         ENDIF
      ENDIF

      allocate(table_lists(N_ST_MOL))
      do i=1, N_ST_MOL
        table_lists(i)%table => null()
      end do

      if(table_use_table) then
        write(*, *) "reading cross section tables"
        do i=1, table_input_cnt
          call table_init(table_buff, table_input_desc(i))
          write(*, "(I6,A16,I8,I8,I8,2X,A)")
     &      table_buff%absorber_id,
     &      adjustr(trim(table_buff%absorber)),
     &      table_buff%np,
     &      table_buff%nt,
     &      table_buff%nw,
     &      trim(table_buff%file_desc)

          write(*, "(2X,A)") trim(table_buff%file_pres)
          write(*, "(2X,A)") trim(table_buff%file_temp)
          write(*, "(2X,A)") trim(table_buff%file_wave)
          write(*, "(2X,A)") trim(table_buff%file_cros)

          err = 0
          call table_mmap(table_buff, err)
          if(err .ne. 0) then
            write(*, *) "error gt_iface: table_map"
          end if
          write(*, "(2X,A16,F16.8,A16,F16.8)")
     &      "wmin=", table_buff%wave(1),
     &      "wmax=", table_buff%wave(table_buff%nw)

          call table_list_add(
     &      table_lists(table_buff%absorber_id)%table, table_buff)
        end do

        deallocate(table_input_desc)
      end if

      IF( DO_LBL ) CALL Read_HITRAN_Pars(HITRAN_DIR, *880)

c     ... Spectroscopic information: atoms and ions
      Emi_FLG = .FALSE. ; N_AT = 0
      IF( ANY((TG_INFO%M_TYPE == 'atom').AND.TG_INFO%FM_FLG) ) THEN
         Emi_FLG = .TRUE.
         CALL ATOMS_SPI(WL_START, WL_END)
      ENDIF

      IF(DO_CLOUD) CALL GENERAL_CLOUD_INIT(WL_START, WL_END, *880)

      IF( FLU_L.AND.(.NOT.DIFB) ) THEN 
         IF( USE_DGF ) THEN
            CALL SCALE_AMPL
         ELSE
            CALL READ_LAND_FLU(*880)
         ENDIF
      ENDIF

      IF( UWT ) THEN
         IF( TRIM(Ref_type)=='wl_dep' ) CALL READ_REFR_IDX(*880)
         IF( OB_Type       =='model'  ) CALL READ_REF_ICE(*880)
      ENDIF
c     ===================================
c     Set DO_AZIMUTH to false if required
c     ===================================
      IF( TRIM(RTM_CORE)=='DOM'.AND.DO_NEAR_NADIR
     $     .AND.(.NOT.RAD_FIELD) ) THEN
         LOSZEN_MAX = MAXVAL(DACOS(DABS(ANG_OUT(2,:)))*RAD_TO_DEG,1)
         SUNZEN_MAX = MAXVAL(DACOS(ANG_OUT(1,:))*RAD_TO_DEG,1)
         IF ( LOSZEN_MAX < NEAR_NADIR_DEFINITION .OR.
     $        SUNZEN_MAX < NEAR_NADIR_DEFINITION ) THEN
            DO_AZIMUTH = .FALSE.
            IF (LOG_LEVEL(1) >= N_RSO) WRITE(LUNIT,*)
     $              ' *** ', 'First harmonic only (nadir/zenith)'
         END IF
      END IF
C     No Fourier series for asymptotic
C     ================================
      IF(TRIM(RTM_CORE) == 'ASYMP') DO_AZIMUTH = .FALSE.

c     ================
c     Call to SCIATRAN
c     ================
      DO_INTERNAL_CHECK = .TRUE.
      NEW_ALT_GRID = .FALSE. ;  READ_FLAG = .FALSE.
      FIRST_RUN = .TRUE.     ;  NEXT_ITERATION = .TRUE.
      NEW_NODES = .FALSE.    ;  NEXT_PIX = .FALSE. ; N_pix = 0
      FACTOR_TPS_COL = 1.0D0
      IF(RET_FLAG.AND..NOT.MOD_READ_FLAG) READ_FLAG = .TRUE.

      DO
         CALL RTM_MAIN(NSPEC, WL_OUT(1:NSPEC), SPECSEGS, FIRST_RUN,
     $        NEW_ALT_GRID, READ_FLAG, WL_START, WL_END, *880)

         If(PolCorrFlg) Call PolCorr (FIRST_RUN)

         IF(.NOT.RET_FLAG) EXIT

         FIRST_RUN = .FALSE. ;  READ_FLAG = .FALSE.
         CALL RETRIEVAL(NSEGS, SPECSEGS, NEXT_ITERATION, NEW_ALT_GRID,
     $        NEXT_PIX, N_pix, *880)

         IF(.NOT.NEXT_ITERATION) THEN
            IF(LOG_LEVEL(2) >= N_RSO) THEN
               WRITE(LUNIT,'(" Total iterations number - ",I2)')
     $              ITERATION
            END IF
            IF( .NOT.NEXT_PIX ) THEN
               EXIT
            ELSE
               NEXT_ITERATION = .TRUE. ;  ITERATION = 1
               N_pix = N_pix + 1
            ENDIF
         END IF
      END DO

      CALL OVERALL_SETTINGS(WL_START)

      IF( SOUT_FLAG ) THEN
         CLOSE(78)  ;  CLOSE(79)  ;  CLOSE(80)
         CLOSE(81)  ;  CLOSE(82)  ;  CLOSE(83)
      ENDIF

      IF(DO_CLOUD) CALL WRITE_CLOUD_SETTINGS
      Call WriteOut (FUNIT,
     $     '  ******************************************')
      Call WriteOut (FUNIT,
     $     '  ***  End of SCIATRAN '//VERSION//' logfile          ')
      Call WriteOut (FUNIT, 
     $     '  ******************************************')

      Call CloseOut (FUNIT)
      Call WriteOutput

      IF(LOG_LEVEL(1) >= N_RSO) THEN
         WRITE(LUNIT,*)
         WRITE(LUNIT,*)
     $        '************** Normal termination ******************'
         IF(WARN_FLAG) THEN
            WRITE(LUNIT,*)
     $        '*********** See errors.log for WARNINGS ************'
            WRITE(LUNIT,*)
         END IF
      END IF

      CLOSE(EU)
      IF(TRIM(LOG_SF) == 'F') CLOSE(LUNIT)

      Status = 0
      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN

 213  CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
      WRITE (EU,*) '   Depth grid in the ocean should be positive.'
      WRITE (EU,*) '   Check input file: ',TRIM(DEP_NAME)
      RETURN 

 214  CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ') 
      WRITE (EU,*) '   Depth grid in the ocean should be '
      WRITE (EU,*) 'given in ascending order.'
      WRITE (EU,*) '    Check input file: ',TRIM(DEP_NAME)
      RETURN 

 200  CALL WRITE_MESSAGE (R_NAME, 'ERROR',
     $     'Line-by-line/ESFT must be switched off below 240 nm !')
      WRITE(LUNIT, *)
      WRITE(LUNIT, *) ' >>>>>>> !!! Abnormal termination !!! <<<<<<<'
      WRITE(LUNIT, *) ' SCIATRAN stopped in gt_iface: see errors.log'
      WRITE(LUNIT, *) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      WRITE(LUNIT, *)
      RETURN

 201  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'One of spectral '//
     $     'points was not associated to a spectral segment. Please'//
     $     ' submit a bug report to the program developers')
      WRITE(LUNIT, *)
      WRITE(LUNIT, *) ' >>>>>>> !!! Abnormal termination !!! <<<<<<<'
      WRITE(LUNIT, *) ' SCIATRAN stopped in gt_iface: see errors.log'
      WRITE(LUNIT, *) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      WRITE(LUNIT, *)
      RETURN

 202  CALL WRITE_MESSAGE (R_NAME, 'ERROR', '')
      WRITE(EU, *) '    The main altitude grid is not monotonic'
      IF( I-1 > 0 ) THEN
         WRITE(EU,'(a,1x,I2,1x,a,1x,F7.2,a)')
     $        '     Z(', I-1,') = ', ZZZ_INP(I-1),' km'
      ENDIF
      WRITE(EU,'(a,1x,I2,1x,a,1x,F7.2,a)')
     $     '     Z(', I,') = ', ZZZ_INP(I),' km'
      WRITE(EU,'(a,1x,I2,1x,a,1x,F7.2,a)')
     $     '     Z(',I+1,') = ', ZZZ_INP(I+1),' km'
      IF( I+2 <= NLAYERS_INP ) THEN
         WRITE(EU,'(a,1x,I2,1x,a,1x,F7.2,a)')
     $        '     Z(', I+2,') = ', ZZZ_INP(I+2),' km'
      ENDIF
      WRITE(EU, *)
     $     '    Check file: ', TRIM(NET_NAME)
      RETURN

 203  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading '//
     $     TRIM(NET_NAME)//' file')

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

      WRITE(LUNIT,*)
      WRITE(LUNIT, *) ' >>>>>>> !!! Abnormal termination !!! <<<<<<<'
      WRITE(LUNIT, *) ' SCIATRAN stopped in gt_iface: see errors.log'
      WRITE(LUNIT, *) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      WRITE(LUNIT,*)

      CLOSE(EU)
      Call CloseOut (FUNIT)
      Call WriteOutput
      IF(TRIM(LOG_SF) == 'F') CLOSE(LUNIT)
      RETURN
      
c **********************************************************************
      CONTAINS
c **********************************************************************

      SUBROUTINE MERRA_AG(*)

      USE TOOLS,  ONLY: SPLIT_STRING
c     ------------------------------------------------------------------
      CHARACTER (LEN=:), ALLOCATABLE :: D_NAME, Z_NAME
      INTEGER N_lay
      REAL (KND), PARAMETER:: Hha = 8.0, Dpre = 15.d0
      REAL (KND) Pre0, Z0, HD
      REAL (KND), ALLOCATABLE:: Z_lay(:), Zmid(:), Zrt(:)
      TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
c     ==================================================================
      
      STR_INP = 'MERRA scenario file'
      CALL Read_Input (STR_INP, READ_ARR, *880)
      CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
      D_NAME = SET_PATH(STR_ARR(1)%S) ; Z_NAME = SET_PATH(STR_ARR(2)%S)
c                WRITE(*,*) '  Z_NAME ', Z_NAME
c     ... Reading MERRA altitude grid:      
      CALL READ_FILE(1, Z_NAME, DATA_R, *880)
      N_lay = Size(DATA_R,1) ;  ALLOCATE( Z_lay(N_lay),Zmid(N_lay) )
      Z_lay(:) = DATA_R(1:N_lay,1)
      Zmid(:) = Z_lay(N_lay:1:-1)
!      WRITE(*,*)
!     $     '   Minimal and maximal altitude of middle aerosol layers'
!              WRITE(*,'(2x,E12.4,2x,F8.4)') Zmid(1), Zmid(N_lay)
c     ... Reading surface pressure
      OPEN(C_UNIT, FILE=D_NAME, STATUS='OLD')
      CALL SKIP_HEADER (C_UNIT)
      READ(C_UNIT,*) Pre0
      CLOSE(C_UNIT)
      Pre0 = Pre0/100.d0  ! Pa --> Hpa (mb)
!      WRITE(*,*) '   Surface pressure  [mb]: ', Pre0 
c     ... Calculate surface elevation
      Z0 = Zmid(1) + Hha*DLOG(1.d0 - Dpre/2.d0/Pre0)
      IF( Z0 < 0.d0 ) Z0 = 0.d0
c      WRITE(*,'(A,2x,F8.2)') '   Surface elevation [m]: ', Z0*1.d3
      NLAYERS_INP = N_lay + 1
      ALLOCATE( Zrt(NLAYERS_INP) )
      Zrt(1) = Z0 
      DO I = 2, NLAYERS_INP
         Zrt(I) = 2.d0*Zmid(I-1) - Zrt(I-1)
      ENDDO
c     ... Check calculated layer tops
      DO I = 2, NLAYERS_INP
         HD = Zmid(I-1)
         IF( HD>Zrt(I-1).AND.HD<Zrt(I) ) CYCLE
         WRITE(*,*) '   Wrong altitude top of aerosol layer '
         STOP '     Stop in "gt_iface.f" '
      ENDDO
      Zrt(1) = 0.d0             !  Start always from see level

c      DO I = 1, NLAYERS_INP
c         IF( I==1 ) THEN
c            WRITE(*,'(2x,I3,*(2x,F8.3))') I, Zrt(I)
c         ELSE
c            WRITE(*,'(2x,I3,*(2x,F8.3))') I, Zrt(I), Zmid(I-1)
c         ENDIF
c      ENDDO

      ALLOCATE( ZZZ_INP(NLAYERS_INP) ) ! A
      ZZZ_INP(:) = Zrt(:)
      GROUND_HEIGHT = Z0
c      WRITE(*,'(A,2x,F8.3)') '  GROUND_HEIGHT  ', GROUND_HEIGHT 
      
c      STOP '    Stop in MERRA_AG'
      RETURN
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
      END SUBROUTINE MERRA_AG
c **********************************************************************
      
      SUBROUTINE READ_REF_ICE(*)
      USE BRDF,   ONLY:  WL_refi_idx, Refi_idx, NIce, Iref_name
c     ---------------------------------------------------------------
      INTEGER II
      REAL (KND) XS, XE
c     ==================================================================
      XS = WL_START /1.d03      !  [nm] ==> [microm] 
      XE = WL_END   /1.d03
c     ... Refractive index of ice (WL input [micron])
      CALL READ_FILE(3, Iref_name, DATA_R, *880, XS, XE,
     $     NPT = 1, Skip_line = 1)
      NIce = Size(DATA_R,1)
      ALLOCATE( WL_refi_idx(Nice), Refi_idx(Nice))
      WL_refi_idx(:) = DATA_R(:,1)*1.d03 ! wavelength in [nm]
      Refi_idx(:)    = DATA_R(:,3)       ! imaginary part of ice refractive index
      DEALLOCATE( DATA_R )
      RETURN
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1      
      END SUBROUTINE READ_REF_ICE
      
c **********************************************************************
c          Reading refractive index of water or ice 
c************************************************************************
      SUBROUTINE READ_REFR_IDX(*)

      USE WATER,     ONLY: Ref_name, RWG, Ref_ind
      USE WATER_ICE, ONLY: IWS, IWT
c     ------------------------------------------------------------------

      CHARACTER (LEN=13), PARAMETER :: R_NAME='READ_REFR_IDX'
      INTEGER  I, N_ref
      REAL (KND)  SALINITY, STEP, XS, XE
      REAL (KND), PARAMETER   :: DN_salin=0.006, Salin1=34.32
      REAL (KND), ALLOCATABLE :: WL_ri(:), Rr(:), Ri(:)
c     ==================================================================

c     ... Refractive index of ice (WL input [micron])
      XS = WL_START /1.d03  ;  XE = WL_END/1.d03 !  [nm] ==> [microm]       
      CALL READ_FILE(3, Ref_name, DATA_R, *880, XS, XE,
     $     NPT = 1, Skip_line = 1)
      N_ref = Size(DATA_R,1)
c      WRITE(*,*) '  N_ref = ', N_ref

      ALLOCATE ( WL_ri(N_ref), Rr(N_ref), Ri(N_ref) )
      WL_ri(:) = DATA_R(1:N_ref,1)*1000.0d0 ! Transform [mkm] => [nm]
      Rr(:) = DATA_R(1:N_ref,2)
      Ri(:) = DATA_R(1:N_ref,3)
      DEALLOCATE( DATA_R )

c      WRITE(*,*) ' Reading from file: ', TRIM(Ref_name)
c     ... Real part of refractive index with salinity correction factor
c         only in the case of water or melt pond
      SALINITY = 37.d0
      IF( UWT.AND.(.NOT.UIT) ) Rr(:) = Rr(:) + DN_salin/Salin1*SALINITY
      IF( IWT ) THEN
         IF( IWS(1)%IW_TOP(1)/= 1 ) 
     $        Rr(:) = Rr(:) + DN_salin/Salin1*SALINITY
      ENDIF

      IF( NSPEC==1 ) THEN
         Nwl_BRF = 1  ;   ALLOCATE( WL_BRF(Nwl_BRF) )
         WL_BRF(1) = WL_START 
      ELSE
         SELECT CASE( TRIM(RWG) )
         CASE('min')
            Nwl_BRF = 2  ;  ALLOCATE( WL_BRF(Nwl_BRF) )
            WL_BRF(1) = WL_START  ;  WL_BRF(2) = WL_END
         CASE('max')
            Nwl_BRF = NSPEC  ;  ALLOCATE( WL_BRF(Nwl_BRF) )
            WL_BRF(:) = WL_OUT(:)
         CASE DEFAULT
            READ(RWG,*,ERR=101) Nwl_BRF
            ALLOCATE( WL_BRF(Nwl_BRF) )
            STEP = (WL_END - WL_START)/DFLOAT(Nwl_BRF - 1)
            WL_BRF(1) = WL_START
            DO I = 2, Nwl_BRF - 1
               WL_BRF(I) = WL_BRF(I-1) + STEP
            ENDDO
            WL_BRF(Nwl_BRF) = WL_END
         END SELECT
      ENDIF
      
      ALLOCATE( RefW_r(Nwl_BRF), RefW_i(Nwl_BRF) )

c     ... Real part of comples refractive index will be
c         extrapolated as constant.
      CALL LIN_INT(N_ref, WL_ri, Rr, Nwl_BRF, WL_BRF, RefW_r, 'C')
c     ... Imaginary part of comples refractive index will be
c         extrapolated as zero.
      CALL LIN_INT(N_ref, WL_ri, Ri, Nwl_BRF, WL_BRF, RefW_i, 'Z')

      IF( Nwl_BRF == 1 ) Ref_type = 'const'
      
cc     ... Constant refractive index for the test purpose
c         RefW_r(2:Nwl_BRF) = RefW_r(1)
c      WRITE(*,*) '     gt_iface: ', TRIM(RWG)
c      WRITE(*,*) '     Nwl_BRF = ', Nwl_BRF
c      WRITE(*,*) '     Ref_type = ', Ref_type
c         DO I = 1, Nwl_BRF
c            WRITE(*,'(2x,F8.3,2x,F7.4,2x,E12.4)')
c     $           WL_BRF(I), RefW_r(I), RefW_i(I)
c         ENDDO
         
c     ... Mean constant value for the refractive index which
c         will be used by calculation of refracted angles
      Ref_ind = SUM( RefW_r(:) )/DFLOAT(Nwl_BRF)
c         WRITE(*,*) '     Mean Refr ind. = ', Ref_ind
c         PAUSE
c      WRITE(*,*) '     End of "READ_REFR_IDX" '

      RETURN
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1      
 101  CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     $       'Error reading from character !')
      WRITE(EU,*) '    Character : ', TRIM(RWG)
      RETURN 1

      END SUBROUTINE READ_REFR_IDX
C******************************************************************
      SUBROUTINE READ_LAND_FLU(*)

      USE IRRAD,      ONLY: WL_IRRAD_DIM
c     ------------------------------------------------------------------

      INTEGER  I, K1, K2, NFU
      INTEGER, PARAMETER :: IU_FLU=47
      CHARACTER (LEN=13), PARAMETER :: R_NAME='READ_LAND_FLU'
      REAL (KND) DO_START, DO_END, WLT, SPE
      REAL (KND), ALLOCATABLE :: TMP(:,:)
      REAL (KND), PARAMETER ::
     $              C = 299792458.0D0, ! Light speed [m/s]
     $              hP = 6.626071D-34  ! Planck's constant [J s]
c     ==================================================================

      DO_START = WL_START - 1.d-10 ; DO_END = WL_END + 1.d-10

      OPEN(IU_FLU, FILE=TRIM(FLU_NAME), STATUS='OLD')
         CALL SKIP_HEADER(IU_FLU)
c        ... Define full number of spectral points in file
         NFU = 0
         DO
            READ(IU_FLU,*,ERR=112,END=311) WLT, SPE
            NFU = NFU + 1
         ENDDO
 311     REWIND(IU_FLU)
         ALLOCATE( TMP(NFU,2) )

         CALL SKIP_HEADER(IU_FLU)
         K1 = 0
c        ... Skip all data having wavelength smaller than DO_START
 222     READ(IU_FLU,*,ERR=112,END=113) WLT, SPE
         K1 = K1 + 1
         IF( WLT < DO_START ) GOTO 222
         BACKSPACE(IU_FLU)
         IF( K1 > 1 ) BACKSPACE(IU_FLU)

c        ... Reading in actual wavelength interval
         K2 = 0
 333     READ(IU_FLU,*,ERR=112,END=444) WLT, SPE
         K2 = K2 + 1 ; TMP(K2,1) = WLT ; TMP(K2,2) = SPE
         IF( WLT > DO_END ) THEN
            IF( K2 > 2 )  GOTO 444
c           ... Read an additional spectral point to perform Akima interpolation
            GOTO 333
         ELSE
            GOTO 333
         ENDIF
 444     NFL = K2
c        ... Is actual spectral interval outside of given in file?
         IF( K1==1 .AND. K2==1 ) GOTO 113

         ALLOCATE( WLF(NFL), SPF(NFL) )
         WLF(:) = TMP(1:NFL,1) ; SPF(:) = TMP(1:NFL,2)
         DEALLOCATE(TMP)
      CLOSE(IU_FLU)

      IF( LOG_LEVEL(1) >= N_ESO ) THEN
         WRITE(*,'(a,I5)')
     $      '   Full number of spectral points in file           : ',NFU
         WRITE(*,'(a,I5)')
     $      '   The number of spectral points in actual interval : ',NFL
         WRITE(*,'(a,2x,F8.3,a,2x,F8.3,a)')
     $      '   Actual wavelengths                               :',
     $        WL_START, '   -',WL_END,' nm'
         WRITE(*,'(a,2x,F8.3,a,2x,F8.3,a)')
     $      '   First and last wavelengths in emission spectra   :',
     $        WLF(1),'   -',WLF(NFL),' nm'
         WRITE(*,'(a,2x,F8.3,a,2x,F8.3,a)')
     $      '   First and last values in emission spectra        :',
     $        SPF(1),'   -',SPF(NFL),' mW/m^2/sr/nm'
      ENDIF

c     **** Input dimension of emission spectra is [mW/m2/sr/nm]  ****
c          Recalculate dimension of emission spectra to be the same
c          as the dimension of solar spectra.

c     Power dimension:
      SELECT CASE( TRIM(WL_IRRAD_DIM(2)) )
      CASE('W')
         SPF(:) = SPF(:)*1.d-3  ! [mW] -> [W]
      CASE('phot/s')
         SPF(:) = SPF(:)*1.d-3              ! [mW] -> [W]
c                   wavelength in [m]
         SPF(:) = SPF(:)*WLF(:)*1.d-9/C/hP  !  [W] -> [phot/s]
      END SELECT

c     Area dimension:
      SELECT CASE( TRIM(WL_IRRAD_DIM(3)) )
      CASE('cm^2')
         SPF(:) = SPF(:)*1.d-4  ! [1/m^2] -> [1/cm^2]
      CASE('m^2')
      END SELECT

c     Spectral interval dimension:
      SELECT CASE( TRIM(WL_IRRAD_DIM(4)) )
      CASE('micron')
         SPF(:) = SPF(:)*1.d+3  ! [1/nm] -> [1/micron]
      CASE('nm')
      END SELECT

      IF(CHK_STAT('flu_sfl',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_sfl' )
         RTM_PTR%APR(1) = FLU_SCAL
         RTM_PTR%CRN(1) = FLU_SCAL
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
      ENDIF

c      WRITE(*,*) '      Emission spectra in dimension of solar spectra'
c      DO I = 1,NFL
c         WRITE(*,*) I, WLF(I), SPF(I)
c      ENDDO
c      PAUSE
c      STOP '   Flu_l'
      RETURN

 112  CALL WRITE_MESSAGE(R_NAME,'ERROR','Error reading data file')
      WRITE(EU,*) '    File: ', TRIM(FLU_NAME)
      RETURN 1

 113  CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
      WRITE(EU,'(a)')
     $    '    No data of the fluorescence emission spectra'
      WRITE(EU,'(a,a)')
     $    '    in the file: ',TRIM(FLU_NAME)
      WRITE(EU,'(a)')
     $    '    is found in the spectral range selected for calculation:'
      WRITE(EU,'(7x,F8.3,a,F8.3,a)') WL_START,'   -   ',WL_END,' nm'
      WRITE(EU,'(a)')
     $    '    Switch of parameter "Include land fluorescence" in the'
      WRITE(EU,'(a)')
     $    '    file "control.inp" or select another emission spectra.'
      RETURN 1

      END SUBROUTINE READ_LAND_FLU
c *****************************************************************

      SUBROUTINE SCALE_AMPL
      USE FLU_LAND,  ONLY: DGF_PAR, CONVERT_AMP
      USE INFO,      ONLY: RET_INFO
      USE IRRAD,     ONLY: WL_IRRAD_DIM
c     ------------------------------------------------------------------
      INTEGER KK
      REAL (KND), PARAMETER ::
     $              C = 299792458.0D0, ! Light speed [m/s]
     $              hP = 6.626071D-34  ! Planck's constant [J s]
c     ==================================================================

c     **** Input dimension of emission spectra is [mW/m2/sr/nm]  ****
c          Recalculate dimension of emission spectra to be the same
c          as the dimension of solar spectra.

c     ... Power dimension:
      CONVERT_AMP = 1.d0
      SELECT CASE( TRIM(WL_IRRAD_DIM(2)) )
      CASE('W')
         DGF_PAR(:)%Amp = DGF_PAR(:)%Amp*1.d-3 ! [mW] -> [W]
         CONVERT_AMP =  CONVERT_AMP*1.d-3
      CASE('phot/s')
         DGF_PAR(:)%Amp = DGF_PAR(:)%Amp*1.d-3 ! [mW] -> [W]
         CONVERT_AMP =  CONVERT_AMP*1.d-3
c         wavelength in [m] !!! should be multiplied by WL in [nm] !!!
         DGF_PAR(:)%Amp = DGF_PAR(:)%Amp*1.d-9/C/hP !  [W] -> [phot/s]
         CONVERT_AMP =  CONVERT_AMP*1.d-9/C/hP
      END SELECT

c     ... Area dimension:
      SELECT CASE( TRIM(WL_IRRAD_DIM(3)) )
      CASE('cm^2')
         DGF_PAR(:)%Amp = DGF_PAR(:)%Amp*1.d-4  ! [1/m^2] -> [1/cm^2]
         CONVERT_AMP =  CONVERT_AMP*1.d-4 
      CASE('m^2')
      END SELECT

c     ... Spectral interval dimension:
      SELECT CASE( TRIM(WL_IRRAD_DIM(4)) )
      CASE('micron')
         DGF_PAR(:)%Amp = DGF_PAR(:)%Amp*1.d+3  ! [1/nm] -> [1/micron]
         CONVERT_AMP =  CONVERT_AMP*1.d+3
      CASE('nm')
      END SELECT

      IF(CHK_STAT('flu_amp-1',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_amp-1' )
         RTM_PTR%APR(1) = DGF_PAR(1)%Amp
         RTM_PTR%CRN(1) = DGF_PAR(1)%Amp
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
         KK = FIND_IDX('flu_amp-1', RET_INFO%NAME)               
         RET_INFO(KK)%PAR_MIN = RET_INFO(KK)%PAR_MIN*CONVERT_AMP
         RET_INFO(KK)%PAR_MAX = RET_INFO(KK)%PAR_MAX*CONVERT_AMP
      ENDIF
      IF(CHK_STAT('flu_wlm-1',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_wlm-1' )
         RTM_PTR%APR(1) = DGF_PAR(1)%wlm
         RTM_PTR%CRN(1) = DGF_PAR(1)%wlm
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
      ENDIF
      IF(CHK_STAT('flu_hwhm-1',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_hwhm-1' )
         RTM_PTR%APR(1) = DGF_PAR(1)%hwhm
         RTM_PTR%CRN(1) = DGF_PAR(1)%hwhm
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
      ENDIF
      IF(CHK_STAT('flu_amp-2',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_amp-2' )
         RTM_PTR%APR(1) = DGF_PAR(2)%Amp
         RTM_PTR%CRN(1) = DGF_PAR(2)%Amp
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
         KK = FIND_IDX('flu_amp-2', RET_INFO%NAME)               
         RET_INFO(KK)%PAR_MIN = RET_INFO(KK)%PAR_MIN*CONVERT_AMP
         RET_INFO(KK)%PAR_MAX = RET_INFO(KK)%PAR_MAX*CONVERT_AMP
      ENDIF
      IF(CHK_STAT('flu_wlm-2',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_wlm-2' )
         RTM_PTR%APR(1) = DGF_PAR(2)%wlm
         RTM_PTR%CRN(1) = DGF_PAR(2)%wlm
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
      ENDIF
      IF(CHK_STAT('flu_hwhm-2',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'flu_hwhm-2' )
         RTM_PTR%APR(1) = DGF_PAR(2)%hwhm
         RTM_PTR%CRN(1) = DGF_PAR(2)%hwhm
         RTM_PTR%NXT(1) = RTM_PTR%CRN(1) 
      ENDIF

      RETURN      
      END SUBROUTINE SCALE_AMPL
c **********************************************************************
      
      SUBROUTINE DEALLOCATE_ARRAYS
      USE RTM_PARS, ONLY: RTM_ST 
      
      IF(ALLOCATED(ST_GAS_IDX)) DEALLOCATE(ST_GAS_IDX)
      IF(ALLOCATED(ZZZ_INP)) DEALLOCATE(ZZZ_INP)
      IF(ALLOCATED(SEG_NUM_OUT)) DEALLOCATE(SEG_NUM_OUT)
      IF(ALLOCATED(DEP_INP)) DEALLOCATE(DEP_INP)
      IF(ALLOCATED(DEP_WS)) DEALLOCATE(DEP_WS, DEP_VS)
      IF(ALLOCATED(R_GAM)) DEALLOCATE(R_GAM)
      IF(ALLOCATED(LA_FLAG)) DEALLOCATE(LA_FLAG)
      IF(ALLOCATED(SEG_WL)) DEALLOCATE(SEG_WL)
      IF(ALLOCATED(WL_BRF)) DEALLOCATE(WL_BRF, RefW_r, RefW_i)
      IF(ALLOCATED(WLF)) DEALLOCATE(WLF, SPF)
      IF(ALLOCATED(MSK_RTM)) DEALLOCATE(MSK_RTM)
      IF(ALLOCATED(WL_SEGS)) DEALLOCATE(WL_SEGS)
      IF(ALLOCATED(table_lists)) DEALLOCATE(table_lists)

      IF(ALLOCATED(RTM_ST)) THEN
         DO II = 1, SIZE(RTM_ST)
            CALL RTM_ST(II)%CLEAN
         END DO
         DEALLOCATE(RTM_ST)
      END IF

      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS

c     *****************************************************************
      Subroutine WriteOutput
      Integer RecL, is, ie
      Character (Len=:), Allocatable:: MesID
         If(DelayOut) Then
            If(OneOutFile) Then
               is = Index(Work_Dir_Inp,"/St") + 1
               If(is /= 1) Then
                  ie = Index(Work_Dir_Inp(is:),"/",Back=.True.)
                  If(ie /= 0) Then
                     MesID = Work_Dir_Inp(is:ie)//"_"
                  Else
                     MesID = Work_Dir_Inp(is:)//"_"
                  End If
               Else
                  MesID = ""
               End If
               RecL = MaxVal(OutD(:)%RecL)
               Open(NewUnit = CU, File=Dir_Out//'Results_All.dat', 
     $              RECL=RecL)
            End If
            
            Do II = 1, Size(OutD)
!               If(.Not.Associated(OutD(II)%First_Elem)) Cycle
               If(.Not.Associated(OutD(II)%Element)) Cycle
               If(OneOutFile) Then
                  write(lunit,*) '>>>>>'//MesID//OutD(II)%Name
                  Write(CU,'(A)') '>>>>>'//MesID//OutD(II)%Name
               Else
                  Open(NewUnit = CU, File=Dir_Out//OutD(II)%Name, 
     $                 RECL=OutD(II)%RecL)
               End If
               OutD(II)%ELEMENT => OutD(II)%First_Elem
               Do While (Associated(OutD(II)%ELEMENT))
!                  write(lunit,*)  OutD(II)%ELEMENT%S
                  Write(CU,'(A)') OutD(II)%ELEMENT%S
                  OutD(II)%NEXT => OutD(II)%ELEMENT%NEXT_ELEM               
                  DEALLOCATE (OutD(II)%ELEMENT)
                  NULLIFY (OutD(II)%ELEMENT)
                  OutD(II)%ELEMENT => OutD(II)%NEXT
               End Do
               If(OneOutFile) Then 
                  write(lunit,*) '<<<<<'//MesID//OutD(II)%Name
                  Write(CU,'(A)') '<<<<<'//MesID//OutD(II)%Name
               Else
                  Close(CU)
               End If
            End Do
            If(OneOutFile) Close(CU)
         End If
         Return
         End Subroutine WriteOutput
c *****************************************************************

      END
C******************************************************************


