C********************************************************************C
C
C	SCIATRAN - Version 4.0 - Last modification: 27 Mar 2017
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_NODE ()
c     ... Called by "rtm_init.f"

      USE LOG_PARS, ONLY: LOG_LEVEL, LUNIT, C_UNIT, N_SSO
      USE NODES
      USE ST_VARS,  ONLY: Pie
      USE TYPES,    ONLY: KND
c     ------------------------------------------------------------------
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GT_NODE'
      INTEGER             I, IDX(1)
      INTEGER, PARAMETER :: N_NODE_DEF = 100
      LOGICAL             ALLOCATE_ALL
      LOGICAL, ALLOCATABLE :: PLUS_FLAG(:) 
      REAL (KND), ALLOCATABLE :: X(:), Y(:), Z(:), W(:), PHI_NODE_1(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF(READ_NODES) THEN       ! Set the number of nodes
         IF(LOG_LEVEL(1) >= N_SSO)  
     $        WRITE(LUNIT,*) ' *** Reading: ', TRIM(FILE_NODES)
         OPEN(C_UNIT, FILE = TRIM(FILE_NODES), STATUS = 'OLD')
         READ(C_UNIT,*) N_NODE
      ELSE
         N_NODE = N_NODE_DEF
         IF(LOG_LEVEL(1) >= N_SSO)  
     $        WRITE(LUNIT,*) ' *** Reading: Default nodes'
      END IF

      ALLOCATE_ALL = .TRUE.     ! Check which arrays need to be allocated
      IF(ALLOCATED(MJU_NODE)) THEN
         IDX(1:1) = SHAPE(MJU_NODE)
         IF(IDX(1) /= N_NODE) THEN  ! Check if the number of nodes is same as before
            DEALLOCATE(PHI_NODE, MJU_NODE, WEIGHTS, SIN_NODE)
         ELSE
            ALLOCATE_ALL = .FALSE.
         END IF
      END IF

      IF(ALLOCATE_ALL) THEN
         ALLOCATE(PHI_NODE(N_NODE), MJU_NODE(N_NODE), WEIGHTS(N_NODE), ! A
     $        SIN_NODE(N_NODE), X(N_NODE), Y(N_NODE), Z(N_NODE), ! A
     $        W(N_NODE), PLUS_FLAG(N_NODE), PHI_NODE_1(N_NODE)) ! A
      ELSE
         ALLOCATE(X(N_NODE), Y(N_NODE), Z(N_NODE), W(N_NODE), ! A
     $        PLUS_FLAG(N_NODE), PHI_NODE_1(N_NODE)) ! A
      END IF

      IF(READ_NODES) THEN
         DO I =  1, N_NODE
            READ(C_UNIT,*) X(I), Y(I), Z(I), W(I)
         END DO
      ELSE
         CALL ST_NODES_DEF (N_NODE, X, Y, Z, W)
      END IF

      WHERE(X /= 0.0D0)
         PHI_NODE_1 = SIGN(DACOS(Y/DSQRT(Y**2 + X**2)),X)
      ELSEWHERE
         PHI_NODE_1 = 0.0D0
      END WHERE

      PLUS_FLAG = .FALSE.
      WHERE(Z >= 0.0D0)  PLUS_FLAG = .TRUE.
      N_PLUS = COUNT(PLUS_FLAG)
      N_MINUS = N_NODE - N_PLUS

      MJU_NODE(1:N_PLUS) = PACK(Z,PLUS_FLAG)

      MJU_NODE(N_PLUS + 1:N_NODE) = PACK(Z,.NOT.PLUS_FLAG)
      PHI_NODE(1:N_PLUS) = PACK(PHI_NODE_1,PLUS_FLAG) 
      PHI_NODE(N_PLUS + 1:N_NODE) = PACK(PHI_NODE_1,.NOT.PLUS_FLAG)
      WEIGHTS(1:N_PLUS) = PACK(W,PLUS_FLAG)
      WEIGHTS(N_PLUS + 1:N_NODE) = PACK(W,.NOT.PLUS_FLAG)

      WHERE( PHI_NODE(:) < 0.d0 ) PHI_NODE(:) = PHI_NODE(:) + 2.d0*Pie
      
      SIN_NODE = DSQRT(1.0D0 - MJU_NODE**2)
      NORM_DN = DOT_PRODUCT(WEIGHTS(N_PLUS + 1:N_NODE),
     $     MJU_NODE(N_PLUS + 1:N_NODE))

      MJU_MINUS => MJU_NODE(N_PLUS + 1:N_NODE)
      MJU_PLUS => MJU_NODE(1:N_PLUS)
      PHI_MINUS => PHI_NODE(N_PLUS + 1:N_NODE)
      W_MINUS => WEIGHTS(N_PLUS + 1:N_NODE)
      W_PLUS => WEIGHTS(1:N_PLUS)

c      DO I = 1, N_PLUS
c         WRITE(*,'(2x,I3,2(2x,F8.3))') I,
c     $        ACOS( MJU_PLUS(I) )/Pie*180.,
c     $        PHI_NODE(I)/Pie*180.
c      ENDDO
c      STOP '   Stop in "gt_node.f" '
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN 
      END   
C**************************************************************************
      SUBROUTINE ST_NODES_DEF (N_NODE_DEF, X_DEF, Y_DEF, Z_DEF, W_DEF)
      USE TYPES, ONLY: KND
      INTEGER             N_NODE_DEF
      REAL (KND), DIMENSION (N_NODE_DEF) :: X_DEF, Y_DEF, Z_DEF, W_DEF  
      
      X_DEF = (/0.00000000000000e+00,  5.58798019977000e-01,
     $     8.34367450478000e-01, -9.35749732373000e-01,
     $     2.88190362252000e-01, -7.72930564392000e-01,
     $     8.19236393744000e-01,  3.59549913985000e-01,
     $     4.13738979681000e-01,  8.27811547383000e-01,
     $     -1.52923564753000e-01, -1.40576181203000e-01,
     $     -8.79535313499000e-01,  3.65576345706000e-01,
     $     -1.10070814836000e-01, -6.82258336003000e-01,
     $     9.71051422841000e-01, -3.83231264954000e-01,
     $     6.14075397970000e-01,  2.58828229831000e-01,
     $     -9.63018054953000e-01,  2.53390009888000e-01,
     $     -7.96778412357000e-01,  9.76997136704000e-01,
     $     -6.70370532402000e-01,  5.35985432922000e-01,
     $     -3.87866579089000e-01,  6.45734959192000e-01,
     $     6.91110152203000e-01, -1.29514954434000e-01,
     $     -7.29890610700000e-02,  7.70871187730000e-02,
     $     -6.49225359894000e-01,  3.07539257280000e-01,
     $     2.75505549192000e-01, -1.64561315010000e-02,
     $     -4.92174334749000e-01,  4.98233017800000e-02,
     $     -4.21537280284000e-01, -4.20469615963000e-01,
     $     -7.38582201307000e-01, -9.56141937638000e-01,
     $     -8.64541807580000e-01, -9.09204419734000e-01,
     $     -6.10591498085000e-01,  8.99094909765000e-01,
     $     7.10485131255000e-01,  2.62663601806000e-01,
     $     -2.91247896194000e-01,  7.03637052557000e-01,
     $     -2.63340998420000e-01, -6.19795007373000e-01,
     $     6.68788216133000e-01, -1.76138670104000e-01,
     $     -7.32166771840000e-02,  4.32004294535000e-01,
     $     -3.84915136637000e-01,  5.82313506766000e-01,
     $     8.35718972909000e-01, -5.97811002850000e-02,
     $     -2.81215016390000e-01, -6.62527505218000e-01,
     $     -7.59627362125000e-01, -8.49469825270000e-02,
     $     4.14116082084000e-01,  1.13163033261000e-01,
     $     2.74150024492000e-01,  4.64057864447000e-01,
     $     -9.91553541490000e-01,  1.89940887658000e-01,
     $     9.09661814944000e-01,  1.83887252653000e-01,
     $     -3.68742456435000e-01,  6.72640762330000e-02,
     $     -4.77664998986000e-01, -6.12830546178000e-01,
     $     -2.76670908925000e-01,  7.22539234741000e-01,
     $     5.37880783882000e-01,  6.22354625420000e-01,
     $     8.30929042810000e-02, -8.00543772691000e-01,
     $     8.08402374646000e-01,  2.22615142753000e-01,
     $     -5.34415442160000e-02, -6.01427320565000e-01,
     $     -4.65564314394000e-01, -3.61041645910000e-01,
     $     5.40516496793000e-01,  8.29423781555000e-01,
     $     -4.92865948280000e-01, -6.05970742441000e-01,
     $     9.11796324792000e-01, -3.41878769580000e-02,
     $     5.60606113701000e-01, -8.42883814353000e-01,
     $     9.75922655568000e-01,  3.34307002381000e-01,
     $     -8.49491786927000e-01, -3.36429301446000e-01/)

      Y_DEF = (/0.00000000000000e+00,  0.00000000000000e+00,
     $     4.85623975239000e-01,  3.32151192678000e-01,
     $     6.19977315887000e-01,  3.72606732802000e-01,
     $     -3.71634786170000e-02, -3.72620616800000e-03,
     $     -8.89543524403000e-01, -5.10793607279000e-01,
     $     8.52909049123000e-01, -8.83818263984000e-01,
     $     -1.55461877870000e-01,  4.41682698139000e-01,
     $     6.09103095383000e-01,  7.30787578904000e-01,
     $     -7.31584403580000e-02, -5.34897525236000e-01,
     $     -7.80001606244000e-01,  1.34196486796000e-01,
     $     1.46097398966000e-01, -9.56541713863000e-01,
     $     -3.59684068220000e-01, -1.44917105397000e-01,
     $     4.85971257790000e-02, -3.05405698016000e-01,
     $     -1.63344278140000e-01, -5.41482827801000e-01,
     $     1.94558403080000e-02, -9.74529359786000e-01,
     $     -3.75564044204000e-01,  6.84126148744000e-01,
     $     -3.09920393157000e-01, -6.19218580676000e-01,
     $     -5.64784013836000e-01,  3.75624012187000e-01,
     $     7.05737534131000e-01, -9.86516528210000e-01,
     $     -5.26259387857000e-01,  8.81955670860000e-01,
     $     -5.21458454390000e-01, -2.18256398284000e-01,
     $     2.08182706771000e-01, -3.93013233589000e-01,
     $     6.98715851388000e-01,  2.61460808912000e-01,
     $     -6.72624095652000e-01, -2.35141295195000e-01,
     $     8.12256975238000e-01,  5.60509224523000e-01,
     $     3.05530845645000e-01,  4.02743258602000e-01,
     $     3.04919583421000e-01, -3.42559139372000e-01,
     $     -4.24262227500000e-03, -7.96680910655000e-01,
     $     9.10348246696000e-01, -3.46758405572000e-01,
     $     -2.28987786925000e-01, -6.85221668673000e-01,
     $     5.71078922170000e-01, -7.36126798120000e-01,
     $     5.60008427753000e-01,  9.87120166612000e-01,
     $     7.02249046280000e-01,  3.39596486583000e-01,
     $     9.61522312191000e-01,  3.23305850663000e-01,
     $     -2.34218127350000e-02,  9.04169947931000e-01,
     $     -4.04397305481000e-01, -3.39153655913000e-01,
     $     2.21444252135000e-01,  8.30899612908000e-01,
     $     5.36270679877000e-01, -2.20337478855000e-01,
     $     -8.18662936842000e-01,  3.62836496148000e-01,
     $     8.12970456788000e-01,  7.72089902368000e-01,
     $     -8.70526352779000e-01, -5.77830767601000e-01,
     $     -3.79787561874000e-01, -8.37434355460000e-01,
     $     -6.48386199411000e-01, -6.52006948769000e-01,
     $     -7.48007632960000e-01, -4.26945783740000e-02,
     $     -6.58291968538000e-01,  5.50065833067000e-01,
     $     -8.51356774961000e-01,  1.81579658472000e-01,
     $     1.25661629830000e-01,  9.73439046000000e-01,
     $     6.62923955196000e-01,  1.42575389900000e-03,
     $     2.16813598335000e-01,  8.83370719268000e-01,
     $     4.95340884074000e-01, -9.31385490065000e-01/)     

      Z_DEF = (/1.00000000000000e+00, -8.29303788048000e-01,
     $     2.60768694931000e-01, -1.18524358536000e-01,
     $     7.29776981612000e-01,  5.13558726242000e-01,
     $     -5.72250475774000e-01,  9.33118414105000e-01,
     $     -1.93732740871000e-01,  2.31986923733000e-01,
     $     -4.99159831385000e-01, -4.46210279494000e-01,
     $     -4.49721287953000e-01, -8.19310887042000e-01,
     $     7.85415708346000e-01, -2.17503442110000e-02,
     $     -2.27391681471000e-01, -7.53006264953000e-01,
     $     1.20452894798000e-01, -9.56555931650000e-01,
     $     2.26388550618000e-01,  1.44296405115000e-01,
     $     4.85563108845000e-01,  1.56446883749000e-01,
     $     -7.40433432965000e-01,  7.87049538031000e-01,
     $     -9.07126321757000e-01,  5.38351845613000e-01,
     $     7.22487527781000e-01,  1.83080319787000e-01,
     $     -9.23917878204000e-01, -7.25278559400000e-01,
     $     -6.94590369913000e-01,  7.22487338696000e-01,
     $     -7.77891837005000e-01, -9.26626028776000e-01,
     $     -5.09606669043000e-01, -1.55893483402000e-01,
     $     7.38483160284000e-01, -2.12977690590000e-01,
     $     -4.27290781854000e-01,  1.95337502025000e-01,
     $     -4.57413842759000e-01, -1.37433334237000e-01,
     $     3.72792410715000e-01, -3.51093418676000e-01,
     $     -2.06851889557000e-01, -9.35797202165000e-01,
     $     5.05384278683000e-01, -4.36719941717000e-01,
     $     9.15042305531000e-01, -6.73536945153000e-01,
     $     -6.78045993726000e-01,  9.22837149733000e-01,
     $     -9.97307033134000e-01,  4.22695890802000e-01,
     $     1.52007918627000e-01, -7.35302378619000e-01,
     $     4.99137648109000e-01, -7.25877045256000e-01,
     $     -7.71224337797000e-01, -1.38472531280000e-01,
     $     -3.30691444637000e-01, -1.35564696093000e-01,
     $     -5.79097701219000e-01,  9.33738911154000e-01,
     $     -1.77934603230000e-02,  8.24696080609000e-01,
     $     -1.27565642111000e-01, -3.82621437527000e-01,
     $     -9.47533733090000e-02,  9.22583479146000e-01,
     $     -9.02768765531000e-01,  5.52341721507000e-01,
     $     6.95880669834000e-01,  7.58874243260000e-01,
     $     5.03233746877000e-01,  5.88461325256000e-01,
     $     -2.23076889705000e-01,  1.28654198841000e-01,
     $     4.85056118789000e-01,  1.58875020122000e-01,
     $     -4.49718810494000e-01, -4.99145468288000e-01,
     $     7.59433563760000e-01,  4.61705660388000e-01,
     $     -4.73005761272000e-01,  9.31571845268000e-01,
     $     -5.23921368960000e-01, -9.73846491170000e-02,
     $     1.79651882135000e-01,  7.74485821003000e-01,
     $     3.90943239978000e-01,  2.26379355930000e-01,
     $     4.96238465768000e-01,  5.38093711845000e-01,
     $     2.38082743280000e-02,  3.28473744003000e-01,
     $     1.81662083299000e-01, -1.39055363167000e-01/)

      W_DEF = (/1.28280360518980e-01, 1.22141197576007e-01, 
     $     1.24119093954765e-01, 1.22140529717066e-01,
     $     1.29803409953984e-01, 1.25958792587123e-01,
     $     1.26865680167206e-01, 1.25915405990458e-01,
     $     1.26865231753498e-01, 1.29803314531125e-01,
     $     1.28279971757792e-01, 1.20404412106547e-01,
     $     1.28280028986458e-01, 1.29803227311726e-01,
     $     1.24118982337402e-01, 1.29803419786462e-01,
     $     1.25958747471631e-01, 1.29803666349522e-01,
     $     1.29803238573535e-01, 1.20402959337574e-01,
     $     1.28279835990002e-01, 1.24119578014095e-01,
     $     1.22140996821549e-01, 1.24119600754349e-01,
     $     1.25958987130243e-01, 1.25916898903636e-01,
     $     1.24119211030224e-01, 1.22140529676485e-01,
     $     1.28279865126101e-01, 1.31130002890018e-01,
     $     1.26865411917778e-01, 1.26865669702267e-01,
     $     1.20404202999664e-01, 1.28279696559666e-01,
     $     1.25915453704100e-01, 1.24119748782076e-01,
     $     1.20403878990697e-01, 1.25958827261070e-01,
     $     1.29803317613784e-01, 1.22140421875020e-01,
     $     1.22140442365614e-01, 1.25916197513366e-01,
     $     1.26865146905595e-01, 1.25915399406610e-01,
     $     1.24119533139075e-01, 1.24119590340720e-01,
     $     1.22140578477548e-01, 1.28280425547801e-01,
     $     1.25959024761470e-01, 1.20402875305687e-01,
     $     1.25958675359974e-01, 1.24119668815875e-01,
     $     1.29803263647228e-01, 1.22140885434590e-01,
     $     1.25958429658613e-01, 1.20403864890917e-01,
     $     1.26865265501058e-01, 1.25916270100718e-01,
     $     1.26865127376235e-01, 1.22140248362937e-01,
     $     1.25958526408302e-01, 1.26865491877346e-01,
     $     1.29803261272763e-01, 1.25916704558935e-01,
     $     1.22141395675019e-01, 1.26865727577352e-01,
     $     1.25915554261231e-01, 1.22140350413611e-01,
     $     1.25916607829215e-01, 1.25915975414305e-01,
     $     1.20403830830259e-01, 1.25915927199631e-01,
     $     1.31129954029297e-01, 1.20404592446922e-01,
     $     1.31130031796787e-01, 1.29803461279322e-01,
     $     1.24119462852075e-01, 1.20404380565651e-01,
     $     1.28280356551544e-01, 1.26865561948245e-01,
     $     1.25958613623168e-01, 1.28280447655917e-01,
     $     1.28279709204665e-01, 1.28280023276739e-01,
     $     1.26865864512943e-01, 1.20403142872105e-01,
     $     1.29803369510730e-01, 1.20403229863016e-01,
     $     1.25916684782504e-01, 1.25958564902665e-01,
     $     1.25958505781200e-01, 1.24119267097096e-01,
     $     1.25959084189823e-01, 1.28279784047612e-01,
     $     1.29803616792532e-01, 1.26865713556705e-01,
     $     1.31129852346085e-01, 1.22140242479776e-01,
     $     1.20403792310265e-01, 1.24119231324063e-01/)

      RETURN
      END
C**************************************************************************
