C********************************************************************C
C
C	SCIATRAN - Version 3.1 - Last modification: 08 Nov 2010
C
C	Copyright (C) 1995-2010, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_PC_INIT (INPUT_ARRAY, PCC)

      USE GRIDS,       ONLY: NLAYERS, ALT_EXT, NL_FINE, Z_INV, 
     $                       INTERP_IDX, Z_FINE
      USE INTERN_DIMS, ONLY: NL_PC, NL_FINE_PC
      USE PHOTOCHEM,   ONLY: N_SOL_PC
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: PCC_TYPE

      TYPE (PCC_type) :: PCC    
      INTEGER       I, J
      REAL (KND)    INPUT_ARRAY(NL_PC, N_SOL_PC)

      CALL ST_PCX (NLAYERS, ALT_EXT, INPUT_ARRAY, PCC%PCX_COEFF)
      DO J = 1, NL_FINE
         CALL ST_PCY (Z_INV(INTERP_IDX(J)), Z_FINE(J), 
     $        PCC%PCX_COEFF(INTERP_IDX(J),:,:), PCC%PCY_COEFF(:,J,:))
      END DO

      RETURN 
      END
C *************************************************************************

      SUBROUTINE ST_PCX (NLAYERS, PCX_EXT, TGA_PC, PCX_COEFF)

      USE PHOTOCHEMISTRY, ONLY: PC_LOG_FLAG, N_SOL_PC

      INTEGER             NLAYERS, I, IDX_Y

      DOUBLE PRECISION    PCX_EXT(NLAYERS + 4)
      DOUBLE PRECISION    TGA_PC(NLAYERS, N_SOL_PC)
      DOUBLE PRECISION    ST(NLAYERS + 4), T(NLAYERS + 4)
      DOUBLE PRECISION    FUNC_X(NLAYERS + 4)
      DOUBLE PRECISION    PCX_COEFF(NLAYERS, N_SOL_PC, 4)

      DO IDX_Y = 1, N_SOL_PC

C     Extrapolation of function:
C     Auxilliary points are moved 2 indices up
C     *****************************************************
         IF(PC_LOG_FLAG) THEN
            DO I = 1, NLAYERS
               FUNC_X(I + 2) = 
     $              DLOG(TGA_PC(NLAYERS - I + 1,N_SOL_PC - IDX_Y + 1))
            END DO
         ELSE
             DO I = 1, NLAYERS
               FUNC_X(I + 2) = 
     $               TGA_PC(NLAYERS - I + 1,N_SOL_PC - IDX_Y + 1)
            END DO
         END IF           

C     Caculation of slopes of curves
C     ******************************
         DO I = 3, NLAYERS + 1
            ST(I) = (FUNC_X(I + 1) - FUNC_X(I))/
     $           (PCX_EXT(I + 1) - PCX_EXT(I))
         END DO
            
         ST(2) = 2.0D0*ST(3) - ST(4)
         ST(1) = 2.0D0*ST(2) - ST(3)
         ST(NLAYERS + 2) = 2.0D0*ST(NLAYERS + 1) - ST(NLAYERS)
         ST(NLAYERS + 3) = 2.0D0*ST(NLAYERS + 2) - ST(NLAYERS + 1)
            
C     Calculation of all polynomial slopes
C     ************************************
         DO I = 3, NLAYERS + 2
            IF(ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1)) THEN
               T(I) = 0.5D0*(ST(I + 1) + ST(I))
            ELSE
               T(I) = (DABS(ST(I + 1) - ST(I))*ST(I - 1) +
     $              DABS(ST(I - 1) - ST(I - 2))*ST(I))/
     $              (DABS(ST(I + 1) - ST(I)) + 
     $              DABS(ST(I - 1) - ST(I - 2)))
            END IF
         END DO
            
C     Calculation of polynomial coefficients
C     **************************************
            
         DO I = 1, NLAYERS - 1
            PCX_COEFF(I,IDX_Y,1) = FUNC_X(I + 2)
            PCX_COEFF(I,IDX_Y,2) = T(I + 2)
            PCX_COEFF(I,IDX_Y,3) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2)  
     $           - T(I + 3))/(PCX_EXT(I + 3) - PCX_EXT(I + 2))
            PCX_COEFF(I,IDX_Y,4) = 
     $           (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))
     $           /(PCX_EXT(I + 3) - PCX_EXT(I + 2))**2
         END DO     

         PCX_COEFF(NLAYERS,IDX_Y,1) =  FUNC_X(NLAYERS + 2)

      END DO                    ! IDX_Y

      PCX_COEFF(NLAYERS,1:N_SOL_PC,2:4) =  0.0D0

      RETURN
      END
      
C *************************************************************************
      SUBROUTINE ST_PCY (X_AX, X_NEW, PCX_COEFF, PCY_COEFF)

      USE PHOTOCHEMISTRY

      INTEGER             IDX_Y, I

      DOUBLE PRECISION    PCX_COEFF(N_SOL_PC, 4), PCY_COEFF(N_SOL_PC, 4)
      DOUBLE PRECISION    ST(N_SOL_PC + 4), T(N_SOL_PC + 4)
      DOUBLE PRECISION    FUNC_Y(N_SOL_PC + 4)

      DOUBLE PRECISION    DELTA, X_NEW, X_AX

      DELTA = X_NEW - X_AX 
      
      DO IDX_Y = 1, N_SOL_PC
         FUNC_Y(IDX_Y + 2) = PCX_COEFF(IDX_Y,1) + 
     $        PCX_COEFF(IDX_Y,2)*DELTA + 
     $        PCX_COEFF(IDX_Y,3)*DELTA**2
     $        + PCX_COEFF(IDX_Y,4)*DELTA**3
      END DO

C     Caculation of slopes of curves
C     ******************************
      DO I = 3, N_SOL_PC + 1
         ST(I) = (FUNC_Y(I + 1) - FUNC_Y(I))/
     $        (PCY_LOC(I + 1) - PCY_LOC(I))
      END DO
      
      ST(2) = 2.0D0*ST(3) - ST(4)
      ST(1) = 2.0D0*ST(2) - ST(3)
      ST(N_SOL_PC + 2) = 2.0D0*ST(N_SOL_PC + 1) - ST(N_SOL_PC)
      ST(N_SOL_PC + 3) = 2.0D0*ST(N_SOL_PC + 2) - ST(N_SOL_PC + 1)
      
C     Calculation of all polynomial slopes
C     ************************************
      DO I = 3, N_SOL_PC + 2
         IF(ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1)) THEN
            T(I) = 0.5D0*(ST(I + 1) + ST(I))
         ELSE
            T(I) = (DABS(ST(I + 1) - ST(I))*ST(I - 1) +
     $           DABS(ST(I - 1) - ST(I - 2))*ST(I))/
     $           (DABS(ST(I + 1) - ST(I)) + 
     $           DABS(ST(I - 1) - ST(I - 2)))
         END IF
      END DO
      
C     Calculation of polynomial coefficients
C     **************************************
      
      DO I = 1, N_SOL_PC - 1
         PCY_COEFF(I,1) = FUNC_Y(I + 2)
         PCY_COEFF(I,2) = T(I + 2)
         PCY_COEFF(I,3) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2)  
     $        - T(I + 3))/(PCY_LOC(I + 3) - PCY_LOC(I + 2))
         PCY_COEFF(I,4) = 
     $        (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))
     $        /(PCY_LOC(I + 3) - PCY_LOC(I + 2))**2
      END DO

      PCY_COEFF(N_SOL_PC,1) = FUNC_Y(N_SOL_PC + 2)
      PCY_COEFF(N_SOL_PC,2:4) = 0.0D0

      RETURN 
      END

C *************************************************************************
