C********************************************************************C
C
C	SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_PHASE_LAYERS( LOA, LOW )
c     ... Call in "dom_spherical.f"

      USE AEROSOLS,    ONLY: LA_N, LA_S, LA_E
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT, 
     $                       Prof_Cpar
      USE CLD_INP,     ONLY: Phm_const, Deriv_Phc, MLCS
      USE GRIDS,       ONLY: NLAYERS, ZZZ
      USE HYDROSOLS,   ONLY: LA_N_HS, LA_S_HS, LA_E_HS
      USE INTERN_DIMS, ONLY: NL_ice, Nm_ice, Nm_vs,
     $                       NL_AER, NL_CLD, Nlg_aer, Nlg_cld
      USE GT_POLYN,    ONLY: NLEGEN
      USE ST_VARS,     ONLY: DO_AEROSOL
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE, LOW_type
      USE WATER,       ONLY: UWT
      USE WATER_ICE,   ONLY: IWS, IWT, LA_N_IW, LA_S_IW, LA_E_IW
      USE WF_NUM,      ONLY: NUM_CLD
c     ------------------------------------------------------------------

c     ----------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters of the atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of ocean
c     ----------------------
c     ... Local variables
      CHARACTER (LEN=15), PARAMETER :: R_NAME = 'GT_PHASE_LAYERS'
      INTEGER   I, J, K, L, Jt, Jb, CLN, N_db, Jmc, KK, IiW
      REAL (KND) DUM
      REAL (KND), ALLOCATABLE :: HH(:)
      REAL (KND), POINTER :: Php(:), Pef(:), Drm(:), Rdb(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      LOA%BET(:) = 0.d0
      
c     =====>  Aerosol present 
      IF( DO_AEROSOL ) THEN 
c         WRITE( *,* ) '  SHAPE( LOA%AER_MOMS )  ', SHAPE( LOA%AER_MOMS )
         DO I = 1,LA_N
            DO K = LA_S(I),LA_E(I)
               LOA%BET(K) = LOA%AER_MOMS(2,I,1)*LOA%SCA(K)
            ENDDO
         ENDDO
      ENDIF
c     =====>  Cloud present 
      IF( DO_CLOUD ) THEN
         DO J = 1,N_CLD_LAYERS
            Jt = J_TOP(J)+1 ; Jb = J_BOT(J)-1 ; CLN = Jb-Jt+1
c            IF( Phm_const ) THEN
            IF( MLCS(J)%Phm_co ) THEN
               Php => LOA%CLD_MOMS(2,J,1,:,1)
               DO Jmc = 1,Nm_ice
                  LOA%BET(Jt:Jb) = LOA%BET(Jt:Jb) + Php(Jmc)
     $                 *LOA%CSM(Jmc,Jt:Jb)
               ENDDO
               CYCLE  ! over cloud layers (J)
            ENDIF
            ALLOCATE( HH(CLN) )            
            KK = 2
            DO Jmc = 1, MLCS(J)%N_pha
c               WRITE(*,*) '  Nlc, Jmc, MLCS(J)%Npf(Jmc)%Ndb_ref  ',
c     $              J, Jmc, MLCS(J)%Npf(Jmc)%Ndb_ref
c               WRITE(*,*) '     --   ', LOA%CLD_MOMS(1,J,1,Jmc,1)
               N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
               IF( N_db == 1 ) THEN
                  KK = KK + 2 ;  CYCLE
               ENDIF
               Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
               Php => LOA%CLD_MOMS(2,J,:,Jmc,1)
               Pef => Prof_Cpar(Jt:Jb,KK)
               CALL LIN_INT(N_db, Rdb, Php, CLN, Pef, HH, 'L')
               LOA%BET(Jt:Jb) = LOA%BET(Jt:Jb)
     $               + HH(1:CLN)*LOA%CSM(Jmc,Jt:Jb)
               KK = KK + 2
            ENDDO ! over water/ice phase (Jmc)
c            WRITE(*,*) '      --------------------------------   '
            DEALLOCATE( HH )
         ENDDO  ! over cloud layers (J)
c   ======================================================================
c        ... Derivative with respect to moments of scattering matrix     
         DO J = 1,N_CLD_LAYERS
            IF( (Deriv_Phc.OR.NUM_CLD).AND.(.NOT.MLCS(J)%Phm_co)) THEN
               Jt = J_TOP(J)+1 ; Jb = J_BOT(J)-1 ; CLN = Jb-Jt+1
               DO Jmc = 1, MLCS(J)%N_pha
                  N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
                  Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
                  DO I = 1,NLEGEN
                     DO L = 1,Nm_vs
                        Php => LOA%CLD_MOMS(I,J,:,Jmc,L)
                        Drm => LOA%DR_MOMS (I,J,:,Jmc,L)
                        Drm(1) = (Php(2) - Php(1))/(Rdb(2) - Rdb(1))
                        DO K = 2,N_db-1
                           Drm(K) = (Php(K+1) - Php(K))
     $                          /(Rdb(K+1) - Rdb(K))
                        ENDDO  ! over R_eff in database (K)
                        Drm(N_db) = (Php(N_db) - Php(N_db-1))
     $                       /(Rdb(N_db) - Rdb(N_db-1))
                     ENDDO  ! over elements of scattering matrix (L)
                  ENDDO  ! over expansion coefficients (I)               
               ENDDO  ! over water/ice phase (Jmc)
            ENDIF
         ENDDO  ! over cloud layers (J)
      ENDIF  ! DO_CLOUD

      IF( .NOT.UWT ) THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      LOW%BET(:) = 0.D0 
      IiW = 1
      IF( IWS(1)%N_HPC > 0 ) THEN
         DO I = 1, LA_N_IW
            DO K = LA_S_IW(I),LA_E_IW(I)
              LOW%BET(K) = 
     $              SUM( LOW%HSOL_MOMS(2,I,1,:,IiW)*LOW%HSCA(K,:,IiW) )
            ENDDO
         ENDDO  ! over hydrosol layers (I)
      ENDIF
      IF( IWT ) IiW = IiW + 1
      IF( IWS(2)%N_HPC > 0 ) THEN
         DO I = 1, LA_N_HS
            DO K = LA_S_HS(I),LA_E_HS(I)
               LOW%BET(K) = LOW%BET(K) 
     $              + SUM(LOW%HSOL_MOMS(2,I,1,:,IiW)*LOW%HSCA(K,:,IiW))
            ENDDO
         ENDDO  ! over hydrosol layers (I)
      ENDIF
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
C********************************************************************C
