********************************************************************C
C
C	SCIATRAN - Version 4.5 - Last modification: 20 Oct 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_PHOTOCHEM (*)

      USE GAS_CONTROL, ONLY : GAS_INDEX, N_ST_MOL
      USE INFO, ONLY: TG_INFO 
      USE PHOTOCHEMISTRY, ONLY : DO_PHOTOCHEM, N_SOL_PC, GAS_IDX_PC,  
     $     XS_PC, DO_TROP, CONST_PROF, CONC_PC, N_PC_INP, N_PC_GASES,  
     $     PROFILE_NAME_PC, ALT_GRID_NAME_PC, CONST_SZA, Z_TP, 
     $     PC_LOG_FLAG
      USE LOG_PARS, ONLY: C_UNIT, LUNIT, EU, LOG_LEVEL, N_SSO
      USE GRIDS, ONLY : NLAYERS, ZZZ
      USE ST_VARS, ONLY : RAD_TO_DEG

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'GT_PHOTOCHEM'

      LOGICAL             INTERP_FLAG
      INTEGER             I_GAS, L_GAS, J, IDX_SOL, N_SOL, NL_PC
      DOUBLE PRECISION    ANG, OUT(1)
      DOUBLE PRECISION, PARAMETER :: CRIT = 1.0D-5    
      DOUBLE PRECISION, ALLOCATABLE :: CONC(:,:), SOL_ANG(:), Z_PC(:),
     $      CONC_READ(:,:), W(:)

      N_PC_GASES = COUNT(DO_PHOTOCHEM(1:N_ST_MOL))

      CALL ST_TRACK (R_NAME, 1, 'I')
      IF(ALLOCATED(CONC_PC)) DEALLOCATE(CONC_PC, XS_PC)
      L_GAS = 0
      DO I_GAS = 1, N_PC_INP
         IF(.NOT.DO_PHOTOCHEM(GAS_IDX_PC(I_GAS))) CYCLE
         L_GAS = L_GAS + 1

         CALL READ_PC(PROFILE_NAME_PC(I_GAS), ALT_GRID_NAME_PC(I_GAS))

         INTERP_FLAG = .FALSE.
         IF(L_GAS == 1) THEN
            N_SOL_PC = N_SOL
            ALLOCATE(CONC_PC(NLAYERS, N_PC_GASES, N_SOL_PC), ! A
     $           XS_PC(N_SOL_PC)) ! A
            XS_PC = SOL_ANG
         ELSE
            IF (N_SOL_PC /= N_SOL) THEN
               INTERP_FLAG = .TRUE.
            ELSE
               IF(ANY(DABS(XS_PC(:) - SOL_ANG(:)) > CRIT)) 
     $              INTERP_FLAG = .TRUE.
            END IF
         END IF
         
         ALLOCATE(CONC(NL_PC, N_SOL_PC)) ! A
         IF(INTERP_FLAG) THEN
            DO J = 1, NL_PC
               CALL AL_INTERP(N_SOL, SOL_ANG, CONC_READ(J,:), N_SOL_PC, 
     $              XS_PC, CONC(J,:), .FALSE., *880)
            END DO
         ELSE
            CONC(:,:) = CONC_READ(:,:)
         END IF

         CONC = CONC*1.0D+5
         DO IDX_SOL = 1, N_SOL_PC
            CALL AL_INTERP (NL_PC, Z_PC, CONC(:,IDX_SOL), NLAYERS, ZZZ, 
     $           CONC_PC(:,L_GAS,IDX_SOL), .FALSE., *880)
         END DO

         IF (DO_TROP(I_GAS)) THEN
            FORALL(IDX_SOL = 1:N_SOL_PC)
               WHERE(ZZZ(:) <= Z_TP) CONC_PC(:,L_GAS,IDX_SOL) = 
     $           TG_INFO(GAS_IDX_PC(I_GAS))%ND(:)
            END FORALL
         END IF         

         IF(XS_PC(2).GT.XS_PC(1)) THEN
            IF(.NOT.ALLOCATED(W)) ALLOCATE(W(N_SOL_PC)) ! A
            W(1:N_SOL_PC) =  XS_PC(N_SOL_PC:1:-1)
            XS_PC(1:N_SOL_PC) = W(1:N_SOL_PC)
            DO J = 1, NLAYERS
               W(1:N_SOL_PC) = CONC_PC(J,L_GAS,N_SOL_PC:1:-1)
               CONC_PC(J,L_GAS,1:N_SOL_PC) = W(1:N_SOL_PC)  
            END DO
         END IF

         IF(LOG_LEVEL(1) >= N_SSO) write(LUNIT,*) '******* '//
     $        TG_INFO(GAS_IDX_PC(I_GAS))%NameUC//
     $        ' photochemestry is included *******'

         PC_LOG_FLAG = .FALSE.
c         PC_LOG_FLAG = .TRUE.

         IF(CONST_PROF(I_GAS)) THEN
            ANG = DCOS(CONST_SZA/RAD_TO_DEG)
            IF(ANG.GE.XS_PC(1)) THEN
               FORALL(J = 1:NLAYERS) 
     $              CONC_PC(J,L_GAS,2:N_SOL_PC) = CONC_PC(J,L_GAS,1)
               GOTO 222
            END IF
            IF(ANG.LE.XS_PC(N_SOL_PC)) THEN
               FORALL(J = 1:NLAYERS) CONC_PC(J,L_GAS,1:N_SOL_PC - 1) = 
     $              CONC_PC(J,L_GAS,N_SOL_PC)
               GOTO 222
            END IF
            IF(.NOT.ALLOCATED(W)) ALLOCATE(W(N_SOL_PC)) ! A
            DO J = 1, NLAYERS
               IF(PC_LOG_FLAG) THEN
                  W(1:N_SOL_PC) = DLOG(CONC_PC(J,L_GAS,1:N_SOL_PC))
               ELSE
                  W(1:N_SOL_PC) = CONC_PC(J,L_GAS,1:N_SOL_PC)
               END IF
               CALL AL_INTERP(N_SOL_PC, XS_PC, W, 1, ANG, OUT, 
     $              .TRUE., *880)
               IF(PC_LOG_FLAG) THEN
                  CONC_PC(J,L_GAS,1:N_SOL_PC) = DEXP(OUT(1))
               ELSE
                  CONC_PC(J,L_GAS,1:N_SOL_PC) = OUT(1)
               END IF
            END DO
 222        CONTINUE
            IF(LOG_LEVEL(1) >= N_SSO) THEN
               write(LUNIT,*) '******* Constant profile for '
     $              //TG_INFO(GAS_IDX_PC(I_GAS))%NameUC//'  *********'
               write(LUNIT,*) 
            END IF
         END IF

         DEALLOCATE(CONC) 
      END DO

      IF(LOG_LEVEL(1) >= N_SSO) THEN
         write(LUNIT,*) DACOS(XS_PC(1))*RAD_TO_DEG, 
     $        DACOS(XS_PC(N_SOL_PC))*RAD_TO_DEG
         write(LUNIT,*)  
      END IF        

      CALL ST_TRACK (R_NAME, 1, 'I')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
C     ****************************************************************
      SUBROUTINE READ_PC (FILENAME_1, FILENAME_2)
      CHARACTER (LEN=100) FILENAME_1, FILENAME_2
      INTEGER  I
      DOUBLE PRECISION  SZA

      OPEN(C_UNIT,FILE = TRIM(FILENAME_1), STATUS = 'old', RECL=2000)
      READ(C_UNIT,*)
      READ(C_UNIT,*) NL_PC
      N_SOL = 0
      DO
         READ(C_UNIT,*,END=201)
         N_SOL = N_SOL + 1
      END DO
 201  CONTINUE
      
      REWIND(C_UNIT)
      IF(L_GAS > 1 ) DEALLOCATE(CONC_READ, SOL_ANG, Z_PC)
      ALLOCATE(CONC_READ(NL_PC,N_SOL), SOL_ANG(N_SOL), Z_PC(NL_PC)) ! A
      READ(C_UNIT,*)
      READ(C_UNIT,*)
      DO I = 1, N_SOL
         READ(C_UNIT,*) SZA, CONC_READ(:,I)
         SOL_ANG(I) = DCOS(SZA/RAD_TO_DEG)
      END DO
      CLOSE(C_UNIT)
      
      OPEN(C_UNIT,FILE = TRIM(FILENAME_2), STATUS = 'old', RECL=2000)
      DO I = 1, NL_PC
         READ(C_UNIT,*) Z_PC(I)
      END DO
      CLOSE(C_UNIT)
      
      END SUBROUTINE
C     ****************************************************************
      
      END

