!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 09 Oct 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine GT_PTCONC_SETUP (*)
  ! Called by "rtm_init.f"

  USE ABSORBERS, ONLY: N_ST_MOL, N_XSEC_MOL, N_LBL_PROF, N_GAS
  USE INFO,      ONLY: TG_INFO, RET_INFO
  USE GRIDS,     ONLY: NLAYERS, ZZZ, DZ
  USE LOG_PARS,  ONLY: EU, LUNIT, C_UNIT, LOG_LEVEL, N_ESO, N_SSO, DIR_OUT
  USE PROFILES,  ONLY: PRESS, AIR_DENSITY, TEMP, DO_SCA_PRE, DO_SCA_O3, &
                       DO_SCA_H2O, SCA_PRE, SCA_H2O, O3_TOTAL_COLUMN, &
                       DO_SCA_NO2, SCA_NO2
  USE RAMAN,     ONLY: O2_CONC
  USE RETR_ITER, ONLY: TG_SCALE_FLAG, TG_SF, NL_PRE, Z_PRE
  USE ST_VARS,   ONLY: FP, SMKM, FACTOR_O4, DO_RAMAN, CIRA_FLG
  USE ST_INPUT,  ONLY: PT_FILENAME, DO_PROFILES_AKIMA,PROF_FILES, PROF_UNITS,&
                       PROF_NCV, PROF_NCZ, PROF_LABELS, N_REPL, NC_PTZ, &
                       MAX_LAY, DO_PT_MPI_PROFILE, NC_P, NC_T, BRO_MIPAS_FLAG,&
                       PROF_UW_NAME, CLIM_LABEL, CHL_CONC_NAME, PT_CDF, PROF_CDF
  USE TOOLS,     ONLY: FIND_IDX, READ_FILE
  USE TYPE_VARS, ONLY: KND
  USE WATER,     ONLY: UWT, Ncol_UW, NC_UW, NL_UW => NLAYERS_UW, ZZZ_UW, &
                       TEMP_UW, PRES_UW, SAL_UW, NC_CHL, CHL_UW, FH_EXTR, &
                       Fua_c, Hua_c, UIT, N_FC, Flu_con, Conc_NAME, Scal_FC,&
                       CDOM_ACR, Fa_NAME, Ha_NAME
  USE WATER_ICE, ONLY: IWS, LoIW, IWT, LA_N_IW, Na_ice
  !------------------------------------------------------------------

  CHARACTER (LEN=*), PARAMETER :: R_NAME='GT_PTCONC_SETUP'
  CHARACTER (LEN = 20)  CN_FC
  INTEGER I, J, K, L, II, LL, NVLAD, KK, KK_1, KK_2, N_COLS, JT, JB, ILN, Nx
  LOGICAL FLG
  REAL (KND)   T_MIN, SLOPE, S1, S2, Z2(2), T2(2)
  REAL (KND)   VMR(MAX_LAY, MAX(N_XSEC_MOL - 1,N_LBL_PROF))
  REAL (KND), PARAMETER :: DDZ = 1.0D-4, &
  ! One DU is 2.69×10^16 ozone molecules per square centimetre:
       DU = 2.69D16, &
  ! One mole contains Avogadro's number entities. Avogadro's number:
       Avo_num = 6.02214D23, &
  ! One mole of water has a mass [grams]:
       Water_mol = 18.01528
  REAL (KND), DIMENSION (NLAYERS) :: TMP, W_OUT, TEMP_CLIM
  REAL (KND), DIMENSION (MAX_LAY) :: Z, PP, TT, S_S, DLP
  REAL (KND), ALLOCATABLE :: C_F(:,:), DATA_R(:,:), Z_act(:), Z_inp(:)
  ! ==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here

  WHERE( TG_INFO(:)%FM_FLG )  TG_INFO(:)%READ_FLG = .TRUE.

  ALLOCATE(PRESS(NLAYERS), TEMP(NLAYERS), AIR_DENSITY(NLAYERS)) ! A
  
  DO II = 1, N_ST_MOL
     IF(.NOT.TG_INFO(II)%READ_FLG) CYCLE
!     write(6,*) II, TG_INFO(II)%Name
     ALLOCATE (TG_INFO(II)%ND(NLAYERS), TG_INFO(II)%ND_INP(NLAYERS))
     TG_INFO(II)%ND = 0.0D0
  END DO
 
  IF(DO_RAMAN) ALLOCATE(O2_CONC(NLAYERS))

  PRESS = 0.d0 ; TEMP = 0.d0  ;  AIR_DENSITY = 0.d0 

  ! Read pressure, temperature. and trace gas VMRs from standard file 
  ! =================================================================
  ! LBL gases
  IF( ANY((TG_INFO%M_TYPE == 'line').AND.TG_INFO%READ_FLG) ) &
       CALL VLAD (NVLAD, PROF_FILES(2), Z, PP, TT, VMR(:,1:N_LBL_PROF), &
       N_LBL_PROF, *880)

  DO II = 1, SIZE(TG_INFO)
     IF(TG_INFO(II)%M_TYPE /= 'line') CYCLE
     IF(.NOT.TG_INFO(II)%READ_FLG)    CYCLE
     K = TG_INFO(II)%IDC
     CALL AL_INTERP (NVLAD, Z(1:NVLAD), VMR(1:NVLAD,K), NLAYERS, &
          ZZZ(1:NLAYERS), TG_INFO(II)%ND(:), DO_PROFILES_AKIMA, *125)
     WHERE(ZZZ(:) < Z(1))     TG_INFO(II)%ND(:) = VMR(1,K)
     WHERE(ZZZ(:) > Z(NVLAD)) TG_INFO(II)%ND(:) = VMR(NVLAD,K)
  END DO

  ! X-section and mixed gases
  FLG = .FALSE.
  DO K = 1, SIZE(TG_INFO)
     IF(TG_INFO(K)%NAME == 'o4') CYCLE
     IF(.NOT.TG_INFO(K)%READ_FLG) CYCLE
     IF(TG_INFO(K)%M_TYPE == 'xsec'.OR.TG_INFO(K)%M_TYPE == 'mixed')  THEN
        FLG = .TRUE.
        EXIT
     END IF
  END DO
  
  IF(DO_PT_MPI_PROFILE.OR.FLG) CALL VLAD(NVLAD, PROF_FILES(1), Z, &
       PP, TT, VMR(:,1:N_XSEC_MOL - 1), N_XSEC_MOL - 1, *880)

  DO II = 1, SIZE(TG_INFO)

     IF(TG_INFO(II)%M_TYPE /= 'xsec'.AND.TG_INFO(II)%M_TYPE /= 'mixed') CYCLE
     IF(.NOT.TG_INFO(II)%READ_FLG)    CYCLE
     IF(TG_INFO(II)%NAME == 'o4')     CYCLE

     K = TG_INFO(II)%IDC
     CALL AL_INTERP (NVLAD, Z(1:NVLAD), VMR(1:NVLAD,K), NLAYERS, &
          ZZZ(1:NLAYERS), TG_INFO(II)%ND(:), DO_PROFILES_AKIMA, *125)
     WHERE(ZZZ(:) < Z(1))     TG_INFO(II)%ND(:) = VMR(1,K)
     WHERE(ZZZ(:) > Z(NVLAD)) TG_INFO(II)%ND(:) = VMR(NVLAD,K)
  END DO

  N_COLS = MAX(NC_PTZ, NC_P, NC_T)
  IF (.NOT.DO_PT_MPI_PROFILE) Then
     Call READ_FILE (N_COLS, PT_FILENAME, DATA_R, *880, I_XC=NC_PTZ, CDF=PT_CDF)
     NVLAD = Size(DATA_R,1)
     Z(1:NVLAD) = DATA_R(:, NC_PTZ)
     PP(1:NVLAD) = DATA_R(:, NC_P)
     TT(1:NVLAD) = DATA_R(:,NC_T)

     IF(Z(1) > Z(NVLAD)) THEN
        Z(1:NVLAD) = Z(NVLAD:1:-1)
        PP(1:NVLAD) = PP(NVLAD:1:-1)
        TT(1:NVLAD) = TT(NVLAD:1:-1)
     END IF
  END IF

  ! Vertical profiles of atoms and ions
  DO II = 1, SIZE(TG_INFO)
     IF( TG_INFO(II)%NAME=='na'.AND.TG_INFO(II)%READ_FLG ) CALL Na_PROFILE
     IF( TG_INFO(II)%NAME=='mgi'.AND.TG_INFO(II)%READ_FLG ) CALL Mgi_PROFILE
     IF( TG_INFO(II)%NAME=='mga'.AND.TG_INFO(II)%READ_FLG ) CALL Mga_PROFILE
  ENDDO

  DLP(1:NVLAD) = DLOG(PP(1:NVLAD))

  ! Check if BOA extrapolation is required 
  ! (linear extrapolation for temperature and log-linear for pressure)
  ! ------------------------------------------------------------------
  IF (Z(1).LT.0.0D0)  CALL WRITE_MESSAGE(R_NAME,  &
       'WARNING', 'Lowest profile altitude < 0.0 km')
      
  IF (Z(1).GT.ZZZ(NLAYERS)) THEN
     IF(NVLAD + 1 > MAX_LAY) GOTO 121
     NVLAD = NVLAD + 1
     Z(1:NVLAD) = CSHIFT(Z(1:NVLAD),-1)
     TT(1:NVLAD) = CSHIFT(TT(1:NVLAD),-1)
     DLP(1:NVLAD) = CSHIFT(DLP(1:NVLAD),-1)

     Z(1) = ZZZ(NLAYERS) - DDZ
     SLOPE = (Z(1) - Z(2))/(Z(2) - Z(3))
     ! TT(1) = DMAX1(230D0,TT(2)+(TT(2)-TT(3))*SLOPE)
     ! TT(1) = TT(2)+(TT(2)-TT(3))*SLOPE
     TT(1) = TT(2)
     DLP(1) = DLP(2) + (DLP(2) - DLP(3))*SLOPE
  ENDIF
      
  ! Check if TOA extrapolation is required 
  ! (linear extrapolation for temperature and log-linear for pressure)
  ! ------------------------------------------------------------------
  IF (Z(NVLAD).LT.ZZZ(1)) THEN
     IF (NVLAD + 1.GT.MAX_LAY) GOTO 121
     NVLAD    = NVLAD + 1
     Z(NVLAD) = ZZZ(1) + DDZ
     SLOPE = (Z(NVLAD) - Z(NVLAD-1))/(Z(NVLAD-1) - Z(NVLAD-2))
     T_MIN = DMIN1(TT(NVLAD-1),190.0D0)
     TT(NVLAD) = DMAX1(T_MIN, TT(NVLAD-1) + (TT(NVLAD-1) - TT(NVLAD-2))*SLOPE)
     DLP(NVLAD) = DLP(NVLAD - 1) + (DLP(NVLAD - 1) - DLP(NVLAD - 2))*SLOPE
  ENDIF

  ! Interpolation of the temperature and of the logarithm
  ! of the pressure array. 
  ! ------------------------------------------------------------------
  CALL AL_INTERP (NVLAD, Z(1:NVLAD), DLP(1:NVLAD), NLAYERS, ZZZ(1:NLAYERS), &
       W_OUT(1:NLAYERS), DO_PROFILES_AKIMA, *122)
  PRESS(1:NLAYERS) = DEXP(W_OUT(1:NLAYERS))

  IF( DO_SCA_PRE )  PRESS(1:NLAYERS) = PRESS(1:NLAYERS)/PRESS(NLAYERS)*SCA_PRE

  KK = FIND_IDX('o3',TG_INFO%NAME)
  If(.Not.TG_INFO(KK)%READ_FLG) Then
     CLIM_LABEL = "NONE"
     DO_SCA_O3 = .FALSE.
  End If
  SELECT CASE(TRIM(CLIM_LABEL))
  CASE('NONE', 'UGAMP')
     CALL AL_INTERP(NVLAD, Z(1:NVLAD), TT(1:NVLAD), NLAYERS, ZZZ(1:NLAYERS), &
          TEMP(1:NLAYERS), DO_PROFILES_AKIMA, *123)
  CASE DEFAULT
     CALL CLIM_IFACE (TEMP_CLIM, TG_INFO(KK)%ND, *880)
     IF(DO_PT_MPI_PROFILE) THEN
        TEMP(1:NLAYERS) = TEMP_CLIM(1:NLAYERS)
     ELSE
        CALL AL_INTERP(NVLAD, Z(1:NVLAD), TT(1:NVLAD), NLAYERS, ZZZ(1:NLAYERS),&
             TEMP(1:NLAYERS), DO_PROFILES_AKIMA, *123)
     END IF
  END SELECT
  
  IF(CIRA_FLG) CALL T_CIRA (TEMP, *880)
  
  DO I = 1, NLAYERS
     IF (TEMP(I) .LE. 0.d0) GOTO 124
  END DO
  
  IF(BRO_MIPAS_FLAG) THEN
     KK = FIND_IDX('bro',TG_INFO%NAME)
     CALL READ_BRO_MIPAS (TG_INFO(KK)%ND, *880)
  END IF

  ! Z2(1) = ZZZ(1)
  ! Z2(2) = ZZZ(NLAYERS)
  ! T2(1) = 250.0
  !!T2(1) = 300.0
  ! T2(2) = 300.0
  ! CALL LIN_INT(2, Z2, T2, NLAYERS, ZZZ, TEMP, 'Z')
  
  ! Vertical homogeneous atmosphere      
  ! ===============================
  ! KK = FIND_IDX('no2',TG_INFO%NAME)
  ! TG_INFO(KK)%ND(1:NLAYERS) = TG_INFO(KK)%ND(1:NLAYERS)*0.1d0
  ! TG_INFO(KK)%ND(NLAYERS) = 20.d0*TG_INFO(KK)%ND(NLAYERS)
  ! PRESS(NLAYERS-5:NLAYERS) = PRESS(NLAYERS)
  ! TEMP(1:NLAYERS)  = TEMP (NLAYERS)
  ! WRITE(*,*) '  Pressure at surface ', PRESS(NLAYERS)
  ! PRESS(:) = 130.d0
  ! TEMP(:)  = 273.d0
  ! PAUSE

  !  WRITE(*,*) ' ****** Pressure from model : '
  !  WRITE(*,'(5x,f7.2)') PRESS(NLAYERS)
  !  PRESS(:) = (1.0d0 - 0.05349)*PRESS(:)
  !  PRESS(:) = PRESS(:)*1.001
  !  WRITE(*,*) ' ****** Pressure after correction : '
  !  WRITE(*,'(5x,f7.2)') PRESS(NLAYERS)
  !  TEMP(:)  = TEMP (:)*1.001d0
  !  TEMP(:)  = TEMP (:) + 0.1d0
  
  ! Compute the density profiles for tracegases and Rayleigh
  ! scattering
  ! ==================================================================
  
  ! According to the ideal gas low:
  ! N(I) = P(I)/(Kb*T(I)); 
  !    N = [mol/m^3], P = [Pa], Kb = [J/K] (Boltzman constant)
  ! N(I) = P(I)/(Kb*T(I))*100*1.e-6 = FP*P(I)/T(I), 
  !    N = [mol/cm^3], P = [hPa] = [mbar]
  ! SMKM = 1.E05 (cm --> km)

  AIR_DENSITY(:) = PRESS(:)*FP/TEMP(:)*SMKM
      
  DO KK = 1, N_ST_MOL
     IF( .NOT.TG_INFO(KK)%FM_FLG )    CYCLE
     IF( TG_INFO(KK)%M_TYPE=='atom' ) CYCLE
     SELECT CASE (TG_INFO(KK)%NAME)
     CASE ('o4') 
        TG_INFO(KK)%ND = (FACTOR_O4*AIR_DENSITY(:)*1.0D-20)**2/SMKM
     CASE DEFAULT
        TG_INFO(KK)%ND = TG_INFO(KK)%ND*AIR_DENSITY(:)*1.0D-6 
     END SELECT
  END DO
  
  IF(TRIM(CLIM_LABEL) == 'UGAMP') THEN
     KK = FIND_IDX('o3',TG_INFO%NAME)
     CALL UGAMP_READ(ZZZ, PRESS, TG_INFO(KK)%ND, *880)
  END IF

  ! open(37, file=DIR_OUT//'/pt.tmp')
  ! do i = 1, nlayers
  !!    write(37,*) zzz(i), press(i), temp(i)
  !    read(37,*) zzz(i), press(i), temp(i)
  ! end do
  ! close(37)
  ! AIR_DENSITY(:) = PRESS(:)*FP/TEMP(:)*SMKM
  
  ! open(37, file=DIR_OUT//'/conc.tmp')
  ! do i = 1, nlayers
  !    write(37,*) zzz(i),  TG_INFO(KK)%ND(i)
  !!    read(37,*) zzz(i),  TG_INFO(KK)%ND(i)
  ! end do
  ! close(37)
  ! stop
  ! KK = FIND_IDX('O3',TG_INFO%NAME)
  ! WHERE(ZZZ <= 6.0D0) TG_INFO(KK)%ND = TG_INFO(KK)%ND*10.
            
  DO I = 1, N_REPL
     KK = FIND_IDX(PROF_LABELS(I)%S,TG_INFO%NAME)
     IF(.NOT.TG_INFO(KK)%FM_FLG) CYCLE
     CALL READ_PROFILES (PROF_FILES(I+2), PROF_NCZ(I), PROF_NCV(I), &
          PROF_UNITS(I)%S, PROF_CDF(I+2)%S, W_OUT, *880)
     TG_INFO(KK)%ND = W_OUT(:)
  END DO

  ! KK = FIND_IDX('SO2',TG_INFO%NAME)
  ! TG_INFO(KK)%ND = TG_INFO(KK)%ND*100.

  ! Scale ozone profile according to the input vertical column
  ! ==========================================================
  ! One mole contains Avogadro's number (approximately 6.02214×10^23) 
  ! entities (atoms, molecules, elemental particles).
  ! One mole of water has a mass of 18.01528 grams.
  Scaling_O3: Block 
    IF(.Not.DO_SCA_O3) Exit Scaling_O3
    KK = FIND_IDX('o3',TG_INFO%NAME)
    IF(.Not.TG_INFO(KK)%FM_FLG) Exit Scaling_O3
    S1 = SUM( TG_INFO(KK)%ND*DZ(:) )/2.0d0/DU
    TG_INFO(KK)%ND = TG_INFO(KK)%ND/S1*O3_TOTAL_COLUMN
    ! IF (LOG_LEVEL(1) >= N_ESO) THEN
    WRITE(LUNIT,*) '  Ozone vertical column before and after scaling [DU]:' 
    WRITE(LUNIT,'(5x,2(2x,F8.3))') S1, SUM(TG_INFO(KK)%ND*DZ(:))/2.0d0/DU
    ! ENDIF
  End Block Scaling_O3
  Scaling_H2O: Block 
    IF(.Not.DO_SCA_H2O) Exit Scaling_H2O
    KK = FIND_IDX('h2o',TG_INFO%NAME)
    IF(.Not.TG_INFO(KK)%FM_FLG) Exit Scaling_H2O
    S1 = SUM(TG_INFO(KK)%ND*DZ(:))/2.0d0/Avo_num*Water_mol
    TG_INFO(KK)%ND = TG_INFO(KK)%ND/S1*SCA_H2O
    ! IF (LOG_LEVEL(1) >= N_ESO) THEN
    WRITE(LUNIT,*) '  H2O vertical column before and after scaling [gr/cm^2]:'
    WRITE(LUNIT,'(5x,2(2x,F8.3))')  S1, &
         SUM(TG_INFO(KK)%ND*DZ(:))/2.0d0/Avo_num*Water_mol
  End Block Scaling_H2O
  Scaling_NO2: Block 
    IF(.Not.DO_SCA_NO2) Exit Scaling_NO2
    KK = FIND_IDX('no2',TG_INFO%NAME)
    IF(.Not.TG_INFO(KK)%FM_FLG) Exit Scaling_NO2
    S1 = SUM(TG_INFO(KK)%ND*DZ(:))/2.0d0 
    TG_INFO(KK)%ND = TG_INFO(KK)%ND/S1*SCA_NO2
    ! IF (LOG_LEVEL(1) >= N_ESO) THEN
    WRITE(LUNIT,*) &
         '  NO2 vertical column before and after scaling [molecule/cm^2]:'
    WRITE(LUNIT,'(5x,2(2x,ES12.4))')  S1, SUM(TG_INFO(KK)%ND*DZ(:))/2.0d0
    ! ENDIF
  End Block Scaling_NO2
      
  ! ------------------
  ! Check some profiles
  ! ------------------
  ! IF (LOG_LEVEL(1) >= N_ESO) THEN ! Will not work if kk or kk_1 > 0!
  !    KK = FIND_IDX('o3',TG_INFO%NAME)
  !    KK_1 = FIND_IDX('h2o',TG_INFO%NAME)
  !    write(LUNIT,*)
  !    write(LUNIT,*) ' Some profiles on SCIATRAN altitude grid:'
  !    write(LUNIT,*) &
  !        ' No      z[km]    p[mbar]       t[K]    o3[ppm]'//&
  !        '   h2o[ppm]' 
  !    do i = 1, nlayers
  !       write(LUNIT,'(i4,5(d11.4))') i, zzz(i), press(i), &
  !           temp(i), TG_INFO(KK)%ND/air_density(i)*1d6,&
  !           TG_INFO(KK_1)%ND/air_density(i)*1d6
  !    enddo
  !    KK_1 = FIND_IDX('no2',TG_INFO%NAME)
  !    S1 = SUM( TG_INFO(KK)%ND*DZ(:) )/2.0d0
  !    S2 = SUM( TG_INFO(KK_1)%ND*DZ(:))/2.0d0
  !    WRITE(LUNIT,*) ' Some vertical columns:'
  !    WRITE(LUNIT,*)'         O3                    NO2     '
  !    WRITE(LUNIT,*)'    [DU]   [mol/cm^2]       [DU]   [mol/cm^2]'
  !    WRITE(LUNIT,'(2x,f7.2,1x,E12.4, 4x,f7.2,1x,E12.4)') S1/DU, S1, S2/DU, S2
  ! ENDIF
  !! STOP
  
  ! K = FIND_IDX('no2',TG_INFO%NAME)
  !! TG_INFO(KK)%ND = TG_INFO(KK)%ND/ 10.d0
  !! TG_INFO(KK)%ND = TG_INFO(KK)%ND/ 100.d0
  !! open(37, file=DIR_OUT//'/conc_no2_0.1.tmp')
  ! open(37, file=DIR_OUT//'/conc_no2.tmp')
  ! do i = 1, nlayers
  !    write(37,*) zzz(i),  TG_INFO(KK)%ND(i)
  ! end do
  ! close(37)

  ! KK = FIND_IDX('o3',TG_INFO%NAME)
  ! TG_INFO(KK)%ND = TG_INFO(KK)%ND*2.0d0
  ! WRITE(*,*) '   KK = ', KK
  ! TG_INFO(KK)%ND(NLAYERS-1) = TG_INFO(KK)%ND(NLAYERS-1)*10.d0
  ! TG_INFO(KK)%ND(NLAYERS) = TG_INFO(KK)%ND(NLAYERS)*10.d0
  ! TG_INFO(KK)%ND = TG_INFO(KK)%ND*1.5d0
  ! WHERE(ZZZ >= 10. .and. ZZZ <= 35.) &
  !     TG_INFO(KK)%ND = TG_INFO(KK)%ND*1.5d0
  ! KK = FIND_IDX('H2O',TG_INFO%NAME)
  ! TG_INFO(KK)%ND = TG_INFO(KK)%ND/4.0d0
  ! WHERE(ZZZ >= 15. .and. ZZZ <= 30.) &
  !     TG_INFO(KK)%ND = TG_INFO(KK)%ND*1.5d0
      
  DO II = 1, N_ST_MOL
     IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
     TG_INFO(II)%ND_INP = TG_INFO(II)%ND
  END DO
  
  IF(TG_SCALE_FLAG) THEN
     DO II = 1, N_ST_MOL
        IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
        CALL LIN_INT (NL_PRE, Z_PRE, TG_SF(:,II), NLAYERS, ZZZ, TMP, 'C')
        TG_INFO(II)%ND = TG_INFO(II)%ND*TMP(:)
     END DO
  END IF
  
  IF(DO_RAMAN) THEN
     ! Preliminary !
     O2_CONC(:) = 2.0948d+05*AIR_DENSITY(:)*1.d-6 
  ENDIF
  
  ! IF(.TRUE.) THEN
  IF(.FALSE.) THEN
     OPEN(C_UNIT, FILE=DIR_OUT//'/input_concetrations.dat')
     WRITE(C_UNIT,'(A$)') '# k   z   p   t '
     DO II = 1, N_ST_MOL
        IF(.NOT.TG_INFO(II)%READ_FLG) CYCLE
        WRITE(C_UNIT,'(2x,A$)') TG_INFO(II)%NAME
     END DO
     WRITE(C_UNIT,'(A)') '[molec/cm3]'
     DO I = 1, NLAYERS
        WRITE(C_UNIT,'(I3,2x,F6.2,10(1x,E11.4)$)') &
             NLAYERS - I + 1, ZZZ(I), PRESS(I), TEMP(I)
        DO II = 1, N_ST_MOL
           IF(.NOT.TG_INFO(II)%READ_FLG) CYCLE
           WRITE(C_UNIT,'(1x,E11.4$)') TG_INFO(II)%ND(I)/1.0D5 
        END DO
        WRITE(C_UNIT,*)
     END DO
     CLOSE(C_UNIT)
     ! STOP
  END IF

  ! Local routine
  CALL WRITE_ABSORBER      
  
  IF( .NOT.UWT ) THEN
     CALL ST_TRACK (R_NAME, 1, 'O')  ;  RETURN
  END IF
  !
  ! If radiative transfer through water and/or ice is calculated
  !
  CALL READ_UWT_PTS(PROF_UW_NAME, Z, PP, TT, S_S, NVLAD, *880)
  
  ALLOCATE( PRES_UW(NL_UW),TEMP_UW(NL_UW), SAL_UW(NL_UW) )
  CALL LIN_INT(NVLAD, Z(1:NVLAD), PP(1:NVLAD), NL_UW, ZZZ_UW, PRES_UW, 'L')
  CALL LIN_INT(NVLAD, Z(1:NVLAD), TT(1:NVLAD), NL_UW, ZZZ_UW, TEMP_UW, 'C')
  CALL LIN_INT(NVLAD, Z(1:NVLAD), S_S(1:NVLAD), NL_UW, ZZZ_UW, SAL_UW, 'C')

  ! Reading chlorophyll concentration [µg/l] = [mg/m^3] in water
  ALLOCATE( CHL_UW(NL_UW) )
  IF( UIT.AND.(.NOT.IWT) ) THEN
     CHL_UW(:) = 0.d0
  ELSE
     CALL READ_CHL(IWS(2)%CHL_CONC_NAME, Z, S_S, NVLAD, *880)
!     WRITE(*,*) '     Chlorophyl concentration before '
!     WRITE(*,*)  Z(1:NVLAD)
!     WRITE(*,*)  S_S(1:NVLAD)
     CALL LIN_INT(NVLAD, Z(1:NVLAD), S_S(1:NVLAD), NL_UW, ZZZ_UW, CHL_UW, 'Z')
!     WRITE(*,*) '     Chlorophyl concentration after '
!     WRITE(*,*)  ZZZ_UW(:)
!     WRITE(*,*)  CHL_UW(:)
  ENDIF
  ! Set concentration of Chl to zero within ice layers:
  IF( IWT ) THEN
     DO Ii = 1, LA_N_IW
        CHL_UW(IWS(1)%IW_TOP(Ii):IWS(1)%IW_BOT(Ii)) = 0.d0
     ENDDO
  ENDIF
  
  IF( LOG_LEVEL(1) >= N_ESO ) THEN
     WRITE(*,*) '  ',' Dep ','    Press ','  Temp','  Salin','  Chl'
     DO I = 1, NL_UW
        WRITE(*,'(1x,F8.4,1x,F7.2,2(1x,F6.2),1x,f8.4)') ZZZ_UW(I), &
             PRES_UW(I), TEMP_UW(I), SAL_UW(I), CHL_UW(I)
     ENDDO
  ENDIF
  ! STOP '    Stop after Chl reading'
      
  ! Reading concentration of fulvic acid [g/m^3]
  IF( IWS(2)%Absw_ful ) THEN
     ALLOCATE( Fua_c(NL_UW) )
     CALL READ_FH_ACIDS(Fa_NAME, Z, S_S, NVLAD, *880)
     ! Interpolation of fulvic acid concentration
     CALL LIN_INT(NVLAD, Z(1:NVLAD), S_S(1:NVLAD), NL_UW, ZZZ_UW, Fua_c, &
          FH_EXTR)
     IF( IWT ) THEN
        DO Ii = 1, LA_N_IW
           Fua_c(IWS(1)%IW_TOP(Ii):IWS(1)%IW_BOT(Ii)) = 0.d0
        ENDDO
     ENDIF
  ENDIF
  ! Reading concentration of humic acid [g/m^3]
  IF( IWS(2)%Absw_hum ) THEN
     ALLOCATE( Hua_c(NL_UW) )
     CALL READ_FH_ACIDS(Ha_NAME, Z, S_S, NVLAD, *880)
     ! Interpolation of humic acid concentration
     CALL LIN_INT(NVLAD, Z(1:NVLAD), S_S(1:NVLAD), NL_UW, ZZZ_UW, Hua_c, &
          FH_EXTR)
     IF( IWT ) THEN
        DO Ii = 1, LA_N_IW
           Hua_c(IWS(1)%IW_TOP(Ii):IWS(1)%IW_BOT(Ii)) = 0.d0
        ENDDO
     ENDIF
  ENDIF
  ! Reading CDOM as ACR
  IF( IWS(2)%CDOM_ACR ) THEN
     ALLOCATE( Flu_con(N_FC,NL_UW), C_F(N_FC,MAX_LAY) )
     CALL READ_F_CONC(Conc_NAME, Z, NVLAD, *880)
     DO I = 1,N_FC
        CALL LIN_INT(NVLAD, Z(1:NVLAD), C_F(I,1:NVLAD), NL_UW, ZZZ_UW, &
             Flu_con(I,:), FH_EXTR)
        ! Scaling of concentration profiles
        Flu_con(I,:) = Scal_FC(I)*Flu_con(I,:)
        IF( IWT ) THEN
           DO Ii = 1, LA_N_IW
              Flu_con(I,IWS(1)%IW_TOP(Ii):IWS(1)%IW_BOT(Ii)) = 0.d0
           ENDDO
        ENDIF
     ENDDO
  ENDIF

  ! Reading concentration of additional absorber in ice
  IF( ANY(IWS(1)%Na_W(:) > 0)) THEN
     ! Additional absorber within ice layers
     ALLOCATE( IWS(1)%AAC_P(NL_UW,Na_ice) )
     IWS(1)%AAC_P(:,:) = 0.d0
     KK = 0
     DO Ii = 1, LA_N_IW
        JT = IWS(1)%IW_TOP(Ii)  ; JB = IWS(1)%IW_BOT(Ii)
        ILN = JB - JT + 1
        ! Depth grid within ice layer
        ALLOCATE( Z_act(ILN) )  ;  Z_act(:) = ZZZ_UW(JT:JB)        
        DO I = 1, IWS(1)%Na_W(Ii)
           CALL READ_AAW(IWS(1)%AFN_conc(I,Ii)%S, Z, S_S, Nx, *880)
           IF( Z(Nx) > 1.d0.OR. Z(1) < 0.d0 ) GOTO 126
           ALLOCATE( Z_inp(Nx) )
           Z_inp(1:Nx) = Z_act(1) + Z(1:Nx)*(Z_act( ILN) - Z_act(1))
           KK = KK + 1
           CALL LIN_INT(Nx, Z_inp, S_S(1:Nx), ILN, Z_act, &
                IWS(1)%AAC_P(JT:JB,KK), 'Z')
           DEALLOCATE( Z_inp )
        ENDDO  ! over additional absorber in ice (I)
        DEALLOCATE( Z_act )
     ENDDO  ! over ice layers (Ii)
     ! STOP '   Stop in "gt_ptconc_setup.f" '
  ENDIF  ! additional absorber in ice layer
  ! ----------------------------------------------------------------------------
  ! Reading concentration of additional absorber in water [mg/m^3]
  IF( IWS(2)%Na_W(1) > 0 ) THEN
     ! Additional absorber within water
     ALLOCATE( IWS(2)%AAC_P(NL_UW,IWS(2)%Na_W(1)) )
     DO I = 1, IWS(2)%Na_W(1)
        CALL READ_AAW(IWS(2)%AFN_conc(I,1)%S, Z, S_S, Nx, *880)
        CALL LIN_INT(Nx, Z(1:Nx), S_S(1:Nx), NL_UW, ZZZ_UW, &
             IWS(2)%AAC_P(:,I), IWS(2)%Extr_T(I,1)%S)
     ENDDO  ! over additional absorber (I)
  ENDIF
  ! Set concentration of additional absorber in water to zero
  ! within all ice layers
  IF( IWT.AND.IWS(2)%Na_W(1) > 0 ) THEN
     DO I = 1, LA_N_IW
        IWS(2)%AAC_P(IWS(1)%IW_TOP(I):IWS(1)%IW_BOT(I),:) = 0.d0
     ENDDO
  ENDIF
      
  ! I = 1
  ! Additional absorber in ice
  IF( ANY(IWS(1)%Na_W(:) > 0).AND.LOG_LEVEL(1) >= N_ESO) THEN
     DO Ii = 1, LA_N_IW
        WRITE(CN_FC,'(I0)') Ii  ! Transform to a character
        WRITE(*,'(4x,A)') IWS(1)%Name_IW//TRIM(CN_FC)
        WRITE(*,'(A,*(2x,A))') '   Extrapolation type: ',&
             (IWS(1)%Extr_T(K,Ii)%S, K = 1,IWS(1)%Na_w(Ii))
     ENDDO
     WRITE(*,*) '  ',' Dep ','  Additional absorber '
     DO J = 1, NL_UW
        WRITE(*,'(1x,F8.4,*(2x,f8.4))') &
             ZZZ_UW(J), (IWS(1)%AAC_P(J,K), K=1,Na_ice)
     ENDDO  ! over depth (J)
  ENDIF
  ! I = 2
  ! Additional absorber in water
  IF( LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(*,'(4x,A)') IWS(2)%Name_IW
     IF( IWS(2)%Na_W(1) > 0 ) THEN
        WRITE(*,'(A,*(2x,A))') '   Extrapolation type: ',&
             (IWS(2)%Extr_T(K,1)%S, K = 1, IWS(2)%Na_W(1))
        WRITE(*,*) '  ',' Dep ','  Additional absorber '
        DO J = 1, NL_UW
           WRITE(*,'(1x,F8.4,*(2x,f8.4))') &
                ZZZ_UW(J), (IWS(2)%AAC_P(J,K), K=1,IWS(2)%Na_W(1))
        ENDDO  ! over depth (J)
        WRITE(*,'(A)') '    ---------------------- '
     ELSE
        WRITE(*,'(A)') '    No additional absorber '
        WRITE(*,'(A)') '    ---------------------- '
     ENDIF
  ENDIF  ! LOG_LEVEL(1) >= N_ESO
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  !******************************************************************
  
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Too many layers required: max_lay too small')
  RETURN 1

122 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error (pressure)')
  RETURN 1

123 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error (temperature)')
  RETURN 1
  
124 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Temperature less/equal zero')
  WRITE(EU,*) '    Diagnostics (layer#,z,T): ', I, ZZZ(I), TEMP(I)
  RETURN 1
  
125 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error ('//&
         TRIM(TG_INFO(K)%Name)//')')
  RETURN 1
      
126 CALL WRITE_MESSAGE(R_NAME,'ERROR','  Input error of depth grid')
  WRITE(EU,*) '      Depth grid for absorption concentration within'
  WRITE(EU,*)'      ice layer has to be dimensionless.'
  WRITE(EU,*)'      Additional absorber: ', I
  WRITE(EU,*)'      Input depth grid: ', Z(1), Z(Nx)
  WRITE(EU,*)'      Check input file: ', IWS(1)%AFN_conc(I,Ii)%S
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
! **********************************************************************

  ! Calculate model global vertical profile of Na atoms represented
  ! as Gaussian distribution with a given mean and variance
  SUBROUTINE Na_PROFILE
    ! Local routine
    USE ST_VARS,    ONLY:  Pie
    ! ------------------------------------------------------------------
    INTEGER I
    REAL (KND)  S1, SIG, Z_MAX, VCD
    REAL (KND), PARAMETER :: P_CAT = 1.d4
    !      Variance        Mean        Column 1/cm^2
    !     SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 3.75d9
    !     SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 5.0d9
    !     SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 6.25d9
    !     SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 7.5d9
    !==================================================================
    ! VCD   = 9.0d9
    ! Z_MAX = 92.0
    ! SIG   = 4.5
    VCD   = TG_INFO(II)%GVPP(1)
    Z_MAX = TG_INFO(II)%GVPP(2)
    SIG   = TG_INFO(II)%GVPP(3)
    
    DO I = 1,NLAYERS
       TG_INFO(II)%ND(I) = DEXP(-0.5d0*((ZZZ(I)-Z_MAX)/SIG)**2)
    ENDDO
    TG_INFO(II)%ND(:) = 1.d0/SIG/DSQRT(2.d0*Pie)*TG_INFO(II)%ND(:)
    
    ! Normalization factor
    S1 = SUM( TG_INFO(II)%ND*DZ(:) )/2.d0
    
    ! Concentration [atoms/cm^2/km]
    TG_INFO(II)%ND(:) = VCD/S1*TG_INFO(II)%ND(:)

    !      S1 = MAXVAL( TG_INFO(II)%ND(:) )
    !      WHERE( TG_INFO(II)%ND(:) < S1/P_CAT ) TG_INFO(II)%ND(:) = 0.d0
    !      WHERE( (1.d0 - TG_INFO(II)%ND(:)/S1) < 1.d0/P_CAT ) &
    !          TG_INFO(II)%ND(:) = 0.d0
    
  END SUBROUTINE Na_PROFILE
  !***********************************************************************
  ! Calculate model global vertical profile of Mgi ions represented
  ! as Gaussian distribution with a given mean and variance
  SUBROUTINE Mgi_PROFILE
    ! Local routine
    USE ST_VARS,    ONLY:  Pie
    ! ------------------------------------------------------------------
    INTEGER I
    REAL (KND)  S1, SIG, Z_MAX, VCD
    REAL (KND), PARAMETER :: P_CAT = 1.d4
    ! Variance        Mean        Column 1/cm^2
    ! SIG = 4.5d0, Z_MAX = 96.d0,  VCD = 3.75d9
    ! SIG = 4.5d0, Z_MAX = 96.d0,  VCD = 5.0d9
    ! SIG = 4.5d0, Z_MAX = 96.d0,  VCD = 6.250d9
    ! ==================================================================

    VCD   = TG_INFO(II)%GVPP(1)
    Z_MAX = TG_INFO(II)%GVPP(2)
    SIG   = TG_INFO(II)%GVPP(3)
    
    DO I = 1,NLAYERS
       TG_INFO(II)%ND(I) = DEXP(-0.5d0*((ZZZ(I)-Z_MAX)/SIG)**2)
    ENDDO
    TG_INFO(II)%ND(:) = 1.d0/SIG/DSQRT(2.d0*Pie)*TG_INFO(II)%ND(:)
    
    ! Normalization factor
    S1 = SUM( TG_INFO(II)%ND*DZ(:) )/2.d0
    
    ! Concentration [atoms/cm^2/km]
    TG_INFO(II)%ND(:) = VCD/S1*TG_INFO(II)%ND(:)

    ! S1 = MAXVAL( TG_INFO(II)%ND(:) )
    ! WHERE( TG_INFO(II)%ND(:) < S1/P_CAT ) TG_INFO(II)%ND(:) = 0.d0

  END SUBROUTINE Mgi_PROFILE

  !**********************************************************************
  ! Calculate model global vertical profile of Mg atom  represented
  ! as Gaussian distribution with a given mean and variance
  SUBROUTINE Mga_PROFILE
    ! Local routine
    USE ST_VARS,    ONLY:  Pie
    ! ------------------------------------------------------------------
    INTEGER I
    REAL (KND)  S1, SIG, Z_MAX, VCD
    REAL (KND), PARAMETER :: P_CAT = 1.d4 
    ! Variance        Mean        Column 1/cm^2
    ! SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 3.1250d9
    ! SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 2.50d9
    ! SIG = 4.5d0, Z_MAX = 92.d0,  VCD = 1.875d9
    ! ==================================================================
    
    VCD   = TG_INFO(II)%GVPP(1)
    Z_MAX = TG_INFO(II)%GVPP(2)
    SIG   = TG_INFO(II)%GVPP(3)
    
    DO I = 1,NLAYERS
       TG_INFO(II)%ND(I) = DEXP(-0.5d0*((ZZZ(I)-Z_MAX)/SIG)**2)
    ENDDO
    TG_INFO(II)%ND(:) = 1.d0/SIG/DSQRT(2.d0*Pie)*TG_INFO(II)%ND(:)
    
    ! Normalization factor
    S1 = SUM( TG_INFO(II)%ND*DZ(:) )/2.d0
    
    ! Concentration [atoms/cm^2/km]
    TG_INFO(II)%ND(:) = VCD/S1*TG_INFO(II)%ND(:)
    
    ! S1 = MAXVAL( TG_INFO(II)%ND(:) )
    ! WHERE( TG_INFO(II)%ND(:) < S1/P_CAT ) TG_INFO(II)%ND(:) = 0.d0

  END SUBROUTINE Mga_PROFILE
  !**********************************************************************

  SUBROUTINE READ_PROFILES (FILENAME, NC_Z, NC_VAL, UNITS, FLG_CDF, VAL_OUT, *)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_PROFILES'
    
    INTEGER             NC_Z, NC_VAL, NL, I_MIN, I_MAX
    CHARACTER (LEN = *) FILENAME, UNITS, FLG_CDF
    REAL (KND)  VAL_OUT(NLAYERS)
    REAL (KND), ALLOCATABLE :: Z_READ(:), VAL_READ(:)
    REAL (KND)  AD
    
    N_COLS = MAX(NC_Z, NC_VAL)
    Call READ_FILE (N_COLS, FILENAME, DATA_R, *880, I_XC=NC_Z, CDF=FLG_CDF)
    NL = Size(DATA_R,1)
    ALLOCATE(Z_READ(NL), VAL_READ(NL))
    Z_READ(:) = DATA_R(:,NC_Z) ;  VAL_READ(:) = DATA_R(:,NC_VAL)
    
    IF(Z(1) > Z(NL)) THEN
       Z_READ(:) = Z_READ(NL:1:-1) ;  VAL_READ(:) = VAL_READ(NL:1:-1)
    END IF
    
    CALL AL_INTERP (NL, Z_READ(1:NL), VAL_READ(1:NL), NLAYERS, ZZZ(1:NLAYERS),&
         VAL_OUT(:), DO_PROFILES_AKIMA, *122)
    I_MIN = MINLOC(Z_READ(1:NL),1)
    I_MAX = MAXLOC(Z_READ(1:NL),1)
    SELECT CASE (UNITS)
    CASE('vmr')
       WHERE(ZZZ(:) < Z_READ(I_MIN)) VAL_OUT(:) = VAL_READ(I_MIN)
       WHERE(ZZZ(:) > Z_READ(I_MAX)) VAL_OUT(:) = VAL_READ(I_MAX)
       VAL_OUT(:) = VAL_OUT(:)*AIR_DENSITY(:)*1.0D-6
    CASE('conc',"molec/m3","molec/cm3")
       IF(ZZZ(NLAYERS) < Z_READ(I_MIN)) THEN
          CALL AL_INTERP (NLAYERS, ZZZ, AIR_DENSITY, 1, Z_READ(I_MIN), &
               AD, .FALSE., *122)
          WHERE(ZZZ(:) < Z_READ(I_MIN)) &
               VAL_OUT(:) = VAL_READ(I_MIN)/AD*AIR_DENSITY(:)
       END IF
       IF(ZZZ(1) > Z_READ(I_MAX)) THEN
          CALL AL_INTERP (NLAYERS, ZZZ, AIR_DENSITY, 1, Z_READ(I_MAX),&
               AD, DO_PROFILES_AKIMA, *122)
          WHERE(ZZZ(:) > Z_READ(I_MAX)) &
               VAL_OUT(:) = VAL_READ(I_MAX)/AD*AIR_DENSITY(:)
       END IF
       If (UNITS == "molec/m3") Then
          VAL_OUT(:) = VAL_OUT(:)*1.0D+3 !m to km conversion
       Else
          VAL_OUT(:) = VAL_OUT(:)*SMKM
       End If
    CASE DEFAULT
       CALL WRITE_MESSAGE(R_NAME, 'ERROR','Wrong profile units')
       WRITE(EU,*) '    Selected units : ', TRIM(UNITS)
    END SELECT
    
    CLOSE(C_UNIT)
    RETURN

122 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Interpolation error (replacement profile)')
    RETURN 1
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
  END SUBROUTINE READ_PROFILES
!**********************************************************************

  SUBROUTINE WRITE_ABSORBER
    
    USE INFO,       ONLY: TG_INFO
    USE LOG_PARS,   ONLY: DIR_OUT
    USE TOOLS,      ONLY: SET_PTR, OpenOut, WriteOut, CloseOut 
    ! -----------------------------------------------------------------

    INTEGER  I, K, KO4, KO2, L_GAS, Unt
    CHARACTER (LEN=3) On_Off
    CHARACTER (LEN=8) T11
    Character (Len=500)  OutStr
    Character (Len=:), Allocatable :: TmpStr
    ! ==================================================================

    L_GAS =  0
    DO II = 1, N_ST_MOL
       IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
       SELECT CASE (TG_INFO(II)%NAME)
       CASE ('o2', 'o4')
          CYCLE
       CASE DEFAULT
          L_GAS = L_GAS + 1
       END SELECT
    ENDDO
    
    Call OpenOut ('SCE_ABSORBER.OUT', Unt)
    CALL WRITE_UD(1,Unt)
    WRITE(OutStr,'(11x,32A)') ('=', I = 1,32)
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,'(12x,A)') 'Gaseous absorber in atmosphere'
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,'(11x,32A)') ('=', I = 1,32)
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*)
    Call WriteOut (Unt, OutStr)
    On_Off = 'off'
    KK = FIND_IDX('o4', TG_INFO%NAME)
    IF( TG_INFO(KK)%FM_FLG ) On_Off = 'on'
    WRITE(OutStr,'(3x,A,1x,A)')'Absorption by O4 :',TRIM(On_Off)
    Call WriteOut (Unt, OutStr)
    On_Off = 'off'
    KK = FIND_IDX('o2', TG_INFO%NAME)
    IF( TG_INFO(KK)%FM_FLG ) On_Off = 'on'
    WRITE(OutStr,'(3x,A,1x,A)')'              O2 :', TRIM(On_Off)
    Call WriteOut (Unt, OutStr)
    
    WRITE(OutStr,*)
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,'(1x,A, 3x,A, 1x,A, 1x,A, 1x,A)') &
         '   N','  Z  ', '    P     ', '   T  ',' Air dens'
    TmpStr = Trim(OutStr)
    DO II = 1,N_ST_MOL
       IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
       SELECT CASE (TG_INFO(II)%NAME)
       CASE ('o2', 'o4')
          CYCLE
       CASE DEFAULT
          WRITE(OutStr,'(6x,A4)') TG_INFO(II)%NAME
          TmpStr = TmpStr//OutStr(1:10)
       END SELECT
    ENDDO
    Call WriteOut (Unt, TmpStr)
    
    T11 = 'mol/cm^3'
    WRITE(OutStr,'(6x,A, 3x,A, 1x,A6, 1x, *(2x,A))')&
         '   km','   mbar  ','    K ', T11, (T11, K=1,L_GAS)
    Call WriteOut (Unt, OutStr)
    
    WRITE(OutStr,'(2x,250A)') ('-', I = 1,39+L_GAS*10)
    Call WriteOut (Unt, OutStr)
    
    DO I = 1, NLAYERS
       WRITE(OutStr,'(1x,I4, 1x,F8.4, 1x,ES10.3, 1x,F6.1, 1x,ES9.2)') &
            I, ZZZ(I), PRESS(I), TEMP(I), AIR_DENSITY(I)/1.d5
       TmpStr = Trim(OutStr)
       DO II = 1,N_ST_MOL
          IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
          SELECT CASE (TG_INFO(II)%NAME)
          CASE ('o2', 'o4')
             CYCLE
          CASE DEFAULT
             WRITE(OutStr,'(1x,ES9.2)') TG_INFO(II)%ND(I)/1.d5
             TmpStr = TmpStr//Trim(OutStr)
          END SELECT
       ENDDO
       Call WriteOut (Unt, TmpStr)
    ENDDO
    
    WRITE(OutStr,'(2x,250A)') ('-', I = 1,39+L_GAS*10)
    Call WriteOut (Unt, OutStr)
    
    TmpStr = ' '
    Do II = 1, 38
       TmpStr = TmpStr//' '
    End Do
    DO II = 1,N_ST_MOL
       IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
       SELECT CASE (TG_INFO(II)%NAME)
       CASE ('o2', 'o4')
          CYCLE
       CASE DEFAULT
          WRITE(OutStr,'(6x,A4)') TG_INFO(II)%NAME
          TmpStr = TmpStr//OutStr(1:10)
       END SELECT
    ENDDO
    Call WriteOut (Unt, TmpStr)
    
    IF( N_GAS > 0 ) THEN
       WRITE(OutStr,'(2x,A,10A)') 
       TmpStr = '  Vertical column [mol/cm^2]:          '
       DO II = 1,N_ST_MOL
          IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
          SELECT CASE (TG_INFO(II)%NAME)
          CASE ('o2', 'o4')
             CYCLE
          CASE DEFAULT
             WRITE(OutStr,'(1x,ES10.3)') SUM(TG_INFO(II)%ND(:)*DZ(:))/2.d0
             TmpStr = TmpStr//Trim(OutStr)
          END SELECT
       ENDDO
       Call WriteOut (Unt, TmpStr)
       
       K = FIND_IDX('o3',TG_INFO%NAME)
       IF( TG_INFO(K)%FM_FLG ) THEN
          WRITE(OutStr,'(2x,A, 2x, F6.2)') 'Ozone vertical column [DU]:',&
               SUM(TG_INFO(K)%ND(:)*DZ(:))/2.d0/DU
          Call WriteOut (Unt, OutStr)
       ENDIF
    ENDIF
    CALL WRITE_UD(2,Unt)
    Call CloseOut (Unt)
    
    RETURN
  END SUBROUTINE WRITE_ABSORBER
  ! ********************************************************************

  SUBROUTINE READ_BRO_MIPAS (BRO_VMR, *)
    USE ST_INPUT, ONLY: DAY_OF_YEAR, YEAR, BRO_MIPAS_FILE
    USE ST_VARS,  ONLY: LATITUDE
    INTEGER              I, MONTH, DAY, J
    INTEGER, PARAMETER :: N_LATS = 35, N_ALT = 42
    CHARACTER (LEN = 30) LINE
    DOUBLE PRECISION     BRO_VMR(NLAYERS), MIPAS_ALT(N_ALT)
    DOUBLE PRECISION     MIPAS_LATS(N_LATS), MIPAS_VMR(N_ALT,N_LATS)
    DOUBLE PRECISION     MIPAS_LOCAL(N_ALT)
    
    CALL CALDAY(DAY_OF_YEAR, YEAR, MONTH, DAY ) 
    
    IF (LOG_LEVEL(1) >= N_SSO) &
         WRITE(LUNIT,*) ' *** Reading: '//TRIM(BRO_MIPAS_FILE) 
    OPEN (C_UNIT, FILE = TRIM(BRO_MIPAS_FILE), STATUS='OLD') 
    DO
       READ(C_UNIT,'(A)') LINE
       SELECT CASE (TRIM(LINE))
       CASE('Altitudes:')
          READ(C_UNIT,*) MIPAS_ALT
       CASE('Latitudes:')
          READ(C_UNIT,*) MIPAS_LATS
       CASE('month =  1')
          DO I = 1, (MONTH - 1)*(N_LATS + 1)
             READ(C_UNIT,*) LINE
          END DO
          READ(C_UNIT,*) MIPAS_VMR
          EXIT
       CASE DEFAULT
          CYCLE
       END SELECT
    END DO
    CLOSE(C_UNIT)      
    
    DO J = 1, N_ALT
       CALL AL_INTERP (N_LATS, MIPAS_LATS(:), MIPAS_VMR(J,:), 1, & 
            LATITUDE, MIPAS_LOCAL(J), .FALSE., *122)
       MIPAS_LOCAL(J) = MIPAS_LOCAL(J)*1.0D+6 !ppV to ppmV
    END DO
    
    CALL AL_INTERP (N_ALT, MIPAS_ALT(:), MIPAS_LOCAL(:), NLAYERS,&  
         ZZZ(1:NLAYERS), BRO_VMR(:), DO_PROFILES_AKIMA, *122)
    WHERE(ZZZ(:) < MIPAS_ALT(1))  BRO_VMR(:) = MIPAS_LOCAL(1)
    WHERE(ZZZ(:) > MIPAS_ALT(N_ALT)) BRO_VMR(:) = MIPAS_LOCAL(N_ALT)
    
    RETURN
    
122 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Interpolation error (MIPAS BrO climatology)')
    RETURN 1
    
  END SUBROUTINE READ_BRO_MIPAS
  !********************************************************************
  !         Reading profiles of pressure, temperature, salinity
  !********************************************************************      
  SUBROUTINE READ_UWT_PTS (FILENAME, Z_READ, P_READ, T_READ, S_READ, NL, *)

    CHARACTER (LEN=*), PARAMETER:: R_NAME = 'READ_UWT_PTS' 
    INTEGER             NL
    CHARACTER (LEN = *) FILENAME
    CHARACTER (LEN=80)  HEADERLINE   
    DOUBLE PRECISION    PAR_LOC( MAXVAL(NC_UW(:)) )
    DOUBLE PRECISION, DIMENSION (MAX_LAY) :: Z_READ, P_READ, T_READ, S_READ, TMP
    
    IF (LOG_LEVEL(1) >= N_SSO)  WRITE(LUNIT,*) ' *** Reading: '//TRIM(FILENAME) 
    
    OPEN (C_UNIT,FILE = FILENAME,STATUS = 'OLD')
    DO
       READ(C_UNIT,'(A)') HEADERLINE
       IF (HEADERLINE(1:1).NE.'#'.AND.HEADERLINE(1:1).NE.';') EXIT
    END DO
    BACKSPACE C_UNIT
    
    NL = 0
    DO
       READ(C_UNIT,*,END = 333, ERR=121) PAR_LOC(:)
       NL = NL + 1
       Z_READ(NL) = PAR_LOC(NC_UW(1))
       P_READ(NL) = PAR_LOC(NC_UW(2))
       T_READ(NL) = PAR_LOC(NC_UW(3))
       S_READ(NL) = PAR_LOC(NC_UW(4))
       IF (NL.GT.MAX_LAY) GOTO 120
    END DO
333 CONTINUE
    
    IF(Z_READ(1) > Z_READ(NL)) THEN
       TMP(1:NL) = Z_READ(NL:1:-1)  ;   Z_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = P_READ(NL:1:-1)  ;   P_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = T_READ(NL:1:-1)  ;   T_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = S_READ(NL:1:-1)  ;   S_READ(1:NL) = TMP(1:NL)
    END IF
    
    CLOSE(C_UNIT)
    RETURN
    
120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*)  '    Too many altitude layers required: max_lay is too small'
    WRITE(EU,*) '    File : ', FILENAME
    WRITE(EU,*) '    Diagnostic (read, max_lay): ', NL, MAX_LAY
    RETURN 1
    
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Reading error profiles for pressure, temperature, salinity.'
    WRITE(EU,*) '    File : ', FILENAME
    RETURN 1

  END SUBROUTINE READ_UWT_PTS
  !********************************************************************
  
  SUBROUTINE READ_CHL(FILENAME, Z_READ, S_READ, NL, *)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_CHL'
    INTEGER             NL
    CHARACTER (LEN = *) FILENAME
    CHARACTER (LEN=80)  HDL   
    DOUBLE PRECISION    PAR_LOC( MAXVAL(IWS(2)%NC_CHL(:)) )
    DOUBLE PRECISION, DIMENSION (MAX_LAY) :: Z_READ, S_READ, TMP
    
    ! WRITE(*,*) '    IWS(2)%NC_CHL(:) ',IWS(2)%NC_CHL(:)

    IF (LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(FILENAME) 

    OPEN(C_UNIT, FILE=FILENAME, STATUS='OLD')
    DO
       READ(C_UNIT,'(A)') HDL
       IF (HDL(1:1).NE.'#'.AND.HDL(1:1).NE.';') EXIT
    END DO
    BACKSPACE C_UNIT
    
    NL = 0
    DO
       READ(C_UNIT,*,END = 333, ERR=121) PAR_LOC(:)
       NL = NL + 1
       Z_READ(NL) = Abs(PAR_LOC(IWS(2)%NC_CHL(1)))
       S_READ(NL) = PAR_LOC(IWS(2)%NC_CHL(2))
       IF( NL.GT.MAX_LAY ) GOTO 120
    END DO
333 CONTINUE
    
    IF(Z_READ(1) > Z_READ(NL)) THEN
       TMP(1:NL) = Z_READ(NL:1:-1)
       Z_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = S_READ(NL:1:-1)
       S_READ(1:NL) = TMP(1:NL)
    END IF
    CLOSE(C_UNIT)
    
    RETURN

120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Too many depth layers required: max_lay is too small'
    WRITE(EU,*) '    File : ', FILENAME
    WRITE(EU,*) '    Diagnostic (read, max_lay): ', NL, MAX_LAY
    RETURN 1
    
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Reading error profile for chlorophyll concentration'
    WRITE(EU,*) '    File : ', FILENAME
    RETURN 1
  END SUBROUTINE READ_CHL
  !**********************************************************************

  SUBROUTINE READ_AAW(FLN, Z_READ, S_READ, NL, *)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_AAW'
    INTEGER    NL
    CHARACTER (LEN = *) FLN
    CHARACTER (LEN=80)  HDL   
    REAL (KND)  PAR_LOC(2)
    REAL (KND), DIMENSION (MAX_LAY) :: Z_READ, S_READ, TMP
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(FLN) 
    
    OPEN(C_UNIT, FILE=FLN, STATUS='OLD')
    DO
       READ(C_UNIT,'(A)') HDL
       IF( HDL(1:1).NE.'#' .AND. HDL(1:1).NE.';' .AND.&
            HDL(1:1).NE.'*' .AND. HDL(1:1).NE.'>' ) EXIT
    END DO
    BACKSPACE C_UNIT
    NL = 0
    DO
       READ(C_UNIT,*,END = 333, ERR=121) PAR_LOC(:)
       NL = NL + 1
       Z_READ(NL) = PAR_LOC(1) ; S_READ(NL) = PAR_LOC(2)
       IF( NL > MAX_LAY ) GOTO 120
    END DO
333 CONTINUE
    
    IF(Z_READ(1) > Z_READ(NL)) THEN
       TMP(1:NL) = Z_READ(NL:1:-1) ;  Z_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = S_READ(NL:1:-1) ;  S_READ(1:NL) = TMP(1:NL)
    END IF
    CLOSE(C_UNIT)
    
    RETURN
    
120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Too many depth layers required: max_lay is too small'
    WRITE(EU,*) '    File : ', FLN
    WRITE(EU,*) '    Diagnostic (read, max_lay): ', NL, MAX_LAY
    RETURN 1
    
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) &
         '    Reading error profile for additional absorber concentration'
    WRITE(EU,*) '    File : ', FLN
    RETURN 1
    
  END SUBROUTINE READ_AAW
  ! ***************************************************************

  SUBROUTINE READ_FH_ACIDS(FLN, Z_READ, S_READ, NL, *)
    
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_FH_ACIDS'
    INTEGER    NL
    CHARACTER (LEN=*)  FLN
    CHARACTER (LEN=80) HDL   
    REAL (KND)  PAR_LOC(2)
    REAL (KND), DIMENSION (MAX_LAY) :: Z_READ, S_READ, TMP
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(FLN) 
    
    OPEN(C_UNIT, FILE=FLN, STATUS='OLD')
    DO
       READ(C_UNIT,'(A)') HDL
       IF( HDL(1:1).NE.'#' .AND. HDL(1:1).NE.';' .AND.&
            HDL(1:1).NE.'*' .AND. HDL(1:1).NE.'>' ) EXIT
    END DO
    BACKSPACE C_UNIT
    NL = 0
    DO
       READ(C_UNIT,*,END = 333, ERR=121) PAR_LOC(:)
       NL = NL + 1
       Z_READ(NL) = PAR_LOC(1) 
       S_READ(NL) = PAR_LOC(2)
       IF( NL > MAX_LAY ) GOTO 120
    END DO
333 CONTINUE

    IF(Z_READ(1) > Z_READ(NL)) THEN
       TMP(1:NL) = Z_READ(NL:1:-1) ; Z_READ(1:NL) = TMP(1:NL)
       TMP(1:NL) = S_READ(NL:1:-1) ; S_READ(1:NL) = TMP(1:NL)
    END IF
    CLOSE(C_UNIT)
    
    RETURN
    
120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Too many depth layers required: max_lay is too small'
    WRITE(EU,*) '    File : ', FLN
    WRITE(EU,*) '    Diagnostic (read, max_lay): ', NL, MAX_LAY
    RETURN 1
    
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Reading error profile for fulvic and humic concentration'
    WRITE(EU,*) '    File : ', FLN
    RETURN 1

  END SUBROUTINE READ_FH_ACIDS
  ! ********************************************************************

  SUBROUTINE READ_F_CONC(FLN, Z_READ, NL, *)

    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_F_CONC'
    INTEGER    NL
    CHARACTER (LEN=*)  FLN
    CHARACTER (LEN=80) HDL   
    REAL (KND)  PAR_LOC(N_FC+1)
    REAL (KND), DIMENSION (MAX_LAY) :: Z_READ, TMP
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(FLN) 
    
    OPEN(C_UNIT, FILE=FLN, STATUS='OLD')
    DO
       READ(C_UNIT,'(A)') HDL
       IF( HDL(1:1).NE.'#' .AND. HDL(1:1).NE.';' .AND.&
            HDL(1:1).NE.'*' .AND. HDL(1:1).NE.'>' ) EXIT
    END DO
    BACKSPACE C_UNIT
    NL = 0
    DO
       READ(C_UNIT,*,END = 333, ERR=121) PAR_LOC(:)
       NL = NL + 1
       Z_READ(NL) = PAR_LOC(1) 
       
       C_F(1:N_FC,NL) = PAR_LOC(2:N_FC+1)
       IF( NL > MAX_LAY ) GOTO 120
    END DO
333 CONTINUE
    IF(Z_READ(1) > Z_READ(NL)) THEN
       TMP(1:NL) = Z_READ(NL:1:-1) ; Z_READ(1:NL) = TMP(1:NL)
       DO I = 1, N_FC 
          TMP(1:NL) = C_F(I,NL:1:-1) 
          C_F(I,1:NL) = TMP(1:NL)
       ENDDO
    END IF
    CLOSE(C_UNIT)
    
    DO I = 1, NL
       WRITE(*,'(1x,F5.1,7(1x,F12.9))') Z_READ(I), (C_F(K,I), K = 1,N_FC)
    ENDDO
    
    RETURN
    
120 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*) '    Too many depth layers required: max_lay is too small'
    WRITE(EU,*) '    File : ', FLN
    WRITE(EU,*) '    Diagnostic (read, max_lay): ', NL, MAX_LAY
    RETURN 1
    
121 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
    WRITE(EU,*)  '    Reading error profile for component concentrations'
    WRITE(EU,*) '    File : ', FLN
    RETURN 1
    
  END SUBROUTINE READ_F_CONC

  !***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(PRESS)) DEALLOCATE(PRESS, TEMP, AIR_DENSITY)
    DO II = 1, N_ST_MOL
       IF(ALLOCATED(TG_INFO(II)%ND)) &
            DEALLOCATE(TG_INFO(II)%ND,TG_INFO(II)%ND_INP)
    END DO
    IF(ALLOCATED(O2_CONC)) DEALLOCATE(O2_CONC)
    IF(ALLOCATED(PRES_UW)) DEALLOCATE(PRES_UW,TEMP_UW, SAL_UW, CHL_UW)
    IF(ALLOCATED(Fua_c)) DEALLOCATE(Fua_c, Hua_c)
    IF(ALLOCATED(Flu_con)) DEALLOCATE(Flu_con, C_F)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS

!***********************************************************************
END Subroutine GT_PTCONC_SETUP
!********************************************************************
