C********************************************************************C
C
C	SCIATRAN - Version 3.5 - Last modification: 24 Oct 2015
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C   GT_RENORM renormalizes all phase function moments (except Rayleigh)
C********************************************************************C
      SUBROUTINE GT_RENORM(NSTRMS)
c     ... Call in "rtm_init.f"

      USE AEROSOLS,    ONLY: N_AER_WAVS, APH_VS, LA_N, Plus_mi, 
     $                       NAW_MI, APH_VS_mi, LA_Nmi, LA_Ndb
      USE CLOUD_INPUT, ONLY: Nphr
      USE CLOUDS,      ONLY: DO_CLOUD, CLD_PHAS_MOMS,
     $                       N_CLD_LAYERS, NCW => N_CLD_NETWAVS
      USE GT_POLYN,    ONLY: NLEGEN, PLM, PLMM, AG
      USE INTERN_DIMS, ONLY: Nm_vs
      USE ST_VARS,     ONLY: DO_AEROSOL
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) :: NSTRMS
c     ... Local variables
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GT_RENORM'
      INTEGER     I, J, K
      REAL (KND)  S, T1(NSTRMS), VM(NLEGEN)
      REAL (KND), POINTER :: Phm_p(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')      

C     ... Renormalize the aerosol phase function moments
      IF ( DO_AEROSOL ) THEN
c        ... The original number of aerosol layers corresponding to 
c            'sc', 'lt', 'mn', or 'wm' aerosol input should be used here
         CALL REN_AER( LA_Ndb, N_AER_WAVS, APH_VS )
         IF( Plus_mi ) CALL REN_AER( LA_Nmi, NAW_MI, APH_VS_mi )
      END IF

C     ... Renormalize the cloud phase function moments
      IF ( DO_CLOUD ) THEN
c         write( *,* ) ' ******** gt_renorm ************* '
c         write( *,* ) '   no renormalization for cloud   '
c         write( *,* ) ' ******************************** '
         DO I = 1, N_CLD_LAYERS
            DO K = 1,Nphr
               DO J = 1, NCW
                  Phm_p => CLD_PHAS_MOMS(J,:,I,K,1,1)
                  T1(:) = MATMUL( Phm_p(:),PLMM(:,:)) + 
     $                 MATMUL(Phm_p(:),PLM(:,:) )
                  S = SUM( AG(:)*T1(:) )*0.5d0
                  IF ( S /= 0.d0 ) Phm_p(:) = Phm_p(:) / S
               ENDDO
            END DO
         ENDDO
         NULLIFY( Phm_p )
      END IF

      CALL ST_TRACK (R_NAME, 1, 'O') 
      RETURN

      CONTAINS
C********************************************************************C
      SUBROUTINE REN_AER( LN, NAW, APH )
      INTEGER LN, NAW
      REAL (KND) APH(NLEGEN,LN,Nm_vs,NAW)

      DO  i = 1, LN
         DO  j = 1, NAW
            VM(:) = APH(:,i,1,j)
            T1(:) = MATMUL(VM(:),PLMM(:,:)) + MATMUL(VM(:),PLM(:,:))
            S = DOT_PRODUCT( AG(:),T1(:) ) * 0.5d0
            APH(:,i,1,j) = VM(:)/S
         ENDDO
      ENDDO
      END SUBROUTINE REN_AER
      
      END
C********************************************************************C

