C********************************************************************
C
C	SCIATRAN - Version 4.4 - Last modification: 28 May 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************
      SUBROUTINE GT_SCATT (JPOS, POS, LOA, *)
c     Called from: "dom_spherical.f", "spher_rad.f", and "st_vod.f"

      USE AEROSOLS,    ONLY: AER_COEFF, NAW=>N_AER_WAVS, AWA=>AER_WAVS,
     $                       AP_Mie, AC_MI, NAW_MI, AW_MI, Plus_mi, 
     $                       AER_DER, EXI => GT_EXTC_IDX, 
     $                       SCI => GT_SCAT_IDX, ASI => GT_ASYM_IDX, 
     $                       AER_RET_FACTOR, BETA_VERT,
     $                       NL_ECSTRA, ECSTRA_FLAG, ZTROPO, EXA_OUT,
     $                       EXA_OUT_WL, EXA_FLG, N_EXA_WL, Aer_ext_lin,
     $                       EXA_OUT_FLAG, USE_AP, AP_par, AP_wlr
      USE ALT_INPUT,   ONLY: ALIN_ACT, ALIN_AER_A, ALIN_AER_S, 
     $                       ALIN_CLD_A, ALIN_CLD_S, SOUT_FLAG,
     $                       SOUT_TAU
      USE CLOUDS,      ONLY: J_TOP, J_BOT, DO_CLOUD, N_CLD_LAYERS,
     $                       SCC_NOSC => SCC_LAYER_NOSCALE, 
     $                       SCI_NOSC => SCI_LAYER_NOSCALE, 
     $                       CLC      => CLD_LAYER_COEFF, 
     $                       CLA      => CLD_LAYER_ARR, CLD_F_DM
      USE CLOUD_INPUT, ONLY: Plus_ice
      USE CLOUDS_SS,   ONLY: DO_RAY_DEP, RAY_DEP, DO_RAY_TAU,  
     $                       RAY_TAU_INP
      USE DATA_RET,    ONLY: RET_FLAG
      USE DELTA_M,     ONLY: FDM => FDM_AER, FDM_mi => FDM_AER_mi
      USE GRIDS,       ONLY: NZ => NLAYERS, DZR, ZZZ, DZ
      USE INTERN_DIMS, ONLY: NL_ice, Nm_ice
      USE LOG_PARS,    ONLY: EU, C_UNIT, DIR_OUT
      USE ST_VARS,     ONLY: NSEGS, DO_AEROSOL, RTM_CORE, SEG_NUM
      USE RETR_ITER,   ONLY: ITERATION
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE
      USE SLAB_2L,     ONLY: Ome0, Taus, TES_2L_SLAB
      USE WF_NUM,      ONLY: NUM_AER, SCA_or_ABS, PERTURB
c     ==================================================================

C     CLD_LAYER_COEFF  = CLC coefficients for extinction, single
c                        scattering albedo and delta-m truncation
C     Calculates extinction and scattering profiles 
C     [ molecule (Rayleigh) + particle (aerosol) + cloud ]
C     Include files of internal parameters and arrays

C     Subroutine input arguments
C     ==========================
C     (a) output and model control
C     ============================
C     NZ   = Number of altitude levels in height grid

C     (d) Aerosol information
C     =======================
C     DO_AEROSOL  = Flag for presence of aerosols
C     N_AER_WAVS = external wavelength dimension
C     AER_COEFF  = Profiles of aerosol extinction, scattering, 
C                  truncation coeffs
C     AER_WAVS   = Wavelengths at which aerosol data is specified

      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) ::  POS
c     ----------------------------------------------------------------
c     ... Local optical parameters atmosphere |
      TYPE (LOA_type) :: LOA
c     ---------------------------
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GT_SCATT'
      CHARACTER (LEN = 6)  STRING
      LOGICAL     ERROR
      INTEGER     I, WA1, WA2, WC1, WC2, NCL, JT, JB, NSL, J1, N_REPL
      INTEGER     J, II, II2, KK, Wm1, Wm2, I_der
      REAL (KND)  DP, UP, DD, Dmi, Umi, Tau, BETA(NL_ECSTRA), Ome,
     $     FACTOR(NZ), AI_COF, EMI
      REAL (KND), ALLOCATABLE :: H1(:), H2(:)
      REAL (KND), DIMENSION(NZ) :: ANG_INT
      REAL (KND), POINTER :: SCR(:), SCA(:), EXA(:), Hex(:,:), Hsc(:,:)
c     ------------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I')

      ERROR = .FALSE.

      SCR => LOA%SCR ; SCA => LOA%SCA ; EXA => LOA%EXA

c     Aerosols; compute extinction and scattering 
c     ===========================================
      IF ( DO_AEROSOL ) THEN
         IF( ALIN_ACT(3) ) THEN
c           ... Extinction and scattering from external input file
            SCA(:) = ALIN_AER_S(JPOS,:)
            EXA(:) = ALIN_AER_S(JPOS,:) + ALIN_AER_A(JPOS,:)
            IF( TRIM(RTM_CORE) == 'DOM' ) LOA%DM_A(:) = FDM(:,1)
         ELSE
c           ... Calculation of extinction and scattering coefficients
            CALL LIN_SEARCH_NEW(POS, DP, WA1, WA2, NAW, AWA)            
            UP = 1.d0 - DP
c           ... Linear interpolation for asymmetry parameter
            LOA%AS_A(:) = AER_COEFF(ASI,1:NZ,WA2)*DP 
     $           + AER_COEFF(ASI,1:NZ,WA1)*UP
            IF( USE_AP ) THEN
c           ... Global Angstrom interpolation
               AI_COF = (AP_wlr/POS)**AP_par
               EXA(:) = AER_COEFF(EXI,:,1)*AI_COF
               SCA(:) = AER_COEFF(SCI,:,1)*AI_COF               
            ELSE
c           ... Local Angstrom exponent parameter
               Hex => AER_COEFF(EXI,1:NZ,:)
c               write(*,*) '   EXI = ', EXI
c               write(*,*) '  AER_COEFF(EXI,NZ,:) ', AER_COEFF(EXI,NZ,:)
c               stop
               
c              ... This routine provides ANG_INT - Angstrom interpolator
               CALL AP_INTERP(NZ, NAW, Hex, AWA, WA1, WA2, POS, ANG_INT)
               Hsc => AER_COEFF(SCI,1:NZ,:)
c              ... This routine provides EXA(:) and SCA(:)
               CALL MAKE_COEFF(NAW, AWA, WA1, WA2, DP, UP, *880)            

            ENDIF

            IF( Plus_mi ) THEN
               CALL LIN_SEARCH_NEW(POS, Dmi, Wm1, Wm2, NAW_MI, AW_MI)
               Umi = 1.d0 - Dmi
               LOA%AS_A(:) = LOA%AS_A(:) + AC_MI(ASI,1:NZ,Wm2)*Dmi 
     $              + AC_MI(ASI,1:NZ,Wm1)*Umi 
               Hex => AC_MI(EXI,1:NZ,:) ; Hsc => AC_MI(SCI,1:NZ,:)
               CALL AP_INTERP
     $              (NZ, NAW_MI, Hex, AW_MI, Wm1, Wm2, POS, ANG_INT)
c               SCA(:) = SCA(:) + Hsc(:,Wm2)*Dmi + Hsc(:,Wm1)*Umi
c               EXA(:) = EXA(:) + Hex(:,Wm2)*Dmi + Hex(:,Wm1)*Umi
c              ... Angstrom interpolation instead of linear
               SCA(:) = SCA(:) + Hsc(:,Wm1)*ANG_INT(:)
               EXA(:) = EXA(:) + Hex(:,Wm1)*ANG_INT(:)
            ENDIF
            
!     WHERE(ZZZ(:) <= 30.0D0) AER_RET_FACTOR(5,:) = 0.0D0
!     Scaling of scattering and extinction coefficients
            SCA(:) = SCA(:)*AER_RET_FACTOR(SEG_NUM(JPOS),:)
            EXA(:) = EXA(:)*AER_RET_FACTOR(SEG_NUM(JPOS),:)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!            WHERE(ZZZ(:) >= 35.0D0)
!               SCA(:) = SCA(:)*0.0D0
!               EXA(:) = EXA(:)*0.0D0
!            END WHERE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!            WHERE(ZZZ(:) <= 5.0D0)
!               SCA(:) = SCA(:)*1000.0D0
!               EXA(:) = EXA(:)*1000.0D0
!            END WHERE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!            write(*,*) '    EXA in gt_scat '
!            do i = nz-1, nz
!               WRITE(6,*) ZZZ(i), exa(i)
!            end do
!            pause
        
            
            IF(EXA_OUT_FLAG.AND.ITERATION == 1) THEN
               EXA_OUT(JPOS,:) = EXA(:)
               EXA_OUT_WL(JPOS) = POS
            END IF
!            IF(EXA_OUT_FLAG) THEN
!               WRITE(STRING, FMT='(I0)') INT(POS)
!               OPEN(C_UNIT,FILE=DIR_OUT//'/aer_ext_'//
!     $              TRIM(STRING)//'.dat')
!               do i = 1, NZ
!                  write(C_UNIT, FMT='(100E18.9)') ZZZ(i), EXA(i), SCA(i)
!c                  write(6,*) ZZZ(i), EXA(i), SCA(i)
!               enddo
!               close(C_UNIT)
!            END IF

c           ... Interpolate Delta-M factor
            IF( TRIM(RTM_CORE)=='DOM' ) THEN
               LOA%DM_A(:) = FDM(1:NZ,WA2)*Dp + FDM(1:NZ,WA1)*Up
               IF( Plus_mi ) LOA%DM_A(:) = LOA%DM_A(:) +
     $              FDM_mi(1:NZ,Wm2)*Dmi + FDM_mi(1:NZ,Wm1)*Umi
            ENDIF
c           ... Numerical perturbation technique for scattering
c               and extinction coefficients
c           ********************************************************!
            IF( NUM_AER ) THEN                                      !
               IF( TRIM(SCA_or_ABS)=='aer_modr' ) CALL Derivative   !
            ENDIF                                                   !
c           ********************************************************!  
            WHERE(SCA(:) <= 0.d0) SCA(:) = TINY(1.d0)
c           ... Specific output for aerosol coefficients
            IF( SOUT_FLAG ) CALL AER_PAR_OUT        
         ENDIF
      ELSE
         LOA%AS_A = 0.d0 ;  SCA = 0.d0 ;  EXA = 0.d0
      ENDIF  ! DO_AEROSOL

c      write(*,*) '   After Aer'
c      open (C_UNIT, file=DIR_OUT//'/aer_cof.wm' )
c      do J1 = 1, NZ
c         write(C_UNIT,'(2x,4(2x,e15.7))') zzz(J1), exa(J1), sca(J1)
c      enddo
c      close(C_UNIT)
c      WRITE(*,*) ' ******** ./DATA_OUT/aer_cof.dat ***** done '

c 
c ------------------------ CLOUDS -----------------------------
C     Clouds; compute extinction and scattering 
      IF( DO_CLOUD ) THEN
         IF( ALIN_ACT(4) ) THEN
            LOA%SCC(:) = ALIN_CLD_S(JPOS,:)*(1.d0 - CLD_F_DM(1,1,1,1))
            LOA%EXC(:) = LOA%SCC(:) + ALIN_CLD_A(JPOS,:)
            LOA%SCC_NOSCALE(:,1) = ALIN_CLD_S(JPOS,:)
            LOA%CSM(1,:) = LOA%SCC(:)
         ELSE
            DP = LOA%DP_CLD  ; WC1 = LOA%Cil 
            UP = 1.d0 - DP   ; WC2 = LOA%Cir
            DO NCL = 1, N_CLD_LAYERS
               Jt = J_TOP(NCL) ; Jb = J_BOT(NCL) ; NSL = JB - JT + 1
c               DO I = 1, NSL
cc                 WRITE(*,*)  I, CLC(NCL,WC1,2,I), CLC(NCL,WC2,2,I)
c                  WRITE(*,*)  I, CLC(NCL,WC1,1,I), CLC(NCL,WC1,2,I)
c               ENDDO
c               PAUSE
               LOA%EXC(Jt:Jb) = CLC(NCL,WC1,1,1:NSL)*UP 
     $              + CLC(NCL,WC2,1,1:NSL)*DP
               If(RTM_CORE /= "TRN") Then
                  LOA%SCC(Jt:Jb) = CLC(NCL,WC1,2,1:NSL)*UP 
     $                 + CLC(NCL,WC2,2,1:NSL)*DP 
                  LOA%SCC_NOSCALE(Jt:Jb,1) = SCC_NOSC(NCL,WC1,1:NSL)*UP 
     $                 + SCC_NOSC(NCL,WC2,1:NSL)*DP
               End If
               IF( .NOT.Plus_ice(NCL) ) CYCLE
c               DO I = 1, NSL
cc                 WRITE(*,*)  I, CLC(NCL,WC1,2,I), CLC(NCL,WC2,2,I)
c                  WRITE(*,*)  I, CLC(NCL,WC1,3,I), CLC(NCL,WC1,4,I)
c               ENDDO
c               PAUSE
               LOA%EXI(Jt:Jb) = CLC(NCL,WC1,3,1:NSL)*UP
     $              + CLC(NCL,WC2,3,1:NSL)*DP
               If(RTM_CORE /= "TRN") Then
                  LOA%SCI(Jt:Jb) = CLC(NCL,WC1,4,1:NSL)*UP
     $                 + CLC(NCL,WC2,4,1:NSL)*DP 
                  LOA%SCC_NOSCALE(Jt:Jb,2) = SCI_NOSC(NCL,WC1,1:NSL)*UP 
     $                 + SCI_NOSC(NCL,WC2,1:NSL)*DP
               End If
            ENDDO               ! over cloud layers (NCL)c
c            WRITE(*,*) '   Nm_ice = ', Nm_ice
c            DO J1 = 1, NZ
c               write(*,'(2x,3(2x,e15.7),4x,2(2x,e15.7))')
c     $              zzz(J1), LOA%EXC(J1), LOA%SCC(J1), 
c     $              LOA%EXI(J1), LOA%SCI(J1)
c            enddo
c            PAUSE

            If(RTM_CORE /= "TRN") Then
               LOA%CSM(1,:) = LOA%SCC(:)
               IF( Nm_ice==2 ) LOA%CSM(2,:) = LOA%SCI(:)
            End If
c                WRITE(*,*) '  Nm_ice =  ', Nm_ice
c                DO I = 1, NZ
c                   WRITE(*,*) ZZZ(I), LOA%CSM(:,I) 
c                ENDDO
c           ... Specific output for cloud coefficients
            IF( SOUT_FLAG ) THEN
               ALLOCATE( H1(NZ), H2(NZ) )
               H1(:) = LOA%SCC_NOSCALE(:,1)
               H2(:) = LOA%EXC(:) - LOA%SCC(:)
               IF( SOUT_TAU ) THEN
                  WRITE(82,'(100(2x,E15.7))') 
     $                 ((H1(I)+H1(I+1))*DZR(I)*0.5D0,I=1,NZ-1)
                  WRITE(83,'(100(2x,E15.7))') 
     $                 ((H2(I)+H2(I+1))*DZR(I)*0.5D0,I=1,NZ-1)
               ELSE
                  WRITE(82,'(100(2x,E15.7))') ( H1(I),I=1,NZ )
                  WRITE(83,'(100(2x,E15.7))') ( H2(I),I=1,NZ )
               ENDIF
               DEALLOCATE( H1,H2 )
            ENDIF  !  SOUT_FLAG        
c            open (C_UNIT, file=DIR_OUT//'/cld_cof.dat' )
c            do J1 = J_TOP(1), J_BOT(1)
c               write(*,'(2x,4(2x,e15.7))') zzz(J1), 
c     $              LOA%exc(J1)+LOA%SCC_NOSCALE(J1,1)*CLD_F_DM(1,1,1,1), 
c     $              LOA%SCC_NOSCALE(J1,1)
c            enddo
         ENDIF  ! ALIN_ACT(4)
         
c         WRITE(*,*) '     Extinction cloud gt_scatt.f: layer ',
c         DO J1 = 1, NZ
c            write(*,'(2x,*(2x,e15.7))')
c     $           zzz(J1), LOA%SCC(J1), LOA%EXC(J1)
c         enddo
c         PAUSE '   In gt_scatt.f '
c         WRITE(*,*) '   JPOS = ',JPOS
c         DO I=1,NZ
c            write(*,'(1x,e15.7,3(2x,e15.7))')
c     $           ZZZ(I), SCA(I), LOA%SCC(I), SCA(I)- LOA%SCC(I)
c     $           ZZZ(I), LOA%DM_A(I), CLD_F_DM(1,1,1,1),
c     $           LOA%DM_A(I) - CLD_F_DM(1,1,1,1)
c         ENDDO
      ENDIF ! DO_CLOUD
      
      IF( JPOS==1.AND.DO_RAY_DEP ) THEN
         WRITE( *,* ) '************ gt_scatt **********************'
         WRITE( *,* ) '  !! RAYLEIGH scr is multiplied by RAY_DEP !!!  '
         WRITE( *,* ) '************ ******** **********************'
      ENDIF
      IF( DO_RAY_DEP ) SCR(:) = SCR(:)*RAY_DEP

      IF( DO_RAY_TAU ) THEN
         ALLOCATE( H1(NZ-1) )
         H1(:) = 0.5d0*(SCR(1:NZ-1) + SCR(2:NZ))
         Tau = 0.0D0 
         DO  J1 = 1, NZ - 1
            Tau = Tau + H1(J1)*DZR(J1)
         ENDDO
c         WRITE(*,*) '***** RAYLEIGH tau before scaling: ', Tau
         SCR(:) = SCR(:)*RAY_TAU_INP/Tau
         H1(:) = 0.5d0*(SCR(1:NZ-1) + SCR(2:NZ))
         Tau = 0.D0 
         DO  J1 = 1, NZ - 1
            Tau = Tau + H1(J1)*DZR(J1)
         ENDDO
         DEALLOCATE( H1 )

c         WRITE(*,'(A,2x,2(1x,F8.3))') 
c     $        '  Altitude range :', ZZZ(1), ZZZ(NZ)
c         WRITE(*,*) '***** Rayleigh tau after scaling: ', Tau         
      ENDIF

c      WRITE(*,*) '   Rayleigh and cloud in "gt_scatt.f" '
c      DO I=1,NZ
c         write(*,'(1x,e15.7,3(2x,e15.7))') ZZZ(I),
c     $        SCR(I), LOA%SCC(I)
cc     $           SCA(I), EXA(I),
cc     $           EXA(I) + LOA%DM_A(I)*SCA(I)/(1.d0 - LOA%DM_A(I) ),
c      ENDDO

c      OPEN(79,file = DIR_OUT//'/scr_cof.dat' )
c         DO I = NZ, 1, -1
c            WRITE(79,'(2x,f8.3,2x,e15.7)') ZZZ(I), SCR(I)
c         ENDDO
c      CLOSE(79)

      IF( SOUT_FLAG ) THEN
         IF( SOUT_TAU ) THEN
            WRITE(79,'(100(2x,E15.7))') 
     $           ((SCR(I)+SCR(I+1))*DZR(I)*0.5D0,I=1,NZ-1)
         ELSE
            WRITE(79,'(100(2x,E15.7))') (SCR(I),I=1,NZ)
         ENDIF
      ENDIF

      IF( TES_2L_SLAB ) THEN
         ALLOCATE( H1(NZ - 1) )
         H1(:) = 0.5d0*(SCR(1:NZ-1) + SCR(2:NZ))/Ome0(1)
         Tau = 0.D0 
         DO  J1 = 1, NZ - 1
            Tau = Tau + H1(J1)*DZR(J1)
         ENDDO
         SCR(:) = SCR(:)*Taus(1)/Tau
         DEALLOCATE( H1 )
      ENDIF

      NULLIFY( SCR, SCA, EXA, Hex, Hsc )

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 111  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $     'Error in lin_search (aerosols)')
      RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c **********************************************************************

      SUBROUTINE AP_INTERP(NZ, NAW, Hex, AWA, WA1, WA2, POS, ANG_INT)
      USE TYPE_VARS,   ONLY: KND
      
      INTEGER, INTENT(IN):: NZ, NAW, WA1, WA2
      REAL(KND) Hex(NZ,NAW), AWA(NAW), POS, ANG_INT(NZ)
c     ------------------------------------------------------------------
      ANG_INT(:) = 1.d0
      WHERE( Hex(:,WA1) > 0.d0 )
         ANG_INT(:) = (AWA(WA1)/POS)**
     $        ( LOG( Hex(:,WA2)/Hex(:,WA1) ) /LOG( AWA(WA1)/AWA(WA2) ) )
      END WHERE
      RETURN
      END SUBROUTINE AP_INTERP

c     **********************************************************************
      SUBROUTINE Derivative
      ALLOCATE( H1(NZ), H2(NZ) )
      I_der = 1
c     ... Derivative of extinction and scattering
c         coefficients w.r.t. modal radius
      Hex => AER_DER(EXI,:,:,I_der)  ;  Hsc => AER_DER(SCI,:,:,I_der)
      IF( Plus_mi ) THEN 
c        ... Mie-layers and standard aerosol layers
         H1(:) = Hsc(:,Wm2)*Dmi + Hsc(:,Wm1)*Umi
         H2(:) = Hex(:,Wm2)*Dmi + Hex(:,Wm1)*Umi
      ELSE
c        ... Mie-layers only
         H1(:) = Hsc(:,Wa2)*Dp + Hsc(:,Wa1)*Up
         H2(:) = Hex(:,Wa2)*Dp + Hex(:,Wa1)*Up
      ENDIF
      SCA(:) = SCA(:) + H1(:)*AP_Mie(:,1) *(PERTURB(:)-1.d0) 
      EXA(:) = EXA(:) + H2(:)*AP_Mie(:,1) *(PERTURB(:)-1.d0) 

      RETURN
      END SUBROUTINE Derivative
      
c **********************************************************************
      SUBROUTINE AER_PAR_OUT
      ALLOCATE( H1(NZ), H2(NZ) )
      H1(:) = SCA(:)/(1.d0 - FDM(:,1))
      H2(:) = EXA(:) - SCA(:)
      IF( SOUT_TAU ) THEN
         WRITE(80,'(100(2x,E15.7))') 
     $        ((H1(I)+H1(I+1))*DZR(I)*0.5d0,I=1,NZ-1)
         WRITE(81,'(100(2x,E15.7))') 
     $        ((H2(I)+H2(I+1))*DZR(I)*0.5d0,I=1,NZ-1)
      ELSE
         WRITE(80,'(100(2x,E15.7))') ( H1(I),I=1,NZ )
         WRITE(81,'(100(2x,E15.7))') ( H2(I),I=1,NZ )
      ENDIF
      DEALLOCATE( H1,H2 )
      RETURN
      END SUBROUTINE AER_PAR_OUT
      
c **********************************************************************
      SUBROUTINE MAKE_COEFF(NW, AW, W1, W2, DP, UP, *)

      USE AEROSOLS,    ONLY: EXA_RET, EXA_RET_WL, EXA_AC, EXA_WL 

      INTEGER NW, W1, W2, KKK, N_WL
      LOGICAL USE_EXA, EXTRAP_FLAG
      REAL (KND) AW(NW), DP, UP
      REAL (KND), POINTER :: EXA_PTR(:,:), WL_PTR(:)

c      WRITE(*,*) '   EXA_OUT_FLAG ', EXA_OUT_FLAG
c      WRITE(*,*) '   EXA_FLG ', EXA_FLG
      
      USE_EXA = .FALSE.
      IF(EXA_OUT_FLAG.AND.ITERATION /= 1) THEN
         USE_EXA = .TRUE.
         EXA_PTR => EXA_RET(3,:,:)
         WL_PTR => EXA_RET_WL(:)
         N_WL = NSEGS
      ELSE
         IF(EXA_FLG) THEN
            USE_EXA = .TRUE.
            EXA_PTR => EXA_AC(:,:)
            WL_PTR => EXA_WL(:)
            N_WL = N_EXA_WL
         END IF
      END IF

c      WRITE(*,*) '    USE_EXA ', USE_EXA
      IF( USE_EXA ) THEN
c     ... This option is not relevant in combination with
c         any aerosol mie-mode !!!!! Not any more !!!!!! Check!!!!
         EXTRAP_FLAG = .FALSE.
         KK = MINLOC(WL_PTR,1)
         IF(POS < WL_PTR(KK)) THEN
            KKK = KK
            EXTRAP_FLAG = .TRUE.
         END IF
         KK = MAXLOC(WL_PTR,1)
         IF(POS > WL_PTR(KK)) THEN
            KKK = KK
            EXTRAP_FLAG = .TRUE.
         END IF
          
         IF(EXTRAP_FLAG) THEN
            CALL LIN_SEARCH_NEW (WL_PTR(KKK), DD, II, II2, NW, AW)
            WHERE(Hex(:,II2)*DD + Hex(:,II)*(1.d0-DD) /= 0.0D0)
               FACTOR(:) = EXA_PTR(:,KKK)/
     $              ( Hex(:,II2)*DD + Hex(:,II)*(1.d0-DD) )
               EXA(:) = FACTOR(:)*( Hex(:,WA2)*DP + Hex(:,WA1)*UP)
            ELSE WHERE
               EXA(:) = 0.0D0
            END WHERE
         ELSE
            CALL LIN_SEARCH_NEW (POS, DD, II, II2, N_WL, WL_PTR)
            CALL AP_INTERP
     $              (NZ, N_WL, EXA_PTR, WL_PTR, II, II2, POS, ANG_INT)
c            EXA(:) = EXA_PTR(:,II2)*DD + EXA_PTR(:,II)*(1.d0 - DD)
c           ... Angstrom interpolation instead of linear
            EXA(:) = EXA_PTR(:,II)*ANG_INT(:)
         END IF
         SCA(:) = EXA(:)
      ELSE
c         write(*,*)  '   Aer_ext_lin = ', Aer_ext_lin 
         IF( Aer_ext_lin ) THEN
c           ... Use linear interpolation for extinction coefficient
            EXA(:) = Hex(:,W2)*DP + Hex(:,W1)*UP
         ELSE
c               write(*,*) '   Hex(Nz,W1) ', Hex(Nz,W1)
c               write(*,*) '  ANG_INT(Nz) ', ANG_INT(Nz)
c        ... Use local Angstrom interpolation for extinction coefficient
            EXA(:) = Hex(:,W1)*ANG_INT(:)
         ENDIF
c         SCA(:) = Hsc(:,W2)*DP + Hsc(:,W1)*UP
         SCA(:) = 0.d0  ; EMI = 100.d0*TINY(1.0d0)
         DO I = 1, NZ
            IF( Hex(I,W2)<EMI.OR.Hex(I,W1)<EMI ) CYCLE
c           ... Use linear interpolation for SSA
            SCA(I) = ( Hsc(I,W2)/Hex(I,W2)*DP + Hsc(I,W1)/Hex(I,W1)*UP )
     $           *EXA(I)
         ENDDO
         
         IF (ECSTRA_FLAG) THEN
            CALL ECSTRA_SPEC(POS, BETA, *880)
            N_REPL = COUNT(ZZZ(:) > ZTROPO)
            EXA(1:N_REPL) = BETA_VERT(:)*BETA(:)
            SCA(1:N_REPL) = EXA(1:N_REPL)
         END IF
      END IF                    ! Use_EXA

      RETURN
      
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
      
      END SUBROUTINE MAKE_COEFF
c **********************************************************************

      END

C********************************************************************C


