C********************************************************************C
C
C       SCIATRAN - Version 4.5 - Last modification: 16 Feb 2022
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_SETTINGS_CHECK (*) 
c     ... Called by "gt_iface.f"

      USE ABSORBERS,   ONLY: LINE_ABS_MODE
      USE AEROSOLS,    ONLY: EXA_FLG, N_AOT, MODIS_AP, L_scal,
     $                       AER_ADD_LEV, SHARP_EDGES
      USE ALB_PARS,    ONLY: ALBEDO_CONST, DO_SPECTRAL_ALBEDO
      USE ALT_INPUT,   ONLY: ALIN_FLAG, ALIN_ACT
      USE BRDF,        ONLY: INP_BRDF, Exact_BRF
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS
      USE CLOUD_INPUT, ONLY: PHASE_NAME, TDS
      USE CLOUDS_SS,   ONLY: DO_SSRCA, DO_RAY_TAU, DO_RAY_DEP, RAY_DEP
      USE X_SECTS,     ONLY: ABS_COEFF_TEMP, ABS_COEFF_NTEMP
      USE DATA_RET,    ONLY: RET_FLAG
      USE DELTA_M,     ONLY: AER_DM_FLG, AER_TRUNC_IDX, HSOL_TRUNC_IDX, 
     $                       D_MF_AER
      USE ESFT,        ONLY: DO_ESFT
      USE FLU_LAND,    ONLY: FLU_L
      USE GAS_CONTROL, ONLY: N_XSEC_MOL
      USE GRIDS,       ONLY: NDEP_INP, DEP_INP
      USE GT_POLYN,    ONLY: NLEGEN
      USE INFO,        ONLY: RET_INFO, TG_INFO
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: IRRAD_FLAG, EXTR_FLUX
      USE LBL,         ONLY: DO_LBL, LBL_WING_CUTOFF
      USE LOG_PARS
      USE PRO_SAIL,    ONLY: TypeLidf
      USE RAMAN,       ONLY: RRS_ADJ, RRS_DO, DEL_RAM
      USE RTM_OUTPUT,  ONLY: WF_INTEGR, WUP, N_OP, TRANS_OUT
      Use SRF_Mod,     Only: SRF
      USE ST_VARS,     ONLY: CONVOL_FLAG, WF_FLAG, OWF_FLAG, RAD_FIELD, 
     $                       RAY_SCATT_IDX, DO_RAMAN, Vec_Sca, 
     $                       DO_AEROSOL, CS_ANALYTIC, NSEGS, RIN_OUT, 
     $                       EARTH_RADIUS, DO_REFRAC, SCATT_MODE, TSM,
     $                       GROUND_HEIGHT, RTM_MODE, ADAPTIVE_GRID,
     $                       LATITUDE, LONGITUDE, RTM_TYPE, RTM_CORE,
     $                       RAD_TO_DEG, BRDF_FLAG, SURF_TYPE, APPR_RRS,
     $                       SOLVER, SPECULAR, ARB_OL, King_factor, 
     $                       INEL_PROC, INEL_WHERE, INT_OUT, Kvs, Finc, 
     $                       AG_aer, DIFB, USE_PSM, DO_AZIMUTH,
     $                       DO_PLANCK, ACCUR, NSTRMS, Sec_Order
      USE SPHER_PARS,  ONLY: ANG_OUT, N_SOL_ANG, N_LOS_ANG, N_AZ_ANG,
     $                       ANG_SEL_FLAG, ALTITUDE, LOS_DEF_FLAG,
     $                       ALT_OUT
      Use SRF_Mod,     Only: SRF
      USE ST_INPUT,    ONLY: BAND_START, BAND_STEP, BAND_NUM, APH_TYPE,
     $                       AER_PAR_TYPE, DEP_NAME
      USE THERMAL_VAR, ONLY: BRIGHT_TEMP
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE VRS,         ONLY: Integr_FRF, STEP_VRS
      USE WATER,       ONLY: UWT, NSTR_W, NLEG_W, SS_PDF, OC_LAYERING, 
     $                       NSTR_T, Obs_POSITION, INS_POSI, INS_VIEW,
     $                       II => INCL_INEL, INEL_DO, INEL_ADJ, SIPI,
     $                       UIT, A_RTmat
      USE WF_NUM,      ONLY: NUM_AER, SCA_or_ABS, NUM_ABS
c     ------------------------------------------------------------------

      CHARACTER (LEN=17), PARAMETER :: R_NAME='GT_SETTINGS_CHECK'
      CHARACTER (LEN=17) TEXT
      CHARACTER (LEN=80) TXT
      CHARACTER (LEN=100) Msg(10)
      LOGICAL   FLUX_SPH, Pol_beam, FLG
      INTEGER   I, J, K, N_Msg, KK
      REAL (KND)  BAND_END, MAX_LOS, MAX_SZA, MIN_AZ, MAX_AZ, S1, DUM
      REAL (KND), PARAMETER :: GT_MINIMUM_ACCUR = 0.01D0
c     ==================================================================

      CALL ST_TRACK (R_NAME, 3, 'I')

c     ... Using of incident polarized light
      Pol_beam = .FALSE.
      IF( Kvs > 1 ) Pol_beam = .TRUE.
      IF( Finc(1,1)==0.d0 ) Pol_beam = .TRUE.

      INEL_PROC = 'off'

c     ... Number of inelastic processes in water:
      K = COUNT( II(:) )
c     ... SIPI will be set to .TRUE. if only single inelastic
c         process in water should be considered
      SIPI = .FALSE.
c         TEST
      IF( K==1 ) SIPI = .TRUE.

      IF( DO_RAMAN ) THEN
         INEL_PROC = 'RRS'
         IF( K==0 ) THEN 
            INEL_WHERE = 'In air'
         ELSE
            INEL_WHERE = 'In air+water'
         ENDIF
      ELSE
         IF( K==0 ) THEN 
            INEL_WHERE = 'Nowhere'
         ELSE
            INEL_WHERE = 'In water'
         ENDIF
      ENDIF

c     ... Polarized incident light
c     ============================
      IF( Pol_beam ) THEN
         Msg(1)='Polarized incident light beam can'//
     $        ' not be used in combination with '
         IF( DO_SSRCA ) THEN
            Msg(2)='  single scattering correction algorithm.'
            Msg(3)='Check "Stokes vector of incident beam" control'//
     $           ' parameter in "control.inp" file'
            Msg(4)='   and "Single scattering correction"'//
     $           ' control parameter in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF( TRIM(SCATT_MODE)=='ss' ) THEN
            Msg(2)='  single scattering mode.'
            Msg(3)='Check "Stokes vector of incident beam" control'//
     $           ' parameter in "control.inp" file'
            Msg(4)='   and "Scattering mode" control parameter'//
     $           ' in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF( CS_ANALYTIC ) THEN
            Msg(2)='  "Analytic for conservative scattering" mode.'
            Msg(3)='Check "Stokes vector of incident beam" control'// 
     $           ' parameter in "control.inp" file'
            Msg(4)='   and "Analytic for conservative scattering"'//
     $           ' control parameter in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
         ENDIF
         IF( DO_PLANCK ) THEN
            Msg(2)='  thermal emission.'
            Msg(3)='Check "Stokes vector of incident beam" control'// 
     $           ' parameter in "control.inp" file'
            Msg(4)='   and "Thermal emission" control parameter'//
     $           ' in "control.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
         ENDIF
      ENDIF  ! "Pol_beam" mode

      SELECT CASE( TRIM(INEL_WHERE) )
      CASE('Nowhere','In air','In water','In air+water')
      CASE DEFAULT
         Msg(1)='Medium where inelastic process should be accounted'
         Msg(2)='  for is wrong defined.'
         Msg(3)='Fatal error. Inform program developer.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
      END SELECT

c     ... It is required only one inelastic process in water 
      IF( K==1 ) THEN
         IF( II(1) ) INEL_PROC = 'VRS'
         IF( II(2) ) INEL_PROC = 'F_Chl'
         IF( II(3) ) INEL_PROC = 'F_CDOM'
      ENDIF
c     ... It is required two inelastic process in water
      IF( K==2 ) THEN
c        ... Both chlorophyll (red) and CDOM (blue) fluorescence are
c            accounted for together
         IF( .NOT.II(1) ) INEL_PROC = 'Fr_Fb'
c        ... Both VRS and CDOM (blue) fluorescence are
c            accounted for together
         IF( .NOT.II(2) ) INEL_PROC = 'VRS_Fb'
c        ... Both VRS and chlorophyll (red) fluorescence are
c            accounted for together
         IF( .NOT.II(3) ) INEL_PROC = 'VRS_Fr'
      ENDIF
c     ... VRS, chlorophyll (red) and CDOM (blue) fluorescence should
c         be accounted for 
      IF( K==3 ) INEL_PROC = 'VRS_Fr_Fb'

      !IF( RAD_FIELD .AND. TRIM(LINE_ABS_MODE)=='esft' ) THEN
      !   Msg(1)='Mode "esft" is supported in the spherical geometry'
      !   Msg(2)='  if "Integration over L-of-S in Fourier space?"'
      !   Msg(3)='  is set to "yes".'
      !   Msg(4)='Check this control parameter in "control_ac.inp".'
      !   CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
      !ENDIF

      IF( Sec_Order ) THEN
         IF( TRIM(RTM_TYPE) /= 'spher_scat') THEN
            Msg(1)='Mode "so" (second order) is supported only if'
            Msg(2)=' "RTM_TYPE" parameter is set to "spher_scat".'
            Msg(3)='Check "RTM_TYPE" in "control.inp" and'
            Msg(4)='  "SCATT_MODE" in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF
         IF( .NOT.RAD_FIELD ) THEN
            Msg(1)='The second order approximation can be used '
            Msg(2)='  in "spher_scat" mode if control parameter'
            Msg(3)='  "Integration over L-of-S in Fourier space?"' 
            Msg(4)='  in "control_ac.inp" file is set to "no".'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF
      ENDIF

      IF( TRIM(RTM_CORE)=='DOM' ) THEN
         IF( TRIM(RTM_TYPE)=='spher_scat') THEN
            IF(TRIM(RTM_MODE)=='flux'.AND.(.NOT.RAD_FIELD)) THEN
               Msg(1)='The fluxes cannot be calculated using'
               Msg(2)='  DOM solver in "spher_scat" mode.'
               Msg(3)='In order to do this set in "control_ac.inp" file' 
               Msg(4)='  "Integration over L-of-S in Fourier space?"' 
               Msg(5)='  control parameter to "no".'
               CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
            ENDIF
            IF( TRIM(RTM_MODE)=='spher_alb'.AND.(.NOT.RAD_FIELD) ) THEN
               Msg(1)='The spherical albedo cannot be calculated using'
               Msg(2)='  DOM solver in "spher_scat" mode.'
               Msg(3)='In order to do this set in "control_ac.inp" file' 
               Msg(4)='  "Integration over L-of-S in Fourier space?"' 
               Msg(5)='  control parameter to "no".'
               CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
            ENDIF
c            IF( RAD_FIELD.AND.USE_PSM.AND.DO_CLOUD ) THEN
c              Msg(1)='The option "Use precalculated scattering matrix"'
c              Msg(2)='  cannot be used if the "DO_CLOUD" flag is set '
c              Msg(3)='  to "true".'
c              Msg(4)='Additional corrections are required in the' 
c              Msg(5)='  subroutine "integrate_los.f90".' 
c              CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
c            ENDIF
         ENDIF  ! 'spher_scat' within 'DOM'
         SELECT CASE (RTM_MODE)
         CASE ('wf')
            CALL CHECK_DOM_WF(*880)
         CASE ('num_pert')
         END SELECT
      ENDIF

      IF( TRIM(SCATT_MODE) == 'dir_ref' ) THEN
         IF( TRIM(RTM_CORE) /= 'DOM' ) THEN
            Msg(1)='Mode "dir_ref" is supported only if'
            Msg(2)=' "RTM-CORE" parameter is set to "DOM".'
            Msg(3)='Check "RTM-CORE" in "control.inp" and'
            Msg(4)='  "SCATT_MODE" in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF
         IF( TRIM(RTM_TYPE) /= 'pp_scat' ) THEN
            Msg(1)='Mode "dir_ref" is implemented in plane-parallel'
            Msg(2)=' geometry only. Check "RTM_TYPE" in "control.inp",'
            Msg(3)='Check "SCATT_MODE" in "control_ac.inp".'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 3, *880)
         ENDIF
         IF( UWT ) THEN
            Msg(1)='Mode "dir_ref" is not supported in coupled version.'
            Msg(2)='Check "Coupling of atmosphere and ocean"'
            Msg(3)='  in "control.inp",'
            Msg(4)='  check "SCATT_MODE" in "control_ac.inp".'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF

         SELECT CASE( TRIM(RTM_MODE) )
         CASE('int','num_pert','wf')
         CASE DEFAULT
            Msg(1)='Mode "dir_ref" is supported for "RTM_MODE"'
            Msg(2)=' "wf", "num_pert", and "int" only.'
            Msg(3)='Check "RTM Mode" in control.inp,'
            Msg(4)=' check "SCATT_MODE" in control_ac.inp.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         END SELECT

         IF( DO_PLANCK ) THEN
            Msg(1)='Mode "dir_ref" is not supported in combination'
            Msg(2)='  with thermal emission.'
            Msg(3)='Check "Thermal emission" in control.inp,'
            Msg(4)='  check "SCATT_MODE" in control_ac.inp.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF  ! DO_PLANCK 
      ENDIF  ! "dir_ref" mode

      IF( UWT ) CALL CHECK_UWT(*880)

c     ... Land fluorescence
c     ---------------------
      IF( FLU_L ) THEN
         IF( Pol_beam ) THEN
            Msg(1)='Polarized incident light beam can not be'
            Msg(2)='  used in combination with land fluorescence.'
            Msg(3)='Check "Stokes vector of incident beam" control' 
            Msg(4)='   parameter in "control.inp" file'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
         ENDIF
         IF( (.NOT.IRRAD_FLAG).AND.(.NOT.DIFB) ) THEN
            Msg(1)='Extra-terrestrial solar spectra in physical units'
            Msg(2)='  is needed to account for the land fluorescence. '
            Msg(3)='Check control parameter "Extra-terrestrial solar '
            Msg(4)='   flux" in the file "control.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
         END IF
         IF( TRIM(SCATT_MODE)/='ms' ) THEN
            IF( TRIM(SCATT_MODE)/='dir_ref' ) THEN
               Msg(1)='Land fluorescence mode is supported in the'
               Msg(2)='  framework of "ms" (full multiple scattering)'
               Msg(3)='  or "dir_ref" (direct reflected light) only.'
               Msg(4)='Check control parameter "Scattering mode"'
               Msg(5)='  in the file "control_ac.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
            ENDIF
         ENDIF
c         IF( TRIM(RTM_TYPE) == 'spher_scat' ) THEN
c            Msg(1)='"RTM_TYPE" spher_scat (spherical geometry)'
c            Msg(2)='  can not be used if land fluorescence switch on.'
c            Msg(3)='Check input files "control.inp" '
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880) 
c         ENDIF
         IF(TRIM(RTM_CORE) == 'CDI') THEN
            Msg(1)='Land fluorescence is not included in "CDI" mode!'
            Msg(2)='  Check input files "control.inp"'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)
         ENDIF
      ENDIF  !  Land fluorescence
c     ---------------------------

      IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
         DUM = MINVAL( ANG_OUT(1,:) )
         IF( TO_REFL_FUNC.AND.DUM<=0.d0 ) THEN
            Msg(1)='Output in the form of "reflection function" cannot'
            Msg(2)='  be used in combination with "spher_scat" option '
            Msg(3)='  if solar zenith angle <= 90 deg. '
            Msg(4)='Check the control parameter "Transform to '
            Msg(5)='  reflection function?" in "control_out.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
         IF( Pol_beam ) THEN
            Msg(1)='Polarized incident light beam can not be'
            Msg(2)='  used in combination with "spher_scat" mode.'
            Msg(3)='Check "Stokes vector of incident beam" control' 
            Msg(4)='   parameter in "control.inp" file'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
         ENDIF
         IF( INT_OUT ) THEN
            Msg(1)='Interpolation technique can not be used in'
            Msg(2)='  combination with the "spher_scat" mode.'
            Msg(3)='To continue set "Post-processing type" control' 
            Msg(4)='  parameter in "control_ac.inp" file'   
            Msg(5)='  to "Int_SF".'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
         IF( WF_FLAG.AND.Vec_Sca=='vect_rte'.AND.AG_aer ) THEN
            Msg(1)='WFs in the spherical atmosphere including '
            Msg(2)='  polarization can not be calculated in combi- '
            Msg(3)='  nation with "Adaptive grid (aerosol, cloud)"'
            Msg(3)='  option.'
            Msg(4)='Check in the "control_ac.inp" file the'
            Msg(5)='  "Adaptive grid (aerosol, cloud)" parameter.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
         ENDIF
      ENDIF  ! Mode 'spher_scat'

c     ... Arbitrary output altitude
      IF( ARB_OL .AND. TRIM(RTM_TYPE) /= 'spher_scat') THEN
         Msg(1)='Output at "arbitrary" altitudes is not'//
     $        ' implemented yet'
         IF( ANY(II(:)) ) THEN
            Msg(2)='  in combination with inelastic processes '
            Msg(3)='  (VRS, Fluorescence).'
            Msg(4)='Check the "Position of the instrument" control '
            Msg(5)='  parameter in the "control_geom.inp" file.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
         ENDIF
         IF( DO_PLANCK ) THEN
            Msg(2)='  for the thermal emission case.'
            Msg(3)='Check in file "control_geom.inp" '
            Msg(4)='  the "Position of the instrument" key sentence.'
            CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
         ENDIF
      ENDIF  ! Arbitrary output altitude

      IF( ARB_OL.AND.AG_Aer ) THEN
         Msg(1)='Output at "arbitrary" altitudes in atmosphere '
         Msg(2)='  is not implemented yet in combination       '
         Msg(3)='  with "Adaptive grid (aerosol, cloud)" option.'
         Msg(4)='Check in the "control_geom.inp" file '
         Msg(5)='  the "Position of the instrument" key parameter.'
         CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 5, *880)
      ENDIF

c      WRITE(*,*) '     DO_RAY_DEP = ', DO_RAY_DEP
      IF( DO_RAY_DEP .AND. RAY_DEP == 0.d0) THEN
         IF( (.NOT.DO_AEROSOL) .AND. (.NOT.DO_CLOUD) 
     $        .AND. (.NOT.DO_PLANCK) ) THEN
            Msg(1)='Rayleigh or aerosol scattering should be'
            Msg(2)='  switched on to perform RT calculations.'
            Msg(3)='Check file "control.inp" and "control_ray.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ENDIF
      ENDIF

      SELECT CASE( TRIM(SOLVER) )
      CASE('Solv_SFI','Solv_MIX')
         IF( ADAPTIVE_GRID .AND. SPECULAR ) THEN
            Msg(1)='Adaptive grid option is not allowed if'
            Msg(2)='  "Specular reflection" is set to .TRUE.'
            Msg(3)='Switch off "Adaptive grid" control parameter'
            Msg(4)='  in "control_ac.inp" file.'
            Msg(5)='Check file "control_brdf.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         END IF
      CASE('Solv_ASI')
         IF( ADAPTIVE_GRID ) THEN
            Msg(1)='Adaptive grid option is not allowed '
            Msg(2)='  if "SOLVER" is set to "Solv_ASI".'
            Msg(3)='Switch off "Adaptive grid" in the file '
            Msg(4)= '  "control_ac.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         END IF
      CASE DEFAULT
         Msg(1)='Value of keyword "SOLVER" is not recognized.'
         Msg(2)='Check setting of this variable '
         Msg(3)='  in the file "read_geom_pars.f".'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
      END SELECT

      IF( N_OP > 1 ) THEN
         IF( TRIM(RTM_MODE)/='int'.AND.TRIM(RTM_MODE)/='flux') THEN
            Msg(1)='Output at multiple user-defined altitudes '
            Msg(2)='  can be used if "RTM Mode" is set to "int" only.'
            Msg(3)='Preliminary restriction. Check input files:'
            Msg(4)='  "control.inp" and "control_geom.inp" '
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
      ENDIF  ! N_OP > 1
      
      IF( LOS_DEF_FLAG=='ga' ) THEN
         IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
            Msg(1)='"Type of LOS definition" control parameter'//
     $           ' can not be set to "ga"'
            Msg(2)='  in "control_geom.inp" file if'//
     $           ' "spher_scat" mode is used.'
            Msg(3)='Check "RTM_TYPE" control parameter'//
     $           ' in "control.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ENDIF
         IF( DO_SSRCA ) THEN
            Msg(1)='Single scattering correction algorithm'//
     $           ' can not be used if'
            Msg(2)='  "Type of LOS definition" control parameter'//
     $           ' is set to "ga"'
            Msg(3)='  in "control_geom.inp" file.'
            Msg(4)='Check "Single scattering correction" control'//
     $           ' parameter in "control_ac.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF( TRIM(RTM_MODE)/='int' ) THEN
            Msg(1)='Wrong "Type of LOS definition" selected.'
            Msg(2)='  The type "ga" can be used if "RTM Mode" set '
            Msg(3)='  to "int" only.'
            Msg(4)='Check input files: '
            Msg(5)='  "control.inp" and "control_geom.inp" '
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
         IF( TRIM(INS_POSI)=='user' ) THEN
            Msg(1)='"Type of LOS definition" "ga" can not be used if '
            Msg(2)='  "Position of the instrument" is set to "user".'
            Msg(3)='Check input file "control_geom.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ENDIF
      ENDIF  ! 'ga' angle selection mode

      IF(TRIM(RTM_MODE) == 'flux' .OR. 
     $     TRIM(RTM_MODE) == 'spher_alb') THEN
         FLUX_SPH = .TRUE.
      ELSE
         FLUX_SPH = .FALSE.
      ENDIF
c     Check output format representation
c     ==================================
c      WRITE(*,*) '  settings_check: ANG_SEL_FLAG     ', ANG_SEL_FLAG 
      IF( ANG_SEL_FLAG == 'one' ) THEN 
         OUT_REPR = 'default'
      ELSE
         IF( TRIM(OUT_REPR) == 'dynamic' ) THEN
            OUT_REPR = 'default'
c           ... Single solar zenith angle
            IF( N_SOL_ANG == 1 ) THEN
               IF( N_LOS_ANG >= N_AZ_ANG ) THEN
                  OUT_REPR = '|Zen\Azm|'
               ELSE
                  OUT_REPR = '|Azm\Zen|'
               ENDIF
            ENDIF
c           ... Single viewing zenith angle
            IF( N_LOS_ANG == 1 ) THEN
               IF( N_SOL_ANG >= N_AZ_ANG ) THEN
                  OUT_REPR = '|SZA\Azm|'
               ELSE
                  OUT_REPR = '|Azm\SZA|'
               ENDIF
            ENDIF
c           ... Single azimuthal angle
            IF( N_AZ_ANG == 1) THEN 
               IF( N_SOL_ANG >= N_LOS_ANG ) THEN
                  OUT_REPR = '|SZA\Zen|'
               ELSE
                  OUT_REPR = '|Zen\SZA|'
               ENDIF
            ENDIF
c           -----------------------------
         ENDIF  ! OUT_REPR = 'dynamic'
      ENDIF

      IF( TRIM(OUT_REPR) == 'default' ) THEN
         Mult_WL_Form = 'wl_in_colm'
         ONE_WL = 'unchanged'
      ENDIF

      IF( ANG_SEL_FLAG == 'all' ) THEN
         SELECT CASE (TRIM(OUT_REPR))
         CASE('|Zen\Azm|','|Azm\Zen|')
            IF( N_SOL_ANG > 1 ) THEN
               TEXT = 'solar zenith'
               GOTO 100
            END IF
         CASE('|SZA\Azm|','|Azm\SZA|')
            IF( N_LOS_ANG > 1 ) THEN
               TEXT = 'viewing'
               GOTO 100
            ENDIF
            CASE('|SZA\Zen|','|Zen\SZA|')
               IF( N_AZ_ANG > 1 ) THEN
                  TEXT = 'azimuth'
                  GOTO 100
               ENDIF
            END SELECT
            GOTO 101
 100        CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Wrong output representation mode selected.')
            WRITE(EU,*) '  The format "'//TRIM(OUT_REPR)//'"'
     $           //' can be used for single '//TRIM(TEXT)
     $           //' angle only.'
            WRITE(EU,*) '  Check the input line below the '
     $           //'"Intensity representation in output file"'
            WRITE(EU,*) 
     $           '  control sequence in "control_out.inp" file.'
            RETURN 1
 101        CONTINUE
      ENDIF
      
      SELECT CASE (TRIM(RTM_MODE))
      CASE ('wf')
cc           ... WF in the vector case
c            IF( Vec_sca == 'vect_rte' ) THEN
c               Msg(1)='Weighting functions can not be calculated '
c               Msg(2)='  in the vector case (under implementation).'
c               Msg(3)='Set "RTM-CORE" parameter in "control.inp" '
c               Msg(4)='  to "DOM_S" or "CDI" to continue.'
c               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
c            ENDIF
         IF(TRIM(RTM_CORE) /= 'CDI') THEN
            S1 = MINVAL(ALT_OUT(:))
            IF( INQ_WF('elev', 'RIN') ) THEN 
               IF( S1 < 10.d0 ) THEN
                  Msg(1)='Weighting function for the surface '
                  Msg(2)='  elevation cannot be calculated for an '
                  Msg(3)='  observer position below than 10 km'
                  Msg(4)='Check "control_wf.inp" and '
                  Msg(5)='  "control_geom.inp" files'
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
               ENDIF
               IF( TRIM(RTM_TYPE) == 'spher_scat') THEN
                  Msg(1)='Weighting function for surface elevation is'
                  Msg(2)='  supported for plane-parallel mode only.'
                  Msg(3)='Check file "control_wf.inp" or "control.inp".'
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
               ENDIF
            ENDIF
         ENDIF
         
      CASE DEFAULT
         FLG = INQ_WF('anps', 'RIN', KK)
         IF(INQ_WF('anps', 'RIN')) THEN
            CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $           'Average number of photon scattering '//
     $           'can be produced in "wf" mode only')
            RET_INFO(KK)%STAT = .FALSE.
         ENDIF
      END SELECT

      SELECT CASE (RTM_MODE)
      CASE ('int', 'flux')
      CASE ('num_pert')
         IF( NUM_AER.AND.TRIM(SCA_or_ABS)=='aer_modr' ) THEN 
            IF( TRIM(AER_PAR_TYPE)=='mi' ) THEN
               Msg(1)='Numerical perturbation of aerosol parameters '
               Msg(2)='  w.r.t. modal radius of PSD function can '
               Msg(3)='  not be performed if "Aerosol parameterization '
               Msg(4)='  type" control parameter is set to "mi"'
               Msg(5)='  (see "control_aer.inp" file).'
               Msg(6)='Use "st_mi", "lt_mi", or "wm_mi" option to '  
               Msg(7)='  to test WF w.r.t. modal radius within '
               Msg(8)='  the Mie-mode.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,8,*880)
            ENDIF
         ENDIF
      CASE ('spher_alb')
         IF(IRRAD_FLAG) THEN
            Msg(1)='For the case of "spher_alb" (spherical albedo) '
            Msg(2)='  EXTR_FLUX should be set to "Pi" and'
            Msg(3)='  IRRAD_FLAG should be set to .FALSE.'
            Msg(4)='Check control.inp file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         END IF            
      CASE ('raman')
         IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
            IF( RAD_FIELD ) THEN
               Msg(1)='Raman scattering can be calculated in the'//
     $              ' spherical geometry'
               Msg(2)=
     $              '  if "Integration over L-of-S in Fourier space?"'
               Msg(3)='  parameter is set to "yes".'
               Msg(4)=
     $              'Check this control parameter in "control_ac.inp".'
               CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
            ENDIF
            IF( RRS_DO ) THEN
               Msg(1)='Raman scattering can be calculated in the'//
     $              ' spherical geometry'
               Msg(2)=
     $              '  using the forward-adjoint technique only.'
               Msg(3)='Set parameter "Numerical technique used '
               Msg(4)='  for RRS" in "control_rrs.inp" to "adj".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
            IF( TRIM(TSM)=='speed' ) THEN
               Msg(1)='Raman scattering can be calculated in the'//
     $              ' spherical geometry'
               Msg(2)='  using mode "memory" only.'
               Msg(3)='Set parameter "Tradeoff speed/memory" '
               Msg(4)='  in "control_rrs.inp" file to "memory".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
         ENDIF  ! 'spher_scat' mode

         IF( TRIM(RTM_TYPE)=='ps_scat' .AND. RRS_DO) THEN
            Msg(1)='Raman scattering in the case of '//
     $           'pseudo-spherical approximation'
            Msg(2)='  can be calculated using the forward-adjoint'//
     $           ' technique only.'
            Msg(3)='Set parameter "Numerical technique used '
            Msg(4)='  for RRS" in "control_rrs.inp" to "adj".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF                  ! 'ps_scat' + RRS_DO

         IF( Vec_sca=='vect_rte' ) THEN
c            IF( TRIM(SCATT_MODE)/='ss' ) THEN
c               Msg(1)='Raman scattering for the vector case'
c               Msg(2)='  and "ms" mode is under implementation.'
c               CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)
c            ENDIF
c               IF( .NOT.RRS_DO.AND.TRIM(SCATT_MODE)/='ss' ) THEN
c                  Msg(1)='Raman scattering can be calculated in the '
c                  Msg(2)='  vector case using the discrete-ordinates'
c                  Msg(3)='  technique only (preliminary restriction).'
c                  Msg(4)='Set parameter "Numerical technique used '
c                  Msg(5)='  for RRS" in "control_rrs.inp" to "dom".'
c                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
c               ENDIF
         ENDIF

         IF( TRIM(RTM_CORE) == 'CDI' ) THEN
            Msg(1)='Raman scattering can be calculated using'
            Msg(2)='  the discrete-ordinates technique only.'
            Msg(3)='Set the parameter "RTM-CORE" in "control.inp" '
            Msg(4)='  file to "DOM_S".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF( NSEGS > 1 ) THEN
            Msg(1)='Rotational Raman calculations is implemented'
            Msg(2)='  yet for a single spectral segment only.'
            Msg(3)='Check "Wavelength segment info" parameter'
            Msg(4)='  in the file "control.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF(.NOT.IRRAD_FLAG) THEN
            CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $           'Radiance in physical units is needed in '//
     $           '"raman" mode, "IRRAD_FLAG" is set on "TRUE"')
            IRRAD_FLAG = .TRUE.
         END IF
c           ... End of 'raman' case ...

      CASE ('amf', 'slant_col', 'block_amf')
         IF(CONVOL_FLAG) THEN
            Msg(1)='Convolution can not be performed in'
            Msg(2)='  "amf", "slant_col", and "block_amf" modes.'
            Msg(3)='Check "Do convolution" parameter'
            Msg(4)='  in the file "control.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         END IF
         IF( TRANS_OUT ) THEN
            Msg(1)='Convolution of fluxes can not be performed in'
            Msg(2)='  "amf", "slant_col", and "block_amf" modes.'
            Msg(3)='Check "Fluxes for path representation" parameter'
            Msg(4)='  in the file "control_out.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
      END SELECT

      IF(RET_FLAG.AND..NOT.CONVOL_FLAG) THEN
         IF(DO_LBL.OR.DO_ESFT) THEN
c            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
c     $           'Monochromatic spectra must be convolved '//
c     $           'for the retrieval ')
            CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $           'Monochromatic spectra are used '//
     $           ' for the retrieval process')
c            RETURN 1
         END IF
      END IF

c      IF( TRIM(SURF_TYPE)=='mRPV_facet') THEN
c         IF( Vec_Sca=='scal_rte' ) THEN
c            Msg(1)='Only vector case is supported if'
c            Msg(2)='  "Surface type" is set to "mRPV_facet".'
c            Msg(3)='Use another "Surface type" in file'
c            Msg(4)='  "control_brdf.inp".'
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
c         ENDIF
c      ENDIF

c     ... Restrictions for option "ocean" specular reflection
      IF( BRDF_FLAG .AND. TRIM(SURF_TYPE)=='ocean') THEN
c         IF( Vec_Sca=='vect_rte'.AND.SPECULAR ) THEN
c            Msg(1)='Only scalar case is supported '
c            Msg(2)='  for specular reflection.'
c            Msg(3)='Set "Specular reflection" parameter in file'
c            Msg(4)='  "control_brdf.inp" to "f".'
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
c         ENDIF
         IF( TRIM(RTM_TYPE)=='spher_scat'.AND.SPECULAR ) THEN
            Msg(1)='The specular reflection is not supported'
            Msg(2)='  in the case of spherical geometry.'
            Msg(3)='Set "Specular reflection" parameter in file'
            Msg(4)='  "control_brdf.inp" to "f" or'
            Msg(5)='  use the mode "pp_scat" instead of "spher_scat"'
            Msg(6)='  in file "control.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         ENDIF
      ENDIF
c     ... Restrictions for option "Reading BRDF from file"
      IF( BRDF_FLAG .AND. TRIM(SURF_TYPE)=='rff') THEN
         IF( Vec_Sca=='vect_rte' ) THEN
            Msg(1)='Only scalar case is supported if'
            Msg(2)='  "Surface type" is set to "rff".'
            Msg(3)='Use another "Surface type" in file'
            Msg(4)='  "control_brdf.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
c         IF( DO_SSRCA ) THEN
c            Msg(1)='Single scattering correction algorithm'
c            Msg(2)='  can not be used if "Surface type" '
c            Msg(3)='  is set to "rff". Check input files: '
c            Msg(4)='  "control_ac.inp" and "control_brdf.inp".'
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
c         ENDIF
      ENDIF

      IF( BRDF_FLAG.AND.DIFB ) THEN
         Msg(1)='The option "Diffuse illumination from below" is not'
         Msg(2)='    implemented yet in combination with BRDF mode.'
         Msg(3)='Check "RTM Mode" and "Surface reflection type" ' 
         Msg(4)='   control parameters in the "control.inp" file'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)               
      ENDIF

      IF( BRDF_FLAG.AND.DO_SSRCA ) THEN         
         IF( INT_OUT ) THEN
            Msg(1)='Single scattering correction algorithm can not'
            Msg(2)='  be used in combination with "Interp" mode.  '
            Msg(3)='To continue set "Single scattering correction"'
            Msg(4)='  control parameter in "control_ac.inp" file  '
            Msg(5)='  to "f" (false).'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
      END IF

      IF( BRDF_FLAG.AND.Exact_BRF ) THEN
         IF( INT_OUT ) THEN
            Msg(1)='In the case of bidirectional reflection the mode'
            Msg(2)='  "Exact_BRF" can not be used in combination'         
            Msg(3)='  with "Interp" mode.'
            Msg(4)='  Check input files: "control_ac.inp", control'
            Msg(5)='  parameter "Post-processing type".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF

         IF( WF_FLAG ) THEN
            Msg(1)='In the case of bidirectional reflection the mode'
            Msg(2)='  "Exact_BRF" can not be used in combination'         
            Msg(3)='  with weighting functions.'
            Msg(4)='  Check input files: "control_brdf.inp", control'
            Msg(5)='  parameter "Exact BRDF for diffuse radiation".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
      ENDIF

c     ... Weighting functions for BRDF parameters
      IF( BRDF_FLAG.AND.INQ_WF('albedo','RIN')) THEN
         SELECT CASE( TRIM(SURF_TYPE) )
         CASE('svk','prosail','Ross-Li','Ross-Li-snow','snow_fasmar')
         CASE DEFAULT
            Msg(1)='In the case of surface bidirectional reflection'
            Msg(2)='  the weighting functions are implemented '
            Msg(3)='  for "Surface type": "svk", "Ross-Li", '
            Msg(4)='  "Ross-Li-snow", "snow_fasmar" or "prosail" only.'
            Msg(5)='Check input files: "control.inp"'
            Msg(6)='  and "control_brdf.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         END SELECT
      ENDIF
      IF( BRDF_FLAG.AND.INQ_WF('prosail_Ala','RIN')) THEN
         IF( TypeLidf==1 ) THEN
            Msg(1)='In the case of "prosail" BRDF model'
            Msg(2)='  weighting function for "average leaf angle" '
            Msg(3)='  does not exists for Leaf inclination '
            Msg(4)='  distribution function type 1.'
            Msg(5)='Check input files: "control.inp"'
            Msg(6)='  and "control_brdf.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         ENDIF
      ENDIF

      SELECT CASE (TRIM(RTM_CORE))
      CASE('ASYMP')
         IF( .NOT.DO_CLOUD ) THEN
            Msg(1)='Asymptotic can be only used if cloud present.'
            Msg(2)='Check parameter "Clouds present?" in'
            Msg(3)='  "control.inp file"'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ELSE
            IF( Vec_Sca == 'vect_rte' ) THEN
               Msg(1)='Only scalar case is supported in asymptotic mode'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,1,*880)
            ENDIF
            IF(N_CLD_LAYERS /= 1) then 
               Msg(1)='Only single cloud layer is supported in '
               Msg(2)='  asymptotic mode.'
               Msg(3)='Check "Number of cloud layers" parameter'
               Msg(4)='  in "cloud.inp" file'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            END IF
            IF( TDS(1)%S == 'mixed' ) THEN
               Msg(1)='Only "water" or "ice" thermodynamic state is '
               Msg(2)='  supported in asymptotic mode'
               Msg(3)='Check "Thermodynamic state" parameter'
               Msg(4)='  in "cloud.inp" file'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
            IF( PHASE_NAME(1)%S == 'hg' .OR. 
     $           PHASE_NAME(1)%S == 'iso') then 
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Only "user" and "db" type of phase function is '
     $              //'supported in asymptotic mode')
               RETURN 1
            END IF
            IF( BRDF_FLAG ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     $              'Asymptotic is not implemented in "BRDF" mode!')
               RETURN 1
            ENDIF
         END IF
         IF(TRIM(RTM_MODE) == 'spher_alb') THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Spherical albedo calculation is possible'
     $           //' with "RTM-CORE" set to "DOM" only')
            RETURN 1
         END IF
         
      CASE ('DOM') 
         IF( FLUX_SPH ) THEN
c            IF( ADAPTIVE_GRID ) THEN
c               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
c     $              'Adaptive grid option '//
c     $              'is not allowed ')
c               WRITE(EU,*)'    if "RTM_MODE" is set to '//
c     $              '"flux" or "spher_alb".'
c               WRITE(EU,*)'    Switch off '//
c     $              '"Adaptive grid" in the file '//
c     $              '"control_ac.inp".'
c               RETURN 1
c            ENDIF
            IF( DO_SSRCA ) THEN
               Msg(1)='Single scattering correction algorithm '
               Msg(2)='  is not allowed if "RTM_MODE" is set to '
               Msg(3)='  "flux" or "spher_alb".'
               Msg(4)='Switch off '
               Msg(5)='  "Single scattering correction" in the file '
               Msg(6)='  "control_ac.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
            ENDIF
         ENDIF
         IF( BRDF_FLAG .AND. (.NOT.FLUX_SPH) ) THEN
            IF( TRIM(SURF_TYPE) == 'ocean' ) THEN 
               IF( TRIM(SS_PDF) == 'Gram-Charlier' ) THEN
               IF( .NOT.DO_SSRCA.AND.TRIM(RTM_TYPE)/='spher_scat' ) THEN
                  Msg(1)='"Gram-Charlier" surface slope PDF '
                  Msg(2)='  requires single scattering correction.'
                  Msg(3)='Switch on '
                  Msg(4)='  "Single scattering correction" in the file '
                  Msg(5)='   "control_ac.inp" or use '
                  Msg(6)='   "Gauss" slope PDF in "control_brdf.inp".'
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
               ENDIF
               ENDIF
            ENDIF            
            IF( NLEGEN < 30 .AND. (.NOT.DO_SSRCA) ) THEN
               Msg(1)='Number of Legendre moments can be too small'
               Msg(2)='  to account for contribution of '
               Msg(3)='  anisotropic surface reflection.'
               Msg(4)='Enhance of '
               Msg(5)='  "Number of Legendre moments" (>30) or switch'
               Msg(6)='   on  "Single scattering correction" in '
               Msg(7)='   the file "control_ac.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,7,*880)
            ENDIF

         ENDIF

         IF(TRIM(RTM_MODE) == 'wf') THEN
            SELECT CASE (TRIM(WF_INTEGR))
            CASE( 'analytic' )
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Wrong WF integration mode selected')
               WRITE(EU,*) '  : ',TRIM(WF_INTEGR)
               WRITE(EU,*) '    Selected WF integration '//
     $              'mode is not implemented in this version : ' 
               RETURN 1
            CASE( 'numeric' )
            CASE( 'nointegr' )
               CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $              'Weighting parameter is set to 0. for this mode')
               WUP = 0.0d0
            CASE DEFAULT
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Wrong WF integration mode selected')
               WRITE(EU,*) '    Selected WF integration '//
     $              'mode not found :', TRIM(WF_INTEGR)
               RETURN 1
            END SELECT
         ENDIF
         
         IF( WUP < 0.0d0 .OR. WUP > 1.0d0 ) THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Weighting parameter must be '//
     $           'in the range [0,1] ')
            WRITE(EU,*) '    Selected Weighting '//
     $           'parametr : ', WUP
            RETURN 1
         ENDIF
         
      CASE ('CDI') 
         IF(BRDF_FLAG) THEN
            IF( TRIM(SURF_TYPE) == 'user' ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Surface type "user" is allowed'
     $              //' with "RTM-CORE" set to "DOM" only')
            RETURN 1
            ENDIF
         ENDIF
         IF(TRIM(RTM_MODE) == 'spher_alb') THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Spherical albedo calculation is possible'
     $           //' with "RTM-CORE" set to "DOM" only')
            RETURN 1
         END IF
         IF(DO_SPECTRAL_ALBEDO.AND.INQ_WF('albedo','RIN',KK)) THEN
            CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $           'In spherical mode albedo weighting function can be '//
     $           'calculated for wavelength independent albedo only')
            RET_INFO(KK)%STAT = .FALSE.
         END IF           
      END SELECT
      
      SELECT CASE (TRIM(SCATT_MODE))
      CASE('ms','dir_ref')
      CASE('ss','ss_ms')
         IF( TRIM(RTM_CORE)=='DOM' ) THEN
c            IF( TRIM(RTM_TYPE)/='spher_scat' ) DO_SSRCA = .TRUE.
            IF( UWT ) THEN
               Msg(1)='Single scattering mode is not implemented'//
     $              ' in the coupled OA model.'
               Msg(2)='Check input files:'//
     $              ' "control_ac.inp" and "control_brdf.inp".'
                CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)
            ENDIF
            IF( TRIM(RTM_MODE)=='flux' ) THEN
               Msg(1)='Fluxes can be calculated using single '//
     $              ' scattering approach in "CDI" mode only.'
               Msg(2)='Check "Scattering mode" parameter in'//
     $              ' "control_ac.inp" file'
               Msg(3)=
     $              '   and "RTM-CORE" parameter in "control.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
            END IF
         ENDIF
      CASE('wf_ss_ms','sm_wf_ss')
c         IF(TRIM(RTM_CORE)=='DOM') THEN
cc            IF( TRIM(RTM_TYPE)/='spher_scat' ) THEN
c               Msg(1)='Settings "sm_wf_ss", and "wf_ss_ms" '//
c     $              ' have only an effect if '
c               Msg(2)=' "CDI" mode is selected. '
c               Msg(3)='Check "Scattering mode" parameter in'//
c     $              ' "control_ac.inp" file.'
c               Msg(4)=
c     $              ' and "RTM-CORE" parameter in "control.inp" file.'
c               CALL WRITE_MSG(R_NAME, 'ERROR', Msg, 4, *880)
c            ENDIF
c         END IF
      CASE('wf_ss')
c         IF(TRIM(RTM_CORE)=='DOM') THEN
c            IF( TRIM(RTM_TYPE)/='spher_scat' ) THEN
c               Msg(1)='Settings "wf_ss" have only an effect if'
c               Msg(2)='a) "CDI" mode is active,'
c               Msg(3)='b) "DOM" mode with "spher_scat" is selected.'
c               Msg(4)='Check "Scattering mode" parameter in'//
c     $              ' "control_ac.inp" file.'
c               Msg(5)=
c     $              ' and "RTM-CORE" parameter in "control.inp" file.'
c               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
c            ENDIF
c         END IF
      CASE DEFAULT
         Msg(1)='Wrong scattering mode selected '
         Msg(2)='Check "Scattering mode" parameter in'//
     $              ' "control_ac.inp" file.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)
      END SELECT

      IF (LATITUDE > 90.0D0 .OR. LATITUDE < -90.0D0) THEN
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong latitude '
     $        //'selected')
         RETURN 1
      ENDIF

      IF (LONGITUDE > 360.0D0 .OR. LONGITUDE < 0.0D0) THEN
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong longitude '
     $        //'selected')
         RETURN 1
      ENDIF

c     Check if RAY_SCATT_IDX valid
c     ----------------------------
      SELECT CASE( RAY_SCATT_IDX )
         CASE (1, 2, 3, 4)
         CASE DEFAULT
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $        'Rayleigh scattering index not valid')
         WRITE(EU,*)
     $        '    Diagnostic: selected index: ', RAY_SCATT_IDX
         RETURN 1
      END SELECT
      
c         New test
      IF( WF_FLAG ) THEN
c         IF( DO_SSRCA ) THEN
c            CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
c            WRITE(EU,*) '    Single scattering correction algorithm '//
c     $           'is under implementation '
c            WRITE(EU,*) 
c     $           '    for calculation of weighting functions.'
c             WRITE(EU,*)
c     $           '    Check "Single scattering correction" control'
c            WRITE(EU,*)
c     $           '    line in the file "control_ac.inp".'
c            RETURN 1
c        ENDIF

         IF(DO_SSRCA.AND.INQ_WF('elev','RIN')) THEN
            CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
            WRITE(EU,*) '    Single scattering correction algorithm '//
     $           'is not allowed '
            WRITE(EU,*) 
     $           '    for calculation of elevation weighting function'
            WRITE(EU,*)
     $           '    Check "Other parameters - weighting functions"'
            WRITE(EU,*)
     $           '    control line in the file "control_wf.inp".'
            RETURN 1
         ENDIF
      ENDIF

c     Ground height at least zero ?
c     -----------------------------
      IF (GROUND_HEIGHT < 0.0D0) THEN
         CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $        'Ground height below/equal zero')
         return 1
      endif

C     Earth radius
c     ------------
      IF ( EARTH_RADIUS.LE.6350.0.OR.EARTH_RADIUS.GT.6400.0) THEN
        CALL WRITE_MESSAGE ( R_NAME, 'WARNING',
     &     'Earth radius out of bounds')
        write(EU,*) '    Earth radius: ', earth_radius
      ENDIF

C     Check computational stream numbers and number of Legendre
c     polynomials 
c     ----------------------------------------------------------
      IF( DO_SSRCA.AND.DO_AEROSOL.AND.(.NOT.AER_DM_FLG) ) THEN
        CALL WRITE_MESSAGE ( R_NAME, 'WARNING',
     $        'Aerosol DELTAM switch on')
c       ... In the case of "CDI" all moments of the phase function
c           will be used if the mode "mie" is switched on
        IF( TRIM(RTM_CORE)/='CDI' ) AER_DM_FLG = .TRUE.
      ENDIF

      IF( DO_AEROSOL ) THEN         
         IF( L_scal.AND.MODIS_AP ) THEN
            Msg(1)='Option "Aerosol scaling factor" can not be used '
            Msg(2)='  in combination with "mod-l", "mod-o", "mod-m" '
            Msg(3)='  aerosol parameterization mode. '
            Msg(4)='Check "Aerosol scaling factor" parameter'
            Msg(5)='  in the "control_aer.inp" file. '            
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)   
         ENDIF
c        ... Run SCIATRAN with multiple aerosol optical thickness (AOT)
         IF( N_AOT > 0 ) THEN
            IF (TRIM(RTM_CORE) /= 'DOM') THEN
               Msg(1)='The multiple AOT option can be used '
               Msg(2)='  in combination with "RTM-CORE" DOM only.'
               Msg(3)='Check "Additional aerosol OT" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF

            SELECT CASE (TRIM(RTM_MODE))
            CASE('int')
            CASE DEFAULT
               Msg(1)='The multiple AOT option can be used in '
               Msg(2)='  combination with "int" mode only.'
               Msg(3)='  "wf" mode is under implementation.'
               Msg(4)='Check "Additional aerosol OT" parameter'
               Msg(5)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
            END SELECT

            IF( TRIM(RTM_TYPE) == 'spher_scat' ) THEN
               Msg(1)='The multiple AOT option can not be used '
               Msg(2)='  in combination with "spher_scat" mode.'
               Msg(3)='Check "Additional aerosol OT" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
            SELECT CASE(TRIM(SCATT_MODE))
            CASE('ms','ss')
            CASE DEFAULT 
               Msg(1)='The multiple AOT option can be used in '
               Msg(2)='  combination with "ms" and "ss" mode only.'
               Msg(3)='Check "Additional aerosol OT" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)            
            END SELECT
            IF( UWT.OR.UIT ) THEN
               Msg(1)='The multiple AOT option can be used in '
               Msg(2)='  surface-atmosphere system only.'
               Msg(3)='Check "Additional aerosol OT" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)            
            ENDIF
         ENDIF  ! N_AOT > 0 
c        ...............................................................
         IF( EXA_FLG ) THEN
            SELECT CASE (TRIM(AER_PAR_TYPE))
            CASE('mi','st_mi','lt_mi','mn_mi','wm_mi')
               Msg(1)='The "Replace aerosol extinction" option can not'
               Msg(2)='  be used in combination with any Mie-mode.'
               Msg(3)='Check "Aerosol parameterization type" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            CASE('pao')
               Msg(1)='The "Replace aerosol extinction" option can not'
               Msg(2)='  be used in combination with "pao" type.'
               Msg(3)='Check "Aerosol parameterization type" parameter'
               Msg(4)='  in "control_aer.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            END SELECT
         ENDIF

         IF( ALIN_ACT(3) ) THEN
            IF( TRIM(AER_PAR_TYPE) /= 'mn' ) THEN
               Msg(1)='The "Alternative input" option requires that'
               Msg(2)='  "Aerosol parameterization selection" parameter'
               Msg(3)='  should be set to "mn" (see "control_aer.inp").'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
            ENDIF
            IF( TRIM(D_MF_AER) /= 'D-MA' ) THEN
               Msg(1)='The "Alternative input" option requires that'
               Msg(2)='  "Aerosol delta-M approximation" parameter'
               Msg(3)='   should be set to "D-MA".'
               Msg(4)='See "control_aer.inp"'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
         ENDIF

         SELECT CASE (TRIM(AER_PAR_TYPE))
         CASE('st','st_mi')
            IF( NLEGEN > 50 ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              'Number of Legendre moments has to be <= 50')
               WRITE(EU,*) '    by using of "SCIATRAN" '
     $              //'aerosol database.'
               WRITE(EU,*) '    see "control_ac.inp" '
     $              //'and "control_aer.inp".'
               RETURN 1
            ENDIF
            IF( Vec_Sca == 'vect_rte' ) THEN
               Msg(1)='SCIATRAN aerosol model can not be used'
               Msg(2)='  in the vector case.'
               Msg(3)='Set appropriate aerosol model in the file'
               Msg(4)='  "control_aer.inp"'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
            IF( DO_SSRCA ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $        'Single scattering correction algorithm can not be used ')
               WRITE(EU,*) 
     $              '    with existing SCIATRAN aerosol models.'
               WRITE(EU,*) 
     $              '    See file "control_ac.inp".'
               RETURN 1
            ENDIF
         CASE('lt','lt_mi')
            IF( Vec_Sca == 'vect_rte' ) THEN
               Msg(1)='LOWTRAN aerosol model can not be used'
               Msg(2)='  in the vector case.'
               Msg(3)='Set appropriate aerosol model in the file'
               Msg(4)='  "control_aer.inp"'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
            ENDIF
         CASE('wm','wm_mi')
         CASE('mn','mn_mi')
            IF( AER_ADD_LEV.AND.SHARP_EDGES ) THEN
               Msg(1)='Option "Add level at boudary layer top"'
               Msg(2)='  is not allowed if "Aerosol as a cloud layer?"'
               Msg(3)='  is set to "yes".'
               Msg(4)='Check these parameters in file "control_aer.inp"'
               Msg(5)='  and in file "man_aer.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)               
            ENDIF
         CASE('pao')
            IF( AER_ADD_LEV.AND.SHARP_EDGES ) THEN
               Msg(1)='Option "Add level at boudary layer top"'
               Msg(2)=
     $            '  is not allowed if "PAO aerosol as a cloud layer?"'
               Msg(3)='  is set to "yes".'
               Msg(4)='Check these parameters in file "control_aer.inp"'
               Msg(5)='  and in file "pao_aer.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)               
            ENDIF
         CASE('mi')
            IF( APH_TYPE/='mie' ) THEN
               Msg(1)='Only Mie phase function is supported'
               Msg(2)='  if option "mi" is selected.'
               Msg(3)='Check parameter in file "control_aer.inp".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
            ENDIF
         END SELECT

         IF( AER_DM_FLG .AND. TRIM(RTM_CORE) /= 'ASYMP') THEN
            IF( AER_TRUNC_IDX >= NLEGEN ) THEN
               CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $              '    Aerosol trunction index >= number of '//
     $              'legendre terms')
               WRITE(EU,'(A,1x,I5)') 
     $              '          AER_TRUNC_IDX = ', AER_TRUNC_IDX
               WRITE(EU,'(A,1x,I5)')
     $              '                 NLEGEN = ', NLEGEN
               RETURN 1
            ENDIF
         END IF
      ENDIF
c ---------------- End check  DO_AEROSOL mode --------------------------
      
      IF (TRIM(RTM_CORE) == 'DOM') THEN
         IF (NLEGEN < 1) THEN
            call write_message(R_NAME, 'ERROR', 
     $           'NLEGEN wrong input to the code')
            return 1
         END IF

         IF( NSTRMS < 2 ) THEN
            call write_message(R_NAME, 'ERROR', 
     $           'NSTRMS input to SCIATRAN is too small')
            write(EU,*) '    NSTRMS: ', nstrms
            return 1
         END IF


         IF ( NSTRMS < (NLEGEN-1)*0.43d0 ) THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
            WRITE(EU,*)
     $           '    Number of streams is too low for numerical'
            WRITE(EU,*)
     $           '    integration of Legendre polynomials.'
            WRITE(EU,*)
     $           '    Recommendation: Number of streams should be'
            WRITE(EU,*)
     $           '    at least larger than 0.43*(NLEGEN-1)'
            WRITE(EU,*)
     $           '    Check "Number of streams" in file'//
     $           ' "control_ac.inp"'
            RETURN 1
         ENDIF
         
C     Check on accuracy of azimuthal series convergence
c     -------------------------------------------------
         IF (DO_AZIMUTH) THEN
            IF (ACCUR.GT.GT_MINIMUM_ACCUR) THEN
               call write_message(R_NAME, 'WARNING', 
     $              'Azimuth convergence crit. not stringent enough ?')
               write(EU,*)
     $              '    recommendation: less than ', gt_minimum_accur
            END IF
         END IF
      END IF


c     Plan-parallel geometry
c     ----------------------
      MAX_SZA = MAXVAL(DACOS(ANG_OUT(1,:)))*RAD_TO_DEG
      IF (TRIM(RTM_TYPE) == 'pp_scat') THEN
         IF (MAX_SZA > 75.0d0) THEN
            call write_message(R_NAME, 'WARNING', 
     $           'SZA too high for plane-parallel mode')
            WRITE(EU,*) '    Should be less than 75 deg '
     $           //'to mitigate the sphericity errors.'
         ENDIF
         IF (DO_REFRAC) CALL WRITE_MESSAGE 
     $        ( R_NAME, 'WARNING', 'Spherical geometry flag '//
     $        'not set for refractive atmosphere')
      ENDIF


C  Check geometrical input
C  =======================
      MAX_AZ = MAXVAL(ANG_OUT(3,:))*RAD_TO_DEG
      MIN_AZ = MINVAL(ANG_OUT(3,:))*RAD_TO_DEG
      IF (MIN_AZ < 0.0.OR.MAX_AZ > 360.0) 
     $     CALL WRITE_MESSAGE ( R_NAME, 'WARNING',
     $     'Relative azimuth angle out of bounds [0 - 360 deg]' )

      IF( TRIM(RTM_TYPE) /= 'spher_scat' ) THEN
         MAX_LOS = MAXVAL(DACOS(ANG_OUT(2,:)))*RAD_TO_DEG
c         IF(MAX_LOS > 40.0) THEN
c            CALL WRITE_MESSAGE (R_NAME,'WARNING','')
c            WRITE(EU,*)
c     $        '    Required viewing angle(s) out of bounds [0 - 40 deg]'
c            WRITE(EU,*)
c     $        '    for plane-parallel and pseudo-spherical mode.'
c         ENDIF
         IF(MAX_SZA > 93.0) CALL WRITE_MESSAGE (R_NAME, 'WARNING',
     $        'Solar zenith angle out of bounds [0 - 93 deg]')
      END IF

c     Albedo between zero and one ?
c     -----------------------------
      if ( (albedo_const .lt. 0.0D0) 
     &      .or. (albedo_const .gt. 1.0D0) ) then
         call write_message(R_NAME, 'WARNING', 
     $        'Albedo (const) outside [0,1]')
      endif


c     Requirements on spectral segments fullfilled ?
c     ----------------------------------------------
c     - end wavelength higher than start wavelength ?
c     -----------------------------------------------
      do i = 1, nsegs
         band_end= band_start(i)+(band_num(i)-1)*band_step(i)
         if (band_end .lt. band_start(i)) then   !!!tk
            call write_message(R_NAME, 'ERROR', 
     $      'Last wavelength of spectral segment less than first one')
            write(EU,*)
     $           '    Diagnostic: segment_number first_wl last_wl: ', i,
     $           band_start(i), band_end
            return 1
         endif
      enddo
c     - start wavelength increasing ?
c     -------------------------------
      do i = 1, nsegs - 1
         if (band_start(i+1).le.band_start(i)) then
            call write_message(R_NAME, 'ERROR', 
     $           'Spectral segments are not ordered by wavelength')
            return 1
         endif
      enddo       

c     - no overlap ?
c     --------------
      DO I = 1, NSEGS-1
         BAND_END = BAND_START(I) + (BAND_NUM(I) - 1)*BAND_STEP(I)
         IF(SRF(I)%LWB(BAND_END) > SRF(I+1)%SWB(BAND_START(I+1))) THEN
            CALL WRITE_MESSAGE(R_NAME, 'Warning', 
c            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $           'Overlap of spectral segments')
            WRITE(EU,*)
     $           '    Diagnostic (seg#, end_act_seg, start_next_seg):'
            WRITE(EU,*) i, SRF(I)%LWB(BAND_END),'>', 
     $           SRF(I+1)%SWB(BAND_START(I+1)),' !'
c            RETURN 1
         ENDIF
      ENDDO

c     X-section temperatures in wrong order ?
c     ---------------------------------------
      DO I = 1, N_XSEC_MOL
         IF (.NOT.TG_INFO(I)%FM_FLG) CYCLE
         IF (ABS_COEFF_NTEMP(I) == 1) CYCLE
         DO J = 2, ABS_COEFF_NTEMP(I)
            IF(ABS_COEFF_TEMP(I,J).LE.ABS_COEFF_TEMP(I,J - 1)) THEN
               call write_message(R_NAME, 'ERROR', 
     $              'Error in X-section temperatures (wrong order)')
               write(EU,*) '    Diagnostic (molec. id):',i
               RETURN 1
            ENDIF
         ENDDO
      ENDDO

c     Clouds and related stuff
c     ========================
      IF( DO_CLOUD ) CALL GT_CLOUD_CHECK(*880)
      
c     Line-by-line
c     ------------
      if (do_lbl) then

         if ((lbl_wing_cutoff.lt.0.0D0).or.
     &       (lbl_wing_cutoff.gt.100.0d0)) then
            call write_message(R_NAME, 'WARNING',
     &           'Check line wing cut-off wavenumber')
            write(EU, *) ' *** Selected value: ',
     &                          lbl_wing_cutoff
         endif
      endif

c     ESFT
c     ----
      IF( DO_ESFT.AND.DO_LBL ) THEN
         Msg(1)='Only line-by-line or ESFT mode is possible.'
         Msg(2)='Check "Line absorber treatment" in "control.inp" file.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)      
      ENDIF
      
c     Convolution
c     -----------
      IF( CONVOL_FLAG ) THEN
         IF( ALIN_FLAG ) THEN
            Msg(1)='Convolution option can not be used in'
            Msg(2)='  combination with the "Alternative input" mode.'
            Msg(3)='Check "Do convolution" control parameter in'
            Msg(4)='  "control.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)      
         ENDIF
         IF (ANY(SRF(:)%Delta(1).LE.0.d0).or.
     $        ANY(SRF(:)%Delta(2).LE.0.d0)) THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     &           'Slit function spectral range less or equal zero')
            RETURN 1
         ENDIF

      ENDIF

c     Rotational Raman scattering calculations
c     ----------------------------------------
      IF( DO_RAMAN ) THEN
         IF( Nvs > 1.AND.RRS_ADJ.AND.RIN_OUT=='log') THEN
            Msg(1)='Output mode "log" for Ring spectra is not allowed'
            Msg(2)='  in the vector case.'
            Msg(3)=' Check the control parameter "Ring spectra '
            Msg(4)='  output mode" in "control_rrs.inp" file.'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)             
         ENDIF
c        ... It can be used 'ms' or 'ss' scattering mode only
         SELECT CASE(TRIM(SCATT_MODE))
         CASE('ms','ss')
         CASE DEFAULT 
            Msg(1)='Raman scattering can be used in combination with'
            Msg(2)='  scattering mode "ms" os "ss" only.'
            Msg(3)=' Check the control parameter "Scattering mode"'
            Msg(4)='  in "control_ac.inp" file.'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)            
         END SELECT
         IF( TO_REFL_FUNC ) THEN
            Msg(1)='Output in the form of "reflection function" cannot'
            Msg(2)='  be used in combination with Raman scattering.'
            Msg(3)='Check the control parameter "Transform to '
            Msg(4)='  reflection function?" in "control_out.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
c        ... Raman scattering in combination with discrete-ordinates
         IF( RRS_DO ) THEN
            IF( Pol_beam ) THEN
               Msg(1)='Polarized incident light beam can not be'
               Msg(2)='  used in combination with rotational Raman'
               Msg(3)='  scattering and discrete-ordinates technique.'
               Msg(4)='Check "Stokes vector of incident beam" control ' 
               Msg(5)='   parameter in "control.inp" file'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)               
            ENDIF
c           ... Adaptive grid
            IF( ADAPTIVE_GRID ) THEN
               Msg(1)='Adaptive grid technique can not be used in'
               Msg(2)='  combination with rotational Raman scattering'
               Msg(3)='  and discrete-ordinates technique.'
               Msg(4)='To continue set "Adaptive grid" control ' 
               Msg(5)='  parameter in "control_ac.inp" file to "f".'   
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
            ENDIF
c           ... Single scattering correction algorithm
            IF( DO_SSRCA .AND. TRIM(SCATT_MODE)/='ss' ) THEN
               Msg(1)='Single scattering correction algorithm can not'
               Msg(2)='  be used in combination with rotational Raman'
               Msg(3)='  scattering and discrete-ordinates technique.'
               Msg(4)='To continue set "Single scattering correction" ' 
               Msg(5)='  control parameter in "control_ac.inp" file'
               Msg(6)='  to "f".'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
            ENDIF
c           ... Post-processing technique is required (INT_OUT=.FALSE.)
            IF( .NOT.INT_OUT.AND.TRIM(RTM_TYPE)/='spher_scat') THEN
               Msg(1)='Source function integration technique can not'//
     $              ' be used in'
               Msg(2)='  combination with rotational Raman scattering'
               Msg(3)='  and discrete-ordinates technique.'
               Msg(4)='To continue set "Post-processing type" control' 
               Msg(5)='  parameter in "control_ac.inp" file'   
               Msg(6)='  to "Interp".'   
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
            ENDIF
c        ... Raman employing DO technique
         ENDIF

c            IF( TRIM(RTM_TYPE)=='ps_scat') THEN
c               Msg(1)='Raman scattering mode in combination with'
c               Msg(2)='  forward-adjoint technique is not implemented'
c               Msg(3)='  in the case of pseudo-spherical atmosphere.' 
c               Msg(4)='Use mode "dom" instead of mode "adj".'
c               Msg(5)='  See "control_rrs.inp" file.'   
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
c            ENDIF
c        ... Raman employing forward-adjoint technique
c         ENDIF
c        ... Angular interpolation of the radiation field is required 
c            (INT_OUT=.TRUE.) alone with the forward-adjoint technique 
c            for the rotational Raman scattering
         IF( INT_OUT.AND.RRS_ADJ.AND.TRIM(SCATT_MODE)/='ss' ) THEN
            Msg(1)='Angular interpolation technique can not be used in'
            Msg(2)='  combination with rotational Raman scattering'
            Msg(3)='  and forward-adjoint technique.'
            Msg(4)='The radiance can be calculated using standard'
            Msg(5)='  post-processing approach.'
            Msg(6)='To continue set "Post-processing type" control'   
            Msg(7)='  parameter in "control_ac.inp" file to "Int_SF".'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,7,*880)
         ENDIF
c        ... Analytic solution for conservative scattering is required 
c            (CS_ANALYTIC=.TRUE.) alone with the forward-adjoint  
c            technique for the rotational Raman scattering
         IF( CS_ANALYTIC.AND.RRS_ADJ.AND.TRIM(SCATT_MODE)/='ss' ) THEN
            Msg(1)='Raman scattering mode in combination with'
            Msg(2)='  forward-adjoint technique requires that parameter'
            Msg(3)='  "Analytic for conservative scattering" has to be'
            Msg(4)='  set to "f" (false).' 
            Msg(5)=' Check this control parameter in'
            Msg(6)='   "control_ac.inp" file.'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         ENDIF
         IF( RAY_SCATT_IDX /= 4 ) THEN
            Msg(1)='Raman scattering mode requires that parameter'
            Msg(2)='  "Rayleigh scattering index" has to be set to "4".'
            Msg(3)='See input file "control_ray.inp" and check that'
            Msg(4)='  "Advanced Rayleigh settings" control parameter in' 
            Msg(5)='  "control.inp" file is set to "t".'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
         IF( TRIM(King_factor) /= 'from file') THEN
            Msg(1)='Raman scattering mode requires that parameter'
            Msg(2)='  "King factor" has to be set to "from file".'
            Msg(3)='See input file "control_ray.inp" and check that'
            Msg(4)='  "Advanced Rayleigh settings" control parameter in' 
            Msg(5)='  "control.inp" file is set to "t".'   
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
         IF( DO_RAY_TAU .OR. DO_RAY_DEP ) THEN
            Msg(1)='Scaling of Rayleigh scattering coefficient'
            Msg(2)='  and Rayleigh optical thickness is not allowed'
            Msg(3)='  in combination with Raman scattering mode.'
            Msg(4)='Check input files: "control_ray.inp"' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF( TRIM(APPR_RRS)=='exact matching'.AND.DO_ESFT ) THEN
            Msg(1)='Raman scattering using ESFT-mode can be calculated'
            Msg(2)='  employing the "simple binning" mode only.'
            Msg(3)='Check the "Approach to calculate contribution'
            Msg(4)='  of Raman lines" control parameter'
            Msg(5)='  in "control_rrs.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
      ENDIF

c     Planck calculations
c     -------------------
      IF( DO_PLANCK ) THEN
         IF( BRIGHT_TEMP ) TO_REFL_FUNC = .FALSE.
         IF( TRIM(RTM_MODE) == 'spher_alb' ) THEN
            Msg(1)='Spherical albedo can not be calculated in'
            Msg(2)='  the case of thermal emission.'
            Msg(3)='Check "RTM Mode" control parameter in'
            Msg(4)='  "control.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
         IF(BRDF_FLAG) THEN
            Msg(1)='Thermal emission is not implemented in "BRDF" mode.'
            Msg(2)='Check "Surface reflection type" control parameter'
            Msg(3)='  in "control.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ENDIF
         IF (TRIM(RTM_CORE) == 'CDI') THEN
            Msg(1)='Thermal emission is not implemented in "CDI" mode.'
            Msg(2)='Check "Thermal emission" control parameter'
            Msg(3)='  in "control.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
         ENDIF
         IF( DO_RAMAN ) THEN
            Msg(1)='Thermal emission and Raman scattering can not be'
            Msg(2)='  calculated simultaneously.'
            Msg(3)='Check "Thermal emission" control parameter'
            Msg(4)='  in "control.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
!         IF( WF_FLAG ) THEN
!            CALL WRITE_MESSAGE(R_NAME, 'WARNING',
!     &      'Thermal emission is not included in weighting functions!')
!         ENDIF
         IF (TRIM(SCATT_MODE) == 'ss_ms') THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     &      'Thermal emission can not be treated in "ss_ms" mode')
            RETURN 1
         ENDIF
      ENDIF

      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS 
c **********************************************************************
      SUBROUTINE CHECK_UWT(*)

      CHARACTER (LEN=9), PARAMETER :: R_NAME='CHECK_UWT'

      CALL ST_TRACK (R_NAME, 3, 'I')

      HSOL_TRUNC_IDX = NLEG_W - 1

c     ... Vibrational Raman scattering and/or Fluorescence
      IF( ANY(II(:)) ) THEN
         IF( DO_ESFT ) THEN
            Msg(1)='ESFT mode is not allowed in combination with'
            Msg(2)='  inelastic scattering processes in ocean.'
            Msg(3)='Check "Line absorber treatment" parameter in '
            Msg(4)='  "control.inp" file. (Temporary restriction)'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)      
         ENDIF  ! DO_ESFT
         IF( CONVOL_FLAG ) THEN
            Msg(1)='Convolution mode is not allowed in combination '
            Msg(2)='  with inelastic scattering processes in ocean.'
            Msg(3)='Check "Do convolution" parameter in "control.inp"'
            Msg(4)='  file. (Temporary restriction)'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)      
         ENDIF  ! CONVOL_FLAG
         IF( NSEGS > 1 ) THEN
            Msg(1)='Calculation of inelastic scattering is implemented'
            Msg(2)='  yet for a single spectral segment only.'
            Msg(3)='Check "Wavelength segment info" parameter'
            Msg(4)='  in the file "control.inp".'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF  ! NSEGS > 1
         IF( LOS_DEF_FLAG /= 'ga' .AND. INEL_DO ) THEN
            SELECT CASE (TRIM(RTM_MODE))
            CASE ('spher_alb','flux')
            CASE DEFAULT
               IF( .NOT.INT_OUT ) THEN
               Msg(1)='Post-processing technique can not be used in'
               Msg(2)='  combination with inelastic processes in water.'
               Msg(3)='The radiance can be obtained via interpolation.'
               Msg(4)='To continue set "Post-processing type" control'
               Msg(5)='  parameter in "control_ac.inp" file'
               Msg(6)='  to "Interp".' 
                  CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
               ENDIF
            END SELECT
         ENDIF  ! LOS_DEF_FLAG and INEL_DO

         IF( INEL_ADJ ) THEN
            Msg(1)='The mode "adj" is preliminary not available in'
            Msg(2)='  combination with inelastic processes in water.'
            Msg(3)='Use mode "dom" instead. See "control_rrs.inp" file'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)

            IF(TRIM(RTM_MODE) == 'flux' .OR.
     $           TRIM(RTM_MODE) == 'spher_alb') THEN
               Msg(1)='Calculation of flux and spherical albedo in'
               Msg(2)='  combination with inelastic processes in water'
               Msg(3)='  is supported for mode "dom" only.'
               Msg(4)='Check parameter "Numerical technique used for '
               Msg(5)='  VRS" in "control_rrs.inp" file.'
               CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
            ENDIF
         ENDIF  ! INEL_ADJ

         IF( DO_PLANCK ) THEN
            Msg(1)='Inelastic processes in water are not supported'
            Msg(2)='  in combination with thermal emission.'
            Msg(3)='Check "Thermal emission" control parameter '
            Msg(4)='  in "control.inp" file.'
            Msg(5)='Check "Inelastic processes wihtin water" control'
            Msg(6)='  parameter in "control_uwt.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         ENDIF  ! DO_PLANCK
         IF( Vec_sca == 'vect_rte' ) THEN
            Msg(1)='Inelastic processes in water are implemented'
            Msg(2)='  in the scalar case only.'
            Msg(3)='Check "RTM-CORE" control parameter'
            Msg(4)='  in "control.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF  ! Vec_sca
c         IF( TRIM(RTM_TYPE) /= 'pp_scat') THEN
c            Msg(1)='Inelastic processes in water are implemented'
c            Msg(2)='  in plane-parallel case only.'
c            Msg(3)='Check "RTM_TYPE" control parameter'
c            Msg(4)='  in "control.inp" file.'
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
c         ENDIF
      ENDIF  ! ANY(II(:)) Vibrational Raman and/or Fluorescence
c     ... Polarization in AOS model
      IF( Vec_sca == 'vect_rte' ) THEN
         IF( TRIM(A_RTmat)=='FT' ) THEN
            Msg(1)='Option "FT" is not implemented in the '
            Msg(2)='  coupled ocean-atmosphere model '
            Msg(3)='  in the vector case (including polarization).'
            Msg(4)='Check "Approximation of RT matrixes" control'
            Msg(5)='   parameter in "control_uwt.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
         ENDIF
      ENDIF
c     ... Thermal emission
      IF( DO_PLANCK ) THEN
         Msg(1)='Thermal emission is not supported in'
         Msg(2)='  ocean-atmosphere RT model.'
         Msg(3)='Check "Thermal emission" control parameter '
         Msg(4)='  in "control.inp" file.'
         Msg(5)='Check "Include radiative transfer within ocean"'
         Msg(6)='   control parameter in "control_uwt.inp" file.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
      ENDIF
c     ... Land fluorescence
      IF( FLU_L ) THEN
         Msg(1)='Land fluorescence is not supported for the coupled'
         Msg(2)='  ocean-atmosphere model.'
         Msg(3)='Check "Include land fluorescence" control parameter'
         Msg(4)='  in "control.inp" file.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
      ENDIF
c     ... Asymptotic 
      IF( TRIM(RTM_CORE) == 'ASYMP' ) THEN
         Msg(1)='Asymptotic model is not supported for the coupled'
         Msg(2)='  ocean-atmosphere model.'
         Msg(3)='Check "RTM-CORE" control parameter'
         Msg(4)='  in "control.inp" file.'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
      ENDIF

c     ... WF and "SPECULAR"
      IF( WF_FLAG .OR. OWF_FLAG ) THEN
c         Msg(1)='Weighting functions are not supported for the coupled'
c         Msg(2)='  ocean-atmosphere model.'
c         Msg(3)='Check "RTM Mode" control parameter'
c         Msg(4)='  in "control.inp" file.'
c         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         IF( SPECULAR ) THEN
            Msg(1)='Weighting functions are not supported'
            Msg(2)='  in the case of "specular" reflection.'
            Msg(3)='Check "Flat or wind-roughed ocean surface"'
            Msg(4)='  control parameter in "control_uwt.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
         ENDIF
      ENDIF
c     ... Adaptive grid
      IF( ADAPTIVE_GRID ) THEN
         Msg(1)='Adaptive grid technique is not implemented'
         Msg(2)='  yet in the coupled OA model.'
         Msg(3)='To continue set "Adaptive grid" control parameter'
         Msg(4)='  in "control_ac.inp" file to "f".'   
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
      ENDIF
      IF( AG_aer ) THEN
         Msg(1)='Adaptive grid technique is not implemented'
         Msg(2)='  yet in the coupled OA model.'
         Msg(3)='To continue calculations set '
         Msg(4)='  "Adaptive grid (aerosol, cloud)" control parameter '
         Msg(5)='   in "control_ac.inp" file to "f".'   
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)
      ENDIF
c     ... Single scattering correction algorithm
      IF( DO_SSRCA ) THEN
         Msg(1)='Single scattering correction algorithm'
         Msg(2)='  is not implemented yet in the coupled OA model.'
         Msg(3)='To continue set "Single scattering correction" control'
         Msg(4)='  parameter in "control_ac.inp" file to "f".'   
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
      END IF

      IF( DO_RAMAN ) THEN
         
         IF( II(1).AND.(SNGL(STEP_VRS)<SNGL(DEL_RAM)) ) THEN
            Msg(1)='Rotational Raman scattering in combination with'
            Msg(2)='  VRS can be properly calculated if integration'
            Msg(3)='  wavelength step is larger or equal to the step'
            Msg(4)='  size for binning of rotational Raman lines.'
            Msg(5)='Check "Integrate with FRF" and "Raman binning step"'
            Msg(6)='  control parameters in "control_rrs.inp" file.'
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
         ENDIF
c        ... RRS in combination with inelastic processes in water
         IF( TRIM(APPR_RRS)=='exact matching'.AND.ANY(II(:)) ) THEN
            Msg(1)='Rotational Raman scattering in combination with'
            Msg(2)='  other inelastic processes can be calculated'
            Msg(3)='  employing the "simple binning" mode only.'
            Msg(4)='Check the "Approach to calculate contribution'
            Msg(5)='  of Raman lines" control parameter'
            Msg(6)='  in "control_rrs.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)            
         ENDIF
         IF( II(1).AND..NOT.Integr_FRF ) THEN
            Msg(1)='Rotational Raman scattering in combination with'
            Msg(2)='  vibrational Raman scattering can be calculated'
            Msg(3)='  setting "Integrate with FRF" parameter to "t".'
            Msg(4)='Check the "Integrate with FRF" control parameter'
            Msg(5)='  in "control_rrs.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
         ENDIF
c         IF( II(2).OR.II(3) ) THEN
c            Msg(1)='Rotational Raman scattering can not be calculated'
c            Msg(2)='  simultaneously with Chl-a or CDOM fluorescence'
c            Msg(3)='  (preliminary restriction).'
c            Msg(4)='Check the "Inelastic processes wihtin water"'
c            Msg(5)='  control parameter in "control_rrs.inp" file.' 
c            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
c         ENDIF
         IF( RRS_ADJ.AND.ANY(II(:)) ) THEN
            Msg(1)='Rotational Raman scattering in combination with'
            Msg(2)='  vibrational Raman scattering or fluorescence '
            Msg(3)='  can be calculated using DO-technique only.'
            Msg(4)='Check the "Numerical technique used for RRS" '
            Msg(5)='  control parameter in "control_rrs.inp" file.' 
            CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)            
         ENDIF
      ENDIF
c      IF( DO_RAMAN ) THEN
c         CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
c         WRITE(EU,*) '  Raman scattering is not implemented yet '
c         WRITE(EU,*) '  in the coupled ocean-atmosphere model.'
c         WRITE(EU,'(A,A)') '   Check input file: ','"control.inp"'
c         RETURN 1      
c      ENDIF

      IF( TRIM(OC_LAYERING) == 'from file' ) THEN 
         DO I = 1,NDEP_INP-1
            IF( DEP_INP(I) > DEP_INP(I+1) ) THEN
               CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
               WRITE(EU,*) '  Depth has to be an monotonically '//
     $              'increasing array.'
               WRITE(EU,*) '  However, we have: '
               WRITE(EU,'(A,I3,A,F7.2)') 
     $              '   Level - ',I,' Depth - ',DEP_INP(I)
               WRITE(EU,'(A,I3,A,F7.2)') 
     $              '   Level - ',I+1,' Depth - ',DEP_INP(I+1)
               WRITE(EU,*) '   Check input file:'
               WRITE(EU,'(A,A)') '  ',TRIM(DEP_NAME)
               RETURN 1
            ENDIF
         ENDDO
      ENDIF

      IF( NSTR_W .NE. NSTRMS ) THEN
         CALL WRITE_MESSAGE(R_NAME, 'WARNING', 
     $        ' The number of streams in the Fresnel cone ')
         WRITE(EU,*) '    should be equal to the number '//
     $        'of streams in the atmosphere.'
         WRITE(EU,*) 
     $        '    It is done in the "gt_settings_check.f"'
         NSTR_W = NSTRMS
      ENDIF

      IF ( NSTR_T + NSTR_W < (NLEG_W-1)*0.43d0 ) THEN
         Msg(1)='Number of streams in water is too low for accurate'
         Msg(2)='  integration of Legendre polynomials.'
         Msg(3)='Recommendation: Number of streams in the water'
         Msg(4)='  should be at least larger than 0.43*(NLEGEN-1).'   
         Msg(5)='Check "Number of streams" control parameter'   
         Msg(6)='  in "control_uwt.inp" file.'   
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)
      ENDIF
      
      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE CHECK_UWT

      END
C********************************************************************C
