C********************************************************************C
C
C	SCIATRAN - Version 4.2 - Last modification: 24 Oct 2020
C
C	Copyright (C) - Institute of remote sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE GT_SURFACE( Mh, JPOS, POS, UDB_A, UDB_W, * )
c     ... Call in "dom_spherical.f" and "spher_rad.f"

      USE ALB_PARS,   ONLY: DO_SPECTRAL_ALBEDO, N_ALBEDO_WAVS_DB,  
     $                      ALBEDO_CONST, ALBEDO_DB, ALBEDO_WAVS_DB, 
     $                      ALBEDO_RET, ALBEDO_PERTR, Nc_ALB, ALB_WEI,
     $                      WS1_A, WS2_A, DPOSALB, WL_Lca, V_Lca, 
     $                      INF_ALB
      USE DATA_RET,   ONLY: RET_FLAG
      USE INFO,       ONLY: WF_DB
      USE RETR_ITER,  ONLY: ITERATION
      USE RTM_OUTPUT, ONLY: ALB_OUT, ALB_WL
      USE RTM_PARS,   ONLY: DATA_ITER
      USE ST_VARS,    ONLY: SEG_NUM, RET_MODE, Pie
      USE TOOLS,      ONLY: INQ_WF, SET_PTR
      USE TYPE_VARS,  ONLY: KND
      USE TYPES,      ONLY: UDB_TYPE
      USE WATER,      ONLY: UWT, ALB_BOTTOM, OB_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: Mh, JPOS
      REAL (KND), INTENT(IN) :: POS
      TYPE (UDB_TYPE) :: UDB_A, UDB_W    ! Boundary conditions
c     -------------------------------
c     ... Local variables.
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GT_SURFACE'
      INTEGER     I, WS1, WS2, Nr, N2
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND)  S1, S2, DP, DUM
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( Mh == 1 ) THEN
         IF( Nc_ALB > 1 ) THEN
            CALL LIN_SEARCH_NEW(POS, DPOSALB, WS1_A, WS2_A, 
     $           N_ALBEDO_WAVS_DB, ALBEDO_WAVS_DB)
c                             WRITE(*,*) '    RET_MODE ', TRIM(RET_MODE)
            SELECT CASE( TRIM(RET_MODE) )
c           ... Retrieval mode
            CASE('aer-hom','aer-ratio')
c              ... Set albedo to zero to calculate black surface radiation
               UDB_A%ALBEDO = 0.d0
               IF( JPOS==1 ) THEN
                  CALL LIN_SEARCH_NEW(WL_Lca, DP, WS1, WS2, 
     $                 N_ALBEDO_WAVS_DB, ALBEDO_WAVS_DB)
                  DO I = 1,Nc_ALB
                     S1 = ALBEDO_DB(WS1,I)
                     S2 = ALBEDO_DB(WS2,I)            
                     V_Lca(I) = S1 + (S2 - S1)*DP
                  ENDDO
               ENDIF
            CASE DEFAULT
               DUM = 1.d0
               IF(INQ_WF('albedo', 'WFN')) THEN
                  CALL SET_PTR( RTM_PTR, 'albedo' )               
                  IF( ITERATION == 1 ) THEN
                     RTM_PTR%APR(1) = ALB_WEI(1,1)
                     RTM_PTR%CRN(1) = ALB_WEI(1,1) 
                  ELSE
                     ALB_WEI(1,1) = RTM_PTR%CRN(1)
                  ENDIF
               ENDIF
               N2 = 0
               IF( Nc_ALB==2.AND.INF_ALB(1) > 0 ) THEN
c                 ... wavelength number for which A and B  coefficients
c                     present absolute surface reflectance
                  Nr = INF_ALB(1)
c                 ... absolute surface reflectance
                  DUM = ALBEDO_DB(Nr,1)*ALB_WEI(1,1) + ALBEDO_DB(Nr,2)
c                      WRITE(*,*) '  NDPI ', ALB_WEI(1,1),
c     $                   '  A and B ', ALBEDO_DB(Nr,1), ALBEDO_DB(Nr,2)
c                      WRITE(*,*) '  Albedo at 560 nm ', DUM
                  N2 = INF_ALB(3)
               ENDIF           
               S1 = SUM( ALBEDO_DB(WS1_A,:)*ALB_WEI(1,:) )
               S2 = SUM( ALBEDO_DB(WS2_A,:)*ALB_WEI(1,:) )            
               UDB_A%ALBEDO = S1 + (S2 - S1)*DPOSALB

c              ... Calculate surface albedo:
               IF( WS1_A <= N2 ) UDB_A%ALBEDO = UDB_A%ALBEDO*DUM

c               UDB_A%ALBEDO = UDB_A%ALBEDO*Pie

                      WRITE(*,*) '  Albedo:', UDB_A%ALBEDO
               IF( UDB_A%ALBEDO < 0.d0 ) THEN
                  WRITE(*,*) 'Current surface albedo < 0 ',UDB_A%ALBEDO
                  STOP 
               ENDIF

            END SELECT
         ELSE
c        ... Nc_ALB = 1 
            IF( DO_SPECTRAL_ALBEDO ) THEN
               CALL LIN_SEARCH_NEW ( POS, DPOSALB, WS1, WS2, 
     $              N_ALBEDO_WAVS_DB, ALBEDO_WAVS_DB )
               S1 = SUM( ALBEDO_DB(WS1,:)*ALB_WEI(1,:) )
               S2 = SUM( ALBEDO_DB(WS2,:)*ALB_WEI(1,:) )
               UDB_A%ALBEDO = S1 + (S2 - S1)*DPOSALB
cc              ... Scaling of spectral albedo for test purpose
c              UDB_A%ALBEDO = UDB_A%ALBEDO * 1.4
            ELSE
               UDB_A%ALBEDO = ALBEDO_CONST
            END IF

            UDB_A%ALB_FS = UDB_A%ALBEDO

            IF( RET_FLAG ) THEN
               IF(INQ_WF('albedo', 'WFN')) THEN
                  IF( Nc_ALB > 1 ) THEN
                     IF(ITERATION == 1) THEN
c                    This part will not work properly as  ALBEDO_RET(3,1) 
c                    is not the scaling factor. It is albedo itself.
                        STOP 'gt_surface: not yet updated!'
                        ALBEDO_RET(:,SEG_NUM(JPOS)) = 1.d0
                     ELSE
                        UDB_A%ALBEDO = ALBEDO_RET(3,1)*UDB_A%ALBEDO
                     END IF
                  ELSE
                     IF(ITERATION == 1) THEN
                        ALBEDO_RET(:,SEG_NUM(JPOS)) = UDB_A%ALBEDO
                     ELSE
                        UDB_A%ALBEDO = ALBEDO_RET(3,SEG_NUM(JPOS))
                     END IF
                  ENDIF
               ENDIF  ! Albedo as parameter to be retrieved
            ENDIF  ! RET_FLAG
         ENDIF  ! Nc_ALB

         IF( UWT ) THEN
            SELECT CASE( OB_TYPE )
            CASE('const')
               UDB_W%ALBEDO = ALB_BOTTOM
            CASE('model')
               CALL BOTTOM_ALB_MODEL
            END SELECT
         ENDIF  ! UWT

         UDB_A%ALBEDO = UDB_A%ALBEDO + ALBEDO_PERTR(SEG_NUM(JPOS))
         IF( ALB_OUT ) ALB_WL(JPOS) = UDB_A%ALBEDO
c               write(*,'(A,1x,f8.4)') 
c     $        '   Surface albedo in "gt_surface.f": ',UDB_A%ALBEDO
c              WRITE(*,*) '    ----- Test Albedo gt_surface -----'
c              WRITE(*,'(A,2x,e12.4)') '    Albedo : ', UDB_A%ALBEDO
c              PAUSE
      ELSE
c     ... Mh > 1 For all harmonics larger than zero
         UDB_A%ALBEDO = 0.d0
         IF( UWT ) UDB_W%ALBEDO = 0.d0
      END IF
 
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
C***********************************************************************

      SUBROUTINE BOTTOM_ALB_MODEL
      USE BRDF,   ONLY:  WL_refi_idx, Refi_idx, NIce, MP_bub, MP_gp
c     ------------------------------------------------------------------
      INTEGER I, K
      REAL (KND)  A0, alpha, Hel, Sig_t, tp, tau_ice, Gam, Refi
c     ==================================================================      
      CALL LIN_SEARCH_NEW(Pos, Gam, I, K, Nice, WL_refi_idx )
      Refi = (1.d0-Gam)*Refi_idx(I) + Gam*Refi_idx(K)
      Sig_t = MP_bub(1)

      alpha = 4.d0*Pie*Refi/POS*1.e9 ! ice absorption [m-1]
      tp = 8.d0/3.d0* alpha/Sig_t
      tau_ice = (Sig_t + alpha)* MP_gp(1)
      gam = 3.d0/4.d0*Sig_t/(Sig_t + alpha)*SQRT(tp*(tp + 2.d0))
      A0 = 1.d0 + tp - SQRT(tp*(tp + 2.d0))
      Hel = DEXP(-2.d0*gam*tau_ice)
c     ... Bottom albedo 
      UDB_W%ALBEDO = A0*(1.d0 - Hel)/(1.d0 - A0*A0*Hel)

      RETURN
      END SUBROUTINE BOTTOM_ALB_MODEL

      END
C***********************************************************************
