C********************************************************************
C
C	SCIATRAN - Version 2.0 - Last modification: 24 Jul 2003
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************
C********************************************************************
C
C       The follolwing subroutines and functions are based on: 
C       NUMERICAL RECIPES - The Art of Scientific Computing
C                           Second Edition
C       Copyright (C) Numerical Recipes Software 1986, 1992 
C              - SUBROUTINE GAULEG
C              - FUNCTION   DFRIDR 
C              - FUNCTION   PLGNDR
C              
C********************************************************************C

      Subroutine GAULEG ( X1, X2, X, W, N )

C     Taken from Numerical Recipes chapter 4
C     Returns Gauss Legendre abscissa and weights for range [X1,X2]

      INTEGER                 N
      DOUBLE PRECISION        X(N), W(N), X1, X2

C     Local variables

      DOUBLE PRECISION        YEPS
      PARAMETER               ( YEPS = 3.0D-14 )
      INTEGER                 M, I, J
      DOUBLE PRECISION        YXM, YXL, YZ, YP1, YP2, YP3, YPP, YZ1

C     *************************************

      M = (N+1)/2
      YXM = 0.5D0 * (X2+X1)
      YXL = 0.5D0 * (X2-X1)
      DO 10 I = 1, M
         YZ = DCOS( 3.141592654D0 * (I-0.25D0) / (N+0.5D0) )
 1       CONTINUE
         YP1 = 1.0D0
         YP2 = 0.0D0
         DO 20 J = 1, N
            YP3 = YP2
            YP2 = YP1
            YP1 = ( (2.0D0*J-1.0D0) * YZ * YP2 - (J-1.0D0) * YP3 ) / J
 20      CONTINUE
         YPP = N * (YZ*YP1-YP2) / (YZ*YZ-1.0D0)
         YZ1 = YZ
         YZ = YZ1 - YP1 / YPP
         IF ( DABS(YZ-YZ1) .GT. YEPS ) GOTO 1
         X(I)     = YXM - YZ * YXL
         X(N+1-I) = YXM + YXL * YZ
         W(I)     = 2.0D0 * YXL / ( (1.0D0-YZ*YZ) * YPP * YPP )
         W(N+1-I) = W(I)
 10   CONTINUE

      END
C********************************************************************C
C********************************************************************C
C
      Function PLGNDR ( L, M, X )

C     Function routine from Numerical Recipes (W. Press et.al. )
C     Chapter 6, p182 .  Given integers M and L such that 0<M<L with
C     equality possible, and variable X in range [-1,1], this function
C     computes the Associated Legendre Polynomials PLM(X) using a stable
C     recurrence relation on L (see p 182)

      INTEGER                 L, M
      DOUBLE PRECISION        X, PLGNDR

      INTEGER                 LL, I
      DOUBLE PRECISION        PMM, SOMX2, FACT, PMMP1, PLL, ONE
      PARAMETER               ( ONE = 1.0D0 )

c     Initialize some variables
c     ( mainly to keep the compilers quiet)
c     -------------------------------------
      pll = 0.0d0

      IF ( M.LT.0 .OR. M.GT.L. OR. DABS(X).GT.ONE ) THEN
         call write_message('plgndr', 'ERROR', 'Bad arguments')
         stop' ERROR function PLGNDR: bad arguments !'
      END IF

C     ------------ Compute P(M,M)  ------------------------

      PMM = ONE
      IF ( M .GT. 0 ) THEN
         SOMX2 = DSQRT ( (ONE - X ) * (ONE + X ) )
         FACT  = ONE
         DO 11 I = 1, M
            PMM  = - PMM * FACT * SOMX2
            FACT = FACT + 2.0D0
 11      CONTINUE
      END IF
      IF ( L .EQ. M ) THEN
         PLGNDR = PMM
      ELSE

C     ------------ Compute P(M,M+1)  ---------------------

         PMMP1 = X * (2*M+1) * PMM
         IF ( L .EQ. M+1 ) THEN
            PLGNDR = PMMP1
         ELSE

C     ------------Compute P(L,M)  -------------------------

            DO 12 LL = M+2, L
               PLL = ( X*(2*LL-1)*PMMP1 - (LL+M-1)*PMM ) / (LL-M)
               PMM = PMMP1
               PMMP1 = PLL
 12         CONTINUE
            PLGNDR = PLL
         END IF

      END IF
      END

C********************************************************************C
C********************************************************************
c Calculation of a numerical derivative
c input : function FUNC, position X, stepsize H
c output: value of derivative DFRIDR, error estimate ERR
!      FUNCTION DFRIDR ( FUNC, X, H, ERR )
!
!      implicit none
!
!      INTEGER NTAB, i, j
!      DOUBLE PRECISION CON, CON2, BIG,  SAFE
!      PARAMETER ( CON = 1.40D0, CON2 = CON * CON, BIG = 1.0D30,
!     &     NTAB = 10, SAFE = 2.0D0 )
!      DOUBLE PRECISION A(NTAB,NTAB), X, H, HH, ERR, ERRT, FAC
!      DOUBLE PRECISION DFRIDR, FUNC
!      EXTERNAL FUNC
!
!      IF ( H .EQ. 0.D0 ) then
!         call write_message('dfridr', 'ERROR', 'Bad argument (H)')
!         STOP 'Error in DFRIDR: H must be nonzero'
!      endif
!
!      HH     = H
!      A(1,1) = ( FUNC(X+HH) - FUNC(X-HH) ) / (2.0D0*HH)
!      ERR    = BIG
!
!      DO I = 2, NTAB            !tk
!         HH     = HH/CON
!         A(1,I) = ( FUNC(X+HH) - FUNC(X-HH) ) / (2.0D0*HH)
!         FAC    = CON2
!         DO  J = 2, I           !tk
!            A(J,I) = (A(J-1,I)*FAC - A(J-1,I-1))/(FAC - 1.0D0)
!            FAC    = CON2 * FAC
!            ERRT   = DMAX1 ( DABS( A(J,I)-A(J-1,I) ),
!     &               DABS( A(J,I)-A(J-1,I-1) ) )
!
!            IF ( ERRT .LE. ERR ) THEN
!
!               ERR = ERRT
!               DFRIDR = A(J,I)
!
!            ENDIF
!
!         End Do                 !tk
!
!         IF (DABS( A(I,I) - A(I-1,I-1) ) .GE. SAFE*ERR) GOTO 999
!
!      End Do                    !tk
!
! 999  CONTINUE
!
!      Return                    !tk
!      END
C********************************************************************

