C***********************************************************************
C
C           SCIATRAN - Version 4.5 - Last modification: 20 Oct 2021
C
C           General solution of homogeneous equation and 
C            particular solutions corresponding to the 
C               direct sun and the Planck function 
C
C***********************************************************************
      SUBROUTINE HOMOG_PARTIC(Mh, NrL, NL, NN, NSZA, LAY_GR, UDB,
     $     ALW, SCA_AL, DTR_AG, DTR, SSB, TE, SOLV_LOC, SPECULAR,
     $     Bet, Mus, T_AU, PSU, PMI, P01, P02, SDO, GSL, PART_U, PART_D,
     $     Ex_AG, Norm_lay, Psim_lay, Psip_lay, JPOS, *, FLAG)
c     ... Called from "solv_dom.f"

      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs, Ng1_brf, Ng2_brf, Nf_pla, NL_pla,
     $                       NN_pla
      USE IRRAD,       ONLY: Fbe
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_DSO, N_ESO, ERROR_TXT, 
     $                       EU, DIR_OUT
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: Vec_Sca, Pie, WF_FLAG, F14, CS_ANALYTIC,
     $                       PSAB, VST, DO_RAMAN, Finc, Kvs, D_sol,
     $                       PREC_FLAG
      USE SPHER_PARS,  ONLY: Nf => NN
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: TE_TYPE, SDO_TYPE, SSB_TYPE, GSL_TYPE,
     $                       UDB_TYPE
      USE WATER,       ONLY: INCL_INEL, ZZZ_UW
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), INTENT(IN) :: SOLV_LOC
      CHARACTER (LEN=1), INTENT(IN) :: FLAG
      LOGICAL,    INTENT(IN)  :: SPECULAR
      INTEGER,    INTENT(IN)  :: Mh, NrL, NL, NN, NSZA, JPOS
      INTEGER,    INTENT(IN)  :: LAY_GR(NL-1,2)
      REAL (KND), INTENT(IN)  :: ALW(NL-1), Mus(NSZA),
     $     PSU(NL,Nvs*NN,Nvs*NN), PMI(NL,Nvs*NN,Nvs*NN),
     $     P01(NL,Nvs*NN,Nvs*NSZA), P02(NL,Nvs*NN,Nvs*NSZA)
      REAL (KND), INTENT(OUT) :: Norm_lay(Nvs*NN,NL-1),
     $     Psip_lay(Nvs*NN,Nvs*NN,NL-1), Psim_lay(Nvs*NN,Nvs*NN,NL-1)
c     ----------------------------------------------------------------
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (GSL_TYPE) :: GSL    ! Angles
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     -------------------------------------------------------------
      REAL (KND), DIMENSION(NL) :: SCA_AL, DTR, T_AU, Bet
      REAL (KND)  DTR_AG(NrL), 
     $     PART_U(NrL,2*Nvs*NN,Kvs*NSZA), PART_D(NrL,2*Nvs*NN,Kvs*NSZA)
      REAL (KND),  TARGET :: Ex_AG(Nvs*NN,NrL-1)
c     Local variables
c     ---------------
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'HOMOG_PARTIC'
      LOGICAL  LO_ADD_ABS
      INTEGER  I, J, K, L, K1, K2, I1, I2, Jco, Jro, Ivs,
     $     FAIL, S_AG, E_AG, NNv, Yi, NJc(Nvs*NN), Njr(Nvs*NN)
      REAL (KND), DIMENSION (Nvs*NN,Nvs*NN) :: Dd, Ds, DdDs, Gp, Gm
      REAL (KND)  ALW_AG, Wdo, S1, PAP, Dum, Hel(NL), 
     $     XML(NN), Cjm(Nvs*NN), Sjm(Nvs*NN), XM_TWO(2*NN)
      REAL (KND), POINTER :: EXP_NU(:), Enu(:)
      REAL (KND), DIMENSION (Nvs*NN) :: Nuj, H3, H4
      COMPLEX (kind(0.0d0)) :: Cnu, Cmu
c     Local arrays for the eigenvalues problem
c     ========================================
      REAL (KND), DIMENSION (Nvs*NN)         :: RR, RI
      REAL (KND), DIMENSION (Nvs*NN,Nvs*NN ) :: VR, VI
      REAL (KND)  WORK( 20*Nvs*NN )
      INTEGER, POINTER ::  E_MIN(:)
      REAL (KND), POINTER ::  EXP_NU_NL(:,:), G_NL(:,:,:), Ajp(:,:,:,:),
     $     Bjp(:,:,:,:), AG(:), XM(:), UU(:), VV(:)
c
      INTEGER     Jc, Zn, Km
      REAL (KND)  CS, Ck, A1, B1, H_1, Re_a, Re_b, Dtj,
     $     QP(Nvs*NN), QM(Nvs*NN), aj(Kvs,Nvs*NN), bj(Kvs,Nvs*NN), 
     $     Psim_AG(Nvs*NN,Nvs*NN), Anv(Kvs,Nvs*NN), Bnv(Kvs,Nvs*NN),
     $     Psip_AG(Nvs*NN,Nvs*NN), Norm_AG(Nvs*NN) 
c      REAL (KND), PARAMETER :: Epsi = 1.0d-04
c      REAL (KND), PARAMETER :: Epsi = 1.0d-07  ! Old v3.1
      REAL (KND), TARGET :: Hnu(Nvs*NN)
      REAL (KND), POINTER :: a_k(:)
      INTEGER  Ko, JJ
      REAL (KND)  Taj, H1_Cs, Es_up, Amu0, Bmu0, Omn
      COMPLEX (kind(0.0d0)) :: Muc, Amuk, Bmuk, C1, C2 

cc      Additional arrays for ASYMTX, DGETRF, and DGETRI
c      INTEGER      Ierr, LWORK, IPIV(Nvs*NN), INFO
c      REAL (KND)   WKD(10*Nvs*NN), WORK_TRI(64*Nvs*NN)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c      WRITE(*,*) ' homog_partic.f --> SSB%Mue ',
c     $     acos(SSB%Mue(1,:))/Pie*180.
c      PAUSE      

c      WRITE(*,*) ' homog_partic.f --> DTR ', SHAPE(DTR)
c      DO I = 1, NL
c         WRITE(*,'(2x,I3,2x,*(1x,e12.4))') I, DTR(I)
c      ENDDO
c      PAUSE      

c      WRITE(*,*) '  SSB%EXS(I,1) in homog_partic '
c      DO I = 1, NL
c         WRITE(*,'(2x,I3,2x,e12.4)') I, SSB%EXS(I,1)
c      ENDDO

c      WRITE(*,*) 
c     $     ' ... Solar exponent at the top of corresponding medium'
c      WRITE(*,'(5x,10(1x,e12.4))') SSB%EXS(1,:)
c      WRITE(*,*) 
c     $     ' ... Solar exponent at the first SZA'
c      WRITE(*,'(5x,10(1x,e12.4))') SSB%EXS(:,1)
c      WRITE(*,*) '  PSU ', SUM(PSU)
c      WRITE(*,*) '  PMI ', SUM(PMI)
c      WRITE(*,*) '  Mh = ', Mh
c      WRITE(*,*) '  UDB%Ftra(1,1,:) ', UDB%Ftra(1,1,:)

      LO_ADD_ABS = .FALSE.
      IF( .NOT.CS_ANALYTIC .AND. Mh==1 ) LO_ADD_ABS = .TRUE.

      XM => GSL%XM ; AG => GSL%AG 
      E_MIN => SDO%E_MIN  ;  EXP_NU_NL => SDO%EXP_NU_NL
      G_NL  => SDO%G_NL   ;  Ajp => SDO%Ajp  ;  Bjp => SDO%Bjp

      Wdo = 1.d0 - Wup
      NNv = Nvs*NN
      HEL(1:NL-1) = Wup*SCA_AL(1:NL-1) + Wdo*SCA_AL(2:NL)
      SDO%MASK_NL(:)  = .FALSE.
      Ajp = 0.d0  ;  Bjp = 0.d0 ; PART_U = 0.0D0 ; PART_D = 0.d0

      XM_TWO( 1   :NN   ) =  XM( 1:NN )
      XM_TWO( NN+1:2*NN ) = -XM( 1:NN )

c       Preliminary to remove multiplication by XM(:) in vector case
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:)
      CASE( 'vect_rte' )
         XML(:) = 1.d0
      END SELECT

c      WRITE(*,*) '   Start do=loop'
      DO I = 1,NrL - 1
         EXP_NU => Ex_AG(:,I)
c         WRITE(*,*) '   -1 '
c        ... Level numbers corresponding to the position of
c            the upper and lower boundaries of the layer "I"
         S_AG = LAY_GR(I,1) ; E_AG = LAY_GR(I,2) + 1 ! The levels of standard grid
         ALW_AG = SUM( ALW(S_AG:E_AG-1) )
         PAP = 1.d0 - SUM( HEL(S_AG:E_AG-1) )/ALW_AG
c         IF( LO_ADD_ABS .AND. PAP < Epsi ) THEN
         IF( LO_ADD_ABS .AND. PAP < PSAB ) THEN
c                   WRITE(*,*) '       Layer ', I, ZZZ(I)
c                   WRITE(*,'(2x,A,2(1x,e15.7))') ' -1 : ', Pap,SCA_AL(I)
            ALW_AG = SUM( HEL(S_AG:E_AG-1) )/(1.d0 - PSAB)
            PAP = 1.d0 - SUM( HEL(S_AG:E_AG-1) )/ALW_AG
         ENDIF
c         WRITE(*,'(A,1x,I3,3(2x,e12.4))') 
c     $        '   -2 ', I, ALW_AG, SUM( HEL(S_AG:E_AG-1) ), PAP
c         Mean value of the phase function sum and difference
c                        in the layer
         DS = 0.0d0 ;  DD = 0.0d0

         FORALL( I1=1:NNv, I2=1:NNv )
            DS(I1,I2) = (SUM( PSU(S_AG+1:E_AG-1,I1,I2),1 ) +
     $         Wup*PSU(S_AG,I1,I2) + Wdo*PSU(E_AG,I1,I2))/ALW_AG
            DD(I1,I2) = (SUM( PMI(S_AG+1:E_AG-1,I1,I2),1 ) +
     $         Wup*PMI(S_AG,I1,I2) + Wdo*PMI(E_AG,I1,I2))/ALW_AG
         END FORALL

cc        ... Exclude multiple scattering
c         DS = 0.d0 ;  DD = 0.d0
c         We do not need to divide DS and DD by 2 in the vector case 
c         because ph_z%Egg(:,:,:,1)(PSU) and ph_z%Egg(:,:,:,2)(PMI) 
c         were calculated without coefficient 2.
c         WRITE(*,*) '   -3 '
         DO K2 = 1,NN
            FORALL(L = Nvs*(K2-1)+1:Nvs*K2) 
              DS(:,L) = DS(:,L)*AG(K2)
              DD(:,L) = DD(:,L)*AG(K2)
            END FORALL
         ENDDO
         DO K = 1,NN
            FORALL(L = Nvs*(K-1)+1:Nvs*K) 
              DS(L,L) = DS(L,L) - 1.d0/XM(K)
              DD(L,L) = DD(L,L) - 1.d0/XM(K)
            END FORALL
         ENDDO

c          WRITE(*,*) '   -4 '
         SELECT CASE(PREC_FLAG)
         CASE('sngl')
            DDDS = SNGL(MATMUL( DD,DS ))
         CASE('doub')
            DDDS = MATMUL( DD,DS )
         END SELECT

         S1 = SUM( DDDS )
         IF( ISNAN(S1) ) THEN
            WRITE(*,*) '   I and SUM( DDDS ) ',I, S1
            STOP '    ******** NaN in DDDS ********** '
         ENDIF
         
c     WRITE(*,*) ' ----- DDDS ---- Layer = ', I
c         WRITE(*,*) SUM(DDDS)

cc            WRITE(*,*) '    Subroutine "ASYMTX" is used'
c            CALL ASYMTX( DdDs, VR, RR, NNv, NNv, NNv, Ierr, WKD )
c            IF( Ierr /= 0 ) THEN
c               WRITE(*,*) '  Ierr  = ', Ierr
c               STOP '   Ierr from direct'
c            ENDIF
c            DdDs = VR
c            CALL DGETRF( NNv, NNv, DdDs, NNv, IPIV, INFO )
c            IF(INFO /= 0 ) THEN 
c               WRITE(LUNIT,*) ' DGETRF: INFO = ', INFO 
c               STOP
c            ENDIF
c            LWORK = 64*NNv
c            CALL DGETRI( NNv, DdDs, NNv, IPIV, WORK_TRI, LWORK, INFO )
cc           ... j-th column of DdDs is a j-th left eigenvector of matrix DdDs
c            VI = TRANSPOSE(DdDs)  ;  RI = 0.d0 

         RR = 0.d0 ; RI = 0.d0
c          WRITE(*,*) '   -5 '
         CALL DGEEV ('V','V', NNv, DDDS, NNv, RR, RI, VI, NNv, 
     $        VR, NNv, WORK, 20*NNv, FAIL)
         IF( FAIL.NE.0 ) WRITE(LUNIT,*) 'EV PROBLEM --> IFAIL = ', FAIL
c         WRITE(*,*) '   +5 '

         Jco = 0 ; Jro = 0 
         DO J = 1,NNv
            IF( RI(J) .EQ. 0.0 ) THEN
               Jro = Jro + 1 ; Njr(Jro) = J
            ENDIF
c           ... If RI(J) > 0 then corresponding eigenvalue is complex
c               and, therefore, RI(J+1) = -RI(J)
            IF( RI(J) > 0.0 ) THEN
c               WRITE(*,'(A,2(1x,I3),E12.4)') 'RI(J)>0.0', I, J, RI(J)
               Jco = Jco + 1 ; NJc(Jco) = J
            ENDIF
         ENDDO

c         IF( FLAG == 'O' ) THEN
c            WRITE(LUNIT,'(A,I3,2x,ES12.4)')
c     $           '   Altitude layer:', I, ZZZ_UW(I)
c            WRITE(LUNIT,*) ' min RR --> ', MINVAL( RR(:) )
c         ENDIF                  ! FLAG == 'O'

         SDO%NcC(S_AG:E_AG-1) = Jco
c         WRITE(*,*) '   -6 '
         IF( Mh==1.AND.(.NOT.LO_ADD_ABS) ) THEN
            K2 = MINLOC( RR(Njr(1:Jro)),1 )
c            IF( PAP < Epsi .OR. RR(Njr(K2)) < 0.d0 ) THEN
            IF( PAP < PSAB .OR. RR(Njr(K2)) < 0.d0 ) THEN               
c               STOP '  !!! Negative eigenvalue "homog_partic.f" !!! '
c               WRITE(*,*) ' Position in real :', K2
c               WRITE(*,*) ' Position in  all :', Njr(K2)
               E_MIN(S_AG:E_AG-1) = K2
               IF(LOG_LEVEL(1) >= N_ESO) THEN
                  IF( FLAG == 'O' ) THEN
                     WRITE(LUNIT,*) 
     $                    ' ******* Conservative case  ************* '
                     WRITE(LUNIT,'(A,I3,2x,ES12.4)')
     $                    '   Altitude layer:', I, ZZZ_UW(I)
c                     WRITE(LUNIT,*) ' E_MIN(I) = ',  E_MIN(S_AG)
                     WRITE(LUNIT,*) ' Pap & min RR -->', PAP,RR(Njr(K2))
                  ENDIF  ! FLAG == 'O'
               ENDIF  ! LOG_LEVEL(1) 
c             ============ Weighting function or Raman ================
c               IF( WF_FLAG .OR. DO_RAMAN .OR. ANY(INCL_INEL(:))) THEN
               IF( WF_FLAG .OR. DO_RAMAN ) THEN
                  ERROR_TXT = 'Pure scattering (no absorption) '//
     $            'is occurred.'
                  CALL WRITE_MESSAGE(R_NAME, 'ERROR', TRIM(ERROR_TXT))
                  WRITE(EU,'(a)') 
     $                 '   RTM mode "wf", "raman", and inelastic '//
     $                 'scattering in water (VRS, Fluorescence)'
                  WRITE(EU,'(a)')  '   can not be performed'
                  WRITE(EU,'(a)')  '   To perform calculations, '//
     $                 'set in "control_ac.inp" file'
                  WRITE(EU,'(a)') '   the "Analytic for conservative '//
     $                 'scattering" control parameter as follows:'
                  WRITE(EU,'(a)')
     $              '   "f  1.d-4"'
                  RETURN 1
               ENDIF
c              =========================================================
               RR(Njr(K2)) = 1.0D-14
               SDO%MASK_NL(S_AG:E_AG-1) =  .TRUE.
c              -----------------------------------------------
            ENDIF
         ENDIF

c         WRITE(*,*) '   After first conserv. case Jco & Jro',Jco, Jro
         DO J = 1, Jro ; L = Njr(J)
c            Nuj(J) = 1.d0/DSQRT( RR(L) )
            Nuj(J) = 1.d0/DSQRT( ABS(RR(L)) )
         ENDDO

         L = Jro + 1
         DO J = 1, Jco ; K = Njc(J)
            Cnu   = DCMP( RR(K),RI(K) ) ; Cmu = 1.d0/SQRT(Cnu)
            Nuj(L) = DBLE( Cmu )        ;  Nuj(L+1) = AIMAG( Cmu )
            L = L + 2
         ENDDO
         
         EXP_NU(1:Jro) = DEXP( -DTR_AG(I+1)/Nuj(1:Jro) )
         DO J = Jro+1, Jro+2*Jco, 2
            Cmu = EXP( -DCMP(DTR_AG(I+1))/DCMP( Nuj(J),Nuj(J+1) ) )
            EXP_NU(J) = DBLE( Cmu ) ; EXP_NU(J+1) = AIMAG( Cmu )
         ENDDO         

c         WRITE(*,*) '  I ', I, SUM( EXP_NU(:) )
         DO K = S_AG,E_AG-1
            EXP_NU_NL(1:Jro,K) = DEXP( -DTR(K+1)/Nuj(1:Jro) )
            DO J = Jro+1, Jro+2*Jco, 2 
               Cmu = EXP( -DCMP(DTR(K+1))/DCMP( Nuj(J),Nuj(J+1) ) )
               EXP_NU_NL(J,K) =DBLE(Cmu) ; EXP_NU_NL(J+1,K) =AIMAG(Cmu)
            ENDDO  ! over complex eigenvalues (J)
            SDO%Nuj_lay(:,K)   = Nuj(:)
         ENDDO  ! over sub-layers of adaptive grid (K)

c         WRITE(*,*) '  ---VR, VI   ',I, SUM(VR(:,:)), SUM(VI(:,:))
c        ... Array VR(:,:) contains all eigenvectors 
         DO J = 1, Jro ; L = Njr(J)
            H3 = MATMUL( DS(:,:),VR(:,L) )*Nuj(J)
c           ... Note that the signs in expressions for vectors Gm and Gp
c           before vector H3 are opposite here to those in STB Eq.(G-NL)
c           because the matrix DS used above has the opposite sign 
c           to the  matrix E in STB Eq.(Matr-E)
            Gm(:,J) = VR(:,L) + H3(:)
            Gp(:,J) = VR(:,L) - H3(:)
         ENDDO
         L = Jro + 1
         DO J = 1,Jco ; K = Njc(J)
            H3 = MATMUL( DS,VR(:,K) )  ;  H4 = MATMUL( DS,VR(:,K+1) ) 
            Gm(:,L)   = VR(:,K)   + (H3*Nuj(L) - H4*Nuj(L+1)) ! real part
            Gm(:,L+1) = VR(:,K+1) + (H4*Nuj(L) + H3*Nuj(L+1)) ! imaginary
            Gp(:,L)   = VR(:,K)   - (H3*Nuj(L) - H4*Nuj(L+1))
            Gp(:,L+1) = VR(:,K+1) - (H4*Nuj(L) + H3*Nuj(L+1))
            L = L + 2
         ENDDO

c        ... G_NL - The matrix of all elementary solutions of homogeneous 
c            equation. In the vector case those vectors are given by 
c            Eqs.(G-NL) in STB.
c             vector-scalar Deleniya na XM(K) net v scalarnom sluchae ??
         IF( Vec_Sca == 'vect_rte' ) THEN
            DO K = 1,NN
               Gp(4*K-3:4*K,1:4*NN)  = Gp(4*K-3:4*K,1:4*NN)*0.5/XM(K)
               Gm(4*K-3:4*K,1:4*NN)  = Gm(4*K-3:4*K,1:4*NN)*0.5/XM(K)
            ENDDO
         ENDIF

c           ******************* adaptive grid *************************
c         WRITE(*,*) '  ---  ',I, SUM(Gp(:,:)), SUM(Gm(:,:))
c        ... The matrix G_NL has the following structure
c         | Phi+ Phi- |  
c         | Phi- Phi+ |
c          Phi+ Phi- vectors are given according to Eqs.(G-NL) in STB
c        Second dimension - Gaussian angles
c        Third dimension  - separation constants
         G_NL(S_AG, 1:NNv      ,1:NNv      ) = Gp(:,:)
         G_NL(S_AG, 1+NNv:2*NNv,1:NNv      ) = Gm(:,:)
         G_NL(S_AG, 1:NNv      ,1+NNv:2*NNv) = Gm(:,:)
         G_NL(S_AG, 1+NNv:2*NNv,1+NNv:2*NNv) = Gp(:,:)

         IF( Vec_Sca == 'scal_rte' ) THEN
            G_NL(S_AG,:,:) = G_NL(S_AG,:,:)*0.5d0
         ENDIF

         IF( SDO%MASK_NL(S_AG) ) THEN ! Conservative
            S1 = 3.D0 - (SUM( Bet(S_AG+1:E_AG-1) )
     $           + Wup*Bet(S_AG) + Wdo*Bet(E_AG))/ALW_AG

            SDO%H1(S_AG:E_AG-1) = S1
            S1 = 3.D0/SDO%H1(S_AG)

            G_NL(S_AG,1:2*NNv,      E_MIN(S_AG)) = 0.0d0
            G_NL(S_AG,1:2*NNv,NNv + E_MIN(S_AG)) = 0.0d0

            G_NL(S_AG,1:2*NNv:Nvs,      E_MIN(S_AG)) =  S1*XM_TWO(:)
            G_NL(S_AG,1:2*NNv:Nvs,NNv + E_MIN(S_AG)) = -S1*XM_TWO(:)
c                  TEST-output
c            EXP_NU(E_MIN(S_AG)) = 1.0d0
         ENDIF
c         WRITE(*,'(A,3(2x,I3))') '     Layer, Jro, Jco ', I, Jro, Jco
c         WRITE(*,*) '2: ', I, SUM(G_NL(S_AG,:,:))
c        ... Particular solution of the direct RTE
c        =========================================
c         WRITE(*,*) ' Before I = ',I, NrL - 1
         CALL PART_SOL_DRTE(*880)
c         WRITE(*,*) ' After I = ',I, NrL - 1

c         if(any(isnan(VECTOR)))
         S1 = SUM( PART_U(I,:,:) )
         IF( ISNAN(S1) ) THEN
            WRITE(*,*) '   I and SUM( PART_U(I,:,:) ) ',I, S1
            WRITE(*,*) '    NaN in PART_U at level = ', I
            STOP
         ENDIF
      END DO  ! over all layers (I)

      NULLIFY(EXP_NU_NL, G_NL, Ajp, Bjp, AG, XM, E_MIN, Enu)
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS

C*********************************************************************
C    Particular solution of the direct RTE and related functions     *
C      This routine is called in do-loop over all layers !!          *
C*********************************************************************
      SUBROUTINE PART_SOL_DRTE(*)

      INTEGER  J1, Jg, Ks, Kg1, Kg2
      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'PART_SOL_DRTE'

c      WRITE(*,*) I, NNv, Jro
      DO J = 1,Jro ; L = Njr(J)
c                     WRITE(*,*)  J, Nuj(J)
         H4 = MATMUL( TRANSPOSE(Dd),VI(:,L) )*Nuj(J)
c        ... Arrays Psim_AG and Psip_AG are adjoint vectors
c            obtained using left eigenvectors of the matrix F*E
c            (see Eq.(Psi-vec) in STB). The multiplier 0.5 by 
c            vectors Psim and Psip will be accounted for later. 
c            The matrix W^(-1) is removed because it appears in
c            combination with matrix W, i.e. W^(-1)*W.
c        ... Note that the signs in expressions for vectors Psim_AG and 
c            Psip_AG before vector H4 are opposite here to those in STB 
c            Eq.(Psi-vec) because the matrix DD used above has 
c            the opposite sign to the  matrix F in STB Eq.(Matr-F).
c            The required sign for vectors Psim_AG and Psip_AG is 
c            obtained using the norm factor, Norm_AG, with negative sign.
c                     WRITE(*,*) J, SUM(VI(:,L)), SUM(H4(:))
         Psim_AG(:,J) =  VI(:,L) + H4(:) 
c                     WRITE(*,*) '   Psim_AG(:,J) ', SUM( Psim_AG(:,J))
         Psip_AG(:,J) = -VI(:,L) + H4(:)     
         Norm_AG(J)   = -SUM( VI(:,L)*VR(:,L) )
      ENDDO
      
c      WRITE(*,*) '    Jro and Jco ', Jro, Jco
c      IF( I == 106 ) THEN
c      WRITE(*,*) '   VI, H4      :', SUM(VI), SUM(H4)
c      WRITE(*,*) '   Psim, Psip 1:', SUM(Psim_AG), SUM(Psip_AG)
c      ENDIF

c     ... If complex separation constants are presented
      K = Jro + 1
      DO J = 1,Jco ; L = Njc(J)
         H3 =  MATMUL( TRANSPOSE(Dd),VI(:,L) )
         H4 = -MATMUL( TRANSPOSE(Dd),VI(:,L+1) )
         Psim_AG(:,K)   =  VI(:,L)   + (H3*Nuj(K) - H4*Nuj(K+1))
         Psim_AG(:,K+1) = -VI(:,L+1) + (H4*Nuj(K) + H3*Nuj(K+1))
         Psip_AG(:,K)   = -VI(:,L)   + (H3*Nuj(K) - H4*Nuj(K+1)) ! real part
         Psip_AG(:,K+1) =  VI(:,L+1) + (H4*Nuj(K) + H3*Nuj(K+1)) ! imaginary
         Norm_AG(K)   = -SUM( VI(:,L)*VR(:,L)  + VI(:,L+1)*VR(:,L+1) ) 
         Norm_AG(K+1) = -SUM( VI(:,L)*VR(:,L+1)- VI(:,L+1)*VR(:,L) ) 
         K = K + 2
      ENDDO

c      WRITE(*,*) '  ---> After  DO J = 1,Jco'

      FORALL( J = S_AG:E_AG-1 ) 
         Norm_lay(:,J)   = Norm_AG(:)
         Psim_lay(:,:,J) = Psim_AG(:,:)
         Psip_lay(:,:,J) = Psip_AG(:,:)
      END FORALL

      IF( TRIM(Solv_LOC) == 'Solv_ASI' ) RETURN

c      WRITE(*,*) '     =======> SPECULAR ', SPECULAR
      
c     ... Additional term for the particular solution in the case of
c         specular reflection (absolute calm water surface). The direct 
c         solar beam mirror-reflected from the surface will be taken 
c         into account 
c     =============> Specular reflection <==========================
      IF( SPECULAR.AND.(.NOT.TE%NO_SUN) ) THEN
c         write(*,*) '  "homog_partic" Kvs =  ', Kvs
c         write(*,*) ' SHAPE(UDB%Fref) ', SHAPE(UDB%Fref)  
c         write(*,'(2x,*(1x,E12.4))') UDB%Fref(1,1,:) 
c         write(*,'(2x,*(1x,E12.4))') UDB%Fref(1,2,:)          
         DO K = 1, NSZA  ;  Ks = (K-1)*Nvs
            Omn = SSB%EXS(NL,K) 
c           ... In the vector case the array UDB%Fref describes the
c               polarization of direct solar light reflected from
c               air-water interface including Fresnel coefficients.            
            DO J = S_AG,E_AG-1 
c              ... One or two DO-solution will be calculated
               DO Ivs = 1,Kvs                    
c                 ... Multiplication by solar Stokes vector to obtain
c                     single scattering source function. 
                  DO Jg = 1,NN
                     Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                     DO K2 = 1,Nvs
                        Qp(Kg1+K2-1) = SUM( (Wup*P02(J,Kg1:Kg2,Ks+K2)
c     $                       + Wdo*P02(J+1,Kg1:Kg2,Ks+K2))*Finc(Ivs,:) )
     $                       + Wdo*P02(J+1,Kg1:Kg2,Ks+K2))
     $                               *UDB%Fref(K,Ivs,:) )
                        Qm(Kg1+K2-1) = SUM( (Wup*P01(J,Kg1:Kg2,Ks+K2)
c     $                       + Wdo*P01(J+1,Kg1:Kg2,Ks+K2))*Finc(Ivs,:) )
     $                       + Wdo*P01(J+1,Kg1:Kg2,Ks+K2))
     $                              *UDB%Fref(K,Ivs,:) )
                     ENDDO  ! over Stokes components (K2)
                  ENDDO  ! over Gauss angles
                  Qp(:) = Fbe*F14*Qp(:)/ALW(J)
                  Qm(:) = Fbe*F14*Qm(:)/ALW(J)
c              >>>>>> Coefficients aj and bj of particular solution 
c                     (reflected solar beam) STB Eqs. (aj-p-spec), (bj-p-spec)
c              --> a) real separation constants
                  DO L = 1,Jro
                     aj(Ivs,L) = SUM(Psip_AG(:,L)*Qp + Psim_AG(:,L)*Qm )
                     bj(Ivs,L) = SUM(Psim_AG(:,L)*Qp + Psip_AG(:,L)*Qm )
                  ENDDO  ! over real separation constants (L)
c              --> a) complex separation constants
                    DO K1 = Jro+1, Jro+2*Jco, 2
                       C2 = DCMP( Norm_AG(K1),Norm_AG(K1+1) ) 
                       A1 = SUM(Psip_AG(:,K1)  *QP + Psim_AG(:,K1)  *QM)
                       B1 = SUM(Psip_AG(:,K1+1)*QP + Psim_AG(:,K1+1)*QM)

                       C1 = DCMP( A1,B1 )
                       aj(Ivs,K1)   = DBLE(C1/C2)*0.5 
                       aj(Ivs,K1+1) = AIMAG(C1/C2)*0.5
                     
                       A1 = SUM(Psim_AG(:,K1)  *QP + Psip_AG(:,K1)  *QM)
                       B1 = SUM(Psim_AG(:,K1+1)*QP + Psip_AG(:,K1+1)*QM)
                       C1 = DCMP( A1,B1 )
                       bj(Ivs,K1)   = DBLE(C1/C2)*0.5 
                       bj(Ivs,K1+1) = AIMAG(C1/C2)*0.5
                    ENDDO  ! over complex separation constants (K1)
                  aj(Ivs,1:Jro) = aj(Ivs,1:Jro)*0.5/Norm_AG(1:Jro) 
                  bj(Ivs,1:Jro) = bj(Ivs,1:Jro)*0.5/Norm_AG(1:Jro)
               ENDDO  ! over DO-solutions (Ivs)
c              ... Save aj and bj for specular reflected solar light
               SDO%Aj_spe (:,:,J,K) = aj(:,:) 
               SDO%Bj_spe (:,:,J,K) = bj(:,:) 

c          >>>>>> Calculation of Aj and Bj tau-depending coefficients
c                     according to Eqs. (Aj-spec) (Bj-spec) in STB
               CS = SSB%Mue(K,J) ; a_k => SSB%SIN_COEF(K,J,:)
               Taj = T_AU(J) ; Dtj = DTR(J+1) 
               CALL AjBj_spec(0) ! output is Anv & Bnv
               S1 = DEXP(-(T_AU(NL)-(Taj+Dtj))/Cs)*Omn
               Ajp(:,J,:,K) = Anv(:,:)*S1
               Bjp(:,J,:,K) = Bnv(:,:)*S1               
            ENDDO   ! over adaptive sub-grids (not present yet)

c          ... Calculation of Aj and Bj tau-depending coefficients 
c              (specular reflection of solar beam) if  
c              within the I-th layer there is pure scattering (no absorption)
c              see Eqs.(Aj-cs-spe-ag) (Bj-cs-spe-ag) in STB
            IF( SDO%MASK_NL(S_AG) ) THEN ! Start conservative case 
               Km = E_MIN(S_AG) 
               Omn = VST*SSB%EXS(NL,K)/4.d0/DTR_AG(I+1)
     $              *UDB%FR_sol(K,1,1) ! ???
               DO J = S_AG,E_AG-1  ;  Cs = SSB%Mue(K,J) 
                  H_1 = 3.d0 - (Wup*Bet(J) + Wdo*Bet(J+1))/ALW(J)
                  Taj = T_AU(J) 
                  Dtj = DTR(J+1)
                  CALL AjBj_spec_pure
                  S1 = DEXP( -(T_AU(NL) - (Taj+Dtj))/Cs )*Omn
c                 ... The same for both DO-solutions
                  Ajp(:,J,Km,K) = Re_a*S1
                  Bjp(:,J,Km,K) = Re_b*S1
               ENDDO  ! over sub-layers (J)
            ENDIF  ! conservative case 
         ENDDO  ! over solar zenith angle (K)
      ENDIF  ! "specular reflection" case
c     <============= Specular reflection >==================

c     ... Additional term for particular solution of thermal emission
      IF( TE%ADD_PLANCK ) CALL PART_SOL_PLANCK

c >>> Particular solution for the direct solar beam
      DO K = 1, NSZA  ;  Ks = (K-1)*Nvs
         IF( TE%NO_SUN ) THEN 
c        ... Thermal emission without solar light
            CALL AjBj_recurrence
            SDO%Aj_sol = 0.0D0
            SDO%Bj_sol = 0.0D0                                 
            GOTO 123
         ENDIF
c        ... The direct solar beam
         IF( S_AG < SSB%N_sha(K) ) Then ! For all layers above shadow
c             ... E_AG-1 > S_AG if the adaptive grid algorithm is used
            DO J = S_AG,E_AG-1 
c                ... One or two DO-solution will be calculated
               DO Ivs = 1,Kvs                    
c                 ... Multiplication of phase matrix by Stokes vector of   
c                     direct solar radiation to obtain single scattering
c                     source function.
c                       Because P01 was designed to describe 
c                     +Gauss --> Sol_ang, we multiply P01^T*Finc(1,:)
c                     to obtain single scattering Sol_ang --> +Gauss.
c                     (See Eqs.(Qp-sol) and (Qm-sol) in STB)
c                     The same hold for matrix P02 to obtain single 
c                     scattering Sol_ang --> -Gauss.
c                       Multiplication P01^T and P02^T from left and right by
c                     D4=diag{1,1,1,-1} is omitted because single scattered
c                     solar radiation does not contain V Stokes component.
c                     !!! It should be corrected in the case of !!!
c                              incoming polarized light
                  DO Jg = 1,NN
                     Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                     DO K2 = 1,Nvs
                        Qp(Kg1+K2-1)= SUM( (Wup*P01(J,Kg1:Kg2,Ks+K2) + 
     $                       Wdo*P01(J+1,Kg1:Kg2,Ks+K2)) !*Finc(Ivs,:) )
     $                       * UDB%Ftra(K,Ivs,:) )
                        Qm(Kg1+K2-1)= SUM( (Wup*P02(J,Kg1:Kg2,Ks+K2) + 
     $                       Wdo*P02(J+1,Kg1:Kg2,Ks+K2)) !*Finc(Ivs,:) )
     $                       * UDB%Ftra(K,Ivs,:) )
                     ENDDO
                  ENDDO
                  Qp(:) = Fbe*F14*Qp(:)/ALW(J)
                  Qm(:) = Fbe*F14*Qm(:)/ALW(J)
c          -----> Coefficients aj and bj for particularly solution 
c                     (direct solar beam) STB Eqs. (aj-d-dir), (bj-d-dir)
c              --> a) real separation constants
                  DO L = 1,Jro
                     aj(Ivs,L)= SUM(Psip_AG(:,L)*Qp + Psim_AG(:,L)*Qm)
                     bj(Ivs,L)= SUM(Psim_AG(:,L)*Qp + Psip_AG(:,L)*Qm)
                  ENDDO  ! over real separation constants (L)
c              --> a) complex separation constants
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     C2 = DCMP( Norm_AG(K1),Norm_AG(K1+1) ) 
                     A1 = SUM(Psip_AG(:,K1)  *QP + Psim_AG(:,K1)  *QM)
                     B1 = SUM(Psip_AG(:,K1+1)*QP + Psim_AG(:,K1+1)*QM)

                     C1 = DCMP( A1,B1 )
                     aj(Ivs,K1)   = DBLE(C1/C2)*0.5 
                     aj(Ivs,K1+1) = AIMAG(C1/C2)*0.5
                     
                     A1 = SUM(Psim_AG(:,K1)  *QP + Psip_AG(:,K1)  *QM)
                     B1 = SUM(Psim_AG(:,K1+1)*QP + Psip_AG(:,K1+1)*QM)
                     C1 = DCMP( A1,B1 )
                     bj(Ivs,K1)   = DBLE(C1/C2)*0.5 
                     bj(Ivs,K1+1) = AIMAG(C1/C2)*0.5
                  ENDDO  ! over complex separation constants (K1)
                  aj(Ivs,1:Jro) = aj(Ivs,1:Jro)*0.5/Norm_AG(1:Jro) 
                  bj(Ivs,1:Jro) = bj(Ivs,1:Jro)*0.5/Norm_AG(1:Jro)
               ENDDO  ! over DO-solutions (Ivs)
c                ... Save aj and bj for direct solar beam
               SDO%Aj_sol (:,:,J,K) = aj(:,:)
               SDO%Bj_sol (:,:,J,K) = bj(:,:)                                 
cc       -----< Coefficients aj and bj are calculated
c
c       -----> Calculation Ajp and Bjp tau-depending coefficients of particular 
c              solution corresponding to the direct solar light
c              according to Eqs. (coef-Aj) (coef-Bj) in STB
c                     WRITE(*,*) '   ', K,J, SSB%Mue(K,J)
               Cs = SSB%Mue(K,J) ; a_k => SSB%SIN_COEF(K,J,:)
               Dtj = DTR(J+1)
c               write(*,*) '   J+1 & DTR(J+1) ', J+1, DTR(J+1)
c               write(*,*) '          AjBj_forw: '
               CALL AjBj_forw(0) ! output is Anv & Bnv
c                ... Ajp and Bjp contain contribution from thermal emission 
c                    and specular reflection if present
               Ajp(:,J,:,K) =  Ajp(:,J,:,K) + Anv(:,:)*SSB%EXS(J,K) 
               Bjp(:,J,:,K) =  Bjp(:,J,:,K) + Bnv(:,:)*SSB%EXS(J,K) 
c       ------ tau-depending coefficients Aj and Bj are calculated.

c                 WRITE(*,*) '   --- anv ', J, SUM(Anv(:,:)),SSB%EXS(J,K) 
c                 WRITE(*,*) '   --- bnv ', J, SUM(Bnv(:,:))                 
            ENDDO ! over sub-layers adaptive grid (J)
c             ... Recurrence relations for Aj and Bj
            CALL AjBj_recurrence
c
c    -----> Calculation of Aj and Bj (direct solar beam, pure scattering)
c              see Eqs.(Aj-cs) (Bj-cs) in STB
            IF( SDO%MASK_NL(S_AG) ) THEN ! Start conservative case 
               Km = E_MIN(S_AG)
               Omn = VST/4.d0/DTR_AG(I+1)  
               DO J = S_AG,E_AG-1 ; Cs = SSB%Mue(K,J)
                  H_1 = 3.d0 - (Wup*Bet(J) + Wdo*Bet(J+1))/ALW(J)
                  H1_CS = H_1*Cs ; Taj = T_AU(J) 
                  Es_up = SSB%EXS(J,K) ; Dtj = DTR(J+1)
                  CALL AjBj_forw_pure
c                   ... The same for both DO-solutions
                  Ajp(:,J,Km,K) = Re_a*Es_up*Omn
                  Bjp(:,J,Km,K) = Re_b*Es_up*Omn
                  IF( .NOT.D_sol(1) ) THEN
                     Ajp(1,J,Km,K) = 0.d0 ; Bjp(1,J,Km,K) = 0.d0
                  ENDIF
               ENDDO  ! over sub-layers adaptive grid (J)
c                ... Recurrence relations
c                    See Eqs.(Aj-cs-rec), (Bj-cs-rec) in STB
               DO J = S_AG+1,E_AG-1 
                  Ajp(:,J,Km,K) = Ajp(:,J,Km,K) + Ajp(:,J-1,Km,K)
               ENDDO
               DO J = E_AG-2, S_AG, -1
                  Bjp(:,J,Km,K) = Bjp(:,J,Km,K) + Bjp(:,J+1,Km,K)
               ENDDO
            ENDIF  ! for conservative case 
c    -----< End of calculation  Aj and Bj coefficients (pure scattering)
         ELSE   ! for all layers within shadow
c             ... For sub-layers containing thermal emission only
            IF( TE%ADD_PLANCK ) CALL AjBj_recurrence
         ENDIF           
 123     CONTINUE
c    -----> Calculation of particular solution  
         DO Ivs = 1,Kvs
            K2 = (Ivs - 1)*NSZA + K
            VV => Ajp(Ivs,E_AG-1,:,K) ; UU => Bjp(Ivs,S_AG,:,K)  
c            WRITE(*,*) ' G_NL ', S_AG,SUM( G_NL(S_AG,:,:) )
c            WRITE(*,*) ' VV ',S_AG , SUM( VV )
c            WRITE(*,*) ' UU ',S_AG, SUM( UU )
c           ... Particular solution at the upper boundary of the layer "I"
            Part_U(I,:,K2) = 
     $           MATMUL(G_NL(S_AG,:,NNv+1:NNv+Jro),UU(1:Jro))
c           ... Particular solution at the lower boundary of the layer "I"
            Part_D(I,:,K2) = MATMUL( G_NL(S_AG,:,1:Jro  ),VV(1:Jro) )
c           ... Including of imaginary separation constants
            DO K1 = Jro+1, Jro+2*Jco, 2
               Part_U(I,:,K2) = Part_U(I,:,K2) + 2.d0*
     $              ( G_NL(S_AG,:,NNv+K1)  *UU(K1) 
     $              - G_NL(S_AG,:,NNv+K1+1)*UU(K1+1))
               Part_D(I,:,K2) = Part_D(I,:,K2) + 2.d0*
     $              ( G_NL(S_AG,:,K1)*VV(K1) 
     $              - G_NL(S_AG,:,K1+1)*VV(K1+1))
            ENDDO  ! over imaginary separation constants (K1)
         ENDDO  ! over DO-solutions (Ivs)
c    ------ End of particular solution.
      ENDDO  ! over solar zenith angle (K)

c      IF( LOG_LEVEL(1) >= N_DSO ) THEN
c         WRITE(*,*) '  End of PART_SOL_DRTE'
c         WRITE(*,*) '   Part_U at the top of layer and first SZA'
c         DO J = 1, NNv
c            WRITE(*,'(1x,I3,2(2x,e12.4))')
c     $           J, Part_U(I,J,1), Part_U(I,NNv+J,1)
c         ENDDO
c         WRITE(*,*) '   Part_D at the bottom of layer and first SZA'
c         DO J = 1, NNv
c            WRITE(*,'(2(1x,I3),*(2x,e12.4))')
c     $           I, J, Part_D(I,J,1) !, Part_D(I,NNv+J,1)
c         ENDDO
c      ENDIF  ! debug output

      END SUBROUTINE PART_SOL_DRTE

c **********************************************************************
      SUBROUTINE AjBj_spec(JI)
      INTEGER, INTENT(IN) :: JI
      S1 = DEXP( -Dtj/CS )
      SELECT CASE(JI)
      CASE(0)
         Enu => Exp_NU_NL(:,J)
      CASE DEFAULT
         Hnu(1:Jro) = DEXP( -Dtj/Nuj(1:Jro) )
         DO Ko = Jro+1, Jro+2*Jco, 2 
            Cmu = EXP( -DCMP(Dtj)/DCMP(Nuj(Ko),Nuj(Ko+1)) )
            Hnu(Ko) = DBLE(Cmu) ; Hnu(Ko+1) = AIMAG(Cmu)
         ENDDO         
c        ... Array Enu is used in  routine C_S_FUNC
         Enu => Hnu
      END SELECT
c      >>>>>> Calculation of C and S functions for particular 
c             solution of direct reflected solar light
      CALL C_S_FUNC(CS, S1, Dtj, SSB%N_sph(K,J), Nf, a_k)
c     --> a) real separation constants:
      FORALL( K1 = 1:Jro )
        Anv(:,K1) = aj(:,K1)*Sjm(K1) ; Bnv(:,K1) = bj(:,K1)*Cjm(K1)
      END FORALL
c     --> b) imaginary separation constants:
      DO K1 = Jro+1, Jro+2*Jco, 2 
         Anv(:,K1)   = aj(:,K1)*Sjm(K1)   - aj(:,K1+1)*Sjm(K1+1)
         Anv(:,K1+1) = aj(:,K1)*Sjm(K1+1) + aj(:,K1+1)*Sjm(K1)
         Bnv(:,K1)   = bj(:,K1)*Cjm(K1)   - bj(:,K1+1)*Cjm(K1+1)
         Bnv(:,K1+1) = bj(:,K1)*Cjm(K1+1) + bj(:,K1+1)*Cjm(K1)
      ENDDO
      END SUBROUTINE AjBj_spec

c **********************************************************************
      SUBROUTINE AjBj_spec_pure
c     ...  See Eq.(ab-mu0-spec) in STB
      Amu0 = -3.d0*Cs + H_1*(Taj        - T_AU(S_AG))
      Bmu0 = -3.d0*Cs - H_1*(T_AU(E_AG) - (Taj+Dtj) )
      S1 = DEXP( -Dtj/Cs )
      Re_a = -Cs*(Amu0*S1 - (Amu0 + H_1*Dtj)   )
      Re_b = -Cs*(Bmu0    - (Bmu0 - H_1*Dtj)*S1)

      END SUBROUTINE AjBj_spec_pure

c **********************************************************************
      SUBROUTINE AjBj_forw(JI)
      INTEGER, INTENT(IN) :: JI

c      WRITE(*,*) '    ----->> Layer, Jco, Cs ', J, Jco, Cs
      S1 = DEXP( -Dtj/Cs )
      SELECT CASE(JI)
      CASE(0)
         Enu => Exp_NU_NL(:,J)
      CASE DEFAULT
         Hnu(1:Jro) = DEXP( -Dtj/Nuj(1:Jro) )
         DO Ko = Jro+1, Jro+2*Jco, 2 
            Cmu = EXP( -DCMP(Dtj)/DCMP(Nuj(Ko),Nuj(Ko+1)) )
            Hnu(Ko) = DBLE(Cmu) ; Hnu(Ko+1) = AIMAG(Cmu)
         ENDDO         
         Enu => Hnu
      END SELECT

c     ------ Calculation of C and S functions needed for particular solution
c            of direct solar light
c      IF( I == 89 ) THEN
c         write(*,*) '   Cs, S1, Dtj, SSB%N_sph(K,J), Nf '
c         write(*,*)   Cs, S1, Dtj, SSB%N_sph(K,J), Nf
c         write(*,*) '   SUM(a_k) ', SUM(a_k)
c      ENDIF
      CALL C_S_FUNC(Cs, S1, Dtj, SSB%N_sph(K,J), Nf, a_k)
c       ------ Calculation of Aj and Bj tau-depending coefficients
c              --> a) real separation constants
c      IF( I == 89 ) THEN
c         write(*,*) 'aj, Cjm', SUM( aj ), SUM(Cjm) 
c         write(*,*) 'bj, Sjm', SUM( bj ), SUM(Sjm) 
c      ENDIF

      FORALL( K1 = 1:Jro )
        Anv(:,K1) = aj(:,K1)*Cjm(K1) ; Bnv(:,K1) = bj(:,K1)*Sjm(K1)
      END FORALL

c      WRITE(*,*) '   ---  ', J, SUM(bj(:,:)), SUM(Sjm(:))
c      WRITE(*,*) '   ---  ', J, SUM(aj(:,:)), SUM(Cjm(:))

c     --> b) imaginary separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2 
         Anv(:,K1)   = aj(:,K1)*Cjm(K1)   - aj(:,K1+1)*Cjm(K1+1)
         Anv(:,K1+1) = aj(:,K1)*Cjm(K1+1) + aj(:,K1+1)*Cjm(K1)
         Bnv(:,K1)   = bj(:,K1)*Sjm(K1)   - bj(:,K1+1)*Sjm(K1+1)
         Bnv(:,K1+1) = bj(:,K1)*Sjm(K1+1) + bj(:,K1+1)*Sjm(K1)
      ENDDO
      END SUBROUTINE AjBj_forw

c **********************************************************************
      SUBROUTINE AjBj_forw_pure
      S1 = DEXP( -Dtj/Cs )
c     ...  See Eq.(ab-mu0) in STB
      Amu0 = 3.d0*Cs + H_1*(T_AU (J)   - T_AU(S_AG))
      Bmu0 = 3.d0*Cs - H_1*(T_AU(E_AG) - T_AU(J+1))
      Re_a = Cs*(Amu0    - (Amu0 + H_1*Dtj)*S1)
      Re_b = Cs*(Bmu0*S1 - (Bmu0 - H_1*Dtj)   )
c     ... If approximation for spherical solar beam is used then
c         See Eqs.(Aj-cs-ps), (Bj-cs-ps) in STB
      Zn = -1
      DO Jc = 1,SSB%N_sph(K,J)-1
         Ck = Dtj/DBLE(Jc)/Pie
         Muc = CONJG( Cs*Ck/DCMP( Ck,-Cs ) )
         Dum = Cs*(3.d0 - H_1)
         Amuk = Dum + H_1*(T_AU (J)   - T_AU(S_AG) + Muc)
         Bmuk = Dum - H_1*(T_AU(E_AG) - T_AU(J+1)  - Muc)
         Re_a = Re_a + SSB%SIN_COEF(K,J,Jc)
     $        *( AIMAG(Muc*Amuk) - AIMAG(Muc*(Amuk+H_1*Dtj))*Zn*S1 )
         Re_b = Re_b + SSB%SIN_COEF(K,J,Jc)
     $        *( AIMAG(Muc*Bmuk)*Zn*S1 - AIMAG(Muc*(Bmuk-H_1*Dtj)) )
         Zn = -Zn
      ENDDO
      
      END SUBROUTINE AjBj_forw_pure

c **********************************************************************
      SUBROUTINE PART_SOL_PLANCK
      INTEGER JJ, Ko
c >>> Particular solution for the Planck function
c     ... For the vector case we don't need to divide by XM(K) 
      DO K = 1,NN
         FORALL(L = Nvs*(K-1)+1:Nvs*K) Qp(L) = 1.d0/XML(K)
      ENDDO
      DO J = S_AG,E_AG-1 
c           --> a) real separation constants
         DO L = 1,Jro
            aj(1,L) = SUM( (Psip_AG(:,L) + Psim_AG(:,L))*Qp )
         ENDDO
c           --> a) complex separation constants
         DO K1 = Jro+1, Jro+2*Jco, 2
            C2 = DCMP( Norm_AG(K1),Norm_AG(K1+1) ) 
            A1 = SUM( (Psip_AG(:,K1)   + Psim_AG(:,K1)  )*Qp )
            B1 = SUM( (Psip_AG(:,K1+1) + Psim_AG(:,K1+1))*Qp )
            C1 = DCMP( A1,B1 )
            aj(1,K1) = DBLE(C1/C2)*0.5 ; aj(1,K1+1) = AIMAG(C1/C2)*0.5
         ENDDO
         aj(1,1:Jro) = aj(1,1:Jro)*0.5/Norm_AG(1:Jro) 
         bj(1,:) = aj(1,:)
         TE%Aj_pla (:,J) = aj(1,:)*TE%NPLANCK(J)*VST
c       >>> Calculation of C and S functions needed for particular solution
c           of the internal source (Planck function)
         CS = -TE%Mup(J) ; a_k => TE%PL_COEF(J,:) ; Dtj = DTR(J+1)
         CALL AjBj_therm(0)
         DO K = 1,NSZA 
            Ajp(1,J,:,K) = Ajp(1,J,:,K) + Anv(1,:)*TE%NPLANCK(J)*VST
            Bjp(1,J,:,K) = Bjp(1,J,:,K) + Bnv(1,:)*TE%NPLANCK(J)*VST
         ENDDO
c      <<<< tau-depending coefficients Aj and Bj are calculated.
      ENDDO  ! over adaptive sub-layers (J)
      END SUBROUTINE PART_SOL_PLANCK

c *********************************************************************
      SUBROUTINE AjBj_therm(JI)
      INTEGER, INTENT(IN) :: JI
      SELECT CASE(JI)
      CASE(0)
         Enu => Exp_NU_NL(:,J)
      CASE DEFAULT
         Hnu(1:Jro) = DEXP( -Dtj/Nuj(1:Jro) )
         DO Ko = Jro+1, Jro+2*Jco, 2 
            Cmu = EXP( -DCMP(Dtj)/DCMP(Nuj(Ko),Nuj(Ko+1)) )
            Hnu(Ko) = DBLE(Cmu) ; Hnu(Ko+1) = AIMAG(Cmu)
         ENDDO         
         Enu => Hnu
      END SELECT
      S1 = DEXP( -CS*Dtj ) 
c     ... Array Enu is used in this routine !
      CALL C_S_FUNC_PLA(CS, S1, Dtj, TE%N_pla(J), Nf_pla, A_K)
c     >>>> Calculation of Aj and Bj tau-depending coefficients (Planck)
c     --> a) real separation constants- performed according to
c     Eqs. (Aj-pla) (Bj-pla) in STB
      Anv(1,1:Jro) = aj(1,1:Jro)*Cjm(1:Jro)
      Bnv(1,1:Jro) = bj(1,1:Jro)*Sjm(1:Jro)
c     --> b) imaginary separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2 
         Anv(1,K1)   = aj(1,K1)*Cjm(K1)   - aj(1,K1+1)*Cjm(K1+1)
         Anv(1,K1+1) = aj(1,K1)*Cjm(K1+1) + aj(1,K1+1)*Cjm(K1)
         Bnv(1,K1)   = bj(1,K1)*Sjm(K1)   - bj(1,K1+1)*Sjm(K1+1)
         Bnv(1,K1+1) = bj(1,K1)*Sjm(K1+1) + bj(1,K1+1)*Sjm(K1)
      ENDDO

      END SUBROUTINE AjBj_therm

c **********************************************************************
c         S1 = DEXP( -Dtj/Mue )
c         Nco  - number of coefficients for correction of spherical 
c                transmission;
c         NA_K - number of fine grid layers (NN = Nf);
c         A_K(1:NA_K) 
c              - coefficients of exponential series
c***********************************************************************
      SUBROUTINE C_S_FUNC(CS, S1, Dti, NCO, NA_K, A_K)
      INTEGER     NCO, NA_K
      REAL (KND)  CS, S1, Dti, A_K(NA_K)
      INTEGER     Zn, Jc
      REAL (KND)  Ck, V_i, V_r, H2, Eme
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2 

      Eme = S1
c     --> a) real separation constants
      WHERE( REAL(Nuj(1:Jro)) /= REAL(CS) )
         Cjm(1:Jro) = Nuj(1:Jro)*CS*(Enu(1:Jro) - Eme)/(Nuj(1:Jro) - CS)
      ELSEWHERE
         Cjm(1:Jro) = Enu(1:Jro)*Dti
      END WHERE
      Sjm(1:Jro) = Nuj(1:Jro)*CS*(1.d0 - Enu(1:Jro)*Eme)/(Nuj(1:Jro)+CS)

c      WRITE(*,*) '   ', CS, Eme
c      WRITE(*,*) SUM(Sjm(:)), SUM(Enu), SUM(  Nuj )
            
c     --> b) complex separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2
         C1  = DCMP( Nuj(K1),Nuj(K1+1) ) 
         Cnu = DCMP( Enu(K1),Enu(K1+1) )

         C2 =  C1*CS*(Cnu - Eme)/(C1 - CS)
         Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)

         C2 = C1*CS*(1.d0 - Cnu*Eme)/(C1 + CS)
         Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
      ENDDO

c     --> c) contribution of additional terms coming from approximation
c           of the solar exponent in the spherical atmosphere.
c           NCO > 1 in this case.
      Zn = -1
c      H2 = Zn*Eme ! Error
c      WRITE(*,*) '  Jco =  ',Jco
      DO Jc = 1,NCO-1
c        ... Corrected by VR 04.14.2015 !!
         H2 = Zn*Eme
c        ... Real separation constants (see Eqs.(C-sp) and (S-sp) in STB)
         Ck = Dti/DBLE(Jc)/Pie
c        ... Cs = Mue, Muc is a complex number
         Muc = Cs*Ck/DCMP( Ck,-Cs )

c         Cjm(1:Jro) = Cjm(1:Jro) 
c     $        + AIMAG( Muc*Nuj(1:Jro)/(Muc-Nuj(1:Jro)) )
c     $        * a_k(Jc)*(H2 - Enu(1:Jro))
         Cjm(1:Jro) = Cjm(1:Jro) 
     $        + AIMAG( Muc*Nuj(1:Jro)/(Nuj(1:Jro) - Muc) )
     $        * a_k(Jc)*(Enu(1:Jro) - H2)

         Sjm(1:Jro) = Sjm(1:Jro) 
     $        + AIMAG( Muc*Nuj(1:Jro)/(Muc+Nuj(1:Jro)) )
     $        * a_k(Jc)*(1.d0 - H2*Enu(1:Jro))

c        ... Complex separation constants 
c            (see Eqs.(C-sp-com)-(S-sp-com) in STB)
         DO K1 = Jro+1, Jro+2*Jco, 2
            C1 = DCMP( Nuj(K1),Nuj(K1+1) )
            C2 = ( CONJG(Muc)*C1/(CONJG(Muc)-C1)
     $           - Muc*C1/(Muc-C1) )*0.5d0
            V_i = AIMAG(C2)*Enu(K1) +  DBLE(C2)*Enu(K1+1)
            V_r =  DBLE(C2)*Enu(K1) - AIMAG(C2)*Enu(K1+1)
            Cjm(K1)   = Cjm(K1)   - a_k(Jc)*(AIMAG(C2)*H2 - V_i)
            Cjm(K1+1) = Cjm(K1+1) + a_k(Jc)*( DBLE(C2)*H2 - V_r)
            Sjm(K1)   = Sjm(K1)   - a_k(Jc)*(AIMAG(C2) - H2*V_i)
            Sjm(K1+1) = Sjm(K1+1) + a_k(Jc)*( DBLE(C2) - H2*V_r)
         ENDDO
         Zn = -Zn
      ENDDO   ! over correction terms (Jc)

c      IF( I == 1 ) THEN  ! I - main altitude grid layer number
c         write(*,*) '   2 - ', Cjm(1:Jro)
c      ENDIF

      END SUBROUTINE C_S_FUNC

c **********************************************************************
      SUBROUTINE C_S_FUNC_PLA(CS, S1, Dti, NCO, NA_K, A_K)
      INTEGER      NCO, NA_K
      REAL (KND)   CS, S1, Dti, A_K(NA_K)
      INTEGER      Zn, Jc
      REAL (KND)   Csk, Ck, V_i, V_r, H2
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2 

c     --> a) real separation constants
      WHERE( REAL(CS*Nuj(1:Jro)) /= REAL(1.d0) )
         Cjm(1:Jro) = Nuj(1:Jro)*(Enu(1:Jro)-S1)/(CS*Nuj(1:Jro)-1.d0)
      ELSEWHERE
         Cjm(1:Jro) = Enu(1:Jro)*Dti
      END WHERE
      Sjm(1:Jro) = Nuj(1:Jro)
     $     *(1.d0 - Enu(1:Jro)*S1)/(CS*Nuj(1:Jro) + 1.d0)
c     --> b) complex separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2
         C1  = DCMP( Nuj(K1),Nuj(K1+1) )
         Cnu = DCMP( Enu(K1),Enu(K1+1) )
         C2 =  C1*(Cnu - S1)/(CS*C1 - 1.d0)
         Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)
         C2 = C1*(1.0d0 - Cnu*S1)/(CS*C1 + 1.d0)
         Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
      ENDDO
c     --> c) contribution of additional terms (K > 0) coming from 
c           approximation of the Planck function.
c           NCO > 1 in this case.
c           See Eqs.(C-te) and (S-te) in STB.
      Zn = -1
c      H2 = Zn*S1
      DO Jc = 1,NCO-1
c        ... Corrected by VR 04.14.2015 !!
         H2 = Zn*S1
         Ck = Dti/DBLE(Jc)/Pie ; Csk = CS*Ck
         Muc = Ck/DCMP( Csk,-1.d0 )
         Cjm(1:Jro) = Cjm(1:Jro) 
     $        + AIMAG( Muc*Nuj(1:Jro)/(Muc-Nuj(1:Jro)) )
     $        * a_k(Jc)*(H2 - Enu(1:Jro))
         Sjm(1:Jro) = Sjm(1:Jro) 
     $        + AIMAG( Muc*Nuj(1:Jro)/(Muc+Nuj(1:Jro)) )
     $        * a_k(Jc)*(1.0d0 - H2*Enu(1:Jro))
         DO K1 = Jro+1, Jro+2*Jco, 2
            C1 = DCMP( Nuj(K1),Nuj(K1+1) )
            C2 = ( CONJG(Muc)*C1/(CONJG(Muc)-C1)
     $           - Muc*C1/(Muc-C1) )*0.5d0
            V_i = AIMAG(C2)*Enu(K1) +  DBLE(C2)*Enu(K1+1)
            V_r =  DBLE(C2)*Enu(K1) - AIMAG(C2)*Enu(K1+1)
            Cjm(K1)   = Cjm(K1)   - a_k(Jc)*(AIMAG(C2)*H2 - V_i)
            Cjm(K1+1) = Cjm(K1+1) + a_k(Jc)*( DBLE(C2)*H2 - V_r)
            Sjm(K1)   = Sjm(K1)   - a_k(Jc)*(AIMAG(C2) - H2*V_i)
            Sjm(K1+1) = Sjm(K1+1) + a_k(Jc)*( DBLE(C2) - H2*V_r)
         ENDDO
         Zn = -Zn
      ENDDO      
      END SUBROUTINE C_S_FUNC_PLA

c **********************************************************************
      SUBROUTINE AjBj_recurrence
      DO J = S_AG+1,E_AG-1 
         Enu => Exp_NU_NL(:,J)            
         DO Ivs = 1,Kvs
            Ajp(Ivs,J,1:Jro,K) = Ajp(Ivs,J,1:Jro,K) 
     $           + Ajp(Ivs,J-1,1:Jro,K)*Enu(1:Jro)
         ENDDO
         DO K1 = Jro+1, Jro+2*Jco, 2
            Ajp(:,J,K1,K) = Ajp(:,J,K1,K) 
     $           + Ajp(:,J-1,K1,K)*Enu(K1) - Ajp(:,J-1,K1+1,K)*Enu(K1+1)
            Ajp(:,J,K1+1,K) = Ajp(:,J,K1+1,K) 
     $           + Ajp(:,J-1,K1,K)*Enu(K1+1) + Ajp(:,J-1,K1+1,K)*Enu(K1)
         ENDDO
      ENDDO  ! over sub-layers of adaptive grid (J)
      DO J = E_AG-2, S_AG, -1
         Enu => Exp_NU_NL(:,J)            
         DO Ivs = 1,Kvs
            Bjp(Ivs,J,1:Jro,K) = Bjp(Ivs,J,1:Jro,K) 
     $           + Bjp(Ivs,J+1,1:Jro,K)*Enu(1:Jro)
         ENDDO
         DO K1 = Jro+1, Jro+2*Jco, 2
            Bjp(:,J,K1,K) = Bjp(:,J,K1,K) 
     $           + Bjp(:,J+1,K1,K)*Enu(K1) - Bjp(:,J+1,K1+1,K)*Enu(K1+1)
            Bjp(:,J,K1+1,K) = Bjp(:,J,K1+1,K) 
     $           + Bjp(:,J+1,K1,K)*Enu(K1+1) + Bjp(:,J+1,K1+1,K)*Enu(K1)
         ENDDO
      ENDDO  ! over sub-layers of adaptive grid (J)
      END SUBROUTINE AjBj_recurrence

      END 
