!*********************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 20 Oct 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!*********************************************************************
SUBROUTINE INFO_INIT (*)
  ! Called by "gt_fileread.f"

  USE ABSORBERS, ONLY: N_ST_MOL, N_LBL_MOL, N_ESFT_MOL, MIX_ABS_START, &
                       GAS_INDEX, N_MIX_MOL, N_XSEC_MOL, N_LBL_PROF
  USE INFO,      ONLY: TG_INFO, PAR_INFO, AER_INFO, CLD_INFO, &
                       PARS_INFORMATION, TRACE_GAS_INFORMATION
  USE PHOTOCHEM, ONLY: DO_PHOTOCHEM
  USE RTM_PARS,  ONLY: RTM_STATE, RTM_ST
  USE TOOLS,     ONLY: UPPER_LOWER_CASE
  USE X_SECTS,   ONLY: ABS_COEFF_NTEMP

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'INFO_INIT'
  CHARACTER (LEN = :), ALLOCATABLE :: M_TYPE
  TYPE Linked_List_Entry
     CHARACTER (LEN = :), ALLOCATABLE :: NAME, LONG_NAME, VSI, M_TYPE
     INTEGER IDC, IdxHT
     TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
  END TYPE Linked_List_Entry

  TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL(), &
       FIRST_ST, ELEM_ST => NULL() 

  INTEGER  II, KK, K_MAX, N_RTM, I_P
  CHARACTER (LEN = :), ALLOCATABLE :: NAME, LONG_NAME, VSI
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 3, 'I')

! Trace gas weighting functions:
  CALL ADD_ENTRY (KK, 'bro', IDC = 5, M_TYPE = 'xsec') ! 1
  CALL ADD_ENTRY (KK, 'no3', IDC = 6, M_TYPE = 'xsec') ! 2
  CALL ADD_ENTRY (KK, 'oclo', IDC = 8, M_TYPE = 'xsec') ! 3
  CALL ADD_ENTRY (KK, 'o4', M_TYPE = 'xsec')         ! 4
  CALL ADD_ENTRY (KK, 'o3',   IDC = 1,  IdxHT = 3,  M_TYPE = 'mixed') ! 5
  CALL ADD_ENTRY (KK, 'no2',  IDC = 2,  IdxHT = 10, M_TYPE = 'mixed') ! 6
  CALL ADD_ENTRY (KK, 'so2',  IDC = 3,  IdxHT = 9,  M_TYPE = 'mixed') ! 7
  CALL ADD_ENTRY (KK, 'clo',  IDC = 4,  IdxHT = 18, M_TYPE = 'mixed') ! 8
  CALL ADD_ENTRY (KK, 'hcho', IDC = 7,  IdxHT = 20, M_TYPE = 'mixed') ! 9
  CALL ADD_ENTRY (KK, 'o2',   IDC = 6,  IdxHT = 7,  M_TYPE = 'line') ! 10
  CALL ADD_ENTRY (KK, 'h2o',  IDC = 1,  IdxHT = 1,  M_TYPE = 'line') ! 11
  CALL ADD_ENTRY (KK, 'co2',  IDC = 2,  IdxHT = 2,  M_TYPE = 'line') ! 12
  CALL ADD_ENTRY (KK, 'n2o',  IDC = 3,  IdxHT = 4,  M_TYPE = 'line') ! 12
  CALL ADD_ENTRY (KK, 'co',   IDC = 4,  IdxHT = 5,  M_TYPE = 'line') ! 14
  CALL ADD_ENTRY (KK, 'ch4',  IDC = 5,  IdxHT = 6,  M_TYPE = 'line') ! 15
  CALL ADD_ENTRY (KK, 'no',   IDC = 7,  IdxHT = 8,  M_TYPE = 'line') ! 16
  CALL ADD_ENTRY (KK, 'nh3',  IDC = 8,  IdxHT = 11, M_TYPE = 'line') ! 17
  CALL ADD_ENTRY (KK, 'hno3', IDC = 9,  IdxHT = 12, M_TYPE = 'line') ! 18
  CALL ADD_ENTRY (KK, 'oh',   IDC = 10, IdxHT = 13, M_TYPE = 'line') ! 19
  CALL ADD_ENTRY (KK, 'hf',   IDC = 11, IdxHT = 14, M_TYPE = 'line') ! 20
  CALL ADD_ENTRY (KK, 'hcl',  IDC = 12, IdxHT = 15, M_TYPE = 'line') ! 21
  CALL ADD_ENTRY (KK, 'hbr',  IDC = 13, IdxHT = 16, M_TYPE = 'line') ! 22
  CALL ADD_ENTRY (KK, 'hi',   IDC = 14, IdxHT = 17, M_TYPE = 'line') ! 23
  CALL ADD_ENTRY (KK, 'na', M_TYPE = 'atom') ! 24
  !  Mg+
  CALL ADD_ENTRY (KK, 'mgi', M_TYPE = 'atom') ! 25
  !  Mg
  CALL ADD_ENTRY (KK, 'mga', M_TYPE = 'atom') ! 26

  CALL FINALIZE_TG (KK, TG_INFO)

  N_ST_MOL = SIZE(TG_INFO)

!  WRITE(*,*) TG_INFO(:)%NAME
!  WRITE(*,*) '      --------------------  '
!  WRITE(*,*) TG_INFO(:)%M_TYPE
!  PAUSE

  DO II = 1,N_ST_MOL 
     IF( TRIM( TG_INFO(II)%M_TYPE )=='mixed') THEN
        MIX_ABS_START = II ; EXIT
     ENDIF
  ENDDO
  N_MIX_MOL  = COUNT( TG_INFO(:)%M_TYPE=='mixed' )
  N_XSEC_MOL = COUNT( TG_INFO(:)%M_TYPE=='xsec' ) + N_MIX_MOL
  N_LBL_PROF = COUNT( TG_INFO(:)%M_TYPE=='line' )
  N_LBL_MOL  = N_MIX_MOL + N_LBL_PROF
  N_ESFT_MOL = N_LBL_MOL

  IF(ALLOCATED(GAS_INDEX)) DEALLOCATE (GAS_INDEX, DO_PHOTOCHEM, &
       ABS_COEFF_NTEMP)
  ALLOCATE(GAS_INDEX(N_ST_MOL), DO_PHOTOCHEM(N_ST_MOL), &
       ABS_COEFF_NTEMP(N_XSEC_MOL + 3) )
  
  DO II = 1, N_ST_MOL
     TG_INFO(II)%NameUC = TRIM( TG_INFO(II)%NAME )
     CALL UPPER_LOWER_CASE (TG_INFO(II)%NameUC,'L2U')
  ENDDO
  
  TG_INFO(:)%FM_FLG = .FALSE.
  TG_INFO(:)%READ_FLG = .FALSE.
  TG_INFO(:)%WF_FLG = .FALSE.
  DO II = 1, N_ST_MOL
     TG_INFO(II)%LONG_NAME = 'Weighting functions of '//TG_INFO(II)%NAME
  END DO
  
  !     ... Aerosol weighting functions:
  NAME = 'aer-ext' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol extinction coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-sca' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol scattering coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-abs' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol absorption coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-erw' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol extinction at reference wavelength'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)

  NAME = 'aer-dens' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particles number density'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-modr_smod' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution mode radius'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-var_smod' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution width'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-modr_f' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution mode radius (fine mode)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-modr_c' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution mode radius (coarse mode)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-var_f' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution width (fine mode)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-var_c' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the aerosol particle size '// &
       'distribution width (coarse mode)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'aer-cm_frac' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the coarse mode fraction'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  CALL FINALIZE (KK, AER_INFO)
  
  !     ... Cloud weighting functions:
  NAME = 'cld-sca_nomix' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud scattering coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-sca_water' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud scattering '// &
       'coefficient (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-sca_ice' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud scattering '// &
       'coefficient (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-abs_nomix' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud absorbtion coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-abs_water' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud absorbtion '// &
       'coefficient (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-abs_ice' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud absorbtion '// &
       'coefficient (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-lwc_nomix' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud liquid water content'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-lwc_water' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud liquid water '// &
       'content (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-lwc_ice' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud liquid '// &
       'water content (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-pnc_nomix' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle '// &
       'number concentration'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-pnc_water' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle '// &
       'number concentration (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-pnc_ice' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle '// &
       'number concentration (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-ref_nomix' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle effective radius'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-ref_water' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle '// &
       'effective radius (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-ref_ice' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the cloud particle '// &
       'effective radius (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-soot' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the soot absorber within the cloud'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-cth' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the cloud top height'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-cbh' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the cloud botton height'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-tau_nomix' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the cloud optical thickness'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-tau_water' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the cloud optical '// &
       'thickness (water)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'cld-tau_ice' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the cloud optical '// &
       'thickness (ice)'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  CALL FINALIZE (KK, CLD_INFO)
  
  !     ... Weighting functions for other atmospheric parameters:
  NAME = 'ray' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the Rayleigh '// &
       'scattering coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'albedo' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the surface albedo'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)

! *********  Parameters of PROSAIL BRDF model ************************
       NAME = 'prosail_Cbr' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the brown pigment content'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Cm' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the leaf mass area'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Cab' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the chlorophyll content'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_LAI' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the leaf area index'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Stc' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the structure coefficient'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Car' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the carotenoid content'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Cw' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the equivalent water thickness'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Ala' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the average leaf angle'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
       NAME = 'prosail_Pso' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the surface moisture'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
! ********* End of PROSAIL BRDF model ********************************
  
  NAME = 'anps' ;  VSI = 'S'
  LONG_NAME = 'Average number of the photon scattering events'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'press' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the pressure'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'temp' ;  VSI = 'V'
  LONG_NAME = 'Weighting functions of the temperature'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  NAME = 'elev' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the surface elevation'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
! ********* Land fluorescence weighting functions ********************
  NAME = 'flu_sfl' ;  VSI = 'S'
  LONG_NAME = 'Scaling factor for the fluorescence spectra '
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)

  NAME = 'flu_amp-1' ;  VSI = 'S'
  LONG_NAME = 'Amplitude of the first Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  NAME = 'flu_wlm-1' ;  VSI = 'S'
  LONG_NAME = 'Position of the first Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  NAME = 'flu_hwhm-1' ;  VSI = 'S'
  LONG_NAME = 'Half width at half maximum of the first Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)

  NAME = 'flu_amp-2' ;  VSI = 'S'
  LONG_NAME = 'Amplitude of the second Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  NAME = 'flu_wlm-2' ;  VSI = 'S'
  LONG_NAME = 'Position of the second Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  NAME = 'flu_hwhm-2' ;  VSI = 'S'
  LONG_NAME = 'Half width at half maximum of the second Gauss'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
! ********* End of land fluorescence weighting functions *************

  NAME = 'th' ;  VSI = 'S'
  LONG_NAME = 'Weighting functions of the tangent height'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI)
  
  CALL FINALIZE (KK, PAR_INFO)

  ! Parameters which do not have analytic weighting functions
  NAME = 'exa_s' ;  VSI = 'S'
  LONG_NAME = 'Scaling factor for stratospheric aerosol extinction'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI, NO_WF = .TRUE.)

  NAME = 'tps_col' ;  VSI = 'S'
  LONG_NAME = 'Tropospheric column'
  CALL ADD_ENTRY (KK, NAME, LONG_NAME, VSI, NO_WF = .TRUE.)
  
  CALL FINALIZE_ST (RTM_ST)

  CALL ST_TRACK (R_NAME, 3, 'O')
  RETURN

CONTAINS
!*********************************************************************
  SUBROUTINE ADD_ENTRY (KK, NAME, LONG_NAME, VSI, PAR_LONG_N,  M_TYPE, &
       IDC, IdxHT, NO_WF)
    
    INTEGER  KK, JJ
    INTEGER, OPTIONAL :: IDC, IdxHT
    LOGICAL  NO_WF_FLG
    LOGICAL, OPTIONAL :: NO_WF
    CHARACTER (LEN = *) NAME
    CHARACTER (LEN = *), OPTIONAL :: VSI, LONG_NAME, PAR_LONG_N, M_TYPE
    
    IF(PRESENT(NO_WF)) THEN
       NO_WF_FLG = NO_WF
    ELSE
       NO_WF_FLG = .FALSE.
    END IF

! Parameters
    IF (.NOT.ASSOCIATED(ELEM_ST)) THEN
       ALLOCATE(FIRST_ST)
       ELEM_ST => FIRST_ST
       N_RTM = 1
    ELSE
       ALLOCATE(ELEM_ST%NEXT_ELEM)
       ELEM_ST => ELEM_ST%NEXT_ELEM
       N_RTM = N_RTM + 1
    ENDIF
    
    ELEM_ST%NAME = NAME
    IF(PRESENT(VSI)) THEN
       ELEM_ST%VSI = VSI 
    ELSE
       ELEM_ST%VSI = 'V'
    END IF
    IF(PRESENT(PAR_LONG_N)) THEN
       ELEM_ST%LONG_NAME = PAR_LONG_N
    ELSE
       ELEM_ST%LONG_NAME = NAME
    END IF

    IF(NO_WF_FLG) RETURN

    ! Weighting functions
    IF (.NOT.ASSOCIATED(ELEMENT)) THEN
       ALLOCATE(FIRST_ELEM)
       ELEMENT => FIRST_ELEM
       KK = 1
    ELSE
       ALLOCATE(ELEMENT%NEXT_ELEM)
       ELEMENT => ELEMENT%NEXT_ELEM
       KK = KK + 1
    ENDIF
    
    ELEMENT%NAME = NAME
    IF(PRESENT(LONG_NAME)) THEN
       ELEMENT%LONG_NAME = LONG_NAME
    ELSE
       ELEMENT%LONG_NAME = 'Weighting functions of '//NAME
    END IF
    IF(PRESENT(VSI)) ELEMENT%VSI = VSI 
    IF(PRESENT(M_TYPE)) ELEMENT%M_TYPE = M_TYPE 
    IF(PRESENT(IDC)) THEN
       ELEMENT%IDC = IDC 
    ELSE
       ELEMENT%IDC = 0 
    END IF
    ! Index of the molecule in the HITRAN data base
    IF(PRESENT(IdxHT)) THEN
       ELEMENT%IdxHT = IdxHT
    ELSE
       ELEMENT%IdxHT = 0 
    END IF

    RETURN
  END SUBROUTINE ADD_ENTRY
!*********************************************************************
  SUBROUTINE FINALIZE (KK, STRUCT)
    
    INTEGER  KK, II
    TYPE (PARS_INFORMATION), ALLOCATABLE :: STRUCT(:) 
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    
    NULLIFY(ELEMENT%NEXT_ELEM)
    IF(ALLOCATED(STRUCT)) DEALLOCATE(STRUCT)
    ALLOCATE (STRUCT(KK))
    
    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       II = II + 1
       STRUCT(II)%NAME = ELEMENT%NAME
       STRUCT(II)%LONG_NAME = ELEMENT%LONG_NAME
       STRUCT(II)%VSI = ELEMENT%VSI
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT%NAME, ELEMENT%LONG_NAME, ELEMENT%VSI)
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO
    
    STRUCT(:)%WF_FLG = .FALSE.
    
    RETURN
  END SUBROUTINE FINALIZE
!*********************************************************************
  SUBROUTINE FINALIZE_TG (KK, STRUCT)
    
    INTEGER  KK, II
    TYPE (TRACE_GAS_INFORMATION), ALLOCATABLE :: STRUCT(:) 
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    
    NULLIFY(ELEMENT%NEXT_ELEM)
    IF(ALLOCATED(STRUCT)) DEALLOCATE(STRUCT)
    ALLOCATE (STRUCT(KK))
    
    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       II = II + 1
       STRUCT(II)%NAME = ELEMENT%NAME
       STRUCT(II)%LONG_NAME = ELEMENT%LONG_NAME
       STRUCT(II)%M_TYPE = ELEMENT%M_TYPE
       STRUCT(II)%IDC = ELEMENT%IDC
       STRUCT(II)%IdxHT = ELEMENT%IdxHT
       Select Case (STRUCT(II)%M_TYPE)
       Case("line", "mixed") 
          STRUCT(II)%IsoMin = 1
          STRUCT(II)%IsoMax = 99
       End Select
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT%NAME, ELEMENT%LONG_NAME, ELEMENT%M_TYPE)
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO

    STRUCT(:)%WF_FLG = .FALSE.
    
    RETURN
  END SUBROUTINE FINALIZE_TG
!*********************************************************************
  SUBROUTINE FINALIZE_ST (STRUCT)
    
    INTEGER  II
    TYPE (RTM_STATE), ALLOCATABLE :: STRUCT(:) 
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    
    NULLIFY(ELEM_ST%NEXT_ELEM)
    ALLOCATE (STRUCT(N_RTM))
    
    II = 0
    ELEM_ST => FIRST_ST
    DO WHILE (ASSOCIATED(ELEM_ST))
       II = II + 1
       STRUCT(II)%NAME = ELEM_ST%NAME
       STRUCT(II)%LONG_NAME = ELEM_ST%LONG_NAME
       STRUCT(II)%VS = ELEM_ST%VSI
       NEXT => ELEM_ST%NEXT_ELEM
       DEALLOCATE (ELEM_ST%NAME, ELEM_ST%LONG_NAME)
       DEALLOCATE (ELEM_ST) ; NULLIFY (ELEM_ST)
       ELEM_ST => NEXT
    END DO
    
    RETURN
  END SUBROUTINE FINALIZE_ST
!*********************************************************************
END SUBROUTINE INFO_INIT
!*********************************************************************
