!****************************************************************************
!
!	SCIATRAN - Version 4.2 - Last modification: 08 Jan 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
Subroutine Init_IO (Work_Dir_Inp, LOG_SF, *)
  Use Data_Ret, Only: RET_FLAG
  Use OutInf,   Only: OutD, Work_Dir, TRC_FLG, EU, LUNIT, N_ESO, N_ZSO, FUNIT, &
                      LOG_LEVEL, N_RSO, N_NSO, L_U, N_SSO, N_DSO, AUNIT, &
                      SurUNIT, OFFSET, L_S, L_E, L_ER, DO_LOGFILE, N_XSO, N_TSO,&
                      OutMode, WrtModRes, C_UNIT, Dir_Out, DelayOut, OneOutFile, &
                      WrtRetPrf, WrtRetFit, LUF
  Use ST_VARS,  Only: MPI_FLG
  Use Tools,    Only: Read_Input, Set_Path, Upper_Lower_Case, OpenOut
  Use Types,    Only: DYN_CHAR
  Use VersNum,  Only: VERSION
  Character (Len=*) Work_Dir_Inp
  Character (LEN=2) LOG_SF
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'Init_IO'
  CHARACTER (LEN=2)  LOG_LEV(2), LOG_TMP
  CHARACTER (LEN=80) TXT
  CHARACTER (LEN=:), Allocatable  :: Str_Inp
  Integer  LL, CU, I, II
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  !==================================================================

  LL = LEN_TRIM(Work_Dir_inp)
  IF(Work_Dir_inp(LL:LL) == '/') THEN
     Work_Dir =  TRIM(Work_Dir_inp)
  ELSE
     Work_Dir =  TRIM(Work_Dir_inp)//'/'
  END IF
  
  ! Error output:
  OPEN(NewUnit = EU, FILE=Work_Dir//'errors.log', STATUS='UNKNOWN')
  WRITE(EU,*) '        ----------------------------------------------'
  WRITE(EU,*) '         SCIATRAN '//TRIM(VERSION)//' warning and error messages:'
  WRITE(EU,*) '        ----------------------------------------------'
  
  ! Check ulimit
  IF(.NOT.MPI_FLG) THEN
     CALL EXECUTE_COMMAND_LINE ('ulimit -s > '//Work_Dir//'chk_stack_limit')
     Open(NewUnit = CU, file = Work_Dir//'chk_stack_limit')
     Read(CU,*) TXT
     Close(CU)
     
     If(Trim(TXT) /= 'unlimited') Then
        call Write_Message(R_Name, 'Warning', 'Stack size is limited')
        Write(EU,*) 'If SCIATRAN crashes with "Segmentation fault" message try to'
        Write(EU,*) 'unlimit the stack size by typing "ulimit -s unlimited" in the '
        Write(EU,*) 'command line before the execution of SCIATRAN'
     End If
     CALL EXECUTE_COMMAND_LINE('rm '//Work_Dir//'chk_stack_limit')
  END IF
  
  ! Display output:
  LUNIT = 6
  Call Write_Err_Log(*880) ! Write all delayed error messages.
  CALL READ_INPUT_FILES (.FALSE., *880)
  
  TRC_FLG(3) = .FALSE.
  CALL Read_Input ('Screen output', READ_ARR, *880, OutputType = 'delayed')
  READ(READ_ARR(1)%S, *) LOG_SF, LOG_LEV
  
  CALL UPPER_LOWER_CASE(LOG_SF, 'L2U')
  IF(TRIM(LOG_SF) == 'F') THEN
     ! Display output is redericter to LogFile.out
     LUNIT = 66
     OPEN(LUNIT, FILE = Work_Dir//'LogFile.out')
  END IF
  
  LOG_LEVEL(:) = N_SSO ;   OFFSET = - 3
  DO I = 1, 2
     LOG_TMP = LOG_LEV(I)
     SELECT CASE(LOG_TMP(2:2))
     CASE('T')
        TRC_FLG(I) = .TRUE.
     CASE DEFAULT
        TRC_FLG(I) = .FALSE.
     END SELECT
     SELECT CASE(LOG_TMP(1:1))
     CASE('N')
        LOG_LEVEL(I) = N_NSO
     CASE('R')
        LOG_LEVEL(I) = N_RSO
     CASE('S')
        LOG_LEVEL(I) = N_SSO
     CASE('E')
        LOG_LEVEL(I) = N_ESO
     CASE('D')
        LOG_LEVEL(I) = N_DSO
     CASE('X')
        LOG_LEVEL(I) = N_XSO
     CASE('Z')
        LOG_LEVEL(I) = N_ZSO
     CASE('T')
        LOG_LEVEL(I) = N_TSO ! compatibility mode
     CASE DEFAULT
        LOG_LEVEL(I) = N_SSO
     END SELECT
  END DO
  TRC_FLG(3) = TRC_FLG(1).OR.TRC_FLG(2)
  
  ! Output to the logfile:
  ! Aerosol logfile:
  AUNIT = 16
  ! Temporary unit number:
  C_UNIT  = 22
  ! Input settings unit (control_brdf.inp):
  SurUNIT = 34
  ! Array of output units:
  STR_INP = 'Output directory'
  CALL Read_Input (STR_INP, READ_ARR, *880, OutputType = 'delayed')
  DIR_OUT = SET_PATH(READ_ARR(1)%S)//'/'
  ! DIR_OUT = Work_Dir//OutSubDir
  If(.Not.Allocated(OutD)) Then
     Allocate(OutD(27))
     Do II = 1, Size(OutD)
        OutD(II)%Recl = 1000
     End Do

     OutD(1)%Name = 'air_density.dat'
     OutD(2)%Name = 'ret_results.dat'
     OutD(3)%Name = 'av_test.dat'; OutD(3)%RecL = 10000
     OutD(4)%Name = 'covar_matr.dat'; OutD(4)%RecL = 10000
     OutD(5)%Name = 'precision.dat' 
     OutD(6)%Name = 'ret_units.dat'
     OutD(7)%Name = 'SCE_ABSORBER.OUT' 
     OutD(8)%Name = 'SCE_AEROSOL.OUT' 
     OutD(9)%Name = 'SCE_INP-PARS.OUT' ; OutD(9)%RecL = 10000 
     OutD(10)%Name = 'SCE_RAYLEIGH.OUT' 
     OutD(11)%Name = 'SCE_SUMMARY.OUT' 
     OutD(12)%Name = 'SCE_VHL.OUT'
     OutD(13)%Name = 'output_map.inf'
     OutD(14)%Name = 'intensity.dat'
     OutD(15)%Name = 'intensity_noring.dat'
     OutD(16)%Name = 'ring.dat'
     OutD(17)%Name = 'output_map.inf'
     OutD(18)%Name = 'irradiance.dat'
     OutD(19)%Name = 'RMS_ALL.dat'
     OutD(20)%Name = 'av1.dat'; OutD(20)%RecL = 10000
     OutD(21)%Name = 'AOD_ALL.dat'
     OutD(22)%Name = 'RLC_ISO.dat'
     OutD(23)%Name = 'RLC_VOL.dat'
     OutD(24)%Name = 'RLC_GEO.dat'
     OutD(25)%Name = 'EXT_RAYLEIGH.OUT'
     OutD(26)%Name = 'ret_errors.dat'
     OutD(27)%Name = 'Kernels_coef.dat'
  End If
     
  If(OutMode == 'HPC') Then
     DelayOut = .True.
     OneOutFile = .True.
  Else
     DelayOut = .False.
     OneOutFile = .False.
  End If
  
  ! Define if we write out intensities and weighting functions from the
  ! forward model
!  If(OutMode == 'HPC') Then
!     If(RET_FLAG) Then
!        WrtModRes = .False.
!     Else
!        WrtModRes = .True.
!     End If
!     WrtRetPrf = .False.
!     WrtRetPrf = .False.
!     WrtRetFit = .False.
!  Else
!     IF(LOG_LEVEL(1) >= N_SSO .OR. LOG_LEVEL(2) >= N_ESO &
!          .OR. TRC_FLG(1) .OR. .NOT.RET_FLAG ) Then
!        WrtModRes = .True.
!     Else 
!        WrtModRes = .False.
!     End If
!     WrtRetPrf = .True.
!     WrtRetFit = .True.
!  End If
  
  Call OpenOut ('SCE_INP-PARS.OUT', FUNIT)
  L_U(1) = FUNIT;  L_U(2) = LUNIT
  LUF(1) = .False.; LUF(2) = .True.
  
  L_S = 1; L_E = 1; L_ER = 1
  IF (LOG_LEVEL(1) >= N_RSO) L_ER = 2
  IF (LOG_LEVEL(1) >= N_SSO) THEN
     L_E = 2
     DO_LOGFILE = .TRUE.
  ELSE
     DO_LOGFILE = .FALSE.
  END IF
  
  Call Write_Out_Log      
 
  Return
  
880 WRITE(EU, *) ' ---> Called by '//R_NAME
  RETURN 1
!***************************************************************************
End Subroutine Init_IO
!***************************************************************************
