!********************************************************************C
!
!	SCIATRAN - Version 4.2 - Last modification: 10 Oct 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!********************************************************************C
SUBROUTINE INIT_NODES ()
  ! Called by "rtm_init.f"

  USE NODES
  USE ST_VARS,  ONLY: Pie
  USE TYPES,    ONLY: KND
  !------------------------------------------------------------------
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'NODES'
  INTEGER             I, IDX(1), NVA1, NN, NVA_INP, II, NAA_INP, NAA, JJ, KK
  LOGICAL             ALLOCATE_ALL
  LOGICAL, ALLOCATABLE :: PLUS_FLAG(:) 
  REAL (KND), ALLOCATABLE :: X(:), Y(:), Z(:), W(:), PHI_NODE_1(:), GRD_V(:),&
       GRD_A(:), WGT_V(:), WGT_A(:), V_NODE(:), WGT(:)
  ! ==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  NVA_INP = 34
  NN = ((NVA_INP + 1)/2 + 1)/2
  NVA1 = 2*(2*NN - 1) - 1
  Allocate(GRD_V(NVA1), WGT_V(NVA1))
  Call GeomProgres (NN, 0.0D0, 60.0D0, 1.2D0, GRD_V(1:NN), .TRUE., .False.)
  Call GeomProgres (NN, 60.0D0, 90.0D0, 2D0, GRD_V(NN:2*NN - 1), .False., .TRUE.)
  Call GeomProgres (NN, 90.0D0, 120.0D0, 2D0, GRD_V(2*NN - 1:3*NN - 2), .TRUE., .False.)
  Call GeomProgres (NN, 120.0D0, 180.0D0, 1.2D0, GRD_V(3*NN - 2:4*NN - 3), .False., .TRUE.)

  GRD_V = Cos(GRD_V/1.8D2*Pie)
  WGT_V(1) = GRD_V(2) - GRD_V(1)
  DO II = 2, NVA1 - 1
     WGT_V(II) = GRD_V(II + 1) - GRD_V(II - 1)
  End DO
  WGT_V(NVA1) = GRD_V(NVA1) - GRD_V(NVA1 - 1)
  WGT_V(:) = Abs(WGT_V(:))/2.0D0/2.0D0!/1.8D2

!  write(6,*) GRD_V
!  write(6,*) SUM(WGT_V)
!  pause

  NAA_INP = 34
  NN = ((NAA_INP + 1)/2 + 1)/2
  NAA = 2*(2*NN - 1) - 1
  Allocate(GRD_A(NAA), WGT_A(NAA))

  Call GeomProgres (NN, 0.0D0, 45.0D0, 1.5D0, GRD_A(1:NN), .TRUE., .False.)
  Call GeomProgres (NN, 45.0D0, 90.0D0, 1.5D0, GRD_A(NN:2*NN - 1), .False., .TRUE.)
  Call GeomProgres (NN, 90.0D0, 135.0D0, 1.5D0, GRD_A(2*NN - 1:3*NN - 2), .TRUE., .False.)
  Call GeomProgres (NN, 135.0D0, 180.0D0, 1.5D0, GRD_A(3*NN - 2:4*NN - 3), .False., .TRUE.)

  WGT_A(1) = GRD_A(2) - GRD_A(1)
  DO II = 2, NAA - 1
     WGT_A(II) = GRD_A(II + 1) - GRD_A(II - 1)
  End DO
  WGT_A(NAA) = GRD_A(NAA) - GRD_A(NAA - 1)
  WGT_A(:) = WGT_A(:)/2.0D0/1.8D2

  
!  write(6,*) SUM(WGT_A(:))
!  write(6,*) GRD_A
! write(6,*) 2*(2*NN - 1) - 1
!stop

  ALLOCATE_ALL = .TRUE.     ! Check which arrays need to be allocated
  IF(ALLOCATED(MJU_NODE)) THEN
     IDX(1:1) = SHAPE(MJU_NODE)
     IF(IDX(1) /= N_NODE) THEN  ! Check if the number of nodes is same as before
        DEALLOCATE(PHI_NODE, MJU_NODE, WEIGHTS, SIN_NODE)
     ELSE
        ALLOCATE_ALL = .FALSE.
     END IF
  END IF

  N_NODE = NVA1*NAA
 
  IF(ALLOCATE_ALL) THEN
     ALLOCATE(PHI_NODE(N_NODE), MJU_NODE(N_NODE), V_NODE(N_NODE),  & ! A
          WEIGHTS(N_NODE), SIN_NODE(N_NODE), PLUS_FLAG(N_NODE), & ! A
          PHI_NODE_1(N_NODE), WGT(N_NODE)) ! A
  END IF

  KK = 0
  Do II = 1, NVA1
     Do JJ = 1, NAA
        KK = KK + 1
!        V_NODE(KK) = Cos(GRD_V(II)/1.8D2*Pie)
        V_NODE(KK) = GRD_V(II)
        PHI_NODE_1(KK) = GRD_A(JJ)/1.8D2*Pie
        WGT(KK) = WGT_V(II)*WGT_A(JJ)
     End Do
  End Do
  WGT(:) = WGT(:)*4.*Pie
  
  PLUS_FLAG = .FALSE.
  WHERE(V_NODE >= 0.0D0)  PLUS_FLAG = .TRUE.
  N_PLUS = COUNT(PLUS_FLAG)
  N_MINUS = N_NODE - N_PLUS

  MJU_NODE(1:N_PLUS) = PACK(V_NODE,PLUS_FLAG)

  MJU_NODE(N_PLUS + 1:N_NODE) = PACK(V_NODE,.NOT.PLUS_FLAG)
  PHI_NODE(1:N_PLUS) = PACK(PHI_NODE_1,PLUS_FLAG) 
  PHI_NODE(N_PLUS + 1:N_NODE) = PACK(PHI_NODE_1,.NOT.PLUS_FLAG)
  WEIGHTS(1:N_PLUS) = PACK(WGT,PLUS_FLAG)
  WEIGHTS(N_PLUS + 1:N_NODE) = PACK(WGT,.NOT.PLUS_FLAG)

  WHERE( PHI_NODE(:) < 0.d0 ) PHI_NODE(:) = PHI_NODE(:) + 2.d0*Pie
  
  SIN_NODE = DSQRT(1.0D0 - MJU_NODE**2)
  NORM_DN = DOT_PRODUCT(WEIGHTS(N_PLUS + 1:N_NODE),&
       MJU_NODE(N_PLUS + 1:N_NODE))
  
  MJU_MINUS => MJU_NODE(N_PLUS + 1:N_NODE)
  MJU_PLUS => MJU_NODE(1:N_PLUS)
  PHI_MINUS => PHI_NODE(N_PLUS + 1:N_NODE)
  W_MINUS => WEIGHTS(N_PLUS + 1:N_NODE)
  W_PLUS => WEIGHTS(1:N_PLUS)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN 
END SUBROUTINE INIT_NODES
!**************************************************************************

    
