!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 19 Nov 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine Integrate_Init (RAD, ALBEDO, DEP_CF, LOA, PCC, ADI, WFPS, WF_OUT, &
     JPOS, ESFT_WEIGHT, INDEX_TH, POS, OSM, *, FLU_Int, TE)
  ! Called by "spher_rad.f90",  "st_dom_tau.f"

  USE BRDF,       ONLY: BRDF_PARAM
  USE BRDF_SUBRS, ONLY: BRDF_ACT
  USE CLD_INP,    ONLY: Nphr, Use_Ydb_SCM, Ydb_ang
  USE CLOUDS,     ONLY: DO_CLOUD, SCAT_ANG
  USE GRIDS,      ONLY: NLAY=>NLAYERS, ZZZ, Z_FINE, NSGP=>NSOL, SZA_EXT, &
                        ALT_EXT, Z_INV, X_INV, INTERP_IDX, NL_FINE
  USE NODES,      ONLY: N_NODE, N_PLUS, N_MINUS, WEIGHTS, W_PLUS, W_MINUS, &
                        MJU_NODE, PHI_NODE, MJU_MINUS, PHI_MINUS, MJU_PLUS, &
                        NORM_DN
  USE INFO,       ONLY: WF_DB
  USE INTERN_DIMS,ONLY: Nvs, Ns_SFN, N_PHI_MS, NL_AER, Nm_vs, NLG_ALL_A, &
                        NL_CLD, NLG_ALL_C, Nm_ice
  USE LOG_PARS,   ONLY: DIR_OUT, LOG_LEVEL, N_ESO, LUNIT, N_DSO, EU
  USE SPHER_PARS, ONLY: N_ITER, DO_SF_OUT, NFA=>N_FOV_ANG, N_LOS_ANG, DELTA_TH,&
                        FOV_WEIGHTS, PE_Mode
  USE RTM_OUTPUT, ONLY: DIR_RAD, ACTFLUX, FLUX_UP, FLUX_DN, DIF_FLUX_UP, &
                        DIF_FLUX_DN, SPEC_OUT, STOK_OUT, WF_TG_FLAG, DIR_FLUX
  USE ST_VARS,    ONLY: PIE, RAD_TO_DEG, RTM_MODE, RTM_TYPE, INT_MS_FLAG, &
                        BRDF_FLAG, GeoD_ITR, GeoD_OUT, GeoD_PRT, &
                        GeoD_PS, WF_MS_FLAG, ND_ANG, USE_PSM, &
                        DISCR_M, E_STEP, DO_AEROSOL, Sel_los, Cr_Sel_los
  USE TOOLS,      ONLY: INQ_WF
  USE TYPES,      ONLY: KND, PCC_TYPE, OSM_TYPE, AuxDataInterp, RES_DATA,  &
                        Geometry_Data, LOA_TYPE, WFS_PS, TE_TYPE
  Use VersNum,    Only: VERSION

  INTEGER     JPOS
  LOGICAL     INDEX_TH(N_LOS_ANG)
  REAL (KND), ALLOCATABLE :: RAD(:,:,:,:), WF_OUT(:,:)
  REAL (KND) DEP_CF(3)
  REAL (KND)  ALBEDO, FLU_Int, ESFT_WEIGHT, POS
  TYPE (PCC_type) :: PCC    
  TYPE (OSM_type) :: OSM    ! Original scattering matrices
  TYPE (TE_type)  :: TE     ! Thermal emission
  TYPE (WFS_PS), ALLOCATABLE :: WFPS(:) ! Pseudo-spherical WFs
  
  ! Local Variables
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'Integrate_Init'
  LOGICAL       LAST_INT, DO_WF, ALT_FLAG, DO_TH_WF
  LOGICAL, DIMENSION (MAX(N_NODE, N_LOS_ANG*NFA)) :: INDEX_TH_LOC
  Logical, Allocatable :: ErrFlg(:,:)
  CHARACTER (LEN = 3)  CP
  CHARACTER (LEN=:), ALLOCATABLE :: ADD_LAB 
  INTEGER    N_OUT, II, I_N, CU, IS, IV, NVA, NSol, NPhi, J, SPHER_IT, IFOV, IP,&
       IA, KK, NAlts, Ks, N_ANG, N_COF, Jv, Jmc, JJ
  REAL (KND), ALLOCATABLE :: CSZA(:,:), SolExp(:,:), DIFF_REFL(:,:,:,:), &
       DIR_REFL(:,:,:,:), ARR(:), FLUP(:,:), BC_S1(:), LBC(:)
  REAL (KND), ALLOCATABLE::  Rx(:,:), A_1(:,:,:,:), A_2(:,:,:,:), A_3(:,:,:,:), &
       A_4(:,:,:,:)
! Real (Knd), Allocatable :: AzA(:), WAz(:)
  REAL (KND), POINTER::  P_ang(:)
  REAL (KND)  SD, SD2, SD3, A_MIN
  TYPE (AuxDataInterp), Target:: ADI
  TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
  TYPE (WFS_PS), ALLOCATABLE :: DRP(:)
  TYPE (RES_DATA), ALLOCATABLE :: RES(:,:), RES_P(:,:)
  TYPE (Geometry_Data), POINTER :: GeoD
  !==================================================================
  INTERFACE
     SUBROUTINE Integrate_Los (IA, IV, GeoD, DEP_CF, LOA, ADI, PCC, TE, &
          RES, ALT_FLAG, WFPS, DO_WF, JPOS, OSM, POS, FLU_Int, ErrFlg, &
          CSZA, SolExp)

       USE GRIDS,   ONLY: NLAYERS
       USE TYPES,   ONLY: KND, PCC_TYPE, OSM_TYPE, AuxDataInterp, &
                          WFS_PS, Geometry_Data, RES_DATA, LOA_TYPE, TE_TYPE
       INTEGER     JPOS, IA, IV
       LOGICAL     ALT_FLAG, DO_WF, ErrFlg 
       REAL (KND)  DEP_CF(3), POS, FLU_Int
       REAL (KND), ALLOCATABLE, OPTIONAL :: CSZA(:,:), SolExp(:,:)
       TYPE (PCC_type) :: PCC    
       TYPE (OSM_type) :: OSM    ! Original scattering matrices
       TYPE (LOA_type) :: LOA
       TYPE (TE_type)  :: TE  
       TYPE (AuxDataInterp), Target:: ADI
       TYPE (RES_DATA) RES
       TYPE (Geometry_Data), TARGET :: GeoD
       TYPE (WFS_PS), ALLOCATABLE :: WFPS(:) ! Pseudo-spherical WFs
     END SUBROUTINE Integrate_Los

     SUBROUTINE Integrate_Occ (IA, IV, GeoD, AL, ADI, PCC, RES, DO_WF)
       USE TYPES,        ONLY: KND, PCC_TYPE, Geometry_Data, AuxDataInterp, &
            RES_DATA
       USE GRIDS,        ONLY: NLAYERS
       INTEGER     IA, IV
       LOGICAL     DO_WF
       REAL (KND)  AL(NLAYERS)
       TYPE (PCC_type) :: PCC    
       TYPE (AuxDataInterp) ADI
       TYPE (Geometry_Data) GeoD
       TYPE (RES_DATA) RES
     END SUBROUTINE Integrate_Occ

     SUBROUTINE BIAKIMA_Y (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
       USE GRIDS,       ONLY: NSOL
       USE TYPES,       ONLY: KND
       REAL (KND)  X_RAD(:,:,:), Y_ALL(:,:,:) 
       REAL (KND)  Y_LOC(NSOL + 4)
       REAL (KND)  X_NEW, X_AX
     END SUBROUTINE BIAKIMA_Y

     SUBROUTINE BIAKIMA_Y_V2 (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
       USE GRIDS,       ONLY: NSOL
       USE TYPES,       ONLY: KND
       REAL (KND)  X_RAD(:,:,:,:), Y_ALL(:,:,:,:) 
       REAL (KND)  Y_LOC(NSOL + 4)
       REAL (KND)  X_NEW, X_AX
     END SUBROUTINE BIAKIMA_Y_V2

     SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, &
          ALBEDO, FLU_Int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
       USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, & 
            Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
       USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
       USE INTERN_DIMS, ONLY: Nvs
       USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
       USE TYPE_VARS, ONLY: KND
       USE WATER,  ONLY: SS_PDF
       INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
       REAL (KND), INTENT(IN) :: ALBEDO, FLU_Int, POS, SOL_EXP, PLANCK_SURF,&
            MuV(NLSA), Mue(NSZA), Phi(NPHI)
       REAL (KND) BC_S1(Nvs), LBC(Nvs)
       REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
       OPTIONAL :: REF_SUR
     END SUBROUTINE REFL_DIR_SPH

END INTERFACE
!==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

!  write(*,*) '=========> "integrate_init" '
  
  SELECT CASE (TRIM(RTM_TYPE))
  CASE ('transmission')
     LAST_INT = .TRUE.
  CASE DEFAULT
     IF(N_ITER == 1.OR..NOT.INT_MS_FLAG(JPOS)) THEN
!       ... Call without "spherical" iteration
        LAST_INT = .TRUE.
     ELSE
!       ... Call with "spherical" iteration
        LAST_INT = .FALSE.
     END IF
  END SELECT

  SPHER_IT = 1
  DO_TH_WF = INQ_WF('th', 'WFN')
!  IF(INT_MS_FLAG(JPOS)) ALLOCATE(ADI%Xs_RAD(NLAY,NSGP,N_NODE,4,Nvs), &
!       ADI%Ys_ALL(NSGP,N_NODE,NL_FINE,4,Nvs), &
!!  ADI%MSRF(NSolGrd,N_NODE,NL_FINE,Nvs)
  !  )
  IF(INT_MS_FLAG(JPOS)) ALLOCATE(ADI%Xs_RAD(NLAY,NSGP,N_NODE,4,Nvs), &
       ADI%Ys_ALL(Nvs,N_NODE,4,NSGP,NL_FINE))

  SELECT CASE (RTM_TYPE)
  CASE('spher_scat', 'ps_scat')
     IF( USE_PSM ) THEN 
        N_ANG = ND_ANG  ;  ALLOCATE( OSM%S_AN(N_ANG) )  ;  N_COF = 1
        CALL SCAT_ANG_GRID( DISCR_M )
        IF(DO_AEROSOL) N_COF = NLG_ALL_A
        IF( DO_CLOUD ) N_COF = MAX(N_COF,NLG_ALL_C)
!       ... Aerosol and/or cloud scattering matrices calculated using
!           all expansion coefficients
        IF( N_COF > 1 ) CALL EK_TO_SM(N_COF, N_ANG)
        IF( DO_CLOUD.AND.Use_Ydb_SCM ) THEN
!       ... Interpolate Yang DB scattering matrix on selected angle grid:
           ALLOCATE( OSM%CLD_LOC(N_ANG,NL_CLD,Nphr,Nm_ice,Nm_vs) )
           DO J = 1,NL_CLD
              DO Jv = 1, Nm_vs
                 DO Jmc = 1, Nm_ice
                    P_ang => OSM%CLD_SCM(1:Ydb_ang,J,1,Jmc,Jv)
                    CALL AL_INTERP(Ydb_ang, SCAT_ANG, P_ang,   &
                         N_ANG, OSM%S_AN, OSM%CLD_LOC(:,J,1,Jmc,Jv),.TRUE.,*880)
                 ENDDO  ! over water/ice phase (Jmc)
              ENDDO  ! over elements of scattering matrix (Jv)
           ENDDO  ! over cloud layers (J)
        ENDIF
     ENDIF
  END SELECT  ! 'spher_scat', 'ps_scat'

998 CONTINUE  ! New "spherical" iteration

  IF(INT_MS_FLAG(JPOS)) THEN
     DO I_N = 1, N_NODE
        DO  Ks = 1, Nvs
           CALL BIAKIMA_X(NLAY, ALT_EXT, NSGP, &
                ADI%Xs_RAD(:,:,I_N,:,Ks), RAD(:,:,I_N,Ks))
        ENDDO  ! over Stokes components (Ks)
     ENDDO  ! over nodes (I_N)
     DO J = 1, NL_FINE
        CALL BIAKIMA_Y_V2(Z_INV(INTERP_IDX(J)), SZA_EXT, Z_FINE(J), &
             ADI%Xs_RAD(INTERP_IDX(J),:,:,:,:), ADI%Ys_ALL(:,:,:,:,J))
     ENDDO
     !write(6,*) ADI%Ys_ALL(1,:,1,15,1)
     !pause
  END IF
  
  INDEX_TH_LOC = .TRUE.
!  write(*,*) '    LAST_INT in "integrate_init" ', LAST_INT
  IF( LAST_INT ) THEN
!    ... no "spherical" iteration
     GeoD => GeoD_OUT
     NAlts =  GeoD%NAlts; NSol = GeoD%NSol
!     write(*,*) '    NAlts  ', NAlts
     DO IV = 1, N_LOS_ANG
        KK = (IV - 1)*NFA
        INDEX_TH_LOC(KK + 1:KK + NFA) = INDEX_TH(IV)
     END DO
!     IF(RTM_MODE=='flux'.OR.RTM_MODE=='spher_alb')  &
     IF(RTM_MODE=='flux'.OR.RTM_MODE=='int_flux'.OR.RTM_MODE=='spher_alb')  &
          ALLOCATE(CSZA(NAlts,NSol), SolExp(NAlts,NSol))
  ELSE
!    ... "spherical" iteration
     GeoD => GeoD_ITR
     NAlts =  GeoD%NAlts; NSol = GeoD%NSol
     Allocate(CSZA(NAlts:NAlts,NSol), SolExp(NAlts:NAlts,NSol))
  ENDIF
  NVA = GeoD%NVA; NPhi = GeoD%NPhi

  IF( LAST_INT.AND.DO_TH_WF ) THEN
     ALLOCATE(RES_P(NAlts,NVA)) ! A
     DO IV = 1, NVA
        DO IA = 1, NAlts
           ALLOCATE(RES_P(IA,IV)%RAD(NSol,NPhi,Nvs)) ! A
           RES_P(IA,IV)%RAD = 0.0D0
        END DO
     END DO
  END IF  ! LAST_INT and DO_TH_WF

  DO_WF = LAST_INT.AND.WF_TG_FLAG
  ALLOCATE( RES(NAlts,NVA)) ! A

  DO IA = 1, NAlts
     DO IV = 1, NVA
        ALLOCATE(RES(IA,IV)%RAD(NSol,NPhi,Nvs)) ! A
        RES(IA,IV)%RAD = 0.0d0
     END DO
  END DO

!  ------------------- DO_WF -----------------------------------------
  IF(DO_WF) THEN
     DO IA = 1, NAlts
        DO IV = 1, NVA
           ALLOCATE(RES(IA,IV)%WF(NLAY,NSol,NPhi)) ! A
           RES(IA,IV)%WF = 0.0D0
        END DO
     END DO
       IF(WF_MS_FLAG(JPOS)) CALL MOVE_ALLOC(WFPS, DRP)
  END IF
! -------------------------------------------------------------------
  
  SELECT CASE (RTM_TYPE)
  CASE ('transmission')
!     DO IDX_LOS = 1, N_LOS
!        IF(.NOT.INDEX_TH_LOC(IDX_LOS)) CYCLE
!        CALL Integrate_Los_OCC (IDX_LOS, XM_0(IDX_LOS), 
!        $           XM_PHI(IDX_LOS), NL_START, NL_END, AL0_OUT, PCC, LOA%AL, 
!        $           X_AL, RESULT(:,IDX_LOS), Z_OUT, ALT_FLAG, DO_WF, 
!        $           WF_RES(:,:,JJ(IDX_LOS)), SPL_AL, *880)
!     END DO                 ! IDX_LOS
     DO IV = 1, NVA
        IF(.NOT.INDEX_TH_LOC(IV)) CYCLE
        DO IA = 1, NAlts
           CALL Integrate_Occ (IA, IV, GeoD, LOA%AL, ADI, PCC, RES(IA,IV), DO_WF)
        END DO
     END DO
    
  CASE('spher_scat', 'ps_scat')
     IF (LOG_LEVEL(1) >= N_DSO)  WRITE(LUNIT,*) ' ** START "integrate_los" **'

     If(Allocated(ErrFlg)) DeAllocate(ErrFlg)
     Allocate(ErrFlg(NAlts, NVA)) ;   ErrFlg = .False.
!$OMP PARALLEL DO PRIVATE(IV, IA) SCHEDULE(DYNAMIC) IF(PE_Mode == 'LOS')
     DO IV = 1, NVA
        !IF(PE_Mode == 'LOS') write(LUNIT,*) IV, "of", NVA
        IF(.NOT.INDEX_TH_LOC(IV)) CYCLE
        DO IA = 1, NAlts
           IF( Sel_los.AND.(.NOT.LAST_INT) ) Then
              IF( ABS(GeoD%LVG(IA,IV)%C_VA(1)) > Cr_Sel_los ) CYCLE
           End If
!            WRITE(*,*) '   Call Integr_los, IA, IV ',  IA, IV
           CALL Integrate_Los (IA, IV, GeoD, DEP_CF, LOA, ADI, PCC,  TE,   &
                RES(IA,IV), ALT_FLAG, DRP, DO_WF, JPOS, OSM, POS, FLU_Int, &
                ErrFlg(IA,IV), CSZA, SolExp)
        END DO  ! over altitudes (IA)
     END DO  ! over directions (IV)
     If(Any(ErrFlg)) Then
        WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
        GoTo 880
     End If
     IF (LOG_LEVEL(1) >= N_DSO) WRITE(LUNIT,*) ' ** "Integrate_Los" is done **'

     ! Tangent height weighting function 
!    -------------------------------------------------------------------
     IF(DO_TH_WF.AND.LAST_INT) THEN
        IF (LOG_LEVEL(1) >= N_DSO)  WRITE(LUNIT,*) &
             ' **** START "Integrate_Los for TH WFs" ****'
        GeoD => GeoD_PRT
        DO IV = 1, NVA
           IF(.NOT.INDEX_TH_LOC(IV)) CYCLE
           DO IA = 1, NAlts
!              CALL Integrate_Los (*880, IA, IV, GeoD_PRT, DEP_CF, LOA, &
!                   ADI, PCC, TE, RES(IA, IV), ALT_FLAG, DRP, DO_WF, JPOS, &
!                   OSM, POS, FLU_Int) 
              CALL Integrate_Los (IA, IV, GeoD_PRT, DEP_CF, LOA, &
                   ADI, PCC, TE, RES(IA, IV), ALT_FLAG, DRP, DO_WF, JPOS, &
                   OSM, POS, FLU_Int, ErrFlg(IA,IV)) 
           END DO
        END DO           
        If(Any(ErrFlg)) GoTo 880
        IF (LOG_LEVEL(1) >= N_DSO) &
             WRITE(LUNIT,*) ' **** "Integrate_Los" is completed ****'
     END IF  ! Tangent height WF
!    -------------------------------------------------------------------
  END SELECT  ! 'transmission' and ('spher_scat', 'ps_scat')

!  STOP '   Stop in "integrate_init.f" '

  IF(.NOT.LAST_INT) THEN
    CALL UPDATE_FIELD
     GOTO 998
  END IF
         
  SELECT CASE (RTM_MODE)
  CASE ('flux','spher_alb')
     CALL FLUX_CDI
!     write(*,*) '     After "flux_cdi" in "integrate_init": option "flux" '

  CASE DEFAULT
     IF( RTM_MODE=='int_flux' ) CALL FLUX_CDI
!     write(*,*) '     After "flux_cdi" in "integrate_init" '
     Allocate(ARR(NFA))
     DO IS = 1, NSol
        DO IV = 1, N_LOS_ANG
           DO IP = 1, NPhi
              DO Ks = 1, Nvs
                 FORALL(KK = 1:NFA) ARR(KK) = RES(1,(IV-1)*NFA+KK)%RAD(IS,IP,Ks)
                 RES(1,IV)%RAD(IS,IP,Ks) = SUM(FOV_WEIGHTS(:)*ARR(:))
              ENDDO  ! over Stokes components (Ks)
              IF(.NOT.DO_WF) CYCLE
              !---------------------------------------------------------------
              DO J = 1, NLAY
                 FORALL(KK = 1:NFA) ARR(KK) = RES(1,(IV-1)*NFA+KK)%WF(J,IS,IP)
                 RES(1,IV)%WF(J,IS,IP) = SUM(FOV_WEIGHTS(:)*ARR(:))
              END DO
              !---------------------------------------------------------------
           END DO  ! over azimuthal angles (IP)
        END DO  ! over LoS angles (IV)
     END DO  ! over solar angles (IS)
        
     IF (LOG_LEVEL(1) >= N_ESO) THEN
        ADD_LAB = ''
        IF(RTM_TYPE == 'ps_scat') ADD_LAB = '_ps'
        open(NewUnit=CU,file=DIR_OUT//'/out_rad'//ADD_LAB//'.dat')
        WRITE(CU,*) '# SCIATRAN '//VERSION
        WRITE(CU,*) '# N_SZA, N_LOS, N_AZM'
        WRITE(CU,*) '# SZA, VIEWING ANGLE, AZIMUTH, INTENSITY'
        WRITE(CU,*) NSol, N_LOS_ANG, NPhi
        DO IS = 1, NSol
           DO IV = 1, N_LOS_ANG
              IFOV = IV + FLOOR(NFA/2.)
              DO IP = 1, NPhi
                 write(CU,'(1x,3(1x,F9.4),*(1x,E12.5))') &
                      ACOS(GeoD%LSG(1,IFOV,IS,IP)%CG_SZA(1))*RAD_TO_DEG,&
                      ACOS(GeoD%LVG(1,IFOV)%C_VA(1))*RAD_TO_DEG, &
                      ACOS(GeoD%LSG(1,IFOV,IS,IP)%SAA(1))*RAD_TO_DEG, &
                      (RES(1,IV)%RAD(IS,IP,Ks),Ks=1,Nvs)
              END DO
           END DO
        END DO
        CLOSE(CU)
     END IF  ! LOG_LEVEL(1) >= N_ESO
        
     N_OUT = 0 
     DO IS = 1, NSol
        DO IV = 1, N_LOS_ANG
           DO IP = 1, NPhi
              N_OUT = N_OUT + 1
!              SPEC_OUT(JPOS,1,N_OUT) = &
!                   SPEC_OUT(JPOS,1,N_OUT) + ESFT_WEIGHT*RES(1,IV)%RAD(IS,IP,1)
              DO Ks = 1, Nvs
                 STOK_OUT(JPOS,1,1,N_OUT,Ks) = STOK_OUT(JPOS,1,1,N_OUT,Ks) &
                      + ESFT_WEIGHT*RES(1,IV)%RAD(IS,IP,Ks)
              ENDDO  ! over Stokes components (Ks)

              IF(DO_WF) WF_OUT(N_OUT,:) = RES(1,IV)%WF(:,IS,IP)
              
              IF(.NOT.INQ_WF('th', 'WFN', II)) CYCLE
              FORALL(KK = 1:NFA) ARR(KK) = RES_P(1,(IV - 1)*NFA + KK)%RAD(IS,IP,1)
              RES_P(1,IV)%RAD(IS,IP,1) = SUM(FOV_WEIGHTS(:)*ARR(:))
              WF_DB(II)%WF(JPOS,N_OUT,1,1) = &
                   WF_DB(II)%WF(JPOS,N_OUT,1,1) + ESFT_WEIGHT*&
                   (RES_P(1,IV)%RAD(IS,IP,1) - RES(1,IV)%RAD(IS,IP,1))/DELTA_TH
              
           END DO
        END DO
     END DO

    DeAllocate(ARR)     
  END SELECT  ! ('flux','spher_alb') and default
  
  DO IA = 1, NAlts
     DO IV = 1, NVA
        DEALLOCATE(RES(IA,IV)%RAD)
        IF(DO_WF) DEALLOCATE(RES(IA,IV)%WF)
     END DO
  END DO
  DEALLOCATE(RES)
  IF(DO_WF.AND.WF_MS_FLAG(JPOS)) THEN
     DO IS = 1, SIZE(DRP)
        DeAllocate(DRP(IS)%RES)
     END DO
     DEALLOCATE(DRP)
  END IF
  IF(INT_MS_FLAG(JPOS)) &
!       DEALLOCATE(ADI%X_RAD, ADI%Y_ALL)
       DEALLOCATE(ADI%Xs_RAD, ADI%Ys_ALL)
!       DEALLOCATE(ADI%Xs_RAD, ADI%Ys_ALL, ADI%MSRF)

  IF( USE_PSM ) THEN
     DEALLOCATE(OSM%S_AN)
     IF(DO_AEROSOL) DEALLOCATE(OSM%AER_SCM)
     IF( DO_CLOUD ) DEALLOCATE(OSM%CLD_LOC)
  ENDIF
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

CONTAINS
!***************************************************************************  
  SUBROUTINE SCAT_ANG_GRID( DISCR )
    CHARACTER (LEN=1)   DISCR
!   Local variable
!   --------------
    INTEGER I
    REAL (KND)  ANG(N_ANG), SCAT(N_ANG), AG(N_ANG-2), XG(N_ANG-2)
!   ----------------------------------------------------------------------
    
!     ... Angel grid for scattering matrix representation
    SELECT CASE( DISCR )
    CASE('G')  !  Gauss grid 
       CALL GAULEG ( -1.D0, 1.D0, XG, AG, N_ANG-2 )
       SCAT(1) = 1.d0 ; SCAT(N_ANG) = -1.d0
       SCAT(2:N_ANG-1) = XG(N_ANG-2:1:-1)      
       ANG(:) = ACOS( SCAT(:) )*RAD_TO_DEG 
    CASE('E')  !  Equidistant grid
       E_STEP = 2.d0/DBLE( N_ANG-1 ) ; ANG(1) = -1.d0         
       DO I = 2,N_ANG
          ANG(I) = ANG(I-1) + E_STEP
       ENDDO
       SCAT(:) = ANG(:)  ;  SCAT(N_ANG) = 1.0d0
    END SELECT
    OSM%S_AN(:) = SCAT(:)

    RETURN
  END SUBROUTINE SCAT_ANG_GRID

!***************************************************************************
  SUBROUTINE EK_TO_SM(N_MOM, N_ANG)
!   Input variables and arrays
!   --------------------------
    INTEGER, INTENT(IN) :: N_MOM, N_ANG
!   Local variable
!   --------------
    INTEGER     I, J, K, Nec
    REAL (KND)  T1, T2
    REAL (KND)  Plm_ss(N_MOM,N_ANG)
    REAL (KND), ALLOCATABLE :: Dp2_ss(:,:), Dm2_ss(:,:), Plm_as(:,:)
    REAL (KND), POINTER:: COF(:,:,:), DUM(:,:,:)
! =======================================================================

!     ... Legendre polynomials for all scattering angles
      CALL WIGNER_PLG(OSM%S_AN, N_ANG, 0, N_MOM, Plm_ss )

      IF(DO_AEROSOL) THEN  ;  Nec = NLG_ALL_A
         ALLOCATE( OSM%AER_SCM(N_ANG, NL_AER, Nm_vs) )
         COF => OSM%AER_ALL(1:Nec,  1:NL_Aer,1:Nm_vs)
         DUM => OSM%AER_SCM(1:N_ANG,1:NL_AER,1:Nm_vs)
         DO K = 1, NL_AER
            DO J = 1,N_ANG
               DUM(J,K,1) = SUM( Plm_ss(1:Nec,J)*COF(:,K,1) )
            ENDDO
         ENDDO
      ENDIF

      IF(DO_CLOUD.AND.(.NOT.Use_Ydb_SCM) ) THEN  ;  Nec = NLG_ALL_C
!         write(*,*) '   Scal Nec = ', Nec
         ALLOCATE( OSM%CLD_LOC(N_ANG, NL_CLD, Nphr, Nm_ice, Nm_vs) )
         DO Jmc = 1, Nm_ice
            COF => OSM%CLD_ALL(1:Nec,  1:NL_Cld, 1, Jmc, 1:Nm_vs) 
            DUM => OSM%CLD_LOC(1:N_ANG,1:NL_CLD, 1, Jmc, 1:Nm_vs) 
            DO K = 1, NL_CLD
               DO J = 1,N_ANG
                  DUM(J,K,1) = SUM( Plm_ss(1:Nec,J)*COF(:,K,1) )
               ENDDO
            ENDDO
         ENDDO  ! over water/ice phase (Jmc)
      ENDIF

      IF( Nm_vs == 1 ) RETURN  ! Scalar case

      ALLOCATE(Dp2_ss(N_MOM,N_ANG), Dm2_ss(N_MOM,N_ANG), Plm_as(N_MOM,N_ANG))
!     ... Wigner functions for all scattering angles
      CALL WIGNER_PLG     (OSM%S_AN, N_ANG, 2, N_MOM, Plm_as )
      CALL WIGNER_FUNCTION(OSM%S_AN, N_ANG, 2, N_MOM, Dp2_ss, Dm2_ss)

      IF(DO_AEROSOL) THEN  ;  Nec = NLG_ALL_A
         COF => OSM%AER_ALL(1:Nec,  1:NL_Aer, 1:Nm_vs)
         DUM => OSM%AER_SCM(1:N_ANG,1:NL_AER, 1:Nm_vs)
         DO K = 1, NL_Aer
            DO J = 1,N_ANG
               T1 = SUM( (COF(:,K,2) + COF(:,K,3))*Dp2_ss(1:Nec,J) )
               T2 = SUM( (COF(:,K,2) - COF(:,K,3))*Dm2_ss(1:Nec,J) )
               DUM(J,K,2) =  0.5*( T1 + T2 )
               DUM(J,K,3) =  0.5*( T1 - T2 )
               DUM(J,K,4) =  SUM( Plm_ss(1:Nec,J)*COF(:,K,4) )         
!           ... The sign '-' is not used here because coefficients 
!                beta1 and beta2 obtain negative sign after input. 
               DUM(J,K,5) = SUM( Plm_as(1:Nec,J)*COF(:,K,5) )
               DUM(J,K,6) = SUM( Plm_as(1:Nec,J)*COF(:,K,6) )
            ENDDO  ! over scattering angles (J)
         ENDDO  ! over scattering layers (K)
      ENDIF

      IF(DO_CLOUD.AND.(.NOT.Use_Ydb_SCM)) THEN  ;  Nec = NLG_ALL_C
!         write(*,*) '   Vect Nec = ', Nec
         DO Jmc = 1, Nm_ice
            COF => OSM%CLD_ALL(1:Nec,  1:NL_Cld, 1, Jmc, 1:Nm_vs) 
            DUM => OSM%CLD_LOC(1:N_ANG,1:NL_CLD, 1, Jmc, 1:Nm_vs) 
            DO K = 1, NL_Aer
               DO J = 1,N_ANG
                  T1 = SUM( (COF(:,K,2) + COF(:,K,3))*Dp2_ss(1:Nec,J) )
                  T2 = SUM( (COF(:,K,2) - COF(:,K,3))*Dm2_ss(1:Nec,J) )
                  DUM(J,K,2) =  0.5*( T1 + T2 )
                  DUM(J,K,3) =  0.5*( T1 - T2 )
                  DUM(J,K,4) =  SUM( Plm_ss(1:Nec,J)*COF(:,K,4) )         
!           ... The sign '-' is not used here because coefficients 
!                beta1 and beta2 obtain negative sign after input. 
                  DUM(J,K,5) = SUM( Plm_as(1:Nec,J)*COF(:,K,5) )
                  DUM(J,K,6) = SUM( Plm_as(1:Nec,J)*COF(:,K,6) )
               ENDDO  ! over scattering angles (J)
            ENDDO  ! over scattering layers (K)
         ENDDO  ! over water/ice phase (Jmc)
      END IF

      RETURN
    END SUBROUTINE EK_TO_SM
!**************************************************************************

! ----------------------------------------------------------------------
  SUBROUTINE UPDATE_FIELD
    INTEGER I
    REAL (KND)  Azi(1) !, PLANCK_SURF
!  ---------------------------------------------------------------------
    ALLOCATE( DIR_REFL(N_PLUS,1,Nvs,Nvs), DIFF_REFL(N_PLUS,N_MINUS,Nvs,Nvs), &
         FLUP(N_MINUS,Nvs), BC_S1(Nvs), LBC(Nvs))
    DIR_REFL = 0.d0  ;  DIFF_REFL = 0.d0
    IF( .NOT.BRDF_FLAG ) THEN
       DIFF_REFL(:,:,1,1) = ALBEDO ; DIR_REFL (:,:,1,1) = ALBEDO
    ELSE
       DO I = 1, N_MINUS
!         ... Contribution into reflection from diffuse radiation         
          DO J = 1, N_PLUS  ;  Azi = PHI_NODE(J) - PHI_MINUS(I)
             CALL REFL_DIR_SPH(1,1,1, MJU_NODE(J:J), MJU_MINUS(I:I), Azi,  &
                  POS, ALBEDO, FLU_Int, SolExp(NLAY,1), TE%PLANCK_SURF, BC_S1, &
                  LBC, Rx)
             DIFF_REFL(J,I,:,:) = Rx(:,:)
          ENDDO
       ENDDO
    ENDIF  ! BRDF_FLAG 

    DO IS = 1, NSol
       DO IA = 1, NLAY  ;  II = 0
          DO IV = 1, NVA
             DO IP = 1, NPhi  ;  II = II + 1
                IF( ABS(GeoD%LVG(IA,IV)%C_VA(1)) > Cr_Sel_los ) CYCLE
                RAD(IA,IS,II,:) = RES(IA,IV)%RAD(IS,IP,:)
             END DO
          END DO
       END DO
       
       !WRITE(*,*) '   BRDF in "update_field" Is = ', Is
       IF( BRDF_FLAG.AND.CSZA(NLAY,IS) > 0.d0 ) THEN
!      ... Contribution into reflection from direct solar light
          DO J = 1, N_PLUS
             CALL REFL_DIR_SPH(1,1,1, MJU_NODE(J:J), CSZA(NLAY,IS:IS), &
                  PHI_NODE(J:J), POS, ALBEDO, FLU_Int, SolExp(NLAY,IS), &
                  TE%PLANCK_SURF, BC_S1, LBC, Rx)
!             WRITE(*,'(A,2(1x,F8.2), 2x,e12.4)') '   C_SZA = ', &
!                  ACOS( MJU_NODE(J:J) )/Pie *180., &
!                  ACOS( CSZA(NLAY,IS:IS) )/Pie *180., Rx(1,1)
             DIR_REFL(J,1,:,:) = Rx(:,:)
          ENDDO  ! over upward directions (J)
       ENDIF  ! BRDF_FLAG and solar angle less than 90 deg.
!              WRITE(*,*) '   NORM_DN - ', NORM_DN
!              DO I = 1, SIZE( MJU_MINUS )
!!                 IF( FLU(I,1) <= 0.d0 )  &
!!                      WRITE(*,*) '       FLU - ', I, FLU(I,1)
!                 IF( MJU_MINUS(I) <= 0.d0 )  &
!                      WRITE(*,*) ' MJU_MINUS - ', I, MJU_MINUS(I)
!                 IF( W_MINUS(I) <= 0.d0 )  &
!                      WRITE(*,*) '   W_MINUS - ', I, W_MINUS(I) 
!              ENDDO
      ! WRITE(*,*) '   ------------------------- '

       DO J = 1, N_PLUS
          DO I = 1, N_MINUS
             DO KK = 1,Nvs
                FLUP(I,KK) = &
                     SUM( DIFF_REFL(J,I,KK,:)*RAD(NLAY,IS,I+N_PLUS,:) )
             ENDDO  ! over Stokes components (KK)
          ENDDO  ! over downward directions (I)
          DO KK = 1, Nvs
             RAD(NLAY,IS,J,KK) = SUM( FLUP(:,KK)*W_MINUS*MJU_MINUS )/NORM_DN &
                  + DIR_REFL(J,1,KK,1)*CSZA(NLAY,IS)*SolExp(NLAY,IS)
          ENDDO  ! over Stokes components (KK)
          RAD(NLAY,IS,J,1) = RAD(NLAY,IS,J,1) + FLU_Int  
       END DO  ! over upward directions (J)
    END DO  ! over solar angles (IS)

    IF(DO_SF_OUT) THEN
       WRITE(CP,'(I0)') SPHER_IT
       ADD_LAB = TRIM(ADJUSTL(CP))
       IF(TRIM(RTM_TYPE) == 'ps_scat') THEN
          open(NewUnit=CU,file=DIR_OUT//'/test_rm_ps.dat-'//ADD_LAB,recl=2000)
       ELSE
          open(NewUnit=CU,file=DIR_OUT//'/test_rm.dat-'//ADD_LAB,recl=2000)
       END IF
       DO IS = 1, NSOL
          write(CU,'(*(1x,E12.5))') &
               ACOS(GeoD%LSG(NLAY,1,IS,1)%CG_SZA(1))*RAD_TO_DEG
       END DO
       DO IS = 1, NSOL
          DO J = 1, NLAY
             write(CU,'(*(1x,E12.5))')  ZZZ(J), &
                  (RAD(J,IS,KK,1), KK = 1,NSol*NPhi)
          END DO
       END DO
       close(CU)
    END IF  ! DO_SF_OUT
               
    DO IA = 1, NAlts
       DO IV = 1, NVA
          DEALLOCATE(RES(IA,IV)%RAD)
       END DO
    END DO
    DEALLOCATE(DIR_REFL, DIFF_REFL, SolExp, CSZA, RES, FLUP, BC_S1, LBC)

    IF(SPHER_IT.EQ.N_ITER - 1) LAST_INT = .TRUE.
    SPHER_IT = SPHER_IT + 1
         
  END SUBROUTINE UPDATE_FIELD
! ----------------------------------------------------------------------

! ----------------------------------------------------------------------
!              Calculate fluxes
! ----------------------------------------------------------------------
  SUBROUTINE FLUX_CDI
    INTEGER I
    REAL (KND) Azi(1) !, PLANCK_SURF
    REAL (KND), ALLOCATABLE:: XMS(:), AGS(:)
    ! ----------------------------------------------------------------------
    
    ALLOCATE( DIR_REFL(N_PLUS,1,Nvs,Nvs), DIFF_REFL(N_PLUS,N_MINUS,Nvs,Nvs), &
         FLUP(N_MINUS,Nvs), BC_S1(Nvs), LBC(Nvs) )
    DIR_REFL = 0.d0 ;  DIFF_REFL = 0.d0
    IF( .NOT.BRDF_FLAG ) THEN
       DIFF_REFL(:,:,1,1) = ALBEDO  ;  DIR_REFL(:,:,1,1) = ALBEDO
    ELSE
       DO I = 1, N_MINUS
!         ... Contribution into reflection from diffuse radiation (FLUX_CDI)
          DO J = 1, N_PLUS  ;  Azi = PHI_NODE(J) - PHI_MINUS(I)
             CALL REFL_DIR_SPH(1,1,1, MJU_NODE(J:J), MJU_MINUS(I:I), Azi,  &
                  POS, ALBEDO, FLU_Int, SolExp(NLAY,1), TE%PLANCK_SURF, BC_S1, &
                  LBC, Rx)
             DIFF_REFL(J,I,:,:) = Rx(:,:)
          ENDDO
       ENDDO
    ENDIF  ! BRDF_FLAG

    Allocate(ARR(N_NODE))
    DO IS = 1, NSol
       IF( BRDF_FLAG.AND.CSZA(NLAY,IS) > 0.d0 ) THEN  !  In FLUX_CDI
          DO J = 1, N_PLUS
             CALL REFL_DIR_SPH(1,1,1, MJU_NODE(J:J), CSZA(NLAY,IS:IS), &
                  PHI_NODE(J:J), POS, ALBEDO, FLU_Int, SolExp(NLAY,IS), &
                  TE%PLANCK_SURF, BC_S1, LBC, Rx)
             DIR_REFL(J,1,:,:) = Rx(:,:)
          ENDDO  ! over upward directions (J)
       ENDIF  ! BRDF_FLAG and solar angle less than 90 deg.
       
       ! There is only one azimuth angle per viewing ange in "flux" mode
       DO IV = 1, N_PLUS 
          DO I = 1, N_MINUS
             DO KK = 1,Nvs
                FLUP(I,KK) = SUM( DIFF_REFL(IV,I,KK,:)* &
                     RES(NLAY,I+N_PLUS)%RAD(IS,1,:) )
             ENDDO  ! over Stokes components (KK)
          ENDDO  ! over downward directions (I)
          DO KK = 1, Nvs
             RES(NLAY,IV)%RAD(IS,1,KK) = &
                  SUM( FLUP(:,KK)*W_MINUS*MJU_MINUS )/NORM_DN         &
                  + DIR_REFL(IV,1,KK,1)*CSZA(NLAY,IS)*SolExp(NLAY,IS)
          ENDDO  ! over Stokes components (KK)
          RES(NLAY,IV)%RAD(IS,1,1) = RES(NLAY,IV)%RAD(IS,1,1) + FLU_Int
       END DO  ! over upward directions (IV)

       DO IA = 1, NLAY
          FORALL(KK = 1:N_NODE) ARR(KK) = RES(IA,KK)%RAD(IS,1,1)
          ACTFLUX(JPOS,IA,IS) = ACTFLUX(JPOS,IA,IS) &
               + ESFT_WEIGHT*SUM(ARR*WEIGHTS)

          FORALL(KK = 1:N_PLUS) ARR(KK) = RES(IA,KK)%RAD(IS,1,1)
          DIF_FLUX_UP(JPOS,IA,IS) = DIF_FLUX_UP(JPOS,IA,IS) + ESFT_WEIGHT*&
               SUM(ARR(1:N_PLUS)*W_PLUS(:)*MJU_PLUS(:))          

          FORALL(KK = N_PLUS + 1:N_NODE) ARR(KK) = RES(IA,KK)%RAD(IS,1,1)
          DIF_FLUX_DN(JPOS,IA,IS) = DIF_FLUX_DN(JPOS,IA,IS) - ESFT_WEIGHT*&
               SUM(ARR(N_PLUS + 1:N_NODE)*W_MINUS(:)*MJU_MINUS(:))
       END DO  ! over level of altitude grid (IA)
    END DO  ! over solar angles (IS)

    DIR_RAD(JPOS,:,:) = DIR_RAD(JPOS,:,:) + ESFT_WEIGHT*PIE*SolExp(:,:)
    DIR_FLUX(JPOS,:,:) = DIR_RAD(JPOS,:,:)*ABS(CSZA(:,:))
    SELECT CASE( RTM_MODE )
    CASE ('flux')
!       WRITE(*,*) '    Mode "flux" '
!       WRITE(*,*) '    DIR_FLUX : '
!       WRITE(*,*) DIR_FLUX(JPOS,:,:)
       WHERE(CSZA(:,:) < 0.0D0)
          FLUX_UP(JPOS,:,:) = DIF_FLUX_UP(JPOS,:,:) + DIR_FLUX(JPOS,:,:)
          FLUX_DN(JPOS,:,:) = DIF_FLUX_DN(JPOS,:,:)
       ELSEWHERE
          FLUX_DN(JPOS,:,:) = DIF_FLUX_DN(JPOS,:,:) + DIR_FLUX(JPOS,:,:)  
          FLUX_UP(JPOS,:,:) = DIF_FLUX_UP(JPOS,:,:)
       END WHERE

    CASE ('spher_alb')
       ALLOCATE( XMS(Nsol), AGS(Nsol) )
      CALL GAULEG( 0.d0, 1.d0, XMS, AGS, NSol )
      DO IA = 1, NLAY
         FLUX_UP(JPOS,IA,1) = SUM(DIF_FLUX_UP(JPOS,IA,:)*AGS(:))/Pie*2.d0 
         FLUX_DN(JPOS,IA,1) = SUM(DIF_FLUX_DN(JPOS,IA,:)*AGS(:))/Pie*2.d0 &
              + 2.0*SUM(DIR_FLUX(JPOS,IA,:)*AGS(:))/Pie
      ENDDO

   END SELECT

    DEALLOCATE(SolExp, CSZA, DIR_REFL, DIFF_REFL, ARR, FLUP, BC_S1, LBC)
    
  END SUBROUTINE FLUX_CDI
! ----------------------------------------------------------------------
  
END Subroutine Integrate_Init
!***************************************************************************
