!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 19 Nov 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
! Called by "integrate_init.f90
SUBROUTINE Integrate_Los (IA, IV, GeoD, DEP_CF, LOA, ADI, PCC, TE, RES, &
     ALT_FLAG, WFPS, DO_WF, JPOS, OSM, POS, FLU_Int, ErrFlg, CSZA, SolExp)

  USE ABS_COEFF_PC, ONLY: PC_ABS_COEFF
  USE AEROSOLS,     ONLY: IDX_AER
  USE BRDF,         ONLY: BRDF_PARAM
  USE CLOUDS,       ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR, SCAT_ANG
  USE GRIDS,        ONLY: NLAYERS, ZZZ, Z_INV, NSOL, X_INV, SZA_EXT, Z_TOA, Z_BOA
  USE INTERN_DIMS,  ONLY: Nvs, Nm_vs, Nlg_Ray, Leg_aer => NLG_ALL_A, &
                          Leg_cld => NLG_ALL_C
  USE LOS_ANGLES,   ONLY: DirSun, Los_Geom_Init, Los_Geom_Sol_Init, &
                          Los_Geom_Clean
  USE LOG_PARS,     ONLY: EU
  USE NODES,        ONLY: N_NODE, MJU_NODE, SIN_NODE, PHI_NODE, NORM_DN, WEIGHTS, &
                          W_MINUS, MJU_MINUS, PHI_MINUS, N_PLUS, N_MINUS, Vz_A, &
                          NAz, NVa, N_Az, Az_A, N_VA, NbN, VbN, PbN, WbN, &
                          USE_AIN, NORM_BN, N_HIN, MJU_HIN, SIN_HIN, PHI_HIN, &
                          USE_HIN, WEI_HIN, Nva_HIN, Naz_HIN
  USE SPHER_PARS,   ONLY: I_FINE, ONE_MS_SZ, X_SHAD
  USE ST_VARS,      ONLY: DO_AEROSOL, DO_PC, BRDF_FLAG, DO_REFRAC, DIFB, &
                          INT_MS_FLAG, WF_MS_FLAG, PreCalcAngs, Pie, &
                          Use_Sym, USE_PSM, ND_ANG, E_STEP, Short_los, &
                          Cr_Short_los, MSF_INT, PreCalcRtMatr, DO_PLANCK,&
                          PreCalcGeom
  USE ST_INPUT,     ONLY: APH_TYPE
  USE TOOLS,        ONLY: GET_SZA_IDX, LOS_MS_IDX
  USE TYPES,        ONLY: KND, PCC_TYPE, OSM_TYPE, LOS_GEOMETRY, WFS_PS, &
                          Geometry_Data, AuxDataInterp, RES_DATA, LOA_TYPE, &
                          TE_TYPE, LOS_SOLAR_GEOM, LOS_VIEW_GEOM, DIR_SOL_GEOM

  INTEGER     JPOS, IA, IV
  LOGICAL     ALT_FLAG, DO_WF
  REAL (KND)  DEP_CF(3), POS, FLU_Int
  REAL (KND), ALLOCATABLE, OPTIONAL :: CSZA(:,:), SolExp(:,:)
  TYPE (PCC_type) :: PCC    
  TYPE (OSM_type) :: OSM   ! Original scattering matrices
  TYPE (LOA_type) :: LOA
  TYPE (TE_type)  :: TE    ! Thermal emission
  TYPE (AuxDataInterp), Target:: ADI
  TYPE (Geometry_Data), TARGET :: GeoD
  TYPE (WFS_PS), ALLOCATABLE :: WFPS(:) ! Pseudo-spherical WFs
  TYPE (RES_DATA) RES
  
! ... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'Integrate_Los'
  CHARACTER (LEN=6) C_POS
  INTEGER  L, I, K, Ks, NLS, NTH, II, IMS1, IMS2, N_DS, ID_N, IP, IS, &
       N_MOM, KK, I1, I2, ILS, JLS, Ci1, N_Poi
  INTEGER, ALLOCATABLE :: LL(:)
  INTEGER, POINTER :: ISEL(:), IOB(:)
  LOGICAL  TH_FLG, FP_FLG, ErrFlg
  REAL (KND)  NORM_PF
  REAL (KND), ALLOCATABLE :: WF_TMP(:), WF(:), BC_WF(:), WF_PS(:,:,:), &
       D_RAD_TH(:,:,:), D_RAD_FP(:,:,:), H2(:), Rx(:,:), D_REF(:,:,:), &
       EX_L(:), SA(:), SC(:), Scl(:), SCR(:), SOL_EXP(:), &
       A_C_1(:), A_C_2(:), D_C(:,:), ASYM_COEFF(:), P_FUNC(:), TH(:), &
       H1(:), E_c(:,:), HH(:), Plf(:), FL(:,:,:), FLU(:,:), &
       Dp2_ss(:), Dm2_ss(:), Plm_as(:), Plm_ss(:), DS_SUN(:,:), DS_EXP(:,:), &
       EX_1(:)
  TARGET FL
  REAL (KND), ALLOCATABLE :: FL_INT(:,:), FL_Mu(:,:), RelAz(:), &
       C_RA(:), C_SA(:), FLL(:,:), FLL_Mu(:,:)
  REAL (KND), ALLOCATABLE, TARGET :: AL_1(:), ZLS_LOC(:), &
       AL_PC(:), Y_TH(:,:,:,:), Y_FP(:,:,:,:), A_UP(:), A_DN(:)
  REAL (KND), POINTER :: AL_2(:), ZLS(:), AL(:), FLP(:,:),  &
       MJU_Poi(:), SIN_Poi(:), PHI_Poi(:), WEI_Poi(:)
  REAL (KND)  DL,  DUM, SD, H_0, PHAS_FUNC, DD, CVA, Mu_in, Si_in, Azm_in, &
       Co_s, Co_ls, Si_s, Mu_sc, Si_sc, Rel_az, RL1(2), RR1(2), T1, T2,  &
       Sv_in(4), Sv_sp(4), Sv_sc(4), Sv_mp(4), F_UL(2,2), F_DR(2,2), &
       BC_S1(Nvs), DD4(Nvs), DD3(Nvs), DEL(Nvs), DDF(Nvs),   &
       CosRA, SSSF(Nvs), S_F(Nvs)
!       CosRA, SSSF(4), S_F(4)
  TYPE(LOS_GEOMETRY), TARGET :: LG
  TYPE (LOS_VIEW_GEOM), TARGET :: LVG
  Type(LOS_Solar_Geom), Target :: LSG_L
  TYPE(LOS_SOLAR_GEOM), POINTER :: LSG
  TYPE(DIR_SOL_GEOM), ALLOCATABLE :: DSB(:)
  !==================================================================

  INTERFACE
     SUBROUTINE BIAKIMA_Y_V2 (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
       USE GRIDS,       ONLY: NSOL
       USE TYPES,       ONLY: KND
       REAL (KND)  X_RAD(:,:,:,:), Y_ALL(:,:,:,:) 
       REAL (KND)  Y_LOC(NSOL + 4)
       REAL (KND)  X_NEW, X_AX
     END SUBROUTINE BIAKIMA_Y_V2

     SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, &
          ALBEDO, FLU_Int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
       USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, & 
            Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
       USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
       USE INTERN_DIMS, ONLY: Nvs
       USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
       USE TYPE_VARS, ONLY: KND
       USE WATER,  ONLY: SS_PDF
       INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
       REAL (KND), INTENT(IN) :: ALBEDO, FLU_Int, POS, SOL_EXP, &
            PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
       REAL (KND) BC_S1(Nvs), LBC(Nvs)
       REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
       OPTIONAL :: REF_SUR
     END SUBROUTINE REFL_DIR_SPH
     
     Subroutine DS_FORM_3 (ZLS, DS, IOB, NLS, A_UP, A_DN, DS_WF)
       Use Grids, Only: Nlayers, ZZZ
       Use Types, Only: KND
       Integer  NLS, IOB(NLS)
       Real (KND)  DS(2,NLS), ZLS(NLS)
       Real (KND), Allocatable ::  A_UP(:), A_DN(:), DS_WF(:,:)
     End Subroutine DS_FORM_3
  END INTERFACE
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
  If(PreCalcAngs) Allocate(DSB(1))

  C_POS = 'ASIS'

  IF( USE_HIN ) THEN
     N_Poi = N_HIN
     MJU_Poi => MJU_HIN(:) ;  PHI_Poi => PHI_HIN(:)
     SIN_Poi => SIN_HIN(:) ;  WEI_Poi => WEI_HIN(:)
     ALLOCATE( FLL_Mu(Nvs,NAz*NVa_HIN) )
  ELSE
     N_Poi = N_NODE
     MJU_Poi => MJU_NODE(:) ;  PHI_Poi => PHI_NODE(:)
     SIN_Poi => SIN_NODE(:) ;  WEI_Poi => WEIGHTS(:)
  ENDIF
  ALLOCATE( RelAz(N_Poi), C_RA(N_Poi), C_SA(N_Poi), FLL(Nvs,N_poi))
  
  H_0 = GeoD%LVG(IA,IV)%H_0 ;  CVA = GeoD%LVG(IA,IV)%C_VA(1)
  If(PreCalcGeom) Then
     NLS = GeoD%LVG(IA,IV)%NLS
     Allocate(LG%DS(2,NLS), LG%S_VA(NLS), LG%C_VA(NLS), LG%C_SZA(NLS), &
          LG%S_SZA(NLS), LG%SAA(NLS))
     If(DO_REFRAC) ALLOCATE(LG%CG_SZA(NLS))
     LG%C_VA(:) = GeoD%LVG(IA,IV)%C_VA(:)
     LG%S_VA(:) = SQRT(1.0D0 - LG%C_VA(:)**2)
     LG%DS(:,:) = GeoD%LVG(IA,IV)%DS(:,:)
     NLS = GeoD%LVG(IA,IV)%NLS ;  NTH = GeoD%LVG(IA,IV)%NTH
     TH_FLG = GeoD%LVG(IA,IV)%TH_FLG ;  FP_FLG = GeoD%LVG(IA,IV)%FP_FLG
     ZLS => GeoD%LVG(IA,IV)%ZLS ;  ISEL => GeoD%LVG(IA,IV)%ISEL
     IOB => GeoD%LVG(IA,IV)%IOB
  Else
     LVG = GeoD%LVG(IA,IV) ;  DeAllocate(LVG%C_VA)
     If(DO_REFRAC) DeAllocate(LSG%CG_SZA)
     Call Los_Geom_Init (LG, LVG, GeoD%Alt(IA), GeoD%AltFlg, GeoD%IDA(IA), CVA)
     NLS = LVG%NLS; NTH = LVG%NTH; TH_FLG = LVG%TH_FLG; FP_FLG = LVG%FP_FLG
     ZLS => LVG%ZLS;  ISEL => LVG%ISEL; IOB => LVG%IOB
     GeoD%LVG(IA,IV)%CVA_TH = LVG%C_VA(LVG%NTH)
     LG%S_VA(:) = SQRT(1.0D0 - LG%C_VA(:)**2)
  End If

! ... This extinction coefficient is used if DO_WF==.TRUE.
!     (in lines  505 - 520) 
  AL => LOA%AL(:)

!  IF(GeoD%Alt(IA) == Z_TOA.AND.CVA <= 0.0D0) THEN
!     WRITE(*,*) '  Z_TOA:  IV, NLS  ', IV, NLS
!     RETURN
!  END IF
!  IF(GeoD%Alt(IA) == Z_BOA.AND.CVA > 0.0D0) THEN
!     WRITE(*,*) '  Z_BOA:  IV, NLS   ', IV, NLS
!     RETURN
!  END IF
  
  N_MOM = MAX( Nlg_Ray,Leg_aer,Leg_cld )
  IF( Nvs > 1 ) ALLOCATE( Dp2_ss(N_MOM), Dm2_ss(N_MOM), Plm_as(N_MOM), &
       Plm_ss(N_MOM), E_c(N_MOM,Nm_vs) ) ! A
  
  ALLOCATE(AL_1(NLS), SC(NLS), D_C(2,NLS))
  AL_1(:) = ADI%AL0_OUT(ISEL(:))

!  write(*,*) '   int_los:       NLS ', NLS
!  write(*,*) '   int_los: AL_1(NLS) ', AL_1(NLS)
  
  FORALL(II = 1:2) D_C(II,:) = DEP_CF(II)*ADI%SCR_OUT(ISEL(:))

  IF( APH_TYPE /= 'mie' ) THEN
     ALLOCATE(ASYM_COEFF(NLS), A_C_1(NLS), A_C_2(NLS))
     IF(DO_AEROSOL) THEN
        ASYM_COEFF(:) = 2.0D0*ADI%ASC_OUT(ISEL(:))
        A_C_1(:) = 1.0D0 + ADI%ASC_OUT(ISEL(:))**2
        A_C_2(:) = ADI%SCA_OUT(ISEL(:))*(1.0D0 - ADI%ASC_OUT(ISEL(:))**2)
     ELSE
        ASYM_COEFF(:) = 0.0D0 ;  A_C_1(:) = 1.0D0 ;  A_C_2(:) = 0.0D0
     END IF
  ELSE
     ALLOCATE(SA(NLS))
     SA(:) = ADI%SCA_OUT(ISEL(:))
  END IF

  IF( APH_TYPE == 'none' ) THEN
     ALLOCATE(SA(NLS))  ;  SA(:) = ADI%SCA_OUT(ISEL(:))
  ENDIF

  IF( DO_PLANCK ) THEN
     ALLOCATE(Plf(NLS))  ;  Plf(:) = ADI%Plf_OUT(ISEL(:))
  ENDIF

  KK = N_MINUS
  IF( USE_AIN ) THEN
     KK = NbN
     ALLOCATE( FL_INT(Nvs,N_Az*N_VA), FL_Mu(Nvs,NAz*N_VA) )
  ENDIF
  ALLOCATE( D_REF(KK,Nvs,Nvs), FLU(KK,Nvs) )
  
  ALLOCATE(SCR(NLS))
  SCR(:) = ADI%SCR_OUT(ISEL(:))

  SC(:) = SCR(:)
  IF( DO_AEROSOL ) SC(:) = SC(:) + ADI%SCA_OUT(ISEL(:))
  IF( DO_CLOUD )  THEN
     ALLOCATE( Scl(NLS) )  ;  Scl(:) = ADI%SCC_NSO(ISEL(:),1)
     SC(:) = SC(:) + Scl(:)
  ENDIF
  
  IF(TH_FLG) Call InterpToAlt (NTH, H_0, Y_TH, D_RAD_TH)
  IF(FP_FLG) Call InterpToAlt (1, ZLS(1), Y_FP, D_RAD_FP)

  ! ****************************************
  ALLOCATE( TH(NLS), SOL_EXP(NLS), P_FUNC(Nvs), &
       HH(Nvs), LL(NLS), EX_L(NLS), EX_1(0:NLS) )

  IF(.NOT.DO_PC) THEN
     EX_L(1) = 1.0D0
     DUM = 0.0D0
     DO I = 2, NLS
        DUM = DUM + AL_1(I - 1)*LG%DS(2,I - 1) + AL_1(I)*LG%DS(1,I) 
        EX_L(I) = DEXP(- DUM)
     END DO
     EX_1(0) = 1.0D0
     DUM = 0.0D0
     DO I = 1, NLS
        DUM = DUM + AL_1(I)*(LG%DS(1,I) + LG%DS(2,I)) 
        EX_1(I) = DEXP(- DUM)
     END DO
  END IF
        
  IF(DO_WF) THEN
     ALLOCATE(WF_TMP(NLAYERS), WF(NLAYERS), BC_WF(NLAYERS), DS_SUN(1,NLAYERS)) ! A
     IF(WF_MS_FLAG(JPOS)) &
          ALLOCATE(WF_PS(NLAYERS, N_NODE, NLS), H2(NLAYERS)) ! A
  END IF

  IF(DO_PC) ALLOCATE(AL_PC, SOURCE = AL_1)

  IF(INT_MS_FLAG(JPOS)) ALLOCATE(H1(Nvs), FL(Nvs,N_NODE,NLS))
     
  JLS = NLS
!  IF( NLS == 1 ) JLS = 0

  DO IS = 1, GeoD%NSol
     DO IP = 1, GeoD%NPhi
        If(.Not.PreCalcGeom) Then
          Call Los_Geom_Sol_Init (LVG, LG, GeoD%LSG(IA,IV,IS,IP)%C_SZA(1),&
               GeoD%LSG(IA,IV,IS,IP)%SAA(1), LSG_L, *880)
          LSG => LSG_L
          GeoD%LSG(IA,IV,IS,IP)%CSZA_TH = LSG%C_SZA(LVG%NTH)
          GeoD%LSG(IA,IV,IS,IP)%SAA_TH = LSG%SAA(LVG%NTH)
          If (DO_REFRAC) GeoD%LSG(IA,IV,IS,IP)%CG_SZA = LG%CG_SZA
        Else
           LSG => GeoD%LSG(IA,IV,IS,IP)
           LG%C_SZA = LSG%C_SZA
           IF(DO_REFRAC) THEN
              LG%CG_SZA = LSG%CG_SZA
           ELSE
              LG%CG_SZA => LG%C_SZA
          END IF
          LG%SAA = LSG%SAA
          !LG%SAA = GeoD%LSG(IA,IV,IS,IP)%SAA + Pie
            LG%S_SZA = SQRT(1.0D0 - LG%C_SZA**2)
        End If
        
        TH =  LSG%TH 
        
        ! This does not work if the beginning of LOS is in the shadow
        CALL LOS_MS_IDX(LSG%CG_SZA, IMS1, IMS2)
           
        ! Consideration of photochemestry
        ! ********************************
        IF(DO_PC) THEN
           CALL PC_ABS_COEFF (ISEL, LG%C_SZA, NLS, PCC, TH_FLG, FP_FLG, AL_PC,&
                H_0, NTH, IOB(NTH), ZLS(1), IOB(1))

           EX_L(1) = 1.0D0 ;  DUM = 0.0D0
           DO I = 2, NLS
              DUM = DUM + AL_PC(I - 1)*LG%DS(2,I - 1) + AL_PC(I)*LG%DS(1,I) 
              EX_L(I) = DEXP(- DUM)
           END DO
           EX_1(0) = 1.0D0 ; DUM = 0.0D0
           DO I = 1, NLS
              DUM = DUM + AL_PC(I)*(LG%DS(1,I) + LG%DS(2,I)) 
              EX_1(I) = DEXP(- DUM)
           END DO
           AL_2 => AL_PC
        ELSE
           AL_2 => AL_1
        END IF  ! DO_PC
        
        DDF(:) = 0.d0  ;   BC_S1(:) = 0.d0 
        IF(DO_WF) THEN
           WF_TMP(:) = 0.0d0  ;  BC_WF(:) = 0.d0
        ENDIF

        IF( INT_MS_FLAG(JPOS) ) &
             CALL GET_SZA_IDX(LG%CG_SZA(IMS1:IMS2), LL(IMS1:IMS2))

        If(DO_WF) Call DS_FORM_3 (ZLS, LG%DS, IOB, JLS, A_UP, A_DN, DS_EXP)
        
        LOS_loop: Do ILS = 1, JLS
           !        LOS_loop: Do ILS = 1, NLS
           S_F = 0.d0  
!          Calculation of direct sunlight source function along LOS
!          ... Sign '-' is used because cosine of SZA(LG%C_SZA(:)) is positive
           Co_s = - LG%C_VA(ILS)*LG%C_SZA(ILS) + &
                LG%S_VA(ILS)*LG%S_SZA(ILS)*COS(LG%SAA(ILS))
           
        IF( Nvs > 1 ) THEN
             ! ... Vector case: source function 
           IF( USE_PSM ) THEN
              CALL SS_VEC_PSM   ! ===> P_FUNC(1:3)from scattering matrix
           ELSE
              CALL SS_VEC_EC    !  ===> P_FUNC(1:3)from expansion coeff.
           ENDIF
        ELSE
              ! ... Scalar case: source function 
           IF( USE_PSM ) THEN
              CALL SOURCE_SCAL_PSM(Co_s)  ! ===> P_FUNC(1)
           ELSE
              CALL SOURCE_SCAL(Co_s)  ! ===> P_FUNC(1)
           ENDIF
        ENDIF

           IF(DO_WF) THEN
              WF = 0.0D0
              IF(WF_MS_FLAG(JPOS)) WF_PS = 0.0D0
           END IF

           If(PreCalcAngs) DSB = LSG%DSB(ILS:ILS)
           CALL DirSun (1, LG%C_SZA(ILS:ILS), TH(ILS:ILS), ZLS(ILS:ILS), &
                IOB(ILS:ILS), AL_2(ILS:ILS), ADI%AL0_OUT, PCC, &
                SOL_EXP(ILS:ILS), LSG%REFR_DSB(ILS:ILS), PreCalcAngs, &
                DO_WF, DS_SUN, DSB)
           
!          ... No solar light in the case of isotropic illumination from below
           IF( DIFB ) SOL_EXP(ILS) = 0.d0
           
!          ... Contribution to single scattering
           SSSF(:) = SOL_EXP(ILS)*P_FUNC(:)/4.d0
!           write(6,*) ilS, SOL_EXP(ILS), P_FUNC(:)
           IF( DO_PLANCK ) SSSF(1) = SSSF(1) + Plf(ILS)

           IF(.NOT.INT_MS_FLAG(JPOS)) GOTO 998
           ! Skip multiple scattering if LOS is outside the valid rande of
           ! solar zenith angls
           IF(ILS < IMS1 .or. ILS > IMS2) GOTO 998 
           
          ! Interpolate Stokes vector on the solar angle: 
           CALL INTERP_SV(ILS)  ! Result in array FL(:,:,:)
           
           ! Calculation of the multiple scattering source function along LOS
           H1 = 0.d0  ;  NORM_PF = 0.d0
           IF(.NOT.DO_WF.OR..NOT.WF_MS_FLAG(JPOS)) THEN
              RelAz(:) = LG%SAA(ILS) - PHI_Poi(:)
              C_RA(:) = Cos(RelAz(:))
              C_SA(:) = MJU_Poi(:)*LG%C_VA(ILS) &
                   + SIN_Poi(:)*LG%S_VA(ILS)*C_RA(:) 
              Where(Abs(C_SA(:)) > 1) C_SA(:) = Sign(1.0D0,C_SA(:))

              IF( USE_HIN ) THEN
                 DO I = 1, NAz
                    DO KK = 1,Nvs
                       CALL LIN_INT(NVa, MJU_NODE(1:N_NODE:NAz), &
                            FL(KK,I:N_NODE:NAz,ILS),  &
                            NVa_HIN, MJU_HIN(1:N_HIN:NAz_HIN), &
                            FLL_Mu(KK,I:NAz*NVa_HIN:NAz), 'Z')
                    ENDDO  ! over Stokes components (KK)
                 ENDDO  ! over downward standard azimuthal nodes (I)                
                 DO I = 1, NVa_HIN
                    I1 = 1 + (I-1)*NAz  ;  I2 = I*NAz
                    DO KK = 1,Nvs
                       CALL LIN_INT(NAz, PHI_NODE(1:NAz), FLL_Mu(KK,I1:I2) ,  &
                            NAz_HIN, PHI_HIN(1:NAz_HIN), &
                            FLL(KK,1+(I-1)*NAz_HIN:I*NAz_HIN), 'Z')
                    ENDDO  ! over Stokes components (KK)
                 ENDDO  ! over downward standard zenith nodes (I)
!                 IF( ILS == 10.and.Iv==10 ) THEN
!                    WRITE(*,*) '   Azimuthal angles ', NAz, NAz_HIN 
!                    I1 = 10
!                    OPEN( 44, file='Befor.dat' )
!                    KK = 1
!                    DO I = I1, N_NODE, NAz
!                       WRITE(44,'(2x,I4,*(1x,e15.7))') KK, &
!                            MJU_NODE(I), FL(1,I,ILS)
!                       KK = KK + 1
!                    ENDDO
!                    CLOSE( 44 )
!                    OPEN( 44, file='After.dat' )
!                    KK = 1
!                    DO I = I1, N_HIN, NAz
!                       WRITE(44,'(2x,I4,*(1x,e15.7))') KK, &
!                            MJU_HIN(I), FLL_Mu(1,I)
!                       KK = KK + 1
!                    ENDDO
!                    CLOSE( 44 )
!                 ENDIF
              ELSE
                 FLL(:,:) = FL(:,:,ILS)
              ENDIF
!  ----------- Integration of product phase matrix times Stokes vector ----------
              DO ID_N = 1, N_Poi
                 IF( Nvs > 1 ) THEN
!                ...  Vector case ...
!                    Sv_in(:) = FL(:,ID_N,ILS)
                    Sv_in(:) = FLL(:,ID_N)
                    Mu_in = MJU_Poi(ID_N) ; Si_in = SIN_Poi(ID_N)
                    ! LG%C_VA(L) - direction of propagation w.r.t z-axis
                    Mu_sc = LG%C_VA(ILS) ; Si_sc = LG%S_VA(ILS)
                    Rel_az = RelAz(ID_N) ; Co_ls = C_SA(ID_N)
                    CosRA = C_RA(ID_N)
                    CALL SOURCE_VEC_MS                    
                    P_FUNC(1) = F_UL(1,1)
!                   =========================================================
                    IF( .NOT.Use_Sym ) THEN                       
                       HH(:) = Sv_mp(:)
                    ELSE
                       HH(1:2) = Sv_mp(1:2)
                       HH(3:4) = Sv_mp(3:4) /2.d0
!                       Sv_in(1:2) = FL(1:2,ID_N,ILS)
!                       Sv_in(3:4) = -FL(3:4,ID_N,ILS)
                       Sv_in(1:2) = FLL(1:2,ID_N)
                       Sv_in(3:4) = -FLL(3:4,ID_N)
                       Rel_az = LG%SAA(ILS) + PHI_Poi(ID_N) - 2.d0*Pie
                       CosRA = Cos(Rel_az)
                       Co_ls = Mu_in*Mu_sc + Si_in*Si_sc*CosRA 
                       If(Abs(Co_ls) > 1) Co_ls = Sign(1.0D0,Co_ls)
                       CALL SOURCE_VEC_MS
                       HH(3:4) = HH(3:4) + Sv_mp(3:4)/2.d0
                    ENDIF
!                   =========================================================
                 ELSE  
!                ...  Scalar case ...
                    Co_ls = MJU_Poi(ID_N)*LG%C_VA(ILS) + SIN_Poi(ID_N)  &
                         * LG%S_VA(ILS)*COS(PHI_Poi(ID_N) - LG%SAA(ILS))
                    IF( Co_ls > 1.0d0  ) Co_ls =  1.0d0
                    IF( Co_ls < -1.0d0 ) Co_ls = -1.0d0
                    IF( USE_PSM ) THEN
!                   ... Interpolation of precalculated phase function 
                       CALL SOURCE_SCAL_PSM(Co_ls)  ! ===> P_FUNC(1)
                    ELSE
!                   ... Calculation of phase function using expansion coefficients
                       CALL SOURCE_SCAL(Co_ls)
                    ENDIF
!                    HH(1) = P_FUNC(1)*FL(1,ID_N,ILS) 
                    HH(1) = P_FUNC(1)*FLL(1,ID_N) 
!                ...  End of scalar case ...
                 ENDIF
                 H1(:) = H1(:) + HH(:)*WEI_Poi(ID_N)
                 NORM_PF = NORM_PF + P_FUNC(1)*WEI_Poi(ID_N)
             END DO  ! over nodes (ID_N)
! ------------------------------------------------------------------------------
!             ... Multiple scattering source function:
              S_F(:) = H1(:)/NORM_PF*SC(ILS)           
!              WRITE(*,'(2x,F8.4,*(2x,ES12.4))')  &
!                   ZLS(ILS), S_F(1), H1(1)              
           ELSE    
!             ------- WFs ---------
              CALL WF_B1
           END IF
           
998        CONTINUE

           IF(DO_WF) THEN
              WF(:) = WF(:)/AL(:) - SSSF(1)*DS_SUN(1,:)
              ! WF(:) = WF(:) - (S_F(1) + SSSF(1))*DS_EXP(ILS,:)*AL(:)
              ! WF_TMP(:) = WF_TMP(:) + WF(ILS,:)*EX_L(ILS)*(LG%DS(1,ILS) + LG%DS(2,ILS))
              II = IOB(ILS)
              Associate(DEX => EX_1(ILS - 1) - EX_1(ILS), SF => S_F(1) + SSSF(1))
                WF(:) = WF(:)/AL_2(ILS)*DEX - &
                      SF/AL_2(ILS)*(EX_1(ILS - 1)*DS_EXP(ILS - 1,:) - EX_1(ILS)*DS_EXP(ILS,:))
                WF(II) = WF(II) - SF/AL_2(ILS)**2*A_DN(ILS)*DEX
                if(II /= 1) WF(II - 1) = WF(II - 1) - SF/AL_2(ILS)**2*A_UP(ILS)*DEX
              End Associate
              
              WF_TMP(:) = WF_TMP(:) + WF(:)*AL(:)
           END IF  ! DO_WF
!            S_F(:) = 0.d0  ! Remove multiple scattering
!            SSSF(:) = 0.d0 ! Remove single scattering
!            DEL(:) = (S_F(:) + SSSF(:)) &
!                *EX_L(ILS)*(LG%DS(1,ILS) + LG%DS(2,ILS))
           DEL(:) = (S_F(:) + SSSF(:))/AL_2(ILS)*(EX_1(ILS - 1) - EX_1(ILS))
           DDF(:) = DDF(:) + DEL(:)
           !           IF(IA == 1 .and. IV == 1) Then
           !IF(IA == 1 ) Then
           !   write(6,*) IA, IV,ILS, DEL(:), DDF(:)
           !End IF
!           WRITE(*,'(2x,A,*(1x,es12.4))') 'DEL', DEL(:) 
!           WRITE(*,'(2x,A,*(1x,es12.4))') 'DDF', DDF(:) 
           If( Abs(DEL(1)/DDF(1)) < Cr_Short_los/(NLS - ILS) ) then
!              WRITE(*,*) ' Out ILS = ', ILS, NLS
              Exit LOS_loop
           End If
!           End Block Integration
        End Do LOS_loop  ! over LoS (ILS)
        ! ********************************************************************
        
! ----------- Contribution of surface reflection ---------------------
!             ----------------------------------
        IF( ZLS(NLS) == Z_BOA .and. INT_MS_FLAG(JPOS)) Then
           CALL INTERP_SV(NLS) ! Interpolate Stokes vector => FL(:,:,NLS)
           IF( BRDF_FLAG ) THEN
              IF( USE_AIN ) THEN
                 CALL DIF_BR_AIN ! ==> BC_S1 using additional integration nodes
              ELSE
                 CALL DIF_BR     ! ==> BC_S1 
              ENDIF
!             ... Contribution of direct solar light into reflectance
              IF(LG%C_SZA(NLS) > 0.d0) THEN
                 CALL REFL_DIR_SPH(1,1,1, LG%C_VA(NLS:NLS), LG%C_SZA(NLS:NLS), &
                      LG%SAA(NLS:NLS), POS, DUM, FLU_Int, DUM, &
                      TE%PLANCK_SURF, DD3, DD4, Rx )
                 DO KK = 1,Nvs
                    BC_S1(KK) = BC_S1(KK) + &
                         Rx(KK,1)*LG%C_SZA(NLS)*SOL_EXP(NLS)
                 ENDDO
              END IF
           ELSE  ! Lambertian reflection
              BC_S1(:) = FL(:,1,NLS) !+ FLU_Int ! Surface reflection term
!              BC_S1(:) = 0.d0
           END IF ! BRDF reflection
                 
           IF(DO_WF.AND.WF_MS_FLAG(JPOS)) BC_WF(:) = WF_PS(:,1,NLS)*EX_L(NLS)

        END IF  ! ZLS(NLS).EQ.Z_BOA
! ----- -End contribution of surface reflection ---------------------

        RES%RAD(IS,IP,:) = DDF(:) + BC_S1(:)*EX_L(NLS)
!           WRITE(*,*) '  Contribution in RES%RAD: '
!           WRITE(*,*) '  Res: ', IS, IP, RES%RAD(IS,IP,:)
        IF(GeoD%Alt(IA) == Z_TOA.AND.CVA <= 0.0D0) THEN
           RES%RAD(IS,IP,:) = 0.0D0
        END IF
        IF(GeoD%Alt(IA) == Z_BOA.AND.CVA > 0.0D0) THEN
           RES%RAD(IS,IP,:)  = BC_S1(:)
        END IF
!        WRITE(*,'(2x,2(1x,I5),*(1x,es12.4))') IS, IP, RES%RAD(IS,IP,:) 
        IF( DO_WF ) THEN
           RES%WF(:,IS,IP) = WF_TMP(:) +  BC_WF(:) - &
                BC_S1(1)*DS_EXP(NLS,:)*AL(:)*EX_L(NLS)
        ENDIF             
!        WRITE(*,*) '    --------- New IP ------------ IP = ', IP+1
     END DO  ! over azimuthal angle (IP)

     Assign_CSZA_SolExp: Block
       IF(.NOT.PRESENT(CSZA)) Exit Assign_CSZA_SolExp
       IF(.NOT.ALLOCATED(CSZA)) Exit Assign_CSZA_SolExp
       IF(SIZE(CSZA,1) == 1 .AND. IA /= NLAYERS) Exit Assign_CSZA_SolExp
       CSZA(IA,IS) = LG%C_SZA(1)
       SolExp(IA,IS) = SOL_EXP(1)
     END Block Assign_CSZA_SolExp
!     WRITE(*,*) '    --------- New sol ------------ IS = ', IS+1
  END DO  ! over solar angles (IS)

  ! DeAllocate arrays
  DEALLOCATE(TH, SOL_EXP, P_FUNC, LL, EX_L, AL_1, SC, D_C, SCR)
  If(PreCalcGeom) Then
     Call Los_Geom_Clean (LG)
  Else
     Call Los_Geom_Clean (LG, LVG, LSG)
  End If
  
  IF(DO_WF) THEN
     DEALLOCATE(WF_TMP, WF, BC_WF, DS_SUN)
     IF(WF_MS_FLAG(JPOS)) DEALLOCATE(H2, WF_PS)
  END IF
  IF(DO_PC) DEALLOCATE(AL_PC)
  IF(INT_MS_FLAG(JPOS)) DEALLOCATE(H1, FL)
  IF(ALLOCATED(Y_TH)) DEALLOCATE(Y_TH)
  IF(ALLOCATED(Y_FP)) DEALLOCATE(Y_FP)
  IF( APH_TYPE /= 'mie' ) THEN
     DEALLOCATE(ASYM_COEFF, A_C_1, A_C_2)
  ELSE
     DEALLOCATE(SA)
  END IF
  IF( APH_TYPE=='none' ) DEALLOCATE(SA)
  IF( DO_CLOUD )  DEALLOCATE(Scl)
  IF( DO_PLANCK ) DEALLOCATE(Plf)
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
880 WRITE(EU, *) ' ---> Error in '//R_NAME
  ErrFlg = .True.
  RETURN
  
CONTAINS
! ********************************************************************************
! **************************************************************************
  SUBROUTINE DIF_BR_AIN
!   ... BRDF for a new downward grid
    DO I = 1, NbN
       Rel_az = LG%SAA(NLS) - PbN(I)
       IF( Rel_az < 0.d0 ) Rel_az =  Rel_az + 2.d0*Pie
       CALL REFL_DIR_SPH(1,1,1, LG%C_VA(NLS), VbN(I), &
            [Rel_az], POS, DUM, FLU_Int, DUM, TE%PLANCK_SURF, &
            DD3, DD4, Rx )
       D_REF(I,:,:) = Rx(:,:)
    ENDDO  ! over downward nodes (I)
    FLP => FL(:,1+N_PLUS:N_NODE,NLS)
!     Interpolate downward radiance at zenith angles for each 
!                 selected azimuthal angle
    DO I = 1, NAz
       DO KK = 1,Nvs
          CALL LIN_INT(NVa, MJU_MINUS(I:N_MINUS:NAz), &
               FLP(KK,I:N_MINUS:NAz),  &
               N_Va, Vz_A, FL_Mu(KK,I:NAz*N_VA:NAz), 'Z')
       ENDDO  ! over Stokes components (KK)
    ENDDO  ! over downward standard azimuthal nodes (I)
!     Interpolate downward radiance at azimuthal angles (scalar preliminary)
!                 for each selected zenith angle
    DO I = 1, N_Va
       I1 = 1 + (I-1)*NAz  ;  I2 = I*NAz
       DO KK = 1,Nvs
          CALL LIN_INT(NAz, PHI_MINUS(1:NAz), FL_Mu(KK,I1:I2) ,  &
               N_Az, AZ_A, FL_INT(KK,1+(I-1)*N_Az:I*N_Az), 'Z')
!         CALL AKIMA_INT(NAz, PHI_MINUS(1:NAz), FL_Mu(KK,I1:I2) ,  &
!              N_Az, AZ_A, FL_INT(KK,1+(I-1)*N_Az:I*N_Az))
       ENDDO  ! over Stokes components (KK)
    ENDDO  ! over downward standard zenith nodes (I)
    DO I = 1, NbN
       DO KK = 1,Nvs
          FLU(I,KK) = SUM( D_REF(I,KK,:)*FL_INT(:,I) )
       ENDDO  ! over Stokes components (KK)
    ENDDO  ! over downward nodes (I)
    DO KK = 1,Nvs
       BC_S1(KK) = SUM( FLU(:,KK)*WbN*VbN )/NORM_BN
    ENDDO
  END SUBROUTINE DIF_BR_AIN
! **************************************************************************

  SUBROUTINE DIF_BR
    DO I = 1, N_MINUS
       Rel_az = LG%SAA(NLS) - PHI_MINUS(I)
       IF( Rel_az < 0.d0 ) Rel_az =  Rel_az + 2.d0*Pie
       CALL REFL_DIR_SPH(1,1,1, LG%C_VA(NLS:NLS), MJU_MINUS(I:I), &
            [Rel_az], POS, DUM, FLU_Int, DUM, TE%PLANCK_SURF, &
            DD3, DD4, Rx )
       D_REF(I,:,:) = Rx(:,:)
    ENDDO  ! over downward nodes (I)                            
!             ... Product of BRDF and downward intensity at the surface
    DO I = 1, N_MINUS
       DO KK = 1,Nvs
          FLU(I,KK) = SUM( D_REF(I,KK,:)*FL(:,I+N_PLUS,NLS) )
       ENDDO  ! over Stokes components (KK)
    ENDDO  ! over downward nodes (I)
!        --------------------------------------------------------------
    IF( Use_Sym ) THEN
       D_REF = 0.d0
       FLU(:,:)  = FLU(:,:)/2.d0
       DO I = 1, N_MINUS
          Rel_az = LG%SAA(NLS) - (2.d0*Pie - PHI_MINUS(I))
          IF( Rel_az < 0.d0 ) Rel_az =  Rel_az + 2.d0*Pie
          CALL REFL_DIR_SPH(1,1,1, LG%C_VA(NLS:NLS), MJU_MINUS(I:I), &
               [Rel_az], POS, DUM, FLU_Int, DUM, TE%PLANCK_SURF, &
               DD3, DD4, Rx )
          D_REF(I,:,:) = Rx(:,:)
       ENDDO  ! over downward nodes (I)
       DO I = 1, N_MINUS
          Sv_in(1:2) =  FL(1:2,I+N_PLUS,NLS) 
          Sv_in(3:4) = -FL(3:4,I+N_PLUS,NLS) 
          DO KK = 1,Nvs
             FLU(I,KK) = FLU(I,KK) + &
                  SUM( D_REF(I,KK,:)*Sv_in(:) )/2.d0
          ENDDO  ! over Stokes components (KK)
       ENDDO  ! over downward nodes (I)
    ENDIF ! Use_Sym for LBC
!              --------------------------------------------------------------
!              ... Contribution from multiple scattering                 
    DO KK = 1,Nvs
       BC_S1(KK) = SUM( FLU(:,KK)*W_MINUS*MJU_MINUS )/NORM_DN
    ENDDO
  END SUBROUTINE DIF_BR
! **************************************************************************
              
  SUBROUTINE WF_B1    
    II = LL(ILS)
    IF (ONE_MS_SZ) THEN
       WF_PS(:,:,ILS) = WFPS(1)%RES(ISEL(ILS),:,:)
    ELSE
       SD = (LG%CG_SZA(ILS) - X_INV(II))/(X_INV(II+1) - X_INV(II))
       DO I = 1, NLAYERS
          WF_PS(I,:,ILS) = (1.0D0 - SD)*WFPS(II)%RES(ISEL(ILS),I,:) &
               + WFPS(II + 1)%RES(ISEL(ILS),I,:)*SD
       END DO
    END IF
           
    If(ILS == NTH.AND.TH_FLG) Then
       IF (ONE_MS_SZ) THEN
          WF_PS(:,:,NTH) = D_RAD_TH(:,II,:)
       ELSE
          SD = (LG%CG_SZA(NTH) - X_INV(II))/(X_INV(II+1) - X_INV(II))
          DO I = 1, NLAYERS
             WF_PS(I,:,NTH) = D_RAD_TH(I,II + 1,:)*SD + &
                  D_RAD_TH(I,II,:)*(1.0D0 - SD)
          END DO
       END IF
    END IF
           
    If(ILS == 1.AND.FP_FLG) Then
       IF (ONE_MS_SZ) THEN
          WF_PS(:,:,1) = D_RAD_FP(:,II,:)
       ELSE
          SD = (LG%CG_SZA(1) - X_INV(II))/(X_INV(II+1) - X_INV(II))
          DO I = 1, NLAYERS
             WF_PS(I,:,1) = D_RAD_FP(I,II + 1,:)*SD + &
                  D_RAD_FP(I,II,:)*(1.0D0 - SD)
          END DO
       END IF
    END IF

    H2 = 0.d0 
    DO ID_N = 1, N_NODE
       Co_ls = MJU_NODE(ID_N)*LG%C_VA(ILS) + SIN_NODE(ID_N) &
            *LG%S_VA(ILS)*COS(PHI_NODE(ID_N) - LG%SAA(ILS)) 
       CALL SOURCE_SCAL(Co_ls)
       H1(1) = H1(1) + P_FUNC(1)*FL(1,ID_N,ILS)*WEIGHTS(ID_N)
       NORM_PF = NORM_PF + P_FUNC(1)*WEIGHTS(ID_N)
       FORALL(I = 1:NLAYERS) H2(I) = H2(I) + P_FUNC(1)  &
            *WEIGHTS(ID_N)*WF_PS(I,ID_N,ILS)
    END DO   ! over nodes (ID_N)

    S_F(1) = H1(1)/NORM_PF*SC(ILS) 
    FORALL( I=1:NLAYERS )  WF(I) = H2(I)/NORM_PF*SC(ILS)

  END SUBROUTINE WF_B1
  
! ************************************************************************
!          Called in the vector case only 
! ************************************************************************
  SUBROUTINE SOURCE_VEC_MS
    
    If(PreCalcRtMatr) Then
       Co_ls = GeoD%LSG(IA,IV,IS,IP)%C_SA(ID_N,ILS)
       Si_s = GeoD%LSG(IA,IV,IS,IP)%S_SA(ID_N,ILS)
       RR1(:) = GeoD%LSG(IA,IV,IS,IP)%RMR(:,ID_N,ILS)
       RL1(:) = GeoD%LSG(IA,IV,IS,IP)%RML(:,ID_N,ILS)
    Else
       Si_s = SQRT(1.0d0 - Co_ls**2)
       CALL ROT_LSP 
    End If
    ! Rotate Stokes vector of the incident diffuse light 
    ! meridional plane ==> scattering plane
    Sv_sp(1) = Sv_in(1)  ;  Sv_sp(4) = Sv_in(4)
    Sv_sp(2) = RR1(1)*Sv_in(2) - RR1(2)*Sv_in(3)
    Sv_sp(3) = RR1(2)*Sv_in(2) + RR1(1)*Sv_in(3)

    IF( USE_PSM ) THEN  ! in "SOURCE_VEC_MS"
       CALL PSM_INTERP 
    ELSE
       CALL SCAT_MATR 
    ENDIF

    !   ... Stokes vector of the scattered light in the scattering plane
    FORALL( KK=1:2) Sv_sc(KK)   = SUM( F_UL(KK,:)*Sv_sp(1:2) )
    FORALL( KK=1:2) Sv_sc(KK+2) = SUM( F_DR(KK,:)*Sv_sp(3:4) )
    
!    ... Rotate Stokes vector of the scattered diffuse light
!        scattering plane ==> meridional plane
    Sv_mp(1) = Sv_sc(1) ;  Sv_mp(4) = Sv_sc(4)
    Sv_mp(2) = RL1(1)*Sv_sc(2) - RL1(2)*Sv_sc(3)
    Sv_mp(3) = RL1(2)*Sv_sc(2) + RL1(1)*Sv_sc(3)
    
  END SUBROUTINE SOURCE_VEC_MS
  
! ************************************************************************
  SUBROUTINE ROT_LSP 
    !   Called by "SOURCE_VEC_MS"
    REAL (KND)  Zn
!  ---------------------------------------------------------------------

    IF( Rel_az < 0.d0 ) Rel_az = 2.d0*Pie + Rel_az 

    IF( REAL(Rel_az)==REAL(Pie).OR.REAL(Rel_az)==0.0.OR.Si_s==0.d0 ) THEN 
!      ... Scattering plane is meridional plane: 
       RL1(1) = 1.d0 ; RL1(2) = 0.d0
       RR1(1) = 1.d0 ; RR1(2) = 0.d0 
       RETURN
    ENDIF
    IF( Rel_az > Pie ) THEN 
       Zn = 1.d0
    ELSE
       Zn = -1.d0
    ENDIF
    IF( REAL(ABS(Mu_sc)) == 1.0 ) THEN
       IF( REAL(ABS(Mu_in)) == 1.0 ) THEN
          T2 = 1.d0 ; T1 = 1.d0 
       Else
          T2 = -SIGN(1.d0,Mu_sc)*CosRA ; T1 = 1.d0
       ENDIF
    Else
       IF( REAL(ABS(Mu_in)) == 1.0 ) THEN
          T1 = -SIGN(1.d0,Mu_in)*CosRA ; T2 = 1.d0
       ELSE
          T1 = (Mu_sc - Mu_in*Co_ls)/Si_in/Si_s  ! cos( sigma_1 )
          T2 = (Mu_in - Mu_sc*Co_ls)/Si_sc/Si_s  ! cos( sigma_2 )
       End If
    End If

!   ... Elements (2,2) and (2,3) of rotation matrix
    RR1(1) = 2.d0*T1*T1 - 1.d0    ! 2*cos^2( 2*sigma_1 ) - 1
    RR1(2) = Zn* 2.d0*SQRT( ABS(1.d0 - T1*T1) )*T1

    RL1(1) = 2.d0*T2*T2 - 1.d0    ! 2*cos^2( 2*sigma_2 ) - 1
    RL1(2) = Zn* 2.d0*SQRT( ABS(1.d0 - T2*T2) )*T2

  END SUBROUTINE ROT_LSP
! **************************************************************************

  SUBROUTINE INTERP_SV(ILS)
    INTEGER  ILS
    REAL (KND) SD2, SD3
    REAL (KND), POINTER:: Arr(:,:,:)
! ---------------------------------------------------------------------
    II = LL(ILS)
    Select_Array: Block
      If(ILS == NTH.AND.TH_FLG) Then
         Arr => Y_TH(:,:,:,II)
         Exit Select_Array
      End If
      If(ILS == 1.AND.FP_FLG) Then
         Arr => Y_FP(:,:,:,II)
         Exit Select_Array
      End If
      Arr => ADI%Ys_ALL(:,:,:,II,ISEL(ILS))
    End Block Select_Array

    FL(:,:,ILS) = Arr(:,:,1)
    IF (.NOT.ONE_MS_SZ) THEN
       SD = LG%CG_SZA(ILS) - X_INV(II)
       FL(:,:,ILS) = FL(:,:,ILS) + Arr(:,:,2)*SD
       Select Case (MSF_INT)
       Case ("LogLin")
          FL(1,:,ILS) = Exp(FL(1,:,ILS))
       Case ("Akima","Spline")
          SD2 = SD*SD ; SD3 = SD2*SD
          FL(:,:,ILS) = FL(:,:,ILS) + Arr(:,:,3)*SD2 + Arr(:,:,4)*SD3
       End Select
    END IF
!    WRITE(*,'(2x,I3,*(1x,ES13.5))') ILS, FL(1,1:5,ILS) 
    Return
  END SUBROUTINE INTERP_SV
  
!***************************************************************************
  SUBROUTINE SOURCE_SCAL(Co_gam)

    USE AEROSOLS,    ONLY: IDX_AER
    USE ST_VARS,     ONLY: RTM_CORE
!  -------------------------------------------------------------------------
    REAL (KND)  Co_gam
    ! Local parameters
    INTEGER     J,  Npl
    REAL (KND)  Shg
    REAL (KND), ALLOCATABLE :: Plm_ss(:)
    REAL (KND), POINTER :: COF(:,:), P_MOM(:)
  !==================================================================

!   ... Contribution of Rayleigh scattering 
    IF( TRIM(RTM_CORE)=='CDI' ) THEN
       P_FUNC(1) = (D_C(1,ILS) + D_C(2,ILS)*Co_gam*Co_gam)
    ELSE
       ALLOCATE( Plm_ss(N_MOM) )
!      ... Legendre polynomials for single scattering angles along L-of-S
       CALL WIGNER_PLG(Co_gam, 1, 0, N_MOM, Plm_ss)
!       write(*,*) '   Plm_ss for N_MOM = ', N_MOM
       Npl = Nlg_Ray 
       P_MOM => LOA%RAY_MOMS(1:Npl,1) ; P_FUNC = 0.d0
       P_FUNC(1) = SUM( Plm_ss(1:Npl)*P_MOM(:) )*SCR(ILS)
    ENDIF

    IF( (.NOT.DO_AEROSOL).AND.(.NOT.DO_CLOUD) )  RETURN
!   ========================================================================
    
    IF( .NOT.ALLOCATED(Plm_ss) ) THEN
       ALLOCATE( Plm_ss(N_MOM) )
       CALL WIGNER_PLG(Co_gam, 1, 0, N_MOM, Plm_ss)
    ENDIF

    AEROSOL_MIE: Block
      IF( .NOT.DO_AEROSOL ) EXIT AEROSOL_MIE
      IF( APH_TYPE/='mie' ) THEN
         Shg = A_C_1(ILS) - ASYM_COEFF(ILS)*Co_gam
         P_FUNC(1) =  P_FUNC(1) + A_C_2(ILS)/Shg/SQRT(Shg)
         EXIT AEROSOL_MIE
      ENDIF
!     ... Aerosol phase function using all expansion coefficients:
      COF => OSM%AER_ALL(:,:,1)
      P_FUNC(1) = P_FUNC(1) &
           + SUM(Plm_ss(:)*COF(:,IDX_AER(IOB(ILS))))*SA(ILS)
    END Block AEROSOL_MIE

    IF( DO_CLOUD ) THEN  ! In "SOURCE_SCAL"
       Ci1 = INT( CLA(IOB(ILS),2) )  
       Npl = Leg_cld
!      ... Use expansion coefficients:
       IF( Ci1 /= 0 ) THEN
          P_FUNC(1) = P_FUNC(1) + Scl(ILS)*SUM(   &
               Plm_ss(1:Npl)*OSM%CLD_ALL(:,Ci1,1,1,1) )
       ENDIF
    ENDIF  ! DO_CLOUD in "SOURCE_SCAL"

    RETURN
  END SUBROUTINE SOURCE_SCAL
! -----------------------------------------------------------------------------
  SUBROUTINE SOURCE_SCAL_PSM(Co_gam)
    REAL (KND)  Co_gam, R_Sum
    REAL (KND), POINTER :: P_MOM(:)
!  =============================================================================
    R_Sum = (DEP_CF(1) + DEP_CF(2)*Co_gam*Co_gam)*SCR(ILS)
    IF( DO_AEROSOL ) THEN
       IF( SA(ILS) > 1.d-20 ) THEN
          II = IDX_AER(IOB(ILS))
          I1 = FLOOR((Co_gam + 1)/E_step) + 1
          I2 = MIN(I1 + 1,ND_ANG)
          DL = (Co_gam - OSM%S_AN(I1))/E_step
          R_Sum = R_Sum + SA(ILS)  &
               *(OSM%AER_SCM(I2,II,1)*DL + OSM%AER_SCM(I1,II,1)*(1.d0-DL))
       ENDIF
    ENDIF
    IF( DO_CLOUD ) THEN  ! In "SOURCE_SCAL"
       Ci1 = INT( CLA(IOB(ILS),2) )  
       IF( Ci1 /= 0 ) THEN
          I1 = FLOOR((Co_gam + 1)/E_step) + 1
          I2 = MIN(I1 + 1,ND_ANG)
          DL = (Co_gam -  OSM%S_AN(I1))/E_step
          P_MOM => OSM%CLD_LOC(:,Ci1,1,1,1) 
          R_Sum = R_Sum + Scl(ILS)  &
               *( P_MOM(I2)*DL + P_MOM(I1)*(1.d0 - DL) )
       ENDIF
    ENDIF
    P_FUNC(1) = R_Sum
  END SUBROUTINE SOURCE_SCAL_PSM
! -----------------------------------------------------------------------------
  
!***************************************************************************
  SUBROUTINE SS_VEC_EC
    USE AEROSOLS,    ONLY: IDX_AER
!  -------------------------------------------------------------------------
  ! Local parameters
    INTEGER     Npl, Ai1
    REAL (KND)  Azm, Zn, Xmu, Mu0, Rot_l(2)
    REAL (KND), ALLOCATABLE :: Plm_as(:), Plm_ss(:)
    REAL (KND), POINTER :: P_MOM(:), PM1(:,:), PM5(:,:)
  !==================================================================
    ALLOCATE( Plm_ss(N_MOM), Plm_as(N_MOM) )

!   ... Polynomials for all scattering angles along L-of-S
    CALL WIGNER_PLG(Co_s, 1, 0, N_MOM, Plm_ss)
    CALL WIGNER_PLG(Co_s, 1, 2, N_MOM, Plm_as)

!   .......... Calculation of rotation matrix ................
    Rot_l = 0.d0
!      Mu and Mu0 have to be correspond to the tau-axis
    Xmu = -Lg%C_VA(ILS) ; Mu0 = LG%C_SZA(ILS) ; Azm = LG%SAA(ILS)
    IF( REAL(Azm)==REAL(Pie) .OR. REAL(Azm)==0.0) THEN 
       Rot_l(1) = 1.d0 ; Rot_l(2) = 0.d0
    ENDIF
    IF( Azm > Pie ) THEN 
       Zn = 1.d0
    ELSE
       Zn = -1.d0
    ENDIF
    COSIG: Block
      IF( REAL(ABS(Xmu)) == 1.0.AND.REAL(ABS(Mu0)) == 1.0 ) THEN
         T2 = 1.d0 ;                    EXIT COSIG
      ENDIF
      IF( REAL(ABS(Xmu)) == 1.0 ) THEN
         T2 = SIGN(1.d0,Xmu)*COS(Azm) ; EXIT COSIG
      ENDIF
      IF( REAL(ABS(Mu0)) == 1.0 ) THEN
         T2 = 1.d0 ;                    EXIT COSIG
      ENDIF
      T2 = ( -Mu0 + Xmu*Co_s )/SQRT( (1.d0 - Xmu*Xmu)*(1.d0 - Co_s**2) )
    END Block COSIG
    Rot_l(1) = 2.d0*T2*T2 - 1.d0 
    Rot_l(2) = Zn*2.d0*SQRT( ABS(1.d0 - T2*T2) )*T2
!   .... End of calculations rotation matrix ..... 

    Npl = Nlg_Ray  ;  P_FUNC = 0.d0
    P_MOM => LOA%RAY_MOMS(1:Npl,1)
    P_FUNC(1) = SUM( Plm_ss(1:Npl)*P_MOM(:) )*SCR(ILS)
    P_FUNC(2:3) = Rot_l(:)*SCR(ILS)   &
         *SUM( Plm_as(1:Npl)*LOA%RAY_MOMS(1:Npl,5))

    IF( DO_AEROSOL ) THEN ; Npl = Leg_aer 
       PM1 => OSM%AER_ALL(:,:,1) ; PM5 => OSM%AER_ALL(:,:,5)
       Ai1 = IDX_AER(IOB(ILS))
       P_FUNC(1) = P_FUNC(1) + SA(ILS)*SUM(Plm_ss(1:Npl)*PM1(:,Ai1)) 
       P_FUNC(2:3) = P_FUNC(2:3) + Rot_l(:)*SA(ILS)  &
               *SUM( Plm_as(1:Npl)*PM5(:,Ai1) )
    ENDIF

    IF( DO_CLOUD ) THEN  ;  Npl = Leg_cld 
       PM1 => OSM%CLD_ALL(:,:,1,1,1) ; PM5 => OSM%CLD_ALL(:,:,1,1,5)
       Ci1 = INT( CLA(IOB(ILS),2) )
       IF( Ci1 /= 0 ) THEN
          P_FUNC(1) = P_FUNC(1) + Scl(ILS)*SUM(Plm_ss(1:Npl)*PM1(:,Ci1))
          P_FUNC(2:3) = P_FUNC(2:3) + Rot_l(:)*Scl(ILS) &
               *SUM( Plm_as(1:Npl)*PM5(:,Ci1) )
       ENDIF
    ENDIF  ! DO_CLOUD in "SS_VEC_EC" 

    RETURN
  END SUBROUTINE SS_VEC_EC
! **********************************************************************

  SUBROUTINE SS_VEC_PSM
    USE AEROSOLS,    ONLY: IDX_AER
!  -------------------------------------------------------------------------
  ! Local parameters
    REAL (KND)  Azm, Zn, Xmu, Mu0, TT1
    REAL (KND)  R_Sum(3), Rot_l(2)
    REAL (KND), POINTER :: P_MOM(:)
!==================================================================
!   .......... Calculation of rotation matrix ................
    Rot_l = 0.d0
!      Mu and Mu0 have to be correspond to the tau-axis
    Xmu = -Lg%C_VA(ILS) ; Mu0 = LG%C_SZA(ILS) ; Azm = LG%SAA(ILS)

    IF( REAL(Azm)==REAL(Pie) .OR. REAL(Azm)==0.0) THEN 
       Rot_l(1) = 1.d0 ; Rot_l(2) = 0.d0
    ENDIF
    IF( Azm > Pie ) THEN 
       Zn = 1.d0
    ELSE
       Zn = -1.d0
    ENDIF
    COSIG: Block
      IF( REAL(ABS(Xmu)) == 1.0.AND.REAL(ABS(Mu0)) == 1.0 ) THEN
         T2 = 1.d0 ;                    EXIT COSIG
      ENDIF
      IF( REAL(ABS(Xmu)) == 1.0 ) THEN
         T2 = SIGN(1.d0,Xmu)*COS(Azm) ; EXIT COSIG
      ENDIF
      IF( REAL(ABS(Mu0)) == 1.0 ) THEN
         T2 = 1.d0 ;                    EXIT COSIG
      ENDIF
      T2 = ( -Mu0 + Xmu*Co_s ) &
           /SQRT( (1.d0 - Xmu*Xmu)*(1.d0 - Co_s**2) )
    END Block COSIG
    Rot_l(1) = 2.d0*T2*T2 - 1.d0 
    Rot_l(2) = Zn*2.d0*SQRT( ABS(1.d0 - T2*T2) )*T2
!   .... End of calculations rotation matrix ..... 

!   ... Rayleigh contribution to the SS source function
    R_Sum(1)   = (DEP_CF(1) + DEP_CF(2)*Co_s*Co_s)*SCR(ILS)
    R_Sum(2:3) = -Rot_l(:)*DEP_CF(2)*(1.d0 - Co_s*Co_s)*SCR(ILS)
    
!   ... Aerosol contribution to the SS source function
    IF( DO_AEROSOL ) THEN
       IF( SA(ILS) > 1.d-20 ) THEN
          II = IDX_AER(IOB(ILS))
          I1 = FLOOR((Co_s + 1)/E_step) + 1
          I2 = MIN(I1 + 1,ND_ANG)
          DL = (Co_s - OSM%S_AN(I1))/E_step
          R_Sum(1) = R_Sum(1) + SA(ILS)  &
               *(OSM%AER_SCM(I2,II,1)*DL + OSM%AER_SCM(I1,II,1)*(1.d0-DL))
          R_Sum(2:3) = R_Sum(2:3) + SA(ILS)*Rot_l(:)  &
               *(OSM%AER_SCM(I2,II,5)*DL + OSM%AER_SCM(I1,II,5)*(1.d0-DL)) 
       ENDIF
    ENDIF  ! DO_AEROSOL

!   ... Cloud contribution to the SS source function
    IF( DO_CLOUD ) THEN  ! In "SS_VEC_PSM"
       Ci1 = INT( CLA(IOB(ILS),2) )
       IF( Ci1 /= 0 ) THEN
          I1 = FLOOR((Co_s + 1)/E_step) + 1
          I2 = MIN(I1 + 1,ND_ANG)
          DL = (Co_s - OSM%S_AN(I1))/E_step
          P_MOM => OSM%CLD_LOC(:,Ci1,1,1,1) 
          R_Sum(1) = R_Sum(1) + &
               Scl(ILS)*(P_MOM(I2)*DL + P_MOM(I1)*(1.d0-DL))
          P_MOM => OSM%CLD_LOC(:,Ci1,1,1,5) 
          R_Sum(2:3) = R_Sum(2:3) + &
               Scl(ILS)*Rot_l(:)*(P_MOM(I2)*DL + P_MOM(I1)*(1.d0-DL))
       ENDIF
    ENDIF  ! DO_CLOUD in "SS_VEC_PSM"

    P_FUNC = 0.d0
    P_FUNC(1:3) = R_Sum(1:3)
    RETURN
  END SUBROUTINE SS_VEC_PSM
! **********************************************************************

  SUBROUTINE SCAT_MATR 
! Called by "SOURCE_VEC_MS"
    
    USE AEROSOLS,  ONLY: IDX_AER
    USE CLOUDS,    ONLY: CLA => CLD_LAYER_ARR
!  -------------------------------------------------------------------------
    INTEGER Nk
    REAL (KND) A_COF(Leg_aer,Nm_vs)
!  *************************************************************************
    
!   ... Max. number of expansion coefficients (Rayleigh, aerosol, cloud) 
    Nk = N_MOM  ;  Ci1 = 0
    IF( DO_CLOUD ) Ci1 = INT( CLA(IOB(ILS),2) )
!   ... There is Rayleigh scattering only in this altitude layer:
    IF( (SA(ILS)<1.d-20).AND.(Ci1==0) ) Nk = Nlg_Ray
    
!   ... Wigner functions for all scattering angles
    CALL WIGNER_PLG      (Co_ls, 1, 0, Nk, Plm_ss )
    CALL WIGNER_PLG      (Co_ls, 1, 2, Nk, Plm_as)
    CALL WIGNER_FUNCTION (Co_ls, 1, 2, Nk, Dp2_ss, Dm2_ss)

    E_c = 0.d0
!   ... Set Rayleigh:
    E_c(1:Nlg_Ray,:) = LOA%RAY_MOMS(1:Nlg_Ray,:)*ADI%SCR_OUT(ISEL(ILS))
    
!   ... Aerosol contribution:
    IF( DO_AEROSOL ) E_c(1:Nk,:) = E_c(1:Nk,:) &
         + OSM%AER_ALL(1:Nk,IDX_AER(IOB(ILS)),:) *SA(ILS)
    
!   ... Cloud contribution:
    IF( Ci1 /= 0 )  E_c(1:Nk,:) = E_c(1:Nk,:) + Scl(ILS)   &
               *OSM%CLD_ALL(1:Nk,Ci1,1,1,:)
       
    T1 = SUM( (E_c(1:Nk,2) + E_c(1:Nk,3))*Dp2_ss(1:Nk) )
    T2 = SUM( (E_c(1:Nk,2) - E_c(1:Nk,3))*Dm2_ss(1:Nk) )

!   ...  Calculate elements of the scattering matrix ...
!   ... Upper left 2X2 block of scattering matrix       
    F_UL(1,1) = SUM( Plm_ss(1:Nk)*E_c(1:Nk,1) ) 
!   ... The sign '-' is not used because coefficients beta1 
!       has negative sign after input. 
    F_UL(1,2) = SUM( Plm_as(1:Nk)*E_c(1:Nk,5) ) ! element F12
    F_UL(2,1) = F_UL(1,2)                  ! element F21 
    F_UL(2,2) =  0.5d0*( T1 + T2 )     ! element F22

!   ... Down right 2X2 block of scattering matrix       
    F_DR(1,1) =  0.5d0*( T1 - T2 )  ! element F33
!   ... The sign '-' is not used because coefficients beta2
!       has negative sign after input. 
    F_DR(1,2) = SUM( Plm_as(1:Nk)*E_c(1:Nk,6) ) ! element F34
    F_DR(2,1) = -F_DR(1,2)                  ! element F43
    F_DR(2,2) = SUM( Plm_ss(1:Nk)*E_c(1:Nk,4) ) ! element F44        

  END SUBROUTINE SCAT_MATR
  
! ***************************************************************************
  SUBROUTINE PSM_INTERP 
! Called by "SOURCE_VEC_MS"
    
    USE AEROSOLS,    ONLY: IDX_AER
!  -------------------------------------------------------------------------
    LOGICAL AER_FLG
    REAL (KND) RAY_SCA, R_RAY(6), Co_ls_2

    RAY_SCA = ADI%SCR_OUT(ISEL(ILS))

    AER_FLG = .TRUE.
    IF( SA(ILS) < 1.d-20 ) AER_FLG = .FALSE.

    Co_ls_2 = Co_ls*Co_ls
    R_RAY(1) =  DEP_CF(1) + DEP_CF(2)*Co_ls_2
    R_RAY(2) =  DEP_CF(2)*(1.d0 + Co_ls_2)
    R_RAY(3) =  DEP_CF(2)*2.d0*Co_ls
    R_RAY(4) =  DEP_CF(3)*Co_ls
    R_RAY(5) = -DEP_CF(2)*Si_s*Si_s
    R_RAY(6) =  0.d0
    R_RAY(:) = R_RAY(:)*RAY_SCA

    IF( DO_AEROSOL.OR.DO_CLOUD ) THEN
       I1 = FLOOR((Co_ls + 1)/E_step) + 1
       I2 = MIN(I1 + 1,ND_ANG)
       DL = (Co_ls -  OSM%S_AN(I1))/E_step
    ENDIF

    Ci1 = 0
    IF( DO_CLOUD ) Ci1 = INT( CLA(IOB(ILS),2) )

    IF( DO_AEROSOL.AND.AER_FLG ) THEN
       II = IDX_AER(IOB(ILS))
       R_RAY(:) = R_RAY(:) + SA(ILS) &
            *( OSM%AER_SCM(I2,II,:)*DL + OSM%AER_SCM(I1,II,:)*(1.d0-DL) )
    ENDIF

    IF( Ci1/=0 ) R_RAY(:) = R_RAY(:) + Scl(ILS) &
         *(OSM%CLD_LOC(I2,Ci1,1,1,:)*DL         &
         + OSM%CLD_LOC(I1,Ci1,1,1,:)*(1.d0-DL))

    F_UL(1,1) =  R_RAY(1)   ;  F_UL(1,2) = R_RAY(5) 
    F_UL(2,1) =  F_UL(1,2)  ;  F_UL(2,2) = R_RAY(2)
    F_DR(1,1) =  R_RAY(3)   ;  F_DR(1,2) = R_RAY(6)  
    F_DR(2,1) = -F_DR(1,2)  ;  F_DR(2,2) = R_RAY(4)

  END SUBROUTINE PSM_INTERP

! ***************************************************************************
  Subroutine InterpToAlt (NP, Alt, YY, D_RAD_NP)
    USE GRIDS, ONLY: Z_FINE
    INTEGER NP, KK, NS, II, Ks
    REAL (KND) Alt, AL0_NP, SCR_NP, SCA_NP, ASC_NP, SCC_NP, Plf_NP, DD
    REAL (KND), ALLOCATABLE :: D_RAD_NP(:,:,:), YY(:,:,:,:)

    KK = NLAYERS - IOB(NP) + 1
    IF(INT_MS_FLAG(JPOS)) THEN
       ALLOCATE (YY(Nvs,N_NODE,4,NSOL))
       CALL BIAKIMA_Y_V2( Z_INV(KK), SZA_EXT, ALT, ADI%Xs_RAD(KK,:,:,:,:), &
               YY(:,:,:,:))
    END IF

    II = NLAYERS - IOB(NP) + 1
    CALL AKIMA_TH_FINE(Z_INV(II), Alt, ADI%X_AL(II ,:), ADI%X_SCR(II ,:), &
         ADI%X_SCA(II,:), ADI%X_ASC(II,:), ADI%X_SCC(II,:,1), ADI%X_Plf(II,:), &
         AL0_NP, SCR_NP, SCA_NP, ASC_NP, SCC_NP, Plf_NP)
    
    AL_1(NP) = AL0_NP ;  D_C(1:2,NP) = DEP_CF(1:2)*SCR_NP ;  SC(NP) = SCR_NP
    
    IF(DO_AEROSOL) THEN
       IF( APH_TYPE /= 'mie' ) THEN
          ASYM_COEFF(NP) = 2.0D0*ASC_NP ;  A_C_1(NP) = 1.0D0 + ASC_NP**2
          A_C_2(NP) = SCA_NP*(1.0D0 - ASC_NP**2)
       ELSE
          SA(NP) = SCA_NP
       END IF
       SC(NP) = SC(NP) + SCA_NP
    END IF
    IF( DO_PLANCK ) Plf(NP) = Plf_NP
    IF( DO_CLOUD ) THEN
       Scl(NP) = SCC_NP
       SC(NP) = SC(NP) + SCC_NP
    ENDIF

    IF(DO_WF.AND.WF_MS_FLAG(JPOS)) THEN
       NS = SIZE(WFPS)
       ALLOCATE(D_RAD_NP(NLAYERS, NS, N_NODE))
       DD = (Alt - Z_FINE(ISEL(NP)))/(Z_FINE(ISEL(NP) + 1) - Z_FINE(ISEL(NP)))
       FORALL(II = 1:NS) D_RAD_NP(:,II,:) = &
            WFPS(II)%RES(ISEL(NP) + 1,:,:)*DD + &
            WFPS(II)%RES(ISEL(NP),:,:)*(1.0D0 - DD)
    END IF
    
    RETURN
  END Subroutine InterpToAlt
  
!***************************************************************************
END SUBROUTINE Integrate_Los

!***************************************************************************
FUNCTION PHAS_FUNC (ASYM_COEFF, A_C_1, A_C_2,  DEP_COEFF_1, DEP_COEFF_2, C_G)
  USE TYPES,  ONLY: KND
  REAL (KND)  PHAS_FUNC, C_G, ASYM_COEFF, S, A_C_1, A_C_2, DEP_COEFF_1, &
       DEP_COEFF_2
!---------------------------------------------------------------------------
  S = A_C_1 - ASYM_COEFF*C_G
  PHAS_FUNC = (DEP_COEFF_1 + DEP_COEFF_2*C_G*C_G) + A_C_2/S/DSQRT(S)
  
  RETURN
END FUNCTION PHAS_FUNC
!***************************************************************************
