!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 30 May 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
! Called by "integrate_init.f
SUBROUTINE Integrate_Occ (IA, IV, GeoD, AL, ADI, PCC, RES, DO_WF)

  USE ABS_COEFF_PC, ONLY: PC_ABS_COEFF
  USE GRIDS,        ONLY: NLAYERS, ZZZ, Z_INV, INTERP_IDX
  USE ST_VARS,      ONLY: DO_PC, F_LGI_EXT
  USE TYPES,        ONLY: KND, PCC_TYPE, Geometry_Data, AuxDataInterp, &
                          RES_DATA

  INTEGER     IA, IV
  LOGICAL     DO_WF
  REAL (KND), DIMENSION (NLAYERS) :: AL
  TYPE (PCC_type) :: PCC    
  TYPE (AuxDataInterp) ADI
  TYPE (Geometry_Data), TARGET :: GeoD
  TYPE (RES_DATA) RES
  
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'Integrate_Occ'
  INTEGER  I, NLS, NTH, II
  INTEGER, POINTER :: ISEL(:), IOB(:)
  LOGICAL  TH_FLG, FP_FLG
  REAL (KND), ALLOCATABLE :: DS_EXP(:), AL_1(:), C_SZA(:), DS(:,:)
  REAL (KND), POINTER :: ZLS(:)
  REAL (KND)  H_0, EX_L, DD
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  H_0 = GeoD%LVG(IA,IV)%H_0 ; IOB => GeoD%LVG(IA,IV)%IOB
  NLS = GeoD%LVG(IA,IV)%NLS ;  NTH = GeoD%LVG(IA,IV)%NTH
  TH_FLG = GeoD%LVG(IA,IV)%TH_FLG ;  FP_FLG = GeoD%LVG(IA,IV)%FP_FLG
  ZLS => GeoD%LVG(IA,IV)%ZLS ;  ISEL => GeoD%LVG(IA,IV)%ISEL
  

  ALLOCATE(AL_1(NLS), DS(2,NLS), C_SZA(NLS))
  C_SZA(:) = GeoD%LVG(IA,IV)%C_VA(:)
  DS(:,:) = GeoD%LVG(IA,IV)%DS(:,:)
     
  IF(.NOT.DO_PC) THEN
     AL_1(:) = ADI%AL0_OUT(ISEL(:))
     IF(FP_FLG) THEN
        II = INTERP_IDX(ISEL(1))
        DD = ZLS(1) - Z_INV(II)
        AL_1(1) = ADI%X_AL(II,1) + ADI%X_AL(II,2)*DD + &
             ADI%X_AL(II,3)*DD**2 + ADI%X_AL(II,4)*DD**3
        IF(F_LGI_EXT) AL_1(1) = EXP(AL_1(1))
     END IF
     IF(TH_FLG) THEN
        II = INTERP_IDX(ISEL(NTH))
        DD = H_0 - Z_INV(II)
        AL_1(NTH) = ADI%X_AL(II,1) + ADI%X_AL(II,2)*DD + &
             ADI%X_AL(II,3)*DD**2 + ADI%X_AL(II,4)*DD**3
        IF(F_LGI_EXT) AL_1(NTH) = EXP(AL_1(NTH))
     END IF
  ELSE
     CALL PC_ABS_COEFF (ISEL, C_SZA, NLS, PCC, TH_FLG, FP_FLG, AL_1, &
          H_0, NTH, IOB(NTH), ZLS(1), IOB(1))
  END IF

  EX_L = 0.0D0
  DO I = 2, NLS
     EX_L = EX_L+ AL_1(I - 1)*DS(2,I - 1) + AL_1(I)*DS(1,I) 
  END DO
  EX_L = DEXP(- EX_L)
  
  RES%RAD(1,1,1) = EX_L
  IF(DO_WF)  THEN
     ALLOCATE(DS_EXP(NLAYERS)) ! A
     CALL DS_FORM_2 (NLAYERS, ZZZ, ZLS, DS, IOB, NLS, DS_EXP)
     RES%WF(:,1,1)  = - EX_L*DS_EXP(:)*AL(:)
  END IF

  ! DeAllocate arrays
  DEALLOCATE(DS, C_SZA, AL_1)
  IF(DO_WF) DEALLOCATE(DS_EXP)
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE Integrate_Occ
!***************************************************************************
