C********************************************************************C
C
C       SCIATRAN - Version 3.3 - Last modification: 8 Jan 2013
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE AL_INTERP (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT, 
     $     AKIMA_FLAG, *)

      USE LOG_CONTROL, ONLY : ERRORUNIT
      
      CHARACTER (LEN = 9), PARAMETER :: ROUTINE_NAME='AL_INTERP'

      INTEGER             N_IN, N_OUT
      LOGICAL             AKIMA_FLAG
      DOUBLE PRECISION    X_IN(N_IN), Y_IN(N_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT)

C     Extrapolation is performed with zeros

      SELECT CASE(N_IN)
      CASE(3:)
         IF (AKIMA_FLAG) THEN
            CALL AKIMA_INT(N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT)  
         ELSE
            CALL LIN_INT (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT, 'Z')
         END IF
      CASE(2)
         CALL LIN_INT (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT, 'Z')
      CASE DEFAULT
         GOTO 200
      END SELECT

      RETURN

 200  CALL WRITE_MESSAGE (ROUTINE_NAME, 'ERROR', 
     $     'Not enough points for interpolation !')
      WRITE(ERRORUNIT, *) '    Akima flag:', AKIMA_FLAG
      WRITE(ERRORUNIT, *) '    Number of input grid points:', N_IN
      RETURN 1

      END

C********************************************************************C
      SUBROUTINE LIN_INT (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT, FLAG)
C     FLAG defines the extrapolation type:
C     'Z' - extrapolation with zero, 'C' - extrapolation with constant,
C     any other character - linear extrapolation
      
      CHARACTER (LEN = 1) FLAG
      INTEGER             I, N_IN, N_OUT, IDX_X, IDX_X1
      DOUBLE PRECISION    X_IN(N_IN), Y_IN(N_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT), T
      INTENT (IN) :: N_IN, X_IN, Y_IN, N_OUT, X_OUT, FLAG

      DO I = 1, N_OUT
         CALL GET_IDX (X_OUT(I), N_IN, X_IN, IDX_X, IDX_X1)

         IF(X_OUT(I).GT.MAXVAL(X_IN).OR.X_OUT(I).LT.MINVAL(X_IN)) THEN
            SELECT CASE (FLAG)
            CASE('Z','z') 
               Y_OUT(I) = 0.0D0
               CYCLE
            CASE('C','c')
               Y_OUT(I) = Y_IN(IDX_X)
               CYCLE
            END SELECT
         END IF
         
         T = (X_OUT(I) - X_IN(IDX_X))/(X_IN(IDX_X1) - X_IN(IDX_X))
         Y_OUT(I) = (1.0D0 - T)*Y_IN(IDX_X) + T*Y_IN(IDX_X1)
      END DO

      RETURN
      END

C********************************************************************C
      SUBROUTINE L_INT (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT, FLAG, 
     $     SORTED)
C     FLAG defines the extrapolation type:
C     'Z' - extrapolation with zero, 'C' - extrapolation with constant,
C     any other character - linear extrapolation
C     If "SORTED" is ".FALSE." the X-axis does not need to increase or 
C     decrease monotonically. 
      
      CHARACTER (LEN = 1) FLAG
      INTEGER             I, N_IN, N_OUT, IDX_X, IDX_X1
      LOGICAL             SORTED
      DOUBLE PRECISION    X_IN(N_IN), Y_IN(N_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT), T
      PROCEDURE( ), POINTER :: PTR_TO_SUBR => NULL()
      INTENT (IN) :: N_IN, X_IN, Y_IN, N_OUT, X_OUT, FLAG, SORTED
      INTERFACE
         SUBROUTINE GET_IDX_UNSRT (VAL, N, AXIS, IDX, IDX_1)
         USE TYPE_VARS,  ONLY: KND
         INTEGER    N, IDX, IDX_1
         REAL (KND) VAL, AXIS(N)
         END SUBROUTINE 
         SUBROUTINE GET_IDX (VAL, N, AXIS, IDX, IDX_1)
         USE TYPE_VARS,  ONLY: KND
         INTEGER    N, IDX, IDX_1
         REAL (KND) VAL, AXIS(N)
         END SUBROUTINE 
      END INTERFACE

      IF(SORTED) THEN
         PTR_TO_SUBR => GET_IDX
      ELSE
         PTR_TO_SUBR => GET_IDX_UNSRT
      END IF

      DO I = 1, N_OUT
         CALL PTR_TO_SUBR (X_OUT(I), N_IN, X_IN, IDX_X, IDX_X1)

         IF(X_OUT(I).GT.MAXVAL(X_IN).OR.X_OUT(I).LT.MINVAL(X_IN)) THEN
            SELECT CASE (FLAG)
            CASE('Z','z') 
               Y_OUT(I) = 0.0D0
               CYCLE
            CASE('C','c')
               Y_OUT(I) = Y_IN(IDX_X)
               CYCLE
            END SELECT
         END IF

         T = (X_OUT(I) - X_IN(IDX_X))/(X_IN(IDX_X1) - X_IN(IDX_X))
         Y_OUT(I) = (1.0D0 - T)*Y_IN(IDX_X) + T*Y_IN(IDX_X1)
      END DO

      RETURN
      END

C*********************************************************************
      SUBROUTINE LIN_INT_OLD (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT)

      INTEGER             I, N_IN, N_OUT, IDX, IDX_1

      DOUBLE PRECISION    X_IN(N_IN), Y_IN(N_IN), X1(N_IN), Y1(N_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT), S

      IF (X_IN(2).GT.X_IN(1)) THEN
         X1(1:N_IN) = X_IN(1:N_IN) ;  Y1(1:N_IN) = Y_IN(1:N_IN)
      ELSE
         X1(1:N_IN) = X_IN(N_IN:1:-1) ;  Y1(1:N_IN) = Y_IN(N_IN:1:-1)
      ENDIF
      
      DO I = 1, N_OUT
         IDX = MINLOC(DABS(X1(:) - X_OUT(I)),1)
         IDX_1 = IDX + INT(SIGN(1.0D0,(X_OUT(I) - X1(IDX))))
         IF(IDX_1 < 1 .OR. IDX_1 > N_IN) THEN
            IF(X_OUT(I).EQ.X1(IDX)) THEN
               Y_OUT(I) = Y1(IDX)
            ELSE
               Y_OUT(I) = 0.0D0
            END IF
            CYCLE
         END IF
         S = (X_OUT(I) - X1(IDX_1))/(X1(IDX) - X1(IDX_1))
         Y_OUT(I) = Y1(IDX)*S + Y1(IDX_1)*(1.0D0 - S)
      END DO

      RETURN
      END

C********************************************************************C
      SUBROUTINE AKIMA_INT (N_IN, X_IN, Y_IN, N_OUT, X_OUT, Y_OUT)

      INTEGER             N_IN, N_OUT, I, IDX
      LOGICAL             MASK(N_IN)
      DOUBLE PRECISION    X_IN(N_IN), Y_IN(N_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT), DELTA, Hf
      DOUBLE PRECISION, DIMENSION (N_IN - 1) :: A, B, C, D
      DOUBLE PRECISION, DIMENSION (N_IN + 4) :: X1, Y1, ST, T

      IF (X_IN(2).GT.X_IN(1)) THEN
         X1(3:N_IN + 2) = X_IN(1:N_IN)
         Y1(3:N_IN + 2) = Y_IN(1:N_IN)
      ELSE
         X1(3:N_IN + 2) = X_IN(N_IN:1:-1)
         Y1(3:N_IN + 2) = Y_IN(N_IN:1:-1)
      ENDIF

      A(:) = 0.0D0 ; B(:) = 0.0D0 ; C(:) = 0.0D0 ; D(:) = 0.0D0

      X1(2) = X1(3) + X1(4) - X1(5)
      X1(1) = X1(2) + X1(3) - X1(4)

      X1(N_IN + 3) = X1(N_IN + 2) + X1(N_IN + 1) - X1(N_IN)
      X1(N_IN + 4) = X1(N_IN + 3) + X1(N_IN + 2) - X1(N_IN + 1)

c     Caculation of slopes of curves
      FORALL(I = 3:N_IN + 1) 
     $     ST(I) = (Y1(I + 1) - Y1(I))/(X1(I + 1) - X1(I))

c     Calculation of extrapolated y-values left and right
      Y1(2) = (X1(3) - X1(2)) * (ST(4) - 2.0D0*ST(3)) + Y1(3)
      ST(2) = (Y1(3) - Y1(2))/(X1(3) - X1(2))

      Y1(1) = (X1(2) - X1(1)) * (ST(3) - 2.0D0*ST(2)) + Y1(2)
      ST(1) = (Y1(2) - Y1(1))/(X1(2) - X1(1))

      Y1(N_IN + 3) = (2.0D0*ST(N_IN + 1) - ST(N_IN))*
     $     (X1(N_IN + 3) - X1(N_IN + 2)) + Y1(N_IN + 2)
      ST(N_IN + 2) = (Y1(N_IN + 3) - Y1(N_IN + 2))/
     $     (X1(N_IN + 3) - X1(N_IN + 2))

      Y1(N_IN + 4) = (2.0D0*ST(N_IN + 2) - ST(N_IN + 1))*
     $     (X1(N_IN + 4) - X1(N_IN + 3)) + Y1(N_IN + 3)
      ST(N_IN + 3) = (Y1(N_IN + 4) - Y1(N_IN + 3))/
     $     (X1(N_IN + 4) - X1(N_IN + 3))

c     Calculation of polynomial slopes
      DO I = 3, N_IN + 2
         Hf = DABS(ST(I + 1) - ST(I)) + DABS(ST(I - 1) - ST(I - 2))
c         IF ( ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1) ) THEN
         IF( Hf == 0.d0 ) THEN
            T(I) = 0.5D0 * (ST(I + 1) + ST(I))
         ELSE
            T(I) = ( DABS(ST(I + 1) - ST(I))*ST(I - 1) +
     $               DABS(ST(I - 1) - ST(I - 2))*ST(I) ) / Hf
         END IF
      END DO

c     Calculation of polynomial coefficients
      FORALL(I = 1:N_IN - 1)
         A(I) = Y1(I + 2)
         B(I) = T(I + 2)
         C(I) = ( 3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3) )/
     $        ( X1(I + 3) - X1(I + 2) )
         D(I) = ( T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2) )/
     $        ( X1(I + 3) - X1(I + 2) )**2
      END FORALL

c  Calculation of interpolated values
      DO I = 1, N_OUT
         MASK(:) =.FALSE.
         WHERE(X1(3:N_IN + 2) <= X_OUT(I)) MASK(:) =.TRUE.
         IDX = COUNT(MASK)
         IF(IDX == 0 .OR. IDX == N_IN) THEN
            IF(X_OUT(I).EQ.X1(IDX + 2)) THEN
               Y_OUT(I) = Y1(IDX + 2)
            ELSE
               Y_OUT(I) = 0.0D0
            END IF
            CYCLE
         END IF
         DELTA = X_OUT(I) - X1(IDX + 2)
         Y_OUT(I) = A(IDX) + B(IDX)*DELTA + C(IDX)*DELTA**2 + 
     $        D(IDX)*DELTA**3
      END DO

      RETURN
      END

C     *****************************************************************
      SUBROUTINE BILIN_INT (NX_IN, X_IN, NY_IN, Y_IN, Z_IN, N_OUT, 
     $     X_OUT, Y_OUT, Z_OUT)

      INTEGER             I, NX_IN, NY_IN, N_OUT
      INTEGER             IDX_X, IDX_X1, IDX_Y, IDX_Y1

      DOUBLE PRECISION    X_IN(NX_IN), Y_IN(NY_IN), Z_IN(NX_IN, NY_IN)
      DOUBLE PRECISION    X_OUT(N_OUT), Y_OUT(N_OUT), Z_OUT(N_OUT), T, U

      DO I = 1, N_OUT
         IF(X_OUT(I).GT.MAXVAL(X_IN).OR.X_OUT(I).LT.MINVAL(X_IN)) THEN
            Z_OUT(I) = 0.0D0
            CYCLE
         END IF
         
         IF(Y_OUT(I).GT.MAXVAL(Y_IN).OR.Y_OUT(I).LT.MINVAL(Y_IN)) THEN
            Z_OUT(I) = 0.0D0
            CYCLE
         END IF

         CALL GET_IDX (X_OUT(I), NX_IN, X_IN, IDX_X, IDX_X1)
         CALL GET_IDX (Y_OUT(I), NY_IN, Y_IN, IDX_Y, IDX_Y1)

         T = (X_OUT(I) - X_IN(IDX_X))/(X_IN(IDX_X1) - X_IN(IDX_X))
         U = (Y_OUT(I) - Y_IN(IDX_Y))/(Y_IN(IDX_Y1) - Y_IN(IDX_Y))

         Z_OUT(I) = (1.0D0 - T)*(1.0D0 - U)*Z_IN(IDX_X,IDX_Y) + 
     $        T*(1.0D0 - U)*Z_IN(IDX_X1,IDX_Y) +
     $        (1.0D0 - T)*U*Z_IN(IDX_X,IDX_Y1) +
     $        T*U*Z_IN(IDX_X1,IDX_Y1)
      END DO

      RETURN
      END

C********************************************************************C
      SUBROUTINE GET_IDX (VAL, N, AXIS, IDX, IDX_1)
      INTEGER             N, IDX, IDX_1
      DOUBLE PRECISION    VAL, AXIS(N)
     
      IDX = MINLOC(DABS(AXIS(:) - VAL),1)
      IF(IDX == 1) THEN
         IDX_1 = 2
         RETURN
      END IF
      IF(IDX == N) THEN
         IDX_1 = N - 1
         RETURN
      END IF
      IDX_1 = IDX + INT(SIGN(1.0D0,
     $     (VAL - AXIS(IDX))/(AXIS(IDX + 1) - AXIS(IDX))))

      RETURN
      END 

C********************************************************************C
      SUBROUTINE GET_IDX_UNSRT (VAL, N, AXIS, IDX, IDX_1)
      USE TYPE_VARS,  ONLY: KND
      INTEGER    N, IDX, IDX_1
      LOGICAL    MASK(N)
      REAL (KND) VAL, AXIS(N)

      MASK = .FALSE.      
      WHERE(AXIS(:) <= VAL) MASK = .TRUE.

      IF(ALL(.NOT.MASK). OR. ALL(MASK)) THEN
         IDX = MINLOC(DABS(AXIS(:) - VAL),1)
         MASK = .TRUE. ;  MASK(IDX) = .FALSE.
         IDX_1 = MINLOC(DABS(AXIS(:) - VAL),1,MASK)
         RETURN
      END IF
      
      IDX = MINLOC(DABS(AXIS(:) - VAL),1,MASK)
      MASK = .FALSE.      
      WHERE(AXIS(:) > VAL) MASK = .TRUE.
      IDX_1 = MINLOC(DABS(AXIS(:) - VAL),1,MASK)

      RETURN
      END 
C********************************************************************C

