C********************************************************************C
C
C       SCIATRAN - Version 4.0 - Last modification: 29 Nov 2018
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE IOP_IFACE(AGR_A, AGR_W, GSL_A, GSL_W, LAS_A, LAS_W, 
     $     LOA, LOW, SDO_A, SDO_W, IOP_A, IOP_W, Obs_POSITION) 
c     ... Call in 'st_dom_tau.f'

      USE GRIDS,       ONLY: DZR
      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: Wup, N_OP      
      USE ST_VARS,     ONLY: N_RHS, GeoD => GeoD_OUT
      USE SPHER_PARS,  ONLY: LOS_DEF_FLAG
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LAS_TYPE, LOA_TYPE, LOW_TYPE, GSL_TYPE, 
     $                       IOP_TYPE, SDO_TYPE, AGR_type
      USE WATER,       ONLY: UWT, ZZZ_UW
c     ------------------------------------------------------------------

      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of water
      TYPE (LAS_TYPE) :: LAS_A, LAS_W ! Linear algebraic system
      TYPE (GSL_type) :: GSL_A, GSL_W ! Angles
      TYPE (IOP_type), TARGET :: IOP_A, IOP_W
      TYPE (SDO_type) :: SDO_A, SDO_W ! DOM auxiliary solution variables
      TYPE (AGR_type) :: AGR_A, AGR_W
c     -------------------------------
c     ... Local variables
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'IOP_IFACE'
      CHARACTER (LEN=5) Obs_POSITION(N_OP)
      LOGICAL     ADAPTIVE_GRID
      INTEGER     I, J, K, IA, NrL, NL, NNv, S_AG, E_AG, Jco, Jro
      REAL (KND)  Wdo
      COMPLEX (kind(0.0d0)) :: Cmu
      INTEGER, POINTER :: LAY_GR(:,:), IDU(:), IDL(:), NcC(:), Id_a(:), 
     $     Id_b(:)
      REAL (KND), POINTER  :: AL(:), ALW(:), T_AU(:), DTR(:), Nu_j(:),
     $     DTR_U(:), DTR_D(:), Ex_U(:,:), Ex_D(:,:), Nuj_Lay(:,:), 
     $     Po_dz(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1 ; Wdo = 1.d0 - Wup
      IF( .NOT.UWT ) THEN
c        ... RT in atmosphere
c        --------------------
         CALL DEFINE_A   ;   CALL MAKE
      ELSE  ! UWT = .TRUE.
c        ... Coupled RT model: upward radiation
c        --------------------------------------
         IF( LOS_DEF_FLAG=='n.a.') THEN  ! for option 'flux'
            CALL DEFINE_A  ;  CALL MAKE
            NULLIFY( T_AU, IDU, IDL, Po_DZ )
            CALL DEFINE_O  ;  CALL MAKE
         ELSE
            SELECT CASE( GeoD%View(IA)%S )
            CASE('up')
               SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
c           ... Observer in the ocean
            CASE('Ocean')
               CALL DEFINE_O  ;  CALL MAKE
c              ... If weighting function in the atmosphere needed
               NULLIFY( T_AU, IDU, IDL, Po_DZ )
               CALL DEFINE_A  ;  CALL MAKE
c           ... Observer in the atmosphere
            CASE('Atmos')
               CALL DEFINE_A  ;  CALL MAKE
               NULLIFY( T_AU, IDU, IDL, Po_DZ )
               CALL DEFINE_O  ;  CALL MAKE
            END SELECT
c           ... Coupled RT model: downward radiation
c           ----------------------------------------
            CASE('down')
               SELECT CASE( TRIM(Obs_POSITION(1)) )
c           ... Observer in the atmosphere
               CASE('Atmos')
                  CALL DEFINE_A  ;  CALL MAKE
c           ... Observer in the ocean
               CASE('Ocean')
                  CALL DEFINE_O  ;  CALL MAKE
                  NULLIFY( T_AU, IDU, IDL, Po_DZ )
                  CALL DEFINE_A  ;  CALL MAKE
               END SELECT
            END SELECT
         ENDIF  ! LOS_DEF_FLAG
      ENDIF  ! UWT

      NULLIFY( LAY_GR, Nu_j, AL, ALW, NcC, 
     $     DTR, DTR_U, DTR_D, Ex_U, Ex_D, Nuj_Lay )
      DEALLOCATE( T_AU, IDU, IDL, Po_DZ )

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE DEFINE_A
c       ... Set known parameters of ocean
        AL      => LOA%AL       ; NNv     =  GSL_A%NN*Nvs
        ADAPTIVE_GRID = AGR_A%ADAPTIVE_GRID
        NL      =  AGR_A%NL     ; NrL     =  AGR_A%NrL  
        LAY_GR  => AGR_A%LAY_GR ; Nuj_lay => SDO_A%Nuj_lay
        NcC     => SDO_A%NcC(:) 

c       ... These parameters will be calculated by "make.f"
        Id_a    => IOP_A%Id_a   ; Id_b    => IOP_A%Id_b
        DTR_U   => IOP_A%DTR_U  ; DTR_D   => IOP_A%DTR_D
        Ex_U    => IOP_A%Ex_U   ; Ex_D    => IOP_A%Ex_D
        DTR     => IOP_A%DTR    ; ALW     => IOP_A%ALW 
        ALLOCATE( T_AU(NL), IDU(NL-1), IDL(NL-1), Po_DZ(NL) )
        Po_DZ(:) = DZR(:)
        IF( ADAPTIVE_GRID ) THEN
           DO K = 1,NRL-1
              DO J = 1, N_RHS
                 DO I = LAY_GR(K,1), LAY_GR(K,2)
                    IOP_A%MVEC(2*NNv*(I-1)+1:2*NNv*I,J) 
     $                   = LAS_A%SVEC_AG(2*NNv*(K-1)+1:2*NNv*K,J)
                 ENDDO
              END DO
           ENDDO
        ELSE
           IOP_A%MVEC = LAS_A%SVEC_AG
        ENDIF
      END SUBROUTINE DEFINE_A
c **********************************************************************

      SUBROUTINE DEFINE_O
c       ... Set known parameters of ocean
        AL      => LOW%AL_W     ; NNv     =  GSL_W%NN*Nvs 
        ADAPTIVE_GRID = AGR_W%ADAPTIVE_GRID
        NL      =  AGR_W%NL     ; NrL     =  AGR_W%NrL  
        LAY_GR  => AGR_W%LAY_GR ; Nuj_lay => SDO_W%Nuj_lay
        NcC     => SDO_W%NcC(:)          

c       ... These parameters will be calculated by "make.f"
        Id_a    => IOP_W%Id_a   ; Id_b    => IOP_W%Id_b
        DTR_U   => IOP_W%DTR_U  ; DTR_D   => IOP_W%DTR_D
        Ex_U    => IOP_W%Ex_U   ; Ex_D    => IOP_W%Ex_D
        DTR     => IOP_W%DTR    ; ALW     => IOP_W%ALW 
        ALLOCATE( T_AU(NL), IDU(NL-1), IDL(NL-1), Po_DZ(NL) )
        Po_DZ(1:NL-1) = ZZZ_UW(2:NL) - ZZZ_UW(1:NL - 1)
        IF( ADAPTIVE_GRID ) THEN
           DO K = 1,NRL-1
              DO J = 1, N_RHS
                 DO I = LAY_GR(K,1), LAY_GR(K,2)
                    IOP_W%MVEC(2*NNv*(I-1)+1:2*NNv*I,J) 
     $                   = LAS_W%SVEC_AG(2*NNv*(K-1)+1:2*NNv*K,J)
                 ENDDO
              END DO
           ENDDO
        ELSE
           IOP_W%MVEC = LAS_W%SVEC_AG
        ENDIF
      END SUBROUTINE DEFINE_O
c **********************************************************************

      SUBROUTINE MAKE
c     ... Weighted extinction coefficient for all layers
      ALW(:) = Wup*AL(1:NL-1) + Wdo*AL(2:NL)
c     ... Optical depth       
      T_AU(1) = 0.D0 
      DO J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + ALW(J)*Po_DZ(J)
      ENDDO
c     ... Layers optical thickness
      DTR(1) = 0.D0 ;  DTR(2:NL) = T_AU(2:NL) - T_AU(1:NL-1)

      DO K = 1, NrL-1
         S_AG = LAY_GR(K,1)    ; E_AG = LAY_GR(K,2)
         IDU(S_AG:E_AG) = S_AG ; IDL(S_AG:E_AG) = E_AG+1
      ENDDO
c     If index "L" is used for the layers of the adaptive grid, and 
c        "p" for the sub-layers of the initial grid then: 
c     DTR_U corresponds to: Tau(p-1) - Tau(L-1)
c     DTR_D corresponds to: Tau(L) - Tau(p). 
c     ... They will be used to calculate auxiliary exponents, i.e., 
c         Ex_U and Ex_D which are in the first terms of the 
c         expression for functions Raj(mu) and Rbj(mu) in 
c         Eqs.(Raj-mu) and (Rbj-mu) in STB (SFI reflection).

      DO I = 1,NL-1
         DTR_U(I) = T_AU(I)      - T_AU(IDU(I))
         DTR_D(I) = T_AU(IDL(I)) - T_AU(I+1)
      ENDDO

      Id_a = (/(I, I = 1,NL-1)/)  ;  Id_b = Id_a
      DO I = 1,NrL - 1
         S_AG = LAY_GR(I,1) ; E_AG = LAY_GR(I,2) + 1 ! Old levels         
         Id_a(S_AG) = 1 ; Id_b(E_AG-1) = -1
      ENDDO
c     
      IF( ADAPTIVE_GRID ) THEN
c         WRITE(*,*) '   Number of layers: ', NL
         DO I = 1,NL-1
            Jco = NcC(I) ; Jro = NNv - 2*Jco ; Nu_j => Nuj_lay(:,I)
            Ex_U(1:Jro,I) = DEXP( -DTR_U(I)/Nu_J(1:Jro) )
            Ex_D(1:Jro,I) = DEXP( -DTR_D(I)/Nu_J(1:Jro) )
            DO J = Jro+1, Jro+2*Jco, 2
               Cmu = EXP( -DCMP(DTR_U(I))/DCMP(Nu_j(J),Nu_j(J+1)) )
               Ex_U(J,I)   = DBLE ( Cmu ) 
               Ex_U(J+1,I) = AIMAG( Cmu )

               Cmu = EXP( -DCMP(DTR_D(I))/DCMP(Nu_j(J),Nu_j(J+1)) )
               Ex_D(J,I)   = DBLE ( Cmu ) 
               Ex_D(J+1,I) = AIMAG( Cmu )
            ENDDO
c            WRITE(*,*) I, Ex_U(Jro,I), Ex_D(Jro,I)
         ENDDO         

      ELSE
         DO I = 1,NL-1
            Jco = NcC(I)  ;  Jro = NNv - 2*Jco
            Ex_U(1:Jro,I) = 1.d0 ; Ex_D(1:Jro,I) = 1.d0
            DO K = Jro+1,Jro+2*Jco,2 
               Ex_U(K,I) = 1.d0 ; Ex_U(K+1,I) = 0.d0
               Ex_D(K,I) = 1.d0 ; Ex_D(K+1,I) = 0.d0
            ENDDO
         ENDDO
      ENDIF
      END SUBROUTINE MAKE

      END
c **********************************************************************
