C********************************************************************C
C
C	SCIATRAN - Version 3.8 - Last modification: 22 Jan 2018
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ITERATE(N_RET, N_DATA, YDIFF, KN, XDIFF, S_Y, S_A_INV,
c     $     S_A, RMS2, CONSTR_ARR, *)
     $     S_A, RMS1, RMS2, CONSTR_ARR, *)
c     Called by retrieval.f90
      USE DATA_RET,  ONLY: DO_IMAT, DO_CONSTRAIN, MIN_EVAL
      USE OutInf,    ONLY: EU, LUNIT, LOG_LEVEL, N_RSO, N_SSO,
     $                     N_SSO, N_ESO, DIR_OUT, C_UNIT
      USE RETR_ITER, ONLY: ITERATION
      Use Tools,     Only: MatrInvers, OpenOut, WriteOut, CloseOut
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER  N_RET, N_DATA
      REAL (KND) KN(N_DATA,N_RET), XDIFF(N_RET), CONSTR_ARR(2,N_RET)
      REAL (KND), DIMENSION (N_DATA) :: YDIFF, S_Y
      REAL (KND), DIMENSION (N_RET,N_RET) :: S_A_INV, S_A
      REAL (KND) RMS1, RMS2
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'ITERATE'
      CHARACTER (LEN=12) OUT_NAME
      Character (Len=50000)  OutStr
      INTEGER, PARAMETER :: FACTOR = 34
      INTEGER  I, J, KK, IRET, INFO, LWORK, Unt
      LOGICAL  EVAL_FLAG(N_RET)
      REAL (KND) SUM0, SUM1, TERMB(N_DATA), S1
      REAL (KND) KNTISY(N_RET,N_DATA), WORK(FACTOR*N_RET), VL(1,N_RET),
     $     EVEC_ALL(N_RET,N_RET), VECTOR(N_RET*(N_RET + 1)/2)
      REAL (KND), DIMENSION (N_RET) :: AV, EVAL, EVAL_ALL, RI, 
     $     KNTISYTMB, XDIFF_SAVE
      REAL (KND), DIMENSION (N_RET,N_RET) :: IMAT, KNTISYKN, ITERMD, 
     $     S_RE, SCF!, S_NS
      REAL (KND), ALLOCATABLE :: A_NO(:), EVEC(:,:), BETA(:), 
     $     BETA_CONSTRAIN(:)
c      REAL (KND), ALLOCATABLE :: RES(:), EV_SPEC(:,:), C_CORR(:) 
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      LWORK = FACTOR*N_RET
      XDIFF_SAVE = XDIFF

      IF (LOG_LEVEL(2) >= N_SSO) THEN
         write (LUNIT,*) ' '
         write (LUNIT,*) 
     $        ' ****************************************************** '
         write (LUNIT,*) 
     $        ' *************** Main inversion routine ***************'
         write (LUNIT,*) 
     $        ' ******************************************************'
         write (LUNIT,*) ' '
      END IF

      WRITE(*,*) '    N_Y = ', N_DATA
      WRITE(*,*) YDIFF(1:N_DATA)

      RMS1 = DSQRT(DOT_PRODUCT(YDIFF(1:N_DATA),YDIFF(1:N_DATA))/N_DATA)
      
c  ---------------------------------------------------------------------
c  Calculate new estimate for state vector xnp1 using n-form of
c  optimal estimation formula:
c  ---------------------------------------------------------------------
c     termb = y - yn + kn * xdiff:
      TERMB = YDIFF + MATMUL(KN, XDIFF)
      
c     kntisy = kn^t * sy^-1:
      FORALL(I = 1:N_RET) KNTISY(I,:) = KN(:,I)/S_Y(:)

c     kntisykn = kntisy * kn = kn^t * sy^-1 * kn (N_RET,N_RET):
      KNTISYKN = MATMUL(KNTISY,KN)

c     itermd = (sainv + kntisy * kn)^-1 = (sa^-1 + kn^t * sy^-1 * kn)^-1:

!      FORALL (I = 1:N_RET) VECTOR(I*(I - 1)/2 + 1:I*(I + 1)/2) = 
!     $     S_A_INV(I,1:I) + KNTISYKN(I,1:I)
!      CALL CHINV3 (N_RET*(N_RET+1)/2,N_RET,VECTOR)
!      FORALL (I = 1:N_RET)  
!     $     ITERMD(I,1:I) = VECTOR(I*(I - 1)/2 + 1:I*(I + 1)/2)
!      FORALL (I = 1:N_RET, J = 1:N_RET, J > I) ITERMD(I,J) = ITERMD(J,I)
      S1 = 0.d0
      do i = 1, N_RET
         S1 = S1 + KNTISYKN(I,I)
      enddo
c      WRITE(*,*) '   Sled_K-1 :', S1/N_RET

      ITERMD = S_A_INV + KNTISYKN
      SCF = 0.0D0
      DO I = 1, N_RET
         SCF(I,I) = 1.0D0/MAXVAL(ITERMD(I,:))
      END DO
      ITERMD = MATMUL(ITERMD,SCF)
      CALL MatrInvers (ITERMD)
      ITERMD = MATMUL(SCF,ITERMD)
       
c  Calculate kntisytmb = kntisy * termb (vector: N_RET):
      KNTISYTMB = MATMUL(KNTISY,TERMB)

c  Random error: S_RE = itermd*kntisykn*itermd
      S_RE = MATMUL(ITERMD,MATMUL(KNTISYKN,ITERMD)) 
c      S_NS = MATMUL(ITERMD,MATMUL(S_A_INV,ITERMD)) 

c  Calculate xdiff = itermd * kntisytmb (vector: N_RET):
      XDIFF = MATMUL(ITERMD,KNTISYTMB)
c      XDIFF = MATMUL(MATMUL(ALPHA_MAT,ITERMD),KNTISYTMB)

c      XDIFF = MATMUL(ITERMD,KNTISYTMB) + 
c     $     MATMUL(ITERMD,MATMUL(ALPHA_MAT,XDIFF_SAVE))

c      XDIFF_SAVE = 0.0D0
c  Calculate averaging kernel av = itermd * kntisykn:
      Call OpenOut ('precision.dat', Unt)
      DO I = 1, N_RET
         WRITE(OutStr,'(I0, *(2x,ES15.7E3))') 
     $        I, S_RE(I,I), S_A(I,I), ITERMD(I,I), 
     $        (S_A(I,I) - ITERMD(I,I))/S_A(I,I)*100.
         Call WriteOut (Unt, OutStr)
      END DO
      Call CloseOut (Unt)

      IF( ITERATION==1 ) THEN  ;  Call OpenOut ('av1.dat', Unt)
      ELSE  ;  Call OpenOut ('av_test.dat', Unt) 
      ENDIF
      DO I = 1, N_RET
         FORALL (J = 1:N_RET) 
     $        AV(J) = DOT_PRODUCT(ITERMD(I,:),KNTISYKN(:,J))
         write(OutStr,'(*(2x,ES15.7E3))') (AV (J), J= 1, N_RET)
         Call WriteOut (Unt, OutStr)
      END DO
      Call CloseOut (Unt)
      
      Call OpenOut ('covar_matr.dat', Unt)
      DO I = 1, N_RET
         write(OutStr,'(*(2x,ES15.7E3))') (S_RE(I,J), J= 1, N_RET)
         Call WriteOut (Unt, OutStr)
      END DO
      Call CloseOut (Unt)

c  ---------------------------------------------------------------------
c  ---------------------------------------------------------------------
c  The following lines are for the informationmatrix retrieval only!

      IF ( DO_IMAT ) THEN
c  Calculate information matrix imat = s_a * kntisykn (N_RET,N_RET):
         IMAT = MATMUL(S_A,KNTISYKN)
c  Calculate Eigenvectors and Eigenvalues larger than 1 of imat:
         EVAL_FLAG = .FALSE. 
         CALL DGEEV('N', 'V', N_RET, IMAT, N_RET, EVAL_ALL, RI, VL, 1,  
     $        EVEC_ALL, N_RET, WORK, LWORK, INFO )
         IF (INFO.NE.0) THEN
            CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Eigenvectors '
     $           //'of the information matrix can not be obtained')
            WRITE(EU,*) 
     $           '    Information matrix --> INFO = ', INFO
            RETURN 1
         END IF
         WHERE(DABS(EVAL_ALL) >= MIN_EVAL) EVAL_FLAG = .TRUE.
         IRET = COUNT(EVAL_FLAG)
         ALLOCATE(EVEC(N_RET,IRET), BETA_CONSTRAIN(IRET), BETA(IRET), 
     $        A_NO(IRET))
         EVAL = PACK(EVAL_ALL,EVAL_FLAG)
         DO I = 1, N_RET
            EVEC(I,:) = PACK(EVEC_ALL(I,:),EVAL_FLAG)
         END DO
         IF (LOG_LEVEL(2) >= N_ESO) THEN 
            OPEN(C_UNIT, file = DIR_OUT//'/evec.out')
            WRITE(C_UNIT,*) IRET, N_RET
            DO J = 1, IRET
               WRITE(C_UNIT,*) EVAL (J)
               DO I = 1, N_RET
                  WRITE(C_UNIT,*) EVEC(I,J)
               END DO
            END DO
            CLOSE(C_UNIT)
         END IF
         IF (LOG_LEVEL(2) >= N_RSO) THEN 
            WRITE(LUNIT,*)
            WRITE(LUNIT,100) IRET
            WRITE(LUNIT,101) MAXVAL(DABS(EVAL(1:IRET))), 
     $           MINVAL(DABS(EVAL(1:IRET)))
            WRITE(LUNIT,*)
 100        FORMAT(2x, ' *** Total number of eigen vectors :',
     $           3x, I2)
 101        FORMAT(2x, ' *** Maximal/minimal  eigen value :', 
     $           3x, E10.3, '/',E10.3)
         END IF
c  Calculate vector a_no (IRET):
         ITERMD(1:N_RET,1:IRET) = MATMUL(KNTISYKN(:,:),EVEC(:,1:IRET))
         FORALL(J = 1:IRET) A_NO(J) = DOT_PRODUCT(ITERMD(:,J),EVEC(:,J))
c  Calculate expansion coefficients beta:
         BETA(1:IRET) = EVAL(1:IRET)/A_NO(1:IRET)/
     $        (1.0D0 + EVAL(1:IRET))*MATMUL(KNTISYTMB(:),EVEC(:,1:IRET))
C     Pearson's correlation coefficients
c         ALLOCATE(EV_SPEC(N_DATA,IRET),RES(N_DATA),C_CORR(IRET))
c         EV_SPEC(:,:) = MATMUL(KN(:,:),EVEC(:,1:IRET))
c         DO I = 1, IRET
c            RES(:) =  YDIFF(:) - MATMUL(EV_SPEC(:,:),BETA(1:IRET)) + 
c     $           BETA(I)*EV_SPEC(:,I)
c            C_CORR(I) = (DOT_PRODUCT(RES(:),EV_SPEC(:,I)) - SUM(RES(:))*
c     $           SUM(EV_SPEC(:,I))/N_DATA)/DSQRT(
c     $           (DOT_PRODUCT(RES(:),RES(:)) - SUM(RES(:))**2/N_DATA)*
c     $           (DOT_PRODUCT(EV_SPEC(:,I),EV_SPEC(:,I)) - 
c     $           SUM(EV_SPEC(:,I))**2/N_DATA))*SIGN(1.0D0,BETA(I))
c         END DO
c  Inversion with constrains:
         IF (DO_CONSTRAIN) THEN
c            CALL CONSTRAIN_INVERT
            CALL CONSTRAIN_INVERT_GALAHAD (N_RET, IRET, N_DATA, EVAL, 
     $           A_NO, TERMB, S_Y, KN, EVEC, BETA_CONSTRAIN, CONSTR_ARR)
c           IF(DO_LOGFILE) THEN
c               WRITE(LUNIT,'("  Beta: ",50(2x,1pd12.5))') 
c     $              (BETA(I), I = 1, IRET)
c               WRITE(LUNIT,
c     $              '("  Beta with constrains: ",50(2x,1pd12.5))') 
c     $              (BETA_CONSTRAIN(I), I = 1, IRET)
c            END IF
            BETA = BETA_CONSTRAIN
         END IF
         XDIFF(:) = MATMUL(EVEC(:,1:IRET),BETA(1:IRET))
c         OPEN(C_UNIT,FILE = DIR_OUT//'/evec_fit.dat')
c         DO I = 1, N_DATA
c            WRITE(C_UNIT,'(600(2X,D21.14))') YDIFF(I), 
c     $           (BETA(J)*DOT_PRODUCT(KN(I,:),EVEC(:,J)), J = 1,IRET)
c         END DO
c         CLOSE(C_UNIT)
      END IF
      
      WRITE(*,*) '   N_RET =  ', N_RET
      DO KK = 1, N_RET
         WRITE(*,'(2x,I3,*(2x,ES12.4))')
     $        KK, XDIFF(KK), XDIFF_SAVE(KK)
      ENDDO
      
      RMS2 = 0.0d0
      DO KK = 1, N_DATA
         SUM0 = DOT_PRODUCT(KN(KK,1:N_RET),XDIFF(:))
         SUM1 = DOT_PRODUCT(KN(KK,1:N_RET),XDIFF_SAVE(:))
         RMS2 = RMS2 + (YDIFF(KK) - SUM0 + SUM1)**2
      END DO
      RMS2 = DSQRT(RMS2/N_DATA)

      IF (LOG_LEVEL(2) >= N_RSO) THEN
         WRITE(LUNIT,*) 
     $        ' ********* RMS before and after main retrieval ********'
         WRITE(LUNIT,'(14x,E13.6,2x,E13.6)') RMS1, RMS2 
         WRITE(LUNIT,*) 
      END IF

      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN

C********************************************************************C
      END
C********************************************************************C
