C**********************************************************************
C                                                                     *
C	SCIATRAN - Version 4.7 - Last modification: 01 Jan 2024       *
C                                                                     *
C	Copyright (C) - Institute of Remote Sensing,                  *
C			University of Bremen, Germany                 *
C                                                                     *
C      Solution of RTE according to DOM for Gauss discrete angles     *
C                                                                     *
C**********************************************************************
      SUBROUTINE ITERATE_COA(JPOS, *, *, Mh, E_W, AGR_A, AGR_W, 
     $     SRR, SRR_Jact, SVR,
     $     GSL_A, GSL_W, SSB_A, SSB_W, TE, LOA, LOW, UDB_A, UDB_W,  
     $     SDO_A, SDO_W, LAS_A, LAS_W, RADOUT, Ph_zW, SLS, W_LEAV,
     $     ADS, ADO_A, ADO_W, Ph_rW, Ph_A, Ph_dA, Ph_C, Ph_Us, Ph_Ks,
     $     Ph_Kl)
c     ... Called from "st_dom_tau.f"

      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: LUNIT, N_ESO, LOG_LEVEL, N_DSO, EU, DIR_OUT
      USE RTM_OUTPUT,  ONLY: N_OP
      USE ST_VARS,     ONLY: Pie, N_RHS, SOLVER, Kvs, Finc, VST
      USE SPHER_PARS,  ONLY: N_AZ_VAR, ANG_OUT, N_LOS_OUT, LOS_DEF_FLAG,
     $                       JV_S, JV_E
      USE SLAB_2L,     ONLY: Ome0, TES_2L_SLAB, Fr_01, Fr_20, 
     $                       Ref_idx, F_Ref_inc, Taus
      USE TYPES,       ONLY: KND, LOA_TYPE, LOW_TYPE, SDO_TYPE, TE_TYPE,
     $                       UDB_TYPE, SSB_TYPE, GSL_TYPE, LAS_TYPE,
     $                       ADO_TYPE, PHA_CLD, AGR_type, VRS_TYPE,
     $                       RRS_TYPE, PHA_MATR, PH_MET, SLS_type
      USE WATER,       ONLY: NSTR_T, NN_W => NSTR_W, A_RTmat, OUT_WLR, 
     $                       NO_COUPLING, MAX_ITER => MAX_ITER_OC
c     ==================================================================

      INTEGER,           INTENT(IN)  :: JPOS, Mh, SRR_Jact
      REAL (KND),        INTENT(IN)  :: E_W 
      REAL (KND)  RADOUT(N_OP,N_LOS_OUT,Nvs), W_LEAV(N_LOS_OUT,Nvs)
c     ----------------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of water
      TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
      TYPE (GSL_TYPE) :: GSL_A, GSL_W ! Angles
      TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
      TYPE (AGR_type) :: AGR_A, AGR_W
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
      TYPE (PHA_MATR) PH_M
      TYPE (SLS_type) :: SLS    ! Spherical line-of-sight
c     ----------------------
      TYPE (SSB_type) :: SSB_A, SSB_W ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO_A, SDO_W ! Boundary conditions
      TYPE (LAS_TYPE) :: LAS_A, LAS_W ! Linear algebraic system
      TYPE (ADO_type) :: ADO_A, ADO_W ! Adjoint DOM auxiliary variables
c     -------------------------------------------------------------
c     ... Adjoint DOM auxiliary spherical geometry |
c     ----------------------------------------------
      TYPE ADS_type
        SEQUENCE
        LOGICAL, POINTER :: Act_ag(:,:)
        REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
      END TYPE ADS_type
      TYPE (ADS_type) :: ADS
c     -------------------------------------------------------------
      TYPE (PH_MET) :: Ph_rW, Ph_Ks, Ph_Kl
c     -------------------------------------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zW, Ph_A, Ph_Us
c     -------------------------------------------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     -------------------------------------------------------------
      CHARACTER (LEN=11), PARAMETER :: R_NAME = 'ITERATE_COA'
      CHARACTER (LEN=100) Msg(10)
      LOGICAL   OUT_FLUX
      INTEGER   I, I1, Ig, J, Jsh, K, Ks, L, NN, NNv, NWv, NN_A, 
     $     NAv, NrL, Nr2, NSA, Yn, INFO, LL1, LR1, I_TER 
      REAL (KND), ALLOCATABLE :: FA_U_IT(:), 
     $     Fs_IN(:), Fs_OUT(:), WLR_INT(:,:), HDU(:), HDW(:)
      REAL (KND) De_A, De_O, Ri2, DUM, Cazm, Sazm,
     $     TOP_W(2*Nvs*GSL_W%NN,N_RHS), BOT_W(2*Nvs*GSL_W%NN,N_RHS), 
     $     TOP_A(2*Nvs*GSL_A%NN,N_RHS), BOT_A(2*Nvs*GSL_A%NN,N_RHS),
     $     Fre_RS(GSL_A%NSZA), 
     $     Fat_u, Fat_d, Fab_u, Fab_d, Fot_u, Fot_d, Fob_u, Fob_d,
     $     RHSA_LOC  (2*Nvs*GSL_A%NN*(AGR_A%NrL-1),N_RHS),
     $     RHSW_LOC  (2*Nvs*GSL_W%NN*(AGR_W%NrL-1),N_RHS)
      REAL (KND), POINTER :: MuSW(:)
c     ==================================================================

      OUT_FLUX = .FALSE.

      CALL ST_TRACK (R_NAME, 1, 'I')

      Ri2 = UDB_A%Ref_wd(1)**2
      I_TER = 1
      NN_A  = GSL_A%NN  ;  NSA = GSL_A%NSZA
      NAv = Nvs*NN_A  ;  NWv = Nvs*GSL_W%NN

      ALLOCATE( FA_U_IT(MAX_ITER), Fs_IN(MAX_ITER), Fs_OUT(MAX_ITER),
     $     HDU(NAv), HDW(NWv) )

      IF( LOG_LEVEL(1) >= N_DSO ) CALL FLUXES_ATM

c      WRITE(*,*) '        ----------------------------             '
c      WRITE(*,*) '           First call in the ocean               '
c      WRITE(*,*) '        ----------------------------             '
c     ... Set the first guess of the downward radiation from the 
c         atmosphere to the ocean. The array UDB_W%FoINC(:,:) 
c         contains down-welling radiation just below the ocean
c         surface.
      SELECT CASE( TRIM(UDB_A%LBC) )
      CASE('Flat water')
c        ... Incoming radiation is within the Fresnel cone.
c            First dimension of array UDB_W%FoINC is NN_A, i.e.,
c            the number of Gauss angles in atmosphere
c         write(*,*) '   ====>> In the atmosphere (bottom) '
         DO I = 1,N_RHS 
            DO Ig = 1, NN_A  ;  Ks = (Ig-1)*Nvs
               DO K = 1, Nvs
                  UDB_W%FoINC(Ks+K,I) = Ri2*
     $                 SUM( UDB_A%Fre_Tv(Ig,K,1:Nvs)
     $                 *LAS_A%SOL_BOT(Ks+1:Ks+Nvs,I) )
               ENDDO  ! over Stokes components (K)
c               write(*,'(2x,I2,*(2x,E12.4))') Ig, 
c     $              LAS_A%SOL_BOT(Ks+1:Ks+Nvs,I)
            ENDDO  ! over Gauss angles (Ig)
c          --------------            
         ENDDO                  ! over right-hand sides of LAS (I)
c     ... end CASE('Flat water')                  
      CASE('Rough water')
         SELECT CASE( TRIM(A_RTmat) )
c        ... Transmission matrix is used ignoring roughness.
         CASE('FT')
c           ... Incoming radiation is within Fresnel cone only!
c               First dimension of array UDB_W%FoINC is NN_W
            UDB_W%FoINC(1:NSTR_T,:) = 0.d0
            DO I = 1,N_RHS ! Scalar case only
               UDB_W%FoINC(1+NSTR_T:NN_W,I) 
     $             = Ri2*LAS_A%SOL_BOT(1:NN_A,I)*UDB_A%Eaw_G(1:NN_A)
            ENDDO
c            write(*,*) UDB_A%Eaw_G(1:NN_A)
            
c        ... Transmission matrix including roughness
         CASE('NT_ICF','NT_ECF')
            DO I = 1,N_RHS   ;  I1 = Nvs*(I-1)
c          ... Downward diffuse radiation just above surface
c              times Gauss weights              
               DO K = 1,Nvs  ;  HDU(K:NAv:Nvs) =
     $              LAS_A%SOL_BOT(K:NAv:Nvs,I)*GSL_A%AG(:)
               ENDDO  ! over Stokes components (K)
               DO K = 1,NWv
c                 ... Contribution of diffuse light
                  UDB_W%FoINC(K,I) = 2.d0*Pie
     $                 * SUM( UDB_A%Taw_GG(K,1:NAv,Mh)*HDU(1:NAv) )
c                      ... Contribution of the direct solar light
     $                 + SUM( UDB_A%Taw_GS(K,I1+1:I1+Nvs,Mh)*Finc(1,:) )
     $                 *SSB_A%EXS(AGR_A%NL,I) *Pie *VST
               ENDDO
            ENDDO
c            write(*,'(A,*(2x,I3))')
c     $           '   Solution in ocean 1; Mh and I_TER: ', Mh, I_TER
c            DO K = 1, GSL_W%NN  ;  I1 = Nvs*(K-1)
c               if( UDB_W%FoINC(I1+1,1) < 1.0d-10 ) CYCLE
c               write(*,'(2x,I3,2x,F8.3,*(1x,e12.4))')
c     $              K,  DACOS(GSL_W%XM(K))/Pie*180.,
c     $              UDB_W%FoINC(I1+1:I1+Nvs,1)
c            ENDDO
         END SELECT
      END SELECT  ! UDB_A%LBC

      IF( (Mh==1).AND.(.NOT.UDB_A%SPECULAR) 
     $     .AND.(LOG_LEVEL(1) >= N_DSO) ) CALL CHECK_BALANCE
      
c     ... Discrete ordinates solution for ocean      
      CALL SOLV_DOM(JPOS, 'O', *880, *990, Mh, E_W, AGR_W, GSL_W, SSB_W,
     $     TE, LOA, LOW, UDB_W, SDO_W, LAS_W, SOLVER, Ph_zW, SLS, ADS,
     $     ADO_W, SRR, SRR_Jact, SVR, Ph_rW, Ph_A, Ph_dA, Ph_C, Ph_Us,
     $     Ph_Ks, Ph_Kl, PH_M, RADOUT)
      
      IF( LOG_LEVEL(1) >= N_DSO ) CALL FLUXES_OCEAN

c     ==================================================================
c     ---> At this step we have:
c       a) the DO solution in the atmosphere under assumption that 
c           there is no water-leaving radiation from the ocean;
c       b) the DO solution in the ocean under assumption that the
c           ocean-atmosphere interface is illuminated by the direct
c           and diffuse radiation from atmosphere.
c     -----------------------------------------------------------------
c      ... These solutions of the LAS for the atmosphere and ocean
c          are stored in arrays LAS_A%SVEC_AG and LAS_W%SVEC_AG, 
c          respectively
c     ==================================================================
c         WRITE(*,*) ' Solution in the atmosphere: "./DATA_OUT/atm.dat"'
c         open(47,file=DIR_OUT//'/atm.dat')
c         do K = 1, NAv
c            WRITE(47,'(2x,e12.4,2x,E15.7,4x,E15.7)') 
c     $           DACOS(GSL_A%XM(K))/Pie*180., 
c     $           LAS_A%SOL_BOT(K,1), LAS_A%SOL_BOT(NAv+K,1)
c         enddo
c         close(47)
      IF( Mh == 1 ) THEN
         DO I = 1, N_RHS
            IF( ANY(LAS_W%SOL_TOP(1:2*Nvs*NN_W:Nvs,I)<0.d0) ) THEN
               J = MINLOC( LAS_W%SOL_TOP(:,I),1 )
               write(*,*) '    I = ', I,  LAS_W%SOL_TOP(J,I)
               DO K = 1, 2*NN_W
                  write(*,'(2x,I3,2x, e12.4)') K, LAS_W%SOL_TOP(K,I)
               ENDDO
c               STOP 'Stop: negative water-leaving radiance is detected'
               Msg(1)='Negative water-leaving radiance is detected.'
               Msg(2)='  Possible reasons (among others): '
               Msg(3)='  1) Number of streams in the ocean and'//
     $              ' atmosphere is too small.'
               Msg(4)='  2) Number of streams in the total internal '//
     $              'reflection zone significantly differs'
               Msg(5)='  as ones in refraction one.'
               Msg(6)='Check control parameters "Number of streams" and'
               Msg(7)='   "Number of streams in total reflection zone" '
               Msg(8)='   in "control_ac.inp" and "control_uwt.inp" '//
     $              'files, respectively.'
               CALL WRITE_MESSAGE(R_NAME, 'WARNING', MSG)
c               CALL WRITE_MSG(R_NAME,'ERROR',Msg,8,*880)
            ENDIF
         ENDDO  ! over right-hand sides (I)
      ENDIF   ! Mh=1
c     ===============================================================
c     Start iteration between ocean and atmosphere radiative transfer
c     ===============================================================
 101  CONTINUE
c     ------------------------------------------------------------------
c      WRITE(*,'(14x,A)') 'Repeat solution for atmosphere :'

c     ... Right-hand side of the linear algebraic system for atmosphere
c         (saved in "solv_dom.f") and copied in the local array RHSA_LOC
      RHSA_LOC = LAS_A%RHS_LAS
      
      NrL = AGR_A%NrL  ;  NN = GSL_A%NN  ;  NNv = Nvs*NN 
      Yn = AGR_A%LAY_GR(NrL-1,1)  ;  Nr2 = 2*NNv*(NrL-1) ;  INFO = 0
c *********************************************************************
c         WRITE(*,*) '        ------------------------------           '
c         WRITE(*,*) '           Next call in the atmosphere           '
c         WRITE(*,*) '        ------------------------------           '
      
c     ... Include water-leaving radiation in the lower boundary 
c         condition for the radiative transfer in the atmosphere
c         write(*,*)
c     $        '   ====>>  New lower boundary condition for atmosphere'
      LL1 = (2*NrL-3)*NNv+1 ; LR1 = 2*NNv*(NrL-1)
      SELECT CASE( TRIM(UDB_A%LBC) )
      CASE('Flat water')
c         WRITE(*,*) '  NSTR_T, NN_W ', NSTR_T, NN_W 
         Jsh = Nvs*(NN_W + NSTR_T)
         DO I = 1,N_RHS
            DO Ig = 1, NN_A  ;  Ks = (Ig-1)*Nvs
               DO K = 1, Nvs
                  UDB_A%FoINC(Ks+K,I) = SUM( UDB_A%Fre_Tv(Ig,K,1:Nvs)
     $                 *LAS_W%SOL_TOP(Jsh+Ks+1:Jsh+Ks+Nvs,I) )/Ri2
               ENDDO  ! over Stokes components (K)
c               write(*,'(2x,I2,*(2x,E12.4))') 
c     $              Ig, LAS_W%SOL_TOP(Jsh+Ks+1:Jsh+Ks+Nvs,I),
c     $              UDB_A%Fre_Tv(Ig,1,1:Nvs), UDB_A%FoINC(Ks+1,I)
            ENDDO  ! over Gauss angles (Ig)
         ENDDO  ! over N_RHS (I)
c         PAUSE
      CASE('Rough water')
         SELECT CASE( TRIM(A_RTmat) )
         CASE('FT')
            DO I = 1,N_RHS
               UDB_A%FoINC(:,I) = LAS_W%SOL_TOP(NN_W+NSTR_T+1:2*NN_W,I)
     $              *UDB_W%Ewa_G(NSTR_T+1:NN_W) /Ri2
            ENDDO
         CASE('NT_ICF','NT_ECF')
c           ... The last Gauss angle was set to 0 deg with zero weight
           DO I = 1,N_RHS
c              ... Upward radiation just below surface times Gauss weights
              DO K = 1,Nvs
                 HDW(K:NWv:Nvs) = GSL_W%AG(:)
     $                *LAS_W%SOL_TOP(NWv+K:2*NWv:Nvs,I)
              ENDDO             ! over Stokes components (K)
c             Sign for U and V components of the Stokes vector
              IF( Nvs > 1 ) THEN
                 HDW(3:NWv:Nvs) = -HDW(3:NWv:Nvs) 
                 HDW(4:NWv:Nvs) = -HDW(4:NWv:Nvs) 
              ENDIF
              DO K = 1,NAv
                 UDB_A%FoINC(K,I) = 2.d0*Pie
     $                *SUM( UDB_W%Twa_GG(K,1:NWv,Mh)*HDW(1:NWv) )
              END DO
               
            ENDDO  ! over right-hand side (I)
c            write(*,*) '    Solution in atmosphere  Mh = ', Mh
c            DO K = 1, GSL_A%NN
c               I1 = Nvs*(K-1)
c               if( UDB_A%FoINC(I1+1,1) < 1.0d-10 ) CYCLE
c               write(*,'(2x,I3,*(1x,e12.4))')
c     $              K, UDB_A%FoINC(I1+1:I1+Nvs,1)
c            ENDDO
            
         END SELECT
      END SELECT  ! UDB_A%LBC

c     ... Use corrected lower boundary condition of atmospheric RTE
c      WRITE(*,*) 'LL1:LR1', LL1,'-',LR1, LR1-LL1+1
c      WRITE(*,*) 'SHAPE(UDB_A%FoINC) ',SHAPE(UDB_A%FoINC)
      DO I = 1,N_RHS
         RHSA_LOC(LL1:LR1,I) = RHSA_LOC(LL1:LR1,I) + UDB_A%FoINC(:,I)
      ENDDO

c     ... Solve LAS for the atmosphere with corrected lower boundary 
c         condition. The solution of the LAS is stored in the array 
c         RHSA_LOC
      CALL DGBTRS ( 'N', NR2, 3*NNv-1, 3*NNv-1, N_RHS, LAS_A%MATTEB, 
     $     9*NNv-2, LAS_A%INDX, RHSA_LOC, NR2, INFO )
      IF (INFO.NE.0) WRITE(LUNIT,*) 
     $     ' DGBTRS error message: INFO =', INFO  

      CALL ST_TOP_BOT(Mh, NrL, NN, NNv, GSL_A%NSZA, SDO_A%NcC(1), 
     $     SDO_A%NcC(Yn), SDO_A%Ex_AG(:,1), SDO_A%Ex_AG(:,2), 
     $     RHSA_LOC, SDO_A%G_NL(1,:,:), SDO_A%G_NL(Yn,:,:), 
     $     LAS_A%PART_U(1,:,:), LAS_A%PART_D(NrL-1,:,:), 
     $     UDB_A, TOP_A, BOT_A, LAS_A%FLUX, LAS_A%Ref_surf, 
     $     GSL_A%AXM, GSL_A%AG, ADO_A) 

c     ... Convergence in the atmosphere for the upward radiation at
c         ocean-atmosphere interface only. It is used to avoid
c         convergence problem in the case of "black" atmosphere.
      De_A = MAXVAL( DABS((BOT_A(NNv+1:2*NNv:Nvs,:) 
     $     - LAS_A%SOL_BOT(NNv+1:2*NNv:Nvs,:) )
     $     /BOT_A(NNv+1:2*NNv:Nvs,:)))*100.
c      WRITE(*,*) '   Intensity difference atm:', I_TER, De_A, ' %'

c      Sv1(:,:) = BOT_A(NNv+1:2*NNv:Nvs,:)
c      DLP(:,:) = SQRT( BOT_A(NNv+2:2*NNv:Nvs,:)**2
c     $     + BOT_A(NNv+3:2*NNv:Nvs,:)**2 )/Sv1(:,;)
cc ------------------------------------------------------------------------
cc     Check intensity and DLP
c      WRITE(*,*) '      1 - NN'
c      DO I = 1, NN  ;  I1 = Nvs*(I-1)
c         WRITE(*,'(2x,I3,3(2x,e12.4))') I, LAS_A%SOL_BOT(I,1),
c     $    BOT_A(I,1), (BOT_A(I,1) - LAS_A%SOL_BOT(I,1))/BOT_A(I,1)*100.
c      ENDDO
c      WRITE(*,'(A,I2,A,I3)') '  NN+1 - 2*NN  Mh = ',Mh,' Iter = ',I_TER
c      DO I = NN+1, 2*NN
c         WRITE(*,'(2x,I3,3(2x,e12.4))') I, LAS_A%SOL_BOT(I,1),
c     $    BOT_A(I,1), (BOT_A(I,1) - LAS_A%SOL_BOT(I,1))/BOT_A(I,1)*100.
c      ENDDO

      
c      PAUSE
c ------------------------------------------------------------------------
      LAS_A%SOL_BOT(:,:) = BOT_A(:,:)
      LAS_A%SOL_TOP(:,:) = TOP_A(:,:)

      IF( NO_COUPLING )  THEN
c        ... Save solution of the LAS for the atmosphere 
c            if the iteration process is finished
         LAS_A%SVEC_AG = RHSA_LOC
         GOTO 123
      ENDIF
c    ==================================================================
c
c          End iterations in the case if COUPLING is set to off
c
c    ==================================================================

      IF( LOG_LEVEL(1) >= N_DSO ) CALL FLUXES_ATM
c    ------------------------------------------------------------------

c     ... Right-hand side of the linear algebraic system for ocean
c         (saved in "solv_dom.f") and copied in the local array RHSW_LOC
      RHSW_LOC = LAS_W%RHS_LAS 

      NrL = AGR_W%NrL  ;  NN  = GSL_W%NN  ;  NNv = Nvs*NN
      Yn  = AGR_W%LAY_GR(NrL-1,1)  ;  Nr2 = 2*NNv*(NrL-1) ;  INFO = 0

c     ... Include diffuse radiance from the atmosphere in the upper 
c         boundary condition for the radiative transfer in the ocean
      SELECT CASE( TRIM(UDB_A%LBC) )
      CASE('Flat water')
         Jsh = Nvs*NSTR_T
         DO I = 1,N_RHS
            DO Ig = 1, NN_A  ;  Ks = (Ig-1)*Nvs
               DO K = 1, Nvs
                  UDB_W%FoINC(Ks+K,I) = Ri2*SUM(UDB_A%Fre_Tv(Ig,K,1:Nvs)
     $                 *LAS_A%SOL_BOT(Ks+1:Ks+Nvs,I) )
               ENDDO  ! over Stokes components (K)
            ENDDO  ! over Gauss angles (Ig)
            RHSW_LOC(Jsh+1:NNv,I) = RHSW_LOC(Jsh+1:NNv,I)
     $           + UDB_W%FoINC(1:NAv,I) 
c           --------------            
         ENDDO  ! over RHS (I)

      CASE('Rough water')
         SELECT CASE( TRIM(A_RTmat) )
c        ... Transmission matrix ignoring roughness
         CASE('FT')
            UDB_W%FoINC(1:NSTR_T,:) = 0.d0
            DO I = 1,N_RHS        ! Scalar case only
               UDB_W%FoINC(1+NSTR_T:NN_W,I) 
     $              = Ri2*LAS_A%SOL_BOT(1:NN_A,I)*UDB_A%Eaw_G(1:NN_A)
               RHSW_LOC(1:NNv,I) = RHSW_LOC(1:NNv,I) + UDB_W%FoINC(:,I)
            ENDDO
c        ... Standard representation of reflection and trans matrices
         CASE('NT_ICF','NT_ECF')
            DO I = 1,N_RHS   ;  I1 = Nvs*(I-1)
c             ... Downward radiation just above surface times Gauss weights
               DO K = 1,Nvs  ;  HDU(K:NAv:Nvs) =
     $              LAS_A%SOL_BOT(K:NAv:Nvs,I)*GSL_A%AG(:)
               ENDDO
               DO K = 1,NWv
c                 ... Contribution of diffuse light
                  UDB_W%FoINC(K,I) = 2.d0*Pie
     $                 * SUM( UDB_A%Taw_GG(K,1:NAv,Mh)*HDU(1:NAv) )
c                      ... Contribution of the direct solar light
     $                 + SUM( UDB_A%Taw_GS(K,I1+1:I1+Nvs,Mh)*Finc(1,:) )
     $                 *SSB_A%EXS(AGR_A%NL,I) *Pie *VST
               ENDDO
               RHSW_LOC(1:NNv,I) = RHSW_LOC(1:NNv,I) + UDB_W%FoINC(:,I)
             ENDDO  ! over solar angles (I)
c             write(*,*) '    Solution in ocean 2: I_TER = ', I_TER
c             DO K = 1, GSL_W%NN  ;  I1 = Nvs*(K-1)
c                if( UDB_W%FoINC(I1+1,1) < 1.0d-10 ) CYCLE
c                write(*,'(2x,I3,*(1x,e12.4))')
c     $               K, UDB_W%FoINC(I1+1:I1+Nvs,1)
c             ENDDO
         END SELECT
      END SELECT
c      WRITE(*,*) 'UDB_W%FoINC(:,1)', SUM( UDB_W%FoINC(:,1) )
c      WRITE(*,*) 'UDB_W%FoINC(:,2)', SUM( UDB_W%FoINC(:,2) )

c     ... Solve LAS for the ocean with corrected upper boundary condition.
c         The solution of the LAS is stored in the array RHSW_LOC
      CALL DGBTRS('N', NR2, 3*NNv-1, 3*NNv-1, N_RHS, LAS_W%MATTEB, 
     $     9*NNv-2, LAS_W%INDX, RHSW_LOC, NR2, INFO )
      IF (INFO.NE.0) WRITE(LUNIT,*) 
     $     ' DGBTRS error message: INFO =', INFO  

      CALL ST_TOP_BOT(Mh, NrL, NN, NNv, GSL_W%NSZA, SDO_W%NcC(1), 
     $     SDO_W%NcC(Yn), SDO_W%Ex_AG(:,1), SDO_W%Ex_AG(:,2), 
     $     RHSW_LOC, SDO_W%G_NL(1,:,:), SDO_W%G_NL(Yn,:,:), 
     $     LAS_W%PART_U(1,:,:), LAS_W%PART_D(NrL-1,:,:), 
     $     UDB_W, TOP_W, BOT_W, LAS_W%FLUX, LAS_W%Ref_surf, 
     $     GSL_W%AXM, GSL_W%AG, ADO_W) 

c      WRITE(*,*)  '    N_RHS = ', N_RHS
c      WRITE(*,'(a,*(1x,e12.4))')  '    Top_w ',TOP_W(1:4,1)
c      WRITE(*,*) 'Shape LAS_W%SOL_TOP '
c      WRITE(*,*)  LAS_W%SOL_TOP(:,1)

c     ... Convergence in the ocean for the downward radiation 
c         just below ocean-atmosphere interface
      De_O = MAXVAL(DABS( (TOP_W(1:NWv:Nvs,:)
     $     - LAS_W%SOL_TOP(1:NWv:Nvs,:) )/TOP_W(1:NWv:Nvs,:)) )*100.
c      WRITE(*,*) '   Intensity difference ocean: ',I_TER, De_O, ' %'

      LAS_W%SOL_TOP(:,:) = TOP_W(:,:)
      LAS_W%SOL_BOT(:,:) = BOT_W(:,:)

      IF( LOG_LEVEL(1) >= N_DSO ) CALL FLUXES_OCEAN

      IF( I_TER >= MAX_ITER ) THEN
         OPEN( 44, file=DIR_OUT//'/converg.dat')
         DO I = 1,I_TER
            WRITE(44,'(2x,I3,3(2x,E15.7), 2x,e12.4)') 
     $           I, FA_U_IT(I), Fs_IN(I), Fs_OUT(I), 
     $           (Fs_IN(I) - Fs_OUT(I))/Fs_IN(I)*100.
         ENDDO
         CLOSE( 44 )
         Msg(1)='Iteration number > MAX_ITER. There is no convergence.'
         MSG(2)='   See file "./DATA_OUT/converg.dat".'
         CALL WRITE_MSG(R_NAME,'ERROR',Msg,2,*880)
      ENDIF

c      IF( De_O > 1.d-1 .OR. De_A > 1.d-1 ) THEN
      IF( De_O > 1.5d-2 .OR. De_A > 1.5d-2 ) THEN
         I_TER = I_TER + 1
         IF( LOG_LEVEL(1) >= N_DSO ) THEN 
            WRITE(LUNIT,*) '                         Harmonic :',Mh
            WRITE(LUNIT,*) '                 Iteration number :',I_TER
            WRITE(LUNIT,*) '     Maximal difference ocean [%] :',De_O
            WRITE(LUNIT,*) '     Maximal difference atmos [%] :',De_A
            WRITE(LUNIT,*) '      GOTO the next iteration  '
         ENDIF
c        ... Repeat calculations in the atmosphere and ocean
         GOTO 101
      END IF

c     ... Save solution of the LAS for the atmosphere and the ocean
c         if the iteration process is finished
      LAS_A%SVEC_AG = RHSA_LOC
      LAS_W%SVEC_AG = RHSW_LOC

c     ... Jump to this line if NO_COUPLING = .TRUE.
 123  CONTINUE

c     ... Iteration process is finished
      IF( LOG_LEVEL(1) >= N_ESO ) THEN 
         WRITE(LUNIT,'(A)')' -----------------------------------------'
         WRITE(LUNIT,'(A,I3)')'                          Harmonic :',Mh
         WRITE(LUNIT,'(A,I3)')
     $        '                  Iteration number :', I_TER
         IF( NO_COUPLING ) THEN
            WRITE(LUNIT,'(A)') 
     $           '         No coupling has been accounted for '
         ELSE
            WRITE(LUNIT,'(A,e12.4)') 
     $           '   Maximal difference ocean [%] :', De_O
            WRITE(LUNIT,'(A,e12.4)') 
     $           '   Maximal difference atmos [%] :', De_A
            WRITE(LUNIT,'(A)') '         End of iteration process '
         ENDIF
         WRITE(LUNIT,'(A)')' -----------------------------------------'
      ENDIF
      
c     ... Output of water-leaving radiance
c     ====================================
      IF( OUT_WLR ) THEN
         ALLOCATE( WLR_INT(GSL_A%NLSA,Nvs) )
         J = 0
         DO I = 1, GSL_A%NSZA
            IF( LOS_DEF_FLAG == 'ga' ) THEN
c              ... Output at all Gauss angles without interpolation
               DO K = 1, Nvs
                  WLR_INT(:,K) = UDB_A%FoINC(K:NAv:Nvs,I)
               ENDDO
            ELSE
c              ... Interpolate transmitted into atmosphere radiation
c                  to the line-of-sight directions
               DO K = 1, Nvs
                  CALL AKIMA_INT(NN_A,GSL_A%XM,UDB_A%FoINC(K:NAv:Nvs,I),
     $                 GSL_A%NLSA, GSL_A%Muv, WLR_INT(:,K))
               ENDDO
            ENDIF
            DO K = JV_S(I), JV_E(I)
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  Cazm = DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  W_LEAV(J,1) = Cazm*WLR_INT(K,1)
                  IF( Nvs==1 ) CYCLE
                  Sazm = 0.5d0*DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  W_LEAV(J,1)   = 0.5d0* W_LEAV(J,1)
                  W_LEAV(J,2)   = 0.5d0* Cazm* WLR_INT(K,2)
                  W_LEAV(J,3:4) = Sazm* WLR_INT(K,3:4)                     
               ENDDO  ! over azimuthal angles (L)
            ENDDO  ! over viewing angles (K)
         ENDDO  ! over solar angles (I)
      ENDIF  ! OUT_WLR
      
c      IF( Mh == 1 ) THEN
c         WRITE(*,*) ' Solution in the atmosphere: "./DATA_OUT/atm.dat"'
c         open(47,file=DIR_OUT//'/atm.dat')
c         do K = 1, NAv
c            WRITE(47,'(2x,e12.4,2(2x,E15.7),4x,2(2x,E15.7))') 
c     $           DACOS(GSL_A%XM(K))/Pie*180., 
c     $           LAS_A%SOL_TOP(K,1), LAS_A%SOL_TOP(Nnv_A+K,1),
c     $           LAS_A%SOL_BOT(K,1), LAS_A%SOL_BOT(NAv+K,1)
c         enddo
c         close(47)
c         WRITE(*,*) 
c     $        ' Solution in the ocean:      "./DATA_OUT/ocean.dat"'
c         open(47,file=DIR_OUT//'/ocean.dat')
c         NWv = Nvs*GSL_W%NN
c         do K = 1, NWv
c            WRITE(47,'(2x,e12.4,2(2x,E15.7),4x,2(2x,E15.7))') 
c     $           DACOS(GSL_W%XM(K))/Pie*180., 
c     $           LAS_W%SOL_TOP(K,1), LAS_W%SOL_TOP(NWv+K,1),
c     $           LAS_W%SOL_BOT(K,1), LAS_W%SOL_BOT(NWv+K,1)
c         enddo
c         close(47)
c      ENDIF

      
      IF( TES_2L_SLAB ) THEN
         WRITE(*,*) ''
         WRITE(*,'(A,L1)')   '   SPECULAR reflection            -  ',
     $        UDB_A%SPECULAR
         WRITE(*,'(A,f5.2)') '   Relative refraction index (L/U) - ',
     $        UDB_A%Ref_wd(1)
         WRITE(*,'(A)')      '   Gauss angles in '
         WRITE(*,'(A,I2)')   '   total-reflection region - ',NSTR_T

         WRITE(*,'(2x,A)')
     $   '------------------------------------------------------------'
         WRITE(*,'(2x,A,26x,A,2x,A,2x,A,2x,A,2x,A)') '|',
     $        '|','Upper layer','|','Lower layer','|'
         WRITE(*,'(2x,A)')
     $   '------------------------------------------------------------'
         WRITE(*,'(2x,A,4x,F5.2,6x,A,4x,F5.2,6x,A)') 
     $        '| Refractive index         |',
     $        Ref_idx(1),'|',Ref_idx(2),'|'
         WRITE(*,'(2x,A,4x,F5.2,6x,A,4x,F5.2,6x,A)') 
     $        '| Optical thickness        |',Taus(1),'|',Taus(2),'|'
         WRITE(*,'(2x,A,4x,F6.3,5x,A,4x,F6.3,5x,A)') 
     $        '| Single scattering albedo |',Ome0(1),'|',Ome0(2),'|'
         WRITE(*,'(2x,A)')
     $   '------------------------------------------------------------'

         WRITE(*,*) '   ----- Reflected flux at the top due to Fresnel:'
         WRITE(*,'(5x,f7.3)') F_Ref_inc
         Fat_u = 2.*Pie*SUM( LAS_A%SOL_TOP(NAv+1:2*NNv:Nvs,1)
     $        *GSL_A%AXM(1:NN_A)*(1.d0 - Fr_01(1:NN)) )
     $        + F_Ref_inc
         Fob_d = 2.*Pie*SUM( LAS_W%SOL_BOT(1:NNv:Nvs,1)
     $        *(1.d0 - Fr_20(:))*GSL_W%AXM(1:NN) )
         WRITE(*,'(A)')
     $        '   Hemispherical reflectivity and transmissivity'
         WRITE(*,'(10x,F8.5,4x,F8.5)') Fat_u/Pie, Fob_d/Pie
      ELSE
c         WRITE(*,*) ''
c         WRITE(*,'(A,L1)')   '   SPECULAR reflection             - ',
c     $        UDB_A%SPECULAR
c         WRITE(*,'(A,f5.2)') '   Relative refraction index (L/U) - ',
c     $        UDB_A%Ref_wd(1)
c         WRITE(*,'(A)')      '   Gauss angles in '
c         WRITE(*,'(A,I2)')   '   total-reflection region - ',NSTR_T

c         WRITE(*,*) ''
c         WRITE(*,'(26x,A)') ' ********** Atmosphere ************** '
c         WRITE(*,*) '                Fluxes at top               '//
c     $        '              Fluxes at bottom'
c         WRITE(*,'(2(16x,A,10x,A))') 
c     $        'downward','upward','downward','upward'
c         WRITE(*,'(7x,2(2x,ES17.9E3),4x,2(2x,ES17.9E3))') 
c     $        Fat_d, Fat_u,  Fab_d, Fab_u
c         WRITE(*,'(30x,A)') 'Energy balance' 
c         WRITE(*,'(15x,A)') 
c     $        'input at top and  output at bottom and energy balance'
c         WRITE(*,'(14x,3(2x,ES17.9E3))') 
c     $        Fat_d - Fat_u,  Fab_d - Fab_u, 
c     $        ((Fat_d - Fat_u) - (Fab_d - Fab_u))/(Fat_d - Fat_u)*100.
c
c         WRITE(*,'(26x,A)') ' ********** Ocean ************** '
c         WRITE(*,*) '                Fluxes at top               '//
c     $        '              fluxes at bottom'
c         WRITE(*,'(2(16x,A,10x,A))') 
c     $        'downward','upward','downward','upward'
c         WRITE(*,'(7x,2(2x,ES17.9E3),4x,2(2x,ES17.9E3))') 
c     $        Fot_d, Fot_u,  Fob_d, Fob_u
c         WRITE(*,'(30x,A)') 'Energy balance' 
c         WRITE(*,'(15x,A)') 
c     $        'input at top and  output at bottom and energy balance'
c         WRITE(*,'(14x,3(2x,ES17.9E3))') 
c     $        Fot_d - Fot_u,  Fob_d - Fob_u, 
c     $        ((Fot_d - Fot_u) - (Fob_d - Fob_u))/(Fot_d - Fot_u)*100.
c         WRITE(*,'(26x,A)') ' ******************************* '
c         WRITE(*,*) ''
      ENDIF

c      stop '   STOP in "ITERATE_COA" '
      DEALLOCATE( FA_U_IT, Fs_IN, Fs_OUT )

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 990  RETURN 2

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c **********************************************************************

      SUBROUTINE CHECK_BALANCE

      REAL (KND) I_Dir, Fd, Fd_ref_Fre, Fd_ref_Cox, Fd_tra_del,
     $     Fd_tra_Fre, Fd_tra_Cox 
         
      CALL Fre_REF(NSA, Fre_RS, GSL_A%MuS, UDB_A%Ref_wd, 'aa')
      WRITE(*,*) '     Energy balance of the direct radiation '
      WRITE(*,*) '    after interaction with atmosphere-ocean '
      WRITE(*,*) '                 interface '

c     ... Intensity of the direct solar radiance 
c         at the surface (contains two delta-functions)
      I_Dir = Pie*SSB_A%EXS(AGR_A%NL,1)

c     ... Flux of the direct solar radiance at the surface
      Fd = I_dir*GSL_A%MuS(1)

c     ... Flux of the reflected solar radiance according to 
c         the Fresnel law:
      Fd_ref_Fre = Fd*Fre_RS(1)

c     ... Flux of the transmitted solar radiance into ocean 
c         according to the Fresnel law:
      Fd_tra_Fre = Fd*(1.d0 - Fre_RS(1))
      
c     ... Flux of the reflected solar radiance using reflection 
c         matrix for Gaussian PDF:
      Fd_ref_Cox = 2.*Pie*I_Dir
     $     *SUM( UDB_A%Ra_GS (1:NAv:Nvs,1,1)*GSL_A%AXM(1:NN_A) )

c     ... Flux of the transmitted solar radiance into ocean 
c         using transmission matrix for Gaussian PDF:
      Fd_tra_Cox = 2.*Pie*I_Dir 
     $     *SUM( UDB_A%Taw_GS(1:NWv:Nvs,1,1)*GSL_W%AXM(1:NN_W) )
      
      Fd_tra_del = Pie*SSB_W%EXS(1,1)*GSL_W%MuS(1)
      
      WRITE(*,*) ' Direct incident :'
      WRITE(*,'(5x,e12.4)') Fd
      WRITE(*,*) 
     $     ' Direct reflected: rough and according to Fresnel'
      WRITE(*,'(5x,2(2x,e12.4))') Fd_ref_Cox, Fd_ref_Fre         
      WRITE(*,*) 
     $     ' Direct transmitted: rough and according to Fresnel'
      WRITE(*,'(5x,2(2x,e12.4))') Fd_tra_Cox, Fd_tra_Fre 
      WRITE(*,*) 
     $     ' Direct transmitted: effective'
      WRITE(*,'(5x,2(2x,e12.4))') Fd_tra_del
      WRITE(*,*) ' Energy loss [%]: rough, according to Fresnel'
      WRITE(*,*) '    and weighted diffuse + Dirac-direct.'
      WRITE(*,'(5x,3(2x,e12.4))') 
     $     ( Fd - (Fd_ref_Cox + Fd_tra_Cox) )/Fd*100.,
     $     ( Fd - (Fd_ref_Fre + Fd_tra_Fre) )/Fd*100.,
     $     ( Fd - (Fd_ref_Cox + Fd_tra_Cox + Fd_tra_del) )/Fd*100.
      
      !PAUSE
      END SUBROUTINE CHECK_BALANCE
c **********************************************************************

      SUBROUTINE FLUXES_ATM
      INTEGER  Nst, Nfi
      REAL (KND)  Mu0, Sol_tr, D_top, D_bot, Fre_RS(GSL_A%NSZA),
     $     Idow, Itra, Iref, Isol, S1, S2

      WRITE(*,*) '  '
      WRITE(*,'(10x,A,I4,A)')
     $     '  =====> Iteration :', I_TER, ' atmosphere <============'

      Mu0    = GSL_A%MuS(1)
      Sol_tr = SSB_A%EXS(AGR_A%NL,1)
      NN  = GSL_A%NN  ;  NNv = Nvs*NN ; NAv = NNv
      Nst = NAv+1
      Nfi = 2*NAv

      IF( UDB_A%SPECULAR ) THEN
         WRITE(*,'(20x,A)') 'Specular reflection  '
c        ... Fresnel reflection for solar angles
         CALL Fre_REF(GSL_A%NSZA, Fre_RS, GSL_A%MuS, UDB_A%Ref_wd, 'aa')
c        ... Specular reflected solar light at the top of atmosphere
         Fat_u = Pie*Sol_tr*Mu0*Fre_RS(1)*Sol_tr
c        ... Specular reflected solar light at the bottom of atmosphere
         Fab_u = Pie*Sol_tr*Mu0*Fre_RS(1)
      ELSE
         Fat_u = 0.d0 ; Fab_u = 0.d0
      ENDIF

      Fat_d = Pie*Mu0
     $     + 2.*Pie*SUM(LAS_A%SOL_TOP(1:NAv:Nvs,1)*GSL_A%AXM(1:NN_A))
      Fat_u = Fat_u
     $     + 2.*Pie*SUM(LAS_A%SOL_TOP(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN_A))

      FA_U_IT(I_TER) = Fat_u
      Fs_IN(I_TER)   = Fat_d
      Fs_OUT(I_TER)  = Fat_u

      IF( Mh == 1  .AND. OUT_FLUX ) THEN
c        ... Total downward flux at bottom
         Fab_d = Pie*Sol_tr*Mu0
     $      + 2.*Pie*SUM( LAS_A%SOL_BOT(1:NNv:Nvs,1)*GSL_A%AXM(1:NN_A) )
c        ... Total upward flux at bottom
         Fab_u = Fab_u
     $      + 2.*Pie*SUM(LAS_A%SOL_BOT(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN_A))

         WRITE(*,'(18x,A,26x,A)') 
     $        'fluxes at top','fluxes at bottom'
         WRITE(*,'(15x,A,15x,A, 17x,A,15x,A)') 
     $        'down','up', 'down','up'
         WRITE(*,'(7x,2(2x,E15.7),4x,2(2x,E15.7))') 
     $        Fat_d, Fat_u,  Fab_d, Fab_u
         D_top = Fat_d - Fat_u ; D_bot = Fab_d - Fab_u
         WRITE(*,*) 
         WRITE(*,'(11x,A,5x,A,5x,A)') 
     $        'incoming at top','"outgoing" at bottom','diff [%]'
         WRITE(*,'(11x,E15.7, 7x,E15.7, 3x,E15.7)') 
     $        D_top,  D_bot, (D_top - D_bot)/D_top*100.

         WRITE(*,'(20x,A)') 
     $        '   -----------------------------------------'

      END IF

c         WRITE(*,*) 
c     $        '   Lower boundary condition in atmosphere and difference'
c         WRITE(*,*) '   Iteration number = ',I_TER
c         IF( UDB_A%SPECULAR ) THEN
c            WRITE(*,*) '    ===> Flat surface  '
c            S1 = 0.d0 ; S2 = 0.d0
c            do K = 1, NNv
cc              ... Reflected from the surface as solution
c               Isol = LAS_A%SOL_BOT(NNv+K,1)
c
cc              ... Incident from atmosphere
c               Idow = LAS_A%SOL_BOT(K,1)
c               S1 = S1 + Idow*GSL_A%AXM(K)
c
cc              ... Transmitted from water
c               Itra = UDB_A%FoINC(K,1)
c               S2 = S2 + Itra*GSL_A%AXM(K)
cc
c               WRITE(*,'(2x,I5,5(2x,E15.7),3x,f7.2)') K, 
c     $              Isol,
cc                   ...       sum             Incident*Fresnel  from water
c     $              UDB_A%Fre_R(K)*Idow+Itra, UDB_A%Fre_R(K)*Idow, Itra,
cc                   ... Difference
c     $              Isol - UDB_A%Fre_R(K)*Idow - Itra,
cc                   ... Gauss angles
c     $              DACOS(GSL_A%XM(K))/Pie*180.
c            enddo
c            WRITE(*,'(2x,3(2x,e12.4))') 
c     $           2.*Pie*S1, 2.*Pie*S2, 2.*Pie*(S1+S2)
c         ELSE
c            WRITE(*,*) '    ===> Rough surface  '
c            S1 = 0.d0 ; S2 = 0.d0
c            do K = 1, NNv
cc              ... Reflected from the surface as solution
c               Isol = LAS_A%SOL_BOT(NNv+K,1)
cc              ... Incident from atmosphere and reflected from water
c               Idow = 2.D0*Pie*
c     $              SUM( UDB_A%Ra_GG(K,:,1)*LAS_A%SOL_BOT(1:NNv,1) )
cc                     including reflected direct beam
c     $              + UDB_A%Ra_GS(K,1,1)*Sol_tr*Pie
c               S1 = S1 + Idow*GSL_A%AXM(K)
c
cc              ... Transmitted from water
c               Itra = UDB_A%FoINC(K,1)
c               S2 = S2 + Itra*GSL_A%AXM(K)
c
c               WRITE(*,'(2x,I5,5(2x,E15.7),3x,f7.2)') K, Isol,
cc                      sum     Incident*R_GG  from water
c     $              Idow+Itra,    Idow,         Itra,
cc                   ... Difference
c     $              Isol - Idow - Itra,
cc                   ... Gauss angles
c     $              DACOS(GSL_A%XM(K))/Pie*180.
c            enddo
c            WRITE(*,'(2x,3(2x,e12.4))') 
c     $           2.*Pie*S1, 2.*Pie*S2, 2.*Pie*(S1+S2)
c         ENDIF

      IF(I_ter == 1 ) THEN
c     ------------------------------------------------------------------
         IF( TES_2L_SLAB ) THEN
            WRITE(*,*) '  -------- Two-layer slab ---------- '
            WRITE(*,*) 
     $           '   Upper boundary condition in atmosphere '
            WRITE(*,*) '   Iteration number = ',I_TER
            do K = 1, NNv
               WRITE(*,'(2x,I5,2(2x,E15.7),3x,f7.2)') 
     $              K, LAS_A%SOL_TOP(K,1), 
     $              (1.d0-Fr_01(K)) + Fr_01(K)*LAS_A%SOL_TOP(NN+K,1), 
     $              DACOS(GSL_A%XM(K))/Pie*180.
            enddo
         ENDIF
c     ------------------------------------------------------------------
      ENDIF
      IF(I_ter > 1 ) THEN
         IF( TES_2L_SLAB ) THEN
            WRITE(*,*) '  -------- Two-layer slab ---------- '
            WRITE(*,*) 
     $           '   Upper boundary condition in atmosphere '
            WRITE(*,*) '   Iteration number = ',I_TER
            do K = 1, NNv
               WRITE(*,'(2x,I5,2(2x,E15.7),3x,f7.2)') 
     $              K, LAS_A%SOL_TOP(K,1), 
     $              (1.d0-Fr_01(K)) + Fr_01(K)*LAS_A%SOL_TOP(NN+K,1), 
     $              DACOS(GSL_A%XM(K))/Pie*180.
            enddo
         ENDIF
c     ------------------------------------------------------------------
      ENDIF
      
      END SUBROUTINE FLUXES_ATM
c **********************************************************************

      SUBROUTINE FLUXES_OCEAN
      INTEGER K1
      REAL (KND) Sol_tr, Fot_d, Fot_u, Fob_d, Fob_u, D_top, D_bot,
     $     dir, diff, Tes(NN_A), Tes_W(NN_W), 
     $     T_pdf, R_pdf, T_eff, C_ef,
     $     Iw_ref(NN_W), Iw_tra(NN_A)

c      WRITE(*,*) '  '
c      WRITE(*,'(10x,A,I4,A)')
c     $     '  =====> Iteration :', I_TER, ' ocean <============'
      IF( (Mh==1).AND.(.NOT.UDB_A%SPECULAR).AND.OUT_FLUX) THEN
         DO K = 1,NN_A  ;  K1 = Nvs*(K-1)
            Iw_tra(K) = 2.d0*Pie*
     $           SUM( UDB_W%Twa_GG(K1+1,1:NWv:Nvs,Mh)*GSL_W%AG(:) )
         END DO
c        ... Transmitted flux 
         T_pdf = 2.*Pie*SUM( Iw_tra(:)*GSL_A%AXM(1:NN_A) )

c        ... Intensity reflected back to the ocean
         DO K = 1,NN_W  ;  K1 = Nvs*(K-1)
            Iw_ref(K) = 2.d0*Pie*
     $           SUM( UDB_W%Rw_GG(K1+1,1:NWv:Nvs,Mh)*GSL_W%AG(:) )
         END DO

c       ... Energy reflected from the ocean top back to the ocean
         R_pdf = 2.*Pie*SUM( Iw_ref(:)*GSL_W%AXM(1:NN_W) )     

c        .... Set transmitted radiance to zero in total reflection cone
         Iw_tra(1:NN_A) = Iw_ref(1+NSTR_T:NN_W)
c        .... 
         T_Eff = 2.*Pie*SUM( Iw_tra(1:NN_A)*GSL_A%AXM(1:NN_A) )
         C_ef = ( Pie - R_pdf )/T_eff
         Iw_tra(1:NN_A) = C_ef*Iw_tra(1:NN_A)

         T_Eff = 2.*Pie*SUM( Iw_tra(1:NN_A)*GSL_A%AXM(1:NN_A) )
 
         WRITE(*,*) '   Compare rough  '
         WRITE(*,'(2x,5(1x,e12.4))') 
     $        T_pdf, R_pdf,  (Pie-T_pdf-R_pdf)/Pie*100.
         WRITE(*,*) '   Compare effective  '
         WRITE(*,'(2x,5(1x,e12.4))') 
     $        T_eff, R_pdf,  (Pie-T_eff-R_pdf)/Pie*100.
c      STOP
      ENDIF

      NN  = GSL_W%NN ; NNv = Nvs*NN 
c     ... "Solar exponent" at the top of the ocean
      Sol_tr = SSB_W%EXS(1,1)

      if( Mh == 1 .AND. OUT_FLUX) THEN
         CALL Fre_REF(GSL_A%NSZA, Fre_RS, GSL_A%MuS, UDB_A%Ref_wd, 'aa')

         WRITE(*,'(10x,A)') 
     $        '--------- Before calculations -------------'
         WRITE(*,'(11x,A)') 
     $        ' Incident on the ocean surface just above interface'
c        ... Direct flux
         dir  = Pie*SSB_A%EXS(AGR_A%NL,1)*GSL_A%MuS(1)
c        ... Diffuse flux
         diff = 2.*Pie*SUM( LAS_A%SOL_BOT(1:NN_A,1)*GSL_A%AXM(:) )
         WRITE(*,'(14x,A,10x,A,10x,A)') 'direct','diffuse','total'
         WRITE(*,'(7x,3(2x,E15.7))') dir, diff, diff + dir

         WRITE(*,'(11x,A)') 
     $        ' Incident on the ocean surface just below interface'
c        ... Direct solar beam in the ocean is calculated ignoring roughness
         dir = Pie*Sol_tr*GSL_W%MuS(1)

         IF( UDB_A%SPECULAR ) THEN
            diff = 2.*Pie*SUM(
     $           LAS_A%SOL_BOT(1:NN_A,1)*GSL_A%AXM(1:NN_A)
     $           *(1.d0 - UDB_A%Fre_R(:)) )*Ri2
         ELSE
            DO K = 1,NN_W
               Tes_W(K) = SUM( UDB_A%Taw_GG(K,1:NN_A,1)
     $              *LAS_A%SOL_BOT(1:NN_A,1)*GSL_A%AG(:) )*2.d0*Pie
            ENDDO
            diff = 2.*Pie*SUM( Tes_W(:)*GSL_W%AXM(1:NN_W) )
         ENDIF

         WRITE(*,'(7x,3(2x,E15.7))') dir, diff, diff + dir
         WRITE(*,'(10x,A)') 
     $        '--------- After calculations --------------'
      ENDIF

      NN  = GSL_W%NN
      NNv = Nvs*NN 

c     ... Solar flux at ocean top
      Fot_d = Pie*Sol_tr*GSL_W%MuS(1)
c     ... Solar flux at ocean bottom
      Fob_d = Pie*SSB_W%EXS(AGR_W%NL,1)*GSL_W%MuS(1)

      Fot_d = Fot_d
     $     + 2.*Pie*SUM( LAS_W%SOL_TOP(1:NNv:Nvs,1)*GSL_W%AXM(1:NN) )
      Fot_u = 2.*Pie
     $     *SUM( LAS_W%SOL_TOP(NNv+1:2*NNv:Nvs,1)*GSL_W%AXM(1:NN))
      
      Fob_d = Fob_d
     $     + 2.*Pie*SUM( LAS_W%SOL_BOT(1:NNv:Nvs,1)*GSL_W%AXM(1:NN))
      Fob_u = 2.*Pie
     $     *SUM( LAS_W%SOL_BOT(NNv+1:2*NNv:Nvs,1)*GSL_W%AXM(1:NN) )

      IF( Mh == 1 .AND. OUT_FLUX ) THEN
         WRITE(*,'(18x,A,26x,A)') 
     $        'fluxes at top','fluxes at bottom'
         WRITE(*,'(15x,A,15x,A, 17x,A,15x,A)') 
     $        'down','up', 'down','up'
         WRITE(*,'(7x,2(2x,E15.7),4x,2(2x,E15.7))') 
     $        Fot_d, Fot_u, Fob_d, Fob_u      
         D_top = Fot_d - Fot_u ; D_bot = Fob_d - Fob_u
         WRITE(*,*) 
         WRITE(*,'(11x,A,5x,A,5x,A)') 
     $        'incoming at top','"outgoing" at bottom','diff [%]'
         WRITE(*,'(11x,E15.7, 7x,E15.7, 3x,E15.7)') 
     $        D_top,  D_bot, (D_top - D_bot)/D_top*100.

         WRITE(*,'(20x,A)') 
     $        '   -----------------------------------------'
      END IF

      Fs_OUT(I_TER) = Fs_OUT(I_TER) +  (Fob_d - Fob_u)/Ri2

      IF( Mh == 1 .AND. OUT_FLUX ) THEN
         WRITE(*,*) '       Downward fluxes in the refractive region'
         dir = 0.d0
         IF( UDB_A%SPECULAR ) dir = Pie*SSB_W%EXS(1,1)*GSL_W%MuS(1)
         diff = 2.*Pie*
     $        SUM( LAS_W%SOL_TOP(NSTR_T+1:NN,1)*GSL_W%AXM(NSTR_T+1:NN) )

         WRITE(*,'(14x,A,10x,A,10x,A)') 'direct','diffuse','total'
         WRITE(*,'(7x,3(2x,E15.7))') dir, diff, diff + dir
         WRITE(*,*) ' '

         WRITE(*,*) '   Upward flux from water  '
         WRITE(*,*) '        in the atmosphere <rough>'
         IF( UDB_A%SPECULAR ) THEN
            WRITE(*,*)  '       not defined '
         ELSE
            DO K = 1,NN_A
               Tes(K) = SUM( UDB_W%Twa_GG(K,1:NN_W,Mh)*
     $            LAS_W%SOL_TOP(NN_W+1:2*NN_W,1)*GSL_W%AG(:) ) *2.d0*Pie
            END DO
            Fot_u = 2.*Pie*SUM( Tes(:)*GSL_A%AXM(1:NN_A) )
            WRITE(*,'(7x,3(2x,E15.7))') Fot_u
         ENDIF

         WRITE(*,*) '   Upward flux from water '
         WRITE(*,*) '        in the atmosphere <Fresnel>'
         DO K = 1,NN_A
            Tes(K) = (1.d0 - UDB_A%Fre_R(K))/Ri2
     $           *LAS_W%SOL_TOP(NN_W+NSTR_T+K,1)
         END DO
         Fot_u = 2.*Pie
     $        *SUM( Tes(:)*GSL_A%AXM(1:NN_A) )
         WRITE(*,'(7x,3(2x,E15.7))') Fot_u


         WRITE(*,*) '       Total upward flux in water'
         Fot_u = 2.*Pie
     $        *SUM( LAS_W%SOL_TOP(NNv+1:2*NNv:Nvs,1)
     $        *GSL_W%AXM(1:NN) )
         WRITE(*,'(7x,3(2x,E15.7))') Fot_u


      ENDIF
c      WRITE(*,*) '   Upper boundary condition in refractive range'
c      K = 1
cc      DO I = NSTR_T+1,NN
cc         WRITE(*,'(4x,I3,2(2x,e15.7))') I, 
cc     $        LAS_W%SOL_TOP(I,1) - LAS_W%SOL_TOP(NN+I,1)*UDB_A%Fre_R(K), 
cc     $        LAS_A%SOL_BOT(K,1)*(1.d0 - UDB_A%Fre_R(K))*Ri2
cc         K = K + 1
cc      ENDDO
c      WRITE(*,*) '   NSTR_T = ', NSTR_T
c      DO I = NSTR_T+1,NN
c         WRITE(*,'(4x,I3,2(2x,e15.7))') I, 
c     $        LAS_W%SOL_TOP(I,1), 
c     $        LAS_W%SOL_TOP(NN+I,1)*UDB_A%Fre_R(K) 
c     $        + LAS_A%SOL_BOT(K,1)*(1.d0 - UDB_A%Fre_R(K))*Ri2
c         K = K + 1
c      ENDDO

c      STOP '   in "FLUXES_OCEAN" '

      END SUBROUTINE FLUXES_OCEAN

      END
