!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 24 Oct 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine LG_Init (Mode, TH_Pert, GeoD, *)
  ! Called by rtm_init.f
  ! This subroutine creates structures containing all geometrical
  ! parameters along all line-of-sights and solar beams
  ! LVG - 2D structure relatated to the viewing geometry (grid, viewing angle,
  !       scattering angles, selection indexes, path length, etc.)
  ! LSG - 3D structure related to the solar illumination (solar zenith angle,
  !       relative azimuth, single scattering angles, etc.)
  ! LSG%DSB - 1D substructure containing variables which relate to the direct
  !           solar beam (solar zenith angles, path lengths, etc.)
  USE GRIDS,      ONLY: NSGP => NSOL, SOL_GRD, ZZZ, NLAYERS
  USE LOG_PARS,   ONLY: EU
  USE NODES,      ONLY: N_NODE, MJU_NODE, SIN_NODE, PHI_NODE
  USE SPHER_PARS, ONLY: NN_STD => NN, ALT_OUT, ANG_FOV_Regular => ANG_FOV, &
                        ANG_FOV_Perturb => ANG_FOV_S, ANG_OUT, &
                        SET_OUT_ALT, N_SOL_ANG, N_LOS_ANG, N_FOV_ANG, &
                        N_AZ_ANG, ANG_SEL_FLAG, Const_ang, NOA => N_OBS_ALT, &
                        LOS_DEF_FLAG, PE_MODE
  use ST_VARS,    ONLY: RTM_MODE, RTM_CORE, RTM_TYPE, INT_OUT, RAD_FIELD, Sel_los,&
                        Cr_Sel_los, PreCalcGeom
  USE TYPES,      ONLY: KND, AngInit, Geometry_Data

  LOGICAL  TH_Pert
  CHARACTER (LEN = *)  Mode
  TYPE (Geometry_Data) GeoD
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'LG_Init'
  CHARACTER (LEN=:), ALLOCATABLE :: FINE_MODE
  INTEGER  NSol, NVA, N_Alts, NPhi, II, KK, NN, IV, IA, IS, IP, ILOS, NVPA, IPP
  LOGICAL  Const_ang_loc
  Logical, Allocatable:: ErrFlg(:,:)
  REAL (KND), POINTER :: ANG_FOV(:,:,:)
  REAL (KND)  SVA
  TYPE (AngInit), ALLOCATABLE :: ANGS(:)
  !==================================================================
  INTERFACE 
     Subroutine LG_Calc (Alt, ANGS, AltFlag, IDA, LVG, LSG, FINE_MODE, SVA, &
          Const_ang, Mode, Full_Init, ErrFlg)
       USE TYPES,        ONLY: KND, AngInit, LOS_SOLAR_GEOM, LOS_VIEW_GEOM
       INTEGER  IDA
       LOGICAL  AltFlag, Const_ang, Full_Init, ErrFlg
  CHARACTER (LEN=*) Mode
  CHARACTER (LEN=:), ALLOCATABLE :: FINE_MODE
       REAL (KND)  SVA, Alt
       TYPE (AngInit) ANGS
       TYPE (LOS_VIEW_GEOM), TARGET ::  LVG
       TYPE (LOS_SOLAR_GEOM), TARGET :: LSG(:,:)
     END Subroutine LG_Calc
  END INTERFACE
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ! Deallocate arrays
  Call DeAllocate_Arrays

!  WRITE(*,*) '   ==> Mode in "lg_init" ', MODE

  IF( RTM_MODE=='spher_alb'.AND.Mode=='OUT'.AND.(.NOT.RAD_FIELD) ) RETURN

  IF( RTM_TYPE /= 'spher_scat' ) THEN
     IF( RTM_MODE == 'flux'.AND.RTM_CORE == 'DOM'.AND.Mode=='OUT') RETURN
  ENDIF

  IF( RTM_MODE == 'vod' ) RETURN
     
  IF (TH_Pert) THEN
     ANG_FOV => ANG_FOV_Perturb
  ELSE
     ANG_FOV => ANG_FOV_Regular
  END IF

  GeoD%AltFlg = .FALSE. ; Const_ang_loc = .FALSE.
  FINE_MODE = 'std'

  SELECT CASE (Mode)
  CASE ('OUT')
     NN = NN_STD
     Const_ang_loc = Const_ang
     
     SELECT CASE (RTM_MODE)
     CASE ('flux','spher_alb')
        NSol = N_SOL_ANG ; NVA = N_NODE ;  N_Alts = NLAYERS ; NPhi = 1
        ALLOCATE(GeoD%Alt, SOURCE = ZZZ)
        ALLOCATE(ANGS(NVA), GeoD%IDA(N_Alts))
        GeoD%IDA = [(IA, IA = 1,N_Alts)]
        DO II = 1, NVA
           ANGS(II)%CVA = MJU_NODE(II)
           ALLOCATE(ANGS(II)%PHI(1)) ;  ALLOCATE(ANGS(II)%PHI(1)%CSZA(NSol))
           ANGS(II)%PHI(1)%SAA = PHI_NODE(II) 
           ANGS(II)%PHI(1)%CSZA(:) = ANG_OUT(1,:)
        END DO
        WRITE(*,*) '   "lg_init" ALLOCATE: flux NVA = ', NVA
        write(*,*) '   Nsol, NPHI ', Nsol, NPHI
        write(*,*) '   shape(GeoD%Alt) ', shape(GeoD%Alt) 
        WRITE(*,*) '              GeoD%AltFlg - ', GeoD%AltFlg
        WRITE(*,*) '              GeoD%IDA:     ', GeoD%IDA 
     CASE DEFAULT
        N_Alts = 1 ;  ALLOCATE(GeoD%Alt(1), GeoD%IDA(1))
        GeoD%Alt(1) = ALT_OUT(1)
        IF(SET_OUT_ALT)  THEN
           GeoD%AltFlg = .TRUE. ;  GeoD%IDA(1) = 0
        ELSE
           GeoD%IDA(1) = MINLOC(ABS(ZZZ - ALT_OUT(1)),1)
        END IF
        
        NVA = N_LOS_ANG*N_FOV_ANG 
        IF(ANG_SEL_FLAG.EQ.'all') THEN
           NSol = N_SOL_ANG ;  NPHI = N_AZ_ANG
        ELSE
           NSol = 1 ;  NPHI = 1
        END IF
!        WRITE(*,*) '   "lg_init" ALLOCATE: not flux NVA = ', NVA
!        write(*,*) '   Nsol, NPHI ', Nsol, NPHI
!        write(*,*) '   shape(GeoD%Alt) ', shape(GeoD%Alt) 
!        WRITE(*,*) '                  GeoD%AltFlg - ',GeoD%AltFlg
!        WRITE(*,*) '              GeoD%IDA: ', GeoD%IDA 
        
        ALLOCATE(ANGS(NVA))
        IV = 0
        NVPA = N_LOS_ANG*NPhi
        DO II = 1, NVPA, NPhi
           DO KK =  1, N_FOV_ANG
              IV = IV + 1
              ANGS(IV)%CVA = ANG_FOV(2,II,KK)
              ALLOCATE(ANGS(IV)%PHI(NPHI))
              DO IP = 1, NPHI
                 IPP = II + IP - 1
                 ANGS(IV)%PHI(IP)%SAA = ANG_FOV(3,IPP,KK)
                 ALLOCATE(ANGS(IV)%PHI(IP)%CSZA(NSol))
                 ANGS(IV)%PHI(IP)%CSZA(:) = ANG_FOV(1,IPP:NVPA*NSol:NVPA,KK)
              END DO
           END DO
        END DO

        ALLOCATE(GeoD%ILO_SVFP(NVA,NSol,NPhi))
        ILOS = 0
        DO IS = 1, NSol
           DO IV = 1, N_LOS_ANG
              DO IP = 1, NPhi
                 ILOS = ILOS + 1
                 DO KK = 1, N_FOV_ANG
                    GeoD%ILO_SVFP((IV - 1)*N_FOV_ANG + KK,IS,IP) = ILOS
                 END DO
              END DO
           END DO
        END DO
     END SELECT  ! CASE not 'flux' and not 'spher_alb'
! ========================= End of case 'OUT' ===========================
     
  CASE ('ITR')
     NSol = NSGP ; NVA = N_NODE ;  NPhi = 1; N_Alts = NLAYERS
     NN = NN_STD
     ALLOCATE(GeoD%Alt, SOURCE = ZZZ)
     ALLOCATE(ANGS(NVA), GeoD%IDA(N_Alts))
     GeoD%IDA = [(IA, IA = 1,N_Alts)]
     DO II = 1, NVA
        ANGS(II)%CVA = MJU_NODE(II)
        ALLOCATE(ANGS(II)%PHI(1)) ;  ALLOCATE(ANGS(II)%PHI(1)%CSZA(NSol))
        ANGS(II)%PHI(1)%SAA = PHI_NODE(II) 
        ANGS(II)%PHI(1)%CSZA(:) = SOL_GRD(1:NSol)
     END DO
! ---------- End of case 'ITR' -----------
     
  CASE ('PS')
     NVA = 1 ;  NPhi = 1;  N_Alts = 1
     ALLOCATE(GeoD%Alt(1), ANGS(1), GeoD%IDA(N_Alts))
     GeoD%IDA = [(IA, IA = 1,N_Alts)] ;  GeoD%Alt(1) = ZZZ(1)
     ANGS(1)%CVA = 1.0D0
     ALLOCATE(ANGS(1)%PHI(1))
     ANGS(1)%PHI(1)%SAA = 0.0D0
     
!     IF(RTM_TYPE == 'ps_scat'.AND.RTM_CORE == 'DOM') THEN
     IF(RTM_CORE == 'DOM') THEN
        FINE_MODE = 'all'
     ELSE
        FINE_MODE = 'none'
     END IF
     NSol = NSGP ;  ALLOCATE(ANGS(1)%PHI(1)%CSZA(NSol))
     ANGS(1)%PHI(1)%CSZA(:) = SOL_GRD(1:NSol)
! ---------- End of case 'PS' -----------

  END SELECT  ! Mode

  GeoD%NAlts = N_Alts; GeoD%NVA = NVA; GeoD%NSol = NSol; GeoD%NPhi = NPhi
  If(.Not.PreCalcGeom.and.(Mode == 'ITR'.or. Mode=='OUT')) Then
     GeoD%Full_Init = .False.
  Else
     GeoD%Full_Init = .True.
  End If

  ALLOCATE (GeoD%LVG(N_Alts, NVA), GeoD%LSG(N_Alts, NVA, NSol, NPhi),&
       ErrFlg(N_Alts, NVA))
  ErrFlg = .False.

  ! Number of viewing angles (azimuth angles are not included)
  !$OMP PARALLEL DO PRIVATE(IV, IA, SVA) SCHEDULE(DYNAMIC) IF(PE_Mode /= 'OFF')
  DO IV = 1, NVA
!     write(6,*) IV, "of", NVA
     IF(Mode == "ITR" .AND. Sel_los .AND. ABS(ANGS(IV)%CVA) > Cr_Sel_los) Then
        DO IA = 1, N_Alts
           ALLOCATE(GeoD%LVG(IA,IV)%C_VA(1))
           GeoD%LVG(IA,IV)%C_VA(1) = ANGS(IV)%CVA
        End Do
        CYCLE
     End If
     SVA = SQRT(1.0D0 - ANGS(IV)%CVA**2)
     DO IA = 1, N_Alts
        CALL LG_Calc (GeoD%Alt(IA), ANGS(IV), GeoD%AltFlg, GeoD%IDA(IA), &
             GeoD%LVG(IA,IV), GeoD%LSG(IA,IV,:,:), FINE_MODE, SVA, &
             Const_ang_loc, Mode, GeoD%Full_Init, ErrFlg(IA,IV))
     END DO  ! over altitude layers (IA)
  END DO
  
  If(Any(ErrFlg)) GoTo 880

  IF(Mode /= 'ITR') Then
     ALLOCATE( GeoD%VIEW(N_Alts) )
     DO IA = 1, N_Alts 
        DO II = 1, SIZE(GeoD%LVG,2)
           IF( GeoD%LVG(IA,II)%C_VA(1) > 0.d0  ) THEN
              ! Downward pseudo source (observation of upward radiation)
              IF( II==1 )               THEN
                 GeoD%VIEW(IA)%S='up' ; CYCLE
              ENDIF
              IF( GeoD%VIEW(IA)%S=='down' ) THEN
                 GeoD%VIEW(IA)%S='both'; EXIT
              ENDIF
           ELSE
              ! Upward pseudo source (observation of downward radiation)
              IF( II==1 )             THEN
                 GeoD%VIEW(IA)%S='down'; CYCLE
              ENDIF
              IF( GeoD%VIEW(IA)%S=='up' ) THEN
                 GeoD%VIEW(IA)%S='both' ; EXIT
              ENDIF
           ENDIF
        ENDDO  ! over all viewing directions (I)
     ENDDO  ! over positions of observer (IA)
     !  WRITE(*,*) '   All directions are characterized as: ', GeoD%VIEW(1)%S
     
     IF(RTM_CORE=='DOM'.AND.RTM_MODE/='flux'.AND.RTM_MODE/='spher_alb')THEN 
        DO IA = 1, N_Alts
           DO II = 1, SIZE(GeoD%LVG,2)
              IF( GeoD%LVG(IA,II)%C_VA(1) > 0.d0 ) THEN
                 GeoD%LVG(IA,:)%IALS = NOA(IA)
                 GeoD%LVG(IA,:)%IALE = NLAYERS-1
              ELSE
                 GeoD%LVG(IA,:)%IALS  = 1
                 GeoD%LVG(IA,:)%IALE = NOA(IA)-1
              ENDIF
           ENDDO

           IF( GeoD%VIEW(IA)%S=='both' ) THEN
              IF( LOS_DEF_FLAG=='ga' ) THEN
                 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
                 WRITE(EU,*) '"Type of LOS definition" control parameter' &
                      //' can not be set to "ga"'
                 WRITE(EU,*) '    in combination within the option "both".'
                 WRITE(EU,*) '    Check the "control_geom.inp" file.'
                 GOTO 880
              ENDIF
              IF( INT_OUT )  THEN
                 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
                 WRITE(EU,*) 'Interpolation technique can not be used '
                 WRITE(EU,*) '    in combination within the option "both".'
                 WRITE(EU,*) '    Check "control_ac.inp" and "control_geom.inp" files.'
                 GOTO 880
              ENDIF
           ENDIF
        ENDDO
        GeoD%MaxLS = MAXVAL(GeoD%LVG(:,:)%NLS)
     ENDIF  ! option 'DOM' but not 'flux' and not 'spher_alb'
  End If ! Mode /= 'ITR'
     
! Deallocate arrays
  DO II = 1, SIZE(ANGS)
     DO KK = 1, SIZE(ANGS(II)%PHI)
        DeAllocate(ANGS(II)%PHI(KK)%CSZA)
     END DO
     DeAllocate(ANGS(II)%PHI)
  END DO
  DeAllocate(ANGS)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1  

  CONTAINS
!***************************************************************************
  Subroutine DeAllocate_Arrays
    USE ST_VARS,      ONLY: DO_PC, DO_REFRAC, PreCalcAngs
    INTEGER I1, I2, I3, I4, I5, Stat
    IF(.NOT.ALLOCATED(GeoD%Alt)) RETURN
    
    DO I1 = 1,SIZE(GeoD%LVG,1)
       DO I2 = 1,SIZE(GeoD%LVG,2)
          ASSOCIATE(VG => GeoD%LVG(I1,I2))
            If(Allocated(VG%ZLS)) DeAllocate(VG%DS, VG%ZLS, VG%ISEL, VG%IOB)
            DeAllocate(VG%C_VA)
            If(.Not.GeoD%Full_Init) DeAllocate(VG%CVA_TH)
          END ASSOCIATE
          IF(RTM_TYPE == 'transmission') CYCLE
          DO I3 = 1,SIZE(GeoD%LSG,3)
             DO I4 = 1,SIZE(GeoD%LSG,4)
                ASSOCIATE(SG => GeoD%LSG(I1,I2,I3,I4))
                  IF(DO_REFRAC) THEN
                     If (Associated(SG%CG_SZA)) DeAllocate(SG%CG_SZA, Stat = Stat)
                  ELSE
                     NULLIFY(SG%CG_SZA)
                  END IF
                  If(Allocated(SG%C_SA)) DeAllocate(SG%C_SA, SG%S_SA, &
                       SG%RMR, SG%RML)
                  IF(PreCalcAngs) THEN
                     DO I5 = 1, SIZE(SG%DSB)
                        IF(.NOT.Allocated(SG%DSB(I5)%ZSOL)) CYCLE
                        DeAllocate(SG%DSB(I5)%ZSOL, SG%DSB(I5)%DS, &
                             SG%DSB(I5)%ISEL, SG%DSB(I5)%IOB)
                        IF(DO_PC) &
                             DeAllocate(SG%DSB(I5)%C_SZA)
                     END DO
                     DeAllocate(SG%DSB)
                  END IF
                  If(Allocated(SG%C_SZA)) DeAllocate(SG%C_SZA, SG%SAA)
                  If(Allocated(SG%TH)) DeAllocate(SG%TH, SG%REFR_DSB)
                  If(.Not.GeoD%Full_Init) DeAllocate(SG%CSZA_TH, SG%SAA_TH)
                END ASSOCIATE
             END DO
          END DO
       END DO
    END DO
    If (Allocated(GeoD%VIEW)) DeAllocate(GeoD%VIEW)
    DeAllocate(GeoD%Alt, GeoD%IDA, GeoD%LVG, GeoD%LSG)
    IF(ALLOCATED(GeoD%ILO_SVFP)) DeAllocate(GeoD%ILO_SVFP)
    IF(ALLOCATED(GeoD%VIEW)) DeAllocate(GeoD%VIEW)
    
    RETURN
  End Subroutine DeAllocate_Arrays
!***************************************************************************
End Subroutine LG_INIT

!***************************************************************************
Subroutine LG_Calc (Alt, ANGS, AltFlag, IDA, LVG, LSG, FINE_MODE, SVA, &
     Const_ang, Mode, Full_Init, ErrFlg)
  USE GRIDS,     ONLY: Z_BOA, ZZZ
  USE LOG_PARS,  ONLY: EU
  USE LOS_ANGLES,ONLY: Los_Geom_Init, Los_Geom_Sol_Init, Los_Geom_Clean, &
                          Los_Geom
  USE NODES,     ONLY: N_NODE, MJU_NODE, SIN_NODE, PHI_NODE
  USE RefrVar,   ONLY: H_NOREF, COS_NOREF
  USE SPHER_PARS,ONLY: I_FINE, I_FINE_SOL
  USE ST_VARS,   ONLY: PreCalcAngs, EARTH_RADIUS, DO_PC, DO_REFRAC, Pie,&
                       LSA_MODE, RTM_TYPE, PreCalcRtMatr
  USE TOOLS,     ONLY: Height_Grid_V3, RotMatr, LOS_MS_IDX
  USE TYPES,     ONLY: KND, AngInit, LOS_SOLAR_GEOM, LOS_VIEW_GEOM, &
                       LOS_GEOMETRY

  INTEGER  IDA
  LOGICAL  AltFlag, Const_ang, Full_Init, ErrFlg
  CHARACTER (LEN=*) Mode
  CHARACTER (LEN=:), ALLOCATABLE :: FINE_MODE
  REAL (KND)  SVA, Alt
  TYPE (AngInit) ANGS
  TYPE (LOS_VIEW_GEOM), TARGET :: LVG
  TYPE (LOS_SOLAR_GEOM), TARGET :: LSG(:,:)
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'LG_Calc'
  INTEGER  NLS, NTH, KK, I, JJ, IS, IP, ID_N, JLS, IMS1, IMS2
  Logical  ALT_FLAG, REFR_FLG
  REAL (KND) CVA, CSOL, Rel_az
!  REAL Rel_az
  TYPE(LOS_GEOMETRY) :: DSB
  TYPE(LOS_GEOMETRY), TARGET:: LG
  TYPE (LOS_SOLAR_GEOM), POINTER :: PSG
  !==================================================================
  !  CALL ST_TRACK (R_NAME, 1, 'I')
  ErrFlg = .False.

  SELECT CASE (LSA_MODE)
  CASE('CDI_PS', 'CDI_PP', 'DOM_NS')
     LVG%H_0 = -9999.
  CASE DEFAULT
     LVG%H_0 = (EARTH_RADIUS + Alt)*SVA - EARTH_RADIUS
  END SELECT
 
  CVA = ANGS%CVA

  ! Check if we need refraction along the LOS
  REFR_FLG = .FALSE.
  Set_Refr_Flag: Block
    SELECT CASE (LSA_MODE)
    CASE('CDI_PS', 'CDI_PP', 'DOM_NS')
       Exit Set_Refr_Flag
    END SELECT
    IF(.NOT.DO_REFRAC) Exit Set_Refr_Flag
    IF(ABS(CVA) >= COS_NOREF) Exit Set_Refr_Flag
    IF(CVA <= 0.0D0) THEN
       IF(Alt < H_NOREF) REFR_FLG = .TRUE.
    ELSE
       IF(LVG%H_0 < H_NOREF) REFR_FLG = .TRUE.
    END IF
  END Block Set_Refr_Flag
  LVG%REFR_FLG = REFR_FLG
  
  IF(REFR_FLG) CALL TH_REFR_V2(Alt, SVA, LVG%H_0, *880)

  If(Full_Init) Then
     Call Los_Geom_Init (LG, LVG, Alt, AltFlag, IDA, CVA, SVA, Const_ang, &
          FINE_MODE)
     NLS = LVG%NLS
     LVG%CVA_TH => LVG%C_VA(LVG%NTH)
  Else
     Allocate(LVG%C_VA(1), LVG%CVA_TH)
     LVG%C_VA(1) = CVA
  End If
  
  DO IP = 1, SIZE(ANGS%PHI)
     IF(RTM_TYPE == 'transmission') CYCLE
     DO IS = 1, SIZE(ANGS%PHI(IP)%CSZA)
        If(Full_Init) Then
           Call Los_Geom_Sol_Init (LVG, LG, ANGS%PHI(IP)%CSZA(IS), &
                ANGS%PHI(IP)%SAA, LSG(IS,IP), *880)
           LSG(IS,IP)%CSZA_TH => LSG(IS,IP)%C_SZA(LVG%NTH)
           LSG(IS,IP)%SAA_TH => LSG(IS,IP)%SAA(LVG%NTH)
        Else
           Allocate(LSG(IS,IP)%C_SZA(1), LSG(IS,IP)%SAA(1), LSG(IS,IP)%CSZA_TH, &
                LSG(IS,IP)%SAA_TH)
           LSG(IS,IP)%C_SZA(1) =  ANGS%PHI(IP)%CSZA(IS)
           LSG(IS,IP)%SAA(1) = ANGS%PHI(IP)%SAA
           IF(DO_REFRAC) THEN
              ALLOCATE(LSG(IS,IP)%CG_SZA(1))
           ELSE
              LSG(IS,IP)%CG_SZA => LSG(IS,IP)%C_SZA
           END IF
        End If
        
        IF(LSA_MODE == 'DOM_NS'.AND.Mode == 'OUT') CYCLE
        If(PreCalcRtMatr .and. Mode /= 'PS') Then
        CALL LOS_MS_IDX(PSG%CG_SZA, IMS1, IMS2)
!        If(Mode /= 'PS') Then
           Allocate(PSG%C_SA(N_NODE,NLS), PSG%S_SA(N_NODE,NLS), &
                PSG%RML(2,N_NODE,NLS), PSG%RMR(2,N_NODE,NLS))
           JLS = NLS
           IF( NLS == 1 ) JLS = 0
           DO KK = 1, JLS
              If(KK < IMS1 .OR. KK > IMS2) CYCLE
           ! Calculation of the rotational matrices
              Do ID_N = 1, N_NODE
                 Rel_az = PSG%SAA(KK) - PHI_NODE(ID_N)
                 PSG%C_SA(ID_N, KK) = MJU_NODE(ID_N)*LVG%C_VA(KK) + &
                      SIN_NODE(ID_N)*LG%S_VA(KK)*COS(Rel_az) 
                 PSG%C_SA(ID_N,KK) = Min(PSG%C_SA(ID_N,KK), 1.0d0)
                 PSG%C_SA(ID_N,KK) = Max(PSG%C_SA(ID_N,KK), -1.0d0)
                 PSG%S_SA(ID_N,KK) = SQRT(1.d0 - PSG%C_SA(ID_N,KK)**2)
                 
                 CALL RotMatr (LVG%C_VA(KK), LG%S_VA(KK), MJU_NODE(ID_N), &
                      SIN_NODE(ID_N), Rel_az, PSG%C_SA(ID_N,KK), &
                      PSG%S_SA(ID_N,KK), PSG%RML(:,ID_N,KK), &
                      PSG%RMR(:,ID_N,KK))
              End Do
           End DO
        End If
        
        IF(.NOT.PreCalcAngs) CYCLE
        Allocate(LSG(IS,IP)%DSB(NLS)) 
        DO KK = 1, NLS
           ! Calculation of the solar geometry
           IF (LG%C_SZA(KK) < 0.0D0 .AND. LSG(IS,IP)%TH(KK) < Z_BOA) CYCLE
           
           IF(ZZZ(LVG%IOB(KK)) == LVG%ZLS(KK)) THEN
              JJ = LVG%IOB(KK)
              ALT_FLAG = .FALSE.
           ELSE
              JJ = 0
              ALT_FLAG = .TRUE.
           END IF
           
           ASSOCIATE(SG => LSG(IS,IP)%DSB(KK))
             
             CSOL = - LG%C_SZA(KK)
!             CALL HEIGHT_GRID_V2 (I_FINE_SOL, CSOL, LSG(IS,IP)%TH(KK), &
!                  LVG%ZLS(KK), ALT_FLAG, JJ, SG%NTH, SG%NLS, SG%ZSOL, &
!                  SG%ISEL, SG%IOB, SG%TH_FLG, SG%FP_FLG)
             CALL HEIGHT_GRID_V3 (I_FINE_SOL, CSOL, LSG(IS,IP)%TH(KK), &
                  LVG%ZLS(KK), ALT_FLAG, JJ, SG%NTH, SG%NLS, SG%ZSOL, &
                  SG%ISEL, SG%IOB, SG%TH_FLG, SG%FP_FLG)
            
             ! Local structure for the direct solar beam
             ALLOCATE(DSB%DS(2,SG%NLS), DSB%DS_C(SG%NLS - 1), &
                  DSB%S_VA(SG%NLS), DSB%C_VA(SG%NLS))

             DSB%C_VA(1) = LG%C_SZA(KK) ;  DSB%S_VA(1) = LG%S_SZA(KK)
             CALL Los_Geom (LSG(IS,IP)%REFR_DSB(KK), SG%ZSOL, SG%NLS, &
                  SG%ISEL, SG%NTH, SG%TH_FLG, SG%FP_FLG, DSB)
             ALLOCATE(SG%DS, SOURCE=DSB%DS)
             IF (DO_PC) THEN
                ALLOCATE(SG%C_SZA(2:SG%NLS))
                SG%C_SZA(2:) = DSB%C_VA(2:)
             END IF
           END ASSOCIATE
           DEALLOCATE(DSB%DS, DSB%DS_C, DSB%S_VA, DSB%C_VA)
        END DO ! Points along the line of sight
     END DO ! Solar zenith angles
  END DO ! Azimuth angles

   
  If(Full_Init) Call Los_Geom_Clean (LG)
  !  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  ErrFlg = .True.
  RETURN
!***************************************************************************
End Subroutine LG_Calc
!***************************************************************************
