C********************************************************************C
C
C	SCIATRAN - Version 3.0 - Last modification: 04 Apr 2008
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C

      Subroutine LIN_SEARCH ( X, XL, K, N, A, ERROR  )

c     This particular operation occurs many times at different places in
c     the  GOME software.  Given an abscissa X to which one wants to
c     interpolate, search through the array A(N) of X-values for those
c     points either side of X.

c     Returns index K, where A(K) is nearest to X on the lower side.
c     Returns also XL = ( x - a(k) ) / ( a(k+1) - a(k) ).

c     NOTE: Extrapolation not implemented (program will stop).
c           Array XL() is assumed to increase monotonically. 

      CHARACTER (LEN = 10), PARAMETER :: ROUTINE_NAME = 'LIN_SEARCH'

c     Input arguments.

      Integer            n
      Double precision   x, a (n)
      logical            error

c     Output arguments.

      Integer            k
      Double Precision   xl

c     Local variables.

      Integer         j, i

C     ************************************************************

      error = .false.

      xl = 0.0d0
      k  = 0
      J  = 0


C     Check input and return with ERROR if necessary
C     ----------------------------------------------

      if ( n.eq.0 ) then
         error = .true.
         call write_message(routine_name, 'ERROR', 
     $   'Array dimension N equal 0')
         Return
      endif

      do i = 1, n-1 
         if ( a(i+1).le.a(i) ) then
            error = .true.
            call write_message(routine_name, 'ERROR', 
     $      'Array A dereases')
            Return
         endif
      enddo

      If ( x .lt. a(1) .or. x .gt. a(n) ) Then
         error = .true.
         call write_message(routine_name, 'ERROR', 
     $   'X outside boundaries (extrapol.req.)')
         Return
      End If

      If ( x .eq. a(1) ) Then
         j  = 1
      Else
         Do  J = 1, N - 1
            If ( x. gt. a(j) .and. x .le. a(j+1) ) Go To 111
         End Do
c        j = n - 1      ! only if no jump to 111 has occurred
         error = .true.
         call write_message(routine_name, 'ERROR', 
     $   'Expected jump to 111 did not happen')
         return
      End If

 111  Continue
      k = j

      If ( k .eq. n .or. (a(k+1).eq.a(k)) ) Then
         error = .true.
         call write_message(routine_name, 'ERROR', 
     $   'XL cannot be evalutated')
         Return
      Else
         xl = ( x - a(k) ) / ( a(k+1) - a(k) )
      End If

      Return
      End

C********************************************************************C
      SUBROUTINE LIN_SEARCH_NEW ( X, XL, KK_1, KK_2, N, A )
c     NOTE: Extrapolation with constant.

      CHARACTER (LEN = 14), PARAMETER :: R_NAME = 'LIN_SEARCH_NEW'
      INTEGER  N, KK_1, KK_2, KM
      DOUBLE PRECISION   X, XL, A(N)

      KM = MINLOC(A(:),1) ; IF( X <= A(KM) ) GOTO 100
      KM = MAXLOC(A(:),1) ; IF( X >= A(KM) ) GOTO 100

      KK_1 = MINLOC(DABS(A(:) - X), 1, A(:) <= X)
      KK_2 = MINLOC(DABS(A(:) - X), 1, A(:)  > X)      
      XL = (X - A(KK_1))/(A(KK_2) - A(KK_1))
      RETURN

 100  KK_2 = KM  ; KK_1 = KM ; XL = 0.D0
      RETURN
      END
C********************************************************************C
