C********************************************************************C
C
C	SCIATRAN - Version 1.1 - Last modification: 1 Feb 1999
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C

************************************************************
*     -Routine           lin_search2                       *
*     -Author:           MV                                *  
*     -Category:         numerical tool                    *   
*     -Dates:            first 7.08.98                     * 
*     -History:                                            * 
*     -Description:      A wrapper Routine for             *
*                        LIN_SEARCH- since:                *
*                        lin_search returns index K, where *
*                        A(K) is NEAREST TO X (the desired *
*                        value) ON THE LOWER SIDE.         *
*                        This routine returns the value    *
*                        with the smallest distance by     *
*                        checking neighbors A(K-1) and     *
*                        A(K+1).                           *
*                        See lin_search for more info      *
*     -Arguments:                                          *
*         -desired value         (1s)                      * 
*         -relative distance     (1s)                      * 
*         -position of nearest                             *
*           value in array       (1s)                      *
*         -size of input vector  (1s)                      * 
*         -input vector          (1v)                      * 
*         -error flag            (1s)                      * 
*     -Notes:                                              *
*     -Final ouput:      position (index) of nearest       *
*                        value in array                    * 
*     -Reference:        lin_search                        *   
*     -Legend:           contenuation line label:          *      
*                        "I"  - input                      *    
*                        "O"  - output                     *  
*                        "M"  - modified within routine    *  
************************************************************  
      Subroutine lin_search2( 
     I     X, 
     O     XL, K, 
     I     N, A, 
     M     error)


c     Routine name

      character*11      routine_name
      parameter        (routine_name = 'lin_search2')

c     Input arguments. 
 
      Integer            n
      Double precision   x, a (n)
      logical            error 

c     Output arguments. 

      Integer            k 
      Double Precision   xl 

c     Local Variables.

      Double Precision  dist
      Double Precision  pre_dist
      Double Precision  act_dist 
      Double Precision  fol_dist 

      error=.false.
      
      call lin_search(X, XL, K, N, A, error)

      if(K .eq. 0) then
         call write_message(routine_name, 'ERROR',
     $        'Cannot find value in Array')
         error=.true.
         return
      End If


c     The first element: we don't
c     have any neighbor on the left-
c     handside: 
      if(K .eq. 1) then
         act_dist = dabs(X-A(K))
         fol_dist = dabs(X-A(K+1))
         if(dist .eq. act_dist) K=K
         if(dist .eq. fol_dist) K=K+1
c     The last element: we don't
c     have any neighbor on the right-
c     handside:  
      else if(K .eq. N) then
         act_dist = dabs(X-A(K))
         pre_dist = dabs(X-A(K-1))            
         dist     = dmin1(act_dist,pre_dist)
         if(dist .eq. act_dist) K=K
         if(dist .eq. pre_dist) K=K-1

c     Everything inbetween: we 
c     neighbors on both sides:
      else 
         act_dist = dabs(X-A(K))
         fol_dist = dabs(X-A(K+1))
         pre_dist = dabs(X-A(K-1))
         dist     = dmin1(act_dist,fol_dist,pre_dist)
         if(dist .eq. act_dist) K=K
         if(dist .eq. fol_dist) K=K+1 
         if(dist .eq. pre_dist) K=K-1
      endif 
      
      
      Return
      End
C********************************************************************C
      
      
