C********************************************************************C
C
C	SCIATRAN - Version 1.2 - Last modification: 30 March 2000
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
C ******************************************************************
C
      Subroutine LINTP2 (n1,x1,y1,n2,x2,y2,error)

C************************************************************************
C
C     SUBROUTINE FOR LINEAR INTERPOLATION OF FUNCTION
C     -----------------------------------------------

C     Input parameters:

C     X1(N1) - Initial argument grid
C     Y1(N1) - Initial function
C     X2(N2) - Necessary argument grid

C     Output parameters:

C     Y2(N2) - Function calculated on new argument grid

C     Other information:

C     This program is specially designed to work in the case when
C     one of the arguments increases and the other decreases !
C     In the case of extrapolation output is constant.
C     The arguments must increase or decrease MONOTONICLY.


c     Routine name

      CHARACTER (LEN = 6), PARAMETER :: ROUTINE_NAME = 'LINTP2'

      Integer           n1,n2
      Double Precision  x1(n1),y1(n1),x2(n2),y2(n2)
      Integer           inv, n2p, j, i2, i1, j2, i
      Double Precision  x, yp, xp, s
      logical           error

      Double Precision  test

c     Initialize some variables
c     (mainly to keep the compilers quiet)
c     ------------------------------------
      n2p = 0
      error = .false.


      If ( (n1 .le. 0    ) .or. (n2 .le. 0     ) .or.
     $     (n1 .gt. 32767) .or. (n2 .gt. 32767) ) Then
         error = .true.
         call write_message(routine_name, 'ERROR', 
     $   'Wrong array dimensions')
         return
      End If
C
C --- Checking the condition for inverting arrays ---
C
      inv = 0

      test = ( x1(n1) - x1(1) ) * ( x2(n2) - x2(1) )

      If ( .not. ((n1 .eq. 1) .or. (n2 .eq. 1)) ) Then

c     This handles the cases of test <= 0.0d0.
c     test > 0 must automatically follow after this IF condition
c     ----------------------------------------------------------
         If ( test .le. 0.0d0 ) Then 
c
C --- Inversion of new grid ---
            inv = 1
            n2p = n2 / 2
            Do j = 1, n2p
               xp = x2(j)
               x2(j) = x2(n2-j+1)
               x2(n2-j+1) = xp
            End Do

         End If
      End If
C
C --- Main block ---
C
      If ( n1 .ne. 1) Then
         s = DSIGN( 1.0d0, x1(n1)-x1(1) )
         i2 = 1
         i1 = 2

 1       Continue
         If ( (x2(i2)-x1(1))*s .le. 0.0d0) Then
            y2(i2) = y1(1)
            i2     = i2 + 1
            If ( i2 .gt. n2 ) Go To 999
            Go To 1
         End If

 3       Continue
         If ( (x2(i2) - x1(i1))*s .le. 0.0d0 ) Then
            x      = (x2(i2) - x1(i1-1)) / (x1(i1) - x1(i1-1))
            y2(i2) = y1(i1)*x + y1(i1-1)*(1.0d0-x)
            i2     = i2+1
            If ( I2 .GT. N2 ) Go To 999
            Go To 3
         End If

         i1 = i1+1
         If ( i1 .le. n1 ) Go To 3

         Do j2 = i2, n2
            y2(j2) = y1(n1)
         End Do
         GOTO 999
      End If

      Do  i=1, n2
         y2(i) =y1(1)
      End Do

 999  Continue
C
C --- Checking the condition for back inversion ---
C
      If ( INV .eq. 1 ) Then
C
C --- Back inversion ---
C
         Do J = 1, N2P
            XP = X2(J)
            YP = Y2(J)
            X2(J) = X2(N2-J+1)
            Y2(J) = Y2(N2-J+1)
            X2(N2-J+1) = XP
            Y2(N2-J+1) = YP
         End Do
         
      End If

C
C --- Exit block ---
C
      Return
      End
C********************************************************************C
