!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 28 Sep 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
ModulE LOS_Angles

 Use Log_Pars,      Only: EU
  Use Refract_Vars, Only: Refr_HiRes
  Use ST_Vars,      Only: Pie, EARTH_RADIUS, LSA_MODE, DO_REFRAC
  Use Types,        Only: Knd, LOS_Geometry, LOS_Solar_Geom, LOS_View_Geom

  INTEGER, ALLOCATABLE :: N_Below(:), N_BE(:), N_AF(:)
  INTEGER, TARGET, ALLOCATABLE :: Lim_gr(:,:,:), IND_LIM_A(:,:), IND_LIM_B(:,:)
  
  TYPE VERT_GRIDS
     REAL (KND), ALLOCATABLE :: C(:), B(:), RC(:), RB(:)
  END TYPE VERT_GRIDS

  PRIVATE :: VERT_GRIDS
CONTAINS
!***************************************************************************
  SUBROUTINE Los_Geom (REFR_LINE, Z_LC, N_DS, IDX, N_TH, TH_FLAG, NEW_LEVEL, LG)
    ! Called by "lg_init.f90" and "integrate_los.f90"   
    
    LOGICAL     TH_FLAG, NEW_LEVEL, REFR_LINE
    INTEGER     N_DS, N_TH, IDX(N_DS)
    REAL (KND)  Z_LC(N_DS)
    TYPE(LOS_GEOMETRY):: LG
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Los_Geom'
    INTEGER  I, J
    REAL (KND)  CVA_LB(2:N_DS)
    TYPE(VERT_GRIDS):: ZL
    !==================================================================
!    CALL ST_TRACK (R_NAME, 1, 'I')

    ALLOCATE (ZL%C(N_DS), ZL%B(N_DS + 1), ZL%RC(N_DS), ZL%RB(N_DS + 1))
    ! Z_LC -  Centers of the layers
    ! Z_LB -  Bountaries of the layers (half of the vertical distance
    !         between the layers). The first and the last eleements are
    !         equal to corresponding layer centers.
    ! Centers of the layers:
    ZL%C(:) = Z_LC(:)
    ZL%RC(:) = ZL%C(:) + EARTH_RADIUS
    ! Level bondaries:
    ZL%B(1) = ZL%C(1) ;  ZL%B(N_DS + 1) = ZL%C(N_DS) 
    DO J = 2, N_DS
       ZL%B(J) = (ZL%C(J - 1) + ZL%C(J))/2.0D0
    END DO
    ZL%RB(:) = ZL%B(:) + EARTH_RADIUS

    IF(REFR_LINE) THEN ! Refractive ray tracing
       IF(Refr_HiRes) THEN
          CALL Los_Geom_Refr_HiRes (ZL, N_DS, N_TH, LG)
       ELSE
          CALL Los_Geom_Refr_LowRes_A (ZL, N_DS, IDX, N_TH, TH_FLAG, &
!          CALL Los_Geom_Refr_LowRes_B (ZL, N_DS, IDX, N_TH, TH_FLAG, &
               NEW_LEVEL, LG)
       END IF

       FORALL(I = 2:N_TH) LG%C_VA(I) = &
            SIGN(SQRT(1.0D0 - LG%S_VA(I)**2),LG%C_VA(1))
       FORALL(I = N_TH + 1:N_DS) LG%C_VA(I) = &
            - SIGN(SQRT(1.0D0 - LG%S_VA(I)**2),LG%C_VA(1))
       
    ELSE ! Non-refractive ray tracing
       LG%S_VA(2:N_DS) = MIN(ZL%RC(1)/ZL%RC(2:N_DS)*LG%S_VA(1),1.0D0)
       FORALL(I = 2:N_TH)
          LG%C_VA(I) = SIGN(SQRT(1.0D0 - LG%S_VA(I)**2),LG%C_VA(1))
          CVA_LB(I) = SIGN(SQRT(1.0D0 - &
               MIN(ZL%RC(1)/ZL%RB(I)*LG%S_VA(1),1.0D0)**2),LG%C_VA(1))
       END FORALL
       FORALL(I = N_TH + 1:N_DS)
          LG%C_VA(I) = - SIGN(SQRT(1.0D0 - LG%S_VA(I)**2),LG%C_VA(1))
          ! Layer boundaries at a half vertical distance between the levels 
          CVA_LB(I) = -SIGN(SQRT(1.0D0 - &
               MIN(ZL%RC(1)/ZL%RB(I)*LG%S_VA(1),1.0D0)**2),LG%C_VA(1))
       END FORALL
           
       LG%DS(1,1) = 0.D0
       FORALL(I = 2:N_DS) &
            LG%DS(1,I) = ABS(ZL%RC(I)*LG%C_VA(I) - ZL%RB(I)*CVA_LB(I))
       
       LG%DS(2,N_DS) = 0.D0
       FORALL(I = 1:N_DS - 1) &
            LG%DS(2,I) = ABS(ZL%RC(I)*LG%C_VA(I) - ZL%RB(I + 1)*CVA_LB(I + 1))

       DO I = 1, N_DS - 1
          LG%DS_C(I) = (LG%DS(2,I) + LG%DS(1,I + 1))
       END DO
    END IF
    
    DEALLOCATE (ZL%C, ZL%B, ZL%RC, ZL%RB)
!    CALL ST_TRACK (R_NAME, 1, 'O')
  END SUBROUTINE Los_Geom

!***************************************************************************
  SUBROUTINE Los_Geom_Refr_LowRes_A (ZL, N_DS, IDX, N_TH, TH_FLAG, &
       NEW_LEVEL, LG)

    USE REFRACT_VARS, ONLY: REFR_IND, INT_TERM, RI_Z, IT_FINE
    USE ST_VARS,      ONLY: ER => EARTH_RADIUS
    
    LOGICAL     TH_FLAG, NEW_LEVEL
    INTEGER     N_DS, N_TH, IDX(N_DS)
    TYPE(VERT_GRIDS):: ZL
    TYPE(LOS_GEOMETRY):: LG
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Los_Geom_Refr'
    INTEGER    JJ, J, I
    REAL (KND) IT_LC(N_DS), IT_LB(N_DS + 1), ASIN_LC(N_DS), ASIN_LB(N_DS + 1)
    REAL*16    D_ANG(2, N_DS)
    REAL (KND) DD, RI_1, CONST
    !==================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    IF(NEW_LEVEL) THEN
       CALL GET_IDX_BELOW (ZL%C(1), JJ, DD)
       RI_1 = ZL%RC(1)*(REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD))
       IT_LC(1) = INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD)
    ELSE
       RI_1 = RI_Z(IDX(1))
       IT_LC(1) = IT_FINE(IDX(1))
    END IF
    ! We consider each layer to be homogeniuos with a mean refractive index
    ! Snell's law accounting for the curvature of the medium:
    ! r*n*sin(Theta) = const 
    CONST = LG%S_VA(1)*RI_1
    
    ! Grid levels
    DO J = 1, N_DS
       RI_1 = RI_Z(IDX(J))
       IT_LC(J) = IT_FINE(IDX(J))
       LG%S_VA(J) = MIN(CONST/RI_1,1.0D0)
       ASIN_LC(J) = ASIN(LG%S_VA(J))
    END DO

    IF(TH_FLAG) THEN
       CALL GET_IDX_BELOW (ZL%C(N_TH), JJ, DD)
       RI_1 = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
       IT_LC(N_TH) = INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD)
       LG%S_VA(N_TH) = MIN(CONST/ZL%RC(N_TH)/RI_1,1.0D0)
       ASIN_LC(N_TH) = ASIN(LG%S_VA(N_TH))
    END IF

    ! Grid boundaries
    DO J = 2, N_DS
       CALL GET_IDX_BELOW (ZL%B(J), JJ, DD)
       RI_1 = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
       IT_LB(J) = INT_TERM(JJ - 1)*DD + INT_TERM(JJ)*(1.0D0 - DD)
       ASIN_LB(J) = ASIN(MIN(CONST/ZL%RB(J)/RI_1,1.0D0))
    END DO
    ASIN_LB(1) = ASIN_LC(1) ;  ASIN_LB(N_DS + 1) = ASIN_LC(N_DS)
    IT_LB(1) = IT_LC(1) ;  IT_LB(N_DS + 1) = IT_LC(N_DS)
    
    D_ANG(1,1) = 0.0D0
    FORALL(J = 2:N_DS) D_ANG(1,J) = &
         (IT_LB(J) + IT_LC(J))*ABS(ASIN_LC(J) - ASIN_LB(J))/2.0D0
    D_ANG(2,N_DS) = 0.0D0
    FORALL(J = 1:N_DS - 1) D_ANG(2,J) = &
         (IT_LC(J) + IT_LB(J + 1))*ABS(ASIN_LC(J) - ASIN_LB(J + 1))/2.0D0
    
    ! We use the geometrical path between the entry and exit points of the
    ! layer because is was found to be nearly equal to the refractive path
    ! calculated with the high resolution routine
    LG%DS(1,1) = 0.0D0
!    FORALL(J = 2:N_DS) LDS(1,J) = SQRT(ZL%RB(J)**2 + ZL%RC(J)**2 &
!         - 2.0D0*ZL%RB(J)*ZL%RC(J)*COS(D_ANG(1,J)))
    FORALL(J = 2:N_DS) LG%DS(1,J) = SQRT((ZL%C(J) - ZL%B(J))**2 + &
         2.0D0*(1.0D0  - COS(D_ANG(1,J)))*(ER**2 + ER*(ZL%C(J) + ZL%B(J))&
         + ZL%C(J)*ZL%B(J)))
    LG%DS(2,N_DS) = 0.0D0
!    FORALL(J = 1:N_DS - 1) LDS(2,J) = SQRT(ZL%RC(J)**2 + ZL%RB(J + 1)**2 &
!         - 2.0D0*ZL%RC(J)*ZL%RB(J + 1)*COS(D_ANG(2,J)))
    FORALL(J = 1:N_DS - 1) LG%DS(2,J) = SQRT((ZL%C(J) - ZL%B(J + 1))**2 + &
         2.0D0*(1.0D0  - COS(D_ANG(2,J)))*(ER**2 + ER*(ZL%C(J) + ZL%B(J + 1))&
         + ZL%C(J)*ZL%B(J + 1)))

    FORALL(I = 1:N_DS - 1) LG%DS_C(I) = (LG%DS(2,I) + LG%DS(1,I + 1))
    CALL ST_TRACK (R_NAME, 1, 'O')
    
    RETURN
  END SUBROUTINE Los_Geom_Refr_LowRes_A

!***************************************************************************
  SUBROUTINE Los_Geom_Refr_LowRes_B (ZL, N_DS, IDX, N_TH, TH_FLAG, &
       NEW_LEVEL, LG)

    USE REFRACT_VARS, ONLY: REFR_IND, RI_Z, RefI
    
    LOGICAL     TH_FLAG, NEW_LEVEL
    INTEGER     N_DS, N_TH, IDX(N_DS)
    TYPE(VERT_GRIDS):: ZL
    TYPE(LOS_GEOMETRY):: LG
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Los_Geom_Refr_LowRes_B'
    INTEGER    JJ, J, I
    REAL (KND) RI_LC(N_DS), CVA_B1(N_DS), CVA_B2(N_DS), CVA_LC(N_DS)
    REAL (KND) DD, RI_1, CONST
    !==================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    IF(NEW_LEVEL) THEN
       CALL GET_IDX_BELOW (ZL%C(1), JJ, DD)
       RI_LC(1) = (REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD))
       RI_1 = ZL%RC(1)*RI_LC(1)
    ELSE
       RI_1 = RI_Z(IDX(1))
       RI_LC(1) = RefI(IDX(1))
    END IF
    ! We consider each layer to be homogeniuos with a mean refractive index
    ! Snell's law accounting for the curvature of the medium:
    ! r*n*sin(Theta) = const 
    CONST = LG%S_VA(1)*RI_1
    
    ! Grid levels
    DO J = 1, N_DS
       RI_LC(J) = RefI(IDX(J))
       LG%S_VA(J) = MIN(CONST/RI_1,1.0D0)
    END DO

    IF(TH_FLAG) THEN
       CALL GET_IDX_BELOW (ZL%C(N_TH), JJ, DD)
       RI_LC(N_TH) = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
       LG%S_VA(N_TH) = MIN(CONST/ZL%RC(N_TH)/RI_LC(N_TH),1.0D0)
    END IF

    CVA_LC = SQRT(1.0D0 - LG%S_VA**2)

    ! Grid boundaries
    CVA_B2(1) = SQRT(1.0D0 - MIN(CONST/ZL%RB(2)/RI_LC(1),1.0D0)**2)
    DO J = 2, N_DS
       CVA_B1(J) = SQRT(1.0D0 - MIN(CONST/ZL%RB(J)/RI_LC(J),1.0D0)**2)
       CVA_B2(J) = SQRT(1.0D0 - MIN(CONST/ZL%RB(J + 1)/RI_LC(J),1.0D0)**2)
    END DO
    
    ! We use the geometrical path between the entry and exit points of the
    ! layer because is was found to be nearly equal to the refractive path
    ! calculated with the high resolution routine
    LG%DS(1,1) = 0.0D0
    FORALL(J = 2:N_DS) &
         LG%DS(1,J) = ABS(ZL%RC(J)*CVA_LC(J) - ZL%RB(J)*CVA_B1(J))
    LG%DS(2,N_DS) = 0.0D0
    FORALL(J = 1:N_DS - 1) &
         LG%DS(2,J) = ABS(ZL%RC(J)*CVA_LC(J) - ZL%RB(J + 1)*CVA_B2(J))
    
    FORALL(I = 1:N_DS - 1) LG%DS_C(I) = (LG%DS(2,I) + LG%DS(1,I + 1))
    CALL ST_TRACK (R_NAME, 1, 'O')
    
    RETURN
  END SUBROUTINE Los_Geom_Refr_LowRes_B

!***************************************************************************
  SUBROUTINE Los_Geom_Refr_HiRes (ZL, N_DS, N_TH, LG)
    
    USE REFRACT_VARS, ONLY: REFR_IND, Z_REFR, RHG_STEP, NLR_FINE
    
    INTEGER     N_DS, N_TH, IDX(N_DS)
    TYPE(VERT_GRIDS):: ZL
    TYPE(LOS_GEOMETRY):: LG
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Los_Geom_Refr_HiRes'
    INTEGER       I_S, I_E, JJ, J, L, K, MAX_LAY, NL, I
    REAL (KND) Z_LB(N_DS + 1), ZR_LC(N_DS), ZR_LB(N_DS + 1), ZR_R(NLR_FINE), &
         D_ANG(2,N_DS)
    REAL (KND) DD, CONST, ZZ, SIN_1, SIN_2, RI_R1, DS
    REAL (KND), ALLOCATABLE :: ZR_RLC(:), ZR_RLB(:), RI_R(:)
    !==================================================================
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    IF(ALLOCATED(LG%CVA_R)) DEALLOCATE(LG%CVA_R, LG%SVA_R1, LG%SVA_R2)
    ALLOCATE(LG%SVA_R1(N_DS), LG%SVA_R2(N_DS), LG%CVA_R(N_DS))
    
    ZR_R = Z_Refr + EARTH_RADIUS
    MAX_LAY = MAXVAL(ABS(ZL%C(:) - ZL%B(2:)))/RHG_STEP + 1
    ALLOCATE(ZR_RLC(MAX_LAY), ZR_RLB(MAX_LAY + 1), RI_R(MAX_LAY))
    
    ! Snell's law accounting for the curvature of the medium:
    ! r*n*sin(Theta) = const
    CALL GET_IDX_BELOW (ZL%C(1), JJ, DD)
    RI_R1 =(REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD))
    CONST = LG%S_VA(1)*RI_R1*ZL%RC(1)
    DO J = 2, N_DS
       CALL GET_IDX_BELOW (ZL%C(J), JJ, DD)
       RI_R1 =(REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD))
       LG%S_VA(J) = MIN(CONST/ZL%RC(J)/RI_R1,1.0D0)
    END DO
    
    LG%DS = 0.0D0 ;  D_ANG = 0.0D0
    DO J = 1, N_DS
       DO L = 1, 2
          IF(L == 1) THEN
             I_S = MINLOC(ABS(ZR_R - ZL%RB(J)),1)
             I_E = MINLOC(ABS(ZR_R - ZL%RC(J)),1)
             NL = ABS(I_E - I_S) + 1
             ZR_RLC(1) = ZL%RB(J)
             ZR_RLC(NL) = ZL%RC(J)
          ELSE
             I_S = MINLOC(ABS(ZR_R - ZL%RC(J)),1)
             I_E = MINLOC(ABS(ZR_R - ZL%RB(J + 1)),1)             
             NL = ABS(I_E - I_S) + 1
             ZR_RLC(1) = ZL%RC(J)
             ZR_RLC(NL) = ZL%RB(J + 1)
          END IF
          
          ZZ = ZR_RLC(1) - EARTH_RADIUS
          CALL GET_IDX_BELOW (ZZ, JJ, DD)
          RI_R(1) = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
          ZZ = ZR_RLC(NL) - EARTH_RADIUS
          CALL GET_IDX_BELOW (ZZ, JJ, DD)
          RI_R(NL) = REFR_IND(JJ - 1)*DD + REFR_IND(JJ)*(1.0D0 - DD)
          
          IF(I_E  >  I_S) THEN
             DO K = 1, NL - 2
                ZR_RLC(K + 1) = ZR_R(K + I_S)
                RI_R(K + 1) = REFR_IND(K + I_S)
             END DO
          ELSE
             DO K = 1, NL - 2
                ZR_RLC(K + 1) = ZR_R(I_S - K)
                RI_R(K + 1) = REFR_IND(I_S - K)
             END DO
          END IF
          
          ZR_RLB(1) = ZR_RLC(1) ;  ZR_RLB(NL + 1) = ZR_RLC(NL) 
          DO K = 2, NL
             ZR_RLB(K) = (ZR_RLC(K - 1) + ZR_RLC(K))/2.0D0
          END DO
          
          ! Each layer of the refractive grid is considered to be homegenious 
          ! with a refractive index defined in its center
          DO K = 1, NL
             SIN_1 = MIN(CONST/ZR_RLB(K)/RI_R(K),1.0D0)
             SIN_2 = MIN(CONST/ZR_RLB(K + 1)/RI_R(K),1.0D0)
             
             DS = ABS(ZR_RLB(K)*SQRT(1.0D0 - SIN_1**2) - &
                  ZR_RLB(K + 1)*SQRT(1.0D0 - SIN_2**2))
             LG%DS(L,J) = LG%DS(L,J) + DS
             D_ANG(L,J) = D_ANG(L,J) + ASIN(DS*SIN_1/ZR_RLB(K + 1))
          END DO
          
       END DO
    END DO
    
    ! Geometrical path between layer centers and effective angles
    FORALL(I = 1:N_DS - 1)
       LG%DS_C(I) = SQRT(ZL%RC(I)**2 + ZL%RC(I + 1)**2 - 2.0D0*&
            ZL%RC(I)*ZL%RC(I + 1)*COS(D_ANG(2,I) + D_ANG(1,I + 1)))
       LG%SVA_R1(I) = SIN(D_ANG(2,I) + D_ANG(1,I + 1))*ZL%RC(I + 1)/LG%DS_C(I)
       LG%SVA_R2(I) = SIN(PIE - (D_ANG(2,I) + D_ANG(1,I + 1)) - &
            ASIN(LG%SVA_R1(I)))
       LG%CVA_R(I) = SQRT(1.0D0 - LG%SVA_R1(I)**2)
    END FORALL
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END SUBROUTINE Los_Geom_Refr_HiRes
  
!***************************************************************************
  SUBROUTINE Los_Geom_Sol (REFR_LINE, Z_LC, N_DS, TH, N_TH, REFR_FLAG, LG, *)
   
    USE LOG_PARS, ONLY: EU
    USE ST_VARS,  ONLY: DO_REFRAC, Pie

    INTEGER  N_DS, N_TH
    LOGICAL  REFR_LINE, REFR_FLAG(N_DS)
    REAL (KND)  Z_LC(N_DS), TH(N_DS)
    TYPE(LOS_GEOMETRY), TARGET:: LG
    INTENT (IN) :: REFR_LINE, Z_LC, N_DS, N_TH
    INTENT (OUT) :: TH, REFR_FLAG
    INTENT (INOUT) :: LG
   
   ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Los_Geom_Sol'
    INTEGER  I
    REAL (KND)  R_Z(N_DS), XM_R(N_DS - 1), SIN_R(N_DS - 1), SIN_R1(N_DS - 1)
    REAL (KND)  COS_PHI, SIN_PHI, ETA_0, ZETA_0, KSI_0
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')

    R_Z(:) = EARTH_RADIUS + Z_LC(:)
    LG%S_SZA(1) = SQRT(1.0D0 - LG%C_SZA(1)**2)

    Angs_along_LOS: Block
      IF(N_DS == 1) EXIT Angs_along_LOS
      
      IF(ABS(LG%C_VA(1)) == 1.0D0) THEN
         LG%C_SZA(2:) =  LG%C_SZA(1)
         LG%S_SZA(2:) =  LG%S_SZA(1)
         LG%SAA(2:)   =   LG%SAA(1)
         EXIT Angs_along_LOS
      END IF

      IF(REFR_LINE .AND. Refr_HiRes) THEN
         XM_R(:) = LG%CVA_R(1:N_DS - 1)
         SIN_R(:) = LG%SVA_R1(1:N_DS - 1)
         SIN_R1(:) = LG%SVA_R2(1:N_DS - 1)
      ELSE
         XM_R(:) = LG%C_VA(1:N_DS - 1)
         SIN_R(:) = LG%S_VA(1:N_DS - 1)
         SIN_R1(:) = LG%S_VA(2:N_DS)
      END IF

      DO I = 1, N_DS - 1
         COS_PHI = COS(LG%SAA(I)) ;  SIN_PHI = SIN(LG%SAA(I))
         ZETA_0 = XM_R(I)*LG%C_SZA(I) - SIN_R(I)*LG%S_SZA(I)*COS_PHI
         KSI_0 = XM_R(I)*LG%S_SZA(I) + SIN_R(I)*LG%C_SZA(I)*COS_PHI
         
         !LG%C_SZA(I + 1) = (R_Z(I)*LG%C_SZA(I) - LG%DS_C(I)*ZETA_0)/R_Z(I + 1)
         LG%C_SZA(I + 1) = (R_Z(I)*LG%C_SZA(I) - LG%DS_C(I)*ZETA_0)/&
              SQRT((R_Z(I)*LG%C_SZA(I) - LG%DS_C(I)*ZETA_0)**2 + &
              (LG%S_SZA(I)*R_Z(I) - LG%DS_C(I)*KSI_0)**2 + &
              (LG%DS_C(I)*SIN_R(I)*SIN_PHI)**2)
         ! LG%C_SZA(I + 1) = LG%C_SZA(I + 1) - SIGN(1.0D-14,LG%C_SZA(I + 1))
         LG%S_SZA(I + 1) = SQRT(1.0D0 - LG%C_SZA(I + 1)**2)
         
         ETA_0 = ((R_Z(I) - LG%DS_C(I)*XM_R(I))*LG%S_SZA(I)*COS_PHI - &
              LG%DS_C(I)*SIN_R(I)*LG%C_SZA(I))/R_Z(I + 1)
         !       ETA_0 = ETA_0 + 1.0D-39 ! numerical stabilization
         
         COS_PHI = ETA_0/(SQRT(ETA_0**2 + (LG%S_SZA(I)*SIN_PHI)**2))
         !       COS_PHI = COS_PHI - SIGN(1.0D-13, COS_PHI) 
         
         !       SIN_PHI = R_Z(I)*LG%S_SZA(I)*SIN_R(I)*SIN_PHI/R_Z(I + 1)/&
         !            (LG%S_SZA(I + 1)*SIN_R1(I) + 1.0D-78)
         SIN_PHI = R_Z(I)*LG%S_SZA(I)*SIN_R(I)*SIN_PHI/R_Z(I + 1)/&
              (LG%S_SZA(I + 1)*SIN_R1(I))
         
         LG%SAA(I + 1) = SIGN(DACOS(COS_PHI),SIN_PHI)
      END DO
    END Block Angs_along_LOS
       
    ! Refraction
    ! ***********
    TH(:) = R_Z(:)*SQRT(1.0D0 - LG%C_SZA(:)**2) - EARTH_RADIUS
    
    IF(DO_REFRAC) THEN
       LG%CG_SZA = LG%C_SZA
       CALL REFR_INTERFACE(N_TH, Z_LC(1:N_TH), LG%C_SZA(1:N_TH), TH(1:N_TH), &
            REFR_FLAG(1:N_TH), *880)
       IF(N_TH + 1<= N_DS) THEN
          CALL REFR_INTERFACE (N_DS - N_TH, Z_LC(N_TH + 1:N_DS), &
               LG%C_SZA(N_TH + 1:N_DS), TH(N_TH + 1:N_DS), &
               REFR_FLAG(N_TH + 1:N_DS), *880)
       END IF
       LG%S_SZA(:) = SQRT(1.0D0 - LG%C_SZA(:)**2)
    ELSE
       LG%CG_SZA => LG%C_SZA
       REFR_FLAG = .FALSE.
    END IF

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading in the calculation of SZA')
    RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  
  END SUBROUTINE Los_Geom_Sol

!***************************************************************************
  Subroutine Los_Geom_Sol_Init (LVG, LG, C_SZA, SAA, LSG, *)
    ! Called by "lg_init.f90", "integrate_los.f90"
    
    Logical Const_ang
    Real (Knd) :: C_SZA, SAA
    TYPE (LOS_VIEW_GEOM), Target :: LVG
    Type (LOS_SOLAR_GEOM), Target :: LSG
    ! Local variables
    Character (LEN = *), Parameter :: R_Name = 'Integrate_Los'
    Integer NLS
    Type(LOS_GEOMETRY), TARGET:: LG
    
    !==================================================================
    NLS = LVG%NLS
    If(.Not.Allocated(LSG%C_SZA)) Allocate(LSG%C_SZA(NLS), LSG%SAA(NLS), &
         LSG%TH(NLS), LSG%REFR_DSB(NLS))
    
    LG%C_SZA(1) = C_SZA ;  LG%SAA(1) = SAA
    Select Case (LSA_MODE)
    Case ('CDI_PS', 'CDI_PP', 'DOM_NS')
       LG%C_SZA(:) = LG%C_SZA(1)
       If(LSA_MODE == 'CDI_PP') Then
          LSG%TH = -9999.
       Else
          LSG%TH = (LVG%ZLS + EARTH_RADIUS)*&
               SQRT(1.0D0 - LG%C_SZA**2) - EARTH_RADIUS
       End If
       IF(DO_REFRAC) THEN
          LG%CG_SZA = LG%C_SZA
          CALL REFR_INTERFACE(NLS, LVG%ZLS, LG%C_SZA, LSG%TH,&
               LSG%REFR_DSB, *880)
       ELSE
          LG%CG_SZA => LG%C_SZA
          LSG%REFR_DSB = .FALSE.
       END IF
       LG%S_SZA(:) = SQRT(1.0D0 - LG%C_SZA(:)**2)
       LG%SAA(:) = LG%SAA(1)
    CASE DEFAULT
       CALL Los_Geom_Sol (LVG%REFR_FLG, LVG%ZLS, LVG%NLS,  &
            LSG%TH, LVG%NTH, LSG%REFR_DSB, LG, *880)
       IF(LG%Const_ang) THEN
          LG%C_SZA(2:) = LG%C_SZA(1)
          LG%S_SZA(2:) = LG%S_SZA(1)
          LG%CG_SZA(2:) = LG%CG_SZA(1)
          LG%SAA(2:) = LG%SAA(1)
       END IF
    END SELECT

    ! SZA and SAA along LOS
    LSG%C_SZA(:) = LG%C_SZA(:)
    WHERE( LG%SAA(:) < 0.0D0 )
       LSG%SAA(:) = LG%SAA(:) + 2.0D0*Pie
    ELSEWHERE
       LSG%SAA(:) = LG%SAA(:)
    END WHERE
    ! Geometrical SZA along LOS
    IF(DO_REFRAC) THEN
       ALLOCATE(LSG%CG_SZA(NLS))
       LSG%CG_SZA = LG%CG_SZA
    ELSE
       LSG%CG_SZA => LSG%C_SZA
    END IF
    
    Return
880 WRITE(EU, *) ' ---> Called by '//R_Name
    Return 1
  End Subroutine Los_Geom_Sol_Init
!***************************************************************************
  Subroutine Los_Geom_Init (LG, LVG, Alt, AltFlag, IDA, CVA, SVA, Const_Ang, &
       F_M)
    ! Called by "lg_init.f90", "integrate_los.f90"
    Use Spher_Pars,Only: I_FINE
    Use ST_Vars,   Only: LSA_MODE
    Use Tools,     Only: Height_Grid_V3
    Type (LOS_GEOMETRY) :: LG
    Type (LOS_VIEW_GEOM), Target :: LVG
    Integer IDA
    Character (Len=:), Allocatable :: F_M
    Logical AltFlag, Const_Ang
    Real (Knd) :: CVA, SVA, Alt
    Optional :: SVA, Const_ang, F_M
    ! Local variables
    Character (LEN = *), Parameter :: R_Name = 'Los_Geom_Init'
    Character (Len=:), Allocatable :: FINE_MODE
    Integer NLS, I
    !==================================================================
    If(Present(F_M)) Then
       FINE_MODE = F_M
    Else
       FINE_MODE = 'std'
    End If
    
    CALL HEIGHT_GRID_V3 (I_FINE, CVA, LVG%H_0, Alt, AltFlag, IDA, LVG%NTH, &
         NLS, LVG%ZLS, LVG%ISEL, LVG%IOB, LVG%TH_FLG, LVG%FP_FLG, FINE_MODE)
    LVG%NLS = NLS;
    IF( CVA > 0.d0) THEN           
       ! Save tangent height or ground altitude for the reflected radiation
       LVG%Th_V => LVG%ZLS(LVG%NTH)
    ELSE
       ! Save altitude of the observer for the transmitted radiation
       LVG%Th_V => LVG%ZLS(1) 
    ENDIF
     
    LVG%NLS = NLS
    Allocate(LG%DS(2,NLS), LG%DS_C(NLS - 1), LG%S_VA(NLS), LG%C_VA(NLS), &
         LG%C_SZA(NLS), LG%S_SZA(NLS), LG%SAA(NLS))
    If(DO_REFRAC) Allocate(LG%CG_SZA(NLS))
       
    LG%C_VA(1) = ABS(CVA)
    If(Present(Const_Ang)) Then
       LG%Const_ang = Const_ang
    Else
       LG%Const_ang =.False.
    End If
    If (Present(SVA)) Then
       LG%S_VA(1) = SVA 
    Else
       LG%S_VA(1) = Sqrt(1.0D0 - CVA**2)
    End If
    Select Case (LSA_MODE)
    Case('DOM_NS') 
       LG%C_VA(:) = CVA
    Case('CDI_PS', 'CDI_PP') 
       LG%C_VA(:) = CVA
       LG%DS(1,1) = 0.0D0 ;  LG%DS(2,NLS) = 0.0D0
       ForAll(I = 2:NLS) LG%DS(1,I) =  &
            Abs((LVG%ZLS(I) -  LVG%ZLS(I - 1))/CVA)/2.0D0
       ForAll(I = 1:NLS - 1) LG%DS(2,I) = LG%DS(1,I + 1)
    Case Default 
       Call Los_Geom (LVG%REFR_FLG, LVG%ZLS, NLS, LVG%ISEL, LVG%NTH,&
            LVG%TH_FLG, LVG%FP_FLG, LG)
       ForAll(I = 1:LVG%NTH) LG%C_VA(I) = SIGN(LG%C_VA(I),CVA)
       ForAll(I = LVG%NTH + 1:NLS) LG%C_VA(I) = - SIGN(LG%C_VA(I),CVA)
    End Select

    Allocate(LVG%DS(2,NLS), LVG%C_VA(NLS))

    LVG%DS = LG%DS
    If(LG%Const_ang) Then
       LVG%C_VA(:) = CVA
    Else
       LVG%C_VA = LG%C_VA
    End If

    Return
  End Subroutine Los_Geom_Init
!***************************************************************************
  Subroutine Los_Geom_Clean (LG, LVG, LSG)
    ! Called by "lg_init.f90", "integrate_los.f90"
    Use ST_Vars, Only: DO_Refrac
    Use Types,   Only: LOS_Solar_Geom, LOS_View_Geom
    Type (LOS_View_Geom), Target :: LVG
    Type (LOS_Solar_Geom), Target :: LSG
    Type (LOS_Geometry) :: LG
    Optional :: LVG, LSG
    !==================================================================

    If(DO_Refrac) Then
       If(Associated(LG%CG_SZA)) DEALLOCATE(LG%CG_SZA)
    Else
       If(Associated(LG%CG_SZA)) NULLIFY(LG%CG_SZA)
    End If
    
    DeAllocate(LG%DS, LG%S_VA, LG%C_VA, LG%C_SZA, LG%S_SZA, LG%SAA)
    If (Allocated(LG%DS_C)) DeAllocate(LG%DS_C)

    If(Present(LVG)) DeAllocate(LVG%DS, LVG%C_VA, LVG%ZLS, LVG%ISEL, LVG%IOB)
    If(Present(LSG)) Then
       DeAllocate(LSG%C_SZA, LSG%SAA, LSG%TH, LSG%REFR_DSB)
       If(DO_Refrac) Then
          If(Associated(LSG%CG_SZA)) DEALLOCATE(LSG%CG_SZA)
       Else
          If(Associated(LSG%CG_SZA)) NULLIFY(LSG%CG_SZA)
       End If
    End If

    Return
  End Subroutine Los_Geom_Clean
  !***************************************************************************
  SUBROUTINE DirSun (NP, C_SZA, TH, Z_N, IOB, AL_N, AL0_OUT, PCC, SOL_EXP, &
       REFR_FLAG, PreCalcAngs, DO_WF, DS_SUN, DSB)
    USE ABS_COEFF_PC, ONLY: PC_ABS_COEFF
    USE GRIDS,        ONLY: Z_BOA, NL_FINE, Z_FINE, NLAYERS, ZZZ
    USE SPHER_PARS,   ONLY: I_FINE_SOL
    USE ST_VARS,      ONLY: DO_PC
    Use Tools,        Only: HEIGHT_GRID_V3
    USE TYPES,        ONLY: KND, PCC_type, DIR_SOL_GEOM, LOS_GEOMETRY
    
    INTEGER  NP
    INTEGER  IOB(NP)
    LOGICAL  DO_WF, PreCalcAngs, REFR_FLAG(NP)
    REAL (KND)  C_SZA(NP), TH(NP), Z_N(NP), AL_N(NP), SOL_EXP(NP)
    REAL (KND)  AL0_OUT(NL_FINE)
    REAL (KND), ALLOCATABLE :: DS_SUN(:,:)
    TYPE (PCC_type) :: PCC    
    TYPE (DIR_SOL_GEOM), ALlocatable :: DSB(:)    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'DirSun'
    INTEGER  JJ, NTH, NL_DS, KK, II
    INTEGER, ALLOCATABLE :: ISEL(:), IOB_DS(:)
    LOGICAL  TH_FLG, ALT_FLAG, FP_FLG
    
    REAL (KND)  SS, EXP_DS, DD, XM
    REAL (KND), ALLOCATABLE :: Z_DS(:), AL_DS(:), XS(:)
    TYPE (LOS_GEOMETRY) :: LG
    OPTIONAL :: DS_SUN, DSB
    !==================================================================
!    CALL ST_TRACK (R_NAME, 1, 'I')
    
!!!$OMP PARALLEL DO PRIVATE(KK, JJ, NTH, NL_DS, Z_DS, ISEL, IOB_DS, TH_FLG, FP_FLG) SCHEDULE(DYNAMIC)
    DO KK = 1, NP
       IF (C_SZA(KK) < 0.0D0 .AND. TH(KK) < Z_BOA) THEN
          SOL_EXP(KK) = 0.0D0
          IF(DO_WF) DS_SUN(KK,:) = 0.0D0
          CYCLE
       END IF

       IF(.NOT.PreCalcAngs) THEN
          IF(ZZZ(IOB(KK)).EQ.Z_N(KK)) THEN
             JJ = IOB(KK)
             ALT_FLAG = .FALSE.
          ELSE
             JJ = 0
             ALT_FLAG = .TRUE.
          END IF
          
          XM = - C_SZA(KK)
          CALL HEIGHT_GRID_V3 (I_FINE_SOL, XM, TH(KK), Z_N(KK), ALT_FLAG, &
               JJ, NTH, NL_DS, Z_DS, ISEL, IOB_DS, TH_FLG, FP_FLG)
!          CALL HEIGHT_GRID_V2 (I_FINE_SOL, XM, TH(KK), Z_N(KK), ALT_FLAG, &
!                         JJ, NTH, NL_DS, Z_DS, ISEL, IOB_DS, TH_FLG, FP_FLG)
         ALLOCATE(LG%DS(2,NL_DS), LG%DS_C(NL_DS - 1), LG%S_VA(NL_DS), &
              LG%C_VA(NL_DS))
         LG%C_VA(1) = C_SZA(KK)
         LG%S_VA(1) = DSQRT(1.0D0 - C_SZA(KK)**2)
         If (LSA_MODE == 'CDI_PP') Then
            LG%C_VA(:) = LG%C_VA(1)
            LG%DS(1,1) = 0.0D0 ;  LG%DS(2,NL_DS) = 0.0D0
            ForAll(II = 2:NL_DS) LG%DS(1,II) =  &
                 Abs((Z_DS(II) -  Z_DS(II - 1))/LG%C_VA(1))/2.0D0
            ForAll(II = 1:NL_DS - 1) LG%DS(2,II) = LG%DS(1,II + 1)
         Else
            CALL Los_Geom (REFR_FLAG(KK), Z_DS, NL_DS, ISEL, NTH, TH_FLG, FP_FLG, LG)
         End If
         IF(DO_PC) ALLOCATE(XS(NL_DS))
      ELSE
         NL_DS = DSB(KK)%NLS ;  NTH = DSB(KK)%NTH ;  TH_FLG= DSB(KK)%TH_FLG 
         ALLOCATE(LG%DS(2,NL_DS), IOB_DS(NL_DS), ISEL(NL_DS), Z_DS(NL_DS))
!         write(*,*) ' DSB(KK)%IOB '
         IOB_DS = DSB(KK)%IOB ;  ISEL = DSB(KK)%ISEL
         Z_DS = DSB(KK)%ZSOL ; LG%DS(:,:) = DSB(KK)%DS(:,:)
         IF(DO_PC) THEN
            ALLOCATE(LG%C_VA(NL_DS), XS(NL_DS))
            LG%C_VA(2:) = DSB(KK)%C_SZA(2:)
         END IF
      END IF

      ALLOCATE(AL_DS(NL_DS))
      AL_DS(1) = AL_N(KK)
      AL_DS(2:NL_DS) = AL0_OUT(ISEL(2:NL_DS))

      IF(TH_FLG) THEN
         DD = (TH(KK) - Z_FINE(ISEL(NTH)))/&
              (Z_FINE(ISEL(NTH) + 1) - Z_FINE(ISEL(NTH)))
         AL_DS(NTH) = DEXP(DLOG(AL0_OUT(ISEL(NTH) + 1))*DD + &
              DLOG(AL0_OUT(ISEL(NTH)))*(1.0D0 - DD))
      END IF
         
      IF(DO_PC) THEN
         XS(1) = C_SZA(KK)
         FORALL(II = 2:NTH - 1) XS(II) = SIGN(LG%C_VA(II),C_SZA(KK))
         CALL PC_ABS_COEFF(ISEL(2:NL_DS), XS(2:NL_DS), NL_DS - 1, PCC, &
              TH_FLG, .FALSE., AL_DS(2:NL_DS), TH(KK), NTH - 1, IOB_DS(NTH))
      END IF

      EXP_DS = 0.0D0 
      DO II = 2, NL_DS
         SS = AL_DS(II - 1)*LG%DS(2,II - 1) + AL_DS(II)*LG%DS(1,II)
         EXP_DS = EXP_DS + SS
      END DO
      SOL_EXP(KK) = DEXP(-EXP_DS)       

      IF(DO_WF) CALL DS_FORM_2 (NLAYERS, ZZZ, Z_DS, LG%DS, IOB_DS, NL_DS, &
           DS_SUN(KK,:))
!     IF(DO_WF) then
!         IF (KK >= 13) THEN
!            write(6,*)  '**', KK,  shape(DS_SUN)
!            write(6,*)  '***', DS_SUN(1,:)
!            pause
!         end if
!      end if
         
      DEALLOCATE(LG%DS, IOB_DS, ISEL, Z_DS, AL_DS)
      IF(.NOT.PreCalcAngs) THEN
         DEALLOCATE(LG%DS_C, LG%S_VA, LG%C_VA)
      ELSE
         IF(DO_PC) DEALLOCATE (LG%C_VA, XS)
      END IF
      
   END DO
      
!   CALL ST_TRACK (R_NAME, 1, 'O')
   RETURN
 END SUBROUTINE DirSun
!***************************************************************************
 END MODULE LOS_ANGLES
!***************************************************************************
