! **********************************************************************
!
!       SCIATRAN - Version 4.0 - Last modification: 22 Aug 2018
!
!       Copyright (C) - Institute of Remote Sensing,
!                        University of Bremen, Germany
!
! **********************************************************************
SUBROUTINE MAKE_AER_PAR(Tau, M_AC,  Cfr, Repa, Impa, MOD_R, SIG, Nam_f )

  USE TYPES, ONLY: KND
! ------------------------------------------------------------------------
  ! Input parameters
  INTEGER, INTENT(IN) :: M_AC
  CHARACTER (LEN=30)  Nam_f
  REAL (KND)  Tau
  ! Output parameters
  REAL (KND) Cfr, Repa(M_AC), Impa(M_AC), MOD_R(1,M_AC), SIG(1,M_AC)
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'MAKE_AER_PAR'
  INTEGER  I, It
!         N_AT is the number of aerosol types
!         N_Mo is the number of modes within aerosol type
  INTEGER, PARAMETER :: N_AT=4, N_Mo = 2
  REAl(KND)  Vol(N_Mo), N01, N02, Const
  CHARACTER (LEN=15)   TYPE_NAMES(N_AT)
  DATA TYPE_NAMES &
       /'Dust_influenced', &
        'Pollution-1    ', &
        'Pollution-2    ', &
        'Pure_Maritime  '/
  ! Approximation of modal radius is given as r_mod = a*Tau^b + c
  REAL(KND) Ra(N_Mo,N_AT), Rb(N_Mo,N_AT), Rc(N_Mo,N_AT)
!          --- Type 1 -----  --- Type 2 -----  --- Type 3 ----  --- Type 4 -----
  DATA Ra /0.048d0, 0.072d0, 0.042d0, 1.122d0, 0.053d0, 0.31d0, 0.115d0, 1.376d0/ !a
  DATA Rb /  1.0d0, 1.0d0,     1.0d0, 1.0d0,     1.0d0, 1.0d0,  -0.05d0, -0.06d0/ !b
  DATA Rc /0.117d0, 1.622d0, 0.137d0, 1.528d0, 0.144d0, 2.195d0,   0.d0, 0.d0   / !c
!********************************************************************************
  ! Approximation of variance is given as for r_mod
  REAL(KND) Va(N_Mo,N_AT), Vb(N_Mo,N_AT), Vc(N_Mo,N_AT)
!           --- Type 1 -----  --- Type 2 -----  --- Type 3 ----  --- Type 4 -----
  DATA Va /0.556d0, 0.531d0, 0.435d0, 0.658d0, 0.499d0, 0.60d0,  0.420d0, 0.732d0/ !a
  DATA Vb /-0.03d0, -0.08d0, 0.076d0, -0.05d0, 0.092d0, -0.06d0, -0.02d0, 0.016d0/ !b
  DATA Vc /   0.d0, 0.d0,       0.d0, 0.d0,       0.d0, 0.d0,       0.d0, 0.d0   / !c
!*********************************************************************************
  ! Approximation of total volume per area is given as for r_mod
  REAL(KND) Av(N_Mo,N_AT), Bv(N_Mo,N_AT), Cv(N_Mo,N_AT)
!       --- Type 1 -----  --- Type 2 -----   --- Type 3 ----  --- Type 4 -----
  DATA Av /0.035d0, 0.617d0, 0.144d0, 0.059d0, 0.141d0, 0.115d0, 0.078d0, 0.588d0/ ! a
  DATA Bv /  1.0d0, 1.15d0,    1.0d0, -0.18d0,   1.0d0, 1.0d0,     1.0d0, 1.0d0  / ! b
  DATA Cv / 0.01d0,  0.0d0,  0.009d0, 0.0d0,   0.015d0, 0.018d0,   0.0d0, 0.002d0/ ! c
!*********************************************************************************
  !  ************** Approximation of refractive index ***************************
  !      Real part:
  REAL(KND) Are(N_Mo,N_AT), Bre(N_Mo,N_AT), Cre(N_Mo,N_AT)
!           --- Type 1 ---      --- Type 2 -----   --- Type 3 ----  --- Type 4 -----
  DATA Are /  1.48d0, 1.48d0,   0.053d0, 0.053d0, 0.037d0, 0.037d0, 1.415d0, 1.363d0/ ! a
  DATA Bre /-0.021d0, -0.021d0,   1.0d0, 1.0d0,     1.0d0, 1.0d0,     0.0d0, 0.0d0  / ! b
  DATA Cre /   0.0d0,  0.0d0,    1.48d0, 1.48d0,  1.419d0, 1.419d0,   0.0d0, 0.0d0  / ! c
  !      Imaginary part:
  REAL(KND) Aim(N_Mo,N_AT), Bim(N_Mo,N_AT), Cim(N_Mo,N_AT)
!              --- Type 1 ---       --- Type 2 -----   --- Type 3 ----  --- Type 4 -----
  DATA Aim / 0.0025d0, 0.0025d0,  -0.002d0, -0.002d0, 0.002d0, 0.002d0, 0.002d0, 3.0d-9/ ! a
  DATA Bim /  0.132d0, 0.132d0,      1.0d0,  1.0d0,     1.0d0, 1.0d0,     0.0d0, 0.0d0 / ! b
  DATA Cim /    0.0d0, 0.0d0,      0.006d0,  0.006d0, 0.005d0, 0.005d0,   0.0d0, 0.0d0 / ! c
! --------------------------------------------------------------------------------

  CALL ST_TRACK (R_NAME, 1, 'I')
  ! Select aerosol type
  It = 0
  DO I = 1, N_AT
     IF( TRIM(TYPE_NAMES(I)) == TRIM(Nam_f) ) THEN
        It = I ; EXIT
     ENDIF
  ENDDO
  IF( It==0 ) THEN
     STOP '   Aerosol type is not identified! '
  ENDIF

  ! Calculate total volume per area:
  DO I = 1,N_Mo
     Vol(I) = Av(I,It)*Tau**Bv(I,It) + Cv(I,It)
  ENDDO

  ! Fine mode fraction
  Cfr = Vol(1)/SUM( Vol(:) )

  WRITE(*,*) '   Selected aerosol type:'
  WRITE(*,*) TRIM( TYPE_NAMES(It) )
  WRITE(*,*) '   Volume fraction:', Cfr
  WRITE(*,*) '   Volume per area V0'
  WRITE(*,'(4x,F6.3)') Vol(1)
  WRITE(*,'(4x,F6.3)') Vol(2)

  ! Calculate modal radii and variances of the fine and coarse mode for 
  !  selected type
  SELECT CASE(It)
  CASE(1)
     MOD_R(1,1) = Ra(1,It)* Tau    **Rb(1,It) + Rc(1,It)
     MOD_R(1,2) = Ra(2,It)*(Tau*Cfr)**Rb(2,It) + Rc(2,It)
     SIG  (1,1) = Va(1,It)*(Tau*Cfr)**Vb(1,It) + Vc(1,It)     
     SIG  (1,2) = Va(2,It)* Tau    **Vb(2,It) + Vc(2,It)     
  CASE(2,3)
     DO I = 1, 2
        MOD_R(1,I) = Ra(I,It)*Tau**Rb(I,It) + Rc(I,It)
        SIG  (1,I) = Va(I,It)*Tau**Vb(I,It) + Vc(I,It)
     ENDDO
  CASE(4)
     DO I = 1, 2
        MOD_R(1,I) = Ra(I,It)*(Tau*Cfr)**Rb(I,It) + Rc(I,It)
        SIG  (1,I) = Va(I,It)*(Tau*Cfr)**Vb(I,It) + Vc(I,It)
     ENDDO
  END SELECT

  ! Calculation of refractive indeces
  DO I = 1, 2
     Repa(I) = Are(I,It)*Tau**Bre(I,It) + Cre(I,It)
     Impa(I) = Aim(I,It)*Tau**Bim(I,It) + Cim(I,It)
  ENDDO

  !  Particles number density of fine and coarse mode
  Const = 3.d0/4.d0/3.1415926536d0
  N01 = Vol(1)*Const/MOD_R(1,1)**3*DEXP( 9.d0/2.d0*SIG(1,1)**2 )
  N02 = Vol(2)*Const/MOD_R(1,2)**3*DEXP( 9.d0/2.d0*SIG(1,2)**2 )

  Cfr = N01/(N01 + N02)
  WRITE(*,*) '   Density fraction:', Cfr

  CALL ST_TRACK (R_NAME, 1, 'O')
  
  RETURN
 END SUBROUTINE MAKE_AER_PAR
