!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 28 Jun 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE MERRA_XB_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
  USE AER_MIE,   ONLY: DENS_PRF
  USE AEROSOLS,  ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, LAY_A, WEL_A, NSCA_A, &
       GE_A, APH_VS, AER_WAVS, NAW => N_AER_WAVS, Greek_Aer, AER_COEFF, Scal_aer, &
       SCI => GT_SCAT_IDX, NG_aer, ASY => GT_ASYM_IDX, Rtau_aer, L_scal, ZS1, &
       EXI => GT_EXTC_IDX, Rwl_aer, ZS2, Plus_mi, NWLS, NWLE, AER_PARAM, &
       AER_SCALING, Nga_db, LA_N, LA_S, LA_E, L_TOP, IDX_AER, LA_Ndb, AOT_out_VAL, &
       AER_FACT, V_PAP, IDX_AER_db, RT_AG
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE DELTA_M,   ONLY: FDM_AER, AER_DM_FLG, D_MF_AER, AER_TRUNC_IDX
  USE GRIDS,     ONLY: NLAYERS, ZZZ, DZ, NLAYERS_INP, ZZZ_INP
  USE GT_POLYN,  ONLY: NLEGEN
  USE INFO,      ONLY: AER_INFO
  USE I_DIMS,    ONLY: Nm_vs
  USE LOG_PARS,  ONLY: Work_Dir, LOG_LEVEL, LUNIT, EU, DIR_OUT, C_UNIT, N_SSO, N_ESO
  USE RTM_PARS,  ONLY: DATA_ITER
  USE ST_VARS,   ONLY: RAD_TO_DEG, Vec_Sca, RTM_TYPE, GROUND_HEIGHT
  USE ST_INPUT,  ONLY: APH_TYPE, ADB => PATH_AER_DB
  USE TOOLS,     ONLY: READ_INPUT, CHK_STAT, SET_PTR, CUT_STR, READ_FILE, SET_PATH, &
                       SPLIT_STRING, NotFound
  USE TYPES,     ONLY: KND, DYN_CHAR
! -----------------------------------------------------------------------------------  
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  ! Local parameters
  LOGICAL            ERROR
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'MERRA_XB_SETUP'
  !  (a) Input variables
  !      AER_DM_FLG        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZZZ             : forward model altitude grid
  !      NLAYERS           : number of layers in altitude grid
  !      NLEGEN            : number of Legendre moments
  !      LUNIT             : LOG file unit number
  !  (b) Output variables
  !      AER_COEFF         : scattering / extinction coefficients and 
  !                          asymmetry parameters 
  !      AER_PHASMOMS      : Legendre moments
  !      NAW               : number of wavelengths of chosen aerosol 
  !                          parameterization 
  !      AER_WAVS          : wavelengths of aerosol data base (depends on
  ! parameterization chosen)
  LOGICAL  Replace_ND, Lscal_g
  INTEGER, PARAMETER :: N_AER_COMP = 15, N_xb = 5
  INTEGER  N_SKIP, N_SKIP_HEADER, N_lay, Nac1, Nac2, NDB_MOM, Du_MOM, N_out
  INTEGER  N_rep, Nz_rep, N_ab, N_ref, Iph_const
  REAL (KND), PARAMETER :: WL_REF = 550.0d0
  REAL (KND)  DUM, DD1, DD2, XS, XE, S1, Nm_lay, Scoe, T_550, Me_tau, &
       TMP, GG, Z_max, Zph_const, Scal_G
  REAL (KND), ALLOCATABLE :: PHAS_LOC(:), PL_RES(:,:), USER_MOM(:,:,:,:), &
       R_COF(:), X_ang(:),  OUT_MOM(:,:), ASYM_PAR(:,:), Nd_rep(:), &
       D1(:), TAU(:), DATA_R(:,:), Rh_DB(:), Air_den(:), Rel_hum(:), &
       Eef(:,:,:), Sef(:,:,:), CSA(:,:), Mas(:,:), MMR(:,:), Num_den(:), &
       Ecs(:,:,:), Scs(:,:,:), Z_lay(:), MOM_Rh(:,:,:,:,:), Nd_comp(:,:), &
       Ecs_xb(:,:,:), Scs_xb(:,:,:), SScs(:,:), Me_ext(:), SEcs_xb(:,:),&
       SScs_xb(:,:), MOM_x(:,:,:), C_WEI(:), Acof(:), Bcof(:), Fr_xb(:) &
       , As_WL(:)
  TARGET ::  MOM_Rh
  CHARACTER (LEN=100), ALLOCATABLE :: Comp_Name(:)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, WL_NAME, RH_NAME, D_NAME, Z_NAME, &
       DIR_name, F_name
  CHARACTER (LEN=7) MOM_DAT(6)
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:), Ind_xb(:,:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:), P_mom(:,:)
  INTEGER  I, I1, II, Ico, J, JJ, K, L, NWL_DB, N_DIM, Jc, Ixb, &
        NLEG_LOC, DUNIT, K1, Itr, N_COMP, LAN, N_rh
!  Variable for dust component
  INTEGER  NWL_du, NWL_ge, NW1, NW2, Nx, Nwl_sg
  CHARACTER (LEN=:), ALLOCATABLE ::  WLd_NAME, WLg_NAME
  REAL (KND), ALLOCATABLE :: Du_WL(:), Sp_WL(:), WL_sg(:), ARR_RH(:), Hel(:), &
       Hel_xb(:,:)
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
  ! Variable for Delta-M
  ! ====================
  REAL (KND)  F_DM, AER_ABS_COEFF
  ! Variable for Delta-fit
  ! ======================
  INTEGER     N_ANG, Nleg_df, N_KAT
  REAL (KND)  F_Dfit_loc
  !=========================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
      
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
!  IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
!     ALLOCATE( DENS_PRF(NLAYERS) )
!     DENS_PRF(:) = 1.d0
!     CALL SET_PTR( RTM_PTR, 'aer-dens' )
!     RTM_PTR%APR(:) = DENS_PRF(:)
!     RTM_PTR%CRN(:) = DENS_PRF(:)    
!  ENDIF

!  Replace_ND = .TRUE.
  Replace_ND = .FALSE.

  ERROR = .FALSE.
! ... Specific for XBAER aerosol type  *******************************
  STR_INP = 'Directory name containing file with regression coefficients'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = CUT_STR(READ_ARR(1)%S)

  STR_INP = 'Filename of A and B coefficients'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  F_name = DIR_name//CUT_STR(STR_ARR(1)%S)
!     WRITE(*,'(A)')  TRIM( F_name )

  STR_INP = 'Total optical thickness'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=101) T_550
  
! ... *********** Test with asymmetry parameter ************
  STR_INP = 'Scaling of asymmetry factor'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=101) Scal_G

  CALL READ_FILE(2, F_name, DATA_R, *880)
  N_ab = Size(DATA_R,1) 
  ALLOCATE( Acof(N_ab), Bcof(N_ab), Fr_xb(N_ab) )
  Acof(:) = DATA_R(:,1)  ;  Bcof(:) = DATA_R(:,2)
  Fr_xb(:) = Acof(:) + Bcof(:)/T_550
!  Fr_xb(:) = Acof(:)*T_550 + Bcof(:)
  Fr_xb(:) = Fr_xb(:)/SUM( Fr_xb(:))
!     WRITE(*,'(5x,A,f8.2)') 'Total AOT: ', T_550
!     WRITE(*,'(5x,A,f8.2)') 'Sum of fractions: ', SUM( Fr_xb(:))
!     DO I = 1, N_ab
!        WRITE(*,'(4x,I2,2(2x,f6.2),4x,E12.4)') I, Acof(I), Bcof(I), Fr_xb(I)
!     ENDDO
  DEALLOCATE( DATA_R )
! ... End specific for XBAER aerosol type  ***************************
!  PAUSE
!
!  *************   Start MERRA   **************************
!
  MOM_DAT(1) = 'P11.dat'  ;  MOM_DAT(2) = 'P22.dat'  ;  MOM_DAT(3) = 'P33.dat'
  MOM_DAT(4) = 'P44.dat'  ;  MOM_DAT(5) = 'P12.dat'  ;  MOM_DAT(6) = 'P34.dat'
  
  STR_INP  = 'Wavelength and humidity grid (MERRA)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  WL_NAME  = TRIM(ADB)//SET_PATH(STR_ARR(1)%S)
  WLd_NAME = TRIM(ADB)//SET_PATH(STR_ARR(2)%S)
  WLg_NAME = TRIM(ADB)//SET_PATH(STR_ARR(3)%S)
  RH_NAME  = TRIM(ADB)//SET_PATH(STR_ARR(4)%S)
  
! ... Reading humidity grid
! ------------------------------------------------------------------ 
  CALL READ_FILE(1, RH_NAME, DATA_R, *880)
  N_rh = Size(DATA_R,1)    ;  ! WRITE(*,*) '   Number of RH = ', N_rh
  ALLOCATE( Rh_DB(N_rh) )  ;   Rh_DB(:) = DATA_R(1:N_rh,1)
  DEALLOCATE( DATA_R )
! -------------------------------------------------------------------
          
! ... Reading wavelength grid (Spherical particles and Dust)
! ------------------------------------------------------------------------------
  CALL MAKE_WL_GRID(WL_Name,  Nwl_DB, Sp_WL, *880 )
  CALL MAKE_WL_GRID(WLd_Name, Nwl_du, Du_WL, *880 )
  
! ... Reading wavelength grid (General)
  XS = START_WAV*1.d-9  ;  XE = END_WAV*1.d-9  !  [nm] ==> [m]   
  IF( WL_REF < START_WAV )   XS = WL_REF*1.d-9        
  IF( WL_REF > END_WAV )     XE = WL_REF*1.d-9        
  CALL READ_FILE(1, WLg_Name, DATA_R, *880, XS, XE, NPT=1)
  NAW = Size(DATA_R,1)
  ALLOCATE( Eef(NAW,N_Aer_COMP,N_rh), Sef(NAW,N_Aer_COMP,N_rh), AER_WAVS(NAW) ) ! A
  AER_WAVS(:) = DATA_R(1:NAW,1)*1.d9  !  [m] ==> [nm]
  N_ref = MINLOC( ABS(AER_WAVS(:) - WL_REF ),1 )
!      WRITE(*,*) '   Selected aerosol wavelength grid [nm] '
!      DO I = 1, NAW
!         WRITE(*,'(2x,I3,2x,F8.2)') I, AER_WAVS(I) 
!      ENDDO
!      WRITE(*,*) '    Selected reference wavelength: '
!      WRITE(*,'(4x,f6.2)')  AER_WAVS(N_ref)
!  STOP
! ------------------------------------------------------------------------------
! ... Numbers of MERRA tracers depending on humidity
  Nac1 = 1  ;   Nac2 = 10
  
  STR_INP  = 'Aerosol components (MERRA)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  N_COMP = SIZE(STR_ARR)
!      WRITE(*,*) '   Number of components = ', N_COMP
  ALLOCATE( Comp_Name(N_Aer_COMP), C_WEI(N_COMP) )
  READ(READ_ARR(2)%S, *, ERR=101) C_WEI(:) ! Fraction of aerosol components
  I = Nac1
  DO II = 1, N_COMP
     Comp_Name(I) = TRIM(ADB)//SET_PATH(STR_ARR(II)%S)
 !    WRITE(*,'(A,I3,1x,A,F5.2)') '  Name ', II, TRIM( Comp_Name(I) ), C_WEI(II)
     I = I + 1
  ENDDO

  STR_INP = 'MERRA scenario file'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  D_NAME = SET_PATH(STR_ARR(1)%S)  ;  Z_NAME = SET_PATH(STR_ARR(2)%S)
  
! ... Reading filename containing MERRA scenario
! -----------------------------------------------------------------  
!  CALL READ_FILE(N_AER_COMP+2, D_NAME, DATA_R, *880, Skip_line = 23)
  CALL READ_FILE(N_AER_COMP+2, D_NAME, DATA_R, *880)
  Nm_lay = Size(DATA_R,1) ! Number  of layers in original input file
!     write(*,*) DATA_R(1,1:4)
  N_lay = Nm_lay
  ALLOCATE( Air_den(N_lay), Rel_hum(N_lay), MMR(N_lay,N_AER_COMP), &
       Nd_comp(N_lay,N_AER_COMP), ARR_RH(N_lay), Hel(N_lay),       &
       Num_den(N_lay), Hel_xb(N_lay,N_xb), Ind_xb(N_xb,2) )
  Air_den(:) = DATA_R(1:N_lay,1)              ! air density [kg/m^3]
  Rel_hum(:) = DATA_R(1:N_lay,2)              ! relative humidity
  MMR(:,:)   = DATA_R(1:N_lay,3:N_AER_COMP+2) ! Mass mixing ratio kg/kg
  FORALL( I=1:N_lay ) MMR(I,:) = MMR(I,:)*C_WEI(:)
  DEALLOCATE( DATA_R, D_NAME )
! =================================================================

! ... Reading filename containing MERRA altitude grid
!     Altitude of the middle of aerosol layers  
! -----------------------------------------------------------------  
  CALL READ_FILE(1, Z_NAME, DATA_R, *880)
  IF( Size(DATA_R,1) /= N_lay ) GOTO 137
  ALLOCATE( Z_lay(N_lay) )
  Z_lay(:) = DATA_R(1:N_lay,1)
!              WRITE(*,*) '   Minimal and maximal altitude Merra:'
!              WRITE(*,'(2x,E12.4,2x,F8.4)') Z_lay(N_lay), Z_lay(1)
! =================================================================

! ... Reading cross-section area per kg dry mass and particles mass
! -----------------------------------------------------------------  
  ALLOCATE( CSA(N_rh,N_Aer_COMP), Mas(N_rh,N_Aer_COMP) )
  CSA = 0.d0 ; Mas = 0.d0
  !  ... Depending on humidity MERRA tracers (BC1, BC2, OC1, OC2, SU, SS1-SS5)
  DO I = Nac1, Nac2
     D_NAME = TRIM(Comp_Name(I))//'/MERRA_area_mass.dat'
     CALL READ_FILE(2, D_NAME, DATA_R, *880)
     CSA(:,I) = DATA_R(1:N_rh,1)*DATA_R(1,2)  ! cross-section area [m^2]
     Mas(:,I) = DATA_R(1:N_rh,2)              ! particles mass [kg]
     DEALLOCATE( DATA_R, D_NAME )
  ENDDO
  !  ... Independent of humidity MERRA tracers (dust in five bins DU1-DU5)
  DO I = Nac2+1, N_COMP
     D_NAME = TRIM(Comp_Name(I))//'/MERRA_area_mass.dat'
     CALL READ_FILE(2, D_NAME, DATA_R, *880)
     CSA(:,I) = DATA_R(1,1)*DATA_R(1,2)      ! cross-section area [m^2]
     Mas(:,I) = DATA_R(1,2)                  ! particles mass [kg]
     DEALLOCATE( DATA_R, D_NAME )
  ENDDO
! =================================================================
! ... Number density vertical profile of each MERRA tracer
  Nd_comp = 0.d0
  DO I = Nac1, N_COMP   
    Nd_comp(:,I) = Air_den(:)*MMR(:,I)/Mas(1,I)  ! Number density [kg/m^3]
 ENDDO
! ...  Number density of aerosol MERRA type
  DO J = 1, N_lay
     Num_den(J) = SUM( Nd_comp(J,:) )
!     WRITE(*,'(2x, I3, *(3x,e12.4))')  J, Num_den(J)
!          J, ( Nd_comp(J,I)/Num_den(J), I = 1,15 )
  ENDDO

!     Read extinction efficiency 
! -----------------------------------------------------------------------------
  ! ... Tracers with spherical shape
  DO I = Nac1, Nac2
     ! ext. and scat. efficiency at standard humidity grid (dimension - N_rh)
     ! and wavelength grid for spherical particles (dimension - Nwl_DB)
     CALL EXT_SCAT(N_rh, Nwl_DB, Sp_WL, *880 ) ! ----> Eef and Sef
  ENDDO
  ! ... Dust
  DO I = Nac2+1, N_COMP
     ! ext. and scat. efficiency at wavelength grid for dust particles
     !   (dimension - Du_WL)
     CALL EXT_SCAT(1, Nwl_du, Du_WL, *880 )  ! ----> Eef and Sef
  ENDDO
! ===============================================================================

  ! Calculation of extinction and scattering cross-section for all
  !    aerosol tracers at standard RH grid (efficiency*cross-section area)
  DO J = 1, NAW
     DO I = Nac1, N_COMP
        Eef(J,I,:) = Eef(J,I,:)*CSA(:,I)
        Sef(J,I,:) = Sef(J,I,:)*CSA(:,I)
     ENDDO  ! over all components (I)
  ENDDO  ! over selected actual WLs (J)
  
  ALLOCATE( Ecs(NAW,N_Aer_COMP,N_lay), Scs(NAW,N_Aer_COMP,N_lay), &
       Ecs_xb(NAW,N_xb,N_lay), Scs_xb(NAW,N_xb,N_lay), &
       SEcs_xb(NAW,N_lay), SScs(NAW,N_lay), SScs_xb(NAW,N_lay) )
  Ecs = 0.d0  ;   Scs = 0.d0
! Interpolate extinction and scattering cross-section for all
!    aerosol tracers on the given RH vertical profile
  DO J = 1, NAW
     ! ... Dependent on humidity
     DO I = Nac1, Nac2
        CALL LIN_INT(N_rh, Rh_DB, Eef(J,I,:), N_lay, Rel_hum, Ecs(J,I,:), 'C')
        Ecs(J,I,:) = Ecs(J,I,:)*Nd_comp(:,I)  ! extinction coefficient [1/m]
        CALL LIN_INT(N_rh, Rh_DB, Sef(J,I,:), N_lay, Rel_hum, Scs(J,I,:), 'C')
        Scs(J,I,:) = Scs(J,I,:)*Nd_comp(:,I)  ! scattering coefficient [1/m]
     ENDDO  ! over tracers (I)
     ! ... Independent of humidity
     DO I = Nac2+1, N_COMP
        Ecs(J,I,:) = Eef(J,I,1)*Nd_comp(:,I)  ! independent of humidity
        Scs(J,I,:) = Sef(J,I,1)*Nd_comp(:,I)  ! independent of humidity
     ENDDO  ! over tracers (I)
     FORALL( K=1:N_lay )  SScs(J,K) = SUM( Scs(J,:,K) ) 
  ENDDO  ! over wavelengths (J)

  Ind_xb(1,1) = 1  ;  Ind_xb(1,2) = 2
  Ind_xb(2,1) = 3  ;  Ind_xb(2,2) = 4
  Ind_xb(3,1) = 5  ;  Ind_xb(3,2) = 5
  Ind_xb(4,1) = 6  ;  Ind_xb(4,2) = 10
  Ind_xb(5,1) = 11 ;  Ind_xb(5,2) = 15

! Calculation of extinction and scattering coefficients of XBAER components  
  DO I = 1, NAW
     DO J = 1, N_lay
        DO Ixb = 1, N_xb  
           Ecs_xb(I,Ixb,J) = SUM( Ecs(I,Ind_xb(Ixb,1):Ind_xb(Ixb,2),J) )
           Scs_xb(I,Ixb,J) = SUM( Scs(I,Ind_xb(Ixb,1):Ind_xb(Ixb,2),J) )
        ENDDO  ! over XBAER components (Ixb)
     ENDDO  ! over aerosol layers (J)
     FORALL( K=1:N_lay )  SScs_xb(I,K) = SUM( Fr_xb(:)*Scs_xb(I,:,K) )
     FORALL( K=1:N_lay )  SEcs_xb(I,K) = SUM( Fr_xb(:)*Ecs_xb(I,:,K) )
!     FORALL( K=1:N_lay )  SScs_xb(I,K) = ABS( SUM( Fr_xb(:)*Scs_xb(I,:,K) ) )
!     FORALL( K=1:N_lay )  SEcs_xb(I,K) = ABS( SUM( Fr_xb(:)*Ecs_xb(I,:,K) ) )
     IF( MINVAL(SScs_xb(I,:),1) <= 0.d0 ) GOTO 140
  ENDDO  ! over wavelength (I)

  ALLOCATE( FDM_AER(NLAYERS,NAW), IDA(NLAYERS), AER_COEFF(3,NLAYERS,NAW), &
       TAU(NAW), Me_ext(NLAYERS) ) !A
  FDM_AER = 0.d0 ; AER_COEFF = 0.d0 

! ---------------- Aerosol layering (Case RT_AG = 'merra'  ----------
  LAN = N_lay
  ALLOCATE( LAS(LAN), LAE(LAN), L_TOP(LAN) )
  ! ... As for all aerosol layering we define altitudes of the top of aerosol
  !     layers which corresponds with MERRA local altitude grid ordered
  !     from the bottom to the top of the atmosphere
!        WRITE(*,*) '    N_lay, NLAYERS ', N_lay, NLAYERS
!  L_TOP(:) = ZZZ(NLAYERS-1:1:-1)
  L_TOP(:) = ZZZ_INP(2:NLAYERS_INP)
!  WRITE(*,*) '         L_TOP from main grid:'
!  WRITE(*,'(2x,E12.4,2x,F8.4)') ZZZ(NLAYERS-1), ZZZ(1)
!  WRITE(*,*) '         L_TOP from ZZZ_INP:'
!  WRITE(*,'(2x,E12.4,2x,F8.4)') L_TOP(1), L_TOP(LAN)
  
!     Read moments of aerosol components with spherical shape (WL grid is Sp_WL)
! -----------------------------------------------------------------------------  
  NW1 = MINLOC( DABS(AER_WAVS(1) - Sp_WL(:)),1 )
  IF( (AER_WAVS(1) - Sp_WL(Nw1)) < 0.d0.AND.NW1 > 1 ) NW1 = Nw1-1 
  NW2 = MINLOC( DABS(AER_WAVS(NAW) - Sp_WL(:)),1 )
  IF( (AER_WAVS(NAW) - Sp_WL(Nw2)) > 0.d0.AND.NW2 < NAW ) NW2 = Nw2+1 
  Nwl_sg = NW2 - NW1 + 1  ;   ALLOCATE( WL_sg(Nwl_sg) )
  WL_sg(:) = Sp_WL(NW1:Nw2)
!  WRITE(*,'(A,*(1x,F6.2))') ' START_WAV, END_WAV ', START_WAV, END_WAV
!  WRITE(*,'(A,*(1x,F6.2))') '    Sub-grid of  Sp_WL: ', WL_sg(:)

  OPEN(C_UNIT, FILE=TRIM(Comp_Name(1))//'/MERRA_moments_'//MOM_DAT(1),STATUS='OLD')
  CALL SKIP_HEADER (C_UNIT)
  READ(C_UNIT,*) NDB_MOM  !;  WRITE(*,*) '   Number of moment  ', NDB_MOM
  CLOSE(C_UNIT)
  ALLOCATE( MOM_Rh(NDB_MOM,NAW,N_rh,N_AER_COMP,Nm_vs), MOM_x(NDB_MOM,Nwl_sg,N_rh) )
  MOM_Rh = 0.d0
  N_SKIP_HEADER = 7
! ... Reading moments of spherical aerosol particles  
  DO Ico = Nac1, Nac2
     DO Jc = 1, Nm_vs
        CALL READ_MOMS(N_rh, Nwl_sg, WL_sg, NDB_MOM, *880)
     ENDDO  ! over elements of scattering matrix (Jc)
  ENDDO  ! over aerosol components (Ico)
! =================================================================================
  
!     Read moments of dust (WL grid is Du_WL)
! -----------------------------------------------------------------------------  
  DEALLOCATE( WL_sg, MOM_x )
  NW1 = MINLOC( DABS(AER_WAVS(1) - Du_WL(:)),1 )
  IF( (AER_WAVS(1) - Du_WL(Nw1)) < 0.d0.AND.NW1 > 1 ) NW1 = Nw1-1 
  NW2 = MINLOC( DABS(AER_WAVS(NAW) - Du_WL(:)),1 )
  IF( (AER_WAVS(NAW) - Du_WL(Nw2)) > 0.d0.AND.NW2 < NAW ) NW2 = Nw2+1 
  Nwl_sg = NW2 - NW1 + 1  ;  ALLOCATE( WL_sg(Nwl_sg) )
  WL_sg(:) = Du_WL(NW1:Nw2)
!  WRITE(*,'(A,*(1x,F6.2))') ' START_WAV, END_WAV ', START_WAV, END_WAV
!  WRITE(*,'(A,*(1x,F6.2))') '    Subgrid of  Du_WL: ', WL_sg(:)

  OPEN(C_UNIT, FILE=TRIM(Comp_Name(Nac2+1))//'/MERRA_moments_'//MOM_DAT(1),STATUS='OLD')
  CALL SKIP_HEADER (C_UNIT)
  READ(C_UNIT,*) Du_MOM  !;  WRITE(*,*) '   Number of moment  ', Du_MOM
  CLOSE(C_UNIT)

  ALLOCATE( MOM_x(Du_MOM,Nwl_sg,1) )
  N_SKIP_HEADER = 7
  DO Ico = Nac2+1, N_COMP
     DO Jc = 1, Nm_vs
        CALL READ_MOMS(1, Nwl_sg, WL_sg, Du_MOM, *880)
     ENDDO  ! over matrix elements (Jc)
  ENDDO  ! over aerosol components (Ico)
! ----------------------------------------------------------------------------------
  
!  WRITE(*,*) '    2nd moment, 2nd Wl, 1-3 Rh and last, All component'
!  DO I = 1, N_COMP
!     write(*,'(1x,I2,*(2x,F7.4))') I, Mom_Rh(2,2,1:3,I,1), Mom_Rh(2,2,N_rh,I,1)  
!  ENDDO

  N_DIM = NDB_MOM 
  ALLOCATE( USER_MOM(NAW,N_DIM,LAN,Nm_vs) ) ! A
  USER_MOM   = 0.d0
! ... Interpolate moments on the actual relative humidity.
!     Ordered from the top of atmosphere to the bottom
  DO I = 1, NAW
     DO L = 1, NDB_MOM
        DO Jc = 1, Nm_vs
           P_mom => Mom_Rh(L,I,:,:,Jc)
           Hel_xb(:,:) = 0.d0  
           DO Ixb = 1, 4
              DO Ico = Ind_xb(Ixb,1), Ind_xb(Ixb,2)
                 CALL LIN_INT(N_rh, Rh_DB, P_mom(:,Ico),N_lay, Rel_hum, ARR_RH,'C')
                 Hel_xb(:,Ixb) = Hel_xb(:,Ixb) + ARR_RH(:)*Scs(I,Ico,:)
              ENDDO  ! over depending on humidity MERRA tracers (Ico)
              Where(Scs_xb(I,Ixb,:)>0.d0) Hel_xb(:,Ixb)=Hel_xb(:,Ixb)/Scs_xb(I,Ixb,:)
           ENDDO   ! over four XBAER component (Ixb)
           ! ... Fifth XBAER AT component
           Ixb = 5
           DO Ico = 11, N_COMP ! for dust independent of humidity          
              Hel_xb(:,Ixb) = Hel_xb(:,Ixb) + P_mom(1,Ico)*Scs(I,Ico,:)
           ENDDO
           Where(Scs_xb(I,Ixb,:)>0.d0) Hel_xb(:,Ixb)=Hel_xb(:,Ixb)/Scs_xb(I,Ixb,:)
           Hel(:) = 0.d0
           DO Ixb = 1, N_xb
              Hel(:) = Hel(:) + Fr_xb(Ixb)*Scs_xb(I,Ixb,:)*Hel_xb(:,Ixb)
           ENDDO
           Where(SScs_xb(I,:)>0.d0) USER_MOM(I,L,:,Jc)=Hel(:)/SScs_xb(I,:)
        ENDDO  ! over scattering matrix elements (Jc)
     ENDDO  ! over moments (L)
  ENDDO  ! over wavelength (I)

  IF( Nm_vs > 1 ) THEN
     USER_MOM(:,:,:,5) = -USER_MOM(:,:,:,5)
     USER_MOM(:,:,:,6) = -USER_MOM(:,:,:,6)
  ENDIF

!  WRITE(*,*) '    ---------------->  Enter Z_max, Zph_const, Scaling asym. '
!  READ(*,*) Z_max, Zph_const, Scal_G
  Z_max = 200.d0
  Zph_const = 0.0d0
!  WRITE(*,*) '    ---------------->  Enter Zph_const, Scaling asym. '
!  READ(*,*) Zph_const, Scal_G
!  WRITE(*,*) '    ---------------->  Enter Scaling asymmetry factor '
!  READ(*,*) Scal_G
  Iph_const = 0
  IF( Zph_const > 0.d0 ) Iph_const = MINLOC( ABS( ZZZ(:) - Zph_const),1 )

  Lscal_g = .FALSE.
!  WRITE(*,*) '   Scaling of asymmetry ', Scal_G
  IF( Scal_G > 0.d0 ) THEN
     Lscal_g = .TRUE.
     ALLOCATE( As_wl(NAW) )
     DO K = 1, LAN
        As_WL(:) = USER_MOM(:,2,K,1)/3.d0
        As_WL(:) = As_WL(:)*Scal_G
!!       !  test setting ---------------------------
!        As_WL(2) = 0.666
!        As_WL(8) = 0.591
!        As_WL(12) = 0.533
!        As_WL(15) = 0.501
!! ---------------------------------------------------
        DO I = 1, NAW
           GG = As_WL(I)
           DO L = 1, N_DIM
              TMP = GG**(L-1)*(2*(L-1) + 1)
              IF( TMP < 1.d-06 ) EXIT
              USER_MOM(I,L,K,1) = TMP
           ENDDO  ! over expansion coefficients (L)
        ENDDO  ! over wavelength (I)
     ENDDO  ! over all aerosol layers (K)
  ENDIF
  
!  IF( Scal_G > 0.d0 ) THEN
!     Lscal_g = .TRUE.
!     ALLOCATE( As_wl(NAW) )
!     As_WL(:) = USER_MOM(:,2,Iph_const,1)/3.d0
!     As_WL(:) = As_WL(:)*Scal_G
!     DO I = 1, NAW
!        GG = As_WL(I)
!        DO L = 1, N_DIM
!           TMP = GG**(L-1)*(2*(L-1) + 1)
!           IF( TMP < 1.d-06 ) EXIT
!           USER_MOM(I,L,:,1) = TMP
!        ENDDO
!     ENDDO
!     WRITE(*,'(A,2x,F6.3)') '   Selected asymmetry parameter ', As_WL(5)
     !  OPEN(47, FILE='./DATA_OUT/Asym_wl.dat')
     !  DO I = 1, NAW
     !     WRITE(47,*) AER_WAVS(I), As_WL(I)
     !  ENDDO
     !  CLOSE(47)
!  ENDIF
  
!!  ****** Constant phase function in all altitude layers
!  IF( Iph_const > 0.AND.(.NOT.Lscal_g)) THEN
!     DO I = 1, NAW
!        DO L = 1, N_DIM
!           USER_MOM(I,L,:,1) = USER_MOM(I,L,Iph_const,1)
!        ENDDO
!     ENDDO
!     WRITE(*,*) '   Asymmetry parameter ', USER_MOM(5,2,Iph_const,1)/3.
!     WRITE(*,*) '   At the altitude ', ZZZ(Iph_const) 
!  ENDIF
  
  IF( OUT_A_SCM ) THEN
     N_out = N_DIM  ;  !  N_out = 300
     IF( WEL_A > NAW )  GOTO 136
     IF( LAY_A > LAN )  THEN
        WRITE(*,'(A,I3)') '   Phase function for all layers ', LAN
        CALL COEF_TO_SCM_ALT(N_OUT, USER_MOM(WEL_A,1:N_OUT,1:LAN,1), &
             NSCA_A, GE_A, PATH_TO_SCM)
     ELSE
        WRITE(*,'(A,I3)') '   Number of aerosol layers ', LAN
!        WRITE(*,'(A,F8.3)') '   Phase function for layer top ', ZZZ( LAS(LAY_A) )
        CALL COEF_TO_SCATMATR(N_OUT, Nm_vs, &
             USER_MOM(WEL_A,1:N_OUT,LAY_A,1:Nm_vs), NSCA_A, GE_A, PATH_TO_SCM)
     ENDIF
!     I = WEL_A
!     ALLOCATE( OUT_MOM(N_OUT,Nm_vs) ) ! A
!     OUT_MOM = 0.d0
!!     S1 = SUM( Scs(I,5:10,LAY_A) )
!     DO L = 1, N_out
!        DO Jc = 1, Nm_vs
!           OUT_MOM(L,Jc) = Mom_Rh(L,I,1, 5,Jc) !*Sef(I,Ico,1)/S1
!!           DO Ico = 5, 10  ! for components depending on humidity
!!              WRITE(*,*) '   Component ', TRIM( Comp_Name(Ico) )
!!              OUT_MOM(L,Jc) = OUT_MOM(L,Jc) &
!!                   + Mom_Rh(L,I,1,Ico,Jc)*Scs(I,Ico,LAY_A)/S1
!!           ENDDO  ! over dust bins (Ico)
!        ENDDO  ! over scattering matrix elements (Jc)
!     ENDDO  ! over moments (L)
!     CALL COEF_TO_SCATMATR(N_out, Nm_vs, OUT_MOM, NSCA_A, GE_A, PATH_TO_SCM)
  ENDIF  ! OUT_A_SCM
  
  ALLOCATE( D1(NAW), ASYM_PAR(LAN,NAW), APH_VS( NLEGEN,LAN,Nm_vs,NAW ))   !A
  APH_VS = 0.d0
  AER_PHASMOMS => APH_VS( :,:,1,: )
  DO I = 1, LAN
     ASYM_PAR(I,:) = USER_MOM(:,2,I,1)/3.d0
  ENDDO

! ... Number of altitude grid corresponding to the selected aerosol layer tops.
!     Ordered from the bottom to the top of atmosphere
  DO I = 1,LAN
     LAS(I) = MINLOC( DABS(L_Top(I)-ZZZ(:)),1 )
  ENDDO
  LAE(1) = NLAYERS
  DO I = 2,LAN
     LAE(I) = LAS(I-1)-1
  ENDDO
!  WRITE(*,*) '  LAS  ZZZ(LAS),   LAE  ZZZ(LAE)  '
!  DO I = 1,LAN
!     WRITE(*,'(3x,2(2x,I3,1x,F8.3))') &
!          LAS(I), ZZZ(LAS(I)), LAE(I), ZZZ(LAE(I))
!  ENDDO
  
  ! Indexes of altitude grid levels which are within aerosol layers
  IDA(:) = LAN
  DO I = 1,LAN - 1
     DD1 = ZZZ(LAS(I)) ;  DD2 = ZZZ(LAE(I))
     DO J = 1,NLAYERS
        IF( ZZZ(J) <= DD1.AND.ZZZ(J) >= DD2 )  IDA(J) = I 
     ENDDO
  ENDDO

  DD1 = ZZZ(MAXVAL(LAE))
  ! Indexes of altitude grid levels which are below of lowest aerosol layer
  DO J = 1,NLAYERS
     IF( ZZZ(J).LE.DD1 ) IDA(J) = 1
  ENDDO

  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) ' Level number, level altitude [km], number of aerosol layer'
     DO I = 1,NLAYERS
        WRITE(LUNIT,'(2x,I3,1x,f7.3,2x,I2)') &
             I, ZZZ(I), IDA(I)
     ENDDO
     WRITE(LUNIT,*) ' ------ Test of aerosol layering "merra" mode --- '
     DO I = 1,LAN
        WRITE(LUNIT,'(2x,I3,2(1x,f7.3))') &
             I, ZZZ(LAS(I)), ZZZ(LAE(I))
     ENDDO
  ENDIF
! ... XBAER-MERRA aerosol type optical thickness at reference wavelength
  JJ = LAN
  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        Me_ext(K1) = SEcs_xb(N_ref,JJ)
     ENDDO
     JJ = JJ - 1
  ENDDO
  Me_tau = SUM( Me_ext(:)*DZ(:) )/2.d0 *1.d3
  Scoe = T_550/Me_tau

  JJ = LAN
  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        DO K = 1, NAW
           AER_COEFF(EXI,K1,K) = SEcs_xb(K,JJ)*Scoe
           AER_COEFF(SCI,K1,K) = SScs_xb(K,JJ)*Scoe
!           AER_COEFF(SCI,K1,K) = AER_COEFF(EXI,K1,K)*0.88
        ENDDO
     ENDDO
     JJ = JJ - 1
  ENDDO

  AER_COEFF(EXI,:,:) = AER_COEFF(EXI,:,:)*AER_SCALING
  AER_COEFF(SCI,:,:) = AER_COEFF(SCI,:,:)*AER_SCALING
  
!! ... Scaling of scattering coefficient
!  DO K = 1, NAW
!     DO I = 1, NLAYERS
!        Dum =  AER_COEFF(SCI,I,K)/AER_COEFF(EXI,I,K)
!!        Dum = Dum *0.918d0  ! Paris MERIS
!!        Dum = Dum *0.925d0   ! Rio-Branco MERIS
!        Dum = Dum *0.96d0   ! Rio-Branco AERONET
!        AER_COEFF(SCI,I,K) = AER_COEFF(EXI,I,K)* Dum
!     ENDDO
!  ENDDO
!!  ... ! test settings
!  AER_COEFF(SCI,:,2) = AER_COEFF(EXI,:,2)* 0.899
  
!  WRITE(*,*) '    ******** Zero aerosol extinction for Z >  ', Z_max
!  DO I = 1, NLAYERS
!     IF( ZZZ(I) >= Z_max ) THEN
!        AER_COEFF(EXI,I,:) = 1.d-12  ;  AER_COEFF(SCI,I,:) = 1.d-12
!     ENDIF
!  ENDDO
  
!=================  Replace_ND ===================================
  IF( Replace_ND ) THEN
     !Reading file containing alternative aerosol number density
     D_NAME = './N_dens.dat'
     CALL READ_FILE(2, D_NAME, DATA_R, *880)
     Nz_rep = Size(DATA_R,1)
     ALLOCATE( Nd_rep(NLAYERS) )
     CALL LIN_INT(Nz_rep, DATA_R(1:Nz_rep,1), DATA_R(1:Nz_rep,2), &
          NLAYERS, ZZZ, Nd_rep, 'Z')
     K = 0
     DO I = 1,N_COMP
        IF( C_WEI(I) /= 0.d0 ) THEN
           K = K + 1 ; N_rep = I 
        ENDIF
     ENDDO
     IF( K /= 1 ) GOTO 139
     DEALLOCATE( DATA_R, D_NAME )
     JJ = LAN
     DO J = 1,LAN
        DO K1 = LAS(J), LAE(J)
           DO K = 1, NAW
              AER_COEFF(EXI,K1,K) = AER_COEFF(EXI,K1,K)/Nd_comp(JJ,N_rep)
              AER_COEFF(SCI,K1,K) = AER_COEFF(SCI,K1,K)/Nd_comp(JJ,N_rep)
           ENDDO  ! over wavelength grid (K)
        ENDDO  ! over sub-layers (K1)
        JJ = JJ - 1
     ENDDO  ! over aerosol layers (J)
     DO K = 1, NAW
        AER_COEFF(EXI,:,K) = AER_COEFF(EXI,:,K)*Nd_rep(:)
        AER_COEFF(SCI,:,K) = AER_COEFF(SCI,:,K)*Nd_rep(:)
     ENDDO  ! over reference wavelengths (K)
  ENDIF  ! Replace_ND
! ===============================================================================

!  WRITE(*,*) '   Extinction coefficient on the RT altitude grid'
!  DO I = 1, NLAYERS-1
!     WRITE(*,'(1x,I3,2x,F8.2,*(2x,e12.4))') I, ZZZ(I), AER_COEFF(EXI,I,2)
!  ENDDO
!  WRITE(*,*) '   Extinction coefficient on the surface'
!  WRITE(*,'(1x,I3,2x,F8.2,*(2x,e12.4))')  &
!          I, ZZZ(NLAYERS), AER_COEFF(EXI,NLAYERS,2)

  AER_COEFF(EXI,:,:) = AER_COEFF(EXI,:,:)*1.d3  ! 1/m --> 1/km
  AER_COEFF(SCI,:,:) = AER_COEFF(SCI,:,:)*1.d3  ! 1/m --> 1/km
  
  ! Single scattering correction algorithm 
  ! --------------------------------------
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     Nga_db = NDB_MOM 
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )
     Greek_aer = 0.d0
     DO I = 1,NAW 
        DO K = 1,Nm_vs
           DO L = 1, LAN
              Greek_aer(:,L,K,I) = USER_MOM(I,:,L,K)
           ENDDO
        ENDDO
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  DO I = 1,LAN 
     DO J = LAS(I),LAE(I)
        AER_COEFF(ASY,J,:) = ASYM_PAR(I,:)
     ENDDO
  ENDDO

  ! Aerosol total optical thickness
  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO
  
!  WRITE(*,*) '   =====> AOT at reference wavelengths: '
!  DO K = 1,NAW
!     WRITE(*,'(2x,I2,2x,F8.3,2x,E12.4)') K, AER_WAVS(K), TAU(K)
!  ENDDO
!  PAUSE
  
!     ... Scaling of number density at all altitudes
  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF (ERROR) GOTO 126
     Scoe = Rtau_aer/( (1.d0-DUM)*TAU(K) + DUM*TAU(K+1) )
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
!     WRITE(*,*) '  New tau:', TAU
  ENDIF
  
  !======================================
  !   Write aerosol scenario output file
  CALL WRITE_SCEN_XB_MERRA(*880)
  !======================================

!  STOP '   Stop after "WRITE_SCEN_XB_MERRA" '
  
  !Allocate array for MODULE DELTA_M
  !=================================
  IF ( AER_DM_FLG ) THEN 
     Itr = AER_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')
        CALL AER_DM_VEC
     CASE('scal_rte')
        SELECT CASE ( TRIM(D_MF_AER) )
        CASE( 'D-FA' )
           N_ANG = 180  ;  ALLOCATE( X_ang(N_ANG) )
           Nleg_df = NDB_MOM 
           ALLOCATE(PHAS_LOC( Nleg_df), PL_RES( Nleg_df,N_ANG )) ! A 
           Nleg_Loc = AER_TRUNC_IDX + 1
           ALLOCATE( R_COF( Nleg_Loc ))
           CALL Start_Delta_Fit( N_ANG, Nleg_df )
           DO I1 = 1,LAN
              ! WRITE(*,*) '  I1 = ',I1
              PHAS_LOC(1:Nleg_df) = USER_MOM(1,1:Nleg_df,I1,1)
              CALL Delta_Fit( N_ANG, Nleg_df, PHAS_LOC )
              DO K1 = LAS(I1), LAE(I1)
                 DO K = 1, NAW
                    FDM_AER(K1,K) = F_Dfit_loc
                    AER_PHASMOMS(1:Nleg_loc,I1,K) = & ! Proverit' 
                         R_cof(1:Nleg_loc)/R_cof(1)
                 ENDDO
              ENDDO
           ENDDO
        CASE( 'D-MA' )
           CALL AER_DM
        END SELECT
     END SELECT
     !"Renormalize" scattering and extinction coefficients
     !====================================================
     DO I = 1, NLAYERS
        DO K = 1,NAW
           ! test for consistence between "mi" and "wmo_ni" modes
           IF( AER_COEFF(EXI,I,K) == 0.d0 ) FDM_AER(I,K) = 0.d0
           F_DM = FDM_AER(I,K)
           AER_ABS_COEFF = AER_COEFF(EXI,I,K ) - AER_COEFF(SCI,I,K)
           AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0-F_DM)
           AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + AER_ABS_COEFF 
        ENDDO
     ENDDO
  ELSE
     DO I1 = 1,LAN
        DO K = 1, NAW
           APH_VS(:,I1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
        ENDDO
     ENDDO
  ENDIF

  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ; IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF

!  STOP '   Stop in "merra_xb_setup.f90" '
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP 
  RETURN 1

126 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Aerosol models name not found')
  RETURN 1
  
128 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Error reading of "WMO_DB_MOM_UV-TIR.dat"')
  RETURN 1

129 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F8.3)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F8.3))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1

131 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Upper aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') & 
       '    Upper aerosol layer:', L_TOP(LAN-1),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZZZ(1),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

132 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Lower aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F6.2,A)') '       Top of aerosol layer:', L_TOP(1),' km'
  WRITE(EU,'(A,2x,F6.2,A)') '       Surface elevation   :', ZZZ(NLAYERS),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and "Height above '
  WRITE(EU,*) '    sea level" in the file "control.inp" '
  RETURN 1

133 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' Aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') '    Aerosol layer:', ZZZ(NLAYERS),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZZZ(1),' km'
  WRITE(EU,*) '    Check "wmo_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

135 CALL WRITE_MESSAGE (R_NAME,'ERROR', &
         'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1

136 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A
  RETURN 1

139 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A,I3)') '    Replacement of aerosol number density profile: '
  WRITE(EU,'(A,I3)') '        can be performed only in the case single '
  WRITE(EU,'(A,I3)') '        selected aerosol component. '
  RETURN 1

140 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A,I3)') '    Negative value of scattering coefficient XBAER type: '
  WRITE(EU,'(A,2x,F8.2)') '        wavelength  ', AER_WAVS(I)
  RETURN 1

  ! ***********************************************************************
137 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A,I3)') '    Number of altitude layers in scenario file: ',N_lay
  WRITE(EU,'(A,I3)') '        does not equal with altitude grid file: ', &
       Size(DATA_R,1)

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
! ************************************************************************
  SUBROUTINE COEF_TO_SCM_ALT(N_MOM, COF, N_ANG, DISCR, OUT_PATH)
!     Input variables and arrays
!     --------------------------
    INTEGER   N_MOM,  N_ANG
    CHARACTER (LEN=1)   DISCR
    CHARACTER (LEN=*)   OUT_PATH
    REAL (KND)   COF(N_MOM,LAN)
!     Local variable
!     --------------
    INTEGER    I, J
    INTEGER, PARAMETER :: OUNIT = 44
    REAL (KND)  STEP, ANG(N_ANG), SCAT(N_ANG), AG(N_ANG-2), XG(N_ANG-2), &
         Plm_ss(N_MOM,N_ANG), F11(N_ANG,LAN)
!    ------------------------------------------------------------------------
!     ... Select grid for scattering angles representation
    SELECT CASE( DISCR )
    CASE('G')
!        ... Gauss grid 
       CALL GAULEG ( -1.D0, 1.D0, XG, AG, N_ANG-2 )
       SCAT(1) = 1.d0 ; SCAT(N_ANG) = -1.d0
       SCAT(2:N_ANG-1) = XG(N_ANG-2:1:-1)      
       ANG(:) = DACOS( SCAT(:) )*RAD_TO_DEG 
    CASE('E')
!        ... Equidistant grid
       STEP = 180.D0/DBLE( N_ANG-1 ) ; ANG(1) = 0.d0         
       DO I = 2,N_ANG
          ANG(I) = ANG(I-1) + STEP
       ENDDO
       SCAT(:) = DCOS( ANG(:)/RAD_TO_DEG )
    CASE DEFAULT
       WRITE(*,'(a)') '  Wrong type of discretization shema'
       WRITE(*,'(a,2x,a)') '  As input :', DISCR
       WRITE(*,'(a)') "  Accepted 'G' or 'E' only"
       STOP '  Stop in "COEF_TO_SCM_ALT"'
    END SELECT

!     ... Legendre polynomials for all scattering angles
    CALL WIGNER_PLG(SCAT, N_ANG, 0, N_MOM, Plm_ss )
    DO I = 1, LAN
       DO J = 1,N_ANG
          F11(J,I) = SUM( Plm_ss(:,J)*COF(:,I) )
       ENDDO  ! over scattering angles (J)
    ENDDO  ! over all aerosol layers (I)

    WRITE(*,*) '   The elements of scattering function'
    WRITE(*,*) '   are written in ',TRIM(OUT_PATH)
    OPEN(OUNIT,FILE=TRIM(OUT_PATH))
    WRITE(OUNIT,'(a)')  &
         '#  Scattering function corresponding to the '// &
         'given expansion coefficients '
    WRITE(OUNIT,'(a)') '#  Column :'
    WRITE(OUNIT,'(a)') '#  1 - scattering angle [deg]'
    WRITE(OUNIT,'(a,I3,a)') &
         '#  2 -',LAN+1, ' normalized phase function for all layers'//&
         ' of MERRA model from TOA to surface'
    DO J = 1,N_ANG
       WRITE(OUNIT,'(2x,f7.2,*(2x,f11.5))') ANG(J), (F11(J,I), I=1,LAN)
    ENDDO
    CLOSE(OUNIT)
      !         PAUSE  '** PAUSE in "COEF_TO_SCM_ALT" (st_tools.f) **'
    RETURN      
  END SUBROUTINE COEF_TO_SCM_ALT
! ************************************************************************
  
  SUBROUTINE READ_MOMS(Nc, Nwl, WL_gr, Nlo_MOM, *)
    INTEGER Nc, Nwl, Nlo_MOM
    REAL (KND) WL_gr(Nwl)
!   -----------------------------------------------------------------------
    OPEN(C_UNIT, FILE=TRIM(Comp_Name(Ico))//'/MERRA_moments_'//MOM_DAT(Jc), &
         STATUS='OLD')
    N_SKIP = Nlo_MOM*(Nw1-1) + N_SKIP_HEADER
    DO I = 1, N_SKIP  ;  READ(C_UNIT,*)  ;  ENDDO
    DO I = 1, Nwl
       DO J = 1, Nlo_MOM
          READ(C_UNIT,*) (MOM_x(J,I,K), K = 1,Nc)
       ENDDO  !  over moments (J)
       IF( Jc==1 .AND. MOM_x(1,I,1) /= 1.d0 ) GOTO 138
    ENDDO  ! over actual wavelengths (I)
    CLOSE(C_UNIT)
!   ... Interpolation on generalized WL grid
    DO K = 1, Nc
       DO J = 1, Nlo_MOM
          CALL LIN_INT(Nwl, WL_gr, MOM_x(J,:,K), NAW, AER_WAVS, &
               MOM_Rh(J,:,K, Ico,Jc), 'C')
       ENDDO  ! over moments (J)
    ENDDO  ! over RH (K)

    RETURN
138 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
    WRITE(EU,'(A,E12.4)') '    First moment of P11 does not equal 1: ', MOM_x(1,I,1)
    WRITE(EU,'(A,I2)')    '    Aerosol component ', Ico
    WRITE(EU,'(A,I2)')    '    Matrix element    ', Jc
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
    
  END SUBROUTINE READ_MOMS
!***********************************************************************

  SUBROUTINE EXT_SCAT(Nco, Nwl, WL_gr, *)
    INTEGER Nco, Nwl
    REAL (KND) WL_gr(Nwl)
!   --------------------------------------------------------------------
    D_NAME = TRIM(Comp_Name(I))//'/MERRA_extinction_efficiency.dat'
    CALL READ_FILE(Nco+1, D_NAME, DATA_R, *880)
    Nx = Size(DATA_R,1)
!    Write(*,*) '   Number WL ', Nx
    DO J = 1, Nco
       ! Interpolation on the actual wavelength grid
       CALL LIN_INT(Nx, WL_gr, DATA_R(1:Nx,J+1), NAW, AER_WAVS, &
            Eef(:,I,J), 'C')
    ENDDO
    DEALLOCATE( DATA_R, D_NAME )
    !               Read scattering efficiency
    D_NAME = TRIM(Comp_Name(I))//'/MERRA_scattering_efficiency.dat'
    CALL READ_FILE(Nco+1, D_NAME, DATA_R, *880)
    DO J = 1, Nco
       ! Interpolation on the actual wavelength grid
       CALL LIN_INT(Nx, WL_gr, DATA_R(1:Nx,J+1), NAW, AER_WAVS, &
            Sef(:,I,J), 'C')
    ENDDO
    DEALLOCATE( DATA_R, D_NAME )

    RETURN
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
   END SUBROUTINE EXT_SCAT
! ***************************************************************************

  SUBROUTINE MAKE_WL_GRID(Name, Nwl, WL, *)
    INTEGER Nwl
    CHARACTER (LEN=*) Name
    REAL (KND), ALLOCATABLE :: WL(:)
    CALL READ_FILE(1, Name, DATA_R, *880)
    Nwl = Size(DATA_R,1)
!         WRITE(*,*) '   Number of WL = ', Nwl
    ALLOCATE( WL(Nwl) )
    WL(:) = DATA_R(1:Nwl,1)*1.d+9 !  m --->  nm
!          WRITE(*,*) '  10 first WLs '
!          WRITE(*,'(*(1x,F6.1))') WL(1:10) 
    DEALLOCATE( DATA_R )    
    RETURN
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  END SUBROUTINE MAKE_WL_GRID
  
!***********************************************************************
  SUBROUTINE Start_Delta_Fit( N_ANG, Nleg_df )
    INTEGER           N_ang, Nleg_df
    ! Local variables
    REAL (KND)  Kat_ang
    REAL (KND), DIMENSION (N_ANG) ::  X_gau, AG_ang
    ! Cut-angle ]deg]
    Kat_ang = 2.0d0
    Kat_ang = DCOS( Kat_ang/RAD_TO_DEG )

    CALL GAULEG ( -1.D0, 1.D0, X_gau, AG_ang, N_ANG )

    ! Invert Gauss grid to start from zero scattering angle 
    X_ang(:) = X_gau(N_ang:1:-1)

    ! Legendre polynomials for all Gauss angles and all Max_mom
    CALL WIGNER_PLG( X_ang, N_ang, 0, Nleg_df, PL_RES )

    ! Calculate phase function for all Gauss nodes
    DO I = 1,N_ang
       IF( X_ang(I) < Kat_ang ) Then
          N_KAT = I
          EXIT
       ENDIF
    ENDDO
    IF (LOG_LEVEL(1) >= N_ESO) THEN
       WRITE(LUNIT,*) 
       WRITE(LUNIT,*) ' ***** Cut-angle for Delta-fit approximation **** '
       WRITE(LUNIT,'(2x,i3,2x,e12.4)') N_KAT,DACOS(X_ang(N_KAT))*RAD_TO_DEG
       WRITE(LUNIT,*) 
    END IF

  END SUBROUTINE Start_Delta_Fit

!***********************************************************************
  SUBROUTINE Delta_Fit( N_ANG, Nleg_df, Phas_moms )
    INTEGER           N_ang, Nleg_df
    REAL (KND)  Phas_moms(Nleg_df)
    ! Local variables
    INTEGER           I, K
    REAL (KND), DIMENSION (N_ANG) :: Phm1, Phm1_RD
    REAL (KND)  A_VEC( (Nleg_loc + 1)*(Nleg_loc + 2)/2 ), PL_Nor( Nleg_loc,N_ANG ),&
         PL_Nor1( Nleg_loc,N_ANG ), A_M( Nleg_loc,Nleg_loc )
    
    DO I = 1, N_ANG
       Phm1(I) = SUM( PL_RES(:,I)*Phas_moms(:) )
    ENDDO
    
    PL_NOR1 = 0.0d0
    DO K = 1,N_ANG
       PL_NOR(:,K) = PL_RES(1:Nleg_loc,K)/Phm1(K)
    ENDDO
    PL_NOR1(:,N_KAT:N_ang) = PL_NOR(:,N_KAT:N_ang)
    A_M(:,:) = MATMUL( PL_NOR(:,:),TRANSPOSE(PL_NOR1(:,:)) )
    
    FORALL (I = 1:Nleg_loc) A_VEC(I*(I - 1)/2 + 1:I*(I + 1)/2) = A_M(I,1:I)
    CALL CHINV3 (Nleg_loc*(Nleg_loc+1)/2, Nleg_loc, A_VEC)
    
    FORALL (I = 1:Nleg_loc) A_M(I,1:I) = A_VEC(I*(I - 1)/2 +1:I*(I + 1)/2)
    FORALL (I = 1:Nleg_loc, J = 1:Nleg_loc, J > I) A_M(I,J) = A_M(J,I)
    R_COF(:) = MATMUL( A_M,SUM( PL_NOR1(:,:),2 ) )
    F_Dfit_loc   = 1.0d0 - R_COF(1)
    !      write( *,* ) ' ---> R_COF(1) = ', R_COF(1)

    DO I = 1, N_ANG
       Phm1_RD(I) = SUM( PL_RES(1:Nleg_loc,I)*R_COF(1:Nleg_loc) )
    ENDDO

    OPEN( NewUnit = DUNIT, file = DIR_OUT//'phase.test' )
    DO I = 1, N_ANG
       WRITE( DUNIT,'(5(2x,e15.7))') X_ANG(I), DACOS(X_ANG(I))/3.14159*180., & 
            Phm1(I), Phm1_RD(I)
    ENDDO
    CLOSE(DUNIT)

  END SUBROUTINE Delta_Fit

!***********************************************************************
  SUBROUTINE AER_DM
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             AER_PHASMOMS(L,I1,K) = ( USER_MOM(K,L,I1,1) - &
                  F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.0D0
  END SUBROUTINE AER_DM
  
!***********************************************************************
  SUBROUTINE AER_DM_VEC
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             APH_VS(L,I1,1:4,K) = ( USER_MOM(K,L,I1,1:4) - &
                  F_DM*DBLE(2*L-1) )/(1.0D0 - F_DM)
             APH_VS(L,I1,5:6,K) = USER_MOM(K,L,I1,5:6)/(1.0D0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    APH_VS(Itr+1:NLEGEN,:,:,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

!***********************************************************************
  SUBROUTINE  WRITE_SCEN_XB_MERRA(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER, Alph, C_a
    USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 
 
    INTEGER    K1, K2, J, Unt
    Character (Len=500)  OutStr
    CHARACTER (LEN=2)  NC_nam(N_xb)
    REAL (KND)  SS_A, Ext, S1, Ex_co(NLAYERS,NAW,N_xb), Tau_co(NAW,N_xb)
    REAL (KND), ALLOCATABLE :: T_OUT(:,:), E_OUT(:), S_OUT(:), T_BL(:), T_UP(:), &
         P_EXT(:,:)
!***********************************************************************
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (lunit,*) ' *** writing: AEROSOL.OUT'
    NC_nam(1) = 'BC'
    NC_nam(2) = 'OC'
    NC_nam(3) = 'SU'
    NC_nam(4) = 'SS'
    NC_nam(5) = 'DU'
    
    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) THEN 
       K1 = LAS(1) ; K2 = LAE(1)
       JJ = LAN
       DO J = 1,LAN
          DO K1 = LAS(J), LAE(J)
             DO K = 1, NAW
                Ex_co(K1,K,:) = Ecs_xb(K,:,JJ)*Scoe
             ENDDO
          ENDDO
          JJ = JJ - 1
       ENDDO
!      ... Optical thickness of XBAER components
       DO K = 1,NAW
          DO J = 1, N_xb
             TAU_co(K,J) = Fr_xb(J)*SUM( Ex_co(:,K,J)*DZ(:) )/2.d0*1.d3  ! 1/m --> 1/km
          ENDDO
       ENDDO
       TAU_co(:,:) = TAU_co(:,:)*AER_SCALING

       ALLOCATE( T_OUT(NOWL_AER,N_xb),E_OUT(NOWL_AER),S_OUT(NOWL_AER),&
            T_BL(NOWL_AER), T_UP(NOWL_AER), P_EXT(NLAYERS,2))

       DO I = 1, NOWL_AER
          CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
          IF (ERROR) GOTO 134
          T_OUT(I,:) = (1.d0 - DUM)*TAU_co(K,:) + DUM*TAU_co(K+1,:)
          
          ! Single scattering albedo at user-defined wavelengths
!          S_OUT(I) = (1.d0 - DUM)*AER_COEFF(SCI,NLAYERS-1,K) &
!               + DUM*AER_COEFF(SCI,NLAYERS-1,K+1)
          ! Extinction coefficient at user-defined wavelengths
!          P_EXT(:,1:2) = AER_COEFF(EXI,:,K:K+1)
!          E_OUT(I) = (1.d0 - DUM)*P_EXT(NLAYERS-1,1) + DUM*P_EXT(NLAYERS-1,2)
          ! Optical thickness of the boundary layer
!          S1 = ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
!               *(ZZZ(K1) - ZZZ(K1+1)) + ( (1.d0-DUM)*P_EXT(K2,1) &
!               + DUM*P_EXT(K2,2) )*(ZZZ(K2-1) - ZZZ(K2))
!          DO J = K1+1,K2-1
!             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
!                  *(ZZZ(J-1) - ZZZ(J+1))
!          ENDDO
!          T_BL(I) = S1/2.d0
          ! Optical thickness of the atmosphere above boundary layer
!          S1 = ( (1.d0-DUM)*P_EXT(1,1) + DUM*P_EXT(1,2) )*(ZZZ(1) - ZZZ(2)) &
!               + ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
!               *(ZZZ(K1-1) - ZZZ(K1))
!          DO J = 2,K1-1
!             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
!                  *(ZZZ(J-1) - ZZZ(J+1))
!          ENDDO
!          T_UP(I) = S1/2.d0
       ENDDO  ! over user defined output wavelengths (I)

!       OPEN(NewUnit=DUNIT,FILE = DIR_OUT//'AOT.OUT')            
!       WRITE(DUNIT,'(A)') '# AOT and single scattering albedo (boundary layer) '
!       WRITE(DUNIT,'(A)') '#        at user-defined wavelengths [nm]:'
!       DO I = 1, NOWL_AER
!          WRITE(DUNIT,'(2x,F8.3,4(2x,E15.7))') OWL_AER(I),&
!               T_BL(I), T_UP(I), T_OUT(I), S_OUT(I)/E_OUT(I)
!       ENDDO
!       CLOSE(DUNIT)

       ! Angstrom parameters for this aerosol model
       ! Alph = DLOG( T_OUT(1)/T_OUT(NOWL_AER) )&
       !      /DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
       ! C_a = T_OUT(1)/OWL_AER(1)**Alph
!       Alph = DLOG( T_BL(1)/T_BL(NOWL_AER) )/DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
!       C_a = T_BL(1)/OWL_AER(1)**Alph
!         WRITE(*,*) '  Angstrom parameters '
!         WRITE(*,*) '  C_a  = ', C_a
!         WRITE(*,*) '  Alph = ', Alph
    ENDIF

    Call OpenOut ('SCE_AEROSOL.OUT', Unt)
    WRITE(OutStr,*) '        SETTINGS FOR XBAER_MERRA AEROSOL MODEL  '
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '     ==========================================='
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) ' '
    Call WriteOut (Unt, OutStr)
  
    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) &
            ' OT of aerosol components at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       WRITE (OutStr,'(17x,A,4(8x,A),7x,A)') Nc_nam(1:5), ' Sum'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(4x,F7.2,*(2x,F8.4))') OWL_AER(I), &
               T_out(I,:), SUM( T_out(I,:) )
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    Call WriteOut (Unt, ' ')

    WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
    Call WriteOut (Unt, OutStr)
              
    Call WriteOut (Unt, ' ')
    Call WriteOut (Unt,' Vertical profiles of aerosol')
    Call WriteOut (Unt,'         extinction coefficient (Ext coeff) and')
    Call WriteOut (Unt,'         single scattering albedo (Ss albedo):')

    DO K = 1,NAW
       Call WriteOut (Unt,'   ')
       WRITE(OutStr,'(1x,a11,F9.2,a6,a6,F8.4)') &
            'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       Call WriteOut (Unt, OutStr)
       Call WriteOut (Unt,' ----------------------------------------------')
       Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = AER_COEFF(SCI,I,K)/Ext
          WRITE(OutStr,'(6x,I3,4x,F6.2,5x,ES9.2,6x,E12.4)') I, ZZZ(I), Ext, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       Call WriteOut (Unt, ' ')
       Call WriteOut (Unt,'    Asymmetry factor used in the aerosol layers :')
       Call WriteOut (Unt,'    ------------------------------------------- :')
       Call WriteOut (Unt,'    Layer  Top [km]  Bottom [km]   Asymmetry    Phase')
       Call WriteOut (Unt,'                                    factor     function')
       JJ = LAN
       DO I = 1,LAN
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,5x,a8)') &
               I, ZZZ(LAS(I)) ,ZZZ(LAE(I)), Asym_Par(JJ,K), 'database'
          Call WriteOut (Unt, OutStr)
          JJ = JJ - 1
       ENDDO
    ENDDO  ! Over wavelengths (K)
          
    Call CloseOut (Unt)
    RETURN

134 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Required output wavelength:      ', OWL_AER(I)
    WRITE(EU,'(A,2x,F7.2,A,2x,F7.2)') &
         '    Selected wavelengths interval: ', &
         AER_WAVS(1), '  -', AER_WAVS(NAW)
    WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1

  END SUBROUTINE  WRITE_SCEN_XB_MERRA

!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
    IF(ALLOCATED( AER_WAVS)) DEALLOCATE(AER_WAVS, AER_COEFF, FDM_AER)
    IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP, APH_VS)
    IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
    IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
    IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
  
END SUBROUTINE MERRA_XB_SETUP
!***********************************************************************

