C********************************************************************C
C
C	SCIATRAN - Version 4.6 - Last modification: 30 Jul 2022
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE MIE_AER_SETUP( START_WAV, END_WAV,* )
c     ... Called by "rtm_init.f"

      USE AER_MIE,     ONLY: MOD_R, NAC, SIG, RD, RI_rea, RI_ima, N_PSD,
     $                       L_ARR, Nst, LAS_mi, LAE_mi, Rmi_lim, M_AC,
     $                       COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, 
     $                       COEF7, COEF8, F_MODE, DENS_PRF, N_01, 
     $                       AOD_Mie, AOD_RET, T_550, Cfr, PM_2i5, 
     $                       PM_2i5_MOD
      USE AEROSOLS,    ONLY: AER_SHIFT, PATH_TO_SCM, N_Dwr, Mie_layer,
     $                       Dwr_mi, Plus_mi, NAW_MI, AW_MI, AC_MI,
     $                       IDX_AER_mi, IDX_AER_db, IDX_AER, AER_DER,
     $                       LA_Nmi, LA_Ndb, LA_N, Use_der, AP_Mie,
     $                       APH_VS_mi, Greek_Aer_mi, APH_DER, MODIS_AP,
     $                       APH_VS_db => APH_VS, D_Greek, MODIS_LO,
     $                       Greek_Aer_db => Greek_Aer,
     $                       LA_S, LA_E, L_TOP, N_AER_WAVS, 
     $                       AW_DB => AER_WAVS, AC_DB => AER_COEFF,  
     $                       OUT_A_SCM, LAY_A, WEL_A, NSCA_A, GE_A,
     $                       NG_aer, NWLS, NWLE, Nga_db, Nga_mi,
     $                       SCI => GT_SCAT_IDX, Rf_max, Bi_PSD,
     $                       ASI => GT_ASYM_IDX, Scal_aer, 
     $                       EXI => GT_EXTC_IDX, Rtau_aer, Rwl_aer,
     $                       AER_SCALING, AOT_out_VAL, L_scal, ZS1, ZS2,
     $                       AER_FACT, V_PAP
      USE CLOUDS_SS,   ONLY: MUM => MAXMOM, DO_SSRCA
      USE DELTA_M,     ONLY: FDM_AER_db => FDM_AER, FDM_AER_mi, 
     $                       AER_DM_FLG, AER_TRUNC_IDX
      USE GRIDS,       ONLY: NLAYERS, ZHOLD => ZZZ, DZ
      USE GT_POLYN,    ONLY: NLEGEN
      USE INFO,        ONLY: AER_INFO
      USE INTERN_DIMS, ONLY: Nm_vs
      USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, 
     $                       N_DSO, Work_Dir
      USE PM_ARR,      ONLY: V_c, V_f, A_c, A_f, Ro_dens
      USE RETR_ITER,   ONLY: ITERATION
      USE RTM_OUTPUT,  ONLY: MAX_AER_WF
      USE RTM_PARS,    ONLY: DATA_ITER, RTM_ST
      USE ST_INPUT,    ONLY: ADB => PATH_AER_DB, S_N_D,  SND_NAME,  
     $                       SND_WL 
      USE ST_VARS,     ONLY: RTM_TYPE, GROUND_HEIGHT, Vec_Sca, Pie, 
     $                       SCATT_MODE, RTM_MODE
      USE TOOLS,       ONLY: CHK_STAT, SET_PTR, FIND_IDX
      USE TYPES,       ONLY: KND, DYN_CHAR
c     ------------------------------------------------------------------

      CHARACTER (LEN = 13), PARAMETER :: R_NAME = 'MIE_AER_SETUP'
      CHARACTER (LEN=200)   STR_READ, STR_READ_ARR(90)      
      CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
C  (a) Input variables
c  ===================
C      AER_DM_FLG        : delta-M approximation yes/no 
C                          <---- From Module DELTA_M
C      AER_TRUNC_IDX     : truncation index for delta-M approximation 
C                          <---- From Module DELTA_M
C      ZHOLD             : forward model altitude grid
C      NLA+ERS           : number of layers in altitude grid
C      NLEGEN            : number of Legendre moments
C      LUNIT             : LOG file unit number
C      ADB               : path to data base

C  (b) Output variables
c  ====================
C      AER_COEFF         : scattering/extinction coefficients and 
C                          asymmetry parameters 
C      AER_PHASMOMS      : Legendre moments
C      NAW               : number of wavelengths of chosen aerosol 
C                          parameterization
C      AER_WAVS          : wavelengths of aerosol data base (depends on
C                          parameterization chosen)

C  (c) Local variables for SCIATRAN parameterization
c  =================================================
C      LAN              : number of aerosol layers
C      LTOP              : upper boundary [km] of aerosol layer
C      RELHUM            : relative humidity [%] of aerosol layer
C      AER_TYPE          : aerosol type 
C      NAC(1:LAN)       : number of aerosol components of each aerosol 
c                          type 
C      RD                : relative particle number densities of the
C                          various aerosol components of an aerosol type
C      N_AER_LEV         : number of aerosol profile levels     
C      AER_LEV           : aerosol profile levels
C      AER_PRF           : aerosol profile (extinction coefficient or
C                          particle number density)
c      NL_DB             : max number of expansion coeff, after Mie calc.
c      M_H               : number of humidities 
c      M_CH              : number of aerosol components 
c                          depending on humidity
c      M_CI              : number of aerosol components
c                          independent on humidity
      INTEGER    NL_DB, M_H, M_CH, M_CI, N_DIM, N_COM
      PARAMETER  (M_H = 8, M_CH = 4, M_CI = 8)
      INTEGER  I, I1, I2, Idr, J, K, K1, Ko, KUNIT, L, Itr, N_AER_LEV, 
     $     N_MIE_WL, NAW, Nfm, Ncm, Nsm, N13, Kw, N_AWF, N_SAM
      LOGICAL  Modr_f, Modr_c, Var_f, Var_c, Frac_c, SAMUM, Ph_const,
     $     Recalc_SCA, SAMUM_PH, FLG, Ang_INTERP
      CHARACTER(LEN=6),  ALLOCATABLE :: DB_NAM(:,:)
      CHARACTER(LEN=6)   H6
      CHARACTER(LEN=30), ALLOCATABLE :: AER_TYPE(:)
      CHARACTER(LEN=70)  Txt_OUT
      REAL (KND)  DB_HUMS(M_H)
      DATA DB_HUMS  /0., 50., 70., 80., 90., 95., 98., 99./
      CHARACTER(LEN=2)   HYM(M_H)
      DATA HYM  /'00', '50', '70', '80', '90', '95', '98', '99'/
      INTEGER  NWL_DB, MCO, N_SKIP, Ig, Id, I0, LAN, DIM_APH_DER, 
     $     Na_st, N_gau, II, N_Ex
      REAL (KND) START_WAV, END_WAV, F_DM, DUM, S1, S2, Ome1, Ome2,
     $     Scoe, DD1, DD2, Z_BOT, Rod, VOL, Dc, Sfac, Tau_c, Tau_f
      INTEGER, ALLOCATABLE :: NH(:), LAS(:), LAE(:), IDA(:), 
     $     N_EC(:,:,:), M_COF(:,:)
      REAL (KND), ALLOCATABLE :: WL_MIE(:), ECO(:,:,:,:), Hel(:,:),
     $     SCO(:,:,:,:), USER_MOM(:,:,:,:,:), AER_LEV(:), AER_PRF(:), 
     $     REL_HUM(:), ASYM_PAR(:,:), D1(:,:), TAU(:), LTOP(:), 
     $     DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:),
     $     DUMA(:), PSD_P(:), SCS(:), WL_SAM(:), SSA_SAM(:), SSA_H1(:),
     $     Sa_inp(:), Phs_inp(:,:,:), Leg_cof(:,:,:), Ratio(:,:), 
     $     Phs_0(:), DBe_PM(:,:,:), DBs_PM(:,:,:), Vo_PM(:,:), 
     $     ECO_PM(:,:,:), SCO_PM(:,:,:), Qe(:,:), PM_tf(:), Z_Ex(:), 
     $     Pr_ex(:), EXB(:), Ext_PM(:), Pr_in(:), PM_vol(:), PM_mas(:),
     $     PM_area(:)
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND), POINTER :: AER_WAVS(:), AER_COEFF(:,:,:), 
     $     AER_PHASMOMS(:,:,:), APH_VS(:,:,:,:), FDM_AER(:,:)
      TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), STR_ARR_1(:)
C     ==================================================================
      INTERFACE
         SUBROUTINE READ_MOD_AER_INP(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $        AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $        DBe_PM, DBs_PM, Vo_PM, *, *)
         USE TYPES,  ONLY: KND
         INTEGER  NAW, LAN, N_COM, N_AER_LEV
         INTEGER, ALLOCATABLE :: N_EC(:,:,:)
         REAL (KND)  Z_BOT
         REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), 
     $        DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:),
     $        AER_LEV(:), AER_PRF(:), DBe_PM(:,:,:), DBs_PM(:,:,:),
     $        Vo_PM(:,:)
         END SUBROUTINE
c        ----------------------------------------------------------------
         SUBROUTINE READ_MOD_AER_OC(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $        AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $        *, *)
         USE TYPES,  ONLY: KND
         INTEGER  NAW, LAN, N_COM, N_AER_LEV
         INTEGER, ALLOCATABLE :: N_EC(:,:,:)
         REAL (KND)  Z_BOT
         REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), 
     $        DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:),
     $        AER_LEV(:), AER_PRF(:)
         END SUBROUTINE
c        ----------------------------------------------------------------
         SUBROUTINE READ_MOD_AER_MAN(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $        AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $        DBe_PM, DBs_PM, Vo_PM, *, *)
         USE TYPES,  ONLY: KND
         INTEGER  NAW, LAN, N_COM, N_AER_LEV
         INTEGER, ALLOCATABLE :: N_EC(:,:,:)
         REAL (KND)  Z_BOT
         REAL (KND), ALLOCATABLE :: WL_MIE(:), LTOP(:), 
     $        DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:),
     $        AER_LEV(:), AER_PRF(:), DBe_PM(:,:,:), DBs_PM(:,:,:),
     $        Vo_PM(:,:)
         END SUBROUTINE
c        ----------------------------------------------------------------
      SUBROUTINE READ_MIE_AER_INP(NAW, START_WAV, END_WAV, WL_MIE,
     $     LAN, Z_BOT, REL_HUM, N_COM, DB_NAM, N_AER_LEV, AER_LEV,
     $     AER_PRF, Vo_PM, AER_TYPE, LTOP, N_EC, DB_EXT, DB_SCA,
     $     DB_MOMS, *, *)
         USE TYPES,  ONLY: KND
         INTEGER  NAW, LAN, N_COM, N_AER_LEV
         INTEGER, ALLOCATABLE :: N_EC(:,:,:)
         REAL (KND)  Z_BOT, START_WAV, END_WAV
         REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), 
     $        DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:),
     $        AER_LEV(:), AER_PRF(:), Vo_PM(:,:)
         CHARACTER(LEN=6),  ALLOCATABLE :: DB_NAM(:,:)
         CHARACTER(LEN=30), ALLOCATABLE :: AER_TYPE(:)
         END SUBROUTINE
      END INTERFACE

C     ==================================================================
C           Parameterization based on the online Mie code calculations
C     ==================================================================
      CALL ST_TRACK (R_NAME, 1, 'I')      

      PM_2i5 = .FALSE.
c     ... PM 2.5 mkm  mode in combination with "mie_aer.inp"
c      PM_2i5 = .TRUE.

      SAMUM = .FALSE. ;  SAMUM_PH = .FALSE.
c     ... SAMUM single scattering albedo and phase function will be used
c      SAMUM = .TRUE.  ;  SAMUM_PH = .TRUE.

c      WRITE(*,*) '   Enter SAMUM and SAMUM_PH'
c      READ(*,*) SAMUM, SAMUM_PH

c      WRITE(*,*) '  SHAPE(AC_DB) ',SHAPE(AC_DB)
      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here

c     ------------------------------------------------------------------
      IF( MODIS_AP ) THEN
         SELECT CASE( TRIM(MODIS_LO) )
         CASE('land') 
            CALL READ_MOD_AER_INP(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $           AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $           DBe_PM, DBs_PM, Vo_PM, *880, *122)
            Txt_OUT  = '        Settings for MODIS land  model  '
         CASE('ocean')
            CALL READ_MOD_AER_OC(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $           AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $           *880, *122)
            Txt_OUT  = '        Settings for MODIS ocen  model  '
         CASE('man')
            IF( Scal_aer ) GOTO 134
            CALL READ_MOD_AER_MAN(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, 
     $           AER_LEV, AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, 
     $           DBe_PM, DBs_PM, Vo_PM, *880, *122)
            Txt_OUT  = '        Settings for MODIS MAN  model  '
         END SELECT
      ELSE
c        ... Read scenario input parameters: file "mi_aer.inp" 
c            and perform Mie calculations using the "sc_mie.f" routine
         CALL READ_MIE_AER_INP(NAW, START_WAV, END_WAV, WL_MIE, LAN,
     $        Z_BOT, REL_HUM, N_COM, DB_NAM, N_AER_LEV, AER_LEV,
     $        AER_PRF, Vo_PM, AER_TYPE, LTOP, N_EC, DB_EXT, DB_SCA,
     $        DB_MOMS, *880, *122)
      ENDIF
c     ------------------------------------------------------------------

c     ... Maximal number of WF for aerosol parameters
      IF( N_dwr > 0 ) MAX_AER_WF = 1+N_dwr

      WL_MIE(:) = WL_MIE(:)*1.d+3 !  [micron] -> [nm]

      SELECT CASE( TRIM(MODIS_LO) )
      CASE('land','man') 
         IF(WL_Mie(1)   > 550.0d0  ) GOTO 135
         IF(WL_Mie(NAW) < 550.0d0  ) GOTO 136
      END SELECT

c     ... Check consistence of spectral ranges
c     -------------------------------------------------------------
      IF( START_WAV < WL_MIE(1) ) THEN
         CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
         WRITE(EU,'(A)') 
     $        '    Required wavelength range is outside of the'//
     $        ' wavelength grid selected for Mie calculations.'
         WRITE(EU,'(A,2x,F9.2)') 
     $        '    First wavelength for Mie calculations: ', WL_MIE(1)
         WRITE(EU,'(A,2x,F9.2)') 
     $        '    First required wavelength            : ', START_WAV
         WRITE(EU,*) '  Check file "mie_aer.inp".'
         WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
      ENDIF
      NWLS = 1 

      IF( END_WAV > WL_MIE(NAW) ) THEN
         CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
         WRITE(EU,'(A)') 
     $        '    Required wavelength range is outside of the'//
     $        ' wavelength grid selected for Mie calculations.'
         WRITE(EU,'(A,2x,F9.2)') 
     $        '    Last wavelength for Mie calculations: ', WL_MIE(NAW)
         WRITE(EU,'(A,2x,F9.2)') 
     $        '    Last required wavelength            : ', END_WAV
         WRITE(EU,*) '  Check file "mie_aer.inp".'
         WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
      ENDIF
      NWLE = NAW

c     ... Maximal number of expansion coefficients after 
c         Mie calculations
      NL_DB = MAXVAL( N_EC(:,:,:) )
c      WRITE(*,'(a,I4)') '   Maximal number of expansion coeff :', NL_DB
      N_DIM = MAX0(NL_DB,NLEGEN)
      
c     ... Allocate local arrays
      ALLOCATE( USER_MOM(N_DIM,LAN,Nm_vs,NAW,N_Dwr+1),            ! A
     $     D1(3,NAW), ECO(2,LAN,NAW,3), SCO(2,LAN,NAW,3),         ! A
     $     Hel(2,NAW), ASYM_PAR(LAN,NAW), LAS(LAN), LAE(LAN),     ! A
     $     SCS(NAW), TAU(NAW), IDA(NLAYERS), Mie_layer(NLAYERS),  ! A
     $     DENS_PRF(NLAYERS) )                                    ! A

      USER_MOM = 0.d0
      IF( PM_2i5 ) THEN
         ALLOCATE( PM_vol(2), PM_area(2),PM_mas(2) )
         PM_vol = 0.d0 ;  PM_area = 0.d0 ; PM_mas = 0.d0
         ALLOCATE( Qe(LAN,NAW), PM_tf(NAW) ) ! A
         Qe = 0.d0 ; PM_tf = 0.d0
      ENDIF
      IF( PM_2i5_MOD ) THEN
         ALLOCATE( ECO_PM(2,LAN,NAW), SCO_PM(2,LAN,NAW),
     $        Ext_PM(NLAYERS) )
         ECO_PM = 0.d0 ; SCO_PM = 0.d0 ; Ext_PM = 0.d0
      ENDIF

      IF( Plus_mi ) THEN
c     ... If "Mie" mode follows any standard aerosol model
         NAW_MI = NAW ; DIM_APH_DER = NLAYERS
         ALLOCATE( AW_MI(NAW), AC_MI(3,NLAYERS,NAW), 
     $        APH_VS_mi(NLEGEN,LAN,Nm_vs,NAW) )
         AER_WAVS => AW_MI ;  AER_COEFF => AC_MI
         APH_VS  => APH_VS_mi
      ELSE
c        ... "Mie-mode" only
         N_AER_WAVS = NAW ; DIM_APH_DER = LAN
         IF(ALLOCATED(AW_DB)) DEALLOCATE(AW_DB, AC_DB, APH_VS_db)
         ALLOCATE( AW_DB(NAW), AC_DB(3,NLAYERS,NAW),
     $        APH_VS_db(NLEGEN,LAN,Nm_vs,NAW) )
         AER_WAVS => AW_DB  ; AER_COEFF => AC_DB
         APH_VS  => APH_VS_db
      ENDIF

      N13 = 1
      Frac_c = .FALSE.
      IF( Dwr_mi ) THEN 
c        ... Check for which PSD parameters WFs should be calculated
         Modr_f = CHK_STAT('aer-modr_f', AER_INFO%NAME,AER_INFO%WF_FLG )
         Modr_c = CHK_STAT('aer-modr_c', AER_INFO%NAME,AER_INFO%WF_FLG )
         Var_f  = CHK_STAT('aer-var_f',  AER_INFO%NAME,AER_INFO%WF_FLG )
         Var_c  = CHK_STAT('aer-var_c',  AER_INFO%NAME,AER_INFO%WF_FLG )
         Frac_c = CHK_STAT('aer-cm_frac',AER_INFO%NAME,AER_INFO%WF_FLG )
c        ... If derivative with respect to the modal radius is used
         IF( USE_Der(1) ) N13 = N13 + 1
c        ... If derivative with respect to the variance is used
         IF( USE_Der(2) ) N13 = N13 + 1
         ALLOCATE( AER_DER(2,NLAYERS,NAW,N_dwr), DUMA(LAN), ! A
     $        APH_DER(NLEGEN,DIM_APH_DER,Nm_vs,NAW,N_Dwr)) ! A
c         WRITE(*,*) '   -----> Set  AER_DER <---- '
c         WRITE(*,*) '   NAW, N_dwr ', NAW,N_dwr
         AER_DER = 0.d0  ;  APH_DER = 0.d0  
c         WRITE(*,*) '   -0', AER_DER(1,:,1,1)
      ENDIF  ! Dwr_mi

      N_01 = 0
      IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG)) N_01 = 1

      N_AWF = COUNT( AER_INFO%WF_FLG )
c                WRITE(*,'(a,I2)') '   Number of aerosol WF ', N_AWF
      ALLOCATE( AP_Mie(NLAYERS,N_AWF), AOD_Mie(2,NAW) ) 
      AP_Mie = 0.d0

      AER_WAVS(:)  = WL_MIE(1:NAW) 
      AER_PHASMOMS => APH_VS( :,:,1,: )
      AER_PHASMOMS = 0.d0 ; AER_COEFF    = 0.d0 

C     =====================================================
C        Interpolate input profile AER_PRF to "NLAYERS" layers
C     =====================================================
C       AER_PRF( 1:N_AER_LEV ) contains the aerosol input profile
C       ( AER_PRF(1) corresponds to the earth surface ).
C       DENS_PRF(1:NLAYERS) contains the aerosol profile 
C       interpolated to the forward model altitude grid 
C     ( DENS_PRF(NLAYERS) corresponds to the earth surface ).
      CALL LIN_INT(N_AER_LEV, AER_LEV, AER_PRF, 
     $     NLAYERS, ZHOLD, DENS_PRF, 'Z')
c
c ****************** Start of aerosol layering *********************
c
c     ... LTop(:) - upper boundary of aerosol layers ( bottom --> top)
      IF( LAN > 1 ) THEN
         IF( LTop(LAN-1) > ZHOLD(1) ) GOTO 125
      ELSE
         IF( LTop(LAN) > ZHOLD(1) ) GOTO 127
      ENDIF
      IF( LTop(1) < ZHOLD(NLAYERS) ) GOTO 126

c     DO I = 1,LAN
c        ... From the top atmosphere to the bottom
c        DUM = LTop(LAN-I+1)
c        LAS(I) = MINLOC( DABS(DUM-ZHOLD(:)),1 )
c        write(*,*) i, las(i), dum, zhold(las(i))
c     ENDDO
c     DO I = 1,LAN-1
c        LAE(I) = LAS(I+1)-1
c     ENDDO
c     LAE(LAN) = MINLOC( DABS(Z_BOT-ZHOLD(:)),1 )

      LAE(1) = MINLOC( DABS(Z_BOT-ZHOLD(:)),1 )
      DO I = 1,LAN
c        ... From the top atmosphere to the bottom
         LAS(I) = MINLOC( DABS(LTop(I)-ZHOLD(:)),1 )
c         write(*,*) i, las(i), LTop(I),zhold(las(I))
      ENDDO

      DO I = 2,LAN
         LAE(I) = LAS(I-1)-1
      ENDDO

c     ... Indexes of altitude grid levels which are above of highest
c         aerosol layers
      IDA(:) = LAN ; Mie_layer(:) = .FALSE.
c      IF( LAN > 1 ) THEN
c     ... Indexes of altitude grid levels which are within aerosol layers
      DO I = 1,LAN
         DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAE(I))
         DO J = 1,NLAYERS
            IF( ZHOLD(J) <= DD1.AND.ZHOLD(J) >= DD2 ) THEN 
               IDA(J) = I ;  Mie_layer(J) = .TRUE.
            ENDIF
         ENDDO
      ENDDO
c      ENDIF
      DD1 = ZHOLD(MAXVAL(LAE))
c     ... Indexes of altitude grid levels which are below of lowest
c         aerosol layer
      DO J = 1,NLAYERS
         IF( ZHOLD(J).LE.DD1 ) IDA(J) = 1
      ENDDO

      IF(LOG_LEVEL(1) >= N_DSO) THEN
         WRITE(LUNIT,*) 
     $        ' ------ Test of aerosol layering  -------- '
         DO I = 1,LAN
            WRITE(LUNIT,'(2x,I3,2(1x,f7.3))')
     $           I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)) 
         ENDDO
         WRITE(LUNIT,*) 
     $        ' ----------------------------------------- '
         WRITE(*,*) '  Aerosol layer index "Mie-layer" parameter'
         DO I = 1, NLAYERS
            WRITE(*,'(2x,I3,2x,F8.3,2x,I2,2x,L)') 
     $           I, ZHOLD(I), IDA(I), Mie_layer(I)
         ENDDO
      ENDIF
c
c ****************** End of aerosol layering *********************
c
c      WRITE(*,*) '   DB_EXT(K1,J,K,1) :  '
c      DO I = 1,LAN
c         K1 = LAN - I + 1
c         DO J = 1, NAC(K1)
c            WRITE(*,'(1x,2(1x,I3),5(1x,E12.4))') I, J, 
c     $           (DB_EXT(K1,J,K,1), K=1,NAW)
c         ENDDO
c      ENDDO

c     ... Extinction, scattering coefficients, and derivatives  
c         of a given aerosol component mixture (aerosol type) 
c         will be calculated from the top of atmosphere to the bottom
c     ------------------------------------------------------------------
      ECO = 0.d0 ; SCO = 0.d0
      MCO = MAXVAL( NAC(:) )
c         WRITE(*,'(a)')    '    ECO and SCO '
c         WRITE(*,'(a,I2)') '    Max number of aerosol component : ', MCO
c         WRITE(*,'(A,10(1x,e12.4))') '      RD :', RD(1,1:Mco)
c         WRITE(*,'(A, 1(1x,e12.4))') ' SUM(RD) :', SUM( RD(1,1:Mco) )
      DO I = 1,LAN
         K1 = I
c        ... In the case of MODIS_AP Cfr is defined in "read_mod_aer_inp.f"
         IF( .NOT.MODIS_AP ) THEN
            Cfr = SUM( RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
c                   WRITE(*,*) F_MODE(1:Mco)
c                   WRITE(*,*) '   Cfr = ', Cfr
         ENDIF

c     **************** Start 1 PM_2i5 **************************************
         IF( PM_2i5 ) THEN
            VOL = SUM( Vo_PM(K1,:)*RD(K1,1:Mco),F_MODE(1:Mco) )
c                WRITE(*,'(2x,*(L))') F_MODE(1:Mco)
c                WRITE(*,'(2x,*(1x,E12.4))') RD(K1,1:Mco)
c                WRITE(*,'(2x,*(1x,E12.4))') Ro_dens            
            PM_mas(1)  = SUM( V_c*RD(K1,1:Mco)*Ro_dens,F_MODE(1:Mco) )
            PM_mas(2)  = SUM( V_f*RD(K1,1:Mco)*Ro_dens,F_MODE(1:Mco) )
            PM_vol(1)  = SUM( V_c*RD(K1,1:Mco),F_MODE(1:Mco) )
            PM_vol(2)  = SUM( V_f*RD(K1,1:Mco),F_MODE(1:Mco) )
            PM_area(1) = SUM( A_c*RD(K1,1:Mco),F_MODE(1:Mco) )
            PM_area(2) = SUM( A_f*RD(K1,1:Mco),F_MODE(1:Mco) )
         ENDIF
         IF( PM_2i5_MOD ) THEN
            FORALL( Kw = 1:NAW ) 
              ECO_PM(1,I,Kw) = 
     $           SUM( DBe_PM(K1,:,Kw)*RD(K1,1:Mco),F_MODE(1:Mco) )
              SCO_PM(1,I,Kw) = 
     $           SUM( DBs_PM(K1,:,Kw)*RD(K1,1:Mco),F_MODE(1:Mco) )
c             ... over coarse mode if any
              ECO_PM(2,I,Kw) = SUM
     $           ( DBe_PM(K1,:,Kw)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
              SCO_PM(2,I,Kw) = SUM
     $           ( DBs_PM(K1,:,Kw)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
            END FORALL
         ENDIF
c **************** End 1 PM_2i5 **************************************

c        ... Coefficients and derivatives if any
         DO Id = 1, N13
c           ... Sum up contribution of aerosol components:
c               over fine mode
            FORALL( Kw=1:NAW ) 
              SCO(1,I,Kw,Id) = 
     $           SUM( DB_SCA(K1,:,Kw,Id)*RD(K1,1:Mco),F_MODE(1:Mco) )
              ECO(1,I,Kw,Id) = 
     $           SUM( DB_EXT(K1,:,Kw,Id)*RD(K1,1:Mco),F_MODE(1:Mco) )
c                over coarse mode if any
              SCO(2,I,Kw,Id) = SUM
     $           ( DB_SCA(K1,:,Kw,Id)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
              ECO(2,I,Kw,Id) = SUM
     $           ( DB_EXT(K1,:,Kw,Id)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
              END FORALL
c           ------------------------------------------------------------
                   IF( Cfr > 0.d0 ) THEN
                      WRITE(*,*) '      Ratio Fine_mode/Coarse_mode :'
                      DO K = 1,NAW
                         WRITE(*,'(2x,F10.3,2(2x,E12.4))') 
     $                        WL_MIE(K), ECO(1,I,K,Id)/ECO(2,I,K,Id)
                      ENDDO
                   ENDIF
c           ------------------------------------------------------------
         ENDDO  ! over derivatives (Id)    
         IF( Bi_PSD ) THEN
            WRITE(*,*) '    Corse mode fraction  :'
            WRITE(*,'(2x,A,2x,I3,2(2x,E12.4))') 'layer', I, Cfr
         ENDIF
      ENDDO  ! over aerosol layers (I)

c     ... Scale aerosol number density using extinction coefficient
c         at a reference wavelength
c     ------------------------------------------------------------------
      SELECT CASE (S_N_D)
      CASE ('SCIA_3D', 'FILE')
         Call Replace_Aerosol_ND (LAN, LAS, LAE, NAW, AER_WAVS, ECO,
     $        Dens_Prf, *880)
      END SELECT
c     --------- End scaling of aerosol number density ------------------

c     ... Multiply ECO and SCO with particle number density profile:
      DO I = 1,LAN
c               WRITE(*,*) '   Compare ECO I = ', I
c               WRITE(*,*) ECO(1,I,1,1), ECO(2,I,1,1)
c               WRITE(*,*) ECO_PM(1,I,1), ECO_PM(2,I,1)
         Hel(1,:) = ECO(1,I,:,1) + ECO(2,I,:,1)
         Hel(2,:) = SCO(1,I,:,1) + SCO(2,I,:,1)
         DO J = LAS(I),LAE(I)
            AER_COEFF(EXI,J,:) = Hel(1,:)*DENS_PRF(J)
            AER_COEFF(SCI,J,:) = Hel(2,:)*DENS_PRF(J)
         ENDDO  ! over standard altitude levels
      ENDDO  ! over aerosol layers (I)

!      do J = 1, Nlayers
!         write(6,*) ZHOLD(J), DENS_PRF(J), AER_COEFF(EXI,J,4)
!      end do
!      pause
      
      IF( L_scal ) THEN
c        ... Scaling of number density between ZS1 - ZS2
         DO I = 1, NAW
            WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
               AER_COEFF(SCI,:,I) = AER_FACT*AER_COEFF(SCI,:,I) 
               AER_COEFF(EXI,:,I) = AER_FACT*AER_COEFF(EXI,:,I) 
            END WHERE
         ENDDO
c        ... Scaling of the scattering coefficient between ZS1 - ZS2
c            in order to change the single scattering albedo as follows:
c               SSA' = 1 - V_PAP*(1 - SSA) 
c            Extinction coefficient remains the same
         DO I = 1, NAW
            WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) 
               AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) 
     $              - (V_PAP - 1.d0)
     $              *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
            END WHERE
         ENDDO
      ENDIF   ! scaling in selected altitude range

      IF( PM_2i5 ) THEN
         OPEN(21, FILE='./DATA_OUT/PM_2i5-Mie.dat' )
         WRITE(21,'(A)') '#    First column - humidity [%]'
         WRITE(21,'(A)')
     $        '#    Second column - extinction cross-section [cm^2]'
         WRITE(21,'(A)')
     $        '#    3 and 4 columns - cutted and full mass [g]'
         WRITE(21,'(A)')
     $        '#    5 and 6 columns - cutted and full volume [cm^3]'
         WRITE(21,'(A)')
     $        '#    7 and 8 columns - cutted and full area [cm^2]'

         WRITE(21,'(2x,F8.3, *(2x,ES15.7))') 
     $        REL_HUM(1), 
     $        1.d-05*AER_COEFF(EXI,NLAYERS,1)/DENS_PRF(NLAYERS),
     $        PM_mas(1)*1.d-12, PM_mas(2)*1.d-12,
     $        PM_vol(1)*1.d-12, PM_vol(2)*1.d-12,
     $        PM_area(1)*1.d-8, PM_area(2)*1.d-8
         CLOSE(21)
c           ... aerosol particle mass density [g/cm^3]
         Rod = 1.25d0
         PM_tf(:) = Rod*Vol/Qe(1,:)*AER_COEFF(EXI,NLAYERS,:)
c                [Vol/Qe] = mkm ;  [AER_COEFF] = km-1
c                convert micron into km
     $        *1.d-9
c                {PM} = g/cm^3 ---> mkg/cm^3
         PM_tf(:) = PM_tf(:)*1.D+06
         WRITE(*,*) ' Vol       =  ', Vol
         WRITE(*,*) '  Qe       =  ', Qe(1,1)
         WRITE(*,*) ' AER_COEFF =  ', AER_COEFF(EXI,NLAYERS,1)
         STOP '  ------------ PM_2i5 Mie mode -------------- '
      ENDIF

!      WRITE(*,*) '     From "mie_aer_setup.f"'
!      WRITE(*,*) '     Extinction cross-section and '
!      WRITE(*,*) '       single scattering albedo:  '
!      DO I = 1, NAW
!         WRITE(*,'(2x,F8.3,2x,e15.7,2x,F7.4)') WL_MIE(I),
!     $        Hel(1,I), Hel(2,I)/Hel(1,I)
!      ENDDO
cc     ... Output of single-scattering albedo for 2 first wavelengths
c      IF( MODIS_AP.AND.(.NOT.PM_2i5) ) THEN
c         Ome1 = Hel(2,1)/Hel(1,1) ; Ome2 = Hel(2,2)/Hel(1,2)
c      ENDIF

c     ... Scale DENS_PROF to match T_550 in the case of options
c         'land' and 'man' (not for 'ocean')
      IF( MODIS_AP.AND.TRIM(MODIS_LO) /= 'ocean' ) THEN
         Ang_INTERP = .TRUE.
         DO K = 1,NAW
            TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
            IF( WL_Mie(K) == 550.0d0 ) THEN
               Ang_INTERP = .FALSE. ; DD1 = TAU(K)
            ENDIF
         ENDDO
         IF( NAW == 1 ) DD1 = TAU(1)         
         IF( .NOT.PM_2i5_MOD.AND.NAW>1.AND.Ang_INTERP ) THEN            
            CALL Angstrom_interp(NAW, WL_Mie, TAU, 550.d0, DD1)
         ENDIF
c        ===============================================================
         IF( PM_2i5_MOD ) THEN
            I = 1
            Tau_c = SUM( AER_COEFF(EXI,:,1)*DZ(:) )/2.d0 ! Coarse mode         
            Ome2 = AER_COEFF(SCI,NLAYERS,1)/AER_COEFF(EXI,NLAYERS,1)

            Hel(1,1) = (ECO_PM(1,I,1) + ECO_PM(2,I,1)) ! for 1 WL only
            Hel(2,1) = (SCO_PM(1,I,1) + SCO_PM(2,I,1)) ! for 1 WL only
            Ome1 = Hel(2,1)/Hel(1,1) ! SSA fine
            DO J = LAS(1),LAE(1)
               Ext_PM(J) = Hel(1,1)*DENS_PRF(J) 
            ENDDO
            Tau_f = SUM( Ext_PM(:)*DZ(:) )/2.d0 ! Fine mode
            Sfac = T_550/(Tau_f + Tau_c )

            WRITE(*,*) '  SSA fine   ', Ome1
            WRITE(*,*) '  SSA coarse ', Ome2
            write(*,*) '  ------------------------------------------- '
            WRITE(*,*) '  Tau before scaling fine   ', Tau_f
            WRITE(*,*) '  Tau before scaling coarse ', Tau_c
            WRITE(*,*) '  Tau before scaling sum    ', Tau_f + Tau_c
            write(*,*) '  ------------------------------------------- '
            WRITE(*,*) '  Tau after scaling fine   ', Tau_f*Sfac
            WRITE(*,*) '  Tau after scaling coarse ', Tau_c*Sfac
            WRITE(*,*) '  Tau after scaling sum    ', (Tau_f+Tau_c)*Sfac

            OPEN(21, FILE='./DATA_OUT/PM_2i5_MOD.dat' )
              WRITE(21,'(A)') '#  C1 - AOT fine mode ( < 0.5 micron)'
              WRITE(21,'(A)') '#  C2 - AOT coarse mode ( > 0.5 micron)'
              WRITE(21,'(A)') '#  C3 - AOT sum '
              WRITE(21,'(A)') '#  C4 - AAOT fine mode ( < 0.5 micron)'
              WRITE(21,'(A)') '#  C5 - AAOT coarse mode ( > 0.5 micron)'
              WRITE(21,'(A)') '#  C6 - AAOT sum '
              DO I = 1, NAW
                 WRITE(21,'(*(1x,ES12.4))') 
     $                Tau_f*Sfac, Tau_c*Sfac, (Tau_f + Tau_c)*Sfac,
     $                Tau_f*Sfac*(1.d0 - Ome1),
     $                Tau_c*Sfac*(1.d0 - Ome2),
     $                Tau_f*Sfac*(1.d0-Ome1) + Tau_c*Sfac*(1.d0-Ome2)
              ENDDO
            CLOSE(21)
            WRITE(*,*) '   Output in "./DATA_OUT/PM_2i5_MOD.dat" '
            STOP '  Stop in "mie_aer_setup.f" option PM_2i5_MOD '
         ENDIF  ! option PM_2i5_MOD 
c        ===============================================================

c        ... Scale the number density profile to match desired optical
c            thickness at 550 nm
c         WRITE(*,*)  '    -------------->  T_550 = ', T_550, DD1
         Sfac = T_550/DD1
         DENS_PRF(:) = Sfac*DENS_PRF(:)
         DO J = LAS(1),LAE(1)
            AER_COEFF(EXI,J,:) = Hel(1,:)*DENS_PRF(J)
            AER_COEFF(SCI,J,:) = Hel(2,:)*DENS_PRF(J)
         ENDDO
         DO K = 1,NAW
            TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
         ENDDO
      ENDIF  ! MODIS_AP option but not 'ocean'

c     ... Reading of single scattering albedo from SAMUM data
      IF( SAMUM ) THEN
         KUNIT = 47
         OPEN( KUNIT,FILE='./SAMUM/SAMUM-SSA.dat',STATUS='OLD' )
            CALL SKIP_HEADER( KUNIT )
            READ(KUNIT,*) N_SAM
            ALLOCATE( WL_SAM(N_SAM), SSA_SAM(N_SAM), SSA_H1(NAW) )
            DO I = 1, N_SAM
               READ(KUNIT,*) WL_SAM(I), SSA_SAM(I)
            ENDDO
            CALL LIN_INT(N_SAM, WL_SAM, SSA_SAM, 
     $           NAW, AER_WAVS, SSA_H1, 'C')
         CLOSE(KUNIT)
c        ... Calculate scattering coefficient using SAMUM SSA
c         WRITE(*,*) '   Enter Recalc_SCA '
c         READ(*,*) Recalc_SCA
c         IF( Recalc_SCA ) THEN
c            DO K = 1,NAW
c               AER_COEFF(SCI,:,K) = AER_COEFF(EXI,:,K)*SSA_H1(K)
c            ENDDO
c         ELSE
            DO K = 1,NAW
               AER_COEFF(EXI,:,K) = AER_COEFF(SCI,:,K)/SSA_H1(K)
            ENDDO
c         ENDIF
         DEALLOCATE( WL_SAM, SSA_SAM, SSA_H1 )
      ENDIF  ! SAMUM

c     ... The same for derivatives if any
      IF( Dwr_mi ) THEN
         AER_DER(1,:,1,1) = 0.d0
         DO I = 1,LAN
            DO J = LAS(I),LAE(I)
               IF( Bi_PSD ) THEN
c                 ... Bimodal particle size distribution:
c                 ---------------------------------------
                  I0 = 1 ; Id = 2 
                  IF( USE_Der(1) ) THEN
c                    ... Dsca/Dmod and Dexa/Dmod fine mode
                     IF( Modr_f ) THEN                  
                        AER_DER(SCI,J,:,I0) = SCO(1,I,:,Id)*DENS_PRF(J)
                        AER_DER(EXI,J,:,I0) = ECO(1,I,:,Id)*DENS_PRF(J)
                        I0 = I0 + 1
                     ENDIF
c                    ... Dsca/Dmod and Dexa/Dmod coarse mode
                     IF( Modr_c ) THEN                  
                        AER_DER(SCI,J,:,I0) = SCO(2,I,:,Id)*DENS_PRF(J)
                        AER_DER(EXI,J,:,I0) = ECO(2,I,:,Id)*DENS_PRF(J)
                        I0 = I0 + 1
                     ENDIF
                     Id = Id + 1
                  ENDIF
                  IF( USE_Der(2) ) THEN
c                    ... Dsca/Dvar and Dexa/Dvar fine mode
                     IF( Var_f ) THEN                  
                        AER_DER(SCI,J,:,I0) = SCO(1,I,:,Id)*DENS_PRF(J)
                        AER_DER(EXI,J,:,I0) = ECO(1,I,:,Id)*DENS_PRF(J)
                        I0 = I0 + 1
                     ENDIF
c                    ... Dsca/Dvar and Dexa/Dvar coarse mode
                     IF( Var_c ) THEN                  
                        AER_DER(SCI,J,:,I0) = SCO(2,I,:,Id)*DENS_PRF(J)
                        AER_DER(EXI,J,:,I0) = ECO(2,I,:,Id)*DENS_PRF(J)
                        I0 = I0 + 1
                     ENDIF
                  ENDIF
c                 ... Dsca/DCf and Dexa/DCf w.r.t. fraction of coarse mode
c                     ----------------------------------------------------
                  IF( Frac_c ) THEN                  
                     IF( Cfr > 0.d0.AND.Cfr < 1.d0 ) THEN
                        AER_DER(SCI,J,:,I0) = (-SCO(1,I,:,1)/(1.d0-Cfr)
     $                       + SCO(2,I,:,1)/Cfr)*DENS_PRF(J)
                        AER_DER(EXI,J,:,I0) =  (-ECO(1,I,:,1)/(1.d0-Cfr) 
     $                       + ECO(2,I,:,1)/Cfr)*DENS_PRF(J)
                     ELSE
                        WRITE(*,*)  '    Cfr =  ', Cfr
                        WRITE(*,*)  ' Degeneration of bi-model PSD '
                        WRITE(*,*)  '     "mie_aer_setup.f"        '
                        STOP        '   !!! Cfr is 0 or 1 !!!  '
                     ENDIF
                  ENDIF
               ELSE
c                ... Mono-modal PSD case
                  FORALL( K=1:NAW ) 
                  AER_DER(SCI,J,K,:)= SCO(1,I,K,2:1+N_Dwr)*DENS_PRF(J)
                  AER_DER(EXI,J,K,:)= ECO(1,I,K,2:1+N_Dwr)*DENS_PRF(J)
                  END FORALL
               ENDIF
            ENDDO  ! over main altitude grid (J)
         ENDDO  ! over aerosol layers (I)
      ENDIF

c     ... Calculate profiles of expansion coefficients
c         and asymmetry parameter selected aerosol types
c         for mono-modal PSD
      IF( .NOT.Bi_PSD ) THEN
         DO I = 1,LAN 
c            K1 = LAN - I + 1
            K1 = I
            DO L = 1,NL_DB  
               DO K = 1,Nm_vs
c                 ... Sum up contribution of aerosol components:
c                     1 - into phase matrix 
                  CALL SUM_MOMS(1, 1, F_MODE, Hel(1,:))
                  USER_MOM(L,I,K,:,1) = Hel(1,:)/SCO(1,I,:,1)
c                     2 - into derivatives (if any)
                  DO I1 = 2,1 + N_dwr
                     CALL SUM_MOMS(1, I1, F_MODE, D1(1,:))
                     CALL SUM_MOMS(I1, 1, F_MODE, D1(2,:))
                     USER_MOM(L,I,K,:,I1) = 
     $                    ( -SCO(1,I,:,I1)*USER_MOM(L,I,K,:,1)
     $                    + D1(1,:) +  D1(2,:) )/SCO(1,I,:,1)
                  ENDDO
               ENDDO  ! over Greek symbols (K)
            ENDDO  ! over expansion coefficients (L)
            ASYM_PAR(I,:) = USER_MOM(2,I,1,:,1)/3.d0
c                WRITE(*,*) '              N_dwr = ', N_dwr
c                WRITE(*,*) '   User_MOM Test Id = ', Id
c                WRITE(*,'(A,1x,I3,2(2x,e12.4))') 'Layer - ', 
c     $               I, SUM( USER_MOM(1:NL_DB,I,:,1,Id) ), 
c     $               SUM( USER_MOM(1:NL_DB,I,:,2,Id) )
         ENDDO                  ! over aerosol layers (I)
      ELSE
c        ... Case of bimodal PSD
c        -----------------------
         DO I = 1,LAN 
c            K1 = LAN - I + 1
            K1 = I 
            DO L = 1,NL_DB  
               DO K = 1,Nm_vs
                 CALL SUM_MOMS(1, 1,  F_MODE,     Hel(1,:))
                 CALL SUM_MOMS(1, 1, .NOT.F_MODE, Hel(2,:))
                  SCS(:) = SCO(1,I,:,1) + SCO(2,I,:,1)
c                 ... Weighted phase matrix
                  USER_MOM(L,I,K,:,1) = (Hel(1,:) + Hel(2,:))/SCS(:)

c                 ... Cycle if no derivatives
                  IF( N_dwr == 0 ) CYCLE

                  I0 = 2 ; Id = 2 
                  IF( USE_Der(1) ) THEN
c                    ... Dcoef/Dmod fine mode (fm)
                     IF( Modr_f ) THEN
c                       ... Hel(1,:) - Phm*Dscat/Dmod 
                        CALL SUM_MOMS(1, I0, F_MODE, Hel(1,:))
c                       ... Hel(2,:) - DPhm/Dmod*scat 
                        CALL SUM_MOMS(I0, 1, F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(1,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
c                    ... Dcoef/Dmod coarse mode
                     IF( Modr_c ) THEN
                        CALL SUM_MOMS(1, I0, .NOT.F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, .NOT.F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(2,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
                     I0 = I0 + 1
                  ENDIF
                  IF( USE_Der(2) ) THEN
c                    ... Dcoef/Dvar fine mode
                     IF( Var_f ) THEN
                        CALL SUM_MOMS(1, I0, F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(1,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
c                    ... Dcoef/Dvar coarse mode
                     IF( Var_c ) THEN
                        CALL SUM_MOMS(1, I0, .NOT.F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, .NOT.F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(2,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
                  ENDIF
c                 ... Dcoef/DCf  w.r.t. fraction of coarse mode
                  IF( Frac_c ) THEN
c                    ... Hel(1,:) - phm_f*sca_f
                     FORALL( Kw=1:NAW ) Hel(1,Kw) = 
     $                    SUM( DB_MOMS(K1,:,Kw,K,L,1)*DB_SCA(K1,:,Kw,1), 
     $                    F_MODE(1:Mco) )
c                    ... Hel(2,:) - phm_c*sca_c
                     FORALL( Kw=1:NAW ) Hel(2,Kw) = 
     $                    SUM( DB_MOMS(K1,:,Kw,K,L,1)*DB_SCA(K1,:,Kw,1),
     $                    .NOT.F_MODE(1:Mco) )
                     FORALL( Kw=1:NAW ) D1(1,Kw) = 
     $                    SUM( DB_SCA(K1,:,Kw,1), F_MODE(1:Mco) )
                     FORALL( Kw=1:NAW ) D1(2,Kw) = 
     $                    SUM( DB_SCA(K1,:,Kw,1), .NOT.F_MODE(1:Mco) )
                     USER_MOM(L,I,K,:,Id) =  
     $                    ( (D1(1,:)-D1(2,:))*USER_MOM(L,I,K,:,1)
     $                    - Hel(1,:) +  Hel(2,:) )/SCS(:)
                  ENDIF
               ENDDO  ! over Greek symbols (K)
            ENDDO  ! over expansion coefficients (L)
            ASYM_PAR(I,:) = USER_MOM(2,I,1,:,1)/3.d0
         ENDDO  ! over aerosol layers (I)
      ENDIF

c     ... Reading phase function from SAMUM data
c     ------------------------------------------------------------------
      IF( SAMUM_PH ) THEN
c        ... Reading ratio for 13 MERIS wavelengths 
c         OPEN(KUNIT,FILE='./SAMUM/ratio-N_300.dat',STATUS='OLD')
c         OPEN(KUNIT,FILE='./SAMUM/ratio-N_300_OPAC.dat',STATUS='OLD')
         OPEN(KUNIT,FILE='./SAMUM/ratio-N_300-2_OPAC.dat',STATUS='OLD')
           CALL SKIP_HEADER( KUNIT )
           READ(KUNIT,*) Na_st
           ALLOCATE( Ratio(Na_st,NAW) )
           DO I = 1, Na_st
              READ(KUNIT,*) DUM, (Ratio(I,K),K=1,NAW)
           ENDDO
         CLOSE(KUNIT)
         
         Ph_const = .FALSE.
c         WRITE(*,*) '   Enter Ph_const '
c         READ(*,*) Ph_const
         IF( Ph_const ) Ratio = 1.d0

         OPEN(KUNIT,FILE='./SAMUM/SAMUM-PhaseFunction.dat',STATUS='OLD')
           CALL SKIP_HEADER( KUNIT )
           READ(KUNIT,*) Na_st
           N_gau = 1500
           ALLOCATE( Sa_inp(Na_st), Phs_inp(Na_st,1,NAW), 
     $          M_COF(1,NAW), Leg_cof(N_gau,1,NAW), Phs_0(Na_st) )
           DO I = 1, Na_st
              READ(KUNIT,*) Sa_inp(I), Phs_0(I)
           ENDDO
         CLOSE(KUNIT)

         DO I = 1,Na_st
            Phs_inp(I,1,:) = Phs_0(I)*Ratio(I,:)
         ENDDO

         CALL LEG_COEFF(1, NAW, N_gau, Na_st, Sa_inp, Phs_inp, 
     $        M_cof, Leg_cof, *880)

         NL_DB = MAXVAL( M_cof(1,:) )
         IF( MAXVAL(M_COF(1,:)) > N_DIM ) THEN
            DEALLOCATE( USER_MOM )
            ALLOCATE( USER_MOM(NL_DB,LAN,1,NAW,1) )
         ENDIF
c         WRITE(*,'(A,13(2x,I4))') 
c     $        '   Number of expansion coef. (SAMUM) ', M_cof(1,:)
          USER_MOM = 0.d0 
          DO K = 1, NAW
             DO I = 1, M_cof(1,K)
                USER_MOM(I,:,1,K,1) = Leg_cof(I,1,K)
             ENDDO   ! over Legendre coefficients (I)
          ENDDO   ! over wavelengths (K)
          DO K = 1, NAW
             ASYM_PAR(:,K) = USER_MOM(2,:,1,K,1)/3.d0
          ENDDO
       ENDIF  ! reading phase function from SAMUM data
c     ------------------------------------------------------------------

c      WRITE(*,'(A,13(1x,F8.3))') 
c     $     '  Asymmetry factor : ', ASYM_PAR(1,:)
c      IF( MODIS_AP.AND.TRIM(MODIS_LO)=='land'  ) THEN
cc        ... Output of single-scattering albedo for 2 first wavelengths
c         WRITE(*,*) '   ********  Test output  ********    '
c         OPEN( 49,file='./DATA_OUT/Aer-Opt.par' )
c        WRITE(49,'(2x,F6.3,2(2x,F7.4,1x,F8.3))') T_550,          
c    $        Ome1, ASYM_PAR(1,1), Ome2, ASYM_PAR(1,2)
c         CLOSE(49)
c      ENDIF

cc *******************************************************************
cc     ---- Below writing of scattering matrix
cc     ... Write out phase matrix for selected basic components
c      OPEN(44,file='./DATA_OUT/out_phf.dat')
c      WRITE(44,'(2x,I5)') N_DIM
c      DO I = 1, N_DIM
c         WRITE(44,'(100(1x,e15.7))') (USER_MOM(I,1,1,K,1), K=1,NAW)
c      ENDDO
c      CLOSE(44)
cc *******************************************************************
cc     ----- End writing of scattering matrix

c     ... Single scattering correction algorithm ->
c              save all expansion coefficients
       IF( DO_SSRCA .OR. TRIM(RTM_TYPE)=='spher_scat'
     $      .OR.SCATT_MODE =='ss' ) THEN
          IF( Plus_mi ) THEN
            Nga_mi = NL_DB
            IF(ALLOCATED(Greek_aer_mi)) DEALLOCATE(Greek_aer_mi)
            ALLOCATE( Greek_aer_mi( Nga_mi,LAN,Nm_vs,NAW ) )
            DO I = 1,NAW 
               Greek_aer_mi(:,:,:,I) = USER_MOM(1:Nga_mi,:,:,I,1)
            ENDDO
            NG_aer = MAX( Nga_db,Nga_mi )
         ELSE
            NG_aer = NL_DB
            IF(ALLOCATED(Greek_aer_db)) DEALLOCATE(Greek_aer_db)
            ALLOCATE( Greek_aer_db( NG_aer,LAN,Nm_vs,NAW ) )
            DO I = 1,NAW 
               Greek_aer_db(:,:,:,I) = USER_MOM(1:NG_aer,:,:,I,1)
            ENDDO
         ENDIF
c        ... Save derivatives of all expansion coefficients
         IF( Dwr_mi ) THEN
            ALLOCATE( D_Greek( NG_aer,LAN,Nm_vs,NAW,N_dwr ) )            
c            WRITE(*,*) '  mie_aer_setup: NL_DB = ', NL_DB
            D_Greek = 0.d0
            DO I = 1,NAW 
               DO Idr = 1, N_dwr
                  D_Greek(1:NL_DB,:,:,I,Idr) = 
     $                 USER_MOM(1:NL_DB,:,:,I,1+Idr)
               ENDDO            ! over derivatives (Idr)
            ENDDO               ! over reference wavelength (I)
         ENDIF
      ENDIF  ! --> single scattering correction algorithm or 'spher_scat'

      IF( OUT_A_SCM ) THEN 
         IF( LAY_A > LAN ) GOTO 130
         IF( WEL_A > NAW )  GOTO 131
         CALL COEF_TO_SCATMATR(NL_DB, Nm_vs,  
     $        USER_MOM(1:NL_DB,LAY_A,1:Nm_vs,WEL_A,1),
     $        NSCA_A, GE_A, PATH_TO_SCM)
      ENDIF

      DO I = 1,LAN 
         DO J = LAS(I),LAE(I)
            AER_COEFF(ASI,J,:) = ASYM_PAR(I,:)
         ENDDO
      ENDDO

c        ... Save aerosol number density profile
      I0 = 1
      IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
         AP_Mie(:,I0) = DENS_PRF(:) ; I0 = I0 + 1
         CALL SET_PTR( RTM_PTR, 'aer-dens' )
         RTM_PTR%APR(:) = DENS_PRF(:)
         RTM_PTR%CRN(:) = DENS_PRF(:)    
      ENDIF

      IF( Dwr_mi ) THEN
         IF( Bi_PSD ) THEN
c           ... Number of all components 
            Nsm = NAC(1)
c           ... Number of components in the fine and coarse mode
            Nfm = COUNT( F_mode(1:Nsm) ) ; Ncm = Nsm - Nfm
            ALLOCATE( PSD_P(Nsm) )
         ENDIF

!         WRITE(*,*) '   Option bimodal  ', Bi_PSD 

         IF( .NOT.Bi_PSD ) THEN
c           ... The option "mono-modal" is selected
c           =======================================
            IF(CHK_STAT('aer-modr_smod', AER_INFO%NAME, 
     $           AER_INFO%WF_FLG) ) THEN
c               WRITE(*,*) '   I0 for AP_MIE : ', I0
               DO I = 1,LAN ; DO J = LAS(I),LAE(I)
                  AP_Mie(J,I0) = MOD_R(I,1)
               ENDDO ; ENDDO
               DO I = 1, LAN
                  DO K = 1, NAC(I)
                     CALL SET_PTR( RTM_PTR,'aer-modr_smod', ISSP=K )
c                     WRITE(*,*) '   Use "SET_PTR":  '
c                     write(*,*) LAS(I), LAE(I)
                     DO J = LAS(I),LAE(I)
                        RTM_PTR%APR(J) = MOD_R(I,K)
                        RTM_PTR%CRN(J) = RTM_PTR%APR(J)
c                        write(*,*)RTM_PTR%APR(J),RTM_PTR%GRD(J)
c                        WRITE(*,*) I, K, J, RTM_PTR%APR(J) 
                     ENDDO
                  ENDDO
               ENDDO
c               write(*,*) MOD_R(:,1)
c               pause
c              ---------------------------------------------------------
c               WRITE(*,*) '  The option "mono-modal" PSD '
c               WRITE(*,*) 
c     $              '  Aerosol number density and other PSD parameters'
c               II = FIND_IDX ('aer-modr_smod', RTM_ST%NAME) 
c               WRITE(*,*) '  Number of "aer-modr_smod" = ', II
c               DO I = 1, NLAYERS
c                  IF( AP_Mie(I,1)==0.d0 ) CYCLE
c                  WRITE(*,'(2x,I3,2x,F8.3,6(2x,E12.4))') 
c     $                 I, ZHOLD(I), AP_Mie(I,1), 
c     $                 (RTM_ST(II)%SPR(K,1)%DAT(1)%APR(I), K = 1,NAC(1))
c               ENDDO
c              ---------------------------------------------------------
               I0 = I0 + 1
            ENDIF
            IF( CHK_STAT('aer-var_smod', AER_INFO%NAME,
     $           AER_INFO%WF_FLG) ) THEN
               DUMA(:) = ( DLOG( SIG(:,1)) )**2
               DO I = 1,LAN ; DO J = LAS(I),LAE(I)
                  AP_Mie(J,I0) = DUMA(I)
               ENDDO ; ENDDO
               DO I = 1, LAN
                  DO K = 1, NAC(I)
                     CALL SET_PTR( RTM_PTR,'aer-var_smod', ISSP=K )
                     DO J = LAS(I),LAE(I)
                        RTM_PTR%APR(J) = ( DLOG( SIG(I,K)) )**2
                        RTM_PTR%CRN(J) = RTM_PTR%APR(J) 
                     ENDDO
                  ENDDO
               ENDDO
            ENDIF
         ELSE
c        ... The option "bimodal" is selected
c        ====================================
            PSD_P(:) = MOD_R(1,1:Nsm) 
c           ...  Weighted modal radii of fine mode:
            S1 = SUM( PSD_P(:)*RD(1,1:Nsm),F_mode(1:Nsm) )/
     $           SUM( RD(1,1:Nsm),F_mode(1:Nsm) )
c           ...  Weighted modal radii of corse mode:
            S2  = SUM( PSD_P(:)*RD(1,1:Nsm),.NOT.F_mode(1:Nsm) )/
     $           SUM( RD(1,1:Nsm),.NOT.F_mode(1:Nsm) )

             WRITE(*,*) '  Weighted modal radii of fine and corse mode:'
             WRITE(*,'(4x,2(1x,F8.3))') S1, S2

            IF( Modr_f ) THEN
               AP_Mie(LAS(1):LAE(1),I0) = S1 ; I0 = I0 + 1
c              ... For single layer only! Preliminary restriction
               DO I = 1, LAN
                  DO K = 1, NAC(I)
                     IF( .NOT.F_mode(K) ) CYCLE
                     CALL SET_PTR( RTM_PTR,'aer-modr_f', ISSP=K )
                     DO J = LAS(I),LAE(I)
                        RTM_PTR%APR(J) = MOD_R(I,K)
                        RTM_PTR%CRN(J) = RTM_PTR%APR(J) 
c                        WRITE(*,*) I, K, J, RTM_PTR%APR(J) 
                     ENDDO  ! over altitude within aerosol layer (J)
                  ENDDO  ! over aerosol components (K)
               ENDDO  ! over aerosol layers (I)
            ENDIF
            IF( Modr_c ) THEN
               AP_Mie(LAS(1):LAE(1),I0) = S2  ; I0 = I0 + 1
               CALL SET_PTR( RTM_PTR, 'aer-modr_c' )
               RTM_PTR%APR(:) = AP_Mie(:,I0)
               RTM_PTR%CRN(:) = RTM_PTR%APR(:) 
            ENDIF

            PSD_P(:) = ( DLOG( SIG(1,1:Nsm) ) )**2  
            S1 = SUM( PSD_P(:)*RD(1,1:Nsm),F_mode(1:Nsm) )/
     $           SUM( RD(1,1:Nsm),F_mode(1:Nsm) )
            S2 = SUM( PSD_P(:)*RD(1,1:Nsm),.NOT.F_mode(1:Nsm) )/
     $           SUM( RD(1,1:Nsm),.NOT.F_mode(1:Nsm) )
c            WRITE(*,*) '  Weighted variance of fine and corse mode:'
c            WRITE(*,'(4x,2(1x,F8.3))') S1, S2
            IF( Var_f ) THEN
               AP_Mie(LAS(1):LAE(1),I0) = S1 ; I0 = I0 + 1
               CALL SET_PTR( RTM_PTR, 'aer-var_f' )
               RTM_PTR%APR(:) = AP_Mie(:,I0)
               RTM_PTR%CRN(:) = RTM_PTR%APR(:) 
            ENDIF
            IF( Var_c ) THEN
               AP_Mie(LAS(1):LAE(1),I0) = S2 ; I0 = I0 + 1
               CALL SET_PTR( RTM_PTR, 'aer-var_c' )
               RTM_PTR%APR(:) = AP_Mie(:,I0)
               RTM_PTR%CRN(:) = RTM_PTR%APR(:) 
            ENDIF
            IF( Frac_c ) THEN
               AP_Mie(LAS(1):LAE(1),I0) = Cfr
               CALL SET_PTR( RTM_PTR, 'aer-cm_frac' )
               RTM_PTR%APR(:) = AP_Mie(:,I0)
               RTM_PTR%CRN(:) = RTM_PTR%APR(:) 
            ENDIF
          ENDIF
      ENDIF   ! Dwr_mi save number density and PSD function parameters

      DO K = 1,NAW
         TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
      ENDDO

      IF( Scal_aer ) THEN
         IF( Rwl_aer < AER_WAVS(1) ) GOTO 132
         IF( Rwl_aer > AER_WAVS(NAW) ) GOTO 133
         Call LIN_SEARCH_NEW(Rwl_aer, Dum, K, K1, NAW, AER_WAVS)
         Scoe = Rtau_aer/( (1.D0 - DUM)*TAU(K) + DUM*TAU(K+1) )
         AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
         AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
         TAU(:) = Scoe*TAU(:)
      ENDIF
      AOT_out_VAL = TAU(1)

cc *******************************************************************
cc     --- Below writing extinction and scattering cross-section
cc     ... Write out cross-sections for selected aerosol type
c      K1 = MAXLOC( AER_COEFF(EXI,:,1),1 )
c      OPEN(44,file='./DATA_OUT/Out_ext-sca-ssa-tau-g.dat')
c        DO K = 1, NAW
c           WRITE(44,'(2x,F8.3,*(1x,e15.7))') AER_WAVS(K), 
c     $          AER_COEFF(EXI,K1,K)/DENS_PRF(K1),
c     $          AER_COEFF(SCI,K1,K)/DENS_PRF(K1),
c     $          AER_COEFF(SCI,K1,K)/AER_COEFF(EXI,K1,K), TAU(K),
c     $          Asym_Par(1,K)
c        ENDDO
c        CLOSE(44)
cc      STOP ' Stop in "mie_aer_setup.f" option: <writing out> '
cc *******************************************************************
cc     --- End writing of cross-sections 

!      WRITE(*,*) ' ======> AOD at all wavelengths: ' 
!      DO I = 1, NAW
!         WRITE(*,'(5x,2(2x,F8.3))') AER_WAVS(I), TAU(I)
!      ENDDO
!      WRITE(*,*) '  Number density at ground including AER_SCALING: '
!      WRITE(*,'(5x,2(2x,E12.4))') DENS_PRF(NLAYERS)
!      WRITE(*,'(5x,2(2x,E12.4))') AER_SCALING
!      WRITE(*,*) ' ======> ' 

c      WRITE(*,*) '  TRIM(RTM_MODE) ', TRIM(RTM_MODE)
      IF( ALLOCATED(AOD_RET) ) THEN
         AOD_RET(1,1) = T_550
      ENDIF

c      IF( N_AWF > 0 ) THEN 
         FORALL(I = 1:2) AOD_Mie(I,:) = TAU(:)
c      ENDIF
c      STOP '  ****** Stop in  "READ_MIE_AER_INP" ****** '

c     Write aerosol scenario output file
c     ===========================================
c      IF( .NOT.MODIS_AP ) CALL WRITE_SCEN_SA(*880)
      CALL WRITE_SCEN_SA(*880)

c      STOP '  Stop in "mie_aer_setup.f"  '

c      LTOP(:) = LTOP(LAN:1:-1)

c     Allocate array for MODULE DELTA_M
c     =================================
      IF( Plus_mi ) THEN
         ALLOCATE ( FDM_AER_mi(NLAYERS,NAW) ) ! A
         FDM_AER => FDM_AER_mi
      ELSE
         IF(ALLOCATED(FDM_AER_db)) DEALLOCATE(FDM_AER_db)
         ALLOCATE ( FDM_AER_db(NLAYERS,NAW) ) ! A
         FDM_AER => FDM_AER_db
      ENDIF

      IF( OUT_A_SCM ) THEN 
         OPEN( 55, file=DIR_OUT//'/mie_exi_ssa.dat' )
         WRITE(55,'(A)')  '#    Mie extinction and SSA at the selected'
         WRITE(55,'(A,2x,e12.4)') '#    discrete wavelength',
     $        WL_MIE(WEL_A)
         DO I = 1,NLAYERS
            IF( REAL(AER_COEFF(EXI,I,1)) == 0.0 ) CYCLE
            WRITE(55,'(1x,ES12.4,2(2x,e12.4))') ZHOLD(I), 
     $           AER_COEFF(EXI,I,WEL_A), 
     $           AER_COEFF(SCI,I,WEL_A)/AER_COEFF(EXI,I,WEL_A)
         ENDDO
         CLOSE(55)
c     ... Legendre coefficients output for wavelength 1
         OPEN( 55, file=DIR_OUT//'/Phn_MIE-1.dat' )
         WRITE(55,'(a)')  
     $        '#  Legendre coefficients of Mie phase function '
         DO I = 1, NL_DB
            WRITE(55,'(2x,I5,2x,E15.7)') I, USER_MOM(I,LAY_A,1,WEL_A,1)
         ENDDO
         CLOSE(55)
      ENDIF
cc     ... Legendre coefficients output for wavelength 2
c      OPEN( 55, file=DIR_OUT//'/Phn_MIE-2.dat' )
c         WRITE(55,'(a)')  
c     $        '#  Legendre coefficients of Mie phase function '
c         DO I = 1, AER_TRUNC_IDX + 1
c            WRITE(55,'(2x,I5,2x,E15.7)') I, USER_MOM(I,1,1,2,1)
c         ENDDO
c      CLOSE(55)

c      STOP
cc     --------------- Derivatives test before DM ------------------
cc
cc      Idr = 1                   ! Rm1, var1
cc      Idr = 2                   ! Rm2, var2
c      Idr = 3                   ! fraction
c      OPEN( 55, file=DIR_OUT//'/test_phm_bdm.dat' )
c         DO I = 1, AER_TRUNC_IDX + 1
c            WRITE(55,*) I, USER_MOM(I,1,1,1,1), USER_MOM(I,1,1,1,Idr+1)
c         ENDDO
c      CLOSE(55)
c      OPEN( 55, file=DIR_OUT//'/test_sca_bdm.dat' )
c         DO I = 35,NLAYERS
c            WRITE(55,*) 
c     $           ZHOLD(I), AER_COEFF(SCI,I,1), AER_DER(SCI,I,1,Idr)
c         ENDDO
c      CLOSE(55)
c      OPEN( 55, file=DIR_OUT//'/test_exi_bdm.dat' )
c         DO I = 35,NLAYERS
c            WRITE(55,*) 
c     $           ZHOLD(I), AER_COEFF(EXI,I,1), AER_DER(EXI,I,1,Idr)
c         ENDDO
c      CLOSE(55)
cc     ==========================================================      
c      !write(*,*)'To compare man and mie'
c      !DO I = NLAYERS,1,-1
c     !       WRITE(*,'(2x,F8.3,2x,E12.4,2x,E12.4)')
c     $            ZHOLD(I), AER_COEFF(EXI,I,1),AER_COEFF(EXI,I,2)
c     !    ENDDO

c     DO I = 1, 20
c         WRITE(*,'(2x,I3,*(2x,e12.4))') 
c     $        I, USER_MOM(I,1,1,1,1), USER_MOM(I,1,5,1,1),
c     $        USER_MOM(I,1,6,1,1)
c      ENDDO

      FDM_AER = 0.d0
      IF( AER_DM_FLG  ) THEN 
         Itr = AER_TRUNC_IDX
         SELECT CASE( Vec_Sca )
         CASE('vect_rte')
            CALL AER_DM_VEC
         CASE('scal_rte')
            CALL AER_DM
         END SELECT
c         WRITE(*,*) '    AER_DM_FLG = ', AER_DM_FLG
c         WRITE(*,'(4x,ES12.4)') AER_COEFF(EXI,NLAYERS,1) 
      ELSE
         APH_VS(:,:,:,:) = USER_MOM(1:NLEGEN,:,:,:,1)
         IF( Dwr_mi ) APH_DER(:,:,:,:,1:N_Dwr) 
     $        = USER_MOM(1:NLEGEN,:,:,:,2:1+N_Dwr)
      ENDIF
c
cc     --------------- Derivatives test after DM ------------------
cc      Idr = 1                   ! Rm1
cc      Idr = 2                   ! Rm2
c      Idr = 3                   ! fraction
c      OPEN( 55, file=DIR_OUT//'/test_phm_dm.dat' )
c         DO I = 1, AER_TRUNC_IDX
c            WRITE(55,*) I, AER_PHASMOMS(I,1,1), 
c     $           APH_DER(I,1,1,1,Idr)
c         ENDDO
c      CLOSE(55)
c      OPEN( 55, file=DIR_OUT//'/test_sca_dm.dat' )
c         DO I = 35,NLAYERS
c            WRITE(55,*) 
c     $           ZHOLD(I), AER_COEFF(SCI,I,1), AER_DER(SCI,I,1,Idr)
c         ENDDO
c      CLOSE(55)
c     ... Check derivative w.r.t. mode radius and variance
c      OPEN( 55, file=DIR_OUT//'/test_exi_dm.dat' )
cc         DO I = 35,NLAYERS
cc            WRITE(55,*) 
cc     $           ZHOLD(I), AER_COEFF(EXI,I,1), AER_DER(EXI,I,1,Idr)
cc         ENDDO
c         I = NLAYERS
c         DO J = 1,NAW
c            WRITE(55,'(2x,F8.3,3(2x,E15.8))') 
c     $           AER_WAVS(J), AER_COEFF(EXI,I,J), 
c     $           AER_DER(EXI,I,J,1), AER_DER(EXI,I,J,2) 
c         ENDDO
c      CLOSE(55)
cc     ==========================================================      

      IF( Plus_mi ) THEN
         !WRITE(*,*) '  Plus_mi  ',Plus_mi
         !WRITE(*,*) '  SHAPE(AC_DB) ',SHAPE(AC_DB)
         !WRITE(*,*) '  ALLOCATED(AC_DB) ', ALLOCATED(AC_DB)
         DO I = 1,LAN 
            DO J = LAS(I),LAE(I)
               AC_DB   (:,J,:) = 0.d0
               FDM_AER_db(J,:) = 0.d0
            ENDDO
         ENDDO         
c         WRITE(*,*) '  End do AC_DB'         
      ENDIF

c     WRITE(*,*) '  After delta-M  '
      IF( Plus_mi ) THEN
c         WRITE(*,*) '  After delta-M  moda Plus_mi'
c         DO I = 1, NLAYERS
c            IF( AER_COEFF(EXI,I,1)==0.d0 ) CYCLE
c            WRITE(*,'(2x,I3,2x,F8.3,2(2x,I2),2(2x,E12.4))') 
c     $           I, ZHOLD(I), IDA(I), IDX_AER_db(I),
c     $           AER_COEFF(EXI,I,1), AC_DB(EXI,I,1)
c         ENDDO
c        ... This will be used in "gt_renorm.f"
         LA_Nmi = LAN
         ALLOCATE( IDX_AER_mi(NLAYERS), LAS_mi(LAN), LAE_mi(LAN) )
c        ... LAS_mi and LAE_mi will be used in "st_update_mie.f"
         LAS_mi(:) = LAS(:) ; LAE_mi(:) = LAE(:)
         IDX_AER_mi = IDA

c        ... In this option the number of aerosol layers 
c            equal to the number of atmospheric layers.
c            LTOP is the local array in this option and 
c            will be not used outside of this routine
         LA_N = NLAYERS-1
         ALLOCATE( LA_S(LA_N), LA_E(LA_N), IDX_AER(NLAYERS) )

         DO I = 1, NLAYERS-1
            LA_S(I) = I ; IDX_AER(I) = I
         ENDDO
         LA_E = LA_S
         LA_E(NLAYERS-1)  = NLAYERS 
         IDX_AER(NLAYERS) = NLAYERS-1

      ELSE
         LA_Ndb = LAN ; LA_N = LAN
         ALLOCATE( LA_S(LAN), LA_E(LAN), L_TOP(LAN), IDX_AER(NLAYERS) )
         LA_S = LAS ; LA_E = LAE ; IDX_AER = IDA ; L_TOP = LTOP
c         WRITE(*,*) '  After delta-M '
c         DO I = 1, NLAYERS
c            WRITE(*,'(2x,I3,2x,F8.3,2x,I2,2x,E12.4)') 
c     $           I, ZHOLD(I), IDA(I), AER_COEFF(EXI,I,1)
c         ENDDO
      ENDIF

      NULLIFY( AER_WAVS, AER_COEFF, AER_PHASMOMS, APH_VS, FDM_AER )
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 101  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
      WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
!      WRITE(EU,*) '    Input line       : ', TRIM(STR_READ) 
      RETURN 1

 102  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Too few values in the field')
      WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
      RETURN 1

 122  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $         'Particle number densities not equal 1')
      WRITE(EU,*) '    for layer: ', I
      WRITE ( EU, * ) '    Check aerosol input file!'
      RETURN 1
      
 125  CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
      WRITE(EU,*) 
     $     '    Upper aerosol layer outside the atmosphere.'
      WRITE(EU,'(A,2x,F9.2,A,F9.2)') 
     $     '    Upper aerosol layer:', LTOP(LAN-1),' - ',LTOP(LAN)
      WRITE(EU,'(A,2x,F9.2,A)') 
     $     '      Top of atmosphere:', ZHOLD(1),' km'
      WRITE(EU,*) '    Check "mie_aer.inp" and altitude grid '
      WRITE(EU,*) '    as defined in the file "control.inp" '
      RETURN 1

 126  CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
      WRITE(EU,*) 
     $     '    Lower aerosol layer outside the atmospherea.'
      WRITE(EU,'(A,2x,F6.2,A)') 
     $     '       Top of aerosol layer:', LTOP(1),' km'
      WRITE(EU,'(A,2x,F6.2,A)') 
     $     '       Surface elevation   :', ZHOLD(NLAYERS),' km'
      WRITE(EU,*) '    Check "mie_aer.inp" and "Height above '
      WRITE(EU,*) '    sea level" in the file "control.inp" '
      RETURN 1

 127  CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
      WRITE(EU,*) 
     $     '    Aerosol layer outside the atmosphere.'
      WRITE(EU,'(A,2x,F9.2,A,F9.2)') 
     $     '    Top of aerosol layer:', LTOP(1)
      WRITE(EU,'(A,2x,F9.2,A)') 
     $     '      Top of atmosphere:', ZHOLD(1),' km'
      WRITE(EU,*) '    Check "mie_aer.inp" and altitude grid '
      WRITE(EU,*) '    as defined in the file "control.inp" '
      RETURN 1

 129  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Reference wavelength for AOT is out of range')
      WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
      WRITE(EU,'(A,2(2x,F7.2))') 
     $     '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
      WRITE(EU,'(A)') 
     $     '    Set reference wavelength between database wavelengths.'
      RETURN 1

 130  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Required number of aerosol layer is out of range')
      WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
      WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
      RETURN 1

 131  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Required number of wavelength is out of range')
      WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
      WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A 
      RETURN 1

 132  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Required reference wavelength is out of range')
      WRITE(EU,'(A,F8.2)') '    Reference wavelengths: ', Rwl_aer
      WRITE(EU,'(A,F8.2)') '       Minimal wavelength: ', AER_WAVS(1)
      WRITE(EU,*)  '   Check "Wavelength grid (AER_OC)" parameter '
      WRITE(EU,*)  '   in "mod_aer_oc.inp" file. '
      RETURN 1
 133  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Required reference wavelength is out of range')
      WRITE(EU,'(A,F8.2)') '    Reference wavelengths: ', Rwl_aer
      WRITE(EU,'(A,F8.2)') '       Maximal wavelength: ', AER_WAVS(NAW)
      WRITE(EU,*)  '   Check "Wavelength grid (AER_OC)" control  '
      WRITE(EU,*)  '   parameter in the "mod_aer_oc.inp" file. '
      RETURN 1
 134  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Scaling of aerosol number density is not allowed')
      WRITE(EU,*)  '   in the MODIS-MAN aerosol model  '
      WRITE(EU,*)  '   Check "Aerosol OT at reference wavelength"   '
      WRITE(EU,*)  '   control parameter in the "control_aer.inp" file.'
      RETURN 1
      
 135  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Wavelength 550 nm is out of range')
      WRITE(EU,'(A,F8.2)') '       Minimal wavelength: ', WL_Mie(1)
      WRITE(EU,*)  '   Check "Wavelength grid ( )" control parameter '
      WRITE(EU,*)  '   in "mod_aer.inp" or "mod_aer_man.inp" files. '
      RETURN 1
 136  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Wavelength 550 nm is out of range')
      WRITE(EU,'(A,F8.2)') '       Maximal wavelength: ', WL_Mie(NAW)
      WRITE(EU,*)  '   Check "Wavelength grid ( )" control parameter '
      WRITE(EU,*)  '   in "mod_aer.inp" or "mod_aer_man.inp" files. '
      RETURN 1
     
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
C**************************************************************************

      SUBROUTINE SUM_MOMS(Im, Is, MASK, RES)
      INTEGER, INTENT(IN) :: Im, Is
      LOGICAL, INTENT(IN) :: MASK(M_AC)
      REAL(KND) RES(NAW)
      INTEGER   Kw

      FORALL( Kw=1:NAW ) Res(Kw) = SUM( DB_MOMS(K1,:,Kw,K,L,Im)
     $     *DB_SCA(K1,:,Kw,Is)*RD(K1,1:Mco), MASK(1:Mco) )

      END SUBROUTINE SUM_MOMS
C**************************************************************************

      SUBROUTINE AER_DM
      REAL (KND) F_DR
      REAL (KND), ALLOCATABLE :: FDR_AER(:,:,:)

      DO I1 = 1,LAN
         DO K = 1,NAW
            F_DM = USER_MOM(Itr+1,I1,1,K,1)/DBLE(2*Itr + 1)
            FDM_AER(LAS(I1):LAE(I1),K) = F_DM
            DO L = 1,Itr
               AER_PHASMOMS(L,I1,K) = (USER_MOM(L,I1,1,K,1) -
     $              F_DM*DBLE(2*L-1))/(1.d0 - F_DM)
            END DO
         ENDDO
      ENDDO
   
      AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.d0

c      "Renormalize" scattering and extinction coefficients
c     ====================================================
      DO I = 1, NLAYERS
         DO K = 1,NAW
            F_DM = FDM_AER(I,K)
            DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
            AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0 - F_DM)
            AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
         ENDDO
      ENDDO

      IF( Dwr_mi ) THEN
         ALLOCATE( FDR_AER(N_dwr,NLAYERS,NAW) )
         FDR_AER = 0.d0
         DO Idr = 1, N_dwr
            DO I1 = 1,LAN
               DO K = 1,NAW
                  F_DM = FDM_AER(LAS(I1),K)
                  F_DR = USER_MOM(Itr+1,I1,1,K,1+Idr)/DBLE(2*Itr + 1)
                  FDR_AER(Idr,LAS(I1):LAE(I1),K) = F_DR
                  DO L = 1,Itr
                     APH_DER(L,I1,1,K,Idr) = 
     $                    (USER_MOM(L,I1,1,K,1+Idr) - F_DR*DBLE(2*L-1))
     $                    /(1.d0 - F_DM) + 
     $                    (USER_MOM(L,I1,1,K, 1) - F_DM*DBLE(2*L-1))
     $                    /(1.d0 - F_DM)**2*F_DR !*DBLE(2*L-1)

                  END DO  ! over expansion coefficients (L)
               ENDDO   ! over reference wavelengths (K)
            ENDDO  ! over aerosol layers (I1)
         ENDDO  ! over derivatives (Idr)

         DO Idr = 1, N_dwr
            DO I = 1, NLAYERS
               DO K = 1,NAW
                  F_DM = FDM_AER(I,K) 
                  F_DR = FDR_AER(Idr,I,K)
                  DUM  = AER_DER(EXI,I,K,Idr) - AER_DER(SCI,I,K,Idr)
                  AER_DER(SCI,I,K,Idr) = 
     $                 AER_DER(SCI,I,K,Idr)*(1.d0 - F_DM)
c                      !!! AER_COEFF contains delta correction already !!!
     $                 - AER_COEFF(SCI,I,K)*F_DR/(1.d0 - F_DM)
                  AER_DER(EXI,I,K,Idr) = AER_DER(SCI,I,K,Idr) + DUM
               ENDDO
            ENDDO
         ENDDO
      ENDIF

      END SUBROUTINE AER_DM

c *********************************************************************
      SUBROUTINE AER_DM_VEC
      REAL (KND) F_DR
      REAL (KND), ALLOCATABLE :: FDR_AER(:,:,:)
c     -----------------------------------------

      DO I1 = 1,LAN
         DO K = 1, NAW
            F_DM = USER_MOM(Itr+1,I1,1,K,1)/DBLE(2*Itr + 1)
            FDM_AER(LAS(I1):LAE(I1),K) = F_DM
            DO L = 1, Itr
               APH_VS(L,I1,1:4,K) = ( USER_MOM(L,I1,1:4,K,1) -
     $              F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
               APH_VS(L,I1,5:6,K) = 
     $              USER_MOM(L,I1,5:6,K,1)/(1.d0 - F_DM)
            ENDDO
         ENDDO
      ENDDO
      APH_VS(Itr+1:NLEGEN,:,:,:) = 0.d0

c     ... "Renormalize" scattering and extinction coefficients
      DO I = 1, NLAYERS
         DO K = 1,NAW
            F_DM = FDM_AER(I,K)
            DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
            AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0 - F_DM)
            AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
         ENDDO
      ENDDO

c     ... Derivatives w.r.t. expansion coefficients
      IF( Dwr_mi ) THEN
         ALLOCATE( FDR_AER(N_dwr,NLAYERS,NAW) )
         FDR_AER = 0.d0
         DO Idr = 1, N_dwr
            DO I1 = 1,LAN
               DO K = 1,NAW
                  F_DM = FDM_AER(LAS(I1),K)
                  F_DR = USER_MOM(Itr+1,I1,1,K,1+Idr)/DBLE(2*Itr + 1)
                  FDR_AER(Idr,LAS(I1):LAE(I1),K) = F_DR
                  DO L = 1,Itr
                     APH_DER(L,I1,1:4,K,Idr) = 
     $                    (USER_MOM(L,I1,1:4,K,1+Idr)-F_DR*DBLE(2*L-1))
     $                    /(1.d0 - F_DM) + 
     $                    (USER_MOM(L,I1,1:4,K,1) - F_DM*DBLE(2*L-1))
     $                    /(1.d0 - F_DM)**2*F_DR !*DBLE(2*L-1)
                     
                     APH_DER(L,I1,5:6,K,Idr) = 
     $                    USER_MOM(L,I1,5:6,K,1+Idr)/(1.d0 - F_DM) + 
     $                    USER_MOM(L,I1,5:6,K,1)/(1.d0 - F_DM)**2
     $                    *F_DR !*DBLE(2*L-1)

                  END DO  ! over expansion coefficients (L)
               ENDDO   ! over reference wavelengths (K)
            ENDDO  ! over aerosol layers (I1)
         ENDDO  ! over derivatives (Idr)

c        ... Renormalize derivative of scattering and absorption coeff.
         DO Idr = 1, N_dwr
            DO I = 1, NLAYERS
               DO K = 1,NAW
                  F_DM = FDM_AER(I,K) ; F_DR = FDR_AER(Idr,I,K)
                  DUM = AER_DER(EXI,I,K,Idr) - AER_DER(SCI,I,K,Idr)
                  AER_DER(SCI,I,K,Idr) = 
     $                 AER_DER(SCI,I,K,Idr)*(1.d0 - F_DM)
     $                 - AER_COEFF(SCI,I,K)*F_DR/(1.d0 - F_DM)
                  AER_DER(EXI,I,K,Idr) = AER_DER(SCI,I,K,Idr) + DUM
               ENDDO
            ENDDO
         ENDDO
      ENDIF

      END SUBROUTINE AER_DM_VEC

c ********************************************************************
      SUBROUTINE  WRITE_SCEN_SA(*)
      USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER
      USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 

      Integer  Unt
      DOUBLE PRECISION    SS_A, Ext
      CHARACTER(LEN=3)    PHT
      Character (Len=500)  OutStr
      REAL (KND), ALLOCATABLE :: T_OUT(:)
c     ------------------------------------------------------------------
      
      IF (LOG_LEVEL(1) >= N_SSO) 
     $     WRITE (lunit,*) ' *** writing: AEROSOL.OUT'

c     Optical thickness at user defined wavelength grid
      IF( NOWL_AER > 0 ) ALLOCATE( T_OUT(NOWL_AER) )
      DO I = 1, NOWL_AER
         Call LIN_SEARCH_NEW(OWL_AER(I), Dum, K, K1, NAW, AER_WAVS)
         T_OUT(I) =  (1.d0 - DUM)*TAU(K) + DUM*TAU(K1)
      ENDDO

      Call OpenOut ('SCE_AEROSOL.OUT', Unt)
      WRITE(OutStr,*) '     ==================================='
      Call WriteOut (Unt, OutStr)
c      WRITE(OutStr,*) '        Settings for Mie aerosol model  '
      WRITE(OutStr,*) Txt_OUT
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,*) '     ==================================='
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,*) ' '
      Call WriteOut (Unt, OutStr)

      IF( Scal_aer ) THEN
         WRITE(OutStr,'(A,F7.3,A,F7.2,A)') 
     $        '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at',
     $        Rwl_aer,' nm wavelength.'
         Call WriteOut (Unt, OutStr)
         
         WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,'(6x,A)') 
     $        '----------------------------------------------------'
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,*) 
         Call WriteOut (Unt, OutStr)
      ENDIF  ! Scal_aer

      WRITE(OutStr,'(A)') 
     $     '  Particle size distribution function :'
      Call WriteOut (Unt, OutStr)
      IF( Bi_PSD ) THEN
         WRITE(OutStr,'(27x,A)') ' lognormal bimodal ' 
      ELSE
         WRITE(OutStr,'(27x,A)') ' lognormal mono-modal '
      ENDIF
      Call WriteOut (Unt, OutStr)
      
      WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
      Call WriteOut (Unt, OutStr)
      WRITE ( OutStr, * )
      Call WriteOut (Unt, OutStr)
      DO I = 1, LAN
c     K1 = LAN-I+1
         K1 = I
         WRITE(OutStr,'(A27,2X,I2)') 
     $        '  Layer :                 :', I
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,'(A27,2(2X,F6.2))') 
     $        '      Top of layer [km]   :', LTOP(I)
         Call WriteOut (Unt, OutStr)
c        --------------------------------------------------------------         
         IF( .NOT.MODIS_AP ) THEN
            WRITE(OutStr,'(A27,2X,F3.0)') 
     $           '  Rel. humidity [%]       :', REL_HUM(K1)
            Call WriteOut (Unt, OutStr)
            WRITE(OutStr,'(A27,2X,A30)') 
     $           '  Aerosol type            :', AER_TYPE (I)
            Call WriteOut (Unt, OutStr)
            WRITE(OutStr,'(27X,A,2x,A)')
     $           '  Component', 'Mix.ratio'
            Call WriteOut (Unt, OutStr)
            DO J = 1,NAC(K1) 
               WRITE(OutStr,'(29X,A,2x,E15.5)') 
     $              TRIM(DB_NAM(K1,J)), RD(K1,J)
               Call WriteOut (Unt, OutStr)
            ENDDO
         ENDIF
c        --------------------------------------------------------------         
         WRITE(OutStr,*)
         Call WriteOut (Unt, OutStr)
      ENDDO                     ! over aerosol layers (I)
      
      IF( NOWL_AER > 0 ) THEN
         WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
         Call WriteOut (Unt, OutStr)
         DO I = 1, NOWL_AER
            WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
            Call WriteOut (Unt, OutStr)
        ENDDO
      ENDIF
      WRITE ( OutStr, * ) ' '
      Call WriteOut (Unt, OutStr)
         
      WRITE(OutStr,'(A47)') 
     $     'Profile type: particle number density [1/cm^3].'
      Call WriteOut (Unt, OutStr)
      
      WRITE(OutStr,*)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,*) ' Vertical profiles of aerosol'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,*) 
     $     '         extinction coefficient (Ext coeff) and'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,*) 
     $     '         single scattering albedo (Ss albedo):'
      Call WriteOut (Unt, OutStr)
      PHT = 'Mie'
      DO K = 1,NAW
         WRITE(OutStr,*) '   '
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,'(1x,a11,F7.2,a6,a6,F8.4)') 
     $        'Wavelength ',AER_WAVS(K),' [nm];',' AOT =', TAU(K)
         Call WriteOut (Unt, OutStr)
         Call WriteOut (Unt, 
     $        ' ----------------------------------------------')
         Call WriteOut (Unt, 
     $        '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
         DO I = 1, NLAYERS
            Ext = AER_COEFF(EXI,I,K)
            IF( Ext == 0.0d0 ) CYCLE
            SS_A = AER_COEFF(SCI,I,K)/Ext
            WRITE(OutStr,'(6x,I3,4x,F6.2,6x,ES9.2,6x,E12.4)') 
     $           I, ZHOLD(I), Ext, SS_A
            Call WriteOut (Unt, OutStr)
         END DO
         WRITE(OutStr,*) ' '
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,*) 
     $        '    Asymmetry factor used in the aerosol layers :'
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,*) 
     $        '    ------------------------------------------- :'
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,*) 
     $        '    Layer  Top [km]  Bottom [km]   Asymmetry    Phase'
         Call WriteOut (Unt, OutStr)
         WRITE(OutStr,*) 
     $        '                                    factor     function'
         Call WriteOut (Unt, OutStr)
         DO I = 1,LAN
            WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)')
     $           I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
            Call WriteOut (Unt, OutStr)
         ENDDO                  ! over aerosol layers (I)
      ENDDO                     ! over reference wavelengths (K)
      
      Call CloseOut (Unt)
      RETURN
      
 130  CALL WRITE_MESSAGE (R_NAME,'ERROR',
     $     'Output wavelength for AOT is out of range')
      WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
      WRITE(EU,'(A,2(2x,F7.2))') 
     $     '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
      WRITE(EU,'(A)') 
     $     '    Set output wavelength between database wavelengths'
      WRITE(EU,'(A)') 
     $     '    in the "control_aer.inp" file.'
      RETURN 1

      END SUBROUTINE  WRITE_SCEN_SA

C***********************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS
      
      IF(ALLOCATED(Mie_layer)) DEALLOCATE(Mie_layer, DENS_PRF)
      IF(ALLOCATED(AW_MI)) DEALLOCATE(AW_MI, AC_MI, APH_VS_mi)
      IF(ALLOCATED(AER_DER)) DEALLOCATE(AER_DER, APH_DER)
      IF(ALLOCATED(AP_Mie)) DEALLOCATE(AP_Mie, AOD_Mie)
      IF(ALLOCATED(D_Greek)) DEALLOCATE(D_Greek)
      IF(ALLOCATED(FDM_AER_mi)) DEALLOCATE(FDM_AER_mi)
      IF(ALLOCATED(IDX_AER_mi)) DEALLOCATE(IDX_AER_mi, LAS_mi, LAE_mi)
      IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
      IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP)
      IF(ALLOCATED(NAC)) DEALLOCATE(NAC, RD, F_MODE, MOD_R, SIG)
      IF(ALLOCATED(L_ARR)) DEALLOCATE(L_ARR)
      IF(ALLOCATED(N_PSD)) DEALLOCATE(N_PSD)
      IF(ALLOCATED(RI_rea)) DEALLOCATE(RI_rea, RI_ima)
      IF(ALLOCATED(COEF1)) DEALLOCATE(COEF1, COEF2, COEF3, COEF4, COEF5, 
     $     COEF6, COEF7, COEF8)

      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS

C***********************************************************************
      END
C**************************************************************************
