!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 16 Dec 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
MODULE TOOLS

  Use BasicTools, Only: UPPER_LOWER_CASE => ULC 
  USE LOG_PARS,   ONLY: EU
  USE TYPES,      ONLY: KND, DYN_CHAR, ArrChar
   

  TYPE AKIMA_VARS  
     REAL (KND), ALLOCATABLE :: X(:), A(:), B(:), C(:), D(:)
  END TYPE AKIMA_VARS

  PRIVATE
  PUBLIC :: FIND_IDX, UPPER_LOWER_CASE, FIND_MATCHES, CHK_STAT, CUT_STR, &
       BLANK_OR_COMMENT, READ_FILE, SET_PATH, INQ_WF, Read_Text,&
       GET_CONTROL_PARS, SPLIT_STRING, AKIMA_INIT, AKIMA_FIN, AKIMA_VARS, &
       AVERAGE, SMOOTH, ALT_SPLINE_INIT, ALT_SPLINE, INIT_PARS, ErrMsg, &
       SET_PTR, Read_Input, Join, Height_Grid_V2, E_CSZA, Read_LineCDF, &
       GET_SZA_IDX, LOS_MS_IDX, IsPar, Array_From_String, NO_PATH, &
       Read_OptCDF, MatrInvers, OpenOut, WriteOut, CloseOut, ResetOut, &
       ReInitPar, NotFound, CutChars, Height_Grid_V3, RotMatr, IsNum, &
       RemoveChars
  
CONTAINS
!***************************************************************************
  FUNCTION INQ_WF (SRC_STR, SELECTOR, IDX_WF)
    USE INFO, ONLY: WF_DB, RET_INFO 
    LOGICAL INQ_WF
    CHARACTER (LEN = *) SRC_STR, SELECTOR
    INTEGER, OPTIONAL :: IDX_WF
    INTENT (IN)  :: SRC_STR, SELECTOR
    INTENT (OUT) :: IDX_WF

!   Local variables
    CHARACTER (LEN = :), ALLOCATABLE :: SLC 
  !==================================================================

    SLC = SELECTOR
    CALL UPPER_LOWER_CASE (SLC,'L2U')
    
    INQ_WF = .FALSE.
    SELECT CASE (SLC)
    CASE ('WFN', 'WFT')
       IF(.NOT.ALLOCATED(WF_DB)) RETURN
    CASE ('RIN', 'RIT')
       IF(.NOT.ALLOCATED(RET_INFO)) RETURN
    END SELECT
    
    SELECT CASE (SLC)
    CASE ('WFN')
       IF(PRESENT(IDX_WF)) THEN
          IDX_WF = FIND_IDX(SRC_STR, WF_DB%NAME)
          IF(IDX_WF > 0) INQ_WF = .TRUE.
       ELSE
          INQ_WF = CHK_STAT(SRC_STR, WF_DB%NAME)
       END IF
    CASE ('WFT')
       IF(PRESENT(IDX_WF)) THEN
          IDX_WF = FIND_IDX(SRC_STR, WF_DB%TYPE)
          IF(IDX_WF > 0) INQ_WF = .TRUE.
       ELSE
          INQ_WF = CHK_STAT(SRC_STR, WF_DB%TYPE)
       END IF
    CASE ('RIN')
       IF(PRESENT(IDX_WF)) THEN
          IDX_WF = FIND_IDX(SRC_STR, RET_INFO%NAME)
          IF(IDX_WF > 0) INQ_WF = .TRUE.
       ELSE
          INQ_WF = CHK_STAT(SRC_STR, RET_INFO%NAME, RET_INFO%STAT, &
               RET_INFO%WF_ON)
       END IF
    CASE ('RIT')
       IF(PRESENT(IDX_WF)) THEN
          IDX_WF = FIND_IDX(SRC_STR, RET_INFO%TYPE)
          IF(IDX_WF > 0) INQ_WF = .TRUE.
       ELSE
          INQ_WF = CHK_STAT(SRC_STR, RET_INFO%TYPE, RET_INFO%STAT, &
               RET_INFO%WF_ON)
       END IF
    END SELECT
    
 RETURN
END FUNCTION INQ_WF
!***************************************************************************
  LOGICAL FUNCTION CHK_STAT (STR_INP, DATA_INP, FLG_INP_1, FLG_INP_2)   
    
    USE INFO, ONLY: NAME_TAG

    CHARACTER (LEN=*) STR_INP
    CHARACTER (LEN=*), OPTIONAL ::  DATA_INP(:)
    LOGICAL, OPTIONAL :: FLG_INP_1(:), FLG_INP_2(:)

!   Local variables
    INTEGER  II, LEN_INP
    LOGICAL, ALLOCATABLE :: FLG(:)
    CHARACTER (LEN=:), ALLOCATABLE :: STR_1, STR_2
    
    CHK_STAT = .FALSE.
    IF(.NOT.PRESENT(DATA_INP)) RETURN
    LEN_INP = LEN(STR_INP)

    ALLOCATE(FLG(SIZE(DATA_INP)))

    IF (PRESENT(FLG_INP_1)) THEN
       IF (PRESENT(FLG_INP_2)) THEN
          FLG = FLG_INP_1.AND.FLG_INP_2
       ELSE
          FLG = FLG_INP_1
       END IF
    ELSE
       FLG = .TRUE.
    END IF
    
    STR_1 = STR_INP
    CALL UPPER_LOWER_CASE (STR_1,'U2L')
    DO II = 1, SIZE(DATA_INP)
       STR_2 = DATA_INP(II)
       CALL UPPER_LOWER_CASE (STR_2,'U2L')
       IF(STR_1 /= STR_2(1:LEN_INP)) CYCLE
       IF(FLG(II)) CHK_STAT = .TRUE.
       EXIT
    END DO

    RETURN 
  END FUNCTION CHK_STAT
  
!***************************************************************************
  FUNCTION FIND_IDX (STR, DATA_INP, FLAG_INP)   

    USE INFO, ONLY: NAME_TAG
    
    INTEGER  FIND_IDX, II, KK
    CHARACTER (LEN=*)  STR
    CHARACTER (LEN=*), OPTIONAL ::   DATA_INP(:)
    LOGICAL, OPTIONAL :: FLAG_INP
    
    ! Local variables
    CHARACTER (LEN=:), ALLOCATABLE :: STR_1, STR_2
    LOGICAL, ALLOCATABLE :: FLAG(:)
    
    FIND_IDX = -1
    IF(.NOT.PRESENT(DATA_INP)) RETURN
    ALLOCATE (FLAG(SIZE(DATA_INP))) 
    STR_1 = TRIM(STR)
    CALL UPPER_LOWER_CASE (STR_1,'U2L')
    
    IF(PRESENT(FLAG_INP)) THEN
       FLAG = FLAG_INP
    ELSE
       FLAG = .TRUE.
    END IF
    
    KK = 0
    DO II = 1, SIZE(DATA_INP)
       IF(.NOT. FLAG(II)) CYCLE
       KK = KK + 1
       STR_2 = TRIM(DATA_INP(II))
       CALL UPPER_LOWER_CASE (STR_2,'U2L')
       IF(STR_1 /= STR_2) CYCLE
       FIND_IDX = KK
       RETURN
    END DO

    RETURN
  END FUNCTION FIND_IDX
  
!***************************************************************************
  SUBROUTINE FIND_MATCHES(*, INP_STRING, DATA_INP, IDX_OUT, ADD_STR)      
    
    USE LOG_PARS, ONLY: EU
    
    CHARACTER (LEN = *)  DATA_INP(:)
    CHARACTER (LEN = *)  INP_STRING, ADD_STR
    LOGICAL              IDX_OUT(:)
    OPTIONAL ADD_STR
    
    ! Local variables
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'FIND_MATCHES'
    CHARACTER (LEN = :), ALLOCATABLE :: PAR_NAME, ADD_S
    INTEGER              KK, I, II, LL, NAME_IDX

    IDX_OUT(:) = .FALSE.
    CALL UPPER_LOWER_CASE (INP_STRING,'U2L')
    IF(PRESENT(ADD_STR)) THEN
       ADD_S = ADD_STR
       CALL UPPER_LOWER_CASE (ADD_STR,'U2L')
    ELSE
       ADD_S = ''
    END IF
    
    LL = LEN_TRIM(INP_STRING) 
    DO II = 1, LL ! Skip leading space and quotation marks (single and double)
       SELECT CASE (IACHAR(INP_STRING(II:II)))
       CASE(32,34,39)   
       CASE DEFAULT 
          EXIT
       END SELECT
    END DO

    IF(INP_STRING(II:MIN(LL,II + 2)) == 'all') THEN
       IDX_OUT(:) = .TRUE.
       RETURN
    END IF

    IF(INP_STRING(II:MIN(LL,II+3)) == 'none') RETURN

    KK = 0 ;  PAR_NAME = '' ; NAME_IDX = 0
    DO I = II, LL
       SELECT CASE (IACHAR(INP_STRING(I:I)))
       CASE(32,44,34,39)      !Space, comma, and quotation marks (single and double)
          IF(KK /= 0) THEN
             NAME_IDX = FIND_IDX(PAR_NAME//ADD_S, DATA_INP)
             IF(PRESENT(ADD_STR).AND.NAME_IDX < 0) &
                  NAME_IDX = FIND_IDX(PAR_NAME, DATA_INP)
             IF(NAME_IDX < 0) GOTO 200 
             IDX_OUT(NAME_IDX) = .TRUE.
          END IF
          KK = 0 ;  PAR_NAME = ''
       CASE DEFAULT 
          KK = KK + 1
          PAR_NAME = PAR_NAME//INP_STRING(I:I)
       END SELECT
    END DO
    IF(KK /= 0) THEN
       NAME_IDX = FIND_IDX(PAR_NAME//ADD_S, DATA_INP)   
       IF(PRESENT(ADD_STR).AND.NAME_IDX < 0) &
            NAME_IDX = FIND_IDX(PAR_NAME, DATA_INP)
       IF(NAME_IDX < 0) GOTO 200 
       IDX_OUT(NAME_IDX) = .TRUE.
    END IF
    
    RETURN
    
200 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Parameter is not found')
    WRITE(EU,*) '    Selected value: ', PAR_NAME
    RETURN 1
    
  END SUBROUTINE FIND_MATCHES

!***************************************************************************
  FUNCTION CUT_STR (STRING)
    
    CHARACTER (LEN = *) STRING
    CHARACTER (LEN = :), ALLOCATABLE :: CUT_STR
    INTEGER  I
    LOGICAL  FLG
    
    CUT_STR = '' ;  FLG = .FALSE.
    DO I = 1, LEN_TRIM(STRING)
       SELECT CASE (IACHAR(STRING(I:I)))
       CASE (32)              ! Space
          IF(FLG) CUT_STR = CUT_STR//STRING(I:I) ! Leading space is skipped
       CASE(34,39)            ! Quotation marks (single and double)
       CASE DEFAULT
          FLG = .TRUE.
          CUT_STR = CUT_STR//STRING(I:I)
       END SELECT
    END DO
    CUT_STR = TRIM(CUT_STR)

    RETURN
  END FUNCTION CUT_STR

!***************************************************************************
! This routine removes leading characters
  FUNCTION CutChars (STRING)
    
    CHARACTER (LEN = *) STRING
    CHARACTER (LEN = :), ALLOCATABLE :: CutChars
    INTEGER  I
    LOGICAL  FLG
    
    CutChars = '' ;  FLG = .FALSE.
    DO I = 1, LEN_TRIM(STRING)
       SELECT CASE (IACHAR(STRING(I:I)))
       CASE (:47,58:)              ! Space
          IF(FLG) CutChars = CutChars//STRING(I:I) ! Leading space is skipped
!       CASE(34,39)            ! Quotation marks (single and double)
       CASE DEFAULT
          FLG = .TRUE.
          CutChars = CutChars//STRING(I:I)
       END SELECT
    END DO
    CutChars = TRIM(CutChars)

    RETURN
  END FUNCTION CutChars

!***************************************************************************
  ! This routine removes all characters wich exeption of internal spaces,
  ! periods, signes and decimal characters
  FUNCTION RemoveChars (STRING)
    
    CHARACTER (LEN = *) STRING
    CHARACTER (LEN = :), ALLOCATABLE :: RemoveChars
    INTEGER  I, LN
    LOGICAL  FLG, PreDigit, PreSpace, PreSign, CheckDot, CheckSign, PreDec
    
    RemoveChars = '' ;  FLG = .FALSE. ; PreDigit = .False.; PreSpace = .False.
    CheckDot = .False. ;  CheckSign = .False. ; PreDec = .False.
    PreSign = .False.
    DO I = 1, LEN_TRIM(STRING)
       SELECT CASE (IACHAR(STRING(I:I)))
       CASE (32)              ! Keep internal spaces
          PreSpace = .True. ; CheckDot = .False. 
          IF(FLG) RemoveChars = RemoveChars//STRING(I:I) 
       CASE (47:57)           ! Digits
          FLG = .TRUE.
          PreDigit = .True. ; PreSpace = .False.; CheckDot = .False.
          CheckSign = .False.; PreDec = .False. ; PreSign = .False.
          RemoveChars = RemoveChars//STRING(I:I) 
       CASE (46)    !dot
          FLG = .TRUE.
          If(CheckDot) Then
             LN = Len(RemoveChars)
             RemoveChars = RemoveChars(1:LN - 1)
             CheckDot = .False.
          Else
             CheckDot = .True.
             If(PreDigit.or.PreSpace.or.PreSign) &
                  RemoveChars = RemoveChars//STRING(I:I)
          End If
          PreDigit = .True. ; PreSpace = .False. ; PreSign = .False.
          CheckSign = .False. ; PreDec = .False.
       CASE (43,45)    ! +, -
          FLG = .TRUE.
          If(CheckSign) Then
             LN = Len(RemoveChars)
             RemoveChars = RemoveChars(1:LN - 1)
             CheckSign = .False.
          Else
             CheckSign = .True.
             If(PreDigit.or.PreSpace.or.PreDec) &
                  RemoveChars = RemoveChars//STRING(I:I)
          End If
          PreSpace = .False.; CheckDot = .False. ; PreSign = .True.
          PreDec = .False.
       CASE (68,69,100,101)    ! D, E, d, e
          If(CheckSign) Then
             LN = Len(RemoveChars)
             RemoveChars = RemoveChars(1:LN - 1)
          End If
          PreSpace = .False.; CheckDot = .False.; CheckSign = .False.
          PreDec = .True. ; PreSign = .False.
          If( PreDigit) RemoveChars = RemoveChars//STRING(I:I) 
       CASE DEFAULT
          If(CheckSign.or.CheckDot) Then
             LN = Len(RemoveChars)
             RemoveChars = RemoveChars(1:LN - 1)
          End If
          PreDigit = .False. ; PreSpace = .False.; CheckDot = .False.
          CheckSign = .False. ; PreDec = .False. ; PreSign = .False.
       END SELECT
    END DO
    RemoveChars = TRIM(RemoveChars)

    RETURN
  END FUNCTION RemoveChars

!***************************************************************************
  FUNCTION Join (ARR_INP, I_S, I_E)
    INTEGER I_S, I_E, II, II_1, II_2
    CHARACTER (LEN=:), ALLOCATABLE :: Join
    TYPE (DYN_CHAR), ALLOCATABLE :: ARR_INP(:)
    OPTIONAL :: I_S, I_E
    !==================================================================
    
    IF(PRESENT(I_S)) THEN
       II_1 = I_S
    ELSE
       II_1 = 1
    END IF

    IF(PRESENT(I_E)) THEN
       II_2 = I_E
    ELSE
       II_2 = SIZE(ARR_INP)
    END IF

    Join = ARR_INP(II_1)%S
    DO II = II_1 + 1, II_2
       Join = Join//' '//ARR_INP(II)%S
    END DO

    RETURN
  END FUNCTION Join
!***************************************************************************
  SUBROUTINE Read_Input (SRC_STR, READ_ARR, *, JOIN, OutputType, REARRANGE, &
       LocF, ULC, Nlines)
    ! Optional parameters:
    !    JOIN - if ".true." joins input from multiple lines into one string
    !    OutputType - if "delayed" forced the output to be stored in the
    !                 "OutLog" linked list rather than printed out
    !    REARRANGE - if ".true." joins input from multiple lines into one
    !                string and the splits it again making an array from
    !                space separated substrings
    !    LocF - if ".true." forbids reading of global settings
    !    ULC - converts the input to upper or lower case if set to "U" or
    !          "L", respectively.
    !    Nlines - if only one input line is specified it will be repeated
    !             Nlines time in the output (no effect if more than one
    !             input line is read) 
    Use OutInf,   Only: L_S, L_E, L_U, LUF, OutLog, FIRST_OUT
    USE ST_INPUT, ONLY: Inp_Pars, CI_Pars
    CHARACTER (LEN=*)   SRC_STR
    CHARACTER (LEN=*), OPTIONAL :: OutputType, ULC
    Integer, Optional :: Nlines
    LOGICAL, OPTIONAL :: JOIN, REARRANGE, LocF
    TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
    ! Local variables:
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Read_Input'
    CHARACTER (LEN=:), ALLOCATABLE :: STRING
    INTEGER I_S, I_E, II, NN, K
    LOGICAL OUTPUT_FLG, Joined, ReArranged, CI_F
    TYPE (DYN_CHAR), POINTER :: Inp_Pars_Loc(:)
    TYPE(DYN_CHAR), Allocatable  :: MSG(:)
    !==================================================================
!    CALL ST_TRACK (R_Name, 3, 'I')

    ! Check local input parameters
    i_s = 0
    if(Allocated(Inp_Pars)) Call Get_Line (SRC_STR, Inp_Pars, i_s, i_e)
    
    if(i_s == 0.and.Allocated(CI_Pars)) Then
       ! If the searched parameter is not present in the local 
       ! parameter list we check common input parameters
       Call Get_Line (SRC_STR, CI_Pars, i_s, i_e)
       Inp_Pars_Loc => CI_Pars
       CI_F = .TRUE.
    Else
       Inp_Pars_Loc => Inp_Pars
       CI_F = .FALSE.
    End if

    if(Allocated(Read_Arr)) Deallocate(Read_Arr)
    if(Present(LocF)) Then
       If(LocF.and.CI_F) Return
    End If
       
    if(i_s == 0) goto 100
    if(i_e == 0) i_e = size(Inp_Pars)
    nn = dim(i_e + 1,i_s)

    Fill_Output: Block
    If(Present(Nlines)) THEN
         If(nn /= Nlines) Then
            If(nn /= 1) GoTo 101
            ! If only one entry is present we assume it is the same for
            ! all required lines
            Allocate(Read_Arr(Nlines))
            ForAll(ii = 1:Nlines) Read_Arr(ii)%S = Inp_Pars_Loc(i_s)%S
            Exit Fill_Output
         End If
      End if
      Allocate(Read_Arr(nn))
      do ii = i_s, i_e
         Read_Arr(ii - i_s + 1)%S = Inp_Pars_Loc(ii)%S
      end do
    End Block Fill_Output

    If(present(OutputType)) Then
       String = OutputType
       CALL UPPER_LOWER_CASE (String,'U2L')
       If(String=='delayed') Then
          IF (.NOT.ASSOCIATED(OutLog)) THEN
             ALLOCATE(FIRST_OUT)
             OutLog => FIRST_OUT
          ELSE
             ALLOCATE(OutLog%NEXT_OUT)
             OutLog => OutLog%NEXT_OUT
          ENDIF
          OutLog%Head = ' *** '//SRC_STR
          Allocate(OutLog%Txt, Source = READ_ARR)
       End If
    Else
       DO K = L_S, L_E
          Call WriteOut(L_U(K),'  *** '//SRC_STR, LUF(K))
          IF(SIZE(READ_ARR) == 0) Call WriteOut(L_U(K),'      --> ', LUF(K))
          DO II = 1, SIZE(READ_ARR)
             Call WriteOut(L_U(K),'      --> '//READ_ARR(II)%S, LUF(K))
          END DO
       END DO
    END IF

    If(Size(READ_ARR) == 0) Then
!       Call ST_TRACK (R_Name, 3, 'O')
       Return
    End If
    
    if(present(join)) then
       Joined = join
    else
       Joined = .false.
    end if
       
    if(present(ReArrange)) then
       ReArranged = ReArrange
    else
       ReArranged = .false.
    end if

    if(joined .or. ReArranged) then
       STRING = READ_ARR(1)%S
       DO II = 2, SIZE(READ_ARR)
          STRING = STRING//' '//READ_ARR(II)%S
       END DO
       if(Joined) then
          Deallocate(Read_Arr) ;  Allocate(Read_Arr(1))
          Read_Arr(1)%S = String
       else
          CALL Split_String(String, Read_Arr)
       end if
    end if

    If(Present(ULC)) Then
       Select Case (ULC)
       Case ("U")
          Do II = 1, Size(Read_Arr)
             CALL UPPER_LOWER_CASE (Read_Arr(II)%S,'L2U')
          End Do
       Case ("L")
          Do II = 1, Size(Read_Arr)
             CALL UPPER_LOWER_CASE (Read_Arr(II)%S,'U2L')
          End Do
       End Select
    End If
       
!    CALL ST_TRACK (R_Name, 3, 'O')
    return
   
100 call Write_Message(R_Name, 'ERROR', 'Key phrase "'//SRC_STR//&
         '" is not found')
    return 1
101 Allocate(Msg(2))
    Msg(1)%S = 'Unexpected number of input lines'
    Msg(2)%S = 'Parameter to read: '//SRC_STR
    call ErrMsg(R_Name, 'e', Msg)
    Return 1
  END SUBROUTINE Read_Input
!***************************************************************************
  Subroutine Get_Line (SRC_STR, Inps, i_s, i_e)
    INTEGER I_S, I_E, II, NN
    CHARACTER (LEN=*)   SRC_STR
    TYPE(DYN_CHAR), ALLOCATABLE:: Inps(:)
    
    NN = LEN_TRIM(ADJUSTL(SRC_STR))
    i_s = 0; i_e = 0
    do ii = 1, size(Inps)
       if(LEN(Inps(ii)%S) == NN.AND.Inps(ii)%S == SRC_STR) i_s = ii + 1
       if(i_s == 0) cycle
       if(Blank_Or_Comment(Inps(ii)%S)) then
          i_e = ii - 1
          exit
       end if
    end do
    Return
  End Subroutine Get_Line
!***************************************************************************
  FUNCTION BLANK_OR_COMMENT (STRING)
    
    LOGICAL            BLANK_OR_COMMENT
    CHARACTER (LEN=*)  STRING
    CHARACTER (LEN=:), ALLOCATABLE ::  STRING_1
    ! Local variables:
    INTEGER            II, LL
    
    BLANK_OR_COMMENT = .FALSE.
    
    LL = LEN_TRIM(STRING)
    IF(LL == 0) THEN
       BLANK_OR_COMMENT = .TRUE.
       RETURN
    END IF
    
    DO II = 1, LL
       IF(IACHAR(STRING(II:II)) == 35) THEN ! look for "#" character 
          BLANK_OR_COMMENT = .TRUE.
          RETURN
       END IF
       IF(IACHAR(STRING(II:II)) /= 32) EXIT ! Only continue if "space" is found
    END DO

    STRING_1 = CUT_STR(STRING)
    CALL UPPER_LOWER_CASE (STRING_1,'U2L')
    IF(STRING_1 == 'none') BLANK_OR_COMMENT = .TRUE.

    
    RETURN
  END FUNCTION BLANK_OR_COMMENT

!***************************************************************************
  ! This subroutione reads the input line with an optional flag "CDF"
  ! NLines define the expected number of lines. If the actual number of
  ! lines is equal 1, all lines are considered to be the same
  ! NameFild defines the number of the field containing the file name
  ! (directory path will be cut off if CDF parameter is precified)
  ! The contents of the CDF paramemeter are always stored in the last field
  SUBROUTINE Read_OptCDF (STR_INP, NameField, ReadStc, *, NLines, NSkip)
    Use Types, Only: DYN_CHAR, ArrChar
    CHARACTER (LEN=*)   STR_INP
    Integer   NameField, NLines, NSkip
    TYPE (ArrChar), ALLOCATABLE :: ReadStc(:)
    ! Local variables:
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Read_OptCDF'
    CHARACTER (LEN=:), ALLOCATABLE :: STRING
    INTEGER II, NN, IJ, Nlns, NF
    TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
    Optional :: Nlines, NSkip
    !==================================================================
    CALL ST_TRACK (R_Name, 3, 'I')

    If(Allocated(ReadStc)) Then
       Do II = 1, Size(ReadStc)
          If(Allocated(ReadStc(II)%Arr)) DeAllocate(ReadStc(II)%Arr)
       End Do
       DeAllocate(ReadStc)
    End If

    CALL Read_Input (STR_INP, READ_ARR, *880)
    NN = SIZE(READ_ARR)
    If(Present(NLines)) Then
       IF(NN < NLines .AND. NN /= 1) GoTo 110
       Nlns = Nlines
    Else
       Nlns = NN
    End If
    
    Allocate(ReadStc(Nlns))
    Do II = 1, Nlns
       NF = NameField
       IF(Present(NSkip)) Then
          If(NSkip >= II) NF = 0
       End If
       Call Read_LineCDF (READ_ARR(II)%S, NF, ReadStc(II)%Arr)
       IF(NN == 1 .AND. Present(Nlines)) THEN 
          ! If only one entry is present we assume it is the same for
          ! all data sets
          DO IJ = 2, Nlns
             Allocate(ReadStc(IJ)%Arr(Size(ReadStc(1)%Arr)))
             ReadStc(IJ)%Arr = ReadStc(1)%Arr
          END DO
          EXIT
       END IF
    End Do
    
    CALL ST_TRACK (R_Name, 3, 'O')
    Return
    
110 CALL WRITE_MESSAGE(R_NAME, 'E', 'Too few input lines in the input field')
    WRITE(EU,*) '    Keyword   : ', TRIM(STR_INP) 
    WRITE(EU,*) '    Required number of input lines: ', NLines 
    RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1
  END SUBROUTINE Read_OptCDF

!***************************************************************************
  Subroutine Read_LineCDF (InpStr, NF, OutArr)
    Use Types, Only: Dyn_Char
    Integer NF
    Character (Len=:), Allocatable :: InpStr
    Type (Dyn_Char), Allocatable :: OutArr(:)
    ! Local variables:
    Integer Lstr, Istr, KK
    Character (Len=*), Parameter :: R_Name = 'Read_LineCDF'
    Type (Dyn_Char), Allocatable :: STR_ARR(:), STR_ARR_1(:)
    !==================================================================
    CALL ST_TRACK (R_Name, 3, 'I')
    
    If(Allocated(OutArr)) Deallocate(OutArr)
    Call Split_String(InpStr, STR_ARR)
    LStr = Size(STR_ARR) ;  IStr = Index(STR_ARR(LStr)%S, 'CDF=')
    Allocate(OutArr(LStr))
    If(Istr /= 0) Then
       Call Split_String(STR_ARR(LStr)%S(IStr:), STR_ARR_1, '=')
       Do KK = 1, LStr - 1
          IF(KK == NF) Then
             OutArr(KK)%S = NO_PATH(STR_ARR(KK)%S)
          ELSE
             OutArr(KK)%S = STR_ARR(KK)%S
          END IF
       End Do
       OutArr(LStr)%S = SET_PATH(STR_ARR_1(2)%S)
    Else
       Do KK = 1, LStr
          If(KK == NF) Then
             OutArr(KK)%S = SET_PATH(STR_ARR(KK)%S)
          Else
             OutArr(KK)%S = STR_ARR(KK)%S
          End If
       End Do
    End IF
    
    CALL ST_TRACK (R_Name, 3, 'O')
    Return
  End Subroutine Read_LineCDF
!***************************************************************************
  SUBROUTINE READ_FILE (N_COLS, FileName, DATA_R, *, XS, XE, DX, &
       NPT, I_XC, XAX, CDF, Skip_line)
    ! Optional parameter I_XC is used only if XAX parameter is not
    ! specified.
    
    USE LOG_PARS,    ONLY: LOG_LEVEL, N_SSO, EU, LUNIT
    USE TYPE_VARS,   ONLY: KND

    INTEGER    N_COLS, NPT, I_XC, Skip_line
    CHARACTER (LEN = *) FileName, CDF
    REAL (KND) XS, XE, DX
    REAL (KND), ALLOCATABLE :: DATA_R(:,:), XAX(:)
    OPTIONAL :: XAX, DX, NPT, CDF, I_XC, XS, XE, Skip_line

    ! Local variables:
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_FILE'
    CHARACTER (LEN = 1)  HEADERLINE
    CHARACTER (LEN = :), ALLOCATABLE :: M_TYPE_1, File, Msg
    CHARACTER (LEN = 200) Line
    INTEGER    N_READ, II, NP_M, K_VAL, C_UNIT, NCL, IXC, NPS, S1, S2
    LOGICAL    CDF_FLG, CHECK_SE, EndFlg
    REAL (KND) X_S, X_E, DD, D_VAL
    REAL (KND), ALLOCATABLE :: ARR_READ(:)
    TYPE Linked_List_Entry
       REAL (KND), ALLOCATABLE :: ARR(:)
       TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
    END TYPE Linked_List_Entry
    TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')

    ! If XAX is written in a separate variable (optional parameter
    ! XAX is specified), the XAX is always assumed to be the first
    ! column in the data file. I_XC parameter is ignored in this case
    IXC = 1
    ! N_COLS is always the number of columns in the DATA_R array
    ! If XAX is specified independently we have to read one column more
    IF(PRESENT(XAX)) THEN
       NCL = N_COLS + 1
       IF(ALLOCATED(XAX)) DEALLOCATE (XAX) 
    ELSE
       NCL = N_COLS
       IF(PRESENT(I_XC)) IXC = I_XC
    END IF
    ALLOCATE(ARR_READ(NCL))
    IF(ALLOCATED(DATA_R)) DEALLOCATE (DATA_R)
   
    IF(PRESENT(NPT)) THEN
       NP_M = NPT 
    ELSE
       NP_M = 0       
    END IF

    ! Both start and end points must be defined, otherwise the
    ! input is ignored
    IF(PRESENT(XS).AND.PRESENT(XE)) THEN
       CHECK_SE = .TRUE.
       IF(PRESENT(DX)) THEN
          X_S = XS - DX ;  X_E = XE + DX 
       ELSE
          X_S = XS ;  X_E = XE
       END IF
    ELSE
       CHECK_SE = .FALSE.
    END IF
                
    CDF_FLG = .False.
    If(Present(CDF)) THEN
       If(LEN_TRIM(CDF) > 0) CDF_FLG = .TRUE.
    End If
    
    If(CDF_FLG) THEN
       File = CDF
    Else
       File = FileName
    End If
        
    Msg = ""
    IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//FILE
    OPEN(NewUNIT=C_UNIT, FILE = FILE, STATUS = 'old', ERR = 115)
    If(CDF_FLG) THEN
       IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) &
            ' *** Looking for section: '//TRIM(FileName)
       Line = ''
       DO 
          Msg = ": Looking for a starting line"
          READ(C_UNIT,*,END=111) Line
          IF(TRIM(Line) == '>>>>>'//TRIM(FileName)) EXIT
       END DO
    End If

    HEADERLINE = '#'
    DO WHILE (ANY(HEADERLINE == [';', '#', '*', '>']))
       Msg = ": Analysing header"
       READ(C_UNIT,*,END=111) HEADERLINE
    END DO
    BACKSPACE(C_UNIT) 
    IF(PRESENT(Skip_line)) THEN
       DO II = 1, Skip_line
          READ(C_UNIT,*) 
       ENDDO
    ENDIF
    ! Skip lines which have less columns than requested
    Do
       Read(C_UNIT, *, ERR = 99, END = 111) ARR_READ
       Backspace(C_UNIT)
       Exit
99     Continue
    End Do
       
    ! If start/end points are defined, skip the beginning lines of the 
    ! file which are outside the selected range
    N_READ  = 0
    IF(CHECK_SE) THEN
       DO
          Msg = ": Skipping starting or ending grid points"
          READ(C_UNIT, *, ERR = 113, END = 111) ARR_READ
          N_READ = N_READ + 1
          If(N_READ  == 1) Then
             ! Fist we define at which side of X_S and X_E the input grid beging
             S1 = Int(Sign(1.0D0,(ARR_READ(IXC) - X_S))) + Int(Sign(1.0D0,(ARR_READ(IXC) - X_E)))
          Else
             ! Then we check if the side changes and exit if this is the case
             S2 = Int(Sign(1.0D0,(ARR_READ(IXC) - X_S))) + Int(Sign(1.0D0,(ARR_READ(IXC) - X_E)))
             IF(S1 /= S2) Exit
          End If
          ! Otherwise we exit if the value is between X_S and X_E
          IF(ARR_READ(IXC) >= X_S.AND.ARR_READ(IXC) <= X_E) EXIT
       END DO
    END IF

    NPS = 0
    DO II = 0, NP_M
       IF(N_READ - II == 0) EXIT
       BACKSPACE (C_UNIT)
       NPS = NPS + 1
    END DO

    II = 0
    N_READ = 0
    EndFlg = .False. 
    Read_Data: DO 
       Read_Line: Block
         READ(C_UNIT, *, ERR=100, END=101) ARR_READ
         Exit Read_Line
100      Continue
         Backspace (C_UNIT)
         Msg = ": Looking for an ending line"
         READ(C_UNIT,*,END=111) Line
         IF(TRIM(Line) /= '<<<<<'//TRIM(FileName)) GoTo 113
101      Continue
    
         EXIT Read_Data
!         IF(.NOT.CHECK_SE) EXIT Read_Data
!         GoTo 111
       END Block Read_Line

       ! Do not check for the first points which are still outside
       ! the interval
       IF(CHECK_SE) Then
          IF(N_READ >= NPS) Then
             IF(ARR_READ(IXC) > X_E.OR.ARR_READ(IXC) < X_S) THEN
                IF(NP_M == 0) Exit
                II = II + 1
                IF(II == NP_M) EndFlg = .True.
             END IF
          End If
       End If
       
       IF (.NOT.ASSOCIATED(ELEMENT)) THEN
          ALLOCATE(FIRST_ELEM)
          ELEMENT => FIRST_ELEM
          N_READ = 1
       ELSE
          ALLOCATE(ELEMENT%NEXT_ELEM)
          ELEMENT => ELEMENT%NEXT_ELEM
          N_READ = N_READ + 1
       ENDIF
       
       ALLOCATE (ELEMENT%ARR(NCL))
       ELEMENT%ARR(:) = ARR_READ(:)
       IF(EndFlg) Exit
    END DO Read_Data
    CLOSE (C_UNIT)

    IF (.NOT.ASSOCIATED(ELEMENT)) GoTo 114 

    NULLIFY(ELEMENT%NEXT_ELEM)
    ALLOCATE(DATA_R(N_READ, N_COLS))
    IF(PRESENT(XAX)) ALLOCATE(XAX(N_READ))
    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       II = II + 1
       IF(PRESENT(XAX)) THEN
          XAX(II) = ELEMENT%ARR(1)
          DATA_R(II,:) = ELEMENT%ARR(2:)
       ELSE
          DATA_R(II,:) = ELEMENT%ARR(:)
       END IF
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT%ARR)
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN

111 CONTINUE
    CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'End of file encountered'//Msg)
    WRITE(EU,*) '    File: '//FILE
    RETURN 1
    
113 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file')
    WRITE(EU,*) '    File: '//FILE
    Call Clean
    RETURN 1
    
114 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'No valid data is found')
    WRITE(EU,*) '    File: '//FILE
    RETURN 1

115 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Data file is not found')
    WRITE(EU,*) '    File: '//FILE
    RETURN 1
  Contains
    Subroutine Clean
      If(.Not.Associated(ELEMENT)) Return
      ELEMENT => FIRST_ELEM
      DO WHILE (ASSOCIATED(ELEMENT))
         NEXT => ELEMENT%NEXT_ELEM
         DEALLOCATE (ELEMENT%ARR)
         DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
         ELEMENT => NEXT
      END DO
    End Subroutine Clean
  END SUBROUTINE READ_FILE

!***************************************************************************
  FUNCTION SET_PATH (INP_DIR)
    USE LOG_PARS, ONLY: Work_Dir
    USE ST_INPUT, ONLY: HOME_DIR
    CHARACTER(LEN = *)  INP_DIR
    CHARACTER(LEN = :), ALLOCATABLE :: SET_PATH
    ! Local variables:
    INTEGER            KK, II, LL

    LL = LEN_TRIM(INP_DIR)
    DO KK = 1, LEN_TRIM(INP_DIR)
       SELECT CASE (IACHAR(INP_DIR(KK:KK))) 
       CASE(32) ! leading space
          II = KK + 1 
       CASE(34, 39) ! ' or "
          II = 2 ;  LL = LL - 1
          EXIT
       CASE DEFAULT
          II = 1 
          EXIT
       END SELECT
    END DO

    SELECT CASE (INP_DIR(II:II))
    CASE ('/')
       SET_PATH = INP_DIR(II:LL)
    CASE ('~')
       SET_PATH = HOME_DIR//INP_DIR(II + 1:LL)
    CASE DEFAULT
       SET_PATH = Work_Dir//INP_DIR(II:LL)
       If(Len(INP_DIR) > II) Then
          If(INP_DIR(II:II + 1) == './') THEN
             If(Len(INP_DIR) > II + 1) THEN
                SET_PATH = Work_Dir//INP_DIR(II + 2:LL)
             ELSE
                SET_PATH = Work_Dir
             END IF
          ELSE
             SET_PATH = Work_Dir//INP_DIR(II:LL)
          END IF
       End If
    END SELECT
    
    RETURN
  END FUNCTION SET_PATH

!***************************************************************************
  FUNCTION NO_PATH (INP_DIR)
    CHARACTER(LEN = *)  INP_DIR
    CHARACTER(LEN = :), ALLOCATABLE :: NO_PATH
    ! Local variables:
    INTEGER            KK

    DO KK = LEN_TRIM(INP_DIR), 1, -1
       IF(INP_DIR(KK:KK) == '/') EXIT
    END DO
    NO_PATH = INP_DIR(KK + 1:)
    
    RETURN
  END FUNCTION NO_PATH

  !***************************************************************************
  SUBROUTINE GET_CONTROL_PARS(INP_STRING, STR_SEG, *)      
    
    USE LOG_PARS,  ONLY: EU
    CHARACTER (LEN = *)  INP_STRING
    TYPE (DYN_CHAR) :: STR_SEG(:) 
    
    ! Local variables
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_CONTROL_PARS'
    CHARACTER (LEN = :), ALLOCATABLE :: STR, MODE
    INTEGER              I, II, LL, NUM, KK, FLAG
    LOGICAL              STAT_ENTRY, STAT_READ, STAT_BLOCK
    LOGICAL              IDX(SIZE(STR_SEG)), IDX_ALL(SIZE(STR_SEG)) 

    LL = LEN_TRIM(INP_STRING) 
    DO II = 1, LL ! Skip leading space and quotation marks (single and double)
       SELECT CASE (IACHAR(INP_STRING(II:II)))
       CASE(32,34,39)      
       CASE DEFAULT 
          EXIT
       END SELECT
    END DO
    
    FORALL(KK = 1:SIZE(STR_SEG)) STR_SEG(KK)%S = ''
    FLAG = 1 ;  IDX(:) = .FALSE. ; IDX_ALL(:) = .FALSE. ; STR = ''
    STAT_ENTRY = .FALSE. ; STAT_READ = .FALSE. ;  STAT_BLOCK = .FALSE.
    DO I = II, LL
       SELECT CASE (IACHAR(INP_STRING(I:I)))
       CASE(32, 44)   ! space and comma  
          IF(STAT_READ.AND.FLAG > 0) STAT_ENTRY = .TRUE.
          IF(STAT_READ.AND.FLAG < 0) STR = STR//INP_STRING(I:I) ! do not skip space and commas in the parameter list
       CASE(58, 59)   ! semi-colon and colon  
          IF(STAT_READ) STAT_ENTRY = .TRUE.
          STAT_BLOCK = .TRUE.
       CASE(34,39) ! quotation marks (single and double)   
       CASE DEFAULT 
          IF(STAT_BLOCK) FLAG = - FLAG  
          STAT_READ = .TRUE. ;  STAT_BLOCK = .FALSE.
          STR = STR//INP_STRING(I:I)
       END SELECT
       
       IF(I == LL) THEN
          IF(FLAG > 0) GOTO 200
          STAT_ENTRY = .TRUE.
       END IF

       IF(.NOT.STAT_ENTRY) CYCLE

       IF(FLAG > 0) THEN
          CALL UPPER_LOWER_CASE (STR,'U2L')
          IF(STR == 'all') THEN
             IDX(:) = .TRUE.
          ELSE 
             READ(STR,*, ERR = 201) NUM
             IDX(NUM) = .TRUE.
          END IF
          WHERE(IDX) IDX_ALL(:) = .TRUE. 
       ELSE
          IF(.NOT.ANY(IDX)) GOTO 202
          FORALL(KK = 1:SIZE(STR_SEG), IDX(KK)) STR_SEG(KK)%S = STR
          IDX(:) = .FALSE.
       END IF
       STAT_ENTRY = .FALSE. ; STAT_READ = .FALSE. ; STR = ''
    END DO

    IF(ANY(.NOT.IDX_ALL)) GOTO 203

    RETURN
    
200 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading control sequence: '// &
         'parameter value is missing')
    RETURN 1
201 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Wrong number in the control '// & 
         'sequence: "'//STR//'"')
    RETURN 1
202 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Wrong input in the control '// & 
         'sequence: '//TRIM(INP_STRING))
    RETURN 1
203 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'One or more input parameters '// & 
         'are missing: '//TRIM(INP_STRING))
    RETURN 1
    
  END SUBROUTINE GET_CONTROL_PARS

!***************************************************************************
  SUBROUTINE SPLIT_STRING (INP_STRING, READ_LIST, SPLIT_STR, DIM)      
    
    USE LOG_PARS,  ONLY: EU
    CHARACTER (LEN = *)  INP_STRING, SPLIT_STR
    INTEGER  DIM
    TYPE (DYN_CHAR), ALLOCATABLE :: READ_LIST(:) 
    OPTIONAL :: SPLIT_STR, DIM

    ! Local variables
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SPLIT_STRING'
    CHARACTER (LEN=:), ALLOCATABLE :: STR, STR_TMP, STR_TMP_1
    INTEGER  I, II, LL, KK, I_INP, LN, SKIP_IDX(2), SPLIT_IDX(2)
    LOGICAL  READ_FLAG, END_FLAG
    TYPE Linked_List_Entry
       CHARACTER (LEN = 200) :: READ_STR
       TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
    END TYPE Linked_List_Entry
    TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT, CURR, PREV
    !==================================================================

!    CALL ST_TRACK (R_NAME, 1, 'I')

    LL = LEN_TRIM(INP_STRING) 
    DO II = 1, LL ! Skip leading space and quotation marks (single and double)
       SELECT CASE (IACHAR(INP_STRING(II:II)))
       CASE(32,34,39)      
       CASE DEFAULT 
          EXIT
       END SELECT
    END DO

    SKIP_IDX = [34, 39] ! quotation marks (single and double)
    IF(PRESENT(SPLIT_STR)) THEN
       SPLIT_IDX = IACHAR(SPLIT_STR)  ! comma, space  
    ELSE
       SPLIT_IDX = [44, 32] ! comma, space  
    END IF
    
    READ_FLAG = .FALSE. ;  END_FLAG = .FALSE. ; STR = ''
    DO I = II, LL
       I_INP = IACHAR(INP_STRING(I:I))
       IF(ALL(SKIP_IDX /= I_INP)) THEN
          IF(ANY(SPLIT_IDX == I_INP)) THEN
             IF(READ_FLAG) THEN
                END_FLAG = .TRUE.
             END IF
          ELSE
             READ_FLAG = .TRUE.
             STR = STR//INP_STRING(I:I)
          END IF
       END IF
       
       IF(I == LL) END_FLAG = .TRUE.

       IF(.NOT.END_FLAG.OR..NOT.READ_FLAG) CYCLE

       IF (.NOT.ASSOCIATED(ELEMENT)) THEN
          ALLOCATE(FIRST_ELEM)
          ELEMENT => FIRST_ELEM
          KK = 1
       ELSE
          ALLOCATE(ELEMENT%NEXT_ELEM)
          ELEMENT => ELEMENT%NEXT_ELEM
          KK = KK + 1
       ENDIF
       ELEMENT%READ_STR = STR    
       READ_FLAG = .FALSE.  ;  END_FLAG  = .FALSE. ;  STR = ''
    END DO
    
    IF(KK == 0) RETURN
    NULLIFY(ELEMENT%NEXT_ELEM)
    
    ! Remove spaces around ":" or "="
    IF(SPLIT_STR /= ':'.AND.SPLIT_STR /= '=') THEN
       ELEMENT => FIRST_ELEM
       DO WHILE (ASSOCIATED(ELEMENT))
          STR_TMP = CUT_STR(ELEMENT%READ_STR)
          IF(ASSOCIATED(ELEMENT%NEXT_ELEM)) THEN
             STR_TMP_1 = CUT_STR(ELEMENT%NEXT_ELEM%READ_STR)
          ELSE
             STR_TMP_1 = ' '
          END IF
          LN = LEN(STR_TMP)
          ! If the first symbol in the next string is ':' or '='
          ! join it to the current one
          ! If the last symbol in the current string is ':' or '='
          ! join it to the next one
          IF (STR_TMP(LN:LN) == ':'.OR.STR_TMP(LN:LN) == '='.OR. &
               STR_TMP_1(1:1) == ':'.OR.STR_TMP_1(1:1) == '=') THEN
             CURR => ELEMENT%NEXT_ELEM
             PREV => ELEMENT
             PREV%READ_STR = STR_TMP//STR_TMP_1
             IF(ASSOCIATED(ELEMENT%NEXT_ELEM)) THEN
                NEXT => CURR%NEXT_ELEM
             ELSE
                NEXT => ELEMENT%NEXT_ELEM ! Not Accociated
             END IF
             DEALLOCATE(ELEMENT%NEXT_ELEM) ; NULLIFY (ELEMENT%NEXT_ELEM)
             ! After joining do one more run with the same element to check
             ! if one more element needs to be joined
             ELEMENT%NEXT_ELEM => NEXT 
             KK = KK - 1
          ELSE
             ELEMENT => ELEMENT%NEXT_ELEM
          END IF
       END DO
    END IF

    IF(ALLOCATED(READ_LIST)) DEALLOCATE(READ_LIST)
    ALLOCATE (READ_LIST(KK))

    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       STR_TMP = CUT_STR(ELEMENT%READ_STR)
       II = II + 1
       READ_LIST(II)%S = CUT_STR(ELEMENT%READ_STR)

       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO

    IF(PRESENT(DIM)) DIM = SIZE(READ_LIST)
    
!    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
 
  END SUBROUTINE SPLIT_STRING

!***************************************************************************
  SUBROUTINE AKIMA_INIT (X_IN, Y_IN, AKI_DAT)

    USE TYPE_VARS, ONLY: KND

    REAL (KND) X_IN(:), Y_IN(:)
    TYPE (AKIMA_VARS) AKI_DAT
    ! Local variables
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'AKIMA_INIT'
    INTEGER  N_IN, I
    REAL (KND)  Hf
    REAL (KND), DIMENSION (SIZE(X_IN) + 4) :: X1, Y1, ST, T
    !==================================================================
    N_IN = SIZE(X_IN)
      
    ALLOCATE(AKI_DAT%A(N_IN), AKI_DAT%B(N_IN), AKI_DAT%C(N_IN), &
         AKI_DAT%D(N_IN), AKI_DAT%X(N_IN + 4))
    
    IF (X_IN(2).GT.X_IN(1)) THEN
       X1(3:N_IN + 2) = X_IN(1:N_IN)
       Y1(3:N_IN + 2) = Y_IN(1:N_IN)
    ELSE
       X1(3:N_IN + 2) = X_IN(N_IN:1:-1)
       Y1(3:N_IN + 2) = Y_IN(N_IN:1:-1)
    ENDIF

    AKI_DAT%A(:) = 0.0D0 ; AKI_DAT%B(:) = 0.0D0 ; AKI_DAT%C(:) = 0.0D0 
    AKI_DAT%D(:) = 0.0D0
    
    X1(2) = X1(3) + X1(4) - X1(5)
    X1(1) = X1(2) + X1(3) - X1(4)

    X1(N_IN + 3) = X1(N_IN + 2) + X1(N_IN + 1) - X1(N_IN)
    X1(N_IN + 4) = X1(N_IN + 3) + X1(N_IN + 2) - X1(N_IN + 1)

    ! Caculation of slopes of curves
    FORALL(I = 3:N_IN + 1) ST(I) = (Y1(I + 1) - Y1(I))/(X1(I + 1) - X1(I))

    ! Calculation of extrapolated y-values left and right
    Y1(2) = (X1(3) - X1(2)) * (ST(4) - 2.0D0*ST(3)) + Y1(3)
    ST(2) = (Y1(3) - Y1(2))/(X1(3) - X1(2))

    Y1(1) = (X1(2) - X1(1)) * (ST(3) - 2.0D0*ST(2)) + Y1(2)
    ST(1) = (Y1(2) - Y1(1))/(X1(2) - X1(1))

    Y1(N_IN + 3) = Y1(N_IN + 2) + &
         (2.0D0*ST(N_IN + 1) - ST(N_IN))*(X1(N_IN + 3) - X1(N_IN + 2))
    ST(N_IN + 2) = (Y1(N_IN + 3) - Y1(N_IN + 2))/(X1(N_IN + 3) - X1(N_IN + 2))

    Y1(N_IN + 4) = Y1(N_IN + 3) + &
         (2.0D0*ST(N_IN + 2) - ST(N_IN + 1))*(X1(N_IN + 4) - X1(N_IN + 3))
    ST(N_IN + 3) = (Y1(N_IN + 4) - Y1(N_IN + 3))/(X1(N_IN + 4) - X1(N_IN + 3))

    ! Calculation of polynomial slopes
    DO I = 3, N_IN + 2
       Hf = DABS(ST(I + 1) - ST(I)) + DABS(ST(I - 1) - ST(I - 2))
       IF( Hf == 0.d0 ) THEN
          T(I) = 0.5D0 * (ST(I + 1) + ST(I))
       ELSE
          T(I) = ( DABS(ST(I + 1) - ST(I))*ST(I - 1) + &
               DABS(ST(I - 1) - ST(I - 2))*ST(I) ) / Hf
       END IF
    END DO

    ! Calculation of polynomial coefficients
    FORALL(I = 1:N_IN - 1)
       AKI_DAT%A(I) = Y1(I + 2)
       AKI_DAT%B(I) = T(I + 2)
       AKI_DAT%C(I) = ( 3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3) )/ &
            ( X1(I + 3) - X1(I + 2) )
       AKI_DAT%D(I) = ( T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2) )/ &
            ( X1(I + 3) - X1(I + 2) )**2
    END FORALL
    
    AKI_DAT%X(:) = X1(:)
    AKI_DAT%A(N_IN) = Y1(N_IN + 2)
    AKI_DAT%B(N_IN) = 0.0D0 ; AKI_DAT%C(N_IN) = 0.0D0; AKI_DAT%D(N_IN) = 0.0D0
    RETURN
  END SUBROUTINE AKIMA_INIT
  !***************************************************************************  

  SUBROUTINE AKIMA_FIN (AKI_DAT, X_OUT, Y_OUT)

    USE TYPE_VARS, ONLY: KND

    REAL (KND) X_OUT(:), Y_OUT(:)
    TYPE (AKIMA_VARS) AKI_DAT
    ! Local variables
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'AKIMA_FIN'
    INTEGER  N_OUT, N_IN, I, IDX
    LOGICAL  MASK(SIZE(AKI_DAT%X) - 4)
    REAL (KND)  DELTA
    !==================================================================
    ! Calculation of interpolated values
    N_OUT = SIZE(X_OUT) ;  N_IN = SIZE(AKI_DAT%X) - 4
    DO I = 1, N_OUT
       MASK(:) =.FALSE.
       WHERE(AKI_DAT%X(3:N_IN + 2) <= X_OUT(I)) MASK(:) =.TRUE.
       IDX = COUNT(MASK)
       IF(IDX == 0 .OR. IDX == N_IN) THEN
          IF(X_OUT(I).EQ.AKI_DAT%X(IDX + 2)) THEN
             Y_OUT(I) = AKI_DAT%A(IDX)
          ELSE
             Y_OUT(I) = 0.0D0
          END IF
          CYCLE
       END IF
       DELTA = X_OUT(I) - AKI_DAT%X(IDX + 2)
       Y_OUT(I) = AKI_DAT%A(IDX) + AKI_DAT%B(IDX)*DELTA + &
            AKI_DAT%C(IDX)*DELTA**2 + AKI_DAT%D(IDX)*DELTA**3
    END DO

    RETURN
  END SUBROUTINE AKIMA_FIN

!*********************************************************************

  SUBROUTINE AVERAGE (WL_1, WL_2, WL_INP, DATA_INP, DATA_OUT, WL_OUT, &
       FLG, MSK_I, MSK_O)
    
    REAL (KND) WL_1(:), WL_2(:), WL_INP(:), DATA_INP(:,:), DATA_OUT(:,:) 
    LOGICAL, OPTIONAL :: FLG(:,:), MSK_I(:), MSK_O(:)
    REAL (KND), OPTIONAL :: WL_OUT(:)
  ! Local parameters
    INTEGER  II, KK, N_AV
    LOGICAL  FLG_SEL(SIZE(WL_INP)), FLG_CHK(SIZE(WL_1),SIZE(DATA_OUT,2))
    LOGICAL  FLG_SEG(SIZE(WL_INP))
    
    IF(PRESENT(FLG)) THEN
       FLG_CHK = FLG
    ELSE
       FLG_CHK = .TRUE.
    END IF

    DATA_OUT = 0.0D0

    DO KK = 1, SIZE(WL_1)
       FLG_SEG =.FALSE.
       WHERE (WL_INP >= MIN(WL_1(KK),WL_2(KK)).AND.&
            WL_INP <= MAX(WL_1(KK),WL_2(KK))) FLG_SEG =.TRUE. 
       ! MSK_I defines points which should be excluded from the averaging
       IF(PRESENT(MSK_I)) THEN
          FLG_SEL = FLG_SEG.AND.MSK_I
       ELSE
          FLG_SEL = FLG_SEG
       END IF
       ! MSK_O is set to .FALSE. is no points to average are found
       IF(PRESENT(MSK_O)) THEN
          IF(ANY(FLG_SEL)) THEN
             MSK_O(KK) = .TRUE. 
          ELSE
             MSK_O(KK) = .FALSE.
          END IF
       END IF
       IF(.NOT.ANY(FLG_SEL)) THEN
          IF(PRESENT(WL_OUT)) WL_OUT(KK) = &
               SUM(WL_INP,FLG_SEG)/FLOAT(COUNT(FLG_SEG))
          CYCLE
       END IF
          
       N_AV = COUNT(FLG_SEL)
       DO II = 1, SIZE(DATA_OUT,2)
          IF(.NOT.FLG_CHK(KK,II)) CYCLE
          DATA_OUT(KK,II) = SUM(DATA_INP(:,II),FLG_SEL)/FLOAT(N_AV)
       END DO

       IF(PRESENT(WL_OUT)) WL_OUT(KK) = SUM(WL_INP,FLG_SEL)/FLOAT(N_AV)
    END DO
  END SUBROUTINE AVERAGE
!*********************************************************************

  SUBROUTINE SMOOTH (DATA, NP, FLG_INP)

    INTEGER  NP
    REAL (KND) DATA(:) 
    LOGICAL, OPTIONAL :: FLG_INP(:)
  ! Local parameters
    INTEGER  II, NPP, I_S, NN
    LOGICAL  FLG(SIZE(DATA))
    REAL (KND), ALLOCATABLE :: DATA_OUT(:)

    NN = FLOOR((NP - 1)/2.) ;  NPP = 2*NN + 1
    
    IF(PRESENT(FLG_INP)) THEN
       FLG = FLG_INP
    ELSE
       FLG = .TRUE.
    END IF

    ALLOCATE(DATA_OUT(SIZE(DATA,1)), SOURCE = DATA)

! Find the first point where the smoothing is to be done
    I_S = NN + 1
    DO WHILE (.NOT.FLG(I_S))
       I_S = I_S + 1
    END DO
    DATA_OUT(I_S) = SUM(DATA(I_S - NN:I_S + NN))

    DO II = I_S + 1, SIZE(DATA) - NN
       IF (.NOT.FLG(II)) THEN
          ! Zero the data point is we do not neet it
          DATA_OUT(II) = 0.0D0
          CYCLE
       END IF
       IF (FLG(II - 1)) THEN
          ! If previous point was calculated
          DATA_OUT(II) = DATA_OUT(II - 1) - DATA(II - NN - 1) + DATA(II + NN)
       ELSE
          DATA_OUT(II) = SUM(DATA(II - NN:II + NN))          
       END IF
    END DO

    DATA = DATA_OUT/DBLE(NPP)

  END SUBROUTINE SMOOTH
!*********************************************************************

  SUBROUTINE ALT_SPLINE_INIT (ARR, SPL_INF, FLG, *)
    USE GRIDS, ONLY: NL => NLAYERS, Z_INV
    USE TYPES, ONLY: KND, SPLINE_INFO
    
    LOGICAL FLG
    REAL (KND)  ARR(:)
    TYPE (SPLINE_INFO) SPL_INF
    ! Local parameters
    integer I
    REAL (KND)  ARR_LC(SIZE(ARR))
    REAL (KND)  WRK(5*(NL+2))
    !==================================================================

    IF(ALLOCATED(SPL_INF%KNT)) DEALLOCATE (SPL_INF%KNT, SPL_INF%BCF) 
    ! Array dimensions must be increased by 2 with respect to values given
    ! in the SLATEC comments. Otherwise a segmentation fault occures.
    ALLOCATE(SPL_INF%KNT(NL + 6), SPL_INF%BCF(NL + 2))

    IF(FLG) THEN
       ARR_LC = DLOG(ARR(NL:1:-1))
    ELSE
       ARR_LC = ARR(NL:1:-1)
    END IF

    CALL DBINT4 (Z_INV, ARR_LC, NL, 2, 2, 0.0D0, 0.0D0, 1, SPL_INF%KNT, &
         SPL_INF%BCF, SPL_INF%NC, SPL_INF%ORD, WRK, *880)

    RETURN 
880 Return 1
  END SUBROUTINE ALT_SPLINE_INIT
!*********************************************************************

  SUBROUTINE ALT_SPLINE (ARR, X, SPL_INF, FLG, *)

    USE TYPES, ONLY: KND, SPLINE_INFO

    LOGICAL FLG
    REAL (KND)  ARR(:), X(:)
    TYPE (SPLINE_INFO) SPL_INF
    ! Local parameters
    INTEGER II, INBV
    REAL (KND)  WRK(3*SPL_INF%ORD)
!    DOUBLE PRECISION, EXTERNAL ::  DBVALU  

    INBV = 1
    DO II = 1, SIZE(ARR)
       Call DBVALU (SPL_INF%KNT, SPL_INF%BCF, SPL_INF%NC, SPL_INF%ORD, &
            0, X(II), INBV, WRK, ARR(II), *880)
    END DO
    IF(FLG) ARR = DEXP(ARR)

    Return
880 Return 1

  END SUBROUTINE ALT_SPLINE
!***************************************************************************

  SUBROUTINE INIT_PARS
    USE INFO,     ONLY: RET_INFO
    USE RTM_PARS, ONLY: RTM_ST, MAX_ITER

    ! Local parameters
    INTEGER I_P, I_RI, I_PARS, I_SBP, N_PARS, N_SBP
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'INIT_PARS'
    !================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')

    DO I_P = 1, SIZE(RTM_ST) 
      IF(ALLOCATED(RET_INFO)) THEN
          I_RI = FIND_IDX(RTM_ST(I_P)%NAME,RET_INFO%NAME)
       ELSE
          I_RI = -1
       END IF

       IF(I_RI > 0) THEN
          N_SBP = RET_INFO(I_RI)%N_SBP*RET_INFO(I_RI)%N_SBP
          N_PARS = RET_INFO(I_RI)%N_SBP*RET_INFO(I_RI)%N_PARS
       ELSE
          N_SBP = 1
          N_PARS = 1
       END IF
       ALLOCATE (RTM_ST(I_P)%SPR(N_PARS,N_SBP))
       ! Sub-parameters, both altitude and spectral
       DO I_SBP = 1, N_SBP 
          DO I_PARS = 1, N_PARS 
             ALLOCATE (RTM_ST(I_P)%SPR(I_PARS,I_SBP)%DAT(MAX_ITER), &
                  RTM_ST(I_P)%SPR(I_PARS,I_SBP)%IND(MAX_ITER))
             RTM_ST(I_P)%SPR(I_PARS,I_SBP)%N_SBP = I_SBP
             RTM_ST(I_P)%SPR(I_PARS,I_SBP)%N_PARS = I_PARS
             RTM_ST(I_P)%SPR(I_PARS,I_SBP)%IND = 1
          END DO
       END DO
    END DO
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END SUBROUTINE INIT_PARS
  
!***************************************************************************
  Subroutine ReInitPar (Name)
    USE INFO,     ONLY: RET_INFO
    USE RTM_PARS, ONLY: RTM_ST, MAX_ITER
    
    CHARACTER (LEN = *) Name
    ! Local parameters
    INTEGER I_P, I_RI, I_PARS, I_SBP, N_PARS, N_SBP
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'INIT_PARS'
    !================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')

    I_P = FIND_IDX(Name, RTM_ST%NAME)
    If(I_P < 1) Return
    I_RI = FIND_IDX(RTM_ST(I_P)%NAME,RET_INFO%NAME)
    If(I_RI < 1) Return
    
    N_SBP = RET_INFO(I_RI)%N_SBP*RET_INFO(I_RI)%N_SBP
    N_PARS = RET_INFO(I_RI)%N_SBP*RET_INFO(I_RI)%N_PARS
    Deallocate(RET_INFO(I_RI)%VSI)
    ALLOCATE(RET_INFO(I_RI)%VSI(N_SBP))
    RET_INFO(I_RI)%VSI(:) = RET_INFO(I_RI)%VS

    ! Sub-parameters, both altitude and spectral
    DO I_SBP = 1, Size(RTM_ST(I_P)%SPR,2) 
       DO I_PARS = 1, Size(RTM_ST(I_P)%SPR,1) 
          DeAllocate(RTM_ST(I_P)%SPR(I_PARS,I_SBP)%DAT, &
               RTM_ST(I_P)%SPR(I_PARS,I_SBP)%IND)
       END DO
    END DO
    DeAllocate(RTM_ST(I_P)%SPR)
    
    ALLOCATE (RTM_ST(I_P)%SPR(N_PARS,N_SBP))
    DO I_SBP = 1, N_SBP 
       DO I_PARS = 1, N_PARS
          ALLOCATE (RTM_ST(I_P)%SPR(I_PARS,I_SBP)%DAT(MAX_ITER), &
               RTM_ST(I_P)%SPR(I_PARS,I_SBP)%IND(MAX_ITER))
          RTM_ST(I_P)%SPR(I_PARS,I_SBP)%N_SBP = I_SBP
          RTM_ST(I_P)%SPR(I_PARS,I_SBP)%N_PARS = I_PARS
          RTM_ST(I_P)%SPR(I_PARS,I_SBP)%IND = 1
       END DO
    END DO
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END Subroutine  ReInitPar
  
!***************************************************************************
  SUBROUTINE SET_PTR (PTR, PAR_NAME, ISSP, IASP, ITR, UPD_IND, LAST_RET)
    USE RETR_ITER, ONLY: ITR_N => ITERATION
    USE RTM_PARS,  ONLY: RTM_ST, DATA_ITER
    TYPE (DATA_ITER), POINTER :: PTR
    CHARACTER (LEN = *) PAR_NAME
    INTEGER, OPTIONAL:: ISSP, IASP, ITR
    LOGICAL, OPTIONAL:: UPD_IND, LAST_RET
    ! Local parameters
    INTEGER I_P, I_SBP, I_SSP, I_ASP, I_ITR, II, NDIM
    LOGICAL UPDATE_IND, FLG_LAST_RET, ALLOC_FLG
    !================================================================
    
    I_SSP = 1 ;  I_ASP = 1 ;  I_ITR = ITR_N
    UPDATE_IND = .FALSE.; FLG_LAST_RET = .FALSE.
    IF(PRESENT(ISSP)) I_SSP = ISSP
    IF(PRESENT(IASP)) I_ASP = IASP
    IF(PRESENT(ITR)) I_ITR = ITR
    IF(PRESENT(UPD_IND)) UPDATE_IND = UPD_IND
    IF(PRESENT(LAST_RET)) FLG_LAST_RET = LAST_RET
    
    I_P = FIND_IDX(PAR_NAME, RTM_ST%NAME)

    IF(FLG_LAST_RET) THEN
       II =  RTM_ST(I_P)%SPR(I_SSP,I_ASP)%IND(I_ITR)
       PTR => RTM_ST(I_P)%SPR(I_SSP,I_ASP)%DAT(II)
    ELSE
       PTR => RTM_ST(I_P)%SPR(I_SSP,I_ASP)%DAT(I_ITR)
    END IF

    ALLOC_FLG = .FALSE.
    IF(.NOT.ALLOCATED(PTR%CRN)) THEN
       ALLOC_FLG = .TRUE.
    ELSE
       IF(RTM_ST(I_P)%VS == 'V') THEN
          IF(SIZE(PTR%CRN) /= SIZE(PTR%GRD)) THEN
             ALLOC_FLG = .TRUE.
             DEALLOCATE(PTR%APR, PTR%CRN, PTR%NXT, PTR%PrcA, PTR%PrcR)
          END IF
       END IF
    END IF
       
    IF(ALLOC_FLG) THEN
       IF(RTM_ST(I_P)%VS == 'V') THEN
          NDIM = SIZE(PTR%GRD)
       ELSE
          NDIM = 1
       END IF
       ALLOCATE(PTR%APR(NDIM), PTR%CRN(NDIM), PTR%NXT(NDIM), PTR%PrcA(NDIM), &
            PTR%PrcR(NDIM))
       PTR%CRN = 0.d0 ; PTR%APR = 0.d0 ; PTR%NXT = 0.d0 
       PTR%PrcA = 0.d0 ; PTR%PrcR = 0.d0 
    ENDIF

    IF(UPDATE_IND) RTM_ST(I_P)%SPR(I_SSP,I_ASP)%IND(I_ITR:) = I_ITR

    RETURN
  END SUBROUTINE SET_PTR

!***************************************************************************
  SUBROUTINE READ_ONE_LINE (IUNIT, STRING, FLG_EOF)
    
    INTEGER            IUNIT, IOSTAT
    LOGICAL            FLG_EOF
    CHARACTER (LEN=1)  CHAR
    CHARACTER (LEN=:), ALLOCATABLE :: STRING
    OPTIONAL :: FLG_EOF
    !==================================================================
    
    IF(PRESENT(FLG_EOF)) THEN
       READ(IUNIT, '(A1)', ADVANCE = 'no', IOSTAT = IOSTAT) CHAR
       IF(IOSTAT /= 0) THEN
          FLG_EOF = .TRUE.
          RETURN
       ELSE
          FLG_EOF = .FALSE.
       END IF
       STRING = CHAR
    ELSE
       STRING = ''
    END IF

    DO
       READ(IUNIT, '(A1)', ADVANCE = 'no', IOSTAT = IOSTAT) CHAR
       IF(IOSTAT /= 0) EXIT
       STRING = STRING//CHAR
    END DO
    
  END SUBROUTINE READ_ONE_LINE

!***************************************************************************
  SUBROUTINE HEIGHT_GRID_V2 (I_FINE_INP, CVA, TH, Z, ALT_FLAG, J_IDX, NTH, &
       NLS, ZLS, ISEL, IOB, TH_FLAG, FP_FLG, FINE_MODE)
    
    USE GRIDS,      ONLY: NLAYERS, ZZZ, INTERP_IDX, NL_FINE, Z_FINE, Z_BOA
    USE SPHER_PARS, ONLY: NN, N_FINE_TH, N_FINE_ST_INP => N_FINE_ST

    INTEGER  I_FINE_INP, J_IDX, NLS, NTH, NN_INP
    INTEGER, ALLOCATABLE :: ISEL(:), IOB(:)
    LOGICAL  TH_FLAG, ALT_FLAG, FP_FLG, FINE_FLG
    CHARACTER (LEN=*) FINE_MODE
    REAL (KND)  CVA, TH, Z
    REAL (KND), ALLOCATABLE :: ZLS(:)
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'HEIGHT_GRID_V2'
    INTEGER     ITH, J, K, II, KK, FL_BELOW, M_FINE, J_OUT, NP, &
         N_FINE_ST_REST, FL_TH, L_FINE, I_FP, K_FINE, I_FINE, N_FINE_ST 
    REAL (KND)  DD, H_0
    REAL (KND), PARAMETER:: D_MIN = 1.0D-5
    TYPE Linked_List_Entry
       INTEGER    ISEL, NUM
       REAL (KND) ZLS
       TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
    END TYPE Linked_List_Entry
    TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    OPTIONAL :: FINE_MODE
    !==================================================================
    !    CALL ST_TRACK (R_NAME, 1, 'I')
    H_0 = TH
    IF(PRESENT(FINE_MODE)) THEN
       SELECT CASE(FINE_MODE)
       CASE ('all')
          I_FINE = 1 ;  N_FINE_ST = 0
       CASE ('none')
          I_FINE = NLAYERS ;  N_FINE_ST = 0
       CASE DEFAULT
          I_FINE = I_FINE_INP
          N_FINE_ST = N_FINE_ST_INP
       END SELECT
    ELSE
       I_FINE = I_FINE_INP
       N_FINE_ST = N_FINE_ST_INP
    END IF
    ! J_IDX is the index of the original grid that matches the input altitude
    ! or zero if they do not match
    J = J_IDX ;  J_OUT = 0 ;  FP_FLG = .FALSE.
    IF(DABS(CVA).LT.1.0D-7) THEN
       CVA = 0.0D0 ;  H_0 = Z
    END IF
    
    Locate_First_Point: Block
      ! Only needed for a user defined altitude
      ! FL_BELOW is a level of the fine grid which is stiktly below the
      ! input altitude
      IF(.NOT.ALT_FLAG) EXIT Locate_First_Point         
      IF(Z.LE.Z_FINE(1)) THEN
         Z = Z_FINE(1) ;  I_FP = NLAYERS + 1 ; FL_BELOW = 0
         EXIT Locate_First_Point 
      END IF
      IF(Z.GE.Z_FINE(NL_FINE)) THEN
         Z = Z_FINE(NL_FINE) ;  I_FP = 2 ;  FL_BELOW = NL_FINE - 1 
         EXIT Locate_First_Point 
      END IF
      
      ! Find the first original grid level below the selected altitude
      DO II = NL_FINE, 1, -NN    
         IF(Z_FINE(II) < Z) EXIT 
      END DO
      ! The number of original grid level below the selected altitude 
      I_FP = NLAYERS - (II - 1)/NN
      
      ! Find the first fine grid level below the selected altitude
      DO FL_BELOW = II + NN, II, -1 
         IF(Z_FINE(FL_BELOW) < Z) EXIT 
      END DO
      
      DD = DABS(Z_FINE(FL_BELOW + 1) - Z_FINE(FL_BELOW))*D_MIN
      ! Check if selected altutude is too close to grid level
      IF(Z - Z_FINE(FL_BELOW) < DD) THEN
         Z = Z_FINE(FL_BELOW) ;  FL_BELOW = FL_BELOW - 1
         EXIT Locate_First_Point 
      END IF
      IF(Z_FINE(FL_BELOW + 1) - Z.LT.DD) THEN
         Z = Z_FINE(FL_BELOW + 1)
         EXIT Locate_First_Point 
      END IF      
      FP_FLG = .TRUE.
      J_OUT = I_FP
    End Block Locate_First_Point
    
    NP = 0    
    ! Initialize storage list, first element is not used to avoid
    ! special treatment in AddElement procedure
    ALLOCATE(FIRST_ELEM) ;  ELEMENT => FIRST_ELEM ;  ELEMENT%NUM = NP 
    
    ! Positive CVA are for line of sights pointing downwards, i.e.
    ! they are negative for angles < 90 deg
    ViewDir: Block
      ! Upwards pointing line-of-sights 
      IF(CVA <= 0.0D0) THEN         
         IF(ALT_FLAG) THEN 
            IF(FP_FLG) Call AddElement (Z, FL_BELOW, NP)
            J = I_FP - 1
            DO KK = FL_BELOW + 1, (NLAYERS - J)*NN 
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF
         
         K_FINE = NLAYERS - MIN(I_FINE, MAX(1,J - N_FINE_ST)) + 1

         DO II = NLAYERS - J + 1, K_FINE - 1
            DO K = 1, NN
               KK = (II - 1)*NN + K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         
         DO II = K_FINE, NLAYERS
            KK = (II - 1)*NN + 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         NTH = NP ;  TH_FLAG = .FALSE.
         EXIT ViewDir
      END IF
      
      ! Downwards pointing line-of-sights 
      IF(FP_FLG) THEN
         IF(DABS(H_0 - Z) < D_MIN) THEN
            H_0 = Z ;  FP_FLG = .FALSE.
         END IF
      END IF
      
      IF(H_0 >= Z_BOA) THEN ! LOS doesn't cross the Earth's surface
         TH_FLAG = .TRUE.
         
         DO II = NL_FINE, 1, -NN
            IF(Z_FINE(II) <= H_0) EXIT
         END DO
         ! Find the first fine grid level below or at the tangent height 
         DO FL_TH = MIN(NL_FINE,II + NN), II, - 1 
            IF(Z_FINE(FL_TH) <= H_0) EXIT 
         END DO
         
         ! Check if tangent height is too close to grid level
         IF(H_0 - Z_FINE(FL_TH).LT.D_MIN) THEN
            H_0 = Z_FINE(FL_TH)
            TH_FLAG = .FALSE.
         ELSE IF(Z_FINE(FL_TH + 1) - H_0.LT.D_MIN) THEN
            H_0 = Z_FINE(FL_TH + 1)
            FL_TH = FL_TH + 1
            TH_FLAG = .FALSE.
         END IF
         ITH = NLAYERS - INTERP_IDX(FL_TH) + 1
         
         IF(ALT_FLAG) THEN 
            IF(FP_FLG) THEN
               Call AddElement (Z, FL_BELOW, NP)
            ELSE
               Call AddElement (Z, FL_BELOW + 1, NP)
            END IF
            
            J = I_FP
            DO KK = FL_BELOW, MAX(FL_TH + 1,(NLAYERS - J)*NN + 2), -1
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF
         
         L_FINE = MIN(I_FINE,ITH - N_FINE_TH - 1)
         K_FINE = MIN(L_FINE,MIN(NLAYERS,J + N_FINE_ST))
         
         DO II = J, K_FINE - 1
            DO K = 0, NN - 1
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         DO II = MAX(J,K_FINE), MAX(J,L_FINE) - 1
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         DO KK = (NLAYERS - MAX(J,L_FINE))*NN + 1, FL_TH + 1, - 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         Call AddElement (H_0, FL_TH, NP)
         NTH = NP
         
         N_FINE_ST_REST = IDIM(N_FINE_ST, ITH - J)
         M_FINE = MIN(L_FINE,ITH - N_FINE_ST_REST - 1)
         
         DO KK = FL_TH + 1, (NLAYERS - MAX(M_FINE,1))*NN + 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         DO II = M_FINE - 1, 1, -1
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
      ELSE  ! LOS cross the Earth's surface

         IF(ALT_FLAG) THEN
            IF(FP_FLG) THEN
               Call AddElement (Z, FL_BELOW, NP)
            ELSE
               Call AddElement (Z, FL_BELOW + 1, NP)
            END IF
            
            J = I_FP
            DO KK = FL_BELOW, (NLAYERS - J)*NN + 2, -1
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF
         
         K_FINE = MIN(I_FINE,MIN(NLAYERS,J + N_FINE_ST))
         
         KK = NL_FINE - (J - 1)*NN
         Call AddElement (Z_FINE(KK), KK, NP)
         DO II = J, K_FINE - 1
            DO K = 1, NN
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         DO II = MAX(J,K_FINE) + 1, MAX(J,I_FINE)
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         DO II = MAX(J,I_FINE), NLAYERS - 1
            DO K = 1, NN
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         
         NTH = NP ;  TH_FLAG = .FALSE.
      END IF
    End Block ViewDir

!    WRITE(*,*) '      in V2 NTH = ', NTH

    ! NLS - number of points along LOS
    ! NTH - number of points along LOS before the tangent height
    ! (NTH = NLS if tangent height is not in the pointing direction) 
    NLS = NP
    NULLIFY (ELEMENT%NEXT_ELEM)
    ALLOCATE(ZLS(NLS), ISEL(NLS), IOB(NLS))
    ELEMENT => FIRST_ELEM
    ! Skip the first element as is it not filled
    NEXT => ELEMENT%NEXT_ELEM
    DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT) ;  ELEMENT => NEXT
    DO WHILE (ASSOCIATED(ELEMENT))
       II = ELEMENT%NUM
       ZLS(II) = ELEMENT%ZLS
       ISEL(II) = ELEMENT%ISEL
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO

    IOB(:) = NLAYERS - INTERP_IDX(ISEL(1:NLS)) + 1
    IF(FP_FLG) IOB(1) = J_OUT
    IF(TH_FLAG) IOB(NTH) = ITH

!    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  CONTAINS
!***************************************************************************
    Subroutine AddElement (ZLS, IDX, COUNTER)
      INTEGER IDX, COUNTER
      REAL (KND) ZLS  
      !==================================================================
      ALLOCATE(ELEMENT%NEXT_ELEM)
      ELEMENT => ELEMENT%NEXT_ELEM
      COUNTER = COUNTER + 1
      ELEMENT%ZLS = ZLS
      ELEMENT%ISEL = IDX
      ELEMENT%NUM = COUNTER
    End Subroutine AddElement
  END SUBROUTINE HEIGHT_GRID_V2
  
!***************************************************************************
  SUBROUTINE HEIGHT_GRID_V3 (I_FINE_INP, CVA, TH, Z, ALT_FLAG, J_IDX, NTH, &
       NLS, ZLS, ISEL, IOB, TH_FLAG, FP_FLG, FINE_MODE)
    
    USE GRIDS,      ONLY: NLAYERS, ZZZ, INTERP_IDX, NL_FINE, Z_FINE, Z_BOA
    USE SPHER_PARS, ONLY: NN, N_FINE_TH, N_FINE_ST_INP => N_FINE_ST

    INTEGER  I_FINE_INP, J_IDX, NLS, NTH, NN_INP
    INTEGER, ALLOCATABLE :: ISEL(:), IOB(:)
    LOGICAL  TH_FLAG, ALT_FLAG, FP_FLG, FINE_FLG
    CHARACTER (LEN=*) FINE_MODE
    REAL (KND)  CVA, TH, Z
    REAL (KND), ALLOCATABLE :: ZLS(:)
    
    ! Local parameters
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'HEIGHT_GRID_V3'
    INTEGER     ITH, J, K, II, KK, FL_BELOW, M_FINE, J_OUT, NP, &
         N_FINE_ST_REST, FL_TH, L_FINE, I_FP, K_FINE, I_FINE, N_FINE_ST
    INTEGER  IdxLoc(2*NL_FINE + 1)
    REAL (KND)  DD, H_0
    REAL (KND), PARAMETER:: D_MIN = 1.0D-5
    REAL (KND) ZLoc(2*NL_FINE + 1)
!    TYPE Linked_List_Entry
!       INTEGER    ISEL, NUM
!       REAL (KND) ZLS
!       TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
!    END TYPE Linked_List_Entry
!    TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
!    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    OPTIONAL :: FINE_MODE
    !==================================================================
    !    CALL ST_TRACK (R_NAME, 1, 'I')
    H_0 = TH
    IF(PRESENT(FINE_MODE)) THEN
       SELECT CASE(FINE_MODE)
       CASE ('all')
          I_FINE = 1 ;  N_FINE_ST = 0
       CASE ('none')
          I_FINE = NLAYERS ;  N_FINE_ST = 0
       CASE DEFAULT
          I_FINE = I_FINE_INP
          N_FINE_ST = N_FINE_ST_INP
       END SELECT
    ELSE
       I_FINE = I_FINE_INP
       N_FINE_ST = N_FINE_ST_INP
    END IF
    ! J_IDX is the index of the original grid that matches the input altitude
    ! or zero if they do not match
    J = J_IDX ;  J_OUT = 0 ;  FP_FLG = .FALSE.
    IF(DABS(CVA).LT.1.0D-7) THEN
       CVA = 0.0D0 ;  H_0 = Z
    END IF
    
    Locate_First_Point: Block
      ! Only needed for a user defined altitude
      ! FL_BELOW is a level of the fine grid which is stiktly below the
      ! input altitude
      IF(.NOT.ALT_FLAG) EXIT Locate_First_Point         
      IF(Z.LE.Z_FINE(1)) THEN
         Z = Z_FINE(1) ;  I_FP = NLAYERS + 1 ; FL_BELOW = 0
         EXIT Locate_First_Point 
      END IF
      IF(Z.GE.Z_FINE(NL_FINE)) THEN
         Z = Z_FINE(NL_FINE) ;  I_FP = 2 ;  FL_BELOW = NL_FINE - 1 
         EXIT Locate_First_Point 
      END IF
      
      ! Find the first original grid level below the selected altitude
      DO II = NL_FINE, 1, -NN    
         IF(Z_FINE(II) < Z) EXIT 
      END DO
      ! The number of original grid level below the selected altitude 
      I_FP = NLAYERS - (II - 1)/NN
      
      ! Find the first fine grid level below the selected altitude
      DO FL_BELOW = II + NN, II, -1 
         IF(Z_FINE(FL_BELOW) < Z) EXIT 
      END DO
      
      DD = DABS(Z_FINE(FL_BELOW + 1) - Z_FINE(FL_BELOW))*D_MIN
      ! Check if selected altutude is too close to grid level
      IF(Z - Z_FINE(FL_BELOW) < DD) THEN
         Z = Z_FINE(FL_BELOW) ;  FL_BELOW = FL_BELOW - 1
         EXIT Locate_First_Point 
      END IF
      IF(Z_FINE(FL_BELOW + 1) - Z.LT.DD) THEN
         Z = Z_FINE(FL_BELOW + 1)
         EXIT Locate_First_Point 
      END IF      
      FP_FLG = .TRUE.
      J_OUT = I_FP
    End Block Locate_First_Point
    
    NP = 0    
    ! Initialize storage list, first element is not used to avoid
    ! special treatment in AddElement procedure
!    ALLOCATE(FIRST_ELEM) ;  ELEMENT => FIRST_ELEM ;  ELEMENT%NUM = NP 
    
    ! Positive CVA are for line of sights pointing downwards, i.e.
    ! they are negative for angles < 90 deg
    ViewDir: Block
      ! Upwards pointing line-of-sights 
      IF(CVA <= 0.0D0) THEN         
         IF(ALT_FLAG) THEN 
            IF(FP_FLG) Call AddElement (Z, FL_BELOW, NP)
            J = I_FP - 1
            DO KK = FL_BELOW + 1, (NLAYERS - J)*NN 
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF
         
         K_FINE = NLAYERS - MIN(I_FINE, MAX(1,J - N_FINE_ST)) + 1

         DO II = NLAYERS - J + 1, K_FINE - 1
            DO K = 1, NN
               KK = (II - 1)*NN + K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         
         DO II = K_FINE, NLAYERS
            KK = (II - 1)*NN + 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         NTH = NP ;  TH_FLAG = .FALSE.
         EXIT ViewDir
      END IF
      
      ! Downwards pointing line-of-sights 
      IF(FP_FLG) THEN
         IF(DABS(H_0 - Z) < D_MIN) THEN
            H_0 = Z ;  FP_FLG = .FALSE.
         END IF
      END IF

      IF(H_0 >= Z_BOA) THEN ! LOS doesn't cross the Earth's surface
         TH_FLAG = .TRUE.
         
         DO II = NL_FINE, 1, -NN
            IF(Z_FINE(II) <= H_0) EXIT
         END DO
         ! Find the first fine grid level below or at the tangent height 
         DO FL_TH = MIN(NL_FINE,II + NN), II, - 1 
            IF(Z_FINE(FL_TH) <= H_0) EXIT 
         END DO
         
         ! Check if tangent height is too close to grid level
         IF(H_0 - Z_FINE(FL_TH).LT.D_MIN) THEN
            H_0 = Z_FINE(FL_TH)
            TH_FLAG = .FALSE.
         ELSE IF(Z_FINE(FL_TH + 1) - H_0.LT.D_MIN) THEN
            H_0 = Z_FINE(FL_TH + 1)
            FL_TH = FL_TH + 1
            TH_FLAG = .FALSE.
         END IF
         ITH = NLAYERS - INTERP_IDX(FL_TH) + 1
         
         IF(ALT_FLAG) THEN 
            IF(FP_FLG) THEN
               Call AddElement (Z, FL_BELOW, NP)
            ELSE
               Call AddElement (Z, FL_BELOW + 1, NP)
            END IF
            
            J = I_FP
            DO KK = FL_BELOW, MAX(FL_TH + 1,(NLAYERS - J)*NN + 2), -1
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF
         
         L_FINE = MIN(I_FINE,ITH - N_FINE_TH - 1)
         K_FINE = MIN(L_FINE,MIN(NLAYERS,J + N_FINE_ST))
         
         DO II = J, K_FINE - 1
            DO K = 0, NN - 1
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         DO II = MAX(J,K_FINE), MAX(J,L_FINE) - 1
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         DO KK = (NLAYERS - MAX(J,L_FINE))*NN + 1, FL_TH + 1, - 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         Call AddElement (H_0, FL_TH, NP)
         NTH = NP
         
         N_FINE_ST_REST = IDIM(N_FINE_ST, ITH - J)
         M_FINE = MIN(L_FINE,ITH - N_FINE_ST_REST - 1)
         
         DO KK = FL_TH + 1, (NLAYERS - MAX(M_FINE,1))*NN + 1
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
         DO II = M_FINE - 1, 1, -1
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         
      ELSE  ! LOS cross the Earth's surface

         IF(ALT_FLAG) THEN
            IF(FP_FLG) THEN
               Call AddElement (Z, FL_BELOW, NP)
            ELSE
               Call AddElement (Z, FL_BELOW + 1, NP)
            END IF
            
            J = I_FP
            DO KK = FL_BELOW, (NLAYERS - J)*NN + 2, -1
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END IF

         K_FINE = MIN(I_FINE,MIN(NLAYERS,J + N_FINE_ST))
!         write(6,*) K_FINE, I_FINE
         
         KK = NL_FINE - (J - 1)*NN
         If(KK > 0) Call AddElement (Z_FINE(KK), KK, NP)
         DO II = J, K_FINE - 1
            DO K = 1, NN
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         DO II = MAX(J,K_FINE) + 1, MAX(J,I_FINE)
            KK = NL_FINE - (II - 1)*NN
            Call AddElement (Z_FINE(KK), KK, NP)
         END DO
         DO II = MAX(J,I_FINE), NLAYERS - 1
            DO K = 1, NN
               KK = NL_FINE - (II - 1)*NN - K
               Call AddElement (Z_FINE(KK), KK, NP)
            END DO
         END DO
         
         NTH = NP ;  TH_FLAG = .FALSE.
      END IF
    End Block ViewDir

!    WRITE(*,*) '      in V2 NTH = ', NTH

    ! NLS - number of points along LOS
    ! NTH - number of points along LOS before the tangent height
    ! (NTH = NLS if tangent height is not in the pointing direction) 
    NLS = NP
    ALLOCATE(ZLS(NLS), ISEL(NLS), IOB(NLS))
    ZLS(:) = ZLoc(1:NLS) ;  ISEL(:) = IdxLoc(1:NLS)
    
!    NULLIFY (ELEMENT%NEXT_ELEM)
!    ALLOCATE(ZLS(NLS), ISEL(NLS), IOB(NLS))
!    ELEMENT => FIRST_ELEM
!    ! Skip the first element as is it not filled
!    NEXT => ELEMENT%NEXT_ELEM
!    DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT) ;  ELEMENT => NEXT
!    DO WHILE (ASSOCIATED(ELEMENT))
!       II = ELEMENT%NUM
!       ZLS(II) = ELEMENT%ZLS
!       ISEL(II) = ELEMENT%ISEL
!       NEXT => ELEMENT%NEXT_ELEM
!       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
!       ELEMENT => NEXT
!    END DO

    IOB(:) = NLAYERS - INTERP_IDX(ISEL(1:NLS)) + 1
    IF(FP_FLG) IOB(1) = J_OUT
    IF(TH_FLAG) IOB(NTH) = ITH

!    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  CONTAINS
!***************************************************************************
    Subroutine AddElement (ZLS, IDX, COUNTER)
      INTEGER IDX, COUNTER
      REAL (KND) ZLS  
      !==================================================================
      COUNTER = COUNTER + 1
      ZLoc(COUNTER) = ZLS ; IdxLoc(COUNTER) = IDX
    End Subroutine AddElement
  END SUBROUTINE HEIGHT_GRID_V3
  
!***************************************************************************
! SUBROUTINE DirSun is now in los_geom.f90
!***************************************************************************
 Function E_CSZA(IES, FLG)
   USE ST_VARS, ONLY: GeoD_OUT
   USE TYPES,   ONLY: KND, Eff_Sol_Idx
   CHARACTER (LEN = 1) FLG
   REAL (KND) E_CSZA
   TYPE(Eff_Sol_Idx) :: IES

   IF(FLG == 'R' .OR. FLG == 'r') THEN
      E_CSZA = GeoD_OUT%LSG(IES%IA,IES%IV,IES%IS,IES%IP)%C_SZA(IES%ILS)
   ELSE
      E_CSZA = GeoD_OUT%LSG(IES%IA,IES%IV,IES%IS,IES%IP)%CG_SZA(IES%ILS)
   END IF
   RETURN
 End Function E_CSZA
 
!***************************************************************************
 Subroutine GET_SZA_IDX (SZA, IDX)
   USE GRIDS,      ONLY: X_INV, NSOL
   USE SPHER_PARS, ONLY: ONE_MS_SZ
   USE TYPES,      ONLY: KND
   INTEGER  IDX(:)
   REAL (KND)  SZA(:)
   ! Local parameters
   INTEGER II, IS
   !==================================================================
   IF (ONE_MS_SZ) THEN
      IDX = 1
      RETURN
   END IF
      
   DO II = 1, SIZE(SZA)
      IF (SZA(II) <= X_INV(1)) THEN
         IDX(II) = 1 
         CYCLE
      END IF
      IF (SZA(II) >= X_INV(NSOL)) THEN
         IDX(II) = NSOL - 1
         CYCLE
      END IF
           
      DO IS = 1, NSOL - 1
         IF(SZA(II) >= X_INV(IS).AND.SZA(II) <= X_INV(IS + 1)) THEN
            IDX(II) = IS
            EXIT
         END IF
      END DO
   END DO
   RETURN
 END Subroutine GET_SZA_IDX

 !***************************************************************************
 Subroutine LOS_MS_IDX (SZA, IMS1, IMS2)
   USE SPHER_PARS, ONLY: X_SHAD
   USE TYPES,      ONLY: KND

   INTEGER IMS1, IMS2
   REAL (KND) SZA(:)
   ! Local parameters
   INTEGER II, NLS
   !==================================================================
   NLS = SIZE(SZA)
   IMS1 = NLS + 1;  IMS2 = 0
   DO II = 1, NLS
      IF(SZA(II) >= X_SHAD) THEN
         IMS1 = II
         EXIT
      END IF
   END DO
   DO II = IMS1, NLS
      IF(SZA(II) >= X_SHAD) THEN
         IMS2 = II
      END IF
   END DO
 END Subroutine LOS_MS_IDX
!***************************************************************************
 Function IsPar (Name)
   USE INFO, ONLY: RET_INFO, PAR_INFO, TG_INFO, AER_INFO, CLD_INFO
   LOGICAL  IsPar
   CHARACTER (LEN =*) Name
   !==================================================================
   IsPar = .TRUE.
   IF(ANY(Name == TG_INFO%NAME) ) RETURN
   IF(ANY(Name == AER_INFO%NAME)) RETURN
   IF(ANY(Name == PAR_INFO%NAME)) RETURN
   IF(ANY(Name == CLD_INFO%NAME)) RETURN
   IsPar = .FALSE.
 END Function IsPar
 !***************************************************************************
 
  SUBROUTINE Read_Text (FILE, READ_ARR, Info, Type, SkipEmpty, SkipHeader)
   USE TYPES,   ONLY: DYN_CHAR_L
   
   CHARACTER (LEN = *) FILE
   Integer  Info
   TYPE (DYN_CHAR_L), ALLOCATABLE :: READ_ARR(:)
   Logical, Optional :: Type, SkipEmpty, SkipHeader
   
   ! Local variables:
   CHARACTER (LEN =*), PARAMETER :: R_NAME = 'Read_Text'
   CHARACTER (LEN=1000)   CHAR
   Character (Len=:), ALLOCATABLE :: STR_READ
   INTEGER    UNT, N_READ, II, IOSTAT, Size, KK
   TYPE Linked_List_Entry
      Character (Len=:), ALLOCATABLE :: STR_READ
      TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
   END TYPE Linked_List_Entry
   TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL(), NEXT
   !=======================================================================
   CALL ST_TRACK (R_NAME, 3, 'I')
   
   Open(NewUnit = UNT, FILE = FILE, STATUS = 'old', iostat = info)
   if(info /= 0) Return

   If (Present(SkipHeader)) Then
      If(SkipHeader) Call Skip_Header (Unt)
   End If
   
   N_READ  = 0
   DO
      STR_READ = ''
      DO
         READ(UNT, '(A)', ADVANCE = 'no', IOSTAT = IOSTAT, END=112, Size=size) CHAR
         STR_READ = STR_READ//CHAR(:Size)
         IF(IOSTAT /= 0) EXIT
      END DO
      SkipEmptyLines: Block
        If(.Not.Present(SkipEmpty)) Exit SkipEmptyLines
        If(.Not.SkipEmpty) Exit SkipEmptyLines
        IF(Len_Trim(STR_READ) /= 0) Exit SkipEmptyLines
        Cycle
      End Block SkipEmptyLines
      IF (.NOT.ASSOCIATED(ELEMENT)) THEN
         ALLOCATE(FIRST_ELEM)
         ELEMENT => FIRST_ELEM
         N_READ = 1
      ELSE
         ALLOCATE(ELEMENT%NEXT_ELEM)
         ELEMENT => ELEMENT%NEXT_ELEM
         N_READ = N_READ + 1
      ENDIF
      ELEMENT%STR_READ = STR_READ
   END DO
   
112 CONTINUE
   CLOSE (UNT)
   NULLIFY(ELEMENT%NEXT_ELEM)
   
   IF(ALLOCATED(READ_ARR)) DEALLOCATE (READ_ARR) 
   ALLOCATE(READ_ARR(N_READ))
   II = 0
   ELEMENT => FIRST_ELEM
   DO WHILE (ASSOCIATED(ELEMENT))
      II = II + 1
      READ_ARR(II)%S = ELEMENT%STR_READ
      READ_ARR(II)%L = LEN(ELEMENT%STR_READ)
      If(PRESENT(Type)) Then
         If(Type) Then
            READ_ARR(II)%Type = 'Data'
            IF(LEN_TRIM(READ_ARR(II)%S) == 0) READ_ARR(II)%Type = 'Text'
            DO KK = 1, READ_ARR(II)%L
               SELECT CASE ( IACHAR( READ_ARR(II)%S(KK:KK) ) )
               CASE( 32,48:57,43,45,46,68,69,100,101 )
               CASE DEFAULT 
                  READ_ARR(II)%Type = 'Text' ;  EXIT 
               END SELECT
            ENDDO
         End If
      End If
      NEXT => ELEMENT%NEXT_ELEM
      DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
      ELEMENT => NEXT
   END DO
   
   CALL ST_TRACK (R_NAME, 3, 'O')
   RETURN
   
 END SUBROUTINE READ_TEXT
 
 !***************************************************************************
 Subroutine ErrMsg (R_Name, stat, msg, DelayedOutput)
    Use Log_Pars, only: ErrLog, WARN_FLAG, FIRST_ERR
    
    CHARACTER (LEN=*) STAT, R_Name
    TYPE(DYN_CHAR), Allocatable, Optional  :: MSG(:)
    Logical, Optional :: DelayedOutput
    ! Local variables
    Integer  II
    character (LEN=:), ALLOCATABLE :: STATUS

    STATUS = STAT
    CALL UPPER_LOWER_CASE (STATUS,'L2U')
    SELECT CASE (STATUS)
    CASE ('E')
       Status = 'ERROR'
    CASE ('W')
       Status = 'WARNING'
    CASE ('T')
       Status = 'TRACING'
    END SELECT
         
    IF(TRIM(STATUS) == 'WARNING') WARN_FLAG = .TRUE.

    Store_Output: Block
      If(.not.present(DelayedOutput)) Exit Store_Output
      If(.not.DelayedOutput) Exit Store_Output
      IF (.NOT.ASSOCIATED(ErrLog)) THEN
         ALLOCATE(FIRST_ERR)
         ErrLog => FIRST_ERR
      ELSE
         ALLOCATE(ErrLog%NEXT_MSG)
         ErrLog => ErrLog%NEXT_MSG
      ENDIF
      
      ErrLog%Type = Status
      ErrLog%SubrN = R_Name
      If(STATUS /= 'TRACING') Then
         Allocate(ErrLog%Msg, Source = Msg)
         Deallocate(Msg)
      End If
      return
    End Block Store_Output

    SELECT CASE (STATUS)
    CASE ('ERROR', 'WARNING')
       write(eu,'(A)') ' *** '//status//' in '//R_Name//': '!//Msg(1)%S
       Do II = 1, Size(MSG)
          write(eu,'(A)') '     '//Msg(II)%S
       End Do
    CASE ('TRACING')
       Write(EU, *) ' ---> Called by '//R_NAME
    END SELECT
    
    Deallocate(Msg)
    Return
  End Subroutine ErrMsg
  !***************************************************************************
  Subroutine Array_From_String (String, Arr, Info)
    Use Types, Only: KND,  DYN_CHAR
    Integer Info
    Character (Len=*) String
    Real (KND), Allocatable :: Arr(:)
    ! Local  parameters
    Character (Len=*), Parameter :: R_NAME='Array_From_String'
    Integer II, NN
    Type (DYN_CHAR), ALLOCATABLE :: STR_ARR(:)
    !==================================================================
    CALL Split_String(String, STR_ARR)
    NN = Size(STR_ARR)
    If(Allocated(Arr)) Deallocate(Arr)
    Allocate(Arr(NN))
    Do II = 1, Size(STR_ARR)
       Read(STR_ARR(II)%S,*,ERR = 201) Arr(II)
    End Do
    Info = 0
    Return

201 Info = 1
    Return
  End Subroutine Array_From_String
!***************************************************************************
  Subroutine MatrInvers(Amat)
    ! Based on Doolittle LU factorization
    Use Types, Only: KND
    REAL (KND)   Amat(:,:)
    ! Local variables
    REAL (KND), ALLOCATABLE :: L(:,:), U(:,:), b(:), d(:), x(:)
    REAL (KND) coeff
    INTEGER I, J, K, NN
    !==================================================================
    NN = SIZE(Amat,1)
    ALLOCATE( L(NN,NN), U(NN,NN), b(NN), d(NN), x(NN))
    
    ! Initialization for matrices L and U and b
    L=0.0 ;  U=0.0 ;  b=0.0
    
    ! step 1: forward elimination
    do k=1, NN-1
       do i=k+1,NN
          coeff=Amat(i,k)/Amat(k,k)
          L(i,k) = coeff
          do j=k+1,NN
             Amat(i,j) = Amat(i,j)-coeff*Amat(k,j)
          end do
       end do
    end do
    
    ! Step 2: prepare L and U matrices 
    ! L matrix is a matrix of the elimination coefficient
    ! + the diagonal elements are 1.0
    FORALL(i=1:NN) L(i,i) = 1.0
    
    ! U matrix is the upper triangular part of A
    FORALL(J=1:NN, I=1:NN, I<=J) U(I,J) = Amat(I,J)
    
    ! Step 3: compute columns of the inverse matrix C
    do k=1,NN
       b(k)=1.0
       d(1) = b(1)
       ! Step 3a: Solve Ld=b using the forward substitution
       do i=2,NN
          d(i)=b(i)
          do j=1,i-1
             d(i) = d(i) - L(i,j)*d(j)
          end do
       end do
       ! Step 3b: Solve Ux=d using the back substitution
       x(NN)=d(NN)/U(NN,NN)
       do i = NN-1,1,-1
          x(i) = d(i)
          do j=NN,i+1,-1
             x(i)=x(i)-U(i,j)*x(j)
          end do
          x(i) = x(i)/u(i,i)
       end do
       ! Step 3c: fill the solutions x(NN) into column k of C
       Amat(:,k) = x(:)
       b(k)=0.0
    end do
  End Subroutine MatrInvers
  !***************************************************************************
  Subroutine OpenOut (FileName, Unt, RecLen)
    Use OutInf, Only: DelayOut, OutD, DIR_OUT
    Integer Unt, II
    Character (Len=*) FileName
    Integer, Optional:: RecLen
    !==================================================================
    Do II = 1, Size(OutD)
       If(OutD(II)%Name == FileName) Exit
    End Do
    If(Present(RecLen)) OutD(II)%RecL = RecLen

    IF(DelayOut) Then
       Unt = II
       ! If the list was previously filled it needs to be reset
       If(Associated(OutD(Unt)%ELEMENT)) Then
          OutD(Unt)%ELEMENT => OutD(Unt)%First_Elem
          Do While (Associated(OutD(Unt)%ELEMENT))
             OutD(Unt)%NEXT => OutD(Unt)%ELEMENT%NEXT_ELEM               
             DEALLOCATE (OutD(Unt)%ELEMENT)
             NULLIFY (OutD(Unt)%ELEMENT)
             OutD(Unt)%ELEMENT => OutD(Unt)%NEXT
          End Do
       End If

       Allocate(OutD(Unt)%First_Elem)
    Else
       OPEN(NEWUNIT=Unt, file=DIR_OUT//FileName, RECL=OutD(II)%RecL, STATUS='REPLACE')
    End If
    Return
  End Subroutine OpenOut
  !***************************************************************************
  Subroutine WriteOut (Unt, OutStr, NoDelay)
    Use OutInf, Only: DelayOut, OutD
    Integer Unt
    Character (Len=*) OutStr
    Logical, Optional :: NoDelay
  ! Local parameters
    Logical Flg
    !==================================================================
    Set_Mode: Block
       Flg = DelayOut
       If(.Not.Present(NoDelay)) Exit Set_Mode
       IF(.Not.NoDelay) Exit Set_Mode
       Flg = .False.
    End Block Set_Mode

    IF(Flg) Then
       If(.NOT.Associated(OutD(Unt)%ELEMENT)) Then
          OutD(Unt)%Element => OutD(Unt)%First_Elem
       Else
          ALLOCATE(OutD(Unt)%ELEMENT%Next_Elem)
          OutD(Unt)%Element => OutD(Unt)%ELEMENT%Next_Elem
       End If
       OutD(Unt)%Element%S = Trim(OutStr)
    Else
       Write(Unt,'(A)') Trim(OutStr)
    End If
    Return
  End Subroutine WriteOut
  !***************************************************************************
  Subroutine CloseOut (Unt)
    Use OutInf, Only: DelayOut, OutD
    Integer Unt
    !==================================================================
    IF(DelayOut) Then
       Nullify(OutD(Unt)%ELEMENT%Next_Elem)                
    Else
       Close(Unt)
    End If
    Return
  End Subroutine CloseOut
!***************************************************************************
  Subroutine ResetOut 
    Use OutInf, Only: OutD
    ! Local  parameters
    Integer II
    !==================================================================
    Do II = 1, Size(OutD)
       OutD(II)%ELEMENT => OutD(II)%First_Elem
       Do While (Associated(OutD(II)%ELEMENT))
          OutD(II)%NEXT => OutD(II)%ELEMENT%NEXT_ELEM               
          DEALLOCATE (OutD(II)%ELEMENT)
          NULLIFY (OutD(II)%ELEMENT)
          OutD(II)%ELEMENT => OutD(II)%NEXT
       End Do
    End Do
    Return
  End Subroutine ResetOut
!**************************************************************************
  SUBROUTINE NotFound (R_NAME, PAR, SEL_VAL, *)
    USE LOG_PARS, ONLY : EU
    CHARACTER (LEN = *) R_NAME, PAR, SEL_VAL
    
    call Write_Message(R_NAME, 'ERROR', ' ')
    Write(EU,*) '     Input value is not recognized.'
    Write(EU,*) '     Input parameter : ', TRIM(PAR)
    Write(EU,*) '     Selected value  : ', TRIM(SEL_VAL)
    
    RETURN 1
  End Subroutine NotFound
!**************************************************************************
  SUBROUTINE RotMatr (Mu_sc, Si_sc, Mu_in, Si_in, Rel_az, Co_ls, Si_s, RL1, RR1)
    USE ST_VARS, ONLY: Pie
    USE TYPES,   ONLY: KND
    
    REAL (KND)  Mu_sc, Si_sc, Mu_in, Si_in, Rel_az, Co_ls, Si_s, RL1(2), RR1(2)
    ! Local parameters
    REAL (KND)  Zn, T1, T2
    !==================================================================

    IF( Rel_az < 0.d0 ) Rel_az = 2.d0*Pie + Rel_az 

    IF( REAL(Rel_az)==REAL(Pie).OR.REAL(Rel_az)==0.0.OR.Si_s==0.d0 ) THEN 
!      ... Scattering plane is meridional plane: 
       RL1(1) = 1.d0 ; RL1(2) = 0.d0
       RR1(1) = 1.d0 ; RR1(2) = 0.d0 
       RETURN
    ENDIF
    IF( Rel_az > Pie ) THEN 
       Zn = 1.d0
    ELSE
       Zn = -1.d0
    ENDIF
    COSIG: Block
      IF( REAL(ABS(Mu_sc)) == 1.0.AND.REAL(ABS(Mu_in)) == 1.0 ) THEN
         T2 = 1.d0 ; T1 = 1.d0 
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Mu_sc)) == 1.0 ) THEN
         T2 = -SIGN(1.d0,Mu_sc)*COS(Rel_az) ; T1 = 1.d0
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Mu_in)) == 1.0 ) THEN
         T1 = -SIGN(1.d0,Mu_in)*COS(Rel_az) ; T2 = 1.d0
         EXIT COSIG
      ENDIF
      T1 = (Mu_sc - Mu_in*Co_ls)/Si_in/Si_s  ! cos( sigma_1 )
      T2 = (Mu_in - Mu_sc*Co_ls)/Si_sc/Si_s  ! cos( sigma_2 )
    END Block COSIG

!   ... Elements (2,2) and (2,3) of rotation matrix
    RR1(1) = 2.d0*T1*T1 - 1.d0    ! 2*cos^2( 2*sigma_1 ) - 1
    RR1(2) = Zn* 2.d0*SQRT( ABS(1.d0 - T1*T1) )*T1

    RL1(1) = 2.d0*T2*T2 - 1.d0    ! 2*cos^2( 2*sigma_2 ) - 1
    RL1(2) = Zn* 2.d0*SQRT( ABS(1.d0 - T2*T2) )*T2

      
  End Subroutine RotMatr
!***************************************************************************
  Logical Function IsNum (Str)
  Character (Len = :), Allocatable :: Str
  Integer II

  IsNum = .True.
  IF(LEN_TRIM(Str) == 0) IsNum = .False.
  Do II = 1, Len(Str)
     Select Case (IACHAR(Str(II:II)))
     CASE( 32,48:57,43,45,46,68,69,100,101 )
     CASE DEFAULT 
        IsNum = .False. ;  EXIT 
     END SELECT
  End Do
     
  End Function  IsNum
!***************************************************************************
END MODULE TOOLS
!***************************************************************************
