!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 03 Aug 2023
!
!	Institute of Remote Sensing, University of Bremen, Germany
!
!***************************************************************************
  Module BasicTools

  Contains
  !***************************************************************************
    SUBROUTINE ULC (INPUT_LABEL, KEY_STRING, I_1, I_2)
      ! Conversion between upper and lower case
      CHARACTER (LEN = *)  INPUT_LABEL, KEY_STRING
      INTEGER I, ASCII_POS, I_S, I_E, SHIFT, II_1, II_2
      INTEGER, OPTIONAL :: I_1, I_2
      
      SELECT CASE (TRIM(KEY_STRING))
      CASE ('u2l','U2L','u2L','U2l')
         I_S = 65 ;  I_E = 90 ;  SHIFT = 32
      CASE DEFAULT
         I_S = 97 ;  I_E = 122 ;  SHIFT = - 32
      END SELECT
      
      IF(PRESENT(I_1)) THEN
         II_1 = I_1
      ELSE
         II_1 = 1
      END IF
      IF(PRESENT(I_2)) THEN
         II_2 = I_2
      ELSE
         II_2 = LEN_TRIM(INPUT_LABEL)
      END IF
      
      DO I = II_1, II_2
         ASCII_POS = IACHAR(INPUT_LABEL(I:I))
         IF(ASCII_POS >= I_S .AND. ASCII_POS <= I_E) &
              ASCII_POS = ASCII_POS + SHIFT
         INPUT_LABEL(I:I) = ACHAR(ASCII_POS)
      END DO
      RETURN
    END SUBROUTINE ULC
  !*************************************************************
  End Module BasicTools
  !*************************************************************
  MODULE ABSORBERS

  INTEGER  N_GAS, AMF_IDX, N_MIX_GAS, N_LBL_MOL, N_ESFT_MOL, &
       MIX_ABS_START, N_ST_MOL, N_MIX_MOL, N_XSEC_MOL, N_LBL_PROF
  INTEGER, ALLOCATABLE :: ST_GAS_IDX(:), GAS_INDEX(:)
  CHARACTER (LEN=:), ALLOCATABLE :: LINE_ABS_MODE  
END MODULE ABSORBERS

!*************************************************************
MODULE ST_VARS
  USE TYPES,  ONLY: KND, Geometry_Data
  CHARACTER (LEN=3)  ANG_INT
  CHARACTER (LEN=4)  PREC_FLAG
  CHARACTER (LEN=10) SOLVER
  CHARACTER (LEN=8)  Vec_Sca
  CHARACTER (LEN=15) INEL_PROC, INEL_WHERE
  CHARACTER (LEN=:), ALLOCATABLE :: RTM_TYPE, RTM_CORE, RTM_MODE, C_SYS, &
       Refr_index, SCATT_MODE, RET_MODE, LL_factor, King_factor, MSF_INT, &
       APPR_RRS, TSM, CONV_Z, RIN_OUT, SURF_TYPE, BRDF_RANGE, RET_CORE
  LOGICAL  DO_AEROSOL, DO_PC, WF_FLAG, OWF_FLAG, RRS_TEST, INT_OUT,&
       DO_SATELLITE, BRDF_FLAG, RING_FLAG, DO_REFRAC, DIFB, SPECULAR, Gauss_Field, &
       CS_ANALYTIC, RAD_FIELD, VHL, Ellip, Interp_RF, Use_Sym, SSC_RF, &
       CONVOL_FLAG, LA_WINS, CONVOL_ALL, NEW_SPEC_GRID, ADAPTIVE_GRID, Add_lev, &
       Enhance_acc, ONE_MS_SZ, ARB_OL, D_sol(2), AG_aer, DO_RAMAN, DO_PLANCK, &
       DO_AZIMUTH, DO_INTERNAL_CHECK, USE_PSM, Sec_Order, Short_los, Sel_los, &
       PreCalcRtMatr, PreCalcGeom
  LOGICAL, POINTER :: OWF_Chl, OWF_Ray, OWF_Use, OWF_Ks, OWF_Kl
  LOGICAL, ALLOCATABLE :: INT_MS_FLAG(:), WF_MS_FLAG(:), LA_FLAG(:), &
       LOW_RES_FLAG(:), Lo_der(:)
  INTEGER  RAY_SCATT_IDX, HR_NSEG, NSPEC_RTM, Kvs, N_HR, N_LR, SDVI, I_LAY, &
       N_RHS, N_Eff, N_Adj, NS_wf, Nh_s, Nwl_RR, N_HR_EXT, Mh_AG, NSTRMS
  INTEGER, ALLOCATABLE :: ID_LIM(:,:,:), M_LIM(:), SHI_LIM(:), &
       SHI_LIM_C(:), IDM(:), LDM(:), Nc_der(:)
  INTEGER, ALLOCATABLE, TARGET :: SEG_NUM_RTM(:), SEG_NUM_OUT(:)
  INTEGER, POINTER :: SEG_NUM(:)
  REAL (KND)  EARTH_RADIUS, RAD_TO_DEG, GROUND_HEIGHT, VST, ACCUR, SS_MS_BOUNDARY, &
       FLUX_MIN_ZZZ, FLUX_MAX_ZZZ, STEP_INP, LATITUDE, LONGITUDE, R_GH, CRIT_AG, &
       CRIT_AG_Mh, M_PLA, PSAB, VS_HC, PVC(4), Cr_Short_los, Cr_Sel_los
  REAL (KND), PARAMETER :: PIE = 3.1415926536D0, &
       FP = 7.2429716D+18, SMKM = 1.0D+5, FACTOR_O4 = 0.2091D0, &
       ! FP = 7.24325825D+18, SMKM = 1.0D+5, FACTOR_O4 = 0.2091D0, ! Old v3.1
       SF_DELTA = 1.0D-8, F14 = 0.25D0, TP_HEIGHT = 8.0D0
  REAL (KND), ALLOCATABLE :: HR_SEG_WL(:,:), WL_RR(:), SOL_ANG(:), Finc(:,:), Ainc(:,:)
  REAL (KND), ALLOCATABLE, TARGET :: WL_RTM(:)
  REAL (KND), POINTER :: WL_GRID(:)

  INTEGER  NSEGS, N_HARM, ND_ANG
  LOGICAL  F_LGI_SCR, F_LGI_SCA, F_LGI_EXT, Emi_FLG, CIRA_FLG, PreCalcAngs, &
       FLG_MS, MPI_FLG, CI_FLG, AKIMA_FLG, Ray_to_WF, AA_BRF
  LOGICAL, ALLOCATABLE :: ERROR(:)
  LOGICAL, ALLOCATABLE, TARGET :: MSK_RTM(:,:)
  CHARACTER (LEN=1) DISCR_M
  CHARACTER (LEN=:), ALLOCATABLE :: LSA_MODE, CI_DIR
  REAL (KND)  E_STEP
  REAL (KND), ALLOCATABLE :: WL_SEGS(:,:)
  TYPE (Geometry_Data), TARGET :: GeoD_ITR, GeoD_OUT, GeoD_PRT, GeoD_PS
END MODULE ST_VARS
!*************************************************************
MODULE RETR_ITER
  USE TYPES,  ONLY: KND
  INTEGER ITERATION, NL_PRE, N_OSC, N_GET, N_NLIN, MAX_OSC
  INTEGER, ALLOCATABLE :: I_MAP(:,:), P_MIN(:)
  LOGICAL  TG_SCALE_FLAG, Decr, RMS_OSC, SCAL_APR, FLG_OSC
  LOGICAL :: ELEV_END = .TRUE.
  Type RMS_DATA 
     Real (Knd) NonLin, Lin
  End type RMS_DATA
  Type (RMS_DATA), Allocatable:: RMS_ALL(:)
  REAL (KND), ALLOCATABLE :: TG_SF(:,:), Z_PRE(:), GHS(:), AOD_ITER(:)
  REAL (KND)  TPC_F, EXA_T_FACTOR, RMS_SAVE, MIN_TAU
END MODULE RETR_ITER

!***************************************************************************
MODULE ST_INPUT
  USE TYPES,    ONLY: KND, DYN_CHAR
  LOGICAL    DO_PROFILES_AKIMA, DO_PT_MPI_PROFILE
  INTEGER    NC_T, N_REPL, NC_PTZ, NC_P, DAY_OF_YEAR, YEAR
  INTEGER, PARAMETER :: MAX_LAY = 400
  INTEGER, ALLOCATABLE :: BAND_NUM(:), PROF_NCZ(:), PROF_NCV(:)
  CHARACTER (LEN=:), ALLOCATABLE :: NET_NAME, ALBEDO_FILENAME, ALBEDO_DB_PATH, &
       PATH_AER_DB, PT_FILENAME, BRO_MIPAS_FILE, CLIM_PATH, DEP_NAME, &
       PROF_UW_NAME, CHL_CONC_NAME, PWAC_NAME, CHL_NAME, TPH_DB_PATH, &
       HOME_DIR, PSI_NAME, APH_TYPE, CIRA_DIR, WL_UNITS, AER_PAR_TYPE, &
       CLIM_LABEL, SND_NAME, S_N_D, PT_CDF, WL_CDF, SOL_CDF, AER_CDF, ALB_CDF
  CHARACTER (LEN = 150), ALLOCATABLE :: PROF_FILES(:)
  LOGICAL         BRO_MIPAS_FLAG
  REAL (KND)    CIRA_ZMIN, SND_WL, AER_ZMIN, AER_ZMAX
  REAL (KND), ALLOCATABLE :: BAND_START(:), BAND_STEP(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: PROF_UNITS(:), PROF_LABELS(:), PROF_CDF(:)
  TYPE (DYN_CHAR), ALLOCATABLE, TARGET :: Inp_Pars(:), CI_Pars(:)
END MODULE ST_INPUT
!***************************************************************************
MODULE PROFILES
  USE TYPES, ONLY: KND
  LOGICAL     DO_SCA_PRE, DO_SCA_O3, DO_SCA_H2O, DO_SCA_NO2
  REAL (KND)  SCA_PRE, O3_TOTAL_COLUMN, SCA_H2O, SCA_NO2
  REAL (KND), ALLOCATABLE ::  AIR_DENSITY(:), PRESS(:), TEMP(:)
END MODULE PROFILES
!***************************************************************************
MODULE INFO
  USE ABSORBERS, ONLY: N_ST_MOL
  USE TYPES,     ONLY: KND, DYN_CHAR, NAME_TAG, NAME_TYPE, LineParsType
  
  TYPE, EXTENDS (NAME_TYPE) :: TRACE_GAS_INFORMATION
     LOGICAL  FM_FLG, WF_FLG, READ_FLG
     INTEGER  IDC, ALP_NLIN, IdxHT, NIso, IsoMin, IsoMax
     REAL (KND)  S_NU, MAX_WL_CS
     REAL (KND), ALLOCATABLE :: ND(:), ND_INP(:), ALP_LI(:), ALP_LP(:), &
          ALP_HW(:), ALP_EP(:,:), GVPP(:), PartFuncR(:,:,:)
     CHARACTER (LEN = 3)  WF_DIM
     CHARACTER (LEN = 20) M_TYPE
     CHARACTER (LEN = :), ALLOCATABLE :: LONG_NAME, NameUC
     CHARACTER (LEN = 20), ALLOCATABLE :: TYPE_WL(:)
     Type (LineParsType), Allocatable :: LinePars(:)
  END TYPE TRACE_GAS_INFORMATION
  TYPE (TRACE_GAS_INFORMATION), ALLOCATABLE :: TG_INFO(:)
  
  TYPE, EXTENDS (NAME_TAG) :: PARS_INFORMATION
     LOGICAL   WF_FLG
     CHARACTER (LEN = 1)  VSI
     CHARACTER (LEN = 3)  WF_DIM
     CHARACTER (LEN = :), ALLOCATABLE :: LONG_NAME
  END TYPE PARS_INFORMATION
  TYPE (PARS_INFORMATION), ALLOCATABLE :: AER_INFO(:), CLD_INFO(:), PAR_INFO(:)
  
  TYPE, EXTENDS (NAME_TYPE) :: WF_DATA_BASE
     INTEGER  NL, NF
     CHARACTER (LEN = :), ALLOCATABLE :: LONG_NAME, DIM
     REAL (KND), ALLOCATABLE :: WF(:,:,:,:)
  END TYPE WF_DATA_BASE
  TYPE (WF_DATA_BASE), ALLOCATABLE, TARGET :: WF_DB(:)
  
  TYPE REGULARIZATON_PARS
     CHARACTER (LEN = 3) R_A ! Normalization
     REAL (KND) SRP ! Scalar retrieval parameter
     REAL (KND), ALLOCATABLE :: Alt(:), Val(:) ! Altitude distribution
  END TYPE REGULARIZATON_PARS

  Type Reject_LOS
     Integer IS, IE
     Logical, Allocatable :: Flg(:)
  End type Reject_LOS
  
  TYPE, EXTENDS (NAME_TYPE) :: RETRIEVAL_INFORMATION
     INTEGER  N_PARS, N_SBP, CountMin, CountMax
     INTEGER, ALLOCATABLE :: ST_NUM(:), SEG_S(:), SEG_E(:), I_S(:), I_E(:) 
     LOGICAL  STAT, WF_ON, WL_SEL
     LOGICAL, ALLOCATABLE :: DS_FLG(:,:)
     CHARACTER (LEN = 3) AR_N
     CHARACTER (LEN = 20) WF_NAME
     CHARACTER (LEN = 1)  VS, EXTR_TYPE_U, EXTR_TYPE_L, LimitMode
     CHARACTER (LEN = :), ALLOCATABLE :: NORM, RP, WF_DIM, LONG_NAME 
     CHARACTER (LEN = 1), ALLOCATABLE :: VSI(:)
     REAL (KND) Z_L, Z_U, AR_P, TR_P, PAR_MAX, PAR_MIN
     REAL (KND), ALLOCATABLE :: PRE(:), APR(:), FACTOR(:), WL_MIN(:), WL_MAX(:)
     TYPE (REGULARIZATON_PARS) R_OE, R_TH, MaxOE, MaxTH
     Type (Reject_LOS), Allocatable :: REJ_LOS(:)
  END TYPE RETRIEVAL_INFORMATION
  TYPE (RETRIEVAL_INFORMATION), ALLOCATABLE :: RET_INFO(:)
  
  TYPE AUXILIARY_INFORMATION 
     INTEGER  NDC
     INTEGER, ALLOCATABLE :: N_STEP(:), SNR_ORD(:)
     LOGICAL  DC_FLAG, NOREF_FLAG
     CHARACTER (LEN=:), ALLOCATABLE :: EXP_DATA_FILE, SOL_DATA_FILE, &
          EDF_CDF, SDF_CDF, SNR_CDF
     REAL (KND), ALLOCATABLE :: SN(:), BIN(:) 
     TYPE (DYN_CHAR), ALLOCATABLE :: SN_FILE(:)
  END TYPE AUXILIARY_INFORMATION
  TYPE (AUXILIARY_INFORMATION), ALLOCATABLE :: AUX_INFO(:)
  
  TYPE STATE_TYPE  ! Retrieval initialization
     INTEGER  PAR_NUM, I_RET_INF, I_SBP
     REAL (KND)  MinNZ
     REAL (KND), ALLOCATABLE :: PRE(:), APR(:), RES(:), FACTOR(:), ReNorm(:)
     LOGICAL USE_FLG
  END TYPE STATE_TYPE

  TYPE (STATE_TYPE), ALLOCATABLE, TARGET :: STATE(:)  
  
END MODULE INFO
!***************************************************************************
MODULE DATA_RET
  USE TYPES, ONLY: KND, DYN_CHAR, XVecType, RegMatrs, MeasInfo
  USE INFO,  ONLY: STATE_TYPE
  LOGICAL  TIH_FLAG, DO_IMAT, DO_CONSTRAIN, MOD_READ_FLAG, RET_FLAG, AVG_RTM, &
       DOUBLET_FLAG, V_INV_FLG, ALLOW_END, FLG_OUTL
  LOGICAL, ALLOCATABLE :: KEEP_APR(:)

  INTEGER  ITERATIONS_LIMIT, ITER_OUTL, N_svc

  CHARACTER (LEN = 10) MAIN_GAS, RET_FORM
  CHARACTER (LEN = :), ALLOCATABLE :: TPS_NAME, MOD_DIR, PRESET_APR_NAME,&
       TihRegMode, LP_Mode

  REAL (KND)  CRIT_RMS, RCOR, Z_CU, Z_CL, TEMP_MAX, CRIT_PAR, TEMP_RET,&
       LER_MIN, LER_MAX, ALB_MIN, ALB_MAX, ALB_CRIT, PRE_MIN, PRE_MAX, &
       PRE_CRIT, MIN_EVAL, TEMP_MIN, RETR_ALB, PRE_RET, Calibr_mult, RET_P1, &
       RET_P2, FACTOR_TPS_COL, TEMP_CRIT, CRIT_OUTL
  REAL (KND), ALLOCATABLE :: APR_ADD(:), TR_ADD(:)
  REAL (KND), POINTER :: WL_MDS(:)

  TYPE POLYNOMIAL_SETTINGS
     LOGICAL   CutFlg 
     REAL (KND), ALLOCATABLE :: CUT_S(:), CUT_E(:) 
  END type POLYNOMIAL_SETTINGS
     
  TYPE MEASUREMENT_DATA_SET
     INTEGER  NORM_WL
     INTEGER, ALLOCATABLE :: REF_NUM(:), ITH_S(:), ITH_E(:), N_POL(:), SM_P(:)
     LOGICAL SNR_COMB, NORM_RAD, NORM_LOG
     LOGICAL, ALLOCATABLE :: SEG_FLAG(:), OWN_REF(:), SOL_FLAG(:), RTH_FLG(:), &
          SCAL_FIT(:), SHI_FLG(:), SQ_SC_FLG(:), SQ_MS_FLG(:), SQ_RS_FLG(:), &
          POL_MODE(:), OUTLIER_FLAG(:), MASK(:,:), LOG_FLG(:), SGA_FLG(:), &
          SNR_CLS(:), SNR_CWL(:), MSK_PRE(:,:), MSK_INP(:), MSK_RTM(:,:), RM_CRS(:)
      CHARACTER (LEN=2), ALLOCATABLE :: SN_MODE(:)     
      CHARACTER (LEN=3), ALLOCATABLE :: BIN_TYPE(:)
      REAL (KND), ALLOCATABLE :: RMS_FACTOR(:), WL_REJ(:,:), SN_FACTOR(:), &
           WL(:), MSMT(:,:,:), SOL_SPEC(:,:), SY(:,:), WL_S(:), WL_E(:), &
           WL_INP(:), MSMT_INP(:,:,:), WL_SY(:)
      TYPE (DYN_CHAR), ALLOCATABLE :: EXP_FORM(:), SVC_Name(:)
      TYPE (POLYNOMIAL_SETTINGS), ALLOCATABLE :: PLN(:)
   END TYPE MEASUREMENT_DATA_SET
   TYPE (MEASUREMENT_DATA_SET), ALLOCATABLE, TARGET :: MDS(:)

   TYPE CORR_SPEC_DB
      INTEGER   NN
      INTEGER, ALLOCATABLE :: SEG_S(:), SEG_E(:)
      LOGICAL, ALLOCATABLE :: OFFSET_MES(:), OFFSET_REF(:), IEC_FLAG(:), &
           TILT_FLAG(:)
      TYPE (DYN_CHAR), ALLOCATABLE :: CDF(:)
      CHARACTER (LEN = 100), ALLOCATABLE :: FILE(:)
      CHARACTER (LEN = 1),   ALLOCATABLE :: SHIFT_MODE(:), LOS_MODE(:)
      REAL (KND),   ALLOCATABLE :: DAT(:,:,:)    
   END TYPE CORR_SPEC_DB
   TYPE (CORR_SPEC_DB) :: CORR_SP

   Type LM_DATA_TYPE
      REAL (KND) LMP, CF, CF_Pre, DCFL
      TYPE (STATE_TYPE), ALLOCATABLE :: STATE(:)
      Type (XVecType), Allocatable :: XVec(:)
      Type (RegMatrs) :: RegM
      Type (MeasInfo) :: MsmtInf
   End type LM_DATA_TYPE
   Type (LM_DATA_TYPE) :: LMD

   CONTAINS
   SUBROUTINE DATA_RET_INIT
      ALLOW_END = .TRUE.
   END SUBROUTINE DATA_RET_INIT
 END MODULE DATA_RET
!*************************************************************************
 MODULE FLU_LAND
   USE TYPES,  ONLY: KND
   INTEGER NFL
   LOGICAL FLU_L, USE_DGF
   CHARACTER (LEN=:), ALLOCATABLE :: FLU_NAME
   REAL (KND) FLU_SCAL, CONVERT_AMP
   REAL (KND), ALLOCATABLE :: WLF(:), SPF(:)
   TYPE DGF_TYPE  ! Double-Gaussian approximation
      REAL (KND) Amp, Wlm, Hwhm
   END TYPE DGF_TYPE   
   TYPE (DGF_TYPE), ALLOCATABLE, TARGET :: DGF_PAR(:)  
 CONTAINS
   FUNCTION GAU(NN,POS,AMP)
     INTEGER NN
     REAL (KND) GAU, POS, AMP, AA
     OPTIONAL :: AMP
     AA = 1.d0
     IF( PRESENT(AMP) ) AA = AMP
     GAU = AA*EXP(-LOG(2.d0)*( (POS - DGF_PAR(NN)%Wlm)/DGF_PAR(NN)%Hwhm)**2 )
   END FUNCTION GAU
 END MODULE FLU_LAND
!***************************************************************************
 MODULE RTM_PARS
   USE TYPES, ONLY: KND
   USE INFO,  ONLY: NAME_TAG, RET_INFO
   INTEGER  MAX_ITER

   TYPE DATA_ITER
      REAL (KND), ALLOCATABLE :: GRD(:), APR(:), CRN(:), NXT(:), PrcA(:), PrcR(:)
   END TYPE DATA_ITER
   TYPE SUB_PARS
      INTEGER  N_SBP, N_PARS 
      INTEGER, ALLOCATABLE :: IND(:)
      TYPE (DATA_ITER), ALLOCATABLE :: DAT(:)
   END TYPE SUB_PARS
   TYPE, EXTENDS (NAME_TAG) :: RTM_STATE
      CHARACTER (LEN = 1)  VS
      CHARACTER (LEN = :), ALLOCATABLE :: LONG_NAME
      TYPE (SUB_PARS), ALLOCATABLE :: SPR(:,:)
    CONTAINS
      PROCEDURE :: CLEAN => CLEAN_PAR
   END TYPE RTM_STATE
   
   TYPE (RTM_STATE), ALLOCATABLE, TARGET :: RTM_ST(:)  
 CONTAINS 
   SUBROUTINE CLEAN_PAR (RTM_ST)
     CLASS (RTM_STATE), TARGET :: RTM_ST
     INTEGER ITR, I_ASP, I_SSP 
     
     IF(.NOT.ALLOCATED(RTM_ST%SPR)) RETURN

!     NULLIFY(RTM_ST%CRN, RTM_ST%PRE, RTM_ST%APR)
     DO I_SSP = 1, SIZE(RTM_ST%SPR,1)
        DO I_ASP = 1, SIZE(RTM_ST%SPR,2)
           IF(ALLOCATED(RTM_ST%SPR(I_SSP,I_ASP)%IND)) &
                DEALLOCATE(RTM_ST%SPR(I_SSP,I_ASP)%IND)
           DO ITR = 1, SIZE(RTM_ST%SPR(I_SSP,I_ASP)%DAT)
              ASSOCIATE (ARR => RTM_ST%SPR(I_SSP,I_ASP)%DAT(ITR))
                IF(ALLOCATED(ARR%GRD)) DEALLOCATE(ARR%GRD)
                IF(ALLOCATED(ARR%APR)) DEALLOCATE(ARR%APR, ARR%CRN, ARR%NXT)
                IF(ALLOCATED(ARR%PrcA)) DEALLOCATE(ARR%PrcA, ARR%PrcR)
              END ASSOCIATE
           END DO
           DEALLOCATE (RTM_ST%SPR(I_SSP,I_ASP)%DAT)
        END DO
     END DO
     DEALLOCATE (RTM_ST%SPR)
   END SUBROUTINE CLEAN_PAR

   SUBROUTINE WRT_ALT (ALT)
     USE RETR_ITER, ONLY: ITR => ITERATION
     
     REAL (KND) ALT(:)
     ! Local parameters
     INTEGER I_P, I_ASP, I_SSP
     !================================================================
     
     DO I_P = 1, SIZE(RTM_ST) 
        DO I_SSP = 1, SIZE(RTM_ST(I_P)%SPR,1)
           DO I_ASP = 1, SIZE(RTM_ST(I_P)%SPR,2)
              ASSOCIATE(ARR => RTM_ST(I_P)%SPR(I_SSP,I_ASP)%DAT(ITR))
                IF(ALLOCATED(ARR%GRD)) DEALLOCATE(ARR%GRD)
                IF(RTM_ST(I_P)%VS == 'V') THEN
                   ALLOCATE (ARR%GRD(SIZE(ALT)), SOURCE = ALT) 
                ELSE
                   ALLOCATE (ARR%GRD(1)) 
                END IF
              END ASSOCIATE
           END DO
        END DO
     END DO

     RETURN
   END SUBROUTINE WRT_ALT

END MODULE RTM_PARS
 !***************************************************************************
MODULE LOG_PARS
  USE TYPES, ONLY: DYN_CHAR
  LOGICAL             DO_LOGFILE, WARN_FLAG, TO_REFL_FUNC
  LOGICAL             TRC_FLG(3)
  INTEGER             LUNIT, C_UNIT, EU, SurUNIT, AUNIT, IS1, IS2
  INTEGER             FUNIT, L_S, L_E, L_ER, L_U(2), LOG_LEVEL(2)
  INTEGER             OFFSET
  INTEGER, PARAMETER :: N_NSO = 1, N_RSO = 2, N_SSO = 3, N_ESO = 4, N_DSO = 5,&
       N_TSO = -1, N_XSO = -2, N_ZSO = 0
  CHARACTER (LEN=120) ERROR_TXT
  CHARACTER (LEN=:), ALLOCATABLE :: DIR_OUT, Work_Dir, Mult_WL_Form, OUT_FOR,&
       FORMAT_FLAG, ONE_WL, OUT_REPR, USE_SHAD, Mode
  TYPE Error_List
     TYPE (DYN_CHAR), ALLOCATABLE :: Msg(:)
     Character (Len=:), ALLOCATABLE :: Type, SubrN
     TYPE (Error_List), POINTER :: NEXT_MSG
  END TYPE Error_List
  TYPE (Error_List), POINTER :: ErrLog => NULL(), FIRST_ERR
  TYPE Output_List
     TYPE (DYN_CHAR), ALLOCATABLE :: Txt(:)
     Character (Len=:), ALLOCATABLE :: Head
     TYPE (Output_List), POINTER :: NEXT_OUT
  END TYPE Output_List
  TYPE (Output_List), POINTER :: OutLog => NULL(), FIRST_OUT

  Type ErrListEntry
     Character(Len=:), Allocatable:: Msg 
     Type (ErrListEntry), Pointer :: NextElem => null()
  End Type ErrListEntry
  
  Type ErrLst
     Type (ErrListEntry), Pointer :: FirstElem => null()
     Type (ErrListEntry), Pointer :: Elem => null()
     Contains
       procedure:: Add => AddErrMsg
       procedure:: Wrt => WriteErrMsg
  End Type ErrLst
  
Contains
   !***************************************************************************
  Subroutine AddErrMsg (this, Msg)
    Character (LEN = *) Msg
    Class (ErrLst), intent(inout) :: this
    
    If (.Not.Associated(this%FirstElem)) Then
       Allocate(this%FirstElem)
       this%Elem => this%FirstElem
    Else
       Allocate(this%Elem%NextElem)
       this%Elem => this%Elem%NextElem
    End If

    this%Elem%Msg = Msg
  End Subroutine AddErrMsg
  !***************************************************************************
  Subroutine WriteErrMsg (this, Rname, Stat)
    Use BasicTools, Only: ULC
    Character (LEN = *) Stat, Rname
    Class (ErrLst) :: this
    Type (ErrListEntry), Pointer :: Next
    ! Local variables
    Character (Len=:), Allocatable :: Status
     
    Nullify(this%Elem%NextElem)
    Status = Stat
    Call ULC (Status,'L2U')
    Select Case (Status)
    Case ('E')
       STATUS = 'ERROR'
    Case ('W')
       STATUS = 'WARNING'
    Case ('T')
       STATUS = 'TRACING'
    End Select
    
    Select Case (Status)
    Case ('ERROR', 'WARNING')
       write(eu,'(A)') ' *** '//status//' in '//Rname//': '
    Case ('TRACING')
       Write(EU, *) ' ---> Called by '//Rname
       Return
    End Select

    this%Elem => this%FirstElem
    Do While (Associated(this%Elem))
       write(eu,'(A)') '     '//this%Elem%Msg
       Next => this%Elem%NextElem
       DEALLOCATE (this%Elem%Msg)
       DEALLOCATE (this%Elem) ; NULLIFY (this%Elem)
       this%Elem => Next
    End Do
  End Subroutine WriteErrMsg 
  !***************************************************************************
  
END MODULE LOG_PARS

 !***************************************************************************
Module OutInf
  USE LOG_PARS
  Character (Len=:), Allocatable:: OutMode
  Logical   WrtModRes, DelayOut, OneOutFile, WrtRetPrf, WrtRetFit, LUF(2)
  TYPE Linked_List_Entry
     Character (Len=:), Allocatable:: S
     TYPE (Linked_List_Entry), POINTER :: Next_Elem
  END TYPE Linked_List_Entry
  TYPE Output_Data
     Integer RecL
     Character (Len=:), Allocatable:: Name
     TYPE (Linked_List_Entry), POINTER :: First_Elem, NEXT, ELEMENT => NULL() 
  END TYPE Output_Data
  TYPE (Output_Data), Allocatable ::  OutD(:)
End Module OutInf
 !***************************************************************************
MODULE LOG_CONTROL
  USE LOG_PARS, ERRORUNIT => EU
END MODULE LOG_CONTROL
!***************************************************************************
MODULE REFRACT_VARS
  USE TYPES,     ONLY: KND
  INTEGER  NL_REFR, NLR_FINE
  INTEGER, PARAMETER :: MAX_SUB = 700
  INTEGER, ALLOCATABLE :: IDX_REFR(:)
  LOGICAL  Refr_HiRes
  REAL (KND) H_NOREF_SOL, COS_NOREF_SOL, H_NOREF, COS_NOREF
  REAL (KND), ALLOCATABLE :: Z_REFR(:), REFR_IND(:), IT_FINE(:),  &
       D_RI(:), INT_TERM(:), RI_Z(:), D_RI_Z(:), RefI(:)
  REAL (KND)    P_UP, RHG_STEP, G_SZA_T, WL_REF
END MODULE REFRACT_VARS

!*************************************************************
MODULE RefrVar
  USE REFRACT_VARS
END MODULE RefrVar
!***************************************************************************
MODULE AEROSOLS
  USE TYPES,  ONLY: KND, DYN_CHAR
  LOGICAL    ECSTRA_FLAG, Scal_aer, AOT_out_LOG, AER_ADD_LEV, Dwr_mi
  LOGICAL    AER_SHIFT, OUT_A_SCM, EXA_FLG, COMPAT_MODE, L_scal, Use_AP, &
       New_lay, Plus_mi, Bi_PSD, EXA_OUT_FLAG, Use_der(2), MODIS_AP, &
       SHARP_EDGES, Aer_ext_lin
  INTEGER    LA_N, NWLS, NWLE, N_AER_WAVS, NG_AER, NOWL_AER, N_Dwr
  INTEGER    LA_Nmi, NAW_MI, LA_Ndb, Nga_mi, Nga_db, N_AOT
  INTEGER    NL_ECSTRA, LAY_A, WEL_A, NSCA_A, N_EXA_WL
  INTEGER, PARAMETER :: GT_SCAT_IDX = 1, GT_EXTC_IDX = 2, &  
       GT_ASYM_IDX = 3, NCOEFF_ECSTRA = 3
  INTEGER, ALLOCATABLE, TARGET :: LA_S(:), LA_E(:), IDX_AER(:), &
       IDX_AER_mi(:), IDX_AER_db(:)
  LOGICAL, ALLOCATABLE :: Mie_layer(:)
  CHARACTER (LEN=1)   GE_A
  CHARACTER (LEN=3)   INT_SCM_A
  CHARACTER (LEN=5)   MODIS_LO
  CHARACTER (LEN=:), ALLOCATABLE :: PATH_TO_SCM, EXA_NAME, AER_PARAM, RT_AG
  REAL (KND)  AOT_out_VAL, AER_SCALING, AER_FACT, ECSTRA_DM, Rwl_aer
  REAL (KND)  Rtau_aer, ZTROPO, Alph, C_a, ZS1, ZS2, V_PAP, Rf_max,&
       Rf_min, Rc_max, AP_par, AP_wlr, CRIT_SCM_A
  REAL (KND) :: FACTOR_EXA_S
  REAL (KND), ALLOCATABLE, TARGET ::  APH_VS(:,:,:,:), Greek_Aer(:,:,:,:), &
       Greek_Aer_mi(:,:,:,:), APH_VS_mi(:,:,:,:), D_Greek(:,:,:,:,:)
  REAL (KND), ALLOCATABLE :: AP_Mie(:,:), AOT_MAL(:), ZA_SE(:), AER_WAVS(:),&
       AER_COEFF(:,:,:), AW_MI(:), AC_MI(:,:,:), AER_RET_FACTOR(:,:), &
       Z_ECSTRA(:), BETA_VERT(:), L_TOP(:), ACOEFF_ECSTRA(:,:), EXA_OUT(:,:),&
       EXA_AC(:,:), EXA_WL(:), OWL_AER(:), EXA_OUT_WL(:), EXA_RET(:,:,:), &
       EXA_RET_WL(:), AER_DER(:,:,:,:), APH_DER(:,:,:,:,:), SCR_WL(:,:),&
       Rpar(:,:), Vpar(:,:), Ri_ab(:), Ri_du(:,:), R_erw(:,:)
  TARGET :: EXA_RET, EXA_RET_WL, EXA_AC, EXA_WL, AER_WAVS, AER_COEFF, AW_MI,&
       AC_MI, L_TOP, AER_DER
  Type AerExt_DBG
     Logical InitF
     Integer MS_2000
     Real (KND), Allocatable :: Lats(:), Lons(:)
     Type (DYN_CHAR), Allocatable :: Data(:)
  End type AerExt_DBG
  Type (AerExt_DBG) :: AerExC_DB
  
End MODULE AEROSOLS

!***************************************************************************
MODULE SPHER_PARS
  USE TYPES,  ONLY: KND
  CHARACTER (LEN=:), ALLOCATABLE :: ANG_SEL_FLAG, LOS_DEF_FLAG, &
       FOV_SHP, PE_MODE
  LOGICAL    SET_OUT_ALT, DO_SF_OUT, FOV_FLAG
  LOGICAL    ONE_MS_SZ, Check_STA, Const_ang
  INTEGER    N_ITER, I_FINE, I_FINE_SOL, NN, N_FINE_ST, N_FINE_TH
  INTEGER    N_LOS_OUT, N_SOL_ANG, N_LOS_ANG, N_AZ_ANG
  INTEGER    N_FOV_ANG, N_SHIFT, N_AZ_VAR, NBE_INP, NAF_INP
  INTEGER    NFOV_WNG, NFOV_CORE
  INTEGER, ALLOCATABLE :: JV_S(:), JV_E(:), JS_E(:), N_OBS_ALT(:), LW_FOV(:)
  REAL (KND), PARAMETER :: DELTA_TH = -0.5D0
  REAL (KND), ALLOCATABLE :: ANG_OUT(:,:), ANG_ORIG(:,:), LOS_ANG(:), &
       ANG_OUT_S(:,:), ALTITUDE(:), AZM_MS(:), FOV_WEIGHTS(:), ALT_OUT(:)
  REAL (KND),ALLOCATABLE,TARGET :: ANG_FOV(:,:,:), ANG_FOV_S(:,:,:) 
  REAL (KND)  X_SHAD, Z_F, Z_SOL_F, DEL_INP, Crit_STA, FOV_FULL, FOV_ANG
  Type SolarGrid
     Character (LEN=:), Allocatable:: Type
     Integer NAng
     Real (Knd) AngMin, AngMax, PR
  End Type SolarGrid
  Type (SolarGrid) SolGrid
END MODULE SPHER_PARS

!*************************************************************
MODULE LOS_PARS
  USE TYPES,  ONLY: KND
  REAL (KND), ALLOCATABLE :: LS_VA(:), LS_AA(:), LS_SA(:)
END MODULE LOS_PARS

!*************************************************************
MODULE ALT_INPUT
  USE TYPES,  ONLY: KND
  LOGICAL    ALIN_FLAG, ALIN_ACT(4), SOUT_FLAG, SOUT_TAU, IPRT, IPRT_ALIN
  INTEGER    NC_AER, NC_CLD, ALIN_NWL, ALIN_NZ
  CHARACTER (LEN=:), ALLOCATABLE :: DIR_SOUT
  REAL (KND), ALLOCATABLE :: ALIN_ABS(:,:),  ALIN_WL(:), ALIN_RAY(:,:), &
       ALIN_RAY_EC(:,:), ALIN_Z(:), ALIN_AER_A(:,:), ALIN_AER_S(:,:), &
       ALIN_AER_EC(:,:), ALIN_CLD_A(:,:), ALIN_CLD_S(:,:), ALIN_CLD_EC(:,:)
END MODULE ALT_INPUT

Module AltInp
  Use ALT_INPUT
End Module AltInp

!*************************************************************
MODULE NODES
  Use Types, Only: Knd
  Integer N_NODE, N_PLUS, N_MINUS, NAz, NVA, N_Az, N_Va, NbN, &
       N_HIN, NAz_HIN, NVA_HIN
  Logical READ_NODES, NEW_NODES, USE_AIN, USE_HIN
  Character (LEN=:), Allocatable :: FILE_NODES, ModeNodes
  Real (Knd) NORM_DN, NORM_BN
  Real (Knd), ALLOCATABLE, TARGET :: PHI_NODE(:), MJU_NODE(:) 
  Real (Knd), ALLOCATABLE, TARGET :: WEIGHTS(:), SIN_NODE(:)
  Real (Knd), ALLOCATABLE, TARGET :: PHI_HIN(:), MJU_HIN(:) 
  Real (Knd), ALLOCATABLE, TARGET :: WEI_HIN(:), SIN_HIN(:)
  Real (Knd), POINTER :: MJU_MINUS(:), MJU_PLUS(:), &
       PHI_MINUS(:), W_MINUS(:), W_PLUS(:)
  Real (Knd), ALLOCATABLE:: Az_A(:), W_Az(:), Vz_A(:), W_Vz(:), &
       VbN(:), PbN(:), WbN(:)
END MODULE NODES

!*************************************************************
MODULE GRIDS
  USE TYPES,  ONLY: KND
  INTEGER  NSOL, NLAYERS, NL_FINE, MAX_LINE, N_WF_L
  INTEGER  NLAYERS_INP, NDEP_INP, NZ_LEV
  INTEGER, ALLOCATABLE :: INTERP_IDX(:)
  LOGICAL, ALLOCATABLE :: ALT_MARK(:), GRID_MARK(:), DEP_MARK(:), GRID_MARK_UW(:)
  REAL (KND), ALLOCATABLE :: ZZZ(:), Z_INV(:), Z_FINE(:), X_INV(:), SZA_EXT(:), &
       ALT_EXT(:), DZR(:), DZ(:), ZZZ_INP(:), DEP_INP(:), Z_LEV(:), Zl_step(:)
  TARGET :: ZZZ, DZ
  INTEGER, PARAMETER :: NSOL_INT = 24
  REAL (KND)  Z_TOA, Z_BOA
  REAL (KND), ALLOCATABLE :: SOL_GRD(:)  
  REAL (KND)  XM_S_INIT(NSOL_INT)
  DATA XM_S_INIT &
       /1.0D0, 0.984808d0, 0.85D0, 0.55D0, 0.3D0, 0.19D0, 0.09D0, 0.03D0, &
       0.003D0, 1.0D-9, - 0.0078D0, - 0.018D0, - 0.023D0, - 0.0309D0, &
       - 0.0399D0, - 0.047D0, - 0.0548D0, - 0.0705D0, - 0.085D0, &
       - 0.099D0, - 0.1156D0, - 0.128D0, - 0.1361D0, - 0.14090123D0/
END MODULE GRIDS

!*************************************************************
MODULE SPHER_CONTROL
  USE SPHER_PARS
END MODULE SPHER_CONTROL

!*************************************************************
MODULE GAS_CONTROL
  USE ABSORBERS
END MODULE GAS_CONTROL

!*************************************************************
MODULE PHOTOCHEM
  LOGICAL            PC_LOG_FLAG
  INTEGER            N_SOL_PC, N_PC_GASES, N_PC_INP
  CHARACTER (LEN = 150), ALLOCATABLE :: ALT_GRID_NAME_PC(:), PROFILE_NAME_PC(:)
  LOGICAL, ALLOCATABLE :: DO_TROP(:), CONST_PROF(:), DO_PHOTOCHEM(:)
  INTEGER, ALLOCATABLE :: GAS_IDX_PC(:), PC_GAS_INDEX(:)
  DOUBLE PRECISION   CONST_SZA, Z_TP
  DOUBLE PRECISION, ALLOCATABLE ::  XS_PC(:), PCY_LOC(:), CONC_PC(:,:,:)
END MODULE PHOTOCHEM

!*************************************************************
MODULE PHOTOCHEMISTRY
  USE PHOTOCHEM
END MODULE PHOTOCHEMISTRY

!*************************************************************
MODULE CROSS_SECTIONS
  USE GAS_CONTROL, ONLY : N_XSEC_MOL
  INTEGER             N_TEMP
  INTEGER, ALLOCATABLE :: ABS_COEFF_NTEMP(:)
  LOGICAL             DO_O3TEMP_GOMEFM, DO_XSECTION_AKIMA
  LOGICAL, ALLOCATABLE :: BP_FLAG(:), SR_H_FLAG(:), O3_UV_CS(:)
  CHARACTER (LEN=100) O3_UV_CS_FILE
  CHARACTER (LEN=100), ALLOCATABLE :: X_SECTION_FILE(:,:)
  CHARACTER (LEN=:), ALLOCATABLE :: X_SECTION_PATH
  DOUBLE PRECISION, ALLOCATABLE :: ABS_COEFF_TEMP(:,:), &
       ABS_COEFF_O3(:,:), O3_BP_COEFF12(:), O3_BP_COEFF13(:), ABS_COEFF_O2(:,:)
  DOUBLE PRECISION, ALLOCATABLE, TARGET :: ABS_COEFF(:,:,:)
  DOUBLE PRECISION    O3_UV_START, O3_UV_END
END MODULE CROSS_SECTIONS

!*************************************************************
MODULE X_SECTS
  USE CROSS_SECTIONS
  LOGICAL RAY_EXT
END MODULE X_SECTS

!*************************************************************
MODULE ESFT
  CHARACTER (LEN=:), ALLOCATABLE :: ESFT_DB_PATH
  LOGICAL             DO_ESFT, DO_ESFT_ANTI_CORRELATION
  LOGICAL, ALLOCATABLE :: DO_ESFT_SPEC_MOL(:,:)
  INTEGER, PARAMETER :: N_ESFT_COEFF = 20
  INTEGER, ALLOCATABLE :: ESFT_NCOEFF(:,:), ESFT_OVERLAP_MODE(:),&
       ESFT_MOL_OVERLAP_IDX1(:), ESFT_MOL_OVERLAP_IDX2(:)
  DOUBLE PRECISION, ALLOCATABLE :: ESFT_ALPHA(:), ESFT_COEFF(:,:,:,:), &
       ESFT_WEIGHTS(:,:,:), D_ESFT_COEFF_DP(:,:,:,:), D_ESFT_COEFF_DT(:,:,:,:)
END MODULE ESFT

!*************************************************************
MODULE LBL
  USE TYPES,  ONLY: KND
  LOGICAL             DO_LBL, OptLineSum
  INTEGER, PARAMETER :: N_HT_MOLS = 55, MAX_ISO = 20, NT = 4510
  INTEGER             NSEGS_LBL, N_TEMP_SETS, N_ISO(N_HT_MOLS)
  Real (Knd), Parameter :: HT_C2 = 1.4387768775039338D0 ! hc/k
  ! Maximum wavenumber to account for the stimulated emission
  Real (Knd)  StEm_MaxWn, LineSumAcc
  Real (KND), ALLOCATABLE :: SEG_WL(:,:), Q_RATIO(:,:,:,:)
  Real (KND)    TDAT(NT), C (31), LBL_WING_CUTOFF
  Real (KND), PARAMETER :: PT_SCALE = 1.001D0
END MODULE LBL
!*************************************************************
MODULE HYDROSOLS
  USE TYPES,  ONLY: KND, DYN_CHAR
  LOGICAL     Scal_HS, OUT_H_SCM
  CHARACTER (LEN=1)  GE_H
  CHARACTER (LEN=3)   INT_SCM_H
  CHARACTER (LEN=3), ALLOCATABLE :: SEL_VSC(:)
  CHARACTER (LEN=:), ALLOCATABLE :: PATH_H_SCM
  TYPE (DYN_CHAR), ALLOCATABLE :: HSOL_MOD(:)
  INTEGER     NG_HS, LA_N_HS, NWLS_HS, NWLE_HS, N_HPC, N_HM, LAY_H, &
       WEL_H, NSCA_H
  INTEGER, ALLOCATABLE :: LA_S_HS(:), LA_E_HS(:), IDX_HS(:)
  REAL (KND)  Rtau_HS, Rwl_HS, HS_SCALING, CRIT_SCM_H
  REAL (KND), ALLOCATABLE, TARGET :: Greek_HS(:,:,:,:,:)
  REAL (KND), ALLOCATABLE :: L_BOT_HS(:)
  
END MODULE HYDROSOLS

!*************************************************************
MODULE AER_MIE
  USE TYPES,  ONLY: KND
  CHARACTER (LEN=30)  Nam_f, Nam_c
  CHARACTER (LEN=100) Sph_Name
  INTEGER  Nst, N_01, M_AC
  LOGICAL  Rm_log, PM_2i5, PM_2i5_MOD
  LOGICAL, ALLOCATABLE :: L_ARR(:,:,:), F_MODE(:)
  INTEGER, ALLOCATABLE :: NAC(:), N_PSD(:), LAS_mi(:), LAE_mi(:)
  REAL (KND)  Rmi_lim(2), T_550, Cfr, R_560(100), T_Black
  REAL (KND), ALLOCATABLE :: MOD_R(:,:), SIG(:,:), RD(:,:),& 
       RI_rea(:,:), RI_ima(:,:), COEF1(:), COEF2(:), DENS_PRF(:),&
       COEF3(:), COEF4(:), COEF5(:), COEF6(:), COEF7(:), COEF8(:),&
       AOD_MIE(:,:), WL_RIa(:), ALL_PIX(:,:), SAVE_PIX(:),&
       AOD_RET(:,:), DIF_RET(:,:), Dif_885(:,:), Ac_par(:)
END MODULE AER_MIE

!*************************************************************
MODULE BRDF
  USE TYPES,  ONLY: KND
  INTEGER   Nwl_BRF, Mh_BRF, N_beta, Nwl_veg, N_Sph_WL, Npg, &
       Nwl_RLi, Nder, Nice, NMOM_ICE, Nrso, Nrvi, Nraz
  INTEGER, PARAMETER :: N_BRDF_PARAM = 6, N_OC_COMP = 3
  LOGICAL  OUT_BRDF, Rep_cosine, Exact_BRF, AB_DB, Use_LRC, Sep_IR, &
       DO_OC_COMP(N_OC_COMP), Ref_to_Abs, L_yellow
  LOGICAL,    ALLOCATABLE :: L_SOL(:)
  CHARACTER (LEN=1)   VS_OC_COMP(N_OC_COMP)
  CHARACTER (LEN=200) RG_name(2)
  CHARACTER (LEN=:), ALLOCATABLE ::  Ro_Li_name, Spha_veg, Spha_soi,&
       Ref_type, INP_BRDF, Ex_Type, Leav_WL, Bet_name, Alph_name, RefF_name, &
       RefW_name, Path_for_BRDF, Foam_WL, IOP_name, Iref_name, Err_RLC_name, &
       B_RLC_name, Pond_type, SCM_name, Yang_DB, RDB_dir, RDB_ref, RDB_ang
  REAL (KND), ALLOCATABLE :: WL_BRF(:), BRF_GG_WL(:,:,:,:), BRF_GS_WL(:,:,:,:), &
       Beta_ang(:), Beta_val(:), WL_veg(:), Alb_svk(:), IVG_cof(:,:), Sph_OUT(:), &
       Sph_WL(:), Al_soi(:), Al_veg(:), X_brf(:), Alph_1(:), Alph_2(:), Beta_1(:), &
       Beta_2(:), Der_GS_WL(:,:,:,:,:), LRC(:,:,:), Der_GG_WL(:,:,:,:,:), &
       WL_refi_idx(:), Refi_idx(:), RLC_RET(:,:,:), D_LRC(:,:,:), MP_gp(:), &
       MP_bub(:), MP_bri(:), SCA_ANG(:), SCA_MAT(:), Sol_r0(:), Vie_r0(:), &
       Azm_r0(:), R0_db(:)
  REAL (KND), ALLOCATABLE, TARGET :: RefW_r(:), RefW_i(:), RefF_i(:), AlfW(:), &
       P_yell(:), Lin_rel(:), Par_mix(:,:), MOM_ICE(:,:,:), Snow_abs(:), &
       ERR_RL(:,:), B_RL(:,:,:), RefI_r(:), RefI_i(:)
  REAL (KND)  BRDF_PARAM(N_BRDF_PARAM), Alf0, RefW_r0, HS_par
  REAL (KND)  RefW_i0, RefF_i0, Abs_pw, Abs_chl, LWC_Foam, NDVI, Bubble_Dim, Cfr_v, &
       N_svk(2), Cr_ang, Ref_ip, SC_sig, Sig_t, SCM_WL, Tau_sl, Yellow_390
  REAL (KND), PARAMETER :: WL0 = 440.0d0, Wgr_svk = 700.0d0
END MODULE BRDF

!*************************************************************
MODULE ALB_PARS
  USE TYPES,  ONLY: KND
  LOGICAL    DO_SPECTRAL_ALBEDO, F_ave
  INTEGER    N_ALBEDO_WAVS_DB, Nc_ALB, WS1_A, WS2_A, N_Lca
  INTEGER    INF_ALB(3)
  REAL (KND) ALBEDO_CONST, SCAL_ALB, DPOSALB, WL_Lca, Min_Lca, Max_Lca
  REAL (KND), ALLOCATABLE :: ALBEDO_DB(:,:), ALBEDO_WAVS_DB(:), ALBEDO_RET(:,:),&
       ALB_WEI(:,:), ALBEDO_PERTR(:), V_Lca(:), GR_Lca(:), ALB_REF(:,:)
END MODULE ALB_PARS

!*************************************************************
MODULE ANISOTROPY
  USE TYPES,  ONLY: KND
  CHARACTER (LEN=:), ALLOCATABLE :: DEPOL_FILENAME
  LOGICAL   DO_DEPOL_WAVELENGTH
  INTEGER, PARAMETER :: DEP_NWS = 37
  REAL (KND)  DEPOL_INPUT
  REAL (KND), ALLOCATABLE :: DEP_WS(:), DEP_VS(:)
END MODULE ANISOTROPY

!*************************************************************
MODULE EMISSION
  USE TYPES,  ONLY: KND
  INTEGER     N_AT
  REAL (KND), ALLOCATABLE :: Emi_MOMS(:,:)
END MODULE EMISSION

!*************************************************************
MODULE DEPOLARIZATION_ALBEDO_AEROSOL
  USE ANISOTROPY
  USE BRDF
  USE ALB_PARS
END MODULE DEPOLARIZATION_ALBEDO_AEROSOL

!*************************************************************
MODULE RAMAN
  USE TYPES,  ONLY: KND
  CHARACTER (LEN=:), ALLOCATABLE :: RRS_NAM_ET, RRS_NAM_PT
  LOGICAL  RRS_ADJ, RRS_DO, Phot_to_Wat, DOM_API, ESR_Split
  INTEGER, PARAMETER :: N_RG = 2
  INTEGER  RAMAN_JBEG(N_RG), RAMAN_JDELTA(N_RG), NR_LINES, N_WL_C, NS_rrs, K_dim
  INTEGER, ALLOCATABLE :: LINE_GAS(:), Nc_rrs(:) 
  DATA RAMAN_JBEG, RAMAN_JDELTA  /1,0,2,1/
  REAL (KND)  WL_FACT, DEL_RAM, RAMAN_B0(N_RG), SPIN_WEI(N_RG,2)
  REAL (KND), ALLOCATABLE :: R_GAM(:,:), O2_CONC(:), DEL_WL(:), Sigma_RR(:,:), &
       Ram_Egg(:,:,:,:), Ram_EgL(:,:,:,:), Ram_Egs(:,:,:,:), Ram_EsL(:,:,:,:)
  DATA SPIN_WEI  /0.0D0, 6.0D0 , 1.0D0, 3.0D0/
  DATA RAMAN_B0 /1.4378D2 , 1.98973D2/
  REAL (KND), PARAMETER :: CtimesH =  2.9979D+08*6.6262D-34, &
       Boltzmann = 1.3806488D-23, PPMV_N2_O2 = 3.7363636D0, INCR_RANGE = 10.d0
END MODULE RAMAN

!*************************************************************
MODULE VRS
  USE TYPES,  ONLY: KND
  CHARACTER (LEN=:), ALLOCATABLE ::  Contour
  LOGICAL  Integr_FRF, Tvs_Use, Add_TRS(2)
  INTEGER  N_phm, Nh
  INTEGER, PARAMETER :: N_FRF = 4
  REAL (KND), PARAMETER :: K_nu = 1.d+7 ! [nm/cm]
  REAL (KND)  WLG_Chl, Sig_Chl, Qe_Chl, A_FChl
  ! Parameters of Raman frequency redistribution function (FRF)
  REAL (KND)  A_FRF(N_FRF), Dnu_FRF(N_FRF), Sig_FRF(N_FRF)
  DATA A_FRF   /0.41d0, 0.39d0, 0.1d0, 0.1d0/
  DATA Dnu_FRF /3250.d0, 3425.d0, 3530.d0, 3625.d0/     ![cm-1]
  DATA Sig_FRF /89.179d0, 74.317d0, 59.453d0, 59.453d0/ ![cm-1]

  ! --- Parameters according to Kattawar & Xu (AO 1992, v.31) ------
  !        A_ref is set as Ai/Nu_i^Hw 
  !  DATA A_FRF   /0.278970d-2, 0.352273d-2, 0.104167d-2, 0.102837d-2/
  !  DATA Dnu_FRF /3247.d0, 3435.d0, 3535.d0, 3622.d0/     ![cm-1]
  !  DATA Sig_FRF /98.946d0, 74.7403d0, 61.1512d0, 59.8772d0/ ![cm-1]
  ! ------------------------------------------------------------------
  REAL (KND)  STEP_VRS, Nsig, Tvs_Exc, Tvs_Act, Rho, Lam_ref, Bet_0R, Lam_int
  REAL (KND), ALLOCATABLE :: VRS_Egg(:,:,:,:), VRS_EgL(:,:,:,:), VRS_Egs(:,:,:,:),&
       VRS_EsL(:,:,:,:), WL_QEred(:), WL_QEblu(:), WLH(:), Apeh(:)
  TARGET :: VRS_Egs
END MODULE VRS
 
!*************************************************************
MODULE IRRAD
  USE TYPES,  ONLY: KND
  INTEGER    N_SOL_HR
  LOGICAL    IRRAD_FLAG, NO_SUN
  LOGICAL, ALLOCATABLE :: ACT_WL_FLAG(:)
  CHARACTER (LEN = 8 )  WL_IRRAD_DIM(5)
  CHARACTER (LEN = :), ALLOCATABLE :: SOL_HR_FILE, FILENAME_SOLSPEC, EXTR_FLUX
  REAL (KND) Fbe, SCALE_SS
  REAL (KND), ALLOCATABLE, TARGET :: SOL_FLUX(:), SOLSPEC_OUT(:), SOLAR_HR(:), &
       WL_SOL_HR(:)
END MODULE IRRAD

!*************************************************************
MODULE IRRADIANCE
  USE IRRAD
END MODULE IRRADIANCE

!*************************************************************
MODULE CLD_INP
  USE TYPES,     ONLY: KND, DYN_CHAR
  INTEGER  WF_CLD, Nphr, N_Rdb, N_ref_sm
  LOGICAL  Use_Dim
  CHARACTER (LEN=:), ALLOCATABLE :: D_MF
! Effective radius and cross-section (at 500 nm) of water
! -------------------------------------------------------
  INTEGER, PARAMETER:: N_ref_db = 20
  DOUBLE PRECISION     Ref_DB(N_ref_db), Ext_DB(N_Ref_db)
  TARGET :: Ref_db
  DATA Ref_DB / &
       2.d0,  4.d0,  6.d0,  8.d0, 10.d0, 12.d0, 14.d0, 16.d0,&
       18.d0, 20.d0, 22.d0, 24.d0, 26.d0, 28.d0, 30.d0, 32.d0,&
       34.d0, 36.d0, 38.d0, 40.d0 /  
  DATA Ext_DB / &
       0.1963225D+02, 0.7502592D+02, 0.1657808D+03, 0.2917167D+03, &
       0.4528533D+03, 0.6489949D+03, 0.8802219D+03, 0.1146456D+04, &
       0.1447561D+04, 0.1783836D+04, 0.2154863D+04, 0.2560929D+04, &
       0.3001912D+04, 0.3477738D+04, 0.3988207D+04, 0.4532858D+04, &
       0.5110783D+04, 0.5720430D+04, 0.6359436D+04, 0.7024544D+04 /  
  CHARACTER (LEN=2)   S_name(N_ref_db)
  DATA S_name / '2',  '4',  '6',  '8', '10', '12', '14', '16', '18', &
       '20', '22', '24', '26', '28', '30', '32', '34', '36', '38', '40'/
  ! Effective radius and extinction [m^2/g] (at 500 nm) ice Baum DB
  ! ---------------------------------------------------------------
  INTEGER, PARAMETER:: N_ref_Bdb = 23
  REAL (KND), TARGET :: Ref_Bdb(N_ref_Bdb)
  REAL (KND)  Ext_Bdb(N_Ref_Bdb), TotA_Bdb(N_Ref_Bdb)
  DATA Ref_Bdb / 5.0d0,  7.5d0, 10.0d0, 12.5d0, 15.0d0, 17.5d0, 20.0d0, &
       22.5d0, 25.0d0, 27.5d0, 30.0d0, 32.5d0, 35.0d0, 37.5d0, 40.0d0, &
       42.5d0, 45.0d0, 47.5d0, 50.0d0, 52.5d0, 55.0d0, 57.5d0, 60.0d0 /  
  DATA Ext_Bdb / 0.28910d0, 0.20880d0, 0.16210d0, 0.12250d0, 0.10180d0, &
       0.08709d0, 0.07605d0, 0.06750d0, 0.06069d0, 0.05511d0, 0.05053d0, &
       0.04658d0, 0.04322d0, 0.04033d0, 0.03781d0, 0.03555d0, 0.03349d0, &
       0.03177d0, 0.03019d0, 0.02872d0, 0.02744d0, 0.02623d0, 0.02514d0 /  
  CHARACTER (LEN=4)   S_name_Bdb(N_ref_Bdb)
  DATA S_name_Bdb /'5.00', '7.50', '10.0', '12.5', '15.0', '17.5', '20.0', &
       '22.5', '25.0', '27.5', '30.0', '32.5', '35.0', '37.5', '40.0', '42.5', &
       '45.0', '47.5', '50.0', '52.5', '55.0', '57.5', '60.0' /  
  ! Side length and cross-section (at 500 nm) of Hexagonal prism
  ! -------------------------------------------------------------
  INTEGER, PARAMETER:: N_ref_Hex = 6
  REAL (KND), TARGET :: Ref_Hex(N_ref_Hex)
  REAL (KND)  Ext_Hex(N_Ref_Hex), aer_ref
  DATA Ref_Hex /12.5D0, 25.D0, 50.d0, 200.D0, 400.D0, 800.D0/
  DATA Ext_Hex /0.4150945D+04, 0.9045247D+04, 0.2139861D+05,&
       0.1635004D+06, 0.5297312D+06, 0.1936024D+07/  
  CHARACTER (LEN=4), TARGET::   S_name_Hex(N_ref_Hex)
  DATA S_name_Hex /'12.5', '25', '50', '200', '400', '800'/
  ! Edge length and cross-section (at 500 nm) of fractal
  ! ----------------------------------------------------
  INTEGER, PARAMETER:: N_ref_Fra = 3
  REAL (KND), TARGET :: Ref_Fra(N_ref_Fra)
  REAL (KND)  Ext_Fra(N_Ref_Fra)
  DATA Ref_Fra /50.d0, 100.D0, 300.D0 /
  DATA Ext_Fra /0.3294182D+04, 0.1386881D+05, 0.1171024D+06/
  CHARACTER (LEN=4), TARGET::   S_name_Fra(N_ref_Fra)
  DATA S_name_Fra /'50', '100', '300'/
  !--------------------------------------------------------------------
  CHARACTER (LEN=:), ALLOCATABLE :: CLOUD_DATA_BASE_PATH, CLD_INHOM,&
       HOW_CLD_COEF, Refr_water_inp, Refr_ice_inp
  CHARACTER (LEN=4),  ALLOCATABLE :: MAN_AUTO_CLD(:) 
  TYPE (DYN_CHAR), ALLOCATABLE :: TDS(:), SUB_LAYERING(:), &
       PHASE_NAME(:), USER_EK_NAME(:,:), Path_PH_CLD(:), DBC_NAME(:,:), &
       Inp_form(:), Add_Abs_mod(:)
  CHARACTER (LEN=8),  ALLOCATABLE :: S_name_YDB(:,:) 
  LOGICAL   OUT_PH_CLD, Phm_const, Deriv_Phc, Use_Bdb, Use_Ydb, Use_Ydb_SCM, &
       USE_PSD
  LOGICAL, ALLOCATABLE :: Plus_ice(:), ADD_ABS(:)
  INTEGER, PARAMETER:: Ydb_ang = 498
  INTEGER Nr_db(2), N_ref_Ydb, N_Ydb, N_Par_mod
  INTEGER,  ALLOCATABLE :: Nf_ice(:), N_ref_ice(:)
  REAL (KND), ALLOCATABLE :: Ref_Ydb(:,:), Vol_Ydb(:,:), Dim_Ydb(:,:), &
       Ext_Ydb(:,:), Ref_ice(:,:), Fd(:,:), D_FU(:)
  TARGET :: Ref_Ydb, Ref_ice, Dim_Ydb
  REAL (KND), ALLOCATABLE :: LC_CLD(:), UC_CLD(:), CON_VAL_CLD(:)
  Type PhaseProf
     Integer Ndb_ref
     Logical Y_Scm
     REAL (KND), ALLOCATABLE :: Pdb_ref(:)
  End type PhaseProf
  TYPE Multi_layer
     Integer Na_Ydb, Nd_s, Nd_e, N_pha, N_WL_SOOT, NZ_SOOT
     LOGICAL Phm_co
     REAL (KND), ALLOCATABLE :: WL_SOOT(:), REFR_SOOT(:,:), Z_soot(:), C_soot(:)
     TYPE (DYN_CHAR), ALLOCATABLE :: Name_Ydb(:)
     TYPE (PhaseProf), Allocatable :: Npf(:)
  END TYPE Multi_layer
  TYPE (Multi_layer), Allocatable, TARGET ::  MLCS(:)

  Type IcePartShapes
     CHARACTER (LEN=:), ALLOCATABLE :: ICE_FORM, P_ROU, ICE_DIR
     REAL (KND), Allocatable :: F_HAB(:), Fr_hab(:)
  End type IcePartShapes
  Type IceCrystalStr
     REAL (KND), Allocatable :: DGR(:)
     TYPE (IcePartShapes), Allocatable :: SHP(:)
  End type IceCrystalStr
   TYPE (IceCrystalStr), Allocatable ::  CRST(:)
END MODULE CLD_INP

!*************************************************************
MODULE CLOUD_INPUT
  USE CLD_INP
END MODULE CLOUD_INPUT

!*************************************************************
MODULE CLOUDS
  USE TYPES,     ONLY: KND, DYN_CHAR
  LOGICAL              DO_CLOUD, Red_WL_Ydb
  INTEGER              N_CLD_LAYERS, N_CLD_NETWAVS, NG_cld, N_Red_WL
  INTEGER, ALLOCATABLE :: N_CLDPROF(:), CLD_DM(:), J_TOP(:), J_BOT(:), CLD_LAYER_NUM(:)
  REAL (KND), ALLOCATABLE :: CLD_LAYER_ARR(:,:), CLD_NETWAVS(:), CLD_LAYER_HEIGHTS(:,:),& 
       TAU_CLD_INP(:,:), IWP(:), CLDPROFILE(:,:,:), PROF_VARS(:), REF_IMAG_W(:), &
       REF_IMAG_I(:), LWP(:), F_Dfit(:,:), Prof_Ref(:,:), Red_WL(:)
  REAL (KND), ALLOCATABLE, TARGET :: CLD_LAYER_COEFF(:,:,:,:), CLD_SAVE_COEFF(:,:,:,:),&
       SCAT_MAT(:,:,:,:,:,:), SCAT_ANG(:), CLD_LAYER_DERIV(:,:,:,:), CLD_F_DM(:,:,:,:), &
       PR_SOOT(:), SCC_LAYER_NOSCALE(:,:,:), CLD_PHAS_MOMS(:,:,:,:,:,:), &
       SCI_LAYER_NOSCALE(:,:,:), Greek_cld(:,:,:,:,:,:), Prof_Cpar(:,:), &
       RefI_r(:), RefI_i(:)
  REAL (KND) EPS_CLD, GT_TL, Ref_ip
  REAL (KND), PARAMETER :: INF_LAYER = 1.0D-04
  CHARACTER (LEN=:), ALLOCATABLE :: SCALE_PROF, TRANS_LAY
  TYPE (DYN_CHAR), ALLOCATABLE ::  Var_type(:,:)
END MODULE CLOUDS

!*************************************************************
MODULE CLOUDS_SS
  LOGICAL           DO_SSRCA, DO_RAY_DEP, DO_RAY_TAU
  INTEGER, PARAMETER :: MAXMOM=5000
  DOUBLE PRECISION  RAY_DEP, RAY_TAU_INP
  DOUBLE PRECISION, ALLOCATABLE :: R_INF_WL(:), G_CLD_WL(:)
END MODULE CLOUDS_SS

Module Cld_SS
  Use CLOUDS_SS
End Module Cld_SS

!*************************************************************
MODULE CLD_RET
  USE TYPES,     ONLY: KND, DYN_CHAR
  INTEGER    MAX_RET_PIX, Ngp_CLD
  PARAMETER  ( MAX_RET_PIX = 100 )
  LOGICAL    RET_TOP, WRT_GT, DO_EXP, RET_STATUS, NEW_CLD
  LOGICAL, ALLOCATABLE :: Ref_wi(:)
  CHARACTER (len=20) CLD_SCENE, RET_QUALITY
  CHARACTER (len=40) PIX_NAME( MAX_RET_PIX )
  CHARACTER (len=3)  GCP(2)
  REAL (KND)  REFL_EXP, D_CTH, D_GT, CFR
  REAL (KND)  GT_MIN, GT_MAX, Lower_ref, CTH_CRIT, &
       CTH_MIN, CTH_MAX, CBH_MIN, CBH_MAX, Aef_fra, Edg_fra
END MODULE CLD_RET

!*************************************************************
MODULE CLOUDS_RETR
  USE CLD_RET
END MODULE CLOUDS_RETR

!*************************************************************
MODULE DELTA_M
  Logical             DO_HSOL_DM, AER_DM_FLG
  Integer             AER_TRUNC_IDX, HSOL_TRUNC_IDX
  CHARACTER (LEN=4)   D_MF_AER
  DOUBLE PRECISION, ALLOCATABLE :: FDM_AER(:,:), F_DLM_HSOL(:,:,:), FDM_AER_mi(:,:)
  TARGET :: FDM_AER, FDM_AER_mi
END MODULE DELTA_M

!*************************************************************
MODULE GT_POLYN
  INTEGER            NLEGEN
  DOUBLE PRECISION, ALLOCATABLE :: PLM(:,:), PLMM(:,:), & 
       AG(:), AG1(:), AXM(:), AG_W(:), SGN(:), AXM_W(:)
  DOUBLE PRECISION, ALLOCATABLE, TARGET :: XM(:), XM_W(:), &
       Plm_G(:,:,:),  Plm_L(:,:,:), Plm_S(:,:,:), & 
       Dp2_G(:,:,:),  Dm2_G(:,:,:), Dp2_L(:,:,:), Dm2_L(:,:,:), &
       Dp2_S(:,:,:),  Dm2_S(:,:,:), Dp2_ss(:,:), Dm2_ss(:,:), Plm_ss(:,:),  &
       Plm_GW(:,:,:), Plm_LW(:,:,:), Plm_SW(:,:,:), Plm_Sm(:,:,:), &
!      ... Vector case COA model
       Dp2_GW(:,:,:),  Dm2_GW(:,:,:), Dp2_LW(:,:,:), Dm2_LW(:,:,:), &
       Dp2_SW(:,:,:),  Dm2_SW(:,:,:)
END MODULE GT_POLYN

!*************************************************************************
MODULE PRO_SAIL
  USE TYPES,  ONLY: KND
  INTEGER  TypeLidf, NW_S, NW_E
  REAL (KND)   LIDFa, LIDFb, LAI, hspot, Stc, Cab, Car, Cbrown, Cw, Cm, Psoil
END MODULE PRO_SAIL

!*************************************************************************
MODULE RTM_OUTPUT
  USE TYPES,  ONLY: KND
  INTEGER  NUM_RAY, N_VOD, N_OP, NU_VEC_PARS, N_LOS
  INTEGER  NR_UWT, N_CHL, NP_Us, NP_Ks, NP_Kl
  INTEGER  IND_NUM_WF, NL_OUT, N_RTM_ARR
  INTEGER  MAX_AER_WF
  LOGICAL  WF_TG_FLAG, DO_VOD_ONLY, NO_RAD_OUT
  LOGICAL  MES_OUT_LS, MES_OUT_SOL, TRANS_OUT, ALB_OUT
  LOGICAL :: EXA_T_FLG = .FALSE.
  LOGICAL, ALLOCATABLE, TARGET :: OWF_FLG_ARR(:)
  CHARACTER (LEN=:), ALLOCATABLE :: WF_INTEGR, WF_UNIT
  CHARACTER (LEN=20), ALLOCATABLE :: OWF_LABEL_V(:)
  REAL (KND)   WUP, MIN_WF_Z, MAX_WF_Z
  REAL (KND), ALLOCATABLE:: WF_TO_AMF(:,:), VERT_COL(:), ALB_WL(:)
  REAL (KND), TARGET, ALLOCATABLE :: RTM_OUT_ARR(:,:,:,:,:), OWF_OUT_VEC(:,:,:,:,:)
  REAL (KND), POINTER :: CHL_WEI(:,:,:), RUW_WEI(:,:,:), Pus_WEI(:,:,:), &
       Pks_WEI(:,:,:), Pkl_WEI(:,:,:)
  REAL (KND), POINTER :: SPEC_OUT(:,:,:), FLUX_UP(:,:,:), FLUX_DN(:,:,:), &
       OUT_VOD_ALL(:,:), DIR_RAD(:,:,:), ACTFLUX(:,:,:), DIF_FLUX_UP(:,:,:), &
       DIF_FLUX_DN(:,:,:), DIF_SCAL_UP(:,:,:), SPEC_OUT_SAVE(:,:), TG_VOD(:,:,:),& 
       DEL_RRS(:,:,:), DR_FIN_TOP(:,:), DR_FIN_GT(:,:), DR_TAU(:,:), R_SS(:,:), &
       STOK_OUT(:,:,:,:,:), DEL_VRS(:,:), DIR_FLUX(:,:,:), Tra_mu0(:,:), Tra_muv(:,:)
END MODULE RTM_OUTPUT
!*************************************************************************
MODULE PS_INTERN
  DOUBLE PRECISION, ALLOCATABLE :: COS_SCAT(:)
END MODULE PS_INTERN

!*************************************************************************
MODULE PM_ARR
  DOUBLE PRECISION, ALLOCATABLE :: V_c(:), V_f(:), A_c(:), A_f(:), Ro_dens(:)
END MODULE PM_ARR

!*************************************************************************
MODULE INTERN_DIMS
  INTEGER   NL_MS, NL_WF_MS, N_NODE_MS, NSOL_MS, N_PHI_MS
  INTEGER   NL_FINE_MS, MAX_LINE_MS, NL_WF, N_NODE_WF
  INTEGER   NSOL_WF, N_WF_L_MS, NL_FINE_WF, N_LOS_WF, N_SFN, Ns_SFN
  INTEGER   NL_PC, NL_FINE_PC, NL_ice, Nm_ice, NL_soot
  INTEGER   Nlg_Ray, NL_AER, NLG_AER, NLG_ALL_A, NL_CLD, NLG_CLD, NLG_ALL_C, Nlg_Ray_W
  INTEGER   NGb, Nm_vs, Nvs, NNv, Nm_vs_W
  INTEGER   Nl_Cr, Ng_Cr, Nls_Cr, Nso_Cr, N_C2
  INTEGER   Nl_Ar, Ng_Ar, Nls_Ar, Nso_Ar, N_A2
  ! For specular reflection
  INTEGER            N_sol_ls
  ! For the thermal emission
  INTEGER            NL_pla, NN_pla, LS_pla, Nf_pla, NL_FINE_pla
  ! For the surface BRDF 
  INTEGER            Ng1_BRF, Ng2_BRF, Ns_BRF, Nh_BRF
END MODULE INTERN_DIMS

!*************************************************************************
MODULE I_DIMS
  USE INTERN_DIMS
END MODULE I_DIMS

!*******************************************************************
MODULE WF_NUM
  USE TYPES,  ONLY: KND
  LOGICAL   NUM_WF, NUM_ABS, NUM_RAY, NUM_AER, NUM_PRE, NUM_OWF, NUM_TEM, &
       NUM_CLD, NO_SOL, NO_LS, SOL_AND_LS, NUM_ORAY, NUM_Chl
  CHARACTER (LEN=:), Allocatable :: SCA_or_ABS
  CHARACTER (LEN=5)  Wat_or_Ice
  REAL (KND), TARGET, ALLOCATABLE :: PERTURB(:), AL_BASE(:), AL_1_BASE(:), AL_1_PERT(:)
END MODULE WF_NUM

!*******************************************************************
MODULE VECT_VARS
  LOGICAL           INC_POL
  DOUBLE PRECISION, ALLOCATABLE :: Rot_r(:,:,:), Rot_l(:,:,:)
END MODULE VECT_VARS

!*******************************************************************
MODULE THERMAL_VAR
  USE TYPES,  ONLY: KND
  LOGICAL  BRIGHT_TEMP, OUT_PLANCK, INTEGR_PLANCK, SET_SOL_USR, Emi_FILE
  LOGICAL  Emi_akima
  INTEGER  Nwl_emis
  REAL (KND)   BTEM, VMAX, EPSIL, WNUMLO, WNUMHI, Pla_acc
  REAL (KND), PARAMETER::   PL_C2 = 1.438786D0, VCUT = 1.5D0,&
       SIGDPI = 5.67032D-8/3.1415926536D0, PL_CONC = 15.D0/3.1415926536D0**4,&
  ! [Planck_const*Light_speed/Boltzmann_constant] [m K]
       HC_K = 0.014387863D0 
  REAL (KND)  VCP(7), SOL_FLUX_USR
  DATA VCP / 10.25, 5.7, 3.9, 2.9, 2.3, 1.9, 0.0 /
  REAL (KND), ALLOCATABLE :: Sur_Emis(:), Wl_Emis(:)
  CHARACTER (LEN=80) Path_for_PLANCK
  CHARACTER (LEN=:), ALLOCATABLE :: Emi_NAME
END MODULE THERMAL_VAR

!*************************************************************
MODULE WATER
  USE TYPES,  ONLY: KND, DYN_CHAR
  
  LOGICAL  UWT, UIT, DO_ABSUW_AKIMA, DO_RTAU_W, ERDR
  LOGICAL  OUT_WLR, NO_COUPLING, SIPI, TS_CORR
  LOGICAL  Sol_off, INCL_VRS, INEL_DO, INEL_ADJ, INCL_INEL(3)
  LOGICAL  Absw_chl, Absw_pw, CDOM_Chl, CDOM_ACR, Absw_ful, Absw_hum, Absw_other
  INTEGER  N_ang_wa, N_ang_aw, Na_W, Nwl_QE, MAX_ITER_OC
  INTEGER  NLAYERS_UW, N_ABS_UW, NLEG_W, NLSA_W
  INTEGER  NSTR_W, NSTR_T, N_FC, Nwl_QEx, Nwl_QEm
  INTEGER  NSZA_W, NNv_W
  INTEGER, PARAMETER :: Ncol_UW = 5
  INTEGER            NC_UW(5), NC_CHL(2)
  REAL (KND) OC_DEP, Ref_ind, RTAU_W, T_ocean, Salin, Chlorophyll, V_small, &
       V_large, W_speed, W_azim, DEP_W_INPUT, ALB_BOTTOM, PW_ABS, Et_wa, &
       Et_aw, FH_REF, Wei_dif
  REAL (KND)  WL_Flu(2), Step_Flu(2)
  CHARACTER (LEN=1)  FH_EXTR
  CHARACTER (LEN=3)  RWG
  CHARACTER (LEN=5)  UP_CROSS
  CHARACTER (LEN=50) Par_PDF
  CHARACTER (LEN=:), ALLOCATABLE :: Fa_NAME, Ha_NAME, Conc_NAME, Ref_name, &
       DEP_W_FILENAME, Flu_CDOM, A_RTmat, OC_LAYERING, VSF_pure_model, &
       DEPOL_WATER, INS_VIEW, SS_PDF, INS_POSI, OB_TYPE
  CHARACTER (LEN=5), ALLOCATABLE :: Obs_POSITION(:)
  REAL (KND), ALLOCATABLE :: Rho_wa_ang(:), Rho_wa(:), Abs_flu(:,:), &
       Rho_aw_ang(:), Rho_aw(:), ZZZ_UW(:), TEMP_UW(:), DZ_UW(:),  &
       PRES_UW(:), SAL_UW(:), ABS_COEFF_UW(:,:), CHL_UW(:),  &
       Ra_GG_WL(:,:,:,:), Taw_GG_WL(:,:,:,:), Rw_GG_WL(:,:,:,:),  &
       Twa_GG_WL(:,:,:,:), Ra_GS_WL(:,:,:,:), Taw_GS_WL(:,:,:,:), &
       AAC_P(:,:), Fu_par(:), Hu_par(:), Fua_c(:), Hua_c(:), WL_QE(:), &
       A_QE(:,:), A_Fa(:), B_Fa(:), A_Ha(:), B_Ha(:), Scal_FC(:), &
       Flu_con(:,:), WL_QEx(:), WL_QEm(:), F_QEx(:,:), F_QEm(:,:), &
       PSIT(:), PSIS(:),Twa_gau(:,:)
  TARGET :: Ra_GG_WL, Taw_GG_WL, Rw_GG_WL, Twa_GG_WL, Ra_GS_WL, Taw_GS_WL, &
       ZZZ_UW, DZ_UW, ABS_COEFF_UW
END MODULE WATER

MODULE WATER_ICE
  USE TYPES,  ONLY: KND, DYN_CHAR  
  INTEGER  N_IW, LA_N_IW, Na_ice
  INTEGER, ALLOCATABLE :: ICE_LAY_NUM(:), LA_S_IW(:), LA_E_IW(:)
  LOGICAL  IWT, LoIW(2)
  REAL (KND), ALLOCATABLE :: L_TOP_I(:), L_BOT_I(:)
  TYPE Multi_layer_IW
     Integer  N_HPC, N_HS_WL, Nwl_BRF !, IW_TOP, IW_BOT, Na_W
     INTEGER  NC_CHL(2)
     INTEGER, ALLOCATABLE :: IW_TOP(:), IW_BOT(:), Na_W(:)
     CHARACTER (LEN=3)  RWG
     CHARACTER (LEN=:), ALLOCATABLE :: VSF_pure_model, Name_IW, PWAC_NAME, &
          CHL_NAME, CHL_CONC_NAME, DEPOL_WATER, DEP_W_FILENAME, Ref_type, &
          Ref_name
     LOGICAL PW_ABS_FILE, DO_RTAU_W, Absw_chl, Absw_pw, CDOM_Chl, &
          CDOM_ACR, Absw_ful, Absw_hum, Absw_other
     REAL (KND) PW_ABS, RTAU_W, DEP_W_INPUT, Ref_ind
     REAL (KND), ALLOCATABLE :: HS_WL(:), HS_COEFF(:,:,:,:), C_NF(:,:), &
          ABS_COEFF_UW(:,:), AAC_P(:,:), HSOL_PHAS_VS(:,:,:,:,:),     &
          RefW_r(:), RefW_i(:)
     TYPE (DYN_CHAR), ALLOCATABLE :: AFN_conc(:,:), AFN_coef(:,:), Extr_T(:,:)
!     TYPE (PhaseProf), Allocatable :: Npf(:)
  END TYPE Multi_layer_IW
  TYPE (Multi_layer_IW), Allocatable, TARGET ::  IWS(:)
END MODULE WATER_ICE

!*************************************************************
MODULE SLAB_2L
  USE TYPES,  ONLY: KND
  LOGICAL  TES_2L_SLAB
  INTEGER  NSTR_T1
  REAL (KND) Ome0(2), Ref_idx(2), Taus(2), F_Ref_inc
  REAL (KND), ALLOCATABLE :: Fr_01(:), Fr_12(:), Fr_20(:)
END MODULE SLAB_2L

!*************************************************************
MODULE CIA_VARS
  USE TYPES,  ONLY: KND
  
  INTEGER, PARAMETER :: N_CIA_P = 4, N_CIA_G = 3
  LOGICAL   N2_Air_Corr
  CHARACTER (LEN = :), ALLOCATABLE :: CIA_DIR
  CHARACTER (LEN = 3) CIA_GAS(N_CIA_G)
  CHARACTER (LEN = 1) CIA_VMR_MODE(N_CIA_G)
  REAL (KND) CIA_VMR_VAL(N_CIA_G)

  TYPE CIA_DATA
     INTEGER N_VAL
     REAL (KND), ALLOCATABLE :: CS(:), TEMP(:)
  END TYPE CIA_DATA
  Type CIA_Main
     Integer N_CIA_B
     Integer, Allocatable :: N_CIA_T(:)
     Logical Flg
     Character (Len=6) Name
     Real (Knd), Allocatable :: DENS(:) 
     Type (CIA_DATA), Allocatable :: DB(:)
  End Type CIA_Main
  Type (CIA_Main), Target:: CIA_INFO(N_CIA_P)
  
CONTAINS
  SUBROUTINE CIA_INIT
    Integer II
    CIA_GAS  = ['N2 ', 'O2 ', 'CO2']
    CIA_VMR_VAL = [0.78D0, 0.21D0, 3.65D-4]

    ! 2000 - 2698 cm-1 and 1850 - 3000 cm-1 bands of N2-N2 pair must be
    ! joined into one band with 10 temperatures because of the overlapping 
    ! spectral range. Because of the "incorrect" order of the CIA bands in 
    ! data file the temperatures in the joined band do not increase 
    ! monotoniusly, be carefull wenn using the temperature interpolation.
    ! For HITRAN 2020 also 0 - 450 cm-1, 0 - 550 cm-1, and 0 - 650 cm-1, 
    ! bands of N2-N2 (34 temperatures in total) as well as 1850 - 3000 cm-1 
    ! and 2000 - 2698 cm-1 bands of O2-N2 (10 temperatures in total) need
    ! to be joined.
    ! HITRAN 2012
    !Call SetInfo (1, "N2-N2", 2, [10, 10]) 
    !Call SetInfo (2, "O2-O2", 5, [15, 3, 1, 1, 1]) 
    !Call SetInfo (3, "O2-N2", 3, [7, 5, 1])
    ! HITRAN 2020
    Call SetInfo (1, "N2-N2", 3, [34, 10, 14]) 
    Call SetInfo (2, "O2-O2", 8, [15, 1, 1, 1, 1, 1, 4, 5]) 
    Call SetInfo (3, "O2-N2", 4, [7, 10, 1, 1]) 
    Call SetInfo (4, 'O2-CO2', 1, [1])
  Contains
    Subroutine SetInfo (II, Name, N_CIA_B, N_CIA_T)
      Integer II, N_CIA_B, N_CIA_T(N_CIA_B)
      Character (Len=*) Name
      CIA_INFO(II)%Name = Name
      CIA_INFO(II)%N_CIA_B = N_CIA_B 
      If(Allocated(CIA_INFO(II)%N_CIA_T)) DeAllocate(CIA_INFO(II)%N_CIA_T)
      Allocate(CIA_INFO(II)%N_CIA_T(CIA_INFO(II)%N_CIA_B))
      CIA_INFO(II)%N_CIA_T = N_CIA_T
      Return
    End Subroutine SetInfo
  END SUBROUTINE CIA_INIT
END MODULE CIA_VARS

!*************************************************************
Module TYPE_VARS
  USE TYPES, ONLY: KND, DCMP
END Module TYPE_VARS
!*************************************************************
