C********************************************************************C
C
C	SCIATRAN - Version 4.6 - Last modification: 05 Apr 2022
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      MODULE BRDF_SUBRS
      USE BRDF,          ONLY : Rep_cosine, Lin_rel, RefW_r, RefW_i, 
     $                          VS_OC_COMP, Nwl_BRF, WL_BRF, HS_par,
     $                          NDVI, Ex_Type, Cr_ang, Par_mix
      USE INTERN_DIMS,   ONLY : Nvs
      USE LOG_PARS,      ONLY : LOG_LEVEL, N_ESO, N_DSO, USE_SHAD, 
     $                          LUNIT
      USE TYPE_VARS,     ONLY : KND
      USE ST_VARS,       ONLY : SURF_TYPE, RAD_TO_DEG, Pie
      USE WATER,         ONLY : W_azim, W_speed, Par_PDF, UP_CROSS

      PRIVATE
      PUBLIC :: BRDF_ACT, BRDF_ACT_DOM
      CONTAINS
      SUBROUTINE BRDF_ACT (BRDF_PARAM, N_ANG_REFL, COS_REFL, PHI_REFL, 
     $     N_ANG_FALL, COS_FALL, PHI_FALL, SURF_REFL)
      INTEGER             N_ANG_REFL, N_ANG_FALL, I, K
      DOUBLE PRECISION, PARAMETER :: PI = 3.1415927
      DOUBLE PRECISION, DIMENSION (N_ANG_REFL) :: COS_REFL, PHI_REFL,
     $     SIN_REFL, TAN_REFL
      DOUBLE PRECISION, DIMENSION (N_ANG_FALL) :: COS_FALL, PHI_FALL,
     $     SIN_FALL, TAN_FALL  
      DOUBLE PRECISION, DIMENSION (:,:) :: SURF_REFL
      DOUBLE PRECISION, DIMENSION (:) :: BRDF_PARAM
      DOUBLE PRECISION    THETA, T_1, T_2, RO_0, KK, COS_PHI, COS_G, H
      DOUBLE PRECISION    G, R, TT, W, W_1, COS_B, TAN_B, SIGMA, P, RO

      SIN_REFL = DSQRT(1.0D0 - COS_REFL**2)
      SIN_FALL = DSQRT(1.0D0 - COS_FALL**2)
      TAN_REFL = DABS(SIN_REFL/COS_REFL) 
      TAN_FALL = DABS(SIN_FALL/COS_FALL)
      RO_0 = BRDF_PARAM(1)
      THETA = BRDF_PARAM(2)
      T_1 = 1 - THETA**2 ;  T_2 = 1 + THETA**2
      KK = BRDF_PARAM(3)

      SELECT CASE (TRIM(SURF_TYPE))
      CASE DEFAULT
         DO I = 1, N_ANG_REFL
            DO K = 1, N_ANG_FALL
               COS_PHI = DCOS(PI + PHI_REFL(I) - PHI_FALL(K))
               COS_G = DABS(COS_REFL(I)*COS_FALL(K)) + 
     $              SIN_REFL(I)*SIN_FALL(K)*COS_PHI
               H = T_1/(T_2 + 2*THETA*COS_G)/DSQRT(T_2 + 2*THETA*COS_G)
               G = DSQRT(
     $              TAN_REFL(I)*TAN_REFL(I) + TAN_FALL(K)*TAN_FALL(K)
     $              - 2.0D0*TAN_REFL(I)*TAN_FALL(K)*COS_PHI)
               R = (1.0D0 - RO_0)/(1.0D0 + G)
               SURF_REFL(I,K) = RO_0*H*(1.0D0 + R)*
     $              (DABS(COS_REFL(I)*COS_FALL(K)))**(KK - 1.0D0)/
     $              (DABS(COS_REFL(I)) + DABS(COS_FALL(K)))**(1.0D0 -KK)
            END DO
         END DO

      CASE('snow')  
         DO I = 1, N_ANG_REFL
            DO K = 1, N_ANG_FALL
               COS_PHI = DCOS(3.1415927 + PHI_REFL(I) - PHI_FALL(K))
               COS_G = DABS(COS_REFL(I)*COS_FALL(K)) + 
     $              SIN_REFL(I)*SIN_FALL(K)*COS_PHI
               H = T_1/(T_2 + 2*THETA*COS_G)/DSQRT(T_2 + 2*THETA*COS_G)
               G = DSQRT(
     $              TAN_REFL(I)*TAN_REFL(I) + TAN_FALL(K)*TAN_FALL(K)
     $              - 2.0D0*TAN_REFL(I)*TAN_FALL(K)*COS_PHI)
               TT = BRDF_PARAM(5)*DEXP(BRDF_PARAM(6)*
     $              (3.1415927 - DACOS(COS_G))) + 1.0D0
               R = (1.0D0 - RO_0)/(1.0D0 + G)
               SURF_REFL(I,K) = RO_0*(H*(1.0D0 + R)*
     $              (DABS(COS_REFL(I)*COS_FALL(K)))**(KK - 1.0D0)/
     $              (DABS(COS_REFL(I)) + DABS(COS_FALL(K)))**(1.0D0 -KK)
     $              + BRDF_PARAM(4)/DABS(COS_FALL(K)))*TT
            END DO
         END DO

      CASE('ocean')  
c         Based on Cox & Munk, 1954
c         BRDF_PARAM(1) - Wind speed
c         BRDF_PARAM(2) - Refraction index ratio (water/air)
         DO I = 1, N_ANG_REFL
            DO K = 1, N_ANG_FALL
               COS_G = DABS(COS_REFL(I)*COS_FALL(K)) + 
     $              SIN_REFL(I)*SIN_FALL(K)*
     $              DCOS(PI + PHI_REFL(I) - PHI_FALL(K))
               W = DACOS(COS_G)/2.0D0
               COS_B = (DABS(COS_REFL(I)) + DABS(COS_FALL(K)))/
     $              DCOS(W)/2.0D0
               TAN_B = DTAN(DACOS(COS_B))
               SIGMA = 0.003 + 0.00512*BRDF_PARAM(1)  
               P = DEXP( -TAN_B**2/SIGMA)/SIGMA !/PI
               W_1 = DASIN(DSIN(W)/BRDF_PARAM(2))
               IF(W.NE.0.0D0) THEN
                  RO = 0.5D0*((DSIN(W - W_1)/DSIN(W + W_1))**2 +
     $                 (DTAN(W - W_1)/DTAN(W + W_1))**2)
               ELSE
                  RO = ((BRDF_PARAM(2) - 1)/(BRDF_PARAM(2) + 1))**2
               END IF
               SURF_REFL(I,K) = RO*P/4.0D0/DABS(COS_REFL(I))/COS_B**4 !*PI
            END DO
         END DO
      END SELECT      
      RETURN
      END SUBROUTINE
c **********************************************************************

      SUBROUTINE BRDF_ACT_DOM(BRDF_PARAM, N_REF, COSR, N_AZM, PHI_REFL,
     $     N_INC, COSI_INP, Nwl_Inp, WL_Inp, SURF_REFL, PDF_TYPE)
c     Call in "general_brdf_setup.f", "gpp_ss_stokes.f"

      CHARACTER (LEN=12), PARAMETER :: R_NAME='BRDF_ACT_DOM'
      INTEGER,           INTENT(IN) :: N_REF, N_AZM, N_INC, Nwl_Inp
      CHARACTER (LEN=*), INTENT(IN) :: PDF_TYPE     
      REAL (KND),        INTENT(IN) :: BRDF_PARAM(:), PHI_REFL(N_AZM),
     $     WL_Inp(Nwl_Inp), COSI_INP(N_INC)
      REAL (KND)  COSI(N_INC), COSR(N_REF)
      REAL (KND), DIMENSION (:,:,:,:) :: SURF_REFL
c     ... Local parameters      
      INTEGER  I, K, L, J, I1, K1, N_OUT, Npo
      REAL (KND), PARAMETER :: PI = 3.1415927d0
      REAL (KND), DIMENSION (N_INC) :: SINI, TANI, SHI
      REAL (KND), DIMENSION (N_REF) :: SINR, TANR, SHR, Wc 
      REAL (KND)  R1_OUT(Nvs*N_REF,Nvs*N_INC,N_AZM,Nwl_Inp)
      REAL (KND)  THE, T_1, T_2, RO_0, KK, CPHI, COS_G, H, SQS, 
     $     SQPI, Eta, Step, X1, X2, Phi_wi, Co_phi,
     $     Si_phi, Sq_c, Sq_u, X1q, X2q, G, R, TT, TAN_B, SIGMA, RO, 
     $     Sig_C, Sig_U, Zx, Zy, Zu, Zc, AW, VCI(3), VCR(3), Unit(3),
     $     R_L, R_P
      REAL (KND)  S1, CS, Fre_r, Zn1, Zn2
c     ... Test output array
c      REAL (KND)  R_OUT(N_REF,N_AZM)
      REAL (KND), DIMENSION (N_REF,N_INC,N_AZM) :: PDF, Mun, XI1
      REAL (KND), TARGET ::  Rezr(1), Rezi(1)
      REAL (KND), POINTER :: P_refr(:), P_refi(:)
      COMPLEX (kind(0.0d0)) :: CC1, CC2, CN1, CN2, CXI2, CN11, CN22
c     ------------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I') 

c      WRITE(*,*) '         Azimuth in DEG :'
c      WRITE(*,'(300(1x,f7.3))') (Phi_REFL(I)/3.141459*180., I=1,N_AZM)

c      Zeroing for the vector case only
      IF( Nvs > 1 ) SURF_REFL = 0.d0
c       Incident beam                   Reflected beam
      COSI = DABS( COSI_INP )          ;  COSR = DABS( COSR )      
      SINI = DSQRT(1.d0 - COSI**2) ;  SINR = DSQRT(1.d0 - COSR**2)
      TANI = DABS(SINI/COSI)       ;  TANR = DABS(SINR/COSR) 
      
c     ... For BRDF of snow and BRDF according to RPV model
      RO_0 = BRDF_PARAM(1) ; THE  = BRDF_PARAM(2)
      T_1 = 1 - THE**2     ; T_2  = 1 + THE**2
      KK = BRDF_PARAM(3)

      SELECT CASE (TRIM(SURF_TYPE))
      CASE DEFAULT              ! All surfaces according to RPV model
         CALL RPV_BRDF
c     ----------------------------------------------------------------------
      CASE('mRPV_facet')         
c        ... Contribution of mRPV BRDF is calculated in SURF_REFL(I1,K1,J,L)
         CALL mRPV_BRDF

         SIGMA = Lin_rel(1) ; SQS = DSQRT( SIGMA ) ; SQPI = SQS/SQRT(PI)
         SHR = 0.d0  ;  SHI = 0.d0
         IF( USE_SHAD=='yes' ) THEN
            IF( REAL(SIGMA) > 1.E-8 ) THEN
               CALL SHADOWING( SHR, SHI )
            ELSE
               WRITE(*,*) '  ---> Breon"s Shadowing '
               SHR(:) = 0.5d0*(1.d0/COSR(:) - 1.d0)
               SHI(:) = 0.5d0*(1.d0/COSI(:) - 1.d0)
            ENDIF
         ENDIF
         DO J = 1, N_AZM
            CPHI = DCOS(PI + PHI_REFL(J))
            DO I = 1, N_REF
               DO K = 1, N_INC
c                 ... Cosine of the phase angle
                  COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
                  AW = DACOS(COS_G)/2.D0
c                 ... Cosine of the angle between a normal to the 
c                     facet and z-axis (reflection)
                  Mun(I,K,J) = (COSR(I) + COSI(K))/DCOS(AW)/2.D0
               ENDDO  ! over incidence angles (K)
            ENDDO  ! over reflection angles (I)
         ENDDO  ! over azimuthal angles (J)
         IF( REAL(SIGMA) > 1.E-8 ) THEN
c           ... Gaussian PDF
            CALL PDF_GAUSS( PDF ) ; Npo = 4
         ELSE
c           ... Breon's PDF
            WRITE(*,*) '  ---> Breon"s PDF '
            PDF = 1.d0 ; Npo = 0
         ENDIF

c        ... Fresnel reflection coefficient
         IF( Nvs==1 ) THEN
            CALL REF_MAT_SCAL( N_REF, COSR, N_AZM, PHI_REFL, N_INC, 
     $           COSI, Nwl_Inp, WL_Inp, R1_OUT )
         ELSE
            CALL REF_MAT_VEC( N_REF, COSR, N_AZM, PHI_REFL, N_INC, 
     $           COSI, Nwl_Inp, WL_Inp, R1_OUT )
         ENDIF

c        ... BRDF mRPV plus facet
         DO I = 1, Nvs*N_REF
            I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
            DO K = 1, Nvs*N_INC
               K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
               G = 1.d0/(1.d0 + SHR(I1) + SHI(K1))
               DO L = 1,Nwl_Inp
                  SURF_REFL(I,K,:,L) = SURF_REFL(I,K,:,L)
     $                 + Par_mix(L,4)*PDF(I1,K1,:)*G
     $                 *R1_OUT(I,K,:,L)/4.d0
     $                 /Mun(I1,K1,:)**Npo/COSI(K1)/COSR(I1)
               ENDDO  ! over reference wavelengths (L)
            ENDDO  ! over incidence angles (K)
         ENDDO  ! over reflection angles (I)

      CASE('snow')                    ;  CALL SNOW_BRDF
      CASE('snow_artm')               ;  CALL SNOW_ARTM
      CASE('white_si')                ;  CALL WHITE_SNOW_DB
c      CASE('white_si')                ;  CALL WHITE_SNOW
      CASE('Ross-Li','Ross-Li-snow')  ;  CALL Ross_Li
      CASE('snow_fasmar')             ;  CALL Snow_fasmar         
      CASE('svk')                     ;  CALL SVK_BRDF
      CASE('melt_pond')               ;  CALL REFL_MELT_POND

      CASE('ocean')  
c         Based on Cox & Munk, 1954
         SIGMA = Lin_rel(1) + Lin_rel(2)*W_speed
         SQS   = DSQRT( SIGMA ) ; SQPI = SQS/SQRT(PI)
c        Calculation of the SHADOWING factors
c        ------------------------------------
         SHR = 0.d0  ;  SHI = 0.d0
         IF( USE_SHAD == 'yes' ) CALL SHADOWING( SHR, SHI )

c        Calculation of the slope PDF
c        ----------------------------
         DO J = 1, N_AZM
            CPHI = DCOS(PI + PHI_REFL(J))
            DO I = 1, N_REF
               DO K = 1, N_INC
c                 ... Cosine of the phase angle
                  COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
                  AW = DACOS(COS_G)/2.D0
c                 ... Cosine of the angle between a normal to the 
c                     facet and z-axis (reflection)
                  Mun(I,K,J) = (COSR(I) + COSI(K))/DCOS(AW)/2.D0
               ENDDO
            ENDDO
         ENDDO

         SELECT CASE( TRIM(PDF_TYPE) )
         CASE('Gauss')
            CALL PDF_GAUSS( PDF )
         CASE('Gram-Charlier')
            CALL GRAM_CHARLIER( PDF )
         END SELECT

c        ... Fresnel Reflection coefficient 
c        ----------------------------------
         SELECT CASE( VS_OC_COMP(2) )
         CASE('s')
c           ... Calculation of the Fresnel reflection coefficient
c               in the scalar case using complex refraction index

            CALL REF_MAT_SCAL( N_REF, COSR, N_AZM, PHI_REFL, N_INC, 
     $           COSI, Nwl_Inp, WL_Inp, R1_OUT )
            DO I = 1, N_REF
               I1 = (I-1)*Nvs + 1
               DO K = 1, N_INC
                  K1 = (K-1)*Nvs + 1
                  DO L = 1,Nwl_Inp
                     SURF_REFL(I1,K1,:,L) = R1_OUT(I1,K1,:,L)/4.d0
     $                    /Mun(I,K,:)**4/COSI(K)
                  ENDDO         ! over reference wavelengths (L)
               ENDDO            ! over incidence angles (K)
            ENDDO               ! over reflection angles (I)
            
         CASE('v')
c           ... Calculation of the Fresnel reflection matrix
c               vector case, refraction index can be complex
c           -------------------------------------------------

cc           ---> Test with Mishchenko surface reflection <----
c            WRITE(*,*) '  To use Mishchenko BRDF enter 0'
c            READ(*,*) N_OUT
c
c            IF( N_OUT == 0 ) THEN
c               CALL BRDF_OCEAN_VEC( N_REF, COSR, N_AZM, PHI_REFL, N_INC, 
c     $              COSI, Nwl_Inp, WL_Inp, R1_OUT )
c               SURF_REFL(:,:,:,:) = R1_OUT(:,:,:,:) ! Preliminary
c               WRITE(*,*) '  ___________________________ '
c               WRITE(*,*) '  _______Mishchenko__________ '
c               WRITE(*,*) '  ___________________________ '
c            ELSE

            CALL REF_MAT_VEC( N_REF, COSR, N_AZM, PHI_REFL, N_INC, 
     $           COSI, Nwl_Inp, WL_Inp, R1_OUT )
c           ... To return the reflection matrix only
c               the following do-loops has to be commented
            DO I = 1, Nvs*N_REF
               I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
               DO K = 1, Nvs*N_INC
                  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
                  DO L = 1,Nwl_Inp
                     SURF_REFL(I,K,:,L) = R1_OUT(I,K,:,L)/4.d0
     $                    /Mun(I1,K1,:)**4/COSI(K1)
                  ENDDO
               ENDDO
            ENDDO
c               WRITE(*,*) '  ___________________________ '
c               WRITE(*,*) '  ______Direct rotation______ '
c               WRITE(*,*) '  ___________________________ '
c            ENDIF

         CASE DEFAULT
            STOP 'Vector or scalar form for glint?'
         END SELECT             ! Vector or scalar form for glint


         Wc(:) = COSR(:)
c        ... Use expression for reflectance of rough water surface 
c        according to V. Kisselev, B. Bulgarelli JQSRT 85 (2004) 419-435
         IF( Rep_cosine ) CALL REPLACE_COS

c        ... Including of the wave slope PDF and SHADOWING:
c        --------------------------------------------------
         DO I = 1, Nvs*N_REF
            I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
            DO K = 1, Nvs*N_INC
               K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )       
               G = 1.d0/(1.d0 + SHR(I1) + SHI(K1))
               DO J = 1,N_AZM
                  SURF_REFL(I,K,J,:) = SURF_REFL(I,K,J,:)*PDF(I1,K1,J)*G
               ENDDO
            ENDDO
         ENDDO
         
c        ... Divide by Mur (cosine of reflection angle):
c        -----------------------------------------------
         DO I = 1, N_Ref
            DO I1 = (I-1)*Nvs+1,I*Nvs
               SURF_REFL(I1,:,:,:) = SURF_REFL(I1,:,:,:)/Wc(I)
            ENDDO
         ENDDO            

         IF (LOG_LEVEL(1) >= N_ESO) THEN
            WRITE(LUNIT,*) ' '
            WRITE(LUNIT,*) '   Fresnel BRF in nadir '//
     $           'direction (at first solar angle)'
            I1 = (N_REF-1)*Nvs + 1 ! In nadir direction
            K = MIN0(3,N_INC)
            DO L = 1,Nwl_Inp
               WRITE(LUNIT,'(1x,F7.3,20(1x,E12.4))') 
     $              WL_INP(L), (SURF_REFL(I1,K1,1,L), K1 = 1,K*Nvs,Nvs)
            ENDDO
         ENDIF

      END SELECT 

      CALL ST_TRACK (R_NAME, 1, 'O') 
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE SHADOWING( SHR, SHI )
      REAL (KND) SHR(N_REF), SHI(N_INC)
      SHR = 0.d0
      DO I = 1, N_REF
         IF( SINR(I) <= 0.0d0 ) CYCLE
         G = COSR(I)/SINR(I)
         SHR(I) = DEXP(-G*G/SIGMA)*SQPI/G - DERFC(G/SQS)
      ENDDO
      SHR(:) = 0.5D0*SHR(:)  ;  SHI = 0.d0
      DO I = 1, N_INC
         IF( SINI(I) <= 0.d0 ) CYCLE
         G = COSI(I)/SINI(I)
         SHI(I) = DEXP(-G*G/SIGMA)*SQPI/G - DERFC(G/SQS)
      ENDDO
      SHI(:) = 0.5D0*SHI(:)
      RETURN
      END SUBROUTINE SHADOWING
c **********************************************************************

      SUBROUTINE PDF_GAUSS( PDF )
      REAL (KND) :: PDF(N_REF,N_INC,N_AZM)
      IF(LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) '    ********************************** '
         WRITE(LUNIT,*) '    *****    Use Gauss PDF        **** '
         WRITE(LUNIT,*) '    ********************************** '
      ENDIF
c     ... The Gaussian PDF is calculated in parts. Here, it is
c         saved 1/Sig^2*EXP[-(1-Mun^2)/Mun^2/Sig^2], where Mun is
c         the cosine of the normal to a wave facet.
      DO J = 1, N_AZM
         DO I = 1, N_REF
            DO K = 1, N_INC
               TAN_B = DTAN( DACOS(Mun(I,K,J)) )
               PDF(I,K,J) = DEXP( -TAN_B*TAN_B/SIGMA)/SIGMA 
            ENDDO
         ENDDO
      ENDDO
      RETURN
      END SUBROUTINE PDF_GAUSS
c **********************************************************************

      SUBROUTINE GRAM_CHARLIER( PDF )
      REAL (KND) :: PDF(N_REF,N_INC,N_AZM)
      REAL (KND)   Cpe(3), Csk(2)
c
c     The parameters of the Gram-Charlier distribution (Cox,Munk)
c
c                            Skewness coefficients for clean surface
      REAL (KND), PARAMETER :: C21 = 0.01d0, C03 = 0.04d0,
     $                         W21 = -0.0086d0, W03 = -0.033d0
c                 Peakedness coefficients for clean surface
      REAL (KND)   Peakedness_CM(3), Peakedness_BH(3)
c                          C40     C22     C04
      DATA Peakedness_CM /0.4d0, 0.12d0, 0.23d0/
      DATA Peakedness_BH /0.3d0, 0.12d0, 0.4d0/

      IF(LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) '    ******************************* '
         WRITE(LUNIT,*) '    **  Use Gram-Charlier PDF    ** '
         WRITE(LUNIT,'(1x,a25,2x,f7.3)') 
     $        '    **     Wind azimuth :', W_azim
         WRITE(LUNIT,*) '    ******************************* '
      ENDIF

      SELECT CASE( TRIM(Par_PDF) )
      CASE('Cox_Munk')
         Csk(1) = (C21 + W21*W_speed)/2.d0 
         Csk(2) = (C03 + W03*W_speed)/6.d0 
         Cpe(1) = Peakedness_CM(1)/24.d0
         Cpe(2) = Peakedness_CM(2)/4.d0
         Cpe(3) = Peakedness_CM(3)/24.d0
c        Sig_C  = 3.d-03 + 1.92d-03*W_speed 
c        Sig_U  = 3.16d-03*W_speed 
      CASE('Breon_Henriot')
         Csk(1) = -9.d-04*W_speed**2/2.d0 
         Csk(2) = -0.45d0/(1.d0 + DEXP(7.d0 - W_speed))/6.d0 
         Cpe(1) = Peakedness_BH(1)/24.d0
         Cpe(2) = Peakedness_BH(2)/4.d0
         Cpe(3) = Peakedness_BH(3)/24.d0
c        Sig_C  = 3.d-03 + 1.85d-03*W_speed    
c        Sig_U  = 1.d-03 + 3.16d-03*W_speed 
      END SELECT
      SELECT CASE( UP_CROSS )
      CASE('ratio')
         Sig_U = lin_rel(3)/(1.d0 + lin_rel(3))*SIGMA
         Sig_C = SIGMA/(1.d0 + lin_rel(3))  
         WRITE(*,*) '   Sig_C   and    Sig_U'
         WRITE(*,*) Sig_C, Sig_U
      CASE('coeff')
         Sig_U = Lin_rel(3) + Lin_rel(4)*W_speed
         Sig_C = Lin_rel(5) + Lin_rel(6)*W_speed
      END SELECT
      Sq_c   = DSQRT(Sig_C)   ;  Sq_u  = DSQRT(Sig_U)
      Phi_wi = W_azim/180.d0*Pi
      Co_phi = DCOS(Phi_wi)  ;  Si_phi = DSIN(Phi_wi)
      DO J = 1, N_AZM
         DO I = 1, N_REF
            DO K = 1, N_INC
               Zx = -( SINR(I)*DCOS(PHI_REFL(J)) - SINI(K) ) /
     $              (COSR(I) + COSI(K))
               Zy = -SINR(I)*DSIN(PHI_REFL(J))/(COSR(I) + COSI(K))
               Zc = -Zy*Co_phi + Zx*Si_phi !cross wind 
               Zu =  Zy*Si_phi + Zx*Co_phi !up wind
               X1 = Zc/Sq_c  ;  X2 = Zu/Sq_u
               X1q = X1*X1   ;  X2q = X2*X2
               PDF(I,K,J) = 
     $              DEXP( -(Zc*Zc/Sig_c + Zu*Zu/Sig_u)/2.d0 )
     $              /Sq_c/Sq_u*( 1.d0 
     $              - Csk(1)*(X1q - 1.d0)*X2
     $              - Csk(2)*(X2q - 3.d0)*X2
     $              + Cpe(1)*(X1**4 - 6.d0*X1q + 3.d0)
     $              + Cpe(2)*(X1q - 1.d0)*(X2q -1.d0)
     $              + Cpe(3)*(X2**4 - 6.d0*X2q + 3.d0) )
            ENDDO
         ENDDO
      ENDDO
      PDF = PDF/2.0d0

      RETURN
      END SUBROUTINE GRAM_CHARLIER
c **********************************************************************
      SUBROUTINE REPLACE_COS
      INTEGER,  PARAMETER :: N_e = 50
      REAL (KND), ALLOCATABLE :: 
     $     Etp(:), Hp(:), Zb(:), Zn(:), Be0(:), Be1(:), 
     $     Pif(:), Mb0(:), Mb1(:), Y0(:), Y1(:)
c     -----------------------------------------

      ALLOCATE( Etp(N_e-1), Hp(N_e-1), Zb(N_e-1), 
     $     Zn(N_e-1), Be0(N_e-1), Be1(N_e-1), Pif(N_e-1),
     $     Mb0(N_e-1), Mb1(N_e-1), Y0(N_e-1), Y1(N_e-1) )

      DO J = 1,N_ref 
         Eta  = COSR(J)
         IF( Eta >= 0.4d0 ) CYCLE
c        ... Integration step for the interval [-1,eta]
         Step = (1.d0 + Eta)/DBLE(N_e-1)
         Etp(1) = -1.d0
         DO I = 2,N_e-1
            Etp(I) = Etp(I-1) + Step
         ENDDO
         Hp(:) = (1.d0 - Eta**2)*(1.0d0 - Etp(:)**2)
         Zb(:) =  2.d0/SIGMA*DSQRT( Hp(:) )/(Eta - Etp(:))**2
         Zn(:) = -2.d0/SIGMA*(1.d0 - Eta*Etp(:))/(Eta - Etp(:))**2
     $        + 1.d0/SIGMA
         Mb0(:) = (1.d0 - Eta*Etp(:))**2 + HP(:)
         Mb1(:) = -DSQRT(HP(:))*( 2.d0*(1.d0 - Eta*Etp(:)) 
     $        + SIGMA/2.d0*(Eta - Etp(:))**2 )         
c        CALL BESSI_0( N_e-1,Zb,Be0 )
c        CALL BESSI_1( N_e-1,Zb,Be1 )      
         WHERE( Zb(:) < 3.75d0 )
            Y0(:) = (Zb(:)/3.75d0)**2
c           ... Modified Bessel function I0
            Be0(:) = 1.0 + Y0(:)*(3.5156229 + Y0(:)*(3.0899424 
     $           + Y0(:)*(1.2067492 + Y0(:)*(0.2659732 
     $           + Y0(:)*(0.360768d-01 + Y0(:)*0.45813d-02)))))
c           ... Modified Bessel function I1
            Be1(:) = Zb(:)*(0.5d0 
     $           + Y0(:)*(0.87890594d0 + Y0(:)*(0.51498869d0 
     $           + Y0(:)*(0.15084934d0 + Y0(:)*(0.2658733d-01
     $           + Y0(:)*(0.301532d-02 + Y0(:)*0.32411d-03) )))))
            
            Pif(:) = 2.d0/SIGMA/(Eta - Etp(:))**4*DEXP(Zn(:))*
     $           ( Mb0(:)*Be0(:) + Mb1(:)*Be1(:) )
         ELSEWHERE
            Y0(:) = 3.75d0/Zb(:)
c           ... Modified Bessel function I0 without mult. EXP(X)/DSQRT(X)
            Be0(:) = 
     $           (0.39894228 + Y0(:)*(0.1328592d-01 
     $           + Y0(:)*(0.225319d-02 + Y0(:)*(-0.157565d-02 
     $           + Y0(:)*(0.916281d-02 
     $           + Y0(:)*(-0.2057706d-01 
     $           + Y0(:)*(0.2635537d-01  
     $           + Y0(:)*(-0.1647633d-01 
     $           + Y0(:)*0.392377d-02) ))))))) 
c           ... Modified Bessel function I1 without mult. EXP(X)/DSQRT(X)
c               assumed that argument is positive
            Y1(:) = 0.2282967d-01 + Y0(:)*( -0.2895312d-01 
     $           + Y0(:)*(0.1787654d-01 - Y0(:)*0.420059d-02) )
            Be1(:) = 0.39894228 
     $           + Y0(:)*( -0.3988024d-01 
     $           + Y0(:)*( -0.362018d-02 + Y0(:)*( 0.163801d-02 
     $           + Y0(:)*(-0.1031555d-01 + Y0(:)*Y1(:)) ))) 
            Pif(:) = 2.d0/SIGMA/(Eta-Etp(:))**4*DEXP(Zn(:)+Zb(:))
     $           /DSQRT(Zb(:))*( Mb0(:)*Be0(:) + Mb1(:)*Be1(:) )
         END WHERE           
         Wc(J) = Step*( Pif(1)/2.0d0 + SUM(Pif(2:N_e-1)) )
      ENDDO

      END SUBROUTINE REPLACE_COS
c **********************************************************************

      SUBROUTINE SNOW_ARTM
      USE BRDF,    ONLY: Nmom_ICE, MOM_ICE, Snow_abs, WL_refi_idx, 
     $                   Refi_idx, Nice, Ref_ip
      USE ST_VARS, ONLY: Pie
c     ------------------------------------------------------------------
      INTEGER  Ws, Ws1, N_USE
c     ... Parameters Ap, Bp, and Cp for reflection function R0 of 
c         semi-infinite medium are taken from  A. Kokhanovsky, et al. (2011)  
c         "Sizing snow grains using backscattered solarlight", 
c         International Journal of Remote Sensing, 32:22, 6975-7008
      REAL (KND), PARAMETER :: Ap = 1.247d0, Bp = 1.186d0, Cp = 5.157d0
      REAL (KND) SIR, PIR, SCAT, R0, Kp, Kmu, Kmu0, P_TETA, DUM, DC, 
     $     Ang, WLO, P_LEG(NMOM_ICE), Gas(Nwl_Inp), S_H
      REAL (KND), TARGET :: MOMS(Nwl_Inp,NMOM_ICE)
      REAL (KND), POINTER :: MOM_USER(:), Abs_ice(:)
c     ==================================================================

      N_USE = NMOM_ICE
c      N_USE = 750
      IF( Nwl_Inp==1 .AND. Nwl_BRF > 1) THEN
c        ... Interpolation of refractive index on the current wavelength
c            using SS_STOKES entry
         WLO = WL_INP(1)
         CALL LIN_SEARCH_NEW(WLO, DC, Ws, Ws1, Nice, WL_refi_idx)
         DUM = (1.d0 - DC)*Refi_idx(Ws) + DC*Refi_idx(Ws1)
         Rezi(1) = DSQRT( 4.d0*Pie*DUM/WLO*2.d0*Ref_ip )
         Abs_ice => Rezi(1:1)
         CALL LIN_SEARCH_NEW(WLO, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
         DO I = 1, N_USE
            MOMS(1,I) = 
     $           (1.d0 - DC)*MOM_ICE(Ws,I,1) + DC*MOM_ICE(Ws1,I,1)
         ENDDO
         Gas(1) = MOMS(1,2)/3.d0
      ELSE
c        ... Using previously interpolated values of refractive indece
         Abs_ice => Snow_abs(:) 
         MOMS(:,:) = MOM_ICE(:,:,1)
         Gas(:) = MOMS(:,2)/3.d0
      ENDIF
c      WRITE(*,*) '  Asymmetry factor '
c     WRITE(*,*) Gas(:)
      
      Kp = 0.66 ! for fractal snow grains
      DO I = 1, N_REF
         I1 = (I-1)*Nvs+1 ; Kmu = 1.d0 + 2.d0*COSR(I)
         DO K = 1, N_INC
            K1 = (K-1)*Nvs+1 ; Kmu0 = 1.d0 + 2.d0*COSI(K)
            SIR = COSI(K) + COSR(I) ; PIR = COSI(K)*COSR(I)
            DO J = 1, N_AZM
               SCAT = -PIR + SINR(I)*SINI(K)*DCOS( PHI_REFL(J) )
c               CALL WIGNER_PLG(SCAT, 1, 0, N_USE, P_LEG(1:N_USE))
               Ang = DACOS( SCAT )/Pie*180.d0
               DO L = 1,Nwl_Inp 
c                  MOM_USER => MOMS(L,1:N_USE)
c                  P_TETA = SUM( P_LEG(1:N_USE)*MOM_USER(:) )
c                  S_H = 1.d0 + Gas(L)*Gas(L) - 2.d0*Gas(L)*SCAT
c                  P_TETA = (1.d0 - Gas(L)*Gas(L))/S_H/DSQRT(S_H)
                 P_TETA = 11.1d0*DEXP( -0.087d0*Ang ) + 
     $                 1.1d0*DEXP( -0.014d0*Ang )
                  R0 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0
                  SURF_REFL(I1,K1,J,L) = 
     $                 R0*DEXP( -Kp*Kmu*Kmu0/R0*Abs_ice(L) ) 
               ENDDO  ! over reference wavelengths (L)
            ENDDO  ! over azimuthal angle (J)
         ENDDO  ! over incident angles (K)
      ENDDO  ! over reflection angle (I)
c      WRITE(*,*) '    SURF_REF  in option snoow_artm  '
      END SUBROUTINE SNOW_ARTM
c     **********************************************************************

      SUBROUTINE WHITE_SNOW_DB
      USE BRDF,    ONLY: Tau_sl, RefI_r, RefI_i, Ref_ip, L_yellow, 
     $                   Yellow_390, Nrso, Nrvi, Nraz, Sol_r0, Vie_r0, 
     $                   Azm_r0, R0_db
      USE ST_VARS, ONLY: Pie
c     ------------------------------------------------------------------
      INTEGER  Ws, Ws1
      INTEGER  Is1, Is2, Iv1, Iv2, Ia1, Ia2, Iss1, Iss2, Ivv1, Ivv2
c     ... Parameters Ap, Bp, and Cp the same as in "SNOW_ARTM"
      REAL (KND), PARAMETER :: Ap = 1.247d0, Bp = 1.186d0, Cp = 5.157d0
      REAL (KND) Dps, Dpv, Dpa, Ups, Upv, Upa, R0_int, Phi_0
      REAL (KND) Rr, U1, D1, T2, T3, x, r1, rU1, rD1, rT2, rT3,
     $     t1, tU1, tD1, tT2, tT3, Abs_yel
      REAL (KND) SIR, PIR, SCAT, R0, Kmu, Kmu0, P_TETA, DC, Ang, WLO
      REAL (KND), POINTER :: P_rr(:), P_ri(:)
      REAL (KND) Td(Nwl_BRF), Omeg(Nwl_BRF), G_par(Nwl_BRF),
     $     Gam(Nwl_BRF), Yps(Nwl_BRF)
c     ==================================================================

      IF( Nwl_Inp==1 .AND. Nwl_BRF > 1) THEN
c        ... Interpolation of refractive index  using SS_STOKES entry
         WLO = WL_INP(1)
         CALL LIN_SEARCH_NEW(WLO, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.d0 - DC)*RefI_r(Ws) + DC*RefI_r(Ws1)
         Rezi(1) = (1.d0 - DC)*RefI_i(Ws) + DC*RefI_i(Ws1)
         P_rr => Rezr(1:1)
         P_ri => Rezi(1:1)
      ELSE
c        ... Using values of refractive indece from input
         P_rr => RefI_r(:) ; P_ri => RefI_i(:) 
      ENDIF

      Abs_yel = 0.d0
      DO L = 1,Nwl_Inp
         IF( L_yellow ) THEN
            IF( WL_INP(L) <= 500.d0 ) THEN
               Abs_yel = Yellow_390*DEXP( -1.5d-2*(WL_INP(L) - 390.d0) ) 
            ELSE
               Abs_yel = Yellow_390
     $              *DEXP(-1.5d-2*110.d0 - 1.1d-2*(WL_INP(L)- 500.d0))
            ENDIF
         ENDIF
         Rr = P_rr(L)
         U1 = 2.0*(5.0*Rr**6 + 8.0*Rr**5 + 6.0*Rr**4 - 5.0*Rr**3 -Rr-1)
         D1 = 3.0*( Rr**3 +  Rr**2 + Rr + 1.0)*(Rr**4 - 1.0)
         T2 = Rr**2*(Rr**2 - 1.0)**2/( Rr**2 + 1.0)**3
     $        *DLOG( (Rr+1.0)/(Rr-1.0) )
         T3 = 8.0*Rr**4*(Rr**4 + 1.0)/(Rr**4-1.0)**2/(Rr**2 + 1.0)
     $        *DLOG( Rr )
         Td(L) = U1/D1 + T2 - T3
c        ... Ref_ip here is mean chord length of a grain [nm]
c            Abs_yel is absorption coefficientof yellow substance [nm^-1]
         x = Rr**2* (4.d0*Pie*P_ri(L)/WL_INP(L) + Abs_yel) *Ref_ip
c         write(*,*) WL_INP(L), Rr**2, x, Td(L)
         Omeg(L) = 1.0 - x*Td(L)/(x + Td(L))
c
         rU1 = Rr*( 3.0*Rr**11 + 3.0*Rr**10 + 25.0*Rr**9 + 25.0*Rr**8
     $        + 22.0*Rr**7  - 282.0*Rr**6 + 138.0*Rr**5
     $        + 186.0*Rr**4 + 151.0*Rr**3 - 89.0*Rr**2 + 13.0*Rr - 3.0)
        rD1 = 24.0*(Rr + 1.0)*(Rr**4 - 1.0)*(Rr**2 + 1.0)**2
        rT2 = 8.0*Rr**4*(Rr**6 - 3.0*Rr**4 + Rr**2 - 1.0)
     $       /(Rr**4 - 1.0)**2/(Rr**2 + 1.0)**2 *DLOG( Rr )
        rT3 = (Rr**8 + 12.0*Rr**6 + 54.0*Rr**4 - 4.0*Rr**2 + 1.0)
     $       *(Rr**2 - 1.0)**2/16.0/(Rr**2+1.0)**4
     $       *DLOG( (Rr+1.0)/(Rr-1.0) )
        r1 = rU1/rD1 + rT2 - rT3
c
        tU1 = (3.0*Rr**8 + 3.0*Rr**7 - 17.0*Rr**6 + 55.0*Rr**5
     $       - 39.0*Rr**4 - 7.0*Rr**3 - 27.0*Rr**2 - 11.0*Rr - 8.0)
        tD1 = 24.0*(Rr+1.0)*(Rr**4 - 1.0)*Rr
        tT2 = (Rr**2 - 1.0)**4/16.0/(Rr**2 + 1.0)**2/Rr
     $       *DLOG( (Rr+1.0)/(Rr-1.0) )
        tT3 = 4.0*Rr**5/(Rr**4-1.0)**2 *DLOG( Rr )
        t1 = tU1/tD1 - tT2 + tT3
        G_par(L) = (r1 + Rr**2*t1**2
     $       /( Td(L)*(1.0-Rr**2) - r1 + Rr**2*(Rr**2+x) ))/Omeg(L)
c      ... Parameters of asymptotic reflection function
        Gam(L) = SQRT( 3.d0*(1.d0 - Omeg(L))*(1.d0 - Omeg(L)*G_par(L)) )
        Yps(L) = 4.0*
     $     SQRT( (1.d0 - Omeg(L))/3.d0/(1.d0 - Omeg(L)*G_par(L)) )
      ENDDO  ! over BRDF wavelength grid (L)

      DO K = 1, N_INC
         K1 = (K-1)*Nvs+1 ; Kmu0 = 3.d0/7.d0*(1.d0 + 2.d0*COSI(K))
         CALL LIN_SEARCH_NEW( COSI(K), Ups, Is1, Is2, Nrso, Sol_r0 )
         Dps = 1.d0 - Ups 
         Iss1 = Nraz*Nrvi*(Is2 - 1) ;  Iss2 = Nraz*Nrvi*(Is1 - 1)
         DO I = 1, N_REF
            CALL LIN_SEARCH_NEW( COSR(I), Upv, Iv1, Iv2, Nrvi, Vie_r0 )
            Dpv = 1.d0 - Upv
            Ivv1 = Nraz*(Iv2-1) ;  Ivv2 = Nraz*(Iv1-1) 
            I1 = (I-1)*Nvs+1 ; Kmu = 3.d0/7.d0*(1.d0 + 2.d0*COSR(I))
            SIR = COSI(K) + COSR(I) ; PIR = COSI(K)*COSR(I)
            DO J = 1, N_AZM
               Phi_0 = PHI_REFL(J)
               IF( Phi_0 > Pie ) Phi_0 = 2.d0*Pie - Phi_0
               CALL LIN_SEARCH_NEW(Phi_0,DPa,Ia1,Ia2,Nraz,Azm_r0)
               Upa = 1.d0 - Dpa
               R0_int = R0_db(Iss1+Ivv1+Ia1)*Ups*Upv*Upa 
     $              + R0_db(Iss2+Ivv1+Ia1)*Dps*Upv*Upa 
     $              + R0_db(Iss2+Ivv1+Ia2)*Dps*Upv*Dpa 
     $              + R0_db(Iss2+Ivv2+Ia1)*Dps*Dpv*Upa 
     $              + R0_db(Iss2+Ivv2+Ia2)*Dps*Dpv*Dpa 
     $              + R0_db(Iss1+Ivv1+Ia2)*Ups*Upv*Dpa
     $              + R0_db(Iss1+Ivv2+Ia1)*Ups*Dpv*Upa
     $              + R0_db(Iss1+Ivv2+Ia2)*Ups*Dpv*Dpa

                     SCAT = -PIR + SINR(I)*SINI(K)*DCOS( PHI_REFL(J) )
               Ang = DACOS( SCAT )/Pie*180.d0
               DO L = 1,Nwl_Inp 
                 P_TETA = 11.1d0*DEXP( -0.087d0*Ang ) + 
     $                 1.1d0*DEXP( -0.014d0*Ang )
                 R0 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0
                 SURF_REFL(I1,K1,J,L) = R0_int*
     $                DSINH( Gam(L)*Tau_sl + Yps(L)
     $                        *(1.d0 - Kmu*Kmu0/R0_int) )
     $                /DSINH(Gam(L)*Tau_sl + Yps(L))
c                  SURF_REFL(I1,K1,J,L) = R0*
c     $                 DSINH( Gam(L)*Tau_sl + Yps(L)
c     $                       *(1.d0 - Kmu*Kmu0/R0) )
c     $                /DSINH(Gam(L)*Tau_sl + Yps(L))
               ENDDO  ! over reference wavelengths (L)
            ENDDO  ! over azimuthal angle (J)
         ENDDO  ! over reflection angle (I)
      ENDDO  ! over incident angles (K)
c      WRITE(*,*) '    SURF_REF  in option white_snoow  '
c      STOP '   Stop in "module_subr.f" ' 
c      PAUSE ' Pause in "module_subr" '
      END SUBROUTINE WHITE_SNOW_DB
c **********************************************************************

      SUBROUTINE WHITE_SNOW
      USE BRDF,    ONLY: Tau_sl,
     $                   RefI_r, RefI_i, Ref_ip, L_yellow, Yellow_390
      USE ST_VARS, ONLY: Pie
c     ------------------------------------------------------------------
      INTEGER  Ws, Ws1
c     ... Parameters Ap, Bp, and Cp the same as in "SNOW_ARTM"
      REAL (KND), PARAMETER :: Ap = 1.247d0, Bp = 1.186d0, Cp = 5.157d0
      REAL (KND) Rr, U1, D1, T2, T3, x, r1, rU1, rD1, rT2, rT3,
     $     t1, tU1, tD1, tT2, tT3, Abs_yel
      REAL (KND) SIR, PIR, SCAT, R0, Kmu, Kmu0, P_TETA, DC, Ang, WLO
      REAL (KND), POINTER :: P_rr(:), P_ri(:)
      REAL (KND) Td(Nwl_BRF), Omeg(Nwl_BRF), G_par(Nwl_BRF),
     $     Gam(Nwl_BRF), Yps(Nwl_BRF)
c     ==================================================================

      IF( Nwl_Inp==1 .AND. Nwl_BRF > 1) THEN
c        ... Interpolation of refractive index  using SS_STOKES entry
         WLO = WL_INP(1)
         CALL LIN_SEARCH_NEW(WLO, DC, Ws, Ws1, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.d0 - DC)*RefI_r(Ws) + DC*RefI_r(Ws1)
         Rezi(1) = (1.d0 - DC)*RefI_i(Ws) + DC*RefI_i(Ws1)
         P_rr => Rezr(1:1)
         P_ri => Rezi(1:1)
      ELSE
c        ... Using values of refractive indece from input
         P_rr => RefI_r(:) ; P_ri => RefI_i(:) 
      ENDIF

      Abs_yel = 0.d0
      DO L = 1,Nwl_Inp
         IF( L_yellow ) THEN
            IF( WL_INP(L) <= 500.d0 ) THEN
               Abs_yel = Yellow_390*DEXP( -1.5d-2*(WL_INP(L) - 390.d0) ) 
            ELSE
               Abs_yel = Yellow_390
     $              *DEXP(-1.5d-2*110.d0 - 1.1d-2*(WL_INP(L)- 500.d0))
            ENDIF
         ENDIF
         Rr = P_rr(L)
         U1 = 2.0*(5.0*Rr**6 + 8.0*Rr**5 + 6.0*Rr**4 - 5.0*Rr**3 -Rr-1)
         D1 = 3.0*( Rr**3 +  Rr**2 + Rr + 1.0)*(Rr**4 - 1.0)
         T2 = Rr**2*(Rr**2 - 1.0)**2/( Rr**2 + 1.0)**3
     $        *DLOG( (Rr+1.0)/(Rr-1.0) )
         T3 = 8.0*Rr**4*(Rr**4 + 1.0)/(Rr**4-1.0)**2/(Rr**2 + 1.0)
     $        *DLOG( Rr )
         Td(L) = U1/D1 + T2 - T3
c        ... Ref_ip here is mean chord length of a grain [nm]
c            Abs_yel is absorption coefficientof yellow substance [nm^-1]
         x = Rr**2* (4.d0*Pie*P_ri(L)/WL_INP(L) + Abs_yel) *Ref_ip
c         write(*,*) WL_INP(L), Rr**2, x, Td(L)
         Omeg(L) = 1.0 - x*Td(L)/(x + Td(L))
c
         rU1 = Rr*( 3.0*Rr**11 + 3.0*Rr**10 + 25.0*Rr**9 + 25.0*Rr**8
     $        + 22.0*Rr**7  - 282.0*Rr**6 + 138.0*Rr**5
     $        + 186.0*Rr**4 + 151.0*Rr**3 - 89.0*Rr**2 + 13.0*Rr - 3.0)
        rD1 = 24.0*(Rr + 1.0)*(Rr**4 - 1.0)*(Rr**2 + 1.0)**2
        rT2 = 8.0*Rr**4*(Rr**6 - 3.0*Rr**4 + Rr**2 - 1.0)
     $       /(Rr**4 - 1.0)**2/(Rr**2 + 1.0)**2 *DLOG( Rr )
        rT3 = (Rr**8 + 12.0*Rr**6 + 54.0*Rr**4 - 4.0*Rr**2 + 1.0)
     $       *(Rr**2 - 1.0)**2/16.0/(Rr**2+1.0)**4
     $       *DLOG( (Rr+1.0)/(Rr-1.0) )
        r1 = rU1/rD1 + rT2 - rT3
c
        tU1 = (3.0*Rr**8 + 3.0*Rr**7 - 17.0*Rr**6 + 55.0*Rr**5
     $       - 39.0*Rr**4 - 7.0*Rr**3 - 27.0*Rr**2 - 11.0*Rr - 8.0)
        tD1 = 24.0*(Rr+1.0)*(Rr**4 - 1.0)*Rr
        tT2 = (Rr**2 - 1.0)**4/16.0/(Rr**2 + 1.0)**2/Rr
     $       *DLOG( (Rr+1.0)/(Rr-1.0) )
        tT3 = 4.0*Rr**5/(Rr**4-1.0)**2 *DLOG( Rr )
        t1 = tU1/tD1 - tT2 + tT3
        G_par(L) = (r1 + Rr**2*t1**2
     $       /( Td(L)*(1.0-Rr**2) - r1 + Rr**2*(Rr**2+x) ))/Omeg(L)
c      ... Parameters of asymptotic reflection function
        Gam(L) = SQRT( 3.d0*(1.d0 - Omeg(L))*(1.d0 - Omeg(L)*G_par(L)) )
        Yps(L) = 4.0*
     $     SQRT( (1.d0 - Omeg(L))/3.d0/(1.d0 - Omeg(L)*G_par(L)) )
      ENDDO

c      OPEN( 44, file = './DATA_OUT/omega.dat' )
c      DO L = 1,Nwl_Inp
c         WRITE(44,'(2x,F8.2,*(2x,e12.4))') WL_INP(L), Omeg(L),
c     $        G_par(L)
c      ENDDO
c      CLOSE(44)
            
      DO I = 1, N_REF
         I1 = (I-1)*Nvs+1 ; Kmu = 3.d0/7.d0*(1.d0 + 2.d0*COSR(I))
         DO K = 1, N_INC
            K1 = (K-1)*Nvs+1 ; Kmu0 = 3.d0/7.d0*(1.d0 + 2.d0*COSI(K))
            SIR = COSI(K) + COSR(I) ; PIR = COSI(K)*COSR(I)
            DO J = 1, N_AZM
               SCAT = -PIR + SINR(I)*SINI(K)*DCOS( PHI_REFL(J) )
               Ang = DACOS( SCAT )/Pie*180.d0
               DO L = 1,Nwl_Inp 
                 P_TETA = 11.1d0*DEXP( -0.087d0*Ang ) + 
     $                 1.1d0*DEXP( -0.014d0*Ang )
                 R0 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0

                 IF( K==33.AND.J==1.AND.L==1 ) THEN
                    WRITE(*,'(2x,3(2x,F8.2),3x,F8.3)')
     $                   DACOS(COSI(K))/Pie*180.d0,
     $                   DACOS(COSR(I))/Pie*180.d0,
     $                   PHI_REFL(J)/Pie*180., R0
                 ENDIF
                 
                  SURF_REFL(I1,K1,J,L) = R0*
     $                 DSINH( Gam(L)*Tau_sl + Yps(L)
     $                       *(1.d0 - Kmu*Kmu0/R0) )
     $                /DSINH(Gam(L)*Tau_sl + Yps(L))
               ENDDO  ! over reference wavelengths (L)
            ENDDO  ! over azimuthal angle (J)
         ENDDO  ! over incident angles (K)
      ENDDO  ! over reflection angle (I)
      WRITE(*,*) '    SURF_REF  in option white_snoow  '
c      PAUSE ' Pause in "module_subr" '
      END SUBROUTINE WHITE_SNOW
c **********************************************************************

      SUBROUTINE SNOW_BRDF
      DO J = 1, N_AZM
         CPHI = DCOS(PI + PHI_REFL(J))
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
               H = T_1/(T_2 + 2*THE*COS_G)/DSQRT(T_2 + 2*THE*COS_G)
               G = DSQRT(TANR(I)*TANR(I) + TANI(K)*TANI(K) 
     $              - 2.0D0*TANR(I)*TANI(K)*CPHI)
               TT = BRDF_PARAM(5)*DEXP(BRDF_PARAM(6)*
     $              (PI - DACOS(COS_G))) + 1.0D0
               R = (1.0D0 - RO_0)/(1.0D0 + G)
               SURF_REFL(I1,K1,J,1) = RO_0*(H*(1.0D0 + R)* !Preliminary
     $              (COSR(I)*COSI(K))**(KK - 1.0D0)/
     $              (COSR(I) + COSI(K))**(1.0D0 - KK) 
     $              + BRDF_PARAM(4)/COSI(K))*TT
            ENDDO
         ENDDO
      ENDDO
      END SUBROUTINE SNOW_BRDF
c **********************************************************************

      SUBROUTINE REFL_MELT_POND
      USE BRDF,  ONLY:  MP_gp, MP_bub, MP_bri, RefW_r, RefW_i, RefI_r,
     $     RefI_i
c     -------------------------------------------------------------------
      REAL (KND)  Sig(2), Sig2, tpar, tau_ice
      
c     ... Scattering transport coefficient for bubbles
      Sig(1) = 1.5d0*MP_bub(1)/MP_bub(2)*(1.d0 - MP_bub(3))  ! [m-1]
c     ... Scattering transport coefficient for brine
      Sig(2) = 1.5d0*MP_bri(1)/MP_bri(2)*(1.d0 - MP_bri(3))  ! [m-1]
      Sig2 = Sig(1) + Sig(2) 
c      tpar = 8.d0/3.d0* Abs_ice(lambda)/Sig2
c      tau_ice = (Sig2 + Abs_ice(lambda))* MP_gp(1)
      END SUBROUTINE REFL_MELT_POND
c **********************************************************************

      SUBROUTINE RPV_BRDF
      DO J = 1, N_AZM
         CPHI = DCOS(PI + PHI_REFL(J))
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
               DO L = 1,Nwl_Inp ! Preliminary
                  H = T_1/(T_2 + 2*THE*COS_G)/DSQRT(T_2 +2*THE*COS_G)
                  G = DSQRT( TANR(I)*TANR(I) + TANI(K)*TANI(K)
     $                 - 2.d0*TANR(I)*TANI(K)*CPHI)
                  R = (1.d0 - RO_0)/(1.d0 + G)
                  SURF_REFL(I1,K1,J,L) = RO_0*H*(1.d0 + R)*
     $                 (COSR(I)*COSI(K))**(KK - 1.d0)/
     $                 (COSR(I) + COSI(K))**(1.d0 - KK) 
               ENDDO
            END DO
         END DO
      ENDDO
      END SUBROUTINE RPV_BRDF

c *********************************************************************
c     In the vector case only the lement R_11 of the BRDF matrix
c                    will be calculated
c *********************************************************************
      SUBROUTINE mRPV_BRDF
      INTEGER Ws, Ws1, I
      REAL (KND) Pok, DC, Par_1(1,3)
      TARGET Par_1
      REAL (KND), POINTER :: Par_po(:,:)
c    --------------------------------------------------------------------
      IF( Nwl_Inp==1 .AND. Nwl_BRF > 1) THEN
         CALL LIN_SEARCH_NEW(WL_INP(1), DC, Ws, Ws1, Nwl_BRF, WL_BRF)
         DO I = 1, 3
            Par_1(1,I) = (1.d0 - Dc)*Par_mix(Ws,I) + Dc*Par_mix(Ws1,I)
         ENDDO
         Par_po => Par_1(1:1,1:3) 
      ELSE
         Par_po => Par_mix(:,1:3) 
      ENDIF

      DO J = 1, N_AZM
         CPHI = DCOS(PI + PHI_REFL(J))
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
               DO L = 1,Nwl_Inp 
                  Pok = Par_po(L,2) - 1.d0
                  SURF_REFL(I1,K1,J,L) = Par_po(L,1)
     $                 *( COSR(I)*COSI(K)*(COSR(I) + COSI(K)) )**Pok
     $                 *DEXP( Par_po(L,3)*COS_G )
               ENDDO  ! over reference wavelengths (L)
            ENDDO  ! over incidence angles (K)
         ENDDO  ! over reflection angles (I)
      ENDDO  ! over azimuthal angles (J)
      END SUBROUTINE mRPV_BRDF
c **********************************************************************

c     ... Ross-Li BRDF using in standard MODIS processing
      SUBROUTINE Ross_Li
      USE BRDF,     ONLY: Nwl_veg, Ker_cof => IVG_cof, Nwl_RLi 
      USE LOG_PARS, ONLY: DIR_OUT
c     -------------------------------------------------------------------
      INTEGER     I_pos, I_inc
      REAL (KND)  Thet, C_c2, TT, Ms, Mp, Fg_max,
     $     Cos_t, Ang_t, SPHI, Del, F_vol, F_geo, D1, S_sec, Dcr
      REAL (KND)  Ex_inc(N_INC), Ex_ref(N_REF)
      REAL (KND), PARAMETER :: Ap = 1.247d0, Bp = 1.186d0, Cp = 5.157d0,
     $     Alpha = 0.3d0, KK0 = 0.9858d0
      REAL (KND) SIR, PIR, SCAT, R0, Kmu, Kmu0, P_TETA, DC, Ang

      Dcr = DCOS( Cr_ang/180.d0*Pie ) ; C_c2 = Pie/4.d0
      DO J = 1, N_AZM
         CPHI = DCOS(PI + PHI_REFL(J))
         SPHI = DSIN(PI + PHI_REFL(J))
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
               S_sec = 1.d0/COSR(I) + 1.d0/COSI(K) 
               Mp = COSR(I)*COSI(K) ; Ms = COSR(I) + COSI(K)
c              ... The phase angle in [rad]
               Thet = DACOS( COS_G ) ;  TT = TANR(I)*TANI(K)
               Del = DSQRT(TANR(I)**2 + TANI(K)**2 - 2.d0*TT*CPHI)
c              ... Volumetric scattering K_vol (RossThick kernel)
c                  including hot-spot correction function
c               F_vol = ( 1.d0 + 1.d0/(1.d0 + Thet/HS_par) )
               F_vol = (2.d0*HS_par + Thet)/(HS_par + Thet)
     $              *( (Pie/2.d0-Thet)*COS_G + DSIN(Thet) )/Ms - C_c2
c              --------------------------------------------------------
               D1 = 2.d0/S_sec*DSQRT( Del**2 + (TT*SPHI)**2 )
c               Cos_t = DMIN1(1.d0,D1)
               IF( D1 > 1.d0 ) THEN
                  Cos_t = 1.d0 
               ELSE
                  Cos_t = D1           
               ENDIF
               Ang_t = DACOS( Cos_t ) 
c              ... Geometric-optical surface scattering (LiSparse-R kernel)
               F_geo = S_sec/Pie*(Ang_t - DSIN(Ang_t)*Cos_t)
     $              - S_sec + (1.d0 + COS_G)/2.d0/Mp
c              ----------------------------------------------------------
               SURF_REFL(I1,K1,J,1) = F_vol               
               SURF_REFL(I1,K1,J,2) = F_geo
            END DO  ! over incidence angles (K)
         END DO  ! over reflectance angles (I)
      ENDDO   ! over azimuthal angles (J)

      IF( TRIM(Ex_Type)=='off' ) THEN 
         IF( Nwl_RLi == 1 ) THEN
c           ... The negative and "too large" values will be replaced
c               with zero and maximal reflectance, respectively
            Fg_max = (1.d0 - Ker_cof(1,1))*0.5d0
            DO J = 1, N_AZM
               DO I = 1, N_REF  ;  I1 = (I-1)*Nvs+1
                  DO K = 1, N_INC  ;  K1 = (K-1)*Nvs+1
                     F_vol = SURF_REFL(I1,K1,J,1)
                     F_geo = SURF_REFL(I1,K1,J,2)
                     IF( F_geo*Ker_cof(3,1)*COSR(I) > Fg_max ) THEN
                        SURF_REFL(I1,K1,J,2) = 
     $                       Fg_max/COSR(I)/Ker_cof(3,1)
                     ENDIF
                     IF( Ker_cof(1,1) + Ker_cof(2,1)*F_vol + 
     $                    Ker_cof(3,1)*F_geo < 0.d0 ) THEN
                        SURF_REFL(I1,K1,J,1) = 
     $                       -(Ker_cof(1,1) + Ker_cof(3,1)*F_geo)
     $                       /Ker_cof(2,1)
                     ENDIF
                  END DO  ! over incidence angles (K)
               END DO  ! over reflectance angles (I)
            ENDDO   ! over azimuthal angles (J)
         ENDIF  ! Nwl_RLi == 1
         IF( Nwl_BRF==2 ) RETURN ! Pure Ross-Li without replacement
      ELSE  ! Ex_Type /= 'off'
c     ... Replace BRF at grazing angles
         I_pos = MINLOC( DABS(COSR(:) - Dcr),1 )
         I_inc = MINLOC( DABS(COSI(:) - Dcr),1 )
         SELECT CASE( TRIM(Ex_Type) )
         CASE('cos')
            Ex_inc(:)=COSI(:)/COSI(I_inc)
            Ex_ref(:)=COSR(:)/COSR(I_pos)
         CASE('const')
            Ex_inc(:) = 1.d0 ; Ex_ref(:) = 1.d0 
         CASE('zero')
            Ex_inc(:) = 0.d0 ; Ex_ref(:) = 0.d0 
         END SELECT
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               IF( COSR(I)<Dcr.AND.COSI(K)<Dcr ) SURF_REFL(I1,K1,:,1:2)
     $              =  SURF_REFL((I_pos-1)*Nvs+1,(I_inc-1)*Nvs+1,:,1:2)
     $              *Ex_ref(I)*Ex_inc(K)
               IF( COSR(I)<Dcr.AND.COSI(K)>=Dcr ) SURF_REFL(I1,K1,:,1:2) 
     $              = SURF_REFL((I_pos-1)*Nvs+1,K1,:,1:2)*Ex_ref(I)
               IF( COSR(I)>=Dcr.AND.COSI(K)<Dcr ) SURF_REFL(I1,K1,:,1:2)
     $               = SURF_REFL(I1,(I_inc-1)*Nvs+1,:,1:2)*Ex_inc(K)
            ENDDO
         ENDDO
      ENDIF
      IF( Nwl_BRF==2 ) RETURN   ! Pure Ross-Li including replacement

c     ... Snow kern
      DO I = 1, N_REF
         I1 = (I-1)*Nvs+1 ; Kmu = 1.d0 + 2.d0*COSR(I)
         DO K = 1, N_INC
            K1 = (K-1)*Nvs+1 ; Kmu0 = 1.d0 + 2.d0*COSI(K)
            SIR = COSI(K) + COSR(I) ; PIR = COSI(K)*COSR(I)
            DO J = 1, N_AZM
               SCAT = -PIR + SINR(I)*SINI(K)*DCOS( PHI_REFL(J) )
               Ang = DACOS( SCAT )/Pie*180.d0
               P_TETA = 11.1d0*DEXP( -0.087d0*Ang ) + 
     $              1.1d0*DEXP( -0.014d0*Ang )
               R0 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0
               SURF_REFL(I1,K1,J,3) = 
     $              R0*(1.d0 + Alpha*Scat*DEXP(Scat)) - KK0
            ENDDO  ! over azimuthal angle (J)
         ENDDO  ! over incident angles (K)
      ENDDO  ! over reflection angle (I)

      RETURN
      END SUBROUTINE Ross_Li

c **********************************************************************
c     ... 4 kernels snow BRDF model (FASMAR)
      SUBROUTINE Snow_fasmar
      USE BRDF,  ONLY: SCA_ANG, SCA_MAT, Snow_abs
      USE CLD_INP, ONLY: Ydb_ang
c     --------------------------------------------------------------------
      INTEGER  II, KK
      REAL (KND)  SIR, PIR, SCAT, R0, Kmu, Kmu0, P_TETA, Ang, Gam, Phm,
     $     K1_00, K2_00, K3_00, Kp, Ry, Ky_00
      REAL (KND), PARAMETER :: Ap = 1.247d0, Bp = 1.186d0, Cp = 5.157d0

      Kp = 0.66   ! for fractal snow grains
c     ... Kernels value at nadir-zenith
      SIR = 2.d0  ;  PIR = 1.d0  ;  Ang = 180.d0
      Kmu0 = 3.d0 ; Kmu = 3.d0
      P_TETA = 11.1d0*DEXP(-0.087d0*Ang) + 1.1d0*DEXP(-0.014d0*Ang)
      K1_00 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0
      K2_00 = SCA_MAT(Ydb_ang)/Sir/4.d0
      K3_00 = -K1_00*Exp( -1.d0)
      Ky_00 = K1_00*DEXP( -Kp*Kmu*Kmu0/K1_00*Snow_abs(1) )
c     ------------------------------------------------------------------
      DO I = 1, N_REF
         I1 = (I-1)*Nvs+1 ; Kmu = 1.d0 + 2.d0*COSR(I)
         DO K = 1, N_INC
            K1 = (K-1)*Nvs+1 ; Kmu0 = 1.d0 + 2.d0*COSI(K)
            SIR = COSI(K) + COSR(I) ; PIR = COSI(K)*COSR(I)
            DO J = 1, N_AZM
               SCAT = -PIR + SINR(I)*SINI(K)*DCOS( PHI_REFL(J) )
               Ang = DACOS( SCAT )/Pie*180.d0
               P_TETA = 11.1d0*DEXP( -0.087d0*Ang ) + 
     $              1.1d0*DEXP( -0.014d0*Ang )
               R0 = (Ap + Bp*SIR + Cp*PIR + P_TETA)/SIR/4.D0
               Ry = R0*DEXP( -Kp*Kmu*Kmu0/R0*Snow_abs(1) ) 
c              ... Interpolation of selected scattering matrix
               CALL LIN_SEARCH_NEW
     $              (Scat, Gam, II, KK, Ydb_ang, SCA_ANG)
               Phm = (1.d0-Gam)*SCA_MAT(II) + Gam*SCA_MAT(KK)               
               SURF_REFL(I1,K1,J,1) = Ry - Ky_00 
c               SURF_REFL(I1,K1,J,1) = R0 - K1_00 
               SURF_REFL(I1,K1,J,2) = Phm/Sir/4.d0 - K2_00 
               SURF_REFL(I1,K1,J,3) = R0*SCAT*EXP( SCAT ) - K3_00 
c               WRITE(*,'(3(1x,I3),3(2x,e12.4))') I,K,J, R0,
c     $              Phm/Sir/4.d0, R0*SCAT*EXP( SCAT )
            ENDDO               ! over azimuthal angle (J)
         ENDDO  ! over incident angles (K)
      ENDDO  ! over reflection angle (I)

      RETURN
      END SUBROUTINE Snow_fasmar

c **********************************************************************

c     ... BRDF using vegetation-soil kernel approach
      SUBROUTINE SVK_BRDF
      USE BRDF,     ONLY: Alph_1, Alph_2, Beta_1, Beta_2
      USE LOG_PARS, ONLY: DIR_OUT
      INTEGER INR(2), INW, LC, I_pos, I_inc
      REAL (KND)  Thet, C_c1, C_c2, TT, Ms, Mp, 
     $     Cos_t, Ang_t, SPHI, Del, F_vol, F_geo, D1, S_sec, Dcr
      REAL (KND)  Ex_inc(N_INC), Ex_ref(N_REF)

cc     ... Parameters for WL < Wgr_svk and WL >= Wgr_svk (see modules.f)
c      DATA Alph_1 /1.d0, 2.d0/
c      DATA Beta_1  /0.5d0, 0.5d0/
c      DATA Alph_2 /0.2d0, -0.05d0/
c      DATA Beta_2  /0.1d0, 0.15d0/
      
      SELECT CASE( TRIM(PDF_TYPE) )
      CASE('first')
         INW = 1 ; INR(1) = 1
      CASE('second')
         INW = 1 ; INR(1) = 2
      CASE('both')
         INW = 2 ; INR(1) = 1 ; INR(2) = 2
      END SELECT

c      DO L = 1, INW 
c         Lc = INR(L)
c         WRITE(*,*) '  f_vol/Ko = ', 
c     $        4.d0/3.d0/Pie*(Alph_1(Lc)*NDVI + Beta_1(Lc))
c         WRITE(*,*) '  f_geo/Ko = ', 
c     $        (Alph_2(Lc)*NDVI + Beta_2(Lc))
c      ENDDO

      Dcr = DCOS( Cr_ang/180.d0*Pie )

      C_c1 = 4.d0/3.d0/Pie ; C_c2 = 1.d0/3.d0
      DO J = 1, N_AZM
         CPHI = DCOS(PI + PHI_REFL(J))
         SPHI = DSIN(PI + PHI_REFL(J))
         DO I = 1, N_REF
            I1 = (I-1)*Nvs+1
            DO K = 1, N_INC
               K1 = (K-1)*Nvs+1
               COS_G = COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
               S_sec = 1.d0/COSR(I) + 1.d0/COSI(K) 
               Mp = COSR(I)*COSI(K) ; Ms = COSR(I) + COSI(K)
c              ... phase angle in [rad]
               Thet = DACOS( COS_G ) ;  TT = TANR(I)*TANI(K)
               Del = DSQRT(TANR(I)**2 + TANI(K)**2 - 2.d0*TT*CPHI)

c               ... Volume scattering term (kernel), STB Eq.(F-vol)
c               F_vol = C_c1/Ms  ! without hot-spot parameter
c               F_vol = C_c1/Ms*( 1.d0 + 1.d0/(1.d0 + Thet/HS_par) )
               F_vol = C_c1/Ms*(2.d0*HS_par + Thet)/(HS_par + Thet)
     $              *( (Pie/2.d0-Thet)*COS_G + DSIN(Thet) ) - C_c2

c              ... Geometrical-optic kernel, STB Eq.(F-geo)
               D1 = 2.d0/S_sec*DSQRT( Del**2 + (TT*SPHI)**2 )
c               Cos_t = DMIN1(1.d0,D1)
               IF( D1 > 1.d0 ) THEN
                  Cos_t = 1.d0 
               ELSE
                  Cos_t = D1           
               ENDIF
               Ang_t = DACOS( Cos_t ) 
               F_geo = S_sec/Pie*(Ang_t - DSIN(Ang_t)*Cos_t)
     $              - S_sec + (1.d0 + COS_G)/2.d0/Mp
               
               DO L = 1, INW ; Lc = INR(L)
                  SURF_REFL(I1,K1,J,L) = 1.d0 
     $                 +  ( Alph_1(Lc)*NDVI + Beta_1(Lc) )*F_vol 
     $                 +  ( Alph_2(Lc)*NDVI + Beta_2(Lc) )*F_geo
               ENDDO
            END DO      ! over incidence angles (K)
         END DO      ! over reflectance angles (I)
      ENDDO       ! over azimuthal angles (J)
  
c      WRITE(*,*) '    Product for F_vol: ',Alph_1(1)*NDVI + Beta_1(1)
c      WRITE(*,*) '    Product for F_geo: ',Alph_2(1)*NDVI + Beta_2(1)
c      WRITE(*,*) '   Ex_TYPE = ', Ex_Type
      IF( TRIM(Ex_Type)=='off' ) RETURN
c
c     ... Replace BRF 
      I_pos = MINLOC( DABS(COSR(:) - Dcr),1 )
      I_inc = MINLOC( DABS(COSI(:) - Dcr),1 )

      SELECT CASE( TRIM(Ex_Type) )
      CASE('cos')
         Ex_inc(:)=COSI(:)/COSI(I_inc); Ex_ref(:)=COSR(:)/COSR(I_pos)
      CASE('const')
         Ex_inc(:) = 1.d0 ; Ex_ref(:) = 1.d0 
      CASE('zero')
         Ex_inc(:) = 0.d0 ; Ex_ref(:) = 0.d0 
      END SELECT

      DO I = 1, N_REF
         I1 = (I-1)*Nvs+1
         DO K = 1, N_INC
            K1 = (K-1)*Nvs+1
            IF( COSR(I)<Dcr .AND. COSI(K)<Dcr ) SURF_REFL(I1,K1,:,:) = 
     $           SURF_REFL((I_pos-1)*Nvs+1,(I_inc-1)*Nvs+1,:,:)*
     $           Ex_ref(I)*Ex_inc(K)
            
            IF( COSR(I)<Dcr .AND. COSI(K)>=Dcr ) SURF_REFL(I1,K1,:,:) = 
     $           SURF_REFL((I_pos-1)*Nvs+1,K1,:,:)*Ex_ref(I)

            IF( COSR(I)>=Dcr .AND. COSI(K)<Dcr ) SURF_REFL(I1,K1,:,:) = 
     $           SURF_REFL(I1,(I_inc-1)*Nvs+1,:,:)*Ex_inc(K)
         ENDDO
      ENDDO

c       WRITE(*,*) ' -1 ', SUM(SURF_REFL(:,:,:,1))
c       WRITE(*,*) ' -2 ', SUM(SURF_REFL(:,:,:,2))

c      OPEN( 78, file=DIR_INP//'/BRF_TES.dat' )
c        WRITE(78,*) '     Kernal-based BRF before normalization '
c        WRITE(78,'(a,f7.3)') 
c     $      '       for spectral range shorter than 700 nm'
c        WRITE(78,'(a,f7.3)') 
c     $      '       for solar zenith angle ',DACOS(COSI(N_INC))/Pie*180.
c        WRITE(78,'(a,2(2x,f7.3))') 
c     $      '       for azimuthal angles ', PHI_REFL(1)/Pie*180., 
c     $       PHI_REFL(N_azm)/Pie*180.
c        DO I = 1, N_REF
c           WRITE(78,'(1x,F7.3,20(1x,E12.4))') 
c     $          DACOS(COSR(I))/Pie*180., SURF_REFL(I,N_INC,1,1),
c     $          SURF_REFL(I,N_INC,N_azm,1)
c        ENDDO
c      CLOSE(78)
      RETURN
      END SUBROUTINE SVK_BRDF
c **********************************************************************

      END SUBROUTINE

      END MODULE BRDF_SUBRS

C********************************************************************C
      MODULE ABS_COEFF_PHOTOCHEM
      USE GRIDS, ONLY : NLAYERS, ZZZ
      USE PHOTOCHEMISTRY, ONLY: N_SOL_PC, XS_PC, PCY_LOC, PC_LOG_FLAG
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: PCC_TYPE
      PRIVATE
      PUBLIC :: PC_ABS_COEFF
      CONTAINS
      SUBROUTINE PC_ABS_COEFF(INDEX_PC, XS, NL_DS, PCC, PC_TH_FLAG, 
     $     FP_FLAG, AL_PC, H_0, TH_IND, ITH, Z_FP, J_OUT)

      TYPE (PCC_type) :: PCC    

      LOGICAL       FP_FLAG, PC_TH_FLAG
      INTEGER       NL_DS, I, ITH, L, J_OUT, N_START, TH_IND
      INTEGER       INDEX_PC(NL_DS), LL(NL_DS)
      REAL (KND)    XS(NL_DS), AL_PC(NL_DS), PCY_TH(4, N_SOL_PC)
      REAL (KND)    H_0, SD, Z_FP
      OPTIONAL :: H_0, TH_IND, ITH, Z_FP, J_OUT

      DO L = 1, NL_DS
         IF (XS(L).LE.XS_PC(N_SOL_PC)) THEN
            LL(L) = 1
            CYCLE
         END IF
         IF (XS(L).GE.XS_PC(1)) THEN
            LL(L) = N_SOL_PC
            CYCLE
         END IF
         
         DO I = 1, N_SOL_PC - 1
            IF(XS(L).GE.XS_PC(N_SOL_PC - I + 1).AND.
     $           XS(L).LT.XS_PC(N_SOL_PC - I)) THEN
               LL(L) = I
               EXIT
            END IF
         END DO
      END DO

      IF(FP_FLAG) THEN
         N_START = 2
         CALL GT_PC_TH (ZZZ(J_OUT), Z_FP, 
     $        PCC%PCX_COEFF(NLAYERS - J_OUT + 1,:,:), PCY_TH)
         SD = DIM(XS(1),XS_PC(N_SOL_PC - LL(1) + 1))
         AL_PC(1) = PCY_TH(1,LL(1)) + PCY_TH(2,LL(1))*SD + 
     $        PCY_TH(3,LL(1))*SD**2 + PCY_TH(4,LL(1))*SD**3
      ELSE
         N_START = 1
      END IF   

      DO L = N_START, NL_DS
         SD = DDIM(XS(L),XS_PC(N_SOL_PC - LL(L) + 1))
         AL_PC(L) = PCC%PCY_COEFF(LL(L),INDEX_PC(L),1) + 
     $        PCC%PCY_COEFF(LL(L),INDEX_PC(L),2)*SD +
     $        PCC%PCY_COEFF(LL(L),INDEX_PC(L),3)*SD**2 + 
     $        PCC%PCY_COEFF(LL(L),INDEX_PC(L),4)*SD**3
      END DO
         
      IF(PC_TH_FLAG) THEN
         CALL GT_PC_TH (ZZZ(ITH), H_0, 
     $        PCC%PCX_COEFF(NLAYERS - ITH + 1,:,:), PCY_TH)
         SD = DIM(XS(TH_IND),XS_PC(N_SOL_PC - LL(TH_IND) + 1))
         AL_PC(TH_IND) = PCY_TH(1,LL(TH_IND)) + 
     $        PCY_TH(2,LL(TH_IND))*SD + PCY_TH(3,LL(TH_IND))*SD**2 +
     $        PCY_TH(4,LL(TH_IND))*SD**3
      END IF   

      IF(PC_LOG_FLAG) AL_PC(1:NL_DS) = DEXP(AL_PC(1:NL_DS)) 

      RETURN
      END SUBROUTINE

      SUBROUTINE GT_PC_TH (X_AX, X, PCX_COEFF, PCY_TH)

      INTEGER       I, IDX_Y
      REAL (KND)    FUNC_Y(N_SOL_PC + 4), ST(N_SOL_PC + 4)
      REAL (KND)    T(N_SOL_PC + 4)
      REAL (KND)    PCX_COEFF(N_SOL_PC, 4), PCY_TH(4, N_SOL_PC)
      REAL (KND)    DELTA, X, X_AX 

      DELTA = X - X_AX 

      DO IDX_Y = 1, N_SOL_PC
         FUNC_Y(IDX_Y + 2) = PCX_COEFF(IDX_Y,1) + 
     $        PCX_COEFF(IDX_Y,2)*DELTA + 
     $        PCX_COEFF(IDX_Y,3)*DELTA**2 + 
     $        PCX_COEFF(IDX_Y,4)*DELTA**3
      END DO

C     Caculation of slopes of curves
C     ******************************
      DO I = 3, N_SOL_PC + 1
         ST(I) = (FUNC_Y(I + 1) - FUNC_Y(I))/
     $        (PCY_LOC(I + 1) - PCY_LOC(I))
      END DO
         
      ST(2) = 2.0D0*ST(3) - ST(4)
      ST(1) = 2.0D0*ST(2) - ST(3)
      ST(N_SOL_PC + 2) = 2.0D0*ST(N_SOL_PC + 1) - ST(N_SOL_PC)
      ST(N_SOL_PC + 3) = 2.0D0*ST(N_SOL_PC + 2) - ST(N_SOL_PC + 1)
      
C     Calculation of all polynomial slopes
C     ************************************
      DO I = 3, N_SOL_PC + 2
         IF(ST(I - 2).EQ.ST(I - 1).AND.ST(I).EQ.ST(I + 1)) THEN
            T(I) = 0.5D0*(ST(I + 1) + ST(I))
         ELSE
            T(I) = (DABS(ST(I + 1) - ST(I))*ST(I - 1) +
     $           DABS(ST(I - 1) - ST(I - 2))*ST(I))/
     $           (DABS(ST(I + 1) - ST(I)) + 
     $           DABS(ST(I - 1) - ST(I - 2)))
         END IF
      END DO
      
C     Calculation of polynomial coefficients
C     **************************************
      
      DO I = 1, N_SOL_PC - 1
         PCY_TH(1,I) = FUNC_Y(I + 2)
         PCY_TH(2,I) = T(I + 2)
         PCY_TH(3,I) = (3.0D0*ST(I + 2) - 2.0D0*T(I + 2) - T(I + 3))/
     $        (PCY_LOC(I + 3) - PCY_LOC(I + 2))
         PCY_TH(4,I) = (T(I + 2) + T(I + 3) - 2.0D0*ST(I + 2))
     $        /(PCY_LOC(I + 3) - PCY_LOC(I + 2))**2
      END DO

      PCY_TH(1,N_SOL_PC) = FUNC_Y(N_SOL_PC + 2)
      PCY_TH(2:4,N_SOL_PC) = 0.0D0

      RETURN 
      END SUBROUTINE
      END MODULE ABS_COEFF_PHOTOCHEM

C********************************************************************C
      MODULE ABS_COEFF_PC
      USE ABS_COEFF_PHOTOCHEM
      END MODULE ABS_COEFF_PC
C********************************************************************C
