!**************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 19 Nov 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!**************************************************************************
Subroutine Nodes_Eqd ()
  ! Called by "rtm_init.f"

  Use NODES
  Use ST_Vars, Only: Pie
  Use Types,   Only: KND
  ! Local parameters
  Character (Len = *), Parameter :: R_NAME = 'Nodes'
  Integer Idx(1), II, KK, JJ, Nh
  Logical AllocAll
  Logical, Allocatable :: PlusFlg(:) 
  Real (Knd), Allocatable :: AzA(:), WAz(:), VA(:), WVA(:), Wgt(:),  &
       VNode(:), PhiNode(:)
!  Real (Knd), Allocatable :: Vz1(:), Vz2(:), WV1(:), WV2(:)
  REAL (KND) A_MIN
  !==================================================================
  Call ST_Track (R_NAME, 1, 'I')
  
  AzA = MakeGrid(0.0D0, 3.6D2, NAz)
  AzA = AzA/1.8D2*Pie
  WAz = CalcWght(AzA)

  VA = MakeGrid(0.0D0, 1.8D2, NVA)
  VA = Cos(VA/1.8D2*Pie)
  WVA = CalcWght(VA)

  AllocAll = .TRUE.     ! Check which arrays need to be allocated
  IF(ALLOCATED(MJU_NODE)) THEN
     IDX(1:1) = SHAPE(MJU_NODE)
     IF(IDX(1) /= N_NODE) THEN  ! Check if the number of nodes is same as before
        DEALLOCATE(PHI_NODE, MJU_NODE, WEIGHTS, SIN_NODE)
     ELSE
        AllocAll = .FALSE.
     END IF
  END IF

  N_NODE = NVA*NAz
 
  IF(AllocAll) THEN
     ALLOCATE(PHI_NODE(N_NODE), MJU_NODE(N_NODE), VNode(N_NODE),  & ! A
          WEIGHTS(N_NODE), SIN_NODE(N_NODE), PlusFlg(N_NODE), & ! A
          PhiNode(N_NODE), WGT(N_NODE)) ! A
  END IF

  KK = 0
  Do II = 1, NVA
     Do JJ = 1, NAz
        KK = KK + 1
        VNode(KK) = VA(II)
        PhiNode(KK) = Aza(JJ)
        WGT(KK) = WVA(II)*WAz(JJ)
     End Do
  End Do
  WGT(:) = WGT(:)*4.*Pie
  
  PlusFlg = .FALSE.
  WHERE(VNode >= 0.0D0) PlusFlg = .TRUE.
  N_PLUS = COUNT(PlusFlg)
  N_MINUS = N_NODE - N_PLUS

  MJU_NODE(1:N_PLUS) = PACK(VNode,PlusFlg)

  MJU_NODE(N_PLUS + 1:N_NODE) = PACK(VNode,.NOT.PlusFlg)
  PHI_NODE(1:N_PLUS) = PACK(PhiNode,PlusFlg) 
  PHI_NODE(N_PLUS + 1:N_NODE) = PACK(PhiNode,.NOT.PlusFlg)
  WEIGHTS(1:N_PLUS) = PACK(WGT,PlusFlg)
  WEIGHTS(N_PLUS + 1:N_NODE) = PACK(WGT,.NOT.PlusFlg)

  WHERE( PHI_NODE(:) < 0.d0 ) PHI_NODE(:) = PHI_NODE(:) + 2.d0*Pie
  
  SIN_NODE = DSQRT(1.0D0 - MJU_NODE**2)
  NORM_DN = DOT_PRODUCT(WEIGHTS(N_PLUS + 1:N_NODE),&
       MJU_NODE(N_PLUS + 1:N_NODE))
  
  MJU_MINUS => MJU_NODE(N_PLUS + 1:N_NODE)
  MJU_PLUS => MJU_NODE(1:N_PLUS)
  PHI_MINUS => PHI_NODE(N_PLUS + 1:N_NODE)
  W_MINUS => WEIGHTS(N_PLUS + 1:N_NODE)
  W_PLUS => WEIGHTS(1:N_PLUS)
  
! ---- Integration nodes to calculate source function along line-of-sight   ----
  IF( USE_HIN ) THEN
     AzA = MakeGrid(0.0D0, 3.6D2, NAz_HIN)
     AzA = AzA/1.8D2*Pie     ;   WAz = CalcWght(AzA)
     VA = MakeGrid(0.0D0, 1.8D2, NVA_HIN)
     VA = Cos(VA/1.8D2*Pie)  ;   WVA = CalcWght(VA)
!     DEALLOCATE( VA, WVA )
!     Nh = NVA_HIN/2
!     A_MIN = MINVAL( MJU_PLUS(:) )  !; A_MIN = ACOS( A_MIN )/Pie*180.
!     A_MIN = 89.99d0
!     Vz1 = MakeGrid(0.d0, A_MIN, Nh)
!     Vz1 = Cos(Vz1/1.8D2*Pie)       ;  WV1 = CalcWght(Vz1)
!     A_MIN = MAXVAL( MJU_MINUS(:) ) !; A_MIN = ACOS( A_MIN )/Pie*180.
!     A_MIN = 90.01d0
!     Vz2 = MakeGrid(A_MIN, 1.8d2, Nh)  
!     Vz2 = Cos(Vz2/1.8D2*Pie)       ;  WV2 = CalcWght(Vz2)
!     ALLOCATE( VA(NVA_HIN), WVA(NVA_HIN) )
!     VA(1:Nh) = Vz1(:)  ;   VA(1+Nh:NVA_HIN) = Vz2(:)
!     WVA(1:Nh) = WV1(:) ;   WVA(1+Nh:NVA_HIN) = WV2(:)     
     N_HIN = NVA_HIN*NAz_HIN
     IF( .NOT.ALLOCATED(MJU_HIN) ) ALLOCATE(PHI_HIN(N_HIN), &
          MJU_HIN(N_HIN), WEI_HIN(N_HIN), SIN_HIN(N_HIN) ) ! A
     KK = 0
     Do II = 1, NVA_HIN
        Do JJ = 1, NAz_HIN
           KK = KK + 1
           MJU_HIN(KK) = VA(II)
           PHI_HIN(KK) = AzA(JJ)
           WEI_HIN(KK) = WVA(II)*WAz(JJ)
        End Do
     End Do
     SIN_HIN = SQRT(1.0d0 - MJU_HIN**2)
     WEI_HIN(:) = WEI_HIN(:)*4.*Pie
     DEALLOCATE( AzA, WAz, VA, WVA )
  ENDIF
  
  IF( .NOT.USE_AIN ) THEN
     Call ST_Track (R_NAME, 1, 'O') ;  RETURN
  ENDIF
  
! ** Additional angle grid in combination with bidirectional reflectance ****
  Az_A = MakeGrid(0.0d0, 3.6d2, N_Az)
  Az_A = Az_A/1.8D2*Pie       ;  W_Az = CalcWght(Az_A)        
  A_MIN = MAXVAL( MJU_MINUS(:) )
  A_MIN = ACOS( A_MIN )/Pie*180. + 0.1d0
  Vz_A = MakeGrid(A_MIN, 1.8d2, N_VA)
!   Vz_A = MakeGrid(90.0d0, 1.8d2, N_VA)
  Vz_A = Cos(Vz_A/1.8D2*Pie)  ;  W_Vz = CalcWght(Vz_A)
  NbN = N_VA*N_Az
  ALLOCATE( VbN(NbN), PbN(NbN), WbN(NbN) )
  KK = 0
  Do II = 1, N_VA
     Do JJ = 1, N_Az
        KK = KK + 1
        VbN(KK) = Vz_A(II) ;  PbN(KK) = Az_a(JJ)
        WbN(KK) = W_Vz(II)*W_Az(JJ)
     End Do
  End Do
  WbN(:) = WbN(:)*2.*Pie
  NORM_BN = DOT_PRODUCT(WbN(:),VbN(:))
!  DO JJ = 1, 2 !N_Az
!     KK = 1
!     DO II = JJ, NbN, N_Az
!        WRITE(*,'(2(2x,I5),2(2x,F8.3))') JJ, KK, &
!             ACOS( VbN(II) )/Pie*180.,PbN(II)/Pie*180.
!        KK = KK + 1
!     ENDDO
!  ENDDO
! ****************************************************************************
  
  Call ST_Track (R_NAME, 1, 'O')
  Return
Contains
  
! *******************************************************************************
  Function MakeGrid (VS, VE, NN)
    Use Types, Only: Knd
    Integer NN
    Real (Knd) VS, VE 
    Real (Knd), Allocatable :: MakeGrid(:)
    ! Local parameters
    Integer II
    Real (Knd) Step
    !==================================================================
    Step = (VE - VS)/(NN - 1)
    Allocate(MakeGrid(NN))
    MakeGrid(1) = VS ; MakeGrid(NN) = VE
    ForAll(II = 2:NN - 1) MakeGrid(II) = VS + Step*(II - 1)
    Return
  End Function MakeGrid
!**************************************************************************
  Function CalcWght (Grid)
    Use Types, Only: Knd
    Real (Knd) VS, VE 
    Real (Knd), Allocatable :: Grid(:), CalcWght(:)
    ! Local parameters
    Integer II, NN
    Real (Knd) Step
    !==================================================================
    Allocate(CalcWght, Mold=Grid)
    NN = Size(Grid)
    CalcWght(1) = Grid(2) - Grid(1) ; CalcWght(NN) = Grid(NN) - Grid(NN - 1)
    ForAll(II = 2:NN - 1) CalcWght(II) = Grid(II + 1) - Grid(II - 1)
    CalcWght = CalcWght/2.0D0/(Grid(NN) - Grid(1))
    Return
  End Function CalcWght
!**************************************************************************
End Subroutine Nodes_Eqd
!**************************************************************************
