!****************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 07 Mar 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
SUBROUTINE OUTPUT(NSPEC, WL_INP, SRR, SVR)
  ! Called by rtm_main.f

  USE AEROSOLS,    ONLY: AOT_out_VAL,  AOT_out_LOG, N_AOT
  USE ALT_INPUT,   ONLY: IPRT
  USE CLOUDS,      ONLY: DO_CLOUD, CLD_LAYER_HEIGHTS
  USE DATA_RET,    ONLY: MOD_READ_FLAG, RET_FLAG
  USE GAS_CONTROL, ONLY: N_GAS, ST_GAS_IDX, AMF_IDX
  USE GRIDS,       ONLY: NLAYERS, ZZZ, DZ
  USE INFO,        ONLY: WF_DB, TG_INFO
  USE INTERN_DIMS, ONLY: Nvs
  USE IRRAD,       ONLY: SOL_FLUX, SOLSPEC_OUT, EXTR_FLUX, IRR_D => WL_IRRAD_DIM
  USE LOG_PARS,    ONLY: C_UNIT, DIR_OUT, OUT_FOR, IS1, IS2, N_DSO, TO_REFL_FUNC, &
                         OUT_REPR, Mult_WL_Form, ONE_WL, FORMAT_FLAG, LUNIT
  USE RAMAN,       ONLY: N_WL_C, RRS_ADJ
  USE RTM_OUTPUT,  ONLY: SPEC_OUT, FLUX_UP, FLUX_DN, DIF_FLUX_UP, DIF_FLUX_DN, &
                         DIR_RAD, ACTFLUX, TG_VOD, OUT_VOD_ALL, N_OP, DIF_SCAL_UP, &
                         ALB_OUT, DEL_RRS, DEL_VRS, STOK_OUT, VERT_COL, DR_FIN_TOP, &
                         DR_FIN_GT, DR_TAU, OWF_OUT_VEC, NU_VEC_PARS, OWF_LABEL_V, &
                         DIR_FLUX, ALB_WL, WF_UNIT, Tra_mu0, Tra_muv, TRANS_OUT
  USE ST_INPUT,    ONLY: WL_UNITS
  USE ST_VARS,     ONLY: WF_FLAG, RTD => RAD_TO_DEG, DO_RAMAN, VHL, FLUX_MIN_ZZZ, &
                         FLUX_MAX_ZZZ, PIE, RIN_OUT, NEW_SPEC_GRID, ADAPTIVE_GRID, &
                         APPR_RRS, RTM_MODE, RTM_CORE, RTM_TYPE, Enhance_acc, &
                         Vec_Sca, M_PLA, INEL_WHERE, SCATT_MODE, Ellip, NS_wf, Nc_der,& 
                         GeoD => GeoD_OUT, BRDF_FLAG, DIFB
  USE SPHER_PARS,  ONLY: ANG_OUT_INP => ANG_OUT, LOS_DEF_FLAG, NLO => N_LOS_OUT, N_SOL_ANG,&
                         N_LOS_ANG, ANG_SEL_FLAG, SET_OUT_ALT, ALTITUDE, ALT_OUT, &
                         N_AZ_ANG
  USE THERMAL_VAR, ONLY: BRIGHT_TEMP, HC_K
  USE TOOLS,       ONLY: INQ_WF, OpenOut, WriteOut, CloseOut
  USE TYPES,       ONLY: KND, RRS_TYPE, VRS_TYPE, DYN_CHAR
  USE WATER,       ONLY: UWT, NLW => NLAYERS_UW, ZZZ_UW, DZ_UW, OUT_WLR, INEL_ADJ, INS_POSI
  Use VersNum,     Only: VERSION
  USE WF_NUM,      ONLY: NUM_WF, NUM_OWF
 
  INTEGER  NSPEC
  REAL (KND) WL_INP(NSPEC)
  TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
  TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman

  ! Local parameters 
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'OUTPUT'
  CHARACTER (LEN = :), ALLOCATABLE :: VERS, FMT_S1, FMT_S2, FMT_M_S, FMT_M_E, STRING, &
       UNITS, STR, OT_int, FST_LABEL, SND_LABEL, NAM_INT, ALT_DIM, INT_LABEL, P_S, &
       FLUX_LABEL, UNITS_FL, TEXT, WL_LABEL
  Character OutStr*1500, WRITE_FLOAT*20, ADD_STRING*20, WRITE_ALT*20, NAM_AOT*4, N_STR*20
  CHARACTER (LEN = 80)  OT_ell(6), CH_OUT
  CHARACTER (LEN = 1)   SV_COMP(4)
  CHARACTER (LEN = 17)  LABELS(3)
  INTEGER  I, J, J1, IA, K, II, Ik, Jo, IDX_LOS, IDX_PAR, LNL, N_E, N_S, I_1, I_2, &
       KK, STEP, N_OUT, GET_RECL, IDL, IS_1, IS_2, IS_3, RANK_F, RANK_S, JJ, N_STEP, &
       N_SCALE, N_SHIFT, N_LOOP, N_COL, N_LINE, N_COMP, REC_L1, REC_L2, REC_L3, NSto, Unt 
  
  INTEGER  NC_out(4), WI(NSPEC)
  LOGICAL  E_FORMAT, GRIPS
  REAL (KND) DUM, ANG_OUT(3,NLO), WL(NSPEC), CR(NLO), ANG_OUT_TMP(3,NLO), &
       VEW_OUT(NLO), CONV_UNITS(NSPEC), IRR_AD(NSPEC)
  REAL (KND), ALLOCATABLE :: Z_OUT(:), AMF(:), FACTOR(:), FLUX_ARR(:,:,:), NORM(:)
  REAL (KND), PARAMETER :: C = 299792458.0D0, hP = 6.626071D-34 
  REAL (KND), POINTER :: LDZ(:), LZ(:), LoWF(:,:,:,:,:)
  TYPE (DYN_CHAR) AngLabel(3)
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  IA = 1
  IF(TRIM(WL_UNITS) == 'cm-1') THEN
     WI = [(II, II = NSPEC,1,-1)]
     WL_LABEL = 'Wavenumber [cm-1]'
     WL = 1.0D+7/WL_INP
  ELSE
     WL_LABEL = 'Wavelength [nm]'
     WI = [(II, II = 1,NSPEC)]
     WL = WL_INP
  END IF

  AngLabel(1)%S = "Solar zenith angle [deg]"
  AngLabel(2)%S = "Viewing angle [deg]"
  AngLabel(3)%S = "Relative azimuth angle [deg]"
  
!  write(*,*) '   In output.f90 '
!  PAUSE
  
  FMT_M_S = '(1X,F12.5,'
  FMT_M_E = '(2X,ES17.9E3))'
!!      For comparison to Mischenko-1990
! 100  FORMAT(1X,F7.3,6000(2X,F7.4))
!!      For comparison to Garcia_Siewert-1989
! 100  FORMAT(1X,F12.5,6000(2X,E14.6))

  ! Test output total transmissions 
  IF( TRANS_OUT ) THEN
     ! Total downward transmittance:
     WRITE(N_STR,'(I0)') N_SOL_ANG
     FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     OPEN( C_UNIT,File=DIR_OUT//'Trans_Dn.dat', RECL=REC_L1 )
     WRITE(C_UNIT,'(A)') '#   Total transmission: unidirectional illumination from above'
     WRITE(C_UNIT,'(A)') '#   Number of spectral points and number of solar angles'
     WRITE(C_UNIT,'(2(2x,I4))') NSPEC, N_SOL_ANG
     DO I = 1, NSPEC
        WRITE(C_UNIT,FMT=FMT_S1) WL(I),(Tra_mu0(I,J),J=1,N_SOL_ANG)
     ENDDO
     CLOSE( C_UNIT )

     WRITE(N_STR,'(I0)') N_LOS_ANG
     FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     SELECT CASE( TRIM(INS_POSI) )
     CASE('boa')
        CH_OUT = '#   Total transmission: unidirectional illumination from below'
     CASE DEFAULT
        CH_OUT = '#   Total transmission: unidirectional illumination from above'
     END SELECT
     OPEN( C_UNIT,File=DIR_OUT//'Trans_Up.dat', RECL=REC_L1 )
     WRITE(C_UNIT,'(A)') TRIM( CH_OUT )
     WRITE(C_UNIT,'(A)') '#   Number of spectral points and number of zenith angles'
     WRITE(C_UNIT,'(2(2x,I4))') NSPEC, N_LOS_ANG
     DO I = 1, NSPEC
        WRITE(C_UNIT,FMT=FMT_S1) WL(I),(Tra_muv(I,K),K=1,N_LOS_ANG)
     ENDDO
     CLOSE( C_UNIT )
  ENDIF

  IF( ALB_OUT ) THEN
     OPEN( C_UNIT,File = DIR_OUT//'albedo_wl.dat')
     WRITE(C_UNIT,'(A)') '#   Wavelength grid and surface albedo'
     WRITE(C_UNIT,'(A)') '#   Number of spectral points '
     WRITE(C_UNIT,'(2x,I4)') NSPEC
     DO I = 1, NSPEC
        WRITE(C_UNIT,'(1X,F12.5,2x,E12.4)') WL(I), ALB_WL(I)
     ENDDO
     CLOSE( C_UNIT )
  ENDIF

  !------------------------------------------------------------------

  ! Specific output for weighting function in case of correlation spectrometer
  !      GRIPS = .TRUE.
  GRIPS = .FALSE.  ! Does not work if WL_LABEL = 'Wavenumber [cm-1]' !!!

  N_OUT = N_OP
  IF( OUT_WLR ) N_OUT = N_OUT + 1

  ! WRITE(*,*) '   SOL_FLUX in "output.f"   '
  ! DO I = 1, NSPEC
  !    WRITE(*,'(2x,F8.3,2x,E12.4)') WL(I), SOL_FLUX(I)
  ! END DO

  ! Test output for geometrical derivatives with asymptotic algorithm
  IF(TRIM(RTM_CORE) == 'ASYMP') THEN
     FMT_S1 = TRIM(FMT_M_S)//'4'//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     OPEN( C_UNIT, file = DIR_OUT//'deriv_geom.test',RECL=REC_L1)
     
     DUM = ANG_OUT_INP(1,1)
     DO II = 1, NSPEC
        I = WI(II)
        WRITE(C_UNIT,FMT=FMT_S1) WL(I), SPEC_OUT(I,1,1)*DUM, &
             DR_FIN_TOP(I,1)*DUM, DR_FIN_GT(I,1)*DUM, DR_TAU(I,1)*DUM
     END DO
     CLOSE( C_UNIT )
     FORALL( I = 1:NSPEC ) SPEC_OUT(I,1,:) = DABS(ANG_OUT_INP(1,1))*SPEC_OUT(I,1,:)
  ENDIF
  !End of test output

  IF( UWT ) THEN
     ALLOCATE( FLUX_ARR(NSPEC, NLAYERS + NLW, N_SOL_ANG) )
  ELSE
     ALLOCATE( FLUX_ARR(NSPEC, NLAYERS, N_SOL_ANG) )
  ENDIF

  IF( LOS_DEF_FLAG == 'ca' ) THEN
     VEW_OUT(1:NLO) = ANG_OUT_INP(2,1:NLO)
  ELSE
     VEW_OUT(1:NLO) = DACOS(ANG_OUT_INP(2,1:NLO))*RTD
  ENDIF
  
  ANG_OUT(1:2,1:NLO) = DACOS(ANG_OUT_INP(1:2,1:NLO))*RTD
  ANG_OUT(3,1:NLO) = ANG_OUT_INP(3,1:NLO)*RTD

  ANG_OUT_TMP = ANG_OUT
  ANG_OUT_TMP(2,1:NLO) = VEW_OUT(1:NLO)

  SELECT CASE(TRIM(RTM_CORE))
  CASE('ASYMP') 
     ADD_STRING = '(asymptotic'
  CASE('VOD') 
     ADD_STRING = ''
  CASE DEFAULT
     SELECT CASE(TRIM(RTM_TYPE))
     CASE('spher_scat')
        ADD_STRING = '('//TRIM(RTM_CORE)//', spherical mode'
     CASE('ps_scat')
        ADD_STRING = '('//TRIM(RTM_CORE)//', pseudo-spherical mode'
     CASE('pp_scat')
        IF( VHL ) THEN
           ADD_STRING = '('//TRIM(RTM_CORE)//', homogeneous layer mode'
        ELSE
           ADD_STRING = '('//TRIM(RTM_CORE)//', plane-parallel mode'
        ENDIF
     CASE('transmission')
        ADD_STRING = '(transmission'
     CASE DEFAULT
        ADD_STRING = ''
     END SELECT
  END SELECT

  SELECT CASE (Vec_Sca)
  CASE('scal_rte')
     P_S = ', scalar RT'
     OT_int = 'Radiance'
     IS1 = 1 ; IS2 = 1
  CASE('vect_rte')
     P_S = ', vector RT'         
     SV_COMP(:) = (/'I', 'Q', 'U', 'V'/)
     IF( IS1 == IS2 ) THEN
        OT_int = 'Stokes component '//SV_COMP(IS1)
     ELSE
        OT_int = 'Stokes components '//SV_COMP(IS1)//' - '//SV_COMP(IS2)
     ENDIF
     IF( Ellip ) THEN
        OT_ell(1) = 'Ellipticity parameters 1-5: ellipse_par.dat'
        OT_ell(2) = '   1 - degree of polarization'
        OT_ell(3) = '   2 - degree of linear polarization'
        OT_ell(4) = '   3 - degree of circular polarization'
        OT_ell(5) = '   4 - ellipticity'
        OT_ell(6) = '   5 - polarization angle'
     ENDIF
  END SELECT
  IF(ADAPTIVE_GRID) P_S = TRIM(P_S)//', Adaptive Grid'
  IF(ENHANCE_ACC) P_S = TRIM(P_S)//', Adaptive Grid with enhanced accuracy'
  IF(TRIM(RTM_CORE) /= 'DOM') P_S = ''

  VERS = '#  SCIATRAN '//TRIM(VERSION)//' '//TRIM(ADD_STRING)//P_S
  IF(TRIM(RTM_MODE) /= 'vod') VERS = TRIM(VERS)//')'

  ! UNITS = ', '//TRIM(IRR_D(2))//'/'//TRIM(IRR_D(3))//'/'//TRIM(IRR_D(4))//'/sr'
  !
  ! UNITS_FL = ', '//TRIM(IRR_D(2))//'/'//TRIM(IRR_D(3))//'/'//TRIM(IRR_D(4))
  
  UNITS = '['//TRIM(IRR_D(2))//'/'//TRIM(IRR_D(3))//'/'//TRIM(IRR_D(4))//'/sr]'
  UNITS_FL = '['//TRIM(IRR_D(2))//'/'//TRIM(IRR_D(3))//'/'//TRIM(IRR_D(4))//']'

  ! Conversion of Stokes vector/intensity using the array CONV_UNITS
  ! assumed that calculations are performed for extraterrestrial 
  ! solar flux equal to Pi.
  SELECT CASE(EXTR_FLUX)
  CASE('File')
     IF(NEW_SPEC_GRID) THEN
        ! In this case the radiance and weighting functions are  
        ! already multiplied by the extraterrestrial solar flux 
        ! (read from file) before the convolution.
        CONV_UNITS(:) = 1.d0/Pie
        ! Array SOLSPEC_OUT is initialized in "st_convol.f" as
        ! SOLSPEC_OUT( MAX(NSPEC_RTM,NSPEC_OUT) )
        IRR_AD(:) =  SOLSPEC_OUT(1:NSPEC)
     ELSE

        ! FORALL(I = 1:NSPEC) IRR_AD(I) = SOL_FLUX(I*(N_WL_C + 1))
        ! New sequence of wavelengths in WL_EXT:

        SELECT CASE( TRIM(INEL_WHERE) )
        CASE('In air')
           SELECT CASE( TRIM(APPR_RRS) )
           CASE('exact matching')
              FORALL(I = 1:NSPEC) IRR_AD(I) = SOL_FLUX((I-1)*(N_WL_C + 1)+1)
           CASE('simple binning')
              DO I = 1,NSPEC
                 IRR_AD(I) = SOL_FLUX( SRR%A2E(I) ) 
              ENDDO
           END SELECT
        CASE('In water','In air+water')
           IRR_AD(:) = PACK(SOL_FLUX(:),SVR%ACT_WL(:))               
        CASE('Nowhere')
           FORALL(I = 1:NSPEC) IRR_AD(I) = SOL_FLUX(I) 
        END SELECT
        
        CONV_UNITS(:) = IRR_AD(:)/Pie
     END IF
     FLUX_LABEL = 'irradiance.dat'
     
     FMT_S1 = TRIM(FMT_M_S)//'1'//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     Call OpenOut ('irradiance.dat', Unt, RecLen=REC_L1)
     Call WriteOut (Unt, TRIM(VERS))
     Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//', Irradiance '//UNITS_FL)
     DO II = 1, NSPEC
        I = WI(II)
        WRITE(OutStr,FMT=FMT_S1) WL(I), IRR_AD(I)
        Call WriteOut (Unt, OutStr)
     END DO
     Call CloseOut (Unt)
  CASE('Unity')
     CONV_UNITS(:) = 1.d0/Pie
     IRR_AD(:) = 1.d0
     FLUX_LABEL = '1.0 '//TRIM(UNITS_FL)
  CASE('Pi')
     CONV_UNITS(:) = 1.d0
     IRR_AD(:) = Pie 
     FLUX_LABEL = 'Pi (3.1415926536) '//TRIM(UNITS_FL)
  END SELECT
  
  CR(:) = 1.d0
  IF(TO_REFL_FUNC) THEN
     SELECT CASE( GeoD%View(IA)%S )
     CASE('up')   ; INT_LABEL = 'Reflectance'
     CASE('down') ; INT_LABEL = 'Transmittance' 
     CASE('both') ; INT_LABEL = 'Up- and Downward' 
     END SELECT
     INT_LABEL = 'Reflection function'
     CONV_UNITS(:) = CONV_UNITS(:)*Pie/IRR_AD(:)
     WHERE( ANG_OUT_INP(1,:) /= 0.d0 ) CR(:) = 1.d0/DABS(ANG_OUT_INP(1,:))
  ELSE
     IF(TRIM(RTM_TYPE) == 'transmission') THEN
        ! Here we account for the fact that transmission is modelled assuming
        ! the unity extraterrrestrial radiance while for the limb scattering
        ! calculations it is assumed to be equal Pi
        SELECT CASE(EXTR_FLUX)
        CASE('File')
           CONV_UNITS(:) = CONV_UNITS(:)*Pie
           INT_LABEL = 'Transmitted radiance'//' '//TRIM(UNITS)
        CASE('Unity')
           CONV_UNITS(:) = CONV_UNITS(:)*Pie
           INT_LABEL = 'Transmission'
        CASE('Pi')
           INT_LABEL = 'Transmission'
        END SELECT
     ELSE
        INT_LABEL = TRIM(OT_int)//' '//TRIM(UNITS)
     END IF
  END IF
  
  !-----------------------------------------------------
  IF( BRIGHT_TEMP ) THEN
     SELECT CASE( TRIM(IRR_D(2)) ) 
     CASE('W')
        DO I = 1,NSPEC
           DUM = WL_INP(I)*1.D-9
           DO Jo = 1,N_OUT
              STOK_OUT(I,1,Jo,:,:) = HC_K/DUM &
                   /DLOG( 2.D0*C/DUM**4 *hP*C/DUM *M_PLA &
                   /(STOK_OUT(I,1,Jo,:,:)*CONV_UNITS(I)) + 1.D0 )
           ENDDO
        ENDDO
     CASE('phot/s')
        DO I = 1,NSPEC
           DUM = WL_INP(I)*1.D-9
           DO Jo = 1,N_OUT
              STOK_OUT(I,1,Jo,:,:) = HC_K/DUM/DLOG( 2.D0*C/DUM**4*M_PLA &
                   /(STOK_OUT(I,1,Jo,:,:)*CONV_UNITS(I)) + 1.D0 )
           ENDDO
        ENDDO
     END SELECT
     CONV_UNITS(:) = 1.d0
  ENDIF  ! BRIGHT_TEMP

  SELECT CASE (TRIM(RTM_MODE))
  CASE ('int','wf','asymp','int_flux') 
     IF( INEL_ADJ ) THEN
        ! Output for vibrational Raman scattering in water
        ! employing forward-adjoint technique
        CALL OUT_VRS
     ELSE
!        WRITE(*,*) '     output 1: ONE_WL ', ONE_WL
        IF( NSPEC > 1 .OR. AOT_out_LOG ) ONE_WL = 'unchanged'
        IF( TRIM(ANG_SEL_FLAG)=='one' )  ONE_WL = 'unchanged'
!        WRITE(*,*) '     output 2: ONE_WL ', ONE_WL
        ! This routine gives output format of intensity
        ! and Stokes vector
        CALL GET_FMT(1, N_OP, AOT_out_LOG)
        SELECT CASE( N_AOT )
        CASE(0)
           ! Output of standard intensity/Stokes-vector
           CALL WRITE_INTENS(1, 1, N_OP, 'intensity.dat')
!           write(*,*) '  Output of standard intensity/Stokes-vector '
!           PAUSE
        CASE DEFAULT
           ! Multiple aerosol optical thickness (AOT)
           DO Ik = 1, N_AOT+1
              WRITE(NAM_AOT,'(I0)') Ik
              NAM_INT = 'intensity.dat-AOT-'//TRIM(NAM_AOT)
              CALL WRITE_INTENS(Ik, 1, N_OP, NAM_INT)
           ENDDO
        END SELECT

        ! Output of water-leaving radiation 
        IF( OUT_WLR )  CALL WRITE_INTENS(1, N_OUT, N_OUT, 'water_leaving.dat')

        ! Output of ellipsometric parameters
        IF( Ellip ) THEN
           CALL GET_FMT(1, N_OP, .FALSE.)
           CALL WRITE_ELLIPSE(1, N_OP, 'ellipse_par.dat')
        ENDIF
        ! Output of fluxes in the case of option 'int_flux'
        IF( TRIM(RTM_MODE)=='int_flux' ) THEN
!           write(*,*) '   Fluxes output in "int_flux" mode: '
!           WRITE(*,*) '  FLUX_MAX_ZZZ =  ', FLUX_MAX_ZZZ
           N_S = COUNT(ZZZ(:).GT.FLUX_MAX_ZZZ) + 1 ; N_E = COUNT(ZZZ(:).GE.FLUX_MIN_ZZZ)
           FLUX_ARR(:,:,:) = DIR_FLUX(:,:,:)
           CALL WRITE_FLUX('dir_rad.dat', 'Direct solar radiation flux ')     
           FLUX_ARR(:,:,:) = ACTFLUX(:,:,:) + DIR_RAD(:,:,:)
           CALL WRITE_FLUX('act_flux.dat', 'Actinic flux ')     
           FLUX_ARR(:,:,:) = DIF_FLUX_UP(:,:,:)
           CALL WRITE_FLUX('dif_flux_up.dat', 'Upwelling diffuse flux ')
           FLUX_ARR(:,:,:) = DIF_FLUX_DN(:,:,:)
           CALL WRITE_FLUX ('dif_flux_dn.dat', 'Downwelling diffuse flux ')
           FLUX_ARR(:,:,:) = FLUX_UP(:,:,:)
           CALL WRITE_FLUX('total_flux_up.dat', 'Upwelling total flux ')
           FLUX_ARR(:,:,:) = FLUX_DN(:,:,:)
           CALL WRITE_FLUX ('total_flux_dn.dat', 'Downwelling total flux ')
           FLUX_ARR(:,:,:) = DIF_SCAL_UP(:,:,:)
           CALL WRITE_FLUX('dif_scal_up.dat', 'Upwelling scalar flux ')
        ENDIF
        ! Write out output in the form of NDS
        IF( GRIPS ) CALL WRITE_OUT_GRIPS
     ENDIF

  CASE ('amf2','slant_col2')
     ALLOCATE(AMF(NLO),FACTOR(N_SOL_ANG))
     
     WRITE(N_STR,'(I0)') NLO
     FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     
     IF(TRIM(RTM_MODE) == 'amf2') THEN
        OPEN(C_UNIT, FILE = DIR_OUT//'amf.dat', RECL=REC_L1)
        !........................................................
        WRITE(C_UNIT,'(A)') TRIM(VERS)
        WRITE(C_UNIT,'(A)') '#  Contents: Air mass factors for '&
             //TRIM(TG_INFO(AMF_IDX)%Name)
        !........................................................
        FACTOR(:) = 1.d0
     ELSE
        OPEN(C_UNIT, FILE = DIR_OUT//'slant_col.dat',RECL=REC_L1)
        !........................................................
        WRITE(C_UNIT,'(A)') TRIM(VERS)
        WRITE(C_UNIT,'(A)') '#  Contents: Slant columns for '//&
             TRIM(TG_INFO(AMF_IDX)%Name)
        !........................................................
        FACTOR(1:N_SOL_ANG) = VERT_COL(1:N_SOL_ANG)
     END IF

     IF(ANG_SEL_FLAG.EQ.'all') THEN
        DO II = 1, NSPEC
           I = WI(II)  ;   AMF(:) = 0.d0
           DO K = 1, N_SOL_ANG
              I_1 = (K - 1)*N_LOS_ANG*N_AZ_ANG + 1
              I_2 = K*N_LOS_ANG*N_AZ_ANG
              IF(TG_VOD(I,K,1) > 0.d0) AMF(I_1:I_2) = &
                   SPEC_OUT(I,1,I_1:I_2)/TG_VOD(I,K,1)*FACTOR(K)
           END DO
           !....................................................
           WRITE(C_UNIT,FMT=FMT_S1) WL(I), (AMF(K), K = 1, NLO)
           !....................................................
        END DO
     ELSE
        DO II = 1, NSPEC
           I = WI(II)
           AMF(:) = 0.d0
           DO K = 1, NLO
              IF(TG_VOD(I,K,1) > 0.d0) &
                   AMF(K) = SPEC_OUT(I,1,K)/TG_VOD(I,K,1)*FACTOR(K)
           END DO
           !....................................................
           WRITE(C_UNIT,FMT=FMT_S1) WL(I), (AMF(K), K = 1, NLO)
           !....................................................
        END DO
     END IF
     CLOSE(C_UNIT)

     FMT_S1 = '(1X,F7.2,2X,ES17.9E3)'
     REC_L1 = GET_RECL( FMT_S1 )
     OPEN(C_UNIT, FILE=DIR_OUT//'vert_col.dat', RECL=REC_L1)
     WRITE(C_UNIT,'(A)') TRIM(VERS)
     WRITE(C_UNIT,'(A)') '#  Local SZA [deg], vertical colums '//&
          'for '//TRIM(TG_INFO(AMF_IDX)%Name)
     STEP = NLO/N_SOL_ANG
     KK = 0
     DO K = 1, NLO, STEP
        KK = KK + 1
        WRITE(C_UNIT,FMT_S1) ANG_OUT(1,K), VERT_COL(KK)
     END DO
     CLOSE(C_UNIT)
     !-----> End CASE ('amf2','slant_col2')         

  CASE ('flux','spher_alb')
     IF( RTM_MODE=='flux'.AND.(.NOT.DIFB) ) THEN
        N_S = COUNT(ZZZ(:).GT.FLUX_MAX_ZZZ) + 1 ; N_E = COUNT(ZZZ(:).GE.FLUX_MIN_ZZZ)
        FLUX_ARR(:,:,:) = DIR_FLUX(:,:,:)
        CALL WRITE_FLUX('dir_rad.dat', 'Direct solar radiation flux ')     
        FLUX_ARR(:,:,:) = ACTFLUX(:,:,:) + DIR_RAD(:,:,:)
        CALL WRITE_FLUX('act_flux.dat', 'Actinic flux ')     
        FLUX_ARR(:,:,:) = DIF_FLUX_UP(:,:,:)
        CALL WRITE_FLUX('dif_flux_up.dat', 'Upwelling diffuse flux ')
        FLUX_ARR(:,:,:) = DIF_FLUX_DN(:,:,:)
        CALL WRITE_FLUX ('dif_flux_dn.dat', 'Downwelling diffuse flux ')
        FLUX_ARR(:,:,:) = FLUX_UP(:,:,:)
        CALL WRITE_FLUX('total_flux_up.dat', 'Upwelling total flux ')
        FLUX_ARR(:,:,:) = FLUX_DN(:,:,:)
        CALL WRITE_FLUX ('total_flux_dn.dat', 'Downwelling total flux ')
        FLUX_ARR(:,:,:) = DIF_SCAL_UP(:,:,:)
        CALL WRITE_FLUX('dif_scal_up.dat', 'Upwelling scalar flux ')
        !------>  End 'flux'
     ELSE
!       ====> CASE 'spher_alb'
        N_S = COUNT(ZZZ(:).GT.FLUX_MAX_ZZZ) + 1
        N_E = COUNT(ZZZ(:).GE.FLUX_MIN_ZZZ)
        SELECT CASE( TRIM(Mult_WL_Form) )
        CASE('wl_in_line')
           WRITE(N_STR,'(I0)') NSPEC
        CASE('wl_in_colm')
           WRITE(N_STR,'(I0)') N_E - N_S + 1
        END SELECT
        FMT_S1 = '(17x,F12.5,'//TRIM(N_STR)//'(7x,F12.5))'
        FMT_S2 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
        REC_L1 = GET_RECL( FMT_S1 )
        REC_L2 = GET_RECL( FMT_S1 )
        REC_L3 = MAX0(REC_L1, REC_L2)
        IF( DIFB ) THEN
!       ... In the case of diffuse (isotropic) illumination from below
!           EXTR_FLUX is set to 'Unity' and CONV_UNITS = 1/Pi, because
!           the incoming intensity is set to 1 (see UDB_A%Flu_int in
!           "dom_spherical.f") and incoming flux from below is equal 
!            to Pi.  
           FORALL(J = 1:NLAYERS) &
                FLUX_ARR(:,J,1) = FLUX_DN(:,J,1)*CONV_UNITS(:)
           CALL WRITE_SPHER_ALB ('refl_iso_ill_below.dat', &
                '#  Reflectance under isotropic illumination from below')     
           FORALL(J = 1:NLAYERS) &
                FLUX_ARR(:,J,1) = FLUX_UP(:,J,1)*CONV_UNITS(:)
           CALL WRITE_SPHER_ALB ('trans_iso_ill_below.dat', &
                '#  Transmittance under isotropic illumination from below')
        ELSE
           FLUX_ARR(:,:,1) = FLUX_UP(:,:,1)
           CALL WRITE_SPHER_ALB ('refl_iso_ill_above.dat', &
                '#  Reflectance under isotropic illumination from above')     
           FLUX_ARR(:,:,1) = FLUX_DN(:,:,1)
           CALL WRITE_SPHER_ALB ('trans_iso_ill_above.dat', &
                '#  Transmittance under isotropic illumination from above')
        ENDIF
     ENDIF !------> End ('spher_alb')
     
  CASE ('vod')
     WRITE(N_STR,'(I0)') N_SOL_ANG*N_GAS
     FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     OPEN(C_UNIT, FILE=DIR_OUT//'tg_vod.dat', RECL=REC_L1)
     WRITE(C_UNIT,'(A)') TRIM(VERS)
     WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//&
          ', Trace gas vertical optical depth'
     DO II = 1, NSPEC
        I = WI(II)
        WRITE(C_UNIT,FMT=FMT_S1) WL(I), ((TG_VOD(I, IDX_LOS, K), &
             IDX_LOS = 1, N_SOL_ANG), K = 1, N_GAS)
     END DO
     CLOSE(C_UNIT)
     IF(TRIM(RTM_CORE) /= 'CDI') THEN
        FMT_S1 = TRIM(FMT_M_S)//'6'//TRIM(FMT_M_E)
        REC_L1 = GET_RECL( FMT_S1 )
        OPEN(C_UNIT, FILE=DIR_OUT//'vod_all.dat', RECL=REC_L1)
        WRITE(C_UNIT,'(A)') TRIM(VERS)
        WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//' Vertical optical '//&
             'depth: Rayleigh, aerosol scattering/absorption,  '
        WRITE(C_UNIT,'(A)') '#  cloud scattering/absorption,'&
             //' and total optical depth (including gases)'
        DO II = 1, NSPEC
           I = WI(II)
           WRITE(C_UNIT,FMT_S1) WL(I), (OUT_VOD_ALL(I,K), K = 1,6)
        END DO
        CLOSE(C_UNIT)
     END IF
     !------> End CASE ('vod')
  CASE ('raman')
     CALL OUT_RRS
  END SELECT

  NSto = 1
  Call OpenOut ('output_map.inf', Unt)

  Call WriteOut(Unt,TRIM(VERS))
  IF(EXTR_FLUX == 'File') Call WriteOut (Unt,'#  Irradiance: irradiance.dat')

  SELECT CASE (RTM_MODE)
  CASE DEFAULT ! all except for 'vod', 'flux', 'spher_alb'
     ALLOCATE (Z_OUT(N_OP))
     TEXT='TOA'
     IF(SET_OUT_ALT) THEN
        Z_OUT(:) = ALTITUDE(:)
        IF( ALTITUDE(1).LT.ZZZ(1) ) TEXT='Observer position'
     ELSE
        Z_OUT(:) = ALT_OUT(:)
     END IF
     
     SELECT CASE (TRIM(RTM_MODE))
     CASE ('int') 
        NSto = IS2 - IS1 + 1
        NC_out(:) = [(I, I = 1,4)] 
        Str = '#  '//OT_int//': intensity.dat'
        IF( OUT_WLR ) Str = Str//',  water_leaving.dat'
        Call WriteOut (Unt, Str)
        IF( Ellip ) THEN
           DO I = 1,6
              Call WriteOut (Unt, '#  '//TRIM(OT_ell(I)))
           ENDDO
        ENDIF
     CASE('num_pert')
        NSto = IS2 - IS1 + 1
        DO I = 1,4 
           NC_out(I) = I 
        ENDDO
        IDX_PAR = 1 ;  STR = ' '//'wf_numeric.dat'
        Call WriteOut (Unt,'#  Weighting functions: '//STR)
        
     CASE ('wf')
        NSto = NS_wf
        NC_out(1:NSto) = NC_der(:)
        Call WriteOut (Unt, '#  '//OT_int//': intensity.dat') 
        IF( UBOUND(WF_DB,1)== 0 ) STR = ' '//'none'
        DO II = 1, UBOUND(WF_DB,1)
           SELECT CASE (WF_DB(II)%NAME)
           CASE ('anps')
              STR = ' '//TRIM(WF_DB(II)%NAME)//'.dat'
           CASE DEFAULT
              STR = ' '//'wf_'//TRIM(WF_DB(II)%NAME)//'.dat'
           END SELECT
        END DO
        Call WriteOut (Unt, '#  Weighting functions: '//STR)
        STRING = 'I-1; Q-2; U-3; V-4'
        Call WriteOut (Unt,'#  Stokes components (SC): '//STR)
        
     CASE ('amf2') 
        NC_out(:) = 1
        Call WriteOut (Unt,'#  Air mass factors: amf.dat')
        Call WriteOut (Unt,'#  Vertical colums: vert_col.dat')
     CASE ('slant_col2') 
        NC_out(:) = 1
        Call WriteOut (Unt,'#  Slant columns: slant_col.dat')
        Call WriteOut (Unt,'#  Vertical colums: vert_col.dat')
        
     CASE ('block_amf') 
        NSto = NS_wf
        NC_out(1:NSto) = NC_der(:)
        Call WriteOut (Unt,'#  Height resolved air mass factors: block_amf.dat')
        
     CASE ('raman') 
        ! ... Correction to include water-leaving output (VR 03.12.23)
        IF( OUT_WLR )  CALL WRITE_INTENS(1, N_OUT, N_OUT, 'water_leaving.dat')
        NSto = IS2 - IS1 + 1
        NC_out(:) = [(I, I = 1,4)] 
        Call WriteOut (Unt,'#  Radiance: intensity.dat')
        IF( RRS_ADJ ) THEN
           Call WriteOut (Unt,'#  Radiance without raman '//&
                'scattering: intensity_noring.dat')
           Call WriteOut (Unt,'#  Ring spectrum : ring.dat')
        ELSE
           Call WriteOut (Unt,'#  Radiance without raman '//&
                'scattering: is not calculated')
           Call WriteOut (Unt,'#  Ring spectrum : is not calculated')
        ENDIF
     END SELECT
     
     NC_out(1) = 1
     
     IF(TRIM(OUT_REPR) == 'default') THEN
        CALL Write_Angles (Unt, NSto, NC_out)
     ELSE
        STR = WRITE_FLOAT(WL(WI(1)), 2)
        IF( ALTITUDE(1) >= 0.d0 ) THEN
           ALT_DIM = 'km'
        ELSE
           ALT_DIM = ' m'
        ENDIF
        
        WRITE(OutStr,'(a,13x,a)') '#  Wavelength',': '//STR//' nm'
        Call WriteOut (Unt, OutStr)

        ! Number of output altitudes
        IF(N_OP == 1) THEN
           STR = WRITE_ALT(DABS(ALTITUDE(1)), 2)
           WRITE(OutStr,'(a,8x,a)') '#  Output altitude',': '//STR//' '//ALT_DIM
           Call WriteOut (Unt, OutStr)
        ENDIF

        LABELS =(/'Solar zenith','Viewing     ', 'Azimuth     '/)

        II = 6 - RANK_S - RANK_F
        WRITE(OutStr,'(a,5x,a,F7.2,a)') '#  '//TRIM(LABELS(II)) &
             //' angle',':',ANG_OUT(II,1),' deg'
        Call WriteOut (Unt, OutStr)

        Call WriteOut (Unt, '#  Columns content in "intensity.dat": ')
        Call WriteOut (Unt, '#  First column       -> '//&
             TRIM(LABELS(RANK_F))//' angle ')
        Call WriteOut (Unt, '#  Subsequent columns -> '//&
             'Stokes components as listed below')
        
        STR = TRIM(LABELS(RANK_S))
        STR(1:1) = CHAR(IACHAR(STR(1:1)) + 32)
        STRING = '# '//' Column number, Stokes component, '//STR//' angle'

        IF(N_OP > 1) STRING = STRING//', output altitude'//' ('//ALT_DIM//')'
        Call WriteOut (Unt,STRING)

        N_COL = 2
        DO I = 1,N_OP
           I_1 =  1 
           I_2 = I_1 + (N_STEP - 1)*N_SHIFT
           DO K = I_1,I_2,N_SHIFT
              DO J = IS1,IS2
                 IF( N_OP == 1 ) THEN
                    WRITE(OutStr,'(1x,I6,4x,I0,2x,F8.3 )') &
                         N_COL, J, ANG_OUT(RANK_S,K)
                 ELSE
                    STR = WRITE_ALT(DABS(ALTITUDE(I)), 2)
                    WRITE(OutStr,'(1x,I6,4x,I0,2x,F8.3,4x,a)') &
                         N_COL, J, ANG_OUT(RANK_S,K), TRIM(STR)
                 ENDIF
                 Call WriteOut (Unt, OutStr)
                 N_COL = N_COL + 1
              ENDDO
           ENDDO
        ENDDO        
     END IF              ! if TRIM(OUT_REPR) /= 'default'

  CASE ('flux', 'spher_alb')
     IF( RTM_MODE=='flux'.AND.(.NOT.DIFB) ) THEN
        Call WriteOut (Unt, '#  Fluxes: dir_rad.dat, act_flux.dat, '//&
             'dif_flux_up.dat dif_flux_dn.dat')
        Call WriteOut (Unt, '#          dif_scal_up.dat '//&
             'total_flux_up.dat total_flux_dn.dat')
        Call WriteOut (Unt, '#  Num., solar zenith angle, altitude')
        K = 0
        DO IDX_LOS = 1, NLO
           ! Write altitudes for atmospheric fluxes 
           DO J = N_S, N_E
              K = K + 1
              WRITE(OutStr,'(I4,2X,F6.2,2X,F10.4)') K, ANG_OUT(1,IDX_LOS), ZZZ(J)
              Call WriteOut (Unt, OutStr)
           END DO
           ! Write depths for underwater fluxes 
           IF( UWT ) THEN
              DO J1 = 1, NLW
                 K = K + 1
                 WRITE(OutStr,'(I4,2X,F6.2,2X,F10.4)') K, ANG_OUT(1,IDX_LOS), -ZZZ_UW(J1)
                 Call WriteOut (Unt, OutStr)
              ENDDO
           ENDIF
        END DO
     ELSE
        IF( DIFB ) THEN
           Call WriteOut (Unt, &
                '#  Reflectance (illumination from below): refl_iso_ill_below.dat ')
           Call WriteOut (Unt, &
                '#  Transmittance (illumination from below): trans_iso_ill_below.dat')
        ELSE
           Call WriteOut (Unt, &
                '#  Reflectance (illumination from above): refl_iso_ill_above.dat ')
           Call WriteOut (Unt, &
                '#  Transmittance (illumination from above): trans_iso_ill_above.dat')
        ENDIF
        Call WriteOut (Unt, '#  Num.,  altitude')
        K = 0
        DO IDX_LOS = 1, NLO        
           DO J = N_S, N_E  ! Write altitudes for atmospheric values
              K = K + 1
              WRITE(OutStr,'(I4,2(2X,F10.4))')  K, ZZZ(J)
              Call WriteOut (Unt, OutStr)
           END DO
           ! Write depths for underwater values
           IF( UWT ) THEN
              DO J1 = 1, NLW  ;  K = K + 1
                 WRITE(OutStr,'(I4,2(2X,F10.4))') K, -ZZZ_UW(J1)
                 Call WriteOut (Unt, OutStr)
              ENDDO
           ENDIF
        END DO
     ENDIF

  CASE ('vod')
     Call WriteOut (Unt, '#  Trace gas vertical optical depth: tg_vod.dat')
     Call WriteOut (Unt, '#  Vertical optical depth for '//&
          'Rayleigh, aerosol, clouds: vod_all.dat') 
     Call WriteOut (Unt, '#              (additional information in file headers)')
     Call WriteOut (Unt, '#  Num., solar zenith angle, trace gas label')
     J = 0
     DO K = 1, N_GAS
        DO IDX_LOS = 1, NLO
           J = J + 1
           WRITE(OutStr,'(I4,2X,F7.2,2x,A4)') J, &
                ANG_OUT(1,IDX_LOS), TG_INFO(ST_GAS_IDX(K))%Name
           Call WriteOut (Unt, OutStr)
        END DO
     END DO
     
  END SELECT
  Call CloseOut (Unt)

  E_FORMAT = .FALSE.
  IF( DO_CLOUD ) THEN
     ! Top height of the lowest cloud layer smaller than 50 m
     IF( CLD_LAYER_HEIGHTS(1,2) < 0.05d0 ) E_FORMAT = .TRUE.
  ENDIF

  ! Output section of the weighting function obtained
  ! using numerical perturbation approach
  ! ---------------------------------------------------------------
  IF( NUM_WF ) THEN
     IF( NUM_OWF ) THEN
        LNL = NLW
        ALLOCATE( NORM(LNL) )  ; LDZ => DZ_UW(:) ; LZ => ZZZ_UW(:)
     ELSE
        LNL = NLAYERS
        ALLOCATE( NORM(LNL) )  ; LDZ => DZ(:) ; LZ => ZZZ(:)
     ENDIF
     NORM(:) = 1.d0 

     IF( TRIM(WF_UNIT)== 'unified' )  NORM(:) = LDZ(:)

     WRITE(N_STR,'(I0)') LNL
     IF( E_FORMAT ) THEN
        FMT_S1 = '('//TRIM(N_STR)//'(2x,E15.7))'
     ELSE
        FMT_S1 = '('//TRIM(N_STR)//'(2x,f12.6))'
     ENDIF
     WRITE(N_STR,'(I0)') (IS2 - IS1 + 1)*NLO
     FMT_S2 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     
     REC_L1 = GET_RECL( FMT_S1 )
     REC_L2 = GET_RECL( FMT_S2 )
     REC_L3 = MAX0(REC_L1, REC_L2)
     
     OPEN(C_UNIT, FILE=DIR_OUT//'wf_numeric.dat', STATUS = 'unknown', RECL=REC_L3)
     WRITE(C_UNIT,'(A)') TRIM(VERS)
     WRITE(C_UNIT,'(A)') '#  Weighting functions '//TRIM(UNITS) 
     WRITE(C_UNIT, '("#  Spectral range: ", f8.3, 1x,"-", 1x,f8.3,2x,A) ')&
          WL(1), WL(NSPEC), TRIM(WL_UNITS)
     WRITE(C_UNIT,'("#  The number of spectral points: ", I0)') NSPEC
     WRITE(C_UNIT,'("#  The number of altitude levels: ", I0)') LNL
     WRITE(C_UNIT,FMT=FMT_S1) LZ(1:LNL)
     
     IF( NUM_OWF ) THEN
        DO J = 1, LNL
           WRITE(C_UNIT,*)
           DO II = 1, NSPEC
              I = WI(II)
              WRITE(C_UNIT,FMT=FMT_S2) WL(I), ( LoWF(I,IDL,J,1,IS1:IS2)/NORM(J)&
                   *CONV_UNITS(I)*CR(IDL), IDL=1,NLO)
           END DO
        END DO
     ELSE
        DO J = 1, LNL
           WRITE(C_UNIT,*)
           DO II = 1, NSPEC
              I = WI(II)
              WRITE(C_UNIT,FMT=FMT_S2) WL(I), ( WF_DB(1)%WF(I,IDL,J,IS1:IS2)/NORM(J)&
                   *CONV_UNITS(I)*CR(IDL), IDL=1,NLO)
           END DO
        END DO
     END IF
     CLOSE(C_UNIT)
  ENDIF
  !------------------------------------------------------------------

  ! If WF for ocean parameters required (NU_VEC_PARS=0 if not)
  DO IDX_PAR = 1, NU_VEC_PARS
     WRITE(N_STR,'(I0)') NLW
     IF( E_FORMAT ) THEN
        FMT_S1 = '('//TRIM(N_STR)//'(2x,E15.7))'
     ELSE
        FMT_S1 = '('//TRIM(N_STR)//'(2x,f12.6))'
     ENDIF
     WRITE(N_STR,'(I0)') NLO
     FMT_S2 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
     REC_L1 = GET_RECL( FMT_S1 )
     REC_L2 = GET_RECL( FMT_S2 )
     REC_L3 = MAX0(REC_L1, REC_L2)
     
     OPEN(C_UNIT, FILE=DIR_OUT//'wf_'//TRIM(OWF_LABEL_V(IDX_PAR))// &
          '.dat', STATUS = 'unknown', RECL=REC_L3)
     WRITE(C_UNIT,'(A)') TRIM(VERS)
     WRITE(C_UNIT,'(A)') '#  Weighting functions '//TRIM(UNITS) 
     WRITE(C_UNIT, '("#  Spectral range: ", f8.3, 1x,"-", 1x,f8.3,2x,A) ')&
          WL(1), WL(NSPEC), TRIM(WL_UNITS)
     WRITE(C_UNIT,'("#  The number of spectral points: ", I0)') NSPEC
     WRITE(C_UNIT,'("#  The number of altitude levels: ", I0)') NLW
     IF( E_FORMAT ) THEN
        WRITE(C_UNIT,FMT=FMT_S1) ZZZ_UW(1:NLW)
     ELSE
        WRITE(C_UNIT,FMT=FMT_S1) ZZZ_UW(1:NLW)
     ENDIF
     DO J = 1, NLW
        WRITE(C_UNIT,*)
        DO II = 1, NSPEC
           I = WI(II)
           WRITE(22,FMT=FMT_S2) WL(I), (OWF_OUT_VEC(I,IDL,J,IDX_PAR,1)&
                *CONV_UNITS(I)*CR(IDL), IDL=1,NLO)
        END DO
     END DO
     CLOSE(C_UNIT)
  ENDDO                     ! over WF for ocean parameters (IDX_PAR)

  IF( .NOT.WF_FLAG ) THEN 
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  ENDIF

  IF(TRIM(RTM_MODE) == 'block_amf') THEN
     CALL WRITE_B_AMF
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  END IF
  
  IF( GRIPS ) THEN
     CALL WRITE_OUT_GRIPS
  ELSE
     CALL WRITE_WF_OUT
  ENDIF
  
  IF( .NOT.RET_FLAG .OR. .NOT.MOD_READ_FLAG ) THEN 
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  ENDIF

  ! Unformatted output
  If(.False.) Then
     OPEN(C_UNIT, FILE=DIR_OUT//'intensity_unformatted.dat',  &
          STATUS = 'unknown', FORM='UNFORMATTED')
     WRITE(C_UNIT) SPEC_OUT
     CLOSE(C_UNIT)
     
     OPEN(C_UNIT, FILE=DIR_OUT//'profiles_unformatted.dat', & 
          STATUS = 'unknown', FORM='UNFORMATTED')
     DO II = 1, UBOUND(TG_INFO,1)
        IF(.NOT.TG_INFO(II)%FM_FLG) CYCLE
        WRITE(C_UNIT) TG_INFO(II)%ND
     END DO
     CLOSE(C_UNIT)
         
     DO IDX_PAR = 1, UBOUND(WF_DB,1)
        OPEN(C_UNIT, FILE=DIR_OUT//'wf_'//TRIM(WF_DB(IDX_PAR)%NAME)&
             //'_unformatted.dat', STATUS='unknown', FORM='UNFORMATTED')
        WRITE(C_UNIT) WF_DB(IDX_PAR)%WF(1:NSPEC,:,:,1)
        CLOSE(C_UNIT)
     END DO
  End If

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
CONTAINS
  !***********************************************************************
  !            Output format specification
  !***********************************************************************
  SUBROUTINE GET_FMT(N1, N2, FLAG_AOT)
    
    INTEGER, INTENT(IN) :: N1, N2
    LOGICAL, INTENT(IN) :: FLAG_AOT
    ! Local parameters
    CHARACTER (LEN=12), PARAMETER:: R_NAME='GET_FMT'
    INTEGER  N_max, NO_PAR
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    ! Number of observation geometries
    N_max  = N2 - N1 + 1
    NO_PAR = IS2 - IS1 + 1

    ! Character OUT_FOR is defined in 'gt_filread.f' as
    ! 'default' or user required format operator
    FMT_S1 = OUT_FOR

    IF(TRIM(ONE_WL) == 'unchanged') THEN 
       ! Always if NSPEC > 1
       SELECT CASE (TRIM(Mult_WL_Form))
       CASE('wl_in_colm')
          WRITE(N_STR,'(I0)') NLO*NO_PAR*N_max
          IF(FORMAT_FLAG == 'default') &
               FMT_S1 = '(1X,F12.5,'//TRIM(N_STR)//'(2X,ES17.9E3))'

       CASE('wl_in_line')
          WRITE(N_STR,'(I0)') NSPEC*NO_PAR*N_max
          IF( FLAG_AOT ) THEN
             FMT_S1 = '(3(1X,F7.2),1x,e12.4,'//TRIM(N_STR)//'(2X,ES17.9E3))'
          ELSE
             IF(FORMAT_FLAG == 'default') FMT_S1 = &
                  '(3(1X,F7.2),'//TRIM(N_STR)//'(2X,ES17.9E3))'
          ENDIF
       END SELECT
    ELSE                      ! If TRIM(ONE_WL) /= 'unchanged'
       !****** Intensity output for a single wavelength ***********
       !----------------------------------------
       IS_1 = INDEX(OUT_REPR,'|', .FALSE.)
       IS_2 = INDEX(OUT_REPR,TRIM('\ '), .FALSE.)
       IS_3 = INDEX(OUT_REPR,'|', .TRUE.)
       FST_LABEL = OUT_REPR(IS_1 + 1:IS_2 - 1)
       SND_LABEL = OUT_REPR(IS_2 + 1:IS_3 - 1)
       
       SELECT CASE (TRIM(FST_LABEL))
       CASE ('SZA')
          N_LOOP = N_SOL_ANG ;  RANK_F = 1
       CASE ('Zen')
          N_LOOP = N_LOS_ANG ;  RANK_F = 2
       CASE ('Azm')
          N_LOOP = N_AZ_ANG ;   RANK_F = 3
       END SELECT
       SELECT CASE (TRIM(SND_LABEL))
       CASE ('SZA')
          N_STEP = N_SOL_ANG ;  RANK_S = 1
       CASE ('Zen')
          N_STEP = N_LOS_ANG ;  RANK_S = 2
       CASE ('Azm')
          N_STEP = N_AZ_ANG  ;  RANK_S = 3
       END SELECT
       IF(RANK_F < RANK_S) THEN
          N_SCALE = N_STEP ; N_SHIFT = 1
       ELSE
          N_SCALE = 1 ; N_SHIFT = N_LOOP
       END IF
       
       IF(FORMAT_FLAG == 'default') THEN
          WRITE(N_STR,'(I0)') N_STEP*NO_PAR*N_max
          FMT_S1 = '(1X,F7.2,'//TRIM(N_STR)//'(2X,ES17.9E3))'
       END IF
       
    ENDIF
    CALL ST_TRACK (R_NAME, 1, 'O')
    
  END SUBROUTINE GET_FMT

  !***************************************************************************
  SUBROUTINE WRITE_OUT_GRIPS

    INTEGER N_PRE, NWL
    CHARACTER (LEN = 100) R_dir, GABS
    CHARACTER (LEN = 100) R_name
    LOGICAL     USE_O2
    REAL (KND)  Itv, TM1, B0, ITOA(NSPEC)
    REAL (KND), ALLOCATABLE:: RWL(:), Tv(:), Tva(:), Tvb(:), DLAM(:),&
         Tg(:,:), Wnds(:,:), W_scp(:), b_co(:), MDS(:)

    ! IF( N_VEC_OUT /= 1 ) THEN
    !    IF( .NOT.INQ_WF('aer', 'WFT') ) THEN
    !       WRITE( *,* )  '*** GRIPS output can be used '//&
    !           'for single vector parameter only ***'
    !    ENDIF
    ! ENDIF

    ! USE_O2 = .FALSE.
      
    !********************** O2 0.76 micron ******************
    USE_O2 = .TRUE.
    R_dir  = '/home/home/rozanov/GRIPS/Respone_func/O2/'
    R_name = 'Response-O2_4filter.dat'
    GABS   = 'OUT-O2'

    !********************** CO ******************************
    ! R_dir  = '/home/home/rozanov/GRIPS/Respone_func/CO_2.35/'
    ! R_name = 'Resp_func-CO_2.35mkm.out'
    ! GABS   = 'OUT'

    !********************** CO2 ******************************
    ! R_dir  = '/home/home/rozanov/GRIPS/Respone_func/CO2_13/'
    ! R_name = 'Resp_func-CO2_13.out'
    ! GABS   = 'OUT'
    
    !********************** CH4 2.28 micron ******************
    ! R_dir  = '/home/home/rozanov/GRIPS/Respone_func/CH4_2.28/'
    ! R_name = 'Resp_func-CH4_2.28mkm.out'
    ! GABS   = 'OUT'
    
    !********************** CH4 1.66 micron ******************
    ! R_dir  = '/home/home/rozanov/GRIPS/Respone_func/CH4_1.66/'
    ! R_name = 'Resp_func-CH4_1.66mkm.out'
    ! GABS   = 'CH4_1.66'
    
    !******************************************************************
    ! Reading response functions
    !******************************************************************
    IF( USE_O2 ) THEN
       OPEN( C_UNIT,FILE=TRIM(R_dir)//TRIM(R_name),STATUS='OLD' )
       CALL SKIP_HEADER (C_UNIT)
       READ(C_UNIT,*)  NWL
       N_PRE = 2
       ALLOCATE( Tv(NWL), Tva(NWL), Tvb(NWL), RWL(NWL), Tg(NWL,N_PRE) )
       DO I = 1, NWL
          READ(C_UNIT,*) RWL(I), Tva(I), Tg(I,1), Tg(I,2), Tvb(I)
       ENDDO
       CLOSE( C_UNIT )
       ! Check the first and last wavelengths
       WRITE(*,*) '   First wavelength [nm] '
       WRITE(*,*) WL(1)
       WRITE(*,*) RWL(1)
       WRITE(*,*) '   Last wavelength [nm] '
       WRITE(*,*) WL(NSPEC)
       WRITE(*,*) RWL(NSPEC)
    ELSE
       OPEN( C_UNIT,FILE=TRIM(R_dir)//TRIM(R_name),STATUS='OLD' )
       CALL SKIP_HEADER (C_UNIT)
       READ(C_UNIT,*)  NWL, N_PRE 
       WRITE(*,*) '  NWL after read:  ', NWL
       ALLOCATE( RWL(NWL), Tv(NWL), Tg(NWL,N_PRE) )
       DO I = 1, NWL
          READ(C_UNIT,*) TM1, RWL(I), Tv(I), (Tg(I,J), J=1,N_PRE)
       ENDDO
       CLOSE( C_UNIT )
       ! Check the first and last wavelengths
       WRITE(*,*) '   First wavelength [nm] '
       WRITE(*,*) WL(1)
       WRITE(*,*) RWL(NSPEC)
       WRITE(*,*) '   Last wavelength [nm] '
       WRITE(*,*) WL(NSPEC)
       WRITE(*,*) RWL(1)
    ENDIF

    ALLOCATE(DLAM(NWL), b_co(N_PRE), Wnds(N_PRE,NLAYERS), MDS(N_PRE))

    ! Check dimension compatibility
    IF( NWL /= NSPEC ) THEN
       WRITE(*,*) '  Different number of wavelengths !!! '
       WRITE(*,*) NWL, NSPEC
       STOP
    ENDIF

    ! Calculate auxiliary functions 
    DLAM(1) = WL(2) - WL(1)
    DO I = 2, NSPEC-1
       DLAM(I) = WL(I+1) - WL(I-1)
    ENDDO
    DLAM(NSPEC) = WL(NSPEC) - WL(NSPEC-1)
      
    ! ******************************************************************
    ! ... Perform calculations of NDS and weighting functions
    ! ******************************************************************
    IF( USE_O2 ) THEN
       ITOA(:) = IRR_AD(:)*DLAM(:)
       B0 = SUM( ITOA(:)*(Tva(:)+Tvb(:)) )/2.d0 
       WRITE(*,*) '   Test output for O2 balance factor :'
       DO J = 1,N_PRE
          b_co(J) = B0/SUM( ITOA(:)*Tg(:,J) )
          WRITE(*,'(2x,I3,2x,E12.4)') J, b_co(J)
       ENDDO
       ! Select intensity at TOA
       IDL = 1 ; Jo = 1 
       ITOA(:) = STOK_OUT(:,1,Jo,IDL,1)*CONV_UNITS(:)*CR(IDL)*DLAM(:)
       ! Calculate MDS = NDS + 1
       Itv = SUM( ITOA(:)*(Tva(:)+Tvb(:)) )/2.d0
       DO J = 1,N_PRE
          MDS(J) = b_co(J)/Itv*SUM( ITOA(:)*Tg(:,J) )
       ENDDO
       ! Output of calculated NDS on the case of O2
       OPEN( C_UNIT,FILE=DIR_OUT//'NDS-'//TRIM(GABS)//'.dat' )
       DO J = 1, N_PRE
          WRITE(C_UNIT,'(1x,I3,1x,e15.7)') J, MDS(J) - 1.
       ENDDO
       CLOSE( C_UNIT )
       
       IF( .NOT.WF_FLAG ) RETURN
       
       ! Calculate NDS weighting function for first vector parameter
       IDX_PAR = 1
       DO J = 1, NLAYERS
          ITOA(:) = WF_DB(IDX_PAR)%WF(:,IDL,J,1)*CONV_UNITS(:)*CR(IDL)*DLAM(:)
          TM1 = SUM( ITOA(:)*(Tva(:)+Tvb(:)) )/2.d0
          DO K = 1, N_PRE
             Wnds(K,J) = ( b_co(K)*SUM( ITOA(:)*Tg(:,K) ) - MDS(K)*TM1 )/Itv
          ENDDO
       ENDDO
       OPEN( C_UNIT,FILE=DIR_OUT//'WF_NDS-'//TRIM(GABS)//'.dat' )
       DO I = 1, NLAYERS
          WRITE(C_UNIT,'(1x,F7.3,10(1x,e15.7))') ZZZ(I), (Wnds(K,I), K=1,N_PRE)
       ENDDO
       CLOSE( C_UNIT )
       
    ELSE
       ITOA(:) = IRR_AD(:)*DLAM(:)
       B0 = SUM( ITOA(:)*Tv(NWL:1:-1) )
       WRITE(*,*) '   Test output for balance factor :'
       DO J = 1,N_PRE
          b_co(J) = B0/SUM( ITOA(:)*Tg(NWL:1:-1,J) )
          WRITE(*,'(2x,I3,2x,E12.4)') J, b_co(J)
       ENDDO
       ! Select intensity at TOA
       IDL = 1 ; Jo = 1 
       ITOA(:) = STOK_OUT(:,1,Jo,IDL,1)*CONV_UNITS(:)*CR(IDL)*DLAM(:)
       ! Calculate MDS = NDS + 1
       Itv = SUM( ITOA(:)*Tv(NWL:1:-1) )
       DO J = 1,N_PRE
          MDS(J) = b_co(J)/Itv*SUM( ITOA(:)*Tg(NWL:1:-1,J) )
       ENDDO
       ! Output of calculated NDS
       OPEN( C_UNIT,FILE=DIR_OUT//'NDS-'//TRIM(GABS)//'.dat' )
       DO J = 1, N_PRE
          WRITE(C_UNIT,'(1x,I3,1x,e15.7)') J, MDS(J) - 1.
       ENDDO
       CLOSE( C_UNIT )

       IF( .NOT.WF_FLAG ) RETURN
      
       ! Calculate NDS weighting function for first vector parameter
       IDX_PAR = 1
       DO J = 1, NLAYERS
          ITOA(:) = WF_DB(IDX_PAR)%WF(:,IDL,J,1)*CONV_UNITS(:)*CR(IDL)*DLAM(:)
          TM1 = SUM( ITOA(:)*Tv(NWL:1:-1) )
          DO K = 1, N_PRE
             Wnds(K,J) = ( b_co(K)*SUM(ITOA(:)*Tg(NWL:1:-1,K)) - MDS(K)*TM1 )/Itv
          ENDDO
       ENDDO
       OPEN( C_UNIT,FILE=DIR_OUT//'WF_NDS-'//TRIM(GABS)//'.dat' )
       DO I = 1, NLAYERS
          WRITE(C_UNIT,'(1x,F7.3,10(1x,e15.7))')  ZZZ(I), (Wnds(K,I), K=1,N_PRE)
       ENDDO
       CLOSE( C_UNIT )

       ! NDS weighting function for aerosol optical thickness
       IF( INQ_WF('aer', 'WFT') ) THEN
          IDX_PAR = 2
          DO J = 1, NLAYERS
             ITOA(:) = ( WF_DB(IDX_PAR)%WF(:,IDL,J,1) + &
                  WF_DB(IDX_PAR + 1)%WF(:,IDL,J,1) )*CONV_UNITS(:)*CR(IDL)*DLAM(:)
             TM1 = SUM( ITOA(:)*Tv(NWL:1:-1) )
             DO K = 1, N_PRE
                Wnds(K,J) = ( b_co(K)*SUM(ITOA(:)*Tg(NWL:1:-1,K))- MDS(K)*TM1 )/Itv
             ENDDO
          ENDDO
          
          OPEN( C_UNIT,FILE=DIR_OUT//'WF_NDS-aer_ext.dat' )
          DO I = 1, NLAYERS
             WRITE(C_UNIT,'(1x,F7.3,10(1x,e15.7))') ZZZ(I), (Wnds(K,I), K=1,N_PRE)
          ENDDO
          CLOSE( C_UNIT )            
       ENDIF                  ! INQ_WF('aer', 'WFT')

       ! Calculate NDS weighting function for scalar parameters
       ! IF( N_SCAL_PARS == 0 ) RETURN
         
       ALLOCATE( W_scp(N_PRE) )
       
       ITOA(:) = WF_DB(IDX_PAR)%WF(:,IDL,1,1)*CONV_UNITS(:)*CR(IDL)*DLAM(:)
       TM1 = SUM( ITOA(:)*Tv(NWL:1:-1) )
       DO K = 1, N_PRE
          W_scp(K) = ( b_co(K)*SUM(ITOA(:)*Tg(NWL:1:-1,K)) - MDS(K)*TM1 )/Itv
       ENDDO
       
       WRITE(*,*) '   Weighting function: ', TRIM(WF_DB(IDX_PAR)%NAME)
       OPEN( C_UNIT,FILE=DIR_OUT//'WF_NDS-'//TRIM(WF_DB(IDX_PAR)%NAME)//'.dat' )
       DO K = 1, N_PRE
          WRITE(C_UNIT,'(1x,I3,(1x,e15.7))') K, W_scp(K)
       ENDDO
       CLOSE( C_UNIT )
    ENDIF
    
  END SUBROUTINE WRITE_OUT_GRIPS
  !***************************************************************************
  SUBROUTINE WRITE_WF_OUT
    
    INTEGER JJ_1, JJ_2, NL, I_L
    CHARACTER(LEN=2) LOC_STR
    LOGICAL  BLANK
    REAL (KND) FACTOR(NSPEC, NLO)
    
    WRITE(N_STR,'(I0)') NLAYERS
    IF( E_FORMAT ) THEN
       FMT_S1 = '('//TRIM(N_STR)//'(2x,E15.7))'
    ELSE
       FMT_S1 = '('//TRIM(N_STR)//'(2x,f12.6))'
    ENDIF
    REC_L1 = GET_RECL( FMT_S1 )
    
    WRITE(N_STR,'(I0)') NLO*NS_wf
    FMT_S2 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
    REC_L2 = GET_RECL( FMT_S2 )
    
    REC_L3 = MAX0(REC_L1, REC_L2)

    ! This loop enables the vertical dimension of WFs into separate files
    ! to be split
    DO II = 1, UBOUND(WF_DB,1)

       BLANK = .TRUE.
       IF( WF_DB(II)%NAME=='albedo'.AND.BRDF_FLAG ) BLANK = .FALSE.

       DO JJ_1 = 1, WF_DB(II)%NF
!          WRITE(*,*) JJ_1, WF_DB(II)%NF, WF_DB(II)%NL
          IF (WF_DB(II)%NF == 1) THEN
             STR = ''
             NL = WF_DB(II)%NL
          ELSE
             WRITE(LOC_STR,'(I0)') JJ_1
             STR = '-'//TRIM(LOC_STR)
             NL = 1
          END IF
          
          OPEN(C_UNIT, FILE=DIR_OUT//'wf_'//TRIM(WF_DB(II)%NAME)//& 
               STR//'.dat', STATUS = 'unknown', RECL=REC_L3)
          WRITE(C_UNIT,'(A)') TRIM(VERS)
          WRITE(C_UNIT,'(A)') '#  '//TRIM(WF_DB(II)%LONG_NAME)//' '//UNITS 
          WRITE(C_UNIT, &
               '("#  Spectral range: ", f8.3, 1x,"-", 1x,f8.3,2X,A) ') &
               WL(1), WL(NSPEC), TRIM(WL_UNITS)
          WRITE(C_UNIT,'("#  The number of spectral points: ",I0)') NSPEC
          IF(WF_DB(II)%NL > 1) THEN
             WRITE(C_UNIT,'("#  The number of altitude levels: ", I0)') NLAYERS
             WRITE(C_UNIT,FMT=FMT_S1) ZZZ(1:NLAYERS)
          END IF
          IF(WF_DB(II)%NAME == 'anps') THEN
             FORALL(KK = 1:NSPEC, I_L = 1:NLO) &
                  FACTOR(KK,I_L) = 1.0D0/SPEC_OUT(KK,1,I_L)
          ELSE
             FORALL(KK = 1:NSPEC, I_L = 1:NLO) &
                  FACTOR(KK,I_L) = CONV_UNITS(KK)*CR(I_L)
          END IF

          DO JJ_2 = JJ_1, NL
             IF( BLANK ) WRITE(C_UNIT,*)
             DO KK = 1, NSPEC
                I = WI(KK)
                WRITE(C_UNIT,FMT=FMT_S2) WL(I), &
                     (WF_DB(II)%WF(I,I_L,JJ_2,1:NS_wf)*FACTOR(I,I_L), I_L=1,NLO)
             END DO
          END DO
          
          CLOSE(C_UNIT)
       END DO
    END DO
    
  END SUBROUTINE WRITE_WF_OUT
!***************************************************************************
  SUBROUTINE WRITE_ELLIPSE(N1, N2, NAME)
    
    INTEGER, INTENT(IN) :: N1, N2
    CHARACTER (LEN = *) NAME
    ! Local parameters
    CHARACTER (LEN=13), PARAMETER:: R_NAME='WRITE_ELLIPSE'
    INTEGER, PARAMETER:: NO_PAR=5
    INTEGER Ij, Jv
    REAL (KND) Q_p, U_p, DUM, Tkap(2), Pol_ang(NSPEC,N_OP,NLO), &
         Beta(NSPEC,N_OP,NLO)
    REAL (KND), POINTER :: Sc1(:,:)

    CALL ST_TRACK (R_NAME, 1, 'I')
    
    DO I = 1,NSPEC
       DO Jo = N1,N2
          DO IDL = 1,NLO
             Q_p = STOK_OUT(I,1,Jo,IDL,2)
             U_p = STOK_OUT(I,1,Jo,IDL,3)
             Beta(I,Jo,IDL) = DATAN( STOK_OUT(I,1,Jo,IDL,4)&
                  /DSQRT( Q_p**2 + U_p**2 ) )/2.d0/Pie*180.d0
             ! Parameter Q does not equal to zero
             IF( DABS(Q_p) .GE. 1.d-12 ) THEN
                DUM = DATAN( U_p/Q_p )
                IF( REAL(DUM) >= 0. ) THEN
                   ! 2*kap in [0,pi/2]
                   Tkap(1) = DUM ;  Tkap(2) = DUM + Pie
                ELSE
                   ! 2*kap in [-pi/2,0]
                   Tkap(1) = DUM + Pie ; Tkap(2) = DUM + 2.d0*Pie
                ENDIF
                DO Ij = 1,2
                   IF( SIGN(1.d0,DCOS(Tkap(Ij))) == SIGN(1.d0,Q_p) )&
                        Pol_ang(I,Jo,IDL) = Tkap(Ij)/2.d0/Pie*180.d0
                ENDDO
                ! Parameter Q equal to zero
             ELSE
                IF( REAL(U_p) > 0. )  Pol_ang(I,Jo,IDL) = 45.d0
                IF( REAL(U_p) < 0. )  Pol_ang(I,Jo,IDL) = 135.d0
                ! neutral or circular polarized
                IF( REAL(U_p) == 0. ) Pol_ang(I,Jo,IDL) = -999.d0
             ENDIF
          ENDDO  ! over line-of-sight (IDL)
       ENDDO  ! over observer position (Jo)
    ENDDO  ! over spectral points (I)


    OPEN(C_UNIT, FILE = DIR_OUT//TRIM(NAME), RECL=150000)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//' Ellipticity parameters'

    IF(TRIM(ONE_WL) == 'unchanged') THEN 
       SELECT CASE (TRIM(Mult_WL_Form))
       CASE('wl_in_colm')
          DO II = 1,NSPEC
             I = WI(II)
             Sc1 => STOK_OUT(I,1,:,:,1)
             WRITE(C_UNIT,FMT=FMT_S1) WL(I), ( (&
                  DSQRT(SUM(STOK_OUT(I,1,Jo,Jv,2:4)**2) )/Sc1(Jo,Jv),&
                  DSQRT(SUM(STOK_OUT(I,1,Jo,Jv,2:3)**2) )/Sc1(Jo,Jv),&
                  STOK_OUT(I,1,Jo,Jv,4)/Sc1(Jo,Jv), Beta(I,Jo,Jv), &
                  Pol_ang(I,Jo,Jv), Jv=1,NLO ), Jo = N1,N2 )
          END DO

       CASE('wl_in_line')
          DO IDL = 1,NLO
             Sc1 => STOK_OUT(:,1,:,IDL,1)
             WRITE(C_UNIT,FMT=FMT_S1) &
                  ANG_OUT(1,IDL), ANG_OUT(2,IDL), ANG_OUT(3,IDL), ( (&
                  ! degree of polarization
                  DSQRT( SUM(STOK_OUT(WI(I),1,Jo,IDL,2:4)**2) )/Sc1(WI(I),Jo),&
                  ! degree of linear polarization
                  DSQRT( SUM(STOK_OUT(WI(I),1,Jo,IDL,2:3)**2) )/Sc1(WI(I),Jo),&
                  ! degree of circular polarization
                  STOK_OUT(WI(I),1,Jo,IDL,4)/Sc1(WI(I),Jo), &
                  ! ellipticity, polarization angle
                  Beta(WI(I),Jo,IDL), Pol_ang(WI(I),Jo,IDL), I=1,NSPEC ), Jo = N1,N2 )
          END DO
       END SELECT
    ELSE                     ! If TRIM(ONE_WL) /= 'unchanged'
       DO I = 1, N_LOOP
          Sc1 => STOK_OUT(1,1,:,:,1)
          I_1 = (I - 1)*N_SCALE + 1 
          I_2 = I_1 + (N_STEP - 1)*N_SHIFT
          WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT_TMP(RANK_F,I_1), ( ( &
               DSQRT( SUM(STOK_OUT(1,1,Jo,Jv,2:4)**2) )/Sc1(Jo,Jv),&
               DSQRT( SUM(STOK_OUT(1,1,Jo,Jv,2:3)**2) )/Sc1(Jo,Jv),&
               STOK_OUT(1,1,Jo,Jv,4)/Sc1(Jo,Jv), Beta(1,Jo,Jv), &
               Pol_ang(1,Jo,Jv), Jv= I_1,I_2,N_SHIFT), Jo = N1,N2 )
       ENDDO
    ENDIF
    
    CLOSE(C_UNIT)
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END SUBROUTINE WRITE_ELLIPSE
!***************************************************************************
  SUBROUTINE WRITE_INTENS(Iot, N1, N2, NAME)

    INTEGER, INTENT(IN) :: Iot, N1, N2
    CHARACTER (LEN = *) NAME
    ! Local parameters
    CHARACTER (LEN=12), PARAMETER:: R_NAME = 'WRITE_INTENS'
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    REC_L1 = GET_RECL( FMT_S1 )

!      WRITE(*,*) '   FMT_S1 = ', TRIM(FMT_S1)
!      WRITE(*,*) '   REC_L1 = ', REC_L1
!      WRITE(*,*) '   TRIM(ONE_WL) - ',TRIM(ONE_WL)
!      WRITE(*,*) '   TRIM(Mult_WL_Form) - ', TRIM(Mult_WL_Form)

!     ... Specific output for JPL 1D RT comparison
!      WRITE(*,*) '  N_LOS_ANG =  ', N_LOS_ANG
!      WRITE(*,*) '    N_SHIFT =  ', N_SHIFT
!      WRITE(*,*) '    N_SCALE =  ', N_SCALE
!      WRITE(*,*) '    N_STEP =  ', N_STEP

!      OPEN(C_UNIT, FILE = DIR_OUT//'Int_Davis.dat')
!        DO I = N_LOS_ANG,2,-1
!           I_1 = (I - 1)*N_SCALE + 1 
!           I_2 = I_1 + (N_STEP - 1)*N_SHIFT
!           WRITE(C_UNIT,FMT=FMT_S1) -ANG_OUT_TMP(2,I_1),
!     $          STOK_OUT(1,Iot,1,I_2,IS1:IS2)*CONV_UNITS(1)*CR(I_2)
!        ENDDO
!        DO I = 1, N_LOS_ANG
!           I_1 = (I - 1)*N_SCALE + 1 
!           I_2 = I_1 + (N_STEP - 1)*N_SHIFT
!           WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT_TMP(2,I_1),
!     $          STOK_OUT(1,Iot,1,I_1,IS1:IS2)*CONV_UNITS(1)*CR(I_1)
!        ENDDO
!      CLOSE(C_UNIT)

    OPEN(C_UNIT, FILE = DIR_OUT//TRIM(NAME), RECL=REC_L1)
    WRITE(C_UNIT,'(A)') TRIM(VERS)

!     *************************************************************
!     ****** Intensity output for multiple wavelengths  ***********
!     *************************************************************
    !    WRITE(*,*) '  CONV_UNITS(:) ', CONV_UNITS(:)
    IF(TRIM(ONE_WL) == 'unchanged') THEN 
!              WRITE(*,*) ' ========>   "unchanged" '
!              WRITE(*,*) ' Mult_WL_Form = ', Mult_WL_Form
       SELECT CASE (TRIM(Mult_WL_Form))
       CASE('wl_in_colm')
          WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//', '//TRIM(INT_LABEL)
          IF(TRIM(RTM_TYPE) /= 'transmission') &
               WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
          ! write output file:
          DO II = 1,NSPEC
             I = WI(II)
             WRITE(C_UNIT,FMT=FMT_S1) WL(I), ( (STOK_OUT(I,Iot,Jo,IDL,IS1:IS2)&
                  *CONV_UNITS(I)*CR(IDL), IDL=1,NLO), Jo = N1,N2)
          END DO
       CASE('wl_in_line')
           IF( AOT_out_LOG ) THEN
              WRITE(C_UNIT,'(A)') '#  Contents: SZA, VA, SAA, AOT, '&
                   //TRIM(INT_LABEL)
              IF(TRIM(RTM_TYPE) /= 'transmission') &
                   WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
             ! write output file including AOT:
             DO IDL = 1,NLO
                WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT(1,IDL), ANG_OUT(2,IDL), &
                     ANG_OUT(3,IDL), AOT_out_VAL, &
                     ((STOK_OUT(WI(I),Iot,Jo,IDL,IS1:IS2)*&
                     CONV_UNITS(WI(I))*CR(IDL), I=1,NSPEC), Jo = N1,N2)
             END DO
          ELSE
              WRITE(C_UNIT,'(A)') '#  Contents: SZA, VA, SAA, '&
                   //TRIM(INT_LABEL)
              IF(TRIM(RTM_TYPE) /= 'transmission') &
                   WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
             DO IDL = 1,NLO
                WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT(1,IDL), ANG_OUT(2,IDL), &
                     ANG_OUT(3,IDL),& 
                     ( (STOK_OUT(WI(I),Iot,Jo,IDL,IS1:IS2)*&
                     CONV_UNITS(WI(I))*CR(IDL), I=1,NSPEC), Jo = N1,N2 )
             END DO
          ENDIF
       END SELECT
       OUT_REPR = 'default'
    ELSE  ! If TRIM(ONE_WL) /= 'unchanged'
       !****** Intensity output for a single wavelength *******
       !-------------------------------------------------------
!       WRITE(*,*) '     Intensity output for a single wavelength  '
!       WRITE(*,*) '   IPRT   ', IPRT
       IF( .NOT.IPRT ) THEN
          WRITE(C_UNIT,'(A)') '#  Contents: '//AngLabel(RANK_F)%S//', '&
               //TRIM(INT_LABEL)
          IF(TRIM(RTM_TYPE) /= 'transmission') &
               WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)

          DO I = 1, N_LOOP
             I_1 = (I - 1)*N_SCALE + 1 
             I_2 = I_1 + (N_STEP - 1)*N_SHIFT
             WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT_TMP(RANK_F,I_1), &
                  ( (STOK_OUT(1,Iot,Jo,K,IS1:IS2)*CONV_UNITS(1)*CR(K), &
                  K = I_1,I_2,N_SHIFT), Jo = N1,N2)
          END DO

!!           ****** Intensity output in the form of "polar plot" *******
!!           -----------------------------------------------------------
!            WRITE(C_UNIT,'(2x,2(2x,I3),2x,F7.3)') 
!     $           N_LOS_ANG, N_AZ_ANG, ANG_OUT_TMP(1,1)
!            WRITE(N_STR,'(I0)') N_AZ_ANG
!            FMT_S2 = '('//TRIM(N_STR)//'(2x,F8.3))'
!            WRITE(C_UNIT,FMT=FMT_S2) ANG_OUT_TMP(3,1:N_AZ_ANG)
!            DO I = 1, N_LOOP
!               I_1 = (I - 1)*N_SCALE + 1 
!               I_2 = I_1 + (N_STEP - 1)*N_SHIFT
!               WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT_TMP(RANK_F,I_1), 
!     $              ( (STOK_OUT(1,Iot,Jo,K,IS1:IS2)*CONV_UNITS(1)*CR(K), 
!     $              K = I_1,I_2,N_SHIFT), Jo=N1,N2)
!            END DO            
       ELSE
          !... Output-format for the comparison with
          !    IPRT benchmark results (!! single SZA only !!)
          !===================================================
          WRITE(C_UNIT,'(A)') '#  Contents: VA, SAA, '//TRIM(INT_LABEL)
          IF(TRIM(RTM_TYPE) /= 'transmission') &
               WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
          FMT_S1 = '(2(1X,F6.1),'//'4'//TRIM(FMT_M_E)
          DO I = 1, N_LOS_ANG
             I_1 = (I - 1)*N_AZ_ANG
             DO J = 1, N_AZ_ANG
                K = I_1 + J
                WRITE(C_UNIT, FMT=FMT_S1) ANG_OUT_TMP(2,K), ANG_OUT_TMP(3,K), &
                     STOK_OUT(1,Iot,1,K,IS1:IS2)*CONV_UNITS(1)*CR(K)
             ENDDO  ! over azimuthal angle (J)
          ENDDO   ! over zenith angles (I)
          !===================================================
       ENDIF
       
    ENDIF
    CLOSE(C_UNIT)      
    CALL ST_TRACK (R_NAME, 1, 'O')
    
  END SUBROUTINE WRITE_INTENS
!***************************************************************************

  SUBROUTINE WRITE_B_AMF
    USE ST_VARS, ONLY: CONV_Z
      
    REAL (KND) V_j, VL(NLAYERS), AMF_B(NSPEC,NLO), Z_BAMF(NLAYERS)
    WRITE(N_STR,'(I0)') NLAYERS
    IF( E_FORMAT ) THEN
       FMT_S1 = '('//TRIM(N_STR)//'(2x,E15.7))'
    ELSE
       FMT_S1 = '('//TRIM(N_STR)//'(2x,f12.6))'
    ENDIF
    WRITE(N_STR,'(I0)') NLO
    FMT_S2 = '(1X,F14.7,'//TRIM(N_STR)//'(2X,ES17.9E3))'
    REC_L1 = GET_RECL( FMT_S1 )
    REC_L2 = GET_RECL( FMT_S2 )
    REC_L3 = MAX0(REC_L1, REC_L2)
    
    OPEN(C_UNIT, FILE=DIR_OUT//'block_amf.dat', STATUS = 'unknown', RECL=REC_L3)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Height resolved air mass factors for ' &
         //TRIM(TG_INFO(AMF_IDX)%Name)         
    WRITE(C_UNIT,'("#  Spectral range: ", f8.3, 1x,"-", 1x,f8.3,2X,A) ')&
         WL(WI(1)), WL(WI(NSPEC)), TRIM(WL_UNITS)
    WRITE(C_UNIT,'("#  The number of spectral points: ", I0)') NSPEC
    WRITE(C_UNIT,'("#  The number of altitude levels: ", I0)') NLAYERS
    IF( CONV_Z == 'z_bamf' ) THEN
       Z_BAMF(1) = ZZZ(1) -  (ZZZ(1) - ZZZ(2))/4.d0
       DO J = 2, NLAYERS-1
          Z_BAMF(J) = ZZZ(J-1)/4.d0 + ZZZ(J)/2.d0 + ZZZ(J+1)/4.d0 
       ENDDO
       Z_BAMF(NLAYERS) = ZZZ(NLAYERS) +  (ZZZ(NLAYERS-1) - ZZZ(NLAYERS))/4.d0
    ELSE
       Z_BAMF(:) = ZZZ(1:NLAYERS)
    ENDIF
    WRITE(C_UNIT,FMT=FMT_S1) Z_BAMF(1:NLAYERS)
    DO J = 1, NLAYERS
       WRITE(C_UNIT,*)
       DO II = 1, NSPEC
          I = WI(II)
          WRITE(C_UNIT,FMT=FMT_S2) WL(I), WF_DB(1)%WF(I,1:NLO,J,1)
       END DO
    END DO
    CLOSE(C_UNIT)
    
    ! Integrated block-amf         
    V_j = SUM(TG_INFO(AMF_IDX)%ND(:)*DZ(:))/2.d0
    DO J = 1, NLAYERS
       VL(J) = TG_INFO(AMF_IDX)%ND(J)*DZ(J)/2.d0
    ENDDO
    DO I = 1, NSPEC
       DO IDL = 1, NLO
          DUM = 0.d0
          DO J = 1, NLAYERS
             IF( REAL(TG_INFO(AMF_IDX)%ND(J)) == 0. ) CYCLE
             DUM = DUM + WF_DB(1)%WF(I,IDL,J,1)*VL(J)
          ENDDO            ! over layers (J)
          AMF_B(I,IDL) = DUM/V_j
       ENDDO               ! over line-of-sight (IDL)
    ENDDO                  ! over spectral points (I)
    
    WRITE(N_STR,'(I0)') NLO
    FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
    REC_L1 = GET_RECL( FMT_S1 )
    OPEN(C_UNIT, FILE = DIR_OUT//'bamf_int.dat', RECL=REC_L1)
    !..........................................................
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//&
         ', Integrated block air mass factors for '//&
         TRIM(TG_INFO(AMF_IDX)%Name)
    DO II = 1, NSPEC
       I = WI(II)
       WRITE(C_UNIT,FMT=FMT_S1) WL(I), (AMF_B(I,K), K = 1, NLO)
    ENDDO
    !..........................................................
    CLOSE(C_UNIT)
    
  END SUBROUTINE WRITE_B_AMF

!***************************************************************************
  SUBROUTINE OUT_RRS
    REAL (KND) HEL(NLO,Nvs)
    
    WRITE(N_STR,'(I0)') NLO*(IS2 - IS1 + 1)
    FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
    REC_L1 = GET_RECL( FMT_S1 )

    ! Output for rotational Raman scattering if single scattering
    ! mode is selected
    IF( TRIM(SCATT_MODE)=='ss' ) THEN
!         WRITE(*,*) '   Output of delta-Raman: single scattering mode'
!         DO I = 1, NSPEC
!!            WRITE(*,*) STOK_OUT(I,1,1,:,IS1:IS2), DEL_RRS(I,:,IS1:IS2)
!            WRITE(*,'(4(1x,E15.7))') DEL_RRS(I,:,IS1:IS2)
!         END DO    
       Call OpenOut ('intensity.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//', '//INT_LABEL)
       Call WriteOut (Unt, '#  Irradiance: '//FLUX_LABEL)
       DO II = 1, NSPEC
          I = WI(II)
          HEL(:,IS1:IS2) = ( STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I)& 
               + DEL_RRS(I,:,IS1:IS2) )
          WRITE(OutStr,FMT=FMT_S1) WL(I),(HEL(IDL,IS1:IS2),IDL=1,NLO)
          Call WriteOut (Unt, OutStr)
       END DO
       Call CloseOut (Unt)
         
       Call OpenOut ('intensity_noring.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//&
            ', Radiance without raman scattering '//UNITS) 
       Call WriteOut (Unt, '#  Irradiance: '//FLUX_LABEL)
       DO II = 1, NSPEC
          I = WI(II)
          HEL(:,IS1:IS2) = STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I) 
          WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2),IDL=1,NLO)
          Call WriteOut (Unt, OutStr)
       END DO
       Call CloseOut (Unt)

       Call OpenOut ('ring.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt, '#  Contents: '//WL_LABEL//', Ring spectrum') 
       Call WriteOut (Unt,  '#  Irradiance: '//FLUX_LABEL)       
       SELECT CASE( RIN_OUT )
       CASE('rel')
          DO II = 1, NSPEC
             I = WI(II)
             DO K = 1, Nvs
                WHERE( STOK_OUT(I,1,1,:,K) /= 0.d0 )
                   HEL(:,K) = DEL_RRS(I,:,K)/STOK_OUT(I,1,1,:,K)/CONV_UNITS(I) 
                ELSE WHERE
                   HEL(:,K) = 0.d0
                END WHERE
             ENDDO
!!! Absolute contribution !!!
!               WRITE(*,*) '  ***********************  '
!               WRITE(*,*) '  Output without devision  '
!               WRITE(*,*) '  ***********************  '
!               HEL(:,IS1:IS2) = DEL_RRS(I,:,IS1:IS2)
             WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2), IDL=1,NLO)
             Call WriteOut (Unt, OutStr)
          END DO
       CASE('abs-sn')
          DO II = 1, NSPEC
             I = WI(II)
             DO K = 1, Nvs
                HEL(:,K) = DEL_RRS(I,:,K)/IRR_AD(I)
             ENDDO
             WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2), IDL=1,NLO)
             Call WriteOut (Unt, OutStr)
          END DO
       CASE('log')
          DO II = 1, NSPEC
             I = WI(II)
             HEL(:,IS1:IS2) = DLOG( STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I) &
                  + DEL_RRS(I,:,IS1:IS2) ) - &
                  DLOG( STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I) )
             WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2), IDL=1,NLO)
             Call WriteOut (Unt, OutStr)
          END DO
       END SELECT
       Call CloseOut (Unt)
       RETURN
    ENDIF  ! RRS in the case of single scattering 

    ! If forward-adjoint technique was selected for rotational Raman scattering
    IF( RRS_ADJ ) THEN
!         WRITE(*,*) 'Output of delta-Raman: multiple scattering mode'
!         DO I = 1, NSPEC
!!            WRITE(*,*) STOK_OUT(I,1,1,:,IS1:IS2), DEL_RRS(I,:,IS1:IS2)
!            WRITE(*,'(4(1x,E15.7))') DEL_RRS(I,:,IS1:IS2)
!         END DO 
       Call OpenOut ('intensity.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//', '//INT_LABEL)
       Call WriteOut (Unt, '#  Irradiance: '//FLUX_LABEL)
       DO II = 1, NSPEC
          I = WI(II)
          DO K = 1, Nvs
             HEL(:,K) = STOK_OUT(I,1,1,:,K)*CONV_UNITS(I)  + DEL_RRS(I,:,K)
          ENDDO  ! over Stokes parameters (K)
          WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2),IDL=1,NLO)
          Call WriteOut (Unt, OutStr)
       END DO  ! over wavelengths (II)
       Call CloseOut (Unt)
        
       Call OpenOut ('intensity_noring.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//&
            ', Radiance without raman scattering '//UNITS) 
       Call WriteOut (Unt, '#  Irradiance: '//FLUX_LABEL)
       DO II = 1, NSPEC
          I = WI(II)
          DO K = 1, Nvs
             HEL(:,K) = STOK_OUT(I,1,1,:,K)*CONV_UNITS(I) 
          ENDDO  ! over Stokes parameters (K)
          WRITE(OutStr,FMT=FMT_S1) WL(I),(HEL(IDL,IS1:IS2),IDL=1,NLO) 
          Call WriteOut (Unt, OutStr)
       END DO  ! over wavelengths (II)          
       Call CloseOut (Unt)
       
       Call OpenOut ('ring.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt, '#  Contents: '//WL_LABEL//', Ring spectrum') 
       Call WriteOut (Unt,  '#  Irradiance: '//FLUX_LABEL)
       SELECT CASE( RIN_OUT )
       CASE('abs-sn')
          DO I = 1, NSPEC
             DO K = 1, Nvs
                HEL(:,K) = DEL_RRS(I,:,K)/IRR_AD(I)
             ENDDO
             WRITE(OutStr,FMT=FMT_S1) WL(I),(HEL(IDL,IS1:IS2), IDL=1,NLO)
             Call WriteOut (Unt, OutStr)
          END DO
       CASE('rel')
          DO I = 1, NSPEC
             DO K = 1, Nvs
                WHERE( STOK_OUT(I,1,1,:,K) /= 0.d0 )
                   HEL(:,K) = DEL_RRS(I,:,K)&
                        /STOK_OUT(I,1,1,:,K)/CONV_UNITS(I) 
                ELSE WHERE
                   HEL(:,K) = 0.d0
                END WHERE
             ENDDO
!!! Absolute contribution !!!
!                    WRITE(*,*) '  ***********************  '
!                    WRITE(*,*) '  Output without devision  '
!                    WRITE(*,*) '  ***********************  '
!                    HEL(:,1) = DEL_RRS(I,:,1) 
             WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2),IDL=1,NLO)  
             Call WriteOut (Unt, OutStr)
          END DO
       CASE('log')
          DO II = 1, NSPEC
             I = WI(II)
             HEL(:,1) = DLOG( SPEC_OUT(I,1,:)*CONV_UNITS(I) + DEL_RRS(I,:,1) )&
                  - DLOG( SPEC_OUT(I,1,:)*CONV_UNITS(I) )
             WRITE(OutStr,FMT=FMT_S1) WL(I), HEL(1:NLO,1)
             Call WriteOut (Unt, OutStr)
          END DO
       END SELECT
       Call CloseOut (Unt)
    ELSE
       !     ... If DO technique was selected for rotational Raman scattering
       Call OpenOut ('intensity.dat', Unt, RecLen=REC_L1)
       Call WriteOut (Unt, TRIM(VERS))
       Call WriteOut (Unt,  '#  Contents: '//WL_LABEL//', '//INT_LABEL)
       Call WriteOut (Unt, '#  Irradiance: '//FLUX_LABEL)
       DO II = 1, NSPEC
          I = WI(II)
          IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
             ! In this case the Raman contribution due to single
             ! scattering should be added
             HEL(:,IS1:IS2) = ( STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I) &
                  + DEL_RRS(I,:,IS1:IS2) )
          ELSE
             HEL(:,IS1:IS2) = STOK_OUT(I,1,1,:,IS1:IS2)*CONV_UNITS(I)
          ENDIF
          WRITE(OutStr,FMT=FMT_S1) WL(I), (HEL(IDL,IS1:IS2),IDL=1,NLO)
          Call WriteOut (Unt, OutStr)
       END DO
       Call CloseOut (Unt)
    ENDIF
    
    RETURN
  END SUBROUTINE OUT_RRS
!***************************************************************************
  SUBROUTINE OUT_VRS
    REAL (KND) HEL(NLO)

    OPEN(C_UNIT, FILE=DIR_OUT//'intensity.dat', RECL=15000)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//', '//TRIM(INT_LABEL)
    WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
    !WRITE(C_UNIT,'(A)') '#  Radiance '//TRIM(UNITS) 
    DO II = 1, NSPEC
       I = WI(II)
       HEL(:) = SPEC_OUT(I,1,:)*CONV_UNITS(I) + DEL_VRS(I,:)
       WRITE(C_UNIT,101) WL(I),( HEL(IDX_LOS), IDX_LOS=1,NLO )
    END DO
    CLOSE(C_UNIT)
        
    OPEN(C_UNIT, FILE=DIR_OUT//'intensity_noVRS.dat', RECL=15000)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//&
         ', Radiance without raman scattering '//TRIM(UNITS) 
    WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
    DO II = 1, NSPEC
       I = WI(II)
       HEL(:) = SPEC_OUT(I,1,:)*CONV_UNITS(I) 
       WRITE(C_UNIT,101) WL(I),( HEL(IDX_LOS), IDX_LOS=1,NLO )
    END DO
    CLOSE(C_UNIT)
    
    OPEN(C_UNIT, FILE=DIR_OUT//'ring_VRS.dat', STATUS = 'unknown', RECL=15000)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//' Ring spectrum' 
    WRITE(C_UNIT,'(A)') '#  Irradiance: '//TRIM(FLUX_LABEL)
    
    DO II = 1, NSPEC
       I = WI(II)
       HEL(:) = DEL_VRS(I,:)
       WRITE(C_UNIT,101) WL(I),(HEL(IDX_LOS), IDX_LOS=1,NLO)
    END DO
    CLOSE(C_UNIT)
    
101 FORMAT(1X,F12.5,6000(2X,ES17.9E3))
    
  END SUBROUTINE OUT_VRS

  !***************************************************************************
  SUBROUTINE WRITE_SPHER_ALB (FILENAME, LABEL)
    CHARACTER (LEN = *)  FILENAME, LABEL
    OPEN(C_UNIT, FILE=DIR_OUT//FILENAME, RECL=REC_L3)
      WRITE(C_UNIT,'(A)') TRIM(VERS)
      WRITE(C_UNIT,'(A)') LABEL
      SELECT CASE( TRIM(Mult_WL_Form) )  
      CASE('wl_in_line')
         WRITE(C_UNIT,FMT=FMT_S1) (WL(WI(I)),I=1,NSPEC)
         DO J = N_S, N_E
            WRITE(C_UNIT,FMT=FMT_S2) ZZZ(J),(FLUX_ARR(WI(I),J,1), I = 1,NSPEC)
         END DO
      CASE('wl_in_colm')
         WRITE(C_UNIT,FMT=FMT_S1) (ZZZ(I),I=N_S,N_E)
         DO II = 1,NSPEC
            J = WI(II)
            WRITE(C_UNIT,FMT=FMT_S2) WL(J), (FLUX_ARR(J,I,1), I = N_S,N_E)
         END DO
      END SELECT
    CLOSE(C_UNIT)
  END SUBROUTINE WRITE_SPHER_ALB
  
  !***************************************************************************
  SUBROUTINE WRITE_FLUX (FILENAME, LABEL)
    INTEGER    Noc
    CHARACTER (LEN = *)  FILENAME, LABEL 
    CHARACTER (LEN = 30) N_STR
    
    Noc = 0  ;  IF( UWT ) Noc = NLW
!    write(*,*) '     What is Mult_WL_Form  ', Mult_WL_Form
    SELECT CASE (Mult_WL_Form)
    CASE('wl_in_colm')
       WRITE(N_STR,'(I0)') N_SOL_ANG*(N_E + Noc - N_S + 1)  
       FMT_S1 = TRIM(FMT_M_S)//TRIM(N_STR)//TRIM(FMT_M_E)
       
    CASE('wl_in_line')
       WRITE(N_STR,'(I0)') N_SOL_ANG*NSPEC
       
       FMT_S1 = '(1X,F10.4,1X,F10.4,'//TRIM(N_STR)//FMT_M_E
       
    END SELECT
    REC_L1 = GET_RECL( FMT_S1 )
    
    OPEN(C_UNIT, FILE = DIR_OUT//FILENAME, RECL=REC_L1)
    WRITE(C_UNIT,'(A)') TRIM(VERS)
    WRITE(C_UNIT,'(A)') '#  Contents: '//WL_LABEL//', '//LABEL//UNITS_FL
    WRITE(C_UNIT,'(A)') '#  Irradiance: '//FLUX_LABEL
    
    SELECT CASE (Mult_WL_Form)
    CASE('wl_in_colm')
       DO II = 1, NSPEC
          I = WI(II)
          WRITE(C_UNIT,FMT=FMT_S1) WL(I), ( (FLUX_ARR(I,J,IDL)*&
               CONV_UNITS(I), J = N_S,N_E+Noc), IDL=1,N_SOL_ANG)
       END DO
    CASE('wl_in_line')
       DO IDL = 1,N_SOL_ANG
          DO J = N_S,N_E
             WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT(1,IDL), ZZZ(J),& 
                  (FLUX_ARR(WI(I),J,IDL)*CONV_UNITS(WI(I)), I=1,NSPEC)
          END DO              ! over layers (J)
       ENDDO                  ! over solar angles (IDL)
       DO IDL = 1,N_SOL_ANG
          DO J = 1,Noc
             WRITE(C_UNIT,FMT=FMT_S1) ANG_OUT(1,IDL), -ZZZ_UW(J), &
                  (FLUX_ARR(WI(I),J+NLAYERS,IDL)*CONV_UNITS(WI(I)), I=1,NSPEC)
          ENDDO               ! over layers (J)
       ENDDO                  ! over solar angles (IDL)
!         DO J = N_S,N_E
!            WRITE(C_UNIT,FMT=FMT_S1) ZZZ(J), ( (FLUX_ARR(I,J,IDL)*
!     $           CONV_UNITS(I), I=1,NSPEC), IDL=1,N_SOL_ANG )
!         END DO
!         DO J = 1,Noc
!            WRITE(C_UNIT,FMT=FMT_S1) -ZZZ_UW(J), 
!     $           ((FLUX_ARR(I,J+NLAYERS,IDL)*CONV_UNITS(I),I=1,NSPEC),
!     $           IDL=1,N_SOL_ANG)
!         END DO

    END SELECT
    CLOSE(C_UNIT)
    
  END SUBROUTINE WRITE_FLUX
!***************************************************************************
END SUBROUTINE OUTPUT
!***************************************************************************
CHARACTER (LEN=20) FUNCTION WRITE_FLOAT (FF, NN)
  USE TYPES,   ONLY : KND
  INTEGER               NN, LL
  REAL (KND)            FF
  
  WRITE(WRITE_FLOAT,'(I0)') FLOOR(FF*10**NN)
  LL = LEN_TRIM(WRITE_FLOAT) - NN
  WRITE_FLOAT(1:LL + NN + 1) = WRITE_FLOAT(1:LL)//'.'//WRITE_FLOAT(LL + 1:LL + NN)

  RETURN
END FUNCTION WRITE_FLOAT
!***************************************************************************
CHARACTER (LEN=20) FUNCTION WRITE_ALT (FF, NN)
  USE TYPES,   ONLY : KND
  CHARACTER (LEN=20)   STR, WRITE_FLOAT
  INTEGER               I, KK, NUM, NN
  REAL (KND)            FF
  
  IF(DABS(FF).GT.10.D0**(-NN)) THEN
     WRITE_ALT =  WRITE_FLOAT (FF, NN)
     RETURN
  END IF
  
  WRITE(STR,'(SP,E8.1)') FF
  READ(STR(7:8),*) KK
  WRITE_ALT = '0.'
  DO I = 1, KK
     WRITE_ALT = TRIM(WRITE_ALT)//'0'
  END DO
  WRITE_ALT = TRIM(WRITE_ALT)//STR(4:4) 
  IF(FF < 0) WRITE_ALT = '-'//TRIM(WRITE_ALT)
  
  RETURN
END FUNCTION WRITE_ALT
!***************************************************************************
INTEGER FUNCTION GET_RECL(FMT_STRING)
  CHARACTER (LEN = *)   FMT_STRING
  CHARACTER (LEN = 200) INP_STR, TMP_STR
  CHARACTER (LEN = 10)  SUB_STR
  INTEGER               LEN_IS, LL, STR_LEN, POS_S, POS_E

  INP_STR = TRIM(FMT_STRING)
  DO
     LEN_IS = LEN_TRIM(INP_STR)
     POS_S = 1 ; POS_E = LEN_IS 
     DO LL = 1, LEN_IS
        SELECT CASE (INP_STR(LL:LL))
        CASE ('(')
           POS_S = LL + 1
        CASE (')')
           POS_E = LL - 1
           EXIT
        END SELECT
     END DO
     IF(POS_S < 2 .AND. POS_E > LEN_IS - 2) EXIT
     
     CALL CALC_LEN (INP_STR(POS_S:POS_E)//',', STR_LEN)
     
     WRITE(SUB_STR, '(A,I0)') 'I', STR_LEN
     TMP_STR = TRIM(SUB_STR)
     IF(POS_S > 2) TMP_STR = INP_STR(1:POS_S - 2)//TRIM(TMP_STR)
     IF(POS_E < LEN_IS - 1) TMP_STR = TRIM(TMP_STR)//INP_STR(POS_E + 2:LEN_IS)
     INP_STR = TRIM(TMP_STR)
         
  END DO
  GET_RECL = MAX0(STR_LEN,500)
      
  RETURN
END FUNCTION GET_RECL
!***************************************************************************
SUBROUTINE CALC_LEN (STR, TL)
  CHARACTER (LEN = *)   STR
  CHARACTER (LEN = 20)  ID_CUR, ID_PRE, STR_READ
  LOGICAL               FC_FLAG, LN_FLAG
  INTEGER               LEN, II, FACTOR, TL, LENGTH

  LEN = LEN_TRIM(STR)
  ID_PRE = 'separator'
  STR_READ = ''
  FC_FLAG = .FALSE. ;  LN_FLAG = .FALSE.
  FACTOR = 0 ;  LENGTH = 0
  TL = 0
  DO II = 1, LEN
     SELECT CASE (IACHAR(STR(II:II)))
     CASE (48:57)
        SELECT CASE (TRIM(ID_PRE))
        CASE ('letter')
           STR_READ = TRIM(STR_READ)//STR(II:II)
           ID_CUR = 'length'
        CASE ('separator')
           STR_READ = TRIM(STR_READ)//STR(II:II)
           ID_CUR = 'factor'
        CASE ('length', 'factor')
           STR_READ = TRIM(STR_READ)//STR(II:II)
           ID_CUR = ID_PRE
        END SELECT
     CASE (65:90,97:122)
        IF(TRIM(ID_PRE) /= 'other') THEN
           ID_CUR = 'letter'
           SELECT CASE (TRIM(ID_PRE))
           CASE ('factor')
              READ(STR_READ, *) FACTOR
              FC_FLAG = .TRUE.
           CASE ('separator')
              FACTOR = 1
              FC_FLAG = .TRUE.
           END SELECT
        END IF
        STR_READ = ''
     CASE DEFAULT 
        SELECT CASE (TRIM(ID_PRE))
        CASE ('letter')
           IF(STR(II-1:II-1) == 'A'.OR.STR(II-1:II-1) == 'a') THEN
              LENGTH = 200
           ELSE
              LENGTH = 1
           END IF
           LN_FLAG = .TRUE.
        CASE ('length')
           IF(STR_READ /= '') THEN
              READ(STR_READ, *) LENGTH
              IF((STR(II-2:II-2) == 'I'.OR.STR(II-2:II-2) == 'i')&
                   .AND.LENGTH == 0) LENGTH = 10
           END IF
           LN_FLAG = .TRUE.
        END SELECT
        STR_READ = ''
     END SELECT
     
     SELECT CASE (IACHAR(STR(II:II)))
     CASE (48:57,65:90,97:122)
     CASE (44,32)
        ID_CUR = 'separator'           
     CASE DEFAULT 
        ID_CUR = 'other'           
     END SELECT
     
     IF(FC_FLAG.AND.LN_FLAG) THEN
        TL = TL + FACTOR*LENGTH
        FC_FLAG = .FALSE.
        LN_FLAG = .FALSE.
        FACTOR = 0 ;  LENGTH = 0
     END IF
     
     ID_PRE = ID_CUR

  END DO
  
  RETURN
END SUBROUTINE CALC_LEN
!***************************************************************************
