C********************************************************************C
C
C	SCIATRAN - Version 4.5 - Last modification: 16 Feb 2022
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE OVERALL_SETTINGS(WL_START)
c     ... Called by "gt_iface.f"

      USE ALB_PARS,    ONLY: ALBEDO_CONST, DO_SPECTRAL_ALBEDO
      USE BRDF,        ONLY: N_svk, Nwl_veg, WL_veg, Alb_svk, Cfr_v, 
     $                       NDVI, N_Sph_WL, Sph_OUT, Sph_WL, INP_BRDF,
     $                       MP_gp, Sig_t
      USE CLOUDS,      ONLY: DO_CLOUD
      USE CLOUDS_SS,   ONLY: DO_RAY_DEP, RAY_DEP, DO_RAY_TAU, 
     $                       RAY_TAU_INP, DO_SSRCA
      USE GRIDS,       ONLY: NLAYERS, ZZZ, GRID_MARK, GRID_MARK_UW
      USE HYDROSOLS,   ONLY: HS_SCALING
      USE INFO,        ONLY: TG_INFO
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: IRRAD_FLAG, EXTR_FLUX, FILENAME_SOLSPEC
      USE LOG_PARS,    ONLY: LUNIT, DIR_OUT, USE_SHAD
      USE RAMAN,       ONLY: RRS_ADJ, RRS_DO
      USE RTM_OUTPUT,  ONLY: N_OP
      USE ST_VARS,     ONLY: DO_AEROSOL, BRDF_FLAG, SURF_TYPE, VHL,
     $                       ADAPTIVE_GRID, CS_ANALYTIC, ARB_OL, 
     $                       RTM_MODE, SCATT_MODE, RTM_CORE,
     $                       SPECULAR, DO_AZIMUTH, RTM_TYPE, INT_OUT,
     $                       APPR_RRS, Finc, Vec_Sca, GeoD => GeoD_Out
      USE SPHER_PARS,  ONLY: ALTITUDE, LOS_DEF_FLAG, N_OBS_ALT,
     $                       SET_OUT_ALT
      Use Tools,       Only: CloseOut, WriteOut, OpenOut
      USE TYPE_VARS,   ONLY: KND
      USE WATER,       ONLY: UWT, Obs_POSITION, NL_UW => NLAYERS_UW,
     $                       ZZZ_UW, DO_RTAU_W, RTAU_W, 
     $                       NO_COUPLING, ALB_BOTTOM, A_RTmat, UIT,
     $                       INCL_INEL, SS_PDF
      USE WATER_ICE,   ONLY: IWS      
c ----------------------------------------------------------------------

      REAL (KND), INTENT(IN) :: WL_START
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'OVERALL_SETTINGS'
      INTEGER   I, K, J, IA, LL, NLX, Ws, Ws1, NL_ACT, Unt
      CHARACTER (LEN=3)   Top_Bot, T_AG, T_SS, T_AN, On_Off
      CHARACTER (LEN=10)  ATM_OC
      CHARACTER (LEN=17)  N_STR
      CHARACTER (LEN=50)  TEXT
      CHARACTER (LEN=80)  TXT_I, TXT_O, TXT_R, TXT_A, TXT_C, TXT_G,
     $     TXT_S, TXT_S1, TXT_Rw, TXT_Aw, TXT_Gw, TXT_OB, O_F, TXT_A1, 
     $     TXT_A2, S_P, TDUM, O_I
      Character (Len=500)  OutStr      
      REAL (KND) DC, Aveg
c     ==================================================================

      CALL ST_TRACK (R_NAME, 3, 'I')  

      IF( RTM_MODE == 'vod' ) THEN
         Call OpenOut ('SCE_SUMMARY.OUT', Unt)
         CALL WRITE_UD(1,Unt)
         Call WriteOut (Unt,' ========================')
         Call WriteOut (Unt,'   Selected mode : VOD')
         Call WriteOut (Unt,' ========================')
         Call CloseOut (Unt)
         CALL ST_TRACK (R_NAME, 3, 'O')
         RETURN
      ENDIF

      IA = 1
c     ... Incident solar flux at the top of atmosphere
c     ------------------------------------------------
      SELECT CASE( TRIM(EXTR_FLUX) )
      CASE('Pi')
         TXT_I = '3.14159 [Any irradiance unit]'
      CASE('Unity')
         TXT_I = '1.0 [Any irradiance unit]'
      CASE('File')
         TXT_I = 'As given in '//'"'//TRIM(FILENAME_SOLSPEC)//'"'
      END SELECT

c     ... Rayleigh setting
c     --------------------
      TXT_R = 'See file "./DATA_OUT/SCE_RAYLEIGH.OUT"'
      IF( DO_RAY_DEP ) THEN
         TXT_R = 'Scaled'                     
         IF( RAY_DEP == 0.d0 ) THEN
            TXT_R = 'Not present'
         ENDIF
      END IF
      IF( DO_RAY_TAU ) THEN
         TXT_R = 'Set by optical thickness'
         IF( REAL(RAY_TAU_INP) == 0.0 ) THEN
            TXT_R = 'Not present'
         ENDIF
      ENDIF

c     ... Aerosol setting
c     -------------------
      IF( VHL ) THEN
         TXT_A = 'See file "./DATA_OUT/SCE_VHL.OUT"'
      ELSE
         TXT_A = 'See file "./DATA_OUT/SCE_AEROSOL.OUT"'
      ENDIF
      IF( .NOT.DO_AEROSOL ) TXT_A = 'Not present'

c     ... Cloud setting
c     -----------------
      TXT_C = 'See file "./DATA_OUT/SCE_CLOUD.OUT"'
      IF( .NOT.DO_CLOUD ) TXT_C = 'Not present'

c     ... Gaseous absorption
c     ----------------------
      LL = COUNT(TG_INFO(:)%FM_FLG)
      TXT_G = 'See file "./DATA_OUT/SCE_ABSORBER.OUT"'
      IF( LL == 0 )  TXT_G = 'Not present'          

c     ... Surface reflection
c     ----------------------
      TXT_S = 'Lambertian reflector'
      IF( BRDF_FLAG ) THEN
         TXT_S = 'Bidirectional reflection'//' - '//TRIM(SURF_TYPE)
      ENDIF
      IF( UWT )  THEN 
         IF( UIT ) THEN
            TXT_S = 'Radiative transfer within atmosphere and ice'
         ELSE
            TXT_S = 'Radiative transfer within atmosphere and ocean'
         ENDIF
         IF( NO_COUPLING ) THEN
            TXT_S1 = ' off'
         ELSE
            TXT_S1 = ' on'
         ENDIF
c        ... Rayleigh setting in water
c        -----------------------------
         TXT_Rw = 'See file "./DATA_OUT/SCE_RAYLEIGH.OUT"'
         IF( DO_RTAU_W ) THEN
            TXT_Rw = 'Set by optical thickness'
            IF( RTAU_W == 0.d0 ) THEN
               TXT_Rw = 'Not present'
            ENDIF
         ENDIF
c        ... Hydrosol setting
c        --------------------
         SELECT CASE( IWS(1)%N_HPC )
         CASE( 1 )
            TXT_Aw = 'See file "./DATA_OUT/SCE_MAN-HSOL.OUT"'
         CASE( 2 )
            TXT_Aw = 'See file "./DATA_OUT/SCE_KOP-HSOL.OUT"'
         CASE( 3 )
            TXT_Aw = 'See file "./DATA_OUT/SCE_KOP-HSOL.OUT"'//
     $           ' & SCE_MAN-HSOL.OUT'
         END SELECT
         IF( HS_SCALING==0.d0.OR.IWS(1)%N_HPC==0 )
     $        TXT_Aw = 'Not present'

c        ... Absorption in water
c        -----------------------
         TXT_Gw = 'See file "./DATA_OUT/SCE_ABSORBER-WATER.OUT"'
         
c        ... Ocean bottom reflectance
c        ----------------------------
         TXT_OB = 'Lambertian'
      ENDIF

c     ... Observation geometry
c     ------------------------
      ATM_OC = 'atmosphere'
      NLX = NLAYERS

      IF( ARB_OL ) THEN
         TEXT = 'at the height [km]:'
      ELSE
         TEXT = 'at the height [km] (standard grid):'
      ENDIF

      IF( TRIM(RTM_MODE)/='flux'.AND.TRIM(RTM_MODE)/='spher_alb') THEN
         IF( .NOT.UWT ) THEN
            SELECT CASE( GeoD%View(IA)%S  )
            CASE('up')
               TXT_O = 'Upward radiation in the atmosphere'
            CASE('down')
               TXT_O = 'Downward radiation in the atmosphere'
            END SELECT
         ELSE
            SELECT CASE( GeoD%View(IA)%S  )
            CASE('up')
               IF( TRIM(Obs_POSITION(1)) == 'Atmos' ) THEN ! Preliminary ??
                  TXT_O = 'Upward radiation in the atmosphere'
               ENDIF
               IF( TRIM(Obs_POSITION(1)) == 'Ocean' ) THEN
                  TXT_O = 'Upward radiation in the ocean'
                  ATM_OC = 'ocean' ; NLX = NL_UW
                  IF( ARB_OL ) THEN
                     TEXT = 'at the depth [m]:'
                  ELSE
                     TEXT = 'at the depth [m] (standard grid):'
                  ENDIF
               ENDIF
            CASE('down')
               IF( TRIM(Obs_POSITION(1)) == 'Atmos' ) THEN
                  TXT_O = 'Downward radiation in the atmosphere'
               ENDIF
               IF( TRIM(Obs_POSITION(1)) == 'Ocean' ) THEN
                  TXT_O = 'Downward radiation in the ocean'            
                  ATM_OC = 'ocean' ; NLX = NL_UW
                  IF( ARB_OL ) THEN
                     TEXT = 'at the depth [m]:'
                  ELSE
                     TEXT = 'at the depth [m] (standard grid):'
                  ENDIF
               ENDIF
            END SELECT
         ENDIF  ! UWT
      ENDIF  ! /='flux', OR /='spher_alb'

      IF( SET_OUT_ALT ) THEN
         Top_Bot = 'use'
      ELSE
         IF(N_OBS_ALT(1) == 1)   Top_Bot = 'top'
         IF(N_OBS_ALT(1) == NLX) Top_Bot = 'bot'
      ENDIF
      IF( TRIM(RTM_MODE) == 'flux') THEN 
         Top_Bot = 'flu'
         TXT_O = 'Fluxes in the atmosphere'
         IF( UWT ) TXT_O = 'Fluxes in the atmosphere in ocean'
      ENDIF
      IF( TRIM(RTM_MODE) == 'spher_alb') THEN 
         Top_Bot = 'spa'
         TXT_O = 
     $   'Spherical albedo and diffuse transmission in the atmosphere'
         IF( UWT ) TXT_O = 'Fluxes in the atmosphere in ocean'
      ENDIF
c     ... Additional features
      T_AG = ' No' ; T_SS = ' No' ; T_AN = ' No'
      IF( ADAPTIVE_GRID ) T_AG = 'Yes'
      IF( DO_SSRCA ) T_SS = 'Yes'         
      IF( CS_ANALYTIC ) T_AN = 'Yes'
      Call OpenOut ('SCE_SUMMARY.OUT', Unt)
        CALL WRITE_UD(1,Unt)
        Call WriteOut (Unt,' ========================')
        Call WriteOut (Unt,'   INCIDENT SOLAR FLUX  :  '//TRIM(TXT_I))
        NL_ACT = NLAYERS-1
        IF( ARB_OL .AND. RTM_CORE == 'DOM' ) 
     $       NL_ACT = NL_ACT - COUNT( GRID_MARK(:) )
c     
        IF( Vec_Sca == 'vect_rte' ) THEN
           WRITE(OutStr,'(A,F5.2,3(1x,F5.2),A)')
     $          '    with Stokes vector  :  |',
     $          Finc(1,1:2), Finc(2,3:4),'|'
           Call WriteOut (Unt, OutStr)
        ENDIF
c     
        Call WriteOut (Unt,' ========================')
        Call WriteOut (Unt,'       ATMOSPHERE       :'//
     $       '   Top [km]     Bottom [km]   Number of layers')
        WRITE(OutStr,'(A,2x,F8.4,6x,F8.4,14x,I3)') 
     $       '       ----------        ', ZZZ(1), ZZZ(NLAYERS), NL_ACT
        Call WriteOut (Unt, OutStr)
        SELECT CASE (TRIM(RTM_TYPE))
        CASE('ps_scat')
           S_P = 'pseudo-spherical'
        CASE('spher_scat')
           S_P = 'approximately spherical'
        CASE DEFAULT
           S_P = 'plane-parallel'
        END SELECT
        WRITE(OutStr,'(6x,A,2x,A)') '       Sphericity :',TRIM(S_P)
        Call WriteOut (Unt, OutStr)
        WRITE(OutStr,'(6x,A,2x,A)') '         Rayleigh :', TRIM(TXT_R)
        Call WriteOut (Unt, OutStr)
        WRITE(OutStr,'(6x,A,2x,A)') '          Aerosol :', TRIM(TXT_A)
        Call WriteOut (Unt, OutStr)
        WRITE(OutStr,'(6x,A,2x,A)') '           Clouds :', TRIM(TXT_C)
        Call WriteOut (Unt, OutStr)
        WRITE(OutStr,'(6x,A,2x,A)') ' Gaseous absorber :', TRIM(TXT_G)
        Call WriteOut (Unt, OutStr)
        On_Off = 'off'
        IF( TRIM(RTM_MODE)=='raman' ) THEN
           On_Off = ' on'
           IF( TRIM(SCATT_MODE) == 'ss' ) THEN
              TDUM = On_Off 
           ELSE
              IF( RRS_DO  ) 
     $             TDUM = On_Off//' (discrete-ordinates technique)'
              IF( RRS_ADJ )
     $             TDUM = On_Off//' (forward-adjoint technique)'
           ENDIF
           WRITE(OutStr,'(6x,A,2x,A)') ' Rotational Raman :',TRIM(TDUM)
           Call WriteOut (Unt, OutStr)

           SELECT CASE( TRIM(APPR_RRS) )
           CASE('exact matching')  
              TDUM = '    "exact matching" approach was selected'
           CASE('simple binning')  
              TDUM = '    "simple binning" approach was selected'
           END SELECT
           WRITE(OutStr,'(6x,A,2x,A)') '                  :',TRIM(TDUM)
           Call WriteOut (Unt, OutStr)
        ELSE
           WRITE(OutStr,'(6x,A,2x,A)') ' Rotational Raman :',On_Off
           Call WriteOut (Unt, OutStr)
        ENDIF
c
        Call WriteOut (Unt, ' ========================')
        Call WriteOut (Unt, '       SURFACE          :  '//TRIM(TXT_S))
        Call WriteOut (Unt, '       -------            ')
        IF( UWT )  THEN 
           WRITE(OutStr,'(6x,A,1x,A)')   
     $          '         Coupling :', TRIM(TXT_S1)
           Call WriteOut (Unt, OutStr)
           IF( SPECULAR ) THEN
              TXT_S1 = 'absolutely flat'
           ELSE
              TXT_S1 = 'wind-roughened'
           ENDIF
        ENDIF

        IF( .NOT.BRDF_FLAG .AND. .NOT.UWT) THEN
           IF( DO_SPECTRAL_ALBEDO ) THEN
              WRITE(OutStr,'(4x,A,2x,A)')
     $          '                    :', 'Spectral dependent albedo'
              Call WriteOut (Unt, OutStr)
           ELSE
              WRITE(OutStr,'(4x,A,2x,F5.2)')
     $          '    Constant albedo :', ALBEDO_CONST
              Call WriteOut (Unt, OutStr)
           ENDIF
        ENDIF

c       ... Surface BRDF models
        SELECT CASE( TRIM(SURF_TYPE) )
        CASE('ocean')
           IF( UIT ) THEN 
              O_I = '  Air-ice interface :'
           ELSE
              O_I = 'Air-water interface :'
           ENDIF
           IF( SPECULAR ) THEN
              TXT_S1 = 'absolutely flat'
              WRITE(OutStr,'(4x,A,2x,A)') TRIM(O_I), TRIM(TXT_S1)
              Call WriteOut (Unt, OutStr)
           ELSE
              TXT_S1 = 'wind-roughened'
              WRITE(OutStr,'(4x,A,2x,A,2x,A,A)') TRIM(O_I),  
     $             TRIM(TXT_S1),TRIM(SS_PDF),' slope PDF'
              Call WriteOut (Unt, OutStr)
           ENDIF

           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('mRPV_facet')
c           WRITE(IUNIT,'(A)') 
c     $          '  mRPV_facet BRDF model :'
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('melt_pond')
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Water depth [cm] :', MP_gp(2)*100.
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '     Ice thickness [cm] :', MP_gp(1)*100.
           Call WriteOut (Unt, OutStr)           
           WRITE(OutStr,'(A)') 
     $          ' Ice transport scattering '
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '      coefficient [m-1] :', Sig_t
           Call WriteOut (Unt, OutStr)
           
c       ... Ross-Li BRDF model
        CASE('Ross-Li')
           Call WriteOut (Unt,'     Ross-Li BRDF model :')
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('Ross-Li-snow')
           Call WriteOut (Unt,'     Ross-Li-snow BRDF model :')
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('snow_fasmar')
           Call WriteOut (Unt,'          snow_fasmar BRDF model :')
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('svk')
           CALL LIN_SEARCH_NEW(WL_START, DC, Ws, Ws1, Nwl_veg, WL_veg)
           Aveg = (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
           WRITE(OutStr,'(A,2x,f5.1)') 
     $          '  Percent of vegetation :',Cfr_v*100.
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,2x,f5.2)') 
     $          '                   NDVI :',NDVI
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE('rff')
           WRITE(OutStr,'(A,2x,A)') 
     $          '         BRDF from file :', TRIM(INP_BRDF)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,3x,10(1x,f8.2))') 
     $          '       Wavelengths [nm] :', (Sph_WL(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(A,4x,10(1x,f7.3))') 
     $          '       Spherical albedo :', (Sph_OUT(I),I=1,N_Sph_WL)
           Call WriteOut (Unt, OutStr)

        CASE DEFAULT
           IF( BRDF_FLAG ) THEN
              WRITE(OutStr,'(A,A)') 
     $             '         RPV BRDF model :', 
     $             '  Spherical albedo is wavelength independent'
              Call WriteOut (Unt, OutStr)
              WRITE(OutStr,'(A,4x,1x,f7.3)') 
     $             '       Spherical albedo :', Sph_OUT(1)
              Call WriteOut (Unt, OutStr)
           ENDIF
        END SELECT

        IF( UWT ) THEN
           IF( UIT ) THEN
              WRITE(OutStr,'(4x,A,2x,A)')
     $             '  Air-ice interface :', TRIM(TXT_S1)
              Call WriteOut (Unt, OutStr)
           ELSE
              WRITE(OutStr,'(4x,A,2x,A)')
     $             'Air-water interface :', TRIM(TXT_S1)
              Call WriteOut (Unt, OutStr)
           ENDIF
           IF( .NOT.SPECULAR ) THEN
              SELECT CASE (TRIM(A_RTmat))
              CASE('NT_ICF')
                 WRITE(OutStr,'(3x,20A,1x,A,2x,A)') (' ', I=1,20),':',
     $                '  Nakagjima and Tanaka R & T matrixes'//
     $                ' including correction factors'
              CASE('FT')
                 WRITE(OutStr,'(3x,20A,1x,A,2x,A)') (' ', I=1,20),':',
     $              '  Transmission matrixes ignoring roughness'
              CASE('NT_ECF')
                 WRITE(OutStr,'(3x,20A,1x,A,2x,A)') (' ', I=1,20),':',
     $                '  Nakagjima and Tanaka R & T matrixes'//
     $                ' excluding correction factors'
              END SELECT
              Call WriteOut (Unt, OutStr)
              TXT_S1 = 'off'
              IF( USE_SHAD=='yes' ) TXT_S1 = 'on'
              WRITE(OutStr,'(4x,A,2x,A)')
     $             '   Shadowing factor :', TRIM(TXT_S1)
              Call WriteOut (Unt, OutStr)
           ENDIF

           NL_ACT = NL_UW-1
           IF( ARB_OL .AND. RTM_CORE == 'DOM' ) 
     $          NL_ACT = NL_ACT - COUNT( GRID_MARK_UW(:) )
           
c           WRITE(*,*) '  NL_UW-1 and NL_ACT ',NL_UW-1, NL_ACT

           Call WriteOut (Unt, ' ========================')
           IF( UIT ) THEN
              Call WriteOut (Unt,  '         ICE            :'//
     $             '   Top  [m]     Bottom  [m]   Number of layers')
           ELSE
              Call WriteOut (Unt,  '        OCEAN           :'//
     $             '   Top  [m]     Bottom  [m]   Number of layers')
           ENDIF
           WRITE(OutStr,'(A,2x,F8.4,6x,F8.4,14x,I3)')  
     $          '        -----            ', ZZZ_UW(1), ZZZ_UW(NL_UW), 
     $          NL_ACT
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(6x,A,2x,A)') 
     $          '         Rayleigh :',TRIM(TXT_Rw)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(6x,A,2x,A)') 
     $          '         Hydrosol :',TRIM(TXT_Aw)
           Call WriteOut (Unt, OutStr)
           WRITE(OutStr,'(6x,A,2x,A)') 
     $          '         Absorber :',TRIM(TXT_Gw)
           Call WriteOut (Unt, OutStr)
           On_Off = 'off'
           IF( INCL_INEL(1) ) On_Off = ' on'
           WRITE(OutStr,'(6x,A,2x,A)') 'Vibrational Raman :',On_Off
           Call WriteOut (Unt, OutStr)
           On_Off = 'off'
           IF( INCL_INEL(2) ) On_Off = ' on'
           WRITE(OutStr,'(5x,A,2x,A)') 'Fluorescence Chl-a :',On_Off
           Call WriteOut (Unt, OutStr)
           On_Off = 'off'
           IF( INCL_INEL(3) ) On_Off = ' on'
           WRITE(OutStr,'(5x,A,2x,A)') 'Fluorescence  CDOM :',On_Off
           Call WriteOut (Unt, OutStr)

           WRITE(OutStr,'(A,2x,A,1x,F5.2)') 
     $          '      Bottom reflection :',TRIM(TXT_OB), ALB_BOTTOM
           Call WriteOut (Unt, OutStr)
c           WRITE(*,*) '       -----------     Ku-Ku '
        ENDIF
c
        Call WriteOut (Unt, ' ========================')
        Call WriteOut (Unt, '   OBSERVATION GEOMETRY :  '//TRIM(TXT_O))
        SELECT CASE(Top_Bot)
        CASE('top')
           WRITE(OutStr,'(5x,A,1x,A)') 'at the top of ',ATM_OC
           Call WriteOut (Unt, '   --------------------  '//OutStr)
        CASE('bot')
           WRITE(OutStr,'(5x,A,1x,A)') 'at the bottom of ',ATM_OC
           Call WriteOut (Unt, '   --------------------  '//OutStr)
        CASE('flu','spa')
           Call WriteOut (Unt, '   --------------------  ')
        CASE DEFAULT
           WRITE(N_STR,'(I0)') N_OP
           WRITE(OutStr,'(5x,A,1x)') TRIM(TEXT)
           Call WriteOut (Unt, '   --------------------  '//OutStr)
           O_F = '(25x,'//TRIM(N_STR)//'(1x,F9.3))'
           WRITE(OutStr,FMT=O_F) (ALTITUDE(I),I=1,N_OP)
           Call WriteOut (Unt, OutStr)
        END SELECT

c           WRITE(*,*) '       -----------     Ku-Ku 2'

c          ... Line-of-sight angles output
c          -------------------------------
        IF( TRIM(RTM_MODE) == 'flux' .OR.
     $      TRIM(RTM_MODE) == 'spher_alb' ) THEN
        ELSE
           IF( LOS_DEF_FLAG == 'ga' ) THEN
              WRITE(OutStr,'(27x,A,1x,F7.3,A)') 
     $             'and Gauss angles used as line-of-sight'
              Call WriteOut (Unt, OutStr)
           ELSE
              WRITE(OutStr,'(27x,A,1x,F7.3,A)') 
     $             'and user defined line-of-sight angles'
              Call WriteOut (Unt, OutStr)
           ENDIF
        ENDIF

c           WRITE(*,*) '       -----------     Ku-Ku 3'

c       ... Scattering treatment
        SELECT CASE( TRIM(SCATT_MODE) )
        CASE('ms')
           TXT_O = 'full multiple scattering' 
        CASE('ss')
           TXT_O = 'single scattering approximation' 
        CASE('dir_ref')
           TXT_O = 'direct reflected solar light ignoring scattering' 
        END SELECT

        Call WriteOut (Unt, ' ========================')
        Call WriteOut (Unt, '   SCATTERING TREATMENT :  '//TRIM(TXT_O))

        On_Off = 'on'
        IF( Nvs==1 ) On_Off = 'off'
        Call WriteOut (Unt, '           Polarization :  '//On_Off)

c          ... Specific options
c          --------------------
        TXT_A1 = 'Adaptive grid  Single scattering   Analytic for'
        TXT_A2 = '                  correction      pure scattering'
        Call WriteOut (Unt,' ========================')
        Call WriteOut (Unt,'    ADDITIONAL FEATURES  :  '//TRIM(TXT_A1))
        Call WriteOut (Unt,'                           '//TRIM(TXT_A2))
        WRITE(OutStr,'(A,6x,A,14x,A,16x,A)') 
     $       '   --------------------  ', T_AG, T_SS, T_AN
        Call WriteOut (Unt, OutStr)

        IF( TRIM(RTM_MODE)=='int'.OR.TRIM(RTM_MODE)=='raman' ) THEN
           IF( INT_OUT ) THEN
              TEXT = 'Angular interpolation of standard DO solution'
           ELSE
              TEXT = 'Integration of source function'
           ENDIF
           IF( LOS_DEF_FLAG == 'ga' ) THEN
              TEXT = 'Not used'
           ENDIF
           Call WriteOut (Unt,'   Post-processing type :  '//TRIM(TEXT))
        ENDIF

        IF( .NOT.DO_AZIMUTH ) THEN
           Call WriteOut (Unt, '                         : '//
     $          "Only zeroth harmonic has been accounted for !!!")
        ENDIF
        WRITE(OutStr,'(1x,24A)') ('=',I=1,24)
c     $       ' ========================'
c        /\
c        ||
        Call WriteOut (Unt, OutStr)

        CALL WRITE_UD(2,Unt)
        Call CloseOut (Unt)

      CALL ST_TRACK (R_NAME, 3, 'O')  
      RETURN
      END
C********************************************************************C
