!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 13 Feb 2019
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE PAO_AER_SETUP(START_WAV, END_WAV, *)
  ! Called by "rtm_init.f"
                              
!  USE AER_MIE,   ONLY: DENS_PRF
  USE AEROSOLS,  ONLY: AER_SHIFT, PATH_TO_SCM, OUT_A_SCM, LAY_A, WEL_A, NSCA_A, &
       GE_A, APH_VS, AER_WAVS, NAW => N_AER_WAVS, Greek_Aer, AER_COEFF, Scal_aer, &
       SCI => GT_SCAT_IDX, NG_aer, ASY => GT_ASYM_IDX, Rtau_aer, L_scal, ZS1, &
       EXI => GT_EXTC_IDX, Rwl_aer, ZS2, Plus_mi, NWLS, NWLE, &
       AER_SCALING, Nga_db, LA_N, LA_S, LA_E, L_TOP, IDX_AER, LA_Ndb, AOT_out_VAL, &
       AER_FACT, V_PAP, IDX_AER_db, SHARP_EDGES, ZA_SE
  USE CLOUDS,    ONLY: DO_CLOUD  
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE DELTA_M,   ONLY: FDM_AER, AER_DM_FLG, D_MF_AER, AER_TRUNC_IDX
  USE GRIDS,     ONLY: NLAYERS, ZHOLD => ZZZ, DZ
  USE GT_POLYN,  ONLY: NLEGEN
  USE INFO,      ONLY: AER_INFO
  USE I_DIMS,    ONLY: Nm_vs
  USE LOG_PARS,  ONLY: Work_Dir, LOG_LEVEL, LUNIT, EU, DIR_OUT, N_SSO, N_ESO
  USE RTM_PARS,  ONLY: DATA_ITER
  USE ST_VARS,   ONLY: RAD_TO_DEG, Vec_Sca, RTM_TYPE, GROUND_HEIGHT
  USE ST_INPUT,  ONLY: APH_TYPE, ADB => PATH_AER_DB
  USE TOOLS,     ONLY: READ_INPUT, CHK_STAT, SET_PTR, CUT_STR, SPLIT_STRING
  USE TYPES,     ONLY: KND, DYN_CHAR
  
  REAL (KND), INTENT(IN) :: START_WAV, END_WAV
  ! Local parameters
  LOGICAL            ERROR
  CHARACTER (LEN = 13), PARAMETER :: R_NAME = 'PAO_AER_SETUP'
  !  (a) Input variables
  !      AER_DM_FLG        : delta-M approximation yes/no 
  !                          <---- From Module DELTA_M
  !      AER_TRUNC_IDX     : truncation index for delta-M approximation 
  !                          <---- From Module DELTA_M
  !      ZHOLD             : forward model altitude grid
  !      NLAYERS           : number of layers in altitude grid
  !      NLEGEN            : number of Legendre moments
  !      LUNIT             : LOG file unit number
  
  !  (b) Output variables
  !      AER_COEFF         : scattering / extinction coefficients and 
  !                          asymmetry parameters 
  !      AER_PHASMOMS      : Legendre moments
  !      NAW               : number of wavelengths of chosen aerosol 
  !                          parameterization 
  !      AER_WAVS          : wavelengths of aerosol data base (depends on
  ! parameterization chosen)
  INTEGER  NA_LEV
  REAL (KND)  DUM, Scoe, DD1, DD2
  INTEGER, ALLOCATABLE :: IDX_W(:)
  REAL (KND), ALLOCATABLE :: USER_MOM(:,:,:,:), DENS_PRF(:), TAU(:), &
       CEXT(:,:), CSCA(:,:), ASYM_PAR(:,:), AER_LEV(:), AER_PRF(:)
  CHARACTER (LEN=15), ALLOCATABLE :: Type_INP(:)
  CHARACTER (LEN=200), ALLOCATABLE :: F_name(:)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, DIR_name
  INTEGER, ALLOCATABLE :: LAS(:), LAE(:), IDA(:), N_MOMS(:)
  REAL (KND), POINTER :: AER_PHASMOMS(:,:,:)
  REAL (KND), ALLOCATABLE :: DATA_R(:,:)
  INTEGER  I, I1, J, K, L, DUNIT, K1, Itr, LAN, N_COLS, NM_MAX, MAX_MOM
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), STR_ARR_1(:) 
  ! Variable for Delta-M
  ! ====================
  REAL (KND)  F_DM, AER_ABS_COEFF
  !=========================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
      
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
  ALLOCATE( DENS_PRF(NLAYERS) )
  ERROR = .FALSE.

  IF( DO_CLOUD.AND.SHARP_EDGES ) GOTO 138

  STR_INP = 'Directory name for aerosol optical parameters'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  DIR_name = CUT_STR(READ_ARR(1)%S)

  STR_INP = 'Filenames of aerosol optical parameters'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  NAW = SIZE(STR_ARR)

  ALLOCATE( F_name(NAW) )
  DO I = 1, NAW
     F_name(I) = DIR_name//CUT_STR(STR_ARR(I)%S)
!     WRITE(*,'(A)')  TRIM( F_name(I) )
  ENDDO
  
  NWLS = 1 ; NWLE = NAW
  NM_MAX = 1000
!  WRITE(*,*) '    Number of wavelengths ', NAW

  ALLOCATE( AER_WAVS(NAW), FDM_AER(NLAYERS,NAW), IDA(NLAYERS), & !A
       AER_COEFF(3,NLAYERS,NAW), TAU(NAW), N_MOMS(NAW), ASYM_PAR(1,NAW), & !A
       CEXT(1,NAW), CSCA(1,NAW), USER_MOM(NAW,NM_MAX,1,Nm_vs) )  ! A
  FDM_AER = 0.d0 ; AER_COEFF = 0.d0 ; USER_MOM   = 0.d0 
  
  N_COLS = 7
  ! Reading   AER_WAVS, CEXT, CSCA, and USER_MOM
  DO I = 1, NAW
!     write(6,*) I, F_name(I)
     CALL READ_AER_POP (N_COLS, F_name(I), DATA_R, *880)
!      write(6,*) '------>', I
     N_MOMS(I) = SIZE( DATA_R(:,1)  )
!     WRITE(*,*) '   N_MOMS = ', N_MOMS(I)
     IF( N_MOMS(I) > NM_MAX ) GOTO 125
     USER_MOM(I,1:N_MOMS(I),1,1:Nm_vs) = DATA_R(:,2:Nm_vs+1)
     ASYM_PAR(1,I) = DATA_R(2,2)/3.d0
  ENDDO
  IF( Nm_vs > 1 ) THEN
     USER_MOM(:,:,:,5) = -USER_MOM(:,:,:,5)
     USER_MOM(:,:,:,6) = -USER_MOM(:,:,:,6)
  ENDIF

!  DO I = 1, N_MOMS(1)
!     WRITE(*,*) I, USER_MOM(1,I,1,1:Nm_vs)
!  ENDDO

  IF( AER_WAVS(NAW) < END_WAV ) GOTO 130  
  IF( START_WAV < AER_WAVS(1) ) THEN
     NWLS = 1 
     CALL WRITE_MESSAGE(R_NAME,'WARNING',' ')
     WRITE(EU,'(A)')  '    Required wavelength range is outside of the'//&
          ' aerosol data base.'
     WRITE(EU,'(A,2x,F9.2)') '    First wavelength in data base:', AER_WAVS(1)
     WRITE(EU,'(A,2x,F9.2)') '    First required wavelength    :', START_WAV
     WRITE(EU,*) '  !!! The extrapolation with constant is used !!!'
  ENDIF


  !=========================================================
  !   Read user-defined aerosol scenario input parameters
  !=========================================================
     STR_INP  = 'PAO aerosol layers'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     LAN = SIZE(READ_ARR)

     ALLOCATE( LAS(LAN), LAE(LAN), L_TOP(LAN), Type_INP(LAN), & !A
          IDX_W(LAN), APH_VS( NLEGEN,LAN,Nm_vs,NAW ) )    !A
     APH_VS = 0.0d0  
     AER_PHASMOMS => APH_VS( :,:,1,: )
!    ... Reading top altitude of aerosol layer
     DO J = 1, LAN
        READ(READ_ARR(J)%S,*,ERR=101) L_TOP(J)
     END DO

     IF( AER_SHIFT ) L_TOP(:) = L_TOP(:) + GROUND_HEIGHT

     IF( LAN > 1 ) THEN
        IF( L_Top(LAN-1) > ZHOLD(1) ) GOTO 131
     ELSE
        IF( L_Top(LAN) > ZHOLD(1) ) GOTO 133
     ENDIF
     IF( L_Top(1) <= ZHOLD(NLAYERS) ) GOTO 132

     STR_INP  = 'PAO aerosol component'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     ! Type_INP(:) bottom --> top
     DO J = 1, LAN
        Type_INP(J) = CUT_STR(READ_ARR(J)%S)
     END DO
     
  STR_INP  = 'Aerosol particle number density (PAO)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  NA_LEV = SIZE(READ_ARR)
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(NA_LEV), AER_PRF(Na_LEV) ) ! A
  DO I = 1, Na_LEV
     READ( READ_ARR(I)%S,* ) AER_LEV(I), AER_PRF(I)
  END DO

  AER_PRF(:) = AER_SCALING*AER_PRF(:)

  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT

  IF( SHARP_EDGES ) AER_LEV(:) = ZA_SE(:)

  ! Interpolate ext. coeff. at the standard altitude grid
!  write(*,*) '   AER_LEV ', AER_LEV
!  write(*,*) '   ZHOLD ', ZHOLD
  CALL LIN_INT(Na_LEV, AER_LEV, AER_PRF, NLAYERS, ZHOLD, DENS_PRF, 'Z')
  
  CALL AER_LAYERING

  ! Scaling of aerosol number density profile if needed
  IF( L_scal ) THEN
     WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 )  &
          DENS_PRF(:) = AER_FACT*DENS_PRF(:)
  ENDIF

  DO J = 1,LAN
     DO K1 = LAS(J), LAE(J)
        I1 = 1
        DO K = 1, NAW
!              IF( K==1 ) THEN
!                 WRITE(*,*) ZHOLD(K1), DENS_PRF(K1)
!              ENDIF
           AER_COEFF(EXI,K1,I1) = CEXT(J,K)*DENS_PRF(K1)
           AER_COEFF(SCI,K1,I1) = CSCA(J,K)*DENS_PRF(K1)
           I1 = I1 + 1
        ENDDO
     ENDDO
  ENDDO

  ! Scaling of the scattering coefficient between ZS1 - ZS2
  ! in order to change the single scattering albedo as follows:
  !    SSA' = 1 - V_PAP*(1 - SSA) 
  ! Extinction coefficient remains the same
  IF( L_scal ) THEN
     DO I = 1, NAW
        WHERE( ZHOLD(:) >= ZS1.AND.ZHOLD(:) <= ZS2 ) &
             AER_COEFF(SCI,:,I) = AER_COEFF(SCI,:,I) - (V_PAP - 1.d0) &
             *(AER_COEFF(EXI,:,I ) - AER_COEFF(SCI,:,I ))
     ENDDO
  ENDIF

  MAX_MOM = MAXVAL( N_MOMS(:) )
  IF (LOG_LEVEL(1) >= N_ESO) THEN
     WRITE(LUNIT,*) '   Max number of expansion coeff. :', MAX_MOM
     WRITE(LUNIT,*) '  ----------------------------------'
  ENDIF

  ! Single scattering correction algorithm 
  ! --------------------------------------
  IF( DO_SSRCA .OR. TRIM(RTM_TYPE) == 'spher_scat' ) THEN
     Nga_db = MAX_MOM 
     ALLOCATE( Greek_aer( Nga_db,LAN,Nm_vs,NAW ) )  ;  Greek_aer = 0.d0
     DO I = 1,NAW 
        DO K = 1,Nm_vs
           DO L = 1, LAN
              Greek_aer(:,L,K,I) = USER_MOM(I,1:Nga_db,L,K)
           ENDDO
        ENDDO
     ENDDO
     IF( .NOT.Plus_mi ) NG_aer = Nga_db
  ENDIF

  IF( OUT_A_SCM ) THEN 
     IF( LAY_A > LAN ) GOTO 135
     IF( WEL_A > NAW )  GOTO 136
     CALL COEF_TO_SCATMATR(MAX_MOM, Nm_vs, &
          USER_MOM(WEL_A,1:MAX_MOM,LAY_A,1:Nm_vs), NSCA_A, GE_A, PATH_TO_SCM)
  ENDIF

  DO I = 1,LAN 
     DO J = LAS(I),LAE(I)
        AER_COEFF(ASY,J,:) = ASYM_PAR(I,:)
     ENDDO
  ENDDO

  ! Aerosol optical thickness according to input N(z)
  DO K = 1,NAW
     TAU(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ENDDO

  ! Scaling input N(z) according to required AOD
  IF( Scal_aer ) THEN
     CALL LIN_SEARCH(Rwl_aer, DUM, K, NAW, AER_WAVS, ERROR)
     IF(ERROR) GOTO 129
     DD1 = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)
     IF( DD1 > 0.d0 ) THEN
        Scoe = Rtau_aer/DD1
     ELSE
        Scoe = 0.d0
     ENDIF
     AER_COEFF(EXI,:,:) = Scoe*AER_COEFF(EXI,:,:)
     AER_COEFF(SCI,:,:) = Scoe*AER_COEFF(SCI,:,:)
     TAU(:) = Scoe*TAU(:)
  ENDIF
  AOT_out_VAL = TAU(1)

  ! WRITE(*,*) '   =====> Inp AOT at reference wl '
  ! DO K = 1,NAW
  !    WRITE(*,'(2x,I2,2x,F8.3,2x,E12.4)') K, AER_WAVS(K),&
  !        SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
  ! ENDDO
       
  !  Write aerosol scenario output file
  CALL WRITE_SCEN_PAO(*880)

  !Allocate array for MODULE DELTA_M
  !=================================
  IF ( AER_DM_FLG ) THEN ;  Itr = AER_TRUNC_IDX
     SELECT CASE( Vec_Sca )
     CASE('vect_rte')  ;  CALL AER_DM_VEC
     CASE('scal_rte')  ;  CALL AER_DM
     END SELECT
     !"Renormalize" scattering and extinction coefficients
     !====================================================
     DO I = 1, NLAYERS
        DO K = 1,NAW
           IF( AER_COEFF(EXI,I,K) == 0.d0 ) FDM_AER(I,K) = 0.d0
           F_DM = FDM_AER(I,K)
           AER_ABS_COEFF = AER_COEFF(EXI,I,K ) - AER_COEFF(SCI,I,K)
           AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0-F_DM)
           AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + AER_ABS_COEFF 
        ENDDO
     ENDDO
  ELSE
     DO I1 = 1,LAN
        DO K = 1, NAW
           APH_VS(:,I1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
        ENDDO
     ENDDO
     ! DO I1 = 1,LAN
     !    DO K1 = LAS(I1), LAE(I1)
     !       DO K = 1, NAW
     !          APH_VS(:,K1,:,K) = USER_MOM(K,1:NLEGEN,I1,:)
     !       ENDDO
     !    ENDDO
     ! ENDDO
  ENDIF

  LA_Ndb = LAN
  IF( .NOT.Plus_mi ) THEN
     LA_N = LAN
     ALLOCATE( LA_S(LAN), LA_E(LAN), IDX_AER(NLAYERS) )
     LA_S = LAS ;  LA_E = LAE  ; IDX_AER = IDA
  ELSE
     ALLOCATE( IDX_AER_db(NLAYERS) )
     IDX_AER_db = IDA
  ENDIF

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP 
  RETURN 1

126 CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error in LIN_SEARCH')
  RETURN 1
  
127 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Aerosol models name not found')
  RETURN 1

129 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Reference wavelength for AOT is out of range')
  WRITE(EU,'(A,F7.2)') '    Reference wavelength: ', Rwl_aer
  WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
  WRITE(EU,'(A)') '    Set reference wavelength between database wavelengths.'
  RETURN 1

125 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Number of expansion coefficient in file')
  WRITE(EU,'(A,I4)') '    Number of coefficient: ', N_MOMS(I)
  WRITE(EU,'(A,I4)') '    Maximal dimension: ', NM_MAX
  WRITE(EU,'(A)') '    Increase NM_MAX in the file "pao_aer_setup.f90".'
  RETURN 1

130 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Required wavelength range is outside of the data base.')
  WRITE(EU,'(A,2x,F9.2)') '    Last wavelength in data base:', AER_WAVS(NAW)
  WRITE(EU,'(A,2x,F9.2)') '    Last required wavelength    :', END_WAV
  WRITE(EU,*) '    Aerosol data base does not exist '
  WRITE(EU,*) '    for wavelengths > 40 micron.'
  RETURN 1

131 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Upper aerosol layer outside atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') & 
       '    Upper aerosol layer:', L_TOP(LAN-1),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

132 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Lower aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F6.2,A)') '       Top of aerosol layer:', L_TOP(1),' km'
  WRITE(EU,'(A,2x,F6.2,A)') '       Surface elevation   :', ZHOLD(NLAYERS),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and "Height above '
  WRITE(EU,*) '    sea level" in the file "control.inp" '
  RETURN 1

133 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' Aerosol layer outside the atmosphere')
  WRITE(EU,'(A,2x,F9.2,A,F9.2)') '    Aerosol layer:', ZHOLD(NLAYERS),' - ',L_TOP(LAN)
  WRITE(EU,'(A,2x,F9.2,A)') '      Top of atmosphere:', ZHOLD(1),' km'
  WRITE(EU,*) '    Check "pao_aer.inp" and altitude grid '
  WRITE(EU,*) '    as defined in the file "control.inp" '
  RETURN 1

135 CALL WRITE_MESSAGE (R_NAME,'ERROR', &
         'Required number of aerosol layer is out of range')
  WRITE(EU,'(A,I3)') '    Number of aerosol layers: ', LAN
  WRITE(EU,'(A,I3)') '    Required layer number   : ', LAY_A
  RETURN 1

136 CALL WRITE_MESSAGE (R_NAME,'ERROR',&
         'Required number of wavelength is out of range')
  WRITE(EU,'(A,I3)') '    Number of wavelengths for aerosol: ',NAW
  WRITE(EU,'(A,I3)') '    Required wavelength number       : ',WEL_A
  RETURN 1

138 CALL WRITE_MESSAGE (R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') '    Option "PAO aerosol as a cloud layer?"'
  WRITE(EU,'(A)') '    can be used in the cloud-free case only.'
  WRITE(EU,'(A)') '    Check input file "pao_aer.inp".'
  RETURN 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
  
CONTAINS
!***********************************************************************

  SUBROUTINE READ_AER_POP (N_COLS, FileName, DATA_R, *)
    
    USE LOG_PARS,    ONLY: LOG_LEVEL, N_SSO, EU, LUNIT
    USE TYPE_VARS,   ONLY: KND

    INTEGER    N_COLS
    CHARACTER (LEN = *) FileName
    REAL (KND), ALLOCATABLE :: DATA_R(:,:)
    ! Local variables:
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_AER_POP'
    CHARACTER (LEN = 1)  HEADERLINE
    CHARACTER (LEN = :), ALLOCATABLE :: File
    CHARACTER (LEN = 200) Line
    INTEGER  N_READ, II, C_UNIT, NCL
    Logical  ChkFlg
    REAL (KND) Lamb
    REAL (KND), ALLOCATABLE :: ARR_READ(:)
    TYPE Linked_List_Entry
       REAL (KND), ALLOCATABLE :: ARR(:)
       TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
    END TYPE Linked_List_Entry
    TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), STR_ARR_1(:) 
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')

    ! N_COLS is always the number of columns in the DATA_R array
    NCL = N_COLS
    ALLOCATE(ARR_READ(NCL))
    IF(ALLOCATED(DATA_R)) DEALLOCATE (DATA_R)

    File = FileName
    IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,'(A)') ' *** Reading: '//TRIM(File) 

    OPEN(NewUNIT=C_UNIT, FILE = FILE, STATUS = 'old')

    HEADERLINE = '#'
    DO WHILE (ANY(HEADERLINE == [';', '#', '*', '>']))
       READ(C_UNIT,*,END=111) HEADERLINE
    END DO
    BACKSPACE(C_UNIT) 

    N_READ  = 0 ; ChkFlg = .True.
    Read_Data: DO 
       Read_Line: Block
         If(.Not. ChkFlg) Exit Read_Line
         READ(C_UNIT,'(A)', END=101) LINE
         If (LEN_TRIM(LINE) == 0) CYCLE
         DO K = 1,LEN_TRIM( LINE )
            SELECT CASE ( IACHAR( LINE(K:K) ) )
            CASE( 32,48:57,43,45,46,68,69,100,101 )
            CASE DEFAULT 
               GOTO 100
            END SELECT
         ENDDO
         Backspace(C_UNIT)
         ChkFlg = .False.
         Exit Read_Line
100      Continue
         SELECT CASE( Line(1:3) )
         CASE('LAM')
            CALL SPLIT_STRING(Line, STR_ARR)
            CALL SPLIT_STRING(STR_ARR(1)%S, STR_ARR_1, '=')
            READ(STR_ARR_1(2)%S,*,ERR=113) Lamb
            AER_WAVS(I) = Lamb*1.d+3  ! transform to [nm]
            WRITE(*,*) ' Lamb =  ', AER_WAVS(I) 
            CYCLE
          CASE('CEX')
            CALL SPLIT_STRING(Line, STR_ARR)
            CALL SPLIT_STRING(STR_ARR(1)%S, STR_ARR_1, '=')
            READ(STR_ARR_1(2)%S,*,ERR=113) CEXT(1,I)   ! Cext
            CALL SPLIT_STRING(STR_ARR(2)%S, STR_ARR_1, '=')
            READ(STR_ARR_1(2)%S,*,ERR=113) CSCA(1,I)   ! Csca
            WRITE(*,*) ' Cext, Csca =  ', CEXT(1,I), CSCA(1,I)
            CYCLE
         CASE DEFAULT
            CYCLE
         END SELECT
101      Continue
         EXIT Read_Data
       END Block Read_Line

       READ(C_UNIT,*,ERR=102,END=102) ARR_READ
       
       IF (.NOT.ASSOCIATED(ELEMENT)) THEN
          ALLOCATE(FIRST_ELEM)
          ELEMENT => FIRST_ELEM
          N_READ = 1
       ELSE
          ALLOCATE(ELEMENT%NEXT_ELEM)
          ELEMENT => ELEMENT%NEXT_ELEM
          N_READ = N_READ + 1
       ENDIF
       
       ALLOCATE (ELEMENT%ARR(NCL))
       ELEMENT%ARR(:) = ARR_READ(:)
       CYCLE
    END DO Read_Data
102 Continue
    CLOSE (C_UNIT)

    IF (.NOT.ASSOCIATED(ELEMENT)) GoTo 114 

    NULLIFY(ELEMENT%NEXT_ELEM)
    ALLOCATE(DATA_R(N_READ, N_COLS))
    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       II = II + 1
       DATA_R(II,:) = ELEMENT%ARR(:)
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT%ARR)
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO

    DO II = 1, 10
       WRITE(*,*) DATA_R(II,1), DATA_R(II,2), DATA_R(II,3)
    ENDDO
!    pause
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN

111 CONTINUE
    CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'End of file encountered')
    WRITE(EU,*) '    File: '//FILE
    RETURN 1
    
113 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file')
    WRITE(EU,*) '    File: '//FILE
    RETURN 1
    
114 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'No valid data is found')
    WRITE(EU,*) '    File: '//FILE
    RETURN 1
    
  END SUBROUTINE READ_AER_POP
!***********************************************************************

  SUBROUTINE AER_LAYERING
    DO I = 1,LAN
       ! Ordered from the bottom to the top 
       LAS(I) = MINLOC( DABS(L_Top(I)-ZHOLD(:)),1 )
    ENDDO
    LAE(1) = NLAYERS
    DO I = 2,LAN ; LAE(I) = LAS(I-1)-1  ; ENDDO
       ! Indexes of altitude grid levels which are within aerosol layers
       IDA(:) = LAN
       DO I = 1,LAN - 1
          DD1 = ZHOLD(LAS(I)) ;  DD2 = ZHOLD(LAE(I))
          DO J = 1,NLAYERS
             IF( ZHOLD(J) <= DD1.AND.ZHOLD(J) >= DD2 )  IDA(J) = I 
          ENDDO
       ENDDO
       DD1 = ZHOLD(MAXVAL(LAE))
       ! Indexes of altitude grid levels which are below of lowest aerosol layer
       DO J = 1,NLAYERS
          IF( ZHOLD(J).LE.DD1 ) IDA(J) = 1
       ENDDO
       IF (LOG_LEVEL(1) >= N_ESO) THEN
          WRITE(LUNIT,*) ' ------ Test of aerosol layering PAO mode --- '
          DO I = 1,LAN
             WRITE(LUNIT,'(2x,I3,2(1x,f7.3),2x,a)') &
                  I, ZHOLD(LAS(I)), ZHOLD(LAE(I)), TRIM( Type_INP(I))
          ENDDO
       ENDIF
  END SUBROUTINE AER_LAYERING
  ! **************************************************************************
  
  SUBROUTINE AER_DM
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             AER_PHASMOMS(L,I1,K) = ( USER_MOM(K,L,I1,1) - &
                  F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    AER_PHASMOMS(Itr+1:NLEGEN,:,:) = 0.0D0
  END SUBROUTINE AER_DM
  
!***********************************************************************
  SUBROUTINE AER_DM_VEC
    DO I1 = 1,LAN
       DO K = 1, NAW
          F_DM = USER_MOM(K,Itr+1,I1,1)/DBLE(2*Itr + 1)
          FDM_AER(LAS(I1):LAE(I1),K) = F_DM
          DO L = 1, Itr
             APH_VS(L,I1,1:4,K) = ( USER_MOM(K,L,I1,1:4) - &
                  F_DM*DBLE(2*L-1) )/(1.0D0 - F_DM)
             APH_VS(L,I1,5:6,K) = USER_MOM(K,L,I1,5:6)/(1.0D0 - F_DM)
          ENDDO
       ENDDO
    ENDDO
    APH_VS(Itr+1:NLEGEN,:,:,:) = 0.0D0
    
  END SUBROUTINE AER_DM_VEC

!***********************************************************************
  SUBROUTINE  WRITE_SCEN_PAO(*)
    USE  AEROSOLS, ONLY: OWL_AER, NOWL_AER, Alph, C_a
    USE TOOLS,     ONLY: OpenOut, WriteOut, CloseOut 
 
    ! M_H               : number of humidities 
    ! M_CH              : number of aerosol components depending on humidity
    ! M_CI              : number of aerosol components independent on humidity
    INTEGER             K1, K2, J, M_CI, M_H, M_CH, Unt
    PARAMETER           (M_CI = 6, M_H = 0, M_CH = 0)
    CHARACTER(LEN=3)    PHT
    Character (Len=500)  OutStr
    REAL (KND)  SS_A, Ext, S1
    REAL (KND), ALLOCATABLE :: T_OUT(:), E_OUT(:), S_OUT(:), T_BL(:), T_UP(:), P_EXT(:,:)
!***********************************************************************
    
    IF (LOG_LEVEL(1) >= N_SSO) WRITE (lunit,*) ' *** writing: AEROSOL.OUT'

    ! Optical thickness at user defined wavelength grid
    IF( NOWL_AER > 0 ) THEN 
       K1 = LAS(1) ; K2 = LAE(1)
       ALLOCATE( T_OUT(NOWL_AER),E_OUT(NOWL_AER),S_OUT(NOWL_AER),  &
            T_BL(NOWL_AER), T_UP(NOWL_AER), P_EXT(NLAYERS,2))
       DO I = 1, NOWL_AER
          CALL LIN_SEARCH(OWL_AER(I), DUM, K, NAW, AER_WAVS, ERROR)
          IF (ERROR) GOTO 134
          T_OUT(I) = (1.d0 - DUM)*TAU(K) + DUM*TAU(K+1)          
          ! Single scattering albedo at user-defined wavelengths
          S_OUT(I) = (1.d0 - DUM)*AER_COEFF(SCI,NLAYERS-1,K) &
               + DUM*AER_COEFF(SCI,NLAYERS-1,K+1)
          ! Extinction coefficient at user-defined wavelengths
          P_EXT(:,1:2) = AER_COEFF(EXI,:,K:K+1)
          E_OUT(I) = (1.d0 - DUM)*P_EXT(NLAYERS-1,1) + DUM*P_EXT(NLAYERS-1,2)
          ! Optical thickness of the boundary layer
          S1 = ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
               *(ZHOLD(K1) - ZHOLD(K1+1)) + ( (1.d0-DUM)*P_EXT(K2,1) &
               + DUM*P_EXT(K2,2) )*(ZHOLD(K2-1) - ZHOLD(K2))
          DO J = K1+1,K2-1
             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
                  *(ZHOLD(J-1) - ZHOLD(J+1))
          ENDDO
          T_BL(I) = S1/2.d0

          ! Optical thickness of the atmosphere above boundary layer
!          S1 = ( (1.d0-DUM)*P_EXT(1,1) + DUM*P_EXT(1,2) )*(ZHOLD(1) - ZHOLD(2)) &
!               + ( (1.d0-DUM)*P_EXT(K1,1) + DUM*P_EXT(K1,2) ) &
!               *(ZHOLD(K1-1) - ZHOLD(K1))
!          DO J = 2,K1-1
!             S1 = S1 + ( (1.d0-DUM)*P_EXT(J,1) + DUM*P_EXT(J,2) ) &
!                  *(ZHOLD(J-1) - ZHOLD(J+1))
!          ENDDO
!          T_UP(I) = S1/2.d0
       ENDDO

       OPEN(NewUnit=DUNIT,FILE = DIR_OUT//'AOT.OUT')            
       WRITE(DUNIT,*) '# AOT and single scattering albedo (boundary layer) '
       WRITE(DUNIT,*) '#        at user-defined wavelengths [nm]:'
       DO I = 1, NOWL_AER
          WRITE(DUNIT,'(2x,F8.3,4(2x,E15.7))') OWL_AER(I),&
               T_BL(I), T_UP(I), T_OUT(I), S_OUT(I)/E_OUT(I)
       ENDDO
       CLOSE(DUNIT)

       ! Angstrom parameters for this aerosol model
       ! Alph = DLOG( T_OUT(1)/T_OUT(NOWL_AER) )&
       !      /DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
       ! C_a = T_OUT(1)/OWL_AER(1)**Alph
       Alph = DLOG( T_BL(1)/T_BL(NOWL_AER) )/DLOG( OWL_AER(1)/OWL_AER(NOWL_AER) )
       C_a = T_BL(1)/OWL_AER(1)**Alph
!         WRITE(*,*) '  Angstrom parameters '
!         WRITE(*,*) '  C_a  = ', C_a
!         WRITE(*,*) '  Alph = ', Alph
    ENDIF

    Call OpenOut ('SCE_AEROSOL.OUT', Unt)
    WRITE(OutStr,*) '          SETTINGS FOR PAO INPUT  '
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) '     ==================================='
    Call WriteOut (Unt, OutStr)
    WRITE(OutStr,*) ' '
    Call WriteOut (Unt, OutStr)
  
    IF( Scal_aer ) THEN
       WRITE(OutStr,'(A,F7.3,A,F9.2,A)') &
       '   !!! ATTENTION !!! AOT is set to ',Rtau_aer,' at', Rwl_aer,' nm wavelength.'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(19x,A)') '  See setting in "control_aer.inp" '
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(6x,A)')  '----------------------------------------------------'
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,*) 
       Call WriteOut (Unt, OutStr)
    ENDIF

    IF( NOWL_AER > 0 ) THEN
       WRITE ( OutStr, * ) ' AOT at user-defined wavelengths [nm]:'
       Call WriteOut (Unt, OutStr)
       DO I = 1, NOWL_AER
          WRITE(OutStr,'(12x,F7.2,4x,F8.4)') OWL_AER(I),T_OUT(I)
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDIF
    Call WriteOut (Unt, ' ')

    WRITE(OutStr,'(A27,2X,I2)') '  Number of aerosol layers:', LAN
    Call WriteOut (Unt, OutStr)
    Call WriteOut (Unt, '  ')
    DO I = 1, LAN
       WRITE(OutStr,'(A27,2X,I2)') '  Layer :                 :', I
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(A27,2X,F6.2)') '  Upper boundary [km]     :', L_TOP(I)
       Call WriteOut (Unt, OutStr)
       WRITE(OutStr,'(A27,3X,A)') '  Aerosol type            :', TRIM(Type_INP(I))
       Call WriteOut (Unt, OutStr)
       
       Call WriteOut (Unt, '')
    ENDDO
          
    WRITE(OutStr,'(A47)') 'Profile type: particle number density [1/cm^3].'
    Call WriteOut (Unt, OutStr)
    
    Call WriteOut (Unt, ' ')
    Call WriteOut (Unt,' Vertical profiles of aerosol')
    Call WriteOut (Unt,'         extinction coefficient (Ext coeff) and')
    Call WriteOut (Unt,'         single scattering albedo (Ss albedo):')
    IF(APH_TYPE == 'mie') THEN
       PHT = 'Mie'
    ELSE
       PHT = 'H-G'
    ENDIF

    DO K = 1,NAW
       Call WriteOut (Unt,'   ')
       WRITE(OutStr,'(1x,a11,F9.2,a6,a6,F8.4)') &
            'Wavelength ',AER_WAVS(K),' [nm];',' AOT =',TAU(K)
       Call WriteOut (Unt, OutStr)
       Call WriteOut (Unt,' ----------------------------------------------')
       Call WriteOut (Unt, '     Number, Alt [km], Ext coeff [km-1], Ss albedo')
       DO I = 1, NLAYERS
          Ext = AER_COEFF(EXI,I,K)
          IF( Ext == 0.0d0 ) CYCLE
          SS_A = AER_COEFF(SCI,I,K)/Ext
          WRITE(OutStr,'(6x,I3,4x,F6.2,5x,ES9.2,6x,E12.4)') I, ZHOLD(I), Ext, SS_A
          Call WriteOut (Unt, OutStr)
       END DO
       Call WriteOut (Unt, ' ')
       Call WriteOut (Unt,'    Asymmetry factor used in the aerosol layers :')
       Call WriteOut (Unt,'    ------------------------------------------- :')
       Call WriteOut (Unt,'    Layer  Top [km]  Bottom [km]   Asymmetry    Phase')
       Call WriteOut (Unt,'                                    factor     function')
       DO I = 1,LAN
          WRITE(OutStr,'(6x,I3,3x,F6.2,4x,F6.2,9x,F6.3,7x,a3)') &
               I, ZHOLD(LAS(I)) ,ZHOLD(LAE(I)), Asym_Par(I,K), PHT
          Call WriteOut (Unt, OutStr)
       ENDDO
    ENDDO                   ! Over wavelengths
          
    Call CloseOut (Unt)
    RETURN

134 CALL WRITE_MESSAGE (R_NAME,'ERROR', 'Output wavelength for AOT is out of range')
    WRITE(EU,'(A,F7.2)') '    Output wavelength: ', OWL_AER(I)
    WRITE(EU,'(A,2(2x,F7.2))') '    Database wavelengths: ', AER_WAVS(1), AER_WAVS(NAW)
    WRITE(EU,'(A)') '    Set output wavelength between database wavelengths'
    WRITE(EU,'(A)') '    in the "control_aer.inp" file.'
    RETURN 1

  END SUBROUTINE  WRITE_SCEN_PAO

!***********************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(DENS_PRF)) DEALLOCATE(DENS_PRF)
    IF(ALLOCATED( AER_WAVS)) DEALLOCATE(AER_WAVS, AER_COEFF, FDM_AER)
    IF(ALLOCATED(L_TOP)) DEALLOCATE(L_TOP, APH_VS)
    IF(ALLOCATED(Greek_aer)) DEALLOCATE(Greek_aer)
    IF(ALLOCATED(LA_S)) DEALLOCATE(LA_S, LA_E, IDX_AER)
    IF(ALLOCATED(IDX_AER_db)) DEALLOCATE(IDX_AER_db)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!***********************************************************************
END SUBROUTINE PAO_AER_SETUP
!***********************************************************************

