C***********************************************************************
C
C	SCIATRAN - Version 3.9 - Last modification: 17 Dec 2017
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C           Particular solution of the adjoint RTE in the
C                     plane-parallel medium
C
C                 Polarization is implemented
C***********************************************************************
      SUBROUTINE PART_SOL_ARTE(NL, NN, NLSA, AGR, SDO, Norm_lay, 
     $     Psim_lay, Psip_lay, Muv, DTR, ALW, E_gL, EXL, ADO)
c     ... Called by "solv_dom.f"

      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: DIR_OUT
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: Pie, N_Adj, NS_wf, Ainc, GeoD => GeoD_OUT
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: SDO_TYPE, ADO_TYPE, AGR_type
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) :: NL, NN, NLSA
      REAL (KND), INTENT(IN) :: Norm_lay(Nvs*NN,NL-1), ALW(NL-1),
     $     Psim_lay(Nvs*NN,Nvs*NN,NL-1), Muv(NLSA), DTR(NL),
     $     Psip_lay(Nvs*NN,Nvs*NN,NL-1), EXL(NL,NLSA),
     $     E_gL(NL,Nvs*NN,Nvs*NLSA,2)
      TARGET :: Norm_lay, Psim_lay, Psip_lay
c     -----------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (AGR_type) :: AGR    ! Adaptive grid
c     -------------------------------------------------------------
c     Local variables
c     ---------------
      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'PART_SOL_ARTE'
      INTEGER     I, J, K, Jg, Jv, Jvp, Jco, Jro, Jwf, K1, K2, Kg1, Kg2,  
     $     Ks, L, NNv, Ya, Yb, Iexl, IA
      REAL (KND)  A1, B1, SA, CV, He1, Wdo, HH(Nvs)
      REAL (KND), DIMENSION(Nvs*NN) :: Qp_t, Qm_t, aj, bj, CjMud, SjMud,
     $     Anv, Bnv
      REAL (KND), POINTER :: P_cs(:), P_sc(:), Nu_j(:), Norm(:),
     $     Psip(:,:), Psim(:,:), Enu(:), Ar1(:), UU(:), VV(:)
      TARGET ::  CjMud, SjMud
      COMPLEX (kind(0.0d0)) :: C1, C2, Cnu
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      NNv = Nvs*NN ; IA = 1

      ADO%PART_TU = 0.d0 ; ADO%PART_TD = 0.d0
      ADO%Aj_ls   = 0.d0 ; ADO%Bj_ls   = 0.d0
      ADO%Ajs     = 0.d0 ; ADO%Bjs     = 0.d0         
      Wdo = 1.d0 - Wup
      
      DO I = AGR%Nxl_1, AGR%Nxl_2
         Ya = AGR%LAY_GR(I,1) ; Yb = AGR%LAY_GR(I,2) + 1
         Norm => Norm_lay(:,Ya)   
         Psim => Psim_lay(:,:,Ya)
         Psip => Psip_lay(:,:,Ya)
         Nu_j => SDO%Nuj_lay (:,Ya) 
         Jco = SDO%NcC(Ya)  ;  Jro = NNv - 2*Jco

         DO Jv = 1, NLSA

            IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN
               P_cs => CjMud(:) ; P_sc => SjMud(:) ; Iexl = 0
            ELSE
               P_sc => CjMud(:) ; P_cs => SjMud(:) ; Iexl = 1
            ENDIF

            Ks = (Jv-1)*Nvs
            CV = MuV(Jv) 
            DO J = Ya,Yb-1
               Enu => SDO%Exp_NU_NL(:,J) ; SA = DEXP( -DTR(J+1)/CV )

c              ... Denominator of the single scattering pseudo-forward
c                  source function does not contain 2*Pi because this
c                  multiplier will be canceled later in the scalar
c                  forward-adjoint product (after integration over phi)
               He1 = 1.d0/ALW(J)/2.d0/CV ! /(2*Pi)

c              ......... C and S functions .......................      
c              --> real separation constants
               WHERE( REAL(Nu_j(1:Jro)) /= REAL(CV) )
                  CjMud(1:Jro) = Nu_j(1:Jro)*CV*
     $                 (Enu(1:Jro) - SA)/(Nu_j(1:Jro) - CV)
               ELSEWHERE
                  CjMud(1:Jro) = Enu(1:Jro)*DTR(J+1)
               END WHERE
               SjMud(1:Jro) = Nu_j(1:Jro)*CV*
     $              (1.d0 - Enu(1:Jro)*SA)/(Nu_j(1:Jro) + CV)
c              --> complex separation constants
               DO K1 = Jro+1, Jro+2*Jco, 2
                  C1  = DCMP( Nu_j(K1),Nu_j(K1+1) )
                  Cnu = DCMP( Enu(K1),Enu(K1+1) )
                  C2 =  C1*CV*(Cnu - SA)/(C1 - CV)
                  CjMud(K1) = DBLE(C2) ; CjMud(K1+1) = AIMAG(C2)
                  C2 = C1*CV*(1.d0 - Cnu*SA)/(C1 + CV)
                  SjMud(K1) = DBLE(C2) ; SjMud(K1+1) = AIMAG(C2)
               ENDDO
c              ---------------------------------------------------------

               DO Jwf = 1,NS_wf
                  Jvp = (Jv - 1)*NS_wf + Jwf

c                 ... For a given line-of-sight angle Jv we need to multiply
c                     the phase matrix by Ainc(1,:) which defines the 
c                     polarization state of a pseudo source.
c                       The matrix M = E_gL(J,:,:,2) is designed to describe 
c                     phase matrix Los <-- Gau, therefore we use M^T*Ainc(1,:)
c                     to obtain phase matrix -Gau <-- -Los.
c                     (see Eq.(Qpar-m) in STB).
                  DO Jg = 1,NN
                     Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                     DO K2 = 1,Nvs
                        Qm_t(Kg1+K2-1) = SUM( 
     $                       ( Wup*E_gL(J,  Kg1:Kg2,Ks+K2,2)
     $                       + Wdo*E_gL(J+1,Kg1:Kg2,Ks+K2,2))
     $                        *Ainc(Jwf,:) )
                     ENDDO
                  ENDDO
c                 ... Multiply the phase matrix by D4 from left
                  IF( Nvs > 1 ) FORALL( K=4:NNv:Nvs ) Qm_t(K) = -Qm_t(K) 

c              ... Multiplication M^T*Ainc(1,:) from left by
c                  D3=diag{1,1,-1,1} is omitted because in expressions
c                  for aj and bj coefficients of particular solution
c                  the source function for negative Gaussian nodes
c                  should by multiplied from left by D=diag{1,1,-1,-1}

c                    The matrix ^M = E_gL(J,:,:,1) is designed so that 
c                  D*^M describes scattering -Los <-- Gau, therefore,
c                  we use [^M]^T*Ainc(1,:) to obtain phase matrix
c                  Gau <-- -Los (see Eq.(Qpar-p) in STB).
                  DO Jg = 1,NN
                     Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                     DO K2 = 1,Nvs
                        Qp_t(Kg1+K2-1) = SUM( 
     $                       ( Wup*E_gL(J,  Kg1:Kg2,Ks+K2,1)
     $                       + Wdo*E_gL(J+1,Kg1:Kg2,Ks+K2,1))
     $                        *Ainc(Jwf,:) )
                     ENDDO
                  ENDDO
c                 ... Multiply the phase matrix by D4 from left
                  IF( Nvs > 1 ) FORALL( K=4:NNv:Nvs ) Qp_t(K) = -Qp_t(K) 

c                 .... Coefficients for particular solution (observation 
c                      angle) of pseudo-forward equation, upward PSS  
c                      (measured reflection)
c                 ......... Coefficients aj and bj ................
c                 --> real separation constants
                  DO L = 1,Jro
                     aj(L) = SUM( Psip(:,L)*Qp_t + Psim(:,L)*QM_t )
                     bj(L) = SUM( Psim(:,L)*Qp_t + Psip(:,L)*QM_t )
                  ENDDO
                  aj(1:Jro) = aj(1:Jro)*0.5/Norm(1:Jro)  
                  bj(1:Jro) = bj(1:Jro)*0.5/Norm(1:Jro)
c                 --> complex separation constants
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     C2 = DCMP( Norm(K1),Norm(K1+1) ) 
                     A1 = SUM(Psip(:,K1)  *QP_t + Psim(:,K1)  *QM_t )
                     B1 = SUM(Psip(:,K1+1)*QP_t + Psim(:,K1+1)*QM_t)
                     C1 = DCMP( A1,B1 )
                     aj(K1) = DBLE(C1/C2)*0.5 
                     aj(K1+1) = AIMAG(C1/C2)*0.5

                     A1 = SUM(Psim(:,K1)  *QP_T + Psip(:,K1)  *QM_T )
                     B1 = SUM(Psim(:,K1+1)*QP_T + Psip(:,K1+1)*QM_T)
                     C1 = DCMP( A1,B1 )
                     bj(K1) = DBLE(C1/C2)*0.5 
                     bj(K1+1) = AIMAG(C1/C2)*0.5
                  ENDDO
                  aj(:) = aj(:)*He1  ;  bj(:) = bj(:)*He1
                  ADO%Aj_ls (:,Jvp,J) = aj(:)  
                  ADO%Bj_ls (:,Jvp,J) = bj(:)

c                  --> real separation constants
                  Anv(1:Jro) = aj(1:Jro)*P_cs(1:Jro)
                  Bnv(1:Jro) = bj(1:Jro)*P_sc(1:Jro)
c                 --> imaginary separation constants
                  DO K1 = Jro+1, Jro+2*Jco, 2 
                     Anv(K1)   = aj(K1)*P_cs(K1)   - aj(K1+1)*P_cs(K1+1)
                     Anv(K1+1) = aj(K1)*P_cs(K1+1) + aj(K1+1)*P_cs(K1)
                     Bnv(K1)   = bj(K1)*P_sc(K1)   - bj(K1+1)*P_sc(K1+1)
                     Bnv(K1+1) = bj(K1)*P_sc(K1+1) + bj(K1+1)*P_sc(K1)
                  ENDDO
                  ADO%Ajs(J,:,Jvp) = Anv(:)*EXL(J+Iexl,Jv)
                  ADO%Bjs(J,:,Jvp) = Bnv(:)*EXL(J+Iexl,Jv)
               ENDDO            ! over number of derivatives (Jwf)
            END DO              ! over sub-layers (J)

            DO Jwf = 1,NS_wf
               Jvp = (Jv - 1)*NS_wf + Jwf

c              ... This do-loop is used if Adaptive grid option is active
               DO J = Ya+1,Yb-1 
                  Enu => SDO%Exp_NU_NL(:,J) ; Ar1 => ADO%Ajs(J,:,Jvp)
                  Ar1(1:Jro) = Ar1(1:Jro) 
     $                 + ADO%Ajs(J-1,1:Jro,Jvp)*Enu(1:Jro)
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     A1 = ADO%Ajs(J-1,K1,Jvp) 
                     B1 = ADO%Ajs(J-1,K1+1,Jvp)
                     Ar1(K1)   = Ar1(K1)   + A1*Enu(K1)   - B1*Enu(K1+1)
                     Ar1(K1+1) = Ar1(K1+1) + A1*Enu(K1+1) + B1*Enu(K1)
                  ENDDO
               ENDDO

c              ... This do-loop is used if Adaptive grid option is active
               DO J = Yb-2, Ya, -1
                  Enu => SDO%Exp_NU_NL(:,J) ; Ar1 => ADO%Bjs(J,:,Jvp)
                  Ar1(1:Jro) = Ar1(1:Jro) 
     $                 + ADO%Bjs(J+1,1:Jro,Jvp)*Enu(1:Jro)
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     A1 = ADO%Bjs(J+1,K1,Jvp) 
                     B1 = ADO%Bjs(J+1,K1+1,Jvp)
                     Ar1(K1)   = Ar1(K1)   + A1*Enu(K1)   - B1*Enu(K1+1)
                     Ar1(K1+1) = Ar1(K1+1) + A1*Enu(K1+1) + B1*Enu(K1)
                  ENDDO
               ENDDO
            
c              ... Particularly solution for observation angle at the top 
c                  (_TU) and the bottom (_TD) of each homogeneous layers
               UU => ADO%Bjs(Ya,:,Jvp) ; VV => ADO%Ajs(Yb-1,:,Jvp)
c              ... Real separation constants
               ADO%Part_TU(I,:,Jvp) = 
     $              MATMUL(SDO%G_NL(Ya,:,NNv+1:NNv+Jro),UU(1:Jro))
               ADO%Part_TD(I,:,Jvp) = 
     $              MATMUL(SDO%G_NL(Ya,:,1:Jro),VV(1:Jro))
c              ... Including imaginary separation constants
               DO K1 = Jro+1, Jro+2*Jco, 2
                  ADO%Part_TU(I,:,Jvp) = ADO%Part_TU(I,:,Jvp) + 2.d0*
     $                 ( SDO%G_NL(Ya,:,NNv+K1)  *UU(K1) 
     $                 - SDO%G_NL(Ya,:,NNv+K1+1)*UU(K1+1))

                  ADO%Part_TD(I,:,Jvp) = ADO%Part_TD(I,:,Jvp) + 2.d0*
     $                 ( SDO%G_NL(Ya,:,K1)  *VV(K1) 
     $                 - SDO%G_NL(Ya,:,K1+1)*VV(K1+1))
               ENDDO            ! over imaginary separation constants (K1)

            ENDDO  ! over number of derivatives (Jwf)
         ENDDO  ! over line-of-sight (Jv)
      ENDDO   ! over standard layers (I)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE PART_SOL_ARTE
