C***********************************************************************
C
C	SCIATRAN - Version 4.5 - Last modification: 14 Dec 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C           Particular solution of the adjoint RTE in the
C                 spherical medium (limb geometry)
c             (modified to account for polarization)
C***********************************************************************
      SUBROUTINE PART_SOL_ARTE_SPHER(Mh, NrL, NL, NN, NLSA, N_OBS_AG, 
     $     LAY_GR, SDO, Norm_lay, Psim_lay, Psip_lay, DTR,
     $     LOA, SLS, ALW, T_AU, ADS, ADO, Ad_Gr)
c     ... Called by "solv_dom.f"

      USE AEROSOLS,    ONLY: LA_N, LA_S, LA_E
      USE CLOUD_INPUT, ONLY: Plus_ice
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT
      USE GRIDS,       ONLY: ZZZ, MAX_LINE
      USE GT_POLYN,    ONLY: Plm_G, Dp2_G, Dm2_G, Leg => NLEGEN,   
     $                       XM, SGN, Plm_S, Dp2_S, Dm2_S
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nlg_ray, Nm_vs, NL_Aer,  
     $                       Nl_Cr, Ng_Cr, Nls_Cr, Nso_Cr, N_C2, 
     $                       Nlg_Cld, NL_Cld, NL_ice, NNv,
     $                       Ng1_brf, Ng2_brf, Ns_brf, Nvs, Nlg_Aer
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_DSO, DIR_OUT
      USE LOS_ANGLES,  ONLY: N_BE, LIM_GR, N_BELOW
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: PIE, N_Adj, DO_AEROSOL, Ainc, N_Eff, M_LIM, 
     $                       ID_LIM, Nsl_sd => SHI_LIM_C, Add_lev, 
     $                       ADAPTIVE_GRID, NS_wf, Vec_sca, SHI_LIM,
     $                       GeoD => GeoD_OUT
      USE SPHER_PARS,  ONLY: NFA => N_FOV_ANG, Nf => NN, 
     $                       N_OBS_ALT, LW_FOV, FOV_WEIGHTS
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: LOA_TYPE, SDO_TYPE, ADO_TYPE, SLS_type
c     ==================================================================

      INTEGER, INTENT(IN) :: Mh, NrL, NL, NN, NLSA, N_OBS_AG, 
     $     LAY_GR(NL-1,2)
      LOGICAL, INTENT(IN) :: Ad_Gr
      REAL (KND), INTENT(IN) :: T_AU(NL), DTR(NL), ALW(NL-1),
     $     Norm_lay(Nvs*NN,NL-1), Psim_lay(Nvs*NN,Nvs*NN,NL-1), 
     $     Psip_lay(Nvs*NN,Nvs*NN,NL-1) 
      TARGET :: Norm_lay, Psim_lay, Psip_lay
c     ----------------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (SLS_type) :: SLS    ! Spherical line-of-sight (LoS)
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables 
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
c     -------------------------------------------------------------
c     ... Adjoint DOM auxiliary spherical geometry |
c     ----------------------------------------------
      TYPE ADS_type
        SEQUENCE
        LOGICAL, POINTER :: Act_ag(:,:)
        REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
      END TYPE ADS_type
      TYPE (ADS_type) :: ADS
c     -------------------------------------------------------------
c     ... Local variables
c     -------------------
      CHARACTER (LEN=19), PARAMETER :: R_NAME = 'PART_SOL_ARTE_SPHER'
      INTEGER  I, J, K, L, K1, K2, L1, Jg, Jv, Jt, Ji, Jf, Jvs, Jw, Jw1,
     $     JL, Iks_c, Kg1, Kg2, Ks, P_M, Nvd, Lma, MLI, IA, KK,
     $     Nth, N_1, Ya, Yb, Iexl, Yla, Ylb, Jro, Jco, Ifov
      INTEGER  SIGN_M, Mhm, Iut, J_c, Jf1, Jf2, Start_DO, End_DO
      REAL (KND)  A1, B1, SA, CV, He1, DTL, Wdo, E_1(2), Edi, Es1, Es2,
     $     WFOV, SS1, SS2
      REAL (KND), DIMENSION(Nvs*NN) :: QP_T, QM_T, aj, bj, CjMud, SjMud,
     $     Enu

      REAL (KND)  AER_LOC(Nlg_Aer,NL), 
c     $     Plm_L(NLSA,Leg,N_eff), SUM_MOM(Leg,Nm_vs,NL), XM_2(2*NN),
     $     Plm_L(NFA*NLSA,Leg,N_eff), SUM_MOM(Leg,Nm_vs,NL), XM_2(2*NN),
     $     Ops(N_Eff)
      TARGET ::   CjMud, SjMud, SUM_MOM

c     ... This array is defined simular to Ph_z%EgL in the p-p case
c      REAL (KND) S_Egl(NL,Nvs*NN,Nvs*N_Eff,2,NLSA)
      REAL (KND) S_Egl(NL,Nvs*NN,Nvs*N_Eff,2,NFA*NLSA)

      REAL (KND), TARGET :: 
     $     H_Ajs(0:NL-1,NNv,NS_wf,NLSA*NFA,2), 
     $     H_Bjs(0:NL-1,NNv,NS_wf,NLSA*NFA,2)

      COMPLEX (kind(0.0d0)) :: C1, C2, Cnu, Cpc, Cen
      REAL (KND), ALLOCATABLE :: Dp2_L(:,:,:), Dm2_L(:,:,:),
     $     Pol_ls(:,:,:) 
      TARGET ::   Dp2_L, Dm2_L, Plm_L, S_Egl
      INTEGER,    POINTER :: IND_LS(:,:,:)
      REAL (KND), POINTER :: P_cs(:), P_sc(:), SCR(:), SCA(:), SCC(:), 
     $     UU(:), VV(:), Hpa(:), Hpb(:), Norm(:), Nuj(:), rs_k(:), 
     $     COLSA(:), X_MOMS(:,:), 
     $     Psim(:,:), Psip(:,:), R_GL(:,:,:), G_NL(:,:,:), Po_g(:,:)
c    -----------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I')

      Mhm = Mh - 1 ; Iut = 1 ; IA = 1
      Wdo = 1.d0 - Wup

      SCR => LOA%SCR ; SCA => LOA%SCA ; SCC => LOA%SCC
c     ... IND_LS was calculated in "appr_spher_ls_main.f"
      IND_LS => SLS%IND_LS
      Lma = Leg      
c      write(*,*) '   SHAPE(ADO%Ajs) ', SHAPE( ADO%Ajs )
      G_NL => SDO%G_NL 

      Po_g => Plm_G(Mh,Mh:Leg,:) ! Plm     Gauss
      L1 = Leg - Mh + 1
         
c     ... Legendre polynomials for Gaussian angles. 
c         In p-p case polynomials were saved in Plm_G(Nh_s,NLEGEN,NSTR) 
c         in "gt_hi_wi.f" for positive values of XM only !
      XM_2(1   :NN  ) =  XM(1:NN)
      XM_2(NN+1:2*NN) = -XM(1:NN)
      K2 = 2*NN
      CALL WIGNER_PLG(XM_2, K2, Mhm, Leg, ADS%Pol_ga)

c ********** Contribution of Rayleigh, aerosol and clouds ***********
c                   into weighted scattering matrix
c     ... Calculate product of the scattering coefficient and
c         the phase function expansion coefficients
      SUM_MOM = 0.d0
c     ******** Rayleigh ********
      IF( Mh <= 3 ) THEN
         DO I = 1, NL
            SUM_MOM(Mh:3,1:Nm_vs,I) = LOA%RAY_MOMS(Mh:3,1:Nm_vs)*SCR(I)
         ENDDO
      ENDIF
c     ******** Aerosol ********
      IF( DO_AEROSOL ) THEN
         DO K1 = 1, Nm_vs
            AER_LOC = 0.d0
            DO I = 1,LA_N
               DO K = LA_S(I),LA_E(I)
                  AER_LOC(:,K) = LOA%AER_MOMS(:,I,K1)
               ENDDO  ! over altitudes within aerosol layers (K)
            ENDDO  ! over aerosol layers (I)
            DO I = 1, NL
               SUM_MOM(Mh:Leg,K1,I) = SUM_MOM(Mh:Leg,K1,I)
     $              + AER_LOC(Mh:Leg,I)*SCA(I)
            ENDDO  ! over altitude levels (I)
         ENDDO  ! over Greek constants (K1)
      ENDIF
c     ******** Cloud ********
      IF( DO_CLOUD ) THEN
         DO K1 = 1,Nm_vs
            DO J = 1, N_CLD_LAYERS
               DO I = J_TOP(J)+1, J_BOT(J)-1
                  SUM_MOM(Mh:Leg,K1,I) = SUM_MOM(Mh:Leg,K1,I) 
     $                 + LOA%CLD_MOMS(Mh:Leg,J,1,1,K1)*SCC(I) 
               ENDDO       ! over sub-layers within cloud layer (I)
               IF( .NOT.Plus_ice(J) ) CYCLE
               DO I = J_TOP(J)+1, J_BOT(J)-1
                  SUM_MOM(Mh:Leg,K1,I) = SUM_MOM(Mh:Leg,K1,I) 
     $                 + LOA%CLD_MOMS(Mh:Leg,J,2,1,K1)*LOA%SCI(I) 
               ENDDO  ! over sub-layers within cloud layer (I)
            ENDDO  ! over cloud layers (J)
         ENDDO  ! over Greek constants (K1)
      ENDIF
c **********************************************************************

c ********** Legendre polynomials and  Wigner functions ***********
c        will be recalculated for each Fourier harmonic Mhm
      IF( Ad_Gr ) THEN
         ALLOCATE( Pol_ls(Leg,NFA*NLSA,N_Eff) )
         DO Jv = 1,NFA*NLSA
            DO JL = 1,M_LIM(Jv) 
               CALL WIGNER_PLG([SLS%Eff_Mu(Jv,JL)], Iut, Mhm,
     $              Leg, Pol_ls(:,Jv,JL))
            ENDDO
         ENDDO
      ENDIF  ! option 'Adaptive grid'
      
c     ... Legendre polynomials for viewing angles along LoS.
c            M_LIM(1:NLSA*NFA) defines the number of sub-intervals along 
c            LoS for observation angles in the range (1:NLSA).
c
c         SLS%Eff_Mu(1:NFA*NLSA,1:N_eff) contains cosines 
c            of the L-of-S direction with respect to the downward 
c            direction (tau-axis) for all sub-intervals and the 
c            observation angles in the range (1:NFA*NLSA).
c            Eff_Mu is calculated in "appr_spher_ls_main" for 
c            all LoS within FOV (see LW_FOV in "geom_along_los.f90")
      DO Jv = 1,NLSA*NFA ; Nvd = M_LIM(Jv)  
         Ops(1:Nvd) = SLS%Eff_Mu(Jv,1:Nvd)
         CALL WIGNER_PLG( Ops(1:Nvd), Nvd, Mhm, Leg, Plm_L(Jv,:,1:Nvd) )
      ENDDO      
c      WRITE(*,*) '  M_LIM(:)  =  ', M_LIM(:)
      S_Egl = 0.d0
      SELECT CASE( Vec_Sca )
      CASE('scal_rte')
         DO I = 1,NL
            DO K = 1,NN 
               DO Jv = 1, NLSA*NFA  
                  DO JL = 1,M_LIM(Jv) 
                     S_Egl(I,K, JL, 1,Jv) = SUM( SUM_MOM(Mh:Leg,1,I)*
     $                    Plm_L(Jv,Mh:Leg, JL)*Po_g(1:L1,K) )/XM(K)
                     S_Egl(I,K, JL, 2,Jv) = SUM( SUM_MOM(Mh:Leg,1,I)*
     $                    Plm_L(Jv,Mh:Leg, JL)*Po_g(1:L1,K)*SGN(1:L1) )
     $                 /XM(K)
                  ENDDO  ! over sub-layers along LoS (JL)
               ENDDO  ! over LoS angles (Jv)

               IF( .NOT.Ad_Gr ) CYCLE  ! ====>
c              ... Used if "Adaptive grid" option is set to 'on'
c                  Calculated for selected direction within FoV only.
               DO Jv = 1, NLSA  ;  Jvs = Jv*NFA-LW_FOV(Jv)
                  DO JL = 1,M_LIM(Jvs) 
                     ADS%PLS(I,K,Jv,JL) = SUM( SUM_MOM(Mh:Leg,1,I)*
     $                    Pol_ls(Mh:Leg,Jvs,JL)*ADS%Pol_ga(Mh:Leg,K) )
     $                    /XM(K)
                     ADS%MLS(I,K,Jv,JL) = SUM( SUM_MOM(Mh:Leg,1,I)*
     $                    Pol_ls(Mh:Leg,Jvs,JL)*ADS%Pol_ga(Mh:Leg,K+NN))
     $                    /XM(K)
                  ENDDO  ! over sub-layers along LoS (JL)
               ENDDO  ! over LoS angles (Jv, Jvs)
            ENDDO  ! over Gauss angles (K)
         ENDDO   ! over standard altitude grid (I)

      CASE('vect_rte')
         ALLOCATE( Dp2_L(NFA*NLSA,Leg,N_eff),Dm2_L(NFA*NLSA,Leg,N_eff) )
         SIGN_M = 1 - 2*MODULO(Mhm,2) ; Dp2_L = 0.d0 ; Dm2_L = 0.d0
         DO Jv = 1,NLSA *NFA 
            Nvd = M_LIM(Jv)
            Ops(1:Nvd) = -SLS%Eff_Mu(Jv,1:Nvd)
c           ... To be compatible with "gt_hi_wi.f" COLSA (here Ops)
c               is used with opposite sign ????
            CALL WIGNER_FUNCTION( Ops(1:Nvd), Nvd, Mhm, Leg, 
     $           Dp2_L(Jv,:,1:Nvd), Dm2_L(Jv,:,1:Nvd) )
            Dp2_L(Jv,:,:) = 0.5d0*( Dp2_L(Jv,:,:) + Dm2_L(Jv,:,:) )
            Dm2_L(Jv,:,:) = Dp2_L(Jv,:,:) - Dm2_L(Jv,:,:)
            Dp2_L(Jv,:,:) = SIGN_M*Dp2_L(Jv,:,:) ! Rlm(mu)
            Dm2_L(Jv,:,:) = SIGN_M*Dm2_L(Jv,:,:) ! Tlm(mu)
         ENDDO  ! over LoS (Jv)
         DO I = 1,NL      
            X_MOMS => SUM_MOM(1:Leg,:,I)
            DO Jv = 1, NLSA *NFA
               R_GL => S_EgL(I,:,1:Nvs*M_LIM(Jv),:,Jv)
c              ... Simular as in "phase_matr.f"
               CALL SCAT_MATR_SPH('Gau', NN, 'LoS', M_LIM(Jv), R_GL)
            ENDDO  ! over LoS (Jv)
         ENDDO  ! over altitude levels (I)
      END SELECT
c     **********************************************************************

c      WRITE(*,*) '  SHAPE(S_Egl) ', SHAPE(S_Egl)
c      DO I = 1,NL
c         WRITE(*,'(2x,I3,2x,F8.3,*(2x,e12.4))') I, ZZZ(I),
c     $        (SUM( S_Egl(I,:, JL, 1,1) ), JL=1,M_LIM(1) )
c      ENDDO
c      PAUSE
      
c     ... Output parameters
      ADO%PART_TU = 0.d0 ; ADO%PART_TD = 0.d0
      ADO%Ajs     = 0.d0 ; ADO%Bjs     = 0.d0         
c     ... ADO%Aj_sph and ADO%Bj_sph are used in "gau_adj_up_down_sph.f"
c         if 'ADAPTIVE_GRID' option is switched on
      ADO%Aj_sph = 0.d0 ; ADO%Bj_sph = 0.d0

      H_ajs = 0.d0 ; H_bjs = 0.d0
      DO Jt = 1, NLSA 
         DO Jv = (Jt-1)*NFA+1,Jt*NFA ! all directions within FoV
            Nth = N_below(Jv)
c           ... For downward direction of instrument:
            IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN
               P_cs => CjMud(:) ; P_sc => SjMud(:) ; Iexl = 0 ; P_M = -1
               N_1 = N_OBS_ALT(1) 
c           ... For upward direction of instrument:
            ELSE
               P_sc => CjMud(:) ; P_cs => SjMud(:) ; Iexl = 1 ; P_M = 1
               N_1 = 1
            ENDIF
c             write(*,*) '                            '
c             write(*,'(A,2(I3,2x,I3))')
c     $            ' **************************** LoS and N_1', Jv, N_1
c             WRITE(*,*) '  M_LIM(Jv) = ', Jv, M_LIM(Jv)
c             WRITE(*,*) ' Number of points along LoS = ',
c     $            GeoD%LVG(IA,Jv)%NLS
c             write(*,'(A,I3,A,I3)') 
c     $            '*** Levels from', N_1,' to ', Nth-1
c             write(*,'(A,F8.3,A,F8.3)') 
c     $            '*** Alts. from', ZZZ(N_1),' to ', ZZZ(Nth)
c             write(*,'(A)')  
c     $     '  level   ID_LIM    S_EgL   Ex(J) Ex(J+1)    Jw  ID_LIM  Jv'
c             write(*,*) '    '
c             WRITE(*,*) '   Calculate H_Ajs ', SHAPE( H_Ajs )
c             WRITE(*,*) '   For all directions within FoV'
c        ... Nth-1 selects the level of standard altitude grid
c            located above tangent height (if any) or above surface
            DO J = N_1, Nth-1 
c               write(*,'(4x,I3$)')  J
               Jco = SDO%NcC(J)            ; Jro = NNv - 2*Jco
               Norm  => Norm_lay(:,J)      ; Psim => Psim_lay(:,:,J)
               Psip  => Psip_lay(:,:,J)    ; Nuj  => SDO%Nuj_lay (:,J)
               Enu(:) = SDO%Exp_NU_NL(:,J) ; DTL   = DTR(J+1)
               He1    = P_M*0.5d0/ALW(J)

               Ks = (ID_LIM(Jv,J,1) - 1)*Nvs ! Check!
c               write(*,'(4x,F8.3,2x,I3)') ZZZ(J), ID_LIM(Jv,J,1)
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*M_LIM(Jv)
     $                 + SHI_LIM(Jv)*Ns_wf + ID_LIM(Jv,J,1) 
c              ... Coefficients for particularly solution (observation angle)
c                  Pseudo-forward equation, before Th (measured reflection)
c                  aj and bj are results of the "make_abj" subroutine
                  CALL MAKE_ABJ
                  ADO%Aj_sph(J,:,Jv,Jw,1) = aj(:)
                  ADO%Bj_sph(J,:,Jv,Jw,1) = bj(:) 

                  IF( GeoD%LVG(IA,Jv)%NLS == 1 ) CYCLE
                  If( ID_LIM(Jv,J,1) == 0 ) CYCLE

                  IF( J == N_below(Jv)-1 ) THEN
                     DTL = (T_AU(J+1) - T_AU(J))/(ZZZ(J) - ZZZ(J+1))*
     $                    (ZZZ(J) - GeoD%LVG(IA,Jv)%Th_V)
c                     WRITE(*,'(A,2(1x,f8.3))') ' level above Th ', 
c     $                    ZZZ(J), GeoD%LVG(IA,Jv)%Th_V
                     Enu(1:Jro) = DEXP( -DTL/Nuj(1:Jro) )
                     DO K1 = Jro+1, Jro+2*Jco, 2
                        Cnu = EXP( -DCMP(DTL)/DCMP( Nuj(K1),Nuj(K1+1) ))
                        Enu(K1) = DBLE( Cnu ) ; Enu(K1+1) = AIMAG( Cnu )
                     ENDDO
                  ENDIF
c              ... Parameters of the approximation transmission function
c                  within layers before Th (see Eq.(Appr_b_ls) in STB) 
                  CV  = SLS%Mue_ls(Jv,J,1) ; SA = DEXP( -DTL/CV )
                  rs_k => SLS%SIN_COEF_LS(Jv,J,:,1) 
                  K1  = SLS%N_sph_ls(Jv,J,1)
c              ... Calculation auxiliary functions C_b and S_b 
c                  (see Eqs. (C_b-fin) and (S_b-fin) in STB)
                  Call SC_FUNC_SPH
     $                 (K1, Nuj, Enu, Cv, SA, DTL, CjMud, SjMud)
c              ... Spherical transmission function at the top and bottom
c                  of j-th layer
                  E_1(1) = SLS%LSE(IA,Jv)%EX( IND_LS(J,Jv,1))   ! top   T_(l-1)
                  E_1(2) = SLS%LSE(IA,Jv)%EX( IND_LS(J+1,Jv,1)) ! bottom T_l
c                  WRITE(*,*) ' Top bot ', J, E_1(1), E_1(2)
                  Edi    = E_1(1+Iexl)
                  Hpa => H_Ajs(J,:,Jw,Jv,1) ; Hpb => H_Bjs(J,:,Jw,Jv,1)
c              ... Calculation of the function Ajb (reflection) or 
c                  Aja (transmission) (Eqs.(Aj_adj_b) and (Aj_adj_a) in STB)
c                 ---->  Real separation constants
                  Hpa(1:Jro) = Hpa(1:Jro) + aj(1:Jro)
     $                 *( E_1(2) - E_1(1)*Enu(1:Jro) 
     $                 - P_cs(1:Jro)*Edi/Nuj(1:Jro) ) 
c              ... Calculation of the function Bjb (reflection) or
c                  Bja (transmission)
c                 (see Eqs.(Bj_adj_b) and (Bj_adj_a) in STB)  
                  Hpb(1:Jro) = Hpb(1:Jro) + bj(1:Jro)
     $                 *( E_1(2)*Enu(1:Jro) - E_1(1) 
     $                 + P_sc(1:Jro)*Edi/Nuj(1:Jro) ) 
c                 ---->  Complex separation constants
                  DO I = Jro+1, Jro+2*Jco, 2
                     Cnu = DCMP( Nuj(I),Nuj(I+1) )
                     Cpc = DCMP( P_cs(I),P_cs(I+1) )
                     Cen = DCMP( Enu(I), Enu(I+1) )
                     C1 = DCMP( aj(I),aj(I+1) )
     $                    *( E_1(2) - E_1(1)*Cen - Cpc*Edi/Cnu )
                     Hpa(I)   = Hpa(I)   + DBLE(C1) 
                     Hpa(I+1) = Hpa(I+1) + AIMAG(C1) 
c                    ... Contribution into Hpb
                     Cpc = DCMP( P_sc(I),P_sc(I+1) )
                     C2 = DCMP( bj(I),bj(I+1) )
     $                    *( E_1(2)*Cen - E_1(1) + Cpc*Edi/Cnu )
                     Hpb(I)   = Hpb(I)   + DBLE(C2) 
                     Hpb(I+1) = Hpb(I+1) + AIMAG(C2)
                  ENDDO ! over complex separation constants (I)
               ENDDO  ! over number of derivatives (Jw, Jw1)
            ENDDO  ! over layers before tangent height (J)
c            write(*,*) '*** End levels from', N_1,' to ', Nth-1
c            write(*,*) '    '
c          **** Additional terms for layers after tangent height ****
            IF( GeoD%LVG(IA,Jv)%NTH == GeoD%LVG(IA,Jv)%NLS ) CYCLE
            IF( GeoD%LVG(IA,Jv)%NTH == 1 ) CYCLE
c            write(*,*) '************** After Th ***************** '
c            write(*,'(A)')  
c     $       '  level   ID_LIM    S_EgL   Ex(J) Ex(J+1)    Jw  ID_LIM  Jv'
c            write(*,*) '    ' 
         DO J = 1, Nth-1
            Jco = SDO%NcC(J)  ;  Jro = NNv - 2*Jco
            Norm  => Norm_lay(:,J)      ; Psim => Psim_lay(:,:,J)
            Psip  => Psip_lay(:,:,J)    ; Nuj  => SDO%Nuj_lay (:,J)
            Enu(:) = SDO%Exp_NU_NL(:,J) ; DTL   = DTR(J+1)
            He1    = 0.5d0/ALW(J)

            Ks = (ID_LIM(Jv,J,2) - 1)*Nvs   
            DO Jw = 1,NS_wf
              Jw1 = (Jw - 1)*M_LIM(Jv)
     $              + SHI_LIM(Jv)*Ns_wf + ID_LIM(Jv,J,2) 
               CALL MAKE_ABJ
               ADO%Aj_sph(J,:,Jv,Jw,2) = aj(:)
               ADO%Bj_sph(J,:,Jv,Jw,2) = bj(:) 

               If( ID_LIM(Jv,J,2) == 0 ) CYCLE
               IF( J == N_below(Jv)-1 ) THEN
                  DTL = (T_AU(J+1) - T_AU(J))/(ZZZ(J) - ZZZ(J+1))*
     $                 (ZZZ(J) - GeoD%LVG(IA,Jv)%Th_V)
c                         WRITE(*,'(A,2(1x,f8.3)$)') ' level above Th ', 
c     $                    ZZZ(J), GeoD%LVG(IA,Jv)%Th_V
                  Enu(1:Jro) = DEXP( -DTL/Nuj(1:Jro) )
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     Cnu = EXP( -DCMP(DTL)/DCMP( Nuj(K1),Nuj(K1+1) ))
                     Enu(K1) = DBLE( Cnu ) ; Enu(K1+1) = AIMAG( Cnu )
                  ENDDO                  
               ENDIF
c              ... Parameters of the approximation transmission function
c                  within layers after Th (see Eq.(Appr_a_ls) in STB) 
               CV  = SLS%Mue_ls(Jv,J,2) ; SA = DEXP( -DTL/CV ) 
               rs_k => SLS%SIN_COEF_LS(Jv,J,:,2)
               K1  = SLS%N_sph_ls(Jv,J,2)
c              ... Calculation auxiliary functions C_a and S_a 
c                  (see Eqs. (C_a-fin) and (S_a-fin) in STB)
               Call SC_FUNC_SPH
     $              (K1, Nuj, Enu, Cv, SA, DTL, CjMud, SjMud)
c              ... Spherical transmission function at the top and bottom
c                  of j-th layer
               Es1 = SLS%LSE(IA,Jv)%EX(IND_LS(J,Jv,2))
               Es2 = SLS%LSE(IA,Jv)%EX(IND_LS(J+1,Jv,2))
               Hpa => H_Ajs(J,:,Jw,Jv,2) ; Hpb => H_Bjs(J,:,Jw,Jv,2)
c                  WRITE( *,'(A,2(2x,I4))' ) '  b  ---> H_Ajs()', J, Jv
c              ... Calculation of the function Aja and Bja 
c                     (see Eqs.(Aj_adj_a) and (Bj_adj_a) in STB)
c                 ---->  Real separation constants
               Hpa(1:Jro) = Hpa(1:Jro) + aj(1:Jro) 
     $             *(Es2 - Es1*Enu(1:Jro) - Es2*SjMud(1:Jro)/Nuj(1:Jro))
               Hpb(1:Jro) = Hpb(1:Jro) + bj(1:Jro)
     $             *(Es2*Enu(1:Jro) - Es1 + Es2*CjMud(1:Jro)/Nuj(1:Jro))
c                 ---->  Complex separation constants
               DO I = Jro+1, Jro+2*Jco, 2
                  Cnu = DCMP( Nuj(I),Nuj(I+1) )
                  Cpc = DCMP( SjMud(I),SjMud(I+1) )
                  Cen = DCMP( Enu(I), Enu(I+1) )
                  C1=DCMP(aj(I),aj(I+1))*(Es2 -Es1*Cen -Cpc*Es2/Cnu)
                  Hpa(I)   = Hpa(I)   + DBLE(C1) 
                  Hpa(I+1) = Hpa(I+1) + AIMAG(C1) 
c                   ... Contribution into Hpb
                  Cpc = DCMP( CjMud(I),CjMud(I+1) )
                  C2 =DCMP(bj(I),bj(I+1))*(Es2*Cen -Es1 +Cpc*Es2/Cnu)
                  Hpb(I)   = Hpb(I)   + DBLE(C2)
                  Hpb(I+1) = Hpb(I+1) + AIMAG(C2) 
               ENDDO  ! over complex separation constants (I)
            ENDDO  ! over number of derivatives (Jw, Jw1)
         ENDDO  ! over layers (J)
c         write(*,*) '*** End levels from  "1"  to ', Nth-1
c        write(*,*) '    '
cc        ************** End tangent height *************************
         ENDDO ! over directions within FOV (Jv)
      ENDDO  ! over LoS (Jt)

c      PAUSE
c----------------------------------------------------------------------
c     ... Integration over FoV 
c-----------------------------------------------------------------------
c      WRITE(*,*) '   Sum over FoV Jv, Jw1 : '
c      WRITE(*,*) ' NLSA and NFA ', NLSA, NFA
c      WRITE(*,*)  ' SHAPE( H_Ajs ) ', SHAPE( H_Ajs )
      DO Jv = 1, NLSA 
         J_c = Jv*NFA - LW_FOV(Jv)  ! selected direction within FoV
         MLI = M_LIM(J_c) 
         Nth = N_below(J_c)  
         Jf1 = (Jv-1)*NFA + 1 ; Jf2 = Jv*NFA
         N_1 = 1
         IF( GeoD%LVG(IA,J_c)%C_VA(1) > 0.d0 ) N_1 = N_OBS_ALT(1) 
c         write(*,*) '       ***** Before Th ******* '
c         WRITE(*,*) '   Altitude range : ', ZZZ(N_1), ZZZ(Nth-1)
c         PAUSE
         DO J = N_1, Nth-1 
            If( ID_LIM(J_c,J,1) == 0 ) CYCLE
            Iks_c  = Nsl_sd(Jv)*Ns_wf + ID_LIM(J_c,J,1) 
            DO Jw = 1,NS_wf
               Jw1 = (Jw - 1)*MLI  + Iks_c
c               WRITE(*,'(5x,3(2x,I3),2x,I3,A,I3)')
c     $              Jv, J, Jw1, Jf1,' - ',Jf2
c               WRITE(*,'(10x,A,*(1x,E12.4))')'+ ',H_Ajs(J,1,1,Jf1:Jf2,1)
c               WRITE(*,'(10x,A,*(1x,E12.4))')'- ',H_Ajs(J,1,1,Jf1:Jf2,2)
               DO K = 1, NNv
c                 ----------------------------------------------------
                  ADO%Ajs(J,K,Jw1) =
     $                 SUM( H_Ajs(J,K,Jw,Jf1:Jf2,1)*FOV_WEIGHTS(1:NFA) )
                  ADO%Bjs(J,K,Jw1) =
     $                 SUM( H_Bjs(J,K,Jw,Jf1:Jf2,1)*FOV_WEIGHTS(1:NFA) )
c                -----------------------------------------------------
               ENDDO
            ENDDO
         ENDDO  ! over (J)
         IF( GeoD%LVG(IA,J_c)%NTH == GeoD%LVG(IA,J_c)%NLS ) CYCLE
         IF( GeoD%LVG(IA,J_c)%NTH == 1 ) CYCLE
c              write(*,*) '       ***** After Th ******* '
c              WRITE(*,*) '   Altitude range :  1   ', Nth-1 
c              write(*,*) '   Selected direction    ', Jv, J_c
c              write(*,*) '   Shift for previous Jv ', Nsl_sd(Jv)*Ns_wf
        DO J = 1, Nth-1
            If( ID_LIM(J_c,J,2) == 0 ) CYCLE
            Iks_c  = Nsl_sd(Jv)*Ns_wf + ID_LIM(J_c,J,2)
            DO Jw = 1,NS_wf
               Jw1 = (Jw - 1)*MLI  + Iks_c
c               WRITE(*,'(5x,3(2x,I3),2x,I3,A,I3)')
c     $              Jv, J, Jw1, Jf1,' - ',Jf2
c               WRITE(*,'(15x,A,*(1x,E12.4))')'- ',H_Ajs(J,1,1,Jf1:Jf2,1) 
c               WRITE(*,'(15x,A,*(1x,E12.4))')'+ ',H_Ajs(J,1,1,Jf1:Jf2,2)
               K2 = ID_LIM(J_c,J,1)
               DO K = 1, NNv
                  SS1 = SUM(H_Ajs(J,K,Jw,Jf1:Jf2,2)*FOV_WEIGHTS(1:NFA)) 
                  SS2 = SUM(H_Bjs(J,K,Jw,Jf1:Jf2,2)*FOV_WEIGHTS(1:NFA))
                  KK = 0
                  DO Jf = Jf1, Jf2  ;  KK = KK + 1
                     IF( Jf==J_c ) CYCLE
                     IF( K2==0.AND.ID_LIM(Jf,J,1) /= 0 ) THEN
                        SS1 = SS1 + H_Ajs(J,K,Jw,Jf,1)*FOV_WEIGHTS(KK)
                        SS2 = SS2 + H_Bjs(J,K,Jw,Jf,1)*FOV_WEIGHTS(KK)
                     ENDIF
                  ENDDO
c              ----------------------------------------------------
                  ADO%Ajs(J,K,Jw1) = SS1
                  ADO%Bjs(J,K,Jw1) = SS2
c              -----------------------------------------------------
               ENDDO  ! over Gaussian angles (K)
           ENDDO  ! over derivatives (Jw)
         ENDDO  ! over layers (J)
      ENDDO  ! over LoS (Jv)
c     
c ***************** Calculate particular solutions *****************
c
      IF( Ad_Gr ) THEN
c         IF( LOG_LEVEL(1) >= N_DSO) 
c         WRITE(LUNIT,*) '**** Full number of AG layers :', NrL-1
c         write(*,*) '   Harmonic Mh =  ', Mh
c         write(*,*) '   NL  = ', NL 
c         write(*,*) '   N_OBS_ALT ', N_OBS_ALT(1)
c         write(*,*) '   NRL = ', NRL 
c         write(*,*) '   N_OBS_AG ', N_OBS_AG

         ADS%Act_ag(:,:) = .FALSE.
c         DO Jv = 1, NLSA  
c            Jvs = Jv*NFA - LW_FOV(Jv) ; MLI = M_LIM(Jvs)
c            Nth = N_below(Jvs)    ; K1  = Nsl_sd(Jv)*Ns_wf            
c            write(*,*) ' Nth = ', Nth
c            IF( GeoD%LVG(IA,Jvs)%C_VA(1) > 0.d0 ) THEN
c               Start_DO = N_OBS_AG ; End_DO = NRL-1
c            ELSE
c               Start_DO = 1 ; End_DO = N_OBS_AG-1
c            ENDIF
c            DO JL = 1, MLI  
c               K1 = K1 + 1
c               DO Jw = 1,NS_wf
c                  Jw1 = (Jw - 1)*MLI + K1
c                  Yla = Lim_gr(Jv,JL,1) ; Ylb = Lim_gr(Jv,JL,2)
cc                        write(*,'(2x,A,*(2x,I3))') 
cc     $                  ' JL, Jw1 ', JL, Jw1, Yla, Ylb
c                  DO I = Start_DO, End_DO
c                     Ya = LAY_GR(I,1) ; Yb = LAY_GR(I,2) + 1
cc                    Ya - number of level in the initial altitude 
cc                         grid corresponding to the top of Ith AG layer
cc                    Yb - number of level in the initial altitude grid 
cc                         corresponding to the bottom of I-th AG layer
cc                     IF( Ya >= Nth .OR. Ya >= Ylb ) EXIT
c                     IF( Yla >= Yb ) CYCLE
c                     ADS%Act_ag(Ya:Yb-1,Jw1) = .TRUE.
c                  ENDDO         ! over main altitude grid (I)
c               ENDDO            ! over number of derivatives (Jw, Jw1)
c            ENDDO               ! over effective angles (JL)
c         ENDDO                  ! over LoS angles (Jv)
         
c        DO I = 1, NL-1
c        write(*,'(2x,I3, *(2x,L))') 
c     $        I, ( ADS%Act_ag(I,K), K = 1, MAXVAL(M_LIM,1))
c        ENDDO
      
         DO Jv = 1, NLSA  ;  Jvs = Jv*NFA - LW_FOV(Jv)
            MLI = M_LIM(Jvs)
            Nth = N_below(Jvs)    ; K1  = Nsl_sd(Jv)*Ns_wf   
            IF( GeoD%LVG(IA,Jvs)%C_VA(1) > 0.d0 ) THEN
               Start_DO = N_OBS_AG ; End_DO = NRL-1
            ELSE
               Start_DO = 1 ; End_DO = N_OBS_AG-1 ! Upward Pseudo-Source
            ENDIF
            DO JL = 1,N_BE(Jvs)  
               Yla = Lim_gr(Jv,JL,1) ; Ylb = Lim_gr(Jv,JL,2)
               K1 = K1 + 1
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + K1
                  DO I = Start_DO, End_DO ! for I-th layer of adaptive grid
                     Ya = LAY_GR(I,1) ; Yb = LAY_GR(I,2) + 1
                     IF( Ya >= Nth .OR. Ya >= Ylb )  EXIT
                     IF( Yla >= Yb ) CYCLE
c                    ... Recurrence relations for tau-depending 
c                        coefficients Aj amd BJ  
                     DO J = Ya+1,Yb-1 
                        ADO%Ajs(J,:,Jw1) = ADO%Ajs(J,:,Jw1) 
     $                       + ADO%Ajs(J-1,:,Jw1)*SDO%Exp_NU_NL(:,J)
                     ENDDO
                     DO J = Yb-2, Ya, -1
                        ADO%Bjs(J,:,Jw1) = ADO%Bjs(J,:,Jw1) 
     $                       + ADO%Bjs(J+1,:,Jw1)*SDO%Exp_NU_NL(:,J)
                     ENDDO
                     CALL MAKE_PART_AG
                  ENDDO  ! over AG sub-layers (I)                  
               ENDDO  ! over number of derivatives (Jw, Jw1)
            ENDDO  ! over limb sub-layers (JL)
c           ------------------------------------------------
            DO JL = N_BE(Jvs)+1, M_LIM(Jvs)  
               Yla = Lim_gr(Jv,JL,1) ; Ylb = Lim_gr(Jv,JL,2)
c               write(*,'(A,2x,I3,2(2x,F8.3))')
c     $              '  JL Lim_gr  ', JL, ZZZ(Yla), ZZZ(Ylb)
               K1 = K1 + 1
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + K1
                  DO I = 1, End_DO
                     Ya = LAY_GR(I,1) ; Yb = LAY_GR(I,2) + 1
                     IF( Ya >= Nth .OR. Ya >= Ylb ) EXIT
                     IF( Yla >= Yb ) CYCLE
c                     WRITE(*,'(A,2x,I3,2(2x,F8.3))')
c     $                    '             ', I, ZZZ(Ya), ZZZ(Yb)
                     CALL MAKE_PART_AG
                  ENDDO ! over AG sub-layers (I)
               ENDDO  ! over number of derivatives (Jw, Jw1)
            ENDDO  ! over limb sub-layers after Th (JL)

         ENDDO  ! over LoS (Jv)

      ELSE  
         DO Jv = 1, NLSA
            J_c  = Jv*NFA - LW_FOV(Jv)
            N_1 = 1  ! For upward direction of selected LoS:
c           ... For downward direction of selected LoS:
            IF( GeoD%LVG(IA,J_c)%C_VA(1) > 0.d0 ) N_1 = N_OBS_ALT(1) 
c            WRITE(*,*) '   Particular solution levels: ', N_1, Nth-1 
            MLI = M_LIM(J_c) ; Nth = N_below(J_c) ; K1 =Nsl_sd(Jv)*Ns_wf
            DO JL = 1,N_BE(J_c)  
               K1 = K1 + 1
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + K1
                  DO I = N_1, Nth-1 
                     Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
                     CALL MAKE_PART                     
                  ENDDO  ! over layers (I)
               ENDDO  ! over number of derivatives (Jw, Jw1)
            ENDDO  ! over limb sub-layers before Th (JL)
c            WRITE(*,*) '   Particular solution levels:  1 ', Nth-1 
            DO JL = N_BE(J_c)+1, M_LIM(J_c)  
               K1 = K1 + 1
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + K1
                  DO I = 1, Nth-1 
                     Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
                     CALL MAKE_PART
                  ENDDO  ! over layers (I)
               ENDDO  ! over number of derivatives (Jw, Jw1)
            ENDDO  ! over limb sub-layers after Th (JL)   
         ENDDO  ! over LoS (Jv)
      ENDIF

c      WRITE(*,*) '    End test output from "part_sol_arte_spher.f" '
c      WRITE(*,*) '  ===== TEST Part_TU N_adj :', N_adj
c      I = 10
c      I = 1
c      OPEN( 49,file=DIR_OUT//'/Part_tu-sph.dat' )
c      WRITE(49,*) '  Altitude  ',ZZZ(I)
c      DO K = 1, 2*NNv
c         WRITE(49,'(1x,I3,5(1x,e12.4))') 
c     $        K, (ADO%Part_TU(I,K,K1), K1=1,N_adj)
c      ENDDO
c      CLOSE(49)
c      PAUSE  '  In  "part_sol_arte_spher.f" '

      NULLIFY( IND_LS, P_cs, P_sc, SCR, SCA, SCC, UU, VV, Hpa, Hpb,
     $     Norm, Nuj, rs_k, COLSA, X_MOMS, Psim, Psip, R_GL, G_NL,
     $     Po_g )
c      STOP '   In "part_sol_arte_spher.f" '

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS

c***********************************************************************
      SUBROUTINE MAKE_PART_AG

      Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
      UU => ADO%Bjs(Ya,:,Jw1) ; VV => ADO%Ajs(Yb-1,:,Jw1)
      ADO%Part_TU(I,:,Jw1) = MATMUL(G_NL(Ya,:,NNv+1:NNv+Jro),UU(1:Jro))
      ADO%Part_TD(I,:,Jw1) = MATMUL(G_NL(Ya,:,    1:Jro    ),VV(1:Jro) )
c     ... Including imaginary separation constants
      DO J = Jro+1, Jro+2*Jco, 2
         ADO%Part_TU(I,:,Jw1) = ADO%Part_TU(I,:,Jw1) + 2.d0*
     $        ( G_NL(Ya,:,NNv+J)  *UU(J) - G_NL(Ya,:,NNv+J+1)*UU(J+1))
         ADO%Part_TD(I,:,Jw1) = ADO%Part_TD(I,:,Jw1) + 2.d0*
     $        ( G_NL(Ya,:,J)  *VV(J) - G_NL(Ya,:,J+1)*VV(J+1))
      ENDDO  ! over imaginary separation constants (J)
      
      END SUBROUTINE MAKE_PART_AG

c***********************************************************************
c       This subroutine is simular to "scat_matr.f" used within
c                   routine "phase_matr.f" 
c              (see comments and explanations therein) 
c***********************************************************************       
      SUBROUTINE SCAT_MATR_SPH(Ang1, N_ang1, Ang2, N_ang2, Uev)
      USE ST_VARS,     ONLY: IDM, LDM
c     ------------------------------------------------------------------

      CHARACTER (LEN=3), INTENT(IN) ::  Ang1, Ang2
      INTEGER,           INTENT(IN) ::  N_ang1, N_ang2
      REAL (KND), POINTER :: Uev(:,:,:)
c     ... Local variables
      INTEGER  I, J, I1, I2, K1, K2, Ks1, Kp1, Ks2, Kp2, L
      REAL (KND)  S1, Pp(Leg,3,3), Lev(Nvs*N_ang1,Nvs*N_ang2,2), M4(4)
      REAL (KND), POINTER :: Po_a(:,:), Por_a(:,:), Pot_a(:,:),
     $     Po_i(:,:), Por_i(:,:), Pot_i(:,:)
c     ------------------------------------------------------------------

      Pp = 0.d0  ;  Lev = 0.d0
c     ... Select polynomial for angle mu_i
      SELECT CASE( Ang1 )
      CASE('Gau')
         Po_i  => Plm_G(Mh,Mh:Lma,:) 
         Por_i => Dp2_G(Mh,Mh:Lma,:) ! Rlm(mu) Gauss
         Pot_i => Dm2_G(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      CASE('LoS')
         Po_i  => Plm_L(Jv,Mh:Lma,:) ! 
         Por_i => Dp2_L(Jv,Mh:Lma,:) ! Rlm(mu) Line-of-sight
         Pot_i => Dm2_L(Jv,Mh:Lma,:) ! Tlm(mu) --,--
      END SELECT

c     ... Select polynomial for angle mu_a
      SELECT CASE( Ang2 )
      CASE('LoS')
c        ... The polynomial are calculated in "gt_hi_wi.f" for 
c            the angles corresponding to the viewing directions
         Po_a  => Plm_L(Jv,Mh:Lma,:) ! 
         Por_a => Dp2_L(Jv,Mh:Lma,:) ! Rlm(mu) Line-of-sight
         Pot_a => Dm2_L(Jv,Mh:Lma,:) ! Tlm(mu) --,--
      CASE('Sol')
         Po_a  => Plm_S(Mh,Mh:Lma,:)
         Por_a => Dp2_S(Mh,Mh:Lma,:) ! Rlm(mu) Solar
         Pot_a => Dm2_S(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      END SELECT

      DO K1 = 1,N_ang1
         Ks1 = 4*(K1-1) ; Kp1 = Ks1 + 1
         DO K2 = 1,N_ang2
            Ks2 = 4*(K2-1) ; Kp2 = Ks2 + 1
c           ... Calculate elements of the upper-left rectangle 
c               matrix 3x3 U^m_l given by Eq.(U-lm) in STB
            Pp(1:L1,1,1) = Po_i (1:L1,K1)*Po_a (1:L1,K2)
            Pp(1:L1,1,2) = Por_i(1:L1,K1)*Po_a (1:L1,K2)
            Pp(1:L1,1,3) =-Pot_i(1:L1,K1)*Po_a (1:L1,K2)

            Pp(1:L1,2,2) = Por_i(1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,2,3) =-Pot_i(1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,3,3) = Pot_i(1:L1,K1)*Pot_a(1:L1,K2)
            L = 1
            DO I = 1,3  ;  DO J = I,3
               Uev(Ks1+I,Ks2+J,1 ) =
     $              SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,I,J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,I,J)
                  I2 = I2 + 2
               ENDDO
               Uev(Ks1+I,Ks2+J,2 ) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |* * * 0|  
c                   |- * * 0|  
c                   |- - * 0|  
c                   |0 0 0 0|  
            Pp(1:L1,2,1) =  Po_i (1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,3,1) = -Po_i (1:L1,K1)*Pot_a(1:L1,K2)
            Pp(1:L1,3,2) = -Por_i(1:L1,K1)*Pot_a(1:L1,K2)
            L = 2
            DO I = 1,2  ; DO J = I+1,3
               Uev(Ks1+J,Ks2+I,1) =
     $              SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,J,I) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,J,I)
                  I2 = I2 + 2
               ENDDO
               Uev(Ks1+J,Ks2+I,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by + 
c               is performed after above do-loop, i.e.,                
c                   |- - - 0|  
c                   |+ - - 0|  
c                   |+ + - 0|  
c                   |0 0 0 0|  

c           ... Uev(:,:1) contains at this step upper-left rectangle matrix 
c                 (3x3) of the matrix U^{ev}_m given by Eq.(Um-ev) in STB;
c               Uev(:,:2) contains at this step upper-left rectangle matrix 
c                 (3x3) of the matrix U^{od}_m given by Eq.(Um-od) in STB
            DO I = 1,3
               DO J = 1,3
                  Uev(Ks1+I,Ks2+J,1) = 
     $                 Uev(Ks1+I,Ks2+J,1) + Uev(Ks1+I,Ks2+J,2)
                  Uev(Ks1+I,Ks2+J,2) = 
     $                 Uev(Ks1+I,Ks2+J,1) - 2.d0*Uev(Ks1+I,Ks2+J,2)
               ENDDO
            ENDDO
c           ... After summation we have:
c               Uev(:,:1) = U^{ev}_m + U^{od}_m
c               Uev(:,:2) = U^{ev}_m - U^{od}_m,
c               and matrices contain product of functions                
c               (Greek symbols not shown):
c                   | Pi*Pa  Ri*Pa -Ti*Pa  0|  
c                   | Pi*Ra  Ri*Ra -Ti*Ra  0|  
c                   |-Pi*Ta -Ri*Ta  Ti*Ta  0|  
c                   |   0      0      0    0|  
c               Attention! This matrix is transposed w.r.t. to  
c               the matrix U^m_l given by Eq.(U-lm) in STB.
c               It is done to obtain the matrices describing
c               scattering process:
c                   Ang1 ---> Ang2 and Ang1 ---> -Ang2

c           ... Calculate elements of the lower-right rectangle 
c               matrix 3x3 L^m_l given by Eq.(L-lm) in STB
            Pp(1:L1,1,2:3) = -Pp(1:L1,1,2:3)
            L = 1
            DO I = 1,3  ;  DO J = 1,I
               Lev(Kp1+I,Kp2+J,1) = SUM(
     $              X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J)
                  I2 = I2 + 2
               ENDDO
               Lev(Kp1+I,Kp2+J,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO            
c           ... The calculation of Lev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |0 0 0 0|  
c                   |0 * - -|  
c                   |0 * * -|  
c                   |0 * * *|  
            L = 3
            DO I = 1,2  ; DO J = I+1,3
               Lev(Kp1+I,Kp2+J,1) = SUM(
     $              X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J)
                  I2 = I2 + 2
               ENDDO
               Lev(Kp1+I,Kp2+J,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO               
c           ... The calculation of Lev matrix elements matched by +  
c               is performed after above do-loop, i.e., 
c                   |0 0 0 0|  
c                   |0 - + +|  
c                   |0 - - +|  
c                   |0 - - -|  

c           ... Lev(:,:1) contains at this step lower-right rectangle matrix 
c                 (3x3) of the matrix L^{ev}_m given by Eq.(Lm-ev) in STB;
c               Lev(:,:2) contains at this step lower-right rectangle matrix 
c                 (3x3) of the matrix L^{od}_m given by Eq.(Lm-od) in STB
            DO I = 1,3
               DO J = 1,3
                  Lev(Kp1+I,Kp2+J,1) = 
     $                 Lev(Kp1+I,Kp2+J,1) + Lev(Kp1+I,Kp2+J,2)
                  Lev(Kp1+I,Kp2+J,2) = 
     $                 Lev(Kp1+I,Kp2+J,1) - 2.d0*Lev(Kp1+I,Kp2+J,2)
               ENDDO
            ENDDO
c           ... After summation:
c               Lev(:,:1) = L^{ev}_m + L^{od}_m
c               Lev(:,:2) = L^{ev}_m - L^{od}_m
c               Matrices contain product of functions
c               (Greek symbols not shown):
c                   | 0    0      0      0  |  
c                   | 0  Ti*Ta -Ri*Ta -Pi*Ta|  
c                   | 0 -Ti*Ra  Ri*Ra  Pi*Ra|  
c                   | 0  Ti*Pa -Ri*Pa  Pi*Pa|  
c               Attention! This matrix is transposed and multiplied by 
c               the matrix D4 (diag{1,1,1,-1}) from left and right w.r.t.   
c               to the matrix L^m_l given by Eq.(L-lm) in STB.
c               It is done to obtain the matrices describing
c               scattering process: 
c                   Ang1 ---> Ang2 and Ang1 ---> -Ang2
               
         ENDDO                  ! over any angles (K2)
      ENDDO                     ! over Gaussian angles (K1)
c     ... After following summation:
c         Lev(:,:,1) =  [U^{ev}_m + U^{od}_m] +  [L^{ev}_m + L^{od}_m]
      Lev(:,:,1) = Uev(:,:,1) + Lev(:,:,1)

c     --------------------------------------------------------------------
c     -------------------  Results ---------------------------------------
c
c     ... Uev(:,:,1) = [U^{ev}_m - U^{od}_m] - [L^{ev}_m - L^{od}_m]
      Uev(:,:,1) = Uev(:,:,2) - Lev(:,:,2)
c         transform the Stokes vector: Ang1 ---> -Ang2
c            This matrix is not multiplied by D=diag{1,1,-1,-1} 
c            from left (see Eq.G-gua+mu0 in STB)! 

c     --------------------------------------------------------------------
c     ... Uev(:,:,2) =  [U^{ev}_m + U^{od}_m] +  [L^{ev}_m + L^{od}_m]
      Uev(:,:,2) = Lev(:,:,1)
c         transform the Stokes vector: Ang1 ---> Ang2
c     --------------------------------------------------------------------

      SELECT CASE( Ang2 )
      CASE('Sol')
c        ... To obtain the same meaning of last arguments 1 and 2
c            as in previous versions of the program
         Uev(:,:,2) = Uev(:,:,1)
c            Matrix R_GS(:,:,2), Egs(:,:,2), P02(:,:,:) describes 
c            scattering process  Gauss --> -Sol_ang.
c            This matrix is not multiplied by D=diag{1,1,-1,-1} 
c            from left (see Eq.(Qm-sol) in STB)! 

         Uev(:,:,1) = Lev(:,:,1)
c            Matrix R_GS(:,:,1), Egs(:,:,1), P01(:,:,:) describes 
c            scattering process  Gauss --> Sol_ang and 
c                                LoS   --> Sol_ang

      END SELECT

      NULLIFY( Po_a, Por_a, Pot_a, Po_i, Por_i, Pot_i )

      RETURN
      END SUBROUTINE SCAT_MATR_SPH

c **********************************************************************
      SUBROUTINE MAKE_PART
c      write(*,*) '   ________  Jw1 for Part_TU _______ ', Jw1
      UU => ADO%Bjs(I,:,Jw1) ; VV => ADO%Ajs(I,:,Jw1)
      ADO%Part_TU(I,:,Jw1) = MATMUL( G_NL(I,:,NNv+1:NNv+Jro),UU(1:Jro) )
      ADO%Part_TD(I,:,Jw1) = MATMUL( G_NL(I,:,    1:Jro)    ,VV(1:Jro) )

c     ... Including imaginary separation constants
      DO J = Jro+1, Jro+2*Jco, 2
         ADO%Part_TU(I,:,Jw1) = ADO%Part_TU(I,:,Jw1) + 2.d0*
     $        ( G_NL(I,:,NNv+J)  *UU(J) - G_NL(I,:,NNv+J+1)*UU(J+1))
        ADO%Part_TD(I,:,Jw1) = ADO%Part_TD(I,:,Jw1) + 2.d0*
     $        ( G_NL(I,:,J)  *VV(J) - G_NL(I,:,J+1)*VV(J+1))
      ENDDO     ! over imaginary separation constants (J)

      END SUBROUTINE MAKE_PART

c **********************************************************************

      SUBROUTINE MAKE_ABJ

      DO Jg = 1,NN
         Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
         DO K2 = 1,Nvs
            Qp_t(Kg1+K2-1) = SUM( 
     $           ( Wup*S_EgL(J,  Kg1:Kg2,Ks+K2,1,Jv)
     $           + Wdo*S_EgL(J+1,Kg1:Kg2,Ks+K2,1,Jv) )*Ainc(Jw,:) )
            Qm_t(Kg1+K2-1) = SUM( 
     $           ( Wup*S_EgL(J,  Kg1:Kg2,Ks+K2, 2,Jv)
     $           + Wdo*S_EgL(J+1,Kg1:Kg2,Ks+K2, 2,Jv) )*Ainc(Jw,:) )
         ENDDO  ! over Stokes components (K2)
      ENDDO  ! over Gaussian angles (Jg)

      IF( Nvs > 1 ) THEN
         Qp_t(4:NNv:Nvs) = -Qp_t(4:NNv:Nvs) 
         Qm_t(4:NNv:Nvs) = -Qm_t(4:NNv:Nvs) 
      ENDIF

c     ... Real separation constants
      DO L = 1,Jro
         aj(L) = SUM( Psip(:,L)*QP_T + Psim(:,L)*QM_T )
         bj(L) = SUM( Psim(:,L)*QP_T + Psip(:,L)*QM_T )
      ENDDO     ! over real separation constants (L)
      aj(1:Jro) = aj(1:Jro)*0.5/Norm(1:Jro)  
      bj(1:Jro) = bj(1:Jro)*0.5/Norm(1:Jro)
c     ... Complex separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2
c        ... Aj
         C2 = DCMP( Norm(K1),Norm(K1+1) ) 
         A1 = SUM(Psip(:,K1)  *QP_t + Psim(:,K1)  *QM_t )
         B1 = SUM(Psip(:,K1+1)*QP_t + Psim(:,K1+1)*QM_t)
         C1 = DCMP( A1,B1 )
         aj(K1) = DBLE(C1/C2)*0.5 ; aj(K1+1) = AIMAG(C1/C2)*0.5
c        ... Bj
         A1 = SUM(Psim(:,K1)  *QP_T + Psip(:,K1)  *QM_T )
         B1 = SUM(Psim(:,K1+1)*QP_T + Psip(:,K1+1)*QM_T)
         C1 = DCMP( A1,B1 )
         bj(K1) = DBLE(C1/C2)*0.5  ; bj(K1+1) = AIMAG(C1/C2)*0.5
      ENDDO ! over complex separation constants (K1)
      aj(:) = aj(:)*He1  ;  bj(:) = bj(:)*He1

      END SUBROUTINE MAKE_ABJ
c **********************************************************************

      SUBROUTINE SC_FUNC_SPH(Nco, Ar, Ear, Sc, Esc, Dtl, Cres, Sres )
      INTEGER      Nco, Jc, K1
      REAL (KND)   Sc, Esc, Dtl, Zn, Ck, V_i, V_r, H1
      REAL (KND), DIMENSION(NNv) ::  Cres, Sres, Ar, Ear
      COMPLEX (kind(0.0d0)) :: Muc

c     ... real separation constants
      Sres(1:Jro) = Ar(1:Jro)*Sc*(1.d0 - Ear(1:Jro)*Esc)
     $     /(Ar(1:Jro) + Sc)
      WHERE( REAL(Ar(1:Jro)) /= REAL(Sc) )
         Cres(1:Jro) = Ar(1:Jro)*Sc*(Ear(1:Jro) - Esc)/(Ar(1:Jro) - Sc)
      ELSEWHERE
         Cres(1:Jro) = Ear(1:Jro)*Dtl
      END WHERE
c     ... complex separation constants
      DO K1 = Jro+1, Jro+2*Jco, 2
         C1  = DCMP( Ar(K1),Ar(K1+1) ) ; Cnu = DCMP( Ear(K1),Ear(K1+1) )
         C2 =  C1*Sc*(Cnu - Esc)/(C1 - Sc)
         Cres(K1) = DBLE(C2) ; Cres(K1+1) = AIMAG(C2)
         C2 = C1*Sc*(1.d0 - Cnu*Esc)/(C1 + Sc)
         Sres(K1) = DBLE(C2) ; Sres(K1+1) = AIMAG(C2)
      ENDDO

c     ... Contribution of additional terms into Sres and Cres 
c         functions caused by approximation of transmission along 
c         LoS in the spherical atmosphere by the sum of exponents 
c         (NCO > 1 in this case).
      Zn = -1
      H1 = Zn*Esc
      DO Jc = 1,Nco-1
         Ck = Dtl/DBLE(Jc)/Pie ; Muc = Sc*Ck/DCMP( Ck,-Sc )
c        ... Real separation constants
         Cres(1:Jro) = Cres(1:Jro) 
     $        + AIMAG( Muc*Ar(1:Jro)/(Muc-Ar(1:Jro)) )
     $        *rs_k(Jc)*(H1 - Ear(1:Jro))
         Sres(1:Jro) = Sres(1:Jro) 
     $        + AIMAG( Muc*Ar(1:Jro)/(Muc+Ar(1:Jro)) )
     $        *rs_k(Jc)*(1.d0 - H1*Ear(1:Jro))
c        ... Complex separation constants
         DO K1 = Jro+1, Jro+2*Jco, 2
            C1 = DCMP( Ar(K1),Ar(K1+1) )
            C2 = ( CONJG(Muc)*C1/(CONJG(Muc)-C1)
     $           - Muc*C1/(Muc-C1) )*0.5d0
            V_i = AIMAG(C2)*Ear(K1) +  DBLE(C2)*Ear(K1+1)
            V_r =  DBLE(C2)*Ear(K1) - AIMAG(C2)*Ear(K1+1)
            Cres(K1)   = Cres(K1)   - rs_k(Jc)*(AIMAG(C2)*H1 - V_i)
            Cres(K1+1) = Cres(K1+1) + rs_k(Jc)*( DBLE(C2)*H1 - V_r)
            Sres(K1)   = Sres(K1)   - rs_k(Jc)*(AIMAG(C2) - H1*V_i)
            Sres(K1+1) = Sres(K1+1) + rs_k(Jc)*( DBLE(C2) - H1*V_r)
         ENDDO
         Zn = -Zn 
      ENDDO

      END SUBROUTINE SC_FUNC_SPH

c **********************************************************************
      END 
