C**********************************************************************
C
C	SCIATRAN - Version 4.7 - Last modification: 23 Nov 2023
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
c
c    Calculation of the particular solution corresponding to the 
c          vibrational Raman scattering and fluorescence 
c                  source functions in water 
c                    
c *********************************************************************
      SUBROUTINE PART_SOL_VRS(JPOS, Mh, NN, NSZA, NL, DTR, ALW, 
     $     Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, PART_D, SDO, SVR)
c     ... Call in "solv_dom.f"

      USE GRIDS,       ONLY: ZZZ
      USE GT_POLYN,    ONLY: AG_W, XM_W
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nvs, Ng1_brf, Ns_brf, 
     $                       NN_pla
      USE IRRAD,       ONLY: Fbe, IRRAD_FLAG, SOL_FLUX, WL_IRRAD_DIM
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_ESO, N_XSO
      USE ST_VARS,     ONLY: Pie, INEL_PROC, Kvs
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, VRS_TYPE
      USE VRS,         ONLY: VRS_Egg, VRS_Egs, Integr_FRF, K_nu, Nsig, 
     $                       Sig_FRF, N_FRF, Dnu_FRF, WLG_Chl, Sig_Chl,
     $                       WL_QEred, WL_QEblu, Qe_Chl, N_phm
      USE WATER,       ONLY: INCL_INEL, CHL_UW, SIPI, Fua_c, Hua_c, 
     $                       N_FC
c     ------------------------------------------------------------------

c     ------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
c     ----------------------
      INTEGER,    INTENT(IN):: JPOS, Mh, NN, NSZA, NL
      REAL (KND), INTENT(IN):: MuS(NSZA), DTR(NL), ALW(NL-1),
     $     Norm_lay(NN,NL-1), Psim_lay(NN,NN,NL-1), Psip_lay(NN,NN,NL-1)
      TARGET :: Psim_lay, Psip_lay
      REAL (KND) PART_U(NL,2*NN,Kvs*NSZA), PART_D(NL,2*NN,Kvs*NSZA)
c     -----------------------------------------------------
c     ... Local variables 
c     -------------------
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'PART_SOL_VRS'
      LOGICAL  INTEGR(SVR%N_AUX)
      INTEGER  I, J, K, Js, LL1, Lr1, Iau, Jact, Ni1, Ni2, Ka, Nsu, Isav
      REAL (KND) Mu0, Nuk, E_Nk, DTA, Cmk, Smk, Rel, D1, DEL,
     $     Nrk, S_Ram, WL_MAX, WL_MIN, WL_ACT, S1, S2, Hha
      REAL (KND), DIMENSION (NN*Nvs) :: E_Nj, Nuj, Sjk, Cjk, Dum,
     $     Bb_kj, Aa_kj, aakj, bakj, abkj, bbkj, SBb_kj, SAa_kj
      REAL (KND), DIMENSION (NN*Nvs,NN*Nvs) :: Faj, Fbj 
      REAL (KND) Cmj(NN,NSZA), Smj(NN,NSZA), Em0(NSZA), UU(NN,NSZA),
     $     VV(NN,NSZA), akd(NN,NSZA), bkd(NN,NSZA), D_D(N_FRF), 
     $     Nuc(N_FRF), Qp(NN,NSZA), Qm(NN,NSZA)
      REAL (KND), ALLOCATABLE :: Phot_Wat(:), Z_DEP(:,:), DEC(:,:), 
     $     S_Gpp(:,:,:), S_Gmp(:,:,:), S_Qp(:,:,:), S_Qm(:,:,:), 
     $     Gpp(:,:), Gmp(:,:)
      TARGET :: Faj, Fbj
      INTEGER, ALLOCATABLE :: N_co(:)
      LOGICAL, ALLOCATABLE :: L_r(:), L_b(:), L_v(:)
      REAL (KND), POINTER :: Psip(:,:), Psim(:,:), Ex_sol(:), Fip(:,:), 
     $     Fim(:,:), ajs(:,:), bjs(:,:), Apj(:,:), Bpj(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IF( LOG_LEVEL(1) <= N_XSO ) THEN
         WRITE(LUNIT,'(16x,A)') 'as input in this routine'
         WRITE(LUNIT,*) '      PART_U:', SUM(PART_U)
         WRITE(LUNIT,*) '      PART_D:', SUM(PART_D)
      ENDIF

c     ... All actual wavelengths are located after all excitation 
c         wavelengths, therefore Jact counts here actual wavelengths
      Jact = SVR%Jact

c     ... WL_act is the current actual wavelength
      WL_act = SVR%WL_EXT(JPOS)

      Ni1 = 1  ;  Ni2 = SVR%N_AUX  ;  DEL = 20.d0  ;  Nsu = N_phm

      ALLOCATE( Phot_wat(Ni2), N_co(N_phm), L_r(Ni2), L_b(Ni2),
     $     S_Qp(N_phm,NN*Nvs,NSZA), S_Qm(N_phm,NN*Nvs,NSZA), L_v(Ni2) ) ! A

c     ... SIPI is set to .TRUE. if only single inelastic
c         process in water should be accounted for
      IF( .NOT.SIPI ) THEN
         ALLOCATE( S_Gpp(N_phm,NN*Nvs,NN*Nvs), Gpp(NN*Nvs,NN*Nvs), 
     $        S_Gmp(N_phm,NN*Nvs,NN*Nvs), Gmp(NN*Nvs,NN*Nvs) ) ! A
      ENDIF

      Phot_Wat(:) = 1.d0
      L_b(:) = .FALSE. ; L_r(:) = .FALSE. ; L_v(:) = .FALSE.
      INTEGR(:) = .FALSE. 

c     ... If contribution of different inelastic processes is
c         accounted for than in arrays DEC(:,:) and Z_DEP(:,:) 
c         below elements 1 of first dimension contain parameters 
c     of VRS
      SELECT CASE( TRIM(INEL_PROC) )
c     ... Vibrational Raman scattering
c     --------------------------------
      CASE('VRS')
c        ... Quadrature coefficients for integration over excitation
c            range are introduced in "ext_wl_vrs.f"
         ALLOCATE( DEC(1,Ni2), Z_DEP(1,NL-1) )
         CALL VRS_PHASE(1) ; CALL INT_RANGE_VRS(1) ; N_co(1) = 1
         INTEGR(:) = L_v(:) 
c         WRITE(*,*) '  part_sol_vrs COUNT(L_v) =  ',COUNT(L_v)

c     ... Chlorophyll fluorescence
c     ----------------------------
      CASE('F_Chl')
         ALLOCATE( DEC(1,Ni2), Z_DEP(1,NL-1) )
         CALL Flu_PHASE(1) ; CALL INT_RANGE_Fr ;  N_co(1) = 1
         INTEGR(:) = L_r(:) ; Ka = COUNT(INTEGR(:))
c        ... For this actual wavelength no emission contribution 
         IF( Ka==0 ) RETURN
         CALL SET_CC_Fr(1)

c     ... CDOM fluorescence
c     ----------------------------
      CASE('F_CDOM')
c        ... N_FC is the number of fluorescent components
         ALLOCATE( DEC(N_FC,Ni2), Z_DEP(N_FC,NL-1)  )
         CALL Flu_PHASE(1) ; CALL INT_RANGE_Fb ; N_co(1) = N_FC
         INTEGR(:) = L_b(:) ; Ka = COUNT(INTEGR(:))
c        ... For this actual wavelength no emission contribution 
         IF( Ka==0 ) RETURN
         CALL SET_CC_Fb(1)

c     ... Chlorophyll and CDOM fluorescence (Not checked for ACR)
c     -------------------------------------
      CASE('Fr_Fb')
         ALLOCATE(DEC(3,Ni2), Z_DEP(3,NL-1))
         CALL Flu_PHASE(1) ; CALL INT_RANGE_Fr ; CALL INT_RANGE_Fb
         N_co(1) = 3 
         INTEGR(:) = L_b(:).OR.L_r(:) ; Ka = COUNT( INTEGR(:) )
c        ... For this actual wavelength no emission contribution 
         IF( Ka==0 ) RETURN
         CALL SET_CC_Fr(1) ; CALL SET_CC_Fb(2)

c     ... Vibrational Raman scattering amd CDOM fluorescence
c     ------------------------------------------------------
      CASE('VRS_Fb')
         ALLOCATE( DEC(N_FC+1,SVR%N_AUX),Z_DEP(N_FC+1,NL-1) )
         CALL VRS_PHASE(1) ; CALL INT_RANGE_VRS(3) ; N_co(1) = 1 
         IF( Mh==1 ) THEN
            CALL Flu_PHASE(2) ; CALL INT_RANGE_Fb  ; N_co(2) = N_FC+1
            INTEGR(:) = L_v(:).OR.L_b(:) ; Ka = COUNT( INTEGR(:) )
            CALL SET_CC_Fb(2)
         ELSE
            Nsu = 0 ; INTEGR(:) = L_v(:)
         ENDIF

c     ... Vibrational Raman scattering amd chlorophyll fluorescence
c     -------------------------------------------------------------
      CASE('VRS_Fr')
         ALLOCATE( DEC(2,SVR%N_AUX),Z_DEP(2,NL-1) )
         CALL VRS_PHASE(1) ; CALL INT_RANGE_VRS(1) ; N_co(1) = 1 
         IF( Mh==1 ) THEN
            CALL Flu_PHASE(2) ; CALL INT_RANGE_Fr  ; N_co(2) = 2
            INTEGR(:) = L_v(:).OR.L_r(:) ; Ka = COUNT( INTEGR(:) )
            CALL SET_CC_Fr(2)
         ELSE
            Nsu = 0 ; INTEGR(:) = L_v(:)
         ENDIF

c     ... Vibrational Raman scattering, chlorophyll and CDOM fluorescence
c               (Not checked for ACR)
c     -------------------------------------------------------------------
      CASE('VRS_Fr_Fb')
         ALLOCATE( DEC(4,SVR%N_AUX),Z_DEP(4,NL-1) )
         CALL VRS_PHASE(1) ; CALL INT_RANGE_VRS(3) ; N_co(1) = 1 
         IF( Mh==1 ) THEN
            CALL Flu_PHASE(2) ; CALL INT_RANGE_Fr  ; CALL INT_RANGE_Fb
            INTEGR(:) = L_v(:).OR.L_b(:).OR.L_r(:) 
            Ka = COUNT(INTEGR(:))
            N_co(2) = 4
            CALL SET_CC_Fr(2) ; CALL SET_CC_Fb(3)
         ELSE
            Nsu = 0 ; INTEGR(:) = L_v(:)
         ENDIF

      END SELECT

      IF( IRRAD_FLAG ) THEN
         Phot_Wat(:) = SOL_FLUX(1:Ni2)/SOL_FLUX(JPOS)
         IF( TRIM(WL_IRRAD_DIM(2)) == 'phot/s') Phot_Wat(:) = 
     $        Phot_Wat(:)*WL_ACT/SVR%WL_EXT(1:Ni2)
      ENDIF
      
c      DO Iau = Ni1,Ni2 
c         IF( .NOT.INTEGR(Iau) ) CYCLE
c         write(*,'(2x,I5,3(2x,L),2x,I4)')
c     $        Iau, L_v(Iau), L_r(Iau), INTEGR(Iau), SVR%Isav(Iau)
c      ENDDO
c     
c     ---------------- Main calculations block ------------------
c
      DO I = 1,NL - 1   ! Over all layers in the ocean
         IF( SDO%MASK_NL(I) ) 
     $        STOP "  VRS can't be calculated for pure scattering "
         Psim => Psim_lay(:,:,I)
         Psip => Psip_lay(:,:,I)
         LL1 = (I-1)*2*NN + 1 ; LR1 = (I-1)*2*NN + NN ; DTA = DTR(I+1)
         UU = 0.d0  ;  VV = 0.d0

c        ... Do-loop over excitation wavelength region (including 
c            rotational Raman if present)
         DO Iau = Ni1,Ni2
            IF( .NOT.INTEGR(Iau) ) CYCLE

            Isav = SVR%Isav(Iau)
            S1 = SUM( DEC(1:N_co(1),Iau)*Z_DEP(1:N_co(1),I) )
c           ... Sum contributions of different phase functions
            Qp (:,:) = S1*S_Qp (1,:,:) ; Qm (:,:) = S1*S_Qm (1,:,:)
            IF( .NOT.SIPI ) THEN 
               Gpp(:,:) = S1*S_Gpp(1,:,:) ; Gmp(:,:) = S1*S_Gmp(1,:,:)
            ENDIF

c           ... Nsu = 1 if single inelastic process is included
            DO K = 2,Nsu
               S2 = SUM( DEC(K:N_co(K),Iau)*Z_DEP(K:N_co(K),I) )
               Qp (:,:) = Qp (:,:) + S2*S_Qp (K,:,:)
               Qm (:,:) = Qm (:,:) + S2*S_Qm (K,:,:)
               Gpp(:,:) = Gpp(:,:) + S2*S_Gpp(K,:,:)
               Gmp(:,:) = Gmp(:,:) + S2*S_Gmp(K,:,:)
            ENDDO

            IF( SIPI ) THEN
               Faj(:,:) = S1*SVR%GNp(Isav,Mh,I,:,:)
               Fbj(:,:) = S1*SVR%GNm(Isav,Mh,I,:,:)
            ELSE
               Fip => SVR%GNp(Isav,Mh,I,:,:) 
               Fim => SVR%GNm(Isav,Mh,I,:,:)
               DO K = 1,NN
                  Faj(:,K) = MATMUL(Gpp(:,:),Fip(:,K)) 
     $                 + MATMUL(Gmp(:,:),Fim(:,K))
                  Fbj(:,K) = MATMUL(Gpp(:,:),Fim(:,K))  
     $                 + MATMUL(Gmp(:,:),Fip(:,K))
               ENDDO
            ENDIF
            
c           ... Solar exponent at excitation wavelength
            Ex_sol => SVR%EXS_AUX(Isav,I,:)
c           ... 
            ajs => SVR%Aj(Isav,Mh,:,I,:)
            bjs => SVR%Bj(Isav,Mh,:,I,:)

c           ... Integration constants Aj and Bj for excitation WL
c                              separation const, SZA
            Apj => SVR%MVEC(Isav,Mh,  LL1:LR1   ,:)
            Bpj => SVR%MVEC(Isav,Mh,LR1+1:LR1+NN,:)

            S_Ram = Phot_Wat(Iau)

c           ... Relation of extinction coefficients actual/excitation
            Rel = ALW(I)/SVR%EXT_AUX(Isav,I)

c           ... Extract separation constants for a excitation wavelength
            Nuj(:) = SVR%Nuj_EXC(Isav,Mh,:,I)*Rel

            E_Nj(:) = DEXP( -DTA/Nuj(:) )
            Em0(:)  = DEXP( -DTA/( Rel*Mus(:)) )

            DO Js = 1,NSZA
               Mu0 = Rel*Mus(Js) ! SZA in water
               WHERE( REAL(Nuj(:)) /= REAL(Mu0) )
                  Cmj(:,Js) = Nuj(:)*Mu0/(Nuj(:)-Mu0)*(E_Nj(:)-Em0(Js)) 
               ELSEWHERE
                  Cmj(:,Js) = E_Nj(:)*DTA 
               END WHERE            
               Smj(:,Js)=Nuj(:)*Mu0/(Nuj(:)+Mu0)*(1.d0-E_Nj(:)*Em0(Js)) 
c              ... Contribution of the single scattering emission source 
c                  function
               DO K = 1,NN
                  akd(K,Js) = SUM(Psip(:,K)*Qp(:,Js)+Psim(:,K)*Qm(:,Js))
                  bkd(K,Js) = SUM(Psim(:,K)*Qp(:,Js)+Psip(:,K)*Qm(:,Js))
               ENDDO  ! over separation constants (K)
               akd(:,Js) = akd(:,Js)*0.5/Norm_lay(:,I) 
               bkd(:,Js) = bkd(:,Js)*0.5/Norm_lay(:,I)
            ENDDO  ! over solar angles (Js)

            DO K = 1,NN ! Over separation constant of actual Wl
c              ... Separation const for actual WL
               Nuk  = SDO%Nuj_lay(K,I)  ;  E_Nk = DEXP( -DTA/Nuk )
               Nrk = 0.5d0/Norm_lay(K,I)

c              ... These coefficients are calculated according to
c                  Eqs.(Saa_kj) and (Sba_kj) in STB
               DO J = 1,NN
                  aakj(J) = SUM(Psip(:,K)*Faj(:,J) + Psim(:,K)*Fbj(:,J)) 
                  bakj(J) = SUM(Psim(:,K)*Faj(:,J) + Psip(:,K)*Fbj(:,J)) 

                  abkj(J) = SUM(Psip(:,K)*Fbj(:,J) + Psim(:,K)*Faj(:,J)) 
                  bbkj(J) = SUM(Psim(:,K)*Fbj(:,J) + Psip(:,K)*Faj(:,J)) 
               ENDDO

               Sjk(:) = Nuj(:)*Nuk/(Nuj(:) + Nuk)*(1.d0 - E_Nj(:)*E_Nk) 
               WHERE( REAL(Nuj(:)) /= REAL(Nuk) )
                  Cjk(:) = Nuj(:)*Nuk/(Nuj(:)-Nuk)*(E_Nj(:) - E_Nk)  
               ELSEWHERE
                  Cjk(:) = E_Nj(:)*DTA 
               END WHERE            

               Dum(:) = Nuk*Nuj(:)/(Nuj(:) + Nuk)
               DO Js = 1,NSZA
                  Mu0 = Rel*Mus(Js) ! SZA in water
                  IF( REAL(Mu0)/= REAL(Nuk) ) THEN  
                     Cmk = Mu0*Nuk/(Mu0 - Nuk)*(Em0(Js) - E_Nk)
                  ELSE
                     Cmk = Em0(Js)*DTA
                  ENDIF
                  Smk = Mu0*Nuk/(Mu0 + Nuk)*(1.d0 - Em0(Js)*E_Nk)
                  
                  IF( REAL(Nuk) == REAL(Mu0) ) THEN
                     Hha = Cmk*DTA
                  ELSE
                     Hha = Nuk*Mu0/(Mu0 - Nuk)*(Cmk - DTA*E_Nk)
                  ENDIF
                  WHERE( REAL(Nuj(:)) /= REAL(Nuk) )
                     Aa_kj(:) = Nuk*Nuj(:)/(Nuj(:)-Nuk)*(Cmj(:,Js)-Cmk)
                     Bb_kj(:) = Nuk*Nuj(:)/(Nuj(:)-Nuk)*(Smj(:,Js)-Smk)
                  ELSEWHERE
                     Aa_kj(:) = Hha
                     Bb_kj(:) = Nuk*Mu0/(Mu0 + Nuk)
     $                    *(Smk - DTA*E_Nk*Em0(Js))
                  END WHERE            
c     ------------------------------------------------------------------
                  SBb_kj(:) = bakj(:)*( Apj(:,Js)*Sjk(:) 
     $                 + ajs(:,Js)*Dum(:)*(Smk - E_Nk*Cmj(:,Js))  /Rel)
     $                 + bbkj(:)*( Bpj(:,Js)*Cjk(:)
     $                            + bjs(:,Js)*Bb_kj(:)  /Rel)
c     ------------------------------------------------------------------
                  SAa_kj(:) = abkj(:)*( Bpj(:,Js)*Sjk(:) 
     $                 + bjs(:,Js)*Dum(:)*(Cmk - E_Nk*Smj(:,Js))  /Rel)
     $                 + aakj(:)*( Apj(:,Js)*Cjk(:)
     $                             + ajs(:,Js)*Aa_kj(:)  /Rel)
c     ------------------------------------------------------------------

c                 ... Perform integration over all excitation wavelengths
                  UU(K,Js) = UU(K,Js) + S_Ram*SUM( SBb_kj(:) )*Nrk
     $                 + S_Ram*bkd(K,Js)*Smk*Ex_sol(Js)/4.d0

                  VV(K,Js) = VV(K,Js) + S_Ram*SUM( SAa_kj(:) )*Nrk
     $                 + S_Ram*akd(K,Js)*Cmk*Ex_sol(Js)/4.d0
               ENDDO  ! over solar angles (Js)
            ENDDO  ! over separation constant of actual Wl (K)
         ENDDO  ! over excitation wavelengths (Iau)
         
         DO Js = 1,NSZA
            Part_U(I,:,Js) = Part_U(I,:,Js)
     $           + MATMUL( SDO%G_NL(I,:,NN+1:2*NN),UU(:,Js) )
            Part_D(I,:,Js) = Part_D(I,:,Js)
     $           + MATMUL( SDO%G_NL(I,:,1:NN),VV(:,Js) )
         ENDDO

      ENDDO  ! over ocean layers (I)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
      SUBROUTINE VRS_PHASE(Nvr)
        INTEGER, INTENT(IN):: Nvr
c       ... For single inelastic process this was done in "st_dom_tau.f"
        IF( .NOT.SIPI ) THEN
c          ... Fourier harmonics (1,2,3) of the Raman phase function 
c              (array VRS_Egg)  were calculated in "phase_function.f" 
c              called in "rtm_init_first.f". First dimension 
c              was divided by XM(:) - cosine of Gauss angles.
           FORALL( K = 1:NN )
             S_Gpp(Nvr,K,:) = VRS_Egg(K,:,1,Mh)*AG_W(:)*0.5d0 ! Mu <=> Mu
             S_Gmp(Nvr,K,:) = VRS_Egg(K,:,2,Mh)*AG_W(:)*0.5d0 ! Mu <=> -Mu
           END FORALL 
        ENDIF
c       ... Coefficients for the inelastically scattered 
c           direct solar light at an excitation wavelength
        S_Qp(Nvr,:,:) = VRS_Egs(:,:,1,Mh) 
        S_Qm(Nvr,:,:) = VRS_Egs(:,:,2,Mh)
      END SUBROUTINE VRS_PHASE

c     ------------------------------------------------------------------
      SUBROUTINE INT_RANGE_VRS(N_out)
      INTEGER, INTENT(IN):: N_out
        Z_DEP(1,:) = 1.d0/ALW(:)

        IF( Integr_FRF ) THEN
c          ... If rotational Raman is presented then second dimension
c              of array SVR%V_FRF(N_out,:,Jact) is full number of 
c              excitation wavelengths for both processes
           DEC(1,:) = SVR%V_FRF(N_out,:,Jact)
c          ... Integration range for a current actual wavelength
           D_D(:) = Nsig*DSQRT(2.d0)*Sig_FRF(:)
           Nuc(:) = K_nu/WL_ACT + Dnu_FRF(:)
           WL_MAX = MAXVAL( K_nu/(Nuc(:) - D_D(:)) )
           WL_MIN = MINVAL( K_nu/(Nuc(:) + D_D(:)) )
c           WRITE(*,'(A,3(2x,F8.3))') 'VRS    :', WL_MIN, WL_MAX
c          ... WL-grid in array SVR%WL_EXT contains excitation  
c              wavelengths (for both Raman if rotational is 
c              present) in 1:SVR%N_aux elements 
           DO Iau = Ni1,Ni2
              IF(SVR%WL_EXT(Iau)>=WL_MIN .AND. SVR%WL_EXT(Iau)<=WL_MAX)
     $             L_v(Iau) = .TRUE. 
           ENDDO
        ELSE
           DEC(1,:) = SVR%V_FRF(N_OUT,:,Jact)
c           DEC(1,:) = 0.d0
           Ni1 = Jact  ;  Ni2 = Jact ; L_v(:) = .TRUE.
        ENDIF
      END SUBROUTINE INT_RANGE_VRS
c **********************************************************************

      SUBROUTINE Flu_PHASE(Nvr)
        INTEGER, INTENT(IN):: Nvr
        CHARACTER (LEN=9), PARAMETER :: R_NAME = 'Flu_PHASE'

        CALL ST_TRACK (R_NAME, 1, 'I')

c       ... It will be accounted for the isotropy of fluorescence
        IF( .NOT.SIPI ) THEN
           FORALL( K = 1:NN )
             S_Gpp(Nvr,K,:) = 1.d0/XM_W(K)*AG_W(:)*0.5d0 ! Mu <=> Mu
             S_Gmp(Nvr,K,:) = 1.d0/XM_W(K)*AG_W(:)*0.5d0 ! Mu <=> -Mu
           END FORALL 
        ENDIF
        S_Qp(Nvr,:,:) = 1.d0
c       ... The same for the single scattering phase function
        FORALL( Js = 1:NSZA ) S_Qp(Nvr,:,Js) = 1.d0/XM_W(:)
        S_Qm = S_Qp
        IF( LOG_LEVEL(1) <= N_XSO ) 
     $       WRITE(LUNIT,*) '      S_Qm:', SUM(S_Qm)

        CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE Flu_PHASE
c     ------------------------------------------------------------------

      SUBROUTINE INT_RANGE_Fr
        WL_MIN = WLG_Chl - 6.d0*Sig_Chl
        WL_MAX = WLG_Chl + 3.d0*Sig_Chl
c       ... Integration takes place over excitation wavelengths if
c           actual wavelength is inside of (+/-)3*sigma range around 
c           the center of the emission band (WLG_Chl)
        IF( WL_ACT>=WL_MIN .AND. WL_ACT<=WL_MAX ) THEN
           DO Iau = Ni1,Ni2
              D1 = SVR%WL_ext(Iau)
              IF(D1>=WL_QEred(1).AND.D1<=WL_QEred(2)) L_r(Iau) = .TRUE.
           ENDDO
        ENDIF
      END SUBROUTINE INT_RANGE_Fr

c     ------------------------------------------------------------------
      SUBROUTINE INT_RANGE_Fb
c       ... No contribution if actual wavelength less than minimal
c           wavelength of the excitation range
        IF( WL_ACT > WL_QEblu(1) ) THEN            
           WL_MAX = DMIN1(WL_QEblu(2),WL_ACT+DEL)
           DO Iau = Ni1,Ni2
              D1 = SVR%WL_EXT(Iau)
              IF(D1>=WL_QEblu(1).AND.D1<=WL_MAX) L_b(Iau) = .TRUE.
           ENDDO
        ENDIF
      END SUBROUTINE INT_RANGE_Fb
c **********************************************************************

      SUBROUTINE SET_CC_Fb(N1)

      USE WATER, ONLY: Nwl_QEm, WL_QEm, F_QEm, Nwl_QEx, WL_QEx, 
     $                 F_QEx, Flu_con, Flu_CDOM
c     ------------------------------------------------------------------
      INTEGER, INTENT(IN):: N1
      INTEGER  I, J, K1, N2
      REAL (KND)  Emm
      REAL (KND), POINTER:: WL_exi(:)
c     ==================================================================

        WL_exi => SVR%WL_EXT(1:Ni2)
        N2 = N1+(N_FC-1)

        SELECT CASE( Flu_CDOM )
c       +++++++++++++++++++++++ 
        CASE('FHa')
           DEC(N1:N2,:) = SVR%V_FRF(1:N_FC,:,Jact) 

           Z_DEP(N1,1:NL-1) = (Fua_c(1:NL-1) + Fua_c(2:NL))/2.d0
     $          /ALW(1:NL-1)
           Z_DEP(N1+1,1:NL-1) = (Hua_c(1:NL-1) + Hua_c(2:NL))/2.d0
     $          /ALW(1:NL-1)
c        +++++++++++++++++++++++ 
         CASE('ACR')
c        ... In this case FRF should be calculated here (preliminary)
            DO J = 1,N_FC
               K1 = N1 + (J-1)
c              ... Emission function will be interpolated at actual wavelength 
               CALL LIN_INT(Nwl_QEm, WL_QEm, F_QEm(J,:), 
     $              1, WL_act, Emm, 'Z')
c              ... Excitation function will be interpolated at a given 
c                  excitation range
               CALL LIN_INT(Nwl_QEx, WL_QEx, F_QEx(J,:), 
     $              Ni2, WL_exi, DEC(K1,:), 'Z')
               DEC(K1,:) = DEC(K1,:)*WL_exi(1:Ni2)*Emm/WL_act

               Z_DEP(K1,1:NL-1) = (Flu_con(J,1:NL-1) + Flu_con(J,2:NL))
     $              /ALW(1:NL-1)/2.d0
            ENDDO
c        +++++++++++++++++++++++ 
         END SELECT

cc            TEST to exclude contribution of CDOM fluorescence
c         DEC(N1:N2,:) = 0.d0

c        ... Quadrature coefficients 
c                  Proverit' index Ka v kombinazii s VRS i Flu_r !!!        
         DEC(N1:N2,1)  = DEC(N1:N2,1) *(WL_exi(2) - WL_exi(1))
         DO I = 2,Ka-1
            DEC(N1:N2,I) = DEC(N1:N2,I)*(WL_exi(I+1) - WL_exi(I-1))
         ENDDO
         DEC(N1:N2,Ka) = DEC(N1:N2,Ka)*(WL_exi(Ka) - WL_exi(Ka-1))
         DEC(N1:N2,:) = DEC(N1:N2,:)*0.5d0 

      END SUBROUTINE SET_CC_Fb
c **********************************************************************

      SUBROUTINE SET_CC_Fr(N1)
      USE WATER_ICE, ONLY: IWS
c     -------------------------------------------------------------------
      INTEGER, INTENT(IN):: N1
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'SET_CC_Fr'
      INTEGER  I
c        ... Contains specific chlorophyll absorption coefficient only
c            i.e., without concentration of chlorophyll
        CALL ST_TRACK (R_NAME, 1, 'I')

        WHERE( L_r(:) ) 
           DEC(N1,:) = Qe_Chl/DSQRT(2.d0*Pie)/Sig_Chl
     $          *DEXP(-(WL_ACT - WLG_Chl)**2/2.d0/Sig_Chl**2 )
     $          *SVR%WL_EXT(1:Ni2)/WL_ACT*IWS(2)%ABS_COEFF_UW(2,1:Ni2)
        ELSEWHERE
           DEC(N1,:) = 0.d0
        END WHERE
c        ... Quadrature coefficients 
c                  Proverit' index Ka v kombinazii s VRS i Flu_b !!!
        DEC(N1,1)  = DEC(N1,1) *(SVR%WL_EXT(2)  - SVR%WL_EXT(1))
        DEC(N1,Ka) = DEC(N1,Ka)*(SVR%WL_EXT(Ka) - SVR%WL_EXT(Ka-1))
        DO I = 2,Ka-1
           DEC(N1,I) = DEC(N1,I)*(SVR%WL_EXT(I+1) - SVR%WL_EXT(I-1))
        ENDDO
        DEC(N1,:) = DEC(N1,:)*0.5d0 

c        ... Term depending on the chlorophyll concentration
        Z_DEP(N1,1:NL-1) = 0.06d0/2.d0/ALW(1:NL-1)
     $       *(CHL_UW(1:NL-1)**0.65d0 + CHL_UW(2:NL)**0.65d0)

        IF( LOG_LEVEL(1) <= N_XSO ) THEN
           DO I = 1, N1
              WRITE(LUNIT,*) '       DEC :', I, SUM(DEC(I,:))
              WRITE(LUNIT,*) '     Z_DEP :', I, SUM(Z_DEP(I,:))
           ENDDO
        END IF

        CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE SET_CC_Fr

c **********************************************************************

      END 

