C***********************************************************************
C
C       SCIATRAN - Version 4.0 - Last modification: 20 Apr 2018
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE PHASE_FUNCTIONS(Mh, NN, NSZA, NLSA, X_MOMS, Leg, Ph_x, 
     $     FLAG)
c     ... Called by "solv_dom.f"
c                 "rtm_init_first.f"

      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE GT_POLYN,    ONLY: XM, Plm_G, Plm_L, Plm_S, Plm_Sm, 
     $                       SGN, XM_W, Plm_GW, Plm_LW, Plm_SW
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs, N_sol_ls
      USE ST_VARS,     ONLY: SSC_RF
      USE TYPES,       ONLY: PH_MET
      USE TYPE_VARS,   ONLY: KND
c     ==================================================================

      CHARACTER (LEN=15), PARAMETER :: R_NAME = 'PHASE_FUNCTIONS'
      INTEGER,      INTENT(IN) :: Mh, NN, NSZA, NLSA, Leg
      REAL (KND),   INTENT(IN) :: X_MOMS(Leg)
      CHARACTER (LEN=1), INTENT(IN) :: FLAG
      TYPE (PH_MET) :: Ph_x
c     -----------------------
      INTEGER     K, K1, K2, L1
      REAL (KND)  Phh(Leg,NN)
      REAL (KND), POINTER :: Po_g(:,:), Po_l(:,:), Po_s(:,:), XM_X(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      L1 = Leg - Mh + 1
      IF( L1 < 1 ) THEN
         Ph_x%Egg = 0.d0  ;  Ph_x%Egs = 0.d0
         Ph_x%Egl = 0.d0  ;  Ph_x%EsL = 0.d0
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      SELECT CASE (FLAG)
      CASE('A','a') 
         Po_g => Plm_G(Mh,Mh:Leg,:) ! Plm     Gauss
         Po_s => Plm_S(Mh,Mh:Leg,:)
         Po_l => Plm_L(Mh,Mh:Leg,:)
         XM_X => XM(:)
      CASE('O','o')
         Po_g => Plm_GW(Mh,Mh:Leg,:) ! Plm     Gauss
         Po_s => Plm_SW(Mh,Mh:Leg,:)
         Po_l => Plm_LW(Mh,Mh:Leg,:)
         XM_X => XM_W(:)
      END SELECT
      
      FORALL( K = 1:NN ) Phh(1:L1,K) = X_MOMS(Mh:Leg)*Po_g(1:L1,K)
      DO K1 = 1,NN 
         DO K2 = 1,NN
            Ph_x%Egg(K1,K2,1) = SUM( Po_g(1:L1,K2)*Phh(1:L1,K1) )
            Ph_x%Egg(K1,K2,2) = 
     $           SUM( Po_g(1:L1,K2)*Phh(1:L1,K1)*SGN(1:L1) )
         ENDDO  ! over Gauss angles (K2)
         DO K2 = 1,NLSA
            Ph_x%Egl(K1,K2,1) = SUM( Po_l(1:L1,K2)*Phh(1:L1,K1) )
            Ph_x%Egl(K1,K2,2) = 
     $           SUM( Po_l(1:L1,K2)*Phh(1:L1,K1)*SGN(1:L1) )
         ENDDO  ! over line-of-sight angles (K2)
         DO K2 = 1,NSZA
            Ph_x%Egs(K1,K2,1) = 
     $           SUM( Po_s(1:L1,K2)*Phh(1:L1,K1)*SGN(1:L1) )
            Ph_x%Egs(K1,K2,2) = SUM( Po_s(1:L1,K2)*Phh(1:L1,K1) )
         ENDDO  ! over solar angles (K2)
c        ... First dimension corresponds to the Gaussian angles
         Ph_x%Egg(K1,:,:) = Ph_x%Egg(K1,:,:)/XM_X(K1)
         Ph_x%Egl(K1,:,:) = Ph_x%Egl(K1,:,:)/XM_X(K1)
         Ph_x%Egs(K1,:,:) = Ph_x%Egs(K1,:,:)/XM_X(K1)
      ENDDO  ! over Gauss angles (K1)

      IF( DO_SSRCA.OR.SSC_RF ) THEN
         Ph_x%EsL = 0.d0
      ELSE
         DO K1 = 1,NLSA
            DO K2 = 1,NSZA
               Ph_x%EsL(K1,K2,1) = 
     $              SUM(X_MOMS(Mh:Leg)*Po_s(1:L1,K2)*Po_l(1:L1,K1))
            ENDDO
         ENDDO
         SELECT CASE (FLAG)
         CASE('A','a') 
            IF( N_sol_ls == 2 ) THEN
               DO K1 = 1,NLSA
                  DO K2 = 1,NSZA
                     Ph_x%EsL(K1,K2,2) = SUM(X_MOMS(Mh:Leg) 
     $                    *Plm_Sm(Mh,Mh:Leg,K2)*Po_l(1:L1,K1))
                  ENDDO
               ENDDO
            ENDIF
         END SELECT
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
