C********************************************************************
C
C       SCIATRAN - Version 4.3 - Last modification: 04 Feb 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                        University of Bremen, Germany
C
C********************************************************************
      SUBROUTINE PHASE_MATR( Mh, NN, NSZA, NLSA, Nlg_X, X_MOMS, Ph_1d,
     $     FLAG)
c     Called by  "rtm_init.f",  "solv_dom.f"

      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE GT_POLYN,    ONLY: NLEGEN, XM, XM_W, SGN,
     $                       Plm_G, Plm_L, Plm_S, Dp2_G, Dm2_G, 
     $                       Dp2_L, Dm2_L, Dp2_S, Dm2_S,
     $                       Plm_GW, Plm_LW, Plm_SW, Dp2_GW, Dm2_GW, 
     $                       Dp2_LW, Dm2_LW, Dp2_SW, Dm2_SW
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs
      USE ST_VARS,     ONLY: IDM, LDM, SSC_RF
      USE TYPES,       ONLY: KND, PH_MET
      USE WATER,       ONLY: NLEG_W
c     ------------------------------------------------------------------
c         ... Integer arrays IDM and LDM are given in "gt_fileread.f"
c             as follows:
c         IDM(1) = 1 ; IDM(2) = 5 ; IDM(3) = 5
c         IDM(4) = 2 ; IDM(5) = 2 ; IDM(6) = 2
c
c         LDM(1) = 3 ; LDM(2) = 3 ; LDM(3) = 3
c         LDM(4) = 6 ; LDM(5) = 6 ; LDM(6) = 4         
c         ... Elements of these arrays correspond to the following
c             components of the Greek symbol matrix:
c             1,2,3,4 - alpha_1,alpha_2,alpha_3,alpha_4,
c                 5,6 - betta_1, betta_2
c             see Eq. (GS-matr) in STB
c     ------------------------------------------------------------------
      INTEGER,    INTENT(IN) ::  Mh, NN, NSZA, NLSA, Nlg_X
      REAL (KND), INTENT(IN) ::  X_MOMS(Nlg_X,Nm_vs)
      CHARACTER (LEN=1), INTENT(IN) :: FLAG
      TYPE (PH_MET), TARGET :: Ph_1d
c    ------------------------------------------------------
c     ... Local variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'PHASE_MATR'
      REAL (KND), TARGET :: XgL (Nvs*NN,Nvs*NLSA,2), 
     $     Xgg (Nvs*NN,Nvs*NN,2)
      INTEGER  Lma, I, I1, Ig, I2, J, K1, K2, K11, Ks1, Ks2, Kp1, Kp2, 
     $     L, L1
      REAL (KND)  S1 !, P_R(Leg,3,3), H1(Leg), H2(Leg)
      REAL (KND), ALLOCATABLE :: Add(:,:,:), P_R(:,:,:), Pp(:,:,:),
     $     H1(:), H2(:)
      REAL (KND), POINTER :: Po_g(:,:), Por(:,:), Pot(:,:),   
     $     R_GG(:,:,:), R_GS(:,:,:), R_GL(:,:,:), R_LS(:,:,:),
     $     XM_X(:), Plm_GX(:,:,:), Plm_SX(:,:,:), Plm_LX(:,:,:),
     $     Dp2_GX(:,:,:), Dm2_GX(:,:,:),
     $     Dp2_SX(:,:,:), Dm2_SX(:,:,:),
     $     Dp2_LX(:,:,:), Dm2_LX(:,:,:)
c     ==================================================================

c      CALL ST_TRACK (R_NAME, 1, 'I')

      Lma = Nlg_X  ;  L1 = Lma - Mh + 1

      SELECT CASE (FLAG)
      CASE('A','a')
         ALLOCATE( P_R(NLEGEN,3,3), H1(NLEGEN), H2(NLEGEN),
     $        Pp(NLEGEN,3,3) )
         Plm_GX => Plm_G ; Plm_SX => Plm_S ; Plm_LX => Plm_L
         Dp2_GX => Dp2_G ; Dm2_GX => Dm2_G
         Dp2_SX => Dp2_S ; Dm2_SX => Dm2_S
         Dp2_LX => Dp2_L ; Dm2_LX => Dm2_L
         XM_X => XM(:)
      CASE('O','o')
         ALLOCATE( P_R(NLEG_W,3,3), H1(NLEG_W), H2(NLEG_W),
     $        Pp(NLEG_W,3,3) )
         Plm_GX => Plm_GW ; Plm_SX => Plm_SW ; Plm_LX => Plm_LW
         Dp2_GX => Dp2_GW ; Dm2_GX => Dm2_GW 
         Dp2_SX => Dp2_SW ; Dm2_SX => Dm2_SW
         Dp2_LX => Dp2_LW ; Dm2_LX => Dm2_LW
         XM_X => XM_W(:)
      END SELECT

      Po_g  => Plm_GX(Mh,Mh:Lma,:) 
      Por   => Dp2_GX(Mh,Mh:Lma,:) ! Rlm(mu) Gauss
      Pot   => Dm2_GX(Mh,Mh:Lma,:) ! Tlm(mu) --,--

      R_GG => Ph_1d%Egg  

c     ... In this version second dimension of array Ph_1d%Egs is 
c         Nvs*NSZA instead of NSZA in previous versions. The structure
c         of this array is the same as Ph_1d%EgL(:,:,1:2). In contrast
c         to previous versions the phase matrix for direct solar light
c         is not premultiplied by the vector F = |1,0,0,0|.
      R_GS => Ph_1d%Egs

c     ... Phase matrix Gauss => line-of-sight. 
c         G_m(mu_ls,-mu_ga) according to Eq.(Gpm) saved in Ph_1d%EgL(:,:,1)
c         G_m(mu_ls, mu_ga) according to Eq.(Gpp) saved in Ph_1d%EgL(:,:,2)
c
c         First dimension of Ph_1d%EgL(:,:,1:2) is the Gaussian angles
c         second - viewing angle
      R_GL => Ph_1d%EgL  ; R_GL = 0.d0

c     ... Phase matrix solar => line-of-sight. 
      R_LS => Ph_1d%EsL ;  R_LS = 0.d0 

      R_GS = 0.d0  ;  R_GG = 0.d0
      XgL = 0.d0 ; Xgg = 0.d0
      
      CALL SCAT_MATR('Gau', NN, 'Sol', NSZA, R_GS)

      DO K1 = 1,NN
         Ks1 = 4*(K1-1) ; Kp1 = Ks1 + 1
c        ... Phase matrix Guass angles => Gauss angles
         DO K2 = 1,NN
            Ks2 = 4*(K2-1) ; Kp2 = Ks2 + 1
c           ... Elements of the upper left 3x3 rectangle  
c               sub-matrix of U^m_l(mu_i,mu_j) given by Eq.(U-lm) 
c               in STB without multiplication by the Greek 
c               symbols (first angle - Gauss, second - Gauss)
            P_R(1:L1,1,1) = Po_g(1:L1,K1)*Po_g(1:L1,K2)
            P_R(1:L1,1,2) = Po_g(1:L1,K1)* Por(1:L1,K2)
            P_R(1:L1,1,3) =-Po_g(1:L1,K1)* Pot(1:L1,K2)
            P_R(1:L1,2,2) = Por (1:L1,K1)* Por(1:L1,K2)
            P_R(1:L1,2,3) =-Por (1:L1,K1)* Pot(1:L1,K2)
            P_R(1:L1,3,3) = Pot (1:L1,K1)* Pot(1:L1,K2)

            L = 1
            DO I = 1,3  ;  DO J = I,3
               R_GG(Ks1+I,Ks2+J,1) = 
     $              SUM( X_MOMS(Mh:Lma:2,IDM(L))*P_R(1:L1:2,I,J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,IDM(L))*P_R(I2,I,J)
                  I2 = I2 + 2
               ENDDO
               R_GG(Ks1+I,Ks2+J,2 ) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |* * * 0|  
c                   |- * * 0|  
c                   |- - * 0|  
c                   |0 0 0 0|  
c           ... R_GG(:,:1) corresponds to the matrix U^{ev}_m 
c                          given by Eq.(Um-ev) in STB;
c               R_GG(:,:2) corresponds to the matrix U^{od}_m 
c                          given by Eq.(Um-od) in STB
            
            P_R(1:L1,1,2:3) = -P_R(1:L1,1,2:3)
            L = 1
            DO I = 1,3  ;  DO J = 1,I
               Xgg(Kp1+I,Kp2+J,1) = SUM( 
     $              X_MOMS(Mh:Lma:2,LDM(L))*P_R(1:L1:2,4-I,4-J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,LDM(L))*P_R(I2,4-I,4-J)
                  I2 = I2 + 2
               ENDDO
               Xgg(Kp1+I,Kp2+J,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |0 0 0 0|
c                   |0 * - -|  
c                   |0 * * -|  
c                   |0 * * *|  
c           ... Xgg(:,:1) corresponds to the matrix L^{ev}_m 
c                         given by Eq.(Lm-ev) in STB;
c               Xgg(:,:2) corresponds to the matrix L^{od}_m 
c                         given by Eq.(Lm-od) in STB
         ENDDO                  ! over Gauss angles (K2)
      ENDDO                     ! over Gauss angles (K1)

c     ... Initialize sub-diagonal elements of matrixes U^{ev}_m 
c         [R_GG(:,:1)] and U^{od}_m [R_GG(:,:2)], respectively, for
c         <Gauss -> Gauss>
      DO K1 = 1,NN
         Ks1 = 4*(K1-1)
         DO K2 = 1,NN
            Ks2 = 4*(K2-1)
            FORALL( I=1:3, J=1:3, I > J ) 
               R_GG(Ks2+I,Ks1+J,: ) = R_GG(Ks1+J,Ks2+I,:)
            END FORALL
         ENDDO
      ENDDO

c     ... Initialize up-diagonal elements of matrixes L^{ev}_m
c         [Xgg(:,:1)] and L^{od}_m [Xgg(:,:2)], respectively, for
c         <Gauss -> Gauss>
      DO K1 = 1,NN
         Kp1 = 4*(K1-1)+1
         DO K2 = 1,NN
            Kp2 = 4*(K2-1)+1
            FORALL( I=1:3, J=1:3, I < J ) 
              Xgg(Kp2+I,Kp1+J,: ) = -Xgg(Kp1+J,Kp2+I,: )
            END FORALL
            Xgg(Kp2+1,Kp1+2,: ) = -Xgg(Kp2+1,Kp1+2,: )
         ENDDO
      ENDDO

      DO K1 = 1,NN
         R_GG(1:4*NN,4*K1-3:4*K1,:) =
     $        R_GG(1:4*NN,4*K1-3:4*K1,:)/XM_X(K1)
         Xgg(1:4*NN,4*K1-3:4*K1,:)  =
     $        Xgg(1:4*NN,4*K1-3:4*K1,:)/XM_X(K1)
      ENDDO         

c     ... R_GG(:,:,1) contains as result U^{ev}_m + L^{od}_m as given by
c         Eq.(Gpp+Gpm) in STB. This is [G(Mi,Mj) + ^G(Mi,-Mj)]/2
      R_GG(:,:,1) = R_GG(:,:,1) + Xgg(:,:,2)

c     ... R_GG(:,:,2) contains as result U^{od}_m + L^{ev}_m as given by
c         Eq.(Gpp-Gpm) in STB. This is [G(Mi,Mj) - ^G(Mi,-Mj)]/2
      R_GG(:,:,2) = R_GG(:,:,2) + Xgg(:,:,1)

c     ... Phase matrix Guass <---> Line-of-sight
c     ------------------------------------------
      CALL SCAT_MATR('Gau', NN, 'LoS', NLSA, R_GL)

      IF( DO_SSRCA.OR.SSC_RF ) THEN
         NULLIFY( Po_g, Por, Pot, R_GG, R_GS, R_GL, R_LS )
c         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

c     ... Phase matrix <line-of-sight <== Solar> 
      CALL SCAT_MATR('LoS', NLSA, 'Sol', NSZA, R_LS)

      NULLIFY( Po_g, Por, Pot, R_GG, R_GS, R_GL, R_LS )

c      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
c     This routine is designed to calculate phase matrices Ph(4x4,1) 
c     and Ph(4x4,2) which should be used to transform Fourier component 
c     of Stokes vector given at the angle a1 (Ang1) to the Fourier  
c     component of Stokes vector at the angle a2 (Ang2). 
c                       Ang1 ---> Ang2
c       ------------------------
c       | Case 1: Gauss --> Los|
c       ------------------------
c                    S'(-a2) ~ Ph(4x4,1)*S'( a1)
c                    S ( a2) = Ph(4x4,2)*S ( a1)
c        Used by 
c                "post_processing.f": to calculate source function from
c                -------------------- Gaussian nodes into viewing direction
c                    ( -/+Gauss --> LoS )
c
c                "part_sol_arte.f": to calculate pseudo-forward single 
c                ------------------ scattering source function for 
c                    particular solution of pseudo-forward RTE 
c                    ( Los --> +/-Gauss )
c                 
c                "st_pp_rrs.f", "st_pp_vrs.f", "st_pp_wf.f", 
c                -------------------------------------------
c                "st_ps_wf.f", "st_sph_rrs.f": for scalar case at this step
c                ----------------------------
c
c **********************************************************************
c                       Ang1 ---> Ang2
c       -----------------------------------------------------------------
c       | Case 2:   Gauss --> +/- Sol [arrays Egs(Nvs*NN,Nvs*NSZA,1:2)] |
c       -----------------------------------------------------------------

c                    S ( a2) = Ph(4x4,1)*S ( a1)
c                    S'(-a2) ~ Ph(4x4,2)*S'( a1)
c        Used by 
c                "homog_partic.f": to calculate forward single scattering 
c                ----------------  source function for particular solution 
c                    of forward RTE ( Sol --> +/-Gauss )  
c
c                "st_up_down_sfi.f": to calculate forward single scattering
c                ------------------  source function for SFI for Gauss field 
c                    if "Adaptive-grid" option is active ( Sol --> +/-Gauss )
c
c                "gau_adj_up_sol.f": to calculate pseudo-forward multiple 
c                ------------------  scattering source function in the solar 
c                    direction ( +/-Gauss --> Sol )
c
c                "st_pp_wf.f", "st_ps_wf.f", "st_pp_owf.f": to calculate 
c                -----------------------------------------  scalar product 
c                    of the forward s-s source function and diffuse adjoint 
c                    intensity
c
c                "fls_asi.f": to calculate scalar product of the forward 
c                -----------  single scattering source function with the 
c                    diffuse adjoint intensity ( Sol --> +/-Gauss )
c ***********************************************************************
c                       Ang1 ---> Ang2
c              Line-of-sight      Solar
c
c      ----------------------------
c      |  Case 3:  Sol <-- +/-LoS |
c      ----------------------------
c                    S ( a2) = Ph(4x4,1)*S ( a1)
c                    S'(-a2) ~ Ph(4x4,2)*S'( a1)

c         "st_up_sfi.f", "st_down_sfi.f" : to calculate forward single 
c         -------------------------------  scattering source function
c                    from the solar direction into LoS direction 
c                    ( Sol --> LoS )

c         "gau_adj_up_sol.f" : to calculate pseudo-forward single 
c         -------------------  scattering source function from the 
c                    opposite to LoS direction into solar direction
c                    ( -Los --> Sol )

c         "st_pp_wf.f", "st_ps_wf.f", "st_pp_owf.f": to calculate 
c         -----------------------------------------  scalar product 
c                    of the forward s-s source function and direct adjoint
c                    intensity
c
c        Used by "fls_asi.f" : to calculate scalar product of the 
c                ------------  forward single scattering source function  
c                    with the direct adjoint intensity 
c                    ( Sol --> LoS )
c **********************************************************************
      SUBROUTINE SCAT_MATR(Ang1, N_ang1, Ang2, N_ang2, Uev)

      CHARACTER (LEN=3), INTENT(IN) ::  Ang1, Ang2
      INTEGER,           INTENT(IN) ::  N_ang1, N_ang2
      REAL (KND), POINTER :: Uev(:,:,:)
c      REAL (KND) Uev(Nvs*N_ang1,Nvs*N_ang2,2)
c     ... Local variables
      INTEGER  I, J, K1, K2, Ks1, Kp1, Ks2, Kp2, L
      REAL (KND)  S1, Lev(Nvs*N_ang1,Nvs*N_ang2,2), M4(4)
      REAL (KND), POINTER :: Po_a(:,:), Por_a(:,:), Pot_a(:,:),
     $     Po_i(:,:), Por_i(:,:), Pot_i(:,:)
c     ------------------------------------------------------------------

      Pp = 0.d0  ;  Lev = 0.d0
c     ... Select polynomial for angle mu_i
      SELECT CASE( Ang1 )
      CASE('Gau')
         Po_i  => Plm_GX(Mh,Mh:Lma,:) 
         Por_i => Dp2_GX(Mh,Mh:Lma,:) ! Rlm(mu) Gauss
         Pot_i => Dm2_GX(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      CASE('LoS')
         Po_i  => Plm_LX(Mh,Mh:Lma,:) ! 
         Por_i => Dp2_LX(Mh,Mh:Lma,:) ! Rlm(mu) Line-of-sight
         Pot_i => Dm2_LX(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      END SELECT

c     ... Select polynomial for angle mu_a
      SELECT CASE( Ang2 )
      CASE('LoS')
c        ... The polynomial are calculated in "gt_hi_wi.f" for 
c            the angles corresponding to the viewing directions
         Po_a  => Plm_LX(Mh,Mh:Lma,:) ! 
         Por_a => Dp2_LX(Mh,Mh:Lma,:) ! Rlm(mu) Line-of-sight
         Pot_a => Dm2_LX(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      CASE('Sol')
         Po_a  => Plm_SX(Mh,Mh:Lma,:)
         Por_a => Dp2_SX(Mh,Mh:Lma,:) ! Rlm(mu) Solar
         Pot_a => Dm2_SX(Mh,Mh:Lma,:) ! Tlm(mu) --,--
      END SELECT

      DO K1 = 1,N_ang1
         Ks1 = 4*(K1-1) ; Kp1 = Ks1 + 1
         DO K2 = 1,N_ang2
            Ks2 = 4*(K2-1) ; Kp2 = Ks2 + 1
c           ... Calculate elements of the upper-left rectangle 
c               matrix 3x3 U^m_l given by Eq.(U-lm) in STB
            Pp(1:L1,1,1) = Po_i (1:L1,K1)*Po_a (1:L1,K2)
            Pp(1:L1,1,2) = Por_i(1:L1,K1)*Po_a (1:L1,K2)
            Pp(1:L1,1,3) =-Pot_i(1:L1,K1)*Po_a (1:L1,K2)

            Pp(1:L1,2,2) = Por_i(1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,2,3) =-Pot_i(1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,3,3) = Pot_i(1:L1,K1)*Pot_a(1:L1,K2)
            L = 1
            DO I = 1,3  ;  DO J = I,3
               Uev(Ks1+I,Ks2+J,1 ) =
     $              SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,I,J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,I,J)
                  I2 = I2 + 2
               ENDDO
               Uev(Ks1+I,Ks2+J,2 ) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |* * * 0|  
c                   |- * * 0|  
c                   |- - * 0|  
c                   |0 0 0 0|  
            Pp(1:L1,2,1) =  Po_i (1:L1,K1)*Por_a(1:L1,K2)
            Pp(1:L1,3,1) = -Po_i (1:L1,K1)*Pot_a(1:L1,K2)
            Pp(1:L1,3,2) = -Por_i(1:L1,K1)*Pot_a(1:L1,K2)
            L = 2
            DO I = 1,2  ; DO J = I+1,3
               Uev(Ks1+J,Ks2+I,1) =
     $              SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,J,I) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,J,I)
                  I2 = I2 + 2
               ENDDO
               Uev(Ks1+J,Ks2+I,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by + 
c               is performed after above do-loop, i.e.,                
c                   |- - - 0|  
c                   |+ - - 0|  
c                   |+ + - 0|  
c                   |0 0 0 0|  

c           ... Uev(:,:1) contains at this step upper-left rectangle matrix 
c                 (3x3) of the matrix U^{ev}_m given by Eq.(Um-ev) in STB;
c               Uev(:,:2) contains at this step upper-left rectangle matrix 
c                 (3x3) of the matrix U^{od}_m given by Eq.(Um-od) in STB
            DO I = 1,3
               DO J = 1,3
                  Uev(Ks1+I,Ks2+J,1) = 
     $                 Uev(Ks1+I,Ks2+J,1) + Uev(Ks1+I,Ks2+J,2)
                  Uev(Ks1+I,Ks2+J,2) = 
     $                 Uev(Ks1+I,Ks2+J,1) - 2.d0*Uev(Ks1+I,Ks2+J,2)
               ENDDO
            ENDDO
c           ... After summation we have:
c               Uev(:,:1) = U^{ev}_m + U^{od}_m
c               Uev(:,:2) = U^{ev}_m - U^{od}_m,
c               and matrices contain product of functions                
c               (Greek symbols not shown):
c                   | Pi*Pa  Ri*Pa -Ti*Pa  0|  
c                   | Pi*Ra  Ri*Ra -Ti*Ra  0|  
c                   |-Pi*Ta -Ri*Ta  Ti*Ta  0|  
c                   |   0      0      0    0|  
c               Attention! This matrix is transposed w.r.t. to  
c               the matrix U^m_l given by Eq.(U-lm) in STB.
c               It is done to obtain the matrices describing
c               scattering process:
c                   Ang1 ---> Ang2 and Ang1 ---> -Ang2

c           ... Calculate elements of the lower-right rectangle 
c               matrix 3x3 L^m_l given by Eq.(L-lm) in STB
            Pp(1:L1,1,2:3) = -Pp(1:L1,1,2:3)
            L = 1
            DO I = 1,3  ;  DO J = 1,I
               Lev(Kp1+I,Kp2+J,1) = SUM(
     $              X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J)
                  I2 = I2 + 2
               ENDDO
               Lev(Kp1+I,Kp2+J,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO            
c           ... The calculation of Lev matrix elements matched by * 
c               is performed after above do-loop, i.e.,
c                   |0 0 0 0|  
c                   |0 * - -|  
c                   |0 * * -|  
c                   |0 * * *|  
            L = 3
            DO I = 1,2  ; DO J = I+1,3
               Lev(Kp1+I,Kp2+J,1) = SUM(
     $              X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J) )
               S1 = 0.d0 ; I2 = 2
               DO I1 = Mh+1,Lma,2
                  S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J)
                  I2 = I2 + 2
               ENDDO
               Lev(Kp1+I,Kp2+J,2) = S1
               L = L + 1
            ENDDO  ;  ENDDO               
c           ... The calculation of Lev matrix elements matched by +  
c               is performed after above do-loop, i.e., 
c                   |0 0 0 0|  
c                   |0 - + +|  
c                   |0 - - +|  
c                   |0 - - -|  

c           ... Lev(:,:1) contains at this step lower-right rectangle matrix 
c                 (3x3) of the matrix L^{ev}_m given by Eq.(Lm-ev) in STB;
c               Lev(:,:2) contains at this step lower-right rectangle matrix 
c                 (3x3) of the matrix L^{od}_m given by Eq.(Lm-od) in STB
            DO I = 1,3
               DO J = 1,3
                  Lev(Kp1+I,Kp2+J,1) = 
     $                 Lev(Kp1+I,Kp2+J,1) + Lev(Kp1+I,Kp2+J,2)
                  Lev(Kp1+I,Kp2+J,2) = 
     $                 Lev(Kp1+I,Kp2+J,1) - 2.d0*Lev(Kp1+I,Kp2+J,2)
               ENDDO
            ENDDO
c           ... After summation:
c               Lev(:,:1) = L^{ev}_m + L^{od}_m
c               Lev(:,:2) = L^{ev}_m - L^{od}_m
c               Matrices contain product of functions
c               (Greek symbols not shown):
c                   | 0    0      0      0  |  
c                   | 0  Ti*Ta -Ri*Ta -Pi*Ta|  
c                   | 0 -Ti*Ra  Ri*Ra  Pi*Ra|  
c                   | 0  Ti*Pa -Ri*Pa  Pi*Pa|  
c               Attention! This matrix is transposed and multiplied by 
c               the matrix D4 (diag{1,1,1,-1}) from left and right w.r.t.   
c               to the matrix L^m_l given by Eq.(L-lm) in STB.
c               It is done to obtain the matrices describing
c               scattering process: 
c                   Ang1 ---> Ang2 and Ang1 ---> -Ang2
               
         ENDDO                  ! over any angles (K2)
      ENDDO                     ! over Gaussian angles (K1)
c     ... After following summation:
c         Lev(:,:,1) =  [U^{ev}_m + U^{od}_m] +  [L^{ev}_m + L^{od}_m]
      Lev(:,:,1) = Uev(:,:,1) + Lev(:,:,1)

c     --------------------------------------------------------------------
c     -------------------  Results ---------------------------------------
c
c     ... Uev(:,:,1) = [U^{ev}_m - U^{od}_m] - [L^{ev}_m - L^{od}_m]
      Uev(:,:,1) = Uev(:,:,2) - Lev(:,:,2)
c         transform the Stokes vector: Ang1 ---> -Ang2
c            This matrix is not multiplied by D=diag{1,1,-1,-1} 
c            from left (see Eq.G-gua+mu0 in STB)! 

c     --------------------------------------------------------------------
c     ... Uev(:,:,2) =  [U^{ev}_m + U^{od}_m] +  [L^{ev}_m + L^{od}_m]
      Uev(:,:,2) = Lev(:,:,1)
c         transform the Stokes vector: Ang1 ---> Ang2
c     --------------------------------------------------------------------

      SELECT CASE( Ang2 )
      CASE('Sol')
c        ... To obtain the same meaning of last arguments 1 and 2
c            as in previous versions of the program
         Uev(:,:,2) = Uev(:,:,1)
c            Matrix R_GS(:,:,2), Egs(:,:,2), P02(:,:,:) describes 
c            scattering process  Gauss --> -Sol_ang.
c            This matrix is not multiplied by D=diag{1,1,-1,-1} 
c            from left (see Eq.(Qm-sol) in STB)! 

         Uev(:,:,1) = Lev(:,:,1)
c            Matrix R_GS(:,:,1), Egs(:,:,1), P01(:,:,:) describes 
c            scattering process  Gauss --> Sol_ang and 
c                                LoS   --> Sol_ang

      END SELECT

      NULLIFY( Po_a, Por_a, Pot_a, Po_i, Por_i, Pot_i )

      RETURN
      END SUBROUTINE SCAT_MATR
c **********************************************************************
      END
