C********************************************************************C
C
C       SCIATRAN - Version 3.6 - Last modification: 11 Jun 2016
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE PHASE_MATR_SPH( Mh, NN, NLO, Nlg_X, X_MOMS, 
     $     N_JL, Plm_los, Dp2_los, Dm2_los, Egs_sph ) 
c     $     Ph_1d )
c     Called by "dom_spherical.f"

      USE GT_POLYN,    ONLY: Leg => NLEGEN, XM, SGN, Plm_G, Dp2_G, Dm2_G 
      USE INTERN_DIMS, ONLY: Nvs, Nm_vs
      USE ST_VARS,     ONLY: IDM, LDM, N_Eff
      USE TYPES,       ONLY: KND
c     ------------------------------------------------------------------
c         ... Integer arrays IDM and LDM are given in "gt_fileread.f"
c             as follows:
c         IDM(1) = 1 ; IDM(2) = 5 ; IDM(3) = 5
c         IDM(4) = 2 ; IDM(5) = 2 ; IDM(6) = 2
c
c         LDM(1) = 3 ; LDM(2) = 3 ; LDM(3) = 3
c         LDM(4) = 6 ; LDM(5) = 6 ; LDM(6) = 4         
c         ... Elements of these arrays correspond to the following
c             components of the Greek symbol matrix:
c             1,2,3,4 - alpha_1,alpha_2,alpha_3,alpha_4,
c                 5,6 - betta_1, betta_2
c             see Eq. (GS-matr) in STB
c     ------------------------------------------------------------------
      INTEGER,    INTENT(IN) ::  Mh, NN, NLO, Nlg_X, N_JL(NLO)
      REAL (KND), INTENT(IN) ::  X_MOMS(Nlg_X,Nm_vs), 
     $     Plm_los(Nlg_X,NLO,N_Eff), Dp2_los(Nlg_X,NLO,N_Eff), 
     $     Dm2_los(Nlg_X,NLO,N_Eff)
      TARGET :: Plm_los, Dp2_los, Dm2_los
      REAL (KND), TARGET ::  Egs_sph(Nvs*NN,Nvs*NLO,N_Eff,2)
c    --------------------------------------------------------------------
c     ... Local variables
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'PHASE_MATR_SPH'
      CHARACTER (LEN=40) NAME
      INTEGER  Lma, I, I1, I2, J, JL, K1, K2, Ks1, Ks2, Kp1, Kp2, 
     $     L, L1, Ka1, Ka2
     $     , Ks, K11, Ka
      REAL (KND)  S1, Pp(Leg,3,3,N_Eff), Lev(Nvs*NN,Nvs*NLO,N_Eff,2)
      REAL (KND), POINTER :: Uev(:,:,:,:)
      REAL (KND), POINTER :: Po_a(:,:,:), Por_a(:,:,:), Pot_a(:,:,:),
     $     Po_i(:,:), Por_i(:,:), Pot_i(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Uev => Egs_sph 
      Lma = Nlg_X  ;  L1 = Lma - Mh + 1

c **********************************************************************
c     This routine is designed to calculate phase matrices Ph(4x4,1) 
c     and Ph(4x4,2) which should be used to transform Fourier component 
c     of Stokes vector given at the angle a1 (Ang1) to the Fourier  
c     component of Stokes vector at the angle a2 (Ang2). 
c                       Ang1 ---> Ang2
c **********************************************************************
c                       Ang1 ---> Ang2
c       -----------------------------------------------------------------
c       | Case 2:   Gauss --> +/- Sol [arrays Egs(Nvs*NN,Nvs*NLO,1:2)] |
c       -----------------------------------------------------------------
c                    S ( a2) = Ph(4x4,1)*S ( a1)
c                    S'(-a2) ~ Ph(4x4,2)*S'( a1)
c        Used by 
c ***********************************************************************
c      WRITE(*,*) '   Zeroing  '
      Pp = 0.d0  ;  Uev = 0.d0
      Lev = 0.d0 

c     ... Select polynomials for Gauss angles mu_i
      Po_i  => Plm_G(Mh,Mh:Lma,:) 
      Por_i => Dp2_G(Mh,Mh:Lma,:) ! Rlm(mu) Gauss
      Pot_i => Dm2_G(Mh,Mh:Lma,:) ! Tlm(mu) --,--
c      WRITE(*,*) '   Pointers 1  '

c     ... Select polynomials for local solar angles
      Po_a  => Plm_los(Mh:Lma,:,:)
      Por_a => Dp2_los(Mh:Lma,:,:) ! Rlm(mu) Solar
      Pot_a => Dm2_los(Mh:Lma,:,:) ! Tlm(mu) --,--
c      WRITE(*,*) '   Pointers 2  '

      DO K1 = 1,NN
         Ks1 = 4*(K1-1) ; Kp1 = Ks1 + 1
         DO K2 = 1,NLO
            Ks2 = 4*(K2-1) ; Kp2 = Ks2 + 1
            DO JL = 1,N_JL(K2)
c               Ks2 = 4*SWF%Ief_Sum(K2)  ; Kp2 = Ks2 + 1
c           ... Calculate elements of the upper-left rectangle 
c               matrix 3x3 U^m_l given by Eq.(U-lm) in STB
               Pp(1:L1,1,1,JL) = Po_i (1:L1,K1)*Po_a (1:L1,K2,JL)
               Pp(1:L1,1,2,JL) = Por_i(1:L1,K1)*Po_a (1:L1,K2,JL)
               Pp(1:L1,1,3,JL) =-Pot_i(1:L1,K1)*Po_a (1:L1,K2,JL)

               Pp(1:L1,2,2,JL) = Por_i(1:L1,K1)*Por_a(1:L1,K2,JL)
               Pp(1:L1,2,3,JL) =-Pot_i(1:L1,K1)*Por_a(1:L1,K2,JL)
               Pp(1:L1,3,3,JL) = Pot_i(1:L1,K1)*Pot_a(1:L1,K2,JL)
               L = 1
               DO I = 1,3  ;  DO J = I,3
                  Uev(Ks1+I,Ks2+J,JL,1 ) =
     $                 SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,I,J,JL) )
                  S1 = 0.d0 ; I2 = 2
                  DO I1 = Mh+1,Lma,2
                     S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,I,J,JL)
                     I2 = I2 + 2
                  ENDDO
                  Uev(Ks1+I,Ks2+J,JL,2 ) = S1
                  L = L + 1
               ENDDO  ;  ENDDO

               Pp(1:L1,2,1,JL) =  Po_i (1:L1,K1)*Por_a(1:L1,K2,JL)
               Pp(1:L1,3,1,JL) = -Po_i (1:L1,K1)*Pot_a(1:L1,K2,JL)
               Pp(1:L1,3,2,JL) = -Por_i(1:L1,K1)*Pot_a(1:L1,K2,JL)
               L = 2
               DO I = 1,2  ; DO J = I+1,3
                  Uev(Ks1+J,Ks2+I,JL,1) =
     $                 SUM( X_MOMS(Mh:Lma:2,IDM(L))*Pp(1:L1:2,J,I,JL) )
                  S1 = 0.d0 ; I2 = 2
                  DO I1 = Mh+1,Lma,2
                     S1 = S1 + X_MOMS(I1,IDM(L))*Pp(I2,J,I,JL)
                     I2 = I2 + 2
                  ENDDO
                  Uev(Ks1+J,Ks2+I,JL,2) = S1
                  L = L + 1
               ENDDO  ;  ENDDO
c           ... The calculation of Uev matrix elements matched by + 
               DO I = 1,3
                  DO J = 1,3
                     Uev(Ks1+I,Ks2+J,JL,1) = Uev(Ks1+I,Ks2+J,JL,1) 
     $                    + Uev(Ks1+I,Ks2+J,JL,2)
                     Uev(Ks1+I,Ks2+J,JL,2) = Uev(Ks1+I,Ks2+J,JL,1) 
     $                    - 2.d0*Uev(Ks1+I,Ks2+J,JL,2)
                  ENDDO
               ENDDO
c           ... Calculate elements of the lower-right rectangle 
c               matrix 3x3 L^m_l given by Eq.(L-lm) in STB
               Pp(1:L1,1,2:3,JL) = -Pp(1:L1,1,2:3,JL)
               L = 1
               DO I = 1,3  ;  DO J = 1,I
                  Lev(Kp1+I,Kp2+J,JL,1) = SUM(
     $                 X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J,JL) )
                  S1 = 0.d0 ; I2 = 2
                  DO I1 = Mh+1,Lma,2
                     S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J,JL)
                     I2 = I2 + 2
                  ENDDO
                  Lev(Kp1+I,Kp2+J,JL,2) = S1
                  L = L + 1
               ENDDO  ;  ENDDO            
c           ... The calculation of Lev matrix elements matched by * 
               L = 3
               DO I = 1,2  ; DO J = I+1,3
                  Lev(Kp1+I,Kp2+J,JL,1) = SUM(
     $                 X_MOMS(Mh:Lma:2,LDM(L))*Pp(1:L1:2,4-I,4-J,JL) )
                  S1 = 0.d0 ; I2 = 2
                  DO I1 = Mh+1,Lma,2
                     S1 = S1 + X_MOMS(I1,LDM(L))*Pp(I2,4-I,4-J,JL)
                     I2 = I2 + 2
                  ENDDO
                  Lev(Kp1+I,Kp2+J,JL,2) = S1
                  L = L + 1
               ENDDO  ;  ENDDO               
c              ... The calculation of Lev matrix elements matched by +  
               DO I = 1,3
                  DO J = 1,3
                     Lev(Kp1+I,Kp2+J,JL,1) = Lev(Kp1+I,Kp2+J,JL,1) 
     $                    + Lev(Kp1+I,Kp2+J,JL,2)
                     Lev(Kp1+I,Kp2+J,JL,2) = Lev(Kp1+I,Kp2+J,JL,1) 
     $                    - 2.d0*Lev(Kp1+I,Kp2+J,JL,2)
                  ENDDO
               ENDDO

            ENDDO  ! over sub-layers along LoS (JL)
         ENDDO   ! over local solar angles (K2)
      ENDDO  ! over Gaussian angles (K1)

cc     ... Output for Jl = 1, and JL = 2
c      IF( NLO == 1 ) THEN
c         NAME = './DATA_OUT/Uev_1-1.dat'
c      ELSE
c         NAME = './DATA_OUT/Uev_1-2.dat'
c      ENDIF

c     OPEN(41,file = TRIM(NAME) )
c     DO K2 = 1,NLO
c        WRITE(41,*) ' LoS ',K2
c        Ks = (K2-1)*Nvs         
c        DO K1 = 1,NN 
c           WRITE(41,*) ' Gauss ',K1
c           K11 = 1
c           DO Ka = (K1-1)*4+1,K1*4
c              WRITE(41,'(2(1x,I3),4(e13.5),2x,4(e13.5))') 
c    $              K1, K11, 
c     $              Lev(Ka,Ks+1:K2*4, 1, 1),
c     $              Lev(Ka,Ks+1:K2*4, 2, 1)
c     $              Lev(Ka,Ks+1:K2*4, 1, 2),
c     $              Lev(Ka,Ks+1:K2*4, 2, 2)
c     $              Uev(Ka,Ks+1:K2*4, 1, 1),
c     $              Uev(Ka,Ks+1:K2*4, 2, 1)
c    $              Uev(Ka,Ks+1:K2*4, 1, 2),
c    $              Uev(Ka,Ks+1:K2*4, 2, 2)
c              K11 = K11 + 1
c           ENDDO               ! Stokes comp. for Gauss angles (Ka)
c        ENDDO                  ! Gauss angles (K1)         
c     ENDDO                     ! Line-of=sight (K2)
c     CLOSE( 41 )
c     STOP

      DO K1 = 1,NN
         Ka1 = 4*(K1-1)+1 ;  Ka2 = K1*4
         DO K2 = 1,NLO
            Ks1 = 4*(K2-1)+1 ;  Ks2 = K2*4
            DO JL = 1,N_JL(K2)
               Lev(Ka1:Ka2,Ks1:Ks2,JL,1) = 
     $             Uev(Ka1:Ka2,Ks1:Ks2,JL,1) + Lev(Ka1:Ka2,Ks1:Ks2,JL,1)
               Uev(Ka1:Ka2,Ks1:Ks2,JL,1) = 
     $             Uev(Ka1:Ka2,Ks1:Ks2,JL,2) - Lev(Ka1:Ka2,Ks1:Ks2,JL,2)

               Uev(Ka1:Ka2,Ks1:Ks2,JL,2) = Uev(Ka1:Ka2,Ks1:Ks2,JL,1)
               Uev(Ka1:Ka2,Ks1:Ks2,JL,1) = Lev(Ka1:Ka2,Ks1:Ks2,JL,1)
            ENDDO
         ENDDO
      ENDDO

c      OPEN(41,file = TRIM(NAME) )
c      DO K2 = 1,NLO
c         WRITE(41,*) ' LoS ',K2
c         Ks = (K2-1)*Nvs         
c         DO K1 = 1,NN 
c            WRITE(41,*) ' Gauss ',K1
c            K11 = 1
c            DO Ka = (K1-1)*4+1,K1*4
c               WRITE(41,'(2(1x,I3),4(e13.5),2x,4(e13.5))') 
c     $              K1, K11, 
cc     $              Lev(Ka,Ks+1:K2*4, 1, 1),
cc     $              Lev(Ka,Ks+1:K2*4, 2, 1)
cc     $              Lev(Ka,Ks+1:K2*4, 1, 2),
cc     $              Lev(Ka,Ks+1:K2*4, 2, 2)
cc     $              Uev(Ka,Ks+1:K2*4, 1, 1),
cc     $              Uev(Ka,Ks+1:K2*4, 2, 1)
c     $              Uev(Ka,Ks+1:K2*4, 1, 2),
c     $              Uev(Ka,Ks+1:K2*4, 2, 2)
c               K11 = K11 + 1
c            ENDDO               ! Stokes comp. for Gauss angles (Ka)
c         ENDDO                  ! Gauss angles (K1)         
c      ENDDO                     ! Line-of=sight (K2)
c      CLOSE( 41 )
c      STOP

      NULLIFY( Uev, Po_a, Por_a, Pot_a, Po_i, Por_i, Pot_i )

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
