!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 17 Aug 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Module Pol_Corr
  Use Types, Only: Knd
  Logical Flg
  Character (Len = :), Allocatable :: Path
  Real(Kind = 8) :: Hgt, SZA, VZA, RAZ
  Type Saved_Measurements
     Real (Knd), Allocatable :: Arr(:)  
  End Type Saved_Measurements
  Type (Saved_Measurements), Allocatable :: InitMsmt(:)
  Private
  Public :: PolCorr, Path, Flg
  
Contains
!***************************************************************************
  Subroutine PolCorr (FIRST_RUN)
    ! Called by gt_iface.f
    Use Alb_Pars,   Only: Albedo_Ret, Albedo_Const
    Use Data_Ret,   Only: MDS
    Use Grids,      Only: ZZZ, NL => NLAYERS
    Use Info,       Only: TG_Info
    Use Profiles,   Only: O3_Total_Column
    Use Spher_Pars, Only: Ang_Out
    Use ST_Vars,    Only: Rad_To_Deg, Ground_Height, NSEGS
    Use Tools,      Only: Find_Idx
    Logical First_Run
    ! Local Variables
    Character (Len = *), Parameter :: R_Name = 'Polariz_Corr'
    Integer Idx, NDS, II, JJ, KK
    Real(Kind = 8) :: TOC, Wgt, Alb, CorrFactor
    Real(kind=8), Dimension(7601) :: CorrSpectrum, Wvl
    !==================================================================
    Call ST_Track (R_Name, 1, 'I')

    NDS = Size(MDS)
    Idx = Find_Idx('o3',TG_Info%Name)
    Associate(ND => TG_Info(Idx)%ND(:))
      TOC = 0.5D0*Dot_Product((ND(1:NL - 1) + ND(2:NL)),(ZZZ(1:NL - 1) - ZZZ(2:NL)))
    End Associate
    TOC = TOC/2.69E16 ! Convert to DU
    
    If(First_Run) Then
       Alb = Albedo_Const          
       Hgt = Ground_Height         
       RAZ = Ang_Out(3,1)*Rad_To_Deg
       SZA = ACOS(ANG_OUT(1,1))*Rad_To_Deg
       VZA = ACOS(ANG_OUT(2,1))*Rad_To_Deg
       If(Allocated(InitMsmt)) Then
          Do II = 1, NDS
             DeAllocate(InitMsmt(II)%Arr)
          End Do
          DeAllocate(InitMsmt)
       End If
       Allocate(InitMsmt(NDS)) 
       Do II = 1, NDS
          Allocate(InitMsmt(II)%Arr, Source = MDS(II)%MSMT(:,1,1))
       End Do
    Else
       Alb = ALBEDO_RET(3,NSEGS) ! Retrieved albedo value
       
!       WRITE(*,*) 'TOTAL OZONE:'
!       WRITE(*,*)  TOC
    END IF

    Call PolarizCorr(Alb, TOC, RAZ, SZA, VZA, Hgt, CorrSpectrum, Wvl)
    Do JJ = 1, Size(MDS(1)%WL)
       DO II = 1, NDS
          Interp: Block
            If(II /= 1) Then
               If(JJ <= Size(MDS(II - 1)%WL)) Then
                  If(MDS(II)%WL(JJ) == MDS(II - 1)%WL(JJ)) Exit Interp
               End If
            End If
         
            If (MDS(II)%WL(JJ) <= Wvl(size(Wvl)) .AND. &
                 MDS(II)%WL(JJ) >= Wvl(1)) Then
               KK = (MDS(II)%WL(JJ) - Wvl(1))/((Wvl(Size(Wvl))- Wvl(1))/&
                    Size(wvl))
               Wgt = (MDS(II)%WL(JJ) - Wvl(KK))/(Wvl(KK + 1) - Wvl(KK))
               CorrFactor = (1.0D0 - Wgt)*CorrSpectrum(KK) + &
                    + Wgt*CorrSpectrum(KK + 1)
            Else
               CorrFactor = 1
            End if
          End Block Interp
          
          MDS(II)%MSMT(JJ,1,1) = InitMsmt(II)%Arr(JJ) * CorrFactor
       End Do
!       write(6,*) JJ, MDS(1)%MSMT(JJ,1,1), MDS(2)%MSMT(JJ,1,1)
    End Do

    Call ST_Track (R_Name, 1, 'O')
    Return
  End Subroutine PolCorr
  !***************************************************************************
  Subroutine PolarizCorr (alb0, ozo0, raz0, sza0, vza0, hgt0, corr_spectrum,wvl)

    ! varaibles for LUT:
    integer(kind=4),parameter :: nspec=7601,tabdim=6
    real(kind=8) :: wl0=267.0d0,wl1=343.0d0,dwl=.01d0
    real(kind=8),dimension(nspec) :: wvl,spectrum1, spectrum2, corr_spectrum
    real(kind=8),dimension(tabdim) :: var0(tabdim)
    real(kind=8) :: sza0,vza0,raz0,alb0,hgt0,ozo0
    !real(kind=8) :: sza1,vza1,raz1,alb1,hgt1,ozo1
    logical :: ipmode=.true.
    integer(kind=4) :: lun,i
    CHARACTER (LEN=200) :: fle1, fle2
    ! ----------------

    fle1= Path//'/intensity_noring_database_267_343.bin'
    fle2= Path//'/pol_database_267_343.bin'

    var0=[raz0,vza0,sza0,alb0,hgt0,ozo0]

    do i=1,nspec
       wvl(i)=wl0+(i-1.d0)*0.01d0
    enddo
    call ring_spec (fle1,wl0,wl1,wvl,spectrum1,nspec, var0,tabdim,ipmode)
    call ring_spec (fle2,wl0,wl1,wvl,spectrum2,nspec, var0,tabdim,ipmode)
    
    do i=1,nspec
       corr_spectrum(i)=spectrum1(i)/spectrum2(i)
    enddo

    ! open(12,file='correction_polarisation.dat', status='unknown',form='formatted')
    ! do i=1,nspec
    !    write(12,'(f10.3,1x,e12.6,1x,e12.6)') wvl(i),spectrum1(i),spectrum2(i)
    ! enddo
    ! close(12)
  end subroutine PolarizCorr
  !***************************************************************************
  subroutine  ring_spec (fle,wl0,wl1,wvl,spectrum,nspec, var0,tabdim,ipmode)
    ! find spectrum for the parameters var0=[raz0,vza0,sza0,sza0,alb0,hgt0,ozo0]
    ! using linear interpolation from a  lookup table
    !
    ! input: var0=[sza0,vza0,raz0,alb0,hgt0,ozo0]
    !        tabdim=6
    !        fle: binary database file name
    !        wl0,wl1: starting and end wavlengths (spacing: 0.01 nm)
    !   parameters for which spectrum shall be obtained from LUT
    !   ipmode=.t. --> linear interpolation
    !   ipmode=.f. --> nearest neightbor
    ! output: wvl, spectrum
    
    !implicit none
    character(200) :: fle
    integer(kind=4),intent(in)  :: nspec,tabdim
    real(kind=8),dimension(nspec),intent(inout) :: wvl,spectrum
    real(kind=8),dimension(tabdim),intent(inout) :: var0
    real(kind=8),intent(inout) :: wl0,wl1
    logical,intent(inout) :: ipmode
    
    real(kind=4),dimension(nspec) :: tmpspec
    integer(kind=8),parameter :: nbyte=4,nvarmax=20
    real(kind=8),dimension(tabdim) :: var1,diff
    integer(kind=8) :: lun,i,i1,i2,i3,i4,i5,i6,i7,i8,i9
    real(kind=8) :: dwl,factor
    !integer(kind=8),parameter :: n1=7,n2=6,n3=12,n4=10,n5=2,n6=6
    !integer(kind=4),parameter :: n1=7,n2=6,n3=12,n4=10,n5=10,n6=6,nbyte=4,tabdim=6
    integer(kind=8) :: nrec
    real(kind=8),dimension(tabdim,nvarmax) :: var
    integer(kind=8),dimension(tabdim) :: jjl,jju,jj,ndim
    real(kind=4) recltest
    integer reclen
    
    
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    ! table setting
    !
    !ndim=[7,6,12,10,2,6]
    ndim=[7,6,13,10,10,6]
    var(1,1:ndim(1))=dble([0,30,60,90,120,150,180]) !raz
    var(2,1:ndim(2))=dble([0,20,40,50,60,70]) !vza
    var(3,1:ndim(3))=dble([0,20,30,40,50,60,65,70,75,80,84,86,88]) !sza
    var(4,1:ndim(4))=dble([0.0,0.03,0.06,.1,.3,.5,.7,.8,.9,1.]) !alb
    !var(5,1:ndim(5))=dble([0,2]) !hgt
    var(5,1:ndim(5))=dble([0,2,4,6,8,10,12,14,16,18]) !hgt
    var(6,1:ndim(6))=dble([100,200,300,400,500,600]) !ozo
    
    
    !end table settings
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    do i=1,tabdim
       call locate_lut(var(i,1:ndim(i)),ndim(i),var0(i),var1(i), jjl(i),jju(i))
       diff(i)=(var1(i)-var(i,jjl(i)))/(var(i,jju(i))-var(i,jjl(i)))
    enddo
    inquire(iolength=reclen) recltest
    open(11 ,file=trim(fle),access='direct', form='unformatted',recl=reclen*nspec)     ! gfortran: nbyte*nspec
    spectrum=0.d0
    if ( ipmode.eqv..false.) then
       do i=1,tabdim
          if (diff(i) .gt. 0.5d0) jjl(i)=jju(i)
       enddo
       nrec=(jjl(tabdim)-1)
       Do i=tabdim-1,1,-1
          nrec=nrec*ndim(i)+jjl(i)-1
       enddo
       nrec=nrec+1
       
       read(11,rec=nrec) tmpspec(1:nspec)
       spectrum(1:nspec) = dble(tmpspec(1:nspec))
    else
!!! note if table dimension changes (tabdim) then add more  loops i7,....
       do i1 = jjl(1), jju(1)
          do i2 = jjl(2), jju(2)
             do i3 = jjl(3), jju(3)
                do i4 = jjl(4), jju(4)
                   do i5 = jjl(5), jju(5)
                      do i6 = jjl(6), jju(6)
                         jj(1)=i1
                         jj(2)=i2
                         jj(3)=i3
                         jj(4)=i4
                         jj(5)=i5
                         jj(6)=i6
                         nrec=(jj(tabdim)-1)
                         Do i=tabdim-1,1,-1
                            nrec=nrec*ndim(i)+jj(i)-1
                         enddo
                         nrec=nrec+1
                         factor=1.d0
                         do i=1,tabdim
                            factor = factor*(1.d0 - (jj(i)-jjl(i)) + 2*diff(i) &
                                 * (jj(i)-jjl(i)) - diff(i))
                         enddo
                         read(11,rec=nrec) tmpspec(1:nspec)
                         spectrum(1:nspec) = spectrum(1:nspec) &
                              +factor*dble(tmpspec(1:nspec))
                      enddo
                   enddo
                enddo
             enddo
          enddo
       enddo
    endif
    close(11)
  end subroutine ring_spec
  !***************************************************************************
  subroutine locate_lut(xx,n,x,xnew,ilow,iup)
    
    !
    ! find the lower of the neighbouring indices of  array bracketing
    ! the search value
    !
    ! input:
    !   xx: array values
    !   x: search value
    ! output:
    !   xnew: updated search value in case x is out of range
    !   (may be  first or last value of array otherwise identical to
    !    variable x)
    !   ilo and iup: bracketing indices of array
    !  IMPLICIT NONE
    INTEGER(kind=8),intent(in) :: n
    INTEGER(kind=8),intent(inout) :: ilow,iup
    REAL(kind=8), DIMENSION(n), INTENT(INOUT) :: xx
    REAL(kind=8), INTENT(INOUT) :: x,xnew
    INTEGER(kind=8) :: jl,jm,ju
    LOGICAL :: ascnd
    !n=size(xx)
    ascnd = (xx(n) >= xx(1))
    !jl=0
    !ju=n+1
    jl=1
    ju=n
    do
       if (ju-jl <= 1) exit
       jm=(ju+jl)/2
       if (ascnd .eqv. (x >= xx(jm))) then
          jl=jm
       else
          ju=jm
       end if
    end do
    xnew=x
    if (x <= xx(1)) then
       ilow   =1
       xnew=xx(1)
    else if (x >= xx(n)) then
       ilow   =n-1
       xnew=xx(n)
    else
       ilow=jl
    end if
    iup=ilow+1
  END subroutine locate_lut
!***************************************************************************
End Module Pol_Corr
!***************************************************************************
