C**********************************************************************
C
C       SCIATRAN - Version 4.5 - Last modification: 12 Nov 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c *********************************************************************
      SUBROUTINE POST_PROCESSING(Mh, E_W, RADOUT, POS,
     $     GSL_A, GSL_W, IOP_A, IOP_W, SSB_A, SSB_W, TE,  
     $     SDO_A, SDO_W, Ph_zA, Ph_zW, LAS_A, LAS_W,
     $     UDB_A, UDB_W, LOA, LOW, AGR_A, AGR_W, GeoD, DELS)
c     ... Called by "st_dom_tau.f"

      USE GRIDS,       ONLY: NL => NLAYERS
      USE INTERN_DIMS, ONLY: NL_pla, Nvs, Ng1_brf, Ns_brf, NN_pla,
     $                       N_sol_ls
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_XSO
      USE NODES,       ONLY: N_NODE, MJU_NODE, N_PLUS, N_MINUS
      USE RTM_OUTPUT,  ONLY: Wup, N_OP
      USE ST_VARS,     ONLY: Vec_Sca, Pie, SPECULAR, WF_FLAG, Kvs,
     $                       Interp_RF!, VST, Finc
      USE SPHER_PARS,  ONLY: JV_S, JV_E, JS_E, N_LOS_ANG,
     $                       N_AZ_VAR, Nf => NN, ANG_OUT, N_LOS_OUT,
     $                       NOA => N_OBS_ALT, SET_OUT_ALT
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: TE_TYPE, SDO_TYPE, UDB_TYPE, SSB_TYPE,
     $                       GSL_TYPE, IOP_TYPE, LAS_TYPE, LOA_TYPE,
     $                       LOW_TYPE, AGR_type, Geometry_Data
      USE WATER,       ONLY: UWT, NL_W => NLAYERS_UW, NSTR_T,
     $                       Obs_POSITION
c    -------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: Mh
      REAL (KND), INTENT(IN) :: E_W, POS
      REAL (KND)  RADOUT(N_OP,N_LOS_OUT,Nvs)
      REAL (KND), ALLOCATABLE :: DELS(:,:,:,:)
c     ----------------------------------
      TYPE (GSL_type) :: GSL_A, GSL_W ! Angles
      TYPE (IOP_type), TARGET  :: IOP_A, IOP_W ! Integral optical parameters
      TYPE (LAS_TYPE) :: LAS_A, LAS_W ! Linear algebraic system and related
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of water
      TYPE (SSB_type) :: SSB_A, SSB_W ! Spherical solar beam
      TYPE (SDO_type) :: SDO_A, SDO_W ! DOM auxiliary solution variables
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
      TYPE (AGR_type) :: AGR_A, AGR_W
c     -------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA, Ph_zW
      TYPE (Geometry_Data) GeoD
c     ... Local parameters
      CHARACTER (LEN=15), PARAMETER :: R_NAME = 'POST_PROCESSING'
      INTEGER  I, J, K, L, IA, Js, Jg, Jo, LL1, NN, NNv, Nla, NNa, NNw, 
     $     N_PLS, NLSA, NSZA, N1, N2, N1s, N2s, K1, Ja
      REAL (KND)  HH
      REAL (KND), ALLOCATABLE ::  XML(:), Sf_a(:,:,:), Sf_b(:,:,:),
     $     P0_LS(:,:,:,:), Sfi_res(:,:,:), Tr_gau(:), X_Ref(:), Hel(:),
     $     Fre(:), Trans(:,:)
      REAL (KND), POINTER :: Phi_p(:,:), Phi_m(:,:), EgL_1(:,:,:),
     $     EgL_2(:,:,:), Muv(:), AG(:), ALW(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      NSZA = GSL_A%NSZA 
      NLSA = N_LOS_ANG 
      IA = 1  

c     ... Specific case SFI to calculate RF at all spherical nodes
      IF( .NOT.Interp_RF) THEN
         NSZA = GeoD%NSOL  !!! Check for option "one" !!!
         NLSA = N_PLUS  ! Number of nodes in upward direction
         ALLOCATE( Trans(Kvs*NSZA,Nvs*NLSA) ) ; Trans = 0.d0
         N1 = 1   ;  N2 = Nvs*N_PLUS
         N1s = 1  ;  N2s = N_PLUS
         CALL SET_ATM
         CALL BEFORE_FLS(Nla, SDO_A)
         CALL ST_UP_SFI(Nla, Mh, NLSA, GSL_A, IOP_A, SSB_A, TE, SDO_A,
     $        Sf_a, Sf_b, P0_LS, N_PLS, LAS_A%FLUX, LAS_A%Ref_surf,
     $        UDB_A, LOA%Bet, Trans, Sfi_res, NSZA, POS)
         DO I = 1, NLa
            DO Js = 1, NSZA
c               WRITE(*,*) ' --- ', I,  Sfi_res(I,Js,1:Nvs*NLSA:Nvs)  
               DO K1 = 1,Nvs    
                  DELs(I,Js,1:N_PLUS,K1) =
     $                 Sfi_res(I,Js,K1:Nvs*NLSA:Nvs)   
               ENDDO  ! over Stokes components (K1)
            ENDDO  ! over selected solar angles (Js)
         ENDDO  ! over main altitude levels (I)
         DEALLOCATE( Trans, XML, Sf_a, Sf_b, P0_LS, Sfi_res )
         
         NLSA = N_MINUS  ! Number of nodes in downward direction
         ALLOCATE( Trans(Kvs*NSZA,Nvs*NLSA) ) ; Trans = 0.d0
         N1 = Nvs*N_PLUS+1 ;  N2 = Nvs*N_NODE
         N1s = N_PLUS+1    ;  N2s = N_NODE
         CALL SET_ATM  ;  CALL BEFORE_FLS( Nla, SDO_A )
         CALL ST_DOWN_SFI(Nla, NLSA, GSL_A, IOP_A, SSB_A, TE, SDO_A,
     $        P0_LS, N_PLS, Sf_a, Sf_b, LOA%Bet, Trans, Sfi_res, 
     $        UDB_A%SPECULAR, NSZA, UDB_A)
         DO I = 1, NLa
            DO Js = 1, NSZA
               DO K1 = 1,Nvs    
                  DELs(I,Js,1+N_PLUS:N_NODE,K1) =
     $                 Sfi_res(I,Js,K1:Nvs*NLSA:Nvs)   
               ENDDO  ! over Stokes components (K1)
            ENDDO  ! over selected solar angles (Js)
         ENDDO  ! over main altitude levels (I)

         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF  ! .NOT.Interp_RF

      N1 = 1   ;  N2 = Nvs*NLSA
      N1s = 1  ;  N2s = NLSA
      ALLOCATE( Trans(Kvs*NSZA,Nvs*NLSA) ) ; Trans = 0.d0

      IF( .NOT.UWT ) THEN
c     -------------------------------------------------
c     ... Atmosphere no coupling
c     -------------------------------------------------
         SELECT CASE(  GeoD%View(IA)%S)
         CASE('up')  ! Upward radiation in the atmosphere
c        ... Last dimension of array P0_LS
            CALL SET_ATM  ;  CALL BEFORE_FLS(Nla, SDO_A)
            CALL ST_UP_SFI(Nla, Mh, NLSA, GSL_A, IOP_A, SSB_A, TE, 
     $           SDO_A, Sf_a, Sf_b, P0_LS, N_PLS, LAS_A%FLUX, 
     $           LAS_A%Ref_surf, UDB_A, LOA%Bet, Trans, Sfi_res, 
     $           NSZA, POS)
c        ... Save upward intensity at all altitude level in directions
c            of line-of-sight (used in WF-routine)
            IOP_A%Ref_dir(:,:,:) = Sfi_res(:,:,:)
            CALL SET_RADOUT
         CASE('down')  ! Downward radiation in the atmosphere
            CALL SET_ATM  ;  CALL BEFORE_FLS( Nla, SDO_A )
            CALL ST_DOWN_SFI( Nla, NLSA, GSL_A, IOP_A, SSB_A, TE, SDO_A,
     $           P0_LS, N_PLS, Sf_a, Sf_b, LOA%Bet, Trans, Sfi_res, 
     $           UDB_A%SPECULAR, NSZA, UDB_A)
c        ... Save downward intensity at all altitude level in directions
c            of line-of-sight (used in WF-routine)
            IOP_A%Ref_dir(:,:,:) = Sfi_res(:,:,:)
            CALL SET_RADOUT
         END SELECT
      ELSE  ! UWT = .TRUE.
c     -------------------------------------------------------------
c     ... Coupled atmosphere-ocean system
c     -------------------------------------------------------------
c         WRITE(*,*) '  GeoD%View(IA)%S  ', GeoD%View(IA)%S
         SELECT CASE(GeoD%View(IA)%S)
         CASE('up')  ! ----->> Upward radiation (downwards looking instrument)
            SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
            CASE('Ocean')  ! Instrument  in ocean
               CALL SET_OCEAN  ;  CALL BEFORE_FLS( Nla, SDO_W )
               CALL ST_UP_SFI(Nla, Mh, NLSA, GSL_W, IOP_W, SSB_W, TE, 
     $              SDO_W, Sf_a, Sf_b, P0_LS, N_PLS, LAS_W%FLUX, 
     $              LAS_W%Ref_surf, UDB_W, LOW%Bet, Trans, Sfi_res,
     $              NSZA, POS)
c               WRITE(*,*) '   in AO_up  ----->  ocean ' 
               IOP_W%Ref_dir(:,:,:) = Sfi_res(:,:,:)
               CALL SET_RADOUT
            CASE('Atmos')  ! Instrument in atmosphere
c             ... Contribution of radiation transmitted from ocean
c                 to atmosphere and reflected from ocean-atmosphere interface
c               write(*,*) '    Instrument in atmosphere '
c               write(*,*) '   SPECULAR   ', SPECULAR
               NNa = GSL_A%NN  ;  NNw = GSL_W%NN
               IF( SPECULAR ) THEN
                  CALL BACK_TO_ATM_FLAT 
               ELSE
                  CALL BACK_TO_ATM_ROUGH
               ENDIF
c               write(*,*) '     Before set_atm  '
               CALL SET_ATM  ;  CALL BEFORE_FLS(Nla, SDO_A)
               CALL ST_UP_SFI(Nla, Mh, NLSA, GSL_A, IOP_A, SSB_A, TE,
     $               SDO_A, Sf_a, Sf_b, P0_LS, N_PLS, LAS_A%FLUX, 
     $              LAS_A%Ref_surf, UDB_A, LOA%Bet, Trans, Sfi_res,
     $              NSZA, POS)
c            ... Save upward intensity at all altitude level in directions
c                 of line-of-sight (used in WF-routine for atmosphere)
c               WRITE(*,*) '   in AO_up  ----->  atmosphere ' 
c               WRITE(LUNIT,*) 
c     $              '-1  Ref_dir in atm. Sfi_Res: ', SUM(Sfi_Res)
               IOP_A%Ref_dir(:,:,:) = Sfi_res(:,:,:)
c               WRITE(*,*) '     Coupling: mode - "Atmos"'
c               WRITE(*,*) IOP_A%Ref_dir(:,1,1) 
               CALL SET_RADOUT
               DEALLOCATE( XML, Sf_a, Sf_b, P0_LS, Sfi_res )

c             ... The calculation of WF in ocean requires upward intensity at 
c                 all altitude level in directions of line-of-sight in ocean
               IF( WF_FLAG ) THEN
                  Trans = 0.d0
                  CALL SET_OCEAN  ;  CALL BEFORE_FLS(Nla, SDO_W)
                  CALL ST_UP_SFI(Nla, Mh, NLSA, GSL_W, IOP_W, SSB_W, TE, 
     $                 SDO_W, Sf_a, Sf_b, P0_LS, N_PLS, LAS_W%FLUX, 
     $                 LAS_W%Ref_surf, UDB_W, LOW%Bet, Trans, Sfi_res,
     $                 NSZA, POS)
c                  WRITE(LUNIT,*) 
c     $              '-2  Ref_dir in ocean Sfi_Res: ', SUM(Sfi_Res)
                  IOP_W%Ref_dir(:,:,:) = Sfi_res(:,:,:)
c                 WRITE(*,*) IOP_W%Ref_dir(:,1,1) 
               END IF  ! WF_FLAG
            END SELECT  ! Obs. position
c        ------------------------------------------------------------------
c        ------------------------------------------------------------------
         CASE('down') !  -------> Downward radiation
            SELECT CASE( TRIM(Obs_POSITION(1)) )
            CASE('Atmos') ! Instrument in  atmosphere
               CALL SET_ATM  ;  CALL BEFORE_FLS( Nla, SDO_A )
               CALL ST_DOWN_SFI( Nla, NLSA, GSL_A, IOP_A, SSB_A, TE, 
     $              SDO_A, P0_LS, N_PLS, Sf_a, Sf_b, LOA%Bet, Trans, 
     $              Sfi_res, UDB_A%SPECULAR, NSZA, UDB_A )
               CALL SET_RADOUT
            CASE('Ocean')  ! Instrument in ocean
c            ... Contribution of radiation transmitted from atmosphere
c                and reflected from ocean-atmosphere interface to the ocean 
               NNa = GSL_A%NN  ;  NNw = GSL_W%NN
               IF( SPECULAR ) THEN
                  CALL BACK_TO_WATER_FLAT 
               ELSE
                  CALL BACK_TO_WATER_ROUGH
               ENDIF
               CALL SET_OCEAN  ;  CALL BEFORE_FLS( Nla, SDO_W )
               CALL ST_DOWN_SFI( Nla, NLSA, GSL_W, IOP_W, SSB_W, TE, 
     $              SDO_W, P0_LS, N_PLS, Sf_a, Sf_b, LOW%Bet, Trans, 
     $              Sfi_res, UDB_W%SPECULAR, NSZA, UDB_A ) 
               CALL SET_RADOUT
            END SELECT  ! Obs. position
         END SELECT  ! Upward or downward radiation
      ENDIF  ! UWT

      NULLIFY( Phi_p, Phi_m )
c      stop
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE SET_ATM

c     ==================================================================
      CHARACTER (LEN=7), PARAMETER :: R_NAME = 'SET_ATM'
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      N_PLS = N_sol_ls
      NN = GSL_A%NN ; NNv = NN*Nvs ; NLa = NL 
c           WRITE(*,*) '   N1s, N2s ', N1s, N2s
c           WRITE(*,*) '  SHAPE( Ph_zA%EgL ) ', SHAPE( Ph_zA%EgL )
      EgL_1 => Ph_zA%EgL(:,:,N1:N2,1)
      EgL_2 => Ph_zA%EgL(:,:,N1:N2,2)
      AG  => GSL_A%AG(:) ; ALW => IOP_A%ALW(:)
      IF( .NOT.Interp_RF) THEN
         Muv => MJU_NODE(N1s:N2s)
c         WRITE(*,*) '   Muv in "post_processing" '
c         DO I = 1, SIZE(Muv)
c            WRITE(LUNIT,'(1x,I3,*(2x,E12.4))') 
c     $           I, Muv(I), MJU_NODE(I)
c         ENDDO
c         PAUSE
      ELSE
         Muv => GSL_A%Muv(:)
      ENDIF
      
      ALLOCATE( XML(NN), Sf_a(NLa-1,Nvs*NLSA,NNv), 
     $     Sf_b(NLa-1,Nvs*NLSA,NNv), 
     $     P0_LS(NLa,Nvs*NLSA,Nvs*NSZA,N_sol_ls),
     $     Sfi_res(Nla,Kvs*NSZA,Nvs*NLSA) )

c     ... Set phase matrix solar => line-of-sight
      P0_LS(:,:,:,:) = Ph_zA%EsL(:,N1:N2,:,:)

      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = GSL_A%XM(:)         
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
      END SELECT

      CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE SET_ATM
c **********************************************************************

      SUBROUTINE SET_OCEAN
c     ==================================================================
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'SET_OCEAN'
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      N_PLS = 2
      NN = GSL_W%NN ; NNv = NN*Nvs ; NLa = NL_W
      EgL_1 => Ph_zW%EgL(:,:,:,1)
      EgL_2 => Ph_zW%EgL(:,:,:,2)
      Muv => GSL_W%Muv(:) ; AG => GSL_W%AG(:) ; ALW => IOP_W%ALW(:)

      ALLOCATE( XML(NN), Sf_a(NLa-1,Nvs*NLSA,NNv), 
     $     Sf_b(NLa-1,Nvs*NLSA,NNv), 
     $     P0_LS(NLa,Nvs*NLSA,Nvs*NSZA,2),
     $     Sfi_res(Nla,Kvs*NSZA,Nvs*NLSA) )

c     ... Set phase matrix solar => line-of-sight
      P0_LS(:,:,:,:) = Ph_zW%EsL(:,:,:,:)

      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = GSL_W%XM(:)         
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
      END SELECT
      CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE SET_OCEAN

c **********************************************************************
c     This routine calculates the multiple scattering source function
c     using the matrix of all elementary solutions and corresponding
c                     elements of the phase matrix
c **********************************************************************
      SUBROUTINE BEFORE_FLS( Nla, SDO )

      USE LOG_PARS, ONLY: LUNIT, LOG_LEVEL, N_XSO
      USE TYPES,    ONLY: SDO_TYPE
c     ------------------------------------------------------------------

      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'BEFORE_FLS'
      INTEGER     Nla, I, Jv, Jg, K2
      REAL (KND)  Wdo, Ugp(Nvs*NLSA,NNv), Ugm(Nvs*NLSA,NNv),
     $            TTT(NNv,Nvs*NLSA)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Wdo = 1.d0 - Wup
c     ... EgL_2(I,:,:) represents according to Eq.(Gpp) in STB
c               Mu_ga => Mu_los (see 'phase_matr.f')
      DO I = 1,NLa - 1
c        ... First dimension is the Gaussian angles. In the scalar case
c            was divided by XM(K) in vector not.
         TTT(:,:) = Wup*EgL_2(I,:,:) + Wdo*EgL_2(I+1,:,:) 
         DO Jv = 1,NLSA 
            DO Jg = 1,NN
               Ugp(Nvs*(Jv-1)+1:Nvs*Jv,Nvs*(Jg-1)+1:Nvs*Jg) = 
     $              TTT(Nvs*(Jg-1)+1:Nvs*Jg,Nvs*(Jv-1)+1:Nvs*Jv)
     $              /DABS(Muv(Jv))
            ENDDO
         ENDDO
c        ... EgL_1(I,:,:) is given according to Eq.(Gpm) in STB
c            (see 'phase_matr.f')
c        ... First dimension is the Gaussian angles. In the scalar
c            case was divided by XM(K) in vector not.

c        ... Because the matrix ^M = EgL_1(I,:,:) is designed so  
c            that D*^M describes the phase mtrix -Muv <-- Gau,
c            we need to use ^M*D  to obtain phase mtrix Muv <-- -Gau.
c            (see second term in Eq.(SF-forw) in STB).
c            The multiplication by D from right is, however, omitted 
c            because below this matrix will be multiplied by the 
c            Stokes vector of the upward radiation which should be
c            multiplied by D from left, i.e. ^M*D * D*S(-Gau)
         TTT(:,:) =   Wup*EgL_1(I,:,:) + Wdo*EgL_1(I+1,:,:) 
         DO Jv = 1, NLSA 
            DO Jg = 1,NN
               Ugm(Nvs*(Jv-1)+1:Nvs*Jv,Nvs*(Jg-1)+1:Nvs*Jg) = 
     $              TTT(Nvs*(Jg-1)+1:Nvs*Jg,Nvs*(Jv-1)+1:Nvs*Jv)
     $              /DABS(Muv(Jv))
            ENDDO
         ENDDO
c        ... In vector case XML(K)=1, in the scalar case
c            XML(K)=XM(K) - cosines of Gaussian angle-grid
         FORALL( K = 1:NN )
           Ugp(:,Nvs*(K-1)+1:Nvs*K) = Ugp(:,Nvs*(K-1)+1:Nvs*K)
     $          *AG(K)*XML(K)*0.5d0
           Ugm(:,Nvs*(K-1)+1:Nvs*K) = Ugm(:,Nvs*(K-1)+1:Nvs*K)
     $          *AG(K)*XML(K)*0.5d0
         END FORALL

c         Scalar version
c         --------------
c         FORALL( K = 1:NLSA )
c         Ugp(K,:) = (Wup*EgL_2(I,:,K) +  Wdo*EgL_2(I+1,:,K))
c     $        *GSL%XM(:)*AG(:)/Muv(K)*0.5d0
c         Ugm(K,:) = (Wup*EgL_1(I,:,K) + Wdo*EgL_1(I+1,:,K))
c     $        *GSL%XM(:)*AG(:)/Muv(K)*0.5d0
c         END FORALL

         Ugp(:,:) = Ugp(:,:)/ALW(I) ; Ugm(:,:) = Ugm(:,:)/ALW(I) 
c         
c        ... The matrix G_NL has the following structure 
c            (see homog_partic.f)
c                       1:NNv   1+NNv:2*NNv
c             1:NNv   |  Phi+,    Phi- |  
c         1+NNv:2*NNv |  Phi-,    Phi+ |
c
c        Second dimension - Gaussian angles
c        Third dimension  - separation constants
         Phi_p => SDO%G_NL(I,    1:NNv,  1:NNv) 
         Phi_m => SDO%G_NL(I,1+NNv:2*NNv,1:NNv)

c       ... Arrays Sf_a and Sf_b are calculated according to 
c           Eqs.(Sfa-d), (Sfb-d) for downward and according to 
c           Eqs.(Sfa-u), (Sfb-u) for upward radiation (see STB)
         DO K = 1,NNv
            Sf_a(I,:,K) = 
     $         MATMUL(Ugp(:,:),Phi_p(:,K)) + MATMUL(Ugm(:,:),Phi_m(:,K))
            Sf_b(I,:,K) = 
     $         MATMUL(Ugp(:,:),Phi_m(:,K)) + MATMUL(Ugm(:,:),Phi_p(:,K))
         ENDDO                  ! over separation constants (K)
      ENDDO

      IF( LOG_LEVEL(1) <= N_XSO ) THEN
        WRITE(LUNIT,*) '    Ph_zA(W)%EsL(:,:,:,:):', SUM(P0_LS(:,:,:,1))
        WRITE(LUNIT,*) '    Ph_zA(W)%EgL(:,:,:,1):', SUM(EgL_1(:,:,:))
        WRITE(LUNIT,*) '    Ph_zA(W)%EgL(:,:,:,2):', SUM(EgL_2(:,:,:))
      ENDIF
      CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE BEFORE_FLS
c *************************************************************************

      SUBROUTINE BACK_TO_ATM_FLAT
      INTEGER K, NNv, I1

      NNv = NNa*Nvs
      ALLOCATE( Tr_gau(NNv) )
      DO Js = 1, NSZA
c        ... Upward radiation in the atmosphere at ocean-atmosphere 
c     interface at all Gauss angles
         Tr_gau(:) = LAS_A%SOL_BOT(NNv+1:2*NNv,Js)
c       ... Interpolate into the line-of-sight directions
         DO K = 1, Nvs
            CALL AKIMA_INT( NNa, GSL_A%XM, Tr_gau(K:NNv:Nvs),  
     $           NLSA, GSL_A%Muv, Trans(Js,K:Nvs*NLSA:Nvs) )
         ENDDO  ! over Stokes components (K)
c        ... Linear extrapolation for line-of-sight angles which 
c            are outside of Gaussian grid
         DO I = 1, NLSA  ;  I1 = (I-1)*Nvs
            HH = GSL_A%Muv(I)
            IF(HH <= GSL_A%XM(NNa).AND.HH >= GSL_A%XM(1)) CYCLE
            DO K = 1, Nvs
               CALL LIN_INT(NNa, GSL_A%XM, Tr_gau(K:NNv:Nvs),
     $              1, HH, Trans(Js,I1+K), 'L')
            ENDDO
         ENDDO
      ENDDO  ! over SZA (Js)
      END SUBROUTINE BACK_TO_ATM_FLAT
      
c ----------------------------------------------------------------------
c     Transmitted from the ocean upward radiation just above the
c               ocean-atmosphere interface
c ----------------------------------------------------------------------
      SUBROUTINE BACK_TO_ATM_ROUGH
      INTEGER  NWv, NAv
      REAL (KND) HDU(Nvs*NNw)

      NWv = NNw*Nvs ;  NAv = NNa*Nvs
      ALLOCATE( Tr_gau(NAv) )
      DO Js = 1, NSZA
         DO K = 1,Nvs
            HDU(K:NWv:Nvs) =
     $           LAS_W%SOL_TOP(NWv+K:2*NWv:Nvs,Js)*GSL_W%AG(:)
         ENDDO  ! over Stokes components (K)
c        ... Sign for U and V components of the Stokes vector
         IF( Nvs > 1 ) THEN
            HDU(3:NWv:Nvs) = -HDU(3:NWv:Nvs) 
            HDU(4:NWv:Nvs) = -HDU(4:NWv:Nvs) 
         ENDIF
         DO I = 1,NAv 
            Tr_gau(I) = SUM( UDB_W%Twa_GG(I,1:NWv,Mh)*HDU )
         ENDDO
c         WRITE(*,*) ' Upward radiation in ocean just below surface'
c         DO I = 1,NWv
c            WRITE(*,'(2x,F8.3,2x,e12.4)')
c     $           DACOS(GSL_W%XM(I))/Pie*180.,
c     $           LAS_W%SOL_TOP(NWv+I,Js) 
c         ENDDO         
c        ... Interpolation of the transmitted from ocean to the atmosphere 
c            radiation onto line-of-sight directions
         DO K = 1, Nvs
            CALL AKIMA_INT( NNa, GSL_A%XM, Tr_gau(K:NAv:Nvs),
     $           NLSA, GSL_A%Muv, Trans(Js,K:Nvs*NLSA:Nvs) )
         ENDDO  ! over Stokes components (K)
c     ... Trans() will be multiplied by 2.d0*Pie in "st_down_sfi.f"
      ENDDO  ! over SZA (Js)

c      WRITE(*,*) '  Transmitted to the atmosphere just above surface'
c      DO K = 1, NLSA
c         WRITE(*,'(2x,f8.3,2x,e12.4)')
c     $        DACOS(GSL_A%Muv(K))/Pie*180., Trans(1,K)
c      ENDDO
      
      END SUBROUTINE BACK_TO_ATM_ROUGH

c **************************************************************************
      SUBROUTINE BACK_TO_WATER_ROUGH
      INTEGER  I1, J1, J2, NWv, NAv
      REAL (KND) HDU(Nvs*NNa), HDW(Nvs*NNw)
c --------------------------------------------------------------------------

      NWv = NNw*Nvs ;  NAv = NNa*Nvs
      ALLOCATE( Tr_gau(NWv), Hel(Nvs*NLSA) )
c     ... Contribution of transmitted from atmosphere to ocean
      DO Js = 1, NSZA
cc        ... Tr_gau is intensity of radiation transmitted to the 
cc            ocean at all Gauss angles and in the nadir direction
c         DO I = 1,NNw   ! Scalar case only !!
c            Tr_gau(I) = SUM( UDB_A%Taw_GG(I,1:NNa,Mh)
c     $           *LAS_A%SOL_BOT(1:NNa*Nvs,Js)*GSL_A%AG(:) )
c         ENDDO
c         Tr_gau(NNw) = LAS_A%SOL_BOT(NNa*Nvs,Js)*UDB_A%Ref_wd(1)**2
c     $        *UDB_A%Eaw_G(NNa)/Pie/2.d0
cc        ... Trans is interpolated at the line-of-sight angles
cc            in the ocean transmitted radiation
c         CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau,
c     $        NLSA, GSL_W%Muv, Trans(Js,:) )
c     =======> Vector case 1 <=======
         DO K = 1,Nvs
            HDU(K:NAv:Nvs) = LAS_A%SOL_BOT(K:NAv:Nvs,Js)*GSL_A%AG(:)
         ENDDO  ! over Stokes components (K)
c        ... Stokes vector of radiation transmitted to the
         DO I = 1,NWv
            Tr_gau(I) = SUM( UDB_A%Taw_GG(I,1:NAv,Mh)*HDU )
         ENDDO
cc     ... Test
c         J1 = Nvs*(NNw-1)+1 ; J2 = Nvs*NNw
c         Tr_gau(J1:J2) =
c     $        LAS_A%SOL_BOT(Nvs*(NNa-1)+1:NNa*Nvs,Js)*UDB_A%Ref_wd(1)**2
c     $        *UDB_A%Eaw_G(NNa)/Pie/2.d0

c     ... Interpolation of the transmitted from atmosphere to the ocean 
c            radiation onto line-of-sight directions
        DO K = 1,Nvs
            CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau(K:NWv:Nvs),
     $           NLSA, GSL_W%Muv, Trans(Js,K:Nvs*NLSA:Nvs) )
         ENDDO
c     =======> End Vector case 1 <=======
      ENDDO  ! over solar angles (Js)

c     ... Contribution of radiation reflected from ocean-atmosphere 
c         interface back to the ocean 
      DO Js = 1, NSZA
c         DO I = 1,NNw         ! Scalar case only !!
c            Tr_gau(I) = SUM( UDB_W%Rw_GG(I,1:NNw,Mh)
c     $           *LAS_W%SOL_TOP(NNw*Nvs+1:2*NNw*Nvs,Js)*GSL_W%AG(:) )
c         ENDDO
c         CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau, NLSA, GSL_W%Muv, Hel )
c     =======> Vector case 2 <=======
         DO K = 1,Nvs
            HDW(K:NWv:Nvs) =
     $           LAS_W%SOL_TOP(NWv+K:2*NWv:Nvs,Js)*GSL_W%AG(:)
         ENDDO  ! over Stokes components (K)
         DO I = 1,NWv 
            Tr_gau(I) = SUM( UDB_W%Rw_GG(I,1:NWv,Mh)*HDW )
         ENDDO
         DO K = 1,Nvs
            CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau(K:NWv:Nvs),
     $           NLSA, GSL_W%Muv, Hel(K:Nvs*NLSA:Nvs) ) 
         ENDDO
c     =======> End Vector case 2 <=======         
c        ... Will be multiplied by 2.d0*Pie in "st_down_sfi.f"
         Trans(Js,:) = Trans(Js,:) + Hel(:)
      ENDDO  ! over solar angle (Js)
      
cc     ... Contribution from direct solar light
cc         ------------------------------------
cc      DO Js = 1, NSZA
cc         DO I = 1,NNw         ! Scalar case only !!
cc            Tr_gau(I) = UDB_A%Taw_GS(I,Js,Mh)
cc     $           *SSB_A%EXS(AGR_A%NL,Js)*0.5d0
cc         ENDDO
cc         CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau, NLSA, GSL_W%Muv, Hel)
cc         Trans(Js,:) = Trans(Js,:) + Hel(:)
cc      ENDDO
c      WRITE(*,*) '    Vst = ', VST
c      DO I = 1, NSZA  ;  I1 = Nvs*(I-1)
c         DO K = 1,NWv
c            Tr_gau(K) = SUM( UDB_A%Taw_GS(K,I1+1:I1+Nvs,Mh)*Finc(1,:) )
c     $                 *SSB_A%EXS(AGR_A%NL,I) !*Pie *VST
c         ENDDO
c         DO K = 1,Nvs
c            CALL AKIMA_INT(NNw, GSL_W%XM, Tr_gau(K:NWv:Nvs),
c     $           NLSA, GSL_W%Muv, Hel(K:Nvs*NLSA:Nvs) ) 
c         ENDDO
c         Trans(I,:) = Trans(I,:) + Hel(:) /Pie/4.d0
c      ENDDO

      END SUBROUTINE BACK_TO_WATER_ROUGH
c ******************************************************************

      SUBROUTINE BACK_TO_WATER_FLAT
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'BACK_TO_WATER_FLAT'
      INTEGER  Ig, I1, Ih, Jv, Jsh, Ks
      REAL (KND)  F_tot(Nstr_T,Nvs,Nvs)
c     ---------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I')
      ALLOCATE( Tr_gau(NNw*Nvs), X_Ref(NNw), Hel(Nvs*NLSA) )
      X_ref(1:NNw) = GSL_W%XM(:)
      DO Js = 1, NSZA
c        ... Contribution of transmitted radiation atmosphere => ocean.
c            -----------------------------------------------------
         Jsh = Nvs*(NNw + NSTR_T)
         DO Ig = 1, NNa  ;  Ks = (Ig-1)*Nvs
c            WRITE(*,'(2x,I3,2(2x,e12.4))')
c     $           Ig, UDB_A%Fre_Tv(Ig,1,1), LAS_A%SOL_BOT(Ks+1:Ks+Nvs,Js)
c            WRITE(*,'(2x,I3,2(2x,e12.4))')
c     $           Ig, UDB_A%Fre_Rv(Ig,1,1),
c     $           LAS_W%SOL_TOP(Jsh+Ks+1:Jsh+Ks+Nvs,Js)
            DO K = 1, Nvs
c              ... transmitted to the ocean in the refraction cone
               Tr_gau(NSTR_T*Nvs+Ks+K) = UDB_A%Ref_wd(1)**2*
     $              SUM( UDB_A%Fre_Tv(Ig,K,1:Nvs)
     $              *LAS_A%SOL_BOT(Ks+1:Ks+Nvs,Js) ) 
c                  ... reflected back in the refraction cone
     $             + SUM( UDB_A%Fre_Rv(Ig,K,1:Nvs)
     $              *LAS_W%SOL_TOP(Jsh+Ks+1:Jsh+Ks+Nvs,Js) ) 
            ENDDO               ! over Stokes components (K)
         ENDDO                  ! over Gauss angles (Ig)
c      --------------
c       ... Contribution of radiation reflected from the ocean-atmosphere
c            interface back to the ocean in the total reflection zone
         Jsh = Nvs*NNw
         DO I = 1, NSTR_T ;  Ks = (I-1)*Nvs
            DO K = 1, Nvs
               Tr_gau(Ks+K) =  SUM( UDB_W%Fre_Rv(I,K,1:Nvs)
     $              *LAS_W%SOL_TOP(Jsh+Ks+1:Jsh+Ks+Nvs,Js) )
            ENDDO
         ENDDO

c     ... Linear extrapolation for line-of-sight angles which 
c            outside of Gaussian grid
         Ih = 1
         DO Jv = 1, NLSA ;  I1 = (Jv-1)*Nvs
            HH = GSL_W%Muv(Jv)
            IF(HH <= X_ref(NNw).AND.HH >= X_ref(1)) THEN
               DO K = 1, Nvs
                  CALL AKIMA_INT( NNw,X_ref,Tr_gau(K:NNw*Nvs:Nvs),
     $                 Ih, HH, Hel(I1+K) )
               ENDDO  ! over Stokes components (K)
            ELSE
               DO K = 1, Nvs
                  CALL LIN_INT( NNw, X_ref, Tr_gau(K:NNw*Nvs:Nvs),
     $                 Ih, HH, Hel(I1+K), 'L')
               ENDDO  ! over Stokes components (K)
            ENDIF
         ENDDO  ! over viewing angles (NLSA)
         Trans(Js,:) = Hel(:)/Pie/2.d0
      ENDDO  ! over solar  angles (Js)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE BACK_TO_WATER_FLAT

c **********************************************************************
c     ...  Contribution to the solution from "ST_(UP)DOWN_SFI"
      SUBROUTINE SET_RADOUT
      INTEGER Jv, Jvs
      REAL (KND) Sazm, Cazm
      J = 0
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         DO Js = 1, NSZA
            DO Jv = JV_S(Js),JV_E(Js)
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  Cazm = DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                  DO Jo = 1,N_OP
                     RADOUT(Jo,J,1) = Cazm*Sfi_res(NOA(Jo),Js,Jv)*E_W
                  ENDDO
               ENDDO
            ENDDO
         ENDDO

      CASE( 'vect_rte' )
c            WRITE(*,*) '   RADOUT FROM "post_processing" '
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  
               LL1 = (Jv-1)*Nvs 
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  Cazm = 0.5d0*DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  Sazm = 0.5d0*DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  DO Jo = 1,N_OP
                     RADOUT(Jo,J,1:2) =  
     $                    Cazm*Sfi_res(NOA(Jo),Js,LL1+1:LL1+2)
                     RADOUT(Jo,J,3:4) =  
     $                    Sazm*Sfi_res(NOA(Jo),Js,LL1+3:LL1+4)
                     
c                    ... Add second DO-solution
                     IF( Kvs==1 ) CYCLE
                     RADOUT(Jo,J,1:2) = RADOUT(Jo,J,1:2) 
     $                    - Sazm*Sfi_res(NOA(Jo),NSZA+Js,LL1+1:LL1+2)
                     RADOUT(Jo,J,3:4) = RADOUT(Jo,J,3:4) 
     $                    + Cazm*Sfi_res(NOA(Jo),NSZA+Js,LL1+3:LL1+4)
                  ENDDO  ! over instrument positions
               ENDDO  ! over azimuthal angles
            ENDDO  ! over zenith angles
         ENDDO  ! over solar angles
      END SELECT

      IF( LOG_LEVEL(1) <= N_XSO ) THEN 
         WRITE(LUNIT,*) '    Sfi_Res: ', SUM(Sfi_Res)
      ENDIF
      RETURN
      END SUBROUTINE SET_RADOUT
c **********************************************************************

      END
