!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 02 Feb 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE PS_RAD (DEP_CF, AS_C, AL_0, ALBEDO, PCC, SCA, SCR, AL0_OUT, &
     RAD_PS, WFPS, OSM, *)
  !  Called by "spher_rad.f"

  USE GRIDS,      ONLY: NSGP=>NSOL, NLAY => NLAYERS, NL_FINE
  USE NODES,      ONLY: N_NODE, PHI_NODE, SIN_NODE, MJU_NODE
  USE LOG_PARS,   ONLY: EU 
  USE SPHER_PARS, ONLY: ONE_MS_SZ
  USE ST_VARS,    ONLY: WF_MS_FLAG, DO_PC, GeoD_PS
  USE TYPES,      ONLY: KND, PCC_TYPE, OSM_TYPE, WFS_PS, FD_LOCAL

  REAL (KND)  ALBEDO, DEP_CF(3), AL0_OUT(NL_FINE)
  REAL (KND), DIMENSION (NLAY):: AS_C, AL_0, SCA, SCR, ASYM_COEFF
  REAL (KND), ALLOCATABLE :: RAD_PS(:,:,:,:)
  TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
  TYPE (PCC_type) :: PCC 
  TYPE (OSM_type) :: OSM
  
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'PS_RAD'
  INTEGER     IS, K, I, L, NSOL
  LOGICAL     FLG_WF
  REAL (KND)  A_C_1(NLAY), A_C_2(NLAY), C_G(N_NODE,N_NODE)
  REAL (KND), ALLOCATABLE :: AL_PC(:,:)
  TYPE (FD_LOCAL), ALLOCATABLE :: LocArrs(:)
  !==================================================================
  INTERFACE
     SUBROUTINE RHS_FORM (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, AL0_OUT, PCC, &
          AL_PC, SCA, SCR, LocArrs, OSM, LVG, LSG, FLG_WF, *)
       USE GRIDS,        ONLY: NLAYERS, NL_FINE
       USE NODES,        ONLY: N_NODE
       USE TYPES,       ONLY: KND, PCC_TYPE, OSM_TYPE, LOS_SOLAR_GEOM, &
                              FD_LOCAL, LOS_VIEW_GEOM
       LOGICAL     FLG_WF
       REAL (KND)  DEP_CF(3), ASYM_COEFF(NLAYERS), SSSF(NLAYERS, N_NODE), &
            AL0_OUT(NL_FINE)
       REAL (KND), DIMENSION (NLAYERS) :: AL_PC, A_C_1, A_C_2, SCA, SCR
       REAL (KND), ALLOCATABLE :: DS_SUN(:,:)
       REAL (KND)    EXM_NL, CSZA_Surf
       TYPE (PCC_type) :: PCC    
       TYPE (OSM_type) :: OSM 
       TYPE (LOS_SOLAR_GEOM) :: LSG
       TYPE (LOS_VIEW_GEOM) :: LVG
       TYPE (FD_LOCAL) :: LocArrs    
     END SUBROUTINE RHS_FORM
     SUBROUTINE SOLVE_DIFF_EQ (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, ALBEDO, &
          AL_PC, SCA, SCR, LocArrs, NRHS, RAD_PS, WFPS, C_G, OSM, FLG_WF)
       USE GRIDS,       ONLY: NLAY => NLAYERS
       USE INTERN_DIMS, ONLY: N_MOM => NLG_ALL_A
       USE NODES,       ONLY: N_NODE
       USE TYPES,       ONLY: KND, OSM_TYPE, WFS_PS, FD_LOCAL
       INTEGER  NRHS
       LOGICAL  FLG_WF
       REAL (KND), INTENT(IN)::  ALBEDO, DEP_CF(3)
       REAL (KND), DIMENSION (NLAY) :: A_C_1, A_C_2, SCA, SCR, AL_PC, &
            ASYM_COEFF
       REAL (KND)  C_G(N_NODE,N_NODE)
       REAL (KND)  RAD_PS(:,:,:)
       TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
       TYPE (OSM_type) :: OSM 
       TYPE (FD_LOCAL) :: LocArrs(NRHS)
     END SUBROUTINE SOLVE_DIFF_EQ
  END INTERFACE
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  ASYM_COEFF(:) = 2.0D0*AS_C(:)
  A_C_1(:) = 1.0D0 + AS_C(:)**2
  A_C_2(:) = (1.0D0 - AS_C(:)**2)

  NSOL = NSGP ;  ALLOCATE(LocArrs(NSOL))
  
  FLG_WF = ALLOCATED(WFPS)
  Allocate(AL_PC(NLAY,NSOL))
  DO IS = 1, NSOL
     ALLOCATE(LocArrs(IS)%SSSF(NLAY, N_NODE))
     IF(FLG_WF) ALLOCATE(LocArrs(IS)%DS_SUN(NLAY, NLAY),&
          WFPS(IS)%RES(NL_FINE, NLAY, N_NODE))
     AL_PC(:,IS) = AL_0
      CALL RHS_FORM (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, AL0_OUT, PCC, AL_PC(:,IS), &
          SCA, SCR, LocArrs(IS), OSM, GeoD_PS%LVG(1,1), GeoD_PS%LSG(1,1,IS,1), &
          FLG_WF, *880)
   END DO
  
  C_G = 0.d0
  FORALL(K = 1:N_NODE, L = 1:N_NODE, L <= K) &
       C_G(K,L) = MJU_NODE(K)*MJU_NODE(L) + &
       SIN_NODE(K)*SIN_NODE(L)*COS(PHI_NODE(K) - PHI_NODE(L))
  
  IF(DO_PC) THEN
     DO IS = 1, NSOL
        CALL SOLVE_DIFF_EQ (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, ALBEDO, &
             AL_PC(:,IS), SCA, SCR, LocArrs(IS), 1, RAD_PS(:,IS:IS,:,1), &
             WFPS, C_G, OSM, FLG_WF)
     END DO
  ELSE
     CALL SOLVE_DIFF_EQ (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, ALBEDO, &
          AL_0, SCA, SCR, LocArrs, NSOL, RAD_PS(:,:,:,1), WFPS, C_G, OSM, FLG_WF)
  END IF

  ! We need to reverse the SZA index in WFs as the radiance is reversed
  ! in course of the akima interpolation 
  IF(FLG_WF.AND.NSOL > 1) WFPS(1:NSOL) = WFPS(NSOL:1:-1)
  DO IS = 1, NSOL
     DeAllocate(LocArrs(IS)%SSSF)
     IF(FLG_WF) DeAllocate(LocArrs(IS)%DS_SUN)
  END DO
  DeAllocate(AL_PC, LocArrs)
     
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
!***************************************************************************
END SUBROUTINE PS_RAD
!***************************************************************************

