!***********************************************************************
!
!	SCIATRAN - Version 4.2 - Last modification: 16 Oct 2019
!
!	Copyright (C) - Institute of Remote Sensing,
!		        University of Bremen, Germany
!
!***********************************************************************
SUBROUTINE QP_SOLV(N_X, N_Y, YDIFF, KN, XDIFF, XA, RMS1, SQ_RTB, *)
!     ... Called  by "retrieval.f90"

  USE AER_MIE,   ONLY: T_550
  USE BRDF,      ONLY: Nwl_RLi, Use_LRC, LRC, D_LRC, ERR_RL, B_RL, Sep_IR, &
                       SC_sig
  USE DATA_RET,  ONLY: PRESET_APR_NAME
  USE INFO,      ONLY: RET_INFO, STATE
  USE LOG_PARS,  ONLY: LUNIT, LOG_LEVEL, N_SSO, DIR_OUT
  USE RTM_PARS,  ONLY: DATA_ITER, RTM_ST
  USE ST_VARS,   ONLY: RET_MODE
  USE TOOLS,     ONLY: INQ_WF, CHK_STAT, SET_PTR, Find_Idx
  USE TYPES,     ONLY: KND

  ! ------------------------------------------------------------------
  !    Initialization of GALAHAD quadratic problem solver
  ! **************************************************
  USE GALAHAD_QPB_double    ! double precision version
  REAL (KND), PARAMETER :: INFINITY = 1.0D+20
  TYPE ( QPT_PROBLEM_TYPE ) :: PRT
  TYPE ( QPB_DATA_TYPE ) :: DATA_STORE
  TYPE ( QPB_CONTROL_TYPE ) :: CONTROL        
  TYPE ( QPB_INFORM_TYPE ) :: INF
  !     ------------------------------------------------------------------

  INTEGER  N_X, N_Y, I_RI
  REAL (KND) XDIFF(N_X), YDIFF(N_Y), KN(N_Y,N_X), XA(N_X)
  ! ... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'QP_SOLV'
  LOGICAL Set_Alb
  INTEGER  I, J, K, KK, II, LL, ISTAT, N_FIT, I_SBP, &
       I_PAR, N_KER, LL0, NCLIN, Iwl, Ike
  INTEGER, PARAMETER :: MAX_ITER = 500   
  REAL (KND) GR_PAR(2,N_X), XSOL(N_X)
  REAL (KND) RMS1, SQ_RTB, Err, Ref, Rco, P_max, P_min
  REAL (KND) ER_SCA(N_Y), RMS(MAX_ITER) !, RLC(3,4)
  REAL (KND) TEK1(N_Y)
  REAL (KND), ALLOCATABLE:: RLC(:,:), X_SWL(:)
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  !  ==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  NCLIN = 0 ! Preliminary
  RMS = 0.0d0
  
  LL = 0  ; LL0 = 0
  DO K = 1, N_X
     IF(.NOT.STATE(K)%USE_FLG) CYCLE
     I_RI = STATE(K)%I_RET_INF
     SELECT CASE (RET_INFO(I_RI)%NAME)
     CASE('aer-dens')
        LL = LL + 1 ; LL0 = LL0 + 1
!        GR_PAR(1,LL) = RET_INFO(I_RI)%PAR_MIN/T_550 - 1.d0
!        GR_PAR(2,LL) = RET_INFO(I_RI)%PAR_MAX/T_550 - 1.d0 
!       ... Set bounds for AOT
        GR_PAR(1,LL) = (T_550 - 1.0d-3)/T_550 - 1.d0
        GR_PAR(2,LL) = (T_550 + 1.0d-3)/T_550 - 1.d0 
        WRITE(*,*) '   T_550 = ', T_550
        WRITE(*,*) '   Constrains for Tau (fixed tau) K, L ', K, LL
        WRITE(*,'(4x,2(2x,E12.4))') GR_PAR(1,LL), GR_PAR(2,LL)        
     CASE('albedo')
        LL = LL + 1
!        WRITE(*,*) '   No bound K, LL ', K, LL
        GR_PAR(1,LL) = -1.0d+20 ;  GR_PAR(2,LL) = +1.0d+20
!        GR_PAR(1,LL) = -1.0d-5 ;  GR_PAR(2,LL) = +1.0d-5
     END SELECT
  ENDDO  ! over parameters to be retrieved (K)

  I_RI = Find_Idx('albedo', RET_INFO%Name)
  N_KER = RET_INFO(I_RI)%N_SBP  ! Number of RL kernels
  I_PAR = RET_INFO(I_RI)%N_PARS ! Number of wavelengths for RL kernels
  ALLOCATE(  RLC(N_KER,I_PAR), X_SWL(N_KER) )
  RLC = 0.d0
  NCLIN = N_KER*(I_PAR-1)
  
! ... Boundary values for all Ross-Li coefficients at reference wavelength
!  WRITE(*,*) '   Ross-Li at reference wavelength: '
  DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
     CALL SET_PTR( RTM_PTR, RET_INFO(I_RI)%NAME,ISSP=I_PAR,IASP=I_SBP)
     RLC(I_SBP,I_Par) = RTM_PTR%CRN(1)
     GR_PAR(1,LL) = B_RL(I_SBP,I_Par,1) - RTM_PTR%CRN(1)
     GR_PAR(2,LL) = B_RL(I_SBP,I_Par,2) - RTM_PTR%CRN(1)
!        WRITE(*,'(3(2x,I3),2x,*(1x,ES12.4))') I_SBP, I_PAR, LL, &
!             RTM_PTR%APR(1), RTM_PTR%CRN(1), GR_PAR(1,LL), GR_PAR(2,LL)
  ENDDO

  WRITE(*,*) '    Ross-li at reference wavelength (previous):'
  WRITE(*,'(5x,*(1x,ES12.4))') RLC(:,I_Par)
  
  TEK1(:) = YDIFF(:)
  N_FIT = N_X  
!     General initialization of the quadratic problem solver
!     ******************************************************
  CONTROL%INFINITY = infinity
  PRT%NEW_PROBLEM_STRUCTURE = .TRUE. 
  CALL SMT_put (PRT%H%TYPE, 'DENSE', ISTAT) 
  CALL SMT_put (PRT%A%TYPE, 'DENSE', ISTAT) 
      
!     Initialization of the quadratic problem solver
!     **********************************************
  CONTROL%INFINITY = 1.0D+20
  PRT%NEW_PROBLEM_STRUCTURE = .TRUE. 

  CALL BOUND_CONSTR( XSOL )

!  Save retrieved Ross-Li coefficients at the reference wavelength only
  DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
     XDIFF(LL)  = XSOL(LL) 
  ENDDO
  
!! =====================================================================
!  CALL BOUND_CONSTR_SWL
!  WRITE(*,*) '    Compare XSOL and X_SWL'
!  DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
!     WRITE(*,'(2x,2(2x,I3),*(2x,E15.7))') &
!          I_SBP, LL, XDIFF(LL), X_SWL(I_SBP)
!     XDIFF(LL)  = X_SWL(I_SBP)
!  ENDDO
!! =====================================================================
   
  ER_SCA(:) = TEK1(:) - MATMUL(KN(:,:),XSOL(:))
  SQ_RTB = DSQRT(DOT_PRODUCT(ER_SCA(:),ER_SCA(:))/DBLE(N_Y))
  RMS1   = DSQRT( DOT_PRODUCT(TEK1(:),TEK1(:))/DBLE(N_Y) )

  IF( LOG_LEVEL(2) >= N_SSO ) THEN 
     write(LUNIT,*) ' '
!     write(LUNIT,*) &
!          '    Without linear constrains with QP algorithm'
!     write(LUNIT,'(3x,*(1x,e12.4))') (XSOL(I), I = 1, N_FIT)
!     write(LUNIT,*) ' '
     write(LUNIT,*) &
          '    RMS before and after first inversion'
     write(LUNIT,'(9x,2(2x,e13.5))') RMS1, SQ_RTB
     write(LUNIT,*) ' '
  ENDIF
  WRITE(*,*) '    Delta Ross-li coefficients at all wavelength  '
  WRITE(*,*) '         after first minimization:'
  DO I_SBP = 1,N_KER 
     WRITE(*,'(5x,I2,*(1x,ES12.4))')  I_SBP, &
          (XSOL(LL0 + I_SBP*K), K = 1,I_Par)
  ENDDO
  
  WRITE(*,*) '    Ross-li coefficients at reference wavelength (IR wavelength) '
  WRITE(*,*) '           before and after first minimization:'
  DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
     WRITE(*,'(5x,I2,*(1x,ES12.4))')  I_SBP, &
          RLC(I_SBP,I_Par), RLC(I_SBP,I_Par) + XDIFF(LL)
  ENDDO
! ******************************************************************************
!       Redefine bound values
! ******************************************************************************
!  WRITE(*,*) '  ******** After redefinition of bound values '
  Set_Alb = .FALSE.
  LL = 0  
  DO K = 1, N_X
     IF(.NOT.STATE(K)%USE_FLG) CYCLE
     I_RI = STATE(K)%I_RET_INF
     SELECT CASE (RET_INFO(I_RI)%NAME)
     CASE('aer-dens')
        LL = LL + 1 
        GR_PAR(1,LL) = RET_INFO(I_RI)%PAR_MIN/T_550 - 1.d0
        GR_PAR(2,LL) = RET_INFO(I_RI)%PAR_MAX/T_550 - 1.d0 
!                 WRITE(*,*) '   T_550 = ', T_550
!                 WRITE(*,*) '   Constrain for tau: '
!                 WRITE(*,*) GR_PAR(1,LL), GR_PAR(2,LL)
     CASE('albedo')
        IF( Set_Alb ) CYCLE
        LL = LL0 + 1  ! counter of bound constraints
!                 WRITE(*,*) ' CRN(1), Rco, Ref, GR_PAR(1,LL), GR_PAR(2,LL), Err   '
        DO Ike = 1, N_KER
           Ref = RLC(Ike,I_Par) + XDIFF(LL0 + Ike*I_PAR)
           DO Iwl = 1,I_PAR
              CALL SET_PTR( RTM_PTR, RET_INFO(I_RI)%NAME,ISSP=Iwl,IASP=Ike)
              IF( USE_LRC ) THEN
!                ... Estimation of RL coefficient using the linear regression
                 Rco = LRC(Ike,Iwl,1)*Ref + LRC(Ike,Iwl,2)
!                ... Absolute error of linear regression relationship
!                 Err = 3.0d0*ABS( D_LRC(Ike,Iwl,1)*Ref + D_LRC(Ike,Iwl,2) )
                 Err = SC_sig*ABS( D_LRC(Ike,Iwl,1)*Ref + D_LRC(Ike,Iwl,2) )
                 P_max = Rco + Err ;  P_min = Rco - Err
              ELSE
                P_max = B_RL(Ike,Iwl,2) ;  P_min = B_RL(Ike,Iwl,1) 
              ENDIF
              GR_PAR(1,LL) = P_min - RTM_PTR%CRN(1)
              GR_PAR(2,LL) = P_max - RTM_PTR%CRN(1)
!                  WRITE(*,'(3(2x,I3),2x,*(1x,ES12.4))') Ike, Iwl, LL, &
!                       RTM_PTR%CRN(1), &
!                       P_min, P_max
!!                       Rco, Ref, P_min, P_max, Err
              LL = LL + 1
           ENDDO  ! over Ross-Li coeff. at wavelengths < IR (Iwl)
        ENDDO  ! over Ross-Li coefficients (Ike)
!         WRITE(*,*) '     ------- Ref. WL ---------  '
        DO Ike = 1,N_KER  ;  LL = LL0 + Ike*I_PAR 
           Ref = RLC(Ike,I_PAR) + XDIFF(LL)
           IF( Sep_IR ) THEN
!              Err = ERR_RL(Ike,I_PAR)
              Err = 1.0d-08  !  Fixed at this step in IR channel
              IF( Ref < 0.d0 ) Err = -Err
              GR_PAR(1,LL) = -Err*Ref
              GR_PAR(2,LL) = +Err*Ref
           ELSE           
              P_max = B_RL(Ike,I_PAR,2) ;  P_min = B_RL(Ike,I_PAR,1)
              GR_PAR(1,LL) = P_min - Ref
              GR_PAR(2,LL) = P_max - Ref
           ENDIF
!                  WRITE(*,'(3(2x,I3),2x,*(1x,ES12.4))') Ike, I_PAR, LL, &
!                       Ref, GR_PAR(1,LL), GR_PAR(2,LL)
        ENDDO  ! over Ross-Li coefficients (Ike)
        Set_Alb = .TRUE.
     END SELECT
  ENDDO  ! over parameters to be retrieved (K)

  TEK1(:) = YDIFF(:) - MATMUL(KN,XDIFF)

  CALL BOUND_CONSTR( XSOL )

  WRITE(*,*) '    Ross-li coefficients at reference wavelength (IR wavelength) '
  WRITE(*,*) '           before and after second minimization:'
  DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
     WRITE(*,'(5x,I2,*(1x,ES12.4))')  I_SBP, &
          RLC(I_SBP,I_Par) + XDIFF(LL), &
          RLC(I_SBP,I_Par) + XDIFF(LL) + XSOL(LL)
  ENDDO

  XDIFF(:)  = XDIFF(:) + XSOL(:)

  ER_SCA(:) = YDIFF(:) - MATMUL(KN(:,:),XDIFF(:))
  SQ_RTB = DSQRT(DOT_PRODUCT(ER_SCA(:),ER_SCA(:))/DBLE(N_Y))

  IF( LOG_LEVEL(2) >= N_SSO ) THEN 
     write(LUNIT,*) '  Second minimization '
!     write(LUNIT,*) &
!          '    Without linear constrains with QP algorithm'
!     write(LUNIT,'(3x,11(1x,e12.4))') (XSOL(I), I = 1, N_FIT)
!     write(LUNIT,*) ' '
     write(LUNIT,*) &
          '    RMS before and after second minimization'
     write(LUNIT,'(9x,2(2x,e13.5))') RMS1, SQ_RTB
     write(LUNIT,*) ' '
  ENDIF

  
!  IF( NCLIN > 0 ) CALL LINEAR_CONSTR

  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN

CONTAINS

!********************************************************************C
  SUBROUTINE BOUND_CONSTR_SWL
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'BOUND_CONSTR_SWL'
    REAL (KND) XSOL(N_KER), KN_P(14,N_KER)
    REAL (KND) GR_P(2,N_KER), TEKP(14)
!   ------------------------------------------------------------------
    
    CALL ST_TRACK (R_NAME, 2, 'I')
    N_FIT = N_KER
    ALLOCATE( PRT%G(N_FIT), PRT%X_L(N_FIT), PRT%X_U(N_FIT), &
         PRT%X(N_FIT), PRT%Z(N_FIT), PRT%H%val(N_FIT*(N_FIT + 1)/2), &
         PRT%A%val(0), PRT%C(0), PRT%C_L(0), PRT%C_U(0), PRT%Y(0) )
    PRT%N = N_FIT ; PRT%M = 0 ; PRT%F = 0.0D0 

    DO I_SBP = 1,N_KER  ;  LL = LL0 + I_SBP*I_PAR
       GR_P(:,I_SBP) = GR_PAR(:,LL)
       WRITE(*,'(3(2x,I3),2x,*(1x,ES12.4))') I_SBP, I_PAR, LL, &
            GR_P(1,I_SBP), GR_P(2,I_SBP)
       KN_P(:,I_SBP) = KN(5:N_Y:5,LL)
    ENDDO
    TEKP(:) = TEK1(5:N_Y:5)

    PRT%X_L(:) = GR_P(1,:) ;  PRT%X_U(:) = GR_P(2,:)
    DO I = 1, N_FIT 
       DO J = 1, I 
          PRT%H%val(I*(I - 1)/2 + J) = DOT_PRODUCT(KN_P(:,I),KN_P(:,J))
       END DO
    END DO
    PRT%G(:) = -MATMUL(TEKP(:),KN_P(:,:))
    PRT%X = 0.0D0 ;  PRT%Z = 0.0D0
    CALL QPB_INITIALIZE (DATA_STORE, CONTROL, INF) 
!  control%SBLS_control%preconditioner = 2
    control%stop_c = 1.0d-11
    CALL QPB_SOLVE(PRT, DATA_STORE, CONTROL, INF) 
    CALL QPB_TERMINATE (DATA_STORE, CONTROL, INF)
    IF (INF%status.NE.0) WRITE(LUNIT,*) 'Quadratic problem solver '// &
       'without linear constraints --> INF%status = ', INF%status
    X_SWL(:) = PRT%X(:)
    DEALLOCATE( PRT%G, PRT%X_L, PRT%X_U, PRT%X, PRT%Z, PRT%A%val,  &
         PRT%H%val, PRT%C, PRT%C_L, PRT%C_U, PRT%Y)
    
    CALL ST_TRACK (R_NAME, 2, 'O')
    RETURN
  END SUBROUTINE BOUND_CONSTR_SWL


  !********************************************************************C
  SUBROUTINE BOUND_CONSTR(XSOL)
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'BOUND_CONSTR'
    REAL (KND) XSOL(N_X)
!   ------------------------------------------------------------------

    CALL ST_TRACK (R_NAME, 2, 'I')
    N_FIT = N_X
    ALLOCATE( PRT%G(N_FIT), PRT%X_L(N_FIT), PRT%X_U(N_FIT), &
         PRT%X(N_FIT), PRT%Z(N_FIT), PRT%H%val(N_FIT*(N_FIT + 1)/2), &
         PRT%A%val(0), PRT%C(0), PRT%C_L(0), PRT%C_U(0), PRT%Y(0) )
    PRT%N = N_FIT ; PRT%M = 0 ; PRT%F = 0.0D0 
    PRT%X_L(:) = GR_PAR(1,:) ;  PRT%X_U(:) = GR_PAR(2,:)
    DO I = 1, N_FIT 
       DO J = 1, I 
          PRT%H%val(I*(I - 1)/2 + J) = DOT_PRODUCT(KN(:,I),KN(:,J))
       END DO
    END DO
    PRT%G(:) = -MATMUL(TEK1(:),KN(:,:))
    PRT%X = 0.0D0 ;  PRT%Z = 0.0D0
    CALL QPB_INITIALIZE (DATA_STORE, CONTROL, INF) 
!  control%SBLS_control%preconditioner = 2
    control%stop_c = 1.0d-11
    CALL QPB_SOLVE(PRT, DATA_STORE, CONTROL, INF) 
    CALL QPB_TERMINATE (DATA_STORE, CONTROL, INF)
    IF (INF%status.NE.0) WRITE(LUNIT,*) 'Quadratic problem solver '// &
       'without linear constraints --> INF%status = ', INF%status
    XSOL(:) = PRT%X(:)
    DEALLOCATE( PRT%G, PRT%X_L, PRT%X_U, PRT%X, PRT%Z, PRT%A%val,  &
         PRT%H%val, PRT%C, PRT%C_L, PRT%C_U, PRT%Y)
    
    CALL ST_TRACK (R_NAME, 2, 'O')
    RETURN
  END SUBROUTINE BOUND_CONSTR

!********************************************************************C
  SUBROUTINE LINEAR_CONSTR
    CHARACTER (LEN = *), PARAMETER :: R_NAME = 'LINEAR_CONSTR'
    INTEGER  I, J, Ico, K, I_VIBR, Icl, Ip1, Nc1, Nsh
    REAL (KND) CL_V, CU_V, SQ_RTA, A1, A2, A3
    REAL (KND) B_MAT(N_Y,N_FIT + 4), C_NORM_PAR(N_FIT)
    REAL (KND), ALLOCATABLE:: A_GR(:,:)
    !------------------------------------------------------------------

    CALL ST_TRACK (R_NAME, 2, 'I')
    
    ALLOCATE( PRT%G(N_FIT), PRT%X_L(N_FIT), PRT%X_U(N_FIT), &
         PRT%X(N_FIT), PRT%Z(N_FIT), PRT%A%val(N_FIT*NCLIN),  &
         PRT%H%val(N_FIT*(N_FIT + 1)/2), PRT%C(NCLIN), &
         PRT%C_L(NCLIN), PRT%C_U(NCLIN), PRT%Y(NCLIN), &
         A_GR(NCLIN,N_FIT) )
    
    CALL QPB_INITIALIZE (DATA_STORE, CONTROL, INF) 
!    control%SBLS_control%preconditioner = 2
    control%stop_c = 1.0d-11
    
!!   ... QP iterative scheme
!    DO
    TEK1(:) = YDIFF(:)
!   ... RMS before retrieval
    SQ_RTA = DSQRT( DOT_PRODUCT(TEK1,TEK1)/DBLE(N_Y) )
!      ... Normalizing coefficients
!       C_NORM_PAR(:) = MAXVAL( DABS(WF_PAR(1:N_Y,:)),1 )
    C_NORM_PAR(:) = 1.0d0
!      ... Weighting function matrix formation
    B_MAT = 0.0d0
    FORALL( J = 1:N_Y ) B_MAT(J,1:N_FIT) = KN(J,:)/C_NORM_PAR(:)
!   ... Initialization of the quadratic problem solver
    PRT%N = N_FIT ; PRT%M = NCLIN ; PRT%F = 0.0d0 
!   ... Bounds for parameters
    PRT%X_L(:) = -INFINITY  ;  PRT%X_U(:) =  INFINITY
    DO I = 1, N_FIT
       PRT%X_L(I) = GR_PAR(1,I)*C_NORM_PAR(I)
       PRT%X_U(I) = GR_PAR(2,I)*C_NORM_PAR(I)
    END DO

    WRITE(*,*) '   Constrains for all parameters:'
    DO I = 1, N_FIT
       WRITE(*,'(5x,I3,*(2x,ES12.4))')   I, PRT%X_L(I), PRT%X_U(I) 
    ENDDO
!   ... Bounds for linear constrains
    PRT%C_L(:) = -INFINITY  ; PRT%C_U(:) =  INFINITY
!      ... Linear constrains
    A_GR = 0.0d0
!      Matrix and vector of linear constrains (case of linear regression)
!=========================================================================
    IF( USE_LRC ) THEN
       Icl = 1  ! counter of linear constraints
       DO Ike = 1, N_KER
          CALL SET_PTR( RTM_PTR, RET_INFO(I_RI)%NAME,ISSP=I_PAR,IASP=Ike)
          Ref = RTM_PTR%CRN(1) + XDIFF(LL0 + Ike*I_PAR)
          LL  = LL0 + Ike*I_PAR ; Nsh = LL0 + (Ike-1)*I_PAR
          DO Iwl = 1,I_PAR-1  ;  Err = ERR_RL(Ike,Iwl)
             CALL SET_PTR( RTM_PTR, RET_INFO(I_RI)%NAME,ISSP=Iwl,IASP=Ike)
             RLC(Ike,Iwl) = RTM_PTR%CRN(1)
             IF( RTM_PTR%CRN(1)<0.0d0 ) Err = -Err
             CL_V = -Err*RTM_PTR%CRN(1) ; CU_V = Err*RTM_PTR%CRN(1)
!                CL_V = -Err*RTM_PTR%APR(1) ; CU_V = Err*RTM_PTR%APR(1)
             PRT%C_L(Icl) = CL_V - (RTM_PTR%CRN(1)-LRC(Ike,Iwl,1)*Ref-LRC(Ike,Iwl,2))
             PRT%C_U(Icl) = CU_V - (RTM_PTR%CRN(1)-LRC(Ike,Iwl,1)*Ref-LRC(Ike,Iwl,2))
             WRITE(*,'(3(2x,I3),2x,*(1x,ES12.4))') Icl, Ike, Iwl, RTM_PTR%CRN(1), &
                  PRT%C_L(Icl), PRT%C_U(Icl)
             A_GR(Icl,Nsh+Iwl) = 1.0d0 !- Err
             A_GR(Icl,LL) = -LRC(Ike,Iwl,1)
             Icl = Icl + 1
          ENDDO  ! over Ross-Li coeff. at wavelengths < IR (Iwl)
          WRITE(*,*) '    -------  '
       ENDDO  ! over Ross-Li coefficients (Ike)
    ENDIF
!=========================================================================
!    PRT%C_L(:) = -INFINITY  ; PRT%C_U(:) =  INFINITY
       
    PRT%X = 0.0d0 ;  PRT%Y = 0.0d0 ;  PRT%Z = 0.0d0
    PRT%H%val = 0.0d0 ; PRT%A%val = 0.0d0 ;  PRT%C = 0.0d0
    DO I = 1, N_FIT 
       DO J = 1, I 
          PRT%H%val(I*(I - 1)/2 + J) = &
               DOT_PRODUCT(B_MAT(:,I),B_MAT(:,J))
       END DO
    END DO
    PRT%G(:) = - MATMUL(TEK1(1:N_Y),B_MAT(1:N_Y,1:N_FIT))
    FORALL( I = 1:NCLIN,J = 1:N_FIT ) &
         PRT%A%val(N_FIT*(I - 1) + J) = A_GR(I,J) 
    CALL QPB_SOLVE(PRT, DATA_STORE, CONTROL, INF) 
    IF (INF%status.NE.0) THEN
       write(*,*) '   ============================================================'
       WRITE(LUNIT,*) 'Quadratic problem solver '// &
         'with linear constraints --> INFO%status = ', INF%status
       write(*,*) '   ============================================================'
       Stop
!       PAUSE
    ENDIF
!    write(*,*) PRT%X(12),  PRT%X(13), PRT%X(13)*LRC(3,Iwl,1)
    
    XDIFF(1:N_FIT) = XDIFF(1:N_FIT) + PRT%X(1:N_FIT)/C_NORM_PAR(1:N_FIT)

!    write(*,*) '    Linear constrain for R_geo at previous iteration'
!    write(*,*) '  ------------------------------------------------------  '
!       Ico = 3
!       A2 = RLC(Ico,4)  ! R_geo at reference wavelength
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) 
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(3x,I2,*(2x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!       write(*,*) '   Linear constrain for R_geo after minimization '
!       write(*,*) '  ------------------------------------------------------  '
!       A2 = RLC(Ico,4) + XDIFF(13) ! R_geo at reference wavelength
!       II = 9
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) + XDIFF(II+Iwl)
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(5x,I2,*(1x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!! ===================================================================================
!    write(*,*) '    Linear constrain for R_vol at previous iteration'
!    write(*,*) '  ------------------------------------------------------  '
!       Ico = 2
!       A2 = RLC(Ico,4)  ! R_geo at reference wavelength
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) 
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(3x,I2,*(2x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!       write(*,*) '   Linear constrain for R_vol after minimization '
!       write(*,*) '  ------------------------------------------------------  '
!       A2 = RLC(Ico,4) + XDIFF(9) ! R_geo at reference wavelength
!       II = 5
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) + XDIFF(II+Iwl)
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(5x,I2,*(1x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!! ===================================================================================
!    write(*,*) '    Linear constrain for R_iso at previous iteration'
!    write(*,*) '  ------------------------------------------------------  '
!       Ico = 1
!       A2 = RLC(Ico,4)  ! R_geo at reference wavelength
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) 
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(3x,I2,*(2x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!       write(*,*) '   Linear constrain for R_iso after minimization '
!       write(*,*) '  ------------------------------------------------------  '
!       A2 = RLC(Ico,4) + XDIFF(5) ! R_geo at reference wavelength
!       II = 1
!       DO Iwl = 1,3 
!          A1 = RLC(Ico,Iwl) + XDIFF(II+Iwl)
!          A3 = A2*LRC(Ico,Iwl,1) + LRC(Ico,Iwl,2)
!          write(*,'(5x,I2,*(1x,ES12.4))')  Iwl, A1, A3, (A1 - A3)/A1*100.
!       ENDDO
!       write(*,*) '  ------------------------------------------------------  '
!
    WRITE(*,*) '    Ross-li at reference wavelength '
    WRITE(*,'(5x,*(1x,ES12.4))') &
         RLC(1,4) + XDIFF(5), RLC(2,4) + XDIFF(9), RLC(3,4) + XDIFF(13)

       
    ER_SCA(:) = TEK1(:) - MATMUL(B_MAT(:,1:N_FIT),PRT%X(1:N_FIT))
    SQ_RTB = DSQRT( DOT_PRODUCT(ER_SCA(:),ER_SCA(:))/DBLE(N_Y) )


    WRITE(LUNIT,'(9x,A,2(1x,e12.5))') &
         'Root mean square before and after:', SQ_RTA, SQ_RTB

    write(*,*) '  ------------------------------------------------------  '

    IF (LOG_LEVEL(2) >= N_SSO) THEN 
       write(LUNIT,*) ' '
       write(LUNIT,*) &
            '        Amplitude for all parameters :'
       WRITE(LUNIT,'(8x,30(1x,e12.4))') (XDIFF(I), I = 1,N_X)
       WRITE(LUNIT,*) ' '
       WRITE(LUNIT,'(9x,A,1x,e12.5)') &
            'Root mean square for this mode:', SQ_RTB
       WRITE(LUNIT,*) ' '
    ENDIF

    CALL QPB_TERMINATE (DATA_STORE, CONTROL, INF)
    DEALLOCATE( PRT%G, PRT%X_L, PRT%X_U, PRT%X, PRT%Z, PRT%A%val, &
         PRT%H%val, PRT%C, PRT%C_L, PRT%C_U, PRT%Y)

       
    CALL ST_TRACK (R_NAME, 2, 'O')
    RETURN
  END SUBROUTINE LINEAR_CONSTR

!********************************************************************C
END SUBROUTINE QP_SOLV

! ********************************************************************
