!***************************************************************************
!
!       SCIATRAN - Version 4.6 - Last modification: 29 Apr 2022
!
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_BRDF(*)
  !  Called by "gt_fileread.f"
  
  USE BRDF,     ONLY: WL_BRF, Nwl_BRF, Par_mix, Exact_BRF, DO_OC_COMP, VS_OC_COMP, &
                      N_OC_COMP, LWC_Foam, Bubble_Dim, Foam_WL, RefF_i0, RefF_name, &
                      Ref_type, RefW_r0, RefW_i0, Alf0, RefW_name, Lin_rel, Rep_cosine,&
                      Bet_name, Leav_WL, Alph_name, OUT_BRDF, Path_for_BRDF, Abs_pw, &
                      Abs_chl, Spha_veg, Spha_soi, Cfr_v, HS_par, Ex_Type, Cr_ang, &
                      Ro_Li_name, N_Sph_WL, Sph_OUT, Sph_WL, INP_BRDF, RG_name, AB_DB, &
                      Alph_1, Alph_2, Beta_1, Beta_2, IOP_name, Iref_name, Ref_ip,&
                      Use_LRC, LRC, D_LRC, Err_RLC_name, B_RLC_name, Sep_IR, SC_sig  &
                      , RefW_r, MP_gp, MP_bub, MP_bri, Pond_type, Ref_to_Abs, SCM_name, &
                      SCM_WL, Yang_DB, Tau_sl, L_yellow, Yellow_390, RDB_dir, RDB_ref, &
                      RDB_ang
  USE LOG_PARS, ONLY: IU => SurUNIT, EU, USE_SHAD, Work_Dir
  USE PRO_SAIL
  USE ST_INPUT, ONLY: PWAC_NAME
  USE ST_VARS,  ONLY: SURF_TYPE, BRDF_RANGE, SPECULAR, Vec_Sca, Pie, RTM_MODE
  USE TOOLS,    ONLY: SET_PATH, Join, CUT_STR, Read_Input, Split_String
  USE TYPES,    ONLY: DYN_CHAR
  USE WATER,    ONLY: T_ocean, Salin, Chlorophyll, V_small, V_large, VSF_pure_model, &
                      W_speed, W_azim, SS_PDF, Par_PDF, UP_CROSS 
  
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAM = 'READ_BRDF'
  CHARACTER (LEN=:), ALLOCATABLE :: Mix_NAM, STR_LOC, Whit, Gli, Leav, STR_INP, &
       LRC_name, Del_LRC_name
  INTEGER   I, NN
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
  !==================================================================

  Use_LRC = .FALSE.
  
  STR_INP = 'Surface type'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  SURF_TYPE = CUT_STR(READ_ARR(1)%S)

!  write(*,*) '  SURF_TYPE  ', SURF_TYPE
!  pause '  in read_brdf  '
  
  STR_INP = 'Output of spherical albedo'
  CALL Read_Input (STR_INP, READ_ARR, *880, REARRANGE=.TRUE.)
  N_Sph_WL = SIZE(READ_ARR)
  IF( N_Sph_WL > 0 ) THEN
     ALLOCATE( Sph_WL(N_Sph_WL), Sph_OUT(N_Sph_WL) )
     DO I = 1, N_Sph_WL
        READ(READ_ARR(I)%S,*,ERR=101) Sph_WL(I)
     END DO
  ELSE
     ALLOCATE( Sph_OUT(1) )
  ENDIF

  STR_INP = 'Exact BRDF for diffuse radiation'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR = 101) Exact_BRF

  SELECT CASE(SURF_TYPE)
  CASE('prosail')
     STR_INP = 'Leaf inclination distribution function'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) TypeLidf, LIDFa, LIDFb
     
     STR_INP = 'Other parameters:'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) LAI, hspot, Cab, Car, Cbrown, Cw, Cm, Stc,  Psoil

  CASE('mRPV_facet')
     STR_INP = 'Filename (including path) containing model parameters'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Mix_NAM = SET_PATH(READ_ARR(1)%S)

     ALLOCATE( Lin_rel(1) )
     STR_INP = 'The slope variance'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Lin_rel(1)

     STR_INP = 'Use shadowing factor?'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     USE_SHAD = CUT_STR(READ_ARR(1)%S)
     SELECT CASE(USE_SHAD)
     CASE('yes','no')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, USE_SHAD, *880)
     END SELECT

     CALL READ_MIX_REFL(*220,*221)

  CASE('rff')
     STR_INP = 'Filename containing BRDF'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     INP_BRDF = SET_PATH(READ_ARR(1)%S)
     STR_INP = 'Wavelength grid'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
     Nwl_BRF = SIZE(READ_ARR)
     ALLOCATE( WL_BRF(Nwl_BRF) )
     DO I = 1, Nwl_BRF
        READ(READ_ARR(I)%S,*,ERR=101) WL_BRF(I)
     END DO
     DEALLOCATE( Sph_OUT, Sph_WL )
     N_Sph_WL = Nwl_BRF
     ALLOCATE( Sph_WL(N_Sph_WL), Sph_OUT(N_Sph_WL) )
     Sph_WL(:) = WL_BRF(:)
!!      Account for Fresnel reflection from surface
!     Ref_type = 'const'
!     RefW_r0 = 1.34d0
!     RefW_i0 = 0.0d0
!     SPECULAR = .TRUE.
     
  CASE('user')
     STR_INP = 'Number of wavelength for BRDF'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Nwl_BRF

  CASE('Ross-Li')
     STR_INP = 'Coefficients of Ross-Li BRDF model'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Ro_Li_name = SET_PATH(READ_ARR(1)%S)
     IF( SIZE(READ_ARR)>1 ) THEN
        ERR_RLC_name = SET_PATH(READ_ARR(2)%S)
        B_RLC_name = SET_PATH(READ_ARR(3)%S)
     ENDIF
     
     STR_INP = 'Hot spot parameter'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) HS_par
     HS_par = HS_par/180.d0*Pie

     STR_INP = 'Extrapolation type of Ross-Li model'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Ex_Type = CUT_STR(STR_ARR(1)%S)
     READ(STR_ARR(2)%S,*,ERR=101) Cr_ang
     SELECT CASE(Ex_Type)
     CASE('cos','const','zero','off')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, Ex_Type, *880)
     END SELECT
     IF( Cr_ang < 70.d0 .OR. Cr_ang >= 90.d0 ) GOTO 219

     STR_INP = 'Use linear regression coefficients'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Use_LRC
     IF( Use_LRC ) THEN
        LRC_name = SET_PATH(READ_ARR(2)%S)
        Del_LRC_name = SET_PATH(READ_ARR(3)%S)
        CALL READ_LRC(*880)
     ENDIF
     IF( RTM_MODE=='ret' ) THEN
        STR_INP = 'Use IR chanel to retrieve Ross-Li coefficients only'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) Sep_IR, SC_sig
        !        WRITE(*,*) Sep_IR, SC_sig
        !        PAUSE
     ENDIF
  CASE('Ross-Li-snow')
     STR_INP = 'Coefficients of Ross-Li-snow BRDF model'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Ro_Li_name = SET_PATH(READ_ARR(1)%S)

     STR_INP = 'Hot spot parameter of Ross-Li kernel'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) HS_par
     HS_par = HS_par/180.d0*Pie

     STR_INP = 'Extrapolation type of Ross-Li kernels'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Ex_Type = CUT_STR(STR_ARR(1)%S)
     READ(STR_ARR(2)%S,*,ERR=101) Cr_ang
     SELECT CASE(Ex_Type)
     CASE('cos','const','zero','off')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, Ex_Type, *880)
     END SELECT
     IF( Cr_ang < 70.d0 .OR. Cr_ang >= 90.d0 ) GOTO 219

  CASE('snow_artm')
          STR_INP = 'Ice particles optical parameters file name'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     IOP_name = SET_PATH(READ_ARR(1)%S)
          STR_INP = 'Ice refractive index file name'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Iref_name = SET_PATH(READ_ARR(1)%S)
          STR_INP = 'Effective radius of ice particles'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Ref_ip
     Ref_ip = Ref_ip*1000.0d0 ! Transform [mkm] => [nm]
     
  CASE('white_si')
     STR_INP = 'Reflectance of semi-infinite non-absorbing medium file name'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     RDB_dir = SET_PATH(STR_ARR(1)%S)
     RDB_ref = SET_PATH(STR_ARR(2)%S)
     RDB_ang = SET_PATH(STR_ARR(3)%S)
     CALL READ_R0_DB(*880)
          STR_INP = 'Refractive index of ice (file name)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Iref_name = SET_PATH(READ_ARR(1)%S)
          STR_INP = 'Mean chord length of a grain'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Ref_ip
     Ref_ip = Ref_ip*1000.0d0 ! Transform [mkm] => [nm]
          STR_INP = 'Optical thickness of snow layer'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Tau_sl
          STR_INP = 'Yellow substance absorption'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     READ(STR_ARR(1)%S,*,ERR=101) L_yellow
     IF( L_yellow ) THEN
        READ(STR_ARR(2)%S,*,ERR=101) Yellow_390
        Yellow_390 = Yellow_390*1.d-9 ! m^-1 ==> nm^-1
     ENDIF
! -----------------------------------------------------------------
  CASE('snow_fasmar')
!     Ref_type = 'wl_dep'
     STR_INP = 'Refractive index of pure ice (snow_fasmar)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Iref_name = SET_PATH(READ_ARR(1)%S)

     STR_INP = 'Path to the Yang database'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Yang_DB  = SET_PATH(STR_ARR(1)%S)
     
     STR_INP = 'Scattering matrix file name and wavelength'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     SCM_name  = Yang_DB//SET_PATH(STR_ARR(1)%S)
!     WRITE(*,*) '  Path to scattering matrix ', SCM_name
!     PAUSE '   Pause in read_brdf.f90 '

     READ(STR_ARR(2)%S,*,ERR=101) SCM_WL
     STR_INP = 'Coefficients of snow_fasmar model'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Ro_Li_name = SET_PATH(READ_ARR(1)%S)
          STR_INP = 'Effective radius of ice crystals'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Ref_ip
     Ref_ip = Ref_ip*1000.0d0 ! Transform [mkm] => [nm]
!     WRITE(*,*) TRIM( SCM_name )
!     WRITE(*,*) '  SCM_WL =  ', SCM_WL
!     PAUSE  '   In "read_brdf.f90" '

  CASE('melt_pond')
     SPECULAR = .TRUE.
     Ref_type = 'wl_dep'
     ALLOCATE( MP_gp(2), MP_bub(3), MP_bri(3) )
        STR_INP = 'Geometrical parameters (melt pond)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) (MP_gp(I), I = 1,2)
     MP_gp(:) =  MP_gp(:)/100.d0  ! [cm] --> [m]

     STR_INP = 'Type of input parameters of under-pond ice'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Pond_type = READ_ARR(1)%S
     SELECT CASE(Pond_type)
     CASE('tsc','imp')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, Pond_type, *880)
     END SELECT

     SELECT CASE(Pond_type)
     CASE('imp')
        STR_INP = 'Air bubbles in ice'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) (MP_bub(I), I = 1,3)
        MP_bub(1) = MP_bub(1)/100.d0
        MP_bub(2) = MP_bub(2)*1.0d-6  !  [micron] --> [m]
        STR_INP = 'Brine inclusions in ice'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) (MP_bri(I), I = 1,3)
        MP_bri(1) = MP_bri(1)/100.d0
        MP_bri(2) = MP_bri(2)*1.0d-6  !  [micron] --> [m]
     CASE('tsc')
        STR_INP = 'Transport scattering coefficient of under-pond ice'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) MP_bub(1)        
     END SELECT
        STR_INP = 'Refractive index of pure ice (file name)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Iref_name = SET_PATH(READ_ARR(1)%S)
     
        STR_INP = 'Refractive index of pure water (file name)'
     CALL Read_Input (STR_INP, READ_ARR, *880)        
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     READ(STR_ARR(1)%S,*,ERR=101) Ref_to_Abs 
     RefW_name  = SET_PATH(STR_ARR(2)%S)
        STR_INP = 'Pure water absorption and scattering coefficients'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     PWAC_NAME = SET_PATH(READ_ARR(1)%S)

  CASE('svk')
     STR_INP = 'Use A&B surface reflection database'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) AB_DB       
         
     IF( AB_DB ) THEN
        STR_INP = 'Filename containing A/B coefficients'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        Spha_veg = SET_PATH(READ_ARR(1)%S)
        Spha_soi = SET_PATH(READ_ARR(1)%S)
     ELSE
        STR_INP = 'Spherical albedo of vegetation and soil'
        CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
        Spha_veg = SET_PATH(READ_ARR(1)%S)
        Spha_soi = SET_PATH(READ_ARR(2)%S)
     ENDIF

     STR_INP = 'Vegetation fraction'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Cfr_v       

     ALLOCATE( Alph_1(2), Alph_2(2), Beta_1(2), Beta_2(2) )
     STR_INP = 'Slope parameters'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) (Alph_1(I),I=1,2), (Alph_2(I),I=1,2)
     STR_INP = 'Intercept parameters'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) (Beta_1(I),I=1,2), (Beta_2(I),I=1,2)

     STR_INP = 'Hot spot parameter'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) HS_par
     HS_par = HS_par/180.d0*Pie
     STR_INP = 'Extrapolation type of svk model'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Ex_Type = CUT_STR(STR_ARR(1)%S)
     READ(STR_ARR(2)%S,*,ERR=101) Cr_ang
     SELECT CASE(Ex_Type)
     CASE('cos','const','zero','off')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, Ex_Type, *880)
     END SELECT
     IF( Cr_ang < 70.d0 .OR. Cr_ang >= 90.d0 ) GOTO 219

  CASE DEFAULT
     STR_INP = 'BRDF spectral range'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     BRDF_RANGE = CUT_STR(READ_ARR(1)%S)
  END SELECT

  ! Input parameters for surface type = 'ocean'
  ! --------------------------------------------
  DO_OC_COMP = .FALSE.
  IF( SURF_TYPE == 'ocean' ) THEN
     STR_INP = 'Specular reflection'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) SPECULAR

     STR_INP = 'Transmission matrix water->air and air->water'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
     RG_name(1) = SET_PATH(READ_ARR(1)%S)
     RG_name(2) = SET_PATH(READ_ARR(2)%S)

     STR_INP = 'Whitecaps     Glint     Water leaving'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
     Whit = CUT_STR(READ_ARR(1)%S) ;  Gli = CUT_STR(READ_ARR(2)%S)
     Leav = CUT_STR(READ_ARR(3)%S)

     DO I = 1, 3
        STR_LOC = CUT_STR(READ_ARR(I)%S)
        SELECT CASE (STR_LOC)
        CASE('scalar')
           DO_OC_COMP(I) = .TRUE. ; VS_OC_COMP(I) = 's'
        CASE('vector')
           SELECT CASE (I)
           CASE(1)
              GOTO 216
           CASE (2)
           CASE (3)
              GOTO 217
           END SELECT
           DO_OC_COMP(I) = .TRUE. ; VS_OC_COMP(I) = 'v'
        CASE('off')
           DO_OC_COMP(I) = .FALSE.; VS_OC_COMP(I) = 'n'
        CASE DEFAULT
           CALL NOT_FOUND (R_NAM, STR_INP, STR_LOC, *880)
        END SELECT
     END DO
     
     IF( .NOT.ANY(DO_OC_COMP) ) GOTO 218

     ! Check that in the case of scalar RTE all oceanic
     ! component are used as scalar as well.
     IF( Vec_Sca == 'scal_rte' ) THEN
        DO I = 1,N_OC_COMP
           IF( VS_OC_COMP(I) == 'v' ) THEN
              CALL WRITE_MESSAGE(R_NAM, 'WARNING', 'All components of ocean '&
                   //'reflectance must be included without polarization,')
              WRITE(EU,*)'     they were automatically switched on "s" !'
              EXIT
           ENDIF
        ENDDO
        VS_OC_COMP(1:N_OC_COMP) = 's'
     ENDIF

     ! Input of seawater general properties
     ! ------------------------------------
     STR_INP = 'Seawater general properties'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     STR_LOC = Join(STR_ARR, 1, 5)
     READ(STR_LOC,*,ERR=101) T_ocean, Salin, Chlorophyll, V_small, V_large
     VSF_pure_model = CUT_STR(STR_ARR(6)%S)
     READ(STR_ARR(7)%S,*,ERR=101) W_speed
     READ(STR_ARR(8)%S,*,ERR=101) W_azim

     SELECT CASE( TRIM(VSF_pure_model) )
     CASE('Morel','Shifrin','Kopelevich','Buiteveld')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, 'Pure seawater scattering function',VSF_pure_model,*880)
     END SELECT

     ! Input parameters for foam reflection (whitecaps)
     ! ------------------------------------------------
     STR_INP = 'Liquid fraction'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) LWC_Foam

     STR_INP = 'Diameter of bubbles'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Bubble_Dim

     ! CALL READ_FIELD ('Foam refractive index input type',IU,*880)
     ! READ(STR_READ,*,ERR=101) Foam_WL
     Foam_WL = 'const'
     SELECT CASE (Foam_WL)
     CASE ('const') 
        RefF_i0 = 0.9243E-09
     CASE('wl_dep')
        STR_INP = 'Imaginary part of refractive index file name'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        RefF_name = SET_PATH(READ_ARR(1)%S)
     CASE DEFAULT
        GOTO 215
     END SELECT
     
     ! Input parameters for Fresnel reflection
     ! ---------------------------------------
     STR_INP = 'Refractive index input type (BRDF)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Ref_type = CUT_STR(READ_ARR(1)%S)
     
     SELECT CASE ( Ref_type )
     CASE ('const') 
        STR_INP = 'Refractive index of water (BRDF)'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) RefW_r0, RefW_i0
     CASE('wl_dep')
        STR_INP = 'Refractive index file name (BRDF)'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        RefW_name = SET_PATH(READ_ARR(1)%S)
     CASE DEFAULT
        GOTO 215
     END SELECT
     
     STR_INP = 'Surface slope PDF'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     SS_PDF = CUT_STR(READ_ARR(1)%S)
     SELECT CASE(SS_PDF)
     CASE('Gauss')
        ALLOCATE( Lin_rel(2) )
        STR_INP = 'Mean square slope'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) Lin_rel(1), Lin_rel(2)
     CASE('Gram-Charlier')
        STR_INP = 'Peakedness and skewness coefficients'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        Par_PDF = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( Par_PDF )
        CASE('Cox_Munk','Breon_Henriot')
        CASE DEFAULT
           CALL NOT_FOUND (R_NAM, STR_INP, Par_PDF, *880)
        END SELECT
        
        STR_INP = 'Upwind and crosswind slopes'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        UP_CROSS = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( UP_CROSS )
        CASE('ratio')
           NN = 3
        CASE('coeff')
           NN = 6
        CASE DEFAULT
           CALL NOT_FOUND (R_NAM, STR_INP, UP_CROSS, *880)
        END SELECT
        ALLOCATE( Lin_rel(NN) )
        STR_INP = 'Mean square slope'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) (Lin_rel(I), I=1,NN)
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, SS_PDF, *880)
     END SELECT
     
     STR_INP = 'Replace cosine'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Rep_cosine
     
     STR_INP = 'Shadowing effects?'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     USE_SHAD = CUT_STR(READ_ARR(1)%S)
     SELECT CASE( USE_SHAD )
     CASE('yes','no')
     CASE DEFAULT
        CALL NOT_FOUND (R_NAM, STR_INP, USE_SHAD, *880)
     END SELECT
  ENDIF  ! SURF_TYPE = 'ocean'
  
  ! Input parameters for water leaving reflection
  ! ---------------------------------------------
  STR_INP = 'Angular scattering coefficient filename'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  Bet_name = SET_PATH(READ_ARR(1)%S)

  STR_INP = 'Absorption coefficients input type'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  Leav_WL = CUT_STR(READ_ARR(1)%S)
  SELECT CASE ( TRIM(Leav_WL) )
  CASE ('const') 
     STR_INP = 'Absorption coefficients'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S, *, ERR = 101) Alf0, Abs_pw, Abs_chl
  CASE('wl_dep')
     STR_INP = 'Absorption coefficients file name'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Alph_name = SET_PATH(READ_ARR(1)%S)
  CASE DEFAULT
     GOTO 222
  END SELECT
  
  ! Output control
  ! --------------
  STR_INP = 'BRDF output control'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ (READ_ARR(1)%S,*,ERR=101) OUT_BRDF
  IF( OUT_BRDF ) THEN
     Path_for_BRDF = SET_PATH(READ_ARR(2)%S)
  ENDIF
  
  RETURN
  
101 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP
  GOTO 881
  
215 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'Refractive index input type '&
         // 'must be "const" or "wl_dep"')
  GOTO 881
  
216 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'Whitecaps reflection '&
         // 'is available in the scalar form only')
  GOTO 881
  
217 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'Water leaving '&
         // 'reflection is available in the scalar form only')
  GOTO 881
  
218 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'At least one '&
         // 'reflectance component should be switch on ')
  WRITE(EU,*) '      for the surface type "ocean".'
  WRITE(EU,*) '      The option "alb_const" or "alb_spec" should be used alternatively.'
  GOTO 881
  
219 CALL WRITE_MESSAGE(R_NAM, 'ERROR', ' ')
  WRITE(EU,*) '      Maximal angle for the extrapolation'
  WRITE(EU,*) '      of kernel-based BRF'
  WRITE(EU,*) '      should be in the range 70 - 90 deg.'
  WRITE(EU,*) '      Check "Extrapolation type" in "control_brdf.inp"'
  GOTO 881
  
220 CALL WRITE_MESSAGE(R_NAM, 'ERROR', ' ')
  WRITE(EU,*) '      Error reading of the input file'
  WRITE(EU,*) '      ',TRIM(Mix_NAM)
  GOTO 881
  
221 CALL WRITE_MESSAGE(R_NAM, 'ERROR', ' ')
  WRITE(EU,*) '      The input file'
  WRITE(EU,*) '      ',TRIM(Mix_NAM)
  WRITE(EU,*) '      does not contain information!'
  GOTO 881
  ! End error information from this routine
222 CALL WRITE_MESSAGE(R_NAM, 'ERROR', 'Absorption coefficients input type '&
         // 'must be "const" or "wl_dep"')
  GOTO 881
  
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAM)
  
881 CONTINUE
  
  RETURN 1
  
CONTAINS
!**************************************************************************
!**************************************************************************
  SUBROUTINE READ_R0_DB(*)
    USE BRDF,      ONLY: Nrso, Nrvi, Nraz, Sol_r0, Vie_r0, Azm_r0, R0_db
    USE LOG_PARS,  ONLY: C_UNIT
    USE ST_VARS,   ONLY: RTD => RAD_TO_DEG
!   ----------------------------------------------------------------------     
    CHARACTER (LEN=*), PARAMETER :: R_NAME='READ_R0_DB'
    INTEGER I, J, K, Ndb, Is1, Is2, Iv1, Iv2, Ia1, Ia2, Ex1, Ex2, &
         Nes, Nev, Nea, Iss1, Iss2, Ivv1, Ivv2
    INTEGER, ALLOCATABLE:: Idx_r0(:)
    LOGICAL  DO_CHECK
    REAL (KND) TMP, Dps, Dpv, Dpa, Ups, Upv, Upa
    REAL (KND), ALLOCATABLE:: Sox(:), Vix(:), Azx(:), Ang1(:), Ang2(:),  &
         Ang3(:), R0_int(:,:,:)
!   ======================================================================
!   ... Reading of database (reflectance of semi-infinite non-absorbing medium)
!    WRITE(*,*)  '  =====> Reading ', RDB_dir//RDB_ref
    OPEN(C_UNIT, FILE = RDB_dir//RDB_ref, STATUS='old')
    CALL SKIP_HEADER (C_UNIT)           
    READ(C_UNIT,*) Nrso, Nrvi, Nraz
    Ndb = Nrso*Nrvi*Nraz
    ALLOCATE(Sol_r0(Nrso), Vie_r0(Nrvi), Azm_r0(Nraz), R0_db(Ndb), Idx_r0(Ndb), &
         Ang1(Ndb), Ang2(Ndb), Ang3(Ndb) )
    READ(C_UNIT,*,ERR=322) TMP, (R0_db(I), I = 1,Ndb )
    CLOSE(C_UNIT)
!   ... Reading os all angles
!    WRITE(*,*)  '  =====> Reading ', RDB_dir//RDB_ang
    OPEN(C_UNIT, FILE = RDB_dir//RDB_ang, STATUS='old')
    CALL SKIP_HEADER (C_UNIT)
    DO K = 1, Ndb
       READ(C_UNIT,*,ERR=323) Idx_r0(K), J, Ang1(K), Ang2(K), Ang3(K)
    ENDDO
    CLOSE(C_UNIT)

!   ... Extract azimuthal angles
    Azm_r0(:) = Ang3(1:Nraz)          ;  Azm_r0(:) = Azm_r0(:)/RTD
!   ... Extract viewing angles
    Vie_r0(:) = Ang2(1:Nraz*Nrvi:Nraz)
    Vie_r0(:) = COS( Vie_r0(:)/RTD )
!   ... Extract solar angles
    Sol_r0(:) = Ang1(1:Ndb:Nraz*Nrvi) ;  Sol_r0(:) = COS( Sol_r0(:)/RTD )  

    DO_CHECK = .FALSE.
!    DO_CHECK = .TRUE.
    IF( DO_CHECK ) THEN
       Nes = 1  ;  Nev = 21 ; Nea = 2
       ALLOCATE( Sox(Nes), Vix(Nev), Azx(Nea), R0_int(Nes,Nev,Nea) )
       Sox(1) = COS( 32.5d0/RTD )
       Vix(:) = COS( (Ang2(1:Nraz*Nrvi:Nraz) + 0.5d0)/RTD )
!       Sox(1) = 32.5d0
!       Vix(:) = Ang2(1:Nraz*Nrvi:Nraz) + 0.5d0
       Azx(1) = 0.0d0/RTD
       Azx(2) = 180.0d0/RTD
!       CALL LIN_SEARCH_NEW( Sox(1), DPs, Is1, Is2, Nrso, Sol_r0 )
!       CALL LIN_SEARCH_NEW( Vix(1), DPv, Iv1, Iv2, Nrvi, Vie_r0 )
!       CALL LIN_SEARCH_NEW( Azx(1), DPa, Ia1, Ia2, Nraz, Azm_r0 )

       DO I = 1, Nes
          CALL LIN_SEARCH_NEW( Sox(I), UPs, Is1, Is2, Nrso, Sol_r0 )
          Dps = 1.d0 - Ups
!          Iss1 = Nraz*Nrvi*(Is1 - 1) ;  Iss2 = Nraz*Nrvi*(Is2 - 1)
          Iss1 = Nraz*Nrvi*(Is2 - 1) ;  Iss2 = Nraz*Nrvi*(Is1 - 1)
          DO J = 1,Nev
             CALL LIN_SEARCH_NEW( Vix(J), UPv, Iv1, Iv2, Nrvi, Vie_r0 )
             Dpv = 1.d0 - Upv
!             write(*,'(A, f10.5)')     '     Vix ', Vix(J)
!             write(*,'(A,*(1x,f10.5))') '  Vie_r0 ', Vie_r0(Iv1), Vie_r0(Iv2)
!             write(*,'(A,*(1x,f10.5))') '    Angl ', &
!                  ACOS(Vie_r0(Iv2))*RTD, ACOS(Vie_r0(Iv1))*RTD
!             Dpv = 1.d0 - Dpv
!             Upv = 1.d0 - Dpv
!             Ivv1 = Nraz*(Iv1-1) ;  Ivv2 = Nraz*(Iv2-1) 
             Ivv1 = Nraz*(Iv2-1) ;  Ivv2 = Nraz*(Iv1-1) 
             DO K = 1, Nea
                CALL LIN_SEARCH_NEW( Azx(K), DPa, Ia1, Ia2, Nraz, Azm_r0 )
                Upa = 1.d0 - Dpa
                R0_int(I,J,K) = R0_db(Iss1+Ivv1+Ia1)*Ups*Upv*Upa &
                     + R0_db(Iss2+Ivv1+Ia1)*Dps*Upv*Upa &
                     + R0_db(Iss2+Ivv1+Ia2)*Dps*Upv*Dpa &
                     + R0_db(Iss2+Ivv2+Ia1)*Dps*Dpv*Upa &
                     + R0_db(Iss2+Ivv2+Ia2)*Dps*Dpv*Dpa &
                     + R0_db(Iss1+Ivv1+Ia2)*Ups*Upv*Dpa &
                     + R0_db(Iss1+Ivv2+Ia1)*Ups*Dpv*Upa &
                     + R0_db(Iss1+Ivv2+Ia2)*Ups*Dpv*Dpa
!                WRITE(*,*) Vix(1), R0_int(1,1,1)
!                WRITE(*,*) Vix(2), R0_int(1,2,1)
             ENDDO  ! over azimuth angles (K)
          ENDDO  ! over zenith angles (J)
       ENDDO  ! over incident angles (I)
       WRITE(*,*) '   Test results "interpolation" '

       OPEN(C_UNIT, FILE = './DATA_OUT/'//'test_interp.dat')
       DO I = 1, Nev
          WRITE(C_UNIT,*) ACOS(Vix(I))*RTD, R0_int(1,I,1), R0_int(1,I,2)
!          WRITE(C_UNIT,*) Vix(I), R0_int(1,I,1), R0_int(1,I,2)
       ENDDO
       CLOSE(C_UNIT)    
       STOP '   Stop in "read_r0_db.f90" '
    ENDIF  ! DO_CHECK

    RETURN
    
322 CALL WRITE_MESSAGE (R_NAME, 'ERROR','Error reading data file!')
    WRITE(EU,*) '    File name : ', RDB_dir//RDB_ref
    RETURN 1

323 CALL WRITE_MESSAGE (R_NAME, 'ERROR','Error reading data file!')
    WRITE(EU,*) '    File name : ', RDB_dir//RDB_ang
    RETURN 1

  END SUBROUTINE READ_R0_DB
  
!**************************************************************************
  SUBROUTINE READ_LRC(*)
    USE LOG_PARS,  ONLY: C_UNIT
    CHARACTER (LEN=*), PARAMETER :: R_NAME='READ_LRC'
    INTEGER Inw, I, K
    REAL (KND) DUM
    
    CALL ST_TRACK (R_NAME, 1, 'I')

!    OPEN(C_UNIT, FILE ='../data/surface/Ross-Li_LRC.dat', STATUS='old')
    OPEN(C_UNIT, FILE = TRIM(LRC_name), STATUS='old')
    CALL SKIP_HEADER (C_UNIT)           
    ! ... Number of wavelength in the file
    READ(C_UNIT,*) Inw
    ALLOCATE( LRC(3,Inw,2) )
    ! ... Reading coefficients
    DO I = 1,Inw
       READ(C_UNIT,*,ERR=222) DUM, (LRC(K,I,1),K=1,3), (LRC(K,I,2),K=1,3)
    ENDDO
    CLOSE(C_UNIT) 
!
!    OPEN(C_UNIT, FILE ='../data/surface/Ross-Li_Del-LRC.dat', STATUS='old')
    OPEN(C_UNIT, FILE = TRIM(Del_LRC_name), STATUS='old')
    CALL SKIP_HEADER (C_UNIT)           
    READ(C_UNIT,*) Inw ;  ALLOCATE( D_LRC(3,Inw,2) )
    ! ... Reading coefficients
    DO I = 1,Inw
       READ(C_UNIT,*,ERR=223) DUM, (D_LRC(K,I,1),K=1,3), (D_LRC(K,I,2),K=1,3)
    ENDDO
    CLOSE(C_UNIT) 

    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
    
222 CALL WRITE_MESSAGE (R_NAME, 'ERROR','Error reading data file!')
    WRITE(EU,*) '    File name : ', TRIM(LRC_name)
    RETURN 1

223 CALL WRITE_MESSAGE (R_NAME, 'ERROR','Error reading data file!')
    WRITE(EU,*) '    File name : ', TRIM(Del_LRC_name)
    RETURN 1
  
  END SUBROUTINE READ_LRC
  
!**************************************************************************

  SUBROUTINE READ_MIX_REFL(*,*)
    USE BRDF,      ONLY: Par_mix, RefW_r, RefW_i
    USE LOG_PARS,  ONLY: C_UNIT
    USE TYPE_VARS, ONLY: KND
    ! Local parameters      
    INTEGER I, K, N
    REAL (KND) TMP
    CHARACTER (LEN=100) TXT
    !------------------------------------------------------------------

    OPEN(C_UNIT, file=TRIM(Mix_NAM), STATUS='OLD')
    CALL SKIP_HEADER (C_UNIT)
    N = 0
    DO
       READ(C_UNIT,*,END=29,ERR=28) TMP
       N = N + 1
    END DO
28  RETURN 1
29  IF( N==0 ) RETURN 2
    CLOSE( C_UNIT )
    
    Nwl_BRF = N
    WRITE(*,*) '    Number of actual wavelength: ', Nwl_BRF
    ALLOCATE( WL_BRF(Nwl_BRF), Par_mix(Nwl_BRF,4), RefW_r(Nwl_BRF), RefW_i(Nwl_BRF) )

    OPEN(C_UNIT, file=TRIM(Mix_NAM), STATUS='OLD')
    CALL SKIP_HEADER (C_UNIT)
    DO I = 1,Nwl_BRF
       READ(C_UNIT,*) WL_BRF(I), (Par_mix(I,K),K=1,4), RefW_r(I), RefW_i(I)
       WRITE(*,'(1x,F8.3,6(2x,F6.3))') WL_BRF(I),(Par_mix(I,K),K=1,4), RefW_r(I), RefW_i(I)
    ENDDO
    CLOSE(C_UNIT)
      
  END SUBROUTINE READ_MIX_REFL
!**************************************************************************
END SUBROUTINE READ_BRDF
!**************************************************************************
