!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 28 Mar 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_GEOM_PARS(Z_BOA, Z_TOA, *)
  ! Call in "gt_iface.f"

  USE CLOUDS_SS,  ONLY: DO_SSRCA
  USE GRIDS,      ONLY: NLAYERS_INP, ZZZ_INP, NDEP_INP, DEP_INP, ALT_MARK, DEP_MARK
  USE INFO,       ONLY: PAR_INFO
  USE OutInf,     ONLY: DO_LOGFILE, LUNIT, EU, DIR_OUT, C_UNIT, L_S, L_E, L_U, &
                        LOG_LEVEL, LUF
  USE LOS_PARS,   ONLY: LS_VA, LS_AA, LS_SA
  USE RAMAN,      ONLY: RRS_ADJ
  USE REFRACT_VARS, ONLY: H_NOREF, COS_NOREF, H_NOREF_SOL, COS_NOREF_SOL, WL_REF
  USE RTM_OUTPUT, ONLY: N_OP
  USE ST_VARS,    ONLY: DO_SATELLITE, RTD => RAD_TO_DEG, INT_OUT, PIE, &
                        DO_REFRAC, DO_PC, RTM_TYPE, ARB_OL, WF_FLAG, SDVI, &
                        N_RHS, RTM_CORE, DO_RAMAN, SOLVER, RTM_MODE, ANG_INT,&
                        SCATT_MODE, Kvs, OWF_FLAG, ERad => EARTH_RADIUS, R_GH, &
                        GROUND_HEIGHT, NS_wf, RAD_FIELD, SOL_ANG, LSA_MODE, &
                        NSTR => NSTRMS
  USE SPHER_PARS, ONLY: SET_OUT_ALT, ANG_OUT, ANG_FOV, FOV_FLAG, N_FOV_ANG, &
                        ALTITUDE, N_LOS_OUT, N_SHIFT, N_AZ_ANG, ANG_FOV_S, &
                        NLA => N_LOS_ANG, NSA => N_SOL_ANG, N_OBS_ALT, &
                        ANG_OUT_S, LOS_DEF_FLAG, ANG_SEL_FLAG, &
                        Z_F, LOS_ANG, ALT_OUT, ANG_ORIG, FOV_ANG, &
                        DELTA_TH, N_AZ_VAR, JV_S, JV_E, JS_E, FOV_WEIGHTS, &
                        NFOV_WNG, NFOV_CORE, FOV_FULL, FOV_SHP
  USE TOOLS,      ONLY: INQ_WF, SPLIT_STRING, Read_Input, Cut_Str, WriteOut, &
                        FIND_IDX
  USE TYPES,      ONLY: KND, DYN_CHAR
  USE WATER,      ONLY: UWT, NSTR_T, Ref_ind, Obs_POSITION, INS_POSI, INS_VIEW, INEL_ADJ
  
  REAL (KND), INTENT(IN) :: Z_TOA, Z_BOA
  ! Local variables:
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_GEOM_PARS'
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, GeomPos
  INTEGER   I, J, K, L, II, IDX_LOS, IDX_SOL, IDX_PHI, IFOV, N_GAU
  INTEGER, ALLOCATABLE :: IdxOut(:)
  LOGICAL   INV_ALT, ChangeAngSel
  LOGICAL, ALLOCATABLE :: Equal_view(:,:), Flg(:)
  REAL (KND), ALLOCATABLE :: AZ_ANG(:), ANG_TMP(:,:), AG_SOL(:), GAU_TES(:), &
       AG_TES(:), XM_T(:), AG_T(:), Z_TMP(:), ALT_TMP(:), CVA(:), CSZA(:), AltInp(:)
  REAL (KND)  DS, C2_TRU, TH_TMP, EPS, Del_ang, S1, MaxVA, MinVA, MinCVA, Angs(3)
  PARAMETER   (EPS = 1.0d-12)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:)
  !==================================================================
  Interface
     Subroutine TextOrFile (StrInp, Arr, *)
       Use Types,  Only: Knd
       CHARACTER (LEN=*), PARAMETER :: R_NAME = 'TextOrFile'
       Character(Len =:), Allocatable ::   StrInp
       Real (Knd), Allocatable :: Arr(:)
     End Subroutine TextOrFile
  End Interface

  CALL ST_TRACK (R_NAME, 3, 'I')
  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here
 
  DO K = L_S, L_E
     Call WriteOut (L_U(K),'',LUF(K))
     Call WriteOut (L_U(K),'  ******************************************',LUF(K))
     Call WriteOut (L_U(K),'  ***  Geometry settings: control_geom.inp  ',LUF(K))
     Call WriteOut (L_U(K),'  ******************************************',LUF(K))
  END DO

  GeomPos = "inst"

  !*************************************************************************
  !************************ Viewing geometry *******************************
  !*************************************************************************
  !
  Refraction: Block 
    IF(RTM_TYPE == 'pp_scat') EXIT Refraction
    IF(LSA_MODE == 'CDI_PP') EXIT Refraction
    STR_INP = 'Refractive geometry'
    CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.True.)
    READ(READ_ARR(1)%S, *, ERR = 102)  DO_REFRAC
    IF(.NOT.DO_REFRAC) EXIT Refraction
    READ(READ_ARR(2)%S, *, ERR = 102)  WL_REF
    H_NOREF_SOL = 35.0D0 ;  COS_NOREF_SOL = 0.34202014D0
    H_NOREF     = 35.0D0 ;      COS_NOREF = 0.34202014D0
    DO II = 3, SIZE(READ_ARR)
       CALL SPLIT_STRING(READ_ARR(II)%S, STR_ARR, '=')
       SELECT CASE (STR_ARR(1)%S)
       CASE('HLS')
          READ(STR_ARR(2)%S, *, ERR = 102) H_NOREF 
       CASE('HSB')
          READ(STR_ARR(2)%S, *, ERR = 102) H_NOREF_SOL
       CASE('ALS')
          READ(STR_ARR(2)%S, *, ERR = 102)  COS_NOREF
          COS_NOREF = COS(COS_NOREF/RTD)
       CASE('ASB')
          READ(STR_ARR(2)%S, *, ERR = 102)  COS_NOREF_SOL
          COS_NOREF_SOL = COS(COS_NOREF_SOL/RTD)
       CASE DEFAULT
          GOTO 102
       END SELECT
    END DO
  END Block Refraction

  STR_INP = 'Earth radius'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR = 101) ERad
  R_GH = GROUND_HEIGHT + ERad

  ReadSZA: Block
    IF(RTM_MODE == 'spher_alb') THEN
       NSA = NSTR 
       IF( NSA < 3 ) GOTO 210
       ALLOCATE (AG_SOL(NSA), CSZA(NSA)) 
       CALL GAULEG(0.d0, 1.d0, CSZA, AG_SOL, NSA)
       DEALLOCATE (AG_SOL) 
       Exit ReadSZA
    End IF
    If(RTM_TYPE == "transmission") Exit ReadSZA
    STR_INP = 'Solar zenith angles'
    Call TextOrFile (STR_INP, CSZA, *880)
    NSA = SIZE(CSZA) 
    ! Check maximum solar zenith angle for pp_scat
    IF(RTM_TYPE == 'pp_scat' .AND. MAXVAL(CSZA) >= 90.d0) GOTO 220
    CSZA(:) = Cos(CSZA/RTD)
  End Block ReadSZA
  
  ! Reduced Input for "spher_alb" mode
  SELECT CASE (RTM_MODE)
  CASE ('flux','vod','spher_alb')
     INS_POSI = 'toa'; ANG_SEL_FLAG = 'all'; LOS_DEF_FLAG = 'n.a.'
     INS_VIEW = 'dn';  DO_SATELLITE = .True.; N_AZ_ANG = 1 ; NLA = 1
     SOLVER = 'Solv_SFI' ;  N_RHS = Kvs*NSA 
     SDVI   = -300 ! not used

     Allocate(N_OBS_ALT(1), Obs_POSITION(1))
     N_OBS_ALT = 0 ; Obs_POSITION =''
     
     N_LOS_OUT = NSA
     ! RAD_FIELD =.True. - calculation fluxes in spherical geometry
     If(RTM_TYPE == "spher_alb" .And. .Not.RAD_FIELD) N_LOS_OUT = 1
     If(RTM_TYPE == "vod" .And. .Not.DO_PC) N_LOS_OUT = 1
     
     Allocate (ANG_OUT(3,NSA), SOL_ANG(NSA))
     SOL_ANG(:) = ACOS(CSZA(:))*RTD
     ANG_OUT(1,:) = CSZA(:) ;  ANG_OUT(2:3,:) = 0.d0
     CALL ST_TRACK (R_NAME, 3, 'O')
     RETURN  ! No more input is needed for these modes
  END SELECT

  STR_INP = 'Position of the instrument'
  CALL Read_Input (STR_INP, READ_ARR, *880, ULC = "L")
  INS_POSI = CUT_STR(READ_ARR(1)%S)

  IF( TRIM(SCATT_MODE)=='dir_ref' .AND. INS_POSI /='toa' ) GOTO 218

  ! The parameter SET_OUT_ALT will be set to .TRUE. if either "user" or "sagl"
  ! mode is active
  SET_OUT_ALT = .FALSE.
  ! The parameter ARB_OL will be set to .TRUE. if input parameter 
  ! INS_POSI is set to 'user'
  ARB_OL = .FALSE. 
  SELECT CASE(INS_POSI)
  CASE('toa','boa')
     ALLOCATE( Obs_POSITION(1) )
     Obs_POSITION(1) = 'Atmos'
  CASE('user')
     SET_OUT_ALT = .TRUE. ;  ARB_OL = .TRUE.
  CASE('sagl')
     SET_OUT_ALT = .TRUE.
  CASE('too','boo')
     ALLOCATE( Obs_POSITION(1) )
     Obs_POSITION(1) = 'Ocean'
     IF(.NOT.UWT) GOTO 211
  CASE DEFAULT
     CALL NOT_FOUND(R_NAME, 'Position of the instrument', INS_POSI, *880)
  END SELECT

  IF( SET_OUT_ALT ) THEN
     STR_INP = 'User-defined output altitude'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     N_OP = SIZE(READ_ARR)
     ALLOCATE( ALTITUDE(N_OP), Obs_POSITION(N_OP), ALT_OUT(N_OP) )
     DO I = 1, N_OP
        READ(READ_ARR(I)%S,*,ERR=101) ALTITUDE(I)
     END DO
     ! N_OP can be larger than 1, if "RTM Mode" parameter is set to  
     ! "int" (intensity). See additional checking in the "gt_settings_check.f"
     IF( N_OP > 1 ) THEN
        IF( TRIM(RTM_CORE) == 'CDI')        GOTO 110
        IF( TRIM(RTM_TYPE) == 'spher_scat'.OR. RTM_TYPE == 'transmission') GOTO 111
        IF( ANY( ALTITUDE(:) > Z_TOA )  )   GOTO 112
        S1 = ALTITUDE(1)
        IF( ALTITUDE(1) == 0.d0 ) S1 = ALTITUDE(N_OP)
        DO I = 1,N_OP
           IF( ALTITUDE(I) == 0.d0 ) CYCLE
           IF( S1/ALTITUDE(I) < 0.d0 ) GOTO 113
        ENDDO
     END IF

     IF( .NOT.UWT ) THEN
        DO I = 1,N_OP
           IF(ALTITUDE(I).LT.Z_BOA) GOTO 207
        ENDDO
     ENDIF
     ! For options "user" and "sagl" the parameter Obs_POSITION is defined here
     DO I = 1, N_OP
        IF(ALTITUDE(I) >= 0.d0) Obs_POSITION(I) = 'Atmos'
        IF(ALTITUDE(I)  < 0.d0) Obs_POSITION(I) = 'Ocean'
     ENDDO
     ! It is done to avoid problem with altitude equal to 0
     ! if other altitudes are negative (instrument in water)
     IF( ANY(ALTITUDE(:) < 0.d0)  ) Obs_POSITION(:) = 'Ocean'

  ELSE
     N_OP = 1
     ALLOCATE( ALTITUDE(N_OP), ALT_OUT(N_OP) )
     SELECT CASE(INS_POSI)
     CASE('toa')       
        ALTITUDE(1) = Z_TOA
     CASE('boa')
        ALTITUDE(1) = Z_BOA
     END SELECT
  ENDIF

  ! Initialization of array N_OBS_ALT(:) in "gt_layer_setup.f"
  ALLOCATE( N_OBS_ALT(N_OP) )

  STR_INP = 'Type of LOS definition'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR)
  LOS_DEF_FLAG = CUT_STR(STR_ARR(1)%S)
  IF(LOS_DEF_FLAG == 'au') READ(STR_ARR(2)%S, *, ERR = 101) NLA
  SELECT CASE (LOS_DEF_FLAG)
  CASE ('va','ca','th')
     STR_INP = 'Angle selection mode'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     ANG_SEL_FLAG = CUT_STR(READ_ARR(1)%S)
     IF(ANG_SEL_FLAG.NE.'one'.AND.ANG_SEL_FLAG.NE.'all') &
          Call Not_Found(R_NAME, STR_INP, ANG_SEL_FLAG, *880)        
  CASE ('ga')
     ANG_SEL_FLAG = 'all'
     IF(RTM_CORE /= 'DOM') GOTO 201
  CASE ('au')
     ANG_SEL_FLAG = 'all'
     IF(RTM_TYPE == 'transmission') GOTO 200
  CASE DEFAULT 
     Call Not_Found(R_NAME, STR_INP, LOS_DEF_FLAG, *880)        
  END SELECT
  Select Case (INS_POSI)
  Case ("toa", "boa","user")
     STR_INP = 'Position to define geometry'
     CALL Read_Input (STR_INP, READ_ARR, *880, ULC = "L")
     GeomPos = CUT_STR(READ_ARR(1)%S)
     If(GeomPos == INS_POSI) GeomPos = "inst"
     Select Case (GeomPos)
     Case ("inst", "toa", "boa/th")
     Case Default
        Call Not_Found(R_NAME, STR_INP, GeomPos, *880)
     End Select
  End Select

  STR_INP = 'Viewing angles'
  SELECT CASE (LOS_DEF_FLAG)
  CASE ('th')
     IF( RTM_TYPE /= 'spher_scat'.AND. RTM_TYPE /= 'transmission' ) GOTO 204
     Call TextOrFile (STR_INP, AltInp, *880)
     NLA = SIZE(AltInp) ; ALLOCATE(CVA(NLA)) !A
     Allocate(Los_Ang, Source = AltInp)
     IF(MaxVal(AltInp) >= ALTITUDE(1) .OR. MaxVal(AltInp)  >= Z_TOA) GoTo 203
     Select Case (GeomPos)
     Case ("inst")
        CVA(:) = SQRT(1.D0 - ((AltInp(:) + ERad)/(ALTITUDE(1) + ERad))**2)
        AltInp(:) = ALTITUDE(1)
     Case ("toa")
        CVA(:) = SQRT(1.D0 - ((AltInp(:) + ERad)/(Z_TOA + ERad))**2)
        AltInp(:) = Z_TOA
     Case ("boa/th")
        CVA(:) = 0.0D0
     End Select
     
  CASE ('va','ca')
     Call TextOrFile (STR_INP, CVA, *880)
     NLA = SIZE(CVA) ; ALLOCATE(AltInp(NLA)) !A
     ! Cosines into angles
     IF( LOS_DEF_FLAG == 'va' ) THEN
        CVA(:) = Cos(CVA(:)/RTD)
     ENDIF
     CVA(:) = Sign( Max(Abs(CVA(:)),1.0D-10),CVA(:) )
     Select Case (GeomPos)
     Case ("inst")
        AltInp(:) = ALTITUDE(1)
     Case ("toa")
        AltInp(:) = Z_TOA
     Case ("boa/th")
        AltInp(:) = Z_BOA
     End Select
     
  CASE ('au')
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange =.TRUE.)
     READ(READ_ARR(1)%S, *, ERR = 101) MinVA
     READ(READ_ARR(2)%S, *, ERR = 101) MaxVA
     READ(READ_ARR(3)%S, *, ERR = 101) NLA
     ALLOCATE(CVA(NLA), AltInp(NLA)) !A
     CVA(1) = MinVA
     DO II = 1, NLA - 1 
        CVA(II + 1) = CVA(1) + (MaxVA - CVA(1))/DBLE(NLA - 1)*II
     END DO
     CVA(:) = Cos(CVA(:)/RTD)
     CVA(:) = Sign(Max(Abs(CVA(:)),1.0D-10),CVA(:))

  CASE ('ga')  ! Output angle grid is Gaussian angle-grid
     SELECT CASE( TRIM(Obs_POSITION(1)) )
     CASE('Atmos')
        ! Number of line-of-sight angles is equal to the number
        ! of Gaussian nodes in the atmosphere
        NLA = NSTR
     CASE('Ocean')
        ! Number of line-of-sight angles is equal to the number
        ! of Gaussian nodes in the ocean
        NLA = NSTR + NSTR_T
!     CASE('Both')
!        STOP '   Not implemented: "read_geom_pars.f90"'
     END SELECT
     ALLOCATE(CVA(NLA), Gau_TES(NSTR), AG_TES(NSTR))
     N_GAU = NSTR
     IF( UWT.OR.INT_OUT ) THEN
        N_GAU = NSTR - 1 ; Gau_TES(NSTR) = 1.d0 ; AG_TES(NSTR) = 0.d0
     ENDIF
     CALL GAULEG(0.d0,1.d0, Gau_TES(1:N_GAU), AG_TES(1:N_GAU),N_GAU)

     SELECT CASE( TRIM(Obs_POSITION(1)) )
     CASE('Atmos')
        CVA(:) = Gau_TES(:)
     CASE('Ocean')
        ALLOCATE( XM_T(NSTR_T), AG_T(NSTR_T) )            
        C2_tru = SQRT( Ref_ind**2 - 1.d0 )/Ref_ind
        CALL GAULEG(0.d0, 1.d0, XM_T, AG_T, NSTR_T)
        ! Transform to the interval [0,C2_tru]
        CVA(1:NSTR_T) = XM_T(:)*C2_tru
        CVA(NSTR_T+1:NLA) = SQRT(Gau_TES(:)**2 + Ref_ind**2 - 1.d0)/Ref_ind
        DEALLOCATE(XM_T, AG_T)            
     END SELECT
     DEALLOCATE( Gau_TES, AG_TES )
     CVA(:) = Sign(Max(Abs(CVA(:)),1.0D-10),CVA(:))
  END SELECT
  
  IF(RTM_TYPE == 'transmission') THEN
     ANG_SEL_FLAG = 'one' ;  NSA = NLA ;  N_AZ_ANG  = NLA
     ALLOCATE(CSZA(NLA), AZ_ANG(NLA))
     CSZA(:) = CVA(:)
  ELSE
     STR_INP = 'Azimuth angles'
     Call TextOrFile (STR_INP, AZ_ANG, *880)
     N_AZ_ANG = SIZE(AZ_ANG) 
     WHERE( AZ_ANG(:) < 0.d0 ) AZ_ANG(:) = AZ_ANG(:) + 3.6D+2
     AZ_ANG(:) = AZ_ANG(:)/RTD
  ENDIF

  If(RTM_CORE == 'ASYMP') Then
     If(NLA > 1) GOTO 205
     IF(NSA > 1)  GOTO 202
     IF(N_AZ_ANG > 1) GOTO 206
  End If

  SELECT CASE (RTM_MODE)
  CASE DEFAULT
!     WRITE(*,*) '   read_geom_pars: ANG_SEL_FLAG  ', ANG_SEL_FLAG
!     WRITE(*,*) NSA,  NLA,  N_AZ_ANG
     SELECT CASE (ANG_SEL_FLAG)
     CASE ('all')
        N_LOS_OUT = NSA*NLA*N_AZ_ANG
     CASE ('one')
        N_LOS_OUT = NSA
        IF(NSA.NE.NLA.OR.NSA.NE.N_AZ_ANG.OR.NLA.NE.N_AZ_ANG)  &
             STOP 'Please set NSA = NLA = N_AZ_ANG'
     END SELECT
  END SELECT

  ALLOCATE(ANG_OUT(3,N_LOS_OUT), IdxOut(N_LOS_OUT)) ! A

  FOV_FLAG = .FALSE. ;  N_FOV_ANG = 1
  FOV: Block
    Select Case (RTM_CORE)
    Case ('ASYMP')
       EXIT FOV
    Case ('DOM')
       IF(RTM_TYPE /='spher_scat' ) EXIT FOV
    End Select
    
    STR_INP = 'Field of view integration'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    READ(READ_ARR(1)%S, *, ERR = 101) FOV_FLAG
    IF( FOV_FLAG.AND.RTM_MODE =='raman' ) THEN
       CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
       WRITE(EU,'(A)') '   The "Field of view integration" mode can not'
       WRITE(EU,'(A)') '      be used in combination with Raman scattering.'
       WRITE(EU,'(A)') '   (Check "control_geom.inp" file)'
       RETURN 1
    ENDIF
          
    IF(.NOT.FOV_FLAG) EXIT FOV 
    STR_INP = 'Field of view size'
    CALL Read_Input (STR_INP, READ_ARR, *880)
    READ(READ_ARR(1)%S, *, ERR = 101) FOV_ANG
    FOV_ANG = FOV_ANG/RTD
    STR_INP = 'FOV integration parameters'
    CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange =.True., ULC="U")
    FOV_SHP = CUT_STR(READ_ARR(1)%S)
    READ(READ_ARR(2)%S, *, ERR = 101)  NFOV_CORE 
    IF(NFOV_CORE < 3 .OR. MODULO(NFOV_CORE, 2) == 0) GOTO 217 
    SELECT CASE(FOV_SHP)
    CASE('BC')
       N_FOV_ANG = NFOV_CORE ;  NFOV_WNG = 0
    CASE('GS')
       READ(READ_ARR(3)%S, *, ERR = 101)  FOV_FULL
       FOV_FULL = FOV_FULL/RTD
       IF(FOV_FULL <= FOV_ANG) GOTO 223
       READ(STR_ARR(4)%S, *, ERR = 101)  NFOV_WNG
       N_FOV_ANG = NFOV_CORE + 2*NFOV_WNG
    CASE DEFAULT
       Call Not_Found(R_NAME, STR_INP, FOV_SHP, *880)
    END SELECT
  END Block FOV

  Z_F = MAX(Z_F, Z_BOA)
           
  !**************************************************************************
  IF(DO_LOGFILE) THEN
     write(LUNIT,*) ' ******************************************'
     write(LUNIT,*) ' *** End of geometry settings section'
     write(LUNIT,*) ' ******************************************'
     write(LUNIT,*)
  END IF
  !**************************************************************************

  ViewDirection: Block
    !    WRITE(*,*) '   CVA  ', CVA
    ! Assigning a direction to angles aroud 90 degs
    Allocate(Flg(Size(CVA)))
    Flg = .True.
    Where(Abs(CVA) < 1.0D-6) Flg = .False.

    ! Check if we only have directions aroud 90 degs
    If(Count(Flg) == 0) Then
       Flg = .True.
       IF(INS_POSI == 'boa') Then
          CVA = - Abs(CVA)
       Else
          CVA = Abs(CVA)
       End If
    End If
       
   If(All(Pack(CVA,Flg) == 0)) Exit ViewDirection
   If(All(Pack(CVA,Flg) > 0)) Then
       INS_VIEW = 'dn';  DO_SATELLITE = .True.
       Where(.Not.Flg) CVA = Abs(CVA)
       IF(INS_POSI == 'boa'.AND.RTM_TYPE == 'spher_scat') GOTO 219
       Exit ViewDirection
    End If
    If(All(Pack(CVA,Flg) < 0)) Then
       INS_VIEW = 'up';  DO_SATELLITE = .False.
       Where(.Not.Flg) CVA = - Abs(CVA)
       IF( INS_POSI == 'toa') GOTO 216            
       Exit ViewDirection
    End If
    Stop "A combination of upward and downward viewing directions is not yet supported"
  End Block ViewDirection
  DeAllocate(Flg)
!  PAUSE
  IF(ANG_SEL_FLAG.EQ.'all') THEN
     II = 0 
     DO IDX_SOL = 1, NSA ;  DO IDX_LOS = 1, NLA ;  DO IDX_PHI = 1, N_AZ_ANG
        II = II + 1
        ANG_OUT(:,II) = [CSZA(IDX_SOL), CVA(IDX_LOS), AZ_ANG(IDX_PHI)]
        IdxOut(II) = IDX_LOS
     END DO ;  END DO ;  END DO
  ELSE
     DO IDX_LOS = 1, N_LOS_OUT
        ANG_OUT(:,IDX_LOS) = [CSZA(IDX_LOS), CVA(IDX_LOS), AZ_ANG(IDX_LOS)]
        IdxOut(IDX_LOS) = IDX_LOS
     END DO
  END IF

  ChangeAngSel = .False.
  ! Check if we need to re-calculate angles to the instrument position
  If(Any(Real(AltInp(:)) /= Real(Altitude(1)))) Then
     ChangeAngSel = .True. ! ANG_SEL_FLAG = 'all' is no more valid and is to be changed
     Do II = 1, N_LOS_OUT        
        IF(Real(AltInp(IdxOut(II))) == Real(Altitude(1))) Cycle
        If(ANG_OUT(2,II)*(Altitude(1) - AltInp(IdxOut(II))) >= 0) Then
           ! We need to reverse the direction of LOS to go back to the instrument
           Angs(:) = ANG_OUT(:,II)
           Angs(2) = - Angs(2); Angs(3) = Angs(3) + Pie
           If(Angs(3) > 2.*Pie) Angs(3) = Angs(3) - 2.*Pie
           CALL ANGLES (Angs, ERad, AltInp(IdxOut(II)), ALTITUDE(1))
           Angs(2) = - Angs(2); Angs(3) = Angs(3) + Pie
           If(Angs(3) > 2.*Pie) Angs(3) = Angs(3) - 2.*Pie
           ANG_OUT(:,II) = Angs(:)
        Else
           ! The angle definition point is behind the instrument, thus we do need
           ! to reverse the direction of LOS
           Angs(:) = ANG_OUT(:,II)
           CALL ANGLES (ANG_OUT(:,II), ERad, AltInp(IdxOut(II)), ALTITUDE(1))
        End If
     End Do
  End If
  
  Allocate (ANG_ORIG, Source = ANG_OUT)   
  ALLOCATE(ANG_FOV(3,N_LOS_OUT,N_FOV_ANG), FOV_WEIGHTS(N_FOV_ANG)) ! A
  CALL FOV_SETUP (ANG_OUT, ANG_FOV)
  
  II = FIND_IDX('TH', PAR_INFO%NAME)
  IF(PAR_INFO(II)%WF_FLG) THEN
     ALLOCATE(ANG_TMP(3,N_LOS_OUT), ANG_OUT_S(3,N_LOS_OUT), & ! A
          ANG_FOV_S(3,N_LOS_OUT,N_FOV_ANG)) ! A
     ANG_TMP = ANG_OUT
     DO II = 1, N_LOS_OUT
        TH_TMP = (ALTITUDE(1) + ERad)*SQRT(1.0D0 - ANG_OUT(2,II)**2)
        TH_TMP = TH_TMP + DELTA_TH
        ANG_TMP(2,II) = SQRT(1.0D0 - (TH_TMP/(ALTITUDE(1) + ERad))**2)
     END DO

     CALL FOV_SETUP (ANG_TMP, ANG_FOV_S)
     ANG_OUT_S(:,:) = ANG_FOV_S(:,:,N_FOV_ANG/2 + 1)
  END IF

  OutputAltitude: Block
    IF( SET_OUT_ALT ) THEN
       ! If an output altitude above TOA than only
       ! single output altitude is allowed (N_OP = 1)
       IF( ALTITUDE(1).GT.Z_TOA ) THEN
          MinCVA = Sqrt(1.0D0 - ((ERad + Z_TOA)/(ERad + Altitude(1)))**2)
          IF(Any(ANG_FOV(2,:,:) < MinCVA)) GOTO 209

          ChangeAngSel = .True. ; ALT_OUT(1) = Z_TOA ;  INS_POSI = 'toa'
          ARB_OL = .FALSE. ; SET_OUT_ALT     = .FALSE. 

          ! Re-calculate angles to TOA
          DO IDX_LOS = 1, N_LOS_OUT
             DO I = 1, N_FOV_ANG
                CALL ANGLES (ANG_FOV(:,IDX_LOS,I), ERad, ALTITUDE(1), Z_TOA)
             END DO
          END DO
          ANG_OUT(:,:) = ANG_FOV(:,:,N_FOV_ANG/2 + 1)

          IF(INQ_WF('th','RIN')) THEN
             DO IDX_LOS = 1, N_LOS_OUT
                DO I = 1, N_FOV_ANG
                   CALL ANGLES (ANG_FOV_S(:,IDX_LOS,I), ERad, ALTITUDE(1), Z_TOA)
                END DO
             END DO
             ANG_OUT_S(:,:) = ANG_FOV_S(:,:,N_FOV_ANG/2 + 1)
          End If
          Exit OutputAltitude
       END IF
    END IF
    ALT_OUT(:) = ALTITUDE(:)
  End Block OutputAltitude
    
  If(ChangeAngSel) Then
     ANG_SEL_FLAG = 'one' ; NSA = N_LOS_OUT ; NLA = N_LOS_OUT ; N_AZ_ANG = N_LOS_OUT
     Allocate(SOL_ANG(NSA))
     SOL_ANG(:) = ACOS(ANG_OUT(1,:))*RTD
  Else
     Allocate(SOL_ANG(NSA))
     SOL_ANG(:) = ACOS(CSZA(:))*RTD
  End If

  ! Parameter ARB_OL is .TRUE. if
  ! a - input parameter INS_POSI was set to 'user'
  ! b - all required output altitudes are below TOA.
  ! In this case all requred output levels will be 
  ! implemented in the main altitude grid ("gt_layer_setup.f")
  IF( ARB_OL .AND. TRIM(RTM_CORE)=='DOM' ) THEN
     ALLOCATE( ALT_TMP(N_OP) )
     ! Preliminary only output for atmosphere or ocean
     SELECT CASE( TRIM(Obs_POSITION(1)) )
     CASE('Atmos')
        ! Altitudes in the atmosphere must be at this step in the increasing order
        INV_ALT = .FALSE.
        IF( N_OP > 1 ) THEN
           IF( ALT_OUT(2) < ALT_OUT(1) ) THEN
              ALT_TMP(:) = ALT_OUT(N_OP:1:-1)
              ALT_OUT(:) = ALT_TMP(:) ; INV_ALT = .TRUE.
           ENDIF
        ENDIF
        ALLOCATE( Z_TMP(NLAYERS_INP+N_OP), ALT_MARK(N_OP) )
        Z_TMP(1) = ZZZ_INP(1) ; J = 2 ; ALT_MARK(:) = .FALSE.
        ! Logical array ALT_MARK(1:N_OP) will be used to mark 
        ! those altitudes which do not coincide with 
        ! the original input height-grid levels
        DO I = 2, NLAYERS_INP
           DO K = 1,N_OP
              DS = ALT_OUT(K)
              IF(DS > ZZZ_INP(I-1).AND.DS < ZZZ_INP(I)) THEN
                 Z_TMP(J) = DS ; ALT_MARK(K) = .TRUE.
                 J = J + 1
              ENDIF
           ENDDO
           Z_TMP(J) = ZZZ_INP(I)  ;  J = J + 1
        ENDDO
        IF( INV_ALT ) ALT_OUT(:) = ALT_TMP(N_OP:1:-1)            
        DEALLOCATE( ZZZ_INP ) ; NLAYERS_INP = J - 1
        ALLOCATE( ZZZ_INP(NLAYERS_INP) )  
        ! Extended input altitude grid which includes 
        ! required output altitudes
        ZZZ_INP(:) = Z_TMP(1:NLAYERS_INP)
        ! If the solution of adjoint equation should
        ! be used (weighting functions or Raman scattering)
        ! then output altitude level (N_OP=1 in this caes)
        ! will be consider as a standard input level
        IF( WF_FLAG .OR. DO_RAMAN ) THEN
           ARB_OL = .FALSE. ; ALT_MARK(1) = .FALSE.
        ENDIF
     CASE('Ocean')
        ! Depths in the ocean maust be at this step 
        ! in the increasing order
        INV_ALT = .FALSE.
        IF( N_OP > 1 ) THEN
           IF( DABS(ALT_OUT(2)) < DABS(ALT_OUT(1)) ) THEN
              ALT_TMP(:) = ALT_OUT(N_OP:1:-1)
              ALT_OUT(:) = ALT_TMP(:) ; INV_ALT = .TRUE.
           ENDIF
        ENDIF
        ALLOCATE( Z_TMP(NDEP_INP+N_OP), DEP_MARK(N_OP) )
        Z_TMP(1) = DEP_INP(1) ; J = 2 ; DEP_MARK(:) = .FALSE.
        ! Logical array DEP_MARK(N_OP) will be used to mark 
        ! those output depths which do not coincide with
        ! the original input-depth grid
        DO I = 2, NDEP_INP
           DO K = 1,N_OP
              DS = DABS(ALT_OUT(K))
              IF(DS > DEP_INP(I-1).AND.DS < DEP_INP(I)) THEN
                 Z_TMP(J) = DS ; DEP_MARK(K) = .TRUE.
                 J = J + 1
              ENDIF
           ENDDO
           Z_TMP(J) = DEP_INP(I)  ;  J = J + 1
        ENDDO
        IF( INV_ALT ) ALT_OUT(:) = ALT_TMP(N_OP:1:-1)
        DEALLOCATE( DEP_INP ) ; NDEP_INP = J - 1
        ALLOCATE( DEP_INP(NDEP_INP) )  
        ! Extended input depth grid which includes required output depths
        DEP_INP(:) = Z_TMP(:)
        ! If the solution of adjoint equation should
        ! be used (weighting functions) then output depth 
        ! level (N_OP=1 in this caes) will be consider as 
        ! a standard input depth-level
        IF( OWF_FLAG ) THEN
           ARB_OL = .FALSE. ; DEP_MARK(1) = .FALSE.
        ENDIF
     END SELECT
  ENDIF

  ! For the test purposes it is implemented here Adjoint Solution 
  ! Interpolation (ASI) approach. It allows us to obtain solution 
  ! of the direct RTE employing instead of the SFI technique the 
  ! solution of the adjoint RTE.
  SOLVER = 'Solv_SFI'
  IF( .NOT.WF_FLAG ) THEN
     IF( DO_SSRCA ) THEN
        SOLVER = 'Solv_SFI'
     ELSE
        SELECT CASE( ANG_INT )
        CASE( 'sfi' )
           SOLVER = 'Solv_SFI'
        CASE( 'asi' )
           SOLVER = 'Solv_ASI'
        END SELECT
     ENDIF
     SELECT CASE( ANG_INT )
     CASE( 'sfi' )
        N_RHS  = Kvs*NSA
        SDVI   = -300       ! not used
     CASE( 'asi' )
        N_RHS  = NLA
        SDVI   = 0         
     END SELECT
  ELSE
     ! Including weighting functions
     SELECT CASE (TRIM(SCATT_MODE))
     CASE('dir_ref')
        N_RHS  = NSA
     CASE('ms')
        SOLVER = 'Solv_MIX'
        N_RHS  = Kvs*NSA + NS_wf*NLA
        SDVI   = Kvs*NSA
     CASE('sm')
        CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The mode "sm" ' &
             //'is not yet implemented for weighting functions')
        RETURN 1
     CASE('wf_ss', 'sm_wf_ss', 'ss')
        IF( RTM_CORE == 'DOM') THEN
           IF(RTM_TYPE /= 'spher_scat' .AND. RTM_TYPE /= 'pp_scat') GOTO 215
        END IF
        SOLVER = 'Solv_SFI'
        N_RHS  = NSA
        SDVI   = -300       ! not used
     END SELECT
  ENDIF

  ! The following settings is relevant if RRS is included.
  !           !! ATTENTION !!
  ! Parameters SOLVER and N_RHS will be redefined in the routibe 
  ! "rtm_main.f" after do-loop over actual wavelengths if
  ! the mode "APPR_RRS" is set to 'simple binning'.
  IF( RRS_ADJ.OR.INEL_ADJ ) THEN 
     SOLVER = 'Solv_MIX'
     ! NS_wf is used preliminary instead of NS_rrs VR 22.04.2016 !!!
     N_RHS  = Kvs*NSA + NS_wf*NLA
     SDVI   = Kvs*NSA         
  ENDIF

  IF( RTM_CORE == 'DOM') THEN
     SELECT CASE (ANG_SEL_FLAG)
     CASE ('all')
        N_SHIFT = NLA*N_AZ_ANG ;  N_AZ_VAR = N_AZ_ANG
        ALLOCATE( JV_S( MAX(NLA,NSA) ), JV_E(NSA), JS_E(NLA) )     ! A 
        JV_S(:) = 1
        JV_E(:) = NLA ; JS_E(:) = NSA
     CASE ('one')
        ! If "SET_OUT_ALT" = .TRUE. and ALT_OUT(1) > TOA than
        !    "all" is switched to "one". It follows that:
        !     N_LOS_OUT = NSA = NLA = N_AZ_ANG 
        N_SHIFT = 0 ; N_AZ_VAR = 1
        ALLOCATE( JV_S(NSA), JV_E(NSA), JS_E(NSA) )              ! A
        FORALL( I = 1:NSA ) JV_S(I) = I
        JV_E(:) = JV_S(:) ; JS_E(:) = JV_S(:)
     END SELECT
     
  ENDIF

  
  Del_ang = 1.d-05/180.d0*Pie
  IF( TRIM(RTM_CORE) == 'DOM' ) THEN
     N_GAU = NSTR
     ALLOCATE( Gau_TES(NSTR), AG_TES(NSTR), Equal_view(N_LOS_OUT,N_FOV_ANG)) ! A
     IF( UWT ) THEN
        N_GAU = NSTR-1 ; Gau_TES(NSTR) = 1.d0 ; AG_TES(NSTR) = 0.d0
     ENDIF
     CALL GAULEG(0.d0,1.d0, Gau_TES(1:N_GAU), AG_TES(1:N_GAU),N_GAU)
     Equal_view = .FALSE.
     DO L = 1, N_LOS_OUT
        DO I = 1, N_FOV_ANG
           DS = ACOS(ANG_FOV(1,L,I))
           WHERE(DABS(DS - ACOS(ABS(ANG_FOV(2,:,:))) ) < Del_ang) Equal_view = .TRUE.
        ENDDO
     ENDDO

     DO I = 1, NSTR
        WHERE(ABS(DACOS(Gau_TES(I)) - ACOS(ABS(ANG_FOV(2,:,:)))) < Del_ang) &
             Equal_view(:,:) = .TRUE.
     ENDDO

     DO L = 1, N_LOS_OUT
        DO I = 1, N_FOV_ANG
           IF( .NOT. Equal_view(L,I) ) CYCLE
           ANG_FOV(2,L,I) = ANG_FOV(2,L,I)*COS(Del_ang) - &
                SQRT(1.d0-ANG_FOV(2,L,I)**2)*SIN(Del_ang)
        ENDDO
     ENDDO

     ANG_OUT(:,:) = ANG_FOV(:,:,N_FOV_ANG/2 + 1)

     IF( ANY(Equal_view) ) THEN
        CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'Zenith viewing angle(s) is(are) '//&
             'shifted to avoid equality to solar or Gauss angle. Shift is equal '//&
             'to -1.0e-7 for cosines of the initial value.')
     ENDIF
  ENDIF

  ALLOCATE( LS_VA(NLA), LS_AA(N_AZ_ANG) )
  SELECT CASE( ANG_SEL_FLAG )
  CASE( 'one' )
     LS_VA(:) = ANG_OUT(2,1:NLA)
  CASE( 'all' )
     LS_VA(:) = ANG_OUT(2,1:NLA*N_AZ_ANG:N_AZ_ANG)
  END SELECT
  LS_AA(:) = ANG_OUT(3,1:N_AZ_ANG)
  
  IF( LOS_DEF_FLAG=='ga' ) THEN
     ! Output of line-of-sight angles used with option "ga"
     II = 0 
     OPEN(C_UNIT,file = DIR_OUT//'/Ang_ga_out.dat')
     WRITE(C_UNIT,'(A)') '# line-of-sight angles used with option "ga"'
     IF(ANG_SEL_FLAG.EQ.'all') THEN
        DO IDX_SOL = 1,NSA
           DO IDX_LOS = 1, NLA
              DO IDX_PHI = 1,N_AZ_ANG ; II = II + 1
                 WRITE(C_UNIT,'(2x,I3,2x,F8.3)') II, DACOS(ANG_OUT(2,II))*RTD 
              END DO
           END DO
        END DO
     ELSE
        DO IDX_LOS = 1,N_LOS_OUT  ;  II = II + 1
           WRITE(C_UNIT,'(2x,I3,2x,F8.3)') II, DACOS(ANG_OUT(2,II))*RTD
        ENDDO
     ENDIF
     CLOSE(C_UNIT)
  ENDIF
  
  CALL ST_TRACK (R_NAME, 3, 'O')
  RETURN          
  
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  RETURN 1
  
102 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  RETURN 1
  
103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Input is not recognized')
  WRITE(EU,*) '    Parameter to read: ', STR_INP
  WRITE(EU,*) '    Unrecognized input: ', READ_ARR(II)%S
  RETURN 1
  
110 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Only one user-defined altitude can be set '//&
         'for the CDI solver ("RTM-CORE" parameter in "control.inp" is set to "CDI")')
  RETURN 1
  
111 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Only one user-defined altitude can '//&
         'be set in the spherical and in the trasmission modes ("RTM_TYPE" '//&
         'parameter in "control.inp" is set to "spher_scat" or "transmission")')
  RETURN 1

112 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)')'   Only one user-defined altitude can be used '
  WRITE(EU,'(A)') '   if it is larger than the top of atmosphere'
  WRITE(EU,'(A)')'   (Check "control_geom.inp" file)'
  RETURN 1

113 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') 'Output at multiple user-defined altitudes '
  WRITE(EU,'(A)') '   can be used in atmosphere or in ocean, but not'
  WRITE(EU,'(A)') '   simultaneously. Check input files: '
  WRITE(EU,'(A)') '   "control_geom.inp" '
  RETURN 1

200 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Parameter "au" '//&
         'can not be combined with "one" or "transmission"')
  RETURN 1

201 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '"ga" setting for "Type of LOS definition"'//&
         'parameter (see gontrol_geom.inp file) can be used only if "RTM-CORE" '//&
         'parameter is set either to "DOM_S" or to "DOM_V" (see control.inp file)')
  RETURN 1
  
202 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Only single SZA in "ASYMP" mode is allowed')
  RETURN 1

203 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Requested tangent height is above '//&
         'the top of the atmosphere or instrument position (whatever is lower)')
  RETURN 1
  

204 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) '   Line-of-sight can be specified by the'//&
       ' tangent height in spherical geometry only.'
  WRITE(EU,*) '   Check "RTM_TYPE" parameter in "control.inp" file'
  WRITE(EU,*) '   and "Type of LOS definition" parameter in "control_geom.inp" file.'
  RETURN 1

205 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Only 1 line-of-sight in "ASYMP" mode is allowed')
  RETURN 1
  
206 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'Only 1 azimuth angle in "ASYMP" mode is allowed')
  RETURN 1
  
207 CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
         'The output altitude must be above BOA')
  WRITE(EU,*) '    Selected altutude, BOA :', ALTITUDE(1), Z_BOA
  RETURN 1

209 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'One or several line of sights do '&
         //'not cross the atmosphere')
  RETURN 1

210 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,'(A)') '    The number of solar zenith angles is too '// &
       'small to integrate over this variable.'
  WRITE(EU,'(A)') '    Increase the number of solar zenith angles (at least 3)'
  WRITE(EU,'(A)') '    in the input file "control_geom.inp".' 
  RETURN 1

211 CALL WRITE_MESSAGE(R_NAME, 'ERROR',& 
         'Wrong definition of the instrument position:')
  WRITE(EU,*) '    The "too" and "boo" modes can only be used if '
  WRITE(EU,*) '     the "Coupling of atmosphere and ocean" '
  WRITE(EU,*) '     control parameter is set to "t"'
  WRITE(EU,*) '     Check "control_geom.inp" '
  RETURN 1

215 CALL WRITE_MESSAGE(R_NAME, 'ERROR', '')
  WRITE(EU,*) '    If "RTM_CORE" control parameter is set to '//&
       '"DOM" the single scattering ' 
  WRITE(EU,*) '    weigthing functions ("ss", "wf_ss" and '//&
       '"sm_wf_ss") can only be calculated '
  WRITE(EU,*) '    if "RTM_TYPE" parameter is set ether to '//&
       '"pp_scat" or to "spher_scat".'
  WRITE(EU,*) '    Check the input file "control_ac.inp".'
  RETURN 1

216 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE(EU,*) '     Downwellig diffuse radiation at TOA (top of atmosphere)'
  WRITE(EU,*) '     is to be calculated, which is zero per definition. '
  WRITE(EU,*) '     The execution is stopped as there is nothing to calculate.'
  WRITE(EU,*) '     Check "Position of the instrument" and "Viewing angles" '
  WRITE(EU,*) '     parameters in "control_geom.inp" file'
  RETURN 1

217 CALL WRITE_MESSAGE(R_NAME, 'ERROR','Parameter "Lines of sight number for FOV '//&
         'integration" must be an odd integer greater or equal to 3')
  RETURN 1

218 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE(EU,'(a)') '   Mode "ref_dir" can be used if '//&
       '"Position of the instrument" is set to "toa".'
  WRITE(EU,'(a)')  '   Check "control_geom.inp",'
  WRITE(EU,'(a)')  '   check "SCATT_MODE" in "control_ac.inp".'
  RETURN 1

219 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE(EU,*) '     In the case of spherical geometry instrument can not be '
  WRITE(EU,*) '     located at the bottom of atmosphere and pointed downwards.'
  WRITE(EU,*) '     Instrument position : ', TRIM(INS_POSI)
  WRITE(EU,*) '     Viewing direction   : ', TRIM(INS_VIEW)
  WRITE(EU,*) '     Check file "control_geom.inp" '
  RETURN 1

220 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ')
  WRITE(EU,*) '     In the case of plane-parallel geometry the '
  WRITE(EU,*) '     solar zenith angle has to be smaller 90 deg.'
  WRITE(EU,*) '     Check control file "control_geom.inp".'
  RETURN 1

223 CALL WRITE_MESSAGE(R_NAME, 'ERROR','Full width of the FOV function '&
         //'must be larger than the field of view size')
  RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1

CONTAINS
!**************************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
      
    IF(ALLOCATED(Obs_POSITION)) DEALLOCATE(Obs_POSITION)
    IF(ALLOCATED(ALTITUDE)) DEALLOCATE(ALTITUDE, ALT_OUT)
    IF(ALLOCATED(N_OBS_ALT)) DEALLOCATE(N_OBS_ALT)
    If(Allocated(SOL_ANG)) DeAllocate (SOL_ANG)
    If(Allocated(LOS_ANG)) DeAllocate (LOS_ANG)
    If(Allocated(ANG_OUT)) DeAllocate (ANG_OUT)
    If(Allocated(ANG_ORIG)) DeAllocate (ANG_ORIG, ANG_FOV, FOV_WEIGHTS)
    If(Allocated(LS_VA)) DeAllocate (LS_VA, LS_AA)
    IF(ALLOCATED(ALT_MARK)) DEALLOCATE(ALT_MARK)
    IF(ALLOCATED(JV_S)) DEALLOCATE(JV_S, JV_E, JS_E)
    IF(ALLOCATED(ANG_OUT_S)) DEALLOCATE(ANG_OUT_S, ANG_FOV_S)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
!**************************************************************************
END SUBROUTINE READ_GEOM_PARS
!**************************************************************************

SUBROUTINE FOV_SETUP (ANG_INP, ANG_FOV)
  ! Called by "read_geom_pars.f" and "retrieval_results.f"

  USE SPHER_PARS, ONLY: FOV_FLAG, N_FOV_ANG, N_LOS_OUT, FOV_ANG, &
                        FOV_WEIGHTS, NFOV_WNG, NFOV_CORE, FOV_FULL, FOV_SHP
  USE TYPES,      ONLY: KND
  
  REAL (KND) ANG_FOV(3,N_LOS_OUT,N_FOV_ANG), ANG_INP(3,N_LOS_OUT)
  ! Local variables:
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'FOV_SETUP'
  INTEGER    I, K, IDX_LOS, ICEN
  REAL (KND) D_WNG, VA(N_LOS_OUT), D_ANG(N_FOV_ANG)
  !==================================================================
  CALL ST_TRACK (R_NAME, 3, 'I')
  
  IF(FOV_FLAG)  THEN
     ANG_FOV(1,:,:) = SPREAD(ANG_INP(1,:),2,N_FOV_ANG)
     ANG_FOV(3,:,:) = SPREAD(ANG_INP(3,:),2,N_FOV_ANG)
     VA(:) = ACOS(ANG_INP(2,:))
     ICEN = FLOOR(NFOV_CORE/2.) + 1 + NFOV_WNG
     IF (FOV_SHP == 'GS') THEN ! Gaussian FOV
        D_WNG = (FOV_FULL - FOV_ANG)/2.0D0/NFOV_WNG
        DO I = 1, NFOV_WNG
           D_ANG(I) =  - FOV_ANG/2.0D0 - D_WNG*(NFOV_WNG - I + 1)
           D_ANG(NFOV_WNG + NFOV_CORE + I) = FOV_ANG/2.0D0 + D_WNG*I
        END DO
        DO I = NFOV_WNG + 1, NFOV_WNG + NFOV_CORE
           D_ANG(I) = (ICEN - I)*FOV_ANG/(NFOV_CORE - 1) 
        END DO
        DO I = 1, N_FOV_ANG
           FOV_WEIGHTS(I) = EXP( -LOG(2.0D0)*(D_ANG(I)/FOV_ANG*2.0D0)**2)
        END DO
     ELSE
        DO I = 1, N_FOV_ANG
           D_ANG(I) = (ICEN - I)*FOV_ANG/(N_FOV_ANG - 1)
        END DO
        FOV_WEIGHTS(:) = 1.0D0
     ENDIF
     FORALL(I = 1:N_FOV_ANG) ANG_FOV(2,:,I) = COS(VA(:) + D_ANG(I))
     FOV_WEIGHTS(:) =  FOV_WEIGHTS(:)/SUM(FOV_WEIGHTS(:))
  ELSE
     ANG_FOV(:,:,1) = ANG_INP(:,:)
     FOV_WEIGHTS(1) = 1.0D0
  END IF
  
  CALL ST_TRACK (R_NAME, 3, 'O')
  RETURN    
END SUBROUTINE FOV_SETUP
!**************************************************************************
Subroutine TextOrFile (StrInp, Arr, *)
  USE OutInf, Only: EU
  Use Tools,  Only: IsNum, Read_File, Set_Path, Read_Input
  Use Types,  Only: Knd, Dyn_Char

  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'TextOrFile'
  Character(Len =:), Allocatable ::   StrInp
  Real (Knd), Allocatable :: Arr(:)
  ! Local variables:
  Integer NN, II, NCol
  Character(Len = :), Allocatable :: FileName
  REAL (KND), ALLOCATABLE :: DATA_R(:,:)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  !==================================================================
  If(Allocated(Arr)) DeAllocate(Arr) 

  CALL Read_Input (StrInp, READ_ARR, *880, ReArrange = .TRUE.)

  If(IsNum(READ_ARR(1)%S)) Then
     NN = SIZE(READ_ARR) ; ALLOCATE(Arr(NN)) !A
     DO II = 1, NN
        READ(READ_ARR(II)%S, *, ERR = 101) Arr(II)
     END DO
  Else
     FileName = Set_Path(READ_ARR(1)%S)
     Read(READ_ARR(2)%S,*, Err = 101) NCol  
     CALL READ_FILE (NCol, FileName, DATA_R, *880)
     NN = SIZE(DATA_R,1) ; ALLOCATE(Arr(NN)) !A
     Arr =  DATA_R(:,NCol)       
  End If
  
  Return
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', StrInp
  Return 1
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  Return 1
End Subroutine TextOrFile
!**************************************************************************
