!***************************************************************************
!
!	SCIATRAN - Version 4.6 - Last modification: 06 Jun 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_INPUT_FILES (CI_RUN, *)
! ... called by "gt_iface.f" 

  USE LOG_PARS, ONLY: EU, Work_Dir
  USE ST_INPUT, ONLY: Inp_Pars, CI_Pars
  use St_Vars,  only: MPI_FLG, CI_FLG, CI_DIR
  Use Tools,    only: ErrMsg
  USE TYPES,    ONLY: KND, DYN_CHAR

  LOGICAL  CI_RUN
  
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_INPUT_FILES'
  CHARACTER (LEN=:), ALLOCATABLE :: FILE, Inp_Dir
  INTEGER, PARAMETER ::  N_FILES = 41
  INTEGER  INFO, II, KK, CU, N_PRE
  LOGICAL  FileFlg, DelayedOutput
  CHARACTER (LEN=15) NAMES(N_FILES) 
  TYPE Linked_List_Entry
     CHARACTER (LEN = :), ALLOCATABLE :: READ_STR
     TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
  END TYPE Linked_List_Entry
  TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL(), NEXT
  TYPE (DYN_CHAR), ALLOCATABLE :: Msg(:)
  TYPE (DYN_CHAR), POINTER :: Inp_Pars_Read(:)
  !==================================================================
  
  call st_track (R_Name, 3, 'I')
  KK = 0

  Names = [&
       'control        ', 'cf_cloud       ', 'control_ac     ', &
       'control_aer    ', 'control_brdf   ', 'control_conv   ', &
       'control_geom   ', 'control_la     ', 'control_out    ', &
       'control_pas    ', 'control_prof   ', 'control_ray    ', &
       'control_ret    ', 'control_rrs    ', 'control_te     ', &
       'control_tls    ', 'control_uwt    ', 'control_wf     ', &
       'kop_hsol       ', 'low_aer        ', 'man_aer        ', &
       'man_hsol       ', 'man_vhl        ', 'mie_aer_general', &
       'mie_aer        ', 'ret_sdp        ', 'rtm_esft       ', &
       'scia_aer       ', 'wmo_aer        ', 'wmo_general    ', &
       'xsections      ', 'control_ali    ', 'mod_aer_oc     ', &
       'mod_aer        ', 'mod_aer_man    ', 'pao_aer        ', &
       'control_uit    ', 'cf_aer_merra   ', 'cf_ice_man     ', &
       'cf_aeronet     ', 'cf_xb_merra    ' ] 
  
  IF(CI_FLG.AND.CI_RUN) THEN
     Inp_Dir = CI_DIR
  Else
     Inp_Dir = Work_Dir
  End If

  DelayedOutput=CI_FLG.AND.CI_RUN

  IF(.NOT.CI_RUN) THEN ! Do not read my_init.inp file in the common input directory
     ! Check if my_init.inp file is peresent
     File = Inp_Dir//'my_init.inp'
     inquire(file=File, exist=FileFlg)
     IF(FileFlg)  THEN
        Allocate(Msg(2))
        Msg(1)%S = 'my_init.inp file is found, '
        Msg(2)%S = 'duplicated settings from other *.inp files will be overriden'
        call ErrMsg(R_Name, 'W', Msg, DelayedOutput)
        open(NewUnit = CU, file = File, Status = 'OLD', iostat = info )
        CALL Read_Control_File (CU, KK)
        close (CU)
     END IF
  ELSE
     FileFlg = .FALSE.
  END IF

  IF( (CI_FLG.AND.CI_RUN) .OR. .NOT.CI_FLG ) THEN
     do ii = 1, n_files
        File = Inp_Dir//'/'//TRIM(Names(ii))//'.inp'
        open(NewUnit = CU, file = File, Status = 'OLD', iostat = info )
        if(info == 0) then
           CALL Read_Control_File (CU, KK)
           close (CU)
        else ! The requested file does not exist
           if(ii > 1.or.FileFlg) then
              Allocate(Msg(1))
              Msg(1)%S = TRIM(Names(ii))//'.inp file is not found'
              call ErrMsg(R_Name, 'W', Msg, DelayedOutput)
              cycle ! Any control file with exception of control.inp
              ! if my_init.inp is pressent control.inp is allowed to miss 
           end if
           !        ! If control.inp is missing all settings are read from the single file
           !        open(NewUnit = CU, file = Inp_Dir//'rtm_ret_all.inp', &
           !             Status = 'OLD', iostat = info )
           !        if(info /= 0) goto 101
           !        CALL Read_Control_File (CU, KK)
           !        close (CU)
           !        exit
           goto 101
        end if
     end do
  END IF

  If(KK /= 0) Then
     IF(CI_RUN) THEN
        Allocate(CI_Pars(KK)) 
        Inp_Pars_Read => CI_Pars !Common input parameters
     ELSE
        If(Allocated(Inp_Pars)) Deallocate(Inp_Pars)
        Allocate(Inp_Pars(KK))
        Inp_Pars_Read => Inp_Pars
     END IF
     
     If(Associated(element)) Nullify(element%next_elem)
     ii = 0
     element => first_elem
     do while (associated(element))
        ii = ii + 1
        Inp_Pars_Read(II)%S = TRIM(ADJUSTL(element%read_str))
        next => element%next_elem
        deallocate (element) ; nullify (element)
        element => next
     end do
  End If

  call st_track (R_Name, 3, 'O')
  return
  
100 CONTINUE
  Allocate(Msg(1))
  Msg(1)%S = 'No control parameters in '//Inp_Dir//' are found'
  call ErrMsg(R_Name, 'e', Msg, DelayedOutput)
  return 1
101 CONTINUE
  Allocate(Msg(1))
  Msg(1)%S = 'No control files are found'
  call ErrMsg(R_Name, 'e', Msg, DelayedOutput)
  return 1
CONTAINS
!***************************************************************************
  SUBROUTINE  Read_Control_File (IUNIT, KK)
    Use Tools, Only: Blank_Or_Comment
    INTEGER  IUNIT, KK, IOSTAT
    Logical  CommentFlg
    CHARACTER (LEN=1)  CHAR
    CHARACTER (LEN=:), ALLOCATABLE :: STRING
    
    CommentFlg = .False.
    DO ! Loop over all lines in the file
       STRING = ''
       DO
          READ(IUNIT, '(A1)', ADVANCE = 'no', IOSTAT = IOSTAT, END = 700) CHAR
          IF(IOSTAT /= 0) EXIT
          STRING = STRING//CHAR
       END DO

       If(Blank_Or_Comment(STRING)) then
          If(CommentFlg) Cycle
          CommentFlg = .True.
       Else
          CommentFlg = .False.
       End If

       IF (.NOT.ASSOCIATED(ELEMENT)) THEN
          ALLOCATE(FIRST_ELEM)
          ELEMENT => FIRST_ELEM
          KK = 1
       ELSE
          ALLOCATE(ELEMENT%NEXT_ELEM)
          ELEMENT => ELEMENT%NEXT_ELEM
          KK = KK + 1
       ENDIF

       ELEMENT%READ_STR = STRING
    END DO
700    CONTINUE
    ! Assign last line 
    IF (.NOT.ASSOCIATED(ELEMENT)) THEN
       ALLOCATE(FIRST_ELEM)
       ELEMENT => FIRST_ELEM
       KK = 1
    ELSE
       ALLOCATE(ELEMENT%NEXT_ELEM)
       ELEMENT => ELEMENT%NEXT_ELEM
       KK = KK + 1
    ENDIF
    
    ELEMENT%READ_STR = STRING

    IF (ASSOCIATED(ELEMENT)) NULLIFY(ELEMENT%NEXT_ELEM)

    RETURN
  END SUBROUTINE Read_Control_File
!***************************************************************************

END SUBROUTINE READ_INPUT_FILES
!***************************************************************************
