!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 19 Mar 2019
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_MIE_AER_INP(NAW, START_WAV, END_WAV, WL_MIE, LAN, Z_BOT, &
     REL_HUM, N_COM, DB_NAM, N_AER_LEV, AER_LEV, AER_PRF, Vo_PM, AER_TYPE, &
     LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, *, *)

  USE AER_MIE,     ONLY: MOD_R, NAC, SIG, RD, RI_rea, RI_ima, N_PSD, L_ARR, &
                         Nst, Rmi_lim, M_AC, Rm_log, COEF1, COEF2, COEF3, &
                         COEF4, COEF5, COEF6, COEF7, COEF8, F_MODE, PM_2i5
  USE AEROSOLS,    ONLY: AER_SHIFT, Use_der, N_Dwr, Dwr_mi, Bi_PSD, Rf_max, &
                         New_lay, AER_SCALING
  USE GRIDS,       ONLY: NLAYERS, ZZZ
  USE INTERN_DIMS, ONLY: Nm_vs
  USE LOG_PARS,    ONLY: EU, Work_Dir, LUNIT, LOG_LEVEL, N_SSO
  USE PM_ARR,      ONLY: V_c, V_f, A_c, A_f, Ro_dens
  USE ST_VARS,     ONLY: GROUND_HEIGHT, Pie
  USE TOOLS,       ONLY: SPLIT_STRING, READ_INPUT, INQ_WF, SET_PATH, CUT_STR,&
                         Read_File, IsNum
  USE TYPES,       ONLY: KND, DYN_CHAR 

  INTEGER  NAW, LAN, N_COM, N_AER_LEV
  INTEGER, ALLOCATABLE :: N_EC(:,:,:)
  REAL (KND)  Z_BOT, START_WAV, END_WAV
  REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), DB_EXT(:,:,:,:), &
       DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:), AER_LEV(:), AER_PRF(:), Vo_PM(:,:)
  CHARACTER(LEN=6),  ALLOCATABLE :: DB_NAM(:,:), DB_NAM_1(:,:)
  CHARACTER(LEN=30), ALLOCATABLE :: AER_TYPE(:), AER_TYPE_1(:)
  CHARACTER(LEN=4), ALLOCATABLE ::   S_NAME(:)
  CHARACTER(LEN=2), ALLOCATABLE ::   H_NAME(:)

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_MIE_AER_INP'
  INTEGER, PARAMETER :: M_H = 8
  REAL (KND)  DB_HUMS(M_H)
  DATA DB_HUMS  /0., 50., 70., 80., 90., 95., 98., 99./
  CHARACTER(LEN=2)   HYM(M_H)
  DATA HYM  /'00', '50', '70', '80', '90', '95', '98', '99'/
  INTEGER   K1, N_RI, I1, I2, J1, NDISTR, Nmi, NPmi, NKmi, max_db, I_p, J_p,&
       N_sel, N_add, M_wl, Jst, Nps, Ma, L1_Max, L, L1, KK, IUNIT
  INTEGER   N_gau, N_MIE_WL, I, J, K, MCO, Id
  INTEGER, ALLOCATABLE :: NH(:), NAC_1(:)
  LOGICAL   NO_COMP, NOWR, L_HEL, CheckFlg
  REAL (KND) S_UM, DP, AA, BB, GAM, LAM, MRR, MRI, R1, R2, ATOP, RVW, RMEAN, &
       RX, AA1, BB1, AA2, BB2, DDELT, ABOT, R2_PM 
  REAL (KND), ALLOCATABLE :: Rep(:), Imp(:), WL_RI(:), AREA(:), VOL(:)
  REAL (KND), ALLOCATABLE :: Repa(:,:), Impa(:,:), WL_RIa(:,:), Mor(:), &
       Var(:), CEXT(:), CSCA(:), CEXT_Dr(:), CSCA_Dr(:), CEXT_Ds(:), &
       CSCA_Ds(:), Grs(:,:,:), Grs_Dr(:,:,:), Grs_Ds(:,:,:), OEXT(:,:), &
       OSCA(:,:), Leg_cof(:,:,:), RD_1(:,:), MOD_R1(:,:), SIG_1(:,:), &
       DATA_R(:,:)
  CHARACTER (LEN=4)   CH4
  INTEGER  Nbo, Nto
  ! Short name of non-spherical aerosol components presented
  ! in OPAC 4.0 data base
  INTEGER, PARAMETER :: NNS = 3
  CHARACTER (LEN=4)  NAM_NSP(3)
  CHARACTER (LEN=:), ALLOCATABLE :: Path_RIC_DB, STR_INP, TXT, H_CHA
  LOGICAL,  ALLOCATABLE :: NON_SPHER(:,:)

  CHARACTER(LEN=6),  ALLOCATABLE :: U_NAM(:) 
  INTEGER, ALLOCATABLE :: NWL_Ri(:), L1max(:), M_cof(:,:)
  INTEGER  LAN_1, L2
  REAL (KND), ALLOCATABLE :: LTOP_1(:), REL_HUM_1(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:), STR_ARR_1(:)
  !=========================================================================
      
  NAM_NSP(1)='minn' ;  NAM_NSP(2)='mian' ;  NAM_NSP(3)='micn'

  STR_INP = 'Path to refractive indexes DB'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  Path_RIC_DB = SET_PATH(READ_ARR(1)%S)
  STR_INP  = 'Aerosol components short names'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_COM = SIZE(READ_ARR)
  ALLOCATE( S_NAME(N_COM),H_NAME(N_COM) )
  DO I = 1, N_COM
     CALL SPLIT_STRING (READ_ARR(I)%S, STR_ARR)
     CALL SPLIT_STRING (STR_ARR(1)%S, STR_ARR_1, "-")
     ! contains short component name
     S_NAME(I) = STR_ARR_1(1)%S
     ! contains '-1' or '-8' depending on water-soluble corresponding component or not
     H_NAME(I) = "-"//STR_ARR_1(2)%S
  ENDDO
  !  WRITE(*,'(A,I3)') '  Number of aerosol component in DB: ',N_COM

  STR_INP  = 'Wavelength grid for the Mie aerosol parameterization'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
  N_MIE_WL = SIZE(READ_ARR)
  NAW = N_MIE_WL  ;  ALLOCATE( WL_MIE(N_MIE_WL) )
  DO I = 1, N_MIE_WL
     READ(READ_ARR(I)%S,*,ERR = 101) WL_MIE(I)
  END DO
  If (MinVal(WL_MIE) > START_WAV .OR. MaxVal(WL_MIE) < END_WAV) Goto 204
  ! Transform [nm] --> [micron]
  WL_MIE(:) = WL_MIE(:)*1.d-3

  STR_INP  = 'Mie aerosol layers'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  LAN = SIZE(STR_ARR) - 1 ;  ALLOCATE( REL_HUM(LAN),  LTOP(LAN) )  ! A
  ! Bottom and tops of aerosol layers ( bottom --> top )
  READ(READ_ARR(1)%S,*, ERR = 101) Z_BOT, ( LTOP(I),   I=1,LAN )
  ! Relative humidity of layers: ( bottom --> top )
  READ(READ_ARR(2)%S,*, ERR = 101)  ( REL_HUM(I), I=1,LAN )
  ! WRITE(*,*) '   Z_BOT   =  ', Z_BOT
  ! WRITE(*,*) '   L_TOP   =  ', (LTOP(I), I=1,LAN)
  ! WRITE(*,*) '   REL_HUM =  ', (REL_HUM(I), I=1,LAN)
  ! PAUSE        
  ! M_AC is defined now in the "gt_fileread.f" routine
  ! NAC(:) has to be allocated because it was deallocated in the 
  ! "mie_aer_setup.f" routine
  ALLOCATE( DB_NAM( LAN,M_AC ), RD(LAN,M_AC), AER_TYPE(LAN), NH(LAN), & ! A
       F_MODE(M_AC), MOD_R(LAN,M_AC), SIG(LAN,M_AC), NON_SPHER(LAN,M_AC), & ! A
       NAC(LAN) ) ! A
  MOD_R = 0.d0 ; SIG = 0.d0
  NON_SPHER(:,:) = .FALSE. ;  RD = 0.d0 ;  F_MODE(:) = .TRUE.

  STR_INP  = 'Adjust aerosol layers within main altitude grid'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*, ERR = 101) New_lay

  IF( PM_2i5 ) THEN
     IF( LAN > 1 ) GOTO 400
     IF( New_lay ) GOTO 401
     ALLOCATE( Vo_PM(LAN,M_AC) ) ;  Vo_PM = 0.d0
     ALLOCATE(V_c(M_AC), V_f(M_AC), A_c(M_AC), A_f(M_AC), Ro_dens(M_AC)) 
     V_c = 0.d0 ; V_f = 0.d0 ; A_c = 0.d0 ; A_f = 0.d0
     Ro_dens = 0.d0
  ENDIF

  IF( New_lay ) THEN
     AER_SHIFT = .FALSE.
     ! Nbo = MINLOC( DABS(ZZZ(:)-Z_BOT),1 )
     ! Nto = MINLOC( DABS(ZZZ(:)-LTOP(LAN)),1 )
     Nbo = COUNT( ZZZ(:) >= Z_BOT)
     Nto = COUNT( ZZZ(:) > LTOP(LAN))+1
!     WRITE(*,*) '   Position of aerosol layers ', ZZZ(Nbo), ZZZ(Nto)
     LAN_1 = Nbo - Nto + 1 !Changed back 22.05.19
     !     LAN_1 = Nbo - Nto !Changed 14.11.17

!     WRITE(*,*) '   Number of aerosol layers after adjusting: ', LAN_1
     ALLOCATE( LTOP_1(LAN_1), REL_HUM_1(LAN_1), NAC_1(LAN_1), &
          DB_NAM_1(LAN_1,M_AC), RD_1(LAN_1,M_AC), AER_TYPE_1(LAN_1), &
          MOD_R1(LAN_1,M_AC), SIG_1(LAN_1,M_AC) )
     MOD_R1 = 0.d0 ; SIG_1 = 0.d0
     K = 1
!     DO I = Nbo-1, Nto, -1 !Chaged 14.11.17
     DO I = Nbo, Nto, -1 !Changed back 22.05.19
        LTOP_1(K) = ZZZ(I) ; K = K + 1
     ENDDO
     
     !write(6,*) 'New layers:', LTOP_1, shape(DB_NAM_1)
     !pause
  ENDIF

  ! ======================================================================
  ! Conversion from input component indices to data base component indices
  ! ======================================================================
  NH = 0
  DO I = 1,LAN
     DO J = 1,M_H
        IF( REL_HUM(I)==DB_HUMS(J) ) NH(I) = J
     ENDDO  ! over humidities in DB (J)
     IF(NH(I)==0) GOTO 123
  ENDDO  ! over aerosol layers (I)

  IF( AER_SHIFT ) THEN 
     Z_BOT   = Z_BOT   + GROUND_HEIGHT
     LTOP(:) = LTOP(:) + GROUND_HEIGHT
  ENDIF

  ! <----- Setting of Use_der(:) replaced in "gt_fileread.f" ---------->
  ! Use derivatives of aerosol optical parameters with respect to the
  ! modal radius and/or variance of the particle size distribution (PSD) function 
  ! Check after
  !  N_dwr = 2
  !  WRITE(*,*) '   Use_der(:) ', Use_der(:)
  !  WRITE(*,*) '        N_dwr ', N_dwr

  IF( ANY(Use_der(:)).OR.N_dwr>0 ) Dwr_mi = .TRUE.

  ! Check after
  IF( Dwr_mi ) THEN
     IF( .NOT.INQ_WF('aer', 'RIT'))    GOTO 124
     IF( Bi_PSD.AND.LAN > 1 ) GOTO 201
  ENDIF

  IF( Bi_PSD ) THEN
     STR_INP  = 'Upper boundary for fine mode'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*, ERR = 101) Rf_max
  ENDIF

  STR_INP  = 'Mode radius for particle size distribution'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  TXT = CUT_STR(READ_ARR(1)%S)
  SELECT CASE ( TXT )
  CASE('log')
     Rm_log = .TRUE.
  CASE('non-log')
     Rm_log = .FALSE.
  CASE DEFAULT
     CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Mode radius representation is not recognized')
     WRITE(EU,*) '    Selected mode :', READ_ARR(1)%S
     RETURN 1           
  END SELECT

  STR_INP  = 'Numerical integration limits'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*, ERR = 101) R1, R2  
  Rmi_lim(1) = R1 ; Rmi_lim(2) = R2

  STR_INP  = 'Aerosol type definition (Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  
  KK = 0
  DO I = 1, LAN ! bottom --> top
     K1 = I
     ! Read long aerosol type definition
     KK = KK + 1
     AER_TYPE(I) = CUT_STR(READ_ARR(KK)%S)
     ! Read number of aerosol components in a layer
     KK = KK + 1
     READ(READ_ARR(KK)%S,*) NAC(K1)
     IF( NAC(K1) > M_AC ) GOTO 103
     ! Check in the case of "Adjust aerosol layers" option that 
     ! number of aerosol components is the same in all layers
     IF( New_lay.AND.I > 1 ) THEN
        IF( NAC(K1) /= NAC(K1-1) ) GOTO 104
     ENDIF

     S_UM = 0.d0
     DO J = 1,NAC(K1)
        ! Read short name, percentage, modal radius, and 
        ! variance of PSD function of aerosol component
        KK = KK + 1
        IF( PM_2i5 ) THEN
           READ(READ_ARR(KK)%S,*) CH4, RD(K1,J), MOD_R(K1,J),SIG(K1,J), Ro_dens(J)
        ELSE
           READ(READ_ARR(KK)%S,*) CH4, RD(K1,J), MOD_R(K1,J),SIG(K1,J)
        ENDIF
        S_UM = S_UM + RD(K1,J)
        IF( Bi_PSD ) THEN
           ! Select course mode
           ! WRITE(*,*) MOD_R(K1,J), Rf_max 
           IF( MOD_R(K1,J) > Rf_max )  F_MODE(J) = .FALSE.
        ENDIF
        ! WRITE(*,*) I, J, CH4
        ! --- Check if the selected component is presented in DB ---
        NO_COMP = .TRUE.
        DO K = 1, N_COM
           IF( CH4==S_NAME(K) ) THEN
              NO_COMP = .FALSE.
              ! Check if the component is non-spherical
              DO L = 1, NNS
                 IF( CH4==NAM_NSP(L) ) THEN
                    NON_SPHER(K1,J) = .TRUE.
                 ENDIF
              ENDDO
              ! Check dependence on the relative humidity and
              ! select datafile-name of refractive index in DB
              IF( H_NAME(K)=='-8' ) THEN
                 DB_NAM(K1,J) = CH4//HYM(NH(I))
              ELSE
                 DB_NAM(K1,J) = CH4//HYM(1)
              ENDIF
           ENDIF
        ENDDO  ! over aerosol comp. in database (K)
        IF( NO_COMP ) GOTO 202
        ! --------------------- End of check ---------------------- 
     ENDDO   ! over required aerosol comp. (J)
!     WRITE(*,*) '     S_UM = ', S_UM
     IF( S_UM < 0.99d0 .OR. S_UM > 1.01d0) RETURN 2
!     KK = KK + 1
  END DO  ! over aerosol layers (I)

  !  Check if the course mode is really presented
  !  WRITE( *,* ) '    Bi_PSD = ', Bi_PSD
  !  WRITE( *,* ) '    Rf_max = ', Rf_max
  IF( Bi_PSD.AND.COUNT(.NOT.F_MODE(:))==0 ) GOTO 203

  ! Read aerosol number density profile
  STR_INP  = 'Particle number density profile'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  ! Check if the first line is a file name
  TXT = Set_Path(READ_ARR(1)%S)
  INQUIRE(FILE=TXT, EXIST=CheckFlg)
  If (CheckFlg) Then
     Call Read_File (2, TXT, DATA_R, *880)
     Allocate (AER_LEV, Source=DATA_R(:,1))
     Allocate (AER_PRF, Source=DATA_R(:,2))
     N_AER_LEV = Size(AER_LEV)
  Else
     If(.Not.IsNum(READ_ARR(1)%S)) GoTo 106
     N_AER_LEV = SIZE(READ_ARR)
     IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
     ALLOCATE( AER_LEV(N_AER_LEV), AER_PRF(N_AER_LEV) ) ! A
     KK = 0
     DO I = 1,N_AER_LEV
        KK = KK + 1
        READ(READ_ARR(KK)%S,*) AER_LEV(I), AER_PRF(I)
     END DO
  End If
  AER_PRF(:) = AER_SCALING*AER_PRF(:)
  
  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT

  !************************ End of Reading *****************************
  IF( New_lay ) THEN
! The number of aerosol components should be the same in all layers
     NAC_1(:) = NAC(1)
     FORALL( I=1:M_AC ) 
        RD_1(:,I)     = RD(1,I) 
        ! DB_NAM_1(:,I) = DB_NAM(1,I)
     END FORALL
     DO I = 1, LAN
        IF( I==1 ) THEN 
           ABOT = Z_BOT ; ATOP = LTOP(I)
        ELSE
           ABOT = LTOP(I-1) ; ATOP = LTOP(I)
        ENDIF
        DO J = 1, LAN_1
           !Changed 20.01.2022, AR
           IF( LTOP_1(J)>=ABOT.AND.LTOP_1(J)<= ATOP ) THEN 
!           IF( LTOP_1(J)>=ABOT.AND.LTOP_1(J)< ATOP ) THEN
              FORALL( K=1:M_AC ) 
                 DB_NAM_1(J,K) = DB_NAM(I,K)
                 MOD_R1  (J,K) = MOD_R (I,K)
                 SIG_1   (J,K) = SIG   (I,K)
              END FORALL
!              write(6,*) DB_NAM_1(J,1)
              REL_HUM_1(J)  = REL_HUM(I)
              AER_TYPE_1(J) = AER_TYPE(I)
           ENDIF
        ENDDO
     ENDDO

     ! Interpolation of modal radius and variance on new altitude grid
     ! DO I = 1, NAC_1(1)
     !    CALL LIN_INT(LAN, LTOP, MOD_R(:,I), LAN_1, LTOP_1, MOD_R1(:,I), 'C')
     !    CALL LIN_INT(LAN, LTOP, SIG(:,I), LAN_1, LTOP_1, SIG_1(:,I), 'C')
     ! ENDDO
     ! CALL LIN_INT(LAN, LTOP, REL_HUM, LAN_1,LTOP_1,REL_HUM_1,'C')
     DEALLOCATE( LTOP, REL_HUM, NAC, DB_NAM, RD, AER_TYPE, MOD_R, SIG )
         
     LAN = LAN_1
     ALLOCATE( LTOP(LAN), REL_HUM(LAN), NAC(LAN), DB_NAM(LAN,M_AC), & !A
          RD(LAN,M_AC), AER_TYPE(LAN), MOD_R(LAN,M_AC), SIG(LAN,M_AC) )

     LTOP = LTOP_1     ; REL_HUM = REL_HUM_1 ; NAC = NAC_1
     DB_NAM = DB_NAM_1 ; RD = RD_1 ; AER_TYPE = AER_TYPE_1
     MOD_R  = MOD_R1   ; SIG = SIG_1

     ! WRITE(*,*) '  New layer tops and humidity'
     ! DO I = 1, LAN
     !   WRITE(*,'(2x, F7.3,2x, F7.3)')  LTOP(I), REL_HUM(I)
     !   DO K = 1, NAC(I)
     !      WRITE(*,'(9x,A,3(1x, F8.3))') DB_NAM(I,K), RD(I,K), MOD_R(I,K), SIG(I,K)
     !   ENDDO
     ! ENDDO
  ENDIF  ! new aerosol layers

  ! The total number of selected aerosol components
  N_sel = SUM( NAC(:) )

  ! If more than one aerosol layer then find maximal number
  ! of aerosol components
  MCO = MAXVAL( NAC(:) )

  IF( ANY(NON_SPHER) ) THEN
     IF(LOG_LEVEL(1) >= N_SSO) THEN
        WRITE(LUNIT,*) '    Non-spherical particles are presented !!'
        ! Check mixing of spherical and non-spherical particles
        IF( COUNT(NON_SPHER(:,:)) < N_sel) THEN
           WRITE(LUNIT,*) '   Mixing of spherical and non-spherical particles'
           WRITE(LUNIT,*) '   is not implemented yet.'
        ENDIF
        IF( LAN > 1) THEN
           WRITE(LUNIT,*) '   Non-spherical particles case is implemented now'
           WRITE(LUNIT,*) '   for single aerosol layer only!'
        ENDIF
     ENDIF
     N_gau = 1000
     ALLOCATE( OEXT(NAW,N_sel), OSCA(NAW,N_sel), Leg_cof(N_gau,N_sel,NAW), & !A
          M_cof(N_sel,NAW) ) !A
     
     ! This routine reads extinction, scattering coefficients and
     ! scattering function (scalar case only!) from OPAC data base
     
     CALL READ_OPAC_NSPH(DB_NAM, LAN, M_AC, N_sel, Path_RIC_DB, WL_MIE, NAW, &
          OEXT, OSCA, N_gau, M_cof, Leg_cof, *880)

     ! Maximal number of Legendre coefficients
     L1_Max = MAXVAL( M_cof )
     ! Maximal fourth dimension of DB_EXT and DB_SCA is 3 
     ! because for each aerosol component these arrays contain
     ! 1 - corresponding coefficient
     ! 2 - derivative w.r.t. modal radius
     ! 3 - derivative w.r.t. variance
     ALLOCATE( DB_EXT(LAN,MCO,NAW,3), DB_SCA(LAN,MCO,NAW,3), N_EC(LAN,MCO,NAW), & !A
          DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3) ) ! A
     DB_EXT = 0.d0 ; DB_SCA = 0.d0 ; DB_MOMS = 0.d0 ;  N_EC = 0
         
     DO I = 1, LAN
        DO J = 1, N_sel
           N_EC  (I,J,:) = M_cof(J,:)
           DB_EXT(I,J,1:NAW,1) = OEXT(:,J)
           DB_SCA(I,J,1:NAW,1) = OSCA(:,J)
           DO K = 1, NAW
              DO L = 1, M_cof(J,K)
                 DB_MOMS(I,J,K,1,L,1) = Leg_cof(L,J,K)
              ENDDO   ! over Legendre coefficients (L)
           ENDDO   ! over reference wavelengths (K)
        ENDDO   ! over non-spherical aerosol components (J)
     ENDDO   ! over aerosol layers (I)
     ! WRITE(*,*) '   ------> Return after  "read_opac_nsph.f" '
     RETURN
     ! STOP '  After  "read_opac_nsph.f" '
  ENDIF  !  optical properties of non-spherical particles 
  !
  ! ------------- If Mie calculations should be performed -------------
  !
  ALLOCATE( L_ARR(N_sel,LAN,N_sel), U_NAM(N_sel) )
  L_ARR = .FALSE.

  IF(LOG_LEVEL(1) >= N_SSO) THEN
     write(LUNIT,'(A,1x,I3)') '  Number of selected components: ', N_sel
     WRITE(LUNIT,'(A)') '  Aerosol layers and DB names (bottom --> top): '
     DO I = 1, LAN
        WRITE(LUNIT,'(1x,I3,4(1x,A))') I, ( DB_NAM(I,J),J=1,NAC(I) )
     ENDDO
  END IF

  ! Select number of different aerosol components:
  Nst = 1
  U_NAM(Nst) = DB_NAM(1,1)
  DO I = 1, LAN  ! bottom --> top
     DO K = 1, NAC(I)
        IF( DB_NAM(I,K)==U_NAM(Nst) )  L_ARR(Nst,I,K)=.TRUE.
     ENDDO  ! over aerosol components (K)
  ENDDO  ! over aerosol layers (I)

  ! L_ARR(1,I,K) contains .true. for such I-layers and 
  ! such K-components which are the same as in DB_NAM(1,1)

  DO Jst = 1, N_sel
     N_add =  N_sel - COUNT( L_ARR(1:Nst,:,:) )
     IF(LOG_LEVEL(1) >= N_SSO) THEN
        WRITE(LUNIT,*)
        WRITE(LUNIT,'(A,1x,I3)') ' ----- selected component: ', Nst
        WRITE(LUNIT,'(A,1x,I2)') '  Number of coincidence', COUNT( L_ARR(Nst,:,:) )
        WRITE(LUNIT,'(A,1x,I2)') '  Number of additional ', N_add
     END IF
     ! There is no additional components
     IF( N_add == 0 ) EXIT
     DO I = 1, LAN  ! bottom --> top
        DO K = 1, NAC(I)
           L_HEL = (.NOT.L_ARR(1,I,K))
           DO J = 1,Nst
              L_HEL = L_HEL.AND.(.NOT.L_ARR(J,I,K))
           ENDDO
           IF( L_HEL ) THEN
              U_NAM(Nst+1) = DB_NAM(I,K)
              GOTO 125
           ENDIF
        ENDDO  ! over components (K)
     ENDDO  ! over aerosol layers (I)
     
125  CONTINUE
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,'(A,2x,A)') &
          '  Next selected component: ', U_NAM(Nst+1)
     DO I = 1, LAN
        DO K = 1, NAC(I)
           IF(DB_NAM(I,K)==U_NAM(Nst+1))  L_ARR(Nst+1,I,K)=.TRUE.
        ENDDO
     ENDDO
     Nst = Nst + 1         
  ENDDO  ! over all selected components (Jst)

  ! Nst is the number of actual Mie calculations should be performed

  ALLOCATE( N_PSD(Nst), NWL_Ri(Nst) )
  DO J = 1, Nst  ! Number of Mie calculations
     N_PSD(J) = COUNT(L_ARR(J,:,:))
  ENDDO

  ! ********************** Write selected scenario *******************   
  ! WRITE(*,*) '   Selection is finished   '
  IF(LOG_LEVEL(1) >= N_SSO) THEN
     DO J = 1, Nst 
        WRITE(LUNIT,'(1x,I3,2x,A)') J, U_NAM(J)
        WRITE(LUNIT,*) ' Number of Mie run ', N_PSD(J)
        DO I = 1, LAN
           DO K = 1, NAC(I)
              IF( L_ARR(J,I,K) ) THEN
                 WRITE(LUNIT,'(5x,2(1x,I3),2x,A,2x,F10.4)') &
                      I, K, DB_NAM(I,K), MOD_R(I,K)
              ENDIF
           ENDDO  ! over components (K)
        ENDDO  ! over aerosol layers (I)                 
     ENDDO  ! over number of Mie calculations (J)
  END IF
  !******************************************************************   
  !
  !... Nst is the number of aerosol components for which 
  !    the refractive index should be read
  !
  !... Reading number of lines (wavelengths) in the file
  !-----------------------------------------------------
  DO J = 1, Nst
     H_CHA = TRIM(Path_RIC_DB)//U_NAM(J)
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,'(x,I3,2x,A)') J, TRIM(H_CHA)
     !----------------------------------------------------
     OPEN(NewUnit=IUNIT, FILE=H_CHA,STATUS='OLD',Err=105)!
     CALL SKIP_HEADER( IUNIT )  ;   K = 1                !
     DO                                                  !
        READ(IUNIT,*,ERR=102,END=300)  ;   K = K + 1     !
     ENDDO                                               !
300  CONTINUE                                            !
     CLOSE (IUNIT)                                       !
     !----------------------------------------------------
     NWL_Ri(J) = K - 1
  ENDDO
  M_wl = MAXVAL( NWL_Ri )
  ! WRITE(*,*) '  M_wl =  ',M_wl

  ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl), WL_RIa(Nst,M_wl) )

  ! Reading wavelength grid and refractive indexes
  ! ----------------------------------------------
  DO J = 1, Nst
     H_CHA = TRIM(Path_RIC_DB)//U_NAM(J)
     IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,'(x,I3,2x,A)') J, TRIM(H_CHA)
     !-------------------------------------------------------------!
     OPEN(NewUnit=IUNIT, FILE=TRIM(H_CHA),STATUS='old',Err=105)    !
     CALL SKIP_HEADER( IUNIT )                                     !
     DO K = 1, NWL_Ri(J)                                           !
        READ(IUNIT,*,ERR=102) WL_RIa(J,K), Repa(J,K), Impa(J,K)    !
     ENDDO                                                         !
! Transform imaginary part in form compatible to the Mie code      !
!        ( at input has to be positive )                           !
     Impa(J,:) = ABS( Impa(J,:) )                                  ! 
!     Impa(J,:) = -Impa(J,:)                                       !
     CLOSE (IUNIT)                                                 !
     !-------------------------------------------------------------!
  ENDDO


  !************ Start Mie calculations ***********************
  !-----------------------------------------------------------
  !... Initialize the MIE-routine with general parameters
!  Nmi  = 100 ; NKmi = 100 ; NPmi = 4 ;  NDISTR = 2 
  Nmi  = 100 ; NKmi = 50 ; NPmi = 4 ;  NDISTR = 2 
!  Nmi  = 100 ; NKmi = 10 ; NPmi = 4 ;  NDISTR = 2 
  !... Calculation precision
  DDELT = 1.d-7

  NOWR = .TRUE. 

  Ma = MAXVAL( N_PSD(:) )
  ALLOCATE( Mor(Ma), Var(Ma), CEXT(Ma), CSCA(Ma), CEXT_Dr(Ma), CEXT_Ds(Ma), & !A
       CSCA_Ds(Ma), CSCA_Dr(Ma), RI_rea(Nst,N_MIE_WL), RI_ima(Nst,N_MIE_WL) )

  RX = 2.d0*Pie*R2/MinVal(WL_MIE(:))
  L1_Max = 2*IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0) 

  ! Maximal fourth dimension of DB_EXT and DB_SCA is 3 
  ! because for each aerosol component these arrays contain
  !    1 - corresponding coefficient
  !    2 - derivative w.r.t. modal radius
  !    3 - derivative w.r.t. variance
  ALLOCATE( DB_EXT(LAN,MCO,NAW,3), DB_SCA(LAN,MCO,NAW,3),   &    ! A
       N_EC(LAN,MCO,NAW), DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3) ) ! A
  DB_EXT = 0.d0 ; DB_SCA = 0.d0 ; DB_MOMS = 0.d0 ;  N_EC = 0

  ALLOCATE( COEF1(L1_MAX), COEF2(L1_MAX), COEF3(L1_MAX), COEF4(L1_MAX), & !A
       COEF5(L1_MAX), COEF6(L1_MAX), COEF7(L1_MAX), COEF8(L1_MAX) )

  DO L1 = 3,L1_MAX 
     L = L1-1
     COEF1(L1) = 1.d0/DFLOAT(L+1)
     COEF2(L1) = DFLOAT(2*L+1)
     COEF3(L1) = 1.d0/DSQRT(DFLOAT((L+1)*(L+1)-4))
     COEF4(L1) = DSQRT(DFLOAT(L*L-4))
     COEF5(L1) = 1.d0/(DFLOAT(L)*DFLOAT((L+1)*(L+1)-4))
     COEF6(L1) = DFLOAT(2*L+1)*DFLOAT(L*(L+1))
     COEF7(L1) = DFLOAT((2*L+1)*4)
     COEF8(L1) = DFLOAT(L+1)*DFLOAT(L*L-4)
  ENDDO

  DO J = 1, Nst
     ! Number of wavelengths for refractive index
     N_RI = NWL_Ri(J)
     L = 1
     DO I = 1, LAN  ! bottom --> top
        DO K = 1, NAC(I)
           IF( L_ARR(J,I,K) ) THEN
              Mor(L) = MOD_R(I,K)
              BB = DLOG( SIG(I,K) ) ; Var(L) = BB**2 
              ! Recalculate modal radius
              IF( .NOT.Rm_log ) THEN
                 Mor(L) =  Mor(L)*DEXP( Var(L) )
              ENDIF
              L = L + 1
           ENDIF
        ENDDO  ! over aerosol components (K)
     ENDDO  ! over aerosol layers (I)

     ! Number of modal radii and variances of log-normal PSD
     Nps = N_PSD(J)
     ! WRITE(*,*) '  Number of Nps =  ', Nps

     ALLOCATE( Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max), &
          Grs_Ds(Nps,6,L1_max), L1max(Nps), AREA(Nps), VOL(Nps))
     ! DO L = 1, Nps
     !    WRITE(*,'(2x,I2,2x,F10.4,2x, F10.4)') L, Mor(L), DEXP( DSQRT(Var(L)) )
     ! ENDDO

     ! WRITE(*,'(A,I2,A,I2)') ' Component ',J,' altogether - ',Nst
     DO K1 = 1, N_MIE_WL
        ! WRITE(*,'(A,I2,A,I2)') '     Wavelength',K1,' altogether - ',NAW
        
        ! Interpolate real and imaginary part of refractive
        ! index on the user defined wavelength grid
        CALL LIN_SEARCH_NEW( WL_MIE(K1), DP, I1, I2, N_RI, WL_RIa(J,1:N_RI) )
        MRR = Repa(J,I2)*DP + Repa(J,I1)*(1.d0 - DP)
        MRI = Impa(J,I2)*DP + Impa(J,I1)*(1.d0 - DP)
        RI_rea(J,K1) = MRR
        RI_ima(J,K1) = MRI
!         WRITE(*,'(A,e12.4)')          '     Wavelength   ', WL_MIE(K1)
!         WRITE(*,'(A,1x,2(2x,e12.4))') '     Real and Imag ', MRR, MRI
        IF( PM_2i5 ) THEN
           IF(LOG_LEVEL(1) >= N_SSO) &
                WRITE(LUNIT,*) '-------------------------------------------'
           R2_PM = 2.5d0
           CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, MRI, &
                R1, R2_PM, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
                CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), &
                CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, &
                AA2, BB2, DDELT, NOWR, L1_max, PM_2i5, .FALSE.)
           L2 = 1
           DO K = 1, NAC(1)
              IF( L_ARR(J,1,K) ) THEN
                 Vo_PM(1,K)    = VOL(L2)
                 V_c(K) = VOL(L2)
                 A_c(K) = AREA(L2)
                 IF(LOG_LEVEL(1) >= N_SSO) &
                      WRITE(LUNIT,'(2(2x,I2),(2x,e12.4))') K, L2, V_c(K), A_c(K)
                 L2 = L2 + 1
              ENDIF
           ENDDO  ! over component (K)
           IF(LOG_LEVEL(1) >= N_SSO) THEN
              WRITE(LUNIT,'(A,2x,5(2x,e12.4))') '    Area: ', AREA(1:Nps)
              WRITE(LUNIT,*) 
              WRITE(LUNIT,'(A,2x,5(2x,e12.4))') '  Volume: ', VOL(1:Nps)
              WRITE(LUNIT,*) '   Nps = ', NPs,'  should be 1 !!!'
              WRITE(LUNIT,*) '------ END of "sc_mie.f" for PM_2.5 -------'
              WRITE(LUNIT,*) '-------------------------------------------'
           ENDIF
        ENDIF

        CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, MRI, R1, &
             R2, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), CSCA(1:Nps), &
             Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), CEXT_Ds(1:Nps), &
             CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, AA2, BB2, DDELT, &
             NOWR, L1_max, .FALSE., .FALSE.)

!        write(6,*) '**', CSCA(1), CEXT(1)
!        pause
        
!           IF(LOG_LEVEL(1) >= N_SSO) &
!             WRITE(LUNIT,'(A,2x,5(2x,e12.4))') '  Vo all: ', VOL(1:Nps)

        ! Save results of Mie calculations
        ! CEXT*1.e-3 --> [km-1] i.e. extinction coefficient
        ! per 1 particle/cm^3

        ! WRITE(*,*) ' ----> CEXT(L) mkm ', CEXT(1)

        IF( PM_2i5 ) THEN
           IF(LOG_LEVEL(1) >= N_SSO) &
                WRITE(LUNIT,*) '-------------------------------------------'
           L2 = 1
           DO K = 1, NAC(1)
              IF( L_ARR(J,1,K) ) THEN
                 V_f(K) = VOL(L2)
                 A_f(K) = AREA(L2)
                 IF(LOG_LEVEL(1) >= N_SSO) &
                      WRITE(LUNIT,'(2(2x,I2),(2x,e12.4))') K, L2, V_f(K), A_f(K)
                 L2 = L2 + 1
              ENDIF
           ENDDO  ! over component (K)
           IF(LOG_LEVEL(1) >= N_SSO) &
                WRITE(LUNIT,*) '-------------------------------------------'
        ENDIF

        L = 1
        ! WRITE(*,'(7x,A)') '... Redistribution over layers and components'
        DO I = 1, LAN 
           DO K = 1, NAC(I)
              IF( L_ARR(J,I,K) ) THEN
!                  WRITE(*,'(A,1x,*(2x,e13.5))') '    Calculated Cext and Csca ',&
!                     CEXT(L)*1.d-3, CSCA(L)*1.d-3, CSCA(L)/CEXT(L)
                 ! WRITE(*,*) '  I, K, L  ', I, K, L
!                  WRITE(*,*) '  L1max(L) ',L1max(L)
                 N_EC(I,K,K1)     = L1max(L)
                 DB_EXT(I,K,K1,1) = CEXT(L)*1.d-3
                 DB_SCA(I,K,K1,1) = CSCA(L)*1.d-3
                 ! WRITE(*,*)
                 ! WRITE(*,*) ' Layer   comp   Mod-r     BB        WL'
                 ! WRITE(*,'(2x,I2,3x,I2,2x,f10.4,2x,e14.6,2x,F8.3)') &
                 !    I, K, Mor(L), Var(L), WL_MIE(K1)
                 ! WRITE(*,'(17x,2(2x,e14.6))') DB_EXT(I,K,K1,1), DB_SCA(I,K,K1,1)
                 ! WRITE(*,'(20x,I4,2x,e15.7)') L1max(L), SUM( Grs(L,1:Nm_vs,1:L1max(L)) )
                 ! Expansion coefficients contain multiplier (2*L-1)
                 DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),1) = Grs(L,1:Nm_vs,1:L1max(L))
                 ! WRITE(*,'(2x,3(2x,e14.6))') CEXT_Dr(L), CSCA_Dr(L),&
                 !    SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )
                 Id = 2
                 IF( USE_Der(1) ) THEN
                    ! Save derivative Dext/Dmod 
                    DB_EXT(I,K,K1,Id) = CEXT_Dr(L)*1.d-3
                    ! Save derivative Dsca/Dmod
                    DB_SCA(I,K,K1,Id) = CSCA_Dr(L)*1.d-3
                    ! Save derivative Dcoef/Dmod
                    DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Dr(L,1:Nm_vs,1:L1max(L))
                    Id = Id + 1
                 ENDIF
                 IF( USE_Der(2) ) THEN
                    ! Save derivative Dext/Dvar 
                    DB_EXT(I,K,K1,Id) = CEXT_Ds(L)*1.d-3
                    ! Save derivative Dsca/Dvar
                    DB_SCA(I,K,K1,Id) = CSCA_Ds(L)*1.d-3
                    ! Save derivative Dcoef/Dvar
                    DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Ds(L,1:Nm_vs,1:L1max(L))
                 ENDIF
                 L = L + 1
              ENDIF
           ENDDO  ! over component (K)
        ENDDO  ! over aerosol layers (I)                
     ENDDO   ! over wavelength (K1)
     DEALLOCATE( Grs, Grs_Dr, Grs_Ds, L1max, AREA, VOL )
  ENDDO  ! over Mie calculations (J)
  
  !***************** End  of Mie calculations *************************

 ! max_db = MAXVAL( N_EC(:,:,:) )
  ! WRITE(*,*) '  DB_MOMS after reading  max_db=NL_DB ',max_db
  ! K1 = N_EC(1,1,2)
  !   WRITE(*,*) '  DB_MOMS after SC_MIE.f '
   ! WRITE(*,'(A,1x,2(2x,e12.4))') ' DB_MOMS - ', &
   !            SUM( DB_MOMS(1,NAC(1),1,:,1:max_db,2) ), &
   !    SUM( DB_MOMS(1,NAC(1),2,:,1:K1,2) )
!   DO I = 1, 20
!      WRITE(*,'(2x,I3,*(2x,e12.4))') &
!           I, DB_MOMS(1,1,1,1,I,1), DB_MOMS(1,1,1,5,I,1), DB_MOMS(1,1,1,6,I,1) 
!   ENDDO

   DEALLOCATE( COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, COEF7, COEF8 )

   IF( Nm_vs == 1 ) RETURN

   DB_MOMS(:,:,:,5,:,:) = -DB_MOMS(:,:,:,5,:,:) 
   DB_MOMS(:,:,:,6,:,:) = -DB_MOMS(:,:,:,6,:,:)
   
  RETURN
  
100 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
  GOTO 880

400 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,*) '   Option PM-2.5 mkm can be used in the case'
  WRITE(EU,*) '   of single aerosol layer only.'
  GOTO 880

401 CALL WRITE_MESSAGE(R_NAME, 'ERROR', ' ')
  WRITE(EU,*) '   Option PM-2.5 mkm can not be used if '
  WRITE(EU,*) ' "Adjust aerosol layers within main altitude grid"'
  WRITE(EU,*) '  option is selected.'
  GOTO 880
  
101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  GOTO 880
  
102 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    from file: ', TRIM(H_CHA) 
  GOTO 880
  
103 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Number of aerosol components '//&
         'is greater than maximal')
  WRITE(EU,*) '    Max number is set to: ', M_AC
  WRITE ( EU, * ) '    Increase parameter M_AC in routine "mie_aer_setup.f"'
  GOTO 880
  
104 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Number of aerosol components '//&
         'should be the same')
  WRITE(EU,*) '    in all initial aerosol layers if the option'
  WRITE(EU,*) '    "Adjust aerosol layers" is used. '
  WRITE(EU,*) '    Check aerosol input file "mie_aer.inp"'
  GOTO 880

105 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'File not found:')
  WRITE(EU,*) TRIM(H_CHA)
  GOTO 880

106 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Wrong input of file does not exist')
  WRITE(EU,*) '    Parameter: ', TRIM(STR_INP) 
  GOTO 880

  
123 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Selected humidity value not '//&
         'allowed (aerosols)')
  WRITE(EU,*) '    ', REL_HUM (LAN-I+1), ' % not allowed'
  WRITE(EU,*) '    Check aerosol input file!'
  GOTO 880
  
124 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Derivatives of aerosol optical parameters')
  WRITE(EU,*) '   can be used if the "RTM Mode" parameter is set'
  WRITE(EU,*) '   to "wf" or "ret" and weighting function for '
  WRITE(EU,*) '   "aerosol" is set to .true. '
  WRITE(EU,*) 'Check "control.inp" and "control_wf.inp" files.'
  GOTO 880
  
201 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) 'Explicit definition of bimodal PSD is allowed'
  WRITE(EU,*) '   in the case of single Mie-layer only.'
  WRITE(EU,*) 'Check aerosol input file "mie_aer.inp"'
  GOTO 880
  
202 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Aerosol component does not exist in database')
  WRITE(EU,*) '      Component short name : ', CH4
  WRITE(EU,*) '         Aerosol component : ', J
  WRITE(EU,*) '             Aerosol layer : ', I
  WRITE(EU,*) '   Check file "mie_aer.inp".'
  GOTO 880
  
203 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The aerosol course mode is not presented')
  WRITE(EU,*) '      Check control parameter "Use bimodal" and'
  WRITE(EU,*) '      "Upper boundary for fine mode" in file '
  WRITE(EU,*) '      "mie_aer.inp"'
  
204 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The aerosol spectral grid does not ' &
         //'cover the selected spectral range.')
  WRITE(EU,*) '      Check control parameter "Wavelength grid for the Mie aerosol parameterization" in "mie_aer.inp" file'
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
!***********************************************************************
END SUBROUTINE READ_MIE_AER_INP
!***********************************************************************
