!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 22 Aug 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_MOD_AER_INP(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, AER_LEV, AER_PRF, &
     LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, DBe_PM, DBs_PM, Vo_PM, *, *)

  USE AER_MIE,  ONLY: Nam_f, Nam_c, WL_RIa, Cfr, MOD_R, NAC, SIG, RD, N_PSD, L_ARR, &
                      Nst, Rmi_lim, T_550, COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, &
                      COEF7, COEF8, F_MODE, M_AC, PM_2i5_MOD
  USE AEROSOLS, ONLY: Rpar, Vpar, Ri_ab, Ri_du, AER_SHIFT, Use_der, N_Dwr, Dwr_mi, &
                      Bi_PSD, AER_SCALING
  USE I_DIMS,   ONLY: Nm_vs
  USE LOG_PARS, ONLY: EU, Work_Dir, LUNIT, N_SSO, LOG_LEVEL
  USE ST_VARS,  ONLY: GROUND_HEIGHT, Pie
  USE TOOLS,    ONLY: INQ_WF, READ_INPUT, CUT_STR, SPLIT_STRING, JOIN
  USE TYPES,    ONLY: KND, DYN_CHAR 
  
  INTEGER  NAW, LAN, N_COM, N_AER_LEV
  INTEGER, ALLOCATABLE :: N_EC(:,:,:)
  REAL (KND)  Z_BOT
  REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), DB_EXT(:,:,:,:), &
       DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:), AER_LEV(:), AER_PRF(:), &
       DBe_PM(:,:,:), DBs_PM(:,:,:), Vo_PM(:,:)

  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_MOD_AER_INP'
  CHARACTER (LEN=4)   CH4
  CHARACTER (LEN=200) H_CHA
  CHARACTER (LEN=6),   ALLOCATABLE :: U_NAM(:) 
  CHARACTER (LEN=:),   ALLOCATABLE :: STR_INP, Nam_fc, STR_LOC
  INTEGER   I, J, K, K1, I1, I2, J1, NDISTR, Nmi, NPmi, NKmi, N_sel, N_MIE_WL, M_wl, &
       Jst, Nps, Ma, L1_Max, L, L1, L2, MCO, Id
  LOGICAL   NO_COMP, NOWR, L_HEL
  REAL (KND) DP, AA, BB, GAM, LAM, MRR, MRI, R1, R2, R2_PM, RVW, RMEAN, RX, AA1, BB1, &
       AA2, BB2, DDELT, Ra, Rb, Va, Vb, Tau, V01, V02, Vd1, Vd2, N01, N02, Const,  &
       Nd1, Nd2, R1_PM
  REAL (KND), ALLOCATABLE :: Rep(:), Imp(:), WL_RI(:), Repa(:,:), Impa(:,:), AREA(:),&
       VOL(:), Mor(:), Var(:), CEXT(:), CSCA(:), CEXT_Dr(:), CSCA_Dr(:), CEXT_Ds(:), &
       CSCA_Ds(:), Grs(:,:,:), Grs_Dr(:,:,:), Grs_Ds(:,:,:)
  INTEGER, ALLOCATABLE :: L1max(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:)
  !=========================================================================     
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  ! If true the specific option of program will be started which 
  ! calculates aerosol mass for particles between 0 - 2.5 mkm
  PM_2i5_MOD = .FALSE.
!  PM_2i5_MOD = .TRUE.

  LAN = 1
  ALLOCATE( Rpar(4,2), Vpar(4,2), Ri_ab(3), Ri_du(4,4) )

  ! Reading of wavelength grid where Mie calculations are be performed
  STR_INP = 'Wavelength grid (MODIS)'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
  N_MIE_WL = SIZE(READ_ARR)
  NAW = N_MIE_WL  ;  ALLOCATE( WL_MIE(N_MIE_WL) )
  DO I = 1, N_MIE_WL
     READ(READ_ARR(I)%S,*,ERR = 301) WL_MIE(I)
  END DO
  
  ! Transform [nm] --> [micron]
  WL_MIE(:) = WL_MIE(:)*1.d-3

  M_AC = 4

  ALLOCATE( NAC(LAN), RD(LAN,M_AC), F_MODE(M_AC), MOD_R(LAN,M_AC),&
       SIG(LAN,M_AC), LTOP(LAN) )  ! A
  
  F_MODE(:) = .TRUE.

  ! Read top and bottom of aerosol layers
  STR_INP = 'Bottom and top of aerosol layer (MODIS)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=301) Z_BOT, LTOP(1)

  IF( AER_SHIFT ) THEN 
     Z_BOT   = Z_BOT + GROUND_HEIGHT
     LTOP(:) = LTOP(:) + GROUND_HEIGHT
  ENDIF
  IF( ANY(Use_der(:)).OR.N_dwr>0 ) Dwr_mi = .TRUE.
  IF( Dwr_mi ) THEN
     IF( .NOT.INQ_WF('aer', 'RIT') )    GOTO 124
     IF( Bi_PSD.AND.LAN > 1 ) GOTO 201
  ENDIF

  STR_INP = 'Aerosol type definition (MODIS)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  Nam_f = CUT_STR(STR_ARR(1)%S)
  IF( Nam_f=='Weakly_absorbing')     Nam_f = 'Weakly absorbing'
  IF( Nam_f=='Moderately_absorbing') Nam_f = 'Moderately absorbing'
  IF( Nam_f=='Strongly_absorbing')   Nam_f = 'Strongly absorbing'
  Nam_c = CUT_STR(STR_ARR(2)%S)
  IF( Nam_c=='Spheroidal_(dust)' ) Nam_c = 'Spheroidal (dust)'
  READ(STR_ARR(3)%S,*,ERR=301) T_550
  READ(STR_ARR(4)%S,*,ERR=301) Cfr
  
  ! Standard settings according to MODIS aerosol model
  NAC(1) = M_AC
  ! Number of wavelengths in MODIS aerosol model
  M_wl = 4
  Nst  = 2
  ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl), WL_RIa(M_wl) )
  ! Read parameters of fine mode (two components):
  ! ----------------------------------------------
   STR_INP = Nam_f
   CALL Read_Input (STR_INP, READ_ARR, *880)
   CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
   STR_LOC = JOIN(STR_ARR,2)
   READ(STR_LOC,*,ERR=301) (Rpar(1,K),K=1,2), (Vpar(1,K),K=1,2),&
        (Ri_ab(K),K=1,3)
   CALL SPLIT_STRING (READ_ARR(2)%S, STR_ARR)
   STR_LOC = JOIN(STR_ARR,2)
   READ(STR_LOC,*,ERR=301) (Rpar(2,K),K=1,2), (Vpar(2,K),K=1,2),&
        (Ri_ab(K),K=1,3)

   ! Read parameters of coarse mode (two components):
   ! ------------------------------------------------
   F_MODE(3:4) = .FALSE.
   STR_INP = Nam_c
   CALL Read_Input (STR_INP, READ_ARR, *880)
   CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
   STR_LOC = JOIN(STR_ARR,2)
   READ(STR_LOC,*,ERR=301) (Rpar(3,K),K=1,2), (Vpar(3,K),K=1,2)
   CALL SPLIT_STRING (READ_ARR(2)%S, STR_ARR)
   STR_LOC = JOIN(STR_ARR,2)
   READ(STR_LOC,*,ERR=301) (Rpar(4,K),K=1,2), (Vpar(4,K),K=1,2)
   DO I = 1, 4
      READ(READ_ARR(I+2)%S,*,ERR=301) WL_RIa(I),(Ri_du(I,J),J=1,4)
   ENDDO

   ! Calculate depending on TAU modal radius and variance (fine):
   Tau = T_550
   SELECT CASE( TRIM(Nam_f) )
   CASE('Strongly absorbing')
      IF( T_550 > 2.d0 )  Tau = 2.d0
      V01 = 0.1748d0 *T_550**0.8914d0
      V02 = 0.1043d0 *T_550**0.6824d0
   CASE('Moderately absorbing')
      IF( T_550 > 2.d0 )  Tau = 2.d0
      V01 = 0.1642d0 *T_550**0.7747d0
      V02 = 0.1482d0 *T_550**0.6846d0
   CASE('Weakly absorbing')
      IF( T_550 > 1.d0 )  Tau = 1.d0
      V01 = 0.1718d0 *T_550**0.8213d0 
      V02 = 0.0934d0 *T_550**0.6394d0
      ! V02 = 0.d0
   END SELECT

   DO I = 1,2
      MOD_R(1,I) = Rpar(I,1) + Rpar(I,2)*Tau
      SIG  (1,I) = Vpar(I,1) + Vpar(I,2)*Tau
   ENDDO
   Const = 3.d0/4.d0/Pie
   ! Particles number of first and second components        
   N01 = V01*Const/MOD_R(1,1)**3*DEXP(9.d0/2.d0*SIG(1,1)**2)
   N02 = V02*Const/MOD_R(1,2)**3*DEXP(9.d0/2.d0*SIG(1,2)**2)
   RD(1,1) = N01/(N01 + N02)
   RD(1,2) = N02/(N01 + N02)

   ! WRITE(*,*) '    Fraction within "fine" mode'
   ! WRITE(*,*) RD(1,1), RD(1,2)
   
   RD(1,1:2) = RD(1,1:2)*(1.d0 - Cfr)

   ! WRITE(*,*) '  Part. number (absorbing mode) '
   ! WRITE(*,*) N01 + N02

   ! Wavelength independent refractive index for absorbing 
   Repa (1,:) = Ri_ab(1) ; Impa (1,:) = Ri_ab(2) + Ri_ab(3)*Tau

   ! Calculate depending on TAU modal radius and variance (coarse):
   Tau = T_550
   IF( T_550 > 1.d0 )  Tau = 1.d0        
   DO I = 3,4
      MOD_R(1,I) = Rpar(I,1)* Tau**Rpar(I,2)
      SIG  (1,I) = Vpar(I,1)* Tau**Vpar(I,2)
   ENDDO
   Vd1 = 0.0871d0 *T_550**1.0260d0
   Vd2 = 0.6786d0 *T_550**1.0569d0
   Nd1 = Vd1*Const/MOD_R(1,3)**3*DEXP(9.d0/2.d0*SIG(1,3)**2)
   Nd2 = Vd2*Const/MOD_R(1,4)**3*DEXP(9.d0/2.d0*SIG(1,4)**2)
   RD(1,3) = Nd1/(Nd1 + Nd2)
   RD(1,4) = Nd2/(Nd1 + Nd2)
   RD(1,3:4) = RD(1,3:4)*Cfr

   ! Refractive index for dust at 4 reference wavelengths
   DO K = 1,M_wl
      Repa (2,K) = Ri_du(K,1)*Tau**Ri_du(K,2) 
      Impa (2,K) = Ri_du(K,3)*Tau**Ri_du(K,4)
   ENDDO

  STR_INP  = 'Numerical integration limits (MODIS)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=301) R1, R2        
  Rmi_lim(1) = R1 ; Rmi_lim(2) = R2

  ! Read aerosol number density profile
  STR_INP  = 'Aerosol particle number density (MODIS)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_AER_LEV = SIZE(READ_ARR)
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(N_AER_LEV), AER_PRF(N_AER_LEV) ) ! A
  DO I = 1,N_AER_LEV
     READ(READ_ARR(I)%S,*) AER_LEV(I), AER_PRF(I)
  END DO
        
  AER_PRF(:) = AER_SCALING*AER_PRF(:)
  
  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT

  ! Check if the course mode is really presented
  IF( COUNT(.NOT.F_MODE(:))==0 ) GOTO 203

  ! If Bi_PSD is set to .TRUE., i.e., the bimodal PSD 
  ! will be used and derivatives with respect to PSD parameters
  ! of the fine and course modes should be calculated
        !!! N_dwr is set to 0 in "gt_fileread.f" !!!
  
  !  WRITE(*,'(2x,A,I2)') '      N_Dwr ', N_Dwr
  !  WRITE(*,*) '  Bi_PSD ' , Bi_PSD
  !  WRITE(*,*) '  USE_Der ', USE_Der

  !  IF( Bi_PSD ) THEN           
  !!    ... derivatives w.r.t. modal radii of fine and course modes. 
  !     IF( USE_Der(1) ) N_Dwr = N_Dwr + 2
  !!    ... derivatives w.r.t. variances of fine and course modes. 
  !     IF( USE_Der(2) ) N_Dwr = N_Dwr + 2
  !!    ... derivatives w.r.t. the fraction of course mode number
  !!        density
  !     N_Dwr = N_Dwr + 1
  !  ELSE
  !!    ... Number of derivatives which will be used in computations:
  !!        the modal radius and/or variance
  !     N_Dwr = COUNT( Use_der(:) )
  !  ENDIF
  
  !!     ... Test version 
  !      WRITE(*,*) '   **************************************    '
  !      WRITE(*,*) '      Message from "mod_aer_setup.f"         '
  !      WRITE(*,*) '    !!! Aerosol density is changed !!!     '
  !      WRITE(*,*) '   **************************************    '
  !!      AER_PRF(:) = 0d0
  !      AER_PRF(:) = AER_PRF(:)*0.67114
  !!      S2 = 5.d+3/AER_PRF(1)
  !!      AER_PRF(:) = AER_PRF(:)*S2
  

  ! Number of selected aerosol components:
  N_sel = SUM( NAC(:) )

  ALLOCATE( L_ARR(N_sel,LAN,N_sel), U_NAM(N_sel) )
  L_ARR = .FALSE.
 
  Nst = 2
  ! Both component of absorbing mode have the sane refractive index
  L_ARR(1,1,1:2) = .TRUE.
  ! Both component of dust mode have the sane refractive index
  L_ARR(2,1,3:4) = .TRUE.

  ALLOCATE( N_PSD(Nst) )
  DO J = 1, Nst             ! Number of Mie calculations
     N_PSD(J) = COUNT(L_ARR(J,:,:))
  ENDDO

  MCO = MAXVAL( NAC(:) )
  ! Maximal fourth dimension of DB_EXT and DB_SCA is 3 
  ! because for each aerosol component these arrays contain
  ! 1 - corresponding coefficient
  ! 2 - derivative w.r.t. modal radius
  ! 3 - derivative w.r.t. variance
  ALLOCATE( DB_EXT(LAN,MCO,NAW,3), DB_SCA(LAN,MCO,NAW,3), N_EC(LAN,MCO,NAW) ) !A
  DB_EXT = 0.d0 ; DB_SCA = 0.d0

  IF( PM_2i5_MOD ) THEN
     ALLOCATE( DBe_PM(LAN,MCO,NAW), DBs_PM(LAN,MCO,NAW), Vo_PM(LAN,MCO) ) 
     DBe_PM = 0.d0 ;  DBs_PM = 0.d0 ;  Vo_PM = 0.d0
  ENDIF
  !********* Start Mie ************************************
  ! Initialize the MIE-routine with general parameters
  Nmi  = 100 ; NKmi = 100 ; NPmi = 4 ;  NDISTR = 2 
  ! Calculation precision
  DDELT = 1.d-7
  ! No screen output
  NOWR = .TRUE. 
!  NOWR = .FALSE. 
  ! Preliminary we consider dust as spherical also !!!
  Ma = 4
  ALLOCATE( Mor(Ma), Var(Ma), CEXT(Ma), CSCA(Ma), CEXT_Dr(Ma), CEXT_Ds(Ma), & !A
       CSCA_Ds(Ma), CSCA_Dr(Ma) )

  RX = 2.d0*Pie*R2/WL_MIE(1)
  L1_Max = 2*IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)

  ! The last dimension of array DB_MOMS is set to 3.
  ! The same reason as by DB_EXT and DB_SCA
  ALLOCATE( DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3) )
  DB_MOMS = 0.d0

  ALLOCATE( COEF1(L1_MAX), COEF2(L1_MAX), COEF3(L1_MAX), COEF4(L1_MAX), &
       COEF5(L1_MAX), COEF6(L1_MAX), COEF7(L1_MAX), COEF8(L1_MAX) )

  DO L1 = 3,L1_MAX 
     L = L1-1
     COEF1(L1) = 1.d0/DFLOAT(L+1)
     COEF2(L1) = DFLOAT(2*L+1)
     COEF3(L1) = 1.d0/DSQRT(DFLOAT((L+1)*(L+1)-4))
     COEF4(L1) = DSQRT(DFLOAT(L*L-4))
     COEF5(L1) = 1.d0/(DFLOAT(L)*DFLOAT((L+1)*(L+1)-4))
     COEF6(L1) = DFLOAT(2*L+1)*DFLOAT(L*(L+1))
     COEF7(L1) = DFLOAT((2*L+1)*4)
     COEF8(L1) = DFLOAT(L+1)*DFLOAT(L*L-4)
  ENDDO

!!  WRITE(*,*) '   ********  Test output  ********    '
!  OPEN( 49,file='./DATA_OUT/Aer-PSD.par' )
!  WRITE(49,'(2x,F6.3,4(2x,F6.3),2x,F6.3)') T_550, MOD_R(1,1), SIG(1,1), &
!       MOD_R(1,2), SIG(1,2), V01/(V01 + V02)
!  CLOSE(49)
  
  N_EC = 0
  
  DO J = 1, Nst
     ! No Mie calculation for course mode if Cfr equal zero
     IF( Cfr==0.d0.AND.J==2 ) CYCLE
     L = 1
     DO K = 1, NAC(1)
        IF( L_ARR(J,1,K) ) THEN
           BB = SIG(1,K) ; Var(L) = BB**2 
           Mor(L) = MOD_R(1,K)*DEXP( -3.d0*BB**2 )
           L = L + 1
        ENDIF
     ENDDO
     ! Number of modal radii and variances of log-normal PSD
     Nps = N_PSD(J)
     ! WRITE(*,*) '  Number of Nps =  ', Nps

     ALLOCATE( Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max), Grs_Ds(Nps,6,L1_max), &
          L1max(Nps), AREA(Nps), VOL(Nps) )

     IF(LOG_LEVEL(1) >= N_SSO) THEN
        WRITE(LUNIT,'(A,I2,A,I2)') ' Component ',J,' altogether - ',Nst
        WRITE(LUNIT,'(A,I2)')       ' Nps =  ', Nps
        WRITE(LUNIT,'(a,2(2x,f8.3))')  '  Mod_r ', Mor(1:Nps)
        WRITE(LUNIT,'(a,2(2x,f8.3))')  '    Var ', DSQRT(Var(1:Nps))
     END IF
        
     ! Over wavelengths where Mie calculations should be performed
     DO K1 = 1, N_MIE_WL
        IF(LOG_LEVEL(1) >= N_SSO) THEN
           WRITE(LUNIT,'(A,I2,A,I2)') '     Wavelength',K1,' altogether - ',NAW
           WRITE(LUNIT,'(A)') '     ----------'
        END IF

        ! Interpolate real and imaginary part of refractive
        ! index on the user defined wavelength grid
        CALL LIN_SEARCH_NEW( WL_MIE(K1), DP, I1, I2, M_wl, WL_RIa )
        MRR = Repa(J,I2)*DP + Repa(J,I1)*(1.d0 - DP)
        MRI = Impa(J,I2)*DP + Impa(J,I1)*(1.d0 - DP)
        ! WRITE(*,'(A,e12.4)') '       Real part: ', MRR
        ! WRITE(*,'(A,e12.4)') '  Imaginary part: ', MRI
        IF( PM_2i5_MOD ) THEN
           ! WRITE(*,*) '-------------------------------------------'
           R2_PM = 0.5d0 ! for fine mode only
!           R2_PM = R2
           CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, &
                MRI, R1, R2_PM, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
                CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), &
                CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, & 
                AA2, BB2, DDELT, NOWR, L1_max, .FALSE., PM_2i5_MOD)
           L2 = 1
           DO K = 1, NAC(1)
              IF( L_ARR(J,1,K) ) THEN
                 DBe_PM(1,K,K1) = CEXT(L2)*1.d-3
                 DBs_PM(1,K,K1) = CSCA(L2)*1.d-3
                 Vo_PM(1,K)    = VOL(L2)
                 WRITE(*,'(17x,2x,e14.6)')  DBe_PM(1,K,K1)*1.d-3, &
                      DBs_PM(1,K,K1)*1.d-3
                 ! WRITE(*,'(2(2x,I2),(2x,e12.4))') K, L2, VO_PM(1,K)
                 L2 = L2 + 1
              ENDIF
           ENDDO  ! over component (K)
           
           IF(LOG_LEVEL(1) >= N_SSO) THEN
              WRITE(LUNIT,*) '       Area: '
              WRITE(LUNIT,'(2x,5(2x,e12.4))') AREA(1:Nps)
              WRITE(LUNIT,*) '     Volume: '
              WRITE(LUNIT,'(2x,5(2x,e12.4))') VOL(1:Nps)
              WRITE(LUNIT,*) '------ END of "sc_mie.f" for PM_2.5_MOD -------'
              WRITE(LUNIT,*) '-------------------------------------------'
           ENDIF
        ENDIF

        R1_PM = R1
        IF( PM_2i5_MOD ) R1_PM = R2_PM ! for coarse mode only 
        CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, MRI,  &
             R1_PM, R2, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
             CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps),  & 
             CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1,  &
             AA2, BB2, DDELT, NOWR, L1_max, .FALSE., PM_2i5_MOD )

        IF(LOG_LEVEL(1) >= N_SSO) THEN
           WRITE(LUNIT,*) '       Area: '
           WRITE(LUNIT,'(2x,5(2x,e12.4))') AREA(1:Nps)
           WRITE(LUNIT,*) '     Volume: '
           WRITE(LUNIT,'(2x,5(2x,e12.4))') VOL(1:Nps)
        ENDIF
        ! Save results of Mie calculations
        ! CEXT*1.e-3 --> [km-1] i.e. extinction coefficient
        ! per 1 particle/cm^3
        L = 1
        DO K = 1, NAC(1)
           IF( L_ARR(J,1,K) ) THEN
              ! WRITE(*,*) '  K, L  ', K, L
              ! WRITE(*,*) '  L1max(L) ',L1max(L)
              N_EC(1,K,K1)     = L1max(L)
              DB_EXT(1,K,K1,1) = CEXT(L)*1.d-3
              DB_SCA(1,K,K1,1) = CSCA(L)*1.d-3
              ! WRITE(*,*)
              ! WRITE(*,*) ' Comp   Mod-r     BB        WL'
              ! WRITE(*,'(3x,I2,2x,f10.4,2x,e14.6,2x,F8.3)') &
              !     K, Mor(L), Var(L), WL_MIE(K1)
              ! WRITE(*,'(17x,A,15x,A,13x,A)') &
              !     '    Ext', 'Sca','SSA'
               WRITE(*,'(17x,2(2x,e14.6),2x,F8.3)') &
                   DB_EXT(1,K,K1,1), DB_SCA(1,K,K1,1), &
                   DB_SCA(1,K,K1,1)/DB_EXT(1,K,K1,1)
              ! WRITE(*,'(20x,I4,2x,e15.7)')L1max(L),Grs(L,1,2)/3.&
              !  SUM( Grs(L,1:Nm_vs,1:L1max(L)) )
              ! Expansion coefficients contain multiplier (2*L-1)
              DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),1) = Grs(L,1:Nm_vs,1:L1max(L))            
              ! WRITE(*,'(2x,3(2x,e14.6))') CEXT_Dr(L), CSCA_Dr(L),&
              !   SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )
              Id = 2
              IF( USE_Der(1) ) THEN
                 ! Save derivative Dext/Dmod 
                 DB_EXT(1,K,K1,Id) = CEXT_Dr(L)*1.d-3
                 ! Save derivative Dsca/Dmod
                 DB_SCA(1,K,K1,Id) = CSCA_Dr(L)*1.d-3
                 ! Save derivative Dcoef/Dmod
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Dr(L,1:Nm_vs,1:L1max(L))
                 Id = Id + 1
              ENDIF
              IF( USE_Der(2) ) THEN
                 ! Save derivative Dext/Dvar 
                 DB_EXT(1,K,K1,Id) = CEXT_Ds(L)*1.d-3
                 ! Save derivative Dsca/Dvar
                 DB_SCA(1,K,K1,Id) = CSCA_Ds(L)*1.d-3
                 ! Save derivative Dcoef/Dvar
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Ds(L,1:Nm_vs,1:L1max(L))
              ENDIF
              L = L + 1
           ENDIF
        ENDDO  ! over component (K)
     ENDDO  ! over wavelength (K1)
     DEALLOCATE( Grs, Grs_Dr, Grs_Ds, L1max, AREA, VOL )
  ENDDO  ! over Mie calculations (J)

 
  DEALLOCATE( COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, COEF7, COEF8 )

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
102 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    from file: ', TRIM(H_CHA) 
  GOTO 880
  
124 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Derivatives of aerosol optical parameters')
  WRITE(EU,*) '   can be used if the "RTM Mode" parameter is set'
  WRITE(EU,*) '   to "wf" or "ret" and weighting function for '
  WRITE(EU,*) '   "aerosol" is set to .true. '
  WRITE(EU,*) 'Check "control.inp" and "control_wf.inp" files.'
  GOTO 880
  
201 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) 'Explicit definition of bimodal PSD is allowed'
  WRITE(EU,*) '   in the case of single Mie-layer only.'
  WRITE(EU,*) 'Check aerosol input file "mod_aer.inp"'
  GOTO 880
  
202 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Aerosol component does not exist in database')
  WRITE(EU,*) '      Component short name : ', CH4
  WRITE(EU,*) '         Aerosol component : ', J
  WRITE(EU,*) '             Aerosol layer : ', I
  WRITE(EU,*) '   Check file "mod_aer.inp".'
  GOTO 880
  
203 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The aerosol course mode is not presented')
  WRITE(EU,*) '      Check control parameter "Use bimodal" and'
  WRITE(EU,*) '      "Upper boundary for fine mode" in file '
  WRITE(EU,*) '      "mod_aer.inp"'
  
301 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP
  GOTO 880

880 WRITE(EU, *) ' ---> Called by '//R_NAME
  
  CALL ST_TRACK (R_NAME, 1, 'O') 
  RETURN 1
  
!***********************************************************************
END SUBROUTINE READ_MOD_AER_INP
!***********************************************************************

