!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 22 Aug 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_MOD_AER_MAN(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, AER_LEV, &
     AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, DBe_PM, DBs_PM, Vo_PM, *, *)
  ! Called by 'mie_aer_setup.f'

  USE AER_MIE,  ONLY: WL_RIa, Cfr, MOD_R, NAC, SIG, T_550, RD, N_PSD, L_ARR, &
                      Nst, Rmi_lim, Nam_f, COEF1, COEF2, COEF3, COEF4, COEF5, &
                      COEF6, COEF7, COEF8, F_MODE, M_AC, PM_2i5, PM_2i5_MOD
  USE AEROSOLS, ONLY: AER_SHIFT, Use_der, Bi_PSD, AER_SCALING
  USE I_DIMS,   ONLY: Nm_vs
  USE LOG_PARS, ONLY: EU, Work_Dir, LUNIT, LOG_LEVEL, N_SSO
  USE ST_VARS,  ONLY: GROUND_HEIGHT, Pie
  USE TOOLS,    ONLY: READ_INPUT, CUT_STR, SPLIT_STRING
  USE TYPES,    ONLY: KND, DYN_CHAR 
  
  INTEGER  NAW, LAN, N_COM, N_AER_LEV
  INTEGER, ALLOCATABLE :: N_EC(:,:,:)
  REAL (KND)  Z_BOT
  REAL (KND), ALLOCATABLE :: WL_MIE(:), LTOP(:), DB_EXT(:,:,:,:), &
       DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:), AER_LEV(:), AER_PRF(:), &
       DBe_PM(:,:,:), DBs_PM(:,:,:), Vo_PM(:,:)
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_MOD_AER_MAN'
  CHARACTER (LEN=:),   ALLOCATABLE :: STR_INP
  INTEGER   I, J, K, K1, I1, I2, J1, KK, NDISTR, Nmi, NPmi, NKmi, N_sel, &
       N_MIE_WL, M_wl, Jst, Nps, Ma, L1_Max, L, L1, L2, MCO, Id
  LOGICAL   NOWR
  REAL (KND) BB, GAM, LAM, MRR, MRI, R1, R2, RVW, RMEAN, RX, AA1, BB1, &
       AA2, BB2, DDELT, R1_PM, R2_PM
  REAL (KND), ALLOCATABLE :: Repa(:,:), Impa(:,:), Mor(:), Var(:), CEXT(:), &
       CSCA(:), CEXT_Dr(:), CSCA_Dr(:), CEXT_Ds(:), CSCA_Ds(:), Grs(:,:,:), &
       Grs_Dr(:,:,:), Grs_Ds(:,:,:), AREA(:), VOL(:)
  INTEGER, ALLOCATABLE :: L1max(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:)
  !=========================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')      

  PM_2i5_MOD = .FALSE.
!  PM_2i5_MOD = .TRUE.

  ! Only single aerosol layer is allowed in this approximation
  LAN = 1

  ! Reading input file 'mod_aer_man.inp'
  ! Reading of wavelength grid where Mie calculations should be performed
  STR_INP = 'Wavelength grid (MODIS Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
  N_MIE_WL = SIZE(READ_ARR)
  NAW = N_MIE_WL  ;  ALLOCATE( WL_MIE(N_MIE_WL) )
  DO I = 1, N_MIE_WL
     READ(READ_ARR(I)%S,*,ERR = 301) WL_MIE(I)
  END DO
  ! Transform [nm] --> [micron]
  WL_MIE(:) = WL_MIE(:)*1.d-3
  
  ! Full number of aerosol components (fine + coarse)
  M_AC = 2
  ALLOCATE( NAC(LAN), RD(LAN,M_AC), F_MODE(M_AC), MOD_R(LAN,M_AC), & !A
       SIG(LAN,M_AC), LTOP(LAN) )  !A
  
  ! Read top and bottom of aerosol layers
  STR_INP = 'Bottom and top of aerosol layer (MODIS Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=301) Z_BOT, LTOP(1)
  
  IF( AER_SHIFT ) THEN 
     Z_BOT   = Z_BOT + GROUND_HEIGHT
     LTOP(:) = LTOP(:) + GROUND_HEIGHT
  ENDIF
  
  STR_INP = 'Aerosol type definition (MODIS Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  Nam_f =CUT_STR(STR_ARR(1)%S)
  READ(STR_ARR(2)%S,*,ERR=301) T_550
  
  ! Standard settings according to MODIS ocean aerosol model
  NAC(1) = M_AC
  ! Number of wavelengths where refractive index is defined
  ! We use here formal M_wl=1 because in this version
  ! the refractive index is wavelength independent
  M_wl = 1  ;  Nst  = 2
  ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl) )
  STR_INP  = 'Numerical integration limits (MODIS Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=301) R1, R2        
  Rmi_lim(1) = R1 ; Rmi_lim(2) = R2
  
  ! Read aerosol number density profile
  STR_INP  = 'Aerosol particle number density (MODIS Mie)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_AER_LEV = SIZE(READ_ARR)
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(N_AER_LEV), AER_PRF(N_AER_LEV) ) ! A
  KK = 0
  DO I = 1,N_AER_LEV
     KK = KK + 1
     READ(READ_ARR(KK)%S,*) AER_LEV(I), AER_PRF(I)
  END DO
  AER_PRF(:) = AER_SCALING*AER_PRF(:)
  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT
  ! ************************ End of Reading *****************************
  
  CALL MAKE_AER_PAR(T_550, M_AC, Cfr, Repa(:,1), Impa(:,1), MOD_R, SIG, Nam_f)
  
  IF(LOG_LEVEL(1) >= N_SSO) THEN
     WRITE(LUNIT,*) '   ----> "read_mod_aer_man.f" '
     WRITE(LUNIT,*) '   Optical thickness ', T_550
     WRITE(LUNIT,*) '   Modal radius and variance: '
     WRITE(LUNIT,'(4x,2(1x,F6.3))') MOD_R(1,1), SIG  (1,1)
     WRITE(LUNIT,'(4x,2(1x,F6.3))') MOD_R(1,2), SIG  (1,2)
     WRITE(LUNIT,*) '   Real and imaginary part: '
     WRITE(LUNIT,'(4x,2(1x,F6.3))') Repa(1,1), Impa  (1,1)
     WRITE(LUNIT,'(4x,2(1x,F6.3))') Repa(2,1), Impa  (2,1)
     WRITE(LUNIT,*) '   Fraction: ', Cfr
  END IF
  
  ! Bi_PSD is set to .TRUE. for this aerosol approximation (see 'gt_fileread.f90')
  IF( Cfr == 0.d0 ) Bi_PSD = .FALSE.
  RD(1,1) = 1.d0 - Cfr  ;  RD(1,2) = Cfr
  F_MODE(1) = .TRUE. ;  F_MODE(2) = .FALSE.
  
  ! Number of selected aerosol components:
  N_sel = SUM( NAC(:) )

  ALLOCATE( L_ARR(N_sel,LAN,N_sel) )
  L_ARR = .FALSE.
  L_ARR(1,1,1) = .TRUE.
  L_ARR(1,1,2) = .FALSE.
  L_ARR(2,1,1) = .FALSE.
  L_ARR(2,1,2) = .TRUE.
  
  Nst = 2
  ALLOCATE( N_PSD(Nst) )
  DO J = 1, Nst             ! Number of Mie calculations
     N_PSD(J) = COUNT(L_ARR(J,:,:))
  ENDDO
  
  MCO = MAXVAL( NAC(:) )
  ! Maximal fourth dimension of DB_EXT and DB_SCA is 3 
  ! because for each aerosol component these arrays contain
  ! 1 - corresponding coefficient
  ! 2 - derivative w.r.t. modal radius
  ! 3 - derivative w.r.t. variance
  ALLOCATE( DB_EXT(LAN,MCO,NAW,3), DB_SCA(LAN,MCO,NAW,3), N_EC(LAN,MCO,NAW) ) !A
  DB_EXT = 0.d0 ; DB_SCA = 0.d0

  IF( PM_2i5_MOD ) THEN
     ALLOCATE( DBe_PM(LAN,MCO,NAW), DBs_PM(LAN,MCO,NAW), Vo_PM(LAN,MCO) ) 
     DBe_PM = 0.d0 ;  DBs_PM = 0.d0 ;  Vo_PM = 0.d0
  ENDIF
  !********* Start Mie ************************************
  !... Initialize the MIE-routine with general parameters
  Nmi  = 100 ; NKmi = 100 ; NPmi = 4 ;  NDISTR = 2 
  ! Calculation precision
  DDELT = 1.d-7
  ! No screen output
  NOWR = .TRUE. 
  ! Preliminary we consider dust as spherical also !!!
  Ma = 4
  ALLOCATE( Mor(Ma), Var(Ma), CEXT(Ma), CSCA(Ma), CEXT_Dr(Ma), CEXT_Ds(Ma), & !A
       CSCA_Ds(Ma), CSCA_Dr(Ma) )

  RX = 2.d0*Pie*R2/WL_MIE(1)
  L1_Max = 2*IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)

  ! The last dimension of array DB_MOMS is set to 3.
  ! The same reason as by DB_EXT and DB_SCA
  ALLOCATE( DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3) )
  DB_MOMS = 0.d0

  ALLOCATE( COEF1(L1_MAX), COEF2(L1_MAX), COEF3(L1_MAX), COEF4(L1_MAX), & !A
       COEF5(L1_MAX), COEF6(L1_MAX), COEF7(L1_MAX), COEF8(L1_MAX) )

  DO L1 = 3,L1_MAX 
     L = L1-1
     COEF1(L1) = 1.d0/DFLOAT(L+1)
     COEF2(L1) = DFLOAT(2*L+1)
     COEF3(L1) = 1.d0/DSQRT(DFLOAT((L+1)*(L+1)-4))
     COEF4(L1) = DSQRT(DFLOAT(L*L-4))
     COEF5(L1) = 1.d0/(DFLOAT(L)*DFLOAT((L+1)*(L+1)-4))
     COEF6(L1) = DFLOAT(2*L+1)*DFLOAT(L*(L+1))
     COEF7(L1) = DFLOAT((2*L+1)*4)
     COEF8(L1) = DFLOAT(L+1)*DFLOAT(L*L-4)
  ENDDO

  N_EC = 0
  
  DO J = 1, Nst
     ! No Mie calculation for course mode if Cfr equal zero
     IF( Cfr==0.d0.AND.J==2 ) CYCLE
     L = 1
     DO K = 1, NAC(1)
        IF( L_ARR(J,1,K) ) THEN
           BB = SIG(1,K) ; Var(L) = BB**2 
           ! Modal radius for volume to MR for number density:
           Mor(L) = MOD_R(1,K)*DEXP( -3.d0*BB**2 )
           L = L + 1
        ENDIF
     ENDDO
     ! Number of modal radii and variances of log-normal PSD
     Nps = N_PSD(J)
     ! WRITE(*,*) '  Number of Nps =  ', Nps
     
     ALLOCATE( Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max), &
          Grs_Ds(Nps,6,L1_max), L1max(Nps), AREA(Nps), VOL(Nps))

     IF(LOG_LEVEL(1) >= N_SSO) THEN
        WRITE(LUNIT,'(A,I2,A,I2)') ' Component ',J,' altogether - ',Nst
        WRITE(LUNIT,'(A,I2)')      ' Nps    =  ', Nps
        WRITE(LUNIT,'(a,2(2x,f8.3))')  ' Mod_r ', Mor(1:Nps)
        WRITE(LUNIT,'(a,2(2x,f8.3))')  '   Var ', DSQRT(Var(1:Nps))
     END IF
      
      ! Over wavelengths where Mie calculations should be performed
     DO K1 = 1, N_MIE_WL
        WRITE(*,'(A,I2,A,I2)') '     Wavelength',K1,' altogether - ',NAW
        WRITE(*,'(A)') '     ----------'
        ! No wavelength dependence in this parameterization
        MRR = Repa(J,1)  ;   MRI = Impa(J,1)
        ! WRITE(*,'(A,e12.4)') '       Real part: ', MRR
        ! WRITE(*,'(A,e12.4)') '  Imaginary part: ', MRI
        IF( PM_2i5_MOD ) THEN
           ! WRITE(*,*) '-------------------------------------------'
           R2_PM = 0.5d0 ! for fine mode only
!           R2_PM = R2
           CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, &
                MRI, R1, R2_PM, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
                CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), &
                CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, & 
                AA2, BB2, DDELT, NOWR, L1_max, .FALSE., PM_2i5_MOD)
           L2 = 1
           DO K = 1, NAC(1)
              IF( L_ARR(J,1,K) ) THEN
                 DBe_PM(1,K,K1) = CEXT(L2)*1.d-3
                 DBs_PM(1,K,K1) = CSCA(L2)*1.d-3
                 Vo_PM(1,K)    = VOL(L2)
                 WRITE(*,'(17x,2x,e14.6)')  DBe_PM(1,K,K1)*1.d-3, &
                      DBs_PM(1,K,K1)*1.d-3
                 ! WRITE(*,'(2(2x,I2),(2x,e12.4))') K, L2, VO_PM(1,K)
                 L2 = L2 + 1
              ENDIF
           ENDDO  ! over component (K)
           
           IF(LOG_LEVEL(1) >= N_SSO) THEN
              WRITE(LUNIT,*) '       Area: '
              WRITE(LUNIT,'(2x,5(2x,e12.4))') AREA(1:Nps)
              WRITE(LUNIT,*) '     Volume: '
              WRITE(LUNIT,'(2x,5(2x,e12.4))') VOL(1:Nps)
              WRITE(LUNIT,*) '------ END of "sc_mie.f" for PM_2.5_MOD -------'
              WRITE(LUNIT,*) '-------------------------------------------'
           ENDIF
        ENDIF

        R1_PM = R1
        IF( PM_2i5_MOD ) R1_PM = R2_PM ! for coarse mode only 
        CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, MRI, &
             R1_PM, R2, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
             CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), &
             CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, &
             AA2, BB2, DDELT, NOWR, L1_max, .FALSE., PM_2i5_MOD )

        ! Save results of Mie calculations
        ! CEXT*1.e-3 --> [km-1] i.e. extinction coefficient
        ! per 1 particle/cm^3
        L = 1
        DO K = 1, NAC(1)

           IF( L_ARR(J,1,K) ) THEN
              ! WRITE(*,*) '  K, L  ', K, L
              ! WRITE(*,*) '  L1max(L) ',L1max(L)
              N_EC(1,K,K1)     = L1max(L)
              DB_EXT(1,K,K1,1) = CEXT(L)*1.d-3
              DB_SCA(1,K,K1,1) = CSCA(L)*1.d-3

              ! WRITE(*,*)
              ! WRITE(*,*) ' Comp   Mod-r     BB        WL'
              ! WRITE(*,'(3x,I2,2x,f10.4,2x,e14.6,2x,F8.3)') &
              !     K, Mor(L), DSQRT(Var(L)), WL_MIE(K1)
              ! WRITE(*,'(17x,A,15x,A,13x,A)') &
              !    '    Ext', 'Sca','SSA'
              ! WRITE(*,'(17x,2(2x,e14.6),2x,F9.4)') &
              !     DB_EXT(1,K,K1,1), DB_SCA(1,K,K1,1), &
              !    DB_SCA(1,K,K1,1)/DB_EXT(1,K,K1,1)
              ! WRITE(*,'(20x,I4,2x,e15.7)')L1max(L),Grs(L,1,2)/3. &
              ! SUM( Grs(L,1:Nm_vs,1:L1max(L)) )
              !... Expansion coefficients contain multiplier (2*L-1)

              DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),1) = Grs(L,1:Nm_vs,1:L1max(L))            
              ! WRITE(*,'(2x,3(2x,e14.6))') &
              !     CEXT_Dr(L), CSCA_Dr(L), &
              !     SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )
              Id = 2
              IF( USE_Der(1) ) THEN
                 ! Save derivative Dext/Dmod 
                 DB_EXT(1,K,K1,Id) = CEXT_Dr(L)*1.d-3
                 ! Save derivative Dsca/Dmod
                 DB_SCA(1,K,K1,Id) = CSCA_Dr(L)*1.d-3
                 ! Save derivative Dcoef/Dmod
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Dr(L,1:Nm_vs,1:L1max(L))
                 Id = Id + 1
              ENDIF
              IF( USE_Der(2) ) THEN
                 ! Save derivative Dext/Dvar 
                 DB_EXT(1,K,K1,Id) = CEXT_Ds(L)*1.d-3
                 ! Save derivative Dsca/Dvar
                 DB_SCA(1,K,K1,Id) = CSCA_Ds(L)*1.d-3
                 ! Save derivative Dcoef/Dvar
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Ds(L,1:Nm_vs,1:L1max(L))
              ENDIF
              L = L + 1
           ENDIF
        ENDDO  ! over component (K)
     ENDDO  ! over wavelength (K1)
     DEALLOCATE( Grs, Grs_Dr, Grs_Ds, L1max, AREA, VOL)
  ENDDO  ! over Mie runs (J)

  
  DEALLOCATE( COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, COEF7, COEF8 )

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

301 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  GOTO 880
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  
  CALL ST_TRACK (R_NAME, 1, 'O') 
  RETURN 1
  
!***********************************************************************
END SUBROUTINE READ_MOD_AER_MAN
!***********************************************************************

