!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 27 Jan 2022
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE READ_MOD_AER_OC(NAW, WL_MIE, LAN, Z_BOT, N_AER_LEV, AER_LEV, &
     AER_PRF, LTOP, N_EC, DB_EXT, DB_SCA, DB_MOMS, *, *)
  ! Called by 'mie_aer_setup.f'

  USE AER_MIE,     ONLY: WL_RIa, Cfr, MOD_R, NAC, SIG, RI_rea, RI_ima, RD, &
                         N_PSD, L_ARR, Nst, Rmi_lim, COEF1, COEF2, COEF3, &
                         COEF4, COEF5, COEF6, COEF7, COEF8, F_MODE, M_AC, PM_2i5
  USE AEROSOLS,    ONLY: Rpar, Vpar, AER_SHIFT, MODIS_LO, Use_der, N_Dwr, &
                         Dwr_mi, Bi_PSD, AER_SCALING
  USE INTERN_DIMS, ONLY: Nm_vs
  USE LOG_PARS,    ONLY: EU, Work_Dir
  USE ST_INPUT,    ONLY: ADB => PATH_AER_DB
  USE ST_VARS,     ONLY: GROUND_HEIGHT, Pie
  USE TOOLS,       ONLY: INQ_WF, READ_Input, CUT_STR, SPLIT_STRING
  USE TYPES,       ONLY: KND, DYN_CHAR 

  INTEGER  NAW, LAN, N_COM, N_AER_LEV
  INTEGER, ALLOCATABLE :: N_EC(:,:,:)
  REAL (KND)  Z_BOT
  REAL (KND), ALLOCATABLE :: WL_MIE(:), REL_HUM(:), LTOP(:), DB_EXT(:,:,:,:), &
       DB_SCA(:,:,:,:), DB_MOMS(:,:,:,:,:,:), AER_LEV(:), AER_PRF(:)

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'READ_MOD_AER_OC'
  CHARACTER (LEN=:),   ALLOCATABLE :: Nam_f, Nam_c, STR_INP
  INTEGER   I, J, K, K1, I1, I2, J1, KK, NDISTR, Nmi, NPmi, NKmi, N_sel, &
       N_MIE_WL, M_wl, Jst, Nps, Ma, L1_Max, L, L1, MCO, Id, CUNIT
  LOGICAL   NO_COMP, NOWR, L_HEL
  REAL (KND) DP, AA, BB, GAM, LAM, MRR, MRI, R1, R2, RVW, RMEAN, RX, AA1, &
       BB1, AA2, BB2, DDELT
  REAL (KND), ALLOCATABLE :: WL_RI(:), Repa(:,:), Impa(:,:), Mor(:), Var(:), &
       CEXT(:), CSCA(:), CEXT_Dr(:), CSCA_Dr(:), CEXT_Ds(:), CSCA_Ds(:), &
       Grs(:,:,:), Grs_Dr(:,:,:), Grs_Ds(:,:,:), AREA(:), VOL(:)
  INTEGER, ALLOCATABLE :: L1max(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:), READ_ARR(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')      

  ! Only single aerosol layer is allowed
  LAN = 1
  ALLOCATE( Rpar(4,2), Vpar(4,2) )

  ! Reading of wavelength grid where Mie calculations should be performed
  STR_INP  = 'Wavelength grid (AER_OC)'
  CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange=.TRUE.)
  N_MIE_WL = SIZE(READ_ARR)
  NAW = N_MIE_WL  
  ALLOCATE( WL_MIE(N_MIE_WL) )
  DO I = 1, N_MIE_WL
     READ(READ_ARR(I)%S,*,ERR = 100) WL_MIE(I)
  END DO

  ! Transform [nm] --> [micron]
  WL_MIE(:) = WL_MIE(:)*1.d-3

  ! Full number of aerosol components (fine + coarse)
  M_AC = 2

  ALLOCATE( NAC(LAN), RD(LAN,M_AC), F_MODE(M_AC), MOD_R(LAN,M_AC), & !A
       SIG(LAN,M_AC), LTOP(LAN) )  ! A

  F_MODE(1) = .TRUE. ;  F_MODE(2) = .FALSE.

  ! Read top and bottom of aerosol layers
  STR_INP = 'Bottom and top of aerosol layer (AER_OC)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=100) Z_BOT, LTOP(1)

  IF( AER_SHIFT ) THEN 
     Z_BOT   = Z_BOT + GROUND_HEIGHT
     LTOP(:) = LTOP(:) + GROUND_HEIGHT
  ENDIF
  IF( ANY(Use_der(:)).OR.N_dwr>0 ) Dwr_mi = .TRUE.
  IF( Dwr_mi ) THEN
     IF( .NOT.INQ_WF('aer', 'RIT') )    GOTO 124
     IF( Bi_PSD.AND.LAN > 1 ) GOTO 201
  ENDIF

  STR_INP = 'Aerosol type definition (AER_OC)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  Nam_f = CUT_STR(STR_ARR(1)%S)
  Nam_c = CUT_STR(STR_ARR(2)%S)
  READ(STR_ARR(3)%S,*,ERR=100) Cfr

  IF( Cfr == 0.d0 ) Bi_PSD = .FALSE.

  ! Standard settings according to MODIS ocean aerosol model
  NAC(1) = M_AC
  ! Number of wavelengths in MODIS aerosol model
  M_wl = 3
  Nst  = 2
  ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl), WL_RIa(M_wl), RI_rea(Nst,N_MIE_WL),&
       RI_ima(Nst,N_MIE_WL) )

  ! Reading parameters of fine node
  OPEN(NewUnit=CUNIT, file=TRIM(ADB)//Nam_f, STATUS = 'OLD' )
  CALL SKIP_HEADER( CUNIT )
  READ(CUNIT,*) MOD_R(1,1), SIG(1,1), M_wl
  DO I = 1, M_WL
     READ(CUNIT,*) WL_RIa(I), Repa(1,I), Impa (1,I)
  ENDDO
  Impa (1,:) = ABS( Impa (1,:) )
!  Impa (1,:) = -Impa (1,:)
  CLOSE(CUNIT)
  ! Reading parameters of coarse node
  OPEN(NewUnit=CUNIT, file=TRIM(ADB)//Nam_c, STATUS = 'OLD' )
  CALL SKIP_HEADER( CUNIT )
  READ(CUNIT,*) MOD_R(1,2), SIG(1,2), M_wl
  DO I = 1, M_WL
     READ(CUNIT,*) WL_RIa(I), Repa(2,I), Impa (2,I)
  ENDDO
  Impa (2,:) = ABS( Impa (2,:) )
!  Impa (2,:) = -Impa (2,:)
  CLOSE(CUNIT)

  RD(1,1) = 1.d0 - Cfr  ;  RD(1,2) = Cfr

  STR_INP = 'Numerical integration limits (AER_OC)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S,*,ERR=100) R1, R2        
  Rmi_lim(1) = R1 ; Rmi_lim(2) = R2

  ! Read aerosol number density profile
  STR_INP  = 'Particle number density profile (AER_OC)'
  CALL Read_Input (STR_INP, READ_ARR, *880)
  N_AER_LEV = SIZE(READ_ARR)
  IF(ALLOCATED(AER_LEV)) DEALLOCATE(AER_LEV, AER_PRF)
  ALLOCATE( AER_LEV(N_AER_LEV), AER_PRF(N_AER_LEV) ) ! A
  DO I = 1,N_AER_LEV
     READ(READ_ARR(I)%S,*) AER_LEV(I), AER_PRF(I)
  END DO
  AER_PRF(:) = AER_SCALING*AER_PRF(:)

  IF( AER_SHIFT ) AER_LEV(:) = AER_LEV(:) + GROUND_HEIGHT
  ! ************************ End of Reading *****************************

  ! Check if the course mode is really presented
  IF( COUNT(.NOT.F_MODE(:))==0 ) GOTO 203
  ! If Bi_PSD is set to .TRUE., i.e., the bimodal PSD will be used and
  ! derivatives with respect to PSD parameters of the fine and course modes
  ! should be calculated
  !!! N_dwr is set to 0 in "gt_fileread.f" !!!
  ! WRITE(*,'(2x,A,I2)') '      N_Dwr ', N_Dwr
  ! WRITE(*,*) '  Bi_PSD ' , Bi_PSD
  ! WRITE(*,*) '  USE_Der ', USE_Der

  ! Number of selected aerosol components:
  N_sel = SUM( NAC(:) )

  ALLOCATE( L_ARR(N_sel,LAN,N_sel) )
  L_ARR = .FALSE. ; L_ARR(1,1,1) = .TRUE. ; L_ARR(2,1,2) = .TRUE.

  Nst = 2
  ALLOCATE( N_PSD(Nst) )
  DO J = 1, Nst             ! Number of Mie calculations
     N_PSD(J) = COUNT(L_ARR(J,:,:))
  ENDDO

  MCO = MAXVAL( NAC(:) )
  ! Maximal fourth dimension of DB_EXT and DB_SCA is 3 
  ! because for each aerosol component these arrays contain
  ! 1 - corresponding coefficient
  ! 2 - derivative w.r.t. modal radius
  ! 3 - derivative w.r.t. variance
  ALLOCATE( DB_EXT(LAN,MCO,NAW,3), DB_SCA(LAN,MCO,NAW,3), N_EC(LAN,MCO,NAW) ) ! A
  DB_EXT = 0.d0 ; DB_SCA = 0.d0

  ! ********* Start Mie ************************************
  ! Initialize the MIE-routine with general parameters
  Nmi  = 100 ; NKmi = 100 ; NPmi = 4 ;  NDISTR = 2 
  ! Calculation precision
  DDELT = 1.d-7

  NOWR = .TRUE. 
  ! Preliminary we consider dust as spherical also !!!
  Ma = 4
  ALLOCATE( Mor(Ma), Var(Ma), CEXT(Ma), CSCA(Ma), CEXT_Dr(Ma), CEXT_Ds(Ma), & !A
       CSCA_Ds(Ma), CSCA_Dr(Ma) )

  RX = 2.d0*Pie*R2/WL_MIE(1)
  L1_Max = 2*IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)

  ! The last dimension of array DB_MOMS is set to 3.
  ! The same reason as by DB_EXT and DB_SCA
  ALLOCATE( DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3) )
  DB_MOMS = 0.d0

  ALLOCATE( COEF1(L1_MAX), COEF2(L1_MAX), COEF3(L1_MAX), COEF4(L1_MAX), & !A
       COEF5(L1_MAX), COEF6(L1_MAX), COEF7(L1_MAX), COEF8(L1_MAX) )

  DO L1 = 3,L1_MAX 
     L = L1-1
     COEF1(L1) = 1.d0/DFLOAT(L+1)
     COEF2(L1) = DFLOAT(2*L+1)
     COEF3(L1) = 1.d0/DSQRT(DFLOAT((L+1)*(L+1)-4))
     COEF4(L1) = DSQRT(DFLOAT(L*L-4))
     COEF5(L1) = 1.d0/(DFLOAT(L)*DFLOAT((L+1)*(L+1)-4))
     COEF6(L1) = DFLOAT(2*L+1)*DFLOAT(L*(L+1))
     COEF7(L1) = DFLOAT((2*L+1)*4)
     COEF8(L1) = DFLOAT(L+1)*DFLOAT(L*L-4)
  ENDDO
  
  N_EC = 0
  DO J = 1, Nst
     ! No Mie calculation for course mode if Cfr equal zero
     IF( Cfr==0.d0.AND.J==2 ) CYCLE
     L = 1
     DO K = 1, NAC(1)
        IF( L_ARR(J,1,K) ) THEN
           BB = SIG(1,K) ; Var(L) = BB**2 
           IF( TRIM(MODIS_LO)=='land' ) THEN
              Mor(L) = MOD_R(1,K)*DEXP( -3.d0*BB**2 )
           ELSE
              Mor(L) = MOD_R(1,K)
           ENDIF
           L = L + 1
        ENDIF
     ENDDO
     ! Number of modal radii and variances of log-normal PSD
     Nps = N_PSD(J)
     ! WRITE(*,*) '  Number of Nps =  ', Nps
     
     ALLOCATE( Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max), & !A
          Grs_Ds(Nps,6,L1_max), L1max(Nps), AREA(Nps), VOL(Nps))

     ! WRITE(*,'(A,I2,A,I2)') ' Component ',J,' altogether - ',Nst
     ! WRITE(*,'(A,I2)')      ' Nps    =  ', Nps
     ! WRITE(*,'(a,2(2x,f8.3))')  ' Mod_r ', Mor(1:Nps)
     ! WRITE(*,'(a,2(2x,f8.3))')  '   Var ', DSQRT(Var(1:Nps))

     ! Over wavelengths where Mie calculations should be performed
     DO K1 = 1, N_MIE_WL
        WRITE(*,'(A,I2,A,I2)') '     Wavelength',K1,' altogether - ',NAW
        WRITE(*,'(A)') '     ----------'

        ! Interpolate real and imaginary part of refractive
        ! index on the user defined wavelength grid
        CALL LIN_SEARCH_NEW( WL_MIE(K1), DP, I1, I2, M_wl, WL_RIa )
        MRR = Repa(J,I2)*DP + Repa(J,I1)*(1.d0 - DP)
        MRI = Impa(J,I2)*DP + Impa(J,I1)*(1.d0 - DP)

        RI_rea(J,K1) = MRR
        RI_ima(J,K1) = MRI

        ! WRITE(*,'(A,e12.4)') '       Real part: ', MRR
        ! WRITE(*,'(A,e12.4)') '  Imaginary part: ', MRI

        CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, WL_MIE(K1), MRR, MRI, &
             R1, R2, Nmi, NPmi, NDISTR, NKmi, L1max, Grs, CEXT(1:Nps), &
             CSCA(1:Nps), Grs_Dr, Grs_Ds, CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), &
             CEXT_Ds(1:Nps), CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, &
             BB1, AA2, BB2, DDELT, NOWR, L1_max, PM_2i5, .FALSE.)

        ! Save results of Mie calculations
        ! CEXT*1.e-3 --> [km-1] i.e. extinction coefficient
        ! per 1 particle/cm^3
        L = 1
        DO K = 1, NAC(1)

           IF( L_ARR(J,1,K) ) THEN
              ! WRITE(*,*) '  K, L  ', K, L
              ! WRITE(*,*) '  L1max(L) ',L1max(L)
              N_EC(1,K,K1)     = L1max(L)
              DB_EXT(1,K,K1,1) = CEXT(L)*1.d-3
              DB_SCA(1,K,K1,1) = CSCA(L)*1.d-3

              ! WRITE(*,*)
              ! WRITE(*,*) ' Comp   Mod-r     BB        WL'
              ! WRITE(*,'(3x,I2,2x,f10.4,2x,e14.6,2x,F8.3)') K, Mor(L), DSQRT(Var(L)), WL_MIE(K1)
              ! WRITE(*,'(17x,A,15x,A,13x,A)') '    Ext', 'Sca','SSA'
              ! WRITE(*,'(17x,2(2x,e14.6),2x,F9.4)') DB_EXT(1,K,K1,1), DB_SCA(1,K,K1,1),&
              !     DB_SCA(1,K,K1,1)/DB_EXT(1,K,K1,1)
              ! WRITE(*,'(20x,I4,2x,e15.7)')L1max(L),Grs(L,1,2)/3.,SUM( Grs(L,1:Nm_vs,1:L1max(L)) )
              ! Expansion coefficients contain multiplier (2*L-1)

              DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),1) = Grs(L,1:Nm_vs,1:L1max(L))            
              ! WRITE(*,'(2x,3(2x,e14.6))') CEXT_Dr(L), CSCA_Dr(L), &
              !    SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )
              Id = 2
              IF( USE_Der(1) ) THEN
                 ! Save derivative Dext/Dmod 
                 DB_EXT(1,K,K1,Id) = CEXT_Dr(L)*1.d-3
                 ! Save derivative Dsca/Dmod
                 DB_SCA(1,K,K1,Id) = CSCA_Dr(L)*1.d-3
                 ! Save derivative Dcoef/Dmod
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Dr(L,1:Nm_vs,1:L1max(L))
                 Id = Id + 1
              ENDIF
              IF( USE_Der(2) ) THEN
                 ! Save derivative Dext/Dvar 
                 DB_EXT(1,K,K1,Id) = CEXT_Ds(L)*1.d-3
                 ! Save derivative Dsca/Dvar
                 DB_SCA(1,K,K1,Id) = CSCA_Ds(L)*1.d-3
                 ! Save derivative Dcoef/Dvar
                 DB_MOMS(1,K,K1,1:Nm_vs,1:L1max(L),Id) = Grs_Ds(L,1:Nm_vs,1:L1max(L))
              ENDIF
              L = L + 1
           ENDIF
        ENDDO  ! over component (K)
     ENDDO  ! over wavelength (K1)
     DEALLOCATE( Grs, Grs_Dr, Grs_Ds, L1max, AREA, VOL)
  ENDDO  ! over Mie runs (J)

  DEALLOCATE( COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, COEF7, COEF8 )

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

100 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP)
  GOTO 880
  

124 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Derivatives of aerosol optical parameters')
  WRITE(EU,*) '   can be used if the "RTM Mode" parameter is set'
  WRITE(EU,*) '   to "wf" or "ret" and weighting function for '
  WRITE(EU,*) '   "aerosol" is set to .true. '
  WRITE(EU,*) 'Check "control.inp" and "control_wf.inp" files.'
  GOTO 880

201 CALL WRITE_MESSAGE(R_NAME,'ERROR',' ')
  WRITE(EU,*) 'Explicit definition of bimodal PSD is allowed'
  WRITE(EU,*) '   in the case of single Mie-layer only.'
  WRITE(EU,*) 'Check aerosol input file "mod_aer.inp"'
  GOTO 880


203 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'The aerosol course mode is not presented')
  WRITE(EU,*) '      Check control parameter "Use bimodal" and'
  WRITE(EU,*) '      "Upper boundary for fine mode" in file '
  WRITE(EU,*) '      "mod_aer.inp"'

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)

  CALL ST_TRACK (R_NAME, 1, 'O') 
  RETURN 1
  
!***************************************************************************
END SUBROUTINE READ_MOD_AER_OC
!***************************************************************************
      
