C***********************************************************************
C
C       SCIATRAN - Version 3.5 - Last modification: 16 Apr 2015
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE READ_OPAC_NSPH(DB_NAM, LAN, M_AC, N_sel, Path_RIC_DB, 
     $     WL_MIE, NAW, OEXT, OSCA, N_gau, M_cof, Leg_cof, *)
c     ... Called by "read_mie_aer_inp.f"

      USE LOG_PARS,    ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_SSO, 
     $                       N_DSO, Work_Dir
      USE ST_VARS,     ONLY: Pie
      USE TYPES,       ONLY: KND
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) :: M_AC, LAN, N_sel, NAW, N_gau
      CHARACTER(LEN=6)  DB_NAM(LAN,M_AC)
      CHARACTER(LEN=*)  Path_RIC_DB
      INTEGER  M_cof(N_sel,NAW)
      REAL (KND), INTENT(IN) :: WL_MIE(NAW)
      REAL (KND)  Leg_cof(N_gau,N_sel,NAW), OEXT(NAW,N_sel),
     $     OSCA(NAW,N_sel)
c     ... Local variables
c     ------------------------------------------------------------------
      CHARACTER (LEN = 14), PARAMETER :: R_NAME = 'READ_OPAC_NSPH'
      INTEGER  I, J, K, L
      INTEGER  O_NWL, O_Nang
      INTEGER, PARAMETER :: IUNIT = 33
      CHARACTER (LEN=200) H_CHA
      REAL (KND), ALLOCATABLE :: OWL(:), hEXT(:), hSCA(:), Ang(:), 
     $     Phm(:,:,:), TMP(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')      
      
c     ... Reading number of wavelengths in OPAC files
      H_CHA = TRIM(Path_RIC_DB)//DB_NAM(1,1)
      WRITE(*,'(2x,A)') TRIM(H_CHA)
c     ----------------------------------------------------
      OPEN(IUNIT, FILE=TRIM(H_CHA),STATUS='OLD')         !
         CALL SKIP_HEADER( IUNIT )  ;   K = 1            !
         DO                                              !
            READ(IUNIT,*,ERR=102,END=300)  ;   K = K + 1 !
         ENDDO                                           !
 300     CONTINUE                                        !
      CLOSE (IUNIT)                                      !
c     ----------------------------------------------------
      O_NWL = K - 1

c     ... Reading number of scattering angles in OPAC files
      WRITE(*,'(2x,A)') TRIM(H_CHA)//'-phm'
c     ----------------------------------------------------
      OPEN(IUNIT, FILE=TRIM(H_CHA)//'-phm',STATUS='OLD') !
         CALL SKIP_HEADER( IUNIT )  ;   K = 1            !
         DO                                              !
            READ(IUNIT,*,ERR=102,END=301)  ;   K = K + 1 !
         ENDDO                                           !
 301     CONTINUE                                        ! 
      CLOSE (IUNIT)                                      !
c     ----------------------------------------------------
      O_Nang = K - 1

      WRITE(*,*) '  Nwl_OP  =  ', O_NWL
      WRITE(*,*) '  Nang_OP =  ', O_Nang

      ALLOCATE( OWL(O_NWL), hEXT(O_NWL), hSCA(O_NWL), 
     $     Ang(O_Nang), Phm(O_Nang,N_sel,NAW), TMP(O_NWL) )

c     ... Reading of extinction and scattering coefficients
c         It is assumed preliminary that LAN = 1
      DO I = 1, LAN
         DO K = 1,N_sel
            H_CHA = TRIM(Path_RIC_DB)//DB_NAM(I,K)
            WRITE(*,'(2x,A)') TRIM(H_CHA)
c           ------------------------------------------------------------
c           ... Reading extinction and scattering coefficient [km-1]
            OPEN(IUNIT, FILE=TRIM(H_CHA),STATUS='OLD') !
            CALL SKIP_HEADER( IUNIT )                  !
            DO J = 1, O_NWL
               READ(IUNIT,*) OWL(J), hEXT(J), hSCA(J)
            ENDDO  ! over wavelengths in data base (J)
c           ... Interpolation on  the Mie-reference wave-grid
            CALL LIN_INT(O_NWL,OWL,hEXT, NAW,WL_MIE,OEXT(:,K),'C')
            CALL LIN_INT(O_NWL,OWL,hSCA, NAW,WL_MIE,OSCA(:,K),'C')
c           ------------------------------------------------------------

            WRITE(*,'(2x,A)') TRIM(H_CHA)//'-phm'
c           ... Reading volume phase function [km-1]
c           ------------------------------------------------------
            OPEN(IUNIT, FILE=TRIM(H_CHA)//'-phm',STATUS='OLD')   !
            CALL SKIP_HEADER( IUNIT )                            !
            DO J = 1, O_Nang                                     !
               READ(IUNIT,*) Ang(J), (TMP(L), L=1,O_NWL)         !
c              ... Recalculate to the scattering function        !
               TMP(:) = TMP(:)/hSCA(:)/4.d0/Pie                  !
c              ... Interpolation of the scattering function on   !
c                  the Mie-reference wave-grid                   !
               CALL LIN_INT(O_NWL, OWL, TMP,                     !
     $              NAW, WL_MIE, Phm(J,K,:), 'C')                !
            ENDDO  ! over scattering angles in data base (J)     !
c           ------------------------------------------------------

         ENDDO  ! over non-spherical components (K)
      ENDDO  ! over aerosol layers (I)
      WRITE(*,*) '   First and last wavelengths: '
      WRITE(*,*)  OWL(1), OWL(O_NWL)
      WRITE(*,*)  Oext(1,1), Oext(NAW,1)

      WRITE(*,*) '   First and last scattering angle: '
      WRITE(*,*) ' and Phm at first Mie wavelength grid value' 
      WRITE(*,*)  Ang(1), Ang(O_Nang)
      WRITE(*,*)  Phm(1,1,1), Phm(O_Nang,1,1)
      
c     ... Expansion coefficients of the phase functions
c         This routine is located in "st_tools.f"
      CALL LEG_COEFF(N_sel, NAW, N_gau, O_Nang, Ang, Phm,
     $     M_cof, Leg_cof, *880)
      

      CALL ST_TRACK (R_NAME, 1, 'O') 
      RETURN

 102  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
      WRITE(EU,*) '    from file: ', TRIM(H_CHA) 
      GOTO 880

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE READ_OPAC_NSPH
