!********************************************************************
!
!       SCIATRAN - Version 4.7 - Last modification: 30 Nov 2023
!
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!********************************************************************
SUBROUTINE READ_UWT(*)
  ! Called by "gt_fileread.f90"
  
  USE BRDF,      ONLY: Lin_rel, RefW_r, RefW_i, Ref_type, Nwl_BRF, MP_gp, MP_bub, &
                       Iref_name
  USE DELTA_M,   ONLY: DO_HSOL_DM, HSOL_TRUNC_IDX
  USE HYDROSOLS, ONLY: HS_SCALING, Scal_HS, Rwl_HS, Rtau_HS, N_HPC,&
                       N_HM, HSOL_MOD, SEL_VSC
  USE LOG_PARS,  ONLY: EU, Work_Dir, C_UNIT, LUNIT, USE_SHAD
  USE SLAB_2L,   ONLY: TES_2L_SLAB, Ome0, Ref_idx, Taus, NSTR_T1
  USE ST_INPUT,  ONLY: DEP_NAME, PROF_UW_NAME, CHL_CONC_NAME, PWAC_NAME, &
                       PSI_NAME, CHL_NAME
  USE ST_VARS,   ONLY: SPECULAR, Gauss_Field, Pie, NSTRMS, C_SYS
  USE TOOLS,     ONLY: SET_PATH, Read_Input, SPLIT_STRING, CUT_STR, Join, &
                       NotFound
  USE TYPES,     ONLY: KND, DYN_CHAR
  USE VRS,       ONLY: STEP_VRS, Nsig, Integr_FRF, Tvs_Exc, Tvs_Act, Tvs_Use,&
                       Add_TRS, Rho, Lam_ref, Dnu_FRF, A_FRF, K_nu, Bet_0R,&
                       Lam_int, Contour, WLG_Chl, Sig_Chl, Qe_Chl, WL_QEred,&
                       WL_QEblu
  USE WATER,     ONLY: VSF_pure_model, W_speed, SS_PDF, Par_PDF, OC_DEP, &
                       NLAYERS_UW, OC_LAYERING, Ncol_UW, NC_UW, DO_ABSUW_AKIMA,&
                       N_ABS_UW, A_RTmat, NC_CHL, NSTR_T, DEPOL_WATER,&
                       DEP_W_INPUT, DEP_W_FILENAME, NLEG_W, NSZA_W, NNv_W, &
                       ALB_BOTTOM, DO_RTAU_W, RTAU_W, Ref_ind, PW_ABS, &
                       OUT_WLR, NO_COUPLING, INCL_INEL, Sol_off, INEL_DO, &
                       INEL_ADJ, WL_Flu, Step_Flu, FH_REF, Fa_NAME, Ha_NAME,&
                       Fu_par, FH_EXTR, Nwl_QE, WL_QE, A_QE, A_Fa, OB_TYPE, &
                       B_Fa, A_Ha, B_Ha, N_FC, Scal_FC, Nwl_QEx, Nwl_QEm, &
                       WL_QEx, WL_QEm, F_QEx, F_QEm, Conc_NAME, RWG, Ref_name,&
                       UIT, MAX_ITER_OC, Absw_chl, Absw_pw, CDOM_Chl, CDOM_ACR,&
                       Absw_ful, Absw_hum, Absw_other, Hu_par, Flu_CDOM, &
                       TS_CORR, Wei_dif, ERDR
  USE WATER_ICE, ONLY: IWS, IWT, N_IW, LoIW, ICE_LAY_NUM, L_TOP_I, L_BOT_I, &
                       LA_N => LA_N_IW, Na_ice
  
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'READ_UWT'
  CHARACTER (LEN=1)   HDL
  CHARACTER (LEN=200) STG, STR_READ, STR_READ_ARR(90)
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP, ADD_STR, CH_INP, Exci_NAME, &
       Emi_NAME, DIR1, DIR2, NAM1, NAM2, DOM_ADJ, STR_LOC, DIR_name
  CHARACTER (LEN=100)  Msg(10)
  LOGICAL    Loc_DO(3), Loc_ADJ(3), LoD
  INTEGER    I, K, II, II_AD, KK, N_INE, N_WL, Na_W, I_READ
  REAL (KND) Inp_val, Shift, Cof, NuR, N_fac, WLS1, WLS2, WLE1, WLE2, DUMM
  REAL (KND), ALLOCATABLE :: Scal_Em(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:), STR_ARR(:), OPTION(:), FH_QE(:), &
       ADD_2(:)
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 3, 'I')

! Preliminary initialization in gt_fileread.f
! -------------------------------------------------------
  Scal_HS    = .FALSE.
  HS_SCALING = 1.d0         ! Scaling factor
  Rwl_HS     = 412.d0       ! Reference wavelength
  Rtau_HS    = 1.d0         ! Optical thickness at ref. wavelength
  DO_HSOL_DM = .TRUE.       ! Delta-M approximation switch on
!      HSOL_TRUNC_IDX ! hydrosol truncation index is defined in 
!                      "gt_settings_check.f" as NLEGEN - 1.
!    ...........................................................
  SET_ADD: Block
    IF( UIT ) THEN
       ADD_STR = ' (ice)'  ! Radiative transfer within ice (control_uit.inp)
       IF(.NOT.IWT) EXIT SET_ADD
    ENDIF
    ADD_STR = ''  ! Radiative transfer within water or sea ice (control_uwt.inp)
  END Block SET_ADD
       
  ALLOCATE( ADD_2(2) )
  ADD_2(1)%S =  ' (ice)'
  ADD_2(2)%S =  ''
  I_READ = 2
!  write(*,*)  '   IWT and UIT ', IWT, UIT
  IF( IWT.OR.UIT ) THEN
!    ... Reading from the "cf_ice_man.inp" file the information about position
!        of an ice layer within water 
     STR_INP = 'Layering of scattering function/matrix (cai)'
!     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     CALL Read_Input (STR_INP, READ_ARR, *880)
     LA_N = SIZE(READ_ARR)     
!     WRITE(*,*) '   Number of ice layers ', LA_N
     IF( N_IW==1.AND.LA_N > 1 ) GOTO 215 ! atmosphere-ice system
     ALLOCATE( L_TOP_I(LA_N), L_BOT_I(LA_N), ICE_LAY_NUM(LA_N),&
          IWS(1)%IW_TOP(LA_N), IWS(1)%IW_BOT(LA_N), IWS(1)%Na_W(LA_N), &
          IWS(2)%IW_TOP(LA_N), IWS(2)%IW_BOT(LA_N) )
!     write(*,*) '   ALLOCXATE amd set to zero  IWS(1)%Na_W(:) '
!     PAUSE
     IWS(1)%Na_W(:) = 0
     DO I = 1,LA_N
        CALL SPLIT_STRING(READ_ARR(I)%S, STR_ARR)
        READ(STR_ARR(1)%S,*,ERR=101) L_TOP_I(I)
        READ(STR_ARR(2)%S,*,ERR=101) L_BOT_I(I)
        READ(STR_ARR(3)%S,*,ERR=101) ICE_LAY_NUM(I)
     ENDDO
!    ... Ice or water medium will be used to define lower boundary 
!        condition for RT through atmosphere     
     IF( L_TOP_I(1)==0.d0 ) THEN
        I_READ = 1 ! ice at the bottom of atmosphere
     ELSE
       I_READ = 2 ! water at the bottom of atmosphere
    ENDIF
 ELSE
!    write(*,*) '    LA_N = ', LA_N
    LA_N = 0 ; ALLOCATE(IWS(1)%Na_W(1))
    !ALLOCATE(IWS(1)%Na_W(LA_N))
    IWS(1)%Na_W(:) = 0
 ENDIF  ! IWT.OR.UIT
 
 IF( .NOT.ALLOCATED(IWS(1)%IW_BOT) ) THEN
    ALLOCATE(IWS(1)%IW_TOP(1), IWS(1)%IW_BOT(1), &
         IWS(2)%IW_TOP(1), IWS(2)%IW_BOT(1) )
 ENDIF
! Setting interface conditions depending on ice or water is air-interface
! -----------------------------------------------------------------------
  STR_INP = 'Flat or wind-roughed ocean surface'//ADD_2(I_READ)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CH_INP = CUT_STR(READ_ARR(1)%S)
  SELECT CASE( CH_INP )
  CASE('flat')
     SPECULAR = .TRUE.
  CASE('roughed')
     SPECULAR = .FALSE.
  CASE DEFAULT
     CALL NotFound(R_NAME, STR_INP, CH_INP,*880)
  END SELECT

  If(Allocated(Lin_rel)) DeAllocate(Lin_rel)
  ALLOCATE( Lin_rel(2) )
  STR_INP = 'Mean square slope and wind speed'//ADD_2(I_READ)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) Lin_rel(1), Lin_rel(2), W_speed

  IF( .NOT.SPECULAR ) THEN
! ===>  Wimd roughed surface:
     STR_INP = 'Approximation of RT matrixes'//ADD_2(I_READ)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
!     A_RTmat = CUT_STR(READ_ARR(1)%S)
     CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
     A_RTmat = CUT_STR( STR_ARR(1)%S )
     READ(STR_ARR(2)%S,*,ERR=101) Wei_dif
     READ(STR_ARR(3)%S,*,ERR=101) ERDR
     IF( .NOT.ERDR.AND.Wei_dif<1.d0 ) GOTO 208
     SELECT CASE( TRIM(A_RTmat) )
     CASE('NT_ICF','FT','NT_ECF')
     CASE DEFAULT
        CALL NotFound(R_NAME, STR_INP, A_RTmat, *880)
     END SELECT
     IF( I_READ==2 ) THEN
        STR_INP = 'Use shadowing factor (AOS)?'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        USE_SHAD = CUT_STR(READ_ARR(1)%S)
        SELECT CASE(USE_SHAD)
        CASE('yes','no')
        CASE DEFAULT
           CALL NOT_FOUND (R_NAME, STR_INP, USE_SHAD, *880)
        END SELECT
     ELSE
!       ... No shadowing in the case of air-ice interface
        USE_SHAD = 'no'
     ENDIF
  ELSE
! ===> Specular reflection:
     A_RTmat = 'NT_ECF'
  ENDIF

! -------------------- Coupling atmosphere ice/water --------------------
  STR_INP = 'Coupling'//ADD_2(2)%S
  IF( LoIW(1) ) STR_INP = 'Coupling'//ADD_2(1)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  SELECT CASE( STR_ARR(1)%S )
  CASE('off')
     NO_COUPLING = .TRUE. ;  MAX_ITER_OC = 1
  CASE('on')
     NO_COUPLING = .FALSE.
     READ(STR_ARR(2)%S, *, ERR = 101) MAX_ITER_OC
  CASE DEFAULT
     CALL NotFound(R_NAME, STR_INP, STR_ARR(1)%S, *880)
  END SELECT
! -----------------------------------------------------------------------

! -------------------- Forward model: marine absorber -------------------
  IWS(:)%Absw_pw    = .FALSE. ; IWS(:)%Absw_chl   = .FALSE.
  IWS(:)%CDOM_Chl   = .FALSE. ; IWS(:)%CDOM_ACR   = .FALSE.
  IWS(:)%Absw_ful   = .FALSE. ; IWS(:)%Absw_hum   = .FALSE.
  IWS(:)%Absw_other = .FALSE.
  DO I = 1,2
!     WRITE(*,*) '  ', I, LoIW(I), ADD_2(I)%S
     IF( LoIW(I) ) THEN
        STR_INP = 'Forward model: marine absorber'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S, *, ERR=101) Absw_pw, Absw_chl, CDOM_Chl, & 
             CDOM_ACR, Absw_ful, Absw_hum, Absw_other
        IWS(I)%Absw_pw    = Absw_pw
        IWS(I)%Absw_chl   = Absw_chl
        IWS(I)%CDOM_Chl   = CDOM_Chl
        IWS(I)%CDOM_ACR   = CDOM_ACR
        IWS(I)%Absw_ful   = Absw_ful
        IWS(I)%Absw_hum   = Absw_hum
        IWS(I)%Absw_other = Absw_other
     ENDIF
     IF( IWS(1)%Absw_chl ) GOTO 210
     IF( IWS(1)%CDOM_Chl ) GOTO 211
     IF( IWS(1)%CDOM_ACR ) GOTO 212
     IF( IWS(1)%Absw_ful.OR.IWS(1)%Absw_hum ) GOTO 213
  ENDDO  ! over ice and water (I)
!  WRITE(*,*) '  IWS(1)%Absw_other  ', IWS(1)%Absw_other 
!  WRITE(*,*) '  IWS(2)%Absw_other  ', IWS(2)%Absw_other 
!  STOP '    Stop in read_uwt.f90'
! -----------------------------------------------------------------------
!
!   ****************** Inelastic processes in water *******************
!  
  N_INE = 3 ! The number of inelastic processes in water is set to 3
  ALLOCATE( OPTION(N_INE) )
  STR_INP = 'Inelastic processes wihtin water'//ADD_2(2)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING (READ_ARR(1)%S, STR_ARR)
  DO I = 1, N_INE
     OPTION(I)%S = CUT_STR(STR_ARR(I)%S)
     ! Check input parameters for inelastic processes
     SELECT CASE( OPTION(I)%S )
     CASE('default','advanced')  ;  IF( LoIW(1) ) GOTO 209
     CASE('off')
     CASE DEFAULT
        CALL NotFound(R_NAME, STR_INP, OPTION(I)%S, *880)
     END SELECT
  ENDDO
  ! Approach for CDOM fluorescence (for water only)
  IF( OPTION(3)%S /= 'off' ) THEN
     STR_INP = 'Select approach to calculate CDOM fluorescence'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     Flu_CDOM = CUT_STR(READ_ARR(1)%S)
     SELECT CASE( Flu_CDOM )
     CASE('FHa','ACR')
     CASE DEFAULT
        CALL NotFound(R_NAME, STR_INP, Flu_CDOM, *880)
     END SELECT
  ENDIF

! CDOM absorption requires Chl absorption => Removed by VR (17.01.19)
!  IF( CDOM_Chl.AND.(.NOT.Absw_chl) ) GOTO 202

  ! Chlorophyll-a fluorescence requires Chl absorption
  IF( (TRIM(OPTION(2)%S) /= 'off').AND.(.NOT.Absw_chl) ) GOTO 201
  ! Check option CDOM fluorescence:
  IF( TRIM(OPTION(3)%S) /= 'off' ) THEN
     SELECT CASE( Flu_CDOM )
     CASE('FHa')
        IF( .NOT.IWS(2)%Absw_ful ) GOTO 205
        IF( .NOT.IWS(2)%Absw_hum ) GOTO 206
     CASE('ACR')
        IF( .NOT.IWS(2)%CDOM_ACR ) GOTO 207
     END SELECT
  ENDIF

  INCL_INEL(:) = .FALSE.  ;  Add_TRS = .FALSE.
  WLS1 = 1.d10 ; WLS2 = 1.d10
  WLE1 = 0.d0  ; WLE2 = 0.d0
  Step_Flu(:) =  1.d+06
  Loc_DO(:) = .TRUE.  ; Loc_ADJ(:) = .FALSE.
  INEL_DO   = .FALSE. ; INEL_ADJ   = .FALSE.

  ! Set or read parameters for VRS depending on OPTION(1)
  IF( TRIM(OPTION(1)%S) /= 'off' ) CALL SET_VRS_PAR(*880)

  ! Set or read parameters for chl-a fluorescence OPTION(2)
  IF( TRIM(OPTION(2)%S) /= 'off' )  CALL SET_FChl_PAR(*880)

  ! Use CDOM fluorescence
  IF( TRIM(OPTION(3)%S) /= 'off' ) THEN
     INCL_INEL(3) = .TRUE. 
     SELECT CASE( Flu_CDOM )
     CASE('FHa')
        ! Number of fluorescent components is fixed
        N_FC = 2
        CALL SET_FCDOM_FHa(*880)
        !---------------------------------------------------------------
     CASE('ACR')
        STR_INP = 'Number of fluorescence components'//ADD_2(2)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S, *, ERR=101) N_FC
        STR_INP = 'Filenames containing excitation and emission functions'&
             //ADD_2(2)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        Exci_NAME = SET_PATH(STR_ARR(1)%S)
        Emi_NAME = SET_PATH(STR_ARR(2)%S)

        ALLOCATE( Scal_Em(N_FC) )
        STR_INP = 'Scaling factors for emission'//ADD_2(2)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*,ERR=101) (Scal_Em(I), I = 1,N_FC)
        ! Set main parameters of fluor components
        CALL SET_FCDOM_ACR(*880)
        !---------------------------------------------------------------
     END SELECT
  ENDIF  ! CDOM fluorescence

  IF( ANY(INCL_INEL(:)) ) THEN
     II = COUNT(Loc_DO(:))  ;  KK = COUNT(Loc_ADJ(:))
     ! Solution will be performed using DO technique
     IF( II == N_INE ) INEL_DO  = .TRUE.
     ! Solution will be performed using forward-adjoint technique
     IF( KK == N_INE ) INEL_ADJ = .TRUE.
     !         WRITE(*,*) INEL_DO, INEL_ADJ
     IF( .NOT.INEL_DO .AND. .NOT.INEL_ADJ ) GOTO 102
  ENDIF  ! ANY(INCL_INEL(:))

  WL_Flu(1)   = DMIN1(WLS1,WLS2)
  WL_Flu(2)   = DMAX1(WLE1,WLE2)
  Step_Flu(1) = DMIN1(Step_Flu(1),Step_Flu(2))
  
!   ************** (1) Inelastic processes in water *******************

  STR_INP = 'Perform calculation in two-layer slab'//ADD_2(2)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) TES_2L_SLAB

  STR_INP = 'Layering of ocean'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  OC_LAYERING = CUT_STR(READ_ARR(1)%S)

! ... Layering of ocean will be defined from
!    "control_uwt.inp" in the case of option "ocean" or "sea_ice"
!    "control_uit.inp" in the case of option "ice"
  II_AD = 1 
  IF( IWT.OR.LoIW(2)) II_AD = 2
!  WRITE(*,*) '    II_AD = ', II_AD
  STR_INP = 'Layering of ocean'//ADD_2(II_AD)%S     
  CALL Read_Input (STR_INP, READ_ARR, *880)
  OC_LAYERING = CUT_STR(READ_ARR(1)%S)
 
  SELECT CASE( OC_LAYERING )
  CASE('equidistant')
     STR_INP = 'Layers number'//ADD_2(II_AD)%S  
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S, *, ERR=101) KK
     NLAYERS_UW = KK + 1
!     WRITE(*,*) '   Layers number  = ', KK
  CASE('from file')
     STR_INP = 'Depth grid file name'//ADD_2(II_AD)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     DEP_NAME = SET_PATH(READ_ARR(1)%S)
!     WRITE(*,*) '   Grid file name  = ', DEP_NAME
  CASE DEFAULT
     CALL NotFound(R_NAME, STR_INP, OC_LAYERING, *880)
  END SELECT
   
  STR_INP = 'Ocean depth'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) OC_DEP
  IF( C_SYS=='ice' ) L_BOT_I(1) = OC_DEP
  IF( UIT.AND.(.NOT.IWT) ) THEN
     IF( REAL(OC_DEP) /= REAL(L_BOT_I(LA_N)) ) GOTO 214
  ENDIF

  STR_INP = 'Profile scenario file name'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
  PROF_UW_NAME = SET_PATH(STR_ARR(1)%S)
  DO I = 1, Ncol_UW
     READ(STR_ARR(I + 1)%S,*,ERR=101) NC_UW(I)
  END DO

  DO I = 1,2
     IF( LoIW(I) ) THEN
        STR_INP = 'File name for chlorophyll concentration'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        IWS(I)%CHL_CONC_NAME = SET_PATH(STR_ARR(1)%S)
        READ(STR_ARR(2)%S,*,ERR=101) IWS(I)%NC_CHL(1)
        READ(STR_ARR(3)%S,*,ERR=101) IWS(I)%NC_CHL(2)
     ENDIF
  ENDDO  ! over ice and water (I)

! Number of absorbing components in water
  N_ABS_UW = 2
! Absorption coefficients of water, ice, and other components
  IWS(:)%PW_ABS_FILE = .FALSE.
  DO I = 1,2
     IF( LoIW(I) ) THEN
        STR_INP = 'Pure water absorption coefficients file name'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        READ(STR_ARR(1)%S,*,ERR=101) LoD   ; IWS(I)%PW_ABS_FILE = LoD
        IWS(I)%PWAC_NAME = SET_PATH(STR_ARR(2)%S)
        READ(STR_ARR(3)%S,*,ERR=101) DUMM  ;  IWS(I)%PW_ABS = DUMM
     ENDIF
  ENDDO  ! over ice and water (I)
  
! ********************** Korrekturen **************************
  STR_INP = 'Psi-functions for water absorption correction'//ADD_2(2)%S
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
  READ(STR_ARR(1)%S,*,ERR=101) TS_CORR
  PSI_NAME = SET_PATH(STR_ARR(2)%S)
! ******************** Korrekturen Ende ***********************

  DO I = 1,2
     IF( LoIW(I) ) THEN
        STR_INP = 'Chlorophyll absorption coefficients file name'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        IWS(I)%CHL_NAME = SET_PATH(READ_ARR(1)%S)
     ENDIF
  ENDDO  ! over ice and water (I)
  
! Include absorption by fulvic or/and humic acid
  IF( IWS(2)%Absw_ful.OR.IWS(2)%Absw_hum ) THEN
     If(Allocated(Fu_par)) DeAllocate (Fu_par, Hu_par)
     ALLOCATE( Fu_par(2), Hu_par(2) )
     STR_INP = 'Parameters of specific absorption coefficient'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) FH_REF,(Fu_par(I),I=1,2),(Hu_par(I),I=1,2)
     STR_INP = 'Concentrations of fulvic and humic acids'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Fa_NAME = SET_PATH(STR_ARR(1)%S)
     FH_EXTR = CUT_STR(STR_ARR(2)%S)
     CALL SPLIT_STRING(READ_ARR(2)%S, STR_ARR )
     Ha_NAME = SET_PATH(STR_ARR(1)%S)
  ENDIF  ! absorption by fulvic or/and humic acid
  
  ! CDOM absorption is included as ACR
  IF( IWS(2)%CDOM_ACR ) THEN
     STR_INP = 'Number of fluorescence components'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) N_FC

     STR_INP = 'Filenames containing concentration profiles'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
     Conc_NAME = SET_PATH(STR_ARR(1)%S)
     FH_EXTR = SET_PATH(STR_ARR(2)%S)

     ALLOCATE( Scal_FC(N_FC) )
     STR_INP = 'Scaling factors for concentrations'//ADD_2(2)%S
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) (Scal_FC(I), I = 1,N_FC)

     ! To calculate absorption coefficients of CDOM in this 
     ! case we need to read excitation factors of fluorescent 
     ! components even if we do not need to include fluorescence
     ! ==========================================================
     IF( OPTION(3)%S == 'off' ) THEN
        STR_INP = 'Filenames containing '//&
             'excitation and emission functions'//ADD_2(2)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        Exci_NAME = SET_PATH(STR_ARR(1)%S)
        Emi_NAME = SET_PATH(STR_ARR(2)%S)

        OPEN(C_UNIT,FILE=Exci_NAME, STATUS='OLD' )
        CALL SKIP_HEADER (C_UNIT)
        N_WL = 0
        ! Number of rows containing parameters
        DO 
           READ(C_UNIT,*,ERR=312,END=311) STG
           N_WL = N_WL + 1 
        ENDDO
311     CONTINUE
        DO I = 1, N_WL + 1
           BACKSPACE(C_UNIT)
        ENDDO
        Nwl_QEx = N_WL
        ALLOCATE( WL_QEx(N_WL), F_QEx(N_FC,N_WL)  )
        DO I = 1, N_WL
           READ(C_UNIT,*) WL_QEx(I), (F_QEx(K,I), K = 1,N_FC)
        ENDDO
        CLOSE(C_UNIT)
     ENDIF
        !---------------------------------------------------------------
  ENDIF  ! CDOM_ACR

! ... Reading additional absorber in ice
  ALLOCATE( IWS(2)%Na_W(1) ) ;  IWS(2)%Na_W(:) = 0
  AD_ABS: Block
    IF( UIT ) THEN
       STR_INP = 'Use additional absorber in water'//ADD_2(1)%S ! ice layers
       CALL Read_Input (STR_INP, READ_ARR, *880)
!            WRITE(*,*) '     Number of input lines = ', Size( READ_ARR )
       CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
!            WRITE(*,*) '     Number of ice layers = ', La_N
!            WRITE(*,*) '     Number of absorber in each layer '
!            WRITE(*,'(A,*(2x,A))') '     ', ( STR_ARR(I)%S, I = 1,La_N )
       DO I = 1, La_N
          READ(STR_ARR(I)%S,*,ERR=101) IWS(1)%Na_W(I)
       ENDDO  ! over ice layers (I)
!     ... Full number of additional absorber in ice layers
       Na_ice = SUM( IWS(1)%Na_W(:) )
!      ... Check that additional absorber are defined for each ice layer:
       IF( La_N>1.AND.(Size( STR_ARR )/=La_N) ) GOTO 216
!      ... Check that setting is given for all additional absorber :
       IF( Na_ice>0.AND.(Size(READ_ARR)-2 /= SUM(IWS(1)%Na_W(:))) ) GOTO 217
           
       Na_W = MAXVAL( IWS(1)%Na_W(:) )

!      ... If additional absorber are presented
       IF( ANY(IWS(1)%Na_W(:) > 0) ) THEN
!         ... Path to directories containing absorption coefficients and concentrations
          CALL SPLIT_STRING(READ_ARR(2)%S, STR_ARR )
          DIR1 = SET_PATH(STR_ARR(1)%S) ;  DIR2 = SET_PATH(STR_ARR(2)%S)
          ALLOCATE( IWS(1)%C_NF(Na_W,LA_N), IWS(1)%AFN_coef(Na_W,La_N),  &
               IWS(1)%AFN_conc(Na_W,LA_N), IWS(1)%Extr_T(Na_W,La_N) ) ! A
          ! Reading filenames of concentrations of all additional absorber
          Kk = 2  !  Skip first two entry lines
          DO Ii = 1, LA_N
             DO I = 1, IWS(1)%Na_W(Ii)
                Kk = Kk + 1
!                WRITE(*,*) Kk, READ_ARR(Kk)%S
                CALL SPLIT_STRING(READ_ARR(Kk)%S, STR_ARR )
                IWS(1)%AFN_coef(I,Ii)%S = DIR1//CUT_STR(STR_ARR(1)%S)
                IWS(1)%AFN_conc(I,Ii)%S = DIR2//CUT_STR(STR_ARR(2)%S)
                READ(STR_ARR(3)%S,*,ERR=101) DUMM
                IWS(1)%C_NF(I,Ii) = DUMM
                IWS(1)%Extr_T(I,Ii)%S = CUT_STR(STR_ARR(4)%S)
!                WRITE(lunit,'(5x,a,2x,a,2x,f5.2,1x,a)') IWS(1)%AFN_coef(I,Ii)%S, &
!                     IWS(1)%AFN_conc(I,Ii)%S, IWS(1)%C_NF(I,Ii), IWS(1)%Extr_T(I,Ii)%S
             ENDDO  ! over additional absorber in ice layer (I)
          ENDDO  ! over ice layers (Ii)
! ------------------------------------------------------------------------------
!          DO Ii = 1, LA_N
!             DO I = 1, IWS(1)%Na_W(Ii)
!                WRITE(lunit,'(5x,a,2x,a,2x,f5.2,1x,a)') IWS(1)%AFN_coef(I,Ii)%S, &
!                     IWS(1)%AFN_conc(I,Ii)%S, IWS(1)%C_NF(I,Ii), IWS(1)%Extr_T(I,Ii)%S
!             ENDDO  ! over additional absorber (I)
!          ENDDO  ! ! over ice layers (Ii)
! -------------------------------------------------------------------------------      
       ENDIF
       IF( .NOT.IWT ) EXIT AD_ABS
    ENDIF  ! UIT
!    PAUSE '  Pause in "read_uwt.f90" '
    
!   ... Reading additional absorber in water
    STR_INP = 'Use additional absorber in water'//ADD_2(2)%S 
    CALL Read_Input (STR_INP, READ_ARR, *880)
    CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
    READ(STR_ARR(1)%S,*,ERR=101) Na_W  ;  IWS(2)%Na_W = Na_W
!    write(*,*) '  Water: IWS(2)%Na_W = ', IWS(2)%Na_W 
    IF( Na_W > 0 ) THEN
       DIR1 = SET_PATH(STR_ARR(2)%S) ;  DIR2 = SET_PATH(STR_ARR(3)%S)
       ALLOCATE( IWS(2)%C_NF(Na_W,1), IWS(2)%AFN_coef(Na_W,1), &
            IWS(2)%AFN_conc(Na_W,1), IWS(2)%Extr_T(Na_W,1) ) ! A
       DO I = 1, Na_W
          CALL SPLIT_STRING(READ_ARR(I + 1)%S, STR_ARR )
          NAM1 = CUT_STR(STR_ARR(1)%S) ; NAM2 = CUT_STR(STR_ARR(2)%S)
          IWS(2)%AFN_coef(I,1)%S = DIR1//NAM1
          IWS(2)%AFN_conc(I,1)%S = DIR2//NAM2
          READ(STR_ARR(3)%S,*,ERR=101) DUMM
          IWS(2)%C_NF(I,1) = DUMM
          IWS(2)%Extr_T(I,1)%S = CUT_STR(STR_ARR(4)%S)
          WRITE(lunit,'(5x,a,2x,a,2x,f5.2,1x,a)') IWS(2)%AFN_coef(I,1)%S, &
               IWS(2)%AFN_conc(I,1)%S, IWS(2)%C_NF(I,1), IWS(2)%Extr_T(I,1)%S
       ENDDO  ! over additional absorber in water (I)
    ENDIF  ! Na_W > 0
  END Block AD_ABS

!  STOP '  Before Do Akima  '
  
  STR_INP = 'Do Akima interpolation'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) DO_ABSUW_AKIMA

  DO I = 1,2
     IF( LoIW(I) ) THEN
        STR_INP = 'Pure seawater scattering function'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        VSF_pure_model = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( VSF_pure_model )
        CASE('Morel','Shifrin','Buiteveld','Kopelevich','off')
        CASE DEFAULT
           CALL NotFound(R_NAME, STR_INP, VSF_pure_model, *880)
        END SELECT
        IWS(I)%VSF_pure_model = VSF_pure_model
!       -------------------------------------------------------
        STR_INP = 'Rayleigh optical thickness of water'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
        READ(STR_ARR(1)%S, *, ERR=101) LoD
        IWS(I)%DO_RTAU_W = LoD
        IF( LoD ) THEN
           READ(STR_ARR(2)%S,*,ERR=101) DUMM
           IWS(I)%RTAU_W = DUMM
        ENDIF
!       -------------------------------------------------------
        STR_INP = 'Pure seawater depolarization ratio'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
        IWS(I)%DEPOL_WATER = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( IWS(I)%DEPOL_WATER )
        CASE('constant')
           STR_INP = 'Constant depolarization ratio'//ADD_2(I)%S
           CALL Read_Input (STR_INP, READ_ARR, *880)
           READ(READ_ARR(1)%S,*,ERR=101) DUMM
           IWS(I)%DEP_W_INPUT = DUMM
        CASE('from file')
           STOP 'Depolarization ratio from file is not implemented'
           STR_INP = 'Depolarization ratio filename'//ADD_2(I)%S
           CALL Read_Input (STR_INP, READ_ARR, *880)
           IWS(I)%DEP_W_FILENAME = SET_PATH(READ_ARR(1)%S)
        CASE DEFAULT
           CALL NotFound(R_NAME, STR_INP, DEPOL_WATER, *880)
        END SELECT
!       -------------------------------------------------------
        STR_INP = 'Refractive index input type'//ADD_2(I)%S
        CALL Read_Input (STR_INP, READ_ARR, *880)
!        WRITE(*,*) '    IWS(I)%Ref_type  ', I, CUT_STR(READ_ARR(1)%S)
        IWS(I)%Ref_type = CUT_STR(READ_ARR(1)%S)
        SELECT CASE( IWS(I)%Ref_type )
        CASE('const')
           IWS(I)%Nwl_BRF = 1 
           ALLOCATE( IWS(I)%RefW_r(IWS(I)%Nwl_BRF), IWS(I)%RefW_i(IWS(I)%Nwl_BRF) )
           STR_INP = 'Refractive index of water'//ADD_2(I)%S
           CALL Read_Input (STR_INP, READ_ARR, *880)
           CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
           READ(STR_ARR(1)%S,*,ERR=101) DUMM
           IWS(I)%RefW_r(1) = DUMM
           IWS(I)%RefW_i(1) = 0.d0
           IF( SIZE( STR_ARR ) == 2 ) THEN
              READ(STR_ARR(2)%S,*,ERR=101) DUMM
              IWS(I)%RefW_i(1) = DUMM
           ENDIF
           IWS(I)%Ref_ind = IWS(I)%RefW_r(1)               
        CASE('wl_dep')
           STR_INP = 'Refractive index file name'//ADD_2(I)%S
           CALL Read_Input (STR_INP, READ_ARR, *880)
           IWS(I)%Ref_name = SET_PATH(READ_ARR(1)%S)
           STR_INP = 'Reference wavelength grid'//ADD_2(I)%S
           CALL Read_Input (STR_INP, READ_ARR, *880)
           IWS(I)%RWG = CUT_STR(READ_ARR(1)%S)
        CASE DEFAULT
           CALL NotFound(R_NAME, STR_INP, Ref_type, *880)
        END SELECT
     ENDIF  ! ice, water, or both
  ENDDO  ! over ice and water (I)

  IF( UIT ) THEN
     STR_INP = 'Layering of scattering function/matrix (cai)'
     CALL Read_Input (STR_INP, READ_ARR, *880, ReArrange = .TRUE.)
     READ(READ_ARR(1)%S,*,ERR=101) DUMM
     IF( DUMM == 0.d0 ) THEN
        ! Ice at the bottom of atmosphere
        II = 1  ;  IWS(1)%IW_TOP(1) = 1
     ELSE
        II = 2  ;  IWS(1)%IW_TOP(1) = 0
     ENDIF
  ELSE
     II = 2
  ENDIF

! ... At the bottom of atmosphere can be water or ice layer
  IF( IWS(II)%Ref_type == 'const' ) THEN
     NWL_BRF = 1
     ALLOCATE( RefW_r(Nwl_BRF), RefW_i(Nwl_BRF))
     RefW_r(1) = IWS(II)%RefW_r(1) 
     RefW_i(1) = IWS(II)%RefW_i(1)
     Ref_ind   = IWS(II)%RefW_r(1)   
!     WRITE(*,*) II, RefW_r(1), RefW_i(1) 
!     PAUSE
  ENDIF
  DO_RTAU_W = IWS(II)%DO_RTAU_W
  RTAU_W    = IWS(II)%RTAU_W 
  Ref_type  = IWS(II)%Ref_type
!  write(*,*) '    Set Ref_name II = ', II
  IF( Ref_type=='wl_dep' )  Ref_name = IWS(II)%Ref_name
  RWG       = IWS(II)%RWG
!  write(*,*) '    After set Ref_name '
!  WRITE(*,*) '   RTAU_W  ', RTAU_W
!  WRITE(*,*) '  File name of water refrac. index ', Ref_name
!  PAUSE
  
! ... Reading of Hydrosol and Inclusion Parameters (HIP)
!   ===========================================================    
  HIP: Block
    IWS(1)%N_HPC = 0  ;  IWS(2)%N_HPC = 0
    IF( UIT ) THEN
!   ... Reading if ice layer is presented
       STR_INP = 'Number of hydrosol models'//ADD_2(1)%S
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) N_HM 
       IF( N_HM > 0 ) THEN
          N_HPC = N_HM
          IWS(1)%N_HPC = N_HPC
       ENDIF
       IF( .NOT.IWT ) EXIT HIP
    ENDIF  ! UIT = .TRUE.
!   ===========================================================    
!   ... Reading for water
!   ===========================================================    
    STR_INP = 'Number of hydrosol models'//ADD_2(2)%S
    CALL Read_Input (STR_INP, READ_ARR, *880)
    READ(READ_ARR(1)%S, *, ERR=101) N_HM
!    WRITE(*,*)  '   N_HM =  ', N_HM
    IF( N_HM > 0 ) THEN
       If (Allocated(HSOL_MOD)) DeAllocate (HSOL_MOD, SEL_VSC)
       ALLOCATE( HSOL_MOD(N_HM), SEL_VSC(3) )
       STR_INP = 'Hydrosol angular scattering'//ADD_2(2)%S
       CALL Read_Input (STR_INP, READ_ARR, *880)
       DO I = 1,N_HM
          HSOL_MOD(I)%S = CUT_STR(READ_ARR(I)%S) 
       ENDDO
!       WRITE(*,*)  '  HSOL_MOD(I)%S  =  ', HSOL_MOD(1)%S
       ! 'User_supplied' mode has to be at first if present
       II = 0
       DO I = 1,N_HM
          IF( HSOL_MOD(I)%S == 'User_supplied') II = I
       ENDDO
!       WRITE(*,*)  '   II =  ', II
       IF( II /= 0 ) THEN
          IF( II /= 1 ) THEN
             ! This is to avoid a compiler bug in gfortran-6
             HSOL_MOD(II)%S = TRIM(HSOL_MOD(1)%S)
             HSOL_MOD(1)%S  = 'User_supplied'
          ENDIF
       ENDIF
       II = 0
       DO I = 1,N_HM
          SELECT CASE( HSOL_MOD(I)%S )
          CASE('User_supplied')
             II = II + 1 ; SEL_VSC(II) = 'Use'
          CASE('Kopelevich')
             II = II + 1 ; SEL_VSC(II) = 'Kos'
             II = II + 1 ; SEL_VSC(II) = 'Kol'
          CASE DEFAULT
             CALL NotFound(R_NAME, STR_INP, HSOL_MOD(I)%S, *880)
          END SELECT
       ENDDO
       N_HPC = II
       IWS(2)%N_HPC = N_HPC
    ENDIF  ! N_HM > 0
!   ===========================================================    
  END Block HIP
  
  STR_INP = 'Number of streams in total reflection zone'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) NSTR_T

  STR_INP = 'Number of OSF Legendre moments'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  READ(READ_ARR(1)%S, *, ERR=101) NLEG_W

!  WRITE(*,*) '    ADD_STR =  ', ADD_STR
!  WRITE(*,*) NSTRMS, NSTR_T, NLEG_W
!  PAUSE
  
  STR_INP = 'Ocean bottom albedo'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
  OB_TYPE = CUT_STR( STR_ARR(1)%S )
!        WRITE(*,*) '   Option - ', OB_TYPE 
  SELECT CASE( OB_TYPE )
  CASE('const')
     READ(STR_ARR(2)%S, *, ERR=101) ALB_BOTTOM
  CASE('model')
     ALLOCATE( MP_gp(1), MP_bub(1) )
     STR_INP = 'Parameters of bottom ice'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )     
     READ(STR_ARR(1)%S,*,ERR=101) MP_gp(1)
     READ(STR_ARR(2)%S,*,ERR=101) MP_bub(1)
     MP_gp(1) =  MP_gp(1)/100.d0  ! [cm] --> [m]
!          WRITE(*,*) '  Ice thicness and tsc  ', MP_gp(1), MP_bub(1)
     Iref_name = SET_PATH(STR_ARR(3)%S)
!          WRITE(*,*) '  File name of ice refrac. index ', Iref_name
  CASE DEFAULT
     CALL NotFound(R_NAME, STR_INP, CH_INP, *880)
  END SELECT

  STR_INP = 'Output of water-leaving radiance'//ADD_STR
  CALL Read_Input (STR_INP, READ_ARR, *880)
  CH_INP = CUT_STR(READ_ARR(1)%S)
  SELECT CASE( CH_INP )
  CASE('off')
     OUT_WLR = .FALSE.
  CASE('on')
     OUT_WLR = .TRUE.
  CASE DEFAULT
     CALL NotFound(R_NAME, STR_INP, CH_INP, *880)
  END SELECT

!  IF( Ref_ind < 1.01d0 ) NSTR_T = 0
  
  NSZA_W = 1

  IF( TES_2L_SLAB ) THEN
     STR_INP = 'Refractive indexes (tls)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S, *, ERR=101) Ref_idx(1), Ref_idx(2)
     STR_INP = 'Single scattering albedo (tls)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Ome0(1), Ome0(2)
     STR_INP = 'Optical thicknesses (tls)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) Taus(1), Taus(2)
     STR_INP = 'Number of streams (tls)'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*,ERR=101) NSTR_T1            
  ENDIF  ! TES_2L_SLAB

  CALL ST_TRACK (R_NAME, 3, 'O')

  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', STR_INP
  RETURN 1

102 Msg(1)='Only single solution technique can be selected'
  Msg(2)='   for all inelastic processes.'
  Msg(3)='Check "Numerical technique used for ..." control '
  Msg(4)='   parameter in the file "control_rrs.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)

312 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file !')
  WRITE(EU,'(A,A)') '    Filename is: ', Exci_NAME
  RETURN 1

201 Msg(1)='Chl fluorescence cannot be calculated '
  Msg(2)='  without absorption by chlorophyll.'
  Msg(3)='Check "Forward model: marine absorber" control parameter '
  Msg(4)='  in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)

!202 Msg(1)='CDOM absorption cannot be calculated '
!  Msg(2)='  without absorption by chlorophyll.'
!  Msg(3)='Check "Forward model: marine absorber" control parameter '
!  Msg(4)='  in the file "control_uwt.inp".'
!  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)

205 Msg(1)='CDOM fluorescence using FHa approach cannot be calculated'
  Msg(2)='  without absorption by fulvic acid.'
  Msg(3)='Check "Forward model: marine absorber" control parameter '
  Msg(4)='  in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
  
206 Msg(1)='CDOM fluorescence using FHa approach cannot be calculated'
  Msg(2)='  without absorption by humic acid.'
  Msg(3)='Check "Forward model: marine absorber" control parameter '
  Msg(4)='  in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
  
207 Msg(1)='CDOM fluorescence using ACR approach cannot be calculated'
  Msg(2)='  without absorption by CDOM ACR.'
  Msg(3)='Check "Forward model: marine absorber" control parameter '
  Msg(4)='  in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)

208 Msg(1)='If the weight of diffusely transmitted radiation'
  Msg(2)='  less than 1, then the direct solar radiation reflected by'
  Msg(3)='  surface in the direction of observer should be calculated'
  Msg(4)='  exactly.'
  Msg(5)='Check second and third entries "Approximation of RT matrixes" '
  Msg(6)='  control parameter in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,6,*880)

209 Msg(1)='Inelastic processes are not implemented  in the case of ice '
  Msg(2)='  and sea ice.'
  Msg(3)='Check the "Inelastic processes wihtin water" control parameter'
  Msg(4)='  in the file "control_uwt.inp".'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
  
210 Msg(1)='Chlorophyll absorption is not implemented in the case of ice '//&
         'and sea ice.'
  Msg(2)='Check the "Forward model: marine absorber (ice)" control parameter'
  Msg(3)='  in the "control_uit.inp" file.'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
  
211 Msg(1)='CDOM absorption is not available in the case of ice '//&
         'and sea ice.'
  Msg(2)='Check the "Forward model: marine absorber (ice)" control parameter'
  Msg(3)='  in the "control_uit.inp" file.'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)

212 Msg(1)='CDOM absorption ACR is not implemented in the case '//&
         'of ice and sea ice.'
  Msg(2)='Check the "Forward model: marine absorber (ice)" control parameter'
  Msg(3)='  in the "control_uit.inp" file.'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)

213 Msg(1)='Absorption by fulvic and humic acid is not implemented'
  Msg(2)= '  in the case of ice and sea ice.'
  Msg(3)='Check the "Forward model: marine absorber (ice)" control parameter'
  Msg(4)='  in the "control_uit.inp" file.'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
  
214 Msg(1)='Ocean depth does not equal to the bottom depth of ice layer.'
  Msg(2)= 'Check the parameter "Ocean depth" in the "control_uit.inp" file '
  Msg(3)='  and the "Layering of scattering function/matrix (cai)" parameter '
  Msg(4)='  in the "cf_ice_man.inp" file.'
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)
  
215 Msg(1)='In the case of atmosphere-ice system (option "ice") '
  Msg(2)='  only one ice layer is allowed. '
  Msg(3)= 'Check the parameter "Layering of scattering function/matrix (cai)" '
  Msg(4)='  in the "cf_ice_man.inp" file. '
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,4,*880)

216 Msg(1)='In the case of atmosphere-water-ice system (option "sea_ice") '
  Msg(2)='  number of additional absorber should be defined '
  Msg(3)='  for each ice layer. '
  Msg(4)= 'Check the parameter "Layering of scattering function/matrix (cai)" '
  Msg(5)='  in the "cf_ice_man.inp" file. '
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,5,*880)

217 Msg(1)='Not enough information for all additional absorber in ice layers'
  Msg(2)= 'Check number of lines after "Use additional absorber in water (ice)" '
  Msg(3)='  parameter in the "control_uit.inp" file. '
  CALL WRITE_MSG(R_NAME,'ERROR',Msg,3,*880)
  
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  
  RETURN 1
  
CONTAINS
!**************************************************************************
  SUBROUTINE SET_VRS_PAR(*)
    ! Number of water molecules in cm-3
    REAL (KND), PARAMETER :: N_mol = 3.34d22

    SELECT CASE( OPTION(1)%S )
    CASE('advanced')
       ! Read VRS parameters from the file
       STR_INP = 'Numerical technique used for VRS'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       DOM_ADJ = CUT_STR(READ_ARR(1)%S)

       SELECT CASE( DOM_ADJ )
       CASE('dom')
          Loc_DO(1) = .TRUE. ; Loc_ADJ(1) = .FALSE.
       CASE('adj')
          Loc_DO(1) = .FALSE. ; Loc_ADJ(1) = .TRUE.
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, DOM_ADJ, *881)
       END SELECT
         
       STR_INP = 'Integrate with FRF'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
       READ(STR_ARR(1)%S, *, ERR=101) Integr_FRF
!       IF(Integr_FRF) THEN
          STR_LOC = Join(STR_ARR, 2)
          READ(STR_LOC, *, ERR=101) Nsig, STEP_VRS
!       END IF

       STR_INP = 'Total Raman scattering coefficients'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
       READ(STR_ARR(1)%S, *, ERR=101) Tvs_Use
       IF(Tvs_Use) THEN
          STR_LOC = Join(STR_ARR, 2)
          READ(STR_LOC, *, ERR=101) Tvs_Exc, Tvs_Act
       END IF
       
       STR_INP = 'Add TRS to extinction'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S,*,ERR=101) Add_TRS(1), Add_TRS(2)

       STR_INP = 'Exclude solar light'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S,*,ERR=101) Sol_off

       STR_INP = 'Depolarization ratio'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S,*,ERR=101) Rho

       STR_INP = 'Basic spectroscopic parameters'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
       STR_LOC = Join(STR_ARR, 2)
       READ(STR_LOC,*,ERR=101) Inp_val, Lam_ref

!       WRITE(*,*) ' Inp_val, Lam_ref ', Inp_val, Lam_ref
        
       Contour = CUT_STR(STR_ARR(4)%S)
!       WRITE(*,*) '   STR_ARR(5)%S ', STR_ARR(5)%S
       READ(STR_ARR(5)%S,*,ERR=101) Lam_int
!       WRITE(*,*)  Lam_int
       
       STR_LOC = CUT_STR(STR_ARR(1)%S)
       SELECT CASE( STR_LOC )
       CASE('TRS')
          Cof = Inp_val
            
       CASE('CS_90')
          ! Calculate total Raman scattering coefficient [m-1]  
          ! corresponding to a given cross section [cm2/molec/sr] 
          ! at 90 deg. scattering angle (see Eq.(CS_to_TRS) in STB)
          Cof = 800.d0*Pie/3.d0*N_mol*(1.d0 + 2.d0*Rho)/(1.d0 + Rho)*Inp_val

       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, STR_INP, *880)
       END SELECT
       !---------------------------------------------------------------

       ! Check input parameter "Contour"
       SELECT CASE( Contour )
       CASE('Resonance')
          Shift = SUM( Dnu_FRF(:)*A_FRF(:) )
          ! Position of the Raman band centrum corresponding
          ! to the excitation wavelength Lam_ref
          NuR = K_nu/Lam_ref - Shift
          N_fac = ( NuR**2/((K_nu/Lam_int)**2 - (K_nu/Lam_ref)**2) )**2 

       CASE('Inverse_power_law')
          N_fac = 1.d0

       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, Contour,*880)
       END SELECT
       !---------------------------------------------------------------
       Bet_0R = Cof/N_fac
           
       INCL_INEL(1) = .TRUE. ;  Gauss_Field = .TRUE.
         
!       WRITE(LUNIT,*) '  Reference wavelength and total Raman ceff.'
!       WRITE(LUNIT,'(2x,f7.2,2x,E12.4)') Lam_ref, Bet_0R*Cof
         
    CASE('default')
       ! Define VRS default parameters
       Loc_DO(1)  = .TRUE.  ; Loc_ADJ(1) = .FALSE.
       ! Integr_FRF = .TRUE.  ; Nsig = 2  ;  STEP_VRS = 0.5d0
       Integr_FRF = .TRUE.  ; Nsig = 1  ;  STEP_VRS = 0.5d0
       Tvs_Use    = .FALSE. ; Sol_off = .FALSE.
       Add_TRS(1) = .FALSE. ; Add_TRS(2) = .TRUE. 
       Rho        = 0.17d0 ; Cof = 2.7d-04 
       Lam_ref = 488.d0 ; Contour = 'Resonance' ; Lam_int = 130.d0       
       Shift = SUM( Dnu_FRF(:)*A_FRF(:) )

       ! Position of the Raman band centrum corresponding
       ! to the excitation wavelength Lam_ref
       NuR = K_nu/Lam_ref - Shift
       N_fac = ( NuR**2/((K_nu/Lam_int)**2 - (K_nu/Lam_ref)**2) )**2 
       Bet_0R = Cof/N_fac
       INCL_INEL(1) = .TRUE. ;  Gauss_Field = .TRUE.
       
 END SELECT

    RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 

880 CONTINUE
881 CONTINUE
   RETURN 1

  END SUBROUTINE SET_VRS_PAR

  !**************************************************************************
  SUBROUTINE SET_FChl_PAR(*)

    INCL_INEL(2) = .TRUE. 
    ALLOCATE( WL_QEred(2) )
    ! Excitation wavelengths range
    WLS1 = 370.d0 ; WLE1 = 690.d0
    WL_QEred(1) = 370.d0 ; WL_QEred(2) = 690.d0

    SELECT CASE( OPTION(2)%S )
    CASE('advanced')
       ! Read Chl-a fluorescence parameters from the input file
       STR_INP = 'Numerical technique used for fluorescence'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       DOM_ADJ = CUT_STR(READ_ARR(1)%S)    
       ! Check input parameter
       SELECT CASE( DOM_ADJ )
       CASE('dom')
          Loc_DO(2) = .TRUE. ; Loc_ADJ(2) = .FALSE.
       CASE('adj')
          Loc_DO(2) = .FALSE. ; Loc_ADJ(2) = .TRUE.
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, DOM_ADJ, *880)
       END SELECT
       STR_INP = 'Integration step over Chl excitation range'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) Step_Flu(1)   
       STR_INP = 'Quantum efficiency'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) Qe_Chl
       STR_INP = 'Gaussian emission band parameters'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) WLG_Chl, Sig_Chl
       ! STOP '  Option "advanced" is not implemented for F_Chl'
    CASE('default')
       Loc_DO(2) = .TRUE. ;  Loc_ADJ(2) = .FALSE.
       ! Integration step over excitation wavelength range
       Step_Flu(1) = 1.d0
       ! Quantum efficiency
       Qe_Chl = 0.01
       ! Gaussian emission band parameters
       WLG_Chl = 685.d0 ; Sig_Chl = 10.6d0
    END SELECT

    RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', STR_INP
    RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1

  END SUBROUTINE SET_FChl_PAR

  ! **************************************************************************
  SUBROUTINE SET_FCDOM_FHa(*)

    INTEGER, PARAMETER:: NF_COL=4, NH_COL=5
    CHARACTER (LEN=2)   RT2
    CHARACTER (LEN=:), ALLOCATABLE :: Fe_NAM, He_NAM
    CHARACTER (LEN=200) RTX
    INTEGER  I, K, N_WL1, NF_USE, NH_USE
    REAL (KND) DUM, RTT( MAX(NH_COL,NF_COL) )
    !==================================================================

    INCL_INEL(3) = .TRUE. 

    ALLOCATE( FH_QE(2), WL_QEblu(2) )

    SELECT CASE( OPTION(3)%S )
    CASE('advanced')
       ! Read CDOM fluorescence parameters from the input file
       STR_INP = 'Numerical technique used for fluorescence'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       DOM_ADJ = CUT_STR(READ_ARR(1)%S)    
       ! Check input parameter
       SELECT CASE( DOM_ADJ )
       CASE('dom')
          Loc_DO(3) = .TRUE. ; Loc_ADJ(3) = .FALSE.
       CASE('adj')
          Loc_DO(3) = .FALSE. ; Loc_ADJ(3) = .TRUE.
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, DOM_ADJ, *880)
       END SELECT
       
       STR_INP = 'Integration step over excitation range'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) Step_Flu(2)
       
       STR_INP = 'Quantum efficiency function'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       CALL SPLIT_STRING(READ_ARR(1)%S, STR_ARR )
       Fe_NAM = CUT_STR(STR_ARR(1)%S)
       He_NAM = CUT_STR(STR_ARR(2)%S)
       ! Fulvic acid
       SELECT CASE( Fe_NAM )
       CASE('FA7')  ; NF_USE = 1
       CASE('FA8')  ; NF_USE = 2
       CASE('FA9')  ; NF_USE = 3
       CASE('FA11') ; NF_USE = 4
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, Fe_NAM, *880)
       END SELECT
       ! Humic acid
       SELECT CASE( He_NAM )
       CASE('HA1')  ; NH_USE = 1
       CASE('HA2')  ; NH_USE = 2
       CASE('HA4')  ; NH_USE = 3
       CASE('HA6')  ; NH_USE = 4
       CASE('HA8')  ; NH_USE = 5
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, He_NAM, *880)
       END SELECT

       !---------------------------------------------------------------
    CASE('default')
       Loc_DO(3) = .TRUE. ;  Loc_ADJ(3) = .FALSE. 
       Step_Flu(2) = 1.d0
       ! Reading parameters for quantum efficiency function according to Hawes  
       NF_USE = 1             ! FA7 
       NH_USE = 4             ! HA6
       !---------------------------------------------------------------
    END SELECT

    ! Excitation range
    WLS2 = 310.d0 ;  WLE2 = 490.d0
    WL_QEblu(1) = 310.d0  ;  WL_QEblu(2) = 490.d0
    ! Parameters of quantum efficiency function fulvic acid:
    DIR_name = SET_PATH('~/DATA_BASES/SPECTRA_UW/')
    FH_QE(1)%S = DIR_name//'QE_fulvic_acid.dat'
    ! humic acid:
    FH_QE(2)%S = DIR_name//'QE_humic_acid.dat'

    WRITE(LUNIT,*) '  Reading:  '//FH_QE(1)%S
    OPEN( C_UNIT, FILE = FH_QE(1)%S, STATUS='OLD' )

    HDL= '#'
    DO WHILE(HDL==';' .OR. HDL=='#'.OR. HDL=='>'.OR. HDL=='*')
       READ(C_UNIT,*) HDL
    END DO
    BACKSPACE(C_UNIT) 

    N_WL = 0
    DO 
       READ(C_UNIT,*,ERR=312,END=311) RTX
       IF( RTX(1:2)=='Wl' ) THEN
          N_WL = N_WL + 1 
       ELSE
          EXIT
       ENDIF
    ENDDO
311 CONTINUE

    DO I = 1, N_WL + 1
       BACKSPACE(C_UNIT)
    ENDDO
    Nwl_QE = N_WL
    ALLOCATE( WL_QE(N_WL), A_QE(2,N_WL), A_Fa(2), B_Fa(2) )
    ! Fulvic acid parameters of frequency redistribution function (FRF)
    DO I = 1, N_WL
       READ(C_UNIT,*) RT2, WL_QE(I), (RTT(K),K=1,NF_COL)
       A_QE(1,I) = RTT(NF_USE)
    ENDDO
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    A_Fa(1) = RTT(NF_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    B_Fa(1) = RTT(NF_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    A_Fa(2) = RTT(NF_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    B_Fa(2) = RTT(NF_USE)
    CLOSE(C_UNIT)
    !
    ! Reading quantum efficiency parameters for humic acid
    !
    WRITE(LUNIT,*) '  Reading:  '//FH_QE(2)%S
    OPEN( C_UNIT, FILE=FH_QE(2)%S,STATUS='OLD' )
    HDL= '#'
    DO WHILE(HDL==';' .OR. HDL=='#'.OR. HDL=='>'.OR. HDL=='*')
       READ(C_UNIT,*) HDL
    END DO
    BACKSPACE(C_UNIT) 
    N_WL1 = 0
    DO 
       READ(C_UNIT,*,ERR=412,END=411) RTX
       IF( RTX(1:2)=='Wl' ) THEN
          N_WL1 = N_WL1 + 1 
       ELSE
          EXIT
       ENDIF
    ENDDO
411 CONTINUE

    IF( N_WL1 /= N_WL ) GOTO 413

    DO I = 1, N_WL + 1
       BACKSPACE(C_UNIT)
    ENDDO
    ALLOCATE( A_Ha(2), B_Ha(2) )
    ! Humic acid parameters of frequency redistribution function (FRF)
    DO I = 1, N_WL
       READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
       A_QE(2,I) = RTT(NH_USE)
    ENDDO

    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    A_Ha(1) = RTT(NH_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    B_Ha(1) = RTT(NH_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    A_Ha(2) = RTT(NH_USE)
    READ(C_UNIT,*) RT2, DUM, (RTT(K),K=1,NF_COL)
    B_Ha(2) = RTT(NH_USE)
    CLOSE(C_UNIT)
    
!    DO I = 1, N_WL
!       WRITE(LUNIT,'(2x,F6.1, 2x, 2(1x,E10.3))') WL_QE(I), A_QE(1,I), A_QE(2,I)
!    ENDDO
!    WRITE(LUNIT,'(A,F6.3, 2x,F6.3)')   ' A1 ',A_Fa(1), A_Ha(1)
!    WRITE(LUNIT,'(A,E12.5,2x,E12.5)')  ' B1 ',B_Fa(1), B_Ha(1)
!    WRITE(LUNIT,'(A,F6.3, 2x,F6.3)')   ' A2 ',A_Fa(2), A_Ha(2)
!    WRITE(LUNIT,'(A,E12.5,2x,E12.5)')  ' B2 ',B_Fa(2), B_Ha(2)
    
    RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', STR_INP
    RETURN 1
    
312 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file !')
    WRITE(EU,'(A,A)') '    Filename is: ',FH_QE(1)%S
    RETURN 1

412 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file !')
    WRITE(EU,'(A,A)') '    Filename is: ',FH_QE(2)%S
    RETURN 1
     
413 CALL WRITE_MESSAGE(R_NAME, 'ERROR',' ' )
    WRITE(EU,'(A)') '    Number of wavelengths in files'
    WRITE(EU,'(4x,A,A,A)') FH_QE(1)%S,' and ',FH_QE(2)%S
    WRITE(EU,'(A)') '    has to be equal !'
    RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    RETURN 1

  END SUBROUTINE SET_FCDOM_FHa

  !**********************************************************************
  SUBROUTINE SET_FCDOM_ACR(*)

    INTEGER, PARAMETER:: NF_COL=4, NH_COL=5
    CHARACTER (LEN=2)   RT2
    CHARACTER (LEN=4)   Fe_NAM, He_NAM
    CHARACTER (LEN=200) RTX
    INTEGER  I, K, N_WL1, NF_USE, NH_USE
    REAL (KND) DUM, RTT( MAX(NH_COL,NF_COL) )
    !==================================================================

    INCL_INEL(3) = .TRUE. 

    ALLOCATE( WL_QEblu(2) )

    SELECT CASE( OPTION(3)%S )
    CASE('advanced')
       ! Read CDOM fluorescence parameters from the input file

       STR_INP = 'Numerical technique used for fluorescence'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       DOM_ADJ = CUT_STR(READ_ARR(1)%S)    
       ! Check input parameter
       SELECT CASE( DOM_ADJ )
       CASE('dom')
          Loc_DO(3) = .TRUE. ; Loc_ADJ(3) = .FALSE.
       CASE('adj')
          Loc_DO(3) = .FALSE. ; Loc_ADJ(3) = .TRUE.
       CASE DEFAULT
          CALL NotFound(R_NAME, STR_INP, DOM_ADJ, *880)
       END SELECT

       STR_INP = 'Integration step over excitation range'
       CALL Read_Input (STR_INP, READ_ARR, *880)
       READ(READ_ARR(1)%S, *, ERR=101) Step_Flu(2)
       
    CASE('default')
       Loc_DO(3) = .TRUE. ;  Loc_ADJ(3) = .FALSE. 
       Step_Flu(2) = 1.d0
    END SELECT

    !
    ! Read excitation functions
    !
    WRITE(LUNIT,*) '  Reading:  '//Exci_NAME
    OPEN( C_UNIT,FILE=Exci_NAME,STATUS='OLD' )
    CALL SKIP_HEADER (C_UNIT)

    N_WL = 0
    ! Number of rows containing parameters
    DO 
       READ(C_UNIT,*,ERR=312,END=311) RTX
       N_WL = N_WL + 1 
    ENDDO
311 CONTINUE

    DO I = 1, N_WL + 1
       BACKSPACE(C_UNIT)
    ENDDO

    Nwl_QEx = N_WL
    ALLOCATE( WL_QEx(N_WL), F_QEx(N_FC,N_WL)  )
    DO I = 1, N_WL
       READ(C_UNIT,*) WL_QEx(I), (F_QEx(K,I), K = 1,N_FC)
    ENDDO
    CLOSE(C_UNIT)
    
    ! DO I = 1, Nwl_QEx 
    !    WRITE(*,'(1x,F6.1,7(1x,F12.9))') &
    !                    WL_QEx(I), (F_QEx(K,I), K = 1,N_FC)
    ! ENDDO

    ! Excitation range
    WLS2 = WL_QEx(1) ;  WLE2 = WL_QEx(Nwl_QEx)
    WL_QEblu(1) = WLS2  ;  WL_QEblu(2) = WLE2

    !
    !Read emission functions
    !
    WRITE(LUNIT,*) '  Reading:  '//Emi_NAME
    OPEN( C_UNIT, FILE=Emi_NAME, STATUS='OLD' )
    CALL SKIP_HEADER (C_UNIT)
    N_WL = 0
    ! Number of rows containing parameters
    DO 
       READ(C_UNIT,*,ERR=412,END=313) RTX
       N_WL = N_WL + 1 
    ENDDO
313 CONTINUE
    DO I = 1, N_WL + 1
       BACKSPACE(C_UNIT)
    ENDDO

    Nwl_QEm = N_WL
    ALLOCATE( WL_QEm(N_WL), F_QEm(N_FC,N_WL)  )
    DO I = 1, N_WL
       READ(C_UNIT,*) WL_QEm(I), (F_QEm(K,I), K = 1,N_FC)
    ENDDO
    CLOSE(C_UNIT)
      
    ! DO I = 1, Nwl_QEm 
    !    WRITE(*,'(1x,F6.1,7(1x,F12.9))') &
    !        WL_QEm(I), (F_QEm(K,I), K = 1,N_FC)
    ! END DO
    !------------------------------------------------------------------
    ! Concentration profiles will be read in "gt_ptconc_setup.f"
    !
    ! Scale emission functions
    DO K = 1, N_FC
       F_QEm(K,:) = F_QEm(K,:)*Scal_Em(K)
    ENDDO

    RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
    WRITE(EU,*) '    Parameter to read: ', STR_INP
    RETURN 1

312 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file !')
    WRITE(EU,'(A,A)') '    Filename is: ',Exci_NAME
    RETURN 1

412 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Error reading data file !')
    WRITE(EU,'(A,A)') '    Filename is: ',Emi_NAME
    RETURN 1
     
880 WRITE(EU, *) ' ---> Called by '//R_NAME

    RETURN 1

  END SUBROUTINE SET_FCDOM_ACR
  
!***************************************************************************
END SUBROUTINE READ_UWT
!***************************************************************************
