C********************************************************************C
C
C       SCIATRAN - Version 3.2 - Last modification: 23 Feb 2013
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE REF_MAT_SCAL( N_ref, X_ref, N_Phi, X1, N_INC, 
     $     COSI, Nwl_Inp, WL_Inp, R1_OUT )

      USE BRDF,        ONLY : RefW_r, RefW_i, Nwl_BRF, WL_BRF
      USE INTERN_DIMS, ONLY : Nvs
      USE LOG_PARS,    ONLY : LUNIT, LOG_LEVEL, N_DSO, N_ESO
      USE ST_VARS,     ONLY : Pie
      USE TYPE_VARS,   ONLY : KND, DCMP
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: N_ref, N_Phi, N_INC, Nwl_Inp
c     ...  X_ref and COSI are absolute values of reflected and 
c          incident polar angle cosines
      REAL (KND), INTENT(IN) :: X_ref(N_ref), X1(N_Phi), COSI(N_INC),
     $     WL_Inp(Nwl_Inp)
      REAL (KND)  R1_out(Nvs*N_ref,Nvs*N_INC,N_Phi,Nwl_Inp)
c     ... Local variables
      INTEGER     I, J, K, L, I1, K1
      REAL (KND), DIMENSION (N_REF,N_INC,N_Phi) :: XI1
      REAL (KND)  Gh, R_L, R_P, TT, SINR, VCI(3), VCR(3), Unit(3)
      REAL (KND),  TARGET :: Rezr(1), Rezi(1) 
      REAL (KND), POINTER :: P_refr(:), P_refi(:)
      COMPLEX (kind(0.0d0)) :: CC1, CC2, CN1, CN2, CN11, CN22, CXI2
c     ==================================================================

c     ... Set BRDF parameters at the needed wavelength grid
c     ----------------------------------------------------- 
      IF( Nwl_Inp==1 .AND. Nwl_BRF > 1) THEN
c        ... Interpolation of refractive index on the current wavelength
c            using SS_STOKES entry
         CALL LIN_SEARCH_NEW(WL_INP(1), Gh, I, K, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.d0 - Gh)*RefW_r(I) + Gh*RefW_r(K)
         P_refr => Rezr(1:1)
         Rezi(1) = (1.d0 - Gh)*RefW_i(I) + Gh*RefW_i(K)
         P_refi => Rezi(1:1)
      ELSE
c        ... Using previously interpolated or constant values of 
c            refractive indeces
         P_refr => RefW_r(:)  ;  P_refi => RefW_i(:)
      ENDIF

      R1_OUT = 0.d0

      DO K = 1, N_INC
c        ... Cartezian components of the unit vector of incident 
c            beam   'OX', 'OY', and 'OZ'
         VCI(1) = DSQRT(1.d0 - COSI(K)**2) 
         VCI(2) = 0.d0 
         VCI(3) = -COSI(K)
         DO I = 1, N_REF
            SINR = DSQRT(1.d0 - X_ref(I)**2)
            DO J = 1, N_Phi
c                   ... Cartezian components of the unit vector of 
c                       reflected beam   'OX', 'OY', and  'OZ'
               VCR(1) = SINR*DCOS(X1(J))
               VCR(2) = SINR*DSIN(X1(J))
               VCR(3) = X_ref(I)
               Unit(:) = VCI(:) - VCR(:)
c                    ... This expression gives the cosine of the incidence
c                        angle w.r.t normal of a wave facet
c                        (see Eq.(Refl-angl) in STB)
               XI1(I,K,J) = SUM( Unit(:)*VCI(:) )
     $              /DSQRT(SUM( Unit(:)**2 ))
            ENDDO
         ENDDO
      ENDDO

      CN1 = CMPLX( 1.d0,0.d0,KND ) ; CN11 = CN1*CN1

      DO L = 1,Nwl_Inp  
         CN2 = CMPLX( P_refr(L),P_refi(L),KND ) ; CN22 = CN2*CN2
         DO J = 1, N_Phi
            DO I = 1, N_REF
               I1 = (I-1)*Nvs+1
               DO K = 1, N_INC
                  K1 = (K-1)*Nvs+1  
                  TT = XI1(I,K,J)
                  CXI2 = 1.d0 - CN11*(1.d0 - TT*TT)/CN22
                  CXI2 = CDSQRT(CXI2)
                  
                  CC1 = CN2*TT ; CC2 = CN1*CXI2
                  R_L = CDABS((CC1 - CC2)/(CC1 + CC2))
                  
                  CC1 = CN1*TT ; CC2 = CN2*CXI2
                  R_P = CDABS((CC1 - CC2)/(CC1 + CC2))
                  
                  R1_OUT(I1,K1,J,L) = 0.5d0*(R_L*R_L + R_P*R_P)
               ENDDO            ! over incidence angles (K)
            ENDDO               ! over reflection angles (I)
         ENDDO                  ! over azimuthal angles (J)
      ENDDO                     ! over reference wavelengths (L)

      RETURN
      END
