C********************************************************************C
C
C       SCIATRAN - Version 3.1 - Last modification: 06 May 2013
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE REF_MAT_VEC( N_ref, X_ref, N_Phi, X1, N_INC, 
     $     COSI, Nwl_Inp, WL_Inp, R1_OUT )

      USE BRDF,        ONLY : RefW_r, RefW_i, Nwl_BRF, WL_BRF
      USE INTERN_DIMS, ONLY : Nvs
      USE LOG_PARS,    ONLY : LUNIT, LOG_LEVEL, N_DSO, N_ESO
      USE ST_VARS,     ONLY : Pie
      USE TYPE_VARS,   ONLY : KND, DCMP
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: N_ref, N_Phi, N_INC, Nwl_Inp
c     ...  X_ref and COSI are absolute values of reflected and 
c          incident polar angle cosines
      REAL (KND), INTENT(IN) :: X_ref(N_ref), X1(N_Phi), COSI(N_INC),
     $     WL_Inp(Nwl_Inp)
      REAL (KND)  R1_out(Nvs*N_ref,Nvs*N_INC,N_Phi,Nwl_Inp)
c     ... Local variables
      INTEGER     I, J, K, L, Nin, Ore
      REAL (KND)  Dmur, PhiI, Gh, R(4,4)
      REAL (KND),  TARGET :: Rezr(1), Rezi(1) 
      REAL (KND), POINTER :: P_refr(:), P_refi(:)
      COMPLEX (kind(0.0d0)) :: CN1, CN2
c     ==================================================================

c      Set BRDF parameters at the needed wavelength grid
c      =================================================
      IF( Nwl_Inp == 1 .AND. Nwl_BRF > 1) THEN
c        ... Interpolation of refractive index on the current wavelength
c            using SS_STOKES entry
         CALL LIN_SEARCH_NEW(WL_INP(1), Gh, I, K, Nwl_BRF, WL_BRF)
         Rezr(1) = (1.d0 - Gh)*RefW_r(I) + Gh*RefW_r(K)
         P_refr => Rezr(1:1)
         Rezi(1) = (1.d0 - Gh)*RefW_i(I) + Gh*RefW_i(K)
         P_refi => Rezi(1:1)
c         IF (LOG_LEVEL(1) >= N_ESO)  THEN
c            WRITE(LUNIT,*) 
c     $           'Interpolated index of refraction for Nwl_BRF = ', 
c     $           Nwl_BRF
c            WRITE(LUNIT,*) '  and Nwl_Inp = ', Nwl_Inp
c            WRITE(LUNIT,'(2x,f9.6,2x,e15.7)')  P_refr(1), P_refi(1)
c         ENDIF
      ELSE
c        ... Using previously interpolated or constant values of 
c            refractive indeces
         P_refr => RefW_r(:)  ;  P_refi => RefW_i(:)
c         IF (LOG_LEVEL(1) >= N_ESO)  THEN
c            WRITE(LUNIT,*) 
c     $           '  Index of refraction as it Nwl_BRF = ', Nwl_BRF
c            WRITE(LUNIT,*) '  and Nwl_Inp = ', Nwl_Inp
c            DO I = 1,Nwl_BRF
c               WRITE(LUNIT,'(2x,I3,2x,F7.2,2x,f9.6,2x,e15.7)')  
c     $              I, WL_BRF(I), P_refr(I), P_refi(I)
c            ENDDO
c         ENDIF
      ENDIF

      R1_OUT = 0.d0
c     ... Refractive indexes of the air   Incident azimuth
      CN1 = DCMP( 1.d0 )     ;             PhiI  = 0.d0

c      Calculation of the reflection matrix
      DO L = 1,Nwl_Inp
c         WRITE(*,*) 'ref_matr_vec :'
c         WRITE(*,'(2x,I4,2x,F8.2,2x,F6.3)') 
c     $        L, WL_INP(L), P_refr(L)
         CN2 = DCMP( P_refr(L), P_refi(L) ) 
         DO J = 1,N_ref
            Ore = Nvs*(J-1)  ;  Dmur = X_ref(J)
            DO I = 1,N_INC      ! 
               Nin = Nvs*(I-1)
               DO K = 1,N_Phi
                  CALL REF_MATR(CN1, CN2, COSI(I), PhiI, Dmur, X1(K), R)
                  R1_out(Ore+1:Ore+3,Nin+1:Nin+3,K,L) = R(1:3,1:3) 
                  R1_out(Nvs*J,      Nvs*I,      K,L) = R(4,4)
               ENDDO  ! over azimuthal angles (K)
            ENDDO  ! over incidence angles (I)
         ENDDO  ! over reflection angles (J)
      ENDDO  ! over reference wavelengths (L)

      RETURN

      CONTAINS
c***********************************************************************
c     Calculation of the Stokes reflection matrix for illumination
c     from above for a statistically rough surface separating two 
c     half-spaces with refractive indices of the upper and lower 
c     half-spaces equal to CN1 and CN2, respectively.
c     The effect of shadowing is not included in this subroutine.

c      Mui = ABS(cosine of the incident zenith angle)
c      PhiI = incident azimuth angle.
c      Mur = ABS(cosine of the reflection zenith angle).
c      PhiR = reflection azimuth angle
c      R - (4X4) reflection matrix
c***********************************************************************

      SUBROUTINE REF_MATR(CN1, CN2, Mui, PhiI, Mur, PhiR, R)

      USE ST_VARS,     ONLY: Pie
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      COMPLEX (kind(0.0d0)) :: CN1, CN2
      REAL (KND)  Mui, PhiI, Mur, PhiR, R(4,4)
c     ... Local variables
      REAL (KND)  Azm, COS_G, T1, T2, Cos_i, Sin_S, Sca_IR, Zn
      REAL (KND), DIMENSION(4,4):: RF0, Rot_l, Rot_r
      COMPLEX (kind(0.0d0))     :: Rc_L, Rc_P, CC1, CC2, Cos_t
c     ==================================================================

c     ... Relative azimuth
      Azm = PhiR - PhiI

c      IF(DABS(Mur-1D0).LT.1d-9) Mur=0.999999999999d0
c      IF(DABS(Mui-1D0).LT.1d-9) Mui=0.999999999999d0

c     ... Cosine of an angle between direction of incidence and 
c         direction opposite to the reflected beam 
c         (see Eq.(Cos-gc) in STB)
      COS_G = Mur*Mui + DSQRT( (1.d0-Mur**2)*(1.d0-Mui**2) )*
     $     DCOS(Pie + Azm)

c     ... Cosine of angle between direction of incidence and reflected 
c         beam (cosine of scattering angle)
      Sca_IR = DCOS(Pie - DACOS(COS_G))

c     ... This is the cosine of the incident angle with respect to the
c         wave facet normal
      Cos_i = DCOS( DACOS(COS_G)/2.d0 )

      Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Cos_i*Cos_i)/CN2/CN2 )
c     ... Calculate parallel (R_L) and perpendicular (R_P) Fresnel
c         reflection coefficients      
      CC1 = CN2*Cos_i ; CC2 = CN1*Cos_t
      Rc_L = (CC1 - CC2)/(CC1 + CC2)      

      CC1 = CN1*Cos_i ; CC2 = CN2*Cos_t
      Rc_P = (CC1 - CC2)/(CC1 + CC2)

      RF0 = 0.d0
c     ... Fresnel reflection matrix including polarization
      RF0(1,1) = 0.5d0*(CDABS(Rc_L)**2 + CDABS(Rc_P)**2) 
      RF0(1,2) = 0.5d0*(CDABS(Rc_L)**2 - CDABS(Rc_P)**2)
      RF0(2,1) = RF0(1,2) ; RF0(2,2) = RF0(1,1)
      RF0(3,3) = DBLE( Rc_L*CONJG(Rc_P) )
      RF0(4,4) = RF0(3,3)
      RF0(3,4) = AIMAG( Rc_L*CONJG(Rc_P) )
c     ... To account for complex refractive index
      RF0(4,3) = -RF0(3,4)

c     ... Meridional plane coincides with the scattering
c         plane. No rotation is needed.
      IF( REAL(Azm)==REAL(Pie) .OR. REAL(Azm)==0.0 ) THEN 
         R = RF0
         RETURN
      ENDIF

c     ... Initialization of rotational matrices
      Rot_l = 0.d0       ;  Rot_r = 0.d0
      Rot_l(1,1) = 1.d0  ;  Rot_r(1,1) = 1.d0
      Rot_l(4,4) = 1.d0  ;  Rot_r(4,4) = 1.d0
c     ... Convention about positive rotation angle:
c         (rotation clockwise when looking into direction 
c          of propagation)
c        | 1     0        0     0 |
c        | 0  cos(2a)  -sin(2a) 0 |
c        | 0  sin(2a)   cos(2a) 0 |
c        | 0     0        0     1 |
c        Incident and reflected polar angles are measured
c        from the positive direction of z-axis
      Sin_S = 1.d0 - Sca_IR**2
      T1 =  (Mur + Mui*Sca_IR)/DSQRT((1.d0 - Mui**2)*Sin_S)
      T2 = -(Mui + Mur*Sca_IR)/DSQRT((1.d0 - Mur**2)*Sin_S)

      IF( REAL(Mur)== 1.0 ) THEN 
         T2 = -DCOS(Azm) ; T1 = 1.d0
      ENDIF
      IF( REAL(Mui)== 1.0 ) THEN
         T1 =  DCOS(Azm) ; T2 = 1.d0
      ENDIF
      IF( REAL(Mur)==1.0 .AND. REAL(Mui)==1.0) THEN
         T1 = 1.d0 ; T2 = 1.d0
      ENDIF

      Rot_r(2,2) = 2.d0*T1*T1 - 1.d0 
      Rot_l(2,2) = 2.d0*T2*T2 - 1.d0 
c     ... 0 < phi_sca - phi_inc < pi
      Zn = -1.d0
c     ... pi < phi_sca - phi_inc < 2*pi
      IF( Azm > Pie )  Zn = 1.d0

      Rot_r(3,2) = Zn*2.d0*DSQRT( DABS(1.d0 - T1*T1) )*T1
      Rot_l(3,2) = Zn*2.d0*DSQRT( DABS(1.d0 - T2*T2) )*T2

      Rot_l(2,3) = -Rot_l(3,2) ; Rot_l(3,3) =  Rot_l(2,2)
      Rot_r(2,3) = -Rot_r(3,2) ; Rot_r(3,3) =  Rot_r(2,2)

      R = MATMUL(Rot_l,MATMUL(RF0,Rot_r))

      RETURN
      END SUBROUTINE REF_MATR 

      END
