!***************************************************************************
!
!	SCIATRAN - Version 4.2 - Last modification: 13 Mar 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine Replace_Aerosol_ND (LAN, LAS, LAE, NAW, AER_WAVS, ECO, Dens_Prf, *)
  Use Aerosols, Only: AerExC_DB
  Use Grids,    Only: Nlayers, ZZZ
  Use Log_Pars, Only: Lunit, EU
  Use ST_Input, Only: S_N_D, SND_NAME, SND_WL, AER_CDF, AER_ZMIN, AER_ZMAX
  Use ST_Vars,  Only: LATITUDE, LONGITUDE
  Use Tools,    Only: Read_File, Split_String, ErrMsg
  Use Types,    Only: KND, DYN_CHAR_L, DYN_CHAR
  
  Integer  LAN, NAW
  Integer  LAS(LAN), LAE(LAN)
  REAL (KND) Dens_Prf(Nlayers), ECO(2,LAN,NAW,3), AER_WAVS(NAW)
  ! Local  parameters
  Character (Len=*), Parameter :: R_NAME='Replace_Aerosol_ND'
  Integer  II, JJ, IU, I1, I2, NSkip, Nlat, NLon, N_Ex, IRead, &
       Info, ILat, ILon
  REAL (KND) Long, Dc
  REAL (KND) EX_SEC(NLAYERS,NAW), EXB(NLAYERS), Pr_in(NLAYERS)
  REAL (KND), ALLOCATABLE :: Z_Ex(:), Pr_ex(:), DATA_R(:,:)
  TYPE (DYN_CHAR_L), ALLOCATABLE :: ReadArr(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: Msg(:), Str_Arr(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  EX_SEC = 0.d0
  ! Extinction cross-section according to selected aerosol type
  DO II = 1,LAN
     DO JJ = LAS(II),LAE(II)
        EX_SEC(JJ,:) = ECO(1,II,:,1) + ECO(2,II,:,1)
     ENDDO  ! over standard altitude levels
  ENDDO  ! over aerosol layers (I)
  
  Select Case (S_N_D)
  Case ('FILE')
     Call Read_File (2, SND_NAME, DATA_R, *880, CDF=AER_CDF)
     N_Ex = Size(DATA_R,1)
     Allocate(Z_Ex(N_Ex), Pr_ex(N_Ex))
     Z_Ex(:) = DATA_R(:,1) ;  Pr_ex(:) = DATA_R(:,2)
  
  Case ('SCIA_3D')
!     MS_2000 = (Year - 2000)*12 + MONTH
!     NLat = Size(AerExC_DB%Lats) 
     NLon = Size(AerExC_DB%Lons)
!     If(MS_2000 < AerExC_DB%MS_2000) Goto 202
!     NSkip = (MS_2000 - AerExC_DB%MS_2000)*NLat*NLon
     ILat = MINLOC(Abs(Latitude - AerExC_DB%Lats),1)
     If(Longitude < 0) Then
        Long = Longitude + 360.D0
     Else
        Long = Longitude
     End If
     ILon = MINLOC(Abs(Long - AerExC_DB%Lons),1)
     If(ILon == NLon) Then
        If(Abs(Long - AerExC_DB%Lons(NLon)) > 360. - Long) ILon = 1
     End If
!     IRead = (NSkip + (ILat - 1)*Nlon + ILon - 1)*3 + 1
     IRead = ((ILat - 1)*Nlon + ILon - 1)*3 + 1
     CALL SPLIT_STRING(AerExC_DB%Data(IRead + 1)%S, STR_ARR)
     N_Ex = Size(STR_ARR)
     ALLOCATE(Z_Ex(N_Ex), Pr_ex(N_Ex))
     Do II = 1, N_Ex
        Read(STR_ARR(II)%S,*)  Z_Ex(II)
     End Do
     CALL SPLIT_STRING(AerExC_DB%Data(IRead + 2)%S, STR_ARR)
     Do II = 1, N_Ex
        Read(STR_ARR(II)%S,*)  Pr_ex(II)
     End Do
  End Select

  CALL LIN_INT(N_ex, Z_Ex, Pr_Ex, NLAYERS, ZZZ, Pr_in, 'Z')
    
  ! Interpolate extinction cross-section on the reference WL         
  CALL LIN_SEARCH_NEW(SND_WL, Dc, I1, I2, NAW, AER_WAVS)
  EXB(:) = (1.d0 - Dc)*EX_SEC(:,I1) + Dc*EX_SEC(:,I2) 
  WHERE( EXB(:) > 0.d0 .AND. ZZZ(:) >= AER_ZMIN .AND. ZZZ(:) <= AER_ZMAX)
     Dens_PRF(:) = Pr_in(:)/EXB(:)
  END WHERE
  WHERE( EXB(:) == 0.d0) Dens_PRF(:) = 0.d0

  CALL ST_TRACK (R_NAME, 1, 'O')
  Return
  
880 Write(EU, *) ' ---> Called by '//TRIM(R_NAME)
  Return 1
202 Continue
  Allocate(Msg(1))
  Msg(1)%S = 'The selected date is not covered by the SCIA aerosol data base'
  call ErrMsg(R_Name, 'e', Msg)
  Return 1
!***************************************************************************
End Subroutine Replace_Aerosol_ND
!***************************************************************************
