!***************************************************************************
!
!	SCIATRAN - Version 4.2 - Last modification: 05 Nov 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE RET_INIT (*)
! ... Call in "retrieval.f90"

! Retrieval of vertical profiles of atmospheric trace gases and various
! scalar parameters
! The routine create auxilary array needed to initialize the measurement 
! vector and constraint matrix
! The following variables have to be initialized here:
!    STATE(1:N_PAR)%APR(:)    - apriori model parameters
!    STATE(1:N_PAR)%PRE(:)    - model parameters at previous iteration
!    STATE(1:N_PAR)%RES(:)    - space holder for the solution
!    STATE(1:N_PAR)%FACTOR(:) - normalization of the retrieval parameter, 
!                               e.g., to convert to VMR
!    STATE(1:N_PAR)%I_RET_INF - element number in the information array RET_INFO
!    STATE(1:N_PAR)%PAR_NUM   - number of the spectral sub-parameter 
!                               in the RET_INFO element
!    STATE(1:N_PAR)%I_SBP     - number of the altitude sub-parameter
!                               in the RET_INFO element
  USE AEROSOLS,    ONLY: AP_Mie, FACTOR_EXA_S
  USE ALB_PARS,    ONLY: ALBEDO_RET, Nc_ALB
  USE DATA_RET,    ONLY: ITERATIONS_LIMIT, FACTOR_TPS_COL, DOUBLET_FLAG
  USE GRIDS,       ONLY: NL => NLAYERS
  USE INFO,        ONLY: RET_INFO, TG_INFO, STATE
  USE LOG_PARS,    ONLY: EU
  USE PROFILES,    ONLY: AIR_DENSITY
  USE RETR_ITER,   ONLY: ITERATION, TPC_F, GHS
  USE RTM_PARS,    ONLY: RTM_ST, DATA_ITER
  USE ST_VARS,     ONLY: GROUND_HEIGHT
  USE TOOLS,       ONLY: FIND_IDX, SET_PTR
  USE TYPES,       ONLY: KND

  ! Local variables:
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RET_INIT'
  INTEGER  I_RI, KK, IDX, I_SEG, KK_1, I_SBP, I_PAR
  LOGICAL  FLG
  REAL (KND), ALLOCATABLE :: APR(:), PRE(:)
  TYPE (DATA_ITER), POINTER :: RTM_PTR
  !==================================================================
  INTERFACE
     SUBROUTINE INIT_STATE (*, KK, II, A_PRE, A_APR, A_FACT, I_PAR, I_SBP,FLG)
       USE INFO,      ONLY: STATE, WF_DB
       USE TOOLS,     ONLY: FIND_IDX
       USE TYPE_VARS, ONLY: KND       
       INTEGER  II, KK, I_PAR, I_SBP
       REAL (KND) A_PRE(:), A_APR(:), A_FACT(:) 
       LOGICAL FLG
       OPTIONAL :: A_PRE,  A_APR, A_FACT, I_PAR, I_SBP,FLG
     END SUBROUTINE INIT_STATE
  END INTERFACE

  CALL ST_TRACK (R_NAME, 2, 'I')
  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'This routine is not a part ' &
  //'of the freely distributed software package')
 
  STOP
  RETURN
END SUBROUTINE RET_INIT
