!**************************************************************************
!
!	SCIATRAN - Version 4.4 - Last modification: 09 May 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
SUBROUTINE RETRIEVAL(NSEGS, SPECSEGS, NEXT_ITERATION, NEW_ALT_GRID, &
     NEXT_PIX, N_pix, *)
!     ... Called by "gt_iface.f"

  USE ALB_PARS,    ONLY: ALBEDO_CONST
  USE AEROSOLS,    ONLY: FACTOR_EXA_S
  USE BRDF,        ONLY: Sep_IR
  USE CLOUDS_RETR, ONLY: RET_QUALITY
  USE DATA_RET,    ONLY: ITERATIONS_LIMIT, CRIT_RMS, DOUBLET_FLAG, MDS, &
                         V_INV_FLG, PRESET_APR_NAME, ALLOW_END, TIH_FLAG, &
                         LMD, FLG_OUTL, ITER_OUTL, CRIT_OUTL, N_svc, TihRegMode
  USE GRIDS,       ONLY: NLAYERS, ZZZ
  USE INFO,        ONLY: RET_INFO, WF_DB, STATE
  USE OutInf,      ONLY: EU, LUNIT, LOG_LEVEL, N_RSO, N_SSO, N_DSO, &
                         DIR_OUT, WrtRetFit
  USE RETR_ITER,   ONLY: ITERATION, RMS_SAVE, I_MAP, RMS_ALL, N_OSC, MAX_OSC, &
                         P_MIN, Decr, RMS_OSC, FLG_OSC, SCAL_APR, N_GET, N_NLIN
  USE RTM_PARS,    ONLY: DATA_ITER
  USE ST_VARS,     ONLY: RET_MODE, RET_CORE, Pie
  USE SPHER_PARS,  ONLY: NLO => N_LOS_OUT
  USE TOOLS,       ONLY: INQ_WF, SET_PTR
  USE TYPES,       ONLY: KND, PREPROC_DATA, XVecType, RegMatrs, MeasInfo

  INTEGER  NSEGS, N_pix, SPECSEGS(2,NSEGS)
  LOGICAL  NEW_ALT_GRID, NEXT_ITERATION, NEXT_PIX
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RETRIEVAL'
  LOGICAL, ALLOCATABLE :: TMP_MASK(:,:)
  INTEGER  I_E, I_S, II, LL, N_PAR, N_X, N_Y, I_SBP, IDS, NDS, I_L, I_SP
  INTEGER  LLs, Isc
  INTEGER  I, I_KE, I_KS, KK, I_ST, I_RI, ITR, N_SP, NN, K_max, N_REM, N_USE
  INTEGER  NY_DS(SIZE(MDS))
  Logical  CnvChk
  REAL (KND)  RMS_NEW, RMS_NEW_LIN, Del, DCFR, Cut, CF_ADD, RES_REL(NLO)
  REAL (KND), Allocatable :: TMP(:)
  TYPE (PREPROC_DATA) PP_DAT(SIZE(MDS), NLO)
  Type (XVecType), Allocatable :: XVec(:)
  Type (RegMatrs) :: RegM
  Type (MeasInfo) :: MsmtInf
  Type (DATA_ITER), POINTER :: RTM_PTR
  !==================================================================
  Interface
     Subroutine Generate_X (XVec)
       USE TYPES,     ONLY: XVecType
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine Generate_X

     Subroutine GENERATE_SA (XVec, RegM)
       USE TYPES, ONLY: XVecType, RegMatrs
       Type (RegMatrs) RegM
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine GENERATE_SA
     
     Subroutine Iterate_V2 (MsmtInf, XVec, RegM, RMS1, RMS2)
       USE TYPES, ONLY: KND, XVecType, RegMatrs, MeasInfo
       REAL (KND) RMS1, RMS2
       Type (RegMatrs) RegM
       Type (MeasInfo) MsmtInf
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine Iterate_V2
     
     Subroutine Write_Fit (KN, XVec, NY_DS, PP_DAT)
       USE DATA_RET,   ONLY: MDS
       USE SPHER_PARS, ONLY: N_LOS_OUT
       USE TYPES, ONLY: KND, PREPROC_DATA, XVecType
       REAL (KND), Allocatable :: KN(:,:)
       INTEGER  NY_DS(SIZE(MDS))
       TYPE (PREPROC_DATA) PP_DAT(SIZE(MDS), N_LOS_OUT)
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine Write_Fit

     Subroutine Ret_Results (XVec, NEXT_ITER, NEW_ALT_GRID, *)
       USE TYPES, ONLY: XVecType
       LOGICAL  NEXT_ITER, NEW_ALT_GRID
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine Ret_Results
     
     Subroutine Update_X (XVec, NEXT_ITERATION)
       USE TYPES,    ONLY: XVecType
       LOGICAL  NEXT_ITERATION 
       Type (XVecType), Allocatable :: XVec(:)
     End Subroutine Update_X
     
  End Interface
  !==================================================================
  CALL ST_TRACK (R_NAME, 2, 'I')
  CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'This routine is not a part ' &
  //'of the freely distributed software package')
 
  STOP
  RETURN
END SUBROUTINE RETRIEVAL
