C************************************************************************
C
C	SCIATRAN - Version 4.3 - Last modification: 08 Dec 2020
C	Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c               Right-hand side of the forward RTE
c
c************************************************************************
      SUBROUTINE RHS_DRTE(FLAG, Mh, NrL, NN, NSZA, XM, AXM, UDB,
     $     Mus, EXS, ADD_PLANCK, PLANCK_SURF, SVEC_AG, PART_U, PART_D)
c     ... Called by "solv_dom.f"

      USE FLU_LAND,    ONLY: FLU_L
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_ESO, DIR_OUT
      USE ST_VARS,     ONLY: N_RHS, Pie, VST, Finc, Kvs, AA_BRF,
     $                       SURF_TYPE
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: UDB_TYPE
c     ------------------------------------------------------------------

      CHARACTER (LEN=1), INTENT(IN) :: FLAG 
      INTEGER,           INTENT(IN) :: Mh, NrL, NN, NSZA
      LOGICAL,           INTENT(IN) :: ADD_PLANCK
      REAL (KND)  SVEC_AG(2*Nvs*NN*(NrL-1),N_RHS)
      REAL (KND), INTENT(IN) :: PLANCK_SURF,  
     $     EXS(NSZA), MuS(NSZA), AXM(NN), XM(NN),
     $     PART_U(NrL,2*Nvs*NN,Kvs*NSZA), PART_D(NrL,2*Nvs*NN,Kvs*NSZA)
      TARGET PART_D, PART_U
c     ------------------------------
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     Local variables
c     ---------------
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'RHS_DRTE'
      INTEGER     I, Ig, Ivs, J, Jvs, Js, K, K1, Ks, NNv, LL1, LR1
      REAL (KND)  S1, S2, S2_BRF(Nvs*NN), HDU(Nvs*NN)
      REAL (KND), POINTER :: Pps(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = Nvs*NN
      SELECT CASE( FLAG )
      CASE('A','a') 
         DO Js = 1,Kvs*NSZA
            SVEC_AG(1:NNv,Js) = -PART_U(1,1:NNv,Js)
        ENDDO
      CASE('O','o') 
         SELECT CASE( TRIM(UDB%UBC) )
         CASE('Flat water')
            DO Js = 1,NSZA
c              ... Vector case 
               DO Ig = 1, NN  ;  Ks = (Ig-1)*Nvs
                  DO K = 1, Nvs
                     SVEC_AG(Ks+K,Js) = -PART_U(1,Ks+K,Js) +
     $                    SUM( UDB%Fre_Rv(Ig,K,1:Nvs)
     $                    * PART_U(1,NNv+Ks+1:NNv+Ks+Nvs,Js) )   
                  ENDDO  ! over Stokes components (K)
               ENDDO  ! over Gauss angles (Ig)
c              --------------            
            ENDDO  ! over solar angles (Js)

         CASE('Rough water')
            Pps => PART_U(1,NNv:2*NNv,:)
            DO Js = 1,NSZA 
               DO K = 1,Nvs    
                  HDU(K:NNv:Nvs) = Pps(K:NNv:Nvs,Js)*AXM(1:NN)/XM(1:NN)
               ENDDO            ! over Stokes components (K)
c               write(*,*) '   Test PART_U in rhs_drte'
c              ... Sign for U and V components of the Stokes vector
               IF( Nvs > 1 ) THEN
                  HDU(3:NNv:Nvs) = -HDU(3:NNv:Nvs) 
                  HDU(4:NNv:Nvs) = -HDU(4:NNv:Nvs) 
               ENDIF
               DO K = 1,NNv
                  SVEC_AG(K,Js) = -PART_U(1,K,Js)
     $                 + SUM( UDB%Rw_GG(K,:,Mh)*HDU(1:NNv) )*2.d0*Pie 
               ENDDO
            ENDDO
         END SELECT
      END SELECT

      DO Js = 1,Kvs*NSZA
         DO I = 1,NrL - 2
            LL1 = (2*I-1)*NNv+1 ; LR1 = (2*I+1)*NNv
            SVEC_AG(LL1:LR1,Js) = PART_U(I+1,:,Js) - PART_D(I,:,Js)
         END DO
      ENDDO

      LL1 = (2*NrL-3)*NNv+1 ; LR1 = 2*NNv*(NrL-1)
c     ... Particular solution in boundary layer
      Pps => PART_D(NrL-1,:,:)
      DO Js = 1,Kvs*NSZA
         SVEC_AG( LL1:LR1,Js ) = -Pps(NNv+1:2*NNv,Js)
      ENDDO

c     ... Change sign of U and V component in the right-hand side
c         of the linear algebraic system matrix
      IF( Nvs>1 ) THEN
         DO K = 1,NN
            DO K1 = 4*K-2, 4*K-1
               SVEC_AG(K1+LL1,1:Kvs*NSZA) = -SVEC_AG(K1+LL1,1:Kvs*NSZA)
            ENDDO
         ENDDO
      ENDIF
      
c      WRITE(*,*) '   rhs_drte: TRIM(UDB%LBC) ====>  ', TRIM(UDB%LBC) 
c     ================================================================
c           R.h.s of the matrix equation in the case of 
c         Lambertian reflection, BRDF, and ocean surface
c     ================================================================
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
c         write(*,*) '   Option "Lambert"  '
c         write(*,'(A,2x,e12.4)') '   EXS(I) at medium bot. = ', EXS(1)
c         write(*,*) '             =================== '
         IF( Mh == 1 ) THEN
c           ... In the case of Lambertian reflector the first Stokes
c               component only will be reflected
            IF( .NOT.AA_BRF ) THEN
               DO Js = 1,NSZA
                  S1 = VST*MuS(Js)*EXS(Js)*Finc(1,1)
                  S2 = SUM( Pps(1:NNv:Nvs,Js)*AXM(1:NN) )
                  DO J = LL1,LR1,Nvs
                     SVEC_AG(J,Js) = SVEC_AG(J,Js) 
     $                    + UDB%ALBEDO*(S1 + 2.d0*S2)
                  ENDDO
               ENDDO
            ENDIF  ! .NOT.AA_BRF
         ENDIF  ! Mh == 1

      CASE('Bidirect')
c         WRITE(*,*) '  ------> rhs_drte.f NSZA = ', NSZA
c         WRITE(*,*) '                   AA_BRF = ', AA_BRF         
         DO Js = 1,NSZA
            S1 = VST*MuS(Js)*EXS(Js) ; Ks = (Js-1)*Nvs
            DO Ivs = 1,Kvs
               Jvs = (Ivs-1)*NSZA + Js
               DO K = 1,Nvs    
                  HDU(K:NNv:Nvs) = Pps(K:NNv:Nvs,Jvs)*AXM(1:NN)
               ENDDO  ! over Stokes components (K)
               DO K = 1,NNv 
                  S2_BRF(K) = SUM( UDB%BRF_GG(K,:,Mh)*HDU(1:NNv) )
               ENDDO  ! over Gaussian angles (K)
c              ... Contribution of direct solar light reflection
               DO K = 1,NNv
                  HDU(K) = SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
               ENDDO  ! over Gaussian angles (K)
c               WRITE(*,*)  Js, ACOS(MuS(Js))/Pie * 180., SUM( HDU )               
c              ... Exclude surface reflection if AA_BRF=.TRUE.
               IF( .NOT.AA_BRF ) SVEC_AG(LL1:LR1,Jvs) = 
     $              SVEC_AG(LL1:LR1,Jvs) + 2.d0*S2_BRF(:) + S1*HDU(:)
            ENDDO  ! over DO-solutions (Ivs)
         ENDDO  ! over solar angles (Js, Ks)

      CASE('Flat water')
c        ... In the case of specular reflection the direct reflected
c            from the surface solar radiation is extracted in the form 
c            of delta-function. 
         DO Js = 1,NSZA
            DO Ig = 1, NN  ;  Ks = (Ig-1)*Nvs
               DO K = 1, Nvs
                  HDU(Ks+K) =
     $                 SUM( UDB%Fre_Rv(Ig,K,:)*Pps(Ks+1:Ks+Nvs,Js) )
               ENDDO            ! over Stokes components (K)
            ENDDO  ! over Gauss angles (Ig)
            S2_BRF(1:NNv) = HDU(1:NNv)
            K = 1
            DO J = LL1,LR1  
               SVEC_AG(J,Js) = SVEC_AG(J,Js) + S2_BRF(K)  !*2.d0
               K = K + 1
            ENDDO
            IF( SURF_TYPE /= 'melt_pond' ) CYCLE
            
c           ... Diffuse radiation from melt pond
            S1 = VST*MuS(Js)*EXS(Js) ; Ks = (Js-1)*Nvs
            Jvs = Js
            DO K = 1,Nvs    
               HDU(K:NNv:Nvs) = Pps(K:NNv:Nvs,Jvs)*AXM(1:NN)
            ENDDO               ! over Stokes components (K)
            DO K = 1,NNv 
               S2_BRF(K) = SUM( UDB%BRF_GG(K,:,Mh)*HDU(1:NNv) )
            ENDDO               ! over Gaussian angles (K)
            DO K = 1,NNv
               HDU(K) = SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(1,:))
            ENDDO  ! over Gaussian angles (K)
            IF( .NOT.AA_BRF ) SVEC_AG(LL1:LR1,Js) = 
     $           SVEC_AG(LL1:LR1,Js) + 2.d0*S2_BRF(:) + S1*HDU(:)

         ENDDO ! over solar angles (Js)

      CASE('Rough water')
c         write(*,*) '   Option "Rough water"  '
c         write(*,'(A,2x,e12.4)') '   EXS(I) at medium bot. = ', EXS(1)
c         write(*,*) '             =================== '
         DO I = 1,NSZA  ;  Ks = (I-1)*Nvs
            DO K = 1,Nvs 
               HDU(K:NNv:Nvs) = 
     $              Pps(K:NNv:Nvs,I)*AXM(1:NN)/XM(1:NN) ! AG !
            ENDDO  ! over Stokes components (K)
            DO K = 1,NNv   ! First dimension of Ra_GG, Ra_GS is NN*Nvs
               S2_BRF(K) = 2.d0*Pie
     $             * SUM( UDB%Ra_GG(K,:,Mh)*HDU(1:NNv) )
     $             + SUM( UDB%Ra_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(1,:) )
     $              *EXS(I)*Pie*VST
            ENDDO
            K = 1
            DO J = LL1,LR1  ! all elements for vector case
               SVEC_AG(J,I) = SVEC_AG(J,I) + S2_BRF(K) 
               K = K + 1
            ENDDO
         ENDDO  ! over SZA (I)
c         write(*,*) 
c     $        '   Test output from "rhs_drte.f" mode "Rough water"'
c         OPEN(44, file=DIR_OUT//'/Ref_Ra.dat' )
c            DO K = 1,NNv 
c               WRITE(44,'(2x,F7.3,2(2x,e15.7))')
c     $              DACOS(XM(K))/Pie*180., 
c     $              2.D0*SUM( UDB%Ra_GG(K,:,Mh)*HDU(1:NNv) )*Pie,
c     $              UDB%Ra_GS(K,1,Mh)*EXS(1) *Pie
c            ENDDO
c         CLOSE(44)
c         STOP
      END SELECT

c     ... Thermal emission from surface
      IF( ADD_PLANCK ) THEN
         DO J = LL1,LR1,Nvs
            SVEC_AG(J,1:NSZA) = SVEC_AG(J,1:NSZA) + VST*PLANCK_SURF
         ENDDO
      ENDIF

c     ... Land fluorescence spectra
      IF( FLU_L .AND. Mh==1 ) THEN
         DO J = LL1,LR1,Nvs
            SVEC_AG(J,:) = SVEC_AG(J,:) + VST*UDB%Flu_int
         ENDDO
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE  RHS_DRTE
c **********************************************************************
