!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 02 Feb 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE RHS_FORM (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, AL0_OUT, PCC, &
     AL_PC, SCA, SCR, LocArrs, OSM, LVG, LSG, FLG_WF, *)
  ! Called by "ps_rad.f90"

  USE ABS_COEFF_PC, ONLY: PC_ABS_COEFF
  USE GRIDS,        ONLY: NLAYERS, ZZZ, Z_INV, NL_FINE, Z_FINE
  USE INTERN_DIMS, ONLY: NL_WF, N_WF_L_MS, NL_PC, NL_FINE_PC, N_MOM => NLG_ALL_A
  USE IRRADIANCE
  USE LOG_PARS,    ONLY: EU
  USE LOS_Angles,  ONLY: DirSun
  USE NODES
  USE RTM_OUTPUT,  ONLY: WF_TG_FLAG
  USE ST_INPUT,    ONLY: APH_TYPE
  USE ST_VARS,     ONLY: WF_MS_FLAG, EARTH_RADIUS, DO_REFRAC, DO_PC, F_LGI_EXT,&
                         PreCalcAngs, WF_MS_FLAG
  USE SPHER_PARS,  ONLY: NN
  USE TYPES,       ONLY: KND, PCC_TYPE, OSM_TYPE, FD_LOCAL, LOS_SOLAR_GEOM, &
                         LOS_VIEW_GEOM

  LOGICAL     FLG_WF
  REAL (KND)  DEP_CF(3), ASYM_COEFF(NLAYERS), AL0_OUT(NL_FINE)
  REAL (KND), DIMENSION (NLAYERS) :: AL_PC, A_C_1, A_C_2, SCA, SCR
  TYPE (PCC_type) :: PCC    
  TYPE (OSM_type) :: OSM    
  TYPE (FD_LOCAL) :: LocArrs    
  TYPE (LOS_VIEW_GEOM) :: LVG
  TYPE (LOS_SOLAR_GEOM) :: LSG

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RHS_FORM'
  LOGICAL, ALLOCATABLE :: REFR_FLAG(:)
  INTEGER     J, I, N_DS
  INTEGER, ALLOCATABLE :: KK(:)
  INTEGER     IDX(NLAYERS)
  REAL (KND), DIMENSION (NLAYERS) :: SOL_EXP, C_SZA, H0
  REAL (KND)  C_G(NLAYERS, N_NODE)
  REAL (KND), ALLOCATABLE :: TH(:)
  INTEGER, ALLOCATABLE :: IOB(:)
  LOGICAL, ALLOCATABLE :: REFR_DSB(:)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  C_SZA(:) = LSG%C_SZA(:)
         
  IF(DO_PC) &
       CALL PC_ABS_COEFF (LVG%ISEL, C_SZA, NLAYERS, PCC, .FALSE., .FALSE., AL_PC)

  CALL DirSun (NLAYERS, C_SZA, LSG%TH, ZZZ, LVG%IOB, AL_PC, AL0_OUT, PCC, &
       SOL_EXP, LSG%REFR_DSB, PreCalcAngs, FLG_WF, LocArrs%DS_SUN, LSG%DSB)

  FORALL(J = 1:NLAYERS) C_G(J,:) = - MJU_NODE(:)*C_SZA(J) + &
       SIN_NODE(:)*SQRT(1.0D0 - C_SZA(J)**2)*COS(PHI_NODE(:))

  IF( APH_TYPE=='mie' ) THEN
     CALL SCAT_MATR(NLAYERS)
  ELSE
     FORALL(J = 1:NLAYERS) LocArrs%SSSF(J,:) = &
          ((DEP_CF(1) + DEP_CF(2)*C_G(J,:)*C_G(J,:))*SCR(J) &
          + A_C_2(J)*SCA(J)/(A_C_1(J) - ASYM_COEFF(J)*C_G(J,:))/&
          SQRT(A_C_1(J) - ASYM_COEFF(J)*C_G(J,:)))/4.d0*SOL_EXP(J)/AL_PC(J)
  ENDIF

  LocArrs%CSZA_Surf = C_SZA(NLAYERS)
  LocArrs%EXM_NL = SOL_EXP(NLAYERS)*LocArrs%CSZA_Surf
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
880 WRITE(EU, *) ' ---> Called by '//R_NAME
  RETURN 1  
CONTAINS
!***************************************************************************
  SUBROUTINE SCAT_MATR(NLN)
    USE AEROSOLS,    ONLY: IDX_AER
    INTEGER     J, K, NLN
    REAL (KND)  T1, T2, Plm_ss(N_MOM,NLN)
    REAL (KND), POINTER :: COF(:,:)
  !==================================================================
    COF => OSM%AER_ALL(:,:,1)
    DO K = 1, N_NODE
       CALL WIGNER_PLG(C_G(1:NLN,K), NLN, 0, N_MOM, Plm_ss )
       DO J = 1, NLN 
          LocArrs%SSSF(J,K) = ( SUM( Plm_ss(:,J)*COF(:,IDX_AER(J)) )*SCA(J) &
               + (DEP_CF(1) + DEP_CF(2)*C_G(J,K)*C_G(J,K)) &
               *SCR(J) )/4.d0*SOL_EXP(J)/AL_PC(J)
       ENDDO   ! over standard layers (J)
    ENDDO    ! over nodes (K)

    RETURN
  END SUBROUTINE SCAT_MATR
!***************************************************************************
END SUBROUTINE RHS_FORM
!***************************************************************************
