!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 02 Feb 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE RHS_WF_FORM (AL_PC, LocArrs, S_EXP, RHS, PHASE_FUNC, SURF_REFL, &
     RHS_WF)

  USE NODES, ONLY: N_NODE, MJU_NODE, N_PLUS
  USE GRIDS, ONLY: NLAY=>NLAYERS, DZR
  USE TYPES, ONLY: KND, FD_LOCAL

  REAL (KND)  AL_PC(NLAY), S_EXP(NLAY,N_NODE), RHS(N_NODE*NLAY)
  REAL (KND)  PHASE_FUNC(NLAY, N_NODE, N_NODE), SURF_REFL(N_PLUS)
  REAL (KND)  RHS_WF(NLAY*N_NODE, NLAY)
  TYPE (FD_LOCAL) :: LocArrs
  
  ! Local variables
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RHS_WF_FORM'
  INTEGER     I, J, JJ, IK, IPS, IPE, IMS, IME, JS, JE
  REAL (KND)  SF(NLAY, N_NODE)
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')
  
  DO J = 1, NLAY
     DO IK = 1, N_NODE
        SF(J,IK) = DOT_PRODUCT(RHS((J - 1)*N_NODE + 1:J*N_NODE),&
             PHASE_FUNC(J,:,IK))
     END DO
  END DO

  SF = SF + LocArrs%SSSF
  ! SF = LocArrs%SSSF

  IPS = 1 ;  IPE = N_PLUS  ; IMS = N_PLUS + 1 ;  IME = N_NODE
  ASSOCIATE (SSSF_PL => LocArrs%SSSF(:,IPS:IPE),&
       SSSF_MI => LocArrs%SSSF(:,IMS:IME), DSS => LocArrs%DS_SUN)
    DO I = 1, NLAY         ! Perturbation level
       DO J = 1, NLAY - 1     ! Height grid level
          JJ = (J - 1)*N_NODE
          RHS_WF(JJ + 1:JJ + N_PLUS,I) = (SSSF_PL(J,:)*DSS(J,I) + &
               SSSF_PL(J + 1,:)*DSS(J + 1,I))*(S_EXP(J,IPS:IPE) - 1.0)
       END DO
        
       DO J = 2, NLAY     ! Height grid level
          JJ = (J - 1)*N_NODE
          RHS_WF(JJ + N_PLUS + 1:JJ + N_NODE,I) = &
               (SSSF_MI(J - 1,:)*LocArrs%DS_SUN(J - 1,I) + &
               SSSF_MI(J,:)*DSS(J,I))*(1.0 - S_EXP(J,IMS:IME))
       END DO
       RHS_WF(:,I) = RHS_WF(:,I)*AL_PC(I)/2.0D0
    END DO
  END ASSOCIATE

  DO J = 1, NLAY - 1     ! Height grid level
     JS = (J - 1)*N_NODE + IPS ;  JE = (J - 1)*N_NODE + IPE
     RHS_WF(JS:JE,J) = RHS_WF(JS:JE,J) - S_EXP(J,IPS:IPE)*DZR(J)/&
          MJU_NODE(IPS:IPE)*AL_PC(J)*(SF(J,IPS:IPE) + SF(J + 1,IPS:IPE))/&
          4.0D0 + SF(J,IPS:IPE)*(S_EXP(J,IPS:IPE) - 1.0)/2.0D0 + RHS(JS:JE)*&
          S_EXP(J,IPS:IPE)/2.0D0*DZR(J)/MJU_NODE(IPS:IPE)*AL_PC(J)
  END DO
  DO J = 2, NLAY     ! Height grid level
     JS = (J - 1)*N_NODE + IMS ;  JE = (J - 1)*N_NODE + IME
     RHS_WF(JS:JE,J) = RHS_WF(JS:JE,J) - S_EXP(J,IMS:IME)*DZR(J - 1)/&
          MJU_NODE(IMS:IME)*AL_PC(J)*(SF(J - 1,IMS:IME) + SF(J,IMS:IME))/&
          4.0D0 + SF(J,IMS:IME)*(1.0 - S_EXP(J,IMS:IME))/2.0D0 + RHS(JS:JE)*&
          S_EXP(J,IMS:IME)*DZR(J - 1)/MJU_NODE(IMS:IME)*AL_PC(J)/2.0D0
  END DO
      
  DO J = 2, NLAY        ! Hight grid level
     JS = (J - 2)*N_NODE + IPS ;  JE = (J - 2)*N_NODE + IPE
     RHS_WF(JS:JE,J) = RHS_WF(JS:JE,J) - S_EXP(J - 1,IPS:IPE)*DZR(J - 1)/&
          MJU_NODE(IPS:IPE)*AL_PC(J)*(SF(J - 1,IPS:IPE) + SF(J,IPS:IPE))/&
          4.0D0 + SF(J,IPS:IPE)*(S_EXP(J - 1,IPS:IPE) - 1.0)/2.0D0 + RHS(JS:JE)&
          *S_EXP(J - 1,IPS:IPE)/2.0D0*DZR(J - 1)/MJU_NODE(IPS:IPE)*AL_PC(J)
  END DO

  DO J = 1, NLAY - 1     ! Hight grid level
     JS = J*N_NODE + IMS ;  JE = J*N_NODE + IME
     RHS_WF(JS:JE,J) = RHS_WF(JS:JE,J) - S_EXP(J + 1,IMS:IME)*DZR(J)/&
          MJU_NODE(IMS:IME)*AL_PC(J)*(SF(J,IMS:IME) + SF(J + 1,IMS:IME))/&
          4.0D0 + SF(J,IMS:IME)*(1.0 - S_EXP(J + 1,IMS:IME))/2.0D0 + RHS(JS:JE)*&
          S_EXP(J + 1,IMS:IME)*DZR(J)/MJU_NODE(IMS:IME)*AL_PC(J)/2.0D0
  END DO


  DO I = 1, NLAY         ! Perturbation level
     JJ = (NLAY - 1)*N_NODE
     DO IK = 1, N_PLUS
        RHS_WF(JJ + IK,I) = - SURF_REFL(IK)*LocArrs%EXM_NL* &
             LocArrs%DS_SUN(NLAY,I)*AL_PC(I)
     END DO
  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE RHS_WF_FORM
!***************************************************************************
