C********************************************************************C
C
C	SCIATRAN - Version 2.0 - Last modification: 21 Jun 2003
C
C	Copyright (C) 1995-1998, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE RING_INTERFACE (NSPEC, WL, SPEC_OUT, DEL_RRS, 
     $     RAD_DIR_FL, PARALLEL_WL, *)

      USE CLD_RET, ONLY : PIE ;  USE  LOG_CONTROL
      USE SPHER_PARS, ONLY : N_LOS_OUT, N_SOL_ANG, N_ITER
      USE GRIDS,         ONLY : NLAYERS, MAX_LINE, ALT_EXT, NSOL
      USE PROFILES,      ONLY : TEMP
      USE RAMAN, R_JB => RAMAN_JBEG , R_JE => NR_LINES, 
     $     R_JD => RAMAN_JDELTA
      USE GAS_CONTROL ;  USE DEPOLARIZATION_ALBEDO_AEROSOL
      USE IRRADIANCE  ;  USE ESFT, ONLY : DO_ESFT ;  USE NODES
      USE ST_VARS, ONLY: F_LGI_SCR, F_LGI_SCA, F_LGI_EXT, AKIMA_FLG
      
      CHARACTER (LEN = 14), PARAMETER :: ROUTINE_NAME = 'RING_INTERFACE'

      LOGICAL             PARALLEL_WL, ERROR

      INTEGER             NSPEC, J_RING, I_GAS, JPOS, N_J_MAX
      INTEGER             I_ACT, I_CEN, N_RING, I
      INTEGER, ALLOCATABLE :: GAS_NUM(:)

      DOUBLE PRECISION    WL(NSPEC), SPEC_OUT(NSPEC, 1, N_LOS_OUT)
      DOUBLE PRECISION    DEL_RRS(NSPEC, N_LOS_OUT)
      DOUBLE PRECISION    DUM(1,1,1), RAD_DIR_FL(N_SOL_ANG, NSPEC)
      DOUBLE PRECISION    CONC(N_RG, NLAYERS)
      DOUBLE PRECISION    GAMMA_VAL(N_RG), S_TMP(NLAYERS)
      DOUBLE PRECISION, ALLOCATABLE :: DELTA_WL(:), WL_ALL(:), N_j(:),
     $     StateN(:,:), Sigma(:,:), WL_LOSS(:), SC_RRS(:,:), 
     $     RHS_RRS(:,:,:), SF_RAMAN(:,:)
      DOUBLE PRECISION    StateSum, PLACZEK, WL_FACTOR


      INTERFACE
         SUBROUTINE SPHER_RAD (*, JPOS, POS, DO_AMF, CWL_FLAG,
     $        WF_OUT_TOTAL, SC_RRS, RHS_RRS, SF_RAMAN, RAD_TOTAL, 
     $        RAD_DIR_FL)
         USE GAS_CONTROL ;  USE GRIDS
         USE SPHER_CONTROL, ONLY : N_LOS_OUT, ANG_OUT, N_SOL_ANG
         LOGICAL             DO_AMF, CWL_FLAG
         INTEGER             JPOS 
         DOUBLE PRECISION    RAD_TOTAL(N_LOS_OUT), POS
         DOUBLE PRECISION, DIMENSION (:,:,:) :: WF_OUT_TOTAL, RHS_RRS
         DOUBLE PRECISION, DIMENSION (:,:) :: SC_RRS, SF_RAMAN
         DOUBLE PRECISION, TARGET :: RAD_DIR_FL(N_SOL_ANG)
         OPTIONAL :: RAD_TOTAL, RAD_DIR_FL
         END SUBROUTINE
      END INTERFACE

      IRRAD_FLAG = .TRUE.

      IF(.NOT.DO_DEPOL_WAVELENGTH)  THEN
         WRITE(LUNIT,*) 'DO_DEPOL_WAVELENGTH'//
     $     'must be TRUE to perform Raman calculations !'
         STOP
      END IF
      IF(N_ITER > 1) THEN
         WRITE(LUNIT,*) 'Raman is avalable in CDI mode only !'
         STOP
      END IF
      IF(DO_ESFT) THEN
         WRITE(LUNIT,*) 'Raman is not avalable in ESFT mode !'
         STOP
      END IF

      CONC(1,:) = O2_CONC(:)
      CONC(2,:) = O2_CONC(:)*PPMV_N2_O2

      N_WL_C = SUM(2*((R_JE - R_JB(:))/R_JD(:) + 1) - IDIM(2,R_JB(:)))

      N_J_MAX = MAXVAL((R_JE - R_JB(:))/R_JD(:) + 1)
      ALLOCATE(DELTA_WL(N_WL_C), GAS_NUM(N_WL_C), SC_RRS(NLAYERS,4), ! A
     $     N_J(0:N_J_MAX), StateN(NLAYERS, 0:N_J_MAX), ! A
     $     WL_LOSS(N_WL_C),     ! A
     $     WL_ALL(NSPEC*(N_WL_C + 1)), RHS_RRS(NLAYERS, N_NODE, NSOL), ! A
     $     SF_RAMAN(N_LOS_OUT, MAX_LINE), SIGMA(N_WL_C,NLAYERS)) ! A

      StateN = 0.0D0
      J_RING = 0
      DO I_GAS = 1,N_RG

         DO I = 1, NLAYERS
            DO I_CEN = R_JB(I_GAS), R_JE, R_JD(I_GAS)
               N_j(I_CEN) = SPIN_WEI(I_GAS,MOD(I_CEN,2) + 1)*
     $              (2.0D0*I_CEN + 1.0D0)*DEXP(-CtimesH*RAMAN_B0(I_GAS)*
     $              I_CEN*(1.0D0 + I_CEN)/Boltzmann/TEMP(I))
            END DO 
            StateSum = SUM(N_j(R_JB(I_GAS):R_JE:R_JD(I_GAS)))
            StateN(I, R_JB(I_GAS):R_JE:R_JD(I_GAS)) = 
     $           N_j(R_JB(I_GAS):R_JE:R_JD(I_GAS))/StateSum
         ENDDO

         DO I_CEN = R_JE, R_JB(I_GAS), - R_JD(I_GAS)
            J_RING = J_RING + 1
            GAS_NUM(J_RING) = I_GAS
            DELTA_WL(J_RING) = - RAMAN_B0(I_GAS)*(4.0D0*I_CEN + 6.0D0) !ASt 
            PLACZEK  = 1.5D0*(I_CEN + 1.0D0)*(I_CEN + 2.0D0)/ 
     $           (2.0D0*I_CEN + 1.0D0)/(2.0D0*I_CEN + 3.0D0)
            Sigma(J_RING,:) = CONC(I_GAS,:)*StateN(:,I_CEN)*PLACZEK
         END DO
         DO I_CEN = R_JB(I_GAS), R_JE, R_JD(I_GAS)
            IF(I_CEN < 2) CYCLE
            J_RING = J_RING + 1
            GAS_NUM(J_RING) = I_GAS
            DELTA_WL(J_RING) = RAMAN_B0(I_GAS)*(4.0D0*I_CEN - 2.0D0) !St
            PLACZEK = 1.5D0*I_CEN*(I_CEN - 1.0D0)/
     $           (2.0D0*I_CEN + 1.0D0)/(2.0D0*I_CEN - 1.0D0) 
            Sigma(J_RING,:) = CONC(I_GAS,:)*StateN(:,I_CEN)*PLACZEK
         END DO

      END DO

      FORALL(I_ACT = 1:NSPEC, I_CEN =  1:N_WL_C) 
     $     WL_ALL((I_ACT - 1)*(N_WL_C + 1) + I_CEN) = 
     $     1.0D+9/(1.0D+9/WL(I_ACT) - DELTA_WL(I_CEN))

      FORALL(I_ACT = 1:NSPEC) 
     $     WL_ALL(I_ACT*(N_WL_C + 1)) = WL(I_ACT)

      N_RING = NSPEC*(N_WL_C + 1)
      CALL ABS_COEFF_READ (N_RING, WL_ALL, *880)

      WL_FACTOR = 256.0D+4*PIE**5.0D0/27.0D0*1.0D+36

c!$OMP PARALLEL PRIVATE(JPOS, I_ACT, I_CEN, I_GAS, WL_LOSS, GAMMA_VAL,
c!$OMP+   RHS_RRS, SF_RAMAN, S_TMP, SC_RRS) IF(PARALLEL_WL)
c!$OMP DO SCHEDULE(STATIC)
      DO I_ACT = 1, NSPEC

         WL_LOSS(:) = 1.0D+9/(1.0D+9/WL(I_ACT) + DELTA_WL(:))

         DO I_GAS = 1,N_RG
            CALL AKIMA_INTERP (DEP_NWS, DEP_WS, 
     $           R_GAM(I_GAS,:), 1, WL(I_ACT), GAMMA_VAL(I_GAS), ERROR)
         END DO

         RHS_RRS = 0.0D0
         SF_RAMAN = 0.0D0

C     Central wavelengths
C     *******************
         DO I_CEN = 1, N_WL_C  
            JPOS = (I_ACT - 1)*(N_WL_C + 1) + I_CEN
            FORALL(I = 1:NLAYERS) S_TMP(I) = WL_FACTOR*SIGMA(I_CEN,I)*
     $           GAMMA_VAL(GAS_NUM(I_CEN))/WL(I_ACT)**4.0D0
            CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, S_TMP, .FALSE., 
     $           SC_RRS(:,:), AKIMA_FLG)
            CALL SPHER_RAD (*880, JPOS, WL_ALL(JPOS), 1, .FALSE., 
     $           .TRUE., DUM, SC_RRS, RHS_RRS, SF_RAMAN)
         END DO              
C        Actual wavelength without Raman
C        *******************************
         FORALL(I = 1:NLAYERS) S_TMP(I) = - WL_FACTOR*SUM(SIGMA(:,I)*
     $           GAMMA_VAL(GAS_NUM(:))/WL_LOSS(1:N_WL_C)**4.0D0)
         CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, S_TMP, .FALSE.,
     $        SC_RRS(:,:), AKIMA_FLG)
         JPOS = JPOS + 1
         CALL SPHER_RAD (*880, JPOS, WL(I_ACT), 1, .FALSE., .TRUE., DUM,
     $        SC_RRS, RHS_RRS, SF_RAMAN, DEL_RRS(I_ACT,:))
C        Actual wavelength with Raman
C        ****************************
         CALL SPHER_RAD(*880, JPOS, WL(I_ACT), 1, .FALSE., .FALSE., DUM,
     $        SC_RRS, RHS_RRS, SF_RAMAN, SPEC_OUT(I_ACT,1,:), 
     $        RAD_DIR_FL(:,I_ACT))
      END DO
c!$OMP END PARALLEL
      RETURN

 880  WRITE(ERRORUNIT, *) ' ---> Called by '//TRIM(ROUTINE_NAME)
      RETURN 1
      END

