!********************************************************************
!
!       SCIATRAN - Version 4.6 - Last modification: 16 Feb 2022
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!*********************************************************************
SUBROUTINE RLC_TO_BRF(NSP, WL_INT)
      
  USE BRDF,       ONLY: BRDF_PARAM, WL_BRF, Nwl_BRF, Nwl_veg, &
                        WL_veg, Ker_cof => IVG_cof
  USE BRDF_SUBRS, ONLY: BRDF_ACT_DOM
  USE DATA_RET,   ONLY: MDS
  USE I_DIMS,     ONLY: Nvs
  USE LOG_PARS,   ONLY: DIR_OUT, C_UNIT, LUNIT
  USE SPHER_PARS, ONLY: NSZA => N_SOL_ANG, N_AZ_ANG, NLSA => N_LOS_ANG,  &
                        NLO => N_LOS_OUT, ANG_OUT 
  USE ST_VARS,    ONLY: SURF_TYPE, Pie
  USE TYPE_VARS,  ONLY: KND
!     ------------------------------------------------------------------
  INTEGER    NSP
  REAL (KND) WL_INT(:)
  OPTIONAL :: NSP, WL_INT
! ... Internal parameters
  CHARACTER (LEN=20) NAM_OUT
  INTEGER     J, Jw, K, Ws, Ws1, IN1, N4, NSPEC
  REAL (KND)  DC, Uc
  REAL (KND)  Phi(N_AZ_ANG), MuV(NLSA), SZA(NSZA)
  REAL (KND), ALLOCATABLE:: R1_OUT(:,:,:,:), S_OUT(:,:), WL1(:)
!     ==================================================================

  IF( PRESENT(WL_INT) ) THEN
     NSPEC = NSP
     ALLOCATE( WL1(NSPEC) ) ; WL1(:) = WL_INT(:)
     NAM_OUT = 'BRF_SURF-APR.dat'
  ELSE
     NSPEC = SIZE(MDS(1)%WL)
     ALLOCATE( WL1(NSPEC) ) ; WL1(:) = MDS(1)%WL(:)
     NAM_OUT = 'BRF_SURF.dat'
  ENDIF
  N4 = 1 
  SZA(:) = ANG_OUT(1,1:NSZA)
  Muv(:) = ANG_OUT(2,1:NLSA)
  Phi(:) = ANG_OUT(3,1:N_AZ_ANG)

  SELECT CASE( TRIM(SURF_TYPE) )
  CASE('Ross-Li')                ;  N4 = N4 + 1
  CASE('Ross-Li-snow','snow_fasmar') ;  N4 = N4 + 2
  END SELECT

  ALLOCATE( R1_OUT(Nvs*NLSA,Nvs*NSZA,N_AZ_ANG,N4), S_OUT(NSZA,NSPEC) )
!  write(*,*) '  ------ Kernels coeff in rlc_to_brf.f90  -----'
!  DO J = 1, SIZE( Ker_cof(:,1) ) 
!     write(*,'(2x,I3,*(1x,ES12.4))')  J, Ker_cof(J,:)
!  ENDDO
!  write(*,*) '    --------------------------------  '
  DO Jw = 1, NSPEC
     CALL BRDF_ACT_DOM( BRDF_PARAM, NLSA, MuV, N_AZ_ANG, Phi, &
          NSZA, SZA, 1, WL1(Jw), R1_OUT, 'none' )
     CALL LIN_SEARCH_NEW(WL1(Jw), DC, Ws, Ws1, Nwl_veg, WL_veg)
     Uc = (1.d0 - Dc)
     S_OUT(:,Jw) = Uc*Ker_cof(1,Ws) + Dc*Ker_cof(1,Ws1)
!    ... Calculations are performed for kernels K_vol and K_geo
     DO J = 1,Nwl_BRF 
        DO K = 1, NSZA
           S_OUT(K,Jw) =  S_OUT(K,Jw) + &
                (Uc*Ker_cof(J+1,Ws) + Dc*Ker_cof(J+1,Ws1)) &
                *R1_OUT(K,K,K,J)
        ENDDO  ! over viewing directions (K) < Mode "one" is assumed >
     ENDDO  ! over kernels (J)
  ENDDO  ! over wavelengths (Jw)
  OPEN(C_UNIT,FILE = DIR_OUT//TRIM(NAM_OUT))
  WRITE(C_UNIT,'(A)') &
       '#   Surface BRF calculated according to coefficients of kernel model'
  DO Jw = 1, NSPEC
     WRITE(C_UNIT,'(2x,F8.3,*(2x,E15.7))') WL1(Jw), &
          (S_OUT(K,Jw),K = 1,NSZA)
  ENDDO
  CLOSE(C_UNIT)
  WRITE(LUNIT,*) '  Retrieved surface BRDF is written out: ', &
       DIR_OUT//TRIM(NAM_OUT)
  
END SUBROUTINE RLC_TO_BRF
