C********************************************************************C
C
C       SCIATRAN - Version 4.7 - Last modification: 09 Aug 2023
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE RTM_INIT(FIRST_RUN, NEW_ALT_GRID, NEW_NODES, NEW_PT, 
     $     WL_START, WL_END, NSPEC, NSPEC_OUT, WL, SRR, *)
c     ... Called by "rtm_main.f"

      USE AEROSOLS,    ONLY: AER_RET_FACTOR, ECSTRA_FLAG, EXA_OUT,
     $                       EXA_OUT_WL, EXA_FLG, FACTOR_EXA_S, EXA_RET, 
     $                       EXA_RET_WL, SCR_WL, Plus_mi, EXA_NAME, 
     $                       N_EXA_WL, EXA_WL, EXA_AC, AER_PARAM
      USE ALT_INPUT,   ONLY: ALIN_ACT
      USE BRDF,        ONLY: MH_BRF, Exact_BRF, Npg, L_SOL
      USE CIA_VARS,    ONLY: CIA_INFO
      USE CLD_RET,     ONLY: NEW_CLD
      USE CLOUDS,      ONLY: DO_CLOUD
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE CLOUD_INPUT, ONLY: PLUS_ICE, ADD_ABS
      USE ESFT,        ONLY: DO_ESFT
      USE GRIDS,       ONLY: NLAYERS, Z_INV, ALT_EXT, X_INV, SZA_EXT,
     $                       NSOL, NL_FINE, ZZZ, SOL_GRD
      USE HYDROSOLS,   ONLY: HSOL_MOD
      USE INFO,        ONLY: RET_INFO, WF_DB, PAR_INFO
      USE INTERN_DIMS, ONLY: Nvs, NL_ice, Nm_ice, NL_soot, Nm_vs, NNv,
     $                       Nh_brf, Nlg_ray, Ng1_brf, Ng2_brf, 
     $                       N_sol_ls, Nlg_ALL_A
      USE IRRAD,       ONLY: WL_IRRAD_DIM
      USE LBL,         ONLY: DO_LBL 
      USE LOG_PARS,    ONLY: EU, DIR_OUT
      USE NODES,       ONLY: N_NODE, MJU_NODE, SIN_NODE, PHI_NODE,
     $                       ModeNodes
      USE PROFILES,    ONLY: PRESS, TEMP 
      USE RAMAN,       ONLY: Ram_Egg, Ram_EgL, Ram_Egs, Ram_EsL, 
     $                       RRS_ADJ, Phot_to_Wat 
      USE RETR_ITER,   ONLY: ITERATION
      USE RTM_OUTPUT,  ONLY: VERT_COL, 
     $                       WF_TO_AMF, NU_VEC_PARS, OWF_OUT_VEC
      USE RTM_PARS,    ONLY: DATA_ITER
      USE SPHER_PARS,  ONLY: N_LOS_OUT, N_SOL_ANG, NLSA => N_LOS_ANG,
     $                       NN, N_ITER
      USE ST_INPUT,    ONLY: AER_PAR_TYPE
      USE ST_VARS,     ONLY: SS_MS_BOUNDARY, DO_AEROSOL, DO_REFRAC, 
     $                       RTM_TYPE, DO_PLANCK, RTM_CORE, NSTRMS, 
     $                       RTM_MODE, WF_FLAG, N_RHS, DO_PC, OWF_FLAG,
     $                       N_Eff, M_LIM, N_Adj, Vec_Sca, RET_MODE,
     $                       DO_RAMAN, NS_wf, TP_HEIGHT, NSEGS, FLG_MS,
     $                       GeoD_ITR, GeoD_OUT, GeoD_PRT, GeoD_PS,
     $                       BRDF_FLAG, SPECULAR, SDVI, PIE, RAD_FIELD,
     $                       SSC_RF, SCATT_MODE
      USE TOOLS,       ONLY: INQ_WF, SET_PTR, CHK_STAT, OpenOut, 
     $                       WriteOut, CloseOut
      USE TYPES,       ONLY: KND, RRS_TYPE, RRS_TYPE, PH_MET
      USE VRS,         ONLY: VRS_Egg, VRS_EgL, VRS_Egs, VRS_EsL, Rho
      USE WATER,       ONLY: UWT, UIT, NLW => NLAYERS_UW, INEL_ADJ,
     $                       INCL_INEL, NSTR_W
      USE WATER_ICE,   ONLY: IWS, IWT
      USE WF_NUM,      ONLY: PERTURB, NUM_WF, NUM_OWF
      USE X_SECTS,     ONLY: RAY_EXT
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN)    :: NSPEC, NSPEC_OUT
      LOGICAL,    INTENT(IN)    :: FIRST_RUN, NEW_ALT_GRID, NEW_PT
      LOGICAL,    INTENT(INOUT) :: NEW_NODES
      REAL (KND), INTENT(IN)    :: WL_START, WL_END
      REAL (KND)  WL(NSPEC)
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
c     ... Local variables
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RTM_INIT'
      INTEGER     I, J, MAX_SPEC, Jlow, Jhigh, Nga, I_RI
      LOGICAL     INIT_GRIDS, INIT_BIA, FLG_LOC
      REAL (KND), ALLOCATABLE :: Z_TMP(:), F_TMP(:), XSOL(:),
     $     RamV_MOMS(:,:), DAG(:)
      TYPE (PH_MET) :: Ph_Ram
      TYPE (DATA_ITER), POINTER :: RTM_PTR
c     ==================================================================
      Interface
         SUBROUTINE BIAKIMA_INIT (Z_INV, ALT_EXT, *, X_INV, SZA_EXT)
            USE GRIDS,      ONLY: NL => NLAYERS, NS => NSOL
            USE TYPES,      ONLY: KND
            REAL (KND) Z_INV(NL), X_INV(NS)
            REAL (KND) ALT_EXT(NL + 4), SZA_EXT(NS + 4)
            Optional :: X_INV, SZA_EXT
         End SUBROUTINE BIAKIMA_INIT
      End Interface
      
      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here

      INIT_GRIDS = .TRUE.  ;   INIT_BIA = .FALSE.

      SELECT CASE(TRIM(RTM_TYPE))
      CASE('pp_scat')
!         INIT_GRIDS = .FALSE.
      CASE('spher_scat', 'transmission')
         INIT_BIA = .TRUE.
      CASE('ps_scat')
         IF(TRIM(RTM_CORE) == 'CDI') INIT_BIA = .TRUE.
      END SELECT
 
      IF(TRIM(RTM_MODE) == 'vod'.AND..NOT.DO_PC) INIT_GRIDS = .FALSE.

      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_NODES) THEN
         IF(TRIM(RTM_CORE) == 'CDI'.OR.RAD_FIELD ) THEN
            Select Case (ModeNodes)
            Case ('eqd')
               Call Nodes_Eqd
            Case Default
               CALL GT_NODE
            End Select
!     CALL INIT_NODES
         ENDIF
      END IF
      
!     First we need to (re-)initialize grids for the altitude and SZA
      FLG_LOC = FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_PT !.OR.NEW_CLD

      Form_Grids: Block
         IF( .NOT.FLG_LOC ) EXIT Form_Grids  ! ====>
!        Additional grids for spherical mode
!        -----------------------------------
         IF(INIT_GRIDS) THEN
!           ... Output: Z_INV, X_INV, NL_FINE, MAX_LINE, Z_FINE, INTERP_IDX
            CALL ST_SPHER_GRIDS (*880)
         ELSE
            NN = 1
            NL_FINE = NLAYERS 
         ENDIF
c        Pressure, Temperature and Gas Concentration Setup
c        Output: PRESS, TEMP, AIR_DENSITY, TG_INFO%ND
c        -------------------------------------------------
         CALL GT_PTCONC_SETUP (*880)

         IF( ITERATION==1 ) THEN
            IF(CHK_STAT('press',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
               CALL SET_PTR( RTM_PTR, 'press' )
               RTM_PTR%APR(:) = PRESS(:)
               RTM_PTR%CRN(:) = PRESS(:)
               RTM_PTR%NXT(:) = RTM_PTR%CRN(:) 
            ENDIF
            IF(CHK_STAT('temp',PAR_INFO%NAME,PAR_INFO%WF_FLG) ) THEN
               CALL SET_PTR( RTM_PTR, 'temp' )
               RTM_PTR%APR(:) = TEMP(:)
               RTM_PTR%CRN(:) = TEMP(:)
               RTM_PTR%NXT(:) = RTM_PTR%CRN(:) 
            ENDIF            
         ENDIF  ! ITERATION==1 

         IF(INIT_GRIDS.AND.DO_REFRAC) CALL REFR_FINE

         GeoD_Init: Block
            CALL LG_Init ('OUT', .FALSE., GeoD_OUT, *880)
!            write(*,*) '  After Lg_init "OUT" in rtm_init ==> GeoD_OUT'

            If(INQ_WF('th', 'RIN')) Then
               Select Case (RTM_CORE)
               Case ('CDI', 'TRN')
                  CALL LG_Init ('OUT', .TRUE., GeoD_PRT, *880)
               End Select
            End If
               
            Select Case (RTM_TYPE)
            Case ('transmission')
               Exit GeoD_Init
            Case ('pp_scat')
               CALL SOL_GRID (*880)
               Exit GeoD_Init
            Case Default
               CALL SOL_GRID (*880)
            End Select 
            
            CALL LG_Init ('PS', .FALSE., GeoD_PS, *880)
!            write(*,*) '   After Lg_init "PS" in rtm_init ==> GeoD_PS'

!            write(*,*) '         FLG_MS  ' ,FLG_MS
            IF(FLG_MS) THEN
c              ... Multiple scattering has to be accounted for
               IF( RTM_CORE == 'DOM'.AND..NOT.RAD_FIELD) Exit GeoD_Init
               IF(N_ITER < 2) Exit GeoD_Init
c              ... "Spherical" iteration
               CALL LG_Init ('ITR', .FALSE., GeoD_ITR, *880)
            END IF
            
         END Block GeoD_Init
!     ... Output: SZA_EXT, ALT_EXT
         IF(INIT_GRIDS.AND.INIT_BIA) THEN
            Select Case (RTM_TYPE)
            Case ('transmission')
               CALL BIAKIMA_INIT(Z_INV, ALT_EXT, *880)
            Case Default
               IF(ALLOCATED(SZA_EXT)) DEALLOCATE (SZA_EXT)
               ALLOCATE(SZA_EXT(NSOL + 4)) 
               CALL BIAKIMA_INIT(Z_INV, ALT_EXT, *880, X_INV, SZA_EXT)
            End Select
         END IF
    
         IF(RTM_CORE /= 'DOM') EXIT Form_Grids  ! =====>
         IF(RTM_TYPE == 'spher_scat') THEN
            IF( WF_FLAG.OR.RRS_ADJ ) SDVI = NSOL
         END IF
         ALLOCATE(XSOL, SOURCE = SOL_GRD) ;  XSOL(:) = -XSOL(:) 
         ALLOCATE( L_SOL(NSOL) )  ;  L_SOL(:) = .FALSE.
c        ... Solar zenith angles w.r.t Z-axis
c            Mark solar angles below horizon
         DO I = 1,NSOL
            IF( ACOS(-XSOL(I))/Pie*180.d0 >= 89.5d0 ) L_SOL(I)=.TRUE.
         ENDDO
         
         CALL GT_HI_WI(XSOL, *880)

c        ... Rotational Raman in the atmosphere
         IF( DO_RAMAN ) CALL SET_RRS
c        ... Vibrational Raman in the ocean
         IF( INCL_INEL(1) ) CALL SET_VRS

         IF(UWT .AND.(.NOT.SPECULAR)) 
     $        CALL TRANS_REFL_MATR_V(NSPEC, XSOL)
c     $        CALL TRANS_REFL_MATR(NSPEC, XSOL)
      END Block Form_Grids
      
cc     ==========================================
cc     ... Bidirectional reflection function setup
cc     ==========================================
c      Npg = 1
c      IF( BRDF_FLAG ) THEN
cc         ... Number of additional (w.r.t. Gaussian) angles, which
cc             will be used as reflection angles in BRF
c         IF( Exact_BRF )  Npg = NLSA         
cc        ... Signs Mu and Mu0 correspond to the z-axis
c         CALL GENERAL_BRDF_SETUP
c     $        (NSPEC, WL, WL_START, WL_END, XSOL, *880)         
cc        ... Internal dimensions for the surface BRDF arrays
cc             used in "dom_spherical" and routines below
c         Ng1_brf = Nvs*(NSTRMS+Npg) ; Ng2_brf = Nvs*NSTRMS
c         Nh_brf  = Mh_BRF
c      ENDIF
         
      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_CLD) THEN
C     ============================================
C     Initialization of altitude-dependend arrays
C     ============================================
         IF( ALIN_ACT(4) ) CALL ADJUST_PROFILES(*880)

         IF( NUM_WF ) THEN
            IF( NUM_OWF ) THEN
               ALLOCATE( PERTURB(NLW) )
            ELSE
               ALLOCATE( PERTURB(NLAYERS) )
            ENDIF
         ELSE
            ALLOCATE( PERTURB(1) )
         ENDIF  ! NUM_WF
         PERTURB(:) = 1.d0
         
c        Setup of cloud layer optical parameters
c        Output: PROF_VARS
c        ---------------------------------------
         NL_ice = 1 ; Nm_ice = 1 ; NL_soot = 1
         IF( DO_CLOUD ) THEN 
            IF( ANY(ADD_ABS) ) NL_soot = NLAYERS
            IF( ANY(Plus_ice) ) THEN 
               NL_ice = NLAYERS ; Nm_ice = 2
            ENDIF
            CALL GT_CLOUD_PARAMETER_SETUP (*880)
c            IF( ANY(Plus_ice) ) THEN 
c               NL_ice = NLAYERS ; Nm_ice = 2
c            ENDIF
         ENDIF
         
c        Setup aerosol scenario
c        Output: APH_VS, AER_WAVS, AER_COEFF, FDM_AER
c        ---------------------------------------------------
         IF( DO_AEROSOL ) THEN         
            ALLOCATE( AER_RET_FACTOR(NSEGS,NLAYERS), Z_TMP(200), 
     $           F_TMP(200))
            AER_RET_FACTOR = 1.0D0
!            FORALL(J = 1:NLAYERS, ZZZ(J) > TP_HEIGHT) 
!     $           AER_RET_FACTOR(:,J) = FACTOR_EXA_S
!            FORALL(J = 1:NLAYERS)
!     $           AER_RET_FACTOR(:,J) = FACTOR_EXA_S
            Jlow = COUNT((ZZZ(:) > TP_HEIGHT))
            Jhigh = COUNT((ZZZ(:) >= 40.0D0))
            Jhigh = MAX(Jhigh,1)
            Jlow  = MAX(Jlow,1)
            AER_RET_FACTOR(:,Jhigh:Jlow) = FACTOR_EXA_S   
!            FORALL(J = 1:NLAYERS, ZZZ(J) < 40.0D0)
!     $           AER_RET_FACTOR(:,J) = FACTOR_EXA_S
            DO I_RI = 1, UBOUND(RET_INFO,1)
               IF (RET_INFO(I_RI)%NAME == 'aer-ext') THEN
                  CALL SET_PTR( RTM_PTR, RET_INFO(I_RI)%NAME )
                  RTM_PTR%CRN(:) = 1.0D0
                  RTM_PTR%APR(:) = 1.0D0 
                  AER_RET_FACTOR(:,:) = 1.0D0     
               ENDIF
            ENDDO
            SELECT CASE (TRIM(RET_MODE))
            CASE('aerosol', 'aer_alb', 'aer_alb_refth') 
               ALLOCATE(SCR_WL(NSPEC,NLAYERS), !A
     $              EXA_OUT(NSPEC,NLAYERS), EXA_OUT_WL(NSPEC), !A
     $              EXA_RET(3,NLAYERS,NSEGS), EXA_RET_WL(NSEGS)) !A 
            END SELECT
            SELECT CASE (AER_PAR_TYPE(1:2))
            CASE('ae')
               CALL AERONET_SETUP(WL_START, WL_END, *880)
            CASE ('pa')
               CALL PAO_AER_SETUP(WL_START, WL_END, *880)
            CASE ('mn')
               CALL MAN_AER_SETUP(WL_START, WL_END, *880)
            CASE ('wm')
               CALL WMO_AER_SETUP(WL_START, WL_END, *880)
            CASE('lt')
               CALL LT_AER_SETUP(WL_START, WL_END, *880)
            CASE('st')
               CALL SC_AER_SETUP(WL_START, WL_END, *880)
            CASE('me')
               CALL MERRA_AER_SETUP(WL_START, WL_END, *880)
            CASE('xb')
               CALL MERRA_XB_SETUP(WL_START, WL_END, *880)
            CASE('mi','mo')
               IF(FIRST_RUN.OR.NEW_ALT_GRID)
     $              CALL MIE_AER_SETUP(WL_START, WL_END, *880)
            END SELECT
            IF( Plus_mi ) 
     $           CALL MIE_AER_SETUP(WL_START, WL_END, *880)
            IF(EXA_FLG) THEN
               CALL REPLACE_EXA(*880)
            ELSE
               IF(ECSTRA_FLAG) CALL ECSTRA_INIT (WL_START, WL_END, *880)
            END IF
         ELSE
            Nlg_ALL_A = 1
         ENDIF  ! DO_AEROSOL
      ENDIF 

      Select Case (RTM_CORE)
      Case("CDI","TRN")
      Case Default
         IF( DO_SSRCA.OR.SSC_RF.OR.SCATT_MODE=='ss' )
     $        CALL GT_HI_WI_SS(SOL_GRD, *880)
      End Select
      
      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_CLD) THEN
c     Setup hydrosol scenario in the case of underwater transport 
c        -----------------------------------------------------------
         IF( UWT ) THEN
            HIP: Block
              IF( UIT ) THEN
                 CALL MAN_ICE_SETUP(WL_START, WL_END, *880)
                 IF( .NOT.IWT ) EXIT HIP
              END IF
              IF( ALLOCATED(HSOL_MOD) ) THEN
                 DO I = 1, SIZE(HSOL_MOD)
c                   ...    !! Attention !! 
c                   CASE 'User_supplied' has to be always before 
c                   case 'Kopelevich' to avoid allocation problems 
                    SELECT CASE( HSOL_MOD(I)%S ) 
                    CASE('User_supplied')
                       CALL MAN_HSOL_SETUP(WL_START, WL_END, *880)
                    CASE('Kopelevich')
                       CALL KOP_HSOL_SETUP(*880)
                    END SELECT
                 ENDDO  ! overe hydrosol components (I)
              ENDIF
            END Block HIP
         ENDIF  ! UWT
         
         IF( UWT ) CALL WRITE_WATER_OUT (*880)
c         stop
         IF( DO_RAMAN ) CALL ST_RRS_SETUP (*880)

c        Phtotochemistry initialization
c        Output: N_SOL_PC, CONC_PC, XS_PC
c        ------------------------------
         IF( DO_PC )  CALL GT_PHOTOCHEM (*880)
         
         IF( RAY_EXT ) CALL RAYLEIGH_EXT(*880)
         
         CALL RAYLEIGH_OUT(*880)
         
         IF(RTM_MODE /= 'vod') THEN
            
c     Internal dimension for thermal emission: NL_FINE is used!
c     ---------------------------------------------------------
            IF( DO_PLANCK ) CALL THERMAL_INIT(*880)
            
c     Initialization of 'DOM' mode
c     ----------------------------
            IF( TRIM(RTM_CORE) == 'DOM' ) THEN
               
C     Renormalise aerosol and cloud phase function moments
C     (note: Rayleigh moments will now be calculated in wavelength loop)
C     =================================================================
               IF( Vec_Sca == 'scal_rte') THEN 
                  IF(DO_AEROSOL .OR. DO_CLOUD) CALL GT_RENORM(NSTRMS)
               ENDIF
               
            ENDIF               ! RTM_CORE = 'DOM' 
         ENDIF                  ! RTM_MODE /= 'vod' 
         

         MAX_SPEC = MAX0(NSPEC, NSPEC_OUT)
         IF( FIRST_RUN ) THEN
            CALL RTM_OUTPUT_ARRAYS (NSPEC, MAX_SPEC, NLAYERS, *880)
         ELSE
            IF( OWF_FLAG .OR. NUM_OWF) THEN
               DEALLOCATE (OWF_OUT_VEC)
               ALLOCATE (OWF_OUT_VEC(MAX_SPEC, N_LOS_OUT, NLW, !A
     $              NU_VEC_PARS, Nvs)) !A
               OWF_OUT_VEC = 0.d0
            ENDIF             

            SELECT CASE (TRIM(RTM_MODE))
            CASE ('amf','slant_col')
               ALLOCATE(VERT_COL(1:N_SOL_ANG)) ! A
               VERT_COL(1:N_SOL_ANG) = 0.0D0
            CASE ('block_amf')
               ALLOCATE(WF_TO_AMF(NSPEC,NLAYERS)) ! A
            END SELECT
         END IF
         
      END IF                    ! FIRST_RUN.OR.NEW_ALT_GRID

      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_PT) THEN
c        ESFT setup
c        Output: ESFT_OVERLAP_MODE, ESFT_ALPHA, ESFT_COEFF 
c                ESFT_MOL_OVERLAP_IDX1, ESFT_MOL_OVERLAP_IDX2, 
c                DO_ESFT_SPEC_MOL, ESFT_NCOEFF, ESFT_WEIGHTS
c                D_ESFT_COEFF_DT, D_ESFT_COEFF_DP
c        -----------------------------------------------------------
            
         IF( TRIM(RTM_CORE) == 'DOM') THEN
c     Set internal dimensions for the scalar or vector case
c     =====================================================
            N_Eff = 1
            SELECT CASE (TRIM(RTM_TYPE))
            CASE('spher_scat')
               CALL GEOM_ALONG_LOS (*880)
               N_RHS = NSOL
               IF( WF_FLAG.OR.RRS_ADJ ) THEN
c                 ... N_Eff defines maximal number of subintervals for 
c                     all line-of-sights
                  N_Eff = MAXVAL( M_LIM(:) )
                  N_RHS = N_RHS + N_Adj
               ENDIF
            CASE DEFAULT
c              ... NL_FINE is defined in the routine ST_SPHER_GRIDS" 
c                          if RTM_TYPE == 'ps_scat' or 'spher_scat'
c                  NL_FINE is set to 1 in "gt_fileread" 
c                          if RTM_TYPE == 'pp_scat'
               N_Adj = NS_wf*NLSA 
               M_LIM(:) = 1
c               WRITE(*,*) ' NS_wf =   ', NS_wf
c               WRITE(*,*) ' N_adj =   ', N_adj
c               Pause
            END SELECT
         END IF
                     
         IF( DO_ESFT ) CALL GT_ESFT_PARAMETER_SETUP(WL, NSPEC, *880)

C        ... Initialization of the partition functions (line-by-line mode only)
         IF( DO_LBL ) CALL ST_PART_SUMS (*880)

C        IF(RING_FLAG) THEN
C        Output arrays
c        MAX_SPEC = MAX0(NSPEC,NSPEC_OUT)
c        CALL RTM_OUTPUT_ARRAYS (NSPEC, MAX_SPEC, NLAYERS, *880)
         
c        CALL RING_INTERFACE (NSPEC, WL, SPEC_OUT, DEL_RRS, 
c     $        RAD_DIR_FL, PARALLEL_WL, *880) 
C        ELSE
c         N_WL_C = 0
         CALL ABS_COEFF_READ (NSPEC, WL, FIRST_RUN, *880)
C        END IF

      END IF ! FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_PT

      IF(ANY(CIA_INFO%Flg)) CALL CIA_DENSITIES
      
      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_NODES) THEN  
         IF(NEW_NODES) NEW_NODES = .FALSE.
         CALL SET_ARRAY_DIM
      END IF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN         
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
C********************************************************************C
      SUBROUTINE SET_RRS
c     ... Depending on the dimension of the solar spectra
      Phot_to_Wat = .FALSE.
      IF( TRIM(WL_IRRAD_DIM(2))=='phot/s' ) Phot_to_Wat = .TRUE.            
      Nga = NNv
      ALLOCATE( Ram_Egg(Nga,Nga,2,3), Ram_EgL(Nga,Nvs*NLSA,2,3),   ! A
     $     Ram_Egs(Nga,Nvs*NSOL,2,3), SRR%Ram_MOMS(Nlg_Ray,Nm_vs), ! A
     $     Ram_EsL(Nvs*NLSA,Nvs*NSOL,N_sol_ls,3),                  ! A
     $     Ph_Ram%Egg(Nga,Nga,2), Ph_Ram%EgL(Nga,Nvs*NLSA,2),      ! A
     $     Ph_Ram%Egs(Nga,Nvs*NSOL,2),                             ! A
     $     Ph_Ram%EsL(Nvs*NLSA,Nvs*NSOL,N_sol_ls) )                ! A
      Ph_Ram%Egg = 0.d0 ; Ph_Ram%EgL = 0.d0
      Ph_Ram%Egs = 0.d0 ; Ph_Ram%EsL = 0.d0
      SELECT CASE (Vec_Sca)
      CASE('scal_rte')
c     ... Rotational Raman phase function moments 
c         in scalar case (see Eq.(Ram-rot-moms) in STB)
         SRR%Ram_MOMS(1,1) = 1.d0 ; SRR%Ram_MOMS(2,1) = 0.d0 
         SRR%Ram_MOMS(3,1) = 1.d0/20.d0
         DO I = 1,3
            CALL PHASE_FUNCTIONS(I, NSTRMS, NSOL, NLSA,
     $           SRR%Ram_MOMS, Nlg_Ray, Ph_Ram, 'A')
            Ram_Egg(:,:,:,I) = Ph_Ram%Egg(:,:,:)
            Ram_EgL(:,:,:,I) = Ph_Ram%EgL(:,:,:)  
            Ram_Egs(:,:,:,I) = Ph_Ram%Egs(:,:,:)  
            Ram_EsL(:,:,:,I) = Ph_Ram%EsL(:,:,:)  
         ENDDO  ! over three harmonics (I)
      CASE('vect_rte')
c     ... Rotational Raman phase function moments 
c         in vector case (see Eq.(Greek-RRS) in STB)
         SRR%Ram_MOMS = 0.d0    !
c        ... first harmonic:         
         SRR%Ram_MOMS(1,1) = 1.d0 ! alpha1
c        ... second harmonic:
         SRR%Ram_MOMS(2,4) = -3.d0/4.d0 ! alpha4              
c        ...  third harmonic:
         SRR%Ram_MOMS(3,1) = 1.d0/20.d0 ! alpha1
         SRR%Ram_MOMS(3,2) = 3.d0/10.d0 ! alpha2
c        ... The sign '-' will be used for the expansion
c            coefficient beta1 because within the SCIATRAN 
c            code the Wigner functions are adopted instead 
c            of generalized spherical functions.
         SRR%Ram_MOMS(3,5) = -DSQRT(3.d0/2.d0)/10.d0 ! beta1
         DO I = 1,3
            CALL PHASE_MATR (I, NSTRMS, NSOL, NLSA, 
     $           Nlg_Ray, SRR%Ram_MOMS, Ph_Ram, 'A')
c        ... Second dimension of array Ph_Ram%Egg(:,|,1:2) 
c            contains 1/XM(:) (see "phase_matr.f"). Moreover,
c            this array is already in the form of the sum 
c            and difference of phase matrixes.
            Ram_Egg(:,:,:,I) = Ph_Ram%Egg(:,:,:)
            Ram_EgL(:,:,:,I) = Ph_Ram%EgL(:,:,:)  
            Ram_Egs(:,:,:,I) = Ph_Ram%Egs(:,:,:)  
            Ram_EsL(:,:,:,I) = Ph_Ram%EsL(:,:,:) 
         ENDDO                  ! over three harmonics (I)
      END SELECT
      DEALLOCATE(Ph_Ram%Egg, Ph_Ram%EgL, Ph_Ram%Egs, Ph_Ram%EsL)
      END SUBROUTINE SET_RRS

C********************************************************************C
      SUBROUTINE SET_VRS  
      Nga = NSTR_W*Nvs
      ALLOCATE( VRS_Egg(Nga,Nga,2,3), RamV_MOMS(Nlg_Ray,Nm_vs), ! A
     $     Ph_Ram%Egg(Nga,Nga,2), Ph_Ram%EgL(Nga,Nvs*NLSA,2), ! A
     $     Ph_Ram%Egs(Nga,Nvs*NSOL,2), VRS_Egs(Nga,Nvs*NSOL,2,3), ! A
     $     Ph_Ram%EsL(Nvs*NLSA,Nvs*NSOL,N_sol_ls) ) ! A
c     ... Solution obtained using forward-adjoint approach
      IF( INEL_ADJ ) THEN
         ALLOCATE( VRS_EgL(Nga,Nvs*NLSA,2,3), ! A
     $        VRS_EsL(Nvs*NLSA,Nvs*NSOL,N_sol_ls,3) ) ! A
      ENDIF 
      Ph_Ram%Egg = 0.d0 ; Ph_Ram%EgL = 0.d0
      Ph_Ram%Egs = 0.d0 ; Ph_Ram%EsL = 0.d0
c     ... Depolarization ratio of water is an input parameter
c         Rho = 0.17d0
c     ... Vibrational Raman phase function moments 
c         (see Eq.(Ram-vib-moms) in STB)
      RamV_MOMS(1,1) = 1.d0 ; RamV_MOMS(2,1) = 0.d0 
      RamV_MOMS(3,1) = 0.5d0*(1.d0 - Rho)/(1.d0 + 2.d0*Rho)
      DO I = 1,3
         CALL PHASE_FUNCTIONS(I, NSTR_W, NSOL, NLSA,
     $        RamV_MOMS, Nlg_Ray, Ph_Ram, 'O')
         VRS_Egg(:,:,:,I) = Ph_Ram%Egg(:,:,:)
         VRS_Egs(:,:,:,I) = Ph_Ram%Egs(:,:,:)  
         IF( INEL_ADJ ) THEN
            VRS_EgL(:,:,:,I) = Ph_Ram%EgL(:,:,:)  
            VRS_EsL(:,:,:,I) = Ph_Ram%EsL(:,:,:)  
         ENDIF
      ENDDO
      DEALLOCATE(Ph_Ram%Egg, Ph_Ram%EgL, Ph_Ram%Egs, Ph_Ram%EsL)
      END SUBROUTINE SET_VRS  

C********************************************************************C
      SUBROUTINE ADJUST_PROFILES(*)

      USE ALT_INPUT, ONLY: ALIN_ACT, ALIN_NWL, ALIN_Z, ALIN_NZ,
     $                     ALIN_ABS, ALIN_RAY, ALIN_AER_A, ALIN_AER_S, 
     $                     ALIN_CLD_A, ALIN_CLD_S
      USE CLOUDS,    ONLY: N_CLD_LAYERS, J_TOP, J_BOT
      USE GRIDS,     ONLY: NZ => NLAYERS, ZZZ
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------
      CHARACTER(LEN = *), PARAMETER :: R_NAME = 'ADJUST_PROFILES'
      INTEGER     I, J, K
      REAL (KND)  TMP(ALIN_NWL,ALIN_NZ)
c     ------------------------------------------------------------------

      IF( ALIN_ACT(1) ) THEN
         TMP(:,:) = ALIN_ABS(:,:) ; DEALLOCATE( ALIN_ABS )
         ALLOCATE( ALIN_ABS(ALIN_NWL,NZ) )
         DO I = 1, ALIN_NWL
            CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $            NZ, ZZZ,  ALIN_ABS(I,:) ,.FALSE., *880)
         ENDDO
      ENDIF
      IF( ALIN_ACT(2) ) THEN
         TMP(:,:) = ALIN_RAY(:,:) ; DEALLOCATE( ALIN_RAY )
         ALLOCATE( ALIN_RAY(ALIN_NWL,NZ) )
         DO I = 1, ALIN_NWL
            CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $            NZ, ZZZ,  ALIN_RAY(I,:) ,.FALSE., *880)
         ENDDO
      ENDIF
      IF( ALIN_ACT(3) ) THEN
         TMP(:,:) = ALIN_AER_S(:,:) ; DEALLOCATE( ALIN_AER_S )
         ALLOCATE( ALIN_AER_S(ALIN_NWL,NZ) )
         DO I = 1, ALIN_NWL
            CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $            NZ, ZZZ,  ALIN_AER_S(I,:) ,.FALSE., *880)
         ENDDO
         TMP(:,:) = ALIN_AER_A(:,:) ; DEALLOCATE( ALIN_AER_A )
         ALLOCATE( ALIN_AER_A(ALIN_NWL,NZ) )
         DO I = 1, ALIN_NWL
            CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $            NZ, ZZZ,  ALIN_AER_A(I,:) ,.FALSE., *880)
         ENDDO
      ENDIF
      TMP(:,:) = ALIN_CLD_A(:,:) ; DEALLOCATE( ALIN_CLD_A )
      ALLOCATE( ALIN_CLD_A(ALIN_NWL,NZ) )
      DO I = 1, ALIN_NWL
         CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $        NZ, ZZZ,  ALIN_CLD_A(I,:) ,.FALSE., *880)
         DO J = 1, N_CLD_LAYERS
            ALIN_CLD_A(I,J_TOP(J)) = 0.d0
            ALIN_CLD_A(I,J_BOT(J)) = 0.d0
         ENDDO
      ENDDO
      TMP(:,:) = ALIN_CLD_S(:,:) ; DEALLOCATE( ALIN_CLD_S )
      ALLOCATE( ALIN_CLD_S(ALIN_NWL,NZ) )
      DO I = 1, ALIN_NWL
         CALL AL_INTERP (ALIN_NZ, ALIN_Z, TMP(I,:),
     $        NZ, ZZZ,  ALIN_CLD_S(I,:) ,.FALSE., *880)
         DO J = 1, N_CLD_LAYERS
            ALIN_CLD_S(I,J_TOP(J)) = 0.d0
            ALIN_CLD_S(I,J_BOT(J)) = 0.d0
         ENDDO
      ENDDO
c           WRITE(*,*) '    After adjustment  '
c           WRITE(*,*) '  --------- CLD S ------------------- '
c           DO I = 1, NZ
cc              IF( ALIN_CLD_S(1,I) == 0.d0 ) CYCLE
c              WRITE(*,'(2x,F8.3,*(2x,e15.7))') 
c     $             ZZZ(I), ALIN_CLD_S(:,I)
c           ENDDO
c           WRITE(*,*) '  --------- CLD A ------------------- '
c           DO I = 1, NZ
cc              IF( ALIN_CLD_A(1,I) == 0.d0 ) CYCLE
c              WRITE(*,'(2x,F8.3,*(2x,e15.7))') 
c     $             ZZZ(I), ALIN_CLD_A(:,I)
c           ENDDO

      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE ADJUST_PROFILES

C********************************************************************C
      SUBROUTINE SET_ARRAY_DIM
c     ... Call in "rtm_init.f"

      USE GRIDS,       ONLY: NSOL, N_WF_L, NL_FINE, MAX_LINE
      USE INTERN_DIMS, ONLY: NL_MS, N_NODE_MS, NSOL_MS, N_PHI_MS, N_SFN, 
     $                       NL_FINE_MS, MAX_LINE_MS, NL_WF, N_NODE_WF, 
     $                       NSOL_WF, NL_PC, N_WF_L_MS, NL_FINE_WF, 
     $                       N_LOS_WF, NL_FINE_PC, Ns_SFN
      USE NODES,       ONLY: N_NODE
      USE PHOTOCHEM,   ONLY: N_SOL_PC 
      USE ST_VARS,     ONLY: INT_MS_FLAG, WF_MS_FLAG
c     ------------------------------------------------------------------
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_ARRAY_DIM'

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF(DO_PC) THEN
         NL_PC = NLAYERS ;   NL_FINE_PC = NL_FINE
      ELSE
         NL_PC = 1 ;   NL_FINE_PC = 1 ;  N_SOL_PC = 1
      END IF      
      
      IF(TRIM(RTM_MODE) == 'vod') RETURN

      IF(ANY(INT_MS_FLAG(1:NSPEC))) THEN
         NL_MS = NLAYERS ; N_NODE_MS = N_NODE  
         NSOL_MS = NSOL
         NL_FINE_MS = NL_FINE ; MAX_LINE_MS = MAX_LINE
      ELSE
         NL_MS = 1 ;  N_NODE_MS = 1 ;  NSOL_MS = 1
         NL_FINE_MS = 1 ;  MAX_LINE_MS = 1
      END IF
      
c     N_SFN is defines as
      N_SFN = N_NODE*Nvs*N_PHI_MS
      Ns_SFN = N_NODE*N_PHI_MS

      IF( WF_FLAG ) THEN
         N_LOS_WF = N_LOS_OUT ;  N_WF_L = NLAYERS
      ELSE
         N_LOS_WF = 1
         IF( RRS_ADJ ) THEN
            N_WF_L = NLAYERS
         ELSE
            N_WF_L = 1
         ENDIF
      END IF
 

      IF(ANY(WF_MS_FLAG(1:NSPEC))) THEN
         NL_WF = NLAYERS ; N_NODE_WF = N_NODE ;  NSOL_WF = NSOL
         N_WF_L_MS = N_WF_L ; NL_FINE_WF = NL_FINE
      ELSE
         NL_WF = 1 ; N_NODE_WF = 1 ;  NSOL_WF = 1
         N_WF_L_MS = 1 ; NL_FINE_WF = 1
      END IF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_ARRAY_DIM
C********************************************************************C

      SUBROUTINE THERMAL_INIT(*)
c     ... Call in "rtm_init.f"

      USE GRIDS,       ONLY: NL_FINE
      USE INTERN_DIMS, ONLY: NL_pla, NN_pla, NNv, LS_pla, Nf_pla,
     $                       NL_FINE_pla
      USE SPHER_PARS,  ONLY: N_LOS_ANG, NN
      USE THERMAL_VAR, ONLY: Nwl_emis, Emi_FILE, Emi_NAME, Sur_Emis,
     $                       Wl_Emis
      USE TOOLS,       ONLY: READ_FILE
c ------------------------------------------------------------------
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'THERMAL_INIT'
      INTEGER II
      REAL (KND), ALLOCATABLE :: DATA_R(:,:)
c====================================================================
      CALL ST_TRACK (R_NAME, 1, 'I')

      NL_pla = NLAYERS  ;  NN_pla = NNv  ;  LS_pla = N_LOS_ANG 
      SELECT CASE (TRIM(RTM_TYPE))
      CASE('pp_scat','ps_scat')
c          ... No storage of Planck function on the fine-grid
c          ... Number of sub-layers for approximation of Planck function
         NL_FINE_pla = 1        ;  Nf_pla = 10
      CASE('spher_scat')
         NL_FINE_pla = NL_FINE  ;  Nf_pla = NN
      END SELECT
c     ... Reading wavelength-dependent emissivity
      IF( Emi_FILE ) THEN
         WRITE(*,*) '    Start and End wavelengths [micron]'
         WRITE(*,'(2x,*(2x,F8.3))') WL_START*1.d-3, WL_END*1.d-3
         CALL READ_FILE(2, Emi_NAME, DATA_R, *880, WL_START, WL_END,
     $        NPT = 2)
         DO II = 1, Size(DATA_R,1)
            WRITE(*,*) DATA_R(II,1), DATA_R(II,2)
         ENDDO
         Nwl_emis = Size(DATA_R,1)
         ALLOCATE(Wl_emis(Nwl_emis), Sur_emis(Nwl_emis))
         WL_emis(:)  = DATA_R(:,1)  ! wavelength in [nm] 
         Sur_emis(:) = DATA_R(:,2)  ! emissivity
         DEALLOCATE( DATA_R )          
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE THERMAL_INIT
c ********************************************************************

      SUBROUTINE RAYLEIGH_EXT(*)
c     ... Call in "rtm_init.f"

      USE PROFILES,  ONLY: AIR_DENSITY
      USE ST_VARS,   ONLY: King_factor, Nwl_RR, WL_RR
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RAYLEIGH_EXT'
      INTEGER  I, K, Unt
      CHARACTER (LEN=80)  TEXT, TT(4)
      Character (Len=500) OutStr
      REAL (KND) WLL, DEP_RR(Nwl_RR), KI_COMP(Nwl_RR,4),
     $     RI_COMP(Nwl_RR,4), D_COMP(Nwl_RR,4), SEC_COMP(Nwl_RR,4)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      TT(1) = 'N2' ; TT(2) = 'O2' 
      TT(3) = 'Ar'  ; TT(4) = 'CO2' 
      DO I = 1,Nwl_RR
         WLL = WL_RR(I)
         CALL KING_COMP(WLL, KI_COMP(I,:), RI_COMP(I,:),
     $        D_COMP(I,:), SEC_COMP(I,:))
      ENDDO

      Call OpenOut ('EXT_RAYLEIGH.OUT', Unt)
c     ... In collection "st_tools.f"
      CALL WRITE_UD(1,Unt)
      
      WRITE(OutStr,'(11x,46A)') ('=', I = 1,46)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,A)') 
     $     ' Rayleigh scattering by atmospheric molecules'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,46A)') ('=', I = 1,46)
      Call WriteOut (Unt, OutStr)
      Call WriteOut (Unt, '')
      
      Call WriteOut (Unt,'   Rayleigh cross-section : Bates')      
c     ... Refractive index ...........................................
      TEXT = 'Bates-84'
      Call WriteOut (Unt,'   Refractive index       : '//TRIM(TEXT))
c     ................................................................      
      Call WriteOut (Unt,'   King factor            : '//
     $     '(6 + 3*D)/(6 - 7*D), D - Depol. ratio')
      
      WRITE(OutStr,'(2x,60A)')  ('-', I = 1,60)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(2x,A,2x,A,2x,A,2x,A)') 
     $     'Wavelength','Depolarization','Cross-section',
     $     '  Refr.ind - 1   '
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(2x,A,2x,A,2x,A,2x,A)') 
     $     '   [nm]   ','    ratio     ','    [cm^2]   ',
     $     '      [non] '
      Call WriteOut (Unt, OutStr)
      DO K = 1, 4
         WRITE(OutStr,'(2x,A,2x,A)') TRIM( TT(K) ),' --------------- '
         Call WriteOut (Unt, OutStr)
         DO I = 1,Nwl_RR
            WRITE(OutStr,'(3x,F7.2,6x,F7.4,6x,ES12.3,7x,ES12.3)') 
     $           WL_RR(I), D_COMP(I,K), SEC_COMP(I,K),
     $           RI_COMP(I,K) - 1.d0 
            Call WriteOut (Unt, OutStr)
         ENDDO
      ENDDO
      WRITE(OutStr,'(2x,60A)')  ('-', I = 1,60)
      Call WriteOut (Unt, OutStr)
      
      CALL WRITE_UD(2,Unt)
      Call CloseOut (Unt)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE RAYLEIGH_EXT
c********************************************************************

      SUBROUTINE RAYLEIGH_OUT(*)
c     ... Call in "rtm_init.f"

      USE CLOUDS_SS, ONLY: DO_RAY_DEP, RAY_DEP, DO_RAY_TAU, RAY_TAU_INP
      USE GRIDS,     ONLY: DZR, NLAYERS
      USE LOG_PARS,  ONLY: ERROR_TXT
      USE PROFILES,  ONLY: AIR_DENSITY
      USE ST_VARS,   ONLY: RAY_SCATT_IDX, Refr_index, King_factor, 
     $                     Nwl_RR, WL_RR
      USE TYPE_VARS, ONLY: KND
      USE WATER,     ONLY: VSF_pure_model, DO_RTAU_W, RTAU_W, Ref_ind,
     $                     DEP_W_INPUT
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RAYLEIGH_OUT'
      INTEGER  I, Unt
      CHARACTER (LEN=80) TEXT, TXT_R, TT
      Character (Len=500)  OutStr
      REAL (KND) WLL, DEPOL, DEP_RR(Nwl_RR), SCR_L(Nwl_RR,NLAYERS), 
     $     R_sec(Nwl_RR), Tau(Nwl_RR)
      REAL (KND), EXTERNAL :: Ray_sec
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      DO I = 1,Nwl_RR
         WLL = WL_RR(I)
         CALL KING_ACTUAL(WLL, DEPOL, *894)
         DEP_RR(I) = DEPOL
         R_sec(I) = Ray_sec(WLL,DEPOL)
         SCR_L(I,:) =  R_sec(I)*AIR_DENSITY(:)
      ENDDO

      IF( DO_RAY_TAU ) THEN
         TXT_R = '    as input     '
         TAU(:) = RAY_TAU_INP
      ELSE
         TXT_R = '   calculated    '
         IF( DO_RAY_DEP ) THEN 
            TXT_R = '     scaled      '
            DO I = 1,Nwl_RR
               SCR_L(I,:) = SCR_L(I,:)*RAY_DEP
            ENDDO
         ENDIF
         TAU(:) = 0.d0
         DO I = 1,NLAYERS-1
            TAU(:) = TAU(:) + (SCR_L(:,I) + SCR_L(:,I+1))*0.5D0*DZR(I)
         END DO
      ENDIF

      Call OpenOut ('SCE_RAYLEIGH.OUT', Unt)

c     ... In collection "st_tools.f"
      CALL WRITE_UD(1,Unt)
      
      WRITE(OutStr,'(11x,46A)') ('=', I = 1,46)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,A)') 
     $     ' Rayleigh scattering by atmospheric molecules'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,46A)') ('=', I = 1,46)
      Call WriteOut (Unt, OutStr)
      Call WriteOut (Unt, '')
      
      SELECT CASE(RAY_SCATT_IDX)
      CASE(1)
         Call WriteOut (Unt,'   Rayleigh cross-section : Bates')
      CASE(2)
         Call WriteOut (Unt,'   Rayleigh cross-section : Bucholtz')
      CASE(3)
         Call WriteOut (Unt,'   Rayleigh cross-section : Bodhaine')
      CASE(4)
         Call WriteOut (Unt,
     $        '   Rayleigh cross-section : User-defined')
      END SELECT
      
c     ... Refractive index ...........................................
      SELECT CASE(TRIM(Refr_index))
      CASE('Edlen53') 
         TEXT = 'Edlen-53'
      CASE('Edlen66')
         TEXT = 'Edlen-66'
      CASE('Peck_Reeder72')
         TEXT = 'Peck and Reeder 72'
      CASE('Bates84')
         TEXT = 'Bates-84'
      END SELECT
      Call WriteOut (Unt,'   Refractive index       : '//TRIM(TEXT))
c     ................................................................
      
      Call WriteOut (Unt,'   King factor            : '//
     $     TRIM(King_factor))
      
      WRITE(OutStr,'(2x,60A)')  ('-', I = 1,60)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(2x,A,2x,A,2x,A,2x,A)') 
     $     'Wavelength','Depolarization','Cross-section',
     $     'Optical thickness'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(2x,A,2x,A,2x,A,2x,A)') 
     $     '   [nm]   ','   factor     ','    [cm^2]   ',
     $     TRIM(TXT_R)
      Call WriteOut (Unt, OutStr)
      DO I = 1,Nwl_RR
         WRITE(OutStr,'(3x,F7.2,6x,F7.4,6x,ES12.3,7x,F8.5)') 
     $        WL_RR(I), DEP_RR(I), R_sec(I), TAU(I)
         Call WriteOut (Unt, OutStr)
      ENDDO
      WRITE(OutStr,'(2x,60A)')  ('-', I = 1,60)
      Call WriteOut (Unt, OutStr)
      
      IF( .NOT.UWT ) GOTO 111
      
      Call WriteOut (Unt,'')
      WRITE(OutStr,'(11x,35A)')  ('=', I = 1,35)
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,A)') ' Rayleigh scattering by pure water'
      Call WriteOut (Unt, OutStr)
      WRITE(OutStr,'(11x,35A)')  ('=', I = 1,35)
      Call WriteOut (Unt, OutStr)
      Call WriteOut (Unt,'')
      TT = '   Angular scattering coefficient :'
      WRITE(OutStr,'(A,2x,A)') TRIM(TT), TRIM(VSF_pure_model)
      Call WriteOut (Unt,'')
      
      IF( DO_RTAU_W ) THEN
         TT = '   User defined Rayleigh optical '
         WRITE(OutStr,'(A)') TRIM(TT)
         Call WriteOut (Unt,'')
         TT = '       optical thickness of water :'
         WRITE(OutStr,'(A,1x,F8.2)') TRIM(TT), RTAU_W
         Call WriteOut (Unt,'')
      ELSE
         TT = '   User defined Rayleigh optical '
         WRITE(OutStr,'(A)') TRIM(TT)
         Call WriteOut (Unt,'')
         TT = '       optical thickness of water :'
         WRITE(OutStr,'(A,1x,A)') TRIM(TT), ' not used'
         Call WriteOut (Unt,'')
      ENDIF
      TT = '   Constant depolarization ratio  :'
      WRITE(OutStr,'(A,1x,F5.2)') TRIM(TT), DEP_W_INPUT
      Call WriteOut (Unt,'')
      TT = '   Mean refractive index of water (real part):'
      WRITE(OutStr,'(A,1x,F6.3)') TRIM(TT), Ref_ind
      Call WriteOut (Unt,'')
      
 111  CONTINUE
      
c     ... In collection "st_tools.f"
      CALL WRITE_UD(2,Unt)
      
      Call CloseOut (Unt)
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      
 894  CALL WRITE_MESSAGE(R_NAME,'ERROR', 'ERROR IN LINTP2')
      ERROR_TXT = 'Error in '//R_NAME
      RETURN 1

      END SUBROUTINE RAYLEIGH_OUT
c********************************************************************

      SUBROUTINE  REPLACE_EXA(*)

      USE LOG_PARS, ONLY: C_UNIT, EU
c     ================== new KW
      USE TOOLS,    ONLY: SPLIT_STRING
      USE TYPES,    ONLY: KND, DYN_CHAR

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'REPLACE_EXA'
      INTEGER     NL_EXA, J, I
      CHARACTER (LEN = 10) STR
      REAL (KND), ALLOCATABLE ::   Z_EXA(:), C_EXA(:,:), TMP_READ(:)
c     ================== new KW
      INTEGER cnohead_1, cnohead_2, wav1found, wav2found    
      CHARACTER (LEN=200)  dumLine        
      TYPE (DYN_CHAR), ALLOCATABLE :: STR_ARR(:)
      REAL (KND)     realhlp, realhlp2

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      OPEN(C_UNIT, FILE = EXA_NAME, STATUS='OLD')
      cnohead_1 = 0
      cnohead_2 = 0
      wav1found = 0
      wav2found = 0
      DO
         READ(C_UNIT,'(A)', END = 787) dumLine
         CALL SPLIT_STRING(dumLine, STR_ARR)  
         DO WHILE (ANY(STR_ARR(1)%S == [';', '#', '*', '>']))
            READ(C_UNIT,'(A)') dumLine
            CALL SPLIT_STRING(dumLine, STR_ARR) 
            IF (SIZE(STR_ARR) >= 2) THEN
            IF (ANY(STR_ARR(2)%S == ['Wavelenghts'])) THEN
               IF (wav1found == 1 .AND. wav2found == 1) GOTO 106
               IF (wav1found == 1) wav2found = 1
               IF (wav1found == 0) wav1found = 1
            ENDIF
            ENDIF
         END DO
         IF (wav1found == 1 .AND. wav2found == 0) THEN
            cnohead_1 = cnohead_1 + 1
         ENDIF
         IF (wav1found == 1 .AND. wav2found == 1) THEN
            cnohead_2 = cnohead_2 + 1
         ENDIF
               
      END DO
 787  CONTINUE
            
      IF (cnohead_1 == cnohead_2) THEN 
         NL_EXA = cnohead_1
      ELSE 
         GOTO 105
      ENDIF
            
      IF (wav1found == 1 .AND. wav2found == 1) THEN 
         N_EXA_WL = 2
      ELSE 
         GOTO 106
      ENDIF
            
      IF (ALLOCATED(EXA_WL)) DEALLOCATE(EXA_WL, EXA_AC)
      ALLOCATE( EXA_WL(N_EXA_WL), EXA_AC(NLAYERS, N_EXA_WL) )
      ALLOCATE(Z_EXA(NL_EXA), C_EXA(NL_EXA,N_EXA_WL))
            
      REWIND(C_UNIT)
            
      cnohead_1 = 0
      cnohead_2 = 0
      wav1found = 0
      wav2found = 0
      DO
         READ(C_UNIT,'(A)', END = 788) dumLine
         CALL SPLIT_STRING(dumLine, STR_ARR)   
         IF (ANY(STR_ARR(1)%S == [';', '#', '*', '>'])) THEN
            IF (SIZE(STR_ARR) >= 2) THEN
               IF (ANY(STR_ARR(2)%S == ['Wavelenghts'])) THEN
                  IF (wav1found == 1) THEN
                     wav2found = 1
                     READ(STR_ARR(4)%S,*) realhlp
                     READ(STR_ARR(6)%S,*) realhlp2
                     EXA_WL(2) = (realhlp + realhlp2) / 2.0D0
                  ENDIF
                  IF (wav1found == 0) THEN 
                     wav1found = 1
                     READ(STR_ARR(4)%S,*) realhlp
                     READ(STR_ARR(6)%S,*) realhlp2
                     EXA_WL(1) =  (realhlp + realhlp2) / 2.0D0
                  ENDIF
               ENDIF               
            ENDIF               
         ELSE
            IF (SIZE(STR_ARR) >= 2) THEN
               IF (wav1found == 1 .AND. wav2found == 0) THEN
                  cnohead_1 = cnohead_1 + 1
                  READ(STR_ARR(1)%S,*) realhlp
                  Z_EXA(cnohead_1) = realhlp
                  READ(STR_ARR(2)%S,*) realhlp
                  C_EXA(cnohead_1,1) = realhlp
               ENDIF   
               IF (wav1found == 1 .AND. wav2found == 1) THEN
                  cnohead_2 = cnohead_2 + 1      
                  READ(STR_ARR(2)%S,*) realhlp
                  C_EXA(cnohead_2,2) = realhlp
               ENDIF   
            ENDIF   
         ENDIF
               
      END DO
 788  CONTINUE
            
      CLOSE(C_UNIT)

!       OPEN(C_UNIT, FILE = EXA_NAME, STATUS='OLD')
!       CALL SKIP_HEADER (C_UNIT)
!       READ(C_UNIT,*) N_EXA_WL
!       ALLOCATE( EXA_WL(N_EXA_WL), EXA_AC(NLAYERS, N_EXA_WL) )
!       READ(C_UNIT,*) EXA_WL
!       NL_EXA = 0
!       DO
!          READ(C_UNIT, *, END = 700) STR
!          IF(LEN_TRIM(STR) > 0) NL_EXA = NL_EXA + 1
!       END DO
!  700  CONTINUE
!       REWIND(C_UNIT)
!       CALL SKIP_HEADER (C_UNIT)
!       READ(C_UNIT,*) 
!       READ(C_UNIT,*)
!       ALLOCATE( Z_EXA(NL_EXA), C_EXA(NL_EXA,N_EXA_WL), 
!      $     TMP_READ(3*N_EXA_WL) )
!       DO J = 1, NL_EXA
!          READ(C_UNIT,*) I, Z_EXA(J), C_EXA(J,:)
! c         READ(C_UNIT,*) Z_EXA(J), TMP_READ(:)
! c         C_EXA(J,:) = TMP_READ(3:3*N_EXA_WL:3)
!       END DO
!       CLOSE(C_UNIT)

      DO I = 1, N_EXA_WL
         CALL LIN_INT(NL_EXA, Z_EXA, C_EXA(:,I), NLAYERS, ZZZ, 
     $        EXA_AC(:,I), 'L')
      END DO

      DEALLOCATE( Z_EXA, C_EXA )

      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN
      
      
 105  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $     'Aerosol grids differ with wavelength')
      WRITE(EU,*) '    Grids: ', cnohead_1, cnohead_2
      RETURN 1
      
 106  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $     'Less or more than 2 wavelengths for aerosol extinction')
      RETURN 1

      END SUBROUTINE REPLACE_EXA

C**************************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS
      IF(ALLOCATED(PERTURB)) DEALLOCATE(PERTURB)
      IF(ALLOCATED(AER_RET_FACTOR)) DEALLOCATE(AER_RET_FACTOR)
      IF(ALLOCATED(SCR_WL)) DEALLOCATE(SCR_WL, EXA_OUT, 
     $     EXA_OUT_WL, EXA_RET, EXA_RET_WL)
      IF(ALLOCATED(VERT_COL)) DEALLOCATE(VERT_COL)
      IF(ALLOCATED(WF_TO_AMF)) DEALLOCATE(WF_TO_AMF)
      IF(ALLOCATED(EXA_WL)) DEALLOCATE(EXA_WL, EXA_AC)
      IF(ALLOCATED(Ram_Egg)) DEALLOCATE(Ram_Egg, Ram_EgL, Ram_Egs, 
     $     Ram_EsL)
      IF(ALLOCATED(VRS_Egg)) DEALLOCATE(VRS_Egg, VRS_Egs)
      IF(ALLOCATED(VRS_EgL)) DEALLOCATE(VRS_EgL, VRS_EsL)
      IF(ALLOCATED(L_SOL)) DEALLOCATE(L_SOL)

      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS

C**************************************************************************

      END
C********************************************************************C
