C********************************************************************C
C
C       SCIATRAN - Version 3.8 - Last modification: 28 Apr 2017
C
C       Copyright (C) 1995-2014, Institute of Remote Sensing,
C                                University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE RTM_INIT_FIRST(NSPEC, NSEGS, WL, WL_START, WL_END, *)
c     ... Called by "rtm_main.f"

      USE ABSORBERS,   ONLY: N_GAS, N_MIX_GAS, MIX_ABS_START, N_MIX_MOL,
     $                       ST_GAS_IDX, N_ST_MOL, LINE_ABS_MODE,
     $                       GAS_INDEX, AMF_IDX
      USE ALB_PARS,    ONLY: ALBEDO_RET, ALBEDO_PERTR
      USE DATA_RET,    ONLY: RET_FLAG
      USE EMISSION,    ONLY: Emi_MOMS
      USE GT_POLYN ,   ONLY: NLEGEN 
      USE INFO,        ONLY: RET_INFO, TG_INFO
      USE INTERN_DIMS, ONLY: Nlg_ray, N_PHI_MS, Nlg_Ray_W,
     $                       Nm_vs, Nm_vs_W, Nvs, N_sol_ls
      USE LOG_PARS,    ONLY: EU, LUNIT, LOG_LEVEL, N_ESO
      USE NODES,       ONLY: N_NODE
      USE PHOTOCHEM,   ONLY: DO_PHOTOCHEM, PC_GAS_INDEX
      USE RTM_OUTPUT,  ONLY: N_OP
      USE SPHER_PARS,  ONLY: N_LOS_OUT, AZM_MS, N_FOV_ANG, N_SOL_ANG, 
     $                       NLSA => N_LOS_ANG
      USE ST_VARS,     ONLY: SS_MS_BOUNDARY, SCATT_MODE, PIE,
     $                       RAD_FIELD, RTM_TYPE, NSTRMS, Enhance_acc,  
     $                       WF_FLAG, RAD_TO_DEG, Nh_s, N_HARM, 
     $                       WF_MS_FLAG, RTM_MODE, RTM_CORE, Vec_Sca,
     $                       INT_MS_FLAG, M_LIM, SOL_ANG, DO_RAMAN, 
     $                       AZ_FLAG => DO_AZIMUTH, Emi_FLG, FLG_MS,
     $                       Sec_order
      USE TOOLS,       ONLY: INQ_WF
      USE TYPES,       ONLY: KND
      USE WATER,       ONLY: UWT
      USE WF_NUM,      ONLY: NUM_WF 

      INTEGER,    INTENT(IN)  :: NSPEC, NSEGS
      REAL (KND), INTENT(IN)  :: WL_START, WL_END
      REAL (KND)  WL(NSPEC)
c     ----------------------------------------------------------
c     ... Local variables 
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RTM_INIT_FIRST'
      INTEGER     I, K, L, II, KK, NH_MAX
      REAL (KND)  STEP_PHI
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here

      GAS_INDEX = 0
c     ... Number of gaseous components included in forward model
      N_GAS = COUNT(TG_INFO%FM_FLG) 
      N_MIX_GAS = COUNT(
     $     TG_INFO(MIX_ABS_START:MIX_ABS_START + N_MIX_MOL - 1)%FM_FLG) 

c     ====================================================
c     Array ST_GAS_IDX needs to be reinitialized
c     after all unnecessary absorbers are turned off! 
c     ====================================================
      DEALLOCATE(ST_GAS_IDX)
      ALLOCATE(ST_GAS_IDX(N_GAS), PC_GAS_INDEX(N_GAS), ! A
     $     ALBEDO_PERTR(NSEGS)) ! A
      ALBEDO_PERTR(:) = 0

      K = 0 ;  L = 0 
      DO I = 1, N_ST_MOL
         IF(.NOT.TG_INFO(I)%FM_FLG) CYCLE
         K = K + 1 ;  GAS_INDEX(I) = K ;  ST_GAS_IDX(K) = I
C        ... TG_INFO%TYPE can be modified by ABS_COEFF_READ and RTM_MAIN
         SELECT CASE (TG_INFO(I)%M_TYPE)
         CASE DEFAULT 
            TG_INFO(I)%TYPE = TG_INFO(I)%M_TYPE

         CASE ('line')
            TG_INFO(I)%TYPE = LINE_ABS_MODE

         CASE ('mixed')
            IF(TRIM(LINE_ABS_MODE) == 'off') THEN
               TG_INFO(I)%TYPE = 'xsec'
            ELSE
               TG_INFO(I)%TYPE = 'mixed'

               IF(ALLOCATED(TG_INFO(I)%TYPE_WL)) 
     $              DEALLOCATE(TG_INFO(I)%TYPE_WL)
               ALLOCATE(TG_INFO(I)%TYPE_WL(NSPEC))
               WHERE(WL(:) <= TG_INFO(I)%MAX_WL_CS ) 
                  TG_INFO(I)%TYPE_WL(:) = 'xsec'
               ELSEWHERE
                  TG_INFO(I)%TYPE_WL(:) = LINE_ABS_MODE   
               END WHERE
            END IF

         END SELECT
         
         IF(.NOT.DO_PHOTOCHEM(I)) CYCLE
         L = L + 1 ;  PC_GAS_INDEX(K) = L
      END DO    ! over all gaseous absorber (I)

c      WRITE(*,*) '   TG_INFO(I)%TYPE '
c      DO I = 1, N_ST_MOL
c         WRITE(*,*) I, TG_INFO(I)%TYPE
c      ENDDO
c      PAUSE

      SELECT CASE (TRIM(RTM_MODE))
      CASE ('amf', 'slant_col')
         IF(GAS_INDEX(AMF_IDX) == 0) GOTO 200
      CASE DEFAULT
      END SELECT

       IF(TRIM(RTM_MODE) == 'vod') THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      ALLOCATE(INT_MS_FLAG(NSPEC), WF_MS_FLAG(NSPEC))
      INT_MS_FLAG(:) = .TRUE.


      IF( .NOT.WF_FLAG ) THEN
         WF_MS_FLAG(:) = .FALSE.
      ELSE
         WF_MS_FLAG(:) = .TRUE.            
      END IF

c      WRITE(*,*) '    INT_MS_FLAG(:) 1 - ', INT_MS_FLAG(:)
c      WRITE(*,*) '    TRIM(SCATT_MODE)   ', TRIM(SCATT_MODE)
      
      SELECT CASE (TRIM(SCATT_MODE))
      CASE('ms')
c     ... Single scattering approximation will be used: 
c     ------------------------------------------------------------------
      CASE('ss')
c            for WF and intensity at all wavelengths
         INT_MS_FLAG(:) = .FALSE. ;  WF_MS_FLAG(:) = .FALSE.
      CASE('ss_ms')
c            for WF and intensity at wavelengths smaller than 
c               "SS_MS_BOUNDARY" value
         WHERE(WL(:) < SS_MS_BOUNDARY)
            INT_MS_FLAG(:) = .FALSE.
            WF_MS_FLAG(:)  = .FALSE.
         END WHERE
      CASE('sm_wf_ss')
c            for WF at all wavelengths
         WF_MS_FLAG(1:NSPEC) = .FALSE.
c            for intensity at wavelengths smaller than SS_MS_BOUNDARY value
         WHERE(WL(1:NSPEC) < SS_MS_BOUNDARY) INT_MS_FLAG(:) = .FALSE.
      CASE('wf_ss')
c            for WF at all wavelengths
         WF_MS_FLAG(1:NSPEC) = .FALSE.
      CASE('wf_ss_ms')
c            for WF at wavelengths smaller than SS_MS_BOUNDARY value
         WHERE(WL(1:NSPEC) < SS_MS_BOUNDARY) WF_MS_FLAG(:) = .FALSE.
      END SELECT
      IF(ANY(INT_MS_FLAG)) THEN
         FLG_MS = .TRUE.
      ELSE
         FLG_MS = .FALSE.         
      END IF
      
c      WRITE(*,*) '    INT_MS_FLAG(:) 2 - ', INT_MS_FLAG(:)
      
      N_PHI_MS = 1
      SELECT CASE (TRIM(RTM_CORE))
      CASE ('CDI')

      CASE ('ASYMP')
         N_NODE = 1             !

      CASE DEFAULT

         IF( AZ_FLAG ) THEN 
            N_HARM = NLEGEN
         ELSE 
            N_HARM = 1 
         ENDIF
c         IF( Sec_order ) N_HARM = 1 
         Nh_max = 250
         Nh_s = MIN(Nlegen,Nh_max)
         Nlg_ray = 3
         
         IF (LOG_LEVEL(1) >= N_ESO)  WRITE(LUNIT,*)
     $        ' ***** Polynomial will be calculated till harmonic :'//
     $        ' -------->> ', Nh_s

         IF( RAD_FIELD ) THEN
            STEP_PHI = 10.d0
            N_PHI_MS = INT(360.0d0/STEP_PHI) + 1
            ALLOCATE( AZM_MS(N_PHI_MS) )
            AZM_MS(1)  = 0.d0
            DO I = 2,N_PHI_MS
               AZM_MS(I) = AZM_MS(I-1) + STEP_PHI
            ENDDO
            AZM_MS(:) = AZM_MS(:)/180.d0*Pie
         ENDIF

         IF( UWT ) THEN
            Nlg_Ray_W = Nlg_ray ;  Nm_vs_W = Nm_vs
         ELSE
            Nlg_Ray_W = 1 ;  Nm_vs_W = 1
         ENDIF

         SELECT CASE (TRIM(RTM_TYPE))
         CASE('spher_scat')
            N_NODE = 2*NSTRMS
            ALLOCATE( M_LIM(NLSA*N_FOV_ANG)) !A
         CASE DEFAULT
            Enhance_acc = .FALSE.
            N_NODE = 1 
            ALLOCATE( M_LIM(NLSA) ) !A
         END SELECT
         
c        ============================================================
c        ... Resonance scattering in the atmosphere (metals and ions)
c        ============================================================
         IF( Emi_FLG ) THEN
            ALLOCATE( Emi_MOMS(Nlg_Ray,Nm_vs) ) ; Emi_MOMS = 0.d0
            SELECT CASE (Vec_Sca)
            CASE('scal_rte')
               Emi_MOMS(1,1) = 1.d0 
               Emi_MOMS(2,1) = 0.d0 
               Emi_MOMS(3,1) = 1.d0/2.d0
            CASE('vect_rte')
               Emi_MOMS(1,1) = 1.d0      ! alpha1 first harmonic
               Emi_MOMS(2,4) = 3.d0/2.d0 ! alpha4 second harmonic
               Emi_MOMS(3,1) = 1.d0/2.d0 ! alpha1 third harmonic
               Emi_MOMS(3,2) = 3.d0      ! alpha2 third harmonic
c             ... The expansion coefficient beta1  will be used  
c                 within the SCIATRAN model with the sign '-' what
c                 coincides with the convention of Hovenier, Domke,
c                 and van der Mee
               Emi_MOMS(3,5) = -DSQRT(3.d0/2.d0) ! beta1 third harmonic
c               Emi_MOMS(3,5) = DSQRT(3.d0/2.d0) ! beta1 third harmonic
            END SELECT            
         ENDIF
c        ... End of resonance scattering in the atmosphere 
c        >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         
c         N_sol_ls = 1
c         IF( SPECULAR ) N_sol_ls = 2
c        ... It is set to 2 to simplify the code
         N_sol_ls = 2

      END SELECT

      IF(INQ_WF('albedo', 'RIN')) ALLOCATE(ALBEDO_RET(3,NSEGS)) ! A

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME,'ERROR', 
     $     'Target absorption species for AMF or slant column '//
     $     'calculations is turned off or has no absorption bands '//
     $     'in the selected spectral range')
      RETURN 1      

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
C**************************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS
      
      IF(ALLOCATED(PC_GAS_INDEX)) DEALLOCATE(PC_GAS_INDEX, ALBEDO_PERTR)
      IF(ALLOCATED(INT_MS_FLAG)) DEALLOCATE(INT_MS_FLAG, WF_MS_FLAG)
      IF(ALLOCATED(AZM_MS)) DEALLOCATE(AZM_MS)
      IF(ALLOCATED(M_LIM)) DEALLOCATE(M_LIM)
      IF(ALLOCATED(ALBEDO_RET)) DEALLOCATE(ALBEDO_RET)
      IF(ALLOCATED(Emi_MOMS)) DeAllocate(Emi_MOMS)
      
      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS
C**************************************************************************

      END 
C********************************************************************C
