C***********************************************************************
C
C       SCIATRAN - Version 4.2 - Last modification: 24 Apr 2020
C 
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE RTM_MAIN (NSPEC_OUT, WL_OUT, SPECSEGS, FIRST_RUN, 
     $     NEW_ALT_GRID, READ_FLAG, WL_START, WL_END, *)
c     ... Called by "gt_iface.f"

      USE ALB_PARS,    ONLY: ALBEDO_CONST
      USE ABSORBERS,   ONLY: GAS_INDEX, AMF_IDX
      USE BRDF,        ONLY: MH_BRF, Exact_BRF, Npg, L_SOL
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE CLD_RET,     ONLY: NEW_CLD
      USE DATA_RET,    ONLY: RET_FLAG, MDS, PRESET_APR_NAME, MDS
      USE ESFT,        ONLY: DO_ESFT, ESFT_NCOEFF
      USE GRIDS,       ONLY: ZZZ, DZR, NLAYERS, MAX_LINE, NSOL
      USE INFO,        ONLY: WF_DB, TG_INFO, RET_INFO
      USE INTERN_DIMS, ONLY: Nlg_ray, Nm_vs, Nvs, Ng1_brf, Ng2_brf,
     $                 Nh_brf
      USE IRRAD,       ONLY: AWF => ACT_WL_FLAG, SOL_FLUX
      USE NODES,       ONLY: NEW_NODES
      USE OutInf,      ONLY: LUNIT, EU, DIR_OUT, N_SSO, LOG_LEVEL, 
     $                       N_RSO, N_ESO, TRC_FLG, WrtModRes
      USE PROFILES,    ONLY: PRESS, TEMP, AIR_DENSITY
      USE RAMAN,       ONLY: N_WL_C, DEL_WL, N_RG, DEL_RAM, RRS_ADJ, 
     $                       K_dim
      USE RETR_ITER,   ONLY: ITR => ITERATION
      USE RTM_OUTPUT,  ONLY: RTM_OUT_ARR, SPEC_OUT, SPEC_OUT_SAVE, 
     $                       OWF_OUT_VEC, STOK_OUT, WF_TO_AMF, MIN_WF_Z, 
     $                       MAX_WF_Z, WF_TG_FLAG
      USE RTM_PARS,    ONLY: WRT_ALT, RTM_ST, DATA_ITER
      USE ST_INPUT,    ONLY: AER_PAR_TYPE
      USE ST_VARS,     ONLY: WF_FLAG, NEW_SPEC_GRID, WL_RTM, N_Adj, TSM,
     $                       NSPEC_RTM, RTM_MODE, Pie, LA_FLAG, NSTRMS,
     $                       RTM_CORE, SEG_NUM_RTM, WL => WL_GRID, 
     $                       RET_MODE, CHECK_FLAG => DO_INTERNAL_CHECK,
     $                       AZ_FLAG => DO_AZIMUTH, DO_RAMAN, APPR_RRS,
     $                       SOLVER, N_RHS, N_HR, N_HR_EXT, RTM_TYPE,
     $                       OWF_FLAG, SEG_NUM_OUT, SEG_NUM, TP_HEIGHT,
     $                       INEL_WHERE, SCATT_MODE, NSEGS, MSK_RTM, 
     $                       WL_SEGS, GeoD_Out, ERROR, BRDF_FLAG
      USE SPHER_PARS,  ONLY: N_LOS_OUT, N_LOS_ANG, DO_SF_OUT, N_SOL_ANG,
     $                       PE_Mode, NLSA => N_LOS_ANG
      USE WF_NUM,      ONLY: NUM_WF, NUM_PRE, NUM_TEM, PERTURB, AL_BASE, 
     $                       AL_1_BASE, AL_1_PERT, NUM_OWF
      USE TOOLS,       ONLY: FIND_IDX, INQ_WF
      USE TYPES,       ONLY: KND, RRS_TYPE, VRS_TYPE
      USE WATER,       ONLY: UWT, NLW => NLAYERS_UW, NSTR_W, ZZZ_UW,
     $     INEL_DO, INEL_ADJ, INCL_INEL
      USE WF_NUM,      ONLY: NUM_AER, SCA_or_ABS
c     ------------------------------------------------------------------

      INTEGER,  INTENT(IN) :: NSPEC_OUT
      LOGICAL,  INTENT(IN) :: FIRST_RUN, NEW_ALT_GRID, READ_FLAG
      INTEGER   SPECSEGS(2,NSEGS)
      REAL (KND), INTENT(IN) ::  WL_START, WL_END
      REAL (KND), TARGET  :: WL_OUT(NSPEC_OUT)

c     ... Local variables
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RTM_MAIN'
      CHARACTER (LEN=12) RUN_TYPE
      LOGICAL   NEW_PT, WF_TG_FLAG_LOC, NEW_WL_GRID, FLG_LS(N_LOS_OUT)
      LOGICAL, ALLOCATABLE :: MASK(:,:), MASK_TMP(:,:)
      INTEGER   I, J, K, L, N1, Jr, NSPEC, JPOS, MAX_SPEC, JSEG
      INTEGER   N_last, KK, Jper, I_ACT, N_ACT, I_ALB_1, I_ALB_2
      INTEGER   NNv, NS_LOC, II, ISP, J_S, J_E, I_L, IRS, IS1, IS2, NG
      INTEGER   IA, IV, NLS
      REAL (KND)  DUM(1,1), DUM_1(1,1,1), DUM_2(1,1), DELTA, WL_1, WL_2
      REAL (KND), ALLOCATABLE :: XSOL(:)
      REAL (KND), POINTER :: WL_ACT(:), Z_LOC(:)
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
      TYPE (DATA_ITER), POINTER :: RTM_PTR
c     ==================================================================
      Interface
         SUBROUTINE DOM_SPHERICAL( LPOS, POS, SRR, SVR, SRR_JACT )
         USE TYPES, ONLY: KND, RRS_TYPE, VRS_TYPE
         INTEGER, INTENT(IN) :: LPOS
         INTEGER, OPTIONAL :: SRR_JACT
         REAL (KND) POS
         TYPE (RRS_type) :: SRR 
         TYPE (VRS_type) :: SVR 
         End SUBROUTINE DOM_SPHERICAL
      End Interface
      
      CALL ST_TRACK (R_NAME, 3, 'I')

      DO_SF_OUT = .FALSE.
c      DO_SF_OUT = .TRUE.
      IF(PE_Mode == 'WL'.OR.LOG_LEVEL(1) < N_SSO) DO_SF_OUT = .FALSE.

      NEW_WL_GRID = .TRUE. ;  SVR%Mh_OUT = 3
      SELECT CASE( TRIM(INEL_WHERE) )
c     ... Vibrational Raman scattering and fluorescence in ocean
c     ----------------------------------------------------------
      CASE('In water')
         IF( NEW_SPEC_GRID ) THEN
            N_ACT = NSPEC_RTM ; WL_ACT => WL_RTM
         ELSE
            N_ACT = NSPEC_OUT ; WL_ACT => WL_OUT
         ENDIF
         CALL EXT_WL_VRS(N_ACT, WL_ACT, SVR, *880)
         NSPEC = SVR%N_EXT ; WL => SVR%WL_EXT

         ALLOCATE( SRR%SEG_NUM_RRS(SVR%N_EXT) ) ! Preliminary 
         ALLOCATE( SVR%Isav(SVR%N_EXT) )
         ALLOCATE( SRR%Act_WL(NSPEC),SRR%Exi_WL(NSPEC) ) 
         
c        ... Preliminary solution to calculate RT using LBL mode
c            !!! Require further consideration to use convolution !!!
         IF( ALLOCATED(LA_FLAG) ) DEALLOCATE( LA_FLAG )
         ALLOCATE( LA_FLAG(NSPEC) ) ; LA_FLAG = .TRUE.

c        ... AWF => ACT_WL_FLAG will be used to extract solar
c            spectra at actual wavelengths
         IF( .NOT.ALLOCATED(AWF) )  ALLOCATE( AWF(NSPEC) ) 
         AWF(:) = SVR%ACT_WL(:) 

c        ... No RRS in atmosphere will be considered
         SRR%Act_WL(:) = .TRUE. ; SRR%Exi_WL(:) = .FALSE.

         SRR%SEG_NUM_RRS(:) = 1
         SEG_NUM  => SRR%SEG_NUM_RRS
         N_HR_EXT = NSPEC

c        ... Number of harmonics that should be saved 
         IF( INCL_INEL(1) ) THEN
c           ... Vibrational Raman scattering
            SVR%Mh_OUT = 3
         ELSE
c           ... Fluorescence
            SVR%Mh_OUT = 1
         ENDIF

c     ... Rotational Raman in atmosphere and inelastic
c         processes in ocean (VRS, Fluorescence)
c     ------------------------------------------------
      CASE('In air+water')
         N_WL_C = 0
         ALLOCATE( SRR%GAM(N_RG) )         
c        ... Basic parameters of rotation Raman scattering 
c            Output are N_WL_C, WL_FACT, DEL_WL(:), LINE_GAS(:)
         CALL ST_RRS_WL(*880)
         IF( NEW_SPEC_GRID ) THEN
            N_ACT = NSPEC_RTM ; WL_ACT => WL_RTM
         ELSE
            N_ACT = NSPEC_OUT ; WL_ACT => WL_OUT
         ENDIF
         SRR%N_ACT = N_ACT

         CALL EXT_WL_ALL(N_ACT, WL_ACT, SRR, SVR, *880)
         NSPEC = SVR%N_EXT ; WL => SRR%WL_EXT

         ALLOCATE( SVR%Isav(SVR%N_EXT) )

c        ... AWF => ACT_WL_FLAG will be used to extract solar
c            spectra at actual wavelengths
         IF( .NOT.ALLOCATED(AWF) )  ALLOCATE( AWF(NSPEC) ) 
c        ... Note that in this case SRR%ACT_WL(:) = SVR%ACT_WL(:)
         AWF(:) = SVR%ACT_WL(:) 

         ALLOCATE( SRR%SEG_NUM_RRS(SRR%N_EXT) ) ! Preliminary 
         SRR%SEG_NUM_RRS(:) = 1
         SEG_NUM  => SRR%SEG_NUM_RRS
         N_HR_EXT = NSPEC
c        ... Preliminary corresponds to VRS and RRS only
         SVR%Mh_OUT = 3

c     ... Rotational Raman scattering in atmosphere
c     ---------------------------------------------
      CASE('In air')
         N_WL_C = 0
         ALLOCATE( SRR%GAM(N_RG) )
         
c        ... Output are N_WL_C, WL_FACT, DEL_WL(:), LINE_GAS(:)
         CALL ST_RRS_WL(*880)

         IF( NEW_SPEC_GRID ) THEN
            N_ACT = NSPEC_RTM ; WL_ACT => WL_RTM
         ELSE
            N_ACT = NSPEC_OUT ; WL_ACT => WL_OUT
         ENDIF
         SRR%N_ACT = N_ACT

         CALL EXT_WL_RRS(N_ACT, WL_ACT, SRR, *880)
         NSPEC = SRR%N_EXT ; WL => SRR%WL_EXT

c        ... AWF => ACT_WL_FLAG will be used to extract solar
c            spectra at actual wavelengths
         IF( .NOT.ALLOCATED(AWF) )  ALLOCATE( AWF(NSPEC) ) 
         AWF(:) = SRR%ACT_WL(:) 

         ALLOCATE( SRR%SEG_NUM_RRS(SRR%N_EXT) ) ! Preliminary 

         ALLOCATE( SVR%Act_WL(NSPEC),SVR%Exi_WL(NSPEC) ) 
         SVR%Act_WL(:) = .TRUE. ; SVR%Exi_WL(:) = .FALSE.

         SRR%SEG_NUM_RRS(:) = 1
         SEG_NUM  => SRR%SEG_NUM_RRS
         N_HR_EXT = NSPEC

c     ... No inelastic processes are included
c     ---------------------------------------
      CASE('Nowhere')
         N_HR_EXT = N_HR

         IF( NEW_SPEC_GRID ) THEN  
            NSPEC = NSPEC_RTM ; WL => WL_RTM 
            SEG_NUM  => SEG_NUM_RTM
         ELSE
            NEW_WL_GRID = .FALSE.
            NSPEC = NSPEC_OUT ; WL => WL_OUT
            SEG_NUM  => SEG_NUM_OUT
         END IF

c     ... AWF => ACT_WL_FLAG will be used to extract solar
c            spectra at actual wavelengths
         IF( .NOT.ALLOCATED(AWF) )  ALLOCATE( AWF(NSPEC) ) 
         AWF(:) = .TRUE.

         ALLOCATE( SRR%Act_WL(NSPEC),SRR%Exi_WL(NSPEC),
     $        SVR%Act_WL(NSPEC),SVR%Exi_WL(NSPEC) ) ! A

c        ... No RRS in atmosphere will be considered
         SRR%Act_WL(:) = .TRUE. ;  SRR%Exi_WL(:) = .FALSE.
c        ... No inelastic processes in ocean will be considered
         SVR%Act_WL(:) = .TRUE. ;  SVR%Exi_WL(:) = .FALSE.

      END SELECT
c     ..........

      ALLOCATE(MASK(NSPEC, N_LOS_OUT))
      IF(NEW_WL_GRID) THEN
c   MASK will be set to .FALSE. for spectral segments where no spectral
c   point is selected 
         DO JSEG = 1, NSEGS
            WL_1 = MINVAL(WL_SEGS(:,JSEG)) 
            WL_2 = MAXVAL(WL_SEGS(:,JSEG))
c   Find all wavelengths in the output wavelength grid which 
c   belong to the segment JSEG
            J_S = COUNT(WL_OUT < WL_1) + 1 ; J_E = COUNT(WL_OUT <= WL_2)
            FLG_LS(:) = ANY(MSK_RTM(J_S:J_E,:),1)
c   Set flags for all spectral points which belong to the segment
            FORALL(ISP = 1:NSPEC, I_L = 1:N_LOS_OUT, 
     $           SEG_NUM(ISP) == JSEG) MASK(ISP,I_L) = FLG_LS(I_L)
        END DO 
      ELSE
         MASK = MSK_RTM
      END IF

      IF(FIRST_RUN) CALL RTM_INIT_FIRST(NSPEC, NSEGS, WL,  
     $     WL_START, WL_END, *880)

      NEW_PT = .FALSE.
      IF (TRIM(RET_MODE) == 'Pres_Temp') NEW_PT = .TRUE.

c     Perform setup of atmospheric layering with/without clouds
c     Output: NLAYERS, ZZZ, DZR, CLD_LAYER_ARR
c     ---------------------------------------------------------
      IF(FIRST_RUN.OR.NEW_ALT_GRID) CALL GT_LAYER_SETUP (*880)         

!     Write current altitude grid in the model parameter data set
      CALL WRT_ALT (ZZZ)

      IF((.NOT.FIRST_RUN).AND.(.NOT.NEW_ALT_GRID)) THEN
         DO IRS = 1, SIZE(RTM_ST)
            DO IS1 = 1, SIZE(RTM_ST(IRS)%SPR,1)
               DO IS2 = 1, SIZE(RTM_ST(IRS)%SPR,2)
                  ASSOCIATE(ARR => RTM_ST(IRS)%SPR(IS1,IS2))
                  IF(.NOT.ALLOCATED(ARR%DAT(ITR - 1)%NXT)) CYCLE
                     IF(RTM_ST(IRS)%VS == "S") THEN
                        NG = 1
                     ELSE
                        NG = SIZE(ARR%DAT(ITR)%GRD)
                     END IF
                     ALLOCATE(ARR%DAT(ITR)%APR(NG),ARR%DAT(ITR)%CRN(NG),
     $                    ARR%DAT(ITR)%NXT(NG))
                     ARR%DAT(ITR)%NXT(:) = 0.0D0
                     ARR%DAT(ITR)%CRN(:) = ARR%DAT(ITR - 1)%NXT(:)
                     ARR%DAT(ITR)%APR(:) = ARR%DAT(ITR - 1)%APR(:)
                  END ASSOCIATE
               END DO
            END DO
         END DO
      ENDIF

      IF(FIRST_RUN.OR.NEW_ALT_GRID.OR.NEW_NODES.OR.NEW_PT
     $     .OR.NEW_CLD) THEN
         CALL RTM_INIT(FIRST_RUN, NEW_ALT_GRID, NEW_NODES, NEW_PT, 
     $        WL_START, WL_END, NSPEC, NSPEC_OUT, WL, SRR, *880)
      ENDIF

      IF(FIRST_RUN.AND.CHECK_FLAG) CALL GT_CHECK (WL, NSPEC, *880)

c     ==========================================
c     ... Bidirectional reflection function setup
c     ==========================================
      Npg = 1
      IF( BRDF_FLAG ) THEN
c         ... Number of additional (w.r.t. Gaussian) angles, which
c             will be used as reflection angles in BRF
         IF( Exact_BRF )  Npg = NLSA         
c        ... Signs Mu and Mu0 correspond to the z-axis
         CALL GENERAL_BRDF_SETUP
c     $        (NSPEC, WL, WL_START, WL_END, XSOL, *880)         
     $        (NSPEC, WL, WL_START, WL_END, *880)         
c        ... Internal dimensions for the surface BRDF arrays
c             used in "dom_spherical" and routines below
         Ng1_brf = Nvs*(NSTRMS+Npg) ; Ng2_brf = Nvs*NSTRMS
         Nh_brf  = Mh_BRF
      ENDIF

C     All output arrays are initialized with zero
      RTM_OUT_ARR = 0.d0
C     All atmospheric weighting functions are re-initialized
      IF( WF_FLAG.OR.NUM_WF ) THEN
         MAX_SPEC = MAX0(NSPEC, NSPEC_OUT)
         CALL RTM_OUTPUT_WFS(MAX_SPEC)
      END IF

      IF( OWF_FLAG ) THEN
         OWF_OUT_VEC(:,:,:,:,:) = 0.d0
      END IF

      SELECT CASE( TRIM(INEL_WHERE) )
c     ... Vibrational Raman scattering in ocean
c     -----------------------------------------
      CASE('In water')
         CALL SET_POINTERS(N_ACT, *880)
         NS_LOC = N_SOL_ANG
c        ... SVR%N_Aux is the number of excitation wavelengths
         ALLOCATE(
     $        SVR%NUJ_EXC(SVR%N_Aux,3,NSTR_W,NLW-1),           ! A
     $        SVR%EXT_AUX(SVR%N_Aux,NLW-1),                    ! A
     $        SVR%EXS_AUX(SVR%N_Aux,NLW,NS_LOC),               ! A
     $        SVR%MVEC   (SVR%N_Aux,3,2*NSTR_W*(NLW-1),N_RHS), ! A
     $        SVR%Aj     (SVR%N_Aux,3,NSTR_W,NLW-1,NS_LOC),    ! A
     $        SVR%Bj     (SVR%N_Aux,3,NSTR_W,NLW-1,NS_LOC),    ! A
     $        SVR%GNp    (SVR%N_Aux,3,NLW-1,NSTR_W,NSTR_W),    ! A
     $        SVR%GNm    (SVR%N_Aux,3,NLW-1,NSTR_W,NSTR_W) )   ! A
c        ... If forward-adjoint technique is selected
         IF( INEL_ADJ ) THEN
            ALLOCATE( SVR%ADJ_ACT(NLW,N_ACT,2*NSTR_W,N_Adj,3), ! A
     $           SVR%EXL_ACT(NLW,N_ACT,N_LOS_ANG) )            ! A
            SVR%ADJ_ACT = 0.d0  ;  SVR%EXL_ACT = 0.d0 
         ENDIF

c     ... Rotational Raman scattering in atmosphere
c     ---------------------------------------------
      CASE('In air')
         CALL SET_POINTERS(N_ACT, *880)
         SRR%M_ESF = 1
         IF( DO_ESFT ) SRR%M_ESF = MAXVAL( ESFT_NCOEFF )

         ALLOCATE( SRR%NESFT(N_ACT) )
c        ... Number of solar angle depends on the selected mode
         NS_LOC = N_SOL_ANG
c        ... Raman scattering in spherical geometry 
         IF( TRIM(RTM_TYPE)=='spher_scat' ) THEN
            NS_LOC = NSOL
            IF(ALLOCATED(SRR%LS_ACT)) THEN
               DO IA = 1, GeoD_OUT%NAlts
                  DO IV = 1, GeoD_OUT%NVA
                     DeAllocate(SRR%LS_ACT(IA,IV)%EX)
                  END DO
               END DO
               DeAllocate(SRR%LS_ACT)
            END IF
            ALLOCATE(SRR%LS_ACT(GeoD_OUT%NAlts,GeoD_OUT%NVA))
            DO IA = 1, GeoD_OUT%NAlts
               DO IV = 1, GeoD_OUT%NVA
                  NLS = GeoD_OUT%LVG(IA,IV)%NLS
c                 ... Allocate corresponding to the selected integration schema
                  ALLOCATE(SRR%LS_ACT(IA,IV)%EX(NLS,N_ACT,SRR%M_ESF) )
               END DO
            END DO
         ENDIF

c        ... If forward-adjoint technique is selected
         IF( RRS_ADJ ) THEN
            ALLOCATE( SRR%EXL_ACT(NLAYERS,N_ACT,N_LOS_ANG), 
     $           SRR%ADJ_ACT(NLAYERS,N_ACT,2*NSTRMS*Nvs,N_Adj,3) )
            SRR%EXL_ACT = 0.d0  ;  SRR%ADJ_ACT = 0.d0
            IF( TRIM(TSM) == 'speed' ) THEN
               ALLOCATE( SRR%GAM_WL(N_RG,N_ACT) )
            ENDIF

c        ... If discrete-ordinates technique is selected
         ELSE
            SELECT CASE( TRIM(APPR_RRS) )
            CASE('exact matching')
c              ... To decrease dimension of arrays, where solution for
c                  excitation wavelengths should be saved, it will be  
c                  saved for this mode solutions at excitation wavelengths
c                  for only one given actual wavelength
               K_dim = N_WL_C + 1
            CASE('simple binning')
c              ... SRR%N_AUX is defined in "ext_wl_rrs.f" as a number of
c                  wavelengths extended WL-grid which should be calculated
c                  before actual
               K_dim = SRR%N_AUX + 1
            END SELECT
            NNv = Nvs*NSTRMS
            ALLOCATE( 
     $           SRR%GAM_WL (N_RG,K_dim),                      ! A
     $           SRR%NUJ_EXC(K_dim,3,NNv,NLAYERS-1),           ! A
     $           SRR%EXT_AUX(K_dim,NLAYERS-1),                 ! A
     $           SRR%EXS_AUX(K_dim,NLAYERS,NS_LOC),            ! A
     $           SRR%MVEC   (K_dim,3,2*NNv*(NLAYERS-1),N_RHS), ! A
     $           SRR%Aj     (K_dim,3,NNv,NLAYERS-1,NS_LOC),    ! A
     $           SRR%Bj     (K_dim,3,NNv,NLAYERS-1,NS_LOC),    ! A
     $           SRR%GNp    (K_dim,3,NLAYERS-1,NNv,NNv),       ! A
     $           SRR%GNm    (K_dim,3,NLAYERS-1,NNv,NNv) )      ! A

         ENDIF                  ! discrete-ordinates technique

c        ... If single scattering mode, single scattering correction
c            algorithm or spherical atmosphere is switched on then
         IF( DO_SSRCA.OR.TRIM(SCATT_MODE)=='ss'.OR.
     $        TRIM(RTM_TYPE)=='spher_scat' ) THEN

            ALLOCATE( SRR%EXT_ACT(NLAYERS,N_ACT,SRR%M_ESF),
     $           SRR%WEI(N_ACT,SRR%M_ESF) )
            SRR%EXT_ACT = 0.d0
         ENDIF
c        ... Allocate additional arrays in SRR structure to speed up
c            the calculations of rotational Raman scattering
         IF( TRIM(TSM) == 'speed' ) THEN
            ALLOCATE( SRR%EXS_ACT(NLAYERS,N_ACT,NS_LOC),
     $           SRR%FOR_ACT(NLAYERS,N_ACT,NS_LOC,2*NSTRMS*Nvs,3))
            SRR%EXS_ACT = 0.d0 ; SRR%FOR_ACT = 0.d0
         END IF

c     ... Rotational Raman in atmosphere and inelastic
c         processes in ocean (VRS, Fluorescence)
c         (DO-technique omly)
c     ------------------------------------------------
      CASE('In air+water')
         CALL SET_POINTERS(N_ACT, *880)
         NS_LOC = N_SOL_ANG
c        ... K_dim is the number of VRS excitation wavelengths
         K_dim = COUNT( SVR%Exi_WL(:) ) + 1
c         WRITE(*,*) '  rtm_main K_dim for VRS = ',K_DIM
c         WRITE(*,*) '               SVR%N_aux = ',SVR%N_aux
         ALLOCATE(
     $        SVR%NUJ_EXC(K_dim,3,NSTR_W,NLW-1),           ! A
     $        SVR%EXT_AUX(K_dim,NLW-1),                    ! A
     $        SVR%EXS_AUX(K_dim,NLW,NS_LOC),               ! A
     $        SVR%MVEC   (K_dim,3,2*NSTR_W*(NLW-1),N_RHS), ! A
     $        SVR%Aj     (K_dim,3,NSTR_W,NLW-1,NS_LOC),    ! A
     $        SVR%Bj     (K_dim,3,NSTR_W,NLW-1,NS_LOC),    ! A
     $        SVR%GNp    (K_dim,3,NLW-1,NSTR_W,NSTR_W),    ! A
     $        SVR%GNm    (K_dim,3,NLW-1,NSTR_W,NSTR_W) )   ! A
         SRR%M_ESF = 1
         IF( DO_ESFT ) SRR%M_ESF = MAXVAL( ESFT_NCOEFF )
c         WRITE(*,*) '    SHAPE(ESFT_NCOEFF)', SHAPE(ESFT_NCOEFF)

c         ALLOCATE( SRR%Ram_MOMS(Nlg_Ray,Nm_vs), SRR%NESFT(N_ACT) )
         ALLOCATE( SRR%NESFT(N_ACT) )

c        ... Set moments of the Raman phase function
c            (moved to "rtm_init_first.f")
c         SRR%Ram_MOMS(1,1) = 1.d0 ; SRR%Ram_MOMS(2,1) = 0.d0 
c         SRR%Ram_MOMS(3,1) = 1.d0/20.d0

c        ... K_dim is the number of RRS excitation wavelengths
         K_dim = COUNT( SRR%Exi_WL(:) ) + 1
c         WRITE(*,*) '  rtm_main K_dim for RRS = ',K_DIM
         ALLOCATE( 
     $        SRR%GAM_WL (N_RG,K_dim),                         ! A
     $        SRR%NUJ_EXC(K_dim,3,NSTRMS,NLAYERS-1),           ! A
     $        SRR%EXT_AUX(K_dim,NLAYERS-1),                    ! A
     $        SRR%EXS_AUX(K_dim,NLAYERS,NS_LOC),               ! A
     $        SRR%MVEC   (K_dim,3,2*NSTRMS*(NLAYERS-1),N_RHS), ! A
     $        SRR%Aj     (K_dim,3,NSTRMS,NLAYERS-1,NS_LOC),    ! A
     $        SRR%Bj     (K_dim,3,NSTRMS,NLAYERS-1,NS_LOC),    ! A
     $        SRR%GNp    (K_dim,3,NLAYERS-1,NSTRMS,NSTRMS),    ! A
     $        SRR%GNm    (K_dim,3,NLAYERS-1,NSTRMS,NSTRMS) )   ! A
c         WRITE(*,*) '  SHAPE(SRR%GNp) ', SHAPE(SRR%GNp)
c         WRITE(*,*) '  SHAPE(SVR%GNp) ', SHAPE(SVR%GNp)
c         stop
c     ... Inelastic processes are not included   
c     ----------------------------------------
      CASE('Nowhere')
         IF(READ_FLAG) THEN
            CALL SET_POINTERS(NSPEC_OUT, *880)      
            CALL SCIA_MOD_READ(NSPEC_OUT, SPEC_OUT, N_LOS_OUT)
            CALL ST_TRACK (R_NAME, 3, 'O')
            RETURN
         ELSE
            CALL SET_POINTERS(NSPEC, *880)      
         END IF 
      END SELECT
    
 334  CONTINUE

      IF(ALLOCATED(ERROR)) DEALLOCATE(ERROR)
      ALLOCATE(ERROR(SIZE(WL)))
      ERROR = .FALSE.
      
      SELECT CASE (RTM_CORE)
      CASE('CDI', 'TRN')
         CALL RUN_CDI (*880)
      CASE('VOD')
         DO JPOS = 1, NSPEC     ! Wavelenght loop
            CALL ST_VOD (JPOS, WL(JPOS))
         END DO       
      CASE DEFAULT
c        ... Exclude calculation of WFs in "integrate_init.f90" and 
c            "integrate_los.f90" in the case RAD_Field = .TRUE. 
c            and WF_FLAG = .TRUE. 
         WF_TG_FLAG = .FALSE.
         CALL RUN_DOM_ASYMP
      END SELECT

      IF(ANY(ERROR)) GOTO 880 
      
      SELECT CASE (TRIM(RTM_MODE)) 
         CASE ('amf', 'slant_col')
            RTM_MODE = TRIM(RTM_MODE)//'2'
            SPEC_OUT_SAVE = SPEC_OUT(:,1,:)
            STOK_OUT = 0.d0
            TG_INFO(AMF_IDX)%TYPE = 'off'
c           ... Repeat calculation without selected gaseous absorber
            GOTO 334

         CASE ('amf2', 'slant_col2')
c            WRITE(*,'(A,*(2x,ES15.7))') ' 1 with abs.  - ',   
c     $           SPEC_OUT_SAVE(:,:), DLOG( SPEC_OUT_SAVE(:,:) )
c            WRITE(*,'(A,*(2x,ES15.7))') '  2   no abs. - ', 
c     $           SPEC_OUT(:,1,:), DLOG( SPEC_OUT(:,1,:) )
            SPEC_OUT(:,1,:) = DLOG(SPEC_OUT(:,1,:)/SPEC_OUT_SAVE(:,:))
                        
         CASE ('block_amf')
            DO II = 1, UBOUND(WF_DB,1)
               IF(WF_DB(II)%TYPE /= 'tg') CYCLE
               FORALL(JPOS = 1:NSPEC, J = 1:NLAYERS, I = 1:N_LOS_OUT)
     $              WF_DB(II)%WF(JPOS,I,J,1) = 
     $              - WF_DB(II)%WF(JPOS,I,J,1)/
     $              SPEC_OUT(JPOS,1,I)/WF_TO_AMF(JPOS,J)
            END DO
      END SELECT

      IF( NEW_SPEC_GRID ) 
     $     CALL ST_CONVOL(NSPEC_OUT, WL_OUT, NSEGS, SPECSEGS, *880)

      IF(WrtModRes) CALL OUTPUT(NSPEC_OUT, WL_OUT, SRR, SVR)

!      open(37, file = '../tmp.tmp')
!      read(37,*) SPEC_OUT
!      do ii = 1, size(WF_DB)
!         read(37,*) WF_DB(II)%WF
!      end do
!      close(37)

      IF( RRS_ADJ ) THEN
         DeAllocate (SRR%EXL_ACT, SRR%ADJ_ACT)
         IF( TRIM(TSM) == 'speed' ) DeAllocate (SRR%GAM_WL)
      ENDIF

      IF (LOG_LEVEL(1) >= N_RSO)  THEN
         WRITE(LUNIT,*)
         WRITE(LUNIT,*) 
     $        '****** RT calculations terminated successfully *****'
         WRITE(LUNIT,*) 
     $        '********* Results are stored according to **********'
         WRITE(LUNIT,*) 
     $        '************ '//DIR_OUT//'output_map.inf *************'
      END IF

      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c ********************************************************************

      SUBROUTINE ST_UPDATE_MAN
      USE AEROSOLS,    ONLY: AER_COEFF, SCI => GT_SCAT_IDX, 
     $                       EXI => GT_EXTC_IDX, R_erw, 
     $                       NAW => N_AER_WAVS, AER_WAVS
      USE AER_MIE,    ONLY: DENS_PRF, T_550, AOD_RET
      USE GRIDS,      ONLY: NLAYERS, DZ
      USE DELTA_M,    ONLY: FDM_AER
      USE INFO,       ONLY: AER_INFO, RET_INFO
      USE ST_VARS,    ONLY: RET_CORE
      USE RETR_ITER,  ONLY: ITERATION, AOD_ITER, SCAL_APR, FLG_OSC,
     $                      RMS_OSC, MAX_OSC, MIN_TAU
      USE RTM_PARS,   ONLY: DATA_ITER
      USE TOOLS,      ONLY: CHK_STAT, SET_PTR
c ----------------------------------------------------------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_UPDATE_MAN'
      LOGICAL E_RR
      INTEGER I, II, KK, KKK, Nrw
      REAL (KND) SSA(NAW), DD1, DD2, DUM, CRIT_TAU, TAU(NAW)
      REAL (KND), PARAMETER :: APR_SMALL = 0.05d0
      TYPE (DATA_ITER), POINTER :: RTM_PTR
c ======================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      CRIT_TAU = 0.1
      
      II = ITERATION - 1
      IF(CHK_STAT('aer-erw',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
c        ... preliminary the same as in "man_aer_setup.f"
         Nrw = 3 
         IF( Nrw > NAW ) Nrw = 2
c        ... AOD at previous iteration
         DD1 = SUM( AER_COEFF(EXI,:,1)*DZ(:) )/2.d0
         IF (LOG_LEVEL(2) >= N_RSO)
     $        WRITE(*,*) '   Tau1 - before ', DD1
         IF( II==1 ) AOD_ITER(II) = DD1
         CALL SET_PTR( RTM_PTR, 'aer-erw', ITR = ITERATION )
         DO I = 1, NLAYERS
            IF( AER_COEFF(EXI,I,Nrw) > 0.d0 ) THEN
               SSA(:) = AER_COEFF(SCI,I,:)/AER_COEFF(EXI,I,:)
            ELSE
              SSA(:) = 0.d0 
            ENDIF
c           ... Set new value of extinction an scattering coefficients
            AER_COEFF(EXI,I,:) = R_erw(I,:)*RTM_PTR%CRN(I) 
            AER_COEFF(SCI,I,:) = SSA(:)*AER_COEFF(EXI,I,:)
         ENDDO 
c        ... AOD at current iteration
         DD2 = SUM( AER_COEFF(EXI,:,1)*DZ(:) )/2.d0
         IF (LOG_LEVEL(2) >= N_RSO)
     $        WRITE(*,*) '   Tau2 - after ', DD2
         AOD_ITER(ITERATION) = DD2
         KKK= FIND_IDX('aer-erw', RET_INFO%NAME)               

c        ---------------------------------------------------------------
         IF( .NOT.SCAL_APR ) GOTO 2222
         IF( FLG_OSC ) THEN
            KK = MINLOC( AOD_ITER(1:ITERATION),1 )
            CALL SET_PTR( RTM_PTR, 'aer-erw' )
c              ... Scale a priori profile according to estimated AOD 
            IF (LOG_LEVEL(2) >= N_RSO) WRITE(*,*) 
     $           '   Scaling factor: ', 1.d0/AOD_ITER(1)*AOD_ITER(KK)
            RTM_PTR%APR(:) = RTM_PTR%APR(:)/AOD_ITER(1)*AOD_ITER(KK)
            AOD_ITER(1) = AOD_ITER(KK)
c                RTM_PTR%CRN(:) = RTM_PTR%APR(:)
            IF (LOG_LEVEL(2) >= N_RSO)  WRITE(*,*) 
     $           '  >>>>>>>>>>>>> Oscilation <<<<<<<<<<<<<<<<<'
c              PAUSE
            MAX_OSC = MAX_OSC + 3
c                 RMS_OSC = .FALSE.
            GOTO 2222
         ENDIF

         IF( DD2 <= MIN_TAU ) THEN
            RET_INFO(KKK)%R_TH%R_A = 'rel'
c            DO I = 1, SIZE( RET_INFO(KKK)%R_OE%Alt )
c               IF( RET_INFO(KKK)%R_OE%Val(I) > APR_SMALL )
c     $              RET_INFO(KKK)%R_OE%Val(I) = APR_SMALL
c            ENDDO
            CALL SET_PTR( RTM_PTR, 'aer-erw' )
c              ... Scale a priori profile according to estimated AOD    
            IF( RET_INFO(KKK)%RP == 'apr' ) THEN
               RTM_PTR%APR(:) = RTM_PTR%APR(:)/AOD_ITER(1)*DD2
               IF (LOG_LEVEL(2) >= N_RSO)  WRITE(*,*) 
     $              '  >>>>>>>>>>>>> Min AOD -apr- <<<<<<<<<<<<<'
c               PAUSE
               MIN_TAU = -1.d0
            ELSE
c              ... If from previous 
               RTM_PTR%CRN(:) = RTM_PTR%APR(:)/AOD_ITER(1)*DD2
               RTM_PTR%APR(:) = RTM_PTR%CRN(:) 
               DO I = 1, NLAYERS
c                 ... Set new value of extinction an scattering coefficients
                  AER_COEFF(EXI,I,:) = R_erw(I,:)*RTM_PTR%CRN(I) 
                  AER_COEFF(SCI,I,:) = SSA(:)*AER_COEFF(EXI,I,:)
               ENDDO
               MIN_TAU = MIN_TAU*0.75d0
               IF( MIN_TAU < 0.01 ) MIN_TAU = -1
               IF (LOG_LEVEL(2) >= N_RSO)  WRITE(*,*) 
     $              '  >>>>>>>>>>>>> Min AOD -prev- <<<<<<<<<<<<'
c               PAUSE
            ENDIF
            AOD_ITER(1) = DD2
         ENDIF
 2222    CONTINUE
c     ---------------------------------------------------------------
c         IF( SCAL_APR.AND.DD2 <= MIN_TAU ) THEN
c            IF( ABS( (DD2-DD1)/DD2 ) <= CRIT_TAU ) THEN
c            ENDIF
c         ENDIF
      ENDIF  ! option 'aer-erw'
c     ---------------------------------------------------------------------

      IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
         CALL SET_PTR( RTM_PTR, 'aer-dens', ITR = II )
         DENS_PRF(:) = RTM_PTR%NXT(:)/RTM_PTR%CRN(:)
         DO I = 1, NLAYERS
            AER_COEFF(SCI,I,:) = DENS_PRF(I)*AER_COEFF(SCI,I,:)
            AER_COEFF(EXI,I,:) = DENS_PRF(I)*AER_COEFF(EXI,I,:) 
         ENDDO 

         DO I = 1,NAW
            TAU(I) = SUM( AER_COEFF(EXI,:,I)*DZ(:) )/2.0d0
         ENDDO
         CALL LIN_SEARCH(550.d0, DUM, KK, NAW, AER_WAVS, E_RR)
c         IF(E_RR) GOTO 133
         T_550 = (1.d0 - DUM)*TAU(KK) + DUM*TAU(KK+1)
         AOD_ITER(ITERATION) = T_550
         AOD_RET(1,ITERATION) = T_550
      ENDIF  ! option 'aer-dens'

      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN
      END SUBROUTINE ST_UPDATE_MAN
c ********************************************************************

      SUBROUTINE RUN_CDI (*)
c     ... Call in "rtm_main.f"

      USE AEROSOLS,   ONLY: AER_RET_FACTOR
      USE ALB_PARS,   ONLY: ALBEDO_PERTR, ALBEDO_RET
      USE DATA_RET,   ONLY: TPS_NAME
      USE RETR_ITER,  ONLY: ITERATION
      USE ST_VARS,    ONLY: WF_MS_FLAG, GROUND_HEIGHT
c     ------------------------------------------------------------------

      INTEGER     KK, JJ, JJJ, II_1, II_2, TPS_IDX
      LOGICAL, ALLOCATABLE ::  WF_MS_FS(:)   
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'RUN_CDI'
      REAL (KND)  DELTA_TPS_COL, DELTA_ELEV, WL_ALB_S, WL_ALB_E
      REAL (KND)  D_ALBEDO(NSEGS)
c      REAL (KND), PARAMETER :: DELTA_ALBEDO = 0.3, DELTA_EXA_S = 0.5, 
c     $     DELTA_EXA_T = 0.5, DELTA_ELEV = 0.5D0
      REAL (KND), PARAMETER :: DELTA_ALBEDO = 0.1, DELTA_EXA_S = 0.1, 
     $     DELTA_EXA_T = 0.5, DELTA_TG = 0.05
!     , WL_ALB_S = 319., WL_ALB_E = 400.

      CALL ST_TRACK (R_NAME, 3, 'I')

      IF(.NOT.ALLOCATED(WF_DB)) THEN ! Intensity mode
         CALL CALL_TO_CDI (1, NSPEC)
         CALL ST_TRACK (R_NAME, 3, 'O')
         RETURN
      END IF

!     This is done in rtm_output_wfs
!      IF(WF_FLAG) THEN
!         WF_TG_FLAG = INQ_WF('tg', 'WFT')
!      ELSE 
!         WF_TG_FLAG = .FALSE.
!      END IF
         
      IF(ITERATION == 1) THEN
         DELTA_ELEV = 3.0D0
      ELSE
         DELTA_ELEV = 1.0D0
      END IF
      IF (GROUND_HEIGHT > 4.0D0) THEN 
         DELTA_ELEV = -DELTA_ELEV
      END IF

      II_1 = 1
      II_2 = NSPEC
         
      DO II = 1, UBOUND(WF_DB,1)
         SELECT CASE (WF_DB(II)%NAME)
         CASE ('tps_col', 'albedo', 'exa_s', 'elev') 
            ALLOCATE(WF_MS_FS(NSPEC))
            EXIT
         END SELECT
      ENDDO
       
      DO II = 1, UBOUND(WF_DB,1)
         SELECT CASE (WF_DB(II)%NAME)
         CASE ('tps_col', 'albedo', 'exa_s', 'elev') 
            WF_MS_FS = WF_MS_FLAG ;  WF_MS_FLAG = .FALSE.
            WF_FLAG = .FALSE.
            WF_TG_FLAG_LOC = WF_TG_FLAG
            WF_TG_FLAG = .FALSE.
         END SELECT

         SELECT CASE (WF_DB(II)%NAME)
         CASE DEFAULT
            CYCLE
         CASE ('tps_col') 
            JJ = COUNT((ZZZ(:) > TP_HEIGHT)) + 1 
            DELTA_TPS_COL = - 0.1D0
            TPS_IDX = FIND_IDX(TPS_NAME, TG_INFO%NAME)
            TG_INFO(TPS_IDX)%ND(JJ:NLAYERS) = 
     $           TG_INFO(TPS_IDX)%ND(JJ:NLAYERS)
     $           *(1.0D0 + DELTA_TPS_COL)

            CALL CALL_TO_CDI (1, NSPEC)
            
            TG_INFO(TPS_IDX)%ND(JJ:NLAYERS) = 
     $           TG_INFO(TPS_IDX)%ND(JJ:NLAYERS)/
     $           (1.0D0 + DELTA_TPS_COL)
         CASE ('albedo') 
            IF(ITERATION == 1) ALBEDO_RET(3,:) = ALBEDO_CONST
            DO KK = 1, NSEGS
               IF(ALBEDO_RET(3,KK)  > 0.5) THEN
                  ALBEDO_PERTR(KK) = - DELTA_ALBEDO
               ELSE
                  ALBEDO_PERTR(KK) = DELTA_ALBEDO
               ENDIF
            END DO

            KK = FIND_IDX('albedo', RET_INFO%NAME)
c            IF(RET_INFO(KK)%WL_SEL) THEN
c               WL_ALB_S = MINVAL(RET_INFO(KK)%WL_MIN(:))
c               WL_ALB_S = MAX(WL_ALB_S, 315.D0)
c               WL_ALB_E = MAXVAL(RET_INFO(KK)%WL_MAX(:))
c               II_1 = COUNT(WL(:) < WL_ALB_S) + 1 
c               II_2 = COUNT(WL(:) <= WL_ALB_E)
c            ELSE
               II_1  = 1 ;  II_2  = NSPEC
c               WL_ALB_S = 1 ;  WL_ALB_E = NSPEC
c     END IF
            IF(PRESET_APR_NAME == 'O3'.AND..NOT.NEW_WL_GRID) THEN
               ALLOCATE (MASK_TMP(NSPEC, N_LOS_OUT), SOURCE = MASK)
               MASK = MDS(2)%MSK_RTM 
               CALL CALL_TO_CDI (II_1, II_2)
               MASK = MASK_TMP
               DEALLOCATE(MASK_TMP)
            ELSE
               CALL CALL_TO_CDI (II_1, II_2)
            END IF
            D_ALBEDO = ALBEDO_PERTR ;  ALBEDO_PERTR = 0.0D0
         CASE ('exa_s')

            JJ = COUNT((ZZZ(:) > TP_HEIGHT))
            JJJ = COUNT((ZZZ(:) >= 40.0D0)) 
            JJ = MAX(JJ,1)
            JJJ = MAX(JJJ,1)
            AER_RET_FACTOR(:,JJJ:JJ) = 
     $           AER_RET_FACTOR(:,JJJ:JJ)*(1.0D0 +  DELTA_EXA_S)

!            JJ = COUNT((ZZZ(:) > TP_HEIGHT)) + 1 
!!            AER_RET_FACTOR(:,1:JJ - 1) = 
!!     $           AER_RET_FACTOR(:,1:JJ - 1)*(1.0D0 +  DELTA_EXA_S)           
!            AER_RET_FACTOR(:,:) =
!     $           AER_RET_FACTOR(:,:)*(1.0D0 +  DELTA_EXA_S) 

            CALL CALL_TO_CDI (1, NSPEC)

            AER_RET_FACTOR(:,JJJ:JJ) = 
     $           AER_RET_FACTOR(:,JJJ:JJ)/(1.0D0 +  DELTA_EXA_S)
            
!!            AER_RET_FACTOR(:,1:JJ - 1) = 
!!     $           AER_RET_FACTOR(:,1:JJ - 1)/(1.0D0 +  DELTA_EXA_S)
!            AER_RET_FACTOR(:,:) =
!     $           AER_RET_FACTOR(:,:)/(1.0D0 +  DELTA_EXA_S)

         CASE ('elev')
            GROUND_HEIGHT = GROUND_HEIGHT + DELTA_ELEV
            CALL GT_LAYER_SETUP (*880)         
            CALL WRT_ALT (ZZZ)
            CALL RTM_INIT (.FALSE., .TRUE., .FALSE., .FALSE., 
     $           WL_START, WL_END, NSPEC, NSPEC_OUT, WL, SRR, *880)

            CALL CALL_TO_CDI (1, NSPEC)
            
            GROUND_HEIGHT = GROUND_HEIGHT - DELTA_ELEV
            WF_MS_FLAG = WF_MS_FS ;  WF_FLAG = .TRUE.
            WF_TG_FLAG = WF_TG_FLAG_LOC
            CALL GT_LAYER_SETUP (*880)         
            CALL WRT_ALT (ZZZ)
            CALL RTM_INIT (.FALSE., .TRUE., .FALSE., .FALSE., 
     $           WL_START, WL_END, NSPEC, NSPEC_OUT, WL, SRR, *880)
         END SELECT


         WF_DB(II)%WF(:,:,1,1) = SPEC_OUT(:,1,:)
         SPEC_OUT = 0.0D0

         SELECT CASE (WF_DB(II)%NAME)
         CASE ('tps_col', 'albedo', 'exa_s') 
            WF_MS_FLAG = WF_MS_FS
            WF_FLAG = .TRUE.
            WF_TG_FLAG = WF_TG_FLAG_LOC
         END SELECT

      END DO

      DO II = 1, UBOUND(WF_DB,1)
         SELECT CASE (WF_DB(II)%NAME)
         CASE ('tps_col', 'albedo', 'exa_s', 'elev') 
            DEALLOCATE(WF_MS_FS)
            EXIT
         END SELECT
      ENDDO

      IF( NUM_WF ) THEN
         PERTURB = 1.d0 ; WF_TG_FLAG = .False.
         IF( NUM_AER.and.SCA_or_ABS == "aerosol extinction") Then
            II = FIND_IDX('aer-ext', WF_DB%Name)
         Else
            II = FIND_IDX('tg', WF_DB%TYPE)
         End If
         DO KK = 1, NLAYERS
            IF(ZZZ(KK) > MAX_WF_Z ) CYCLE
            IF(ZZZ(KK) < MIN_WF_Z ) EXIT
            PERTURB(KK) = 1.d0 + DELTA_TG
            CALL CALL_TO_CDI (1, NSPEC)
            WF_DB(II)%WF(:,:,KK,1) = SPEC_OUT(:,1,:)
            SPEC_OUT = 0.0D0
            PERTURB(KK) = 1.d0
         END DO
      END IF

      CALL CALL_TO_CDI (1, NSPEC)
       
      DO II = 1, UBOUND(WF_DB,1)
         SELECT CASE (WF_DB(II)%NAME)
         CASE ('albedo')
            IF(RET_INFO(II)%WL_SEL) THEN
               WL_ALB_S = MINVAL(RET_INFO(II)%WL_MIN(:))
               WL_ALB_E = MAXVAL(RET_INFO(II)%WL_MAX(:))
               II_1 = COUNT(WL(:) < WL_ALB_S) + 1 
               II_2 = COUNT(WL(:) <= WL_ALB_E)
            ELSE
	      II_1 = 1 ;  II_2 = NSPEC
            END IF
            
            DO KK = II_1, II_2
               WF_DB(II)%WF(KK,:,1,1)  = (WF_DB(II)%WF(KK,:,1,1) - 
     $              SPEC_OUT(KK,1,:))/D_ALBEDO(SEG_NUM(KK))
            END DO
            IF(WF_DB(II)%DIM == 'rel') THEN
               DO JPOS = II_1, II_2
                  WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1)*
     $                 ALBEDO_RET(3,SEG_NUM(JPOS))
               END DO
            END IF
         CASE ('exa_s')
            WF_DB(II)%WF(:,:,1,1)  = 
     $           (WF_DB(II)%WF(:,:,1,1) - SPEC_OUT(:,1,:))/DELTA_EXA_S
            IF(WF_DB(II)%DIM == 'abs') STOP 'Not implemented!'
         CASE ('tps_col')
            WF_DB(II)%WF(:,:,1,1)  = 
     $           (WF_DB(II)%WF(:,:,1,1) - SPEC_OUT(:,1,:))/DELTA_TPS_COL
            IF(WF_DB(II)%DIM == 'abs') STOP 'Not implemented!'
         CASE ('elev')
            WF_DB(II)%WF(:,:,1,1)  =
     $           (WF_DB(II)%WF(:,:,1,1) - SPEC_OUT(:,1,:))/DELTA_ELEV
            IF(WF_DB(II)%DIM == 'rel') STOP 'Not implemented!'
         END SELECT
      END DO
      
      IF( NUM_WF ) THEN
         DO II = 1, UBOUND(WF_DB,1)
            IF(WF_DB(II)%TYPE /= 'tg'.and.
     $           SCA_or_ABS /= "aerosol extinction") CYCLE
            DO KK = 1, NLAYERS
               IF(ZZZ(KK) > MAX_WF_Z ) CYCLE
               IF(ZZZ(KK) < MIN_WF_Z ) EXIT
               WF_DB(II)%WF(:,:,KK,1)  =
     $              (WF_DB(II)%WF(:,:,KK,1) - SPEC_OUT(:,1,:))/DELTA_TG
            END DO
         END DO
      END IF

      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END SUBROUTINE RUN_CDI
c***********************************************************************
      SUBROUTINE CALL_TO_CDI (II_1, II_2)
      INTEGER II_1, II_2

      ERROR = .FALSE.
!$OMP PARALLEL DO PRIVATE(JPOS) SCHEDULE(DYNAMIC) IF(PE_Mode == 'WL')
        DO JPOS = II_1, II_2      ! Wavelenght loop
           IF(ALL(.NOT.MASK(JPOS,:))) CYCLE
           CALL SPHER_RAD (JPOS, WL(JPOS), MASK(JPOS,:))
        END DO
        RETURN
      END SUBROUTINE CALL_TO_CDI
c***********************************************************************

      SUBROUTINE RUN_DOM_ASYMP
c     ... Call in "rtm_main.f"

      USE GRIDS,       ONLY: MAX_LINE
      USE INTERN_DIMS, ONLY: Nvs
      USE RETR_ITER,   ONLY: ITERATION
      USE RTM_OUTPUT,  ONLY: DEL_RRS, DEL_VRS
      USE ST_VARS,     ONLY: DO_AEROSOL, RET_MODE, RTM_MODE
      USE TOOLS,       ONLY: INQ_WF
c     ------------------------------------------------------------------

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RUN_DOM_ASYMP'
      REAL (KND)  Int_0(N_LOS_OUT,Nvs)
      REAL (KND), ALLOCATABLE :: IR_RAD(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 3, 'I')

c      IF( TRIM(RET_MODE)=='aer-bim'.AND.ITERATION > 1) THEN
c      IF( RTM_MODE =='ret'.AND.ITERATION > 1) THEN
      IF( RTM_MODE =='wf'.AND.ITERATION > 1) THEN
         SELECT CASE( TRIM(RET_MODE) )
         CASE('aer-bim','aer-hom','aer-ratio')
            CALL ST_UPDATE_MIE

         CASE DEFAULT
            SELECT CASE (AER_PAR_TYPE(1:2))
            CASE ('mn','wm','lt','st','pa')
               CALL ST_UPDATE_MAN
            CASE('mi','mo')
               IF(      INQ_WF('aer-modr', 'RIN')
     $              .OR.INQ_WF('aer-var', 'RIN')
     $              .OR. INQ_WF('aer-dens', 'RIN')
     $              .OR. INQ_WF('aer-cm_frac', 'RIN') )
     $              CALL ST_UPDATE_MIE
            END SELECT

         END SELECT
      ENDIF  ! RTM_MODE =='wf'.AND.ITERATION > 1

      ALLOCATE( SVR%USE_CONV(NSPEC) )

      IF( NUM_WF ) THEN
         SELECT CASE (RTM_TYPE)
         CASE('pp_scat')
            ALLOCATE( AL_BASE(NLAYERS), AL_1_BASE(NLAYERS), ! A
     $           AL_1_PERT(NLAYERS) ) ! A
         CASE DEFAULT
            ALLOCATE( AL_BASE(NLAYERS), AL_1_BASE(MAX_LINE), ! A
     $           AL_1_PERT(MAX_LINE) ) ! A
         END SELECT

         WRITE(*,*) 
         WRITE(*,*) 
     $        '*****************************************************'
         WRITE(*,*) 
     $        '---> You start a specific SCIATRAN mode which is used'
         WRITE(*,*) 
     $        '     for numerical calculation of weighting function '
         WRITE(*,*) 
     $        '     employing the numerical perturbation technique. '
         IF( NUM_OWF ) THEN
            WRITE(*,*) 
     $        '       Perturbation will be performed for ocean  '
            WRITE(*,*) 
     $        '                    parameters.                     '
         ELSE
            WRITE(*,*) 
     $        '     Perturbation will be performed for atmosphere  '
            WRITE(*,*) 
     $        '                    parameters.                     '
         ENDIF
         WRITE(*,*) 
     $        '---> Enter perturbation parameter :                 '
         WRITE(*,*) 
     $        '     absolute perturbation for temperature (Del T)  '
         WRITE(*,*)
     $        '     relative perturbation for other parameters, '
         WRITE(*,*)
     $        '                             i.e., (Del P)/P in [%] '
         WRITE(*,*) 
     $        '*****************************************************'
         READ(*,*) DELTA
c         DELTA = 0.1d0
         IF( .NOT.NUM_TEM ) DELTA = DELTA/100.d0
         
         IF( NUM_OWF ) THEN 
            N_last = NLW ; Z_LOC => ZZZ_UW(:)
         ELSE
            N_last = NLAYERS ; Z_LOC => ZZZ(:)
         ENDIF
      ELSE
         IF(.NOT.ALLOCATED(AL_BASE)) 
     $        ALLOCATE(AL_BASE(1), AL_1_BASE(1), AL_1_PERT(1)) 
         N_last = 0
      ENDIF
      
      KK = 1
      DO Jper = 0, N_last       ! for the numerical perturbation only
         IF( NUM_WF ) THEN
            IF(Jper > 0 ) THEN
               IF(Z_LOC(KK) > MAX_WF_Z ) THEN 
                  KK = KK + 1  ;  CYCLE
               ENDIF
               IF(Z_LOC(KK) < MIN_WF_Z ) EXIT
               KK = KK + 1
            ENDIF
         ENDIF  ! NUM_WF

         IF( NUM_TEM ) THEN
            ! ... Additive perturbation:
            PERTURB(:) = 0.d0
            IF( Jper > 0 ) THEN
               PERTURB(Jper) = PERTURB(Jper) + DELTA 
            ENDIF
         ELSE 
            ! ... Multiplicative perturbation:
            PERTURB(:) = 1.d0
            IF( Jper > 0 ) THEN
               PERTURB(Jper) = PERTURB(Jper)*(1.d0 + DELTA)
            ENDIF
         ENDIF

!          $OMP PARALLEL DO PRIVATE(JPOS) SCHEDULE(DYNAMIC) IF(PE_Mode == 'WL')

         SELECT CASE( TRIM(INEL_WHERE) )
c        ... Vibrational Raman scattering and fluorescence in ocean
c        ----------------------------------------------------------
         CASE('In water')
            SVR%USE_CONV(:) = SVR%ACT_WL(:)
c           ... Discrete-ordinates solution of inelastic scattering
            IF( INEL_DO ) THEN
               SVR%Jexi = 0 ; SVR%Jact = 0
c              ... Do-loop over all wavelengths extended WL-grid
               DO JPOS = 1,SVR%N_EXT
                  IF( SVR%Act_WL(JPOS) ) SVR%Jact = SVR%Jact + 1
                  IF( SVR%Exi_WL(JPOS) ) THEN
                     SVR%Jexi = SVR%Jexi + 1
                     SVR%Isav(JPOS) = SVR%Jexi
                  ENDIF
                  CALL DOM_SPHERICAL(JPOS, WL(JPOS), SRR, SVR)
               ENDDO  
            ENDIF

c           ... Forward-adjoint technique for inelastic scattering
            IF( INEL_ADJ ) THEN
               SVR%Jact = 0
c              ... Over all wavelengths extended wl-grid
               DO JPOS = 1,SVR%N_EXT
c                 ... Skip all excitation wavelengths
                  IF( SVR%Exi_WL(JPOS) ) CYCLE
c                 ...  Count actual wavelengths
                  SVR%Jact = SVR%Jact + 1
                  CALL DOM_SPHERICAL(JPOS, WL(JPOS), SRR, SVR)
               ENDDO  
c              ... Switch off the solution of the adjoint equation
               SOLVER = 'Solv_SFI'  ;  N_RHS  = NS_LOC

               SVR%Jexi = 0 
c              ... Over all wavelengths extended wl-grid
               DO JPOS = 1,SVR%N_EXT
c                 ... Skip all actual wavelengths
                  IF( SVR%ACT_WL(JPOS) ) CYCLE
c                 ...  Count excitation wavelengths
                  SVR%Jexi = SVR%Jexi + 1
                  CALL DOM_SPHERICAL(JPOS, WL(JPOS), SRR, SVR)
               ENDDO   
            ENDIF

c        ... Rotational Raman in atmosphere and inelastic
c            processes in ocean (VRS, Fluorescence)
c        ------------------------------------------------
         CASE('In air+water')
c           ... Convergence check will be performed for actual 
c               wavelengths only
            SVR%USE_CONV(:) = SVR%ACT_WL(:)
c           ... Discrete-ordinates solution of inelastic scattering 
c               is implemented only
            IF( INEL_DO ) THEN
               SVR%Jact = 0 ;  SRR%Jact = 0
               SVR%Jexi = 0 ;  SRR%Jexi = 0
               SVR%Isav(:) = -99
               ALLOCATE( IR_RAD(N_ACT) )
               IR_RAD(:) = PACK(SOL_FLUX(:),AWF(:))
                  WRITE(*,*) '  N_act           = ',N_act
                  WRITE(*,*) '  NSPEC_RTM       = ',NSPEC_RTM
                  WRITE(*,*) '  NSPEC_OUT       = ',NSPEC_OUT
                  WRITE(*,*) '  SHAPE(SOL_FLUX)   ', SHAPE(SOL_FLUX)
                  WRITE(*,*) '       SHAPE(AWF)   ', SHAPE(AWF)
c                  WRITE(*,*) IR_RAD(:)
c                  STOP
               DO I = 1,SVR%N_EXT
c                 ... Skip all actual wavelengths
                  IF( SVR%Act_WL(I) ) CYCLE
c                 ... Skip empty RRS bins which do not pertain to
c                     VRS excitation wavelengths
                  IF(.NOT.SVR%Exi_WL(I).AND.SRR%NUM_IN(I)==0) CYCLE

c                 ... SVR%Jexi counts VRS excitation WLs
                  IF( SVR%Exi_WL(I) ) THEN
                     SVR%Jexi    = SVR%Jexi + 1
                     SVR%Isav(I) = SVR%Jexi
                  ENDIF

c                 ... SRR%Jexi counts RRS excitation WLs
                  IF( SRR%Exi_WL(I) ) THEN 
                     SRR%Jexi = SRR%Jexi + 1
c                    ... Save corresponding wavelength number of the 
c                        extended WL-grid where calculation is performed
                     SRR%Npo_sav(SRR%Jexi) = I
c                     WRITE(*,*) ' Save RRS  ',I, SRR%Jexi, SRR%NUM_IN(I)
                  ENDIF
c                 ... Start RTM solution
                  CALL DOM_SPHERICAL(I, WL(I), SRR, SVR)
               ENDDO  

c              ... Full number of RRS excitation wavelengths 
c                  where calculations were performed and saved 
               SRR%N_sav = SRR%Jexi
c               WRITE(*,*) '   rtm_main SRR%N_sav = ', SRR%N_sav
c               STOP

               DO JPOS = 1,SVR%N_EXT
c                 ... Skip all excitation wavelengths
                  IF( .NOT.SVR%Act_WL(JPOS) ) CYCLE
                  SVR%Jact = SVR%Jact + 1
                  SRR%Jact = SRR%Jact + 1
c                 ... Start RTM solution
                  CALL DOM_SPHERICAL(JPOS, WL(JPOS), SRR, SVR)
               ENDDO  ! over all actual wavelengths (JPOS)
            ENDIF

c        ... Rotational Raman scattering in atmosphere
c        ---------------------------------------------
         CASE('In air')
            DEL_RRS = 0.d0 ; SVR%USE_CONV(:) = SRR%Act_WL(:)

            IF( RRS_ADJ ) THEN  ! Forward-adjoint technique for RRS
c              ====================================================
c              ... First run over all actual wavelengths and after 
c                  that over excitation ones.
               SELECT CASE( TRIM(APPR_RRS) )
               CASE('exact matching')  ;  CALL RRS_ADJ_EM
               CASE('simple binning')  ;  CALL RRS_ADJ_SB
               END SELECT
c              ... DEL_RRS is calculated already in absolute physical 
c                  units, therefore, we divide here DEL_RRS by solar 
c                  irradiance because in "st_convol.f" all arrays will be 
c                  multiplied by this irradiance and only then convoluted.
               IF( NEW_SPEC_GRID ) THEN
                  ALLOCATE( IR_RAD(N_ACT) )
                  IR_RAD(:) = PACK(SOL_FLUX(:),AWF(:))
                  DO I = 1,N_ACT
                     DEL_RRS(I,:,:) = DEL_RRS(I,:,:)/IR_RAD(I)
                  ENDDO
               ENDIF
            ELSE             ! Discrete-ordinates technique for RRS
c              ====================================================
c              ... First run over all excitation wavelengths and after 
c                  that over actual ones.
               SELECT CASE( TRIM(APPR_RRS) )
               CASE('exact matching')  ;  CALL RRS_DO_EM
               CASE('simple binning')  ;  CALL RRS_DO_SB
               END SELECT
            ENDIF

c        ... No inelastic processes included   
c        -----------------------------------
         CASE('Nowhere')
            SVR%USE_CONV(:) = .TRUE.
!$OMP PARALLEL DO PRIVATE(JPOS) SCHEDULE(DYNAMIC) IF(PE_Mode == 'WL')
            DO JPOS = 1,NSPEC
               CALL DOM_SPHERICAL(JPOS, WL(JPOS), SRR, SVR, JPOS)
            ENDDO  
c     WRITE(*,*) '      Intensity  :', STOK_OUT(1,1,1,1,:)

         END SELECT

         IF(LOG_LEVEL(1) >= N_ESO.AND..NOT.AZ_FLAG) THEN
            WRITE(LUNIT,'(A)') '  *** Azimuthal series flag not set'
         ENDIF
            
         IF( NUM_WF ) THEN
            IF(Jper > 0) THEN
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' Perturbed layer :', Z_LOC(Jper)
               WRITE(*,'(A,*(2x,es15.7))')
     $                    '  Intensity mean :', Int_0(1,:)
               WRITE(*,'(A,*(2x,es15.7))')
     $                    '      Intensity  :', STOK_OUT(1,1,1,1,:)
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
               WRITE(*,*) ' *******************************************'
            ENDIF
            IF(Jper == 0) THEN 
               Int_0(:,:) = STOK_OUT(1,1,1,:,:)
               STOK_OUT = 0.d0
            ELSE
               IF( NUM_OWF ) THEN 
                  OWF_OUT_VEC(1,:,Jper,1,1:Nvs) = 
     $                 ( STOK_OUT(1,1,1,:,:) - Int_0(:,:))/Delta
               ELSE
                  WF_DB(1)%WF(1,:,Jper,1:Nvs) = 
     $                 ( STOK_OUT(1,1,1,:,:) - Int_0(:,:))/Delta
               ENDIF
              STOK_OUT = 0.d0
            ENDIF
         ENDIF  ! NUM_WF
         IF( NUM_PRE ) THEN 
            PRESS(:) = PRESS(:)/PERTURB(:)
            AIR_DENSITY(:) = AIR_DENSITY(:)/PERTURB(:)
         ENDIF  ! NUM_PRE
         IF( NUM_TEM ) THEN
            TEMP(:) = TEMP(:) - PERTURB(:)
            AIR_DENSITY(:) = AIR_DENSITY(:) - 
     $           (-AIR_DENSITY(:)/Temp(:))*PERTURB(:)
         ENDIF  ! NUM_TEM
      ENDDO  ! numerical perturbation (Jper)
      
      CALL ST_TRACK (R_NAME, 3, 'O')
      RETURN
      END SUBROUTINE RUN_DOM_ASYMP
c********************************************************************

      SUBROUTINE RRS_DO_EM
      INTEGER Ia, Jr
c       ... Over actual wavelengths of the extended WL-grid
        DO Ia = 1,N_ACT
c          ... Jr defines jump to the next actual wavelength
           Jr = (Ia-1)*(N_WL_C+1) 
c          ... WL_ACT contains values of all actual wavelengths
           SRR%WL4 = WL_ACT(Ia)**4.d0
c          ... Do-loop over excitation wavelength, skipping
c              first actual one in the sequence N_WL_C+1
           SRR%Jact = 0  ;  SRR%Jexi = 0               
           DO I = 2,N_WL_C+1
c             ... K is the position of I-th excitation wavelength
c                 in the extended WL-grid
              K = Jr + I
cc             ... SRR%Jact counts excitation wavelengths
c              SRR%Jact = SRR%Jact + 1
c             ... SRR%Jexi counts excitation wavelengths
              SRR%Jexi = SRR%Jexi + 1
              CALL DOM_SPHERICAL(K, WL(K), SRR, SVR)
           ENDDO  ! over excitation wavelengths (I)
c          ... SRR%Jact is the actual wavelength number
           SRR%Jact = Ia
c          ... K is the position of Ia-th actual wavelength
c              in the extended WL-grid
           K = Jr + 1
           CALL DOM_SPHERICAL(K, WL(K), SRR, SVR)
        ENDDO  ! over actual wavelengths (Ia)

        RETURN
      END SUBROUTINE RRS_DO_EM

c **********************************************************************
c         Rotational Raman scattering using discrete-ordinates
c            technique within "simple binning" mode
c **********************************************************************
      SUBROUTINE RRS_DO_SB
      INTEGER I
      LOGICAL Act_WL_SAV(SRR%N_EXT)
      
      Act_WL_SAV(:) = SRR%Act_WL(:)

      SRR%Act_WL(:)   = .FALSE. ; SRR%Exi_WL(:) = .TRUE.
      SVR%USE_CONV(:) = .FALSE.

      SRR%Jexi = 0 
c     ... Do-loop over extended WL-grid saving all needed results 
c         for not empty bins
      DO I = 1, SRR%N_EXT
c        ... Skip "empty" bins
         IF( SRR%NUM_IN(I)==0 ) CYCLE
c        ... SRR%Jexi counts calculated excitation wavelengths
         SRR%Jexi = SRR%Jexi + 1
c        ... Save corresponding wavelength number of the extended
c            WL-grid where calculation is performed
         SRR%Npo_sav(SRR%Jexi) = I
c         WRITE(*,*) ' Save RRS  ',  I, SRR%Jexi, SRR%NUM_IN(I)
c        ... Start RT calculations
         CALL DOM_SPHERICAL(I, WL(I), SRR, SVR)
      ENDDO  
c     ... Full number of wavelengths in extended WL-grid
c         where calculations were performed and saved
      SRR%N_sav = SRR%Jexi
c      WRITE(*,*) '  Number of saved WL ', SRR%N_sav

c     ... Restore marks for actual and excitation wavelengths
      SRR%Act_WL(:)   = Act_WL_SAV(:) 
      SRR%Exi_WL(:)   = .NOT.SRR%Act_WL(:)
      SVR%USE_CONV(:) = ACT_WL_SAV(:)

c     ... Do-loop over actual wavelengths of the extended WL-grid
      SRR%Jact = 0                  
      DO I = 1, SRR%N_EXT
c        ... Skip excitation wavelengths
         IF( SRR%Exi_WL(I) ) CYCLE
c        ... SRR%Jact counts calculated actual wavelengths
         SRR%Jact = SRR%Jact + 1
         CALL DOM_SPHERICAL(I, WL(I), SRR, SVR)
      ENDDO  

      RETURN
      END SUBROUTINE RRS_DO_SB

c **********************************************************************
c    Saving of solutions for actual wavelengths and results of linear 
c        perturbation will be performed in routine "st_pp_rrs.f"
c **********************************************************************
      SUBROUTINE RRS_ADJ_EM
      INTEGER Ia, Jr
c       ... Over actual wavelengths of extended WL-grid
        DO Ia = 1,N_ACT
c          ... Jr defines jump to the next actual wavelength
           Jr = (Ia-1)*(N_WL_C+1) 
c          ... WL_Act contains values of all actual wavelengths
           SRR%WL4 = WL_Act(Ia)**4.d0
c          ... Counts actual wavelengths
           SRR%Jact = Ia
c          ... For the next actual wavelength do-loop over all 
c              corresponding excitation wavelength, starting 
c              from an actual one.
           DO I = 1,N_WL_C+1
              K = Jr + I
              CALL DOM_SPHERICAL(K, WL(K), SRR, SVR)
           ENDDO
        ENDDO  
        RETURN
      END SUBROUTINE RRS_ADJ_EM

c **********************************************************************
c    Saving of solutions for actual wavelengths and results of linear 
c        perturbation will be performed in routine "st_pp_rrs.f"
c **********************************************************************
      SUBROUTINE RRS_ADJ_SB
      INTEGER Ie
c       ... Using forward-adjoint technique, first run 
c           should be done over all actual wavelengths
c                          ---------------------------
        SRR%Jact = 0
c       ... Do-loop over all wavelengths of extended WL-grid
        DO Ie = 1,SRR%N_EXT                  
c          ... Skip excitation wavelengths
           IF( SRR%Exi_WL(Ie) ) CYCLE
c          ... SRR%Jact counts the actual wavelengths
           SRR%Jact = SRR%Jact + 1
           CALL DOM_SPHERICAL(Ie, WL(Ie), SRR, SVR)
        ENDDO
        
c       ... Switch off the solution of the adjoint equation
        SOLVER = 'Solv_SFI'  ;  N_RHS  = NS_LOC

c       ... Tradeoff speed/memory
c       --------------------------------
        SELECT CASE( TRIM(TSM) )
        CASE('memory')
           SRR%Act_WL   = .FALSE. ; SRR%Exi_WL(:) = .TRUE.
           SVR%USE_CONV = .FALSE.
c          ... Do-loop over all wavelengths of extended WL-grid
           DO Ie = 1,SRR%N_EXT
c             ... Skip "empty" bins. The calculation for actual 
c                 wavelengths will be repeated
              IF( SRR%NUM_IN(Ie)==0 ) CYCLE
              CALL DOM_SPHERICAL(Ie, WL(Ie), SRR, SVR)
           ENDDO
c     --------------------------------
        CASE('speed')
c          ... Do-loop over all wavelengths of extended WL-grid
           DO Ie = 1,SRR%N_EXT
c             ... Skip actual wavelengths and "empty" bins. 
c                 The calculation for actual wavelengths 
c                 will be not repeated.
              IF( SRR%Act_WL(Ie).OR.SRR%NUM_IN(Ie)==0 ) CYCLE
              CALL DOM_SPHERICAL(Ie, WL(Ie), SRR, SVR)
           ENDDO
        END SELECT
        RETURN
      END SUBROUTINE RRS_ADJ_SB

      END
C********************************************************************
