!********************************************************************C
!
!	SCIATRAN - Version 4.7 - Last modification: 03 Dec 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!********************************************************************C
SUBROUTINE RTM_OUTPUT_ARRAYS(NSPEC, MAX_SPEC, NLAYERS, *)
  !     ... Call in "rtm_init.f"

  USE ABSORBERS,   ONLY: N_GAS
  USE AEROSOLS,    ONLY: N_AOT
  USE INTERN_DIMS, ONLY: Nvs
  USE LOG_PARS,    ONLY: EU
  USE RTM_OUTPUT,  ONLY: RTM_OUT_ARR, N_OP, N_VOD, VERT_COL, NL_OUT, N_LOS, &
                         N_RTM_ARR, WF_TO_AMF, NU_VEC_PARS, NR_UWT, N_CHL, NP_Us, &
                         NP_Ks, NP_Kl, OWF_OUT_VEC, OWF_LABEL_V, ALB_OUT, ALB_WL, &
                         TRANS_OUT
  USE ST_VARS,     ONLY: RTM_MODE, WF_FLAG, DO_PC, RTM_CORE, OWF_FLAG, OWF_Ray, &
                         OWF_Chl, OWF_Use, OWF_Ks, OWF_Kl, RAD_FIELD, I_LAY
  USE SPHER_PARS,  ONLY: N_LOS_OUT, N_SOL_ANG
  USE WATER,       ONLY: UWT, NLW => NLAYERS_UW, OUT_WLR, INEL_ADJ
  USE WF_NUM,      ONLY: NUM_WF, NUM_OWF
  !------------------------------------------------------------------

  INTEGER, INTENT(IN) :: NSPEC, MAX_SPEC, NLAYERS
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RTM_OUTPUT_ARRAYS'
  INTEGER   I, K, II
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  CALL DEALLOCATE_ARRAYS
  I_LAY = 0
  SELECT CASE (TRIM(RTM_MODE))
  CASE ('asymp' )  
     N_RTM_ARR = 5 ;  NL_OUT = 1; N_LOS = N_LOS_OUT

  CASE ('int')  
     NL_OUT = N_OP ; N_LOS = N_LOS_OUT         
     ! ... Preliminary it is assumed that VRS using forward-adjoint
     !     technique for scalar case only
     IF( INEL_ADJ ) THEN
        N_RTM_ARR = 2
     ELSE
        N_RTM_ARR = Nvs
     ENDIF
     ! ... Additional dimension is used for water-leaving radiance
     IF( OUT_WLR ) NL_OUT = NL_OUT + 1
     N_RTM_ARR = N_RTM_ARR + 2

  CASE ('wf','num_pert')  
     ! ... Set to 1 because N_OP=1 if 'wf' is used
     NL_OUT = 1; N_LOS = N_LOS_OUT ; N_RTM_ARR = Nvs            
     ! ... Additional dimension is used for water-leaving radiance
     IF( OUT_WLR ) NL_OUT = NL_OUT + 1
     ! ... Preliminary for arrays Tra_mu0 and Tra_muv
     IF( TRANS_OUT ) N_RTM_ARR = N_RTM_ARR + 2

  CASE ('block_amf')  
     N_RTM_ARR = Nvs ;  NL_OUT = 1; N_LOS = N_LOS_OUT
     ALLOCATE(WF_TO_AMF(NSPEC,NLAYERS))

  CASE ('amf', 'slant_col')  
     N_RTM_ARR = Nvs + 2 ;  NL_OUT = 1; N_LOS = N_LOS_OUT
     ALLOCATE(VERT_COL(1:N_SOL_ANG)) ! A
     VERT_COL(1:N_SOL_ANG) = 0.d0

  CASE ('flux') 
     NL_OUT = NLAYERS;  IF( UWT ) NL_OUT = NL_OUT + NLW
     N_LOS = N_SOL_ANG
     N_RTM_ARR = 8
     
  CASE ('int_flux')
     NL_OUT = NLAYERS; IF( UWT ) NL_OUT = NL_OUT + NLW
     I_LAY = NL_OUT
     NL_OUT = NL_OUT + N_OP
     IF( OUT_WLR ) NL_OUT = NL_OUT + 1
     N_LOS = MAX0(N_SOL_ANG,N_LOS_OUT)
     N_RTM_ARR = 8
     ! Below additional dimensions from intensity 
     N_RTM_ARR = N_RTM_ARR + Nvs
     N_RTM_ARR = N_RTM_ARR + 2
     
  CASE ('spher_alb')
     IF( RAD_FIELD ) THEN ! the same as 'flux' in spherical geometry
        N_RTM_ARR = 8 ;  NL_OUT = NLAYERS; N_LOS = N_SOL_ANG
     ELSE
        N_RTM_ARR = 2 ;  NL_OUT = NLAYERS; N_LOS = 1
     ENDIF
     IF( UWT ) NL_OUT = NL_OUT + NLW
     
  CASE ('vod')
     NL_OUT = 1
     N_LOS = N_SOL_ANG
     N_VOD = N_GAS + 6
     N_RTM_ARR = N_VOD + N_GAS

  CASE ('raman')
     N_RTM_ARR = 2*Nvs ;  NL_OUT = 1 ; N_LOS = N_LOS_OUT
!     IF( UWT ) NL_OUT = NL_OUT + NLW
     ! ... Correction to include water-leaving output (VR 03.12.23)
     IF( UWT ) THEN
        NL_OUT = NL_OUT + NLW
        IF( OUT_WLR ) NL_OUT = NL_OUT + 1
     END IF
  CASE DEFAULT
     GOTO 200
     
  END SELECT

  IF( ALB_OUT ) ALLOCATE( ALB_WL(NSPEC) )
  ALLOCATE(RTM_OUT_ARR(MAX_SPEC,N_AOT+1,NL_OUT,N_LOS,N_RTM_ARR)) ! A
  RTM_OUT_ARR = 0.0D0

!     ... There are weighting functions for ocean parameters
  NU_VEC_PARS = 0
  IF( OWF_FLAG .OR. NUM_OWF ) THEN
     IF( OWF_CHL ) THEN
        NU_VEC_PARS = NU_VEC_PARS + 1 ; N_CHL = NU_VEC_PARS 
     ENDIF
     IF( OWF_Ray ) THEN
        NU_VEC_PARS = NU_VEC_PARS + 1 ; NR_UWT = NU_VEC_PARS 
     ENDIF
     IF( OWF_Use ) THEN
        NU_VEC_PARS = NU_VEC_PARS + 1 ; NP_Us = NU_VEC_PARS 
     ENDIF
     IF( OWF_Ks ) THEN
        NU_VEC_PARS = NU_VEC_PARS + 1 ; NP_Ks = NU_VEC_PARS 
     ENDIF
     IF( OWF_Kl ) THEN
        NU_VEC_PARS = NU_VEC_PARS + 1 ; NP_Kl = NU_VEC_PARS 
     ENDIF
     ALLOCATE( OWF_LABEL_V(NU_VEC_PARS), &
          OWF_OUT_VEC(MAX_SPEC, N_LOS_OUT, NLW, NU_VEC_PARS, Nvs) ) !A   

     IF( OWF_CHL ) OWF_LABEL_V(N_CHL)  = 'Chl'
     IF( OWF_Ray ) OWF_LABEL_V(NR_UWT) = 'Ray-water'
     IF( OWF_Use ) OWF_LABEL_V(NP_Us)  = 'Pm-user'
     IF( OWF_Ks  ) OWF_LABEL_V(NP_Ks)  = 'Pm-small'
     IF( OWF_Kl  ) OWF_LABEL_V(NP_Kl)  = 'Pm-large'
  ENDIF

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
  
200 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'RTM mode not found')
  WRITE(EU,*) '    Selected RTM mode not found :', TRIM(RTM_MODE)
  RETURN 1
  
CONTAINS
!***************************************************************************
SUBROUTINE DEALLOCATE_ARRAYS
    IF(ALLOCATED(WF_TO_AMF)) DEALLOCATE(WF_TO_AMF)
    IF(ALLOCATED(VERT_COL)) DEALLOCATE(VERT_COL)
    IF(ALLOCATED(ALB_WL)) DEALLOCATE(ALB_WL)
    IF(ALLOCATED(RTM_OUT_ARR)) DEALLOCATE (RTM_OUT_ARR)
    IF(ALLOCATED(OWF_LABEL_V)) DEALLOCATE (OWF_LABEL_V, OWF_OUT_VEC)    
END SUBROUTINE DEALLOCATE_ARRAYS
!***************************************************************************
END SUBROUTINE RTM_OUTPUT_ARRAYS

!***************************************************************************
SUBROUTINE RTM_OUTPUT_WFS(MAX_SPEC)
  !     ... Call in "rtm_main.f"

  USE ALB_PARS,    ONLY: Nc_ALB
  USE CLOUDS,      ONLY: N_CLD_LAYERS
  USE GRIDS,       ONLY: ZZZ, NLAYERS
  USE INFO,        ONLY: RET_INFO, WF_DB
  USE INTERN_DIMS, ONLY: Nvs
  USE RTM_OUTPUT,  ONLY: WF_TG_FLAG
  USE SPHER_PARS,  ONLY: N_LOS_OUT
  USE ST_VARS,     ONLY: RTM_TYPE
  !------------------------------------------------------------------

  INTEGER, INTENT(IN) :: MAX_SPEC
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'RTM_OUTPUT_WFS'
  INTEGER   KK, II, N_WF, N_SBP, JJ
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  
  IF (ALLOCATED(WF_DB)) THEN
     DO II = 1, UBOUND(WF_DB,1)
        DEALLOCATE(WF_DB(II)%WF)
     END DO
     DEALLOCATE(WF_DB)
  END IF

  N_WF = COUNT(RET_INFO%STAT.AND.RET_INFO%WF_ON)

!  WRITE(*,*) '  Number N_WF =  ', N_WF

  ALLOCATE(WF_DB(N_WF))
  KK = 0
  DO II = 1, UBOUND(RET_INFO,1)
     IF(.NOT.RET_INFO(II)%STAT.OR..NOT.RET_INFO(II)%WF_ON) CYCLE

     KK = KK + 1
     N_SBP = RET_INFO(II)%N_SBP
!     WRITE(*,*) '   N_SBP ', II, N_SBP     
     IF(ALLOCATED(RET_INFO(II)%I_S)) &
          DEALLOCATE(RET_INFO(II)%I_S, RET_INFO(II)%I_E)
     ALLOCATE(RET_INFO(II)%I_S(N_SBP), RET_INFO(II)%I_E(N_SBP))

     IF(RET_INFO(II)%VS /= 'S') THEN
        WF_DB(KK)%NL = NLAYERS
        IF(N_SBP == 1) THEN
           RET_INFO(II)%I_S = COUNT(SNGL(ZZZ(:)) > SNGL(RET_INFO(II)%Z_U)) + 1
           RET_INFO(II)%I_E = COUNT(SNGL(ZZZ(:)) >= SNGL(RET_INFO(II)%Z_L))
        ELSE
           ! Here selection of several altitude ranges from a parameter
           ! can be done
           STOP 'Not yet implemented !'
        END IF
     ELSE
!        WRITE(*,*) '  Check WF_DB(KK)%NL ', KK, N_SBP
!        PAUSE
        WF_DB(KK)%NL = N_SBP
        IF(N_SBP == 1) THEN
           RET_INFO(II)%I_S(:) = 1
        ELSE
           ! Each column is a sub-parameter
           RET_INFO(II)%I_S(:) = [(JJ, JJ = 1,N_SBP)]
        END IF
        RET_INFO(II)%I_E(:) = RET_INFO(II)%I_S(:)
     END IF
     WF_DB(KK)%NAME = RET_INFO(II)%WF_NAME
     WF_DB(KK)%LONG_NAME = RET_INFO(II)%LONG_NAME
     WF_DB(KK)%TYPE = RET_INFO(II)%TYPE

     ! !!! Test: Option 'rel' for all !!!
     WF_DB(KK)%DIM = RET_INFO(II)%WF_DIM
!     WF_DB(KK)%DIM = 'rel'

     WF_DB(KK)%NF = 1
     ALLOCATE(WF_DB(KK)%WF(MAX_SPEC, N_LOS_OUT, WF_DB(KK)%NL, Nvs))
     WF_DB(KK)%WF = 0.0D0
 
     IF(RET_INFO(II)%TYPE == 'tg') WF_TG_FLAG = .TRUE.
     If(RTM_TYPE == 'transmission'.AND.WF_DB(KK)%Name == 'aer-ext') &
          WF_TG_FLAG = .TRUE.
  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE RTM_OUTPUT_WFS

!***************************************************************************
SUBROUTINE SET_POINTERS(NSPEC, *)
  ! Call in "rtm_main.f" and "st_convol.f"

  USE ABSORBERS,   ONLY: N_GAS
  USE INTERN_DIMS, ONLY: Nvs
  USE LOG_PARS,    ONLY: EU, LUNIT
  USE RTM_OUTPUT,  ONLY: RTM_OUT_ARR, SPEC_OUT, FLUX_UP, FLUX_DN, DIF_FLUX_UP, &
                         DIF_FLUX_DN, DIR_RAD, ACTFLUX, OUT_VOD_ALL, DO_VOD_ONLY, &
                         TG_VOD, NO_RAD_OUT, N_VOD, SPEC_OUT_SAVE, R_SS, DEL_RRS, &
                         N_OP, STOK_OUT, DR_FIN_TOP, DR_TAU, DR_FIN_GT, DIF_SCAL_UP, &
                         DEL_VRS, DIR_FLUX, Tra_mu0, Tra_muv, TRANS_OUT
  USE ST_VARS,     ONLY: RTM_MODE, RAD_FIELD, I_LAY
  USE SPHER_PARS,  ONLY: N_SOL_ANG
  USE WATER,       ONLY: INEL_ADJ, OUT_WLR
  
  INTEGER, INTENT(IN) :: NSPEC

  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_POINTERS'
  INTEGER  II
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  
  DO_VOD_ONLY = .FALSE.
  NO_RAD_OUT  = .FALSE.

  SELECT CASE (TRIM(RTM_MODE))
  CASE ('asymp') 
     SPEC_OUT   => RTM_OUT_ARR(1:NSPEC,1,1:1,:,1) 
     STOK_OUT   => RTM_OUT_ARR(1:NSPEC,:,1:1,:,1:Nvs) ! A
     DR_FIN_TOP => RTM_OUT_ARR(1:NSPEC,1,1,:,2) 
     DR_FIN_GT  => RTM_OUT_ARR(1:NSPEC,1,1,:,3)
     DR_TAU     => RTM_OUT_ARR(1:NSPEC,1,1,:,4)
     R_SS       => RTM_OUT_ARR(1:NSPEC,1,1,:,5)

  CASE ('int') 
     SPEC_OUT => RTM_OUT_ARR(1:NSPEC,1,1:N_OP,:,1)   ! A
     ! Preliminary it is assumed that VRS using forward-adjoint
     ! technique for scalar case only, i.e., Nvs = 1
     IF( INEL_ADJ ) THEN
        STOK_OUT => RTM_OUT_ARR(1:NSPEC,:,:,:,1:Nvs) ! A
        DEL_VRS  => RTM_OUT_ARR(1:NSPEC,1,1,:,2)     ! A         
     ELSE
        STOK_OUT => RTM_OUT_ARR(1:NSPEC,:,:,:,1:Nvs) ! A
     ENDIF

  CASE ('amf','slant_col','amf2','slant_col2') 
     SPEC_OUT      => RTM_OUT_ARR(1:NSPEC,1,1:1,:,1) ! A
     STOK_OUT      => RTM_OUT_ARR(1:NSPEC,:,1:1,:,1:Nvs) ! A
     SPEC_OUT_SAVE => RTM_OUT_ARR(1:NSPEC,1,1,:,Nvs+1) ! A
     TG_VOD => RTM_OUT_ARR(1:NSPEC,1,1,1:N_SOL_ANG,Nvs+2:Nvs+2) ! A

  CASE ('flux') 
     NO_RAD_OUT = .TRUE.
     DIR_RAD     => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,1) ! A
     ACTFLUX     => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,2) ! A
     FLUX_UP     => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,3) ! A
     FLUX_DN     => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,4) ! A
     DIF_FLUX_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,5) ! A
     DIF_FLUX_DN => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,6) ! A
     DIF_SCAL_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,7) ! A
     DIR_FLUX    => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,8) ! A
     
  CASE ('int_flux')
     DIR_RAD     => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,1) ! A
     ACTFLUX     => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,2) ! A
     FLUX_UP     => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,3) ! A
     FLUX_DN     => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,4) ! A
     DIF_FLUX_UP => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,5) ! A
     DIF_FLUX_DN => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,6) ! A
     DIF_SCAL_UP => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,7) ! A
     DIR_FLUX    => RTM_OUT_ARR(1:NSPEC,1,1:I_LAY,1:N_SOL_ANG,8) ! A
     II = N_OP;  IF( OUT_WLR ) II = II + 1
     SPEC_OUT    => RTM_OUT_ARR(1:NSPEC,1,I_LAY+1:I_LAY+II,:,8+1)           ! A
     STOK_OUT    => RTM_OUT_ARR(1:NSPEC,:,I_LAY+1:I_LAY+II,:,8+1:8+Nvs)     ! A

  CASE ('spher_alb') 
     NO_RAD_OUT = .TRUE.
     IF( RAD_FIELD ) THEN ! the same as 'flux' in spherical geometry
        DIR_RAD => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,1) ! A
        ACTFLUX => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,2) ! A
        FLUX_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,3) ! A
        FLUX_DN => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,4) ! A
        DIF_FLUX_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,5) ! A
        DIF_FLUX_DN => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,6) ! A
        DIF_SCAL_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,7) ! A
        DIR_FLUX => RTM_OUT_ARR(1:NSPEC,1,:,1:N_SOL_ANG,8) ! A
     ELSE
        FLUX_UP => RTM_OUT_ARR(1:NSPEC,1,:,1:1,1) ! A
        FLUX_DN => RTM_OUT_ARR(1:NSPEC,1,:,1:1,2) ! A
     ENDIF

  CASE ('vod')
     DO_VOD_ONLY = .TRUE.
     NO_RAD_OUT = .TRUE.
     TG_VOD => RTM_OUT_ARR(1:NSPEC,1,1,:,1:N_GAS) ! A
     OUT_VOD_ALL => RTM_OUT_ARR(1:NSPEC,1,1,1,N_GAS + 1:N_VOD) ! A
     
  CASE ('raman')
     SPEC_OUT => RTM_OUT_ARR(1:NSPEC,1,1:1,:, 1 ) ! A
     STOK_OUT => RTM_OUT_ARR(1:NSPEC,:, : ,:,1:Nvs) ! A
     DEL_RRS  => RTM_OUT_ARR(1:NSPEC,1, 1 ,:,Nvs+1:2*Nvs) ! A

  CASE ('wf','block_amf','num_pert') 
     SPEC_OUT => RTM_OUT_ARR(1:NSPEC,1,1:1,:,1) ! A
     STOK_OUT => RTM_OUT_ARR(1:NSPEC,:, : ,:,1:Nvs) ! A
     IF( TRANS_OUT ) THEN
        Tra_mu0  => RTM_OUT_ARR(1:NSPEC,1,1,:,Nvs+1) ! A
        Tra_muv  => RTM_OUT_ARR(1:NSPEC,1,1,:,Nvs+2) ! A
     ENDIF
     CALL SET_POINTERS_WF(NSPEC)

  CASE DEFAULT
     CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'RTM mode not found')
     WRITE(EU,*) '    Selected RTM mode not found :', TRIM(RTM_MODE)
     RETURN 1

  END SELECT

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE SET_POINTERS

!***************************************************************************
SUBROUTINE SET_POINTERS_WF(NSPEC)
  ! Call in "rtm_init.f" and "set_pointers.f"
  
  USE INFO,        ONLY: WF_DB
  USE INTERN_DIMS, ONLY: Nvs
  USE RTM_OUTPUT,  ONLY: NR_UWT, N_CHL, NP_Us, NP_Ks, NP_Kl, CHL_WEI, &
                         RUW_WEI, Pus_WEI, Pks_WEI, Pkl_WEI, OWF_OUT_VEC
  USE ST_VARS,     ONLY: RTM_CORE, OWF_FLAG, OWF_Ray, OWF_Chl, OWF_Use, OWF_Ks, OWF_Kl

  INTEGER, INTENT(IN) ::  NSPEC
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_POINTERS_WF'
  !==================================================================
  
  CALL ST_TRACK (R_NAME, 1, 'I')

  IF(TRIM(RTM_CORE) == 'DOM') THEN
     IF( OWF_FLAG ) THEN
        IF(OWF_CHL) CHL_WEI => OWF_OUT_VEC(1:NSPEC,:,:,N_CHL,1) 
        IF(OWF_Ray) RUW_WEI => OWF_OUT_VEC(1:NSPEC,:,:,NR_UWT,1) 
        IF(OWF_Use) Pus_WEI => OWF_OUT_VEC(1:NSPEC,:,:,NP_Us,1) 
        IF(OWF_Ks)  Pks_WEI => OWF_OUT_VEC(1:NSPEC,:,:,NP_Ks,1) 
        IF(OWF_Kl)  Pkl_WEI => OWF_OUT_VEC(1:NSPEC,:,:,NP_Kl,1) 
     ENDIF
  END IF

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
END SUBROUTINE SET_POINTERS_WF

!***************************************************************************
