!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 03 Jan 2024
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE SCM_TO_EC(MAX_CO, N_GAU, LA_N, NAW, Nel, ESM, Names, INT_SCM, &
      CRIT_SCM, Lay_out, WL_out, OUT_SCM, M_Lay_Wav, USER_MOM, OUT_PATH, *)
  Use Cld_SS,   ONLY: MUM => MAXMOM
  USE I_DIMS,   ONLY: Nm_vs, Nvs
  USE LOG_PARS, ONLY: LUNIT, EU, DIR_OUT, LOG_LEVEL, N_ESO, C_UNIT
  USE ST_VARS,  ONLY: Pie, RTD => RAD_TO_DEG
  USE TYPES,    ONLY: KND, DYN_CHAR
!  -----------------------------------------------------------------------
  Character (Len = *), PARAMETER :: R_NAME = 'SCM_TO_EC'
  INTEGER, INTENT(IN) :: MAX_CO, N_GAU, Nel, Lay_out, WL_out, LA_N, NAW
  CHARACTER (LEN=3)  ESM(Nel), INT_SCM
  INTEGER   M_Lay_Wav(LA_N,NAW)
  INTEGER   Na_st, I, J, K, I1, KK, N_COEFF, L,  INBV, N_mom
  LOGICAL   Error, Log_mat(4), OUT_SCM
  CHARACTER (LEN=*)   OUT_PATH
  REAL (KND)  Norm, S1, Clk, Cofp, Cofm, TMP1, CRIT_SCM
  REAL (KND) USER_MOM(MUM,LA_N,Nm_vs,NAW)  
  REAL (KND)  Poly(MAX_CO,N_gau), DUM_READ(Max_co,Nel+1)
  REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, Phs_gau, Phx, Del
  REAL (KND), ALLOCATABLE :: Sa_inp(:), Sa_rad(:), Ph_out(:), &
       Phs_log(:), D_02g(:,:), D_P2g(:,:), D_M2g(:,:), Cf(:), &
       Sca_mat(:,:), Int_mat(:,:), Ph_ret(:,:)
  REAL (KND), ALLOCATABLE ::  WORK(:), B_COEFF(:), S_NODES(:)
  TYPE (DYN_CHAR), ALLOCATABLE :: Names(:,:)
  INTEGER  OUNIT
  PARAMETER  ( OUNIT = 55 )
! ==========================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')  
  Error = .FALSE.
  ! Gauss grid and weights for integration range (-1,1)
  CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
  ! Legendre polynomials for all Gauss angles
  CALL WIGNER_PLG( X_gau, N_gau, 0, Max_co, Poly )
  X_rad(:) = DACOS( X_gau(N_gau:1:-1) )
  IF( Nvs > 1 ) THEN
     ALLOCATE( D_02g(Max_co,N_gau), D_P2g(Max_co,N_gau), & 
          D_M2g(Max_co,N_gau), Cf(Nm_vs) )
     CALL WIGNER_PLG     ( X_gau, N_gau, 2, Max_co, D_02g )
     CALL WIGNER_FUNCTION( X_gau, N_gau, 2, Max_co, D_P2g, D_M2g )
  ENDIF  ! Nvs > 1 (vector case)

  ! Reading of the user scattering function/matrix 
  DO I1 = 1,LA_N
     DO K = 1,NAW
        OPEN(C_UNIT, FILE=Names(I1,K)%S, STATUS='OLD') 
        CALL SKIP_HEADER (C_UNIT)           
        DO I = 1,Max_co
           READ(C_UNIT,*,END=128,ERR=129) (DUM_READ(I,J), J=1,Nel+1)
        END DO
129     CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Error reading data file !')
        WRITE(EU,*) '    File name : ', TRIM(Names(I1,K)%S) 
        RETURN 1             
128     CONTINUE
        Na_st = I - 1  ! Number of scattering angles
        CLOSE(C_UNIT)
        !          WRITE(*,*) '  Number of scattering angles:', Na_st
        !          WRITE(*,*) '  First :', DUM_READ(1,1:2)
        !          WRITE(*,*) '  Last  :', DUM_READ(Na_st,1:2)
        ALLOCATE( Sa_inp(Na_st), Sa_rad(Na_st),  &
             Phs_log(Na_st) )              ! A          

        ALLOCATE( WORK(5*(Na_st+2)),B_COEFF(Na_st+2),S_NODES(Na_st+6) )
        ALLOCATE( Sca_mat(Na_st,Nm_vs), Int_mat(N_gau,Nm_vs) )
        
        ! Original angle grid and scattering function 
        Sa_inp(:)  = DUM_READ(1:Na_st,1)
        Sca_mat(:,1) = DUM_READ(1:Na_st,2)

        ! Convert [deg] to [rad] for interpolation of Ph 
        Sa_rad(:) = Sa_inp(:)/RTD
        ! Sa_inp(:) = DCOS( Sa_rad(Na_st:1:-1) )

        ! Interpolate scattering function on the Gauss grid
        Phs_log(:) = DLOG( Sca_mat(:,1) )
        SELECT CASE( INT_SCM )
        CASE('aki')
           CALL AKIMA_INTERP (Na_st, Sa_rad, Phs_log, N_gau,  &
                X_rad, Phs_gau, Error)  
           IF( Error ) THEN
              Call WRITE_MESSAGE (R_NAME,'ERROR', 'Interpolation error')
              RETURN 1
           ENDIF
        CASE('lin')
           CALL LIN_INT (Na_st, Sa_rad, Phs_log, N_gau, X_rad, Phs_gau, 'L')
        CASE('spl')
           CALL DBINT4 (Sa_rad, Phs_log, Na_st, 2, 2, 0.d0, 0.d0, 1,  &
                S_NODES, B_COEFF, N_COEFF, KK, WORK, *880) 
           INBV = 1
           DO I = 1, N_gau
              CALL DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0,  &
                   X_rad(I), INBV, WORK(1:12), Phs_gau(I), *880 )
           END DO
        END SELECT
        ! Invert to start from -1
        Phs_gau(:) = DEXP( Phs_gau(N_gau:1:-1) )
        ! Norm of the interpolated phase function
        Norm = SUM( Phs_gau(:)*AG_gau(:) )*0.5d0
        IF (LOG_LEVEL(1) >= N_ESO) THEN
           S1 = 0.d0
           DO I = 1, Na_st - 1
              S1 = S1 + (Sca_mat(I,1)*DSIN(Sa_rad(I)) &
                   + Sca_mat(I+1,1)*DSIN(Sa_rad(I+1)))/2.d0 &
                   *(Sa_rad(I+1) - Sa_rad(I))
           ENDDO  ! over scattering angle (I)
           WRITE(LUNIT,*) '   Integral of the phase function '
           WRITE(LUNIT,*) '     Initial scat-angle grid: ', S1*2.*Pie
           WRITE(LUNIT,*) '     Extended scat-angle grid:', Norm*4.*Pie
           ! Number of expansion coefficients of intrpolated Ph
           WRITE(*,*) '   Max_co = ', Max_co
        ENDIF
        ! Expansion of F11 matrix element          
        Phx(:) = 0.d0
        DO L = 1,Max_co
           TMP1 = (DBLE(L)-0.5d0)*SUM( Phs_gau(:)*Poly(L,:)*AG_gau(:) )&
                /Norm
           USER_MOM(L,I1,1,K) = TMP1
           DO J = 1,N_gau
              Phx(J) = Phx(J) + Poly(L,J)*TMP1
           ENDDO  ! over Gauss angles (J)
           Del(:) = DABS( Phs_gau(:) - Phx(:)*Norm )/Phs_gau(:)*100.
           IF( MAXVAL( Del(:) ) <= CRIT_SCM ) EXIT 
        ENDDO
        M_Lay_Wav(I1,K) = L
        IF( OUT_SCM.AND.I1==Lay_out.AND.K==WL_out.AND.Nvs==1 ) THEN
           N_mom = M_Lay_Wav(I1,K)
           ALLOCATE( Ph_out(N_gau) )
           DO J = 1,N_gau
              Ph_out(J) = SUM(Poly(1:N_mom,J)*USER_MOM(1:N_mom,I1,1,K))
           ENDDO
           ! WRITE(*,*) '   Norm of ph after expansion',&
           ! SUM( Ph_out(:)*AG_gau(:) )*0.5d0
           OPEN( OUNIT,FILE=TRIM(OUT_PATH) )
           !             OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat.out')
           WRITE(OUNIT,'(a)') '#  Interpolated original and '//&
                'recalculated phase functions'
           WRITE(OUNIT,'(a,2x,I3)') '#  Layer number  :', I1
           WRITE(OUNIT,'(a,2x,I3)') '#  Wavelength    :', K
           WRITE(OUNIT,'(a,2x,E12.4)') '#  Norm of interpolated Ph', Norm
           WRITE(OUNIT,'(a,2x,I5)') &
                '#  Number of expansion coefficients used', M_Lay_Wav(I1,K)
           DO J = 1,N_gau
              WRITE(OUNIT,'(1x,3(e15.7))') DACOS(X_gau(J))*RTD, &
                   Phs_gau(J), Ph_out(J)*Norm
           ENDDO
           CLOSE(OUNIT)
        ENDIF
        IF (LOG_LEVEL(1) >= N_ESO) &
             WRITE(LUNIT,*) '   Number of expansion coeff: ', L-1
        IF( Nvs == 1 ) THEN
           DEALLOCATE( Sa_inp, Sa_rad, Phs_log, Sca_mat, Int_mat, &
                WORK, B_COEFF, S_NODES)          
           CYCLE
        ENDIF
        
!         *************************************************          
! ***************** Other elements of scattering matrix *************
!         *************************************************          
        Log_mat(:) = .FALSE.
        DO I = 2, Nel
           SELECT CASE( ESM(I) )
           CASE('F22')  ;  SCA_MAT(:,2) = DUM_READ(1:Na_st,I+1)
              Log_mat(2) = .TRUE.
           CASE('F33')  ;  SCA_MAT(:,3) = DUM_READ(1:Na_st,I+1)
              Log_mat(3) = .TRUE.
           CASE('F44')  ;  SCA_MAT(:,4) = DUM_READ(1:Na_st,I+1)
              Log_mat(4) = .TRUE.
           CASE('F12','F21') ;  SCA_MAT(:,5) = DUM_READ(1:Na_st,I+1)
           CASE('F34','F43') ;  SCA_MAT(:,6) = DUM_READ(1:Na_st,I+1)
           END SELECT
        ENDDO  ! over scattering matrix elements (I)
        IF( .NOT.Log_mat(2) ) SCA_MAT(:,2) = DUM_READ(1:Na_st,2)
        IF( .NOT.Log_mat(3) ) SCA_MAT(:,3) = SCA_MAT(:,4) 
        IF( .NOT.Log_mat(4) ) SCA_MAT(:,4) = SCA_MAT(:,3) 
        
        DO J = 1, Nm_vs
           SELECT CASE( INT_SCM )
           CASE('aki')
              CALL AKIMA_INTERP (Na_st, Sa_rad, Sca_mat(:,J), N_gau,  &
                   X_rad, Int_mat(:,J), Error)  
              IF( Error ) THEN
                 Call WRITE_MESSAGE (R_NAME,'ERROR', 'Interpolation error')
                 RETURN 1
              ENDIF
           CASE('lin')
              CALL LIN_INT (Na_st, Sa_rad, Sca_mat(:,J), &
                   N_gau, X_rad, Int_mat(:,J), 'L')
           CASE('spl')
              CALL DBINT4 (Sa_rad, Sca_mat(:,J), Na_st, 2, 2, 0.d0, 0.d0, 1,  &
                   S_NODES, B_COEFF, N_COEFF, KK, WORK, *880) 
              INBV = 1
              DO I = 1, N_gau
                 CALL DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0,  &
                      X_rad(I), INBV, WORK(1:12), Int_mat(I,J), *880 )
              END DO
           END SELECT
           Int_mat(:,J) = Int_mat(N_gau:1:-1,J)
        ENDDO  ! over 6 matrix elements (J)
        N_mom = M_Lay_Wav(I1,K)
        DO L = 1, N_mom
           Clk = (DBLE(L) - 0.5d0)/Norm
           !             Cf(1) =  Clk*SUM( Int_mat(:,1)*Poly(L,:)*AG_gau(:) )
           Cf(4) =  SUM( Int_mat(:,4)*Poly(L,:)*AG_gau(:) )
           Cf(5) = -SUM( Int_mat(:,5)*D_02g(L,:)*AG_gau(:) )
           Cf(6) = -SUM( Int_mat(:,6)*D_02g(L,:)*AG_gau(:) )
           Cofm =   SUM( (Int_mat(:,2) - Int_mat(:,3))*D_M2g(L,:)*AG_gau(:) )
           Cofp =   SUM( (Int_mat(:,2) + Int_mat(:,3))*D_P2g(L,:)*AG_gau(:) )
           Cf(2) = (Cofp + Cofm)/2.d0  ;  Cf(3) = (Cofp - Cofm)/2.d0
           Cf(2:Nm_vs) = Clk*Cf(2:Nm_vs)
           Cf(5:6) = -Cf(5:6)
           USER_MOM(L,I1,2:Nm_vs,K) = Cf(2:Nm_vs)
        ENDDO
        
        IF( OUT_SCM.AND.I1==Lay_out.AND.K ==WL_out ) THEN
           IF(.NOT.ALLOCATED(Ph_ret) ) ALLOCATE( Ph_ret(N_gau,Nm_vs) )
           Ph_ret = 0.d0
           DO L = 1, N_mom
              Cf(:) = USER_MOM(L,I1,:,K)
              Ph_ret(:,1) = Ph_ret(:,1) + Poly(L,:) *Cf(1)
              Ph_ret(:,4) = Ph_ret(:,4) + Poly(L,:) *Cf(4)
              Ph_ret(:,5) = Ph_ret(:,5) - D_02g(L,:) *Cf(5)
              Ph_ret(:,6) = Ph_ret(:,6) - D_02g(L,:) *Cf(6)
              Cofp = Cf(2) + Cf(3)  ;  Cofm = Cf(2) - Cf(3)
              Ph_ret(:,2) = Ph_ret(:,2) &
                   + (D_P2g(L,:)*Cofp + D_M2g(L,:)*Cofm)/2.d0
              Ph_ret(:,3) = Ph_ret(:,3) &
                   + (D_P2g(L,:)*Cofp - D_M2g(L,:)*Cofm)/2.d0
           ENDDO
!           OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat-ori.out')
!           DO J = 1,Na_st
!              WRITE(OUNIT,'(1x,*(e15.7))') Sa_rad(J)*RTD, &
!                   (Sca_mat(J,I), I = 1,Nm_vs)
!           ENDDO
!           CLOSE(OUNIT)
           OPEN( OUNIT,FILE=TRIM(OUT_PATH) )
           WRITE(OUNIT,'(a)') '#  Interpolated original and '//&
                'recalculated scattering matrices'
           WRITE(OUNIT,'(a,2x,I5)') &
                '#  Number of expansion coefficients used', M_Lay_Wav(I1,K)
           WRITE(OUNIT,'(a)')  '# 1st    column - Gauss angles'
           WRITE(OUNIT,'(a)')  '# 2 - 7  columns - interpolated original matrix'
           WRITE(OUNIT,'(a)')  '# 8 - 13 columns - recalculated matrix'
           DO J = 1,N_gau
              WRITE(OUNIT,'(1x,*(e15.7))') DACOS(X_gau(J))*RTD, &
                   (Int_mat(J,I), I = 1,Nm_vs),  &
                   (Ph_ret(J,I)*Norm, I = 1,Nm_vs)
           ENDDO
           CLOSE(OUNIT)           
!           OPEN(OUNIT,FILE=DIR_OUT//'/Ph_scat-ret.out')
!           DO J = 1,N_gau
!              WRITE(OUNIT,'(1x,*(e15.7))') DACOS(X_gau(J))*RTD, &
!                   (Ph_ret(J,I)*Norm, I = 1,Nm_vs)
!           ENDDO
!           CLOSE(OUNIT)
        ENDIF
        
        DEALLOCATE( Sa_inp, Sa_rad, Phs_log, Sca_mat, Int_mat, &
             WORK, B_COEFF, S_NODES)          
     ENDDO  ! over reference wavelength grid (K)
  ENDDO  ! over layers (I1)
  
  CALL ST_TRACK (R_NAME, 1, 'O')  
  RETURN
  
880 CONTINUE    
  RETURN 1
  
END SUBROUTINE SCM_TO_EC
