C********************************************************************C
C
C	SCIATRAN - Version 4.0 - Last modification: 20 Dec 2018
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE SET_LIMITS (I_DS, JSEG, N_PAR, N_CRS, WIDE_SHI, GR_PAR)
c     ... Call in "preprocessing.f"

      USE AER_MIE,   ONLY: LAS_mi, T_550
      USE INFO,      ONLY: RET_INFO, STATE
      USE LOG_PARS,  ONLY: LUNIT
      USE ST_VARS,   ONLY: RET_MODE
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER     I_DS, JSEG, N_PAR, N_CRS
      REAL (KND)  GR_PAR(2,N_PAR + N_CRS), WIDE_SHI(2,N_CRS + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS'      
      INTEGER K, LL, I_RLI, I_RI, I_SBP
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      SELECT CASE ( TRIM(RET_MODE) )
      CASE ('std', 'tg_ext', 'th_o2')
         CALL SET_LIMITS_TG (I_DS, JSEG, N_PAR, N_CRS, WIDE_SHI, GR_PAR)

      CASE ('cth')
         CALL SET_LIMITS_CTH (N_PAR, N_CRS, WIDE_SHI, GR_PAR)

      CASE('flu')
         CALL SET_LIMITS_FLU (N_PAR, N_CRS, WIDE_SHI, GR_PAR)
        
      CASE ('AOT_n-1','AOT_n-2')
         WRITE(LUNIT,*) '      Mode :', TRIM(RET_MODE)
         WRITE(LUNIT,*) '***** No limits are used ******'
      CASE ('lwc_ref','aer-bim','aer-hom')
         CALL SET_LIMITS_REF_BIM (I_DS, JSEG, N_PAR, N_CRS, WIDE_SHI, 
     $        GR_PAR)
      CASE ('Pres_Temp')
         CALL SET_LIMITS_PT (N_PAR, N_CRS, WIDE_SHI, GR_PAR)
      CASE ('ler')
         CALL SET_LIMITS_LER (N_PAR, N_CRS, WIDE_SHI, GR_PAR)
      CASE ('aerosol_spec')
         CALL SET_LIMITS_TG (I_DS, JSEG, N_PAR, N_CRS, WIDE_SHI, GR_PAR)
      END SELECT

      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN
      END

C***********************************************************************
      SUBROUTINE SET_LIMITS_TG (I_DS, JSEG, N_PAR, N_CRS, WIDE_SHI, 
     $     GR_PAR)
c     ... Call in "preprocessing.f"

      USE DATA_RET,  ONLY: MDS
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER  I_DS, JSEG, N_PAR, N_CRS
      REAL (KND)  GR_PAR(2,N_PAR + N_CRS), WIDE_SHI(2,N_CRS + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_TG'      
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      WIDE_SHI = 0.d0
C     Model spectrum
      WIDE_SHI(1,1) = -0.3d0
      WIDE_SHI(2,1) = 0.3d0
!      WIDE_SHI(1,1) = -4.0d0
!      WIDE_SHI(2,1) = 4.0d0 
C     Solar spectrum
      WIDE_SHI(1,2) = -0.1d0
      WIDE_SHI(2,2) = 0.1d0
C     Spectral corrections
      IF(N_CRS > 0) THEN
         WIDE_SHI(1,3:N_CRS + 2) = -0.3d0
         WIDE_SHI(2,3:N_CRS + 2) = 0.3d0
      END IF

      IF(MDS(I_DS)%EXP_FORM(JSEG)%S.NE.'undefined') 
     $     WIDE_SHI(1:2,2) = 0.0d0

c      GR_PAR(1,1:N_PAR) = -1.0d+20
c      GR_PAR(2,1:N_PAR) = 1.0d+20
      GR_PAR(1,1:N_PAR) = -3.0d0; GR_PAR(2,1:N_PAR) = +50.0d0
c      GR_PAR(1,1:N_PAR) = -3.0d0; GR_PAR(2,1:N_PAR) = +3.0d0
      GR_PAR(1,N_PAR + 1:N_PAR + N_CRS) = -1.0d+20
      GR_PAR(2,N_PAR + 1:N_PAR + N_CRS) = +1.0d+20

c      IF(IEC_FLAG) THEN
c         GR_PAR(1,N_PAR + N_CRS) = 0.5d+0
c         GR_PAR(2,N_PAR + N_CRS) = 1.5d+0
c      END IF

c      GR_PAR(1,N_PAR + 1:N_PAR + 1) =  1.1d0
c      GR_PAR(2,N_PAR + 1:N_PAR + 1) =  1.1d0
c
c      GR_PAR(1,N_PAR + 2:N_PAR + N_CRS) = -1.0d+20
c      GR_PAR(2,N_PAR + 2:N_PAR + N_CRS) = +1.0d+20
 
      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN
      END

C***********************************************************************
      SUBROUTINE SET_LIMITS_CTH(N_PAR, N_RIN, WIDE_SHI, GR_PAR)
c     ... Call in "preprocessing.f"

      USE CLOUDS,      ONLY: CLD_LAYER_HEIGHTS
      USE CLOUDS_RETR, ONLY: GT_MIN, GT_MAX, CTH_MIN, CTH_MAX, 
     $                       CBH_MIN, CBH_MAX, GCP, Ngp_CLD
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------
 
      INTEGER  N_PAR, N_RIN
      REAL (KND) GR_PAR(2,N_PAR + N_RIN), WIDE_SHI(2,N_RIN + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_CTH'      
      INTEGER  I
      REAL (KND) CTH, CBH, G_T
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      WIDE_SHI = 0.d0
C     Model spectrum
      WIDE_SHI(1,1) = -0.1d0
      WIDE_SHI(2,1) = 0.1d0
C     Solar spectrum
      WIDE_SHI(1,2) = -0.1d0
      WIDE_SHI(2,2) = 0.1d0
C     Spectral corrections
      WIDE_SHI(1,2:N_RIN + 2) = -0.1d0
      WIDE_SHI(2,2:N_RIN + 2) = 0.1d0

      CTH = CLD_LAYER_HEIGHTS(1,2)
      CBH = CLD_LAYER_HEIGHTS(1,1)
      G_T = CTH - CBH

c      WRITE(*,*) '   "set_limits":    N_PAR =  ', N_PAR
c      WRITE(*,*) '   "set_limits":  Ngp_CLD =  ', Ngp_CLD

c     ... constrain for new estimation of cloud top height
      DO I = 1,  Ngp_CLD
         SELECT CASE( GCP(I) )
         CASE('top')
            GR_PAR(1,I) = CTH_MIN - CTH
            GR_PAR(2,I) = CTH_MAX - CTH
         CASE('thi')
            GR_PAR(1,I) = GT_MIN - G_T
            GR_PAR(2,I) = GT_MAX - G_T
            IF( Ngp_CLD == 1 ) THEN
               GR_PAR(1,1) = CBH_MIN - CBH
               GR_PAR(2,1) = CTH_MAX - CTH            
            ENDIF
         CASE('bot')
            GR_PAR(1,I) = CBH_MIN - CBH
            GR_PAR(2,I) = CBH_MAX - CBH
         END SELECT
      ENDDO

c     ... No restrictions for parameter tau_cld
      IF( N_PAR > Ngp_CLD ) THEN
         GR_PAR(1,N_PAR) = -1.0d+20
         GR_PAR(2,N_PAR) = +1.0d+20
      ENDIF

      GR_PAR(1,N_PAR + 1:N_PAR + N_RIN) = -1.0d+20
      GR_PAR(2,N_PAR + 1:N_PAR + N_RIN) = +1.0d+20

      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN
      END SUBROUTINE SET_LIMITS_CTH

C**********************************************************************
C***********************************************************************
      SUBROUTINE SET_LIMITS_PT(N_PAR, N_RIN, WIDE_SHI, GR_PAR)
c     ... Call in "preprocessing.f"

      USE DATA_RET,  ONLY: RET_FORM
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER  N_PAR, N_RIN
      REAL (KND) GR_PAR(2,N_PAR + N_RIN), WIDE_SHI(2,N_RIN + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_PT'
      INTEGER   I
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')
 
      WIDE_SHI = 0.d0
C     Model spectrum
      WIDE_SHI(1,1) = -0.3d0
      WIDE_SHI(2,1) =  0.3d0
C     Solar spectrum
      SELECT CASE( TRIM(RET_FORM) )
      CASE( 'Rel_refl' )
         WIDE_SHI(1,2) = 0.0d0
         WIDE_SHI(2,2) = 0.0d0
      CASE( 'Log','Rel_log' )
         WIDE_SHI(1,2) = -0.3d0
         WIDE_SHI(2,2) =  0.3d0
      END SELECT
C     Spectral corrections
      IF( N_RIN > 0 ) THEN
         WIDE_SHI(1,2:N_RIN + 2) = -0.3d0
         WIDE_SHI(2,2:N_RIN + 2) =  0.3d0
      ENDIF
c     ... constrain for new estimation of P and T
      DO I = 1, N_PAR
         GR_PAR(1,I) = -1.0d+20
         GR_PAR(2,I) = +1.0d+20
      ENDDO

      IF( N_RIN > 0 ) THEN
         GR_PAR(1,N_PAR + 1:N_PAR + N_RIN) = -1.0d+20
         GR_PAR(2,N_PAR + 1:N_PAR + N_RIN) = +1.0d+20
      ENDIF

      CALL ST_TRACK (R_NAME, 2, 'O')
      RETURN
      END

C***********************************************************************
      SUBROUTINE SET_LIMITS_LER (N_PAR, N_RIN, WIDE_SHI, GR_PAR)
c     ... Call in "preprocessing.f"

      USE DATA_RET,  ONLY: LER_MIN, LER_MAX
      USE ST_VARS,   ONLY: GROUND_HEIGHT
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER  N_PAR, N_RIN
      REAL (KND)  GR_PAR(2,N_PAR + N_RIN), WIDE_SHI(2,N_RIN + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_LER'      
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      WIDE_SHI = 0.0D0
C     Model spectrum
      WIDE_SHI(1,1) = -0.3d0
      WIDE_SHI(2,1) = 0.3d0
C     Solar spectrum
      WIDE_SHI(1,2) = -0.3d0
      WIDE_SHI(2,2) = 0.3d0
C     Spectral corrections
      IF( N_RIN > 0 ) THEN
         WIDE_SHI(1,2:N_RIN + 2) = -0.3d0
         WIDE_SHI(2,2:N_RIN + 2) = 0.3d0
      ENDIF
c     ... constrain for new estimation of surface height
      GR_PAR(1,1) = LER_MIN - GROUND_HEIGHT
      GR_PAR(2,1) = LER_MAX - GROUND_HEIGHT

      IF( N_RIN > 0 ) THEN
         GR_PAR(1,N_PAR + 1:N_PAR + N_RIN) = -1.0d+20
         GR_PAR(2,N_PAR + 1:N_PAR + N_RIN) = +1.0d+20
      ENDIF

      CALL ST_TRACK (R_NAME, 2, 'I')
      RETURN
      END

C***********************************************************************
      SUBROUTINE SET_LIMITS_REF_BIM(I_DS, JSEG, N_PAR, N_RIN, WIDE_SHI, 
     $     GR_PAR)
c     ... Call in "set_limits.f"

      USE AEROSOLS,  ONLY: AP_Mie, MODIS_AP
      USE AER_MIE,   ONLY: LAS_mi, T_550
      USE ALB_PARS,  ONLY: Nc_ALB, ALB_WEI
      USE BRDF,      ONLY: Cfr_v
      USE CLOUDS,    ONLY: N_CLDPROF, CLDPROFILE, Tau_cld_inp
      USE DATA_RET,  ONLY: MDS
      USE GRIDS,     ONLY: NLAYERS, ZZZ, DZ
      USE INFO,      ONLY: RET_INFO
      USE RTM_PARS,  ONLY: DATA_ITER
      USE ST_VARS,   ONLY: BRDF_FLAG
      USE TOOLS,     ONLY: SET_PTR
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER  I_DS, JSEG, N_PAR, N_RIN
      REAL (KND) GR_PAR(2,N_PAR + N_RIN), WIDE_SHI(2,N_RIN + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_REF_BIM'      
      INTEGER    I, KK, II
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND)  R_apr
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      WIDE_SHI = 0.d0

      KK = 1
      DO II = 1, UBOUND(RET_INFO,1)

         IF( .NOT.RET_INFO(II)%STAT ) CYCLE

         CALL SET_PTR( RTM_PTR, RET_INFO(II)%NAME )

         SELECT CASE (RET_INFO(II)%NAME)
c        Linear profile of effective radius and COT retrieval
c        ****************************************************
         CASE ('cld-ref_top') ;  R_apr = CLDPROFILE(1,1,3)
         CASE ('cld-ref_bot') ;  R_apr = CLDPROFILE(1,1,3)
         CASE ('cld-tau_nomix')     ;  R_apr = Tau_cld_inp(1,1)
         CASE ('cld-pnc_nomix')     ;  R_apr = 0.d0

c        Bimodal PSD function aerosol retrieval
c        **************************************
         CASE('aer-dens')
c           ... Scaling factor of aerosol number density ??
            IF( MODIS_AP ) THEN
               R_apr = T_550
            ELSE
               R_apr = RTM_PTR%CRN(NLAYERS)
               write(*,*) 
     $               '  RTM_PTR%CRN(NLAYERS)  = ', RTM_PTR%CRN(NLAYERS)
c                R_apr = Ap_Mie(NLAYERS,KK)
c               write(*,*) '   Ap_Mie(NLAYERS,KK) = ', Ap_Mie(NLAYERS,KK)
            ENDIF
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0 
c            GR_PAR(1,KK) = 0.d0
c            GR_PAR(2,KK) = 0.d0

         CASE('aer-modr_f')
c           ... A priori value of the fine mode modal radius:
            R_apr = RTM_PTR%CRN(LAS_mi(1))
c            R_apr = Ap_Mie(LAS_mi(1),KK)
            write(*,*) '   Apr. val. Rf = ', R_apr
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0

         CASE('aer-modr_c')
c           ... A priori value of the course mode modal radius:
            R_apr = RTM_PTR%CRN(LAS_mi(1))
c            R_apr = Ap_Mie(LAS_mi(1),KK)
            write(*,*) '   Apr. val. Rc = ', R_apr
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0

         CASE('aer-var_f')
c           ... A priori value of variance (fine mode):
            R_apr = RTM_PTR%CRN(LAS_mi(1))
c            R_apr = Ap_Mie(LAS_mi(1),KK)
            write(*,*) '   Apr. val. Vf = ', R_apr
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0

         CASE('aer-var_c')
c           ... A priori value of variance (course mode) :
            R_apr = RTM_PTR%CRN(LAS_mi(1))
c            R_apr = Ap_Mie(LAS_mi(1),KK)
            write(*,*) '   Apr. val. Vc = ', R_apr
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0

         CASE('aer-cm_frac')
c           ... A priori value of the fraction:
            R_apr = RTM_PTR%CRN(LAS_mi(1))
c            R_apr = Ap_Mie(LAS_mi(1),KK)
            write(*,*) '   Apr. val. Fr = ', R_apr
            GR_PAR(1,KK) = RET_INFO(II)%PAR_MIN/R_apr - 1.d0
            GR_PAR(2,KK) = RET_INFO(II)%PAR_MAX/R_apr - 1.d0
            IF( RET_INFO(KK)%NORM=='abs' ) THEN
               GR_PAR(1,KK) = GR_PAR(1,KK) *R_apr
               GR_PAR(2,KK) = GR_PAR(2,KK) *R_apr
            ENDIF

         CASE('albedo')
            
c            WRITE(*,*) ' albedo KK =   ',KK
            IF( BRDF_FLAG ) THEN
               GR_PAR(1,KK) = -0.99d0
               GR_PAR(2,KK) = 1.d0/Cfr_v - 1.d0
            ELSE
c              ... Scaling of given albedo spectrum:
               IF( Nc_ALB==1 ) THEN
                  GR_PAR(1,I) = -0.9d0
                  GR_PAR(2,I) = +2.0d0
               ELSE
c                 ... Albedo is represented as a sum of Nc_ALB
c                     basic spectra:
                  DO I = 1, Nc_ALB
c                     IF( I==1 ) THEN
c                        GR_PAR(1,KK+I-1) =  0.d0 
c                        GR_PAR(2,KK+I-1) =  0.001d0
c                     ELSE
c                        GR_PAR(1,KK+I-1) =  0.d0 
c                        GR_PAR(2,KK+I-1) =  0.001d0 
c                     ENDIF
                     IF( I==1.OR.I==2 ) THEN
c                        WRITE(*,*) '  Albedo : ', I, KK+I-1
                        GR_PAR(1,KK+I-1) =  -1.0d+20
                        GR_PAR(2,KK+I-1) =  +1.0d+20
                     ELSE
c                        WRITE(*,*) '  Albedo : ', I, KK+I-1
                        GR_PAR(1,KK+I-1) =  -1.0d+20
                        GR_PAR(2,KK+I-1) =  +1.0d+20
c                        GR_PAR(1,KK+I-1) =  0.01d0 - ALB_WEI(1,I)
c                        GR_PAR(2,KK+I-1) =  0.1d0 - ALB_WEI(1,I)
c                        GR_PAR(2,KK+I-1) =  0.062d0 - ALB_WEI(1,I)
                     ENDIF
c                     GR_PAR(1,KK+I-1) =  0.0d0 
c                     GR_PAR(2,KK+I-1) =  0.001d0 
                  ENDDO
                  KK = KK + (Nc_ALB-1)
               ENDIF
            ENDIF

         CASE DEFAULT 
            CYCLE
         END SELECT

         KK = KK + 1   
      ENDDO  ! over parameters (II)

      IF( N_RIN > 0 ) THEN
         GR_PAR(1,N_PAR + 1:N_PAR + N_RIN) = -1.0d+20
         GR_PAR(2,N_PAR + 1:N_PAR + N_RIN) = +1.0d+20
      ENDIF

      IF( .NOT.MDS(I_DS)%SHI_FLG(JSEG)) THEN
         CALL ST_TRACK (R_NAME, 2, 'O')
         RETURN
      ENDIF

C     Model spectrum
      WIDE_SHI(1,1) = -0.3d0
      WIDE_SHI(2,1) = 0.3d0
C     Solar spectrum
      WIDE_SHI(1,2) = -0.3d0
      WIDE_SHI(2,2) = 0.3d0
C     Spectral corrections
      IF( N_RIN > 0 ) THEN
         WIDE_SHI(1,2:N_RIN + 2) = -0.3d0
         WIDE_SHI(2,2:N_RIN + 2) = 0.3d0
      ENDIF

      CALL ST_TRACK (R_NAME, 2, 'O')

      RETURN
      END
C***********************************************************************

      SUBROUTINE SET_LIMITS_FLU(N_PAR, N_RIN, WIDE_SHI, GR_PAR)
c     ... Call in "preprocessing.f"

      USE CLOUDS,      ONLY: CLD_LAYER_HEIGHTS
      USE CLOUDS_RETR, ONLY: GT_MIN, GT_MAX, CTH_MIN, CTH_MAX, 
     $                       CBH_MIN, CBH_MAX, GCP
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------
 
      INTEGER  N_PAR, N_RIN
      REAL (KND) GR_PAR(2,N_PAR + N_RIN), WIDE_SHI(2,N_RIN + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SET_LIMITS_FLU'      
      INTEGER  I
      REAL (KND) CTH, CBH, G_T
c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')

      WIDE_SHI = 0.0D0

c     ... Model spectrum
      WIDE_SHI(1,1) = -0.3d0
      WIDE_SHI(2,1) = 0.3d0
c     ... Solar spectrum
      WIDE_SHI(1,2) = -0.3d0
      WIDE_SHI(2,2) =  0.3d0
cc     ... Solar spectrum
c      WIDE_SHI(1,2) = -0.001d0
c      WIDE_SHI(2,2) =  0.001d0
c     ... Spectral corrections
      IF( N_RIN > 0 ) THEN
c         WIDE_SHI(1,2:N_RIN + 2) = -0.001d0
c         WIDE_SHI(2,2:N_RIN + 2) =  0.001d0
         WIDE_SHI(1,2:N_RIN + 2) = -0.3d0
         WIDE_SHI(2,2:N_RIN + 2) =  0.3d0

      ENDIF
      GR_PAR(1,1) = -1.0d+20
      GR_PAR(2,1) = +1.0d+20

      IF( N_RIN > 0 ) THEN
         GR_PAR(1,N_PAR + 1:N_PAR + N_RIN) = -1.0d+20
         GR_PAR(2,N_PAR + 1:N_PAR + N_RIN) = +1.0d+20
      ENDIF

      CALL ST_TRACK (R_NAME, 2, 'O')

      RETURN
      END

C***********************************************************************
