C********************************************************************C
C
C	SCIATRAN - Version 4.5 - Last modification: 12 Dec 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C

C    These routines are used for the output of all weighting functions 
C    --------------------------------------------------------------
c ***********************************************************************
      SUBROUTINE SET_OUTPUT_CHL(JPOS, LOW, WFO, E_W)
c     ... Call in "st_adjoint_o.f"

      USE INFO,        ONLY: WF_DB
      USE RTM_OUTPUT,  ONLY: WF_UNIT, CHL_WEI
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOW_TYPE
      USE WATER,       ONLY: NLW => NLAYERS_UW, DZ => DZ_UW
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) :: E_W
c     ------------------------------
      TYPE (LOW_type) :: LOW    ! Local optical parameters of ocean
c     ----------------------
      TYPE WFO_type             ! Local weighting functions (ocean)
      SEQUENCE
      REAL (KND), POINTER :: WF_ext(:,:), WF_sca(:,:,:)
      END TYPE WFO_type 
      TYPE (WFO_type) :: WFO
c     ----------------------
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_CHL'
      INTEGER     Js, KK
      LOGICAL FLG
      REAL (KND)  OMEG(NLW), Loc_WF(N_SL,NLW)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      FLG = INQ_WF('ocn_chl', 'WFN', KK)
      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)

      FORALL(Js = 1:N_SL) Loc_WF(Js,:) = WFO%WF_ext(:,Js)*OMEG(:)

      IF( WF_DB(KK)%DIM=='rel' ) FORALL(Js = 1:N_SL) Loc_WF(Js,:) = 
c     $     Loc_WF(Js,:)*LOW%ABS_UW(:,2)
c            TEST
     $     Loc_WF(Js,:)*LOW%ABS_UW(:,1)

      CHL_WEI(JPOS,:,:) = CHL_WEI(JPOS,:,:) + Loc_WF(:,:)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_CHL
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_Psca(JPOS, LOW, WFO, E_W)
c     ... Call in "st_adjoint_o.f"

      USE INFO,        ONLY: WF_DB
      USE RTM_OUTPUT,  ONLY: WF_UNIT, RUW_WEI, Pus_WEI
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: OWF_Ray, OWF_Use
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOW_TYPE
      USE WATER,       ONLY: NLW => NLAYERS_UW, DZ => DZ_UW
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) :: E_W
c     ------------------------------
      TYPE (LOW_type) :: LOW    ! Local optical parameters of ocean
c     ----------------------
      TYPE WFO_type             ! Local weighting functions (ocean)
      SEQUENCE
      REAL (KND), POINTER :: WF_ext(:,:), WF_sca(:,:,:)
      END TYPE WFO_type 
      TYPE (WFO_type) :: WFO
c     ----------------------
c     ... Local parameters
      CHARACTER (LEN=15), PARAMETER :: R_NAME = 'SET_OUTPUT_Psca'
      INTEGER     Js, IPM, KK
      LOGICAL FLG
      REAL (KND)  OMEG(NLW), Loc_WF(N_SL,NLW)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      FLG = INQ_WF('ocn_psca', 'WFN', KK)

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)
      
      IPM = 0
      IF( OWF_Ray ) THEN
         IPM = IPM + 1
         FORALL(Js = 1:N_SL) Loc_WF(Js,:) = 
     $        (WFO%WF_ext(:,Js) + WFO%WF_sca(:,Js,IPM))*OMEG(:)
         IF( WF_DB(KK)%DIM=='rel' ) FORALL(Js = 1:N_SL)  
     $        Loc_WF(Js,:) = Loc_WF(Js,:)*LOW%SCR_W(:)
         RUW_WEI(JPOS,:,:) = RUW_WEI(JPOS,:,:) + Loc_WF(:,:)
      ENDIF

      IF( OWF_Use ) THEN
         IPM = IPM + 1
         FORALL(Js = 1:N_SL) Loc_WF(Js,:) = 
     $        (WFO%WF_ext(:,Js) + WFO%WF_sca(:,Js,IPM))*OMEG(:)
         IF( WF_DB(KK)%DIM=='rel' ) FORALL(Js = 1:N_SL)  
     $        Loc_WF(Js,:) = Loc_WF(Js,:)*LOW%HSCA(:,1,1)
         Pus_WEI(JPOS,:,:) = Pus_WEI(JPOS,:,:) + Loc_WF(:,:)
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_Psca
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_CWF(JPOS, LWF, ADD_PLANCK, TAC, E_W )
c     ... Call in "st_adjoint_a.f"
c                 "st_ss_stokes.f"

      USE ABSORBERS,   ONLY: N_GAS, GAS_INDEX
      USE GRIDS,       ONLY: NL => NLAYERS, DZ
      USE INFO,        ONLY: WF_DB, TG_INFO
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: NS_wf
      USE TOOLS,       ONLY: FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, TAC(NL,N_GAS)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_CWF'
      INTEGER    K, Js, Jw, INDG, II, JJ
      REAL (KND)  OMEG(NL), WF_loc(NL,N_SL), TG_WF(N_SL,NL)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)

      DO Jw = 1,NS_wf
         WF_loc(:,:) = LWF%WF_ext(:,:,Jw)
c     ... For first component of the Stokes vector !!!
         IF( ADD_PLANCK ) WF_loc(:,:) = WF_loc(:,:) + LWF%WF_TE(:,:)
         DO II = 1, UBOUND(WF_DB,1)
            IF(WF_DB(II)%TYPE /= 'tg') CYCLE
            JJ = FIND_IDX(WF_DB(II)%NAME,TG_INFO%NAME)
            INDG = GAS_INDEX(JJ)
            FORALL (Js = 1:N_SL) TG_WF(Js,:) = 
     $           WF_loc(:,Js)*TAC(:,INDG)*OMEG(:)
            IF(WF_DB(II)%DIM == 'rel' ) FORALL (Js = 1:N_SL) 
     $              TG_WF(Js,:) = TG_WF(Js,:)*TG_INFO(JJ)%ND(:)
            WF_DB(II)%WF(JPOS,:,:,Jw) = 
     $           WF_DB(II)%WF(JPOS,:,:,Jw) + TG_WF(:,:)
         END DO  ! over all WF (II)
      ENDDO  ! over Stokes component derivatives (Jw)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_CWF
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_TWF
     $     (JPOS, LWF, SCR, TAC, ADD_PLANCK, TWF_ABS, E_W)
c     ... Call in "st_adjoint_a.f"

      USE ABSORBERS,   ONLY: N_GAS, GAS_INDEX
      USE GRIDS,       ONLY: NL => NLAYERS, DZ
      USE INFO,        ONLY: WF_DB, TG_INFO
      USE PROFILES,    ONLY: TEMP
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE ST_VARS,     ONLY: WL => WL_GRID, NS_wf
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE THERMAL_VAR, ONLY: HC_K
      USE TOOLS,       ONLY: INQ_WF, FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, TWF_ABS(NL), SCR(NL), TAC(NL,N_GAS)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_TWF'
      LOGICAL  FLG
      INTEGER    Js, Jw, II, JJ, KK, INDG
      REAL (KND) OMEG(NL), DLB(NL), T_loWF(N_SL,NL,NS_wf),
     $     LOC_AR(NL,N_SL), TG_WF(N_SL,NL)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)      
      T_loWF = 0.d0

      FLG = INQ_WF('temp', 'WFN', II)
      DO Jw = 1,NS_wf
         IF( ADD_PLANCK .AND. Jw == 1) THEN 
c           ... For first component of the Stokes vector !!!
            DLB(:) = DEXP( HC_K/(WL(JPOS)*1.0D-9)/TEMP(:) )
c           ... Derivative of the Planck function logarithm w.r.t. temperature
c               Delta_Planck/Planck = [D (log Planck)/D Temp]*Delta_Temp
            DLB(:) = DLB(:)*HC_K/(WL(JPOS)*1.0D-9)/TEMP(:)**2
     $           /(DLB(:) - 1.d0)
            FORALL( Js = 1:N_SL ) T_loWF(Js,:,1) = 
     $           ( (LWF%WF_ext(:,Js,1) + LWF%WF_TE(:,Js))*TWF_ABS(:)
     $           + LWF%WF_TE_TEM(:,Js)*DLB(:) )*OMEG(:)
         ELSE
c           ... Contribution of the gas absorption cross-section to the 
c               temperature weighting function (for absolute variation)
            FORALL( Js = 1:N_SL ) T_loWF(Js,:,Jw)  = 
     $          LWF%WF_ext(:,Js,Jw)*TWF_ABS(:)*OMEG(:)
         ENDIF
c        ... Contribution of the Rayleigh scattering coefficient to the 
c            temperature weighting function (for absolute variation)
         FORALL(Js = 1:N_SL) LOC_AR(:,Js) =  
     $        (LWF%WF_ext(:,Js,Jw) + LWF%WF_scr(:,Js,Jw))*OMEG(:)

c        ... Contribution of absorption gas number density variation 
c             to the temperature WF (for absolute variation)
         TG_WF = 0.d0
c         DO KK = 1, UBOUND(TG_INFO,1)
c            IF( .NOT.TG_INFO(KK)%FM_FLG ) CYCLE
c            IF( TG_INFO(KK)%WF_FLG ) CYCLE
c           INDG = GAS_INDEX(KK)
c            FORALL (Js = 1:N_SL) TG_WF(Js,:) = TG_WF(Js,:) 
c     $           - LWF%WF_ext(:,Js,Jw)*TAC(:,INDG)*OMEG(:)
c     $           *TG_INFO(KK)%ND(:)/TEMP(:)
c         END DO  ! over all absorber (KK)
         FORALL(Js = 1:N_SL) WF_DB(II)%WF(JPOS,Js,:,Jw) = 
     $        WF_DB(II)%WF(JPOS,Js,:,Jw) + T_loWF(Js,:,Jw)
     $        - LOC_AR(:,Js)*SCR(:)/TEMP(:) + TG_WF(Js,:) 
      ENDDO   ! over Stokes component derivatives (Jw)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_TWF
c  ***********************************************************************

      SUBROUTINE SET_OUTPUT_PWF
     $     (JPOS, LWF, SCR, TAC, ADD_PLANCK, PWF_ABS, E_W)
c     ... Call in "st_adjoint_a.f"

      USE ABSORBERS,  ONLY: N_GAS, GAS_INDEX
      USE GRIDS,      ONLY: NL => NLAYERS, DZ
      USE INFO,       ONLY: WF_DB, TG_INFO
      USE RTM_OUTPUT, ONLY: WF_UNIT
      USE SPHER_PARS, ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,    ONLY: NS_wf
      USE TOOLS,      ONLY: INQ_WF
      USE TYPE_VARS,  ONLY: KND
      USE TYPES,      ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, PWF_ABS(NL), SCR(NL), TAC(NL,N_GAS)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_PWF'
      LOGICAL  FLG
      INTEGER  Js, II, Jw, JJ, KK, INDG
      REAL (KND) OMEG(NL), P_WF(N_SL,NL,NS_wf), LOC_AR(NL,N_SL),
     $     TG_WF(N_SL,NL)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)

      FLG = INQ_WF('press', 'WFN', II)
      DO Jw = 1,NS_wf
c        ... Contribution of the Rayleigh scattering coefficient to the 
c            pressure WF (for relative variation always)
         FORALL(Js = 1:N_SL) LOC_AR(:,Js) =  
     $        (LWF%WF_ext(:,Js,Jw) + LWF%WF_scr(:,Js,Jw))*OMEG(:)

         FORALL( Js = 1:N_SL ) P_WF(Js,:,Jw) = LWF%WF_ext(:,Js,Jw)
         IF( ADD_PLANCK .AND. Jw == 1 ) THEN 
            FORALL( Js = 1:N_SL ) 
     $           P_WF(Js,:,1) = P_WF(Js,:,1) + LWF%WF_TE(:,Js)
         ENDIF
         FORALL( Js = 1:N_SL ) P_WF(Js,:,Jw) = 
     $        P_WF(Js,:,Jw)*PWF_ABS(:)*OMEG(:)

c        ... Contribution of absorption gas number density variation 
c             to the pressure WF (for absolute variation??)
         TG_WF = 0.d0
c         DO KK = 1, UBOUND(TG_INFO,1)
c            IF( .NOT.TG_INFO(KK)%FM_FLG ) CYCLE
c            IF( TG_INFO(KK)%WF_FLG ) CYCLE
c            INDG = GAS_INDEX(KK)
c            FORALL (Js = 1:N_SL) TG_WF(Js,:) = TG_WF(Js,:) 
c     $           + LWF%WF_ext(:,Js,Jw)*TAC(:,INDG)*OMEG(:)
c     $           *TG_INFO(KK)%ND(:)
c         END DO  ! over all absorber (KK)
c        ...............................................................
         FORALL( Js = 1:N_SL ) WF_DB(II)%WF(JPOS,Js,:,Jw) = 
     $        WF_DB(II)%WF(JPOS,Js,:,Jw) + P_WF(Js,:,Jw)
     $        + LOC_AR(:,Js)*SCR(:) + TG_WF(Js,:)
      END DO  ! over WFs for Stokes components (Jw)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_PWF

c ***********************************************************************
      SUBROUTINE SET_OUTPUT_RAY(JPOS, LWF, SCR, E_W)
c     ... Call in "st_adjoint_a.f", "st_ss_stokes.f", "gpp_ss_stokes.f"

      USE GRIDS,       ONLY: NL => NLAYERS, DZ
      USE INFO,        ONLY: WF_DB
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: NS_wf
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) :: E_W, SCR(NL)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_RAY'
      INTEGER  Js, Jw, II
      LOGICAL FLG
      REAL (KND)  LOC_AR(N_SL,NL), OMEG(NL)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
 
      IF(INQ_WF('anps', 'WFN', II)) THEN
         FORALL(Js=1:N_SL) WF_DB(II)%WF(JPOS,Js,:,1) = 
     $        WF_DB(II)%WF(JPOS,Js,:,1) + LWF%WF_scr(:,Js,1)*SCR(:)*E_W
      ENDIF

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)
      FLG = INQ_WF('ray', 'WFN', II)
      IF( WF_DB(II)%DIM=='rel' ) OMEG(:) = OMEG(:)*SCR(:)
      DO Jw = 1,NS_wf
         FORALL(Js = 1:N_SL) LOC_AR(Js,:) =  
     $        (LWF%WF_ext(:,Js,Jw) + LWF%WF_scr(:,Js,Jw))*OMEG(:)
         WF_DB(II)%WF(JPOS,:,:,Jw) = 
     $        WF_DB(II)%WF(JPOS,:,:,Jw) + LOC_AR(:,:)
      ENDDO   ! over Stokes component derivatives (Jw)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_RAY
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_MET(JPOS, LWF, ECM, E_W)
c     ... Call in "st_adjoint_a.f", "st_ss_stokes.f", "gpp_ss_stokes.f"

      USE GRIDS,       ONLY: NL => NLAYERS, DZ
      USE INFO,        ONLY: WF_DB, TG_INFO
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: NS_wf
      USE TOOLS,       ONLY: INQ_WF, FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS
      REAL (KND), INTENT(IN) :: E_W, ECM(NL)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_MET'
      INTEGER  Js, Jw, II, JJ
      LOGICAL  FLG
      REAL (KND)  OMEG(NL)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)
      FLG = INQ_WF('atom', 'WFT', II)   
      JJ = FIND_IDX(WF_DB(II)%NAME,TG_INFO%NAME)
      OMEG(:) = OMEG(:)*ECM(:)
      IF( WF_DB(II)%DIM == 'rel' ) OMEG(:) = OMEG(:)*TG_INFO(JJ)%ND(:)
      DO Jw = 1,NS_wf
         FORALL(Js = 1:N_SL) WF_DB(II)%WF(JPOS,Js,:,Jw) =       
     $        WF_DB(II)%WF(JPOS,Js,:,Jw) 
     $        + (LWF%WF_ext(:,Js,Jw) + LWF%WF_met(:,Js,Jw))*OMEG(:) 
      ENDDO  ! over Stokes component derivatives (Jw)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_AER(JPOS, LWF, SCA, EXA, ADD_PLANCK, E_W, 
     $     POS)
c     ... Call in "st_adjoint_a.f"

      USE AER_MIE,     ONLY: N_01
      USE AEROSOLS,    ONLY: Dwr_mi, N_dwr, Plus_mi, AER_DER, NAW_MI, 
     $                       AW_MI, NAW => N_AER_WAVS, AER_WAVS, AP_Mie,  
     $                       Mie_layer, R_erw
      USE GRIDS,       ONLY: NL => NLAYERS, DZ, ZZZ
      USE INFO,        ONLY: RET_INFO, WF_DB
      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: WF_UNIT, MAX_AER_WF
      USE RTM_PARS,    ONLY: DATA_ITER
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: NS_wf
      USE TOOLS,       ONLY: SET_PTR, INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------
      INTEGER,    INTENT(IN) :: JPOS 
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, POS, SCA(NL), EXA(NL)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_AER'
      LOGICAL     FLG
      INTEGER     I, Js, Jw, Wm1, Wm2, N_AWV, Idr, II, KK
      REAL (KND)  Dmi, OMEG(NL), AER_WF(N_SL,NL,MAX_AER_WF,Nvs),
     $     Sca_WF(NL,N_SL,NS_wf), Abs_WF(NL,N_SL,NS_wf)
      REAL (KND), POINTER :: AWV(:)
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND), ALLOCATABLE :: D_sca(:), D_ext(:), S_E(:), A_E(:), 
     $     HEL(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      OMEG(:) = E_W
      IF( TRIM(WF_UNIT) == 'unified' ) OMEG(:) = OMEG(:)/DZ(:)

      AER_WF = 0.d0
c     .... Aerosol WF for absolute variation of scattering coefficient
      FORALL( Js=1:N_SL,Jw=1:NS_wf ) AER_WF(Js,:,1,Jw) = 
     $     (LWF%WF_ext(:,Js,Jw) + LWF%WF_sca(:,Js,Jw,1))*OMEG(:)
  
c     .... Aerosol WF for absolute variation of absorption coefficient
      FORALL( Js=1:N_SL, Jw=1:NS_wf )
     $     AER_WF(Js,:,2,Jw) = LWF%WF_ext(:,Js,Jw)*OMEG(:)

      IF( ADD_PLANCK ) THEN 
c        ... Implemented for first component of Stokes vector only !!!
         FORALL(Js = 1:N_SL) AER_WF(Js,:,2,1) 
     $        = AER_WF(Js,:,2,1) + LWF%WF_TE(:,Js)*OMEG(:)
      ENDIF

c     IF( Plus_mi ) THEN
c      ... It was changed to calculate WF for relative variation of 
c          aerosol number density for the Mie case if there are not 
c          derivatives with respect to parameters of PSD function
      FLG = INQ_WF('aer-dens', 'WFN', II)
c      WRITE(*,*) '  FLG  =  ', FLG
c      WRITE(*,*) '  Dwr_mi =  ', Dwr_mi
c      WRITE(*,*) '  II =  ', II

      IF( Dwr_mi.OR.II > 0 ) THEN
c         WRITE(*,*) '  Dwr_mi OR II ', Dwr_mi, II, N_dwr
c        ... Derivatives on the parameters of PSD function
c         WRITE(*,*) '   - 1'
         FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
c          ... Abs. variation of absorption coefficient
            Abs_WF(:,Js,Jw) =   LWF%WF_ext(:,Js,Jw)*OMEG(:)
c          ... Abs. variation of scattering coefficient
           Sca_WF(:,Js,Jw) = ( LWF%WF_ext(:,Js,Jw) 
     $                       + LWF%WF_sca(:,Js,Jw,1) )*OMEG(:)           
         END FORALL

         AER_WF = 0.d0
c        !! Mie_layer is allocated in "mie_aer" mode only !!
         IF( Dwr_mi ) THEN
            DO Js = 1, N_SL
               DO Jw = 1, Ns_wf
                  WHERE( .NOT.Mie_layer(:) ) 
                     Abs_WF(:,Js,Jw) = 0.d0  ;  Sca_WF(:,Js,Jw) = 0.d0
                  END WHERE
               ENDDO
            ENDDO
         ENDIF

c        ... WF for relative variation of aerosol number density
c        !! M_01 was installed in "mie_aer" mode only !!
c        IF( N_01 == 1 ) THEN
         IF( FLG ) THEN
            FORALL( Js=1:N_SL,Jw=1:NS_wf )  AER_WF(Js,:,1,Jw) = 
     $           Sca_WF(:,Js,Jw)*SCA(:)+Abs_WF(:,Js,Jw)*(EXA(:)-SCA(:))
         ENDIF

         IF( Plus_mi ) THEN
            N_AWV = NAW_MI  ;  AWV => AW_MI
         ELSE
c            WRITE(*,*) '  .not.Plus_mi  ', N_AER_WAVS
            N_AWV = NAW  ;  AWV => AER_WAVS
         ENDIF

         ALLOCATE( D_sca(NL), D_ext(NL) )

c        ... over derivative of PSD parameters if any
         DO Idr = 1,N_dwr
            KK = N_01 + Idr
            CALL LIN_SEARCH_NEW(POS, Dmi, Wm1, Wm2, N_AWV, AWV)
c           ... Interpolate derivative d_sca/d_rmod
            D_sca(:) =  Dmi*AER_DER(1,1:NL,Wm2,Idr) + 
     $           (1.d0-Dmi)*AER_DER(1,1:NL,Wm1,Idr)
            
c           ... Interpolate derivative d_ext/d_rmod
            D_ext(:) =  Dmi*AER_DER(2,1:NL,Wm2,Idr) + 
     $           (1.d0-Dmi)*AER_DER(2,1:NL,Wm1,Idr)

cc            ... Phase matrix only
c            FORALL( Js=1:N_SL, Jw=1:NS_wf ) AER_WF(Js,:,KK,Jw) 
c     $           = LWF%WF_sca(:,Js,Jw,1+Idr)*OMEG(:) *AP_Mie(:,KK)

cc           ... Scattering and extinction coefficients without 
cc               contribution of the phase matrix
c            FORALL( Js=1:N_SL, Jw=1:NS_wf ) AER_WF(Js,:,KK,Jw) 
c     $           = ( Sca_WF(:,Js,Jw)*D_sca(:) 
c     $             + Abs_WF(:,Js,Jw)*(D_ext(:) - D_sca(:)) )
c     $                     *AP_Mie(:,KK)
c
            
            FORALL( Js=1:N_SL,Jw=1:NS_wf ) AER_WF(Js,:,KK,Jw) = 
c                ... Contribution of variation:
c                    - SCA coef caused by variation of PSD parameters
     $           ( Sca_WF(:,Js,Jw)*D_sca(:)  
c                    - ABS coef caused by variation of PSD parameters
     $           + Abs_WF(:,Js,Jw)*(D_ext(:) - D_sca(:))
c                    - Scat matr caused by variation of PSD parameters
     $           + LWF%WF_sca(:,Js,Jw,1+Idr)*OMEG(:) )
c                 **** calculation in relative variation moved below ****
cc                          mean profile of PSD parameter
c     $                    *AP_Mie(:,KK) !/MAXVAL( AP_Mie(:,KK) )
c
c                   WRITE(*,*) '    Out1 AER_WF for index  ', 
c     $                  KK, MAXVAL( AP_Mie(:,KK) )
c                   WRITE(*,*) KK, Sca_WF(:,1,1)*D_sca(:) 
c     $                  + Abs_WF(:,1,1)*(D_ext(:) - D_sca(:))
c     $                  + LWF%WF_sca(:,1,1,1+Idr)*OMEG(:) 
         ENDDO  ! over derivatives w.r.t. PSD function (Idr,KK) 
      ENDIF  ! Dwr_mi OR aer-dens

      KK = 0
      DO II = 1, UBOUND(WF_DB,1)
c         IF( .NOT.RET_INFO(II)%STAT ) CYCLE
c         WRITE(*,*)  II, WF_DB(II)%NAME

         CALL SET_PTR( RTM_PTR, WF_DB(II)%NAME)

         SELECT CASE (WF_DB(II)%NAME)
         CASE ('aer-sca')
c            WRITE(*,*)  '  WF_DB(II)%DIM = ', WF_DB(II)%DIM
            IF(WF_DB(II)%DIM == 'rel') FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
     $           AER_WF(Js,:,1,Jw) = AER_WF(Js,:,1,Jw)*SCA(:)
            WF_DB(II)%WF(JPOS,:,:,:) = 
     $           WF_DB(II)%WF(JPOS,:,:,:) + AER_WF(:,:,1,:)

         CASE ('aer-abs')
            IF(WF_DB(II)%DIM == 'rel') FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
     $           AER_WF(Js,:,2,Jw) = AER_WF(Js,:,2,Jw)*(EXA(:)-SCA(:))
           WF_DB(II)%WF(JPOS,:,:,:) = 
     $           WF_DB(II)%WF(JPOS,:,:,:) + AER_WF(:,:,2,:)

         CASE ('aer-ext','aer-erw')
            ALLOCATE( S_E(NL),A_E(NL),HEL(NL) )
            S_E(:) = SCA(:) ; A_E(:) = EXA(:)-SCA(:) 
            HEL(:) = 1.d0
            IF( WF_DB(II)%DIM == 'abs' ) THEN
               WHERE( EXA(:) > 0.d0 ) 
                  S_E(:) = SCA(:)/EXA(:)
                  A_E(:) = ( EXA(:)-SCA(:) )/EXA(:)
               END WHERE
               IF( WF_DB(II)%NAME=='aer-erw' ) THEN
                  CALL LIN_SEARCH_NEW(POS, Dmi, Wm1, Wm2, NAW, AER_WAVS)
                  HEL(:) = 
     $                 Dmi*R_erw(:,Wm2) + (1.d0-Dmi)*R_erw(:,Wm1)
               ENDIF
            ENDIF

            FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
               AER_WF(Js,:,1,Jw) = AER_WF(Js,:,1,Jw)*S_E(:)*HEL(:)
               AER_WF(Js,:,2,Jw) = AER_WF(Js,:,2,Jw)*A_E(:)*HEL(:)
            END FORALL            

            WF_DB(II)%WF(JPOS,:,:,:) = WF_DB(II)%WF(JPOS,:,:,:)
     $           + AER_WF(:,:,1,:) + AER_WF(:,:,2,:)

         CASE ('aer-dens')
            KK = KK + 1
c           ... WF for aerosol density is calculated above for relative
c               variation
c                         WRITE(*,*) '   WF aer-dens KK =  ',KK
c                         WRITE(*,*) '   WF_DB(II)%DIM = ', WF_DB(II)%DIM
            IF(WF_DB(II)%DIM == 'abs') THEN 
               DO Js = 1, N_SL
                  DO Jw = 1, Ns_wf
c                    WHERE( AP_Mie(:,KK) > 0.d0 ) 
c                    AER_WF(Js,:,KK,Jw)=AER_WF(Js,:,KK,Jw)/AP_Mie(:,KK) 
                     WHERE( RTM_PTR%CRN(:)>0.d0 ) 
                        AER_WF(Js,:,KK,Jw) = 
     $                       AER_WF(Js,:,KK,Jw)/RTM_PTR%CRN(:) 
                     END WHERE
                  ENDDO
               ENDDO
            ENDIF
            WF_DB(II)%WF(JPOS,:,:,:) = WF_DB(II)%WF(JPOS,:,:,:) 
     $           + AER_WF(:,:,KK,:)

         CASE ('aer-modr_smod','aer-var_smod')
            KK = KK + 1
c                           WRITE(*,*) '   Modal radius: ', WF_DB(II)%DIM 
c                           WRITE(*,*) RTM_PTR%CRN(:)
            IF(WF_DB(II)%DIM == 'rel') FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
     $           AER_WF(Js,:,KK,Jw) = AER_WF(Js,:,KK,Jw)*RTM_PTR%CRN(:)  
c     $           AER_WF(Js,:,KK,Jw) = AER_WF(Js,:,KK,Jw)*AP_Mie(:,KK)
            WF_DB(II)%WF(JPOS,:,:,:) = 
     $           WF_DB(II)%WF(JPOS,:,:,:) + AER_WF(:,:,KK,:)

         CASE ('aer-modr_f','aer-modr_c','aer-var_f','aer-var_c',
     $           'aer-cm_frac')
            KK = KK + 1
            IF(WF_DB(II)%DIM == 'rel') FORALL( Js=1:N_SL, Jw=1:NS_wf ) 
     $           AER_WF(Js,:,KK,Jw) = AER_WF(Js,:,KK,Jw)*RTM_PTR%CRN(:)
            WF_DB(II)%WF(JPOS,:,:,:) = 
     $           WF_DB(II)%WF(JPOS,:,:,:) + AER_WF(:,:,KK,:)

        CASE ('anps')
            FORALL(Js=1:N_SL) WF_DB(II)%WF(JPOS,Js,1,:) = 
     $           WF_DB(II)%WF(JPOS,Js,1,:) + 
     $           LWF%WF_sca(:,Js,1,1)*SCA(:)*E_W
         END SELECT
      END DO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
c ***********************************************************************

      SUBROUTINE INTERPOL_DRIV(Cil, Cir, DP_CLD, Nco, ABS_DR, SCC_DR)
c     ... Call in "set_output_cld.f"
c                 "set_output_mix.f"

      USE CLOUDS,      ONLY: CLD_LD => CLD_LAYER_DERIV, N_CLD_LAYERS, 
     $                       J_TOP, J_BOT 
      USE GRIDS,       ONLY: NL => NLAYERS
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      INTEGER     Cil, Cir, Nco
      REAL (KND)  DP_CLD
      REAL (KND), DIMENSION (NL):: ABS_DR, SCC_DR
c     ... Local variables
      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'INTERPOL_DRIV'
      INTEGER     I, Jt, Jb, NSL
      REAL (KND), POINTER :: A1_p(:), A2_p(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      ABS_DR = 0.0d0 ; SCC_DR = 0.0d0

      DO I = 1, N_CLD_LAYERS
         Jt = J_TOP(I) ; Jb = J_BOT(I) ; NSL = Jb - Jt + 1
         A1_p => CLD_LD(I,Cil,Nco,:) ; A2_p => CLD_LD(I,Cir,Nco,:)
c        ... Interpolate derivative of the absorption coefficient
c            w.r.t. effective radius on the wavelength
         ABS_DR(Jt:Jb) = A1_p(1:NSL)*(1.d0-DP_CLD) + A2_p(1:NSL)*DP_CLD
c        --------------------------------------------------------------
         A1_p => CLD_LD(I,Cil,Nco+1,:) ; A2_p => CLD_LD(I,Cir,Nco+1,:)
c        ... Interpolate derivative of the scattering coefficient
c            w.r.t. effective radius on the wavelength
         SCC_DR(Jt:Jb) = A1_p(1:NSL)*(1.d0-DP_CLD) + A2_p(1:NSL)*DP_CLD
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE INTERPOL_DRIV
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_CLD(JPOS, LWF, Cil, Cir, DP_CLD, EXC, SCC,  
     $     DM_CLD, ASO, ADD_PLANCK, E_W, SCC_NS)
c     ... Call in "st_adjoint_a.f"

      USE CLOUDS,      ONLY: TAU_CLD_INP, N_CLD_LAYERS, J_TOP, J_BOT, 
     $                       LWP, IWP, Prof_Cpar, Var_type, 
     $                       Prof_Ref
      USE CLOUD_INPUT, ONLY: ADD_ABS, TDS, Deriv_Phc, Nphr, Ref_DB, 
     $                       N_ref_db, Plus_ice, N_ref_ice, Ref_ice
      USE GRIDS,       ONLY: NL => NLAYERS, DZ, ZZZ, DZR
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: NL_soot, Nm_ice
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: WmsF => WF_MS_FLAG
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS, Cil, Cir
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, DP_CLD, SCC(NL), EXC(NL), 
     $     DM_CLD(N_CLD_LAYERS,Nphr,Nm_ice), SCC_NS(NL)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
c     ------------------------------------------------------------------
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_CLD'
      CHARACTER (LEN=10) C_HEL
      INTEGER    I, K, II, KK, Js, JT, JB, NCL, Nco, Ws, Ws1, IVT,
     $     LL, N_db
      REAL (KND) Cons_tau, DvT, DvB, Tes, DUM, OU1(NL,N_SL), 
     $     OU2(NL,N_SL), ASO(NL_soot), DM_DRe(NL), Pnor, T_ref
      REAL (KND), DIMENSION (NL) :: ABS_DR, XAR, Dos, Doa, SCC_DR
      REAL (KND), ALLOCATABLE :: CLD_WF(:,:), GPT_WF(:), 
     $     ALD(:), RtoA(:), Ou3(:,:), DsDz(:), DaDz(:)
      REAL (KND), POINTER :: DT1(:), DT2(:), Rdb(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      DT1 => LWF%DT1  ;  DT2 => LWF%DT2
 
      IF( INQ_WF('anps','WFN',II) ) 
     $     FORALL(Js=1:N_SL) WF_DB(II)%WF(JPOS,Js,:,1) = 
     $     WF_DB(II)%WF(JPOS,Js,:,1) + LWF%WF_scc(:,Js,1)*SCC(:)*E_W

c     ... WF for cloud geometrical parameters and optical thickness:
c         It is implemented for single solar and line-of-sight angle only
      IF( INQ_WF('cld-s', 'WFT') ) THEN

         ALLOCATE( GPT_WF(N_SL), DsDz(NL), DaDz(NL) )

         DO NCL = 1, N_CLD_LAYERS
c           ... Shift for next cloud layer
            IVT = 1 
            IF( Plus_ice(NCL).AND.TDS(NCL)%S/='mixed' ) IVT = 2

            Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
c           ... Derivative Dsca/Dz 
            CALL DCOF_DZ(SCC, ZZZ, DsDz, NL, Jt, Jb)
            Dos(:) = EXC(:) - SCC(:)
c           ... Derivative Dabs/Dz 
            CALL DCOF_DZ(Dos, ZZZ, DaDz, NL, Jt, Jb)

            Jt = J_TOP(NCL) ; Jb = J_BOT(NCL) 
            SELECT CASE ( Var_type(NCL,1)%S )
            CASE('IWC','LWC','PNC','CNC')
               Cons_tau = 0.d0
            CASE DEFAULT
               Cons_tau = 1.d0/( ZZZ(Jt)-ZZZ(Jb) )
            END SELECT
c           ... Multiplier (Z - Zb)/L, where L = Zt - Zb
            Xar(Jt:Jb) = (ZZZ(Jt:Jb)-ZZZ(Jb))/(ZZZ(Jt) - ZZZ(Jb))
c           ... Cloud top
            Dos(Jt:Jb) = ( -Xar(Jt:Jb)*DsDz(Jt:Jb)-Cons_tau )*SCC(Jt:Jb)
            Doa(Jt:Jb) = ( -Xar(Jt:Jb)*DaDz(Jt:Jb)
     $           - Cons_tau )*(EXC(Jt:Jb)-SCC(Jt:Jb))
            DvT = 0.d0 
            DO I = J_TOP(NCL), J_BOT(NCL)-1
               DvT = DvT + 
     $              ( (DT1(I)  +DT2(I)  )*Dos(I)   + DT1(I)  *Doa(I)
     $              + (DT1(I+1)+DT2(I+1))*Dos(I+1) + DT1(I+1)*Doa(I+1) )
     $              *0.5D0*DZR(I)
            ENDDO
c           ... Cloud bottom
            Dos(Jt:Jb) = ( -(1.0d0 - Xar(Jt:Jb))*DsDz(Jt:Jb)
     $           + Cons_tau )*SCC(Jt:Jb)
            Doa(Jt:Jb) = ( -(1.0d0 - Xar(Jt:Jb))*DaDz(Jt:Jb)
     $           + Cons_tau )*(EXC(Jt:Jb)-SCC(Jt:Jb))
            DvB = 0.d0
            DO I = J_TOP(NCL), J_BOT(NCL)-1
               Dvb = Dvb + ( (DT1(I) + DT2(I))*Dos(I) + DT1(I)*Doa(I)
     $              + (DT1(I+1)+DT2(I+1))*Dos(I+1) + DT1(I+1)*Doa(I+1) )
     $              *0.5D0*DZR(I)
            ENDDO
            
c           First infinitesimal cloud layer is not used for calculation
c           -----------------------------------------------------------
            Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
c           ... Over all required parameters (CTH, CBH, Tau) 
            DO K = 1, UBOUND(WF_DB,1)
               SELECT CASE( WF_DB(K)%NAME )
               CASE DEFAULT
                  CYCLE
               CASE( 'cld-cth' )
                  Tes = DT1(Jt)*EXC(Jt) + DT2(Jt)*SCC(Jt) 
                  GPT_WF(:) = (Tes + DvT)
               CASE( 'cld-cbh' )
                  Tes = DT1(Jb)*EXC(Jb) + DT2(Jb)*SCC(Jb)
                  GPT_WF(:) = (-Tes + DvB)
               CASE( 'cld-tau_ice')
                  GPT_WF(:) = 0.d0
               CASE( 'cld-tau_nomix','cld-tau_water')
                  Tes = 0.d0
                  DO I = Jt-1,Jb
                     Tes = Tes + 
     $                    ( DT1(I)  *EXC(I)  + DT2(I)  *SCC(I)
     $                    + DT1(I+1)*EXC(I+1)+ DT2(I+1)*SCC(I+1) )
     $                    *0.5D0*DZR(I)
                  ENDDO
                  Pnor = 1.0d0
c                 ... For the absolute variation of the opt. thickness
                  IF(WF_DB(K)%DIM=='abs' ) Pnor = Tau_cld_inp(NCL,IVT)
                  GPT_WF(:) = Tes/Pnor
c                  DO Js = 1,N_SL
c                     GPT_WF(Js) = Tes/Tau_cld_inp(NCL,IVT)
c                  ENDDO
               END SELECT
c              ... II, K - shift for cloud layer and "scalar" parameters
               WF_DB(K)%WF(JPOS,:,NCL,1) = WF_DB(K)%WF(JPOS,:,NCL,1) 
     $              + GPT_WF(:)*E_W
            ENDDO  ! over all "scalar" parameters (K)
         ENDDO  ! over cloud layers (NCL)
      ENDIF  ! cloud "scalar" parameters                

c     ... Return if no weighting function for parameter-function required
c         ---------------------------------------------------------------
c                  ---------------------------------------------
      IF( .NOT.INQ_WF('cld-v', 'WFT') ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      END IF

      ALLOCATE( CLD_WF(N_SL,NL), ALD(NL), RtoA(NL) )

c     ... For first component of the Stokes vector !!!
      Ou2(:,:) =  LWF%WF_ext(:,:,1)*E_W
      IF( ADD_PLANCK ) Ou2(:,:) = Ou2(:,:) + LWF%WF_TE(:,:)*E_W

      Ou1(:,:) = (LWF%WF_ext(:,:,1) + LWF%WF_scc(:,:,1))*E_W

c     ... All auxiliary function are written for relative variation
c         of scattering and absorption coefficients
      ALD(:) = EXC(:) - SCC(:)

c     ... There is an additional absorber within cloud/snow layer
      IF( ANY(ADD_ABS) ) THEN
         ALLOCATE( Ou3(NL,N_SL) )
c        ... ASO is soot absorption coefficient
         FORALL( Js=1:N_SL ) Ou3(:,Js) = Ou2(:,Js)*ASO(:)
         ALD(:) = ALD(:) - ASO(:)
      END IF

      FORALL( Js=1:N_SL ) 
c        ... WF for the relative variation of scattering coefficient
         Ou1(:,Js) = Ou1(:,Js)*SCC(:)
c        ... WF for the relative variation of absorption coefficient
         Ou2(:,Js) = Ou2(:,Js)*ALD(:)
      END FORALL

c      DO K = 1, NL
c         WRITE(*,'(1x,e17.8,*(2x,e12.4))')  ZZZ(K), Ou1(K,1), SCC(K),
c     $        Ou2(K,1), ALD(K)
c      ENDDO

c     ... Do-loop over parameter-function WF if present
c     -------------------------------------------------
      DO I = 1, UBOUND(WF_DB,1)
         LL = LEN_TRIM(WF_DB(I)%NAME)

c         WRITE(*,*) '  WF_DB(I)%TYPE ', WF_DB(I)%TYPE
c         WRITE(*,*) '  WF_DB(I)%NAME(LL - 2:LL) ',
c     $        WF_DB(I)%NAME(LL - 2:LL)
         
         IF( WF_DB(I)%TYPE/='cld-v' ) CYCLE
         IF(WF_DB(I)%NAME(LL - 2:LL) == 'ice') CYCLE
         RtoA(:) = 1.d0 ; ; CLD_WF = 0.d0
         SELECT CASE (WF_DB(I)%NAME)
c        ... Scattering coefficient
         CASE('cld-sca_nomix','cld-sca_water')
c        ---------------------------------------------------------------
            IF(WF_DB(I)%DIM=='abs' ) 
     $           FORALL( K=1:NL, SCC(K)/=0.d0 ) RtoA(K) = 1.d0/SCC(K)
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,Js)

c        ... Absorption coefficient
         CASE('cld-abs_nomix','cld-abs_water')
c        ---------------------------------------------------------------
            IF( WF_DB(I)%DIM=='abs' ) 
     $           FORALL( K=1:NL, ALD(K)/=0.d0 ) RtoA(K) = 1.d0/ALD(K)
            FORALL( JS=1:N_SL ) CLD_WF(Js,:) = Ou2(:,JS)

c        ... Absorption coefficient of soot
         CASE('cld-soot')
c        ---------------------------------------------------------------
            FORALL( JS=1:N_SL ) CLD_WF(Js,:) = Ou3(:,JS)
            IF( WF_DB(I)%DIM=='abs' ) 
     $           FORALL( K=1:NL, ASO(K)/=0.d0 ) RtoA(K) = 1.d0/ASO(K)

c        ... Liquid water content and particle number density
         CASE('cld-lwc_nomix','cld-lwc_water','cld-pnc_nomix',
     $           'cld-pnc_water')  
c        ---------------------------------------------------------------
            IF( WF_DB(I)%DIM=='abs' ) FORALL( K=1:NL, SCC(K)/=0.d0 ) 
     $           RtoA(K) = 1.d0/Prof_Cpar(K,1)
c           ... Standard representation 
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,Js) + Ou2(:,Js)

c           ... If "LWP"/"IWP" or "TAU" option is used
            DO NCL = 1, N_CLD_LAYERS
               Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
               SELECT CASE( Var_type(NCL,1)%S )
c              ... See second term in Eq.(WFlwc_LWP) in STB
c                  Prof_Cpar(Jt:Jb,1) = LWC(z)/1000
               CASE('LWP')
                  Dos(Jt:Jb) = Prof_Cpar(Jt:Jb,1)*0.5d0*DZ(Jt:Jb)*1.d3
     $                 /LWP(NCL)
               CASE('IWP')
                  Dos(Jt:Jb) = Prof_Cpar(Jt:Jb,1)*0.5d0*DZ(Jt:Jb)*1.d3
     $                 /IWP(NCL)
               CASE('TAU')
c                 ... See second term in Eq.(WFlwc_tau) in STB
c                     Prof_Ref(Jt:Jb,1) = ext(z)/tau_c
                  Dos(Jt:Jb) = Prof_Ref(Jt:Jb,1)*0.5d0*DZ(Jt:Jb) 
               CASE DEFAULT
                  CYCLE
               END SELECT
c              ... LWC weighting function under assumption of
c                  constant LWP   Eq.(WFlwc_LWP) in STB
c                  constant tau_c Eq.(WFlwc_tau) in STB
               FORALL(Js = 1:N_SL) 
                 CLD_WF(Js,Jt:Jb) = CLD_WF(Js,Jt:Jb)
     $              - SUM(Ou1(Jt:Jb,Js) + Ou2(Jt:Jb,Js))*Dos(Jt:Jb) 
               END FORALL
            ENDDO  ! over cloud layers ('lwc','pnc') (Ncl)

c        ... Effective radius of water droplets
         CASE('cld-ref_nomix','cld-ref_water')
c        ---------------------------------------------------------------

            IF( WF_DB(I)%DIM=='abs' ) FORALL( K=1:NL, SCC(K)/=0.d0 ) 
     $           RtoA(K) = 1.d0/Prof_Cpar(K,2)
c
c            DO K = 1, NL
c               IF( SCC(K)/=0.d0 ) THEN
c                  WRITE(*,'(4x,i3,2x,f8.3)') K, Prof_Cpar(K,2)
c               ENDIF
c            ENDDO
c
c            
            Nco = 1 
c           ... Wavelength interpolation of derivatives:
c               SCC_DR - Dsca/Dref*(Ref/sca) ; ABS_DR - Dabs/Dref*(Ref/abs) 
            CALL INTERPOL_DRIV(Cil, Cir, DP_CLD, Nco, ABS_DR, SCC_DR)
c           ... Weighting function of Ref for a pair (LWC,Ref) 
c               (see Eq.(WFr_LWC-R) in STB)
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,Js)*SCC_DR(:) 
     $           + Ou2(:,Js)*ABS_DR(:)
c            write(*,*) '  Deriv_Phc  -------- ', Deriv_Phc 
c           ... Contribution of the phase function derivative w.r.t.
c               R_eff (corresponds to the relative variation of R_ef)
            IF( Deriv_Phc ) THEN
               SS_WFs: Block
c              ----------------
c                ... variation of expansion coefficients truncated phase
c                     function caused by R_ef
               FORALL( Js=1:N_SL ) CLD_WF(Js,:) = CLD_WF(Js,:) 
     $              + LWF%DPh_Dre(:,Js,1)*Prof_Cpar(:,2)*SCC(:)*E_W
c     $              + LWF%DPh_Dre(:,Js,1)*Prof_Cpar(:,2)*SCC_NS(:)*E_W
               
c                 WRITE(*,*) '  WmsF(JPOS) ', WmsF(JPOS)
                 IF( .NOT.WmsF(JPOS) ) EXIT SS_WFs  ! ====>
c                ... Derivative w.r.t D-M factor
c                 WRITE(*,*) '  Contribution w.r.t D-M factor '
                 DM_DRe = 0.d0
                 DO NCL = 1, N_CLD_LAYERS
                    SELECT CASE( TDS(NCL)%S )
                    CASE('water') ; N_db = N_ref_db  ; Rdb => Ref_db(:)
                    CASE('ice')
                       N_db =N_ref_ice(Ncl) ; Rdb => Ref_ice(Ncl,1:N_db)
                    END SELECT
c                   ... DM_CLD - derivatives of delta-M factor w.r.t. R_ef
c                       calculated in "gt_hi_wd.f"
                    DO KK = J_TOP(NCL)+1, J_BOT(NCL)-1 
                       CALL LIN_SEARCH_NEW(Prof_Cpar(KK,2), DUM, Ws, 
     $                      Ws1, N_db, Rdb)
                       DM_Dre(KK) = DM_CLD(NCL,Ws1,1)*DUM
     $                      + DM_CLD(NCL,Ws,1)*(1.d0 - DUM) 
                    ENDDO  ! over sub-layers within cloud (KK)
                 ENDDO  ! over cloud layers (NCL)                 
c                ... This term contains derivative [D Ln(1-f)]/[D Ln(R_ef)],
c                     where f is delta-m scaling factor
                 FORALL( Js=1:N_SL ) CLD_WF(Js,:) = CLD_WF(Js,:) 
     $                -(LWF%WF_ext(:,Js,1) + LWF%WF_scc(:,Js,1))
     $                *DM_DRe(:)*Prof_Cpar(:,2)*SCC_NS(:)*E_W
cc     $                - LWF%SSC_DM(:,Js,1)
cc     $                *DM_DRe(:)*Prof_Cpar(:,2)*SCC_NS(:)*E_W              
              END Block SS_WFs
c             ----------------
            ENDIF  ! Deriv_Phc 

            DO NCL = 1, N_CLD_LAYERS
               Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
               SELECT CASE( Var_type(NCL,1)%S )
               CASE('PNC')
                  FORALL( Js=1:N_SL ) CLD_WF(Js,:) = CLD_WF(Js,:)
     $               + (Ou1(:,Js) + Ou2(:,Js))*3.d0
               CASE('TAU')
c                 ... Weighting function of Ref under assumption tau_c=const
c                     (see Eq.(WFr_tau-c) in STB) Prof_Ref(Jt:Jb,2) 
c                     contains Dext/Dref*[R_ef/tau_c] at 500 nm
                  FORALL(Js = 1:N_SL) CLD_WF(Js,Jt:Jb)= CLD_WF(Js,Jt:Jb)
     $                 - SUM( Ou1(Jt:Jb,Js) + Ou2(Jt:Jb,Js) )
     $                 *Prof_Ref(Jt:Jb,2)*0.5d0*DZ(Jt:Jb)
               END SELECT
            ENDDO  ! over cloud layers (options 'ref_nomix','ref_water')

c        ---------------------------------------------------------------
         END SELECT

         FORALL(Js=1:N_SL) CLD_WF(Js,:) = CLD_WF(Js,:)*RtoA(:)
         IF(TRIM(WF_UNIT)== 'unified') THEN
            FORALL(Js=1:N_SL) CLD_WF(Js,:) = CLD_WF(Js,:)/DZ(1:NL) 
         ENDIF

         WF_DB(I)%WF(JPOS,:,:,1) = WF_DB(I)%WF(JPOS,:,:,1) + CLD_WF(:,:)
      ENDDO  ! over all WF for parameter-function (I)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE SET_OUTPUT_CLD
c ***********************************************************************
      SUBROUTINE DCOF_DZ(AIN, ZIN, AOU, NL, Jt, Jb)
      USE TYPE_VARS,   ONLY: KND
      INTEGER, INTENT(IN) :: NL, Jt, Jb
      REAL (KND), INTENT(IN) :: AIN(NL), ZIN(NL)
      REAL (KND), INTENT(OUT) :: AOU(NL)
      INTEGER I
        AOU = 0.d0
        AOU(Jt) = (AIN(Jt)-AIN(Jt+1))/(Zin(Jt)-Zin(Jt+1))
        AOU(Jb) = (AIN(Jb)-AIN(Jb-1))/(Zin(Jb)-Zin(Jb-1))
        DO I = Jt+1, Jb-1
           AOU(I) = (Ain(I-1)-Ain(I+1))/(Zin(I-1)-Zin(I+1))
        ENDDO
        WHERE( Ain(:) /= 0.0d0 )
           Aou(:) = Aou(:)/Ain(:)
        ELSE WHERE
           Aou(:) = 0.d0
        END WHERE            
      RETURN
      END SUBROUTINE DCOF_DZ
c ***********************************************************************

      SUBROUTINE SET_OUTPUT_MIX(JPOS, LWF, Cil, Cir, DP_CLD,  
     $     EXI, SCI, ADD_PLANCK, E_W)
c     ... Call in "st_adjoint_a.f"

      USE CLOUDS,      ONLY: IWP, N_CLD_LAYERS, J_TOP, J_BOT, Var_type,  
     $                       TAU_CLD_INP, Prof_Cpar, Prof_Ref
      USE CLOUD_INPUT, ONLY: Plus_ice
      USE GRIDS,       ONLY: NL => NLAYERS, DZ, ZZZ, DZR
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: NL_ice
      USE RTM_OUTPUT,  ONLY: WF_UNIT
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LWF_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: JPOS, Cil, Cir
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: E_W, DP_CLD, SCI(NL_ice), EXI(NL)
      TYPE (LWF_type) :: LWF    ! Local weighting functions
c     ... Local parameters
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'SET_OUTPUT_MIX'
      CHARACTER (LEN=10) C_HEL
      INTEGER    I, K, Js, Jt, Jb, NCL, Nco, IVT, LL, II
      REAL (KND) Dvt, Dvb, Tes, Cons_tau, OU1(NL,N_SL), OU2(NL,N_SL),
     $     Pnor
      REAL (KND), DIMENSION (NL) :: SCC_DR, ABS_DR, Doa, Dos, Xar
      REAL (KND), ALLOCATABLE :: CLD_WF(:,:), ALD(:), GPT_WF(:), 
     $     RtoA(:), DsDz(:), DaDz(:)
      REAL (KND), POINTER :: DT1(:), DT2(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      DT1 => LWF%DT1  ;  DT2 => LWF%DT2m

      IF(INQ_WF('anps', 'WFN', II)) THEN
         FORALL(Js=1:N_SL) WF_DB(II)%WF(JPOS,Js,:,1) = 
     $        WF_DB(II)%WF(JPOS,Js,:,1) + LWF%WF_scm(:,Js,1)*SCI(:)*E_W
      ENDIF

c     ... WF for cloud geometrical parameters and optical thickness:
c         additional term in the case of mixed cloud
      IF( INQ_WF('cld-s', 'WFT') ) THEN
         ALLOCATE( GPT_WF(N_SL), DsDz(NL), DaDz(NL)  )

         DO NCL = 1, N_CLD_LAYERS
c           ... Shift for a next cloud layer
            IVT = 1 
            IF( Plus_ice(NCL) ) IVT = 2

            Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
c           ... Derivative Dsca/Dz 
            CALL DCOF_DZ(SCI, ZZZ, DsDz, NL, Jt, Jb)
c           ... Derivative Dabs/Dz 
            Dos(:) = EXI(:) - SCI(:)
            CALL DCOF_DZ(Dos, ZZZ, DaDz, NL, Jt, Jb)

            Jt = J_TOP(NCL) ; Jb = J_BOT(NCL) 
            SELECT CASE ( Var_type(NCL,2)%S )
            CASE('IWC','CNC')
               Cons_tau = 0.d0
            CASE DEFAULT
               Cons_tau = 1.d0/( ZZZ(Jt)-ZZZ(Jb) )
            END SELECT

            Xar(Jt:Jb) = (ZZZ(Jt:Jb)-ZZZ(Jb))/(ZZZ(Jt) - ZZZ(Jb))
            Dos(Jt:Jb) = (-Xar(Jt:Jb)*DsDz(Jt:Jb)-Cons_tau)*SCI(Jt:Jb)
            Doa(Jt:Jb) = (-Xar(Jt:Jb)*DaDz(Jt:Jb)
     $           - Cons_tau )*(EXI(Jt:Jb)-SCI(Jt:Jb))
            DvT = 0.d0 
            DO I = J_TOP(NCL), J_BOT(NCL)-1
               DvT = DvT + 
     $              ( (DT1(I)  +DT2(I)  )*Dos(I)   + DT1(I)  *Doa(I)
     $              + (DT1(I+1)+DT2(I+1))*Dos(I+1) + DT1(I+1)*Doa(I+1) )
     $              *0.5D0*DZR(I)
            ENDDO
            Dos(Jt:Jb) = (-(1.0d0 - Xar(Jt:Jb))*DsDz(Jt:Jb)
     $           + Cons_tau )*SCI(Jt:Jb)
            Doa(Jt:Jb) = ( -(1.0d0 - Xar(Jt:Jb))*DaDz(Jt:Jb) 
     $          + Cons_tau )*(EXI(Jt:Jb)-SCI(Jt:Jb))
            DvB = 0.d0
            DO I = J_TOP(NCL), J_BOT(NCL)-1
               Dvb = Dvb + ( (DT1(I) + DT2(I))*Dos(I) + DT1(I)*Doa(I)
     $              + (DT1(I+1)+DT2(I+1))*Dos(I+1) + DT1(I+1)*Doa(I+1) )
     $              *0.5D0*DZR(I)
            ENDDO            
c           First infinitesimal cloud layer is not used for calculation
c           -----------------------------------------------------------
            Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
c           ... Over required parameters (CTH, CBH, Tau)
            DO K = 1, UBOUND(WF_DB,1)
               SELECT CASE(WF_DB(K)%NAME)
               CASE DEFAULT
                  CYCLE
               CASE( 'cld-cth' )
                  Tes = DT1(Jt)*EXI(Jt) + DT2(Jt)*SCI(Jt) 
                  GPT_WF(:) = (Tes + DvT)
c                  WRITE(*,*) '  2 CTH: ', Tes + DvT
               CASE( 'cld-cbh' )
                  Tes = DT1(Jb)*EXI(Jb)  +  DT2(Jb)*SCI(Jb)
                  GPT_WF(:) = (-Tes + DvB)
c                  WRITE(*,*) '  2 CBH: ', -Tes + DvB
               CASE( 'cld-tau_water' )
                  GPT_WF(:) = 0.d0
               CASE( 'cld-tau_nomix','cld-tau_ice' )
                  Tes = 0.d0
                  DO I = Jt-1,Jb
                     Tes = Tes + 
     $                    ( DT1(I)  *EXI(I)  + DT2(I)  *SCI(I)
     $                    + DT1(I+1)*EXI(I+1)+ DT2(I+1)*SCI(I+1) )
     $                    *0.5D0*DZR(I)
                  ENDDO
                  Pnor = 1.d0
c                 ... For the absolute variation of the opt. thickness
                  IF(WF_DB(K)%DIM=='abs' ) Pnor = Tau_cld_inp(NCL,IVT)
                  GPT_WF(:) = Tes/Pnor
               END SELECT
               WF_DB(K)%WF(JPOS,:,NCL,1) = WF_DB(K)%WF(JPOS,:,NCL,1)
     $              + GPT_WF(:)*E_W
            ENDDO               ! All parameters
         ENDDO                  ! All cloud layers
      ENDIF                     ! If MAX_GPT_WF > 0

c     ... Return if no weighting function for parameter-function required
      IF(.NOT.INQ_WF('cld-v', 'WFT')) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      ALLOCATE( CLD_WF(N_SL,NL), ALD(NL), RtoA(NL) )

c     ... For first component of the Stokes vector !!!
      Ou2(:,:) =  LWF%WF_ext(:,:,1)*E_W
      IF( ADD_PLANCK ) Ou2(:,:) = Ou2(:,:) + LWF%WF_TE(:,:)*E_W
      Ou1(:,:) = (LWF%WF_ext(:,:,1) + LWF%WF_scm(:,:,1))*E_W

c     ... All auxiliary function are written for relative variation
c         of scattering and absorption coefficients
      ALD(:) = EXI(:) - SCI(:)
      FORALL( Js = 1:N_SL ) 
        Ou1(:,Js) = Ou1(:,Js)*SCI(:) ; Ou2(:,Js) = Ou2(:,Js)*ALD(:)
      END FORALL

c     ... Do-loop over parameter-function WF      
c     --------------------------------------
      DO I = 1, UBOUND(WF_DB,1)
         LL = LEN_TRIM(WF_DB(I)%NAME)
c          WRITE(*,*) '   I =  ', I, LL
c          WRITE(*,*) 'WF_DB(I)%NAME ', WF_DB(I)%NAME
c          WRITE(*,*) I, WF_DB(I)%NAME(LL - 2:LL)
         IF( WF_DB(I)%TYPE/='cld-v' ) CYCLE
         IF(WF_DB(I)%NAME(LL - 2:LL) /= 'ice') CYCLE
         RtoA(:) = 1.d0 ; CLD_WF = 0.d0
         SELECT CASE ( WF_DB(I)%NAME )
c        ... Scattering coefficient
        CASE('cld-sca_ice')
            IF( WF_DB(I)%DIM=='abs' ) 
     $           FORALL( K=1:NL, SCI(K)/=0.d0 ) RtoA(K) = 1.d0/SCI(K)
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,Js)

c        ... Absorption coefficient of droplets
         CASE('cld-abs_ice')
            IF( WF_DB(I)%DIM=='abs' ) 
     $           FORALL( K=1:NL, ALD(K)/=0.d0 ) RtoA(K) = 1.d0/ALD(K)
            FORALL( JS=1:N_SL ) CLD_WF(JS,:) = Ou2(:,JS)

c        ... Ice water content and crystal number density
c        ------------------------------------------------
         CASE('cld-lwc_ice','cld-pnc_ice')  
            IF( WF_DB(I)%DIM=='abs' ) FORALL( K=1:NL, SCI(K)/=0.d0 ) 
     $           RtoA(K) = 1.d0/Prof_Cpar(K,3)
c           ... Standard representation
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,Js) + Ou2(:,Js)

c           ... If "IWP" or "TAU" option is used
            DO NCL = 1, N_CLD_LAYERS
               Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
               SELECT CASE ( Var_type(NCL,2)%S )
               CASE('IWP')
                  Dos(Jt:Jb) = Prof_Cpar(Jt:Jb,3)*0.5d0*DZ(Jt:Jb)*1.d3
     $                 /IWP(NCL)
               CASE('TAU')
                  Dos(Jt:Jb) = Prof_Ref(Jt:Jb,3)*0.5d0*DZ(Jt:Jb)
               CASE DEFAULT
                  CYCLE
               END SELECT
c              ... IWC weighting function under assumption of
c                  constant IWP and constant tau_i

c               WRITE(*,*) '  "lwc_ice", NCL = ', NCL,
c     $              SUM( Prof_Ref(Jt:Jb,3)), SUM( Ou1(Jt:Jb,Js) ), 
c     $              SUM( Ou2(Jt:Jb,Js) )

               FORALL( Js=1:N_SL )  CLD_WF(Js,Jt:Jb) = CLD_WF(Js,Jt:Jb) 
     $              - SUM( Ou1(Jt:Jb,Js) + Ou2(Jt:Jb,Js) )*Dos(Jt:Jb)
            ENDDO               ! Over cloud layers

c        ... Effective radius of ice crystal
c        -----------------------------------
         CASE('cld-ref_ice')

            IF( WF_DB(I)%DIM=='abs' ) FORALL( K=1:NL, SCI(K)/=0.d0 ) 
     $           RtoA(K) = 1.d0/Prof_Cpar(K,4)
            Nco = 3
            CALL INTERPOL_DRIV(Cil, Cir, DP_CLD, Nco, ABS_DR, SCC_DR)

            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = Ou1(:,JS)*SCC_DR(:)
     $           + Ou2(:,JS)*ABS_DR(:)

            DO NCL = 1, N_CLD_LAYERS
               Jt = J_TOP(NCL)+1 ; Jb = J_BOT(NCL)-1
               SELECT CASE ( Var_type(NCL,2)%S )
               CASE('TAU')
                  Dos(Jt:Jb) = Prof_Ref(Jt:Jb,4)*0.5d0*DZ(Jt:Jb)
               CASE DEFAULT
                  CYCLE
               END SELECT
               FORALL( Js=1:N_SL ) CLD_WF(Js,Jt:Jb) = CLD_WF(Js,Jt:Jb) 
     $              - SUM( Ou1(Jt:Jb,Js) + Ou2(Jt:Jb,Js) )*Dos(Jt:Jb)
           ENDDO
         END SELECT

         FORALL( Js=1:N_SL ) CLD_WF(Js,:) = CLD_WF(Js,:)*RtoA(:)

         IF(TRIM(WF_UNIT)== 'unified') THEN
            FORALL( Js=1:N_SL ) CLD_WF(Js,:) = CLD_WF(Js,:)/DZ(1:NL) 
         ENDIF

         WF_DB(I)%WF(JPOS,:,:,1) = WF_DB(I)%WF(JPOS,:,:,1) + CLD_WF(:,:)
      ENDDO                     ! Over all WF for parameter-function

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
