C********************************************************************C
C
C	SCIATRAN - Version 4.4 - Last modification: 27 Mar 2021
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE SFI_FIELD(JPOS, RAD, AL, E_W, OSM, LOA, LSE, TE, ADI)
c     Called by "st_dom_tau.f"
      
      USE AEROSOLS,    ONLY: IDX_AER
      USE CLOUDS,      ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR
      USE GRIDS,       ONLY: NLAYERS, ZZZ, Z_FINE, NSOL, SZA_EXT, 
     $                       ALT_EXT, Z_INV, NL_FINE, X_INV, INTERP_IDX
      USE GT_POLYN,    ONLY: NLg => NLEGEN, XM_G => XM, AG1
      USE INTERN_DIMS, ONLY: Nvs, N_SFN, Leg_aer => NLG_ALL_A, Nlg_Ray,
     $                       Leg_cld => NLG_ALL_C, Nm_vs, N_PHI_MS
      USE NODES,       ONLY: NN_MS => N_NODE
      USE LOG_PARS,    ONLY: LOG_LEVEL, LUNIT, N_DSO
      USE LOS_Angles,  ONLY: DirSun
      USE RTM_OUTPUT,  ONLY: STOK_OUT
      USE ST_VARS,     ONLY: Pie, DO_AEROSOL, Vec_Sca, WF_FLAG,
     $                       GeoD => GeoD_OUT, PreCalcAngs
      USE SPHER_PARS,  ONLY: N_LOS_ANG, AZM_MS, NFA => N_FOV_ANG,
     $                       ONE_MS_SZ, FOV_WEIGHTS
      USE ST_INPUT,    ONLY: APH_TYPE
      USE TOOLS,       ONLY: Get_Sza_Idx, LOS_MS_IDX
      USE TYPES,       ONLY: KND, LOA_type, PCC_TYPE, OSM_TYPE, TE_TYPE,
     $                       LSE_type, AuxDataInterp
      USE WF_NUM,      ONLY: NUM_WF, NO_SOL, NO_LS, AL_1_BASE, AL_1_PERT
c     ------------------------------------------------------------------

c     ... Local optical parameters atmosphere |
c     -----------------------------------------
      TYPE (AuxDataInterp), TARGET :: ADI
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE(LSE_type), TARGET  :: LSE(GeoD%NAlts, GeoD%NVA)
      TYPE (TE_type)  :: TE     ! Thermal emission 
c     ----------------------
      INTEGER    JPOS
      REAL (KND) E_W, AL(NLAYERS)
      REAL (KND), ALLOCATABLE :: RAD(:,:,:)
      INTENT(IN) :: JPOS, E_W, AL
      ! Local variables
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'SFI_FIELD'
      INTEGER  Idn, II, J, I, K, L, Npl, Npo, NTH, Ks, NSTR,
     $     NR, Ja, Jz, L1, IVS, IVE, IV, IS, IP, IA, ILOS, IMS1, IMS2
      INTEGER  LL(GeoD%MaxLS)
      INTEGER, POINTER ::  SID(:), IOB(:)
      INTEGER, ALLOCATABLE :: CI(:)
      LOGICAL  DO_WF
      REAL (KND) Mu_in, Azm_in, Mu_sc, Azm_sc, Si_in, Si_sc, Rel_az
      REAL (KND) Mu_Ls, T1, TTT1, P_F, SD, Z_OUT, D1, Co_s
      REAL (KND) XM_2(NN_MS), AG1_2(NN_MS), DUM(Nvs), BC_S1(Nvs),
     $     Jms(N_PHI_MS,Nvs), Si_sp(Nvs), Ss_sp(Nvs), 
     $     Ss_mp(NN_MS,Nvs), RR1(2), RL1(2), Sm_Ur(2), Sm_Dl(2)
c     $     RoR(MAX_LINE_MS,NN_MS,N_PHI_MS,2),
c     $     RoL(MAX_LINE_MS,NN_MS,N_PHI_MS,2)
      REAL (KND), ALLOCATABLE :: DS_SUN(:,:)
      REAL (KND), DIMENSION (GeoD%MaxLS,Nvs) :: SSSF, S_F
      REAL (KND), DIMENSION (GeoD%MaxLS) :: SOL_EXP, S_H, COSCA, AL_1,
     $     SCRL
      REAL (KND), POINTER :: ALO(:), SCRO(:), SCAO(:), ASCO(:), DM_AO(:)
      REAL (KND) RESULT(GeoD%NAlts,GeoD%NVA,GeoD%NSol,GeoD%NPhi,Nvs) ! A
      REAL (KND) Hve(4), Phm(GeoD%MaxLS,Nvs), Rot_l(GeoD%MaxLS,Nvs)
      REAL (KND) FL(N_SFN,GeoD%MaxLS), X_RAD(NLAYERS, NSOL, N_SFN, 4), 
     $           Y_ALL(NSOL, N_SFN, NL_FINE, 4), Y_TH(NSOL,N_SFN,4),
     $           Y_FP(NSOL,N_SFN,4), R_SNG(NLAYERS,NSOL)
      REAL (KND), POINTER :: ZLS(:), XM_1(:), P_MOM(:), EX_L(:),
     $     XM_S1(:), PHI_LOS(:), XM_SG(:), TH(:), SCCO(:,:),
     $     SCC_NSO(:,:)
      REAL (KND), ALLOCATABLE :: G_aer(:), SCA_LS(:), DM_LS(:),
     $     Therm(:), SCC_LS(:), SCCNO_LS(:)
      TYPE (PCC_type) :: PCC    ! Photochemistry
c     ==================================================================
      INTERFACE
         SUBROUTINE BIAKIMA_Y (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
             USE GRIDS,       ONLY: NSOL
             USE TYPES,       ONLY: KND
             REAL (KND)  X_RAD(:,:,:), Y_ALL(:,:,:) 
             REAL (KND)  Y_LOC(NSOL + 4)
             REAL (KND)  X_NEW, X_AX
             END SUBROUTINE BIAKIMA_Y
      END INTERFACE
c     ==================================================================
      CALL ST_TRACK (R_NAME, 1, 'I')
      IF( LOG_LEVEL(1) >= N_DSO ) 
     $     WRITE(LUNIT,'(20x,A)') ' *** '//R_NAME

      DO_WF = WF_FLAG
C     Interpolation of diffuse radiance
C     **********************************
      IF (LOG_LEVEL(1) >= N_DSO)  
     $     WRITE(LUNIT,*) ' CALL LIN_END_FINE :: NL_FINE = ', NL_FINE

      DM_AO => ADI%DM_AO  ;  ALO => ADI%AL0_OUT ;  SCRO => ADI%SCR_OUT
      SCAO => ADI%SCA_OUT ;  ASCO => ADI%ASC_OUT

      IF( DO_CLOUD ) THEN
         SCC_NSO => ADI%SCC_NSO ;  SCCO => ADI%SCCO
      ENDIF

      IF( TE%ADD_PLANCK ) ALLOCATE( Therm(GeoD%MaxLS) )

c     ... Over all zenith angles (up and down), azimuthal angles, 
c     and all Stokes components
      DO Idn = 1,N_SFN
         R_SNG(:,:) = SNGL( RAD(:,:,Idn) )
         CALL BIAKIMA_X(NLAYERS, ALT_EXT, NSOL, X_RAD(:,:,Idn,:), R_SNG)
      END DO         
      DO J = 1, NL_FINE
         CALL BIAKIMA_Y(Z_INV(INTERP_IDX(J)), SZA_EXT, Z_FINE(J),
     $        X_RAD(INTERP_IDX(J),:,:,:), Y_ALL(:,:,J,:))
      END DO

      Z_OUT = GeoD%Alt(1)

      IF(DO_WF) ALLOCATE( DS_SUN(GeoD%MaxLS,NLAYERS) )  ! A

      Npo = MAX( Nlg_Ray,Leg_aer,Leg_cld )
                
      NSTR = NN_MS/2
      XM_2(1:NSTR)       =  XM_G(:) !Downward
      XM_2(1+NSTR:NN_MS) = -XM_G(:)
      AG1_2(1:NSTR) = AG1(:) ; AG1_2(1+NSTR:NN_MS) = AG1(:)      

      IA = 1
      DO IV = 1, GeoD%NVA       ! Number of viewing angles including FOV
         Mu_Ls = GeoD%LVG(IA,IV)%C_VA(1)
         NTH   = GeoD%LVG(IA,IV)%NTH
         NR    = GeoD%LVG(IA,IV)%NLS
         ZLS  => GeoD%LVG(IA,IV)%ZLS
         SID  => GeoD%LVG(IA,IV)%ISEL
         IOB  => GeoD%LVG(IA,IV)%IOB
         
         AL_1(1:NR)  = ALO(SID(:)) ;  SCRL(1:NR)  = SCRO(SID(:))
         IF( DO_AEROSOL ) THEN
            ALLOCATE(G_aer(NR), SCA_LS(NR), DM_LS(NR) )
            G_aer(:)   = ASCO (SID(:))         
            SCA_LS (:) = SCAO (SID(:)) 
            DM_LS  (:) = DM_AO(SID(:))
         END IF         

         IF( DO_CLOUD ) THEN
            ALLOCATE(CI(NR), SCC_LS(NR), SCCNO_LS(NR) )
            CI(:) =  INT( CLA(IOB(:),2) )
            SCC_LS  (:) = SCCO   (SID(:),1)
            SCCNO_LS(:) = SCC_NSO(SID(:),1)
         ENDIF

         IF( TE%ADD_PLANCK ) THEN
            ALLOCATE(Therm(NR))
            Therm(:) = TE%PLA_FINE( SID(:) )
         END IF
         
         IF( GeoD%LVG(IA,IV)%TH_FLG )
     $        CALL INT_ON_TH_LEV(NTH, IOB(NTH), Y_TH)
         IF( GeoD%LVG(IA,IV)%FP_FLG )
     $        CALL INT_ON_TH_LEV(1, IOB(1), Y_FP)
         
c        ... Attention !!!
c          XM_1 is the cosine of the L-of-S direction with  
c          respect to the downward direction, i.e., 
c          XM_1 is the cosine between the direction of radiation 
c          propagation and z-axis.
         XM_1 => GeoD%LVG(IA,IV)%C_VA
         EX_L => LSE(IA,IV)%EX(1:NR)

         DO IS = 1, GeoD%NSol   ! Nuber of solar angles
            DO IP = 1, GeoD%NPhi ! Nuber of azimuth angles
!              This does not work if the beginning of LOS is in the shadow
               CALL LOS_MS_IDX(GeoD%LSG(IA,IV,IS,IP)%CG_SZA, IMS1, IMS2)
               
c              ... Azimuthal angle [rad] along line-of-sight
               PHI_LOS => GeoD%LSG(IA,IV,IS,IP)%SAA(:)
               
c              ... Attention !!!
c                XM_S1 is the cosine of the solar angle between the direction
c                of the solar beam and downward direction at a point of L-of-S
               XM_S1 => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  

C              ... Attention, to pick the appropriate values from the 
c                multiple scattering contribution we need geometrical SZA 
C                rather than refractive one.
               XM_SG => GeoD%LSG(IA,IV,IS,IP)%CG_SZA(:)  

c              ... Calculation of direct sunlight source function along LOS
c                  in the subroutine after "CONTAINS"
               CALL SINGLE_IN_MS_MODE

c              ... Calculation of multiple scattering source function along LOS
               CALL GET_SZA_IDX(XM_SG(IMS1:IMS2), LL(IMS1:IMS2))
    
               FL = 0.d0
               FORALL(L = IMS1:IMS2) FL(:,L) = Y_ALL(LL(L),:,SID(L),1)
               IF (.NOT.ONE_MS_SZ) THEN
                  DO L = IMS1, IMS2
                     K = LL(L)
                     SD = XM_SG(L) - X_INV(K)
                     FL(:,L) =  FL(:,L) + SD*(Y_ALL(K,:,SID(L),2)
     $                    + SD*( Y_ALL(K,:,SID(L),3) 
     $                            + Y_ALL(K,:,SID(L),4)*SD))
                  END DO
               END IF

               IF( GeoD%LVG(IA,IV)%TH_FLG.AND.
     $              (NTH.GE.IMS1.AND.NTH.LE.IMS2)) THEN
                  K = LL(NTH)
                  FL(:,NTH) = Y_TH(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(NTH) - X_INV(K)
                     FL(:,NTH) =  FL(:,NTH) + SD*(Y_TH(K,:,2) +
     $                    SD*(Y_TH(K,:,3) + Y_TH(K,:,4)*SD))
                  END IF
               END IF
         
               IF(GeoD%LVG(IA,IV)%FP_FLG) THEN
                  K = LL(1)
                  FL(:,1) = Y_FP(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(1) - X_INV(K)
                     FL(:,1) =  FL(:,1) + SD*(Y_FP(K,:,2) +
     $                    SD*(Y_FP(K,:,3) + Y_FP(K,:,4)*SD))
                  END IF
               END IF

               SELECT CASE( Vec_Sca )
               CASE( 'vect_rte' )
                  DO L = IMS1, IMS2
                     Mu_sc = -XM_1(L) ;  Azm_sc = PHI_LOS(L) 
                     Si_sc = DSQRT(1.d0 - Mu_sc**2)
                     
                     DO Ja = 1,N_PHI_MS
                        Azm_in = AZM_MS(Ja)
                        Rel_az = Azm_sc - Azm_in
                        IF( Rel_az < 0.d0 ) Rel_az = 2.d0*Pie + Rel_az
c                       ... Shift at all zenith angles and Stokes component
c                           to obtain next azimuth
                        Ks = Nvs*NN_MS*(Ja - 1)
                        
                        DO Jz = 1,NN_MS
                           Mu_in = XM_2(Jz) 
                           Si_in = DSQRT(1.d0 - Mu_in**2)
                           Co_s = Mu_in*Mu_sc + Si_sc*Si_in*DCOS(Rel_az)
                           IF(REAL(Rel_az) == REAL(Pie) 
     $                          .OR. REAL(Rel_az) == 0.0) THEN 
c                             ... Scattering plane is meridional plane:
                              RL1(1) = 1.d0 ; RL1(2) = 0.d0
                              RR1(1) = 1.d0 ; RR1(2) = 0.d0 
                           ELSE
                              CALL ROT_LSP
                           ENDIF
c                          ... Shift at next zenith angle
                           L1 = Ks + (Jz-1)*Nvs
c                          ... Select four Stokes components
                           Hve(1:4) = FL(L1+1:L1+4,L)
c                          ... Rotate Stokes vector of the incident diffuse light 
                           Si_sp(1) = Hve(1)
                           Si_sp(2) = RR1(1)*Hve(2) - RR1(2)*Hve(3)
                           Si_sp(3) = RR1(2)*Hve(2) + RR1(1)*Hve(3) 
                           Si_sp(4) = Hve(4)

cc                         ... Rayleigh scattering matrix
                           Sm_Ur(1) =   1.d0 + Co_s**2
                           Sm_Ur(2) = -(1.d0 - Co_s**2)
                           Sm_Dl(1) =   2.d0*Co_s
                           Sm_Dl(2) =   0.d0

c                         ... Stokes vector of the scattered light in the 
c                             scattering plane
                           Ss_sp(1) = SUM(Sm_Ur(1:2)*Si_sp(1:2))
                           Ss_sp(2) = 
     $                          Sm_Ur(2)*Si_sp(1) + Sm_Ur(1)*Si_sp(2)
                           Ss_sp(3) = SUM(Sm_Dl(1:2)*Si_sp(3:4))
                           Ss_sp(4) = 
     $                          Sm_Dl(2)*Si_sp(3) + Sm_Dl(1)*Si_sp(4)
                           Ss_sp(:) = 0.75d0*Ss_sp(:)

c                          ... Rotate Stokes vector of the scattered diffuse
c                              light from a scattering to the meridional plane
                           Ss_mp(Jz,1) = Ss_sp(1)
                           Ss_mp(Jz,2) =  
     $                          RL1(1)*Ss_sp(2) - RL1(2)*Ss_sp(3)
                           Ss_mp(Jz,3) = 
     $                          RL1(2)*Ss_sp(2) + RL1(1)*Ss_sp(3)
                           Ss_mp(Jz,4) = Ss_sp(4)
                        ENDDO  ! over basis zenith angles (Jz)
                        DO I = 1,4
                           Jms(Ja,I) = SUM( AG1_2(:)*Ss_mp(:,I) )
                        ENDDO
                     ENDDO  ! over basis azimuthal angles (Ja)
c                    ... Integrate over azimuthal angle
                     DO I = 1,4
                        T1 = 0.d0
                        DO Ja = 2,N_PHI_MS
                           T1 = T1 + (Jms(Ja,I) + Jms(Ja-1,I))
     $                          *(AZM_MS(Ja) - AZM_MS(Ja-1))/2.d0
                        ENDDO
                        S_F(L,I) = SCRL(L)*T1/2.d0/Pie
                     ENDDO  ! over Stokes components (I)
                  ENDDO
               CASE( 'scal_rte' )
                  DO L = IMS1, IMS2
                     Mu_sc = -XM_1(L) ;  Azm_sc = PHI_LOS(L)
                     Si_sc = DSQRT(1.d0 - Mu_sc**2)                     
                     DO Ja = 1,N_PHI_MS
                        Rel_az = Azm_sc - AZM_MS(Ja) 
                        Ks = NN_MS*(Ja - 1)                        
                        DO Jz = 1,NN_MS
                           Mu_in = XM_2(Jz) 
                           Si_in = DSQRT(1.d0 - Mu_in**2)
                           Co_s = Mu_in*Mu_sc + Si_sc*Si_in*DCOS(Rel_az)
                           Ss_mp(Jz,1) = 0.75d0
     $                          *(1.d0 + Co_s**2) *FL(Ks+Jz,L)
                        ENDDO  ! over basis zenith angles (Jz)
                        Jms(Ja,1) = SUM( AG1_2(:)*Ss_mp(:,1) )
                     ENDDO  ! over basis azimuthal angles (Ja)
c                    ... Integrate over azimuthal angle
                     T1 = 0.d0
                     DO Ja = 2,N_PHI_MS
                        T1 = T1 + (Jms(Ja,1) + Jms(Ja-1,1))
     $                       *(AZM_MS(Ja) - AZM_MS(Ja-1))/2.d0
                     ENDDO
                     S_F(L,1) = SCRL(L)*T1/2.d0 /Pie
                  ENDDO  ! over line-of-sight (L)

               END SELECT

               S_F(1:IMS1-1,:) = 0.d0 ; S_F(IMS2+1:Nr,:) = 0.d0
               
               Surface_Refl: BLOCK
                  BC_S1 = 0.0D0
                  IF(IMS2 <= 0) EXIT Surface_Refl
                  IF( ZLS(IMS2) /= ZZZ(NLAYERS) ) EXIT Surface_Refl
                  BC_S1 = 0.0d0
c                   Proverit' dlya reflected  !! Lambert only !!
                  BC_S1(1) = FL(Nvs*Nstr+1,Nr) ! Surface reflection term
                  IF(Nvs > 1) BC_S1 = BC_S1/2.0d0 
               END BLOCK Surface_Refl ! Reflection by surface
                
               IF(Z_OUT.EQ.ZZZ(1).AND.Mu_Ls.LE.0) THEN
                  RESULT(IA,IV,IS,IP,:) = 0.0D0
                  CYCLE
               END IF
               IF(Z_OUT.EQ.ZZZ(NLAYERS).AND.Mu_Ls.GT.0) THEN
                  RESULT(IA,IV,IS,IP,:) = BC_S1(:)
                  CYCLE
               END IF
               IF(Nr.EQ.1) THEN
                  RESULT(IA,IV,IS,IP,:) = 0.0D0
                  CYCLE
               END IF

c              For test with numerical perturbations
               IF( NUM_WF ) THEN 
c                  To calulate without contribution from solar exponent
                  IF( NO_SOL ) AL_1(1:NR) = AL_1_PERT(1:NR)
c                 To calulate without contribution from LS exponent
                  IF( NO_LS  ) AL_1(1:NR) = AL_1_BASE(1:NR)
               ENDIF

               S_F(:,:) = SSSF(:,:) + S_F(:,:)
              
               DUM = 0.0D0
               DO I = 1, Nr
c                  write(6,'(1x,I3,4(1x,e12.4))') I,  S_F(I,:) 
c                  write(6,'(1x,I3,2(1x,e12.4))') I,  EX_L(I), 
c     $              (GeoD%LVG(IA,IV)%DS(1,I) + GeoD%LVG(IA,IV)%DS(2,I))
                  DUM = DUM + S_F(I,:)*EX_L(I)*(GeoD%LVG(IA,IV)%DS(1,I)
     $                 + GeoD%LVG(IA,IV)%DS(2,I))
               END DO
!     pause
               WRITE(*,*) DUM
               RESULT(IA,IV,IS,IP,:) = DUM(:) + BC_S1(:)*EX_L(Nr)
               
            END DO  ! Azimuth angles
         END DO  ! Solar zenith angles
         NULLIFY(ZLS, SID, IOB, XM_1, EX_L, XM_S1, XM_SG)
         IF( DO_AEROSOL ) DEALLOCATE(G_aer, SCA_LS, DM_LS)
         IF( DO_CLOUD )   DEALLOCATE(CI, SCC_LS, SCCNO_LS)
         IF( TE%ADD_PLANCK ) DEALLOCATE(Therm)
      END DO                     ! Number of viewing angles

      WRITE(*,*) ' ---->> sfi_field : GeoD%NPhi = ', GeoD%NPhi
      ILOS = 0
      DO IS = 1, GeoD%NSol
         DO IV = 1, N_LOS_ANG
            IVS = (IV - 1)*NFA + 1 ; IVE = IV*NFA
            DO IP = 1, GeoD%NPhi
               ILOS = ILOS + 1
               DO K = 1,Nvs
                  STOK_OUT(JPOS,1,1,ILOS,K) = STOK_OUT(JPOS,1,1,ILOS,K) 
     $                 + SUM(RESULT(IA,IVS:IVE,IS,IP,K)*FOV_WEIGHTS(:))
               ENDDO
            ENDDO
         END DO
      END DO

      WRITE(*,'(3x,4(1x,E15.7))') STOK_OUT(JPOS,1,1,1,:)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS

c *************************************************************************
c     This subroutine calculates the contribution in radiation along 
c     line-of-sight from the direct scattered solar beam. Therefore, it 
c     should be used here all phase matrix expansion coefficients.

      SUBROUTINE SINGLE_IN_MS_MODE
      REAL (KND)  Wig0_ss(Npo,NR), Wig2_ss(Npo,NR)

      TH => GeoD%LSG(IA,IV,IS,IP)%TH(:)

c       Scattering angle along line-of-sight
c       ------------------------------------
        COSCA(1:NR) = -XM_1(1:NR)*XM_S1(1:NR) 
     $       + DSQRT(1.0d0 - XM_1(1:NR)*XM_1(1:NR))
     $       * DSQRT(1.0D0 - XM_S1(1:NR)*XM_S1(1:NR))
     $       * DCOS(PHI_LOS(1:NR))

c       Solar exponent at all point along LS
c       ------------------------------------
        ASSOCIATE (LVG => GeoD%LVG(IA,IV), LSG => GeoD%LSG(IA,IV,IS,IP))
           CALL DirSun (NR, XM_S1, TH, ZLS, LVG%IOB, AL_1(1:NR), ALO, 
     $       PCC, SOL_EXP(1:NR), LSG%REFR_DSB, PreCalcAngs, DO_WF, 
     $       DS_SUN, LSG%DSB)
        END ASSOCIATE
c       Legendre polynomials for all scattering angles
c       ----------------------------------------------
        CALL WIGNER_PLG(COSCA(1:NR), NR, 0, Npo, Wig0_ss(:,1:NR))

c       Rayleigh contribution to the source function
c       --------------------------------------------
        Npl = Nlg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)
        DO J = 1,NR
           Phm(J,1) = SCRL(J)*SUM( Wig0_ss(1:Npl,J)*P_MOM(:) )
        ENDDO
        IF( Nm_vs > 1 ) THEN
           Phm(:,4) = 0.0d0
           CALL WIGNER_PLG( COSCA(1:NR), NR, 2, Npo, Wig2_ss(:,1:NR) )
           CALL ROT_MATR
           P_MOM => LOA%RAY_MOMS(1:Npl,5) 
           DO J = 1,NR
              T1 = SCRL(J)*SUM( Wig2_ss(1:Npl,J)*P_MOM(:) )
              Phm(J,2:3) = T1*Rot_l(J,2:3)
           ENDDO
        ENDIF
        SSSF(1:NR,:) = Phm(1:NR,:)

c        WRITE(*,*) SSSF(1:NR,2)
                  
c       Contribution from aerosol phase function
c       ----------------------------------------
        IF( DO_AEROSOL ) THEN
           SELECT CASE( APH_TYPE )
           CASE('h-g')
              S_H(1:NR) = 1.0d0 + G_aer(1:Nr)*G_aer(1:Nr)
     $             - 2.0d0*G_aer(1:Nr)*COSCA(1:Nr)
              S_H(1:NR) = (1.0d0 - G_aer(1:Nr)*G_aer(1:Nr))
     $             /S_H(1:Nr)/DSQRT(S_H(1:Nr))
              SSSF(1:NR,1) = SSSF(1:NR,1) 
     $             + S_H(1:Nr)*SCA_LS(1:Nr)/(1.D0 - DM_LS(1:Nr))
           CASE('mie')
              Npl = Leg_Aer  
              DO J = 1,NR
                 Phm(J,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),1) )
              ENDDO
              IF( Nvs > 1 ) THEN
                 DO J = 1,NR
                    T1 = SUM( Wig2_ss(1:Npl,J)
     $                   *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),5) )
                    Phm(J,2:3) = T1*Rot_l(J,2:3)
                 ENDDO                     
              ENDIF
              DO K = 1,Nvs
                 SSSF(1:NR,K) = SSSF(1:NR,K) + Phm(1:Nr,K)
     $                *SCA_LS(1:Nr)/(1.D0 - DM_LS(1:Nr))
              ENDDO
           END SELECT
        ENDIF

        IF( DO_CLOUD ) THEN
           Npl = Leg_cld ; Phm(:,:) = 0.d0
           FORALL( J=1:Nr, CI(J)>0 ) Phm(J,1) =
     $          SUM( Wig0_ss(1:Npl,J)*OSM%CLD_ALL(1:Npl,CI(J),1,1,1) )
           IF( Nvs > 1 ) THEN
              FORALL( J=1:Nr, CI(J)>0 ) Phm(J,2:3) = Rot_l(J,2:3)
     $             *SUM(Wig2_ss(1:Npl,J)*OSM%CLD_ALL(1:Npl,CI(J),1,1,5))
           ENDIF
           DO K = 1,Nvs
              SSSF(1:NR,K) = SSSF(1:NR,K) + Phm(1:Nr,K)*SCCNO_LS(1:Nr)
           ENDDO
        ENDIF

        DO K = 1,Nvs
           SSSF(1:NR,K) = SSSF(1:NR,K)*SOL_EXP(1:NR)/4.0D0
        ENDDO
 
        IF( TE%ADD_PLANCK ) SSSF(1:NR,1) = SSSF(1:NR,1) + Therm(1:NR)
        
c        write(*,*) '   sfi_field: SOL_exp:'
c        WRITE(*,*) Sol_exp(1:Nr  )

      END SUBROUTINE SINGLE_IN_MS_MODE


c ************************************************************************
      SUBROUTINE ROT_LSP
      REAL (KND)  Zn, T1, T2, Si_s

      Si_s = DSQRT(1.D0 - Co_s**2)
      IF( Rel_az > Pie ) THEN 
         Zn = 1.d0
      ELSE
         Zn = -1.d0
      ENDIF
      IF( DABS(Mu_sc) == 1.d0 ) THEN
         T2 = DSIGN(1.d0,Mu_sc)*DCOS(Rel_az)
      ELSE
         T2 = -(Mu_in - Mu_sc*Co_s)/Si_sc/Si_s
      ENDIF
      
      IF( DABS(Mu_in) == 1.d0 ) THEN
         T1 = DSIGN(1.d0,Mu_in)*DCOS(Rel_az) 
      ELSE
         T1 = -(Mu_sc - Mu_in*Co_s)/Si_in/Si_s
      ENDIF
      
      RL1(1) = (2.d0*T2*T2 - 1.d0 )
      RL1(2) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
      RR1(1) = (2.d0*T1*T1 - 1.d0 )
      RR1(2) = Zn*2.d0*DSQRT(DABS(1.d0 - T1*T1))*T1
      
      END SUBROUTINE ROT_LSP

cc ************************************************************************
cc     This routine calculates rotation matrixes for Stokes vector
cc     of local plane-parallel radiation field
c      SUBROUTINE ROT_GAU_LS
c      REAL (KND)  Mu_in, Azm_in, Zn, T1, T2, Mu_sc, Azm_sc,
c     $            Si_in, Si_sc, Co_s, Si_s, Rel_az
c      DO J = 1,NR               ! Over line-of-sight
c         Mu_sc = -XM_1(J) ;  Azm_sc = PHI_LOS(J)
c         Si_sc = DSQRT(1.d0 - Mu_sc**2)
c         DO I = 1, NN_MS        ! Over all Gauss zenith angles
c            Mu_in = XM_2(I) ; Si_in = DSQRT(1.d0 - Mu_in**2)
c            DO K = 1, N_PHI_MS  ! Over all azimuthal angles
c               Azm_in = AZM_MS(K)
c               Rel_az = Azm_sc - Azm_in
c               IF( Rel_az < 0.d0 ) Rel_az = 2.d0*Pie + Rel_az
c               Co_s = Mu_in*Mu_sc + Si_sc*Si_in*DCOS(Rel_az)
c
c               Si_s = DSQRT(1.D0 - Co_s**2)
cc              ... Scattering plane is meridional plane:
c               IF(REAL(Rel_az) == REAL(Pie).OR.REAL(Rel_az) == 0.0) THEN 
c                  RoL(J,I,K,1) = 1.d0 ; RoL(J,I,K,2) = 0.d0
c                  RoR(J,I,K,1) = 1.d0 ; RoR(J,I,K,2) = 0.d0                 
c                  CYCLE
c               ENDIF
c               IF( Rel_az > Pie ) THEN 
c                  Zn = 1.d0
c               ELSE
c                  Zn = -1.d0
c               ENDIF
c               IF( DABS(Mu_sc) == 1.d0 ) THEN
c                  T2 = DSIGN(1.d0,Mu_sc)*DCOS(Rel_az)
c               ELSE
c                  T2 = -(Mu_in - Mu_sc*Co_s)/Si_sc/Si_s
c               ENDIF
c
c               IF( DABS(Mu_in) == 1.d0 ) THEN
c                  T1 = DSIGN(1.d0,Mu_in)*DCOS(Rel_az) 
c               ELSE
c                  T1 = -(Mu_sc - Mu_in*Co_s)/Si_in/Si_s
c               ENDIF
c
c               RoL(J,I,K,1) = 2.d0*T2*T2 - 1.d0 
c               RoL(J,I,K,2) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
c
c               RoR(J,I,K,1) = 2.d0*T1*T1 - 1.d0 
c               RoR(J,I,K,2) = Zn*2.d0*DSQRT(DABS(1.d0 - T1*T1))*T1
c            ENDDO
c         ENDDO
c      ENDDO
cc
cc     Arrays RoL(:,1:2) and RoR(:,1:2) conain only two elements
cc       of the full "left" and "right" rotation matrixes as follows
cc
cc        |  1      0         0        0  |
cc        |  0   RoL(:,1)  -RoL(:,2)   0  |
cc        |  0   RoL(:,2)   RoL(:,1)   0  |
cc        |  0      0         0        1  |
cc
c      END SUBROUTINE ROT_GAU_LS
c
c *************************************************************************
      SUBROUTINE ROT_MATR
      REAL (KND)  Azm, Zn, T2, Xmu, Mu0
      Rot_l = 0.d0
c      Mu and Mu0 have to be correspond to the tau-axis
c      write(*,*) '   --->> cos(theta) and cos(theta_0) along l-of-s'
      DO J = 1,NR
c         WRITE(*,'(3x,I3,2(2x,e12.4),2x,F7.3)') 
c     $        J, XM_1(J), XM_S1(J), DACOS( COSCA(J) )/Pie*180.

         Xmu = -XM_1(J)  ;  Mu0 = XM_S1(J) ; Azm = PHI_LOS(J)
         IF( REAL(Azm) == REAL(Pie) .OR. REAL(AZM) == 0.0) THEN 
            Rot_l(J,2) = 1.d0
            Rot_l(J,3) = 0.d0
            CYCLE
         ENDIF
         IF( Azm > Pie ) THEN 
            Zn = 1.0d0
         ELSE
            Zn = -1.0d0
         ENDIF
         IF( DABS(Xmu) == 1.0d0 ) THEN
            T2 = DSIGN(1.0d0,Xmu)*DCOS(Azm)
         ELSE
            T2 = -( Mu0 - Xmu*COSCA(J) )
     $           /DSQRT((1.D0 - Xmu*Xmu)*(1.D0-COSCA(J)**2))
         ENDIF
         Rot_l(J,2) = 2.d0*T2*T2 - 1.d0 
         Rot_l(J,3) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
      ENDDO
      END SUBROUTINE ROT_MATR

* *************************************************************************
      SUBROUTINE INT_ON_TH_LEV(Poi, INI, Rez)
      INTEGER     Poi, INI, K2
      REAL (KND)  ZV, Rez(NSOL,N_SFN,4)

c        WRITE(*,*) '=======> ... TH_FLAG'
        ZV = ZLS(Poi)
        CALL BIAKIMA_Y( Z_INV(NLAYERS-INI+1), 
     $       SZA_EXT, ZV, X_RAD(NLAYERS-INI+1,:,:,:), Rez)
        K2 = SID(Poi)
        D1 = (ZV - Z_FINE(K2+1))/(Z_FINE(K2) - Z_FINE(K2+1))

        AL_1(Poi) = ALO(K2+1)  + (ALO (K2) - ALO (K2+1))*D1
        SCRL(Poi) = SCRO(K2+1) + (SCRO(K2) - SCRO(K2+1))*D1 

        IF( DO_AEROSOL ) THEN
           G_aer(Poi)  = ASCO(K2+1) + (ASCO(K2) - ASCO(K2+1))*D1
           SCA_LS(Poi) = SCAO(K2+1) + (SCAO(K2) - SCAO(K2+1))*D1
        END IF   

        IF( DO_CLOUD ) THEN
           SCC_LS(Poi) = SCCO(K2+1,1) + (SCCO(K2,1) - SCCO(K2+1,1))*D1
        ENDIF

        IF( TE%ADD_PLANCK ) Therm(Poi) = TE%PLA_FINE(K2+1) 
     $     + (TE%PLA_FINE(K2) - TE%PLA_FINE(K2+1))*D1  
      END SUBROUTINE INT_ON_TH_LEV
C     *****************************************************************
      END
C *************************************************************************
