C***********************************************************************
C
C       SCIATRAN - Version 4.7 - Last modification: 18 Aug 2023
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE SFI_SPHER(JPOS, JJESFT, Mha, POS, RAD, AL, E_W, OSM, 
     $     LOA, LSE, PWF, SRR, SRR_Jact, IND_LS, TE,
     $     ALBEDO, FLU_int, Ref_surf, FLUX, ADI, RADOUT,
     $     NN_MS)
c     ... Called by "st_dom_tau.f"

      USE AEROSOLS,    ONLY: IDX_AER, N_dwr, Dwr_mi
      USE BRDF,        ONLY: Npg
      USE CLD_INP,     ONLY: Ydb_ang, MLCS, Phm_const
      USE CLOUDS,      ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR, SCAT_ANG
      USE EMISSION,    ONLY: N_AT
      USE GRIDS,       ONLY: NLAYERS, ZZZ, Z_FINE, NSOL, SZA_EXT, 
     $                       SOL_GRD, ALT_EXT, Z_INV, NL_F => NL_FINE, 
     $                       X_INV, INTERP_IDX
      USE GT_POLYN,    ONLY: NLg => NLEGEN, XM_G => XM, AG1
      USE INTERN_DIMS, ONLY: Nm_vs, Nvs, Nlg_ray, Leg_aer => NLG_ALL_A, 
     $                       Leg_cld => NLG_ALL_C, Nm_ice
      USE IRRAD,       ONLY: SOL_FLUX, NO_SUN
      USE LOG_PARS,    ONLY: LOG_LEVEL, LUNIT, N_DSO, N_ESO
      USE LOS_Angles,  ONLY: DirSun
c      USE NODES,       ONLY: NN_MS => N_NODE
      USE RAMAN,       ONLY: RRS_ADJ, WL_FACT, LINE_GAS, Sigma_RR, 
     $                       Phot_to_Wat
      USE RTM_OUTPUT,  ONLY: N_OP, DEL_RRS
      USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT, X_SHAD,
     $                       NFA => N_FOV_ANG, ONE_MS_SZ, FOV_WEIGHTS
      USE ST_INPUT,    ONLY: APH_TYPE
      USE ST_VARS,     ONLY: Pie, DO_AEROSOL, Vec_Sca, WF_FLAG, Kvs,
     $                       WL_GRID, N_RHS, NSTRMS, BRDF_FLAG, Ns_wf,
     $                       Nc_der, VST, Emi_FLG, WF_MS_FLAG,
     $                       GeoD => GeoD_Out, PreCalcAngs, Ray_to_WF
      USE TOOLS,       ONLY: INQ_WF, Get_Sza_Idx, LOS_MS_IDX
      USE TYPES,       ONLY: KND, PWF_TYPE, LOA_type, PCC_TYPE, TE_TYPE,
     $                       OSM_TYPE, RRS_TYPE, AuxDataInterp, LSE_type
      USE WF_NUM,      ONLY: NUM_WF, NO_SOL, NO_LS, AL_1_BASE, AL_1_PERT
c     ------------------------------------------------------------------

c     ---------------------------
      INTEGER NN_MS, SRR_Jact
      INTEGER JPOS, JJESFT, Mha, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
      REAL (KND)  ALBEDO, FLU_int, E_W, POS, 
     $     FLUX(Kvs*NSOL), AL(NLAYERS), Ref_surf(N_RHS,Nvs*(NSTRMS+Npg))
      REAL (KND)  RADOUT(N_OP,NLO,Nvs)
      REAL (KND), ALLOCATABLE :: RAD(:,:,:)
      TYPE (AuxDataInterp), TARGET :: ADI
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE (PWF_type) :: PWF    ! Weighting functions
      TYPE (RRS_type), TARGET :: SRR    ! RRS related parameters
      TYPE(LSE_type), TARGET  :: LSE(GeoD%NAlts, GeoD%NVA)
      TYPE (TE_type)  :: TE     ! Thermal emission 
      
      INTENT(IN) :: JPOS, JJESFT, Mha, IND_LS, ALBEDO, E_W, POS,
     $     FLUX, AL, Ref_surf, LOA, OSM, PWF, ADI
c     $     ,RAD
      INTENT(INOUT) :: RADOUT, SRR
      ! Local variables
      CHARACTER (LEN=*),  PARAMETER:: R_NAME = 'SFI_SPHER'
      INTEGER  N_OUT, Idl, Ids, Idn, II, N_LOS, J, I, K, L, K1, 
     $     K2, R_DIM, Npl, Npo, NLN1, NL_NEW, Js, Jss, ITH, J_OUT, 
     $     NSTR, NR, N_DS, Jv, Jf, Jw, Jw1, Jex, R_lin, Ka,
     $     ID_CLD, NRL, NNv, To_act, Iae, Nsf, Idr, Index, IV, NTH,
     $     IS, IP, IA, IMS1, IMS2
      INTEGER  LL(GeoD%MaxLS)
      INTEGER, POINTER ::  SID(:), IOB(:)
      INTEGER, ALLOCATABLE :: CI(:)
      LOGICAL  DO_WF
      REAL (KND) Mu_Ls, TTT1, P_F, SD, Z_OUT, D1, IRR, IR0, Mult, WFOV
      REAL (KND) Pol_ls(NLg,GeoD%MaxLS), Pol_ga(NLg,NN_MS), LBC(Nvs),
     $     XM_2(NN_MS), AG1_2(NN_MS), DUM(Nvs), BC_Dir(Nvs), Te_Surf,
     $     FL(NN_MS*Nvs, GeoD%MaxLS), S_R(GeoD%MaxLS,Nvs,1+N_dwr),
     $     BC_Dif(Nvs), BC_wf(Nvs)
      REAL (KND), ALLOCATABLE :: DS_SUN(:,:), Del_ss(:,:,:,:,:,:),
     $     COF(:,:), COF_FI(:,:), Wes(:), COF_LS(:,:), AL_0(:,:,:),
     $     AL_F(:,:,:), AL_L(:,:,:), S_TE(:)
      TARGET :: AL_L, COF_LS
      
      REAL(KND) Fg_INT(Nvs), Tls(NSOL,Nvs), X_ref(NSTRMS+1)

      REAL (KND), DIMENSION (NSOL,NN_MS*Nvs,4) :: Y_TH, Y_FP
      REAL (KND), DIMENSION (GeoD%MaxLS,Nvs) :: SSSF, S_F, S_F_Tes
      REAL (KND), DIMENSION (GeoD%MaxLS) :: AL_1, SOL_EXP,  
     $     S_H, COSCA, SCRL
      REAL (KND), POINTER :: ALO(:), SCRO(:), SCAO(:), ASCO(:), DM_AO(:)
c     ... Test arrays for WF functions in ss approximation
      REAL (KND) S_R_Tes(GeoD%MaxLS,Nvs,1+N_dwr)
      REAL (KND) R_SNG(NLAYERS,NSOL), RAD_OUT(NLO,Nvs),
     $     SUM_MOM(NLg, GeoD%MaxLS), Phm(GeoD%MaxLS,Nvs),
     $     Rot_l(GeoD%MaxLS,Nvs)
      REAL (KND) RESULT(GeoD%NAlts,GeoD%NVA,GeoD%NSol,GeoD%NPhi,Nvs) ! A
      REAL (KND) X_RAD(NLAYERS, NSOL, NN_MS*Nvs, 4), 
     $     Y_ALL(NSOL, NN_MS*Nvs, NL_F, 4)
      REAL (KND), ALLOCATABLE :: SF_TE(:), Phm_mc(:,:), EfWI_LS(:,:),
     $     SCC_LS(:,:), SCCNO_LS(:,:), DS_sun_surf(:), EMCL(:,:), 
     $     EMCO(:,:), S_RR(:,:), DM_LS(:), G_aer(:), SCA_LS(:), EX_1(:)
      REAL (KND), POINTER :: P_MOM(:), XM_1(:), D_DS(:,:), EX_L(:),
     $     PHI_LOS(:), XM_S1(:), XM_SG(:), SCCO(:,:), SCC_NSO(:,:),
     $     ZLS(:)
      TYPE (PCC_type) :: PCC    ! Photochemistry
c     ==================================================================
      INTERFACE
         SUBROUTINE PHM_ALONG_LS (JPOS, Mh, IMS1, IMS2, XM_1, RAY_MOMS,
     $        AER_MOMS, CLD_MOMS, SCRL, EMCL, H36, SCAL, DM_LS, SCCL,
     $         Phi, FL, IOB, S_F, SRR, S_RR)
            USE CLOUD_INPUT, ONLY: Nphr
            USE INTERN_DIMS, ONLY: Nlg_Ray, Nm_vs, NL_AER, NLG_AER,
     $        NL_CLD, NLG_CLD, Nm_ice
            USE TYPES,       ONLY: KND, RRS_TYPE
            INTEGER      JPOS, Mh, IMS1, IMS2, IOB(:)
            REAL (KND), DIMENSION (:) :: XM_1, SCRL, Phi
            REAL (KND)  RAY_MOMS(Nlg_Ray,Nm_vs),
     $           AER_MOMS(Nlg_Aer,NL_Aer,Nm_vs),
     $           CLD_MOMS(Nlg_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs)
            REAL (KND), ALLOCATABLE :: EMCL(:,:), H36(:,:,:), S_RR(:,:),
     $           SCAL(:), DM_LS(:), SCCL(:,:) 
            REAL (KND)  FL(:,:), S_F(:,:)
            TYPE (RRS_type) :: SRR 
       END SUBROUTINE PHM_ALONG_LS

       SUBROUTINE SET_OUT_RAD (ARR_INP, ARR_OUT)
          USE SPHER_PARS,  ONLY: N_LOS_OUT
          USE INTERN_DIMS, ONLY: Nvs
          USE TYPES,       ONLY: KND
          REAL (KND) ARR_INP(:,:,:,:,:), ARR_OUT(N_LOS_OUT,Nvs) 
       END SUBROUTINE SET_OUT_RAD

       SUBROUTINE BIAKIMA_Y (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
          USE GRIDS,       ONLY: NSOL
          USE TYPES,       ONLY: KND
          REAL (KND)  X_RAD(:,:,:), Y_ALL(:,:,:) 
          REAL (KND)  Y_LOC(NSOL + 4)
          REAL (KND)  X_NEW, X_AX
       END SUBROUTINE BIAKIMA_Y

       SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, 
     $     ALBEDO, FLU_int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
          USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, 
     $                    Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
          USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
          USE INTERN_DIMS, ONLY: Nvs
          USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
          USE TYPE_VARS, ONLY: KND
          USE WATER,  ONLY: SS_PDF
          INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
          REAL (KND), INTENT(IN) :: ALBEDO, FLU_int, POS, SOL_EXP,
     $          PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
          REAL (KND) BC_S1(Nvs), LBC(Nvs)
          REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
          OPTIONAL :: REF_SUR
       END SUBROUTINE REFL_DIR_SPH
      END INTERFACE
c     ==================================================================
      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1
      BC_Dir(:) = 0.d0 ; LBC(:) = 0.d0
      DO_WF = .FALSE.
      IF( WF_FLAG ) DO_WF = .TRUE.
      NNv = Nvs*NSTRMS
      X_ref(1:NSTRMS) = XM_G(:) ; X_ref(NSTRMS+1) = 1.d0

      IF( LOG_LEVEL(1) >= N_DSO) THEN
         Js = 1
         WRITE(*,*) ' *******  "sfi_spher" Mha =  ', Mha
         WRITE(*,*) '    Reflected from surface '
         WRITE(*,*) '         for SZA    : ',Js
         WRITE(*,*) '                NNv = ', NNv
         K1 = 1
         DO K = NNv+1,2*NNv
            WRITE(*,'(5x,I5,*(2x,E12.4))') K1, DACOS(XM_G(K1))/Pie*180.,
     $           RAD(NLAYERS-1,Js,K)
            K1 = K1 + 1
         ENDDO
      ENDIF
c         PAUSE
c     WRITE(*,*) '  SHAPE(RAD) ', SHAPE(RAD)
c      WRITE(*,*)  '  2 SUM(RAD) - ', SUM( RAD )
cc               L = NLAYERS  ;  K2 = 0
c               L = 1  ;  K2 = 0
c               WRITE(*,*) '  Upward radiation: NSOL = ',NSOL
c               WRITE(*,*) '     before interpolation '
c               WRITE(*,*) '  At the level L =  ',L
c               DO K1 = 1, NNv, Nvs
c                  K2 = K2 + 1
c                  WRITE(*,'(1x,I3,3(2x,e12.4))') 
c     $                 K2, RAD(L,NSOL,K1)
cc     $                 K2, (RAD(L,I,NNv+K1), I=1,NSOL)
c               ENDDO
c               K2 = 0
c               WRITE(*,*) '  Upward radiation:  Q components'
c               WRITE(*,*) '     before interpolation '
c               WRITE(*,*) '  At the level L =  ',L
c               DO K1 = 2, NNv, Nvs
c                  K2 = K2 + 1
c                  WRITE(*,'(1x,I3,3(2x,e12.4))') 
c     $                 K2, (RAD(L,I,NNv+K1), I=1,NSOL)
c               ENDDO
c      ... Interpolation coefficient for the radiance field
c      WRITE(*,*)  '    Solar angles: '
c      DO I = 1, NSOL
c         WRITE(*,*) I, ACOS(SOL_GRD(I))/Pie*180.
c      ENDDO
      
      IF( Emi_FLG ) THEN
         ALLOCATE( EMCL(N_AT,GeoD%MaxLS), EMCO(N_AT,NL_F) )
         DO Ka = 1, N_AT
            CALL LIN_FINE_1(NLAYERS, ZZZ, NL_F, Z_FINE, INTERP_IDX, 
     $           LOA%EMC(Ka,:), EMCO(Ka,:), .FALSE.)
         ENDDO
      ENDIF
      
C     Interpolation of diffuse radiance
C     **********************************
      IF (LOG_LEVEL(1) >= N_DSO)  
     $     WRITE(LUNIT,*) ' CALL LIN_END_FINE :: NL_FINE = ', NL_F

      ALO => ADI%AL0_OUT ;  SCRO => ADI%SCR_OUT

c     ... Aerosol on the fine grid:
c     ------------------------------------------------------------------
c     delta-M scaling factor
      DM_AO => ADI%DM_AO  
c         scattering coeff. without delta-m scaling
      SCAO => ADI%SCA_OUT
c         asymmetry parameter
      ASCO => ADI%ASC_OUT
c     ------------------------------------------------------------------
      
      IF( DO_CLOUD ) THEN
         SCC_NSO => ADI%SCC_NSO ! without delta-m scaling
         SCCO    => ADI%SCCO       ! with delta-m scaling
      ENDIF

      Te_Surf = 0.d0
      IF( TE%ADD_PLANCK ) THEN
         ALLOCATE( SF_TE(GeoD%MaxLS), S_TE(GeoD%MaxLS) )
         S_TE = 0.d0
      ENDIF
c     ... In the array RAD(:,:,1:2*NNv) elements 
c             1:NNv   - contain downward radiation
c         NNv+1:2*NNv - contain upward radiation
c     ... Over all zenith angles (up and down) and all Stokes components
c          NN_MS = 2*NSTRMS was set in "rtm_init_first.f"
      DO Idn = 1, NN_MS*Nvs
         R_SNG(:,:) = SNGL( RAD(:,:,Idn) )
         CALL BIAKIMA_X(NLAYERS, ALT_EXT, NSOL, X_RAD(:,:,Idn,:), R_SNG)
      ENDDO   ! over Gaussian angles and all Stokes components (Idn)
      DO J = 1, NL_F
         CALL BIAKIMA_Y(Z_INV(INTERP_IDX(J)), SZA_EXT, Z_FINE(J),
     $        X_RAD(INTERP_IDX(J),:,:,:), Y_ALL(:,:,J,:))
      END DO

      Z_OUT = GeoD%Alt(1) 
      RESULT(:,:,:,:,:) = 0.d0

c     ... RRS employing adjoint technique
c         ===============================
      IF( RRS_ADJ ) CALL RRS_FIRST

      IF(DO_WF) ALLOCATE( DS_SUN(GeoD%MaxLS,NLAYERS),
     $     DS_sun_surf(NLAYERS) ) ! A

      Npo = MAX( Nlg_Ray,Leg_aer,Leg_cld )

      IF (LOG_LEVEL(1) >= N_ESO)  WRITE(LUNIT, '(6x,A,2X,I2)')   
     $     '     ********* Mharm ******* ', Mha
                
      NSTR = NN_MS/2 
      XM_2(1:NSTR)       =  XM_G(:) !Downward
      XM_2(1+NSTR:NN_MS) = -XM_G(:)
      AG1_2(1:NSTR) = AG1(:) ; AG1_2(1+NSTR:NN_MS) = AG1(:)      
      CALL WIGNER_PLG(XM_2, NN_MS, Mha-1, NLg, Pol_ga)

c      WRITE(*,*) ' -------> "sfi_spher.f"  Mharm = ', Mha
      
      DO IV = 1, GeoD%NVA  ! Number of viewing angles including FOV

         Jf = IV  - (CEILING( FLOAT(IV)/FLOAT(NFA) ) - 1)*NFA
         WFOV = FOV_WEIGHTS(Jf)

         Mu_Ls = GeoD%LVG(IA,IV)%C_VA(1)
         NTH   = GeoD%LVG(IA,IV)%NTH
         NR    = GeoD%LVG(IA,IV)%NLS
         ZLS  => GeoD%LVG(IA,IV)%ZLS
         SID  => GeoD%LVG(IA,IV)%ISEL
         IOB  => GeoD%LVG(IA,IV)%IOB

c        ... Output altitude (Z_OUT) at TOA
         IF( Z_OUT == ZZZ(1).AND.Mu_Ls <= 0.0D0 )  CYCLE

         IF( DO_AEROSOL ) 
     $        ALLOCATE(G_aer(NR), SCA_LS(NR), DM_LS(NR) )

         IF( DO_CLOUD ) THEN
            ALLOCATE( CI(NR), SCC_LS(NR,Nm_ice), SCCNO_LS(NR,Nm_ice),
     $           Phm_mc(Nm_ice,Nvs) )
            CI(:) =  INT( CLA(IOB(:),2) )
            DO J = 1, Nm_ice
               SCC_LS  (:,J) = SCCO   (SID(:),J)
               SCCNO_LS(:,J) = SCC_NSO(SID(:),J)
            ENDDO
c            IF( .NOT.Phm_const ) THEN
            IF( .NOT.ANY(MLCS(:)%Phm_co) ) THEN               
               ALLOCATE( EfWI_LS(NR,Nm_ice) )
               DO J = 1, Nm_ice
                  EfWI_LS(:,J) = ADI%EfWI(SID(:),J)
               ENDDO
            END IF  ! Phm_const
         END IF  ! DO_CLOUD
c     ... XM_1 is the cosine of the L-of-S direction with respect 
c         to the downward direction, i.e., XM_1 is the cosine 
c         between the direction of radiation propagation and z-axis.
         XM_1 => GeoD%LVG(IA,IV)%C_VA
         EX_L => LSE(IA,IV)%EX (1:NR)
         ALLOCATE( EX_1(0:NR) )
c        EX_1 => LSE(IA,IV)%EX1(0:NR)
         EX_1 = LSE(IA,IV)%EX1(0:NR)
         D_DS => GeoD%LVG(IA,IV)%DS
c         IF( Mha==1 ) THEN
c            WRITE(*,*) '  IV ',IV, EX_1(0), EX_1(1), EX_1(NR)
c            WRITE(*,*) '  == ',IV,
c     $           LSE(IA,IV)%EX1(0), LSE(IA,IV)%EX1(1),LSE(IA,IV)%EX1(NR)
c         ENDIF
c     ... Setting of scattering and extinction coefficients
c         along l-of-s including interpolation on the tangent height
         CALL SET_COEFF

         IF(GeoD%LVG(IA,IV)%TH_FLG)
     $        CALL INT_ON_TH_LEV(NTH, IOB(NTH), Y_TH)
         IF(GeoD%LVG(IA,IV)%FP_FLG)
     $        CALL INT_ON_TH_LEV(1, IOB(1), Y_FP)
         
         DO IS = 1, GeoD%NSol   ! Number of solar angles
            DO IP = 1, GeoD%NPhi ! Number of azimuth angles
               CALL LOS_MS_IDX(GeoD%LSG(IA,IV,IS,IP)%CG_SZA, IMS1, IMS2)
c                  WRITE(*,*) '   IMS1 and IMS2  ', IMS1, IMS2               
c              ... Azimuthal angle [rad] along line-of-sight
               PHI_LOS => GeoD%LSG(IA,IV,IS,IP)%SAA(:)
               
c              ... Attention !!!
c              XM_S1 is the cosine of the solar angle between the direction
c              of the solar beam and downward direction at a point of L-of-S
               XM_S1 => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  
C              Attention, to pick the appropriate values from the multiple 
c              scattering contribution we need
C              geometrical SZA rather than refractive one.
               XM_SG => GeoD%LSG(IA,IV,IS,IP)%CG_SZA(:)
c               XM_SG => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  
               
c              ... Single scattering source function along LOS
c                  including Raman scattering (if any)
               SSSF(1:Nr,:) = 0.d0 ; S_R(1:NR,:,:) = 0.d0
               SOL_EXP = 0.d0
               IF(DO_WF) DS_SUN = 0.d0
               IF( Mha==1 ) THEN
                  IF( .NOT.NO_SUN ) CALL SINGLE_IN_MS_MODE ! --> SSSF
               ENDIF
c              -----------------------------------               
cc             ... Single scatt test for intensity
c              S_F = 0.d0  ;  BC_Dir  = 0.d0
c              IF( .NOT.DO_WF ) GOTO 998
cc             -----------------------------------            
               CALL GET_SZA_IDX(XM_SG(IMS1:IMS2), LL(IMS1:IMS2))

               FL = 0.d0
               FORALL(L = IMS1:IMS2) FL(:,L) = Y_ALL(LL(L),:,SID(L),1)
               IF (.NOT.ONE_MS_SZ) THEN
                  DO L = IMS1, IMS2
                     K = LL(L)
                     SD = XM_SG(L) - X_INV(K)
                     FL(:,L) =  FL(:,L) + 
     $                    SD*( Y_ALL(K,:,SID(L),2) +
     $                        SD*(Y_ALL(K,:,SID(L),3) +
     $                            Y_ALL(K,:,SID(L),4)*SD) )
                  END DO
               END IF
c              ... As in the case of RAD the first dimension of array FL(:,L)
c                      1:NNv   - contain downward radiation
c                  NNv+1:2*NNv - contain upward radiation
c              Output of interpolated radiation field onto l-of-s grid
c               WRITE(*,*) '  Output file containing FL(:,:)  '
c               WRITE(*,*) '            NN_MS =  ', NN_MS
c               WRITE(*,*) '        Nvs*NN_MS =  ', NN_MS*Nvs
c               OPEN(55,file='./DATA_OUT/FL_OUT.dat',RECL=100000)
c                   DO K1 = 1, NLN2
c                      WRITE(55,'(130(1x,e15.7))') Z_NEW(K1), 
c     $                     (FL(Jss,K1), Jss = 1,NN_MS*Nvs)
c                   ENDDO
c               CLOSE(55)
c *************************************************************************
c               WRITE(*,*) '  Reading of file containing FL(:,:)  '
c               FL = 0.d0
c               OPEN(55,file='./DATA_OUT/FL_OUT.dat')
c                   DO K1 = 1, NLN2
c                      READ(55,*) Z_NEW(K1), 
c     $                     (FL(Jss,K1), Jss = 1,NN_MS*Nvs)
c                   ENDDO
c               CLOSE(55)
c *************************************************************************
c               L  = 1
c               K2 = 0
c               WRITE(*,*) '  Upward radiation: I and Q components'
c               WRITE(*,*) '     after interpolation SZA ', 
c     $              DACOS( XM_SG(L) )/Pie*180.
c               WRITE(*,*) '  At the level L =  ',L
c               DO K1 = 2, NNv, Nvs
c                  K2 = K2 + 1
c                  WRITE(*,'(1x,I3,2(2x,e12.4))') 
c     $                 K2, FL(NNv+K1-1,L), FL(NNv+K1,L)
c               ENDDO
c               PAUSE

               IF( GeoD%LVG(IA,IV)%TH_FLG.AND.
     $              (NTH.GE.IMS1.AND.NTH.LE.IMS2)) THEN
                  K = LL(NTH)
                  FL(:,NTH) = Y_TH(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(NTH) - X_INV(K)
                     FL(:,NTH) =  FL(:,NTH) + SD*(Y_TH(K,:,2) +
     $                    SD*(Y_TH(K,:,3) + Y_TH(K,:,4)*SD))
                  END IF
               END IF
         
               IF(GeoD%LVG(IA,IV)%FP_FLG) THEN
                  K = LL(1)
                  FL(:,1) = Y_FP(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(1) - X_INV(K)
                     FL(:,1) =  FL(:,1) + SD*(Y_FP(K,:,2) +
     $                    SD*(Y_FP(K,:,3) + Y_FP(K,:,4)*SD))
                  END IF
               END IF

               SELECT CASE( Vec_Sca )
               CASE( 'vect_rte' )
c              ... Calculation of: S_F(:,:), S_RR(:,:) in vector case
c                  including Raman scattering
                  CALL PHM_ALONG_LS(JPOS, Mha, IMS1, IMS2, -XM_1,  
     $                 LOA%RAY_MOMS, LOA%AER_MOMS, LOA%CLD_MOMS, 
     $                 SCRL, EMCL, LOA%EPF, SCA_LS, DM_LS, SCC_LS, 
     $                 PHI_LOS, FL, IOB, S_F, SRR, S_RR)
               CASE( 'scal_rte' )
c              ... Calculation of: S_F(:,1), S_R(:,1,:) in scalar case
c                  including Raman scattering
                  CALL PHF_ALONG_LS  ! --> S_F, S_R
               END SELECT

c              ... Integration over line-of-sight RRS contribution both
c                  in scalar and vector case:
               IF( RRS_ADJ ) THEN
                  CALL SUM_RRS_MS
                  IF(SRR%Exi_WL(JPOS)) CYCLE
               END IF
               S_F(1:IMS1-1,:) = 0.d0  ; S_R(1:IMS1-1,1,:) = 0.d0
               S_F(IMS2+1:Nr,:) = 0.d0 ; S_R(IMS2+1:Nr,1,:) = 0.d0
c                         WRITE(*,*) ' Z_NEW(Nr) and ZZZ(NLAYERS) ',
c     $                                Z_NEW(Nr), ZZZ(NLAYERS)
c              ... Total radiation reflected by surface (if any)
c              =================================================       
               LBC(:) = 0.d0
               Surface_Refl: BLOCK
                  Te_Surf = 0.d0
                  BC_Dir = 0.d0 ; BC_Dif = 0.d0 ; BC_wf = 0.d0
                  IF(IMS2 <= 0)                   EXIT Surface_Refl
                  IF( ZLS(IMS2) /= ZZZ(NLAYERS) ) EXIT Surface_Refl
c                 ... Include surface emission into "tg" WF calculations
                  IF( TE%ADD_PLANCK ) Te_Surf = TE%PLANCK_SURF
c                 ... Direct solar radiation reflected by surface
                  IF( Mha==1 ) THEN 
c                    ... The same as in "st_ss_stokes.f". The sign of 
c                        XM_1, and XM_S1 is irrelevant here.
                     CALL REFL_DIR_SPH(1,1,1, XM_1(Nr:Nr), XM_S1(Nr:Nr),
     $                    PHI_LOS(Nr:Nr), POS, ALBEDO, FLU_int,  
     $                    SOL_EXP(Nr), TE%PLANCK_SURF, BC_Dir, LBC)
c                    BC_Dir = 0.d0 ! without reflection of direct radiation
                  ENDIF  ! Mha==1

!     Check if surface in the valid range of SZAs, is not, no reflection
!     of the diffuse radiation
c                 ... Diffuse radiation reflected by surface
c     ------------------------------------------
                  IF( .NOT.BRDF_FLAG ) THEN
c                    ... Lambertian surface reflection (diffuse radiation)
                     Tls = 0.d0 ; BC_Dif(:) = 0.d0
                     Tls(:,1) = Albedo*2.d0*FLUX(1:NSOL)
c                    ... Interpolation at a local solar angle along l-of-s
c                         for all basis solar angles
                     IF( NSOL== 1 ) THEN
                        BC_Dif(1) = Tls(1,1)
                     ELSE
                        CALL AKIMA_INT(NSOL, SOL_GRD, Tls(:,1),
     $                       1, XM_S1(Nr), BC_Dif(1) )
                     ENDIF
                     BC_wf(:) =  BC_Dif(:)
                  ELSE
c                 ... Bidirectional surface reflection
c                 ------------------------------------------------------ 
c                    ... The array Ref_surf contains the reflected 
c                        radiation from the surface (see "st_top_bot.f")
c                        calculated on the solar angles grid
                     DO Jss = 1,NSOL
                        DO K1 = 1,Nvs 
                           CALL AKIMA_INT(NSTRMS+1, X_ref, 
     $                          Ref_surf(Jss,K1:Nvs*(NSTRMS+1):Nvs),
     $                          1, XM_1(Nr), Fg_INT(K1) )
                        ENDDO  ! over Stokes components (K1)
c                       ... Interpolation at a given L-of-S angle
                        Tls(Jss,:) = 2.d0*Fg_INT(:)
                     ENDDO  ! over solar angles grid (Jss)
c                    ... Interpolation at a given solar angle: BC_Dif
                     IF( NSOL== 1 ) THEN
                        BC_Dif(:) = Tls(1,:)
                     ELSE
                        DO K1 = 1,Nvs 
                           CALL AKIMA_INT(NSOL, SOL_GRD, Tls(:,K1),
     $                          1, XM_S1(Nr), BC_Dif(K1) )
                        ENDDO  ! over Stokes components (K1)
                     ENDIF
c                     WRITE(*,'(2x,I3,3(1x,f8.2))') IV,
c     $                    DACOS(XM_1(Nr))/Pie*180.,
c     $                    DACOS(XM_S1(Nr))/Pie*180.,
c     $                    PHI_LOS(Nr)/Pie*180.
c                     !!! VR correction 08.03.2015 !!!
                     D1 = PHI_LOS(NR)*DBLE(Mha-1)
                     BC_Dif(1) = BC_Dif(1)*DCOS( D1 )   !*VST
                     IF( Nvs > 1 ) THEN
                        BC_Dif(2)   = BC_Dif(2)*DCOS( D1 )   !*VST
                        BC_Dif(3:4) = BC_Dif(3:4)*DSIN( D1 ) !*VST
                     ENDIF  
                     BC_wf(:) =  BC_Dif(:) !* VST
                  ENDIF  ! BRDF_FLAG

c                 ------------------------------------------------------
c                  BC_Dif(:) = 0.d0 ! test without diff reflection
               END BLOCK Surface_Refl ! Reflection by surface
            
c              ... Output altitude (Z_OUT) at BOA
               IF( Z_OUT == ZZZ(NLAYERS).AND.Mu_Ls > 0 ) THEN
                  RESULT(IA,IV,IS,IP,:) = BC_Dir(:)
c                  CYCLE
               END IF

c ==================== Block WF ==================================  
               IF( DO_WF ) THEN 
                  IF( WF_MS_FLAG(JPOS) ) THEN
c                 ... WF in MS representation
                     IF( TE%ADD_PLANCK ) THEN
                        S_F_Tes(:,1)   = S_F(:,1) + SF_TE(:)
                        S_R_Tes(:,1,1) = S_TE(:)
                     ELSE
                        S_F_Tes = S_F !0.d0
                        S_R_Tes = S_R  
                     ENDIF
c                    ... Because there is no direct reflected light 
c                        from surface
                     DS_sun_surf(:) = 0.d0 ! Eto verno ! 
                     LBC(:) = LBC(:)*VST
                  ELSE
c                 ... WF in SS approximation:
                     S_F_Tes = 0.d0 ; S_R_Tes = S_R ; BC_wf(:) = 0.d0
                     IF( Mha > 1 )  S_R_Tes = 0.d0 ; LBC(:) = LBC(:)*VST
                     DS_sun_surf(:) = DS_SUN(Nr,:)
                  ENDIF

cc                ***************************************************
c                  WRITE(*,*) '   ****** SS mode for WF ***** '
c                  S_F_Tes = 0.d0 ; S_R_Tes = S_R ; BC_wf(:) = 0.d0
c                  IF( Mha > 1 )  S_R_Tes = 0.d0 ; LBC(:) = LBC(:)*VST
c                  DS_sun_surf(:) = DS_SUN(Nr,:)
cc                 ***************************************************
c                  WRITE(*,*) ' S_R do sph ==> ', S_R(1:10,1,1)

                  CALL SPHER_WF(SSSF, DS_SUN, S_F_Tes, S_R_Tes, EX_L, 
     $                 EX_1, Nr, D_DS, IOB, ZLS, Mha, AL_1, PWF, IV, IS, 
     $                 IP, JPOS, TE%ADD_PLANCK)
c                 ... Array DS_SUN contains after call of "spher_wf.f"
c                     pathlength elements along l-of-s
c                  WRITE(*,*) ' Ds_view ', SUM( DS_SUN(Nr,:) ) 
                  Index = GeoD%ILO_SVFP(IV,IS,IP)
c                 ... Contribution of extinction coefficient variation 
c                     within viewing direction transmission "exponent"
c                     of direct and diffuse reflected from surface radiation
c                     ( See Eq.(T2-sph) in STB )
                  DO Jw = 1,Ns_wf   ;   Jw1 = Nc_der(Jw)
c                     WRITE(*,'(*(2x,e12.4))')
c     $                    LBC(Jw1), SOL_EXP(Nr)*EX_L(Nr)*XM_S1(Nr)*WFOV
c                     WRITE(*,'(*(2x,e12.4))') DS_sun_surf(:)
c                     WRITE(*,'(*(2x,e12.4))') DS_SUN(Nr,:)
                     PWF%WF_esp(:,Index,Jw) = PWF%WF_esp(:,Index,Jw) 
c                           ... See Eq.(T2-sph) in STB
     $                    - BC_wf(Jw1)*DS_SUN(Nr,:)*EX_L(Nr) *WFOV 
c                           ... See Eq.(T1-sph) in STB
     $                    - LBC(Jw1)*( DS_sun_surf(:) + DS_SUN(Nr,:) )
     $                      *SOL_EXP(Nr)*EX_L(Nr)*XM_S1(Nr)*WFOV
c                           ... Thermal emission (absorption only!!!)
     $                    - Te_Surf*DS_SUN(Nr,:)*EX_L(Nr) *WFOV
c                     WRITE(*,'(*(2x,e12.4))') PWF%WF_esp(:,Index,Jw)
                  ENDDO
               ENDIF  !  DO_WF
c ==================== End block WF ==================================  

c              For test with numerical perturbations
               IF( NUM_WF ) THEN 
c                  To calulate without contribution from solar exponent
                  IF( NO_SOL ) AL_1(:) = AL_1_PERT(:)
c                 To calulate without contribution from LS exponent
                  IF( NO_LS  ) AL_1(:) = AL_1_BASE(:)
               ENDIF

cc             ... Exclude multiple scattering contribution
c              S_F = 0.d0  ;  BC_Dif  = 0.d0
cc             ... Exclude single scattering contribution
c               SSSF = 0.d0  ;  BC_Dir  = 0.d0

c              ... Sum contribution of single and multiple scattering
c                  source functions
               S_F(:,:) = SSSF(:,:) + S_F(:,:)
               IF( TE%ADD_PLANCK ) S_F(:,1) = S_F(:,1) + SF_TE(:)

               DUM = 0.d0
               DO I = 1, Nr
c                  DUM = DUM + S_F(I,:)*EX_L(I)*(D_DS(1,I) + D_DS(2,I))
                  DUM = DUM + S_F(I,:)/AL_1(I)*(EX_1(I-1) - EX_1(I))
               END DO
c               WRITE(*,'(2x,I3,*(2x,e12.4))') Iv, BC_Dir(1), BC_Dif(1)
               RESULT(IA,IV,IS,IP,:) = DUM(:)
     $              + (BC_Dir(:) + BC_Dif(:)/VST)*EX_L(Nr)

               IF( LOG_LEVEL(1) >= N_DSO ) THEN
                  WRITE(*,*) '  Contribution in RESULT: '
                  WRITE(*,*) '  Res: ', IV, IP, RESULT(IA,IV,IS,IP,:)
               ENDIF  ! debug output
               
            END DO ! over azimuth angles (IP)
         END DO ! over solar zenith angles (IS)

         NULLIFY(ZLS, SID, IOB, XM_1, EX_L, XM_S1, XM_SG)
         DEALLOCATE( EX_1 )

         IF( DO_AEROSOL ) DEALLOCATE(G_aer, SCA_LS, DM_LS) ! DA
         IF( DO_CLOUD ) THEN
            DEALLOCATE(CI, SCC_LS, SCCNO_LS, Phm_mc) ! DA
c            IF( .NOT.Phm_const ) DEALLOCATE( EfWI_LS )
            IF( .NOT.ANY(MLCS(:)%Phm_co) ) DEALLOCATE( EfWI_LS )
         END IF
      END DO  ! over viewing angles including FOV (IV)

c      WRITE(*,*) '  ----- sfi_spher.f ------ '
c      WRITE(*,*) '  PWF%WF_esp(:,:,1) is calculated:'
c      WRITE(*,*) SUM( PWF%WF_esp(:,:,1) )

      IF( RRS_ADJ.AND.SRR%Exi_WL(JPOS) ) THEN
         DO Jex = 1,SRR%NUM_IN(JPOS)
            To_act = SRR%ACT_IN(JPOS,Jex) 
            CALL SET_OUT_RAD(Del_ss(:,:,:,:,:,Jex), RAD_OUT)     
            DEL_RRS(To_act,1:NLO,:) = DEL_RRS(To_act,1:NLO,:) 
     $           + RAD_OUT(1:NLO,:)*E_W
         ENDDO
c        ... Standard calculations for excitation wavelength do not need
c             in the case of rotational Raman scattering
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN 
      ENDIF
      
c     ... Raman output for an actual wavelength
      IF( RRS_ADJ ) THEN
         CALL SET_OUT_RAD(Del_ss(:,:,:,:,:,1), RAD_OUT) 
         DEL_RRS(SRR_Jact,1:NLO,:) = DEL_RRS(SRR_Jact,1:NLO,:) 
     $        + RAD_OUT(1:NLO,:)*E_W
      ENDIF

c     ... Standard output
c      WRITE(*,*) '   ***** Mha =  ',Mha
      CALL SET_OUT_RAD(RESULT, RAD_OUT)
      
c     ... This array contains the contribution to the intensity/Stokes 
c         vector of the Fourier harmonic Mha.
c         If Mha > 1 will be multiplied by 2 in "gpp_converge.f".
      RADOUT(1,1:NLO,:) = E_W*RAD_OUT(1:NLO,:)
      IF( LOG_LEVEL(1) >= N_DSO ) THEN
         WRITE(*,*) '  Contribution in RADOUT :', NLO
         WRITE(*,'(3x,4(1x,E15.7))') RADOUT(1,1:NLO,:)
      ENDIF  ! debug output

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
c    Below: RRS_FIRST, SINGLE_IN_MS_MODE, SUM_RRS_MS, PHF_ALONG_LS,
c           ROT_MATR,  SET_COEFF, INT_ON_TH_LEV
c **********************************************************************
      SUBROUTINE RRS_FIRST
         Mult = 1.d0
         IF( Mha>1 ) Mult = 2.d0 
         ALLOCATE( S_RR(GeoD%MaxLS,Nvs) )
         ALLOCATE( Wes(SRR%M_ESF) )

c        ... Actual wavelength ...
         IF( SRR%ACT_WL(JPOS) ) THEN
c           ... Number of Raman lines (dummy)
            NRL = 1
            ALLOCATE( COF(NLAYERS,NRL), COF_FI(NL_F,NRL), 
     $           COF_LS(GeoD%MaxLS,NRL) )
c           ... Set solar irradiance at actual WL corresponding
c               to its position in the extended grid
            IRR = -SOL_FLUX(JPOS)/Pie*Mult  ;  To_act = SRR_Jact
c           ... Save extinction coefficient for actual wavelength
            SRR%EXT_ACT(:,To_act,JJESFT) = LOA%AL(:)
c     ... Save transmission along l-of-s at actual wavelength
            FORALL(IV = 1:GeoD%NVA) SRR%LS_ACT(IA,IV)%
     $           EX(:,To_act,JJESFT) = LSE(IA,IV)%EX(:)
            COF(:,1) = SRR%R_TOT*LOA%SCR(:)

         ELSE
c           ... Set extraterrestrial solar flux at excitation wavelength
            IRR = SOL_FLUX(JPOS)/Pie*Mult
c            IRR = 0.d0
c           ... Number of Raman lines
            NRL = SRR%NUM_IN(JPOS)            
c          ... Number of ESFT coefficients at actual wavelength
            ALLOCATE( COF(NLAYERS,NRL), COF_FI(NL_F,NRL), ! A
     $           COF_LS(GeoD%MaxLS,NRL),                  ! A
     $           AL_0( NLAYERS,SRR%M_ESF,NRL),            ! A
     $           AL_F(    NL_F,SRR%M_ESF,NRL),            ! A
     $           AL_L(GeoD%MaxLS,SRR%M_ESF,NRL) )         ! A
            DO Jex = 1,SRR%NUM_IN(JPOS)
               To_act = SRR%ACT_IN(JPOS,Jex) 
               R_lin  = SRR%RIN_IN(JPOS,Jex)
               Iae    = SRR%A2E(To_act)
c              ... Calculate Raman scattering coefficient
               COF(:,Jex) = WL_FACT*Sigma_RR(R_lin,:)/WL_GRID(Iae)**4
     $              *SRR%GAM(LINE_GAS(R_lin))
c              ... Number of ESFT coefficients at actual wavelength
               Nsf = SRR%NESFT(To_act)
c                       WRITE(*,*) '   Nsf = ', Nsf
c                       WRITE(*,*) SRR%NESFT(To_act)
               AL_0(:,1:Nsf,Jex) = SRR%EXT_ACT(:,To_act,1:Nsf)
c              ... Interpolation on the "fine" altitude grid
               DO I = 1,Nsf
                  CALL LIN_FINE_1(NLAYERS,ZZZ,NL_F,Z_FINE,INTERP_IDX, 
     $                 AL_0(:,I,Jex), AL_F(:,I,Jex), .FALSE.)
               ENDDO
            ENDDO
         ENDIF  ! excitation wavelength
         ALLOCATE(
     $        Del_ss(GeoD%NAlts,GeoD%NVA,GeoD%NSol,GeoD%NPhi,Nvs,NRL))
         Del_ss = 0.d0
c        ... Interpolation on the "fine" altitude grid
         DO I = 1,NRL
            CALL LIN_FINE_1(NLAYERS, ZZZ, NL_F, Z_FINE, INTERP_IDX, 
     $           COF(:,I), COF_FI(:,I), .FALSE.)
         ENDDO  ! over Raman lines (I)
      END SUBROUTINE RRS_FIRST

c *************************************************************************
c     This subroutine calculates the contribution in radiation along 
c     line-of-sight from the direct scattered solar beam. Therefore, it 
c     should be used here all phase matrix expansion coefficients.

      SUBROUTINE SINGLE_IN_MS_MODE
      INTEGER  Isf, I1, I2, Ia1, Ia2, Jmc, Ncl, N_db
      REAL (KND)  Aef, DL, DLa, Ph_W, Wig0_ss(Npo,NR), Wig2_ss(Npo,NR),
     $     HH(NR,Nvs)
      REAL (KND)  Rang(Ydb_ang), Rco(Leg_cld)
      REAL (KND), POINTER :: ALP(:), EXL(:), Rdb(:), Pma(:,:)

c       Scattering angle along line-of-sight
c       ------------------------------------
        COSCA(1:NR) = -XM_1(1:NR)*XM_S1(1:NR) 
     $     + DSQRT(1.d0 - XM_1(1:NR)*XM_1(1:NR))
     $     *DSQRT(1.D0 - XM_S1(1:NR)*XM_S1(1:NR))
     $     *DCOS(PHI_LOS(1:NR))

c     Solar exponent at all point along LS
c       ------------------------------------
        ASSOCIATE (LVG => GeoD%LVG(IA,IV), LSG => GeoD%LSG(IA,IV,IS,IP))
        CALL DirSun (NR, XM_S1, LSG%TH, ZLS, LVG%IOB, AL_1(1:NR), 
     $       ALO, PCC, SOL_EXP(1:NR), LSG%REFR_DSB, PreCalcAngs, DO_WF,
     $       DS_SUN, LSG%DSB)
        END ASSOCIATE

c       Legendre polynomials for all scattering angles
c     ----------------------------------------------
        CALL WIGNER_PLG(COSCA(1:NR), NR, 0, Npo, Wig0_ss(:,1:NR))
        IF( Nm_vs > 1 ) THEN
           CALL WIGNER_PLG(COSCA(1:NR), NR, 2, Npo, Wig2_ss(:,1:NR))
           CALL ROT_MATR
        ENDIF

c       ... Raman scattering should be accounted for within the 
c           single scattering mode
        IF( RRS_ADJ ) THEN
           Npl = Nlg_Ray  ;  P_MOM => SRR%RAM_MOMS(1:Npl,1)
           Phm(:,:) = 0.d0
           FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0_ss(1:Npl,J)*P_MOM(:) )
           IF( Nvs > 1 ) THEN
              P_MOM => SRR%RAM_MOMS(1:Npl,5) 
              FORALL( J = 1:NR ) Phm(J,2:3) = Rot_l(J,2:3)
     $             *SUM( Wig2_ss(1:Npl,J)*P_MOM(:) )
           ENDIF

           IF( .NOT.SRR%Exi_WL(JPOS) ) THEN
c            ... S_F is used as dummy array within "ram_ssf.f"
              FORALL( K=1:Nvs ) S_F(1:Nr,K) = 
     $             COF_LS(1:Nr,1)*Phm(1:NR,K)*SOL_EXP(1:Nr)/4.d0
c          ... Numerical integration along a line-of-sight
              DUM = 0.0D0
              DO I = 1, Nr
c                 DUM = DUM + S_F(I,:)*EX_L(I)*(D_DS(1,I) + D_DS(2,I))
                 DUM = DUM + S_F(I,:)/AL_1(I)*(EX_1(I-1) - EX_1(I))
              END DO
c            ... - Raman scattering contribution (loss) in 
c                the single scattering approximation
              Del_ss(IA,IV,IS,IP,:,1) = Del_ss(IA,IV,IS,IP,:,1) +
     $             IRR*DUM(:)
           ELSE
c             ... Over all Raman lines within corresponding excitation bin 
              Ph_W = 1.d0
              DO Jex = 1,SRR%NUM_IN(JPOS)
c                ... The actual wavelength number which becomes
c                    contribution from Jex-th Raman line
                 To_act = SRR%ACT_IN(JPOS,Jex) 

                 IF( Phot_to_Wat ) 
     $                Ph_W = WL_GRID( SRR%A2E(To_act) )/WL_GRID(JPOS)

c                ... Number of ESFT coefficients at actual wavelength
                 Nsf = SRR%NESFT(To_act)
c                ... ESFT coefficients at actual wavelength (stored 
c                    in "dom_spherical.f"
                 Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
c                ... S_F is used here as dummy array 
                 FORALL( K=1:Nvs ) S_F(1:Nr,K) = 
     $                COF_LS(1:Nr,Jex)*Phm(1:Nr,K)*SOL_EXP(1:Nr)/4.d0
c                ... Do-loop over ESFT coefficients (if present)
c                    To integrate over l-of-s it will be used here
c                    solar exponent        - at excitation wavelength
c                    extinction coeff and
c                    exponent along l-of-s - at actual wavelength
                 DO Isf = 1,Nsf
                    ALP => AL_L(:,Isf,Jex)
                    EXL => SRR%LS_ACT(IA,IV)%EX(:,To_act,Isf)

c                ... Numerical integration along a line-of-sight
                    DUM = 0.0D0
                    DO I = 1, Nr
                       DUM = DUM +
     $                      S_F(I,:)*EXL(I)*(D_DS(1,I) + D_DS(2,I))
                    END DO
c                   ... + Raman scattering contribution (win) in  
c                       the single scattering approximation
                    Del_ss(IA,IV,IS,IP,:,Jex) =Del_ss(IA,IV,IS,IP,:,Jex)
     $                   + IRR*Ph_W*Wes(Isf)*DUM(:)
                 ENDDO  ! over esft coefficients (Isf)
              ENDDO  ! over Raman lines from this bin (Jex)
              RETURN
           ENDIF  ! Excitation wavelength
        ENDIF  ! Raman scattering in RRS_ADJ and single scattering mode

c       ... Rayleigh contribution to the source function
        Npl = Nlg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)
        DO J = 1,NR
           HH(J,1) = SUM( Wig0_ss(1:Npl,J)*P_MOM(:) )
        ENDDO
        IF( Nm_vs > 1 ) THEN
           HH(:,4) = 0.d0
           P_MOM => LOA%RAY_MOMS(1:Npl,5) 
           DO J = 1,NR
              HH(J,2:3) = Rot_l(J,2:3)*SUM(Wig2_ss(1:Npl,J)*P_MOM(:))
           ENDDO
        ENDIF
        DO K = 1,Nvs
           SSSF(1:NR,K) = SCRL(1:NR)*HH(1:NR,K)
        ENDDO

c       ... If WF for Rayleigh scat. should be calculated
        IF( Ray_to_WF ) S_R(1:NR,1,1) = 
     $       SOL_EXP(1:NR)/4.d0*HH(1:Nr,1)/AL_1(1:Nr)

c       ... If atoms or ions emission should be calculated:
c       ----------------------------------------------------------------
        IF( Emi_FLG ) THEN
           DO Ka = 1,N_AT
              DO J = 1,NR
                 HH(J,1) = SUM( Wig0_ss(1:3,J)*LOA%EPF(Ka,:,1) )
              ENDDO
              IF( Nm_vs > 1 ) THEN
                 HH(:,4) = 0.d0
                 DO J = 1,NR
                    HH(J,2:3) = Rot_l(J,2:3)
     $                   *SUM(Wig2_ss(1:Npl,J)*LOA%EPF(Ka,:,5))
                 ENDDO
              ENDIF
              FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) + 
     $             EMCL(Ka,1:NR)*HH(1:NR,K)
           ENDDO  ! over atoms (Ka)

           IF(INQ_WF('atom', 'WFT')) S_R(1:NR,1,1) = 
     $          SOL_EXP(1:NR)/4.d0*HH(1:Nr,1)/AL_1(1:Nr)
        ENDIF  ! atoms or ions 
c       ----------------------------------------------------------------

c       ... Aerosol contribution to the source function
        IF( DO_AEROSOL ) THEN
           SELECT CASE( APH_TYPE )
           CASE('h-g')
c                   write(*,*) '   H_G-parameter'
c                   write(*,*) G_aer(1:Nr)
              S_H(1:NR) = 1.d0 + G_aer(1:Nr)*G_aer(1:Nr)
     $             - 2.d0*G_aer(1:Nr)*COSCA(1:Nr)
              S_H(1:NR) = (1.d0 - G_aer(1:Nr)*G_aer(1:Nr))
     $             /S_H(1:Nr)/DSQRT(S_H(1:Nr))
              SSSF(1:NR,1) = SSSF(1:NR,1) 
c                    ... SCA_LS does not contain delta-m factor
     $             + S_H(1:Nr)*SCA_LS(1:Nr)
c     $             + S_H(1:Nr)*SCA_LS(1:Nr)/(1.D0 - DM_LS(1:Nr))
           CASE('mie')
              Npl = Leg_Aer ; Phm(:,:) = 0.d0
c              WRITE(*,*) '    Aerosol mie in SS mode NR = ', NR
              DO J = 1,NR
c                 WRITE(*,'(2(1x,I3), *(2x,e12.4))') J, IDX_AER(IOB(J)),
c     $                SUM( OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),1) ),
c     $                SCA_LS(J)
                 Phm(J,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),1) )
              ENDDO
              S_H(1:NR) = Phm(1:Nr,1)
              IF( Nvs > 1 ) THEN
                 DO J = 1,NR
                    Phm(J,2:3) = Rot_l(J,2:3)*SUM( Wig2_ss(1:Npl,J)
     $                   *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),5) )
                 ENDDO                     
              ENDIF
              DO K = 1,Nvs
c                ... SCA_LS does not contain delta-m factor
                 SSSF(1:NR,K) = SSSF(1:NR,K) + Phm(1:Nr,K)*SCA_LS(1:Nr)
c     $                *SCA_LS(1:Nr)/(1.d0 - DM_LS(1:Nr))
              ENDDO
           END SELECT
        ENDIF  ! "DO_AEROSOL"

c       ... Cloud contribution to the source function
        IF( DO_CLOUD ) THEN
           Phm_mc = 0.d0  ;  Npl = Leg_cld
           DO J = 1,Nr
              IF( CI(J) == 0 ) CYCLE
              Ncl = CI(J)
              DO Jmc = 1, MLCS(Ncl)%N_pha
c                 IF( Phm_const ) THEN
                 IF( MLCS(Ncl)%Phm_co ) THEN
                    Ia1 = 1 ; Ia2 = 1 ; DLa= 0.5d0
                 ELSE 
                    N_db = MLCS(Ncl)%Npf(Jmc)%Ndb_ref
                    Rdb => MLCS(Ncl)%Npf(Jmc)%Pdb_ref(:)
                    Aef = EfWI_LS(J,Jmc)
                    CALL LIN_SEARCH_NEW(Aef,DLa,Ia1,Ia2,N_db,Rdb)
                 ENDIF
c                ------------------------------------------------ 
                 IF( MLCS(Ncl)%Npf(Jmc)%Y_Scm ) THEN
c                   ... Use scattering matrix:                    
                    CALL LIN_SEARCH_NEW(COSCA(J), DL, I1, I2, 
     $                   Ydb_ang, SCAT_ANG)
                    Pma => OSM%CLD_SCM(:,Ncl,:,Jmc,1)
c                   ... Interpolate on R_eff
                    Rang(:) = Pma(:,Ia2)*DLa + Pma(:,Ia1)*(1.d0-DLa)
c                   ... Interpolate on scat. angle
                    Phm_mc(Jmc,1) = Rang(I2)*DL + Rang(I1)*(1.d0-DL)
                    IF( Nvs == 1 ) CYCLE
                    Pma => OSM%CLD_SCM(:,Ncl,:,Jmc,5)
c                   ... Interpolate on R_eff
                    Rang(:) = Pma(:,Ia2)*DLa + Pma(:,Ia1)*(1.d0-DLa)
c                   ... Interpolate on scat. angle
                    Phm_mc(Jmc,2:3) = Rot_l(J,2:3)*
     $                   Rang(I2)*DL + Rang(I1)*(1.d0 - DL)
                    CYCLE
                 ENDIF
c                ------------------------------------------------ 
c                ... Use expansion coefficients:
                 Rco(:) = OSM%CLD_ALL(:,Ncl,Ia2,Jmc,1)*DLa 
     $                + OSM%CLD_ALL(:,Ncl,Ia1,Jmc,1)*(1.d0 - DLa)
                 Phm_mc(Jmc,1) = SUM ( Wig0_ss(1:Npl,J)*Rco(:) )
                 IF( Nvs == 1 ) CYCLE
                   Rco(:) = OSM%CLD_ALL(:,Ncl,Ia2,Jmc,5)*DLa 
     $                + OSM%CLD_ALL(:,Ncl,Ia1,Jmc,5)*(1.d0 - DLa)
                   Phm_mc(Jmc,2:3) = Rot_l(J,2:3)
     $                  *SUM( Wig2_ss(1:Npl,J)*Rco(:) )
              ENDDO  ! over water/ice phase (Jmc)
              DO K = 1, Nm_ice
                 SSSF(J,:) = SSSF(J,:) + Phm_mc(K,:)*SCCNO_LS(J,K)
              ENDDO  ! over water/ice phase (K)
           ENDDO  ! over LoS (J)              
c          ------------------------------------------------------
        END IF  ! "DO_CLOUD"
        
        DO K = 1,Nvs
           SSSF(1:Nr,K) = SSSF(1:Nr,K)*SOL_EXP(1:Nr)/4.d0
        ENDDO  
        
        IF(INQ_WF('aer', 'WFT')) THEN
c           WRITE(*,*) '   NR  "aer", "WFT"', NR
           S_R(1:NR,1,1) = SOL_EXP(1:NR)/4.d0*S_H(1:NR)/AL_1(1:Nr)
c           write(*,*) '  S_R (in ss)  ', S_R(:,1,1)
           IF( Dwr_mi ) THEN
              Npl = Leg_Aer  
              DO Idr = 1,N_dwr
                 FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%DER_ALL(1:Npl,IDX_AER(IOB(J)),1,Idr) )
                 S_R(1:Nr,1,1+Idr ) = Phm(1:Nr,1)*SOL_EXP(1:NR)/4.d0
c                     ... SCA_LS does not contain delta-m factor
     $                *SCA_LS(1:Nr)/AL_1(1:Nr)
c     $                *SCA_LS(1:Nr)/(1.d0 - DM_LS(1:Nr))/AL_1(1:Nr)
              ENDDO
           ENDIF
        ENDIF

      END SUBROUTINE SINGLE_IN_MS_MODE
c *************************************************************************
c
      SUBROUTINE SUM_RRS_MS

      INTEGER  Isf
      REAL (KND) Ph_W
      REAL (KND), POINTER :: ALP(:), RRC(:), EXL(:)
c     ------------------------------------------------------------------
c     ... In the vector case array S_RR is multiplied by 0.5 ("phm_along_ls.f")
c         in the scalar case by 1 ("phf_along_ls.f"). 
c         Therefore, after multiplication by Mult thr coefficients of Fourier 
c         series have the following multiplier for harmonic Mh:
c                    Mh = 1     Mh > 1
c         scalar:      1          2
c         vector:      0.5        1
c
      IF( .NOT.SRR%Exi_WL(JPOS) ) THEN
c        ... Numerical integration along a line-of-sight
         DUM = 0.0D0
         DO I = 1, Nr
            DUM = DUM +
     $           S_RR(I,:)*EX_L(I)*(D_DS(1,I) + D_DS(2,I))*COF_LS(I,1)
         END DO
c        ... - Raman scattering contribution (loss) in
c              multiple scattering source function
         Del_ss(IA,IV,IS,IP,:,1) = Del_ss(IA,IV,IS,IP,:,1) + IRR*DUM(:)
      ELSE
c        ... Over all Raman lines within corresponding excitation bin 
         Ph_W = 1.d0
         DO Jex = 1,SRR%NUM_IN(JPOS)
c           ... The actual wavelength number which becomes
c               contribution from Jex-th Raman line
            To_act = SRR%ACT_IN(JPOS,Jex) 
            IF( Phot_to_Wat ) 
     $           Ph_W = WL_GRID( SRR%A2E(To_act) )/WL_GRID(JPOS)
c           ... Number of ESFT coefficients at actual wavelength
            Nsf = SRR%NESFT(To_act)
c           ... ESFT coefficients at actual wavelength (stored 
c               in "dom_spherical.f"
            Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
            RRC => COF_LS(:,Jex)
c           ... Do-loop over ESFT coefficients (if present)
c               To integrate over l-of-s it will be used here
c               solar exponent        - at excitation wavelength
c               extinction coeff and
c               exponent along l-of-s - at actual wavelength
            DO Isf = 1,Nsf
               ALP => AL_L(:,Isf,Jex)
               EXL => SRR%LS_ACT(IA,IV)%EX(:,To_act,Isf)
c              ... Numerical integration along a line-of-sight
               DUM = 0.0D0
               DO I = 1, Nr
                  DUM = DUM +
     $                 S_RR(I,:)*EXL(I)*(D_DS(1,I) + D_DS(2,I))*RRC(I)
               END DO
c              ... + Raman scattering contribution (win) in 
c                    multiple scattering source function
               Del_ss(IA,IV,IS,IP,:,Jex) = Del_ss(IA,IV,IS,IP,:,Jex) +
     $              IRR*Ph_W*Wes(Isf)*DUM(:)
             ENDDO  ! over esft coefficients (Isf)
         ENDDO  ! over Raman lines from this bin (Jex)
      ENDIF  ! Excitation wavelength

      END SUBROUTINE SUM_RRS_MS

c *************************************************************************
c     This subroutine calculates the contribution in radiation along 
c     line-of-sight from the multiple scattering source function for 
c     each Fourier harmonic (scalar case). Therefore, it should be  
c                            ===========
c     used here NLEGEN phase matrix expansion coefficients only. 
c *************************************************************************
      SUBROUTINE PHF_ALONG_LS   ! For scalar case

      CALL WIGNER_PLG( -XM_1(IMS1:IMS2), IMS2 - IMS1 + 1, Mha-1, NLg,
     $     Pol_ls(:,IMS1:IMS2) )

c--------------------------------------------------------------------------
c     ... Raman scattering should be accounted for within the 
c           multiple scattering mode
      IF( RRS_ADJ ) THEN
         DO L = IMS1, IMS2 
            TTT1 = 0.d0
            DO K = 1,NN_MS
               P_F = SUM( SRR%RAM_MOMS(Mha:Nlg_Ray,1)
     $              *Pol_ls(Mha:Nlg_Ray,L)*Pol_ga(Mha:Nlg_Ray,K) ) 
               TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
            END DO                         
            S_RR(L,1) = TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )
         END DO  
         IF( SRR%Exi_WL(JPOS) ) RETURN
      ENDIF  ! Raman scattering in "RRS_ADJ" mode
c--------------------------------------------------------------------------

      SUM_MOM = 0.d0
      IF( Mha <= Nlg_ray ) THEN
         DO L = IMS1, IMS2
            SUM_MOM(Mha:Nlg_ray,L) = 
     $           LOA%RAY_MOMS(Mha:Nlg_Ray,1)*SCRL(L)
         ENDDO
c          ... Contribution of atoms/ions emission if any 
c          --------------------------------------------------------
         IF( Emi_FLG ) THEN
            DO Ka = 1, N_AT
               DO L = IMS1, IMS2
                  SUM_MOM(Mha:Nlg_ray,L) = SUM_MOM(Mha:Nlg_ray,L)
     $                 + LOA%EPF(Ka,Mha:Nlg_Ray,1)*EMCL(Ka,L)
               ENDDO  ! over line of sight (L)
            ENDDO  ! over absorber (Ka)
         ENDIF
c          --------------------------------------------------------
      ENDIF  ! "Mha <= Nlg_ray" Rayleigh and atoms/ions

      IF( DO_AEROSOL ) THEN
         DO L = IMS1, IMS2
            SUM_MOM(Mha:Nlg,L) = SUM_MOM(Mha:Nlg,L)
     $           + LOA%AER_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1)*SCA_LS(L)
     $           * (1.d0 - DM_LS(L))  ! ??
c            WRITE(*,'(2x,I3, *(2x, E12.4))') L, 
c     $           SUM( LOA%AER_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1) )
c     $            *SCA_LS(L) * (1.d0 - DM_LS(L))
         ENDDO
      ENDIF  ! "DO_AEROSOL"  MS mode

      IF( DO_CLOUD ) THEN
         FORALL( L=IMS1:IMS2, CI(L)>0 ) SUM_MOM(Mha:Nlg,L) = 
     $        SUM_MOM(Mha:Nlg,L)
     $        + LOA%CLD_MOMS(Mha:Nlg,CI(L),1,1,1)*SCC_LS(L,1)
         IF( Nm_ice==2 ) FORALL( L=IMS1:IMS2, CI(L)>0 )
     $        SUM_MOM(Mha:Nlg,L) = SUM_MOM(Mha:Nlg,L)
     $        + LOA%CLD_MOMS(Mha:Nlg,CI(L),1,2,1)*SCC_LS(L,2)
      ENDIF  ! "DO_CLOUD" MS mode
        
      DO L = IMS1, IMS2 ; TTT1 = 0.d0
         DO K = 1,NN_MS
            P_F = SUM( SUM_MOM(Mha:Nlg,L)
     $           *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K) ) 
            TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
         END DO                         
         S_F(L,1) = TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )
      END DO   

      IF( Ray_to_WF ) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%RAY_MOMS(Mha:3,1)*Pol_ls(Mha:3,L)
     $                 *Pol_ga(Mha:3,K) )   
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
c                    Single scatt test
c                ... Array S_R is set to zero for Mh > 0
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
            END DO   
         ENDIF   ! multiple scattering for Rayleigh WFs
      ENDIF  ! Rayleigh WF

      IF(INQ_WF('atom', 'WFT')) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
            Ka = 1
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%EPF(Ka,Mha:3,1)*Pol_ls(Mha:3,L)
     $                 *Pol_ga(Mha:3,K) )   
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
c                ... Array S_R is set to zero for Mh > 0
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
            END DO   
         ENDIF   ! multiple scattering for atoms WFs
      ENDIF  ! Atoms WF

      IF( INQ_WF('aer', 'WFT') ) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
c            write(*,*) '   Number of moments Nlg ', Nlg
c           write(*,*) '  S_R (do ms) --- ', S_R(1:10,1,1)
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%AER_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1)
     $                 *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K)) 
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
cc                ... Array S_R is set to zero for Mh > 0
c                    Single scatt test (don't use these summation)
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L) 
            END DO   
c            write(*,*) '  S_R (in ms --- )  ', S_R(1:10,1,1)
c           ------------------------------------------------------------
            IF( Dwr_mi ) THEN
               DO Idr = 1,N_dwr
                  DO L = IMS1, IMS2 ; TTT1 = 0.d0
                     DO K = 1,NN_MS
                        P_F = SUM( 
     $                       LOA%ADR_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1,Idr)
     $                       *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K) ) 
                        TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
                     END DO                         
c                           Single scatt test         
c                      ... Array S_R is set to zero for Mh > 0
                     S_R(L,1,1+Idr) = S_R(L,1,1+Idr)
     $                    + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
     $                    * SCA_LS(L)*(1.d0 - DM_LS(L))
                  ENDDO  ! over line-of-dight (L)
               ENDDO  ! over PSD function derivatives (Idr)
            ENDIF  ! Dwr_mi
c           ------------------------------------------------------------
         ENDIF  ! multiple scattering for WFs
      ENDIF  ! Aerosol WF

      END SUBROUTINE PHF_ALONG_LS

c *************************************************************************
      SUBROUTINE ROT_MATR
      REAL (KND)  Azm, Zn, T2, Xmu, Mu0
      Rot_l = 0.d0
c     ... Within this subroutine Mu (direction of radiation) 
c         and Mu0 (direction of solar radiation) have to be 
c         given with respect to the tau-axis !!!
      DO J = 1,NR
         Xmu = -XM_1(J)  ;  Mu0 = XM_S1(J) ; Azm = PHI_LOS(J)
         IF( REAL(Azm) == REAL(Pie) .OR. REAL(AZM) == 0.0) THEN 
            Rot_l(J,2) = 1.d0
            Rot_l(J,3) = 0.d0
            CYCLE
         ENDIF
         IF( Azm > Pie ) THEN 
            Zn = 1.0d0
         ELSE
            Zn = -1.0d0
         ENDIF
         IF( DABS(Xmu) == 1.0d0 ) THEN
            T2 = DSIGN(1.d0,Xmu)*DCOS(Azm)
         ELSE
            T2 = -( Mu0 - Xmu*COSCA(J) )
     $           /DSQRT((1.D0 - Xmu*Xmu)*(1.D0-COSCA(J)**2))
         ENDIF
         Rot_l(J,2) = 2.d0*T2*T2 - 1.d0 
         Rot_l(J,3) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
      ENDDO
      END SUBROUTINE ROT_MATR

c     ******************************************************************
      SUBROUTINE SET_COEFF
      INTEGER  J, Ji(2), Ki(2), I, K
      REAL (KND) Di(2)
        Ji(:) = 0
        IF( GeoD%LVG(IA,IV)%TH_FLG ) THEN
           Ji(1) = NTH ; Ki(1) = SID(NTH) ; K2 = Ki(1)
           Di(1) = ( ZLS(NTH) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF
        IF( GeoD%LVG(IA,IV)%FP_FLG ) THEN
           Ji(2) = 1 ; Ki(2) = SID(1) ; K2 = Ki(2)
           Di(2) = ( ZLS(1) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF
        AL_1(1:NR) = ALO( SID(1:NR) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I)
           AL_1(Ji(I)) = ALO(K2+1) + (ALO(K2) - ALO(K2+1))*Di(I)
        ENDDO

        IF( RRS_ADJ ) THEN 
           DO Jex = 1,NRL
              COF_LS(1:NR,Jex) = COF_FI( SID(1:NR),Jex )
              DO I = 1,2
                 IF( Ji(I)==0 ) CYCLE
                 K2 = Ki(I)
                 COF_LS(Ji(I),Jex) = COF_FI(K2+1,Jex) 
     $                + (COF_FI(K2,Jex) - COF_FI(K2+1,Jex))*Di(I)
              ENDDO
              IF( SRR%ACT_WL(JPOS) ) CYCLE

              DO K = 1, SRR%M_ESF
                 AL_L(1:Nr,K,Jex) = AL_F( SID(1:Nr),K,Jex )
                 DO I = 1,2
                    IF( Ji(I)==0 ) CYCLE
                    K2 = Ki(I)
                    AL_L(Ji(I),K,Jex) = AL_F(K2+1,K,Jex) 
     $                   + (AL_F(K2,K,Jex) - AL_F(K2+1,K,Jex))*Di(I)
                 ENDDO                 
              ENDDO  ! over esft coefficients (K)
           ENDDO  ! over Raman lines (Jex)
        ENDIF

c       ... If wavelength is marked as excitation contribution
c           of all "standard" component will be ignored
        IF( SRR%Exi_WL(JPOS) ) RETURN 

        SCRL(1:Nr) = SCRO( SID(1:Nr) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I) 
           SCRL(Ji(I)) = SCRO(K2+1) + (SCRO(K2)-SCRO(K2+1))*Di(I)
        ENDDO

        IF( Emi_FLG ) THEN
           DO J = 1,N_AT 
              EMCL(J,1:Nr) = EMCO( J,SID(1:Nr) )
              DO I = 1,2 
                 IF( Ji(I)==0 ) CYCLE
                 K2 = Ki(I) 
                 EMCL(J,Ji(I)) = EMCO(J,K2+1) 
     $                + (EMCO(J,K2) - EMCO(J,K2+1))*Di(I)
              ENDDO
           ENDDO
        ENDIF

        IF( DO_AEROSOL ) THEN
           G_aer (1:NR) = ASCO( SID(1:NR) ) 
           SCA_LS(1:NR) = SCAO( SID(1:NR) )
           DM_LS (1:NR) = DM_AO(SID(1:NR) )
           DO I = 1,2 
              IF( Ji(I)==0 ) CYCLE
              K2 = Ki(I) ; D1 = Di(I)
              G_aer (Ji(I)) = ASCO(K2+1) + (ASCO(K2) - ASCO(K2+1))*D1 
              SCA_LS(Ji(I)) = SCAO(K2+1) + (SCAO(K2) - SCAO(K2+1))*D1 
              DM_LS (Ji(I)) = DM_AO(K2+1)+ (DM_AO(K2)-DM_AO(K2+1))*D1
           ENDDO
        ENDIF

        IF( DO_CLOUD ) THEN
           CI    (1:NR)   =  INT( CLA(IOB(1:NR),2) )
           DO J = 1, Nm_ice
              SCC_LS  (1:NR,J) = SCCO   (SID(1:NR),J)
              SCCNO_LS(1:NR,J) = SCC_NSO(SID(1:NR),J)
           ENDDO
        ENDIF

        IF( TE%ADD_PLANCK ) THEN 
c          ... PLA_FINE contains product of Plank function and absorption coeff.
           SF_TE(1:NR) = TE%PLA_FINE( SID(1:NR) ) 
           DO I = 1,2  ;  IF( Ji(I)==0 ) CYCLE   ;  K2 = Ki(I)
              SF_TE(Ji(I)) = TE%PLA_FINE(K2+1) 
     $             + (TE%PLA_FINE(K2) - TE%PLA_FINE(K2+1))*Di(I)
           ENDDO
           IF( DO_WF ) THEN
              S_TE(1:NR) = TE%B_FINE( SID(1:NR) ) 
              DO I = 1,2  ;  IF( Ji(I)==0 ) CYCLE  ;  K2 = Ki(I)
                 S_TE(Ji(I)) = TE%B_FINE(K2+1) 
     $                + (TE%B_FINE(K2) - TE%B_FINE(K2+1))*Di(I)
              ENDDO
           ENDIF
        ENDIF      
      END SUBROUTINE SET_COEFF

* *************************************************************************
      SUBROUTINE INT_ON_TH_LEV(Poi, INI, Rez)
      INTEGER     Poi, INI
      REAL (KND)  ZV, Rez(NSOL,NN_MS*Nvs,4)

        ZV = ZLS(Poi)
        CALL BIAKIMA_Y( Z_INV(NLAYERS-INI+1), 
     $       SZA_EXT, ZV, X_RAD(NLAYERS-INI+1,:,:,:), Rez)

      END SUBROUTINE INT_ON_TH_LEV
C     *****************************************************************
      END
C *************************************************************************
c     Set output array
      SUBROUTINE SET_OUT_RAD (ARR_INP, ARR_OUT)
      USE SPHER_PARS,  ONLY: N_LOS_ANG, NFA => N_FOV_ANG, FOV_WEIGHTS,
     $     N_LOS_OUT
      USE ST_VARS,     ONLY: GeoD => GeoD_Out
      USE INTERN_DIMS, ONLY: Nvs
      USE TYPES,       ONLY: KND
      REAL (KND) ARR_INP(:,:,:,:,:), ARR_OUT(N_LOS_OUT,Nvs) 
      INTEGER IS, IV, IP, KK, N_OUT, IA, II
      REAL (KND) ARR_LOC(NFA)

      IA = 1 ;  N_OUT = 0 
      DO IS = 1, GeoD%NSol
         DO IV = 1, N_LOS_ANG
            DO IP = 1, GeoD%NPhi
               N_OUT = N_OUT + 1
               DO II = 1, Nvs
                  FORALL(KK = 1:NFA) ARR_LOC(KK) = 
     $                 ARR_INP(IA,(IV - 1)*NFA + KK,IS,IP,II)
c                  DO KK = 1, NFA 
c                     ARR_LOC(KK) = 
c     $                    ARR_INP(IA,(IV - 1)*NFA + KK,IS,IP,II)
c                     WRITE(*,'(2x,I3,2(2x,E12.4))') 
c     $                    KK, ARR_LOC(KK), FOV_WEIGHTS(KK)
c                  ENDDO
                  ARR_OUT(N_OUT,II) = SUM(FOV_WEIGHTS(:)*ARR_LOC(:))
               END DO
            END DO
         END DO
      END DO

      END SUBROUTINE SET_OUT_RAD
c *************************************************************************

