C***********************************************************************
C
C       SCIATRAN - Version 4.5 - Last modification: 12 Dec 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE SFI_SPHER_RF(JPOS, JJESFT, Mha, POS, RAD, AL, E_W, OSM, 
     $     LOA, LSE, PWF, SRR, IND_LS, TE, ALBEDO,
     $     FLU_int, Ref_surf, FLUX, ADI, NN_MS)
c     ... Called by "st_dom_tau.f"

      USE AEROSOLS,    ONLY: IDX_AER, N_dwr, Dwr_mi
      USE BRDF,        ONLY: Npg
      USE CLD_INP,     ONLY: Ydb_ang, MLCS
      USE CLOUDS,      ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR, SCAT_ANG
      USE EMISSION,    ONLY: N_AT
      USE GRIDS,       ONLY: NLAYERS, ZZZ, Z_FINE, NSOL, SZA_EXT, 
     $                       SOL_GRD, ALT_EXT, Z_INV, NL_F => NL_FINE, 
     $                       X_INV, INTERP_IDX
      USE GT_POLYN,    ONLY: NLg => NLEGEN, XM_G => XM, AG1
      USE INTERN_DIMS, ONLY: Nm_vs, Nvs, Nlg_ray, Leg_aer => NLG_ALL_A, 
     $                       Leg_cld => NLG_ALL_C, Nm_ice
      USE IRRAD,       ONLY: NO_SUN
      USE LOG_PARS,    ONLY: LOG_LEVEL, LUNIT, N_DSO, N_ESO
      USE LOS_Angles,  ONLY: DirSun
      USE RTM_OUTPUT,  ONLY: N_OP, DEL_RRS
      USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT, X_SHAD,
     $                       NFA => N_FOV_ANG, ONE_MS_SZ, FOV_WEIGHTS
      USE ST_INPUT,    ONLY: APH_TYPE
      USE ST_VARS,     ONLY: Pie, DO_AEROSOL, Vec_Sca, Kvs,
     $                       WL_GRID, N_RHS, NSTRMS, BRDF_FLAG, Ns_wf,
     $                       Nc_der, VST, Emi_FLG, WF_MS_FLAG,
     $                       GeoD => GeoD_Out, PreCalcAngs, Ray_to_WF
      USE TOOLS,       ONLY: INQ_WF, Get_Sza_Idx, LOS_MS_IDX
      USE TYPES,       ONLY: KND, PWF_TYPE, LOA_type, PCC_TYPE, TE_TYPE,
     $                       OSM_TYPE, RRS_TYPE, AuxDataInterp, LSE_type
      USE WF_NUM,      ONLY: NUM_WF, NO_SOL, NO_LS, AL_1_BASE, AL_1_PERT
c     ------------------------------------------------------------------

c     ---------------------------
      INTEGER NN_MS
      INTEGER JPOS, JJESFT, Mha, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
      REAL (KND)  ALBEDO, FLU_int, E_W, POS, 
     $     FLUX(Kvs*NSOL), AL(NLAYERS), Ref_surf(N_RHS,Nvs*(NSTRMS+Npg))
      REAL (KND), ALLOCATABLE :: RAD(:,:,:)
      TYPE (AuxDataInterp), TARGET :: ADI
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE (PWF_type) :: PWF    ! Weighting functions
      TYPE (RRS_type), TARGET :: SRR    ! RRS related parameters
      TYPE(LSE_type), TARGET  :: LSE(GeoD%NAlts, GeoD%NVA)
      TYPE (TE_type)  :: TE     ! Thermal emission 
      
      INTENT(IN) :: JPOS, JJESFT, Mha, IND_LS, ALBEDO, E_W, POS,
     $     FLUX, AL, Ref_surf, 
     $     LOA, OSM, PWF, ADI
c     $     ,RAD
      INTENT(INOUT) :: SRR
      ! Local variables
      CHARACTER (LEN=*),  PARAMETER:: R_NAME = 'SFI_SPHER_RF'
      INTEGER  N_OUT, Idl, Ids, Idn, II, N_LOS, J, I, K, L, K1, 
     $     K2, Npl, Npo, Js, Jss, NSTR, NR, Jv, Jf, Jw, Jw1, Ka,
     $     ID_CLD, NNv, Idr, Index, IV, NTH, IS, IP, IA, IMS1, IMS2
      INTEGER  LL(GeoD%MaxLS)
      INTEGER, POINTER ::  SID(:), IOB(:)
      INTEGER, ALLOCATABLE :: CI(:)
      LOGICAL  DO_WF
      REAL (KND) Mu_Ls, TTT1, P_F, SD, Z_OUT, D1, WFOV
      REAL (KND) Pol_ga(NLg,NN_MS), XM_2(NN_MS), AG1_2(NN_MS), 
     $     FL(NN_MS*Nvs,GeoD%MaxLS), S_R(GeoD%MaxLS,Nvs,1+N_dwr)
      REAL(KND)  Tls(NSOL,Nvs), X_ref(NSTRMS+1)
      REAL (KND), DIMENSION (Nvs) :: LBC, BC_Dir, BC_Dif, BC_wf, Fg_INT
      REAL (KND), DIMENSION (GeoD%MaxLS) :: AL_1, SOL_EXP, S_H, COSCA,
     $     SCRL
      REAL (KND), DIMENSION (GeoD%MaxLS,Nvs) :: SSSF, S_F, S_F_Tes, Phm,
     $     Rot_l
      REAL (KND), DIMENSION (NLg,GeoD%MaxLS) :: Pol_ls, SUM_MOM
      REAL (KND), DIMENSION (NSOL,NN_MS*Nvs,4) :: Y_TH, Y_FP
      REAL (KND) S_R_Tes(GeoD%MaxLS,Nvs,1+N_dwr), R_SNG(NLAYERS,NSOL), 
     $     X_RAD(NLAYERS,NSOL,NN_MS*Nvs,4), Y_ALL(NSOL,NN_MS*Nvs,NL_F,4)
      REAL (KND), ALLOCATABLE :: Therm(:), SCC_LS(:,:), SCCNO_LS(:,:),
     $     WF_dir(:), EMCL(:,:), EMCO(:,:), S_RR(:,:), DM_LS(:),
     $     G_aer(:), SCA_LS(:), DS_SUN(:,:), Phm_mc(:,:), EX_1(:)
      REAL (KND), POINTER :: ALO(:), SCRO(:), SCAO(:), ASCO(:), 
     $     DM_AO(:), P_MOM(:), XM_1(:), D_DS(:,:), EX_L(:),
     $     PHI_LOS(:), XM_S1(:), XM_SG(:), SCCO(:,:), SCC_NSO(:,:),
     $     ZLS(:)
      TYPE (PCC_type) :: PCC    ! Photochemistry
c     ==================================================================
      INTERFACE
         SUBROUTINE PHM_ALONG_LS (JPOS, Mh, IMS1, IMS2, XM_1, RAY_MOMS,
     $        AER_MOMS, CLD_MOMS, SCRL, EMCL, H36, SCAL, DM_LS, SCCL,
     $         Phi, FL, IOB, S_F, SRR, S_RR)
            USE CLOUD_INPUT, ONLY: Nphr
            USE INTERN_DIMS, ONLY: Nlg_Ray, Nm_vs, NL_AER, NLG_AER,
     $        NL_CLD, NLG_CLD, Nm_ice
            USE TYPES,       ONLY: KND, RRS_TYPE
            INTEGER      JPOS, Mh, IMS1, IMS2, IOB(:)
            REAL (KND), DIMENSION (:) :: XM_1, SCRL, Phi
            REAL (KND)  RAY_MOMS(Nlg_Ray,Nm_vs),
     $           AER_MOMS(Nlg_Aer,NL_Aer,Nm_vs),
     $           CLD_MOMS(Nlg_Cld,NL_Cld,Nphr,Nm_ice,Nm_vs)
            REAL (KND), ALLOCATABLE :: EMCL(:,:), H36(:,:,:), S_RR(:,:),
     $           SCAL(:), DM_LS(:), SCCL(:,:) 
            REAL (KND)  FL(:,:), S_F(:,:)
            TYPE (RRS_type) :: SRR 
       END SUBROUTINE PHM_ALONG_LS

       SUBROUTINE BIAKIMA_Y (X_AX, Y_LOC, X_NEW, X_RAD, Y_ALL)
          USE GRIDS,       ONLY: NSOL
          USE TYPES,       ONLY: KND
          REAL (KND)  X_RAD(:,:,:), Y_ALL(:,:,:) 
          REAL (KND)  Y_LOC(NSOL + 4)
          REAL (KND)  X_NEW, X_AX
       END SUBROUTINE BIAKIMA_Y

       SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, 
     $     ALBEDO, FLU_int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
          USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, 
     $                    Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
          USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
          USE INTERN_DIMS, ONLY: Nvs
          USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
          USE TYPE_VARS, ONLY: KND
          USE WATER,  ONLY: SS_PDF
          INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
          REAL (KND), INTENT(IN) :: ALBEDO, FLU_int, POS, SOL_EXP,
     $          PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
          REAL (KND) BC_S1(Nvs), LBC(Nvs)
          REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
          OPTIONAL :: REF_SUR
       END SUBROUTINE REFL_DIR_SPH
      END INTERFACE
c     ==================================================================
      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1
      BC_Dir(:) = 0.d0 ; LBC(:) = 0.d0
      DO_WF = .TRUE.
      NNv = Nvs*NSTRMS
      X_ref(1:NSTRMS) = XM_G(:) ; X_ref(NSTRMS+1) = 1.d0

      IF( Emi_FLG ) THEN
         ALLOCATE( EMCL(N_AT,GeoD%MaxLS), EMCO(N_AT,NL_F) )
         DO Ka = 1, N_AT
            CALL LIN_FINE_1(NLAYERS, ZZZ, NL_F, Z_FINE, INTERP_IDX, 
     $           LOA%EMC(Ka,:), EMCO(Ka,:), .FALSE.)
         ENDDO
      ENDIF
      
      ALO => ADI%AL0_OUT ;  SCRO => ADI%SCR_OUT

c     ... Aerosol on the fine grid:
c     ------------------------------------------------------------------
c     delta-M scaling factor
      DM_AO => ADI%DM_AO  
c         scattering coeff. without delta-m scaling
      SCAO => ADI%SCA_OUT
c         asymmetry parameter
      ASCO => ADI%ASC_OUT
c     ------------------------------------------------------------------
      
      IF( DO_CLOUD ) THEN
         SCC_NSO => ADI%SCC_NSO ! without delta-m scaling
         SCCO => ADI%SCCO       ! with delta-m scaling
      ENDIF

      IF( TE%ADD_PLANCK ) ALLOCATE( Therm(GeoD%MaxLS) )

c     ... In the array RAD(:,:,1:2*NNv) elements 
c             1:NNv   - contain downward radiation
c         NNv+1:2*NNv - contain upward radiation
c     ... Over all zenith angles (up and down) and all Stokes components
c          NN_MS = 2*NSTRMS was set in "rtm_init_first.f"
      DO Idn = 1, NN_MS*Nvs
         R_SNG(:,:) = SNGL( RAD(:,:,Idn) )
         CALL BIAKIMA_X(NLAYERS, ALT_EXT, NSOL, X_RAD(:,:,Idn,:), R_SNG)
      ENDDO   ! over Gaussian angles and all Stokes components (Idn)
      DO J = 1, NL_F
         CALL BIAKIMA_Y(Z_INV(INTERP_IDX(J)), SZA_EXT, Z_FINE(J),
     $        X_RAD(INTERP_IDX(J),:,:,:), Y_ALL(:,:,J,:))
      END DO

      Z_OUT = GeoD%Alt(1) 
      ALLOCATE( DS_SUN(GeoD%MaxLS,NLAYERS), WF_dir(NLAYERS) ) ! A
      Npo = MAX( Nlg_Ray,Leg_aer,Leg_cld )

      IF (LOG_LEVEL(1) >= N_ESO)  WRITE(LUNIT, '(6x,A,2X,I2)')   
     $     '     ********* Mharm ******* ', Mha
                
      NSTR = NN_MS/2 
      XM_2(1:NSTR)       =  XM_G(:) !Downward
      XM_2(1+NSTR:NN_MS) = -XM_G(:)
      AG1_2(1:NSTR) = AG1(:) ; AG1_2(1+NSTR:NN_MS) = AG1(:)      
      CALL WIGNER_PLG(XM_2, NN_MS, Mha-1, NLg, Pol_ga)

      DO IV = 1, GeoD%NVA       ! Number of viewing angles including FOV

         Jf = IV  - (CEILING( FLOAT(IV)/FLOAT(NFA) ) - 1)*NFA
         WFOV = FOV_WEIGHTS(Jf)

         Mu_Ls = GeoD%LVG(IA,IV)%C_VA(1)
         NTH   = GeoD%LVG(IA,IV)%NTH
         NR    = GeoD%LVG(IA,IV)%NLS
         ZLS  => GeoD%LVG(IA,IV)%ZLS
         SID  => GeoD%LVG(IA,IV)%ISEL
         IOB  => GeoD%LVG(IA,IV)%IOB

c        ... Output altitude (Z_OUT) at TOA and instrument directed upward
         IF( Z_OUT == ZZZ(1).AND.Mu_Ls <= 0.0D0 )  CYCLE

         IF( DO_AEROSOL ) 
     $        ALLOCATE(G_aer(NR), SCA_LS(NR), DM_LS(NR) )
c         IF( TE%ADD_PLANCK ) ALLOCATE(Therm(NR))
         IF( DO_CLOUD ) THEN
            ALLOCATE(CI(NR), SCC_LS(NR,Nm_ice), SCCNO_LS(NR,Nm_ice),
     $           Phm_mc(Nm_ice,Nvs) )
            CI(:) =  INT( CLA(IOB(:),2) )
            DO J = 1, Nm_ice
               SCC_LS  (:,J) = SCCO   (SID(:),J)
               SCCNO_LS(:,J) = SCC_NSO(SID(:),J)
            ENDDO
         ENDIF
c     ... XM_1 is the cosine of the L-of-S direction with respect 
c         to the downward direction, i.e., XM_1 is the cosine 
c         between the direction of radiation propagation and z-axis.
         XM_1 => GeoD%LVG(IA,IV)%C_VA
         EX_L => LSE(IA,IV)%EX(1:NR)
         ALLOCATE( EX_1(0:NR) )
         EX_1(:) = LSE(IA,IV)%EX1(0:NR)
         D_DS => GeoD%LVG(IA,IV)%DS

c     ... Setting of scattering and extinction coefficients
c         along l-of-s including interpolation on the tangent height
         CALL SET_COEFF

         IF(GeoD%LVG(IA,IV)%TH_FLG)
     $        CALL INT_ON_TH_LEV(NTH, IOB(NTH), Y_TH)
         IF(GeoD%LVG(IA,IV)%FP_FLG)
     $        CALL INT_ON_TH_LEV(1, IOB(1), Y_FP)
         
         DO IS = 1, GeoD%NSol   ! Number of solar angles
            DO IP = 1, GeoD%NPhi ! Number of azimuth angles
               CALL LOS_MS_IDX(GeoD%LSG(IA,IV,IS,IP)%CG_SZA, IMS1, IMS2)
c                  WRITE(*,*) '   IMS1 and IMS2  ', IMS1, IMS2               
c              ... Azimuthal angle [rad] along line-of-sight
               PHI_LOS => GeoD%LSG(IA,IV,IS,IP)%SAA(:)
               
c              ... Attention !!!
c              XM_S1 is the cosine of the solar angle between the direction
c              of the solar beam and downward direction at a point of L-of-S
               XM_S1 => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  
C              Attention, to pick the appropriate values from the multiple 
c              scattering contribution we need
C              geometrical SZA rather than refractive one.
               XM_SG => GeoD%LSG(IA,IV,IS,IP)%CG_SZA(:)
c               XM_SG => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  
               
c              ... Single scattering source function along LOS
               SSSF(1:Nr,:) = 0.d0 ; S_R(1:NR,:,:) = 0.d0
               SOL_EXP = 0.d0
               IF( Mha==1 ) THEN
                  IF( .NOT.NO_SUN ) CALL SINGLE_IN_MS_MODE
                  IF( TE%ADD_PLANCK ) SSSF(1:NR,1) = 
     $                 SSSF(1:Nr,1) + Therm(1:Nr)
               ENDIF

               CALL GET_SZA_IDX(XM_SG(IMS1:IMS2), LL(IMS1:IMS2))

               FL = 0.d0
               FORALL(L = IMS1:IMS2) FL(:,L) = Y_ALL(LL(L),:,SID(L),1)
               IF (.NOT.ONE_MS_SZ) THEN
                  DO L = IMS1, IMS2
                     K = LL(L)
                     SD = XM_SG(L) - X_INV(K)
                     FL(:,L) =  FL(:,L) + 
     $                    SD*( Y_ALL(K,:,SID(L),2) +
     $                        SD*(Y_ALL(K,:,SID(L),3) +
     $                            Y_ALL(K,:,SID(L),4)*SD) )
                  END DO
               END IF

               IF( GeoD%LVG(IA,IV)%TH_FLG.AND.
     $              (NTH.GE.IMS1.AND.NTH.LE.IMS2)) THEN
                  K = LL(NTH)
                  FL(:,NTH) = Y_TH(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(NTH) - X_INV(K)
                     FL(:,NTH) =  FL(:,NTH) + SD*(Y_TH(K,:,2) +
     $                    SD*(Y_TH(K,:,3) + Y_TH(K,:,4)*SD))
                  END IF
               END IF
         
               IF(GeoD%LVG(IA,IV)%FP_FLG) THEN
                  K = LL(1)
                  FL(:,1) = Y_FP(K,:,1)
                  IF (.NOT.ONE_MS_SZ) THEN
                     SD = XM_SG(1) - X_INV(K)
                     FL(:,1) =  FL(:,1) + SD*(Y_FP(K,:,2) +
     $                    SD*(Y_FP(K,:,3) + Y_FP(K,:,4)*SD))
                  END IF
               END IF

               SELECT CASE( Vec_Sca )
               CASE( 'vect_rte' )
c              ... Calculation of: S_F(:,:), S_RR(:,:) in vector case
                  CALL PHM_ALONG_LS(JPOS, Mha, IMS1, IMS2, -XM_1,  
     $                 LOA%RAY_MOMS, LOA%AER_MOMS, LOA%CLD_MOMS, 
     $                 SCRL, EMCL, LOA%EPF, SCA_LS, DM_LS, SCC_LS, 
     $                 PHI_LOS, FL, IOB, S_F, SRR, S_RR)
               CASE( 'scal_rte' )
c              ... Calculation of: S_F(:,1), S_R(:,1,:) in scalar case
                  CALL PHF_ALONG_LS
               END SELECT

               S_F(1:IMS1-1,:) = 0.d0  ; S_R(1:IMS1-1,1,:) = 0.d0
               S_F(IMS2+1:Nr,:) = 0.d0 ; S_R(IMS2+1:Nr,1,:) = 0.d0
               
c              ... Total radiation reflected by surface (if any)
c              =================================================       
               LBC(:) = 0.d0
               Surface_Refl: BLOCK
                  BC_Dir = 0.d0 ; BC_Dif = 0.d0 ; BC_wf = 0.d0
                  IF(IMS2 <= 0)                   EXIT Surface_Refl
                  IF( ZLS(IMS2) /= ZZZ(NLAYERS) ) EXIT Surface_Refl
c                 ... Direct solar radiation reflected by surface
                  IF( Mha==1 ) THEN 
c                    ... The same as in "st_ss_stokes.f". The sign of 
c                        XM_1, and XM_S1 is irrelevant here.
                     CALL REFL_DIR_SPH(1,1,1, XM_1(Nr:Nr), XM_S1(Nr:Nr),
     $                    PHI_LOS(Nr:Nr), POS, ALBEDO, FLU_int,  
     $                    SOL_EXP(Nr), TE%PLANCK_SURF, BC_Dir, LBC)
c                    BC_Dir = 0.d0 ! without reflection of direct radiation
                  ENDIF         ! Mha==1

!     Check if surface in the valid range of SZAs, if not, no reflection
!     of the diffuse radiation
c                 ... Diffuse radiation reflected by surface
c     ------------------------------------------
                  IF( .NOT.BRDF_FLAG ) THEN
c                    ... Lambertian surface reflection (diffuse radiation)
                     Tls = 0.d0 ; BC_Dif(:) = 0.d0
                     Tls(:,1) = Albedo*2.d0*FLUX(1:NSOL)
c                    ... Interpolation at a local solar angle along l-of-s
c                         for all basis solar angles
                     IF( NSOL== 1 ) THEN
                        BC_Dif(1) = Tls(1,1)
                     ELSE
                        CALL AKIMA_INT(NSOL, SOL_GRD, Tls(:,1),
     $                       1, XM_S1(Nr), BC_Dif(1) )
                     ENDIF
                     BC_wf(:) =  BC_Dif(:)
                  ELSE
c                 ... Bidirectional surface reflection
c                 ------------------------------------------------------ 
c                    ... The array Ref_surf contains the reflected 
c                        radiation from the surface (see "st_top_bot.f")
c                        calculated on the solar angles grid
                     DO Jss = 1,NSOL
                        DO K1 = 1,Nvs 
                           CALL AKIMA_INT(NSTRMS+1, X_ref, 
     $                          Ref_surf(Jss,K1:Nvs*(NSTRMS+1):Nvs),
     $                          1, XM_1(Nr), Fg_INT(K1) )
                        ENDDO  ! over Stokes components (K1)
c                       ... Interpolation at a given L-of-S angle
                        Tls(Jss,:) = 2.d0*Fg_INT(:)
                     ENDDO  ! over solar angles grid (Jss)
c                    ... Interpolation at a given solar angle: BC_Dif
                     IF( NSOL== 1 ) THEN
                        BC_Dif(:) = Tls(1,:)
                     ELSE
                        DO K1 = 1,Nvs 
                           CALL AKIMA_INT(NSOL, SOL_GRD, Tls(:,K1),
     $                          1, XM_S1(Nr), BC_Dif(K1) )
                        ENDDO  ! over Stokes components (K1)
                     ENDIF
c                     !!! VR correction 08.03.2015 !!!
                     D1 = PHI_LOS(NR)*DBLE(Mha-1)
                     BC_Dif(1) = BC_Dif(1)*DCOS( D1 )   !*VST
                     IF( Nvs > 1 ) THEN
                        BC_Dif(2)   = BC_Dif(2)*DCOS( D1 )   !*VST
                        BC_Dif(3:4) = BC_Dif(3:4)*DSIN( D1 ) !*VST
                     ENDIF  
                     BC_wf(:) =  BC_Dif(:) !* VST
                  ENDIF  ! BRDF_FLAG

c                 ------------------------------------------------------
c                  BC_Dif(:) = 0.d0 ! test without diff reflection
               END BLOCK Surface_Refl ! Reflection by surface

               IF( WF_MS_FLAG(JPOS) ) THEN
c              ... WF in MS representation
                  S_F_Tes = S_F !0.d0 
                  S_R_Tes = S_R  
                  WF_dir(:) = 0.d0 ! Eto verno ! 
                  LBC(:) = LBC(:)*VST
               ELSE
c              ... WF in SS approximation:
                  S_F_Tes = 0.d0 ; S_R_Tes = S_R ; BC_wf(:) = 0.d0
                  IF( Mha > 1 )  S_R_Tes = 0.d0 ; LBC(:) = LBC(:)*VST
                  WF_dir(:) = DS_SUN(Nr,:)
               ENDIF

               CALL SPHER_WF(SSSF, DS_SUN, S_F_Tes, S_R_Tes, EX_L, EX_1,
     $              Nr, D_DS, IOB, ZLS, Mha, AL_1, PWF, IV, IS, IP,
     $              JPOS, TE%ADD_PLANCK)

c              ... Array DS_SUN contains after call of "spher_wf.f"
c                  pathlength elements along l-of-s
               Index = GeoD%ILO_SVFP(IV,IS,IP)
c              ... Contribution of extinction coefficient variation 
c                  within viewing direction transmission "exponent"
c                  of direct and diffuse reflected from surface radiation
c                  ( See Eq.(T2-sph) in STB )
               DO Jw = 1,Ns_wf
                  Jw1 = Nc_der(Jw)
                  PWF%WF_esp(:,Index,Jw) = PWF%WF_esp(:,Index,Jw) 
     $                 - BC_wf(Jw1)*DS_SUN(Nr,:)*EX_L(Nr) *WFOV
c                        ... Direct solar light
     $                 - LBC(Jw1)*( WF_dir(:) + DS_SUN(Nr,:) )
     $                 *SOL_EXP(Nr)*EX_L(Nr)*XM_S1(Nr) *WFOV
               ENDDO

               IF( NUM_WF ) THEN 
c                 ... To calulate without contribution from solar exponent
                  IF( NO_SOL ) AL_1(:) = AL_1_PERT(:)
c                 ... To calulate without contribution from LS exponent
                  IF( NO_LS  ) AL_1(:) = AL_1_BASE(:)
               ENDIF

            END DO  ! over azimuth angles (IP)
         END DO  ! over solar zenith angles (IS)

         NULLIFY(ZLS, SID, IOB, XM_1, EX_L, XM_S1, XM_SG)
         DEALLOCATE( EX_1 )
         IF( DO_AEROSOL ) DEALLOCATE(G_aer, SCA_LS, DM_LS)  ! DA
         IF( DO_CLOUD )   DEALLOCATE(CI, SCC_LS, SCCNO_LS)  ! DA
c         IF( TE%ADD_PLANCK ) DEALLOCATE(Therm)
      END DO  ! over viewing angles including FOV (IV)
      
      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN
      CONTAINS
      
c *************************************************************************
c     This subroutine calculates the contribution in radiation along 
c     line-of-sight from the direct scattered solar beam. Therefore, it 
c     should be used here all phase matrix expansion coefficients.

      SUBROUTINE SINGLE_IN_MS_MODE
      INTEGER  Isf, I1, I2, Jmc
      REAL (KND)  DL, Ph_W, Wig0_ss(Npo,NR), Wig2_ss(Npo,NR), HH(NR,Nvs)
      REAL (KND), POINTER :: ALP(:), EXL(:)

c       Scattering angle along line-of-sight
c       ------------------------------------
        COSCA(1:NR) = -XM_1(1:NR)*XM_S1(1:NR) 
     $     + DSQRT(1.d0 - XM_1(1:NR)*XM_1(1:NR))
     $     *DSQRT(1.D0 - XM_S1(1:NR)*XM_S1(1:NR))
     $     *DCOS(PHI_LOS(1:NR))

c     Solar exponent at all point along LS
c       ------------------------------------
        ASSOCIATE (LVG => GeoD%LVG(IA,IV), LSG => GeoD%LSG(IA,IV,IS,IP))
        CALL DirSun (NR, XM_S1, LSG%TH, ZLS, LVG%IOB, AL_1(1:NR), 
     $       ALO, PCC, SOL_EXP(1:NR), LSG%REFR_DSB, PreCalcAngs, DO_WF,
     $       DS_SUN, LSG%DSB)
        END ASSOCIATE

c       Legendre polynomials for all scattering angles
c     ----------------------------------------------
        CALL WIGNER_PLG(COSCA(1:NR), NR, 0, Npo, Wig0_ss(:,1:NR))
        IF( Nm_vs > 1 ) THEN
           CALL WIGNER_PLG(COSCA(1:NR), NR, 2, Npo, Wig2_ss(:,1:NR))
           CALL ROT_MATR
        ENDIF

c       ... Rayleigh contribution to the source function
        Npl = Nlg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)
        DO J = 1,NR
           HH(J,1) = SUM( Wig0_ss(1:Npl,J)*P_MOM(:) )
        ENDDO
        IF( Nm_vs > 1 ) THEN
           HH(:,4) = 0.d0
c           CALL WIGNER_PLG( COSCA(1:NR), NR, 2, Npo, Wig2_ss(:,1:NR) )
c           CALL ROT_MATR
           P_MOM => LOA%RAY_MOMS(1:Npl,5) 
           DO J = 1,NR
              HH(J,2:3) = Rot_l(J,2:3)*SUM(Wig2_ss(1:Npl,J)*P_MOM(:))
           ENDDO
        ENDIF
        DO K = 1,Nvs
           SSSF(1:NR,K) = SCRL(1:NR)*HH(1:NR,K)
        ENDDO

c       ... If WF for Rayleigh scat. should be calculated
        IF( Ray_to_WF ) S_R(1:NR,1,1) = 
     $       SOL_EXP(1:NR)/4.d0*HH(1:Nr,1)/AL_1(1:Nr)

c       ... If atoms or ions emission should be calculated:
c       ----------------------------------------------------------------
        IF( Emi_FLG ) THEN
           DO Ka = 1,N_AT
              DO J = 1,NR
                 HH(J,1) = SUM( Wig0_ss(1:3,J)*LOA%EPF(Ka,:,1) )
              ENDDO
              IF( Nm_vs > 1 ) THEN
                 HH(:,4) = 0.d0
                 DO J = 1,NR
                    HH(J,2:3) = Rot_l(J,2:3)
     $                   *SUM(Wig2_ss(1:Npl,J)*LOA%EPF(Ka,:,5))
                 ENDDO
              ENDIF
              FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) + 
     $             EMCL(Ka,1:NR)*HH(1:NR,K)
           ENDDO  ! over atoms (Ka)

           IF(INQ_WF('atom', 'WFT')) S_R(1:NR,1,1) = 
     $          SOL_EXP(1:NR)/4.d0*HH(1:Nr,1)/AL_1(1:Nr)
        ENDIF  ! atoms or ions 
c       ----------------------------------------------------------------

c       ... Aerosol contribution to the source function
        IF( DO_AEROSOL ) THEN
           SELECT CASE( APH_TYPE )
           CASE('h-g')
c                   write(*,*) '   H_G-parameter'
c                   write(*,*) G_aer(1:Nr)
              S_H(1:NR) = 1.d0 + G_aer(1:Nr)*G_aer(1:Nr)
     $             - 2.d0*G_aer(1:Nr)*COSCA(1:Nr)
              S_H(1:NR) = (1.d0 - G_aer(1:Nr)*G_aer(1:Nr))
     $             /S_H(1:Nr)/DSQRT(S_H(1:Nr))
              SSSF(1:NR,1) = SSSF(1:NR,1) 
c                    ... SCA_LS does not contain delta-m factor
     $             + S_H(1:Nr)*SCA_LS(1:Nr)
c     $             + S_H(1:Nr)*SCA_LS(1:Nr)/(1.D0 - DM_LS(1:Nr))
           CASE('mie')
              Npl = Leg_Aer ; Phm(:,:) = 0.d0
              DO J = 1,NR
c                 WRITE(*,*) J, IDX_AER(IOB(J))
                 Phm(J,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),1) )
              ENDDO
              S_H(1:NR) = Phm(1:Nr,1)
              IF( Nvs > 1 ) THEN
                 DO J = 1,NR
                    Phm(J,2:3) = Rot_l(J,2:3)*SUM( Wig2_ss(1:Npl,J)
     $                   *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),5) )
                 ENDDO                     
              ENDIF
              DO K = 1,Nvs
c                ... SCA_LS does not contain delta-m factor
                 SSSF(1:NR,K) = SSSF(1:NR,K) + Phm(1:Nr,K)*SCA_LS(1:Nr)
c     $                *SCA_LS(1:Nr)/(1.d0 - DM_LS(1:Nr))
              ENDDO
           END SELECT
        ENDIF

c       ... Cloud contribution to the source function
        IF( DO_CLOUD ) THEN
c       ----------------------------------------------------------------
           Phm_mc = 0.d0  ;  Npl = Leg_cld 
           DO J = 1,Nr
              IF( CI(J) == 0 ) CYCLE
              DO Jmc = 1, MLCS(CI(J))%N_pha
                 IF( MLCS(CI(J))%Npf(Jmc)%Y_Scm ) THEN
c                ... Use scattering matrix:
                    P_MOM => OSM%CLD_SCM(1:Ydb_ang,CI(J),1,Jmc,1) 
                    CALL LIN_SEARCH_NEW
     $                   (COSCA(J), DL, I1, I2, Ydb_ang, SCAT_ANG)
                    Phm_mc(Jmc,1) = P_MOM(I2)*DL + P_MOM(I1)*(1.d0 - DL)
                    IF( Nvs == 1 ) CYCLE
                    P_MOM => OSM%CLD_SCM(1:Ydb_ang,CI(J),1,Jmc,5) 
                    Phm_mc(Jmc,2:3) = Rot_l(J,2:3)*
     $                   P_MOM(I2)*DL + P_MOM(I1)*(1.d0 - DL)
                    CYCLE
                 END IF
c                ... Use expansion coefficients:
                 Phm_mc(Jmc,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%CLD_ALL(1:Npl,CI(J),1,Jmc,1) )
                 IF( Nvs == 1 ) CYCLE
                 Phm(Jmc,2:3) = Rot_l(J,2:3)*SUM(Wig2_ss(1:Npl,J)
     $                *OSM%CLD_ALL(1:Npl,CI(J),1,Jmc,5))
              ENDDO  ! over water/ice phase (Jmc)
              DO K = 1, Nm_ice
                 SSSF(J,:) = SSSF(J,:) + Phm(K,:)*SCCNO_LS(J,K)
              ENDDO
           ENDDO  ! over points along LoS (J)
c       ----------------------------------------------------------------
        END IF ! DO_CLOUD

c        WRITE(*,*) '   Solar exponent in "sfi_spher_RF.f" ', NR
        DO K = 1,Nvs
           SSSF(1:NR,K) = SSSF(1:NR,K)*SOL_EXP(1:NR)/4.d0
        ENDDO
        
cc       ... Thermal emission contribution to the source function
c        IF( TE%ADD_PLANCK ) SSSF(1:NR,1) = SSSF(1:NR,1) + Therm(1:NR)
        
        IF(INQ_WF('aer', 'WFT')) THEN
c           WRITE(*,*) '   NR  "aer", "WFT"', NR
           S_R(1:NR,1,1) = SOL_EXP(1:NR)/4.d0*S_H(1:NR)/AL_1(1:Nr)
c           write(*,*) '  S_R (in ss)  ', S_R(:,1,1)
           IF( Dwr_mi ) THEN
              Npl = Leg_Aer  
              DO Idr = 1,N_dwr
                 FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0_ss(1:Npl,J)
     $                *OSM%DER_ALL(1:Npl,IDX_AER(IOB(J)),1,Idr) )
                 S_R(1:Nr,1,1+Idr ) = Phm(1:Nr,1)*SOL_EXP(1:NR)/4.d0
c                     ... SCA_LS does not contain delta-m factor
     $                *SCA_LS(1:Nr)/AL_1(1:Nr)
c     $                *SCA_LS(1:Nr)/(1.d0 - DM_LS(1:Nr))/AL_1(1:Nr)
              ENDDO
           ENDIF
        ENDIF

      END SUBROUTINE SINGLE_IN_MS_MODE
      
c *************************************************************************
c     This subroutine calculates the contribution in radiation along 
c     line-of-sight from the multiple scattering source function for 
c     each Fourier harmonic (scalar case). Therefore, it should be  
c                            ===========
c     used here NLEGEN phase matrix expansion coefficients only. 
c *************************************************************************
      SUBROUTINE PHF_ALONG_LS   ! For scalar case

      CALL WIGNER_PLG( -XM_1(IMS1:IMS2), IMS2 - IMS1 + 1, Mha-1, NLg,
     $     Pol_ls(:,IMS1:IMS2) )

      SUM_MOM = 0.d0

      IF( Mha <= Nlg_ray ) THEN
         DO L = IMS1, IMS2
            SUM_MOM(Mha:Nlg_ray,L) = 
     $           LOA%RAY_MOMS(Mha:Nlg_Ray,1)*SCRL(L)
         ENDDO
c          ... Contribution of atoms/ions emission if any 
c          --------------------------------------------------------
         IF( Emi_FLG ) THEN
            DO Ka = 1, N_AT
               DO L = IMS1, IMS2
                  SUM_MOM(Mha:Nlg_ray,L) = SUM_MOM(Mha:Nlg_ray,L)
     $                 + LOA%EPF(Ka,Mha:Nlg_Ray,1)*EMCL(Ka,L)
               ENDDO  ! over line of sight (L)
            ENDDO  ! over absorber (Ka)
         ENDIF
c          --------------------------------------------------------
      ENDIF

      IF( DO_AEROSOL ) THEN
         DO L = IMS1, IMS2
            SUM_MOM(Mha:Nlg,L) = SUM_MOM(Mha:Nlg,L)
     $           + LOA%AER_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1)*SCA_LS(L)
     $           * (1.d0 - DM_LS(L))
         ENDDO
      ENDIF

      IF( DO_CLOUD ) THEN
         FORALL( L=IMS1:IMS2, CI(L)>0 ) SUM_MOM(Mha:Nlg,L) = 
     $        SUM_MOM(Mha:Nlg,L)
     $        + LOA%CLD_MOMS(Mha:Nlg,CI(L),1,1,1)*SCC_LS(L,1)
         IF( Nm_ice==2 ) FORALL( L=IMS1:IMS2, CI(L)>0 )
     $        SUM_MOM(Mha:Nlg,L) = SUM_MOM(Mha:Nlg,L) 
     $        + LOA%CLD_MOMS(Mha:Nlg,CI(L),1,2,1)*SCC_LS(L,2)
      ENDIF
        
      DO L = IMS1, IMS2 ; TTT1 = 0.d0
         DO K = 1,NN_MS
            P_F = SUM( SUM_MOM(Mha:Nlg,L)
     $           *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K) ) 
            TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
         END DO                         
         S_F(L,1) = TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )
      END DO   

      IF( Ray_to_WF ) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%RAY_MOMS(Mha:3,1)*Pol_ls(Mha:3,L)
     $                 *Pol_ga(Mha:3,K) )   
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
c                    Single scatt test
c                ... Array S_R is set to zero for Mh > 0
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
            END DO   
         ENDIF   ! multiple scattering for Rayleigh WFs
      ENDIF  ! Rayleigh WF

      IF(INQ_WF('atom', 'WFT')) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
            Ka = 1
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%EPF(Ka,Mha:3,1)*Pol_ls(Mha:3,L)
     $                 *Pol_ga(Mha:3,K) )   
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
c                ... Array S_R is set to zero for Mh > 0
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
            END DO   
         ENDIF   ! multiple scattering for atoms WFs
      ENDIF  ! Atoms WF

      IF( INQ_WF('aer', 'WFT') ) THEN
         IF( WF_MS_FLAG(JPOS) ) THEN
c            write(*,*) '   Number of moments Nlg ', Nlg
c           write(*,*) '  S_R (do ms) --- ', S_R(:,1,1)
            DO L = IMS1, IMS2 ; TTT1 = 0.d0
               DO K = 1,NN_MS
                  P_F = SUM( LOA%AER_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1)
     $                 *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K)) 
                  TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
               END DO                         
c                    Single scatt test         
c                ... Array S_R is set to zero for Mh > 0
               S_R(L,1,1) = S_R(L,1,1)
     $              + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L) 
            END DO   
c           write(*,*) '  S_R (in ms --- )  ', S_R(:,1,1)
c           ------------------------------------------------------------
            IF( Dwr_mi ) THEN
               DO Idr = 1,N_dwr
                  DO L = IMS1, IMS2 ; TTT1 = 0.d0
                     DO K = 1,NN_MS
                        P_F = SUM( 
     $                       LOA%ADR_MOMS(Mha:Nlg,IDX_AER(IOB(L)),1,Idr)
     $                       *Pol_ls(Mha:Nlg,L)*Pol_ga(Mha:Nlg,K) ) 
                        TTT1 = TTT1 + P_F*AG1_2(K)*FL(K,L)
                     END DO                         
c                           Single scatt test         
c                      ... Array S_R is set to zero for Mh > 0
                     S_R(L,1,1+Idr) = S_R(L,1,1+Idr)
     $                    + TTT1*DCOS( DBLE(Mha-1)*PHI_LOS(L) )/AL_1(L)
     $                    * SCA_LS(L)*(1.d0 - DM_LS(L))
                  ENDDO  ! over line-of-dight (L)
               ENDDO  ! over PSD function derivatives (Idr)
            ENDIF  ! Dwr_mi
c           ------------------------------------------------------------
         ENDIF  ! multiple scattering for WFs
      ENDIF  ! Aerosol WF

      END SUBROUTINE PHF_ALONG_LS

c *************************************************************************
      SUBROUTINE ROT_MATR
      REAL (KND)  Azm, Zn, T2, Xmu, Mu0
      Rot_l = 0.d0
c     ... Within this subroutine Mu (direction of radiation) 
c         and Mu0 (direction of solar radiation) have to be 
c         given with respect to the tau-axis !!!
      DO J = 1,NR
         Xmu = -XM_1(J)  ;  Mu0 = XM_S1(J) ; Azm = PHI_LOS(J)
         IF( REAL(Azm) == REAL(Pie) .OR. REAL(AZM) == 0.0) THEN 
            Rot_l(J,2) = 1.d0
            Rot_l(J,3) = 0.d0
            CYCLE
         ENDIF
         IF( Azm > Pie ) THEN 
            Zn = 1.0d0
         ELSE
            Zn = -1.0d0
         ENDIF
         IF( DABS(Xmu) == 1.0d0 ) THEN
            T2 = DSIGN(1.d0,Xmu)*DCOS(Azm)
         ELSE
            T2 = -( Mu0 - Xmu*COSCA(J) )
     $           /DSQRT((1.D0 - Xmu*Xmu)*(1.D0-COSCA(J)**2))
         ENDIF
         Rot_l(J,2) = 2.d0*T2*T2 - 1.d0 
         Rot_l(J,3) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
      ENDDO
      END SUBROUTINE ROT_MATR

c     ******************************************************************
      SUBROUTINE SET_COEFF
      INTEGER  J, Ji(2), Ki(2), I, K
      REAL (KND) Di(2)
        Ji(:) = 0
        IF( GeoD%LVG(IA,IV)%TH_FLG ) THEN
           Ji(1) = NTH ; Ki(1) = SID(NTH) ; K2 = Ki(1)
           Di(1) = ( ZLS(NTH) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF
        IF( GeoD%LVG(IA,IV)%FP_FLG ) THEN
           Ji(2) = 1 ; Ki(2) = SID(1) ; K2 = Ki(2)
           Di(2) = ( ZLS(1) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF
        AL_1(1:NR) = ALO( SID(1:NR) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I)
           AL_1(Ji(I)) = ALO(K2+1) + (ALO(K2) - ALO(K2+1))*Di(I)
        ENDDO

        SCRL(1:Nr) = SCRO( SID(1:Nr) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I) 
           SCRL(Ji(I)) = SCRO(K2+1) + (SCRO(K2)-SCRO(K2+1))*Di(I)
        ENDDO

        IF( Emi_FLG ) THEN
           DO J = 1,N_AT 
              EMCL(J,1:Nr) = EMCO( J,SID(1:Nr) )
              DO I = 1,2 
                 IF( Ji(I)==0 ) CYCLE
                 K2 = Ki(I) 
                 EMCL(J,Ji(I)) = EMCO(J,K2+1) 
     $                + (EMCO(J,K2) - EMCO(J,K2+1))*Di(I)
              ENDDO
           ENDDO
        ENDIF

        IF( DO_AEROSOL ) THEN
           G_aer (1:NR) = ASCO( SID(1:NR) ) 
           SCA_LS(1:NR) = SCAO( SID(1:NR) )
           DM_LS (1:NR) = DM_AO(SID(1:NR) )
           DO I = 1,2 
              IF( Ji(I)==0 ) CYCLE
              K2 = Ki(I) ; D1 = Di(I)
              G_aer (Ji(I)) = ASCO(K2+1) + (ASCO(K2) - ASCO(K2+1))*D1 
              SCA_LS(Ji(I)) = SCAO(K2+1) + (SCAO(K2) - SCAO(K2+1))*D1 
              DM_LS (Ji(I)) = DM_AO(K2+1)+ (DM_AO(K2)-DM_AO(K2+1))*D1
           ENDDO
        ENDIF

        IF( DO_CLOUD ) THEN
           CI    (1:NR)   =  INT( CLA(IOB(1:NR),2) )
           DO J = 1, Nm_ice
              SCC_LS  (1:NR,J)   = SCCO (SID(1:NR),J)
              SCCNO_LS(1:NR,J) = SCC_NSO(SID(1:NR),J)
           ENDDO
        ENDIF

        IF( TE%ADD_PLANCK ) THEN 
           Therm(1:NR) = TE%PLA_FINE( SID(1:NR) ) 
           DO I = 1,2
              IF( Ji(I)==0 ) CYCLE
              K2 = Ki(I)
              Therm(Ji(I)) = TE%PLA_FINE(K2+1) 
     $             + (TE%PLA_FINE(K2) - TE%PLA_FINE(K2+1))*Di(I)
           ENDDO
        ENDIF        
      END SUBROUTINE SET_COEFF

* *************************************************************************
      SUBROUTINE INT_ON_TH_LEV(Poi, INI, Rez)
      INTEGER     Poi, INI
      REAL (KND)  ZV, Rez(NSOL,NN_MS*Nvs,4)

        ZV = ZLS(Poi)
        CALL BIAKIMA_Y( Z_INV(NLAYERS-INI+1), 
     $       SZA_EXT, ZV, X_RAD(NLAYERS-INI+1,:,:,:), Rez)

      END SUBROUTINE INT_ON_TH_LEV
C     *****************************************************************
      END

