C********************************************************************C
C
C       SCIATRAN - Version 4.2 - Last modification: 18 Mar 2020
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
* Input parameters :                                                   
*                                                                      
*     LCD%WF          - two-dimensional array of "column" weighting function  
*                    for all parameter and wavelength                      
*     N_CEN        - first true dimension - number of wavelength           
*     N_PAR        - second true dimension - number of parameter           
*                                                                       
*     LCD%RAD      - one-dimensional(1:N_CEN) array with model radiance    
*     CRS%DAT      - two-dimensional(1:N_CEN, N_CRS) array with spectral     
*                    corrections                                           
*     LCD%SOL      - one-dimensional(1:N_CEN) array with Solar spectrum    
*     LCD%MSMT     - one-dimensional(1:N_CEN) array with Earthshine        
*                    spectrum                                              
*     WIDE_S       - two-dimensional array (1:2,1:CRS%N_CRS+2) maximal shift   
*                    and squeeze range [nm] for right and left spectral    
*                    point for : Simulated Earthshine spectrum, Solar      
*                                spectrum and spectral corrections         
*     GR_P         - two-dimensional array (1:2,1:(N_PAR+1)) lower and     
*                    upper bounds for N_PAR parameter and Ring spectrum    
*     LCD%WL       - one-dimensional(1:N_CEN) array with wavelength [nm]   
*     IW           - window number for output file formation               
*     POL_MODE     - with or without polynomial substraction               
*     ER_SCA       - on exit: absolute errror after retrieval with this    
*                             routine                                       
*     CRS%IEC_FLAG - if 'TRUE' turns on the interpolation error correction 
*                    mode                                                  
*     CRS%TILT_FLAG - if 'TRUE' calculates the tilt correction spectrum
*                                                                      
************************************************************************
      SUBROUTINE SHI_SQU (N_CEN, N_PAR, LCD, CRS, WIDE_S, GR_P, I_DS,  
     $     IW, POL_MODE, ER_SCA, AMP_PAR, L_P, JTH, INDEX, NUR, SQ_RTB, 
     $     SS_PARS, *)
c     ... Call in "preprocessing.f"
 
      USE LOG_PARS,  ONLY: DIR_OUT, LOG_LEVEL, N_ESO, N_SSO, N_DSO, 
     $                     LUNIT, C_UNIT
      USE DATA_RET,  ONLY: MDS
      USE TYPES,     ONLY: KND, PREPROC_DATA, LOCAL_DATA, CORR_SPEC
c     ------------------------------------------------------------------

C     Initialization of GALAHAD quadratic problem solver
C     **************************************************
      USE GALAHAD_QPB_double    ! double precision version
      REAL (KND), PARAMETER :: INFINITY = 1.0D+20
      TYPE ( QPT_PROBLEM_TYPE ) :: PRT
      TYPE ( QPB_DATA_TYPE ) :: DATA_STORE
      TYPE ( QPB_CONTROL_TYPE ) :: CONTROL        
      TYPE ( QPB_INFORM_TYPE ) :: INFO

C     Input parameters and auxiliary internal variables
C     *************************************************
      INTEGER  N_CEN, N_PAR, L_P, JTH, IW, NUR, N_SC
      INTEGER  JJ, ISTAT, NN, I_DS, JSEG
      LOGICAL  POL_MODE
      LOGICAL  INDEX(N_PAR)
      LOGICAL  IDX_TMP_1(N_PAR)
      TYPE (LOCAL_DATA), TARGET :: LCD
      TYPE (CORR_SPEC), TARGET :: CRS
      REAL (KND)  WIDE_S(2,CRS%N_CRS + 2), GR_P(2,N_PAR + CRS%N_CRS), 
     $     ER_SCA(NUR), AMP_PAR(N_PAR), SS_PARS(2,CRS%N_CRS + 2)
c     ... Local parameters
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SHI_SQU'
      CHARACTER (LEN = :), ALLOCATABLE :: CONV_TYPE
      CHARACTER (LEN=1) NAM_WIN_1
      CHARACTER (LEN=4) NAM_TH
      LOGICAL IDX_RIN(CRS%N_CRS), IDX_TMP_2(CRS%N_CRS)
      LOGICAL, ALLOCATABLE :: IDX_MOD_SS(:), IDX_SOL_SS(:), IDX_SHI(:), 
     $     IDX_SQ(:), IDX_EXT(:)
      INTEGER  I, J, K, II, IP, N_FIT, N_GAS, I_VIBR, N_SHI, N_SQ, 
     $     N_PARL, NCLIN, ITERATION
      INTEGER, PARAMETER :: MAX_ITER = 500   
      REAL (KND)  SQ_RTA, SQ_RTB, S, SHI_MAX
      REAL (KND)  WL_NEW(NUR), RMS(MAX_ITER)
      REAL (KND), ALLOCATABLE :: GR_PAR(:,:), WF_ALL(:,:), RESULT(:), 
     $     WF_PAR(:,:), WIDE_SHI(:,:), SHI_SQU_PAR(:,:)

c     ... Test parameters
      CHARACTER (LEN=7) QP_LIB

c     ==================================================================

      CALL ST_TRACK (R_NAME, 2, 'I')
      CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'This routine is not a part '
     $     //'of the freely distributed software package')
 
      STOP
      RETURN
      END
