************************************************************************
*                                            V.Rozanov,   A. Rozanov   *
*                                             16.09.96     25.07.02    *
*                                                          08.03.06    *
*                                                          24.08.07    *
*                                                                      *
*        This version of shi_squ routine is based on                   *
*                program slant_sat-wei.f                               *
*        Version with constrains for all variable                      *
*                                                                      *
* Input parameter :                                                    *
*                                                                      *
*     W_F - two dimension array of "column" weighting function         *
*           for all parameter and wavelength                           *
*      N_CEN  - first true dimension - number of wavelength            *
*      N_PAR   - second true dimension - number of parameter           *
*                                                                      *
*     RAD_I    - one-dimensional(1:N_CEN) array with model radiance    *
*     RIN      - one-dimensional(1:N_CEN) array with Ring spectrum     *
*     SOL_S    - one-dimensional(1:N_CEN) array with Solar spectrum    *
*     ERD_S    - one-dimensional(1:N_CEN) array with Earthshine        *
*                spectrum                                              *
*     WIDE_SHI - two-dimensional array (1:2,1:N_RIN+2) maximal shift   *
*                and squeeze range [nm] for right and left spectral    *
*                point for : Simulated Earthshine spectrum, Solar      *
*                            spectrum and spectral corrections         *
*     GR_PAR   - two-dimensional array (1:2,1:(N_PAR+1)) lower and     *
*                upper bounds for N_PAR parameter and Ring spectrum    *
*     WL       - one-dimensional(1:N_CEN) array with wavelength [nm]   *
*     IW     - window number for output file formation                 *
*    POL_MODE    - with or without polynomial substraction             *
*    ER_SCA   - on exit: absolute errror after retrieval with this     *
*                        routine                                       *
*       VDM   - virtual dimension of array MATRIX                      *
*                                                                      *
************************************************************************
      SUBROUTINE SHI_SQU (W_F, N_CEN, N_PAR, RAD, RIN, SOL_S, ERD_S,
     $     WIDE_SHI, GR_P, WL, AMP_RIN, IW, POL_MODE, ER_SCA, 
     $     AMP_PAR, LOST_POINT, JTH, INDEX, NUR, MATRIX, VDM, SQU_FLAG, 
     $     N_RIN, SQ_RTB, SHI_SQU_PAR, RING_MODE)
 
      USE LOG_CONTROL, ONLY: DIR_OUT, LOG_LEVEL, N_ESO, N_SSO, N_DSO, 
     $     LUNIT, C_UNIT

C     Initialization of GALAHAD quadratic problem solver
C     **************************************************
      USE GALAHAD_QPB_double    ! double precision version
      DOUBLE PRECISION, PARAMETER :: INFINITY = 1.0D+20
      TYPE ( QPT_PROBLEM_TYPE ) :: PRT
      TYPE ( QPB_DATA_TYPE ) :: DATA_STORE
      TYPE ( QPB_CONTROL_TYPE ) :: CONTROL        
      TYPE ( QPB_INFORM_TYPE ) :: INFO

C     Input parameters and auxiliary internal variables
C     *************************************************
      INTEGER           LOST_POINT, N_CEN, N_PAR, N_RIN, VDM, N_FIT, II
      INTEGER           JTH, IW, N_GAS, IP, I, N_POS, K, I_VIBR, NUR, J
      INTEGER           N_SHI, N_SQ, N_PARL, NCLIN, NCTOTL, ITERATION
      INTEGER, PARAMETER :: MAX_ITER = 500   

      CHARACTER (LEN=1) NAM_WIN_1, RING_MODE(N_RIN)
      CHARACTER (LEN=3) INT_TO_CHAR
      CHARACTER (LEN=4) NAM_TH
      CHARACTER (LEN=7) QP_LIB

      LOGICAL           POL_MODE, LGA, LRI, LSH, LSQ
      LOGICAL           SQU_FLAG(3), INDEX(N_PAR)
      LOGICAL, ALLOCATABLE :: IDX_MOD_SS(:), IDX_SOL_SS(:), IDX_SHI(:), 
     $     IDX_SQ(:), IDX_EXT(:)

      DOUBLE PRECISION, DIMENSION (N_CEN) :: SOL_S, ERD_S, RAD, WL  
      DOUBLE PRECISION, DIMENSION (NUR) :: ER_SCA, WL_NEW
      DOUBLE PRECISION  W_F(N_CEN,N_PAR), SHI_SQU_PAR(2,N_RIN + 2)
      DOUBLE PRECISION  RMS(MAX_ITER), GR_P(2,N_PAR + N_RIN)
      DOUBLE PRECISION  WIDE_SHI(2,N_RIN + 2), AMP_PAR(N_PAR)
      DOUBLE PRECISION  RIN(N_CEN,MAX(1,N_RIN)), MATRIX(VDM,VDM)
      DOUBLE PRECISION  AMP_RIN(MAX(1,N_RIN)), SHIFTS(2,N_RIN + 2)
      DOUBLE PRECISION, ALLOCATABLE :: GR_PAR(:,:), WF_ALL(:,:), 
     $     RESULT(:), RESULT_OLD(:), Y_SOL(:), Y_RAD(:), WF_PAR(:,:), 
     $     OUT_ERD(:)

      DOUBLE PRECISION  SQ_RTA, SQ_RTB, S, STEP, SHI_MAX

      EXTERNAL          INT_TO_CHAR

C     General initialization of the quadratic problem solver
C     ******************************************************
      CONTROL%INFINITY = infinity
      PRT%NEW_PROBLEM_STRUCTURE = .TRUE. 
      CALL SMT_put (PRT%H%TYPE, 'DENSE') 
      CALL SMT_put (PRT%A%TYPE, 'DENSE') 
C     ******************************************************

      LGA = .FALSE.; LRI = .FALSE.; LSH = .FALSE.; LSQ = .FALSE.

      IF (LOG_LEVEL(2) >= N_ESO) THEN 
         write(LUNIT,*) 
     $        ' Wavelength range for shift and squeeze correction '
         write(LUNIT,'(2x,2(2x,f8.3))') WL(1),WL(N_CEN)
      END IF
      IF (LOG_LEVEL(2) >= N_SSO) THEN 
         IF(N_RIN > 0) THEN
            write(LUNIT,*) 
     $           ' ****************************************** '
            write(LUNIT,*) 
     $           '   Shift/squeeze with spectral corrections  '
            write(LUNIT,*) 
     $           ' ****************************************** '
         ELSE
            write(LUNIT,*) 
     $           ' ********************************************* '
            write(LUNIT,*) 
     $           '   Shift/squeeze without spectral corrections  '
            write(LUNIT,*) 
     $           ' ********************************************* '
         END IF      
      ENDIF
      IF (LOG_LEVEL(2) >= N_ESO) 
     $     WRITE(LUNIT,*) ' N_CEN, I_DANGER :' ,N_CEN, LOST_POINT

      IF (LOG_LEVEL(2) >= N_ESO) THEN 
         NAM_WIN_1 = CHAR(48+IW)
         NAM_TH = '-'//INT_TO_CHAR(JTH)

         OPEN(11,FILE=DIR_OUT//'rad-mod-0'//NAM_WIN_1//TRIM(NAM_TH))
         DO K = 1 + LOST_POINT, N_CEN - LOST_POINT
            WRITE(11,'(1x,f10.4,30(1x,e12.4))')  
     $           WL(K), RAD(K), (W_F(K,IP), IP = 1,N_PAR)
         END DO
         CLOSE(11)

         OPEN(11,FILE=DIR_OUT//'mes-dat-0'//NAM_WIN_1//TRIM(NAM_TH))
         DO K = 1 + LOST_POINT, N_CEN - LOST_POINT
            WRITE(11,'(1x,f10.4,30(1x,e12.4))')  
     $           WL(K), ERD_S(K), SOL_S(K)
         END DO
         CLOSE(11)
      ENDIF

      DO IP = 1, N_PAR
         S = DSQRT(DOT_PRODUCT(W_F(1:N_CEN,IP),W_F(1:N_CEN,IP)))/
     $        DBLE(N_CEN)
         IF( S.LE.1.0D-50) INDEX(IP) = .FALSE.
      END DO

      N_GAS = COUNT(INDEX(1:N_PAR))
      N_FIT = N_GAS + N_RIN
      NUR = N_CEN - 2*LOST_POINT
      ALLOCATE(GR_PAR(2,N_FIT), WF_ALL(N_CEN,N_FIT), RESULT(N_FIT), 
     $     RESULT_OLD(N_FIT), Y_SOL(NUR), Y_RAD(NUR), WF_PAR(NUR,N_FIT), 
     $     OUT_ERD(NUR), IDX_MOD_SS(N_RIN + 2), IDX_SOL_SS(N_RIN + 2),
     $     IDX_SHI(N_RIN + 2), IDX_SQ(N_RIN + 2), IDX_EXT(N_FIT + 2))
      RESULT = 0.0D0

      WL_NEW(1:NUR) = WL(1 + LOST_POINT:N_CEN - LOST_POINT)
      FORALL(I = 1:N_CEN) 
         WF_ALL(I,1:N_GAS) = PACK(W_F(I,:),INDEX(:))
         WF_ALL(I,N_GAS + 1:N_FIT) = RIN(I,1:N_RIN)
      END FORALL   

      FORALL(I = 1:2) GR_PAR(I,1:N_GAS) = PACK(GR_P(I,1:N_PAR),INDEX(:))
      GR_PAR(:,N_GAS + 1:N_FIT) = GR_P(:,N_PAR + 1:N_FIT)

      QP_LIB = 'GALAHAD'
c      QP_LIB = 'NAG'
      IF(QP_LIB == 'GALAHAD') THEN
         CALL FIRST_GUESS
      ELSE
         CALL GUESS_0L(N_CEN, LOST_POINT, N_FIT, WL, WL_NEW, NUR, SOL_S, 
     $        ERD_S, WF_ALL, RESULT, RAD, POL_MODE, GR_PAR, VDM, MATRIX)
      END IF

      SHI_MAX = 0.0D0 ; ITERATION = 0; I_VIBR = 0; RMS = 0.0D0
      SHI_SQU_PAR = 0.0D0

C     Initialization of the quadratic problem solver (iterative part)
C     ***************************************************************
C     Setting of shift and squeeze flags 
      IDX_SHI(1:2)  = .TRUE. ! Model and solar spectra
      IDX_SHI(3:N_RIN + 2)  = .FALSE.
C     Correction spectra:
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'I') IDX_SHI(3:N_RIN + 2) = .TRUE. 
      IDX_SQ = IDX_SHI
      IF(.NOT.SQU_FLAG(3)) IDX_SQ(1) = .FALSE. ! Model spectrum 
      IF(.NOT.SQU_FLAG(2)) IDX_SQ(2) = .FALSE. ! Solar spectrum
C     Correction spectra:
      IF(.NOT.SQU_FLAG(1)) IDX_SQ(3:N_RIN + 2) = .FALSE.

      N_SHI = COUNT(IDX_SHI)
      N_SQ = COUNT(IDX_SQ)

      IDX_EXT(:) = .TRUE. 
      IDX_EXT(3:N_GAS + 2) = .FALSE.

C     Common shift and squeeze for model spectrum, weighting functions,  
C     and selected correction spectra.
      IDX_MOD_SS = .FALSE.
C     Model spectrum
      IDX_MOD_SS(1) = .TRUE.                
C     Correction spectra
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'M') IDX_MOD_SS(3:N_RIN + 2) = .TRUE. 

C     Common shift and squeeze for solar spectrum, and selected 
C     correction spectra.
      IDX_SOL_SS = .FALSE.
C     Solar spectrum
      IDX_SOL_SS(2) = .TRUE. 
C     Correction spectra
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'S') IDX_SOL_SS(3:N_RIN + 2) = .TRUE. 

      N_PARL = N_FIT + N_SHI + N_SQ 
      NCLIN = 2*N_SHI ;  NCTOTL = N_PARL + NCLIN

      ALLOCATE( PRT%G(N_PARL), PRT%X_L(N_PARL), PRT%X_U(N_PARL), 
     $     PRT%X(N_PARL), PRT%Z(N_PARL), PRT%A%val(N_PARL*NCLIN),   
     $     PRT%H%val(N_PARL*(N_PARL + 1)/2), PRT%C(NCLIN), 
     $     PRT%C_L(NCLIN), PRT%C_U(NCLIN), PRT%Y(NCLIN))
      CALL QPB_INITIALIZE (DATA_STORE, CONTROL) 
C     ***************************************************************

 981  CONTINUE

      ITERATION = ITERATION + 1

      RESULT_OLD =  RESULT
      IF(QP_LIB == 'GALAHAD') THEN
         CALL QPB_ITERATE
      ELSE
         CALL GUESS_IL(N_CEN, LOST_POINT, N_FIT, WL, WL_NEW, SOL_S, 
     $        ERD_S, WF_ALL, RESULT, SHI_SQU_PAR, NUR, SQ_RTA, SQ_RTB, 
     $        ITERATION, WIDE_SHI, N_GAS, N_RIN, POL_MODE, RAD, GR_PAR, 
     $        ER_SCA, VDM, MATRIX, SQU_FLAG, RING_MODE, Y_SOL, Y_RAD, 
     $        WF_PAR)
      END IF

      FORALL(I = 1:N_RIN + 2) 
         SHIFTS(1,I) = 
     $     SHI_SQU_PAR(1,I) + SHI_SQU_PAR(2,I)*WL(1 + LOST_POINT)
         SHIFTS(2,I) = 
     $        SHI_SQU_PAR(1,I) + SHI_SQU_PAR(2,I)*WL(N_CEN - LOST_POINT)
      END FORALL

      CALL CHECK_CONVERG(N_GAS, N_RIN, RESULT_OLD, RESULT, SQ_RTA, 
     $     SQ_RTB, SHIFTS, LGA, LRI, LSH, ITERATION, N_CEN, WL, 
     $     SHI_MAX)

      RMS(ITERATION) = SQ_RTB

      IF(LGA.AND.LRI.AND.LSH) THEN
         IF (LOG_LEVEL(2) >= N_SSO) THEN 
            WRITE(LUNIT,*) 
     $           ' ******* Convergence for all parameters *******'
            write(LUNIT,*) ' '
         ENDIF
         GO TO 982
      END IF

      IF(ITERATION.GT.2) THEN
         II = ITERATION
         IF((RMS(II-2)-RMS(II-1)).LE.0.0D0.AND.
     >      (RMS(II-1)-RMS(II)).LE.0.0D0) THEN
            IF (LOG_LEVEL(2) >= N_SSO) THEN 
               WRITE(LUNIT,*) 
     $              ' ******* Convergence on nonlinear RMS ******* '
               write(LUNIT,*) ' '
            ENDIF
            GO TO 982
         END IF
         IF((RMS(II-1) - RMS(II-2))*(RMS(II) - RMS(II-1)).LE.0.0D0) THEN
            I_VIBR = I_VIBR + 1
            IF(I_VIBR.GT.10) THEN
               IF(RMS(II).LT.RMS(II-1)) THEN
                  IF (LOG_LEVEL(2) >= N_SSO) THEN 
                     WRITE(LUNIT,*) ' ******* RMS vibration '//
     $                    'on nonlinear mode *******'
                     write(LUNIT,*) ' '
                  ENDIF
                  GO TO 982
               END IF
            END IF
         END IF
      END IF

c      IF(DELTA_SQRT.LT.0.01D0) THEN
c         WRITE(6,*) '**** Convergent on root mean squere ****'
c         GO TO 982
c      END IF

      goto 981         !  New iteration

 982  continue

      CALL QPB_TERMINATE (DATA_STORE, CONTROL, INFO)
      DEALLOCATE( PRT%G, PRT%X_L, PRT%X_U, PRT%X, PRT%Z, PRT%A%val,   
     $     PRT%H%val, PRT%C, PRT%C_L, PRT%C_U, PRT%Y)

      AMP_PAR(1:N_PAR) = UNPACK(RESULT(1:N_GAS), INDEX, 0.0D0)
      AMP_RIN(1:N_RIN) = RESULT(N_GAS + 1:N_FIT)

c      write(6,*) '*************************************************'
c      write(6,*) "* Results are written into the file"// 
c     $     "'retr_gome-0"//NAM_WIN_1//"'"
c      write(6,*) '*************************************************'
c      WRITE(6,*) ' '
c      
      IF (LOG_LEVEL(2) >= N_ESO) THEN 
         OUT_ERD(:) = ERD_S(1 + LOST_POINT:N_CEN - LOST_POINT)
         IF(POL_MODE) THEN
            Y_SOL = MATMUL(MATRIX,Y_SOL)
            Y_RAD = MATMUL(MATRIX,Y_RAD)
            OUT_ERD = MATMUL(MATRIX,OUT_ERD)
         END IF
         OPEN(1,file=DIR_OUT//'retr_spec-0'//NAM_WIN_1//TRIM(NAM_TH))
c        write(1,'(1x,3(1x,i4),3x,l1)') 
c     $     NUR, N_ALL, N_RIN, POL_MODE
         DO J = 1,NUR
            write(1,'(1x,f10.4,30(1x,e12.4))')
     $           WL(J + LOST_POINT), OUT_ERD(J) - Y_SOL(J), Y_RAD(J), 
     $           (WF_PAR(J,K)*RESULT(K), K=1, N_FIT) 
         END DO
         CLOSE(1)
      END IF
      
      IF (LOG_LEVEL(2) < N_SSO) RETURN 

c      WRITE(6,*) '       **** Retrieval of experimental data **** '
      write(LUNIT,*) 
     $     '        Amplitude for all parameters :'
      WRITE(LUNIT,'(8x,30(1x,e12.4))') (RESULT(I), I = 1,N_GAS)
      WRITE(LUNIT,*) ' '
      
      IF(N_RIN > 0) write(6,*) 
     $     '        Amplitude shift and squeeze for Ring spectrum :'
      DO I = 1, N_RIN
         IP = I + 2 
         WRITE(LUNIT,'(8x,3(1x,e12.4))') RESULT(N_GAS + I), 
     $        SHIFTS(1,IP), SHIFTS(2,IP)
         WRITE(LUNIT,*) ' '
      END DO
      
      write(LUNIT,*) '        Shift for model spectrum :'
      write(LUNIT,3002) SHIFTS(1,1), SHIFTS(2,1)
      write(LUNIT,*) ' '
      
      write(LUNIT,*) '        Shift for solar spectrum :'
      write(LUNIT,3002) SHIFTS(1,2), SHIFTS(2,2) 
      write(LUNIT,*) ' '
      
      WRITE(LUNIT,'(9x,A,1x,e11.5)') 
     $     'Root mean square for this mode:', SQ_RTB
      
      write(LUNIT,*) '        Total iterations number:',ITERATION

 3002 FORMAT(8x,' between:',1x,e12.4,'   and ',1x,e12.4)

      RETURN

      CONTAINS
C     ******************************************************************
      SUBROUTINE FIRST_GUESS
    
      INTEGER              L_P, I, J
      DOUBLE PRECISION     TEK1(NUR), Y1(NUR)

C     Input data preparation
C     **********************
      L_P = LOST_POINT
      TEK1(:) = ERD_S(1 + L_P:N_CEN - L_P) - SOL_S(1 + L_P:N_CEN - L_P) 
     $     - RAD(1 + L_P:N_CEN - L_P)
      WF_PAR(1:NUR,:) = WF_ALL(1 + L_P:N_CEN - L_P,:)
         
      IF(POL_MODE) THEN
         IF (LOG_LEVEL(2) >= N_SSO) THEN 
            WRITE(LUNIT,*) 
     $           ' * Subtraction of a polynomial from all parameters'
         ENDIF
         TEK1 = MATMUL(MATRIX,TEK1)
         WF_PAR = MATMUL(MATRIX,WF_PAR)
      END IF

      IF (LOG_LEVEL(2) >= N_DSO) THEN 
         WRITE(6,*) ' Retrieval without shift and squeeze'
         OPEN(C_UNIT,file = DIR_OUT//'test.par-0-ss')
         DO J = 1,NUR           
            WRITE(1,'(12(e12.4))') WL_NEW(J), TEK1(J), 
     $           (WF_PAR(J,I), I= 1, N_FIT)
         END DO
         CLOSE(C_UNIT)
      ENDIF

C     Initialization of the quadratic problem solver
C     **********************************************
      ALLOCATE( PRT%G(N_FIT), PRT%X_L(N_FIT), PRT%X_U(N_FIT), 
     $     PRT%X(N_FIT), PRT%Z(N_FIT), PRT%H%val(N_FIT*(N_FIT + 1)/2),
     $     PRT%A%val(0), PRT%C(0), PRT%C_L(0), PRT%C_U(0), PRT%Y(0))

      PRT%N = N_FIT ; PRT%M = 0 ; PRT%F = 0.0D0 

      PRT%X_L(:) = GR_PAR(1,1:N_FIT)
      PRT%X_U(:) = GR_PAR(2,1:N_FIT)

      DO I = 1, N_FIT 
         DO J = 1, I 
            PRT%H%val(I*(I - 1)/2 + J) = 
     $           DOT_PRODUCT(WF_PAR(:,I),WF_PAR(:,J))
         END DO
      END DO
      PRT%G(:) = - MATMUL(TEK1(1:NUR),WF_PAR(1:NUR,1:N_FIT))

      PRT%X = 0.0D0 ;  PRT%Z = 0.0D0
      CALL QPB_INITIALIZE (DATA_STORE, CONTROL) 
      CALL QPB_SOLVE (PRT, DATA_STORE, CONTROL, INFO) 
      CALL QPB_TERMINATE (DATA_STORE, CONTROL, INFO)

      RESULT(1:N_FIT) = PRT%X
      IF (INFO%status.NE.0) WRITE(LUNIT,*) 'Quadratic problem solver '//
     $     'without linear constraints --> INFO%status = ', INFO%status
      
      Y1(:) = TEK1(:) - MATMUL(WF_PAR(:,:),RESULT(:))
      SQ_RTB = DSQRT(DOT_PRODUCT(Y1(:),Y1(:))/DBLE(NUR))
      S = DSQRT(DOT_PRODUCT(TEK1(:),TEK1(:))/DBLE(NUR))

      IF (LOG_LEVEL(2) >= N_SSO) THEN 
         write(LUNIT,*) ' '
         write(LUNIT,*) 
     $        '    Without shift correction with QP algorithm'
         write(LUNIT,'(3x,11(1x,e12.4))') (RESULT(I), I = 1,N_FIT)
         write(LUNIT,*) ' '
         write(LUNIT,*) 
     $        '    RMS before and after first inversion'
         write(LUNIT,'(9x,2(2x,e13.5))') S,SQ_RTB
         write(LUNIT,*) ' '
      ENDIF

      DEALLOCATE( PRT%G, PRT%X_L, PRT%X_U, PRT%X, PRT%Z, PRT%H%val,
     $     PRT%A%val, PRT%C, PRT%C_L, PRT%C_U, PRT%Y)

      RETURN
      END SUBROUTINE
C**********************************************************************
      SUBROUTINE QPB_ITERATE

      INTEGER            L_P, N_LOW, N_UPP, LL, IP, I, J, K, KK
      LOGICAL            IDX_ALL(N_FIT + 2)

      DOUBLE PRECISION   B_MAT(NUR,N_FIT + 2*N_RIN + 4)
      DOUBLE PRECISION   F_SS(N_CEN), C_NORM_PAR(N_FIT)
      DOUBLE PRECISION   C_NORM_SHI(N_RIN + 2), C_NORM_SQ(N_RIN + 2)
      DOUBLE PRECISION   WF_SHI(NUR,N_RIN + 2), WF_SHI_ALL(NUR,N_FIT +2)
      DOUBLE PRECISION   WF_SQ(NUR,N_RIN + 2), WF_SQ_ALL(NUR,N_FIT +2)
      DOUBLE PRECISION   A_GR(NCLIN,N_PARL), TEK1(NUR), Y1(NUR)

      DOUBLE PRECISION, PARAMETER :: ZNAK = 1.0D0

      L_P = LOST_POINT
      IF (LOG_LEVEL(2) >= N_ESO) WRITE(LUNIT,*) 
     $     '******** INTERPOLATION WITH SPLINE ********* '

C     Apply shift and squeeze correction from the previous iteration
C     Model spectrum and parameter weighting functions:
      F_SS(:) = SHI_SQU_PAR(1,1) + SHI_SQU_PAR(2,1)*WL(:)
      SHIFTS(1,1) = F_SS(1 + L_P) ;  SHIFTS(2,1) = F_SS(N_CEN - L_P)
      CALL SHIFT_1 (WL, RAD, N_CEN, F_SS, Y_RAD, L_P, ZNAK)
      DO I = 1, N_GAS 
         CALL SHIFT_1(WL, WF_ALL(:,I), N_CEN, F_SS, WF_PAR(:,I), 
     $         L_P, ZNAK)
      END DO
C     Solar spectrum:
      F_SS(:) = SHI_SQU_PAR(1,2) + SHI_SQU_PAR(2,2)*WL(:)
      SHIFTS(1,2) = F_SS(1 + L_P) ;  SHIFTS(2,2) = F_SS(N_CEN - L_P)
      CALL SHIFT_1 (WL, SOL_S, N_CEN, F_SS, Y_SOL, L_P, ZNAK)
C     Spectral corrections:
      DO I = 1, N_RIN
         IP = N_GAS + I 
         F_SS(:) = SHI_SQU_PAR(1,I + 2) + SHI_SQU_PAR(2,I + 2)*WL(:)
         SHIFTS(1,I + 2) = F_SS(1 + L_P) 
         SHIFTS(2,I + 2) = F_SS(N_CEN - L_P)
         CALL SHIFT_1(WL, WF_ALL(:,IP), N_CEN, F_SS, WF_PAR(:,IP), L_P, 
     $        ZNAK)
      END DO

      TEK1(:) = ERD_S(1 + L_P:N_CEN - L_P) - Y_SOL(:) - Y_RAD(:)

C     Calculate weighting functions for shift
C     Model spectrum:
      CALL DIFFER_EXP(WL_NEW, NUR, Y_RAD, WF_SHI_ALL(:,1))    
C     Solar spectrum:
      CALL DIFFER_EXP(WL_NEW, NUR, Y_SOL, WF_SHI_ALL(:,2)) 
C     Weighting functions and spectral corrections:
      DO I = 1, N_FIT 
         CALL DIFFER_EXP(WL_NEW, NUR, WF_PAR(:,I), WF_SHI_ALL(:,I + 2))
      END DO

      IF(POL_MODE) THEN
c         IF(ITERATION .EQ.1) THEN
c            IF (LOG_LEVEL(2) >= N_SSO) THEN 
c               write(LUNIT,*) ' **** Subtraction of a polynomial '//
c     $              'from all parameters'
c            ENDIF
c         END IF
         TEK1 = MATMUL(MATRIX,TEK1)
         WF_PAR = MATMUL(MATRIX,WF_PAR)
         WF_SHI_ALL = MATMUL(MATRIX,WF_SHI_ALL)
      END IF

c    Use RESULT as mean value for shift and squeeze weighting function
      FORALL(I = 1:N_FIT) 
     $     WF_SHI_ALL(:,I + 2) = WF_SHI_ALL(:,I + 2)*RESULT(I)
C     Calculate weighting functions for squeeze
      FORALL (I = 1:N_FIT + 2) 
     $     WF_SQ_ALL(:,I) = - WF_SHI_ALL(:,I)*WL_NEW(:)

      TEK1(:) = TEK1(:) - MATMUL(WF_PAR,RESULT)

c     RMS before retrieval
      SQ_RTA = DSQRT(DOT_PRODUCT(TEK1,TEK1)/DBLE(NUR))
      
      IDX_ALL = UNPACK(IDX_MOD_SS,IDX_EXT,.TRUE.)
      FORALL(I = 1:NUR) Y1(I) = SUM(WF_SHI_ALL(I,:),IDX_ALL)
      WF_SHI_ALL(:,1) = Y1(:)
      IF(SQU_FLAG(3)) THEN ! If squeeze for model spectrum is required
         FORALL(I = 1:NUR) Y1(I) = SUM(WF_SQ_ALL(I,:),IDX_ALL)
         WF_SQ_ALL(:,1) = Y1(:)
      END IF

      IDX_ALL = UNPACK(IDX_SOL_SS,IDX_EXT,.FALSE.)
      FORALL(I = 1:NUR) Y1(I) = SUM(WF_SHI_ALL(I,:),IDX_ALL)
      WF_SHI_ALL(:,2) = Y1(:)
      IF(SQU_FLAG(2)) THEN ! If squeeze for model spectrum is required
         FORALL(I = 1:NUR) Y1(I) = SUM(WF_SQ_ALL(I,:),IDX_ALL)
         WF_SQ_ALL(:,2) = Y1(:)
      END IF

C     Pack weighting functions for shift and squeeze
      IDX_ALL = UNPACK(IDX_SHI,IDX_EXT,.FALSE.)
      FORALL (J = 1:NUR) 
     $     WF_SHI(J,1:N_SHI) = PACK(WF_SHI_ALL(J,:),IDX_ALL)
      IDX_ALL = UNPACK(IDX_SQ,IDX_EXT,.FALSE.)
      FORALL (J = 1:NUR) 
     $     WF_SQ(J,1:N_SQ) = PACK(WF_SQ_ALL(J,:),IDX_ALL)

C     Normalizing coefficients
      C_NORM_PAR(:) = MAXVAL(DABS(WF_PAR(1:NUR,:)),1)
      C_NORM_SHI(1:N_SHI) = MAXVAL(DABS(WF_SHI(1:NUR,1:N_SHI)),1)
      C_NORM_SQ(1:N_SQ) = MAXVAL(DABS(WF_SQ(1:NUR,1:N_SQ)),1)

C     Weighting function matrix formation
C     ***********************************
      FORALL (J = 1:NUR) 
         B_MAT(J,1:N_FIT) = WF_PAR(J,:)/C_NORM_PAR
         B_MAT(J,N_FIT + 1:N_FIT + N_SHI) = 
     $        WF_SHI(J,1:N_SHI)/C_NORM_SHI(1:N_SHI)
         B_MAT(J,N_FIT + N_SHI + 1:N_PARL) = 
     $        WF_SQ(J,1:N_SQ)/C_NORM_SQ(1:N_SQ)
      END FORALL      

C     Initialization of the quadratic problem solver
C     **********************************************
      PRT%N = N_PARL ; PRT%M = NCLIN ; PRT%F = 0.0D0 

C     Bounds for parameters
C     *********************
      PRT%X_L(:) = -INFINITY 
      PRT%X_U(:) =  INFINITY
      DO I = 1, N_FIT
         IF(DABS(GR_PAR(1,I)).LT.INFINITY) 
     $        PRT%X_L(I) = (-RESULT(I) + GR_PAR(1,I))*C_NORM_PAR(I)
         IF(DABS(GR_PAR(2,I)).LT.INFINITY) 
     $        PRT%X_U(I) = (-RESULT(I) + GR_PAR(2,I))*C_NORM_PAR(I)
      END DO

C     Bounds for linear constrains
C     ****************************
      N_LOW = 1 + L_P ;  N_UPP = N_CEN - L_P
C     Left spectral point
      PRT%C_L(1:N_SHI) = PACK(WIDE_SHI(1,:) - SHIFTS(1,:),IDX_SHI)
      PRT%C_U(1:N_SHI) = PACK(WIDE_SHI(2,:) - SHIFTS(1,:),IDX_SHI)
C     Right spectral point 
      PRT%C_L(N_SHI + 1:2*N_SHI) = 
     $     PACK(WIDE_SHI(1,:) - SHIFTS(2,:),IDX_SHI)
      PRT%C_U(N_SHI + 1:2*N_SHI) = 
     $     PACK(WIDE_SHI(2,:) - SHIFTS(2,:),IDX_SHI)

C     Linear constrains
C     *****************
      A_GR = 0.0D0 
      KK = 0 ; LL =0
      DO K = 1, N_RIN + 2
         IF(.NOT.IDX_SHI(K)) CYCLE
         KK = KK + 1
         A_GR(KK,N_FIT + KK) = 1.0D0/C_NORM_SHI(KK)
         A_GR(KK + N_SHI,N_FIT + KK) = 1.0D0/C_NORM_SHI(KK)
         IF(.NOT.IDX_SQ(K)) CYCLE
         LL = LL + 1
         A_GR(KK,N_FIT + N_SHI + LL) = - WL(N_LOW)/C_NORM_SQ(LL)
         A_GR(KK + N_SHI,N_FIT + N_SHI + LL) = - WL(N_UPP)/C_NORM_SQ(LL)
      END DO

      PRT%X = 0.0D0 ;  PRT%Y = 0.0D0 ;  PRT%Z = 0.0D0
      DO I = 1, N_PARL 
         DO J = 1, I 
            PRT%H%val(I*(I - 1)/2 + J) = 
     $           DOT_PRODUCT(B_MAT(:,I),B_MAT(:,J))
         END DO
      END DO
      PRT%G(:) = - MATMUL(TEK1(1:NUR),B_MAT(1:NUR,1:N_PARL))
      FORALL(I = 1:NCLIN,J=1:N_PARL) 
     $     PRT%A%val(N_PARL*(I - 1) + J) = A_GR(I,J) 

      CALL QPB_SOLVE (PRT, DATA_STORE, CONTROL, INFO) 

      IF (INFO%status.NE.0) WRITE(LUNIT,*) 'Quadratic problem solver '//
     $     'with linear constraints --> INFO%status = ', INFO%status
      
      RESULT(1:N_FIT) = 
     $     RESULT(1:N_FIT) + PRT%X(1:N_FIT)/C_NORM_PAR(1:N_FIT)
      SHI_SQU_PAR(1,1:N_RIN + 2) = SHI_SQU_PAR(1,1:N_RIN + 2) +
     $     UNPACK(PRT%X(N_FIT + 1:N_FIT + N_SHI)/C_NORM_SHI(1:N_SHI),
     $     IDX_SHI, 0.0D0) 
      SHI_SQU_PAR(2,1:N_RIN + 2) = SHI_SQU_PAR(2,1:N_RIN + 2) -
     $     UNPACK(PRT%X(N_FIT + N_SHI + 1:N_PARL)/C_NORM_SQ(1:N_SQ),
     $     IDX_SQ, 0.0D0) 

      WHERE(IDX_MOD_SS(3:N_RIN + 2)) 
         SHI_SQU_PAR(1,3:N_RIN + 2) = SHI_SQU_PAR(1,1) 
         SHI_SQU_PAR(2,3:N_RIN + 2) = SHI_SQU_PAR(2,1) 
      END WHERE

      WHERE(IDX_SOL_SS(3:N_RIN + 2)) 
         SHI_SQU_PAR(1,3:N_RIN + 2) = SHI_SQU_PAR(1,2) 
         SHI_SQU_PAR(2,3:N_RIN + 2) = SHI_SQU_PAR(2,2) 
      END WHERE

      ER_SCA(:) = TEK1(:) - MATMUL(B_MAT(:,1:N_PARL),PRT%X(1:N_PARL))
      SQ_RTB = DSQRT(DOT_PRODUCT(ER_SCA(:),ER_SCA(:))/DBLE(NUR))

      RETURN
      END SUBROUTINE
C**********************************************************************

      END

C**********************************************************************
      SUBROUTINE DIFFER_EXP(A_WL, N_WL, X, Y)

      INTEGER           N_WL, I, LC_K, LWR_K, J, IFAIL
      INTEGER           N_COEFF, KK, INBV
      DOUBLE PRECISION  C(N_WL + 4), WRK(6*N_WL+16), LAMDA(N_WL + 4)
      DOUBLE PRECISION, DIMENSION (N_WL) :: A_WL, X, Y, B_WL, Y1, B_SC, 
     $     C_SC, D_SC
      DOUBLE PRECISION  B_COEFF(N_WL + 2), S_NODES(N_WL + 6)
      DOUBLE PRECISION  WORK(5*(N_WL + 2))
      DOUBLE PRECISION, EXTERNAL :: SEVAL, DBVALU  
      DOUBLE PRECISION  D_DIF, DUMMI, FIT

c      D_DIF = 1.0D-05
      D_DIF = 1.0D-03
      DO I = 1,N_WL
         B_WL(I) = A_WL(I) + D_DIF
      END DO

      LC_K = N_WL+4 
      LWR_K = 6*N_WL+16
c      CALL E01BAF(N_WL, A_WL, X, LAMDA, C, LC_K, WRK, LWR_K, IFAIL)         
c      CALL SPLINE (N_WL, A_WL, X, B_SC, C_SC, D_SC)
      CALL DBINT4 (A_WL, X, N_WL, 2, 2, 0.0, 0.0, 1, S_NODES,
     $     B_COEFF, N_COEFF, KK, WORK) 
      INBV = 1
      DO I = 1,N_WL
         DUMMI = B_WL(I)
         IF(DUMMI.LT.A_WL(1)) CYCLE
         IF(DUMMI.GT.A_WL(N_WL)) EXIT
c         CALL E02BBF(LC_K, LAMDA, C, DUMMI, FIT, IFAIL)
c         Y(I) = FIT
c         Y(I) = SEVAL(N_WL, B_WL(I), A_WL, X, B_SC, C_SC, D_SC)
         Y(I) = DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0, B_WL(I), INBV, 
     $        WORK(1:12))
      END DO

      Y(1) = (Y(1) - X(1))/D_DIF  
      DO I = 1,N_WL
         B_WL(I) = A_WL(I) - D_DIF
      END DO

      INBV = 1
      DO I = 1,N_WL
         DUMMI = B_WL(I)
         IF(DUMMI.LT.A_WL(1)) CYCLE
         IF(DUMMI.GT.A_WL(N_WL)) EXIT
c         CALL E02BBF(LC_K, LAMDA, C, DUMMI, FIT, IFAIL)
c         Y1(I) = FIT
c         Y1(I) = SEVAL(N_WL, B_WL(I), A_WL, X, B_SC, C_SC, D_SC)
         Y1(I) = DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0, B_WL(I),  
     $        INBV, WORK(1:12))
      END DO

      Y(N_WL) = - (Y1(N_WL) - X(N_WL))/D_DIF     

      DO J = 2,N_WL - 1
         Y(J) = (Y(J) - Y1(J))/(2.0d0*D_DIF) 
      END DO
      RETURN
      END

C     ******************************************************************

      SUBROUTINE GUESS_0L(N_CEN, L_P, N_FIT, WL, WL_NEW, NUR, SOL_S, 
     $     ERD_S, WF_ALL, RES, RAD, POL_MODE, GR_PAR, VDM, MATRIX)

      USE LOG_CONTROL, ONLY: DIR_OUT, LOG_LEVEL, N_SSO, N_DSO, LUNIT

      INTEGER            M_PAR, N_CEN, L_P, N_GAS, N_FIT, NUR, K, ITER
      INTEGER            I, J, N2, IP, N_PARL, L, NCLIN, N_POS, M_WORK
      INTEGER            NCTOTL, LIWORK, IFAIL, IPI, VDM
      INTEGER            N_COD(N_FIT), IWORK(2*N_FIT)
      INTEGER, PARAMETER :: NROWA = 1

      LOGICAL            LP, ORTHOG, POL_MODE

      DOUBLE PRECISION   GR_PAR(2,N_FIT), A_GR(NROWA,N_FIT)
      DOUBLE PRECISION   MATRIX(VDM, VDM), WF_PAR(NUR,N_FIT)
      DOUBLE PRECISION   HESS(N_FIT,N_FIT), WF_ALL(N_CEN,N_FIT)
      DOUBLE PRECISION, DIMENSION (N_FIT) :: CVEC, ISTATE, CLAMDA, 
     $     C_NORM, BL, BU, RES, FEATOL
      DOUBLE PRECISION, DIMENSION (N_CEN) :: SOL_S, ERD_S, WL, RAD
      DOUBLE PRECISION, DIMENSION (NUR) :: Y1, TEK1, WL_NEW
      DOUBLE PRECISION, ALLOCATABLE :: WORK(:)

      DOUBLE PRECISION   S, SQ_RTB, BIGBND, OBJ

      EXTERNAL  QPHESS

      M_WORK = 2*N_FIT**2 + 4*N_FIT + NROWA
      ALLOCATE(WORK(M_WORK))
      N_PARL = N_FIT

      TEK1(1:NUR) = ERD_S(1 + L_P:N_CEN - L_P) - 
     $     SOL_S(1 + L_P:N_CEN - L_P) - RAD(1 + L_P:N_CEN - L_P)
      WF_PAR(1:NUR,:) = WF_ALL(1 + L_P:N_CEN - L_P,:)
         
      IF(POL_MODE) THEN
         IF (LOG_LEVEL(2) >= N_SSO) THEN 
               write(LUNIT,*) ' * Subtraction of a polynomial '//
     $              'from all parameters'
         ENDIF
         TEK1 = MATMUL(MATRIX,TEK1)
         WF_PAR = MATMUL(MATRIX,WF_PAR)
      END IF

      IF (LOG_LEVEL(2) >= N_DSO) THEN 
         WRITE(LUNIT,*) ' Retrieval without shift and squeeze'
         open(1,file = DIR_OUT//'test.par-0-ss')
         DO J = 1,NUR           
            WRITE(1,'(12(e12.4))') WL_NEW(J), TEK1(J), 
     $           (WF_PAR(J,I), I= 1, N_FIT)
         END DO
         close(1)
      ENDIF

      HESS(1:N_PARL,1:N_PARL) = MATMUL(TRANSPOSE(WF_PAR),WF_PAR)
      CVEC(1:N_PARL) = - MATMUL(TEK1(1:NUR),WF_PAR(1:NUR,1:N_PARL))

      BL(1:N_PARL) = GR_PAR(1,1:N_PARL)
      BU(1:N_PARL) = GR_PAR(2,1:N_PARL)
      FEATOL(1:N_PARL) = 1.D-07

      NCLIN = 0; NCTOTL = N_PARL + NCLIN
      LP = .FALSE.; ORTHOG = .FALSE.
      BIGBND = 1.D+10; LIWORK = 2*N_FIT; IFAIL = 0
      CALL E04NAF (30, 0, N_PARL, NCLIN, NCTOTL, NROWA, N_FIT, N_FIT,
     $     BIGBND, A_GR, BL, BU, CVEC, FEATOL, HESS, QPHESS, .TRUE., LP,
     $     ORTHOG, RES, ISTATE, ITER, OBJ, CLAMDA, IWORK, LIWORK, WORK, 
     $     M_WORK, IFAIL)

      Y1(1:NUR) = TEK1(1:NUR) - 
     $     MATMUL(WF_PAR(1:NUR,1:N_PARL),RES(1:N_PARL))
      SQ_RTB = DSQRT(DOT_PRODUCT(Y1(1:NUR),Y1(1:NUR))/DBLE(NUR))
      S = DSQRT(DOT_PRODUCT(TEK1(1:NUR),TEK1(1:NUR))/DBLE(NUR))

      IF (LOG_LEVEL(2) >= N_SSO) THEN 
         write(LUNIT,*) ' Without shift correction with QP algorithm'
         write(LUNIT,'(11(1x,e12.4))') (RES(I),I = 1,N_PARL)
         write(LUNIT,*) ' '
         write(LUNIT,*) 'RMS before and after first inversion'
         write(LUNIT,'(5x,2(2x,e13.5))') S,SQ_RTB
      ENDIF

      RETURN
      END

C     ****************************************************
      SUBROUTINE GUESS_IL (N_CEN, L_P, N_FIT, WL, WL_NEW, SOL_S, ERD_S, 
     $     WF_ALL, RESULT, SHI_SQU_PAR, NUR, SQ_RTA, SQ_RTB, ITERATION, 
     $     WIDE_SHI, N_GAS, N_RIN, POL_MODE, RAD, GR_PAR, ER_SCA, VDM, 
     $     MATRIX, SQU_FLAG, RING_MODE, Y_SOL, Y_RAD, WF_PAR)

      USE LOG_CONTROL, ONLY: LOG_LEVEL, N_SSO, N_ESO, LUNIT

      INTEGER            N_PARL, M_WORK, ITMAX, ITER, L_P, NROWA 
      INTEGER            N_CEN, N_LOW, N_UPP, LIWORK, NCTOTL, LL, IP
      INTEGER            N_GAS, NUR, NCLIN, IFAIL, ITERATION, K, KK
      INTEGER            N_FIT, VDM, N_LIN, N_RIN, I, J, N_SHI, N_SQ
      INTEGER, ALLOCATABLE :: IWORK(:)

      CHARACTER (LEN=1)  RING_MODE(N_RIN)

      LOGICAL            COLD, LP, ORTHOG, POL_MODE, SQU_FLAG(3)
      LOGICAL            IDX_MOD_SS(N_RIN + 2), IDX_SOL_SS(N_RIN + 2)
      LOGICAL            IDX_SHI(N_RIN + 2), IDX_SQ(N_RIN + 2)
      LOGICAL            IDX_EXT(N_FIT + 2), IDX_ALL(N_FIT + 2)

      DOUBLE PRECISION   F_SS(N_CEN), WIDE_SHI(2,N_RIN + 2)
      DOUBLE PRECISION   SHI_SQU_PAR(2,N_RIN + 2), WF_TMP(NUR,N_GAS)
      DOUBLE PRECISION   B_MAT(NUR,N_FIT + 2*N_RIN + 4)
      DOUBLE PRECISION   MATRIX(VDM, VDM), GR_PAR(2,N_FIT)
      DOUBLE PRECISION   RESULT(N_FIT), C_NORM_PAR(N_FIT)
      DOUBLE PRECISION   C_NORM_SHI(N_RIN + 2), C_NORM_SQ(N_RIN + 2)
      DOUBLE PRECISION   WF_ALL(N_CEN,N_FIT), WF_PAR(NUR,N_FIT)
      DOUBLE PRECISION   WF_SHI(NUR,N_RIN + 2), WF_SHI_ALL(NUR,N_FIT +2)
      DOUBLE PRECISION   WF_SQ(NUR,N_RIN + 2), WF_SQ_ALL(NUR,N_FIT +2)
      DOUBLE PRECISION   SHIFTS(2,N_RIN + 2)
      DOUBLE PRECISION, DIMENSION (N_CEN) :: WL, RAD, ERD_S, SOL_S
      DOUBLE PRECISION, DIMENSION (NUR) :: ER_SCA, Y_RAD, Y_SOL, WL_NEW, 
     $     TEK1, Y1
      DOUBLE PRECISION, ALLOCATABLE :: WORK(:), HESS(:,:), A_GR(:,:),
     $     CLAMDA(:), BL(:), BU(:), FEATOL(:), ISTATE(:), RES(:), 
     $     CVEC(:)

      DOUBLE PRECISION   BIGBND, SQ_RTA, SQ_RTB, ZNAK, OBJ

      EXTERNAL QPHESS

      ZNAK = 1.0D0
      IF (LOG_LEVEL(2) >= N_ESO)
     $     WRITE(6,*) '******** INTERPOLATION WITH SPLINE ********* '

C     Apply shift and squeeze correction from the previous iteration
C     Model spectrum and parameter weighting functions:
      F_SS(:) = SHI_SQU_PAR(1,1) + SHI_SQU_PAR(2,1)*WL(:)
      SHIFTS(1,1) = F_SS(1 + L_P) ;  SHIFTS(2,1) = F_SS(N_CEN - L_P)
      CALL SHIFT_1 (WL, RAD, N_CEN, F_SS, Y_RAD, L_P, ZNAK)
      DO I = 1, N_GAS 
         CALL SHIFT_1(WL, WF_ALL(:,I), N_CEN, F_SS, WF_PAR(:,I), 
     $         L_P, ZNAK)
      END DO
C     Solar spectrum:
      F_SS(:) = SHI_SQU_PAR(1,2) + SHI_SQU_PAR(2,2)*WL(:)
      SHIFTS(1,2) = F_SS(1 + L_P) ;  SHIFTS(2,2) = F_SS(N_CEN - L_P)
      CALL SHIFT_1 (WL, SOL_S, N_CEN, F_SS, Y_SOL, L_P, ZNAK)
C     Spectral corrections:
      DO I = 1, N_RIN
         IP = N_GAS + I 
         F_SS(:) = SHI_SQU_PAR(1,I + 2) + SHI_SQU_PAR(2,I + 2)*WL(:)
         SHIFTS(1,I + 2) = F_SS(1 + L_P) 
         SHIFTS(2,I + 2) = F_SS(N_CEN - L_P)
         CALL SHIFT_1(WL, WF_ALL(:,IP), N_CEN, F_SS, WF_PAR(:,IP), L_P, 
     $        ZNAK)
      END DO

      TEK1(:) = ERD_S(1 + L_P:N_CEN - L_P) - Y_SOL(:) - Y_RAD(:)

C     Calculate weighting functions for shift
C     Model spectrum:
      CALL DIFFER_EXP(WL_NEW, NUR, Y_RAD, WF_SHI_ALL(:,1))    
C     Solar spectrum:
      CALL DIFFER_EXP(WL_NEW, NUR, Y_SOL, WF_SHI_ALL(:,2)) 
C     Weighting functions and spectral corrections:
      DO I = 1, N_FIT 
         CALL DIFFER_EXP(WL_NEW, NUR, WF_PAR(:,I), WF_SHI_ALL(:,I + 2))
      END DO

      IF(POL_MODE) THEN
c         IF(ITERATION .EQ.1) THEN
c            IF (LOG_LEVEL(2) >= N_SSO) THEN 
c               write(LUNIT,*) '**** Subtraction of a polynomial '//
c     $              'from all parameters'
c            ENDIF
c         END IF
         TEK1 = MATMUL(MATRIX,TEK1)
         WF_PAR = MATMUL(MATRIX,WF_PAR)
         WF_SHI_ALL = MATMUL(MATRIX,WF_SHI_ALL)
      END IF

c    Use RESULT as mean value for shift and squeeze weighting function
      FORALL(I = 1:N_FIT) 
     $     WF_SHI_ALL(:,I + 2) = WF_SHI_ALL(:,I + 2)*RESULT(I)
C     Calculate weighting functions for squeeze
      FORALL (I = 1:N_FIT + 2) 
     $     WF_SQ_ALL(:,I) = - WF_SHI_ALL(:,I)*WL_NEW(:)

      TEK1(:) = TEK1(:) - MATMUL(WF_PAR,RESULT)

c     RMS before retrieval
      SQ_RTA = DSQRT(DOT_PRODUCT(TEK1,TEK1)/DBLE(NUR))
      
C     Setting of shift and squeeze flags 
      IDX_SHI(1:2)  = .TRUE. ! Model and solar spectra
      IDX_SHI(3:N_RIN + 2)  = .FALSE.
C     Correction spectra:
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'I') IDX_SHI(3:N_RIN + 2) = .TRUE. 
      IDX_EXT(:) = .TRUE. 
      IDX_EXT(3:N_GAS + 2) = .FALSE.

      IDX_SQ = IDX_SHI
      IF(.NOT.SQU_FLAG(3)) IDX_SQ(1) = .FALSE. ! Model spectrum 
      IF(.NOT.SQU_FLAG(2)) IDX_SQ(2) = .FALSE. ! Solar spectrum
C     Correction spectra:
      IF(.NOT.SQU_FLAG(1)) IDX_SQ(3:N_RIN + 2) = .FALSE.

      N_SHI = COUNT(IDX_SHI)
      N_SQ = COUNT(IDX_SQ)

C     Common shift and squeeze for model spectrum, weighting functions,  
C     and selected correction spectra.
      IDX_MOD_SS = .FALSE.
C     Model spectrum
      IDX_MOD_SS(1) = .TRUE.                
C     Correction spectra
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'M') IDX_MOD_SS(3:N_RIN + 2) = .TRUE. 

      IDX_ALL = UNPACK(IDX_MOD_SS,IDX_EXT,.TRUE.)
      FORALL(I = 1:NUR) Y1(I) = SUM(WF_SHI_ALL(I,:),IDX_ALL)
      WF_SHI_ALL(:,1) = Y1(:)
      IF(SQU_FLAG(3)) THEN ! If squeeze for model spectrum is required
         FORALL(I = 1:NUR) Y1(I) = SUM(WF_SQ_ALL(I,:),IDX_ALL)
         WF_SQ_ALL(:,1) = Y1(:)
      END IF

C     Common shift and squeeze for solar spectrum, and selected 
C     correction spectra.
      IDX_SOL_SS = .FALSE.
C     Solar spectrum
      IDX_SOL_SS(2) = .TRUE. 
C     Correction spectra
      IF( N_RIN > 0 )
     $     WHERE(RING_MODE(:) == 'S') IDX_SOL_SS(3:N_RIN + 2) = .TRUE. 

      IDX_ALL = UNPACK(IDX_SOL_SS,IDX_EXT,.FALSE.)
      FORALL(I = 1:NUR) Y1(I) = SUM(WF_SHI_ALL(I,:),IDX_ALL)
      WF_SHI_ALL(:,2) = Y1(:)
      IF(SQU_FLAG(2)) THEN ! If squeeze for model spectrum is required
         FORALL(I = 1:NUR) Y1(I) = SUM(WF_SQ_ALL(I,:),IDX_ALL)
         WF_SQ_ALL(:,2) = Y1(:)
      END IF

C     Pack weighting functions for shift and squeeze
      IDX_ALL = UNPACK(IDX_SHI,IDX_EXT,.FALSE.)
      FORALL (J = 1:NUR) 
     $     WF_SHI(J,1:N_SHI) = PACK(WF_SHI_ALL(J,:),IDX_ALL)
      IDX_ALL = UNPACK(IDX_SQ,IDX_EXT,.FALSE.)
      FORALL (J = 1:NUR) 
     $     WF_SQ(J,1:N_SQ) = PACK(WF_SQ_ALL(J,:),IDX_ALL)

C     Normalizing coefficients
      C_NORM_PAR(:) = MAXVAL(DABS(WF_PAR(1:NUR,:)),1)
      C_NORM_SHI(1:N_SHI) = MAXVAL(DABS(WF_SHI(1:NUR,1:N_SHI)),1)
      C_NORM_SQ(1:N_SQ) = MAXVAL(DABS(WF_SQ(1:NUR,1:N_SQ)),1)

      N_PARL = N_FIT + N_SHI + N_SQ 
      N_LIN = N_SHI ;  NCLIN = 2*N_LIN
      NCTOTL = N_PARL + NCLIN

      NROWA = NCLIN; LIWORK = 2*N_PARL
      M_WORK = 2*N_PARL**2 + 6*N_PARL + 4*NCLIN + NROWA
      ALLOCATE(WORK(M_WORK), IWORK(LIWORK), A_GR(NROWA,N_PARL), 
     $     HESS(N_PARL,N_PARL), CLAMDA(NCTOTL), BL(NCTOTL), BU(NCTOTL), 
     $     FEATOL(NCTOTL), ISTATE(NCTOTL), RES(N_PARL), CVEC(N_PARL))

      BIGBND = 1.D+10
      BL = 0.0D0; BU = 0.0D0; FEATOL = 0.0D0

C     Weighting function matrix formation
C     ***********************************
      FORALL (J = 1:NUR) 
         B_MAT(J,1:N_FIT) = WF_PAR(J,:)/C_NORM_PAR
         B_MAT(J,N_FIT + 1:N_FIT + N_SHI) = 
     $        WF_SHI(J,1:N_SHI)/C_NORM_SHI(1:N_SHI)
         B_MAT(J,N_FIT + N_SHI + 1:N_PARL) = 
     $        WF_SQ(J,1:N_SQ)/C_NORM_SQ(1:N_SQ)
      END FORALL      

C     Bounds  initialization for QP_NAG 
C     *********************************
      FEATOL(1:N_FIT) =  1.D-05 
      FEATOL(N_FIT + 1:N_PARL) =  1.D-03 
      FEATOL(N_PARL + 1:N_PARL + NCLIN) = 1.D-06

C     Bounds for parameters
C     *********************
      BL(1:N_PARL) = -1.0D+20  
      BU(1:N_PARL) =  1.0D+20 
      DO I = 1, N_FIT
         IF(DABS(GR_PAR(1,I)).LT.BIGBND) 
     $        BL(I) = (-RESULT(I) + GR_PAR(1,I))*C_NORM_PAR(I)
         IF(DABS(GR_PAR(2,I)).LT.BIGBND) 
     $        BU(I) = (-RESULT(I) + GR_PAR(2,I))*C_NORM_PAR(I)
      END DO

C     Bounds for linear constrains
C     ****************************
      N_LOW = 1 + L_P ;  N_UPP = N_CEN - L_P
C     Left spectral point
      BL(N_PARL + 1:N_PARL + N_SHI) =
     $     PACK(WIDE_SHI(1,:) - SHIFTS(1,:),IDX_SHI)
      BU(N_PARL + 1:N_PARL + N_SHI) = 
     $     PACK(WIDE_SHI(2,:) - SHIFTS(1,:),IDX_SHI)
C     Right spectral point 
      BL(N_PARL + N_SHI + 1:N_PARL + 2*N_SHI) =
     $     PACK(WIDE_SHI(1,:) - SHIFTS(2,:),IDX_SHI)
      BU(N_PARL + N_SHI + 1:N_PARL + 2*N_SHI) = 
     $     PACK(WIDE_SHI(2,:) - SHIFTS(2,:),IDX_SHI)

C     Linear constrains
C     *****************
      A_GR = 0.0D0 
      KK = 0 ; LL =0
      DO K = 1, N_RIN + 2
         IF(.NOT.IDX_SHI(K)) CYCLE
         KK = KK + 1
         A_GR(KK,N_FIT + KK) = 1.0D0/C_NORM_SHI(KK)
         A_GR(KK + N_SHI,N_FIT + KK) = 1.0D0/C_NORM_SHI(KK)
         IF(.NOT.IDX_SQ(K)) CYCLE
         LL = LL + 1
         A_GR(KK,N_FIT + N_SHI + LL) = - WL(N_LOW)/C_NORM_SQ(LL)
         A_GR(KK + N_SHI,N_FIT + N_SHI + LL) = - WL(N_UPP)/C_NORM_SQ(LL)
      END DO

      HESS(1:N_PARL,1:N_PARL) = 
     $     MATMUL(TRANSPOSE(B_MAT(:,1:N_PARL)),B_MAT(:,1:N_PARL)) 
      CVEC(1:N_PARL) = - MATMUL(TEK1,B_MAT(:,1:N_PARL))

      COLD   = .TRUE. ;  LP = .FALSE. ;  ORTHOG = .TRUE. !  .FALSE.
      IFAIL = - 1;  ISTATE = 0;  ITMAX = 50 

      RES(:) = 0.0D0
      CALL E04NAF (ITMAX, 0, N_PARL, NCLIN, NCTOTL, NROWA, N_PARL,
     $     N_PARL, BIGBND, A_GR, BL, BU, CVEC, FEATOL, HESS, QPHESS, 
     $     COLD, LP, ORTHOG, RES, ISTATE, ITER, OBJ, CLAMDA, IWORK, 
     $     LIWORK, WORK, M_WORK, IFAIL)
      
      RESULT(1:N_FIT) = 
     $     RESULT(1:N_FIT) + RES(1:N_FIT)/C_NORM_PAR(1:N_FIT)
      SHI_SQU_PAR(1,1:N_RIN + 2) = SHI_SQU_PAR(1,1:N_RIN + 2) +
     $     UNPACK(RES(N_FIT + 1:N_FIT + N_SHI)/C_NORM_SHI(1:N_SHI),
     $     IDX_SHI, 0.0D0) 
      SHI_SQU_PAR(2,1:N_RIN + 2) = SHI_SQU_PAR(2,1:N_RIN + 2) -
     $     UNPACK(RES(N_FIT + N_SHI + 1:N_PARL)/C_NORM_SQ(1:N_SQ),
     $     IDX_SQ, 0.0D0) 

      WHERE(IDX_MOD_SS(3:N_RIN + 2)) 
         SHI_SQU_PAR(1,3:N_RIN + 2) = SHI_SQU_PAR(1,1) 
         SHI_SQU_PAR(2,3:N_RIN + 2) = SHI_SQU_PAR(2,1) 
      END WHERE

      WHERE(IDX_SOL_SS(3:N_RIN + 2)) 
         SHI_SQU_PAR(1,3:N_RIN + 2) = SHI_SQU_PAR(1,2) 
         SHI_SQU_PAR(2,3:N_RIN + 2) = SHI_SQU_PAR(2,2) 
      END WHERE

      ER_SCA(:) = TEK1(:) - MATMUL(B_MAT(:,1:N_PARL),RES(1:N_PARL))
      SQ_RTB = DSQRT(DOT_PRODUCT(ER_SCA(:),ER_SCA(:))/DBLE(NUR))

      RETURN
      END

C     ************************************************************

      SUBROUTINE CHECK_CONVERG(N_GAS, N_RIN, RESULT_OLD, RESULT, SQ_RTA, 
     $     SQ_RTB, SHIFTS, LGA, LRI, LSH, ITERATION, N_CEN, WL, SHI_MAX)

      USE LOG_CONTROL, ONLY: LOG_LEVEL, N_SSO, LUNIT

      INTEGER           M_PAR, N_GAS, N_RIN, ITERATION, NKO1, N_U, I
      INTEGER           IMAXG, N_L, IMAXR, NKO2, NKO3, N_CEN
      INTEGER, PARAMETER :: MAX_ITER = 500
      DOUBLE PRECISION  RESULT_OLD(N_GAS + N_RIN), RESULT(N_GAS + N_RIN)
      DOUBLE PRECISION  Y(N_GAS + N_RIN), WL(N_CEN)
      DOUBLE PRECISION  F_SS(N_CEN, N_RIN + 2), SHIFTS(2,N_RIN + 2)
      DOUBLE PRECISION  SQ_RTA, SQ_RTB, DIV_MAXG, SHI_MAX, SQI_MAX
      DOUBLE PRECISION  DELTA_SQRT, DIV_MAXR, SHI_MAX_NEW
      LOGICAL           LGA, LRI, LSH

      NKO1 = 0 ; Y = 0.0d0
      DO I = 1,N_GAS
         IF( RESULT(I) > 0.0d0 ) THEN
            NKO1 = NKO1 + 1
            Y(NKO1) = (RESULT_OLD(I)/RESULT(I) - 1.0D0)*100.
         ENDIF
      END DO
      DIV_MAXG = MAXVAL(DABS(Y(1:NKO1)))

      IF(N_RIN .NE. 0 ) THEN
         NKO1 = 0
         DO I = N_GAS + 1,N_GAS + N_RIN 
            NKO1 = NKO1 + 1
            IF( RESULT(I) > 0.0d0 ) THEN
               NKO1 = NKO1 + 1
               Y(NKO1) = (RESULT_OLD(I)/RESULT(I) - 1.0D0)*100.
            ENDIF
         END DO
         DIV_MAXR = MAXVAL(DABS(Y(1:NKO1)))
      ELSE
         DIV_MAXR = 0.0D0
      END IF

      SHI_MAX_NEW = MAXVAL(DABS(SHIFTS(:,:)))

c      WRITE(6,'(1x,6(1x,e10.2))') (Y(I),I=1,6)

c      write(6,*) '*** Iteration number      - ', ITERATION,'  ***'
c      write(6,*) 'Max diviation for slant   - ', IMAXG,DIV_MAXG
c      write(6,*) 'Max diviation for Ring    - ', IMAXR,DIV_MAXR
c      write(6,*) 'Max shift                 - ', IMAX2,SHI_MAX
c      write(6,*) 'Max squeze                - ', IMAX3,SQI_MAX
c      write(6,*) 'Msr for this iteration befor and after '

      DELTA_SQRT = (SQ_RTA-SQ_RTB)/SQ_RTA*100.

c      write(6,'(2(1x,e15.7),1x,f7.3)') SQ_RTA,SQ_RTB,DELTA_SQRT

      IF(DABS(DIV_MAXG).LT.0.01d0)  LGA = .TRUE.
      IF(DABS(DIV_MAXR).LT.0.1d0)  LRI = .TRUE.
      IF(DABS(SHI_MAX_NEW - SHI_MAX).LT.1.0d-04) LSH = .TRUE.

      SHI_MAX = SHI_MAX_NEW

      IF(ITERATION.GT.MAX_ITER) THEN
         IF (LOG_LEVEL(2) >= N_SSO) 
     $        write(LUNIT,*) '**** Iteration limit is reached ****'
         LGA = .TRUE. ;   LRI = .TRUE. ;   LSH = .TRUE.
      END IF
      RETURN
      END

C ******************************************************************

      SUBROUTINE QPHESS(N,NROWH,NCOLH,JTHCOL,HESS,X,HX)
C      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER           NCOLH, NROWH, N, JTHCOL
      DOUBLE PRECISION  HESS(NROWH,NCOLH),X(N),HX(N)
      HX(1:N) = MATMUL(HESS(1:N,1:N),X(1:N))
      RETURN
      END

