!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 21 Sep 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE SOL_GRID (*)
  ! Called by "rtm_init.f"

  USE GRIDS,      ONLY: NSOL_INT, SOL_GRD, XM_S_INIT, NSGP => NSOL, ZZZ_INP, &
                        Z_TOA, Z_BOA, X_INV
  USE LOG_PARS,   ONLY: LUNIT, LOG_LEVEL, N_SSO, N_ESO, ErrLst
  USE ST_VARS,    ONLY: EARTH_RADIUS, RAD_TO_DEG, Pie, RTM_MODE, RTM_TYPE, &
                        RTM_CORE, GROUND_HEIGHT, GeoD => GeoD_Out, SOL_ANG,&
                        RAD_FIELD, Interp_RF
!                        RTM_CORE, GROUND_HEIGHT, GeoD => GeoD_ITR, SOL_ANG
  USE SPHER_PARS, ONLY: N_SOL_ANG, X_SHAD, ONE_MS_SZ, SolGrid
  USE TYPES,      ONLY: KND
  
  CHARACTER (LEN = *), PARAMETER ::  R_NAME = 'SOL_GRID'
  INTEGER   J, I_S, JJ, IA, IV, IS, IP, Iadd
  Character (Len=:), Allocatable:: FStr
  Character (Len=2) Str
  REAL (KND)  DELTA_S, CSOL_ONE, CSOL, CSOL_MIN, CSOL_MAX, CSOL_TH_MIN, &
              CSOL_TH_MAX, DD
  REAL (KND), ALLOCATABLE :: ANG_S(:)
  Type (ErrLst) :: WrtErr
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  
  IF(ALLOCATED(SOL_GRD)) DeAllocate(SOL_GRD)
  Set_Out_Angs: Block
    IF( RTM_CORE == 'TRN' ) Exit Set_Out_Angs
    IF( RTM_CORE == 'CDI' ) Exit Set_Out_Angs
    IF( RTM_CORE == 'DOM'.AND.RTM_TYPE == 'spher_scat' ) Exit Set_Out_Angs
    ! For 'pp'scat' and 'ps_scat' in 'DOM' the grid angles are set
    ! to output solar angles
    NSGP = N_SOL_ANG ;  ALLOCATE(SOL_GRD(NSGP))
    SOL_GRD = DCOS(SOL_ANG(:)/RAD_TO_DEG) 
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN
  END Block Set_Out_Angs
  
  ! Standard grid of the  write(*,*) '   RTM_TYPE ',  cosine of solar zenith angles
  CSOL_MIN = 999.D0 ; CSOL_MAX = -999.D0
  CSOL_TH_MIN = 999.D0 ; CSOL_TH_MAX = -999.D0
  SELECT CASE (SolGrid%TYPE)
  CASE ('opt_db', 'uda', 'udc', 'udg', 'uda_th')
     DO IA = 1, GeoD%NAlts 
        DO IV = 1, GeoD%NVA 
           DO IS = 1, GeoD%NSol
              DO IP = 1, GeoD%NPhi
!               The minimum solar senith angle (max CSOl) is defined by using
!               the refractive angle rather than geometrical one. This is
!               needed to avoid extrapolation then interpolating the phase
!               matrix on the refractive solar zenith angle. For the maximum
!               solar zenith angle we keep the geometrical value to avoid
!               possible extrapolation for the pseodo-spherical radiative field.
                 CSOL = MAXVAL(GeoD%LSG(IA,IV,IS,IP)%C_SZA(:))
                 IF(CSOL > CSOL_MAX) CSOL_MAX = CSOL
                 CSOL = MINVAL(GeoD%LSG(IA,IV,IS,IP)%CG_SZA(:))
                 IF(CSOL < CSOL_MIN) CSOL_MIN = CSOL
                 CSOL = GeoD%LSG(IA,IV,IS,IP)%C_SZA(GeoD%LVG(IA,IV)%NTH)
                 IF(CSOL > CSOL_TH_MAX) CSOL_TH_MAX = CSOL
                 CSOL = GeoD%LSG(IA,IV,IS,IP)%CG_SZA(GeoD%LVG(IA,IV)%NTH)
                 IF(CSOL < CSOL_TH_MIN) CSOL_TH_MIN = CSOL
              END DO
           END DO
        END DO
     END DO
  CASE ('ura', 'urc', 'urg')
     CSOL_MAX = Cos(SolGrid%AngMin/RAD_TO_DEG)
     CSOL_MIN = Cos(SolGrid%AngMax/RAD_TO_DEG)
     CSOL_TH_MAX = CSOL_MAX
     CSOL_TH_MIN = CSOL_MIN
  CASE ('one')
     CSOL_ONE = 0.0D0
     ! Calculate average SZA at TH or BOA for downward pointing LOSes and
     ! for the first point for upward pointing LOSes
     DO IA = 1, GeoD%NAlts 
        DO IV = 1, GeoD%NVA
           IF(GeoD%LVG(IA,IV)%C_VA(1) > 0.0D0) THEN
              JJ = GeoD%LVG(IA,IV)%NTH
           ELSE
              JJ = 1
           END IF
           DO IS = 1, GeoD%NSol
              DO IP = 1, GeoD%NPhi
                 CSOL_ONE = CSOL_ONE + GeoD%LSG(IA,IV,IS,IP)%CG_SZA(JJ)
              END DO
           END DO
        END DO
     END DO
     CSOL_ONE = CSOL_ONE/DBLE(GeoD%NAlts*GeoD%NVA*GeoD%NSol*GeoD%NPhi)
  END SELECT

  ONE_MS_SZ = .FALSE. ;  X_SHAD = XM_S_INIT(NSOL_INT)
  SELECT CASE (SolGrid%TYPE)
  CASE ('full_db')
     NSGP = NSOL_INT
     ALLOCATE(SOL_GRD(NSGP))
     SOL_GRD = XM_S_INIT
  CASE ('opt_db')
     DO NSGP = 9, NSOL_INT - 1
        IF(CSOL_MIN.GE.XM_S_INIT(NSGP)) EXIT
     END DO
     ALLOCATE(SOL_GRD(NSGP))
     SOL_GRD = XM_S_INIT(1:NSGP) 
  CASE ('one')
     ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
     SOL_GRD = CSOL_ONE
  CASE ('udc', 'urc')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum anf maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        ! Equidistant intervals of the solar angles
        NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP) )
        DELTA_S = (CSOL_MAX - CSOL_MIN)/(NSGP - 1)         
        SOL_GRD(1) = CSOL_MAX
        DO J = 1, NSGP - 1
           SOL_GRD(J + 1) = SOL_GRD(J) - DELTA_S
        ENDDO
        SOL_GRD(NSGP) = CSOL_MIN
     End If
  CASE ('uda','ura')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum and maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        ! Equidistant intervals of the solar angles
        NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP) )
        DELTA_S = (DACOS(CSOL_MAX) - DACOS(CSOL_MIN))/(NSGP - 1)         
        SOL_GRD(1) = CSOL_MAX
        DO J = 1, NSGP - 1
           SOL_GRD(J + 1) = COS(DACOS(SOL_GRD(J)) - DELTA_S)
        ENDDO
        SOL_GRD(NSGP) = CSOL_MIN
     End If
  CASE ('uda_th')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum and maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        If(CSOL_TH_MAX - CSOL_TH_MIN < 1.0D-3) Then
           NSGP = 3 ;  ALLOCATE( SOL_GRD(NSGP) )
           SOL_GRD(1) = CSOL_MAX
           SOL_GRD(2) = CSOL_TH_MAX
           SOL_GRD(3) = CSOL_MIN
        Else
           ! Equidistant intervals of the solar angles
           NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP) )
           DELTA_S = (ACOS(CSOL_TH_MAX) - ACOS(CSOL_TH_MIN))/(NSGP - 3)
           SOL_GRD(1) = CSOL_MAX
           SOL_GRD(2) = CSOL_TH_MAX
           DO J = 2, NSGP - 3
              SOL_GRD(J + 1) = COS(ACOS(SOL_GRD(J)) - DELTA_S)
           ENDDO
           SOL_GRD(NSGP - 1) = CSOL_TH_MIN
           SOL_GRD(NSGP) = CSOL_MIN
        End If
     End If
  CASE ('udg_0')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum and maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        ! Geometrical progression intervals of the solar angles
        NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP) )
        DELTA_S = 1.2D0 ! Progression ratio
        !Geometric progression
        DO J = 1, NSGP
           SOL_GRD(J) = DELTA_S**(J - 1)
        End Do
        SOL_GRD(:) = SOL_GRD(:)/(SOL_GRD(NSGP) - 1)*&
             (ACOS(CSOL_MIN) - ACOS(CSOL_MAX))
        SOL_GRD(:) = SOL_GRD(:)- SOL_GRD(1)
        DO J = 1, NSGP
           SOL_GRD(J) = ACOS(CSOL_MIN) - SOL_GRD(J)
        END DO
        SOL_GRD(1:NSGP) = Cos(SOL_GRD(NSGP:1:-1))
     End If
  CASE ('udg', 'urg')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum and maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        ! Geometrical progression intervals of the solar angles
        NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP) )
        DELTA_S = SolGrid%PR ! Progression ratio
        If(Real(CSOL_TH_MAX) /= Real(CSOL_MAX)) Then
           Iadd = 1
        Else
           Iadd = 0
        End If
        !Geometric progression
        DO J = 1, NSGP - Iadd
           SOL_GRD(J) = DELTA_S**(J - 1)
        End Do
        SOL_GRD(:) = SOL_GRD(:)/(SOL_GRD(NSGP) - 1)*&
             (ACOS(CSOL_MIN) - ACOS(CSOL_TH_MAX))
        SOL_GRD(:) = SOL_GRD(:) - SOL_GRD(1)
        DO J = 1, NSGP - Iadd
           SOL_GRD(J) = ACOS(CSOL_MIN) - SOL_GRD(J)
        END DO
        SOL_GRD(Iadd + 1:NSGP) = Cos(SOL_GRD(NSGP - Iadd:1:-1))
        If(Iadd == 1) SOL_GRD(1) = CSOL_MAX
!        SOL_GRD(1) = Cos(Max(Acos(CSOL_MAX) - 15./180.*Pie,0.0d0))
     End If
  CASE ('udg_xx')
     If(CSOL_MAX - CSOL_MIN < 1.0D-3) Then
        ! Switch to one angle in minimum and maximum angles are too close
        ONE_MS_SZ = .TRUE. ;  NSGP = 1 ;  ALLOCATE(SOL_GRD(NSGP))
        SOL_GRD = (CSOL_MAX + CSOL_MIN)/2.0D0
     Else       
        ! Geometrical progression intervals of the solar angles
        NSGP = SolGrid%NAng ;  ALLOCATE( SOL_GRD(NSGP + 3) )
        DELTA_S = 1.2D0 ! Progression ratio
        !Geometric progression
        DO J = 1, NSGP - 1
           SOL_GRD(J) = DELTA_S**(J - 1)
        End Do
        SOL_GRD(:) = SOL_GRD(:)/(SOL_GRD(NSGP - 1) - 1)*&
             (ACOS(CSOL_MIN) - ACOS(CSOL_TH_MAX))
        SOL_GRD(:) = SOL_GRD(:)- SOL_GRD(1)
        DO J = 1, NSGP - 1
           SOL_GRD(J) = ACOS(CSOL_MIN) - SOL_GRD(J)
        END DO
        SOL_GRD(2:NSGP) = Cos(SOL_GRD(NSGP - 1:1:-1))
        SOL_GRD(1) = CSOL_MAX
        SOL_GRD(NSGP + 1) = Cos(ACOS(CSOL_MIN) + 4./180.*Pie)
        SOL_GRD(NSGP + 2) = Cos(ACOS(CSOL_MIN) + 7./180.*Pie)
        SOL_GRD(NSGP + 3) = Cos(ACOS(CSOL_MIN) + 9./180.*Pie)
        NSGP = NSGP + 3
     End If
  END SELECT

!     write(6,*) Acos(CSOL_MAX)*RAD_TO_DEG, Acos(CSOL_MIN)*RAD_TO_DEG
!     write(6,*) Acos(CSOL_TH_MAX)*RAD_TO_DEG, Acos(CSOL_TH_MIN)*RAD_TO_DEG
!     write(6,*) Acos(SOL_GRD)*RAD_TO_DEG
!     write(6,*) CSOL_MIN, CSOL_MAX
!     write(6,*) SOL_GRD
!     pause

  IF(LOG_LEVEL(1) >= N_SSO) THEN
     DD = MinVal(Abs(Acos(SOL_GRD(1:Size(SOL_GRD) - 1))*RAD_TO_DEG - Acos(SOL_GRD(2:Size(SOL_GRD)))*RAD_TO_DEG))
     FStr = "F"
     write(Str,"(I0)") Ceiling(Abs(Log10(DD))) + 4
     FStr = FStr//Trim(Str)//"."
     write(Str,"(I0)") Ceiling(Abs(Log10(DD)))
     FStr = FStr//Trim(Str)
     WRITE(LUNIT,'(A, 200(1X,'//FStr//'))') "  Solar Grid:", Acos(SOL_GRD)*RAD_TO_DEG
!     WRITE(LUNIT, &
!          '(2X,"*** Minimum solar zenith angle ",F5.2," deg")') &
!          DACOS(SOL_GRD(1))*RAD_TO_DEG
!     WRITE(LUNIT, &
!          '(2X,"*** Maximum solar zenith angle ",F5.2," deg")') &
!          DACOS(SOL_GRD(NSGP))*RAD_TO_DEG
!     WRITE(LUNIT,*)
  END IF

  CheckSettings: Block
    If (MaxVal(Acos(SOL_GRD)*RAD_TO_DEG) <= 90.) Exit CheckSettings
    If (.Not.RAD_FIELD) Exit CheckSettings
    If (.Not.Interp_RF) Exit CheckSettings
    Call WrtErr%Add &
         ('Initialization of radiation field by interpoplation is not')
    Call WrtErr%Add &
         ('  allowed for solar zenith angles larger than 90 degrees.')
    Call WrtErr%Add &
         ('Please set the "Initialization of radiation field" control ')
    Call WrtErr%Add &
         ('  parameter in "control_ac.inp" file to "RF_SFI".')
    call WrtErr%Wrt(R_Name, 'e')
    Return 1
  End Block CheckSettings
   
  
!  WRITE(*,*) '   Used solar grid: NSGP = NSOL = ',NSGP
!  DO J = 1, SIZE(SOL_GRD)
!     WRITE(*,'(2x,I3,2(2x,F8.3))') J, DACOS(SOL_GRD(J))*RAD_TO_DEG, SOL_GRD(J)
!  ENDDO
  
  IF(ALLOCATED(X_INV)) DEALLOCATE(X_INV)
  ALLOCATE( X_INV(NSGP) ) ; X_INV(1:NSGP) = SOL_GRD(NSGP:1:-1)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE SOL_GRID
!***************************************************************************

