C***********************************************************************
C
C       SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C       Solution of RTE according to DOM for Gauss discrete angles
C
C***********************************************************************
      SUBROUTINE SOLV_DOM(JPOS, FLAG, *, *, Mh, E_W, AGR, GSL, SSB, 
     $     TE, LOA, LOW, UDB, SDO, LAS, SOLV_LOC, Ph_z, SLS, ADS, 
     $     ADO, SRR, SRR_Jact, SVR, Ph_R, Ph_A, Ph_dA, Ph_C, Ph_Us,
     $     Ph_Ks, Ph_Kl, PH_M, RADOUT)
c     ... Called by  "st_dom_tau.f": 
c                      AGR, GSL, SSB, UDB, SDO, LAS, ADO <== ..._A
c                    "iterate_coa.f": 
c                      AGR, GSL, SSB, UDB, SDO, LAS, ADO <== ..._W

      USE AEROSOLS,    ONLY: LA_N, LA_S, LA_E, Dwr_mi, N_dwr
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT, 
     $                       Prof_Cpar
      USE CLOUD_INPUT, ONLY: Plus_ice, Phm_const, N_ref_db, MLCS,
     $                       Ref_DB, N_ref_ice, Ref_ice, Deriv_Phc
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE EMISSION,    ONLY: N_AT, Emi_MOMS
      USE GRIDS,       ONLY: DZR_A => DZR, ZZZ
      USE GT_POLYN,    ONLY: Leg => NLEGEN, AG
      USE HYDROSOLS,   ONLY: LA_N_HS, LA_S_HS, LA_E_HS, SEL_VSC
      USE INTERN_DIMS, ONLY: Nlg_ray, Nlg_Aer, Nlg_Cld, Nvs, N_sol_ls, 
     $                       Nm_vs, NGb, Nm_ice
      USE LOG_PARS,    ONLY: LUNIT, N_ESO, LOG_LEVEL, N_DSO, N_RSO,
     $                       ERROR_TXT, EU, N_XSO, DIR_OUT
      USE NODES,       ONLY: N_NODE, MJU_NODE
      USE RAMAN,       ONLY: Ram_Egg, Ram_EgL, Ram_Egs, Ram_EsL, RRS_DO,
     $                       RRS_ADJ, DOM_API
      USE RTM_OUTPUT,  ONLY: Wup, N_OP
      USE ST_VARS,     ONLY: Vec_Sca, DO_AEROSOL, N_RHS, N_Adj, SDVI, 
     $                       M_LIM, Pie, RTM_TYPE, VST, OWF_Ks, Finc,  
     $                       OWF_Kl, OWF_Use, WF_FLAG, Kvs, Emi_FLG, 
     $                       GeoD => GeoD_OUT, GeoD_PS, RAD_FIELD,
     $                       Interp_RF
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, NOA => N_OBS_ALT,  
     $                       ANG_OUT, NFA => N_FOV_ANG, N_LOS_OUT, 
     $                       LOS_DEF_FLAG, LW_FOV
      USE TOOLS,       ONLY: FIND_IDX, INQ_WF
      USE TYPES,       ONLY: KND, LOA_TYPE, LOW_TYPE, SDO_TYPE, TE_TYPE,
     $                       UDB_TYPE, SSB_TYPE, GSL_TYPE, LAS_TYPE, 
     $                       ADO_TYPE, PHA_CLD, AGR_type, VRS_TYPE, 
     $                       RRS_TYPE, PHA_MATR, PH_MET, SLS_type
      USE SLAB_2L,     ONLY: TES_2L_SLAB, NSTR_T1, Fr_01, Fr_20, Ref_idx
      USE WATER,       ONLY: UWT, UIT, NL_W => NLAYERS_UW, ZZZ_UW, 
     $                       NSTR_T, Leg_W => NLEG_W, INEL_DO
      USE WATER_ICE,   ONLY: IWS, IWT, LA_N_IW, LA_S_IW, LA_E_IW
      USE WF_NUM,      ONLY: NUM_CLD
c     ------------------------------------------------------------------

      INTEGER,           INTENT(IN)  :: JPOS, Mh, SRR_Jact
      CHARACTER (LEN=1), INTENT(IN)  :: FLAG
      CHARACTER (LEN=*), INTENT(IN)  :: SOLV_LOC
      REAL (KND)  RADOUT(N_OP,N_LOS_OUT,Nvs)
c     ----------------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LOW_type) :: LOW    ! Local optical parameters of water
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
      TYPE (GSL_TYPE) :: GSL    ! Angles
      TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
      TYPE (AGR_type) :: AGR
      TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
      TYPE (SLS_type) :: SLS    ! Spherical line-of-sight
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO    ! Boundary conditions
      TYPE (LAS_TYPE) :: LAS    ! Linear algebraic system
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (PHA_MATR) PH_M
c     -------------------------------------------------------------
c     ... Adjoint DOM auxiliary spherical geometry |
c     ----------------------------------------------
      TYPE ADS_type
        SEQUENCE
        LOGICAL, POINTER :: Act_ag(:,:)
        REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
      END TYPE ADS_type
      TYPE (ADS_type) :: ADS
c     -------------------------------------------------------------
      TYPE (PH_MET) :: Ph_1d, Ph_R, Ph_Ks, Ph_Kl
c     -------------------------------------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_z, Ph_A, Ph_Us
c     -------------------------------------------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     --------------------------------------------------------------
      REAL (KND),        INTENT(IN)  :: E_W
c     -----------------------------------------------------
c     Local variables
c     ---------------
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'SOLV_DOM'
      LOGICAL  Logi, PP_SPH
      INTEGER I, J, K, L, Idr, K1, Js, Jv, Jo, Jco, Jro, Lk, Yi, Yn, NN,
     $     NSZA, NLSA, INFO, Nr2, LL1, LR1, S_AG, E_AG, NNv, NL, NrL, 
     $     N_db, I1, I2, Jvs, IA, Jmc, KK, IiW
      REAL (KND) S1, Wdo, DL
      REAL (KND), ALLOCATABLE :: T_AU(:), DTR(:), SCA_AL(:), DZR(:),
     $     RHS(:), TE2(:), DTR_AG(:), ALW(:), Ex_AG(:,:), Norm_lay(:,:),
     $     PSU(:,:,:), PMI(:,:,:), Psip_lay(:,:,:), Psim_lay(:,:,:),
     $     Mux(:)
      TARGET  Ex_AG, Mux
      INTEGER,    POINTER :: LAY_GR(:,:)
      REAL (KND), POINTER :: SCR(:), SCA(:), SCC(:), AL(:), Pm(:),
     $     Por(:,:), P01(:,:,:), P02(:,:,:), PVEC(:,:), HSC(:),
     $     G_NL(:,:,:), XM(:), AXM(:), MuS(:), MuV(:), BET(:), 
     $     Part_TD(:,:,:), SVEC_AG(:,:), XM_TWO(:), Fre_r(:),
     $     PART_D(:,:,:), PART_U(:,:,:), Rdb(:)
c     ==================================================================

c      WRITE(*,*) ' SHAPE(RADOUT) solv_dom: ', SHAPE(RADOUT)
c      WRITE(*,*) RADOUT

      CALL ST_TRACK (R_NAME, 1, 'I')
      IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(A,I3)') '   Mh = ',Mh
      IA = 1
c     ... Set all angles parameters. Depending on the medium
c         will be used here the structure GSL_A or GSL_W
      NN  = GSL%NN ; NNv = Nvs*NN ; XM => GSL%XM ; AXM => GSL%AXM  

      PP_SPH = RAD_FIELD.AND.(.NOT.Interp_RF)
c      WRITE(*,*) '    RAD_FIELD ', RAD_FIELD
c      WRITE(*,*) '    Interp_RF ', Interp_RF
c      WRITE(*,*) '    PP_SPH    ', PP_SPH
      IF( RAD_FIELD.AND.(.NOT.Interp_RF) ) THEN
         NLSA = N_NODE ;  MuV => MJU_NODE
         NSZA = GeoD_PS%NSOL
         ALLOCATE( Mux(NSZA) )
         FORALL( K = 1:NSZA ) Mux(K) = 
     $        GeoD_PS%LSG(1,1,K,1)%C_SZA(1)
         MuS => Mux
      ELSE
         NLSA = GSL%NLSA ; MuV => GSL%MuV
         NSZA = GSL%NSZA ; MuS => GSL%MuS
      ENDIF

c      WRITE(*,*) '  NSZA in "solv_dom.f": ', NSZA
c      WRITE(*,*) '  NLSA in "solv_dom.f": ', NLSA
c      PAUSE

c     ... Set altitude grid parameters. Depending on the medium
c         will be used here the structure AGR_A or AGR_W
      NL = AGR%NL ; NrL = AGR%NrL 
      LAY_GR => AGR%LAY_GR

c     ... Set particular solutions and auxiliary array
      Part_TD => ADO%Part_TD  ;  PART_U  => LAS%PART_U
      PART_D  => LAS%PART_D   ;  SVEC_AG => LAS%SVEC_AG

      ALLOCATE( T_AU(NL), DTR(NL), SCA_AL(NL), DZR(NL),
     $     RHS(2*NNv), TE2(2*NNv), PSU(NL,NNv,NNv), PMI(NL,NNv,NNv),
     $     Ex_AG(NNv,NrL-1), DTR_AG(NrL), ALW(NL-1), XM_TWO(2*NN),
     $     Psip_lay(NNv,NNv,NL-1), Psim_lay(NNv,NNv,NL-1),
     $     Norm_lay(NNv,NL-1) )

      XM_TWO( 1   :NN   ) =  XM( 1:NN )
      XM_TWO( NN+1:2*NN ) = -XM( 1:NN )

      ALLOCATE( Fre_r(NSZA),
c               NLSA - nodes if spherical
c              
     $     Ph_1d%Egg(NNv,NNv,2), Ph_1d%EgL(NNv,Nvs*NLSA,2),
     $     Ph_1d%Egs(NNv,Nvs*NSZA,2),
     $     Ph_1d%EsL(Nvs*NLSA,Nvs*NSZA,N_sol_ls) )

c     ... Local pointers is associated with structure elements
c         corresponding to the local optical parameters of the
c         atmosphere or of the water:
      SELECT CASE(FLAG)
      CASE('A','a')
         SCR => LOA%SCR ; SCA => LOA%SCA ; SCC => LOA%SCC
         AL  => LOA%AL
         BET => LOA%BET
         SCA_AL(:) = SCR(:) + SCA(:) + SCC(:)
         IF( ANY(Plus_ice) ) SCA_AL(:) = SCA_AL(:) + LOA%SCI(:)

         IF( Emi_FLG ) THEN
            DO I = 1, N_AT
               SCA_AL(:) = SCA_AL(:) + LOA%EMC(I,:)
            ENDDO
         ENDIF

         DZR(:) = DZR_A(:)
         IF( LOG_LEVEL(1) >= N_ESO ) THEN
            WRITE(LUNIT,*) ' *****************************************'
            WRITE(LUNIT,*) ' ************* Atmosphere ****************'
            WRITE(LUNIT,*) ' *****************************************'
         ENDIF
c    ... end case('A')
      CASE('O','o')
         IF( LOG_LEVEL(1) >= N_ESO ) THEN
            WRITE(LUNIT,*) ' ************************************'
            WRITE(LUNIT,*) ' ************* Ocean ****************'
            WRITE(LUNIT,*) ' ************************************'
         ENDIF
         SCR => LOW%SCR_W ; AL  => LOW%AL_W
         BET => LOW%BET
         SCA_AL(:) = SCR(:)
         DO I = 1, IWS(1)%N_HPC
            SCA_AL(:) = SCA_AL(:) + LOW%HSCA(:,I,1)
         ENDDO
         DZR (NL_W) = 0.d0
         DZR(1:NL_W - 1) = ZZZ_UW(2:NL_W) - ZZZ_UW(1:NL_W - 1)
c    ... end case('O')
      END SELECT

      G_NL => SDO%G_NL  ; Wdo = 1.d0 - Wup
      ALW(:) = Wup*AL(1:NL-1) + Wdo*AL(2:NL)
c      write(*,*) '    Test AL: '
c      DO J = 1, NL
c         write(*,*) J, AL(J)
c      ENDDO      
c      PAUSE
      T_AU(1) = 0.d0
      DO J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + ALW(J)*DZR(J)
      ENDDO
      DTR(1) = 0.d0
      DTR(2:NL) = T_AU(2:NL) - T_AU(1:NL-1)
      
      IF (LOG_LEVEL(1) >= N_ESO)  WRITE(LUNIT,*)
     $     ' ===> TAU in solv_dom :', T_AU(NL)

c     ... Phase function/matrixes for atmosphere/ocean
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
c     ------------------------------------------------------------------
         CALL MAKE_PHASE_SCAL
c         WRITE(*,*) ' 1 ', SUM( Ph_z%Egg(:,:,:,1) )
c         WRITE(*,*) ' 2 ', SUM( Ph_z%Egg(:,:,:,2) )
c        ... Second dimension of array Ph_z%Egg(:,|,:,1:2) contains
c            1/XM(:) (see "phase_functions.f"). In the scalar case 
c            this array contains discrete representation of the phase
c            matrix Gauss -> Gauss. In this routine should be used 
c            sum and difference of these matrixes.
         PSU(:,:,:) = 0.5d0*(Ph_z%Egg(:,:,:,1) + Ph_z%Egg(:,:,:,2))
         PMI(:,:,:) = 0.5d0*(Ph_z%Egg(:,:,:,1) - Ph_z%Egg(:,:,:,2))
c        ... Second dimension of array Ph_z%Egs(:,|,:,1:2) contains
c            1/XM(:) (see "phase_functions.f"). This array contains 
c            discrete representation of the phase matrix Solar -> Gauss. 
         P01 => Ph_z%Egs(:,:,:,1) ; P02 => Ph_z%Egs(:,:,:,2)
      CASE( 'vect_rte' )
c     ------------------------------------------------------------------
         CALL MAKE_PHASE_VECT
c        ... Third dimension of array Ph_z%Egg(:,:,|,1:2) contains
c            1/XM(:) (see "phase_matr.f"). These arrays are already 
c            in the form of the sum and difference of phase matrixes.
         PSU(:,:,:) = ph_z%Egg(:,:,:,1)
         PMI(:,:,:) = ph_z%Egg(:,:,:,2)

c       ... In contrast to the scalar case array ph_z%Egs(:,|,:,1:2) 
c           does not contain 1/XM(:).
c                 +Gauss <= +Sol              +Gauss <= -Sol
         P01 => ph_z%Egs(:,:,:,1)  ;  P02 => ph_z%Egs(:,:,:,2)
c       ... In the vector case F14=1/4 shoud be replaced by 2*F14=1/2,
c           therefore, P01, P02 should be multiplied by 2.
         P01 = 2.d0*P01        ;  P02   = 2.d0*P02

c        ... The same reason as by P01 and P02
         Ph_z%EsL = 2.d0*Ph_z%EsL 
      END SELECT
c     ------------------------------------------------------------------
c     ... Scalar case: 
c         Second dimension of array Ph_z%EgL(:,|,:,1:2) contains
c            1/XM(:) (see "phase_functions.f"). 
c     ... Vector case: 
c         The array Ph_z%EgL(:,:,:,1:2) does not contain 1/XM(:)
c         (see "phase_matr.f"). 
c         Ph_z%EgL(:,|,:,2) contains  discrete representation of the 
c                           phase matrix Gauss ->  Los_ang.
c         Ph_z%EgL(:,|,:,1) contains  discrete representation of the 
c                           phase matrix Gauss -> -Los_ang.
c         (see "phase_matr.f").

      SELECT CASE( TRIM(RTM_TYPE) )
      CASE('pp_scat', 'ps_scat')
         IF( UDB%BRDF_FLAG.AND.(.NOT.DO_SSRCA).AND.(.NOT.UWT)
     $        .AND.(Mh > 1) ) THEN
            S1 = SUM(PSU + PMI)
            IF(GeoD%View(IA)%S=='up'.AND. (DABS(S1) < -1.0d-12) 
c            IF(GeoD%View(IA)%S=='up'.AND. (DABS(S1) < 1.0d-12) 
     $           .AND. .NOT.WF_FLAG) THEN
               WRITE(*,*) '    TRANS_WITHOUT_SCAT ==> Mh = ', Mh
               IF( UDB%SPECULAR ) THEN
                  WRITE(*,*) '    RETURN 2 from IF ==> '
                  RADOUT(:,:,:) = 0.d0
                  CALL ST_TRACK (R_NAME, 1, 'O')
                  RETURN 2
               ENDIF
c              ... Transform reflected "harmonic" from the bottom to 
c                  the top neglecting all scattering processes in 
c                  the atmosphere:
               CALL TRANS_WITHOUT_SCAT
               CALL ST_TRACK (R_NAME, 1, 'O')
               RETURN 2
            ENDIF
         ENDIF
      CASE DEFAULT
      END SELECT

c      write(*,*) ' --- solv_dom 2'
      DTR_AG(1) = 0.d0
      DO K = 1,NrL-1
         S_AG = LAY_GR(K,1) ; E_AG = LAY_GR(K,2) + 1 ! Old levels
         DTR_AG(K+1) = T_AU(E_AG) - T_AU(S_AG)
      ENDDO

      IF(LOG_LEVEL(1) >= N_ESO)
     $     WRITE(LUNIT,*) ' Start main do NRL = ', NrL

cc      IF( SVR%ACT_WL(JPOS) ) THEN
c         PSU = 0.d0  ;  PMI = 0.d0 
cc      ENDIF
      
c     ... General solution of the homogeneous equation and
c         particular solution of the inhomogeneous equation
      CALL HOMOG_PARTIC(Mh, NrL, NL, NN, NSZA, LAY_GR, UDB,
     $     ALW, SCA_AL, DTR_AG, DTR, SSB, TE, SOLV_LOC, UDB%SPECULAR,
     $     Bet, Mus, T_AU, PSU, PMI, P01, P02, SDO, GSL, PART_U, PART_D,
     $     Ex_AG, Norm_lay, Psim_lay, Psip_lay, JPOS, *880, FLAG )

c      WRITE(*,*) ' --> After "homog_partic.f" <--- '

      SDO%Ex_AG(:,1) = Ex_AG(:,1)
      SDO%Ex_AG(:,2) = Ex_AG(:,NrL-1)

      IF( LOG_LEVEL(1) <= N_XSO ) THEN
c         WRITE(*,*)  '  After: '
         WRITE(LUNIT,*) '    PART_U:', SUM(PART_U)
         WRITE(LUNIT,*) '    PART_D:', SUM(PART_D)
         IF( ISNAN(SUM(PART_U)) ) STOP '    stop after HOMOG_PARTIC.f '
      ENDIF

c     ... Calculation of particular solution corresponding 
c         to inelastic processes. It will be added to the regular 
c         part of particular solution
c     ==================================================================
      SELECT CASE(FLAG)
      CASE('A','a')
c     ... RRS (rotational Raman scattering in atmosphere)
         Logi = SRR%ACT_WL(JPOS).AND.Mh<=3
         IF( RRS_DO.AND.Logi ) THEN 
            CALL ACT_RRS
            IF( DOM_API ) THEN
               CALL PART_SOL_RRS_SIMPL(JPOS, Mh, NN, NSZA, NL, DTR, ALW,
     $              Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, 
     $              PART_D, SDO, SRR, SRR_Jact)
            ELSE
               CALL PART_SOL_RRS(JPOS, Mh, NN, NSZA, NL, DTR, ALW,
     $              Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, 
     $              PART_D, SDO, SRR, SRR_Jact)
c               CALL PART_SOL_RRS_1(JPOS, Mh, NN, NSZA, NL, DTR, ALW,
c     $              Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, 
c     $              PART_D, SDO, SRR, SRR_Jact, LOA%SCR, SSB%EXS)
            ENDIF
         ENDIF
         IF( LOG_LEVEL(1) <= N_XSO ) THEN
            WRITE(LUNIT,*) '    PART_U:', SUM(PART_U)
            WRITE(LUNIT,*) '    PART_D:', SUM(PART_D)
         ENDIF
c    ... end case('A')
      CASE('O','o')
c     ... VRS (vibrational Raman scattering) and/or fluorescence
         Logi = SVR%ACT_WL(JPOS).AND.Mh<=SVR%Mh_OUT
         IF( INEL_DO .AND. Logi ) THEN 
            CALL PART_SOL_VRS(JPOS, Mh, NN, NSZA, NL, DTR, ALW,
     $           Norm_lay, Psim_lay, Psip_lay, MuS, PART_U, 
     $           PART_D, SDO, SVR)
         ENDIF
         IF( LOG_LEVEL(1) <= N_XSO ) THEN
            WRITE(LUNIT,*) '    PART_U:', SUM(PART_U)
            WRITE(LUNIT,*) '    PART_D:', SUM(PART_D)
         ENDIF
c    ... end case('O')
      END SELECT
c     ==================================================================

c      ... Matrix of the linear system and its LU-decomposition
      CALL FORM_MATRIX(FLAG, NrL, NL, Mh, NN, NNv, SDO, LAY_GR,
     $     Ex_AG, DTR_AG, UDB, GSL, LAS%MATTEB, LAS%INDX)

      SVEC_AG = 0.d0

      SELECT CASE ( TRIM(SOLV_LOC) )
      CASE( 'Solv_SFI' )
c        ... R.h.s. of the forward RTE for NSZA solar zenith angles
c            will be stored in SVEC_AG(:,1:NSZA). 
c            In the case of 'spher_scat':
c                   NSZA is the number of the solar angles grid
c                        (see "sol_grid.f")
c            In the case of 'pp_scat', 'ps_scat':
c                   NSZA is the actual number of solar angles
         CALL RHS_DRTE(FLAG, Mh, NrL, NN, NSZA, XM, AXM, UDB,
     $        Mus, SSB%EXS(NL,:), TE%ADD_PLANCK, TE%PLANCK_SURF,
     $        SVEC_AG, PART_U, PART_D)
         IF( LOG_LEVEL(1) <= N_XSO ) THEN
            WRITE(LUNIT,*) '    SVEC_AG:', SUM(SVEC_AG)
         ENDIF

c        ... Save right hand-side of LAS into array LAS%RHS_LAS
         LAS%RHS_LAS = SVEC_AG(:,1:Kvs*NSZA)

         IF( TES_2L_SLAB .AND. FLAG == 'A' ) THEN
c           ... Add the diffuse isotropical radiation with the intensity 1.
c               transmitted over the upper interface into a first medium
            DO I = 1,NSZA
               LAS%RHS_LAS(1+NSTR_T1:NN,I) = LAS%RHS_LAS(1+NSTR_T1:NN,I)
     $              + (1.d0 - Fr_01(1+NSTR_T1:NN))*Ref_idx(1)**2
            ENDDO
         ENDIF  ! TES_2L_SLAB

c         WRITE(*,*) '  Kvs,NSZA,N_RHS  ', Kvs, NSZA, N_RHS
         PVEC => SVEC_AG(:,1:Kvs*NSZA)
         SELECT CASE(FLAG)
         CASE('A','a')
c            write(*,*) '   Atmosphere ', Mh
c           ... Lower boundary condition of RTE in atmosphere contains 
c               contribution of the water leaving radiation. 
c               This is set to zero for first run in "dom_spherical.f"
            LL1 = (2*NrL-3)*NNv+1 ; LR1 = 2*NNv*(NrL-1)
            FORALL(I = 1:Kvs*NSZA) PVEC(LL1:LR1,I) = PVEC(LL1:LR1,I)
     $           + UDB%FoINC(:,I)
c           ... Test with two-layer slab
            IF( TES_2L_SLAB ) THEN
               FORALL(I = 1:NSZA) PVEC(1+NSTR_T1:NNv,I) = 
     $              PVEC(1+NSTR_T1:NNv,I) + 
     $              (1.d0 - Fr_01(1+NSTR_T1:NN))*Ref_idx(1)**2
            ENDIF  ! TES_2L_SLAB
c
         CASE('O','o')
c           ... Upper boundary condition of RTE in the ocean contains 
c               contribution of the downward atmospheric radiation.
            LL1 = 1
            IF( UDB%UBC=='Flat water' ) LL1 = LL1 + NSTR_T *Nvs
            FORALL(I = 1:Kvs*NSZA) PVEC(LL1:NNv,I) = PVEC(LL1:NNv,I)
     $           + UDB%FoINC(:,I)
         END SELECT
c         write(*,*) ' --- solv_dom 5'

c********* Pseudo-forward RTE only ********************
      CASE( 'Solv_ASI' )
c        ... Particular solution of the pseudo-forward RTE 
         CALL PART_SOL_ARTE(NL, NN, NLSA, AGR, SDO, Norm_lay, 
     $     Psim_lay, Psip_lay, Muv, DTR, ALW, Ph_z%EgL, 
     $     GSL%EXL, ADO)
c        ... R.h.s. of the pseudo-forward RTE for NLSA 
c            observation angles will be stored in SVEC_AG(:,1:NLSA)
         CALL RHS_ARTE
         PVEC => SVEC_AG(:,1:NLSA)

c********* Forward and pseudo-forward RTE ********************
      CASE( 'Solv_MIX' )
         CALL RHS_DRTE(FLAG, Mh, NrL, NN, NSZA, XM, AXM, UDB,
     $        MuS, SSB%EXS(NL,:), TE%ADD_PLANCK, TE%PLANCK_SURF,
     $        SVEC_AG, PART_U, PART_D)

c        ... Particular solution of the pseudo-forward RTE
         SELECT CASE( TRIM(RTM_TYPE) )
         CASE('spher_scat')
          CALL PART_SOL_ARTE_SPHER(Mh, NrL, NL, NN, GSL%NLSA, AGR%N_OBS,
c            CALL PART_SOL_ARTE_SPHER(Mh, NrL, NL, NN, NLSA, AGR%N_OBS,
     $           LAY_GR, SDO, Norm_lay, Psim_lay, Psip_lay, DTR,
     $           LOA, SLS, ALW, T_AU, ADS, ADO, AGR%ADAPTIVE_GRID)
         CASE DEFAULT
            CALL PART_SOL_ARTE(NL, NN, NLSA, AGR, SDO, Norm_lay,
     $           Psim_lay, Psip_lay, Muv, DTR, ALW, Ph_z%Egl,
     $           GSL%EXL, ADO)
         END SELECT
c        ... R.h.s. of the pseudo-forward RTE for N_adj "observation" 
c            angles will be stored in SVEC_AG(:,NSZA+1:NSZA+N_Adj).
c            N_Adj = NS_wf*NLSA in the plane-parallel case.
         CALL RHS_ARTE
         PVEC => SVEC_AG(:,1:N_RHS)

c        ... Save right hand-side of LAS into array LAS%RHS_LAS
         LAS%RHS_LAS = SVEC_AG(:,1:N_RHS)
         SELECT CASE(FLAG)
         CASE('A','a')
            LL1 = (2*NrL-3)*NNv+1 ; LR1 = 2*NNv*(NrL-1)
         CASE('O','o')
            LL1 = 1 ; LR1 = NNv
            IF( UDB%UBC=='Flat water' ) LL1 = LL1 + NSTR_T *Nvs
        END SELECT
c       ... Second dimension of UDB%FoINC(:,:) is in this case
c           NSZA + N_Adj. This array contains contribution of 
c           the downward atmospheric radiation in the upper boundary
c           condition for the ocean for forward and pseudo-forward RTE
        FORALL(I = 1:N_RHS) PVEC(LL1:LR1,I) = PVEC(LL1:LR1,I)
     $       + UDB%FoINC(:,I) 
      END SELECT

c     ... Solution of linear equation system with multiple r.h.s
c         Solve LAS for atmosphere/ocean with the corrected 
c         lower/upper boundary condition. The solution of the LAS 
c         is stored into array PVEC => SVEC_AG => LAS%SVEC_AG
      INFO = 0  ;  Nr2 = 2*NNv*(NrL-1)
             IF(LOG_LEVEL(1) >= N_DSO) WRITE(LUNIT,*) '   Strat DGBTRS'
c             write(*,*) ' ---  PVEC 2: ', SUM( PVEC )
c            write(*,*) '  LAS%MATTEB: ', SUM( LAS%MATTEB )
      CALL DGBTRS('N', NR2, 3*NNv-1, 3*NNv-1, N_RHS, LAS%MATTEB,
     $     9*NNv-2, LAS%INDX, PVEC, NR2, INFO)
      IF (INFO.NE.0) WRITE(LUNIT,*)
     $             ' DGBTRS error message: INFO =', INFO
c            write(*,*) ' ---  PVEC 3: ', SUM( PVEC )

      Yn = LAY_GR(NrL-1,1)
c     ... "Vertical" dimension of G_NL is NL and not NrL
      DO K = 1,NrL-1
         Yi = LAY_GR(K,1)
         DO J = Yi+1, LAY_GR(K,2)
            G_NL(J,:,:) = G_NL(Yi,:,:)
         ENDDO
      ENDDO
      IF( SDO%MASK_NL(1) ) THEN ! Conservative
         S1 = 3.d0/SDO%H1(1)  ;  K1 = SDO%E_MIN(1)
c        ... The value is given at the top of first layer
         G_NL(1,1:2*NNv:Nvs,    K1) =  S1*XM_TWO(:) + DTR_AG(2)
         G_NL(1,1:2*NNv:Nvs,NNv+K1) = -S1*XM_TWO(:)
      ENDIF
      IF( SDO%MASK_NL(Yn) ) THEN ! Conservative
         S1 = 3.d0/SDO%H1(Yn) ; K1 = SDO%E_MIN(Yn)
c        ... The value is given at the bottom of last layer
         G_NL(Yn,1:2*NNv:Nvs,    K1) =  S1*XM_TWO(:)
         G_NL(Yn,1:2*NNv:Nvs,NNv+K1) = -S1*XM_TWO(:) + DTR_AG(NrL)
      ENDIF
      
      LL1 = (2*NrL-4)*NNv + 1 ; LR1 = (2*NrL-3)*NNv ; Lk = LL1 - 1
      Jco = SDO%NcC(Yn)  ;  Jro = NNv - 2*Jco ; HSC => Ex_AG(:,NrL-1)
      SELECT CASE ( TRIM(SOLV_LOC) )
      CASE( 'Solv_ASI' )
         DO Jv = 1,NLSA
            RHS(:)     = PVEC(LL1:NR2,Jv)
            RHS(1:Jro) = PVEC(LL1:Lk+Jro,Jv)*HSC(1:Jro)
            DO K = Jro+1, Jro+2*Jco, 2
               RHS(K)  = PVEC(Lk+K,  Jv)*HSC(K)+PVEC(Lk+K+1,Jv)*HSC(K+1)
               RHS(K+1)= PVEC(Lk+K+1,Jv)*HSC(K)-PVEC(Lk+K,  Jv)*HSC(K+1)
            ENDDO
            TE2(:) = MATMUL(G_NL(Yn,:,:),RHS(:)) + PART_TD(NrL-1,:,Jv)
            ADO%FLUX_D(Jv) = SUM( TE2(1:NNv:Nvs)*AXM(1:NN) )
         ENDDO
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN

      CASE('Solv_MIX')
         DO I = 1,N_Adj
            Jv = I + SDVI
            RHS(:)     = PVEC(LL1:NR2,Jv)
            RHS(1:Jro) = PVEC(LL1:Lk+Jro,Jv)*HSC(1:Jro)
            DO K = Jro+1, Jro+2*Jco, 2
               RHS(K)  = PVEC(Lk+K,  Jv)*HSC(K)+PVEC(Lk+K+1,Jv)*HSC(K+1)
               RHS(K+1)= PVEC(Lk+K+1,Jv)*HSC(K)-PVEC(Lk+K,  Jv)*HSC(K+1)
            ENDDO
            TE2(:) = MATMUL(G_NL(Yn,:,:),RHS(:)) + PART_TD(NrL-1,:,I)
            ADO%FLUX_D(I) = SUM( TE2(1:NNv:Nvs)*AXM(1:NN) )
         ENDDO
      END SELECT                ! SOLV_LOC

c     ... 2nd dimension of pointer PVEC(:,:) is N_RHS
      CALL ST_TOP_BOT(Mh, NrL, NN, NNv, NSZA, SDO%NcC(1), SDO%NcC(Yn),
     $     Ex_AG(:,1), Ex_AG(:,NrL-1), PVEC,
     $     G_NL(1,:,:), G_NL(Yn,:,:), PART_U(1,:,:),
     $     PART_D(NrL-1,:,:), UDB, LAS%SOL_TOP, LAS%SOL_BOT,
     $     LAS%FLUX, LAS%Ref_surf, AXM, GSL%AG, ADO)

c     ===> Test output of fluxes       
      IF( UWT.AND.LOG_LEVEL(1) >= N_ESO ) THEN
         WRITE(LUNIT,*) '  Flux reflected from bottom: "solv_dom.f" '
         SELECT CASE(FLAG)
         CASE('A','a')
            WRITE(LUNIT,*) '       **** Atmosphere **** '
            IF( UDB%SPECULAR ) THEN
               WRITE(LUNIT,*) ' --> Case: "Specular reflection" '
               CALL Fre_REF(NSZA, Fre_r, MuS, UDB%Ref_wd, 'aa')
               DO K = 1, NSZA
                  WRITE(LUNIT,*) 2.*Pie*
     $                 SUM( LAS%SOL_BOT(NNv+1:2*NNv:Nvs,K)*AXM(1:NN) )
     $                 + Pie*SSB%EXS(NL,K)*MuS(K)*Fre_r(K)
               END DO
            ELSE
               WRITE(LUNIT,*) ' --> Case: "Any non mirrored surface"'
               DO K = 1,NSZA
                  WRITE(LUNIT,*) 2.*Pie
     $                 *SUM( LAS%SOL_BOT(NNv+1:2*NNv:Nvs,K)*AXM(1:NN) )
               ENDDO
            ENDIF
         CASE('O','o')
            WRITE(LUNIT,*) '       ****    Ocean   **** '
            DO K = 1,NSZA
               WRITE(LUNIT,*) 2.*Pie
     $              *SUM( LAS%SOL_BOT(NNv+1:2*NNv:Nvs,K)*AXM(1:NN) )
            ENDDO
         END SELECT
      ENDIF  ! UWT.AND.LOG_LEVEL(1)
c     <=== Test output of fluxes       

      NULLIFY(LAY_GR, SCR, SCA, SCC, AL, Pm, Por, P01, P02, PVEC,
     $     G_NL, Part_TD, SVEC_AG, PART_D, PART_U,
     $     XM, AXM, MuS, MuV, BET)

c     WRITE(*,*) '    After NULLIFY '

      DEALLOCATE(T_AU, DTR, SCA_AL, DZR, RHS, TE2, PSU, PMI,
     $     Ex_AG, DTR_AG, ALW, XM_TWO, Psip_lay, Psim_lay, Norm_lay,
     $     Ph_1d%Egg, Ph_1d%EgL, Ph_1d%Egs, Ph_1d%EsL)

c      WRITE(*,*) '    After   DEALLOCATE      '

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS

c **********************************************************************
c     Here will be calculated all needed auxiliary arrays at actual 
c         wavelength corresponding to the rotational Raman 
c                   scattering in atmosphere
c    The same as the routine "SAVE_EXC_RRS.F" called in "st_dom_tau.f"
c        but for calculations at the excitation wavelengths
c **********************************************************************
      SUBROUTINE ACT_RRS
c     Call in "solv_dom.f"

      USE RAMAN,  ONLY: Ram_Egg, K_dim
c     ------------------------------------------------------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ACT_RRS'
      INTEGER  K, J, Js, NNv, NL, Jwl
c     4+shi
      REAL (KND)  Hel, Gmp(GSL%NN*Nvs,GSL%NN*Nvs), 
     $     Gpp(GSL%NN*Nvs,GSL%NN*Nvs)
      REAL (KND), ALLOCATABLE :: TTT(:,:)
      REAL (KND), POINTER :: Fip(:,:), Fim(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Jwl = K_dim ; NL = AGR%NL
c     ... Save extinction coefficient and solar exponent
      IF( Mh == 1 ) THEN
         SRR%EXT_AUX(Jwl,:)   = (LOA%AL(1:NL-1) + LOA%AL(2:NL))/2.d0
c         SRR%EXS_AUX(Jwl,:,:) = SSB%EXS(:,:)
      ENDIF

c     ... Save separation and integration constants
      SRR%Nuj_EXC(Jwl,Mh,:,:) = SDO%Nuj_Lay(:,:)

      IF( ANY(SDO%NcC(:) > 0) ) THEN
         WRITE(*,*) '  Complex eigenvalues: '
         WRITE(*,'(A)') '    the altitude levels and '
         WRITE(*,'(A)') '  number of complex eigenvalues'
         DO I = 1, NL-1
            IF( SDO%NcC(I)==0 ) CYCLE
            WRITE(*,'(2x,I3,2x,I2)') I, SDO%NcC(I)
         ENDDO
c         PAUSE
      ENDIF
c     ...  CHECK negative or small values
      DO I = 1, NL-1 
         DO K = 1, GSL%NN*Nvs
            IF( SDO%Nuj_Lay(K,I) < 0.d0 ) THEN
               WRITE(*,*) '  Nuj - negative "st_dom_tau.f" '
               WRITE(*,*) Jwl, I, K,  SDO%Nuj_Lay(K,I)
            ENDIF
         ENDDO
      ENDDO

      SRR%MVEC(Jwl,Mh,:,:) = LAS%SVEC_AG(:,:)

c     ... Coefficients aj and bj of the particular solution at
c         actual wavelength include multipliers 
c         EXP(-Tau_(l-1)/Mu0) calculated at these wavelengths
      DO I = 1,NL-1
         DO Js = 1,NSZA  
           SRR%Aj(Jwl,Mh,:,I,Js) = SDO%Aj_sol(1,:,I,Js)*SSB%EXS(I,Js)
           SRR%Bj(Jwl,Mh,:,I,Js) = SDO%Bj_sol(1,:,I,Js)*SSB%EXS(I,Js)
         ENDDO
      ENDDO

      NNv = GSL%NN*Nvs
c     ... Fourier harmonics (1,2,3) of the Raman phase function 
c         (array Ram_Egg) were calculated in "phase_function.f"
c         (scalar case) or in "Phase_matr.f" (vector case) called in
c         "rtm_init_first.f". 
      SELECT CASE ( Vec_Sca )
      CASE( 'scal_rte' )
c        ... First dimension of Ram_Egg(:,:,1:2,Mh) contains 1/XM(:)
c            cosine of Gauss angles.
         FORALL( K = 1:NNv )
           Gpp(K,:) = Ram_Egg(K,:,1,Mh)*GSL%AG(:)*0.5d0 ! Mu <=> Mu
           Gmp(K,:) = Ram_Egg(K,:,2,Mh)*GSL%AG(:)*0.5d0 ! Mu <=> -Mu
         END FORALL 
      CASE( 'vect_rte' )
c        ... See comments in "st_up_down_sfi.f" to the array Pgg.
         ALLOCATE( TTT(NNv,NNv) ) 

         TTT(:,:) = Ram_Egg(:,:,1,Mh) + Ram_Egg(:,:,2,Mh)
         FORALL( K = 1:GSL%NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL%XM(K) 
         FORALL( K = 1:GSL%NN ) Gpp(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL%AG(K)*0.5d0

         TTT(:,:) = Ram_Egg(:,:,1,Mh) - Ram_Egg(:,:,2,Mh)
         FORALL( K = 1:GSL%NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL%XM(K) 
         FORALL( K = 1:GSL%NN ) Gmp(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL%AG(K)*0.5d0

      END SELECT

      DO I = 1,NL - 1
         Fip => SDO%G_NL(I,      1:NNv,1:NNv) 
         Fim => SDO%G_NL(I,1+NNv:2*NNv,1:NNv)
         DO K = 1,NNv
            SRR%GNp(Jwl,Mh,I,:,K) = 
     $           MATMUL(Gpp(:,:),Fip(:,K)) + MATMUL(Gmp(:,:),Fim(:,K))
            SRR%GNm(Jwl,Mh,I,:,K) = 
     $           MATMUL(Gpp(:,:),Fim(:,K)) + MATMUL(Gmp(:,:),Fip(:,K))
         ENDDO
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE ACT_RRS
c **********************************************************************

c *************************************************************************
c     ... R.h.s. of the pseudo-forward RTE
c              Polarization is implemented
c**************************************************************************
      SUBROUTINE RHS_ARTE

      USE LOG_PARS, ONLY: LUNIT, LOG_LEVEL, N_ESO
      USE ST_VARS,  ONLY: Nc_der, NS_wf, SLC => SHI_LIM_C
c     ------------------------------------------------------------------

      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'RHS_ARTE'
      INTEGER    K1, JL, Jw, Jw1, LL1, LL2, LR1, INI, Ka, Ke, Kli, MLI
      REAL (KND) S2, S2_BRF(Nvs*NN), HDU(Nvs*NN), R_DIR, Zn, Alb
      REAL (KND), POINTER :: Pps(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c      WRITE(*,*) '  SHAPE(SVEC_AG) ', SHAPE(SVEC_AG)
c      WRITE(*,*) '    rhs_arte: N_ADJ = ', N_Adj
c      PAUSE
c      WRITE(*,*) '    SDVI  = ', SDVI
c      WRITE(*,*) '  SHAPE( ADO%PART_TU ) ',  SHAPE( ADO%PART_TU ) 
c      WRITE(*,*) '    rhs_arte : check PART_TU '
c         K = 2 
c         DO I = 1, NrL-1
c            LL1 = 1 ; LR1 = 2*NNv 
c            write(*,'(1x,2(1x,I3),2x,*(1x,e12.4))')
c     $           I, K, MINVAL( ADO%PART_TU(I,LL1:LR1,K),1 ),
c     $           MAXVAL( ADO%PART_TU(I,LL1:LR1,K),1 )
c     ENDDO
c      DO I = 1, NrL-1
c         LL1 = 1 ; LR1 = 2*NNv 
c         write(*,'(1x,I3,2x,F8.3,*(1x,e12.4))')
c     $        I, ZZZ(I), 
c     $        (SUM( ADO%PART_TU(I,LL1:LR1,K) ),K = 1, N_Adj) 
c      ENDDO
c      Pause
      SELECT CASE( FLAG )
      CASE('A','a') 
         DO K = 1,N_Adj
            SVEC_AG(1:NNv,K+SDVI) = -ADO%PART_TU(1,1:NNv,K)
        ENDDO
      CASE('O','o') 
         SELECT CASE( TRIM(UDB%UBC) )
         CASE('Flat water')
            STOP '  "rhs_arte.f" - "Flat water" not implemented '
            DO K = 1,N_Adj       ! Scalar case only
               SVEC_AG(1:NNv,K+SDVI) = -ADO%PART_TU(1,1:NNv,K)
c              ... Fre_R has to be redefined and recalculated for LS angles
     $              + UDB%Fre_R(:)*ADO%PART_TU(1,NNv+1:2*NNv,K)
            ENDDO
         CASE('Rough water')
            WRITE(*,*) '  Rough water is used for pseudo-forward'
            DO K = 1,N_Adj
               DO K1 = 1,NNv
                  SVEC_AG(K1,K+SDVI) = -ADO%PART_TU(1,K1,K)
     $                 + SUM( UDB%Rw_GG(K1,:,Mh)
     $                 *ADO%PART_TU(1,NNv+1:2*NNv,K)
     $                 *AXM(1:NN)/XM(1:NN) )*2.d0*Pie !  AG !
               ENDDO
            ENDDO
         END SELECT
      END SELECT

      DO K = 1,N_Adj
         DO I = 1,NrL - 2
            LL1 = (2*I-1)*NNv + 1 ; LR1 = (2*I+1)*NNv
            SVEC_AG(LL1:LR1,K+SDVI) = ADO%PART_TU(I+1,:,K)
     $           - PART_TD(I,:,K)
         END DO
      ENDDO

      LL1 = (2*NrL-3)*NNv+1 ; LL2 = 2*NNv*(NrL-1)
c     ... Particular solution in boundary layer
      Pps => PART_TD(NrL-1,:,:)
      FORALL( K = 1:N_Adj ) 
     $     SVEC_AG(LL1:LL2,K+SDVI) = -Pps(NNv+1:2*NNv,K)

c     ... Change sign of U and V component in the right-hand side
c         of the linear algebraic system
      IF( Nvs>1 ) THEN
         Ka = SDVI+1 ; Ke = SDVI+N_Adj
         DO K = 1,NN
            DO K1 = 4*K-2,4*K-1
               SVEC_AG(K1+LL1,Ka:Ke) = -SVEC_AG(K1+LL1,Ka:Ke)
            ENDDO
         ENDDO
      ENDIF
c     ================================================================
c     R.h.s of the matrix equation in the case of LAMBERT REFLECTION
c                      or BRDF  (Adjoint RTE)
c     ================================================================
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
c     ==================================================================
         Alb = UDB%ALBEDO
         LBC: Block
           IF( Mh > 1 ) EXIT LBC
           SELECT CASE( TRIM(RTM_TYPE) )
           CASE('spher_scat')
c          -------------------------------------------------------------
c             ... NLSA is used for actual input values
              DO Jv = 1, GSL%NLSA  ;  Jvs = Jv*NFA - LW_FOV(Jv)
                 MLI = M_LIM(Jvs)
c                ... Contribution of direct pseudo-forward radiation
c                    GSL%EXL(NL,Jv) = 0.d0 in the case of limb LoS
                 R_DIR = 2.d0*ALB*GSL%EXL(NL,Jv)
                 K1 = SLC(Jv)*NS_wf
                 DO JL = 1, MLI      ;  K1 = K1 + 1
                    DO Jw = 1,NS_wf  ;  Jw1 = (Jw-1)*MLI + K1
                       S2 = 2.d0*ALB*SUM( Pps(1:NNv:Nvs,Jw1)*AXM(1:NN) )
                       IF( Nc_der(Jw)==1.AND.JL==MLI ) S2 = S2 + R_DIR
                       DO J = LL1,LL2,Nvs
                          SVEC_AG(J,Jw1+SDVI) = SVEC_AG(J,Jw1+SDVI) + S2
                       ENDDO
                    ENDDO  ! over number of Stokes WF (Jw)
                 ENDDO  ! over pp-approximations (JL)
              ENDDO  ! over linr-of-sight (Jv)

           CASE DEFAULT  !!! 'pp_scat' and 'ps_scat' !!!
c          -------------------------------------------------------------
              DO Jv = 1,NLSA
                 R_DIR = 0.d0
                 IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) R_DIR = 
     $                2.d0*ALB*GSL%EXL(NL,Jv)
                 DO Jw = 1,NS_wf  ;  Jw1 = NS_wf*(Jv-1) + Jw
                    S2 = 2.d0*ALB*SUM( Pps(1:NNv:Nvs,Jw1)*AXM(1:NN))
c                   ... IF WFs for first Stokes component ...........
                    IF( Nc_der(Jw)==1 ) S2 = S2 + R_DIR 
                    DO J = LL1,LL2,Nvs
                       SVEC_AG(J,Jw1+SDVI) = SVEC_AG(J,Jw1+SDVI) + S2
                    ENDDO
                 ENDDO  ! over Stokes component derivatives (Jw,Jw1)
              ENDDO  ! over linr-of-sight (Jv)
c          -------------------------------------------------------------
           END SELECT
         END Block LBC

c     ==================================================================
      CASE('Bidirect')
         SELECT CASE( TRIM(RTM_TYPE) )
         CASE('spher_scat')
c        ---------------------------------------------------------------
c           ... NLSA is used for actual input values
            DO Jv = 1, GSL%NLSA   ;  Jvs = Jv*NFA-LW_FOV(Jv)
               MLI = M_LIM(Jvs)   ;  Kli = SLC(Jv)*NS_wf
               R_DIR = 2.d0*GSL%EXL(NL,Jv)
               DO JL = 1, MLI 
                  DO Jw = 1,NS_wf ;  Jw1 = (Jw-1)*MLI + Kli + JL
                     DO K = 1,Nvs 
                        HDU(K:NNv:Nvs) = Pps(K:NNv:Nvs,Jw1)*AXM(1:NN)
                     ENDDO ! over Stokes components (K)
                     DO K = 1,NNv 
                        S2_BRF(K) = 2.d0
     $                       *SUM(UDB%BRF_GG(K,:,Mh)*HDU(1:NNv))
                     ENDDO  ! over Gaussian angles
                     IF( R_DIR > 0.d0 .AND. JL==MLI) THEN
                        Zn = 1.d0
                        IF( Nc_der(Jw) > 2 ) Zn = -1.d0
                        INI = Nvs*(SDVI+Jv-1) + Nc_der(Jw)
c                       ... Contribution of direct pseudo-forward radiation
                        S2_BRF(:) = S2_BRF(:)
     $                       + R_DIR*Zn*UDB%BRF_GS(1:NNv,INI,Mh)
                     ENDIF
                     SVEC_AG(LL1:LL2,Jw1+SDVI) = 
     $                    SVEC_AG(LL1:LL2,Jw1+SDVI) + S2_BRF(1:NNv)
                  ENDDO  ! over number of Stokes derivatives(Jw, Jw1)
               ENDDO  ! over sub-intervals along LoS (JL)
            ENDDO  ! over line-of-sight (Jv)

         CASE('pp_scat','ps_scat')
c        ---------------------------------------------------------------
c            write(*,*) '   rhs_adj pp_scat '
c            WRITE(*,*) '      NS_wf  ', NS_wf            
            DO Jv = 1,NLSA  ;  R_DIR = 0.d0
c               WRITE(*,*) '  GeoD%LVG(IA,Jv)%C_VA(1) ', Jv, 
c     $              GeoD%LVG(IA,Jv)%C_VA(1)
               IF( GeoD%LVG(IA,Jv)%C_VA(1)>0.d0 ) R_DIR = GSL%EXL(NL,Jv)
               DO Jw = 1,NS_wf  ;  Jw1 = NS_wf*(Jv-1) + Jw
                  DO K = 1,Nvs        
                     HDU(K:NNv:Nvs) = Pps(K:NNv:Nvs,Jw1)*AXM(1:NN)
                  ENDDO  ! over Stokes components (K)
                  DO K = 1,NNv 
                     S2_BRF(K) = SUM( UDB%BRF_GG(K,:,Mh)*HDU(1:NNv) )
                  ENDDO
                  IF( R_DIR > 0.d0 ) THEN
                     Zn = 1.d0
                     IF( Nc_der(Jw) > 2 ) Zn = -1.d0
                     INI = Nvs*(SDVI+Jv-1) + Nc_der(Jw)
c                    ... Contribution of direct pseudo-forward radiation
c                     WRITE(*,*) '  Mh, INI for BRF_SS ', Mh, INI
                     S2_BRF(:) = S2_BRF(:)
     $                    + R_DIR*Zn*UDB%BRF_GS(1:NNv,INI,Mh)
                  ENDIF
                  SVEC_AG(LL1:LL2,Jw1+SDVI) = SVEC_AG(LL1:LL2,Jw1+SDVI)
     $                 + 2.d0*S2_BRF(1:NNv) 
               ENDDO  ! over Stokes component derivatives (Jw)
            ENDDO  ! over LoS (Jv)
         END SELECT
c        ---------------------------------------------------------------
c     This is option is relevant for coupled ocean-atmosphere version
      CASE('Rough water')
         WRITE(*,*) '    -----> Option "Rough water" '
c     !!!  Need correction as for "Bidirect" above !!!
         DO I = 1,N_Adj
            DO K = 1,Nvs        ! for all Stokes components
               HDU(K:NNv:Nvs) = 
     $              Pps(K:NNv:Nvs,I)*AXM(1:NN)/XM(1:NN) ! AG !
            ENDDO
            DO K = 1,NNv  ! First dimension of Ra_GG is NN*Nvs
               S2_BRF(K) = SUM( UDB%Ra_GG(K,:,Mh)*HDU(1:NNv) )
            ENDDO
            K = 1  
            DO J = LL1,LL2  ! all elements for vector case
               SVEC_AG(J,I+SDVI)= SVEC_AG(J,I+SDVI) + 2.d0*Pie*S2_BRF(K)
               K = K + 1
            ENDDO
         ENDDO
         IF( GeoD%View(IA)%S=='up' ) THEN
            Stop ' "rhs_arte.f" line 949. Need to be checked'
            DO I = 1,N_Adj
               K = 1  ;  INI = Nvs*(SDVI+I-1) + Nc_der(1)
               DO J = LL1,LL2   ! all elements for vector case
                  SVEC_AG(J,I+SDVI) = SVEC_AG(J,I+SDVI) 
     $                 + 2.d0*Pie*UDB%Ra_GS(K,INI,Mh)*GSL%EXL(NL,I)
                  K = K + 1
               ENDDO
            ENDDO
         ENDIF
      END SELECT

      CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE RHS_ARTE

c **********************************************************************
      SUBROUTINE TRANS_WITHOUT_SCAT
      USE BRDF,    ONLY: X_brf, Exact_brf, Npg
      USE ST_VARS, ONLY: INT_OUT
c     ------------------------------------------------------------------

      INTEGER      Ivs, LL1, K, Ks
      REAL (KND)   S2, Cazm, Sazm
      REAL (KND)   Fs_INT(Nvs*NLSA), Tls(Kvs*NSZA,Nvs*NLSA),
     $     Ref_dir(NL,Kvs*NSZA,Nvs*NLSA), HDU(Nvs*NGb)
c     ==================================================================

      DO Js = 1,NSZA
         S2 = VST*MuS(Js)*SSB%EXS(NL,Js) ; Ks = (Js-1)*Nvs
         IF( LOS_DEF_FLAG == 'ga' ) THEN
            DO Ivs = 1,Kvs
c              ... Reflection of direct "solar" light from the surface
c                  at all Gaussian angles which are equal to viewing
c                  angles for this mode
               DO K = 1,Nvs*NN
                  HDU(K) = 
     $                 SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
               ENDDO
               Tls((Ivs-1)*NSZA+Js,:) = S2*HDU(:)
            ENDDO  ! over DO-solutions (Ivs)
         ELSE
            DO Ivs = 1,Kvs
c              ... Reflection of direct "solar" light from the surface
               IF( Exact_brf ) THEN
                  K1 = 1
                  DO K = Nvs*NN+1, Nvs*(NN+Npg)
                     Fs_INT(K1) = 
     $                    SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
                     K1 = K1 + 1
                  ENDDO
               ELSE
                  DO K = 1,Nvs*NGb
                     HDU(K) = 
     $                    SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
                  ENDDO
                  DO K1 = 1,Nvs
                     CALL AKIMA_INT( NGb, X_brf, HDU(K1:Nvs*NGb:Nvs),
     $                    NLSA, Muv, Fs_INT(K1:Nvs*NLSA:Nvs) )
                  ENDDO         ! over Stokes components (K1)
               ENDIF
               Tls((Ivs-1)*NSZA+Js,:) = S2*Fs_INT(:)
            ENDDO               ! over DO-solutions (Ivs)
         ENDIF                  ! 'ga'
      ENDDO                     ! over solar angles (Js)

      Ref_dir(NL,:,:) = Tls(:,:)
      DO I = NL-1,1,-1
         DO Jv = 1,NLSA
            S2 = DEXP(-DTR(I+1)/Muv(Jv))
            DO K1 = (Jv-1)*Nvs+1,Jv*Nvs 
               Tls(:,K1) = Tls(:,K1)*S2
            ENDDO               ! over Stokes components (K1)
         ENDDO                  ! over viewing angles (Jv)
         Ref_dir(I,:,:) = Tls(:,:)
      ENDDO                     ! over layers (I)

      J = 0
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
c         WRITE(*,*) '          Trans_without_scatt scalar  ------>'
         DO I = 1,NSZA
            DO K = JV_S(I),JV_E(I)
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  DO Jo = 1,N_OP
                     RADOUT(Jo,J,1) = Ref_dir(NOA(Jo),I,K)*E_W
     $                    *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      CASE( 'vect_rte' )
c         WRITE(*,*) '          Trans_without_scatt  vector ------>'
         DO Js = 1,NSZA
            DO K = JV_S(Js),JV_E(Js)  ;  LL1 = (K-1)*Nvs
               DO L = 1,N_AZ_VAR  ;  J = J + 1
                  Cazm = 0.5d0*DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  Sazm = 0.5d0*DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )*E_W
                  DO Jo = 1,N_OP
                     RADOUT(Jo,J,1:2) =  
     $                    Cazm*Ref_dir(NOA(Jo),Js,LL1+1:LL1+2)
                     RADOUT(Jo,J,3:4) = 
     $                    Sazm*Ref_dir(NOA(Jo),Js,LL1+3:LL1+4)
                     IF( Kvs==1 ) CYCLE

c                    ... Add second DO-solution
                     RADOUT(Jo,J,1:2) = RADOUT(Jo,J,1:2) 
     $                    - Sazm*Ref_dir(NOA(Jo),NSZA+Js,LL1+1:LL1+2)
                     RADOUT(Jo,J,3:4) = RADOUT(Jo,J,3:4) 
     $                    + Cazm*Ref_dir(NOA(Jo),NSZA+Js,LL1+3:LL1+4)
                  ENDDO         ! over instrument positions
               ENDDO            ! over azimuthal angles
            ENDDO               ! over zenith angles
         ENDDO                  ! over solar angles
      END SELECT
      END SUBROUTINE TRANS_WITHOUT_SCAT

c **********************************************************************
      SUBROUTINE MAKE_PHASE_SCAL

      USE INFO,    ONLY: TG_INFO
      USE WF_NUM,  ONLY: PERTURB
c     ------------------------------------------------------------------
c     ... Local parameters
      INTEGER  Ka, Jt, Jb, Jr, Im, Jrr
      CHARACTER (LEN=4) NAM_MH
      REAL (KND)  Aef, DUM, Phi(Leg), ISO_MOMS(Nlg_Ray), H3(3), H1(NL),
     $     Der_int(Leg)
      TARGET      ISO_MOMS
      REAL (KND), POINTER :: Php(:)
c     ==================================================================

      IF(LOG_LEVEL(1)>=N_ESO) WRITE(LUNIT,*) ' *** MAKE_PHASE_SCAL'

      SELECT CASE (FLAG)
      CASE('A','a')
c     ------------------ Rayleigh --------------------------------------
         Pm => LOA%RAY_MOMS(:,1) 
         CALL PHASE_FUNCTIONS
     $        (Mh, NN, NSZA, NLSA, Pm, Nlg_Ray, Ph_R, 'A')
         IF( Mh <= 3 ) CALL MULT_R_ST(SCR, Ph_R, Ph_z, 1, NL, NL)
c     ------------------------------------------------------------------
c     ------------------ Metals/Ions -----------------------------------
         IF( Emi_FLG.AND.(Mh <= 3) ) THEN
            DO Ka = 1, N_AT
               H3(:) = LOA%EPF(Ka,:,1)
c               WRITE(*,*) '  PH_net  ', LOA%EPF(Ka,:,1)
               CALL PHASE_FUNCTIONS 
     $              (Mh, NN, NSZA, NLSA, H3, Nlg_Ray, Ph_1d, 'A')
               H1(:) = LOA%EMC(Ka,:)
               DO I = 1,NL
                  Ph_z%Egg(I,:,:,:)=Ph_z%Egg(I,:,:,:) + H1(I)*Ph_1d%Egg
                  Ph_z%EgL(I,:,:,:)=Ph_z%EgL(I,:,:,:) + H1(I)*Ph_1d%EgL
                  Ph_z%Egs(I,:,:,:)=Ph_z%Egs(I,:,:,:) + H1(I)*Ph_1d%Egs
                  Ph_z%EsL(I,:,:,:)=Ph_z%EsL(I,:,:,:) + H1(I)*Ph_1d%EsL
               ENDDO
c               WRITE(*,*) '   - 2  '
               Ph_M%MET(Ka)%Egg(:,:,:) = Ph_1d%Egg(:,:,:)
               Ph_M%MET(Ka)%Egl(:,:,:) = Ph_1d%Egl(:,:,:)
               Ph_M%MET(Ka)%Egs(:,:,:) = Ph_1d%Egs(:,:,:)
               Ph_M%MET(Ka)%EsL(:,:,:) = Ph_1d%EsL(:,:,:)
            ENDDO  ! over atoms (Ka)
         ENDIF  ! atoms emission
c     ------------------------------------------------------------------
c     ------------------ Raman -----------------------------------------
c        ... If discrete-ordinate technique is selected to calculate
c            rotational Raman scattering, then here the energy loss due 
c            to the total Raman scattering should be subtract at actual 
c            wavelength
c         Logi = RRS_DO.OR.RRS_ADJ
c         IF( Logi.AND.SRR%ACT_WL(JPOS).AND.Mh<=3 ) THEN
         IF( RRS_DO.AND.SRR%ACT_WL(JPOS).AND.Mh<=3 ) THEN
            DO I = 1,NL
c              ... This is total Raman scattering coefficient at actual WL
               DUM = SRR%R_TOT*SCR(I)
               Ph_z%Egg(I,:,:,:) = Ph_z%Egg(I,:,:,:) 
     $              - DUM*Ram_Egg(:,:,:,Mh) !/2.d0
               Ph_z%EgL(I,:,:,:) = Ph_z%EgL(I,:,:,:) 
     $              - DUM*Ram_EgL(:,:,:,Mh)
               Ph_z%Egs(I,:,:,:) = Ph_z%Egs(I,:,:,:) 
     $              - DUM*Ram_Egs(:,:,:,Mh)
               Ph_z%EsL(I,:,:,:) = Ph_z%EsL(I,:,:,:) 
     $              - DUM*Ram_EsL(:,:,:,Mh)
            ENDDO
         ENDIF
c     ------------------------------------------------------------------
c     ----------------  Aerosol ----------------------------------------
         IF( DO_AEROSOL ) THEN
            DO J = 1,LA_N
               IF( LA_S(J) == 0 ) CYCLE
               Pm => LOA%AER_MOMS(:,J,1)
               CALL PHASE_FUNCTIONS
     $              (Mh, NN, NSZA, NLSA, Pm, Leg, Ph_1d, 'A')
               CALL MULT_R_ST(SCA, Ph_1d, Ph_z, LA_S(J), LA_E(J), NL)
c               WRITE(*,*) J, LA_S(J), LA_E(J), SCA(LA_S(J))
c              ... No weighting function for aerosol
               IF( .NOT.INQ_WF('aer', 'WFT') ) CYCLE ! ===> 
                 Ph_A%Egg(J,:,:,:) = Ph_1d%Egg
                 Ph_A%Egs(J,:,:,:) = Ph_1d%Egs
                 IF( PP_SPH ) CYCLE ! ===> 
                   Ph_A%Egl(J,:,:,:) = Ph_1d%Egl
                   Ph_A%EsL(J,:,:,1) = Ph_1d%EsL(:,:,1)
            ENDDO  ! End do loop over aerosol layers
            
c           ... Derivatives of phase function expansion coefficients are
c               used if aerosol Mie-mode is active
            IF( Dwr_mi ) THEN
               DO J = 1,LA_N
                  DO Idr = 1,N_dwr
                     Pm => LOA%ADR_MOMS(:,J,1,Idr)
                     CALL PHASE_FUNCTIONS
     $                    (Mh, NN, NSZA, NLSA, Pm, Leg, Ph_1d,'A')
                     Ph_dA%Egg(J,:,:,:,Idr) = Ph_1d%Egg
                     Ph_dA%Egs(J,:,:,:,Idr) = Ph_1d%Egs
                     IF( PP_SPH ) CYCLE ! ===> 
                       Ph_dA%Egl(J,:,:,:,Idr) = Ph_1d%Egl
                       Ph_dA%EsL(J,:,:,1,Idr) = Ph_1d%EsL(:,:,1)
                  ENDDO
               ENDDO  ! over aerosol layers (J)
            ENDIF
         ENDIF
c     ------------------------------------------------------------------
c        ... Cloud phase function: scalar case
         IF( DO_CLOUD ) THEN
c            write(*,*) '     ====> solv_dom  Cloud phase function '
            DO J = 1,N_CLD_LAYERS
               Jt = J_TOP(J)+1 ; Jb = J_BOT(J)-1
               IF( MLCS(J)%Phm_co ) THEN
c              ... Constant phase function within this cloud layer:
c                  ------------------------------------------------
                  DO Jmc = 1, Nm_ice
                     Pm => LOA%CLD_MOMS(:,J,1,Jmc,1)
                     CALL PHASE_FUNCTIONS
     $                    (Mh, NN, NSZA, NLSA, Pm, Leg, Ph_1d, 'A')
                     CALL MULT_R_ST(LOA%CSM(Jmc,:),Ph_1d,Ph_z,Jt,Jb,NL)
c                    ... No weighting function for clouds
                     IF( .NOT.INQ_WF('cld', 'WFT') )
     $                    CYCLE  ! over water/ice phase (Jmc)
                     Ph_C%C_GG(J,1,:,:,:,Jmc) = Ph_1d%Egg
                     Ph_C%C_GS(J,1,:,:,:,Jmc) = Ph_1d%Egs
                     IF( PP_SPH ) CYCLE ! ===> 
                       Ph_C%C_GL(J,1,:,:,:,Jmc) = Ph_1d%Egl
                       Ph_C%C_LS(J,1,:,:,Jmc)   = Ph_1d%EsL(:,:,1)
                  ENDDO ! over water/ice phase (Jmc)
                  CYCLE ! to next cloud layers (J)
               ENDIF  ! MLCS(Ncl)%Phm_co
c                  ------------------------------------------------------
c              ... The dependence of phase function on effective radius 
c                  is accounted for in forward MS scalar calculations
c                  ------------------------------------------------------
               KK = 2
c               write(*,*) '    Cloud layer ', J
               DO Jmc = 1, MLCS(J)%N_pha
                  N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
c                  write(*,*) '       Jmc & N_db ', Jmc, N_db
                  IF( N_db == 1 ) THEN
                     KK = KK + 2  ;  CYCLE
                  ENDIF
                  Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
c                  write(*,*) '             selected Rdb  '
c                  write(*,'(*(1x,e10.2))') Rdb(:)
c                  write(*,*) '             interpo;ate on '
c                  write(*,'(*(1x,e10.2))') Prof_Cpar(Jt:Jb,KK)
                  Jrr = 0
                  DO Jr = Jt,Jb
                     Aef = Prof_Cpar(Jr,KK)
                     IF( NUM_CLD ) THEN
                        CALL LIN_SEARCH_NEW(Aef, DL, I1, I2,
     $                       N_db, Rdb)
c                      ... Interpolation of EK derivative on the 
c                          perturbed effective radius
                        Der_int(:) = DL*LOA%DR_MOMS(:,J,I2,Jmc,1)
     $                       + (1.d0 - DL)*LOA%DR_MOMS(:,J,I1,Jmc,1)
                        Aef = Aef/PERTURB(Jr)
                        CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
c                      ... Interpolation of EK on the 
c                          non-perturbed effective radius
                        Phi(:) = LOA%CLD_MOMS(:,J,I2,Jmc,1)*DL 
     $                       + LOA%CLD_MOMS(:,J,I1,Jmc,1)*(1.d0 - DL)
c                       ... Linear perturbation of EK
                        Phi(:) =  Phi(:) +  Der_int(:)
     $                       *(Aef*PERTURB(Jr) - Aef)
                        Aef = Aef*PERTURB(Jr)
                     ELSE
                        CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
c                       ... Interpolate on a given effective radius
                        Phi(:) = LOA%CLD_MOMS(:,J,I2,Jmc,1)*DL 
     $                         + LOA%CLD_MOMS(:,J,I1,Jmc,1)*(1.d0 - DL)
c                        write(*,*) '   Phi:'
c                        write(*,'(*(1x,e10.2))') Phi(:)
                     ENDIF
                     CALL PHASE_FUNCTIONS
     $                    (Mh, NN, NSZA, NLSA, Phi,Leg, Ph_1d, 'A')
                     Ph_z%Egg(Jr,:,:,:) = Ph_z%Egg(Jr,:,:,:) 
     $                        + LOA%CSM(Jmc,Jr)*Ph_1d%Egg(:,:,:)
                     Ph_z%EgL(Jr,:,:,:) = Ph_z%EgL(Jr,:,:,:) 
     $                        + LOA%CSM(Jmc,Jr)*Ph_1d%EgL(:,:,:)
                     Ph_z%Egs(Jr,:,:,:) = Ph_z%Egs(Jr,:,:,:) 
     $                        + LOA%CSM(Jmc,Jr)*Ph_1d%Egs(:,:,:)
                     Ph_z%EsL(Jr,:,:,:) = Ph_z%EsL(Jr,:,:,:) 
     $                        + LOA%CSM(Jmc,Jr)*Ph_1d%EsL(:,:,:)
                     IF( .NOT.INQ_WF('cld','WFT') ) CYCLE ! ====>
                       Jrr = Jrr + 1
                       Ph_C%C_GG(J,Jrr,:,:,:,Jmc) = Ph_1d%Egg(:,:,:)
                       Ph_C%C_GS(J,Jrr,:,:,:,Jmc) = Ph_1d%Egs(:,:,:)
                       IF( PP_SPH ) CYCLE ! ===> 
                         Ph_C%C_GL(J,Jrr,:,:,:,Jmc) = Ph_1d%Egl(:,:,:)
                         Ph_C%C_LS(J,Jrr,:,:,Jmc)   = Ph_1d%EsL(:,:,1)
!                         Jrr = Jrr + 1
                  ENDDO  ! over sub-layers within cloud (Jr,Jrr)
                  KK = KK + 2
               ENDDO  ! over water/ice phase (Jmc)
c              ----------------------------------------------------------
            ENDDO  ! over cloud layers (J)
c           ... WFs w.r.t. effective radius accounting 
c               for contribution of phase function
            IF( Deriv_Phc ) THEN
               DO J = 1,N_CLD_LAYERS
                  Jt = J_TOP(J)+1 ; Jb = J_BOT(J)-1
                  KK = 2
                  DO Jmc = 1, MLCS(J)%N_pha
                     N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
                     Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
                     Jrr = 0  !Jrr = 1
                     DO Jr = Jt,Jb
                        Aef = Prof_Cpar(Jr,KK) 
                        CALL LIN_SEARCH_NEW(Aef, DL, I1, I2, N_db, Rdb)
                        Phi(:) = LOA%DR_MOMS(:,J,I2,Jmc,1)*DL 
     $                       + LOA%DR_MOMS(:,J,I1,Jmc,1)*(1.d0 - DL)
                        CALL PHASE_FUNCTIONS
     $                       (Mh, NN, NSZA, NLSA, Phi, Leg, Ph_1d, 'A')
                        Jrr = Jrr + 1
                        Ph_C%Dr_GG(J,Jrr,:,:,:,Jmc) = Ph_1d%Egg(:,:,:)
                        Ph_C%Dr_GS(J,Jrr,:,:,:,Jmc) = Ph_1d%Egs(:,:,:)
                        IF( PP_SPH ) CYCLE ! ===> 
                          Ph_C%Dr_GL(J,Jrr,:,:,:,Jmc) = Ph_1d%Egl(:,:,:)
                          Ph_C%Dr_LS(J,Jrr,:,:,Jmc)   = Ph_1d%EsL(:,:,1)
c                         Jrr = Jrr + 1
                     ENDDO ! over sub-layers within cloud (Jr, Jrr)
                     KK = KK + 2
                  ENDDO  ! over water/ice phase (Jmc)
               ENDDO  ! over cloud layers
            ENDIF ! derivatives of phase function w.r.t. R_ef 
         ENDIF  ! DO_CLOUD scalar case
c     ===========================================================+++++++
c     ---> end case('A' - atmosphere)

      CASE('O','o')
         Pm => LOW%RAY_MOMS_W(:,1) ; HSC => LOW%SCR_W
         CALL PHASE_FUNCTIONS
     $        (Mh, NN, NSZA, NLSA, Pm, Nlg_Ray, Ph_R, 'O')
         IF( Mh <= 3 ) CALL MULT_R_ST(HSC, Ph_R, Ph_z, 1, NL_W, NL)
c        ... Phase matrix for ice inclusions         
         IiW = 1
         DO I = 1,IWS(1)%N_HPC  ! Number of ice inclusions
c            write(*,*) '   Inclusion ', I
            HSC => LOW%HSCA(:,I,IiW)
            DO J = 1, LA_N_IW   ! Number of ice layers
c               write(*,*) '      Ice layer ', J
               IF( LA_S_IW(J) == 0 ) CYCLE  
               Pm => LOW%HSOL_MOMS(:,J,1,I,IiW)
c               write(*,*) ' Pm ', Pm(:)
               CALL PHASE_FUNCTIONS
     $              (Mh, NN, NSZA, NLSA, Pm, Leg_W, Ph_1d, 'O')

c               S1 = SUM( Ph_1d%Egg )
c               write(*,*) ' SUM( Ph_1d%Egg) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egg ) ',I, S1
c                  STOP '    ******** NaN in Ph_1d%Egg ********** '
c               ENDIF
c               S1 = SUM( Ph_1d%Egs )
c               write(*,*) ' SUM( Ph_1d%Egs) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egs ) ',I, S1
c                  STOP '    ******** NaN in Ph_1d%Egs ********** '
c               ENDIF
c               S1 = SUM( Ph_1d%Egl )
c               write(*,*) ' SUM( Ph_1d%Egl) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egl ) ',I, S1
c                  STOP '    ******** NaN in Ph_1d%Egl ********** '
c               ENDIF
c               S1 = SUM( Ph_1d%EsL )
c               write(*,*) ' SUM( Ph_1d%EsL) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( EsL ) ',I, S1
c                  STOP '    ******** NaN in Ph_1d%EsL ********** '
c               ENDIF

               CALL MULT_R_ST
     $              (HSC, Ph_1d, Ph_z, LA_S_IW(J), LA_E_IW(J), NL)

c               S1 = SUM( Ph_z%Egg )
c               write(*,*) ' SUM( Ph_z%Egg) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egg ) ',I, S1
c                  STOP '    ******** NaN in Ph_z%Egg ********** '
c               ENDIF
c               S1 = SUM( Ph_z%Egs )
c               write(*,*) ' SUM( Ph_z%Egs) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egs ) ',I, S1
c                  STOP '    ******** NaN in Ph_z%Egs ********** '
c               ENDIF
c               S1 = SUM( Ph_z%Egl )
c               write(*,*) ' SUM( Ph_z%Egl) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( Egl ) ',I, S1
c                  STOP '    ******** NaN in Ph_z%Egl ********** '
c               ENDIF
c               S1 = SUM( Ph_z%EsL )
c               write(*,*) ' SUM( Ph_z%EsL) ', S1
c               IF( ISNAN(S1) ) THEN
c                  WRITE(*,*) '   I and SUM( EsL ) ',I, S1
c                  STOP '    ******** NaN in Ph_z%EsL ********** '
c               ENDIF

            ENDDO               ! over ice layers (J)
         ENDDO  ! over ice inclusions (I)
         IF( IWT ) IiW = IiW + 1

         DO I = 1,IWS(2)%N_HPC  ! Number of hydrosol components
            HSC => LOW%HSCA(:,I,IiW)
            SELECT CASE( SEL_VSC(I) )
            CASE('Kos')
               Pm => LOW%HSOL_MOMS(:,1,1,I,IiW) 
               CALL PHASE_FUNCTIONS
     $              (Mh, NN, NSZA, NLSA, Pm, Leg_W, Ph_1d, 'O')
               CALL MULT_R_ST(HSC, Ph_1d, Ph_z, 1, NL_W, NL)
               IF( WF_FLAG ) THEN 
                  IF( OWF_Ks ) THEN
                     Ph_Ks%Egg = Ph_1d%Egg  ; Ph_Ks%Egl = Ph_1d%Egl
                     Ph_Ks%Egs = Ph_1d%Egs    
                     Ph_Ks%EsL(:,:,1)= Ph_1d%EsL(:,:,1)
                  ENDIF
               ENDIF
            CASE('Kol')
               Pm => LOW%HSOL_MOMS(:,1,1,I,IiW) 
               CALL PHASE_FUNCTIONS
     $              (Mh, NN, NSZA, NLSA, Pm, Leg_W, Ph_1d, 'O')
               CALL MULT_R_ST(HSC, Ph_1d, Ph_z, 1, NL_W, NL)             
               IF( WF_FLAG ) THEN 
                  IF( OWF_Kl ) THEN
                     Ph_Kl%Egg = Ph_1d%Egg  ; Ph_Kl%Egl = Ph_1d%Egl
                     Ph_Kl%Egs = Ph_1d%Egs    
                     Ph_Kl%EsL(:,:,1)= Ph_1d%EsL(:,:,1)
                  ENDIF
               ENDIF
            CASE('Use')
               DO J = 1, LA_N_HS ! Number of hydrosol layers
                  IF( LA_S_HS(J) == 0 ) CYCLE
                  Pm => LOW%HSOL_MOMS(:,J,1,I,IiW)
c                  WRITE(*,*) '    h-sol layer ', J
c                  WRITE(*,*) 'Depth levels ', LA_S_HS(J), LA_E_HS(J)
                  CALL PHASE_FUNCTIONS
     $                 (Mh, NN, NSZA, NLSA, Pm, Leg_W, Ph_1d,'O')
                  CALL MULT_R_ST
     $                 (HSC, Ph_1d, Ph_z, LA_S_HS(J), LA_E_HS(J), NL)
                  IF( WF_FLAG ) THEN 
                     IF( OWF_Use ) THEN
                        Ph_Us%Egg(J,:,:,:) = Ph_1d%Egg
                        Ph_Us%Egl(J,:,:,:) = Ph_1d%Egl
                        Ph_Us%Egs(J,:,:,:) = Ph_1d%Egs
                        Ph_Us%EsL(J,:,:,1) = Ph_1d%EsL(:,:,1)
                     ENDIF
                  ENDIF
               ENDDO  ! over hydrosol layers (J)
            END SELECT
         ENDDO  ! over hydrosol components (I)
c     ---> end case('O')
      END SELECT

      END SUBROUTINE MAKE_PHASE_SCAL

c ******************************************************************
      SUBROUTINE MAKE_PHASE_VECT
      USE INFO,    ONLY: TG_INFO

      INTEGER  Ka, Jt, Jb, Jr, Im, Ico
      REAL (KND)  Aef, DUM, Phi(Nlg_Cld,Nm_vs)
      REAL (KND), POINTER :: Php(:)

      IF(LOG_LEVEL(1)>=N_ESO)  WRITE(LUNIT,*) ' *** MAKE_PHASE_VECT'

      SELECT CASE (FLAG)
      CASE('A','a')
c     --------------- Rayleigh --------------------------------------
c        ... Correction 23.04.2014 VR
         IF( Mh<=3 ) THEN
            CALL PHASE_MATR
     $           (Mh, NN, NSZA, NLSA, Nlg_Ray, LOA%RAY_MOMS, Ph_r, FLAG)
            CALL MULT_R_ST(SCR, Ph_r, ph_z, 1, NL, NL)
         ELSE
            Ph_z%Egg = 0.d0 ; Ph_z%EgL = 0.d0 
            Ph_z%Egs = 0.d0 ; Ph_z%EsL = 0.d0
         ENDIF

c      ... If discrete-ordinate technique is selected to calculate
c          rotational Raman scattering, then here the energy loss due 
c          to the total Raman scattering should be subtract at actual 
c          wavelength
c       Logi = RRS_DO.OR.RRS_ADJ
c       IF( Logi.AND.SRR%ACT_WL(JPOS).AND.Mh<=3 ) THEN
         IF( RRS_DO.AND.SRR%ACT_WL(JPOS).AND.Mh<=3 ) THEN
            DO I = 1,NL
c              ... This is total Raman scattering coefficient at actual WL
               DUM = SRR%R_TOT*SCR(I)
c               DUM = 0.d0
               Ph_z%Egg(I,:,:,:)=Ph_z%Egg(I,:,:,:)-DUM*Ram_Egg(:,:,:,Mh)
               Ph_z%EgL(I,:,:,:)=Ph_z%EgL(I,:,:,:)-DUM*Ram_EgL(:,:,:,Mh)
               Ph_z%Egs(I,:,:,:)=Ph_z%Egs(I,:,:,:)-DUM*Ram_Egs(:,:,:,Mh)
               Ph_z%EsL(I,:,:,:)=Ph_z%EsL(I,:,:,:)-DUM*Ram_EsL(:,:,:,Mh)
            ENDDO  ! over altitude levels (I)
         ENDIF

         IF( Emi_FLG.AND.(Mh <= 3) ) THEN
            DO Ka = 1, N_AT
               CALL PHASE_MATR(Mh, NN, NSZA, NLSA, 
     $              Nlg_Ray, LOA%EPF(Ka,:,:), PH_M%MET(Ka), FLAG)
               CALL MULT_R_ST(LOA%EMC(Ka,:), PH_M%MET(Ka),
     $              Ph_z, 1, NL, NL)
            ENDDO
         ENDIF  ! atoms emission (vector case)

c     ... Aerosol phase matrices: vector case
         IF( DO_AEROSOL ) THEN
            DO J = 1,LA_N 
               IF( LA_S(J) == 0 ) CYCLE
               Por => LOA%AER_MOMS(:,J,:)
c              WRITE(*,*) '  Aerosol '
               CALL PHASE_MATR
     $              (Mh, NN, NSZA, NLSA, Nlg_Aer, Por, Ph_1d, FLAG)
               CALL MULT_R_ST(SCA, Ph_1d, ph_z, LA_S(J), LA_E(J), NL)
c             ... No weighting function for aerosol
               IF( .NOT.INQ_WF('aer', 'WFT') ) CYCLE
c              ... First dimension of arrays in Ph_A% structure is LA_N
                 Ph_A%Egg(J,:,:,:) = Ph_1d%Egg
                 Ph_A%Egs(J,:,:,:) = Ph_1d%Egs
                 IF( PP_SPH ) CYCLE  ! ====>
                   Ph_A%Egl(J,:,:,:) = Ph_1d%Egl
                   Ph_A%EsL(J,:,:,1) = Ph_1d%EsL(:,:,1)
            ENDDO  ! over aerosol layers (J)
         
c        ... Derivatives of phase matrix expansion coefficients are
c            used if aerosol Mie-mode is active
            IF( Dwr_mi ) THEN
               DO J = 1,LA_N
                  DO Idr = 1,N_dwr
                     Por => LOA%ADR_MOMS(:,J,:,Idr)
                     CALL PHASE_MATR
     $                   (Mh, NN, NSZA, NLSA, Nlg_Aer, Por, Ph_1d, FLAG)
c                    ... First dimension of arrays in Ph_dA% structure is LA_N
                     Ph_dA%Egg(J,:,:,:,Idr) = Ph_1d%Egg
                     Ph_dA%Egs(J,:,:,:,Idr) = Ph_1d%Egs
                     IF( PP_SPH ) CYCLE  ! ====>
                       Ph_dA%Egl(J,:,:,:,Idr) = Ph_1d%Egl
                       Ph_dA%EsL(J,:,:,1,Idr) = Ph_1d%EsL(:,:,1)
                  ENDDO
               ENDDO  ! over aerosol layers (J)
            ENDIF  ! Dwr_mi
         ENDIF  ! DO_AEROSOL

c     ... Cloud phase matrix: vector case
         IF( DO_CLOUD ) THEN 
            DO J = 1,N_CLD_LAYERS 
               Jt = J_TOP(J)+1 ; Jb = J_BOT(J)-1
               IF( MLCS(J)%Phm_co ) THEN
c              ... Constant phase function within cloud layer
c                  ------------------------------------------
                  DO Jmc = 1, Nm_ice
                     Por => LOA%CLD_MOMS(:,J,1,Jmc,:)
                     CALL PHASE_MATR(Mh, NN, NSZA, NLSA, Nlg_Cld,
     $                    Por, Ph_1d, FLAG)
                     CALL MULT_R_ST(LOA%CSM(Jmc,:),Ph_1d,ph_z,Jt,Jb,NL)
                  ENDDO  ! over water/ice phase (Jmc)
                  CYCLE  ! to next cloud layers (J)
               ENDIF
c              ... The dependence of phase function on effective radius 
c                  is accounted for in forward MS vector calculations
c                  ------------------------------------------------------
               KK = 2
               DO Jmc = 1, MLCS(J)%N_pha
                  N_db = MLCS(J)%Npf(Jmc)%Ndb_ref
                  Rdb => MLCS(J)%Npf(Jmc)%Pdb_ref(:)
                  DO Jr = Jt,Jb
                     Aef = Prof_Cpar(Jr,KK) ; Phi(1,1) = 1.d0
                     DO Im = 2,Nlg_Cld
                        Php => LOA%CLD_MOMS(Im,J,:,Jmc,1)
                        CALL LIN_INT(N_db,Rdb,Php, 1,Aef,Phi(Im,1),'L')
                     ENDDO
                     DO Ico = 2,Nm_vs
                        DO Im = 1,Nlg_Cld
                           Php => LOA%CLD_MOMS(Im,J,:,Jmc,Ico) 
                           CALL LIN_INT
     $                          (N_db,Rdb,Php, 1,Aef,Phi(Im,Ico),'L')
                        ENDDO
                     ENDDO
                     CALL PHASE_MATR
     $                   (Mh, NN, NSZA, NLSA, Nlg_Cld, Phi, Ph_1d, FLAG)
                     Ph_z%Egg(Jr,:,:,:) = Ph_z%Egg(Jr,:,:,:) 
     $                    + LOA%CSM(Jmc,Jr)*Ph_1d%Egg(:,:,:)
                     Ph_z%EgL(Jr,:,:,:) = Ph_z%EgL(Jr,:,:,:) 
     $                    + LOA%CSM(Jmc,Jr)*Ph_1d%EgL(:,:,:)
                     Ph_z%Egs(Jr,:,:,:) = Ph_z%Egs(Jr,:,:,:) 
     $                    + LOA%CSM(Jmc,Jr)*Ph_1d%Egs(:,:,:)
                     Ph_z%EsL(Jr,:,:,:) = Ph_z%EsL(Jr,:,:,:) 
     $                    + LOA%CSM(Jmc,Jr)*Ph_1d%EsL(:,:,:)
                  ENDDO   ! End over sub-layers within a cloud layer
                  KK = KK + 2
               ENDDO  ! over water/ice phase (Jmc)
c           -------------------------------------------------------------
            ENDDO  !  over cloud layers (J)
         ENDIF  ! DO_CLOUD

c     ---> end case('A' - atmosphere)
      CASE('O','o')
         HSC => LOW%SCR_W
         CALL PHASE_MATR (Mh, NN, NSZA, NLSA, 
     $        Nlg_Ray, LOW%RAY_MOMS_W, Ph_r, FLAG)
         IF( Mh<=3 ) CALL MULT_R_ST(HSC, Ph_r, ph_z, 1, NL_W, NL) ! NL=NL_W
         IiW = 1
         DO I = 1,IWS(1)%N_HPC
            HSC => LOW%HSCA(:,I,IiW)
            DO J = 1, LA_N_IW
               IF( LA_S_IW(J)==0 ) CYCLE
               Por => LOW%HSOL_MOMS(:,J,:,I,IiW) 
               CALL PHASE_MATR
     $              (Mh, NN, NSZA, NLSA,  Leg_W, Por, Ph_1d, FLAG)
               CALL MULT_R_ST
     $              (HSC, Ph_1d, Ph_z, LA_S_IW(J), LA_E_IW(J), NL)
            ENDDO  ! over ice layers (J)
         ENDDO  ! over ice inclusions (I)
         IF( IWT ) IiW = IiW + 1

         DO I = 1,IWS(2)%N_HPC
            HSC => LOW%HSCA(:,I,IiW)
            SELECT CASE( SEL_VSC(I) )
            CASE('Kos')
               Por => LOW%HSOL_MOMS(:,1,:,I,IiW)
               CALL PHASE_MATR
     $              (Mh, NN, NSZA, NLSA, Leg_W, Por, Ph_1d, FLAG)
               CALL MULT_R_ST
     $              (HSC, Ph_1d, ph_z, 1, NL_W, NL)
            CASE('Kol')
               Por => LOW%HSOL_MOMS(:,1,:,I,IiW)
               CALL PHASE_MATR
     $              (Mh, NN, NSZA, NLSA, Leg_W, Por, Ph_1d, FLAG)
               CALL MULT_R_ST
     $              (HSC, Ph_1d, ph_z, 1, NL_W, NL)
            CASE('Use')
               DO J = 1, LA_N_HS ! Number of hydrosol layers
                  IF( LA_S_HS(J)==0 ) CYCLE
                  Por => LOW%HSOL_MOMS(:,J,:,I,IiW)
                  CALL PHASE_MATR
     $                 (Mh, NN, NSZA, NLSA, Leg_W, Por, Ph_1d, FLAG)
                  CALL MULT_R_ST
     $                 (HSC, Ph_1d, ph_z, LA_S_HS(J), LA_E_HS(J), NL)
               ENDDO  ! over hydrosol layers (J)
            CASE DEFAULT
               WRITE(*,*) ' In the vector case implemented  '
               WRITE(*,*) '  the option "User_supplied"  '
               WRITE(*,*) 'hydrosol angular scattering only. '
               STOP '     Stop in "solv_dom.f"  '
            END SELECT
         ENDDO  ! over hydrosol components (I)
c     ---> end case('O')
      END SELECT

      END SUBROUTINE MAKE_PHASE_VECT
c ***************************************************************************

      END
c **********************************************************************
      SUBROUTINE MULT_R_ST( RA, ST1, STZ, Ns, Nf, Ndi)
      USE TYPES,   ONLY: KND, PH_MET

      INTEGER,    INTENT(IN) :: Ns, Nf, Ndi
      REAL (KND), INTENT(IN) :: RA(Ndi)
      TYPE (PH_MET) :: ST1
c     ------------------------------------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: STZ
c     -------------------------------------------------------------
      INTEGER I

      DO I = Ns,Nf
         STZ%Egg(I,:,:,:) = STZ%Egg(I,:,:,:) + RA(I)*ST1%Egg
         STZ%EgL(I,:,:,:) = STZ%EgL(I,:,:,:) + RA(I)*ST1%EgL
         STZ%Egs(I,:,:,:) = STZ%Egs(I,:,:,:) + RA(I)*ST1%Egs
         STZ%EsL(I,:,:,:) = STZ%EsL(I,:,:,:) + RA(I)*ST1%EsL
      ENDDO

      RETURN
      END SUBROUTINE MULT_R_ST
