!***************************************************************************
!
!	SCIATRAN - Version 3.8 - Last modification: 02 Feb 2017
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE SOLVE_DIFF_EQ (DEP_CF, A_C_1, A_C_2, ASYM_COEFF, ALBEDO, AL_PC, &
     SCA, SCR, LocArrs, NRHS, RAD_PS, WFPS, C_G, OSM, FLG_WF)
  ! Called by "ps_rad.f90"
      
  USE AEROSOLS,    ONLY: IDX_AER
  USE BRDF,        ONLY: BRDF_PARAM
  USE BRDF_SUBRS,  ONLY: BRDF_ACT
  USE GRIDS,       ONLY: NLAY => NLAYERS, ZZZ, DZR, Z_FINE, NL_FINE, INTERP_IDX
  USE INTERN_DIMS, ONLY: N_MOM => NLG_ALL_A
  USE NODES,       ONLY: N_NODE, MJU_NODE, WEIGHTS, PHI_NODE, N_MINUS, &
                         MJU_MINUS, PHI_MINUS, N_PLUS
  USE ST_INPUT,    ONLY: APH_TYPE
  USE ST_VARS,     ONLY: BRDF_FLAG
  USE TYPES,       ONLY: KND, OSM_TYPE, WFS_PS, FD_LOCAL

  INTEGER  NRHS
  LOGICAL  FLG_WF
  REAL (KND), INTENT(IN)::  ALBEDO, DEP_CF(3)
  REAL (KND), DIMENSION (NLAY) :: A_C_1, A_C_2, SCA, SCR, AL_PC, ASYM_COEFF
  REAL (KND)  C_G(N_NODE,N_NODE)
  REAL (KND)  RAD_PS(:,:,:)
  TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
  TYPE (OSM_type) :: OSM 
  TYPE (FD_LOCAL) :: LocArrs(NRHS)

  ! Local variables
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SOLVE_DIFF_EQ'
  INTEGER       I, J, IDX_ROW, IDX_COL, K, L, INFO, IS, MDim, I_ND
  INTEGER       KU, KL, IDX_LEVEL, IDX_BLOCK, II_1, II_2, IDX_NODE
  INTEGER       IPIV(NLAY*N_NODE)

  REAL (KND)    PHASE_FUNC(NLAY, N_NODE, N_NODE), W_1(N_NODE)
  REAL (KND)    RHS(N_NODE*NLAY,NRHS), RAY_PF(N_NODE,N_NODE), Pol(N_MOM,1)
  REAL (KND)    S_EXP(NLAY,N_NODE), SURF_REFL(N_PLUS, MAX(N_MINUS,NRHS))
  REAL (KND)    S, DELTA
  REAL (KND), ALLOCATABLE :: AB(:,:), RHS_WF(:,:)
  REAL (KND), POINTER :: COF(:,:)
  !==================================================================
  INTERFACE
     SUBROUTINE RHS_WF_FORM  (AL_PC, LocArrs, S_EXP, RHS, PHASE_FUNC, SURF_REFL, &
          RHS_WF)
       USE NODES, ONLY : N_NODE, N_PLUS
       USE GRIDS, ONLY : NLAY=>NLAYERS
       USE TYPES,  ONLY: KND, FD_LOCAL
       REAL (KND)  AL_PC(NLAY), S_EXP(NLAY,N_NODE), RHS(N_NODE*NLAY)
       REAL (KND)  PHASE_FUNC(NLAY, N_NODE, N_NODE), SURF_REFL(N_PLUS)
       REAL (KND)  RHS_WF(NLAY*N_NODE, NLAY)
       TYPE (FD_LOCAL) :: LocArrs
     END SUBROUTINE RHS_WF_FORM
  END INTERFACE
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  KU = 2*N_NODE - 1 ;  KL = 2*N_NODE - 1 ;   MDim = 2*KL + KU + 1
  ALLOCATE(AB(MDim,NLAY*N_NODE))
  
  W_1(:) = WEIGHTS(:)/DOT_PRODUCT(WEIGHTS(N_PLUS + 1:),MJU_NODE(N_PLUS + 1:))

  S = DLOG(HUGE(1.0D0))/1.0D+1
  FORALL(J = 1:NLAY - 1,K = 1:N_PLUS) S_EXP(J,K) = DEXP( &
       DMIN1(S,(AL_PC(J) + AL_PC(J + 1))/2.0*DZR(J)/MJU_NODE(K)))

  FORALL(J = 2:NLAY,K = N_PLUS + 1:N_NODE)  S_EXP(J,K) = &
       DEXP(DMIN1(S,- (AL_PC(J - 1) + AL_PC(J))/2.0*DZR(J - 1)/MJU_NODE(K)))

  IF( APH_TYPE=='mie' ) THEN
     ! Legendre polynomials for scattering angles N_node*N_node
     COF => OSM%AER_ALL(:,:,1)
     DO K = 1, N_NODE
        DO L = 1, K
           CALL WIGNER_PLG(C_G(K,L), 1, 0, N_MOM, Pol)
           DO J = 1, NLAY 
              PHASE_FUNC(J,K,L) = SUM( Pol(:,1)*COF(:,IDX_AER(J)) )
           ENDDO
        ENDDO
     ENDDO
  ELSE
     FORALL(K = 1:N_NODE, L = 1:N_NODE, L <= K) PHASE_FUNC(:,K,L) = &
          A_C_2(:)/(A_C_1(:) - ASYM_COEFF(:)*C_G(K,L))/ &
          SQRT(A_C_1(:) - ASYM_COEFF(:)*C_G(K,L))
  ENDIF

  FORALL(K = 1:N_NODE, L = 1:N_NODE, L <= K) RAY_PF(K,L) = &
         (DEP_CF(1) + DEP_CF(2)*C_G(K,L)*C_G(K,L))
  
  FORALL(K = 1:N_NODE, L = 1:N_NODE, L > K)
     PHASE_FUNC(:,K,L) = PHASE_FUNC(:,L,K)
     RAY_PF(K,L) = RAY_PF(L,K)
  END FORALL

  FORALL(K = 1:N_NODE) RAY_PF(:,K) = WEIGHTS(:)*RAY_PF(:,K)/&
       DOT_PRODUCT(WEIGHTS(:),RAY_PF(:,K))

  FORALL(IDX_LEVEL = 1:NLAY, K = 1:N_NODE) PHASE_FUNC(IDX_LEVEL,:,K) = &
       WEIGHTS(:)*PHASE_FUNC(IDX_LEVEL,:,K)/&
       DOT_PRODUCT(WEIGHTS(:),PHASE_FUNC(IDX_LEVEL,:,K))

  FORALL(IDX_LEVEL = 1:NLAY) PHASE_FUNC(IDX_LEVEL,:,:) = &
       (PHASE_FUNC(IDX_LEVEL,:,:)*SCA(IDX_LEVEL) + &
       RAY_PF(:,:)*SCR(IDX_LEVEL))/2.0D0/AL_PC(IDX_LEVEL)

  AB = 0.0D0
  
  ! Boundary conditions for the downwards travelling radiation at TOA
  ! *****************************************************************
  IDX_LEVEL = 1             ! Row-blocks
  IDX_BLOCK = 1             ! Column-blocks
  
  DO IDX_ROW = N_PLUS + 1, N_NODE
     I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
     J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
     AB(KL + KU + I - J + 1,J) = 1.0D0
  END DO

  ! Blocks containing the multiple scattering source function
  ! *********************************************************
  DO IDX_LEVEL = 1, NLAY - 1  
     DO IDX_ROW = 1, N_PLUS
        I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
        S = S_EXP(IDX_LEVEL,IDX_ROW) - 1.0
        DO IDX_BLOCK = IDX_LEVEL, IDX_LEVEL + 1
           DO IDX_COL = 1, N_NODE
              J = (IDX_BLOCK - 1)*N_NODE + IDX_COL
              AB(KL + KU + I - J + 1,J) = &
                   PHASE_FUNC(IDX_BLOCK,IDX_COL,IDX_ROW)*S
           END DO
        END DO
     END DO
  END DO

  DO IDX_LEVEL = 2, NLAY  
     DO IDX_ROW = N_PLUS + 1, N_NODE
        I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
        S = 1.0 - S_EXP(IDX_LEVEL,IDX_ROW)
        DO IDX_BLOCK = IDX_LEVEL - 1, IDX_LEVEL
           DO IDX_COL = 1, N_NODE
              J = (IDX_BLOCK - 1)*N_NODE + IDX_COL
              AB(KL + KU + I - J + 1,J) = &
                   PHASE_FUNC(IDX_BLOCK,IDX_COL,IDX_ROW)*S
           END DO
        END DO
     END DO
  END DO

  ! Extiction in diagonal blocks
  ! ****************************
  DO IDX_LEVEL = 1, NLAY - 1  
     IDX_BLOCK = IDX_LEVEL
     DO IDX_ROW = 1, N_PLUS
        J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
        AB(KL + KU + 1,J) = AB(KL + KU + 1,J) - S_EXP(IDX_LEVEL,IDX_ROW)      
     END DO
  END DO
  
  DO IDX_LEVEL = 2, NLAY  
     IDX_BLOCK = IDX_LEVEL
     DO IDX_ROW = N_PLUS + 1, N_NODE
        J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
        AB(KL + KU + 1,J) = AB(KL + KU + 1,J) + S_EXP(IDX_LEVEL,IDX_ROW)
     END DO
  END DO
  
  ! Unity in  super- and subdiagonal blocks
  ! ***************************************
  DO IDX_LEVEL = 1, NLAY - 1  
     IDX_BLOCK = IDX_LEVEL + 1
     DO IDX_ROW = 1, N_PLUS
        I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
        J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
        AB(KL + KU + I - J + 1,J) = AB(KL + KU + I - J + 1,J) + 1.0
     END DO
  END DO

  DO IDX_LEVEL = 2, NLAY
     IDX_BLOCK = IDX_LEVEL - 1
     DO IDX_ROW = N_PLUS + 1, N_NODE
        I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
        J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
        AB(KL + KU + I - J + 1,J) = AB(KL + KU + I - J + 1,J) - 1.0
     END DO
  END DO
          
  ! Bounary conditions for the upwards travelling radiation at BOA
  ! **************************************************************
  IDX_LEVEL = NLAY             
  IDX_BLOCK = NLAY   

  IF(.NOT.BRDF_FLAG) THEN
     SURF_REFL = ALBEDO
  ELSE
     CALL BRDF_ACT (BRDF_PARAM, N_PLUS, MJU_NODE(1:N_PLUS), PHI_NODE(1:N_PLUS),&
          N_MINUS, MJU_MINUS, PHI_MINUS, SURF_REFL(:,1:N_MINUS))
  END IF

  DO IDX_ROW = 1, N_PLUS

     I = (IDX_LEVEL - 1)*N_NODE + IDX_ROW
     J = (IDX_BLOCK - 1)*N_NODE + IDX_ROW
     AB(KL + KU + I - J + 1,J) = 1.0D0
     
     DO IDX_COL = N_PLUS + 1, N_NODE
        J = (IDX_BLOCK - 1)*N_NODE + IDX_COL
        AB(KL + KU + I - J + 1,J) = -  MJU_NODE(IDX_COL)* &
             SURF_REFL(IDX_ROW, IDX_COL - N_PLUS)*W_1(IDX_COL)
     END DO
     
  END DO

  ! LU decomposition
  ! ****************
  AB = SNGL(AB)
  CALL DGBTRF (N_NODE*NLAY, N_NODE*NLAY, KL, KU, AB, MDim, IPIV, INFO)  

  ! Right-hand side vector formation
  ! ********************************

  RHS = 0.0

  DO K = 1, NRHS
     ASSOCIATE (SSSF_PL => LocArrs(K)%SSSF(:,1:N_PLUS),&
          SSSF_MI => LocArrs(K)%SSSF(:,N_PLUS + 1:N_NODE))
       DO J = 1, NLAY - 1
          RHS((J - 1)*N_NODE + 1:(J - 1)*N_NODE + N_PLUS,K) = &
               (SSSF_PL(J,:) + SSSF_PL(J + 1,:))*(S_EXP(J,1:N_PLUS) - 1.0)
       END DO
       DO J = 2, NLAY
          RHS((J - 1)*N_NODE + N_PLUS + 1:J*N_NODE,K) = (SSSF_MI(J - 1,:) + &
               SSSF_MI(J,:))*(1.0 - S_EXP(J,N_PLUS + 1:N_NODE))
       END DO
     END ASSOCIATE
  END DO

  RHS = -0.5D0*RHS

  IF(BRDF_FLAG) THEN
     DO IS = 1, NRHS 
        CALL BRDF_ACT (BRDF_PARAM, N_PLUS, MJU_NODE(1:N_PLUS), PHI_NODE(1:N_PLUS),&
             1, [LocArrs(IS)%CSZA_Surf], [0.0D0], SURF_REFL(:,IS:IS))
     END DO
  END IF

  RHS(N_PLUS + 1:N_NODE,:) = 0.0D0
  DO K = 1, NRHS
     DO I = 1, N_PLUS
        RHS((NLAY - 1)*N_NODE + I,K) = SURF_REFL(I,K)*LocArrs(K)%EXM_NL
     END DO
  END DO

  ! Calculation of the solution
  ! ***************************
  RHS = SNGL(RHS)
  CALL DGBTRS ('n', N_NODE*NLAY, KL, KU, NRHS, AB, MDim, IPIV, RHS, &
       N_NODE*NLAY, INFO )

  RHS(N_PLUS + 1:N_NODE,:) = 0.0D0
  WHERE(RHS(:,:) < 1.0D-20) RHS(:,:) = 0.0D0

  FORALL (J = 1:NLAY, I_ND = 1:N_NODE) &
       RAD_PS(J,:,I_ND) = RHS((J - 1)*N_NODE + I_ND,:)

  IF(.NOT.FLG_WF) THEN
     DeAllocate(AB)
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  END IF

  ALLOCATE (RHS_WF(NLAY*N_NODE, NLAY*NRHS))
  RHS_WF = 0.0D0
  DO K = 1, NRHS         ! SZA
     II_1 = (K - 1)*NLAY + 1
     II_2 = K*NLAY
     CALL RHS_WF_FORM (AL_PC, LocArrs(K), S_EXP, RHS(:,K), PHASE_FUNC, &
          SURF_REFL(:,K), RHS_WF(:,II_1:II_2))
  END DO

  CALL DGBTRS ('n', N_NODE*NLAY, KL, KU, NRHS*NLAY, AB, MDim, IPIV, RHS_WF,&
       N_NODE*NLAY, INFO )
     
  DO K = 1, NL_FINE - 1
     J = NLAY - INTERP_IDX(K) + 1
     DELTA = (Z_FINE(K) - ZZZ(J))/(ZZZ(J - 1) - ZZZ(J))
     FORALL (I = 1:NLAY, IS = 1:NRHS) WFPS(IS)%RES(K,I,:) = DELTA* &
          RHS_WF((J - 2)*N_NODE + 1:(J - 1)*N_NODE,(IS - 1)*NLAY + I) + &
          RHS_WF((J - 1)*N_NODE + 1:J*N_NODE,(IS  - 1)*NLAY + I)*(1.0D0 - DELTA)
  END DO
  FORALL (I = 1:NLAY, IS = 1:NRHS) WFPS(IS)%RES(NL_FINE,I,:) =  &
       RHS_WF(1:N_NODE,(IS  - 1)*NLAY + I)

  DeAllocate(AB, RHS_WF)
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
!***************************************************************************
END SUBROUTINE SOLVE_DIFF_EQ
!***************************************************************************
