!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 23 Jul 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE Spher_Rad (JPOS, POS, INDEX_TH)
  ! Called by "rtm_main.f"
  
  USE ABSORBERS,   ONLY: N_GAS, GAS_INDEX, AMF_IDX, N_ESFT_MOL, ST_GAS_IDX
  USE AEROSOLS,    ONLY: LA_N, Plus_mi, NG_aer, N_AER_WAVS, AER_WAVS, NAW_MI, &
                         AW_MI, Mie_layer, Idi => Nga_db, Imi => Nga_mi, &
                         Greek_aer, Greek_Aer_mi, IDX_AER_db, IDX_AER_mi
  Use Clouds,      Only: DO_CLOUD, NCW => N_CLD_NETWAVS, CLW => CLD_NETWAVS
  USE ESFT,        ONLY: N_ESFT_COEFF, DO_ESFT_SPEC_MOL, DO_ESFT
  USE GRIDS,       ONLY: NLAYERS, ZZZ, Z_INV, NSGP=>NSOL, SOL_GRD, NL_FINE,  &
                         Z_FINE, INTERP_IDX, ALT_EXT, DZ
  USE INTERN_DIMS, ONLY: NL_Aer, Nm_vs, NLG_ALL_A, Nvs, Nm_ice
  USE INFO,        ONLY: WF_DB, TG_INFO
  USE LOG_PARS,    ONLY: LOG_LEVEL, EU, LUNIT, DO_LOGFILE, DIR_OUT, N_ESO, N_DSO, N_RSO
  USE NODES,       ONLY: N_NODE, WEIGHTS
  USE PHOTOCHEM,   ONLY: N_SOL_PC, DO_PHOTOCHEM, CONC_PC, PC_GAS_INDEX
  USE RTM_OUTPUT,  ONLY: TG_VOD, VERT_COL, WF_TO_AMF, WF_TG_FLAG
  USE SPHER_PARS,  ONLY: N_LOS_OUT, N_SOL_ANG, DO_SF_OUT
  USE ST_INPUT,    ONLY: APH_TYPE
  USE ST_VARS,     ONLY: RAD_TO_DEG, DO_AEROSOL, BRDF_FLAG, DO_PC, RTM_TYPE,&
                         INT_MS_FLAG, RTM_MODE, King_factor, F_LGI_SCR, &
                         F_LGI_SCA, F_LGI_EXT, ONE_MS_SZ, WF_MS_FLAG, ERROR, &
                         AKIMA_FLG
  USE TOOLS,       ONLY: FIND_IDX
  USE TYPES,       ONLY: KND, LOA_type, PWF_type, LOW_TYPE, PCC_TYPE, UDB_TYPE, &
       OSM_TYPE, AuxDataInterp, WFS_PS, TE_TYPE
  
  USE WF_NUM,      ONLY: NUM_AER, SCA_or_ABS, PERTURB
  
  INTEGER     JPOS
  REAL (KND)  POS
  ! Local variables
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SPHER_RAD'
  LOGICAL    VC_FLAG, ESFT_FLAG_LOC, FLG_WFPS
  LOGICAL    INDEX_TH(N_LOS_OUT)
  
  INTEGER    NSOL, J, I_GAS, I, JJESFT, IDX_SOL, NESFT_IDX, KK, II, CU
  INTEGER    IDX_LOS, Ws, Ws1, JESFT, K, IDX_NODE, NESFT, Wm, Wm1, IS
  INTEGER    E_IDX(2*N_ESFT_COEFF,N_ESFT_MOL)
  
  REAL (KND)  TG_CS(NLAYERS, N_GAS), TGA(NLAYERS), DEP_CF(3), FLU_Int
  REAL (KND)  TG_CROSS_SEC(NLAYERS, N_GAS) , FFF(NLAYERS)
  REAL (KND), ALLOCATABLE :: TG_TMP(:,:), RAD_PS(:,:,:,:), WF_OUT(:,:)
  REAL (KND), POINTER :: AS_C(:), AL(:)
  REAL (KND)  DEPOL, ESFT_WEIGHT, S_1, S_2, Dc, Dm, ALBEDO
  
  TYPE (LOA_type) :: LOA  ! Local optical parameters of atmosphere
  TYPE (LOW_type) :: LOW  ! Local optical parameters of water
  TYPE (OSM_type) :: OSM  ! Original scattering matrices
  TYPE (PWF_type) :: PWF  ! General weighting function 
  TYPE (PCC_type) :: PCC  ! Photochemistry   
  TYPE (TE_type)  :: TE    ! Thermal emission
  TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
  TYPE (AuxDataInterp), Target :: ADI
  TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
  
  !==================================================================
  INTERFACE
     Subroutine PS_RAD (DEP_CF, AS_C, AL_0, ALBEDO, PCC, SCA, SCR, AL0_OUT, &
          RAD_PS, WFPS, OSM, *)
       
       USE GRIDS,      ONLY: NLAY => NLAYERS, NL_FINE
       USE TYPES,      ONLY: KND, PCC_TYPE, OSM_TYPE, WFS_PS
       
       REAL (KND)  ALBEDO, DEP_CF(3), AL0_OUT(NL_FINE)
       REAL (KND), DIMENSION (NLAY):: AS_C, AL_0, SCA, SCR, ASYM_COEFF
       REAL (KND), ALLOCATABLE :: RAD_PS(:,:,:,:)
       TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
       TYPE (PCC_type) :: PCC 
       TYPE (OSM_type) :: OSM
     End Subroutine PS_RAD
       
     Subroutine Integrate_Init (RAD, ALBEDO, DEP_CF, LOA, PCC,  ADI, &
          WFPS, WF_OUT, JPOS, ESFT_WEIGHT, INDEX_TH, POS, OSM, *,  &
          FLU_Int, TE)
       USE GRIDS,      ONLY: NLAY=>NLAYERS
       USE SPHER_PARS, ONLY: N_LOS_ANG
       USE TYPES,      ONLY: KND, PCC_TYPE, OSM_TYPE, AuxDataInterp, &
                             WFS_PS, LOA_TYPE, TE_TYPE
       INTEGER     JPOS
       LOGICAL     INDEX_TH(N_LOS_ANG)
       REAL (KND), ALLOCATABLE :: RAD(:,:,:,:), WF_OUT(:,:)
       REAL (KND) AL(NLAY), DEP_CF(3)
       REAL (KND)  ALBEDO, FLU_Int, ESFT_WEIGHT, POS
       TYPE (PCC_type) :: PCC    
       TYPE (OSM_type) :: OSM  
       TYPE (TE_type)  :: TE  
       TYPE (AuxDataInterp), Target :: ADI
       TYPE (LOA_type) :: LOA    
       TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
     END Subroutine Integrate_Init
  END INTERFACE
  !==================================================================
  IF(ANY(ERROR)) RETURN
  CALL ST_TRACK (R_NAME, 1, 'I')

  IF (LOG_LEVEL(1) >= N_RSO) WRITE (LUNIT, &
       '(2X,"*** Radiance calculation point ",I5, " at", 1PG14.6, " nm")') JPOS, POS

  FLU_Int = 0.d0 ! no fluorescence within CDI mode !!!

  ALLOCATE(ADI%X_SCR(NLAYERS,4), ADI%X_AL(NLAYERS,4), ADI%X_SCA(NLAYERS,4), &
       ADI%X_ASC(NLAYERS,4), ADI%AL0_OUT(NL_FINE), ADI%SCR_OUT(NL_FINE), &
       ADI%SCA_OUT(NL_FINE), ADI%ASC_OUT(NL_FINE))

  ! ... No clouds in CDI !
  IF( ALLOCATED(ADI%SCC_NSO) ) DEALLOCATE( ADI%SCC_NSO,ADI%X_SCC )
  ALLOCATE( ADI%SCC_NSO(NL_FINE,Nm_ice), ADI%X_SCC(NLAYERS,4,Nm_ice) )
  ADI%X_SCC = 0.d0
  
  ! ... No thermal emission in CDI !
  IF( ALLOCATED(ADI%Plf_OUT) ) DEALLOCATE( ADI%Plf_OUT, ADI%X_Plf )
  ALLOCATE( ADI%Plf_OUT(NL_FINE), ADI%X_Plf(NLAYERS,4) )
  ADI%X_Plf = 0.d0
  
  IF(INT_MS_FLAG(JPOS)) THEN
     IF (ONE_MS_SZ) THEN
     ! Solar zenith angle at geometric tangent point
        NSOL = 1 
     ELSE
        NSOL = NSGP
     END IF
     ALLOCATE(RAD_PS(NLAYERS, NSOL, N_NODE, Nvs))
     FLG_WFPS = WF_TG_FLAG.AND.WF_MS_FLAG(JPOS)
     IF(FLG_WFPS) ALLOCATE(WFPS(NSOL))
  END IF
      
  !     Determine depolarisation ratio for actual wavelength
  CALL KING_ACTUAL(POS, DEPOL, *200)

! Calculate extinction and scattering profiles
! (Aerosols, clouds, and Rayleigh).

  ! ... Allocate elements of the structure LOA
  ALLOCATE(LOA%SCR(NLAYERS),  LOA%EXA(NLAYERS), LOA%AS_A(NLAYERS), &
       LOA%SCA(NLAYERS),  LOA%SCC(NLAYERS), LOA%EXC(NLAYERS), LOA%AL(NLAYERS), &
       LOA%EXI(NLAYERS), &
       ! ... Allocate only needed elements of the structure PWF
       PWF%P_WF_ABSTERM(NLAYERS), PWF%T_WF_ABSTERM(NLAYERS))
  ! Photochemistry
  IF(DO_PC) Allocate(PCC%PCX_COEFF(NLAYERS, N_SOL_PC, 4), &
       PCC%PCY_COEFF(N_SOL_PC, NL_FINE, 4))
  IF(WF_TG_FLAG) ALLOCATE (WF_OUT(N_LOS_OUT, NLAYERS))

  LOA%EXC = 0.0D0 ; LOA%EXI = 0.0D0 
  ! ------------------------------------------------------------
  AS_C => LOA%AS_A
  AL   => LOA%AL

  IF(DO_AEROSOL.AND.RTM_TYPE /= 'transmission') THEN
     NL_Aer = LA_N 
     IF( Plus_mi ) NL_Aer = NLAYERS
     
     IF( APH_TYPE=='mie' ) THEN
        NLG_ALL_A = NG_aer
        ALLOCATE( OSM%AER_ALL(NG_aer,NL_Aer,Nm_vs) ) ! A
        OSM%AER_ALL = 0.d0
        Call LIN_SEARCH_NEW(POS, Dc, Ws, Ws1, N_AER_WAVS, AER_WAVS)
        ! ... Mie-layers plus standard aerosol layers
        IF( Plus_mi ) THEN  
           Call LIN_SEARCH_NEW(POS, Dm, Wm, Wm1, NAW_MI, AW_MI)
           DO I = 1, NLAYERS
              IF( .NOT.Mie_layer(I) ) THEN
                 OSM%AER_ALL(1:Idi,I,1:Nm_vs) = (1.d0 - Dc) &
                      *Greek_aer(1:Idi,IDX_AER_db(I),1:Nm_vs,Ws) &
                      + Dc*Greek_aer(1:Idi,IDX_AER_db(I),1:Nm_vs,Ws1)
              ELSE
                 OSM%AER_ALL(1:Imi,I,1:Nm_vs) = (1.d0 - Dm) &
                      *Greek_aer_mi(1:Imi,IDX_AER_mi(I),1:Nm_vs,Wm) &
                      + Dm*Greek_aer_mi(1:Imi,IDX_AER_mi(I),1:Nm_vs,Wm1)
              ENDIF
           ENDDO
           ! ... Mie-layers or standard aerosol layers only
        ELSE                
           OSM%AER_ALL(:,:,1:Nm_vs) = (1.d0 - Dc)*Greek_aer(:,:,1:Nm_vs,Ws) &
                + Dc*Greek_aer(:,:,1:Nm_vs,Ws1)            
        ENDIF
     ENDIF  ! APH_TYPE = 'mie'
  ENDIF

  IF( DO_CLOUD ) THEN
     ! This version allows extrapolation as a constant
     CALL LIN_SEARCH_NEW(POS, LOA%DP_CLD, LOA%Cil, LOA%Cir, NCW,CLW)
  ENDIF
 
  CALL RAYLEIGH_SCATT (JPOS, POS, DEPOL, LOA%SCR, *880)
  CALL GT_SCATT (JPOS, POS, LOA, *880)
  IF( NUM_AER.and.SCA_or_ABS == "aerosol extinction") &
       LOA%EXA(:) = LOA%EXA(:)*PERTURB(:)
  
  DEP_CF(1) = 3.0D0*(1.0D0 + DEPOL)/(4.0D0 + 2.0D0*DEPOL)
  DEP_CF(2)  = 3.0D0*(1.0D0 - DEPOL)/(4.0D0 + 2.0D0*DEPOL)
  
  IF(.NOT.BRDF_FLAG) THEN
     CALL GT_SURFACE( 1, JPOS, POS, UDB_A, UDB_W, *880 ) 
     ALBEDO = UDB_A%ALBEDO
  ELSE                         
     ! Output array: BRDF_PARAM       
     CALL BRDF_SETUP
  END IF

  IF(DO_ESFT) THEN
     ESFT_FLAG_LOC = ANY(DO_ESFT_SPEC_MOL(JPOS,:))
  ELSE
     ESFT_FLAG_LOC = .FALSE.
  END IF

  IF(ESFT_FLAG_LOC) THEN
     ! Subroutines IDX_ESFT and WEIGHT_ESFT are in st_esft.f
     CALL IDX_ESFT(JPOS, E_IDX, NESFT, NESFT_IDX, *880)
  ELSE
     NESFT = 1
  ENDIF
      
  ! ESFT loop
  ! ---------
  TG_CS(:,:) = 0.0D0
  DO JJESFT = 1, NESFT
     IF(ESFT_FLAG_LOC) THEN
        CALL WEIGHT_ESFT(JPOS, JJESFT, JESFT, NESFT, NESFT_IDX, ESFT_WEIGHT)
     ELSE
        JESFT       = JJESFT
        ESFT_WEIGHT = 1.0D0
     ENDIF

     CALL ST_ABS_COEFF(JPOS, POS, JJESFT, E_IDX, TG_CROSS_SEC, LOA, PWF, &
          PCC, TGA, *880)

     TG_CS(:,:) = TG_CS(:,:) + ESFT_WEIGHT*TG_CROSS_SEC(:,:)

     CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, AL, F_LGI_EXT, ADI%X_AL, AKIMA_FLG)
     CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, LOA%SCR, F_LGI_SCR, ADI%X_SCR, &
          AKIMA_FLG)
     IF(DO_AEROSOL) THEN
        CALL AKIMA_ALL_FINE (NLAYERS, ALT_EXT, LOA%SCA, F_LGI_SCA, ADI%X_SCA,&
             AKIMA_FLG)
        CALL AKIMA_ALL_FINE(NLAYERS, ALT_EXT, AS_C, .FALSE., ADI%X_ASC, &
             AKIMA_FLG)
     ELSE
        ADI%X_SCA = 0.0D0
        ADI%X_ASC = 0.0D0
     END IF
     CALL AKIMA_END_FINE (NLAYERS, Z_INV, NL_FINE, Z_FINE, INTERP_IDX,     &
          ADI%X_AL, ADI%X_SCR, ADI%X_SCA, ADI%X_ASC, ADI%X_SCC, ADI%X_Plf, &
          ADI%AL0_OUT, ADI%SCR_OUT, ADI%SCA_OUT, ADI%ASC_OUT, ADI%SCC_NSO, &
          ADI%Plf_OUT )
     
     IF(INT_MS_FLAG(JPOS)) THEN
        IF (LOG_LEVEL(1) >= N_ESO) &
             WRITE (LUNIT,*) ' Calculation of pseudo-spherical solution ....' 
        CALL PS_RAD (DEP_CF, AS_C, AL, ALBEDO, PCC, LOA%SCA, LOA%SCR, &
             ADI%AL0_OUT, RAD_PS, WFPS, OSM, *880)
        
!     DAY_ANGLE = PIE*2.0D0*(DAY_OF_YEAR - 1)/DAYS_IN_YEAR 
!     SCALE = (1.000110D0 + 0.034221D0*DCOS(DAY_ANGLE) +
!     $        0.001280D0*DSIN(DAY_ANGLE) + 0.000719D0*DCOS(2.D0*
!     $        DAY_ANGLE) + 0.000077D0*DSIN(2.D0*DAY_ANGLE))/PIE
!     write(6,*) '***************', RAD_PS(1,7,1,1)*SCALE*
!     $        SOL_FLUX(JPOS)
!     c         RAD_TOTAL(1) = RAD_PS(1,5,1,1)
!     RAD_TOTAL(1) = RAD_PS(1,7,1,1)
            
        IF(.FALSE.) THEN
           ! IF(DO_SF_OUT) THEN
           open(NewUnit=CU,file=DIR_OUT//'/test_rm_ps.dat',recl=2000)
           DO IDX_SOL = 1, NSOL
              write(CU,'(50(1x,E13.6))') DACOS(SOL_GRD(IDX_SOL))*RAD_TO_DEG
           END DO
           DO IDX_SOL = 1, NSOL
              DO J = 1, NLAYERS
                 write(CU,'(500(1x,E13.6))')  ZZZ(J), (RAD_PS(J, &
                      IDX_SOL,IDX_NODE,1), IDX_NODE = 1, N_NODE)
              END DO
           END DO
           close(cu) 
           open(NewUnit=CU,file=DIR_OUT//'/test_mi_ps.dat',recl=2000)
           DO J = 1, NLAYERS
              write(CU,'(1X,F6.2)')  ZZZ(J)
           END DO
           DO IDX_SOL = 10, NSOL
              DO J = 1, NLAYERS
                 S_1 = 0
                 S_2 = 0
                 DO IDX_LOS = 1, N_NODE
                    S_1 = S_1 + RAD_PS(J, IDX_SOL, IDX_LOS,1)*WEIGHTS(IDX_LOS)
                    S_2 = S_2 + WEIGHTS(IDX_LOS)
                 END DO
                 FFF(J) = S_1/S_2
              END DO
              write(22,'(2X,F6.2,100(1x,E15.8))') DACOS(SOL_GRD(IDX_SOL))* &
                   RAD_TO_DEG, (FFF(J), J = 1, NLAYERS)
           END DO
           close(CU)
        END IF
            
     ELSE                   ! INT_MS_FLAG(JPOS)
        IF (LOG_LEVEL(1) >= N_ESO) &
             WRITE (LUNIT,*)  ' ******* Single scattering mode *******'
     END IF

     IF (LOG_LEVEL(1) >= N_ESO) THEN
        WRITE (LUNIT,*) 
        IF(TRIM(RTM_TYPE) == 'ps_scat') THEN
           write(LUNIT,*) ' Calculation of pseudo-spherical solution ...'
        ELSE
           write(LUNIT,*) ' Calculation of spherical solution ....'
        END IF
     END IF
         
     IF (LOG_LEVEL(1) >= N_DSO)  &
          WRITE(LUNIT,*) ' **** START "Integrate_Init" ****'

     CALL Integrate_Init (RAD_PS, ALBEDO, DEP_CF, LOA, PCC, ADI, WFPS, &
          WF_OUT, JPOS, ESFT_WEIGHT, INDEX_TH, POS, OSM, *880, FLU_int, TE)

     IF(WF_TG_FLAG) THEN
        DO II = 1, UBOUND(WF_DB,1)
           If(WF_DB(II)%TYPE == 'tg') Then
              KK = FIND_IDX(WF_DB(II)%NAME,TG_INFO%NAME)
              FORALL(J = 1:NLAYERS, I = 1:N_LOS_OUT) &
                   WF_DB(II)%WF(JPOS,I,J,1) = WF_DB(II)%WF(JPOS,I,J,1) + &
                   ESFT_WEIGHT*WF_OUT(I,J)/AL(J)*TG_CROSS_SEC(J,GAS_INDEX(KK))
           End If
           If(RTM_TYPE == 'transmission'.AND.WF_DB(II)%Name == 'aer-ext') Then
              FORALL(J = 1:NLAYERS, I = 1:N_LOS_OUT) &
                   WF_DB(II)%WF(JPOS,I,J,1) = WF_DB(II)%WF(JPOS,I,J,1) + &
                   ESFT_WEIGHT*WF_OUT(I,J)/AL(J)
           End If
        END DO
     END IF
  END DO                    ! ESFT coefficients

  IF (LOG_LEVEL(1) >= N_ESO) WRITE (LUNIT,*) 
  
  IF(WF_TG_FLAG) THEN
     DO II = 1, UBOUND(WF_DB,1)
        IF(WF_DB(II)%DIM /= 'rel') CYCLE
        IF(WF_DB(II)%TYPE == 'tg') Then
           KK = FIND_IDX(WF_DB(II)%NAME,TG_INFO%NAME)
           FORALL(J = 1:NLAYERS) WF_DB(II)%WF(JPOS,:,J,1) = &
                WF_DB(II)%WF(JPOS,:,J,1)*TG_INFO(KK)%ND(J)
        End If
        If(RTM_TYPE == 'transmission'.AND.WF_DB(II)%Name == 'aer-ext') Then
           FORALL(J = 1:NLAYERS) WF_DB(II)%WF(JPOS,:,J,1) = &
                WF_DB(II)%WF(JPOS,:,J,1)*LOA%EXA(J)
        End If
     END DO
  END IF

  IF(BRDF_FLAG.AND.DO_LOGFILE)  CALL BRDF_ALBEDO 
  
  VC_FLAG = .FALSE.
  
  SELECT CASE (RTM_MODE)
  CASE ('block_amf')
!     FORALL(J = 1:NLAYERS)  WF_TO_AMF(JPOS,J) = &
!          0.5D0*DZ(J)*TG_CS(J,GAS_INDEX(AMF_IDX))
     FORALL(J = 1:NLAYERS-1)  WF_TO_AMF(JPOS,J) = &
          0.5D0*DZ(J)*TG_CS(J,GAS_INDEX(AMF_IDX))
     WF_TO_AMF(JPOS,NLAYERS) = &
          DZ(NLAYERS)*TG_CS(NLAYERS,GAS_INDEX(AMF_IDX))
  CASE ('amf', 'slant_col')
     IF(JPOS == 1) VC_FLAG = .TRUE.
     
     I_GAS = GAS_INDEX(AMF_IDX)
     IF(DO_PHOTOCHEM(ST_GAS_IDX(I_GAS))) THEN
        
        ALLOCATE(TG_TMP(NLAYERS,N_SOL_PC)) ! A
        
        FORALL(IDX_SOL = 1:N_SOL_PC) &
             TG_TMP(1:NLAYERS,IDX_SOL) = TG_CS(1:NLAYERS,I_GAS)*&
             CONC_PC(1:NLAYERS,PC_GAS_INDEX(I_GAS),IDX_SOL)
            
        CALL VERT_INTEGR_PC(TG_TMP(1:NLAYERS,1:N_SOL_PC), &
             TG_VOD(JPOS,1:N_SOL_ANG,1))
            
        IF(VC_FLAG) CALL VERT_INTEGR_PC (&
             CONC_PC(1:NLAYERS,PC_GAS_INDEX(I_GAS),1:N_SOL_PC), VERT_COL(1:N_SOL_PC))
            
     ELSE
            
        FFF(:) = TG_CS(1:NLAYERS,I_GAS)*TG_INFO(AMF_IDX)%ND(:)
        CALL VERT_INTEGR(FFF(:), TG_VOD(JPOS,:,1))
        
        IF(VC_FLAG) THEN
           CALL VERT_INTEGR(TG_INFO(AMF_IDX)%ND, VERT_COL(1:N_SOL_ANG))
           IF( TG_INFO(AMF_IDX)%NAME == 'o4' ) VERT_COL(:) = VERT_COL(:)*1.D+40
        END IF
     END IF
  END SELECT
      
  DEALLOCATE( LOA%SCR, LOA%EXA, LOA%AS_A, LOA%SCA, LOA%EXC, LOA%EXI, LOA%AL, &
       PWF%P_WF_ABSTERM, PWF%T_WF_ABSTERM, ADI%X_SCR, ADI%X_AL, ADI%X_SCA, &
       ADI%X_ASC, ADI%AL0_OUT, ADI%SCR_OUT, ADI%SCA_OUT, ADI%ASC_OUT )
  IF(WF_TG_FLAG) DeAllocate (WF_OUT)
  IF(DO_PC) DeAllocate (PCC%PCX_COEFF, PCC%PCY_COEFF)
  IF(INT_MS_FLAG(JPOS)) DEALLOCATE(RAD_PS)

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

200 CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Interpolation error: Depolarization factor')
  GOTO 881
  
880 WRITE(EU, *) ' ---> Called by '//R_NAME
881 ERROR(JPOS) = .TRUE.
  WRITE(EU, *) ' ---> Error inside the wavelenght loop'
  WRITE(EU, *) ' ---> Calling sequence: gt_iface - rtm_main'
  
  WRITE(LUNIT, *)
  WRITE(LUNIT, *) ' >>>>>>>> !!! Abnormal termination !!! <<<<<<<'
  WRITE(LUNIT, *) ' SCIATRAN stopped in spher_rad: see errors.log'
  WRITE(LUNIT, *) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
  WRITE(LUNIT, *)
  
!***************************************************************************
END SUBROUTINE Spher_Rad
!***************************************************************************

