!***************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 10 Dec 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE SPHER_WF(SSSF, DS_SUN, S_F, S_Rin, EX_L, EX_1, Nr, D_DS, IOB,  &
     ZLS, Mha, AL_1, PWF, IV, IS, IP, JPOS, ADD_TE)
  ! Called by "sfi_spher.f", "st_ss_stokes.f"

  USE AEROSOLS,    ONLY: Dwr_mi, N_dwr
  USE GRIDS,       ONLY: NLAYERS, ZZZ, DZR, MAX_LINE, N_WF_L
  USE INFO,        ONLY: WF_DB
  USE INTERN_DIMS, ONLY: Nvs
  USE LOS_ANGLES,  ONLY: N_Below
  USE ST_VARS,     ONLY: Nc_der, Ns_wf, VST, IMF => INT_MS_FLAG,  &
                         GeoD => GeoD_OUT, Ray_to_WF 
  USE SPHER_PARS,  ONLY: NFA => N_FOV_ANG, FOV_WEIGHTS
  USE TOOLS,       ONLY: INQ_WF
  USE TYPE_VARS,   ONLY: KND
  USE TYPES,       ONLY: PWF_TYPE
  !------------------------------------------------------------------
  TYPE (PWF_type) :: PWF    ! Weighting functions
  !     ----------------------
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SPHER_WF'
  INTEGER, INTENT (IN) :: Nr, Mha, IV, IS, IP, JPOS
  LOGICAL  ADD_TE
  INTEGER  IOB(NR)
  REAL (KND), TARGET :: DS_SUN(GeoD%MaxLS,NLAYERS)
  REAL (KND), TARGET :: S_Rin(GeoD%MaxLS,Nvs,1+N_dwr)
  REAL (KND), DIMENSION  (GeoD%MaxLS,Nvs) :: SSSF, S_F
  REAL (KND)  EX_L(NR), EX_1(0:NR), ZLS(NR), AL_1(GeoD%MaxLS)
  ! Local parameters
  INTEGER   I, J, K, L, II, ILS, L1, Index, Idr, Jw, Jw1, KK
  REAL (KND)  S2, WF(NR,NLAYERS), LOC_VST, D_DS(2,NR), WF_TMP(NLAYERS), &
       WF_A(NLAYERS), Z_E(0:NLAYERS), WFOV
  REAL (KND), ALLOCATABLE :: A_UP(:), A_DN(:), DS_EXP(:,:)
  REAL (KND), POINTER :: S_R(:,:)
! ==================================================================
  INTERFACE
     Subroutine DS_FORM_3 (ZLS, DS, IOB, NLS, A_UP, A_DN, DS_WF)
       Use Grids, Only: Nlayers, ZZZ
       Use Types, Only: KND
       Integer  NLS, IOB(NLS)
       Real (KND)  DS(2,NLS), ZLS(NLS)
       Real (KND), Allocatable ::  A_UP(:), A_DN(:), DS_WF(:,:)
     End Subroutine DS_FORM_3
  END INTERFACE

  CALL ST_TRACK (R_NAME, 1, 'I')

  LOC_VST = VST
  ! Remove multiplication by VST if this routine is called 
  ! using single scattering approximation
  IF( .NOT.IMF(JPOS) ) LOC_VST = 1.d0

  Index = GeoD%ILO_SVFP(IV,IS,IP)
  WFOV = FOV_WEIGHTS( IV-(CEILING( FLOAT(IV)/FLOAT(NFA) )-1)*NFA )

  Call DS_FORM_3( ZLS, D_DS, IOB, NR, A_UP, A_DN, DS_EXP )

  DO Jw = 1, Ns_wf
     Jw1 = Nc_der(Jw)  ;  WF_TMP = 0.d0
     DO I = 1, Nr
        WF_A = 0.d0
!       ... Contribution of extinction coefficient variations within  
!           viewing direction transmission "exponent" after scattering events
!           (See Eq.(T4-sph) in STB)
        IF( Mha == 1 ) WF_A(:) = WF_A(:) - SSSF(I,Jw1)*DS_SUN(I,:)
        II = IOB(I)
        Associate(Dex => EX_1(I-1)-EX_1(I), SF => S_F(I,Jw1)+SSSF(I,Jw1))
          WF_A(:) = WF_A(:)/AL_1(I)*Dex   &
               - (EX_1(I-1)*DS_EXP(I-1,:) - EX_1(I)*DS_EXP(I,:))*SF/AL_1(I)
          WF_A(II) = WF_A(II) - SF/AL_1(I)**2*A_DN(I)*Dex
          if(II /= 1) WF_A(II-1) = WF_A(II-1) - SF/AL_1(I)**2*A_UP(I)*Dex
        End Associate
        WF_TMP(:) = WF_TMP(:) + WF_A(:)
     END DO  ! along LoS (I)
!    ... In this routine we employ the multiplier VST
!        VST = 1 in the scalar case; VST = 2 in the vector case
!        to properly use coefficients in Fourier series expansion of 
!        weighting functions. (see "phm_along_ls.f" with coefficients 0.5 for 
!        array S_F and coefficient EWH in "st_adjoint_a.f")
     PWF%WF_esp(:,Index,Jw) = PWF%WF_esp(:,Index,Jw) + LOC_VST*WF_TMP(:) *WFOV
  ENDDO  ! over number of Stokes component derivatives (Jw, Jw1)
  
  DS_SUN(Nr,:) = DS_EXP(Nr,:)

  IF( (.NOT.Ray_to_WF).AND.(.NOT.ADD_TE).AND.(.NOT.INQ_WF('aer', 'WFT')) &
       .AND. .NOT.INQ_WF('atom', 'WFT')) THEN
     CALL ST_TRACK (R_NAME, 1, 'O')
     RETURN
  END IF

  IF( Ray_to_WF.OR.INQ_WF('aer','WFT').OR.INQ_WF('atom','WFT') ) THEN
     DO Idr = 1,1+N_dwr
        S_R => S_Rin(:,:,Idr)  ;  WF_TMP = 0.d0
        CALL WF_plus
        L1 = N_below(IV)
        PWF%WF_ssp(1:L1,Index,1,Idr) = PWF%WF_ssp(1:L1,Index,1,Idr) + &
             WF_TMP(1:L1) *WFOV    
!     write(*,*) '  2 -  WF_TMP(:) ', Mha, '  --- ', WF_TMP(1:L1)     
     ENDDO  ! over derivatives wrt PSD parameters (Idr)
  ENDIF

!  WRITE(*,'(*(2x,e12.4))') S_Rin(:,1,1)
  
  IF( ADD_TE ) THEN
     S_R => S_Rin(:,:,1)  ;  WF_TMP = 0.D0
!     CALL WF_plus
     DO II = 1, NR
        KK = IOB(II)
        WF_TMP(KK) = WF_TMP(KK) +  S_R(II,1)*A_DN(II)* &
!             EX_L(II)*(D_DS(1,II) + D_DS(2,II))      
             (EX_1(II-1) - EX_1(II))/AL_1(II) 
        IF(KK == 1) CYCLE
        WF_TMP(KK - 1) = WF_TMP(KK - 1) + S_R(II,1)*A_UP(II)* &
!             EX_L(II)*(D_DS(1,II) + D_DS(2,II))
             (EX_1(II-1) - EX_1(II))/AL_1(II) 
     ENDDO
     L1 = N_below(IV)
     PWF%WF_esp(1:L1,Index,1) = PWF%WF_esp(1:L1,Index,1) + &
          LOC_VST*WF_TMP(1:L1) *WFOV
  ENDIF  ! ADD_TE
  
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
CONTAINS
!***************************************************************************
  SUBROUTINE WF_plus
    DO II = 1, NR
       KK = IOB(II)
       WF_TMP(KK) = WF_TMP(KK) +  S_R(II,1)*  &
!            EX_L(II)*AL_1(II)*(D_DS(1,II) + D_DS(2,II))            
            (EX_1(II-1) - EX_1(II)) *A_DN(II)
       IF(KK == 1) CYCLE
       WF_TMP(KK - 1) = WF_TMP(KK - 1) + S_R(II,1)* &
!            EX_L(II)*AL_1(II)*(D_DS(1,II) + D_DS(2,II))
            (EX_1(II-1) - EX_1(II)) *A_UP(II)
    ENDDO
  END SUBROUTINE WF_PLUS

END SUBROUTINE SPHER_WF
!***************************************************************************
