!**************************************************************************
!
!	SCIATRAN - Version 4.5 - Last modification: 20 Oct 2021
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!****************************************************************************
Subroutine ST_ABS_COEFF(JPOS, POS, JJESFT, E_IDX, TG_CROSS_SEC, LOA, PWF, &
     PCC, TGA, *)
  ! Called by "dom_spherical.f90", "spher_rad.f90", and "st_vod.f"

  USE ABSORBERS,   ONLY: N_GAS, ST_GAS_IDX, MIX_ABS_START, N_ESFT_MOL, &
                         N_XSEC_MOL, GAS_INDEX
  USE ALT_INPUT,   ONLY: ALIN_ACT, ALIN_ABS, SOUT_FLAG, SOUT_TAU
  USE CIA_VARS,    ONLY: CIA_INFO
  USE ESFT,        ONLY: ESFT_COEFF, D_ESFT_COEFF_DP, N_ESFT_COEFF, &
                         D_ESFT_COEFF_DT, DO_ESFT_SPEC_MOL
  USE GRIDS,       ONLY: NLAYERS, ZZZ, DZR, ALT_EXT, NL_FINE, Z_INV, &
                         Z_FINE, INTERP_IDX
  USE INFO,        ONLY: TG_INFO
  USE INTERN_DIMS, ONLY: NL_PC
  USE LOG_PARS,    ONLY: EU, LUNIT, LOG_LEVEL, N_ESO
  USE LBL,         ONLY: PT_SCALE
  USE PHOTOCHEM,   ONLY: N_SOL_PC, CONC_PC, DO_PHOTOCHEM, PC_GAS_INDEX
  USE PROFILES,    ONLY: PPP => PRESS, TTT => TEMP
  USE RTM_OUTPUT,  ONLY: IND_NUM_WF
  USE ST_VARS,     ONLY: Pie, WF_FLAG, DO_PC, LA_FLAG, RTM_CORE, DO_RAMAN
  USE TOOLS,       ONLY: INQ_WF, FIND_IDX
  USE TYPE_VARS,   ONLY: KND
  USE TYPES,       ONLY: LOA_TYPE, PWF_TYPE, PCC_TYPE
  USE WF_NUM,      ONLY: NUM_ABS, NUM_TEM, PERTURB
  USE X_SECTS,     ONLY: ABS_COEFF, ABS_COEFF_O3, ABS_COEFF_O2,&
                         XS_TEMP => ABS_COEFF_TEMP, O3_BP_COEFF12,&
                         O3_BP_COEFF13, BP_FLAG, SR_H_FLAG, O3_UV_CS
  use cs_table !mhilker
  USE RETR_ITER,   ONLY: ITERATION

  INTEGER, INTENT(IN) ::  JPOS, JJESFT
  INTEGER, INTENT(IN) ::  E_IDX(2*N_ESFT_COEFF,N_ESFT_MOL)
  REAL (KND), INTENT(IN) ::  POS
  REAL (KND), DIMENSION(NLAYERS), INTENT(OUT) :: TGA
  REAL (KND), INTENT(OUT)::  TG_CROSS_SEC(NLAYERS,N_GAS)
  TYPE (LOA_type) :: LOA    ! optical parameters of atmosphere
  TYPE (PWF_type) :: PWF    ! General weighting function
  TYPE (PCC_type) :: PCC    ! Photochemistry
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_ABS_COEFF'
  CHARACTER (LEN=:), ALLOCATABLE :: TYPE_SELECTOR, FMT_STR
  CHARACTER (LEN=10) TMP_STR
  LOGICAL     COMPARE_TO_GSS, LINES_FOUND
  Logical, Allocatable :: UseLine(:)
  INTEGER     IDX, J, IG, KK, I, II, ISTART, IEND, INDG, I_P, II_1, II_2, CU,&
       N_T
  REAL (KND)  GET_ABS_COEFF, SURF_TEMP, WL, ABS_COEFF_ACT, ABS_COEFF_ACT1, &
       ABS_COEFF_ACT2, Thel(6), TGA_PC(NL_PC, N_SOL_PC), D_CIA_CS
  REAL (KND)  CIA_CS(NLAYERS)
  REAL (KND), ALLOCATABLE :: DEL(:), HW_D(:)
  REAL (KND),     POINTER :: X_SECTION(:,:), CS_DB(:), T_DB(:) 
  integer     tab_err !mhilker
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
 
  X_SECTION => ABS_COEFF(:,JPOS,:)

  TGA = 0.0D0 ; TG_CROSS_SEC  = 0.0D0 ;  TGA_PC = 0.0
  IF( WF_FLAG ) THEN
     PWF%P_WF_ABSTERM = 0.0D0 ;  PWF%T_WF_ABSTERM = 0.0D0
  END IF

  SURF_TEMP = TTT(NLAYERS)

  KK = 0
  DO IG = 1, Size(TG_INFO)
     If(.Not.TG_INFO(IG)%FM_FLG) Cycle
     Kk = KK + 1
     IF (TG_INFO(IG)%TYPE == "mixed") THEN
        TYPE_SELECTOR = TRIM(TG_INFO(IG)%TYPE_WL(JPOS))
     ELSE
        TYPE_SELECTOR = TRIM(TG_INFO(IG)%TYPE)
     END IF
     SELECT CASE (TYPE_SELECTOR)
     CASE ('atom')
        ! Number of emission lines
        II_1 = TG_INFO(IG)%ALP_NLIN 
        ALLOCATE( DEL(II_1),HW_D(II_1) )
        DEL(:) = ( POS - 1.d7/TG_INFO(IG)%ALP_LP(:) )**2
        DO I = 1, NLAYERS
           ! Doppler half-width [nm]
           HW_D(:) = TG_INFO(IG)%ALP_HW(:)*DSQRT(TTT(I))
           ! Monochromatic cross-section [cm^2] at WL = POS
           TG_CROSS_SEC(I,KK) = SUM( TG_INFO(IG)%ALP_LI(:)* &
                EXP( -DEL(:)/HW_D(:)**2 )/HW_D(:)/DSQRT(Pie) )
        END DO
        DEALLOCATE( DEL,HW_D )

     CASE ('xsec')
        DO I = 1, NLAYERS
           TG_CROSS_SEC(I,KK) = &
                DIM(GET_ABS_COEFF(KK, IG, JPOS, TTT(I), IDX), 0.0D0)
           IF( .NOT.WF_FLAG.OR.IDX == 0 ) CYCLE
           ! TEMPERATURE W/F TERM
           IF(X_SECTION(KK,IDX - 1) >= 0.0D0.AND.&
                X_SECTION(KK,IDX) >= 0.0D0) PWF%T_WF_ABSTERM(I) = &
                PWF%T_WF_ABSTERM(I) + TG_INFO(IG)%ND(I)* &
                (X_SECTION(KK,IDX) - X_SECTION(KK,IDX - 1))/ &
                (XS_TEMP(IG,IDX) - XS_TEMP(IG,IDX - 1))
        END DO

     CASE ('uv_o3')
        IF(BP_FLAG(JPOS)) THEN
           II = COUNT(BP_FLAG(1:JPOS))
           FORALL(I = 1:NLAYERS) TG_CROSS_SEC(I,KK) = &
                DIM(ABS_COEFF_O3(II,I), 0.0D0)
           ! TEMPERATURE W/F TERM
           IF( WF_FLAG ) FORALL(I = 1:NLAYERS) &
                PWF%T_WF_ABSTERM(I) = PWF%T_WF_ABSTERM(I) + &
                TG_INFO(IG)%ND(I)*(O3_BP_COEFF12(II) + &
                2.0D0*O3_BP_COEFF13(II)*(TTT(I) - 273.15D0))
           CYCLE
        END IF

        IF(O3_UV_CS(JPOS)) THEN
           II = N_XSEC_MOL + 1
        ELSE
           II = FIND_IDX('o3',TG_INFO%NAME)
        END IF

        DO I = 1, NLAYERS
           TG_CROSS_SEC(I,KK) = DIM( &
                GET_ABS_COEFF(KK, II, JPOS, TTT(I), IDX), 0.0D0)

           IF( .NOT.WF_FLAG.OR.IDX == 0 ) CYCLE
           ! TEMPERATURE W/F TERM
           IF(X_SECTION(KK,IDX - 1) >= 0.0D0.AND. &
                X_SECTION(KK,IDX) >= 0.0D0) PWF%T_WF_ABSTERM(I) = &
                PWF%T_WF_ABSTERM(I) + TG_INFO(IG)%ND(I)*&
                (X_SECTION(KK,IDX) - X_SECTION(KK,IDX - 1))/&
                (XS_TEMP(II,IDX) - XS_TEMP(II,IDX - 1))
        END DO

     CASE ('uv_o2')
        IF(SR_H_FLAG(JPOS)) THEN
           II = COUNT(SR_H_FLAG(1:JPOS))
           FORALL(I = 1:NLAYERS) TG_CROSS_SEC(I,KK) = &
                DIM(ABS_COEFF_O2(II,I), 0.0D0)
        END IF

     CASE ('esft')
        J = ST_GAS_IDX(KK) - MIX_ABS_START + 1 ! Check if  OK!!!!!
        IF(.NOT.DO_ESFT_SPEC_MOL(JPOS,J)) CYCLE
        II = E_IDX(JJESFT,J)
        DO I = 1, NLAYERS
           TG_CROSS_SEC(I,KK) = ESFT_COEFF(JPOS,II,I,J)
           IF( .NOT.WF_FLAG.OR.TRIM(RTM_CORE) /= 'DOM' ) CYCLE
           ! TEMPERATURE W/F TERM in ESFT mode
           ! =================================
           PWF%P_WF_ABSTERM(I) = PWF%P_WF_ABSTERM(I) + TG_INFO(IG)%ND(I)*&
                D_ESFT_COEFF_DP(JPOS,II,I,J)*PPP(I)
           PWF%T_WF_ABSTERM(I) = PWF%T_WF_ABSTERM(I) + &
                TG_INFO(IG)%ND(I)*D_ESFT_COEFF_DT(JPOS,II,I,J)
        ENDDO

     CASE ('lbl')
        IF(.NOT.DO_RAMAN) THEN
           IF(.NOT.LA_FLAG(JPOS)) CYCLE
        END IF

        ! If no lines found for a mixed gas
        If(.Not.Allocated(TG_INFO(IG)%LinePars)) Cycle 
        WL = 1.0d7/POS
        !mhilker
        if(associated(table_lists(IG)%table)) then
           do I=1,NLAYERS
              call table_interpolate(table_lists(IG)%table,&
                   PPP(I)*1d2,TTT(I),WL,ABS_COEFF_ACT, tab_err)
              if((tab_err .ne. 0) .and. (tab_err .ne. -100)) GoTo 200
              TG_CROSS_SEC(I,KK) = ABS_COEFF_ACT
              
              if(INQ_WF('temp', 'WFN')) then
                 call table_interpolate(table_lists(IG)%table,&
                      PPP(I)*1d2,TTT(I)*PT_SCALE, WL,ABS_COEFF_ACT1, tab_err)
                  if((tab_err .ne. 0) .and. (tab_err .ne. -100)) GoTo 200
                  PWF%T_WF_ABSTERM(I) = PWF%T_WF_ABSTERM(I) + &
                       TG_INFO(IG)%ND(I)*(ABS_COEFF_ACT1 - ABS_COEFF_ACT)/&
                       (TTT(I)*(PT_SCALE - 1.0D0))
               end if

               if(INQ_WF('press', 'WFN')) then
                  call table_interpolate(table_lists(IG)%table,&
                       PPP(I)*1d2*PT_SCALE,TTT(I), WL,ABS_COEFF_ACT2, tab_err)
                  if((tab_err .ne. 0) .and. (tab_err .ne. -100)) GoTo 200
                  PWF%P_WF_ABSTERM(I) = PWF%P_WF_ABSTERM(I) + &
                       TG_INFO(IG)%ND(I)*(ABS_COEFF_ACT2 - ABS_COEFF_ACT)/&
                       (PPP(I)*(PT_SCALE - 1.0D0))*PPP(I)
               end if
            end do

         else

            ! LINE_SEARCH and VOIGT_SHAPE subroutines are in line_shapes.f90
            CALL LINE_SEARCH (IG, WL, LINES_FOUND, ISTART, IEND)
            IF(.NOT.LINES_FOUND) CYCLE
            If(Allocated(UseLine)) DeAllocate(UseLine)
            Allocate(UseLine(IEND - ISTART + 1))
            UseLine = .True.
            DO I = NLAYERS, 1, -1
               CALL VOIGT_SHAPE(ISTART, IEND, IG, WL, I, 1, TTT(I), &
                    PPP(I), ABS_COEFF_ACT, *880, UseLine)
               TG_CROSS_SEC(I,KK) = ABS_COEFF_ACT

               ! TEMPERATURE W/F TERM in lbl mode
               ! ================================
               IF(INQ_WF('temp', 'WFN')) THEN
                  CALL VOIGT_SHAPE(ISTART, IEND, IG, WL, I, 2, &
                       TTT(I)*PT_SCALE, PPP(I), ABS_COEFF_ACT1, *880, UseLine)
                  PWF%T_WF_ABSTERM(I) = PWF%T_WF_ABSTERM(I) + &
                       TG_INFO(IG)%ND(I)*(ABS_COEFF_ACT1 - ABS_COEFF_ACT)/&
                       (TTT(I)*(PT_SCALE - 1.0D0))
               END IF
               IF(INQ_WF('press', 'WFN')) THEN
                  CALL VOIGT_SHAPE(ISTART, IEND, IG, WL, I, 1, TTT(I),&
                       PPP(I)*PT_SCALE, ABS_COEFF_ACT2, *880, UseLine)
                  PWF%P_WF_ABSTERM(I) = PWF%P_WF_ABSTERM(I) + &
                       TG_INFO(IG)%ND(I)*(ABS_COEFF_ACT2 - ABS_COEFF_ACT)/&
                       (PPP(I)*(PT_SCALE - 1.0D0))*PPP(I)
               END IF
            ENDDO

         end if

      CASE ('off')
         TG_CROSS_SEC(:,KK) = 0.d0

      CASE DEFAULT
         CALL WRITE_MESSAGE (R_NAME, 'ERROR', 'Wrong gas type !')
         RETURN 1

      END SELECT
   END DO

   ! WRITE(6,*) ' Perturbation num_abs:', NUM_ABS, NUM_TEM 
   ! PAUSE

   ! IF( NUM_ABS .OR. NUM_TEM ) THEN
   IF( NUM_ABS ) THEN
      INDG = GAS_INDEX(IND_NUM_WF)
      WRITE(6,*) ' Perturbation for component:', INDG
      TG_CROSS_SEC(:,INDG) = TG_CROSS_SEC(:,INDG)*PERTURB(:)
   END IF

   KK = 0
   DO IG = 1, Size(TG_INFO)
      If(.Not.TG_INFO(IG)%FM_FLG) Cycle
      KK = KK + 1
      IF(DO_PHOTOCHEM(IG)) THEN
         DO J = 1, NLAYERS
            TGA_PC(J,:) = TGA_PC(J,:) + &
                 CONC_PC(J,PC_GAS_INDEX(KK),:)*TG_CROSS_SEC(J,KK)
         END DO
      ELSE
         TGA(:) = TGA(:) + TG_INFO(IG)%ND(:)*TG_CROSS_SEC(:,KK)
      END IF
   END DO
      
   IF(ANY(CIA_INFO%Flg)) THEN
      DO I_P = 1, Size(CIA_INFO)
         If(.Not.CIA_INFO(I_P)%Flg) Cycle
         If(.Not.Allocated(CIA_INFO(I_P)%DB(JPOS)%TEMP)) Cycle
         ! Number of temperatures
         SELECT CASE(Size(CIA_INFO(I_P)%DB(JPOS)%TEMP)) 
         CASE (0)
            CYCLE
         CASE (1)
            CIA_CS(:) = DIM(CIA_INFO(I_P)%DB(JPOS)%CS(1), 0.0D0)
            TGA(:) = TGA(:) + CIA_INFO(I_P)%DENS(:)*CIA_CS(:)
            CYCLE ! Temperature WFs cannot be calculated
         END SELECT

         CS_DB => CIA_INFO(I_P)%DB(JPOS)%CS
         T_DB => CIA_INFO(I_P)%DB(JPOS)%TEMP 

         N_T = Size(T_DB)
         DO I = 1, NLAYERS
            CALL GET_IDX_UNSRT (TTT(I), N_T, T_DB, II_1, II_2)

            IF(TTT(I) >= MAXVAL(T_DB).OR.TTT(I) <= MINVAL(T_DB)) THEN
               CIA_CS(I) = DIM(CS_DB(II_1), 0.0D0)
               CYCLE ! Temperature WFs cannot be calculated
            END IF
               
            D_CIA_CS = (CS_DB(II_2) - CS_DB(II_1))/(T_DB(II_2) - T_DB(II_1))
            CIA_CS(I) = DDIM(CS_DB(II_1) + &
                 (TTT(I) - T_DB(II_1))*D_CIA_CS, 0.0D0)

            IF(.NOT.INQ_WF('temp', 'WFN')) CYCLE
            IF(CS_DB(II_1) <= 0.0D0.AND.CS_DB(II_2) <= 0.D0) CYCLE

            PWF%T_WF_ABSTERM(I) = PWF%T_WF_ABSTERM(I) + &
                 CIA_INFO(I_P)%DENS(I)*D_CIA_CS
         END DO

         TGA(:) = TGA(:) + CIA_INFO(I_P)%DENS(:)*CIA_CS(:)

      END DO
   END IF
   !==============================================================
   COMPARE_TO_GSS = .FALSE.
   IF( COMPARE_TO_GSS ) THEN
      OPEN(NewUnit=CU,file = './Benchmark/Japan_GSS/Inp-1.dat', STATUS='OLD')
      DO I = 1,7
         READ(CU,*)
      ENDDO
      READ(CU,*) J, (Thel(KK), KK=1,6)

      Thel(2:3) = 0.d0

      TGA(2) = (Thel(1) - Thel(2) - Thel(3))/(ZZZ(1) - ZZZ(2))
      TGA(1) = TGA(2)
      ! LOA%SCR(2) = Thel(3)/(ZZZ(1) - ZZZ(2))
      ! LOA%SCR(1) = LOA%SCR(2)
      DO I = 2,49
         READ(CU,*) J, (Thel(KK), KK=1,6)
         TGA(I+1) = 2.*(Thel(1) - Thel(2) - Thel(3)) &
              /(ZZZ(I) - ZZZ(I+1)) - TGA(I)
         ! LOA%SCR(I+1) = 2.*Thel(3) &
         !     /(ZZZ(I) - ZZZ(I+1)) - LOA%SCR(I)
      ENDDO
      CLOSE(CU)
   ENDIF  ! COMPARE_TO_GSS

   IF( ALIN_ACT(1) ) TGA(:) = ALIN_ABS(JPOS,:)

   IF( SOUT_FLAG ) THEN
      WRITE(TMP_STR,'(I0)') NLAYERS 
      FMT_STR = '('//TRIM(TMP_STR)//'(2x,E15.7))'
      IF( SOUT_TAU ) THEN
         WRITE(78,FMT_STR) ((TGA(I)+TGA(I+1))*DZR(I)*0.5D0,I=1,NLAYERS-1)
      ELSE
         WRITE(78,FMT_STR) (TGA(I),I=1,NLAYERS)
      ENDIF
   ENDIF

   LOA%AL(:) = TGA(:) + LOA%SCR(:) + LOA%EXA(:) + LOA%EXC(:) + LOA%EXI(:)

!   DO J = 1, NLAYERS
!      WRITE(*,*) J, LOA%EXC(J),  LOA%EXI(J)
!   ENDDO
!   PAUSE
   ! WRITE(*,*) '   st_abs_coeff.f  LOA%AL(NLAYERS) and EXA'
   ! WRITE(*,'(4x,ES12.4)') LOA%AL(NLAYERS)
   ! WRITE(*,'(2x,*(1x,ES12.4))') LOA%EXA(:)
   IF( LOG_LEVEL(1) >= N_ESO ) THEN
      WRITE(LUNIT,*) '   SUM(TGA) ', SUM(TGA) 
      WRITE(LUNIT,*) '   SUM(SCR) ', SUM(LOA%SCR) 
      WRITE(LUNIT,*) '   SUM(EXA) ', SUM(LOA%EXA) 
      WRITE(LUNIT,*) '   SUM(EXC) ', SUM(LOA%EXC) 
      WRITE(LUNIT,*) '   SUM(EXI) ', SUM(LOA%EXI) 
   ENDIF
   IF(DO_PC) THEN
      DO J = 1, NLAYERS
         TGA_PC(J,:) = TGA_PC(J,:) + LOA%AL(J)
      END DO
      CALL GT_PC_INIT (TGA_PC, PCC)
   END IF
   
   CALL ST_TRACK (R_NAME, 1, 'O')
   RETURN
   
200 Continue
   call write_message(r_name, 'error', '')
   write(EU, *) "table_interpolate returns: ", tab_err
   write(EU, *) "absorber: ", TG_INFO(IG)%Name
   write(EU, *) "wavelength: ", pos
   write(EU, *) "layer: ", i
   Return 1
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
   RETURN 1

!****************************************************************************
 End Subroutine ST_ABS_COEFF
!****************************************************************************
