c***********************************************************************
c
c       SCIATRAN - Version 4.3 - Last modification: 27 Jan 2021
c
c       Copyright (C) - Institute of Remote Sensing,
c                       University of Bremen, Germany
c
c       Use adjoint RTE for calculation of weighting function of 
c        atmospheric parameters and rotational Raman scattering
c
c***********************************************************************
      SUBROUTINE ST_ADJOINT_A(JPOS, POS, JJESFT, Iot, Mh, E_W, DO_OUT, 
     $     ADS, AGR_A, ADO_A, GSL_A, IOP_A, LAS_A, LOA, LWF, Ph_zA, 
     $     Ph_R, Ph_A, Ph_dA, Ph_C, PWF, SWF, SDO_A, SLS, SRR, SRR_Jact,  
     $     TE, TAC, Abs_soot, UDB_A, ASOL_ADJ, OSM, SSB, PH_M, RADOUT,*)
c     ... Called by "st_dom_tau.f"
      
      USE ABSORBERS,   ONLY: N_GAS
      USE AEROSOLS,    ONLY: N_dwr
      USE CLOUD_INPUT, ONLY: Plus_ice
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE GRIDS,       ONLY: NL => NLAYERS
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: NL_pla, NL_soot, Nvs
      USE LOG_PARS,    ONLY: LUNIT, N_ESO, LOG_LEVEL, EU, DIR_OUT
      USE GT_POLYN,    ONLY: Leg => NLEGEN, AG, AG1 
      USE RTM_OUTPUT,  ONLY: N_OP, TRANS_OUT
      USE ST_VARS,     ONLY: DO_RAMAN, N_Adj, RTM_TYPE, SDVI, WF_FLAG,
     $                       SCATT_MODE, SOLVER, VS_HC, BRDF_FLAG, 
     $                       NS_wf, RET_MODE, Ray_to_WF
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE TOOLS,       ONLY: INQ_WF, FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: ADO_TYPE, LOA_TYPE, PWF_TYPE, SDO_TYPE,  
     $                       GSL_TYPE, LAS_TYPE, IOP_TYPE, RRS_TYPE, 
     $                       LWF_TYPE, TE_TYPE,  UDB_TYPE, PHA_CLD, 
     $                       AGR_type, OSM_TYPE, SSB_TYPE, PHA_MATR,
     $                       PH_MET, SWF_type, SLS_type
c     ------------------------------------------------------------------

c     -------------------------------------------------------------
c     ... Adjoint DOM auxiliary spherical geometry |
c     ----------------------------------------------
      TYPE ADS_type
        SEQUENCE
        LOGICAL, POINTER :: Act_ag(:,:)
        REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
      END TYPE ADS_type
      TYPE (ADS_type) :: ADS
c     -----------------
      TYPE (PH_MET) :: Ph_R
c     -----------------
      TYPE phase_tdz
       SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zA, Ph_A
c     -------------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     --------------------------
      TYPE (SLS_type) :: SLS
      TYPE (ADO_type) :: ADO_A  ! Adjoint DOM auxiliary variables
      TYPE (AGR_type) :: AGR_A
      TYPE (IOP_TYPE) :: IOP_A  ! Integral optical parameters
      TYPE (LAS_TYPE) :: LAS_A  ! Linear algebraic system
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE (GSL_type) :: GSL_A  ! Angles
      TYPE (PWF_type) :: PWF    ! General weighting function
      TYPE (RRS_type) :: SRR
      TYPE (SDO_type) :: SDO_A  ! DOM auxiliary solution variables
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (SWF_type) :: SWF    ! "Spherical" weighting function 
      TYPE (TE_type)  :: TE     ! Thermal emission 
      TYPE (UDB_TYPE) :: UDB_A  ! Boundary conditions
      TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
      TYPE (PHA_MATR) PH_M
c     ----------------------------------------------------
c     ... Input-output parameters
      INTEGER,    INTENT(IN) :: JPOS, JJESFT, Iot, Mh, SRR_Jact
      REAL (KND), INTENT(IN) :: POS, E_W, TAC(NL,N_GAS), 
     $     Abs_soot(NL_soot), ASOL_ADJ(N_Adj,Nvs*GSL_A%NSZA)
      REAL (KND)  RADOUT(N_OP,N_SL,Nvs)
      LOGICAL DO_OUT
c     ... Local parameters
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'ST_ADJOINT_A'
      INTEGER  I, J, K, L, Ka, K1, Js, Jv, Jg
      INTEGER  NN, NNv, NrL, NSZA, NLSA, LL1, LR1
      INTEGER, POINTER :: LAY_GR(:,:)
      REAL (KND) EWH, ALBEDO
      REAL (KND), POINTER :: XM(:), AXM(:), MuS(:), MuV(:), EXS(:,:),
     $     EXL(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      NrL = AGR_A%NrL  ;  LAY_GR => AGR_A%LAY_GR
      EXS => SSB%EXS
      ALBEDO = UDB_A%ALBEDO
      NN  = GSL_A%NN ; NSZA = GSL_A%NSZA ; NLSA = GSL_A%NLSA
      NNv = Nvs*NN
      XM => GSL_A%XM ; AXM => GSL_A%AXM  ; MuS => GSL_A%MuS
      MuV => GSL_A%MuV
      EXL => GSL_A%EXL

c     ... The following IF-construction is implemented to avoid
c         usage of arrays which are defined alone with solution
c         of the pseudo-forward equation. This solution is not exist
c         for non-actual wavelengths if APPR_RRS is set to 
c         'simple binning' (see setting for SOLVER in "rtm_main.f")
      IF( TRIM(SOLVER)=='Solv_MIX' ) THEN
         DO K = 1,NRL-1
            DO Jv = 1,N_Adj
               DO I = LAY_GR(K,1), LAY_GR(K,2)
                  ADO_A%LVEC(2*NNv*(I-1)+1:2*NNv*I,Jv) = 
     $                 LAS_A%SVEC_AG(2*NNv*(K-1)+1:2*NNv*K,Jv+SDVI)
               END DO
            ENDDO
         ENDDO

         IF( TRIM(SCATT_MODE) == 'ms' ) THEN
c         ... Intensity field of the pseudo-forward RTE at Gauss 
c             angles for all viewing angles: AFI(NL,2*NN,N_Adj) (ADO_)
            IF( AGR_A%ADAPTIVE_GRID ) THEN
               SELECT CASE ( TRIM(RTM_TYPE) )
               CASE( 'spher_scat' ) 
                  CALL GAU_Adj_UP_DOWN_SPH
     $                 (NL, NN, NLSA, Mh, ALBEDO, GSL_A, 
     $                 IOP_A, Ph_zA%Egg, SDO_A, SLS, ADS, ADO_A, UDB_A,
     $                 LAS_A%Ref_surf )
               CASE DEFAULT         
c                  write(*,*) '     Adaptive Afi Mh = ', Mh
                  CALL GAU_Adj_UP_DOWN
     $                 (NL, NN, NLSA, Mh, ALBEDO, XM, AG1, MuV, EXL, 
     $                 IOP_A, Ph_zA%Egg, SDO_A, Ph_zA%Egl, ADO_A, UDB_A,
     $                 LAS_A%Ref_surf )
               END SELECT
            ELSE
               CALL Field_Adj_Gau(NL, NN, ADO_A%LVEC, SDO_A, 
     $              ADO_A%PART_TU, ADO_A%PART_TD, ADO_A%AFI, 
     $              LAS_A%SOL_TOP)
            ENDIF
         ENDIF  ! SCATT_MODE == 'ms'
      ENDIF  ! 'Solv_MIX'

      IF( DO_RAMAN.AND.Mh<=3 ) THEN
         IF( SRR%Exi_WL(JPOS).AND.Mh==3 ) DO_OUT = .TRUE.
c        ... Select appropriate routine to calculate contribution
c            of the rotational Raman scattering in the framework
c            of the forward-adjoint technique
         SELECT CASE( TRIM(RTM_TYPE) )
         CASE( 'pp_scat','ps_scat' ) 
            CALL ST_PP_RRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, IOP_A%DFI, 
     $           ADO_A%AFI, EXL, EXS, MuV, LOA, SRR, SRR_Jact)
         CASE( 'spher_scat' ) 
            CALL ST_SPH_RRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, IOP_A%DFI, 
     $           ADO_A%AFI, EXL, EXS, MuV, LOA, SRR, SRR_Jact, SWF )
         CASE DEFAULT
            WRITE(*,*) '   For "RTM_TYPE" = ', TRIM(RTM_TYPE)
            WRITE(*,*) '   Raman scattering is not implemented yet'
            STOP '   Sorry! '
         END SELECT
      ENDIF  ! Raman and Mh<=3

c     ... Skip weighting function calculation
c     ---------------------------------------
      IF( .NOT.WF_FLAG ) GOTO 990

c     ... This subroutine is located in "st_flux_spher_alb.f"
c         Calculation of diffuse and direct transmission
      IF( Mh==1.AND.TRANS_OUT ) CALL ST_TRANS(JPOS, JJESFT, E_W, NL,  
     $     GSL_A, SSB%EXS, IOP_A%DFI, ADO_A%AFI)

      ALLOCATE( LWF%WF_ext(NL,N_SL,NS_wf), LWF%WF_TE(NL_pla,N_SL) )
      IF(INQ_WF('temp', 'WFN')) 
     $     ALLOCATE( LWF%WF_TE_TEM(NL_pla,N_SL) )
      IF( Ray_to_WF ) ALLOCATE( LWF%WF_scr(NL,N_SL,NS_wf) )

c     ... Preliminary replace Rayleigh
      IF(INQ_WF('atom', 'WFT')) 
     $     ALLOCATE( LWF%WF_met(NL,N_SL,NS_wf) )

      IF( INQ_WF('aer', 'WFT') ) THEN 
         ALLOCATE( LWF%WF_sca(NL,N_SL,NS_wf,1+N_dwr) )
         LWF%WF_sca = 0.d0
      ENDIF

      IF( INQ_WF('cld', 'WFT') ) THEN
         ALLOCATE( LWF%WF_scc(NL,N_SL,NS_wf), LWF%DT1(NL), LWF%DT2(NL),
     $        LWF%Dph_Dre(NL,N_SL,NS_wf), ! A
     $        LWF%SSC_DM(NL,N_SL,NS_wf) ) ! A
         LWF%SSC_DM = 0.d0
         IF( ANY(Plus_ice) ) ALLOCATE( LWF%WF_scm(NL,N_SL,NS_wf), 
     $        LWF%DT2m(NL) )    ! A
      ENDIF

c    *******************************************************************
c     ... Calculation of weighting functions for different atmospheric
c                 parameters and surface albedo
c    *******************************************************************

      IF( TRIM(RTM_TYPE) /= 'spher_scat' ) THEN 
c     ... Solution of pseudo-forward RTE in the solar direction      
c         output: IOP_A%Ref_sol(NL,NLSA,NSZA) used for WF calculation
         CALL GAU_Adj_UP_SOL(NL, NN, NLSA, NSZA, XM, AG1, MuV, MuS, 
     $        EXL, SDO_A, IOP_A, Ph_zA, ADO_A, ASOL_ADJ)
      ENDIF  ! 'pp_scat', 'ps_scat'

c     ... Take into account coefficient in the Fourier series
c         for summation of weighting functions. 
c             ---------------------------------          
c         The zeroth Fourier harmonic has coefficient:
c           1   - in the scalar case (VS_HC = 2 )
c           0.5 - in the vector case (VS_HC = 1 )
      EWH = E_W*VS_HC/2.d0
      IF( Mh > 1 ) EWH = VS_HC*E_W

      SELECT CASE (TRIM(RTM_TYPE))
      CASE( 'pp_scat' ) 
         IF(INQ_WF('elev', 'WFN')) THEN
            CALL ST_ELEVATION_WF(NSZA, NN, JPOS, IOP_A%Ref_sol(NL,:,:), 
     $           IOP_A%Ref_dir(NL,:,:), EXS(NL,:), EXL(NL,:), MuV, Mh, 
     $           LOA%SCR(NL), LOA%SCA(NL), LOA%AL(NL), 
     $           Ph_zA%Egs(NL,:,:,:), 
     $           Ph_zA%Egg(NL,:,:,:), Ph_zA%EgL(NL,:,:,:), 
     $           Ph_zA%EsL(NL,:,:,1), EWH, IOP_A%DFI(NL,:,:), 
     $           ADO_A%AFI(NL,:,:))
         ENDIF

         CALL ST_PP_WF(NN, NSZA, Mh, LWF, IOP_A%DFI, ADO_A%AFI, EXL, 
     $        EXS, IOP_A%Ref_sol, IOP_A%Ref_dir, MuV, LOA, Ph_R, Ph_A,  
     $        Ph_dA, Ph_C, TE%ADD_PLANCK, TE%NPLANCK)

      CASE( 'ps_scat' ) 
         CALL ST_PS_WF(NN, NSZA, Mh, LWF, IOP_A%DFI, ADO_A%AFI, EXL, 
     $        EXS, IOP_A%Ref_sol, IOP_A%Ref_dir, MuV, LOA, Ph_R, Ph_A,   
     $        Ph_dA, Ph_C, SWF, Ph_zA%EsL, TE%ADD_PLANCK, TE%NPLANCK,  
     $        Ph_zA%Egs, MuS)

      CASE( 'spher_scat' )
c         WRITE(*,*) '  EXL  '
c         WRITE(*,*) EXL(:,1)
c        ... Elements of PWF structure are calculated in "spher_wf.f"
         CALL ST_SPH_WF(JPOS, NN, NSZA, Mh, PWF, LOA, IOP_A%DFI,  
     $        ADO_A%AFI, Ph_zA%Egs, ALBEDO, ADO_A%FLUX_D, EXL(NL,:),   
     $        Ph_A%Egg, Ph_dA, Ph_R%Egg, ADS%Pol_ga, SWF, 
     $        TE%ADD_PLANCK, TE%NPLANCK, TE%OPLANCK_SURF, LWF, PH_M,
     $        LAS_A%Ref_surf, UDB_A%LBC, UDB_A)
 
      END SELECT

c     ... Single scattering correction algorithm for radiance and 
c         weighting functions in the plane-parallel and pseudo-spherical
c         geometry 
      IF( TRIM(RTM_TYPE)/='spher_scat'.AND.DO_SSRCA.AND.Mh==1) THEN 
         CALL GPP_SS_STOKES(JPOS, JJESFT, Iot, POS, ALBEDO,   
     $        E_W, OSM, GSL_A, LOA, LWF, PWF, SSB, TE, SRR, SRR_Jact,   
     $        RADOUT, TAC, Abs_soot, UDB_A%FLU_int, *880)
      ENDIF  ! 'pp_scat', 'ps_scat'
      
c      WRITE(*,*) '  Dimension of TAC:  ', SHAPE(TAC)
      IF(INQ_WF('tg', 'WFT')) 
     $     CALL SET_OUTPUT_CWF(JPOS, LWF, TE%ADD_PLANCK, TAC, EWH)

      IF( INQ_WF('aer', 'WFT') ) CALL SET_OUTPUT_AER(JPOS, LWF, 
     $     LOA%SCA, LOA%EXA, TE%ADD_PLANCK, EWH, POS)
     
      IF( INQ_WF('cld', 'WFT') ) THEN
         CALL SET_OUTPUT_CLD(JPOS, LWF, LOA%Cil, LOA%Cir, LOA%DP_CLD, 
     $        LOA%EXC, LOA%SCC, LOA%DM_CLD, Abs_soot, TE%ADD_PLANCK,  
     $        EWH, LOA%SCC_NOSCALE(:,1))
         IF( ANY(Plus_ice) ) 
     $        CALL SET_OUTPUT_MIX(JPOS, LWF, LOA%Cil, LOA%Cir,   
     $        LOA%DP_CLD, LOA%EXI, LOA%SCI, TE%ADD_PLANCK, EWH)
      ENDIF

      IF(INQ_WF('press', 'WFN')) CALL SET_OUTPUT_PWF(JPOS, LWF, 
     $     LOA%SCR, TAC, TE%ADD_PLANCK, PWF%P_WF_ABSTERM, EWH)

      IF(INQ_WF('temp', 'WFN')) CALL SET_OUTPUT_TWF(JPOS, LWF, 
     $     LOA%SCR, TAC, TE%ADD_PLANCK, PWF%T_WF_ABSTERM, EWH)

      IF(INQ_WF('ray', 'WFN')) 
     $     CALL SET_OUTPUT_RAY(JPOS, LWF, LOA%SCR, EWH)

      IF(INQ_WF('atom', 'WFT')) THEN
c         Ka = 1
c         CALL SET_OUTPUT_MET(JPOS, LWF, LOA%EMC(Ka,:), EWH)
         Ka = FIND_IDX('atom', LOA%TG_CS%TYPE)
         CALL SET_OUTPUT_MET(JPOS, LWF, LOA%TG_CS(Ka)%CS(:), EWH)
      ENDIF

      IF(INQ_WF('prosail','RIT')) THEN
         CALL GENERAL_BRDF_WF(NN, NSZA, NLSA, Mh, JPOS, POS, EWH, 
     $        MuS, MuV, EXL(NL,:), EXS(NL,:),
     $        ADO_A%AFI(NL,:,:), UDB_A, LAS_A%SOL_BOT)
      ENDIF

      IF(INQ_WF('albedo', 'WFN')) THEN 
         IF( BRDF_FLAG ) THEN
            CALL ST_BRDF_WF(NN, NSZA, NLSA, Mh, JPOS, POS, EWH, MuS, 
     $           MuV, ADO_A%FLUX_D, EXL(NL,:), LAS_A%FLUX, EXS(NL,:), 
     $           ADO_A%AFI(NL,:,:), UDB_A, LAS_A%Ref_surf,
     $           LAS_A%SOL_BOT)
         ELSE
            CALL ST_ALBEDO_WF(NSZA, NLSA, Mh, JPOS, EWH, MuS, ALBEDO,
     $           ADO_A%FLUX_D, EXL(NL,:), LAS_A%FLUX, EXS(NL,:), 
     $           SWF%IEffS )
         ENDIF
      END IF

c      IF(INQ_WF('flu_sfl', 'WFN')) CALL ST_SFL_WF(NSZA, NLSA, Mh, 
      IF(INQ_WF('flu', 'WFN')) CALL ST_SFL_WF(NSZA, NLSA, Mh, 
     $     JPOS, POS, EWH, ADO_A%FLUX_D, EXL(NL,:), UDB_A%Flu_int)

c     ... Deallocate all elements of the structure LWF
      DEALLOCATE( LWF%WF_ext, LWF%WF_TE )
      IF(INQ_WF('temp', 'WFN')) DEALLOCATE( LWF%WF_TE_TEM )
      IF( Ray_to_WF )  DEALLOCATE( LWF%WF_scr )
      IF(INQ_WF('atom', 'WFT')) DEALLOCATE( LWF%WF_met )
      IF(INQ_WF('aer', 'WFT'))  DEALLOCATE( LWF%WF_sca )
      IF(INQ_WF('cld', 'WFT')) THEN
         DEALLOCATE( LWF%WF_scc, LWF%DT1, LWF%DT2, LWF%Dph_Dre,
     $        LWF%SSC_DM) ! D
         IF( ANY(Plus_ice) ) DEALLOCATE( LWF%WF_scm, LWF%DT2m )
      ENDIF

 990  CONTINUE

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      END
