c***********************************************************************
c
c	SCIATRAN - Version 4.5 - Last modification: 18 Jun 2021
c
c	Copyright (C) 1995-2010, Institute of Remote Sensing,
c			         University of Bremen, Germany
c
c       Use adjoint RTE for calculation of weighting function of 
c        atmospheric parameters and rotational Raman scattering
c
c***********************************************************************
      SUBROUTINE ST_ADJOINT_O(JPOS, JJESFT, Mh, E_W, UDB_W, EXS, 
     $     AGR, ADO, GSL, IOP, LAS, LOW, SDO, SVR, TE, Ph_zW, 
     $     Ph_rW, Ph_Us, Ph_Ks, Ph_Kl, ASOL_ADJ, DO_OUT)
c     ... Call in "st_dom_tau.f"
c              EXS <= SSB_W%EXS
c              AGR <= AGR_W ; ADO <= ADO_W ; GSL <= GSL_W
c              IOP <= IOP_W ; LAS <= LAS_W ; SDO <= SDO_W

      USE GT_POLYN,    ONLY: AG_W
      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: OWF_FLG_ARR
      USE SPHER_PARS,  ONLY: N_SL => N_LOS_OUT
      USE ST_VARS,     ONLY: N_Adj, SDVI, OWF_FLAG, SCATT_MODE, OWF_CHL,
     $                       OWF_Ray, OWF_Use, SOLVER
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: ADO_TYPE, LOW_TYPE, SDO_TYPE, UDB_TYPE, 
     $                       GSL_TYPE, LAS_TYPE, IOP_TYPE, TE_TYPE, 
     $                       AGR_type, VRS_TYPE, PH_MET
      USE WATER,       ONLY: INEL_ADJ
      USE WATER_ICE,   ONLY: IWS
c     ------------------------------------------------------------------

      TYPE (PH_MET) :: Ph_rW, Ph_Ks, Ph_Kl
c     -----------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_zW, Ph_Us
c     -------------------------------
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (AGR_type) :: AGR    ! Altitude grid
      TYPE (IOP_TYPE) :: IOP    ! Integral optical parameters
      TYPE (LAS_TYPE) :: LAS    ! Linear algebraic system
      TYPE (LOW_type) :: LOW    ! Local optical parameters of ocean
      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (TE_type)  :: TE     ! Thermal emission 
      TYPE (UDB_TYPE) :: UDB_W  ! Boundary conditions
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
c     ... Input parameters
      INTEGER,    INTENT(IN) :: JPOS, JJESFT, Mh
      REAL (KND), INTENT(IN) :: E_W, EXS(AGR%NL,GSL%NSZA), 
     $     ASOL_ADJ(N_Adj,Nvs*GSL%NSZA)
      LOGICAL DO_OUT
c     ... Local parameters
c     -------------
      TYPE WFO_type             ! Local weighting functions (ocean)
      SEQUENCE
      REAL (KND), POINTER :: WF_ext(:,:), WF_sca(:,:,:)
      END TYPE WFO_type 
      TYPE (WFO_type) :: WFO
c     ----------------------
      CHARACTER (LEN=12), PARAMETER :: R_NAME = 'ST_ADJOINT_O'
      INTEGER  I, J, K, L, K1, Js, Jv, Jg
      INTEGER  NN, NL, NrL, NSZA, NLSA
      INTEGER, POINTER :: LAY_GR(:,:)
      REAL (KND) EWH, AG1(GSL%NN)
      REAL (KND), POINTER :: XM(:), AXM(:), MuS(:), MuV(:), EXL(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NL = AGR%NL  ;  NrL = AGR%NrL  ;  LAY_GR => AGR%LAY_GR

c     ... Angles dimension:
c         Gauss             Solar            Line-of-sight
      NN   = GSL%NN    ; NSZA =  GSL%NSZA ; NLSA =  GSL%NLSA
c     ... Cosine of angles:
      XM  => GSL%XM    ; MuS  => GSL%MuS  ; MuV => GSL%MuV 
c     ... Weights
      AXM  => GSL%AXM  ; AG1(:) = AG_W(:)*0.5d0
c     ... Exponent 
      EXL  => GSL%EXL  

c     ... The following IF-construction is implemented to avoid
c         usage of arrays which are defined alone with solution
c         of the pseudo-forward equation. This solution is not exist
c         for non-actual wavelengths if APPR_RRS is set to 
c         'simple binning' (see setting for SOLVER in "rtm_main.f")
      IF( TRIM(SOLVER)=='Solv_MIX' ) THEN
         DO K = 1,NRL-1
            DO Jv = 1,N_Adj
               DO I = LAY_GR(K,1), LAY_GR(K,2)
                  ADO%LVEC(2*NN*(I-1)+1:2*NN*I,Jv) = 
     $                 LAS%SVEC_AG(2*NN*(K-1)+1:2*NN*K,Jv+SDVI)
               END DO
            ENDDO
         ENDDO
         IF( TRIM(SCATT_MODE) == 'ms' ) THEN
c         ... Intensity field of the pseudo-forward RTE at Gauss 
c             angles for all viewing angles: AFI(NL,2*NN,NLSA) (ADO_)
            IF( AGR%ADAPTIVE_GRID ) THEN
               CALL GAU_Adj_UP_DOWN
     $              (NL, NN, NLSA, Mh, UDB_W%ALBEDO, XM, AG1, MuV, EXL, 
     $              IOP, Ph_zW%Egg, SDO, Ph_zW%Egl, ADO, UDB_W,
     $              LAS%Ref_surf )
            ELSE
               CALL Field_Adj_Gau(NL, NN, ADO%LVEC, SDO, ADO%PART_TU, 
     $              ADO%PART_TD, ADO%AFI, LAS%SOL_TOP)
            ENDIF
         ENDIF
      ENDIF
      
      IF( INEL_ADJ .AND. Mh<=3 ) THEN
         IF( SVR%Exi_WL(JPOS) .AND. Mh==3 ) DO_OUT = .TRUE.
         CALL ST_PP_VRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, IOP%DFI,
     $        ADO%AFI, EXL, EXS, MuV, LOW, SVR )
      ENDIF

c     ... Skip weighting function calculation
c     ---------------------------------------
      IF( .NOT.OWF_FLAG ) GOTO 990

c     ... Auxiliary weighting function WFO%WF_sca(;,:,N_HPC+1) 
c         contains Rayleigh contribution and contribution of
c         N_HPC component of particulate mater
      ALLOCATE(WFO%WF_ext(NL,N_SL), WFO%WF_sca(NL,N_SL,IWS(1)%N_HPC+1) )

c    *******************************************************************
c     ... Calculation of weighting functions for different ocean
c                           parameters
c    *******************************************************************

c     ... Solution of pseudo-forward RTE in the solar direction      
c     ---------------------------------------------------------
c     ... output: IOP%Ref_sol(NL,NLSA,NSZA) used for WF calculation
      CALL GAU_Adj_UP_SOL(NL, NN, NLSA, NSZA, XM, AG1, MuV, MuS, 
     $     EXL, SDO, IOP, Ph_zW, ADO, ASOL_ADJ)

      EWH = E_W
c     ... Take into account coefficient in the Fourier series
      IF( Mh > 1 ) EWH = 2.d0*EWH

      CALL ST_PP_OWF(NN, NSZA, Mh, WFO, GSL, IOP, ADO%AFI, EXS, LOW,
     $     Ph_Us, Ph_rW, Ph_Ks, Ph_Kl, TE%ADD_PLANCK, TE%NPLANCK)

      IF(OWF_CHL) CALL SET_OUTPUT_CHL(JPOS, LOW, WFO, EWH)

      IF( ANY(OWF_FLG_ARR(2:5)) ) 
     $     CALL SET_OUTPUT_Psca(JPOS, LOW, WFO, EWH)

      DEALLOCATE( WFO%WF_ext, WFO%WF_sca )

 990  CONTINUE

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
