C***********************************************************************
C
C       SCIATRAN - Version 4.5 - Last modification: 12 Feb 2022
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c **********************************************************************
      SUBROUTINE ST_ALBEDO_WF(NSZA, NLSA, Mh, JPOS, E_W, MuS, ALBEDO,
     $     FLUX_D, EXL, FLUX, EXS, IEffS )
c     ... Call in "st_adjoint_a.f"

      USE ALB_PARS,   ONLY: ADB => ALBEDO_DB, Nc_ALB, WS1_A, WS2_A, 
     $                      DPOSALB, ALB_WEI, INF_ALB
      USE LOG_PARS,   ONLY: LUNIT, LOG_LEVEL
      USE INFO,       ONLY: WF_DB
      USE ST_VARS,    ONLY: RTM_TYPE, N_Adj, M_LIM, SHI_LIM_C, 
     $                      N_Eff, RET_MODE, GeoD => GeoD_OUT
      USE SPHER_PARS, ONLY: JV_S, JV_E, N_LOS_OUT, LW_FOV, 
     $                      N_SOL_ANG, N_AZ_VAR, NFA => N_FOV_ANG
      USE TOOLS,      ONLY: INQ_WF, E_CSZA
      USE TYPES    ,  ONLY: KND, Eff_Sol_Idx
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NSZA, NLSA, Mh, JPOS
      REAL (KND), INTENT(IN) :: ALBEDO, E_W, MuS(NSZA), Flux(NSZA), 
     $     EXS(NSZA), EXL(NLSA), FLUX_D(N_Adj)
      TYPE (Eff_Sol_Idx) :: IEffS(N_LOS_OUT,N_Eff)
c     ... Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_ALBEDO_WF'
      LOGICAL  FLG
      INTEGER  I, J, K, Ja, Js, Jv, JL, Iks, K1, II, Nr, N1, N2, IA
      REAL (KND)  RR1, S1, S2, ALB_WF(N_LOS_OUT), ASM(NLSA), ADR(NSZA), 
     $     ALB_COMP, CSOL, Muv
c     ==================================================================

      IF( Mh > 1 ) RETURN
      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1
      IF( TRIM(RET_MODE)=='aer-hom' ) THEN
         FLG = INQ_WF('albedo','WFN',II)
c        ... Weighting functions are basic albedo spectra
         DO I = 1, Nc_ALB
            WF_DB(II)%WF(JPOS,:,I,1) = E_W*( ADB(WS2_A,I)*DPOSALB 
     $           + ADB(WS1_A,I)*(1.d0 - DPOSALB) ) 
         ENDDO         
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

c     ... Lambertian albedo WF for downward or upward radiation
c     ---------------------------------------------------------
      SELECT CASE (TRIM(RTM_TYPE))
      CASE( 'spher_scat' )
         ADR(:) = (MuS(:)*EXS(:) + 2.0d0*FLUX(:)) ; K1 = 0  
         DO Js = 1,N_SOL_ANG
            DO Jv = Jv_S(Js),Jv_E(Js) ; Iks = SHI_LIM_C(Jv)
               Muv = GeoD%LVG(IA,Jv)%C_VA(1)
               DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
                  IF( Muv > 0.d0 ) THEN ! reflected radiation
                     CSOL = E_CSZA(IEffS(K1,1),'G')
                     IF( NSZA == 1 ) THEN
                        RR1 = ADR(1)
                     ELSE
                        CALL AKIMA_INT( NSZA, MuS, ADR, 1, CSOL, RR1 )
                     ENDIF
                     ALB_WF(K1) = RR1*(EXL(Jv) + FLUX_D(Iks+1))
                     II = 2
                  ELSE
                    ALB_WF(K1) = 0.d0 ; II = 1
                  ENDIF
                  DO JL = II, M_LIM( Jv*NFA - LW_FOV(Jv) ) 
                     CSOL = E_CSZA(IEffS(K1,JL),'G')
                     IF( NSZA == 1 ) THEN
                        RR1 = ADR(1)
                     ELSE
                        CALL AKIMA_INT( NSZA, MuS, ADR, 1, CSOL, RR1 )
                     ENDIF
                     ALB_WF(K1) = ALB_WF(K1) + RR1*FLUX_D(Iks+JL)
                  ENDDO                        
               ENDDO
            ENDDO
         ENDDO
      CASE DEFAULT
c        ... Plane-parallel and pseudo-spherical case
         J = 0
         DO I = 1,NSZA
            DO Jv = Jv_S(I),Jv_E(I) ; Muv = GeoD%LVG(IA,Jv)%C_VA(1)
               ASM(:) = FLUX_D(:) ! See Eq.(WF-Lamb-tra) in STB
c                                   See Eq.(WF-Lamb-refl) in STB
               IF( Muv > 0.d0 ) ASM(:) = ASM(:) + EXL(:) 
               S1 = ASM(Jv)*(MuS(I)*EXS(I) + 2.d0*FLUX(I))
               DO Ja = 1,N_AZ_VAR  ;  J = J + 1
                  ALB_WF(J) = S1
               ENDDO
            ENDDO
         ENDDO
      END SELECT

      
      FLG = INQ_WF('albedo', 'WFN', II)
      IF( Nc_ALB > 1 ) THEN
         IF(INF_ALB(1) == 0 ) THEN
            DO I = 1, Nc_ALB
               S1 = ADB(WS1_A,I)  ;  S2 = ADB(WS2_A,I) 
               ALB_COMP = S1 + (S2 - S1)*DPOSALB
c              ... for the absolute variation of coefficients
               WF_DB(II)%WF(JPOS,:,I,1) = WF_DB(II)%WF(JPOS,:,I,1) 
     $              + ALB_WF(:)*E_W*ALB_COMP
            ENDDO
         ELSE
c           ... The surface reflection is approximated as ratio 
c               for all wavelengths shorter or equal to Lam(N2)
            Nr = INF_ALB(1)  ;  N2 = INF_ALB(3)
            IF( WS1_A <= N2 ) THEN
               ALB_COMP =  2.d0*ALB_WEI(1,1)*ADB(Nr,1)*ADB(JPOS,1) 
     $              + ADB(Nr,1)*ADB(JPOS,2) + ADB(Nr,2)*ADB(JPOS,1)
            ELSE
               ALB_COMP = ADB(JPOS,1)
            ENDIF
c              ... for the absolute variation of A-function coefficient
            I = 1
            WF_DB(II)%WF(JPOS,:,I,1) = WF_DB(II)%WF(JPOS,:,I,1) 
     $           + ALB_WF(:)*E_W*ALB_COMP
c                   WRITE(*,*) '   JPOS = ', JPOS
c                   WRITE(*,*) '          ', WF_DB(II)%WF(JPOS,:,I,1)
         ENDIF
      ELSE
c     ... Nc_ALB = 1 
         S1 = E_W
c         WRITE(*,*) '   WF_DB(II)%DIM = ', WF_DB(II)%DIM
c         WRITE(*,*) '   N_LOS_OUT     = ', N_LOS_OUT  
         IF( WF_DB(II)%DIM == 'rel') S1 = ALBEDO*S1
         WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1) 
     $        + S1*ALB_WF(:)
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
