C********************************************************************C
C
C       SCIATRAN - Version 4.5 - Last modification: 16 Feb 2022
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c *********************************************************************
      SUBROUTINE ST_BRDF_WF(NN, NSZA, NLSA, Mh, JPOS, POS, E_W, MuS, 
     $     MuV, FLUX_D, EXL, FLUX, EXS, AFI, UDB_A, Ref_surf, SOL_BOT)
c     ... Call in "st_adjoint_a.f"

      USE BRDF,        ONLY: Nwl_veg, WL_veg, Al_veg, Al_soi, Cfr_v, 
     $                       Npg, Exact_brf, N_svk, Wgr_svk, Alb_svk, 
     $                       AB_DB, Nwl_BRF, BRF_GG_WL, BRF_GS_WL,
     $                       Ker_cof => IVG_cof
      USE GT_POLYN,    ONLY: XM, AG, AG1, AXM
      USE INFO,        ONLY: WF_DB, RET_INFO
      USE INTERN_DIMS, ONLY: Nvs, Ng1_brf, Ng2_brf, Ns_brf
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_LOS_OUT, ANG_OUT, 
     $                       Naz => N_AZ_VAR
      USE ST_VARS,     ONLY: RTM_TYPE, N_Adj, N_RHS, SDVI, SURF_TYPE,
     $                       GeoD => GeoD_OUT, Pie
      USE TOOLS,       ONLY: FIND_IDX
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: UDB_TYPE
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NN, NSZA, NLSA, Mh, JPOS
      REAL (KND), INTENT(IN) :: E_W, POS, MuS(NSZA), FLUX_D(N_Adj), 
     $     EXS(NSZA), EXL(NLSA), Flux(NSZA),
     $     AFI(2*NN*Nvs,N_Adj), MuV(NLSA), Ref_surf(N_RHS,Nvs*(NN+Npg)),
     $     SOL_BOT(2*NN*Nvs,N_RHS)
c     -------------------------------
      TYPE (UDB_TYPE) :: UDB_A  ! Boundary conditions
c     ... Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_BRDF_WF'
      LOGICAL  LDE
      INTEGER  I, J, K, II, Ko, Ja, Js, Jss, Jv, JL, Iks, K1, Ws, Ws1, 
     $     IA, NNv, Kf, Nco, N1
      REAL (KND)  DC, Aveg, Asoi, Alam, S1, Ter12, Ter1
      REAL (KND)  ASM(NLSA), Adj_int(Nvs*NSZA), RHS(2*NN*Nvs),
     $     For_int(Nvs*NLSA), X_ref(NN+1)
      REAL (KND), ALLOCATABLE:: Lamb_wf(:), RL_COF(:), Rgg(:,:,:), 
     $     Rgs(:,:,:), Ter2(:,:,:), Ter3(:,:,:), Ter4(:,:,:),
     $     ALB_WF(:,:), AL_WF(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IA = 1  ;  NNv = NN*Nvs
c     ... Linear directional effect
c      LDE = .TRUE.
      LDE = .FALSE.
      IF( LDE ) THEN
         ALLOCATE( Lamb_wf(N_LOS_OUT) ) ; Lamb_wf(:) = 0.d0
      ENDIF
      SELECT CASE( TRIM(SURF_TYPE) )
      CASE('Ross-Li')                ; Nco = 3
      CASE('Ross-Li-snow','snow_fasmar') ; Nco = 4
      CASE('svk')                    ; Nco = 1
      END SELECT

      ALLOCATE( Rgg(N_RHS,Nvs*(NN+Npg),Nco), Rgs(Ng1_brf,Ns_brf,Nco),
     $     Ter2(NLSA,Nvs*NSZA,Nco), Ter3(NSZA,Nvs*NLSA,Nco),
     $     Ter4(NSZA,Nvs*NLSA,Nco),
     $     ALB_WF(N_LOS_OUT,Nco), AL_WF(N_LOS_OUT,Nco) )

      Rgg = 0.d0 ;  Rgs = 0.d0
      X_ref(1:NN) = XM(:) ; X_ref(NN+1) = 1.d0
      N1 = NN+1
      
      SELECT CASE( TRIM(SURF_TYPE) )
      CASE('Ross-Li','Ross-Li-snow','snow_fasmar')
         ALLOCATE( RL_COF(Nwl_BRF+1) )
         CALL LIN_SEARCH_NEW(POS, Dc, Ws, Ws1, Nwl_veg, WL_veg)
c        ... Kernels coefficients at actual wavelength         
         RL_COF(:) = (1.d0 - Dc)*Ker_cof(:,Ws) + Dc*Ker_cof(:,Ws1)
         IF( Mh==1 )  Rgs(:,:,1) = 1.d0 
         DO Kf = 1,Nco-1
            Rgs(:,:,Kf+1) = BRF_GS_WL(:,:,Mh,Kf)
         ENDDO  ! over kernel <except Iso> (Kf)

c         DO Js = 1, N_RHS
c            DO J = 1,Nvs
c               RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AXM(1:NN)
c            ENDDO
c            DO Kf = 1,Nco-1
cc              ... Reflected from surface (into Gauss nodes) forward and
cc                  pseudo-forward intensity corresponding to kernels R_vol and R_geo
c               DO K = 1, Nvs*(NN+Npg) 
c                  Rgg(Js,K,Kf+1)= SUM( BRF_GG_WL(K,:,Mh,Kf)*RHS(1:NNv) )
c               ENDDO
c            ENDDO  ! over kernel <except Iso> (Kf)
c            IF( Mh > 1 ) CYCLE
c            Rgg(Js,:,1) = SUM( RHS(1:NNv) )
c         ENDDO                  ! over rhs (Js)

         DO Js = 1, NSZA
            DO J = 1,Nvs
               RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AXM(1:NN)
            ENDDO
            DO Kf = 1,Nco-1
c              ... Reflected from surface (into Gauss nodes) forward
c                  intensity corresponding to kernels R_vol and R_geo
               DO K = 1, Nvs*(NN+Npg) 
                  Rgg(Js,K,Kf+1)= SUM( BRF_GG_WL(K,:,Mh,Kf)*RHS(1:NNv) )
               ENDDO
            ENDDO  ! over kernel <except Iso> (Kf)
            IF( Mh > 1 ) CYCLE
            Rgg(Js,:,1) = SUM( RHS(1:NNv) )
         ENDDO                  ! over rhs (Js)
         Ja = 0
         DO Js = NSZA + 1, N_RHS
            Ja = Ja + 1
            RHS(1:NNv:Nvs) = SOL_BOT(1:NN,Js)*AXM(1:NN) 
            DO Kf = 1,Nco-1
c              ... Reflected from surface (into Gauss nodes) pseudo-forward
c                  intensity corresponding to kernels R_vol and R_geo
               DO K = 1, Nvs*(NN+Npg) 
                  Rgg(Js,K,Kf+1)= SUM( BRF_GG_WL(K,:,Mh,Kf)*RHS(1:NNv) )
               ENDDO
            ENDDO  ! over kernel <except Iso> (Kf)
            IF( Mh > 1 ) CYCLE
            Rgg(Js,:,1) = SUM( RHS(1:NNv) )
         ENDDO  ! over rhs (Js)
         
      CASE('svk')
c        ... Ref_surf was calculated in "st_top_bot.f". It contains forward 
c            and pseudo-forward Stokes vector reflected from the 
c            surface into Gauss nodes
         Rgg(:,:,1) = Ref_surf(:,:)
         Rgs(:,:,1) = UDB_A%BRF_GS(:,:,Mh)
      END SELECT

      IF( Exact_brf )  STOP '   Exact_brf in "st_brdf_wf.f" '

      DO Jv = 1,NLSA  ;  Ko = Jv + SDVI
         DO Kf = 1,Nco
c           ... Interpolate reflected from the surface pseudo-forward  
c               intensity into solar directions 
            DO K1 = 1,Nvs 
               CALL AKIMA_INT(N1, X_ref, Rgg(Ko,K1:Nvs*N1:Nvs,Kf),  
     $              NSZA, MuS, Adj_int(K1:Nvs*NSZA:Nvs) )
            ENDDO  ! over Stokes components (K1)
c           ... Second term in Eq.(WF_BRDF-m) (see STB)
            Ter2(Jv,:,Kf) = Adj_int(:) *EXS(:)*Mus(:) ! Scalar only !
         ENDDO  ! over parameters of BRDF (Kf)
      ENDDO  ! over line-of-sight (Jv)

c     ... Ter1 - first term in Eq.(WF_BRDF-m) (see STB): scalar product of 
c         reflected from surface forward and pseudo-forward intensity
      DO Kf = 1,Nco ; J = 0
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)
               Ter1 = 2.d0*SUM( Rgg(Js,1:NN,Kf)*AFI(1:NN,Jv)*AXM(:) )
               Ter12 = Ter1 + Ter2(Jv,Js,Kf)
               DO Ja = 1,Naz  ;  J = J + 1
                  ALB_WF(J,Kf) = Ter12*DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over LoS angles (Jv)
         ENDDO  ! over solar angles (Js)
      ENDDO  ! over kernels of BRDF (Kf)

c     !!! Below for scalar case only !!! Reflected radiation
      IF( GeoD%VIEW(IA)%S=='up' ) THEN
c        ... Interpolate reflected from the surface  
c            forward intensity onto observation direction 
         DO Js = 1,NSZA
            DO Kf = 1,Nco
               DO K1 = 1,Nvs 
                  CALL AKIMA_INT(N1, X_ref, Rgg(Js,K1:Nvs*N1:Nvs,Kf),  
     $                 NLSA, Muv, For_int(K1:Nvs*NLSA:Nvs) )
               ENDDO  ! over Stokes components (K1)
c              ... Thrid term in Eq.(WF_BRDF-m) (see STB)
               Ter3(Js,:,Kf) = 2.d0*For_int(:)*EXL(:) 
            ENDDO  ! over parameters of BRDF (Kf)
         ENDDO  ! over solar angles (Js)

         DO Js = 1,NSZA  ;  Jss = (Js-1)*Nvs+1
            DO Kf = 1,Nco
               DO K1 = 1,Nvs
c                 ... Interpolation of directional reflection onto LoS direction
                  CALL AKIMA_INT(N1, X_ref, Rgs(K1:Nvs*N1:Nvs,Jss,Kf),
     $                 NLSA, Muv, For_int(K1:Nvs*NLSA:Nvs) )
               ENDDO
c              ... Fourth term in Eq.(WF_BRDF-m) (see STB)
               Ter4(Js,:,Kf) = EXS(Js)*EXL(:)*MuS(Js)*For_int(:)
            ENDDO
         ENDDO         
              IF( Mh==1.AND.LDE ) CALL LDE_1
         J = 0
         DO Js = 1,NSZA
            DO Jv = JV_S(Js),JV_E(Js)  
               DO Ja = 1,Naz  ;  J = J + 1
                  ALB_WF(J,:) = ALB_WF(J,:) + 
c     $                 Ter3(Js,Jv,:)
     $                 ( Ter3(Js,Jv,:) + Ter4(Js,Jv,:) )
     $                 *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
               ENDDO
            ENDDO
         ENDDO
      ENDIF   ! reflected radiation
      
      II = FIND_IDX('albedo', WF_DB%NAME)

      IF( LDE ) THEN
         CALL LDE_2 ; CALL ST_TRACK(R_NAME,1,'O') ; RETURN
      ENDIF

      SELECT CASE( TRIM(SURF_TYPE) )
c     ... Variation of the vegetation fraction
      CASE('svk')
         CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
         Aveg = (1.d0 - DC)*Al_veg(Ws) + DC*Al_veg(Ws1)
         Asoi = (1.d0 - DC)*Al_soi(Ws) + DC*Al_soi(Ws1)
         Alam  = (Aveg - Asoi)/(Cfr_v*Aveg + (1.d0-Cfr_v)*Asoi)
         AL_WF(:,1) = Alam*ALB_WF(:,1)*E_W
         IF( WF_DB(II)%DIM=='rel' ) AL_WF(:,1) = Cfr_v*AL_WF(:,1)
      CASE('Ross-Li','Ross-Li-snow','snow_fasmar')
         AL_WF(:,:) = ALB_WF(:,:)*E_W
         IF( WF_DB(II)%DIM=='rel' )  FORALL( Kf = 1:Nco)
     $        AL_WF(:,Kf) = RL_COF(Kf)*AL_WF(:,Kf)
c         WRITE(*,*) '  Al_wf(:,1)  '
c         DO I = 1, N_LOS_OUT
c            WRITE(*,'(2x,I2,*(2x,e12.4))') I, (AL_WF(I,Kf), Kf = 1,Nco)
c         ENDDO
      END SELECT

      WF_DB(II)%WF(JPOS,:,:,1) = WF_DB(II)%WF(JPOS,:,:,1) + AL_WF(:,:)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c     ==================================================================

      SUBROUTINE LDE_1
c     ----------------------------------------------------------
c     ... WF for absolute variation of Lambertian surface albedo 
c         (if LDE should be calculated) See Eq.(WF-Lamb-refl) in STB
      ASM(:) = EXL(:) + FLUX_D(:)  ;  J = 0
      DO Js = 1,NSZA
         DO Jv = JV_S(Js),JV_E(Js)  
            S1 = ASM(Jv)*( MuS(Js)*EXS(Js) + 2.d0*FLUX(Js) )
            DO Ja = 1,Naz ; J = J + 1 ; Lamb_wf(J) = S1 ; ENDDO
         ENDDO
      ENDDO
      END SUBROUTINE LDE_1
c     ------------------------------------------------------------------

      SUBROUTINE LDE_2
      CALL LIN_SEARCH_NEW(POS, DC, Ws, Ws1, Nwl_veg, WL_veg)
      IF( AB_DB ) THEN
         Alam  =  (1.d0 - DC)*Alb_svk(Ws) + DC*Alb_svk(Ws1)
      ELSE
         Aveg = (1.d0 - DC)*Al_veg(Ws) + DC*Al_veg(Ws1)
         Asoi = (1.d0 - DC)*Al_soi(Ws) + DC*Al_soi(Ws1)
         Alam  = Cfr_v*Aveg + (1.d0-Cfr_v)*Asoi
      ENDIF
      Ws = 1
      IF( POS >= Wgr_svk ) Ws = Ws + 1
c        ... N_svk(Ws) is normalization factor (see "general_brdf_setup.f")
      WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1) 
     $     + N_svk(Ws)*(ALB_WF(:,1) - Alam*Lamb_wf(:))*E_W /Alam
      END SUBROUTINE LDE_2

c     ==================================================================
      END
