!***************************************************************************
!
!	SCIATRAN - Version 4.7 - Last modification: 20 Oct 2023
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!       In this routine cosines of line-of-sight are positive  
!                 in tau-direction (downwards) !
!
!***************************************************************************
SUBROUTINE ST_DOM_TAU(JPOS, JJESFT, Iot, Mh, POS, E_W, AGR_A, AGR_W, GSL_A, &
     GSL_W, UDB_A, UDB_W, LOA, LOW, LWF, TAC, OSM, SLS, SSB_A, SSB_W, TE, &
     SWF, PWF, Abs_soot, RAD_PS, RADs_PS, DEPOL, DO_OUT, DO_CONV1, SRR, &
     SRR_Jact, SVR, RADOUT, W_LEAV, PH_M, ADI, *)
! Called by "dom_spherical.f90
  
  USE ABSORBERS, ONLY: N_GAS
  USE AEROSOLS,  ONLY: Dwr_mi, N_dwr
  USE BRDF,      ONLY: Exact_BRF, Npg
  USE CLOUDS_SS, ONLY: DO_SSRCA
  USE GT_POLYN,  ONLY: Leg => NLEGEN, AG
  USE EMISSION,  ONLY: N_AT
  USE HYDROSOLS, ONLY: NHL => LA_N_HS
  USE I_DIMS,    ONLY: Nm_ice, Nl_Cr, Ng_Cr, Nls_Cr, Nso_Cr, N_C2, Nl_Ar, &
                      Ng_Ar, Nls_Ar, Nso_Ar, N_A2, Nvs, NNv, N_sol_ls, &
                      N_SFN, NL_soot, NL_MS, N_NODE_MS, NSOL_MS, N_PHI_MS,&
                      Ns_SFN
  USE LOG_PARS,  ONLY: LUNIT, N_ESO, LOG_LEVEL, N_DSO, ERROR_TXT, EU, N_SSO
  USE NODES,     ONLY: N_NODE, N_PLUS, N_MINUS, WEIGHTS, W_PLUS, W_MINUS,&
                       MJU_NODE, MJU_MINUS, MJU_PLUS, PHI_NODE
  USE RAMAN,     ONLY: RRS_ADJ, RRS_DO
  USE RTM_OUTPUT,ONLY: N_OP
  USE ST_VARS,   ONLY: RTM_MODE, Pie, WF_FLAG, GeoD_Out, Gauss_Field, &
                       RTM_TYPE, DO_RAMAN, N_RHS, SOLVER, Vec_Sca, N_Eff, &
                       N_Adj, INT_OUT, RAD_FIELD, OWF_FLAG, OWF_Ks, OWF_Kl,&
                       SDVI, OWF_Chl, OWF_Ray, OWF_Use, VST, Nc_der, Kvs, &
                       NS_wf, Ainc, GeoD_PS, Interp_RF, SSC_RF, Sec_order,&
                       NSTRMS
  USE SPHER_PARS,ONLY: N_SL => N_LOS_OUT, ANG_OUT, JV_S, JV_E, N_AZ_VAR, &
                       N_AZ_ANG, AZM_MS, LOS_DEF_FLAG, NFA => N_FOV_ANG
  USE TOOLS,     ONLY: INQ_WF
  USE TYPES,     ONLY: KND, LOA_TYPE, PWF_TYPE, LOW_TYPE, SDO_TYPE, TE_TYPE,&
                       UDB_TYPE, SSB_TYPE, GSL_TYPE, LAS_TYPE, IOP_TYPE, &
                       RRS_TYPE, OSM_TYPE, ADO_TYPE, PHA_CLD, AGR_type, &
                       LWF_TYPE, VRS_TYPE, PHA_MATR, PH_MET, SWF_type, &
                       SLS_type, AuxDataInterp
  USE WATER,     ONLY: UWT, NL_W => NLAYERS_UW, NN_W => NSTR_W, NLEG_W,&
                       NLSA_W, NSZA_W, NNv_W, ERDR, Obs_POSITION, INCL_INEL,&
                       INEL_DO, INEL_ADJ

  TYPE (AGR_type) :: AGR_A, AGR_W
  TYPE (IOP_TYPE), TARGET :: IOP_A, IOP_W ! Integral optical parameters
  TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere
  TYPE (LOW_type) :: LOW    ! Local optical parameters of water
  TYPE (LWF_type) :: LWF    ! Local weighting functions
  TYPE (OSM_type) :: OSM    ! Original scattering matrices
  TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
  TYPE (GSL_type) :: GSL_A, GSL_W ! Angles
  TYPE (RRS_type) :: SRR    ! Structure for rotational Raman
  TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
  TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
  TYPE (AuxDataInterp), Target :: ADI
  !-------------------------------
  ! Input parameters
  INTEGER,    INTENT(IN) :: JPOS, JJESFT, Iot, Mh, SRR_Jact
  REAL (KND), INTENT(IN) :: E_W, POS, TAC(AGR_A%NL,N_GAS), &
       Abs_soot(NL_soot), DEPOL
  REAL (KND) RADOUT(N_OP,N_SL,Nvs), W_LEAV(N_SL,Nvs)
  REAL (KND), ALLOCATABLE :: RAD_PS(:,:,:), RADs_PS(:,:,:,:)
  ! Input/Output if RAD_FIELD is set to "t"
  LOGICAL DO_OUT, DO_CONV1
  TYPE (SLS_type) :: SLS ! Spherical line-of-sight
  TYPE (SSB_type) :: SSB_A, SSB_W ! Spherical solar beam
  TYPE (TE_type)  :: TE     ! Thermal emission 
  TYPE (PWF_type) :: PWF    ! General weighting function
  TYPE (LAS_TYPE) :: LAS_A, LAS_W ! Linear algebraic system
  TYPE (SWF_type) :: SWF     ! "Spherical" weighting function 
  TYPE (PH_MET)   :: Ph_R, Ph_rW, Ph_Ks, Ph_Kl
  !--------------------------------------------
  TYPE phase_tdz
     SEQUENCE
     REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:), Egs(:,:,:,:), &
          EsL(:,:,:,:)
  END TYPE phase_tdz
  TYPE (phase_tdz) :: Ph_zA, Ph_zW, Ph_A, Ph_Us
  !---------------------------------------
  TYPE Dphase_tdz
     SEQUENCE
     REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:), &
          Egs(:,:,:,:,:), EsL(:,:,:,:,:)
  END TYPE Dphase_tdz
  TYPE (Dphase_tdz) :: Ph_dA
  !---------------------------------------
  TYPE (SDO_type) :: SDO_A, SDO_W ! DOM auxiliary solution variables
  TYPE (ADO_type) :: ADO_A, ADO_W ! Adjoint DOM auxiliary variables
  !----------------------------------------------------------
  ! Adjoint DOM auxiliary spherical geometry |
  !-------------------------------------------
  TYPE ADS_type
     SEQUENCE
     LOGICAL, POINTER :: Act_ag(:,:)
     REAL (KND), POINTER :: Pol_ga(:,:), PLS(:,:,:,:), MLS(:,:,:,:)
  END TYPE ADS_type
  TYPE (ADS_type) :: ADS

  TYPE (PHA_MATR) PH_M
!----------------------------------------------------------
! Local parameters
!----------------------------------------------------------
  CHARACTER (LEN=10), PARAMETER :: R_NAME = 'ST_DOM_TAU'
  INTEGER  NL, NN, NSZA, NLSA, Nz, Jact
  INTEGER  I, J, K, L, IA, K1, K2, INI, NrL, Js, Jv, Jg, NN_MS
  REAL (KND), ALLOCATABLE :: X_REF(:), Fg_INT(:), ASOL_ADJ(:,:), &
       Tr_gau(:), DELS(:,:,:,:)
  REAL (KND), POINTER :: P_RD(:)
  !==================================================================
  INTERFACE
     SUBROUTINE POST_PROCESSING(Mh, E_W, RADOUT, POS, GSL_A, GSL_W, IOP_A, &
          IOP_W, SSB_A, SSB_W, TE, SDO_A, SDO_W, Ph_zA, Ph_zW, LAS_A, &
          LAS_W, UDB_A, UDB_W, LOA, LOW, AGR_A, AGR_W, GeoD, DELS)
       USE I_DIMS,    ONLY: Nvs
       USE RTM_OUTPUT,ONLY: N_OP
       USE SPHER_PARS,ONLY: N_LOS_OUT
       USE TYPES,     ONLY: TE_TYPE, SDO_TYPE, UDB_TYPE, SSB_TYPE, GSL_TYPE, &
                            IOP_TYPE, LAS_TYPE, LOA_TYPE, LOW_TYPE, &
                            AGR_type, Geometry_Data
       USE TYPE_VARS,   ONLY: KND
       INTEGER,    INTENT(IN) :: Mh
       REAL (KND), INTENT(IN) :: E_W, POS
       REAL (KND)  RADOUT(N_OP,N_LOS_OUT,Nvs)
       REAL (KND), ALLOCATABLE :: DELS(:,:,:,:)
       TYPE (GSL_type) :: GSL_A, GSL_W 
       TYPE (IOP_type), TARGET  :: IOP_A, IOP_W 
       TYPE (LAS_TYPE) :: LAS_A, LAS_W 
       TYPE (LOA_type) :: LOA 
       TYPE (LOW_type) :: LOW 
       TYPE (SSB_type) :: SSB_A, SSB_W 
       TYPE (SDO_type) :: SDO_A, SDO_W 
       TYPE (TE_type)  :: TE  
       TYPE (UDB_TYPE) :: UDB_A, UDB_W 
       TYPE (AGR_type) :: AGR_A, AGR_W
       TYPE phase_tdz
          SEQUENCE
          REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),&
               Egs(:,:,:,:), EsL(:,:,:,:)
       END TYPE phase_tdz
       TYPE (phase_tdz) :: Ph_zA, Ph_zW
       TYPE (Geometry_Data) GeoD
     END SUBROUTINE POST_PROCESSING
     !-----------------------------------------------------      
     SUBROUTINE SFI_FIELD(JPOS, RAD, AL, E_W, OSM, LOA, LSE, TE, ADi)
       USE GRIDS,  ONLY: NLAYERS
       USE ST_VARS,ONLY: GeoD => GeoD_Out
       USE TYPES,  ONLY: KND, LOA_type, OSM_TYPE, LSE_type,&
                         AuxDataInterp, TE_TYPE
       TYPE (AuxDataInterp), TARGET :: ADI
       TYPE (LOA_type) :: LOA 
       TYPE (OSM_type) :: OSM 
       TYPE(LSE_type), TARGET :: LSE(GeoD%NAlts, GeoD%NVA)
       TYPE (TE_type)  :: TE
       INTEGER    JPOS
       REAL (KND) E_W, AL(NLAYERS)
       REAL (KND), ALLOCATABLE :: RAD(:,:,:)
     END SUBROUTINE SFI_FIELD
     !-----------------------------------------------------      
     SUBROUTINE SFI_SPHER(JPOS, JJESFT, Mha, POS, RAD, AL, E_W, OSM, LOA,&
          LSE, PWF, SRR, SRR_Jact, IND_LS, TE, ALBEDO, FLU_Int, Ref_surf, &
          FLUX, ADI, RADOUT, NN_MS)
       USE BRDF,        ONLY: Npg
       USE GRIDS,       ONLY: NLAYERS, NSOL
       USE INTERN_DIMS, ONLY: Nvs
       USE RTM_OUTPUT,  ONLY: N_OP
       USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT, NFA => N_FOV_ANG
       USE ST_VARS,     ONLY: Kvs, N_RHS, NSTRMS, Nc_der, GeoD => GeoD_Out
       USE TYPES,       ONLY: KND, PWF_TYPE, LOA_type, OSM_TYPE, RRS_TYPE,&
                              AuxDataInterp, LSE_type, TE_TYPE
       INTEGER  NN_MS, SRR_Jact
       INTEGER JPOS, JJESFT, Mha, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
       REAL (KND)  ALBEDO, FLU_Int, E_W, POS, FLUX(Kvs*NSOL), AL(NLAYERS),&
            Ref_surf(N_RHS,Nvs*(NSTRMS+Npg))
       REAL (KND)  RADOUT(N_OP,NLO,Nvs)
       REAL (KND), ALLOCATABLE :: RAD(:,:,:)
       TYPE (AuxDataInterp), TARGET :: ADI
       TYPE (LOA_type) :: LOA ! Local optical parameters atmosphere
       TYPE (OSM_type) :: OSM ! Original scattering matrices
       TYPE (PWF_type) :: PWF ! Weighting functions
       TYPE (RRS_type) :: SRR ! RRS related parameters
       TYPE(LSE_type), TARGET :: LSE(GeoD%NAlts, GeoD%NVA)
       TYPE (TE_type)  :: TE ! Thermal emission 
            
       INTENT(IN) :: JPOS, JJESFT, Mha, IND_LS, ALBEDO, E_W, POS, FLUX, AL,&
            Ref_surf, RAD, LOA, OSM, PWF, ADI
       INTENT(INOUT) :: RADOUT, SRR            
     END SUBROUTINE SFI_SPHER
     !---------------------------------------------------------------
     SUBROUTINE SFI_SPHER_RF(JPOS, JJESFT, Mha, POS, RAD, AL, E_W, OSM, LOA,&
          LSE, PWF, SRR, IND_LS, TE, ALBEDO, FLU_Int, Ref_surf, FLUX, ADI, &
          NN_MS)
       USE BRDF,        ONLY: Npg
       USE GRIDS,       ONLY: NLAYERS, NSOL
       USE INTERN_DIMS, ONLY: Nvs
       USE RTM_OUTPUT,  ONLY: N_OP
       USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT, NFA => N_FOV_ANG
       USE ST_VARS,     ONLY: Kvs, N_RHS, NSTRMS, Nc_der, GeoD => GeoD_Out
       USE TYPES,       ONLY: KND, PWF_TYPE, LOA_type, OSM_TYPE, RRS_TYPE,&
            AuxDataInterp, LSE_type, TE_TYPE
       INTEGER  NN_MS
       INTEGER JPOS, JJESFT, Mha, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
       REAL (KND)  ALBEDO, FLU_Int, E_W, POS, FLUX(Kvs*NSOL), AL(NLAYERS),&
            Ref_surf(N_RHS,Nvs*(NSTRMS+Npg))
       REAL (KND), ALLOCATABLE :: RAD(:,:,:)
       TYPE (AuxDataInterp), TARGET :: ADI
       TYPE (LOA_type) :: LOA ! Local optical parameters atmosphere
       TYPE (OSM_type) :: OSM ! Original scattering matrices
       TYPE (PWF_type) :: PWF ! Weighting functions
       TYPE (RRS_type) :: SRR ! RRS related parameters
       TYPE(LSE_type), TARGET :: LSE(GeoD%NAlts, GeoD%NVA)
       TYPE (TE_type)  :: TE ! Thermal emission 
            
       INTENT(IN) :: JPOS, JJESFT, Mha, IND_LS, ALBEDO, E_W, POS, FLUX, AL,&
            Ref_surf, RAD, LOA, OSM, PWF, ADI
       INTENT(INOUT) :: SRR            
     END SUBROUTINE SFI_SPHER_RF
  END INTERFACE
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')
  IF( LOG_LEVEL(1)>=N_ESO ) WRITE(LUNIT,'(20x,A)') '  *** '//R_NAME

  NN  = GSL_A%NN ; NSZA = GSL_A%NSZA 
  NL  = AGR_A%NL ; NrL = AGR_A%NrL   
  NLSA = GSL_A%NLSA
  IF( .NOT.Interp_RF ) NLSA = N_NODE
  IA = 1

  ! Allocate arrays relevant to RT in atmosphere  
  !---------------------------------------------
  CALL ALLOCATE_ATM

  ! Allocate arrays relevant to RT in ocean
  ! ---------------------------------------
  IF( UWT ) CALL ALLOCATE_OCEAN
  
  IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) ' +++++++++ Harmonic : ', Mh
!
!     Discrete-ordinates solution for Gauss angles
!     --------------------------------------------
  IF( LOG_LEVEL(1) >= N_ESO ) THEN
     WRITE(LUNIT,*) '      ----------------------------     '
     WRITE(LUNIT,*) '      First call in the atmosphere     '
     WRITE(LUNIT,*) '      ----------------------------     '
  ENDIF

  IF( Sec_order ) GOTO 2121
      
  ! Standard discrete-ordinate solution of RTE in atmosphere
  ! --------------------------------------------------------
  ! WRITE(*,*) ' SHAPE(RADOUT) st_dom_tau: ', SHAPE(RADOUT)
  ! WRITE(*,*) RADOUT
  CALL SOLV_DOM(JPOS, 'A', *880, *990, Mh, E_W, AGR_A, GSL_A, SSB_A, TE, &
       LOA, LOW, UDB_A, SDO_A, LAS_A, SOLVER, Ph_zA, SLS, ADS, ADO_A, SRR,&
       SRR_Jact, SVR, Ph_R, Ph_A, Ph_dA, Ph_C, Ph_Us, Ph_Ks, Ph_Kl, PH_M,&
       RADOUT)

! Screen output of fluxes at ocean-atmosphere interface
!------------------------------------------------------
! IF( LOG_LEVEL(1) >= N_ESO ) CALL WRITE_FLUXES
! CALL WRITE_FLUXES
      
  IF( UWT ) THEN 
! Iterative solution of coupled ocean-atmospheric RTE
! ---------------------------------------------------
     CALL ITERATE_COA(JPOS, *880, *990, Mh, E_W, AGR_A, AGR_W, SRR, &
          SRR_Jact, SVR, GSL_A, GSL_W, SSB_A, SSB_W, TE, LOA, LOW, UDB_A, &
          UDB_W, SDO_A, SDO_W, LAS_A, LAS_W, RADOUT, Ph_zW, SLS, W_LEAV, &
          ADS, ADO_A, ADO_W, Ph_rW, Ph_A, Ph_dA, Ph_C, Ph_Us, Ph_Ks, Ph_Kl)
     !-------------------------------------------------------
  ENDIF

  ! If DO technique is used for VRS, Chl-a or CDOM fluorescence 
  ! (INEL_DO=.TRUE.), then all required parameters will be saved  
  ! at all excitation wavelength (SVR%Exi_WL(JPOS)=.TRUE.).
  ! After that returns this routine
  !--------------------------------------------------------------
  IF( INEL_DO.AND.SVR%Exi_WL(JPOS) ) CALL SAVE_EXC_VRS(*990)
  
  ! If DO technique is used for RRS (RRS_DO=.TRUE.)
  ! then all required parameters will be saved at all excitation  
  ! wavelength (the case SRR%Exi_WL(JPOS)=.TRUE.). 
  ! After that returns this routine
  !--------------------------------------------------------------
  IF( RRS_DO.AND.SRR%Exi_WL(JPOS) ) CALL SAVE_EXC_RRS(*990)

  ! Set integral optical parameters of atmosphere, ocean, or both
  !--------------------------------------------------------------
  CALL IOP_IFACE(AGR_A, AGR_W, GSL_A, GSL_W, LAS_A, LAS_W, LOA, LOW, SDO_A, &
       SDO_W, IOP_A, IOP_W, Obs_POSITION)

  ! 'ga' - output in RADOUT at Gauss angles only and than "RETURN"
  !-------------------------------------------------------
  ! INT_OUT - interpolation of the radiation field on the desired 
  ! viewing angles instead of post-processing in the case of  
  ! inelastic scattering and then "RETURN"
  !-------------------------------------------------------
  IF( LOS_DEF_FLAG=='ga'.OR.INT_OUT )  CALL SIMPL_OUTPUT(*990)

  ! ********************************************************************
  ! Routine "FIELD_IFACE.f" is used below to calculate intensity 
  ! field for all Gauss angles and all altitudes for the cases:
  !             Flux, Spherical albedo, Spherical geometry, 
  !             Weighting function, 
  !             Rotational Raman (forward-adjoint technique)
  ! *******************************************************************

  ! Main output of this routine is array DFI
  IF( Gauss_Field ) CALL FIELD_IFACE( Mh, AGR_A, AGR_W, GSL_A, GSL_W, IOP_A,&
       IOP_W, LAS_A, LAS_W, LOA, LOW, Ph_zA, Ph_zW, SDO_A, SDO_W, SSB_A, &
       SSB_W, TE, UDB_A, UDB_W, Obs_POSITION )
  
!*******************************************************************
! The subroutine "st_flux_spher_alb.f" used below in the case 
!     of fluxes or spherical albedo calculations for 
!     plane-parallel or pseudo-spherical approximation
! In spherical geometry such calculations is performed in the
!     subroutine "integrate_init.f" ---> "flux_cdi.f"      
!*******************************************************************

!  IF( .NOT.RAD_FIELD ) THEN 
  IF( .NOT.RAD_FIELD.AND.Mh==1 ) THEN 
    SELECT CASE( TRIM(RTM_MODE) )
    CASE('flux', 'spher_alb', 'int_flux')
       Jact = JPOS
       IF( ANY(INCL_INEL(:)) ) Jact = SVR%Jact
       IF( DO_RAMAN )          Jact = SRR_Jact
       CALL ST_FLUX_SPHER_ALB ( Jact, JJESFT, E_W, AGR_A%NL, LOA%AL, GSL_A,&
            SSB_A%EXS, IOP_A%DFI, 'Atmos', UDB_A )
       IF( UWT ) CALL ST_FLUX_SPHER_ALB ( Jact, JJESFT, E_W, AGR_W%NL, &
            LOA%AL, GSL_W, SSB_W%EXS, IOP_W%DFI, 'Ocean', UDB_W )
       IF( TRIM(RTM_MODE) /= 'int_flux' ) THEN
          CALL DEALLOCATE_ATM
          IF( UWT ) CALL DEALLOCATE_OCEAN            
       ENDIF
       IF( TRIM(RTM_MODE)=='flux' ) THEN
          CALL ST_TRACK (R_NAME, 1, 'O')
          RETURN  ! Exit this routine --->
       ENDIF
    END SELECT
 ENDIF  ! .NOT.RAD_FIELD

!*******************************************************************
!Intensity/Stokes vector at user defined viewing directions
!   "sfi_spher.f"       - in the spherical atmosphere using
!                         approximative spherical approach;
!   "post_processing.f" - in plane-parallel medium,  
!                         the pseudo-spherical approximation,
!                         flag "RAD_FIELD" is set to .TRUE.
!   "fls_asi.f"         - employing adjoint technique 
!                         (scalar only);
!   "sphere_field.f"    - intensity/Stokes vector of radiation 
!                         field at all altitudes, Gauss, and 
!                         azimuthal angle.
!*******************************************************************
 PWF%WF_esp = 0.d0 ; PWF%WF_ssp = 0.d0

2121 CONTINUE
 
 SELECT CASE ( TRIM(RTM_TYPE) )
 CASE( 'spher_scat' )
    !Calculation of radiation field at spherical nodes
    !===============================================================
    IF( RAD_FIELD ) THEN 
       ALLOCATE( DELs(NL_MS, NSOL_MS, N_NODE, Nvs) )
       ! SFI technique for radiation field at spherical nodes
       IF( .NOT.Interp_RF ) THEN
          IF( Sec_Order ) THEN
             DELS = 0.d0
          ELSE
             CALL POST_PROCESSING(Mh, E_W, RADOUT, POS, GSL_A, GSL_W, &
                  IOP_A, IOP_W, SSB_A, SSB_W, TE, SDO_A, SDO_W, Ph_zA, &
                  Ph_zW, LAS_A, LAS_W, UDB_A, UDB_W, LOA, LOW, AGR_A, &
                  AGR_W, GeoD_PS, DELS)
          ENDIF  ! .not.Sec_order
          IF( SSC_RF.AND.Mh==1 )  CALL GPP_SS_RF(JPOS, JJESFT, POS, &
               UDB_A%ALBEDO, E_W, OSM, LOA, SSB_A, TE, DELS, &
               UDB_A%FLU_int, *880) 
       ELSE
          DELS = 0.d0
          ! Interpolate from Gaussian zenith angles to the
          ! spherical nodes
          ! WRITE(*,*)  '    Mh = ', Mh
          ! WRITE(*,*)  '   NNv = ', NNv
          ! WRITE(*,'(*(2x,E12.4))')  IOP_A%DFI(NL_MS,1,1:Nvs)
          DO I = 1, NL_MS
             DO Js = 1, NSOL_MS
                P_RD => IOP_A%DFI(I,Js,:)
                DO K1 = 1,Nvs    
                   ! downward radiation
                   CALL AKIMA_INT(NN, GSL_A%XM, P_RD(K1:NNv:Nvs), N_MINUS, &
                        -MJU_MINUS, DELs(I,Js,1+N_PLUS:N_NODE,K1) )
                   ! upward radiation
                   CALL AKIMA_INT (NN, GSL_A%XM, P_RD(NNv+K1:2*NNv:Nvs), &
                        N_PLUS, MJU_PLUS, DELs(I,Js,1:N_PLUS,K1) )
                   ! U and V components should be taken with sign '-'  
                   ! for upward radiation (see Eq.(Int-up) in STB)
                   IF( K1 > 2 ) DELs(I,Js,1:N_PLUS,K1) = -DELs(I,Js,1:N_PLUS,K1)
                ENDDO  ! over Stokes components (K1)
                !!DO K1 = 1,Nvs    
                ! DO K1 = 2,Nvs    
                ! downward radiation
                !    CALL LIN_INT(NN, GSL_A%XM, P_RD(K1:NNv:Nvs), N_MINUS, &
                !       -MJU_MINUS, DELs(I,Js,1+N_PLUS:N_NODE,K1), 'L')
                ! upward radiation
                !    CALL AKIMA_INT (NN, GSL_A%XM, P_RD(NNv+K1:2*NNv:Nvs), 
                !        N_PLUS, MJU_PLUS, DELs(I,Js,1:N_PLUS,K1),'L' ) 
                ! ENDDO  ! over Stokes components (K1)
                
             ENDDO  ! over selected solar angles (Js)
          ENDDO  ! over main altitude levels (I)
       ENDIF
       CALL SPHERE_FIELD_S (*880)
       IF( WF_FLAG ) THEN
          NN_MS = 2*NSTRMS
          ! Intensity will be not calculated               
          CALL SFI_SPHER_RF(JPOS, JJESFT, Mh, POS, IOP_A%DFI, LOA%AL, E_W, &
               OSM, LOA, SLS%LSE, PWF, SRR, SLS%IND_LS, TE, UDB_A%ALBEDO, &
               UDB_A%FLU_Int, LAS_A%Ref_surf, LAS_A%FLUX, ADI, NN_MS)
       ENDIF
    ELSE
       ! Intensity in the spherical atmosphere (SFI technique)
       ! for each Fourier harmonic as well as weighting functions
       ! (structure PWF) in the single scattering approximation
       ! IOP_A%DFI(NL,Kvs*NSZA,2*NNv)

       CALL SFI_SPHER(JPOS, JJESFT, Mh, POS, IOP_A%DFI, LOA%AL, E_W, OSM, &
            LOA, SLS%LSE, PWF, SRR, SRR_Jact, SLS%IND_LS, TE, UDB_A%ALBEDO, &
            UDB_A%FLU_Int, LAS_A%Ref_surf, LAS_A%FLUX, ADI, RADOUT, N_NODE)
    ENDIF

 CASE( 'pp_scat', 'ps_scat' )
    SELECT CASE ( TRIM(SOLVER) )
    CASE( 'Solv_SFI', 'Solv_MIX' )
       ! Intensity in the plane-parallel medium (SFI technique)
       CALL POST_PROCESSING(Mh, E_W, RADOUT, POS, GSL_A, GSL_W, IOP_A, &
            IOP_W, SSB_A, SSB_W, TE, SDO_A, SDO_W, Ph_zA, Ph_zW, LAS_A, &
            LAS_W, UDB_A, UDB_W, LOA, LOW, AGR_A, AGR_W, GeoD_OUT, DELS)
       
       ! Test output for the comparison with the pseudo-forward 
       ! radiation obtained in the framework of forw-adj approach
       ! CALL Field_Dir_Gau(GSL_A%NN, NSZA, AGR_A%NL, GSL_A%XM, IOP_A%DTR, &
       !      LAS_A%PART_D, IOP_A%MVEC, LAS_A%SOL_TOP, SDO_A, IOP_A%DFI)

    CASE( 'Solv_ASI' )
! Test output of the pseudo-forward radiation
       CALL Field_Pf_Gau(GSL_A%NN, NSZA, AGR_A%NL, Mh, GSL_A%XM, GSL_A%AG,&
            Ph_zA, IOP_A%ALW, IOP_A%DTR, ADO_A%Part_TD, ADO_A%Part_TU,&
            IOP_A%MVEC, SDO_A, IOP_A%DFI, SSB_A%EXS, GSL_A%EXL, GSL_A%MuV,&
            NLSA, E_W, RADOUT, UDB_A%ALBEDO, ADO_A, GSL_A%MuS)

       ! Forward intensity via solution of the pseudo-forward RTE
       ! CALL FLS_ASI(NL, NN, NSZA, NLSA, Mh, GSL_A%MuS, GSL_A%MuV, &
       !    UDB_A%ALBEDO, E_W, IOP_A%ALW, IOP_A%DTR, GSL_A%XM, AG, Ph_zA,&
       !    IOP_A%MVEC, SDO_A, ADO_A, GSL_A%EXL, SSB_A%EXS, IOP_A%Ex_D, RADOUT)
    END SELECT

! Single scattering correction algorithm for radiance in the
! case of plane-parallel and pseudo-spherical atmosphere
    IF( .NOT.WF_FLAG.AND.DO_SSRCA.AND.Mh==1 ) THEN
       ! WRITE(*,*) '    ++++++++++ "gpp_ss_stokes" ++++++++++++ '
       CALL GPP_SS_STOKES(JPOS, JJESFT, Iot, POS, UDB_A%ALBEDO, E_W, OSM, &
            GSL_A, LOA, LWF, PWF, SSB_A, TE, SRR, SRR_Jact, RADOUT, TAC, &
            Abs_soot, UDB_A%FLU_int, *880)
    ENDIF
    IF( UWT.AND.Mh==1.AND.ERDR ) CALL REFL_TO_ATM
    !-----------------------------------------------------------------------
    
 END SELECT

 !*************** End of radiance output *****************************
 IF( .NOT.WF_FLAG .AND. .NOT.RRS_ADJ .AND. .NOT.INEL_ADJ) THEN 
    ! Deallocate atmosphere/ocean RT relevant arrays and return
    CALL DEALLOCATE_ATM
    IF( UWT ) CALL DEALLOCATE_OCEAN
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN                 ! Exit this routine ----------->
 ENDIF

! Depending on the observation geometry and coupling 
! it will be here calculated the contribution in the 
! pseudo-forward intensity in the anti-solar directions  
! from the bottom of the corresponding medium
 ALLOCATE( ASOL_ADJ(N_Adj,Nvs*NSZA), X_ref(NN+1), Fg_INT(Nvs*NSZA),&
      Tr_gau(Nvs*NN) )

 IF( TRIM(RTM_TYPE)/='spher_scat' ) THEN
    IF( .NOT.UWT ) THEN
       ! Atmosphere without coupling
       CALL ASOL_ADJ_ATM  ! Upward and downward 
    ELSE
       ! Coupled atmosphere-ocean system
       SELECT CASE( GeoD_Out%View(IA)%S )
       CASE('up')  
          CALL ASOL_ADJ_COA  ! Upward radiation
       CASE('down')
          ! Downward radiation
          SELECT CASE( TRIM(Obs_POSITION(1)) )
          CASE('Atmos')    ! In atmosphere
          CASE('Ocean')    ! In ocean
          END SELECT       ! End of downward rad.
       END SELECT
    ENDIF  ! UWT
 ENDIF  ! not 'spher_scat' mode
 
!*************************************************************
! Following routine will be used in the cases: 
!   - weighting functions for atmosphere;
!   - rotational Raman scattering in the atmosphere 
!     (forward-adjoint technique).
!*************************************************************
 CALL ST_ADJOINT_A(JPOS, POS, JJESFT, Iot, Mh, E_W, DO_OUT, ADS, AGR_A,&
      ADO_A, GSL_A, IOP_A, LAS_A, LOA, LWF, Ph_zA, Ph_R, Ph_A, Ph_dA, Ph_C,&
      PWF, SWF, SDO_A, SLS, SRR, SRR_Jact, TE, TAC, Abs_soot, UDB_A, &
      ASOL_ADJ, OSM, SSB_A, PH_M, RADOUT, *880)

!*************************************************************
! Following routine will be used in the case of:
!   - weighting functions for ocean;
!   - vibrational Raman scattering (forward-adjoint technique);
!   - chlorophyll fluorescence.
!*************************************************************
 IF( OWF_FLAG.OR.INEL_ADJ ) THEN

    IF( TRIM(RTM_TYPE)/='spher_scat' ) THEN
       SELECT CASE( TRIM(UDB_W%LBC) )
          ! Preliminary the ocean bottom reflection is Lambertian only
       CASE('Lambert')
          ASOL_ADJ(:,:) = 0.d0
          ! In the case 'pp_scat' or 'ps_scat' N_Adj = NS_wf*NLSA
          ! is set in subroutine "rtm_init.f". No FoV integration.
          DO Js = 1,Nvs*NSZA,Nvs
             ASOL_ADJ(:,Js) = 2.d0*UDB_W%Albedo*ADO_W%FLUX_D(:)
             DO Jv = 1, N_adj
                IF( GeoD_Out%LVG(IA,Jv)%C_VA(1) <= 0.d0 ) CYCLE
                ASOL_ADJ(Jv,Js) = ASOL_ADJ(Jv,Js) + &
                     UDB_W%Albedo*GSL_W%EXL(NL_W,Jv)*VST
             ENDDO  ! over observation angles (Jv)
          ENDDO  ! over solar angles (Js)
       CASE DEFAULT
          STOP ' "st_dom_tau.f" Only Lambertian surface for ocean'
       END SELECT
    ENDIF  ! /='spher_scat' 
    
    CALL ST_ADJOINT_O(JPOS, JJESFT, Mh, E_W, UDB_W, SSB_W%EXS, AGR_W, &
         ADO_W, GSL_W, IOP_W, LAS_W, LOW, SDO_W, SVR, TE, Ph_zW, Ph_rW, &
         Ph_Us, Ph_Ks, Ph_Kl, ASOL_ADJ, DO_OUT)
 ENDIF  ! OWF_FLAG.OR.INEL_ADJ
 
990 CONTINUE

 CALL DEALLOCATE_ATM  
 IF( UWT ) CALL DEALLOCATE_OCEAN
 
 CALL ST_TRACK (R_NAME, 1, 'O')
 RETURN
 
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
 RETURN 1
CONTAINS
!***********************************************************************
  SUBROUTINE REFL_TO_ATM
    USE SPHER_PARS, ONLY: ANG_SEL_FLAG
    USE ST_VARS,    ONLY: Finc
    INTEGER Ks, Ja
    REAL (KND) DUM, S1
    REAL (KND), ALLOCATABLE :: Tls(:,:), HDU(:,:,:)
    
    IF( .NOT.ALLOCATED(Tls) ) &
         ALLOCATE( Tls(N_SL,Nvs), HDU(NSZA,Nvs*NLSA,N_AZ_ANG) )

! Reflected direct solar light by the ocean-atmosphere interface
! into direction of an instrument located in atmosphere
    DO Js = 1,NSZA ; Ks = (Js-1)*Nvs
       DO Ja = 1, N_AZ_ANG
          DO K = 1,Nvs*NLSA
             HDU(Js,K,Ja) = SUM(UDB_A%Ref_AA_LOS(K,Ks+1:Ks+Nvs,Ja)*Finc(1,:))
          ENDDO
       ENDDO
    ENDDO
    !-----------------------------------------------------------------------
    SELECT CASE( ANG_SEL_FLAG )
    CASE( 'one' )
       DO Js = 1,NSZA  ;  DUM = SSB_A%EXS(NL,Js)*GSL_A%EXL(NL,Js)
          DO I = 1,Nvs
             Tls(Js,I) = DUM*HDU(Js,(Js-1)*Nvs+I,Js)
          ENDDO
       ENDDO
    CASE( 'all' )
       J = 0
       DO Js = 1,NSZA               ;  DUM = SSB_A%EXS(NL,Js)
          DO Jv = JV_S(Js),JV_E(Js) ;  S1  = GSL_A%EXL(NL,Jv)
             DO Ja = 1,N_AZ_VAR     ;  J = J + 1
                DO I = 1,Nvs
                   Tls(J,I) = DUM*S1*HDU(Js,(Jv-1)*Nvs+I,Ja)
                ENDDO
             ENDDO
          ENDDO
       ENDDO
    END SELECT
    
    DO I = 1,N_OP
       RADOUT(I,:,:) = RADOUT(I,:,:) + Tls(:,:) *Pie
    ENDDO
    ! PAUSE
  END SUBROUTINE REFL_TO_ATM

!***********************************************************************
!    This routine will be used in the case of "pp_scat" and "ps_scat"
!                                              --------     ---------
!***********************************************************************
  SUBROUTINE ASOL_ADJ_ATM
    CHARACTER (LEN=12), PARAMETER :: R_NAME = 'ASOL_ADJ_ATM'
    INTEGER  Jw1, Jw, K2, Kh
    REAL (KND), ALLOCATABLE :: HDU(:)
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    SELECT CASE( TRIM(UDB_A%LBC) )
    CASE('Lambert')
       ! First Stokes component only in the case of 
       ! Lambertian reflection
       ASOL_ADJ(:,:) = 0.d0  
       DO Js = 1,Nvs*NSZA,Nvs
          ASOL_ADJ(:,Js) = 2.d0*UDB_A%Albedo*ADO_A%FLUX_D(:) *VST
       ENDDO
       DO Jw = 1,NS_wf
          ! Nc_der(1:NS_wf) contains the number of Stokes 
          ! component for which WF should be calculated
          IF( Nc_der(Jw) /= 1 ) CYCLE
          DO Jv = 1, NLSA ; Jw1 = NS_wf*(Jv-1) + Jw
             IF( GeoD_Out%LVG(IA,Jv)%C_VA(1)>0.d0.AND.(.NOT.DO_SSRCA) ) THEN 
                DO Js = 1,Nvs*NSZA,Nvs 
                   ASOL_ADJ(Jw1,Js) = ASOL_ADJ(Jw1,Js) &
                        + 2.d0*UDB_A%Albedo*GSL_A%EXL(NL,Jv)*VST
                ENDDO  ! solar angles (Js)
             ENDIF  ! reflected radiation and DO_SSRCA=.FALSE.
          ENDDO  ! over LoS (Jv)
       ENDDO  ! over derivatives Stokes components (Jw)
       
    CASE('Bidirect')         
       Kh = Nvs*(NN+1)
       ! Option Exact_BRF is removed from this place
       ! Ref_surf calculated in "st_top_bot.f" contains 
       ! forward and pseudo-forward Stokes vector components
       ! reflected from a surface at all Gauss and viewing angles
       X_ref(1:NN) = GSL_A%XM(:) ; X_ref(NN+1) = 1.d0
       DO Jv = 1,Ns_wf*NLSA
          !Interpolate reflected from surface pseudo-forward "radiation"
          ! on the solar directions
          DO K1 = 1,Nvs 
             CALL AKIMA_INT(NN+1, X_ref, LAS_A%Ref_surf(Jv + SDVI,K1:Kh:Nvs),&  
                  NSZA, GSL_A%MuS, Fg_INT(K1:Nvs*NSZA:Nvs))
          ENDDO  ! over all Stokes components (K1)
          ASOL_ADJ(Jv,:) = 2.d0*Fg_INT(:)  *VST
       ENDDO

       IF( GeoD_Out%View(IA)%S=='down'.OR.DO_SSRCA ) THEN
          CALL ST_TRACK (R_NAME, 1, 'O')
          RETURN
       ENDIF

       ALLOCATE( HDU(Kh) )
       ! write(*,*) '   Asol_adj '
       DO Jv = 1,NLSA
          IF( GeoD_Out%LVG(IA,Jv)%C_VA(1) <= 0.d0) CYCLE  ! downward directions
          INI = Nvs*(SDVI+Jv-1) ;  Jw1 = NS_wf*(Jv-1)
          DO Jw = 1,NS_wf
             K2 = INI + Nc_der(Jw)
             HDU(1:Kh) = UDB_A%BRF_GS(1:Kh,K2,Mh)
             DO K1 = 1,Nvs
                CALL AKIMA_INT(NN+1, X_ref, HDU(K1:Nvs*(NN+1):Nvs), &  
                     NSZA, GSL_A%MuS, Fg_INT(K1:Nvs*NSZA:Nvs) )
             ENDDO
             ! write(*,*) '   --- ', Jv, Jw, Jw1+Jw
             ASOL_ADJ(Jw1+Jw,:) = ASOL_ADJ(Jw1+Jw,:) &
                  + 2.d0*GSL_A%EXL(NL,Jv)*Fg_INT(:) *VST
          ENDDO  ! over number of Stokes WF (Jw)
       ENDDO  ! over line-of-sight (Jv)
       
    END SELECT  ! Lambert and Bidirect
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    
  END SUBROUTINE ASOL_ADJ_ATM
!**********************************************************************
  SUBROUTINE ASOL_ADJ_COA
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ASOL_ADJ_COA'
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
    CASE('Atmos')   ! Observer in atmosphere
       SELECT CASE( TRIM(UDB_A%LBC) )
       CASE('Flat water')
          STOP ' ------> "Flat water" option is not implemented'
          
       CASE('Rough water')
          ! Pseudo-forward radiation reflected from the rough  
          ! water surface in the atmosphere in the solar direction
          ! Contribution of diffuse reflected radiation
          DO Jv = 1,NLSA
             DO K1 = 1,Nvs    ! for all Stokes components
                CALL AKIMA_INT( NN, GSL_A%XM, &
                     LAS_A%Ref_surf(Jv+SDVI,K1:Nvs*NN:Nvs), & 
                     NSZA, GSL_A%MuS, Fg_INT(K1:Nvs*NSZA:Nvs) )
             ENDDO
             ASOL_ADJ(Jv,:) = 2.d0*Pie*Fg_INT(:) *2.d0*Pie ! Test it!!
          ENDDO

          ! Transmitted pseudo-forward radiation from the water
          ! to the atmosphere in the solar direction
          K1 = GSL_W%NN
          DO Jv = 1,NLSA
             DO I = 1,NN - 1  ! Scalar case only !!
                Tr_gau(I) = SUM( UDB_W%Twa_GG(I,1:K1,Mh) &
                     *LAS_W%SOL_TOP(K1*Nvs+1:2*K1*Nvs,Jv+SDVI)*GSL_W%AG(:) )
             ENDDO
             Tr_gau(NN) = LAS_W%SOL_TOP(2*K1*Nvs,Jv+SDVI) &
                  *UDB_W%Ewa_G(K1)/UDB_A%Ref_wd(1)**2 /Pie/2.d0
             ! Interpolate transmitted into atmosphere pseudo-forward 
             ! radiation to the solar directions
             CALL AKIMA_INT(NN, GSL_A%XM, Tr_gau, NSZA, GSL_A%MuS, Fg_INT)
             ASOL_ADJ(Jv,:) = ASOL_ADJ(Jv,:) + 2.d0*Pie*Fg_INT(:)*2.d0*Pie   ! Test it too!!
          ENDDO
       END SELECT

    CASE('Ocean')             ! Observer in ocean
       SELECT CASE( TRIM(UDB_W%LBC) )
       CASE('Lambert')
          FORALL(Js = 1:NSZA) ASOL_ADJ(:,Js) = 2.d0*&
               UDB_W%Albedo*( ADO_W%FLUX_D(:) + GSL_W%EXL(NL_W,:) )

       CASE('Bidirect')         
          STOP ' BRDF from ocean bottom is not implemented yet'
          
       END SELECT
    END SELECT                ! End of upward rad.
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    
  END SUBROUTINE ASOL_ADJ_COA

!**********************************************************************
!    Here will be saved all needed results obtained at excitation 
!        wavelength corresponding to the inelastic processes
!          within water (VRS, CHl-a and CDOM fluorescence)
!**********************************************************************
  SUBROUTINE SAVE_EXC_VRS(*)
    !     Called by "st_dom_tau.f"
    
    USE ST_VARS, ONLY: INEL_PROC
    USE WATER,   ONLY: SIPI
    USE VRS,     ONLY: VRS_Egg
    !------------------------------------------------------------------

    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SAVE_EXC_VRS'
    INTEGER  Js, Jwl, NNv, Nl
    REAL (KND), DIMENSION (GSL_W%NN*Nvs,GSL_W%NN*Nvs) :: Gpp, Gmp
    REAL (KND), POINTER :: Fip(:,:), Fim(:,:)
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')
    IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(A)') '  *** '//R_NAME
    
    Jwl = SVR%Jexi
    Nl  = AGR_W%NL
    
    ! Save extinction coefficient and solar exponent at all VRS
    ! or fluorescence excitation wavelengths
    IF( Mh == 1 ) THEN
       SVR%EXT_AUX(Jwl,:) = (LOW%AL_W(1:Nl-1) + LOW%AL_W(2:Nl))/2.d0
       SVR%EXS_AUX(Jwl,:,:) = SSB_W%EXS(:,:)
    ENDIF
    ! WRITE(*,*) ' --> ', SUM(LOW%AL_W), SSB_W%EXS(Nl,:)

    ! Save separation constants and integration constants at all
    ! excitation wavelengths
    SVR%Nuj_EXC(Jwl,Mh,:,:) = SDO_W%Nuj_Lay(:,:)
    SVR%MVEC   (Jwl,Mh,:,:) = LAS_W%SVEC_AG(:,:)

! Coefficients aj and bj of the particular solution at
! excitation wavelengths include multipliers 
! EXP(-Tau_(l-1)/Mu0) calculated at these wavelengths
    DO I = 1,Nl-1
       DO Js = 1,NSZA  
          SVR%Aj(Jwl,Mh,:,I,Js) =SDO_W%Aj_sol(1,:,I,Js)*SSB_W%EXS(I,Js)
          SVR%Bj(Jwl,Mh,:,I,Js) =SDO_W%Bj_sol(1,:,I,Js)*SSB_W%EXS(I,Js)
       ENDDO
    ENDDO

    ! WRITE(*,*) ' --> ', SUM( SDO_W%Aj_sol(1,:,:,:1)), &
    !    SUM( SDO_W%Bj_sol(1,:,:,:1))

    NNv = GSL_W%NN*Nvs

    ! If only single inelastic process in water should be considered
    IF( SIPI ) THEN
       SELECT CASE( TRIM(INEL_PROC) )
       CASE('VRS')
          ! Fourier harmonics (1,2,3) of the Raman phase function 
          ! (array VRS_Egg) were calculated in "phase_function.f" 
          ! called in "rtm_init_first.f". First dimension was 
          ! divided by XM(:) cosines of Gauss angles.
          FORALL( K = 1:GSL_W%NN )
             Gpp(K,:) = VRS_Egg(K,:,1,Mh)*GSL_W%AG(:)*0.5d0 ! Mu <=> Mu
             Gmp(K,:) = VRS_Egg(K,:,2,Mh)*GSL_W%AG(:)*0.5d0 ! Mu <=> -Mu
          END FORALL
          ! Matrix G_NL was calculated in "homog_partic.f" and it has the 
          ! following structure:
          ! | Phi+ Phi- |  
          ! | Phi- Phi+ |
          DO I = 1,Nl - 1
             Fip => SDO_W%G_NL(I,      1:NNv,1:NNv) 
             Fim => SDO_W%G_NL(I,1+NNv:2*NNv,1:NNv)
             ! Arrays SVR%GNp and SVR%GNm are calculated according to
             ! Eqs.(Faj) and (Fbj) in STB. First dimension of these  
             ! arrays is Gauss angles, second - separation constants. 
             ! They represent components of the vibrational Raman 
             ! source function (see Eq.(Jram) in STB) and include 
             ! multiplier 1/2 but does not contains Raman scattering 
             ! coefficient and does not divided by extinction coefficient.
             ! It will be done in "part_sol_vrs.f". Due to the symmetry 
             ! relations (see Eq.(Symm-rel) in STB)   we calculate here 
             ! SVR%GNp(+mu,:) and SVR%GNm(+mu,:) only.
             DO K = 1,NNv
                SVR%GNp(Jwl,Mh,I,:,K) = MATMUL(Gpp(:,:),Fip(:,K)) & 
                     + MATMUL(Gmp(:,:),Fim(:,K))
                SVR%GNm(Jwl,Mh,I,:,K) = MATMUL(Gpp(:,:),Fim(:,K)) &
                     + MATMUL(Gmp(:,:),Fip(:,K))
             ENDDO
          ENDDO
       CASE DEFAULT
          FORALL( K = 1:GSL_W%NN )
             Gpp(K,:) = 1.d0/GSL_W%XM(K)*GSL_W%AG(:)*0.5d0 
          END FORALL
          Gmp(:,:) = Gpp(:,:)
          DO I = 1,Nl - 1
             Fip => SDO_W%G_NL(I,      1:NNv,1:NNv) 
             Fim => SDO_W%G_NL(I,1+NNv:2*NNv,1:NNv)
             DO K = 1,NNv
                SVR%GNp(Jwl,Mh,I,:,K) = MATMUL(Gpp(:,:),Fip(:,K)) &
                     + MATMUL(Gmp(:,:),Fim(:,K))
                SVR%GNm(Jwl,Mh,I,:,K) = MATMUL(Gpp(:,:),Fim(:,K)) &
                     + MATMUL(Gmp(:,:),Fip(:,K))
             ENDDO
          ENDDO
          ! STOP '   st_dom_tau "Not implemented yet"'
       END SELECT
    ELSE
       DO I = 1,Nl - 1
          Fip => SDO_W%G_NL(I,      1:NNv,1:NNv) 
          Fim => SDO_W%G_NL(I,1+NNv:2*NNv,1:NNv)
          DO K = 1,NNv
             SVR%GNp(Jwl,Mh,I,:,K) = Fip(:,K)
             SVR%GNm(Jwl,Mh,I,:,K) = Fim(:,K)
          ENDDO
       ENDDO
    ENDIF

    !      WRITE(*,'(1x,2(1x,I4),2(1x,e12.4))') JPOS, Jwl, &
    !          SUM(SVR%GNp(Jwl,Mh,:,:,:)), SUM(SVR%GNm(Jwl,Mh,:,:,:))

    ! If Jpos-th wavelength is excitation WL for VRS and
    ! RRS simultaneously, then DO_OUT should be defined
    ! in subroutine "save_exc_rrs.f"
    IF( SRR%Exi_WL(JPOS) ) THEN
       CALL ST_TRACK (R_NAME, 1, 'O')
       RETURN
    ENDIF
    
    ! SVR%Lo_VRS(:)=.TRUE. for VRS excitation wavelengths 
    IF( SVR%Lo_VRS(JPOS) ) THEN
       ! Three harmonics should be saved for VRS. 
       IF( Mh==3 ) DO_OUT = .TRUE.
    ELSE
       ! One harmonic only should be saved for fluorescence processes.
       IF( Mh==1 ) DO_OUT = .TRUE.
    ENDIF
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN 1
    
  END SUBROUTINE SAVE_EXC_VRS

!**********************************************************************
!    Here will be saved all needed results obtained at excitation 
!           wavelength corresponding to the rotational Raman 
!                     scattering in atmosphere
!**********************************************************************
  SUBROUTINE SAVE_EXC_RRS(*)
    ! Called by "st_dom_tau.f"

    ! 2+shi
    USE RAMAN,  ONLY: Ram_Egg

    !------------------------------------------------------------------
    CHARACTER (LEN=12), PARAMETER :: R_NAME = 'SAVE_EXC_RRS'
    INTEGER  K, J, Js, NNv, NL, Jwl
    ! 4+shi
    REAL (KND)  Hel, Gmp(GSL_A%NN*Nvs,GSL_A%NN*Nvs), &
         Gpp(GSL_A%NN*Nvs,GSL_A%NN*Nvs)
    REAL (KND), ALLOCATABLE :: TTT(:,:)
    REAL (KND), POINTER :: Fip(:,:), Fim(:,:)
    !==================================================================

    CALL ST_TRACK (R_NAME, 1, 'I')
    IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(A)') '  *** '//R_NAME

    ! If mode APPR_RRS='exact matching' is selected
    ! ---------------------------------------------
    ! then for j-th actual wavelength all corresponding data will 
    ! be saved in [ 1:N_WL_C ] elements of corresponding arrays, where 
    ! N_WL_C is the number of excitation wavelengths which produced 
    ! Raman lines having the position at an actual wavelength (see 
    ! "ext_wl_rrs.f"). To save memory the same arrays will be used 
    ! for the next actual wavelength.
    
    Jwl = SRR%Jexi
    
    NL = AGR_A%NL
    ! Save extinction coefficient and solar exponent
    IF( Mh == 1 ) THEN
       SRR%EXT_AUX(Jwl,:)   = (LOA%AL(1:NL-1) + LOA%AL(2:NL))/2.d0
       SRR%EXS_AUX(Jwl,:,:) = SSB_A%EXS(:,:)
    ENDIF
    
    ! Save separation and integration constants
    SRR%Nuj_EXC(Jwl,Mh,:,:) = SDO_A%Nuj_Lay(:,:)

    IF( ANY(SDO_A%NcC(:) > 0) ) THEN
       WRITE(*,*) '  Complex eigenvalues: '
       WRITE(*,'(A)') '    the altitude levels and '
       WRITE(*,'(A)') '  number of complex eigenvalues'
       DO I = 1, NL-1
          IF( SDO_A%NcC(I)==0 ) CYCLE
          WRITE(*,'(2x,I3,2x,I2)') I, SDO_A%NcC(I)
       ENDDO
       ! PAUSE
    ENDIF

    ! CHECK negative or small values
    DO I = 1, NL-1 
       DO K = 1, GSL_A%NN*Nvs
          IF( SDO_A%Nuj_Lay(K,I) < 0.d0 ) THEN
             WRITE(*,*) '  Nuj - negative "st_dom_tau.f" '
             WRITE(*,*) Jwl, I, K,  SDO_A%Nuj_Lay(K,I)
             ! PAUSE
          ENDIF
       ENDDO
    ENDDO
    
    SRR%MVEC   (Jwl,Mh,:,:) = LAS_A%SVEC_AG(:,:)
    
    ! Coefficients aj and bj of the particular solution at
    ! excitation wavelengths include multipliers 
    ! EXP(-Tau_(l-1)/Mu0) calculated at these wavelengths
    ! WRITE(*,*) '   Shape(SSB_A%EXS) ',Shape(SSB_A%EXS)
    ! WRITE(*,*) '   Shape(SDO_A%Aj_sol) ',Shape(SDO_A%Aj_sol)
    DO I = 1,NL-1
       DO Js = 1,NSZA  
          SRR%Aj(Jwl,Mh,:,I,Js) =SDO_A%Aj_sol(1,:,I,Js)*SSB_A%EXS(I,Js)
          SRR%Bj(Jwl,Mh,:,I,Js) =SDO_A%Bj_sol(1,:,I,Js)*SSB_A%EXS(I,Js)
       ENDDO
    ENDDO
    
    NNv = GSL_A%NN*Nvs
    
    ! Fourier harmonics (1,2,3) of the Raman phase function 
    ! (array Ram_Egg) were calculated in "phase_function.f"
    ! (scalar case) or in "Phase_matr.f" (vector case) called in
    ! "rtm_init_first.f". 
    SELECT CASE ( Vec_Sca )
    CASE( 'scal_rte' )
       ! First dimension of Ram_Egg(:,:,1:2,Mh) contains 1/XM(:)
       ! cosine of Gauss angles.
       FORALL( K = 1:NNv )
          Gpp(K,:) = Ram_Egg(K,:,1,Mh)*GSL_A%AG(:)*0.5d0 ! Mu <=> Mu
          Gmp(K,:) = Ram_Egg(K,:,2,Mh)*GSL_A%AG(:)*0.5d0 ! Mu <=> -Mu
       END FORALL
    CASE( 'vect_rte' )
       ! See comments in "st_up_down_sfi.f" to the array Pgg.
       ALLOCATE( TTT(NNv,NNv) ) 

       TTT(:,:) = Ram_Egg(:,:,1,Mh) + Ram_Egg(:,:,2,Mh)
       FORALL( K = 1:GSL_A%NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) =  &
            TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL_A%XM(K) 
       FORALL( K = 1:GSL_A%NN ) Gpp(:,Nvs*(K-1)+1:Nvs*K) =  &
            TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL_A%AG(K)*0.5d0

       TTT(:,:) = Ram_Egg(:,:,1,Mh) - Ram_Egg(:,:,2,Mh)
       FORALL( K = 1:GSL_A%NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) =  &
            TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL_A%XM(K) 
       FORALL( K = 1:GSL_A%NN ) Gmp(:,Nvs*(K-1)+1:Nvs*K) =  &
            TTT(:,Nvs*(K-1)+1:Nvs*K)*GSL_A%AG(K)*0.5d0
       
    END SELECT

    DO I = 1,NL - 1
       Fip => SDO_A%G_NL(I,      1:NNv,1:NNv) 
       Fim => SDO_A%G_NL(I,1+NNv:2*NNv,1:NNv)
       DO K = 1,NNv
          SRR%GNp(Jwl,Mh,I,:,K) =  &
               MATMUL(Gpp(:,:),Fip(:,K)) + MATMUL(Gmp(:,:),Fim(:,K))
          SRR%GNm(Jwl,Mh,I,:,K) =  &
               MATMUL(Gpp(:,:),Fim(:,K)) + MATMUL(Gmp(:,:),Fip(:,K))
       ENDDO
    ENDDO
    
    ! Three harmonic should be saved only in the scalar case
    IF( Mh==3 ) DO_OUT = .TRUE.
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN 1
  END SUBROUTINE SAVE_EXC_RRS
  !**********************************************************************

  SUBROUTINE SPHERE_FIELD (*)
    ! Called by "st_dom_tau.f"

    USE CLOUDS,      ONLY: DO_CLOUD
    USE INTERN_DIMS, ONLY: Nlg_ray
    USE ST_VARS,     ONLY: DO_AEROSOL, BRDF_FLAG, ACCUR
    !------------------------------------------------------------------
    CHARACTER (LEN=12), PARAMETER :: R_NAME = 'SPHERE_FIELD'
    LOGICAL  DO_CONV
    INTEGER  L1, L2, Ks
    REAL (KND) DIFER, Cosi, Sini
    REAL (KND), PARAMETER :: Min_Sc2 = 1.d-06
    REAL (KND) DELTA(NL_MS, NSOL_MS, N_SFN)
    !==================================================================
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    
    IF( Mh == 1) THEN
       DO_CONV1 = .FALSE.
       SELECT CASE( Vec_Sca )
       CASE( 'scal_rte' )
          DO L = 1,N_PHI_MS
             Ks = N_NODE_MS*(L-1) ! Shift to the next azimuthal angle
             DO I = 1,N_NODE_MS
                DELTA(:,:,Ks+I) = IOP_A%DFI(:,:,I)
             ENDDO
          ENDDO
          
          ! WRITE(*,*) '    Top of the atmosphere (scalar) ',N_NODE_MS        
!          DO I =  N_NODE_MS/2+1, N_NODE_MS
!             WRITE(*,'(1x,I3,*(1x,e12.4))')  I, DELTA(1,1,I), &
!                  DELTA(1,1,I+N_NODE_MS),  DELTA(1,1,I+N_NODE_MS*2)
!            ENDDO

          RAD_PS = DELTA
       CASE( 'vect_rte' )
          ! as in scalar case multiplier 2. is included 
          ! in Fourier series terms
          DO L = 1,N_PHI_MS
             Ks = Nvs*N_NODE_MS*(L-1) ! Shift to the next azimuthal angle
             DO K = 1,N_NODE_MS/2 ! Downwards radiation
                L1 = (K-1)*Nvs ; L2 = Ks + L1
                DELTA(:,:,L2+1:L2+4) = IOP_A%DFI(:,:,L1+1:L1+4)
             ENDDO
             DO K = N_NODE_MS/2+1,N_NODE_MS ! Upward radiation
                L1 = (K-1)*Nvs ; L2 = Ks + L1
                DELTA(:,:,L2+1:L2+2) = IOP_A%DFI(:,:,L1+1:L1+2)
                ! Sign according to Eq.(I_hom) Nd (I_part)
                DELTA(:,:,L2+3:L2+4) = -IOP_A%DFI(:,:,L1+3:L1+4)
             ENDDO
          ENDDO
          RAD_PS = 0.5d0*DELTA
          
          ! WRITE(*,*) '    Top of the atmosphere (vector) I ',N_NODE_MS        
          !DO I =  Nvs*N_NODE_MS/2, Nvs*(N_NODE_MS-1), Nvs
          !   WRITE(*,'(1x,I3,*(1x,e12.4))')  I, 0.5d0*DELTA(1,1,I+1 ), &
          !        0.5d0*DELTA(1,1,I+1 + N_NODE_MS*Nvs), &
          !        0.5d0*DELTA(1,1,I+1 + N_NODE_MS*Nvs*2)
          !ENDDO

       END SELECT
    ELSE
       ! AZM_MS - azimuthal angles [radian] (see "rtm_init_first.f")
       SELECT CASE( Vec_Sca )
       CASE( 'scal_rte' )
          DO L = 1,N_PHI_MS
             Cosi = 2.d0*E_W*DCOS( AZM_MS(L)*DBLE(Mh-1) )
             Ks = N_NODE_MS*(L - 1)
             DO I = 1,N_NODE_MS
                DELTA(:,:,Ks+I) = IOP_A%DFI(:,:,I)*Cosi
             ENDDO
          ENDDO
       CASE( 'vect_rte' )
          ! as in scalar case multiplier 2. is included 
          ! in Fourier series terms
          DO L = 1,N_PHI_MS
             Cosi = E_W*DCOS( AZM_MS(L)*DBLE(Mh-1) )
             Sini = E_W*DSIN( AZM_MS(L)*DBLE(Mh-1) )
             Ks = Nvs*N_NODE_MS*(L - 1)
             DO K = 1,N_NODE_MS/2
                L1 = (K-1)*Nvs ; L2 = Ks + L1
                DELTA(:,:,L2+1:L2+2) = IOP_A%DFI(:,:,L1+1:L1+2)*Cosi
                DELTA(:,:,L2+3:L2+4) = IOP_A%DFI(:,:,L1+3:L1+4)*Sini
             ENDDO
             DO K = N_NODE_MS/2 + 1,N_NODE_MS
                L1 = (K-1)*Nvs ; L2 = Ks + L1
                DELTA(:,:,L2+1:L2+2) = IOP_A%DFI(:,:,L1+1:L1+2)*Cosi
                ! Sign according to Eq.(I_hom) and (I_part)
                DELTA(:,:,L2+3:L2+4) = -IOP_A%DFI(:,:,L1+3:L1+4)*Sini
             ENDDO
          ENDDO
       END SELECT
       ! Add the Fourier harmonic to the intensity value
       RAD_PS = RAD_PS + DELTA
    ENDIF
      
    ! Check convergence for all radiation field
    IF( Mh > 1 ) THEN
       ! WRITE(*,*) '  NSZA =  ', NSZA
       ! WRITE(*,*) '  NSOL_MS =  ', NSOL_MS
       DO_CONV = .TRUE.
       SELECT CASE( Vec_Sca )
       CASE( 'scal_rte' )
          DIFER = MAXVAL( DABS( DELTA(:,:,:)/RAD_PS(:,:,:) ) )            
          IF (LOG_LEVEL(1) >= N_SSO)  Then
             WRITE(Lunit,*) '           SCALAR CASE Mh = ',Mh
             WRITE(Lunit,*) '  ------>> Difference and accuracy creat. '
             WRITE(Lunit,'(5x,2(2x,e12.4))') DIFER, ACCUR
          End If
       CASE( 'vect_rte' )
          DO J = 1, NL
             DO I = 1,NSZA
                DO K = 1,N_SFN
                   IF( DABS(RAD_PS(J,I,K)) > Min_Sc2 ) THEN 
                      DELTA(J,I,K) = DELTA(J,I,K)/RAD_PS(J,I,K)
                   ELSE
                      DELTA(J,I,K) = 0.d0
                   ENDIF
                ENDDO
             ENDDO
          ENDDO
          DIFER = MAXVAL( DABS(DELTA(:,:,:)) )
          
          IF (LOG_LEVEL(1) >= N_SSO)  Then
             WRITE(Lunit,*) '           VECTOR CASE Mh = ',Mh
             WRITE(Lunit,*) '  ------>> Difference and accuracy creat. '
             WRITE(Lunit,'(5x,2(2x,e12.4))') DIFER, ACCUR
          End If
          
       END SELECT
       
       IF ( DO_CONV .AND. (DIFER > ACCUR) ) DO_CONV = .FALSE.  
       
       ! set output flag
       !----------------
       IF ( DO_CONV1 .AND. DO_CONV) DO_OUT = .TRUE.
       DO_CONV1 = DO_CONV
       
       IF( (.NOT.DO_AEROSOL).AND.(.NOT.DO_CLOUD)) THEN 
          IF((Mh == Nlg_Ray).AND.(.NOT.BRDF_FLAG)) DO_OUT = .TRUE.
       ENDIF
    ENDIF
    
    IF( DO_OUT ) THEN
       CALL SFI_FIELD(JPOS, RAD_PS, LOA%AL, E_W, OSM, LOA, SLS%LSE, TE, ADI)
    ENDIF
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN 
  END SUBROUTINE SPHERE_FIELD
  !**********************************************************************
  
  SUBROUTINE SPHERE_FIELD_S (*)
    ! Called by "st_dom_tau.f"
    
    USE CLOUDS,      ONLY: DO_CLOUD
    USE INTERN_DIMS, ONLY: Nlg_ray
    USE ST_VARS,     ONLY: DO_AEROSOL, BRDF_FLAG, ACCUR, DIFB, &
                           NSTRMS, V1_S2 => VS_HC, N_HARM, DO_AZIMUTH
    USE TYPES,       ONLY: WFS_PS, PCC_TYPE
    !------------------------------------------------------------------
    TYPE (PCC_type) :: PCC    ! Photochemistry   
    TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SPHERE_FIELD_S'
    LOGICAL  DO_CONV
    INTEGER  I, J, K, L1, Ks, Ja
    REAL (KND) DIFER, Cosi, Azi, DEP_CF(3)
    REAL (KND), PARAMETER :: Min_Sc2 = 1.d-06
    REAL (KND), ALLOCATABLE :: WF_OUT(:,:)
    LOGICAL   INDEX_TH(N_SL)
    REAL (KND) H_plus(N_PLUS)
    INTEGER    IH_plus(N_PLUS)
    !==================================================================

    INTERFACE
       Subroutine Integrate_Init (RAD, ALBEDO, DEP_CF, LOA, PCC,  ADI,  WFPS,&
            WF_OUT, JPOS, ESFT_WEIGHT, INDEX_TH, POS, OSM, *, FLU_Int, TE)
         USE GRIDS,      ONLY: NLAY=>NLAYERS
         USE SPHER_PARS, ONLY: N_LOS_ANG
         USE TYPES,      ONLY: KND, PCC_TYPE, OSM_TYPE, AuxDataInterp,&
                               WFS_PS, LOA_TYPE, TE_TYPE
         INTEGER     JPOS
         REAL (KND), ALLOCATABLE :: RAD(:,:,:,:), WF_OUT(:,:)
         REAL (KND) AL(NLAY), DEP_CF(3)
         REAL (KND)  ALBEDO, FLU_Int, ESFT_WEIGHT, POS
         LOGICAL    INDEX_TH(N_LOS_ANG)
         TYPE (PCC_type) :: PCC    
         TYPE (OSM_type) :: OSM  
         TYPE (LOA_type) :: LOA 
         TYPE (TE_type)  :: TE  
         TYPE (AuxDataInterp) :: ADI
         TYPE (WFS_PS), ALLOCATABLE :: WFPS(:)
       END Subroutine Integrate_Init
    END INTERFACE
    
    CALL ST_TRACK (R_NAME, 1, 'I')
    INDEX_TH(:) = .TRUE.

    IF( Mh == 1) THEN
       DO_CONV  = .FALSE. ;  DO_CONV1 = .FALSE.
       ! Change sign if interpolation is used
       IF( Nvs > 1 ) DELs(:,:,:,3:4) = DELs(:,:,:,3:4)
       RADs_PS = DELs *V1_S2/2.0d0
    ELSE
       DO Ja = 1, N_Node ;  Azi = PHI_NODE(Ja)
          Azi = Azi*DBLE(Mh-1) ; Cosi = E_W*DCOS(Azi)
          DELs(:,:,Ja,1) = DELs(:,:,Ja,1)*Cosi*V1_S2
          IF( Nvs == 1 ) CYCLE
          DELs(:,:,Ja,2)   =  DELs(:,:,Ja,2)*Cosi
          DELs(:,:,Ja,3:4) = DELs(:,:,Ja,3:4)*E_W*DSIN(Azi)
       ENDDO  ! over spherical node (Ja)
       ! Add next Fourier harmonic to the Stokes vector
       RADs_PS = RADs_PS + DELs
    ENDIF
    
    ! Check convergence for all radiation field
    IF( Mh > 1 ) THEN
       DO_CONV = .TRUE.
       DELs(:,:,:,1) = DELs(:,:,:,1)/RADs_PS(:,:,:,1)
       IF( Nvs > 1 ) THEN
          DELs(:,:,:,2:4) = 0.d0
          DO I = 1, NL_MS
             DO Js = 1, NSOL_MS
                DO Ja = 1, N_Node
                   DO Ks = 2, Nvs
                      IF( DABS(RADs_PS(I,Js,Ja,Ks)) > Min_Sc2 ) &
                           DELs(I,Js,Ja,Ks) = &
                           DELs(I,Js,Ja,Ks)/RADs_PS(I,Js,Ja,Ks)
                   ENDDO  ! over Stokes components (Ks)
                ENDDO  ! over spherical node (Ja)
             ENDDO  ! over solar angles (Js)
          ENDDO  ! over main altitude grid
       ENDIF
       DIFER = MAXVAL( DABS(DELs(:,:,:,1:Nvs)) )
         
       IF (LOG_LEVEL(1) >= N_DSO)  Then
          WRITE(Lunit,'(A,2x,I3)') &
               '  ------>> Difference and accuracy creat. Mh = ', Mh
          WRITE(Lunit,'(5x,2(2x,e12.4))') DIFER, ACCUR
       End If
            
       IF ( DO_CONV .AND. (DIFER > ACCUR) ) DO_CONV = .FALSE.  
       ! set output flag
       !----------------
       IF ( DO_CONV1 .AND. DO_CONV) DO_OUT = .TRUE.
       DO_CONV1 = DO_CONV
       
       IF( (.NOT.DO_AEROSOL).AND.(.NOT.DO_CLOUD)) THEN 
          IF((Mh == Nlg_Ray).AND.(.NOT.BRDF_FLAG)) DO_OUT = .TRUE.
       ENDIF
       
       IF( Mh == N_HARM ) THEN
          ! WRITE(*,*) '   Number of maximal harmonic is reached '
          DO_OUT = .TRUE.
       ENDIF
    ENDIF  ! Mh > 1
    
    ! No contribution of higher harmonics under diffuse 
    ! ilumination from below
    IF ( DIFB .OR. Sec_Order.OR. (.NOT.DO_AZIMUTH) ) THEN
       DO_CONV1 = .TRUE.  ;  DO_OUT   = .TRUE.
    ENDIF
    
    IF( DO_OUT ) THEN
       ! Set wavelength dependent multiplier (Rayleigh scattering matrix)
       DEP_CF(1) = 3.0d0*(1.0d0 + DEPOL)/(4.0d0 + 2.0d0*DEPOL)
       DEP_CF(2) = 3.0d0*(1.0d0 - DEPOL)/(4.0d0 + 2.0d0*DEPOL)
       DEP_CF(3) = 1.5d0*(1.0d0 - 2.d0*DEPOL)/(1.0d0 + DEPOL/2.0d0)
       
       CALL Integrate_Init(RADs_PS, UDB_A%ALB_FS, DEP_CF, LOA, PCC, ADI, &
            WFPS, WF_OUT, JPOS, E_W, INDEX_TH, POS, OSM, *880, &
            UDB_A%FLU_Int, TE)
    ENDIF

    CALL ST_TRACK (R_NAME, 1, 'O')
    Return
    
880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN 1
  END SUBROUTINE SPHERE_FIELD_S
  
  !**********************************************************************
  SUBROUTINE ALLOCATE_ATM
    
    USE CLOUDS,      ONLY: DO_CLOUD, CLD_LAYER_NUM
    USE CLOUD_INPUT, ONLY: Phm_const, MLCS
    !------------------------------------------------------------------

    INTEGER  Max_SL, KK
    !==================================================================

    ! Allocate elements of the structure IOP_A
    !-----------------------------------------
    ALLOCATE( IOP_A%Id_a(NL-1), IOP_A%Id_b(NL-1), IOP_A%ALW(NL-1), & ! A 
         IOP_A%DTR(NL), IOP_A%DTR_U(NL), IOP_A%DTR_D(NL), & ! A 
         IOP_A%MVEC(2*NNv*(NL-1),N_RHS), IOP_A%DFI(NL,Kvs*NSZA,2*NNv), & ! A
         IOP_A%Ref_dir(NL,Kvs*NSZA,Nvs*NLSA), &   !A
         IOP_A%Ref_sol(NL,N_Adj,Nvs*NSZA), &      ! A
         IOP_A%Ex_U(NNv,NL-1), IOP_A%Ex_D(NNv,NL-1))  ! A
    DO KK = 1, N_AT
       ALLOCATE( PH_M%MET(KK)%Egg(NNv,NNv,2), &
            PH_M%MET(KK)%EgL(NNv,Nvs*NLSA,2),  &   !A
            PH_M%MET(KK)%Egs(NNv,Nvs*NSZA,2), &    !A
            PH_M%MET(KK)%EsL(Nvs*NLSA,Nvs*NSZA,N_sol_ls) )   !A
    ENDDO

    ! Test allocation 
    ! ALLOCATE( PH_M%RAY%Egg(NNv,NNv,2), &
    !      PH_M%RAY%EgL(NNv,Nvs*NLSA,2), &
    !      PH_M%RAY%Egs(NNv,Nvs*NSZA,2), &
    !      PH_M%RAY%EsL(Nvs*NLSA,Nvs*NSZA,N_sol_ls) )

    ALLOCATE(Ph_r%Egg(NNv,NNv,2), ph_r%EgL(NNv,Nvs*NLSA,2), &! A
         Ph_r%Egs(NNv,Nvs*NSZA,2), ph_r%EsL(Nvs*NLSA,Nvs*NSZA,N_sol_ls),& ! A
         ! Structure for aerosol phase matrices
         Ph_A%Egg(Nl_Ar,Ng_Ar,Ng_Ar, N_A2), & ! A
         Ph_A%EgL(Nl_Ar,Ng_Ar,Nls_Ar,N_A2), & ! A
         Ph_A%Egs(Nl_Ar,Ng_Ar,Nso_Ar,N_A2), & ! A
         Ph_A%EsL(Nl_Ar,Nls_Ar,Nso_Ar,N_sol_ls), & ! A
         ! Structure for sum of all phase matrices
            Ph_zA%Egg(NL,NNv,NNv,2), Ph_zA%EgL(NL,NNv,Nvs*NLSA,2), & ! A
            Ph_zA%Egs(NL,NNv,Nvs*NSZA,2), & ! A
            Ph_zA%EsL(NL,Nvs*NLSA,Nvs*NSZA,N_sol_ls)) ! A

    Ph_zA%Egg = 0.d0 ; Ph_zA%EgL = 0.d0 
    Ph_zA%Egs = 0.d0 ; Ph_zA%EsL = 0.d0

    ! Use derivatives pf aerosol optical parameters wrt 
    ! modal radius and variance of PSD function
    IF( Dwr_mi ) THEN
       ALLOCATE( Ph_dA%Egg(Nl_Ar,Ng_Ar, Ng_Ar, N_A2, N_dwr), & ! A
            Ph_dA%EgL(Nl_Ar,Ng_Ar, Nls_Ar,N_A2, N_dwr), & ! A
            Ph_dA%Egs(Nl_Ar,Ng_Ar, Nso_Ar,N_A2, N_dwr), & ! A
            Ph_dA%EsL(Nl_Ar,Nls_Ar,Nso_Ar,N_sol_ls, N_dwr) )  ! A
    ENDIF

    ! Cloud phase functions related to WF
    !------------------------------------
    Max_SL = 1
    IF( DO_CLOUD ) THEN
!       IF( Phm_const ) THEN
       IF( ANY(MLCS(:)%Phm_co) ) THEN
          Max_SL = 1
       ELSE
          Max_SL = INT( MAXVAL( CLD_LAYER_NUM,1 ) ) + 3
       ENDIF
       IF( .NOT.ANY(MLCS(:)%Phm_co).AND.INQ_WF('cld', 'WFT') ) THEN
          ALLOCATE(Ph_C%Dr_GG(Nl_Cr, Max_sl, Ng_Cr, Ng_Cr, N_C2,Nm_ice),& ! A
               Ph_C%Dr_GL(Nl_Cr, Max_sl, Ng_Cr, Nls_Cr,N_C2,Nm_ice),& ! A
               Ph_C%Dr_GS(Nl_Cr, Max_sl, Ng_Cr, Nso_Cr,N_C2,Nm_ice),& ! A
               Ph_C%Dr_LS(Nl_Cr, Max_sl, Nls_Cr,Nso_Cr,Nm_ice) ) ! A 
          Ph_C%Dr_GG = 0.d0
          Ph_C%Dr_GL = 0.d0
          Ph_C%Dr_GS = 0.d0
          Ph_C%Dr_LS = 0.d0           
       ENDIF
    ENDIF  ! DO_CLOUD
    

    ALLOCATE(Ph_C%C_GG(Nl_Cr, Max_sl, Ng_Cr, Ng_Cr, N_C2,Nm_ice),& ! A
         Ph_C%C_GL(Nl_Cr, Max_sl, Ng_Cr, Nls_Cr,N_C2,Nm_ice),& ! A
         Ph_C%C_GS(Nl_Cr, Max_sl, Ng_Cr, Nso_Cr,N_C2,Nm_ice),& ! A
         Ph_C%C_LS(Nl_Cr, Max_sl, Nls_Cr,Nso_Cr,Nm_ice) )     ! A 
    Ph_C%C_GG = 0.d0
    Ph_C%C_GL = 0.d0
    Ph_C%C_GS = 0.d0
    Ph_C%C_LS = 0.d0
    
    ALLOCATE( SDO_A%H1(NL), SDO_A%NcC(NL-1), SDO_A%E_MIN(NL-1),&
         SDO_A%MASK_NL(NL-1), SDO_A%Nuj_Lay(Nvs*NN,NL-1),&     ! A
         SDO_A%EXP_NU_NL(Nvs*NN,NL-1), SDO_A%Ex_AG(Nvs*NN,2),& ! A
         SDO_A%Aj_sol (Kvs,Nvs*NN,NL-1,NSZA),&                 ! A
         SDO_A%Bj_sol (Kvs,Nvs*NN,NL-1,NSZA), &                ! A
         SDO_A%G_NL   (NL,2*NNv,2*NNv) )                      ! A
    SDO_A%Aj_sol = 0.0D0 ;  SDO_A%Bj_sol = 0.0D0
    
    IF( UDB_A%SPECULAR ) THEN
       ALLOCATE( SDO_A%Aj_spe (Kvs,Nvs*NN,NL-1,NSZA),& ! A
            SDO_A%Bj_spe (Kvs,Nvs*NN,NL-1,NSZA) ) ! A
    ENDIF
        
    ALLOCATE( SDO_A%Ajp(Kvs,0:NL-1,Nvs*NN,NSZA), &
         SDO_A%Bjp (Kvs,0:NL-1,Nvs*NN,NSZA) ) ! A

    ! Adjoint RTE related arrays (atmosphere)
    ! N_Adj is set in "rtm_init.f" or in "geom_along_los.f"
    ! in case of RTM_TYPE = 'spher_scat'
    
    ! WRITE(*,*) '    N_adj in "st_dom_tau"  ', n_adj
    ! PAUSE

    ALLOCATE( ADO_A%Part_TU(NRL,2*NNv,N_Adj),& ! A 
         ADO_A%Part_TD(NRL,2*NNv,N_Adj), ADO_A%AFI(NL,2*NNv,N_Adj),& ! A
         ADO_A%Ajs(0:NL-1,NNv,N_Adj), ADO_A%Bjs(0:NL-1,NNv,N_Adj), & ! A
         ADO_A%LVEC(2*NNv*(NL-1),N_Adj),ADO_A%FLUX_D(N_Adj) )       ! A

    ! Adjoint RTE Spherical geometry
    !-------------------------------
    IF( TRIM(RTM_TYPE) == 'spher_scat' ) THEN
       ALLOCATE( ADS%Act_ag(NL-1,N_Adj), ADS%Pol_ga(Leg,2*NN),&     ! A
            ADS%PLS(NL,NN,NLSA,N_Eff), ADS%MLS(NL,NN,NLSA,N_Eff),&  ! A
            ADO_A%Aj_sph(NL-1,NNv,NLSA*NFA,NS_wf,2),&               ! A
            ADO_A%Bj_sph(NL-1,NNv,NLSA*NFA,NS_wf,2) ) ! A
    ELSE
       ALLOCATE( ADO_A%Aj_ls(NNv,N_Adj,NL-1), ADO_A%Bj_ls(NNv,N_Adj,NL-1) ) ! A
    ENDIF
! Solution of the linear algebraic system in the atmosphere
!----------------------------------------------------------
    ALLOCATE( LAS_A%Flux(Kvs*NSZA), LAS_A%Ref_surf(N_RHS,Nvs*(NN+Npg)),&  ! A
         LAS_A%SOL_TOP(2*NNv,N_RHS), LAS_A%SOL_BOT(2*NNv,N_RHS), & ! A
         LAS_A%PART_U(NRL,2*NNv,Kvs*NSZA), LAS_A%PART_D(NRL,2*NNv,Kvs*NSZA),&! A
         LAS_A%INDX(2*NNv*(NRL-1)), LAS_A%SVEC_AG(2*NNv*(NRL-1),N_RHS), & ! A
         LAS_A%MATTEB (9*NNv-2,2*NNv*(NRL-1)), & ! A
         LAS_A%RHS_LAS(2*NNv*(NRL-1),N_RHS))   ! A
    
    LAS_A%SOL_TOP = 0.d0
    LAS_A%SOL_BOT = 0.d0
  END SUBROUTINE ALLOCATE_ATM
  !**********************************************************************

  SUBROUTINE DEALLOCATE_ATM

    USE CLOUDS,      ONLY: DO_CLOUD
    USE CLOUD_INPUT, ONLY: Phm_const, MLCS
    INTEGER  KK
    !------------------------------------------------------------------

    IF(N_AT /= 0) THEN
       DO KK = 1, N_AT
          DEALLOCATE( PH_M%MET(KK)%Egg, PH_M%MET(KK)%EgL, &
               PH_M%MET(KK)%Egs, PH_M%MET(KK)%EsL )
       END DO
    END IF

    DEALLOCATE( &
    ! Elements of the structure LAS_A
         LAS_A%Flux, LAS_A%Ref_surf, LAS_A%SOL_TOP, LAS_A%SOL_BOT, & ! D
         LAS_A%PART_U, LAS_A%PART_D, LAS_A%INDX, LAS_A%SVEC_AG, & ! D
         LAS_A%MATTEB, LAS_A%RHS_LAS, & ! D
    !... 
         Ph_r%Egg, ph_r%EgL, Ph_r%Egs, ph_r%EsL, Ph_zA%Egg, Ph_zA%EgL, & ! D
         Ph_zA%Egs, Ph_zA%EsL, Ph_A%Egg, Ph_A%EgL, Ph_A%Egs, Ph_A%EsL, & ! D
         ! Cloud phase function related to WF
         Ph_C%C_GG, Ph_C%C_GL, Ph_C%C_GS, Ph_C%C_LS, & ! D
         ! Elements of the structure SDO_A
         SDO_A%H1, SDO_A%NcC, SDO_A%E_MIN, SDO_A%MASK_NL, SDO_A%Nuj_Lay,& ! D
         SDO_A%EXP_NU_NL, SDO_A%Ex_AG, SDO_A%Aj_sol, SDO_A%Bj_sol, & ! D
         SDO_A%Ajp, SDO_A%Bjp, SDO_A%G_NL, &         ! D
         ! Elements of the structure IOP_A
         IOP_A%Id_a, IOP_A%Id_b, IOP_A%ALW, IOP_A%DTR, IOP_A%DTR_U, & ! D
         IOP_A%DTR_D, IOP_A%MVEC, IOP_A%DFI,  IOP_A%Ex_U, IOP_A%Ref_dir, & ! D
         IOP_A%Ref_sol, IOP_A%Ex_D )         ! D
      
    IF( UDB_A%SPECULAR ) DEALLOCATE( SDO_A%Aj_spe, SDO_A%Bj_spe ) ! D

    IF( Dwr_mi ) DEALLOCATE( Ph_dA%Egg, Ph_dA%EgL, Ph_dA%Egs, Ph_dA%EsL ) ! D

    IF( DO_CLOUD ) THEN
       IF( .NOT.ANY(MLCS(:)%Phm_co) .AND. INQ_WF('cld', 'WFT') ) &
            DEALLOCATE( Ph_C%Dr_GG, Ph_C%Dr_GL, Ph_C%Dr_GS, Ph_C%Dr_LS)
    ENDIF
    
    ! Adjoint RTE related arrays (atmosphere)
    DEALLOCATE( ADO_A%Part_TU, ADO_A%Part_TD, ADO_A%Afi, &  ! D
         ADO_A%Ajs, ADO_A%Bjs, ADO_A%LVEC, ADO_A%FLUX_D ) ! D

    ! Adjoint RTE Spherical geometry
    IF( TRIM(RTM_TYPE) == 'spher_scat' ) THEN
       DEALLOCATE( ADS%Act_ag, ADS%Pol_ga, ADS%PLS, ADS%MLS, &! D
            ADO_A%Aj_sph, ADO_A%Bj_sph ) ! D              
    ELSE
       DEALLOCATE( ADO_A%Aj_ls, ADO_A%Bj_ls ) ! D
    ENDIF
  END SUBROUTINE DEALLOCATE_ATM
  !**********************************************************************

  SUBROUTINE ALLOCATE_OCEAN
    INTEGER NRL_W, Jgv
    
    NRL_W = AGR_W%NRL
    NNv_W = Nvs*GSL_W%NN
    Nz = NL_W       ; K1 = Nvs*GSL_W%NN
    Js = GSL_W%NSZA ; Jv = GSL_W%NLSA
    Jg = GSL_W%NN  

! Allocate elements of the structure IOP_W
!-----------------------------------------
    ALLOCATE( IOP_W%Id_a(Nz-1), IOP_W%Id_b(Nz-1), IOP_W%ALW(Nz-1), &
         IOP_W%DTR(Nz), IOP_W%DTR_U(Nz), IOP_W%DTR_D(Nz), &        ! A
         IOP_W%MVEC(2*K1*(Nz-1),N_RHS), IOP_W%DFI(Nz,Kvs*Js,2*K1), &! A
         IOP_W%Ref_dir(Nz,Kvs*Js,Nvs*Jv), IOP_W%Ref_sol(Nz,N_Adj,Nvs*Js), &! A
         IOP_W%Ex_U(K1,Nz-1), IOP_W%Ex_D(K1,Nz-1) )               ! A

    ALLOCATE( Ph_rW%Egg(K1,K1,2), ph_rW%EgL(K1,Nvs*Jv,2), &! A
         Ph_rW%Egs(K1,Nvs*Js,2), ph_rW%EsL(Nvs*Jv,Nvs*Js,2), &! A
         LAS_W%Flux(Kvs*Js), LAS_W%Ref_surf(N_RHS,Nvs*(Jg+Npg)), & ! A
         LAS_W%SOL_TOP(2*K1,N_RHS), LAS_W%SOL_BOT(2*K1,N_RHS), &  ! A
         LAS_W%PART_U(NRL_W,2*K1,Kvs*Js), LAS_W%PART_D(NRL_W,2*K1,Kvs*Js), &! A
         LAS_W%INDX (2*K1*(NRL_W-1)), LAS_W%MATTEB (9*K1-2,2*K1*(NRL_W-1)),&! A 
         ! It is assumed preliminary that the number of r.h.s.
         ! of the ocean RTE is the same as atmospheric RTE
         LAS_W%SVEC_AG(2*K1*(NRL_W-1),N_RHS), & ! A
         LAS_W%RHS_LAS(2*K1*(NRL_W-1),N_RHS)  ) ! A

    LAS_W%SOL_TOP = 0.d0  ;  LAS_W%SOL_BOT = 0.d0

    Jgv = Nvs*Jg
    ! Adjoint RTE related arrays (ocean)
    ALLOCATE( ADO_W%AFI(Nz,2*Jgv,N_Adj), ADO_W%Part_TU(NRL_W,2*Jgv,N_Adj),&! A
         ADO_W%Part_TD(NRL_W,2*Jgv,N_Adj), ADO_W%Aj_ls(Jgv,N_Adj,Nz-1),& ! A
         ADO_W%Bj_ls(Jgv,N_Adj,Nz-1), ADO_W%Ajs(0:Nz-1,Jgv,N_Adj), & ! A
         ADO_W%Bjs(0:Nz-1,Jgv,N_Adj),& ! A
         ADO_W%LVEC(2*Jgv*(Nz-1),N_Adj), ADO_W%FLUX_D(N_Adj) ) ! A

    ALLOCATE( SDO_W%H1(Nz), SDO_W%NcC(Nz-1), SDO_W%E_MIN(Nz-1),& ! A
         SDO_W%MASK_NL(Nz-1), SDO_W%Nuj_Lay(K1,Nz-1), & ! A
         SDO_W%EXP_NU_NL(K1,Nz-1), SDO_W%Ex_AG(K1,2), & ! A
         SDO_W%Aj_sol(Kvs,K1,Nz-1,Js), SDO_W%Bj_sol(Kvs,K1,Nz-1,Js), & ! A
         SDO_W%G_NL(Nz,2*K1,2*K1) ) ! A
    SDO_W%Aj_sol = 0.0D0 ;  SDO_W%Bj_sol = 0.0D0 

    IF( WF_FLAG ) THEN 
       ! Particulate matter weighting function (small particles)
       IF( OWF_Ks ) ALLOCATE( Ph_Ks%Egg(K1,K1,2), ph_Ks%EgL(K1,Nvs*Jv,2), & ! A
            Ph_Ks%Egs(K1,Nvs*Js,2), ph_Ks%EsL(Nvs*Jv,Nvs*Js,2) )   ! A
       ! Particulate matter weighting function (larger particles)
       IF( OWF_Kl ) ALLOCATE( Ph_Kl%Egg(K1,K1,2), ph_Kl%EgL(K1,Nvs*Jv,2), & ! A
            Ph_Kl%Egs(K1,Nvs*Js,2), ph_Kl%EsL(Nvs*Jv,Nvs*Js,2) )   ! A
       ! Particulate matter weighting function (user supplied)
       IF( OWF_Use ) ALLOCATE( Ph_Us%Egg(Nhl,K1,K1,2), & ! A
            Ph_Us%EgL(Nhl,K1,Nvs*Jv,2), Ph_Us%Egs(Nhl,K1,Nvs*Js,2), & ! A
            Ph_Us%EsL(Nhl,Nvs*Jv,Nvs*Js,2) ) ! A
    ENDIF

    ALLOCATE(SDO_W%Ajp(Kvs,0:Nz-1,K1,Js), SDO_W%Bjp(Kvs,0:Nz-1,K1,Js))

    ALLOCATE( Ph_zW%Egg(Nz,K1,K1,2), Ph_zW%EgL(Nz,K1,Nvs*Jv,2),& ! A
         Ph_zW%Egs(Nz,K1,Nvs*Js,2), Ph_zW%EsL(Nz,Nvs*Jv,Nvs*Js,2) ) ! A

    Ph_zW%Egg = 0.D0 ; Ph_zW%EgL = 0.D0 
    Ph_zW%Egs = 0.D0 ; Ph_zW%EsL = 0.D0
    
  END SUBROUTINE ALLOCATE_OCEAN
  !**********************************************************************

  SUBROUTINE DEALLOCATE_OCEAN
    DEALLOCATE( Ph_rW%Egg, ph_rW%EgL, Ph_rW%Egs, ph_rW%EsL, & ! D
         Ph_zW%Egg, Ph_zW%EgL, Ph_zW%Egs, Ph_zW%EsL, & ! D
         ! Elements of the structure SDO_W
         SDO_W%H1, SDO_W%NcC, SDO_W%E_MIN, SDO_W%MASK_NL, SDO_W%Nuj_Lay,& ! D
         SDO_W%EXP_NU_NL, SDO_W%Ex_AG, SDO_W%Aj_sol, SDO_W%Bj_sol, & ! D
         SDO_W%Ajp, SDO_W%Bjp, SDO_W%G_NL, &         ! D
         ! Elements of the structure LAS_W
         LAS_W%Flux, LAS_W%Ref_surf, LAS_W%SOL_TOP, LAS_W%SOL_BOT, & ! D
         LAS_W%PART_U, LAS_W%PART_D, LAS_W%INDX, LAS_W%SVEC_AG, & ! D
         LAS_W%MATTEB, LAS_W%RHS_LAS,& ! D
         ! Elements of the structure IOP_W
         IOP_W%Id_a, IOP_W%Id_b, IOP_W%ALW, IOP_W%DTR, IOP_W%DTR_U, & ! D
         IOP_W%DTR_D, IOP_W%MVEC, IOP_W%DFI, IOP_W%Ref_dir, IOP_W%Ref_sol,& ! D
         IOP_W%Ex_U, IOP_W%Ex_D) ! D
    ! Adjoint RTE related arrays
    DEALLOCATE( ADO_W%Part_TU, ADO_W%Part_TD, ADO_W%Afi, ADO_W%Aj_ls, & ! D
         ADO_W%Bj_ls, ADO_W%Ajs, ADO_W%Bjs, ADO_W%LVEC, ADO_W%FLUX_D ) ! D

    IF( WF_FLAG ) THEN 
       ! Particulate matter weighting function (small particles)
       IF( OWF_Ks ) DEALLOCATE( Ph_Ks%Egg, ph_Ks%EgL, Ph_Ks%Egs, ph_Ks%EsL ) ! D
       ! Particulate matter weighting function (larger particles)
       IF( OWF_Kl ) DEALLOCATE( Ph_Kl%Egg, ph_Kl%EgL, Ph_Kl%Egs, ph_Kl%EsL ) ! D
       ! Particulate matter weighting function (user supplied)
       IF( OWF_Use ) DEALLOCATE(Ph_Us%Egg, ph_Us%EgL, Ph_Us%Egs, ph_Us%EsL) ! D
    ENDIF

  END SUBROUTINE DEALLOCATE_OCEAN
  !**********************************************************************

  SUBROUTINE SIMPL_OUTPUT(*)
    ! Call by "st_dom_tau.f"
    USE SPHER_PARS,  ONLY: NOA => N_OBS_ALT
    !------------------------------------------------------------------

    CHARACTER (LEN=*), PARAMETER :: R_NAME = 'SIMPL_OUTPUT'
    INTEGER  Ivs, J1, Jo, Jc, LL1, NN_Lo, Iou, K1, K2, NN_Ls, Nn_v
    REAL (KND)  Del, DUM, Cazm, Sazm
    REAL (KND), ALLOCATABLE :: Hv_OUT(:,:,:), Extr(:)
    REAL (KND), POINTER :: OUT_R(:,:,:), XM_LOC(:), Muv_Lo(:)
    !==================================================================
      
    CALL ST_TRACK (R_NAME, 1, 'I')
    IF( LOG_LEVEL(1) >= N_ESO ) WRITE(LUNIT,'(A)') '  *** '//R_NAME
    
    SELECT CASE(TRIM(Obs_POSITION(1)))
    CASE('Atmos')
       Nn_V   = NNv
       NN_Lo  = GSL_A%NN      ;  NN_ls = GSL_A%NLSA
       XM_LOC  => GSL_A%XM(:) ; Muv_Lo => GSL_A%MuV(:)
       IF( AGR_A%ADAPTIVE_GRID ) THEN
          ! In this case the following routine is used to properly 
          ! calculate intensity at user defined altitudes, i.e.,
          ! array DFI
          CALL ST_UP_DOWN_SFI(Mh, NN_Lo, NSZA, AGR_A%NL, IOP_A,  SSB_A, TE,&
               SDO_A, LAS_A%Ref_surf, UDB_A, LOA%Bet, Ph_zA, GSL_A, 'Atmos')
       ELSE
          CALL Field_Dir_Gau(GSL_A%NN, NSZA, AGR_A%NL, GSL_A%XM, IOP_A%DTR,&
               LAS_A%PART_D, IOP_A%MVEC, LAS_A%SOL_TOP, SDO_A, IOP_A%DFI)
       ENDIF
       K1 = 1 ; K2 = NNv ; Del = 1.d0
       IF( GeoD_Out%VIEW(IA)%S=='up' ) THEN
          K1 = K1 + NNv ; K2 = K2 + NNv 
          Del = -1.d0
       ENDIF
       OUT_R => IOP_A%DFI(:,:,K1:K2)
       ! WRITE(*,*) K1, K2
       ! WRITE(*,*) ' OUT_R  ', SUM(OUT_R)
       !-------------------------------------------
       !       write(*,*) '    from DFI "st_dom_tau" simple Mh = ', Mh
       !       J1 = 1
       !       DO K = NNv+Nvs*10 + 1, NNv+Nvs*10 + 4
       !          WRITE(*,'(2x,I2,2(2x,e15.7))')
       !   $           J1, IOP_A%DFI(NL,1,J1), IOP_A%DFI(NL,1,K)
       !          J1 = J1 + 1
       !       ENDDO
       !-------------------------------------------
         
    CASE('Ocean')
       Nn_V  = NNv_W
       NN_Lo = GSL_W%NN       ;  NN_ls = GSL_W%NLSA
       XM_LOC  => GSL_W%XM(:) ; Muv_Lo => GSL_W%MuV(:)
       IF( AGR_W%ADAPTIVE_GRID ) THEN
          ! In this case the following routine is used to properly 
          ! calculate intensity at user defined altitudes
          CALL ST_UP_DOWN_SFI(Mh, NN_Lo, NSZA, AGR_W%NL, IOP_W, SSB_W, TE,&
               SDO_W, LAS_W%Ref_surf, UDB_W, LOW%Bet, Ph_zW, GSL_W, 'Ocean')
       ELSE
          CALL Field_Dir_Gau(GSL_W%NN, NSZA, AGR_W%NL, GSL_W%XM, IOP_W%DTR,&
               LAS_W%PART_D, IOP_W%MVEC, LAS_W%SOL_TOP, SDO_W, IOP_W%DFI)
       ENDIF
       K1 = 1 ; K2 = NNv_W  ; Del = 1.d0
       IF( GeoD_Out%VIEW(IA)%S=='up' ) THEN
          K1 = K1 + NNv_W ; K2 = K2 + NNv_W ; Del = -1.d0
       ENDIF
       OUT_R => IOP_W%DFI(:,:,K1:K2)
       ! WRITE(*,*) K1, K2
       ! WRITE(*,*) ' OUT_R  ', SUM(OUT_R)
    CASE('Both')
       
    END SELECT
    
    SELECT CASE( Vec_Sca )
    CASE('scal_rte')  ;  DUM = E_W
    CASE('vect_rte')  ;  DUM = 0.5d0*E_W
    END SELECT
    
    ALLOCATE( Extr(NN_Lo),Hv_OUT(Kvs,Nvs,NN_Ls) )
    DO Jo = 1, N_OP
       Iou = NOA(Jo) ; J = 0
       ! Do-loop over solar angles
       DO Js = 1,NSZA
          DO Ivs = 1,Kvs
             K2 = (Ivs - 1)*NSZA + Js
             IF( LOS_DEF_FLAG == 'ga' ) THEN
                ! Output at all Gauss angles without interpolation 
                DO Jc = 1,Nvs
                   Hv_OUT(Ivs,Jc,:) = OUT_R(Iou,K2,Jc:Nn_v:Nvs)
                ENDDO
             ELSE
! Interpolate radiation field into viewing directions
! WRITE(*,*) ' Interpolation in "simpl_output.f" '
! WRITE(*,*) ' NN_Lo = ', NN_Lo
                DO Jc = 1,Nvs 
                   EXTR(:) = OUT_R(Iou,K2,Jc:Nn_v:Nvs)
                   ! WRITE(*,*) '  EXTR ', EXTR(:) 
                   ! CALL LIN_INT(NN_Lo, XM_LOC, Extr, &
                   !     NN_Ls, Muv_Lo, Hv_OUT(Ivs,Jc,:), 'L')
                   CALL AKIMA_INT(NN_Lo, XM_LOC, Extr, &
                        NN_Ls, Muv_Lo, Hv_OUT(Ivs,Jc,:))
                   ! WRITE(*,'(3x,2(1x,I2),2x,E12.4)')& 
                   !      Ivs, Jc, Hv_OUT(Ivs,Jc,1)
                ENDDO
             ENDIF
          ENDDO               ! over DO-solutions (Ivs)

          DO K = JV_S(Js),JV_E(Js) 
             DO L = 1,N_AZ_VAR  ;  J = J + 1  
                Cazm = DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                RADOUT(Jo,J,1) = DUM*Hv_OUT(1,1,K)*Cazm
                IF( Nvs==1 ) CYCLE
                Sazm = DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )
                RADOUT(Jo,J,2) = DUM*Hv_OUT(1,2,K)*Cazm
                ! U and V components of the Stokes vector
                ! should be taken with sign '-' for up-willing
                ! radiation (see Eq.(Int-up) in STB)
                RADOUT(Jo,J,3:4) = Del*DUM*Hv_OUT(1,3:4,K)*Sazm
                !  WRITE(*,'(2x,2(1x,I3),3(2x,E12.4))') &
                !      K, J, ANG_OUT(3,J), RADOUT(Jo,J,3:4)
                IF( Kvs==1 ) CYCLE

                RADOUT(Jo,J,1:2) = RADOUT(Jo,J,1:2) - DUM*Hv_OUT(2,1:2,K)*Sazm
                RADOUT(Jo,J,3:4) = RADOUT(Jo,J,3:4) + &
                     Del*DUM*Hv_OUT(2,3:4,K)*Cazm
             ENDDO
          ENDDO
       ENDDO
    ENDDO
    
    CALL ST_TRACK (R_NAME, 1, 'O')
    RETURN 1
    
  END SUBROUTINE SIMPL_OUTPUT
  !**********************************************************************

!c      SUBROUTINE WRITE_FLUXES
!c      USE GT_POLYN,    ONLY: XM, AG
!c
!c      INTEGER    Nst, Nfi
!c      REAL (KND) Fat_d, Fat_u, Fab_d, Fab_u, D_top, D_bot, Mu0, Sol_tr,
!c     $     Fre_RS(GSL_A%NSZA), F_tra, Ref_ind,
!c     $     XM_W(NN), AG_W(NN), AXM_W(NN), Fre_NN(NN)
!c
!c      IF( Mh > 1 ) RETURN
!c
!c      WRITE(LUNIT,*) 
!c      WRITE(LUNIT,*) '   "WRITE_FLUXES" in the "ST_DOM_TAU" '
!c      WRITE(LUNIT,*) 
!c      Mu0    = GSL_A%MuS(1)
!c      Sol_tr = SSB_A%EXS(AGR_A%NL,1)
!c      Nst = NNv+1
!c      Nfi = 2*NNv
!c      IF( UDB_A%SPECULAR ) THEN
!c         WRITE(LUNIT,'(20x,A)') 
!c     $        '   ***** Specular reflection ******* '
!c
!c         CALL Fre_REF(GSL_A%NSZA, Fre_RS, GSL_A%MuS, UDB_A%Ref_wd, 'aa')
!c
!c         WRITE(LUNIT,*) '   Fresnel reflection coeff (interface):'
!c         DO I = 1, GSL_A%NSZA
!c            WRITE(LUNIT,'(2x,I0,3(e12.4))') I, Fre_RS(I)
!c         ENDDO
!c
!cc        ... downward flux at the top
!c         Fat_d = Pie*Mu0
!c     $        + 2.*Pie*SUM( LAS_A%SOL_TOP(1:NNv:Nvs,1)*GSL_A%AXM(1:NN) )
!cc        ... upward flux at the top
!c         Fat_u = Pie*Sol_tr*Mu0*Fre_RS(1)*Sol_tr
!c     $       + 2.*Pie*SUM( LAS_A%SOL_TOP(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN))
!cc        ... downward flux at the bottom
!c         Fab_d = 2.*Pie*SUM( LAS_A%SOL_BOT(1:NNv:Nvs,1)*GSL_A%AXM(1:NN))
!c     $        + Pie*Sol_tr*Mu0
!cc        ... upward flux at the bottom
!c         Fab_u = Pie*Sol_tr*Mu0*Fre_RS(1)
!c     $       + 2.*Pie*SUM( LAS_A%SOL_BOT(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN))
!c
!c         Ref_ind = UDB_A%Ref_wd(1)
!c         XM_W(:) = DSQRT( XM(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind
!c         AG_W(:) = AG(:)*XM(:)/XM_W(:) !*Ref_ind**2
!c         AXM_W(:) = XM_W(:) *AG_W(:)
!c         CALL Fre_REF(NN, Fre_NN, XM, UDB_A%Ref_wd, 'aa')
!c         AXM_W(:) = AXM_W(:)*(1.d0 - Fre_NN(:))
!cc        ... transmitted flux in water
!c         F_tra = 2.*Pie*SUM( LAS_A%SOL_BOT(1:NNv:Nvs,1)*AXM_W(1:NN))
!c
!c         WRITE(*,'(18x,A)') 'fluxes at bottom'
!c         WRITE(*,*) '              down           up        trans     '
!c         WRITE(*,'(3(2x,e12.4),3x,e12.4)')
!c     $        Fab_d, Fab_u, F_tra, Fab_d - (Fab_u + F_tra)
!c         
!c      ELSE
!c         WRITE(LUNIT,'(20x,A)') 
!c     $        '   ***** Lambert or BRDF reflection ********'
!cc     ... downward flux at the top
!c         Fat_d = Pie*SSB_A%EXS(1,1)*Mu0
!c     $        + 2.*Pie*SUM( LAS_A%SOL_TOP(1:NNv:Nvs,1)*GSL_A%AXM(1:NN) )
!cc     ... upward flux at the top
!c         Fat_u = 2.*Pie*
!c     $        SUM( LAS_A%SOL_TOP(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN))
!c
!cc     ... downward flux at the bottom
!c         Fab_d = 2.*Pie*SUM( LAS_A%SOL_BOT(1:NNv:Nvs,1)*GSL_A%AXM(1:NN))
!c     $        + Pie*Sol_tr*Mu0
!cc     ... upward flux at the bottom
!c         Fab_u = 2.*Pie*
!c     $        SUM( LAS_A%SOL_BOT(Nst:Nfi:Nvs,1)*GSL_A%AXM(1:NN))
!c
!c      ENDIF
!c      
!c      WRITE(*,'(18x,A,26x,A)') 
!c     $     'fluxes at top','fluxes at bottom'
!c      WRITE(*,*) '              down           up             '//
!c     $     '          down             up '
!c      WRITE(*,'(7x,2(2x,E15.7),4x,2(2x,E15.7))') 
!c     $     Fat_d, Fat_u,  Fab_d, Fab_u
!c
!c      D_top = Fat_d - Fat_u ; D_bot = Fab_d - Fab_u
!c      WRITE(*,*) 
!c      WRITE(*,'(11x,A,5x,A,5x,A)') 
!c     $     'incoming at top','"outgoing" at bottom','diff [%]'
!c      WRITE(*,'(11x,E15.7, 7x,E15.7, 3x,E15.7)') 
!c     $     D_top,  D_bot, (D_top - D_bot)/D_top*100.
!c
!c         WRITE(*,'(20x,A)') 
!c     $        '   -----------------------------------------'
!cc      pause
!c      RETURN
!c      END SUBROUTINE WRITE_FLUXES

!*********************************************************************
END SUBROUTINE ST_DOM_TAU
!*********************************************************************
