C**********************************************************************
C
C       SCIATRAN - Version 4.5 - Last modification: 24 Jul 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c      SFI technique to calculate transmitted radiance at all level
c         for all solar angles in the direction of viewing angles
c                  Thermal emission is accounted for. 
c
c *********************************************************************
      SUBROUTINE ST_DOWN_SFI(NL, NLSA, GSL, IOP, SSB, TE, SDO, P0_LS, 
     $     N_PLS, Sf_a, Sf_b, Bet, Trans, Sfi_res, SPECULAR, NSZA, UDB)
c     ... Call in "post_processing.f"

      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nvs, NN_pla, LS_pla
      USE IRRAD,       ONLY: Fbe
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_DSO, DIR_OUT
      USE NODES,       ONLY: MJU_MINUS
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: Pie, F14, VST, Finc, Kvs, D_sol, RAD_FIELD
      USE SPHER_PARS,  ONLY: JV_S, JS_E, Nf => NN
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: TE_TYPE, SDO_TYPE, SSB_TYPE, GSL_TYPE,
     $                       IOP_TYPE, UDB_TYPE
c     ==================================================================

      INTEGER,    INTENT(IN) :: NL, N_PLS, NLSA, NSZA
      LOGICAL,    INTENT(IN) :: SPECULAR
c     ----------------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (IOP_type), TARGET :: IOP    ! Integral optical parameters
      TYPE (SSB_type) :: SSB    !  Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (UDB_TYPE) :: UDB   ! Boundary conditions
c     -------------------------------------------------------------
      REAL (KND)  P0_LS(NL,Nvs*NLSA,Nvs*NSZA,N_PLS), 
     $     Sf_a(NL-1,Nvs*NLSA,GSL%NN*Nvs), 
     $     Sf_b(NL-1,Nvs*NLSA,GSL%NN*Nvs),  
     $     Sfi_res(NL,Kvs*NSZA,Nvs*NLSA), 
     $     Trans(Kvs*NSZA,Nvs*NLSA)
      REAL (KND), DIMENSION(NL) :: Bet
c     Local variables 
c     ---------------
      CHARACTER (LEN=11), PARAMETER :: R_NAME = 'ST_DOWN_SFI'
      INTEGER  I, K1, K2, Ko, Ks, Kv, Js, Jv, Jc, Zn, Km, Jco, Jro, 
     $     Ivs, Jvs, NN, NNv, LL1, Lr1, Lk
      REAL (KND)  CMd, A_COF, B_COF, P_COF, C_1, DTA, Ck, H_1,
     $     C_DP, C_UM, SOU_p, CMd_p, Wdo, Cs, Cv, Ecv, SMd, S1,
     $     Cks, Tls(Kvs*NSZA,Nvs*NLSA), E_NLSA(NLSA), 
     $     SOU(NSZA), Itj(NL-1,Kvs*NSZA,Nvs*NLSA), Muv(NLSA),
     $     S_L(Nvs,Nvs)
      REAL (KND), DIMENSION (GSL%NN*Nvs) :: Apa, Bpa, Tt1, Tt2,
     $     Sjd, Cjd
      INTEGER, POINTER :: Id_a(:), Id_b(:)
      INTEGER, ALLOCATABLE :: Jh1(:), Jh2(:)
      REAL (KND), ALLOCATABLE :: T_AU(:), HPL(:,:,:)
      REAL (KND), POINTER :: MVEC(:,:), Nuj(:), Enu(:), 
     $     EXP_D(:), EXP_U(:), EXS(:,:)
      LOGICAL  PURE(GSL%NN*Nvs)
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2, Cnu, Cmu
c      REAL (KND), TARGET :: Hnu(GSL%NN*Nvs)
c    ===================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NN = GSL%NN  ;  NNv = NN*Nvs 
      MVEC => IOP%MVEC(:,:)
      Id_a => IOP%Id_a(:) ;  Id_b => IOP%Id_b(:)

c      WRITE(*,*) '  down_sfi : SHAPE( MuS ) ', SHAPE( GSL%MuS )

      IF( RAD_FIELD ) THEN
         Muv(:) = DABS( MJU_MINUS(:) )
         ALLOCATE( Jh1, SOURCE=[(1,I=1,NLSA)] )
         ALLOCATE( Jh2, SOURCE=[(NSZA,I=1,NLSA)] )
      ELSE
         Muv(:) = GSL%Muv(:)
         ALLOCATE( Jh1, SOURCE=Jv_S )
         ALLOCATE( Jh2, SOURCE=Js_E )
      ENDIF

      Itj = 0.d0  ;  Wdo = 1.d0 - Wup
      EXS => SSB%EXS
      
      IF( SPECULAR ) THEN
         ALLOCATE( T_AU(NL) ) ; T_AU(1) = 0.d0
         DO I = 2,NL
            T_AU(I) = SUM( IOP%DTR(1:I) )
         ENDDO
      ENDIF

      IF( TE%ADD_PLANCK ) THEN
         ALLOCATE( HPL(NL_pla-1,NN_pla,SIZE(MuV)) ) 
         DO I = 1,NL - 1
            DTA = IOP%DTR(I+1)  ;  SOU_p = DEXP( TE%Mup(I)*DTA )  
            CS = -TE%Mup(I)
            DO Jv = 1,NLSA
               CV = MuV(Jv) ; Ecv = DEXP( -DTA/CV )
               IF( REAL(CS*CV) /= REAL(1.d0) ) THEN
                  CMd_p = CV*(SOU_p - Ecv)/(1.d0 - CS*CV)
               ELSE
                  CMd_p = Ecv*DTA
               ENDIF
               Zn = -1
               DO Jc = 1,TE%N_pla(I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Cks = Ck*Cs
                  Muc = Ck/DCMP( Cks,-1.d0 )
                  Cmd_p = CMd_p + AIMAG( Muc*Cv/(Muc - Cv) )
     $                 *TE%PL_COEF(I,Jc)*(Zn*SOU_p - Ecv)
                  Zn = -Zn
               ENDDO
               HPL(I,:,Jv) = TE%Aj_pla(:,I)*CMd_p*CV
c              ... First Stokes component only
               IF( .NOT.DO_SSRCA ) Itj(I,1:NSZA,(Jv-1)*Nvs+1)
     $              = Cmd_p*TE%NPLANCK(I)/CV*VST
            ENDDO
         ENDDO
      ENDIF

      DO I = 1,NL - 1
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco
c        ... Set PURE(E_MIN(I)) to .false. if in the layer pure scattering
         PURE(:) = .TRUE. 
         IF( SDO%MASK_NL(I) ) PURE(SDO%E_MIN(I)) = .FALSE.
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1 - 1
         Nuj   => SDO%Nuj_lay(:,I) 
         EXP_D => IOP%Ex_D(:,I)
         EXP_U => IOP%Ex_U(:,I)

         DTA = IOP%DTR(I+1)
         Enu => SDO%EXP_NU_NL(:,I)   
         SOU(:) = DEXP( -DTA/SSB%Mue(:,I) )

         DO Jv = 1,NLSA
            Kv = (Jv-1)*Nvs
c            WRITE(*,*) ' Start Jv: ', Jv
            CV = MuV(Jv) ; ECV = DEXP( -DTA/CV )            
            Sjd(1:Jro) = Nuj(1:Jro)*CV/(Nuj(1:Jro) + CV)
     $           *( 1.0d0 - Enu(1:Jro)*ECV )*Exp_D(1:Jro)
            WHERE( REAL(Nuj(1:Jro)) /= REAL(CV) )
               Cjd(1:Jro) = Nuj(1:Jro)*CV/(Nuj(1:Jro) - CV)
     $              *(Enu(1:Jro) - ECV)*Exp_U(1:Jro)
            ELSEWHERE
               Cjd(1:Jro) = Enu(1:Jro)*DTA*Exp_U(1:Jro)
            END WHERE            
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nuj(K1),Nuj(K1+1) )
               Cnu = DCMP( Enu(K1),Enu(K1+1) )

               C2 = C1*CV*(Cnu - ECV)/(C1 - CV)
     $              *DCMP(EXP_U(K1),EXP_U(K1+1))
               Cjd(K1) = DBLE(C2) ; Cjd(K1+1) = AIMAG(C2)
               
               C2 = C1*CV*(1.d0 - Cnu*ECV)/(C1 + CV)
     $              *DCMP(EXP_D(K1),EXP_D(K1+1))
               Sjd(K1) = DBLE(C2) ; Sjd(K1+1) = AIMAG(C2)
            ENDDO

            DO Js = Jh1(Jv),Jh2(Jv)
               Ks = (Js-1)*Nvs
               CS = SSB%Mue(Js,I) 
               IF( REAL(CS) /= REAL(CV) ) THEN
                  CMd = CS*CV*(SOU(Js) - Ecv)/(CS - CV)
               ELSE
                  CMd = Ecv*DTA
               ENDIF
               Zn = -1
               DO Jc = 1,SSB%N_sph(Js,I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-Cs )
                  CMd = CMd + AIMAG( Muc*Cv/(Muc-Cv) )
     $                 *SSB%SIN_COEF(Js,I,Jc)*(Zn*SOU(Js) - Ecv)
                  Zn = -Zn
               ENDDO

c       >>>>>> Calculation of the integral over layer from the 
c              tau-depending part of the particular solution

c               WRITE(*,*) Id_a(I)-1, SDO%Ajp(1,Id_a(I)-1,:,Js)
c               WRITE(*,*) '    -----------------    '
               DO Ivs = 1,Kvs
                  Jvs = (Ivs - 1)*NSZA + Js
                  Apa(:) = ( SDO%Ajp(Ivs,I,:,Js) 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 - SDO%Ajp(Ivs,Id_a(I)-1,:,Js)*ECV 
     $                 - SDO%Aj_sol(Ivs,:,I,Js)*CMd*EXS(I,Js) )*CV 

                  Bpa(:) = ( -SDO%Bjp(Ivs,I,:,Js)*ECV 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 + SDO%Bjp(Ivs,Id_b(I)+1,:,Js)
     $                 + SDO%Bj_sol(Ivs,:,I,Js)*CMd*EXS(I,Js) )*CV 

c     ************* Specular reflection start *************************
                  IF( SPECULAR ) THEN
                     Smd = Cs*Cv/(Cs + Cv)*(1.d0 - SOU(Js)*ECV)
                     S1 = Smd*EXS(NL,Js)
     $                    *DEXP(-(T_AU(NL)-(T_AU(I)+DTA))/Cs )
cc                 ... We use here that in the scalar case 
cc                     aj(specular)=bj(dir) and bj(specular)=aj(dir)
c                     Apa(:) = Apa(:) - SDO%Bj_sol(Ivs,:,I,Js)*S1*Cv
c                     Bpa(:) = Bpa(:) + SDO%Aj_sol(Ivs,:,I,Js)*S1*Cv 
c                  ... In the vector case aj_spe and bj_spe contains
c                      Fresnel coefficient (see homog_partic.f)
                     Apa(:) = Apa(:) - SDO%Aj_spe(Ivs,:,I,Js)*S1*CV 
                     Bpa(:) = Bpa(:) + SDO%Bj_spe(Ivs,:,I,Js)*S1*CV 

                     S_L(:,:) = Fbe*F14*SMd*EXS(NL,Js)
     $                    *DEXP(-(T_AU(NL)-(T_AU(I)+DTA))/Cs )/Cv
     $                    *( Wup*P0_LS(I,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,2)
     $                    + Wdo*P0_LS(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,2) )
     $                    /IOP%ALW(I)
                     K2 = 1
                     DO K1 = Kv+1,Kv+Nvs
                        Itj(I,Jvs,K1) = Itj(I,Jvs,K1) 
c     $                       + SUM( Finc(Ivs,:)*S_L(:,K2) )
     $                       + SUM( UDB%Fref(Js,Ivs,:)*S_L(:,K2) )
                        K2 = K2 + 1
                     ENDDO
                  ENDIF   ! specular reflection
c     ************* Specular reflection end ***************************
                  
                  IF( TE%ADD_PLANCK.AND.Ivs==1 ) THEN
                     Apa(:) = Apa(:) - HPL(I,:,Jv)
                     Bpa(:) = Bpa(:) + HPL(I,:,Jv)
                  ENDIF

c              --> a) real separation constants
                  WHERE( PURE(1:Jro) )               
                     Tt1(1:Jro) = MVEC(LL1:Lk+Jro,Jvs)*Cjd(1:Jro) 
     $                    + Apa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) - Cv)
                     Tt2(1:Jro) = MVEC(LR1+1:LR1+Jro,Jvs)*Sjd(1:Jro)
     $                    + Bpa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) + Cv)
                  ELSEWHERE
                     Tt1(1:Jro) = 0.d0 ; Tt2(1:Jro) = 0.d0
                  END WHERE            
c              --> b) imaginary separation constants    
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     Cnu = DCMP( Nuj(K1),Nuj(K1+1) )
                     C1 = DCMP(Apa(K1),Apa(K1+1))*Cnu/(Cnu-Cv)

                     Tt1(K1) = MVEC(Lk+K1,Jvs)*Cjd(K1) + 
     $                    MVEC(Lk+K1+1,Jvs)*Cjd(K1+1) + 2.0d0*DBLE(C1)
                     Tt1(K1+1) = MVEC(Lk+K1+1,Jvs)*Cjd(K1) - 
     $                    MVEC(Lk+K1,Jvs)*Cjd(K1+1) - 2.0d0*AIMAG(C1)

                     C2 = DCMP(Bpa(K1),Bpa(K1+1))*Cnu/(Cnu+Cv)

                     Tt2(K1) = MVEC(LR1+K1,Jvs)*Sjd(K1) +
     $                    MVEC(LR1+K1+1,Jvs)*Sjd(K1+1) + 2.d0*DBLE(C2)
                     Tt2(K1+1) = MVEC(LR1+K1+1,Jvs)*Sjd(K1) - 
     $                    MVEC(LR1+K1,Jvs)*Sjd(K1+1) - 2.d0*AIMAG(C2)
                  ENDDO
c       <<<<<< End of calculation of tau-depending part of particular solution
c           
c       >>>>>> Integral over tau from single and multiple scattering source 
c              functions in the I-th atmospheric layer
c                  (see Eq.(Del-Q-tra) in STB)
                  S_L(:,:) = Fbe*F14*Cmd*EXS(I,Js)/CV/IOP%ALW(I)
     $                 *( Wup*P0_LS(I,  Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1)
     $                  + Wdo*P0_LS(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) )
c                  IF( Jv < 4 ) THEN
c                     WRITE(*,'(2x,2(1x,I3),*(2x,e12.4))')
c     $                    I, Jv, EXS(I,Js), S_L(1,1)
c                  ENDIF
                  K2 = 1
                  DO K1 = Kv+1,Kv+Nvs
                     Itj(I,Jvs,K1) = Itj(I,Jvs,K1) 
     $                    + SUM(Finc(Ivs,:)*S_L(:,K2)) 
     $                    + SUM( Tt1(:)*Sf_a(I,K1,:) 
     $                         + Tt2(:)*Sf_b(I,K1,:) )
c     $                    + SUM( Tt1(:)*Sf_a(I,K1,:) ) 
c     $                    + SUM( Tt2(:)*Sf_b(I,K1,:) ) 
                     K2 = K2 + 1
                  ENDDO         ! over Stokes components (K1,K2)
c       <<<<<< 
               ENDDO  ! over DO-solutions (Ivs)->(Jvs)
            ENDDO   ! over solar angles (Js)
         ENDDO  ! over l-of-s angles (Jv)
         IF( ANY(.NOT.PURE(:)) ) CALL PURE_SCAT_DN
      END DO ! over layers (I)

c      WRITE(*,*) '   NL = ', NL
c      DO Jv = 1,NLSA
c         WRITE(*,'(2x,I3,2x,F8.3)')  Jv, DACOS(MuV(Jv))/Pie*180.
c      ENDDO
c      WRITE(*,*) ' SHAPE(Sfi_res) :',SHAPE(Sfi_res)
c     Contribution from all atmospheric layers:
c      Tls(:,:) = 0.d0  
      Tls(:,:) = 2.d0*Pie*Trans(:,:)
      Sfi_res(1,:,:) = Tls(:,:)
      DO I = 1,NL-1
         E_NLSA(:) = DEXP(-IOP%DTR(I+1)/MuV(:))
         DO Jv = 1,NLSA
            DO K1 = (Jv-1)*Nvs+1,Jv*Nvs
               Tls(:,K1) = Tls(:,K1)*E_NLSA(Jv) + Itj(I,:,K1)
            ENDDO
         ENDDO
         Sfi_res(I+1,:,:) = Tls(:,:) 
      ENDDO
c      WRITE(*,*) '    Sfi_res(NL,1,J ) and  '
c      WRITE(*,*) '    Sfi_res(NL-1,1,J ) in "in st_down_sfi" '
c      IF( Vec_Sca == 'vect_rte' ) THEN
c         Js = 1
c         WRITE(*,*) '   ****** Test top upward from ST_DOWN_SFI :'
c         WRITE(*,*) '   Mharm = ', Mh-1
c         WRITE(*,*) '    NLSA = ', NLSA
c         Open(12,file=DIR_OUT//'/Tra_dir.dat',RECL=15000)
c         DO I = 1, NL
c            WRITE(12,'(130(1x,e15.7))')
c     $           ZZZ(I), ( Sfi_res(I,Js,J ), J=1,Nvs*NLSA)
c         ENDDO
c         Close(12)
c      ENDIF
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************
      
      SUBROUTINE PURE_SCAT_DN
      H_1 = 3.d0 - (Wup*Bet(I) + Wdo*Bet(I+1))/IOP%ALW(I)
      Km = SDO%E_MIN(I)
      DO Jv = 1,NLSA
         CV = MuV(JV) ; ECV = DEXP( -DTA/CV ) 
         C_1 = 3.d0*CV/H_1
         C_DP = C_1 + IOP%DTR_D(I) ; C_UM = C_1 - IOP%DTR_U(I)
         A_COF = -DTA*ECV + C_DP*(1.d0 - ECV)
         B_COF =  DTA     - C_UM*(1.d0 - ECV)
         DO Js = Jh1(Jv),Jh2(Jv)
            CS = SSB%Mue(Js,I) 
            IF( REAL(CS) == REAL(CV) ) THEN
               CMd = 0.d0
            ELSE
               CMd = CS*CV*(SOU(Js) - ECV)/(CS - CV)
            ENDIF
            Zn = -1
            DO Jc = 1,SSB%N_sph(Js,I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-Cs )
               CMd = CMd + AIMAG( Muc*Cv/(Muc-Cv) )
     $              *SSB%SIN_COEF(Js,I,Jc)*(Zn*SOU(Js) - Ecv)
               Zn = -Zn
            ENDDO

            K1 = (Jv-1)*Nvs+1
            DO Ivs = 1,Kvs
               IF(.NOT.D_sol(Ivs)) CYCLE
c               WRITE(*,*) ' Add PURE sfi  '
               Jvs = (Ivs - 1)*NSZA + Js
c                  ... Ajp(:,Id_a(I)-1,:,:)=0 if "ADAPTIVE_GRID"=.false.  
               P_cof = -( SDO%Ajp(Ivs,Id_a(I)-1,Km,Js)*(C_DP+DTA) 
     $              - SDO%Bjp(Ivs,I,Km,Js)*C_UM )*ECV 
     $              + SDO%Ajp(Ivs,I,Km,Js)*C_DP 
c                  ... Bjp(:,Id_b(I)-1,:,:)=0 if "ADAPTIVE_GRID"=.false. 
     $              + SDO%Bjp(Ivs,Id_b(I)+1,Km,Js)*(DTA-C_UM)
     $              - 0.25d0*( 3.d0
     $              * (GSL%Mus(Js)+CV) - GSL%Mus(Js)*H_1 )
     $              * CMd*EXS(I,Js) *Vst ! test vector case Pochemu ??
c              ... The first Stokes component only
               Itj(I,Jvs,K1) = Itj(I,Jvs,K1) + P_COF
     $              + MVEC(Lk+Km,Jvs)*A_COF + MVEC(LR1+Km,Jvs)*B_COF
            ENDDO               ! over DO-solutions (Ivs)->(Jvs)
         ENDDO                  ! over solar angles (Js)
      ENDDO                     ! over zenith angles (Jv)
      END SUBROUTINE PURE_SCAT_DN
c **********************************************************************

      END 

