C ************************************************************************
C
C       SCIATRAN - Version 3.5 - Last modification: 12 Jul 2015
C     
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C ************************************************************************

C       This routine is used for the calculation of the surface
C             elevation WF for a plane-parallel geometry
C     ---------------------------------------------------------
      SUBROUTINE ST_ELEVATION_WF(NSZA, NN, JPOS, Ref_sol, Ref_dir, EXS,
     $     EXL, Muv, Mharm, SCR, SCA, AL, Pgs, Pgg, PgL, PsL, 
     $     E_W, DFI, AFI)
c     ... Call in "st_dom_tau.f"

      USE GT_POLYN,    ONLY: XM, AXM, AG, AG1
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: Nvs 
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, 
     $                       N_LOS_OUT, NLSA => N_LOS_ANG 
      USE ST_VARS,     ONLY: F14, Nc_der, Finc, Kvs, N_Adj
      USE TOOLS,       ONLY: FIND_IDX
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      INTEGER     NSZA, NN, Mharm, JPOS
      REAL (KND)  E_W, SCR, SCA, AL
      REAL (KND)  EXS(NSZA), EXL(NLSA), Muv(NLSA), 
     $     Ref_sol(N_Adj,Nvs*NSZA), Ref_dir(Kvs*NSZA,Nvs*NLSA), 
     $     DFI(Kvs*NSZA,2*NN*Nvs), AFI(2*NN*Nvs,N_Adj), 
     $     Pgs(Nvs*NN,NSZA,2), Pgg(Nvs*NN,Nvs*NN,2), 
     $     PsL(Nvs*NLSA,Nvs*NSZA), PgL(Nvs*NN,Nvs*NLSA,2)
c     ... Local parameters
      CHARACTER (LEN=15) R_NAME
      PARAMETER  (R_NAME = 'ST_ELEVATION_WF')
      INTEGER  Js, Jv, Ja, Jvf, Jva, K, K1, L, NNv, II
      REAL (KND)  Ext_tes, WF_ele(N_LOS_OUT), Comp(N_LOS_OUT), 
     $     G_Gp(NN,NN), G_Gm(NN,NN), L_Gp(NN,NLSA), 
     $     L_Gm(NN,NLSA), S_Gp(NN), S_Gm(NN), S_L(NLSA)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = NN*Nvs
c      WRITE(*,*) ' In st_elevation_wf'
      K1 = 0 ; Ext_tes = 0.d0
      DO Js = 1, NSZA
         DO Jv = JV_S(Js),JV_E(Js)
            Jvf = (Jv-1)*Nvs + Nc_der(1)
            DO K = 1, Nvs
               Ext_tes = Ext_tes + SUM( 
     $              (DFI(Js,K:NNv:Nvs)*AFI(K+NNv:2*NNv:Nvs,Jv)
     $              +  DFI(Js,K+NNv:2*NNv:Nvs)*AFI(K:NNv:Nvs,Jv))
     $              *AG(1:NN) )
            ENDDO
            Ext_tes = Ext_tes + Ref_dir(Js,Jvf)*EXL(Jv)/Muv(Jv)
     $           + SUM( Ref_sol(Jv,(Js-1)*Nvs+1:Js*Nvs)*Finc(1,:) ) 
     $           *EXS(Js)/2.d0

            DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
               WF_ele(K1) = -Ext_tes*DCOS(ANG_OUT(3,K1)*DBLE(Mharm-1))
            ENDDO
         ENDDO                  ! do loop over line-of-sight angles
      ENDDO                     ! do loop over solar zenith angles
c      WRITE(*,*) '  In st_elevation_wf - 1'

      G_Gp(:,:) = Pgg(:,:,1)
      G_Gm(:,:) = Pgg(:,:,2)
      L_Gp(:,:) = PgL(:,:,1)/2.d0 
      L_Gm(:,:) = PgL(:,:,2)/2.d0 

      K1 = 0
c      WRITE(*,*) '   In st_elevation_wf - 2'
      DO JS = 1,NSZA ; L = JS
         S_Gp(:) = F14*XM(:)*AG(:)*Pgs(:,L,1)
         S_Gm(:) = F14*XM(:)*AG(:)*Pgs(:,L,2)
         DO Jv = JV_S(L),JV_E(L)
            S_L(JV) = F14*PsL(Jv,L)/MuV(JV) 
         ENDDO
c         WRITE(*,*) '    In st_elevation_wf - 3'
         CALL ELEV_WF_COMP 
      ENDDO

      II = FIND_IDX('elev', WF_DB%NAME)
      WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1) 
     $     - (WF_ele(:)*AL + Comp(:))*E_W

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************
      SUBROUTINE ELEV_WF_COMP 
      REAL (KND)   R_T(2*NN), Tet(NLSA)
      REAL (KND), DIMENSION (NN) :: QP_T, QM_T

      DO JV = JV_S(JS),JV_E(JS)
c        ... contribution from the term containing Js(tau,Mu_v)
         Tet(Jv) = S_L(Jv)*EXL(Jv)*EXS(Js)

c           ... Contribution from the term < Js,I*d >
         Tet(Jv) = Tet(Jv) + SUM( Afi(NN+1:2*NN,Jv)*S_gp(:)
     $        +  Afi(1:NN,Jv)*S_gm(:) )*EXS(Js)

c           ... Contribution from the term < Jd,I*d >
         R_T = 0.0d0
         DO K = 1,NN
            Qp_T(:) = G_Gp(K,:)*XM(K)*AG1(:)
            Qm_T(:) = G_Gm(K,:)*XM(K)*AG1(:)
            R_T(1:NN) = R_T(1:NN) + 
     $           (Qm_T*Afi(K,Jv) + Qp_T*Afi(K+NN,Jv))*AG(K)
            R_T(NN+1:2*NN) = R_T(NN+1:2*NN) + 
     $           (Qm_T*Afi(K+NN,Jv) + Qp_T*Afi(K,Jv) )*AG(K)
         ENDDO
         Tet(Jv) = Tet(Jv) + SUM( R_T(:)*DFI(Js,:) )

c        ... contribution from the term containing Jd(tau,Mu_v)
         R_T(1:NN)      = L_Gm(:,JV)*AXM(1:NN) 
         R_T(1+NN:2*NN) = L_Gp(:,JV)*AXM(1:NN) 
         Tet(Jv) = Tet(Jv) + SUM(DFI(JS,:)*R_T(:))*EXL(JV)/MuV(JV) 
         DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
            Comp(K1) = Tet(Jv)*DCOS(ANG_OUT(3,K1)*DBLE(Mharm-1))
         ENDDO
      ENDDO
      
      RETURN
      END SUBROUTINE ELEV_WF_COMP

      END
