C********************************************************************C
C
C       SCIATRAN - Version 3.1 - Last modification: 27 Oct 2010
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE IDX_ESFT(JPOS, E_IDX, NESFT, NESFT_IDX, *)
c     ... Call in "dom_spherical.f"
c                 "spher_rad.f"

      USE ABSORBERS, ONLY: N_ESFT_MOL
      USE LOG_PARS,  ONLY: DO_LOGFILE, LUNIT, LOG_LEVEL
      USE ESFT,      ONLY: ESFT_OVERLAP_MODE, DO_ESFT_ANTI_CORRELATION, 
     $                     DO_ESFT_SPEC_MOL, N_ESFT_COEFF, ESFT_NCOEFF,
     $                     ESFT_MOL_OVERLAP_IDX1, ESFT_MOL_OVERLAP_IDX2
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'IDX_ESFT'
      INTEGER  JPOS, NESFT, NESFT_IDX, E_IDX(2*N_ESFT_COEFF,N_ESFT_MOL)
c     ... Local variables
      INTEGER  I, J, N1, N2, F_J, S_J, HIA(2*N_ESFT_COEFF)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     Determine ESFT molecule index of ESFT absorber
c     with highest number of coefficients 
c     (might be 1; might even be no ESFT absorber in AMF mode)
      NESFT = 1
      NESFT_IDX = 0
      DO J = 1, N_ESFT_MOL
         IF(DO_ESFT_SPEC_MOL(JPOS,J).AND.
     $        ESFT_NCOEFF(JPOS,J).GT.NESFT) THEN
            NESFT = ESFT_NCOEFF(JPOS,J)
            NESFT_IDX = J
         ENDIF
      ENDDO

c     For I+corr and I-corr calculations
      IF( ESFT_OVERLAP_MODE(JPOS) .EQ. 2) THEN
         NESFT = 2*NESFT
      ENDIF

      IF( DO_ESFT_ANTI_CORRELATION.AND.
     &     (ESFT_OVERLAP_MODE(JPOS).NE.2).AND.DO_LOGFILE ) THEN
         WRITE(LUNIT,*) 
     &        " *** WARNING: ESFT IN ANTI-CORRELATION MODE !"
      ENDIF
 
      IF( ESFT_OVERLAP_MODE(JPOS) .EQ. 2 ) THEN
         FORALL ( I = 1:NESFT/2 )       HIA(I) = I
         FORALL ( I = NESFT/2+1:NESFT ) HIA(I) = NESFT + 1 - I
      ELSE
         FORALL ( I = 1:NESFT ) HIA(I) = I
      ENDIF

      IF( .NOT. DO_ESFT_ANTI_CORRELATION ) THEN          ! TRUE DEFAULT MODE:
         DO J = 1, N_ESFT_MOL
            IF(DO_ESFT_SPEC_MOL(JPOS,J)) THEN
               IF( ESFT_OVERLAP_MODE(JPOS) .NE. 2 ) THEN ! SIMPLE OVERLAP
                  IF( ESFT_NCOEFF(JPOS,J) .EQ. 1 ) THEN
                     E_IDX( 1:NESFT,J ) = 1
                  ELSE
                     E_IDX( 1:NESFT,J ) = HIA(1:NESFT)
                  ENDIF
               ELSE                                      ! ALPHA OVERLAP MODE
                  IF( ESFT_NCOEFF(JPOS,J) .EQ. 1 ) THEN
                     E_IDX( 1:NESFT,J ) = 1
                  ENDIF
                  IF( J .EQ. ESFT_MOL_OVERLAP_IDX1(JPOS) ) THEN
                     E_IDX( 1:NESFT/2,J )       = HIA(1:NESFT/2)
                     E_IDX( NESFT/2+1:NESFT,J ) = HIA(1:NESFT/2)
                  ENDIF 
                  IF (J .EQ. ESFT_MOL_OVERLAP_IDX2(JPOS)) THEN
                     E_IDX( 1:NESFT,J ) = HIA(1:NESFT)
                  ENDIF
               END IF
            ENDIF
         ENDDO 
      END IF

C     ! ANTI-CORRELATION MODE (FOR ALPHA CALCULATIONS):
C     ====================================================
c     * assumptions: only 2 ESFT absorber present
c     (with ncoeff > 1)
c     * smallest coeff. of gas 1 used along with 
c     largest coeff. of gas 2 etc., i.e.
c     assuming perfect anti-correlation of cross-sect.
      IF( DO_ESFT_ANTI_CORRELATION ) THEN 
         F_J = 0 ; S_J = 0
         DO J = 1, N_ESFT_MOL
            IF(DO_ESFT_SPEC_MOL(JPOS,J)) THEN
               IF( ESFT_NCOEFF( JPOS,J ) .EQ.1 ) THEN
                  E_IDX( 1:NESFT,J ) = 1                  
               ENDIF
               IF( ESFT_NCOEFF( JPOS,J ) > 1 .AND. F_J .EQ. 0 ) THEN
                  E_IDX( 1:NESFT,J ) = HIA(1:NESFT)
                  F_J = J ; N1 = ESFT_NCOEFF(JPOS,J)
               ENDIF
               IF( ESFT_NCOEFF(JPOS,J) > 1 .AND. F_J.NE.J ) THEN
                  E_IDX( 1:NESFT,J ) = HIA(NESFT:1:-1)
                  S_J = S_J + 1 ; N2 = ESFT_NCOEFF(JPOS,J)
               ENDIF
            ENDIF
         ENDDO 
         IF( S_J > 1 )    GO TO 801
         IF( N1 .NE. N2 ) GO TO 802
      END IF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 801  CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     &     'TOO MANY ESFT ABSORBER IN ANTI-CORR MODE')
      RETURN 1
 802  CALL WRITE_MESSAGE(R_NAME, 'ERROR',
     &     'NUMBER OF ESFT COEFF. NOT EQUAL IN ANTI-CORR MODE')
      RETURN 1
      END 

c *************************************************************************
      SUBROUTINE WEIGHT_ESFT(JPOS, JJESFT, JESFT, NESFT, NESFT_IDX,
     $     ESFT_WEIGHT)
c     ... Call in "dom_spherical.f"
c                 "spher_rad.f"

      USE ESFT,      ONLY: ESFT_OVERLAP_MODE, ESFT_MOL_OVERLAP_IDX1,
     $                     ESFT_WEIGHTS, ESFT_ALPHA
      USE LOG_PARS,  ONLY: LUNIT, LOG_LEVEL
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------

      INTEGER    JPOS, JJESFT, JESFT, NESFT, NESFT_IDX
      REAL (KND) ESFT_WEIGHT
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'WEIGHT_ESFT'
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( ESFT_OVERLAP_MODE(JPOS) .EQ. 2 ) THEN
         IF( JJESFT > NESFT/2 ) THEN
C           ALPHA ANTI-CORRELATION PART (I-CORR)
            JESFT = JJESFT - NESFT/2
            ESFT_WEIGHT = ESFT_WEIGHTS(JPOS,JESFT,
     &           ESFT_MOL_OVERLAP_IDX1(JPOS))*(1.0D0-ESFT_ALPHA(JPOS))
         ELSE
C           ALPHA CORRELATION PART (I+CORR)
            JESFT = JJESFT
            ESFT_WEIGHT = ESFT_WEIGHTS(JPOS,JESFT,
     &           ESFT_MOL_OVERLAP_IDX1(JPOS))*ESFT_ALPHA(JPOS)
         ENDIF
      ELSE
         JESFT = JJESFT
         IF(NESFT > 1 ) THEN
            ESFT_WEIGHT = ESFT_WEIGHTS(JPOS,JESFT,NESFT_IDX)
         ELSE
            ESFT_WEIGHT = 1.0D0
         ENDIF
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END 
c *************************************************************************
