********************************************************************
C
C       SCIATRAN - Version 4.6 - Last modification: 07 Mar 2023
C
C           Flux, spherical albedo, diffuse transmission
C
C********************************************************************
      SUBROUTINE ST_FLUX_SPHER_ALB(JPOS, JJESFT, E_W, NL, AL_TES, GSL, 
     $     EXS, DFI, Obs_POS, UDB)
c     ... Call in 'st_dom_tau.f'

      USE GRIDS,       ONLY: NLAYERS
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: Fbe
      USE RTM_OUTPUT,  ONLY: DIF_FLUX_UP, FLUX_UP, ACTFLUX, N_OP,
     $                       DIF_FLUX_DN, FLUX_DN, DIR_RAD, DIF_SCAL_UP,
     $                       DIR_FLUX
      USE ST_VARS,     ONLY: PIE, RTM_MODE, VST, Kvs, DIFB
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: GSL_TYPE, UDB_TYPE
      USE WATER,       ONLY: UWT, NLAYERS_UW, Sol_off
c      USE WATER,       ONLY: Ref_ind, UWT, NLAYERS_UW, Sol_off
c     ------------------------------------------------------------------

      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     ----------------------
      INTEGER,           INTENT(IN) :: JPOS, JJESFT, NL
      REAL (KND),        INTENT(IN) :: E_W, EXS(NL,GSL%NSZA), 
     $     DFI(NL,Kvs*GSL%NSZA,2*Nvs*GSL%NN), AL_TES(NL)
      CHARACTER (LEN=5), INTENT(IN) :: Obs_POS
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_FLUX_SPHER_ALB'
      INTEGER     I, K, Js, NNv, NSZA, NN, N1, N2, Np
      REAL (KND)  S1, S2, Ref_ind
      REAL (KND), POINTER :: AXM(:), AG(:), MuS(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NSZA = GSL%NSZA  ;  NN = GSL%NN      
      AXM => GSL%AXM   ;  AG => GSL%AG  ;  MuS => GSL%MuS
      SELECT CASE( TRIM(Obs_POS) ) ! Preliminary ??
      CASE('Atmos')
         N1 = 1 ; N2 = NLAYERS ; Np = 0
      CASE('Ocean')
         N1 = NLAYERS + 1 ; N2 = NLAYERS + NLAYERS_UW ; Np = NLAYERS
      END SELECT
      
      NNv = Nvs*NN
      SELECT CASE( TRIM(RTM_MODE) )
      CASE('flux','int_flux') ;  CALL DOM_FLUX  
      CASE('spher_alb')       ;  CALL DOM_SPHER_ALB  
      END SELECT

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c ********************************************************************
c       Fluxes calculation 
c ********************************************************************
      SUBROUTINE DOM_FLUX
      USE GRIDS,   ONLY: DZR
      
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'DOM_FLUX'
      REAL (KND) Rus(NSZA), Fres(NSZA) ! Preliminary
c     ... Test arrays
      REAL TEX, ALW(NLAYERS-1)

      CALL ST_TRACK (R_NAME, 1, 'I')
      
      IF( JJESFT == 1 ) THEN
         ACTFLUX    (JPOS,N1:N2,:) = 0.d0
         DIF_SCAL_UP(JPOS,N1:N2,:) = 0.d0
         DIR_RAD    (JPOS,N1:N2,:) = 0.d0
         DIF_FLUX_DN(JPOS,N1:N2,:) = 0.d0
         DIF_FLUX_UP(JPOS,N1:N2,:) = 0.d0
         DIR_FLUX   (JPOS,N1:N2,:) = 0.d0
      ENDIF
c      IF( JPOS == 1 ) THEN
c         OPEN( 47, file='./DATA_OUT/Int_dn.dat' )
c         WRITE(47,'(A)') '#  column 1 - cosines of Gauss angles'
c         WRITE(47,'(A)') '#  column 2 - product mu*weight'
c         WRITE(47,'(A)') '#  column 3 - Fresnel reflection coefficient'
c         WRITE(47,'(A)') '#  column 4 - downward intensity'
c         WRITE(47,'(A)') '#  column 5 - upward intensity'
c         DO K = 1, NNv
c            WRITE(47,'(*(2x,e15.7))') GSL%XM(K),  GSL%AXM(K),
c     $           UDB%Fre_Rv(K,1,1), DFI(NL,1,K), DFI(NL,1,NNv+K)
c         ENDDO
c         CLOSE(47)
c         stop
c      ENDIF      
      S1 = Pie*E_W
      S2 = 2.d0*S1   /VST       ! test
      DO K = 1,NL
c         write(*,*) K, EXS(K,1)
         I = K + Np
         DIR_RAD(JPOS,I,:) = DIR_RAD(JPOS,I,:) + S1*EXS(K,:)
         DO Js = 1,NSZA             
c           ... Diffuse downward flux
            DIF_FLUX_DN(JPOS,I,Js) = DIF_FLUX_DN(JPOS,I,Js) 
     $           + S2*SUM( DFI(K,Js,1:NNv:Nvs)*AXM(1:NN) )

c           ... Diffuse upward flux
            DIF_FLUX_UP(JPOS,I,Js) = DIF_FLUX_UP(JPOS,I,Js) 
     $           + S2*SUM( DFI(K,Js,NNv+1:2*NNv:Nvs)*AXM(1:NN) ) 

c           ... Diffuse upward scalar flux (comparison for Mobly et al.)
            DIF_SCAL_UP(JPOS,I,Js) = DIF_SCAL_UP(JPOS,I,Js) 
     $           + S2*SUM( DFI(K,Js,NNv+1:2*NNv:Nvs)*AG(1:NN) ) 

c           ... Actinic flux (contribution of diffuse radiation)
            ACTFLUX(JPOS,I,Js) = ACTFLUX(JPOS,I,Js)
     $           + S2*SUM( DFI(K,Js,NNv+1:2*NNv:Nvs)*AG(1:NN) 
     $           + DFI(K,Js,1:NNv:Nvs)*AG(1:NN) )              
         ENDDO                  ! over solar angles (Js)
c        ... Direct flux
         DIR_FLUX(JPOS,I,:) = DIR_RAD(JPOS,I,:)*DABS(Mus(:))
c         write(*,*) I, DIR_FLUX(JPOS,I,1)
         WHERE( Mus(:) >= 0.d0 )
            FLUX_DN(JPOS,I,:) = 
     $           DIF_FLUX_DN(JPOS,I,:) + DIR_FLUX(JPOS,I,:)
            FLUX_UP(JPOS,I,:) = DIF_FLUX_UP(JPOS,I,:)
         ELSEWHERE
            FLUX_UP(JPOS,I,:) = 
     $           DIF_FLUX_UP(JPOS,I,:) + DIR_FLUX(JPOS,I,:)
            FLUX_DN(JPOS,I,:) = DIF_FLUX_DN(JPOS,I,:) 
         END WHERE           
c         IF( K==1 .AND. TRIM(Obs_POS)=='Ocean' ) THEN
c            WRITE(*,*) ' I = ',I
c            WRITE(*,*)
c     $           Mus(1), DIR_RAD(JPOS,I,:)*Mus(:), DIF_FLUX_DN(JPOS,I,:)
c         ENDIF
      ENDDO  ! over layers (K)
c
c      PAUSE
      IF( DIFB ) THEN
         ALW(:) = (AL_TES(1:NLAYERS-1) + AL_TES(2:NLAYERS))/2.d0
         TEX = SUM( ALW(1:NLAYERS-1)*DZR(1:NLAYERS-1) )
         I = 7
         TEX = EXP( -TEX/GSL%XM(I))
         WRITE(*,'(A,2x,f9.4)')
     $        '  Output angle ', DACOS(GSL%XM(I))/Pie*180.
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Direct transmittance:'
         WRITE(*,*) '  ---------------------'
         WRITE(*,'(A,*(2x,e15.7))') '  iso from below', TEX
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Diffuse transmittance:'
         WRITE(*,*) '  ----------------------'
         WRITE(*,'(A,2x,e15.7)') '  iso from below', 
     $        DFI(1,1,NN+I) - TEX
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Total transmittance:'
         WRITE(*,*) '  --------------------'
         WRITE(*,'(A,2x,e15.7)') '  iso from below', 
     $        DFI(1,1,NN+I)
      ENDIF

      IF( UDB%SPECULAR ) THEN
         Ref_ind = UDB%Ref_wd(1)
         Rus(:) = DSQRT( Mus(:)**2 + Ref_ind**2 - 1.d0 )/Ref_ind
         Fres(:) = 0.5d0*
     $        (  ( (MuS - Ref_ind*Rus)/(MuS + Ref_ind*RuS) )**2
     $        +  ( (RuS - Ref_ind*MuS)/(RuS + Ref_ind*MuS) )**2 ) 
c         WRITE(*,*) '    Fresnel reflection coefficient in st_flux ...'
         IF( Fbe > 0.d0 .AND. (.NOT.Sol_off) ) THEN
c           ... Reflected from the surface direct flux:
            DO I = 1,NL
               FLUX_UP(JPOS,I,:) = FLUX_UP(JPOS,I,:) 
     $              + EXS(NL,:)*(EXS(NL,:)/EXS(I,:)) ! Preliminary
     $              *MuS(:)*E_W*Pie*Fres(:)
            ENDDO
c            STOP
         ENDIF
      END IF

      CALL ST_TRACK (R_NAME, 1, 'O')
      END SUBROUTINE DOM_FLUX

c ********************************************************************
c       Spherical albedo calculation 
c ********************************************************************
      SUBROUTINE DOM_SPHER_ALB
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'DOM_SPHER_ALB'
      REAL (KND)   D_UP(NSZA), D_DN(NSZA), AGS(NSZA), XM_SOL(NSZA)
      REAL (KND)  Rus(NSZA), Fres(NSZA) ! Preliminary
c     ------------------------------------------------------------------
      
      CALL ST_TRACK (R_NAME, 1, 'I')
      
      CALL GAULEG( 0.D0, 1.D0, XM_SOL, AGS, NSZA )

      IF( JJESFT .EQ. 1 ) THEN
         FLUX_DN (JPOS,N1:N2,:) = 0.d0
         FLUX_UP (JPOS,N1:N2,:) = 0.d0
      ENDIF
      S1 = E_W 
      S2 = 4.d0*S1/VST
      DO K = 1,NL  ; I = K + Np
         DO Js = 1,NSZA
c           ... Integration of intensity over mu d_mu
            D_UP(Js) = SUM( DFI(K,Js,NNv+1:2*NNv:Nvs)*AXM(1:NN) ) 
            D_DN(Js) = SUM( DFI(K,Js,1:NNv:Nvs)      *AXM(1:NN) )
         ENDDO  ! over solar angles
         FLUX_DN(JPOS,I,1) = FLUX_DN(JPOS,I,1) 
c             ... Integration over d_mu0
     $        + S2*SUM(D_DN(:)*AGS(:))
c             ... Add direct solar contribution for transmitted radiation
     $        + 2.0*SUM(EXS(K,:)*AGS(:)*XM_SOL(:))
         FLUX_UP(JPOS,I,1) = FLUX_UP(JPOS,I,1) 
c             ... Integration over d_mu0
     $        + S2*SUM(D_UP(:)*AGS(:))
      ENDDO  ! over levels (K)

      IF( UDB%SPECULAR ) THEN
         IF( Fbe > 0.d0 .AND. (.NOT.Sol_off) ) THEN
c            ... Reflected from the surface direct flux:
            DO I = 1,NL
               DO Js = 1,NSZA
                  IF( EXS(NL,Js) < 1.d-10 ) THEN
                     D_up(Js) = 0.d0
                  ELSE
                     D_UP(Js) = EXS(NL,Js)*(EXS(NL,Js)/EXS(I,Js)) ! Preliminary
     $                    *XM_SOL(Js)*E_W*2.d0*UDB%FR_sol(Js,1,1)
                  ENDIF
               ENDDO            ! over solar angles (Js)
               FLUX_UP(JPOS,I,1) = FLUX_UP(JPOS,I,1) 
     $              + SUM(D_UP(:)*AGS(:))
            ENDDO  ! over levels (K)
         ENDIF
      END IF  ! Specular

      CALL ST_TRACK (R_NAME, 1, 'O')

      END SUBROUTINE DOM_SPHER_ALB

      END
      
C********************************************************************
C       SCIATRAN - Version 4.2 - Last modification: 17 Nov 2019
C                     diffuse transmission
C********************************************************************
      SUBROUTINE ST_TRANS(JPOS, JJESFT, E_W, NL, GSL, EXS, DFI, AFI)
c     ... Call in 'st_dom_tau.f'

      USE GRIDS,       ONLY: NLAYERS
      USE INTERN_DIMS, ONLY: Nvs
      USE RTM_OUTPUT,  ONLY: Tra_mu0, Tra_muv 
      USE SPHER_PARS,  ONLY: NLSA => N_LOS_ANG
      USE ST_VARS,     ONLY: Pie, VST, Kvs, N_Adj, NS_wf
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: GSL_TYPE
      USE WATER,       ONLY: INS_POSI
c     ------------------------------------------------------------------

      TYPE (GSL_type) :: GSL    ! Angles
c     ----------------------
      INTEGER,     INTENT(IN) :: JPOS, JJESFT, NL
      REAL (KND),  INTENT(IN) :: E_W, EXS(NL,GSL%NSZA), 
     $     DFI(NL,Kvs*GSL%NSZA,2*Nvs*GSL%NN), AFI(NL,2*Nvs*GSL%NN,N_Adj)
      TARGET :: AFI
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_TRANS'
      INTEGER     Js, Jv, Jad, NNv, NSZA, NN
      REAL (KND)  S1, S2, F_extr(GSL%NSZA), A_extr(NLSA)
      REAL (KND), POINTER :: AXM(:), PO_AFI(:,:), PO_EXL(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NSZA = GSL%NSZA  ;  NN = GSL%NN      
      AXM => GSL%AXM   ;  NNv = Nvs*NN

      IF( JJESFT == 1 ) THEN
         Tra_mu0(JPOS,:) = 0.d0  ;  Tra_muv(JPOS,:) = 0.d0
      ENDIF

      F_extr(:) = GSL%MuS(:) *Pie
      A_extr(:) = 2.d0 *Pie *VST

c     ... VST = 2 in the vector case
      S1 = Pie*E_W
      S2 = 2.d0*S1 /VST
      
c     ... Diffuse downward transmission (Mu0 directions)
      DO Js = 1,NSZA             
         Tra_mu0(JPOS,Js) = Tra_mu0(JPOS,Js) 
     $        + SUM( DFI(NL,Js,1:NNv:Nvs)*AXM(1:NN) )*S2
      ENDDO 
c     ... Diffuse downward  + Direct transmission
      Tra_mu0(JPOS,1:NSZA) = Tra_mu0(JPOS,1:NSZA)/F_extr(:) + EXS(NL,:)

c     ... Diffuse downward transmission (Muv directions)
      SELECT CASE( TRIM(INS_POSI) )
      CASE('boa')
c         ... Diffuse upward flux (unidirectional illumination from below)
         PO_AFI => AFI(1,NNv+1:2*NNv,:) ; PO_EXL => GSL%EXL(1,:)
      CASE DEFAULT
c        ... Diffuse downward flux (unidirectional illumination from above)
         PO_AFI => AFI(NL,1:NNv,:) ; PO_EXL => GSL%EXL(NL,:)
      END SELECT
      DO Jv = 1,NLSA ;  Jad = (Jv-1)*NS_WF + 1
         Tra_muv(JPOS,Jv) = Tra_muv(JPOS,Jv) + 
     $        S2*SUM( PO_AFI(1:NNv:Nvs,Jad)*AXM(1:NN) )/A_extr(Jv)
      ENDDO      
      Tra_muv(JPOS,1:NLSA) = Tra_muv(JPOS,1:NLSA) + PO_EXL(:)      
      
      SELECT CASE( TRIM(INS_POSI) )
      CASE('boa')
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Direct transmittance:'
         WRITE(*,*) '  ---------------------'
         WRITE(*,'(A,*(2x,e15.7))') '  uni from above', EXS(NL,1)
         WRITE(*,'(A,*(2x,e15.7))') '  uni from below', GSL%EXL(1,1)         
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Diffuse transmittance:'
         WRITE(*,*) '  ----------------------'
         WRITE(*,'(A,*(2x,e15.7))') '  uni from above',
     $        SUM( DFI(NL,1,1:NNv:Nvs)*AXM(1:NN) )*S2/F_extr(1)
         WRITE(*,'(A,*(2x,e15.7))') '  uni from below',
     $        SUM( AFI(1,NNv+1:2*NNv:Nvs,1)*AXM(1:NN) ) 
         WRITE(*,*) '  ----------------------------------'
         WRITE(*,*) '  Total transmittance:'
         WRITE(*,*) '  --------------------'
         WRITE(*,'(A,*(2x,e15.7))') '  uni from above',
     $        SUM( DFI(NL,1,1:NNv:Nvs)*AXM(1:NN) )*S2/F_extr(1)
     $        + EXS(NL,1)
         WRITE(*,'(A,*(2x,e15.7))') '  uni from below',
     $        SUM( AFI(1,NNv+1:2*NNv:Nvs,1)*AXM(1:NN) ) + GSL%EXL(1,:)
         WRITE(*,*) '  ----------------------------------'
      CASE DEFAULT
         WRITE(*,*)
     $        'Direct flux forward and direct pseodo-forward flux:'
         WRITE(*,'(2x,I2,2(2x,e15.7))') JPOS, 
     $        EXS(NL,1)*Pie*GSL%MuS(1), GSL%EXL(NL,1)*Pie*GSL%MuV(1) 
         WRITE(*,*)  'Diffuse flux forward at bottom (mono from above):'
         WRITE(*,*)
     $        'Diffuse flux pseodo-forward at bottom (mono from above):'
         WRITE(*,'(2x,I2,2(2x,e15.7))') JPOS, 
     $        SUM( DFI(NL,1,1:NNv:Nvs)*AXM(1:NN) )*S2,
     $        SUM( AFI(NL,1:NNv:Nvs,1)*AXM(1:NN) ) *Pie*GSL%MuV(1)
         WRITE(*,*) 'Diffuse transmittance:'
         WRITE(*,'(2x,I2,2(2x,e15.7))') JPOS, 
     $        SUM( DFI(NL,1,1:NNv:Nvs)*AXM(1:NN) )*S2/F_extr(1),
     $        SUM( AFI(NL,1:NNv:Nvs,1)*AXM(1:NN) )
      END SELECT

c         WRITE(*,'(2x,I2,2(2x,e15.7))') 
c     $     JPOS, Tra_mu0(JPOS,1), Tra_muv(JPOS,1)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
c **********************************************************************
