C***********************************************************************
C
C       SCIATRAN - Version 3.9 - Last modification: 22 Dec 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE ST_PP_OWF(NN, NSZA, Mh, WFO, GSL, IOP, AFI, EXS, LOW,
     $     Ph_Us, Ph_rW, Ph_Ks, Ph_Kl, ADD_PLANCK, NPLANCK)
c     ... Called by "st_adjoint_o.f"

      USE GT_POLYN,    ONLY: AG => AG_W
      USE HYDROSOLS,   ONLY: LA_N => LA_N_HS, L1 => LA_S_HS, 
     $                       L2 => LA_E_HS
      USE INTERN_DIMS, ONLY: Nvs, NL_pla
      USE LOG_PARS,    ONLY: DIR_OUT
      USE RTM_OUTPUT,  ONLY: WF_INTEGR
      USE ST_VARS,     ONLY: F14, N_Adj, OWF_Ray, OWF_Ks, OWF_Kl,
     $                       OWF_Use, GeoD => GeoD_Out
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, 
     $                       N_LOS_OUT, NLSA => N_LOS_ANG, N_OBS_ALT 
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: GSL_TYPE, LOW_TYPE, IOP_TYPE, PH_MET
      USE WATER,       ONLY: NL => NLAYERS_UW, ZZZ_UW, Obs_POSITION
c     ==================================================================

      INTEGER, INTENT(IN) ::  NN, NSZA, Mh
      LOGICAL, INTENT(IN) ::  ADD_PLANCK
c     ----------------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (LOW_type) :: LOW    ! Local optical parameters ocean
      TYPE (IOP_TYPE), TARGET :: IOP    ! Integral optical parameters
      TYPE (PH_MET)   :: Ph_rW, Ph_Ks, Ph_Kl
c     ----------------------
      TYPE WFO_type             ! Local weighting functions (ocean)
      SEQUENCE
      REAL (KND), POINTER :: WF_ext(:,:), WF_sca(:,:,:)
      END TYPE WFO_type 
      TYPE (WFO_type) :: WFO
c     --------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_Us
c     -----------------------------------------------------
      REAL (KND), INTENT(IN) :: EXS(NL,NSZA), NPLANCK(NL_pla), 
     $     AFI(NL,2*NN*Nvs,N_Adj)
c     ... Local variables      
c     -------------------
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'ST_PP_OWF'
      INTEGER  I, J, K, IA, K1, L, Js, Jv, Ja, N_1, N_2, IPM, Kls
      REAL (KND) Wup, Wdo, Jad(NL), Ext_tes(NL), Tet(NL), DZR(NL),
c                           ... thermal emission
     $     AG1(NN), SCA_AL(NL_pla), PL_abs(NL_pla), BT(NL_pla)
      REAL (KND), DIMENSION (NL,NN,NN)   :: G_Gp, G_Gm
      REAL (KND), DIMENSION (NL,NN,NLSA) :: L_Gp, L_Gm
      REAL (KND)  S_L(NL,NLSA), S_Gm(NL,NN), S_Gp(NL,NN)
      REAL (KND), POINTER :: Muv(:), XM(:),
     $     AL(:), WF_ext(:,:), WF_OUT_SS(:,:), DFI(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     ... Water extinction coefficient
      AL  => LOW%AL_W  ;  DFI => IOP%DFI ;  WF_ext => WFO%WF_ext
      MuV => GSL%MuV   ;  XM  => GSL%XM  ;  AG1(:) = AG(:)*0.5d0
      IA = 1
c      Close(12)
cc     ... Gauss field of pseudo-forward RTE in ocean
c      Open(12,file=DIR_OUT//'/AFI_O.dat',RECL=15000)
c        Jv = 1
c        DO I = 1, NL
c           WRITE( 12,'(130(1x,e15.7))') (AFI(I,J,Jv), J = 1,2*NN)
c        ENDDO
c      Close(12)
c     ... Radiation in the direction of instrument and in
c         the solar direction
c      Open(12,file=DIR_OUT//'/Dir-Sol_O.dat',RECL=15000)
c        Jv = 1 ; Js = 1
c        DO I = 1, NL
c           WRITE( 12,'(1x,F8.3,2(1x,e15.7))') 
c     $          ZZZ_UW(I), IOP%Ref_dir(I,Js,Jv), IOP%Ref_sol(I,Jv,Js)
c        ENDDO
c      Close(12)

      DZR(NL) = 0.d0
      DZR(1:NL-1) = ZZZ_UW(2:NL) - ZZZ_UW(1:NL-1)
      Wup = 0.5d0 ; Wdo = 1.d0 - Wup

      SELECT CASE( GeoD%View(IA)%S )
c     ------->> Measured upward radiation 
      CASE('up') 
         SELECT CASE( TRIM(Obs_POSITION(1)) ) ! Preliminary ??
         CASE('Ocean')          ! Observer in ocean
            N_1 = N_OBS_ALT(1) ; N_2 = NL-1
         CASE('Atmos')          ! Observer in atmosphere
            N_1 = 1 ; N_2 = NL-1
         END SELECT
c     -------> Measured downward radiation
      CASE('down') 
         SELECT CASE( TRIM(Obs_POSITION(1)) )
         CASE('Atmos')          ! Observer in atmosphere
            STOP ' "AO_down"@"Atmos" is not implemented: "st_pp_owf.f"'
         CASE('Ocean')          ! Observer in ocean
            STOP ' "AO_down"@"Ocean" is not implemented: "st_pp_owf.f"'
         END SELECT
      END SELECT

c     ... Terms are not depend on the 'component' phase function and SZA 
c     ------------------------------------------------------------------
      K1 = 0  ;  Tet = 0.d0
      DO Js = 1, NSZA
         DO Jv = JV_S(Js),JV_E(Js)
c           ... Integrated over zenith angle product of the forward
c               and adjoint diffuse intensities in the ocean
            DO I = 1,NL
               Ext_tes(I) = SUM( (DFI(I,Js,1:NN)*AFI(I,1+NN:2*NN,Jv)
     $              +  DFI(I,Js,1+NN:2*NN)*AFI(I,1:NN,Jv))*AG(1:NN) )
            ENDDO
c           ... Diffuse adjoint in the solar direction times direct
c               solar (Ref_sol from "GAU_Adj_UP_SOL_SFI")
            Ext_tes(:) =  Ext_tes(:) 
     $           + IOP%Ref_sol(:,Jv,Js)*EXS(:,Js)/2.d0 
c           ... Diffuse forward in the direction of observation times
c               direct adjoint
c            Tet(:) = IOP%Ref_dir(:,Js,Jv)*GSL%EXL(:,Jv)/Muv(Jv)
            Tet(:) = IOP%Ref_dir(:,Js,Jv)*GSL%EXL(:,Jv)/ANG_OUT(2,Jv)
            DO I = N_1,N_2
               Tet(I) = (Tet(I) + Tet(I+1))*0.5d0*DZR(I)
            ENDDO

            DO I = 1,NL-1
               Ext_tes(I) = (Ext_tes(I) + Ext_tes(I+1))*0.5d0*DZR(I)
            ENDDO
            Ext_tes(N_1:N_2) = Ext_tes(N_1:N_2) + Tet(N_1:N_2)

            Jad(1)      = Wup*Ext_tes(1)
            Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2) + Wup*Ext_tes(2:NL-1) 
            Jad(NL)     = Ext_tes(NL-1)*Wdo 

            DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
               WF_ext(:,K1) = 
     $              -Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) ) 
            ENDDO
         ENDDO                  ! do loop over line-of-sight angles
      ENDDO                     ! do loop over solar zenith angles

c     ... Terms depending on the phase function
c     -----------------------------------------
      IPM = 1
      IF( OWF_Ray ) THEN 
         WF_OUT_SS => WFO%WF_sca(:,:,IPM)  ;  IPM = IPM + 1
         DO I = 1,NL
            G_Gp(I,:,:) = ph_rw%Egg(:,:,1)
            G_Gm(I,:,:) = ph_rw%Egg(:,:,2)
            L_Gp(I,:,:) = ph_rw%EgL(:,:,1)/2.d0 
            L_Gm(I,:,:) = ph_rw%EgL(:,:,2)/2.d0 
         ENDDO
         Kls = 0
         DO JS = 1,NSZA ; L = JS
            FORALL( K = 1:NN, I = 1:NL )
               S_Gp(I,K) = F14*ph_rw%Egs(K,L,1)*XM(K)*AG(K)
               S_Gm(I,K) = F14*ph_rw%Egs(K,L,2)*XM(K)*AG(K)
            END FORALL
            DO JV = JV_S(L),JV_E(L)
               S_L(:,JV) =  F14*ph_rw%EsL(JV,L,1)/MuV(JV)
            ENDDO
            CALL M_POINT_WF_COMP
         ENDDO
      ENDIF

c     ... WF for scattering coefficient of user defined particulate matter
      IF( OWF_Use ) THEN 
         WF_OUT_SS => WFO%WF_sca(:,:,IPM) ;  IPM = IPM + 1
         G_Gp = 0.d0 ; G_Gm = 0.d0 ; L_Gp = 0.d0 ; L_Gm = 0.d0
         S_Gp = 0.d0 ; S_Gm = 0.d0 ; S_L  = 0.d0
         DO J = 1,LA_N
            DO I = L1(J),L2(J)
               G_Gp(I,:,:) = Ph_Us%Egg(J,:,:,1)
               G_Gm(I,:,:) = Ph_Us%Egg(J,:,:,2) 
               L_Gp(I,:,:) = Ph_Us%EgL(J,:,:,1)/2.d0
               L_Gm(I,:,:) = Ph_Us%EgL(J,:,:,2)/2.d0
            ENDDO
         ENDDO
         Kls = 0
         DO Js = 1,NSZA ; L = Js
            DO J = 1,LA_N
               DO I = L1(J),L2(J)
                  S_Gp(I,:) = F14*Ph_Us%Egs(J,:,L,1)*XM(:)*AG(:)
                  S_Gm(I,:) = F14*Ph_Us%Egs(J,:,L,2)*XM(:)*AG(:)
               ENDDO
            ENDDO
            DO JV = JV_S(L),JV_E(L)
               FORALL( J=1:LA_N )
                  S_L(L1(J):L2(J),Jv) = 
     $              F14*Ph_Us%EsL(J,Jv,L,1)/MuV(Jv)
               END FORALL
            ENDDO
            CALL M_POINT_WF_COMP
         ENDDO
      ENDIF

c     ... Small particulate matter weighting function (by Kopelevich)
      IF( OWF_Ks ) THEN 
         WF_OUT_SS => WFO%WF_sca(:,:,IPM)  ;  IPM = IPM + 1
         DO I = 1,NL
            G_Gp(I,:,:) = ph_Ks%Egg(:,:,1)
            G_Gm(I,:,:) = ph_Ks%Egg(:,:,2)
            L_Gp(I,:,:) = ph_Ks%EgL(:,:,1)/2.d0 
            L_Gm(I,:,:) = ph_Ks%EgL(:,:,2)/2.d0 
         ENDDO
         Kls = 0
         DO JS = 1,NSZA ; L = JS
            FORALL( K = 1:NN, I = 1:NL )
               S_Gp(I,K) = F14*ph_Ks%Egs(K,L,1)*XM(K)*AG(K)
               S_Gm(I,K) = F14*ph_Ks%Egs(K,L,2)*XM(K)*AG(K)
            END FORALL
            DO JV = JV_S(L),JV_E(L)
               S_L(:,JV) =  F14*ph_Ks%EsL(JV,L,1)/MuV(JV)
            ENDDO
            CALL M_POINT_WF_COMP
         ENDDO
      ENDIF

c     ... Large particulate matter weighting function (by Kopelevich)
      IF( OWF_Kl ) THEN 
         WF_OUT_SS => WFO%WF_sca(:,:,IPM)  ;  IPM = IPM + 1
         DO I = 1,NL
            G_Gp(I,:,:) = ph_Kl%Egg(:,:,1)
            G_Gm(I,:,:) = ph_Kl%Egg(:,:,2)
            L_Gp(I,:,:) = ph_Kl%EgL(:,:,1)/2.d0 
            L_Gm(I,:,:) = ph_Kl%EgL(:,:,2)/2.d0 
         ENDDO
         Kls = 0
         DO JS = 1,NSZA ; L = JS
            FORALL( K = 1:NN, I = 1:NL )
               S_Gp(I,K) = F14*ph_Kl%Egs(K,L,1)*XM(K)*AG(K)
               S_Gm(I,K) = F14*ph_Kl%Egs(K,L,2)*XM(K)*AG(K)
            END FORALL
            DO JV = JV_S(L),JV_E(L)
               S_L(:,JV) =  F14*ph_Kl%EsL(JV,L,1)/MuV(JV)
            ENDDO
            CALL M_POINT_WF_COMP
         ENDDO
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c **********************************************************************
      SUBROUTINE M_POINT_WF_COMP

c     ... Local parameters
      REAL (KND)  R_T(2*NN), T_Wc(NL,NLSA), Ewf(NL,NLSA), Tet(NL,NLSA), 
     $     QP_T(NN), QM_T(NN)

      Tet = 0.0d0
c     contribution from the term containing Js(tau,Mu_v)
      DO Jv = JV_S(Js),JV_E(Js)
         Tet(N_1:N_2+1,Jv) = 
     $        S_L(N_1:N_2+1,Jv)*GSL%EXL(N_1:N_2+1,Jv)*EXS(N_1:N_2+1,Js)
      ENDDO

      T_Wc = 0.d0
      DO I = 1,NL
         DO Jv = JV_S(Js),JV_E(Js)
c           ... Contribution from the term < Js,I*d >
            T_Wc(I,Jv) = T_Wc(I,Jv) + SUM( Afi(I,NN+1:2*NN,Jv)*S_gp(I,:)
     $           +  Afi(I,1:NN,Jv)*S_gm(I,:) )*EXS(I,Js)
c           ... Contribution from the term < Jd,I*d >
            R_T = 0.d0
            DO K = 1,NN
               Qp_T(:) = G_Gp(I,K,:)*XM(K)*AG1(:)
               Qm_T(:) = G_Gm(I,K,:)*XM(K)*AG1(:)
               R_T(1:NN) = R_T(1:NN) + 
     $             (Qm_T*Afi(I,K,Jv) + Qp_T*Afi(I,K+NN,Jv))*AG(K)
               R_T(NN+1:2*NN) = R_T(NN+1:2*NN) + 
     $             (Qm_T*Afi(I,K+NN,Jv) + Qp_T*Afi(I,K,Jv) )*AG(K)
            ENDDO
            T_Wc(I,Jv) = T_Wc(I,Jv) + SUM( R_T(:)*DFI(I,Js,:) )
         ENDDO
      ENDDO

c     ... Contribution from the term containing Jd(tau,Mu_v)
      DO I = N_1, N_2 + 1
         DO Jv = JV_S(Js),JV_E(Js)
            R_T(1:NN)      = L_Gm(I,:,JV)*GSL%AXM(1:NN) 
            R_T(1+NN:2*NN) = L_Gp(I,:,JV)*GSL%AXM(1:NN) 
            Tet(I,Jv) = Tet(I,Jv) 
     $           + SUM(DFI(I,JS,:)*R_T(:))*GSL%EXL(I,JV)/MuV(JV) 
         ENDDO
      ENDDO

      SELECT CASE ( TRIM(WF_INTEGR) )
        CASE( 'nointegr' ) 
           DO JV = JV_S(JS),JV_E(JS)
               DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
                  WF_OUT_SS(:,Kls) = (T_Wc(:,Jv) + Tet(:,Jv))
     $                 *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
               ENDDO
           ENDDO
        CASE DEFAULT
           DO I = N_1,N_2
              Tet(I,:) = (Tet(I,:) + Tet(I+1,:))*0.5d0*DZR(I)
           ENDDO
           DO JV = JV_S(JS),JV_E(JS)
              Ewf(1:NL-1,JV) = 
     $             (T_Wc(1:NL-1,Jv) + T_Wc(2:NL,Jv))*0.5d0*DZR(1:NL-1)
              Ewf(N_1:N_2,JV) = Ewf(N_1:N_2,JV) + Tet(N_1:N_2,Jv)
           ENDDO
      END SELECT

      IF( Wup > 0.d0 .AND. Wup < 1.d0 ) Then
         DO JV = JV_S(JS),JV_E(JS)
            DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
               WF_OUT_SS(1,Kls) = Wup*Ewf(1,JV)
               DO I = 1,NL-2
                  WF_OUT_SS(I+1,Kls) = Wdo*Ewf(I,JV) 
     $                 + Wup*Ewf(I+1,JV)
               ENDDO
               WF_OUT_SS(NL,Kls) = Wdo*Ewf(NL-1,JV)
               WF_OUT_SS(:,Kls) = WF_OUT_SS(:,Kls)
     $                 *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
            ENDDO
         ENDDO
      ENDIF

      RETURN
      END SUBROUTINE M_POINT_WF_COMP
c **********************************************************************
      END 
