C***********************************************************************
C
C       SCIATRAN - Version 4.3 - Last modification: 27 Jan 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE ST_PP_RRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, DFI, 
     $     AFI, EXL, EXS, Muv, LOA, SRR, SRR_Jact )
c     ... Call in "st_adjoint_a.f"

      USE ABSORBERS,   ONLY: N_GAS
      USE GRIDS,       ONLY: NL => NLAYERS, DZR
      USE GT_POLYN,    ONLY: XM, AXM, AG, AG1
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: SOL_FLUX 
      USE RAMAN,       ONLY: Ram_Egg, Ram_EgL, Ram_Egs, Ram_EsL, 
     $                       WL_FACT, LINE_GAS, Sigma_RR, Phot_to_Wat
      USE RTM_OUTPUT,  ONLY: N_OP, DEL_RRS
      USE ST_VARS,     ONLY: F14, N_Adj, Pie, APPR_RRS, WL_GRID, TSM, 
     $                       Vec_sca, VST, Finc, VS_HC, Nc_der, NS_wf,
     $                       GeoD => GeoD_OUT
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, N_LOS_OUT, 
     $                       NLSA => N_LOS_ANG, N_OBS_ALT
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE, RRS_TYPE
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) ::  NN, NSZA, Mh, JPOS, JJESFT, SRR_Jact
c     --------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (RRS_type) :: SRR    ! RRS related parameters
c     --------------------------
      REAL (KND), INTENT(IN) :: E_W, Muv(NLSA)
      REAL (KND), TARGET :: AFI(NL,2*NN*Nvs,N_Adj), EXL(NL,NLSA),
     $     DFI(NL,NSZA,2*NN*Nvs), EXS(NL,NSZA)
c     ... Local variables      
c     -------------------
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'ST_PP_RRS'
      INTEGER   I, J, K, Kg, KK, K_s, Ks, Kv, L, Iae, Ita, Js, Jv, Rli,
     $      J_act, Iac, K1, NNv, Ja, Jg, Jw, Jwa, Jw1, IA
      INTEGER, POINTER :: Lst(:), Lfi(:)
      REAL (KND)  IRR, IR0, MULT, Ewei, WL0
      REAL (KND)  COF(NL), R_T(2*NN*Nvs),
     $     XML(Nvs*NN), AGL(Nvs*NN), XMT(Nvs*NN), S3(Nvs), S4(Nvs), 
     $     RHS(2*NN*Nvs)
      REAL (KND), DIMENSION (NN*Nvs,NLSA*Nvs) :: Egl_m, Egl_p
      REAL (KND), DIMENSION (NN*Nvs,NN*Nvs)   :: G_Gp, G_Gm
      REAL (KND), DIMENSION (Nvs*NLSA,NN*Nvs) :: Ugp, Ugm
      REAL (KND), DIMENSION (NN*Nvs)          :: S_Gm, S_Gp
      REAL (KND), ALLOCATABLE :: Twc(:,:,:), Tet(:,:,:), S_L(:,:),
     $     Ewf(:), HEL(:), RES(:,:,:,:)
      REAL (KND), POINTER :: SCR(:), Pad(:,:,:), PXL(:,:), Pfo(:,:,:),
     $     PXS(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = NN*Nvs
      Ewei = E_W  ;  SCR => LOA%SCR  

      ALLOCATE( TWc(NL,NLSA,NS_wf), Tet(NL,NLSA,NS_wf),       ! A
     $     S_L(NLSA,NS_wf), Ewf(NL-1), HEL(NL),  ! A
     $     RES(N_OP,NSZA,NLSA,NS_wf) )                          ! A

      IA = 1 ; Lst => GeoD%LVG(IA,:)%IALS ; Lfi => GeoD%LVG(IA,:)%IALE

c     ... Terms are not depend on the 'component' phase function and SZA 
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:) ; AGL(:) = AG(:)
         K_s = 1
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
         DO K = 1,NNv ; K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K1) ;  XMT(K) = XM(K1)
         ENDDO
         K_s = 4
         IF( Mh > 1 ) K_s = 4
      END SELECT

c     ... in the scalar case (VS_HC = 2 )
c         in the vector case (VS_HC = 1 )
      MULT = VS_HC/2.d0
      IF( Mh > 1 ) MULT = VS_HC
c      WRITE(*,*) '           ====> Mh :', Mh

c     .... Terms depending on the phase function
c      WRITE(*,*) ' Start scat WF '
      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
         FORALL( K=1:NNv ) 
           G_Gp(K,:) = Ram_Egg(K,:,1,Mh) *XML(K)*AGL(:)*0.5d0
           G_Gm(K,:) = Ram_Egg(K,:,2,Mh) *XML(K)*AGL(:)*0.5d0
         END FORALL
      CASE( 'vect_rte' )
         FORALL( K=1:NNv ) 
c          ... This is the components of the matrix G(Mi,Mj) 
c              Eq.(Gau-pp) in STB. 
           G_Gp(K,:) = XMT(:)*AGL(:)*0.5d0 
     $        *(Ram_Egg(K,:,1,Mh) + Ram_Egg(K,:,2,Mh)) 
c          ... This is the components of the matrix ^G(Mi,-Mj) 
c              Eq.(Gau-pm) in STB. 
           G_Gm(K,:) = XMT(:)*AGL(:) *0.5d0 
     $          *(Ram_Egg(K,:,1,Mh) - Ram_Egg(K,:,2,Mh))
         END FORALL
      END SELECT

c     For all actual wavelengths
c     ------------------------------------------------------------------
      IF( SRR%ACT_WL(JPOS) ) THEN
c        ... Set extraterrestrial solar flux at actual WL corresponding
c            to its position in the array WL_Ext
         IRR = -SOL_FLUX(JPOS)/Pie*MULT
c         IRR = 0.d0

c        ... Save solution of adjoint equation for three harmonics and
c            all actual wavelengths
         SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) = 
     $        SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) + AFI(:,:,:)*E_W

c        ... Save exponential term along line-of-sight directions
         IF( Mh==1 ) SRR%EXL_ACT(:,SRR_Jact,:) = 
     $        SRR%EXL_ACT(:,SRR_Jact,:) + EXL(:,:)*E_W

c        ... Save solution of the forward equation for three harmonics
         IF( TRIM(TSM)=='speed' ) THEN
            SRR%FOR_ACT(:,SRR_Jact,:,:,Mh) = 
     $           SRR%FOR_ACT(:,SRR_Jact,:,:,Mh) + Dfi(:,:,:)*E_W
            IF( Mh==1 ) SRR%EXS_ACT(:,SRR_Jact,:) = 
     $           SRR%EXS_ACT(:,SRR_Jact,:) + EXS(:,:)*E_W
         ENDIF
c        ... Current number of actual wavelength
         Ita = SRR_Jact
c        ... Set scattering coefficient as Raman total which is
c            proportional to the Rayleigh scattering coefficient
         COF(:) = SRR%R_TOT*SCR(:)
c        ... AFI and DFI are calculated for current actual wavelength
         Pad => AFI(:,:,:) ; PXL => EXL(:,:)
         Pfo => DFI(:,:,:) ; PXS => EXS(:,:)
c        ... Perform computations of DEL_RRS(:,:,:) caused by 
c            the Raman scattering energy loss at the current
c            actual wavelength
         CALL SUM_RAM

c        ... The following actions will be done if 'speed'-option was  
c            chosen and calculations are performed already for last  
c            actual wavelength and all esft-coefficients.
         IF( Ita==SRR%N_ACT.AND.TRIM(TSM)=='speed'.AND. 
     $       JJESFT==SRR%NESFT(SRR%N_ACT) ) THEN
            Ewei = 1.d0
c           ... Over all actual wavelengths to calculate energy gain
            DO Iac = 1, SRR%N_ACT 
c              ... Position of actual WL in the extended WL-grid
               Iae = SRR%A2E(Iac)
               IR0 = SOL_FLUX(Iae)/Pie*MULT
c              ... Over all excitation WL within this bin
               DO J_act = 1,SRR%NUM_IN(Iae)
c                 ... Ita gives the actual WL number for
c                     which contributes this excitation WL
                  Ita = SRR%ACT_IN(Iae,J_act)
                  IRR = IR0
                  IF( Phot_to_Wat ) IRR = IR0*WL_GRID(SRR%A2E(Ita))
     $                 /WL_GRID(Iae)
c                 ... Rli Raman line transition number
                  Rli  = SRR%RIN_IN(Iae,J_act)
c                 ... Raman scattering coefficient
                  COF(:) = WL_FACT*Sigma_RR(Rli,:)
     $                 /WL_GRID(SRR%A2E(Ita))**4
     $                 *SRR%GAM_WL(LINE_GAS(Rli),Iac)
c                 ... Set pointers to the saved above solution 
c                     of forward and adjoint equations
                  Pad => SRR%ADJ_ACT(:,Ita,:,:,Mh)
                  PXL => SRR%EXL_ACT(:,Ita,:) 
                  Pfo => SRR%FOR_ACT(:,Iac,:,:,Mh)
                  PXS => SRR%EXS_ACT(:,Iac,:)
c                 ... Contribution to DEL_RRS(:,:,:) due to 
c                     Raman scattering energy gain 
                  CALL SUM_RAM
               ENDDO
            ENDDO
         END IF

c     For all not empty bins and excitation wavelengths
c     ------------------------------------------------------------------
      ELSE
c         WL0 = 393.48
c        ... Set extraterrestrial solar flux at the corresponding 
c            excitation WL
         IR0 = SOL_FLUX(JPOS)/Pie*MULT
c        ... No contribution from excitation wavelength
c         IR0 = 0.d0

c        ... Over all excitation WL within this bin
         DO J_act = 1,SRR%NUM_IN(JPOS)
c           ... Ita gives the actual WL number for
c               which contributes this excitation WL
            Ita = SRR%ACT_IN(JPOS,J_act) 
            Iae = SRR%A2E(Ita)
            IRR = IR0
            IF( Phot_to_Wat ) IRR = IR0*WL_GRID(Iae)/WL_GRID(JPOS)
            Rli  = SRR%RIN_IN(JPOS,J_act)

c            ... Test with constant wavelength WL0
c            COF(:) = WL_FACT*Sigma_RR(Rli,:)/WL0**4
c     $           *SRR%GAM(LINE_GAS(Rli))

            COF(:) = WL_FACT*Sigma_RR(Rli,:)/WL_GRID(Iae)**4
     $           *SRR%GAM(LINE_GAS(Rli))

c           ... Set pointers to the current solution of forward equations
            Pfo => DFI(:,:,:) ; PXS => EXS(:,:)
c           ... Set pointers to the saved above solution of adjoint 
c               equations
            Pad => SRR%ADJ_ACT(:,Ita,:,:,Mh)
            PXL => SRR%EXL_ACT(:,Ita,:)
c           ... Perform computations of DEL_RRS(:,:,:) caused by 
c               the Raman scattering energy gain 
            CALL SUM_RAM
         ENDDO
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c *********************************************************************

      SUBROUTINE SUM_RAM
c      REAL (KND), DIMENSION (NL,NLSA) :: T1, T3, T4

      DO Jv = 1,Nvs*NLSA
         Egl_m(:,Jv) = Ram_EgL(:,Jv,1,Mh)*AGL(:)*XML(:)/2.d0
c           ... Egl_p  LoS <-- Gau  (see Eq.(G+muv+mui) in STB)
c               is used as is it
         Egl_p(:,Jv) = Ram_EgL(:,Jv,2,Mh)*AGL(:)*XML(:)/2.d0
      ENDDO
      DO Jv = 1,NLSA  ;  Kv = Nvs*(Jv-1) + 1
         DO Jg = 1,NN ;  Kg = Nvs*(Jg-1) + 1
            Ugp(Kv:Nvs*Jv,Kg:Nvs*Jg) = Egl_p(Kg:Nvs*Jg,Kv:Nvs*Jv)
            Ugm(Kv:Nvs*Jv,Kg:Nvs*Jg) = Egl_m(Kg:Nvs*Jg,Kv:Nvs*Jv)
         ENDDO
      ENDDO

      Ewf = 0.d0
      DO Js = 1,NSZA 
c        ... Single scattering source function including quadrature 
c            coefficients (see explanations in "st_pp_wf.f"
         Ks = (Js-1)*Nvs
         DO K = 1,NN
            K1 = (K-1)*Nvs
            S_Gp(K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $           *MATMUL( Finc(1,:),
     $           Ram_Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,1,Mh) )
            S_Gm(K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $           *MATMUL( Finc(1,:),
     $           Ram_Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,2,Mh) )
         ENDDO
         IF( Nvs > 1 ) THEN
            S_Gp(4:NNv:Nvs) = -S_Gp(4:NNv:Nvs)
            S_Gm(3:NNv:Nvs) = -S_Gm(3:NNv:Nvs)
         ENDIF
c
c         WRITE(*,*) '   SHAPE(Ram_EsL)  ', SHAPE(Ram_EsL)
         DO Jv = Jv_S(Js),Jv_E(Js)
            Kv = (Jv-1)*Nvs
            DO Jw = 1,NS_wf
               S_L(Jv,Jw) =  F14/MuV(Jv)*VST*SUM(Finc(1,:)
     $              *Ram_EsL(Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1,Mh))
            ENDDO  ! over RRS Stokes component (Jw)
         ENDDO  ! over line-of-sight (Jv)

         Tet = 0.d0  ;  TWc = 0.d0
         DO Jv = Jv_S(JS),Jv_E(JS)
c           ... Contribution from the term containing Js(tau,Mu_v).
c               Term 4 in Eq.(Ram_pp-fin) of STB
            DO Jw = 1,NS_wf
               FORALL( KK = Lst(Jv):Lfi(Jv)+1 )  Tet(KK,Jv,Jw) = 
     $              S_L(Jv,Jw)*PXL(KK,Jv)*PXS(KK,Js)
            ENDDO  ! over RRS Stokes component (Jw)
            DO I = 1,NL
c              ... Contribution from the term < I*d,P_ram >.
c                  Term 2 in Eq.(Ram_pp-fin) of STB
               DO Jw = 1,NS_wf
                  RHS(:) = Pad(I,:, NS_wf*(Jv-1)+Jw)
                  IF( Nvs > 1 ) THEN
                     RHS(NNv+3:2*NNv:Nvs)= -RHS(NNv+3:2*NNv:Nvs)
                     RHS(NNv+4:2*NNv:Nvs)= -RHS(NNv+4:2*NNv:Nvs)
                  ENDIF
                  DO K = 1,Nvs
                     S4(K) = SUM( 
     $                    ( RHS(NNv+K:2*NNv:Nvs)*S_gp(K:NNv:Nvs)
     $                     +RHS(    K:NNv  :Nvs)*S_gm(K:NNv:Nvs) )
     $                    *AG(1:NN) )
                  ENDDO
                  IF( Nvs > 1 ) S4(3) = -S4(3)
                  TWc(I,Jv,Jw) = TWc(I,Jv,Jw) + SUM(S4(1:K_s))*PXS(I,Js)
c                           WRITE(*,'(2x,3(1x,I3),2x,E15.7)') 
c     $                                I, Jv, Jw, SUM( S4(1:K_s) )
               ENDDO  ! over RRS Stokes component (Jw)
c              ... Contribution from the term 1 < I*d,Jd >
c              =========================================
               R_T = 0.d0
c              >>>  Modification for vector case
c              ... Multiple scattering source function in directions  
c                  +Gua and -Gau (see Eq.(SFG+) and (SFG-) in STB)
               R_T(1:NNv) = MATMUL( G_Gp(:,:),Pfo(I,Js,1:NNv) )
     $              + MATMUL( G_Gm(:,:),Pfo(I,Js,NNv+1:2*NNv) )
               R_T(NNv+1:2*NNv) = MATMUL( G_Gm(:,:),Pfo(I,Js,1:NNv) )
     $              + MATMUL( G_Gp(:,:),Pfo(I,Js,NNv+1:2*NNv) )
               DO Jw = 1,NS_wf
                  S3 = 0.d0 ; Jw1 = NS_wf*(Jv-1) + Jw
                  DO K = 1,Nvs
                     S3(K) = SUM( 
     $               ( R_T(    K:NNv  :Nvs)*Pad(I,NNv+K:2*NNv:Nvs,Jw1)
     $               + R_T(NNv+K:2*NNv:Nvs)*Pad(I,    K:  NNv:Nvs,Jw1) )
     $                    *AG(1:NN) )
                  ENDDO
c                 ... We do not need to change the sign of 3th component 
c                    in the scalar product because we have not multiplied 
c                    R_T(NNv+1:2*NNv) and AFI(I,NNv+1:2*NNv,Jv). However,
c                    the results should multiplied by D4
                  IF( Nvs > 1 ) S3(4) = -S3(4)
                  TWc(I,Jv,Jw) = TWc(I,Jv,Jw) + SUM( S3(1:K_s) )
               ENDDO  ! over ring Stokes component (Jw)
c              =========================================
            ENDDO  ! overe layer (I)
         ENDDO  ! over line-of-sight (Jv)

         DO Jw = 1,NS_wf
            DO Jv = Jv_S(Js),Jv_E(Js) ; Kv = (Jv-1)*Nvs + Nc_der(Jw)
c              ... contribution from the term containing Jd(tau,Mu_v).
c                  Term 3 in Eq.(Ram_pp-fin) of STB
               FORALL( KK = Lst(Jv):Lfi(Jv)+1 ) Tet(KK,Jv,Jw) = 
     $              Tet(KK,Jv,Jw) + PXL(KK,Jv)/MuV(Jv) 
     $              *SUM( Ugp(Kv,:)*Pfo(KK,Js,    1:NNv  ) 
     $                  + Ugm(Kv,:)*Pfo(KK,Js,NNv+1:2*NNv) )
c              ... Integration over layers
               FORALL( KK = Lst(Jv):Lfi(Jv) ) HEL(KK) = 0.5d0*DZR(KK)
     $              *(Tet(KK,Jv,Jw)*COF(KK) + Tet(KK+1,Jv,Jw)*COF(KK+1))
               FORALL( KK = 1:NL-1 ) Ewf(KK) = 0.5d0*DZR(KK)
     $              *(TWc(KK,Jv,Jw)*COF(KK) + TWc(KK+1,Jv,Jw)*COF(KK+1))
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ewf(KK) = Ewf(KK) +HEL(KK)
               RES(1,Js,Jv,Jw) = SUM( Ewf(:) )
            ENDDO  ! over LoS (Jv)
         ENDDO  ! over RRS Stokes component (Jw)
      ENDDO  ! over solar angles (Js)

c     ... Contribution of inelastic RRS terms
      J = 0
      DO Js = 1,NSZA
         DO Jv = Jv_S(Js),Jv_E(Js)
            DO Ja = 1,N_AZ_VAR  ;  J = J + 1
               DO Jw = 1,NS_wf
                  Jwa = Nc_der(Jw)
                  IF( Jwa > 2 ) THEN
                     DEL_RRS(Ita,J,Jwa) = DEL_RRS(Ita,J,Jwa)
     $                    + IRR*RES(1,Js,Jv,Jw)*Ewei
     $                    *DSIN( ANG_OUT(3,J)*DBLE(Mh-1) )
                  ELSE
                     DEL_RRS(Ita,J,Jwa) = DEL_RRS(Ita,J,Jwa)
     $                    + IRR*RES(1,Js,Jv,Jw)*Ewei
     $                    *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
                  ENDIF
               ENDDO ! over RRS Stokes component (Jw)
            ENDDO  ! over azimuthal angels (Ja)
         ENDDO  ! over viewing angles (Jv)
      ENDDO  ! over solar angles (Js)
      
      RETURN
      END SUBROUTINE SUM_RAM
         
c **********************************************************************
      END 
