C********************************************************************C
C
C	SCIATRAN - Version 3.9 - Last modification: 19 Dec 2017
C
C	Copyright (C) 1995-2010, Institute of Remote Sensing,
C			         University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ST_PP_VRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, DFI,  
     $     AFI, EXL, EXS, Muv, LOW, SVR )
c     ... Call in "st_adjoint_o.f"

      USE GT_POLYN,    ONLY: XM=>XM_W, AXM=>AXM_W, AG=>AG_W
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: SOL_FLUX 
      USE RTM_OUTPUT,  ONLY: N_OP, DEL_VRS
      USE ST_VARS,     ONLY: F14, N_Adj, Pie, GeoD => GeoD_OUT
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, N_LOS_OUT, 
     $                       NLSA => N_LOS_ANG, N_OBS_ALT
      USE TYPES,       ONLY: KND, LOW_TYPE, VRS_TYPE
      USE VRS,         ONLY: VRS_Egg, VRS_EgL, VRS_Egs, VRS_EsL
      USE WATER,       ONLY: NL => NLAYERS_UW, ZZZ_UW
c     ------------------------------------------------------------------

      INTEGER, INTENT(IN) ::  NN, NSZA, Mh, JPOS, JJESFT
c     --------------------------
      TYPE (LOW_type) :: LOW    ! Local optical parameters atmosphere
      TYPE (VRS_type) :: SVR    ! RRS related parameters
c     --------------------------
      REAL (KND), INTENT(IN) :: E_W, Muv(NLSA)
      REAL (KND), TARGET :: AFI(NL,2*NN,N_Adj), EXL(NL,NLSA),
     $     DFI(NL,NSZA,2*NN*Nvs), EXS(NL,NSZA)
c     ... Local variables      
c     -------------------
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'ST_PP_VRS'
      INTEGER   I, IA, J, K, KK, L, Jo, Js, Jv, To_act, J_act
      INTEGER, POINTER :: Lst(:), Lfi(:)
      REAL (KND)  IRR, MULT, Ewei
      REAL (KND)  COF(NL), R_T(2*NN), RES(N_OP,NSZA,NLSA), DZR(NL),
     $     Ewf(NL-1), Tet(NL), T_Wc(NL)
      REAL (KND), DIMENSION (NN)      :: S_Gm, S_Gp, QP_T, QM_T
      REAL (KND), POINTER :: SCR(:), Pad(:,:,:), PXL(:,:), Pfo(:,:,:),
     $     PXS(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Ewei = E_W  ;  SCR => LOW%SCR_W  ; IA = 1

c     ... Terms are not depend on the 'component' phase function and SZA 
      Lst => GeoD%LVG(IA,:)%IALS
      Lfi => GeoD%LVG(IA,:)%IALE

      DZR(NL) = 0.d0
      DZR(1:NL - 1) = ZZZ_UW(2:NL) - ZZZ_UW(1:NL - 1)

      MULT = 1.d0
      IF( Mh > 1 ) MULT = 2.d0
c      WRITE(*,*) '           ====> Mh :', Mh

c     ... For actual wavelengths only
      IF( SVR%ACT_WL(JPOS) ) THEN
c        ... It is assumed preliminary that first SVR%N_AUX position 
c            in the array WL_EXT are excitation wavelengths and after
c            that are positioned all actual wavelengths. 
         To_act = SVR%Jact

c        ... Save solution of adjoint equation for three harmonics and
c            all actual wavelengths
         SVR%ADJ_ACT(:,To_act,:,:,Mh) = 
     $        SVR%ADJ_ACT(:,To_act,:,:,Mh) + Afi(:,:,:)*E_W

c        ... Save exponential term along line-of-sight directions
         IF( Mh==1 ) SVR%EXL_ACT(:,To_act,:) = 
     $        SVR%EXL_ACT(:,To_act,:) + EXL(:,:)*E_W

      ELSE
c        ... Set extraterrestrial solar flux at excitation WL
         IRR = SOL_FLUX(JPOS)/Pie*MULT

c        ... Over all actual wavelengths 
         DO J_act = 1,SVR%N_EXT-SVR%N_AUX
            To_act = J_act
            COF(:) = 0.00264  !/LOW%AL_W(:)

            Pfo => DFI(:,:,:) ; PXS => EXS(:,:)
c            Pad => SVR%ADJ_ACT(:,To_act,:,:,Mh)
c            PXL => SVR%EXL_ACT(:,To_act,:)         
            Pad => SVR%ADJ_ACT(:,2,:,:,Mh)
            PXL => SVR%EXL_ACT(:,2,:)         
            CALL SUM_RAM
         ENDDO
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c *********************************************************************

      SUBROUTINE SUM_RAM

      DO Js = 1,NSZA 
         FORALL( K = 1:NN )
           S_Gp(K) = F14*VRS_Egs(K,Js,1,Mh)*XM(K)*AG(K)
           S_Gm(K) = F14*VRS_Egs(K,Js,2,Mh)*XM(K)*AG(K)
         END FORALL

         DO Jv = Jv_S(Js),Jv_E(Js)  ;  T_Wc = 0.d0
            DO I = 1,NL
c              ... Contribution from the term < I*d,P_ram >
               T_Wc(I) = T_Wc(I) 
     $              + SUM( Pad(I,NN+1:2*NN,Jv)*S_gp(:)
     $                  +  Pad(I,1:NN,Jv)*S_gm(:) )*PXS(I,Js)
               R_T = 0.d0
c              ... Contribution from the term < I*d,Jd >
               DO K = 1,NN
                  Qp_T(:) = VRS_Egg(K,:,1,Mh)*XM(K)*AG(:)*0.5d0
                  Qm_T(:) = VRS_Egg(K,:,2,Mh)*XM(K)*AG(:)*0.5d0
                  R_T(1:NN) = R_T(1:NN) + 
     $                 (Qm_T*Pad(I,K,Jv) + Qp_T*Pad(I,K+NN,Jv))*AG(K)
                  R_T(NN+1:2*NN) = R_T(NN+1:2*NN) + 
     $                 (Qm_T*Pad(I,K+NN,Jv) + Qp_T*Pad(I,K,Jv) )*AG(K)
               ENDDO  ! over Gaussian angles (K)
               T_Wc(I) = T_Wc(I) + SUM( R_T(:)*Pfo(I,Js,:) )
            ENDDO  ! over layers (I)
c          ... Contribution from the term containing Js(tau,Mu_v)
            FORALL( KK = Lst(Jv):Lfi(Jv)+1 ) Tet(KK) = F14
     $           *VRS_EsL(Jv,Js,1,Mh)/MuV(Jv)*PXL(KK,Jv)*PXS(KK,Js)
c           ... Contribution from the term containing Jd(tau,Mu_v)
            R_T(1:NN)      = VRS_EgL(:,Jv,2,Mh)/2.d0*AXM(1:NN) 
            R_T(1+NN:2*NN) = VRS_EgL(:,Jv,1,Mh)/2.d0*AXM(1:NN) 
            FORALL( KK = Lst(Jv):Lfi(Jv)+1 ) Tet(KK) = Tet(KK) 
     $           + SUM(Pfo(KK,Js,:)*R_T(:))*PXL(KK,Jv)/MuV(Jv) 
c           ... Integration over layers
            FORALL( KK = 1:NL-1 ) Ewf(KK) = (T_Wc(KK)*COF(KK)
     $           + T_Wc(KK+1)*COF(KK+1) )*0.5d0*DZR(KK)            
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Ewf(KK) = Ewf(KK)  
     $           + 0.5d0*DZR(KK)*(Tet(KK)*COF(KK) + Tet(KK+1)*COF(KK+1))
            RES(1,Js,Jv) = SUM( Ewf(:) )
         ENDDO  ! over viewing angles (Jv)
      ENDDO  ! over solar angles (Js)

c     ... Contribution of inelastic VRS terms
      J = 0
      DO Js = 1,NSZA
         DO Jv = JV_S(Js),JV_E(Js)
            DO L = 1,N_AZ_VAR  ;  J = J + 1
               DO Jo = 1,1
                  DEL_VRS(To_act,J) = DEL_VRS(To_act,J)
     $                 + IRR*RES(Jo,Js,Jv)*Ewei
     $                 *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      
      RETURN
      END SUBROUTINE SUM_RAM
         
c **********************************************************************
      END 
