C***********************************************************************
C
C       SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE ST_PP_WF(NN, NSZA, Mh, LWF, DFI, AFI, EXL, EXS, 
     $     Ref_sol, Ref_dir, Muv, LOA, Ph_R, Ph_A, Ph_dA, Ph_C, 
     $     ADD_PLANCK, NPLANCK)
c     ... Call in "st_adjoint_a.f"

      USE AEROSOLS,    ONLY: LA_N, L1 => LA_S, L2 => LA_E, Dwr_mi, N_dwr
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT
      USE CLOUD_INPUT, ONLY: Plus_ice, Phm_const, Deriv_Phc, MLCS
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, ZZZ
      USE GT_POLYN,    ONLY: XM, AXM, AG, AG1
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: NL_ice, Nm_ice, Nvs, NL_soot, NL_pla,
     $                       Nl_Cr, Ng_Cr, Nls_Cr, Nso_Cr, N_C2,
     $                       Nl_Ar, Ng_Ar, Nls_Ar, Nso_Ar, N_A2
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_RSO, N_ESO, N_SSO 
      USE RTM_OUTPUT,  ONLY: WF_INTEGR
      USE ST_VARS,     ONLY: F14, N_Adj, Nc_der, Finc, Ainc, Vec_sca, 
     $                       VST, Kvs, NS_wf, GeoD => GeoD_OUT,
     $                       Ray_to_WF
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, 
     $                       N_LOS_OUT, NLSA => N_LOS_ANG, 
     $                       N_SL => N_LOS_OUT
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE, LWF_TYPE, PHA_CLD, PH_MET
c     ==================================================================

      INTEGER, INTENT(IN) ::  NN, NSZA, Mh
      LOGICAL, INTENT(IN) ::  ADD_PLANCK
c     ----------------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
      TYPE (PH_MET)   :: Ph_R
c     --------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_A
c     -------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     -----------------------------------------------------
      REAL (KND), INTENT(IN) :: EXS(NL,NSZA), EXL(NL,NLSA),  
     $     Muv(NLSA), Ref_dir(NL,Kvs*NSZA,Nvs*NLSA), NPLANCK(NL_pla), 
     $     Ref_sol(NL,N_Adj,Nvs*NSZA), DFI(NL,NSZA,2*NN*Nvs), 
     $     AFI(NL,2*NN*Nvs,N_Adj)
c     ... Local variables      
c     -------------------
      CHARACTER (LEN=8),PARAMETER ::  R_NAME = 'ST_PP_WF'
      INTEGER    I, I1, J, K, KK, Jr, K1, K2, Ks, Kv, L, Ja, Js, Jv,  
     $     Jvf, Jw, Jw1, NNv, DCH, K_s, Idr, Kls, IA
      INTEGER, POINTER :: Lst(:), Lfi(:)
      REAL (KND) Wup, Wdo, Jad(NL), Ext_tes(NL), Tet(NL), XML(Nvs*NN), 
     $     AGL(Nvs*NN), XMT(Nvs*NN), Ter5(NL), S4(Nvs), S3(Nvs),
c          ... Thermal emission arrays
     $     SCA_AL(NL_pla), PL_abs(NL_pla), BT(NL_pla)
      REAL (KND), DIMENSION (NL,Nvs*NN,Nvs*NN)   :: G_Gp, G_Gm
      REAL (KND), DIMENSION (NL,Nvs*NN,Nvs*NLSA) :: Egl_m, Egl_p
      REAL (KND), DIMENSION (NL,Nvs*NN)  :: S_Gm, S_Gp
      REAL (KND)  S_L(NL,NLSA,NS_wf)
      REAL (KND), POINTER :: SCR(:), SCA(:), EXA(:), SCC(:), EXC(:), 
     $     EXI(:), ECO(:), WF_ext(:,:,:), WF_TE(:,:), WF_OUT_SS(:,:,:),
     $     Per(:,:,:,:), P_in(:,:,:,:,:), P_in4(:,:,:,:),
     $     WF_TE_TEM(:,:), DT1(:), DT2(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IA = 1 
      NNv = NN*Nvs
      SCR => LOA%SCR ; SCA => LOA%SCA ; EXA => LOA%EXA
      SCC => LOA%SCC ; EXC => LOA%EXC ; EXI => LOA%EXI
      ECO => LOA%AL
c      WRITE(*,*)  '   Mh - ', Mh
c      WRITE(*,*) '   st_pp_wf.f  DO_SSRCA - ', DO_SSRCA 
c      WRITE(*,*) '   DFI ', SUM( DFI(1:NL,1:NSZA,1:2*NN*Nvs) ) 
c      WRITE(*,*) '   AFI ', SUM( AFI(1:NL,1:2*NN*Nvs,1:N_Adj) ) 
c      WRITE(*,*) '  st_pp_wf: SHAPE(Afi)  ', SHAPE(Afi)
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:) ; AGL(:) = AG(:)
         K_s = 1
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
         DO K = 1,NNv ; K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K1) ;  XMT(K) = XM(K1)
         ENDDO
         K_s = 4
         IF( Mh > 1 ) K_s = 4
      END SELECT

      WF_ext => LWF%WF_ext
      IF(INQ_WF('cld', 'WFT')) DT1 => LWF%DT1

      Wup = 0.5d0 ; Wdo = 1.d0 - Wup

      Lst => GeoD%LVG(IA,:)%IALS
      Lfi => GeoD%LVG(IA,:)%IALE

c     <---- Start Planck contribution (without polarization!!!)----->
      IF( ADD_PLANCK )  CALL MAKE_WF_TE

      Ter5 = 0.d0  ;  Ext_tes(:) = 0.d0
      K1 = 0
      DO Js = 1, NSZA
         Ks = (Js-1)*Nvs
         DO Jv = Jv_S(Js),Jv_E(Js)
            DO Jw = 1,NS_wf
c              ... Stokes component number for which should be calculated
c                  derivative in the array AFI(:,:,:) and ref_sol
               Jw1 = NS_wf*(Jv-1) + Jw
c              ... Stokes component number for which should be calculated
c                  derivative in the array ref_dir(:,:,:)
               Jvf = (Jv-1)*Nvs + Nc_der(Jw)
               DO I = 1,NL
c                 ... Term 2 in Eq.(WF-sv) of STB (without -omega(tau))
c                     =================================================
                  DO K = 1,Nvs
                     S4(K) = SUM( 
     $                ( DFI(I,Js,K:NNv:Nvs)*AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $                + DFI(I,Js,NNv+K:2*NNv:Nvs)*AFI(I,K:NNv:Nvs,Jw1) )
     $                    *AG(1:NN) )
                  ENDDO  ! over Stokes components (K)
                  IF( Nvs > 1 ) S4(4) = -S4(4)
                  Ext_tes(I) = SUM( S4(1:K_s) ) 

c                 ... Term 1 in Eq.(WF-sv) of STB (without -omega(tau))
c                    ==================================================
c                    Ref_sol is the first Stokes component of the 
c                    pseudo-forward radiation in the direction 
c                    opposite to the direction of solar light
c                    (calculated in "gau_adj_up_sol.f").
                  Ext_tes(I) =  Ext_tes(I) + EXS(I,Js)/2.d0*
     $                 SUM( Ref_sol(I,Jw1,Ks+1:Ks+Nvs)*Finc(1,:) )
               ENDDO  ! over altitude layers (I)

c              ... Term 5 in Eq.(WF-sv) of STB (without -omega(tau))
c                  =================================================
c                  Ref_dir is the forward radiation in the direction
c                  of observation (see "post_processing.f"). 
c                  It is used here selected Stokes component only, Jvf,
c                  for which the derivative should be calculated.
               Ter5(:) = Ref_dir(:,Js,Jvf)*EXL(:,Jv)/Muv(Jv)

               IF(INQ_WF('cld', 'WFT')) THEN
                  DT1(:) = -( Ext_tes(:) + Ref_dir(:,Js,Jvf)*EXL(:,Jv)
     $                 /Muv(Jv) )*DCOS( ANG_OUT(3,1)*DBLE(Mh-1) )
                  WHERE( SCC(:) == 0.d0 )
                     Ter5(:) = 0.d0 ; Ext_tes(:) = 0.d0
                  END WHERE
               ENDIF

c              ... Averaging over two layers
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ter5(KK) =
     $              (Ter5(KK) + Ter5(KK+1))*0.5d0*DZR(KK)
               FORALL( KK = 1:NL-1 )  Ext_tes(KK) =
     $              (Ext_tes(KK) + Ext_tes(KK+1))*0.5d0*DZR(KK)

               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) =
     $              Ext_tes(KK) + Ter5(KK)

               Jad(1)      = Wup*Ext_tes(1)
               Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2) + Wup*Ext_tes(2:NL-1) 
               Jad(NL)     = Ext_tes(NL-1)*Wdo 

               IF( Nc_der(Jw) > 2 ) THEN
                  DO Ja = 1,N_AZ_VAR  ;  IF(Jw==1) K1 = K1 + 1
                     WF_ext(:,K1,Jw) = 
     $                    -Jad(:)*DSIN( ANG_OUT(3,K1)*DBLE(Mh-1) )
                  ENDDO            
               ELSE
                  DO Ja = 1,N_AZ_VAR  ;  IF(Jw==1) K1 = K1 + 1
                     WF_ext(:,K1,Jw) = 
     $                    -Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) )
                  ENDDO            
               ENDIF
            ENDDO  ! over Stokes component derivatives (Jw,Jw1)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)
c      IF( Mh==1 ) THEN
c         DO I = 1, NL
c            WRITE(*,'(2x,I3, *(2x,e12.4))') I, WF_ext(I,1,1),
c     $           WF_TE(I,1), WF_ext(I,1,1) + WF_TE(I,1)
c         ENDDO
c     ENDIF
      
c     ... Thermal emission contribution into temperature WF
c     ---------------------------------------------------------------
      IF( INQ_WF('temp', 'WFN').AND.ADD_PLANCK )  CALL MAKE_WF_TE_TEM
c     ---------------------------------------------------------------

c     ************************************************************
c               Terms depending on the phase function
c     ************************************************************
c     ... Weighting functions for Rayleigh
c     ------------------------------------------------------------
      IF( Ray_to_WF ) THEN 
         WF_OUT_SS => LWF%WF_scr ; DCH = 1

c        ... Dimension of Ph_R%Egg(NNv,NNv,2)
         SELECT CASE( Vec_sca )
         CASE( 'scal_rte' )
            FORALL( K=1:NNv ) 
              G_Gp(1,K,:) = Ph_R%Egg(K,:,1)*XML(K)*AGL(:)*0.5d0
              G_Gm(1,K,:) = Ph_R%Egg(K,:,2)*XML(K)*AGL(:)*0.5d0
            END FORALL
         CASE( 'vect_rte' )
c           ... Phase function to calculate Term 3
c               It is the multiple scattering source function 
c               all Gaussian nodes
            FORALL( K=1:NNv ) 
c             ... This is the components of the matrix G(Mi,Mj) 
c                 Eq.(Gau-pp) in STB. 
              G_Gp(1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Ph_R%Egg(K,:,1) + Ph_R%Egg(K,:,2)) 
c             ... This is the components of the matrix ^G(Mi,-Mj) 
c                 Eq.(Gau-pm) in STB. 
              G_Gm(1,K,:) = XMT(:)*AGL(:) *0.5d0 
     $             *(Ph_R%Egg(K,:,1) - Ph_R%Egg(K,:,2))
            END FORALL
         END SELECT
         FORALL( I=2:NL )
            G_Gp(I,:,:) = G_Gp(1,:,:) ; G_Gm(I,:,:) = G_Gm(1,:,:)
         END FORALL

c        ------------- Phase matrix to Term 6 -------------------
c            This term should be used for calculation of the 
c            multiple scattering source function in the observation
c            direction (LoS <-- +-Gau).
c              First dimension of array Ph_r%EgL(|,:,1:2) contains
c            1/XM(:) in the scalar case and does not in vector.
c              The matrix ^M = EgL(:,Jv,1) is designed so that 
c            D*^M is the phase matrix -LoS <-- Gau 
c            (see Eq.(G+muv-mui) in STB). To obtain phase matrix 
c            Los <-- -Gau  we need to use ^M*D.
c              We do not do this here because below we multiply ^M 
c            by DFI(I,Js,1+NNv:2*NNv) which should be multiplied by 
c            D from left
         DO Jv = 1,Nvs*NLSA
            Egl_m(1,:,Jv) = Ph_R%EgL(:,Jv,1)*AGL(:)*XML(:)/2.d0
c           ... Egl_p  LoS <-- Gau  (see Eq.(G+muv+mui) in STB)
c               is used as is it
            Egl_p(1,:,Jv) = Ph_R%EgL(:,Jv,2)*AGL(:)*XML(:)/2.d0
         ENDDO
         FORALL( I=2:NL )
            Egl_m(I,:,:) = Egl_m(1,:,:) ; Egl_p(I,:,:) = Egl_p(1,:,:)
         END FORALL
c        --------------------------------------------------------
c        ------------- Phase matrix to Term 4 -------------------
c            This term should be used for calculation of the 
c            single scattering source function Q into Gaussian 
c            nodes (+- Gau <-- Sol).

c        ... First dimension of array Ph_r%Egs(|,:,1:2) contains
c               1/XM(:) in the scalar case and does not contain in vector
c               The matrix M = Egs(:,:,1) was designed to describe phase 
c            matrix Sol <-- Gau (see routine "phase_matr.f").
c               We multiply M^T*Finc(1,:) to obtain single scattering 
c            Sol --> Gau. 
c            The multiplication of M^T by D4 from right is omitted
c            because it is assumed that Finc=diag{1,0,0,0}.
         Kls = 0
         DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
            DO K = 1,NN
               K1 = (K-1)*Nvs
               S_Gp(1,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $              *MATMUL( Finc(1,:),
     $              Ph_R%Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,1) )
            ENDDO
c           ... The multiplication of Egs(:,:,1)^T by D4 from left is down
c               See Eq.(Qp-sol) in STB  
            IF( Nvs > 1 ) THEN
               S_Gp(1,4:NNv:Nvs) = -S_Gp(1,4:NNv:Nvs)
            ENDIF
c        ... Because the matrix ^M = Egs(:,:,2) was designed so that  
c            the matrix D*^M describes -Sol <-- Gau (see "phase_matr.f") 
c            we multiply [^M]^T*Finc(1,:) to obtain phase matrix
c            -Gau <-- Sol. 
c            The multiplication of [^M]^T by D4 from right is omitted
c            because it is assumed that Finc=diag{1,0,0,0}.
            DO K = 1,NN
               K1 = (K-1)*Nvs
               S_Gm(1,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $              *MATMUL( Finc(1,:),
     $                   Ph_R%Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,2) )
            ENDDO
c           ... The multiplication of [^M]^T by D3 from left is down
c               See Eq.(Qm-sol) in STB  
            IF( Nvs > 1 )  S_Gm(1,3:NNv:Nvs) = -S_Gm(1,3:NNv:Nvs)

c            This is the single scattering source function  
c            (Sol --> +/- Gau) given by Eq.(SS-sf) in STB 
c            ( without e^(-tau/Mu0) )
            FORALL( I=2:NL )
               S_Gp(I,:) = S_Gp(1,:) ; S_Gm(I,:) = S_Gm(1,:)
            END FORALL

c           ---------- Phase matrix to Term 7 ------------
c               It is needed the single scattering forward source 
c               function into observation direction, i.e. Los <--- Sol. 
c                 Dimension of Ph_R%EsL(Nvs*NLSA,Nvs*NSZA,1:2)
c                 The matrix M = EsL(:,:,1) is designed to calculate 
c               the phase matrix Sol <-- Los ( see "phase_matr.f").
c                 To obtain phase matrix Los <--- Sol we use 
c               D4*[M^T]*D4 (see Eq.(Qp-sol-muv) in STB).
               DO Jv = JV_S(L),JV_E(L)
                  Kv = (Jv-1)*Nvs
                  DO Jw = 1,NS_wf
                     S_L(:,Jv,Jw) =  F14/MuV(Jv)*VST
     $                    *SUM(Finc(1,:)
     $                    *Ph_R%EsL(Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1))
                  ENDDO       ! over Stokes component derivatives (Jw)
               ENDDO      ! over line-of-sight (Jv)
            CALL M_POINT_WF_COMP(1)
         ENDDO   ! over solar angles (Js & L)
      ENDIF   ! DO_RET_RAY
c     ------------------------------------------------------------

c     ... Weighting functions for aerosol parameters
c     ------------------------------------------------------------
      IF( INQ_WF('aer', 'WFT') ) THEN 
         WF_OUT_SS => LWF%WF_sca(:,:,:,1) ; DCH = 2 
         CALL ZEROING
         CALL Local_Aer( Ph_A,1 )
c        ... Derivatives w.r.t. expansion coeff. of the scattering matrix
         IF( Dwr_mi ) THEN
            DO Idr = 1,N_Dwr
               WF_OUT_SS => LWF%WF_sca(:,:,:,1+Idr) 
               CALL ZEROING
               CALL Local_DAer( Ph_dA,2 )
            ENDDO
         ENDIF ! Dwr_mi 
      ENDIF ! DO_RET_AER
c     ------------------------------------------------------------

c     ... Weighting functions for water or ice clouds
c     ------------------------------------------------------------
      IF(INQ_WF('cld', 'WFT')) THEN 
         WF_OUT_SS => LWF%WF_scc(:,:,:) ; DT2 => LWF%DT2 ; DCH = 3
         CALL ZEROING
         P_in => Ph_C%C_GG(:,:,:,:,:,1) ; CALL CLD_GP_GM
         P_in => Ph_C%C_GL(:,:,:,:,:,1) ; CALL CLD_Egl
         P_in => Ph_C%C_GS(:,:,:,:,:,1)
         P_in4 => Ph_C%C_LS(:,:,:,:,1)
         Kls = 0
         DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
            CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
         ENDDO    ! over solar zenith angle (Js)
c         WRITE(*,*) '   LWF%WF_scc ', SUM(  LWF%WF_scc(1:NL,1,1) )
c        ... Add the contribution of the derivative of the phase function
c            w.r.t. effective radius
         IF( Deriv_Phc ) THEN
            WF_OUT_SS  => LWF%Dph_Dre(:,:,:) ; DCH = 4
            CALL ZEROING
            P_in => Ph_C%Dr_GG(:,:,:,:,:,1) ;  CALL CLD_GP_GM
            P_in => Ph_C%Dr_GL(:,:,:,:,:,1) ;  CALL CLD_Egl
            P_in => Ph_C%Dr_GS(:,:,:,:,:,1)
            P_in4 => Ph_C%Dr_LS(:,:,:,:,1)
            Kls = 0
            DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
               CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
            ENDDO   ! over solar zenith angle
         ENDIF
         IF( ANY(Plus_ice) ) THEN 
            WF_OUT_SS => LWF%WF_scm ; DT2 => LWF%DT2m  ;  DCH = 3
            CALL ZEROING
            P_in => Ph_C%C_GG(:,:,:,:,:,2) ;  CALL CLD_GP_GM
            P_in => Ph_C%C_GL(:,:,:,:,:,2) ;  CALL CLD_Egl
            P_in => Ph_C%C_GS(:,:,:,:,:,2) 
            P_in4 => Ph_C%C_LS(:,:,:,:,2)
            Kls = 0
            DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
               CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
            ENDDO    ! over solar zenith angle (Js)
         ENDIF ! Plus ice
      ENDIF ! DO_RET_CLD
c     ------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c **********************************************************************
c     Calculation of auxiliary matrixes G_Gp znd G_Gm: aerosol
c **********************************************************************
      SUBROUTINE Local_Aer( Phx,Ipa )

c     ... Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     --------------------------
      TYPE phase_tdz
      SEQUENCE
      REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Phx
c     ---------------------------------------------------

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
         DO J = 1,LA_N
            DO I = L1(J),L2(J)
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Phx%Egg(J,K,:,1)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Phx%Egg(J,K,:,2)*XML(K)*AGL(:)*0.5d0
               END FORALL
            ENDDO
         ENDDO

      CASE( 'vect_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
c         WRITE(*,*) '      Test for last layer '
         DO J = 1,LA_N ; I1 = L1(J)
            FORALL( K=1:NNv ) 
              G_Gp(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1) + Phx%Egg(J,K,:,2)) 
              G_Gm(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1) - Phx%Egg(J,K,:,2))              
            END FORALL
            DO I = L1(J)+1,L2(J)
c               WRITE(*,*) ' Test for last layer ', I1, I
               G_Gp(I,:,:) = G_Gp(I1,:,:) 
               G_Gm(I,:,:) = G_Gm(I1,:,:)
            ENDDO
         ENDDO
c         WRITE(*,*) '      Test for last layer '
      END SELECT

c     ... Auxiliary function Egl_m(:,:,:) and Egl_p(:,:,:)
      DO J = 1,LA_N
         DO I = L1(J),L2(J)
            DO Jv = 1,Nvs*NLSA 
               Egl_m(I,:,Jv) = Phx%EgL(J,:,Jv,1)*AGL(:)*XML(:)/2.d0
               Egl_p(I,:,Jv) = Phx%EgL(J,:,Jv,2)*AGL(:)*XML(:)/2.d0
            ENDDO               ! over viewing angles (Jv)
         ENDDO                  ! over sub-layers within layer (I)
      ENDDO                     ! over aerosol layers (J)

      Kls = 0
      DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
         DO J = 1,LA_N
c           ... Auxiliary functions S_Gp(:,:) and S_Gm(:,:)
c               in  "Local_Aer"
            DO I = L1(J),L2(J)
               DO K = 1,NN
                  K1 = (K-1)*Nvs
                  S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,1))
                  S_Gm(I,K1+1:K*Nvs) =  F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,2))
               ENDDO            ! over Gaussian angles (K)
               IF( Nvs > 1 ) THEN
                  S_Gp(I,4:NNv:Nvs) = -S_Gp(I,4:NNv:Nvs)
                  S_Gm(I,3:NNv:Nvs) = -S_Gm(I,3:NNv:Nvs)
               ENDIF
            ENDDO               ! over sub-layers within layer (I)

c           ... Auxiliary function S_L(:,:,:)
            DO Jv = JV_S(L),JV_E(L)
               Kv = (Jv-1)*Nvs
               DO Jw = 1,NS_wf
                  S_L(L1(J):L2(J),Jv,Jw) = F14*VST/MuV(Jv)
     $                 *SUM(Finc(1,:)
     $                 *Phx%EsL(J,Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1))
               ENDDO     ! over Stokes component derivatives (Jw)
            ENDDO      ! over line-of-sight (Jv)
         ENDDO      ! over aerosol layers (J)
c         WRITE( *,* ) '  Start "M_POINT" ', Ipa
         CALL M_POINT_WF_COMP(Ipa)
      ENDDO    ! over solar angles (Js), (L)
      
      END SUBROUTINE Local_Aer
c ************************************************************************
c     Calculation of auxiliary matrixes G_Gp znd G_Gm: aerosol derivatives
c ************************************************************************
      SUBROUTINE Local_DAer( Phx,Ipa )

c     ... Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     --------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $       Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Phx
c     ---------------------------------------------------
      REAL (KND) C_AV(NL)

      DO I = 1,NL-1
         C_AV(I) = (LOA%SCA(I) + LOA%SCA(I+1))/2.d0
      ENDDO
      C_AV(NL) = (LOA%SCA(NL) + LOA%SCA(NL-1))/2.d0

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)

c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        '   Ph_DA%Egg ', SUM( Phx%Egg(:,:,:,1,Idr) )

         DO J = 1,LA_N
            DO I = L1(J),L2(J)
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Phx%Egg(J,K,:,1,Idr)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Phx%Egg(J,K,:,2,Idr)*XML(K)*AGL(:)*0.5d0
               END FORALL
            ENDDO
         ENDDO

      CASE( 'vect_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
c         WRITE(*,*) '      Test for last layer '
         DO J = 1,LA_N ; I1 = L1(J)
            FORALL( K=1:NNv ) 
              G_Gp(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1,Idr) + Phx%Egg(J,K,:,2,Idr)) 
              G_Gm(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1,Idr) - Phx%Egg(J,K,:,2,Idr))              
            END FORALL
            DO I = L1(J)+1,L2(J)
c               WRITE(*,*) ' Test for last layer ', I1, I
               G_Gp(I,:,:) = G_Gp(I1,:,:) 
               G_Gm(I,:,:) = G_Gm(I1,:,:)
            ENDDO
         ENDDO
      END SELECT

c     ... Auxiliary function Egl_m(:,:,:) and Egl_p(:,:,:)
      DO J = 1,LA_N
         DO I = L1(J),L2(J)
            DO Jv = 1,Nvs*NLSA 
               Egl_m(I,:,Jv) = Phx%EgL(J,:,Jv,1,Idr)*AGL(:)*XML(:)/2.d0
               Egl_p(I,:,Jv) = Phx%EgL(J,:,Jv,2,Idr)*AGL(:)*XML(:)/2.d0
            ENDDO               ! over viewing angles (Jv)
         ENDDO                  ! over sub-layers within layer (I)
      ENDDO                     ! over aerosol layers (J)

      FORALL( I=1:NL )
        G_Gp(I,:,:) = G_Gp (I,:,:)*C_AV(I)
        G_Gm(I,:,:) = G_Gm (I,:,:)*C_AV(I)
        Egl_m(I,:,:)= Egl_m(I,:,:)*C_AV(I)
        Egl_p(I,:,:)= Egl_p(I,:,:)*C_AV(I)
      END FORALL

      Kls = 0
      DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
         DO J = 1,LA_N
c           ... Auxiliary functions S_Gp(:,:) and S_Gm(:,:)
            DO I = L1(J),L2(J)
               DO K = 1,NN
                  K1 = (K-1)*Nvs
                  S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,1,Idr))
                  S_Gm(I,K1+1:K*Nvs) =  F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,2,Idr))
               ENDDO            ! over Gaussian angles (K)
               IF( Nvs > 1 ) THEN
                  S_Gp(I,4:NNv:Nvs) = -S_Gp(I,4:NNv:Nvs)
                  S_Gm(I,3:NNv:Nvs) = -S_Gm(I,3:NNv:Nvs)
               ENDIF
            ENDDO               ! over sub-layers within layer (I)

c           ... Auxiliary function S_L(:,:,:)
            DO Jv = JV_S(L),JV_E(L)
               Kv = (Jv-1)*Nvs
               DO Jw = 1,NS_wf
                  S_L(L1(J):L2(J),Jv,Jw) = F14*VST/MuV(Jv)
     $                 *SUM(Finc(1,:)
     $                 *Phx%EsL(J,Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1,Idr))
               ENDDO            ! over Stokes component derivatives (Jw)
            ENDDO               ! over line-of-sight (Jv)
         ENDDO                  ! over aerosol layers (J)
         
         FORALL( I=1:NL )
           S_Gp(I,:) = S_Gp (I,:)*C_AV(I)
           S_Gm(I,:) = S_Gm (I,:)*C_AV(I)
           S_L(I,:,:)= S_L(I,:,:)*C_AV(I)
         END FORALL

c         WRITE( *,* ) '  Start "M_POINT" Idr = ', Idr
c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        '  G_Gp, G_Gm   ', SUM( G_Gp ), SUM( G_Gm )
c         WRITE( *, '(A,2(1x,E15.7))') 
c     $        '  S_Gp, S_Gm   ', SUM( S_Gp ), SUM( S_Gm ) 
c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        ' Egl_p, Egl_m  ', SUM( Egl_p), SUM( Egl_m ) 

         CALL M_POINT_WF_COMP(Ipa)
c         WRITE( *,* ) '   WF_OUT_SS ', SUM( WF_OUT_SS )
      ENDDO     ! over solar angles (Js), (L)
      
      END SUBROUTINE Local_DAer
c **********************************************************************

      SUBROUTINE ZEROING
      G_Gp = 0.d0 ; G_Gm = 0.d0 ; Egl_m = 0.d0 ; Egl_p = 0.d0
      S_Gp = 0.d0 ; S_Gm = 0.d0 ; S_L  = 0.d0      
      END SUBROUTINE ZEROING
c **********************************************************************
c     Calculation of auxiliary matrixes G_Gp znd G_Gm: clouds
c **********************************************************************
      SUBROUTINE CLD_GP_GM

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            Per => P_in(J,:,:,:,:)
            DO I = J_TOP(J)+1, J_BOT(J)-1
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Per(Jr,K,:,1)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Per(Jr,K,:,2)*XML(K)*AGL(:)*0.5d0
               END FORALL
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO  !  over altitude within cloud layer (I) 
         ENDDO  ! over cloud layers (J)
      CASE( 'vect_rte' )
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            Per => P_in(J,:,:,:,:)
            DO I = J_TOP(J)+1,J_BOT(J)-1
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = XMT(:)*AGL(:)*0.5d0
     $              *( Per(Jr,K,:,1) + Per(Jr,K,:,2) )
               G_Gm(I,K,:) = XMT(:)*AGL(:)*0.5d0
     $              *( Per(Jr,K,:,1) - Per(Jr,K,:,2) )
               END FORALL
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO  ! over altitude within cloud layer (I) 
         ENDDO  ! over cloud layers (J)
      END SELECT  
      END SUBROUTINE CLD_GP_GM

c **********************************************************************
      SUBROUTINE CLD_Egl
      DO J = 1,N_CLD_LAYERS ; Jr = 1
         Per => P_in(J,:,:,:,:)
         DO I = J_TOP(J)+1,J_BOT(J)-1
            DO Jv = 1,Nvs*NLSA
               Egl_m(I,:,Jv) = Per(Jr,:,Jv,1)*XML(:)*AGL(:)/2.d0
               Egl_p(I,:,Jv) = Per(Jr,:,Jv,2)*XML(:)*AGL(:)/2.d0
            ENDDO      ! over viewing angles (Jv)
            IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
         ENDDO  ! over altitude within cloud layer (I)           
      ENDDO  ! over cloud layers (J)
      END SUBROUTINE CLD_Egl

c **********************************************************************
      SUBROUTINE CLD_SG_SL
      DO J = 1,N_CLD_LAYERS ; Jr = 1
c     ... Cloud S_Gp and S_Gm:
         Per => P_in(J,:,:,:,:) 
         DO I = J_TOP(J)+1, J_BOT(J)-1
            DO K = 1,NN
               K1 = (K-1)*Nvs
               S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $              *MATMUL( Finc(1,:),
     $              Per(Jr,K1+1:K*Nvs,Ks+1:Js*Nvs,1))
               S_Gm(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $              *MATMUL( Finc(1,:),
     $              Per(Jr,K1+1:K*Nvs,Ks+1:Js*Nvs,2))
            ENDDO               ! over Gaussian angles (K)
            IF( Nvs > 1 ) THEN
               FORALL( K = 4:NNv:Nvs ) S_Gp(I,K) = -S_Gp(I,K)
               FORALL( K = 3:NNv:Nvs ) S_Gm(I,K) = -S_Gm(I,K)
            ENDIF
            IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
         ENDDO   ! over altitudes within layer (I)
      ENDDO  ! over cloud layers (J)
c     ... Cloud S_L:
      DO Jv = JV_S(L),JV_E(L)
         Kv = (Jv-1)*Nvs
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            DO I = J_TOP(J)+1,J_BOT(J)-1
               DO Jw = 1,NS_wf
                  S_L(I,Jv,Jw) = F14*VST/MuV(Jv)*SUM( Finc(1,:)
     $                 *P_in4(J,Jr,Kv+1:Kv+Nvs,Ks+Nc_der(Jw)) )
               ENDDO
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO      ! over altitude within cloud layer (I)
         ENDDO      ! over cloud layers (J)
      ENDDO      ! over viewing angles (Jv)
      END SUBROUTINE CLD_SG_SL

c **********************************************************************
      SUBROUTINE M_POINT_WF_COMP(Ipa)
c     ... Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     ... Local parameters
      INTEGER     Jg, Jw, Jw1
      REAL (KND), DIMENSION(NL,NLSA,NS_wf) :: T_Wc, Ewf, Tsl
      REAL (KND), DIMENSION(Nvs,NNv)  :: Ugp, Ugm
      REAL (KND), DIMENSION(2*NNv)    :: R_T, RHS 
c     ==================================================================

      Tsl = 0.d0  ;  T_Wc = 0.d0
      DO Jv = Jv_S(Js),Jv_E(Js)
c        ... Term 7 in Eq.(WF-sv) of STB, This is the single scattering 
c            source function (LoS <-- Sol) given by Eq.(SS-sf) in STB   
c           (without multiplier e^(-tau/Mu0))

c         write(*,*) '  Mh =  ', Mh
         DO Jw = 1,NS_wf
            FORALL( KK = Lst(Jv):Lfi(Jv)+1 ) Tsl(KK,Jv,Jw) = 
     $           S_L(KK,Jv,Jw)*EXL(KK,Jv)*EXS(KK,Js)
         ENDDO  ! over Stokes component derivatives (Jw)
         DO I = 1,NL
c              --------------- Test T4 -----------------------
            DO Jw = 1,NS_wf
               RHS(:) = AFI(I,:, NS_wf*(Jv-1)+Jw)
               IF( Nvs > 1 ) THEN
                  RHS(NNv+3:2*NNv:Nvs)= -RHS(NNv+3:2*NNv:Nvs)
                  RHS(NNv+4:2*NNv:Nvs)= -RHS(NNv+4:2*NNv:Nvs)
               ENDIF
               DO K = 1,Nvs
                  S4(K) = SUM( 
     $                 ( RHS(NNv+K:2*NNv:Nvs)*S_gp(I,K:NNv:Nvs)
     $                  +RHS(    K:NNv  :Nvs)*S_gm(I,K:NNv:Nvs) )
     $                 *AG(1:NN) )
               ENDDO
c              ... Change sign of 3th component in the scalar product
               IF( Nvs > 1 ) S4(3) = -S4(3)
c              ... Term 4 in Eq.(WF-sv) of STB (contribution of <Q,I*d>)
               T_Wc(I,Jv,Jw) = T_Wc(I,Jv,Jw) 
     $              + SUM( S4(1:K_s) )*EXS(I,Js)
            ENDDO  ! over Stokes component derivatives (Jw)

c           ... Term 3 in Eq.(WF-sv) of STB (contribution of <Jd,I*d>)
c               Multiple scattering source function in +Gua directions
c               is calculated according to Eq.(SFG+)
            R_T(1:NNv) = MATMUL( G_Gp(I,:,:),DFI(I,Js,1:NNv) )
     $           + MATMUL( G_Gm(I,:,:),DFI(I,Js,NNv+1:2*NNv) )
c           ... Multiple scattering source function in directions -Gau 
c               is calculated according to Eq.(SFG-). 
c               We do not multiply R_T(NNv+1:2*NNv) by D from left as 
c               states in Eq.(SFG-) because the AFI(I,NNv+1:2*NNv,Jv) 
c               (pseudo-adjoint Stokes vector) is not multiplied by D also.
            R_T(NNv+1:2*NNv) = MATMUL( G_Gm(I,:,:),DFI(I,Js,1:NNv) )
     $           + MATMUL( G_Gp(I,:,:),DFI(I,Js,NNv+1:2*NNv) )
c           ------------ Test T3 ------------------------------------
            DO Jw = 1,NS_wf
               S3 = 0.d0 ; Jw1 = NS_wf*(Jv-1) + Jw
               DO K = 1,Nvs
                  S3(K) = SUM( 
     $              (  R_T(    K:NNv  :Nvs)*AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $               + R_T(NNv+K:2*NNv:Nvs)*AFI(I,    K:  NNv:Nvs,Jw1) )
     $                 *AG(1:NN) )
               ENDDO
c             ... We do not need to change the sign of 3th component 
c                 in the scalar product because we have not multiplied 
c                 R_T(NNv+1:2*NNv) and AFI(I,NNv+1:2*NNv,Jv). However,
c                the results should multiplied by D4
               IF( Nvs > 1 ) S3(4) = -S3(4)
               T_Wc(I,Jv,Jw) = T_Wc(I,Jv,Jw) + SUM( S3(1:K_s) )
            ENDDO  ! over Stokes component derivatives (Jw)

         ENDDO  ! over layers (I)
c        ... Term 6 in Eq.(WF-sv) of STB (contribution of Jd*Ei)
         K1 = Nvs*(Jv-1)+1
         DO I = Lst(Jv), Lfi(Jv) + 1
            DO Jg = 1,NN ; K2 = Nvs*(Jg-1)+1
               Ugp(1:Nvs,K2:Nvs*Jg) = Egl_p(I,K2:Nvs*Jg,K1:Nvs*Jv)
               Ugm(1:Nvs,K2:Nvs*Jg) = Egl_m(I,K2:Nvs*Jg,K1:Nvs*Jv)
            ENDDO
            DO Jw = 1,NS_wf  ;  Kv = Nc_der(Jw)
               Tsl(I,Jv,Jw) = Tsl(I,Jv,Jw) +
     $              SUM( Ugp(Kv,:)*DFI(I,Js,    1:NNv  ) 
     $                 + Ugm(Kv,:)*DFI(I,Js,NNv+1:2*NNv) )
     $              *EXL(I,Jv)/MuV(Jv) 
            ENDDO  ! over Stokes component derivatives (Jw)
         ENDDO  ! over altitude levels (I)
      ENDDO  ! over line-of-sight (Jv)

c     ... Cloud does not corrected
      IF( DCH == 3 ) DT2(:) = ( T_Wc(:,1,1) + Tsl(:,1,1) )
     $     *DCOS( ANG_OUT(3,1)*DBLE(Mh-1) ) 

      SELECT CASE ( TRIM(WF_INTEGR) )
      CASE( 'nointegr' ) 
         DO Jv = Jv_S(JS),Jv_E(JS)
            DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
               WF_OUT_SS(:,Kls,1) = (T_Wc(:,Jv,1) + Tsl(:,Jv,1))
     $              *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
            ENDDO
         ENDDO
      CASE DEFAULT

         DO Jw = 1,NS_wf
            DO Jv = Jv_S(JS),Jv_E(JS)
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Tsl(KK,Jv,Jw) = 
     $              (Tsl(KK,Jv,Jw) + Tsl(KK+1,Jv,Jw))*0.5d0*DZR(KK)
               Ewf(1:NL-1,Jv,Jw) = 0.5d0*DZR(1:NL-1)*
     $              (T_Wc(1:NL-1,Jv,Jw) + T_Wc(2:NL,Jv,Jw))
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ewf(KK,Jv,Jw) = 
     $              Ewf(KK,Jv,Jw) + Tsl(KK,Jv,Jw)
            ENDDO  ! over line-of-sights (Jv)
         ENDDO  ! over Stokes component derivatives (Jw)
      END SELECT
      

      IF( Wup > 0.d0 .AND. Wup < 1.d0 ) Then
         DO Jv = Jv_S(Js),Jv_E(Js)
            DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
               DO Jw = 1,NS_wf
                  WF_OUT_SS(1,Kls,Jw) = Wup*Ewf(1,Jv,Jw)
                  DO I = 1,NL-2
                     WF_OUT_SS(I+1,Kls,Jw) = Wdo*Ewf(I,Jv,Jw) 
     $                    + Wup*Ewf(I+1,Jv,Jw)
                  ENDDO
                  WF_OUT_SS(NL,Kls,Jw) = Wdo*Ewf(NL-1,Jv,Jw)
                  IF( Nc_der(Jw) > 2 ) THEN
                     WF_OUT_SS(:,Kls,Jw) = WF_OUT_SS(:,Kls,Jw)
     $                    *DSIN( ANG_OUT(3,Kls)*DBLE(Mh-1) )
                  ELSE
                     WF_OUT_SS(:,Kls,Jw) = WF_OUT_SS(:,Kls,Jw)
     $                    *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
                  ENDIF
               ENDDO  ! over Stokes component derivatives (Jw)
            ENDDO  ! over azimuthal angels (Ja)
         ENDDO  ! over viewing angles (Jv)

      ENDIF

      RETURN
      END SUBROUTINE M_POINT_WF_COMP
c **********************************************************************

      SUBROUTINE MAKE_WF_TE

      WF_TE => LWF%WF_TE
      SCA_AL(:) = SCR(:) + SCA(:) + SCC(:)
      IF( ANY(Plus_ice) ) SCA_AL(:) = SCA_AL(:) + LOA%SCI(:)
      PL_abs(NL) = 1.d0
c     ... Absorption probability
      PL_abs(1:NL-1) = 1.d0 - (Wup*SCA_AL(1:NL-1) + Wdo*SCA_AL(2:NL))/
     $     (Wup*ECO(1:NL-1) + Wdo*ECO(2:NL))
      WHERE( PL_abs(:) > 0.d0 )
c        ... NPLANCK contains product (1 - omega(tau))*Plank_function(tau)
         BT(:) = NPLANCK(:)/PL_abs(:)
      ELSEWHERE
         BT(:) = 0.d0
      END WHERE

      K1 = 0
      DO Js = 1, NSZA
         DO Jv = Jv_S(Js),Jv_E(Js)
c           ... First term in Eq.(Wf-abs-DD) of STB containing B(tau)
            FORALL( KK = 1:NL )  Ext_tes(KK) = BT(KK)*
     $           SUM((AFI(KK,1+NN:2*NN,Jv) + AFI(KK,1:NN,Jv))*AG(1:NN))
c           ... Second term in Eq.(Wf-abs-DD) of STB containing B(tau)
            Tet(:) = BT(:)*EXL(:,Jv)/Muv(Jv) 

            FORALL( KK = Lst(Jv):Lfi(Jv) ) Tet(KK) =
     $           (Tet(KK) + Tet(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = 1:NL-1 ) Ext_tes(KK) =
     $           (Ext_tes(KK) + Ext_tes(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) = Ext_tes(KK)
     $           + Tet(KK)            

            Jad(1)      = Wup*Ext_tes(1)
            Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2) + Wup*Ext_tes(2:NL-1) 
            Jad(NL)     = Ext_tes(NL-1)*Wdo 

            DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
               WF_TE(:,K1) = 
     $              Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) ) 
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)

      END SUBROUTINE MAKE_WF_TE
c **********************************************************************
      
      SUBROUTINE MAKE_WF_TE_TEM 

      WF_TE_TEM => LWF%WF_TE_TEM
      PL_abs(:) = ECO(:) - SCA_AL(:) 
c     ... Calculate scalar product of the adjoint intensity
c         and Abs_coef(z)*Planck(z), i.e., a term in the 
c         right-hand side of the linearized forward RTE corresponding
c         to the relative variation of the Planck function 
c         ( Delta_Planck/Planck )
      K1 = 0
      DO Js = 1, NSZA
         DO Jv = JV_S(Js),JV_E(Js)
            Tet(:) = PL_abs(:)*BT(:)*EXL(:,Jv)/Muv(Jv) 
            FORALL( KK = 1:NL )  Ext_tes(KK) = PL_abs(KK)*BT(KK)*
     $           SUM((AFI(KK,1+NN:2*NN,Jv)+AFI(KK,1:NN,Jv))*AG(1:NN))
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Tet(KK) = 
     $           (Tet(KK) + Tet(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = 1:NL-1 ) Ext_tes(KK) = 
     $           (Ext_tes(KK) + Ext_tes(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) = Ext_tes(KK)
     $           + Tet(KK)
            Jad(1) = Wup*Ext_tes(1)
            Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2)+Wup*Ext_tes(2:NL-1) 
            Jad(NL) = Ext_tes(NL-1)*Wdo 
            DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
               WF_TE_TEM(:,K1) = 
     $              Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) ) 
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)
      END SUBROUTINE MAKE_WF_TE_TEM
      
c **********************************************************************
      END 
