C***********************************************************************
C
C	SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c **********************************************************************
      SUBROUTINE ST_PS_WF(NN, NSZA, Mh, LWF, DFI, AFI, EXL, EXS, 
     $     Ref_sol, Ref_dir, Muv, LOA, Ph_R, Ph_A, Ph_dA, Ph_C, SWF,
     $     PsL, ADD_PLANCK, NPLANCK, S_GS, Mus)
c     ... Call in "st_adjoint_a.f"
      
      USE ABSORBERS,   ONLY: N_GAS
      USE AEROSOLS,    ONLY: LA_N, L1 => LA_S, L2 => LA_E, Dwr_mi, N_dwr
      USE CLOUDS,      ONLY: DO_CLOUD, N_CLD_LAYERS, J_TOP, J_BOT
      USE CLOUD_INPUT, ONLY: Plus_ice, Phm_const, Deriv_Phc, MLCS
      USE GT_POLYN,    ONLY: XM, AXM, AG, AG1
      USE GRIDS,       ONLY: NL => NLAYERS, ZZZ, DZR, N_WF_L,  
     $                       Z_NI => Z_FINE, N_P => NL_FINE
      USE INFO,        ONLY: WF_DB
      USE INTERN_DIMS, ONLY: Nvs, NL_pla, NL_ice, Nm_ice,
     $                       Nl_Cr, Ng_Cr, Nls_Cr, Nso_Cr, N_C2,
     $                       Nl_Ar, Ng_Ar, Nls_Ar, Nso_Ar, N_A2, 
     $                       NL_soot, N_sol_ls
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_ESO
      USE RTM_OUTPUT,  ONLY: WF_INTEGR, TRANS_OUT
      USE ST_VARS,     ONLY: F14, N_Adj, N_Eff, Vec_sca, NS_wf, Nc_der,
     $                       Finc, VST, GeoD => GeoD_OUT, Ray_to_WF
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, Nf => NN,
     $                       N_LOS_OUT, NLSA => N_LOS_ANG, N_OBS_ALT
      USE TOOLS,       ONLY: INQ_WF
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: LOA_TYPE, LWF_TYPE, PHA_CLD, PH_MET, 
     $                       SWF_type
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NN, NSZA, Mh
      LOGICAL,    INTENT(IN) :: ADD_PLANCK
      REAL (KND), INTENT(IN) :: Muv(NLSA), EXS(NL,NSZA), 
     $     EXL(NL,NLSA), Mus(NSZA), NPLANCK(NL_pla), 
     $     Ref_dir(NL,NSZA,Nvs*NLSA), Ref_sol(NL,N_Adj,Nvs*NSZA), 
     $     AFI(NL,2*NN*Nvs,N_Adj), DFI(NL,NSZA,2*NN*Nvs), 
     $     PsL(NL,Nvs*NLSA,Nvs*NSZA,N_sol_ls),
     $     S_GS(NL,Nvs*NN,Nvs*NSZA,2)
c     ----------------------------------------------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PHA_CLD)  :: Ph_C   ! Phase matrixes of clouds
      TYPE (PH_MET)   :: Ph_R
      TYPE (SWF_type) :: SWF    ! "Spherical" weighting function
c     ----------------------------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_A
c     ---------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     -----------------------------------------------------
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=8), PARAMETER :: R_NAME = 'ST_PS_WF'
      INTEGER  I, I1, IA, Idr, J, Ja, Jg, Jr, Js, Jv, Jw, Jw1, Jvf, K, 
     $     K1, Ks, Kv, K_s, Kls, Kg1, Kg2, L, N_fine, N_OBS, 
     $     DCH, NNv, K2, KK
      INTEGER, POINTER :: Lst(:), Lfi(:)
      REAL (KND)  Wup, Wdo, Ext_tes(NL), T_AU(NL), ALW(NL), Tet(NL), 
     $     SCA_AL(NL_pla), PL_abs(NL_pla), BT(NL_pla), Tmp(NL), Phm(NL),
     $     LS_Loc(N_P,NLSA), Phf(N_P), WF_T(N_P,NL), Jint(N_P), Jad(NL),
     $     Z_IN(N_P), R_T(2*NN*Nvs), Ter1(NL), Ter5(NL), S4(Nvs)
      REAL (KND)  Pv2(NN*Nvs,Nvs), Pv1(NN*Nvs,Nvs)
      REAL (KND), DIMENSION (Nvs*NN) :: AGL, XML, XMT, Qp, Qm
      REAL (KND), DIMENSION (NL,Nvs*NN,Nvs*NN)   :: G_Gp, G_Gm
      REAL (KND), DIMENSION (NL,Nvs*NN,Nvs*NLSA) :: Egl_m, Egl_p
c                                        should be removed
      REAL (KND), DIMENSION (NL,NN,NLSA) :: L_Gp, L_Gm

      REAL (KND), DIMENSION (NL,Nvs*NN) :: S_Gm, S_Gp
      REAL (KND)  S_L(NL,NLSA,NS_wf)
      REAL (KND), POINTER :: SCR(:), SCA(:), EXA(:), SCC(:), EXC(:), 
     $     EXI(:), AL(:), WF_ext(:,:,:), WF_OUT_SS(:,:,:), WF_TE(:,:), 
     $     WF_TE_TEM(:,:), DT1(:), DT2(:), Per(:,:,:,:), 
     $     P_in(:,:,:,:,:), P_in4(:,:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = NN*Nvs
      N_OBS = N_OBS_ALT(1)

      SCR => LOA%SCR ; SCA => LOA%SCA ; EXA => LOA%EXA
      SCC => LOA%SCC ; EXC => LOA%EXC ; EXI => LOA%EXI
      AL  => LOA%AL

      IF( TRANS_OUT ) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')  ;  RETURN
      ENDIF
      
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:) ; AGL(:) = AG(:)
         K_s = 1
      CASE( 'vect_rte' )
         stop 'Not yet implemented'
         XML(:) = 1.d0  
         DO K = 1,NNv ; K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K1) ;  XMT(K) = XM(K1)
         ENDDO
         K_s = 4
         IF( Mh > 1 ) K_s = 4
      END SELECT

      WF_ext => LWF%WF_ext
      IF(INQ_WF('cld', 'WFT')) DT1 => LWF%DT1

      Wup = 0.5d0 ; Wdo = 1.0d0 - Wup

      ALW(1:NL - 1) = Wup*AL(1:NL-1) + Wdo*AL(2:NL)
      T_AU(1) = 0.D0 
      DO J = 1, NL - 1
         T_AU(J+1) = T_AU(J) + ALW(J)*DZR(J) 
      ENDDO

c     Terms are not depend on the 'component' phase function
c                    and SZA      
      Z_IN(:) = Z_NI(N_P:1:-1)
      N_fine = (N_OBS-1)*Nf + 1

      IF(LOG_LEVEL(1) >= N_ESO) THEN
         WRITE(LUNIT,*) ' N_OBS = ', N_OBS, ZZZ(N_OBS)
         WRITE(LUNIT,*) ' Observer alt. fine grid: ', Z_IN(N_fine)
c         WRITE(LUNIT,*) ' Check tau : ', SWF%T_fine(N_fine), T_AU(N_OBS)
         WRITE(LUNIT,*) ' Check tau : ', SWF%T_fine(N_P), T_AU(NL)
      END IF

      IA = 1 ; Lst => GeoD%LVG(IA,:)%IALS ; Lfi => GeoD%LVG(IA,:)%IALE
      Ls_Loc = 0.d0
      DO Jv = 1, NLSA
         IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) THEN
            Ls_loc(N_fine:N_P,Jv) = 
     $           DEXP(-(SWF%T_fine(N_fine:N_P) - T_AU(N_OBS))/Muv(Jv))
         ELSE
            Ls_Loc(1:N_fine,Jv) = 
     $           DEXP(-(T_AU(N_OBS) - SWF%T_fine(1:N_fine) )/Muv(Jv))
         END IF
      ENDDO  ! over LoS (Jv)

c ----------------------------------------------------------------------
      IF( ADD_PLANCK ) CALL MAKE_WF_TE    ! thermal emission contribution
c ----------------------------------------------------------------------
      K1 = 0  ;  Ter5 = 0.d0
      DO Js = 1, NSZA
         Ks = (Js-1)*Nvs
         DO Jv = Jv_S(Js),Jv_E(Js)
c           ... Term corresponds to the scalar product (Id*,Id)
            DO Jw = 1,NS_wf
c              ... Stokes component number for which should be calculated
c                  derivative in the array AFI(:,:,:) and ref_sol
               Jw1 = NS_wf*(Jv-1) + Jw
c              ... Stokes component number for which should be calculated
c                  derivative in the array ref_dir(:,:,:)
               Jvf = (Jv-1)*Nvs + Nc_der(Jw)
               DO I = 1,NL
                  DO K = 1,Nvs
                     S4(K) = SUM( 
     $                ( DFI(I,Js,K:NNv:Nvs)*AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $                + DFI(I,Js,NNv+K:2*NNv:Nvs)*AFI(I,K:NNv:Nvs,Jw1) )
     $                    *AG(1:NN) )
                  ENDDO  ! over Stokes components (K)
                  IF( Nvs > 1 ) S4(4) = -S4(4)
c                 ... Term 1 in Eq. (WFe-DD-ps) of STB
c                 ====================================
                  Ter1(I) = SUM( S4(1:K_s) ) 
c                 ====================================
               ENDDO   ! over altitude layers (I)
c ------------------------------------------------------------------------
c                  DO I = 1, NL
c                     R_T(1:NN)      = S_GS(I,:,Js,2)*XM(:)*AG1(:) !Ok
c                     R_T(1+NN:2*NN) = S_GS(I,:,Js,1)*XM(:)*AG1(:) !Ok
c                     Jad(I)  = SUM( AFI(I,:,Jv)*R_T(:) ) 
c                  ENDDO
c              ... It is used the same arrays as in "st_sph_wf.f"
               DO I = 1, NL
                  DO J = 1,NNv
                     Pv1(J,1:Nvs) = S_GS(I,J,Ks+1:Ks+Nvs,1)
                     Pv2(J,1:Nvs) = S_GS(I,J,Ks+1:Ks+Nvs,2)
                  ENDDO  ! over Gauss angles (J)
                  DO Jg = 1,NN
                     Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                     DO K2 = 1,Nvs
                        Qp(Kg1+K2-1)= SUM(Pv1(Kg1:Kg2,K2)*Finc(1,:) )
                        Qm(Kg1+K2-1)= SUM(Pv2(Kg1:Kg2,K2)*Finc(1,:) )
                     ENDDO
                  ENDDO  ! over Gauss angles (Jg)
                  R_T(1:NNv)       = Qm(:)*XML(:)*AGL(:)
                  R_T(1+NNv:2*NNv) = Qp(:)*XML(:)*AGL(:)
                  DO K = 1,Nvs
                     S4(K) = SUM( 
     $                    ( R_T(K:NNv:Nvs)*AFI(I,K:NNv:Nvs,Jw1)
     $                    + R_T(NNv+K:2*NNv:Nvs)
     $                    *AFI(I,NNv+K:2*NNv:Nvs,Jw1) )  )
                  ENDDO  ! over Stokes component (K)
c                 ... Term 3 in Eq. (WFe-DD-ps) of STB
c                     ==================================
c                  Jad(I) = F14*SUM( S4(:) )  *2.d0
                  Jad(I) = SUM( S4(:) )  *0.5d0 ! Pochoze tak !
               ENDDO   ! over altitude layers (I)

c               WRITE(*,*) '  Compare Jad ps: ', Jad(:)
c              ... Interpolate Jad on the "fine" altitude grid => Jint
               CALL AL_INTERP(NL,ZZZ,Jad, N_P,Z_IN,Jint, .FALSE.,*125)
c              ... Integration over fine sub-grid using DIR_SUN
c                  Ph_zA%EsL is used within this routine as Psl
               Phm(:) = PsL(:,Jv,Js,1)/2.d0/MuV(Jv)  
c              ... Interpolation phase matrix on the "fine" altitude grid
               CALL AL_INTERP(NL,ZZZ,Phm, N_P,Z_IN,Phf, .FALSE., *125)

               DO L = 1, N_P 
c                 ... Integrand in the penultimate term of Eq. (WFe-DD-ps)
c                 =======================================================
                  WF_T(L,:) = -SWF%DS_SUN_JL(L,:,Js,1)*
     $                 (Jint(L) +Ls_Loc(L,Jv)*Phf(L) )*SWF%SOL_Exp(L,Js) 
c                  WRITE(*,'(1x,I4,4(2x,e12.4))') L,  Jint(L)
!                  WRITE(*,'(1x,I4,*(1x,e12.4))')
!     $                 L, SWF%DS_SUN_JL(L,:,Js,1)
               ENDDO
c               stop
               TMP(:) = WF_T(1,:)*(Z_IN(1) - Z_IN(2))
     $              + WF_T(N_P,:)*(Z_IN(N_P-1) - Z_IN(N_P))
               DO I = 2, N_P - 1
                  TMP(:) = TMP(:) +  WF_T(I,:)*(Z_IN(I-1)-Z_IN(I+1))
               END DO
c              ... Ref_sol contains the reflected pseudo-forward
c                  intensity in the solar direction
               TMP(:) = TMP(:)/2.d0 /2.d0
c                   ... Last term in Eq. (WFe-DD-ps) of STB
c                   ================================================= 
     $              - SUM( Ref_sol(NL,Jw1,Ks+1:Ks+Nvs)*Finc(1,:) )
     $              *EXS(NL,Js)/2.d0*Mus(Js)*SWF%DS_SUN_JL(N_P,:,Js,1) 
c                   ================================================= 

c              ... !! TMP is already integrated over a layer !!
c                  This can not be used for calculation of cloud WF 
c                  such as CTH, CBH, and TAU 
c                  !!! require correction !!!
               IF( INQ_WF('cld', 'WFT') ) DT1(:) = 
     $              (-Ter1(:) -Ref_dir(:,Js,Jvf)*EXL(:,Jv)/Muv(Jv)
     $              + TMP(:) )*DCOS( ANG_OUT(3,1)*DBLE(Mh-1) )
               
c              ... Ref_dir upward (or downward) forward intensity at 
c                  all altitude levels in direction of line-of-sight 
c              ... Term d*I_d in Eq. (WFe-DD-ps) of STB
c              ==============================================
               Ter5(:) = Ref_dir(:,Js,Jvf)*EXL(:,Jv)/Muv(Jv)
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ter5(KK) = 
     $              (Ter5(KK) + Ter5(KK+1))*0.5d0*DZR(KK)
c              ===============================================
               FORALL( KK = 1:NL-1 )  Ter1(KK) =
     $              (Ter1(KK) + Ter1(KK+1))*0.5d0*DZR(KK)
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) = 
     $              Ter1(KK) + Ter5(KK)
               Jad(1)      = Wup*Ext_tes(1)
               Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2) + Wup*Ext_tes(2:NL-1) 
               Jad(NL)     = Ext_tes(NL-1)*Wdo
               IF( Nc_der(Jw) > 2 ) THEN
                  DO Ja = 1,N_AZ_VAR  ;  IF(Jw==1) K1 = K1 + 1
                     WF_ext(:,K1,Jw) = (-Jad(:) + TMP(:))
     $                    *DSIN( ANG_OUT(3,K1)*DBLE(Mh-1) )
                  ENDDO            
               ELSE
                  DO Ja = 1,N_AZ_VAR  ;  IF(Jw==1)  K1 = K1 + 1
                     WF_ext(:,K1,Jw) = (-Jad(:) + TMP(:))
     $                    *DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) )
                  ENDDO
               ENDIF
            ENDDO  ! over Stokes component derivatives (Jw,Jw1)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)

c     ... Thermal emission contribution into temperature WF
      IF( INQ_WF('temp', 'WFN').AND.ADD_PLANCK ) CALL MAKE_WF_TE_TEM 
c
c     ... Terms depending on the phase function
c     *****************************************
      IF( Ray_to_WF ) THEN 
         WF_OUT_SS => LWF%WF_scr  ;  DCH = 1
         SELECT CASE( Vec_sca )
         CASE( 'scal_rte' )
            FORALL( K=1:NNv ) 
              G_Gp(1,K,:) = Ph_R%Egg(K,:,1)*XML(K)*AGL(:)*0.5d0
              G_Gm(1,K,:) = Ph_R%Egg(K,:,2)*XML(K)*AGL(:)*0.5d0
            END FORALL
         CASE( 'vect_rte' )
            FORALL( K=1:NNv ) 
              G_Gp(1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Ph_R%Egg(K,:,1) + Ph_R%Egg(K,:,2)) 
              G_Gm(1,K,:) = XMT(:)*AGL(:) *0.5d0 
     $             *(Ph_R%Egg(K,:,1) - Ph_R%Egg(K,:,2))
            END FORALL
         END SELECT
         FORALL( I=2:NL )
            G_Gp(I,:,:) = G_Gp(1,:,:) ; G_Gm(I,:,:) = G_Gm(1,:,:)
         END FORALL
c        ---------------------------------------------------------------
c              DO I = 1,NL
c                 L_Gp(I,:,:) = Ph_R%EgL(:,:,1)/2.d0 
c                 L_Gm(I,:,:) = Ph_R%EgL(:,:,2)/2.d0 
c              ENDDO
c        ... As in the case of "st_pp_wf.f" used instead of L_Gp, L_Gm
         DO Jv = 1,Nvs*NLSA
            Egl_m(1,:,Jv) = Ph_R%EgL(:,Jv,1)*AGL(:)*XML(:)/2.d0
            Egl_p(1,:,Jv) = Ph_R%EgL(:,Jv,2)*AGL(:)*XML(:)/2.d0
         ENDDO
         FORALL( I=2:NL )
            Egl_m(I,:,:) = Egl_m(1,:,:) ; Egl_p(I,:,:) = Egl_p(1,:,:)
         END FORALL
c        ---------------------------------------------------------------
         Kls = 0
c                  DO Js = 1,NSZA ; L = Js
c                     FORALL( K = 1:NN, I = 1:NL )
c                        S_Gp(I,K) = F14*Ph_R%Egs(K,L,1)*XM(K)*AG(K)
c                        S_Gm(I,K) = F14*Ph_R%Egs(K,L,2)*XM(K)*AG(K)
c                     END FORALL
c                     DO JV = JV_S(L),JV_E(L)
c                        S_L(:,JV) =  F14*Ph_R%EsL(JV,L,1)/MuV(JV)
c                     ENDDO
c                     CALL M_POINT_WF_COMP(1)
c                  ENDDO
         DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
            DO K = 1,NN
               K1 = (K-1)*Nvs
               S_Gp(1,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $              *MATMUL( Finc(1,:),
     $              Ph_R%Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,1) )
               S_Gm(1,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs) 
     $              *MATMUL( Finc(1,:),
     $              Ph_R%Egs(K1+1:K*Nvs,Ks+1:Js*Nvs,2) )
            ENDDO
            IF( Nvs > 1 ) THEN
               S_Gp(1,4:NNv:Nvs) = -S_Gp(1,4:NNv:Nvs)
               S_Gm(1,3:NNv:Nvs) = -S_Gm(1,3:NNv:Nvs)
            ENDIF
c           ... Copy for all other altitude levels
            FORALL( I=2:NL )
               S_Gp(I,:) = S_Gp(1,:) ; S_Gm(I,:) = S_Gm(1,:)
            END FORALL

            DO Jv = JV_S(L),JV_E(L)
               Kv = (Jv-1)*Nvs
               DO Jw = 1, NS_wf
                  S_L(:,Jv,Jw) =  F14/MuV(Jv)*VST*SUM(Finc(1,:)
     $                 *Ph_R%EsL(Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1))
               ENDDO      ! over Stokes component derivatives (Jw)
            ENDDO    ! over line-of-sight (Jv)
c           ------------------------------------------------
            CALL M_POINT_WF_COMP(1)
         ENDDO   ! over solar angles (Js,L)
      ENDIF  ! Rayleigh
c =======================================================================

      IF (INQ_WF('aer', 'WFT')) THEN 
         WRITE(*,*) '    No WFs for Aerosol '
         STOP       '     in pseudo-spherical approximation '
         WF_OUT_SS => LWF%WF_sca(:,:,:,1)  ;  DCH = 2 
c        ... Derivative wrt Stokes component is not implemented yet 
c            Derivatives wrt PSD function are not implemented yet
c                   G_Gp = 0.d0 ; G_Gm = 0.d0
c                   L_Gp = 0.d0 ; L_Gm = 0.d0
c                   S_Gp = 0.d0 ; S_Gm = 0.d0 ; S_L  = 0.d0
c                   DO J = 1,LA_N
c                      DO I = L1(J),L2(J)
c                         G_Gp(I,:,:) = Ph_A%Egg(J,:,:,1)
c                         G_Gm(I,:,:) = Ph_A%Egg(J,:,:,2) 
c                         L_Gp(I,:,:) = Ph_A%EgL(J,:,:,1)/2.0D0
c                         L_Gm(I,:,:) = Ph_A%EgL(J,:,:,2)/2.0D0
c                      ENDDO
c                   ENDDO
c                   Kls = 0
c                   DO JS = 1,NSZA ; L = JS
c                      DO J = 1,LA_N
c                         DO I = L1(J),L2(J)
c                            S_Gp(I,:) = F14*Ph_A%Egs(J,:,L,1)*XM(:)*AG(:)
c                            S_Gm(I,:) = F14*Ph_A%Egs(J,:,L,2)*XM(:)*AG(:)
c                         ENDDO
c                      ENDDO
c                      DO JV = JV_S(L),JV_E(L)
c                         FORALL( J=1:LA_N )
c                            S_L(L1(J):L2(J),Jv) = 
c               $              F14*Ph_A%EsL(J,Jv,L,1)/MuV(Jv)
c                         END FORALL
c                      ENDDO
c                      CALL M_POINT_WF_COMP(1)
c                   ENDDO
         CALL ZEROING
         CALL Local_Aer( Ph_A,1 )
c        ... Derivatives w.r.t. expansion coeff. of the scattering matrix
         IF( Dwr_mi ) THEN
            DO Idr = 1,N_Dwr
               WF_OUT_SS => LWF%WF_sca(:,:,:,1+Idr) 
               CALL ZEROING
               CALL Local_DAer( Ph_dA,2 )
            ENDDO
         ENDIF   ! Dwr_mi 
      ENDIF   ! Aerosol
c =======================================================================
      
      IF (INQ_WF('cld', 'WFT')) THEN 
         WRITE(*,*) '    No WFs for Cloud '
         STOP       '     in pseudo-spherical approximation '
c        ... Weighting functions for water or ice clouds
c            -------------------------------------------
         WF_OUT_SS => LWF%WF_scc(:,:,:) ; DT2 => LWF%DT2 ; DCH = 3
         CALL ZEROING
         P_in => Ph_C%C_GG(:,:,:,:,:,1) ; CALL CLD_GP_GM
         P_in => Ph_C%C_GL(:,:,:,:,:,1) ; CALL CLD_Egl
         P_in => Ph_C%C_GS(:,:,:,:,:,1)
         P_in4 => Ph_C%C_LS(:,:,:,:,1)
         Kls = 0
         DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
            CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
         ENDDO    ! over solar zenith angle (Js)

c        ... Add the contribution of the derivative of the phase function
c            w.r.t. effective radius
         IF( Deriv_Phc ) THEN
            WF_OUT_SS => LWF%Dph_Dre(:,:,:) ; DCH = 4
            CALL ZEROING
            P_in => Ph_C%Dr_GG(:,:,:,:,:,1) ;  CALL CLD_GP_GM
            P_in => Ph_C%Dr_GL(:,:,:,:,:,1) ;  CALL CLD_Egl
            P_in => Ph_C%Dr_GS(:,:,:,:,:,1)
            P_in4 => Ph_C%Dr_LS(:,:,:,:,1)
            Kls = 0
            DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
               CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
            ENDDO   ! over solar zenith angle
         ENDIF

         IF( ANY(Plus_ice) ) THEN 
            WF_OUT_SS => LWF%WF_scm ; DT2 => LWF%DT2m  ;  DCH = 3
            CALL ZEROING
            P_in => Ph_C%C_GG(:,:,:,:,:,2) ;  CALL CLD_GP_GM
            P_in => Ph_C%C_GL(:,:,:,:,:,2) ;  CALL CLD_Egl
            P_in => Ph_C%C_GS(:,:,:,:,:,2) 
            P_in4 => Ph_C%C_LS(:,:,:,:,2)
            Kls = 0
            DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
               CALL CLD_SG_SL ; CALL M_POINT_WF_COMP(1)
            ENDDO    ! over solar zenith angle (Js)
         ENDIF ! Plus ice
      ENDIF  ! Clouds
c =======================================================================

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
 125  STOP '...... Interpolation ERROR ..... '

      CONTAINS
c **********************************************************************

      SUBROUTINE Local_Aer( Phx,Ipa )

c     ... Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     --------------------------
      TYPE phase_tdz
      SEQUENCE
      REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:),
     $     Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Phx
c     ---------------------------------------------------

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
         DO J = 1,LA_N
            DO I = L1(J),L2(J)
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Phx%Egg(J,K,:,1)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Phx%Egg(J,K,:,2)*XML(K)*AGL(:)*0.5d0
               END FORALL
            ENDDO
         ENDDO

      CASE( 'vect_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
c         WRITE(*,*) '      Test for last layer '
         DO J = 1,LA_N ; I1 = L1(J)
            FORALL( K=1:NNv ) 
              G_Gp(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1) + Phx%Egg(J,K,:,2)) 
              G_Gm(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1) - Phx%Egg(J,K,:,2))              
            END FORALL
            DO I = L1(J)+1,L2(J)
c               WRITE(*,*) ' Test for last layer ', I1, I
               G_Gp(I,:,:) = G_Gp(I1,:,:) 
               G_Gm(I,:,:) = G_Gm(I1,:,:)
            ENDDO
         ENDDO
c         WRITE(*,*) '      Test for last layer '
      END SELECT

c     ... Auxiliary function Egl_m(:,:,:) and Egl_p(:,:,:)
      DO J = 1,LA_N
         DO I = L1(J),L2(J)
            DO Jv = 1,Nvs*NLSA 
               Egl_m(I,:,Jv) = Phx%EgL(J,:,Jv,1)*AGL(:)*XML(:)/2.d0
               Egl_p(I,:,Jv) = Phx%EgL(J,:,Jv,2)*AGL(:)*XML(:)/2.d0
            ENDDO               ! over viewing angles (Jv)
         ENDDO                  ! over sub-layers within layer (I)
      ENDDO                     ! over aerosol layers (J)

      Kls = 0
      DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
         DO J = 1,LA_N
c           ... Auxiliary functions S_Gp(:,:) and S_Gm(:,:)
c               in  "Local_Aer"
            DO I = L1(J),L2(J)
               DO K = 1,NN
                  K1 = (K-1)*Nvs
                  S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,1))
                  S_Gm(I,K1+1:K*Nvs) =  F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,2))
               ENDDO            ! over Gaussian angles (K)
               IF( Nvs > 1 ) THEN
                  S_Gp(I,4:NNv:Nvs) = -S_Gp(I,4:NNv:Nvs)
                  S_Gm(I,3:NNv:Nvs) = -S_Gm(I,3:NNv:Nvs)
               ENDIF
            ENDDO               ! over sub-layers within layer (I)

c           ... Auxiliary function S_L(:,:,:)
            DO Jv = JV_S(L),JV_E(L)
               Kv = (Jv-1)*Nvs
               DO Jw = 1,NS_wf
                  S_L(L1(J):L2(J),Jv,Jw) = F14*VST/MuV(Jv)
     $                 *SUM(Finc(1,:)
     $                 *Phx%EsL(J,Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1))
               ENDDO     ! over Stokes component derivatives (Jw)
            ENDDO      ! over line-of-sight (Jv)
         ENDDO      ! over aerosol layers (J)
c         WRITE( *,* ) '  Start "M_POINT" ', Ipa
         CALL M_POINT_WF_COMP(Ipa)
      ENDDO    ! over solar angles (Js), (L)
      
      END SUBROUTINE Local_Aer
c **********************************************************************

      SUBROUTINE Local_DAer( Phx,Ipa )

c     ... Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     --------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $       Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Phx
c     ---------------------------------------------------
      REAL (KND) C_AV(NL)

      DO I = 1,NL-1
         C_AV(I) = (LOA%SCA(I) + LOA%SCA(I+1))/2.d0
      ENDDO
      C_AV(NL) = (LOA%SCA(NL) + LOA%SCA(NL-1))/2.d0

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)

c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        '   Ph_DA%Egg ', SUM( Phx%Egg(:,:,:,1,Idr) )

         DO J = 1,LA_N
            DO I = L1(J),L2(J)
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Phx%Egg(J,K,:,1,Idr)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Phx%Egg(J,K,:,2,Idr)*XML(K)*AGL(:)*0.5d0
               END FORALL
            ENDDO
         ENDDO

      CASE( 'vect_rte' )
c        ... Auxiliary function G_Gp(:,:,:) and G_Gm(:,:,:)
c         WRITE(*,*) '      Test for last layer '
         DO J = 1,LA_N ; I1 = L1(J)
            FORALL( K=1:NNv ) 
              G_Gp(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1,Idr) + Phx%Egg(J,K,:,2,Idr)) 
              G_Gm(I1,K,:) = XMT(:)*AGL(:)*0.5d0 
     $             *(Phx%Egg(J,K,:,1,Idr) - Phx%Egg(J,K,:,2,Idr))              
            END FORALL
            DO I = L1(J)+1,L2(J)
c               WRITE(*,*) ' Test for last layer ', I1, I
               G_Gp(I,:,:) = G_Gp(I1,:,:) 
               G_Gm(I,:,:) = G_Gm(I1,:,:)
            ENDDO
         ENDDO
      END SELECT

c     ... Auxiliary function Egl_m(:,:,:) and Egl_p(:,:,:)
      DO J = 1,LA_N
         DO I = L1(J),L2(J)
            DO Jv = 1,Nvs*NLSA 
               Egl_m(I,:,Jv) = Phx%EgL(J,:,Jv,1,Idr)*AGL(:)*XML(:)/2.d0
               Egl_p(I,:,Jv) = Phx%EgL(J,:,Jv,2,Idr)*AGL(:)*XML(:)/2.d0
            ENDDO               ! over viewing angles (Jv)
         ENDDO                  ! over sub-layers within layer (I)
      ENDDO                     ! over aerosol layers (J)

      FORALL( I=1:NL )
        G_Gp(I,:,:) = G_Gp (I,:,:)*C_AV(I)
        G_Gm(I,:,:) = G_Gm (I,:,:)*C_AV(I)
        Egl_m(I,:,:)= Egl_m(I,:,:)*C_AV(I)
        Egl_p(I,:,:)= Egl_p(I,:,:)*C_AV(I)
      END FORALL

      Kls = 0
      DO Js = 1,NSZA ; L = Js ; Ks = (Js-1)*Nvs
         DO J = 1,LA_N
c           ... Auxiliary functions S_Gp(:,:) and S_Gm(:,:)
            DO I = L1(J),L2(J)
               DO K = 1,NN
                  K1 = (K-1)*Nvs
                  S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,1,Idr))
                  S_Gm(I,K1+1:K*Nvs) =  F14*VST*XML(K1+1:K*Nvs)
     $                 *MATMUL( Finc(1,:),
     $                 Phx%Egs(J,K1+1:K*Nvs,Ks+1:Js*Nvs,2,Idr))
               ENDDO            ! over Gaussian angles (K)
               IF( Nvs > 1 ) THEN
                  S_Gp(I,4:NNv:Nvs) = -S_Gp(I,4:NNv:Nvs)
                  S_Gm(I,3:NNv:Nvs) = -S_Gm(I,3:NNv:Nvs)
               ENDIF
            ENDDO               ! over sub-layers within layer (I)

c           ... Auxiliary function S_L(:,:,:)
            DO Jv = JV_S(L),JV_E(L)
               Kv = (Jv-1)*Nvs
               DO Jw = 1,NS_wf
                  S_L(L1(J):L2(J),Jv,Jw) = F14*VST/MuV(Jv)
     $                 *SUM(Finc(1,:)
     $                 *Phx%EsL(J,Kv+1:Kv+Nvs,Ks+Nc_der(Jw),1,Idr))
               ENDDO            ! over Stokes component derivatives (Jw)
            ENDDO               ! over line-of-sight (Jv)
         ENDDO                  ! over aerosol layers (J)
         
         FORALL( I=1:NL )
           S_Gp(I,:) = S_Gp (I,:)*C_AV(I)
           S_Gm(I,:) = S_Gm (I,:)*C_AV(I)
           S_L(I,:,:)= S_L(I,:,:)*C_AV(I)
         END FORALL

c         WRITE( *,* ) '  Start "M_POINT" Idr = ', Idr
c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        '  G_Gp, G_Gm   ', SUM( G_Gp ), SUM( G_Gm )
c         WRITE( *, '(A,2(1x,E15.7))') 
c     $        '  S_Gp, S_Gm   ', SUM( S_Gp ), SUM( S_Gm ) 
c         WRITE( *,'(A,2(1x,E15.7))' ) 
c     $        ' Egl_p, Egl_m  ', SUM( Egl_p), SUM( Egl_m ) 

         CALL M_POINT_WF_COMP(Ipa)
c         WRITE( *,* ) '   WF_OUT_SS ', SUM( WF_OUT_SS )
      ENDDO     ! over solar angles (Js), (L)
      
      END SUBROUTINE Local_DAer
c **********************************************************************

      SUBROUTINE ZEROING
      G_Gp = 0.d0 ; G_Gm = 0.d0 ; Egl_m = 0.d0 ; Egl_p = 0.d0
      S_Gp = 0.d0 ; S_Gm = 0.d0 ; S_L  = 0.d0      
      END SUBROUTINE ZEROING
c **********************************************************************

      SUBROUTINE CLD_GP_GM

      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            Per => P_in(J,:,:,:,:)
            DO I = J_TOP(J)+1, J_BOT(J)-1
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = Per(Jr,K,:,1)*XML(K)*AGL(:)*0.5d0
               G_Gm(I,K,:) = Per(Jr,K,:,2)*XML(K)*AGL(:)*0.5d0
               END FORALL
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO  !  over altitude within cloud layer (I) 
         ENDDO  ! over cloud layers (J)
      CASE( 'vect_rte' )
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            Per => P_in(J,:,:,:,:)
            DO I = J_TOP(J)+1,J_BOT(J)-1
               FORALL( K=1:NNv ) 
               G_Gp(I,K,:) = XMT(:)*AGL(:)*0.5d0
     $              *( Per(Jr,K,:,1) + Per(Jr,K,:,2) )
               G_Gm(I,K,:) = XMT(:)*AGL(:)*0.5d0
     $              *( Per(Jr,K,:,1) - Per(Jr,K,:,2) )
               END FORALL
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO  ! over altitude within cloud layer (I) 
         ENDDO  ! over cloud layers (J)
      END SELECT   ! G_Gp and G_Gm scalar-vector
      END SUBROUTINE CLD_GP_GM

c **********************************************************************
      SUBROUTINE CLD_Egl
      DO J = 1,N_CLD_LAYERS ; Jr = 1
         Per => P_in(J,:,:,:,:)
         DO I = J_TOP(J)+1,J_BOT(J)-1
            DO Jv = 1,Nvs*NLSA
               Egl_m(I,:,Jv) = Per(Jr,:,Jv,1)*XML(:)*AGL(:)/2.d0
               Egl_p(I,:,Jv) = Per(Jr,:,Jv,2)*XML(:)*AGL(:)/2.d0
            ENDDO      ! over viewing angles (Jv)
            IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
         ENDDO  ! over altitude within cloud layer (I)           
      ENDDO  ! over cloud layers (J)
      END SUBROUTINE CLD_Egl

c **********************************************************************
      SUBROUTINE CLD_SG_SL
      DO J = 1,N_CLD_LAYERS ; Jr = 1
c     ... Cloud S_Gp and S_Gm:
         Per => P_in(J,:,:,:,:) 
         DO I = J_TOP(J)+1, J_BOT(J)-1
            DO K = 1,NN
               K1 = (K-1)*Nvs
               S_Gp(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $              *MATMUL( Finc(1,:),
     $              Per(Jr,K1+1:K*Nvs,Ks+1:Js*Nvs,1))
               S_Gm(I,K1+1:K*Nvs) = F14*VST*XML(K1+1:K*Nvs)
     $              *MATMUL( Finc(1,:),
     $              Per(Jr,K1+1:K*Nvs,Ks+1:Js*Nvs,2))
            ENDDO               ! over Gaussian angles (K)
            IF( Nvs > 1 ) THEN
               FORALL( K = 4:NNv:Nvs ) S_Gp(I,K) = -S_Gp(I,K)
               FORALL( K = 3:NNv:Nvs ) S_Gm(I,K) = -S_Gm(I,K)
            ENDIF
            IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
         ENDDO  ! over altitudes within layer (I)
      ENDDO  ! over cloud layers (J)
c     ... Cloud S_L:
      DO Jv = JV_S(L),JV_E(L)
         Kv = (Jv-1)*Nvs
         DO J = 1,N_CLD_LAYERS ; Jr = 1
            DO I = J_TOP(J)+1,J_BOT(J)-1
               DO Jw = 1,NS_wf
                  S_L(I,Jv,Jw) = F14*VST/MuV(Jv)*SUM( Finc(1,:)
     $                 *P_in4(J,Jr,Kv+1:Kv+Nvs,Ks+Nc_der(Jw)) )
               ENDDO
               IF( .NOT.MLCS(J)%Phm_co ) Jr = Jr + 1
            ENDDO   ! over altitude within cloud layer (I)
         ENDDO   ! over cloud layers (J)
      ENDDO   ! over viewing angles (Jv)
      END SUBROUTINE CLD_SG_SL
c **********************************************************************

      SUBROUTINE M_POINT_WF_COMP (Ipa)

c     ,,, Input parameters
      INTEGER, INTENT(IN) :: Ipa
c     ... Local parameters
      INTEGER     Jg, Jw, Jw1
      REAL (KND), DIMENSION(NL,NLSA,NS_wf) :: T_Wc, Ewf, Tet
      REAL (KND), DIMENSION(Nvs,NNv)  :: Ugp, Ugm
      REAL (KND), DIMENSION(2*NNv)         :: R_T, RHS 
      REAL (KND) S3(Nvs)
c     REAL (KND)  QP_T(NN), QM_T(NN)

      Tet = 0.d0
c     contribution from the term containing Js(tau,Mu_v)
      DO Jv = Jv_S(Js),Jv_E(Js) 
         DO Jw = 1,NS_wf
            FORALL( KK = Lst(Jv):Lfi(Jv)+1 ) Tet(KK,Jv,Jw) = 
     $           S_L(KK,Jv,Jw)*EXL(KK,Jv)*EXS(KK,Js)
         ENDDO   ! over Stokes component derivatives (Jw)
      ENDDO   ! over line-of-sight (Jv)

      T_Wc = 0.d0
      DO I = 1,NL
         DO Jv = JV_S(Js),JV_E(Js)
c           ... Contribution from the term < Js,I*d >
c                      T_Wc(I,Jv,1) = T_Wc(I,Jv,1) 
c     $                     + SUM( Afi(I,NN+1:2*NN,Jv)*S_gp(I,:)
c     $                     +  Afi(I,1:NN,Jv)*S_gm(I,:) )*EXS(I,Js)
            DO Jw = 1,NS_wf
               RHS(:) = AFI(I,:, NS_wf*(Jv-1)+Jw)
               IF( Nvs > 1 ) THEN
                  RHS(NNv+3:2*NNv:Nvs)= -RHS(NNv+3:2*NNv:Nvs)
                  RHS(NNv+4:2*NNv:Nvs)= -RHS(NNv+4:2*NNv:Nvs)
               ENDIF
               DO K = 1,Nvs
                  S4(K) = SUM( 
     $                 ( RHS(NNv+K:2*NNv:Nvs)*S_gp(I,K:NNv:Nvs)
     $                  +RHS(    K:NNv  :Nvs)*S_gm(I,K:NNv:Nvs) )
     $                 *AG(1:NN) )
               ENDDO
c              ... Change sign of 3th component in the scalar product
               IF( Nvs > 1 ) S4(3) = -S4(3)
               T_Wc(I,Jv,Jw) = T_Wc(I,Jv,Jw) + SUM(S4(1:K_s))*EXS(I,Js)
            ENDDO   ! over Stokes component derivatives (Jw)
c                   ... Contribution from the term < Jd,I*d >
c                   R_T = 0.d0
c                   DO K = 1,NN
c                      Qp_T(:) = G_Gp(I,K,:)*XM(K)*AG1(:)
c                      Qm_T(:) = G_Gm(I,K,:)*XM(K)*AG1(:)
c                      R_T(1:NN) = R_T(1:NN) + 
c     $                    (Qm_T*Afi(I,K,Jv) + Qp_T*Afi(I,K+NN,Jv))*AG(K)
c                      R_T(NN+1:2*NN) = R_T(NN+1:2*NN) + 
c     $                    (Qm_T*Afi(I,K+NN,Jv) + Qp_T*Afi(I,K,Jv) )*AG(K)
c                   ENDDO
c                   T_Wc(I,Jv,1) = T_Wc(I,Jv,1) + SUM( R_T(:)*DFI(I,Js,:) )
            R_T(1:NNv) = MATMUL( G_Gp(I,:,:),DFI(I,Js,1:NNv) )
     $           + MATMUL( G_Gm(I,:,:),DFI(I,Js,NNv+1:2*NNv) )
            R_T(NNv+1:2*NNv) = MATMUL( G_Gm(I,:,:),DFI(I,Js,1:NNv) )
     $           + MATMUL( G_Gp(I,:,:),DFI(I,Js,NNv+1:2*NNv) )
            DO Jw = 1,NS_wf
               S3 = 0.d0 ; Jw1 = NS_wf*(Jv-1) + Jw
               DO K = 1,Nvs
                  S3(K) = SUM( 
     $              (  R_T(    K:NNv  :Nvs)*AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $               + R_T(NNv+K:2*NNv:Nvs)*AFI(I,    K:  NNv:Nvs,Jw1) )
     $                 *AG(1:NN) )
               ENDDO
               IF( Nvs > 1 ) S3(4) = -S3(4)
               T_Wc(I,Jv,Jw) = T_Wc(I,Jv,Jw) + SUM( S3(1:K_s) )
            ENDDO  ! over Stokes component derivatives (Jw)
         ENDDO  ! over line-of-sight (Jv)
      ENDDO  ! over layers (I)

c              contribution from the term containing Jd(tau,Mu_v)
c               DO I = N_1, N_2 + 1
c                  DO Jv = JV_S(Js),JV_E(Js)
c                     R_T(1:NN)      = L_Gm(I,:,JV)*AXM(1:NN) 
c                     R_T(1+NN:2*NN) = L_Gp(I,:,JV)*AXM(1:NN) 
c                     Tet(I,Jv,1) = Tet(I,Jv,1) 
c              $           + SUM(DFI(I,JS,:)*R_T(:))*EXL(I,JV)/MuV(JV) 
c                  ENDDO
c               ENDDO
      Ugp = 0.d0 ; Ugm = 0.d0
      DO Jv = JV_S(Js),JV_E(Js) ; K1 = Nvs*(Jv-1)+1
         DO I = Lst(Jv),Lfi(Jv) + 1
            DO Jg = 1,NN ; K2 = Nvs*(Jg-1)+1
               Ugp(1:Nvs,K2:Nvs*Jg) = Egl_p(I,K2:Nvs*Jg,K1:Nvs*Jv)
               Ugm(1:Nvs,K2:Nvs*Jg) = Egl_m(I,K2:Nvs*Jg,K1:Nvs*Jv)
            ENDDO  ! over Gauss angles (Jg)
            DO Jw = 1,NS_wf ; Kv = Nc_der(Jw)
               Tet(I,Jv,Jw) = Tet(I,Jv,Jw) + 
     $              SUM( Ugp(Kv,:)*DFI(I,Js,    1:NNv  ) 
     $                 + Ugm(Kv,:)*DFI(I,Js,NNv+1:2*NNv) )
     $              *EXL(I,Jv)/MuV(Jv) 
            ENDDO  ! over Stokes component derivatives (Jw)
         ENDDO  ! over altitude levels (I)
      ENDDO  ! over line-of-sight (Jv)
c ----------------------------------------------------------------------

      IF( DCH == 3 ) DT2(:) = ( T_Wc(:,1,1) + Tet(:,1,1) )
     $     *DCOS( ANG_OUT(3,1)*DBLE(Mh-1) ) 
      
      SELECT CASE ( TRIM(WF_INTEGR) )
      CASE( 'nointegr' ) 
         DO JV = JV_S(JS),JV_E(JS)
            DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
               WF_OUT_SS(:,Kls,1) = (T_Wc(:,Jv,1) + Tet(:,Jv,1))
     $              *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
            ENDDO
         ENDDO
      CASE DEFAULT
         DO Jw = 1,NS_wf
            DO Jv = Jv_S(JS),Jv_E(JS)
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Tet(KK,Jv,Jw) = 
     $              (Tet(KK,Jv,Jw) + Tet(KK+1,Jv,Jw))*0.5d0*DZR(KK)
               FORALL( KK = 1:NL-1 ) Ewf(KK,Jv,Jw) = 0.5d0*DZR(KK)
     $              *(T_Wc(KK,Jv,Jw) + T_Wc(KK+1,Jv,Jw))
               FORALL( KK = Lst(Jv):Lfi(Jv) ) Ewf(KK,Jv,Jw) = 
     $              Ewf(KK,Jv,Jw) + Tet(KK,Jv,Jw)
            ENDDO  ! over line-of-sights (Jv)
         ENDDO  ! over Stokes component derivatives (Jw)
      END SELECT

      IF( Wup > 0.0d0 .AND. Wup < 1.0d0 ) Then
         DO JV = JV_S(JS),JV_E(JS)
            DO Ja = 1,N_AZ_VAR  ;  Kls = Kls + 1
               DO Jw = 1,NS_wf
                  WF_OUT_SS(1,Kls,Jw) = Wup*Ewf(1,Jv,Jw)
                  DO I = 1,NL-2
                     WF_OUT_SS(I+1,Kls,Jw) = Wdo*Ewf(I,Jv,Jw) 
     $                    + Wup*Ewf(I+1,Jv,Jw)
                  ENDDO
                  WF_OUT_SS(NL,Kls,Jw) = Wdo*Ewf(NL-1,Jv,Jw)
                  IF( Nc_der(Jw) > 2 ) THEN
                     WF_OUT_SS(:,Kls,Jw) = WF_OUT_SS(:,Kls,Jw)
     $                    *DSIN( ANG_OUT(3,Kls)*DBLE(Mh-1) )
                  ELSE
                     WF_OUT_SS(:,Kls,Jw) = WF_OUT_SS(:,Kls,Jw)
     $                 *DCOS( ANG_OUT(3,Kls)*DBLE(Mh-1) )
                  ENDIF
               ENDDO  ! over Stokes component derivatives (Jw)
            ENDDO  ! over azimuthal angels (Ja)
         ENDDO  ! over viewing angles (Jv)
      ENDIF

      RETURN
      END SUBROUTINE M_POINT_WF_COMP
c **********************************************************************

      SUBROUTINE MAKE_WF_TE
      WF_TE => LWF%WF_TE
      SCA_AL(:) = SCR(:) + SCA(:) + SCC(:)
      IF( ANY(Plus_ice) ) SCA_AL(:) = SCA_AL(:) + LOA%SCI(:)
      PL_abs(NL) = 1.d0
c        ... Absorption probability
      PL_abs(1:NL-1) = 1.d0 - (Wup*SCA_AL(1:NL-1)+Wdo*SCA_AL(2:NL))/
     $     (Wup*AL(1:NL-1) + Wdo*AL(2:NL))
      WHERE( PL_abs(:) > 0.d0 )
c        ... NPLANCK contains product (1 - omega(tau))*Plank_function(tau)
         BT(:) = NPLANCK(:)/PL_abs(:)
      ELSEWHERE
         BT(:) = 0.d0
      END WHERE
      K1 = 0
      DO Js = 1, NSZA
         DO Jv = Jv_S(Js),Jv_E(Js)
            Tet(:) = BT(:)*EXL(:,Jv)/Muv(Jv) 
            FORALL( KK = 1:NL )  Ext_tes(KK) = BT(KK)*
     $           SUM((AFI(KK,1+NN:2*NN,Jv)+AFI(KK,1:NN,Jv))*AG(1:NN))

            FORALL( KK = Lst(Jv):Lfi(Jv) ) Tet(KK) =
     $           (Tet(KK) + Tet(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = 1:NL-1 ) Ext_tes(KK) =
     $           (Ext_tes(KK) + Ext_tes(KK+1))*0.5d0*DZR(KK)

            FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) = 
     $           Ext_tes(KK) + Tet(KK)

            Jad(1)      = Wup*Ext_tes(1)
            Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2) + Wup*Ext_tes(2:NL-1) 
            Jad(NL)     = Ext_tes(NL-1)*Wdo 

            DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
               WF_TE(:,K1) = Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) ) 
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)
      END SUBROUTINE MAKE_WF_TE
c **********************************************************************

      SUBROUTINE MAKE_WF_TE_TEM 
      WF_TE_TEM => LWF%WF_TE_TEM
      PL_abs(:) = AL(:) - SCA_AL(:) ; K1 = 0
c           ... See comment in "st_pp_wf.f"
      DO Js = 1, NSZA
         DO Jv = Jv_S(Js),Jv_E(Js)
            Tet(:) = PL_abs(:)*BT(:)*EXL(:,Jv)/Muv(Jv) 
            FORALL( KK = 1:NL ) Ext_tes(KK) = PL_abs(KK)*BT(KK)*
     $           SUM((AFI(KK,1+NN:2*NN,Jv)+AFI(KK,1:NN,Jv))*AG(1:NN))
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Tet(KK) = 
     $           (Tet(KK) + Tet(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = 1:NL-1 ) Ext_tes(KK) = 
     $           (Ext_tes(KK) + Ext_tes(KK+1))*0.5d0*DZR(KK)
            FORALL( KK = Lst(Jv):Lfi(Jv) ) Ext_tes(KK) = 
     $           Ext_tes(KK) + Tet(KK)
            
            Jad(1) = Wup*Ext_tes(1)
            Jad(2:NL-1) = Wdo*Ext_tes(1:NL-2)+Wup*Ext_tes(2:NL-1) 
            Jad(NL) = Ext_tes(NL-1)*Wdo 
            
            DO Ja = 1,N_AZ_VAR  ;  K1 = K1 + 1
               WF_TE_TEM(:,K1) = 
     $              Jad(:)*DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) ) 
            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO  ! over solar zenith angles (Js)

      END SUBROUTINE MAKE_WF_TE_TEM 
c **********************************************************************

      END
