c **********************************************************************
      DOUBLE PRECISION FUNCTION Ray_sec(POS, DEPOL)
c     ... Called by "rtm_init.f", "rayleigh_scatt.f"

      USE LOG_PARS,  ONLY: N_ESO, LOG_LEVEL, LUNIT
      USE ST_VARS,   ONLY: FP, SMKM, RAY_SCATT_IDX, Pie, PVC, 
     $                     LL_factor, Refr_index, King_factor
      USE TYPE_VARS, ONLY: KND
c     ==================================================================   

      REAL (KND), INTENT(IN) :: POS, DEPOL
      INTEGER IDI
      REAL (KND)  VV, X, Refi, N_Lo, Ps, tC, S1, S2, 
     $     R4(4), Ki(4), Cor_CO2
      REAL (KND), PARAMETER :: 
c          Avogadro's number [molecules/mol]
     $     N_Avo = 6.0221367d23, 
c          Molar volume [L/mol] at 0 C and 1013.25 mb  
     $     Mol_V = 22.4141d0, 
c          Loschmidt number [molecules/cm^3] at 
c               0 C, 1013.25 mb             15 C, 1013.25 mb   
     $     Ln_0C = N_Avo/Mol_V*1.d-03, Ln_15C = Ln_0C*273.15d0/288.15d0
C     ... Constants for the Bucholtz-95 approximation
      REAL (KND), DIMENSION (2):: HF0, HF1, HF2, HF3
      DATA  HF0, HF1, HF2, HF3
     $     /3.01577d-28, 4.01061d-28, 3.55212d0, 3.99668d0, 1.35579, 
     $     1.10298d-03, 0.11563d0, 2.71393d-02 /
c     ------------------------------------------------------------------
c     ... Constants for the Bodhaine et.al Rayleigh scattering cross-section
      REAL (KND), PARAMETER :: Ba = 1.0455996d0, Bb= -341.29061d0,
     $     Bc = -0.90230850d0, Bd = 0.0027059889d0 , Be = -85.968563d0
c     ------------------------------------------------------------------
      REAL (KND), EXTERNAL :: Refr_N2, Refr_O2, Refr_Ar, 
     $     Refr_CO2, King_N2, King_O2, King_Ar, King_CO2,
     $     Refr_air_E53, Refr_air_E66, Refr_air_PR
c     ==================================================================   

      SELECT CASE( RAY_SCATT_IDX )
      CASE(1)                   ! Bates
         IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) ' ---> Bates-84'
         VV = POS*1.d-07     ! Transform to cm
         S1 = 32.d0*Pie**3/3.d0/VV**4/Ln_0C**2

         R4(1) = Refr_N2(POS) - 1.d0  ;  R4(2) = Refr_O2(POS)  - 1.d0
         R4(3) = Refr_Ar(POS) - 1.d0  ;  R4(4) = Refr_CO2(POS) - 1.d0

         Ki(1) = King_N2(POS) ; Ki(2) = King_O2(POS)
         Ki(3) = King_Ar(POS) ; Ki(4) = King_CO2(POS)

         Ray_sec = S1*SUM( PVC(:)*R4(:)**2*Ki(:) )

c         WRITE(79,'(1x,F7.3,61(2x,E15.7))') 
c     $        POS, R4(1), R4(2), R4(3), R4(4)

      CASE(2)                   ! Bucholtz 
         IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) ' ---> Bucholtz-95'
         VV = POS*1.0D-3 
         IDI = 1  ;  IF( VV > 0.5d0 ) IDI = 2         
         Ray_sec = HF0(IDI)/VV**(HF1(IDI) + HF2(IDI)*VV + HF3(IDI)/VV)

      CASE(3)                   ! Bodhaine approximation for 360 ppm CO2 
         IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) ' ---> Bodhaine-99'
         VV = POS*POS*1.0D-6 
         Ray_sec = (Ba + Bb/VV + Bc*VV)/(1.0d0 + Bd/VV + Be*VV)*1.0d-28

      CASE(4)
         IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,*) ' ---> User defined'
c        ... Select Refractive index:
         N_Lo = Ln_15C ; Cor_CO2 = 1.d0 + 0.54d0*(PVC(4) - 0.0003d0)
         SELECT CASE( TRIM(Refr_index) )
         CASE('Edlen53')
            Refi = 1.d0 + (Refr_air_E53(POS) - 1.d0)*Cor_CO2
         CASE('Edlen66')
            Refi = 1.d0 + (Refr_air_E66(POS) - 1.d0)*Cor_CO2
         CASE('Peck_Reeder72')
            Refi = 1.d0 + (Refr_air_PR(POS) - 1.d0)*Cor_CO2
         CASE('Bates84')
            N_Lo = Ln_0C
            R4(1) = Refr_N2(POS)-1.d0  ;  R4(2) = Refr_O2(POS) -1.d0
            R4(3) = Refr_Ar(POS)-1.d0  ;  R4(4) = Refr_CO2(POS)-1.d0
            Refi = 1.d0 + SUM( PVC(:)*R4(:) )
         END SELECT
c        ... Select Lorentz-Lorenz factor:
         SELECT CASE( LL_factor )
         CASE('a')
            S2 = 24.d0*Pie**3/(POS*1.d-07)**4/N_Lo**2
     $           *( (Refi**2 - 1.d0)/(Refi**2 + 2.d0) )**2
         CASE('b')
            S2 = 32.d0*Pie**3/3.d0/(POS*1.d-07)**4/N_Lo**2
     $           *(Refi - 1.d0)**2
         CASE('c')
            S2 = 8.d0*Pie**3/3.d0/(POS*1.d-07)**4/N_LO**2
     $           *(Refi**2 - 1.d0)**2
         CASE('t')
C           ... Compatibility mode (SCIATRAN 2) 
            VV = 1.0D0/(POS * POS * 1.0D-6)
            S2 = 3.307336D+06*(70.41D0 + 31590.0D0/(157.39D0 - VV)
     $           + 841.27D0/(50.429D0 - VV))**2*VV*VV/
     $           (2.686763d11)**2*1d-16
         END SELECT
         Ray_sec = S2*(6.d0 + 3.d0*DEPOL)/(6.d0 - 7.d0*DEPOL) 
      END SELECT

cc     ... Test with Bodhaine approximation (Ring comparison study)
c      VV = POS*POS*1.0D-6 
c      Ray_sec = (Ba + Bb/VV + Bc*VV)/(1.0d0 + Bd/VV + Be*VV)*1.0d-28

      RETURN
      END FUNCTION

c **********************************************************************
      SUBROUTINE KING_ACTUAL(POS, DEPOL, *)
c     ... Called by "rtm_init.f", "dom_spherical.f",
c                   "st_vod.f", and "spher_rad.f"

      USE ANISOTROPY, ONLY: DEPOL_INPUT, DEP_NWS, DEP_WS, DEP_VS
      USE LOG_PARS,   ONLY: LUNIT, N_ESO, LOG_LEVEL, N_DSO
      USE ST_VARS,    ONLY: King_factor, PVC
      USE TYPE_VARS,  ONLY: KND
c     ==================================================================

      REAL (KND), INTENT(IN)  :: POS
      REAL (KND), INTENT(OUT) :: DEPOL
c     ... Local variables
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'KING_ACTUAL'
      LOGICAL ERROR
      REAL (KND)  Fk, S1, S2, POS_TMP(1), 
     $     DEPOL_TMP(1), R4(4), Ki(4) !, PVC_TES(4)
      REAL (KND), EXTERNAL :: Refr_N2, Refr_O2, Refr_Ar, 
     $     Refr_CO2, King_N2, King_O2, King_Ar, King_CO2
c    ===================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
 
      SELECT CASE( TRIM(King_factor) )
      CASE('constant')
         DEPOL = DEPOL_INPUT
c         write(*,*) '  const ', DEPOL
      CASE('effective')
         R4(1) = Refr_N2(POS) - 1.d0  ;  R4(2) = Refr_O2(POS)  - 1.d0
         R4(3) = Refr_Ar(POS) - 1.d0  ;  R4(4) = Refr_CO2(POS) - 1.d0
         Ki(1) = King_N2(POS) ; Ki(2) = King_O2(POS)
         Ki(3) = King_Ar(POS) ; Ki(4) = King_CO2(POS)
         S1 = SUM( PVC(:)*R4(:)**2*Ki(:) )
         S2 = SUM( PVC(:)*R4(:) )
         Fk = S1/S2**2            
         DEPOL = 6.d0*(Fk - 1.d0)/(3.d0 + 7.d0*Fk)
c         write(*,*) '  effe ', Fk, DEPOL
      CASE('weighted')
c         PVC_TES = PVC 
c         PVC_TES(4) = 0.0003d0
         Ki(1) = King_N2(POS) ; Ki(2) = King_O2(POS)
         Ki(3) = King_Ar(POS) ; Ki(4) = King_CO2(POS)
         Fk = SUM( PVC(:)*Ki(:) )/SUM( PVC(:) )
c         Fk = SUM( PVC_TES(:)*Ki(:) )/SUM( PVC_TES(:) )
         DEPOL = 6.d0*(Fk - 1.d0)/(3.d0 + 7.d0*Fk)
c         write(*,*) '  wei ', Fk, DEPOL
      CASE('from file')
         POS_TMP(1) = POS
         CALL LINTP2(DEP_NWS, DEP_WS, DEP_VS, 1, POS_TMP, DEPOL_TMP, 
     $        ERROR) 
         IF( ERROR ) RETURN 1
         DEPOL = DEPOL_TMP(1)
c         write(*,*) '  file ', DEPOL
cc         ... Test with constant depolarization factor (Ring comparison study)
c         DEPOL = 0.0295d0
      END SELECT

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE KING_ACTUAL
c *************************************************************************

      DOUBLE PRECISION FUNCTION Refr_N2 (X)
      DOUBLE PRECISION   X, DC, W_loc, DWL
      INTEGER            IDI
      LOGICAL            ERROR
c     ... Number of spectral intervals where coefficients for the 
c         approximation of N2 refractive index are given
      INTEGER, PARAMETER :: NI_N2 = 3, Nwl = NI_N2 + 1
c     ... Wavelengths in micrometer for these intervals
      DOUBLE PRECISION WL_N2(Nwl)
      DATA WL_N2 /0.0d0, 0.254d0, 0.468d0, 30.0d0/
c     ... Coefficients in the approximation of N2 refractive index
      DOUBLE PRECISION A_N2(NI_N2), B_N2(NI_N2), C_N2
      DATA A_N2/6998.749d0, 5989.242d0, 6855.200d0/
      DATA B_N2, C_N2 
     $     /3233582.0d0, 3363266.3d0, 3243157.0d0, 144.0d0/
      DOUBLE PRECISION SIG, AV
c      DATA SIG, AV /4.0d-03, 2.27684009d0/
      DATA SIG, AV /3.0d-03, 2.27684009d0/
c     ==================================================================

      W_loc = X*1.0d-03         ! Transform to micrometer
      CALL LIN_SEARCH(W_loc, DC, IDI, Nwl, WL_N2, ERROR) 
      
      DWL = 0.468d0 - W_loc

cc     ... Original version of Bates
c      Refr_N2 = 1.d0 
c     $     + (A_N2(IDI) + B_N2(IDI)/(C_N2 - 1.d0/W_loc**2))*1.d-08

      Refr_N2 = 1.d0 
     $     + ( A_N2(IDI) + B_N2(IDI)/(C_N2 - 1.d0/W_loc**2) 
     $     + AV*SIGN( DEXP(-DABS(DWL)/sig),DWL ) )*1.d-08

      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION Refr_O2 (X)
      DOUBLE PRECISION   X, DC, W_loc
      INTEGER            IDI
      LOGICAL            ERROR
c     ... Number of spectral intervals where coefficients for the 
c         approximation of O2 refractive index are given
      INTEGER, PARAMETER :: NI_O2 = 4, Nwl = NI_O2 + 1
c     ... Wavelengths in micrometer for these intervals
      DOUBLE PRECISION WL_O2(Nwl)
      DATA WL_O2 /0.0d0, 0.221d0, 0.288d0, 0.546d0, 30d0/
c     ... Coefficients in the approximation of O2 refractive index
      DOUBLE PRECISION A_O2(NI_O2), B_O2(NI_O2), C_O2
      DATA A_O2 /23796.7d0, 22120.4d0, 20564.8d0, 21351.1d0/
      DATA B_O2, C_O2 
     $     /168988.4d0, 203187.6d0, 248089.9d0, 218567.0d0, 40.9/
      W_loc = X*1.0d-03         ! Transform to micrometer
      CALL LIN_SEARCH(W_loc, DC, IDI, Nwl, WL_O2, ERROR) 
      Refr_O2 = 1.0d0 
     $     + (A_O2(IDI) + B_O2(IDI)/(C_O2 - 1.d0/W_loc**2))*1.d-08
      RETURN
      END FUNCTION
c *************************************************************************
      DOUBLE PRECISION FUNCTION Refr_Ar (X)
      DOUBLE PRECISION   X, W_loc
      W_loc = X*10.d0           ! Transform to angstrem
      Refr_Ar = DSQRT( 1.d0 + 
     $    5.547d-04*(1.D0 + 5.15d05/W_loc**2 + 4.19d11/W_loc**4) )
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION Refr_CO2 (X)
      DOUBLE PRECISION   X, W_loc
      W_loc = X*1.0d-03         ! Transform to micrometer
      W_loc = 1.d0/W_loc/W_loc
      Refr_CO2 = 1.d0 + ( 22822.1d0 + 117.8d0*W_loc 
     $     + 2406030.d0/(130.d0 - W_loc)
     $     + 15997.d0/(38.9d0 - W_loc) )*1.d-08
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION King_N2 (X)
      DOUBLE PRECISION  X, W_loc
c     ... Coefficients for N2 King factor as a function of wavelength
      DOUBLE PRECISION, PARAMETER:: KN1 = 1.034d0, KN2 = 3.17d-04      
      W_loc = X*1.0d-03
      King_N2 = KN1 + KN2/W_loc**2
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION King_O2 (X)
      DOUBLE PRECISION  X
c     ... Coefficients for O2 King factor as a function of wavelength
      DOUBLE PRECISION, PARAMETER::
     $     KO1 = 1.096d0, KO2 = 1.385d-03, KO3 = 1.448d-04
      King_O2 = KO1 + KO2/X**2*1.D6 + KO3/X**4*1.D12
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION King_Ar (X)
      DOUBLE PRECISION  X
      King_Ar = 1.d0
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION King_CO2 (X)
      DOUBLE PRECISION  X
      King_CO2 = 1.15d0
      RETURN
      END FUNCTION

c ************************************************************************
      DOUBLE PRECISION FUNCTION Refr_air_PR (X)
      DOUBLE PRECISION   X, DC, W_loc
      INTEGER  IDI
      LOGICAL  ERROR
c     ... Number of spectral intervals where coefficients for the 
c         approximation of air refractive index are given
      INTEGER, PARAMETER :: NI_PR = 2, Nwl = NI_PR + 1
c     ... Wavelengths in micrometer for these intervals
      DOUBLE PRECISION WL_PR(Nwl)
      DATA WL_PR /0.0d0, 0.23d0, 30d0/
      DOUBLE PRECISION A0(NI_PR), A1(NI_PR), A2(NI_PR), 
     $     B1(NI_PR), B2(NI_PR)
      DATA A0 /8060.51d0, 0.d0/
      DATA A1 /2480990.0d0, 5791817.0d0/
      DATA A2 /17455.7d0, 167909.0d0/
      DATA B1 /132.274d0, 238.0185d0/
      DATA B2 /39.32957d0, 57.362d0/
      W_loc = X*1.0d-03         ! Transform to micrometer
      CALL LIN_SEARCH(W_loc, DC, IDI, Nwl, WL_PR, ERROR) 
c      write(*,*) '  air PR interval :', IDI
      W_loc = 1.d0/W_loc/W_loc
      Refr_air_PR = 1.d0 + ( A0(IDI) + A1(IDI)/(B1(IDI) - W_loc) 
     $     + A2(IDI)/(B2(IDI) - W_loc) )*1.d-08
      RETURN
      END FUNCTION
c **********************************************************************

c     ... Refractive index of air for 15C and 1013.25mb
      DOUBLE PRECISION FUNCTION Refr_air_E53 (X)
      DOUBLE PRECISION   X, W_loc
      DOUBLE PRECISION A0, A1, A2, B1, B2
      DATA A0, A1, A2, B1, B2 
     $     /6432.80d0, 2949810.d0, 25540.d0, 146.d0, 41.d0/
      W_loc = X*1.0d-03         ! Transform to micrometer
      W_loc = 1.d0/W_loc/W_loc
      Refr_air_E53 = 
     $     1.d0 + ( A0 + A1/(B1-W_loc) + A2/(B2-W_loc) )*1.d-08
      RETURN
      END FUNCTION

c *************************************************************************
      DOUBLE PRECISION FUNCTION Refr_air_E66 (X)
      DOUBLE PRECISION   X, W_loc
      DOUBLE PRECISION A0, A1, A2, B1, B2
      DATA A0, A1, A2, B1, B2 
     $     /8342.13d0, 2406030.d0, 15997.d0, 130.d0, 38.9d0/
      W_loc = X*1.0d-03         ! Transform to micrometer
      W_loc = 1.d0/W_loc/W_loc
      Refr_air_E66 = 
     $     1.d0 + ( A0 + A1/(B1-W_loc) + A2/(B2-W_loc) )*1.d-08
      RETURN
      END FUNCTION

c **********************************************************************
      SUBROUTINE KING_COMP(POS, Ki, Ref, Dep, Sec )
c     ... Called by "rtm_init.f"

      USE ST_VARS,    ONLY: Pie, LL_factor
      USE TYPE_VARS,  ONLY: KND
c     ==================================================================

      REAL (KND), INTENT(IN)  :: POS
      REAL (KND), INTENT(OUT) :: Ki(4), Ref(4), Dep(4), Sec(4)
c     ... Local variables
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'KING_COMP'
      INTEGER I
      REAL (KND)  S2, Refi, N_Lo
      REAL (KND), EXTERNAL :: Refr_N2, Refr_O2, Refr_Ar, 
     $     Refr_CO2, King_N2, King_O2, King_Ar, King_CO2
      REAL (KND), PARAMETER :: 
c          Avogadro's number [molecules/mol]
     $     N_Avo = 6.0221367d23, 
c          Molar volume [L/mol] at 0 C and 1013.25 mb  
     $     Mol_V = 22.4141d0, 
c          Loschmidt number [molecules/cm^3] at 
c               0 C, 1013.25 mb             15 C, 1013.25 mb   
     $     Ln_0C = N_Avo/Mol_V*1.d-03, Ln_15C = Ln_0C*273.15d0/288.15d0
c    ===================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
 
      N_Lo = Ln_0C
c     ... Set refractive index
      Ref(1) = Refr_N2(POS) ;  Ref(2) = Refr_O2(POS) 
      Ref(3) = Refr_Ar(POS) ;  Ref(4) = Refr_CO2(POS)
c     ... Set King factor
      Ki(1) = King_N2(POS) ; Ki(2) = King_O2(POS)
      Ki(3) = King_Ar(POS) ; Ki(4) = King_CO2(POS)
c     ... Set depolarization ratio
      Dep(:) = 6.d0*(Ki(:) - 1.d0)/(3.d0 + 7.d0*Ki(:))
c     ... Select Lorentz-Lorenz factor:
      DO I = 1,4
         Refi = Ref(I)
         SELECT CASE( LL_factor )
         CASE('a')
            S2 = 24.d0*Pie**3/(POS*1.d-07)**4/N_Lo**2
     $           *( (Refi**2 - 1.d0)/(Refi**2 + 2.d0) )**2
         CASE('b')
            S2 = 32.d0*Pie**3/3.d0/(POS*1.d-07)**4/N_Lo**2
     $           *(Refi - 1.d0)**2
         CASE('c')
            S2 = 8.d0*Pie**3/3.d0/(POS*1.d-07)**4/N_LO**2
     $           *(Refi**2 - 1.d0)**2
         END SELECT
         Sec(I) = S2 !*(6.d0 + 3.d0*DEP(I))/(6.d0 - 7.d0*DEP(I)) 
      ENDDO
         
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE KING_COMP
c *************************************************************************
