C********************************************************************C
C
C       SCIATRAN - Version 3.7 - Last modification: 18 Feb 2017
C
C       Copyright (C) Institute of Remote Sensing,
C                     University of Bremen, Germany
C       STB - SCIATRAN theoretical background
C********************************************************************C
      SUBROUTINE ST_RRS_SETUP(*)
c     ... Call in "rtm_init.f"

      USE ST_VARS,   ONLY: Pie
      USE LOG_PARS,  ONLY: EU
      USE GRIDS,     ONLY: NLAYERS
      USE PROFILES,  ONLY: TEMP
      USE RAMAN,     ONLY: N_WL_C, O2_CONC, PPMV_N2_O2, SPIN_WEI,
     $                     CtimesH, RAMAN_B0, Boltzmann, ESR_Split,
     $                     R_JB => RAMAN_JBEG, Jla => NR_LINES, 
     $                     R_JD => RAMAN_JDELTA, N_RG, SIGMA_RR,
     $                     DEL_WL, RRS_NAM_ET, RRS_NAM_PT
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------
      
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_RRS_SETUP'
      INTEGER   I, J, Jr, I_GAS, N_J_MAX, Jfi, Jst
      REAL (KND)  CONC(N_RG,NLAYERS)
      REAL (KND), ALLOCATABLE :: N_j(:), StateN(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

c     ... O2_CONC is calculated in "gt_ptconc_setup.f"
      CONC(1,:) = O2_CONC(:)
      CONC(2,:) = O2_CONC(:)*PPMV_N2_O2

      N_J_MAX = MAXVAL((Jla - R_JB(:))/R_JD(:) + 1)

      If(Allocated(SIGMA_RR)) DeAllocate(SIGMA_RR)
      ALLOCATE( N_J(0:N_J_MAX), StateN(NLAYERS,0:N_J_MAX), ! A
     $     SIGMA_RR(N_WL_C,NLAYERS) ) ! A

      StateN = 0.d0
      Jr = 0
      IF( ESR_Split ) THEN
         CALL Calc_O2_coef(1)
         CALL Calc_RRS_coef(2)
      ELSE
         DO I_GAS = 1,N_RG
            CALL Calc_RRS_coef(I_GAS)
         END DO  ! over O2 and N2 (I_GAS)

cc ***************** Test output ****************************
c         WRITE(*,*) '    Sigma_RR in rrs_setup'
c         OPEN(44, file='./DATA_OUT/Ram_cof.tes' )
c         DO I = 1,25
c            WRITE(44,*) I, DEL_WL(I), Sigma_RR(I,1)
c         ENDDO
c         CLOSE(44)
cc         stop
cc ***************** End test output ************************
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c **********************************************************************
      SUBROUTINE Calc_RRS_coef(I_GAS)
      INTEGER, INTENT(IN) :: I_GAS
      REAL (KND)  StateSum, PLACZEK, DUM

      DUM = CtimesH*RAMAN_B0(I_GAS)/Boltzmann
      Jfi =  R_JB(I_GAS) ; Jst = R_JD(I_GAS)
c        ... 1st gas - O2 : Jfi = 1, Jst = 2
c            2nd gas - N2 : Jfi = 0, Jst = 1
      DO I = 1, NLAYERS
         DO J = Jfi,Jla,Jst
            N_j(J) = SPIN_WEI(I_GAS,MOD(J,2) + 1)*
     $           (2*J + 1.d0)*DEXP(-J*(J + 1.d0)*DUM/TEMP(I))
         END DO 
         StateSum = SUM( N_j(Jfi:Jla:Jst) )
c           ... Fractional population of initial states
         StateN(I,Jfi:Jla:Jst) = N_j(Jfi:Jla:Jst)/StateSum
      ENDDO
c        ... RRS coefficient for Stokes and anti-Stokes line
      DO J = Jla,Jfi,-Jst
c            WRITE(*,*) ' Stokes ', J, SUM(StateN(:,J))
         Jr = Jr + 1
c           ... Placzek-Teller for Stokes lines J -> J + 2
         PLACZEK = 1.5d0*(J + 1.d0)*(J + 2.d0)/(2*J+1.d0)/(2*J+3.d0)
c           ... Rotational Raman scattering coefficient without WL_FACTOR,  
c               WL^4, and gamma^2 (anisotropy of polarizability)
c               (see Eq.(Ram-cross) in STB)
         Sigma_RR(Jr,:) = CONC(I_GAS,:)*StateN(:,J)*PLACZEK
      END DO
      DO J = Jfi,Jla,Jst
         IF(J < 2) CYCLE
         Jr = Jr + 1
c           ... Placzek-Teller for anti-Stokes lines J -> J - 2
         PLACZEK = 1.5D0*J*(J - 1.d0)/(2*J + 1.d0)/(2*J - 1.d0) 
c           ... Rotational Raman scattering coefficient without WL_FACTOR, 
c               WL^4, and gamma^2 (anisotropy of polarizability)
c               (see Eq.(Ram-cross) in STB)
         Sigma_RR(Jr,:) = CONC(I_GAS,:)*StateN(:,J)*PLACZEK
      END DO

      END SUBROUTINE Calc_RRS_coef
c **********************************************************************

      SUBROUTINE Calc_O2_coef(I_GAS)
      INTEGER, INTENT(IN) :: I_GAS
      INTEGER  I, C_UNIT, Ne_bas, N_tra
c      CHARACTER (LEN=80) INP_NAME
      REAL (KND)  C2
      REAL (KND), ALLOCATABLE :: E_bas(:), J_bas(:), StateSum(:),
     $     E_tra(:), J_tra(:), De_tra(:), PTC(:), NJ(:,:)
c     ------------------------------------------------------------------

      C2 = 1.438769d0     
c      INP_NAME = '../../DATA_BASES/RRS/o2_ET-KC.dat'
c      INP_NAME = '../../DATA_BASES/RRS/O2_ET-HITRAN-2012.dat'
      C_UNIT = 44
      OPEN(C_UNIT,FILE=TRIM(RRS_NAM_ET),STATUS='OLD')
      CALL SKIP_HEADER (C_UNIT) 
         READ(C_UNIT,*) Ne_bas
         ALLOCATE( E_bas(Ne_bas), J_bas(Ne_bas), StateSum(NLAYERS) )
         DO I = 1, Ne_bas
            READ(C_UNIT,*) J_bas(I), E_bas(I)
         ENDDO
      CLOSE(C_UNIT)

      DO I = 1, NLAYERS
         StateSum(I) = SUM( 
     $        (2*J_bas(:) + 1.d0)*DEXP(-C2*E_bas(:)/TEMP(I)) )
      ENDDO

c      INP_NAME = '../../DATA_BASES/RRS/o2_PT-KC.dat'
c      INP_NAME = '../../DATA_BASES/RRS/O2_PT-HITRAN-2012.dat'
      C_UNIT = 44
      OPEN(C_UNIT,FILE=TRIM(RRS_NAM_PT),STATUS='OLD')
      CALL SKIP_HEADER (C_UNIT) 
         READ(C_UNIT,*) N_tra
         ALLOCATE( E_tra(N_tra), J_tra(N_tra), De_tra(N_tra), 
     $        PTC(N_tra), NJ(NLAYERS,N_tra)  )
         DO I = 1, N_tra
            READ(C_UNIT,*) 
     $           J_tra(I), E_tra(I), De_tra(I), PTC(I) 
         ENDDO
      CLOSE(C_UNIT)

      DO I = 1, NLAYERS
         NJ(I,:) = (2*J_tra(:) + 1.d0)*DEXP(-C2*E_tra(:)/TEMP(I))
     $        /StateSum(I)
      ENDDO
      DO J = 1, N_tra
         Jr = Jr + 1         
         Sigma_RR(Jr,:) = CONC(I_GAS,:)*PTC(J)*NJ(:,J)
      ENDDO
cc ***************** Test output ****************************
c      WRITE(*,*) '    Sigma_RR in rrs_setup'
c      OPEN(44, file='./DATA_OUT/Ram_cof.back' )
c         DO I = 1,N_tra
c            WRITE(44,*) I, DEL_WL(I), Sigma_RR(I,1)
c         ENDDO
c      CLOSE(44)
cc      stop
c ***************** End test output ************************
      END SUBROUTINE Calc_O2_coef

c **********************************************************************
      END

