C********************************************************************C
C
C       SCIATRAN - Version 3.7 - Last modification: 18 Feb 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C       STB - SCIATRAN theoretical background
C********************************************************************C
      SUBROUTINE ST_RRS_WL(*)
c     ... Call in "rtm_main.f"

      USE ST_VARS,   ONLY: Pie
      USE LOG_PARS,  ONLY: EU
      USE RAMAN,     ONLY: N_WL_C, RAMAN_B0, DEL_WL, LINE_GAS, 
     $                     R_JB => RAMAN_JBEG, Jla => NR_LINES, 
     $                     R_JD => RAMAN_JDELTA, N_RG, WL_FACT,
     $                     ESR_Split, RRS_NAM_ET, RRS_NAM_PT
      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------
      
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_RRS_WL'
      INTEGER   I, J, Jr, I_GAS, Jfi, Jst
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Jr = 0
      IF( ESR_Split ) THEN
cc        ... Read O2 transitions energy
         CALL Read_Energy_Transitions(1)
c        ... Calculate N2 transitions energy
         CALL Calc_Energy_Transitions(2)
      ELSE
         N_WL_C = SUM(2*((Jla - R_JB(:))/R_JD(:) + 1) - IDIM(2,R_JB(:)))
         If(Allocated(DEL_WL)) DeAllocate(Del_Wl, Line_Gas)
         ALLOCATE( DEL_WL(N_WL_C), LINE_GAS(N_WL_C) ) ! A
         DO I_GAS = 1, N_RG
            CALL Calc_Energy_Transitions(I_GAS)
         ENDDO
      ENDIF
c     ... Save constant multiplier in the expression for Raman 
c         scattering cross-section (see Eq.(Ram-cross) in STB)
      WL_FACT = 256.d+4*Pie**5.d0/27.d0*1.d+36

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1

      CONTAINS
c **********************************************************************
      SUBROUTINE Calc_Energy_Transitions(I_GAS)
      INTEGER, INTENT(IN) :: I_GAS

      Jfi =  R_JB(I_GAS) ; Jst = R_JD(I_GAS)
c     ... 1st gas - O2 : Jfi = 1, Jst = 2
c         2nd gas - N2 : Jfi = 0, Jst = 1
      DO J = Jla,Jfi,-Jst
         Jr = Jr + 1
         LINE_GAS(Jr) = I_GAS
c        ... Energy shift for Stokes lines. Rotational constants
c            RAMAN_B0(I_GAS) are given in "modules.f" in [m-1].
         DEL_WL(Jr) = -RAMAN_B0(I_GAS)*(4.d0*J + 6.d0) 
      END DO  ! over energy levels (J)
      DO J = Jfi,Jla,Jst
         IF(J < 2) CYCLE
         Jr = Jr + 1
         LINE_GAS(Jr) = I_GAS
c        ... Energy shift for anti-Stokes lines
         DEL_WL(Jr) = RAMAN_B0(I_GAS)*(4.d0*J - 2.d0)
      END DO  ! over energy levels (J)

      END SUBROUTINE Calc_Energy_Transitions
c **********************************************************************

      SUBROUTINE Read_Energy_Transitions(I_GAS)
      INTEGER, INTENT(IN) :: I_GAS
      INTEGER  I, C_UNIT,  N_tra, N_N2
c      CHARACTER (LEN=80) INP_NAME
      REAL (KND)  C2
      REAL (KND), ALLOCATABLE ::  E_tra(:), J_tra(:), De_tra(:),
     $     PTC(:)

c      INP_NAME = '../../DATA_BASES/RRS/o2_PT-KC.dat'
c      INP_NAME = '../../DATA_BASES/RRS/O2_PT-HITRAN-2012.dat'
      C_UNIT = 44
      OPEN(C_UNIT,FILE=TRIM(RRS_NAM_PT),STATUS='OLD')
      CALL SKIP_HEADER (C_UNIT) 
         READ(C_UNIT,*) N_tra
         ALLOCATE( E_tra(N_tra), J_tra(N_tra), De_tra(N_tra), 
     $        PTC(N_tra) )
         DO I = 1, N_tra
            READ(C_UNIT,*) 
     $           J_tra(I), E_tra(I), De_tra(I), PTC(I) 
         ENDDO
      CLOSE(C_UNIT)

      N_N2 = 2* ((Jla - R_JB(2))/R_JD(2) + 1) - IDIM(2,R_JB(2))

      N_WL_C = N_tra + N_N2
      ALLOCATE( DEL_WL(N_WL_C), LINE_GAS(N_WL_C) ) ! A

      DO I = 1,N_tra
         Jr = Jr + 1 ; LINE_GAS(Jr) = I_GAS
c        ... Transform to [m-1]
         DEL_WL(Jr) = - De_tra(I) *1.d+2
      ENDDO

      END SUBROUTINE Read_Energy_Transitions

c **********************************************************************
      END

