C********************************************************************C
C
C       SCIATRAN - Version 3.9 - Last modification: 17 Dec 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c *********************************************************************
      SUBROUTINE ST_SFL_WF(NSZA, NLSA, Mh, JPOS, POS, E_W, FLUX_D, EXL, 
     $     Spe)
c     ... Call in "st_adjoint_a.f"

      USE INFO,       ONLY: WF_DB
      USE IRRAD,      ONLY: SOL_FLUX, WL_IRRAD_DIM
      USE FLU_LAND,   ONLY: FLU_SCAL, USE_DGF, DGF_PAR, GAU
      USE RTM_PARS,   ONLY: DATA_ITER
      USE ST_VARS,    ONLY: N_Adj, Pie, GeoD => GeoD_OUT
      USE SPHER_PARS, ONLY: JV_S, JV_E, N_LOS_OUT, N_SOL_ANG, N_AZ_VAR
      USE TOOLS,      ONLY: FIND_IDX, SET_PTR
      USE TYPE_VARS,  ONLY: KND
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NSZA, NLSA, Mh, JPOS
      REAL (KND), INTENT(IN) :: E_W, EXL(NLSA), FLUX_D(N_Adj), Spe, POS
c     ... Local parameters
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_SFL_WF'
      INTEGER     I, J, K, IDX, II, Jv, IA
      REAL (KND)  DUM, Phot, LO_WF(N_LOS_OUT)
      TYPE (DATA_ITER), POINTER :: RTM_PTR
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')
      IA = 1
      IDX = FIND_IDX('sfl', WF_DB%NAME)

      IF( Mh == 1 ) THEN
         J = 0
         DO I = 1,NSZA
            DO Jv = Jv_S(I),Jv_E(I)  ;  J = J + 1
               LO_WF(J) = FLUX_D(Jv)
               IF( GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0 ) 
     $              LO_WF(J) = LO_WF(J) + EXL(Jv)      ! reflected radiation
            ENDDO
         ENDDO
         LO_WF(:) = LO_WF(:)*E_W
c         IF( WF_DB(IDX)%DIM == 'rel' )  LO_WF(:) = FLU_SCAL*LO_WF(:)
      ELSE
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      Phot = 1.d0
      IF( TRIM(WL_IRRAD_DIM(2))=='phot/s' ) Phot = POS

      DO II = 1, UBOUND(WF_DB,1)

         CALL SET_PTR( RTM_PTR, WF_DB(II)%NAME)

         SELECT CASE (WF_DB(II)%NAME)
         CASE ('flu_sfl')
            DUM = SPE
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_amp-1')
            DUM = GAU(1,POS)*Phot/SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_wlm-1')
            DUM = GAU(1,POS)*2.d0*DLOG(2.d0)*DGF_PAR(1)%Amp*Phot
     $           *(POS - DGF_PAR(1)%Wlm)/DGF_PAR(1)%Hwhm**2
     $           /SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_hwhm-1')
            DUM = GAU(1,POS)*2.d0*DLOG(2.d0)*DGF_PAR(1)%Amp*Phot
     $           *(POS - DGF_PAR(1)%Wlm)**2/DGF_PAR(1)%Hwhm**3 
     $           /SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_amp-2')
            DUM = GAU(2,POS)*Phot/SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_wlm-2')
            DUM = GAU(2,POS)*2.d0*DLOG(2.d0)*DGF_PAR(2)%Amp*Phot
     $           *(POS - DGF_PAR(2)%Wlm)/DGF_PAR(2)%Hwhm**2 
     $           /SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE('flu_hwhm-2')
            DUM = GAU(2,POS)*2.d0*DLOG(2.d0)*DGF_PAR(2)%Amp*Phot
     $           *(POS - DGF_PAR(2)%Wlm)**2/DGF_PAR(2)%Hwhm**3 
     $           /SOL_FLUX(JPOS)*Pie
            IF(WF_DB(II)%DIM=='rel')  DUM = RTM_PTR%CRN(1)*DUM
         CASE DEFAULT
            CYCLE
         END SELECT

         WF_DB(II)%WF(JPOS,:,1,1) = WF_DB(II)%WF(JPOS,:,1,1)
     $        + LO_WF(:)*DUM     
      ENDDO  ! over weighing functions (II)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
