C***********************************************************************
C
C       SCIATRAN - Version 4.3 - Last modification: 27 Jan 2021
C
c       Copyright (C) - Institute of Remote Sensing,
c                       University of Bremen, Germany
C
C***********************************************************************
      SUBROUTINE ST_SPH_RRS( NN, NSZA, Mh, JPOS, JJESFT, E_W, DFI, 
     $     AFI, EXL, EXS, Muv, LOA, SRR, SRR_Jact, SWF )
c     ... Call in "st_adjoint_a.f"

      USE ABSORBERS,   ONLY: N_GAS
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, SOL_GRD
      USE GT_POLYN,    ONLY: NLG => NLEGEN, XM, AXM, AG, AG1
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: SOL_FLUX 
      USE LOG_PARS,    ONLY: LUNIT
      USE RAMAN,       ONLY: Ram_Egg, Ram_EgL, Ram_Egs, Ram_EsL,
     $                       WL_FACT, LINE_GAS, Sigma_RR, Phot_to_Wat
      USE RTM_OUTPUT,  ONLY: N_OP, DEL_RRS
      USE ST_VARS,     ONLY: F14, N_Adj, Pie, APPR_RRS, WL_GRID, TSM, 
     $                       N_Eff, Vec_sca, VS_HC, VST, Finc, Nc_der,
     $                       NS_wf
      USE SPHER_PARS,  ONLY: JV_S, JV_E, N_AZ_VAR, ANG_OUT, 
     $                       NLO => N_LOS_OUT, NFA => N_FOV_ANG,
     $                       NLSA => N_LOS_ANG, N_SOL_ANG
      USE TOOLS,       ONLY: E_CSZA
      USE TYPES,       ONLY: KND, LOA_TYPE, RRS_TYPE, SWF_type
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) ::  NN, NSZA, Mh, JPOS, JJESFT, SRR_Jact
c     --------------------------
      TYPE (LOA_type) :: LOA    ! Local optical parameters atmosphere
      TYPE (RRS_type) :: SRR    ! RRS related parameters
      TYPE (SWF_type) :: SWF    ! "Spherical" weighting function
c     ------------------------
      REAL (KND), INTENT(IN) :: E_W, Muv(NLSA)
      REAL (KND), TARGET :: AFI(NL,2*NN*Nvs,N_Adj), EXL(NL,NLSA),
     $     DFI(NL,NSZA,2*NN*Nvs), EXS(NL,NSZA)
c     ... Local variables      
c     -------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_SPH_RRS'
      INTEGER   I, J, K, K1, L, Iae, Ita, Ja, JL, Js, Jv, Rli, J_act,
     $      Iac, NNv, K_s, Jw, Jw1, Jwa, Mli, Kli
      REAL (KND)  IRR, IR0, MULT, Ewei, S1, S3(Nvs), S4(Nvs), Cz
      REAL (KND)  COF(NL), COSL(NSZA), RES(N_OP,NLO,NS_wf)
      REAL (KND), DIMENSION (NN*Nvs)        :: XML, AGL, XMT
      REAL (KND), DIMENSION (NN*Nvs,NN*Nvs) :: G_Gp, G_Gm
      REAL (KND), DIMENSION (NL,NLO,NS_wf)  :: T_Wc, Ewf
      REAL (KND), DIMENSION (NN*Nvs,NLO,N_Eff) :: S_Gm, S_Gp
      REAL (KND), DIMENSION (2*NN*Nvs)         :: R_T, Pfo_i, RHS
      REAL (KND), POINTER :: SCR(:), P_MOMS(:), Pad(:,:,:), 
     $     Pfo(:,:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF( ISNAN( SUM(DEL_RRS) ) ) THEN
         STOP '   DEL_RRS in st_sph_rrs'
      ENDIF

      Ewei = E_W  ;  SCR => LOA%SCR  
      COSL(:) = SOL_GRD(1:NSZA)
      P_MOMS => SRR%Ram_MOMS(Mh:3,1)

      NNv = Nvs*NN
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:) ; AGL(:) = AG(:) ; K_s = 1
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
         DO K = 1,NNv  ;  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K1) ;  XMT(K) = XM(K1)
         ENDDO
         K_s = 4
      END SELECT

c     ... In the scalar case (VS_HC = 2 )
c         In the vector case (VS_HC = 1 )
      MULT = VS_HC/2.d0
      IF( Mh > 1 ) MULT = VS_HC
c      WRITE(*,*) '           ====> Mh :', Mh
c     .... Terms depending on the phase function
      SELECT CASE( Vec_sca )
      CASE( 'scal_rte' )
         FORALL( K=1:NNv ) 
           G_Gp(K,:) = Ram_Egg(K,:,1,Mh) *XML(K)*AGL(:)*0.5d0
           G_Gm(K,:) = Ram_Egg(K,:,2,Mh) *XML(K)*AGL(:)*0.5d0
         END FORALL
      CASE( 'vect_rte' )
         FORALL( K=1:NNv ) 
c          ... This is the components of the matrix G(Mi,Mj) 
c              Eq.(Gau-pp) in STB. 
           G_Gp(K,:) = XMT(:)*AGL(:)*0.5d0 
     $        *(Ram_Egg(K,:,1,Mh) + Ram_Egg(K,:,2,Mh)) 
c          ... This is the components of the matrix ^G(Mi,-Mj) 
c              Eq.(Gau-pm) in STB. 
           G_Gm(K,:) = XMT(:)*AGL(:) *0.5d0 
     $          *(Ram_Egg(K,:,1,Mh) - Ram_Egg(K,:,2,Mh))
         END FORALL
      END SELECT

c     ... For all actual wavelengths which are processed at first in the
c         case of forward-adjoint technique (RRS_ADJ = .true.)
c     ------------------------------------------------------------------
      IF( SRR%ACT_WL(JPOS) ) THEN
c        ... Set extraterrestrial solar flux at actual WL corresponding
c            to its position in the array WL_Ext
         IRR = -SOL_FLUX(JPOS)/Pie*MULT
c         IRR = 0.d0

c        ... Save solution of pseudo-forward equation for three harmonics
c             and all actual wavelengths
         SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) = 
     $        SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) + AFI(:,:,:)*E_W
c         WRITE(*,*) '   SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) ', Mh,
c     $        SUM( SRR%ADJ_ACT(:,SRR_Jact,:,:,Mh) )
c        ... Save exponential term along line-of-sight directions
         IF( Mh==1 ) SRR%EXL_ACT(:,SRR_Jact,:) = 
     $        SRR%EXL_ACT(:,SRR_Jact,:) + EXL(:,:)*E_W

c        ... Save solution of the forward equation for three harmonics
c             and all actual wavelengths
         IF( TRIM(TSM)=='speed' ) THEN
            SRR%FOR_ACT(:,SRR_Jact,:,:,Mh) = 
     $           SRR%FOR_ACT(:,SRR_Jact,:,:,Mh) + Dfi(:,:,:)*E_W
            IF( Mh==1 ) SRR%EXS_ACT(:,SRR_Jact,:) = 
     $           SRR%EXS_ACT(:,SRR_Jact,:) + EXS(:,:)*E_W
         ENDIF
c        ... Current number of actual wavelength
         Ita = SRR_Jact
c        ... Set scattering coefficient as Raman total
         COF(:) = SRR%R_TOT*SCR(:)
c        ... AFI and DFI are calculated for current actual wavelength
         Pad => AFI(:,:,:) 
         Pfo => DFI(:,:,:) 
c         WRITE(*,*) '  Pfo ', Mh, SUM( Pfo )
c        ... Perform computations of DEL_RRS(:,:,:) caused by 
c            the Raman scattering energy loss at the current
c            actual wavelength
         CALL SUM_RAM_SPH

c        ... The following actions will be done if 'speed'-option was  
c            chosen and calculations are performed already for last  
c            actual wavelength and all esft-coefficients.
         IF( Ita==SRR%N_ACT.AND.TRIM(TSM)=='speed'.AND. 
     $       JJESFT==SRR%NESFT(SRR%N_ACT) ) THEN
            Ewei = 1.d0
c           ... Over all actual wavelengths to calculate energy gain
            DO Iac = 1, SRR%N_ACT 
c              ... Position of actual WL in the extended WL-grid
               Iae = SRR%A2E(Iac)
               IR0 = SOL_FLUX(Iae)/Pie*MULT
c              ... Over all excitation WL within this bin
               DO J_act = 1,SRR%NUM_IN(Iae)
c                 ... Ita gives the actual WL number for
c                     which contributes this excitation WL
                  Ita = SRR%ACT_IN(Iae,J_act)
                  IRR = IR0
                  IF( Phot_to_Wat ) IRR = IR0*WL_GRID(SRR%A2E(Ita))
     $                 /WL_GRID(Iae)
c                 ... Rli Raman line transition number
                  Rli  = SRR%RIN_IN(Iae,J_act)
c                 ... Raman scattering coefficient
                  COF(:) = WL_FACT*Sigma_RR(Rli,:)
     $                 /WL_GRID(SRR%A2E(Ita))**4
     $                 *SRR%GAM_WL(LINE_GAS(Rli),Iac)
c                 ... Set pointers to the saved above solution 
c                     of forward and adjoint equations
                  Pad => SRR%ADJ_ACT(:,Ita,:,:,Mh)
                  Pfo => SRR%FOR_ACT(:,Iac,:,:,Mh)
c                 ... Contribution to DEL_RRS(:,:,:) due to 
c                     Raman scattering energy gain 
                  CALL SUM_RAM_SPH
               ENDDO  ! over excitation WLs within a bin (J_act)
            ENDDO  ! over actual wavelengths to calculate energy gain
         END IF  ! 'speed'-option 

c     ... For all not empty bins and excitation wavelengths
c     ------------------------------------------------------------------
      ELSE
c        ... Set extraterrestrial solar flux at the corresponding 
c            excitation WL
         IR0 = SOL_FLUX(JPOS)/Pie*MULT
c         IR0 = 0.d0

c        ... Over all excitation WL within this bin
         DO J_act = 1,SRR%NUM_IN(JPOS)
c           ... Ita gives the actual WL number for
c               which contributes this excitation WL
            Ita = SRR%ACT_IN(JPOS,J_act) 
            Iae = SRR%A2E(Ita)
            IRR = IR0
            IF( Phot_to_Wat ) IRR = IR0*WL_GRID(Iae)/WL_GRID(JPOS)
            Rli  = SRR%RIN_IN(JPOS,J_act)
            COF(:) = WL_FACT*Sigma_RR(Rli,:)/WL_GRID(Iae)**4
     $           *SRR%GAM(LINE_GAS(Rli))
c           ... Set pointers to the current solution of forward equations
            Pfo => DFI(:,:,:) 
c           ... Set pointers to the saved above solution of adjoint 
c               equations
            Pad => SRR%ADJ_ACT(:,Ita,:,:,Mh)
c           ... Perform computations of DEL_RRS(:,:,:) caused by 
c               the Raman scattering energy gain 

            CALL SUM_RAM_SPH
         ENDDO
      ENDIF

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c *********************************************************************

      SUBROUTINE SUM_RAM_SPH
c     ... Local parameters
      INTEGER    JL, Ks, KK
      REAL (KND), POINTER :: E1(:,:), E2(:,:)

      DO K1 = 1,NLO
         Ks = (K1-1)*Nvs
         DO JL = 1,SWF%Ief_Sol(K1)
            E1 => SWF%Egs_sph(:,:,JL,1,Mh)
            E2 => SWF%Egs_sph(:,:,JL,2,Mh)
           DO K = 1,NN
               Kk = (K-1)*Nvs
               S_Gp(Kk+1:K*Nvs,K1,JL) = F14*VST*XML(Kk+1:K*Nvs)  
     $              *MATMUL( Finc(1,:),E1(Kk+1:K*Nvs,Ks+1:K1*Nvs) )
               S_Gm(Kk+1:K*Nvs,K1,JL) = F14*VST*XML(Kk+1:K*Nvs)  
     $              *MATMUL( Finc(1,:),E2(Kk+1:K*Nvs,Ks+1:K1*Nvs) )
            ENDDO
            IF( Nvs > 1 ) THEN
               S_Gp(4:NNv:Nvs,K1,JL) = -S_Gp(4:NNv:Nvs,K1,JL)
               S_Gm(3:NNv:Nvs,K1,JL) = -S_Gm(3:NNv:Nvs,K1,JL)
            ENDIF
         ENDDO
         Nullify(E1,E2)
      ENDDO

      T_Wc = 0.d0
      DO I = 1,NL 
         DO K1 = 1, NLO
            Mli = SWF%Ief_Sol(K1) ;  Kli = NS_wf*SWF%Ief_Sum(K1)
            DO JL = 1,SWF%Ief_Sol(K1)
c              ... Contribution from the term < I*d,P_ram >.
c                  Term 2 in Eq.(Ram_pp-fin) of STB
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + Kli + JL
                  RHS(:) = Pad(I,:,Jw1)

c                 ... To check the scalar case
                  IF( Nvs > 1 ) THEN
                     RHS(NNv+3:2*NNv:Nvs)= -RHS(NNv+3:2*NNv:Nvs)
                     RHS(NNv+4:2*NNv:Nvs)= -RHS(NNv+4:2*NNv:Nvs)
                  ENDIF
                  DO K = 1,Nvs
                     S4(K) = SUM( 
     $                    ( RHS(NNv+K:2*NNv:Nvs)*S_gp(K:NNv:Nvs,K1,JL)
     $                    + RHS(    K:NNv  :Nvs)*S_gm(K:NNv:Nvs,K1,JL) )
     $                    *AG(1:NN) )
                  ENDDO
                  IF( Nvs > 1 ) S4(3) = -S4(3)
                  T_Wc(I,K1,Jw) = T_Wc(I,K1,Jw) 
     $                 + SUM( S4(1:K_s) )*SWF%SOL_JL(I,K1,JL)

               ENDDO  ! over RRS Stokes component (Jw)
            ENDDO  ! over sub-layers (JL)
         ENDDO  ! over line-of-sight (K1)
      ENDDO  ! over layers (I)

      DO I = 1,NL 
         DO K1 = 1,NLO
            Mli = SWF%Ief_Sol(K1) ;  Kli = NS_wf*SWF%Ief_Sum(K1)
            DO JL = 1,Mli 
               Cz = E_CSZA(SWF%IEffS(K1,JL),'G')

c              ... Interpolation of the forward MS intensity
c                  at a local solar angle Cz
               DO K = 1,NNv
                  CALL AKIMA_INT( NSZA,COSL,Pfo(I,:,K), 1,Cz,Pfo_i(K) )
                  CALL AKIMA_INT( NSZA,COSL,Pfo(I,:,K+NNv),
     $                 1, Cz, Pfo_i(K+NNv) )
               ENDDO
               R_T(1:NNv) = MATMUL( G_Gp(:,:),Pfo_i(1:NNv) )
     $              + MATMUL( G_Gm(:,:),Pfo_i(NNv+1:2*NNv) )
               R_T(NNv+1:2*NNv) = MATMUL( G_Gm(:,:),Pfo_i(1:NNv) )
     $              + MATMUL( G_Gp(:,:),Pfo_i(NNv+1:2*NNv) )
               DO Jw = 1,NS_wf
                  Jw1 = (Jw - 1)*MLI + Kli + JL
                  S3 = 0.d0 
                  DO K = 1,Nvs
                     S3(K) = SUM( 
     $              ( R_T(    K:NNv  :Nvs)*Pad(I,NNv+K:2*NNv:Nvs,Jw1)
     $              + R_T(NNv+K:2*NNv:Nvs)*Pad(I,    K:  NNv:Nvs,Jw1) )
     $                    *AG(1:NN) )
                  ENDDO 
c                ... We do not need to change the sign of 3th component 
c                    in the inner product because we have not multiplied 
c                    R_T(NNv+1:2*NNv) and AFI(I,NNv+1:2*NNv,Jv). However,
c                    the results should multiplied by D4
                  IF( Nvs > 1 ) S3(4) = -S3(4)
                  T_Wc(I,K1,Jw) = T_Wc(I,K1,Jw) + SUM( S3(1:K_s) )
               ENDDO  ! over ring Stokes component (Jw)
            ENDDO  ! over "limb" sub-layers (JL)
         ENDDO  ! over line-og-sight (K1)
      ENDDO  ! over layers (I)

      FORALL( K=1:NLO, Jw=1:NS_wf ) Ewf(1:NL-1,K,Jw) = 
     $     ( T_Wc(1:NL-1,K,Jw)*COF(1:NL-1)
     $     + T_Wc(2:  NL,K,Jw)*COF(2:NL) )*0.5d0*DZR(1:NL-1)
         
      FORALL( K=1:NLO, Jw=1:NS_wf ) 
     $     RES(1,K,Jw) = SUM( Ewf(1:NL-1,K,Jw) )

c     ... Contribution of inelastic RRS terms to the Stokes vector
      DO K1 = 1,NLO
         DO Jw = 1,NS_wf  ;  Jwa = Nc_der(Jw)
            IF( Jwa > 2 ) THEN
               DEL_RRS(Ita,K1,Jwa) = DEL_RRS(Ita,K1,Jwa)
     $              + IRR*RES(1, K1,Jw)*Ewei
     $              *DSIN( ANG_OUT(3,K1)*DBLE(Mh-1) )
            ELSE
               DEL_RRS(Ita,K1,Jwa) = DEL_RRS(Ita,K1,Jwa)
     $              + IRR*RES(1, K1,Jw)*Ewei
     $              *DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) )
            ENDIF
         ENDDO  ! over ring Stokes component (Jw)
      ENDDO  ! over l-of-s (K1)
      
      RETURN
      END SUBROUTINE SUM_RAM_SPH
         
c **********************************************************************
      END 
