C********************************************************************C
C
C	SCIATRAN - Version 4.4 - Last modification: 09 May 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
c *********************************************************************
      SUBROUTINE ST_SPH_WF(JPOS, NN, NSZA, Mh, PWF, LOA, DFI, AFI, 
     $     S_GS, ALB, FLUX_D, EXL, A_GG, Ph_dA, R_GG, Pol_ga, SWF, 
     $     ADD_PLANCK, NPLANCK, S_PLA, LWF, PH_M, Ref_surf, LBC, UDB)
c     ... Called by "st_adjoint_a.f"

      USE AEROSOLS,    ONLY: LA_N, L1 => LA_S, L2 => LA_E, Dwr_mi, N_dwr
      USE BRDF,        ONLY: Npg
      USE CLOUD_INPUT, ONLY: Plus_ice
      USE GRIDS,       ONLY: NL => NLAYERS, DZR, DZ, SOL_GRD, ZZZ
      USE GT_POLYN,    ONLY: NLEGEN, XM, AG, AG1
      USE INTERN_DIMS, ONLY: NL_AER, NLG_AER, NL_pla, Nm_vs, Nvs, 
     $                       Nlg_Ray, Nl_Ar, Ng_Ar, N_A2
      USE LOG_PARS,    ONLY: LUNIT, N_ESO, LOG_LEVEL, N_DSO
      USE RTM_OUTPUT,  ONLY: WF_INTEGR
      USE ST_VARS,     ONLY: N_Eff, PIE, F14, GeoD => GeoD_OUT,
     $                       M_LIM, SHI_LIM_C, SCATT_MODE, WF_MS_FLAG,
     $                       Vec_sca, NS_wf, Nc_der, VST, N_RHS, SDVI, 
     $                       N_Adj, Finc, Ray_to_WF
      USE SPHER_PARS,  ONLY: ANG_OUT, NLSA => N_LOS_ANG, LW_FOV,
     $                       N_SOL_ANG, N_AZ_VAR, NFA => N_FOV_ANG,  
     $                       JV_S, JV_E, NLO => N_LOS_OUT, ONE_MS_SZ
      USE TOOLS,       ONLY: INQ_WF, E_CSZA
      USE TYPES,       ONLY: KND, LOA_TYPE, LWF_TYPE, PWF_TYPE,  
     $                       PHA_MATR, UDB_TYPE, SWF_type
c     ------------------------------------------------------------------

      INTEGER,           INTENT(IN) :: JPOS, NN, NSZA, Mh
      LOGICAL,           INTENT(IN) :: ADD_PLANCK
      CHARACTER (LEN=*), INTENT(IN) :: LBC   
      REAL (KND),        INTENT(IN) :: ALB, NPLANCK(NL_pla), S_PLA,
     $     EXL(NLSA), A_GG(Nl_Ar,Ng_Ar,Ng_Ar,N_A2), FLUX_D(N_Adj), 
     $     R_GG(Nvs*NN,Nvs*NN,2), Pol_ga(Nlegen,2*NN),
     $     DFI(NL,NSZA,2*NN*Nvs), Afi(NL,2*Nvs*NN,N_Adj),
     $     S_GS(NL,Nvs*NN,Nvs*NSZA,2), Ref_surf(N_RHS,Nvs*(NN+Npg))
      TARGET :: S_GS
c     ------------------------------------------------------------------
c     ... Local optical parameters atmosphere |
c     -----------------------------------------
      TYPE (LOA_type), TARGET :: LOA
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (PWF_type) :: PWF    ! General weighting function
      TYPE (PHA_MATR) PH_M
      TYPE (UDB_TYPE) :: UDB  ! Boundary conditions
      TYPE (SWF_type) :: SWF  ! "Spherical" weighting function 
c     ------------------------
      TYPE Dphase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:),
     $     Egs(:,:,:,:,:), EsL(:,:,:,:,:)
      END TYPE Dphase_tdz
      TYPE (Dphase_tdz) :: Ph_dA
c     --------------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_SPH_WF'
      INTEGER   I, J, K, L, Iks, Js, Jv, JL, Ja, Jw, Jw1, K1, Kv, K_s
      INTEGER   NNv, DCH, Ka, Idr, IA
      LOGICAL   R_SURF
      REAL (KND)  Cz, Wup, S2, Wdo, WF_T(NL,NL),
     $     Jad(NL,N_Adj,Ns_wf), R_T(2*Nvs*NN), COSL(NSZA), AO(2*Nvs*NN),
     $     Help(NL,NLO,Ns_wf), XML(Nvs*NN), AGL(Nvs*NN), 
     $     Ext_tes(NL,Ns_wf)
      REAL (KND), DIMENSION (NL) :: Tmp, HH, Jint , Ttt
      REAL (KND), DIMENSION (NL,NN,NN), TARGET :: G_Gp, G_Gm      
      REAL (KND), ALLOCATABLE:: Dum(:), BT(:), PL_ABS(:)
      REAL (KND), POINTER ::  SCA(:),  
     $     WF_ext(:,:,:), WF_OUT_SS(:,:), P01(:,:,:), P02(:,:,:)
c    -------------------------------------------------------------------

      CALL ST_TRACK (R_NAME, 1, 'I')

      IA = 1
      Wup = 0.5d0 ; Wdo = 1.d0 - Wup
      NNv = Nvs*NN
      SELECT CASE( Vec_Sca )
      CASE( 'scal_rte' )
         XML(:) = XM(:) ; AGL(:) = AG(:) ; K_s = 1
      CASE( 'vect_rte' )
         XML(:) = 1.d0  
         DO K = 1,NNv 
            K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )
            AGL(K) = AG(K1) 
         ENDDO
         K_s = 4
      END SELECT

c     ... Preliminary (WF fir "tg" including thermal emission)
      IF( ADD_PLANCK )  THEN
         LWF%WF_TE = 0.d0
         ALLOCATE( Dum(NL), BT(NL), PL_ABS(NL) ) 
         Dum(:) = LOA%SCR(:) + LOA%SCA(:) + LOA%SCC(:)
         IF( ANY(Plus_ice) ) Dum(:) = Dum(:) + LOA%SCI(:)
         PL_abs(NL) = 1.d0
         PL_abs(1:NL-1) = 1.d0 - (Wup*Dum(1:NL-1) + Wdo*Dum(2:NL))
     $        /(Wup*LOA%AL(1:NL-1) + Wdo*LOA%AL(2:NL))
         WHERE( PL_abs(:) > 0.d0 )
c        ... NPLANCK contains product (1 - omega(tau))*Plank_function(tau)
            BT(:) = NPLANCK(:)/PL_abs(:)
         ELSEWHERE
            BT(:) = 0.d0
         END WHERE
c         WRITE(*,*) '   BT :'
c         WRITE(*, '(*(2x,E12.4))') BT(:)
      ENDIF  ! Thermal emission
      
c     ... The array LWF%WF_ext contains final weighting functions
c         if SCATT_MODE = 'ms' 
c                         WF is the sum of WFs calculated in 
c                         this routine and in the routine 
c                         "spher_wf.f" called by "sfi_spher.f"
c                          (array PWF%WF_esp(:,:,:)).
c         if SCATT_MOED = 'wf_ss' It does not work !!!
c                         WF is used as calculated in routine 
c                         "st_ss_stokes.f" (array PWF%WF_esp(:,:,:)).
      WF_ext => LWF%WF_ext

c                   Ph_z%Egs(NL,NNv,Nvs*NSZA,2)
c     ... Second dimension of array Ph_z%Egs(:,|,:,1:2) (S_GS) contains
c         1/XM(:) (see "phase_functions.f"). 
c         In the vector case array ph_z%Egs(:,|,:,1:2) does not contain 1/XM(:).
c         This array contains 
c         discrete representation of the phase matrix Solar -> Gauss. 
      P01 => S_GS(:,:,:,1)  ;  P02 => S_GS(:,:,:,2)

      IF( WF_MS_FLAG(JPOS) ) THEN
c        ... Weighting function related to the attenuation of raidation
         CALL MS_WF_EXT
      ELSE
         WF_ext(:,:,:) = PWF%WF_esp(:,:,:)
      ENDIF                     ! WF in ss-approximation

c     ... Terms depending on the phase function
c -------------------------> Rayleigh <---------------------------------
      IF( Ray_to_WF ) THEN 
         WF_OUT_SS => LWF%WF_scr(:,:,1)
         IF( WF_MS_FLAG(JPOS) ) THEN
c           ... WF in ms-approximation
            DCH = 1
            DO I = 1,NL
               DO K = 1,NN
                  G_Gp(I,K,:) = R_GG(K,:,1)*XM(K)*AG1(:)
                  G_Gm(I,K,:) = R_GG(K,:,2)*XM(K)*AG1(:)
               ENDDO
            ENDDO
            CALL M_POINT_WF_COMP_SPH(1)
         ELSE
c           ... WF in ss-approximation
            WF_OUT_SS(:,:) = PWF%WF_ssp(:,:,1,1)
         ENDIF
      ENDIF  ! "Rayleigh"
c ======================================================================
      
c ---------------------> Atoms and ions <------------------------------
      IF(INQ_WF('atom', 'WFT')) THEN 
c        ... Preliminary the same array as for Rayleigh
         WF_OUT_SS => LWF%WF_met(:,:,1)
         IF( WF_MS_FLAG(JPOS) ) THEN
c           ... WF in ms-approximation
            Ka = 1
            DCH = 1
            DO I = 1,NL
               DO K = 1,NN
                  G_Gp(I,K,:) = PH_M%MET(Ka)%Egg(K,:,1)*XM(K)*AG1(:)
                  G_Gm(I,K,:) = PH_M%MET(Ka)%Egg(K,:,2)*XM(K)*AG1(:)
               ENDDO
            ENDDO
            CALL M_POINT_WF_COMP_SPH(1)
         ELSE
c           ... WF in ss-approximation
            WF_OUT_SS(:,:) = PWF%WF_ssp(:,:,1,1)
         ENDIF
      ENDIF  ! "Atoms and ions"    
c ======================================================================

c -------------------------> Aerosol <---------------------------------
      IF(INQ_WF('aer', 'WFT')) THEN 
c        ... Derivative wrt Stokes component not yet implemented
         G_Gp = 0.d0 ; G_Gm = 0.d0

         IF( WF_MS_FLAG(JPOS) ) THEN
c           ... MS approximation for WF
            WF_OUT_SS => LWF%WF_sca(:,:,1,1)
            DCH = 2 
            DO J = 1,LA_N
               DO I = L1(J),L2(J)
                  DO K = 1,NN
                     G_Gp(I,K,:) = A_GG(J,K,:,1)*XM(K)*AG1(:)
                     G_Gm(I,K,:) = A_GG(J,K,:,2)*XM(K)*AG1(:)
                  ENDDO
               ENDDO
            ENDDO
            CALL M_POINT_WF_COMP_SPH(1)

c           ... WF for PSD function parameters
            IF( Dwr_mi ) THEN
               DO Idr = 1, N_dwr
c                 ... Derivative wrt Stokes component not yet implemented
                  WF_OUT_SS => LWF%WF_sca(:,:,1,1+Idr)
                  G_Gp = 0.d0 ; G_Gm = 0.d0 ; DCH = 2 
                  DO J = 1,LA_N
                     DO I = L1(J),L2(J)
                        DO K = 1,NN
                           G_Gp(I,K,:) = 
     $                          Ph_dA%Egg(J,K,:,1,Idr)*XM(K)*AG1(:) 
                           G_Gm(I,K,:) = 
     $                          Ph_dA%Egg(J,K,:,2,Idr)*XM(K)*AG1(:) 
                        ENDDO
                     ENDDO
                  ENDDO
                  DO I = 1,NL-1
                     S2 = (LOA%SCA(I) + LOA%SCA(I+1))/2.d0
                     G_Gp(I,:,:) = G_Gp(I,:,:)*S2
                     G_Gm(I,:,:) = G_Gm(I,:,:)*S2
                  ENDDO
                  S2 = (LOA%SCA(NL) + LOA%SCA(NL-1))/2.d0
                  G_Gp(NL,:,:) = G_Gp(NL,:,:)*S2
                  G_Gm(NL,:,:) = G_Gm(NL,:,:)*S2
                  CALL M_POINT_WF_COMP_SPH(2) 
               ENDDO            ! over derivatives of PSD parameters (Idr)
            ENDIF               ! WF for PSD function
         ELSE
c            ... WF in ss-approximation
c                PWF%WF_ssp is calculated in "spher_wf.f"
            LWF%WF_sca(:,:,1,:) = PWF%WF_ssp(:,:,1,:)
         ENDIF
      ENDIF  ! "Aerosol"
c     ======================================================================
      
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE MS_WF_EXT

      INTEGER    K2, Kg1, Kg2, Kh, Kli, Jg, Isa, Isf, INI, Jvm, Mli
      REAL (KND) H1, S4(Nvs), Qp(NNv), Qm(NNv), Pv2(NNv,Nvs)
      REAL (KND) Pfr(N_adj,Nvs), Pfr0(Nvs), Pfr1(Nvs), Pv1(NNv,Nvs)
      REAL (KND), ALLOCATABLE :: Xp1(:), HDU(:), Pf_Ref(:,:,:)
c     ------------------------------------------------------------------

      SCA => LOA%SCA 

c     ... Cosines of the solar zenith angles grid
      COSL(:) = SOL_GRD(1:NSZA)
      ALLOCATE( Pf_ref(NLO,N_Eff,Ns_wf) )
      Pf_ref = 0.d0

c     ... EXL(:) equal to zero for limb viewing directions
c --------> Surface reflection contribution <---------------------------
      SELECT CASE( TRIM(LBC) )
      CASE('Lambert')
c         write(*,*) '  St_sph_wf -0: Albedo = ', Alb
         Kv = 0
         DO Js = 1,N_SOL_ANG 
            DO Jv = JV_S(Js),JV_E(Js) 
               R_Surf = .FALSE.
c               IF(GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0) R_Surf = .TRUE.
               IF( EXL(Jv) > 0.d0) R_Surf = .TRUE.
               MLI = M_LIM(Jv*NFA-LW_FOV(Jv))   
               Kli = Ns_wf*SHI_LIM_C(Jv)
               DO Ja = 1,N_AZ_VAR    ;  Kv = Kv + 1                  
                  DO JL = 1,MLI
                     DO Jw = 1,NS_wf
                        IF( Nc_der(Jw)>1 ) CYCLE
                        Jw1 = Kli + (Jw - 1)*MLI + JL
                        H1 = 2.d0*Alb*FLUX_D(Jw1)*VST
                        IF( R_Surf.AND.JL==MLI )
     $                       H1 = H1 + 2.d0*Alb*EXL(Jv)*VST
                        Pf_ref(Kv,JL,Jw) = H1
                     ENDDO  ! over derivatives for Stokes components (Jw)
                  ENDDO  ! over sub-intervals along LoS (JL)
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over LoS angles (Jv)
         ENDDO  ! over solar angels (Js)

      CASE('Bidirect')         
         Kh = Nvs*(NN+1)
         ALLOCATE(Xp1(NN+1), HDU(Kh))
         Xp1(1:NN)=XM(:)  ;  Xp1(NN+1)=1.d0  
         Kv = 0
         DO Js = 1,N_SOL_ANG 
            DO Jv = JV_S(Js),JV_E(Js)  
               R_Surf = .FALSE.
c               IF(GeoD%LVG(IA,Jv)%C_VA(1) > 0.d0) R_Surf = .TRUE.
               IF(EXL(Jv) > 0.d0) R_Surf = .TRUE.
               MLI = M_LIM(Jv*NFA-LW_FOV(Jv)) 
               Kli = Ns_wf*SHI_LIM_C(Jv) 
               DO Ja = 1,N_AZ_VAR   
                  Kv = Kv + 1

                  INI = Nvs*(SDVI + Jv - 1) 
                  DO JL = 1,MLI
                     Cz = E_CSZA(SWF%IEffS(Kv,JL),'R')
                     DO Jw = 1,NS_wf
                        Jw1 =  Kli + (Jw-1)*MLI + JL

                        DO K1 = 1,Nvs
                           CALL AKIMA_INT
     $                          (NN+1, Xp1, Ref_surf(Jw1,K1:Kh:Nvs), 
     $                          1, Cz, Pfr1(K1) )
                        ENDDO   ! over Stokes vector components (K1)
c                               WRITE(*,'(5(1x,I2),2(1x,e12.4))') Js, Jv, 
c     $                                 Ja, Kv, JL, Cz, Pfr1(1)
                        Pf_ref(Kv,JL,Jw) = 2.d0*Pfr1(Nc_der(Jw)) !*VST

                        IF( Nc_der(Jw) /= 1 ) CYCLE

                        IF( R_Surf.AND.JL==MLI ) THEN 
                           HDU(1:Kh) = 
     $                          UDB%BRF_GS(1:Kh,INI+Nc_der(Jw),Mh)
                           IF( Nc_der(Jw)>2 ) HDU(:) = -HDU(:)
                           DO K1 = 1,Nvs
                              CALL AKIMA_INT(NN+1, Xp1,   
     $                             HDU(K1:Kh:Nvs), 1, Cz, Pfr0(K1) )
                           ENDDO ! over Stokes vector components (K1)
                           Pf_ref(Kv,JL,Jw) = Pf_ref(Kv,JL,Jw) 
     $                          + 2.d0*Pfr0(Nc_der(Jw))*EXL(Jv) *VST
                        ENDIF

                     ENDDO  ! over number of Stokes WF (Jw)
                  ENDDO  ! over sub-intervals (JL)
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over LoS angles (Jv)
         ENDDO  ! over solar angels (Js)
      END SELECT  ! 'Lambert' or 'Bidirect'
c ======================================================================

c      write(*,*) '  St_sph_wf -1 Mh = ', Mh
c      WRITE(*,*) '  --------------------------------------------------'
c      WRITE(*,*) '   SHAPE( SWF%DS_SUN_JL ) ',SHAPE( SWF%DS_SUN_JL ) 
c      WRITE(*,*) '  --------------------------------------------------'
      Kv = 0
      DO Js = 1,N_SOL_ANG
         DO Jv = JV_S(Js),JV_E(Js)
            MLI = M_LIM(Jv*NFA-LW_FOV(Jv)) ; Kli = SHI_LIM_C(Jv)*Ns_wf
            DO Ja = 1,N_AZ_VAR 
               Kv = Kv + 1   ! Kv counts viewing directions
               DO I = 1,NL
                  Ext_tes(I,:) = 0.d0 
                  DO JL = 1,MLI
                     Cz = E_CSZA(SWF%IEffS(Kv,JL),'G')
                     IF( Cz < COSL(NSZA).AND..NOT.ONE_MS_SZ ) CYCLE
                     Iks = SHI_LIM_C(Jv) + JL
                     IF(ONE_MS_SZ) THEN
                        AO(:) = DFI(I,1,:)
                     ELSE
c                    ... Interpolation of forward intensity on the effective 
c                        solar angle Cz
                        DO J = 1,NNv
                           CALL AKIMA_INT(NSZA, COSL, DFI(I,:,J),
     $                          1, Cz, AO(J) ) 
                           CALL AKIMA_INT(NSZA, COSL, DFI(I,:,J+NNv), 
     $                          1, Cz, AO(J+NNv) ) 
                        ENDDO
                     END IF
                     Cz = E_CSZA(SWF%IEffS(Kv,JL),'R')
                     IF(ONE_MS_SZ) THEN
                        DO J = 1,NNv
                           DO K = 1,Nvs
                              Pv2(J,K) = P02(I,J,K)
                              Pv1(J,K) = P01(I,J,K)
                           END DO
                        END DO
                     ELSE
c                    ... Interpolation of phase matrix Solar --> Gauss 
c                        on effective solar angle including refraction
                        DO J = 1,NNv
                           DO K = 1,Nvs
                              CALL AKIMA_INT(NSZA, COSL, 
     $                             P02(I,J,K:Nvs*NSZA:Nvs),1,Cz,
     $                             Pv2(J,K) ) 
                              CALL AKIMA_INT (NSZA, COSL, 
     $                             P01(I,J,K:Nvs*NSZA:Nvs),1,Cz,
     $                             Pv1(J,K) ) 
                           ENDDO  ! over Stokes components (K)
                        ENDDO  ! over Gaussian angles (J)
                     END IF ! ONE_MS_SZ
                     DO Jg = 1,NN
                        Kg1 = (Jg-1)*Nvs+1 ; Kg2 = Jg*Nvs
                        DO K2 = 1,Nvs
                           Qp(Kg1+K2-1)= SUM(Pv1(Kg1:Kg2,K2)*Finc(1,:) )
                           Qm(Kg1+K2-1)= SUM(Pv2(Kg1:Kg2,K2)*Finc(1,:) )
                        ENDDO
                     ENDDO
                     R_T(1:NNv)       = Qm(:)*XML(:)*AGL(:)
                     R_T(1+NNv:2*NNv) = Qp(:)*XML(:)*AGL(:)
                     DO Jw = 1, Ns_wf
                        Jw1 = Kli + (Jw - 1)*MLI + JL
c1     ... Inner product of diffuse forward and adjoint intensity
c     ---------------------------------------------------------------
                        DO K = 1,Nvs
                           S4(K) = SUM( 
     $                      ( AO(K:NNv:Nvs)*AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $                      + AO(NNv+K:2*NNv:Nvs)*AFI(I,K:NNv:Nvs,Jw1) )
     $                          *AG(1:NN) )
                        ENDDO   ! over Stokes component (K)
                        IF( Nvs > 1 ) S4(4) = -S4(4)
c                       ...  Term 1 in Eq.(Wfe-sph-lim) of STB
                        Ext_tes(I,Jw) = Ext_tes(I,Jw) + SUM( S4(1:K_s) )
c1      ---------------------------------------------------------------
                        
c2     ... Inner product of phase function and adjoint intensity
c      ---------------------------------------------------------------
                        DO K = 1,Nvs
                           S4(K) = SUM( 
     $                          ( R_T(K:NNv:Nvs)*AFI(I,K:NNv:Nvs,Jw1)
     $                          + R_T(NNv+K:2*NNv:Nvs)
     $                          *AFI(I,NNv+K:2*NNv:Nvs,Jw1) )  )
                        ENDDO  ! over Stokes component (K)
                        Jad(I,Iks,Jw) = F14*SUM( S4(:) )
c2      ---------------------------------------------------------------
                        IF( .NOT.ADD_PLANCK ) CYCLE
                        IF( Jw1==2 ) THEN
c                        IF( Nc_der(Jw)==1 ) THEN
                           K = 1
c                           WRITE(*,*)  '  ------>> Jw, Jw1 ', Jw, Jw1
c                       ...  Term 3 in Eq.(Wfe-sph-lim) of STB
                           Ext_tes(I,Jw) = Ext_tes(I,Jw)
     $                          - BT(I)* SUM( 
     $                          ( AFI(I,NNv+K:2*NNv:Nvs,Jw1)
     $                          + AFI(I,K:NNv:Nvs,Jw1) )*AG(1:NN) )
c     $                          - S_PLA *SUM( XM(:)
c     $                          *AFI(I,NNv+K:2*NNv:Nvs,Jw1)*AG(1:NN) )
c     $                          *AFI(I,K:NNv:Nvs,Jw1)*AG(1:NN) ) 
                        ENDIF
                     ENDDO  ! over derivatives for Stokes components (Jw, Jw1)
                  ENDDO  ! over line-of-sight (JL,Iks)
               ENDDO  ! over altitude levels (I)
               
c              ... SWF%DS_SUN_JL is calculated in "appr_spher_ls_main.f"
               DO Jw = 1, Ns_wf
                  WF_T = 0.d0 ; HH = 0.d0
                  DO JL = 1,MLI
                     Iks = SHI_LIM_C(Jv) + JL
c                    ...  WF_T is Term 2 in Eq.(Wfe-sph-lim) of STB
                     DO L = 1, NL 
                        WF_T(L,:) = WF_T(L,:) - SWF%DS_SUN_JL(L,:,Kv,JL)
     $                       *Jad(L,Iks,Jw) *SWF%SOL_JL(L,Kv,JL)
                     ENDDO  ! over altitude levels (L)
                     Cz = E_CSZA(SWF%IEffS(Kv,JL),'R')
c                    ... Surface contribution
                     HH(:) = HH(:) - Pf_ref(Kv,JL,Jw)
c                         ... Solar exponent at the bottom
     $                    *SWF%SOL_JL(NL,Kv,JL)/2.d0*Cz
     $                    *SWF%DS_SUN_JL(NL,:,Kv,JL) 
                  ENDDO  ! over sub-layers along L-of-S (JL)
c                        write(*,*) '  St_sph_wf -3 '
                  TMP(:) = WF_T(1,:)*DZR(1) + WF_T(NL,:)*DZR(NL-1)
                  DO I = 2, NL - 1
                     TMP(:) = TMP(:) +  WF_T(I,:)*DZ(I) 
                  END DO
                  TMP(:) = TMP(:)/2.d0  + HH(:)
                  
c                  WRITE(*,*) '   From HH for Mh ', Mh, SUM( HH(1:NL) ) 
                  DO I = 1,NL-1
                     Ext_tes(I,Jw) = (Ext_tes(I,Jw) + Ext_tes(I+1,Jw))
     $                    *0.5d0*DZR(I)
                  ENDDO
                  Jint(1) = Wup*Ext_tes(1,Jw)
                  DO I = 1,NL-2
                     Jint(I+1) = Wdo*Ext_tes(I,Jw) + Wup*Ext_tes(I+1,Jw) 
                  ENDDO
                  Jint(NL) = Ext_tes(NL-1,Jw) *Wdo
                  Help(1:NL,Kv,Jw) = -Jint(1:NL) + TMP(1:NL)
              ENDDO  ! over Stokes component derivatives (Jw)

            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over line-of-sight angles (Jv)
      ENDDO   ! over actual solar zenith angles (Js)

c     ... Interpolation on the given effective solar zenith angle      
      Kv = 0
      DO Js = 1, N_SOL_ANG
         DO Jv = JV_S(Js), JV_E(Js)
            DO Ja = 1,N_AZ_VAR  ;  Kv = Kv + 1
               DO Jw = 1,NS_wf
                  IF( Nc_der(Jw) > 2 ) THEN
                     H1 = DSIN( ANG_OUT(3,Kv)*DBLE(Mh-1) )
                  ELSE
                     H1 = DCOS( ANG_OUT(3,Kv)*DBLE(Mh-1) )
                  ENDIF
                  WF_ext(:,Kv,Jw) = H1 * Help(:,Kv,Jw)
               ENDDO  ! over Stokes component derivatives (Jw)

            ENDDO  ! over azimuthal angles (Ja)
         ENDDO  ! over zenith angles (Jv)
      ENDDO  ! over solar angles (Js)
c      WRITE(*,*) '  ----- st_sph_wf.f ------ '

c     Single scatt test         
      DO Jw = 1,NS_wf
         WF_ext(:,:,Jw) = WF_ext(:,:,Jw)  + PWF%WF_esp(:,:,Jw)
c         WF_ext(:,:,Jw) = PWF%WF_esp(:,:,Jw)
      ENDDO

      RETURN
      END SUBROUTINE MS_WF_EXT
c **********************************************************************

      SUBROUTINE M_POINT_WF_COMP_SPH(Ipa)
c     ... Local parameters
      INTEGER, INTENT(IN) :: Ipa
      INTEGER    Iks, JL, K1
      REAL (KND) S1, R_T(2*NN), AO1(2*NN), S_Gm(NN), S_Gp(NN)
      REAL (KND) T_Wc(NL,NLO), Pol_sef(Nlegen,NLO,N_Eff)
      REAL (KND), TARGET:: AER_LOC(Nlg_Aer,NL)
      REAL (KND), POINTER :: P_MOMS(:), QP_T(:,:), QM_T(:,:)

      Kv = 0
c     ... Legendre polynomials for angles along l-of-s
      DO Js = 1,N_SOL_ANG 
         DO Jv = JV_S(Js),JV_E(Js)
            DO Ja = 1,N_AZ_VAR ; Kv = Kv + 1
               DO JL = 1,M_LIM(Jv*NFA-LW_FOV(Jv))
                  S1 = - E_CSZA(SWF%IEffS(Kv,JL),'R')
                  CALL WIGNER_PLG(S1,1,Mh-1,Nlegen,Pol_sef(:,Kv,JL))
               ENDDO
            ENDDO
         ENDDO
      ENDDO
      
      T_Wc = 0.d0
      IF( DCH==1 .AND. (Mh.LE. 3) ) THEN
c        ... WF for Rayleigh scattering
         IF( Ray_to_WF ) P_MOMS => LOA%RAY_MOMS(Mh:3,1)
c        ... WF for atoms and ions
         IF(INQ_WF('atom', 'WFT'))  P_MOMS => LOA%EPF(Ka,Mh:3,1)
         
         DO I = 1,NL 
            K1 = 0
            DO Js = 1,N_SOL_ANG 
               DO Jv = JV_S(Js),JV_E(Js)
                  DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
                     DO JL = 1,M_LIM(Jv*NFA-LW_FOV(Jv)) 
                        Iks = SHI_LIM_C(Jv)+JL
c                       ... Single scattering source function with 
c                           quadrature coefficients
                        DO K = 1,NN
                           S_Gm(K) = SUM( P_MOMS(:)*Pol_sef(Mh:3,K1,JL) 
     $                          *Pol_ga(Mh:3,K) )*AG(K)
                           S_Gp(K) = SUM( P_MOMS(:)*Pol_sef(Mh:3,K1,JL) 
     $                          *Pol_ga(Mh:3,K+NN) )*AG(K)
                        ENDDO         
c                       ... Contribution from the term < Js,I*d >
                        T_Wc(I,K1) = T_Wc(I,K1) + SWF%SOL_JL(I,K1,JL)*
     $                       SUM( Afi(I,NN+1:2*NN,Iks)*S_Gp(:)
     $                       + Afi(I,1:NN,Iks)*S_Gm(:) )
                     ENDDO
                  ENDDO  ! over azimuthal angles (Ja)
               ENDDO  ! over line-of-sight angles (Jv)
            ENDDO  ! over solar zenith angles (Js)
         ENDDO  ! over layers (I)
      ELSE
c        ... WF for Aerosol scattering
         AER_LOC = 0.d0
         IF( Ipa==1 ) THEN
            DO I = 1,LA_N
c               WRITE(*,*) '  Aerosol layer: ', I, L1(I), L2(I)
               DO K = L1(I),L2(I)
                  AER_LOC(:,K) = LOA%AER_MOMS(:,I,1)
               ENDDO
            ENDDO
         ELSE
            DO I = 1,LA_N
               DO K = L1(I),L2(I)
                  AER_LOC(:,K) = LOA%ADR_MOMS(:,I,1,Idr) *SCA(K)
               ENDDO
            ENDDO
         ENDIF
         DO I = 1,NL
            P_MOMS => AER_LOC(Mh:Nlegen,I) ; K1 = 0
            DO Js = 1,N_SOL_ANG 
               DO Jv = JV_S(Js),JV_E(Js)
                  DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
                     DO JL = 1,M_LIM(Jv*NFA-LW_FOV(Jv)) 
                        Iks = SHI_LIM_C(Jv)+JL
                        DO K = 1,NN
                           S_Gm(K) = SUM( P_MOMS(:)
     $                          *Pol_sef(Mh:Nlegen,K1,JL) 
     $                          *Pol_ga(Mh:Nlegen,K) )*AG(K)
                           S_Gp(K) = SUM( P_MOMS(:)
     $                          *Pol_sef(Mh:Nlegen,K1,JL) 
     $                          *Pol_ga(Mh:Nlegen,K+NN) )*AG(K)
                        ENDDO         
c                       ... Contribution from the term < Js,I*d >
                        T_Wc(I,K1) = T_Wc(I,K1) + SWF%SOL_JL(I,K1,JL)*
     $                       SUM( Afi(I,NN+1:2*NN,Iks)*S_Gp(:)
     $                       + Afi(I,1:NN,Iks)*S_Gm(:) )
                     ENDDO
                  ENDDO  ! over azimuthal angles (Ja)
               ENDDO  ! over line-of-sight angles (Jv)   
            ENDDO  ! over solar zenith angles (Js)
         ENDDO  ! over layers (I)
      ENDIF
      T_Wc = F14*T_Wc

      DO I = 1,NL 
         K1 = 0
         Qp_T => G_Gp(I,:,:) ; Qm_T => G_Gm(I,:,:)
         DO Js = 1,N_SOL_ANG 
            DO Jv = JV_S(Js), JV_E(Js)
               DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
                  DO JL = 1,M_LIM(Jv*NFA-LW_FOV(Jv))
                     Iks = SHI_LIM_C(Jv)+JL
                     Cz = E_CSZA(SWF%IEffS(K1,JL),'G')
c                            WRITE(*,*) I,K1, Dacos(Cz)/Pie*180.
                     R_T = 0.d0
                     DO K = 1,NN
                        IF(ONE_MS_SZ) THEN
                           AO1(K) = DFI(I,1,K)
                           AO1(K+NN) = DFI(I,1,K+NN)
                        ELSE
                           CALL AKIMA_INT( NSZA, COSL, DFI(I,:,K),  
     $                          1, Cz, AO1(K) )
                           CALL AKIMA_INT( NSZA, COSL, DFI(I,:,K+NN), 
     $                          1, Cz, AO1(K+NN) )
                        END IF

                        R_T(1:NN) = R_T(1:NN) + (Qm_T(K,:)*Afi(I,K,Iks) 
     $                       + Qp_T(K,:)*Afi(I,K+NN,Iks))*AG(K)
                        R_T(NN+1:2*NN) = R_T(NN+1:2*NN) + 
     $                       (Qm_T(K,:)*Afi(I,K+NN,Iks) 
     $                       + Qp_T(K,:)*Afi(I,K,Iks) )*AG(K)
                     ENDDO      ! over Gaussian nodes (K)
c                    ... Contribution from the term < Jd,I*d >
                     T_Wc(I,K1) = T_Wc(I,K1) + SUM( R_T(:)*AO1(:) )
                  ENDDO
               ENDDO  ! over azimuthal angles (Ja)
            ENDDO  ! over line-of-sight angles (Jv)
         ENDDO  ! over solar zenith angles (Js)
      ENDDO  ! over layers (I)

c      WRITE(*,*) '  DZR(NL-1) ', DZR(NL-1)
      
      SELECT CASE ( TRIM(WF_INTEGR) )
      CASE( 'nointegr' ) 
         STOP ' Option "nointegr" is not exist '
      CASE DEFAULT
         K1 = 0
         DO Js = 1,N_SOL_ANG
            DO Jv = JV_S(Js),JV_E(Js) 
               DO Ja = 1,N_AZ_VAR ; K1 = K1 + 1
c                  DO Jw = 1,NS_wf  ! not implemented yet
                  Help(1:NL-1,K1,1) = 0.5d0*DZR(1:NL-1)*
     $                 (T_Wc(1:NL-1,K1) + T_Wc(2:NL,K1))*
     $                 DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) )
                  Help(NL,K1,1) = 0.5d0*DZR(NL-1)*
     $                 T_Wc(NL,K1)* DCOS( ANG_OUT(3,K1)*DBLE(Mh-1) )
c                  ENDDO
               ENDDO
            ENDDO
         ENDDO
      END SELECT     

      IF( Wup > 0.d0 .AND. Wup < 1.d0 ) Then
         WF_OUT_SS(1,:) = Wup*Help(1,:,1)
         DO I = 1,NL-2
            WF_OUT_SS(I+1,:) = Wdo*Help(I,:,1) + Wup*Help(I+1,:,1)
         ENDDO
         WF_OUT_SS(NL,:) = Help(NL-1,:,1)*Wdo 
      ENDIF
 
      IF( Ipa==1 ) THEN
         WF_OUT_SS(:,:) = WF_OUT_SS(:,:) + PWF%WF_ssp(:,:,1,Ipa)
c        ... Single scatt test (no contribution of MS)
c         WF_OUT_SS(:,:) = PWF%WF_ssp(:,:,1,Ipa)
      ELSE
         WF_OUT_SS(:,:) = WF_OUT_SS(:,:) + PWF%WF_ssp(:,:,1,Idr+1)
c        ... Single scatt test (no contribution of MS)
c         WF_OUT_SS(:,:) = PWF%WF_ssp(:,:,1,Idr+1)
      ENDIF

      RETURN
      END SUBROUTINE M_POINT_WF_COMP_SPH
c ***********************************************************************
      END 

