C********************************************************************C
C
C       SCIATRAN - Version 3.8 - Last modification: 14 Mar 2017
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ST_SPHER_GRIDS (*)
c     ... Called by "rtm_init.f"

      USE GRIDS,      ONLY: NLAYERS, ZZZ, SOL_GRD, Z_INV, ALT_EXT,  
     $                      NL_FINE, Z_FINE, INTERP_IDX, MAX_LINE
      USE LOG_PARS,   ONLY: EU
      USE SPHER_PARS, ONLY: NN, I_FINE, N_FINE_ST, N_FINE_TH, Z_F, 
     $                      Z_SOL_F, I_FINE_SOL
      USE ST_VARS,    ONLY: RTM_TYPE, RTM_CORE
c     ------------------------------------------------------------------

      CHARACTER (LEN=*), PARAMETER ::  R_NAME='ST_SPHER_GRIDS'
      INTEGER              K, J, I_MAX
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                             ! conditionally deallocated here

      ALLOCATE(Z_INV(NLAYERS), Z_FINE((NLAYERS - 1)*NN + 1), !
     $     ALT_EXT(NLAYERS + 4), INTERP_IDX((NLAYERS - 1)*NN + 1)) ! A
      Z_INV(1:NLAYERS) = ZZZ(NLAYERS:1:-1)

      NL_FINE = 0
c     ... Fine altitude grid Z_FINE in ascending order
      DO J = 1, NLAYERS - 1
         DO K = 1, NN
            NL_FINE = NL_FINE + 1
            Z_FINE(NL_FINE) =  Z_INV(J) + 
     $           (Z_INV(J + 1) - Z_INV(J))/DBLE(NN)*DBLE(K - 1)
            INTERP_IDX(NL_FINE) =  J
         END DO                 ! over the fine grid layers
      END DO                    ! over standard layers

      NL_FINE = NL_FINE + 1
      IF( NL_FINE.GT.(NLAYERS - 1)*NN + 1 ) GOTO 200
      Z_FINE(NL_FINE) =  Z_INV(NLAYERS)
      INTERP_IDX(NL_FINE) = NLAYERS 

      DO J = 1, NLAYERS
         IF (ZZZ(J) .LE. Z_F) EXIT
      END DO
      I_FINE = MIN(J,NLAYERS)

      DO J = 1, NLAYERS
         IF (ZZZ(J) .LE. Z_SOL_F) EXIT
      END DO
      I_FINE_SOL = MIN(J,NLAYERS)

      I_MAX = MIN(I_FINE,I_FINE_SOL)

      MAX_LINE = (N_FINE_ST + 1)*(NN - 1) + 
     $     MAX((NLAYERS - I_MAX),(N_FINE_TH + 1))*2*(NN - 1) + 
c     $     MAX((NLAYERS - I_FINE),(N_FINE_TH + 1))*2*(NN - 1) + 
     $     2*NLAYERS - 1

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME,'ERROR','Wrong array allocation')
      WRITE (EU,*) '    Allocated:', (NLAYERS - 1)*NN + 1
      WRITE (EU,*) '    Required:', NL_FINE
      RETURN 1
      
      CONTAINS
C**************************************************************************
      SUBROUTINE DEALLOCATE_ARRAYS
      
      IF(ALLOCATED(Z_INV)) DEALLOCATE(Z_INV, Z_FINE, ALT_EXT,
     $     INTERP_IDX)

      RETURN
      END SUBROUTINE DEALLOCATE_ARRAYS

C**************************************************************************
      END
C**********************************************************************
