C***********************************************************************
C
C       SCIATRAN - Version 4.4 - Last modification: 24 Apr 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************
       
C           Subroutine to calculate the Stokes vector in the 
C      single scattering approximation in the spherical geometry.
C        It is assumed that incident radiation is unpolarized !!!
C      ----------------------------------------------------------
      SUBROUTINE ST_SS_STOKES(JPOS, JJESFT, POS, LOA, E_W, ALBEDO, 
     $     FLU_int, OSM, TAC, PWF, SRR, SRR_Jact, LSE, IND_LS, 
     $     TE, ADI, RADOUT)
c     ... Call in "dom_spherical.f"

      USE ABSORBERS,   ONLY: N_GAS
      USE AEROSOLS,    ONLY: IDX_AER, Dwr_mi, N_dwr
      USE CLD_INP,     ONLY: Ydb_ang, MLCS
      USE CLOUDS,      ONLY: DO_CLOUD, CLA => CLD_LAYER_ARR, SCAT_ANG
      USE EMISSION,    ONLY: N_AT, Emi_MOMS
      USE GRIDS,       ONLY: NLAYERS, ZZZ, Z_FINE, NL_F => NL_FINE, 
     $                       INTERP_IDX
      USE INTERN_DIMS, ONLY: Leg_aer => NLG_ALL_A, Leg_cld => NLG_ALL_C, 
     $                       Nm_vs, Nvs, Nlg_ray, Nm_ice, NL_pla
      USE IRRAD,       ONLY: SOL_FLUX, NO_SUN
      USE LOG_PARS,    ONLY: LOG_LEVEL, LUNIT, N_DSO, N_ESO 
      USE LOS_Angles,  ONLY: DirSun
      USE RAMAN,       ONLY: WL_FACT, LINE_GAS, Sigma_RR 
      USE RTM_OUTPUT,  ONLY: STOK_OUT, DEL_RRS, N_OP
      USE SPHER_PARS,  ONLY: N_LOS_ANG, NLO => N_LOS_OUT,
     $                       NFA => N_FOV_ANG, FOV_WEIGHTS
      USE ST_INPUT,    ONLY: APH_TYPE
      USE ST_VARS,     ONLY: Pie, DO_AEROSOL, WF_FLAG, DO_PLANCK, Ns_wf,
     $                       DO_RAMAN, WL_GRID, SCATT_MODE, Nc_der, 
     $                       Emi_FLG, GeoD => GeoD_OUT, PreCalcAngs,
     $                       Ray_to_WF
      USE TOOLS,       ONLY: FIND_IDX, INQ_WF
      USE TYPES,       ONLY: KND, LOA_TYPE, PCC_TYPE, PWF_TYPE, TE_TYPE,
     $                       OSM_TYPE, LWF_TYPE, RRS_TYPE, LSE_TYPE,
     $                       AuxDataInterp
      USE WF_NUM,      ONLY: NUM_WF, NO_SOL, NO_LS, AL_1_BASE, AL_1_PERT
c     ----------------------------------------------------------------

      INTEGER JPOS, JJESFT, SRR_Jact, IND_LS(NLAYERS,N_LOS_ANG*NFA,2)
      REAL (KND) E_W, ALBEDO, FLU_int, POS, 
     $     TAC(NLAYERS,N_GAS), RADOUT(N_OP,NLO,Nvs)
c     ... Input structures:
      TYPE (AuxDataInterp), TARGET :: ADI
      TYPE (LOA_type) :: LOA    ! Local optical parameters of atmosphere 
      TYPE (LWF_type) :: LWF    ! Local weighting functions
      TYPE (OSM_type) :: OSM    ! Original scattering matrices
      TYPE (PWF_type) :: PWF    ! General weighting function 
      TYPE (RRS_type), TARGET :: SRR    ! RRS related parameters
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (LSE_type), TARGET :: LSE(GeoD%NAlts, GeoD%NVA)
      
      INTENT(IN) :: JPOS, JJESFT, IND_LS, ALBEDO, FLU_int, E_W, POS,
     $     LOA, OSM, PWF, ADI
      INTENT(INOUT) :: RADOUT, SRR
c     ------------------------------------------------------------------
c     ... Local parameters
c     --------------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_SS_STOKES'
      LOGICAL   DO_WF
      LOGICAL, POINTER ::  REFR_FLAG(:)
      INTEGER   Mha, Idl, Ids, II, Iae, J, I, K, K1, K2, Jw, Jw1, Js,
     $     NR, N_DS, Jv, Jf, Jt, KK, Ja, Jex, Nsf, NLN1, Index, Idr,
     $     To_act, Npl, Npo, NRL, Ka, R_lin, NTH, IA, IV, IS, IP, Ifov
      INTEGER, POINTER :: IOB(:)
      INTEGER, POINTER ::  SID(:)
      INTEGER, ALLOCATABLE :: CI(:)
      REAL (KND) Mu_Ls, Z_OUT, D1, IRR, DUM(Nvs), BC_S1(Nvs), LBC(Nvs),
     $     RAD_OUT(NLO,Nvs), Phm(GeoD%MaxLS,Nvs), SCAD(NLAYERS), 
     $     S_R(GeoD%MaxLS,Nvs,1+N_dwr), W_FOV, Te_Surf
     $     , Dum1(Nvs), DDD(4,Nvs)
      REAL (KND) RESULT(GeoD%NAlts,GeoD%NVA,GeoD%NSol,GeoD%NPhi,Nvs) ! A
      REAL (KND), ALLOCATABLE :: DS_SUN(:,:), Wig0(:,:),
     $     Wig2(:,:), Rot_l(:,:), SF_TE(:), SCCNO_LS(:,:),
     $     Del_ss(:,:,:,:,:,:), COF(:,:), COF_FI(:,:), Wes(:),
     $     COF_LS(:,:), AL_0(:,:,:), AL_F(:,:,:), AL_L(:,:,:), R_j(:,:),
     $     DS_sun_surf(:), EMCL(:,:), EMCO(:,:), EX_1(:)
      TARGET :: AL_L
      REAL (KND), DIMENSION(GeoD%MaxLS,Nvs) :: SSSF, S_F      
      REAL (KND), DIMENSION(GeoD%MaxLS) :: AL_1, SOL_EXP, S_H, G_aer,
     $     COSCA, SCRL, SCAL
      REAL (KND), POINTER :: P_MOM(:), XM_1(:), TH(:), ZLS(:), XM_S1(:), 
     $     EX_L(:), ALO(:), SCRO(:), SCAO(:), ASCO(:), 
     $     PHI_LOS(:), D_DS(:,:), SCC_NSO(:,:), EXL(:)
      TYPE (PCC_type) :: PCC    ! Photochemistry
c     ==================================================================
      INTERFACE
         SUBROUTINE SET_OUT_RAD (ARR_INP, ARR_OUT)
            USE SPHER_PARS,  ONLY: N_LOS_OUT
            USE INTERN_DIMS, ONLY: Nvs
            USE TYPES,       ONLY: KND
            REAL (KND) ARR_INP(:,:,:,:,:), ARR_OUT(N_LOS_OUT,Nvs) 
         END SUBROUTINE SET_OUT_RAD
c        ---------------------------------------------------------------
         SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, 
     $     ALBEDO, FLU_int, SOL_EXP, PLANCK_SURF, BC_S1, LBC, REF_SUR)
            USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, 
     $        Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
            USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
            USE INTERN_DIMS, ONLY: Nvs
            USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
            USE TYPE_VARS, ONLY: KND
            USE WATER,  ONLY: SS_PDF
            INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
            REAL (KND), INTENT(IN) :: ALBEDO, FLU_int, POS, SOL_EXP, 
     $           PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
            REAL (KND) BC_S1(Nvs), LBC(Nvs)
            REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
            OPTIONAL :: REF_SUR
         END SUBROUTINE REFL_DIR_SPH
      END INTERFACE
c     ==================================================================
      
      CALL ST_TRACK (R_NAME, 1, 'I')
c         if SCATT_MODE = 'ms' 
c                          This routine is not used
c         if SCATT_MODE = 'ss' 
c                          WFs are calculated in this routine in the 
c                          single scattering approximation and saved
c                          in arrays PWF%WF_esp and PWF%WF_ssp and used
c                          as output in pointers LWF%WF_...
c         if SCATT_MODE = 'wf_ss' 
c                          !!! it does not work !!!
c                         
      TE%NO_SUN = NO_SUN
c     --- Allocated in "dom_spherical.f"
      PWF%WF_esp = 0.d0 ; PWF%WF_ssp = 0.d0 
            
      Mha = 1  ;  DO_WF = WF_FLAG ; IA = 1 

      IF( Emi_FLG ) THEN
         ALLOCATE( EMCL(N_AT,GeoD%MaxLS), EMCO(N_AT,NL_F) )
         DO Ka = 1, N_AT
            CALL LIN_FINE_1(NLAYERS, ZZZ, NL_F, Z_FINE, INTERP_IDX, 
     $           LOA%EMC(Ka,:), EMCO(Ka,:), .FALSE.)
         ENDDO
      ENDIF  ! atoms emission

      IF( DO_AEROSOL ) THEN
c        ... Aerosol scattering coefficient without Delta-M scaling
         WHERE( LOA%DM_A(:) /= 1.d0 )
            SCAD(:) = LOA%SCA(:)/(1.d0 - LOA%DM_A(:))
         ELSEWHERE
            SCAD(:) = LOA%SCA(:)
         END WHERE
         
         !DO J = 1, NLAYERS
         !   WRITE(*,'(2x,F8.4*(2x,e12.4))') ZZZ(J), SCAD(J)
         !ENDDO
         
      ENDIF  ! DO_AEROSOL

      Te_Surf = 0.d0
      IF( DO_PLANCK ) THEN
         ALLOCATE( SF_TE(GeoD%MaxLS), LWF%WF_TE(NL_pla,NLO) )
         LWF%WF_TE = 0.d0
      ENDIF

      ALO => ADI%AL0_OUT  ;  SCRO => ADI%SCR_OUT
      SCAO => ADI%SCA_OUT ;  ASCO => ADI%ASC_OUT
      IF( DO_CLOUD ) SCC_NSO => ADI%SCC_NSO
!      WRITE(*,*) '   st_ss_stokes.f  ADI%AL0_OUT '
!      WRITE(*,'(4x,ES12.4)') ALO(NL_F)      
      Z_OUT = GeoD%Alt(1)  
      RESULT(:,:,:,:,:) = 0.d0

c     ... If SCATT_MODE=='ss' then RRS_ADJ flag is set to .true.
      IF( DO_RAMAN ) THEN
         ALLOCATE( Wes(SRR%M_ESF), R_j(GeoD%MaxLS,SRR%M_ESF) )
c        ... Actual wavelength ...
         IF( SRR%ACT_WL(JPOS) ) THEN
c           ... Number of Raman lines (dummy)
            NRL = 1
            ALLOCATE( COF(NLAYERS,NRL), COF_FI(NL_F,NRL), 
     $           COF_LS(GeoD%MaxLS,NRL) )
c           ... Set solar irradiance at actual WL corresponding
c               to its position in the extended grid
            IRR = -SOL_FLUX(JPOS)/Pie
c            WRITE(*,'(10x,I4,2x,es14.6)') JPOS, -SOL_FLUX(JPOS)
            To_act = SRR_Jact
c            IRR = 0.d0

c           ... Save extinction coefficient for actual wavelength
            SRR%EXT_ACT(:,To_act,JJESFT) = LOA%AL(:)
c           ... Save transmission along l-of-s at actual wavelength
            FORALL(IV = 1:GeoD%NVA) SRR%LS_ACT(IA,IV)%
     $           EX(:,To_act,JJESFT) = LSE(IA,IV)%EX(:)
            COF(:,1) = SRR%R_TOT*LOA%SCR(:)
         ELSE
c           ... Set extraterrestrial solar flux at excitation wavelength
            IRR = SOL_FLUX(JPOS)/Pie
c            WRITE(*,'(10x,I4,2x,es14.6)') JPOS, SOL_FLUX(JPOS)
c            IRR = 0.d0
c           ... Number of Raman lines
            NRL = SRR%NUM_IN(JPOS)            
c          ... Number of ESFT coefficients at actual wavelength
            ALLOCATE( COF(NLAYERS,NRL), COF_FI(NL_F,NRL), ! A
     $           COF_LS(GeoD%MaxLS,NRL),                    ! A
     $           AL_0( NLAYERS,SRR%M_ESF,NRL),            ! A
     $           AL_F(    NL_F,SRR%M_ESF,NRL),            ! A
     $           AL_L(GeoD%MaxLS,SRR%M_ESF,NRL) )         ! A
            DO Jex = 1,SRR%NUM_IN(JPOS)
               To_act = SRR%ACT_IN(JPOS,Jex) 
               R_lin  = SRR%RIN_IN(JPOS,Jex)
               Iae    = SRR%A2E(To_act)
c              ... Calculate Raman scattering coefficient
               COF(:,Jex) = WL_FACT*Sigma_RR(R_lin,:)/WL_GRID(Iae)**4
     $              *SRR%GAM(LINE_GAS(R_lin))
c              ... Number of ESFT coefficients at actual wavelength
               Nsf = SRR%NESFT(To_act)
c               WRITE(*,*) '   Nsf = ', Nsf
c               WRITE(*,*) SRR%NESFT(To_act)
               AL_0(:,1:Nsf,Jex) = SRR%EXT_ACT(:,To_act,1:Nsf)
c              ... Interpolation on the "fine" altitude grid
               DO I = 1,Nsf
                  CALL LIN_FINE_1(NLAYERS,ZZZ,NL_F,Z_FINE,INTERP_IDX, 
     $                 AL_0(:,I,Jex), AL_F(:,I,Jex), .FALSE.)
               ENDDO
            ENDDO
         ENDIF  ! excitation wavelength
         ALLOCATE(
     $        Del_ss(GeoD%NAlts,GeoD%NVA,GeoD%NSol,GeoD%NPhi,Nvs,NRL))
         Del_ss = 0.d0
c        ... Interpolation on the "fine" altitude grid
         DO I = 1,NRL
            CALL LIN_FINE_1(NLAYERS, ZZZ, NL_F, Z_FINE, INTERP_IDX, 
     $           COF(:,I), COF_FI(:,I), .FALSE.)
         ENDDO
      ENDIF   ! DO_RAMAN

      IF(DO_WF) ALLOCATE( DS_SUN(GeoD%MaxLS,NLAYERS),
     $     DS_sun_surf(NLAYERS) )    ! A

c     ... Allocate Wigner-functions along line-of-sight
      Npo = MAX( Nlg_Ray,Leg_aer,Leg_cld )
      ALLOCATE( Wig0(Npo,GeoD%MaxLS) )
c      WRITE(*,*) '   MAX_LINE = ', MAX_LINE
c      WRITE(*,*) '        Npo = ', Npo
      IF( Nvs > 1 ) THEN
         ALLOCATE( Wig2(Npo,GeoD%MaxLS), Rot_l(GeoD%MaxLS,Nvs) )
      ENDIF

      !WRITE(*,*) ' -------> "st_ss_stokes.f"   '

      DO IV = 1, GeoD%NVA       ! Number of viewing angles including FOV

         Ifov = IV  - (CEILING( FLOAT(IV)/FLOAT(NFA) ) - 1)*NFA
         W_FOV = FOV_WEIGHTS(Ifov)

         Mu_Ls = GeoD%LVG(IA,IV)%C_VA(1)
         NTH   = GeoD%LVG(IA,IV)%NTH
         NR    = GeoD%LVG(IA,IV)%NLS
         ZLS  => GeoD%LVG(IA,IV)%ZLS
         SID  => GeoD%LVG(IA,IV)%ISEL
         IOB  => GeoD%LVG(IA,IV)%IOB

         IF( Nr==1 ) CYCLE
         IF( Z_OUT == ZZZ(1).AND.Mu_Ls <= 0.0D0 ) CYCLE

         IF( DO_CLOUD ) ALLOCATE( CI(NR), SCCNO_LS(NR,Nm_ice) )
      
c     ... Cosine of polar angle along l-of-s correspond to the z-axis, 
c     i.e., positive for reflection and negative for transmission.
         XM_1 => GeoD%LVG(IA,IV)%C_VA
         EX_L => LSE(IA,IV)%EX (1:NR)
         ALLOCATE( EX_1(0:NR) )
         EX_1(:) = LSE(IA,IV)%EX1(0:NR)
         D_DS => GeoD%LVG(IA,IV)%DS
c        ... Setting of scattering and extinction coefficients
c             along l-of-s including interpolation on the tangent height
         CALL SET_COEFF

         DO IS = 1, GeoD%NSol   ! Nuber of solar angles
            DO IP = 1, GeoD%NPhi ! Nuber of azimuth angles

               PHI_LOS => GeoD%LSG(IA,IV,IS,IP)%SAA(:)

c              ... Cosine of the solar angle along l-of-s is positive 
c                  for the Sun above horizon.
               XM_S1 => GeoD%LSG(IA,IV,IS,IP)%C_SZA(:)  
               TH => GeoD%LSG(IA,IV,IS,IP)%TH(:)
               IF( TE%NO_SUN ) THEN
                  SOL_EXP(1:Nr) = 0.d0
               ELSE
c                 ... Scattering angle along line-of-sight
                  COSCA(1:NR) = -XM_1(1:NR)*XM_S1(1:NR) ! scalar
     $                 + DSQRT(1.d0 - XM_1(1:NR)*XM_1(1:NR))
     $                 * DSQRT(1.D0 - XM_S1(1:NR)*XM_S1(1:NR))
     $                 * DCOS(PHI_LOS(1:NR))
c                 ... Solar exponent at all point along line-of-sight
                  ASSOCIATE (LVG => GeoD%LVG(IA,IV),
     $                 LSG => GeoD%LSG(IA,IV,IS,IP))
                  CALL DirSun (NR, XM_S1, TH, ZLS, LVG%IOB, AL_1, ALO, 
     $                 PCC, SOL_EXP(1:NR), LSG%REFR_DSB, PreCalcAngs,
     $                  DO_WF, DS_SUN, LSG%DSB)
                  END ASSOCIATE
               ENDIF

               Te_Surf = 0.d0
               BC_S1  = 0.d0 ; LBC = 0.d0
c              ... Lambertian reflection is used if l-of-s 
c                  starts at the Earth surface
               IF( ZLS(Nr) == ZZZ(NLAYERS).AND.XM_S1(Nr) > 0.d0 ) THEN

                  !WRITE(*,*) IV, DACOS(XM_S1(Nr:Nr))/Pie*180.

                  CALL REFL_DIR_SPH(1,1,1, XM_1(Nr:Nr), XM_S1(Nr:Nr),  
     $                 PHI_LOS(Nr:Nr), POS, ALBEDO, FLU_int,  
     $                 SOL_EXP(Nr), TE%PLANCK_SURF, BC_S1, LBC)
                  IF( DO_PLANCK ) Te_Surf = TE%PLANCK_SURF
               ENDIF
c               write(*,*) '   Mu_Ls  ', Mu_Ls
               IF( Z_OUT==ZZZ(NLAYERS).AND.Mu_Ls>0 ) THEN
                  RESULT(IA,IV,IS,IP,:) = BC_S1(:) ; CYCLE
               END IF
c               write(*,*) '  TE%NO_SUN   ', TE%NO_SUN
               IF( .NOT.TE%NO_SUN ) THEN
c                 ... Legendre polynomials for all scattering angles
                  CALL WIGNER_PLG(COSCA(1:NR), NR, 0, Npo, Wig0(:,1:Nr)) 
                  IF( Nvs > 1 ) THEN
c                    ... d-Wigner functions for all scattering angles along l-of-s
                     CALL WIGNER_PLG
     $                    (COSCA(1:NR), NR, 2, Npo, Wig2(:,1:NR))
c                    ... Rotation matrix for all points along l-of-s
                     CALL ROT_MATR
                  ENDIF  ! vector case <===
               ENDIF  ! .NOT.TE%NO_SUN 

c              ... Raman contribution for actual and excitation 
c                  wavelengths will be calculated
c              -------------------------------------------------
               IF( DO_RAMAN ) THEN
                  CALL RAM_SSF
                  IF( SRR%Exi_WL(JPOS) ) CYCLE
               ENDIF            ! DO_RAMAN
c              -------------------------------------------------
               SSSF = 0.d0
               IF( .NOT.TE%NO_SUN ) THEN
c                 ... Rayleigh contribution to the source function
                  CALL RAY_SSF
c                 ... Aerosol contribution to the source function
                  IF( DO_AEROSOL ) CALL AER_SSF
c                 ... Clouds contribution to the source function
                  IF( DO_CLOUD )  CALL CLD_SSF
c                 ... Source function of direct solar light is done
                  FORALL( K=1:Nvs ) SSSF(1:Nr,K) = 
     $                 SSSF(1:Nr,K)*SOL_EXP(1:Nr)/4.d0  
               ENDIF

c              ... No contribution of multiple scattering 
               S_F = 0.d0 
c              ... Thermal emission contribution to the source function
               IF( DO_PLANCK )  S_F(:,1) = S_F(:,1) + SF_TE(:)
                  
               IF( DO_WF ) THEN
c                 ... Contribution of the extinction coefficient 
c                     variation at solar path TOA-Surface should be 
c                     calculated before run the routine "spher_wf.f" because 
c                     array DS_SUN will be recalculated within this routine
c                     into DS_EXP
                  DS_sun_surf(:) = DS_SUN(Nr,:)
c                 ... Arrays PWF%WF_ssp and PWF%WF_esp(:,Index,Jw)
c                     calculated in this subroutine
c                  WRITE(*,*) ' S_R do sph ==> ', S_R(1:10,1,1)

                  CALL SPHER_WF(SSSF, DS_SUN, S_F, S_R, EX_L, EX_1, Nr,
     $                 D_DS, IOB, ZLS, Mha, AL_1, PWF, IV, IS, IP, JPOS,
     $                 DO_PLANCK)
                  Index = GeoD%ILO_SVFP(IV,IS,IP)
c                 ... PWF%WF_esp(:,Index,Jw) calculated in SPHER_WF
                  DO Jw = 1,Ns_wf  ;  Jw1 = Nc_der(Jw)
c                     WRITE(*,'(*(2x,e12.4))')
c     $                    LBC(Jw1), SOL_EXP(Nr)*EX_L(Nr)*XM_S1(Nr)*W_FOV
c                     WRITE(*,'(*(2x,e12.4))') DS_sun_surf(:)
c                     WRITE(*,'(*(2x,e12.4))') DS_SUN(Nr,:)
                     PWF%WF_esp(:,Index,Jw) = PWF%WF_esp(:,Index,Jw) 
     $                    - LBC(Jw1)*( DS_sun_surf(:) + DS_SUN(Nr,:) )
     $                      *SOL_EXP(Nr)*EX_L(Nr)*XM_S1(Nr)*W_FOV
     $                    - Te_Surf*DS_SUN(Nr,:)*EX_L(Nr) *W_FOV!*XM_S1(Nr)
c                     WRITE(*,'(*(2x,e12.4))') PWF%WF_esp(:,Index,Jw)
                  ENDDO  ! over Stokes components (Jw)
               ENDIF  !  DO_WF

c              ... For test with numerical perturbations
               IF( NUM_WF ) THEN
c                 ... To calulate without contribution from solar exponent
                  IF( NO_SOL ) AL_1(:) = AL_1_PERT(:)
c                 ... To calulate without contribution from LS exponent
                  IF( NO_LS  ) AL_1(:) = AL_1_BASE(:)
               ENDIF
c               write(*,*) SSSF(1,1), SSSF(Nr,1)
c               write(*,*) S_F(1,1), S_F(Nr,1)
               
c              ... Numerical integration along a line-of-sight
               S_F(:,:) = S_F(:,:) + SSSF(:,:)  
               DUM = 0.d0
               DO I = 1, Nr
                  DUM = DUM + S_F(I,:)*(EX_1(I-1) - EX_1(I))/AL_1(I)
               END DO
               
               SELECT CASE( TRIM(SCATT_MODE) )
               CASE('ss')
c                  write(*,*) '   "st_ss_stokes.f "'
c                  write(*,*)  BC_S1(:), EX_L(Nr)
                  RESULT(IA,IV,IS,IP,:) = DUM(:) + BC_S1(:)*EX_L(Nr)
               CASE DEFAULT
c                 ... Reflection from surface will be accounted for in
c                     "sfi_spher.f"
                  RESULT(IA,IV,IS,IP,:) = DUM(:)
               END SELECT
            END DO  ! over azimuth angles (IP)
         END DO  ! over solar zenith angles (IS)
         NULLIFY(ZLS, SID, IOB, XM_1, EX_L, XM_S1, TH)
         DEALLOCATE( EX_1 )
         IF( DO_CLOUD ) DEALLOCATE(CI, SCCNO_LS)
      END DO  ! over viewing angles including FOV (IV)

      IF( SRR%Exi_WL(JPOS) ) THEN 
         DO Jex = 1,SRR%NUM_IN(JPOS)
            To_act = SRR%ACT_IN(JPOS,Jex) 
            CALL SET_OUT_RAD(Del_ss(:,:,:,:,:,Jex), RAD_OUT)     
            DEL_RRS(To_act,1:NLO,:) = DEL_RRS(To_act,1:NLO,:) 
     $           + RAD_OUT(1:NLO,:)*E_W
c            WRITE(*,*) '  DEL_RRS exi ', To_act, DEL_RRS(To_act,:,:)
         ENDDO
c         ... No standard calculations for excitation wavelength 
c             in the case of rotational Raman scattering
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN 
      ENDIF

c     ... Raman output for an actual wavelength
      IF( DO_RAMAN ) THEN
         CALL SET_OUT_RAD(Del_ss(:,:,:,:,:,1), RAD_OUT)     
         DEL_RRS(SRR_Jact,1:NLO,:) = DEL_RRS(SRR_Jact,1:NLO,:) 
     $        + RAD_OUT(1:NLO,:)*E_W
c            WRITE(*,*) '  DEL_RRS act ss ', 
c     $        SRR_Jact, DEL_RRS(SRR_Jact,:,:)
      ENDIF

      CALL SET_OUT_RAD(RESULT, RAD_OUT)

      IF(LOG_LEVEL(1) >= N_ESO) WRITE(LUNIT,'(a24,4(1x,e12.4))')
     $     'Single scattering spher:', RAD_OUT(1,:)*E_W

      SELECT CASE( TRIM(SCATT_MODE) )
      CASE('ss')
c        ... In the case of single scattering approximation results are
c            stored direct in the output pointer-array STOK_OUT(;,:,:,:,:)
c            N_OP = 1 for this mode
         STOK_OUT(SRR_Jact,1,1,1:NLO,:) = STOK_OUT(SRR_Jact,1,1,1:NLO,:) 
     $        + RAD_OUT(1:NLO,:)*E_W
c         DO I = 1, NLO
c            WRITE(*,'(1x,I3,*(1x,e15.7))') I, Rad_out(I,1),
c     $           STOK_OUT(SRR_Jact,1,1,I,1) 
c         ENDDO
      CASE DEFAULT
c        ... Results are stored in temporary array RADOUT(:,:,:)
c            N_OP = 1 for this mode
          RADOUT(1,:,:) = RAD_OUT(1:NLO,:)*E_W
      END SELECT

c     ... Output for WF in the case of single scattering approximation
c     ================================================================
      IF( WF_FLAG ) THEN
         LWF%WF_ext => PWF%WF_esp
         IF(INQ_WF('tg', 'WFT')) 
     $        CALL SET_OUTPUT_CWF(JPOS, LWF, DO_PLANCK, TAC, E_W)
         IF(INQ_WF('press', 'WFN')) CALL SET_OUTPUT_PWF 
     $       (JPOS, LWF, LOA%SCR, TAC, DO_PLANCK, PWF%P_WF_ABSTERM, E_W)
         IF(INQ_WF('temp', 'WFN')) CALL SET_OUTPUT_TWF
     $       (JPOS, LWF, LOA%SCR, TAC, DO_PLANCK, PWF%T_WF_ABSTERM, E_W)
         IF( Ray_to_WF ) THEN 
            LWF%WF_scr => PWF%WF_ssp(:,:,:,1)
            IF(INQ_WF('ray', 'WFN')) 
     $           CALL SET_OUTPUT_RAY(JPOS, LWF, LOA%SCR, E_W)
         ENDIF
         IF(INQ_WF('atom', 'WFT')) THEN  ;  Ka = 1            
c           ... Preliminary in the same array as Rayleigh
            LWF%WF_met => PWF%WF_ssp(:,:,:,1)
c            CALL SET_OUTPUT_MET(JPOS, LWF, LOA%EMC(Ka,:), E_W)
            Ka = FIND_IDX('atom', LOA%TG_CS%TYPE)
            CALL SET_OUTPUT_MET(JPOS, LWF, LOA%TG_CS(Ka)%CS(:), E_W)
         ENDIF
         IF(INQ_WF('aer', 'WFT')) THEN 
            LWF%WF_sca => PWF%WF_ssp
            CALL SET_OUTPUT_AER(JPOS, LWF, LOA%SCA, LOA%EXA, 
     $           DO_PLANCK, E_W, POS)
         ENDIF
      ENDIF  ! WF_FLAG

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS

c *************************************************************************
c     ... Raman scattering source function 
      SUBROUTINE RAM_SSF
      INTEGER  Isf
      REAL (KND), POINTER :: ALP(:)

        Npl = Nlg_Ray
        P_MOM => SRR%RAM_MOMS(1:Npl,1) ; Phm(:,:) = 0.d0
        FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0(1:Npl,J)*P_MOM(:) )

        IF( Nvs > 1 ) THEN
           P_MOM => SRR%RAM_MOMS(1:Npl,5) 
           FORALL( J = 1:NR ) Phm(J,2:3) = Rot_l(J,2:3)
     $          *SUM( Wig2(1:Npl,J)*P_MOM(:) )
        ENDIF

        IF( .NOT.SRR%Exi_WL(JPOS) ) THEN
c          ... S_F is used as dummy array within "ram_ssf.f"
           FORALL( K=1:Nvs ) S_F(1:Nr,K) = 
     $          COF_LS(1:Nr,1)*Phm(1:NR,K)*SOL_EXP(1:Nr)/4.d0
c          ... Numerical integration along a line-of-sight
           DUM = 0.0D0
           DO I = 1, Nr
c              DUM = DUM + S_F(I,:)*EX_L(I)*(D_DS(1,I) + D_DS(2,I))
              DUM = DUM + S_F(I,:)/AL_1(I)*(EX_1(I-1) - EX_1(I))
           END DO

           Del_ss(1,IV,IS,IP,:,1) = Del_ss(1,IV,IS,IP,:,1) + IRR*DUM(:)
        ELSE
c       ... Over all Raman lines within corresponding excitation bin 
           DO Jex = 1,SRR%NUM_IN(JPOS)
c             ... The actual wavelength number which becomes
c                 contribution from Jex-th Raman line
              To_act = SRR%ACT_IN(JPOS,Jex) 
c             ... Number of ESFT coefficients at actual wavelength
              Nsf = SRR%NESFT(To_act)
c             ... ESFT coefficients at actual wavelength (stored 
c                 in "dom_spherical.f"
              Wes(1:Nsf) = SRR%WEI(To_act,1:Nsf)
c             ... S_F is used as dummy array within "ram_ssf_exi.f"
              FORALL( K=1:Nvs ) S_F(1:Nr,K) = 
     $             COF_LS(1:Nr,Jex)*Phm(1:Nr,K)*SOL_EXP(1:Nr)/4.d0
c             ... Do-loop over ESFT coefficients (if present)
c                 To integrate over l-of-s it will be used here
c                 solar exponent        - at excitation wavelength
c                 extinction coeff and
c                 exponent along l-of-s - at actual wavelength
              DO Isf = 1,Nsf
                 ALP => AL_L(:,Isf,Jex)
                 EXL => SRR%LS_ACT(IA,IV)%EX(:,To_act,Isf)
c                ... Numerical integration along a line-of-sight
                 DUM = 0.0D0
                 DO I = 1, Nr
                    DUM = DUM + S_F(I,:)*EXL(I)*(D_DS(1,I) + D_DS(2,I))
c                    DUM = DUM + S_F(I,:)/ALP(I)*(EXL(I-1) - EXL(I))
                 END DO
                 Del_ss(1,IV,IS,IP,:,Jex) = Del_ss(1,IV,IS,IP,:,Jex)
     $                + IRR*Wes(Isf)*DUM(:)
              ENDDO  ! over esft coefficients (Isf)
           ENDDO  ! over Raman lines from this bin (Jex)
        ENDIF  ! excitation wavelength
      END SUBROUTINE RAM_SSF

c *************************************************************************
c     ... Rayleigh scattering source function
      SUBROUTINE RAY_SSF
      REAL (KND)  HH(GeoD%MaxLS,Nvs)

        Npl = Nlg_Ray  ;  P_MOM => LOA%RAY_MOMS(1:Npl,1)
        FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0(1:Npl,J)*P_MOM(:) )
c       ... If polarization is accounted for
        IF( Nvs > 1 ) THEN
           Phm(:,4) = 0.d0
           P_MOM => LOA%RAY_MOMS(1:Npl,5) 
           FORALL( J=1:NR ) Phm(J,2:3) = Rot_l(J,2:3)
     $          *SUM( Wig2(1:Npl,J)*P_MOM(:) )
        ENDIF  ! Rayleigh vector
c       ... Rayleigh contribution into source function
        FORALL( K=1:Nvs ) SSSF(1:NR,K) = SCRL(1:NR)*Phm(1:NR,K)

c     ... If atoms or ions emission should be calculated:
        IF( Emi_FLG ) THEN
           DO Ka = 1,N_AT
              DO J = 1,NR
                 HH(J,1) = SUM( Wig0(1:Npl,J)*LOA%EPF(Ka,:,1) )
              ENDDO
              IF( Nm_vs > 1 ) THEN
                 HH(:,4) = 0.d0
                 DO J = 1,NR
                    HH(J,2:3) = Rot_l(J,2:3)
     $                   *SUM( Wig2(1:Npl,J)*LOA%EPF(Ka,:,5) )
                 ENDDO
              ENDIF
              FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) + 
     $             EMCL(Ka,1:NR)*HH(1:NR,K)
           ENDDO   ! over atoms (Ka)
        ENDIF  ! atoms emission

c       ... If WF for Rayleigh scattering should be calculated
        IF( Ray_to_WF ) S_R(1:NR,1,1) = 
     $       SOL_EXP(1:NR)/4.d0*Phm(1:Nr,1)/AL_1(1:Nr)
        
c       ... If WF for emission should be calculated (N_AT=1 only!!)
        IF(INQ_WF('atom', 'WFT')) S_R(1:NR,1,1) = 
     $       SOL_EXP(1:NR)/4.d0* HH(1:Nr,1) /AL_1(1:Nr)

      END SUBROUTINE RAY_SSF

c *************************************************************************
c     ... Add cloud scattering source function
      SUBROUTINE CLD_SSF
      INTEGER  I1, I2, Jmc
      REAL (KND) DL, Phm_mc(GeoD%MaxLS,Nm_ice,Nvs)
      Phm_mc = 0.d0
      Npl = Leg_cld
      DO J = 1,Nr
         IF( CI(J) == 0 ) CYCLE
         DO Jmc = 1, MLCS(CI(J))%N_pha
            IF( MLCS(CI(J))%Npf(Jmc)%Y_Scm ) THEN
c           ... Use scattering matrix:
               P_MOM => OSM%CLD_SCM(1:Ydb_ang,CI(J),1,Jmc,1) 
               CALL LIN_SEARCH_NEW
     $              (COSCA(J), DL, I1, I2, Ydb_ang, SCAT_ANG)
               Phm_mc(J,Jmc,1) = P_MOM(I2)*DL + P_MOM(I1)*(1.d0 - DL)
               IF( Nvs == 1 ) CYCLE
               P_MOM => OSM%CLD_SCM(1:Ydb_ang,CI(J),1,Jmc,5) 
               Phm_mc(J,Jmc,2:3) = Rot_l(J,2:3)*
     $              P_MOM(I2)*DL + P_MOM(I1)*(1.d0 - DL)
               CYCLE
            ENDIF
c           ... Use expansion coefficient:
            Phm_mc(J,Jmc,1) =
     $           SUM(Wig0(1:Npl,J)*OSM%CLD_ALL(1:Npl,CI(J),1,Jmc,1))
            IF( Nvs == 1 ) CYCLE
            Phm_mc(J,Jmc,2:3) = Rot_l(J,2:3)
     $           *SUM(Wig2(1:Npl,J)*OSM%CLD_ALL(1:Npl,CI(J),1,Jmc,5))
         ENDDO  ! over water/ice pgase (Jmc)
      ENDDO  ! over points along LoS (J)

      FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) 
     $     + Phm_mc(1:Nr,1,K)*SCCNO_LS(1:Nr,1)
      IF( Nm_ice == 2 ) FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) 
     $     + Phm_mc(1:Nr,2,K)*SCCNO_LS(1:Nr,2)
      
      END SUBROUTINE CLD_SSF

c *************************************************************************
c     ... Add aerosol scattering source function
      SUBROUTINE AER_SSF
      
      SELECT CASE( APH_TYPE )
      CASE('mie')  
c         WRITE(*,*) '   =====> Option "mie" '
c     ... Aerosol scattering is defined according to 
c           expansion coefficients
        Npl = Leg_Aer  ; Phm(:,:) = 0.d0
        FORALL( J=1:Nr ) Phm(J,1) = SUM( Wig0(1:Npl,J)
     $          *OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),1) )
        S_H(1:NR) = Phm(1:Nr,1)
c--------------------------------------------------------------------- 
!        WRITE(*,*) '   st_ss_stokes: Layer Nr = ', Nr!
!        WRITE(*,*) '   st_ss_stokes:      Npl = ', Npl
!        K = Nr
!        DO J = 1, Nr
!           WRITE(*,'(2x,I3, 2x,F8.4, 2(2x,e12.4),2x,E15.7)')
!     $          IDX_AER(IOB(J)), 
!     $          Z_FINE(K), D_DS(1,J), D_DS(2,J), Ex_L(J)
!           K = K - 1
!        ENDDO
c--------------------------------------------------------------------- 
        IF( Nvs > 1 ) THEN
           FORALL( J=1:Nr ) Phm(J,2:3) = Rot_l(J,2:3)
     $        *SUM( Wig2(1:Npl,J)*OSM%AER_ALL(1:Npl,IDX_AER(IOB(J)),5) )
        ENDIF
        FORALL( K=1:Nvs ) SSSF(1:NR,K) = SSSF(1:NR,K) 
     $       + Phm(1:Nr,K)*SCAL(1:Nr)        
c              WRITE(*,*) '   Phm ',  Phm(1:Nr,1)                

c       ... If weighting function for aerosol scattering/absorption
c           should be calculated
        IF( INQ_WF('aer', 'WFT')  ) THEN 
c           WRITE(*,*) '   =====> Option "mie" '
           S_R(1:NR,1,1) = SOL_EXP(1:NR)/4.d0*S_H(1:NR)/AL_1(1:Nr)
c           write(*,*) '  S_R (0)  ', S_R(1:10,1,1)
           IF( Dwr_mi ) THEN
              DO Idr = 1,N_dwr
                 FORALL(J = 1:Nr) Phm(J,1) = SUM( Wig0(1:Npl,J)
     $                *OSM%DER_ALL(1:Npl,IDX_AER(IOB(J)),1,Idr) )
                 S_R(1:Nr,1,1+Idr ) = Phm(1:Nr,1)
     $                *SOL_EXP(1:NR)/4.d0*SCAL(1:Nr)/AL_1(1:Nr)
              ENDDO
           ENDIF
        ENDIF

      CASE('h-g') 
c        ... Aerosol scattering is defined according to H-G 
c            phase function
         S_H(1:NR) = 1.d0 + G_aer(1:Nr)*G_aer(1:Nr)
     $        - 2.d0*G_aer(1:Nr)*COSCA(1:Nr)
         S_H(1:NR) = (1.d0 - G_aer(1:Nr)*G_aer(1:Nr))
     $        /S_H(1:Nr)/DSQRT(S_H(1:Nr))
c            WRITE(*,*) G_aer(1), COSCA(1), SCAL(1)
         SSSF(1:NR,1) = SSSF(1:NR,1) + S_H(1:Nr)*SCAL(1:Nr)
c       ... If weighting function for aerosol scattering/absorption
c           should be calculated
        IF( INQ_WF('aer', 'WFT')  ) 
     $        S_R(1:NR,1,1) = SOL_EXP(1:NR)/4.d0*S_H(1:NR)/AL_1(1:Nr)

      END SELECT

      END SUBROUTINE AER_SSF

c *************************************************************************
c     ... Rotational matrix
      SUBROUTINE ROT_MATR
      REAL (KND)  Azm, Zn, T2, Xmu, Mu0

      Rot_l = 0.d0
c      Mu and Mu0 have to be correspond to the tau-axis
      DO J = 1,NR
         Xmu = -XM_1(J)  ;  Mu0 = XM_S1(J) ; Azm = PHI_LOS(J)
         IF( REAL(Azm)==REAL(Pie) .OR. REAL(AZM)==0.0) THEN 
            Rot_l(J,2) = 1.d0
            Rot_l(J,3) = 0.d0
            CYCLE
         ENDIF
         IF( Azm > Pie ) THEN 
            Zn = 1.d0
         ELSE
            Zn = -1.d0
         ENDIF
         IF( DABS(Xmu) == 1.0d0 ) THEN
            T2 = DSIGN(1.d0,Xmu)*DCOS(Azm)
         ELSE
            T2 = -( Mu0 - Xmu*COSCA(J) )
     $           /DSQRT((1.d0 - Xmu*Xmu)*(1.d0-COSCA(J)**2))
         ENDIF
         Rot_l(J,2) = 2.d0*T2*T2 - 1.d0 
         Rot_l(J,3) = Zn*2.d0*DSQRT(DABS(1.d0 - T2*T2))*T2
      ENDDO
      END SUBROUTINE ROT_MATR

c     ******************************************************************
      SUBROUTINE SET_COEFF
      INTEGER  J, Ji(2), Ki(2), I, K
      REAL (KND) Di(2)
        Ji(:) = 0
        IF( GeoD%LVG(IA,IV)%TH_FLG ) THEN
           Ji(1) = NTH ; Ki(1) = SID(NTH) ; K2 = Ki(1)
           Di(1) = ( ZLS(NTH) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF
        IF( GeoD%LVG(IA,IV)%FP_FLG ) THEN
           Ji(2) = 1 ; Ki(2) = SID(1) ; K2 = Ki(2)
           Di(2) = ( ZLS(1) - Z_FINE(K2+1) )
     $          /( Z_FINE(K2) - Z_FINE(K2+1) )
        ENDIF

        AL_1(1:NR) = ALO( SID(1:NR) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I)
           AL_1(Ji(I)) = ALO(K2+1) + (ALO(K2) - ALO(K2+1))*Di(I)
        ENDDO

        IF( DO_RAMAN ) THEN 
           DO Jex = 1,NRL
              COF_LS(1:NR,Jex) = COF_FI( SID(1:NR),Jex )
              DO I = 1,2
                 IF( Ji(I)==0 ) CYCLE
                 K2 = Ki(I)
                 COF_LS(Ji(I),Jex) = COF_FI(K2+1,Jex) 
     $                + (COF_FI(K2,Jex) - COF_FI(K2+1,Jex))*Di(I)
              ENDDO
              IF( SRR%ACT_WL(JPOS) ) CYCLE

              DO K = 1, SRR%M_ESF
                 AL_L(1:Nr,K,Jex) = AL_F( SID(1:Nr),K,Jex )
                 DO I = 1,2
                    IF( Ji(I)==0 ) CYCLE
                    K2 = Ki(I)
                    AL_L(Ji(I),K,Jex) = AL_F(K2+1,K,Jex) 
     $                   + (AL_F(K2,K,Jex) - AL_F(K2+1,K,Jex))*Di(I)
                 ENDDO                 
              ENDDO    ! over esft coefficients (K)
           ENDDO    ! over Raman lines (Jex)
        ENDIF
c       ... If wavelength is marked as excitation contribution
c           of all "standard" component will be ignored
        IF( SRR%Exi_WL(JPOS) ) RETURN 

        SCRL(1:Nr) = SCRO( SID(1:Nr) )
        DO I = 1,2 
           IF( Ji(I)==0 ) CYCLE
           K2 = Ki(I) 
           SCRL(Ji(I)) = SCRO(K2+1) + (SCRO(K2)-SCRO(K2+1))*Di(I)
        ENDDO

        IF( Emi_FLG ) THEN
c          ... Emission coefficient along line-of-sight
           DO J = 1,N_AT
              EMCL(J,1:Nr) = EMCO( J,SID(1:Nr) )
              DO I = 1,2 
                 IF( Ji(I)==0 ) CYCLE
                 K2 = Ki(I) 
                 EMCL(J,Ji(I)) = EMCO(J,K2+1) 
     $                + (EMCO(J,K2) - EMCO(J,K2+1))*Di(I)
              ENDDO
           ENDDO
        ENDIF ! atoms emission

        IF( DO_AEROSOL ) THEN
           G_aer(1:NR) = ASCO( SID(1:NR) ) 
c                WRITE(*,*) '   SCAO( SID(1:NR) ) ', SCAO( SID(1:NR) )
           SCAL(1:NR)  = SCAO( SID(1:NR) )
           DO I = 1,2 
              IF( Ji(I)==0 ) CYCLE
              K2 = Ki(I) ; D1 = Di(I)
              G_aer(Ji(I)) = ASCO(K2+1) + (ASCO(K2)-ASCO(K2+1))*D1 
              SCAL (Ji(I)) = SCAO(K2+1) + (SCAO(K2)-SCAO(K2+1))*D1 
           ENDDO
        ENDIF

        IF( DO_CLOUD ) THEN
           CI(1:NR) =  INT( CLA(IOB(1:NR),2) )
           DO J = 1, Nm_ice
              SCCNO_LS(1:NR,J) = SCC_NSO(SID(1:NR),J)
           ENDDO
        ENDIF

        IF( DO_PLANCK ) THEN 
           SF_TE(1:NR) = TE%PLA_FINE( SID(1:NR) ) 
           DO I = 1,2  ;  IF( Ji(I)==0 ) CYCLE  ;  K2 = Ki(I)
              SF_TE(Ji(I)) = TE%PLA_FINE(K2+1) 
     $             + (TE%PLA_FINE(K2) - TE%PLA_FINE(K2+1))*Di(I)
           ENDDO
           IF( DO_WF ) THEN
              S_R(1:NR,1,1) = TE%B_FINE( SID(1:NR) ) 
              DO I = 1,2  ;  IF( Ji(I)==0 ) CYCLE  ;  K2 = Ki(I)
                 S_R(Ji(I),1,1) = TE%B_FINE(K2+1) 
     $                + (TE%B_FINE(K2) - TE%B_FINE(K2+1))*Di(I)
              ENDDO
           ENDIF
        ENDIF
        
      END SUBROUTINE SET_COEFF

      END
C *************************************************************************
