C***********************************************************************
C
C       SCIATRAN - Version 3.6 - Last modification: 06 Jul 2016
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C***********************************************************************

C       Collection of tools. 
c       Contains:  Angstrom_interp, LOWER_TO_UPPER, SHIFT_1,   
C                  SKIP_HEADER, INT_TO_CHAR, CHINV3, CHINV3_OLD, 
c                  SMOOFT with dependencies, WRITE_UD, REALFT, 
c                  COEF_TO_SCATMATR
C
C**************************************************************************

      SUBROUTINE Angstrom_interp(NWL, WLI, TAUI, WL_REF, T_REF)

      USE TYPE_VARS, ONLY: KND

      INTEGER,   INTENT(IN) :: NWL
      REAL(KND), INTENT(IN) :: WL_REF
      REAL(KND), INTENT(IN) :: WLI(NWL), TAUI(NWL)
      REAL(KND) T_REF
c     ... Local parameters
      INTEGER I0, I1, I2
      REAL(KND) Pok
c     ==================================================================
      IF( NWL== 2 ) THEN
         I1 = 1 ; I2 = 2
      ELSE
         IF( WL_REF >= WLI(NWL) ) THEN
            I1 = NWL - 1 ; I2 = NWL
         ENDIF
         IF( WL_REF <= WLI(NWL) ) THEN
            I1 = 1 ; I2 = 2
         ENDIF            
         I0 = MINLOC( DABS(WLI(:) - WL_REF),1 )
         IF( WL_REF >= WLI(I0) ) THEN 
            I1 = I0 ; I2 = I0 + 1
         ELSE
            IF( I0 == 1 ) THEN
               I1 = 1 ; I2 = 2
            ELSE
               I1 = I0 - 1 ; I2 = I0
            ENDIF
         ENDIF
      END IF
      Pok = DLOG( TAUI(I1)/TAUI(I2) )/DLOG( WLI(I1)/WLI(I2) )
      T_REF = TAUI(I2)*(WL_REF/WLI(I2))**Pok
      END SUBROUTINE Angstrom_interp

C***********************************************************************
      SUBROUTINE LOWER_TO_UPPER (INPUT_LABEL)
      INTEGER           ASCII_POS, I
      CHARACTER (LEN=*) INPUT_LABEL
      DO I = 1, LEN_TRIM(INPUT_LABEL)
         ASCII_POS = IACHAR(INPUT_LABEL(I:I))
         IF(ASCII_POS >= 97 .AND. ASCII_POS <= 122) 
     $        ASCII_POS = ASCII_POS - 32
         INPUT_LABEL(I:I) = ACHAR(ASCII_POS)
      END DO
      RETURN
      END

C***********************************************************************
      SUBROUTINE SHIFT_1(A0_WL, AR0, N_CEN, SME, DIFF, L_P, ZNAK, *)
      USE LOG_CONTROL, ONLY: LOG_LEVEL, N_RSO, LUNIT

      USE OutInf,      ONLY: EU
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SHIFT_1'
      INTEGER           N_CEN, INBV, L_P, I, N_COEFF, KK
c      INTEGER           LCK, LWRK, IFAIL
c      DOUBLE PRECISION LAMDA(N_CEN+4),  C(N_CEN+4), WRK(6*N_CEN+16)
      DOUBLE PRECISION, DIMENSION (N_CEN) :: A0_WL, AR0, SME, X
c     $     ,B_SC, C_SC, D_SC
      DOUBLE PRECISION  DIFF(N_CEN - 2*L_P), WORK(5*(N_CEN+2)), ZNAK
      DOUBLE PRECISION  B_COEFF(N_CEN + 2), S_NODES(N_CEN+6)
      DOUBLE PRECISION, EXTERNAL :: SEVAL

c      LCK = N_CEN + 4; LWRK = 6*N_CEN + 16
      X(1:N_CEN) = A0_WL(1:N_CEN) + ZNAK*SME(1:N_CEN)

      IF(X(1 + L_P).LT.A0_WL(1)) GoTo 200
      IF(X(N_CEN - L_P).GT.A0_WL(N_CEN)) GoTo 201

c      CALL E01BAF(N_CEN, A0_WL, AR0, LAMDA, C, LCK, WRK, LWRK, IFAIL)     
c      CALL SPLINE (N_CEN, A0_WL, AR0, B_SC, C_SC, D_SC) 
      CALL DBINT4 (A0_WL, AR0, N_CEN, 2, 2, 0.0D0, 0.0D0, 1, S_NODES,
     $     B_COEFF, N_COEFF, KK, WORK,*880) 
      INBV = 1
      DO I = L_P + 1, N_CEN - L_P
c         CALL E02BBF(LCK, LAMDA, C, X(I), DIFF(I - L_P), IFAIL)
c         DIFF(I - L_P) = 
c     $        SEVAL(N_CEN, X(I), A0_WL, AR0, B_SC, C_SC, D_SC)
          Call DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0,  
     $        X(I), INBV, WORK(1:12), DIFF(I - L_P), *880)
      END DO

      RETURN
      
 200  CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong value for a shift')
      WRITE(EU,*) '********* Left bound ***********'
      WRITE(EU,*) X(1 + L_P),A0_WL(1)
      WRITE(EU,*) '********************************'
      RETURN 1

 201  CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong value for a shift')
      WRITE(EU,*) '********* Right bound ***********'
      WRITE(EU,*) X(N_CEN - L_P),A0_WL(N_CEN)
      WRITE(EU,*) '*********************************'
      RETURN 1
        
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      return 1
      END

C***********************************************************************
      SUBROUTINE SHIFT_2(A0_WL, AR0, N_CEN, SME, DIFF, L_P, ZNAK, *)
      USE LOG_CONTROL, ONLY: LOG_LEVEL, N_RSO, LUNIT

      USE OutInf,      ONLY: EU     
      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'SHIFT_1'
      INTEGER           N_CEN, INBV, L_P, I, N_COEFF, KK
c      INTEGER           LCK, LWRK, IFAIL
c      DOUBLE PRECISION LAMDA(N_CEN+4),  C(N_CEN+4), WRK(6*N_CEN+16)
      DOUBLE PRECISION, DIMENSION (N_CEN) :: A0_WL, AR0, SME, X
c     $     ,B_SC, C_SC, D_SC
      DOUBLE PRECISION  DIFF(N_CEN - 2*L_P), WORK(5*(N_CEN+2)), ZNAK
      DOUBLE PRECISION  B_COEFF(N_CEN + 2), S_NODES(N_CEN+6)
      DOUBLE PRECISION, EXTERNAL :: SEVAL 

c      LCK = N_CEN + 4; LWRK = 6*N_CEN + 16
      X(1:N_CEN) = A0_WL(1:N_CEN) + ZNAK*SME(1:N_CEN)

      IF(X(1 + L_P).LT.A0_WL(1)) GoTo 200
      IF(X(N_CEN - L_P).GT.A0_WL(N_CEN)) GoTo 201

c      CALL E01BAF(N_CEN, A0_WL, AR0, LAMDA, C, LCK, WRK, LWRK, IFAIL)     
c      CALL SPLINE (N_CEN, A0_WL, AR0, B_SC, C_SC, D_SC) 
      CALL DBINT4 (X, AR0, N_CEN, 2, 2, 0.0D0, 0.0D0, 1, S_NODES,
     $     B_COEFF, N_COEFF, KK, WORK, *880) 
      INBV = 1
      DO I = L_P + 1, N_CEN - L_P
c         CALL E02BBF(LCK, LAMDA, C, X(I), DIFF(I - L_P), IFAIL)
c         DIFF(I - L_P) = 
c     $        SEVAL(N_CEN, X(I), A0_WL, AR0, B_SC, C_SC, D_SC)
          Call DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0,  
     $        A0_WL(I), INBV, WORK(1:12), DIFF(I - L_P), *880)
      END DO

      RETURN

 200  CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong value for a shift')
      WRITE(EU,*) '********* Left bound ***********'
      WRITE(EU,*) X(1 + L_P),A0_WL(1)
      WRITE(EU,*) '********************************'
      RETURN 1

 201  CALL WRITE_MESSAGE(R_NAME,'ERROR', 'Wrong value for a shift')
      WRITE(EU,*) '********* Right bound ***********'
      WRITE(EU,*) X(N_CEN - L_P),A0_WL(N_CEN)
      WRITE(EU,*) '*********************************'
      RETURN 1

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      Return 1
      END

C  ********************************************
      CHARACTER FUNCTION INT_TO_CHAR (INPUT_VAL)
      INTEGER             INPUT_VAL, N_DIG, I, JJ, LL, K
      DOUBLE PRECISION    VAL

      N_DIG = FLOOR(ALOG10(FLOAT(INPUT_VAL)))
      LL = LEN(INT_TO_CHAR)
      VAL = DBLE(INPUT_VAL)
      K = 0
      DO I = N_DIG, 0, -1
         K = K + 1
         JJ = FLOOR(VAL/10.**I) 
         INT_TO_CHAR(K:K) = ACHAR(48 + JJ)
         VAL = VAL - JJ*10**I
      END DO
      DO I = N_DIG + 2, LL
         INT_TO_CHAR(I:I) = ' '
      END DO

      RETURN
      END
C **********************************************************************
      SUBROUTINE SKIP_HEADER (INP_UNIT)
      INTEGER             INP_UNIT, K
      CHARACTER (LEN=200) CH_READ
      LOGICAL             NEXT_LINE
      
      DO
         NEXT_LINE = .FALSE.
         READ(INP_UNIT,*) CH_READ
c         WRITE(*,*) "'"//TRIM(CH_READ)//"'"
         DO K = 1,LEN_TRIM( CH_READ )
            SELECT CASE ( IACHAR( CH_READ(K:K) ) )
            CASE( 48:57,43,45,46,68,69,100,101 )
            CASE DEFAULT 
               NEXT_LINE = .TRUE.  ;  EXIT 
            END SELECT
         ENDDO
         IF( .NOT.NEXT_LINE ) THEN 
            BACKSPACE(INP_UNIT)  ;  EXIT
         ENDIF
      ENDDO
      
      RETURN
      END

C***********************************************************************
C*                                                                     *
C*               SUBROUTINE FOR MATRIX INVERSION                       *
C*                                                                     *
C***********************************************************************
!      SUBROUTINE CHINV3_OLD(NVEC,N,A)
!
!      INTEGER           P, Q, R, S, T, U, NVEC, N, I, J, K, K1
!      DOUBLE PRECISION  A(NVEC)
!      DOUBLE PRECISION  X, Y
!      P = 0
!      DO I = 1, N
!         Q = P + 1
!         R = 0
!         DO J = 1, I
!            X = A(P + 1)
!            IF(Q.LE.P) THEN
!               DO K = Q, P
!                  R= R + 1
!                  X = X - A(K)*A(R)
!               END DO
!            END IF
!            R=R+1
!            P=P+1
!            IF(I.NE.J) THEN
!               A(P)=X*A(R)
!               CYCLE
!            END IF
!            A(P)=1/DSQRT(X)
!         END DO
!      END DO
!C  INVERS L MATRIX
!      P=0
!      R=0
!      T=0
!      DO I= 2, N
!         P=P+1
!         R=R+I
!         Y=A(R+1)
!         DO J = 2, I
!            P=P+1
!            T=T+1
!            S=T
!            U=I-2
!            X=0
!            IF(P.GT.R) CYCLE
!            DO K1 = P,R
!               K=R + P - K1
!               X=X - A(K)*A(S)
!               S=S-U
!               U=U-1
!            END DO
!            A(P)=X*Y
!         END DO 
!      END DO
!C  INVERS A MATRIX      
!      P=0
!      DO I = 1, N
!         R = I*(I + 1)/2
!         DO J=1,I
!            S=R
!            P=I*(I - 1)/2 + J
!            T=P
!            X=0
!            DO K=I,N
!               X=X+A(S)*A(T) 
!               S=S+K
!               T=T+K
!            END DO
!          A(P)=X
!      END DO
!      END DO
!      DO I = 1, N
!         R = I*(I + 1)/2
!         DO J = 1, I
!            P = R + J
!            S=R
!            T=P
!            X=0
!            DO K=I,N
!               X=X+A(S)*A(T)
!               S=S+K
!               T=T+K
!            END DO
!            A(P)=X
!      END DO
!      END DO
!      RETURN
!      END
C***********************************************************************
C*                                                                     *
C*               SUBROUTINE FOR MATRIX INVERSION                       *
C*                    CRAY optimized version                           *   
C*                                                                     *
C***********************************************************************
      SUBROUTINE CHINV3(NVEC, N, A)

      USE TYPE_VARS, ONLY: KND

      INTEGER           NVEC, N, K, I, J
      REAL (KND)  A(NVEC), X

      A(1)=1.0D0/DSQRT(A(1))
      FORALL (I = 2:N) A(I*(I - 1)/2 + 1) = A(I*(I - 1)/2 + 1)*A(1)

      DO I = 2, N
         DO J = 2, I - 1
            X = A(I*(I - 1)/2 + J)
            DO K = 1, J - 1
               X = X - A(I*(I - 1)/2 + K)*A(J*(J - 1)/2 + K)
            END DO
            A(I*(I - 1)/2 + J)=X*A(J*(J - 1)/2 + J)
         END DO
         X = A(I*(I + 1)/2)
         DO K = I*(I - 1)/2 + 1, I*(I + 1)/2  - 1
            X = X - A(K)*A(K)
         END DO
         A(I*(I + 1)/2) = 1.0D0/DSQRT(X)
      END DO

C  INVERS L MATRIX
      DO I= 2, N
         DO J = 2, I
            X = 0
            DO  K = 1, I - J + 1
               X = X - A(I*(I + 1)/2 - K)*
     $              A(J - 1 + (I - K - 1)*(I - K)/2)
            END DO
            A(I*(I - 1)/2 + J - 1)=A(I*(I + 1)/2)*X
         END DO
      END DO

C  INVERS A MATRIX
      DO I= 1, N
         DO J = 1, I
            X = 0
            DO  K = I, N
               X = X + A(K*(K - 1)/2 + I)*A(K*(K - 1)/2 + J)
            END DO
            A(I*(I - 1)/2 + J) = X
         END DO
      END DO

      RETURN
      END
C ******************************************************************
!
!      SUBROUTINE SMOOFT(Z_R,N,PTS)
!      
!      INTEGER           MMAX, N, I, M, NMIN, J, MO2, K
!      PARAMETER         (MMAX=6*1024)
!      DOUBLE PRECISION  Z_R(N), Y(MMAX)
!      DOUBLE PRECISION  PTS, CONST, Y1, YN, RN1, FAC
!
!      DO I = 1,N
!         Y(I) = Z_R(I)
!      END DO
!      
!      M=2
!      NMIN=N+2.*PTS
! 1    IF(M.LT.NMIN)THEN
!         M=2*M
!         GO TO 1
!      ENDIF
!      IF(M.GT.MMAX) PAUSE 'MMAX too small in routine SMOOFT'
!      CONST=(PTS/M)**2
!      Y1=Y(1)
!      YN=Y(N)
!      RN1=1./(DBLE(N)-1.)
!      DO J=1,N
!         Y(J)=Y(J)-RN1*(Y1*(N-J)+YN*(J-1))
!      ENDDO
!      IF(N+1.LE.M)THEN
!         DO J=N+1,M
!            Y(J)=0.
!         ENDDO
!      ENDIF
!      MO2=M/2
!      CALL REALFT(Y,MO2,1)
!      Y(1)=Y(1)/MO2
!      FAC=1.
!      DO J=1,MO2-1
!         K=2*J+1
!         IF(FAC.NE.0.)THEN
!c     1			FAC=AMAX1(0.,(1.-CONST*J**2)/MO2)
!            FAC=DMAX1(0.D0,(1.D0-CONST*J**2)/DBLE(MO2))
!            Y(K)=FAC*Y(K)
!            Y(K+1)=FAC*Y(K+1)
!         ELSE
!            Y(K)=0.
!            Y(K+1)=0.
!         ENDIF
!      ENDDO
!c     1	FAC=AMAX1(0.,(1.-0.25*PTS**2)/MO2)
!      FAC=DMAX1(0.D0,(1.D0-0.25D0*PTS**2)/DBLE(MO2))
!      Y(2)=FAC*Y(2)
!      CALL REALFT(Y,MO2,-1)
!      DO J=1,N
!         Y(J)=RN1*(Y1*(N-J)+YN*(J-1))+Y(J)
!      ENDDO
!      
!      DO I = 1,N
!         Z_R(I) = Y(I)
!      END DO
!      
!      RETURN
!      END
!C **********************************************************************
      SUBROUTINE WRITE_UD(IPAR, UNIT)

      Use Tools,   Only: WriteOut
      Use VersNum, Only: VERSION
      
      INTEGER, INTENT(IN) :: UNIT, IPAR
      INTEGER  I
      Character (Len=500)  OutStr
c     ==================================================================
      SELECT CASE (IPAR)
      CASE(1)
         WRITE(OutStr,'(2x,60A)') ('=', I = 1,21),'  SCIATRAN '
     $        //TRIM(VERSION)//'  ', ('=', I = 1,20)
         Call WriteOut (UNIT, OutStr)
         WRITE(OutStr,*)
         Call WriteOut (UNIT, OutStr)
      CASE(2)
         WRITE(OutStr,'(2x,60A)') 
         Call WriteOut (UNIT, OutStr)
         WRITE(OutStr,'(2x,60A)') ('=', I = 1,23),'  End of file  ', 
     $        ('=', I = 1,22)
         Call WriteOut (UNIT, OutStr)
      END SELECT

      RETURN
      END
C ***********************************************************************
!
!      SUBROUTINE REALFT(DATA,N,ISIGN)
C        IMPLICIT REAL*8(A-H,O-Z)
c1	REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
!
!      INTEGER           N, ISIGN, N2P3, I, I1, I2, I3, I4
!      DOUBLE PRECISION  DATA(2*N)
!      DOUBLE PRECISION  THETA, WR, WI, WPR, WPI, WTEMP
!      DOUBLE PRECISION  C1, C2, WRS, WIS, H1R, H1I, H2R, H2I
! 
!      THETA=3.141592653589793D0/DBLE(N)
!      C1=0.5
!      IF (ISIGN.EQ.1) THEN
!         C2=-0.5
!         CALL FOUR1(DATA,N,+1)
!      ELSE
!         C2=0.5
!         THETA=-THETA
!      ENDIF
!      WPR=-2.0D0*DSIN(0.5D0*THETA)**2
!      WPI=DSIN(THETA)
!      WR=1.0D0+WPR
!      WI=WPI
!      N2P3=2*N+3
!      DO I=2,N/2+1
!         I1=2*I-1
!         I2=I1+1
!         I3=N2P3-I2
!         I4=I3+1
!         WRS=SNGL(WR)
!         WIS=SNGL(WI)
!         H1R=C1*(DATA(I1)+DATA(I3))
!              H1I=C1*(DATA(I2)-DATA(I4))
!              H2R=-C2*(DATA(I2)+DATA(I4))
!              H2I=C2*(DATA(I1)-DATA(I3))
!              DATA(I1)=H1R+WRS*H2R-WIS*H2I
!              DATA(I2)=H1I+WRS*H2I+WIS*H2R
!              DATA(I3)=H1R-WRS*H2R+WIS*H2I
!              DATA(I4)=-H1I+WRS*H2I+WIS*H2R
!              WTEMP=WR
!              WR=WR*WPR-WI*WPI+WR
!              WI=WI*WPR+WTEMP*WPI+WI
!           ENDDO
!           IF (ISIGN.EQ.1) THEN
!              H1R=DATA(1)
!              DATA(1)=H1R+DATA(2)
!              DATA(2)=H1R-DATA(2)
!           ELSE
!              H1R=DATA(1)
!              DATA(1)=C1*(H1R+DATA(2))
!              DATA(2)=C1*(H1R-DATA(2))
!              CALL FOUR1(DATA,N,-1)
!           ENDIF
!           RETURN
!           END
!
!      SUBROUTINE FOUR1(DATA,NN,ISIGN)
!C      IMPLICIT REAL*8(A-H,O-Z)
!c1	REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
!      INTEGER           NN, ISIGN, N, J, I, M, MMAX, ISTEP
!      DOUBLE PRECISION  DATA(2*NN)
!      DOUBLE PRECISION  WR,WI,WPR,WPI,WTEMP,THETA, TEMPR, TEMPI
!      N=2*NN
!      J=1
!      DO I=1,N,2
!         IF(J.GT.I)THEN
!            TEMPR=DATA(J)
!            TEMPI=DATA(J+1)
!            DATA(J)=DATA(I)
!            DATA(J+1)=DATA(I+1)
!            DATA(I)=TEMPR
!            DATA(I+1)=TEMPI
!         ENDIF
!         M=N/2
! 1       IF ((M.GE.2).AND.(J.GT.M)) THEN
!            J=J-M
!            M=M/2
!            GO TO 1
!         ENDIF
!         J=J+M
!      ENDDO
!      MMAX=2
! 2    IF (N.GT.MMAX) THEN
!         ISTEP=2*MMAX
!         THETA=6.28318530717959D0/(ISIGN*MMAX)
!         WPR=-2.D0*DSIN(0.5D0*THETA)**2
!         WPI=DSIN(THETA)
!         WR=1.D0
!         WI=0.D0
!         DO M=1,MMAX,2
!            DO I=M,N,ISTEP
!               J=I+MMAX
!               TEMPR=SNGL(WR)*DATA(J)-SNGL(WI)*DATA(J+1)
!               TEMPI=SNGL(WR)*DATA(J+1)+SNGL(WI)*DATA(J)
!               DATA(J)=DATA(I)-TEMPR
!               DATA(J+1)=DATA(I+1)-TEMPI
!               DATA(I)=DATA(I)+TEMPR
!               DATA(I+1)=DATA(I+1)+TEMPI
!            ENDDO
!            WTEMP=WR
!            WR=WR*WPR-WI*WPI+WR
!            WI=WI*WPR+WTEMP*WPI+WI
!         ENDDO
!         MMAX=ISTEP
!         GO TO 2
!      ENDIF
!      RETURN
!      END
c***********************************************************************
c            
c    This routine calculates expansion coefficients of Legendre series
c                   of the scattering function
c     Input parameters:
c          NaC    -   number of aerosol components;
c          NaW    -   number of reference wavelengths;
c          N_gau  -   number of Gauss angles;
c          Na_st  -   number of scattering angles;
c          Sa_inp(1:Na_st) 
c                 -   scattering angles [deg];
c          Phs_inp(1:Na_st,1:NaC,1:NaW)
c                 -   scattering function;
c     Output parameters:
c          M_cof(1:NaC,1:NaW)
c                 -   number of expansion coefficients;
c          Leg_cof(1:N_gau,1:NaC,1:NaW)
c                 -   expansion coefficients;
c***********************************************************************
      SUBROUTINE LEG_COEFF(NaC, NaW, N_gau, Na_st, Sa_inp, Phs_inp, 
     $     M_cof, Leg_cof, *)

      USE TYPE_VARS, ONLY: KND
c     ------------------------------------------------------------------
      INTEGER, INTENT(IN) :: NaC, NaW, Na_st, N_gau
      INTEGER  M_cof(NaC,NaW)
      REAL (KND) Sa_inp(Na_st), Phs_inp(Na_st,NaC,NaW), 
     $     Leg_cof(N_gau,NaC,NaW)
c     ------------------------------------------------------------------
c     Local variables
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'LEG_COEFF'
      INTEGER  I, J, K, L, Ncof
      LOGICAL  Error
      REAL (KND)  Norm, TMP1, RTD
      REAL (KND), DIMENSION (N_gau) :: X_gau, AG_gau, X_rad, Phs_gau
      REAL (KND), DIMENSION (Na_st) :: Sa_rad, Phs_loc
      REAL (KND), ALLOCATABLE ::  Poly(:,:)
c     ------------------------------------------------------------------

c     ... Maximal number of expansion coefficients less than 
c         the number of scattering angles in the range [0 - 180]
      Ncof = N_gau

      ALLOCATE( Poly(Ncof,N_gau) )
      RTD = 180.d0/DACOS(-1.d0)

c     ... Gauss grid and weights for integration (-1,1)
      CALL GAULEG ( -1.d0, 1.d0, X_gau, AG_gau, N_gau )
c     ... Legendre polynomials for all Gauss angles
      CALL WIGNER_PLG( X_gau, N_gau, 0, Ncof, Poly )
      X_rad(:) = DACOS( X_gau(N_gau:1:-1) )
c     ... Convert [deg] to [rad] for interpolation of phase function 
      Sa_rad(:) = Sa_inp(:)/RTD

      DO I = 1, NaC
         DO J = 1, NaW
            Phs_loc(:) = Phs_inp(:,I,J)
c           ... Interpolate scattering function on the Gauss grid
            Phs_loc(:) = DLOG( Phs_loc(:) )
            CALL AKIMA_INTERP(Na_st, Sa_rad, Phs_loc, N_gau, X_rad,  
     $           Phs_gau, Error)  
            IF( Error ) THEN
               Call WRITE_MESSAGE 
     $              (R_NAME,'ERROR', 'Interpolation error')
               RETURN 1
            ENDIF
c           ... Invert to start from -1
            Phs_gau(:) = DEXP( Phs_gau(N_gau:1:-1) )
c           ... Norm of the interpolated phase function
            Norm = SUM( Phs_gau(:)*AG_gau(:) )*0.5d0
c           ... Expansion coefficients of the intrpolated phase function
            DO L = 1, Ncof
               TMP1 = (DBLE(L) - 0.5d0)
     $              *SUM( Phs_gau(:)*Poly(L,:)*AG_gau(:) )/Norm
               IF( TMP1 < 1.d-06 ) EXIT
c               WRITE(*,'(1x,I5,2x,e12.4)') L, TMP1
               Leg_cof(L,I,J) = TMP1
            ENDDO
            M_Cof(I,J) = L-1
         ENDDO   ! over wavelengths (J)
      ENDDO   ! over aerosol components (I)
      
      RETURN
      END SUBROUTINE LEG_COEFF

C*********************************************************************
c         This routine calculates the scattering matrix elements 
c     using as input the expansion coefficients in Wigner functions
c              instead of generalized spherical functions.
c     Input parameters:
c     N_MOM     - number of expansion coefficients;
c     Nm_vs     - number of matrix elements: 1 or 6 in the scalar or
c                                          vector case, respectively;
c     COF(1:N_MOM,1:Nm_vs) 
c               - expansion coefficients of scattering matrix elements;
c     N_ANG     - number of scattering angles [0,180] for which the 
c                 scattering matrix elements will be calculated;
c     DISCR     - type of discretization the range of scattering 
c                 angles [0-180]: 
c                 'G' - Gauss grid 
c                 'E' - equidistant grid
c     OUT_PATH  - path and name for output file containing the 
c                 scattering matrix elements 
c*********************************************************************
      SUBROUTINE COEF_TO_SCATMATR
     $     (N_MOM, Nm_vs, COF, N_ANG, DISCR, OUT_PATH)
c     Input variables and arrays
c     --------------------------
      INTEGER             N_MOM, Nm_vs, N_ANG
      CHARACTER (LEN=1)   DISCR
      CHARACTER (LEN=*)   OUT_PATH
      DOUBLE PRECISION    COF(N_MOM,Nm_vs)
c     Local variable
c     --------------
      INTEGER             I, J
      INTEGER, PARAMETER ::
     $                    OUNIT = 44
      DOUBLE PRECISION    RAD_TO_DEG, STEP, T1, T2
      DOUBLE PRECISION    ANG(N_ANG), 
     $                    SCAT(N_ANG), AG(N_ANG-2), XG(N_ANG-2)      
      DOUBLE PRECISION    Plm_ss(N_MOM,N_ANG), F11(N_ANG)
      DOUBLE PRECISION, ALLOCATABLE ::    
     $                    Dp2_ss(:,:), Dm2_ss(:,:), Plm_as(:,:), 
     $                    F22(:), F33(:), F44(:), F12(:), F34(:)

      RAD_TO_DEG = 180.D0/DACOS(-1.D0)
c      WRITE(*,*) '     N_MOM = ',N_MOM
c      !WRITE(*,*) '     Nm_vs = ',Nm_vs
c     ! WRITE(*,*) '     N_ANG = ',N_ANG
c      !WRITE(*,*) '     DISCR = ',DISCR
c     ! WRITE(*,*) '     OUT_PATH = ',TRIM(OUT_PATH)
      IF( Nm_vs /= 1 .AND. Nm_vs /= 6 ) THEN
         WRITE(*,'(a)') '  Wrong dimension of scattering matrix'
         WRITE(*,'(a,2x,I2)') '  As input :', Nm_vs
         WRITE(*,'(a)') 
     $        "  Accepted 1 for scalar or 6 for vector case only"
         STOP '  Stop in "COEF_TO_SCATMATR"'
      ENDIF

c     ... Select grid for scattering angles representation
      SELECT CASE( DISCR )
      CASE('G')
c        ... Gauss grid 
         CALL GAULEG ( -1.D0, 1.D0, XG, AG, N_ANG-2 )
         SCAT(1) = 1.d0 ; SCAT(N_ANG) = -1.d0
         SCAT(2:N_ANG-1) = XG(N_ANG-2:1:-1)      
         ANG(:) = DACOS( SCAT(:) )*RAD_TO_DEG 
      CASE('E')
c        ... Equidistant grid
         STEP = 180.D0/DBLE( N_ANG-1 ) ; ANG(1) = 0.d0         
         DO I = 2,N_ANG
            ANG(I) = ANG(I-1) + STEP
         ENDDO
         SCAT(:) = DCOS( ANG(:)/RAD_TO_DEG )
      CASE DEFAULT
         WRITE(*,'(a)') '  Wrong type of discretization shema'
         WRITE(*,'(a,2x,a)') '  As input :', DISCR
         WRITE(*,'(a)') "  Accepted 'G' or 'E' only"
         STOP '  Stop in "COEF_TO_SCATMATR"'
      END SELECT

c     ... Legendre polynomials for all scattering angles
      CALL WIGNER_PLG(SCAT, N_ANG, 0, N_MOM, Plm_ss )
      DO J = 1,N_ANG
         F11(J) = SUM( Plm_ss(:,J)*COF(:,1) )
      ENDDO

      IF( Nm_vs == 1 ) THEN
         WRITE(*,*) '   The elements of scattering function'
         WRITE(*,*) '   are written in ',TRIM(OUT_PATH)
         OPEN(OUNIT,FILE=TRIM(OUT_PATH))
          WRITE(OUNIT,'(a)') 
     $       '#  Scattering function corresponding to the '//
     $        'given expansion coefficients '
          WRITE(OUNIT,'(a)') '#  Column :'
          WRITE(OUNIT,'(a)') '#  1 - scattering angle [deg]'
          WRITE(OUNIT,'(a)') '#  2 - normalized phase function'
           DO I = 1,N_ANG
              WRITE(OUNIT,'(2x,f7.2,4(2x,f15.5))') ANG(I), F11(I)
           ENDDO
         CLOSE(OUNIT)
c         PAUSE  '** PAUSE in "COEF_TO_SCATMATR" (st_tools.f) **'
         RETURN
      ENDIF

      ALLOCATE( Dp2_ss(N_MOM,N_ANG), Dm2_ss(N_MOM,N_ANG),
     $     Plm_as(N_MOM,N_ANG), F22(N_ANG), F33(N_ANG), ! A 
     $     F44(N_ANG), F12(N_ANG), F34(N_ANG) )        ! A
      
c     ... Wigner functions for all scattering angles
      CALL WIGNER_PLG(SCAT, N_ANG, 2, N_MOM, Plm_as )
      CALL WIGNER_FUNCTION
     $     (SCAT, N_ANG, 2, N_MOM, Dp2_ss, Dm2_ss )

      DO J = 1,N_ANG
         T1 = SUM( (COF(:,2) + COF(:,3))*Dp2_ss(:,J) )
         T2 = SUM( (COF(:,2) - COF(:,3))*Dm2_ss(:,J) )
         F22(J) =  0.5*( T1 + T2 )
         F33(J) =  0.5*( T1 - T2 )
         F44(J) =  SUM( Plm_ss(:,J)*COF(:,4) )         
c        ... The sign '-' is not used here because coefficients 
c            beta1 and beta2 obtain negative sign after input. 
         F12(J) = SUM( Plm_as(:,J)*COF(:,5) )
         F34(J) = SUM( Plm_as(:,J)*COF(:,6) )
      ENDDO

      WRITE(*,*) '   The elements of scattering matrix'
      WRITE(*,*) '   are written in ',TRIM(OUT_PATH)
      OPEN(OUNIT,FILE=TRIM(OUT_PATH))
      WRITE(OUNIT,'(a)') 
     $     '#  Scattering matrix elements corresponding to the '//
     $     'given expansion coefficients '
      WRITE(OUNIT,'(a)') '#  Column :'
      WRITE(OUNIT,'(a)') '#  1 - scattering angle [deg]'
      WRITE(OUNIT,'(a)') '#  2 - F11'
      WRITE(OUNIT,'(a)') '#  3 - F22'
      WRITE(OUNIT,'(a)') '#  4 - F33'
      WRITE(OUNIT,'(a)') '#  5 - F44'
      WRITE(OUNIT,'(a)') '#  6 - F12'
      WRITE(OUNIT,'(a)') '#  7 - F34'
      DO I = 1,N_ANG
         WRITE(OUNIT,'(2x,f7.2,6(2x,F12.5))') ANG(I), 
     $        F11(I), F22(I), F33(I), F44(I), F12(I), F34(I)
      ENDDO
c      PAUSE
c     $ '** PAUSE in "COEF_TO_SCATMATR" (see collection st_tools.f) **'
      RETURN
      END
C**************************************************************************
