C***********************************************************************
C
C	SCIATRAN - Version 4.3 - Last modification: 03 Dec 2020
C
C              Solution at the top and the bottom of a medium
C                  at all corresponding Gauss angles
C
C***********************************************************************
      SUBROUTINE ST_TOP_BOT(Mh, NrL, NN, NNv, NSZA, Nc1, Nc2, Ex1, ExN, 
     $     PVEC, G_1, G_2, PA_1, PA_2, UDB, SOL_TOP, SOL_BOT, FLUX, 
     $     Ref_surf, AXM, AG, ADO)
c     ... Called by "solv_dom.f" and "iterate_coa.f:"

      USE BRDF,        ONLY: Exact_brf, Npg
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: LUNIT, LOG_LEVEL, N_XSO, N_DSO 
      USE ST_VARS,     ONLY: N_RHS, N_Adj, WF_FLAG, OWF_FLAG, SDVI, Kvs
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: ADO_TYPE, UDB_TYPE
      USE WATER,       ONLY: UWT
c     -----------------------------------------------------------------

      INTEGER,    INTENT(IN) ::  Mh, NrL, NN, NNv, NSZA, Nc1, Nc2
      REAL (KND), INTENT(IN), DIMENSION(NNv) :: Ex1, ExN
      REAL (KND), INTENT(IN), DIMENSION(2*NNv,2*NNv):: G_1, G_2
      REAL (KND), INTENT(IN), DIMENSION(2*NNv,Kvs*NSZA) :: PA_1, PA_2    
      REAL (KND), INTENT(IN) :: PVEC(2*NNv*(NRL-1),N_RHS), AXM(NN),
     $     AG(NN)
      TARGET :: PVEC
      REAL (KND), INTENT(OUT) :: Flux(Kvs*NSZA), 
     $     Ref_surf(N_RHS,Nvs*(NN+Npg))
      REAL (KND), INTENT(OUT), DIMENSION(2*NNv,N_RHS) :: SOL_TOP, 
     $     SOL_BOT
c     ------------------------------------------------------------------
      TYPE (ADO_type) :: ADO    ! Adjoint DOM auxiliary variables
      TYPE (UDB_TYPE) :: UDB
c     ... Local variables
      CHARACTER (LEN=10), PARAMETER :: R_NAME = 'ST_TOP_BOT'
      INTEGER  J, K, JJ, Js, Jco, Jro, LL1, LR1, Lk, Nr2
      REAL (KND)  RHS(2*NNv)
      REAL (KND), POINTER :: LPV(:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      LL1 = (2*NrL - 4)*NNv + 1 ; LR1 = (2*NrL - 3)*NNv ; Lk = LL1 - 1
      Nr2 = 2*NNv*(NrL - 1)

c     ... Forward solution at the top and bottom of a medium
      DO Js = 1,Kvs*NSZA
c        ... At the top of the medium
c         LPV => PVEC(1:NNv,Js)
         Jco = Nc1  ;  Jro = NNv - 2*Jco
         RHS(1:NNv)         = PVEC(    1:NNv    ,Js)
         RHS(1+NNv:NNv+Jro) = PVEC(1+NNv:NNv+Jro,Js)*Ex1(1:Jro)
         DO K = Jro+1, Jro+2*Jco, 2
            RHS(NNv+K) = 
     $           PVEC(NNv+K,Js)  *Ex1(K) + PVEC(NNv+K+1,Js)*Ex1(K+1)
            RHS(NNv+K+1) = 
     $           PVEC(NNv+K+1,Js)*Ex1(K) - PVEC(NNv+K,Js)  *Ex1(K+1)
         ENDDO
c        ... In this equation PA_1(:,:) is PART_U(1,:,:)
         SOL_TOP(:,Js) = MATMUL(G_1(:,:),RHS(1:2*NNv) ) + PA_1(:,Js)

c     ... At the bottom of the medium
         Jco = Nc2  ;  Jro = NNv - 2*Jco
         RHS(:)     = PVEC(LL1:Nr2,Js)
         RHS(1:Jro) = PVEC(LL1:Lk+Jro,Js)*ExN(1:Jro)
         DO K = Jro+1, Jro+2*Jco, 2
            RHS(K)   = PVEC(Lk+K,  Js)*ExN(K) + PVEC(Lk+K+1,Js)*ExN(K+1)
            RHS(K+1) = PVEC(Lk+K+1,Js)*ExN(K) - PVEC(Lk+K,  Js)*ExN(K+1)
         ENDDO
c        ... PA_2(:,:) is PART_D(NrL-1,:,:)
c            G_2(:,:)  is G_NL(Yn,:,:) ; Yn = LAY_GR(NrL-1,1)
         SOL_BOT(:,Js) = MATMUL(G_2(:,:),RHS(:)) + PA_2(:,Js)         

         FLUX(Js) = SUM( SOL_BOT(1:NNv:Nvs,Js)*AXM(1:NN) )
      ENDDO  ! over solar angles (Js)

c      write(*,*) '    Stoks at the top "ST_TOP_BOT" Mh = ', Mh
c      DO K = 1,4
c         WRITE(*,'(2x,I2,2x,e15.7)'), K, SOL_TOP(NNv+J+K,1)
c      ENDDO

cc     ... Solution at the top and bottom of the medium for a given
cc         solar zenith angle Js
c      Js = 1
c      write(*,*) '    N_RHS equal ', N_RHS
c      IF( Mh == 1 ) THEN
c         WRITE(*,*) '   Solution at the top and the bottom '//
c     $        'of the corresponding medium (First Stokes component)'
cc     $        'of the corresponding medium (V Stokes component)'
c         WRITE(*,'(6x,2(8x,A),5x,2(8x,A))') 
c     $        'Downwards','Upwards','Downwards','Upwards'
c         do K = 1, NNv, Nvs
cc         do K = 4, NNv, Nvs
c            WRITE(*,'(2x,I5,2(2x,E15.7),4x,2(2x,E15.7))') 
c     $           K, SOL_TOP(K,Js), SOL_TOP(NNv+K,Js),
c     $           SOL_BOT(K,Js), SOL_BOT(NNv+K,Js)
c         enddo
c      ENDIF
c      
cc     ... Solution at the top of the atmosphere
cc      IF( LOG_LEVEL(1) >= N_DSO ) THEN
c         Js = 1
c         WRITE(*,*) '   Solution at the top of atmosphere Js = ',Js
c         do k = 1, NNv
c            WRITE(*,'(2x,I5,*(2x,E15.7))') 
c     $           k, SOL_TOP(k,Js), SOL_TOP(NNv+k,Js)
c         enddo
cc        ... Solution at the bottom of the atmosphere
c         WRITE(*,*) '   Solution at the bottom of atmosphere'
c         Js = 1
c         do k = 1, NNv
c            WRITE(*,'(2x,I5,*(2x,E15.7))') 
c     $           k, SOL_BOT(k,Js), SOL_BOT(NNv+k,Js)
c         enddo
ccc      ENDIF  ! Debug output
c         STOP
         
      IF( LOG_LEVEL(1) <= N_XSO ) THEN
         WRITE(LUNIT,*) '    SOL_TOP:', SUM(SOL_TOP)
         WRITE(LUNIT,*) '    SOL_BOT:', SUM(SOL_BOT)
      ENDIF

c     ... Surface-reflected diffuse intensity or Stokes vector
c         forward RTE
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
         DO Js = 1,Kvs*NSZA
            FLUX(Js) = SUM( SOL_BOT(1:NNv:Nvs,Js)*AXM(1:NN) )
         ENDDO

      CASE('Bidirect')
         DO Js = 1,Kvs*NSZA
            DO J = 1,Nvs
               RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AXM(1:NN)
            ENDDO  ! over Stokes components (J)
c           ... Save reflected from the surface diffuse forward 
c               radiation for all Gaussian and viewing angles
c            WRITE(*,*) '   SUM(RHS) =  ', SUM(RHS)
c            WRITE(*,*) '   SUM(UDB%BRF_GG) =  ', SUM(UDB%BRF_GG(:,:,Mh))
            DO K = 1,Nvs*(NN+Npg) 
               Ref_surf(Js,K) = SUM( UDB%BRF_GG(K,:,Mh)*RHS(1:NNv) )
            ENDDO              
         ENDDO  ! over solar angle (Js)

      CASE('Rough water')
         DO Js = 1,Kvs*NSZA
            DO J = 1,Nvs 
               RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AG(1:NN)
            ENDDO  ! over Stokes components (J)
c           ... Save reflected from the surface diffuse forward 
c               radiation for all Gaussian angles and nadir direction
            DO K = 1,Nvs*NN
               Ref_surf(Js,K) = SUM( UDB%Ra_GG(K,:,Mh)*RHS(1:NNv) )
            ENDDO
         ENDDO
c        ... Solution at the bottom of the atmosphere
c         WRITE(*,*) '   Downward at the bottom NN = ', NN
c         Js = 1
c         do K = 1, NN ;  JJ = Nvs*(K-1)
c            WRITE(*,'(2x,I5,*(2x,E12.4))') 
c     $           k, SOL_BOT(JJ+1:JJ+Nvs,Js)
c         enddo
c         WRITE(*,*) '   Upward at the bottom NN = ', NN
c         Js = 1
c         do K = 1, NN ;  JJ = Nvs*(K-1) + NNv
c            WRITE(*,'(2x,I5,*(2x,E12.4))') 
c     $           k, SOL_BOT(JJ+1:JJ+Nvs,Js)
c         enddo

      END SELECT

c     <<<<<<<<<<<<<<<<<<<<<<  Weighting functions >>>>>>>>>>>>>>>>>>>>>>
c     ... If WFs for atmospheric or ocean parameters are required than 
c         solution at the top and bottom of the medium will be calculated 
c         for the pseudo-forward equation also
      IF( WF_FLAG .OR. OWF_FLAG ) THEN
c        ... Pseudo-forward solution at the top and bottom of a medium
         DO JJ = 1,N_Adj
            Js = JJ + SDVI
c           ... Pseudo-forward solution at the top of the medium
            LPV => PVEC(:,Js) ; Jco = Nc1  ;  Jro = NNv - 2*Jco
            RHS(1:NNv)         = LPV(    1:NNv    )
            RHS(1+NNv:NNv+Jro) = LPV(1+NNv:NNv+Jro)*Ex1(1:Jro)
            DO K = Jro+1, Jro+2*Jco, 2
               RHS(NNv+K)  = LPV(NNv+K)  *Ex1(K) + LPV(NNv+K+1)*Ex1(K+1)
               RHS(NNv+K+1)= LPV(NNv+K+1)*Ex1(K) - LPV(NNv+K)  *Ex1(K+1)
            ENDDO
            SOL_TOP(:,Js) = MATMUL(G_1(:,:),RHS(1:2*NNv) ) 
     $           + ADO%PART_TU(1,:,JJ)
 
c           ... Pseudo-forward solution at the bottom of the medium
            Jco = Nc2  ;  Jro = NNv - 2*Jco
            RHS(:)     = LPV(LL1:Nr2)
            RHS(1:Jro) = LPV(LL1:Lk+Jro)*ExN(1:Jro)
            DO K = Jro+1, Jro+2*Jco, 2
               RHS(K)   = LPV(Lk+K)  *ExN(K) + LPV(Lk+K+1)*ExN(K+1)
               RHS(K+1) = LPV(Lk+K+1)*ExN(K) - LPV(Lk+K)  *ExN(K+1)
            ENDDO
            SOL_BOT(:,Js) = MATMUL(G_2(:,:),RHS(:))
     $           + ADO%PART_TD(NrL-1,:,JJ)
         ENDDO

c        ... Surface-reflected diffuse intensity or Stokes vector
c            of pseudo-forward RTE
         SELECT CASE( TRIM(UDB%LBC) )
         CASE('Lambert')
c           ... In this case will be used array FLUX_D

         CASE('Bidirect')
c                      WRITE(*,*) '   "st_top_bot.f" option  Bidirect '
c                      WRITE(*,*) '    NSZA  ', NSZA
c                      WRITE(*,*) '    N_Adj ', N_Adj
c                      WRITE(*,*) '    N_RHS ', N_RHS
c                      WRITE(*,*) '  SHAPE(Ref_surf) ', SHAPE(Ref_surf) 
            DO Js = Kvs*NSZA+1,N_RHS
               DO J = 1,Nvs     ! for all Stokes components
                  RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AXM(1:NN)
               ENDDO
c              ... Save reflected from the surface pseudo-forward 
c                  radiation for all Gaussian and viewing angles
               DO K = 1,Nvs*(NN+Npg) 
                  Ref_surf(Js,K) = SUM( UDB%BRF_GG(K,:,Mh)*RHS(1:NNv) )
               ENDDO
            ENDDO

         CASE('Rough water')
            DO Js = Kvs*NSZA+1,N_RHS
               DO J = 1,Nvs     ! for all Stokes components
                  RHS(J:NNv:Nvs) = SOL_BOT(J:NNv:Nvs,Js)*AG(1:NN)
               ENDDO
c              ... Save reflected from the surface pseudo-forward 
c                  radiation for all Gaussian angles and nadir direction
c               DO K = 1,Nvs*(NN+1) 
               DO K = 1,Nvs*NN
                  Ref_surf(Js,K) = SUM( UDB%Ra_GG(K,:,Mh)*RHS(1:NNv) )
               ENDDO
            ENDDO

         END SELECT
      ENDIF  ! WF_FLAG .OR. OWF_FLAG

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END SUBROUTINE
c **********************************************************************
