c***********************************************************************
c 
c       SCIATRAN - Version 3.9 - Last modification: 29 Dec 2017
c
c       Copyright (C) - Institute of Remote Sensing,
c                       University of Bremen, Germany
c
c    Calculation of the reflected and transmitted forward intensity 
c    at all Gauss angles for all levels and all solar zenith angles
c         empoying Source Function Integration technique
c
c    The routine is used if "Adaptive-grid" option is active
c **********************************************************************
      SUBROUTINE ST_UP_DOWN_SFI(Mh, NN, NSZA, NL, IOP, SSB, TE, 
     $     SDO, Ref_surf, UDB, Bet, Ph_z, GSL, MEDIUM)
c     ... Called by "field_iface.f"
c                   "st_dom_tau.f"

      USE BRDF,        ONLY: Npg
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE FLU_LAND,    ONLY: FLU_L
      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nvs, Ng1_brf, Ns_brf, 
     $                       NN_pla
      USE IRRAD,       ONLY: Fbe
      USE LOG_PARS,    ONLY: DIR_OUT
      USE RTM_OUTPUT,  ONLY: Wup
      USE ST_VARS,     ONLY: N_RHS, Pie, F14, Vec_Sca, VST, Finc, Kvs
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: TE_TYPE, SDO_TYPE, UDB_TYPE, SSB_TYPE,
     $                       IOP_TYPE, GSL_TYPE
c     ==================================================================

      INTEGER,           INTENT(IN) :: Mh, NN, NSZA, NL
      CHARACTER (LEN=5), INTENT(IN) :: MEDIUM
c     ------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (IOP_TYPE), TARGET :: IOP    ! Integral optical parameters
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     ----------------------
      REAL (KND), INTENT(IN):: Ref_surf(N_RHS,Nvs*(NN+Npg))
      REAL (KND), DIMENSION(NL), INTENT(IN)::  Bet
c     -----------------------------------------------------
      TYPE phase_tdz
        SEQUENCE
        REAL (KND), POINTER ::  Egg(:,:,:,:), EgL(:,:,:,:), 
     $       Egs(:,:,:,:), EsL(:,:,:,:)
      END TYPE phase_tdz
      TYPE (phase_tdz) :: Ph_z
c     ... Local variables 
c     -------------------
      CHARACTER (LEN=14), PARAMETER :: R_NAME = 'ST_UP_DOWN_SFI'
      LOGICAL  PURE(NN*Nvs)
      INTEGER  I, Ivs, Js, Jv, Jvs, Jc, Jg, Jco, Jro, 
     $     K, K1, K2, Kc, Kg1, Kg2, Km, Ks, 
     $     LL1, Lr1, Lk, Max_Jco, NNv, Zn
      INTEGER,    POINTER :: Id_a(:), Id_b(:)
      REAL (KND) Cs, Cg, Ecg, Smd_p, Cmd_p, DTA, Sou_p, Wdo,
     $     CMM, A_COF, B_COF, P_COF, R_cof, C_1, Ck, Smm, 
     $     A_Ref, B_Ref, H_1, C_DP, C_UP, C_DM, C_UM, Flux(Kvs*NSZA),
     $     C_dbl, C_aim, S_dbl, S_aim, Cks, Ac(2), Bc(2), 
     $     SOU(NSZA), Faj(NN*Nvs,NN*Nvs), Fbj(NN*Nvs,NN*Nvs), 
     $     HPR(NL_pla-1,NN_pla,NN), HPT(NL_pla-1,NN_pla,NN)
      REAL (KND) Irl(NL-1,Kvs*NSZA,NN*Nvs), Itl(NL-1,Kvs*NSZA,NN*Nvs),
     $     Tru(Kvs*NSZA,NN*Nvs), Trg(Kvs*NSZA,NN*Nvs), V_XM(NN), 
     $     Exp_XM(NL-1,NN), Qp(Nvs), Qm(Nvs)
      REAL (KND), DIMENSION (NN*Nvs) :: Apr, Bpr, Apt, Bpt,
     $     Tr1, Tr2, Tt1, Tt2, Nup, Num, Sjm, Cjm
      REAL (KND), POINTER :: DTR(:), ALW(:), DTR_U(:), DTR_D(:), 
     $     Ex_U(:,:), Ex_D(:,:), XM(:), AXM(:), AG(:), MuS(:),
     $     Nu_j(:), Exp_Nu(:), EXP_D(:), EXP_U(:), 
     $     P1_Mv(:), P2_Mv(:), EXS(:,:), SIN_COEF(:,:,:),
     $     G(:,:,:), Pgg(:,:,:,:), P01(:,:,:), P02(:,:,:)
      REAL (KND), ALLOCATABLE :: HDU(:)
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2, Cnu
      COMPLEX (kind(0.0d0)), ALLOCATABLE :: Cj_d(:), Cj_u(:), 
     $     Sj_d(:), Sj_u(:)      
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      NNv = NN*Nvs
      XM    => GSL%XM    ; AXM   => GSL%AXM   ; AG => GSL%AG
      MuS   => GSL%MuS   ; DTR   => IOP%DTR   ; ALW => IOP%ALW 
      DTR_U => IOP%DTR_U ; DTR_D => IOP%DTR_D
      Ex_U  => IOP%Ex_U  ; Ex_D  => IOP%Ex_D
      Id_a  => IOP%Id_a  ; Id_b  => IOP%Id_b

      Max_Jco = MAXVAL( SDO%NcC(:) )
      IF( Max_Jco > 0 ) ALLOCATE( Cj_d(Max_Jco), Cj_u(Max_Jco), 
     $     Sj_d(Max_Jco), Sj_u(Max_Jco) )

      Pgg => Ph_z%Egg

c     ... Dimension of Ph_z%Egs(NL,Nvs*NN,Nvs*NSZA,2)
c         It is used here to calculate scattering of 
c         forward radiation from the solar direction 
c         into Gaussian angle directions
      P01 => Ph_z%Egs(:,:,:,1) ; P02 => Ph_z%Egs(:,:,:,2)
      EXS => SSB%EXS           ;   SIN_COEF => SSB%SIN_COEF
      G   => SDO%G_NL

      V_XM(:) = 1.d0
      IF( Vec_Sca=='vect_rte' ) V_XM(:) = XM(:)

      Itl = 0.d0  ;   Irl = 0.d0  ;   Wdo = 1.d0 - Wup
      IF( TE%ADD_PLANCK ) THEN
         DO I = 1,NL - 1        ! do-loop over layers
            DTA = DTR(I+1)  ;  SOU_p = DEXP( TE%Mup(I)*DTA )  
            CS = -TE%Mup(I)
            DO Jv = 1,NN     ! do-loop over Gauss angles
               Cg = XM(Jv) ; Ecg = DEXP( -DTA/Cg ) 
               Smd_p = Cg/(1.d0 + Cs*Cg)*( 1.d0 - SOU_p*Ecg )
               IF( REAL(CS*Cg) /= REAL(1.d0) ) THEN
                  Cmd_p = Cg*(SOU_p - Ecg)/(1.d0 - CS*Cg)
               ELSE
                  Cmd_p = Ecg*DTA
               ENDIF
               Zn = -1
               DO Jc = 1,TE%N_pla(I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Cks = Ck*Cs
                  Muc = Ck/DCMP( Cks,-1.d0 )
                  Smd_p = Smd_p + AIMAG( Muc*Cg/(Muc + Cg) )
     $                 *TE%PL_COEF(I,Jc)*(1.d0 - Zn*SOU_p*Ecg)
                  Cmd_p = Cmd_p + AIMAG( Muc*Cg/(Muc - Cg) )
     $                 *TE%PL_COEF(I,Jc)*(Zn*SOU_p - Ecg)
                  Zn = -Zn
               ENDDO
               HPR(I,:,Jv) = TE%Aj_pla(:,I)*Smd_p*Cg
               HPT(I,:,Jv) = TE%Aj_pla(:,I)*Cmd_p*Cg
               IF( .NOT.DO_SSRCA ) THEN 
                  DO K1 = (Jv-1)*Nvs+1, Jv*Nvs, Nvs
                     Irl(I,1:NSZA,K1) = SMd_p*TE%NPLANCK(I)/Cg*VST
                     Itl(I,1:NSZA,K1) = CMd_p*TE%NPLANCK(I)/Cg*VST
                  ENDDO  ! over the first Stokes component (K1)
               ENDIF
            ENDDO  ! overe Gaussian angles (Jv)
         ENDDO  ! over layers (I)
      ENDIF  ! thermal emission is presented

c      WRITE(*,*) '    Start do-loop over layers'
      DO I = 1,NL - 1
c        ... Skip calculation within standard layer (see dom_spherical.f)
         IF( GSL%AG_ZERO(I) ) CYCLE
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco

         PURE(:) = .TRUE.
         IF( SDO%MASK_NL(I) ) PURE(SDO%E_MIN(I)) = .FALSE.

c         from "aux_in_layer": Faj, Fbj, DTA, Nu_j,  
c                Exp_Nu, EXP_D, EXP_U, and LL1 = (I-1)*2*NNv + 1 
c                LR1 = (I-1)*2*NNv + NNv
         CALL AUX_IN_LAYER  
         SOU(:) = DEXP( -DTA/SSB%Mue(:,I) )
         Lk = LL1 - 1
         DO Jg = 1,NN           ! Over Gauss zenith angles
            Kg1 = (Jg-1)*Nvs+1 ;  Kg2 = Jg*Nvs

            Cg = XM(Jg) ; Ecg = DEXP( -DTA/Cg )            
            Nup(1:Jro) = Nu_j(1:Jro)/(Nu_j(1:Jro) + Cg) 

            Sjm(1:Jro) = Nu_j(1:Jro)*Cg/(Nu_j(1:Jro) + Cg) 
     $           *( 1.0d0 - Exp_Nu(1:Jro)*Ecg ) 
            WHERE( REAL(Nu_j(1:Jro)) /= REAL(Cg) )
               Num(1:Jro) = Nu_j(1:Jro)/(Nu_j(1:Jro) - Cg)
               Cjm(1:Jro) = Nu_j(1:Jro)*Cg/(Nu_j(1:Jro) - Cg)
     $              *(Exp_Nu(1:Jro) - Ecg)  
            ELSEWHERE
               Cjm(1:Jro) = Exp_Nu(1:Jro)*DTA
            END WHERE            
            Kc = 1
c           ... Use complex separation constants:
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nu_j(K1),Nu_j(K1+1) )
               Cnu = DCMP( Exp_Nu(K1),Exp_Nu(K1+1) )
               C2 = C1*Cg*(Cnu - ECg)/(C1 - Cg) 
c                   Cjm(K1) = DBLE(C2) ; Cjm(K1+1) = AIMAG(C2)
               Cj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Cj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))

               C2 = C1*Cg*(1.d0 - Cnu*Ecg)/(C1 + Cg)
c                   Sjm(K1) = DBLE(C2) ; Sjm(K1+1) = AIMAG(C2)
               Sj_d(Kc) = C2*DCMP(EXP_D(K1),EXP_D(K1+1))
               Sj_u(Kc) = C2*DCMP(EXP_U(K1),EXP_U(K1+1))
               Kc = Kc + 1
            ENDDO

            DO Js = 1,NSZA

               CS = SSB%Mue(Js,I)
               Cmm = CS*Cg/(CS - Cg)*(SOU(Js) - ECg)
               Smm = CS*Cg/(CS + Cg)*( 1.d0 - SOU(Js)*ECg )
               Zn = -1
               DO Jc = 1,SSB%N_sph(Js,I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-CS )
                  Smm = Smm + AIMAG( Muc*Cg/(Muc+Cg) )
     $                 *SIN_COEF(Js,I,Jc)*( 1.0d0 - Zn*SOU(Js)*Ecg )
                  Cmm = Cmm + AIMAG( Muc*Cg/(Muc-Cg) )
     $                 *SIN_COEF(Js,I,Jc)*( Zn*SOU(Js) - Ecg )
                  Zn = -Zn
               ENDDO
               
               DO Ivs = 1,Kvs
                  Jvs = (Ivs - 1)*NSZA + Js
                  P1_Mv => IOP%MVEC(  LL1:LR1,    Jvs )
                  P2_Mv => IOP%MVEC(LR1+1:LR1+NNv,Jvs )

c              ... Auxiliary functions for reflection and transmission:
                  Apr(:) = ( -SDO%Ajp(Ivs,I,:,Js)*Ecg 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 + SDO%Ajp(Ivs,Id_a(I)-1,:,Js)
     $                 + SDO%Aj_sol(Ivs,:,I,Js)*Smm*EXS(I,Js) )*Cg  

                  Bpr(:) = ( SDO%Bjp(Ivs,I,:,Js) 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 - SDO%Bjp(Ivs,Id_b(I)+1,:,Js)*Ecg
     $                 - SDO%Bj_sol(Ivs,:,I,Js)*Smm*EXS(I,Js) )*Cg  

                  Apt(:) = ( SDO%Ajp(Ivs,I,:,Js) 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 - SDO%Ajp(Ivs,Id_a(I)-1,:,Js)*Ecg
     $                 - SDO%Aj_sol(Ivs,:,I,Js)*Cmm*EXS(I,Js) )*Cg  

                  Bpt(:) = ( -SDO%Bjp(Ivs,I,:,Js)*Ecg 
c                     ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                 + SDO%Bjp(Ivs,Id_b(I)+1,:,Js)
     $                 + SDO%Bj_sol(Ivs,:,I,Js)*Cmm*EXS(I,Js) )*Cg  

                  IF( TE%ADD_PLANCK.AND.Ivs==1 ) THEN
                     Apr(:) = Apr(:) + HPR(I,:,Jg)
                     Bpr(:) = Bpr(:) - HPR(I,:,Jg)
                     Apt(:) = Apt(:) - HPT(I,:,Jg)
                     Bpt(:) = Bpt(:) + HPT(I,:,Jg) 
                  ENDIF

                  WHERE( REAL(Nu_j(1:Jro)) /= REAL(Cg) )
                     Tt1(1:Jro) = Apt(1:Jro)*Num(1:Jro)
                     Tr2(1:Jro) = Bpr(1:Jro)*Num(1:Jro)
                  ELSEWHERE
                     Tt1(1:Jro) = SDO%Aj_sol(Ivs,1:Jro,I,Js)
     $                    *Cg*Cs/(Cs - Cg)*(Cmm - DTA*Ecg)*EXS(I,Js)
                     Tr2(1:Jro) = SDO%Bj_sol(Ivs,1:Jro,I,Js)
     $                    *Cg*Cs/(Cs + Cg)*(Smm - DTA*Ecg*SOU(Js))
     $                    *EXS(I,Js)
                  END WHERE            

                  WHERE( PURE(1:Jro) ) 
c                        < Reflected >
                     Tr1(1:Jro) = Apr(1:Jro)*Nup(1:Jro)
     $                    + P1_Mv(1:Jro)*EXP_U(1:Jro)*Sjm(1:Jro)
                     Tr2(1:Jro) = Tr2(1:Jro) + 
     $                    P2_Mv(1:Jro)*Cjm(1:Jro)*EXP_D(1:Jro)
c                        < Transmitted >
                     Tt1(1:Jro) = Tt1(1:Jro) 
     $                    + P1_Mv(1:Jro)*Cjm(1:Jro)*Exp_U(1:Jro) 
                     Tt2(1:Jro) = Bpt(1:Jro)*Nup(1:Jro)
     $                    + P2_Mv(1:Jro)*Sjm(1:Jro)*Exp_D(1:Jro)
                  ELSEWHERE
                     Tt1(1:Jro) = 0.d0 ; Tt2(1:Jro) = 0.d0
                     Tr1(1:Jro) = 0.d0 ; Tr2(1:Jro) = 0.d0
                  END WHERE            

c                 ... Calculate Tr1, Tr2, Tt1, and Tt2 for 
c                     Jro+1:Jro+2*Jco
                  IF( Jco > 0 ) CALL COMPLEX_Tr_Tt
               
c                 ...   Because P01 was designed to describe 
c                     +Gauss --> Sol_ang, we multiply P01^T*Finc(1,:)
c                     to obtain single scattering Sol_ang --> +Gauss.
c                     The same hold for matrix P02 to obtain single 
c                     scattering Sol_ang --> -Gauss.
c                       Multiplication P01^T and P02^T from left and right by
c                     D4=diag{1,1,1,-1} is omitted because single scattered
c                     solar radiation does not contain V Stokes component.
c                     !!! It should be corrected in the case of !!!
c                              incoming polarized light
                  Ks = (Js-1)*Nvs
                  DO K2 = 1,Nvs
                     Qp(K2) = SUM( (Wup*P01(I,Kg1:Kg2,Ks+K2)
     $                    + Wdo*P01(I+1,Kg1:Kg2,Ks+K2))*Finc(Ivs,:) )
                     Qm(K2) = SUM( (Wup*P02(I,Kg1:Kg2,Ks+K2)
     $                    + Wdo*P02(I+1,Kg1:Kg2,Ks+K2))*Finc(Ivs,:) )
                  ENDDO
                  Qp(:) = Qp(:)*Fbe*F14*EXS(I,Js)/V_XM(Jg)/ALW(I)
                  Qm(:) = Qm(:)*Fbe*F14*EXS(I,Js)/V_XM(Jg)/ALW(I)

                  K2 = 1
                  DO K1 = (Jg-1)*Nvs+1, Jg*Nvs
c                          Transmitted
                     Itl(I,Jvs,K1) = Cmm*Qp(K2)
     $                    + SUM( Tt1(:)*Faj(K1,:) + Tt2(:)*Fbj(K1,:) )
c                          Reflected_sum
                     Irl(I,Jvs,K1) = Smm*Qm(K2)
     $                    + SUM( Tr1(:)*Fbj(K1,:) + Tr2(:)*Faj(K1,:) )
                     K2 = K2 + 1
                  ENDDO  ! over Stokes components (K1)
               ENDDO  ! over DO-solutions (Ivs)
            ENDDO  ! over solar angles (Js)
         ENDDO  ! over Gauss angles (Jg)

c        ...  layer with conservative scattering
         IF( ANY(.NOT.PURE(:)) )  CALL PURE_SCAT

      END DO                    ! over all layers (I)

c     ... Down-welling radiation field at all Gaussian zenith angles 
c         recalculated using adaptive grid assumptions
      FORALL( I=1:NL-1 ) Exp_XM(I,:) = DEXP(-DTR(I+1)/XM(:))

c     ... Diffuse radiation incoming in the medium
      SELECT CASE( MEDIUM )
      CASE('Atmos')
         Trg(:,:) = 0.d0  
      CASE('Ocean')
c        ... Transmitted from atmosphere to the ocean and 
c            reflected from air-water interface to the ocean
         Trg(:,:) = 0.d0  
      END SELECT

      IOP%DFI = 0.d0
      IOP%DFI(1,:,1:NNv) = Trg(:,:)

      DO I = 1,NL-1
         DO Jg = 1,NN
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Trg(:,K1) = Trg(:,K1)*Exp_XM(I,Jg) + Itl(I,:,K1)
            ENDDO
         ENDDO
         IOP%DFI(I+1,:,1:NNv) = Trg(:,:) 
      ENDDO

      FORALL( Js=1:Kvs*NSZA ) Flux(Js) 
     $     = SUM(IOP%DFI(NL,Js,1:NNv:Nvs)*AXM(1:NN))

c     ... Up-welling radiation field at all Gaussian zenith angles 
c         recalculated using adaptive grid assumptions

c     **************************************************************
c              Including BRDF in ST_UP_DOWN_SFI
c     **************************************************************
      IF( UDB%BRDF_FLAG ) THEN
         Tru = 0.d0
         DO Js = 1,Kvs*NSZA
            Tru(Js,:) = 2.d0*Ref_surf(Js,1:NNv)
         ENDDO
c         write(*,'(a,*(2x,e12.4))') '  Dfi Tru 1 ', Tru(1,2:4)
         IF( .NOT.DO_SSRCA ) THEN ! If single scattering corr. is not used
            ALLOCATE( HDU(NNv) )
            DO Js = 1,NSZA
               Ks = (Js-1)*Nvs
               DO Ivs = 1,Kvs
                  Jvs = (Ivs-1)*GSL%NSZA + Js
                  DO K = 1,Nvs*NN
                     HDU(K) = SUM
     $                    (UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
                  ENDDO  ! over Gaussian angles (K)
                  Tru(Jvs,:) = Tru(Jvs,:) + VST*MuS(Js)*EXS(NL,Js)
     $                 *HDU(1:NNv)
               ENDDO  ! over DO-solutions (Ivs) -> Jvs
            ENDDO  ! over solar angles (Js)
c            write(*,'(a,*(2x,e12.4))') '  Dfi Tru 2 ', Tru(2,2:4)
         ENDIF
      ELSE  ! Lambertian reflection
         LAMBERT_INIT: Block
           Tru = 0.d0 
         IF( Mh > 1 )   EXIT LAMBERT_INIT ! ==> 
           FORALL( Jg = 1:NNv:Nvs ) Tru(1:NSZA,Jg) = 
     $        2.d0*UDB%Albedo*Flux(1:NSZA)
         IF( DO_SSRCA ) EXIT LAMBERT_INIT ! ==> 
c           ... VST = 2 (vector case) VST = 1 (scalar case)
           FORALL( Jg = 1:NNv:Nvs ) Tru(1:NSZA,Jg) = Tru(1:NSZA,Jg) 
     $          + UDB%Albedo*MuS(:)*EXS(NL,:)*VST
         END Block LAMBERT_INIT
c         Tru = 0.d0
c         FORALL( Jg = 1:NNv:Nvs ) Tru(1:NSZA,Jg) 
c     $        = 2.d0*UDB%Albedo*Flux(1:NSZA)
c         IF( .NOT.DO_SSRCA ) THEN
c            FORALL( Jg = 1:NNv:Nvs ) Tru(1:NSZA,Jg) = Tru(1:NSZA,Jg) 
c     $           + UDB%Albedo*MuS(:)*EXS(NL,:)*VST
c         ENDIF
      ENDIF  ! Surface reflection

c     ... Thermal emission
      IF( TE%ADD_PLANCK .AND. (.NOT.DO_SSRCA) ) FORALL(Jg = 1:NNv:Nvs) 
     $     Tru(1:NSZA,Jg) = Tru(1:NSZA,Jg) + VST*TE%PLANCK_SURF

c     ... Land fluorescence spectra
      IF( FLU_L.AND.Mh==1.AND.(.NOT.DO_SSRCA)) THEN
         FORALL(Jg = 1:NNv:Nvs)  Tru(:,Jg) = Tru(:,Jg) + VST*UDB%Flu_int
      ENDIF

c         K = 1
c         WRITE(*,*) '   Tru & Ref_surf befor sign '
c         DO I = 1, NNv
c            WRITE(*,'(5x,I5,*(2x,E12.4))') I, Tru(1,I), Ref_surf(1,I)
c            K = K + 1
c         ENDDO

c     ... U and V components of the Stokes vector
c         sleduet menyat' znak tol'ko u Tru 
      IF( Vec_Sca == 'vect_rte' ) THEN
         DO Jg = 1,NN 
            DO K1 = (Jg-1)*Nvs+3, Jg*Nvs
               Tru(:,K1)   = -Tru(:,K1)
            ENDDO 
         ENDDO
      ENDIF

      IOP%DFI(NL,:,NNv+1:2*NNv) = Tru(:,:)
      DO I = NL - 1,1,-1
         DO Jg = 1,NN
            DO K1 = (Jg-1)*Nvs+1,Jg*Nvs
               Tru(:,K1) = Tru(:,K1)*Exp_XM(I,Jg) + Irl(I,:,K1)
            ENDDO
         ENDDO  ! over Gaussian angles (Jg)
         IOP%DFI(I,:,NNv+1:2*NNv) = Tru(:,:) 
      ENDDO  ! over main altitude layers (I)

c         Js = 1
c         WRITE(*,*) ' ******* DFI(NL,1,ref) and  '
c         WRITE(*,*) '         DFI(NL-1,1,ref) in "st_up_down_sfi" : '
c         WRITE(*,*) '         for SZA    : ',Js
c         K = 1
c         DO I = NNv+1,2*NNv
c            WRITE(*,'(5x,I5,*(2x,E12.4))') I, IOP%DFI(NL,Js,I),
c     $           IOP%DFI(NL-1,Js,I)
c            K = K + 1
c         ENDDO

c     WRITE(*,*) ' Number of column in DFI :', 2*NNv
c         Open(12,file=DIR_OUT//'/DFI_sfi.new',RECL=15000)
c           DO I = 1, NL
c              WRITE( 12,'(130(1x,e15.7))')
c     $             ZZZ(I),(IOP%DFI(I,Js,J), J = 1,2*NNv)
c           ENDDO
c         Close(12)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS

c **********************************************************************
      SUBROUTINE COMPLEX_Tr_Tt
      Kc = 1
      DO K1 = Jro+1, Jro+2*Jco, 2
         Cnu = DCMP( Nu_j(K1),Nu_j(K1+1) )
         C1 = DCMP(Apr(K1),Apr(K1+1))*Cnu/(Cnu+Cg)
         S_dbl = DBLE(Sj_u(Kc)) ; S_aim = AIMAG(Sj_u(Kc))
         Tr1(K1) = P1_Mv(K1)*S_dbl + P1_Mv(K1+1)*S_aim   + 2.d0*DBLE(C1)
         Tr1(K1+1)= P1_Mv(K1+1)*S_dbl - P1_Mv(K1)*S_aim - 2.d0*AIMAG(C1)
         
         C1 = DCMP(Apt(K1),Apt(K1+1))*Cnu/(Cnu-Cg)
         C_dbl = DBLE(Cj_u(Kc)) ; C_aim = AIMAG(Cj_u(Kc))
         Tt1(K1)  = P1_Mv(K1) *C_dbl + P1_Mv(K1+1)*C_aim + 2.d0*DBLE(C1)
         Tt1(K1+1)= P1_Mv(K1+1)*C_dbl - P1_Mv(K1)*C_aim - 2.d0*AIMAG(C1)
         
         C2 = DCMP(Bpr(K1),Bpr(K1+1))*Cnu/(Cnu-Cg)
         C_dbl = DBLE(Cj_d(Kc)) ; C_aim = AIMAG(Cj_d(Kc))
         Tr2(K1)  = P2_Mv(K1) *C_dbl + P2_Mv(K1+1)*C_aim + 2.d0*DBLE(C2)
         Tr2(K1+1)= P2_Mv(K1+1)*C_dbl - P2_Mv(K1)*C_aim - 2.d0*AIMAG(C2)

         C2 = DCMP(Bpt(K1),Bpt(K1+1))*Cnu/(Cnu+Cg)
         S_dbl = DBLE(Sj_d(Kc)) ; S_aim = AIMAG(Sj_d(Kc))
         Tt2(K1)  = P2_Mv(K1)*S_dbl + P2_Mv(K1+1)*S_aim  + 2.d0*DBLE(C2)
         Tt2(K1+1)= P2_Mv(K1+1)*S_dbl - P2_Mv(K1)*S_aim - 2.d0*AIMAG(C2)
         Kc = Kc + 1
      ENDDO  ! over complex separation constants (Kc)
      END SUBROUTINE COMPLEX_Tr_Tt

c **********************************************************************
      SUBROUTINE AUX_IN_LAYER
      REAL (KND), DIMENSION (NNv,NNv) :: Ph_g1, Ph_g2, TTT
      REAL (KND), POINTER :: Gp_p(:,:), Gm_p(:,:)

      SELECT CASE( Vec_Sca )
      CASE( 'vect_rte' )
c         ... In the vector case Pgg(I,:,:,1) and Pgg(I,:,:,2) contain
c             sum and difference of phase matrixes. Therefore, the phase
c             matrix corresponding to the scattering +Gauss <-- +Gauss
c             is calculated below
         TTT(:,:) = Wup*Pgg(I,:,:,1) + Wdo*Pgg(I+1,:,:,1)
         TTT(:,:) = TTT(:,:) + (Wup*Pgg(I,:,:,2) + Wdo*Pgg(I+1,:,:,2))

c        ... In the vector case second dimension of array TTT contains
c            multiplier 1/XM(K) (aee "phase_matr.f"). At this step we 
c            need to remove it.
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 

c        ... First dimension of array TTT is divided by XM(K) because we 
c            need to calculate the source function for the formal 
c            solution in directions of all Gaussian angles
         DO Jv = 1,NN
            Ph_g1(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
c        ... Second dimension is now multiplied by Gauss weights
         FORALL( K = 1:NN ) Ph_g1(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g1(:,Nvs*(K-1)+1:Nvs*K)*AG(K)*0.5d0

c        ... It should be reproduced further (see above) the 
c            phase matrix corresponding to the scattering +Gauss <-- -Gauss
         TTT(:,:) =  Wup*Pgg(I,:,:,2) + Wdo*Pgg(I+1,:,:,2)
         TTT(:,:) = (Wup*Pgg(I,:,:,1) + Wdo*Pgg(I+1,:,:,1)) - TTT(:,:)
         FORALL( K = 1:NN ) TTT(:,Nvs*(K-1)+1:Nvs*K) = 
     $        TTT(:,Nvs*(K-1)+1:Nvs*K)*XM(K) 
         DO Jv = 1,NN
            Ph_g2(Nvs*(Jv-1)+1:Nvs*Jv,:) = 
     $           TTT(Nvs*(Jv-1)+1:Nvs*Jv,:)/XM(Jv)
         ENDDO
         FORALL( K = 1:NN ) Ph_g2(:,Nvs*(K-1)+1:Nvs*K) = 
     $        Ph_g2(:,Nvs*(K-1)+1:Nvs*K)*AG(K)*0.5d0
      CASE( 'scal_rte' )
c        ... First dimension of arrays Ph_g1(K,:) and Ph_g2(K,:) contains 
c            multiplier 1/XM(K) (see routine "phase_matr.f"). It should
c            be used here because we need to calculate the source function
c            for the formal solution in directions of all Gaussian angles
         FORALL( K = 1:NN )
c          ... Mu <=> Mu
           Ph_g1(K,:) = (Wup*Pgg(I,K,:,1) + Wdo*Pgg(I+1,K,:,1))*AG(:)
     $          *0.5d0
c          ... Mu <=> -Mu
           Ph_g2(K,:) = (Wup*Pgg(I,K,:,2) + Wdo*Pgg(I+1,K,:,2))*AG(:)
     $          *0.5d0
         END FORALL 
      END SELECT

      Ph_g1(:,:) = Ph_g1(:,:)/ALW(I) ; Ph_g2(:,:) = Ph_g2(:,:)/ALW(I)

c     ... Here we use matrix of all elementary solutions
      Gp_p => G(I,1:NNv,1:NNv)       ; Gm_p => G(I,1+NNv:2*NNv,1:NNv)

      DO K = 1,NNv
         Faj(:,K) = MATMUL(Ph_g2(:,:),Gm_p(:,K)) +
     $        MATMUL(Ph_g1(:,:),Gp_p(:,K))
         Fbj(:,K) = MATMUL(Ph_g2(:,:),Gp_p(:,K)) + 
     $        MATMUL(Ph_g1(:,:),Gm_p(:,K))
      ENDDO
      LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv
      DTA = DTR(I+1)
      Nu_j   =>  SDO%Nuj_lay(:,I) 
      Exp_Nu =>  SDO%EXP_NU_NL  (:,I)
      EXP_D  =>  Ex_D(:,I)
      EXP_U  =>  Ex_U(:,I)
      END SUBROUTINE AUX_IN_LAYER

c ****************************************************************
      SUBROUTINE PURE_SCAT
      INTEGER Jg

      H_1 = 3.d0 - (Wup*Bet(I) + Wdo*Bet(I+1))/ALW(I)
      Km = SDO%E_MIN(I)
      DO Jg = 1,NN              ! Jv => Jg
         Cg = XM(Jg) ; Ecg = DEXP( -DTA/Cg ) ! Muv => XM
         C_1 = 3.d0*Cg/H_1
         C_DM = C_1 - DTR_D(I)
         C_UP = C_1 + DTR_U(I)
         C_DP = C_1 + DTR_D(I)
         C_UM = C_1 - DTR_U(I)
c               Transmission 
         A_cof = -DTA*Ecg + C_DP*(1.d0 - Ecg)
         B_cof =  DTA     - C_UM*(1.d0 - Ecg)
c                Reflection
         A_Ref =  DTA     - C_DM*(1.d0 - Ecg)
         B_Ref = -DTA*Ecg + C_UP*(1.d0 - Ecg)

         DO Js = 1,NSZA
            CS = SSB%Mue(Js,I)
            Smm = CS*Cg/(CS + Cg)*( 1.d0 - SOU(Js)*Ecg )
            Cmm = CS*Cg*(SOU(Js) - Ecg)/(CS - Cg)
            Zn = -1
            DO Jc = 1,SSB%N_sph(Js,I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-CS )
               Smm = Smm + AIMAG( Muc*Cg/(Muc+Cg) )
     $              *SIN_COEF(Js,I,Jc)*( 1.d0 - Zn*SOU(Js)*Ecg )
               Cmm = Cmm + AIMAG( Muc*Cg/(Muc-Cg) )
     $              *SIN_COEF(Js,I,Jc)*( Zn*SOU(Js) - Ecg )
               Zn = -Zn
            ENDDO
            Smm = Smm*EXS(I,JS)/4.d0 *VST

            K1 = (Jg-1)*Nvs+1
            DO Ivs = 1,Kvs
               Jvs = (Ivs-1)*NSZA+Js
               Ac(1) = SDO%Ajp(Ivs,I,Km,Js)
c                    ... This term is zero if ADAPTIVE_GRID =.FALSE.
               Ac(2) = SDO%Ajp(Ivs,Id_a(I)-1,Km,Js)
               Bc(1) = SDO%Bjp(Ivs,I,Km,Js)
c                    ... This term is zero if ADAPTIVE_GRID =.FALSE.
               Bc(2) = SDO%Bjp(Ivs,Id_b(I)+1,Km,Js)

               R_cof = (3.d0*(Mus(Js)-Cg)-Mus(Js)*H_1)*Smm
     $              + Ac(2)*(DTA - C_DM) + Bc(1)*C_UP 
     $              +( Ac(1)*C_DM - Bc(2)*(C_UP+DTA) )*Ecg
               
               P_cof = -( Ac(2)*(C_DP+DTA) - Bc(1)*C_UM )*Ecg
     $              +( Ac(1)*C_DP + Bc(2)*(DTA - C_UM) )
     $              -0.25D0*( 3.d0*(Mus(Js)+Cg) - Mus(Js)*H_1)
     $              *Cmm*EXS(I,JS)*Vst

c                    ... The first Stokes component only
               Irl(I,Jvs,K1) = Irl(I,Jvs,K1) + R_cof
     $              + IOP%MVEC(Lk+Km,Jvs)*A_ref 
     $              + IOP%MVEC(LR1+Km,Jvs)*B_ref
               Itl(I,Jvs,K1) = Itl(I,Jvs,K1) + P_cof
     $              + IOP%MVEC(Lk+Km,Jvs)*A_COF 
     $              + IOP%MVEC(LR1+Km,Jvs)*B_COF
            ENDDO  ! over DO-solutions (Ivs)
         ENDDO  ! over solar angles (Js)
      ENDDO  ! over Gaussian angles (Jg)

      END SUBROUTINE PURE_SCAT
c ************************************************************

      END 

