C**********************************************************************
C
C       SCIATRAN - Version 4.5 - Last modification: 10 Nov 2021
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
c
c      SFI technique to calculate reflected radiance at all level
c           for all solar angles and all viewing angles.
c                  Thermal emission is accounted for. 
c      See theoretical consideration in Sec.(SFI-Reflection) of STB
c      STB --> SCIATRAN Theoretical Background
c *********************************************************************
      SUBROUTINE ST_UP_SFI(NL, Mh, NLSA, GSL, IOP, SSB, TE, SDO,
     $     Sf_a, Sf_b, P0_LS, N_PLS, FLUX, Ref_surf, UDB, Bet, Trans, 
     $     Sfi_res, NSZA, POS)
c     ... Called by "post_processing.f"

      USE BRDF,        ONLY: Exact_BRF, Npg, L_SOL
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE FLU_LAND,    ONLY: FLU_L
      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: NL_pla, Nf_pla, Nvs, Ng1_brf, Ns_brf, 
     $                       NN_pla, LS_pla
      USE IRRAD,       ONLY: Fbe
      USE LOG_PARS,    ONLY: DIR_OUT
      USE NODES,       ONLY: MJU_PLUS, PHI_NODE
      USE RTM_OUTPUT,  ONLY: Wup, N_OP
      USE ST_VARS,     ONLY: N_RHS, Pie, F14, VST, Finc, Kvs, D_sol,
     $                       RAD_FIELD, SURF_TYPE
      USE SPHER_PARS,  ONLY: Jv_S, Js_E, Nf => NN
      USE TYPE_VARS,   ONLY: KND, DCMP
      USE TYPES,       ONLY: TE_TYPE, SDO_TYPE, UDB_TYPE, SSB_TYPE,
     $                       GSL_TYPE, IOP_TYPE
      USE WATER,       ONLY: ERDR
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NL, Mh, N_PLS, NLSA, NSZA
c     ----------------------------------------------------------------
      TYPE (GSL_type) :: GSL    ! Angles
      TYPE (IOP_type), TARGET :: IOP    ! Integral optical parameters
      TYPE (SSB_type) :: SSB    ! Spherical solar beam
      TYPE (TE_type)  :: TE     ! Thermal emission
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (UDB_TYPE) :: UDB    ! Boundary conditions
c     -------------------------------------------------------------
      REAL (KND), INTENT(IN) :: Trans(Kvs*NSZA,Nvs*NLSA), POS
c     ... P0_LS(:,:,:,:) is set to  Ph_zA%EsL(:,:,:,:) in "post_processing.f"
      REAL (KND) P0_LS(NL,Nvs*NLSA,Nvs*NSZA,N_PLS), 
     $     Sf_a(NL-1,Nvs*NLSA,GSL%NN*Nvs), 
     $     Sf_b(NL-1,Nvs*NLSA,GSL%NN*Nvs), 
     $     Ref_surf(N_RHS,Nvs*(GSL%NN+Npg)),  
     $     Sfi_res(NL,Kvs*NSZA,Nvs*NLSA), Flux(Kvs*NSZA),
     $     Bet(NL), S_L(Nvs,Nvs)
c     Local variables 
c     ---------------
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'ST_UP_SFI'
      INTEGER  I, Ivs, J, Jco, Js, Jv, Jvs, Jc, K, K1, K2, Ko, Ks, Kv,    
     $       Km, Jro, LL1, Lr1, Lk, NN, NNv, Zn
      LOGICAL  PURE(GSL%NN*Nvs)
      REAL (KND)  Cs, Cv, Ecv, DTA, Ck, SMd, SMd_p, Smd_e, SOU_p, Wdo,
     $     S1, CMd
      REAL (KND) Tls(Kvs*NSZA,Nvs*NLSA), Fs_INT(Nvs*NLSA),  
     $     Fg_INT(Nvs*NLSA), SOU(NSZA), Muv(NLSA),
     $     Usol(NSZA)
c          ... Check spherical integration option
     $     ,D_DS(2,NL), SSSF(NL), EX_L(NL), D2
      REAL (KND), DIMENSION (GSL%NN*Nvs) :: Tr1, Tr2, Sju, Cju
      REAL (KND), DIMENSION (GSL%NN*Nvs) :: Apa, Bpa
      REAL (KND), DIMENSION (Kvs,GSL%NN*Nvs,NSZA) :: Hapa, Hbpa
      COMPLEX (kind(0.0d0)) :: Muc, C1, C2, Cnu, Cmu
      INTEGER, POINTER :: Id_a(:), Id_b(:)
      INTEGER, ALLOCATABLE :: Jh1(:), Jh2(:)
      REAL (KND), POINTER :: MuS(:), MVEC(:,:), Nuj(:), Enu(:),
     $     EXP_D(:), EXP_U(:), EXS(:,:)
      REAL (KND), ALLOCATABLE :: Irj(:,:,:), X_ref(:), HDU(:), 
     $     BC_S1(:), LBC(:), T_AU(:), HPL(:,:,:) 
      REAL (KND)  DtU, DtL
      REAL (KND), TARGET :: Hnu(GSL%NN*Nvs), Exu(GSL%NN*Nvs)
c    ===================================================================

      INTERFACE

         SUBROUTINE REFL_DIR_SPH(NLSA, NSZA, NPHI, MuV, Mue, Phi, POS, 
     $        ALBEDO, FLU_int, SOL_EXP, PLANCK_SURF, BC_S1, LBC,
     $        REF_SUR)
        USE BRDF, ONLY: BRDF_PARAM, DO_OC_COMP, IVG_cof, 
     $       Nwl_veg, WL_veg, Wgr_svk, Alb_svk, N_svk
        USE BRDF_SUBRS,  ONLY: BRDF_ACT_DOM
        USE INTERN_DIMS, ONLY: Nvs
        USE ST_VARS, ONLY: SURF_TYPE, BRDF_FLAG
        USE TYPE_VARS, ONLY: KND
        USE WATER,  ONLY: SS_PDF
        INTEGER,    INTENT(IN) :: NLSA, NSZA, NPHI
        REAL (KND), INTENT(IN) :: ALBEDO, FLU_int, POS, SOL_EXP,
     $       PLANCK_SURF, MuV(NLSA), Mue(NSZA), Phi(NPHI)
        REAL (KND) BC_S1(Nvs), LBC(Nvs)
        REAL(KND), ALLOCATABLE :: REF_SUR(:,:)
        OPTIONAL :: REF_SUR
        END SUBROUTINE REFL_DIR_SPH

      END INTERFACE
c    ===================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Wdo = 1.d0 - Wup
      NN = GSL%NN        ;  NNv = NN*Nvs
      MuS  => GSL%MuS(:) ;  MVEC => IOP%MVEC(:,:)
c      WRITE(*,*) '  st_up_sfi.f  SIZE(GSL%MuS) ', SIZE(GSL%MuS) 
c      WRITE(*,*) '  st_up_sfi.f --> GSL%MuS ',
c     $     acos(GSL%MuS(:))/Pie*180.
c      PAUSE  
      Id_a => IOP%Id_a   ;  Id_b => IOP%Id_b  ;  EXS => SSB%EXS
      IF( RAD_FIELD ) THEN
         Muv(:) = DABS( MJU_PLUS(:) )
         ALLOCATE( Jh1, SOURCE=[(1,I=1,NLSA)] )
         ALLOCATE( Jh2, SOURCE=[(NSZA,I=1,NLSA)] )
         IF( Mh == 1 ) ALLOCATE( BC_S1(Nvs), LBC(Nvs) )
      ELSE
         Muv(:) = GSL%Muv(:)
         ALLOCATE( Jh1, SOURCE=Jv_S )
         ALLOCATE( Jh2, SOURCE=Js_E )
      ENDIF

      ALLOCATE( Irj(NL-1,Kvs*NSZA,Nvs*NLSA) )
      Irj = 0.d0

c ******************Specular reflection start *************************
      IF( UDB%SPECULAR ) THEN         
         ALLOCATE( T_AU(NL) ) ; T_AU(1) = 0.d0
         DO I = 2,NL
            T_AU(I) = SUM( IOP%DTR(1:I) )
         ENDDO
      ENDIF  ! specular reflection
c******************  Specular reflection end *****************************

c****************** Thermal emission start *****************************
      IF( TE%ADD_PLANCK ) THEN
        ALLOCATE( HPL(NL_pla-1,NN_pla,SIZE(MuV)) )
         DO I = 1,NL - 1        ! do-loop over layers
            DTA = IOP%DTR(I+1)  ;  SOU_p = DEXP( TE%Mup(I)*DTA )  
            Cs = -TE%Mup(I)
            DO Jv = 1,NLSA      ! do-loop over line-of-sight angles
               Cv = MuV(Jv) ; ECV = DEXP( -DTA/Cv ) 
c              ... Calculate the function S_te as given by Eq.(S-te) in STB
               CALL Sf_PLA_REAL( Cs, Cv, SOU_p, ECV, SMd_p )
c              ... Second term in functions RAj(mu) and RBj(mu) in 
c                  Eqs.(RAj-the) (RBj-the) in STB
               HPL(I,:,Jv) = TE%Aj_pla(:,I)*SMd_p*CV
               IF( .NOT.DO_SSRCA ) THEN 
                  DO K1 = (Jv-1)*Nvs+1, Jv*Nvs, Nvs
                     Irj(I,1:NSZA,K1) = SMd_p*TE%NPLANCK(I)/CV*VST
                  ENDDO  ! over first Stokes component
               ENDIF   
            ENDDO  ! over line-of-sight angles (Jv)
         ENDDO  ! over layers (I)
      ENDIF  ! thermal emission
c****************** Thermal emission end *******************************

      DO I = 1,NL - 1           ! do-loop over layers
         Jco = SDO%NcC(I)  ;  Jro = NNv - 2*Jco 
         LL1 = (I-1)*2*NNv + 1 ; LR1 = (I-1)*2*NNv + NNv ; Lk = LL1 - 1

c        ... Set PURE(E_MIN(I)) to .false. if in the layer pure scattering
         PURE(:) = .TRUE. 
         IF( SDO%MASK_NL(I) ) PURE(SDO%E_MIN(I)) = .FALSE.
         Nuj   =>  SDO%Nuj_lay(:,I) 
c        ... These functions were calculated in "iop_iface.f"
         EXP_D => IOP%Ex_D(:,I) ; EXP_U => IOP%Ex_U(:,I)

         DTA = IOP%DTR(I+1)
         SOU(:) = DEXP( -DTA/SSB%Mue(:,I) )
         DtU = 0.d0
c            Hapa(:,:) = SDO%Ajp(1,I,:,:)
         Hapa(:,:,:) = 0.d0
         Hbpa(:,:,:) = SDO%Bjp(:,I,:,:)
         Exu(1:Jro) = 1.d0
         DO Ko = Jro+1,Jro+2*Jco,2 
            Exu(Ko) = 1.d0 ; Exu(Ko+1) = 0.d0
         ENDDO
         Usol(:) = 1.d0
         Enu => SDO%EXP_NU_NL(:,I)

         DO Jv = 1,NLSA  ! do-loop over L-of-S angles
            Kv = (Jv - 1)*Nvs
            Cv = MuV(Jv) ; ECV = DEXP( -DTA/Cv )            
c            IF( Cv==1.d0 ) Cv = 0.9999d0
c    >>>>>> Calculation of S(nuj,mu) and C(nuj,mu) functions in the 
c           first term of the expression for functions Raj(mu) and  
c           Rbj(mu) in Eqs.(Raj-mu) and (Rbj-mu) in STB 
c           --> a) real separation constants
            Sju(1:Jro) = Nuj(1:Jro)*CV/(Nuj(1:Jro) + CV)
     $           *( 1.d0 - Enu(1:Jro)*ECV )*EXP_U(1:Jro)*Exu(1:Jro) 
            WHERE( REAL(Nuj(1:Jro)) /= REAL(CV) )
               Cju(1:Jro) = Nuj(1:Jro)*CV/(Nuj(1:Jro) - CV)
     $              *(Enu(1:Jro) - ECV)*EXP_D(1:Jro)
            ELSEWHERE
               Cju(1:Jro) = Enu(1:Jro)*DTA*EXP_D(1:Jro)
            END WHERE            
c           --> b) imaginary separation constants
            DO K1 = Jro+1, Jro+2*Jco, 2
               C1  = DCMP( Nuj(K1),Nuj(K1+1) )
               Cnu = DCMP( Enu(K1),Enu(K1+1) )
               C2 = C1*CV*(Cnu-ECV)/(C1-CV)
     $              *DCMP(EXP_D(K1),EXP_D(K1+1))
               Cju(K1) = DBLE(C2) ; Cju(K1+1) = AIMAG(C2)

               C2 = C1*CV*(1.d0 - Cnu*ECV)/(C1+CV)
     $              *DCMP(EXP_U(K1),EXP_U(K1+1))
     $              *DCMP(Exu(K1),Exu(K1+1))
               Sju(K1) = DBLE(C2) ; Sju(K1+1) = AIMAG(C2)
            ENDDO
c    <<<<<< End of S(nuj,mu) and C(nuj,mu) functions calculations. 
c
c    >>>>>> Calculation of S(mu,mu0) function in the 
c           second term of the expression for functions RAj(mu) and  
c           RBj(mu) in Eqs.(RAj-dir) (RBj-dir) in STB
            DO Js = Jh1(Jv),Jh2(Jv) ! do-loop over solar zenith angles
               Ks = (Js - 1)*Nvs
               CS = SSB%Mue(Js,I) 
               Smd = Cs*Cv/(Cs + Cv)*(1.d0 - SOU(Js)*ECV)
               Zn = -1
c              ... If approximation for spherical atmosphere is used:
               DO Jc = 1,SSB%N_sph(Js,I)-1
                  Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-CS )
                  SMd = SMd + AIMAG( Muc*Cv/(Muc+Cv) )
     $                 *SSB%SIN_COEF(Js,I,Jc)*(1.0d0 - Zn*SOU(Js)*ECV)
                  Zn = -Zn
               ENDDO
c              <<<<<< End of S(mu,mu0) function calculations. 

               Smd_e = Smd*EXS(I,Js)*Usol(Js)
               DO Ivs = 1,Kvs
                  Jvs = (Ivs - 1)*NSZA + Js               
c       >>>>>> Calculation of the term in square brackets of the function
c              RAj(mu) in Eq.(RAj-dir) in STB
                  Apa(:) = Hapa(Ivs,:,Js)*CV 
     $                 + ( -SDO%Ajp(Ivs,I,:,Js)*ECV 
c                         ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                     + SDO%Ajp(Ivs,Id_a(I)-1,:,Js)
     $                     + SDO%Aj_sol(Ivs,:,I,Js)*Smd_e )*CV 

c              Calculation of the term in square brackets of the function
c              RBj(mu) in Eq.(RBj-dir) in STB
                  Bpa(:) = ( Hbpa(Ivs,:,Js) *Usol(Js)
c                         ... This term is zero if ADAPTIVE_GRID =.FALSE.
     $                     - SDO%Bjp(Ivs,Id_b(I)+1,:,Js)*ECV
     $                     - SDO%Bj_sol(Ivs,:,I,Js)*Smd_e )*CV 

c     ************* Specular reflection start *************************
                  IF( UDB%SPECULAR ) THEN
                     IF( REAL(CS) /= REAL(CV) ) THEN
                        Cmd = CS*CV*(SOU(Js) - Ecv)/(CS - CV)
                     ELSE
                        Cmd = Ecv*DTA
                     ENDIF
                     S1 = Cmd*EXS(NL,Js)
     $                    *DEXP(-(T_AU(NL)-(T_AU(I)+DTA))/CS )
c                 ... We use here that in the scalar case 
c                     aj(specular)=bj(dir) and bj(specular)=aj(dir)
c                     Apa(:) = Apa(:) + SDO%Bj_sol(Ivs,:,I,Js)*S1*CV 
c                     Bpa(:) = Bpa(:) - SDO%Aj_sol(Ivs,:,I,Js)*S1*CV 
c                  ... In the vector case aj_spe and bj_spe contains
c                      Fresnel coefficient (see homog_partic.f)
                     Apa(:) = Apa(:) + SDO%Aj_spe(Ivs,:,I,Js)*S1*CV 
                     Bpa(:) = Bpa(:) - SDO%Bj_spe(Ivs,:,I,Js)*S1*CV 
c                    ... Reflected solar beam
                     S_L(:,:) = Fbe*F14*Cmd*EXS(NL,Js) 
     $                    *DEXP(-(T_AU(NL)-(T_AU(I)+DTA))/Cs )/CV
     $                    *( Wup*P0_LS(I,  Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,2)
     $                    + Wdo*P0_LS(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,2) )
     $                    /IOP%ALW(I)
                     K2 = 1
                     DO K1 = Kv+1,Kv+Nvs
                        Irj(I,Jvs,K1) = Irj(I,Jvs,K1)
c     $                       + SUM( Finc(Ivs,:)*S_L(:,K2) )
     $                       + SUM( UDB%Fref(Js,Ivs,:)*S_L(:,K2) )
                        K2 = K2 + 1
                     ENDDO  ! over all Stokes components
c                     WRITE(*,*) ' UDB%Fref ', UDB%Fref(Js,Ivs,1)
                  ENDIF         ! specular reflection
c     ************* Specular reflection end ***************************

                  IF( TE%ADD_PLANCK.AND.Ivs==1 ) THEN
                     Apa(:) = Apa(:) + HPL(I,:,Jv)
                     Bpa(:) = Bpa(:) - HPL(I,:,Jv)
                  ENDIF  ! thermal emission

c       >>>>>> Sum all contributions into functions Raj(mu) and  
c              Rbj(mu) in Eqs.(Raj-mu) and (Rbj-mu) in STB 
c              --> a) real separation constants
                  WHERE( PURE(1:Jro) )  
                     Tr1(1:Jro) = MVEC(LL1:Lk+Jro,Jvs)*Sju(1:Jro)
     $                    + Apa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) + Cv)
                     Tr2(1:Jro) = MVEC(LR1+1:LR1+Jro,Jvs)*Cju(1:Jro)
     $                    + Bpa(1:Jro)*Nuj(1:Jro)/(Nuj(1:Jro) - Cv)
                  ELSEWHERE
                     Tr1(1:Jro) = 0.d0 ; Tr2(1:Jro) = 0.d0
                  END WHERE
c                          WRITE(*,*) '  Apa =  ', SUM( Apa(1:Jro) )
c                --> b) imaginary separation constants    
                  DO K1 = Jro+1, Jro+2*Jco, 2
                     Cnu = DCMP( Nuj(K1),Nuj(K1+1) )
                     C1 = DCMP(Apa(K1),Apa(K1+1))*Cnu/(Cnu+Cv)
                     Tr1(K1) = MVEC(Lk+K1,Jvs)*Sju(K1) + 
     $                    MVEC(Lk+K1+1,Jvs)*Sju(K1+1) + 2.d0*DBLE(C1)
                     Tr1(K1+1) = MVEC(Lk+K1+1,Jvs)*Sju(K1) - 
     $                    MVEC(Lk+K1,Jvs)*Sju(K1+1) - 2.d0*AIMAG(C1)

                     C2 = DCMP(Bpa(K1),Bpa(K1+1))*Cnu/(Cnu - Cv)
                     Tr2(K1) = MVEC(LR1+K1,Jvs)*Cju(K1) +
     $                    MVEC(LR1+K1+1,Jvs)*Cju(K1+1) + 2.d0*DBLE(C2)
                     Tr2(K1+1) = MVEC(LR1+K1+1,Jvs)*Cju(K1) - 
     $                    MVEC(LR1+K1,Jvs)*Cju(K1+1) - 2.d0*AIMAG(C2)
                  ENDDO
c                  write(*,*) '    b - ', SUM( Tr2(:) )

c       <<<<<< End of calculation functions Raj(mu) and  Rbj(mu)
c           
c       >>>>>> Integral over tau from single and multiple scattering source 
c                 functions in the I-th atmospheric layer
c                 (see Eq.(Del-Q-ref) in STB)
c                 ... Direct solar beam
                  S_L(:,:) = Fbe*F14*Smd_e/Cv/IOP%ALW(I)
     $                 *( Wup*P0_LS(I,  Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1)
     $                 + Wdo*P0_LS(I+1,Kv+1:Kv+Nvs,Ks+1:Ks+Nvs,1) )
                  K2 = 1
                  DO K1 = Kv+1,Kv+Nvs
                     Irj(I,Jvs,K1) = Irj(I,Jvs,K1) 
     $                    + SUM( Finc(Ivs,:)*S_L(:,K2) )
     $                    + SUM( Tr1(:)*Sf_a(I,K1,:)
     $                         + Tr2(:)*Sf_b(I,K1,:) )
                     K2 = K2 + 1
                  ENDDO  ! over all Stokes components (K1)

               ENDDO  ! over DO-solutions (Ivs)->(Jvs)
            ENDDO  ! over solar zenith angles (Js)
         ENDDO  ! over line-of-sight angles (Jv)
         IF( ANY(.NOT.PURE(:)) ) CALL PURE_SCAT_UP
      END DO  ! over layers (I)

c     **************************************************************
c        Include contribution of the surface-reflected radiation
c     **************************************************************
      Tls = 0.d0 
      SELECT CASE( TRIM(UDB%LBC) )
      CASE('Lambert')
         IF( Mh==1 ) THEN
            FORALL(Jv = 1:Nvs*NLSA:Nvs) 
     $           Tls(1:NSZA,Jv) = 2.d0*UDB%Albedo*FLUX(1:NSZA)
c            IF( .NOT.DO_SSRCA.AND.(.NOT.TE%NO_SUN) ) THEN
            IF( .NOT.DO_SSRCA ) THEN
c              ... VST=2 (vector) VST=1 (scalar) see Eq.(ILam-ref) in STB
               FORALL(Jv=1:Nvs*NLSA:Nvs) Tls(1:NSZA,Jv) = Tls(1:NSZA,Jv) 
     $              + UDB%Albedo*VST*MuS(:)*EXS(NL,:)*Finc(1,1)
            ENDIF  ! single scat. corr. does not use
         ENDIF

      CASE('Bidirect')         
         ALLOCATE( X_ref(NN+1) )
         X_ref(1:NN) = GSL%XM(:) ; X_ref(NN+1) = 1.d0
         DO Js = 1,Kvs*NSZA
            IF( Exact_BRF ) THEN
c              ... Reflected radiation is calculated at viewing directions
               Fg_INT(:) = Ref_surf(Js,Nvs*NN+1:Nvs*(NN+Npg))
            ELSE
c              ... Interpolate reflected from surface "radiation"
c                  on the viewing directions
               DO K1 = 1,Nvs       
                  CALL AKIMA_INT( NN+1, X_ref, 
     $                 Ref_surf(Js,K1:Nvs*(NN+1):Nvs),  
     $                 NLSA, Muv, Fg_INT(K1:Nvs*NLSA:Nvs) )
               ENDDO  ! over Stokes components
            ENDIF  ! Exact_BRF
            Tls(Js,:) = 2.d0*Fg_INT(:)
         ENDDO         

         IF( .NOT.DO_SSRCA ) THEN 
c           ------------------------------------------------------------
            Surf_refl: Block
              IF( .NOT.RAD_FIELD ) THEN
                 CALL REFL_HARM  ;  EXIT Surf_refl
              ENDIF
              IF( Mh > 1 )  EXIT Surf_refl
c              WRITE(*,*) ' -------> RAD_FIELD = .TRUE.  '
c              WRITE(*,*) '    SUM(Tls) = ', SUM(Tls)
              DO Jv = 1, NLSA   ! = N_PLUS in ths case
                 K1 = (Jv-1)*Nvs
                 DO Js = 1,NSZA
                    IF( L_SOL(Js) ) CYCLE
                    CALL REFL_DIR_SPH(1,1,1, Muv(Jv:Jv), MuS(Js:Js), 
     $                   PHI_NODE(Jv:Jv), POS, UDB%ALBEDO, UDB%FLU_int,
     $                   EXS(NL,Js), TE%PLANCK_SURF, BC_S1, LBC)
                    DO Jc = 1, Nvs
                       Tls(Js,K1+Jc) = Tls(Js,K1+Jc) + BC_S1(Jc)*VST 
                    ENDDO  ! over Stokes components (Jc)
                 ENDDO  ! over solar angles (Js)
              ENDDO  ! over LoS angles (Jv)              
            End Block Surf_refl
c           ------------------------------------------------------------
         ENDIF  ! single scat. corr. does not use

      CASE('Flat water')
c         WRITE(*,*) '    ------> "Flat water" option'
         DO Js = 1,NSZA
            Tls(Js,:) = Tls(Js,:) + Trans(Js,:)
         ENDDO
         
         IF( SURF_TYPE=='melt_pond'.AND.(.NOT.DO_SSRCA) ) THEN
            ALLOCATE( X_ref(NN+1) )
            X_ref(1:NN) = GSL%XM(:) ; X_ref(NN+1) = 1.d0
            CALL REFL_HARM 
         ENDIF
         
c     ===> 'Rough water'
      CASE('Rough water')
c     ... The radiation reflected from the rough water surface and 
c         radiation transmitted from the ocean is included
c        ... Contribution of diffuse reflected radiation
c         WRITE(*,*) '   Option "Rough water"'
c         WRITE(*,*) '   SHAPE(Ref_surf)  ', SHAPE(Ref_surf) 
c         WRITE(*,*) '   SHAPE(GSL%XM)  ', SHAPE(GSL%XM)
c         WRITE(*,'(A,2x,2(1x,F8.3))') '  First and last Gauss angles ', 
c     $         DACOS(GSL%XM(1))/Pie*180., DACOS(GSL%XM(NN))/Pie*180.
c         WRITE(*,'(A,2x,*(1x,F8.3))') '  First LOS angles ', 
c     $         DACOS(Muv(1))/Pie*180.
         
         DO Js = 1,Kvs*NSZA
c            write(*,*) '  Ref_surf  '
c            write(*,'(*(1x,e12.4))') Ref_surf(Js,Nvs*(NN-1)+1:Nvs*NN)
            DO K1 = 1,Nvs 
               CALL AKIMA_INT(NN, GSL%XM, Ref_surf(Js,K1:Nvs*NN:Nvs),  
     $              NLSA, Muv, Fg_INT(K1:Nvs*NLSA:Nvs) )
            ENDDO  ! over Stokes components (K1)
            Tls(Js,:) = 2.d0*Pie*Fg_INT(:) + 2.d0*Pie*Trans(Js,:)
c            write(*,*) '  Ref_surf  after interpolation'
c            write(*,'(*(1x,e12.4))') Fg_INT(1:Nvs)
         ENDDO
         
c         IF( .NOT.DO_SSRCA ) THEN 
         IF( .NOT.ERDR ) THEN 
c        ... Contribution of direct reflected radiation for each harmonic
            ALLOCATE( HDU(Nvs*NN) )
            DO Js = 1,NSZA ; Ks = (Js-1)*Nvs
               DO K = 1,Nvs*NN
                  HDU(K) = SUM(UDB%Ra_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(1,:))
               ENDDO
               DO K1 = 1,Nvs
                  CALL AKIMA_INT(NN, GSL%XM, HDU(K1:Nvs*NN:Nvs),  
     $                 NLSA, Muv, Fs_INT(K1:Nvs*NLSA:Nvs) )
               ENDDO
               Tls(Js,:) = Tls(Js,:) + Pie*EXS(NL,Js)*Fs_INT(:) *VST
            ENDDO  ! over SZA (Js)
         ENDIF  ! If single scattering corr. does not use
c     <=== 'Rough water'
      END SELECT

      IF( TE%ADD_PLANCK .AND. (.NOT.DO_SSRCA) ) THEN
         FORALL( Jv = 1:Nvs*NLSA:Nvs ) Tls(1:NSZA,Jv) = Tls(1:NSZA,Jv) 
     $        + VST*TE%PLANCK_SURF
      ENDIF
      
c     ... If land fluorescence is accounted for
      IF( Flu_L.AND.Mh==1.AND.(.NOT.DO_SSRCA) ) 
     $     FORALL( Jv = 1:Nvs*NLSA:Nvs ) Tls(:,Jv) = Tls(:,Jv) 
     $     + VST*UDB%Flu_int

      Sfi_res(NL,:,:) = Tls(:,:)
      DO I = NL - 1,1,-1
         DO Jv = 1,NLSA
            S1 = DEXP(-IOP%DTR(I+1)/Muv(Jv))
            DO K1 = (Jv-1)*Nvs+1,Jv*Nvs
               Tls(:,K1) = S1*Tls(:,K1) + Irj(I,:,K1)
            ENDDO
         ENDDO  ! over observation angles (Jv)
         Sfi_res(I,:,:) = Tls(:,:)
      ENDDO  ! over main altitude layers (I)
      
c      WRITE(*,*) '   sfi_res  in "st_up_sfi" Mh = ', Mh
c      DO I = 1, NLSA  ;   K1 = (I-1)*Nvs
c         WRITE(*,'(5x,I5,*(2x,E12.4))') I, Sfi_res(1,1,K1+1:K1+Nvs)
c        ENDDO      
c **********************************************************************
c      WRITE(*,*) '    Sfi_res(1,1,J ) and  '
c      WRITE(*,*) '    Sfi_res(2,1,J ) in "in st_up_sfi" '
c      DO J = 1, Nvs*NLSA
c         WRITE(*,'(2x,I4,*(1x,e12.4))')  
c     $        J, Sfi_res(1,1,J), Sfi_res(2,1,J)
c      ENDDO
      
c      IF( Vec_Sca == 'vect_rte' ) THEN
c         Js = 1
c         WRITE(*,*) '   ****** Test top upward from ST_UP_SFI :'
c         WRITE(*,*) '   Mharm = ', Mh-1
c         WRITE(*,*) '    NLSA = ', NLSA
c         Open(12,file=DIR_OUT//'/Sfi_res.dat',RECL=15000)
c         DO I = 1, NL
c            WRITE(12,'(130(1x,e15.7))')
c     $           ZZZ(I), ( Sfi_res(I,Js,J ), J=1,Nvs*NLSA)
c         ENDDO
c         Close(12)
c      ELSE
c         WRITE(*,*) '   ****** Test top upward from ST_UP_SFI scalar:'
c         WRITE(*,*) '   Mharm = ', Mh-1
c         WRITE(*,*) '    NLSA = ', NLSA
c         DO I = 1, NSZA
c            WRITE(*,'(1x,I2,4(2x,e15.7))') I, Sfi_res( N_OBS,I,1 )
c         ENDDO
c      ENDIF
cc      STOP '  Stop in ST_UP_SFI'

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      CONTAINS
c **********************************************************************

      SUBROUTINE REFL_HARM
      ALLOCATE( HDU((GSL%NN+Npg)*Nvs) )
      DO Js = 1,NSZA
c       ... About VST see Eq.(LBC-BRDF-harm-s) and  (LBC-BRDF-harm-v)
c           in STB for the scalar and vector case respectively.
         S1 = VST*MuS(Js)*EXS(NL,Js) ; Ks = (Js-1)*Nvs
         DO Ivs = 1,Kvs
            Jvs = (Ivs-1)*NSZA + Js
c           ... Reflection of direct "solar" light from the surface
            IF( Exact_BRF ) THEN
               WRITE(*,*) '   Exact_BRF in "st_up_sfi.f" direct'
               DO K = 1,Nvs*(NN+Npg)
                  HDU(K) = SUM
     $                 (UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
               ENDDO
               Fs_INT(1:Nvs*NLSA) = HDU(Nvs*NN+1:Nvs*(NN+Npg))
            ELSE
c              ... Interpolation of BRDF is used
               DO K = 1,Nvs*(NN+1)
                  HDU(K) = SUM(UDB%BRF_GS(K,Ks+1:Ks+Nvs,Mh)*Finc(Ivs,:))
               ENDDO
               DO K1 = 1,Nvs
                  CALL AKIMA_INT(NN+1,X_ref,HDU(K1:Nvs*(NN+1):Nvs),  
     $                 NLSA, Muv, Fs_INT(K1:Nvs*NLSA:Nvs) )
               ENDDO
            ENDIF
c            WRITE(*,*) '  Fs_INT(:) ', Fs_INT(:)
            Tls(Jvs,:) = Tls(Jvs,:) + S1*Fs_INT(:) 
         ENDDO  ! over DO-solutions (Ivs)->Jvs
      ENDDO  ! over solar angles (Js)
      END SUBROUTINE REFL_HARM

c *********************************************************************
      SUBROUTINE Sf_PLA_REAL( Cs, Cv, ECS, ECV, Sres )
      REAL (KND)    Cs, CV, ECS, ECV, Sres, Ck, Csk
      INTEGER       Jc, Zn
      COMPLEX (kind(0.0d0)) :: Muc
      Sres = Cv/(1.d0 + Cs*Cv)*( 1.d0 - ECS*ECV )
      Zn = -1
c     ... TE%N_pla(I) - number of coefficients a_k used in the 
c         approximation of the Planck function according 
c         to Eq.(Planck-appr) (see STB)
      DO Jc = 1,TE%N_pla(I)-1
         Ck = DTA/DBLE(Jc)/Pie ; Csk = Ck*Cs
         Muc = Ck/DCMP( Csk,-1.d0 )
         Sres = Sres + AIMAG( Muc*Cv/(Muc + Cv) )
     $        *TE%PL_COEF(I,Jc)*(1.d0 - Zn*ECS*ECV)
         Zn = -Zn
      ENDDO
      END SUBROUTINE Sf_PLA_REAL
c *********************************************************************

      SUBROUTINE PURE_SCAT_UP
c     ... The contribution in reflected intensity in the case of 
c         conservative scattering is performed according to 
c         Eqs.(R-amu), - (R-cmu) in STB.
      REAL (KND) H1, a_mu, b_mu, C_1, R_amu, R_bmu, R_cmu
c            WRITE(*,*) '**** conservative: ',I
      H1 = 3.d0 - (Wup*Bet(I) + Wdo*Bet(I+1))/IOP%ALW(I)
      Km = SDO%E_MIN(I)
      DO Jv = 1,NLSA
         Cv = MuV(Jv) ; ECV = DEXP( -DTA/CV )          
         C_1 = 3.d0*Cv/H1
         a_mu = IOP%DTR_D(I) - C_1
c        ... DtU is added according to Eq.(b-mu-arb) in STB
         b_mu = IOP%DTR_U(I) + C_1 + DtU
         R_amu =  a_mu*(1.d0 - ECV) + DTA     
         R_bmu =  b_mu*(1.d0 - ECV) - DTA*ECV 
         DO Js = Jh1(Jv),Jh2(Jv)
            Cs = SSB%Mue(Js,I) 
            Smd = Cs*Cv/(Cs + Cv)*(1.d0 - SOU(Js)*ECV)
            Zn = -1
            DO Jc = 1,SSB%N_sph(Js,I)-1
               Ck = DTA/DBLE(Jc)/Pie ; Muc = CS*Ck/DCMP( Ck,-CS )
               Smd = Smd + AIMAG( Muc*Cv/(Muc+Cv) )
     $              *SSB%SIN_COEF(Js,I,Jc)*(1.d0-Zn*SOU(Js)*ECV)
               Zn = -Zn
            ENDDO
            Smd = Smd*EXS(I,Js)*Usol(Js)/4.d0 *VST
c                  Smd = SMd*EXS(I,Js)/2.d0 ! test vector case Pochemu ??
            K1 = (Jv-1)*Nvs+1
            DO Ivs = 1,Kvs
               IF(.NOT.D_sol(Ivs)) CYCLE
               Jvs = (Ivs-1)*NSZA + Js
               R_cmu = (3.d0*(Mus(Js)-Cv) - Mus(Js)*H1)*SMd 
     $              + (a_mu + DTA)
c                   ... This term = 0 if ADAPTIVE_GRID =.FALSE.
     $              *(SDO%Ajp(Ivs,Id_a(I)-1,Km,Js) + Hapa(Ivs,Km,Js))
     $              + b_mu*Hbpa(Ivs,Km,Js)*Usol(Js)
     $              - ( a_mu*SDO%Ajp(Ivs,I,Km,Js)
c                    ... This term = 0 if ADAPTIVE_GRID =.FALSE.
     $              + (b_mu + DTA)*SDO%Bjp(Ivs,Id_b(I)+1,Km,Js) )*ECV
c              ... The first Stokes component only
               Irj(I,Jvs,K1) = Irj(I,Jvs,K1) + R_cmu
     $              + MVEC(Lk+Km,Jvs)*R_amu + MVEC(LR1+Km,Jvs)*R_bmu
            ENDDO
         ENDDO
      ENDDO      
      END SUBROUTINE PURE_SCAT_UP
c *********************************************************************

      END 
