C**********************************************************************
C
C	SCIATRAN - Version 3.1 - Last modification: 03 Feb 2012
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
c
c    Integration of the vibrational Raman scattering source function
c       to simulate reflected radiation (single scattering case)
c      This is a test version to check implementation of particular
c                    solution in general case
c *********************************************************************
      SUBROUTINE ST_UP_VRS(JPOS, Mh, NN, NSZA, NL, DTR, ALW, 
     $     MuS, Muv, SDO, SVR, RADOUT, *)
c     ... Call in "solv_dom.f"

      USE GRIDS,       ONLY: ZZZ
      USE INTERN_DIMS, ONLY: Nvs
      USE IRRAD,       ONLY: Fbe, IRRAD_FLAG, SOL_FLUX
      USE RTM_OUTPUT,  ONLY: N_OP
      USE SPHER_PARS,  ONLY: NLSA => N_LOS_ANG, N_SL => N_LOS_OUT,
     $                       N_AZ_VAR, JV_S, JV_E, ANG_OUT, 
     $                       NOA => N_OBS_ALT
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: SDO_TYPE, VRS_TYPE
c     ------------------------------------------------------------------

c     ------------------------------------------
      TYPE (SDO_type) :: SDO    ! DOM auxiliary solution variables
      TYPE (VRS_type) :: SVR    ! Structure for vibrational Raman
c     ----------------------
      INTEGER,    INTENT(IN):: JPOS, Mh, NN, NSZA, NL
      REAL (KND), INTENT(IN):: MuS(NSZA), DTR(NL), ALW(NL-1),
     $     Muv(NLSA)
      REAL (KND) RADOUT(N_OP,N_SL,Nvs)
c     -----------------------------------------------------
c     ... Local variables 
c     -------------------
      CHARACTER (LEN=9), PARAMETER :: R_NAME = 'ST_UP_VRS'
      INTEGER  I, J, K, L, Js, LL1, Lr1, Iau, Jact
      REAL (KND) Mu0, Nuk, E_Nk, DTA, Cmk, Smk, Rel, 
     $     S_Ram
      REAL (KND), DIMENSION (NN*Nvs) :: E_Nj, Nuj, Sjk, Cjk, Dum,
     $     Bb_kj, Ba_kj
      REAL (KND) Cmj(NN,NSZA), Smj(NN,NSZA), Em0(NSZA), 
     $     Irj(NL-1,NSZA,NLSA), Tls(NSZA,NLSA), Sfi_res(NL,NSZA,NLSA)
      REAL (KND), POINTER :: Faj(:,:), Fbj(:,:), 
     $     ajs(:,:), bjs(:,:), Apj(:,:), Bpj(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')


      Jact = SVR%Jact
!      WRITE(*,*) '    Harmonic Mh = ',Mh

      DO I = 1,NL - 1
         IF( SDO%MASK_NL(I) ) 
     $        STOP "  VRS can't be calculated for pure scattering "

         LL1 = (I-1)*2*NN + 1 ; LR1 = (I-1)*2*NN + NN ; DTA = DTR(I+1)

c         DO Iau = 1,SVR%N_AUX
         DO Iau = 1,1
            WRITE(*,*) '          Excitation WL :',Iau
c           ... Known constants in the particular solutions of
c               excitation wavelengths
            ajs => SVR%Aj(Iau,Mh,:,I,:)
            bjs => SVR%Bj(Iau,Mh,:,I,:)

c           ... Integration constants Aj and Bj for excitation WL
c                              separation const, SZA
            Apj => SVR%MVEC(Iau,Mh,  LL1:LR1   ,:)
            Bpj => SVR%MVEC(Iau,Mh,LR1+1:LR1+NN,:)

            WRITE(*,*) '         Raman scat :',SVR%V_FRF(1,Iau,Jact)
c            S_Ram = SVR%V_FRF(1,Iau,Jact)/ALW(I)
c            S_Ram = 0.0063/ALW(I) !*10.d0
            S_Ram = 0.00264/ALW(I) !*10.d0 
            S_Ram = S_RAM*SOL_FLUX(Iau)/SOL_FLUX(JPOS)

c           ... First dimension is the Gauss angles, 
c               second - separation constants of excitation wavelengths
            Faj => SVR%Faj(Iau,Mh,I,:,:)
            Fbj => SVR%Fbj(Iau,Mh,I,:,:)

c           ... Relation of extinction coefficients actual/excitation
            Rel = ALW(I)/SVR%EXT_AUX(Iau,I)
            WRITE(*,*) '         Relation ', Rel, ALW(I)

c           ... Extract separation constants for a excitation wavelength
            Nuj(:) = SVR%NUJ_EXC(Iau,Mh,:,I)*Rel

            E_Nj(:) = DEXP( -DTA/Nuj(:) )
            Em0(:)  = DEXP( -DTA/( Rel*Mus(:)) )

            DO Js = 1,NSZA
               Mu0 = Rel*Mus(Js) ! SZA in water
               WHERE( REAL(Nuj(:)) /= REAL(Mu0) )
                  Cmj(:,Js) = Nuj(:)*Mu0/(Nuj(:)-Mu0)*(E_Nj(:)-Em0(Js)) 
               ELSEWHERE
                  Cmj(:,Js) = E_Nj(:)*DTA 
               END WHERE            
               Smj(:,Js)=Nuj(:)*Mu0/(Nuj(:)+Mu0)*(1.d0-E_Nj(:)*Em0(Js)) 
            ENDDO

            DO K = 1,NLSA        ! Over all observation angles
c              ... Cosine of observation angle
               Nuk  = Muv(K)  
               E_Nk = DEXP( -DTA/Nuk )            

               Sjk(:) = Nuj(:)*Nuk/(Nuj(:) + Nuk)*(1.d0 - E_Nj(:)*E_Nk) 
               WHERE( REAL(Nuj(:)) /= REAL(Nuk) )
                  Cjk(:) = Nuj(:)*Nuk/(Nuj(:)-Nuk)*(E_Nj(:) - E_Nk)  
               ELSEWHERE
                  Cjk(:) = E_Nj(:)*DTA 
               END WHERE            

               Dum(:) = Nuk*Nuj(:)/(Nuj(:) + Nuk)
               DO Js = 1,NSZA
                  Mu0 = Rel*Mus(Js) ! SZA in water
                  Cmk = Mu0*Nuk/(Mu0 - Nuk)*(Em0(Js) - E_Nk)
                  Smk = Mu0*Nuk/(Mu0 + Nuk)*(1.d0 - Em0(Js)*E_Nk)
                  WHERE( REAL(Nuj(:)) /= REAL(Nuk) )
                     Bb_kj(:) = Nuk*Nuj(:)/(Nuj(:)-Nuk)*(Smj(:,Js)-Smk)
                  ELSEWHERE
                     Bb_kj(:) = Nuk*Mu0/(Mu0 + Nuk)
     $                    *(Smk - DTA*E_Nk*Em0(Js))
                  END WHERE            
                  
                  Ba_kj(:) = Apj(:,Js)*Sjk(:) 
     $                 + ajs(:,Js)*Dum(:)*(Smk - E_Nk*Cmj(:,Js))
                  Bb_kj(:) = Bpj(:,Js)*Cjk(:) + bjs(:,Js)*Bb_kj(:)
                  
                  Irj(I,Js,K) = SUM( Ba_kj*Fbj(K,:) + Bb_kj*Faj(K,:))
     $                 *S_Ram
               ENDDO            ! over solar angles
            ENDDO               ! over all observation angles
         ENDDO                  ! over excitation wavelengths
         

      ENDDO                     ! over depth layers

c     ... No contribution from bottom
      Tls = 0.d0 

      Sfi_res(NL,:,:) = Tls(:,:)
      DO I = NL - 1,1,-1
         DO K = 1,NLSA
            Tls(:,K) = Tls(:,K)*DEXP(-DTR(I+1)/Muv(K)) + Irj(I,:,K)
         ENDDO
         Sfi_res(I,:,:) = Tls(:,:) 
      ENDDO

      J = 0
      DO I = 1,NSZA
         DO K = JV_S(I),JV_E(I)
            DO L = 1,N_AZ_VAR  ;  J = J + 1                  
               RADOUT(1,J,1) = Sfi_res(NOA(1),I,K)
     $              *DCOS( ANG_OUT(3,J)*DBLE(Mh-1) )
            ENDDO
         ENDDO
      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')

      RETURN 1

      END 

