C********************************************************************C
C
C	SCIATRAN - Version 3.8 - Last modification: 07 May 2017
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ST_UPDATE_MIE
c     ... Call in "rtm_main.f"

      USE AER_MIE,     ONLY: MOD_R, NAC, SIG, RD, RI_rea, RI_ima, N_PSD,
     $                       L_ARR, Nst, LAS_mi, LAE_mi, M_AC, AOD_RET,
     $                       COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, 
     $                       COEF7, COEF8, Rmi_lim, F_MODE, DENS_PRF, 
     $                       AOD_Mie, T_550, Nam_f, Nam_c, WL_RIa, Cfr,
     $                       Rm_log, PM_2i5
      USE AEROSOLS,    ONLY: NAW_MI, AW_MI, LA_Nmi, Nga_mi, Nga_db,
     $                       N_AER_WAVS, AW_DB => AER_WAVS, LA_Ndb, 
     $                       Greek_Aer_db => Greek_Aer, Greek_Aer_mi, 
     $                       APH_VS_db => APH_VS, APH_VS_mi, 
     $                       AC_DB => AER_COEFF, AC_MI, IDX_AER_mi, 
     $                       IDX_AER_db, Plus_mi, LA_S, LA_E,
     $                       Dwr_mi, Use_der, N_Dwr, AER_DER, Bi_PSD,
     $                       SCI => GT_SCAT_IDX, EXI => GT_EXTC_IDX,
     $                       ASI => GT_ASYM_IDX, AP_Mie, NG_aer, 
     $                       D_Greek, APH_DER, MODIS_AP, Rpar, Vpar, 
     $                       Ri_ab, Ri_du, MODIS_LO
      USE CLOUDS_SS,   ONLY: DO_SSRCA
      USE DELTA_M,     ONLY: AER_DM_FLG, FDM_AER_db => FDM_AER, 
     $                       FDM_AER_mi, AER_TRUNC_IDX
      USE GRIDS,       ONLY: NLAYERS, ZZZ, DZ
      USE GT_POLYN,    ONLY: NLEGEN
      USE INFO,        ONLY: AER_INFO
      USE INTERN_DIMS, ONLY: Nm_vs
      USE OutInf,      ONLY: LUNIT
      USE RETR_ITER,   ONLY: ITERATION
      USE RTM_PARS,    ONLY: DATA_ITER, RTM_ST
      USE ST_VARS,     ONLY: RTM_TYPE, Vec_Sca, Pie
      USE TOOLS,       ONLY: CHK_STAT, SET_PTR
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      CHARACTER (LEN=13), PARAMETER :: R_NAME = 'ST_UPDATE_MIE'
      INTEGER, PARAMETER :: NMIE=10000, NPL=2*NMIE
      LOGICAL    NOWR, Modr_f, Modr_c, Var_f, Var_c, Frac_c
      INTEGER    I, J, K, II, K1, L, L1, I1, I2, I0, Id, Idr, N_AWF
      INTEGER    Ma, MCO, Nmi, NKmi, NPmi, NDISTR, Nps, L1_max, NL_DB,
     $     N_DIM, Itr, N13, Kw, M_wl, NAW, LAN
      INTEGER, ALLOCATABLE :: N_EC(:,:,:), L1max(:), LAS(:), LAE(:)
      REAL (KND) MRR, MRI, RX, GAM, R1, R2, F_DM, DUM,  
     $     RVW, RMEAN, BB, AA1, BB1, AA2, BB2, DDELT, TAU, 
     $     N01, N02, Nd1, Nd2, DP, Const, Cdo
      REAL (KND), ALLOCATABLE :: Mor(:), Var(:), CEXT(:), CSCA(:), 
     $     CEXT_Dr(:),CEXT_Ds(:), CSCA_Dr(:), CSCA_Ds(:), 
     $     DB_MOMS(:,:,:,:,:,:), D1(:,:), Hel(:,:), TAU_WL(:),
     $     DB_EXT(:,:,:,:), DB_SCA(:,:,:,:), ECO(:,:,:,:), SCO(:,:,:,:),
     $     Grs(:,:,:), Grs_Dr(:,:,:), Grs_Ds(:,:,:), ASYM_PAR(:,:),
     $     USER_MOM(:,:,:,:,:), SCS(:),  Repa(:,:), Impa(:,:), AREA(:), 
     $     VOL(:)
      TYPE (DATA_ITER), POINTER :: RTM_PTR
      REAL (KND), POINTER :: AER_WAVS(:), AER_COEFF(:,:,:), 
     $     AER_PHASMOMS(:,:,:), APH_VS(:,:,:,:), FDM_AER(:,:)
c     ------------------------------------------------------------------
      CALL ST_TRACK (R_NAME, 1, 'I')      

c     ... Number of aerosol WF:
      N_AWF = COUNT( AER_INFO%WF_FLG )
      IF( N_AWF==0 ) THEN
         WRITE(*,*) '  N_AWF==0   RETURN   =======> '
!         PAUSE
         CALL ST_TRACK (R_NAME, 1, 'O')  ;  RETURN
      ENDIF

      IF( Plus_mi ) THEN
         NAW = NAW_MI      ;  AER_WAVS => AW_MI ; LAN = LA_Nmi
         APH_VS  => APH_VS_mi ; AER_COEFF => AC_MI
         ALLOCATE( LAS(LAN), LAE(LAN) )
         LAS = LAS_mi ; LAE = LAE_mi ; FDM_AER => FDM_AER_mi
      ELSE
         NAW = N_AER_WAVS  ;  AER_WAVS => AW_DB ; LAN = LA_Ndb
         APH_VS  => APH_VS_db ; AER_COEFF => AC_DB
         ALLOCATE( LAS(LAN), LAE(LAN) )
         LAS = LA_S ; LAE = LA_E ; FDM_AER => FDM_AER_db
      ENDIF

c     ... If MODIS aerosol parameterization is used
c     ----------------------------------------------
      IF( MODIS_AP.AND.TRIM(MODIS_LO) /= 'ocean' ) THEN
         SELECT CASE( TRIM(MODIS_LO) )
c        ********************************************
c          MODIS aerosol parameterization over land
c        ********************************************
         CASE('land')
c           T_550 is in this case the retrieved aerosol 
c             optical thickness at 550 nm. 
            IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
               II = ITERATION - 1
               CALL SET_PTR(RTM_PTR, 'aer-dens', ITR = II)
                   WRITE(*,*) ' T_550 ',T_550, ' Iteration ', II
               DO I = 1, NLAYERS
                  IF( RTM_PTR%CRN(I) > 0.d0 ) THEN
                     DUM = RTM_PTR%NXT(I)/RTM_PTR%CRN(I)
                     EXIT
                  ENDIF
               ENDDO
               T_550 = T_550*DUM
                   WRITE(*,*) ' T_550 ',T_550, ' Iteration ', ITERATION
            ENDIF  ! mode 'aer-dens'
            M_wl = 4
            ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl) )
c       ------>  "Fine" mode 
c           ... Calculate TAU-depending modal radius and variance 
c               for 3 predefined aerosol types (fine):
            Tau = T_550
            SELECT CASE( TRIM(Nam_f) )
            CASE('Strongly absorbing')
               IF( T_550 > 2.d0 )  Tau = 2.d0
               N01 = 0.1748d0 *T_550**0.8914d0
               N02 = 0.1043d0 *T_550**0.6824d0
            CASE('Moderately absorbing')
               IF( T_550 > 2.d0 )  Tau = 2.d0
               N01 = 0.1642d0 *T_550**0.7747d0
               N02 = 0.1482d0 *T_550**0.6846d0
            CASE('Weakly absorbing')
               IF( T_550 > 1.d0 )  Tau = 1.d0
               N01 = 0.1718d0 *T_550**0.8213d0 
               N02 = 0.0934d0 *T_550**0.6394d0 
            END SELECT
            DO I = 1,2
               MOD_R(1,I) = Rpar(I,1) + Rpar(I,2)*Tau
               SIG  (1,I) = Vpar(I,1) + Vpar(I,2)*Tau
            ENDDO
            Const = 3.d0/4.d0/Pie
c          ... Particles number of first and second components
            N01 = N01*Const/MOD_R(1,1)**3*DEXP(9.d0/2.d0*SIG(1,1)**2)
            N02 = N02*Const/MOD_R(1,2)**3*DEXP(9.d0/2.d0*SIG(1,2)**2)
c           ... Fraction of first and second components within "fine" mode
            RD(1,1) = N01/(N01 + N02) ; RD(1,2) = N02/(N01 + N02)
                  WRITE(*,*) '    Fraction within "fine" mode'
                  WRITE(*,*) RD(1,1), RD(1,2)
            RD(1,1:2) = RD(1,1:2)*(1.d0 - Cfr)
c           ... Wavelength independent refractive index for absorbing 
            Repa (1,:) = Ri_ab(1) ; Impa (1,:) = Ri_ab(2) + Ri_ab(3)*Tau

c       ------>  "Coarse" mode [Spheroidal (dust)]
c           ... Calculate TAU-depending modal radius and variance (coarse):
            Tau = T_550
            IF( T_550 > 1.d0 )  Tau = 1.d0        
            DO I = 3,4
               MOD_R(1,I) = Rpar(I,1)* Tau**Rpar(I,2)
               SIG  (1,I) = Vpar(I,1)* Tau**Vpar(I,2)
            ENDDO
            N01 = 0.0871d0*T_550**1.0260d0
            N02 = 0.6786d0*T_550**1.0569d0
            Nd1 = N01*Const/MOD_R(1,3)**3*DEXP(9.d0/2.d0*SIG(1,3)**2)
            Nd2 = N02*Const/MOD_R(1,4)**3*DEXP(9.d0/2.d0*SIG(1,4)**2)
c           ... Fraction of first and second components within "coarse" mode
            RD(1,3) = Nd1/(Nd1 + Nd2) ; RD(1,4) = Nd2/(Nd1 + Nd2)
            RD(1,3:4) = RD(1,3:4)*Cfr
c           ... Refractive index for dust at 4 reference wavelengths
            DO K = 1,M_wl
               Repa (2,K) = Ri_du(K,1)*Tau**Ri_du(K,2) 
               Impa (2,K) = Ri_du(K,3)*Tau**Ri_du(K,4)
            ENDDO          
c        ***********************************************************
c          MODIS aerosol parameterization (maritime aerosol network)
c        ***********************************************************
         CASE('man')
            M_wl = 1  
            ALLOCATE( Repa(Nst,M_wl), Impa(Nst,M_wl) )
            CALL MAKE_AER_PAR(T_550, M_AC, Cfr, Repa(:,1), Impa(:,1),
     $           MOD_R, SIG, Nam_f)
            RD(1,1) = 1.d0 - Cfr  ;  RD(1,2) = Cfr
                WRITE(*,*) '   ----> "st_update_mie.f" '
                WRITE(*,*) '   Optical thickness ', T_550
                WRITE(*,*) '   Modal radius and variance: '
                WRITE(*,'(4x,2(1x,F6.3))') MOD_R(1,1), SIG  (1,1)
                WRITE(*,'(4x,2(1x,F6.3))') MOD_R(1,2), SIG  (1,2)
                WRITE(*,*) '   Real and imaginary part: '
                WRITE(*,'(4x,2(1x,F6.3))') Repa(1,1), Impa  (1,1)
                WRITE(*,'(4x,2(1x,F6.3))') Repa(2,1), Impa  (2,1)
                WRITE(*,*) '   Fraction: ', Cfr
         END SELECT  ! 'land' and 'man' within MODIS_AP
      ELSE
c     ***********************************************************
c        Mie option in the case of another parameterization
c                     as "MODIS"-land 
c     ***********************************************************
         I0 = 0
         IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN
            CALL SET_PTR( RTM_PTR, 'aer-dens' )
            WRITE(LUNIT,*) '  Scaling of apriori density '
            WRITE(LUNIT,*) RTM_PTR%CRN(LAS(1))/RTM_PTR%APR(LAS(1))
            DENS_PRF(:) = RTM_PTR%CRN(:)
            I0 = I0 + 1
            AP_Mie(:,I0) = DENS_PRF(:)
         ENDIF  ! 'aer-dens'
c        ----------------------------------------------------------------

         IF(CHK_STAT('aer-modr_smod',AER_INFO%NAME,AER_INFO%WF_FLG))THEN
            WRITE(LUNIT,*) '      Old modal radii: '
            WRITE(LUNIT,'(2x,5(1x,e12.4))') (MOD_R(1,K), K = 1, NAC(1))
            DO I = 1, LAN
               DO K = 1, NAC(I)
                  CALL SET_PTR( RTM_PTR,'aer-modr_smod', ISSP=K )
                  DO J = LAS(I),LAE(I)
                     MOD_R(I,K) = RTM_PTR%CRN(J)
                  ENDDO
               ENDDO
            ENDDO
            WRITE(LUNIT,*) '      New modal radii: '
            WRITE(LUNIT,'(2x,5(1x,e12.4))') (MOD_R(1,K), K = 1, NAC(1))
c            WRITE(6,*) '  RTM_PTR%CRN(:) ', RTM_PTR%CRN(:)
         ENDIF  ! 'aer-modr_smod' 
c        ----------------------------------------------------------------

         IF(CHK_STAT('aer-var_smod',AER_INFO%NAME,AER_INFO%WF_FLG))THEN
            DO I = 1, LAN
               DO K = 1, NAC(I)
                  CALL SET_PTR( RTM_PTR, 'aer-var_smod', ISSP=K )
                  DO J = LAS(I),LAE(I)
                     SIG(I,K) = DEXP( DSQRT( RTM_PTR%CRN(J) ) )
                  ENDDO
               ENDDO
            ENDDO
         ENDIF  ! 'aer-var_smod'
c        ----------------------------------------------------------------

         IF(CHK_STAT('aer-cm_frac',AER_INFO%NAME,AER_INFO%WF_FLG)) THEN
            CALL SET_PTR( RTM_PTR, 'aer-cm_frac' )
c           ... Retrieved course fraction (the same at all layers)
            Cfr = RTM_PTR%CRN(LAS(1))
            WRITE(*,*) '   Check Cm_frac in Ap_mie and RTM_PTR: ', Cfr
            I0 = I0 + 1
            AP_Mie(LAS(1):LAE(1),I0) = Cfr
         ENDIF  ! 'aer-cm_frac'
c        ----------------------------------------------------------------
c          DO I = 1, NLAYERS
c             WRITE(*,'(2x,F7.3, 3(1x,E12.4))') ZZZ(I), DENS_PRF(I)
c          ENDDO
c          DO I = 1, LAN
c             WRITE(*,'(2x,I3, 2(1x,E12.4))') I, MOD_R(I,1), SIG(I,1)
c          ENDDO
      ENDIF  ! non MODIS aerosol parameterization 

c     ... Transform [nm] --> [micron]
      AER_WAVS(:) = AER_WAVS(:)*1.d-3
          
c     ... No screen output in routine "sc_mie.f"
      NOWR = .TRUE. 

      MCO = MAXVAL( NAC(:) )
      Ma  = MAXVAL( N_PSD(:) )
      ALLOCATE( Mor(Ma), Var(Ma), CEXT(Ma), CSCA(Ma), CEXT_Dr(Ma), ! A
     $     CEXT_Ds(Ma), CSCA_Dr(Ma), CSCA_Ds(Ma), TAU_WL(NAW) )    ! A

c     ********* Start Mie ************************************
c     ... Initialize the MIE-routine with general parameters
      Nmi  = 100 ; NKmi = 100 ; NPmi = 4 ;  NDISTR = 2 
c     ... Range for the numerical integration
c     ---------------------------------------
      R1 = Rmi_lim(1) ; R2 = Rmi_lim(2) ; DDELT = 1.d-7
c       ... Transform [nm] --> [micron]

      RX = 2.d0*Pie*R2/AER_WAVS(1)
      L1_Max = 2*IDINT(RX + 4.05d0*RX**0.33333d0 + 8d0)

      ALLOCATE( DB_MOMS(LAN,MCO,NAW,Nm_vs,L1_Max,3),    ! A
     $     N_EC(LAN,MCO,NAW), DB_EXT(LAN,MCO,NAW,3),    ! A
     $     DB_SCA(LAN,MCO,NAW,3),                       ! A
     $     COEF1(L1_MAX), COEF2(L1_MAX), COEF3(L1_MAX), ! A
     $     COEF4(L1_MAX), COEF5(L1_MAX), COEF6(L1_MAX), ! A
     $     COEF7(L1_MAX), COEF8(L1_MAX) )               ! A

      N_EC = 0
      DB_EXT = 0.d0 ; DB_SCA = 0.d0 ; DB_MOMS = 0.d0

      DO L1 = 3,L1_MAX 
         L = L1-1
         COEF1(L1) = 1.d0/DFLOAT(L+1)
         COEF2(L1) = DFLOAT(2*L+1)
         COEF3(L1) = 1.d0/DSQRT(DFLOAT((L+1)*(L+1)-4))
         COEF4(L1) = DSQRT(DFLOAT(L*L-4))
         COEF5(L1) = 1.d0/(DFLOAT(L)*DFLOAT((L+1)*(L+1)-4))
         COEF6(L1) = DFLOAT(2*L+1)*DFLOAT(L*(L+1))
         COEF7(L1) = DFLOAT((2*L+1)*4)
         COEF8(L1) = DFLOAT(L+1)*DFLOAT(L*L-4)
      ENDDO
c             WRITE(*,*) '   SIG updated  '
c             WRITE(*,'(2x,10(1x,e12.4))') SIG(1,1:Mco)
      DO J = 1, Nst
         IF( MODIS_AP.AND.Cfr==0.d0.AND.J==2 ) CYCLE
c        ... Number of wavelengths for refractive index
         L = 1
         DO I = 1, LAN 
            DO K = 1, NAC(I)
               IF( L_ARR(J,I,K) ) THEN
c                 ... Use modal radius and variance to recalculate
c                     optical parameters
                  IF( MODIS_AP ) THEN
                     BB = SIG(I,K) ; Var(L) = BB**2 
                     SELECT CASE( TRIM(MODIS_LO))
                     CASE('land','man') 
                        Mor(L) = MOD_R(I,K)*DEXP( -3.d0*BB**2 )
                     CASE('ocean')
                        Mor(L) = MOD_R(I,K)
                     END SELECT
                     L = L + 1
                  ELSE
                     Mor(L) = MOD_R(I,K)
                     BB = DLOG( SIG(I,K) ) ; Var(L) = BB**2 
c                    ... Recalculate modal radius
                     IF( .NOT.Rm_log ) THEN
                        Mor(L) =  Mor(L)*DEXP( Var(L) )
                     ENDIF
                     L = L + 1
                  ENDIF
               ENDIF
            ENDDO 
         ENDDO                  
c        ... Number of modal radii and variances of log-normal PSD
         Nps = N_PSD(J)

         ALLOCATE( Grs(Nps,6,L1_max), Grs_Dr(Nps,6,L1_max),          ! A
     $        Grs_Ds(Nps,6,L1_max), L1max(Nps), AREA(Nps), VOL(Nps)) ! A

c         WRITE(*,'(A,I2,A,I2)') ' Component ',J,' altogether - ',Nst
c         WRITE(*,'(A,I2)')       ' Nps =  ', Nps
c         WRITE(*,'(a,2(2x,f8.3))')  '  Mod_r and Var: ' 
c         IF( MODIS_AP ) THEN
c            DO I = 1, Nps 
c               WRITE(*,'(2x,I2, 2(2x,f8.3))') I, Mor(I), DSQRT(Var(I))
c            ENDDO
c         ELSE
c            DO I = 1, Nps 
c               WRITE(*,'(2x,I2, 2(2x,f8.3))') 
c     $              I, Mor(I), DEXP(DSQRT(Var(I)))
c            ENDDO
c         ENDIF
         DO K1 = 1, NAW
c            WRITE(*,'(A,I2,A,I2)') 
c     $           '     Wavelength',K1,' altogether - ',NAW
c           .. Real and imaginary part of refractive index
            IF( MODIS_AP ) THEN
               SELECT CASE( TRIM(MODIS_LO))
               CASE('land' )
                  CALL LIN_SEARCH_NEW(AER_WAVS(K1),DP,I1,I2,M_wl,WL_RIa)
                  MRR = Repa(J,I2)*DP + Repa(J,I1)*(1.d0 - DP)
                  MRI = Impa(J,I2)*DP + Impa(J,I1)*(1.d0 - DP)
               CASE('man')
                  MRR = Repa(J,1)  ;   MRI = Impa(J,1)
               CASE('ocean')
                  MRR = RI_rea(J,K1) ;  MRI = RI_ima(J,K1)
               END SELECT
            ELSE
               MRR = RI_rea(J,K1) ;  MRI = RI_ima(J,K1)
            ENDIF

            CALL SC_MIE(Mor(1:Nps), Var(1:Nps), GAM, Nps, AER_WAVS(K1), 
     $           MRR, MRI, R1, R2, Nmi, NPmi, NDISTR, NKmi, L1max, 
     $           Grs, CEXT(1:Nps), CSCA(1:Nps), Grs_Dr, Grs_Ds, 
     $           CEXT_Dr(1:Nps), CSCA_Dr(1:Nps), CEXT_Ds(1:Nps), 
     $           CSCA_Ds(1:Nps), AREA, VOL, RVW, RMEAN, AA1, BB1, AA2,
     $           BB2, DDELT, NOWR, L1_Max, PM_2i5, .FALSE.)

c           ... Save results of Mie calculations
c               CEXT*1.e-3 --> [km-1] i.e. extinction coefficient
c               per 1 particle/cm^3
             L = 1
c             WRITE(*,*) '  Redistribution over layers and components'
             DO I = 1, LAN 
                DO K = 1, NAC(I)
                   IF( L_ARR(J,I,K) ) THEN

                      N_EC(I,K,K1)     = L1max(L)
                      DB_EXT(I,K,K1,1) = CEXT(L)*1.e-3
                      DB_SCA(I,K,K1,1) = CSCA(L)*1.e-3

c                     ... Expansion coefficients contain multiplier (2*L-1)
                      DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),1) = 
     $                     Grs(L,1:Nm_vs,1:L1max(L))            

c                      WRITE(*,'(2x,2(2x,e14.6),2x,I5)')
c     $                     CEXT(L), CSCA(L), L1max(L)
c     $                     SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )

                      Id = 2
                      IF( USE_Der(1) ) THEN
c                        ... Save derivative Dext/Dmod 
                         DB_EXT(I,K,K1,Id) = CEXT_Dr(L)*1.e-3
c                        ... Save derivative Dsca/Dmod
                         DB_SCA(I,K,K1,Id) = CSCA_Dr(L)*1.e-3
c                        ... Save derivative Dcoef/Dmod
                         DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),Id) = 
     $                        Grs_Dr(L,1:Nm_vs,1:L1max(L))
c                      WRITE(*,'(17x,2(2x,e14.6))')
c     $                     DB_EXT(I,K,K1,Id), DB_SCA(I,K,K1,Id)
c                      WRITE(*,'(20x,I4,2x,e15.7)') L1max(L),
c     $                     SUM( Grs_Dr(L,1:Nm_vs,1:L1max(L)) )
                         Id = Id + 1
                      ENDIF
                      IF( USE_Der(2) ) THEN
c                        ... Save derivative Dext/Dvar 
                         DB_EXT(I,K,K1,Id) = CEXT_Ds(L)*1.e-3
c                        ... Save derivative Dsca/Dvar
                         DB_SCA(I,K,K1,Id) = CSCA_Ds(L)*1.e-3
c                        ... Save derivative Dcoef/Dvar
                         DB_MOMS(I,K,K1,1:Nm_vs,1:L1max(L),Id) = 
     $                        Grs_Ds(L,1:Nm_vs,1:L1max(L))
                      ENDIF
                      L = L + 1
                   ENDIF
                ENDDO  ! over component (K)
             ENDDO  ! over aerosol layers (I)
          ENDDO  ! over wavelength (K1)
          DEALLOCATE( Grs, Grs_Dr, Grs_Ds, L1max, AREA, VOL) ! D
       ENDDO     ! over Mie calculations (J)

      DEALLOCATE( COEF1, COEF2, COEF3, COEF4, COEF5, COEF6, COEF7, 
     $     COEF8 )

      AER_WAVS(:) = AER_WAVS(:)*1.d+3 !  [micron] -> [nm]

c     ******************************************************************
c     Calculate update for aerosol extinction, scattering coefficient
c                        and phase matrixes
c     ******************************************************************

c     ... Extinction and scattering coefficients of a given aerosol 
c         component mixture (aerosol type) will be calculated 
c         from the top of atmosphere to the bottom within
c         layers marked as "mie_layer"
      N13 = 1
c     ... If derivative with respect to the modal radius is used
      IF( USE_Der(1) ) N13 = N13 + 1
c     ... If derivative with respect to the variance is used
      IF( USE_Der(2) ) N13 = N13 + 1
c     ... Check for which PSD parameters WFs should be calculated
      Modr_f = CHK_STAT('aer-modr_f', AER_INFO%NAME,AER_INFO%WF_FLG )
      Modr_c = CHK_STAT('aer-modr_c', AER_INFO%NAME,AER_INFO%WF_FLG )
      Var_f  = CHK_STAT('aer-var_f',  AER_INFO%NAME,AER_INFO%WF_FLG )
      Var_c  = CHK_STAT('aer-var_c',  AER_INFO%NAME,AER_INFO%WF_FLG )
      Frac_c = CHK_STAT('aer-cm_frac',AER_INFO%NAME,AER_INFO%WF_FLG )

      ALLOCATE( ECO(2,LAN,NAW,3), SCO(2,LAN,NAW,3), D1(3,NAW), ! A
     $     ASYM_PAR(LAN,NAW), Hel(2,NAW), SCS(NAW) )           ! A

      IF( Frac_c ) THEN
c        ... Update aerosol fraction RD(:,:)
         DO I = 1,LAN
c            K1 = LAN - I + 1
            K1 = I
            Cdo = SUM(RD(K1,1:Mco),.NOT.F_MODE(1:Mco))
            WHERE( F_MODE(1:Mco) )
               RD(K1,1:Mco) = RD(K1,1:Mco)*(1.d0 - Cfr)/(1.d0 - Cdo)
            ELSE WHERE
               RD(K1,1:Mco) = RD(K1,1:Mco)*Cfr/Cdo
            END WHERE
         ENDDO
      ENDIF

      ECO = 0.d0 ; SCO = 0.d0
c      WRITE(*,*) '    ECO and SCO     '
      WRITE(*,*) '    Check fraction of aerosol components    '
      WRITE(*,*) '      Max number of aerosol component : ', MCO
      WRITE(*,'(A,10(1x,e12.4))') '      RD :', RD(1,1:Mco)
      WRITE(*,'(A, 1(1x,e12.4))') ' SUM(RD) :', SUM( RD(1,1:Mco) )
c      PAUSE
      DO I = 1,LAN
c         K1 = LAN - I + 1
         K1 = I
         IF( .NOT.MODIS_AP )  Cfr = SUM(RD(K1,1:Mco),.NOT.F_MODE(1:Mco))

c        ... Coefficients and derivatives if any
         DO Id = 1, N13
c           ... Sum up contribution of aerosol components:
c               over fine mode
            FORALL( Kw=1:NAW ) 
              SCO(1,I,Kw,Id) = 
     $           SUM( DB_SCA(K1,:,Kw,Id)*RD(K1,1:Mco),F_MODE(1:Mco) )
              ECO(1,I,Kw,Id) = 
     $           SUM( DB_EXT(K1,:,Kw,Id)*RD(K1,1:Mco),F_MODE(1:Mco) )
c                over course mode if any
              SCO(2,I,Kw,Id) = SUM
     $           ( DB_SCA(K1,:,Kw,Id)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
              ECO(2,I,Kw,Id) = SUM
     $           ( DB_EXT(K1,:,Kw,Id)*RD(K1,1:Mco),.NOT.F_MODE(1:Mco) )
            END FORALL
c            WRITE(*,*) '      Fine mode :'
c            WRITE(*,'(2(2x,I3),2(2x,E12.4))') 
c     $           I, Id, ECO(1,I,1,Id), SCO(1,I,1,Id)
c            WRITE(*,*) '     Course mode :'
c            WRITE(*,'(2(2x,I3),2(2x,E12.4))') 
c     $           I, Id, ECO(2,I,1,Id), SCO(2,I,1,Id)
         ENDDO  ! over derivatives (Id)   
      ENDDO  ! over aerosol layers (I)

c     ... Multiply ECO and SCO with particle number density profile:
c         DENS_PRF(:) was saved at the main altitude grid snd updated 
c         after retrieval in "preproc_results.f"
      AER_COEFF = 0.d0
      DO I = 1,LAN
         Hel(1,:) = ECO(1,I,:,1) + ECO(2,I,:,1)
         Hel(2,:) = SCO(1,I,:,1) + SCO(2,I,:,1)
         DO J = LAS(I),LAE(I)
            AER_COEFF(EXI,J,:) = Hel(1,:)*DENS_PRF(J)
            AER_COEFF(SCI,J,:) = Hel(2,:)*DENS_PRF(J)
         ENDDO
      ENDDO

      DO K = 1,NAW
         TAU_WL(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
      ENDDO

      IF( MODIS_AP ) THEN      
         IF(TRIM(MODIS_LO)=='land'.OR.TRIM(MODIS_LO)=='man') THEN
                WRITE(*,*) '      T_550 as input in "st_update_mie.f"'
                WRITE(*,'(5x,2x,F8.3)') T_550
c            CALL Angstrom_interp(NAW, AER_WAVS, TAU_WL, 550.d0, Nd1)
c                WRITE(*,*) '    Tau at 550 nm before scaling'
            CALL Angstrom_interp(NAW, AER_WAVS, TAU_WL, 500.d0, Nd1)
                WRITE(*,*) '    Tau at 500 nm before scaling'

                WRITE(*,'(5x,2x,F8.3)') Nd1
c           ... Renormalize the density profile
            DENS_PRF(:) = T_550/Nd1*DENS_PRF(:)
            DO J = LAS(1),LAE(1)
               AER_COEFF(EXI,J,:) = Hel(1,:)*DENS_PRF(J)
               AER_COEFF(SCI,J,:) = Hel(2,:)*DENS_PRF(J)
            ENDDO
c           ... Recalculate optical thickness after scaling
            DO K = 1,NAW
               TAU_WL(K) = SUM( AER_COEFF(EXI,:,K)*DZ(:) )/2.d0
            ENDDO
c            CALL Angstrom_interp(NAW, AER_WAVS, TAU_WL, 550.d0, Nd1)
c                WRITE(*,*) '     Tau at 550 nm after scaling'
            CALL Angstrom_interp(NAW, AER_WAVS, TAU_WL, 500.d0, Nd1)
                WRITE(*,*) '     Tau at 500 nm after scaling'

                WRITE(*,'(5x,2x,F8.3)') Nd1
                WRITE(*,*) ' ======> ' 
         ENDIF  ! 'land' or 'man'
         IF(CHK_STAT('aer-dens',AER_INFO%NAME,AER_INFO%WF_FLG) ) THEN 
            AOD_RET(1,ITERATION) = Nd1 ! Save AOD at 500 nm
         ELSE
            AOD_RET(1,ITERATION) = TAU_WL(NAW)
         ENDIF
      ELSE
c        ... Save retrieved AOD at last Mie wavelength:      
         AOD_RET(1,ITERATION) = TAU_WL(NAW)
      ENDIF                     ! MODIS_AP

c     ... Save retrieved AOD at Mie wavelength grid:      
      AOD_Mie(2,:) = TAU_WL(:)

      WRITE(*,*) ' ======> AOD at all wavelengths: ' 
      DO I = 1, NAW
         WRITE(*,'(5x,2(2x,F8.3))') AER_WAVS(I), TAU_WL(I)
      ENDDO

c     ... Calculate derivatives of scattering and absorption coefficients
c      WRITE(*,*) '   Dwr_mi = ', Dwr_mi

c     ... Calculation of derivatives if any
      IF( Dwr_mi ) CALL MAKE_AER_DER

c     ... Maximal number of expansion coefficients after 
c         Mie calculations
      NL_DB = MAXVAL( N_EC(:,:,:) )
c      WRITE(*,*) '   Maximal number of expansion coeff ',NL_DB
      N_DIM = MAX0(NL_DB,NLEGEN)

      ALLOCATE( USER_MOM(N_DIM,LAN,Nm_vs,NAW,N_Dwr+1) ) ! A

      USER_MOM = 0.d0

c     ... Calculate profiles of expansion coefficients
c         and asymmetry parameter selected aerosol types
c         for mono-modal PSD
      IF( .NOT.Bi_PSD ) THEN
         DO I = 1,LAN 
c            K1 = LAN - I + 1
            K1 = I 
            DO L = 1,NL_DB  
               DO K = 1,Nm_vs
c                 ... Sum up contribution of aerosol components:
c                     into phase matrix 
                  CALL SUM_MOMS(1, 1, F_MODE, Hel(1,:))
                  USER_MOM(L,I,K,:,1) = Hel(1,:)/SCO(1,I,:,1)
c                     into derivatives if any
                  DO I1 = 2,1 + N_dwr
                     CALL SUM_MOMS(1, I1, F_MODE, D1(1,:))
                     CALL SUM_MOMS(I1, 1, F_MODE, D1(2,:))
                     USER_MOM(L,I,K,:,I1) = 
     $                    ( -SCO(1,I,:,I1)*USER_MOM(L,I,K,:,1)
     $                    + D1(1,:) +  D1(2,:) )/SCO(1,I,:,1)
                  ENDDO
               ENDDO            ! over Greek symbols (K)
            ENDDO               ! over expansion coefficients (L)
            ASYM_PAR(I,:) = USER_MOM(2,I,1,:,1)/3.d0
         ENDDO                  ! over aerosol layers (I)
      ELSE
c        ... Case of bimodal PSD
c        -----------------------
         DO I = 1,LAN 
c            K1 = LAN - I + 1
            K1 = I 
            DO L = 1,NL_DB  
               DO K = 1,Nm_vs
                 CALL SUM_MOMS(1, 1,  F_MODE,     Hel(1,:))
                 CALL SUM_MOMS(1, 1, .NOT.F_MODE, Hel(2,:))
                  SCS(:) = SCO(1,I,:,1) + SCO(2,I,:,1)
c                 ... Weighted phase matrix
                  USER_MOM(L,I,K,:,1) = (Hel(1,:) + Hel(2,:))/SCS(:)

                  I0 = 2 ; Id = 2 
                  IF( USE_Der(1) ) THEN
c                    ... Dcoef/Dmod fine mode
                     IF( Modr_f ) THEN
                        CALL SUM_MOMS(1, I0, F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(1,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
c                    ... Dcoef/Dmod course mode
                     IF( Modr_c ) THEN
                        CALL SUM_MOMS(1, I0, .NOT.F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, .NOT.F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(2,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
                     I0 = I0 + 1
                  ENDIF
                  IF( USE_Der(2) ) THEN
c                    ... Dcoef/Dvar fine mode
                     IF( Var_f ) THEN
                        CALL SUM_MOMS(1, I0, F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(1,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
c                    ... Dcoef/Dvar course mode
                     IF( Var_c ) THEN
                        CALL SUM_MOMS(1, I0, .NOT.F_MODE, Hel(1,:))
                        CALL SUM_MOMS(I0, 1, .NOT.F_MODE, Hel(2,:))
                        USER_MOM(L,I,K,:,Id) = 
     $                       ( -SCO(2,I,:,I0)*USER_MOM(L,I,K,:,1)
     $                       + Hel(1,:) +  Hel(2,:) )/SCS(:)
                        Id = Id + 1 
                     ENDIF
                  ENDIF
c                 ... Dcoef/DCf  w.r.t. fraction of course mode
                  IF( Frac_c ) THEN
                     FORALL( Kw=1:NAW ) Hel(1,Kw) = 
     $                    SUM( DB_MOMS(K1,:,Kw,K,L,1)*DB_SCA(K1,:,Kw,1), 
     $                    F_MODE(1:Mco) )
                     FORALL( Kw=1:NAW ) Hel(2,Kw) = 
     $                    SUM( DB_MOMS(K1,:,Kw,K,L,1)*DB_SCA(K1,:,Kw,1),
     $                    .NOT.F_MODE(1:Mco) )
                     FORALL( Kw=1:NAW ) D1(1,Kw) = 
     $                    SUM( DB_SCA(K1,:,Kw,1), F_MODE(1:Mco) )
                     FORALL( Kw=1:NAW ) D1(2,Kw) = 
     $                    SUM( DB_SCA(K1,:,Kw,1), .NOT.F_MODE(1:Mco) )
                     USER_MOM(L,I,K,:,Id) =  
     $                    ( (D1(1,:)-D1(2,:))*USER_MOM(L,I,K,:,1)
     $                    - Hel(1,:) +  Hel(2,:) )/SCS(:)
                  ENDIF         ! Frac_c
               ENDDO            ! over Greek symbols (K)
            ENDDO               ! over expansion coefficients (L)
            ASYM_PAR(I,:) = USER_MOM(2,I,1,:,1)/3.d0
         ENDDO                  ! over aerosol layers (I)
      ENDIF

c     ... Single scattering correction algorithm ->
c              save all expansion coefficients
      IF( DO_SSRCA .OR. TRIM(RTM_TYPE)=='spher_scat' ) THEN
         IF( Plus_mi ) THEN
            Nga_mi = NL_DB
            DEALLOCATE( Greek_aer_mi ) ! D
            ALLOCATE( Greek_aer_mi(Nga_mi,LAN,Nm_vs,NAW) ) ! A
            DO I = 1,NAW 
               Greek_aer_mi(:,:,:,I) = USER_MOM(1:Nga_mi,:,:,I,1)
            ENDDO
            NG_aer = MAX( Nga_db,Nga_mi )
         ELSE
            NG_aer = NL_DB
            IF(ALLOCATED(Greek_aer_db)) DEALLOCATE(Greek_aer_db)
            ALLOCATE( Greek_aer_db( NG_aer,LAN,Nm_vs,NAW ) )
            DO I = 1,NAW 
               Greek_aer_db(:,:,:,I) = USER_MOM(1:NG_aer,:,:,I,1)
            ENDDO
         ENDIF

c        ... Save derivatives of all expansion coefficients
         IF( Dwr_mi ) THEN
            DEALLOCATE( D_Greek )                           ! D
            ALLOCATE( D_Greek(NG_aer,LAN,Nm_vs,NAW,N_dwr) ) ! A
            D_Greek = 0.d0
            DO I = 1,NAW 
               DO Idr = 1, N_dwr
                  D_Greek(1:NL_DB,:,:,I,Idr) = 
     $                 USER_MOM(1:NL_DB,:,:,I,1+Idr)
               ENDDO
            ENDDO
         ENDIF
      ENDIF  ! --> single scattering correction algorithm or 'spher_scat'

c      WRITE(*,*) '  After  DO_SSRCA'

      DO I = 1,LAN 
         DO J = LAS(I),LAE(I)
            AER_COEFF(ASI,J,:) = ASYM_PAR(I,:)
         ENDDO
      ENDDO

c      WRITE(*,*) '  After  AER_COEFF(ASI,J,:)'

      IF( Dwr_mi ) APH_DER = 0.d0

      FDM_AER = 0.d0
      IF( AER_DM_FLG  ) THEN 
         Itr = AER_TRUNC_IDX
         SELECT CASE( Vec_Sca )
         CASE('vect_rte')
c            WRITE(*,*) '   Start AER_DM_VEC '
            CALL AER_DM_VEC
c            WRITE(*,*) '   End AER_DM_VEC '
         CASE('scal_rte')
c            WRITE(*,*) '   AER_DM '
            CALL AER_DM
         END SELECT
      ELSE
         APH_VS(:,:,:,:) = USER_MOM(1:NLEGEN,:,:,:,1)
         IF( Dwr_mi ) APH_DER(:,:,:,:,1:N_Dwr) 
     $        = USER_MOM(1:NLEGEN,:,:,:,2:1+N_Dwr)
      ENDIF

c      WRITE(*,*) '  After delta-M  moda Plus_mi'
c      DO I = 1, NLAYERS
c         WRITE(*,'(2x,I3,2x,F8.3,2(2x,I2),2(2x,E12.4))') 
c     $        I, ZZZ(I), IDX_AER_mi(I), IDX_AER_db(I),
c     $        AER_COEFF(EXI,I,1), AC_DB(EXI,I,1)
c      ENDDO

      CALL ST_TRACK (R_NAME, 1, 'O')      
      RETURN

      CONTAINS
c **********************************************************************
      SUBROUTINE MAKE_AER_DER
      DO I = 1,LAN
         DO J = LAS(I),LAE(I)
            IF( Bi_PSD ) THEN
c              ... Bimodal particle size distribution:
c              ---------------------------------------
               I0 = 1 ; Id = 2 
               IF( USE_Der(1) ) THEN
c                 ... Dsca/Dmod and Dexa/Dmod fine mode
                  IF( Modr_f ) THEN                  
                     AER_DER(SCI,J,:,I0) = SCO(1,I,:,Id)*DENS_PRF(J)
                     AER_DER(EXI,J,:,I0) = ECO(1,I,:,Id)*DENS_PRF(J)
                     I0 = I0 + 1
                  ENDIF         ! Modr_f
c                 ... Dsca/Dmod and Dexa/Dmod course mode
                  IF( Modr_c ) THEN
                     AER_DER(SCI,J,:,I0) = SCO(2,I,:,Id)*DENS_PRF(J)
                     AER_DER(EXI,J,:,I0) = ECO(2,I,:,Id)*DENS_PRF(J)
                     I0 = I0 + 1
                  ENDIF         ! Modr_c 
                  Id = Id + 1
               ENDIF            ! Use_der(1)
               IF( USE_Der(2) ) THEN
c                 ... Dsca/Dvar and Dexa/Dvar fine mode
                  IF( Var_f ) THEN                  
                     AER_DER(SCI,J,:,I0) = SCO(1,I,:,Id)*DENS_PRF(J)
                     AER_DER(EXI,J,:,I0) = ECO(1,I,:,Id)*DENS_PRF(J)
                     I0 = I0 + 1
                  ENDIF         ! Var_f
c                 ... Dsca/Dvar and Dexa/Dvar course mode
                  IF( Var_c ) THEN                  
                     AER_DER(SCI,J,:,I0) = SCO(2,I,:,Id)*DENS_PRF(J)
                     AER_DER(EXI,J,:,I0) = ECO(2,I,:,Id)*DENS_PRF(J)
                     I0 = I0 + 1
                  ENDIF         ! Var_c
               ENDIF            ! Use_der(2)
c              ... Dsca/DCf and Dexa/DCf w.r.t. fraction of course mode
c                  ----------------------------------------------------
               IF( Frac_c ) THEN                  
                  IF( Cfr > 0.d0.AND.Cfr < 1.d0 ) THEN
                     AER_DER(SCI,J,:,I0) = (-SCO(1,I,:,1)/(1.d0-Cfr)
     $                    + SCO(2,I,:,1)/Cfr)*DENS_PRF(J)
                     AER_DER(EXI,J,:,I0) =  (-ECO(1,I,:,1)/(1.d0-Cfr) 
     $                    + ECO(2,I,:,1)/Cfr)*DENS_PRF(J)
                  ELSE
                     WRITE(*,*)  ' Degeneration of bi-model PSD '
                     WRITE(*,*)  '     "mie_aer_setup.f"        '
                     STOP        '   !!! Cfr is 0 or 1 !!!  '
                  ENDIF
               ENDIF            ! Frac_c
            ELSE
               FORALL( K=1:NAW ) 
                 AER_DER(SCI,J,K,:)= SCO(1,I,K,2:1+N_Dwr)*DENS_PRF(J)
                 AER_DER(EXI,J,K,:)= ECO(1,I,K,2:1+N_Dwr)*DENS_PRF(J)
               END FORALL
            ENDIF
         ENDDO                  ! over main altitude grid (J)
      ENDDO                     ! over aerosol layers (I)

      RETURN
      END SUBROUTINE MAKE_AER_DER
c *************************************************************************

      SUBROUTINE SUM_MOMS(Im, Is, MASK, RES)
      INTEGER, INTENT(IN) :: Im, Is
      LOGICAL, INTENT(IN) :: MASK(M_AC)
      REAL(KND) RES(NAW)
      INTEGER   Kw

      FORALL( Kw=1:NAW ) Res(Kw) = SUM( DB_MOMS(K1,:,Kw,K,L,Im)
     $     *DB_SCA(K1,:,Kw,Is)*RD(K1,1:Mco), MASK(1:Mco) )

      END SUBROUTINE SUM_MOMS
c **********************************************************************
      SUBROUTINE AER_DM
      REAL (KND) F_DR
      REAL (KND), ALLOCATABLE :: FDR_AER(:,:,:)

      DO I1 = 1,LAN
         DO K = 1,NAW
            F_DM = USER_MOM(Itr+1,I1,1,K,1)/DBLE(2*Itr + 1)
            FDM_AER(LAS(I1):LAE(I1),K) = F_DM
            DO L = 1,Itr
               APH_VS(L,I1,1,K) = (USER_MOM(L,I1,1,K,1) -
     $              F_DM*DBLE(2*L-1))/(1.d0 - F_DM)
            END DO
         ENDDO
      ENDDO
      APH_VS(Itr+1:NLEGEN,:,1,:) = 0.d0

c      "Renormalize" scattering and extinction coefficients
c     ====================================================
      DO I = 1, NLAYERS
         DO K = 1,NAW
            F_DM = FDM_AER(I,K)
            DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
            AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0 - F_DM)
            AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
         ENDDO
      ENDDO

      IF( Dwr_mi ) THEN
         ALLOCATE( FDR_AER(N_dwr,NLAYERS,NAW) ) ! A
         DO Idr = 1, N_dwr
            DO I1 = 1,LAN
               DO K = 1,NAW
                  F_DM = FDM_AER(LAS(I1),K)
                  F_DR = USER_MOM(Itr+1,I1,1,K,1+Idr)/DBLE(2*Itr + 1)
                  FDR_AER(Idr,LAS(I1):LAE(I1),K) = F_DR
                  DO L = 1,Itr
                     APH_DER(L,I1,1,K,Idr) = 
     $                    (USER_MOM(L,I1,1,K,1+Idr) - F_DR*DBLE(2*L-1))
     $                    /(1.d0 - F_DM) + 
     $                    (USER_MOM(L,I1,1,K, 1) - F_DM*DBLE(2*L-1))
     $                    /(1.d0 - F_DM)**2*F_DR !*DBLE(2*L-1)
                  END DO        ! over expansion coefficients (L)
               ENDDO            ! over reference wavelengths (K)
            ENDDO               ! over aerosol layers (I1)
         ENDDO                  ! over derivatives (Idr)

         DO Idr = 1, N_dwr
            DO I = 1, NLAYERS
               DO K = 1,NAW
                  F_DM = FDM_AER(I,K) 
                  F_DR = FDR_AER(Idr,I,K)
                  DUM = AER_DER(EXI,I,K,Idr) - AER_DER(SCI,I,K,Idr)
                  AER_DER(SCI,I,K,Idr) = 
     $                 AER_DER(SCI,I,K,Idr)*(1.d0 - F_DM)
     $                 - AER_COEFF(SCI,I,K)*F_DR
                  AER_DER(EXI,I,K,Idr) = AER_DER(SCI,I,K,Idr) + DUM
               ENDDO
            ENDDO
         ENDDO
      ENDIF

      END SUBROUTINE AER_DM

c *********************************************************************
      SUBROUTINE AER_DM_VEC
      REAL (KND) F_DR
      REAL (KND), ALLOCATABLE :: FDR_AER(:,:,:)
c     -----------------------------------------
      DO I1 = 1,LAN
         DO K = 1, NAW
            F_DM = USER_MOM(Itr+1,I1,1,K,1)/DBLE(2*Itr + 1)
            FDM_AER(LAS(I1):LAE(I1),K) = F_DM
            DO L = 1, Itr
               APH_VS(L,I1,1:4,K) = ( USER_MOM(L,I1,1:4,K,1) -
     $              F_DM*DBLE(2*L-1) )/(1.d0 - F_DM)
               APH_VS(L,I1,5:6,K) = 
     $              USER_MOM(L,I1,5:6,K,1)/(1.d0 - F_DM)
            ENDDO
         ENDDO
      ENDDO
      APH_VS(Itr+1:NLEGEN,:,:,:) = 0.d0

c     ... "Renormalize" scattering and extinction coefficients
      DO I = 1, NLAYERS
         DO K = 1,NAW
            F_DM = FDM_AER(I,K)
            DUM = AER_COEFF(EXI,I,K) - AER_COEFF(SCI,I,K)
            AER_COEFF(SCI,I,K) = AER_COEFF(SCI,I,K)*(1.d0 - F_DM)
            AER_COEFF(EXI,I,K) = AER_COEFF(SCI,I,K) + DUM
         ENDDO
      ENDDO

c     ... Derivatives w.r.t. expansion coefficients
      IF( Dwr_mi ) THEN
         ALLOCATE( FDR_AER(N_dwr,NLAYERS,NAW) ) ! A
         DO Idr = 1, N_dwr
            DO I1 = 1,LAN
               DO K = 1,NAW
                  F_DM = FDM_AER(LAS(I1),K)
                  F_DR = USER_MOM(Itr+1,I1,1,K,1+Idr)/DBLE(2*Itr + 1)
                  FDR_AER(Idr,LAS(I1):LAE(I1),K) = F_DR
                  DO L = 1,Itr
                     APH_DER(L,I1,1:4,K,Idr) = 
     $                    (USER_MOM(L,I1,1:4,K,1+Idr)-F_DR*DBLE(2*L-1))
     $                    /(1.d0 - F_DM) + 
     $                    (USER_MOM(L,I1,1:4,K,1) - F_DM*DBLE(2*L-1))
     $                    /(1.d0 - F_DM)**2*F_DR !*DBLE(2*L-1)
                     
                     APH_DER(L,I1,5:6,K,Idr) = 
     $                    USER_MOM(L,I1,5:6,K,1+Idr)/(1.d0 - F_DM) + 
     $                    USER_MOM(L,I1,5:6,K,1)/(1.d0 - F_DM)**2
     $                    *F_DR !*DBLE(2*L-1)

                  END DO        ! over expansion coefficients (L)
               ENDDO            ! over reference wavelengths (K)
            ENDDO               ! over aerosol layers (I1)
         ENDDO                  ! over derivatives (Idr)

c        ... Renormalize derivative of scattering and absorption coeff.
         DO Idr = 1, N_dwr
            DO I = 1, NLAYERS
               DO K = 1,NAW
                  F_DM = FDM_AER(I,K) 
                  F_DR = FDR_AER(Idr,I,K)
                  DUM = AER_DER(EXI,I,K,Idr) - AER_DER(SCI,I,K,Idr)
                  AER_DER(SCI,I,K,Idr) = 
     $                 AER_DER(SCI,I,K,Idr)*(1.d0 - F_DM)
     $                 - AER_COEFF(SCI,I,K)*F_DR
                  AER_DER(EXI,I,K,Idr) = AER_DER(SCI,I,K,Idr) + DUM
               ENDDO
            ENDDO
         ENDDO
      ENDIF

      END SUBROUTINE AER_DM_VEC

      END 
