!*********************************************************************
!
!       SCIATRAN - Version 3.3 - Last modification: 13 Mar 2014
!
!       Copyright (C) 1995-2012, Institute of Remote Sensing,
!                                University of Bremen, Germany
!
!*********************************************************************
!
!       Collection of tools in FORTRAN 90 and higher. 
!
!       Contains: CONVOLVE_GAUSS
!*********************************************************************

SUBROUTINE CONVOLVE_GAUSS (N_INP, WL_INP, ARR_INP, N_OUT, WL_OUT, ARR_OUT)
  
  USE TYPE_VARS,   ONLY: KND

  INTEGER N_INP, N_OUT
  REAL(KND) WL_INP(N_INP), ARR_INP(N_INP), WL_OUT(N_OUT), ARR_OUT(N_OUT)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'CONVOLVE_GAUSS'
  INTEGER, PARAMETER :: N_FWS = 4
  INTEGER   N_WL_S, N_WL_E, LL, JJ
  REAL(KND) DELTA, WL_L, WL_R, WEIGHT, D_SUM, W_SUM 
  REAL(KND) FWHM(N_OUT)
  !==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  FWHM(:) = 0.195D0
!  FWHM(:) = 0.20D0

  DO LL = 1, N_OUT
     DELTA = FWHM(LL)*N_FWS/2.0D0
     WL_L = MAX(WL_OUT(LL) - DELTA, WL_INP(1))
     WL_R = MIN(WL_OUT(LL) + DELTA, WL_INP(N_INP))
     N_WL_S = COUNT(WL_INP(:) < WL_L) + 1
     N_WL_E = COUNT(WL_INP(:) < WL_R)
    
     W_SUM = 0.0D0 ; D_SUM = 0.0D0
     DO JJ = N_WL_S, N_WL_E
        WEIGHT = EXP(-DLOG(2.0D0)*(2.*(WL_OUT(LL) - WL_INP(JJ)))**2/FWHM(LL)**2)
        W_SUM = W_SUM + WEIGHT
        D_SUM = D_SUM + WEIGHT*ARR_INP(JJ)
     END DO
     ARR_OUT(LL) = D_SUM/W_SUM
  END DO

  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
END SUBROUTINE CONVOLVE_GAUSS

!*********************************************************************
SUBROUTINE GET_CORRECTION (SSP_M, SSP_C, N_OUT, WL_OUT, L_P, CORR, *)

  USE IRRAD,     ONLY: SOLAR_HR, WL_SOL_HR, N_SOL_HR
  USE TYPE_VARS,   ONLY: KND
  
  INTENT (IN)  SSP_M, SSP_C, N_OUT, WL_OUT, L_P
  INTENT (OUT) CORR
  
  INTEGER     N_OUT, L_P
  REAL (KND)  SSP_M(2), SSP_C(2), WL_OUT(N_OUT), CORR(N_OUT - 2*L_P)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_CORRECTION'
  INTEGER     INBV, II, KK, N_COEFF
  REAL (KND)  D_DIF, WL_MAIN(N_SOL_HR), WL_LOC(N_OUT), SOL_C(N_OUT), &
              SOL_I(N_OUT - 2*L_P), WORK(5*(N_OUT+2)), B_COEFF(N_OUT + 2), &
              S_NODES(N_OUT+6)
  !REAL (KND), EXTERNAL :: DBVALU  
  !==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  WL_MAIN(:) = WL_SOL_HR(:)*(1 + SSP_M(2)) + SSP_M(1)
  WL_LOC(:) = WL_OUT(:)*(1 + SSP_C(2)) + SSP_C(1)

!  CALL CONVOLVE_GAUSS (N_SOL_HR, WL_MAIN, SOLAR_HR, N_OUT, WL_OUT, SOL_C) 
  CALL CONVOLVE_GAUSS (N_SOL_HR, WL_MAIN, SOLAR_HR, N_OUT, WL_LOC, SOL_C) 
  SOL_C = DLOG(SOL_C)
     
!  CALL DBINT4 (WL_OUT, SOL_C, N_OUT, 2, 2, 0.0D0, 0.0D0, 1, S_NODES, &
!       B_COEFF, N_COEFF, KK, WORK, *880) 
!  INBV = 1
!  DO II = L_P + 1, N_OUT - L_P
!     SOL_I(II - L_P) = DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0, WL_LOC(II), &
!          INBV, WORK(1:12), *880)
!  END DO
  CALL DBINT4 (WL_LOC, SOL_C, N_OUT, 2, 2, 0.0D0, 0.0D0, 1, S_NODES, &
       B_COEFF, N_COEFF, KK, WORK, *880) 
  INBV = 1
  DO II = L_P + 1, N_OUT - L_P
     Call DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0, WL_OUT(II), &
          INBV, WORK(1:12), SOL_I(II - L_P), *880)
  END DO
  
!  CALL CONVOLVE_GAUSS (N_SOL_HR, WL_MAIN, SOLAR_HR, N_OUT, WL_LOC, SOL_C) 
  CALL CONVOLVE_GAUSS (N_SOL_HR, WL_MAIN, SOLAR_HR, N_OUT, WL_OUT, SOL_C) 
  SOL_C = DLOG(SOL_C)

  CORR(:) = SOL_C(1 + L_P:N_OUT - L_P) - SOL_I(:)
 
  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
880 Return 1
END SUBROUTINE GET_CORRECTION

!*********************************************************************
SUBROUTINE GET_CORR_DERIV (SSP_M, SSP_C, N_OUT, WL_OUT, L_P, WF_M, WF_C, *)

  USE TYPE_VARS, ONLY: KND
  
  INTENT (IN)  SSP_M, SSP_C, N_OUT, WL_OUT, L_P
  INTENT (OUT) WF_M, WF_C
  
  INTEGER     N_OUT, L_P
  REAL (KND)  SSP_M(2), SSP_C(2), WL_OUT(N_OUT), WF_M(N_OUT - 2*L_P), &
              SSP_L(2), WF_C(N_OUT - 2*L_P)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_CORRECTION'
  INTEGER     INBV
  REAL (KND)  D_DIF, CORR_1(N_OUT - 2*L_P), CORR_2(N_OUT - 2*L_P)
  !==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  D_DIF = 1.0D-03

  SSP_L(1) = SSP_C(1) + D_DIF
  SSP_L(2) = SSP_C(2)
  CALL GET_CORRECTION (SSP_M, SSP_L, N_OUT, WL_OUT, L_P, CORR_1, *880)
  SSP_L(1) = SSP_C(1) - D_DIF
  CALL GET_CORRECTION (SSP_M, SSP_L, N_OUT, WL_OUT, L_P, CORR_2, *880)
  WF_C(:) = (CORR_1(:) - CORR_2(:))/(2.0D0*D_DIF)


  SSP_L(1) = SSP_M(1) + D_DIF
  SSP_L(2) = SSP_M(2)
  CALL GET_CORRECTION (SSP_L, SSP_C, N_OUT, WL_OUT, L_P, CORR_1, *880)
  SSP_L(1) = SSP_M(1) - D_DIF
  CALL GET_CORRECTION (SSP_L, SSP_C, N_OUT, WL_OUT, L_P, CORR_2, *880)
  WF_M(:) = (CORR_1(:) - CORR_2(:))/(2.0D0*D_DIF)

   CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
880 Return 1
END SUBROUTINE GET_CORR_DERIV

!*********************************************************************
SUBROUTINE GET_TILT (SSP_M, N_OUT, WL_OUT, RAD_MOD, RAD_MOD_S, L_P, TILT, *)

  USE IRRAD,     ONLY: SOLAR_HR, WL_SOL_HR, N_SOL_HR
  USE TYPE_VARS,   ONLY: KND
  
  INTENT (IN)  SSP_M, N_OUT, RAD_MOD, RAD_MOD_S, WL_OUT, L_P
  INTENT (OUT) TILT
  
  INTEGER     N_OUT, L_P
  REAL (KND)  SSP_M(2), WL_OUT(N_OUT), RAD_MOD(N_OUT), RAD_MOD_S(N_OUT - 2*L_P), &
              TILT(N_OUT - 2*L_P)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_TILT'
  INTEGER     INBV, II, KK, N_COEFF, NP, N_WL_SOL
  LOGICAL     FLG(N_SOL_HR)
  REAL (KND)  D_DIF, WL_LOC(N_OUT), RAD_HR_C(N_OUT - 2*L_P), S_NODES(N_OUT+6), &
              WORK(5*(N_OUT+2)), B_COEFF(N_OUT + 2), WL(N_OUT - 2*L_P), &
              RAD_LR(N_OUT - 2*L_P)
  REAL (KND), ALLOCATABLE :: WL_SOL(:), WL_MAIN(:), SOL(:), RAD_HR(:)
  !REAL (KND), EXTERNAL :: DBVALU  
  !==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  FLG(:) = .FALSE.
  WHERE(WL_SOL_HR(:) >= MINVAL(WL_OUT) .AND. &
       WL_SOL_HR(:) <= MAXVAL(WL_OUT)) FLG(:) = .TRUE.
  N_WL_SOL = COUNT(FLG)
  ALLOCATE(WL_SOL(N_WL_SOL), WL_MAIN(N_WL_SOL), SOL(N_WL_SOL), RAD_HR(N_WL_SOL))
  WL_SOL(:) = PACK(WL_SOL_HR(:), FLG(:))
  SOL(:) = PACK(SOLAR_HR(:), FLG(:))

  WL_MAIN(:) = WL_SOL(:)*(1 + SSP_M(2)) + SSP_M(1)
  NP = N_OUT - 2*L_P
  WL = WL_OUT(L_P + 1:N_OUT - L_P)

  CALL CONVOLVE_GAUSS (N_WL_SOL, WL_MAIN, SOL, NP, WL, RAD_LR) 
  RAD_LR(:)= DLOG(RAD_LR(:)) + RAD_MOD_S(:)
  
  CALL DBINT4 (WL_OUT, RAD_MOD, N_OUT, 2, 2, 0.0D0, 0.0D0, 1, S_NODES, &
       B_COEFF, N_COEFF, KK, WORK,*880) 
  INBV = 1
  DO II = 1, N_WL_SOL
     Call DBVALU (S_NODES, B_COEFF, N_COEFF, KK, 0, WL_SOL(II), &
          INBV, WORK(1:12), RAD_HR(II), *880)
     RAD_HR(II) = DEXP(RAD_HR(II))*SOL(II)
  END DO
  
  CALL CONVOLVE_GAUSS (N_WL_SOL, WL_MAIN, RAD_HR, NP, WL, RAD_HR_C) 
  RAD_HR_C(:) = DLOG(RAD_HR_C(:))

  TILT(:) = RAD_HR_C(:) - RAD_LR(:)

!  open(23, file = 'tilt_test.dat')
!  do ii = 1, NP
!     write(23,*) wl(ii), RAD_HR_C(ii), RAD_LR(ii), TILT(ii)
!  end do
!  close(23)
!  stop
  
  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
880 Return 1
END SUBROUTINE GET_TILT

!*********************************************************************
SUBROUTINE GET_TILT_DERIV (SSP_M, N_OUT, WL_OUT, RAD_MOD, L_P, WF_M, *)

  USE TYPE_VARS,   ONLY: KND
  
  INTENT (IN)  SSP_M, N_OUT, WL_OUT, RAD_MOD, L_P
  INTENT (OUT) WF_M
  
  INTEGER     N_OUT, L_P
  REAL (KND)  SSP_M(2), WL_OUT(N_OUT), RAD_MOD(N_OUT), WF_M(N_OUT - 2*L_P)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_TILT_DERIV'
  REAL (KND)  D_DIF, TILT_1(N_OUT - 2*L_P), TILT_2(N_OUT - 2*L_P), &
              RAD_MOD_S(N_OUT - 2*L_P), F_SS(N_OUT), SSP_L(2)
  !==================================================================

  CALL ST_TRACK (R_NAME, 2, 'I')

  D_DIF = 1.0D-03 ; SSP_L(2) = SSP_M(2)

  SSP_L(1) = SSP_M(1) + D_DIF
  F_SS(:) = SSP_L(1) + SSP_L(2)*WL_OUT(:)
  CALL SHIFT_2(WL_OUT, RAD_MOD, N_OUT, F_SS, RAD_MOD_S, L_P, 1.0D0,*880)
  CALL GET_TILT (SSP_L, N_OUT, WL_OUT, RAD_MOD, RAD_MOD_S, L_P, TILT_1,*880) 
  
  SSP_L(1) = SSP_M(1) - D_DIF
  F_SS(:) = SSP_L(1) + SSP_L(2)*WL_OUT(:)
  CALL SHIFT_2(WL_OUT, RAD_MOD, N_OUT, F_SS, RAD_MOD_S, L_P, 1.0D0,*880)
  CALL GET_TILT (SSP_L, N_OUT, WL_OUT, RAD_MOD, RAD_MOD_S, L_P, TILT_2,*880) 
  
  WF_M(:) = (TILT_1(:) - TILT_2(:))/(2.0D0*D_DIF)
 
  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
880 Return 1
END SUBROUTINE GET_TILT_DERIV

!*********************************************************************
SUBROUTINE GET_DERIV(SSP_P, N_OUT, WL_OUT, PAR_ARR, L_P, WF_P, *)

  USE TYPE_VARS,   ONLY: KND
  
  INTENT (IN)  SSP_P, N_OUT, WL_OUT, PAR_ARR, L_P
  INTENT (OUT) WF_P
  
  INTEGER     N_OUT, L_P
  REAL (KND)  SSP_P(2), WL_OUT(N_OUT), PAR_ARR(N_OUT), WF_P(N_OUT - 2*L_P)
  !... Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'GET_DERIV'
  REAL (KND)  D_DIF, RES_1(N_OUT - 2*L_P), RES_2(N_OUT - 2*L_P), &
              F_SS(N_OUT), SSP_L(2)
  !==================================================================
  CALL ST_TRACK (R_NAME, 2, 'I')

  D_DIF = 1.0D-03 ; SSP_L(2) = SSP_P(2)

  SSP_L(1) = SSP_P(1) + D_DIF
  F_SS(:) = SSP_L(1) + SSP_L(2)*WL_OUT(:)
  CALL SHIFT_2(WL_OUT, PAR_ARR, N_OUT, F_SS, RES_1, L_P, 1.0D0,*880)

  SSP_L(1) = SSP_P(1) - D_DIF
  F_SS(:) = SSP_L(1) + SSP_L(2)*WL_OUT(:)
  CALL SHIFT_2(WL_OUT, PAR_ARR, N_OUT, F_SS, RES_2, L_P, 1.0D0,*880)
  
  WF_P(:) = (RES_1(:) - RES_2(:))/(2.0D0*D_DIF)
 
  CALL ST_TRACK (R_NAME, 2, 'O')
  RETURN
880 Return 1
END SUBROUTINE GET_DERIV
!*********************************************************************
Subroutine GeomProgres (NN, SVal, EVal, PR, GRD, IncrDens, Reverse)
  Use Types, Only: KND
  Integer  NN, J
  Logical  Reverse, IncrDens
  Real (KND) PR, SVal, EVal, GRD(NN)
  
  DO J = 1, NN
     GRD(J) = PR**(J - 1)
  End Do
  GRD(:) = GRD(:)/(GRD(NN) - 1)*(EVal - SVal)
  GRD(:) = GRD(:)- GRD(1)
  If(IncrDens) Then
     DO J = 1, NN
        GRD(J) = SVal + GRD(J)
     END DO
  Else
     DO J = 1, NN
        GRD(J) = EVal - GRD(J)
     END DO
  End If
  If (Reverse) GRD(1:NN) = GRD(NN:1:-1)
  Return
End Subroutine GeomProgres
!*********************************************************************
