!********************************************************************C
!
!       SCIATRAN - Version 4.7 - Last modification: 02 Jan 2024
!
!       Copyright (C) 1995-2014, Institute of Remote Sensing,
!                                University of Bremen, Germany
!
!********************************************************************C
SUBROUTINE ST_UWT_COEFF(JPOS, POS, LOW, UDB_A, UDB_W, GSL_A, &
     GSL_W, DEP_W, SVR, *)
!     ... Call in "dom_spherical.f"

  USE BRDF,      ONLY: WL0, Alf0, Nwl_BRF
  USE LOG_PARS,  ONLY: LUNIT, LOG_LEVEL, N_ESO 
  USE GT_POLYN,  ONLY: XM, AXM, XM_W, AXM_W
  USE HYDROSOLS, ONLY: SEL_VSC
  USE INTERN_DIMS,ONLY: Nvs
  USE SLAB_2L,   ONLY: Ome0, Taus, TES_2L_SLAB
  USE ST_VARS,   ONLY: PIE, NN_A => NSTRMS
  USE TYPES,     ONLY: KND, LOW_TYPE, UDB_TYPE, GSL_type, VRS_TYPE
  USE VRS,       ONLY: Add_TRS, WLG_Chl, Sig_Chl, Qe_Chl, A_FChl, &
                       WL_QEred, WL_QEblu, Nh, WLH, Apeh
  USE WATER,     ONLY: NSTR_T, ABS_COEFF_UW, RTAU_W, Na_W, AAC_P, &
                       SAL_UW, Ts => TEMP_UW, CHL_UW, UIT,&
                       VSF_pure_model, A_RTmat, NN_W => NSTR_W, &
                       DO_RTAU_W, NL_UW => NLAYERS_UW, ZZZ_UW, &
                       DZ_UW, Et_wa, Et_aw, INCL_INEL, Flu_CDOM,&
                       FH_REF, Fu_par, Hu_par, Fua_c, Hua_c, &
                       Nwl_QE, WL_QE, Abs_flu, N_FC, Nwl_QEx, &
                       WL_QEx, F_QEx, Flu_con, Absw_Chl, Absw_pw, &
                       CDOM_Chl, CDOM_ACR, Absw_ful, Absw_hum, &
                       Absw_other, PSIT, PSIS, TS_CORR
  USE WATER_ICE, ONLY: IWS, IWT, LoIW, LA_N_IW, Na_ice
  USE WF_NUM,    ONLY: NUM_ORAY, NUM_Chl, PERTURB
! ------------------------------------------------------------------

  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'ST_UWT_COEFF'
  INTEGER,    INTENT(IN) :: JPOS
  REAL (KND), INTENT(IN) ::  POS
  TYPE (LOW_type) :: LOW          ! Optical parameters of water
  TYPE (UDB_TYPE) :: UDB_A, UDB_W ! Boundary conditions
  TYPE (GSL_type) :: GSL_A, GSL_W ! Angles 
  TYPE (VRS_type) :: SVR          ! Vibrational Raman scattering
!     ... Local parameters
  INTEGER   I, I1, I2, Ii, IiW, K, K1, K2, KK, J, J1, WA1, WA2, Dim_a, Dim_s, N_col
  INTEGER, PARAMETER :: EUNIT=77
  CHARACTER (LEN=6) C_POS
  LOGICAL   ERROR, Add
  REAL (KND) Ndl, NdlT, DP, UP, TAU, DEP_W, Tau_abs, Tau_sca, Tau_ray
  REAL (KND), DIMENSION (NL_UW):: SF, B_T, Nref, NdT, B_90, aw_corr
  REAL (KND), ALLOCATABLE :: H1(:), Add_abs(:,:,:), Out_Abs(:,:), &
       Out_sca(:,:), Out_ext(:,:)
  REAL (KND), PARAMETER ::  Ts0 = 20.d0, Tk = 293.15d0, Kb = 1.38054d0, &
       Sal0 = 37.d0
!  ==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')      

  SELECT CASE( TRIM(A_RTmat) )
  CASE('NT_ICF')
!    ... Effective transmission coefficient (water -> air)
!        and (air -> water) for the nadir direction only
!        (see "trans_refl_matr.f")
!        Ewa_G and Eaw_G calculated here are used in
!        "ASOL_ADJ_COA" in "st_dom_tau.f"
!         WRITE(*,*) '   st_uwt_coeff Nwl_BRF = ', Nwl_BRF
     IF( Nwl_BRF == 1 ) THEN
        UDB_W%Ewa_G(NN_W) = Et_wa
        UDB_A%Eaw_G(NN_A) = Et_aw
     ELSE
!        WRITE(*,*) '   SHAPE(AXM_W) ', SHAPE(AXM_W)
!        WRITE(*,*) '   SHAPE(UDB_W%Rw_GG) ', SHAPE(UDB_W%Rw_GG)
!        J1 = Nvs*NN_A
        J1 = Nvs*NN_W
        UDB_W%Ewa_G(NN_W) = 1.d0 - 2.d0*Pie/XM_W(NN_W)  &
             *SUM( UDB_W%Rw_GG(1:J1:Nvs,Nvs*(NN_W-1)+1,1)*AXM_W(:) )
        J1 = Nvs*NN_A
        UDB_A%Eaw_G(NN_A) = 1.d0 - 2.d0*Pie/XM(NN_A) &
             *SUM( UDB_A%Ra_GG(1:J1:Nvs,Nvs*(NN_A-1)+1,1)*AXM(:) )
     ENDIF
  CASE('FT')
!        ... Effective transmission coefficient (water -> air)
     DO K = 1,NN_W
        UDB_W%Ewa_G(K) = 1.d0 - 2.d0*Pie/GSL_W%XM(K) &
             *SUM( UDB_W%Rw_GG(1:NN_W,K,1)*GSL_W%AXM(:) )
     END DO
     UDB_W%Ewa_G(1:NSTR_T) = 0.d0
!    ... Effective transmission coefficient (air -> water)
!        Ewa_G and Eaw_G calculated here are used in in "iterate_coa.f"
     DO K = 1,NN_A !! Scalar case !!
        UDB_A%Eaw_G(K) = 1.d0 - 2.d0*Pie/GSL_A%XM(K) &
             *SUM( UDB_A%Ra_GG(1:NN_A,K,1)*GSL_A%AXM(:) )
     END DO
  END SELECT
      
! Pure water scattering coefficient as a function of depth 
!    in an ocean or ice layer starting from ocean surface
! ===========================================================
  LOW%SCR_W(:) = 0.d0
  DO IiW = 2, 1, -1
     IF( .NOT.LoIW(IiW) ) CYCLE

     I1 = IWS(IiW)%IW_TOP(1)  ;  I2 = IWS(IiW)%IW_BOT(1)
     IF( IWS(IiW)%DO_RTAU_W ) THEN
        Sf(:) = 1.d0
     ELSE
        Sf(:) = 1.d0 + 0.3d0*SAL_UW(:)/Sal0
     ENDIF

     SELECT CASE( TRIM(IWS(IiW)%VSF_pure_model) )
     CASE('Morel')
        LOW%SCR_W(I1:I2) = 3.5d-03*(450.d0/POS)**4.32*Sf(I1:I2)
     CASE('Shifrin')
        LOW%SCR_W(I1:I2) = 1.49d-03*(546.d0/POS)**4.17*Sf(I1:I2)
     CASE('Kopelevich')
        LOW%SCR_W(I1:I2) = 1.7d-03*(550.d0/POS)**4.3
     CASE('Buiteveld')
        B_T(I1:I2) = (5.062271d0 - 0.03179d0*Ts(I1:I2)  &
             + 0.000407d0*Ts(I1:I2)*Ts(I1:I2))         ! *1.d-10
        Nref(I1:I2) = 1.3247d0 + 3.3d03/POS**2 - 3.2d07/POS**4  &
             - 2.5d-06*Ts(I1:I2)*Ts(I1:I2)
        Ndl    = (-0.000156d0*POS + 1.5989d0)           ! *1.d-10
        NdT(I1:I2) = ( 1.61857d0 - 0.005785*Ts(I1:I2) ) ! *1.d-10
        NdlT   = ( 1.61857d0 - 0.005785*Ts0 )           ! *1.d-10
        B_90(I1:I2) = 2.d0*Pie**2/POS**4/B_T(I1:I2)*Kb*Tk*Nref(I1:I2)**2*1.d03  &
             *Ndl*NdT(I1:I2)/NdlT*(6.d0 + 6.d0*Dep_W)/(6.d0 - 7.d0*Dep_W) &
             *Sf(I1:I2)
        LOW%SCR_W(I1:I2) = 4.d0*Pie*2.d0/3.d0*(2.d0 + Dep_W)/(1.d0 + Dep_W) &
             *B_90(I1:I2)  ! Scattering coeff [m-1]
     CASE('off')
        LOW%SCR_W(I1:I2) = 0.d0
     END SELECT
  ENDDO  ! over water and ice (IiW)

! ... Set scattering coefficient in water to zero within ice layers:
  IF( LA_N_IW > 1 ) THEN
     DO I1 = 2, LA_N_IW
        LOW%SCR_W(IWS(1)%IW_TOP(I1):IWS(1)%IW_BOT(I1)) = 0.d0
     ENDDO
  ENDIF

!  LOW%SCR_W(IWS(1)%IW_BOT(1)+1:NL_UW) = 0.d0
!  LOW%SCR_W(IWS(1)%IW_BOT(1)+1:NL_UW) = LOW%SCR_W(IWS(1)%IW_BOT(1)+1:NL_UW)/10.d0

  IF( LOG_LEVEL(1) >= N_ESO ) THEN
     WRITE(LUNIT,*) '   Rayleigh scattering coefficient in water [m^-1]'
     DO I = 1, NL_UW
        WRITE(LUNIT,'(2x,F9.4,2x,F8.4)') ZZZ_UW(I), LOW%SCR_W(I)
     ENDDO
     WRITE(LUNIT,*) '   -----------------------------------------------'
  ENDIF

  IiW = 1
! ... Scattering and extinction coefficients of inclusions in ice layers
  DO I = 1, IWS(1)%N_HPC
     CALL LIN_SEARCH &
          (POS, DP, WA1, IWS(1)%N_HS_WL, IWS(1)%HS_WL, ERROR)
     If (ERROR) GOTO 112
     WA2 = WA1 + 1 ; UP = 1.d0 - DP
     LOW%HSCA(:,I,IiW) = IWS(1)%HS_COEFF(1,1:NL_UW,WA2,I)*DP &
          + IWS(1)%HS_COEFF(1,1:NL_UW,WA1,I)*UP
     LOW%HEXT(:,I,IiW) = IWS(1)%HS_COEFF(2,1:NL_UW,WA2,I)*DP &
          + IWS(1)%HS_COEFF(2,1:NL_UW,WA1,I)*UP
  ENDDO  ! over inclusions in ice (I)
  IF( IWT ) IiW = IiW + 1

! ... Scattering and extinction coefficients of hydrosol in water
!     Hydrosol scatt. and ext. coefficients HS_COEFF were set to zero  
!     within ice layer in the subroutine "man_hsol_setup.f90"
  DO I = 1, IWS(2)%N_HPC
     SELECT CASE( SEL_VSC(I) )
     CASE('Kos')
        LOW%HSCA(:,I,IiW) = IWS(2)%HS_COEFF(1,1:NL_UW,1,I) &
             *(550.d0/POS)**1.7
        LOW%HEXT(:,I,IiW) = LOW%HSCA(:,I,IiW)
     CASE('Kol')
        LOW%HSCA(:,I,IiW) = IWS(2)%HS_COEFF(1,1:NL_UW,1,I) &
             *(550.d0/POS)**0.3
        LOW%HEXT(:,I,IiW) = LOW%HSCA(:,I,IiW)
     CASE('Use')
        CALL LIN_SEARCH  &
             (POS, DP, WA1, IWS(2)%N_HS_WL, IWS(2)%HS_WL, ERROR)
        If (ERROR) GOTO 112
        WA2 = WA1 + 1 ; UP = 1.D0 - DP
        LOW%HSCA(:,I,IiW) = IWS(2)%HS_COEFF(1,1:NL_UW,WA2,I)*DP &
             + IWS(2)%HS_COEFF(1,1:NL_UW,WA1,I)*UP
        LOW%HEXT(:,I,IiW) = IWS(2)%HS_COEFF(2,1:NL_UW,WA2,I)*DP &
             + IWS(2)%HS_COEFF(2,1:NL_UW,WA1,I)*UP
!        WRITE(*,*)  '  I, IiW, LOW%HEXT ', I, IiW, LOW%HEXT(:,I,IiW)
     END SELECT

     IF( LOG_LEVEL(1)>=N_ESO ) WRITE(LUNIT,*) '  Tau_hsol = ', &
          I, SEL_VSC(I), SUM(LOW%HEXT(:,I,IiW)*DZ_UW)/2.d0
  ENDDO  ! over hydrosol components (I)
  
  IF( DO_RTAU_W ) THEN
     ALLOCATE( H1(NL_UW - 1) )
     IF( LOG_LEVEL(1) >= N_ESO ) THEN
        WRITE(LUNIT,*) '       Normalize Rayleigh scr of water'
        WRITE(LUNIT,*) '       Shape(LOW%SCR_W) :',Shape(LOW%SCR_W)
        WRITE(LUNIT,*) '       NL_UW = ',NL_UW
     END IF
     H1(:) = 0.5d0*(LOW%SCR_W(1:NL_UW-1) + LOW%SCR_W(2:NL_UW))
     Tau = 0.D0 
     DO  J1 = 1, NL_UW - 1
        Tau = Tau + H1(J1)*( ZZZ_UW(J1 + 1) - ZZZ_UW(J1) )
     ENDDO
     IF( LOG_LEVEL(1) >= N_ESO )  &
          WRITE(LUNIT,*) '    Tau Water Ray:  ',Tau
!          WRITE(LUNIT,*) '           RTAU_W = ',RTAU_W
     LOW%SCR_W(:) = LOW%SCR_W(:)*RTAU_W/Tau
  ENDIF

  IF( TES_2L_SLAB ) THEN
     ALLOCATE( H1(NL_UW - 1) )
     H1(:) = 0.5d0*(LOW%SCR_W(1:NL_UW - 1) + LOW%SCR_W(2:NL_UW))
     H1(:) = H1(:)/Ome0(2)
     Tau = 0.0D0 
     DO  J1 = 1, NL_UW - 1
        Tau = Tau + H1(J1)*( ZZZ_UW(J1 + 1) - ZZZ_UW(J1) )
     ENDDO
     LOW%SCR_W(:) = LOW%SCR_W(:)*Taus(2)/Tau
  ENDIF

  IF( TS_CORR ) THEN
     aw_corr(:) = SAL_UW(:)*PSIS(JPOS) + (Ts(:) - Ts0)*PSIT(JPOS)
!     WRITE(*,*) PSIS(JPOS), PSIT(JPOS)
  ELSE
     aw_corr(:) = 0.d0
  ENDIF
  
  LOW%ABS_UW(:,:) = 0.d0
  IF( IWS(2)%Absw_pw ) THEN  ! Absorption by pure water
!     WRITE(*,*) '   Absorption by  pure wate is included'
     LOW%ABS_UW(:,1) = IWS(2)%ABS_COEFF_UW(1,JPOS) + aw_corr(:)
  ENDIF
  IF( IWS(1)%Absw_pw ) THEN  ! Absorption by pure ice
!     WRITE(*,*) '   Absorption by  pure ice is included'
     DO Ii = 1, LA_N_IW
!        WRITE(*,*) '     within layer '
        I1 = IWS(1)%IW_TOP(Ii)  ;  I2 = IWS(1)%IW_BOT(Ii)
        LOW%ABS_UW(I1:I2,1) = IWS(1)%ABS_COEFF_UW(1,JPOS)
     ENDDO
  ENDIF

!  DO I = 1, NL_UW
!     WRITE(*,'(2x,F9.4,*(2x,E12.4))') &
!          ZZZ_UW(I), LOW%ABS_UW(I,1)!, LOW%ABS_UW(I,1) - aw_corr(I)
!  ENDDO

!         TEST perturbation of water absorption instead of Chl
  IF( NUM_Chl ) LOW%ABS_UW(:,1) = LOW%ABS_UW(:,1)*PERTURB(:)

  Add = (Add_TRS(1).AND..NOT.SVR%ACT_WL(JPOS)).OR.  &
       (Add_TRS(2).AND.SVR%ACT_WL(JPOS))
! ... Include in water absorption coefficient an attenuation caused by the 
!     total vibrational Raman scattering at actual wavelengths only
  IF( Add ) THEN
     WRITE(*,*) '!!! Total VRS coef. is added to extinction !!!'
     LOW%ABS_UW(:,1) = LOW%ABS_UW(:,1) + SVR%Tot_Vrs(JPOS)
  ENDIF

! ... Add chlorophyll absorption coefficient 
  DO I = 2, 1, -1 !  Start from water absorption
     IF( (.NOT.LoIW(I)).OR.(.NOT.IWS(I)%Absw_chl)) CYCLE
     I1 = IWS(I)%IW_TOP(1)  ;  I2 = IWS(I)%IW_BOT(1)
     IF(IWS(I)%Absw_Chl.AND.(.NOT.IWS(I)%Absw_other)) LOW%ABS_UW(I1:I2,2) &
          = 0.06d0*CHL_UW(I1:I2)**0.65d0*IWS(I)%ABS_COEFF_UW(2,JPOS)
  ENDDO ! over water and ice (I)
  
!  ... Add fluorescence absorption part of chlorophyll-a
  IF( INCL_INEL(2).AND.SVR%ACT_WL(JPOS) ) THEN
     IF( POS>=WL_QEred(1).AND.POS<=WL_QEred(2) ) THEN 
        CALL LIN_INT(Nh, WLh, Apeh, 1, POS, Dp, 'C')
        LOW%ABS_UW(:,2) =  &
             LOW%ABS_UW(:,2)*(1.d0 + Qe_Chl*SVR%WL_EXT(JPOS)*Dp)
     ENDIF
  ENDIF

!     Dissolved organic matter or yellow matter absorption coefficient
!     ================================================================
!  ... Add CDOM absorption according to Morel and Moritorena (in water only)
  IF( IWS(2)%CDOM_Chl ) THEN
     LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2) + 0.2d0* &
       (Alf0 + 0.06d0*CHL_UW(:)**0.65d0)*DEXP(-0.014d0*(POS-WL0))
!c         !!! This specific case is used for comparison with HL !!! 
!          (0.06d0*CHL_UW(:)**0.65d0)*DEXP(-0.014d0*(POS-WL0))
  ENDIF
  
!     ... Add absorption by fulvic acid
  IF( Absw_ful ) THEN
!     ... Fua_c(:) is concentrations of fulvic acid
     DP = FH_REF - POS
     LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2)  &
          + Fua_c(:)*Fu_par(1)*DEXP(Fu_par(2)*DP)
  ENDIF

!     ... Add absorption by humic acid
  IF( Absw_hum ) THEN
!    ... Hua_c(:) is concentrations of humic acid
     DP = FH_REF - POS
     LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2) &
          + Hua_c(:)*Hu_par(1)*DEXP(Hu_par(2)*DP)
  ENDIF

! ... Add CDOM absorption according ACR approach
  IF( CDOM_ACR ) THEN
!    ... Do-loop over fluorescent components
     DO J = 1,N_FC
!       ... Interpolation of the excitation factor on the current 
!           wavelength POS
        CALL LIN_INT(Nwl_QEx, WL_QEx, F_QEx(J,:), 1, POS, DP, 'Z')
!           ... Add absorption coefficient of fluorescent component
        LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2) + DP*Flu_con(J,:)
     ENDDO  ! over fluorescent components (J)
  ENDIF

!     ... Add fluorescence absorption part of CDOM
!      IF( INCL_INEL(3) ) THEN
  IF( INCL_INEL(3).AND.SVR%ACT_WL(JPOS) ) THEN
     IF( POS>=WL_QEblu(1).AND.POS<=WL_QEblu(2) ) THEN
        SELECT CASE(Flu_CDOM)
        CASE('FHa')
           CALL LIN_INT(Nwl_QE, WL_QE, Abs_flu(1,:), 1,POS,DP, 'Z')
           CALL LIN_INT(Nwl_QE, WL_QE, Abs_flu(2,:), 1,POS,UP, 'Z')
!                  TEST
           LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2)+Fua_c(:)*DP+Hua_c(:)*UP
        CASE('ACR')  
           DO J = 1,N_FC
              CALL LIN_INT(Nwl_QEx, WL_QEx, Abs_flu(J,:), &
                   1, POS, DP,'Z')
              LOW%ABS_UW(:,2) = LOW%ABS_UW(:,2) + DP*Flu_con(J,:)
           ENDDO
        END SELECT
     ENDIF
  ENDIF

  IF( NUM_ORAY ) LOW%SCR_W(:) = LOW%SCR_W(:) *PERTURB(:)

! ... Total extinction coefficient without additional absorber
  DO I = 1, NL_UW
     LOW%AL_W(I) = LOW%SCR_W(I) + SUM( LOW%ABS_UW(I,1:2) )
  ENDDO

  II = SUM( IWS(1)%Na_W(:) )
  J1 = MAX( II,IWS(2)%Na_W(1) )
  IF( J1 > 0 ) THEN
     ALLOCATE( Add_abs(NL_UW,J1,2) )
     Add_abs = 0.d0
!   ... Absorption coefficients of additional absorbers within ice
     KK = 0
     DO Ii = 1, LA_N_IW
        I1 = IWS(1)%IW_TOP(Ii)  ;  I2 = IWS(1)%IW_BOT(Ii)
        DO I = 1, IWS(1)%Na_W(Ii)
           KK = KK + 1
           Add_abs(I1:I2,KK,1) = IWS(1)%AAC_P(I1:I2,KK) &
                *IWS(1)%ABS_COEFF_UW(2+KK,JPOS)/IWS(1)%C_NF(I,Ii)
           LOW%AL_W(I1:I2) = LOW%AL_W(I1:I2) + Add_abs(I1:I2,KK,1)
        ENDDO  ! over additional absorber in water (I)
     ENDDO  ! over ice layers (Ii)
     
!   ... Absorption coefficients of additional absorbers within water
     K1 = IWS(2)%IW_TOP(1)  ;  K2 = IWS(2)%IW_BOT(1)
     DO I = 1, IWS(2)%Na_W(1)
        ! Concentration of additional absorber in water, IWS(2)%AAC_P(:,:) 
        !   was set to zero within all ice layers in "gt_ptconc_setup.f"
        FORALL( J = K1:K2 ) Add_abs(J,I,2)  &
             = IWS(2)%AAC_P(J,I)*IWS(2)%ABS_COEFF_UW(2+I,JPOS)
        LOW%AL_W(:) = LOW%AL_W(:) + Add_abs(:,I,2)
     ENDDO  ! over additional absorber in water (I)
     
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     IF( LOG_LEVEL(1) >= N_ESO ) THEN
        write(*,*) '    Check additional absorber (except chlorophyll and CDOM): '
        IF( ANY(IWS(1)%Na_W(:)>0) ) THEN
           write(*,*) '    In ice layers N_abs : ', IWS(1)%Na_W(:)
           DO I = 1, NL_UW
              WRITE(*,'(2x,F9.4,*(2x,E12.4))') &
                   ZZZ_UW(I), (Add_abs(I,K,1), K = 1,Na_ice )
           ENDDO
        ENDIF  ! absorber in ice
        IF( IWS(2)%Na_W(1) > 0 ) THEN
           write(*,*) '    In water layers N_abs = ', IWS(2)%Na_W(1)
           DO I = 1, NL_UW
              WRITE(*,'(2x,F9.4,*(2x,E12.4))') &
                   ZZZ_UW(I), (Add_abs(I,K,2), K = 1,IWS(2)%Na_W(1) )
           ENDDO
        ENDIF  ! absorber in water
     ENDIF  ! LOG_LEVEL
! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  ENDIF  ! additions absorber in ice or water
  
! ... Add extinction of inclusions in an ice layer
  IiW = 1
  IF( IWS(1)%N_HPC > 0 ) THEN
     DO I = 1, NL_UW
        LOW%AL_W(I) = LOW%AL_W(I) + SUM( LOW%HEXT(I,:,IiW) )
     ENDDO
  ENDIF
  IF( IWT )  IiW = IiW + 1
! ... Add extinction of inclusions in water
  IF( IWS(2)%N_HPC > 0 ) THEN
     DO I = 1, NL_UW
        LOW%AL_W(I) = LOW%AL_W(I) + SUM( LOW%HEXT(I,:,IiW) )
     ENDDO
  ENDIF

!!  IF( JPOS == 1.OR. JPOS==14 ) THEN
!     Tau_ray = SUM( LOW%SCR_W(:)*DZ_UW )/2.d0
!     Tau_sca = SUM( (LOW%HEXT(:,1,1) + LOW%HEXT(:,2,1) )*DZ_UW)/2.d0
!     Tau_abs = SUM( LOW%ABS_UW(:,1)*DZ_UW )/2.d0
!     Tau = SUM( LOW%AL_W(:)*DZ_UW )/2.d0
!!     WRITE( *,* ) '  WL, Tau_water: '
!!     WRITE(*,'(1x,F8.3,*(1x,e12.4))') POS, Tau_abs, Tau_ray, Tau_sca, &
!!          Tau, (Tau_ray + Tau_sca)/Tau
!!  ENDIF

!  C_POS = 'ASIS'
!  IF ( JPOS > 1 ) C_POS = 'APPEND'
!  OPEN(EUNIT,file='./DATA_OUT/Tau_wi.dat',POSITION=TRIM(C_POS))
!     WRITE(EUNIT,'(1x,F8.3,*(1x,e12.4$))') POS, Tau_abs, Tau_ray, Tau_sca, &
!          Tau, (Tau_ray + Tau_sca)/Tau
!  WRITE(EUNIT,100) 
!  CLOSE( EUNIT )
  
! *********** Write out settings of optical parameters *************
  IF( LOG_LEVEL(1) >= N_ESO ) THEN
     Dim_a = 2 + SUM( IWS(1)%Na_w(:) ) + IWS(2)%Na_w(1)
     ALLOCATE( Out_Abs(NL_UW,Dim_a ) )
     Out_Abs(:,1:2) = LOW%ABS_UW(:,1:2)
     DO I = 1,Na_ice
        Out_Abs(:,2+I) = Add_abs(:,I,1)
     ENDDO
     K = 1
     DO I = Na_ice+1, Na_ice+IWS(2)%Na_w(1) 
        Out_Abs(:,2+I) = Add_abs(:,K,2)  ;  K = K + 1
     ENDDO
     IiW = 1  ;  Dim_s = SUM( IWS(:)%N_HPC )
     ALLOCATE( Out_sca(NL_UW, MAX(1,Dim_s)), Out_ext(NL_UW, MAX(1,Dim_s)))
     DO I = 1, IWS(1)%N_HPC
        Out_ext(:,I) = LOW%HEXT(:,I,1)
        Out_sca(:,I) = LOW%HSCA(:,I,1)
     ENDDO
     IF( IWT )  IiW = IiW + 1
     K = 1
     DO I = IWS(1)%N_HPC+1, IWS(1)%N_HPC+IWS(2)%N_HPC
        Out_ext(:,I) = LOW%HEXT(:,K,IiW)
        Out_sca(:,I) = LOW%HSCA(:,K,IiW)
        K = K + 1
     ENDDO
     WRITE(LUNIT,'(15x,A,1x,A)') ' Ice', 'Water'
     WRITE(LUNIT,'(A,*(2x,I2))') '  absorber:   ', SUM( IWS(1)%Na_w(:) ), IWS(2)%Na_w(1)
     WRITE(LUNIT,'(A,*(2x,I2))') '  inclusions: ', IWS(:)%N_HPC
     WRITE(LUNIT,*)  &
          '  ********** Abs. coeff. in water and ice [m-1] **********'
     WRITE(LUNIT,'(10x,A)') 'First column  - depth in [m] '
     WRITE(LUNIT,'(10x,A)') 'Second column - absorption by pure water/ice '
     WRITE(LUNIT,'(10x,A)') 'Third column  - absorption by chlorophyll in water'
     N_col = 4
     IF( ANY(IWS(1)%Na_w(:)>0) ) THEN
        WRITE(LUNIT,'(10x,I2,A,I2,A)') N_col,' - ', N_col+SUM(IWS(1)%Na_w(:))-1,&
             ' columns - additional absorber in ice'
        N_col = N_col + SUM(IWS(1)%Na_w(:))
     ENDIF
     IF( IWS(2)%Na_w(1) > 0 ) THEN
        WRITE(LUNIT,'(10x,I2,A,I2,A)') N_col,' - ',N_col+IWS(2)%Na_w(1)-1,&
             ' columns - additional absorber in water'
        N_col = N_col+IWS(2)%Na_w(1)
     ENDIF
     IF( IWS(1)%N_HPC > 0 ) THEN
        WRITE(LUNIT,'(10x,I2,A,I2,A)') N_col,' - ',N_col+IWS(1)%N_HPC-1,&
             ' columns - absorption by inclusions in ice'
        N_col = N_col+IWS(1)%N_HPC
     ENDIF
     IF( IWS(2)%N_HPC > 0 ) THEN
        WRITE(LUNIT,'(10x,I2,A,I2,A)') N_col,' - ',N_col+IWS(2)%N_HPC-1,&
             ' columns - absorption by inclusions in water'
        N_col = N_col+IWS(2)%N_HPC
     ENDIF
     DO I = 1, NL_UW
        WRITE(LUNIT,'(1x,F9.4,1x,*(1x,e12.4))')  ZZZ_UW(I), &
             (Out_Abs(I,K), K = 1,Dim_a),  &
             ( Out_ext(I,K) - Out_sca(I,K), K =1,Dim_s)
     ENDDO
     
     WRITE(LUNIT,*)  &
          '  ******************************************************'
     WRITE(LUNIT,*)  &
          '  ********** Scat coeff in water and ice [m-1] **********'
     WRITE(LUNIT,'(5x,A,7x,A,5x,A)') &
          'z[m]','Rayleigh','Inclusions --->'
!        -----------------------------------------------------------
     DO I = 1, NL_UW
        WRITE(LUNIT,'(2x,F9.4,1x,*(2x,e12.4))')  &
             ZZZ_UW(I), LOW%SCR_W(I), (Out_sca(I,K), K =1,Dim_s)
     ENDDO
     WRITE(LUNIT,*)  &
          '  ***********************************************'

     C_POS = 'ASIS'
     IF ( JPOS > 1 ) C_POS = 'APPEND'
     OPEN(EUNIT,file='./DATA_OUT/total_abs_coeff.dat',POSITION=TRIM(C_POS))
     WRITE(EUNIT,100) POS, (SUM(LOW%ABS_UW(i,:)),i=1,NL_UW)
     CLOSE( EUNIT )
100  FORMAT(2x,f9.4,1x,*(2x,e12.4))
  END IF  ! LOG_LEVEL(1) >= N_ESO 

! ******************************************************************

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

112 CALL WRITE_MESSAGE(R_NAME, 'ERROR',  &
         'Error in lin_search (hydrosol)')
  RETURN 1
!**********************************************************************
END SUBROUTINE ST_UWT_COEFF
!**********************************************************************

