C********************************************************************C
C
C       SCIATRAN - Version 4.5 - Last modification: 18 Nov 2014
C
C       Copyright (C) 1995-2014, Institute of Remote Sensing,
C                                University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE ST_VOD (JPOS, POS)

      USE ABSORBERS,   ONLY: N_GAS, N_ESFT_MOL
      USE LOG_PARS,    ONLY: LUNIT, EU, LOG_LEVEL, N_RSO
      USE GRIDS,       ONLY: NLAYERS
      USE ESFT,        ONLY: N_ESFT_COEFF, DO_ESFT_SPEC_MOL, DO_ESFT
      USE INTERN_DIMS, ONLY: NL_PC, NL_FINE_PC
      USE PHOTOCHEM,   ONLY: N_SOL_PC
      USE TYPES,       ONLY: KND, LOA_TYPE, PWF_TYPE, LOW_TYPE, PCC_TYPE
c     ------------------------------------------------------------------

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'ST_VOD'
      LOGICAL    ESFT_FLAG_LOC
      INTEGER    JJESFT, NESFT_IDX, JESFT, JPOS, NESFT
      INTEGER    E_IDX(2*N_ESFT_COEFF,N_ESFT_MOL)
c     --------------------------------------------
c     ... Local optical parameters of atmosphere |
      TYPE (LOA_type) :: LOA
c     ... Local optical parameters of water |
      TYPE (LOW_type) :: LOW
c     --------------------------------
c     ... General weighting function |
      TYPE (PWF_type) :: PWF
c     ... Photochemistry |
      TYPE (PCC_type) :: PCC    
c     ----------------------
      REAL (KND)  TG_CS(NLAYERS, N_GAS), TGA(NLAYERS)
      REAL (KND)  TG_CROSS_SEC(NLAYERS, N_GAS)
      REAL (KND)  DEPOL, POS, ESFT_WEIGHT
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      IF (LOG_LEVEL(1) >= N_RSO) WRITE (LUNIT, 
     $     '(2X,"*** Vertical optical depth calculation point ",I0, 
     $     " at", 1PG14.6, " nm")') JPOS, POS

C     Determine depolarisation ratio for actual wavelength
      CALL KING_ACTUAL(POS, DEPOL, *200)

C     Calculate extinction and scattering profiles
c     (Aerosols, clouds, and Rayleigh).

c     ... Allocate elements of the structure LOA
      ALLOCATE(LOA%AL(NLAYERS), LOA%SCR(NLAYERS),  LOA%EXA(NLAYERS), 
     $     LOA%SCA(NLAYERS),  LOA%SCC(NLAYERS), LOA%EXC(NLAYERS),  
     $     LOA%AS_A(NLAYERS), LOA%EXI(NLAYERS), 
     $     PCC%PCX_COEFF(NL_PC, N_SOL_PC, 4),
     $     PCC%PCY_COEFF(N_SOL_PC, NL_FINE_PC, 4))
      LOA%SCC = 0.0D0 ; LOA%EXC = 0.0D0 ; LOA%EXI = 0.0D0 
c    ------------------------------------------------------------

      CALL RAYLEIGH_SCATT ( JPOS, POS, DEPOL, LOA%SCR, *880 )

      CALL GT_SCATT (JPOS, POS, LOA, *880)

      IF(DO_ESFT) THEN
         ESFT_FLAG_LOC = ANY(DO_ESFT_SPEC_MOL(JPOS,:))
      ELSE
         ESFT_FLAG_LOC = .FALSE.
      END IF

      IF(ESFT_FLAG_LOC) THEN
C     Subroutines IDX_ESFT and WEIGHT_ESFT are in st_esft.f
         CALL IDX_ESFT(JPOS, E_IDX, NESFT, NESFT_IDX, *880)
      ELSE
         NESFT = 1
      ENDIF
      
c     ESFT loop
c     ---------
      TG_CS(:,:) = 0.0D0
      DO JJESFT = 1, NESFT
         
         IF(ESFT_FLAG_LOC) THEN
            CALL WEIGHT_ESFT(JPOS, JJESFT, JESFT, NESFT, NESFT_IDX,
     $           ESFT_WEIGHT)
         ELSE
            JESFT       = JJESFT
            ESFT_WEIGHT = 1.0D0
         ENDIF

         CALL ST_ABS_COEFF (JPOS, POS, JJESFT, E_IDX, TG_CROSS_SEC, LOA,
     $        PWF, PCC, TGA, *880)

         TG_CS(:,:) = TG_CS(:,:) + ESFT_WEIGHT*TG_CROSS_SEC(:,:)
            
      END DO                    ! ESFT coefficients

      CALL OPTICAL_DEPTH(JPOS, POS, LOA, LOW, TG_CS, *880)             

      DEALLOCATE(LOA%AL, LOA%SCR,  LOA%EXA, LOA%SCA,  LOA%SCC, LOA%EXC,
     $     LOA%AS_A, LOA%EXI, PCC%PCX_COEFF, PCC%PCY_COEFF)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME,'ERROR', 
     $     'Interpolation error: Depolarization factor')
      GOTO 881
 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
 881  WRITE(EU, *) ' ---> Error inside the wavelenght loop'
      WRITE(EU, *) ' ---> Calling sequence: gt_iface - rtm_main'

      WRITE(LUNIT, *)
      WRITE(LUNIT, *) ' >>>>>> !!! Abnormal termination !!! <<<<<<'
      WRITE(LUNIT, *) ' SCIATRAN stopped in st_vod: see errors.log'
      WRITE(LUNIT, *) ' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^'
      WRITE(LUNIT, *)
      STOP

C********************************************************************C
      END
C********************************************************************C

      SUBROUTINE OPTICAL_DEPTH (JPOS, POS, LOA, LOW, TG_CS, *) 

      USE ABS_COEFF_PHOTOCHEM, ONLY : PC_ABS_COEFF
      USE ABSORBERS,  ONLY: N_GAS, ST_GAS_IDX
      USE GRIDS,      ONLY: DZR, NLAYERS
      USE LOG_PARS,   ONLY: LUNIT, LOG_LEVEL, N_ESO, EU
      USE INFO,       ONLY: TG_INFO
      USE PHOTOCHEM,  ONLY: N_SOL_PC, CONC_PC, DO_PHOTOCHEM, 
     $                      PC_GAS_INDEX
      USE RTM_OUTPUT, ONLY: TG_VOD, OUT_VOD_ALL
      USE SPHER_PARS, ONLY: N_SOL_ANG
      USE ST_VARS,    ONLY: RTM_MODE
      USE TYPE_VARS,  ONLY: KND
      USE TYPES,      ONLY: LOA_TYPE, LOW_TYPE
      USE WATER,      ONLY: UWT, NLAYERS_UW, ZZZ_UW, DZ_UW
      USE WATER_ICE,  ONLY: IWS
c     ==================================================================

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'OPTICAL_DEPTH'

      INTEGER, INTENT(IN)    ::  JPOS
      REAL (KND), INTENT(IN) ::  POS
c     ----------------------------------------------------------------
c     ... Local optical parameters of atmosphere |
      TYPE (LOA_type) :: LOA
c     ... Local optical parameters of water |
      TYPE (LOW_type) :: LOW
c     ----------------------
      REAL (KND)   TG_CS(NLAYERS, N_GAS)
c     ... Local parameters
c     --------------------
      INTEGER      I_GAS, J, IDX_SOL, KK
      REAL (KND)   TG_TMP(NLAYERS,N_SOL_PC), TG_LOC(N_SOL_ANG,N_GAS)
      REAL (KND)   RAY_SC, AER_EX, CLD_EX, CLD_SC, RW_VOD, AW_VOD, S1,
     $             TG_AB, AER_SC, TAUZERO, HDZ, HSOL_VOD
c    -------------------------------------------------------------------
      
      CALL ST_TRACK (R_NAME, 1, 'I')

      I_GAS = 0
      DO KK = 1, UBOUND(TG_INFO,1)
         IF(.NOT.TG_INFO(KK)%FM_FLG) CYCLE
         I_GAS =  I_GAS + 1
         IF(DO_PHOTOCHEM(ST_GAS_IDX(I_GAS))) THEN

            FORALL(IDX_SOL = 1:N_SOL_PC) 
     $           TG_TMP(:,IDX_SOL) = TG_CS(1:NLAYERS,I_GAS)*
     $           CONC_PC(1:NLAYERS,PC_GAS_INDEX(I_GAS),IDX_SOL)

            CALL VERT_INTEGR_PC(TG_TMP(1:NLAYERS,1:N_SOL_PC),
     $           TG_LOC(1:N_SOL_ANG,I_GAS), *880)

         ELSE
            
            TG_TMP(:,1) = TG_CS(1:NLAYERS,I_GAS)*TG_INFO(KK)%ND(:)
            CALL VERT_INTEGR(TG_TMP(:,1), TG_LOC(:,I_GAS))
            
         END IF
      END DO

      RAY_SC = 0.D0 ; AER_SC = 0.D0 ; AER_EX = 0.D0 ; CLD_SC = 0.D0
      CLD_EX = 0.D0 ; 

C     ... Cumulative optical depths
      DO J = 1, NLAYERS - 1
         HDZ = 0.5D0*DZR(J)
         RAY_SC = RAY_SC + (LOA%SCR(J) + LOA%SCR(J + 1))*HDZ
         AER_EX = AER_EX + (LOA%EXA(J) + LOA%EXA(J + 1))*HDZ
         AER_SC = AER_SC + (LOA%SCA(J) + LOA%SCA(J + 1))*HDZ
         CLD_EX = CLD_EX + (LOA%EXC(J) + LOA%EXC(J + 1))*HDZ
         CLD_SC = CLD_SC + (LOA%SCC(J) + LOA%SCC(J + 1))*HDZ
      END DO
      TG_AB  = SUM(TG_LOC(1,1:N_GAS))
      TAUZERO = TG_AB + RAY_SC + AER_EX + CLD_EX

      IF( UWT ) THEN
         RW_VOD = 0.d0 ; S1 = 0.d0
         DO J = 1, NLAYERS_UW
            RW_VOD = RW_VOD + LOW%SCR_W(J)*DZ_UW(J)/2.d0
            S1     = S1     + LOW%AL_W(J)*DZ_UW(J)/2.d0
         ENDDO

         HSOL_VOD = 0.d0
         IF( IWS(1)%N_HPC > 0 ) THEN
            DO J = 1, NLAYERS_UW
               HSOL_VOD = HSOL_VOD + SUM(LOW%HEXT(J,:,1))*DZ_UW(J)/2.d0
            ENDDO
         ENDIF
         AW_VOD = DABS(S1 - RW_VOD - HSOL_VOD)
      ENDIF

      IF (LOG_LEVEL(1) >= N_ESO)  THEN
         WRITE(LUNIT, '(3x,A,1X,F9.2,1X,4(D11.4))') 
     $        '  *** OD (W T R A C):',
     $        POS, TG_AB, RAY_SC, AER_EX, CLD_EX 
         IF( UWT ) THEN
            WRITE(LUNIT, '(3x,A,14X,4(D11.4))') 
     $           '  *** OD in ocean     :', AW_VOD, RW_VOD, HSOL_VOD
         ENDIF
      ENDIF

      IF (TRIM(RTM_MODE) /= 'vod') THEN 
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      ENDIF

      TG_VOD(JPOS,1:N_SOL_ANG,1:N_GAS) = TG_LOC(:,:)

c     ... Store some results in new vertical optical depth arrays
      OUT_VOD_ALL(JPOS,1) = RAY_SC
      OUT_VOD_ALL(JPOS,2) = AER_SC
      OUT_VOD_ALL(JPOS,3) = AER_EX - AER_SC
      OUT_VOD_ALL(JPOS,4) = CLD_SC
      OUT_VOD_ALL(JPOS,5) = CLD_EX - CLD_SC
      OUT_VOD_ALL(JPOS,6) = TAUZERO

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN      

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
c *******************************************************************
      END
c *******************************************************************

      SUBROUTINE VERT_INTEGR_PC (INPUT_ARRAY, OUTPUT_ARRAY, *)

      USE ABS_COEFF_PHOTOCHEM, ONLY: PC_ABS_COEFF
      USE GRIDS,       ONLY: NLAYERS, Z_INV, NL_FINE, Z_FINE,  ALT_EXT, 
     $                       INTERP_IDX
      USE INTERN_DIMS, ONLY: NL_PC, NL_FINE_PC
      USE LOG_PARS,    ONLY: EU
      USE PHOTOCHEM,   ONLY: N_SOL_PC
      USE SPHER_PARS,  ONLY: N_LOS_OUT, ANG_OUT, N_SOL_ANG
      USE ST_VARS,     ONLY: EARTH_RADIUS, DO_REFRAC
      USE TYPE_VARS,   ONLY: KND
      USE TYPES,       ONLY: PCC_TYPE

      TYPE (PCC_type) :: PCC    

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'VERT_INTEGR_PC'

      INTEGER             STEP, II, IDX_SOL, I, J
      LOGICAL             REFR_FLAG(NL_FINE)
      REAL (KND)          S_1, INPUT_ARRAY(NLAYERS, N_SOL_PC)
      REAL (KND)          OUTPUT_ARRAY(N_SOL_ANG), TG_OD(NL_FINE)
      REAL (KND), DIMENSION (NL_FINE) :: COS_PSI, TH
      
      ALLOCATE( PCC%PCX_COEFF(NL_PC, N_SOL_PC, 4),
     $     PCC%PCY_COEFF(N_SOL_PC, NL_FINE_PC, 4))
      CALL GT_PC_INIT (INPUT_ARRAY, PCC)
      
      II = 0
      STEP = N_LOS_OUT/N_SOL_ANG
      DO IDX_SOL = 1, N_LOS_OUT, STEP
         II = II + 1
         COS_PSI(1:NL_FINE) = ANG_OUT(1,IDX_SOL) 
         TH(1:NL_FINE) = (EARTH_RADIUS + Z_FINE(1:NL_FINE))*
     $        DSQRT(1.0D0 - COS_PSI(1:NL_FINE)**2) - EARTH_RADIUS
         
         IF(DO_REFRAC) THEN
            CALL REFR_INTERFACE(NL_FINE, Z_FINE, COS_PSI, TH, 
     $           REFR_FLAG, *880)
         ELSE
            REFR_FLAG = .FALSE.
         END IF
         
         CALL PC_ABS_COEFF((/(J, J=1,NL_FINE)/), COS_PSI, NL_FINE,  
     $        PCC, .FALSE., .FALSE., TG_OD)
         
         S_1 = 0.0D0
         DO I = 1, NL_FINE - 1
            S_1 = S_1 + (TG_OD(I) + TG_OD(I + 1))*
     $           (Z_FINE(I + 1) - Z_FINE(I))
         END DO

         OUTPUT_ARRAY(II) = S_1/2.0D0
      END DO

      DEALLOCATE( PCC%PCX_COEFF, PCC%PCY_COEFF)
      RETURN

 880  WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
      RETURN 1
C********************************************************************C
      END 
C********************************************************************C

      SUBROUTINE VERT_INTEGR (INPUT_ARRAY, OUTPUT_ARRAY)

      USE GRIDS,      ONLY: NLAYERS, ZZZ, NL_FINE, Z_FINE
      USE SPHER_PARS, ONLY: N_SOL_ANG
      USE TYPE_VARS,   ONLY: KND
         
      INTEGER       I
      REAL (KND)    S_1, INPUT_ARRAY(NLAYERS), TG_OD(NL_FINE)
      REAL (KND)    OUTPUT_ARRAY(N_SOL_ANG), Z_OD(NL_FINE)

      IF(NL_FINE == NLAYERS) THEN
         TG_OD(:) = INPUT_ARRAY(:)
         Z_OD(:) = ZZZ(:)
      ELSE
         Z_OD(:) = Z_FINE(:)
         CALL AKIMA_INT 
     $        (NLAYERS, ZZZ, INPUT_ARRAY, NL_FINE, Z_OD, TG_OD)
      END IF

      S_1 = 0.0D0
      DO I = 1, NL_FINE - 1
         S_1 = S_1 + (TG_OD(I) + TG_OD(I + 1))*
     $        DABS(Z_OD(I + 1) - Z_OD(I))
      END DO

      OUTPUT_ARRAY(:) = S_1/2.0D0

      RETURN
C********************************************************************C
      END
C********************************************************************C


