!***************************************************************************
!
!	SCIATRAN - Version 4.3 - Last modification: 17 Nov 2020
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
SUBROUTINE ST_WL_SETUP (WL_INP, NSPEC, SPECSEGS, NSEG_INP, SEG_NUM_INP, &
     WL_START, WL_END, *)
  ! Called by "gt_iface.f"

  USE ABSORBERS, ONLY: N_ST_MOL, MIX_ABS_START
  USE ESFT,      ONLY: DO_ESFT
  USE INFO,      ONLY: TG_INFO
  USE LOG_PARS,  ONLY: LUNIT, LOG_LEVEL, EU, N_DSO, Work_Dir
  Use SRF_Mod,   Only: SRF
  USE ST_VARS,   ONLY: CONVOL_FLAG, LA_WINS, WL_SEGS, CONVOL_ALL, HR_SEG_WL, &
                       HR_NSEG, NSPEC_RTM, WL_RTM, STEP_INP, LOW_RES_FLAG, &
                       LA_FLAG, NEW_SPEC_GRID, SF_DELTA, SEG_NUM_RTM
  USE LBL,       ONLY: NSEGS => NSEGS_LBL, SEG_WL
  USE TOOLS,     ONLY: SPLIT_STRING, READ_INPUT
  USE TYPES,     ONLY: KND, DYN_CHAR 
!***************************************************************************

  INTEGER,    INTENT(IN) :: NSPEC, NSEG_INP
  INTEGER,    INTENT(IN) :: SPECSEGS(2,NSEG_INP), SEG_NUM_INP(NSPEC)
  REAL (KND)  WL_INP(NSPEC)
  REAL (KND), INTENT(OUT) :: WL_START, WL_END
  ! Local variables
  CHARACTER (LEN=*), PARAMETER :: R_NAME='ST_WL_SETUP'
  CHARACTER (LEN=12)  NAME_STRING
  CHARACTER (LEN=:), ALLOCATABLE :: STR_INP
  INTEGER    I, I1, J, JJ, K, L, LL, KK, NUM, MAX_DIM, N_DB, NN, N_DB_ENTRIES,&
       JSEG, JSEG_Pre, N_WL_DB, J_S, J_E, N_S, N_STEPS, KS
  INTEGER,   PARAMETER :: MAX_ENTRIES = 200
  INTEGER    DB_NP(MAX_ENTRIES), STEP_RANGE(NSEG_INP + HR_NSEG)
  LOGICAL    OUT_FLAG, OFF_MSG, LA_ON(NSPEC)
  REAL (KND) SF_C1, SF_C2, START_WL, SEG_WL_INP(2,NSEG_INP)

  INTEGER     KK1, KK2, N_OUT, N_RES, NHW
  INTEGER,   PARAMETER :: M_EMI_SG = 4
  INTEGER     N_FIN(M_EMI_SG)
  REAL (KND)  FS_STEP(M_EMI_SG), GR_LR(M_EMI_SG,2)
  REAL (KND)  WL1, WL2, WLP, Cen, DOP_HW
  REAL (KND), ALLOCATABLE:: WL_FIN(:,:), WL_HEL(:)
  logical,    allocatable:: FIN_FLG(:)
  
  REAL (KND), DIMENSION (MAX_ENTRIES) :: DB_S, DB_E, DB_SAMP
  INTEGER,    ALLOCATABLE :: SEG_DB(:)
  REAL (KND), ALLOCATABLE :: WL_DB(:), WL_S(:), WL_E(:), STEP(:) 

  TYPE Linked_List_Entry
     LOGICAL    MX_FLG
     REAL (KND) WL_ELEM
     TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT_ELEM
  END TYPE Linked_List_Entry
  TYPE (LINKED_LIST_ENTRY), POINTER :: FIRST_ELEM, ELEMENT => NULL()
  TYPE (DYN_CHAR), ALLOCATABLE :: READ_ARR(:)
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')

  CALL DEALLOCATE_ARRAYS ! All arrays allocated below must be 
                         ! conditionally deallocated here

  ! HR_NSEG - number of spectral windows for line absorber:
  ! used if "esft", "lbl", or "atom"
  ALLOCATE( SEG_WL(2,NSEG_INP + HR_NSEG) ) !A 

  ! Internal segments
  ! -----------------
  DO  I = 1, NSEG_INP
     ! Frst and last wavelength of each segment
     SEG_WL_INP(1,I) = WL_INP(SPECSEGS(1,I) + 1)
     SEG_WL_INP(2,I) = WL_INP(SPECSEGS(1,I) + SPECSEGS(2,I))
  END DO

  ! 'Line absorber windows' and "not convolution for all points"
  IF( LA_WINS.AND..NOT.CONVOL_ALL ) THEN
     KK = 0
     DO JSEG = 1, NSEG_INP  
        DO L = 1, HR_NSEG
           IF( HR_SEG_WL(2,L) >= SEG_WL_INP(1,JSEG).AND. &
                HR_SEG_WL(1,L) <= SEG_WL_INP(2,JSEG)) THEN
              KK = KK + 1
              IF(KK > HR_NSEG + NSEG_INP) GOTO 200
              ! SEG_WL(1,KK) = MAX(SEG_WL_INP(1,JSEG),HR_SEG_WL(1,L)) &
              !     - SF_DWL(JSEG,1)
              ! SEG_WL(2,KK) = MIN(SEG_WL_INP(2,JSEG),HR_SEG_WL(2,L)) &
              !     + SF_DWL(JSEG,2)
              SEG_WL(1,KK) = SRF(JSEG)%SWB(MAX(SEG_WL_INP(1,JSEG),HR_SEG_WL(1,L)))
              SEG_WL(2,KK) = SRF(JSEG)%LWB(MIN(SEG_WL_INP(2,JSEG),HR_SEG_WL(2,L))) 
           END IF
        END DO   ! over "Spectral windows for line absorbers" (L)
     END DO   ! over spectral segments

     IF(KK == 0) THEN
        NEW_SPEC_GRID = .FALSE.
        ! LA_FLAG will be used in "gt_esft_parameter_setup.f", "st_abs_coeff.f"
        ALLOCATE(LA_FLAG(NSPEC))     ;  LA_FLAG = .FALSE.
        WL_START = MINVAL(WL_INP(:)) ;  WL_END  = MAXVAL(WL_INP(:))
        CALL ST_TRACK (R_NAME, 1, 'O')
        RETURN
     END IF
     NSEGS = KK
  ELSE
     ! Convolution for all points: First and last wavelength of 
     ! each segments including margin for convolution
     ! FORALL( JSEG = 1:NSEG_INP )
     !    SEG_WL(1,JSEG) = SEG_WL_INP(1,JSEG) - SF_DWL(JSEG,1)
     !    SEG_WL(2,JSEG) = SEG_WL_INP(2,JSEG) + SF_DWL(JSEG,2)
     ! END FORALL
     Do JSEG = 1, NSEG_INP
        SEG_WL(1,JSEG) = SRF(JSEG)%SWB(SEG_WL_INP(1,JSEG))
        SEG_WL(2,JSEG) = SRF(JSEG)%LWB(SEG_WL_INP(2,JSEG))
     End Do
     NSEGS = NSEG_INP
  END IF

  N_DB = 0 
  IF(DO_ESFT) THEN

     ! READ ESFT USER DATA BASE INFO
     ! -----------------------------         
     DO K = MIX_ABS_START, N_ST_MOL
        IF(.NOT.TG_INFO(K)%FM_FLG ) CYCLE
        STR_INP  = 'ESFT '//TRIM(TG_INFO(K)%NameUC)//' data bases'
        CALL Read_Input (STR_INP, READ_ARR, *880)
        READ(READ_ARR(1)%S,*, ERR = 101) N_DB_ENTRIES
        N_DB = N_DB + N_DB_ENTRIES
        IF (N_DB_ENTRIES <= 0 ) THEN
           OFF_MSG = .FALSE.
           SELECT CASE (TG_INFO(K)%M_TYPE)
           CASE('line') 
              TG_INFO(K)%FM_FLG = .FALSE.  
              TG_INFO(K)%WF_FLG = .FALSE.
              OFF_MSG = .TRUE.
           CASE('mixed') 
              IF(TG_INFO(K)%MAX_WL_CS < MINVAL(SEG_WL(1,:))) THEN
                 TG_INFO(K)%FM_FLG = .FALSE.  
                 TG_INFO(K)%WF_FLG = .FALSE.
                 OFF_MSG = .TRUE.
              END IF
           END SELECT
           IF(OFF_MSG) THEN
              CALL WRITE_MESSAGE(R_NAME, 'WARNING', 'No ESFT'&
                   //' data bases --> '//TRIM(TG_INFO(K)%NameUC))
              WRITE(EU,*) '    '//TRIM(TG_INFO(K)%NameUC)//&
                   ' switched off'
           END IF
        ENDIF
        IF (N_DB > MAX_ENTRIES) THEN
           CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Too many ESFT data bases entries')
           RETURN 1
        ENDIF
        KK = 2
        DO I = N_DB - N_DB_ENTRIES + 1, N_DB
           READ(READ_ARR(KK)%S,*, ERR = 101) NUM, NAME_STRING,&
                DB_S(I), DB_E(I), DB_SAMP(I), DB_NP(I)
           KK = KK + 1
        END DO
     END DO
        
     STR_INP  = 'Number and widths of ESFT spectral intervals'
     CALL Read_Input (STR_INP, READ_ARR, *880)
     READ(READ_ARR(1)%S,*, ERR = 101) N_STEPS
     ALLOCATE(STEP(0:N_STEPS), WL_S(N_STEPS), WL_E(N_STEPS))
     STEP(0) = STEP_INP
     READ(READ_ARR(2)%S,*, ERR = 101) (STEP(I), I = 1, N_STEPS)
     ! ------------------ End "ESFT" option -------------------------
  ELSE
     ALLOCATE(STEP(0:1))
     ! Internal wavelength step is the input in "control_conv.inp"
     STEP(0) = STEP_INP
  END IF
      
  STEP_RANGE(:) = 0
  IF(DO_ESFT) THEN
     ! ------------------ Start "ESFT" option -------------------------
     DO L = 1, N_STEPS
        WL_S(L) = MINVAL(DB_S(1:N_DB),DB_SAMP(1:N_DB) == STEP(L))
        WL_E(L) = MAXVAL(DB_E(1:N_DB),DB_SAMP(1:N_DB) == STEP(L))
     END DO
     
     DO JSEG = 1, NSEGS  
        DO L = 1, N_STEPS
           IF(WL_E(L) >= SEG_WL(1,JSEG).AND. WL_S(L) <= SEG_WL(2,JSEG)) THEN
              IF(STEP_RANGE(JSEG) /= 0) GOTO 201
              STEP_RANGE(JSEG) = L
           END IF
        END DO
     END DO
     ! ------------------ End "ESFT" option -------------------------
  END IF

  N_FIN(:) = 0
  IF( ANY((TG_INFO%M_TYPE == 'atom').AND.TG_INFO%FM_FLG) ) THEN
     ! Fine wavelength grid for emission lines
     I1 = 0
     DO JJ = 1, SIZE( TG_INFO%NAME )
        IF( .NOT.TG_INFO(JJ)%FM_FLG ) CYCLE
        SELECT CASE( TG_INFO(JJ)%NAME )
        CASE('mgi')
           I1 = I1 + 1
           Cen = 279.635d0   ! D2 emission line center (vacuum) [nm]
           DOP_HW = 0.5d-3   ! Doppler half-width   [nm]
           FS_STEP(I1) = DOP_HW/5.d0 ; NHW = 3
           CALL FG_RANGE
           I1 = I1 + 1
           Cen = 280.35259d0   ! D1 emission line center (vacuum) [nm]
           DOP_HW = 0.5d-3   ! Doppler half-width   [nm]
           FS_STEP(I1) = DOP_HW/5.d0
           CALL FG_RANGE
        CASE('mga')
           Cen = 285.2965d0  ! Emission line center (vacuum) [nm]
           DOP_HW = 0.5d-3   ! Doppler half-width   [nm]
           IF( Cen < WL_INP(NSPEC).AND.Cen > WL_INP(1) ) THEN
              I1 = I1 + 1
              FS_STEP(I1) = DOP_HW/2.d0 ; NHW = 3
              CALL FG_RANGE
           END IF
        CASE('na')
           I1 = I1 + 1
           Cen = 589.158d0 ! D2 emission line center (vacuum) [nm]
           DOP_HW = 0.7d-3   ! Doppler half-width   [nm]
           FS_STEP(I1) = DOP_HW/5.d0 ; NHW = 5
           CALL FG_RANGE
           I1 = I1 + 1
           Cen = 589.7555d0 ! D1 emission line center (vacuum) [nm]
           FS_STEP(I1) = DOP_HW/5.d0
           CALL FG_RANGE
        END SELECT
     ENDDO
     ! Spectral range for hyper-fine internal grid around
     ! emission lines
     ALLOCATE( WL_FIN( M_EMI_SG,MAXVAL(N_FIN) ) )
     DO I = 1, I1
        WL_FIN(I,1) = GR_LR(I,1) ; K = 1 ; KK = 0
        DO WHILE(WL_FIN(I,K) <= GR_LR(I,2))
           K = K + 1 ; KK = KK + 1
           IF( K > N_FIN(I) ) EXIT
           WL_FIN(I,K) = GR_LR(I,1) + KK*FS_STEP(I)
        END DO
     ENDDO
  ENDIF

  MAX_DIM = 0
  DO JSEG = 1, NSEGS
     MAX_DIM = MAX_DIM + IDNINT( (SEG_WL(2,JSEG) - SEG_WL(1,JSEG))&
          /STEP(STEP_RANGE(JSEG)) ) + 3
  END DO
  MAX_DIM = MAX_DIM + N_FIN(1)
  
  ALLOCATE(WL_DB(MAX_DIM), SEG_DB(MAX_DIM), LOW_RES_FLAG(NSPEC))

  ! Internal wavelength grid
  ! ------------------------
  K = 1
  DO JSEG = 1, NSEGS 
     LL = STEP_RANGE(JSEG)
     IF(LL == 0) THEN
        START_WL = SEG_WL(1,JSEG)
     ELSE
        N_S = FLOOR((SEG_WL(1,JSEG) - WL_S(LL) - 0.5D0*STEP(LL))/STEP(LL))
        START_WL = WL_S(LL) + STEP(LL)*(N_S + 0.5D0)
     END IF
     
     WL_DB(K) = START_WL  ;  SEG_DB(K) = JSEG   
     KK = 0
     DO WHILE(WL_DB(K) <= SEG_WL(2,JSEG))
        K = K + 1 ;  KK = KK + 1
        WL_DB(K) = START_WL + KK*STEP(LL)
        SEG_DB(K) = JSEG
     END DO
  END DO  ! over spectral segments (JSEG) 
      
  !      N_WL_DB = K - 1  ! can eventually lead to N_WL_DB = 0 
  N_WL_DB = K 

  IF( ANY(N_FIN(:) > 0) ) THEN
     ! Redefine WL_DB using fine wavelength grid for emission line

     DO I = 1, I1
        NN = N_FIN(I)
        IF(WL_FIN(I,NN) < WL_DB(1) .OR. WL_FIN(I,1) > WL_DB(N_WL_DB)) CYCLE
        KK1 = COUNT( WL_DB(1:N_WL_DB) < WL_FIN(I,1) )
        KK2 = COUNT( WL_DB(1:N_WL_DB) < WL_FIN(I,NN) ) + 1
        N_OUT = KK2 - KK1 - 1
        N_RES = N_WL_DB + NN - N_OUT
        ALLOCATE( WL_HEL(N_RES) )
        WL_HEL(1:KK1)          = WL_DB(1:KK1)
        WL_HEL(KK1+1:KK1+NN)   = WL_FIN(I,1:NN) 
        WL_HEL(KK1+NN+1:N_RES) = WL_DB(KK2:N_WL_DB)
        
        DEALLOCATE( WL_DB )
        N_WL_DB = N_RES
        ALLOCATE( WL_DB(N_WL_DB) ) 
        WL_DB(:) = WL_HEL(:)
        DEALLOCATE( WL_HEL )
     END DO

     ! Redefine SEG_DB:
     DEALLOCATE( SEG_DB ) ; ALLOCATE( SEG_DB(N_WL_DB) ) 
     K = 1
     DO JSEG = 1, NSEGS           
        SEG_DB(K) = JSEG
        DO WHILE(WL_DB(K) <= SEG_WL(2,JSEG))
           K = K + 1 ; SEG_DB(K) = JSEG
        ENDDO
     ENDDO
     ! ...............................................................
  END IF  ! ANY N_FIN > 0

  ! Find high resolution points
  ! ---------------------------
  LA_ON(:) = .FALSE.
  DO I = 1, NSPEC
     OUT_FLAG = .TRUE.
     DO K = 1, N_DB
        IF(WL_INP(I) >= DB_S(K).AND.WL_INP(I) <= DB_E(K)) THEN
           OUT_FLAG = .FALSE.
           EXIT
        END IF
     END DO
     IF(DO_ESFT.AND.OUT_FLAG) CYCLE
         
     IF( LA_WINS ) THEN
        ! "Line absorber windows" option is used ...
        DO L = 1, HR_NSEG
           IF(WL_INP(I) >= HR_SEG_WL(1,L).AND.WL_INP(I) <= HR_SEG_WL(2,L)) THEN
              LA_ON(I) = .TRUE.
              EXIT
           END IF
        END DO
        ! ..............................................
     ELSE
        LA_ON(I) = .TRUE.
     END IF
  END DO   ! over output wavelengths (I)
    
  IF( CONVOL_ALL ) THEN
     ! Convolution for all spectral points
     LOW_RES_FLAG(:) = .FALSE.
  ELSE
     ! Convolution for "Line absorber windows"
     LOW_RES_FLAG(:) = .NOT.LA_ON(:)
  END IF

  NEW_SPEC_GRID = .TRUE.
 
  ! Generate wavelength grid for RTM
  ! --------------------------------
  ! New version

  IF(CONVOL_FLAG) THEN   ! main logical flag for convolution
     IF( CONVOL_ALL ) THEN
        JSEG_Pre = -999
        DO I = 1,N_WL_DB
           JSEG = SEG_DB(I)
           ! WL1 = WL_INP(1)     - (SF_DWL(JSEG,1) + SF_DELTA)
           ! WL2 = WL_INP(NSPEC) + (SF_DWL(JSEG,2) + SF_DELTA)
           If(JSEG /= JSEG_Pre) Then
              WL1 = SRF(JSEG)%SWB(WL_INP(1)) - SF_DELTA
              WL2 = SRF(JSEG)%LWB(WL_INP(NSPEC)) + SF_DELTA
              JSEG_Pre = JSEG
           End If
           IF( WL_DB(I) >= WL1.AND.WL_DB(I) <= WL2 ) THEN
              CALL ADD_ENTRY(KS, WL_DB(I), .TRUE.)
           ENDIF
        ENDDO
        
     ELSE
                
        DO I = 1,NSPEC
           IF (LOW_RES_FLAG(I)) CALL ADD_ENTRY(KS, WL_INP(I), .FALSE.)
        ENDDO
        DO I = 1,N_WL_DB
           JSEG = SEG_DB(I)
           DO K = 1,HR_NSEG
              ! WL1 = HR_SEG_WL(1,K) - (SF_DWL(JSEG,1) + SF_DELTA)
              ! WL2 = HR_SEG_WL(2,K) + (SF_DWL(JSEG,2) + SF_DELTA)
              WL1 = SRF(JSEG)%SWB(HR_SEG_WL(1,K)) - SF_DELTA
              WL2 = SRF(JSEG)%LWB(HR_SEG_WL(2,K)) + SF_DELTA
              IF( WL_DB(I) >= WL1.AND.WL_DB(I) <= WL2 ) THEN
                 CALL ADD_ENTRY(KS, WL_DB(I), .TRUE.)
                 EXIT
              ENDIF
           ENDDO
        ENDDO
     ENDIF
  ELSE
     DO I = 1, NSPEC     
        IF(LOW_RES_FLAG(I)) THEN
           CALL ADD_ENTRY(KS, WL_INP(I), .FALSE.)
        ELSE
           KK = MINLOC(DABS(WL_DB(1:N_WL_DB) - WL_INP(I)),1)
           CALL ADD_ENTRY(KS, WL_DB(KK), .TRUE.)
        ENDIF
     END DO
  ENDIF    

  CALL FINALIZE (KS, WL_RTM, FIN_FLG)

  ! Return to old variables:
  NSPEC_RTM = SIZE( WL_RTM )
  WL_START = MINVAL(WL_RTM(1:NSPEC_RTM))
  WL_END   = MAXVAL(WL_RTM(1:NSPEC_RTM))

  ALLOCATE( LA_FLAG(NSPEC_RTM) )
  LA_FLAG(:) =  FIN_FLG(:)

  ! Used : rtm_main.f, gt_scatt.f, gt_surface.f:
  ALLOCATE( SEG_NUM_RTM(NSPEC_RTM))
  SEG_NUM_RTM = 0

  ! The following two lines are needed as ESFT wavelengths might be
  ! outside the selected interval as data base points closest  to the
  ! original grid are selected.
  WHERE( WL_RTM(:) < WL1) SEG_NUM_RTM(:) = 1
  WHERE( WL_RTM(:) > WL2) SEG_NUM_RTM(:) =  NSEG_INP
  DO JSEG = 1, NSEG_INP
     ! WL1 = SEG_WL_INP(1,JSEG) - (SF_DWL(JSEG,1) + SF_DELTA)
     ! WL2 = SEG_WL_INP(2,JSEG) + (SF_DWL(JSEG,1) + SF_DELTA)
!     write(6,*) JSEG, SEG_WL_INP(1,JSEG), SEG_WL_INP(2,JSEG), SRF(JSEG)%SWB(SEG_WL_INP(1,JSEG)), SRF(JSEG)%LWB(SEG_WL_INP(2,JSEG))
     WL1 = SRF(JSEG)%SWB(SEG_WL_INP(1,JSEG)) - SF_DELTA
     WL2 = SRF(JSEG)%LWB(SEG_WL_INP(2,JSEG)) + SF_DELTA
     WHERE( WL_RTM(:) >= WL1.AND.WL_RTM(:) <= WL2) SEG_NUM_RTM(:) = JSEG
  END DO
      
  ! Now go for modified segments to account for a change of boundaries
  !  DO JSEG = 1, NSEGS
  ! This case does not work with 1 measurements point and ESFT turned on
  
  ! producing all zeros in SEG_NUM_RTM
  !     WHERE( WL_RTM(:) >= SEG_WL(1,JSEG) - SF_DWL(JSEG,1).AND.
  ! $        WL_RTM(:) <= SEG_WL(2,JSEG)  + SF_DWL(JSEG,1)) SEG_NUM_RTM(:) = JSEG
  ! This case does not work for water vapor retrieval producing zeros in
  ! SEG_NUM_RTM at hte beginning and ends of the spectral segments.         
  !     WHERE(WL_RTM(:) >= WL_SEGS(1,JSEG).AND.
  ! $        WL_RTM(:) <= WL_SEGS(2,JSEG)) SEG_NUM_RTM(:) = JSEG
  !  END DO

  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN

101 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Reading error')
  WRITE(EU,*) '    Parameter to read: ', TRIM(STR_INP) 
  RETURN 1
  
200 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Too many spectral segments for line absorber')
  WRITE(EU,*) '    Spectral segment number: ', KK
  RETURN 1

 201  CALL WRITE_MESSAGE(R_NAME, 'ERROR', &
           'ESFT data having different spectral resolution '//&
           'must not be convolved with the same slit fuction')
  WRITE(EU,*) '    Spectral segment start and end wavelength: '
  WRITE(EU,*) '    ', SEG_WL(1,JSEG), SEG_WL(2,JSEG)
  WRITE(EU,*) '    End wavelength and step from the first ESFT database : '
  WRITE(EU,*) '    ', WL_E(L - 1), STEP(L - 1)
  WRITE(EU,*) '    Start wavelength and step from the second ESFT database : '
  WRITE(EU,*) '    ', WL_S(L), STEP(L)
  RETURN 1

880 WRITE(EU, *) ' ---> Called by '//TRIM(R_NAME)
  RETURN 1
      
CONTAINS
  !*********************************************************************

  SUBROUTINE FG_RANGE  
    GR_LR(I1,1) = Cen - DOP_HW*NHW
    GR_LR(I1,2) = Cen + DOP_HW*NHW
    N_FIN(I1) = IDNINT( (GR_LR(I1,2) - GR_LR(I1,1))/FS_STEP(I1) ) + 1
    RETURN
  END SUBROUTINE FG_RANGE
!*********************************************************************
  SUBROUTINE ADD_ENTRY (KK, WL_INP, FLG)      
    LOGICAL  FLG
    INTEGER  KK
    REAL (KND)  WL_INP
    IF (.NOT.ASSOCIATED(ELEMENT)) THEN
       ALLOCATE(FIRST_ELEM) ; ELEMENT => FIRST_ELEM ; KK = 1
    ELSE
       ALLOCATE(ELEMENT%NEXT_ELEM) ; ELEMENT => ELEMENT%NEXT_ELEM
       KK = KK + 1
    ENDIF
    ELEMENT%WL_ELEM = WL_INP
    ELEMENT%MX_FLG = FLG
    RETURN
  END SUBROUTINE ADD_ENTRY
  !*********************************************************************
  SUBROUTINE FINALIZE (KK, WL_SORT, LO_SORT )
    LOGICAL, ALLOCATABLE::  FLG(:)
    INTEGER  KK, II, I, J, K1, K0
    TYPE (LINKED_LIST_ENTRY), POINTER :: NEXT
    INTEGER,    ALLOCATABLE:: I_WL(:)
    LOGICAL,    ALLOCATABLE:: LO_SORT(:), ARR(:)
    REAL (KND), ALLOCATABLE:: WL_RTM(:), WL_SORT(:)
    
    NULLIFY(ELEMENT%NEXT_ELEM)
    IF(ALLOCATED(WL_RTM)) DEALLOCATE(WL_RTM, FLG)
    
    ALLOCATE( WL_RTM(KK), FLG(KK) )
    
    II = 0
    ELEMENT => FIRST_ELEM
    DO WHILE (ASSOCIATED(ELEMENT))
       II = II + 1
       WL_RTM(II) = ELEMENT%WL_ELEM
       FLG(II) = ELEMENT%MX_FLG
       NEXT => ELEMENT%NEXT_ELEM
       DEALLOCATE (ELEMENT) ; NULLIFY (ELEMENT)
       ELEMENT => NEXT
    END DO
    
    ALLOCATE( ARR(KK),I_WL(KK) )
    ARR = .TRUE.
    I = 0  ;  K0 = 0
    DO J = 1, KK
       K1 = MINLOC(WL_RTM(:),1,ARR)
       ARR(K1) = .FALSE.
       IF( K0 > 0 ) THEN
          IF( SNGL(WL_RTM(K1))==SNGL(WL_RTM(K0)) ) CYCLE
       ENDIF
       I = I + 1
       I_WL(I) = K1
       K0 = K1
    ENDDO
    
    ALLOCATE( WL_SORT(I), LO_SORT(I) )
    WL_SORT(:) = WL_RTM( I_WL(1:I) )
    LO_SORT(:) = FLG( I_WL(1:I) )
    
    
    RETURN
  END SUBROUTINE FINALIZE
  ! *****************************************************************
  SUBROUTINE DEALLOCATE_ARRAYS
    
    IF(ALLOCATED(SEG_WL)) DEALLOCATE(SEG_WL)
    IF(ALLOCATED(LA_FLAG)) DEALLOCATE(LA_FLAG)
    IF(ALLOCATED(WL_RTM)) DEALLOCATE(WL_RTM, SEG_NUM_RTM)
    IF(ALLOCATED(LOW_RES_FLAG)) DEALLOCATE(LOW_RES_FLAG)
    
    RETURN
  END SUBROUTINE DEALLOCATE_ARRAYS
  ! *****************************************************************
END SUBROUTINE ST_WL_SETUP
! ***********************************************************************


