!*********************************************************************
!
!	SCIATRAN - Version 3.6 - Last modification: 28 Jan 2016
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!*********************************************************************
SUBROUTINE T_CIRA (TTT, *)
  !    ... Call in "gt_ptconc_setup.f"
  USE GRIDS,    ONLY: NLAYERS, ZZZ
  USE LOG_PARS, ONLY: LOG_LEVEL, LUNIT, N_SSO
  USE ST_INPUT, ONLY: DAY_OF_YEAR, YEAR, DO_PROFILES_AKIMA, CIRA_DIR, CIRA_ZMIN
  USE ST_VARS,  ONLY: LATITUDE
  USE TYPES,    ONLY: KND
  !------------------------------------------------------------------

  REAL (KND) TTT(NLAYERS)
  ! Local parameters
  CHARACTER (LEN = *), PARAMETER :: R_NAME = 'T_CIRA'
  CHARACTER (LEN = :), ALLOCATABLE :: CIRA_FILE
  INTEGER , PARAMETER :: N_LATS = 17, N_DATA = 71 ! Number of data lines
  INTEGER  CU, MM, DD, N_HEAD, N_SKIP, LAT_IDX, II, NN
  REAL (KND) ARR_READ(N_DATA,N_LATS + 3), LATS(N_LATS), T_OUT(NLAYERS)
  !==================================================================

  CALL ST_TRACK (R_NAME, 1, 'I')

  N_HEAD = 6 ! Number of header lines
  CALL CALDAY(DAY_OF_YEAR, YEAR, MM, DD)
  
  IF(LATITUDE >= 0) THEN
     CIRA_FILE = CIRA_DIR//'/nht.lsn'
     LATS = [0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80]
  ELSE
     CIRA_FILE = CIRA_DIR//'/sht.lsn'
     LATS = [-80, -75, -70, -65, -60, -55, -50, -45, -40, -35, -30, -25, &
          -20, -15, -10, -5, 0]
  END IF

  LAT_IDX = MINLOC(DABS(LATITUDE - LATS),1)
     
  IF(LOG_LEVEL(1) >= N_SSO) WRITE(LUNIT,*) ' *** Reading: '//TRIM(CIRA_FILE)
  OPEN (NEWUNIT=CU, FILE=CIRA_FILE, STATUS='OLD')
  N_SKIP = N_DATA*(MM - 1) + N_HEAD*MM
  DO II = 1, N_SKIP
     READ(CU,*) 
  END DO
  DO II = 1, N_DATA
     READ(CU,*) ARR_READ(II,:)
  END DO
  Close(CU)
  NN = COUNT(ZZZ >= CIRA_ZMIN)
  CALL AL_INTERP (N_DATA, ARR_READ(:,3), ARR_READ(:,LAT_IDX + 3), NN, &
       ZZZ(1:NN), T_OUT(1:NN), DO_PROFILES_AKIMA, *100)
  TTT(1:NN) = T_OUT(1:NN)
  !  WHERE(ZZZ >= CIRA_ZMIN) TTT(:) = T_OUT(:)
    
  RETURN

100 CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'Interpolation error: CIRA 1986 data')
  RETURN 1
!*********************************************************************
END SUBROUTINE T_CIRA
!*********************************************************************
