C********************************************************************C
C
C	SCIATRAN - Version 4.2 - Last modification: 07 Jan 2020
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE TRANS_REFL_MATR(NSPEC, XSOL) 
c     ... Called by "rtm_init_first.f"

      USE BRDF,        ONLY: Nwl_BRF, Lin_rel, RefW_r, RefW_i
      USE GRIDS,       ONLY: NSOL
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: DIR_OUT
      USE LOS_PARS,    ONLY: LS_VA
      USE GT_POLYN,    ONLY: XM, AXM, XM_W, AXM_W
      USE SPHER_PARS,  ONLY: NLSA => N_LOS_ANG
      USE ST_VARS,     ONLY: RAD_TO_DEG, Pie, NSTR => NSTRMS, WF_FLAG
      USE TYPES,       ONLY: KND
      USE WATER,       ONLY: NSTR_W, W_speed, Ra_GG_WL, 
     $                       Taw_GG_WL, Rw_GG_WL, Twa_GG_WL, Ra_GS_WL, 
     $                       Taw_GS_WL, A_RTmat, Et_aw, Et_wa, Ref_ind
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NSPEC
      REAL (KND), INTENT(IN) :: XSOL(NSOL)
c     Local variables
c     ---------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME='TRANS_REFL_MATR'
      LOGICAL  CHECK_1, CHECK_F
c      INTEGER, PARAMETER :: N_Azm = 100, M_max = 50
      INTEGER, PARAMETER :: N_Azm = 180, M_max = 90
c      INTEGER, PARAMETER :: N_Azm = 260, M_max = 130
      INTEGER  I, J, K, I1, Jw, K1, N_REF, N_INC, Mh, N_REFw, N_Beam
      REAL (KND)  Mr, Mri, CPHI, G, SQS, SQPI, SIGMA, Hsqr, Mut, Rd,
     $     Cot, Coi, Tfr, Zn1, Zn2, S1, DM, Mur, Cos_r, Scat_r, Td,
     $     Scat_t, Rfr_w, Rfr, Arg, ERF, Def
      REAL (KND), DIMENSION (N_Azm)     :: X_phi, Wei_phi, Dc
      REAL (KND), DIMENSION (NSTR+1)    :: COSR,  SINR,  SHR
      REAL (KND), DIMENSION (NSTR_W+1)  :: COSRw, SINRw, SHRw
c      REAL (KND)  Res1(NSTR), Res2(NSTR_W)
      REAL (KND), ALLOCATABLE :: COSI(:), SINI(:), SHI(:), 
     $     Ref_AA(:,:,:,:), Tra_AW(:,:,:,:), Ref_WW(:,:,:,:), 
     $     Tra_WA(:,:,:,:)
      REAL (KND), POINTER :: P_ref(:,:), P_tra(:,:)
      EXTERNAL ERF
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      CHECK_1 = .FALSE.  ;   CHECK_F = .FALSE.
c      CHECK_1 = .TRUE.   ;   CHECK_F = .TRUE.

c     ... The number of reference wavelength (Nwl_BRF) is set to 1 in
c         "read_uwt.f" if constant refractive index is used.
c         Nwl_BRF is defined in "gt_iface.f" if refractive index
c         depends on the wavelength

c     ... The number of specific incident angles which will be used 
c         in calculation of reflection matrix equal to 
c                NSOL         if adjoint equation is not used and
c                NSOL + NLSA  if adjoint equation is used
      N_Beam = NSOL
      IF( WF_FLAG ) N_Beam = N_Beam + NLSA

c     ... Last dimension is the number of referent wavelength
      ALLOCATE( 
     $     Ra_GG_WL (NSTR  +1,NSTR,  M_Max,Nwl_BRF), 
     $     Ra_GS_WL (NSTR  +1,N_Beam,M_Max,Nwl_BRF), ! A
     $     Taw_GG_WL(NSTR_W+1,NSTR,  M_Max,Nwl_BRF), ! A
     $     Taw_GS_WL(NSTR_W+1,NSOL,  M_Max,Nwl_BRF), ! A
     $     Rw_GG_WL (NSTR_W+1,NSTR_W,M_Max,Nwl_BRF), ! A
     $     Twa_GG_WL(NSTR  +1,NSTR_W,M_Max,Nwl_BRF) ) ! A
      
c     ... Local allocatable arrays
      ALLOCATE( COSI(NSTR+N_Beam), SINI(NSTR+N_Beam), SHI(NSTR+N_Beam),
     $     Ref_AA(NSTR+1  ,NSTR+N_Beam,N_AZM, Nwl_BRF),
     $     Tra_AW(NSTR_W+1,NSTR+NSOL  ,N_AZM, Nwl_BRF), 
     $     Ref_WW(NSTR_W+1,NSTR_W     ,N_AZM, Nwl_BRF),
     $     Tra_WA(NSTR+1  ,NSTR_W     ,N_AZM, Nwl_BRF) )

      N_REF = NSTR + 1
      COSR(1:NSTR) = XM(:) ;  COSR(NSTR+1) = 1.d0
      SINR(:) = DSQRT(1.D0 - COSR(:)**2)

c     ... Reflection water => water
c      write(*,*) '   Number of NSTR_W ',NSTR_W
      N_REFw = NSTR_W + 1
      COSRw(1:NSTR_W) = XM_W(:) ;  COSRw(NSTR_W+1) = 1.d0
      SINRw(:) = DSQRT(1.D0 - COSRw(:)**2)

      N_INC = NSTR + N_Beam
      COSI(1:NSTR) = XM(:) ;  COSI(NSTR+1:NSTR+NSOL) = DABS( XSOL(:) )
c     ... If adjoint then add cosines of viewing angles
      IF( WF_FLAG ) COSI(NSTR+NSOL+1:NSTR+N_Beam) = LS_VA(:)

      SINI(:) = DSQRT(1.D0 - COSI(:)**2)

      SIGMA = Lin_rel(1) + Lin_rel(2)*W_speed

c     ... Shadowing factor
      SQS = DSQRT( SIGMA )  ;  SQPI = SQS/SQRT(Pie) ; SHR = 0.d0
      DO I = 1, N_REF
         IF( SINR(I) <= 0.0d0 ) CYCLE
         G = COSR(I)/SINR(I) ; Arg = G/SQS
         SHR(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
      ENDDO
      SHI = 0.d0
      DO I = 1, N_INC
         IF( SINI(I) <= 0.0d0 ) CYCLE
         G = COSI(I)/SINI(I) ; Arg = G/SQS
         SHI(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
      ENDDO
      SHRw = 0.d0
      DO I = 1, N_REFw
         IF( SINRw(I) <= 0.0d0 ) CYCLE
         G = COSRw(I)/SINRw(I) ; Arg = G/SQS
         SHRw(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
      ENDDO

c     ... Gauss grid and weights for the interval [-1,1]
      CALL GAULEG ( -1.d0, 1.d0, X_phi, Wei_phi, N_Azm )
c     ... Transform grid to the interval [0,Pie]
      X_phi(:) = (Pie*X_phi(:) + Pie)*0.5d0
c     ... Transform weights to the integration over interval [0,1]
      Wei_phi(:) = Wei_phi(:)*0.5d0

c      Mr = 1.34d0  ;  Mri = 1.d0/Mr
      DO Jw = 1, Nwl_BRF
         Mr = RefW_r(Jw)  ;  Mri = 1.d0/Mr
c         WRITE(*,*) '   1 Mr = ', Mr
         DO J = 1, N_AZM
            CPHI = DCOS(X_phi(J))
c           Transmissivity air => water
c           ===========================
            DO I = 1, NSTR_W+1  ! Angles for transmitted radiation (In water)
               I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
               DO K = 1, NSTR+NSOL ! Angles for incident radiation (In air)
                  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )       
c                 ... Shadowing factor
                  G = 1.d0/(1.D0 + SHRw(I1) + SHI(K1))
c                 ... Cosine of scat. angle between incident and transmitted
                  Scat_t = COSRw(I)*COSI(K) + SINRw(I)*SINI(K)*CPHI

                  Hsqr  = DSQRT( Mr**2 - 2.d0*Mr*Scat_t + 1 )
c                 ... Cosine between local normal and z-axis
                  Mut = (Mr*COSRw(I) - COSI(K))/Hsqr
c                 ... Cosine between transmitted and local normal
                  Cot = (Scat_t - Mr)/Hsqr
c                 ... Cosine between incident and local normal
                  Coi = (1.d0 - Mr*Scat_t)/Hsqr
                  IF( Scat_t - 1.d0/Mr >= 0.d0 .AND.
     $                 (Mr*COSRw(I) - COSI(K)) >= 0.d0) THEN 
c                 ... Fresnel transmission coefficient
                     Zn1 = 1.d0/(Mr*Coi + Cot)**2
                     Zn2 = 1.d0/(Coi + Mr*Cot)**2 
                     Tfr = 2.d0*Mr*Cot*Coi*(Zn1 + Zn2)
                     Mut = Mut*Mut
                     Tra_AW(I,K,J,Jw) = (Mr**2*Cot*Coi)
     $                    /(COSRw(I)*(Coi-Mr*Cot)**2*Pie*Mut**2*SIGMA)
     $                    *Tfr*DEXP(-(1.d0-Mut)/SIGMA/Mut)*G
                  ELSE
                     Tra_AW(I,K,J,Jw) = 0.d0
                  ENDIF

               ENDDO            ! over incident in air (K)
            ENDDO               ! over transmitted in water (I)

            if( j==1 )   write(*,*) '   SUM( Tra_AW ) ',
     $           SUM( Tra_AW(:,:,J,Jw) )

c           Transmissivity water => air
c           ===========================
            DO I = 1, NSTR+1    ! Angles for transmitted radiation (In air)
               I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
               DO K = 1, NSTR_W ! Angles for incident radiation (In water)
                  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )       
c                 ... Shadowing factor
                  G = 1.d0/(1.D0 + SHR(I1) + SHRw(K1))
c                 ... Cosine of scat. angle between incident and transmitted
                  Scat_t = COSR(I)*COSRw(K) + SINR(I)*SINRw(K)*CPHI
                  
                  Hsqr  = DSQRT( Mri**2 - 2.d0*Mri*Scat_t + 1 )
c                 ... Cosine between local normal and z-axis
                  Mut = (COSRw(K) - Mri*COSR(I))/Hsqr
c                 ... Cosine between transmitted and local normal
                  Cot = (Scat_t - Mri)/Hsqr
c                 ... Cosine between incident and local normal
                  Coi = (1.d0 - Mri*Scat_t)/Hsqr
                  IF( Scat_t - 1.d0/Mr >= 0.d0 .AND.
     $                 (COSRw(K) - Mri*COSR(I)) >= 0.d0) THEN 
c                    ... Fresnel transmission coefficient
                     Zn1 = 1.d0/(Mri*Coi + Cot)**2
                     Zn2 = 1.d0/(Coi + Mri*Cot)**2 
                     Tfr = 2.d0*Mri*Cot*Coi*(Zn1 + Zn2) ; Mut = Mut*Mut
                     Tra_WA(I,K,J,Jw) = (Mri**2*Cot*Coi)
     $                    /(COSR(I)*(Coi-Mri*Cot)**2*Pie*Mut**2*SIGMA)
     $                    *Tfr*DEXP(-(1.d0-Mut)/SIGMA/Mut)*G
                  ELSE
                     Tra_WA(I,K,J,Jw) = 0.d0
                  ENDIF
                  
               ENDDO  ! over incident in water (K)
            ENDDO  ! over transmitted in air (I)

            if( j==1 ) write(*,*) '   SUM( Tra_WA ) ',
     $           SUM( Tra_WA(:,:,J,Jw) )
            
         ENDDO  ! over azimuth (J)


c     ============================================
c     Reflectivity air => air  &  water => water
c     ============================================
         DO J = 1, N_AZM
            CPHI = DCOS(X_phi(J))
c           ========================
c           Reflectivity air => air 
c           ========================
            DO I = 1, N_REF  
               I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
               DO K = 1, N_INC
                  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )       
c                 ... Shadowing factor
                  G = 1.d0/(1.d0 + SHR(I1) + SHI(K1))
c                 ... Cosine of scat. angle between incident and reflected
                  Scat_r = -COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
c                 ... Cosine between local normal and z-axis
                  Mur = (COSR(I) + COSI(K))/DSQRT(2.d0*(1.d0-Scat_r))
c                 ... Cosine between reflected and local normal
                  Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )

c                     corresponding refracted angle
                  S1 = DSQRT(Cos_r**2 + Mr**2 - 1.d0)/Mr
c                 ... Fresnel reflection coefficient
                  Zn1 = (Mr*Cos_r+S1)**2 ; Zn2 = (Cos_r+Mr*S1)**2 
                  Rfr = 0.5d0*( (Mr*Cos_r-S1)**2/Zn1 
     $                 + (Cos_r-Mr*S1)**2/Zn2 )
c                 ... Reflectivity
                  Mur = Mur*Mur
                  Ref_AA(I,K,J,Jw) = Rfr/(4.d0*COSR(I)*Pie*Mur**2)
c                   ... Exclude PDF and shadowing to compare with brdf case
     $                 *DEXP(-(1.d0-Mur)/SIGMA/Mur)/SIGMA*G
               ENDDO  ! over incidence angles (K)
            ENDDO  ! over reflection angles (I)

            if( j==1 ) write(*,*) '   SUM( Ref_AA ) ',
     $           SUM( Ref_AA(:,:,J,Jw) )

c           ===========================
c           Reflectivity water => water
c           ===========================
            DO I = 1, N_REFw    ! Number of discrete angles in ocean
               I1 = CEILING( FLOAT(I)/FLOAT(Nvs) )
               DO K = 1, N_REFw-1
                  K1 = CEILING( FLOAT(K)/FLOAT(Nvs) )       
c                 ... Shadowing factor
                  G = 1.d0/(1.D0 + SHRw(I1) + SHRw(K1))
c                 ... Cosine of scat. angle between incident and reflected
                  Scat_r = -COSRw(I)*COSRw(K) + SINRw(I)*SINRw(K)*CPHI
c                 ... Cosine between local normal and z-axis
                  Mur = (COSRw(I) + COSRw(K))/DSQRT(2.d0*(1.d0-Scat_r))
c                 ... Cosine between reflected and local normal
                  Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )

                  IF( Scat_r - (2.d0*Mri**2 - 1.d0) >= 0.d0 ) THEN
                     Rfr_w = 1.d0
                  ELSE
c                    ... corresponding refracted angle
                     S1 = DSQRT(Cos_r**2 + Mri**2 - 1.d0)/Mri
c                    ... Fresnel reflection coefficient
                     Zn1 = (Mri*Cos_r+S1)**2 ; Zn2 = (Cos_r+Mri*S1)**2 
                     Rfr_w = 0.5d0*( (Mri*Cos_r-S1)**2/Zn1
     $                    + (Cos_r-Mri*S1)**2/Zn2 )
                  ENDIF
c                 ... Reflectivity
                  Mur = Mur*Mur
                  Ref_WW(I,K,J,Jw) = 1.d0/
     $                 (4.d0*COSRw(I)*Pie*Mur**2*SIGMA)
     $                 *Rfr_w*DEXP(-(1.d0-Mur)/SIGMA/Mur)*G
               ENDDO            ! over (K)
            ENDDO               ! over (I)
            if( j==1 ) write(*,*) '   SUM( Ref_WW ) ',
     $           SUM( Ref_WW(:,:,J,Jw) )
         ENDDO                  ! over azimuthal angles (J)
      ENDDO                     ! over reference wavelength (Jw)

      

c            K1 = 10
c            WRITE(*,*) '   Scalar case coupling'
c            WRITE(*,*) '      incident angle'
c            WRITE(*,'(2x,I2,2x,F7.3)') K1, DACOS(COSI(K1))/Pie*180.
c            WRITE(*,*) '      azimuthal angle'
c            WRITE(*,'(6x,F7.3)')   X_phi(10)/Pie*180.
c            DO I = 1, N_ref
c               WRITE(*,'(2x,F7.3,2x,E15.7)') 
c     $              DACOS(COSR(I))/Pie*180., Ref_AA(I,K1,10,1)
cc              ... to compare with brdf case
c     $              *COSR(I)/COSI(K1)*Pie
c            ENDDO

c            K1 = 10
c            WRITE(*,*) '   Scalar case coupling'
c            WRITE(*,*) '      incident angle'
c            WRITE(*,'(2x,I2,2x,F7.3)') K1, DACOS(COSI(K1))/Pie*180.
c            WRITE(*,*) '      azimuthal angle'
c            WRITE(*,'(6x,F7.3)')   X_phi(10)/Pie*180.
c            DO I = 1, N_ref
c               WRITE(*,'(2x,F7.3,2x,E15.7)') 
c     $              DACOS(COSR(I))/Pie*180., Ref_AA(I,K1,10)
cc              ... to compare with brdf case
c     $              *COSR(I)/COSI(K1)*Pie
c            ENDDO

      IF( CHECK_1 ) CALL CHECK_RT

c     ... Fourier expansion coefficients in the scalar case
c     -----------------------------------------------------
      Ra_GG_WL = 0.d0  ;  Twa_GG_WL = 0.d0
      Rw_GG_WL = 0.d0  ;  Taw_GG_WL = 0.d0
      Ra_GS_WL = 0.d0  ;  Taw_GS_WL = 0.d0

      DO Jw = 1,Nwl_BRF
         DO Mh = 1,M_max        ! do-loop over Fourier components
            DM = DFLOAT(Mh-1)
            Dc(:) = DCOS(DM*X_phi(:))*Wei_phi(:)
            DO I = 1,N_Azm      ! Numerical integration over azimuthal angle
               Ra_GG_WL(:,:,Mh,Jw) = Ra_GG_WL(:,:,Mh,Jw) 
     $              + Ref_AA(:,1:NSTR,I,Jw)*Dc(I)
               Ra_GS_WL(:,:,Mh,Jw) = Ra_GS_WL(:,:,Mh,Jw) 
     $              + Ref_AA(:,NSTR+1:NSTR+N_Beam,I,Jw)*Dc(I)
               
               Taw_GG_WL(:,:,Mh,Jw) = Taw_GG_WL(:,:,Mh,Jw) 
     $              + TRA_AW(:,1:NSTR,I,Jw)*Dc(I)
               Taw_GS_WL(:,:,Mh,Jw) = Taw_GS_WL(:,:,Mh,Jw) 
     $              + TRA_AW(:,NSTR+1:NSTR+NSOL,I,Jw)*Dc(I)
            
               Rw_GG_WL(:,:,Mh,Jw) = Rw_GG_WL(:,:,Mh,Jw) 
     $              + Ref_WW(:,:,I,Jw)*Dc(I)
            
               Twa_GG_WL(:,:,Mh,Jw) = Twa_GG_WL(:,:,Mh,Jw) 
     $              + TRA_WA(:,:,I,Jw)*Dc(I)
            ENDDO
         ENDDO
      ENDDO

cc     ... Test output for zeroth harmonic
c      WRITE(*,*) '   Matrix Tra_AW(:,:) dimension', NSTR_W,'*',NSTR
c      OPEN( 55, file=DIR_OUT//'/TAW_tes.dat' )
c         J = 10
c         DO I = 1, NSTR_W 
c            WRITE(55,'(1x,F8.3,17(1x,e12.4))') DACOS(XM_W(I))/Pie*180.,
c     $           (Tra_AW(I,K,J,1), K = 1,NSTR)
c         ENDDO
c      CLOSE( 55 )
c
c      WRITE(*,*) '   Matrix Tra_WA(:,:,1) dimension', NSTR,'*',NSTR_W
c      OPEN( 55, file=DIR_OUT//'/TWA_tes.dat' )
c         J = 10
c         DO I = 1, NSTR
c            WRITE(55,'(1x,F8.3,33(1x,e12.4))') DACOS(XM(I))/Pie*180.,
c     $           (Tra_WA(I,K,J,1), K = 1,NSTR_W)
c         ENDDO
c      CLOSE( 55 )
c      STOP '  Stop  in "trans_refl_matr.f"'
c
c         Mh = 1
c         DM = DFLOAT(Mh-1)
c         Dc(:) = DCOS(DM*X_phi(:))*Wei_phi(:)
c         DO I = 1,N_Azm
c            Taw_GS_WL(:,:,Mh,1) = Taw_GS_WL(:,:,Mh,1) 
c     $           + TRA_AW(:,NSTR+1:NSTR+NSOL,I,1)*Dc(I)
c         ENDDO
c         DO K = 1, NSTR_W+1
c            write(*,'(2x,I3,2(2x,e12.4))') I, Taw_GS_WL(K,1,1,1)
c         ENDDO
         
c     ... The calculation of Et_wa and Et_aw is shifted in 
c         routine "st_uwt_coeff.f" in the case Nwl_BRF > 1
      IF( Nwl_BRF == 1 ) THEN
c        ... Effective transmission coefficient (water -> air)
c            for the nadir direction only
         Et_wa = 1.d0 - 2.d0*Pie/XM_W(NSTR_W)
     $        *SUM( Rw_GG_WL(1:NSTR_W,NSTR_W,1,1)*AXM_W(:) )

c        ... Effective transmission coefficient (air -> water)
c            for the nadir direction only
         Et_aw = 1.d0 - 2.d0*Pie/XM(NSTR)
     $        *SUM( Ra_GG_WL(1:NSTR,NSTR,1,1)*AXM(:) )
      ENDIF

c      WRITE(*,*) '  Et_wa ', Et_wa
c      WRITE(*,*) '  Et_aw ', Et_aw

c      WRITE(*,*) '  Et_wa ', Et_wa
c      WRITE(*,*) '  Et_aw ', Et_aw


c     ... Perform Nakagima and Tanaka correction approach
c     ---------------------------------------------------
      IF( TRIM(A_RTmat)=='SS_NT') THEN
c        ... Nakagjima & Tanaka correction Ocean -> Atmosphere:
         DO Jw = 1,Nwl_BRF
            P_ref => Rw_GG_WL (1:NSTR_W,1:NSTR_W,1,Jw)
            P_tra => Twa_GG_WL(1:NSTR,  1:NSTR_W,1,Jw)
            DO K = 1,NSTR_W
c              ... Flux reflected back to the ocean
               Rd = SUM( P_ref(:,K)*AXM_W(:) )
c              ... Flux transmitted into atmosphere
               Td = SUM( P_tra(:,K)*AXM(:) )
c              ... Energy balance
               Def = 1.d0 - 2.d0*Pie*(Rd + Td)/XM_W(K)
               Rw_GG_WL(1:NSTR_W,K,:,Jw) = Rw_GG_WL(1:NSTR_W,K,:,Jw)
     $              /(1.d0 - Def)
               Twa_GG_WL(1:NSTR,K,:,Jw) = Twa_GG_WL(1:NSTR,K,:,Jw)
     $              /(1.d0 - Def)
            ENDDO

c           ... Nakagjima & Tanaka correction Atmosphere -> Ocean:
            P_ref => Ra_GG_WL (1:NSTR,  :,1,Jw)
            P_tra => Taw_GG_WL(1:NSTR_W,:,1,Jw)
            DO K = 1,NSTR
c              ... Flux reflected back to the atmosphere
               Rd = SUM( P_ref(:,K)*AXM(:) )
c              ... Flux transmitted into ocean
               Td = SUM( P_tra(:,K)*AXM_W(:) )
c              ... Energy balance
               Def = 1.d0 - 2.d0*Pie*(Rd + Td)/XM(K)
               Ra_GG_WL(1:NSTR,K,:,Jw) = Ra_GG_WL(1:NSTR,K,:,Jw)
     $              /(1.d0 - Def)
               Taw_GG_WL(1:NSTR_W,K,:,Jw) = Taw_GG_WL(1:NSTR_W,K,:,Jw)
     $              /(1.d0 - Def)
            ENDDO
c          WRITE(*,*) 
c           ... The same correction but for solar directions
            P_ref => Ra_GS_WL (1:NSTR,  :,1,Jw)
            P_tra => Taw_GS_WL(1:NSTR_W,:,1,Jw)
            DO K = 1, N_Beam
               Rd = SUM( P_ref(:,K)*AXM(:) )
               Td = SUM( P_tra(:,K)*AXM_W(:) )
               Def = 1.d0 - 2.d0*Pie*(Rd + Td)/DABS( XSOL(K) )
c               P_ref(:,K) = P_ref(:,K)/(1.d0 - Def)
c               P_tra(:,K) = P_tra(:,K)/(1.d0 - Def)
               Ra_GS_WL(1:NSTR,K,:,Jw) = Ra_GS_WL(1:NSTR,K,:,Jw)
     $              /(1.d0 - Def)
               Taw_GS_WL(1:NSTR_W,K,:,Jw) = Taw_GS_WL(1:NSTR_W,K,:,Jw)
     $              /(1.d0 - Def)
            ENDDO
         ENDDO                  ! over reference wavelengths (Jw)
cc     c           ... Test output 
c         OPEN(77,file = DIR_OUT//'/diff_aw.dat')
c         DO K = 1,NSTR
c            WRITE(77,*) DACOS(XM(K))/Pie*180., Res1(K)
c         ENDDO
c         CLOSE(77)
c         OPEN(77,file = DIR_OUT//'/diff_wa.dat')
c         DO K = 1,NSTR_W
c            WRITE(77,*) DACOS(XM_W(K))/Pie*180., Res2(K)
c         ENDDO
c         CLOSE(77)
c         PAUSE
      ENDIF  ! A_RTmat='SS_NT'
      
c      WRITE(*,*) '  Ra_GG_WL :', SUM( Ra_GG_WL )
c      WRITE(*,*) '  Rw_GG_WL :', SUM( Rw_GG_WL )
c      WRITE(*,*) ' Twa_GG_WL :', SUM( Twa_GG_WL )
c      WRITE(*,*) ' Taw_GG_WL :', SUM( Taw_GG_WL )


c      WRITE(*,*) '  Ra_GG_WL :', SUM( Ra_GG_WL )
c      WRITE(*,*) '  Rw_GG_WL :', SUM( Rw_GG_WL )
c      WRITE(*,*) ' Twa_GG_WL :', SUM( Twa_GG_WL )
c      WRITE(*,*) ' Taw_GG_WL :', SUM( Taw_GG_WL )

c     ... Test of the Fourier expansion for solar angle 
      IF( CHECK_F ) CALL CHECK_FOURIER
      
c      STOP '   stop in "trans_refl_matr.f" '
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c **********************************************************************

      SUBROUTINE CHECK_RT
      USE WATER,   ONLY : Ref_ind

      REAL (KND) F_ref, F_tra, SW(NSOL), Fr1(NSOL), SA(NSOL),
     $     O_Raa(NSTR,NSTR+NSOL), O_Rww(NSTR_W,NSTR_W),
     $     O_Taw(NSTR_W,NSTR+NSOL), O_Twa(NSTR,NSTR_W),
     $     Raa_m(NSTR), Rww_m(NSTR_W), Tao_m(NSTR), Toa_m(NSTR_W)

c     ... Integration over azimuthal angle "air->air" (reflection)
      DO I = 1, NSTR
         DO K = 1, NSTR+NSOL
            O_Raa(I,K) = SUM( Ref_AA(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
c     ... Reflected flux:
      F_ref = SUM( O_Raa(1:NSTR,NSTR+NSOL)*AXM(:) )
c      F_ref = SUM( O_Raa(1:NSTR,NSTR)*AXM(:) )

            WRITE(*,*) '   Refl_aa NCOL = ',NSTR+NSOL+1,
     $      ' file: DIR_OUT//"/refl_aa.dat"'
      OPEN( 44,file=DIR_OUT//'/refl_aa.dat' )
      DO I = 1,NSTR
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSR(I))*RAD_TO_DEG, (O_Raa(I,K), K = 1,NSTR+NSOL)
      ENDDO
      CLOSE(44)
c     ... Integration over azimuth "air->water" (transmission)  
      DO I = 1,NSTR_W
         DO K = 1, NSTR+NSOL
            O_Taw(I,K) = SUM( TRA_AW(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
c     ... Transmitted flux:
      F_tra = SUM( O_Taw(1:NSTR_W,NSTR+NSOL)*AXM_W(:) )
c      F_tra = SUM( O_Taw(1:NSTR_W,NSTR)*AXM_W(:) )
      WRITE(*,*) ' Comparison of the incident, refle. and trans. fluxes'
      WRITE(*,'(4(2x,1Pe12.4))') 
     $     DABS(XSOL(1)), F_ref, F_tra, F_ref + F_tra

      SA(:) = DABS(XSOL(1))
      WRITE(*,*) '    XSOL in atm:', SA(:)
      SW(:) =  DSQRT(SA(:)**2 + Ref_ind**2 - 1.d0)/Ref_ind
      WRITE(*,*) '    XSOL in wat:', SW(1)
      WRITE(*,*) '     Ref_ind   :',Ref_ind
      Fr1(:) = 0.5d0*
     $     (( (SA(:) - Ref_ind*SW(:))/(SA(:) + Ref_ind*SW(:)) )**2
     $     + ( (SW(:) - Ref_ind*SA(:))/(SW(:) + Ref_ind*SA(:)) )**2)
      WRITE(*,*) '     Fresnel Ref;   :',Fr1(:)
      WRITE(*,*) ' Comparison of the transmitted fluxes'
      WRITE(*,'(4(2x,1Pe12.4))') 
     $     DABS(XSOL(1)), F_tra, SA(1)/SW(1)*(1.d0 - Fr1(1))*SW(1)

            WRITE(*,*) '   Tra_aw NCOL = ',NSTR+NSOL+1,
     $     '   File : "./DATA_OUT/tra_aw.dat"'
      OPEN( 44,file=DIR_OUT//'/tra_aw.dat' )
      DO I = 1,NSTR_W
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, (O_Taw(I,K), K = 1,NSTR+NSOL)
      ENDDO
      CLOSE(44)
c     ... Unidirectional reflectivity and transmissivity (air->water)      
      DO K = 1,NSTR
         Raa_m(K) = SUM( O_Raa(1:NSTR,K)*AXM(:)  )/COSI(K)
         Tao_m(K) = SUM( O_Taw(:,K)*AXM_W(:))/COSI(K)
      ENDDO
c     ... Output to test of the deficiency (reflection-transmission)
            WRITE(*,*) '   File : "./DATA_OUT/ref_tra_aw.dat"'
      OPEN( 44,file=DIR_OUT//'/ref_tra_aw.dat' )
      DO K = 1,NSTR
         WRITE(44,'(2x,f7.3,3(2x,e12.4E3))') 
     $        DACOS(COSI(K))*RAD_TO_DEG, Raa_m(K), Tao_m(K),
     $        1.d0 - Raa_m(K) - Tao_m(K)
      ENDDO
      CLOSE(44)


c     ******************************************************************
c          ************* water => atmosphere ****************
c     ******************************************************************
c     ... Integration over azimuthal angle "water->water" (reflection)
      DO I = 1, NSTR_W
         DO K = 1, NSTR_W
            O_Rww(I,K) = SUM( Ref_WW(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
         WRITE(*,*) '   Refl_ww NCOL = ',NSTR_W + 1,
     $     ' file: "./DATA_OUT/refl_ww.dat"'
      OPEN( 44,file=DIR_OUT//'/refl_ww.dat' )
      DO I = 1,NSTR_W
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, (O_Rww(I,K), K = 1,NSTR_W)
      ENDDO
      CLOSE(44)

c      Integration over azimuth "water=>air"  (transmission)    
      DO I = 1,NSTR
         DO K = 1, NSTR_W
            O_Twa(I,K) = SUM( TRA_WA(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
            WRITE(*,*) '   Tra_wa NCOL = ',NSTR_W + 1,
     $     '   File : "./DATA_OUT/tra_wa.dat"'
      OPEN( 44,file = DIR_OUT//'/tra_wa.dat' )
      DO I = 1,NSTR
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSI(I))*RAD_TO_DEG, (O_Twa(I,K), K = 1,NSTR_W)
      ENDDO
      CLOSE(44)
c     ... Unidirectional reflectivity and transmissivity (water->air)
      DO K = 1,NSTR_W
         Rww_m(K) = SUM( O_Rww(:,K)*AXM_W(:) )/COSRw(K)
         Toa_m(K) = SUM( O_Twa(:,K)*AXM(:) )  /COSRw(K)
      ENDDO
c     ... Output to test of the deficiency (reflection-transmission)
            WRITE(*,*) '   File : "./DATA_OUT/ref_tra_wa.dat"'
      OPEN( 44,file=DIR_OUT//'/ref_tra_wa.dat' )
      DO K = 1,NSTR_W
         WRITE(44,'(2x,f7.3,4(2x,e12.4E3))') 
     $        DACOS(COSRw(K))*RAD_TO_DEG, Rww_m(K), Toa_m(K),
     $        1.d0 - Rww_m(K) - Toa_m(K)
      ENDDO
      CLOSE(44)
c      STOP '   STOP in "check_rt.f" within "trans_refl_matr.f""'
      END SUBROUTINE CHECK_RT
c **********************************************************************

      SUBROUTINE CHECK_FOURIER
      INTEGER Nphi
      REAL (KND) S_Res(NSTR+1,NSOL), T_res(NSTR_W+1,NSOL)

c     Output azimuthal angle
      Nphi =  1
      DO I = 1,NSOL
         S_res(:,I) = Ra_GS_WL(:,I,1,1)
         T_res(:,I) = Taw_GS_WL(:,I,1,1)
         DO Mh = 2,M_max        ! do-loop over Fourier components
            DM = DCOS(DFLOAT(Mh-1)*X_phi(Nphi))
            S_res(:,I) = S_res(:,I) + 2.d0* Ra_GS_WL(:,I,Mh,1)*DM
            T_res(:,I) = T_res(:,I) + 2.d0*Taw_GS_WL(:,I,Mh,1)*DM
         ENDDO
      ENDDO      

      WRITE(*,*) '    Fourier expansion for azimuthal angle:'
      WRITE(*,'(10x,F7.3)') X_phi(Nphi)*RAD_TO_DEG
      OPEN( 44,file=DIR_OUT//'/Raa_azm.dat' )
      DO I = 1,NSTR+1
         WRITE(44,'(2x,F7.3,10(1x,e12.4E3))') 
     $        DACOS(COSR(I))*RAD_TO_DEG, 
     $        (S_res(I,K), Ref_AA(I,NSTR+K,Nphi,1), K = 1,NSOL)
      ENDDO
      CLOSE(44)

      OPEN( 44,file=DIR_OUT//'/Taw_azm.dat' )
      DO I = 1,NSTR_W+1
         WRITE(44,'(2x,F7.3,10(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, 
     $        (T_res(I,K), Tra_AW(I,NSTR+K,Nphi,1), K = 1,NSOL)
      ENDDO
      CLOSE(44)
c      PAUSE '   Pause in "check_fourier.f" '
      END SUBROUTINE CHECK_FOURIER

      END

c **********************************************************************
      SUBROUTINE Fre_REF(N, FR, AIN, Ref_inp, CAW)

      USE TYPES,  ONLY : KND
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: N
      CHARACTER (LEN=2), INTENT(IN) :: CAW
      REAL (KND), INTENT(IN) :: Ref_inp(2), AIN(N)
      REAL (KND)  Mr, FR(N), ATR(N), Acr, Ref_idx
c     ==================================================================
      Ref_idx = Ref_inp(1)
      SELECT CASE( CAW )
      CASE('ww')
         Mr = 1.d0/Ref_idx
         Acr = DSQRT( 1.d0 - Mr*Mr )
      CASE('aa')
         Mr = Ref_idx  ! relative refractive index water/air
         Acr = 0.d0
      END SELECT

      WHERE( AIN(:) > Acr )
c        ... Cosine of the refracted angle
         ATR(:) = DSQRT(AIN(:)*AIN(:) + Mr*Mr - 1.d0)/Mr
         FR(:) = 0.5d0*(
     $        ((Mr*AIN(:) -    ATR(:))/(Mr*AIN(:) +    ATR(:)))**2 
     $        + ((AIN(:)    - Mr*ATR(:))/(   AIN(:) + Mr*ATR(:)))**2)
      ELSEWHERE
         FR(:) = 1.d0
      END WHERE
      RETURN
      END

c **********************************************************************
      SUBROUTINE Fre_Ref_v(Nst, FRv, Cos_i, Ref_idx)

      USE INTERN_DIMS, ONLY: Nvs
      USE ST_VARS,     ONLY: Pie
      USE TYPES,       ONLY: KND, DCMP
c     ------------------------------------------------------------------
      INTEGER,           INTENT(IN) :: Nst
      REAL (KND),        INTENT(IN) :: Ref_idx(2), Cos_i(Nst)
      REAL (KND)  FRv(Nst,Nvs,Nvs)
c     ... Local variables
      INTEGER I
      REAL (KND)  Coi, RF0(4,4)
      COMPLEX (kind(0.0d0)) :: CN1, CN2
      COMPLEX (kind(0.0d0)) :: Rc_L, Rc_P, Cos_t
c     ==================================================================

c     ... Refractive indexes of the air  
      CN1 = DCMP( 1.d0 ) 
c     ... Refractive indexes of the water  
      CN2 = DCMP( Ref_idx(1),Ref_idx(2) )  
      DO I = 1, Nst
         Coi = Cos_i(I)
c        ... Cosine of the refracted angle
         Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Coi*Coi)/CN2/CN2 )
c        ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
         Rc_L = (CN2*Coi - CN1*Cos_t)/(CN2*Coi + CN1*Cos_t)      
         Rc_P = (CN1*Coi - CN2*Cos_t)/(CN1*Coi + CN2*Cos_t)
         IF( Nvs == 1 ) THEN
            FRv(I,1,1) = 0.5d0*(CDABS(Rc_L)**2 + CDABS(Rc_P)**2) 
            CYCLE
         ENDIF
c        ... Fresnel reflection matrix including polarization
         RF0 = 0.d0
         RF0(1,1) = 0.5d0*(CDABS(Rc_L)**2 + CDABS(Rc_P)**2) 
         RF0(1,2) = 0.5d0*(CDABS(Rc_L)**2 - CDABS(Rc_P)**2)
         RF0(2,1) = RF0(1,2) ; RF0(2,2) = RF0(1,1)
         RF0(3,3) = DBLE( Rc_L*CONJG(Rc_P) )   ;  RF0(4,4) = RF0(3,3)
         RF0(3,4) = AIMAG( Rc_L*CONJG(Rc_P) )  ;  RF0(4,3) = -RF0(3,4)
         FRv(I,1:4,1:4) = RF0(:,:)
      ENDDO  ! over incident angles (I)
      
      RETURN
      END SUBROUTINE Fre_Ref_v
c *************************************************************************

      SUBROUTINE Fre_tot(Nst, FRv, Cos_i, Ref_idx)

      USE INTERN_DIMS, ONLY: Nvs
      USE TYPES,       ONLY: KND, DCMP
c     ------------------------------------------------------------------
      INTEGER,     INTENT(IN) :: Nst
      REAL (KND),  INTENT(IN) :: Ref_idx(2), Cos_i(Nst)
      REAL (KND)  FRv(Nst,Nvs,Nvs)
c     ... Local variables
      INTEGER I
      REAL (KND)  Di, RF0(Nvs,Nvs)
      COMPLEX (kind(0.0d0)) :: CN1, CN2
      COMPLEX (kind(0.0d0)) :: Rc_L, Rc_P, Cos_t
c     ==================================================================
c     ... Refractive indexes of the air  
      CN2 = DCMP( 1.d0 ) 
c     ... Refractive indexes of the water  
      CN1 = DCMP( Ref_idx(1),Ref_idx(2) )

      DO I = 1, Nst
c        ... Cosine of the incident angle
         Di = Cos_i(I)
c        ... Cosine of the refracted angle
         Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Di*Di)/CN2/CN2 )
c     ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
         Rc_L = (CN2*Di - CN1*Cos_t)/(CN2*Di + CN1*Cos_t)      
         Rc_P = (CN1*Di - CN2*Cos_t)/(CN1*Di + CN2*Cos_t)
         IF( Nvs == 1 ) THEN
            FRv(I,1,1) = 0.5d0*(CDABS(Rc_L)**2 + CDABS(Rc_P)**2) 
            CYCLE
         ENDIF
         
c        ... Fresnel reflection matrix including polarization
         RF0 = 0.d0
         RF0(1,1) = 0.5d0*(CDABS(Rc_L)**2 + CDABS(Rc_P)**2) 
         RF0(1,2) = 0.5d0*(CDABS(Rc_L)**2 - CDABS(Rc_P)**2)
         RF0(2,1) = RF0(1,2) ; RF0(2,2) = RF0(1,1)
         RF0(3,3) = DBLE( Rc_L*CONJG(Rc_P) )   ;  RF0(4,4) = RF0(3,3)
         RF0(3,4) = AIMAG( Rc_L*CONJG(Rc_P) )  ;  RF0(4,3) = -RF0(3,4)
         FRv(I,1:4,1:4) = RF0(:,:)         
      ENDDO  ! over incident angles (I)
      
      RETURN
      END SUBROUTINE Fre_tot
c *************************************************************************
