C********************************************************************C
C
C	SCIATRAN - Version 4.7 - Last modification: 25 Dec 2023
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE TRANS_REFL_MATR_V(NSPEC, XSOL) 
c     ... Called by "rtm_init_first.f"

      USE BRDF,        ONLY: Nwl_BRF, Lin_rel, RefW_r, RefW_i
      USE GRIDS,       ONLY: NSOL
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: DIR_OUT, USE_SHAD
      USE LOS_PARS,    ONLY: LS_VA
      USE GT_POLYN,    ONLY: XM, AXM, XM_W, AXM_W
      USE SPHER_PARS,  ONLY: NLSA => N_LOS_ANG
      USE ST_VARS,     ONLY: RAD_TO_DEG, Pie, NSTR => NSTRMS, WF_FLAG,
     $                       N_HARM
      USE TYPES,       ONLY: KND, DCMP
      USE WATER,       ONLY: NSTR_W, W_speed, Ra_GG_WL, 
     $                       Taw_GG_WL, Rw_GG_WL, Twa_GG_WL, Ra_GS_WL, 
     $                       Taw_GS_WL, A_RTmat, Et_aw, Et_wa, Ref_ind
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: NSPEC
      REAL (KND), INTENT(IN) :: XSOL(NSOL)
c     Local variables
c     ---------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME='TRANS_REFL_MATR_V'
      LOGICAL  CHECK_1, CHECK_F, Vect
c      INTEGER, PARAMETER :: N_Azm = 100, M_max = 50
c      INTEGER, PARAMETER :: N_Azm = 180, M_max = 90
c      INTEGER, PARAMETER :: N_Azm = 260, M_max = 130
      INTEGER  I, J, K, Jw, N_REF, N_INC, Mh, N_REFw, N_Beam, Nvi,
     $     Nvk, N2, NWv, NAv, N_Azm, M_max
      REAL (KND)  Mr, Mri, CPHI, G, SQS, SQPI, SIGMA, Hsqr, Mut, Cot, 
     $     Coi, S1, DM, Mur, Cos_r, Scat_r, Scat_t, Arg, ERF, T1, T2
c      REAL (KND), DIMENSION (N_Azm)   :: X_phi, Wei_phi, Dc
      REAL (KND), DIMENSION (NSTR)    :: COSR,  SINR,  SHR
      REAL (KND), DIMENSION (NSTR_W)  :: COSRw, SINRw, SHRw
      REAL (KND), ALLOCATABLE :: COSI(:), SINI(:), SHI(:), 
     $     Ref_AA(:,:,:,:), Tra_AW(:,:,:,:), Ref_WW(:,:,:,:),
     $     X_phi(:), Wei_phi(:), Dc(:),
     $     Tra_WA(:,:,:,:), Rot_l(:,:), Rot_r(:,:), Ds(:), Dha(:,:,:)
      TARGET  Ref_AA, TRA_AW, Ref_WW, Tra_WA, Dha
      REAL (KND), POINTER :: P_ref(:,:), P_tra(:,:), Aaz(:,:,:), 
     $     Aha(:,:)
      REAL (KND)  TF0(Nvs,Nvs)
      COMPLEX (kind(0.0d0)) :: CN1, CN2, Tc_L, Tc_P, Cos_t
      EXTERNAL ERF
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      N_Azm = 2*N_HARM  ;  M_MAX = N_HARM
c      write(*,*) '   N_Azm and M_max eto  ', N_azm, M_max
      ALLOCATE( X_phi(N_Azm), Wei_phi(N_Azm), Dc(N_Azm) )
      
      CHECK_1 = .FALSE.  ;   CHECK_F = .FALSE.
c      CHECK_1 = .TRUE.   ;   CHECK_F = .FALSE.

c     ... The number of reference wavelength (Nwl_BRF) is set to 1 in
c         "read_uwt.f" if constant refractive index is used.
c         Nwl_BRF is defined in "gt_iface.f" if refractive index
c         depends on the wavelength

c     ... The number of specific incident angles which will be used 
c         in calculation of reflection matrix equal to 
c                NSOL         if adjoint equation is not used and
c                NSOL + NLSA  if adjoint equation is used
      N_Beam = NSOL
      IF( WF_FLAG ) N_Beam = N_Beam + NLSA

      Vect = .FALSE.
      IF( Nvs > 1 ) THEN
         ALLOCATE( Rot_l(4,4), Rot_r(4,4), Ds(N_Azm),
     $        Dha(Nvs*NSTR_W,Nvs*NSTR_W, Nwl_BRF) )
c     ... Initialization of rotational matrices
         Rot_l = 0.d0       ;  Rot_r = 0.d0
         Rot_l(1,1) = 1.d0  ;  Rot_r(1,1) = 1.d0
         Rot_l(4,4) = 1.d0  ;  Rot_r(4,4) = 1.d0
         Vect = .TRUE.
      ENDIF
      N_REF  = NSTR  ;  N_REFw = NSTR_W 
      N_INC = NSTR + N_Beam

c     ... Last dimension is the number of referent wavelength
      If (Allocated(Ra_GG_WL)) DeAllocate(Ra_GG_WL, Ra_GS_WL, Taw_GG_WL,
     $     Taw_GS_WL, Rw_GG_WL, Twa_GG_WL)
      ALLOCATE( 
     $     Ra_GG_WL (Nvs*NSTR,   Nvs*NSTR,   M_Max, Nwl_BRF), 
     $     Ra_GS_WL (Nvs*NSTR,   Nvs*N_Beam, M_Max, Nwl_BRF), ! A
     $     Taw_GG_WL(Nvs*NSTR_W, Nvs*NSTR,   M_Max, Nwl_BRF), ! A
     $     Taw_GS_WL(Nvs*NSTR_W, Nvs*NSOL,   M_Max, Nwl_BRF), ! A
     $     Rw_GG_WL (Nvs*NSTR_W, Nvs*NSTR_W, M_Max, Nwl_BRF), ! A
     $     Twa_GG_WL(Nvs*NSTR,   Nvs*NSTR_W, M_Max, Nwl_BRF) ) ! A
      
c     ... Local allocatable arrays
      ALLOCATE( COSI(NSTR+N_Beam), SINI(NSTR+N_Beam), SHI(NSTR+N_Beam),
     $     Tra_AW(Nvs*NSTR_W, Nvs*(NSTR+NSOL),   N_AZM, Nwl_BRF), 
     $     Tra_WA(Nvs*NSTR,   Nvs*NSTR_W,        N_AZM, Nwl_BRF),
     $     Ref_AA(Nvs*NSTR,   Nvs*(NSTR+N_Beam), N_AZM, Nwl_BRF),
     $     Ref_WW(Nvs*NSTR_W, Nvs*NSTR_W,        N_AZM, Nwl_BRF) )

cc     ... COSR(NSTR) = COSR(NSTR+1) the same nadir direction two times !!
      COSR(1:NSTR) = XM(:)
      SINR(:) = DSQRT(1.D0 - COSR(:)**2)

c     ... Reflection water => water
      COSRw(1:NSTR_W) = XM_W(:)
      SINRw(:) = DSQRT(1.D0 - COSRw(:)**2)
      
      COSI(1:NSTR) = XM(:) ;  COSI(NSTR+1:NSTR+NSOL) = DABS( XSOL(:) )
c     ... If adjoint then add cosines of viewing angles
      IF( WF_FLAG ) COSI(NSTR+NSOL+1:NSTR+N_Beam) = LS_VA(:)

      SINI(:) = DSQRT(1.D0 - COSI(:)**2)
      SIGMA = Lin_rel(1) + Lin_rel(2)*W_speed

c     ... Shadowing factor
      SELECT CASE(USE_SHAD)
      CASE('yes')
         SQS = DSQRT( SIGMA )  ;  SQPI = SQS/SQRT(Pie) ; SHR = 0.d0
         DO I = 1, N_REF
            IF( SINR(I) <= 0.0d0 ) CYCLE
            G = COSR(I)/SINR(I) ; Arg = G/SQS
            SHR(I) = 0.5d0*(DEXP(-G*G/SIGMA) *SQPI/G - 1.d0 + ERF(Arg))
         ENDDO
         SHI = 0.d0
         DO I = 1, N_INC
            IF( SINI(I) <= 0.0d0 ) CYCLE
            G = COSI(I)/SINI(I) ; Arg = G/SQS
            SHI(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
         ENDDO            
         SHRw = 0.d0
         DO I = 1, N_REFw
            IF( SINRw(I) <= 0.0d0 ) CYCLE
            G = COSRw(I)/SINRw(I) ; Arg = G/SQS
            SHRw(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
         ENDDO
c         DO I = 1, N_REFw
c            WRITE(*,*) DACOS(COSRw(I))/Pie*180., SHRw(I)
c         ENDDO
      CASE('no')
         SHI = 0.d0 ; SHR = 0.d0 ; SHRw = 0.d0
      END SELECT

c     ... Gauss grid and weights for the interval [-1,1]
      CALL GAULEG ( -1.d0, 1.d0, X_phi, Wei_phi, N_Azm )
c     ... Transform grid to the interval [0,Pie]
      X_phi(:) = (Pie*X_phi(:) + Pie)*0.5d0
c     ... Transform weights to the integration over interval [0,1]
      Wei_phi(:) = Wei_phi(:)*0.5d0

c      Mr = 1.34d0  ;  Mri = 1.d0/Mr
      DO Jw = 1, Nwl_BRF
         Mr = RefW_r(Jw)  ;  Mri = 1.d0/Mr
c        =================================================================
c                  Transmission air => water
c        =================================================================
c        ... Complex refractive indexe of air and water
         CN1 = DCMP( 1.d0 ) ; CN2 = DCMP( RefW_r(Jw),RefW_i(Jw) )  
         DO J = 1, N_AZM
            CPHI = DCOS(X_phi(J))
            DO I = 1, NSTR_W       ;  Nvi = Nvs*(I-1)
               DO K = 1, NSTR+NSOL ;  Nvk = Nvs*(K-1)
c                 ... Shadowing factor
                  G = 1.d0/(1.d0 + SHRw(I) + SHI(K))
                  Scat_t = COSRw(I)*COSI(K) + SINRw(I)*SINI(K)*CPHI
c                 ... Normalization factor (see Eq.(C-fact) in STB)
                  Hsqr  = DSQRT( Mr**2 - 2.d0*Mr*Scat_t + 1 )
c                 ... Cosine between local normal and z-axis
c                     ( see Eq.(Mun-aw) in STB )
                  Mut = (Mr*COSRw(I) - COSI(K))/Hsqr
c                 ... Cosine between transmitted and local normal
c                     ( see Eq.(loc-cot-aw) in STB )
                  Cot = (Scat_t - Mr)/Hsqr
c                  Cot = -(Scat_t - Mr)/Hsqr
c                 ... Cosine between incident and local normal
c                      ( see Eq.(loc-coi-aw) in STB )
                  Coi = (1.d0 - Mr*Scat_t)/Hsqr
c                  Coi = -(1.d0 - Mr*Scat_t)/Hsqr
                  IF( (Scat_t - 1.d0/Mr) >= 0.d0 .AND.
     $                 (Mr*COSRw(I) - COSI(K)) >= 0.d0) THEN 
c                    ... Fresnel transmission matrix TF0(4,4)
c                        or transmission coefficient TF0(1,1)
                     CALL Fre_Tr_v
                     IF(Vect) CALL ROT_RL(-COSRw(I), SINRw(I),
     $                   -COSI(K), SINI(K), Scat_t)
                     Mut = Mut*Mut
                     Tra_AW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw)
     $                    = (Mr**2*Cot*Coi)
     $                    /(COSRw(I)*(Coi-Mr*Cot)**2*Pie*Mut**2*SIGMA)
     $                   *DEXP(-(1.d0-Mut)/SIGMA/Mut)*G*TF0(1:Nvs,1:Nvs)
                  ELSE
                     Tra_AW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw) = 0.d0
                  ENDIF
               ENDDO  ! over incident in air (K)
            ENDDO  ! over transmitted in water (I)
         ENDDO  ! over azimuth (J)
c         stop
c        ========================================================
c                  Transmission water => air
c        =======================================================
c        ... Complex refractive indexe of water and air
         CN1 = DCMP( RefW_r(Jw),RefW_i(Jw) )  ;  CN2 = DCMP( 1.d0 )
         DO J = 1, N_AZM
            CPHI = DCOS(X_phi(J))
            DO I = 1, NSTR       ;  Nvi = Nvs*(I-1)
               DO K = 1, NSTR_W  ;  Nvk = Nvs*(K-1)
c                 ... Shadowing factor
                  G = 1.d0/(1.D0 + SHR(I) + SHRw(K))
c                 ... Cosine of scat. angle between incident and transmitted
                  Scat_t = COSR(I)*COSRw(K) + SINR(I)*SINRw(K)*CPHI
c                 ...  Normalization factor
                  Hsqr  = DSQRT( Mri**2 - 2.d0*Mri*Scat_t + 1 )
c                 ... Cosine between local normal and z-axis
                  Mut = (COSRw(K) - Mri*COSR(I))/Hsqr
c                 ... Cosine between transmitted and local normal
c                     ( see Eq.(loc-cot-wa) in STB )
                  Cot = (Scat_t - Mri)/Hsqr
c                 ... Cosine between incident and local normal
c                     ( see Eq.(loc-coi-wa) in STB )
                  Coi = (1.d0 - Mri*Scat_t)/Hsqr

                  IF( (Scat_t - 1.d0/Mr) >= 0.d0 .AND.
     $                 (COSRw(K) - Mri*COSR(I)) >= 0.d0) THEN 
                     CALL Fre_Tr_v
                     IF(Vect) CALL ROT_RL(COSR(I), SINR(I),
     $                   COSRw(K), SINRw(K), Scat_t)
                     Mut = Mut*Mut
                     Tra_WA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw)
     $                    = (Mri**2*Cot*Coi)
     $                    /(COSR(I)*(Coi-Mri*Cot)**2*Pie*Mut**2*SIGMA)
     $                   *DEXP(-(1.d0-Mut)/SIGMA/Mut)*G*TF0(1:Nvs,1:Nvs)
                  ELSE
                     Tra_WA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw) = 0.d0
                  ENDIF
               ENDDO  ! over incident in water (K)
            ENDDO  ! over transmitted in air (I)
         ENDDO  ! over azimuth (J)

c        ========================================================
c            Reflection air => air  &  water => water
c        ========================================================
c        ... Complex refractive indexe of air and water
         CN1 = DCMP( 1.d0 ) ; CN2 = DCMP( RefW_r(Jw),RefW_i(Jw) )  
         DO J = 1, N_AZM
            CPHI = DCOS(X_phi(J))
c           ========================
c            Reflection air => air 
c           ========================
            DO I = 1, N_REF     ;  Nvi = Nvs*(I-1)
               DO K = 1, N_INC  ;  Nvk = Nvs*(K-1)
c                 ... Shadowing factor
                  G = 1.d0/(1.d0 + SHR(I) + SHI(K))
c                 ... Cosine of scat. angle between incident and reflected
                  Scat_r = -COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
c                 ... Cosine between local normal and z-axis
c                     ( see STB Eq.(Ang-Mun) )
                  Mur = (COSR(I) + COSI(K))/DSQRT(2.d0*(1.d0-Scat_r))
c                 ... Cosine between reflected and local normal
c                     ( see STB Eq.(Cos-lock) )
                  Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )
c                     corresponding refracted angle
                  S1 = DSQRT(Cos_r**2 + Mr**2 - 1.d0)/Mr
                  CALL Fre_Ra_v
                  IF(Vect) CALL ROT_RL(COSR(I), SINR(I),
     $                 -COSI(K), SINI(K), Scat_r)
c                 ... Reflectivity
                  Mur = Mur*Mur
                  Ref_AA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw)
     $                 = TF0(1:Nvs,1:Nvs)/(4.d0*COSR(I)*Pie*Mur**2)
c                 ... Exclude PDF and shadowing to compare with brdf case
     $                 *DEXP(-(1.d0-Mur)/SIGMA/Mur)/SIGMA*G
               ENDDO  ! over incidence angles (K)
            ENDDO  ! over reflection angles (I)
         ENDDO  ! over azimuthal angles (J)
c           ===========================
c           Reflection water => water
c           ===========================
c        ... Complex refractive indexe of water and air
         CN1 = DCMP( RefW_r(Jw),RefW_i(Jw) )  ;  CN2 = DCMP( 1.d0 )
         DO J = 1, N_AZM          ;  CPHI = DCOS(X_phi(J))
            DO I = 1, N_REFw      ;  Nvi = Nvs*(I-1)
               DO K = 1, N_REFw   ;  Nvk = Nvs*(K-1)
c                 ... Shadowing factor
                  G = 1.d0/(1.D0 + SHRw(I) + SHRw(K))
c                 ... Cosine of scat. angle between incident and reflected
                  Scat_r = -COSRw(I)*COSRw(K) + SINRw(I)*SINRw(K)*CPHI
c                 ... Cosine between local normal and z-axis
                  Mur = (COSRw(I) + COSRw(K))/DSQRT(2.d0*(1.d0-Scat_r))
c                 ... Cosine between reflected and local normal
                  Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )
                  CALL Fre_Rw_v
                  IF(Vect) CALL ROT_RL(-COSRw(I), SINRw(I),
     $                 COSRw(K), SINRw(K), Scat_r)
c                 ... Reflection matrix
                  Mur = Mur*Mur
                  Ref_WW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J,Jw) = 1.d0/
     $                 (4.d0*COSRw(I)*Pie*Mur**2*SIGMA)
     $                 *DEXP(-(1.d0-Mur)/SIGMA/Mur)*G*TF0(1:Nvs,1:Nvs)
               ENDDO  ! over (K)
            ENDDO  ! over angles in ocean (I)
         ENDDO  ! over azimuthal angles (J)
      ENDDO  ! over reference wavelength (Jw)


      IF( CHECK_1 ) CALL CHECK_RT

c     ... Fourier expansion coefficients in the scalar case
c     -----------------------------------------------------
      Ra_GG_WL = 0.d0  ;  Twa_GG_WL = 0.d0
      Rw_GG_WL = 0.d0  ;  Taw_GG_WL = 0.d0
      Ra_GS_WL = 0.d0  ;  Taw_GS_WL = 0.d0

      IF( Vect ) THEN
         DO Mh = 1,M_max   ;   DM = DFLOAT(Mh-1)
            Dc(:) = DCOS(DM*X_phi(:))*Wei_phi(:)
            Ds(:) = DSIN(DM*X_phi(:))*Wei_phi(:)
c           ====================================================
c           ... Fourier series: Ra_GG_WL, Ra_GS_WL
c           ====================================================
            DO J = 1,N_ref     ;  Nvi = 4*(J-1) ! NSTR after last correction
               DO I = 1,N_inc  ;  Nvk = 4*(I-1) ! NSTR + N_Beam
                  DO Jw = 1,Nwl_BRF
                     Aaz =>  Ref_AA(Nvi+1:Nvi+4,Nvk+1:Nvk+4, :,Jw)
                     Aha =>  Dha   (Nvi+1:Nvi+4,Nvk+1:Nvk+4, Jw)
                     CALL Fourier
                  ENDDO  ! over reference wavelengths (Jw)
               ENDDO  ! over incidence (Gauss +Sol <+LOS>) (I)
            ENDDO  ! over reflection [Gauss + 1 nadir (J)]
            N2 = 4*N_REF
            Ra_GG_WL(:,1:4*NSTR , Mh,:)  = Dha(1:N2,1:4*NSTR, :)
            Ra_GS_WL(:,1:4*N_Beam, Mh,:) = Dha(1:N2,4*NSTR+1:4*N_INC, :)
c           ====================================================
c           ... Fourier series Taw_GG_WL, Taw_GS_WL
c           ====================================================
            DO J = 1,N_REFw       ;  Nvi = 4*(J-1) ! NSTR_W  after last correction
               DO I = 1,NSTR+NSOL ;  Nvk = 4*(I-1) ! NSTR + NSOL
                  DO Jw = 1,Nwl_BRF
                     Aaz => TRA_AW(Nvi+1:Nvi+4,Nvk+1:Nvk+4, :,Jw)
                     Aha => Dha   (Nvi+1:Nvi+4,Nvk+1:Nvk+4, Jw)
                     CALL Fourier
                  ENDDO  ! over reference wavelengths (Jw)
               ENDDO   ! over incidence (Gauss +Sol <+LOS>) (I)
            ENDDO  ! over reflection [Gauss + 1 nadir (J)]
            Taw_GG_WL(:,1:4*NSTR, Mh,:) =Dha(:,       1:4*NSTR, :)
            Taw_GS_WL(:,1:4*NSOL, Mh,:) =Dha(:,4*NSTR+1:4*(NSTR+NSOL),:)
c           ====================================================
c            ... Fourier series Rw_GG_WL
c           ====================================================
            DO J = 1,N_REFw    ;  Nvi = 4*(J-1) ! NSTR_W after last correction
               DO I = 1,NSTR_w ;  Nvk = 4*(I-1) ! NSTR_w
                  DO Jw = 1,Nwl_BRF
                     Aaz =>  Ref_WW(Nvi+1:Nvi+4,Nvk+1:Nvk+4, :,Jw)
                     Aha =>  Dha   (Nvi+1:Nvi+4,Nvk+1:Nvk+4, Jw)
                     CALL Fourier
                  ENDDO  ! over reference wavelengths (Jw)
               ENDDO  ! over incidence (Gauss +Sol <+LOS>) (I)
            ENDDO  ! over reflection [Gauss + 1 nadir (J)]
            Rw_GG_WL(:,1:4*NSTR_W, Mh,:) = Dha(:,1:4*NSTR_W,:)
c           ====================================================
c           ... Fourier series Twa_GG_WL
c           ====================================================
            DO J = 1,NSTR       ;  Nvi = 4*(J-1) ! NSTR after last correction
               DO I = 1,NSTR_w  ;  Nvk = 4*(I-1) ! NSTR_w
                  DO Jw = 1,Nwl_BRF
                     Aaz =>  Tra_WA(Nvi+1:Nvi+4,Nvk+1:Nvk+4, :,Jw)
                     Aha =>  Dha   (Nvi+1:Nvi+4,Nvk+1:Nvk+4, Jw)
                     CALL Fourier
                  ENDDO         ! over reference wavelengths (Jw)
               ENDDO            ! over incidence (Gauss +Sol <+LOS>) (I)
            ENDDO               ! over reflection [Gauss + 1 nadir (J)]
c            N2 = 4*(NSTR + 1)
            N2 = 4*NSTR
            Twa_GG_WL(:,1:4*NSTR_W, Mh,:) = Dha(1:N2,1:4*NSTR_W,:)
         ENDDO         
c        ===> End vector case <===          
      ELSE
         DO Jw = 1,Nwl_BRF
            DO Mh = 1,M_max 
               DM = DFLOAT(Mh-1)
               Dc(:) = DCOS(DM*X_phi(:))*Wei_phi(:)
               DO I = 1,N_Azm
                  Ra_GG_WL(:,:,Mh,Jw) = Ra_GG_WL(:,:,Mh,Jw) 
     $                 + Ref_AA(:,1:NSTR,I,Jw)*Dc(I)
                  Ra_GS_WL(:,:,Mh,Jw) = Ra_GS_WL(:,:,Mh,Jw) 
     $                 + Ref_AA(:,NSTR+1:NSTR+N_Beam,I,Jw)*Dc(I)
               
                  Taw_GG_WL(:,:,Mh,Jw) = Taw_GG_WL(:,:,Mh,Jw) 
     $                 + TRA_AW(:,1:NSTR,I,Jw)*Dc(I)
                  Taw_GS_WL(:,:,Mh,Jw) = Taw_GS_WL(:,:,Mh,Jw) 
     $                 + TRA_AW(:,NSTR+1:NSTR+NSOL,I,Jw)*Dc(I)
            
                  Rw_GG_WL(:,:,Mh,Jw) = Rw_GG_WL(:,:,Mh,Jw) 
     $                 + Ref_WW(:,:,I,Jw)*Dc(I)
            
                  Twa_GG_WL(:,:,Mh,Jw) = Twa_GG_WL(:,:,Mh,Jw) 
     $                 + TRA_WA(:,:,I,Jw)*Dc(I)
               ENDDO  ! over azimuthal angle (I)
            ENDDO  ! over Fourier harmonics (Mh)
         ENDDO  ! over reference wavelengths (Jw)
c     ===> End scalar case <===          
      ENDIF

c      write(*,*) '  Nwl_BRF  ', Nwl_BRF
      NWv = Nvs*NSTR_W  ;   NAv = Nvs*NSTR

      IF( Nwl_BRF == 1 ) THEN
c        ... Effective transmission coefficient (water -> air)
c            for the nadir direction only
         Et_wa = 1.d0 - 2.d0*Pie/XM_W(NSTR_W)
     $        *SUM( Rw_GG_WL(1:NWv:Nvs,Nvs*(NSTR_W-1)+1,1,1)
     $        *AXM_W(:) )
c        ... Effective transmission coefficient (air -> water)
c            for the nadir direction only
c         write(*,*) '   Calculation Et_aw ', XM(NSTR)
c         write(*,*) '  shape(Ra_GG_WL) ', shape(Ra_GG_WL) 
c         write(*,*) '  shape(Axm)      ', shape(Axm)
c         write(*,*) '  Axm(NSTR)       ', Axm(NSTR) 
         Et_aw = 1.d0 - 2.d0*Pie/XM(NSTR)
     $        *SUM( Ra_GG_WL(1:NAv:Nvs,Nvs*(NSTR-1)+1,1,1)
     $        *AXM(:) )
c         write(*,*) '  Et_aw =       ', Et_aw
      ENDIF
c     ... The calculation of Et_wa and Et_aw is shifted in 
c         routine "st_uwt_coeff.f" in the case Nwl_BRF > 1

c     ... Test of the Fourier expansion for solar angle 
      IF( CHECK_F ) CALL CHECK_FOURIER
      
c      STOP '   stop in "trans_refl_matr_v.f" '
      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS

c **********************************************************************
      SUBROUTINE Fourier
      DO K = 1,4
         Aha(K,K) = SUM( Aaz(K,K,:)*Dc(:) )
      ENDDO  ! over Stokes components (K)
      Aha(1,2) = SUM( Aaz(1,2,:)*Dc(:) )
      Aha(2,1) = SUM( Aaz(2,1,:)*Dc(:) )
      DO K = 1,2
         Aha(3,K) =  SUM( Aaz(3,K,:)*Ds(:) )
         Aha(K,3) = -SUM( Aaz(K,3,:)*Ds(:) )
      ENDDO
      END SUBROUTINE Fourier

c **********************************************************************
      SUBROUTINE Fre_Tr_v
c     ... Parallel (T_L) and perpendicular (T_P) Fresnel
c         transmission coefficients      
      Tc_L = 2.d0*CDSQRT(CN1*CN2*Coi*Cot) /(CN2*Coi + CN1*Cot)      
      Tc_P = 2.d0*CDSQRT(CN1*CN2*Coi*Cot) /(CN1*Coi + CN2*Cot)
      IF( Nvs == 1 ) THEN
         TF0 (1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         RETURN
      ENDIF
c     ... Fresnel transmission matrix including polarization
      TF0 = 0.d0
      TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
      TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
      TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
      TF0(3,3) = DBLE( Tc_L*CONJG(Tc_P) )   ;  TF0(4,4) = TF0(3,3)
      TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
      RETURN
      END SUBROUTINE Fre_Tr_v      
c **********************************************************************
      SUBROUTINE Fre_Ra_v
         Coi = Cos_r ;  Cot = S1
c        ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
         Tc_L = (CN2*Coi - CN1*Cot)/(CN2*Coi + CN1*Cot)      
         Tc_P = (CN1*Coi - CN2*Cot)/(CN1*Coi + CN2*Cot)
         IF( Nvs == 1 ) THEN
            TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
            RETURN
         ENDIF
c        ... Fresnel reflection matrix including polarization
         TF0 = 0.d0
         TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
         TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
         TF0(3,3) = DBLE( Tc_L*CONJG(Tc_P) )   ;  TF0(4,4) = TF0(3,3)
         TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
         RETURN
      END SUBROUTINE Fre_Ra_v
c **********************************************************************

      SUBROUTINE Fre_Rw_v
      Coi = Cos_r
      Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Coi*Coi)/CN2/CN2 )
c     ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
      Tc_L = (CN2*Coi - CN1*Cos_t)/(CN2*Coi + CN1*Cos_t)      
      Tc_P = (CN1*Coi - CN2*Cos_t)/(CN1*Coi + CN2*Cos_t)
      IF( Nvs == 1 ) THEN
         TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         RETURN
      ENDIF
c        ... Fresnel reflection matrix including polarization
      TF0 = 0.d0
      TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
      TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
      TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
      TF0(3,3) = DBLE ( Tc_L*CONJG(Tc_P) )  ;  TF0(4,4) = TF0(3,3)
      TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
      RETURN
      END SUBROUTINE Fre_Rw_v
c **********************************************************************

      SUBROUTINE ROT_RL(Co_sc, Si_sc, Co_in, Si_in, Co_s)
      REAL (KND), INTENT(IN) :: Co_sc, Si_sc, Co_in, Si_in, Co_s
      REAL (KND) Si_s
c    The relative azimuth angle always 0 < rel_azm < pi
cc     ... 0 < phi_sca - phi_inc < pi
c      Zn = -1.d0
cc     ... pi < phi_sca - phi_inc < 2*pi
c      IF( Azm > Pie )  Zn = 1.d0

c     ... Convention about positive rotation angle:
c         rotation clockwise when looking into direction 
c          of propagation
c        | 1     0        0     0 |
c        | 0  cos(2a)  -sin(2a) 0 |
c        | 0  sin(2a)   cos(2a) 0 |
c        | 0     0        0     1 |
c        Sine of scattering angle
      Si_s = SQRT(1.d0 - Co_s*Co_s)
      COSIG: Block
      IF( REAL(ABS(Co_sc)) == 1.0.AND.REAL(ABS(Co_in)) == 1.0 ) THEN
         T2 = 1.d0 ; T1 = 1.d0 
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Co_sc)) == 1.0 ) THEN
         T2 = -SIGN(1.d0,Co_sc)*CPHI ; T1 = 1.d0
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Co_in)) == 1.0 ) THEN
         T1 = -SIGN(1.d0,Co_in)*CPHI ; T2 = 1.d0
         EXIT COSIG
      ENDIF
      T1 = (Co_sc - Co_in*Co_s)/Si_in/Si_s ! cos( sigma_1 )
      T2 = (Co_in - Co_sc*Co_s)/Si_sc/Si_s ! cos( sigma_2 )
      END Block COSIG

      Rot_r(2,2) =  2.d0*T1*T1 - 1.d0          ! 2*cos^2( 2*sigma_1 ) - 1
      Rot_r(3,2) = -2.d0*DSQRT( DABS(1.d0 - T1*T1) )*T1 ! Zn set to -1.d0
      Rot_r(2,3) = -Rot_r(3,2) ; Rot_r(3,3) =  Rot_r(2,2)

      Rot_l(2,2) =  2.d0*T2*T2 - 1.d0          ! 2*cos^2( 2*sigma_2 ) - 1
      Rot_l(3,2) = -2.d0*DSQRT( DABS(1.d0 - T2*T2) )*T2 ! Zn set to -1.d0
      Rot_l(2,3) = -Rot_l(3,2) ; Rot_l(3,3) =  Rot_l(2,2)
 
      TF0 = MATMUL(Rot_l,MATMUL(TF0,Rot_r))      
      END SUBROUTINE ROT_RL
      
c **********************************************************************
      SUBROUTINE CHECK_RT
      USE WATER,   ONLY : Ref_ind

      REAL (KND) F_ref, F_tra, SW(NSOL), Fr1(NSOL), SA(NSOL),
     $     O_Raa(NSTR,NSTR+NSOL), O_Rww(NSTR_W,NSTR_W),
     $     O_Taw(NSTR_W,NSTR+NSOL), O_Twa(NSTR,NSTR_W),
     $     Raa_m(NSTR), Rww_m(NSTR_W), Tao_m(NSTR), Toa_m(NSTR_W)

c     ... Integration over azimuthal angle "air->air" (reflection)
      DO I = 1, NSTR
         DO K = 1, NSTR+NSOL
            O_Raa(I,K) = SUM( Ref_AA(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
c     ... Reflected flux:
      F_ref = SUM( O_Raa(1:NSTR,NSTR+NSOL)*AXM(:) )
c      F_ref = SUM( O_Raa(1:NSTR,NSTR)*AXM(:) )

      WRITE(*,*) '   Refl_aa NCOL = ', NSTR+NSOL+1,
     $     ' file: DIR_OUT//"/refl_aa.dat"'
      OPEN( 44,file=DIR_OUT//'/refl_aa.dat' )
      DO I = 1,NSTR
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSR(I))*RAD_TO_DEG, (O_Raa(I,K), K = 1,NSTR+NSOL)
      ENDDO
      CLOSE(44)
      
c     ... Integration over azimuth "air->water" (transmission)  
      DO I = 1,NSTR_W
         DO K = 1, NSTR+NSOL
            O_Taw(I,K) = SUM( TRA_AW(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
c     ... Transmitted flux:
      F_tra = SUM( O_Taw(1:NSTR_W,NSTR+NSOL)*AXM_W(:) )
c      F_tra = SUM( O_Taw(1:NSTR_W,NSTR)*AXM_W(:) )
      WRITE(*,*) ' Comparison of the incident, refle. and trans. fluxes'
      WRITE(*,'(4(2x,1Pe12.4))') 
     $     DABS(XSOL(1)), F_ref, F_tra, F_ref + F_tra

      SA(:) = DABS(XSOL(1))
      WRITE(*,*) '    XSOL in atm:', SA(:)
      SW(:) =  DSQRT(SA(:)**2 + Ref_ind**2 - 1.d0)/Ref_ind
      WRITE(*,*) '    XSOL in wat:', SW(1)
      WRITE(*,*) '     Ref_ind   :',Ref_ind
      Fr1(:) = 0.5d0*
     $     (( (SA(:) - Ref_ind*SW(:))/(SA(:) + Ref_ind*SW(:)) )**2
     $     + ( (SW(:) - Ref_ind*SA(:))/(SW(:) + Ref_ind*SA(:)) )**2)
      WRITE(*,*) '     Fresnel Ref;   :',Fr1(:)
      WRITE(*,*) ' Comparison of the transmitted fluxes'
      WRITE(*,'(4(2x,1Pe12.4))') 
     $     DABS(XSOL(1)), F_tra, SA(1)/SW(1)*(1.d0 - Fr1(1))*SW(1)

            WRITE(*,*) '   Tra_aw NCOL = ',NSTR+NSOL+1,
     $     '   File : "./DATA_OUT/tra_aw.dat"'
      OPEN( 44,file=DIR_OUT//'/tra_aw.dat' )
      DO I = 1,NSTR_W
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, (O_Taw(I,K), K = 1,NSTR+NSOL)
      ENDDO
      CLOSE(44)
      
c     ... Unidirectional reflectivity and transmissivity (air->water)      
      DO K = 1,NSTR
         Raa_m(K) = SUM( O_Raa(1:NSTR,K)*AXM(:)  )/COSI(K)
         Tao_m(K) = SUM( O_Taw(:,K)*AXM_W(:))/COSI(K)
      ENDDO
c     ... Output to test of the deficiency (reflection-transmission)
      WRITE(*,*) '   File : "./DATA_OUT/ref_tra_aw.dat"'
      OPEN( 44,file=DIR_OUT//'/ref_tra_aw.dat' )
      DO K = 1,NSTR
         WRITE(44,'(2x,f7.3,3(2x,e12.4E3))') 
     $        DACOS(COSI(K))*RAD_TO_DEG, Raa_m(K), Tao_m(K),
     $        1.d0 - Raa_m(K) - Tao_m(K)
      ENDDO
      CLOSE(44)

c     ******************************************************************
c          ************* water => atmosphere ****************
c     ******************************************************************
c     ... Integration over azimuthal angle "water->water" (reflection)
      DO I = 1, NSTR_W
         DO K = 1, NSTR_W
            O_Rww(I,K) = SUM( Ref_WW(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
      WRITE(*,*) '   Refl_ww NCOL = ',NSTR_W + 1,
     $     ' file: "./DATA_OUT/refl_ww.dat"'
      OPEN( 44,file=DIR_OUT//'/refl_ww.dat' )
      DO I = 1,NSTR_W
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, (O_Rww(I,K), K = 1,NSTR_W)
      ENDDO
      CLOSE(44)

c      Integration over azimuth "water=>air"  (transmission)    
      DO I = 1,NSTR
         DO K = 1, NSTR_W
            O_Twa(I,K) = SUM( TRA_WA(I,K,:,1)*Wei_phi(:) )*2.d0*Pie
         ENDDO
      ENDDO
      WRITE(*,*) '   Tra_wa NCOL = ',NSTR_W + 1,
     $     '   File : "./DATA_OUT/tra_wa.dat"'
      OPEN( 44,file = DIR_OUT//'/tra_wa.dat' )
      DO I = 1,NSTR
         WRITE(44,'(2x,F7.3,100(1x,e12.4E3))') 
     $        DACOS(COSI(I))*RAD_TO_DEG, (O_Twa(I,K), K = 1,NSTR_W)
      ENDDO
      CLOSE(44)
      
c     ... Unidirectional reflectivity and transmissivity (water->air)
      DO K = 1,NSTR_W
         Rww_m(K) = SUM( O_Rww(:,K)*AXM_W(:) )/COSRw(K)
         Toa_m(K) = SUM( O_Twa(:,K)*AXM(:) )  /COSRw(K)
      ENDDO
c     ... Output to test of the deficiency (reflection-transmission)
      WRITE(*,*) '   File : "./DATA_OUT/ref_tra_wa.dat"'
      OPEN( 44,file=DIR_OUT//'/ref_tra_wa.dat' )
      DO K = 1,NSTR_W
         WRITE(44,'(2x,f7.3,4(2x,e12.4E3))') 
     $        DACOS(COSRw(K))*RAD_TO_DEG, Rww_m(K), Toa_m(K),
     $        1.d0 - Rww_m(K) - Toa_m(K)
      ENDDO
      CLOSE(44)
      STOP '   STOP in "check_rt.f" within "trans_refl_matr_v.f""'
      END SUBROUTINE CHECK_RT
c **********************************************************************

      SUBROUTINE CHECK_FOURIER
      INTEGER Nphi
      REAL (KND) S_Res(NSTR+1,NSOL), T_res(NSTR_W+1,NSOL)

c     Output azimuthal angle
      Nphi =  1
      DO I = 1,NSOL
         S_res(:,I) = Ra_GS_WL(:,I,1,1)
         T_res(:,I) = Taw_GS_WL(:,I,1,1)
         DO Mh = 2,M_max        ! do-loop over Fourier components
            DM = DCOS(DFLOAT(Mh-1)*X_phi(Nphi))
            S_res(:,I) = S_res(:,I) + 2.d0* Ra_GS_WL(:,I,Mh,1)*DM
            T_res(:,I) = T_res(:,I) + 2.d0*Taw_GS_WL(:,I,Mh,1)*DM
         ENDDO
      ENDDO      

      WRITE(*,*) '    Fourier expansion for azimuthal angle:'
      WRITE(*,'(10x,F7.3)') X_phi(Nphi)*RAD_TO_DEG
      OPEN( 44,file=DIR_OUT//'/Raa_azm.dat' )
      DO I = 1,NSTR+1
         WRITE(44,'(2x,F7.3,10(1x,e12.4E3))') 
     $        DACOS(COSR(I))*RAD_TO_DEG, 
     $        (S_res(I,K), Ref_AA(I,NSTR+K,Nphi,1), K = 1,NSOL)
      ENDDO
      CLOSE(44)

      OPEN( 44,file=DIR_OUT//'/Taw_azm.dat' )
      DO I = 1,NSTR_W+1
         WRITE(44,'(2x,F7.3,10(1x,e12.4E3))') 
     $        DACOS(COSRw(I))*RAD_TO_DEG, 
     $        (T_res(I,K), Tra_AW(I,NSTR+K,Nphi,1), K = 1,NSOL)
      ENDDO
      CLOSE(44)
c      PAUSE '   Pause in "check_fourier.f" '
      END SUBROUTINE CHECK_FOURIER

      END
c ****************************************************************************

      SUBROUTINE Fre_Taw_v(Nst, FTv, Cos_i, Ref_idx)

      USE INTERN_DIMS, ONLY: Nvs
      USE TYPES,       ONLY: KND, DCMP
c     ------------------------------------------------------------------
      INTEGER,           INTENT(IN) :: Nst
      REAL (KND),        INTENT(IN) :: Ref_idx(2), Cos_i(Nst)
      REAL (KND)  FTv(Nst,Nvs,Nvs)
c     ... Local variables
      INTEGER I
      REAL (KND)  Coi, TF0(4,4)
      COMPLEX (kind(0.0d0)) :: CN1, CN2
      COMPLEX (kind(0.0d0)) :: Tc_L, Tc_P, Cos_t
c     ==================================================================

c     ... Refractive indexes of the air  
      CN1 = DCMP( 1.d0 ) 
c     ... Refractive indexes of the water  
      CN2 = DCMP( Ref_idx(1),Ref_idx(2) )  
      DO I = 1, Nst
         Coi = Cos_i(I)
c        ... Cosine of the refracted angle
         Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Coi*Coi)/CN2/CN2 )
c        ... Parallel (T_L) and perpendicular (T_P) Fresnel
c            transmission coefficients      
         Tc_L = 2.d0*CDSQRT(CN1*CN2*Coi*Cos_t) /(CN2*Coi + CN1*Cos_t)      
         Tc_P = 2.d0*CDSQRT(CN1*CN2*Coi*Cos_t) /(CN1*Coi + CN2*Cos_t)
         IF( Nvs == 1 ) THEN
            FTv(I,1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
            CYCLE
         ENDIF
c        ... Fresnel transmission matrix including polarization
         TF0 = 0.d0
         TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
         TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
         TF0(3,3) = DBLE( Tc_L*CONJG(Tc_P) )   ;  TF0(4,4) = TF0(3,3)
         TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
         FTv(I,1:4,1:4) = TF0(:,:)
      ENDDO  ! over incident angles (I)
      
      RETURN
      END SUBROUTINE Fre_Taw_v      
c **********************************************************************
