C********************************************************************C
C
C	SCIATRAN - Version 4.3 - Last modification: 24 Dec 2020
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE TRM_LOS(N_INC, N_REF, N_AZM, COSI, COSR, X_phi, R_ind,
     $     Ref_AA, Tra_AW) 
c     ... Called by "dom_spherical.f"

      USE BRDF,        ONLY: Lin_rel
      USE INTERN_DIMS, ONLY: Nvs
      USE LOG_PARS,    ONLY: USE_SHAD
      USE GT_POLYN,    ONLY: XM_W
      USE ST_VARS,     ONLY: Pie
      USE TYPES,       ONLY: KND, DCMP
      USE WATER,       ONLY: NSTR_W, W_speed
c     ------------------------------------------------------------------

      INTEGER,    INTENT(IN) :: N_INC, N_REF, N_AZM
      REAL (KND), INTENT(IN) :: COSI(N_INC), COSR(N_REF), X_phi(N_AZM),
     $     R_ind(2)
      REAL (KND) Ref_AA(Nvs*N_REF,Nvs*N_INC,N_AZM),
     $     Tra_AW(Nvs*N_INC,Nvs*N_INC,1)
c     Local variables
c     ---------------
      CHARACTER (LEN=*), PARAMETER :: R_NAME='TRM_LOS'
      LOGICAL  Vect
      INTEGER  I, J, K, N_REFw, Nvi, Nvk
      REAL (KND)  Mr, Mri, CPHI, G, SQS, SQPI, SIGMA, Hsqr, Mut, Rd,
     $     Cot, Coi, Tfr, Zn1, Zn2, S1, DM, Mur, Cos_r, Scat_r, Td,
     $     Scat_t, Arg, ERF, Def, T1, T2, Rfr_W, Zn
      REAL (KND), DIMENSION (N_REF)   :: SINR,  SHR
c      REAL (KND), DIMENSION (NSTR_W)  :: COSRw, SINRw, SHRw
      REAL (KND), DIMENSION (N_INC)  :: COSRw, SINRw, SHRw
      REAL (KND), ALLOCATABLE :: SINI(:), SHI(:),  Rot_l(:,:), 
     $     Rot_r(:,:)
!      REAL (KND), ALLOCATABLE ::  Ref_WW(:,:,:), Tra_WA(:,:,:), Tra_AW(:,:,:)
      REAL (KND)  TF0(Nvs,Nvs)
      COMPLEX (kind(0.0d0)) :: CN1, CN2, Tc_L, Tc_P, Cos_t
      EXTERNAL ERF
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      Vect = .FALSE.
      IF( Nvs > 1 ) THEN
         ALLOCATE( Rot_l(4,4), Rot_r(4,4) )
c     ... Initialization of rotational matrices
         Rot_l = 0.d0       ;  Rot_r = 0.d0
         Rot_l(1,1) = 1.d0  ;  Rot_r(1,1) = 1.d0
         Rot_l(4,4) = 1.d0  ;  Rot_r(4,4) = 1.d0
         Vect = .TRUE.
      ENDIF
c      N_REFw = NSTR_W 
      N_REFw = N_INC 

c     ... Local allocatable arrays
      ALLOCATE( SINI(N_INC), SHI(N_INC)
c     $     ,Tra_AW(Nvs*NSTR_W, Nvs*N_INC,  N_AZM), 
c     $     Tra_WA(Nvs*N_REF,  Nvs*NSTR_W, N_AZM),
c     $     Ref_WW(Nvs*NSTR_W, Nvs*NSTR_W, N_AZM)
     $     )

      SINR(:) = DSQRT(1.D0 - COSR(:)**2)
c     ... Reflection water => water
c      COSRw(1:NSTR_W) = XM_W(:)
      Mr = R_ind(1)  
      COSRw(:) = DSQRT( COSI(:)**2 + Mr**2 - 1.d0 )/Mr
      SINRw(:) = DSQRT(1.D0 - COSRw(:)**2)
      
      SINI(:) = DSQRT(1.D0 - COSI(:)**2)
      SIGMA = Lin_rel(1) + Lin_rel(2)*W_speed
c     ... Shadowing factor
      SELECT CASE(USE_SHAD)
      CASE('yes')
         SQS = DSQRT( SIGMA )  ;  SQPI = SQS/SQRT(Pie) ; SHR = 0.d0
         DO I = 1, N_REF
            IF( SINR(I) <= 0.0d0 ) CYCLE
            G = COSR(I)/SINR(I) ; Arg = G/SQS
            SHR(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
         ENDDO
         SHI = 0.d0
         DO I = 1, N_INC
            IF( SINI(I) <= 0.0d0 ) CYCLE
            G = COSI(I)/SINI(I) ; Arg = G/SQS
            SHI(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
         ENDDO
         SHRw = 0.d0
         DO I = 1, N_REFw
            IF( SINRw(I) <= 0.0d0 ) CYCLE
            G = COSRw(I)/SINRw(I) ; Arg = G/SQS
            SHRw(I) = 0.5D0*(DEXP(-G*G/SIGMA)*SQPI/G - 1.d0 + ERF(Arg))
         ENDDO
      CASE('no')
         SHI = 0.d0 ; SHR = 0.d0 ; SHRw = 0.d0
      END SELECT

      Mr = R_ind(1)  ;  Mri = 1.d0/Mr
c     =================================================================
c           Transmission air => water
c     =================================================================
c     ... Complex refractive indexe of air and water
      CN1 = DCMP( 1.d0 ) ; CN2 = DCMP( R_ind(1),R_ind(2) )  
c      DO J = 1, N_AZM
c         CPHI = DCOS(X_phi(J))
c         DO I = 1, NSTR_W    ! Angles for transmitted radiation (in water)
      DO J = 1, 1
         CPHI = 1.0d0
         Zn = -1.0d0
         DO I = 1, N_INC  ! Angles for transmitted radiation (in water)
            Nvi = Nvs*(I-1)
            DO K = 1, N_INC     ! Angles for incident radiation (in air)
               Nvk = Nvs*(K-1)
c              ... Shadowing factor
               G = 1.d0/(1.d0 + SHRw(I) + SHI(K))
c                 ... Cosine of scat. angle between incident and transmitted
               Scat_t = COSRw(I)*COSI(K) + SINRw(I)*SINI(K)*CPHI
c                 ... Normalization factor (see Eq.(C-fact) in STB)
               Hsqr  = DSQRT( Mr**2 - 2.d0*Mr*Scat_t + 1 )
c                 ... Cosine between local normal and z-axis
c                     ( see Eq.(Mun-aw) in STB )
               Mut = (Mr*COSRw(I) - COSI(K))/Hsqr
c                 ... Cosine between transmitted and local normal
c                     ( see Eq.(loc-cot-aw) in STB )
               Cot = (Scat_t - Mr)/Hsqr
c                  Cot = -(Scat_t - Mr)/Hsqr
c                 ... Cosine between incident and local normal
c                      ( see Eq.(loc-coi-aw) in STB )
               Coi = (1.d0 - Mr*Scat_t)/Hsqr
c                  Coi = -(1.d0 - Mr*Scat_t)/Hsqr
               IF( (Scat_t - 1.d0/Mr) >= 0.d0 .AND.
     $              (Mr*COSRw(I) - COSI(K)) >= 0.d0) THEN 
c                 ... Fresnel transmission matrix or coefficient:
c                     TF0(4,4) or TF0(1,1), respectively
                  CALL Fre_Tr_v
                  IF(Vect) CALL ROT_RL(-COSRw(I), SINRw(I),
     $                 -COSI(K), SINI(K), Scat_t)
                  Mut = Mut*Mut
                  Tra_AW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J)
     $                 = (Mr**2*Cot*Coi)
     $                 /(COSRw(I)*(Coi-Mr*Cot)**2*Pie*Mut**2*SIGMA)
     $                 *DEXP(-(1.d0-Mut)/SIGMA/Mut)*G*TF0(1:Nvs,1:Nvs)
               ELSE
                  Tra_AW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J) = 0.d0
               ENDIF
            ENDDO  ! over incident in air (K)
         ENDDO  ! over transmitted in water (I)
      ENDDO  ! over azimuth (J)
c         stop
cc     ========================================================
cc               Transmission water => air
cc     =======================================================
cc        ... Complex refractive indexe of water and air
c      CN1 = DCMP( R_ind(1),R_ind(2) )  ;  CN2 = DCMP( 1.d0 )
c      DO J = 1, N_AZM
c         CPHI = DCOS(X_phi(J))
c         DO I = 1, N_REF
c            Nvi = Nvs*(I-1)
c            DO K = 1, NSTR_W    ! Angles for incident radiation (in water)
c               Nvk = Nvs*(K-1)
cc                 ... Shadowing factor
c               G = 1.d0/(1.D0 + SHR(I) + SHRw(K))
cc                 ... Cosine of scat. angle between incident and transmitted
c               Scat_t = COSR(I)*COSRw(K) + SINR(I)*SINRw(K)*CPHI
cc                 ...  Normalization factor
c               Hsqr  = DSQRT( Mri**2 - 2.d0*Mri*Scat_t + 1 )
cc                 ... Cosine between local normal and z-axis
c               Mut = (COSRw(K) - Mri*COSR(I))/Hsqr
cc                 ... Cosine between transmitted and local normal
cc                     ( see Eq.(loc-cot-wa) in STB )
c               Cot = (Scat_t - Mri)/Hsqr
cc                 ... Cosine between incident and local normal
cc                     ( see Eq.(loc-coi-wa) in STB )
c               Coi = (1.d0 - Mri*Scat_t)/Hsqr
c
c               IF( (Scat_t - 1.d0/Mr) >= 0.d0 .AND.
c     $              (COSRw(K) - Mri*COSR(I)) >= 0.d0) THEN 
cc                    ... Fresnel transmission coefficient
c                  Zn1 = 1.d0/(Mri*Coi + Cot)**2
c                  Zn2 = 1.d0/(Coi + Mri*Cot)**2 
c                  Tfr = 2.d0*Mri*Cot*Coi*(Zn1 + Zn2)
c                  CALL Fre_Tr_v
c                  IF(Vect) CALL ROT_RL(COSR(I), SINR(I),
c     $                 COSRw(K), SINRw(K), Scat_t)
c                  Mut = Mut*Mut
c                  Tra_WA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J)
c     $                 = (Mri**2*Cot*Coi)
c     $                 /(COSR(I)*(Coi-Mri*Cot)**2*Pie*Mut**2*SIGMA)
c     $                 *DEXP(-(1.d0-Mut)/SIGMA/Mut)*G*TF0(1:Nvs,1:Nvs)
c               ELSE
c                  Tra_WA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J) = 0.d0
c               ENDIF
c            ENDDO  ! over incident in water (K)
c         ENDDO  ! over transmitted in air (I)
c      ENDDO  ! over azimuth (J)

c     ========================================================
c            Reflection air => air  &  water => water
c     ========================================================
c        ... Complex refractive indexe of air and water
      CN1 = DCMP( 1.d0 ) ; CN2 = DCMP( R_ind(1),R_ind(2) )  
      DO J = 1, N_AZM
         CPHI = DCOS(X_phi(J))
         IF( X_phi(J) > Pie ) THEN 
            Zn = 1.0d0
         ELSE
            Zn = -1.0d0
         ENDIF         
c        ======================================
c             Reflection air => air 
c        ======================================
         DO I = 1, N_REF     ;  Nvi = Nvs*(I-1)
            DO K = 1, N_INC  ;  Nvk = Nvs*(K-1)
c              ... Shadowing factor
               G = 1.d0/(1.d0 + SHR(I) + SHI(K))
c              ... Cosine of scat. angle between incident and reflected
               Scat_r = -COSR(I)*COSI(K) + SINR(I)*SINI(K)*CPHI
c              ... Cosine between local normal and z-axis
c                  ( see STB Eq.(Ang-Mun) )
               Mur = (COSR(I) + COSI(K))/DSQRT(2.d0*(1.d0-Scat_r))
c              ... Cosine between reflected and local normal
c                  ( see STB Eq.(Cos-lock) )
               Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )
c                     corresponding refracted angle
               S1 = DSQRT(Cos_r**2 + Mr**2 - 1.d0)/Mr
               CALL Fre_Ra_v
               IF(Vect) CALL ROT_RL(COSR(I), SINR(I),
     $              -COSI(K), SINI(K), Scat_r)
c                 ... Reflectivity
               Mur = Mur*Mur
               Ref_AA(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J)
     $              = TF0(1:Nvs,1:Nvs)/(4.d0*COSR(I)*Pie*Mur**2)
c                   ... Exclude PDF and shadowing to compare with brdf case
     $              *DEXP(-(1.d0-Mur)/SIGMA/Mur)/SIGMA*G
            ENDDO  ! over incidence angles (K)
         ENDDO  ! over reflection angles (I)
      ENDDO  ! over azimuthal angles (J)
cc        ===========================
cc        Reflection water => water
cc        ===========================
cc        ... Complex refractive indexe of water and air
c      CN1 = DCMP( R_ind(1),R_ind(2) )  ;  CN2 = DCMP( 1.d0 )
c      DO J = 1, N_AZM
c         CPHI = DCOS(X_phi(J))
c         DO I = 1, N_REFw       ;  Nvi = Nvs*(I-1)
c            DO K = 1, N_REFw   ;  Nvk = Nvs*(K-1)
cc              ... Shadowing factor
c               G = 1.d0/(1.D0 + SHRw(I) + SHRw(K))
cc              ... Cosine of scat. angle between incident and reflected
c               Scat_r = -COSRw(I)*COSRw(K) + SINRw(I)*SINRw(K)*CPHI
cc              ... Cosine between local normal and z-axis
c               Mur = (COSRw(I) + COSRw(K))/DSQRT(2.d0*(1.d0-Scat_r))
cc              ... Cosine between reflected and local normal
c               Cos_r = DSQRT( (1.d0-Scat_r)/2.d0 )
c               IF( Scat_r - (2.d0*Mri**2 - 1.d0) >= 0.d0 ) THEN
c                  Rfr_w = 1.d0
c               ELSE
cc                 ... corresponding refracted angle
c                  S1 = DSQRT(Cos_r**2 + Mri**2 - 1.d0)/Mri
cc                 ... Fresnel reflection coefficient
c                  Zn1 = (Mri*Cos_r+S1)**2 ; Zn2 = (Cos_r+Mri*S1)**2 
c                  Rfr_w = 0.5d0*( (Mri*Cos_r-S1)**2/Zn1
c     $                 + (Cos_r-Mri*S1)**2/Zn2 )
c               ENDIF
c               CALL Fre_Rw_v
c               IF(Vect) CALL ROT_RL(-COSRw(I), SINRw(I),
c     $              COSRw(K), SINRw(K), Scat_r)
cc                 ... Reflection matrix
c               Mur = Mur*Mur
c               Ref_WW(Nvi+1:Nvi+Nvs,Nvk+1:Nvk+Nvs,J) = 1.d0/
c     $              (4.d0*COSRw(I)*Pie*Mur**2*SIGMA)
c     $              *DEXP(-(1.d0-Mur)/SIGMA/Mur)*G*TF0(1:Nvs,1:Nvs)
c            ENDDO  ! over (K)
c         ENDDO  ! over angles in ocean (I)
c      ENDDO  ! over azimuthal angles (J)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      CONTAINS
c **********************************************************************
      SUBROUTINE Fre_Tr_v
c     ... Parallel (T_L) and perpendicular (T_P) Fresnel
c         transmission coefficients      
      Tc_L = 2.d0*CDSQRT(CN1*CN2*Coi*Cot) /(CN2*Coi + CN1*Cot)      
      Tc_P = 2.d0*CDSQRT(CN1*CN2*Coi*Cot) /(CN1*Coi + CN2*Cot)
      IF( Nvs == 1 ) THEN
         TF0 (1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         RETURN
      ENDIF
c     ... Fresnel transmission matrix including polarization
      TF0 = 0.d0
      TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
      TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
      TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
      TF0(3,3) = DBLE( Tc_L*CONJG(Tc_P) )   ;  TF0(4,4) = TF0(3,3)
      TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
      RETURN
      END SUBROUTINE Fre_Tr_v      
c **********************************************************************
      SUBROUTINE Fre_Ra_v
         Coi = Cos_r ;  Cot = S1
c        ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
         Tc_L = (CN2*Coi - CN1*Cot)/(CN2*Coi + CN1*Cot)      
         Tc_P = (CN1*Coi - CN2*Cot)/(CN1*Coi + CN2*Cot)
         IF( Nvs == 1 ) THEN
            TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
            RETURN
         ENDIF
c        ... Fresnel reflection matrix including polarization
         TF0 = 0.d0
         TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
         TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
         TF0(3,3) = DBLE( Tc_L*CONJG(Tc_P) )   ;  TF0(4,4) = TF0(3,3)
         TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
         RETURN
      END SUBROUTINE Fre_Ra_v
c **********************************************************************

      SUBROUTINE Fre_Rw_v
      Coi = Cos_r
      Cos_t = CDSQRT( 1.d0 - CN1*CN1*(1.d0 - Coi*Coi)/CN2/CN2 )
c     ... Parallel (R_L) and perpendicular (R_P) Fresnel
c            reflection coefficients      
      Tc_L = (CN2*Coi - CN1*Cos_t)/(CN2*Coi + CN1*Cos_t)      
      Tc_P = (CN1*Coi - CN2*Cos_t)/(CN1*Coi + CN2*Cos_t)
      IF( Nvs == 1 ) THEN
         TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
         RETURN
      ENDIF
c        ... Fresnel reflection matrix including polarization
      TF0 = 0.d0
      TF0(1,1) = 0.5d0*(CDABS(Tc_L)**2 + CDABS(Tc_P)**2) 
      TF0(1,2) = 0.5d0*(CDABS(Tc_L)**2 - CDABS(Tc_P)**2)
      TF0(2,1) = TF0(1,2) ; TF0(2,2) = TF0(1,1)
      TF0(3,3) = DBLE ( Tc_L*CONJG(Tc_P) )  ;  TF0(4,4) = TF0(3,3)
      TF0(3,4) = AIMAG( Tc_L*CONJG(Tc_P) )  ;  TF0(4,3) = -TF0(3,4)
      RETURN
      END SUBROUTINE Fre_Rw_v
c **********************************************************************

      SUBROUTINE ROT_RL(Co_sc, Si_sc, Co_in, Si_in, Co_s)
      REAL (KND), INTENT(IN) :: Co_sc, Si_sc, Co_in, Si_in, Co_s
      REAL (KND) Si_s
c    The relative azimuth angle always 0 < rel_azm < pi
cc     ... 0 < phi_sca - phi_inc < pi
c      Zn = -1.d0
cc     ... pi < phi_sca - phi_inc < 2*pi
c      IF( Azm > Pie )  Zn = 1.d0

c     ... Convention about positive rotation angle:
c         rotation clockwise when looking into direction 
c          of propagation
c        | 1     0        0     0 |
c        | 0  cos(2a)  -sin(2a) 0 |
c        | 0  sin(2a)   cos(2a) 0 |
c        | 0     0        0     1 |
c        Sine of scattering angle
      Si_s = SQRT(1.d0 - Co_s*Co_s)
      COSIG: Block
      IF( REAL(ABS(Co_sc)) == 1.0.AND.REAL(ABS(Co_in)) == 1.0 ) THEN
         T2 = 1.d0 ; T1 = 1.d0 
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Co_sc)) == 1.0 ) THEN
         T2 = -SIGN(1.d0,Co_sc)*CPHI ; T1 = 1.d0
         EXIT COSIG
      ENDIF
      IF( REAL(ABS(Co_in)) == 1.0 ) THEN
         T1 = -SIGN(1.d0,Co_in)*CPHI ; T2 = 1.d0
         EXIT COSIG
      ENDIF
      T1 = (Co_sc - Co_in*Co_s)/Si_in/Si_s ! cos( sigma_1 )
      T2 = (Co_in - Co_sc*Co_s)/Si_sc/Si_s ! cos( sigma_2 )
      END Block COSIG

      Rot_r(2,2) =  2.d0*T1*T1 - 1.d0          ! 2*cos^2( 2*sigma_1 ) - 1
c      Rot_r(3,2) = -2.d0*DSQRT( DABS(1.d0 - T1*T1) )*T1 ! Zn set to -1.d0
      Rot_r(3,2) = Zn*2.d0*DSQRT( DABS(1.d0 - T1*T1) )*T1 ! Zn set to -1.d0
      Rot_r(2,3) = -Rot_r(3,2) ; Rot_r(3,3) =  Rot_r(2,2)

      Rot_l(2,2) =  2.d0*T2*T2 - 1.d0          ! 2*cos^2( 2*sigma_2 ) - 1
c      Rot_l(3,2) = -2.d0*DSQRT( DABS(1.d0 - T2*T2) )*T2 ! Zn set to -1.d0
      Rot_l(3,2) = Zn*2.d0*DSQRT( DABS(1.d0 - T2*T2) )*T2 ! Zn set to -1.d0
      Rot_l(2,3) = -Rot_l(3,2) ; Rot_l(3,3) =  Rot_l(2,2)
 
      TF0 = MATMUL(Rot_l,MATMUL(TF0,Rot_r))      
      END SUBROUTINE ROT_RL
      
      END 

c **********************************************************************
