!**************************************************************************
!
!       SCIATRAN - Version 4.5 - Last modification: 23 Jul 2021
!
!       Copyright (C) - Institute of Remote Sensing,
!                       University of Bremen, Germany
!
!**************************************************************************
MODULE TYPES
  INTEGER, PARAMETER :: KND = kind(0.0d0)
  
  TYPE DYN_CHAR          
     CHARACTER (LEN = :), ALLOCATABLE :: S
  END TYPE DYN_CHAR
  TYPE, EXTENDS (DYN_CHAR) :: DYN_CHAR_L
     INTEGER L
     Character (Len=4) Type
  END type DYN_CHAR_L
     
  TYPE NAME_TAG          
     CHARACTER (LEN = 20)  NAME
  END TYPE NAME_TAG

  TYPE , EXTENDS (NAME_TAG) :: NAME_TYPE          
     CHARACTER (LEN = 20)  TYPE
  END TYPE NAME_TYPE

  Type ArrChar
     TYPE (DYN_CHAR), ALLOCATABLE :: Arr(:)
  End Type ArrChar

  TYPE ADO_type
     REAL (KND), POINTER :: FLUX_D(:), LVEC(:,:), &
          Part_TU(:,:,:), Part_TD(:,:,:), Aj_ls(:,:,:),&
          Bj_ls(:,:,:), Ajs(:,:,:), Bjs(:,:,:), Afi(:,:,:), &
          Aj_sph(:,:,:,:,:), Bj_sph(:,:,:,:,:)
  END TYPE ADO_type

  TYPE AGR_type
     LOGICAL  ADAPTIVE_GRID
     INTEGER  NrL, NL, N_OBS, Nxl_1, Nxl_2
     INTEGER, POINTER :: LAY_GR(:,:)
  END TYPE AGR_type

  TYPE, EXTENDS (NAME_TAG) :: PH_MET
     REAL (KND), ALLOCATABLE ::  Egg(:,:,:), EgL(:,:,:), Egs(:,:,:), EsL(:,:,:)
  END TYPE PH_MET
  TYPE PH_AER
     REAL (KND), ALLOCATABLE ::  Egg(:,:,:,:), EgL(:,:,:,:), Egs(:,:,:,:), &
          EsL(:,:,:,:)
  END TYPE PH_AER
  TYPE PH_ADR
     REAL (KND), ALLOCATABLE ::  Egg(:,:,:,:,:), EgL(:,:,:,:,:), Egs(:,:,:,:,:), &
          EsL(:,:,:,:,:)
  END TYPE PH_ADR
  TYPE PH_CLD
     REAL (KND), ALLOCATABLE ::  Egg(:,:,:,:,:,:), EgL(:,:,:,:,:,:), &
          Egs(:,:,:,:,:,:), EsL(:,:,:,:,:,:)
  END TYPE PH_CLD
  TYPE PHA_MATR
     TYPE (PH_MET)  RAY
     TYPE (PH_AER)  AER
     TYPE (PH_ADR)  ADR
     TYPE (PH_CLD)  CLD
     TYPE (PH_MET), ALLOCATABLE :: MET(:)
  END type PHA_MATR

  TYPE UDB_TYPE  ! Boundary conditions
     LOGICAL  BRDF_FLAG, SPECULAR
     CHARACTER (LEN=11) LBC, UBC
     REAL (KND)  ALBEDO, Flu_int, ALB_FS
     REAL (KND), ALLOCATABLE :: Ref_wd(:)
     REAL (KND), POINTER :: BRF_GG(:,:,:), BRF_GS(:,:,:), Fre_R(:), Ewa_G(:), & 
          Eaw_G(:), Ra_GG(:,:,:), Taw_GG(:,:,:), Ra_GS(:,:,:), Taw_GS(:,:,:), &
          Rw_GG(:,:,:), Twa_GG(:,:,:), FoINC(:,:), Fre_Rv(:,:,:), Fre_Tv(:,:,:), &
          FR_sol(:,:,:), Der_GG(:,:,:,:), Der_GS(:,:,:,:), Fref(:,:,:),  &
          Ftra(:,:,:), Ref_AA_LOS(:,:,:)
  END TYPE UDB_type

  TYPE Eff_Sol_Idx
     INTEGER IA, IV, IS, IP, ILS 
  END type Eff_Sol_Idx
  TYPE SWF_type
     INTEGER, POINTER :: Ief_Sol(:), Ief_Sum(:)
     REAL (KND), POINTER :: T_fine(:), SOL_EXP(:,:),  & 
          SOL_JL(:,:,:), Egs_sph(:,:,:,:,:)
     REAL (KND), ALLOCATABLE :: DS_SUN_JL(:,:,:,:)
     TYPE(Eff_Sol_Idx), ALLOCATABLE :: IEffS(:,:)
  END TYPE SWF_type

  TYPE VRS_type
     LOGICAL, POINTER :: Act_WL(:), Exi_WL(:), USE_CONV(:)
     INTEGER  N_ACT, Jact, Jexi, N_EXT, N_AUX, Mh_OUT
     INTEGER, POINTER :: Isav(:)
     LOGICAL, POINTER :: Lo_VRS(:)
     REAL (KND), POINTER :: Tot_VRS(:), WL_EXT(:), V_FRF(:,:,:), EXT_AUX(:,:),&
          EXL_ACT(:,:,:), EXS_AUX(:,:,:)
     REAL (KND), DIMENSION(:,:,:,:), POINTER :: MVEC, NUJ_EXC
     REAL (KND), DIMENSION(:,:,:,:,:), POINTER :: GNm, ADJ_ACT, GNp, Aj, Bj
  END TYPE VRS_type
  
  TYPE PWF_type             ! Standard weighting functions
     REAL (KND), POINTER :: P_WF_ABSTERM(:), T_WF_ABSTERM(:), WF_esp(:,:,:), &
          WF_ssp(:,:,:,:)
  END TYPE PWF_type

  TYPE IOP_type
     INTEGER, ALLOCATABLE :: Id_a(:), Id_b(:)
     REAL (KND), ALLOCATABLE:: ALW(:), DTR(:), DTR_U(:), DTR_D(:), Ex_U(:,:), &
          Ex_D(:,:), MVEC(:,:), DFI(:,:,:), REF_DIR(:,:,:), REF_SOL(:,:,:) 
  END TYPE IOP_type

  TYPE , EXTENDS (NAME_TYPE) :: CROSS_SECTIONS          
     REAL (KND), ALLOCATABLE :: CS(:), E_p(:)
  END TYPE CROSS_SECTIONS 

  TYPE LOA_type             ! Optical parameters of atmosphere
     INTEGER     Cil, Cir, N1_CLD
     REAL (KND)  DP_CLD
     REAL (KND), POINTER :: SCR(:), RAY_MOMS(:,:), DM_CLD(:,:,:), SCA(:), &
          EXA(:), AS_A(:), DM_A(:), AER_MOMS(:,:,:), SCC(:), SCC_NOSCALE(:,:), &
          EXC(:), CLD_MOMS(:,:,:,:,:), DR_MOMS(:,:,:,:,:), ADR_MOMS(:,:,:,:), &
          AL(:), SCI(:), EXI(:), BET(:), EMC(:,:), CSM(:,:)
     REAL (KND), ALLOCATABLE :: EPF(:,:,:)
     TYPE (CROSS_SECTIONS), ALLOCATABLE :: TG_CS(:)
  END TYPE LOA_type

  TYPE PHA_CLD
     REAL (KND), DIMENSION(:,:,:,:,:,:), POINTER :: C_GG, C_GL, C_GS, Dr_GG, &
          Dr_GL, Dr_GS
     REAL (KND), DIMENSION(:,:,:,:,:), POINTER :: C_LS, Dr_LS
  END TYPE PHA_CLD

  TYPE LOW_type             ! Optical parameters of water
     REAL (KND), POINTER :: SCR_W(:), RAY_MOMS_W(:,:), HSCA(:,:,:),  &
          HEXT(:,:,:), AL_W(:), ABS_UW(:,:), BET(:), HSOL_MOMS(:,:,:,:,:)
  END TYPE LOW_type

  TYPE LWF_type             ! Local weighting functions
     REAL (KND), POINTER :: WF_ext(:,:,:), WF_TE(:,:), WF_TE_TEM(:,:), &
          WF_scr(:,:,:), WF_sca(:,:,:,:), WF_scc(:,:,:), DT1(:), DT2(:), &
          WF_met(:,:,:), WF_scm(:,:,:), DT2m(:), Dph_Dre(:,:,:), &
          SSC_DM(:,:,:)
  END TYPE LWF_type

  TYPE GSL_type             ! Angles 
     INTEGER NN, NSZA, NLSA
     REAL (KND), POINTER :: XM(:), AXM(:), AG(:), MuS(:), MuV(:), EXL(:,:)
     LOGICAL, POINTER :: AG_ZERO(:)
  END TYPE GSL_type

  TYPE SDO_type             ! Standard DOM auxiliary solution variables
     INTEGER, POINTER :: NcC(:), E_MIN(:)
     LOGICAL, POINTER :: MASK_NL(:)
     REAL (KND), POINTER :: Nuj_Lay(:,:), EXP_NU_NL(:,:), Ex_AG(:,:), &
          Aj_sol(:,:,:,:), Bj_sol(:,:,:,:), Ajp(:,:,:,:), Bjp(:,:,:,:), &
          Aj_spe(:,:,:,:), Bj_spe(:,:,:,:), G_NL(:,:,:), H1(:)
  END TYPE SDO_type

  TYPE LAS_TYPE             ! Linear algebraic system and related
     INTEGER, POINTER :: INDX(:)
     REAL (KND), POINTER :: Flux(:), Ref_surf(:,:), SOL_BOT(:,:), SOL_TOP(:,:),&
          PART_U(:,:,:), PART_D(:,:,:), SVEC_AG(:,:), MATTEB(:,:), RHS_LAS(:,:)
  END TYPE LAS_TYPE

  TYPE TE_type              ! Thermal emission
     LOGICAL  ADD_PLANCK, NO_SUN
     INTEGER, POINTER :: N_pla(:)
     REAL (KND)  PLANCK_SURF,  OPLANCK_SURF
     REAL (KND), POINTER :: Mup(:), PL_COEF(:,:), NPLANCK(:), &
          OPLANCK(:), Aj_pla(:,:)
     REAL (KND), ALLOCATABLE ::  PLA_FINE(:), B_FINE(:)
  END TYPE TE_type

  TYPE OSM_type             ! Original scattering matrices
     REAL (KND), POINTER :: AER_ALL(:,:,:),  DER_ALL(:,:,:,:), &
          AER_SCM(:,:,:), S_AN(:), CLD_LOC(:,:,:,:,:), &
          CLD_ALL(:,:,:,:,:), Der_CLD_ALL(:,:,:,:,:),  &
          Der_CLD_SCM(:,:,:,:,:), CLD_SCM(:,:,:,:,:)
  END TYPE OSM_type

  TYPE SSB_type             ! Spherical solar beam
     INTEGER, POINTER ::  N_sha(:), N_sph(:,:)
     REAL (KND), POINTER ::  Mue(:,:), EXS(:,:), SIN_COEF(:,:,:)
  END TYPE SSB_type

  TYPE PCC_type             ! Photochemistry
     REAL (KND), POINTER ::  PCX_COEFF(:,:,:), PCY_COEFF(:,:,:)
  END TYPE PCC_type

  TYPE RET_PARAMS          ! Retrieval parameter labels
     INTEGER N_GASES, N_PARS
     CHARACTER (LEN=10), POINTER ::  GASES(:,:), PARS(:,:)
     REAL (KND), POINTER ::  APR_P(:), APR_G(:), TR_P(:), TR_G(:)  
  END TYPE RET_PARAMS
  
  TYPE, EXTENDS (NAME_TAG) :: PREPROC_WF
     REAL (KND), ALLOCATABLE :: WF(:,:,:)
  END TYPE PREPROC_WF

  TYPE PREPROC_DATA
     LOGICAL, ALLOCATABLE :: MSK(:)
     REAL (KND), ALLOCATABLE :: MSMT(:,:), RTM(:,:), SY(:,:), SOL(:), WL(:)
     TYPE (PREPROC_WF), ALLOCATABLE :: PAR(:)
  END TYPE PREPROC_DATA

  TYPE LOCAL_DATA
     LOGICAL, ALLOCATABLE :: IDX_SP(:)
     REAL (KND), ALLOCATABLE :: MATR(:,:), WL(:), MSMT(:), SOL(:), WF(:,:), RTM(:)
  END TYPE LOCAL_DATA

  TYPE CORR_SPEC
     INTEGER N_CRS
     LOGICAL IEC_FLAG, TILT_FLAG
     CHARACTER (LEN = 1), ALLOCATABLE :: MODE(:)
     REAL (KND), ALLOCATABLE :: DAT(:,:), AMP(:)
  END TYPE CORR_SPEC

  TYPE SPLINE_INFO
     INTEGER NC, ORD
     DOUBLE PRECISION, ALLOCATABLE :: BCF(:), WRK(:), KNT(:)
  END type SPLINE_INFO

  TYPE LOS_GEOMETRY
     Logical Const_ang
     REAL (KND), ALLOCATABLE :: DS(:,:), DS_C(:), S_VA(:), C_VA(:),  &
          SVA_R1(:), SVA_R2(:), CVA_R(:), C_SZA(:), S_SZA(:), SAA(:)
     REAL (KND), POINTER :: CG_SZA(:) ! Geometrical SZA
  END TYPE LOS_GEOMETRY

  ! Line-of-sight veiwing geometry
  TYPE LOS_VIEW_GEOM  ! LVG array
     INTEGER  NLS, NTH, IALS, IALE
     LOGICAL  TH_FLG, FP_FLG, LS_FLG, REFR_FLG
     REAL (KND) H_0
     REAL (KND), POINTER :: CVA_TH, Th_V
     INTEGER, ALLOCATABLE :: ISEL(:), IOB(:)
     REAL (KND), ALLOCATABLE :: DS(:,:), C_VA(:), ZLS(:)
  END TYPE LOS_VIEW_GEOM

  TYPE DIR_SOL_GEOM ! DSB array
     INTEGER  NLS, NTH
     LOGICAL  TH_FLG, FP_FLG
     INTEGER, ALLOCATABLE :: ISEL(:), IOB(:)
     REAL (KND), ALLOCATABLE :: C_SZA(:), ZSOL(:), DS(:,:)
  END TYPE DIR_SOL_GEOM
  
  ! Line-of-sight solar geometry
  TYPE LOS_SOLAR_GEOM ! LSG array
     LOGICAL, ALLOCATABLE :: REFR_DSB(:)
     REAL (KND), ALLOCATABLE :: C_SZA(:), SAA(:), TH(:), &
          C_SA(:,:), S_SA(:,:), RML(:,:,:), RMR(:,:,:)
     REAL (KND), POINTER :: CSZA_TH, SAA_TH, CG_SZA(:) ! Geometrical SZA
     TYPE(DIR_SOL_GEOM), ALLOCATABLE :: DSB(:)
  END TYPE LOS_SOLAR_GEOM
  
  TYPE Geometry_Data
     INTEGER NAlts, NVA, NSol, NPhi, MaxLS
     INTEGER, ALLOCATABLE :: IDA(:), ILO_SVFP(:,:,:)
     LOGICAL AltFlg, Full_Init
     REAL (KND), ALLOCATABLE :: Alt(:)
     TYPE(DYN_CHAR), ALLOCATABLE :: VIEW(:)
     TYPE(LOS_VIEW_GEOM), ALLOCATABLE :: LVG(:,:) ! Alt, VA
     TYPE (LOS_SOLAR_GEOM), ALLOCATABLE ::  LSG(:,:,:,:) ! Alt, VA, Sol, Az
  END TYPE Geometry_Data

  Type PhiBlock
     REAL (KND) SAA
     REAL (KND), ALLOCATABLE :: CSZA(:)
  END type PhiBlock
  
  TYPE AngInit
     REAL (KND) CVA
     TYPE(PhiBlock), ALLOCATABLE :: PHI(:)
  END TYPE AngInit
  
  TYPE AuxDataInterp
     REAL (KND), ALLOCATABLE :: X_RAD(:,:,:,:), Y_ALL(:,:,:,:), X_SCR(:,:), &
          X_AL(:,:), X_SCA(:,:), X_ASC(:,:), X_Plf(:,:), X_SCC(:,:,:),  &
          AL0_OUT(:), SCR_OUT(:), SCA_OUT(:), ASC_OUT(:), DM_AO(:),     &
          SCC_NSO(:,:), SCCO(:,:), XS_RAD(:,:,:,:,:), YS_ALL(:,:,:,:,:), &
          EfWI(:,:), MSRF(:,:,:,:), Plf_OUT(:)
  END TYPE AuxDataInterp

  TYPE RES_DATA
     REAL (KND), ALLOCATABLE :: RAD(:,:,:), WF(:,:,:) 
  END TYPE RES_DATA

  Type FD_LOCAL
     REAL (KND) CSZA_Surf, EXM_NL
     REAL (KND), ALLOCATABLE :: SSSF(:,:), DS_SUN(:,:)
  End type FD_LOCAL

  TYPE WFS_PS
     REAL (KND), ALLOCATABLE :: RES(:,:,:)
  END TYPE WFS_PS

  ! Spherical line-of-sight (DOM) 
  TYPE LSE_type
     REAL (KND), ALLOCATABLE :: EX(:), EX1(:)
  END type LSE_type
  TYPE LSE_3D_type
     REAL (KND), ALLOCATABLE :: EX(:,:,:) 
  END type LSE_3D_type
  TYPE SLS_type
     INTEGER,    POINTER :: IND_LS(:,:,:), N_sph_ls(:,:,:)
     REAL (KND), POINTER :: Mue_ls(:,:,:), Eff_mu(:,:), SIN_COEF_LS(:,:,:,:)
     TYPE(LSE_type), ALLOCATABLE :: LSE(:,:)
  END TYPE SLS_type
  
  TYPE RRS_type
     LOGICAL, POINTER :: ACT_WL(:), Exi_WL(:)
     INTEGER  N_ACT, Jact, Jexi, N_EXT, N_AUX, M_ESF, N_sav
     INTEGER, POINTER :: NUM_IN(:), NESFT(:), ACT_IN(:,:), RIN_IN(:,:), A2E(:),&
          SEG_NUM_RRS(:), NPO_sav(:), N_EtoA(:,:)
     REAL (KND)  R_TOT, WL4
     REAL (KND), POINTER :: GAM(:), GAM_WL(:,:), WL_EXT(:), RAM_MOMS(:,:), &
          WEI(:,:), EXT_AUX(:,:)
     REAL (KND), DIMENSION(:,:,:), POINTER :: EXL_ACT, EXT_ACT, EXS_ACT, EXS_AUX
     REAL (KND), DIMENSION(:,:,:,:), POINTER :: NUJ_EXC, MVEC
     REAL (KND), DIMENSION(:,:,:,:,:), POINTER :: ADJ_ACT, FOR_ACT, Aj, Bj, GNp, GNm
     TYPE (LSE_3D_TYPE), ALLOCATABLE :: LS_ACT(:,:)
  END TYPE RRS_type

  Type XVecType
     Integer    IRI, IST, I_S, I_E
     REAL (KND) APR, PRE, DIFF, FCR, FcrA, PrcA, PrcR, Res
  End type XVecType

  Type RegMatrs
     REAL (KND), Allocatable :: SAD(:), ISA(:,:), TRM(:,:)
  End type RegMatrs
  
  Type MeasInfo
     REAL (KND), Allocatable :: SY(:), YDiff(:), KN(:,:)
  End type MeasInfo

  Type LineParsType
     Integer Iso
     Real (Knd) Pos, Int, LorW, LSE, TExp, DopW, Shift
  End Type LineParsType

!***************************************************************************
CONTAINS
  FUNCTION DCMP(Re,Im)
    REAL    (KND) :: Re, Im
    COMPLEX (KND) :: DCMP
    OPTIONAL :: Im
    IF( PRESENT(Im) ) THEN
       DCMP = CMPLX(Re,Im,KND)
    ELSE
       DCMP = CMPLX(Re,0.d0,KND)
    ENDIF
    RETURN
  END FUNCTION DCMP
!***************************************************************************
END MODULE TYPES
!***************************************************************************

