C********************************************************************C
C
C       SCIATRAN - Version 3.4 - Last modification: 15 Jan 2015
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE UGAMP_READ(ZZZ, PRESS, O3_OUT, *)

      USE GRIDS,    ONLY: NLAYERS
      USE LOG_PARS, ONLY: DIR_OUT, LOG_LEVEL, N_SSO, C_UNIT
      USE ST_VARS,  ONLY:  LATITUDE, LONGITUDE      
      USE ST_INPUT, ONLY: CLIM_PATH, YEAR, DAY_OF_YEAR

      CHARACTER (LEN = *), PARAMETER :: R_NAME='UGAMP_READ'

      CHARACTER (LEN=200) BASE_DIR, INFNM
      CHARACTER(LEN = 3)  MONTH(12)
      CHARACTER(LEN = 2), PARAMETER ::  STR_1 = 'r5', STR_2 = '5y' 

      INTEGER             MN, DAY, I, J, K
      INTEGER, PARAMETER :: NLAT = 73, NLONG = 145, NL = 47

      DOUBLE PRECISION    LAT_DB(NLAT), LONG_DB(NLONG), PRESS_DB(NL)
      DOUBLE PRECISION    O3_TC(NLONG, NLAT, NL), O3_TC_LL(NL)
      DOUBLE PRECISION, DIMENSION (NLAYERS) :: O3_OUT, ZZZ, PRESS, 
     $     LOG_PRESS
      DOUBLE PRECISION, DIMENSION (NLAYERS - 1) :: Z_M, PRESS_M, O3_COL,
     $     LOG_PRESS_M
      DOUBLE PRECISION, PARAMETER :: DU = 2.6867D16    
C     Auxiliary variables for monitoring purpose
C     ******************************************
      LOGICAL             CHECK
      DOUBLE PRECISION, DIMENSION (NL) :: Z_DB, Z_DB_M, LOG_PRESS_DB, 
     $     O3_C1, O3_C2
      DOUBLE PRECISION    TMP, TMP_1

      CALL ST_TRACK (R_NAME, 1, 'I')

      MONTH = (/"jan", "feb", "mar", "apr", "may","jun","jul", "aug", 
     $     "sep", "oct", "nov", "dec"/)

      BASE_DIR = TRIM(CLIM_PATH)//'/UGAMP'

C     THE COORDINATES OF THE 3-DIMENSIONAL CLIMATOLOGY
      DO I = 1, NLAT
         LAT_DB(I) = - 90.0 + FLOAT(I - 1)*2.5
      ENDDO
      DO I = 1, NLONG
         LONG_DB(I)= 0.0 + FLOAT(I - 1)*2.5
      ENDDO

      CALL CALDAY(DAY_OF_YEAR, YEAR, MN, DAY) 

      INFNM = TRIM(BASE_DIR)//'/y'//STR_1//'/'//MONTH(MN)//STR_2//'m'
      IF (LOG_LEVEL(1) >= N_SSO) 
     $     CALL WRITE_STRING (" *** Reading: "//TRIM(INFNM)) 
C     READ in the 3-D climatology.   
      OPEN(C_UNIT, FILE=TRIM(INFNM), STATUS='OLD', ERR=200)
      READ(C_UNIT,*)
      READ(C_UNIT,*)
      READ(C_UNIT,*) PRESS_DB(1:NL)
      READ(C_UNIT,*)
      READ(C_UNIT,*) O3_TC(1:NLONG - 1, 1:NLAT, 1:NL)
      CLOSE(C_UNIT)
      O3_TC(NLONG, 1:NLAT, 1:NL) = O3_TC(1, 1:NLAT, 1:NL)

      WHERE(O3_TC.EQ.-1.0) O3_TC = 0.0
      DO J = 1, NL
         CALL BILIN_INT(NLONG, LONG_DB, NLAT, LAT_DB, O3_TC(:,:,J), 1,
     $        LONGITUDE, LATITUDE, O3_TC_LL(J))
      END DO
      O3_TC_LL = O3_TC_LL*DU ! Conversion from Dobson Units

      DO J = 1, NLAYERS - 1
         Z_M(J) = (ZZZ(J) + ZZZ(J + 1))/2.0D0
      END DO
      LOG_PRESS = DLOG(PRESS)
      CALL LIN_INT(NLAYERS, ZZZ, LOG_PRESS, NLAYERS - 1, Z_M,  
     $     LOG_PRESS_M, 'E')
      PRESS_M = DEXP(LOG_PRESS_M)
      CALL AL_INTERP(NL, PRESS_DB, O3_TC_LL, NLAYERS - 1, PRESS_M,  
     $     O3_COL, .TRUE., *201)
      WHERE(PRESS_M <= PRESS_DB(1)) O3_COL(:) = O3_TC_LL(1)
      WHERE(PRESS_M >= PRESS_DB(NL)) O3_COL(:) = O3_TC_LL(NL)
      
      O3_OUT(1) = 0.0D0
      DO K = 2, NLAYERS - 1
         O3_OUT(K) = MAX((O3_COL(K) - O3_COL(K - 1))/
     $        (Z_M(K - 1) - Z_M(K)), 0.0D0)
      END DO
      O3_OUT(NLAYERS) = O3_OUT(NLAYERS - 1)

      CHECK = .FALSE.
c      CHECK = .TRUE.
      IF(.NOT.CHECK) THEN
         CALL ST_TRACK (R_NAME, 1, 'O')
         RETURN
      END IF

      LOG_PRESS_DB = DLOG(PRESS_DB)
      CALL LIN_INT(NLAYERS, LOG_PRESS, ZZZ, NL, LOG_PRESS_DB, Z_DB, 'E')
      CALL AL_INTERP(NLAYERS, ZZZ, O3_OUT, NL, Z_DB, O3_C1, .TRUE.,*201)
      Z_DB_M(1) = Z_DB(1) +  (Z_DB(1) - Z_DB(2))/2.0D0
      DO K = 2, NL
         Z_DB_M(K) = (Z_DB(K - 1) + Z_DB(K))/2.0D0
      END DO
      CALL AL_INTERP(NLAYERS, ZZZ, O3_OUT, NL, Z_DB_M, O3_C2, 
     $     .TRUE., *201)

      OPEN(37, FILE=DIR_OUT//'O3_columns.dat')
      DO J = 2, NL
         TMP = O3_C1(1)*(Z_DB(1) - Z_DB(2)) + 
     $        O3_C1(J)*(Z_DB(J - 1) - Z_DB(J))
         DO K = 2, J - 1
            TMP = TMP + O3_C1(K)*(Z_DB(K - 1) - Z_DB(K + 1))
         END DO
         TMP = TMP/2.0D0
         TMP_1 = O3_C2(1)*(Z_DB(1) - Z_DB(2))
         DO K = 2, J
            TMP_1 = TMP_1 + O3_C2(K)*(Z_DB(K - 1) - Z_DB(K))
         END DO
         WRITE(37,'(F6.2,3(2x,ES21.7))') Z_DB(J), 
     $        TMP, TMP_1, O3_TC_LL(J)
      END DO
         
      CLOSE(37)

      CALL ST_TRACK (R_NAME, 1, 'I')
      STOP
      RETURN

 200  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 'UGAMP climatology '
     $     //'file is not found: '//TRIM(INFNM))
      RETURN 1

 201  CALL WRITE_MESSAGE(R_NAME, 'ERROR', 
     $     'Interpolation error (O3 from UGAMP climatology)')
      RETURN 1
     
      RETURN 1
      END
C********************************************************************C
