C********************************************************************C
C
C	SCIATRAN - Version 3.1 - Last modification: 27 Jul 2010
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE VLAD (NVLAD, PROFILE_FILE, Z, P, T, VMR, N_MOL, *)

      USE LOG_CONTROL, ONLY : ERRORUNIT, LUNIT, DO_LOGFILE, C_UNIT
      USE ST_INPUT, ONLY: MAX_LAY

      CHARACTER (LEN = 4), PARAMETER :: ROUTINE_NAME = 'VLAD'

      INTEGER            M, K, KDUM, NVLAD, N_MOL
      CHARACTER (LEN=*)  PROFILE_FILE
      CHARACTER (LEN=80) HEADERLINE   

      DOUBLE PRECISION  VMR(MAX_LAY,N_MOL), TEST(N_MOL + 1), ZOLD 
      DOUBLE PRECISION, DIMENSION (MAX_LAY) :: Z, T, P

      KDUM = 0

      IF (DO_LOGFILE)
     &   WRITE(LUNIT,*) ' *** Reading: ', TRIM(PROFILE_FILE) 
      OPEN (C_UNIT,FILE=PROFILE_FILE,STATUS='OLD')              

c     Skip headerlines
c     ----------------
 1    CONTINUE
      READ(C_UNIT,'(A)') HEADERLINE
      IF (HEADERLINE(1:1).EQ.';') GOTO 1
      BACKSPACE C_UNIT

c     Read number of layers
c     ---------------------
      READ(C_UNIT,*) NVLAD
      IF (NVLAD.GT.MAX_LAY) GOTO 120
      BACKSPACE C_UNIT

c     Read profiles
c     -------------
      ZOLD = 200.0D0
      READ(C_UNIT,*) KDUM, Z(1), P(1), T(1), (TEST(M), M = 1, N_MOL + 1)
      IF(INT(TEST(N_MOL + 1)) /= KDUM - 1) GOTO 123
      BACKSPACE C_UNIT
      BACKSPACE C_UNIT
      DO K = NVLAD,1,-1
         READ(C_UNIT,*) KDUM, Z(K), P(K), T(K), (VMR(K,M), M = 1, N_MOL)
         IF (KDUM.NE.K) GOTO 121
         IF (Z(K).GE.ZOLD) GOTO 122
         ZOLD = Z(K)
      ENDDO

      CLOSE(C_UNIT)
      RETURN

 120  CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $     'Too many altitude levels profile file')
      WRITE(ERRORUNIT,*) '    File : '//TRIM(PROFILE_FILE)
      WRITE(ERRORUNIT,*) '    Diagnostic (read, max): ', NVLAD, MAX_LAY
      RETURN 1

 121  CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $               'Wrong numbering in profile file')
      WRITE(ERRORUNIT,*) '    File : '//TRIM(PROFILE_FILE)
      WRITE(ERRORUNIT,*) '    Diagnostic (read, expected): ', KDUM, K
      RETURN 1

 122  CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $               'Wrong profile input (incr. altitude)')
      WRITE(ERRORUNIT,*) '    File : '//TRIM(PROFILE_FILE)
      WRITE(ERRORUNIT,*) '    Diagnostic (layer#, altitude): ', K, Z(K)
      RETURN 1

 123  CALL WRITE_MESSAGE(ROUTINE_NAME, 'ERROR', 
     $               'Wrong format of the profile input file')
      WRITE(ERRORUNIT,*) '    File : '//TRIM(PROFILE_FILE)
      RETURN 1

      END
C************************************************************************
