c *************************************************************************
      SUBROUTINE WIGNER_PLG( XM, NS, M, NLEG, RES )
      Implicit none
c     Input variables
c     ---------------
      INTEGER, INTENT(IN) :: NS, M, NLEG
      DOUBLE PRECISION XM(NS)
c     Output varable
c     ---------------
      DOUBLE PRECISION RES(NLEG,NS)
c     Local variables
c     ---------------
      INTEGER          I, L
      DOUBLE PRECISION MFAC 
      DOUBLE PRECISION HM(NS)
      IF( M .EQ. 0 ) THEN
         RES(1,:) = 1.0d0 ; RES(2,:) = XM(:)
         DO L = 2,NLEG - 1
            RES(L+1,:) =  ((2*L-1)*XM(:)*RES(L,:) - (L-1)*RES(L-1,:))/L
         ENDDO
         RETURN
      ENDIF
      RES(1:M,:) = 0.d0 ; MFAC = 1.d0 
      DO I = 1,M ; MFAC = MFAC*(M+I)/I ; ENDDO       
      HM(:) = DSQRT( (1.0d0 - XM(:))*(1.0d0 + XM(:)) )/2.0d0
c     ... Without (-1)**M as has been used by Siewert
      RES(M+1,:) = DSQRT(MFAC)*HM(:)**M 
      DO L = M+1,NLEG - 1
         RES(L+1,:) = ( (2*L-1)*XM(:)*RES(L,:) 
     $        - DSQRT(DBLE((L-1-M)*(L-1+M)))*RES(L-1,:) )
     $        /DSQRT(DBLE((L-M)*(L+M)))
      ENDDO
      RETURN
      END
c *************************************************************************
      SUBROUTINE WIGNER_FUNCTION( XM, NS, M, NLEG, DWF_P2, DWF_M2 )
      Implicit none
c     Input variables
c     ---------------
      INTEGER          NS, M, NLEG
      DOUBLE PRECISION XM(NS)
c     Output varable
c     ---------------
      DOUBLE PRECISION DWF_P2(NLEG,NS),DWF_M2(NLEG,NS) 
c     Local variables
c     ---------------
      INTEGER          S0, I, L
      DOUBLE PRECISION NFAC
      DOUBLE PRECISION C1, C2, LPM, LMM
      DOUBLE PRECISION HM(NS), HP(NS)

      HM(:) = 1.0d0 - XM(:) ; HP(:) = 1.0d0 + XM(:)
      S0 = MAX(M,2)
      DWF_P2(1:S0,:) = 0.0d0 ; DWF_M2(1:S0,:) = 0.0d0
      IF( M .EQ. 0 ) THEN
         DWF_P2(S0+1,:) = DSQRT(6.D0)/4.*HM(:)*HP(:)
         DWF_M2(S0+1,:) = DWF_P2(S0+1,:)
      ENDIF
      IF( M .EQ. 1 ) THEN
         DWF_P2(S0+1,:) =  0.5*DSQRT(HM(:)*HP(:))*HP(:)
         DWF_M2(S0+1,:) =  0.5*DSQRT(HP(:)*HM(:))*HM(:)
      ENDIF
      IF( M .EQ. 2 ) THEN
         DWF_P2(S0+1,:) =  HP(:)*HP(:)*0.25d0
         DWF_M2(S0+1,:) =  HM(:)*HM(:)*0.25d0
      ENDIF
      IF( M .GT. 2 ) THEN
         NFAC = 0.5d0*DBLE(M*(M-1))
         DO I = 3,M ; NFAC = NFAC*(M+I)/I ; ENDDO 
         C1 = (1-2*MODULO(M,2))*DSQRT(NFAC)/2.0d0**M
         HM(:) = dsqrt(HM(:)) ; HP(:) = dsqrt(HP(:))
         DWF_P2(S0+1,:) = C1 * HM(:)**(M-2)*HP(:)**(M+2) 
         DWF_M2(S0+1,:) = C1 * HP(:)**(M-2)*HM(:)**(M+2)
      ENDIF
      DO L = S0+1,NLEG - 1
         LMM = DBLE( L-M ) ; LPM = DBLE( L+M )
         C1 = 1.0d0/DBLE(L-1)/DSQRT( LMM * LPM *(L+2) * (L-2) )
         C2 = C1*DBLE(L)*DSQRT( (LMM-1)*(LPM-1)*(L-3)*(L+1) )
         DWF_P2(L+1,:) = 
     $    C1*(2*L-1)*(L*(L-1)*XM(:)-2*M)*DWF_P2(L,:) - C2*DWF_P2(L-1,:)
         DWF_M2(L+1,:) = 
     $   -C1*(2*L-1)*(L*(L-1)*XM(:)+2*M)*DWF_M2(L,:) - C2*DWF_M2(L-1,:)
      ENDDO

      DO L = 0,NLEG-1
         DWF_M2(L+1,:) = (1 - 2*MODULO(L+M,2)) * DWF_M2(L+1,:)
      ENDDO

      RETURN
      END
c ****************************************************************
