!***************************************************************************
!
!	SCIATRAN - Version 4.0 - Last modification: 14 Nov 2018
!
!	Copyright (C) - Institute of Remote Sensing,
!			University of Bremen, Germany
!
!***************************************************************************
Subroutine Write_Angles (UNT, NSto, NC_out)
  USE ST_VARS,     ONLY: RTM_TYPE
  INTEGER UNT, NSto, NC_out(4)
  !==================================================================
  Select Case (RTM_TYPE)
  Case ('transmission')
     Call Write_Angles_Trans (UNT, NSto, NC_out)
  Case Default
     Call Write_Angles_Std (UNT, NSto, NC_out)
  End Select
  Return
End Subroutine Write_Angles
!***************************************************************************
Subroutine Write_Angles_Std (UNT, NSto, NC_out)
  ! Called by output.f
  USE SPHER_PARS,  ONLY: NFA =>  N_FOV_ANG
  USE ST_VARS,     ONLY: GeoD => GeoD_OUT, Pie, EARTH_RADIUS
  USE TOOLS,       ONLY: WriteOut
  USE TYPES,       ONLY: KND, LOS_VIEW_GEOM, LOS_SOLAR_GEOM
  INTEGER UNT, NSto, NC_out(4)
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Write_Angles_Std'
  INTEGER  IA, IV, IS, IP, J1, IVS, J
  REAL (KND) R2D
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  Call WriteOut (Unt, &
       '# Geometry data: each line of this file lists angles for each column ')
  Call WriteOut (Unt, '#                in intensity.dat  file')
  Call WriteOut (Unt, '# 1st column : sequential number of the line of sight')
  Call WriteOut (Unt, '# 2nd column : number of the Stockes component')
  Call WriteOut (Unt, '# 3rd column : solar zenith angle at the output altitude in degrees')
  Call WriteOut (Unt, '#              (geometrical)')
  Call WriteOut (Unt, &
       '# 4th column : viewing angle at the output altitude in degrees')
  Call WriteOut (Unt, &
       '# 5th column : relative azimuth angle at the output altitude in degrees')
  Call WriteOut (Unt, &
       '# 6th column : geometrical tangent height in km (999 if TH is not in ')
  Call WriteOut (Unt, '#              the viewing direction)')
  Call WriteOut (Unt, &
       '# 7th column : refractive tangent height in km (999 if TH is not in ')
  Call WriteOut (Unt, '#              the viewing direction)')
  Call WriteOut (Unt, &
       '# 8th column : solar zenith angle (refractive) at the tangent height ')
  Call WriteOut (Unt, &
       '#              or at the end of the line of sight (TOA for upward and BOA ')
  Call WriteOut (Unt, '#              for downward viewing directions) in degrees')
  Call WriteOut (Unt, '#              (set to 180 if the point is not illuminated)')
  Call WriteOut (Unt, &
       '# 9th column : viewing angle (refractive) at the tangent height ')
  Call WriteOut (Unt, &
       '#              or at the end of the line of sight (TOA for upward and BOA ')
  Call WriteOut (Unt, '#              for downward viewing directions) in degrees')
  Call WriteOut (Unt, &
       '# 10th column: relative azimith angle (refractive) at the tangent height ')
  Call WriteOut (Unt, &
       '#              or at the end of the line of sight (TOA for upward and BOA ')
  Call WriteOut (Unt, '#              for downward viewing directions) in degrees')
  Call WriteOut (Unt, '#              (set to 180 if the point is not illuminated)')
  Call WriteOut (Unt, &
       '# 11th column: single scattering angle (refractive) at the tangent ')
  Call WriteOut (Unt, &
       '#              height or at the end of the line for downward pointing ')
  Call WriteOut (Unt, &
       '#              LOS or at the first point for upward pointing LOS')
  Call WriteOut (Unt, &
       '#              (set to 90 if the point is not illuminated)')
  Call WriteOut (Unt, '# 12th column: ouput altititude in km')
               
  J1 = 1
  R2D = 180.0D0/PIE
  DO IA = 1, GeoD%NAlts
     DO IS = 1, GeoD%NSol
        IVS = CEILING(FLOAT(NFA)/2.) 
        DO IV = IVS, GeoD%NVA, NFA
           DO IP = 1, GeoD%NPhi
              CALL WriteData(GeoD%LVG(IA,IV), GeoD%LSG(IA,IV,IS,IP))
           ENDDO
        ENDDO
     ENDDO
  ENDDO
               
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
CONTAINS
!***************************************************************************
  Subroutine WriteData (LVG, LSG)
    INTEGER JJ
    REAL(KND) SVA, H0_G, H0_R, SSA, SSZA
    Character (Len=500)  OutStr
    TYPE (LOS_VIEW_GEOM) LVG
    TYPE (LOS_SOLAR_GEOM) LSG
    !==================================================================        
    IF(LVG%NTH < LVG%NLS) THEN
       SVA = SQRT(1.0D0 - LVG%C_VA(1)**2) 
       H0_G = (EARTH_RADIUS + GeoD%Alt(IA))*SVA - EARTH_RADIUS
       H0_R = LVG%H_0
    ELSE
       H0_G = 999. ; H0_R = 999.
    END IF
!    IF(LVG%C_VA(1) > 0.0D0) THEN
!       JJ = LVG%NTH
!    ELSE
!       JJ = 1
!    END IF
!    SVA = SQRT(1.0D0 - LVG%C_VA(JJ)**2) 
!    SSZA = SQRT(1.0D0 - LSG%C_SZA(JJ)**2) 
!    SSA = ACOS(-LVG%C_VA(JJ)*LSG%C_SZA(JJ) + SVA*SSZA*COS(LSG%SAA(JJ)))*R2D
    SVA = SQRT(1.0D0 - LVG%CVA_TH**2) 
    SSZA = SQRT(1.0D0 - LSG%CSZA_TH**2) 
    SSA = ACOS(-LVG%CVA_TH*LSG%CSZA_TH + SVA*SSZA*COS(LSG%SAA_TH))*R2D
                              
    DO J = 1, NSto
       WRITE(OutStr,'(I0,2x,I0,10(2X,F6.2))') &
            J1, NC_out(J), ACOS(LSG%CG_SZA(1))*R2D, &
            ACOS(LVG%C_VA(1))*R2D, LSG%SAA(1)*R2D, H0_G, H0_R, &
!            ACOS(LSG%C_SZA(JJ))*R2D, ACOS(LVG%C_VA(JJ))*R2D, LSG%SAA(JJ)*R2D, SSA, &
            ACOS(LSG%CSZA_TH)*R2D, ACOS(LVG%CVA_TH)*R2D, LSG%SAA_TH*R2D, SSA, &
            GeoD%Alt(IA)
       Call WriteOut (Unt, OutStr)
       J1 = J1 + 1
    ENDDO
    RETURN
  END Subroutine WriteData
!***************************************************************************
End Subroutine Write_Angles_Std
!***************************************************************************
Subroutine Write_Angles_Trans (UNT, NSto, NC_out)
  ! Called by output.f
  USE SPHER_PARS,  ONLY: NFA =>  N_FOV_ANG
  USE ST_VARS,     ONLY: GeoD => GeoD_OUT, Pie, EARTH_RADIUS
  USE TOOLS,       ONLY: WriteOut
  USE TYPES,       ONLY: KND, LOS_VIEW_GEOM
  INTEGER UNT, NSto, NC_out(4)
  ! Local parameters
  CHARACTER (LEN=*), PARAMETER :: R_NAME = 'Write_Angles_Trans'
  INTEGER  IA, IV, IS, IP, J1, IVS, J
  REAL (KND) R2D
  !==================================================================
  CALL ST_TRACK (R_NAME, 1, 'I')

  Call WriteOut (Unt, &
       '# Geometry data: each line of this file lists angles for each column ')
  Call WriteOut (Unt, '#                in intensity.dat  file')
  Call WriteOut (Unt, '# 1st column : sequential number of the line of sight')
  Call WriteOut (Unt, '# 2nd column : number of the Stockes component')
  Call WriteOut (Unt, '# 3rd column : solar zenith angle at the output altitude in degrees')
  Call WriteOut (Unt, &
       '# 4th column : geometrical tangent height in km (999 if TH is not in ')
  Call WriteOut (Unt, '#              the viewing direction)')
  Call WriteOut (Unt, &
       '# 5th column : refractive tangent height in km (999 if TH is not in ')
  Call WriteOut (Unt, '#              the viewing direction)')
  Call WriteOut (Unt, '# 6th column: ouput altititude in km')
               
  J1 = 1
  R2D = 180.0D0/PIE
  DO IA = 1, GeoD%NAlts
     IVS = CEILING(FLOAT(NFA)/2.) 
     DO IV = IVS, GeoD%NVA, NFA
        CALL WriteData_Trans(GeoD%LVG(IA,IV))
     ENDDO
  ENDDO
               
  CALL ST_TRACK (R_NAME, 1, 'O')
  RETURN
CONTAINS
!***************************************************************************
  Subroutine WriteData_Trans (LVG)
    INTEGER JJ
    REAL(KND) H0_G, H0_R, SVA
    Character (Len=500)  OutStr
    TYPE (LOS_VIEW_GEOM) LVG
    !==================================================================        
    IF(LVG%NTH < LVG%NLS) THEN
       SVA = SQRT(1.0D0 - LVG%C_VA(1)**2) 
       H0_G = (EARTH_RADIUS + GeoD%Alt(IA))*SVA - EARTH_RADIUS
       H0_R = LVG%H_0
    ELSE
       H0_G = 999. ; H0_R = 999.
    END IF
    IF(LVG%C_VA(1) > 0.0D0) THEN
       JJ = LVG%NTH
    ELSE
       JJ = 1
    END IF
                              
    DO J = 1, NSto
       WRITE(OutStr,'(I0,2x,I0,2X,F6.2,2(2X,F6.2),2X,F6.2)') &
            J1, NC_out(J), ACOS(-LVG%C_VA(1))*R2D, H0_G, H0_R, &
            GeoD%Alt(IA)
       Call WriteOut (Unt, OutStr)
       J1 = J1 + 1
    ENDDO
    RETURN
  END Subroutine WriteData_Trans
!***************************************************************************
End Subroutine Write_Angles_Trans
!***************************************************************************
