C********************************************************************C
C
C	SCIATRAN - Version 4.7 - Last modification: 16 Aug 2023
C
C	Copyright (C) - Institute of Remote Sensing,
C			University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE WRITE_CLOUD_SETTINGS 
c     ... Call in "gt_iface.f"

      USE CLOUD_INPUT, ONLY: USER_EK_NAME, D_MF, DBC_NAME, TDS, MLCS,
     $                       CON_VAL_CLD, MAN_AUTO_CLD, CLD_INHOM,
     $                       LC_CLD, UC_CLD, PHASE_NAME, SUB_LAYERING,
     $                       HOW_CLD_COEF, Plus_ice, ADD_ABS, Phm_const
      USE CLOUDS,      ONLY: CLD_NETWAVS, TAU_CLD_INP, LWP, J_TOP,
     $                       NCW => N_CLD_NETWAVS, IWP, CLD_LAYER_NUM,
     $                       CLPH => CLD_DM, VT => Var_type, J_BOT, 
     $                       CLH => CLD_LAYER_HEIGHTS, SCALE_PROF,
     $                       N_C_L => N_CLD_LAYERS, NG_cld
      USE GRIDS,       ONLY: NLAYERS, ZZZ
      USE INFO,        ONLY: CLD_INFO
      USE INTERN_DIMS, ONLY: NL_soot
      USE LOG_PARS,    ONLY: C_UNIT, DIR_OUT
      USE TYPE_VARS,   ONLY: KND
c     ------------------------------------------------------------------

      CHARACTER (LEN = *), PARAMETER :: R_NAME = 'WRITE_CLOUD_SETTINGS'
      CHARACTER (LEN=:), ALLOCATABLE :: STR_V, STR_S
      CHARACTER (LEN=17)  N_STR
      CHARACTER (LEN=30)  PRIN(10)
      CHARACTER (LEN=50)  INP_TYPE(10), TXT, O_F, O_1
      INTEGER    I, K, J, I1, INX, N_TXT, N_DEL, II
      INTEGER, DIMENSION (N_C_L) :: IDW, IDI
      REAL (KND) Abs_in(NL_soot), TAU_1(N_C_L), TAU_2(N_C_L)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      DO I = 1, N_C_L 
         SELECT CASE( TDS(I)%S )
         CASE('alay')
            IDW(I) = 1 ; IDI(I) = 2
            PRIN(I) = 'aerosol layer' 
            IWP(I) = 0.0  ;  LWP(I) = 0.0
            INP_TYPE(I) = VT(I,1)%S
         CASE('water')
            IDW(I) = 1 ; IDI(I) = 2
            PRIN(I) = 'Water' ; IWP(I) = 0.0
            INP_TYPE(I) = VT(I,1)%S
         CASE('ice','ice-sm')
            INX = 1
            IDI(I) = 1 ; IDW(I) = 2
            IF( ANY(Plus_ice) ) THEN
               INX = 2
               IDW(I) = 1 ; IDI(I) = 2
            ENDIF
            PRIN(I) = 'Ice' ; LWP(I) = 0.0
            INP_TYPE(I) = VT(I,INX)%S
         CASE('mixed','mixed-sm')
            IDW(I) = 1 ; IDI(I) = 2
            PRIN(I) = 'Water/Ice'
            INP_TYPE(I) = VT(I,1)%S//'/'//VT(I,2)%S
         END SELECT
      END DO
      
      
      OPEN(C_UNIT, FILE=DIR_OUT//'SCE_CLOUD.OUT')
         K = 1
         CALL WRITE_UD(1,C_UNIT)

         WRITE(C_UNIT,*) ' ******************************************'
         WRITE(C_UNIT,*) '             CLOUD PARAMETERS              '
         WRITE(C_UNIT,*) ' ******************************************'
         WRITE(C_UNIT,*)

         WRITE(C_UNIT,'(4x,a,3x,i2)') 'Number of cloud layers : ', N_C_L

c         WRITE(C_UNIT,'(13x,a,2x,10(a,10x))') 
c     $        'Type of cloud :',(TRIM(PRIN(I)),I=1,N_C_L)

         N_TXT = 31 ; N_DEL = 20
         O_1 = ''
         DO I = 1, N_C_L
            WRITE(N_STR,'(I0)') N_TXT + (I-1)*N_DEL
            O_1 = TRIM(O_1)//',T'//TRIM(N_STR)//',A'
         ENDDO
         O_1 = TRIM(O_1)//')'
         O_F = '(13x,A'//TRIM(O_1)//')'
         WRITE(C_UNIT,FMT=O_F) 'Type of cloud :',
     $         (TRIM(PRIN(I)), I = 1,N_C_L)

         WRITE(C_UNIT,FMT='(3x,A'//TRIM(O_1)//')') 
     $        'Type of input parameter :',
     $        (TRIM(INP_TYPE(I)),I=1,N_C_L)
         DO I = 1,N_C_L
            IF( TRIM(INP_TYPE(I))=='TAU' ) THEN
               SELECT CASE( TDS(I)%S )
               CASE('water')
                  SELECT CASE( SCALE_PROF )
                  CASE('Scale-NC')
                     WRITE(C_UNIT,'(20x,A)') 
     $                    'scaled :  number concentration'
                  CASE('Scale-WC')
                     WRITE(C_UNIT,'(20x,A)') 
     $                    'scaled :  water content'
                  END SELECT
c                  EXIT
               CASE('ice')
                  WRITE(C_UNIT,'(20x,A)') 
     $                 'scaled :  ice water content'
               CASE('mixed','mixed-sm')
                  SELECT CASE( SCALE_PROF )
                  CASE('Scale-NC')
                     WRITE(C_UNIT,'(20x,A)') 
     $                    'scaled :  number concentration (water)'
                  CASE('Scale-WC')
                     WRITE(C_UNIT,'(20x,A)') 
     $                    'scaled :  liquid water content'
                  END SELECT
                  WRITE(C_UNIT,'(20x,A)') 
     $                 'scaled :  ice water content'
               END SELECT
            ENDIF
         ENDDO

         WRITE (C_UNIT,'(11x,a,10(2x,f9.5,6x))') 'Cloud Base (km) :',
     $        (REAL( CLH(i,1) ), i = 1,N_C_L)
         WRITE (C_UNIT,'(11x,a,10(2x,f9.5,6x))') 'Cloud Top  (km) :',
     $        (REAL( CLH(i,2) ), i = 1,N_C_L)
         
         WRITE (C_UNIT,'(6x,a,2x,10(4x,I3,9x))') 
     $        'Number of sub-layers :', 
     $        (INT( CLD_LAYER_NUM(i) ),i =1,N_C_L) 
         WRITE(C_UNIT,'(13x,a,*(2x,a))')
     $        'Sub-grid type :', (SUB_LAYERING(I)%S,I=1,N_C_L)

         WRITE(C_UNIT,'(1x,a)') 
     $        'Optical thickness (500 nm)'
         IF( TDS(1)%S /= 'alay') THEN
            WRITE(C_UNIT,'(3x,a,10(2x,f9.3,4x))') 
     $        '                  Water : ',
     $        ( Tau_CLD_inp(I,IDW(I)),I=1,N_C_L )
            WRITE(C_UNIT,'(3x,a,10(2x,f9.3,4x))') 
     $        '                  Ice   : ',
     $        ( Tau_CLD_inp(I,IDI(I)),I=1,N_C_L )
         ELSE
            WRITE(C_UNIT,'(3x,a,10(2x,f9.3,4x))') 
     $        '                Aerosol : ',
     $        ( Tau_CLD_inp(I,IDW(I)),I=1,N_C_L )
         ENDIF
         WRITE(C_UNIT,'(1x,a,10(2x,f9.3,4x))') 
     $        'Liquid water path [g/m^2] : ',(LWP(I),I=1,N_C_L)
c         WRITE(C_UNIT,'(1x,a,10(2x,f9.3,4x))') 
         WRITE(C_UNIT,'(1x,a,10(2x,ES12.4,4x))') 
     $        '   Ice water path [g/m^2] : ',(IWP(I),I=1,N_C_L)
         
         WRITE (C_UNIT,'(7x,A,2x,10(A,17x))') 'Phase Function type :', 
     $        ( PHASE_NAME(I)%S, I = 1,N_C_L)
         
         IF( PHASE_NAME(1)%S /= 'user' ) THEN             
            WRITE (C_UNIT,'(7x,A)') 
     $           'Phase function name  '
c            IF( Phm_const ) THEN
c            DO I = 1, N_C_L
c               IF( MLCS(I)%Phm_co ) THEN
                  WRITE(C_UNIT,FMT='(3x,A'//TRIM(O_1)//')') 
     $                 '                  Water :',
     $                 (DBC_NAME(I1,1)%S, I1=1,N_C_L)
                  WRITE(C_UNIT,FMT='(3x,A'//TRIM(O_1)//')') 
     $                 '                  Ice   :',
     $                 (DBC_NAME(I1,2)%S, I1=1,N_C_L)
c               ELSE
c                  WRITE(C_UNIT,FMT='(3x,A,2x,A)') 
c     $                 '                  Water :',
c     $                 'all from DB if presented'
c                  WRITE(C_UNIT,FMT='(3x,A,2x,A)') 
c     $                 '                   Ice  :',
c     $                 'all from DB if presented'
c               ENDIF
c            ENDDO
         ELSE
            WRITE (C_UNIT,'(1x,A)') 'Phase function for layers : '
            IF( TDS(1)%S /= 'alay') THEN
               WRITE(C_UNIT,'(3x,a,10(2x,A))') 
     $              '                  Water : ',
     $              (USER_EK_NAME(I,1)%S,I=1,N_C_L )
               WRITE(C_UNIT,'(3x,a,10(2x,A))') 
     $              '                  Ice   : ',
     $              ( USER_EK_NAME(I,2)%S,I=1,N_C_L )
            ELSE
               WRITE(C_UNIT,'(3x,a,10(2x,A))') 
     $              '               see file : ',
     $              ( USER_EK_NAME(I,2)%S,I=1,N_C_L )
            ENDIF
         ENDIF  ! PHASE_NAME(1)%S /= 'user'

         SELECT CASE( TRIM(HOW_CLD_COEF) )
         CASE('from data base')
            WRITE (C_UNIT,'(1x,A,10(2x,A))') 
     $           'Scattering and extinction ' 
            WRITE (C_UNIT,'(14x,A,10(2x,A))') 
     $           'coefficients :', 
     $           'from data base'
         CASE('analytically')
            WRITE (C_UNIT,'(1x,A,10(2x,A))') 
     $           'Scattering and extinction ' 
            WRITE (C_UNIT,'(14x,A,10(2x,A))') 
     $           'coefficients :', 
     $           'analytically'
         END SELECT
         
         SELECT CASE ( TRIM(D_MF) )
         CASE( 'D-M' )
            WRITE(C_UNIT,'(3x,A,2x,A3)') 
     $           'Delta-M approximation ? :', 'Yes'
            WRITE(C_UNIT,'(2x,A,2x,I3)') 
     $           'Delta-M truncation index :', CLPH(1)
         CASE( 'D-F' )
            WRITE(C_UNIT,'(3x,A,2x,A3)') 
     $           'Delta-fit approximation ? :', 'Yes'
            WRITE(C_UNIT,'(2x,A,2x,I3)') 
     $           'Delta-fit truncation index :', CLPH(1)
            
            DO I = 1, N_C_L
               WRITE(C_UNIT,'(2x,A,2x,I2)') 'Cloud layer', I
               WRITE(C_UNIT,'(2x,A)')       '-------------------------'
               SELECT CASE (MAN_AUTO_CLD(I)) 
               CASE('manu')
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Manual definition of the forward cone'
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'The forward cone value is set to :'
                  WRITE(C_UNIT,'(36x,f6.3,1x,A3)') LC_CLD(I),'deg'
               CASE('auto')
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Automatic definition of the forward cone'
                  WRITE(C_UNIT,'(2x,A,2x,f6.3,1x,A3,1x,f6.3,1x,A3)') 
     $                 'The range of the forward cone:',
     $                 LC_CLD(I),' - ',UC_CLD(I),'deg'
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Quasi-optimal value of the forward cone :'
                  WRITE(C_UNIT,'(34x,f6.3,1x,A3)') CON_VAL_CLD(I),'deg'
               END SELECT
            ENDDO
         CASE( 'D-S' )
            WRITE(C_UNIT,'(3x,A,2x,A3)') 
     $           'Delta-S approximation ? :', 'Yes'
            WRITE(C_UNIT,'(2x,A,2x,I3)') 
     $           'Delta-S truncation index :', CLPH(1)
            DO I = 1, N_C_L
               WRITE(C_UNIT,'(2x,A,2x,I2)') 'Cloud layer', I
               WRITE(C_UNIT,'(2x,A)')       '-------------------------'
               SELECT CASE (MAN_AUTO_CLD(I)) 
               CASE('manu')
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Manual definition of the forward cone'
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'The forward cone value is set to :'
                  WRITE(C_UNIT,'(36x,f6.3,1x,A3)') LC_CLD(I),'deg'
               CASE('auto')
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Automatic definition of the forward cone'
                  WRITE(C_UNIT,'(2x,A,2x,f6.3,1x,A3,1x,f6.3,1x,A3)') 
     $                 'The range of the forward cone:',
     $                 LC_CLD(I),' - ',UC_CLD(I),'deg'
                  WRITE(C_UNIT,'(2x,A)') 
     $                 'Quasi-optimal value of the forward cone :'
                  WRITE(C_UNIT,'(34x,f6.3,1x,A3)') CON_VAL_CLD(I),'deg'
               END SELECT
            ENDDO  ! over cloud layers (I)
         END SELECT

         WRITE(C_UNIT,'(2x,A,2x,I5)') 
     $        '  Max number of moments  :', Ng_cld
         
         SELECT CASE( TRIM(CLD_INHOM) )
         CASE('yes')         
            TXT = 'vertically inhomogeneous'
         CASE('no')
            TXT = 'vertically homogeneous'
         END SELECT

         WRITE (C_UNIT,'(12x,A)') 
     $        'Profiling type  '
         WRITE(C_UNIT,'(14x,A,2x,A)') 
     $        'coefficients :',TRIM(TXT)

         IF( MLCS(1)%Phm_co ) THEN
            WRITE(C_UNIT,'(12x,A,2x,A)') 
     $           'phase function :','constant'
         ELSE
            WRITE(C_UNIT,'(12x,A,2x,A)') 
     $           'phase function :','depends on Ref'
         ENDIF
c     $        '                          :',
         
         STR_V = '' ;  STR_S = ''
         DO II = 1, UBOUND(CLD_INFO, 1)
            IF(.NOT.CLD_INFO(II)%WF_FLG) CYCLE
            IF(CLD_INFO(II)%VSI == 'V') THEN
               STR_V = STR_V//' '//CLD_INFO(II)%NAME
            ELSE
               STR_S = STR_S//' '//CLD_INFO(II)%NAME
            END IF
         END DO

         IF(LEN(STR_V) > 0 .AND. LEN(STR_S) > 0) 
     $        WRITE(C_UNIT,'(A)') '          WF for clouds   : Yes'
         IF(LEN(STR_V) > 0 ) 
     $        WRITE(C_UNIT,'(A)') '       Parameter-function : '//STR_V
         IF(LEN(STR_S) > 0 ) 
     $        WRITE(C_UNIT,'(A)') '         Scalar parameter : '//STR_S
         
         WRITE (C_UNIT, '(2x, a)')
     $        'Cloud Optical Properties '
         WRITE (C_UNIT, '(A, f9.2, A, f9.2, A)')
     $        'are specified in the range :', 
     $        cld_netwavs(1), ' - ',cld_netwavs(NCW),' nm'

c -----------------------------------------------------------------------
         IF( ANY(ADD_ABS) ) THEN
            CALL ADD_SOOT(cld_netwavs(1), Abs_in)
            DO I = 1,N_C_L
               TAU_1(I) = 0.d0
               DO J = J_TOP(I), J_BOT(I)-1
                  TAU_1(I) = TAU_1(I) + (Abs_in(J)+Abs_in(J+1))/2.d0
     $                 *(ZZZ(J)-ZZZ(J+1))
               ENDDO
            ENDDO
            CALL ADD_SOOT(cld_netwavs(NCW), Abs_in)
            DO I = 1,N_C_L
               TAU_2(I) = 0.d0
               DO J = J_TOP(I), J_BOT(I)-1
                  TAU_2(I) = TAU_2(I) + (Abs_in(J)+Abs_in(J+1))/2.d0
     $                 *(ZZZ(J)-ZZZ(J+1))
               ENDDO
            ENDDO
            WRITE(C_UNIT, '(2x, a)')
     $           'Additional absorber with '
            WRITE(C_UNIT, '(2x,a,1x, 1x,e10.2,a,e10.2)')
     $           '       optical thickness :', TAU_1(1),' - ',TAU_2(1)
         ELSE
            WRITE(C_UNIT, '(1x,a,2x,A)')
     $           '      Additional absorber :', '  not present'
         ENDIF
c -----------------------------------------------------------------------
         WRITE (C_UNIT, '(2x, a)')
     $        '   Vertical profiles of  '
         WRITE (C_UNIT, '(2x, a)')
     $        '        cloud parameters '
         WRITE (C_UNIT, '(2x, a,1x, A)')
     $        '        are specified in :', './DATA_OUT/prof_cld.dat'
         
         WRITE (C_UNIT,'(A)') 
     $        ' ------------------------------ *'
         DO I = 1, N_C_L 
            SELECT CASE( TDS(I)%S )
            CASE('water')
               SELECT CASE ( VT(I,1)%S ) 
               CASE ('LWP') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - liquid water path [g/m^2]'
               CASE ('LWC') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - liquid water content [g/m^3]'
               CASE ('PNC') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - Partical density profile [1/cm^3]'
               CASE ('TAU') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - Optical thickness'
               END SELECT
            CASE('ice')
               SELECT CASE ( VT(I,1)%S ) 
               CASE ('IWP') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - ice water path [g/m^2]'
               CASE ('IWC') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - ice water content [g/m^3]'
               CASE ('CNC') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - Crystals density profile [1/cm^3]'
               CASE ('TAU') 
                  WRITE (C_UNIT,'(A,2x,A)') VT(I,1)%S,
     $                 ' - Optical thickness'
               END SELECT
            CASE('mixed','mixed-sm')
               INP_TYPE(I) = VT(I,1)%S//'/'//VT(I,2)%S
               DO J = 1,2
                  SELECT CASE ( VT(I,J)%S ) 
                  CASE ('IWP') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - ice water path [g/m^2]'
                  CASE ('IWC') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - ice water content [g/m^3]'
                  CASE ('CNC') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - Crystals density profile [1/cm^3]'
                  CASE ('TAU') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - Optical thickness'
                  CASE ('LWP') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - liquid water path [g/m^2]'
                  CASE ('LWC') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - liquid water content [g/m^3]'
                  CASE ('PNC') 
                     WRITE (C_UNIT,'(A,2x,A)') VT(I,J)%S,
     $                    ' - Partical density profile [1/cm^3]'
                  END SELECT
               ENDDO
            END SELECT
         END DO

         CALL WRITE_UD(2,C_UNIT)      
      CLOSE(C_UNIT)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN
      END
C********************************************************************C
