C********************************************************************C
C
C       SCIATRAN - Version 4.6 - Last modification: 21 Dec 2022
C
C       Copyright (C) - Institute of Remote Sensing,
C                       University of Bremen, Germany
C
C********************************************************************C
      SUBROUTINE WRITE_WATER_OUT (*)
c     ... Called from "rtm_init.f"

      USE GRIDS,     ONLY: NLAYERS
      USE LOG_PARS,  ONLY: DIR_OUT
      USE PROFILES,  ONLY: TEMP, PRESS
      USE ST_VARS,   ONLY: PIE, RRS_TEST
      USE ST_INPUT,  ONLY: PWAC_NAME, CHL_NAME 
      USE TYPE_VARS, ONLY: KND
      USE WATER,     ONLY: UWT, DO_ABSUW_AKIMA, N_ABS_UW, Na_W,
     $                     PW_ABS, NL_UW => NLAYERS_UW,
     $                     ZZZ_UW, TEMP_UW, PRES_UW, SAL_UW, CHL_UW,
     $                     Fua_c, Hua_c, N_FC, Flu_con, Absw_Chl
      USE WATER_ICE, ONLY: IWS, LA_N_IW, Na_ice
c     ------------------------------------------------------------------

c     ... Local variables
      CHARACTER (LEN=15), PARAMETER :: R_NAME = 'WRITE_WATER_OUT'
      CHARACTER (LEN=120) T_all
      CHARACTER (LEN=80) TT, TT1
      CHARACTER (LEN = 20)  N_STR, CN_FC
      CHARACTER (LEN = :), ALLOCATABLE :: FMT_M_S, FMT_M_E, FMT_S1
      INTEGER, PARAMETER :: WOT=77
      INTEGER   I, Ii, Ico, K, NM_abs
      REAL (KND) CHL_TEM(NL_UW)
      REAL (KND), ALLOCATABLE :: C_TMP(:,:)
c     ==================================================================

      CALL ST_TRACK (R_NAME, 1, 'I')

      OPEN(WOT, FILE = DIR_OUT//'SCE_ABSORBER-WATER.OUT')
         CALL WRITE_UD(1,WOT)
c     ... Absorption by pure water
        WRITE(WOT,'(11x,23A)') ('=', I=1,23)
        WRITE(WOT,'(12x,A)') 'Pure water absorption'
        WRITE(WOT,'(11x,23A)') ('=', I=1,23)
c        WRITE(WOT,*) 
c        write(*,*) '    IWS -  1 '
        DO I = 1, 2
           IF( IWS(I)%PW_ABS_FILE ) THEN
              TT = '   absorption coefficient from file: '
              WRITE(WOT,'(4x,A,2x,A)') IWS(I)%Name_IW, TRIM(TT)
              WRITE(WOT,'(12x,A)') TRIM(IWS(I)%PWAC_NAME)
           ELSE
              TT = '   Constant absorption coefficient:  '
              WRITE(WOT,'(A,2x,F8.5,2x,A)')
     $             TRIM(TT), IWS(I)%PW_ABS, '[1/m]'
           ENDIF
        ENDDO  ! over ice water (I)
c        write(*,*) '    IWS -  2 '

        NM_abs = 0
c       ... Absorption by chlorophyll 
c       =============================
        WRITE(WOT,*) 
        WRITE(WOT,'(11x,36A)') ('=', I=1,36)
        WRITE(WOT,'(12x,A)') 'Absorption by chlorophyll'
        WRITE(WOT,'(11x,36A)') ('=', I=1,36)
c        WRITE(WOT,*)
c        write(*,*) '    IWS -  3 '
        DO I = 1,2
           IF( .NOT.IWS(I)%Absw_Chl ) THEN
              TT = '   absorption is not included'
              WRITE(WOT,'(4x,A,2x,A)') IWS(I)%Name_IW, TRIM(TT)
              CHL_TEM(:) = 0.d0
           ELSE
              TT = '   normalized absorption coefficient from file: '
              WRITE(WOT,'(4x,A,2x,A)') IWS(I)%Name_IW, TRIM(TT)
              WRITE(WOT,'(12x,A)') TRIM(IWS(I)%CHL_NAME)           
              TT = '   *** Calculation of absorption coefficient '//
     $             'performed according to '
              WRITE(WOT,'(A)') TRIM(TT)         
              TT = '    Morel & Maritorena and Prieur & Sathyendranath'
              WRITE(WOT,'(3x,A)') TRIM(TT)         
              NM_abs = NM_abs + 1
              CHL_TEM(:) = CHL_UW(:)
           ENDIF
        ENDDO  ! over ice water (I)
c        write(*,*) '    IWS -  4 '

c       ... Absorption by CDOM
c       =======================
        WRITE(WOT,*) 
        WRITE(WOT,'(11x,36A)') ('=', I=1,36)
        WRITE(WOT,'(12x,A)') 'Absorption by CDOM'
        WRITE(WOT,'(11x,36A)') ('=', I=1,36)
c        WRITE(WOT,*) 
        IF( IWS(2)%CDOM_Chl ) THEN
           TT = '   Calculation of absorption coefficient performed '
        ELSE
           TT = '   Absorption is not included '
        ENDIF

        IF( IWS(2)%CDOM_Chl ) TT =
     $       '   according to Morel & Maritorena and '//
     $       'Prieur & Sathyendranath'

        IF( IWS(2)%CDOM_ACR ) THEN 
           TT = '   multiplying the excitation factor by '//
     $       'concentration of fluorescent component'
           NM_abs = NM_abs + N_FC
        ENDIF

        IF( IWS(2)%Absw_ful ) THEN 
           TT = '   using specific absorption of folvic acid'
           WRITE(WOT,'(4x,A,2x,A)') IWS(2)%Name_IW, TRIM(TT)         
          NM_abs = NM_abs + 1           
        ENDIF

        IF( IWS(2)%Absw_hum ) THEN
           TT = '   using specific absorption of humic acid'
           WRITE(WOT,'(4x,A,2x,A)') IWS(2)%Name_IW, TRIM(TT)         
           NM_abs = NM_abs + 1           
        ENDIF
        IF( .NOT.IWS(2)%CDOM_Chl ) 
     $       WRITE(WOT,'(4x,A,2x,A)') IWS(2)%Name_IW, TRIM(TT)  
c    <====  End absorption by CDOM  =======
        
c        write(*,*) '    IWS -  6 '
        NM_abs = NM_abs + SUM( IWS(1)%Na_W(:) ) + IWS(2)%Na_W(1)
        IF( NM_abs > 0 ) ALLOCATE( C_TMP(NL_UW,NM_abs) )
c        write(*,*) '    IWS -  7 '
 
c       ... Absorption by other ocean water constituents
c       =================================================
        WRITE(WOT,*) 
        WRITE(WOT,'(11x,46A)') ('=', I=1,46)
        WRITE(WOT,'(12x,A)') 
     $       'Absorption by other ocean water constituents'
        WRITE(WOT,'(11x,46A)') ('=', I=1,46)
c        WRITE(WOT,*)
c       ... Water
        IF( IWS(2)%Na_W(1) > 0 ) THEN
           TT = '   Absorption coefficient from file: '
           WRITE(WOT,'(4x,A,2x,A)') IWS(2)%Name_IW, TRIM(TT)
           DO I = 1,IWS(2)%Na_W(1)
              WRITE(WOT,'(12x,I2,1x,A)') I, IWS(2)%AFN_coef(I,1)%S
           ENDDO
        ELSE
           TT = '   Absorption is not included '
           WRITE(WOT,'(4x,A,2x,A)') IWS(2)%Name_IW, TRIM(TT)           
        ENDIF                   ! additional absorber in water
c       ... Ice
        DO Ii = 1, LA_N_IW
           WRITE(CN_FC,'(I0)') Ii  ! Transform to a character
           IF( IWS(1)%Na_W(Ii) > 0 ) THEN
              TT = '   Absorption coefficient from file: '
              WRITE(WOT,'(4x,A,2x,A)')
     $             IWS(1)%Name_IW//TRIM(CN_FC), TRIM(TT)
              DO I = 1,IWS(1)%Na_W(Ii)
                 WRITE(WOT,'(12x,I2,1x,A)') I, IWS(1)%AFN_coef(I,Ii)%S
              ENDDO
           ELSE
              TT = '   Absorption is not included '
              WRITE(WOT,'(4x,A,2x,A)')
     $             IWS(1)%Name_IW//TRIM(CN_FC), TRIM(TT)           
           ENDIF
        ENDDO  ! over ice layers (Ii)

c       ... Concentration profile
        WRITE(WOT,*) 
        WRITE(WOT,'(11x,43A)') ('=', I=1,43)
        WRITE(WOT,'(12x,A)') 'Depth profile of absorber concentrati1ons'
        WRITE(WOT,'(11x,43A)') ('=', I=1,43)
        WRITE(WOT,*) 
        
        T_all = '   Dep     Press   Temp  Salin'
        Ico = 1
        IF( IWS(2)%Absw_chl ) THEN
           C_TMP(:,Ico) = CHL_UW(:) ; Ico = Ico + 1
           T_all = TRIM(T_all)//'      Chl'
        ENDIF
        IF( IWS(2)%Absw_ful ) THEN
           C_TMP(:,Ico) = Fua_c(:) ; Ico = Ico + 1
           T_all = TRIM(T_all)//'   Fulvic'
        ENDIF
        IF( IWS(2)%Absw_hum ) THEN
           C_TMP(:,Ico) = Hua_c(:) ; Ico = Ico + 1
           T_all = TRIM(T_all)//'    Humic'
        ENDIF
        IF( IWS(2)%CDOM_ACR ) THEN
           DO K = 1, N_FC
              C_TMP(:,Ico) = Flu_con(K,:)
              Ico = Ico + 1
           ENDDO
           WRITE(CN_FC,'(I0)') N_FC
           T_all = TRIM(T_all)//'    '//TRIM(CN_FC)//
     $          ' Fluorescent components ---> '
        ENDIF
!        I = 1 ! Ice additional absorber
        IF( ANY( IWS(1)%Na_W(:)>0 ) ) THEN
           DO K = 1, Na_ice
              C_TMP(:,Ico) = IWS(1)%AAC_P(:,K)
              Ico = Ico + 1
           ENDDO
        ENDIF
!        I = 2 ! Water additional absorber
        IF( IWS(2)%Na_W(1) > 0 ) THEN
           DO K = 1, IWS(2)%Na_W(1)
              C_TMP(:,Ico) = IWS(2)%AAC_P(:,K)
              Ico = Ico + 1
           ENDDO
        ENDIF
        WRITE(CN_FC,'(I0)') SUM(IWS(1)%Na_W(:)) + IWS(2)%Na_W(1) 
        T_all = TRIM(T_all)//'    '//TRIM(CN_FC)//
     $       ' Other '

c        FMT_M_S = '(3x,F6.1,1x,F7.2,2(1x,F6.2)'
        FMT_M_S = '(3x,F9.4,1x,F7.2,2(1x,F6.2)'
        FMT_M_E = '(1X,f9.4))'
        WRITE(N_STR,'(I0)') NM_abs

        IF (NM_abs == 0) THEN
           FMT_S1 = TRIM(FMT_M_S)//')'
        ELSE
           FMT_S1 = TRIM(FMT_M_S)//','//TRIM(N_STR)//TRIM(FMT_M_E)
        ENDIF

        WRITE(WOT,'(3x,A)') TRIM(T_all)
        DO I = 1, NL_UW
           WRITE(WOT,FMT = FMT_S1) 
     $          ZZZ_UW(I), PRES_UW(I), TEMP_UW(I), SAL_UW(I),
     $          (C_TMP(I,K), K=1,NM_abs)
        ENDDO

        CALL WRITE_UD(2,WOT)
      CLOSE(WOT)

      CALL ST_TRACK (R_NAME, 1, 'O')
      RETURN

      END
c********************************************************************
